// wrapped by build app
define("ibm/tivoli/tip/dijit/CMSMenuProviderV2", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/tip/dijit/CMSMenuProvider"], function(dijit,dojo,dojox){
/**
 *  The CMSMenuProviderV2 is an advanced version of the CMSMenuProvider.
 *  This provider allows for resource context fitlers for multiple resources.
 *  It is recommended that you use this version of the CMS menu provider unless you
 *  have some very specific needs.
 *
 * {
 *  rcfs: 
 *  [
 *     [ // per resource
 *       { name : "SourceToken", value : "1.1", isNamingAttribute : false},
 *       { name : "Signature", value : "192.168.1.1", isNamingAttribute : true}
 *     ],
 *     [
 *       { name : "SourceToken", value : "1.2"},
 *       { name : "Signature", value : "192.168.1.2", isNamingAttribute : true }
 *     ]
 *  ],
 *  launchTypes: ["WEB_URL", "PORTAL_PAGE"],
 *  excludedAppIDs: [ { name: "app1", indicator: "isLaunchingProductAndExcludeMyMenus" }, { name: "app2"} ],
 *  substitutionVariables: [ {name: "foo", value: "bar"} ],
 *  options: [ {name: "option", value: "optval"} ]
 * } 
 */
dojo.require("ibm.tivoli.tip.dijit.CMSMenuProvider"); 
dojo.provide("ibm.tivoli.tip.dijit.CMSMenuProviderV2");
 
dojo.declare("ibm.tivoli.tip.dijit.CMSMenuProviderV2", 
             ibm.tivoli.tip.dijit.CMSMenuProvider, 
{
   // This menu uses the same getCMSMenus command, but on the server, we use a different
   // data provider for this version of the CMS menu provider.
   getMenusCommandName: "getCMSMenus",

   getMenuActionCommandName: "getCMSAction",

   // Note that this method should not be called from an outside program.  Here we override the base
   // implmementation of this just in case to help with debugging.
   getRCFURL: function (kw) 
   {
      console.error("Not valid for CMSMenuProviderV2 since we use different context.");
   },

   getMenuItems: function(context) 
   {
      var kw = null;

      kw = {
              sync:         true,
              noBusyDialog: true,
              content:      {
                               command:              this.getMenusCommandName,
                               arg_idPrefix:         this.getIdPrefix(),
                               arg_portletNamespace: this.global.namespace,
                               arg_parentId:         this.parentid,
                               arg_context: dojo.toJson(context)
                            },
              timeout:      this.requestTimeout,
              onResponse:   {
                               callback: dojo.hitch( this, function( response ) { this.menus_ = response; } )
                            }
            };

      if (this.menuName) 
      {
         kw.content.arg_menuName = this.menuName;
      }
      
      ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).post( kw );

      return this.menus_;
   },
   
   getMenuCommandFromServer: function (commandName, context, menuItem) 
   {
      var kw = {
                  content:   {
                                command: this.getMenuActionCommandName,
                                arg_menuCommandName: commandName,
                                arg_idPrefix: this.getIdPrefix(),
                                arg_portletNamespace: this.global.namespace,
                                arg_parentId: this.parentid,
                                arg_context: dojo.toJson(context)
                             },
                  timeout:   this.requestTimeout,
                  onResponse:{
                                callback: dojo.hitch(this, this.handleMenuAction)
                             }
               };
               
       if (this.menuName) 
       {
          kw.content.arg_menuName = this.menuName;
       }
      
       ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).post( kw );
   }

});
});
