// wrapped by build app
define("ibm/tivoli/tip/dojo/EventUtil", ["dijit","dojo","dojox"], function(dijit,dojo,dojox){
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
dojo.provide("ibm.tivoli.tip.dojo.EventUtil");

dojo.declare("ibm.tivoli.tip.dojo.EventUtil",
	          null,
{
	// Simple method to return whether or not a value has a value
	_hasValue: function(obj){
		return typeof obj != "undefined" && obj != null;
	},

	_doEval: function(scope,str){
		if (typeof scope == "undefined" || scope == null)
			return null;
	    var obj = null;
	    try {
	    	var subParts = str.split(".");
	    	obj=scope;
	    	for (var i=0; i<subParts.length; i++){
	    		obj = obj[subParts[i]];
	    	}
	    	//obj=eval("scope."+str);
	    }
	    catch (e) {
	    	obj=null
	    }
	    return obj;
	},
	// Indicates if the expression has a member
	// Scope is the object being checked, str indicates the member to find.
	_hasMember: function(scope, str){
	    var obj = this._doEval(scope,str);
		return typeof obj != "undefined" && obj != null;
	},

	// Pass in a string and this will return a value associated with it.
	// If the string has an error, the returned value is null.  Very
	// useful for deeply nested references.
	_getValue: function(scope,str){
	    var obj = this._doEval(scope,str);
	    if (typeof obj == "undefined")
	    	obj = null;
		return obj;
	},

	// Only adds the specified attribute to the specified object if
	// the attribute does not already exist
	_safeAdd: function(obj,attr,value){
		if (!this._hasValue(obj[attr]))
			obj[attr]=value;
	},

	// This function takes an event using the new event context and 
	// modifies the event by adding the old context to the event by
	// interrogating the proper fields of the new event.  Note that
	// multiple resources are not supported in the old event context.
	// srcEvent: is the event base, it will be updated upon return
	// payload: optional parameter to point to the payload of the event
    //          if specified, this will be used instead of srcEvent.payload
	addOldContext: function(srcEvent, /*optional*/ payload){
		if (this._hasMember(srcEvent,"payload") || this._hasValue(payload)){
			var productCount=1;
			var increment=false;
			var evtPayload=(this._hasValue(payload)?payload:this._getValue(srcEvent,"payload"));
			
			// Check for case if emitter created a resources array of 1
			if (this._hasMember(evtPayload,"resources.length") && (evtPayload.resources.length==1))
				evtPayload=evtPayload.resources[0];

			var clazztype=this._getValue(evtPayload,"cdm.attributes.ClassType");
			
			// Enumerate through all of the payload
			for (var attr in evtPayload) {
			   if (evtPayload[attr]!=null){
				   if (attr=="product"){
						for (var name in evtPayload.product) {
							increment=false;
							var product = evtPayload.product[name];
							// Get all product targeted attributes
							for (var prodName in product){
								// If the attribute is the cdm attribute, need to enumerate it.
								if (prodName == "cdm"){
									for (var prodCdm in product.cdm){
										if (prodCdm == "ClassType")
											this._safeAdd(srcEvent,"CDMClass"+productCount,product.cdm[prodCdm]);
										else
											this._safeAdd(srcEvent,prodCdm+productCount,product.cdm[prodCdm]);
										increment=true;
									}
									if (clazztype != null)
										this._safeAdd(srcEvent,"CDMClass"+productCount,clazztype);
								} else {
									this._safeAdd(srcEvent,prodName+productCount,product[prodName]);
									increment=true;
								}
							}
							if (increment)
								productCount++;
						}
				   }
				   else if (attr=="cdm" && this._hasMember(evtPayload,"cdm.attributes")){
						for (var attr in evtPayload.cdm.attributes) {
							// Add all attributes with their given name; including ClassType
							this._safeAdd(srcEvent,attr,evtPayload.cdm.attributes[attr]);
						}			
				   }
				   else if (attr != "framework" && attr != "resources"){
					   this._safeAdd(srcEvent,attr,evtPayload[attr]);
				   }
			   }
			}
		}
		return srcEvent;
	},

	_parseProductKey: function(mssname){
		var key = null;
		//var pattern = /\+ProductName=.+$/g;
		//var key = mssname.match(pattern);
		//console.debug(key);
		var pattern = "+ProductName=";
		var i = mssname.indexOf(pattern);
		if (i>0){
			var start = i + pattern.length;
			var end = mssname.indexOf("+",start);
			if (end>0)
				key = mssname.substr(start,end-start);
			else
				key = mssname.substr(start);
		}
		if(key != null)
			key = key.replace(/ /g,"");
			
		return key;
	},

	_addCdmAttribute: function(evtPayload, attribute, value){
		if (!this._hasValue(evtPayload.cdm))
			evtPayload.cdm = {};
		if (!this._hasValue(evtPayload.cdm.attributes))
			evtPayload.cdm.attributes = {};
		evtPayload.cdm.attributes[attribute] = value;
	},

	_addMssMember: function(evtPayload, attrList, mssNumber, productkey, attribute){
		if (this._hasMember(attrList,attribute+mssNumber)){

			// Some special processing for CDMClass
			if (attribute=="CDMClass"){
				if (!this._hasMember(evtPayload,"cdm.attributes.ClassType")){
					this._addCdmAttribute(evtPayload, "ClassType", attrList[attribute+mssNumber]);
				}
				this._safeAdd(evtPayload.product[productkey].cdm,"ClassType",attrList[attribute+mssNumber]);
			} else {
				this._safeAdd(evtPayload.product[productkey].cdm,attribute,attrList[attribute+mssNumber]);
			}
			
			delete attrList[attribute+mssNumber];
		}
	},

	_getMssMembers: function(evtPayload, attrList, mssNumber){
		var mssname=attrList["MSSName"+mssNumber];
		var productkey = attrList["ProviderType"+mssNumber];
		if (!this._hasValue(productkey))
		    productkey = this._parseProductKey(mssname);
		if (productkey != null){
			if (!this._hasValue(evtPayload.product))
				evtPayload.product={};
			if (!this._hasValue(evtPayload.product[productkey]))
				evtPayload.product[productkey]={};
			if (!this._hasValue(evtPayload.product[productkey].cdm))
				evtPayload.product[productkey].cdm={};
			
			// This case if there are no numbers after MSSName in the old context
			if (mssNumber == ""){
				// The old format only ever added numbers to MSSName, SourceToken, and CDMClass
				this._addMssMember(evtPayload, attrList, mssNumber, productkey, "MSSName");
				this._addMssMember(evtPayload, attrList, mssNumber, productkey, "SourceToken");
				this._addMssMember(evtPayload, attrList, mssNumber, productkey, "CDMClass");
			} else {
				// If here, there are numbers after MSSName (like MSSName1), we include all
				// attributes ending with the same number in that providertype.
				for (var attr in attrList){
					if (attr.lastIndexOf(mssNumber)==(attr.length-(""+mssNumber).length)){
						var prop = attr.substr(0,attr.length-(""+mssNumber).length);
						
						if (prop=="CDMClass" || prop=="MSSName" || prop=="SourceToken"){
							this._addMssMember(evtPayload, attrList, mssNumber, productkey, prop);
						} else {
							this._safeAdd(evtPayload.product[productkey],prop,attrList[attr]);
						}

						delete attrList[attr];
					}
				}
			}
			
		}
	},

	// The purpose of this method is to attempt to create a "new" context out of
	// an event constructed with old context.
	// srcEvent - the source event
	// payload - optional parameter.  By default this method will add srcEvent.payload
    //           to put the context, but you can specify and alternate location if needed.
	addNewContext: function(srcEvent, /*optional*/ payload){
		// bypass if event already has a payload; therefore, only run this method
		// if a payload object was specified, or there is not already a payload
		if ((this._hasValue(payload) || !this._hasMember(srcEvent,"payload")) && this._hasValue(srcEvent)){  
		    // Gather all of the attributes into an array so we can track what has been processed.
			var attrList={};
			for (var attr in srcEvent) {
				attrList[attr]=srcEvent[attr];
			}
			var evtPayload={};
			if (this._hasValue(payload)){
				evtPayload=payload;
			} else {
				srcEvent.payload = {};
				evtPayload = srcEvent.payload;
			}
			
			// See if ClassType was coded in the old context.  If so, add it; otherwise, we will
			// try to get it from CDMClass later.
			if (this._hasMember(attrList,"ClassType") && !this._hasMember(evtPayload,"cdm.attributes.ClassType")){
				this._addCdmAttribute(evtPayload, "ClassType", attrList["ClassType"]);
				delete attrList["ClassType"];
			}
			
			if (this._hasMember(attrList,"MSSName")){
				// There is just "MSSName", so assume just one MSSName, SourceToken, CDMClass.
				this._getMssMembers(evtPayload,attrList,"");
			} else {
				var mssNumber = 1;
				while (this._hasMember(attrList,"MSSName"+mssNumber))
				{
					this._getMssMembers(evtPayload,attrList,mssNumber);
				    mssNumber++;	
				}
			}
			
			// New for Breeze: add lookup for OslcResourceURI
			var _OslcResourceURI="OslcResourceURI";
			if (this._hasMember(attrList, _OslcResourceURI)){
				var uris=[];
				var uri= {
						    uri: attrList[_OslcResourceURI],
				            type: "product"   // These are always product URIs, meaning URIs that came from a product
				         };
				uris.push(uri);
				this._safeAdd(evtPayload, "uris", uris);
				delete attrList[_OslcResourceURI];
			}
			
			// Now place all the other properties inline
			for (var mem in attrList){
				// do not add name, type, or source as they are part of the event meta data
				if (mem != "name" && mem != "source" && mem != "type"){
					this._safeAdd(evtPayload,mem,attrList[mem]);
				}
			}
		}
     },
     
     // Pass an event to this method, and it will return the number of resources present.
     // The minimum number will always be 1 unless the srcEvent is null in which case 0 is returned.
     getNumberOfResources: function(srcEvent){
    	 if (!this._hasValue(srcEvent)){                    // Got a null object
    		 return 0;
    	 }
    	 if (!this._hasValue(srcEvent.payload)){            // Event is old style, so we know only one resource
    		 return 1;
    	 }
    	 if (!this._hasValue(srcEvent.payload.resources)){  // if no resource array, then only one resource
    		 return 1;
    	 }
    	 return srcEvent.payload.resources.length;
     },
     
     // Returns the attributes for the resource at the given index in the event.
     // Note for some events like TimeSet, there may not be an actual resource, but in this
     // case you pass an index of 0
     getAttributes: function(srcEvent, index){
    	 if (!this._hasValue(srcEvent)){                    // Got a null object
    		 console.error("EventUtil: Bad event.");
    		 return null;
    	 }
    	 if (!this._hasValue(index) || index<0){            // Bad index
    		 console.error("EventUtil: Bad Index.");
    		 return null;
    	 }
    	 
    	 var numItems = this.getNumberOfResources(srcEvent);
    	 if (numItems < 1 || index>= numItems){        // Index out of bounds
    		 console.error("EventUtil: Index out of bounds.");
    		 return null;
    	 }
    	 
    	 var payload = srcEvent.payload;
    	 if (!this._hasValue(srcEvent.payload)){            // Event is old style, so we know only one resource
    		 addNewContext(srcEvent,payload);          // Create new style payload to work with. Do not change source event.
    	 }
    	 
    	 if (!this._hasValue(srcEvent.payload)){            // Should not happen
    		 return null;
    	 }
    	 
    	 if (this._hasValue(srcEvent.payload.resources)){
 			var attrList={};
			for (var attr in srcEvent.payload) {
				if (attr != "resources" && attr!="framework"){
					attrList[attr]=srcEvent.payload[attr];
				}
			}
			dojo.mixin(attrList,srcEvent.payload.resources[index]);
			return attrList;
    	 } else {                                      // No payload.resources
    		 if (index == 0){
    			 return srcEvent.payload;
    		 } else {                                  // Should not occur
        		 console.error("EventUtil: Index error.");
        		 return null;
    		 }
    	 }
    	 
    	 return null;
     },
     
     // Returns the attributes for a given resource in the event as a list with the 'product'
     // attribute flattened.  This method takes the mssname and/or providerType as optional.
     // If you only want to specify the providerType, then pass null for the mssname
     // As an example, if the event looks like this:
     //
     // event=   {
     //       name: "http://ibm.com/tip#NodeClickedOn",
     //       resources: [{
     //       product: {
     //                   IBMTivoliMonitoringServices: { cdm:{
     //                                      MSSName: "ibm-cdm:///CDMMSS/Hostname=loco.raleigh.ibm.com+ManufacturerName=IBM+ProductName=IBM Tivoli Monitoring Services",
     //                                      SourceToken: "ip_address=9.27.144.150"
     //                                     },
     //                                     Color: "blue"
     //                                   }
     //                }}]
     //       uris: [{uri: "http://myserver:port/items/item1"}],
     //    }
     //
     // If this method is called with parameters (event, 0,  "ibm-cdm:///CDMMSS/Hostname=loco.raleigh.ibm.com+ManufacturerName=IBM+ProductName=IBM Tivoli Monitoring Services")
     // the returned result is
     //
     // {
     //   MSSName: "ibm-cdm:///CDMMSS/Hostname=loco.raleigh.ibm.com+ManufacturerName=IBM+ProductName=IBM Tivoli Monitoring Services",
     //   SourceToken: "ip_address=9.27.144.150",
     //   Color: "blue",
     //   uris: [{uri: "http://myserver:port/items/item1"}]
     // }
     //
     // If providerType or MSSName is not provided, then the product information will not be returned.  So in this example, the result 
     // for the parameters (event, 0) will be
     //
     // {
     //   uris: [{uri: "http://myserver:port/items/item1"}]
     // }
     //
     // null is returned on error
     
     getAttributesWithFlatProduct: function(srcEvent, index, /*optional*/ mssname, /*optional*/ providerType){
    	 if (!this._hasValue(srcEvent)){
    		 console.warn("no srcEvent passed to getAttributesWithFlatProduct");
    		 return null;
    	 }
    	 var attributes = this.getAttributes(srcEvent, index);
    	 if (attributes==null){
    		 return null;
    	 }
    	 var retAttrs={};
         for (var attr in attributes) {
        	 if (attr=='product'){
        		 this._addFlatProductData(retAttrs,attributes[attr],mssname,providerType);
        	 } else {
        		 retAttrs[attr] = attributes[attr];
        	 }
         }
    	 return retAttrs;
     },
     
     
     _addFlatProductData: function(target,productData,mssname,inputProviderType){
    	 var providerType = null;
    	 if (!this._hasValue(productData)){
    		 return;
    	 }
    	 if (!this._hasValue(mssname) && !this._hasValue(inputProviderType)){
    		 return;
    	 }
    	 if (this._hasValue(inputProviderType)){
    		 providerType = inputProviderType;
    	 }
    	 if (this._hasValue(mssname)){
    		 if (!this._hasValue(providerType)){
    			 providerType = this._parseProductKey(mssname);
    		 }
    	 }
    	 if (this._hasValue(productData) && this._hasValue(productData[providerType])){
    		 var providerAttrs = productData[providerType];
    		 if (this._hasValue(mssname) && this._hasValue(providerAttrs.cdm)){ // need to validate mssname value
    			 if (this._hasValue(providerAttrs.cdm.MSSName)){
    				 if (providerAttrs.cdm.MSSName != mssname){
    					 return;  // if the mssname specified does not match mssname, then do not include this product data
    				 }
    			 }
    		 }
        	 for (var attr in providerAttrs){
        		 if (attr == "cdm"){
            		 dojo.mixin(target,providerAttrs[attr]);
        		 } else {
        			 target[attr]=providerAttrs[attr];
        		 }
        	 }
    	 }
    	 return;
     },
     
     // This method will accept an input object and convert all the attribute values to
     // strings.  This is helpful for an object needing to send attributes as parameters
     // on a URL.
     // The returned object will contain all the same attributes as the input object, but
     // any dojo objects or arrays will have been converted to JSON strings.
     attributesToStrings: function(source){
    	 if (!this._hasValue(source)){
    		 console.warn("source is null in attributesToStrings");
    		 return null;
    	 }
    	 var output={};
    	 for (var attr in source){
    		 if (typeof source[attr] == "string"){
    			 output[attr]=source[attr];
    		 } else {
        		 console.log(attr+" is type "+typeof source[attr]);
    			 output[attr]=dojo.toJson(source[attr]);
    		 }
    	 }
    	 return output;
     },
     
     // This method is a utility to help the caller construct an event.
     // As parameters, the caller passes in an object that contains members
     // that control how the event is created.
     // Minimally, the caller passes in a member called 'valueMap'
     // that contains all the attribute names and values to be included in
     // the event.
     // {
     //   valueMap:   [{
     //                  MSSName: "ServiceInstanceID",
     //                  SourceToken: "12345",
     //                  bar: "http://somehost:8080/foo"
     //               }]
     // }
     // This method will construct the event based on the attribute names
     // given.  There should be one object in the 'attributes' array for
     // each unique resource to be represented in the event.
     //
     // Optionally, the caller may also include a mapping to assist
     // in the construction of the event.  This mapping uses the substitution
     // pattern of '%%variable%%' where this entire string will be replaced
     // with the value of the attribute that has the name 'variable' from the
     // 'attributes' list (provided above). This value MUST BE A STRING!!
     // The following is the resource, but it must be passed as a string.
     //
     //   resourceMap: {
     //       product: {
     //                   MyProviderType: {
     //                                      MSSName: %%MSSName%%,
     //                                      SourceToken: %%SourceToken%%
     //                                   }
     //                }
     //       uris: [{uri: %%bar%%}],
     //            }
     // 
     // param: eventName is simply the name of the event to send.
     createEvent: function(attributes,eventName){
    	 if (!this._hasValue(attributes)){
    		 console.warn("No attributes provided to EventUtil.createEvent");
    		 return null;
    	 }
    	 if (!this._hasValue(eventName)){
    		 console.warn("No eventName provided to EventUtil.createEvent");
    		 return null;
    	 }
    	 if (!this._hasValue(attributes.valueMap)){
    		 console.warn("No attributes.valueMap provided provided to EventUtil.createEvent");
    		 return null;
    	 }
    	 
    	 var event=this._buildEvent(attributes);
    	 event.name = eventName;
    	 return event;	 
     },
     
     _buildEvent: function(attributes){
    	 var event = {};
    	 event.payload={};
    	 event.payload.resources=[];
    	 
    	 // Loop through each set of attributes represented in the attributes array.
    	 for (var resourceAttrs in attributes.valueMap) {
    		 var payload = {};
    		 if (this._hasValue(attributes.resourceMap)){
            	 payload = this._contextFromMap(attributes.valueMap[resourceAttrs], attributes.resourceMap);
    		 } else {
            	 this.addNewContext(attributes.valueMap[resourceAttrs], payload);
    		 }
        	 event.payload.resources.push(payload);
    	 }
    	 return event;
     },
     
     // Construct the payload given a mapping
     _contextFromMap: function(values, mapStr){
    	 //var mapStr = dojo.toJson(map);  // convert to string to make substitutions easy
    	 for (var name in values){       // for each value in the values object, loop to replace
    		 var re = new RegExp("%%"+name+"%%","g");
    	     mapStr = mapStr.replace(re, values[name]);
    	 }
    	 return dojo.fromJson(mapStr);
     }
     
     
});

});
