// wrapped by build app
define("ibm/tivoli/tip/dojo/_Global", ["dijit","dojo","dojox","dojo/i18n!ibm/tivoli/tip/dojo/nls/resources"], function(dijit,dojo,dojox){
/****************************************************************************
* _Global -- the actual Global object
*
* Notice that there is a '_' underscore prefixing this object.  This is
* important because removing it will delete the Global module functions
* such as getSingleton.  This is OK because we will never construct one
* of these things without calling getSingleton.
****************************************************************************/
dojo.requireLocalization("ibm.tivoli.tip.dojo", "resources");
dojo.provide("ibm.tivoli.tip.dojo._Global");

dojo.declare("ibm.tivoli.tip.dojo._Global",null,
{	 
  _actionBroker:  null,
  _portletTitle: null,
  contentIsolation: false,
  namespace: "",
	
	constructor: function( context, contentIsolation )
	{
      //add all functions, fields, objects inside context to this object
      dojo.mixin( this, context );
      
      //create path to JSON controller servlet
      this.controller = context.controllerURL;
         
      // Create resource bundle for TIP based resource strings
      this.nlsTIPResources = dojo.i18n.getLocalization( "ibm.tivoli.tip.dojo", "resources" );
      
	  //Setting Namespace variable
	  this.namespace = context.namespace;
	  
	  //Setting Boolean variable to show MessageDialog in Content Isolation mode
      this.contentIsolation = contentIsolation;
	},

  /************************************************************************
  * Get the action broker.
  ************************************************************************/
  /* ibm.tivoli.tip.dojo.ActionBroker */ 
  getActionBroker : function() 
  {
    return( this._actionBroker );
  },

  /************************************************************************
  * Set the action broker.
  ************************************************************************/
  /* ibm.tivoli.tip.dojo.ActionBroker */ 
  setActionBroker : function( /* ibm.tivoli.tip.dojo.ActionBroker */ broker ) 
  {
    return( this._actionBroker = broker );
  },
  
  getPortletTitle : function () {
  	return this._portletTitle;
  },
  
  setPortletTitle: function (/*String*/ title) {
  	this._portletTitle = title;
  },
	
	/************************************************************************
	* Generate Conextual URL
	************************************************************************/
	contextualURL : function( baseURL ) 
	{
  	return( this.contextPath + baseURL );
	},

	/************************************************************************
	* Generate namespaced ID from a BASE ID.
	************************************************************************/
	namespaceID: function( baseID ) 
	{
  	return( this.namespace + baseID );
	},	

  /************************************************************************
  * show error message dialog
  ************************************************************************/
  showErrorMessage: function( msgMixin )
  {
    var msgKW = {
              type:         "Error",
              showCheckbox: false,
			  namespace: this.namespace,
			  contentIsolation: this.contentIsolation,
              portletTitle: this._portletTitle
            };

    dojo.mixin( msgKW, msgMixin );
    // var x = new ibm.tivoli.tip.dijit.TIPMessageDialog( msgKW ); 
    // Refer 40366 - TWL RTC
	var x = new dojoe.messagedialog.MessageDialog( msgKW );
    x.show();
  },

  /************************************************************************
  * Retrieve a specific NLS string from the TIP resource bundle.  Deployers
  * could create a method like this for their own product that calls nls
  * with their resource bundle for convenience.  This is meant to only
  * be called by TIP code.
  ************************************************************************/
  tipnls: function( nlsKey, inserts )
  {
     return this.nls( nlsKey, inserts, this.nlsTIPResources );
  },

  /************************************************************************
  * Retrieve a specific NLS string optionally performing substitution(s).
  * This is a fairly generic routine meant to be used by exploiters of 
  * the Global object.  If an exploiter specifies a resource bundle in 
  * the Global context via the name 'nlsResources' when creating the Global object
  * then a bundle need not be specified on the invocation of this method
  * because it will use nlsResources instead.
  ************************************************************************/
  nls: function( nlsKey, inserts, myBundle )
  {
    // attempt to retrieve the specified string via its key
    var s = null;
    
    // If a bundle has been specified, then use it, else
    // use the bundle specified on nlsResources.
    if (myBundle !== null)
    {
       s = myBundle[ nlsKey ];
    }
    else
    {
       s = this.nlsResources[ nlsKey ];
    }

    // substitute "key" if not able to resolve
    if ( ! s )
    {
      s = nlsKey;
    }

    // string insert(s) specified?
    if ( inserts ) 
    {
      // if not already a KW object -or- array 
      if ( ! ( typeof inserts == 'object' ) )
      {
        // convert single number/string/... to array
        inserts = [ inserts ];
      }

      // perform the substitutions
      s = dojo.string.substitute( s, inserts );
    }

    return( s );
  },
  
  showMessageDialog: function (type, msgID, msg, showCheckbox) {
	if (!showCheckbox) {
		showCheckbox = false;
	}
    var modalWindow = new dojoe.messagedialog.MessageDialog (
		                                            { 
		                                              type: type,
	    	                                          message: msg,
	        	                                      messageId: msgID,
													  namespace: this.namespace,
													  contentIsolation: this.contentIsolation,
	        	                                      showCheckbox: showCheckbox
	                    	                        }); 
	modalWindow.show();
  },

  showMessage: function (msgMixin) {
	var messageid = msgMixin.messageId;
	var messagetype = messageid.substring(messageid.length-1, messageid.length);
	var type = "";
	switch(messagetype)
	{
		case "E":
			type = "error";
			break;
		case "W":
			type = "warning";
			break;
		case "C":
			type = "confirm";
			break;
		case "V":
			type = "validation";
			break;
		default:
			type = "";
	}
	
    var msgKW = {
              type: type,
              showCheckbox: false,
			  namespace: this.namespace,
			  contentIsolation: this.contentIsolation,
              portletTitle: this._portletTitle
            };

    dojo.mixin( msgKW, msgMixin );
    var x = new dojoe.messagedialog.MessageDialog( msgKW );
	x.show();
  },
  
  handleServerException: function (exception)  {
 	console.error ("Received server exception: " + exception.messageID + ": " + exception.messageString);
	console.error ("with server stack trace: ");
	console.error (exception.stackTrace);
	this.showMessageDialog (exception.type, exception.messageID, exception.messageString, false);
  },
  
	isBidiChar: function (c){
		if (c >= 0x05d0 && c <= 0x065f ||
			c >= 0x066a && c <= 0x06ef ||
			c >= 0x06fa && c <= 0x07ff ||
			c >= 0xfb1d && c <= 0xfdff ||
			c >= 0xfe70 && c <= 0xfefc) {
			console.log ("isBidiChar: true: " + c);
			return true;
		} else {
			console.log ("isBidiChar: false: " + c);
			return false;
		}
	},

	isLatinChar: function(c){
    	if((c > 64 && c < 91)||(c > 96 && c < 123)) {
	        return true;
		} else {            
    		return false;
    	}    
	},

	checkContextual: function (text){
//		var text = obj.value;
		var len = text.length;
		for(var i = 0;i < len;i++)  {
			symbol = text.charCodeAt(i);
			if(this.isBidiChar(symbol)) {
				return "RTL";
		//	} else if(this.isLatinChar(symbol)) {
		//		return "LTR";
			}
		}
		return "LTR";
	},
  
  bidiEnableLabel: function (text) {
		var result = text;
		
		if (this.bidiTxtDir == "RTL") {
			result = "&#8235;" + text + "&#8236;"; 
		} else if (this.bidiTxtDir == "CONTEXTUAL") {
			var contextDir = this.checkContextual (text)
			if (contextDir == "RTL") {
				result = "&#8235;" + text + "&#8236;";
			} else {
				result = text;
			}
		} else {  // ltf
			result = text;
		}

		return (result);
  }
  
    
} );
});
