/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/activealerts/ActiveAlertsIgnoretDialogTask", [
    "dijit",
    "dojo",
    "dojox",
    "dijit/_WidgetBase",
    "dijit/_Templated",
    "dijit/Dialog",
    "dijit/form/Button",
    "dojox/data/dom",
    "ibm/tivoli/navmodel/Trace",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/on",
    "dojo/topic",
    "dojo/dom-construct",
    "dojo/dom-style",
    "dojo/_base/window",
    "dijit/registry",
    "dijit/layout/ContentPane",
    "dojo/i18n!ibm/usmi/nls/frameworkResources",
    "dojo/i18n!ibm/usmi/nls/defaultTaskResources",
    "dojo/i18n!ibm/usmi/activealerts/nls/ActiveAlertsResources",
    "ibm/usmi/core/utils/Logger",
],

function (dijit, dojo, dojox, _WidgetBase, _Templated, _Dialog, _Button, dom, theTrace, declare, lang, on, topic,
        construct, style, win, registry, ContentPane, FrameworkMRI, DefaultMRI,ActiveAlertsMRI,Logger) {

	//Create our widget!
	return declare("ibm.usmi.tasks.activealerts.ActiveAlertsIgnoreDialogTask", [
	    _WidgetBase,
	    ContentPane
	], {

	    // private members       
	    _args : null,
	    _task : null,
	    _selectionName : null,
	    FrameworkResources : FrameworkMRI,
	    DefaultResources : DefaultMRI,
	    ActiveAlertsResources : ActiveAlertsMRI,
	    _ActiveAlertsLogger : null,
	    // _trace: [private] ibm.tivoli.navmodel.Trace
	    // Utility for trace and logging
	    //_trace : new theTrace("ibm.usmi.tasks.activealerts.ActiveAlertsIgnoreDialogTask"),

	    constructor : function (/*constructor_kwArgs*/kwArgs) {
		    // summary:
		    //		Coming soon
		    // description:
		    //		Coming soon

		    this._args = {
		        launchParms : "",
		        menuItem : "",
		        context : ""
		    };

		    lang.mixin(this._args, kwArgs);

		    //	this._datasetId = kwArgs.datasetId;
		    //	this._taskId = kwArgs.taskId;
		    //	this._oids = kwArgs.oIds;
			
		    // create the logger
			this._ActiveAlertsLogger = new Logger("ibm.usmi.activealerts", "ActiveAlerts");
			
		    // Run any parent postCreate processes - can be done at any point
		    this.inherited(arguments);
		    if (this._ActiveAlertsLogger.isFinest()) {
				this._ActiveAlertsLogger.exiting("Active Alerts Ignore Dialog Task is constructed", "constructor");
			}

	    },

	    startup : function () {
		    this.inherited(arguments);
	    },

	    postCreate : function () {

		    var getTasksError = function (error, request) {
			    this._navWidget.hideBusyPane();

			    //Give the user some feedback
			    this.displayException(error);
		    };

		    this.getDataSet().getTasks({
		        onComplete : lang.hitch(this, this._getTasksComplete),
		        onError : lang.hitch(this, this.getTasksError),
		        taskCategory : "menu", //TODO: Could I specifically request nav hints instead of all tasks?
		        selection : this.getSelection()
		    });

		    // Get a DOM node reference for the root of our widget
		    var domNode = this.domNode;

		    // Run any parent postCreate processes - can be done at any point
		    this.inherited(arguments);
		    var selections = this._args.context.selection.getItems();
		    var selectionName = selections[0].getLabel();
		    this._selectionName = selectionName;
		    if (!(this._args.launchParms.confirmationText)) {
			    this._args.launchParms.confirmationText = dojo.string.substitute(this.DefaultResources.CONFIRMMSG, [
			        1,
			        this._selectionName]);
		    }

		    // text div		
		    var activeAlertDiv = construct.create("div", {
		        "innerHTML" : this._args.launchParms.confirmationText,
		        style : "padding: 5px"
		    });

		    construct.place(activeAlertDiv, domNode);
		    
		    var radioOnlyDiv = construct.create("div");
		    style.set(radioOnlyDiv, "padding", "5px 5px 5px 20px");
		    construct.place(radioOnlyDiv, domNode);
		    
		    var radioOnly = new dijit.form.RadioButton({	          
	            value: "only",
	            id : this.id + "_radioOnly",
	            checked: true,
	            value:"only",
	            name: "resource"
	        }, radioOnlyDiv.appendChild(construct.create("div")));
		    
			// label for radioOnly
			var labelVal = dojo.doc.createElement('label');
			dojo.attr(labelVal, "for", "radioOnly");
			dojo.attr(labelVal, "innerHTML", this.ActiveAlertsResources.resourceOnly)
			radioOnlyDiv.appendChild(labelVal);
			
		    var radioaAllDiv = construct.create("div");
		    style.set(radioaAllDiv, "padding", "5px 5px 5px 20px");
		    construct.place(radioaAllDiv, domNode);
		    
		    var radioaAll = new dijit.form.RadioButton({	            
	            id : this.id + "_radioAll",
	            value: "all",
	            name: "resource"	
	        }, radioaAllDiv.appendChild(construct.create("div")));
			
		    // label for radioAll
			var labelVal2 = dojo.doc.createElement('label');
			dojo.attr(labelVal2, "for", "radioAll");
			dojo.attr(labelVal2, "innerHTML", this.ActiveAlertsResources.allResources)
			radioaAllDiv.appendChild(labelVal2);			
		    
		    var activeAlertBottomDiv = construct.create("div");
		    style.set(activeAlertBottomDiv, "padding", "5px 5px 5px 10px");
		    construct.place(activeAlertBottomDiv, domNode, "last");

		    var saveButton = new dijit.form.Button({
		        id : this.id + "_DialogOKButton",
		        label : this.FrameworkResources.OK,
		    }, activeAlertBottomDiv.appendChild(construct.create("div")));

		    var cancelButton = new dijit.form.Button({
		        id : this.id + "_DialogCancelButton",
		        label : this.FrameworkResources.CANCEL,
		    }, activeAlertBottomDiv.appendChild(construct.create("div")));

		    on(saveButton, "click", lang.hitch(this, "onOK"));
		    on(cancelButton, "click", lang.hitch(this, "onCancel"));
		    
	    },

	    // get task from id
	    _getTasksComplete : function (tasks) {

		    //Want to loop through tasks looking for
		    if (tasks) {
			    for ( var i = 0; i < tasks.length; i++) {
				    var task = tasks[i];
				    // found are task.
				    if (task.getId() === this.getTaskId()) {

					    this.setTask(task);

					    break;
				    } else if (task.getChildTasks()) {
					    var subTasks = task.getChildTasks();
					    if (subTasks.length > 0) {
						    this._getTasksComplete(subTasks);
						    if (this.getTask()) {
							    break;
						    }
					    }
				    }
			    }
		    }
	    },

	    //override the onCancel method to also destroy the whole widget.
	    onCancel : function () {
		    // summary:
		    //		Coming soon
		    // description:
		    //		Coming soon
	    	topic.publish("DefaultDialogDismissal", []);
		    dijit.byId(this.id + "_DialogCancelButton").destroyRecursive();
			dijit.byId(this.id + "_radioOnly").destroyRecursive();
			dijit.byId(this.id + "_radioAll").destroyRecursive();
		    this.destroyRecursive();
		    this.inherited(arguments);
	    },

	    //override the onCancel method to also destroy the whole widget.
	    onOK : function () {
		    // summary:
		    //		Coming soon
		    // description:
		    //		Coming soon
	
	    	var taskParms = {resourceOnly: dijit.byId(this.id + "_radioOnly").get("checked")};
		    this.getTask().execute({
		        onError : dojo.hitch(this, this._handleExecuteOnServerError),
		        selection : this.getSelection(),
		        selectionParms: taskParms,
				sync: true
		    });

		    this.onCancel();
	    },

	    getDataSet : function () {
		    return this._args.context.dataset;
	    },

	    getTaskId : function () {
		    return this._args.menuItem.actionID;
	    },

	    getTask : function () {
		    return this._task;
	    },

	    setTask : function (t) {
		    this._task = t;
	    },

	    getSelection : function () {
		    var selectedThings = this._args.context.selection;
		    var currentSelection = null;
		    if (selectedThings) {
			    currentSelection = new ibm.tivoli.navmodel.Selection({
				    version : this.getDataSet().getVersion()
			    });
			    currentSelection.addItems(selectedThings.getItemIds());
			    currentSelection.addLinks(selectedThings.getLinkIds());
			    currentSelection.addRelationships(selectedThings.getRelationshipIds());
		    }
		    return currentSelection;
	    },

	    getBodyHeight : function () {
		    return (0.1 * ((window.innerHeight !== undefined) ? window.innerHeight : document.body.offsetHeight));
	    },

	    _handleExecuteOnServerError : function (error, exception, ioArgs) {
		    // summary:
		    //		Coming soon
		    // description:
		    //		Coming soon
		    // error:
		    //		
		    // exception:
		    //		
		    // ioArgs:
		    //		

		    if (this._trace.isError()) {
			    this._trace.error(">>> error executing server task = " + error, "_handleExecuteOnServerError");
		    }

		    //Give the user some feedback
		    if (this.view) {
			    this.view.displayException(error);
		    }
	    },
	}); // and that's it!
});
