/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/core/framework/FrameworkMessage", ["dojo/_base/kernel",
        "dojo/_base/declare",
        "dojo/topic",
        "dijit/_WidgetBase",
        "dijit/registry",
        "dijit/_base/manager", 
        // IDX imports
        "idx/oneui/messaging/SingleMessage",
        "idx/oneui/messaging/ModalDialog",  
        // NextGen imports 
        "ibm/usmi/core/utils/Logger"],
    function (dojo, declare, topic, _WidgetBase, registry, manager, SingleMessage, ModalDialog, Logger) {

    return declare("ibm.usmi.core.framework.FrameworkMessage", [_WidgetBase, SingleMessage], {

        dateFormat : {
            datePattern : 'dd MMMM y, ', 
            timePattern : 'hh:mm a'}, 

        style : "width: 98%;", 

        type : "error",

        showAction : false,

        showRefresh : false,

        operatorResponse : null,

        moreDetailsURL : null,

        postCreate : function() {
            console.log("post create");
            this.inherited(arguments);
        },

        onClick : function(e) {
             this.inherited(arguments);             

             var container = dijit.byId("mainContentAreaLayout");
             container.resize();
        },

        onMoreDetails : function(e) {
             this.inherited(arguments);  
             console.log("More Details clicked."); 
             if (this.operatorResponse != null) {
                 this.showMoreDetailsDialog(this.operatorResponse);          

             } else if (this.moreDetailsURL != null) {
                 this.launchMoreDetails(this.moreDetailsURL);
             }
        },

        destroy : function() {
             this.inherited(arguments);             

             var container = dijit.byId("mainContentAreaLayout");
             container.resize();
        },

        launchMoreDetails : function(url) {
             window.open(url ,"_blank", null);
        },

        showMoreDetailsDialog : function(operatorResponse) {
             var moreDetailsDialog = manager.byId("moreDetailsDialog");
             if (moreDetailsDialog) {
                 moreDetailsDialog.destroy();
                 moreDetailsDialog = null;
                 registry.remove("moreDetailsDialog");
             }
                 moreDetailsDialog = new ModalDialog({
                     id:"moreDetailsDialog",
                     type: "error",
                     text: operatorResponse,
                     info: [{
                         title: "Fix this problem",
                         content: "The server might not be available because someone pulled out the plug.<br/><br/>" + 
                         "Plug in and restart the server. " + 
                         "<button data-dojo-type=\"dijit.form.Button\" data-dojo-props='\"class\":\"secondaryButton\", onClick:function(){ alert(\"clicked Restart\"); }, value:\"Secondary\"'>Restart</button>" +
                         "<br/><br/>If the problem persists, install a locked door to keep the people away from the server, then restart it."
                     },{
                         title: "Get more help",
                         content: "<a href='#'>Email this message to my administrator</a><br/><br/>" + 
                         "<a href='#'>Live chat with my administrator</a><br/><br/>" + 
                         "<a href='#'>Search support</a><br/><br/>" + 
                         "<a href='#'>Search communities</a>"
                     }],
                     messageId: "CAT123456",
                     messageRef: "http://www.google.com/",
                     messageTimeStamp: new Date(2011, 8, 1)
                 });

				moreDetailsDialog.show();
			}
    });
});
