/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
/**
 * NextGenUIMenuUtils is a group of utility functions to use during the creation
 * of a dojo menu
 */
define("ibm/usmi/menus/NextGenUIMenuUtils", [
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dijit/_WidgetBase",
	"dijit/MenuItem",
	"idx/oneui/MenuHeading",
	"dojo/dom-construct",
	"dijit/registry",
	"dojo/_base/lang",
	"ibm/usmi/core/utils/Logger",
	"dijit/Tooltip",
	"ibm/usmi/core/utils/_UsmiBrowserPersistence",
	"dojo/on",
	"dijit/focus",
	"dojo/dom"
], function (dojo, declare, WidgetBase, MenuItem, MenuHeading, domConstruct, registry, lang, Logger, Tooltip,
		_UsmiBrowserPersistence, on, focus, dom) {
	return declare([
		WidgetBase,
		_UsmiBrowserPersistence
	], {

		_nextGenUIMenuUtilsLogger : new Logger("ibm.usmi.menus", "NextGenUIMenuUtils"),

		getFunctionName : function (
		/*String*/pFunctionAndParams) {
			var startOfArgs;
			startOfArgs = pFunctionAndParams.indexOf('(');
			return pFunctionAndParams.slice(0, startOfArgs);
		},

		/**
		 * gets the function arguments from a string in the form of
		 * 'functionName(args)'
		 */
		getFunctionArgs : function (
		/*String*/pFunctionAndParams) {
			var startOfArgs, endOfArgs, args, i, returnArgs;
			returnArgs = [];
			startOfArgs = pFunctionAndParams.indexOf('(');
			endOfArgs = pFunctionAndParams.lastIndexOf(')');
			args = (pFunctionAndParams.slice(startOfArgs + 1, endOfArgs)).split(",");
			for (i = 0; i < args.length; i++) {
				returnArgs.push(args[i].slice(1, args[i].length - 1));
			}
			return returnArgs;
		},

		/**
		 * gets the inner function and parameters from a string in the form of
		 * 'functionName(innerFunction(args))'
		 */
		getInnerFunctionAndParams : function (
		/*String*/pFunctionAndParams) {
			var startOfArgs, endOfArgs;
			startOfArgs = pFunctionAndParams.indexOf('(');
			endOfArgs = pFunctionAndParams.lastIndexOf(')');
			return pFunctionAndParams.slice(startOfArgs + 1, endOfArgs);
		},

		/**
		 * creates a menu item for the menu param pName: the title of the menu
		 * item param pColumn: the menu column to put the menu item in param
		 * pTask: the task link the menu item launches. The task link will be
		 * either in the form of function(params) or
		 * function(innerFunction(params))
		 */
		createMenuItem : function (
		/*String*/
		pName, /*String*/
		pColumn, /*String*/
		pTask, /*Resource bundle*/MRI) {
			var functionName, func, args, menuItem, innerFunctionCall, innerFunction, menuLabel;

			if (MRI[pName]) {
				menuLabel = MRI[pName];
			} else {
				menuLabel = pName;
			}
			menuItem = new MenuItem({
				"label" : menuLabel,
				"column" : pColumn,
				"class" : "mi-workitem-item"
			});

			if (pTask) {
				functionName = this.getFunctionName(pTask);
				func = window[functionName];
				args = this.getFunctionArgs(pTask);
				//check to make sure the args don't contain another function that must be executed
				if (args[0].indexOf('(') === -1) {
					menuItem.set("onClick", lang.hitch(menuItem, function () {
						if (this.tooltip) {
							this.tooltip.close();
						}
						// [ Defect 187346
						// Close the drop down menu dialog, it stays open sometimes.
						if (menuItem.getParent()) {
							if (menuItem.getParent().getParent()) {
								if (menuItem.getParent().getParent().close) {
									menuItem.getParent().getParent().close();
								}
							}
						}
						// Defect 187346]
						func.apply(null, args);
					}));
					if (functionName.length >= 15 && functionName.substr(0, 15) === 'launchInContext') {
						menuItem.set("class", "mi-workitem-item launchInContext");
						menuItem.set("label", menuLabel + "  ");
						menuItem.set("onBlur", lang.hitch(menuItem, function () {
							if (focus.curNode === menuItem.domNode) {
								//for FF/Chrome, while the MenuBarItem is hover and opened, then the focus manager
								//would set the focus to the parent menu node, not the menu item node, and in focus manager
								//if we got a blur event, normally the focus.curNode is null, here it's not null and equal
								//the menu item node, refocus it to the menu item node, and does not remove the launch in context
								//icon
								focus.focus(this.focusNode);
								return;
							}
							if (this.tooltip) {
								this.tooltip.close();
							}
							if (this.imgElement) {
								domConstruct.destroy(this.imgElement);
								this.imgElement = null;
							}
						}));
						menuItem.productName = this.getGlobalData("brandedProductKey", false);
						var tooltip = new Tooltip({
							connectId : [ menuItem.containerNode
							],
							label : lang.replace(MRI.LiCTooltip, [ menuItem.productName
							])
						});
						menuItem.set("tooltip", tooltip);
						menuItem.set("onFocus", lang.hitch(menuItem, function () {
							if (!this.imgElement) {
								var containerNode = this.get("containerNode");
								var img = dojo.doc.createElement('img');
								dojo.attr(img, {
									"src" : "js/ibm/usmi/core/framework/themes/images/ac10_opennewtab_24.png",
									"alt" : "launch in context image"
								});
								domConstruct.place(img, containerNode);
								this.set("imgElement", img);
							}

						}));
					}
				}
				//the params are another function that needs to be resolved
				else {
					innerFunctionCall = this.getInnerFunctionAndParams(pTask);
					innerFunction = window[this.getFunctionName(innerFunctionCall)];
					args = this.getFunctionArgs(innerFunctionCall);
					menuItem.set("onClick", function () {
						var itemLink = innerFunction(args[0]);
						func(itemLink);
					});
				}
			}

			return menuItem;
		},

		/**
		 * creates a menu heading for the menu param pName: the header title
		 * param pColumn: the menu column to put the header in
		 */
		createMenuHeading : function (
		/*String*/
		pName, /*String*/
		pColumn, /*Resource Bundle*/MRI) {
			var headingName = MRI[pName];
			return new MenuHeading({
				"label" : headingName,
				"column" : pColumn,
				"class" : "mi-workitem-category"
			});
		},

		/**
		 * create an image for the menu. It's created as a menu heading so that
		 * it's not hoverable or clickable param pImagePath: the path to the
		 * image param pColumn: the menu column to put the image in
		 */
		createMenuHeadingImage : function (
		/*String*/
		pImagePath, /*String*/
		pColumn) {
			var heading = new MenuHeading({
				"column" : pColumn,
				"class" : "megaMenuIcon"

			}), img = dojo.doc.createElement('img');

			dojo.attr(img, {
				"src" : pImagePath,
				"alt" : "Menu image"
			});
			domConstruct.place(img, heading.containerNode);

			return heading;
		},

		/**
		 * creats a line of links to add to the menu param pLinks: an array of
		 * link objects that contain the properties "link" and "text". The
		 * "link" property is the hyperlink, the "text" is the text that should
		 * be displayed for the link
		 */
		createLinks : function (
		/*array*/
		pLinks, /*Resource Bundle*/MRI) {
			var linksDiv, hrElement, verticalRuleElement, hrefElement, linksTitle, i;
			linksDiv = dojo.doc.createElement('div');
			dojo.attr(linksDiv, {
				"class" : "nextGenMenuLinks"
			});
			hrElement = dojo.doc.createElement('hr');
			dojo.attr(hrElement, {
				"class" : "nextGenMenuHorizontalRule"
			});
			domConstruct.place(hrElement, linksDiv);
			if (pLinks) {
				var productName = this.getGlobalData("brandedProductKey", false);
				for (i = 0; i < pLinks.length; i++) {
					if (i !== 0) {
						verticalRuleElement = dojo.doc.createElement('span');
						dojo.attr(verticalRuleElement, {
							"class" : "nextGenMenuVerticalRule"
						});
						domConstruct.place(verticalRuleElement, linksDiv);

					}
					linksTitle = pLinks[i].title;
					hrefElement = dojo.doc.createElement('span');
					hrefElement.setAttribute("class", "nextGenMenuLink");
					on(hrefElement, "click", lang.hitch(pLinks[i], function () {
						eval(this.task);
					}));

					if (MRI[linksTitle]) {
						hrefElement.innerHTML = MRI[linksTitle];
					} else {
						hrefElement.innerHTML = linksTitle;
					}
					domConstruct.place(hrefElement, linksDiv);

					if (pLinks[i].task.length >= 15 && pLinks[i].task.substr(0, 15) === 'launchInContext') {
						var img = dojo.doc.createElement('img');
						dojo.attr(img, {
							"src" : "js/ibm/usmi/core/framework/themes/images/ac10_opennewtab_24.png",
							"alt" : "launch in context image",
							"style" : "padding-left:5px; display:none;"
						});
						var tooltip = new Tooltip({
							connectId : [ hrefElement
							],
							position : [ "below"
							],
							label : lang.replace(MRI.LiCTooltip, [ productName
							])
						});
						on(hrefElement, "mouseover", function () {
							this.nextSibling.style.display = "inline";
						});
						on(hrefElement, "mouseout", function () {
							this.nextSibling.style.display = "none";
						});
						domConstruct.place(img, linksDiv);
					}
				}
			}
			return linksDiv;
		},

		/**
		 * should just be able to use dijit.byId(pId) but for some reason it is
		 * returning null. So using own function for now...
		 */
		getWidgetFromRegistry : function (
		/*String*/
		pId) {
			var widgetArray, i, wid;
			if (this._nextGenUIMenuUtilsLogger.isFinest()) {
				this._nextGenUIMenuUtilsLogger.entering("pId: " + pId, "getWidgetFromRegistry");
			}
			widgetArray = registry.toArray();

			for (i = 0; i < widgetArray.length; i++) {
				wid = widgetArray[i];
				if (wid.id === pId) {
					if (this._nextGenUIMenuUtilsLogger.isFinest()) {
						this._nextGenUIMenuUtilsLogger.exiting("widget found, returning widget",
								"getWidgetFromRegistry");
					}
					return wid;
				}
			}

			if (this._nextGenUIMenuUtilsLogger.isFinest()) {
				this._nextGenUIMenuUtilsLogger.exiting("widget not found, returning null", "getWidgetFromRegistry");
			}
			return null;
		},

		constructor : function (args) {
			dojo.safeMixin(this, args);
		}
	});

});
