/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
/*global define */
define(
	"ibm/usmi/scheduler/JobLaunchWizard", [ "dojo/_base/declare",
		"dojo/_base/lang",
		"dojo/_base/array",
		"dojo/date/locale",
		"dojo/topic",
		"dojo/query",
		"dojoe/messagedialog/MessageDialog",
		"dijit/_WidgetBase",
		"dijit/_TemplatedMixin",
		"dijit/_WidgetsInTemplateMixin",
		"dijit/layout/StackContainer",
		"dijit/layout/BorderContainer",
		"dijit/layout/ContentPane",
		"dijit/form/Button",
		"dijit/form/RadioButton",
		"dijit/form/NumberSpinner",
		"dijit/form/CheckBox",
		"dijit/form/ValidationTextBox",
		"dijit/form/DateTextBox",
		"dijit/registry",
		"dijit/form/Select",
		"dijit/form/MultiSelect",
		"dojox/form/TimeSpinner",
		"ibm/usmi/core/utils/Logger",
		"ibm/usmi/util/BaseWidget",
		"ibm/usmi/widgets/util/Constants",
		"ibm/usmi/widgets/wizard/DirectorBorderContainer",
		"ibm/usmi/widgets/wizard/DirectorWizard",
		"ibm/usmi/widgets/wizard/DirectorPage",
		"./wizard_pages/SchedulePage",
		"./wizard_pages/NotificationsPage",
		"./wizard_pages/OptionsPage",
		"ibm/tivoli/navmodel/Dataset",
		"ibm/tivoli/navmodel/Selection",
		"ibm/usmi/util/DefaultMessageHandler",
		"dojo/i18n!./wizard_pages/nls/JobLaunchWizardResources"
	],
	function (
		declare, lang, array, locale, topic, query, MessageDialog, _WidgetBase, _TemplatedMixin,
		_WidgetsInTemplateMixin, StackContainer, BorderContainer, ContentPane, Button, RadioButton, NumberSpinner,
		CheckBox, ValidationTextBox, DateTextBox, registry, Select, MultiSelect, TimeSpinner, Logger, BaseWidget, Constants,
		DirectorBorderContainer, DirectorWizard, DirectorPage, SchedulePage, NotificationsPage, OptionsPage, Dataset,
		Selection, DefaultMessageHandler, mri) {

		// I declare that I am extending support.wizard.DirectorWizard, and BaseWidget
		return declare(
			"ibm.usmi.scheduler.JobLaunchWizard",
			[ DirectorWizard,
				BaseWidget ],
			{
				_dataset : null,
				persistResources : mri,

				iTaskId : null,
				jobName : null,

				//array of MEIDs to target
				MEIDs : null,

				taskSpecificParameters : null,

				publishReturn : null, // will set to the return we want to pass back to callers

				taskParms : null,
				tasks : [],

				// finishLabel : "**NLS**  Submit  **NLS**", // this setting appears to have no impact...I'd needed to do the this.resetFinishLabel call in the postCreate method....
				layout : "horizontal",
				//If noarrows is "false" wizard will be displayed with arrows, if it is "true" wizard will be displayed without arrows.	
				renderNoArrows : false,

				//If showCancelWarning is "true" confirmation message dialog will  be displayed on click of cancel, otherwise it will not be displayed. Default is false. 
				showCancelWarning : true,

				SCHEDULE_JOB_TASK_ID : "UsmiJobLauncherScheduleTask",
				RUN_JOB_NOW_TASK_ID : "UsmiJobLauncherRunNowTask",

				// Logger
				JobLaunchWizardLogger : new Logger("ibm.usmi.scheduler", "JobLaunchWizard"),

				// This method is called when a page in the table of contents is selected.  Only allow them to jump around if the current page 
				// is valid....
				jumpBackToPage : function (
					aPage) {

					// get the current page
					var curr_page = this._pageContainer.selectedChildWidget;

					// only allow us to jump to a different page if the current page is valid...
					if (curr_page.isValid()) {
						this.inherited(arguments);
					}
				},

				postCreate : function () {
					this.inherited(arguments);

					// retrieve the launch parameters...
					this.retrievelaunchParms();

					// retrieve the tasks related to the ITaskId
					this.retrieveTasks();

					this.resetFinishLabel(this.persistResources.SUBMIT_BUTTON_LABEL); // this was needed to set the button text since the "finishLabel" setting appeared to have no impact on the constructor above

					// If we don't yet have a job name (it wasn't passed in to us), we'll try to retrieve it from the task we retrieved
					if (!this.jobName) {
						// retrieve the job name from the first task we've got
						this.jobName = this.tasks[0].getLabel();
					}

					// Add the wizard pages....
					// First add the schedule page -- indicate it's required, and pass in the job name to be used as the base for the 
					// default job name....
					this.addPage(new SchedulePage({
						required : true,
						jobNameBase : this.jobName
					}));

					this.addPage(new NotificationsPage());
					this.addPage(new OptionsPage());

					// Attempt to set the title of the dialog to the job name
					if (this.launchParms) {
						this.launchParms.dialogTitle = this.jobName;
					}

					// Set the dialog size (so it displays)
					this.resize({
						w : 700,
						h : 700
					});

					// default our return value to "cancel" (assume the user will be exiting the 
					// dialog via the cancel button)
					this.publishReturn = Constants.TASK_LAUNCH_CANCELED;

				},

				retrievelaunchParms : function () {
					// get the values from the launch parameters 

					// First get the task ID -- REQUIRED:  we can't continue without one
					if (this.launchParms) {
						this.iTaskId = this.launchParms.taskID;
					} // else, NO launch parms -- iTaskId may have been passed directly to us, so we'll wait to display any error message....

					// set the target context for the task -- Optional:  this may not be a targeted task....or the MEIDs may have been passed directly to us on
					// the constructor....
					if (this.launchParms) {
						if (this.launchParms.MEIDS) {
							var mesSplit = this.launchParms.MEIDS.split(',');
							if (mesSplit.length > 0) {
								this.MEIDs = [];
								for ( var j = 0; j < mesSplit.length; j++) {
									this.MEIDs[j] = parseInt(mesSplit[j], 10);
								}
							}
						}
					}
				},

				retrieveTasks : function () {
					var moduleName = "retrieveTasks";
					var msgHandler = new DefaultMessageHandler();
					var msgId = ""; // not specifying a message ID
					var theMessageTitle = "";
					var theOperatorResponse = "";
					var msgSeverity = DefaultMessageHandler.ERROR;

					if (!this.iTaskId) {
						// This should never happen....but in case it does, we don't want to just not open and give no indication as to why...issue our generic error message.
						// First, log what's happened....
						if (this.JobLaunchWizardLogger.isError()) {
							this.JobLaunchWizardLogger
								.error(
									"JobLaunchWizard:  called without an iTaskId being passed in.  The job launch cannot continue.",
									moduleName);
						}

						// Build the error message.
						theMessageTitle = this.persistResources.GENERIC_ERROR_MESSAGE;
						theOperatorResponse = this.persistResources.GENERIC_ERROR_RESPONSE;
						msgHandler.displayMessage(msgId, msgSeverity, theMessageTitle, theOperatorResponse, true);
					} else {
						// First try to retrieve the task that the taskId is for:
						// Log the task ID that was passed in...
						if (this.JobLaunchWizardLogger.isFine()) {
							this.JobLaunchWizardLogger.fine("JobLaunchWizard:  The task ID that was passed in is:  " +
								this.iTaskId, moduleName);
						}

						var theDatasetValue = "JOB_LAUNCHER:" + this.iTaskId;
						try {
							this._dataset = new Dataset({
								baseUrl : "/ibm/tivoli/rest",
								provider : "USMI",
								datasource : "",
								dataset : theDatasetValue,
								parameters : {

								},
								defaultProperties : []
							});

							var tasks = this._dataset.getTasks({
								onComplete : lang.hitch(this, this._retrievedTasks),
								onError : lang.hitch(this, this._retrievedTasksError),
								taskCategory : "menu", //TODO: Could I specifically request nav hints instead of all tasks?
								itemIndexes : "0",
								sync : true
							});

						} catch (ex) {
							//There was a problem creating the dataset
							if (this.JobLaunchWizardLogger.isError()) {
								this.JobLaunchWizardLogger
									.error(
										"JobLaunchWizard:  A problem occurred creating our dataset threw an exception.  The task ID that was passed in is:  " +
											this.iTaskId, moduleName);
								this.JobLaunchWizardLogger.error(
									"JobLaunchWizard:  The exception is:  " + ex, moduleName);
							}

							msgHandler.displayException(ex);
						}

					}

				},

				_retrievedTasksError : function () {
					var msgHandler = new DefaultMessageHandler();
					var moduleName = "retrievedTasksError";
					var msgId = ""; // not specifying a message ID
					var theMessageTitle = "";
					var theOperatorResponse = "";
					var msgSeverity = DefaultMessageHandler.ERROR;

					// This should never happen....but in case it does, we don't want to just not open and give no indication as to why...issue our generic error message.
					if (this.JobLaunchWizardLogger.isError()) {
						this.JobLaunchWizardLogger
							.error(
								"JobLaunchWizard:  Action to retrieve the task was not successful.  The task ID that was passed in is:  " +
									this.iTaskId, moduleName);
					}
					theMessageTitle = this.persistResources.GENERIC_ERROR_MESSAGE;
					theOperatorResponse = this.persistResources.GENERIC_ERROR_RESPONSE;
					msgHandler.displayMessage(msgId, msgSeverity, theMessageTitle, theOperatorResponse, true);
				},

				_retrievedTasks : function (
					tasks) {
					// We got the appropriate tasks....save them away...
					this.tasks = tasks;
				},

				// Perform the activity.
				OnOK : function (
					wiz) {

					// build the full list of task parms needed to launch the task (will retrieve the user input from each of the schedule-related pages)
					this.buildTaskParameters();

					var execTask;

					// Retrieve the proper task -- Check to see if we are "schedule" or "run now"
					var taskToRetrieve = this.SCHEDULE_JOB_TASK_ID; // assume we are a scheduled job...
					// If the user chose the option to "run now", then we want a different task
					if (this.taskParms.StartDate === "RunNow") {
						taskToRetrieve = this.RUN_JOB_NOW_TASK_ID;
					}

					//Want to loop through the list of tasks we're initial retrieved looking for that correct task.
					if (this.tasks) {
						for ( var i = 0; i < this.tasks.length; i++) {
							var task = this.tasks[i];
							// found our task.
							if (task.getId() === taskToRetrieve) {

								execTask = task;

								break;
							}
						}
					}

					// execute the task....
					execTask.execute({
						onError : lang.hitch(this, this._handleExecuteOnServerError),
						onComplete : lang.hitch(this, this._getExecuteComplete),
						selection : new Selection({
							version : this._dataset.getVersion()
						}),
						selectionParms : this.taskParms,
						sync : true
					});

				},

				buildTaskParameters : function () {

					// define the values we will need to set to launch the task
					// schedule page information:
					var jobNameParm = null; // retrieve from schedule page.
					//var manageableElementsParm = null; // currently we do not support elements being passed in 
					//var startDateParm = null; // needs to be time in milliseconds from epoch that the job will begin running					
					var repeatOptionsParm = null;
					var frequency_option = null;
					var on_weekend_option = null;
					var forWeekDays_option = null;
					var forMonthDates_option = null;
					var customDates_option = null;
					var duration_option = null;

					// notification page information:
					var notificationOptionsParm = null;

					// additional options page options:
					var additionalOptionsParm = null;
					//var propertiesParm = null;

					// Retrieve the information from the Schedule page (page 0)....
					var schedulePage = this._pages[0];

					// retrieve the job name (required by every job)
					jobNameParm = schedulePage.jobName.get('value');

					var startDate = (new Date()).getTime(); // TODO:  initial value needed?  (perhaps just for this prototyping....)

					// retrieve the "runnow" radio button....if it's checked, all other scheduling information is moot:
					var runNow = schedulePage.runNowRadio.get('value');

					if (runNow === "now") {
						// If we know the user only wants to run the task "right now", then set that value for the start date
						startDate = "RunNow";
					} else {
						// retrieve the repeat option selected
						frequency_option = schedulePage.getRepeatSelectionValue();

						// retrieve the specific options based on the repeat option selected -- start with "custom" because 
						// it defines its own dates and times on which to run the job
						if (frequency_option === "CUSTOM") { // custom repeat option
							// "custom" appears to require only a list of custom dates/times
							// retrieve all of the custom dates
							var customDatesContainer = schedulePage.selectedCustomDates.containerNode;
							customDates_option = [];
							for ( var i = 0; i < customDatesContainer.options.length; i++) {
								customDates_option.push((new Date(customDatesContainer.options.item(i).value))
									.getTime());
							}
						} else { // we're something other than "CUSTOM"
							// the remaining repeat options require a date and time on which to begin the scheduled job...
							// retrieve the start date (scheduled date and time) -- built from the scheduleDate field and scheduleTime field...				
							var startDate_date = schedulePage.scheduleDate.get('value');
							var startDate_time = schedulePage.scheduleTime.get('value');

							// create our start date based off the date field.
							startDate = startDate_date;

							// retrieve the hours and minutes from the time field
							var hours = locale.format(startDate_time, {
								timePattern : "H",
								selector : "time"
							});
							var mins = locale.format(startDate_time, {
								timePattern : "mm",
								selector : "time"
							});
							// set the time field on our start date
							startDate.setHours(hours, mins);

							// need the time in milliseconds from epoch -- so get the time from startDate
							// you can verify that time via this Epoch 
							// converter: http://www.epochconverter.com/
							startDate = (startDate.getTime());

							if (frequency_option !== "ONCE") { // We're not on a repeat once schedule (and we know we're not a custom schedule)
								// Retrieve the Duration value (common to all repeat option except custom/once)					
								var theDuration = schedulePage.forDurationRadio.get('value');

								var durationType = null;
								var durationValue = null;

								if (theDuration === "for") {
									durationType = schedulePage.getDurationRepeatForUnitsValue(); // the units like weeks/months/years
									durationValue = schedulePage.getDurationRepeatForValue(); // the value (like "6")
								} else { // else we're "until" or "forever"
									theDuration = schedulePage.untilDurationRadio.get('value');
									if (theDuration === "until") { // we're "until"
										durationType = "UNTIL_ENDDATE";

										durationValue = ((new Date(schedulePage.repeatUntil.get('value'))).getTime());
									} else { // must be repeat forever (unlimited)....there is no additional duration information that needs to be retrieved.
										durationType = "FOREVER";
									}
								}

								// define the duration option
								duration_option = {
									Type : durationType,
									Value : durationValue
								};

								// the "weekly" repeat option is the only one that does not require value for what to do on weekends...
								if (frequency_option !== "WEEKLY") { // if we're not doing the weekly repeat option, retrieve the weekend choice
									on_weekend_option = schedulePage.getWeekEndOptionValue();
								} // end repeat weekly

								// the "monthly" repeat option also requires the additional "dates" value.
								if (frequency_option === "MONTHLY_DATES") { // monthly repeat options
									// retrieve the monthly dates values...(just like was done for custom dates)
									// "monthly" appears to require only a list of dates of the month
									// retrieve all of the monthly dates 
									var monthlyDatesContainer = schedulePage.selectedMonthlyDates.containerNode;
									forMonthDates_option = [];
									for ( var j = 0; j < monthlyDatesContainer.options.length; j++) {
										var aMonthlyDate = monthlyDatesContainer.options.item(j).value;
										var intMonthlyDate = parseInt(aMonthlyDate, 10);

										forMonthDates_option.push(intMonthlyDate);
									}
								} // end repeat monthly

								// If we're schedule to run weekly, determine on which days of the week we'll run
								if (frequency_option === "WEEKLY") {
									forWeekDays_option = [];
									if (schedulePage.sundayDay.get('value')) {
										forWeekDays_option.push("SUNDAY");
									}
									if (schedulePage.mondayDay.get('value')) {
										forWeekDays_option.push("MONDAY");
									}
									if (schedulePage.tuesdayDay.get('value')) {
										forWeekDays_option.push("TUESDAY");
									}
									if (schedulePage.wednesdayDay.get('value')) {
										forWeekDays_option.push("WEDNESDAY");
									}
									if (schedulePage.thursdayDay.get('value')) {
										forWeekDays_option.push("THURSDAY");
									}
									if (schedulePage.fridayDay.get('value')) {
										forWeekDays_option.push("FRIDAY");
									}
									if (schedulePage.saturdayDay.get('value')) {
										forWeekDays_option.push("SATURDAY");
									}
								}

								//else if (repeatOption === "YEARLY") {
								// Repeat yearly...there are no additional values that need to be retrieved.
								//} else if (repeatOption === "HOURLY") {
								// Repeat hourly....there are no additional values that need to be retrieved.						
								//} else if (repeatOption === "DAILY") {
								// Repeat daily....there are no additional values that need to be retrieved.
								//}

							} // end we are NOT a repeat "once" schedule 

						} // end we are not NOT a custom date

						// At this point, we should have all of the information from the schedule page properly set
						repeatOptionsParm = {
							Frequency : frequency_option,
							OnWeekEnds : on_weekend_option,
							ForWeekDays : forWeekDays_option,
							ForMonthDates : forMonthDates_option,
							CustomDates : customDates_option,
							Duration : duration_option
						};
					} // else we're not "run now" -- we've now set all our repeatOptions....

					// Retrieve the information from the Notifications page (page 1)
					var notificationsPage = this._pages[1];
					// Retrieve the notify flags
					var notifyTrigger = [];
					//var notifyTrigger = null;
					// retrieve the flag for notify when the task begins
					if (notificationsPage.whenJobBegins.get('value')) {
						notifyTrigger.push("JOB_BEGINS");
					}
					if (notificationsPage.whenJobEndsSuccessfully.get('value')) {
						notifyTrigger.push("JOB_COMPLETES");
					}
					if (notificationsPage.whenJobFails.get('value')) {
						notifyTrigger.push("JOB_FAILS");
					}

					// retrieve the email information...
					var emailAddr = notificationsPage.notificationEmailAddress.get('value');
					var serverAddr = notificationsPage.notificationEmailServerName.get('value');
					var serverPort = parseInt(notificationsPage.notificationEmailServerPort.get('value'), 10);

					// retrieve the errorThresholdType
					var thresholdTypeAny = notificationsPage.anyErrorRadio.getValue('value');
					var thresholdTypePercent = notificationsPage.percentFailedRadio.getValue('value');
					var thresholdTypeNumber = notificationsPage.numberFailedRadio.getValue('value');
					
					// retrieve the errorThreshold value
					var thresholdPercentFailed = notificationsPage.thresholdPercentFailed.getValue('value');
					var thresholdNumberFailed = notificationsPage.thresholdNumberFailed.getValue('value');

					// At this point, we should have all of the information from the notifications page properly set
					if (thresholdTypePercent === 'true') {
						notificationOptionsParm = {
						NotifyTrigger : notifyTrigger,
						PercentTargetsFailed : thresholdPercentFailed,
						EmailAddress : emailAddr,
						EmailServer : serverAddr,
						EmailServerPort : serverPort
						};
					} else if (thresholdTypeNumber) {
						notificationOptionsParm = {
						NotifyTrigger : notifyTrigger,
						NumberTargetsFailed : thresholdNumberFailed,
						EmailAddress : emailAddr,
						EmailServer : serverAddr,
						EmailServerPort : serverPort
						};
					} else {
						notificationOptionsParm = {
						NotifyTrigger : notifyTrigger,
						EmailAddress : emailAddr,
						EmailServer : serverAddr,
						EmailServerPort : serverPort
						};
					}
					/*
					notificationOptionsParm = {
						NotifyTrigger : notifyTrigger,
						PercentTargetsFailed : thresholdPercentFailed,
						NumberTargetsFailed : thresholdNumberFailed,
						EmailAddress : emailAddr,
						EmailServer : serverAddr,
						EmailServerPort : serverPort
					};
					*/

					// Retrieve the information from the Options page (page 2)
					var optionsPage = this._pages[2];

					// retrieve the flag using server time
					var useManagementServerTime = (optionsPage.useMgmtServerTime.get('value') === "true"); // boolean: true (use mgmt server time)/false (use endpoint time)

					// retrieve the flag for 
					var failUnavailableSystems = (optionsPage.failSystems.get('value') !== "true"); // boolean:  true (run when avail)/false (fail) 

					// retrieve the maximum number of job instances
					var numberOfInstances = optionsPage.maxiumJobInstances.get('value');

					// At this point, we should have all of the information from the options page properly set
					additionalOptionsParm = {
						ManagementServerTime : useManagementServerTime,
						RunWhenAvailable : failUnavailableSystems,
						MaxActivationHistory : numberOfInstances
					};

					// Now, build the taskParms....
					this.taskParms = {
						DisplayName : jobNameParm, // the name of the job (from the schedule page)

						ManageableElements : this.MEIDs, // the target context of the job (passed in to me)

						// date and time to start the task (from the schedule page).  
						// Will either be the number of milliseconds from epoch OR it will have a special value:  "RunNow"
						StartDate : startDate,

						// the task repeat information (from the schedule page) -- this could be null if the user chose to "RunNow"
						RepeatOptions : repeatOptionsParm,

						// the notification options for the task (from the notification page)
						NotificationOptions : notificationOptionsParm,

						// the additional options for the task (from the options page)
						AdditionalOptions : additionalOptionsParm,

						// the task specific parameters that were passed in to us
						Properties : this.taskSpecificParameters
					};

				},

				_getExecuteComplete : function (
					result, arg) {
					var r = result;
					var a = arg;
					var moduleName = "_getExecuteComplete";

					// TODO:  Consider how to add a link to the job details in the message -- based on the job ID....r.JobId
					var theMessageTitle = "";
					if (this.taskParms.StartDate === "RunNow") {
						theMessageTitle = this.persistResources.JOB_SUCCESSFULLY_RUN_NOW + this.taskParms.DisplayName;
						if (this.JobLaunchWizardLogger.isFine()) {
							this.JobLaunchWizardLogger.fine(
								"JobLaunchWizard:  Successfully The created and started job with name: " +
									this.taskParms.DisplayName + " and job id: " + r.JobId, moduleName);
						}
					} else {
						theMessageTitle = this.persistResources.JOB_SUCCESSFULLY_SCHEDULED + this.taskParms.DisplayName;
						if (this.JobLaunchWizardLogger.isFine()) {
							this.JobLaunchWizardLogger.fine(
								"JobLaunchWizard:  Successfully The created and scheduled job with name: " +
									this.taskParms.DisplayName + " and job id: " + r.JobId, moduleName);
						}
					}

					// Indicate how the execute attempt resulted for our return value
					if (this.taskParms.StartDate === "RunNow") {
						this.publishReturn = Constants.TASK_LAUNCH_SUBMIT_SUCCESSFUL;
					} else {
						this.publishReturn = Constants.TASK_LAUNCH_SUBMIT_SUCCESSFUL;
					}
					
					// Close the dialog...
					this.OnCancel();

					// Display the message...
					var msgHandler = new DefaultMessageHandler();
					var msgId = ""; // not specifying a message ID
					var msgSeverity = DefaultMessageHandler.SUCCESS;

					// Note that currently, this method on msgHandler can take a list of targets and prepends them with 
					// "&sourceToken=" -- this doesn't work for launching into
					// the jobs properties -- it needs &DCMJobId=xxxx...so I'll manually build my own URL.
					// msgHandler.displayMessageWithLaunchInContext(msgId, msgSeverity, theMessageTitle, null, null, null, url, this.persistResources.VIEW_JOB_LINK_TEXT, false);

					// build the URL for the link to launch the job properties:
					// TODO:  Is it okay to launch without the task area and nav area -- and to hide the banner/taskbar area?
					var url = window.location.protocol +
						'//' +
						location.host +
						'/ibm/console/xLaunch.do?pageID=com.ibm.usmi.console.scheduler.navigationElement&hideTaskArea=false&disableNavArea=false&hideBannerArea=false&hideTaskBarArea=false';
					url = url + '&DCMJobId=' + r.JobId;

					msgHandler.displayMessageWithLaunchInContext(
						msgId, msgSeverity, theMessageTitle, null, null, null, url,
						this.persistResources.VIEW_JOB_LINK_TEXT, false);
				},

				_handleExecuteOnServerError : function (
					error, exception, ioArgs) {
					var err = error;
					var exc = exception;
					var a = ioArgs;
					var moduleName = "_handleExecuteOnServerError";

					if (this.JobLaunchWizardLogger.error()) {
						this.JobLaunchWizardLogger.error("JobLaunchWizard:  Failed to create job with name: " +
							this.taskParms.DisplayName, moduleName);
						this.JobLaunchWizardLogger
							.error("JobLaunchWizard:  The error received is:  " + err, moduleName);
						this.JobLaunchWizardLogger.error("JobLaunchWizard:  The exception is:  " + exc, moduleName);
					}

					var theMessageTitle = this.persistResources.JOB_CREATION_FAILED;

					var theOperatorResponse = this.persistResources.JOB_FAILED_RESPONSE;
					var theMessageDesription = exc.getMsgText();

					var msgDesc = theOperatorResponse + "<p>" + theMessageDesription;

					// Indicate how the execute attempt resulted for our return value
					if (this.taskParms.StartDate === "RunNow") {
						this.publishReturn = Constants.TASK_LAUNCH_SUBMIT_FAILED;
					} else {
						this.publishReturn = Constants.TASK_LAUNCH_SUBMIT_FAILED;;
					}
					
					// Close the dialog...
					this.OnCancel();

					// Display the message...as a popup
					var msgHandler = new DefaultMessageHandler();
					var msgId = ""; // not specifying a message ID
					var msgSeverity = DefaultMessageHandler.ERROR;
					msgHandler.displayMessage(msgId, msgSeverity, theMessageTitle, msgDesc, true);
				},

				//overriding from Wizard
				OnCancel : function (
					wiz) {
					// publish my dismissal
					topic.publish("DefaultDialogDismissal", []);

					// publish my result -- passing the proper return (Constants.TASK_LAUNCH_CANCELED, Constants.TASK_LAUNCH_SUBMIT_SUCCESSFUL, Constants.TASK_LAUNCH_SUBMIT_FAILED)
					topic.publish(Constants.TOPIC_JOB_LAUNCH_RESULT, [this.publishReturn]);

					this.destroyRecursive();
					this.inherited(arguments);
				},

				/*
				destroy : function () {
					// summary:
					//		Coming soon
					// description:
					//		Coming soon
					var buttonId =  this.id + "_DialogCancelButton";  //  this._outerContainer.id;  
					if (registry.byId(buttonId)) {
						var widgets = dijit.findWidgets(this);
						dojo.forEach(widgets, function (
							w) {
							w.destroyRecursive(true);
						});
						registry.byId(this.id).destroyRecursive();

					}
					
					this.inherited(arguments);
				},
				*/

				destroyRecursive : function () {
					this.inherited(arguments);
				},

				//overriding from Wizard
				_onConfirm : function (
					buttonPressedId, messageId, checked, wiz) {
					if (buttonPressedId === 0) {
						this.OnCancel(wiz);
					}
				},

				getBodyHeight : function () {
					return (0.1 * ((window.innerHeight !== undefined) ? window.innerHeight : document.body.offsetHeight));
				}

				/*
				//force load the Wizard.css, not sure why this doesn't get loaded by the Wizard widget
				load_css : function (
					cssFile) {
					var styleSheetLink = document.createElement('link');
					styleSheetLink.rel = 'stylesheet';
					styleSheetLink.type = 'text/css';
					styleSheetLink.href = 'js/dojoe/wizard/themes/claro/Wizard.css';
					query("head")[0].appendChild(styleSheetLink);
								}
				*/
			});
	});
