/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/statuspod/AlertStatusPod", [
	"ibm/tivoli/tip/TCTTable",
	"ibm/tivoli/tip/TCTNavModel",
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dijit/PopupMenuBarItem",
	"./StatusPodMenuDialog",
	"./AlertStatusPodDialog",
	"ibm/usmi/util/DataStore",
	"dojo/i18n!./nls/statuspodResources",
	"ibm/usmi/core/utils/_UsmiBrowserPersistence",
	"dojo/DeferredList",
	"dojo/_base/Deferred",
	'dojo/_base/unload',
	"./Constants" ], function (TCTTable, TCTNavModel, dojo, declare, lang, PopupMenuBarItem, StatusPodMenuDialog,
	AlertStatusPodDialog, DataStore, MRI, _UsmiBrowserPersistence, DeferredList, Deferred, baseUnload, Constants) {
	return declare("ibm.usmi.statuspod.AlertStatusPod", [
		PopupMenuBarItem,
		_UsmiBrowserPersistence ],
		{
			label : MRI.StatusPodLoading,

			dialog : null,
			listeningDataset : null,
			problemsDataset : null,
			complianceDataset : null,
			problemHandler : null,
			complianceHandler : null,
			menuParent : null,
			blockEvents : true,
			pendingEvents : false,
			_PERSISTENCE_ALERTS_ID : "ibm.usmi.statuspod.AlertStatusPod.contents",
			_PERSISTENCE_ALERTS_COUNT : "ibm.usmi.statuspod.AlertStatusPod.contents.count",

			problemsGroupProperties : [
				"DisplayName",
				"ObjectType",
				"STATUS",
				"STATUS_CAT:ComplianceStatusCategory",
				"STATUS_CAT:Problems",
				"STATUSPOD_SORT:Problems"],
				
			compliancesGroupProperties : [
				 "DisplayName",
				 "ObjectType",
				 "STATUS",
				 "STATUS_CAT:ComplianceStatusCategory",
				 "STATUS_CAT:Problems",
				 "STATUSPOD_SORT:ComplianceStatusCategory"],

			problemsGroupSort : [ {
				attribute : "STATUSPOD_SORT:Problems",
				descending : true
			} ],
			
			nonCompliancesGroupSort : [ {
				attribute : "STATUSPOD_SORT:ComplianceStatusCategory",
				descending : true
			} ],


			postCreate : function () {
				this.inherited(arguments);
				this.dialog = new AlertStatusPodDialog();
				this.popup = new StatusPodMenuDialog({
					content : this.dialog
				});
			},
			startup : function () {
				this.inherited(arguments);
				this._createDatasetListener();
				this._refreshDatasets();
			},
			
	        destroy : function () {
	            // clean up the datastore
				if (this.listeningDataset) {
					this.listeningDataset.getDataset().removeEventListener(this);
					this.listeningDataset.destroy();
					this.listeningDataset = null;
				}
			
				if (this.problemsDataset) {
					this.problemsDataset.destroy();
					this.problemsDataset = null;
				}
				if (this.complianceDataset) {
					this.complianceDataset.destroy();
					this.complianceDataset = null;
				}
					
	            this.inherited(arguments);
	        },
			_refreshDatasets : function () {
				var cache = this.getData(this._PERSISTENCE_ALERTS_ID, true);
				if (cache) {
					this._refreshProblems(cache.problems, true);
					this._refreshCompliance(cache.compliance, true);
					this._updateTopElement(true);
					return;
				}
				if (this.problemsDataset) {
					this.problemsDataset.destroy();
				}
				if (this.complianceDataset) {
					this.complianceDataset.destroy();
				}
				this.problemHandler = new Deferred();
				this.complianceHandler = new Deferred();
				var handlers = new DeferredList([
					this.problemHandler,
					this.complianceHandler ]);
				handlers.then(lang.hitch(this, function (result) {
					// only remove on Destroy so we don't do this at logout. 
					// deleting datasets at logout casues redirect popups
					//baseUnload.addOnUnload(this.problemsDataset.getDataset(), "destroy");
					//baseUnload.addOnUnload(this.complianceDataset.getDataset(), "destroy");
					this.setData(this._PERSISTENCE_ALERTS_ID, this.dialog.cache, true);
					this._updateTopElement();
					this.dialog.clearCache();

				}));
				this.problemsDataset = this.makeDataSet("SYSTEM_STATUSPOD", lang
					.hitch(this, this._refreshProblems), this.problemsGroupProperties, {
					groupId:"STATUS_PROBLEMS",
					STATUS_CAT: "Problems"
				}, this.problemsGroupSort);
				this.complianceDataset = this.makeDataSet("SYSTEM_STATUSPOD", lang.hitch(this,
					this._refreshCompliance), this.compliancesGroupProperties, {
					groupId:"STATUS_COMPLIANCE",
					STATUS_CAT: "ComplianceStatusCategory"
				}, this.nonCompliancesGroupSort);

			},
			_updateTopElement : function (fromCache) {
				var problemItems = this.dialog.statusItems;
				var complianceItems = this.dialog.complianceItems;
				var idx = 0, idx2 = 0, itemId, duplicates = 0;
				if (complianceItems) {
					for (idx=0; idx < complianceItems.length; idx++) {
						itemId = complianceItems[idx].itemId;
						for (idx2 = 0; idx2 < problemItems.length; idx2++) {
							if (itemId === problemItems[idx2].itemId) {
								duplicates++;
								break;
							}
						}
					}

				}
				var totalCounter = problemItems.length + complianceItems.length - duplicates;
				dojo.byId('alertStatusPodHeaderText').innerHTML = lang.replace(MRI.StatusHeaderText, [ totalCounter ]);
				var overallStatusIcon = this.determineOverallStatusIcon(this.determineStatusIcon(problemItems), this
					.determineStatusIcon(complianceItems));
				this.label = "<span class='" + overallStatusIcon + "'></span>"
					+ lang.replace(MRI.Status, [ totalCounter ]);
				this.containerNode.innerHTML = this.label;
				window.setTimeout(dojo.hitch(this, this._unblockEvents), '5000');
			},
			_refreshProblems : function (items, fromCache) {
				this.dialog.updateProblems(items, fromCache);
				if (this.problemHandler) {
					this.problemHandler.resolve();
				}
			},
			_refreshCompliance : function (items, fromCache) {
				this.dialog.updateCompliances(items, fromCache);
				if (this.complianceHandler) {
					this.complianceHandler.resolve();
				}
			},
			makeDataSet : function (datasetId, callback, groupProperties, parameters, sortProp) {
				return new DataStore({
					datasetId : datasetId,
					getTasks : false,
					initReadyCallback : callback					
				}, {
					sort : sortProp,
					scope : this,
					defaultProperties : groupProperties,
					createEventRouter : false,
					clearCacheOnRefresh : true,
					parameters: parameters
				//do not create event router
				});
			},

			determineStatusIcon : function (items) {
				var overallStatusIcon = Constants.STATUS_CLASS_NORMAL;
				if (!items || items.length === 0) {
					return overallStatusIcon;
				}
				var item = items[0];
				//var statusVal = item.getProperty("STATUS").getValue().getValue();
				var statusVal = item.severityId;
				if (statusVal === 1) {
					overallStatusIcon = Constants.STATUS_CLASS_WARNING;
				} else if (statusVal === 0) {
					overallStatusIcon = Constants.STATUS_CLASS_ERROR;
				} else if (statusVal === 5) {
					overallStatusIcon = Constants.STATUS_CLASS_UNKNOWN;
				}
				return overallStatusIcon;
			},

			determineOverallStatusIcon : function (statusProblems, statusCompliances) {
				if (statusProblems === Constants.STATUS_CLASS_ERROR
					|| statusCompliances === Constants.STATUS_CLASS_ERROR) {
					return Constants.STATUS_CLASS_ERROR;
				}
				if (statusProblems === Constants.STATUS_CLASS_WARNING
					|| statusCompliances === Constants.STATUS_CLASS_WARNING) {
					return Constants.STATUS_CLASS_WARNING;
				}
				
				if (statusProblems === Constants.STATUS_CLASS_NORMAL
						&& statusCompliances === Constants.STATUS_CLASS_UNKNOWN) {
						return Constants.STATUS_CLASS_UNKNOWN;
				}
				
				return Constants.STATUS_CLASS_NORMAL;
			},
			_setEventHandler : function () {
				this.listeningDataset.getDataset().addEventListener(this, lang.hitch(this, this._cleanDialog));
				var cachedCount = this.getData(this._PERSISTENCE_ALERTS_COUNT, false);
				var currentCount = this.listeningDataset.getDatasetAttribute("STATUS_COUNT");

				if (cachedCount) {
					if (cachedCount != currentCount) {
						this._cleanDialog();
					}
				} else {
					this.setData(this._PERSISTENCE_ALERTS_COUNT, currentCount, false);
				}
			},
			_createDatasetListener : function () {
				if (this.listeningDataset) {
					this.listeningDataset.destroy();
				}
				this.listeningDataset = new DataStore({
					datasetId : 'StatusPodCollection',
					getItems : false,
					retrieveItemProperties : false,
					getTasks : false,
					initReadyCallback : lang.hitch(this, this._setEventHandler)
				});
			},
			_cleanDialog : function () {
				if (this.blockEvents) { //Refresh is in progress, don't refresh
					this.pendingEvent = true; //save the need to refresh again later
					return;
				}
				this.blockEvents = true;
				this.pendingEvent = false;
				this.removeData(this._PERSISTENCE_ALERTS_ID);
				this.removeData(this._PERSISTENCE_ALERTS_COUNT);
				this._createDatasetListener();
				this.dialog.clearDescendants();
				this.label = MRI.StatusPodRefreshing;
				this.containerNode.innerHTML = this.label;
				dojo.byId('alertStatusPodHeaderText').innerHTML = MRI.StatusHeaderTextRefreshing;
				this._refreshDatasets();

			},
			_unblockEvents : function () {
				this.blockEvents = false; //Refresh is done, we can now listen for events again
				if (this.pendingEvent) { //did any more events come in during our refresh? if so, refresh again
					this._cleanDialog();
				}
			}
		});
});
