/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/statuspod/JobStatusPod", [
	"ibm/tivoli/tip/TCTTable",
	"ibm/tivoli/tip/TCTNavModel",
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dijit/PopupMenuBarItem",
	"ibm/tivoli/tip/navwidget/dijit/DefaultMessageHandler",
	"./JobStatusDataSet",
	"idx/oneui/MenuDialog",
	"./JobStatusPodDialog",
	"dojo/DeferredList",
	"dojo/_base/Deferred",
	'dojo/_base/unload',
	"./Constants",
	"dojo/_base/connect",
	"dojo/string",
	"dojo/i18n!./nls/statuspodResources",
	"ibm/usmi/core/utils/Logger"
], function (TCTTable, TCTNavModel, dojo, declare, lang, PopupMenuBarItem, DefaultMessageHandler, Dataset,
		MenuDialog, JobStatusPodDialog, DeferredList, Deferred, baseUnload, Constants, connect, string, MRI, Logger) {
	return declare("ibm.usmi.statuspod.JobStatusPod", [ PopupMenuBarItem
	], {
		label : MRI.JobMenuBarItemLoadingLabel,

		dialog : null,

		dataset : null,

		jobsCallbacks : null,
		
		jobsCallbackHanlders: null,
		
		originCallback: null,

		parentIds : [
			"Jobs",
			"Errors",
			"Active",
			"Completed"
		],
		
		jobDatasetName: "SCHEDULER_STATUSPOD",
		
		sortProp : [ {
			attribute : "LAST_RUN",
			descending : true
		}
		],
		
		totalCount: 0,
		
		iconClass: "status-msgs-icon-confirmation",
		
		_logger : new Logger("ibm.usmi.statuspod", "JobStatusPod"),
		
		blockEvents : true,
		pendingEvents : 0,

		postCreate : function () {
			this.inherited(arguments);
			this.dialog = new JobStatusPodDialog();
			this.popup = new MenuDialog({
				content : this.dialog
			});
		},

		startup : function () {
			this._logger.finest("startup job status pod", "startup");
			this.inherited(arguments);
			connect.subscribe(Constants.updateJobsLabel, this, function (message) {
				this.updateLabel(message);
			});
			this.jobsCallbacks = [
				lang.hitch(this.dialog, this.dialog.updateParentsCountNumber),
				lang.hitch(this.dialog, this.dialog.updateErrorJobs),
				lang.hitch(this.dialog, this.dialog.updateActiveJobs),
				lang.hitch(this.dialog, this.dialog.updateCompletedJobs)
			];
			this.jobsCallbackHanlders = [
			    new Deferred(),  new Deferred(),  new Deferred(),  new Deferred()
			];
			this.originCallback=new Deferred();
			this.makeDataSet();

		},

		refreshCallback : function () {
			var log=this._logger;
			log.finest("job status pod refresh callback was called", "refreshCallback");
			if (this.blockEvents) { //Refresh is in progress, don't refresh
				this._logger.finest("job status pod refresh callback is blocking, set pending state", "refreshCallback");
				this.pendingEvents = this.pendingEvents+1; //save the need to refresh again later
				return;
			}
			this.blockEvents = true;
			this.pendingEvents = 0;
			this.label = MRI.JobPodRefreshing;
			this.containerNode.innerHTML = this.label;
			this.dialog.clearDescendants();
			if(this.dataset){
				this.dataset.destroy();
			}
			delete this.jobsCallbackHanlders;
			delete this.originCallback;
			this.jobsCallbackHanlders = [
			     new Deferred(),  new Deferred(),  new Deferred(),  new Deferred()
			];
			this.originCallback=new Deferred();
			this.makeDataSet();
		},
		
		_unblockEvents : function () {
			this._logger.finest("job status pod unblock the event refreshing", "_unblockEvents");
			this.blockEvents = false; //Refresh is done, we can now listen for events again
			if (this.pendingEvents > 0) { //did any more events come in during our refresh? if so, refresh again
				this._logger.finest("there are pending events, begine to refresh", "_unblockEvents");
				this.refreshCallback();
			}
		},

		makeDataSet : function () {
			var i;
			var dataset = new Dataset({
				baseUrl : "/ibm/tivoli/rest",
				provider : "USMI",
				datasource : "",
				dataset : this.jobDatasetName,
				clearCacheOnRefresh : true,
				onComplete : lang.hitch(this, function (dataset, kwArgs) {
					this._logger.finest("dataset loading completed ", "makeDataSet");
					this.dataset=dataset;
					this.fetchOriginals();
					var parentIds=this.parentIds;
					var callbacks = this.jobsCallbacks;
					var handlers = this.jobsCallbackHanlders;
					for (i = 0; i < parentIds.length; i++) {
						this.fetchItems(parentIds[i], callbacks[i], handlers[i]);
					}

				}),
				onError : function (error, exception, ioArgs) {
					var msgHandler = new DefaultMessageHandler({});
					msgHandler.displayException(exception);
				},
				parameters : {
					maxSPElementsToken : 10
				},
				defaultProperties : []
			});
			this.dataset=dataset;
			this.dialog.dataset=dataset;
			dataset.addEventListener(this, lang.hitch(this, this.refreshCallback));

			// only remove on Destroy so we don't do this at logout. 
			// deleting datasets at logout casues redirect popups
			//baseUnload.addOnUnload(dataset, "destroy");

			if(!this.jobsCallbackHanlders || !this.originCallback){
				return;
			}
			var deferreds = lang.clone(this.jobsCallbackHanlders);
			deferreds.push(this.originCallback);
			var handlers = new DeferredList(deferreds);
			handlers.then(lang.hitch(this, function (result) {
				var delay='5000';
				if(this.pendingEvents >= 3){
					//if too many pending events, then use large delay
					delay='15000';
				}
				this._logger.finest("completed all deferreds", "makeDataSet");
				this.updateLabel();
				window.setTimeout(dojo.hitch(this, this._unblockEvents), delay);
			}));
		},

		destroy : function () {
			 if (this.dataset) {
				 this.dataset.destroy();
				 this.dataset = null;
			 }
			 this.inherited(arguments);
		},
		fetchItems : function (parentId, callback, handler) {
			this.dataset.getRelatedItems({
				onComplete : lang.hitch(this, function (items, kwArgs) {
					callback(items);
					if(handler){
						handler.resolve();
					}
				}),
				onError : function (error, exception, ioArgs) {
					var msgHandler = new DefaultMessageHandler({});
					msgHandler.displayException(exception);
					if(handler){
						handler.resolve();
					}
				},
				itemId: parentId,
				start : 0,
				count : 10,
				scope : this,
				sort : this.sortProp,
				properties: ["all"],
				relationships : "child"
			});
		},
		
		fetchOriginals : function () {
			this.dataset.getOriginItems({
				onComplete : lang.hitch(this, function (items, kwArgs) {
					items = items || [];
					var itemIndex = 0;
					var itemsLength = items.length;
					if(itemsLength>0) {
						var item = items[0];
						this.totalCount = item.getProperty("CATEGORY_COUNT").getValue();
						
					}
					this.originCallback.resolve();
				}),
				onError : function (error, exception, ioArgs) {
					var msgHandler = new DefaultMessageHandler({});
					msgHandler.displayException(exception);
					if(this.originCallback){
						this.originCallback.resolve();
					}
				},
				start : 0,
				count : 1,
				properties: ["all"],
				scope : this
			});
		},

		updateLabel : function (count) {
			if (count && count > 0) {
				this.iconClass="status-msgs-icon-error";
			}
			if(count===0){
				this.iconClass="status-msgs-icon-confirmation";
			}
			this.label=string.substitute("<span class='${iconClass}'></span>", {
				iconClass: this.iconClass
			});
			this.label=this.label+lang.replace(MRI.JobMenuBarItemLabel, [this.totalCount]);
			this.containerNode.innerHTML = this.label;
		}
		
	});
});
