// wrapped by build app
define("ibm/usmi/util/DefaultURLLaunchTask", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/Trace,dojox/data/dom,ibm/usmi/util/DefaultMessageHandler,ibm/usmi/core/utils/Logger"], function(dijit,dojo,dojox){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

dojo.provide("ibm.usmi.util.DefaultURLLaunchTask");

dojo.require("ibm.tivoli.navmodel.Trace");
dojo.require("dojox.data.dom");
dojo.require("ibm.usmi.util.DefaultMessageHandler");
dojo.require("ibm.usmi.core.utils.Logger");

ibm.usmi.util.DefaultURLLaunchTask._trace = new ibm.tivoli.navmodel.Trace("ibm.usmi.util.DefaultURLLaunchTask");
ibm.usmi.util.DefaultURLLaunchTask._defaultURLLaunchTaskLogger = new ibm.usmi.core.utils.Logger("ibm.usmi.util", "DefaultURLLaunchTask");

dojo.declare("ibm.usmi.util.DefaultURLLaunchTask", null, {

    // private members
    _launchParms : null,
    _menuItem : null,
    _context : null,
    _trace : ibm.usmi.util.DefaultURLLaunchTask._trace,
    
    // the logger
    _defaultURLLaunchTaskLogger : ibm.usmi.util.DefaultURLLaunchTask._defaultURLLaunchTaskLogger,

    constructor : function() {
        // summary:
        //        Coming soon
        // description:
        //        Coming soon

        // Register for a URL launch task event
        dojo.subscribe("DefaultURLLaunch", this, "_handleURLLaunch");

    },

    _handleURLLaunch : function(launchParms, menuItem, context) {
        // summary:
        //        Coming soon
        // description:
        //        Coming soon

        this._launchParms = launchParms;
        this._menuItem = menuItem;
        this._context = context;

        var runServerTaskAndLaunchURL = function(tasks) {
            var task = null;

            var launchUrl = function() {
                this.launchURL(this._launchParms);
            };

            // Get the selected task from all available tasks
            task = this.fetchTask(tasks);

            // Call execute on the server task
            task.execute({
                onComplete : dojo.hitch(this, launchUrl),
                onError : dojo.hitch(this, this._handleExecuteOnServerError),
                selection : this.getSelection(this._context)
            });
        };

        // Special case for group export - need to call execute on task to populate file to be exported first
        if (menuItem && menuItem.actionID == "GroupExport") {

            // set the task
            context.dataset.getTasks({
                onComplete : dojo.hitch(this, runServerTaskAndLaunchURL),
                onError : dojo.hitch(this, this.getTasksError),
                taskCategory : "menu", //TODO: Could I specifically request nav hints instead of all tasks?
                selection : this.getSelection(context)
            });
        } else {
            this.launchURL(launchParms);
        }
    },

    fetchTask : function(tasks) {

        //Want to loop through tasks looking for match to menuItem actionID
        if (tasks) {
            for ( var i = 0; i < tasks.length; i++) {
                if (tasks[i].getId() === this._menuItem.actionID) {
                    return tasks[i];
                } else if (tasks[i].getChildTasks()) {
                    var subTasks = tasks[i].getChildTasks();
                    if (subTasks.length > 0) {
                        var task = this.fetchTask(subTasks);
                        if (task && (task.getId() === this._menuItem.actionID)) {
                            return task;
                        }
                    }
                }
            }
        }
    },

    launchURL : function(launchParms) {

        // Check launchParms to decide how to handle
        if (launchParms) {
            // ISCPage launch
            if (launchParms.launchType === "ISCPage") {      
            	try {
	                if (this._context.selection && !(this._context.selection.isEmpty())) {
	                    var targets = this._context.selection.getItemIds();
	                	launchInContext(launchParms.pageID, targets, false);
	                } else {
	                	launchInContext(launchParms.pageID);
	                } 
            	} catch (ex) {
                    //There was a problem launching the window
                    var msgHandler = new ibm.usmi.util.DefaultMessageHandler({});
                    msgHandler.displayException(ex);
                }
            }

            // URL launch
            else if (launchParms.launchType === "URL") {
                if (launchParms.urlToLaunch) {
                    if (launchParms.windowId) {
                        try {
                            var openWindow = window.open(launchParms.urlToLaunch, launchParms.windowId);
                            openWindow.focus();
                        } catch (ex) {
                            //There was a problem launching the window
                            var msgHandler = new ibm.usmi.util.DefaultMessageHandler({});
                            msgHandler.displayException(ex);
                        }
                    } else {
                        try {
                            var openWindow = window.open(launchParms.urlToLaunch);
                            openWindow.focus();
                        } catch (ex) {
                            //There was a problem launching the window
                            var msgHandler = new ibm.usmi.util.DefaultMessageHandler({});
                            msgHandler.displayException(ex);
                        }
                    }
                } else {
					if (this._defaultURLLaunchTaskLogger.isWarn()) {
						this._defaultURLLaunchTaskLogger.warn("launchParms are not valid - urlToLaunch not defined", "launchURL");
					}
                }
            }

            // Widget launch, used by DefaultDojoUrlTask, were we are loading new widgets
            else if (launchParms.launchType === "Widget") {

                if(launchParms.urlParameters) {
                    // Launch the widget within the framework
                    try {
                        launchNextGenPage(launchParms.urlParameters);
                    } catch (ex) {
                        //There was a problem launching the window
                        var msgHandler = new ibm.usmi.util.DefaultMessageHandler({});
                        msgHandler.displayException(ex);
                    }
                }
                else {
					if (this._defaultURLLaunchTaskLogger.isWarn()) {
						this._defaultURLLaunchTaskLogger.warn("launchParms are not valid - urlParameters not defined", "launchURL");
					}
                }
            } else {
				if (this._defaultURLLaunchTaskLogger.isWarn()) {
					this._defaultURLLaunchTaskLogger.warn("launchParms are not valid - launchType not define or not valid", "launchURL");
				}
            }

        } else {
			if (this._defaultURLLaunchTaskLogger.isWarn()) {
				this._defaultURLLaunchTaskLogger.warn("launchParms are not defined", "launchURL");
			}
        }
    },

    getSelection : function(context) {
        var selectedThings = context.selection; //this._args.context.selection;
        var currentSelection = null;
        if (selectedThings) {
            currentSelection = new ibm.tivoli.navmodel.Selection({
                version : context.dataset.getVersion()
            }); //this.getDataSet().getVersion()});
            currentSelection.addItems(selectedThings.getItemIds());
            currentSelection.addLinks(selectedThings.getLinkIds());
            currentSelection.addRelationships(selectedThings.getRelationshipIds());
        }
        return currentSelection;
    },

    getTasksError : function(error, request) {
        //There was a problem getting the tasks from the dataset
        var msgHandler = new ibm.usmi.util.DefaultMessageHandler({});
        msgHandler.displayException(error);
    },

    _handleExecuteOnServerError : function(error, exception, ioArgs) {
        // summary:
        //        Coming soon
        // description:
        //        Coming soon
        // error:
        //
        // exception:
        //
        // ioArgs:
        //

        if (this._trace.isError()) {
            this._trace.error(">>> error executing server task = " + error, "_handleExecuteOnServerError");
        }

        //There was a problem executing the task on the server
        var msgHandler = new ibm.usmi.util.DefaultMessageHandler({});
        msgHandler.displayException(exception);
    },
});

});
