/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/widgets/FrameworkMessage", [
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dojo/_base/lang",
    "dojo/_base/sniff", // has("ie")
	"dojo/topic",
	"dojo/dom-class",
	"dojo/dom-attr",
	"dojo/dom-construct",
	"dijit/_WidgetBase",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	// IDX imports
	"idx/oneui/messaging/SingleMessage",
	// NextGen imports 
	"ibm/usmi/core/utils/Logger"
], function (dojo, declare, _lang, has, topic, domClass, domAttr, domconstruct, _WidgetBase, _TemplatedMixin,
		_WidgetsInTemplateMixin, SingleMessage, Logger) {

	return declare("ibm.usmi.widgets.FrameworkMessage", [
		_WidgetBase,
		_TemplatedMixin,
		_WidgetsInTemplateMixin,
		SingleMessage
	], {

		dateFormat : {
			datePattern : 'dd MMMM y, ',
			timePattern : 'hh:mm a'
		},

		style : "width: 98%;",

		showViewDetails : true,

		// external launch url (not Launch-in-Context)
		moreDetailsURL : null,

		// customize the "More Details" label (e.g. "View Job Properties")
		alternateMoreDetailsLabel : null,

		// Create a launch-in-context url w/ page id
		internalLaunch_pageId : null,

		// Create a launch-in-context url w/ targets
		internalLaunch_targets : null,

		// Launch a full URL in context (rather than creating a URL with pageId and targets)
		internalLaunch_url : null,

		/**
		 * More details isn't enough? Add another launch link then override
		 * onCustomLaunch() to do whatever you want Set this string for an
		 * additional launch point. Note: you really should use more details
		 * first.
		 */
		customLaunchLabel : null,

		postCreate : function () {
			this.inherited(arguments);
			if (this.alternateMoreDetailsLabel != null) {
				domAttr.set(this.viewDetailsNode, "innerHTML", this.alternateMoreDetailsLabel);
			}
			this._setShowActionAttr(false);
			this._setShowRefreshAttr(false);
		},

		buildRendering : function () {
			this.inherited(arguments);

			if (this.customLaunchLabel != null && this.bodyNode != null) {
				//this._injectTemplate(this.bodyNode, '<button id="testButton" dojoType="dijit.form.Button" dojoAttachPoint="testButton" dojoAttachEvent="onClick:onCustomLaunch()" name="test">Test</button>');
				this._injectTemplate(this.bodyNode,
				//'<span><div id="sep" class="idxMessageSeparator" dojoAttachPoint="sep">|</div></span>' + 
				'<a class="idxMessageViewDetails" href="javascript: void(0);"'
						+ ' target="_blank" dojoAttachPoint="testNode"'
						+ ' dojoAttachEvent="ondijitclick:onCustomLaunch" name="test">Test</a>');

				domAttr.set(this.testNode, "innerHTML", this.customLaunchLabel);

			}
		},

		/**
		 * Construct UI from a template, injecting the resulting DOM items as
		 * children on of the supplied container node.
		 * 
		 * @param {Object}
		 *            containerNode
		 * @param {Object}
		 *            templateString
		 */
		_injectTemplate : function (containerNode, templateString) {

			// this code is generalised from _Templated.buildRendering

			// Look up cached version of template, or download to cache.
			var cached = _TemplatedMixin.getCachedTemplate(templateString, true);

			var node;
			if (_lang.isString(cached)) {
				// if the cache returned a string, it contains replacement parameters,
				// so replace them and create DOM
				node = domconstruct.toDom(this._stringRepl(cached));
			} else {
				// if the cache returned a node, all we have to do is clone it
				node = cached.cloneNode(true);
			}

			// recurse through the node, looking for, and attaching to, our
			// attachment points and events, which should be defined on the template node.
			this._attachTemplateNodes(node, function (n, p) {
				return n.getAttribute(p);
			});

			// append resolved template as child of container
			containerNode.appendChild(node);
		},

		onClick : function (e) {
			this.inherited(arguments);
			this._refreshMainArea();
		},
		onClose : function () {
			this.inherited(arguments);
			this._refreshMainArea();
		},
		_refreshMainArea : function () {
			var container = dijit.byId("mainContentAreaLayout");
			if (container) {
				container.resize();
			}
		},

		onMoreDetails : function (e) {
			this.inherited(arguments);
			//if (this.operatorResponse != null) {
			//    this.showMoreDetailsDialog(this.operatorResponse);
			if (this.moreDetailsURL != null) {
				this._launchMoreDetails(this.moreDetailsURL);
			} else if (this.internalLaunch_pageId != null || this.internalLaunch_url != null) {
				launchInContext(this.internalLaunch_pageId, this.internalLaunch_targets, this.internalLaunch_url);
			}
	
			// eat the event, otherwise IE tries to reload the page, which fails
			// it onunloads the page but never onloads it
			if(has("ie")){
				e.preventDefault(); 
			}
		},

		destroy : function () {
			this.inherited(arguments);
			this._refreshMainArea();
		},

		/**
		 * Called by the customLaunch link. Override to provide funcion.
		 */
		onCustomLaunch : function (e) {
			topic.publish("message_CustomLaunch", {
				msgId : this.messageId,
				customLaunchLabel : this.customLaunchLabel
			});
		},

		/**
		 * Launch an external URL into a new browser window. Note: Use
		 * launchInContext() to launch internal URL's
		 */
		_launchMoreDetails : function (url) {
			window.open(url, "_blank", null);
		},

		/**
		 * Eventually this will be replaced by something similar in
		 * idx.oneui.messaging.SingleMessage. We are waiting on a fix pack from
		 * them, but I wanted to enable this funciton now.
		 */
		_setShowViewDetailsAttr : function (/*Boolean*/value) {
			this._set("showViewDetails", value);
			domClass.toggle(this.viewDetailsNode, "dijitHidden", !this.showViewDetails);
		}
	});

	/*
	 * Message type. Can be 'error', 'warning', 'success', 'information', 'critical', 'attention', 'compliance'
	 */
	ibm.usmi.widgets.FrameworkMessage.ERROR = "error";
	ibm.usmi.widgets.FrameworkMessage.WARNING = "warning";
	ibm.usmi.widgets.FrameworkMessage.INFO = "information";
	ibm.usmi.widgets.FrameworkMessage.SUCCESS = "success";
	ibm.usmi.widgets.FrameworkMessage.CRITICAL = "critical";
	ibm.usmi.widgets.FrameworkMessage.ATTENTION = "attention";
	ibm.usmi.widgets.FrameworkMessage.COMPLIANCE = "compliance";

});
