/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

define("ibm/usmi/widgets/PropertyNotebook", ["dojo/_base/kernel",
        "dojo/_base/declare",
        "dijit/_WidgetBase",
        "dijit/layout/ContentPane",
        "dijit/layout/BorderContainer",
        // TWL imports
        "ibm/tivoli/navmodel/Dataset",
        "ibm/usmi/util/DefaultMessageHandler",
        "ibm/tivoli/tip/navwidget/properties/dijit/PropertiesNotebook",
        "ibm/tivoli/tip/navwidget/properties/dijit/PropertiesNotebookGeneralTab",
        "ibm/tivoli/tip/navwidget/dojo/NavWidgetUtil",
        "ibm/tivoli/tip/navwidget/dijit/NavWidgetObjectPluggableView",
        "ibm/tivoli/tip/navwidget/properties/PropertyRenderer",
        "ibm/tivoli/tip/navwidget/dijit/NavWidgetPluggableView",
        "ibm/tivoli/tip/navwidget/dijit/NavWidgetTabContainerObjectPluggableView",
        // NextGen imports
        "ibm/usmi/util/BaseWidget",
        "ibm/usmi/widgets/util/PropertyNotebookDAHelper",
        "ibm/usmi/widgets/util/DefaultContextMenuProvider",
        "ibm/usmi/widgets/util/DefaultMenuProviderHelper",
        "ibm/usmi/widgets/util/Constants",
        "ibm/usmi/core/utils/Logger",
        "dojo/i18n!./nls/CommonWidgetsResources"],
    function (dojo, declare, _WidgetBase, ContentPane, BorderContainer,
            Dataset, DefaultMessageHandler, PropertiesNotebook, PropertiesNotebookGeneralTab, NavWidgetUtil, NavWidgetObjectPluggableView, PropertyRenderer, NavWidgetPluggableView, NavWidgetTabContainerObjectPluggableView,
            BaseWidget, PropertyNotebookDAHelper, DefaultContextMenuProvider, DefaultMenuProviderHelper, Constants, Logger, mri) {


    return declare([_WidgetBase, BaseWidget, ContentPane], {

        // context menu provider that is passed off to pluggableview
        cmProvider: null,

        // dataset object
        dataset: null,

        // boolean: property notebook stand alone
        standAlone: true,

        // attr that can be set html tag
        propsWidgetDefinitionId : "",

        defaultActionHelper : null,

        //resource bundle
        _resources : mri,

        ///////////////////////////////////////////////////////////////////
        // private members
        ///////////////////////////////////////////////////////////////////
        // handle to TWL properties notebook
        _properties: null,

        // Flag for the first time being called or we have a new widget def, then go ahead with creation
        _createTabbedViewsPreviouslyCalled: false,

        // the logger
        _propertyNotebookLogger: new Logger("ibm.usmi.widgets", "PropertyNotebook"),

        DEFAULT_COLLECTION: "RESOURCE_DETAILS_COLLECTION", //ask aaron about override

        constructor: function(/*constructor_kwArgs*/kwArgs) {
            // summary:
            //        Constructs PropertyNotebook object.

            if (this._propertyNotebookLogger.isFinest()) {
                this._propertyNotebookLogger.entering("", "constructor");
            }

            this.title = this._resources.PropertyNotebookTitle;

            if(this.getUrlParm("standAlone")) {
                this.standAlone = this.getUrlParm("standAlone");
            }

            if (this._propertyNotebookLogger.isFinest()) {
                this._propertyNotebookLogger.exiting("", "constructor");
            }
        },

        startup: function() {
            // summary:
            //         Called after any DOM fragments have been actually added to the document
            // tags:
            //        protected
            this.inherited(arguments);
        },

        postCreate: function(){
            // summary:
            //         Called after a widget's dom has been setup
            // tags:
            //        protected

            //Call superclass
            this.inherited(arguments);

            // construct menu provider to handle tasks that are set on properties
            this._createCMProvider();

            // create a default context menu provider
            this.defaultActionHelper = new PropertyNotebookDAHelper();

            // widget def on URl Overrides all
            var defOnUrl = this.getUrlParmsObj().propsWidgetDefinitionId;

            if(defOnUrl) {
                this.propsWidgetDefinitionId = defOnUrl;
            } // check to see if it was specified as attribute
            else if (!this.propsWidgetDefinitionId) {
                this.propsWidgetDefinitionId = "UsmiPropertiesNotebook";
            }

            this._properties = new PropertiesNotebook({
                id: "_propertiesNotebook" + this.id,
                style: Constants.CSS_MAX_SIZE,
                widgetDefinitionId: this.propsWidgetDefinitionId,
                cmProvider: this.cmProvider,
                daHelper : this.defaultActionHelper
            });

            // try calling adding our seperate tabs here
            this._setDatasetCreatePluggableViews();

            // had to do this for now, couldnt get the intial view to display without phisically changing the
            // size of the broswer window, when i was a bordercontainer and using addChild();
            dojo.place(this._properties.domNode, this.containerNode);
            dojo.style(this._properties.domNode, {height: "100%", width: "100%", overflow:"visible"});


            if (this.standAlone) {
                this.loadDataset();
            }
        },

        loadDataset: function() {
               // summary:
            //        Called if property Notebook is stand alone.  Use the dataset and source token that was passed
            //        in on URL or constructor.
            // create dataset
            if (this._propertyNotebookLogger.isFinest()) {
                this._propertyNotebookLogger.entering("", "loadDataset");
            }
            if(!this.providerID) {
               // load default
               this.providerID = Constants.DEFAULT_PROVIDER;
               }

           if (!this.collectionID ) {
                   // load default
                this.collectionID = this.DEFAULT_COLLECTION;
            }

            this.dataset = new Dataset({
                       onComplete: dojo.hitch(this, this.loadData),
                       onError: function(error, exception, ioArgs) {
                           // display error to user on exception
                        var msgHandler = new DefaultMessageHandler({});
                        msgHandler.displayException(exception);
                       },
                       baseUrl: Constants.BASE_REST_URL,
                       provider: this.providerID,
                    datasource: "",
                    dataset: this.collectionID,
                    parameters: {sourceToken: this.sourceToken},
                    defaultProperties: [Constants.PROPS_ALL]
              });

            if (this._propertyNotebookLogger.isFinest()) {
                this._propertyNotebookLogger.exiting("", "loadDataset");
            }
        },

        loadData: function(/*ibm.tivoli.navmodel.Dataset*/dataset) {
               // summary:
            //        Called when dataset completes loading, this method is called, which then gets the Item on the dataset,
            // tags:
            //        callback
            dataset.getItems({
                onComplete: dojo.hitch(this, this.setItemOnProperties),
               // onComplete: function(items, kwArgs) { this.makeProperties(dataset, items[0]); },
                onError: function(error, exception, ioArgs) { alert(error); },
                start: 0,
                count: null
            });
        },

        // set the item on the TWL properties notebook
        setItemOnProperties: function(/*ibm.tivoli.navmodel.Item[]*/items) {
               // summary:
            //        Called when properties are returned from Item
            // tags:
            //        callback
            if (this._propertyNotebookLogger.isFinest()) {
                this._propertyNotebookLogger.entering("", "setItemOnProperties");
            }
            var item = items[0];

            this._properties.setDataset(this.dataset);
            this._properties.setObjects([item], null, null, null);
            this._properties.resize();

            if (this._propertyNotebookLogger.isFinest()) {
                this._propertyNotebookLogger.exiting("", "setItemOnProperties");
            }
        },

        setObjects: function(/*ibm.tivoli.navmodel.Item[]*/ item, /*ibm.tivoli.navmodel.Link[]*/ links, /*ibm.tivoli.navmodel.Link[]*/ relationships){
            this._properties.setDataset(this.dataset);
            this._properties.setObjects(item, null, null, null);

            this._properties.startup();
            this._properties.resize();
        },

        setDataset: function(/*ibm.tivoli.navmodel.Dataset*/dataset){
            this.dataset = dataset;
        },

        destroy: function() {
            // summary:
            //        Clean up any propertyNotebook data
            if (this._propertyNotebookLogger.isFinest()) {
                this._propertyNotebookLogger.entering("", "destroy");
            }

            if (this.dataset) {
                this.dataset.destroy();
                this.dataset = null;
            }

            if(this._properties) {
                this._properties.destroy();
                this._properties = null;
            }

            this.cmProvider = null;
            this.defaultActionHelper = null;

            this.inherited(arguments);

            if (this._propertyNotebookLogger.isFinest()) {
                this._propertyNotebookLogger.entering("", "destroy");
            }
        },

        _updateTargets: function(/*ibm.tivoli.navmodel.Dataset*/dataset, /*ibm.tivoli.navmodel.Item[]*/items) {
            // summary:
            //        Retarget event method.   Update the item that the property Notebook is viewing
            if (this._propertyNotebookLogger.isFinest()) {
                this._propertyNotebookLogger.entering("", "_updateTargets");
            }

            if(items && items.length > 1 ) {
                this._properties.setObjects([items[0]], null, null, null);
            }

            if (this._propertyNotebookLogger.isFinest()) {
                this._propertyNotebookLogger.exiting("", "_updateTargets");
            }

        },

        // This is called by NavWidgetPluggable View in its contructor
        _createCMProvider: function(){
            // summary:
            //        Coming soon
            // description:
            //        Coming soon

            //Create the menu provider that we'll pass off to the pluggable views
            if (!this.cmProvider) {
                var helper = new DefaultMenuProviderHelper();
                this.cmProvider = new DefaultContextMenuProvider({
                                    view: this,
                                    getMenuItemsCallback: dojo.hitch(helper, helper.getMenuItems),
                                    getMenuCommandCallback: dojo.hitch(helper, helper.getMenuCommand)
                });
            }


        },

        _setDatasetCreatePluggableViews: function() {
            // summary:
            //        Coming soon
            // description:
            //        Coming soon
            // tags:
            //        protected

            if (this._propertyNotebookLogger.isFinest()) {
                this._propertyNotebookLogger.entering("", "_setDatasetCreatePluggableViews");
            }

            //If this is the first time being called or we have a new widget def, then go ahead with creation
            if (!this._createTabbedViewsPreviouslyCalled) {

                //Process pluggable views
                // TODO we need to determine what the plug point is, is it getTasks, and if type is propery
                // or some xml file we read in?  this would be similar to widgetdefs, so just reuse that?
                // reuse some existing navmodel structure... need, id and full class name to js widget that implements
                // ibm.tivoli.tip.navwidget.dijit.NavWidgetObjectPluggableView

    //            var testObj = {viewId: "CUSTOM_PROGRAMMITIC_TAB",
    //                           className: "demo.PropertiesNotebookDemoTab"};

                var childPluggableViews = null, i, viewInfo, viewArgs, viewInstance, iterator, view, chainSelectionChanged;
    //            var childPluggableViews = [testObj];

                if (childPluggableViews) {
                    for (i = 0; i < childPluggableViews.length; i += 1) {

                        if (this._propertyNotebookLogger.isFiner()) {
                            this._propertyNotebookLogger.finer("Pluggable Views count: " + childPluggableViews.length);
                        }
                        viewInfo = childPluggableViews[i];

                        viewArgs = {
                                id: this.id + "_" + viewInfo.viewId,
                                viewId: viewInfo.viewId,
                                //dataset: this.dataset,
                                cmProvider: this.cmProvider,
                                daHelper : this.defaultActionHelper,
                                parentPluggableView: this._properties
                            };

                        viewInstance = NavWidgetUtil.getObjectInstance(viewInfo.className, viewArgs);

                        if (viewInstance) {
                            this._properties.addPluggableView(viewInstance);
                        }
                    }
                }

                //Set up to listener for selection changes on all the views
                 chainSelectionChanged = function (objectContext) {
                    if (!this.selection.isChanging()) {
                         this.selection.chainSelectionChanged(objectContext);
                    }
                };
                iterator = this._properties.getPluggableViewIterator();
                while (!iterator.atEnd()) {
                    if (this._propertyNotebookLogger.isFiner()) {
                        this._propertyNotebookLogger.finer("Pluggable Views selection change listener connect: ");
                    }

                    view = iterator.get().value;

                    dojo.connect(view.selection, Constants.SELECTION_CHANGE, this, chainSelectionChanged);
                }
            }

            //Remember that we've created pluggable views before
            this._createTabbedViewsPreviouslyCalled = true;

            if (this._propertyNotebookLogger.isFinest()) {
                this._propertyNotebookLogger.exiting("", "_setDatasetCreatePluggableViews");
            }
        }

    });
});
