/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
// Finder Base
define("ibm/usmi/widgets/finder/Finder", [
	"dojo/_base/lang",
	"dojo/dom",
	"dojo/_base/array",
	"dojo/on",
	"dojo/_base/connect",
	"dojo/dom-construct",
	"dojo/dom-class",
	"dijit",
	"dojo/_base/declare",
	"dijit/_WidgetBase",
	// Need this because we are declaring a widget in our template
	"dojo/cookie",
	"dijit/form/TextBox",
	"dijit/TooltipDialog",
	"dijit/popup",
	"dijit/layout/TabContainer",
	"dijit/layout/ContentPane",
	"dojo/i18n!ibm/usmi/nls/frameworkResources",
	"ibm/usmi/util/DataStore",
	"ibm/usmi/util/Utils",
	"ibm/usmi/scheduler/JobLaunchWizard",
	"./TaskDataStore",
	"./ResourceDataStore",
	"./GroupDataStore",
	"ibm/tivoli/navmodel/Dataset"
], function (djLang, djDom, djArray, on, connect, djDomC, djClass, dijit, declare, _WidgetBase, djCookie, djTextBox,
		djTooltipDialog, djPopup, djTabContainer, djContentPane, MRI, DataStore, Utils, JobLaunchWizard, TaskDataStore,
		ResourceDataStore, GroupDataStore, CuriDataset) {
	return declare([ _WidgetBase
	], {
		// templateString : template,
		finderHelpTooltip : MRI.FinderTooltipHelp,
		searchTextHintString : "hint",
		tooltipHelp : MRI.FinderTooltipHelp,

		_searchTextHintString : null,
		_tabbedContainer : null,

		// references to finder elements
		_finderDomObj : null, // finder main dom container
		_searchDetailDlg : null, // finder results tooltip
		// dialog
		tooltipShown : null,
		// finder state variables
		_searchDlgIsOpen : false,
		_searchDlgInitialized : false,
		_latestSearchEntry : null,
		_currentView : '0',
		_refreshImage : null,
		_currentResultCountArray : null,

		_dataStore : null,

		//Refresh variables
		blockEvents : true,
		pendingEvents : false,
		initialized : false,
		searchOpen : false,
		origStyle : "dijitReset dijitInline dijitButtonNode",
		refreshStyle : "dijitReset dijitInline finderRefresh",
		lastRefresh : 0,
		manualRefresh : false,

		// Data stores containing cached lists
		_taskDataStore : null,
		_resourceDataStore : null,
		_groupDataStore : null,
		_indexedItems : null,
		// _keyPressed : 0,

		constructor : function () {
			this.inherited(arguments);
			this._indexedItems = [];
			Utils.CMTraceTimeInit("finderTimer");
			this._searchTextHintString = MRI.FinderInitialText;
			this.searchTextHintString = this._searchTextHintString;
			this._currentSingleResultArray = {
				taskId : null,
				resourceId : null,
				groupId : null
			};
		},
		postCreate : function () {
			this.inherited(arguments);
			this.initializeFieldReferences();
			this.initializeFinderHtml();

			this._dataStore = new DataStore({
				datasetId : 'FinderCollection',
				initReadyCallback : djLang.hitch(this, this.initializeDataStores)
			});

		},
		/*
		 * 
			// setTimeout(djLang.hitch(this, this.foo), 50);
		 * foo : function() { this._dataStore = new DataStore({
		 * datasetId : 'FinderCollection', homeParms : 150, getItems :
		 * true, retrieveItemProperties : true, getTasks : true,
		 * initReadyCallback : djLang.hitch(this,
		 * this.initializeDataStores) }); },
		 */
		incrementRefreshMinute : function () {
			if (this.lastRefresh === 15) {
				this._refreshCache();
			} else {
				this.lastRefresh = this.lastRefresh + 1;
				if (this._refreshImage) {
					this.changeRefreshCount();
				}
			}
			setTimeout(djLang.hitch(this, 'incrementRefreshMinute'), 60000);
		},
		changeRefreshCount : function () {
			this._refreshImage.title = djLang.replace(MRI.RefreshCache, [
				this.lastRefresh,
				MRI.Minutes
			]);

		},
		initializeDataStores : function () {
			/*this._taskDataStore = new TaskDataStore('tasks', this.retrieveDatasetCache([
				'tasks',
				'jobs',
				'ngenTask'
			]));*/
			this.initializeResourcesAndGroups(true);
			setTimeout(djLang.hitch(this, 'incrementRefreshMinute'), 60000);
		},
		_doRefreshProcessing : function () {
			if (this._searchDlgIsOpen) {
				this.searchOpen = true;
			}
			djDom.byId("widget_findersearchBox").nextSibling.childNodes[0].className = this.refreshStyle;
			if (this._latestSearchEntry) {
				djCookie("finder.lastSearch", this._latestSearchEntry, {
					expires : 1
				});
			}
			var inputField = dijit.byId("findersearchBox").domNode.childNodes[0].childNodes[0];
			this.closeSearchDetailDlg();
			inputField.value = MRI.FinderRefreshing;
			inputField.disabled = true;
		},
		_manualRefresh : function () {
			if (this._searchDlgIsOpen) {
				this.searchOpen = true;
			}

			this.manualRefresh = true;
			this._doRefreshProcessing();
			this._refreshCache();

		},
		_refreshCache : function () {
			if (this.blockEvents) { //Refresh is in progress, don't refresh
				this.pendingEvent = true; //save the need to refresh again later
				return;
			}

			this.lastRefresh = 0;
			this.blockEvents = true;
			this.pendingEvent = false;
			if (!this.manualRefresh) {
				this._doRefreshProcessing();
			} else {
				this.manualRefresh = false;
			}
			this._dataStore.destroy();
			if (this._resourceDataStore) {
				this._resourceDataStore.destroy();
				this._resourceDataStore = null;
			}
			if (this._groupDataStore) {
				this._groupDataStore.destroy();
				this._groupDataStore = null;
			}
			this._dataStore = new DataStore({
				datasetId : 'FinderCollection',
				initReadyCallback : djLang.hitch(this, this.initializeResourcesAndGroups)
			});
		},
		initializeResourcesAndGroups : function (initial) {
			this._dataStore.getDataset().addEventListener(this, djLang.hitch(this, this._refreshCache));
			this._taskDataStore = new TaskDataStore('tasks', this.retrieveDatasetCache([
				'tasks',
				'jobs',
				'ngenTask'
			]));
			this._resourceDataStore = new ResourceDataStore('resources', this.retrieveDatasetCache('resources'));
			this._groupDataStore = new GroupDataStore('groups', this.retrieveDatasetCache('groups'));
			window.setTimeout(djLang.hitch(this, this._unblockEvents), '10000');
			if (initial === true) {} else {
				this.initializeFinderHtml();
				djDom.byId("widget_findersearchBox").nextSibling.childNodes[0].className = this.origStyle;
				dijit.byId("findersearchBox").domNode.childNodes[0].childNodes[0].disabled = false;
				if (this.searchOpen) {
					this.searchOpen = false;
					this.finderKeyPressed({});
				}
			}

		},
		_unblockEvents : function () {
			this.blockEvents = false; //Refresh is done, we can now listen for events again
			if (this.pendingEvent) { //did any more events come in during our refresh? if so, refresh again
				this._refreshCache();
			}
		},
		retrieveDatasetCache : function (
		/* String or array of strings*/types) {
			var items = [], idx;
			djArray.forEach(this._dataStore.getItems(), djLang.hitch(this, function (entry, i) {
				var matches = false;
				var type = entry.getProperty('FINDER_TYPE_COLUMN').getValue();
				if (types instanceof Array) {
					for (idx = 0; idx < types.length; idx++) {
						if (types[idx] === type) {
							matches = true;
						}
					}
				} else {
					matches = type === types;
				}
				if (matches) {
					var iconString = null;
					if (entry.getProperty('FINDER_ICON_COLUMN')) {
						iconString = entry.getProperty('FINDER_ICON_COLUMN').getValue();
					}
					var itId = entry.getProperty('FINDER_ID_COLUMN').getValue();
					items.push({
						id : itId,
						label : entry.getProperty('FINDER_NAME_COLUMN').getValue(),
						category : entry.getProperty('FINDER_CATEGORY_COLUMN').getValue(),
						icon : iconString
					});
					this._indexedItems[itId] = entry;
				}
			}));
			return items;
		},

		initializeFieldReferences : function (domId) {
			this._finderDomObj = djDom.byId('finder');
		},
		initializeFinderHtml : function () {
			var inputFieldWidget = dijit.byId("findersearchBox");
			var inputField = inputFieldWidget.domNode.childNodes[0].childNodes[0];

			var lastEnteredSearch = djCookie("finder.lastSearch");
			if (lastEnteredSearch) {
				inputField.value = lastEnteredSearch;
				djCookie("finder.lastSearch", null, {
					expires : -1
				});
			} else {
				inputField.value = this._searchTextHintString;
			}
			if (!this.initialized) {
				on(inputField, "blur", djLang.hitch(this, this.doSearchBoxBlur));
				on(inputField, "focus", djLang.hitch(this, this.doSearchBoxFocus));
				on(inputField, "keyup", djLang.hitch(this, this.finderKeyPressed));
				this.initialized = true;
			}

			if (dijit.byId("finderdialog")) {
				dijit.byId("finderdialog").destroy();
			}
			this._searchDetailDlg = new djTooltipDialog({
				id : "finderdialog",
				style : "width:450px;height:150px;",
				resize : function () {}/* HACK, figure out why resize kills the dialog*/
			});
			this._searchDetailDlg.startup();

		},
		fetchFailed : function (error, request) {
			console.error("finder: lookup failed.");
			Utils.CMFinest('finder: ' + error);
		},
		doSearchBoxBlur : function () {
			if (dijit.byId("findersearchBox").get('value') === "") {
				dijit.byId("findersearchBox").set('value', this._searchTextHintString);
			}
		},
		doSearchBoxFocus : function () {
			if (dijit.byId("findersearchBox").get('value') === this._searchTextHintString) {
				dijit.byId("findersearchBox").set('value', "");
			}
			dijit.byId("findersearchBox").focus();
		},
		closeSearchDetailDlg : function (keepFocusOnInput) {
			if (this._searchDetailDlg) {
				djPopup.close(this._searchDetailDlg);
				this._searchDetailDlg.destroyRecursive();
				this._searchDetailDlg = new djTooltipDialog({
					id : "finderdialog",
					style : "width:450px;height:150px;",
					resize : function () {}/* HACK, figure out why resize kills the dialog*/
				});
				this._searchDetailDlg.startup();
			}
			// dijit.byId("findersearchBox").attr("value",
			// this._searchTextHintString)
			this._searchDlgIsOpen = false;
			this._latestSearchEntry = '';
			var inputFieldWidget = dijit.byId("findersearchBox");
			var inputField = inputFieldWidget.domNode.childNodes[0].childNodes[0];
			if (keepFocusOnInput !== true) {
				inputField.value = this._searchTextHintString;
				inputField.blur();
			}
			this._taskDataStore.clearTooltips();
			this._resourceDataStore.clearTooltips();
			this._groupDataStore.clearTooltips();

			// var obj = dijit.byId("findersearchBox").domNode;
			// obj.blur();
		},
		doTaskSearch : function (myitems, myrequest) {
			if (myitems.length === 1) {
				this._currentSingleResultArray.taskId = myitems[0].id;
			} else {
				this._currentSingleResultArray.taskId = null;
			}
			this.doSearch(myitems, myrequest, this._taskDataStore,
			/*'special#_com.ibm.usmi.console.taskmap'*/'special#_none', MRI.MoreTasks, 'tasks', true);
		},
		doResourceSearch : function (myitems, myrequest) {
			if (myitems.length === 1) {
				this._currentSingleResultArray.resourceId = myitems[0].id;
			} else {
				this._currentSingleResultArray.resourceId = null;
			}
			this.doSearch(myitems, myrequest, this._resourceDataStore,
					'special#_com.ibm.usmi.console.tasks.QuickFindTask', MRI.MoreResources, 'resources', false);
		},
		doGroupSearch : function (myitems, myrequest) {
			if (myitems.length === 1) {
				this._currentSingleResultArray.groupId = myitems[0].id;
			} else {
				this._currentSingleResultArray.groupId = null;
			}
			this.doSearch(myitems, myrequest, this._groupDataStore, 'special#_none', MRI.MoreGroups, 'groups', true);
		},
		doSearch : function (myitems, myrequest, dataStore, moreItemsTask, moreItemsText, typeOfSearch,
				moreItemsNotClickable) {
			Utils.CMTraceTimeDelta("finderTimer", "doSearch" + typeOfSearch, true);
			var searchString = dijit.byId("findersearchBox").get('value');
			if (searchString === "" || searchString === this._searchTextHintString) {
				return;
			}
			if (myitems) {
				if (myitems.length > 0) {
					dataStore.clearSelection();
					Utils.CMTraceTimeDelta("finderTimer", "processing results", true);
					dataStore.populateCategoriesWithSelectedItems(myitems);
					Utils.CMTraceTimeDelta("finderTimer", "results populated", true);
					this.initializeDialogConstruct(typeOfSearch, myitems.length);
					Utils.CMTraceTimeDelta("finderTimer", "initialized dialog", true);
					dataStore.createCategoryHTMLElements(myitems.length, this, moreItemsTask, moreItemsText,
							moreItemsNotClickable);
					Utils.CMTraceTimeDelta("finderTimer", "results UI created", true);

				} else {
					this.initializeDialogConstruct(typeOfSearch);
					var noMatchesFoundText;
					if (typeOfSearch === 'resources') {
						noMatchesFoundText = MRI.NoResourcesFound;
					} else if (typeOfSearch === 'tasks') {
						noMatchesFoundText = MRI.NoTasksFound;
					} else if (typeOfSearch === 'groups') {
						noMatchesFoundText = MRI.NoGroupsFound;
					}
					djDomC.create("div", {
						innerHTML : noMatchesFoundText
					}, djDom.byId('finder' + typeOfSearch + "_containerDiv"));
				}
				// djDom.byId('finder' + typeOfSearch +
				// "Menu_count").innerHTML = " ("
				// +myitems.length + ")";
			}
			if (typeOfSearch === 'resources') {
				this._tabbedContainer.startup();
				var selectedTab = this._tabbedContainer.selectedChildWidget;
				djClass.remove(selectedTab.domNode, "dijitHidden");
				djClass.add(selectedTab.domNode, "dijitVisible");
			}
		},
		doClick : function (event, id) {
			var selectedTaskId, clickType = 1;
			if (!event) {
				selectedTaskId = id;
			} else {
				clickType = event.which;
				selectedTaskId = event.currentTarget.id;
			}
			this.closeSearchDetailDlg();
			Utils.CMFinest('finder: doClick' + selectedTaskId);
			var type = selectedTaskId.split('#_')[0];
			var itemId = selectedTaskId.split('#_')[1];
			if (type.indexOf('categorySection') !== -1) {
				var category = this._resourceDataStore.getCategories()[itemId];
				var ids = category.getIds();
				selectedTaskId = 'resourceList#_';
				var i;
				for (i = 0; i < ids.length; i++) {
					if (i !== 0) {
						selectedTaskId = selectedTaskId + ",";
					}
					selectedTaskId = selectedTaskId + ids[i];
				}
			}

			// get item from dataset, since our cache doesn't
			// have all the info

			if (itemId.indexOf("com.ibm.usmi.console.taskmap") !== -1) {
				launchNextGenPage("workAreaDijit=ibm/usmi/widgets/ResourceNavigator&dataset=GROUP:USMI_TASK_MAP");
				return;
			} else if (itemId.indexOf("com.ibm.usmi.console.tasks.QuickFindTask") !== -1) {
				launchNextGenPage("workAreaDijit=ibm/usmi/widgets/ResourceNavigator&dataset=GROUP:ALL_ENDPOINTS");
				return;
			}
			var item = this._dataStore.getItemFromProperty("FINDER_ID_COLUMN", itemId);
			if (type === 'tasks') {
				if (item) {
					var taskType = item.getProperty('FINDER_TYPE_COLUMN').getValue();
					if (taskType === 'tasks') {
						var pageid = item.getProperty('FINDER_PAGE_ID_COLUMN').getValue();
						launchInContext(pageid);
						// interactive ISC task, launch it
					} else if (taskType === 'jobs') {
						// non interactive task, launch launcher
						// dialog widget
						this.showSchedulerForJob(itemId, item.getProperty('FINDER_NAME_COLUMN').getValue());
					} else if (taskType === 'ngenTask') {
						// non interactive task, launch launcher
						// dialog widget

						var pageid = item.getProperty('FINDER_PAGE_ID_COLUMN').getValue();
						if (clickType > 1) {
							window.open(window.location.href.split("#")[0] + "#" + pageid);
						} else {
							launchNextGenPage(pageid);
						}
					}
				}
			} else if (type === 'resources') {
				var resourceTypeProperty = item.getProperty('FINDER_RESOURCE_TYPE_COLUMN');
				var url = "workAreaDijit=ibm/usmi/widgets/WorkArea&sourceToken=" + itemId + "&selector=details";
				if (resourceTypeProperty) {
					var resourceType = (resourceTypeProperty.getValue());
					if (resourceType === 'System Chassis' || resourceType === 'Server' || resourceType === 'Switch'
							|| resourceType === 'Generic System') {
						url = "workAreaDijit=fsm/Pages/fsmChassis/chassisMapPage&sourceToken=" + itemId
								+ "&selector=ibm/usmi/dashboard/ResourceDashBoard&selectedGroup=chassis";

					}

				}
				if (clickType > 1) {
					window.open(window.location.href.split("#")[0] + "#" + url);
				} else {
					launchNextGenPage(url);

				}
			} else if (type === 'groups') {
				var url = "workAreaDijit=ibm/usmi/widgets/ResourceNavigator&dataset=GROUP:" + itemId;
				if (clickType > 1) {
					window.open(window.location.href.split("#")[0] + "#" + url);
				} else {
					launchNextGenPage(url);
				}
			}
		},
		setTooltip : function (id) {
			this.tooltipShown = id;
		},
		showSchedulerForJob : function (jobId, jobTitle) {
			var wizard = new JobLaunchWizard({
				id : "launchFinderJobScheduler",
				iTaskId : jobId,
				jobName : jobTitle
			});
			launchSchedulerDialog(jobTitle, wizard);

		},
		finderKeyPressed : function (event) {
			Utils.CMFinest('finder: keyPressed');
			Utils.CMTraceTimeDelta("finderTimer2", "keyPressed", true);
			if (this.tooltipShown) {
				this.tooltipShown.close();

			}
			if (event.keyCode === 27) {
				dijit.byId("findersearchBox").set("value", "");
			} else if (event.keyCode === 13) {
				var resultId;
				if (this._currentView === 'resources') {
					resultId = this._currentSingleResultArray.resourceId;
				} else if (this._currentView === 'groups') {
					resultId = this._currentSingleResultArray.groupId;
				} else if (this._currentView === 'tasks') {
					resultId = this._currentSingleResultArray.taskId;
				}
				if (resultId) {
					this.doClick(null, this._currentView + "#_" + resultId);
				}
				return;

			}
			var searchStringEntered = dijit.byId("findersearchBox").get('value');
			if (this._latestSearchEntry === searchStringEntered) {
				return;
			} else {
				this._latestSearchEntry = searchStringEntered;
			}
			if ((searchStringEntered === "") || (searchStringEntered === this._searchTextHintString)) { // ||
				// searchStringEntered.length
				// == 1
				this.closeSearchDetailDlg(true);
				return;
			}
			// this._keyPressed++;
			if (!this._searchDlgIsOpen || this._searchDetailDlg._isShown() === false) {
				djPopup.open({
					popup : this._searchDetailDlg,
					around : dijit.byId("findersearchBox").domNode,
					onCancel : djLang.hitch(this, this.closeSearchDetailDlg),
					parent : dijit.byId("findersearchBox")
				});
			}

			this._searchDlgIsOpen = true;
			dijit.byId("findersearchBox").focus();
			var searchTerm = searchStringEntered;
			Utils.CMFinest('finder: query: ' + searchTerm);
			this._searchDlgInitialized = false;
			this.initiateSearch(searchTerm, 0);
			// window.setTimeout(djLang.hitch(this,
			// this.initiateSearch, searchTerm, 0,
			// this._keyPressed), 500);

		},
		initiateSearch : function (searchTerm, state/* ,keyPressedNumber*/) {
			/*
			 * if(this._keyPressed !== keyPressedNumber) { return; }
			 */
			if (this._taskDataStore && this._groupDataStore && this._resourceDataStore) {
				if (djDom.byId('searchingText')) {
					djDomC.destroy('searchingText');
				}
				this._taskDataStore.initiateTaskSearch(searchTerm, this);
				this._groupDataStore.initiateTaskSearch(searchTerm, this);
				this._resourceDataStore.initiateTaskSearch(searchTerm, this);
			} else {
				var stateNumber = state % 3;
				if (!djDom.byId('searchingText')) {
					djDomC.create("div", {
						id : 'searchingText',
						innerHTML : MRI['Searching' + stateNumber]
					}, this._searchDetailDlg.containerNode);
				} else {
					djDom.byId('searchingText').innerHTML = MRI['Searching' + stateNumber];
				}
				setTimeout(djLang.hitch(this, 'initiateSearch', searchTerm, state + 1 /* , keyPressedNumber*/), 200);
			}
		},

		initializeDialogConstruct : function (typeOfSearch, items) {
			if (this._searchDlgInitialized === false) {
				this._searchDlgInitialized = true;
				djDomC.empty(this._searchDetailDlg.containerNode);
				djDomC.create("img", {
					onclick : djLang.hitch(this, this.closeSearchDetailDlg),
					src : "js/ibm/usmi/core/framework/themes/images/title_close.gif",
					title : MRI.Close,
					className : "closeImg"
				}, this._searchDetailDlg.containerNode);
				this._refreshImage = djDomC.create("img", {
					onclick : djLang.hitch(this, this._manualRefresh),
					src : "js/ibm/usmi/core/framework/themes/images/ac16_refresh.png",
					className : "refreshImg"
				}, this._searchDetailDlg.containerNode);
				this.changeRefreshCount();

				var header = djDomC.create("span", {
					id : "finderfinderMenu",
					style : "display:inline;"
				}, this._searchDetailDlg.containerNode);

				var mainDivTabContainer = djDomC.create("div", {
					id : 'mainDivTabContainer'
				}, this._searchDetailDlg.containerNode);
				if (dijit.byId("findertab")) {
					dijit.byId("findertab").destroy();
				}
				this._tabbedContainer = new djTabContainer({
					id : "findertab",
					style : "width:100%;",
					doLayout : true,
					useMenu : false,
					useSlider : false
				}, mainDivTabContainer);
				var lastSelectedTab = djCookie("finder.currentView");
				if (lastSelectedTab && lastSelectedTab.length > 0) {
					this._currentView = lastSelectedTab;
				}
				this._tabbedContainer.addChild(this.initializeMenu('resources', MRI.FinderResourceMenu, header,
						this._tabbedContainer, (this._currentView === 'resources')));
				this._tabbedContainer.addChild(this.initializeMenu('groups', MRI.FinderGroupMenu, header,
						this._tabbedContainer, (this._currentView === 'groups')));
				this._tabbedContainer.addChild(this.initializeMenu('tasks', MRI.FinderTaskMenu, header,
						this._tabbedContainer, (this._currentView === 'tasks')));
				connect.connect(this._tabbedContainer, "selectChild", djLang.hitch(this, this.tabSelected));
			}
			// add the count in parenthesis to the tabs
			var numItems;
			if (items) {
				numItems = items;
			} else {
				numItems = 0;
			}
			if (this._tabbedContainer) {
				var children = this._tabbedContainer.getChildren();
				var idx = 0;
				if (typeOfSearch === 'groups') {
					idx = 1;
				} else if (typeOfSearch === 'tasks') {
					idx = 2;
				}
				children[idx].title = children[idx].title + "(" + numItems + ")";
			}
		},

		tabSelected : function (child) {
			if (child.content.id.indexOf('tasks') !== -1) {
				this._currentView = 'tasks';
			} else if (child.content.id.indexOf('resources') !== -1) {
				this._currentView = 'resources';
			} else if (child.content.id.indexOf('groups') !== -1) {
				this._currentView = 'groups';
			}
			djCookie("finder.currentView", this._currentView, {
				expires : 30
			});
			Utils.CMFinest('finder: ' + child);
			Utils.CMFinest('finder: ' + this._currentView);
		},

		initializeMenu : function (type, headerText, header, tabbedContainer, isSelected) {
			var div = djDomC.create("div", {
				id : 'finder' + type + "_containerDiv"
			});
			var cp1 = new djContentPane({
				title : headerText,
				content : div,
				refreshOnShow : true,
				selected : isSelected
			});
			return cp1;
		},
		destroy : function () {
			this.inherited(arguments);
			if (this._dataStore) {
				this._dataStore.destroy();
				this._dataStore = null;
			}
			if (this._resourceDataStore) {
				this._resourceDataStore.destroy();
				this._resourceDataStore = null;
			}
			if (this._groupDataStore) {
				this._groupDataStore.destroy();
				this._groupDataStore = null;
			}
			if (this._taskDataStore) {
				this._taskDataStore.destroy();
				this._taskDataStore = null;
			}

		}
	});
});
