/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
// Finder FinderBaseDataStore
define("ibm/usmi/widgets/finder/FinderBaseDataStore", [
	"dojo/_base/lang",
	"dojo/on",
	"dojo/dom",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dojo/i18n!ibm/usmi/nls/frameworkResources",
	"dijit",
	"dijit/Tooltip",
	"dojo/_base/declare",
	"dojox/data/AndOrReadStore",
	"ibm/usmi/util/Utils",
	"./TaskCategory"
], function (djLang, on, djDom, djClass, djDomC, MRI, dijit, tooltip, declare, djAndOrReadStore, Utils, TaskCategory) {

	return declare(null, {

		_dataStore : null,
		_type : null,
		_categories : null,
		_toolTips : null,

		constructor : function (type, domId) {
			this._type = type;
			this._toolTips = [];
		},
		getCategories : function () {
			return this._categories;
		},

		populateCategoriesWithSelectedItems : function (myitems) {
			var i = 0, item;
			for (i = 0; i < myitems.length; i++) {
				item = myitems[i];
				this._categories[item.categoryIdx].addSelected(item.objectIdx);
			}
			Utils.CMFinest('finder: populatedCategories for ' + i + ' items');
		},

		populateSearchResultItem : function (resultItemDomElement, finderObj, extraClass, notClickable) {
			var classname = 'SelectableCommonAction';
			if (notClickable) {
				classname = 'notSelectableCommonAction';
			}
			if (extraClass) {
				classname = extraClass + " " + classname;
			}
			djClass.add(resultItemDomElement, classname);
			if (notClickable !== true) {
				on(djDom.byId(resultItemDomElement), "mouseover", function () {
					this.className = classname + "Hover";
				});
				on(djDom.byId(resultItemDomElement), "mouseout", function () {
					this.className = classname;
				});
				on(djDom.byId(resultItemDomElement), "mousedown", djLang.hitch(finderObj, finderObj.doClick));

			}
		},

		createCategoryHTMLElements : function (numberOfResults, finderObj, moreId, moreText, notClickable) {
			var count, moreActionsDiv, sectionDomNode, sectionTitleDomNode, itemId, itemDiv;
			var maxNumberOfResults = 32;
			var numberOfItemsCurrentlyShowing = 0;
			var i = 1, j;
			for (i = 1; i < this._categories.length; i++) {
				count = this._categories[i].getSelectedCount();
				if (count === 0) {
					continue;
				}

				//create category section         
				//Utils.CMFinest('finder: creating category: ' + this._categories[i].getCategory());
				sectionDomNode = djDomC.create("div", {
					id : "categorySection" + this._type + "#_" + i
				}, djDom.byId('finder' + this._type + "_containerDiv"));
				sectionTitleDomNode = this.objHighlightSearchStr(sectionDomNode, dijit.byId('finder' + "searchBox")
						.get('value'), this._categories[i].getCategory(), 'true');
				//djClass.add(sectionTitle, "categorySectionTitle");
				for (j = 0; j < count; j++) {
					if (maxNumberOfResults <= 0) {
						break;
					}
					itemId = this._type + "#_" + this._categories[i].getSelectedId(j);
					var innerId = itemId + "innerDiv";
					var icon = this._categories[i].getSelectedIcon(j);
					var iconHtml = "";
					if (icon) {
						iconHtml = "<img class=\"finderIcon\" src=\"" + icon + "\"/>";
					}
					itemDiv = djDomC.create("div", {
						id : itemId,
						oncontextmenu : "return false;",
						innerHTML : iconHtml + "<div id=\"" + innerId + "\"/>"
					}, "categorySection" + this._type + "#_" + i);
					this.objHighlightSearchStr(djDom.byId(innerId), dijit.byId('finder' + "searchBox").get('value'),
							this._categories[i].getSelectedName(j));
					var toolTipLabel = this._constructTooltip(finderObj, this._categories[i].getSelectedId(j));
					var tp = new tooltip({
						connectId : [ itemDiv
						],
						label : toolTipLabel,
						closable : true,
						position : [ "before"
						]
					//id: overlayId+"_buttonTooltip"
					});
					on(itemDiv, "mouseover", djLang.hitch(finderObj, finderObj.setTooltip, tp));
					//djDom.byId('workAreaDiv').innerHTML = toolTipLabel;
					this._toolTips.push(tp);
					this.populateSearchResultItem(itemId, finderObj);
					maxNumberOfResults--;
					numberOfItemsCurrentlyShowing++;
				}
				if (maxNumberOfResults <= 0) {
					break;
				}

				maxNumberOfResults = maxNumberOfResults - 2;
			}
			if (maxNumberOfResults <= 0) {

				moreActionsDiv = djDomC.create("div", {
					id : moreId,
					innerHTML : '<BR/><span> ... ' + ' ' + moreText + '</span>'
				}, djDom.byId('finder' + this._type + "_containerDiv"));
				this.populateSearchResultItem(moreActionsDiv, finderObj, 'finderMoreTasks', notClickable);
			}

		},

		clearSelection : function () {
			var i;
			for (i = 0; i < this._categories.length; i++) {
				if (this._categories[i]) {
					this._categories[i].clearSelection();
				} else {
					break;
				}
			}
		},

		populateItems : function (list) {
			this._categories = [];
			if (!list || list === "") {
				return;
			}
			var size = list.length;
			var i = 0;
			var items = [];
			this._categories[0] = new TaskCategory("Uncategorized");
			for (i = 0; i < list.length; i++) {
				var object = list[i];
				var catIdx = 0;
				var objectIdx;
				for (catIdx = 0; catIdx < list.length + 1; catIdx++) {
					if (this._categories[catIdx]) {
						if (this._categories[catIdx].getCategory() === object.category) {
							var tempcategory = this._categories[catIdx];
							objectIdx = tempcategory.add(object.id, object.label, object.icon);
							break;
						}
					} else {
						this._categories[catIdx] = new TaskCategory(object.category);
						objectIdx = this._categories[catIdx].add(object.id, object.label, object.icon);
						break;
					}
				}
				items[i] = {
					'id' : object.id,
					'name' : object.label,
					'category' : object.category,
					'categoryIdx' : catIdx,
					'objectIdx' : objectIdx
				};
			}
			Utils.CMFinest('finder: parsed items, total: ' + i);
			var dataStore = {
				identifier : 'id',
				label : 'name',
				items : items
			};
			this._dataStore = new djAndOrReadStore({
				data : dataStore
			});
			Utils.CMFinest("finder: " + this._dataStore);

		},
		objHighlightSearchStr : function (targEl, searchStr, targetStr, isCategory) {
			targEl.innerHTML = "";
			var type = "span";
			var textClass = "";
			if (isCategory) {
				type = "h1";
				textClass = "categorySectionTitle";
			}
			var resultEl = djDomC.create(type, {
				className : textClass
			}, targEl);

			var re = new RegExp(searchStr, "i");
			var startPos = targetStr.search(re);

			//if no match just return the string as-is 
			if (startPos <= -1) {
				djDomC.create("span", {
					innerHTML : targetStr
				}, resultEl);
				return resultEl;
			}

			var s0 = djDomC.create("span", {
				innerHTML : targetStr.substring(0, startPos)
			}, resultEl);
			var s1 = djDomC.create("span", {
				innerHTML : targetStr.substring(startPos, startPos + searchStr.length)
			}, resultEl);
			djClass.add(s1, "SearchSubstringHighlight");
			djDomC.create("span", {
				innerHTML : targetStr.substring(startPos + searchStr.length, targetStr.length)
			}, resultEl);
			return resultEl;

		},
		_constructTooltip : function (finderObj, itemId) {
			var tooltip, clickHere, category, categoryLabel, label, labelHeader, additional;
			var item = finderObj._indexedItems[itemId];
			var itemName = item.getProperty('FINDER_NAME_COLUMN').getValue();
			var itemType = item.getProperty('FINDER_TYPE_COLUMN').getValue();
			var itemCategory = item.getProperty('FINDER_CATEGORY_COLUMN').getValue();
			var itemIcon /*= item.getProperty('FINDER_ICON_COLUMN').getValue()*/;
			if (item.getProperty('TOOLTIP_PROPS_COLUMN').getValue()
					&& item.getProperty('TOOLTIP_PROPS_COLUMN').getValue().length > 0) {
				additional = JSON.parse(item.getProperty('TOOLTIP_PROPS_COLUMN').getValue());
			}
			if (this._type === 'resources') {
				clickHere = MRI.FinderTooltipClickResources;
				labelHeader = djLang.replace(MRI.FinderTooltipEntryType, [ itemCategory
				]);
			} else if (this._type === 'tasks') {
				categoryLabel = MRI.FinderTooltipCategory;
				if (itemType === 'tasks') {
					labelHeader = MRI.FinderTooltipExternalTask;
					clickHere = MRI.FinderTooltipClickTask;
				} else if (itemType === 'jobs') {
					labelHeader = MRI.FinderTooltipJob;
					clickHere = MRI.FinderTooltipClickJob;
				} else if (itemType === 'urlTasks') {
					labelHeader = MRI.FinderTooltipUrl;
					clickHere = MRI.FinderTooltipClickUrl;
				} else if (itemType === 'ngenTask') {
					labelHeader = MRI.FinderTooltipPage;
					clickHere = MRI.FinderTooltipClickPage;
				}
				//description = 'Description: ' + item.getProperty('FINDER_DESCRIPTION_COLUMN').getValue() + "<BR/>";

				//var item = 

			} else if (this._type === 'groups') {
				clickHere = MRI.FinderTooltipClickGroup;
				labelHeader = djLang.replace(MRI.FinderTooltipEntryType, [ itemCategory
				]);
			}
			var iconHtml = "";
			if (itemIcon) {
				iconHtml = "<img class=\"finderToolTipIcon\" src=\"" + window.location.protocol + "//"
						+ window.location.host + itemIcon + "\"/>";
			}
			label = "<span>" + iconHtml + "<span class=\"finderTooltipHeaderText\">" + labelHeader + itemName
					+ "</span><BR/>";
			var categoryHtml = "";
			if (categoryLabel) {
				category = categoryLabel + itemCategory;
				categoryHtml = "<span class=\"finderTooltipCategoryText\">" + category + "</span><BR/>";
			}
			var additionalStr = '', idx = 0;
			if (additional) {
				for (; idx < additional.length; idx++) {
					var tPropLabel = additional[idx].LABEL;
					var tPropValue = additional[idx].VALUE;
					if (tPropValue && tPropValue.length > 0) {
						additionalStr += tPropLabel + ": " + tPropValue + "<BR/>";
					}
				}
				/*additionalStr += "<BR/>"*/
			}
			tooltip = label + categoryHtml + additionalStr + "<BR/><span class=\"finderToolTipClickHere\">" + clickHere
					+ "</span>";
			return tooltip;
		},
		destroy : function () {
			this.inherited(arguments);
			if (this._dataStore) {
				this._dataStore = null;
			}
			this.clearTooltips();

			if (this._categories) {
				var i = 0;
				for (; i < this._categories.length; i++) {
					this._categories[i].destroy;
				}
				this._categories.length = 0;
			}
		},
		clearTooltips : function () {
			if (this._toolTips) {
				var i = 0;
				for (; i < this._toolTips.length; i++) {
					this._toolTips[i].destroy();
				}
				this._toolTips.length = 0;
			}
		}
	});
});
