/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/widgets/table/DefaultItemsTableView", [
	"dojo/_base/declare",
	"dojo/_base/kernel",
	"dojo/_base/lang",
	"dojo/_base/array",
	"dojo/aspect",
	"dijit/registry",
	"dojo/i18n!./nls/ColumnChooser",
	"ibm/tivoli/tip/navwidget/table/dijit/ItemsTableView",
	"ibm/usmi/widgets/table/ConfigureColumnsDialog",
	"ibm/usmi/core/utils/Logger",
	"dijit/place",
	"dojo/_base/connect",
	"ibm/usmi/widgets/table/DefaultTablePersistenceMixin"
], function (declare, dojo, lang, array, aspect, registry, ConfigureColumnsMRI, ItemsTableView, ConfigureColumnsDialog,
		Logger, place, connect, DefaultTablePersistenceMixin) {

	return declare([ ItemsTableView, DefaultTablePersistenceMixin
	], {

		// the logger
		_defaultItemsTableViewLogger : new Logger("ibm.usmi.widgets.table", "DefaultItemsTableView"),

		constructor : function () {
			this._pDEFAULT_WIDGET_DEF_INFO = {
				className : "ibm.tivoli.navmodel.TableWidgetDef",
				baseUrl : "/ibm/tivoli/rest",
				provider : "any",
				datasource : "any",
				widget : "table",
				definition : "UsmiItemsTable"
			};
		},

		augmentGridParams : function (gridParams) {
			// For info on gridParams, look at ibm/tivoli/tip/navwidget/table/dijit/AbstractTableView
			// and also at dojoe/table/Table.
			// Look at the source; do not trust the comments.
			this.inherited(arguments);

			gridParams.plugins.nestedSorting = false;

			// Make it scroll by default, instead of paging
			gridParams.plugins.pagination = false;

			// Get rid of the checkbox column for selection
			gridParams.plugins.indirectSelection = false;

			// Get rid of the refresh menu option
			gridParams.toolbarOptions.refreshMenu = false;

			// Get rid of the printer menu option
			gridParams.toolbarOptions.printerMenu = false;

			// Get rid of the export menu option
			gridParams.toolbarOptions.exporterMenu = false;

			// Disable drag-and-drop of rows and columns
			gridParams.plugins.dnd = false;

			// Row selection only, not columns or cells
			gridParams.plugins.selector = {
				"cell" : "disabled",
				"row" : "multiple",
				"col" : "disabled"
			};

			gridParams = this.persistenceGridParams(this._dataStore, gridParams);

			return gridParams;
		},

		// AbstractTableView._initTableBilder connects to onTableLoad, and inadvertently shows the filter bar.
		// This aspect is called afterward, and hides the filter bar again.
		_initTableBuilder : function () {
			//TODO, hack, memory leak on NavModelTable.js, luckily we can clean up here

			var toolbarId, configureTableButtonId, configureTableButton, actionMenu, menuItems, lastMenuItem;
			if (this._table && this._table._tooltip) {
				this._table._tooltip.destroy();
			}
			this.inherited(arguments);
			aspect.after(this._table, "onTableLoad", lang.hitch(this, function () {
				if (this._table) {
					this._table.setFilter(null, true);

					//@188636 START
					this.initPersistence(this, this._grid, this._table._toolbar._quickFilter.searchField);
					//@188636 END
				}
			}));

			//Hack in our column chooser
			if (this._table && this._table._toolbar) {
				if (this._defaultItemsTableViewLogger.isFinest()) {
					this._defaultItemsTableViewLogger.finest("setting the custom Column Chooser", "_initTableBuilder");
				}
				this._table._toolbar._configureTable = function () {
					this.configureTableDialog = new ConfigureColumnsDialog({
						grid : this.inGrid,
						_structure : this._origStructure
					});
					this.configureTableDialog.show();
				};
				toolbarId = this._table._toolbar.id;
				if (toolbarId) {
					configureTableButtonId = "configureColIcon" + toolbarId;
					configureTableButton = registry.byId(configureTableButtonId);
					if (configureTableButton) {
						configureTableButton.set("title", ConfigureColumnsMRI.CONFIG_COLUMNS);
					}
				}
				if (this._table._toolbar.actionListWidget) {
					actionMenu = this._table._toolbar.actionListWidget.dropDown;
					if (actionMenu) {
						menuItems = actionMenu.getChildren();
						if (menuItems && menuItems.length > 0) {
							lastMenuItem = menuItems[menuItems.length - 1];
							if (lastMenuItem && lastMenuItem.iconClass === "configureColIcon") {
								lastMenuItem.set("title", ConfigureColumnsMRI.CONFIG_COLUMNS);
								lastMenuItem.set("label", ConfigureColumnsMRI.CONFIG_COLUMNS);
							}
						}
					}
				}

			}
			
			
			//need to change the total items in filter bar message to the dataset items count
			if(this._table && this._table._grid && this._table._grid.filterBar){
				aspect.before(this._table._grid.filterBar, "_buildFilterState", lang.hitch(this, function () {
					this._table._grid.filterBar._totalSize=this.dataset.getItemCount();
					//need to show the filter bar if it's not showed
					if(!this._table._grid.filterBar.isVisible()){
						this._table._grid.filterBar.show();
					}
				}));
			}
			
			
			if(this._table && this._table._grid && this._table._grid && this._table._grid.model){
				connect.connect(this._table._grid.model, "onNew", lang.hitch(this, this.refreshFilterBar));
				connect.connect(this._table._grid.model, "onDelete", lang.hitch(this, this.refreshFilterBar));
				connect.connect(this._table._grid.model, "onSet", lang.hitch(this, this.refreshFilterBar));
			}
			
		},	
		
		refreshFilterBar: function(){
			if(this._table && this._table._grid && this._table._grid && this._table._grid.model && this._table._grid.filterBar){
				if(!this._table._grid.filterBar.isVisible()){
					//if filter bar is not visible, we do not want to refresh it
					return;
				}
				this._table._grid.filterBar._currentSize = this._table._grid.model.size();
				this._table._grid.filterBar._buildFilterState();
			}
		},

		_cleanUpImagePath : function (imagePath) {
			return this.inherited(arguments);
		},		

		handleLinkClickAction : function (viewId, objectId, propId, objectType) {
			// summary:
			//        Method to override base implementation so we get notified and can
			//      handle when a link is clicked in a table cell
			if (this._defaultItemsTableViewLogger.isFinest()) {
				this._defaultItemsTableViewLogger.entering("", "handleLinkClickAction");
				this._defaultItemsTableViewLogger.finest("handling link click for table view - prop: " + propId,
						"handleLinkClickAction");
			}
			var object, property;

			//ask the dataset for the correct object based on the objects id.
			//since we have already rendered these items, the value should be
			//cached and should be quick.
			switch (objectType) {
			case "ibm.tivoli.navmodel.Item":
				object = this.dataset.getItem({
					itemId : objectId
				});
				break;
			case "ibm.tivoli.navmodel.Link":
				object = this.dataset.getLink({
					itemId : objectId
				});
				break;
			case "ibm.tivoli.navmodel.Relationship":
				object = this.dataset.getRelationship({
					itemId : objectId
				});
				break;
			default:
				if (this._defaultItemsTableViewLogger.isFinest()) {
					this._defaultItemsTableViewLogger.finest("unknown objectType: " + objectType,
							"handleLinkClickAction");
				}
			}

			if (object) {
				property = object.getProperty(propId);
			}

			if (object && property) {
				this.daHelper.executeDefaultAction(this, property);
			}

			if (this._defaultItemsTableViewLogger.isFinest()) {
				this._defaultItemsTableViewLogger.exiting("", "handleLinkClickAction");
			}
		},

		// Override getContextMenuAdapter, so we can reposition the context menu.
		// Necessary since the menu will change size after it is first displayed,
		// as more items come in over CURI.		
		getContextMenuAdapter : function () {
			// get the ContextMenuAdapter, which I will later return.
			var cma = this.inherited(arguments);
			var coords;

			// Before the ContextMenuAdapter.contextMenuOpen method
			aspect.before(cma, "contextMenuOpen", function (event) {
				// Save the original mouse-click coordinates for later
				coords = {
					x : event.pageX,
					y : event.pageY
				};
			});

			// After the ContextMenuAdapter._menuItemsLoaded method
			aspect.after(cma, "_menuItemsLoaded", function (success) {
				// Put the menu in the correct place, so it does not go off the page
				if (success) {
					var menu = this.getMenu();
					var orient = menu.isLeftToRight() ? 'L' : 'R';
					var wrapper = menu.domNode.parentNode;
					if (wrapper !== null && coords) {
						place.at(wrapper, coords, orient == 'R' ? [
							'TR',
							'BR',
							'TL',
							'BL'
						] : [
							'TL',
							'BL',
							'TR',
							'BR'
						]);
					}
				}
			}, true);

			return cma;
		},

		destroy : function () {
			//TODO, hack, memory leak on NavModelTable.js, luckily we can clean up here
			if (this._table && this._table._tooltip) {
				this._table._tooltip.destroy();
				this._table._tooltip = null;
			}
			if (this._table && this._table._tooltipProperties) {
				this._table._tooltipProperties.destroyRecursive();
				this._table._tooltipProperties=null;
			}
			this.inherited(arguments);
		}

	});
});
