/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/widgets/table/DefaultTablePersistenceMixin", [
	"dojo/_base/declare",
	"dojo/_base/kernel",
	"dojo/_base/lang",
	"dojo/_base/array",
	"dojo/aspect",
	"dijit/registry",
	"ibm/usmi/widgets/table/DefaultItemsTableViewPersist"
], function (declare, dojo, lang, array, aspect, registry, Persist) {

	return declare([], {

		// add gridparams necessary for persistence
		persistenceGridParams : function (datastore, gridParams) {
			var persistId;
			if (!gridParams.id) {
				if (datastore && datastore.dataset && datastore.dataset._dataset) {
					persistId = "persistgrid:" + datastore.dataset._dataset.id;
				} else {
					persistId = "emptygrid";
				}
			} else {
				persistId = gridParams.id;
			}
			
			var module = {
				moduleClass : Persist,
				key : this._buildPersistenceKey(persistId)
			};
			if (gridParams.modules) {
				gridParams.modules.push(module);
			} else {
				gridParams.modules = [ module ];
			}
			return gridParams;
		},

		// hook up persistence
		initPersistence : function (t, g, searchField) {
			var selection;
			var filter;

			// check parameters for whether row selection should be persisted...
			if (g.persist) {
				filter = g.persist.registerAndLoad('filter', function () {
					if (g.filterBar) {
						return g.filterBar.filterData;
					} else {
						return null;
					}
				});

				selection = g.persist.registerAndLoad('selection', function () {
					if (g.select) {
						return g.select.row.getSelected();
					} else {
						return null;
					}
				});

				if (filter && g.filterBar) {
					// if it is a single basic filter, insert text into quick filter field
					if (searchField && filter.type === "any" && filter.conditions && filter.conditions.length === 1
							&& filter.conditions[0].condition === "contain") {
						searchField.value = filter.conditions[0].value;
					}
					g.filterBar.applyFilter(filter);
					g.filterBar.show(); // ensure filter bar displays
				}

				if (selection) {
					var model = g.model;
					array.forEach(selection, function (id) {
						model.markById(id, 1);
					});
					model.when();
				
// David said following was needed for toolbar revalidation - internally crashes for tree tables					
//					if(g.select.row) {
//						g.select.row.onSelected();
//					}

					// scroll to first selection
					// index/scroll needs to be delayed until the model data shows up
					// TODO: find a better method than a timeout delay
					setTimeout(function () {
						if (g.vScroller) {
							var selectIndex = -1;
							array.forEach(selection, function (id) {
								var index = model.idToIndex(id);
								if (selectIndex === -1 || selectIndex > index) {
									selectIndex = index;
								}
							});
							g.vScroller.scrollToRow(selectIndex);
						}
					}, 1000);
				}
			}
		},

		// build a new persistence key based on the url and sorted hash parameters
		// strip out sourceToken and other variables that artificially make the key unique for table
		_buildPersistenceKey : function (id) {
			var hash = dojo.hash();

			// build up an array so we can sort it
			var storedParams = [];
			var params = hash.split('&');
			for ( var i = 0; i < params.length; i++) {
				if (params[i].split('=')[0] !== "sourceToken") {
					storedParams.push(params[i]);
				}
			}

			// sort it...
			var newHash = "";
			storedParams.sort();
			for ( var j = 0; j < storedParams.length; j++) {
				if (j > 0) {
					newHash += "&";
				}
				newHash += storedParams[j];
			}

			var loc = window.location;
			return loc.protocol + "//" + loc.host + loc.pathname + "?" + newHash + '/' + id;
		}

	});
});