/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

define("ibm/usmi/widgets/table/_TableOptions", [
    "dojo/dom",
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dijit/_WidgetBase",
	"dijit/registry",
	"ibm/usmi/util/DataStore",
	"ibm/tivoli/navmodel/Dataset",
	"dojoe/tablex/_TableOptions",
	"ibm/usmi/widgets/table/_ColumnVisible",
	"dojo/store/Memory",
	"gridx/core/model/cache/Sync",
	"gridx/Grid",
	"gridx/modules/move/Row",
	"gridx/modules/dnd/Row",
	"gridx/modules/CellWidget"
], function (dom, declare, array, lang, _WidgetBase, registry, DataStore, Dataset, TableOptions, ColumnVisible, Memory,
		Cache, Grid, MoveRow, DndRow, CellWidget) {
	return declare([
		_WidgetBase,
		TableOptions
	], {
		allColumns : null,
		columnMap : {},
		defaultColumns : null,
		//checkedColumns : [],
		structure : null,
		store : null,
		gridSelected : null,
		nextGridValue : 0,

		postCreate : function () {
			this.structure = [
				{
					id : 'name',
					field : 'name',
					name : '',
					//width : '200px'
				},
				{
					id : 'remove',
					field : 'remove',
					name : '',
					width : '20px',
					widgetsInCell : true,
					decorator : function () {
						//Generate cell widget template string
						return [
							'<span ',
							'data-dojo-attach-point="btn" ',
							'class="dijitReset dijitInline dijitIcon dijitIconDelete" ',
							'></span>'
						].join('');
					},
					setCellValue : function (data) {
						//"this" is the cell widget
						this.btn.onclick = lang.hitch(this, function (evt) {
							var thisContext = registry.byId(data.context);
							thisContext.removeUsingButton(data.tableId, data.columnId);
						});
					}
				}
			];
			this.store = new Memory({
				data : []
			});

			this.initializeDataStores();
			this.gridSelected = Grid({
				id : this.configColumnsWidget.id + 'selectedColumnsGrid',
				cacheClass : Cache,
				store : this.store,
				structure : this.structure,
				headerHidden : true
			});

			this.gridSelected.placeAt(this.configColumnsWidget.id + 'selectedColumnsdiv');
			this.gridSelected.startup();
		},
		destroy : function () {
			//this is for destroying child templated widgets 
			//this should happen by itself but need to figure out why dijit._Templated is not taking care of it...
			var widgets = registry.findWidgets(this.domNode);
			array.forEach(widgets, function (w) {
				if (w) {
					if (w.destroyRecursive) {
						w.destroyRecursive();
					} else if (w.destroy) {
						w.destroy();
					}
				}
			});
			// this is for widgets added as child to this widget and their children..
			//not present in template
			var widgetArr = this.getChildren();
			array.forEach(widgetArr, function (child) {
				if (child) {
					if (child.destroyRecursive) {
						child.destroyRecursive();
					} else if (child.destroy) {
						child.destroy();
					}
				}
			});
			this.allColumns = null;
			this.columnMap = null;
			this.defaultColumns = null;
			this.checkedColumns = null;
			this.gridSelected = null;
			this.store = null;
			this.structure = null;
			this.inherited(arguments);
		},
		removeUsingButton : function (tableId, columnId) {
			this.store.remove(tableId);
			columnWidget = registry.byId(this.id + "column_" + columnId);
			columnWidget.setVisible(false);
		},
		addColumnsVisible : function (columns) {
			var widget = null;
			var column = null;
			var tableColumns = this.grid._columns;
			console.info(this.grid.getParent().getParent()._dataStore);
			//columns = this._columnUnion(columns, this.grid.layout.cells);
			var i;
			for (i in columns) {
				if (i) {
					column = columns[i];
					/*this.grid.layout.cells.push(column);
					if (column.field && !dom.byId(this.id + "column_" + column.field)) {
						console.info(column.visible);
						widget = new ColumnVisible({
							id : this.id + "column_" + column.field,
							columnid : this.id + "column_" + column.field,
							checked : column.visible === undefined ? !column.hidden : column.visible,
							label : column.name,
							columnChooser : this
						});
						this.addChild(widget);
						if (widget.isVisible() && this.checkedColumns.indexOf(widget.label) === -1) {
							this.checkedColumns.push(widget.label);
						}
					} else */

					if (column.getId && column.getId() && !dom.byId(this.id + "column_" + column.getId())) {
						widget = new ColumnVisible({
							id : this.id + "column_" + column.getId(),
							columnid : this.id + "column_" + column.getId(),
							checked : this._isColumnVisibleInTable(column, tableColumns),//column.hidden,
							label : column.getLabel(),
							columnChooser : this
						});
						this.addChild(widget);
					}
				}
			}

			for ( var k in tableColumns) {
				column = tableColumns[k];
				var temp = this.store.query({
					name : column.name
				});
				if ((!this.store.query({
					name : column.name
				}) || temp.length === 0)) {
					var array = [];
					array["tableId"] = this.nextGridValue;
					array["columnId"] = column.field;
					array["context"] = this;
					this.store.put({
						id : this.nextGridValue,
						order : this.nextGridValue,
						name : registry.byId(this.id + "column_" + column.field).label,
						remove : {
							tableId : this.nextGridValue,
							columnId : column.field,
							context : this.id
						}
					});
					this.nextGridValue++;
				}
			}
			this.configColumnsWidget.loadingDiv.style.display = "none";
			this.configColumnsWidget.columnsDiv.style.display = "block";
			this.configColumnsWidget.columnsDiv.style.paddingLeft = "30px";
			this.configColumnsWidget.columnsDiv.style.textIndent = "-30px"; 
		},

		_isColumnVisibleInTable : function (column, tableColumns) {
			var tableColumn;
			for ( var i = 0; tableColumns && i < tableColumns.length; i++) {
				tableColumn = tableColumns[i];
				if (!tableColumn.field) {
					continue;
				}
				if (!column.getId) {
					break;
				}
				if (column.getId() === tableColumn.field) {
					this.columnMap[tableColumn.field] = column;//put the raw column to cache
					if (tableColumn.visible !== undefined) {
						return tableColumn.visible;
					} else {
						return !tableColumn.hidden;
					}
				}
			}
			if (column.visible !== undefined) {
				return column.visible;
			}
			return false;
		},

		_columnUnion : function (array1, array2) {
			var array3 = array1;
			var element = null;
			var element1 = null;
			var element2 = null;
			var noMatch = true;
			for ( var i in array2) {
				element = array2[i];
				for ( var j in array1) {
					if (array1[j].getId && element.field) {
						element1 = array1[j].getId();
						element2 = element.field;
					} else if (element.getId && array1[j].field) {
						element1 = element.getId();
						element2 = array1[j].field;
					}
					if (element1 === element2) {
						noMatch = false;
						this.columnMap[element1] = array3[j];
						array3[j] = element;
					}
				}
				if (noMatch) {
					array3.push(element);
				}
				noMatch = true;
			}
			return array3;
		},

		onSelectAll : function () {
			//for(var j=0; j<this.grid.structure.length; j++) {
			//var columns = this.grid.structure[j].rows;    
			//Anupama :24May11, replacing for..in construct with regular for loop since for..in construct interfering with changes in Array prototype in IE7.
			//26600: select all,de-select all feature in Table configure options dialog not working on IE7 browser.
			//for (var i in columns) {
			/*LS - 13/01/2012 - 30956 - Unresponsive script msg when using configure options */
			//removing the indirect way of fetching columns through grid structure and getting it through layout.cells directly
			// this is improve performance of this feature reducing no of iterations...
			var k;
			for (k = 0; k < this.allColumns.length; k++) {
				var column = this.allColumns[k];
				var columnWidget;
				if (column && column.field) {
					columnWidget = registry.byId(this.id + "column_" + column.field);
					columnWidget.setVisible(true);
				} else {
					columnWidget = registry.byId(this.id + "column_" + column.getId());
					columnWidget.setVisible(true);
				}
			}
			//}
		},
		onDeselectAll : function () {
			//for(var j=0; j<this.grid.structure.length; j++) {
			//var columns = this.grid.structure[j].rows;
			//Anupama :24May11, replacing for..in construct with regular for loop since for..in construct interfering with changes in Array prototype in IE7.
			//26600: select all,de-select all feature in Table configure options dialog not working on IE7 browser.
			//for (var i in columns) {
			/*LS - 13/01/2012 - 30956 - Unresponsive script msg when using configure options */
			//removing the indirect way of fetching columns through grid structure and getting it through layout.cells directly
			// this is improve performance of this feature reducing no of iterations...
			for ( var k = 0; k < this.allColumns.length; k++) {
				var column = this.allColumns[k];
				var columnWidget;
				if (column && column.field) {
					columnWidget = registry.byId(this.id + "column_" + column.field);
					columnWidget.setVisible(false);
				} else {
					columnWidget = registry.byId(this.id + "column_" + column.getId());
					columnWidget.setVisible(false);
				}
			}
			this.store.destroy();
			this.store = new Memory({
				data : []
			});
			//this.checkedColumns = [];
			//}
		},
		onRestDefaults : function () {
			var columnWidget;
			for (var k = 0; k < this.nextGridValue; k++){
				if(this.store.get(k)){
					columnWidget = registry.byId(this.id + "column_" + this.store.get(k).remove.columnId);
					columnWidget.setVisible(false);
				}
			}
			this.store = new Memory({
                data : []
			});
			this.nextGridValue = 0;
			for ( var j in this.defaultColumns) {
				if (j) {
					if (this.defaultColumns[j].getId()) {
						columnWidget = registry.byId(this.id + "column_" + this.defaultColumns[j].getId());
						columnWidget.setVisible(true);
					}
				}
			}
		},
		updatedCheck : function (elementWidget) {
			var inStore = this.store.query({
				name : elementWidget.label
			});
			if (elementWidget.isVisible() && (!this.store.query({
				name : elementWidget.label
			}) || inStore.length === 0)) {
				//this.checkedColumns.push(elementWidget.label);
				this.store.put({
					id : this.nextGridValue,
					order : this.nextGridValue,
					name : elementWidget.label,
					remove : {
						tableId : this.nextGridValue,
						columnId : elementWidget.id.replace(this.id + "column_", ""),
						context : this.id
					//this
					}
				});
				this.nextGridValue++;
			} else {
				if (!elementWidget.isVisible() && this.store.query({
					name : elementWidget.label
				}) && inStore.length !== 0) {
					this.store.remove(this.store.query({
						name : elementWidget.label
					})[0].id);
				}
			}
			//this.checkedColumns.sort();
			//var list = '<table>';
			this.gridSelected.destroy();
			/*this.store = new Memory({
				data : []
			});
			for ( var i in this.checkedColumns) {
				if (i) {
					this.store.put({
						id : i,
						order : i,
						name : this.checkedColumns[i]
					});
					//	list += '<tr><td>' + this.checkedColumns[i] + '</td></tr>';
				}
			}*/
			//list += '</table>';
			//Create a grid to display the currently selected columns, and allowing reorder of the columns
			this.gridSelected = Grid({
				id : this.configColumnsWidget.id + 'selectedColumnsGrid',
				cacheClass : Cache,
				store : this.store,
				structure : this.structure,
				style : 'width: 100%; height: 98%; overflow: auto',
				selectRowTriggerOnCell : true,
				headerHidden : true,
				modules : [
					MoveRow,
					DndRow,
					CellWidget
				]
			});
			this.gridSelected.placeAt(this.configColumnsWidget.id + 'selectedColumnsdiv');
			this.gridSelected.startup();

			//Adjust the formatting so it shows up correctly instead of being presented with multiple layers of scroll bars,
			//This is a *hack*, but will do for the short term
			dom.byId(this.configColumnsWidget.id + 'selectedColumnsdiv').setAttribute('style',
					'width: 95%; height: 98%; left: 10px; top: 2px; position: absolute;');
			//dom.byId(this.configColumnsWidget.id + 'selectedColumnsdiv').innerHTML = list;
		},
		orderColumns : function (selectedColumns) {
			var orderedColumns = [];
			var orderedRows = [];
			var counter;
			//Get the list of selected columns from the grid and then sort using the order field, as that is what the table uses for it's sorting
			//and it will be accurate
			for ( var i in this.gridSelected.store.data) {
				var dataPoint = this.gridSelected.store.data[i];
				counter = 0;
				while (counter < orderedRows.length && orderedRows[counter].order < dataPoint.order) {
					counter++;
				}
				orderedRows.splice(counter, 0, dataPoint);
			}
			for ( var i in orderedRows) {
				if (selectedColumns[orderedRows[i].name]) {
					orderedColumns.push(selectedColumns[orderedRows[i].name]);
				}
			}
			return orderedColumns;
		},

		_isColumnVisible : function (column) {
			var columnWidget;
			if (column && column.field) {
				columnWidget = registry.byId(this.id + "column_" + column.field);
				if (columnWidget) {
					return columnWidget.isVisible();
				}
			} else if (column && column.getId) {
				columnWidget = registry.byId(this.id + "column_" + column.getId());
				if (columnWidget) {
					return columnWidget.isVisible();
				}
			}
			return false;
		},

		_getRawColumn : function (column) {
			if (column && column.field) {
				return this.columnMap[column.field];
			} else {
				return column;
			}
		},

		onSave : function () {
			/*LS - 13/01/2012 - 30956 - Unresponsive script msg when using configure options */
			//removing the indirect way of fetching columns through grid structure and getting it through layout.cells directly
			// this is improve performance of this feature reducing no of iterations...     
			var selectedColumns = {};
			var tableColumns = this.grid._columns;
			var column, rawColumn;
			//retain current presents columns
			for ( var i = 0; i < tableColumns.length; i++) {
				column = tableColumns[i];
				rawColumn = this._getRawColumn(column);
				if (!rawColumn) {
					continue;
				}
				if (this._isColumnVisible(column) && !selectedColumns[rawColumn.getLabel()]) {
					selectedColumns[rawColumn.getLabel()] = rawColumn;
				}
			}

			console.log(selectedColumns);

			//add new selected columns
			for ( var k = 0; k < this.allColumns.length; k++) {
				column = this.allColumns[k];
				rawColumn = this._getRawColumn(column);
				if (column && !selectedColumns[rawColumn.getLabel()]) {
					if (this._isColumnVisible(column)) {
						selectedColumns[rawColumn.getLabel()] = rawColumn;
					}
				}
			}

			if (selectedColumns === {}) {
				console.log("selected items is zero, use default columns instead");
				selectedColumns = this.defaultColumns;
				this.onRestDefaults();
			}
			if (selectedColumns !== this.defaultColumns) {
				selectedColumns = this.orderColumns(selectedColumns);
			}

			var setColumnsForMixin = function () {
				//console.info('XYZZY - This Got called!!!!!');
				this.grid.getParent().getParent()._dataStore.dataset.destroy();
				//if the table is using widget definition, we need to reset the table.view defintion
				if (this.grid.getParent().getParent()._tableWidgetDefinition
						&& this.grid.getParent().getParent()._tableWidgetDefinition._widgetDef
						&& this.grid.getParent().getParent()._tableWidgetDefinition._widgetDef.table
						&& this.grid.getParent().getParent()._tableWidgetDefinition._widgetDef.table.view) {
					this.grid.getParent().getParent()._tableWidgetDefinition._widgetDef.table.view = [];
				}
				this.grid.getParent().getParent()._dataStore.dataset = new Dataset(
						this.grid.getParent().getParent()._dataStore.dataset.getArgs());
				//this.destroy();
			};

			console.info(selectedColumns);
			this.grid.store.dataset.setColumns({
				columns : selectedColumns,
				onComplete : lang.hitch(this, setColumnsForMixin)
			});
			/*Ls - 17/01/2012 - 30956 - further reducing the time taken to hide/show columns*/
			/*this.grid.views.forEach(function (v, i) {
				var w = v.width;
				if (w && w != "auto") {
					v._togglingColumn = dojo.marginBox(v.structure.cells[0].getHeaderNode()).w || 0;
				}
				v.update();
			});*/
			//this.checkedColumns = [];
		},
		initializeDataStores : function (domId) {
			var gotAllColumns = function (cols) {
				this.allColumns = cols;
				this.addColumnsVisible(cols);
			};
			var gotColumnsError = function () {
				console.info('Error on get columns');
			};
			var dataset = this.grid.store.dataset;
			var columnArgsForMixin = {
				type : this._pTableType,
				onComplete : lang.hitch(this, gotAllColumns),
				onError : lang.hitch(this, gotColumnsError),
				properties : [ "all"
				]
			};
			dataset.getColumns(columnArgsForMixin);

			var gotDefaultColumns = function (cols) {
				this.defaultColumns = cols;
			};

			columnArgsForMixin = {
				type : this._pTableType,
				onComplete : lang.hitch(this, gotDefaultColumns),
				onError : lang.hitch(this, gotColumnsError),
				properties : [ "default"
				]
			};
			dataset.getColumns(columnArgsForMixin);

		}
	});
});
