/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/widgets/util/PropertyNotebookDAHelper", [
    "dojo/_base/kernel",
    "dojo/_base/declare",
    "dojo/topic",
    // NextGen imports
    "ibm/tivoli/tip/navwidget/topology/dijit/DefaultActionHelper",
    "ibm/tivoli/tip/navwidget/dojo/NavWidgetPluggableViewObjectContext",
    "ibm/usmi/core/utils/Logger"
], function (dojo, declare, topic, DefaultActionHelper, Selection, Logger) {

    return declare([ DefaultActionHelper
    ], {

        runCustomAction : true,
        _args : null,
        _task : null,
        urlProperty : "URL",

        // the logger
        _daLogger : new Logger("ibm.usmi.widgets.util", "PropertyNotebookDAHelper"),

        /*
         * Method which check context for default server task and executes if needed
         * else executes the callback method
         */
        executeDefaultAction : function (/*Object*/ objectContext, /*Property*/ property) {

            this._args = {
                launchParms : "",
                menuItem : "",
                context : ""
            };

            dojo.mixin(this._args, objectContext);

            if(property){
                this.executePropertyAction(objectContext, property);
            }

        },

        executePropertyAction: function(/*Object*/ objectContext, /*Property*/ property){
            // summary:
            //        Method which checks context for default server task and executes if needed
            //        else executes the callback method
            // description:
            //        Coming soon
            // objectContext:
            //        Coming soon
            if (this._daLogger.isFinest()) {
                this._daLogger.entering("", "executePropertyAction");
            }
            var defaultTask = property.getTask();
            if (defaultTask) {

                if (defaultTask.getType() === "javascript") {
                    if (this._daLogger.isFinest()) {
                        this._daLogger.finest("default property task " + defaultTask.getId(),
                                "executePropertyAction");
                    }

                    var jsonLaunchParms = defaultTask.getLaunchJavascript(), launchParms = dojo.fromJson(jsonLaunchParms);

                    // selection is not set when coming from propterties page, need to get itemids
                    // out of object context and set on selection object
                    objectContext.selection = this.getSelection(objectContext._items);
                    launchParms.selection = objectContext.selection;

                    if (launchParms.topic) {
                        topic.publish(launchParms.topic, launchParms, defaultTask, objectContext);
                    }
                    else {
                        if (this._daLogger.isWarn()) {
                            this._daLogger.warn("default property task " + defaultTask.getId()+
                                    "did not specify topic in launch parms", "executePropertyAction");
                        }
                    }
                    return;
                }
            }
            if (this._daLogger.isFinest()) {
                this._daLogger.exiting("", "executePropertyAction");
            }
        },

        getDataSet : function () {
            return this._args.dataset;
        },

        getSelection : function (items) {
            var currentSelection = new Selection({dataset : this.getDataSet(), itemIds: [items[0].getId()]});
            return currentSelection;
        }
    });
});