/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

/*
setup.js is a holding place for global functions and variables. This file must
be included before dojo since it contains the logic to set up the dojo config object.
The workAreaDijit and selector are set in Dojo config based on URL parameters
*/

//this function prevents the default browser behavior of the ESCAPE key, exploiters can still listen for events for this key
function cancelEscapeKey (event) {
	if (event.keyCode == 27) {
		if (event.preventDefault) {
			event.preventDefault();
		}
	}
}

//Disable the default behavior of the Escape key to prevent an amq/event router exception that forces a logout
if (window.addEventListener) {
	window.addEventListener('keydown', cancelEscapeKey, false);
} else if (window.attachEvent) {
	window.attachEvent('onkeydown', cancelEscapeKey);
}
//TODO: in the future, refacter this into dojo classes except for the methods that need to run pre-dojo
//dialog launch references
var dialog;
var wizard;
var dialogCloseSubscription;

var messageHandlers = [];
var pageLoadAssembly = false; //to switch to old page-load design, turn this to true
var sessionId;
var popupBlockerWarningMessage;
var gInvalidUser = false;	//if this is an invalid user

//stores the properties that pertain to NGUI
var nguiProperties = {
	// stores what parts of the banner should be hidden, these values can be 
	// overriden by setting the parameter hide=banner,status,jobs,user,help,
	// productName,menu,finder
	//TODO: put into a NGUI object instead of into the dojo config object
	hideBanner : false,
	hideStatus : false,
	hideJobs : false,
	hideUser : false,
	hideHelp : false,
	hideProductName : false,
	hideMenuBar : false,
	productNameUnclickable : true,
	logoutEventProcessed : false
};

var dojoConfig = {
	async : false,
	parseOnLoad : false,
	isDebug : true,

	// This will be provided (optionally) by path= in the URL.
	// There can be any number of them.
	paths : {},

	// This will be provided by workAreaDijit= in the URL.
	// It will be used to fill the work area, i.e. the largest part of the screen.
	workAreaDijit : undefined,

	// This is for ISC session id, or XSS parameter.
	iscSession : undefined,

	// TODO: comment - selector is tree,list,none
	selector : undefined,

	trace : {
		"all" : "warn"
	},
	packages : [
		// If you are registering a package that has an identical name and location, you can just pass a string
		// instead, and it will configure it using that string for both the "name" and "location" properties. Handy!
		{
			name : 'ibm',
			location : '../ibm'
		},
		{
			name : 'oneui',
			location : '../../../../idx/oneui'
		},
		{
			name : 'fsm',
			location : '/com.ibm.itme.webconsole/scripts'
		}
	],
	locale : determineLanguage()

};
function determineLanguage () {
    /* Uncomment when they decide to support more than just English.
	var req = new XMLHttpRequest();
	req.open('GET', document.location, false);
	req.send(null);
	var responseLanguage = req.getResponseHeader("content-language").toLowerCase();
	return responseLanguage; */
	return "en-us";
}
/*
	register and clearMessageHandler are called internally by the message handlers
	when they are created and destroyed. They ensure proper message clean up when
	user navigates away from the page
*/
function registerMessageHandler (obj) {
	messageHandlers.push(obj);
}
function clearMessageHandlers () {
	var i = 0;
	for (; i < messageHandlers.length; i++) {
		messageHandlers[i].destroy();
	}
	messageHandlers.length = 0;
}
/*
 * Internal function. this function decodes the URL when user navigates from one page to the other
 * It saves all the parameters of the URL so that main.js can construct the proper widgets
 */
function processHash (changedHashParm) {
	var hideParameter;
	dojoConfig.selector = undefined;
	var changedHash = changedHashParm.replace(/%2F/g, '/');
	changedHash = changedHash.replace(/%3A/g, ':');
	if (pageLoadAssembly && changedHash.indexOf('#') !== -1) {
		changedHash = changedHash.split('#')[0];
	}
	var parms = changedHash.split("&");
	// now parms is an array of parameters on the URL
	//parses the hash parms and places them into global variables
	for ( var i = 0; i < parms.length; i++) {
		var keyVal = parms[i].split("=");
		if (keyVal.length === 2) {
			// we have a key and a value
			if (keyVal[0] === "path") {
				// It's of the form "key=pkg%3Apath"
				var pkgPath = keyVal[1].split("%3A");
				if ((pkgPath.length !== 2)) {
					pkgPath = keyVal[1].split(":");
				}
				if (pkgPath.length === 2) {
					dojoConfig.packages[3] = {
						name : pkgPath[0],
						location : pkgPath[1]
					};

				}
			} else if (keyVal[0] === "workAreaDijit") {
				dojoConfig.workAreaDijit = keyVal[1];
			} else if (keyVal[0] === "selector") {
				dojoConfig.selector = keyVal[1];
			} else if (keyVal[0] === "hide") {
				hideParameter = keyVal[1].split(",");
				for ( var j = 0; j < hideParameter.length; j++) {
					if (hideParameter[j] === 'banner') {
						nguiProperties.hideBanner = true;
					} else if (hideParameter[j] === 'status') {
						nguiProperties.hideStatus = true;
						nguiProperties.productNameUnclickable = true;
					} else if (hideParameter[j] === 'jobs') {
						nguiProperties.hideJobs = true;
						nguiProperties.productNameUnclickable = true;
					} else if (hideParameter[j] === 'user') {
						nguiProperties.hideUser = true;
						nguiProperties.productNameUnclickable = true;
					} else if (hideParameter[j] === 'help') {
						nguiProperties.hideHelp = true;
						nguiProperties.productNameUnclickable = true;
					} else if (hideParameter[j] === 'productName') {
						nguiProperties.hideProductName = true;
						nguiProperties.productNameUnclickable = true;
					} else if (hideParameter[j] === 'menuBar') {
						nguiProperties.hideMenuBar = true;
						nguiProperties.productNameUnclickable = true;
					}
				}
			}
		}
	}
	//if there was no workAreaDijit specified, take them to workArea+dashboard
	if (dojoConfig.workAreaDijit === undefined) {
		dojoConfig.workAreaDijit = "ibm/usmi/widgets/WorkArea";
		dojoConfig.selector = "ibm/usmi/dashboard/ResourceDashBoard";
	}
}
/*
 * Internal method. this gets called once when page loads or refreshes. 
 * User navigation invokes the processHash function 
 */
function processUrl () {
	// Parse the URL parameters.
	// It would be great to write the following code with Dojo,
	// but there's a chicken-and-egg problem. Have to do this work
	// before Dojo gets loaded, so can't use it. Any alternatives?
	var splitChar = "#";
	if (pageLoadAssembly) {
		splitChar = "?";
	}
	var splitUrl = document.location.href.split(splitChar);
	if (splitUrl.length > 1) {
		// get rid of any hash
		processHash(splitUrl[1]);
	} else {
		processHash(splitUrl[0]);
	}
}

processUrl();

function launchURL (/*String*/url) {
	var launchURLwindow = window.open(url);
	if (launchURLwindow === null || typeof (launchURLwindow) === 'undefined') {
		alert(popupBlockerWarningMessage);
	} else {
		launchURLwindow.focus();
	}
}

function launchURLSmallWindow (/*String*/url) {
	var launchURLwindow = window.open(url, '', 'width=600,height=800');
	if (launchURLwindow === null || typeof (launchURLwindow) === 'undefined') {
		alert(popupBlockerWarningMessage);
	} else {
		launchURLwindow.focus();
	}
}

/*
 * pass in the parameters of a NGen page and this method will launch the page.
 */
function launchNextGenPage (URLparms) {
	if (pageLoadAssembly) {
		document.location = "index.html?" + URLparms;
	} else {
		document.location = "index.html#" + URLparms;
	}
}

// this will be bound to the menu items inside a mega menu widget
function menuClicked (URLparms) {
	launchNextGenPage(URLparms);
}

function getMenuItemLink (selectorHome) {
	var rtnLink = "";
	// Compose the URL for navigator. It has two parts. One is for selector tree, and another is for the table.
	// selector=tree to indicate the tree should be displayed.
	// selectorHome specifies the root node
	// selectorTitle is for the selector title. 
	if (selectorHome === "GROUP:ALL_SYSTEMS") {
		rtnLink = "workAreaDijit=ibm/usmi/widgets/ResourceNavigator&dataset=GROUP:ALL_SYSTEMS";
	} else if (selectorHome === "GROUP:NO_ACCESS") {
		rtnLink = "workAreaDijit=ibm/usmi/widgets/ResourceNavigator&dataset=GROUP:NO_ACCESS";
	}
	return rtnLink;
}

function signout () {
	// log out ISC first, if succeeds, quit FSM
	logoutISC();
}

/* don't use, internal, cleans up dialogs that are shown via launchSchedulerDialog() */
function closeDialog () {
	dialog.hide();
	dialog.destroy();
	wizard.destroy();
	dialogCloseSubscription.remove();
}
/**
 * Launches the jobLaunchWizard widget. Common function that takes care of
 * closing/cleanup
 * 
 * @param dialogTitle -
 *            title string of the dialog
 * @param jobLaunchWizard -
 *            the JobLaunchWizard object you created that contains all the
 *            params.
 */
function launchSchedulerDialog (/*String*/dialogTitle, /*JobLaunchWizard object*/jobLaunchWizard) {
	var height = 0.8 * ((window.innerHeight !== undefined) ? window.innerHeight : document.body.offsetHeight);
    dialog = new dijit.Dialog({
            id : 'frameworkSchedulerDialog',
            title : dialogTitle,
            style : "overflow: auto; height: " + height + "px"
    });
    wizard = jobLaunchWizard;
    dialog.set("content", wizard);

    dialog.show();
    wizard.startup();
    wizard.resize({
            w : 700,
            h : 700
    });

    wizard.updateUI();
    dialog.layout();

    require([ "dojo/topic"
    ], function (topic) {
            dialogCloseSubscription = topic.subscribe("DefaultDialogDismissal", closeDialog);
    });
}
