require({cache:{
'ibm/usmi/menus/NextGenUIMenuBar':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
/**
 * NextGenUIMenuBar extends the menu bar from One UI to add support for creating
 * the menus dynamically from a collection. A menu bar is composed of multiple
 * popup menu bar items. An example of a popup menu bar item is the "Global"
 * menu. Each popup menu bar item then contains a menu dialog. The menu dialog
 * allows the popup menu bar item to contain not only contain a menu, but links
 * and search boxes as well. The menu inside a menu dialog can contain menu
 * items, menu headings, and a link area. The order that these items appear in
 * is determined by a weight attribute assigned to each item in the collection
 */

define("ibm/usmi/menus/NextGenUIMenuBar", [
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dijit/_WidgetBase",
	"dojo/query",
	"idx/oneui/MenuBar",
	"dijit/PopupMenuBarItem",
	"dijit/MenuItem",
	"idx/oneui/MenuDialog",
	"idx/oneui/Menu",
	"dijit/form/TextBox",
	"dijit/form/Button",
	"ibm/usmi/util/DataStore",
	"ibm/usmi/core/utils/_UsmiBrowserPersistence",
	"dojo/i18n!ibm/usmi/nls/frameworkResources",
	"ibm/usmi/menus/NextGenUIMenuUtils",
	"ibm/usmi/core/utils/Logger",
	"dojo/topic",
	"dijit/registry",
	"dojo/_base/array",
	"dojo/dom-class"
], function (dojo, declare, WidgetBase, query, MenuBar, PopupMenuBarItem, MenuItem, MenuDialog, Menu, TextBox, Button,
		DataStore, _UsmiBrowserPersistence, MRI, NextGenUIMenuUtils, Logger, topic, registry, array, domClass) {
	return declare([
		WidgetBase,
		MenuBar,
		_UsmiBrowserPersistence
	], {

		_menuBar : null,

		_dataStore : null,

		_PERSISTENCE_MENUBAR_ID : "ibm.usmi.menus.NextGenUIMenuBar.contents",

		_nextGenUIMenuUtils : new NextGenUIMenuUtils({}),

		_nextGenUIMenuBarLogger : new Logger("ibm.usmi.menus", "NextGenUIMenuBar"),

		constructor : function (args) {
			dojo.safeMixin(this, args);

		},

		postCreate : function () {
			this.inherited(arguments);
			this._menuBar = this.getData(this._PERSISTENCE_MENUBAR_ID, true);
			if (this._nextGenUIMenuBarLogger.isInfo()) {
				this._nextGenUIMenuBarLogger.info("Checked session storage for menu bar, value is: " + this._menuBar,
						"postCreate");
			}
			if (this._menuBar) {
				this._createMenuBar();
			} else {
				if (this._nextGenUIMenuBarLogger.isInfo()) {
					this._nextGenUIMenuBarLogger.info(
							"Menu bar hasn't been stored yet, need to pull contents from server.", "postCreate");
				}
				this._dataStore = new DataStore({
					datasetId : 'HeaderMegaMenuCollection',
					homeParms : null,
					getItems : true,
					retrieveItemProperties : true,
					getTasks : false,
					initReadyCallback : dojo.hitch(this, this.initializeDataStores)
				});
			}
		},

		initializeDataStores : function () {
			var oneUIHeader=null;
			if (this._nextGenUIMenuBarLogger.isInfo()) {
				this._nextGenUIMenuBarLogger.info(
						"Menu bar data retrieved from server. Converting CURI data into the menu object",
						"initializeDataStores");
			}

			this._convertMenuCollectionToMenuObject();
			if (this._nextGenUIMenuBarLogger.isInfo()) {
				this._nextGenUIMenuBarLogger.info("Saving menu data to the session", "initializeDataStores");
			}
			this.setData(this._PERSISTENCE_MENUBAR_ID, this._menuBar, true);
			this._createMenuBar();
			oneUIHeader=registry.byId("oneUIHeader");
			if(oneUIHeader && oneUIHeader.secondaryNavigationInHeadShowed){
				//if OneUIHeader is presented and it has attempted to show the secondaryNavigation, 
				//we need to show the secondaryNavigation again to make sure all children of secondaryNavigation 
				//would be showed properly in OneUIHeader
				oneUIHeader._showSecondaryNavigationInHead();
				if (this._nextGenUIMenuBarLogger.isInfo()) {
					this._nextGenUIMenuBarLogger.info(
							"Rendering secondaryNavigation with OneUIHeader stuff",
							"initializeDataStores");
				}
			}
		},

		destroy : function () {
			// clean up the datastore
			if (this._dataStore) {
				this._dataStore.getDataset().removeEventListener(this);
				this._dataStore.destroy();
				this._dataStore = null;
			}
			
			this.inherited(arguments);
		},
        
		/**
		 * gets a menu from the menu bar object using its menu ID, Returns null
		 * if that menu doesn't exist in the menu bar
		 */
		_getMenuFromMenuBar : function (
		/*String*/menuId) {
			var i;
			for (i = 0; i < this._menuBar.menus.length; i++) {
				if (this._menuBar.menus[i].id === menuId) {
					return this._menuBar.menus[i];
				}
			}
			return null;

		},

		/**
		 * Finds and returns the link menu item from a given menu. Returns null
		 * if the menu does not contain a link menu item
		 */
		_getMenuLinksFromMenu : function (
		/*Array*/menu) {
			var i;
			for (i = 0; i < menu.items.length; i++) {
				if (menu.items[i].type === 'link') {
					return menu.items[i];
				}
			}
			return null;
		},

		/**
		 * converts a collection into a menu bar object for easy storage in
		 * session persistence
		 */
		_convertMenuCollectionToMenuObject : function () {
			if (this._nextGenUIMenuBarLogger.isFinest()) {
				this._nextGenUIMenuBarLogger.entering("", "_convertMenuCollectionToMenuObject");
			}
			var itemType, menuId, title, menu, items, sortedItems, links, i;

			items = this._dataStore.getItems();
			//sortedItems = new Array(items.length);
			sortedItems = [];
			for (i = 0; i < items.length; i++) {
				sortedItems[items[i].getProperty('ORDER_COLUMN').getValue()] = items[i];
			}

			this._menuBar = {
				"menus" : []
			};

			/*
				First create the menus themselves reading from the collection. 
				Once the menus are created, then read through the
				collection to populate them. 
			*/
			dojo.forEach(sortedItems, dojo.hitch(this, function (entry, i) {
				itemType = entry.getProperty('TYPE_COLUMN').getValue();
				menuId = entry.getProperty('MENU_ID_COLUMN').getValue();
				title = entry.getProperty('TITLE_COLUMN').getValue();

				if (itemType === 'menuBar') {
					// the help menu is currently it's own menu so don't include it here
					if (menuId !== 'HelpMenu') {

						this._menuBar.menus.push({
							"type" : itemType,
							"title" : title,
							"id" : menuId,
							"items" : []
						});
					}

				}
			}));

			/* populate the menus*/
			dojo.forEach(sortedItems, dojo.hitch(this, function (entry, i) {
				itemType = entry.getProperty('TYPE_COLUMN').getValue();
				menuId = entry.getProperty('MENU_ID_COLUMN').getValue();
				if (menuId !== 'HelpMenu') {
					menu = this._getMenuFromMenuBar(menuId);
					if (itemType === 'menuItem') {
						var oneitem = {
							"type" : itemType,
							"column" : entry.getProperty('COLUMN_COLUMN').getValue(),
							"title" : entry.getProperty('TITLE_COLUMN').getValue(),
							"task" : entry.getProperty('TASK_COLUMN').getValue(),
							"id" : menuId
						};
						menu.items.push(oneitem);
						this._increaseCount(menu.items, oneitem.column);
					} else if (itemType === 'menuHeading') {
						menu.items.push({
							"type" : itemType,
							"column" : entry.getProperty('COLUMN_COLUMN').getValue(),
							"title" : entry.getProperty('TITLE_COLUMN').getValue(),
							"id" : menuId,
							"count" : 0
						});
					} else if (itemType === 'menuHeadingImage') {
						menu.items.push({
							"type" : itemType,
							"column" : entry.getProperty('COLUMN_COLUMN').getValue(),
							"image" : entry.getProperty('IMAGE_COLUMN').getValue(),
							"id" : menuId
						});
					} else if (itemType === 'link') {
						links = this._getMenuLinksFromMenu(menu);
						if (!links) {
							links = {
								"type" : itemType,
								"id" : menuId,
								"links" : []
							};
							menu.items.push(links);
						}
						links.links.push({
							"id" : menuId,
							"title" : entry.getProperty('TITLE_COLUMN').getValue(),
							"task" : entry.getProperty('TASK_COLUMN').getValue()
						});

					}
				}
			}));
			
			// Go through menus to delete the empty columns
			dojo.forEach(this._menuBar.menus, dojo.hitch(this, function (menulist, i) {
				var copyMenus = new Array();
				var rowDeleted = -1;	// Which column is current empty
				var rowDeletedNum = 0;	// How many empty columns
				// Go inside each menuid
				dojo.forEach(menulist.items, dojo.hitch(this, function (entry, i) {
					var skipped = false;
					if (entry.type === "menuHeading") {
						if (entry.count === 0) {
							// Mark the empty column
							rowDeleted = parseInt(entry.column);
							rowDeletedNum += 1;
							skipped = true;
						}
					} 
					if (entry.column) {
						if (rowDeletedNum > 0) {
							if (parseInt(entry.column)>rowDeleted) {
								// Move the following columns
								entry.column = (parseInt(entry.column)-rowDeletedNum)+"";
							}
						}
					}
					if (!skipped) {					
						copyMenus.push(entry);					
					}
				}));
				// Copy back the processed items
				menulist.items = copyMenus;
			}));			
			
			if (this._nextGenUIMenuBarLogger.isFinest()) {
				this._nextGenUIMenuBarLogger.exiting("", "_convertMenuCollectionToMenuObject");
			}
		},
		
		/**
		 * Count items under one menu heading
		 */
		_increaseCount : function (items, column) {
			dojo.forEach(items, dojo.hitch(this, function (entry, i) {				
				if (entry.column === column && entry.type === "menuHeading") {
					entry.count += 1;
				}				
			}));
		},		

		/**
		 * creates the menu headings from a menu bar object that's created from
		 * a collection, creates a new popup menu bar item, menu dialog, and
		 * submenu for each one.
		 */
		_createMenuBar : function () {
			if (this._nextGenUIMenuBarLogger.isFinest()) {
				this._nextGenUIMenuBarLogger.entering("Creating a menu bar with " + this._menuBar.menus.length
						+ " menus", "_createMenuBar");
			}
			// the popup menu being parsed
			var popup,
			// the menu dialog that will contain all the popup menus
			menuDialog, menuTitle, i;

			// only parse the menu headings if the menu bar data has been set
			if (this._menuBar) {
				this.menuHeadersParsed = true;

				//now that we've stored the menu headers in the menu bar object, create the actual menus
				//dojo.forEach(this._menuBar.menus, dojo.hitch(this, function (entry, i) {
				for (i = 0; i < this._menuBar.menus.length; i++) {
					menuTitle = this._menuBar.menus[i].title;
					if (this._nextGenUIMenuBarLogger.isFinest()) {
						this._nextGenUIMenuBarLogger.finest("Creating menu: " + menuTitle, "_createMenuBar");
					}
					popup = new PopupMenuBarItem({});
					menuDialog = new MenuDialog();
					popup.set("label", MRI[menuTitle]);
					menuDialog = new MenuDialog({});
					popup.set("popup", menuDialog);
					//explicitly set an ID so that we can find the widget later to assign
					//it its submenu
					popup.set("id", this._menuBar.menus[i].id);
					this.addChild(popup);
				}
			}

			if (this._nextGenUIMenuBarLogger.isInfo()) {
				this._nextGenUIMenuBarLogger.info("Publishing topic that the menu bar is loaded", "_createMenuBar");
			}

			topic.publish("nextgen-ui-menubar-loaded", null);

			if (this._nextGenUIMenuBarLogger.isFinest()) {
				this._nextGenUIMenuBarLogger.exiting("", "_createMenuBar");
			}
		},

		/**
		 * creates the contents of a popupmenu using the menu bar object
		 * constructed using the collection. In order the parse the menu
		 * contents, the menu headers must be parsed first. If the
		 * parseMenuHeadings() function hasn't been called, then it gets called
		 * in this function
		 */
		populateMenuBarMenus : function () {
			if (this._nextGenUIMenuBarLogger.isFinest()) {
				this._nextGenUIMenuBarLogger.entering("", "_populateMenuBarMenus");
			}
			//the menu being populated
			var menu,
			//the widget ID of the menu being populated
			menuId, i, j,
			//the menu items in a menu
			items,
			//stores the contents of a menu while parsing through a menu, 
			//used to set the contents of the menu dialog for a menu
			nodeList,
			//reference to the popup mwnu widget in the menu bar
			popup,
			//true if the menu, where the menu items are contained, has been included in the nodelist
			menuIncluded = false;

			if (this._menuBar) {
				for (i = 0; i < this._menuBar.menus.length; i++) {
					items = this._menuBar.menus[i].items;
					menuId = this._menuBar.menus[i].id;
					//each popup menu contains a submenu, so create it right away
					menu = new Menu({
						"class" : "menu-workitem"
					});
					nodeList = new query.NodeList();
					var headings = [];
					for (j = 0; j < items.length; j++) {
						if (items[j].type === 'menuItem') {
							headings[items[j].column] = true;
						}
					}
					for (j = 0; j < items.length; j++) {
						//check to see if the item type corresponds to an item contained in the menu. If it does
						//push the menu onto the nodelist if it hasn't been pushed on already
						if (items[j].type === 'menuItem' || items[j].type === 'menuHeading'
								|| items[j].type === 'menuHeadingImage') {
							if (!menuIncluded) {
								menuIncluded = true;
								nodeList.push(menu.domNode);
							}
						}
						if (items[j].type === 'menuItem') {
							menu.addChild((this._nextGenUIMenuUtils.createMenuItem(items[j].title, items[j].column,
									items[j].task, MRI)));
						} else if (items[j].type === 'menuHeading') {
							if (headings[items[j].column]) {
								menu.addChild(this._nextGenUIMenuUtils.createMenuHeading(items[j].title,
										items[j].column, MRI));
							}
						} else if (items[j].type === 'menuHeadingImage') {
							menu.addChild(this._nextGenUIMenuUtils.createMenuHeadingImage(items[j].image,
									items[j].column));
						} else if (items[j].type === 'link') {
							nodeList.push(this._nextGenUIMenuUtils.createLinks(items[j].links, MRI));
						}
					}

					popup = this._nextGenUIMenuUtils.getWidgetFromRegistry(menuId);
					if (popup) {
						popup.popup.set("content", nodeList);
					} else {
						if (this._nextGenUIMenuBarLogger.isWarn()) {
							this._nextGenUIMenuBarLogger.warn(
									"Popup widget doesn't exist in the registry. Unable to add menu to popup a menu.",
									"_populateMenuBarMenus");
						}
					}

					menuIncluded = false;
				}
			} else {
				if (this._nextGenUIMenuBarLogger.isWarn()) {
					this._nextGenUIMenuBarLogger.warn("The menu bar object hasn't been created yet.",
							"_populateMenuBarMenus");
				}
			}
			if (this._nextGenUIMenuBarLogger.isFinest()) {
				this._nextGenUIMenuBarLogger.exiting("", "_populateMenuBarMenus");
			}
		},
		
		
		onItemHover: function(/*MenuItem*/ item){
			var childMenuItems;
			this.inherited(arguments);
			childMenuItems=this.getChildren();
			//hack the defect XB126110
			array.forEach(childMenuItems, function(menuBarItem){
				if(menuBarItem === item){
					return;
				}
				if(domClass.contains(menuBarItem.domNode, "dijitMenuItemSelected")){
					domClass.remove(menuBarItem.domNode, "dijitMenuItemSelected");
					//console.log("remove the selected css class");
				}
				
			});
			
		}

	});

});

},
'gridx/modules/exporter/CSV':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"../../core/_Module",
	"./Exporter"
], function(declare, lang, _Module){

/*=====
	dojo.declare('__CSVExportArgs', __ExportArgs, {
		//seperator: String?
		//		The seperator string used in CSV. Default to comma ','.
		//newLine: String?
		//		The new line string used in CSV. Deault to '\r\n';
	});

	var __ExportContext = function(){
		//columnIds: String[]
		//		Available for header.
		//columnId: String
		//		Available for header cell or a body cell.
		//rowIds: String[]
		//		Available for a progress
		//rowId: String
		//		Available for a row or a body cell.
		//data: Anything
		//		Available for a body cell
	}
=====*/

	return declare(/*===== "gridx.modules.exporter.CSV", =====*/_Module, {
		// summary:
		//		This module provides the API to export grid contents to CSV format string

		name: 'exportCsv',

		forced: ['exporter'],

		getAPIPath: function(){
			return {
				exporter: {
					toCSV: lang.hitch(this, this.toCSV) 
				}
			};
		},
	
		//Public ---------------------------------------------------------------------
		toCSV: function(/* __CSVExportArgs? */ args){
			// summary:
			//		Export the grid contents to CSV according to the given args.
			//		This method should be called through grid.exporter.toCSV();
			// args: __CSVExportArgs?
			//		The args to configure the export result and the export process.
			// returns:
			//		A deferred object indicating when the export process is completed,
			//		and then pass the exported CSV string to callbacks.
			return this.grid.exporter._export(this, args || {});	//dojo.Deferred
		},

		//Package --------------------------------------------------------------------
		initialize: function(/* __CSVExportArgs */ args){
			// tags:
			//		private
			this._s = args.separator || ",";
			this._n = args.newLine || "\r\n";
			this._lines = [];
		},

		beforeHeader: function(){
			// tags:
			//		private
			this._cells = [];
		},

		handleHeaderCell: function(/* __ExportContext */ context){
			// tags:
			//		private
			this._cells.push(context.column.name());
		},

		afterHeader: function(){
			// tags:
			//		private
			this._lines.push(this._cells.join(this._s));
		},

		beforeRow: function(){
			// tags:
			//		private
			this._cells = [];
		},

		handleCell: function(/* __ExportContext */ context){
			// tags:
			//		private
			var data = String(context.data).replace(/"/g, '""');
			if(data.indexOf(this._s) >= 0 || data.search(/[" \t\r\n]/) >= 0){
				data = '"' + data + '"';
			}
			this._cells.push(data);
		},

		afterRow: function(){
			// tags:
			//		private
			this._lines.push(this._cells.join(this._s));
		},

		getResult: function(){
			// tags:
			//		private
			return this._lines.join(this._n);
		}
	});
});

},
'ibm/tivoli/tip/navwidget/table/dijit/RelationshipsTableView':function(){
// wrapped by build app
define("ibm/tivoli/tip/navwidget/table/dijit/RelationshipsTableView", ["dijit","dojo","dojox","dojo/require!dojo/data/ItemFileReadStore,ibm/tivoli/navmodel/DatasetRelationshipsStore,ibm/tivoli/tip/navwidget/table/dijit/AbstractTableView,ibm/tivoli/tip/navwidget/table/dijit/NavModelTable"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide ("ibm.tivoli.tip.navwidget.table.dijit.RelationshipsTableView");

dojo.require("dojo.data.ItemFileReadStore");   // 49426  Gridx
dojo.require("ibm.tivoli.navmodel.DatasetRelationshipsStore");
dojo.require ("ibm.tivoli.tip.navwidget.table.dijit.AbstractTableView");
dojo.require("ibm.tivoli.tip.navwidget.table.dijit.NavModelTable");
ibm.tivoli.tip.navwidget.table.dijit.RelationshipsTableView._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.tip.navwidget.table.dijit.RelationshipsTableView");

dojo.declare ("ibm.tivoli.tip.navwidget.table.dijit.RelationshipsTableView",
	ibm.tivoli.tip.navwidget.table.dijit.AbstractTableView,
	{		
	// summary:
	//		Coming soon
	// description:
	//		Coming soon
	
	//***************************
	// "Private"
	//***************************
	
	// _datasetRelationshipsStore: [private] Object
	//		
//	_datasetRelationshipsStore: null,
	
	// _trace: [private] Trace
	//		Utility for trace and logging
	_trace: ibm.tivoli.tip.navwidget.table.dijit.RelationshipsTableView._trace,
	
	constructor: function () {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		//this._global = ibm.tivoli.tip.dojo.Global.setSingleton( "234", " ");
		this.title = this.commonResources.RELATIONSHIP_VIEW;
		this.labelIconClass = "relationshipViewIcon";
	//	this._pTableDef = "DefaultRelationshipsTable";
	    this._pDEFAULT_WIDGET_DEF_INFO = {
				className: "ibm.tivoli.navmodel.TableWidgetDef",
				baseUrl: "/ibm/tivoli/rest",
		    	provider: "any",
		    	datasource: "any",
		    	widget: "table",
		    	definition: "DefaultRelationshipsTable"
			},
		this._pTableType = "relationship";
		this._pTableClassName = "ibm.tivoli.tip.navwidget.table.dijit.NavModelTable";
		this._pDefaultColumns = [
				{id: "targetItem", label: this.commonResources.TARGETITEM, type: "string"},
				{id: "label", label: this.commonResources.TYPE, type: "string"},
				{id: "sourceItem", label:this.commonResources.SOURCEITEM, type: "string", sortOrder: 0} ];
	},

	isApplicable: function(/*ibm.tivoli.navmodel.Dataset*/dataset){
		// Primarily interested in the following view types:
        //	return viewType == "graph" || viewType == "tree" || viewType == "tree:reverse" || viewType == "table_relationship";
        return this.inherited(arguments);
	},

	
	_createTableStore: function(columnIds){		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// columnIds:
		//		
		
		// create or update store

		if (!this._dataStore) {    // 49426   Gridx (split out to always return some store)
            if (this.dataset) {	
            	this._dataStore = new ibm.tivoli.navmodel.DatasetRelationshipsStore({dataset:this.dataset, defaultProperties:columnIds, returnItem:true});			
            }
            else {
				//No dataset, so just create an empty store
				var emptyStore = 	new dojo.data.ItemFileReadStore({
										data: { items: []}
	                				});
				this._dataStore = emptyStore;
            }
		   this.inherited(arguments);
		}
	},	
	
	
	/////////////////////////////////////////////////
	// Selection change handling
	/////////////////////////////////////////////////
	_handlingSelectOrDeselectAll: false,
	_handlingSelectionChangedCallback: false,
	_gridSelectionChangedCallback: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		this._handlingSelectionChangedCallback = true;
		
		if (!this.selection.isChanging()&& ! this._handlingSelectOrDeselectAll) {
			//Get the grid's selection... will be an array of Client Dataset Relationships
			var selectedRows = this._grid.selection.getSelected(); 

			//Set the selected relationships on the view's selection object
			this.selection.setSelectedObjects(null, null, selectedRows, null);
		}
		
		this._handlingSelectionChangedCallback = false;
	}
});

});

},
'dijit/Tooltip':function(){
define([
	"dojo/_base/array", // array.forEach array.indexOf array.map
	"dojo/_base/declare", // declare
	"dojo/_base/fx", // fx.fadeIn fx.fadeOut
	"dojo/dom", // dom.byId
	"dojo/dom-class", // domClass.add
	"dojo/dom-geometry", // domGeometry.getMarginBox domGeometry.position
	"dojo/dom-style", // domStyle.set, domStyle.get
	"dojo/_base/lang", // lang.hitch lang.isArrayLike
	"dojo/_base/sniff", // has("ie")
	"dojo/_base/window", // win.body
	"./_base/manager",	// manager.defaultDuration
	"./place",
	"./_Widget",
	"./_TemplatedMixin",
	"./BackgroundIframe",
	"dojo/text!./templates/Tooltip.html",
	"."		// sets dijit.showTooltip etc. for back-compat
], function(array, declare, fx, dom, domClass, domGeometry, domStyle, lang, has, win,
			manager, place, _Widget, _TemplatedMixin, BackgroundIframe, template, dijit){

/*=====
	var _Widget = dijit._Widget;
	var BackgroundIframe = dijit.BackgroundIframe;
	var _TemplatedMixin = dijit._TemplatedMixin;
=====*/

	// module:
	//		dijit/Tooltip
	// summary:
	//		Defines dijit.Tooltip widget (to display a tooltip), showTooltip()/hideTooltip(), and _MasterTooltip


	var MasterTooltip = declare("dijit._MasterTooltip", [_Widget, _TemplatedMixin], {
		// summary:
		//		Internal widget that holds the actual tooltip markup,
		//		which occurs once per page.
		//		Called by Tooltip widgets which are just containers to hold
		//		the markup
		// tags:
		//		protected

		// duration: Integer
		//		Milliseconds to fade in/fade out
		duration: manager.defaultDuration,

		templateString: template,

		postCreate: function(){
			win.body().appendChild(this.domNode);

			this.bgIframe = new BackgroundIframe(this.domNode);

			// Setup fade-in and fade-out functions.
			this.fadeIn = fx.fadeIn({ node: this.domNode, duration: this.duration, onEnd: lang.hitch(this, "_onShow") });
			this.fadeOut = fx.fadeOut({ node: this.domNode, duration: this.duration, onEnd: lang.hitch(this, "_onHide") });
		},

		show: function(innerHTML, aroundNode, position, rtl, textDir){
			// summary:
			//		Display tooltip w/specified contents to right of specified node
			//		(To left if there's no space on the right, or if rtl == true)
			// innerHTML: String
			//		Contents of the tooltip
			// aroundNode: DomNode || dijit.__Rectangle
			//		Specifies that tooltip should be next to this node / area
			// position: String[]?
			//		List of positions to try to position tooltip (ex: ["right", "above"])
			// rtl: Boolean?
			//		Corresponds to `WidgetBase.dir` attribute, where false means "ltr" and true
			//		means "rtl"; specifies GUI direction, not text direction.
			// textDir: String?
			//		Corresponds to `WidgetBase.textdir` attribute; specifies direction of text.


			if(this.aroundNode && this.aroundNode === aroundNode && this.containerNode.innerHTML == innerHTML){
				return;
			}

			// reset width; it may have been set by orient() on a previous tooltip show()
			this.domNode.width = "auto";

			if(this.fadeOut.status() == "playing"){
				// previous tooltip is being hidden; wait until the hide completes then show new one
				this._onDeck=arguments;
				return;
			}
			this.containerNode.innerHTML=innerHTML;
			
			this.set("textDir", textDir);
			this.containerNode.align = rtl? "right" : "left"; //fix the text alignment

			var pos = place.around(this.domNode, aroundNode,
				position && position.length ? position : Tooltip.defaultPosition, !rtl, lang.hitch(this, "orient"));

			// Position the tooltip connector for middle alignment.
			// This could not have been done in orient() since the tooltip wasn't positioned at that time.
			var aroundNodeCoords = pos.aroundNodePos;
			if(pos.corner.charAt(0) == 'M' && pos.aroundCorner.charAt(0) == 'M'){
				this.connectorNode.style.top = aroundNodeCoords.y + ((aroundNodeCoords.h - this.connectorNode.offsetHeight) >> 1) - pos.y + "px";
				this.connectorNode.style.left = "";
			}else if(pos.corner.charAt(1) == 'M' && pos.aroundCorner.charAt(1) == 'M'){
				this.connectorNode.style.left = aroundNodeCoords.x + ((aroundNodeCoords.w - this.connectorNode.offsetWidth) >> 1) - pos.x + "px";
			}

			// show it
			domStyle.set(this.domNode, "opacity", 0);
			this.fadeIn.play();
			this.isShowingNow = true;
			this.aroundNode = aroundNode;
		},

		orient: function(/*DomNode*/ node, /*String*/ aroundCorner, /*String*/ tooltipCorner, /*Object*/ spaceAvailable, /*Object*/ aroundNodeCoords){
			// summary:
			//		Private function to set CSS for tooltip node based on which position it's in.
			//		This is called by the dijit popup code.   It will also reduce the tooltip's
			//		width to whatever width is available
			// tags:
			//		protected
			this.connectorNode.style.top = ""; //reset to default

			//Adjust the spaceAvailable width, without changing the spaceAvailable object
			var tooltipSpaceAvaliableWidth = spaceAvailable.w - this.connectorNode.offsetWidth;

			node.className = "dijitTooltip " +
				{
					"MR-ML": "dijitTooltipRight",
					"ML-MR": "dijitTooltipLeft",
					"TM-BM": "dijitTooltipAbove",
					"BM-TM": "dijitTooltipBelow",
					"BL-TL": "dijitTooltipBelow dijitTooltipABLeft",
					"TL-BL": "dijitTooltipAbove dijitTooltipABLeft",
					"BR-TR": "dijitTooltipBelow dijitTooltipABRight",
					"TR-BR": "dijitTooltipAbove dijitTooltipABRight",
					"BR-BL": "dijitTooltipRight",
					"BL-BR": "dijitTooltipLeft"
				}[aroundCorner + "-" + tooltipCorner];

			// reduce tooltip's width to the amount of width available, so that it doesn't overflow screen
			this.domNode.style.width = "auto";
			var size = domGeometry.getContentBox(this.domNode);

			var width = Math.min((Math.max(tooltipSpaceAvaliableWidth,1)), size.w);
			var widthWasReduced = width < size.w;

			this.domNode.style.width = width+"px";

			//Adjust width for tooltips that have a really long word or a nowrap setting
			if(widthWasReduced){
				this.containerNode.style.overflow = "auto"; //temp change to overflow to detect if our tooltip needs to be wider to support the content
				var scrollWidth = this.containerNode.scrollWidth;
				this.containerNode.style.overflow = "visible"; //change it back
				if(scrollWidth > width){
					scrollWidth = scrollWidth + domStyle.get(this.domNode,"paddingLeft") + domStyle.get(this.domNode,"paddingRight");
					this.domNode.style.width = scrollWidth + "px";
				}
			}

			// Reposition the tooltip connector.
			if(tooltipCorner.charAt(0) == 'B' && aroundCorner.charAt(0) == 'B'){
				var mb = domGeometry.getMarginBox(node);
				var tooltipConnectorHeight = this.connectorNode.offsetHeight;
				if(mb.h > spaceAvailable.h){
					// The tooltip starts at the top of the page and will extend past the aroundNode
					var aroundNodePlacement = spaceAvailable.h - ((aroundNodeCoords.h + tooltipConnectorHeight) >> 1);
					this.connectorNode.style.top = aroundNodePlacement + "px";
					this.connectorNode.style.bottom = "";
				}else{
					// Align center of connector with center of aroundNode, except don't let bottom
					// of connector extend below bottom of tooltip content, or top of connector
					// extend past top of tooltip content
					this.connectorNode.style.bottom = Math.min(
						Math.max(aroundNodeCoords.h/2 - tooltipConnectorHeight/2, 0),
						mb.h - tooltipConnectorHeight) + "px";
					this.connectorNode.style.top = "";
				}
			}else{
				// reset the tooltip back to the defaults
				this.connectorNode.style.top = "";
				this.connectorNode.style.bottom = "";
			}

			return Math.max(0, size.w - tooltipSpaceAvaliableWidth);
		},

		_onShow: function(){
			// summary:
			//		Called at end of fade-in operation
			// tags:
			//		protected
			if(has("ie")){
				// the arrow won't show up on a node w/an opacity filter
				this.domNode.style.filter="";
			}
		},

		hide: function(aroundNode){
			// summary:
			//		Hide the tooltip

			if(this._onDeck && this._onDeck[1] == aroundNode){
				// this hide request is for a show() that hasn't even started yet;
				// just cancel the pending show()
				this._onDeck=null;
			}else if(this.aroundNode === aroundNode){
				// this hide request is for the currently displayed tooltip
				this.fadeIn.stop();
				this.isShowingNow = false;
				this.aroundNode = null;
				this.fadeOut.play();
			}else{
				// just ignore the call, it's for a tooltip that has already been erased
			}
		},

		_onHide: function(){
			// summary:
			//		Called at end of fade-out operation
			// tags:
			//		protected

			this.domNode.style.cssText="";	// to position offscreen again
			this.containerNode.innerHTML="";
			if(this._onDeck){
				// a show request has been queued up; do it now
				this.show.apply(this, this._onDeck);
				this._onDeck=null;
			}
		},
		
		_setAutoTextDir: function(/*Object*/node){
		    // summary:
		    //	    Resolve "auto" text direction for children nodes
		    // tags:
		    //		private

            this.applyTextDir(node, has("ie") ? node.outerText : node.textContent);
            array.forEach(node.children, function(child){this._setAutoTextDir(child); }, this);
		},
		
		_setTextDirAttr: function(/*String*/ textDir){
		    // summary:
		    //		Setter for textDir.
		    // description:
		    //		Users shouldn't call this function; they should be calling
		    //		set('textDir', value)
		    // tags:
		    //		private
	
            this._set("textDir", typeof textDir != 'undefined'? textDir : "");
    	    if (textDir == "auto"){
    	        this._setAutoTextDir(this.containerNode);
    	    }else{
    	        this.containerNode.dir = this.textDir;
    	    }  		             		        
        }
	});

	dijit.showTooltip = function(innerHTML, aroundNode, position, rtl, textDir){
		// summary:
		//		Static method to display tooltip w/specified contents in specified position.
		//		See description of dijit.Tooltip.defaultPosition for details on position parameter.
		//		If position is not specified then dijit.Tooltip.defaultPosition is used.
		// innerHTML: String
		//		Contents of the tooltip
		// aroundNode: dijit.__Rectangle
		//		Specifies that tooltip should be next to this node / area
		// position: String[]?
		//		List of positions to try to position tooltip (ex: ["right", "above"])
		// rtl: Boolean?
		//		Corresponds to `WidgetBase.dir` attribute, where false means "ltr" and true
		//		means "rtl"; specifies GUI direction, not text direction.
		// textDir: String?
		//		Corresponds to `WidgetBase.textdir` attribute; specifies direction of text.

		// after/before don't work, but they used to, so for back-compat convert them to after-centered, before-centered
		if(position){
			position = array.map(position, function(val){
				return {after: "after-centered", before: "before-centered"}[val] || val;
			});
		}

		if(!Tooltip._masterTT){ dijit._masterTT = Tooltip._masterTT = new MasterTooltip(); }
		return Tooltip._masterTT.show(innerHTML, aroundNode, position, rtl, textDir);
	};

	dijit.hideTooltip = function(aroundNode){
		// summary:
		//		Static method to hide the tooltip displayed via showTooltip()
		return Tooltip._masterTT && Tooltip._masterTT.hide(aroundNode);
	};

	var Tooltip = declare("dijit.Tooltip", _Widget, {
		// summary:
		//		Pops up a tooltip (a help message) when you hover over a node.

		// label: String
		//		Text to display in the tooltip.
		//		Specified as innerHTML when creating the widget from markup.
		label: "",

		// showDelay: Integer
		//		Number of milliseconds to wait after hovering over/focusing on the object, before
		//		the tooltip is displayed.
		showDelay: 400,

		// connectId: String|String[]
		//		Id of domNode(s) to attach the tooltip to.
		//		When user hovers over specified dom node, the tooltip will appear.
		connectId: [],

		// position: String[]
		//		See description of `dijit.Tooltip.defaultPosition` for details on position parameter.
		position: [],

		_setConnectIdAttr: function(/*String|String[]*/ newId){
			// summary:
			//		Connect to specified node(s)

			// Remove connections to old nodes (if there are any)
			array.forEach(this._connections || [], function(nested){
				array.forEach(nested, lang.hitch(this, "disconnect"));
			}, this);

			// Make array of id's to connect to, excluding entries for nodes that don't exist yet, see startup()
			this._connectIds = array.filter(lang.isArrayLike(newId) ? newId : (newId ? [newId] : []),
					function(id){ return dom.byId(id); });

			// Make connections
			this._connections = array.map(this._connectIds, function(id){
				var node = dom.byId(id);
				return [
					this.connect(node, "onmouseenter", "_onHover"),
					this.connect(node, "onmouseleave", "_onUnHover"),
					this.connect(node, "onfocus", "_onHover"),
					this.connect(node, "onblur", "_onUnHover")
				];
			}, this);

			this._set("connectId", newId);
		},

		addTarget: function(/*DOMNODE || String*/ node){
			// summary:
			//		Attach tooltip to specified node if it's not already connected

			// TODO: remove in 2.0 and just use set("connectId", ...) interface

			var id = node.id || node;
			if(array.indexOf(this._connectIds, id) == -1){
				this.set("connectId", this._connectIds.concat(id));
			}
		},

		removeTarget: function(/*DomNode || String*/ node){
			// summary:
			//		Detach tooltip from specified node

			// TODO: remove in 2.0 and just use set("connectId", ...) interface

			var id = node.id || node,	// map from DOMNode back to plain id string
				idx = array.indexOf(this._connectIds, id);
			if(idx >= 0){
				// remove id (modifies original this._connectIds but that's OK in this case)
				this._connectIds.splice(idx, 1);
				this.set("connectId", this._connectIds);
			}
		},

		buildRendering: function(){
			this.inherited(arguments);
			domClass.add(this.domNode,"dijitTooltipData");
		},

		startup: function(){
			this.inherited(arguments);

			// If this tooltip was created in a template, or for some other reason the specified connectId[s]
			// didn't exist during the widget's initialization, then connect now.
			var ids = this.connectId;
			array.forEach(lang.isArrayLike(ids) ? ids : [ids], this.addTarget, this);
		},

		_onHover: function(/*Event*/ e){
			// summary:
			//		Despite the name of this method, it actually handles both hover and focus
			//		events on the target node, setting a timer to show the tooltip.
			// tags:
			//		private
			if(!this._showTimer){
				var target = e.target;
				this._showTimer = setTimeout(lang.hitch(this, function(){this.open(target)}), this.showDelay);
			}
		},

		_onUnHover: function(/*Event*/ /*===== e =====*/){
			// summary:
			//		Despite the name of this method, it actually handles both mouseleave and blur
			//		events on the target node, hiding the tooltip.
			// tags:
			//		private

			// keep a tooltip open if the associated element still has focus (even though the
			// mouse moved away)
			if(this._focus){ return; }

			if(this._showTimer){
				clearTimeout(this._showTimer);
				delete this._showTimer;
			}
			this.close();
		},

		open: function(/*DomNode*/ target){
 			// summary:
			//		Display the tooltip; usually not called directly.
			// tags:
			//		private

			if(this._showTimer){
				clearTimeout(this._showTimer);
				delete this._showTimer;
			}
			Tooltip.show(this.label || this.domNode.innerHTML, target, this.position, !this.isLeftToRight(), this.textDir);

			this._connectNode = target;
			this.onShow(target, this.position);
		},

		close: function(){
			// summary:
			//		Hide the tooltip or cancel timer for show of tooltip
			// tags:
			//		private

			if(this._connectNode){
				// if tooltip is currently shown
				Tooltip.hide(this._connectNode);
				delete this._connectNode;
				this.onHide();
			}
			if(this._showTimer){
				// if tooltip is scheduled to be shown (after a brief delay)
				clearTimeout(this._showTimer);
				delete this._showTimer;
			}
		},

		onShow: function(/*===== target, position =====*/){
			// summary:
			//		Called when the tooltip is shown
			// tags:
			//		callback
		},

		onHide: function(){
			// summary:
			//		Called when the tooltip is hidden
			// tags:
			//		callback
		},

		uninitialize: function(){
			this.close();
			this.inherited(arguments);
		}
	});

	Tooltip._MasterTooltip = MasterTooltip;		// for monkey patching
	Tooltip.show = dijit.showTooltip;		// export function through module return value
	Tooltip.hide = dijit.hideTooltip;		// export function through module return value

	// dijit.Tooltip.defaultPosition: String[]
	//		This variable controls the position of tooltips, if the position is not specified to
	//		the Tooltip widget or *TextBox widget itself.  It's an array of strings with the values
	//		possible for `dijit/place::around()`.   The recommended values are:
	//
	//			* before-centered: centers tooltip to the left of the anchor node/widget, or to the right
	//				 in the case of RTL scripts like Hebrew and Arabic
	//			* after-centered: centers tooltip to the right of the anchor node/widget, or to the left
	//				 in the case of RTL scripts like Hebrew and Arabic
	//			* above-centered: tooltip is centered above anchor node
	//			* below-centered: tooltip is centered above anchor node
	//
	//		The list is positions is tried, in order, until a position is found where the tooltip fits
	//		within the viewport.
	//
	//		Be careful setting this parameter.  A value of "above-centered" may work fine until the user scrolls
	//		the screen so that there's no room above the target node.   Nodes with drop downs, like
	//		DropDownButton or FilteringSelect, are especially problematic, in that you need to be sure
	//		that the drop down and tooltip don't overlap, even when the viewport is scrolled so that there
	//		is only room below (or above) the target node, but not both.
	Tooltip.defaultPosition = ["after-centered", "before-centered"];


	return Tooltip;
});

},
'dojox/grid/LazyTreeGridStoreModel':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dijit/tree/ForestStoreModel"], function(declare, array, lang, ForestStoreModel){

return declare("dojox.grid.LazyTreeGridStoreModel", ForestStoreModel, {

	// There are different approaches to get children for client-side
	// DataStore (e.g. dojo.data.ItemFileReadStore) or server-side DataStore
	// (e.g. dojox.data.QueryReadStore), so we need to be sure what kind of
	// DataStore is being used
	serverStore: false, // server side store
	
	constructor: function(/* Object */ args){
		this.serverStore = !!args.serverStore;
	},

	mayHaveChildren: function(/*dojo.data.Item*/ item){
		var children = null;
		return array.some(this.childrenAttrs, function(attr){
				children = this.store.getValue(item, attr);
				if(lang.isString(children)){
					return parseInt(children, 10) > 0 || children.toLowerCase() === "true" ? true : false;
				}else if(typeof children == "number"){
					return children > 0;
				}else if(typeof children == "boolean"){
					return children;
				}else if(this.store.isItem(children)){
					children = this.store.getValues(item, attr);
					return lang.isArray(children) ? children.length > 0 : false;
				}else{
					return false;
				}
		}, this);
	},
	
	getChildren: function(/*dojo.data.Item*/parentItem, /*function(items, size)*/onComplete, /*function*/ onError, /*object*/queryObj){
		if(queryObj){
			var start = queryObj.start || 0,
				count = queryObj.count,
				parentId = queryObj.parentId,
				sort = queryObj.sort;
			if(parentItem === this.root){
				this.root.size = 0;
				this.store.fetch({
					start: start,
					count: count,
					sort: sort,
					query: this.query,
					onBegin: lang.hitch(this, function(size){
						this.root.size = size;
					}),
					onComplete: lang.hitch(this, function(items){
						onComplete(items, queryObj, this.root.size);
					}),
					onError: onError
				});
			}else{
				var store = this.store;
				if(!store.isItemLoaded(parentItem)){
					var getChildren = lang.hitch(this, arguments.callee);
					store.loadItem({
						item: parentItem,
						onItem: function(parentItem){
							getChildren(parentItem, onComplete, onError, queryObj);
						},
						onError: onError
					});
					return;
				}
				if(this.serverStore && !this._isChildrenLoaded(parentItem)){
					this.childrenSize = 0;
					this.store.fetch({
						start: start,
						count: count,
						sort: sort,
						query: lang.mixin({parentId: parentId}, this.query || {}),
						onBegin: lang.hitch(this, function(size){
							this.childrenSize = size;
						}),
						onComplete: lang.hitch(this, function(items){
							onComplete(items, queryObj, this.childrenSize);
						}),
						onError: onError
					});
				}else{
					this.inherited(arguments);
				}
			}
		}else{
			this.inherited(arguments);
		}
	},
	
	_isChildrenLoaded: function(parentItem){
		// summary:
		//		Check if all children of the given item have been loaded
		var children = null;
		return array.every(this.childrenAttrs, function(attr){
			children = this.store.getValues(parentItem, attr);
			return array.every(children, function(c){
				//Since we allow lazy loading, that is, children might be "true" instead of an item, so check isItem first.
				return this.store.isItem(c) && this.store.isItemLoaded(c);
			}, this);
		}, this);
	},
	
	//overwritten
	onNewItem: function(item, parentInfo){ },
	
	onDeleteItem: function(item){ }
});
});

},
'dojoe/tablex/_ColumnVisible':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!dijit/_Widget,dijit/_TemplatedMixin,dijit/_WidgetsInTemplateMixin"], function(dijit,dojo,dojox){
dojo.provide("dojoe.tablex._ColumnVisible");

dojo.require("dijit._Widget");
dojo.require("dijit._TemplatedMixin");
dojo.require("dijit._WidgetsInTemplateMixin");

// define the widget class
dojo.declare (
	"dojoe.tablex._ColumnVisible", [ dijit._Widget, dijit._TemplatedMixin, dijit._WidgetsInTemplateMixin ],{
	
	// summary:
	//		Private class - Configure columns - holds hide/show status for columns
	
	columnid: null,
	label: null,
	checked: false,
	widgetsInTemplate: true,
	// Path to the template
	templateString : dojo.cache("dojoe.tablex", "templates/ColumnVisible.html", "<div>\r\n\t<input dojoType=\"dijit.form.CheckBox\" data-dojo-attach-point=\"checkbox\" name=\"${columnid}CB\" id=\"${columnid}CB\" value=\"\" role=\"checkbox\" />\r\n\t\t&#160;\r\n\t<label for=\"${columnid}CB\">${label}</label>\r\n</div>\r\n"),		
	constructor: function () {		
	},
	postCreate: function () {		
		if (this.checked) {
			this.checkbox.setChecked (this.checked);
		    //Defect 27350: Safari:In Table, Select All/ Deselect All for config columns popup does not work as expected for safari browser.
		    //Anupama -  workaround, adding a fake event
		    dijit.byId(this.checkbox.id.toString()).set("disabled", true);
		    dijit.byId(this.checkbox.id.toString()).set("disabled", false);
		}		
	},
	setVisible: function (checked) {
		this.checkbox.setChecked (checked);
		//Defect 27350: Safari:In Table, Select All/ Deselect All for config columns popup does not work as expected for safari browser.
		//Anupama -  workaround, adding a fake event
		dijit.byId(this.checkbox.id.toString()).set("disabled", true);
		dijit.byId(this.checkbox.id.toString()).set("disabled", false);
	},
	isVisible: function () {
		return this.checkbox.checked;
		
	},
	destroy: function () {		
		//memory leak issue: custom destroy method to take care of removing all widgets					
		try{
			delete this.resources_;
			//this is for destroying child templated widgets 
			//this should happen by itself but need to figure out why dijit._Templated is not taking care of it...
			var widgets = dijit.findWidgets(this.domNode);
			dojo.forEach(widgets, function(w) {
				if(w){
					if(w.destroyRecursive){
						w.destroyRecursive();
					} else if(w.destroy){
						w.destroy();
					}
				}
			});
			// this is for widgets added as child to this widget and their children..
			//not present in template
			var widgetArr = this.getChildren();
			dojo.forEach(widgetArr, function (child) {
				if(child){
					if(child.destroyRecursive){
						child.destroyRecursive();
					} else if(child.destroy){
						child.destroy();
					}
				}
			});
			this.inherited(arguments);
		}catch(ex){
			console.warn("Widget destroy failed: "+this.id+", ex="+ex.name+":"+ex.message);
		}
	}	
});
});

},
'ibm/usmi/widgets/finder/ResourceDataStore':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
// Finder FinderBaseDataStore
define("ibm/usmi/widgets/finder/ResourceDataStore", [
	"dojo/_base/lang",
	"dojo/_base/declare",
	"ibm/usmi/util/Utils",
	"./FinderBaseDataStore"
], function (djLang, declare, Utils, FinderBaseDataStore) {

	return declare("itme.finder.ResourceDataStore", [ FinderBaseDataStore
	], {

		constructor : function (type, cache) {
			this.populateItems(cache);
		},
		populateItems : function () {
			this.inherited(arguments);
		},

		initiateTaskSearch : function (searchString, finderObj) {
			var query = "name:'*" + searchString + "*' OR category:'*" + searchString + "*'";
			Utils.CMTraceTimeDelta("finderTimer", "before initiating search", true);
			if (this._dataStore) {
				this._dataStore.fetch({
					query : {
						complexQuery : query
					},
					queryOptions : {
						ignoreCase : true
					},
					onError : djLang.hitch(finderObj, finderObj.fetchFailed),
					onComplete : djLang.hitch(finderObj, finderObj.doResourceSearch)
				});
			} else {
				finderObj.doResourceSearch(new Array(0), null);
			}
		},
		destroy : function () {
			this.inherited(arguments);
		}
	});
});

},
'gridx/core/model/extensions/Move':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/Deferred",
	"dojo/DeferredList",
	'../_Extension'
], function(declare, lang, Deferred, DeferredList, _Extension){

	var hitch = lang.hitch;

	function getMoves(indexes, target){
		//Transform arbitrary index array to an array of standard moves: [start, count, target].
		var i, len, arr = [], moves = [], move, 
			beforeBegin = 1, afterBegin = 1,
			beforeTarget = target, afterTarget = target, pos;
		for(i = 0, len = indexes.length; i < len; ++i){
			arr[indexes[i]] = 1;
		}
		for(i = 0, len = arr.length; i < len; ++i){
			if(arr[i]){
				if(i < target){
					if(beforeBegin){
						beforeBegin = 0;
						move = [i, 1];
						pos = moves.length;
						moves.unshift(move);
					}else if(arr[i - 1]){
						++move[1];
					}
				}else{
					if(afterBegin){
						afterBegin = 0;
						move = [i, 1, afterTarget];
						moves.push(move);
						++afterTarget;
					}else if(arr[i - 1]){
						++move[1];
						++afterTarget;
					}
				}
			}else{
				beforeBegin = afterBegin = 1;
			}
		}
		for(i = 0; i <= pos; ++i){
			move = moves[i];
			move[2] = beforeTarget;
			beforeTarget -= move[1];
		}
		return moves;
	}

	function mapIndex(start, count, target, map){
		//Do the actual mapping index work.
		var mapping = {}, from, to, i, revMap = {};

		if(target > start + count){
			//target is after the range
			for(i = 0; i < count; ++i){
				mapping[start + i] = target + i - count;
			}
			for(i = 0; i < target - start - count; ++i){
				mapping[start + count + i] = start + i;
			}
		}else if(target < start){
			//target is before the range
			for(i = 0; i < count; ++i){
				mapping[start + i] = target + i;
			}
			for(i = 0; i < start - target; ++i){
				mapping[target + i] = target + i + count;
			}
		}else{
			//target is in the range
			return;
		}
		for(from in map){
			revMap[map[from]] = parseInt(from, 10);
		}
		for(from in mapping){
			to = mapping[from];
			if(revMap.hasOwnProperty(from)){
				from = revMap[from];
			}
			if(from == to){
				delete map[from];
			}else{
				map[from] = to;
			}
		}
	}
	return declare(/*===== "gridx.core.model.extensions.Move", =====*/_Extension, {
		// Not compatible with Sort and Map extensions!
		name: 'move',

		priority: 10,

		constructor: function(model, args){
			var t = this, options = model._cache.options = model._cache.options || {};
			t._mixinAPI('move', 'moveIndexes', 'insert');
			model.onMoved = function(){};
			//User can customize how to deal with moved rows by providing his own updateStore method.
			if(args.updateStore){
				t.updateStore = args.updateStore;
			}
			if(args.moveSortInfo){
				//User can customize the whole sort info.
				options.sort = args.moveSortInfo;
			}else{
				//Or just provide a field indicating the moving order (recommended)
				options.sort = [{
					attribute: t.moveField = args.moveField || 'order',
					descending: t.moveFieldDescending = args.moveFieldDescending || false
				}];
			}
		},

		//Public---------------------------------------------------------------------
		//moveField: 'order',

		//moveFieldDescending: false,

		//moveSortInfo: [],

		clear: function(){},

		move: function(start, count, target){
			// summary
			//		Move rows by row index.
			if(start >= 0 && start < Infinity && 
				count > 0 && count < Infinity && 
				target >= 0 && target < Infinity && 
				(target < start || target > start + count)){
				this.model._addCmd({
					name: '_cmdMove',
					scope: this,
					args: [start, count, target],
					async: 1
				});
			}
		},

		moveIndexes: function(indexes, target){
			this.model._addCmd({
				name: '_cmdMove',
				scope: this,
				args: [indexes, target],
				async: 1
			});
		},

		insert: function(dataArray, prevItem, nextItem){
			var finished = new Deferred(),
				success = hitch(finished, finished.callback),
				fail = hitch(finished, finished.errback),
				moveField = this.moveField,
				store = this.model.store,
				i, data, dl = [],
				getValue = function(item){
					return store.fetch ? store.getValue(item, moveField) : item[moveField];
				},
				prevValue = prevItem ? getValue(prevItem) : null,
				nextValue = nextItem ? getValue(nextItem) : null;
			for(i = 0; data = dataArray[i]; ++i){
				if(prevValue === null && nextValue === null){
					//No data in grid
					prevValue = Math.random();
				}else if(prevValue === null){
					//Be first row in grid
					prevValue = nextValue - 1;
				}else if(nextValue === null){
					//Be last row in grid
					prevValue = prevValue + 1;
				}else{
					//Between 2 existing rows
					prevValue = (prevValue + nextValue) / 2;
				}
				data[moveField] = prevValue;
				if(store.fetch){
					store.newItem(data);
				}else{
					var d = new Deferred();
					Deferred.when(store.add(data), hitch(d, d.callback), hitch(d, d.errback));
					dl.push(d);
				}
			}
			if(store.fetch){
				store.save({
					onComplete: success,
					onError: fail
				});
			}else{
				new DeferredList(dl).then(success, fail);
			}
			return finished;
		},
		
		updateStore: function(finishDef, moves, map){
			// summary:
			//		Do the actual moving work here, that is to change the moveField of moved rows.
			//		User can overwrite this function to provide customized logic.
			//		Here a default implementation is provided. This implementation requires the
			//		moveField is a field of number type, and it can accept any number. In other words,
			//		this moveField is just a field to indicate row order without any other meanings.
			//		This default implementation tries to reduce the overall requests sent to store, 
			//		because the current store implementation sends a separate PUT command to store for every
			//		single item one by one.
			//
			//		Note: this is more like a public attribute rather than a public method, because users
			//		should assign a value to is rather can directly call it.
			//
			// finishDef: dojo.Deferred
			//		A Deferred object to indicate when the update is finished. finishDef.callback() or finishDef.errback()
			//		must be called in this function, otherwise the grid will break.
			// moves: Array
			//		This is an array of arrays. Each element is an array of 3 numbers: startIndex, rowCount, targetIndex.
			//		These numbers represents a single movement operation, which has the following semantic:
			//			"rowCount" rows from index "startIndex" are moved to index "targetIndex"
			//		These movement operations take place in the given order, so the index in later movement is different from
			//		those in the previous movement, since it is based on the result of the previous movement.
			// map: Association array
			//		An index mapping from the original indexes to the new indexes. This map is generated from the "moves" argument,
			//		so it has exactly the same information as "moves". But it is pre-processed to reflect the final index mapping
			//		after all these movements are done, so it might be easier to use.
			var reverseIndexes = [], info = [], indexes = [], ranges = [],
				from, to, m = {}, i, dif, cat = {}, mostDif, maxCount = 0, t = this, 
				inner = t.inner, store = t.model.store, moveField = t.moveField, dl = [],

				findBefore = function(to){
					if(to > 0){
						return reverseIndexes[to - 1] === undefined ? to - 1 : reverseIndexes[to - 1];
					}else{
						return -1;
					}
				},
				findAfter = function(to){
					for(; to < reverseIndexes.length; ++to){
						var from = reverseIndexes[to];
						if(!info[from]){
							return from === undefined ? to : from;
						}
					}
					return to;
				},
				getItem = function(index){
					var row = inner._call('byIndex', [index]);
					return row && row.item;
				},
				getValue = function(item){
					return store.fetch ? store.getValue(item, moveField) : item[moveField];
				},
				calcValue = function(index, size){
					var context = info[index];
					if(context){
						return context.value;
					}else if(index < 0){
						return getValue(getItem(0)) - 1;
					}else if(index < size){
						return getValue(getItem(index));
					}else{
						return getValue(getItem(size - 1)) + 1;
					}
				},
				setValue = function(item, value){
					if(store.fetch){
						store.setValue(item, moveField, value);
					}else{
						item = lang.clone(item);
						item[moveField] = value;
//                        console.log('setValue:', item, moveField, value);
						var d = new Deferred();
						Deferred.when(store.put(item, {
							overwrite: true
						}), hitch(d, d.callback));
						dl.push(d);
					}
				},
				saveStore = function(){
					if(store.fetch){
						store.save({
							onComplete: function(){
								finishDef.callback();
							},
							onError: function(e){
								finishDef.errback(e);
							}
						});
					}else{
						(new DeferredList(dl)).then(function(){
							finishDef.callback();
						}, function(e){
							finishDef.errback(e);
						});
					}
				};

			var first = Infinity;
			for(from in map){
				from = parseInt(from, 10);
				to = map[from];
				m[from] = to;
				if(to < first){
					first = to;
				}
				reverseIndexes[to] = from;
			}
			for(i = first; i < reverseIndexes.length; ++i){
				if(reverseIndexes[i] === undefined){
					reverseIndexes[i] = i;
					m[i] = i;
				}
			}
			//Categorize mappings
			for(from in m){
				from = parseInt(from, 10);
				to = m[from];
				dif = to - from;
				if(cat[dif] === undefined){
					cat[dif] = 1;
				}else{
					++cat[dif];
				}
			}
			//Find the category with most rows
			for(dif in cat){
				if(cat[dif] > maxCount){
					maxCount = cat[dif];
					mostDif = dif;
				}
			}
			//Find out the rows that need to update (all the rows not belong to the biggest category)
			for(to = 0; to < reverseIndexes.length; ++to){
				from = reverseIndexes[to];
				if(from !== undefined && to - from != mostDif){
					info[from] = {};
					indexes.push(from);
				}
			}
			//For every row to change, find it's previous and next row in the final order.
			for(i = 0; i < indexes.length; ++i){
				from = indexes[i];
				to = m[from];
				var before = info[from].before = findBefore(to),
					after = info[from].after = findAfter(to);
				ranges.push({
					start: from,
					count: 1
				}, {
					start: after,
					count: 1
				});
				if(before >= 0){
					ranges.push({
						start: before,
						count: 1
					});
				}
			}
//            console.log('info:', info, cat);
			//Apply the change to store.
			inner._call('when', [{
				id: [],
				range: ranges
			}, function(){
				var size = inner._call('size');
				for(var i = 0; i < indexes.length; ++i){
					from = indexes[i];
					var fromItem = getItem(from),
						context = info[from],
						beforeValue = calcValue(context.before, size),
						afterValue = calcValue(context.after, size),
						value = (beforeValue + afterValue) / 2;
					context.value = value;
					setValue(fromItem, value);
				}
				saveStore();
			}]);
		},

		//Private--------------------------------------------------------------------
		_cmdMove: function(){
			//Process the move command
			var d = new Deferred(), t = this,
				m = t.model, i, args,
				map = {}, moved, moves = [],
				size = t.inner._call('size');

			for(i = 0; args = arguments[i]; ++i){
				m._msg('beforeMove', args);
				if(args.length == 2){
					moves = moves.concat(getMoves(args[0], args[1]));
				}else{
					moves.push(args);
				}
			}
			for(i = 0; args = moves[i]; ++i){
				mapIndex(args[0], args[1], args[2], map);
			}
			for(i in map){
				moved = 1;
				break;
			}
			if(moved){
				t.updateStore(d, moves, map);
			}else{
				d.callback();
			}
			d.then(function(){
				m._cache.clear();
				m._msg('moved', map);
				m.onMoved(moves, map);
			});
			return d;
		}
	});
});

},
'dojo/date/stamp':function(){
define(["../_base/kernel", "../_base/lang", "../_base/array"], function(dojo, lang, array) {
	// module:
	//		dojo/date/stamp
	// summary:
	//		TODOC

lang.getObject("date.stamp", true, dojo);

// Methods to convert dates to or from a wire (string) format using well-known conventions

dojo.date.stamp.fromISOString = function(/*String*/formattedString, /*Number?*/defaultTime){
	//	summary:
	//		Returns a Date object given a string formatted according to a subset of the ISO-8601 standard.
	//
	//	description:
	//		Accepts a string formatted according to a profile of ISO8601 as defined by
	//		[RFC3339](http://www.ietf.org/rfc/rfc3339.txt), except that partial input is allowed.
	//		Can also process dates as specified [by the W3C](http://www.w3.org/TR/NOTE-datetime)
	//		The following combinations are valid:
	//
	//			* dates only
	//			|	* yyyy
	//			|	* yyyy-MM
	//			|	* yyyy-MM-dd
	// 			* times only, with an optional time zone appended
	//			|	* THH:mm
	//			|	* THH:mm:ss
	//			|	* THH:mm:ss.SSS
	// 			* and "datetimes" which could be any combination of the above
	//
	//		timezones may be specified as Z (for UTC) or +/- followed by a time expression HH:mm
	//		Assumes the local time zone if not specified.  Does not validate.  Improperly formatted
	//		input may return null.  Arguments which are out of bounds will be handled
	// 		by the Date constructor (e.g. January 32nd typically gets resolved to February 1st)
	//		Only years between 100 and 9999 are supported.
	//
  	//	formattedString:
	//		A string such as 2005-06-30T08:05:00-07:00 or 2005-06-30 or T08:05:00
	//
	//	defaultTime:
	//		Used for defaults for fields omitted in the formattedString.
	//		Uses 1970-01-01T00:00:00.0Z by default.

	if(!dojo.date.stamp._isoRegExp){
		dojo.date.stamp._isoRegExp =
//TODO: could be more restrictive and check for 00-59, etc.
			/^(?:(\d{4})(?:-(\d{2})(?:-(\d{2}))?)?)?(?:T(\d{2}):(\d{2})(?::(\d{2})(.\d+)?)?((?:[+-](\d{2}):(\d{2}))|Z)?)?$/;
	}

	var match = dojo.date.stamp._isoRegExp.exec(formattedString),
		result = null;

	if(match){
		match.shift();
		if(match[1]){match[1]--;} // Javascript Date months are 0-based
		if(match[6]){match[6] *= 1000;} // Javascript Date expects fractional seconds as milliseconds

		if(defaultTime){
			// mix in defaultTime.  Relatively expensive, so use || operators for the fast path of defaultTime === 0
			defaultTime = new Date(defaultTime);
			array.forEach(array.map(["FullYear", "Month", "Date", "Hours", "Minutes", "Seconds", "Milliseconds"], function(prop){
				return defaultTime["get" + prop]();
			}), function(value, index){
				match[index] = match[index] || value;
			});
		}
		result = new Date(match[0]||1970, match[1]||0, match[2]||1, match[3]||0, match[4]||0, match[5]||0, match[6]||0); //TODO: UTC defaults
		if(match[0] < 100){
			result.setFullYear(match[0] || 1970);
		}

		var offset = 0,
			zoneSign = match[7] && match[7].charAt(0);
		if(zoneSign != 'Z'){
			offset = ((match[8] || 0) * 60) + (Number(match[9]) || 0);
			if(zoneSign != '-'){ offset *= -1; }
		}
		if(zoneSign){
			offset -= result.getTimezoneOffset();
		}
		if(offset){
			result.setTime(result.getTime() + offset * 60000);
		}
	}

	return result; // Date or null
};

/*=====
	dojo.date.stamp.__Options = function(){
		//	selector: String
		//		"date" or "time" for partial formatting of the Date object.
		//		Both date and time will be formatted by default.
		//	zulu: Boolean
		//		if true, UTC/GMT is used for a timezone
		//	milliseconds: Boolean
		//		if true, output milliseconds
		this.selector = selector;
		this.zulu = zulu;
		this.milliseconds = milliseconds;
	}
=====*/

dojo.date.stamp.toISOString = function(/*Date*/dateObject, /*dojo.date.stamp.__Options?*/options){
	//	summary:
	//		Format a Date object as a string according a subset of the ISO-8601 standard
	//
	//	description:
	//		When options.selector is omitted, output follows [RFC3339](http://www.ietf.org/rfc/rfc3339.txt)
	//		The local time zone is included as an offset from GMT, except when selector=='time' (time without a date)
	//		Does not check bounds.  Only years between 100 and 9999 are supported.
	//
	//	dateObject:
	//		A Date object

	var _ = function(n){ return (n < 10) ? "0" + n : n; };
	options = options || {};
	var formattedDate = [],
		getter = options.zulu ? "getUTC" : "get",
		date = "";
	if(options.selector != "time"){
		var year = dateObject[getter+"FullYear"]();
		date = ["0000".substr((year+"").length)+year, _(dateObject[getter+"Month"]()+1), _(dateObject[getter+"Date"]())].join('-');
	}
	formattedDate.push(date);
	if(options.selector != "date"){
		var time = [_(dateObject[getter+"Hours"]()), _(dateObject[getter+"Minutes"]()), _(dateObject[getter+"Seconds"]())].join(':');
		var millis = dateObject[getter+"Milliseconds"]();
		if(options.milliseconds){
			time += "."+ (millis < 100 ? "0" : "") + _(millis);
		}
		if(options.zulu){
			time += "Z";
		}else if(options.selector != "time"){
			var timezoneOffset = dateObject.getTimezoneOffset();
			var absOffset = Math.abs(timezoneOffset);
			time += (timezoneOffset > 0 ? "-" : "+") +
				_(Math.floor(absOffset/60)) + ":" + _(absOffset%60);
		}
		formattedDate.push(time);
	}
	return formattedDate.join('T'); // String
};

return dojo.date.stamp;
});

},
'dojox/validate':function(){
define(["./validate/_base"], function(validate){
	/*===== 
	dojox.validate = {
		// summary: Additional validation routines for Strings, Numbers, credit cards, and other esoteric needs. 
	};
	=====*/
	return validate;
});

},
'dijit/MenuSeparator':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.setSelectable
	"./_WidgetBase",
	"./_TemplatedMixin",
	"./_Contained",
	"dojo/text!./templates/MenuSeparator.html"
], function(declare, dom, _WidgetBase, _TemplatedMixin, _Contained, template){

/*=====
	var _WidgetBase = dijit._WidgetBase;
	var _TemplatedMixin = dijit._TemplatedMixin;
	var _Contained = dijit._Contained;
=====*/

	// module:
	//		dijit/MenuSeparator
	// summary:
	//		A line between two menu items

	return declare("dijit.MenuSeparator", [_WidgetBase, _TemplatedMixin, _Contained], {
		// summary:
		//		A line between two menu items

		templateString: template,

		buildRendering: function(){
			this.inherited(arguments);
			dom.setSelectable(this.domNode, false);
		},

		isFocusable: function(){
			// summary:
			//		Override to always return false
			// tags:
			//		protected

			return false; // Boolean
		}
	});
});

},
'dijit/CalendarLite':function(){
define([
	"dojo/_base/array", // array.forEach array.map
	"dojo/_base/declare", // declare
	"dojo/cldr/supplemental", // cldrSupplemental.getFirstDayOfWeek
	"dojo/date", // date
	"dojo/date/locale",
	"dojo/dom", // dom.setSelectable
	"dojo/dom-class", // domClass.contains
	"dojo/_base/event", // event.stop
	"dojo/_base/lang", // lang.getObject, lang.hitch
	"dojo/_base/sniff", // has("ie") has("webkit")
	"dojo/string", // string.substitute
	"dojo/_base/window", // win.doc.createTextNode
	"./_WidgetBase",
	"./_TemplatedMixin",
	"dojo/text!./templates/Calendar.html"
], function(array, declare, cldrSupplemental, date, local, dom, domClass, event, lang, has, string, win,
			_WidgetBase, _TemplatedMixin, template){

/*=====
	var _WidgetBase = dijit._WidgetBase;
	var _TemplatedMixin = dijit._TemplatedMixin;
=====*/

	// module:
	//		dijit/CalendarLite
	// summary:
	//		Lightweight version of Calendar widget aimed towards mobile use

	var CalendarLite = declare("dijit.CalendarLite", [_WidgetBase, _TemplatedMixin], {
		// summary:
		//		Lightweight version of Calendar widget aimed towards mobile use
		//
		// description:
		//		A simple GUI for choosing a date in the context of a monthly calendar.
		//		This widget can't be used in a form because it doesn't serialize the date to an
		//		`<input>` field.  For a form element, use dijit.form.DateTextBox instead.
		//
		//		Note that the parser takes all dates attributes passed in the
		//		[RFC 3339 format](http://www.faqs.org/rfcs/rfc3339.html), e.g. `2005-06-30T08:05:00-07:00`
		//		so that they are serializable and locale-independent.
		//
		//		Also note that this widget isn't keyboard accessible; use dijit.Calendar for that
		// example:
		//	|	var calendar = new dijit.CalendarLite({}, dojo.byId("calendarNode"));
		//
		// example:
		//	|	<div data-dojo-type="dijit.CalendarLite"></div>

		// Template for main calendar
		templateString: template,

		// Template for cell for a day of the week (ex: M)
		dowTemplateString: '<th class="dijitReset dijitCalendarDayLabelTemplate" role="columnheader"><span class="dijitCalendarDayLabel">${d}</span></th>',

		// Templates for a single date (ex: 13), and for a row for a week (ex: 20 21 22 23 24 25 26)
		dateTemplateString: '<td class="dijitReset" role="gridcell" data-dojo-attach-point="dateCells"><span class="dijitCalendarDateLabel" data-dojo-attach-point="dateLabels"></span></td>',
		weekTemplateString: '<tr class="dijitReset dijitCalendarWeekTemplate" role="row">${d}${d}${d}${d}${d}${d}${d}</tr>',

		// value: Date
		//		The currently selected Date, initially set to invalid date to indicate no selection.
		value: new Date(""),
		// TODO: for 2.0 make this a string (ISO format) rather than a Date

		// datePackage: String
		//		JavaScript object containing Calendar functions.  Uses Gregorian Calendar routines
		//		from dojo.date by default.
		datePackage: date,

		// dayWidth: String
		//		How to represent the days of the week in the calendar header. See locale
		dayWidth: "narrow",

		// tabIndex: Integer
		//		Order fields are traversed when user hits the tab key
		tabIndex: "0",

		// currentFocus: Date
		//		Date object containing the currently focused date, or the date which would be focused
		//		if the calendar itself was focused.   Also indicates which year and month to display,
		//		i.e. the current "page" the calendar is on.
		currentFocus: new Date(),

		baseClass:"dijitCalendar",

		_isValidDate: function(/*Date*/ value){
			// summary:
			//		Runs various tests on the value, checking that it's a valid date, rather
			//		than blank or NaN.
			// tags:
			//		private
			return value && !isNaN(value) && typeof value == "object" &&
				value.toString() != this.constructor.prototype.value.toString();
		},

		_getValueAttr: function(){
			// summary:
			//		Support get('value')

			// this.value is set to 1AM, but return midnight, local time for back-compat
			if(this.value && !isNaN(this.value)){
				var value = new this.dateClassObj(this.value);
				value.setHours(0, 0, 0, 0);

				// If daylight savings pushes midnight to the previous date, fix the Date
				// object to point at 1am so it will represent the correct day. See #9366
				if(value.getDate() < this.value.getDate()){
					value = this.dateFuncObj.add(value, "hour", 1);
				}
				return value;
			}else{
				return null;
			}
		},

		_setValueAttr: function(/*Date|Number*/ value, /*Boolean*/ priorityChange){
			// summary:
			//		Support set("value", ...)
			// description:
			// 		Set the current date and update the UI.  If the date is disabled, the value will
			//		not change, but the display will change to the corresponding month.
			// value:
			//		Either a Date or the number of seconds since 1970.
			// tags:
			//      protected
			if(value){
				// convert from Number to Date, or make copy of Date object so that setHours() call below
				// doesn't affect original value
				value = new this.dateClassObj(value);
			}
			if(this._isValidDate(value)){
				if(!this._isValidDate(this.value) || this.dateFuncObj.compare(value, this.value)){
					value.setHours(1, 0, 0, 0); // round to nearest day (1am to avoid issues when DST shift occurs at midnight, see #8521, #9366)

					if(!this.isDisabledDate(value, this.lang)){
						this._set("value", value);

						// Set focus cell to the new value.   Arguably this should only happen when there isn't a current
						// focus point.   This will also repopulate the grid, showing the new selected value (and possibly
						// new month/year).
						this.set("currentFocus", value);

						if(priorityChange || typeof priorityChange == "undefined"){
							this.onChange(this.get('value'));
						}
					}
				}
			}else{
				// clear value, and repopulate grid (to deselect the previously selected day) without changing currentFocus
				this._set("value", null);
				this.set("currentFocus", this.currentFocus);
			}
		},

		_setText: function(node, text){
			// summary:
			//		This just sets the content of node to the specified text.
			//		Can't do "node.innerHTML=text" because of an IE bug w/tables, see #3434.
			// tags:
			//      private
			while(node.firstChild){
				node.removeChild(node.firstChild);
			}
			node.appendChild(win.doc.createTextNode(text));
		},

		_populateGrid: function(){
			// summary:
			//      Fills in the calendar grid with each day (1-31)
			// tags:
			//      private

			var month = new this.dateClassObj(this.currentFocus);
			month.setDate(1);

			var firstDay = month.getDay(),
				daysInMonth = this.dateFuncObj.getDaysInMonth(month),
				daysInPreviousMonth = this.dateFuncObj.getDaysInMonth(this.dateFuncObj.add(month, "month", -1)),
				today = new this.dateClassObj(),
				dayOffset = cldrSupplemental.getFirstDayOfWeek(this.lang);
			if(dayOffset > firstDay){ dayOffset -= 7; }

			// Mapping from date (as specified by number returned from Date.valueOf()) to corresponding <td>
			this._date2cell = {};

			// Iterate through dates in the calendar and fill in date numbers and style info
			array.forEach(this.dateCells, function(template, idx){
				var i = idx + dayOffset;
				var date = new this.dateClassObj(month),
					number, clazz = "dijitCalendar", adj = 0;

				if(i < firstDay){
					number = daysInPreviousMonth - firstDay + i + 1;
					adj = -1;
					clazz += "Previous";
				}else if(i >= (firstDay + daysInMonth)){
					number = i - firstDay - daysInMonth + 1;
					adj = 1;
					clazz += "Next";
				}else{
					number = i - firstDay + 1;
					clazz += "Current";
				}

				if(adj){
					date = this.dateFuncObj.add(date, "month", adj);
				}
				date.setDate(number);

				if(!this.dateFuncObj.compare(date, today, "date")){
					clazz = "dijitCalendarCurrentDate " + clazz;
				}

				if(this._isSelectedDate(date, this.lang)){
					clazz = "dijitCalendarSelectedDate " + clazz;
					template.setAttribute("aria-selected", true);
				}else{
					template.setAttribute("aria-selected", false);
				}

				if(this.isDisabledDate(date, this.lang)){
					clazz = "dijitCalendarDisabledDate " + clazz;
					template.setAttribute("aria-disabled", true);
				}else{
					clazz = "dijitCalendarEnabledDate " + clazz;
					template.removeAttribute("aria-disabled");
				}

				var clazz2 = this.getClassForDate(date, this.lang);
				if(clazz2){
					clazz = clazz2 + " " + clazz;
				}

				template.className = clazz + "Month dijitCalendarDateTemplate";

				// Each cell has an associated integer value representing it's date
				var dateVal = date.valueOf();
				this._date2cell[dateVal] = template;
				template.dijitDateValue = dateVal;

				// Set Date string (ex: "13").
				this._setText(this.dateLabels[idx], date.getDateLocalized ? date.getDateLocalized(this.lang) : date.getDate());
			}, this);

			// set name of this month
			this.monthWidget.set("month", month);

			// Fill in localized prev/current/next years
			var y = month.getFullYear() - 1;
			var d = new this.dateClassObj();
			array.forEach(["previous", "current", "next"], function(name){
				d.setFullYear(y++);
				this._setText(this[name+"YearLabelNode"],
					this.dateLocaleModule.format(d, {selector:'year', locale:this.lang}));
			}, this);
		},

		goToToday: function(){
			// summary:
			//      Sets calendar's value to today's date
			this.set('value', new this.dateClassObj());
		},

		constructor: function(/*Object*/args){
			this.datePackage = args.datePackage || this.datePackage;
			this.dateFuncObj = typeof this.datePackage == "string" ?
				lang.getObject(this.datePackage, false) :// "string" part for back-compat, remove for 2.0
				this.datePackage;
			this.dateClassObj = this.dateFuncObj.Date || Date;
			this.dateLocaleModule = lang.getObject("locale", false, this.dateFuncObj);
		},

		_createMonthWidget: function(){
			// summary:
			//		Creates the drop down button that displays the current month and lets user pick a new one

			return CalendarLite._MonthWidget({
				id: this.id + "_mw",
				lang: this.lang,
				dateLocaleModule: this.dateLocaleModule
			}, this.monthNode);
		},

		buildRendering: function(){
			// Markup for days of the week (referenced from template)
			var d = this.dowTemplateString,
				dayNames = this.dateLocaleModule.getNames('days', this.dayWidth, 'standAlone', this.lang),
				dayOffset = cldrSupplemental.getFirstDayOfWeek(this.lang);
			this.dayCellsHtml = string.substitute([d,d,d,d,d,d,d].join(""), {d: ""}, function(){
				return dayNames[dayOffset++ % 7]
			});

			// Markup for dates of the month (referenced from template), but without numbers filled in
			var r = string.substitute(this.weekTemplateString, {d: this.dateTemplateString});
			this.dateRowsHtml = [r,r,r,r,r,r].join("");

			// Instantiate from template.
			// dateCells and dateLabels arrays filled when _Templated parses my template.
			this.dateCells = [];
			this.dateLabels = [];
			this.inherited(arguments);

			dom.setSelectable(this.domNode, false);

			var dateObj = new this.dateClassObj(this.currentFocus);

			this._supportingWidgets.push(this.monthWidget = this._createMonthWidget());

			this.set('currentFocus', dateObj, false);	// draw the grid to the month specified by currentFocus

			// Set up connects for increment/decrement of months/years
			var connect = lang.hitch(this, function(nodeProp, part, amount){
				this.connect(this[nodeProp], "onclick", function(){
					this._setCurrentFocusAttr(this.dateFuncObj.add(this.currentFocus, part, amount));
				});
			});
			connect("incrementMonth", "month", 1);
			connect("decrementMonth", "month", -1);
			connect("nextYearLabelNode", "year", 1);
			connect("previousYearLabelNode", "year", -1);
		},

		_setCurrentFocusAttr: function(/*Date*/ date, /*Boolean*/ forceFocus){
			// summary:
			//		If the calendar currently has focus, then focuses specified date,
			//		changing the currently displayed month/year if necessary.
			//		If the calendar doesn't have focus, updates currently
			//		displayed month/year, and sets the cell that will get focus.
			// forceFocus:
			//		If true, will focus() the cell even if calendar itself doesn't have focus

			var oldFocus = this.currentFocus,
				oldCell = oldFocus && this._date2cell ? this._date2cell[oldFocus.valueOf()] : null;

			// round specified value to nearest day (1am to avoid issues when DST shift occurs at midnight, see #8521, #9366)
			date = new this.dateClassObj(date);
			date.setHours(1, 0, 0, 0);

			this._set("currentFocus", date);

			// TODO: only re-populate grid when month/year has changed
			this._populateGrid();

			// set tabIndex=0 on new cell, and focus it (but only if Calendar itself is focused)
			var newCell = this._date2cell[date.valueOf()];
			newCell.setAttribute("tabIndex", this.tabIndex);
			if(this.focused || forceFocus){
				newCell.focus();
			}

			// set tabIndex=-1 on old focusable cell
			if(oldCell && oldCell != newCell){
				if(has("webkit")){	// see #11064 about webkit bug
					oldCell.setAttribute("tabIndex", "-1");
				}else{
					oldCell.removeAttribute("tabIndex");
				}
			}
		},

		focus: function(){
			// summary:
			//		Focus the calendar by focusing one of the calendar cells
			this._setCurrentFocusAttr(this.currentFocus, true);
		},

		_onDayClick: function(/*Event*/ evt){
			// summary:
			//      Handler for day clicks, selects the date if appropriate
			// tags:
			//      protected
			event.stop(evt);
			for(var node = evt.target; node && !node.dijitDateValue; node = node.parentNode);
			if(node && !domClass.contains(node, "dijitCalendarDisabledDate")){
				this.set('value', node.dijitDateValue);
			}
		},

		onChange: function(/*Date*/ /*===== date =====*/){
			// summary:
			//		Called only when the selected date has changed
		},

		_isSelectedDate: function(dateObject /*===== , locale =====*/){
			// summary:
			//		Extension point so developers can subclass Calendar to
			//		support multiple (concurrently) selected dates
			// dateObject: Date
			// locale: String?
			// tags:
			//		protected extension
			return this._isValidDate(this.value) && !this.dateFuncObj.compare(dateObject, this.value, "date")
		},

		isDisabledDate: function(/*===== dateObject, locale =====*/){
			// summary:
			//		May be overridden to disable certain dates in the calendar e.g. `isDisabledDate=dojo.date.locale.isWeekend`
			// dateObject: Date
			// locale: String?
			// tags:
			//      extension
/*=====
			return false; // Boolean
=====*/
		},

		getClassForDate: function(/*===== dateObject, locale =====*/){
			// summary:
			//		May be overridden to return CSS classes to associate with the date entry for the given dateObject,
			//		for example to indicate a holiday in specified locale.
			// dateObject: Date
			// locale: String?
			// tags:
			//      extension

/*=====
			return ""; // String
=====*/
		}
	});

	CalendarLite._MonthWidget = declare("dijit.CalendarLite._MonthWidget", _WidgetBase, {
		// summary:
		//		Displays name of current month padded to the width of the month
		//		w/the longest name, so that changing months doesn't change width.
		//
		//		Create as new dijit.Calendar._MonthWidget({
		//			lang: ...,
		//			dateLocaleModule: ...
		//		})

		_setMonthAttr: function(month){
			// summary:
			//		Set the current month to display as a label
			var monthNames = this.dateLocaleModule.getNames('months', 'wide', 'standAlone', this.lang, month),
				spacer =
					(has("ie") == 6 ? "" :	"<div class='dijitSpacer'>" +
						array.map(monthNames, function(s){ return "<div>" + s + "</div>"; }).join("") + "</div>");

			// Set name of current month and also fill in spacer element with all the month names
			// (invisible) so that the maximum width will affect layout.   But not on IE6 because then
			// the center <TH> overlaps the right <TH> (due to a browser bug).
			this.domNode.innerHTML =
				spacer +
				"<div class='dijitCalendarMonthLabel dijitCalendarCurrentMonthLabel'>" +
				monthNames[month.getMonth()] + "</div>";
		}
	});

	return CalendarLite;
});

},
'ibm/tivoli/navmodel/Item':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/Dataset,ibm/tivoli/navmodel/Property,ibm/tivoli/navmodel/Status,ibm/tivoli/navmodel/LinkStyle,ibm/tivoli/navmodel/Exception,ibm/tivoli/navmodel/Utils,ibm/tivoli/navmodel/Trace"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Item"); 
 
// includes
dojo.require("ibm.tivoli.navmodel.Dataset");
dojo.require("ibm.tivoli.navmodel.Property");
dojo.require("ibm.tivoli.navmodel.Status");
dojo.require("ibm.tivoli.navmodel.LinkStyle");
dojo.require("ibm.tivoli.navmodel.Exception");
dojo.require("ibm.tivoli.navmodel.Utils");
dojo.require("ibm.tivoli.navmodel.Trace");

ibm.tivoli.navmodel.Item._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.Item");

////////////////////////////////////
// Item
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Item", null, {
	// private members       
	_item: null,
	_dataset: null,
	_props: null,
	_propsById: null,
	_nullPropIds: null,
	_allProps: false,
	_allMetadata: false,
	_status: null,
	_style: null,
	_defaultTask: null,
	_expiration: null,
	_trace: ibm.tivoli.navmodel.Item._trace,
	
	/*=====
	constructor_kwArgs: function(item, dataset, propIds, allProperties, allMetadata, expiration) {
		// item: Object
		//		son item object
		// dataset: Object
		//		ibm.tivoli.navmodel.Dataset that owns this item
		// propIds: String[]
		//		optional array indicating the property Id's that were requested with 
		//		the item; so it can be determined if any properties are null
		// allProperties: Boolean
		//		optional: indicates if the item already has "all" properties
		// allMetadata: Boolean
		//		optional: indicates if item properties already have metadata
		// expiration: Long
		//		optional: expiration time;  will use (current time + interval) if not supplied
	},
	=====*/

	 constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs Item object.
		this._item = kwArgs.item;
		if (!this._item) {
			this._item = {};
		}
		
		this._dataset = kwArgs.dataset;
		this._props = [];
		this._propsById = {};
		this._nullPropIds = {};			
		this._allProps = kwArgs.allProperties;
		this._allMetadata = kwArgs.allMetadata;
		
		if (this._item.properties) {
			var propsLength = this._item.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var prop = new ibm.tivoli.navmodel.Property({property: this._item.properties[propIndex], dataset: this._dataset, objectType: "item"});
				this._props[propIndex] = prop;
				this._propsById[prop.getId()] = prop;
			}
			
			// see if we should mark any properties as null because they were not returned
			if (kwArgs.propIds) {
				propsLength = kwArgs.propIds.length;
				for (propIndex = 0;  propIndex < propsLength;  ++propIndex) {
					var propId = kwArgs.propIds[propIndex];				
					if (!this._propsById[propId]) {
						this._nullPropIds[propId] = true;
					}
				}
			}
		}
		
		// expiration was supplied
		if (kwArgs.expiration) {
			this._expiration = kwArgs.expiration;
		}
		// reset to current time + interval
		else {
			this.resetExpiration();
		}
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	
	mixin: function(item, resetLazyLoad) {
		// summary:
		//		Mixin properties and status from this Item object	
		if (item && item !== this) {
			this.removeAttribute("style");
			this._style = null;
			
			// reset lazy load indicators;  mixin will
			// provide the current values
			if (resetLazyLoad) {
				this.resetLazyLoad();
			}
			
			this.mixinAttributes(item.getAttributes());
			this.mixinProperties(item._props, item._nullPropIds);
			this._status = item.getStatuses({widgetDef: null});
			this._defaultTask = item.getDefaultTask();			
			this._allProps = (this._allProps || item._allProps);
			this._allMetadata = (item._allProps && item._allMetadata);				
		}	
	},

	getDataset: function() {
		// summary:
		//		Returns the dataset that owns the item
		return this._dataset;
	},

	getObjectType: function() {
		// summary:
		//		Indicate that this object is an "item"
		return "item";
	},

	getAttribute: function(key) {
		// summary:
		//		Returns general attribute for requested key
		return this._item[key];
	},

	setAttribute: function(key, value) {		
		// summary:
		//		Set general attribute for requested key
		this._item[key] = value;
	},
	
	removeAttribute: function(key) {
		// summary:
		//		Remove general attribute for requested key
		delete this._item[key];
	},

	getAttributes: function() {		
		// summary:
		//		Get all general attributes for the item
		var item = dojo.clone(this._item);
		
		// make sure "properties" are current as we may have received events
		if (this._props && this._props.length > 0) {
			item.properties = [];
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				item.properties[propIndex] = this._props[propIndex].getAttributes();
			}
		}
		
		return item;		
	},

	mixinAttributes: function(attr) {		
		// summary:
		//		Mixin general attributes for the item
		dojo.mixin(this._item, attr);		
	},

	getUri: function() {
		if (!this._item.uri) {
			this._item.uri = this._dataset.getAttribute("uri") + "/items/" + encodeURIComponent(this.getId());
		}
	
		return this._dataset.getArg("baseUrl") + this._item.uri;
	},
	
	getId: function() {		
		// summary:
		//		Returns item id
		return this._item.id;
	},

	setId: function(id) {		
		// summary:
		//		Set the item id
		this._item.id = id;		
	},

	getType: function() {		
		// summary:
		//		Returns the type of item
		return this._item.type;
	},
	
	setType: function(type) {		
		// summary:
		//		Set the item type
		this._item.type = type;		
	},

	getTypeLabel: function() {		
		// summary:
		//		Returns globalized label for the item type
		var label = this._item.typeLabel;
		if (!label) {
			var style = this.getItemStyle({widgetDef: null});
			if (style) {
				label = style.getLabel();
			}
		}
		
		return label;
	},

	setTypeLabel: function(typeLabel) {		
		// summary:
		//		Set the globalized label for the item type
		this._item.typeLabel = typeLabel;		
	},

	getLabel: function() {		
		// summary:
		//		Returns globalized label for the item
		return this._item.label;
	},

	setLabel: function(label) {		
		// summary:
		//		Set the globalized label for the item
		this._item.label = label;		
	},

	getDescription: function() {		
		// summary:
		//		Returns globalized description for the item
		return this._item.description;
	},

	setDescription: function(description) {		
		// summary:
		//		Set the globalized description for the item
		this._item.description = description;		
	},

	getTooltip: function() {		
		// summary:
		//		Returns globalized toolip for the item
		return this._item.tooltip;
	},

	setTooltip: function(tooltip) {		
		// summary:
		//		Set the globalized tooltip for the item
		this._item.tooltip = tooltip;
	},
	
	isLazyLoad: function() {		
		// summary:
		//		Returns true if the item is elligible for "lazy load"
		return (this._item.lazyLoad !== undefined ? this._item.lazyLoad : false);
	},

	setLazyLoad: function(lazyLoad) {		
		// summary:
		//		Set the lazy load attribute
		this._item.lazyLoad = lazyLoad;
	},

	getLazyLoadRelationships: function() {		
		// summary:
		//		Returns the relationships that should be explored for related items
		return this._item.relationshipsKey;
	},

	resetLazyLoad: function() {
		// summary:
		//		Resets current attributes determing whether a lazyLoad
		//		should be performed to find related items
		delete this._item.lazyLoad;
		delete this._item.relationshipsKey;				
	},
	
	setLazyLoadRelationships: function(relationshipsKey) {		
		// summary:
		//		Set the relationships that should be explored for related items
		this._item.relationshipsKey = relationshipsKey;
	},

	isSubgraph: function() {
		// summary:
		//		Returns true if the item represents a nested subgraph parent
		return (this._item.subgraph !== undefined ? this._item.subgraph : false);
	},
	
	setSubgraph: function(subgraph) {		
		// summary:
		//		Set the attribute indicating if the item is a nested subgraph parent
		this._item.subgraph = subgraph;
	},
	
	getSubgraphId: function() {
		// summary:
		//		Returns id of nested subgraph parent; no value if item is not in a subgraph
		return this._item.subgraphId;
	},
	
	setSubgraphId: function(subgraphId) {
		// summary:
		//		Set the id of the sub-graph parent
		this._item.subgraphId = subgraphId;
	},
	
	isSubgraphOrigin: function() {
		// summary:
		//		Returns true if the item is an origin within a nested subgraph
		return (this._item.subgraphOrigin !== undefined ? this._item.subgraphOrigin : false);
	},
	
	setSubgraphOrigin: function(subgraphOrigin) {
		// summary:
		//		Set the attribute indicating if the item is an origin within a nested subgraph
		this._item.subgraphOrigin = subgraphOrigin;
	},
	
	/*=====
	getStatuses_kwArgs: function(widgetDef) {
		// widgetDef: Object
		//		widget definition can override attributes
	},
	=====*/

	getStatuses: function(/*getStatuses_kwArgs*/ kwArgs) {		
		// summary:
		//		Returns ibm.tivoli.navmodel.Status[] objects indicating the item status values
		var args = {
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		// if expired mixin from dataset cache
		this._validateExpiration();
				
		if (!this._status && this._item.status) {
			this._status = [];
			var statusLength = this._item.status.length;
			for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
				if (this._item.status.label && this._item.status.image) {
					this._status.push(new ibm.tivoli.navmodel.Status({status: this._item.status[statusIndex], version: this._version()}));				
				}
				else {
					var statusArgs = {
						type: "item",
						statusId: this._item.status[statusIndex].id,
						statusValue: this._item.status[statusIndex].value				
					};
				
					var statusObj = this._dataset.getStatus(statusArgs);
					if (statusObj) {				
						this._status.push(statusObj);
					}
					else {
						this._trace.error("Could not locate status with id=" + statusArgs.statusId + " value=" + statusArgs.statusValue, "getStatuses");
					}
				}
			}			
		}
		
		// merge widget definition status
		var status = this._status;
		if (status && args.widgetDef) {
			status = this._status.slice(0);
			var statusLength2 = this._status.length;
			for (var statusIndex2 = 0;  statusIndex2 < statusLength2;  ++statusIndex2) {
				status[statusIndex2] = this._status[statusIndex2].clone(args.widgetDef, "item");
			}			
		}
		
		return status;		
	},

	setStatuses: function(statuses) {		
		// summary:
		//		Set ibm.tivoli.navmodel.Status[] objects indicating the item status values
		this._status = statuses;
	},

	supportsProperties: function() {		
		// summary:
		//		Indicates if "Properties" is supported for the item
		return (this._item.supportsProperties !== undefined ? this._item.supportsProperties : true);
	},

	setSupportsProperties: function(supportsProperties) {		
		// summary:
		//		Set whether item  supports "Properties"
		this._item.supportsProperties = supportsProperties;
	},

	hasCachedMetadata: function() {		
		// summary:
		//		Returns true if item has all properties and associated metadata cached
		return this._allMetadata;
	},

	hasAllCachedProperties: function() {		
		// summary:
		//		Returns true if item has all properties cached
		return this._allProps;
	},

	hasCachedProperty: function(/*String*/ propId) {
		// summary:
		//		Returns true if we have the requested property in the cache
		// propId
		//		property id
		return (this._propsById[propId] ? true : false);
	},
	
	hasCachedProperties: function(/*String[]*/ properties) {		
		// summary:
		//		Returns true if we have all the requested properties in the cache
		// properties:
		//		array of property ids
		if (properties) {
			var propsLength = properties.length;
			
			// asking if we have "all" properties
			if (propsLength === 1 && properties[0] === "all") {
				return this._allProps;
			}
			
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				if (!this._propsById[properties[propIndex]]) {
					return false;
				}
			}
		}
		
		return true;
	},

	getCachedPropertyIds: function(expandAll) {		
		// summary:
		//		Returns array of property Ids currently cached in the item
		var propIds = [];
		
		// have all the properties
		if (!expandAll && this._allProps) {
			propIds.push("all");
		}
		// get individual property ids	
		else if (this._props) {
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				propIds.push(this._props[propIndex].getId());
			}
		}
		return propIds;
	},
	
	/*=====
	properties_onComplete: function(properties, kwArgs){},
	=====*/
	
	/*=====
	_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	getProperties_kwArgs: function(onComplete, onError, properties, metadata, sync) {
		// onComplete: properties_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		property ids to fetch or "all"
		// metadata: Boolean
		//		indicates whether metadata is required
		// sync: Boolean
		//		indicates if call is synchronous or asynchronous
	},
	=====*/

	getProperties: function(/*getProperties_kwArgs*/ kwArgs) {	
		// summary:
		//		Asynchronously returns properties for the item
		var args = {                
			onComplete: function(properties, args){
			},
			onError: function(error, exception, ioArgs){				
			},                
			properties: ["all"],
			metadata: false,
			sync: false
		};   
		dojo.mixin(args, kwArgs);         
		
		// if expired mixin from dataset cache
		this._validateExpiration(kwArgs);
				
		// fetch all properties
		if (args.properties.length === 1 && args.properties[0] === "all") {
			if (this._allProps && (!args.metadata || this._allMetadata)) {				
				if (args.sync) {
					args.onComplete(this._props, args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, this._props, args), 0);
				}		
			}
			else {
				this._fetchProperties(args);
			}
		}		
		// fetch selective properties
		else {
			// look up properties in cache
			var props = [];
			var propsLength = args.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var propId = args.properties[propIndex];
				var prop = this._propsById[propId];				
				if (prop && (!args.metadata || prop.hasMetadata())) {
					props[propIndex] = prop;					
				}
				else {
					break;
				}
			}
			
			// if we found all properties in cache then return them
			if (props.length === propsLength) {
				if (args.sync) {
					args.onComplete(props, args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, props, args), 0);
				}
			}
			// otherwise we need to fetch
			else {
				this._fetchProperties(args);
			}
		}
	},

	setProperties: function(properties) {		
		// summary:
		//		Set the ibm.tivoli.navmodel.Property[] objects for the item
		//		
		//		Note: client side only operation;  no request to rest service
		this._propsById = {};
		this._nullPropIds = {};
		this._props = [];
		
		if (properties) {
			var propsLength = properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var prop = properties[propIndex];
				this._propsById[prop.getId()] = prop;
			}
		}		
	},

	mixinProperties: function(properties, nullPropIds) {		
		// summary:
		//		Mixin the ibm.tivoli.navmodel.Property[] objects for the item
		//		
		//		Note: client side only operation;  no request to rest service
		
		// mixin property objects
		if (properties) {
			var propsLength = properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				this.mixinProperty(properties[propIndex]);
			}					
		}
		
		// mixin any propIds that were null
		if (nullPropIds) {
			for (var propId in nullPropIds) {
				if (nullPropIds.hasOwnProperty(propId)) {
					this._nullPropIds[propId] = true;
				}
			}
		}
	},

	getProperty: function(/*String*/ propId) {		
		// summary:
		//		Synchronously returns requested property
		// propId:
		//		property id to return
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		
		// if expired mixin from dataset cache
		this._validateExpiration();
						
		// check prop cache
		var prop = this._propsById[propId];
		
		if (!prop) {
			// we have asked for this property before and it wasn't found
			if (this._nullPropIds[propId]) {
				return null;
			}
			
			// check default properties on item	
			if (this._item.properties) {
				var propsLength = this._item.properties.length;
				for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
					var rawProp = this._item.properties[propIndex];
					if (rawProp.id === propId) {
						prop = new ibm.tivoli.navmodel.Property({property: rawProp, dataset: this._dataset, objectType: "item"});
						
						// cache property
						this._propsById[propId] = prop;
						break;
					}
				}				
			}
		}	
		
		// fetch it
		if (!prop && !this._allProps) {
			prop = this._fetchProperty(propId);			
		}		
		
		return prop;
	},

	setProperty: function(property) {		
		// summary:
		//		Set ibm.tivoli.navmodel.Property for the item
		//		
		//		Note: client side only operation;  no request to rest service
		var found = this._propsById[property.getId()];		
		if (found) {
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				if (this._props[propIndex].getId() === property.getId()) {
					this._props[propIndex] = property;
					break;
				}
			}
		}
		else {
			this._props.push(property);
		}		
		
		this._propsById[property.getId()] = property;
	},

	mixinProperty: function(property) {		
		// summary:
		//		Mixin ibm.tivoli.navmodel.Property for the item
		//		
		//		Note: client side only operation;  no request to rest service
		var prop = this._propsById[property.getId()];
		if (prop) {
			prop.mixin(property);
		}
		else {
			this._props.push(property);
			this._propsById[property.getId()] = property;
		}	
	},
	
	/*=====
	_onComplete: function(kwArgs){},
	=====*/
	
	/*=====
	editProperties_kwArgs: function(onComplete, onError, properties) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		array of property ids to set
	},
	=====*/

	editProperties: function(/*editProperties_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously modify properties for the item
		var args = {                
			onComplete: function(properties, args){
			},
			onError: function(error, exception, ioArgs){				
			},                
			properties: null
		};
		dojo.mixin(args, kwArgs);
		
		if (args.properties && args.properties.length > 0) {
			var item = dojo.clone(this._item);
			item.properties = [];
			
			var propsLength = args.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				item.properties[propIndex] = args.properties[propIndex].getAttributes();
			}
			
			var json = dojo.toJson(item, false);
			
			dojo.xhrPost ({
				url: ibm.tivoli.navmodel.Utils.getParamUrl(this.getUri(), this._dataset),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version()),
					'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "itemView"),
					'Content-Length': json.length
				},
				handleAs: "json",
				postData: json,
				sync: false,			
				load: dojo.hitch(this, function(result, ioArgs) {
					args.onComplete(args);						
				}),
				error: dojo.hitch(function(error, ioArgs) {
					exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "setProperties", this._trace, this._version());				               
				})
			});	
		}
		// nothing to set
		else {
			setTimeout(dojo.hitch(this, args.onComplete, args), 0);
		}		
	},

	 editProperty: function(/*String*/ property) {		
		// summary:
		//		Synchronously modify property for the item
		// property:
		//		property id to return
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var exception = null;
		
		if (property) {
			var item = dojo.clone(this._item);
			item.properties = [property.getAttributes()];
			
			var json = dojo.toJson(item, false);
			
			dojo.xhrPost ({
				url: ibm.tivoli.navmodel.Utils.getParamUrl(this.getUri(), this._dataset),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version()),
					'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "itemView"),
					'Content-Length': json.length
				},
				handleAs: "json",
				postData: json,
				sync: true,			
				load: dojo.hitch(this, function(result, ioArgs) {						
				}),
				error: dojo.hitch(this, function(error, ioArgs) {
					exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "setProperty", this._trace, this._version());									              
				})
			});				
		}
		
		if (exception) {
			throw exception;
		}
	},

	hasOwnItemStyle: function() {		
		// summary:
		//		Returns true if the item has an ItemStyle that overrides the default
		//		item style for the item type  (i.e. different than the one retrieved
		//		from the dataset)
		return (this._item.style ? true: false);
	},
	
	/*=====
	getItemStyle_kwArgs: function(widgetDef) {
		// widgetDef: Object
		//		widget definition can override attributes
	},
	=====*/

	getItemStyle: function(/*getItemStyle_kwArgs*/ kwArgs) {		
		// summary:
		//		Returns the ibm.tivoli.navmodel.ItemStyle for this item
		var args = {
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		if (!this._style) {
			if (this._item.style) {
				this._style = new ibm.tivoli.navmodel.ItemStyle({style: this._item.style, version: this._version()});
			}
			else {
				this._style = this._dataset.getItemStyle({itemType: this.getType(), widgetDef: null});
			}
		}
		
		// merge widget definition styles 
		var style = this._style;
		if (args.widgetDef) {
			// clone and merge widget definition attributes				
			if (style) {
				style = style.clone(args.widgetDef);
			}
			// unknown style; see if it exists in widget definition
			else {
				var widgetStyle = args.widgetDef.getItemStyle(this.getType());
				if (widgetStyle) {
					style = this._dataset._createItemStyle(args.widgetDef, widgetStyle);
				}
			}			
		}
		
		return style;
	},

	setItemStyle: function(style) {		
		// summary:
		//		Set item style for the item
		this._style = style;
	},

	getDefaultTask: function() {		
		// summary:
		//		Returns optional default task associated with the item
		if (!this._defaultTask && this._item.defaultTask) {
			this._defaultTask = new ibm.tivoli.navmodel.Task({task: this._item.defaultTask, dataset: this._dataset});
		}
		
		return this._defaultTask;
	},

	setDefaultTask: function(defaultTask) {		
		// summary:
		//		Set optional default task associated with the item
		this._defaultTask = defaultTask;
	},

	isOrigin: function() {		
		// summary:
		//		Returns true if the item is an origin of a topology or tree
		return (this._item.origin !== undefined ? this._item.origin : false);
	},

	setOrigin: function(isOrigin) {		
		// summary:
		//		Set the value indicating if the Item is an origin of a topology or tree
		this._item.origin = isOrigin;		
	},
	
	/*=====
	itemsAndLinks_onComplete: function(items, links, datasetItems, kwArgs){},
	=====*/
	
	/*=====
	getRelatedItems_kwArgs: function(onComplete, onError, relationships, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: itemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationships: String
		//		comma delimited list of relationships to determine related items
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		keywords: child, parent, all (if null; value from item will be used)
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getRelatedItems: function(/*getRelatedItems_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously returns items that are related to the requested item
		var args = {                
			onComplete: function(items, links, datasetItems, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			itemId: this.getId(),
			relationships: this.getLazyLoadRelationships(),
			start: 0,
			count: null,
			properties: null,
			metadata: false,
			sort: null,
			condition: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		if (!args.relationships) {
			args.relationships = this.getLazyLoadRelationships();
		}
		
		if (!args.properties) {
			args.properties = this.getCachedPropertyIds();
		}
		
		// call the dataset to get the related items
		this._dataset.getRelatedItems(args);
	},

	isExpired: function() {		
		// summary:
		//		Returns true if the item is currently expired
		return (this._expiration ? false : true);
	},

	checkExpiration: function(nowTime) {		
		// summary:
		//		Check the expiration time on the item; return true if expired
		var expired = false;
		
		// already set to null from a previous check
		if (!this._expiration) {
			expired = true;
		}
		// check agaist the time to see if expired
		else if (nowTime >= this._expiration) {
			this._expiration = null;
			expired = true;
		}
		
		return expired;		
	},

	resetExpiration: function() {		
		// summary:
		//		Rest the expiration time for this item
		if (this._dataset.cacheExpires()) {
			this._expiration = this._dataset.getNewExpirationTime();
		}
		else {
			this._expiration = -1;
		}
	},
			
	/////////////////////////////
	// private methods
	/////////////////////////////

	_version: function() {		
		// summary:
		//		Returns rest version
		return this._dataset.getVersion();
	},

	_validateExpiration: function(kwArgs) {		
		// summary:
		//		If item is expired; mixin with a current item from the dataset cache
		if (this._dataset.cacheExpires()) {
			// expired
			if (this.isExpired()) {
				var properties = null;
				var metadata = false;
				if (kwArgs) {
					properties = kwArgs.properties;
					metadata = kwArgs.metadata;
				}			
				if (!properties) {
					properties = this.getCachedPropertyIds();
				}
		
				var item = this._dataset.getItem({itemId: this.getId(), properties: properties, metadata: metadata});
				if (item) {
					this.mixin(item);
				}			
			}
			// NOT expired; reset expiration since touched
			else {
				this.resetExpiration();
			}
		}
	},
	
	/*=====
	fetchProperties_kwArgs: function(onComplete, onError, properties, metadata, sync) {
		// onComplete: properties_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		property ids to fetch or "all"
		// metadata: Boolean
		//		indicates whether metadata is required
		// sync: Boolean
		//		indicates if call is synchronous or asynchronous	
	},
	=====*/
	
	_fetchProperties: function(/*fetchProperties_kwArgs*/ kwArgs) {	
		// summary:
		//		Asynchronously returns properties for the item
		this._props = [];
		this._allProps = false;
		this._allMetadata = false;
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this._dataset);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		content.optimize = true;
		content.metadata = kwArgs.metadata;
		
		// are we fetching "all" properties
		var fetchAll = (kwArgs.properties.length === 1 && kwArgs.properties[0] === "all");
		
		return dojo.xhrGet({
			url: this.getUri(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "itemView")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var props = [];
				var rawProps = result.properties;
				var rawPropsLength = rawProps.length;
				for (var propIndex = 0;  propIndex < rawPropsLength;  ++propIndex) {
					var prop = new ibm.tivoli.navmodel.Property({property: rawProps[propIndex], dataset: this._dataset, objectType: "item"});
					props[propIndex] = prop;
					
					if (fetchAll) {
						this._props[propIndex] = prop;
					}
					
					this._propsById[prop.getId()] = prop;
				}
				
				if (fetchAll) {
					this._allProps = true;
					this._allMetadata = kwArgs.metadata;
				}
				
				kwArgs.onComplete(props, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_fetchProperties", this._trace, this._version()), ioArgs);			
			})
		});		
	},

	_fetchProperty: function(/*String*/ propId) {		
		// summary:
		//		Synchronously returns requested property
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var exception = null;
		var prop = null;
				
		var content = ibm.tivoli.navmodel.Utils.getContent();		
		ibm.tivoli.navmodel.Utils.mixinParams(content, this._dataset);
		content.optimize = true;
		content.properties = propId;
		
		dojo.xhrGet({
			url: this.getUri(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "itemView")
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				if (result.properties && result.properties.length > 0) {
					prop = new ibm.tivoli.navmodel.Property({property: result.properties[0], dataset: this._dataset, objectType: "item"});
				
					// cache property
					this._propsById[propId] = prop;
				}
				// property was not found; remember so we don't ask again
				else {
					this._nullPropIds[propId] = true;
				}		
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_fetchProperty", this._trace, this._version());				
			})
		});
		
		if (exception) {
			throw exception;
		}
		
		return prop;
	}
});

});

},
'dojox/grid/_View':function(){
define([
	"dojo",
	"dijit/registry",
	"../main",
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/_base/connect",
	"dojo/_base/sniff",
	"dojo/query",
	"dojo/_base/window",
	"dojo/text!./resources/View.html",
	"dojo/dnd/Source",
	"dijit/_Widget",
	"dijit/_TemplatedMixin",
	"dojox/html/metrics",
	"./util",
	"dojo/_base/html",
	"./_Builder",
	"dojo/dnd/Avatar",
	"dojo/dnd/Manager"
], function(dojo, dijit, dojox, declare, array, lang, connect, has, query,
	win, template, Source, _Widget, _TemplatedMixin, metrics, util, html, _Builder, Avatar){

	// a private function
	var getStyleText = function(inNode, inStyleText){
		return inNode.style.cssText == undefined ? inNode.getAttribute("style") : inNode.style.cssText;
	};

	// some public functions
	var _View = declare('dojox.grid._View', [_Widget, _TemplatedMixin], {
		// summary:
		//		A collection of grid columns. A grid is comprised of a set of views that stack horizontally.
		//		Grid creates views automatically based on grid's layout structure.
		//		Users should typically not need to access individual views directly.
		//
		// defaultWidth: String
		//		Default width of the view
		defaultWidth: "18em",

		// viewWidth: String
		// 		Width for the view, in valid css unit
		viewWidth: "",

		templateString: template,
		
		themeable: false,
		classTag: 'dojoxGrid',
		marginBottom: 0,
		rowPad: 2,

		// _togglingColumn: int
		//		Width of the column being toggled (-1 for none)
		_togglingColumn: -1,
		
		// _headerBuilderClass: Object
		//		The class to use for our header builder
		_headerBuilderClass: _Builder._HeaderBuilder,
		
		// _contentBuilderClass: Object
		//		The class to use for our content builder
		_contentBuilderClass: _Builder._ContentBuilder,
		
		postMixInProperties: function(){
			this.rowNodes = {};
		},

		postCreate: function(){
			this.connect(this.scrollboxNode,"onscroll","doscroll");
			util.funnelEvents(this.contentNode, this, "doContentEvent", [ 'mouseover', 'mouseout', 'click', 'dblclick', 'contextmenu', 'mousedown' ]);
			util.funnelEvents(this.headerNode, this, "doHeaderEvent", [ 'dblclick', 'mouseover', 'mouseout', 'mousemove', 'mousedown', 'click', 'contextmenu' ]);
			this.content = new this._contentBuilderClass(this);
			this.header = new this._headerBuilderClass(this);
			//BiDi: in RTL case, style width='9000em' causes scrolling problem in head node
			if(!this.grid.isLeftToRight()){
				this.headerNodeContainer.style.width = "";
			}
		},

		destroy: function(){
			html.destroy(this.headerNode);
			delete this.headerNode;
			for(var i in this.rowNodes){
				this._cleanupRowWidgets(this.rowNodes[i]);
				html.destroy(this.rowNodes[i]);
			}
			this.rowNodes = {};
			if(this.source){
				this.source.destroy();
			}
			this.inherited(arguments);
		},

		// focus
		focus: function(){
			if(has("ie") || has("webkit") || has("opera")){
				this.hiddenFocusNode.focus();
			}else{
				this.scrollboxNode.focus();
			}
		},

		setStructure: function(inStructure){
			var vs = (this.structure = inStructure);
			// FIXME: similar logic is duplicated in layout
			if(vs.width && !isNaN(vs.width)){
				this.viewWidth = vs.width + 'em';
			}else{
				//When noscroll, row table width should not be "auto", or the cell width will be invalid when editing the cell.
				this.viewWidth = vs.width || (vs.noscroll ? '' : this.viewWidth); //|| this.defaultWidth;
			}
			this._onBeforeRow = vs.onBeforeRow||function(){};
			this._onAfterRow = vs.onAfterRow||function(){};
			this.noscroll = vs.noscroll;
			if(this.noscroll){
				this.scrollboxNode.style.overflow = "hidden";
			}
			this.simpleStructure = Boolean(vs.cells.length == 1);
			// bookkeeping
			this.testFlexCells();
			// accomodate new structure
			this.updateStructure();
		},
		
		_cleanupRowWidgets: function(inRowNode){
			// Summary:
			//		Cleans up the widgets for the given row node so that
			//		we can reattach them if needed
			if(inRowNode){
				array.forEach(query("[widgetId]", inRowNode).map(dijit.byNode), function(w){
					if(w._destroyOnRemove){
						w.destroy();
						delete w;
					}else if(w.domNode && w.domNode.parentNode){
						w.domNode.parentNode.removeChild(w.domNode);
					}
				});
			}
		},
		
		onBeforeRow: function(inRowIndex, cells){
			this._onBeforeRow(inRowIndex, cells);
			if(inRowIndex >= 0){
				this._cleanupRowWidgets(this.getRowNode(inRowIndex));
			}
		},
		
		onAfterRow: function(inRowIndex, cells, inRowNode){
			this._onAfterRow(inRowIndex, cells, inRowNode);
			var g = this.grid;
			array.forEach(query(".dojoxGridStubNode", inRowNode), function(n){
				if(n && n.parentNode){
					var lw = n.getAttribute("linkWidget");
					var cellIdx = window.parseInt(html.attr(n, "cellIdx"), 10);
					var cellDef = g.getCell(cellIdx);
					var w = dijit.byId(lw);
					if(w){
						n.parentNode.replaceChild(w.domNode, n);
						if(!w._started){
							w.startup();
						}
						dojo.destroy(n);
					}else{
						n.innerHTML = "";
					}
				}
			}, this);
		},

		testFlexCells: function(){
			// FIXME: cheater, this function does double duty as initializer and tester
			this.flexCells = false;
			for(var j=0, row; (row=this.structure.cells[j]); j++){
				for(var i=0, cell; (cell=row[i]); i++){
					cell.view = this;
					this.flexCells = this.flexCells || cell.isFlex();
				}
			}
			return this.flexCells;
		},

		updateStructure: function(){
			// header builder needs to update table map
			this.header.update();
			// content builder needs to update markup cache
			this.content.update();
		},

		getScrollbarWidth: function(){
			var hasScrollSpace = this.hasVScrollbar();
			var overflow = html.style(this.scrollboxNode, "overflow");
			if(this.noscroll || !overflow || overflow == "hidden"){
				hasScrollSpace = false;
			}else if(overflow == "scroll"){
				hasScrollSpace = true;
			}
			return (hasScrollSpace ? metrics.getScrollbar().w : 0); // Integer
		},

		getColumnsWidth: function(){
			var h = this.headerContentNode;
			return h && h.firstChild ? h.firstChild.offsetWidth : 0; // Integer
		},

		setColumnsWidth: function(width){
			this.headerContentNode.firstChild.style.width = width + 'px';
			if(this.viewWidth){
				this.viewWidth = width + 'px';
			}
		},

		getWidth: function(){
			return this.viewWidth || (this.getColumnsWidth()+this.getScrollbarWidth()) +'px'; // String
		},

		getContentWidth: function(){
			return Math.max(0, html._getContentBox(this.domNode).w - this.getScrollbarWidth()) + 'px'; // String
		},

		render: function(){
			this.scrollboxNode.style.height = '';
			this.renderHeader();
			if(this._togglingColumn >= 0){
				this.setColumnsWidth(this.getColumnsWidth() - this._togglingColumn);
				this._togglingColumn = -1;
			}
			var cells = this.grid.layout.cells;
			var getSibling = lang.hitch(this, function(node, before){
				!this.grid.isLeftToRight() && (before = !before);
				var inc = before?-1:1;
				var idx = this.header.getCellNodeIndex(node) + inc;
				var cell = cells[idx];
				while(cell && cell.getHeaderNode() && cell.getHeaderNode().style.display == "none"){
					idx += inc;
					cell = cells[idx];
				}
				if(cell){
					return cell.getHeaderNode();
				}
				return null;
			});
			if(this.grid.columnReordering && this.simpleStructure){
				if(this.source){
					this.source.destroy();
				}
				
				// Create the top and bottom markers
				var bottomMarkerId = "dojoxGrid_bottomMarker";
				var topMarkerId = "dojoxGrid_topMarker";
				if(this.bottomMarker){
					html.destroy(this.bottomMarker);
				}
				this.bottomMarker = html.byId(bottomMarkerId);
				if(this.topMarker){
					html.destroy(this.topMarker);
				}
				this.topMarker = html.byId(topMarkerId);
				if (!this.bottomMarker) {
					this.bottomMarker = html.create("div", {
						"id": bottomMarkerId,
						"class": "dojoxGridColPlaceBottom"
					}, win.body());
					this._hide(this.bottomMarker);

					
					this.topMarker = html.create("div", {
						"id": topMarkerId,
						"class": "dojoxGridColPlaceTop"
					}, win.body());
					this._hide(this.topMarker);
				}
				this.arrowDim = html.contentBox(this.bottomMarker);

				var headerHeight = html.contentBox(this.headerContentNode.firstChild.rows[0]).h;
				
				this.source = new Source(this.headerContentNode.firstChild.rows[0], {
					horizontal: true,
					accept: [ "gridColumn_" + this.grid.id ],
					viewIndex: this.index,
					generateText: false,
					onMouseDown: lang.hitch(this, function(e){
						this.header.decorateEvent(e);
						if((this.header.overRightResizeArea(e) || this.header.overLeftResizeArea(e)) &&
							this.header.canResize(e) && !this.header.moveable){
							this.header.beginColumnResize(e);
						}else{
							if(this.grid.headerMenu){
								this.grid.headerMenu.onCancel(true);
							}
							// IE reports a left click as 1, where everything else reports 0
							if(e.button === (has("ie") < 9 ? 1 : 0)){
								Source.prototype.onMouseDown.call(this.source, e);
							}
						}
					}),
					onMouseOver: lang.hitch(this, function(e){
						var src = this.source;
						if(src._getChildByEvent(e)){
							Source.prototype.onMouseOver.apply(src, arguments);
						}
					}),
					_markTargetAnchor: lang.hitch(this, function(before){
						var src = this.source;
						if(src.current == src.targetAnchor && src.before == before){ return; }
						if(src.targetAnchor && getSibling(src.targetAnchor, src.before)){
							src._removeItemClass(getSibling(src.targetAnchor, src.before), src.before ? "After" : "Before");
						}
						Source.prototype._markTargetAnchor.call(src, before);
						
						var target = before ? src.targetAnchor : getSibling(src.targetAnchor, src.before);
						var endAdd = 0;

						if (!target) {
							target = src.targetAnchor;
							endAdd = html.contentBox(target).w + this.arrowDim.w/2 + 2;
						}

						var pos = html.position(target, true);
						var left = Math.floor(pos.x - this.arrowDim.w/2 + endAdd);

						html.style(this.bottomMarker, "visibility", "visible");
						html.style(this.topMarker, "visibility", "visible");
						html.style(this.bottomMarker, {
							"left": left + "px",
							"top" : (headerHeight + pos.y) + "px"
						});

						html.style(this.topMarker, {
							"left": left + "px",
							"top" : (pos.y - this.arrowDim.h) + "px"
						});

						if(src.targetAnchor && getSibling(src.targetAnchor, src.before)){
							src._addItemClass(getSibling(src.targetAnchor, src.before), src.before ? "After" : "Before");
						}
					}),
					_unmarkTargetAnchor: lang.hitch(this, function(){
						var src = this.source;
						if(!src.targetAnchor){ return; }
						if(src.targetAnchor && getSibling(src.targetAnchor, src.before)){
							src._removeItemClass(getSibling(src.targetAnchor, src.before), src.before ? "After" : "Before");
						}
						this._hide(this.bottomMarker);
						this._hide(this.topMarker);
						Source.prototype._unmarkTargetAnchor.call(src);
					}),
					destroy: lang.hitch(this, function(){
						connect.disconnect(this._source_conn);
						connect.unsubscribe(this._source_sub);
						Source.prototype.destroy.call(this.source);
						if(this.bottomMarker){
							html.destroy(this.bottomMarker);
							delete this.bottomMarker;
						}
						if(this.topMarker){
							html.destroy(this.topMarker);
							delete this.topMarker;
						}
					}),
					onDndCancel: lang.hitch(this, function(){
						Source.prototype.onDndCancel.call(this.source);
						this._hide(this.bottomMarker);
						this._hide(this.topMarker);
					})
				});

				this._source_conn = connect.connect(this.source, "onDndDrop", this, "_onDndDrop");
				this._source_sub = connect.subscribe("/dnd/drop/before", this, "_onDndDropBefore");
				this.source.startup();
			}
		},
		
		_hide: function(node){
			html.style(node, {
				left: "-10000px",
				top: "-10000px",
				"visibility": "hidden"
			});
		},

		_onDndDropBefore: function(source, nodes, copy){
			if(dojo.dnd.manager().target !== this.source){
				return;
			}
			this.source._targetNode = this.source.targetAnchor;
			this.source._beforeTarget = this.source.before;
			var views = this.grid.views.views;
			var srcView = views[source.viewIndex];
			var tgtView = views[this.index];
			if(tgtView != srcView){
				srcView.convertColPctToFixed();
				tgtView.convertColPctToFixed();
			}
		},

		_onDndDrop: function(source, nodes, copy){
			if(dojo.dnd.manager().target !== this.source){
				if(dojo.dnd.manager().source === this.source){
					this._removingColumn = true;
				}
				return;
			}
			this._hide(this.bottomMarker);
			this._hide(this.topMarker);

			var getIdx = function(n){
				return n ? html.attr(n, "idx") : null;
			};
			var w = html.marginBox(nodes[0]).w;
			if(source.viewIndex !== this.index){
				var views = this.grid.views.views;
				var srcView = views[source.viewIndex];
				var tgtView = views[this.index];
				if(srcView.viewWidth && srcView.viewWidth != "auto"){
					srcView.setColumnsWidth(srcView.getColumnsWidth() - w);
				}
				if(tgtView.viewWidth && tgtView.viewWidth != "auto"){
					tgtView.setColumnsWidth(tgtView.getColumnsWidth());
				}
			}
			var stn = this.source._targetNode;
			var stb = this.source._beforeTarget;
			!this.grid.isLeftToRight() && (stb = !stb);
			var layout = this.grid.layout;
			var idx = this.index;
			delete this.source._targetNode;
			delete this.source._beforeTarget;
			
			layout.moveColumn(
				source.viewIndex,
				idx,
				getIdx(nodes[0]),
				getIdx(stn),
				stb);
		},

		renderHeader: function(){
			this.headerContentNode.innerHTML = this.header.generateHtml(this._getHeaderContent);
			if(this.flexCells){
				this.contentWidth = this.getContentWidth();
				this.headerContentNode.firstChild.style.width = this.contentWidth;
			}
			util.fire(this, "onAfterRow", [-1, this.structure.cells, this.headerContentNode]);
		},

		// note: not called in 'view' context
		_getHeaderContent: function(inCell){
			var n = inCell.name || inCell.grid.getCellName(inCell);
			if(/^\s+$/.test(n)){
				n = '&nbsp;'//otherwise arrow styles will be messed up
			}
			var ret = [ '<div class="dojoxGridSortNode' ];
			
			if(inCell.index != inCell.grid.getSortIndex()){
				ret.push('">');
			}else{
				ret = ret.concat([ ' ',
							inCell.grid.sortInfo > 0 ? 'dojoxGridSortUp' : 'dojoxGridSortDown',
							'"><div class="dojoxGridArrowButtonChar">',
							inCell.grid.sortInfo > 0 ? '&#9650;' : '&#9660;',
							'</div><div class="dojoxGridArrowButtonNode" role="presentation"></div>',
							'<div class="dojoxGridColCaption">']);
			}
			ret = ret.concat([n, '</div></div>']);
			return ret.join('');
		},

		resize: function(){
			this.adaptHeight();
			this.adaptWidth();
		},

		hasHScrollbar: function(reset){
			var hadScroll = this._hasHScroll||false;
			if(this._hasHScroll == undefined || reset){
				if(this.noscroll){
					this._hasHScroll = false;
				}else{
					var style = html.style(this.scrollboxNode, "overflow");
					if(style == "hidden"){
						this._hasHScroll = false;
					}else if(style == "scroll"){
						this._hasHScroll = true;
					}else{
						this._hasHScroll = (this.scrollboxNode.offsetWidth - this.getScrollbarWidth() < this.contentNode.offsetWidth );
					}
				}
			}
			if(hadScroll !== this._hasHScroll){
				this.grid.update();
			}
			return this._hasHScroll; // Boolean
		},

		hasVScrollbar: function(reset){
			var hadScroll = this._hasVScroll||false;
			if(this._hasVScroll == undefined || reset){
				if(this.noscroll){
					this._hasVScroll = false;
				}else{
					var style = html.style(this.scrollboxNode, "overflow");
					if(style == "hidden"){
						this._hasVScroll = false;
					}else if(style == "scroll"){
						this._hasVScroll = true;
					}else{
						this._hasVScroll = (this.scrollboxNode.scrollHeight > this.scrollboxNode.clientHeight);
					}
				}
			}
			if(hadScroll !== this._hasVScroll){
				this.grid.update();
			}
			return this._hasVScroll; // Boolean
		},
		
		convertColPctToFixed: function(){
			// Fix any percentage widths to be pixel values
			var hasPct = false;
			this.grid.initialWidth = "";
			var cellNodes = query("th", this.headerContentNode);
			var fixedWidths = array.map(cellNodes, function(c, vIdx){
				var w = c.style.width;
				html.attr(c, "vIdx", vIdx);
				if(w && w.slice(-1) == "%"){
					hasPct = true;
				}else if(w && w.slice(-2) == "px"){
					return window.parseInt(w, 10);
				}
				return html.contentBox(c).w;
			});
			if(hasPct){
				array.forEach(this.grid.layout.cells, function(cell, idx){
					if(cell.view == this){
						var cellNode = cell.view.getHeaderCellNode(cell.index);
						if(cellNode && html.hasAttr(cellNode, "vIdx")){
							var vIdx = window.parseInt(html.attr(cellNode, "vIdx"));
							this.setColWidth(idx, fixedWidths[vIdx]);
							html.removeAttr(cellNode, "vIdx");
						}
					}
				}, this);
				return true;
			}
			return false;
		},

		adaptHeight: function(minusScroll){
			if(!this.grid._autoHeight){
				var h = (this.domNode.style.height && parseInt(this.domNode.style.height.replace(/px/,''), 10)) || this.domNode.clientHeight;
				var self = this;
				var checkOtherViewScrollers = function(){
					var v;
					for(var i in self.grid.views.views){
						v = self.grid.views.views[i];
						if(v !== self && v.hasHScrollbar()){
							return true;
						}
					}
					return false;
				};
				if(minusScroll || (this.noscroll && checkOtherViewScrollers())){
					h -= metrics.getScrollbar().h;
				}
				util.setStyleHeightPx(this.scrollboxNode, h);
			}
			this.hasVScrollbar(true);
		},

		adaptWidth: function(){
			if(this.flexCells){
				// the view content width
				this.contentWidth = this.getContentWidth();
				this.headerContentNode.firstChild.style.width = this.contentWidth;
			}
			// FIXME: it should be easier to get w from this.scrollboxNode.clientWidth,
			// but clientWidth seemingly does not include scrollbar width in some cases
			var w = this.scrollboxNode.offsetWidth - this.getScrollbarWidth();
			if(!this._removingColumn){
				w = Math.max(w, this.getColumnsWidth()) + 'px';
			}else{
				w = Math.min(w, this.getColumnsWidth()) + 'px';
				this._removingColumn = false;
			}
			var cn = this.contentNode;
			cn.style.width = w;
			this.hasHScrollbar(true);
		},

		setSize: function(w, h){
			var ds = this.domNode.style;
			var hs = this.headerNode.style;

			if(w){
				ds.width = w;
				hs.width = w;
			}
			ds.height = (h >= 0 ? h + 'px' : '');
		},

		renderRow: function(inRowIndex){
			var rowNode = this.createRowNode(inRowIndex);
			this.buildRow(inRowIndex, rowNode);
			//this.grid.edit.restore(this, inRowIndex);
			return rowNode;
		},

		createRowNode: function(inRowIndex){
			var node = document.createElement("div");
			node.className = this.classTag + 'Row';
			if (this instanceof dojox.grid._RowSelector){
				html.attr(node,"role","presentation");
			}else{
				html.attr(node,"role","row");
				if (this.grid.selectionMode != "none") {
					node.setAttribute("aria-selected", "false"); //rows can be selected so add aria-selected prop
				}
			}
			node[util.gridViewTag] = this.id;
			node[util.rowIndexTag] = inRowIndex;
			this.rowNodes[inRowIndex] = node;
			return node;
		},

		buildRow: function(inRowIndex, inRowNode){
			
			this.buildRowContent(inRowIndex, inRowNode);
		  	
			this.styleRow(inRowIndex, inRowNode);
		  
		 
		},

		buildRowContent: function(inRowIndex, inRowNode){
			inRowNode.innerHTML = this.content.generateHtml(inRowIndex, inRowIndex);
			if(this.flexCells && this.contentWidth){
				// FIXME: accessing firstChild here breaks encapsulation
				inRowNode.firstChild.style.width = this.contentWidth;
			}
			util.fire(this, "onAfterRow", [inRowIndex, this.structure.cells, inRowNode]);
		},

		rowRemoved:function(inRowIndex){
			if(inRowIndex >= 0){
				this._cleanupRowWidgets(this.getRowNode(inRowIndex));
			}
			this.grid.edit.save(this, inRowIndex);
			delete this.rowNodes[inRowIndex];
		},

		getRowNode: function(inRowIndex){
			return this.rowNodes[inRowIndex];
		},

		getCellNode: function(inRowIndex, inCellIndex){
			var row = this.getRowNode(inRowIndex);
			if(row){
				return this.content.getCellNode(row, inCellIndex);
			}
		},

		getHeaderCellNode: function(inCellIndex){
			if(this.headerContentNode){
				return this.header.getCellNode(this.headerContentNode, inCellIndex);
			}
		},

		// styling
		styleRow: function(inRowIndex, inRowNode){
			inRowNode._style = getStyleText(inRowNode);
			this.styleRowNode(inRowIndex, inRowNode);
		},

		styleRowNode: function(inRowIndex, inRowNode){
			if(inRowNode){
				this.doStyleRowNode(inRowIndex, inRowNode);
			}
		},

		doStyleRowNode: function(inRowIndex, inRowNode){
			this.grid.styleRowNode(inRowIndex, inRowNode);
		},

		// updating
		updateRow: function(inRowIndex){
			var rowNode = this.getRowNode(inRowIndex);
			if(rowNode){
				rowNode.style.height = '';
				this.buildRow(inRowIndex, rowNode);
			}
			return rowNode;
		},

		updateRowStyles: function(inRowIndex){
			this.styleRowNode(inRowIndex, this.getRowNode(inRowIndex));
		},

		// scrolling
		lastTop: 0,
		firstScroll:0,

		doscroll: function(inEvent){
			//var s = dojo.marginBox(this.headerContentNode.firstChild);
			var isLtr = this.grid.isLeftToRight();
			if(this.firstScroll < 2){
				if((!isLtr && this.firstScroll == 1) || (isLtr && this.firstScroll === 0)){
					var s = html.marginBox(this.headerNodeContainer);
					if(has("ie")){
						this.headerNodeContainer.style.width = s.w + this.getScrollbarWidth() + 'px';
					}else if(has("mozilla")){
						//TODO currently only for FF, not sure for safari and opera
						this.headerNodeContainer.style.width = s.w - this.getScrollbarWidth() + 'px';
						//this.headerNodeContainer.style.width = s.w + 'px';
						//set scroll to right in FF
						this.scrollboxNode.scrollLeft = isLtr ?
							this.scrollboxNode.clientWidth - this.scrollboxNode.scrollWidth :
							this.scrollboxNode.scrollWidth - this.scrollboxNode.clientWidth;
					}
				}
				this.firstScroll++;
			}
			this.headerNode.scrollLeft = this.scrollboxNode.scrollLeft;
			// 'lastTop' is a semaphore to prevent feedback-loop with setScrollTop below
			var top = this.scrollboxNode.scrollTop;
			if(top !== this.lastTop){
				this.grid.scrollTo(top);
			}
		},

		setScrollTop: function(inTop){
			// 'lastTop' is a semaphore to prevent feedback-loop with doScroll above
			this.lastTop = inTop;
			this.scrollboxNode.scrollTop = inTop;
			return this.scrollboxNode.scrollTop;
		},

		// event handlers (direct from DOM)
		doContentEvent: function(e){
			if(this.content.decorateEvent(e)){
				this.grid.onContentEvent(e);
			}
		},

		doHeaderEvent: function(e){
			if(this.header.decorateEvent(e)){
				this.grid.onHeaderEvent(e);
			}
		},

		// event dispatch(from Grid)
		dispatchContentEvent: function(e){
			return this.content.dispatchEvent(e);
		},

		dispatchHeaderEvent: function(e){
			return this.header.dispatchEvent(e);
		},

		// column resizing
		setColWidth: function(inIndex, inWidth){
			this.grid.setCellWidth(inIndex, inWidth + 'px');
		},

		update: function(){
			if(!this.domNode){
				return;
			}
			this.content.update();
			this.grid.update();
			//get scroll after update or scroll left setting goes wrong on IE.
			//See trac: #8040
			var left = this.scrollboxNode.scrollLeft;
			this.scrollboxNode.scrollLeft = left;
			this.headerNode.scrollLeft = left;
		}
	});

	var _GridAvatar = declare("dojox.grid._GridAvatar", Avatar, {
		construct: function(){
			var dd = win.doc;

			var a = dd.createElement("table");
			a.cellPadding = a.cellSpacing = "0";
			a.className = "dojoxGridDndAvatar";
			a.style.position = "absolute";
			a.style.zIndex = 1999;
			a.style.margin = "0px"; // to avoid dojo.marginBox() problems with table's margins
			var b = dd.createElement("tbody");
			var tr = dd.createElement("tr");
			var td = dd.createElement("td");
			var img = dd.createElement("td");
			tr.className = "dojoxGridDndAvatarItem";
			img.className = "dojoxGridDndAvatarItemImage";
			img.style.width = "16px";
			var source = this.manager.source, node;
			if(source.creator){
				// create an avatar representation of the node
				node = source._normalizedCreator(source.getItem(this.manager.nodes[0].id).data, "avatar").node;
			}else{
				// or just clone the node and hope it works
				node = this.manager.nodes[0].cloneNode(true);
				var table, tbody;
				if(node.tagName.toLowerCase() == "tr"){
					// insert extra table nodes
					table = dd.createElement("table");
					tbody = dd.createElement("tbody");
					tbody.appendChild(node);
					table.appendChild(tbody);
					node = table;
				}else if(node.tagName.toLowerCase() == "th"){
					// insert extra table nodes
					table = dd.createElement("table");
					tbody = dd.createElement("tbody");
					var r = dd.createElement("tr");
					table.cellPadding = table.cellSpacing = "0";
					r.appendChild(node);
					tbody.appendChild(r);
					table.appendChild(tbody);
					node = table;
				}
			}
			node.id = "";
			td.appendChild(node);
			tr.appendChild(img);
			tr.appendChild(td);
			html.style(tr, "opacity", 0.9);
			b.appendChild(tr);

			a.appendChild(b);
			this.node = a;

			var m = dojo.dnd.manager();
			this.oldOffsetY = m.OFFSET_Y;
			m.OFFSET_Y = 1;
		},
		destroy: function(){
			dojo.dnd.manager().OFFSET_Y = this.oldOffsetY;
			this.inherited(arguments);
		}
	});

	var oldMakeAvatar = dojo.dnd.manager().makeAvatar;
	dojo.dnd.manager().makeAvatar = function(){
		var src = this.source;
		if(src.viewIndex !== undefined && !html.hasClass(win.body(),"dijit_a11y")){
			return new _GridAvatar(this);
		}
		return oldMakeAvatar.call(dojo.dnd.manager());
	};

	return _View;

});
},
'gridx/core/model/extensions/ClientFilter':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/_base/Deferred",
	'../_Extension'
], function(declare, array, lang, Deferred, _Extension){

	var hitch = lang.hitch,
		forEach = array.forEach,
		indexOf = array.indexOf;

	return declare(/*===== "gridx.core.model.extensions.ClientFilter", =====*/_Extension, {
		// Not compatible with Map extension!
		name: 'clientFilter',

		priority: 20,

		constructor: function(model, args){
			this.pageSize = args.pageSize || 100;
			this._mixinAPI('filter', 'hasFilter');
			model.onFilterProgress = function(){};
			this.aspect(model, '_msg', '_receiveMsg');
			this.aspect(model, 'setStore', 'clear');
		},

		//Public---------------------------------------------------------------------

		//pageSize: 100,

		clear: function(){
			this._ids = 0;
			this._indexes = {};
		},

		filter: function(checker){
			this.model._addCmd({
				name: '_cmdFilter',
				scope: this,
				args: arguments,
				async: 1
			});
		},

		hasFilter: function(){
			return !!this._ids;
		},

		byIndex: function(index){
			var t = this,
				ids = t._ids,
				inner = t.inner,
				id = ids && ids[index];
			return ids ? t.model.isId(id) && inner._call('byId', [id]) : inner._call('byIndex', arguments);
		},

		byId: function(id){
			return (this.ids && this._indexes[id] === undefined) ? null : this.inner._call('byId', arguments);
		},

		indexToId: function(index){
			return this._ids ? this._ids[index] : this.inner._call('indexToId', arguments);
		},

		idToIndex: function(id){
			if(this._ids){
				var idx = indexOf(this._ids, id);
				return idx >= 0 ? idx : undefined;
			}
			return this.inner._call('idToIndex', arguments);
		},

		size: function(){
			return this._ids ? this._ids.length : this.inner._call('size', arguments);
		},

		when: function(args, callback){
			var t = this,
				f = function(){
					if(t._ids){
						t._mapWhenArgs(args);
					}
					return t.inner._call('when', [args, callback]);
				};
			if(t._refilter){
				t._refilter = 0;
				if(t._ids){
					var d = new Deferred();
					t._reFilter().then(function(){
						f().then(hitch(d, d.callback), hitch(d, d.errback));
					});
					return d;
				}
			}
			return f();
		},

		//Private---------------------------------------------------------------------
		_cmdFilter: function(){
			var a = arguments;
			return this._filter.apply(this, a[a.length - 1]);
		},

		_filter: function(checker){
			var t = this,
				oldSize = t.size();
			t.clear();
			if(lang.isFunction(checker)){
				var ids = [];
				return t.model.scan({
					start: 0,
					pageSize: t.pageSize,
					whenScope: t,
					whenFunc: t.when
				}, function(rows, s){
					var i, id, row,
						end = s + rows.length;
					for(i = s; i < end; ++i){
						id = t.indexToId(i);
						row = t.byIndex(i);
						if(row){
							if(checker(row, id)){
								ids.push(id);
								t._indexes[id] = i;
							}
						}else{
							break;
						}
					}
				}).then(function(){
					if(ids.length == t.size()){
						//Filtered item size equals cache size, so filter is useless.
						t.clear();
					}else{
						t._ids = ids;
						t.model._msg('filter', ids);
					}
				}, 0, t.model.onFilterProgress);
			}else{
				var d = new Deferred();
				d.callback();
				return d;
			}
		},

		_mapWhenArgs: function(args){
			//Map ids and index ranges to what the store needs.
			var t = this, ranges = [], size = t._ids.length;
			args.id = array.filter(args.id, function(id){
				return t._indexes[id] !== undefined;
			});
			forEach(args.range, function(r){
				if(!r.count || r.count < 0){
					//For open ranges, must limit the size because we know the filtered size here.
					var cnt = size - r.start;
					if(cnt <= 0){
						return;
					}
					r.count = cnt;
				}
				for(var i = 0; i < r.count; ++i){
					var idx = t._mapIndex(i + r.start);
					if(idx !== undefined){
						ranges.push({
							start: idx,
							count: 1
						});
					}
				}
			});
			args.range = ranges;
		},

		_mapMoveArgs: function(args){
			var t = this;
			if(args.length == 3){
				var indexes = [];
				for(var i = args[0], end = args[0] + args[1]; i < end; ++i){
					indexes.push(t._mapIndex(i));
				}
				args[0] = indexes;
				args[1] = t._mapIndex(args[2]);
				args.pop();
			}else{
				args[0] = array.map(args[0], function(index){
					return t._mapIndex(index);
				});
				args[1] = t._mapIndex(args[1]);
			}
		},

		_mapIndex: function(index){
			return this._indexes[this._ids[index]];
		},

		_moveFiltered: function(start, count, target){
			var t = this, size = t._ids.length;
			if(start >= 0 && start < size && 
				count > 0 && count < Infinity && 
				target >= 0 && target < size && 
				(target < start || target > start + count)){

				var i, len, indexes = [];
				for(i = start, len = start + count; i < len; ++i){
					indexes.push(t._mapIndex(i));
				}
				t.inner._call('moveIndexes', [indexes, t._mapIndex(target)]);
			}
		},

		_reFilter: function(){
			var t = this;
			return t.inner._call('when', [{
				id: t._ids,
				range: []
			}, function(){
				forEach(t._ids, function(id){
					var idx = t.inner._call('idToIndex', [id]);
					t._indexes[id] = idx;
				});
				t._ids.sort(function(a, b){
					return t._indexes[a] - t._indexes[b];
				});
			}]);
		},

		_onMoved: function(map){
			var t = this;
			forEach(t._ids, function(id){
				var oldIdx = t._indexes[id];
				if(map[oldIdx] !== undefined){
					t._indexes[id] = map[oldIdx];
				}
			});
			t._ids.sort(function(a, b){
				return t._indexes[a] - t._indexes[b];
			});
		},

		_receiveMsg: function(msg, args){
			var t = this;
			if(t._ids){
				if(msg == 'storeChange'){
					t._refilter = 1;
				}else if(msg == 'moved'){
					t._onMoved(args);
				}else if(msg == 'beforeMove'){
					t._mapMoveArgs(args);
				}
			}
		},

		_onNew: function(id){
			var t = this;
			if(t._ids){
				t._ids.push(id);
				t._refilter = 1;
			}
			t.onNew.apply(t, arguments);
		},

		_onDelete: function(id, index, row){
			var t = this, indexes = t._indexes, ids = t._ids;
			if(ids){
				var i = indexOf(ids, id),
					idx = indexes[id];
				if(i >= 0){
					ids.splice(i, 1);
				}
				if(i >= 0 && idx !== undefined){
					index = i;
					for(i in indexes){
						if(indexes[i] > idx){
							--indexes[i];
						}
					}
				}else{
					index = undefined;
					t._refilter = 1;
				}
			}
			t.onDelete(id, index, row);
		}
	});
});

},
'dijit/hccss':function(){
define([
	"require",			// require.toUrl
	"dojo/_base/config", // config.blankGif
	"dojo/dom-class", // domClass.add domConstruct.create domStyle.getComputedStyle
	"dojo/dom-construct", // domClass.add domConstruct.create domStyle.getComputedStyle
	"dojo/dom-style", // domClass.add domConstruct.create domStyle.getComputedStyle
	"dojo/ready", // ready
	"dojo/_base/sniff", // has("ie") has("mozilla")
	"dojo/_base/window" // win.body
], function(require, config, domClass, domConstruct, domStyle, ready, has, win){

	// module:
	//		dijit/hccss
	// summary:
	//		Test if computer is in high contrast mode, and sets dijit_a11y flag on <body> if it is.

	if(has("ie") || has("mozilla")){	// NOTE: checking in Safari messes things up
		// priority is 90 to run ahead of parser priority of 100
		ready(90, function(){
			// summary:
			//		Detects if we are in high-contrast mode or not

			// create div for testing if high contrast mode is on or images are turned off
			var div = domConstruct.create("div",{
				id: "a11yTestNode",
				style:{
					cssText:'border: 1px solid;'
						+ 'border-color:red green;'
						+ 'position: absolute;'
						+ 'height: 5px;'
						+ 'top: -999px;'
						+ 'background-image: url("' + (config.blankGif || require.toUrl("dojo/resources/blank.gif")) + '");'
				}
			}, win.body());

			// test it
			var cs = domStyle.getComputedStyle(div);
			if(cs){
				var bkImg = cs.backgroundImage;
				var needsA11y = (cs.borderTopColor == cs.borderRightColor) || (bkImg != null && (bkImg == "none" || bkImg == "url(invalid-url:)" ));
				if(needsA11y){
					domClass.add(win.body(), "dijit_a11y");
				}
				if(has("ie")){
					div.outerHTML = "";		// prevent mixed-content warning, see http://support.microsoft.com/kb/925014
				}else{
					win.body().removeChild(div);
				}
			}
		});
	}
});

},
'ibm/tivoli/tip/navwidget/dijit/NavWidgetTabContainerPluggableView':function(){
// wrapped by build app
define("ibm/tivoli/tip/navwidget/dijit/NavWidgetTabContainerPluggableView", ["dijit","dojo","dojox","dojo/require!dijit/layout/BorderContainer,dijit/_Widget,ibm/tivoli/navmodel/Trace,ibm/tivoli/tip/navwidget/dijit/NavWidgetPluggableView,dijit/layout/TabContainer,dijit/layout/ContentPane"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide ("ibm.tivoli.tip.navwidget.dijit.NavWidgetTabContainerPluggableView");

dojo.require ("dijit.layout.BorderContainer");
dojo.require("dijit._Widget");
dojo.require("ibm.tivoli.navmodel.Trace");
dojo.require("ibm.tivoli.tip.navwidget.dijit.NavWidgetPluggableView");
dojo.require("dijit.layout.TabContainer");
dojo.require("dijit.layout.ContentPane");

ibm.tivoli.tip.navwidget.dijit.NavWidgetTabContainerPluggableView._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.tip.navwidget.dijit.NavWidgetTabContainerPluggableView");

dojo.declare ("ibm.tivoli.tip.navwidget.dijit.NavWidgetTabContainerPluggableView",
	[dijit.layout.TabContainer, ibm.tivoli.tip.navwidget.dijit.NavWidgetPluggableView],
	{
		// summary:
		//		Base class for a view that wants to be in a tab container
		// description:
		//		Coming soon
	
		// _trace: [private] Trace
		//		Utility for trace and logging
	    _trace: ibm.tivoli.tip.navwidget.dijit.NavWidgetTabContainerPluggableView._trace,

		postCreate: function(){		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
	
			//Call superclass
			this.inherited(arguments);
		},
		
		startup: function(){		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			this.inherited(arguments);
		},
		
		
		destroy: function(){		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
            this.inherited(arguments);
		}
});

});

},
'dijit/PopupMenuItem':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom-style", // domStyle.set
	"dojo/query", // query
	"dojo/_base/window", // win.body
	"./registry",	// registry.byNode
	"./MenuItem",
	"./hccss"
], function(declare, domStyle, query, win, registry, MenuItem){

/*=====
	var MenuItem = dijit.MenuItem;
=====*/

	// module:
	//		dijit/PopupMenuItem
	// summary:
	//		An item in a Menu that spawn a drop down (usually a drop down menu)

	return declare("dijit.PopupMenuItem", MenuItem, {
		// summary:
		//		An item in a Menu that spawn a drop down (usually a drop down menu)

		_fillContent: function(){
			// summary:
			//		When Menu is declared in markup, this code gets the menu label and
			//		the popup widget from the srcNodeRef.
			// description:
			//		srcNodeRefinnerHTML contains both the menu item text and a popup widget
			//		The first part holds the menu item text and the second part is the popup
			// example:
			// |	<div data-dojo-type="dijit.PopupMenuItem">
			// |		<span>pick me</span>
			// |		<popup> ... </popup>
			// |	</div>
			// tags:
			//		protected

			if(this.srcNodeRef){
				var nodes = query("*", this.srcNodeRef);
				this.inherited(arguments, [nodes[0]]);

				// save pointer to srcNode so we can grab the drop down widget after it's instantiated
				this.dropDownContainer = this.srcNodeRef;
			}
		},

		startup: function(){
			if(this._started){ return; }
			this.inherited(arguments);

			// we didn't copy the dropdown widget from the this.srcNodeRef, so it's in no-man's
			// land now.  move it to win.doc.body.
			if(!this.popup){
				var node = query("[widgetId]", this.dropDownContainer)[0];
				this.popup = registry.byNode(node);
			}
			win.body().appendChild(this.popup.domNode);
			this.popup.startup();

			this.popup.domNode.style.display="none";
			if(this.arrowWrapper){
				domStyle.set(this.arrowWrapper, "visibility", "");
			}
			this.focusNode.setAttribute("aria-haspopup", "true");
		},

		destroyDescendants: function(/*Boolean*/ preserveDom){
			if(this.popup){
				// Destroy the popup, unless it's already been destroyed.  This can happen because
				// the popup is a direct child of <body> even though it's logically my child.
				if(!this.popup._destroyed){
					this.popup.destroyRecursive(preserveDom);
				}
				delete this.popup;
			}
			this.inherited(arguments);
		}
	});
});

},
'dijit/form/_FormSelectWidget':function(){
define([
	"dojo/_base/array", // array.filter array.forEach array.map array.some
	"dojo/aspect", // aspect.after
	"dojo/data/util/sorter", // util.sorter.createSortFunction
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.setSelectable
	"dojo/dom-class", // domClass.toggle
	"dojo/_base/kernel",	// _scopeName
	"dojo/_base/lang", // lang.delegate lang.isArray lang.isObject lang.hitch
	"dojo/query", // query
	"./_FormValueWidget"
], function(array, aspect, sorter, declare, dom, domClass, kernel, lang, query, _FormValueWidget){

/*=====
	var _FormValueWidget = dijit.form._FormValueWidget;
=====*/

// module:
//		dijit/form/_FormSelectWidget
// summary:
//		Extends _FormValueWidget in order to provide "select-specific"
//		values - i.e., those values that are unique to <select> elements.


/*=====
dijit.form.__SelectOption = function(){
	// value: String
	//		The value of the option.  Setting to empty (or missing) will
	//		place a separator at that location
	// label: String
	//		The label for our option.  It can contain html tags.
	// selected: Boolean
	//		Whether or not we are a selected option
	// disabled: Boolean
	//		Whether or not this specific option is disabled
	this.value = value;
	this.label = label;
	this.selected = selected;
	this.disabled = disabled;
}
=====*/

return declare("dijit.form._FormSelectWidget", _FormValueWidget, {
	// summary:
	//		Extends _FormValueWidget in order to provide "select-specific"
	//		values - i.e., those values that are unique to <select> elements.
	//		This also provides the mechanism for reading the elements from
	//		a store, if desired.

	// multiple: [const] Boolean
	//		Whether or not we are multi-valued
	multiple: false,

	// options: dijit.form.__SelectOption[]
	//		The set of options for our select item.  Roughly corresponds to
	//		the html <option> tag.
	options: null,

	// store: dojo.data.api.Identity
	//		A store which, at the very least implements dojo.data.api.Identity
	//		to use for getting our list of options - rather than reading them
	//		from the <option> html tags.
	store: null,

	// query: object
	//		A query to use when fetching items from our store
	query: null,

	// queryOptions: object
	//		Query options to use when fetching from the store
	queryOptions: null,

	// onFetch: Function
	//		A callback to do with an onFetch - but before any items are actually
	//		iterated over (i.e. to filter even further what you want to add)
	onFetch: null,

	// sortByLabel: Boolean
	//		Flag to sort the options returned from a store by the label of
	//		the store.
	sortByLabel: true,


	// loadChildrenOnOpen: Boolean
	//		By default loadChildren is called when the items are fetched from the
	//		store.  This property allows delaying loadChildren (and the creation
	//		of the options/menuitems) until the user clicks the button to open the
	//		dropdown.
	loadChildrenOnOpen: false,

	getOptions: function(/*anything*/ valueOrIdx){
		// summary:
		//		Returns a given option (or options).
		// valueOrIdx:
		//		If passed in as a string, that string is used to look up the option
		//		in the array of options - based on the value property.
		//		(See dijit.form.__SelectOption).
		//
		//		If passed in a number, then the option with the given index (0-based)
		//		within this select will be returned.
		//
		//		If passed in a dijit.form.__SelectOption, the same option will be
		//		returned if and only if it exists within this select.
		//
		//		If passed an array, then an array will be returned with each element
		//		in the array being looked up.
		//
		//		If not passed a value, then all options will be returned
		//
		// returns:
		//		The option corresponding with the given value or index.  null
		//		is returned if any of the following are true:
		//			- A string value is passed in which doesn't exist
		//			- An index is passed in which is outside the bounds of the array of options
		//			- A dijit.form.__SelectOption is passed in which is not a part of the select

		// NOTE: the compare for passing in a dijit.form.__SelectOption checks
		//		if the value property matches - NOT if the exact option exists
		// NOTE: if passing in an array, null elements will be placed in the returned
		//		array when a value is not found.
		var lookupValue = valueOrIdx, opts = this.options || [], l = opts.length;

		if(lookupValue === undefined){
			return opts; // dijit.form.__SelectOption[]
		}
		if(lang.isArray(lookupValue)){
			return array.map(lookupValue, "return this.getOptions(item);", this); // dijit.form.__SelectOption[]
		}
		if(lang.isObject(valueOrIdx)){
			// We were passed an option - so see if it's in our array (directly),
			// and if it's not, try and find it by value.
			if(!array.some(this.options, function(o, idx){
				if(o === lookupValue ||
					(o.value && o.value === lookupValue.value)){
					lookupValue = idx;
					return true;
				}
				return false;
			})){
				lookupValue = -1;
			}
		}
		if(typeof lookupValue == "string"){
			for(var i=0; i<l; i++){
				if(opts[i].value === lookupValue){
					lookupValue = i;
					break;
				}
			}
		}
		if(typeof lookupValue == "number" && lookupValue >= 0 && lookupValue < l){
			return this.options[lookupValue]; // dijit.form.__SelectOption
		}
		return null; // null
	},

	addOption: function(/*dijit.form.__SelectOption|dijit.form.__SelectOption[]*/ option){
		// summary:
		//		Adds an option or options to the end of the select.  If value
		//		of the option is empty or missing, a separator is created instead.
		//		Passing in an array of options will yield slightly better performance
		//		since the children are only loaded once.
		if(!lang.isArray(option)){ option = [option]; }
		array.forEach(option, function(i){
			if(i && lang.isObject(i)){
				this.options.push(i);
			}
		}, this);
		this._loadChildren();
	},

	removeOption: function(/*String|dijit.form.__SelectOption|Number|Array*/ valueOrIdx){
		// summary:
		//		Removes the given option or options.  You can remove by string
		//		(in which case the value is removed), number (in which case the
		//		index in the options array is removed), or select option (in
		//		which case, the select option with a matching value is removed).
		//		You can also pass in an array of those values for a slightly
		//		better performance since the children are only loaded once.
		if(!lang.isArray(valueOrIdx)){ valueOrIdx = [valueOrIdx]; }
		var oldOpts = this.getOptions(valueOrIdx);
		array.forEach(oldOpts, function(i){
			// We can get null back in our array - if our option was not found.  In
			// that case, we don't want to blow up...
			if(i){
				this.options = array.filter(this.options, function(node){
					return (node.value !== i.value || node.label !== i.label);
				});
				this._removeOptionItem(i);
			}
		}, this);
		this._loadChildren();
	},

	updateOption: function(/*dijit.form.__SelectOption|dijit.form.__SelectOption[]*/ newOption){
		// summary:
		//		Updates the values of the given option.  The option to update
		//		is matched based on the value of the entered option.  Passing
		//		in an array of new options will yield better performance since
		//		the children will only be loaded once.
		if(!lang.isArray(newOption)){ newOption = [newOption]; }
		array.forEach(newOption, function(i){
			var oldOpt = this.getOptions(i), k;
			if(oldOpt){
				for(k in i){ oldOpt[k] = i[k]; }
			}
		}, this);
		this._loadChildren();
	},

	setStore: function(/*dojo.data.api.Identity*/ store,
						/*anything?*/ selectedValue,
						/*Object?*/ fetchArgs){
		// summary:
		//		Sets the store you would like to use with this select widget.
		//		The selected value is the value of the new store to set.  This
		//		function returns the original store, in case you want to reuse
		//		it or something.
		// store: dojo.data.api.Identity
		//		The store you would like to use - it MUST implement dojo.data.api.Identity,
		//		and MAY implement dojo.data.api.Notification.
		// selectedValue: anything?
		//		The value that this widget should set itself to *after* the store
		//		has been loaded
		// fetchArgs: Object?
		//		The arguments that will be passed to the store's fetch() function
		var oStore = this.store;
		fetchArgs = fetchArgs || {};
		if(oStore !== store){
			// Our store has changed, so update our notifications
			var h;
			while(h = this._notifyConnections.pop()){ h.remove(); }

			if(store && store.getFeatures()["dojo.data.api.Notification"]){
				this._notifyConnections = [
					aspect.after(store, "onNew", lang.hitch(this, "_onNewItem"), true),
					aspect.after(store, "onDelete", lang.hitch(this, "_onDeleteItem"), true),
					aspect.after(store, "onSet", lang.hitch(this, "_onSetItem"), true)
				];
			}
			this._set("store", store);
		}

		// Turn off change notifications while we make all these changes
		this._onChangeActive = false;

		// Remove existing options (if there are any)
		if(this.options && this.options.length){
			this.removeOption(this.options);
		}

		// Add our new options
		if(store){
			this._loadingStore = true;
			store.fetch(lang.delegate(fetchArgs, {
				onComplete: function(items, opts){
					if(this.sortByLabel && !fetchArgs.sort && items.length){
						items.sort(sorter.createSortFunction([{
							attribute: store.getLabelAttributes(items[0])[0]
						}], store));
					}

					if(fetchArgs.onFetch){
							items = fetchArgs.onFetch.call(this, items, opts);
					}
					// TODO: Add these guys as a batch, instead of separately
					array.forEach(items, function(i){
						this._addOptionForItem(i);
					}, this);

					// Set our value (which might be undefined), and then tweak
					// it to send a change event with the real value
					this._loadingStore = false;
						this.set("value", "_pendingValue" in this ? this._pendingValue : selectedValue);
					delete this._pendingValue;

					if(!this.loadChildrenOnOpen){
						this._loadChildren();
					}else{
						this._pseudoLoadChildren(items);
					}
					this._fetchedWith = opts;
					this._lastValueReported = this.multiple ? [] : null;
					this._onChangeActive = true;
					this.onSetStore();
					this._handleOnChange(this.value);
				},
				scope: this
			}));
		}else{
			delete this._fetchedWith;
		}
		return oStore;	// dojo.data.api.Identity
	},

	// TODO: implement set() and watch() for store and query, although not sure how to handle
	// setting them individually rather than together (as in setStore() above)

	_setValueAttr: function(/*anything*/ newValue, /*Boolean?*/ priorityChange){
		// summary:
		//		set the value of the widget.
		//		If a string is passed, then we set our value from looking it up.
		if(this._loadingStore){
			// Our store is loading - so save our value, and we'll set it when
			// we're done
			this._pendingValue = newValue;
			return;
		}
		var opts = this.getOptions() || [];
		if(!lang.isArray(newValue)){
			newValue = [newValue];
		}
		array.forEach(newValue, function(i, idx){
			if(!lang.isObject(i)){
				i = i + "";
			}
			if(typeof i === "string"){
				newValue[idx] = array.filter(opts, function(node){
					return node.value === i;
				})[0] || {value: "", label: ""};
			}
		}, this);

		// Make sure some sane default is set
		newValue = array.filter(newValue, function(i){ return i && i.value; });
		if(!this.multiple && (!newValue[0] || !newValue[0].value) && opts.length){
			newValue[0] = opts[0];
		}
		array.forEach(opts, function(i){
			i.selected = array.some(newValue, function(v){ return v.value === i.value; });
		});
		var val = array.map(newValue, function(i){ return i.value; }),
			disp = array.map(newValue, function(i){ return i.label; });

		this._set("value", this.multiple ? val : val[0]);
		this._setDisplay(this.multiple ? disp : disp[0]);
		this._updateSelection();
		this._handleOnChange(this.value, priorityChange);
	},

	_getDisplayedValueAttr: function(){
		// summary:
		//		returns the displayed value of the widget
		var val = this.get("value");
		if(!lang.isArray(val)){
			val = [val];
		}
		var ret = array.map(this.getOptions(val), function(v){
			if(v && "label" in v){
				return v.label;
			}else if(v){
				return v.value;
			}
			return null;
		}, this);
		return this.multiple ? ret : ret[0];
	},

	_loadChildren: function(){
		// summary:
		//		Loads the children represented by this widget's options.
		//		reset the menu to make it populatable on the next click
		if(this._loadingStore){ return; }
		array.forEach(this._getChildren(), function(child){
			child.destroyRecursive();
		});
		// Add each menu item
		array.forEach(this.options, this._addOptionItem, this);

		// Update states
		this._updateSelection();
	},

	_updateSelection: function(){
		// summary:
		//		Sets the "selected" class on the item for styling purposes
		this._set("value", this._getValueFromOpts());
		var val = this.value;
		if(!lang.isArray(val)){
			val = [val];
		}
		if(val && val[0]){
			array.forEach(this._getChildren(), function(child){
				var isSelected = array.some(val, function(v){
					return child.option && (v === child.option.value);
				});
				domClass.toggle(child.domNode, this.baseClass + "SelectedOption", isSelected);
				child.domNode.setAttribute("aria-selected", isSelected);
			}, this);
		}
	},

	_getValueFromOpts: function(){
		// summary:
		//		Returns the value of the widget by reading the options for
		//		the selected flag
		var opts = this.getOptions() || [];
		if(!this.multiple && opts.length){
			// Mirror what a select does - choose the first one
			var opt = array.filter(opts, function(i){
				return i.selected;
			})[0];
			if(opt && opt.value){
				return opt.value
			}else{
				opts[0].selected = true;
				return opts[0].value;
			}
		}else if(this.multiple){
			// Set value to be the sum of all selected
			return array.map(array.filter(opts, function(i){
				return i.selected;
			}), function(i){
				return i.value;
			}) || [];
		}
		return "";
	},

	// Internal functions to call when we have store notifications come in
	_onNewItem: function(/*item*/ item, /*Object?*/ parentInfo){
		if(!parentInfo || !parentInfo.parent){
			// Only add it if we are top-level
			this._addOptionForItem(item);
		}
	},
	_onDeleteItem: function(/*item*/ item){
		var store = this.store;
		this.removeOption(store.getIdentity(item));
	},
	_onSetItem: function(/*item*/ item){
		this.updateOption(this._getOptionObjForItem(item));
	},

	_getOptionObjForItem: function(item){
		// summary:
		//		Returns an option object based off the given item.  The "value"
		//		of the option item will be the identity of the item, the "label"
		//		of the option will be the label of the item.  If the item contains
		//		children, the children value of the item will be set
		var store = this.store, label = store.getLabel(item),
			value = (label ? store.getIdentity(item) : null);
		return {value: value, label: label, item:item}; // dijit.form.__SelectOption
	},

	_addOptionForItem: function(/*item*/ item){
		// summary:
		//		Creates (and adds) the option for the given item
		var store = this.store;
		if(!store.isItemLoaded(item)){
			// We are not loaded - so let's load it and add later
			store.loadItem({item: item, onItem: function(i){
				this._addOptionForItem(i);
			},
			scope: this});
			return;
		}
		var newOpt = this._getOptionObjForItem(item);
		this.addOption(newOpt);
	},

	constructor: function(/*Object*/ keywordArgs){
		// summary:
		//		Saves off our value, if we have an initial one set so we
		//		can use it if we have a store as well (see startup())
		this._oValue = (keywordArgs || {}).value || null;
		this._notifyConnections = [];
	},

	buildRendering: function(){
		this.inherited(arguments);
		dom.setSelectable(this.focusNode, false);
	},

	_fillContent: function(){
		// summary:
		//		Loads our options and sets up our dropdown correctly.  We
		//		don't want any content, so we don't call any inherit chain
		//		function.
		var opts = this.options;
		if(!opts){
			opts = this.options = this.srcNodeRef ? query("> *",
						this.srcNodeRef).map(function(node){
							if(node.getAttribute("type") === "separator"){
								return { value: "", label: "", selected: false, disabled: false };
							}
							return {
								value: (node.getAttribute("data-" + kernel._scopeName + "-value") || node.getAttribute("value")),
										label: String(node.innerHTML),
								// FIXME: disabled and selected are not valid on complex markup children (which is why we're
								// looking for data-dojo-value above.  perhaps we should data-dojo-props="" this whole thing?)
								// decide before 1.6
										selected: node.getAttribute("selected") || false,
								disabled: node.getAttribute("disabled") || false
							};
						}, this) : [];
		}
		if(!this.value){
			this._set("value", this._getValueFromOpts());
		}else if(this.multiple && typeof this.value == "string"){
			this._set("value", this.value.split(","));
		}
	},

	postCreate: function(){
		// summary:
		//		sets up our event handling that we need for functioning
		//		as a select
		this.inherited(arguments);

		// Make our event connections for updating state
		this.connect(this, "onChange", "_updateSelection");
		this.connect(this, "startup", "_loadChildren");

		this._setValueAttr(this.value, null);
	},

	startup: function(){
		// summary:
		//		Connects in our store, if we have one defined
		this.inherited(arguments);
		var store = this.store, fetchArgs = {};
		array.forEach(["query", "queryOptions", "onFetch"], function(i){
			if(this[i]){
				fetchArgs[i] = this[i];
			}
			delete this[i];
		}, this);
		if(store && store.getFeatures()["dojo.data.api.Identity"]){
			// Temporarily set our store to null so that it will get set
			// and connected appropriately
			this.store = null;
			this.setStore(store, this._oValue, fetchArgs);
		}
	},

	destroy: function(){
		// summary:
		//		Clean up our connections
		var h;
		while(h = this._notifyConnections.pop()){ h.remove(); }
		this.inherited(arguments);
	},

	_addOptionItem: function(/*dijit.form.__SelectOption*/ /*===== option =====*/){
		// summary:
		//		User-overridable function which, for the given option, adds an
		//		item to the select.  If the option doesn't have a value, then a
		//		separator is added in that place.  Make sure to store the option
		//		in the created option widget.
	},

	_removeOptionItem: function(/*dijit.form.__SelectOption*/ /*===== option =====*/){
		// summary:
		//		User-overridable function which, for the given option, removes
		//		its item from the select.
	},

	_setDisplay: function(/*String or String[]*/ /*===== newDisplay =====*/){
		// summary:
		//		Overridable function which will set the display for the
		//		widget.  newDisplay is either a string (in the case of
		//		single selects) or array of strings (in the case of multi-selects)
	},

	_getChildren: function(){
		// summary:
		//		Overridable function to return the children that this widget contains.
		return [];
	},

	_getSelectedOptionsAttr: function(){
		// summary:
		//		hooks into this.attr to provide a mechanism for getting the
		//		option items for the current value of the widget.
		return this.getOptions(this.get("value"));
	},

	_pseudoLoadChildren: function(/*item[]*/ /*===== items =====*/){
		// summary:
		//		a function that will "fake" loading children, if needed, and
		//		if we have set to not load children until the widget opens.
		// items:
		//		An array of items that will be loaded, when needed
	},

	onSetStore: function(){
		// summary:
		//		a function that can be connected to in order to receive a
		//		notification that the store has finished loading and all options
		//		from that store are available
	}
});

});

},
'dojox/grid/_SelectionPreserver':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/connect",
	"dojo/_base/lang",
	"dojo/_base/array"
], function(declare, connect, lang, array){

return declare("dojox.grid._SelectionPreserver", null, {
	// summary:
	//		Preserve selections across various user actions.
	//
	// description:
	//		When this feature is turned on, Grid will try to preserve selections across actions, e.g. sorting, filtering etc.
	//
	//		Precondition - Identifier(id) is required for store since id is the only way for differentiating row items.
	//		Known issue - The preserved selections might be inaccurate if some unloaded rows are previously selected by range(e.g.SHIFT + click)
	//
	// example:
	// |	//To turn on this - please set 'keepSelection' attribute to true
	// |	<div dojoType="dojox.grid.DataGrid" keepSelection = true .../>
	// |	<div dojoType="dojox.grid.TreeGrid" keepSelection = true .../>
	// |	<div dojoType="dojox.grid.LazyTreeGrid" keepSelection = true .../>
	
	constructor: function(selection){
		this.selection = selection;
		var grid = this.grid = selection.grid;
		this.reset();
		this._connects = [
			connect.connect(grid, '_setStore', this, 'reset'),
			connect.connect(grid, '_addItem', this, '_reSelectById'),
			connect.connect(selection, 'addToSelection', lang.hitch(this, '_selectById', true)),
			connect.connect(selection, 'deselect', lang.hitch(this, '_selectById', false)),
			connect.connect(selection, 'deselectAll', this, 'reset')
		];
	},
	destroy: function(){
		this.reset();
		array.forEach(this._connects, connect.disconnect);
		delete this._connects;
	},
	reset: function(){
		this._selectedById = {};
	},

	_reSelectById: function(item, index){
		// summary:
		//		When some rows is fetched, determine whether it should be selected.
		if(item && this.grid._hasIdentity){
			var id = this.grid.store.getIdentity(item);
			if(this._selectedById.hasOwnProperty(id)){
				this.selection.selected[index] = this._selectedById[id];
			}
		}
	},
	_selectById: function(toSelect, inItemOrIndex){
		// summary:
		//		Record selected rows by ID.
		if(this.selection.mode == 'none' || !this.grid._hasIdentity){ return; }
		var item = inItemOrIndex, g = this.grid;
		if(typeof inItemOrIndex == "number" || typeof inItemOrIndex == "string"){
			var entry = g._by_idx[inItemOrIndex];
			item = entry && entry.item;
		}
		if(item){
			this._selectedById[g.store.getIdentity(item)] = !!toSelect;
		}
		return item;
	}
});
});

},
'dojoe/tablex/plugins/Menu':function(){
//>>built
define("dojoe/tablex/plugins/Menu", ["dojo", "dijit", "dojox/grid/enhanced/plugins/Menu"], function(dojo, dijit) {

	// summary:
	//		plugin to enable Menu
	// description:
	//		Please refer to dojox.grid.enhanced.plugins.Menu
	
	//return dojox.grid.enhanced.plugins.Menu;
});
},
'dojox/form/TimeSpinner':function(){
define([
	"dojo/_base/lang",
	"dojo/_base/event",
	"dijit/form/_Spinner",
	"dojo/keys",
	"dojo/date",
	"dojo/date/locale",
	"dojo/date/stamp",
	"dojo/_base/declare"
], function(lang, event, Spinner, keys, dateUtil, dateLocale, dateStamp, declare){
	/*=====
		Spinner = dijit.form._Spinner;
	=====*/
return declare( "dojox.form.TimeSpinner", Spinner,
{
	// summary: Time Spinner
	// description: This widget is the same as a normal NumberSpinner, but for the time component of a date object instead

	required: false,

	adjust: function(/* Object */ val, /*Number*/ delta){
		return dateUtil.add(val, "minute", delta)
	},

	//FIXME should we allow for constraints in this widget?
	isValid: function(){return true;},

	smallDelta: 5,

	largeDelta: 30,

	timeoutChangeRate: 0.50,

	parse: function(time, locale){
		return dateLocale.parse(time, {selector:"time", formatLength:"short"});
	},

	format: function(time, locale){
		if(lang.isString(time)){ return time; }
		return dateLocale.format(time, {selector:"time", formatLength:"short"});
	},

	serialize: dateStamp.toISOString,

	value: "12:00 AM",

       _onKeyPress: function(e){
                if((e.charOrCode == keys.HOME || e.charOrCode == keys.END) && !(e.ctrlKey || e.altKey || e.metaKey)
                && typeof this.get('value') != 'undefined' /* gibberish, so HOME and END are default editing keys*/){
                        var value = this.constraints[(e.charOrCode == keys.HOME ? "min" : "max")];
                        if(value){
                                this._setValueAttr(value,true);
                        }
                        // eat home or end key whether we change the value or not
                        event.stop(e);
                }
        }


});
});

},
'gridx/modules/filter/FilterConfirmDialog':function(){
define([
	"dojo/_base/declare",
	"dojo/string",
	"dijit/Dialog",
	"dojo/text!../../templates/FilterConfirmDialog.html",
	"dojo/i18n!../../nls/FilterBar"
], function(declare, string, Dialog, template, i18n){

	return declare(Dialog, {
		title: i18n.clearFilterDialogTitle,
		cssClass: 'gridxFilterConfirmDialog',
		autofocus: false,
		postCreate: function(){
			this.inherited(arguments);
			this.set('content', string.substitute(template, i18n));
			var arr = dijit.findWidgets(this.domNode);
			this.btnClear = arr[0];
			this.btnCancel = arr[1];
			this.connect(this.btnCancel, 'onClick', 'hide');
			this.connect(this.btnClear, 'onClick', 'onExecute');
		},
		onExecute: function(){
			this.execute();
		},
		execute: function(){}
	});
});

},
'ibm/tivoli/tip/navwidget/tree/dijit/TreeTableSelection':function(){
// wrapped by build app
define("ibm/tivoli/tip/navwidget/tree/dijit/TreeTableSelection", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/tip/navwidget/dojo/NavWidgetPluggableViewSelection,ibm/tivoli/tip/navwidget/dojo/NavWidgetUtil"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.tree.dijit.TreeTableSelection");

dojo.require("ibm.tivoli.tip.navwidget.dojo.NavWidgetPluggableViewSelection");
dojo.require("ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil");

dojo.declare("ibm.tivoli.tip.navwidget.tree.dijit.TreeTableSelection", ibm.tivoli.tip.navwidget.dojo.NavWidgetPluggableViewSelection, {
	// data members
	_ignore: false,
	
	setMode: function(mode){
		this._pluggableView._table._grid.selection.setMode(mode);
		
		// Call superclass
		this.inherited(arguments);
	},
	
	setSelectedObjectIds: function(/*String[]*/itemIds, /*String[]*/linkIds, /*String[]*/relationshipIds) {
		// Mark we're changing
		this._changing = true;
		
		if (!this._ignore) {			
			// Clear selection
			var grid = this._pluggableView._table._grid;
			
			// NOTE: for gridx this would need to be grid.select.row.clear()
			//  ...but since we have no way (yet) of looking up an item by
			//  it's itemId in the gridx table's model to get the row...we can't support this now
			console.log("ERROR: setSelectedObjectIds not supported yet for GRIDX based table");
			grid.selection.clear();
				
			// Deal with items
			if (itemIds) {
				var length = itemIds.length;
				for (var i = 0; i < length; i++) {
					var item = ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getItemByAbsoluteId(this._dataset, itemIds[i]);
					var idx = grid.getItemIndex(item);
					if (idx >= 0) {
						grid.selection.addToSelection(idx);
					}
				}
			}					
		}
			
		//Call superclass
		this.inherited(arguments);
			
		//Mark we're done changing
		this._changing = false;
	},

	setSelectedObjects: function(/*ibm.tivoli.navmodel.Item[]*/items, /*ibm.tivoli.navmodel.Link[]*/links, /*ibm.tivoli.navmodel.Relationship[]*/relationships, /*Integer[]*/rowIds) {		
		// Mark we're changing
		this._changing = true;
			
		if (!this._ignore) {			
			// Clear selection
			var grid = this._pluggableView._table._grid;
			
			// grid.select.row.clear();    remove for now...no longer needed?
				
			// Deal with items
            if (items && items.length > 0 && rowIds && rowIds.length > 0) {
                dojo.forEach(rowIds, function(rowId) { 
                      grid.select.row.selectById(rowId);
                   });
            }
            // need else case for calling in programmatically with only items
            // but that will require lookup from item to row in gridx
            grid.model.when();   // Necessary?  Check with gridx developers					
		}
		
    	//Call superclass
		this.inherited(arguments);
			
		//Mark we're done changing
		this._changing = false;		
	},
    
	updateSelectedObjects: function(/*ibm.tivoli.navmodel.Item[]*/items, /*Integer[]*/rowIds) {	
		this._ignore = true;
   		this.setSelectedObjects(items, null, null, rowIds);
   		this._ignore = false;
   	}	
});
});

},
'ibm/tivoli/tip/dojo/EventUtil':function(){
// wrapped by build app
define("ibm/tivoli/tip/dojo/EventUtil", ["dijit","dojo","dojox"], function(dijit,dojo,dojox){
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
dojo.provide("ibm.tivoli.tip.dojo.EventUtil");

dojo.declare("ibm.tivoli.tip.dojo.EventUtil",
	          null,
{
	// Simple method to return whether or not a value has a value
	_hasValue: function(obj){
		return typeof obj != "undefined" && obj != null;
	},

	_doEval: function(scope,str){
		if (typeof scope == "undefined" || scope == null)
			return null;
	    var obj = null;
	    try {
	    	var subParts = str.split(".");
	    	obj=scope;
	    	for (var i=0; i<subParts.length; i++){
	    		obj = obj[subParts[i]];
	    	}
	    	//obj=eval("scope."+str);
	    }
	    catch (e) {
	    	obj=null
	    }
	    return obj;
	},
	// Indicates if the expression has a member
	// Scope is the object being checked, str indicates the member to find.
	_hasMember: function(scope, str){
	    var obj = this._doEval(scope,str);
		return typeof obj != "undefined" && obj != null;
	},

	// Pass in a string and this will return a value associated with it.
	// If the string has an error, the returned value is null.  Very
	// useful for deeply nested references.
	_getValue: function(scope,str){
	    var obj = this._doEval(scope,str);
	    if (typeof obj == "undefined")
	    	obj = null;
		return obj;
	},

	// Only adds the specified attribute to the specified object if
	// the attribute does not already exist
	_safeAdd: function(obj,attr,value){
		if (!this._hasValue(obj[attr]))
			obj[attr]=value;
	},

	// This function takes an event using the new event context and 
	// modifies the event by adding the old context to the event by
	// interrogating the proper fields of the new event.  Note that
	// multiple resources are not supported in the old event context.
	// srcEvent: is the event base, it will be updated upon return
	// payload: optional parameter to point to the payload of the event
    //          if specified, this will be used instead of srcEvent.payload
	addOldContext: function(srcEvent, /*optional*/ payload){
		if (this._hasMember(srcEvent,"payload") || this._hasValue(payload)){
			var productCount=1;
			var increment=false;
			var evtPayload=(this._hasValue(payload)?payload:this._getValue(srcEvent,"payload"));
			
			// Check for case if emitter created a resources array of 1
			if (this._hasMember(evtPayload,"resources.length") && (evtPayload.resources.length==1))
				evtPayload=evtPayload.resources[0];

			var clazztype=this._getValue(evtPayload,"cdm.attributes.ClassType");
			
			// Enumerate through all of the payload
			for (var attr in evtPayload) {
			   if (evtPayload[attr]!=null){
				   if (attr=="product"){
						for (var name in evtPayload.product) {
							increment=false;
							var product = evtPayload.product[name];
							// Get all product targeted attributes
							for (var prodName in product){
								// If the attribute is the cdm attribute, need to enumerate it.
								if (prodName == "cdm"){
									for (var prodCdm in product.cdm){
										if (prodCdm == "ClassType")
											this._safeAdd(srcEvent,"CDMClass"+productCount,product.cdm[prodCdm]);
										else
											this._safeAdd(srcEvent,prodCdm+productCount,product.cdm[prodCdm]);
										increment=true;
									}
									if (clazztype != null)
										this._safeAdd(srcEvent,"CDMClass"+productCount,clazztype);
								} else {
									this._safeAdd(srcEvent,prodName+productCount,product[prodName]);
									increment=true;
								}
							}
							if (increment)
								productCount++;
						}
				   }
				   else if (attr=="cdm" && this._hasMember(evtPayload,"cdm.attributes")){
						for (var attr in evtPayload.cdm.attributes) {
							// Add all attributes with their given name; including ClassType
							this._safeAdd(srcEvent,attr,evtPayload.cdm.attributes[attr]);
						}			
				   }
				   else if (attr != "framework" && attr != "resources"){
					   this._safeAdd(srcEvent,attr,evtPayload[attr]);
				   }
			   }
			}
		}
		return srcEvent;
	},

	_parseProductKey: function(mssname){
		var key = null;
		//var pattern = /\+ProductName=.+$/g;
		//var key = mssname.match(pattern);
		//console.debug(key);
		var pattern = "+ProductName=";
		var i = mssname.indexOf(pattern);
		if (i>0){
			var start = i + pattern.length;
			var end = mssname.indexOf("+",start);
			if (end>0)
				key = mssname.substr(start,end-start);
			else
				key = mssname.substr(start);
		}
		if(key != null)
			key = key.replace(/ /g,"");
			
		return key;
	},

	_addCdmAttribute: function(evtPayload, attribute, value){
		if (!this._hasValue(evtPayload.cdm))
			evtPayload.cdm = {};
		if (!this._hasValue(evtPayload.cdm.attributes))
			evtPayload.cdm.attributes = {};
		evtPayload.cdm.attributes[attribute] = value;
	},

	_addMssMember: function(evtPayload, attrList, mssNumber, productkey, attribute){
		if (this._hasMember(attrList,attribute+mssNumber)){

			// Some special processing for CDMClass
			if (attribute=="CDMClass"){
				if (!this._hasMember(evtPayload,"cdm.attributes.ClassType")){
					this._addCdmAttribute(evtPayload, "ClassType", attrList[attribute+mssNumber]);
				}
				this._safeAdd(evtPayload.product[productkey].cdm,"ClassType",attrList[attribute+mssNumber]);
			} else {
				this._safeAdd(evtPayload.product[productkey].cdm,attribute,attrList[attribute+mssNumber]);
			}
			
			delete attrList[attribute+mssNumber];
		}
	},

	_getMssMembers: function(evtPayload, attrList, mssNumber){
		var mssname=attrList["MSSName"+mssNumber];
		var productkey = attrList["ProviderType"+mssNumber];
		if (!this._hasValue(productkey))
		    productkey = this._parseProductKey(mssname);
		if (productkey != null){
			if (!this._hasValue(evtPayload.product))
				evtPayload.product={};
			if (!this._hasValue(evtPayload.product[productkey]))
				evtPayload.product[productkey]={};
			if (!this._hasValue(evtPayload.product[productkey].cdm))
				evtPayload.product[productkey].cdm={};
			
			// This case if there are no numbers after MSSName in the old context
			if (mssNumber == ""){
				// The old format only ever added numbers to MSSName, SourceToken, and CDMClass
				this._addMssMember(evtPayload, attrList, mssNumber, productkey, "MSSName");
				this._addMssMember(evtPayload, attrList, mssNumber, productkey, "SourceToken");
				this._addMssMember(evtPayload, attrList, mssNumber, productkey, "CDMClass");
			} else {
				// If here, there are numbers after MSSName (like MSSName1), we include all
				// attributes ending with the same number in that providertype.
				for (var attr in attrList){
					if (attr.lastIndexOf(mssNumber)==(attr.length-(""+mssNumber).length)){
						var prop = attr.substr(0,attr.length-(""+mssNumber).length);
						
						if (prop=="CDMClass" || prop=="MSSName" || prop=="SourceToken"){
							this._addMssMember(evtPayload, attrList, mssNumber, productkey, prop);
						} else {
							this._safeAdd(evtPayload.product[productkey],prop,attrList[attr]);
						}

						delete attrList[attr];
					}
				}
			}
			
		}
	},

	// The purpose of this method is to attempt to create a "new" context out of
	// an event constructed with old context.
	// srcEvent - the source event
	// payload - optional parameter.  By default this method will add srcEvent.payload
    //           to put the context, but you can specify and alternate location if needed.
	addNewContext: function(srcEvent, /*optional*/ payload){
		// bypass if event already has a payload; therefore, only run this method
		// if a payload object was specified, or there is not already a payload
		if ((this._hasValue(payload) || !this._hasMember(srcEvent,"payload")) && this._hasValue(srcEvent)){  
		    // Gather all of the attributes into an array so we can track what has been processed.
			var attrList={};
			for (var attr in srcEvent) {
				attrList[attr]=srcEvent[attr];
			}
			var evtPayload={};
			if (this._hasValue(payload)){
				evtPayload=payload;
			} else {
				srcEvent.payload = {};
				evtPayload = srcEvent.payload;
			}
			
			// See if ClassType was coded in the old context.  If so, add it; otherwise, we will
			// try to get it from CDMClass later.
			if (this._hasMember(attrList,"ClassType") && !this._hasMember(evtPayload,"cdm.attributes.ClassType")){
				this._addCdmAttribute(evtPayload, "ClassType", attrList["ClassType"]);
				delete attrList["ClassType"];
			}
			
			if (this._hasMember(attrList,"MSSName")){
				// There is just "MSSName", so assume just one MSSName, SourceToken, CDMClass.
				this._getMssMembers(evtPayload,attrList,"");
			} else {
				var mssNumber = 1;
				while (this._hasMember(attrList,"MSSName"+mssNumber))
				{
					this._getMssMembers(evtPayload,attrList,mssNumber);
				    mssNumber++;	
				}
			}
			
			// New for Breeze: add lookup for OslcResourceURI
			var _OslcResourceURI="OslcResourceURI";
			if (this._hasMember(attrList, _OslcResourceURI)){
				var uris=[];
				var uri= {
						    uri: attrList[_OslcResourceURI],
				            type: "product"   // These are always product URIs, meaning URIs that came from a product
				         };
				uris.push(uri);
				this._safeAdd(evtPayload, "uris", uris);
				delete attrList[_OslcResourceURI];
			}
			
			// Now place all the other properties inline
			for (var mem in attrList){
				// do not add name, type, or source as they are part of the event meta data
				if (mem != "name" && mem != "source" && mem != "type"){
					this._safeAdd(evtPayload,mem,attrList[mem]);
				}
			}
		}
     },
     
     // Pass an event to this method, and it will return the number of resources present.
     // The minimum number will always be 1 unless the srcEvent is null in which case 0 is returned.
     getNumberOfResources: function(srcEvent){
    	 if (!this._hasValue(srcEvent)){                    // Got a null object
    		 return 0;
    	 }
    	 if (!this._hasValue(srcEvent.payload)){            // Event is old style, so we know only one resource
    		 return 1;
    	 }
    	 if (!this._hasValue(srcEvent.payload.resources)){  // if no resource array, then only one resource
    		 return 1;
    	 }
    	 return srcEvent.payload.resources.length;
     },
     
     // Returns the attributes for the resource at the given index in the event.
     // Note for some events like TimeSet, there may not be an actual resource, but in this
     // case you pass an index of 0
     getAttributes: function(srcEvent, index){
    	 if (!this._hasValue(srcEvent)){                    // Got a null object
    		 console.error("EventUtil: Bad event.");
    		 return null;
    	 }
    	 if (!this._hasValue(index) || index<0){            // Bad index
    		 console.error("EventUtil: Bad Index.");
    		 return null;
    	 }
    	 
    	 var numItems = this.getNumberOfResources(srcEvent);
    	 if (numItems < 1 || index>= numItems){        // Index out of bounds
    		 console.error("EventUtil: Index out of bounds.");
    		 return null;
    	 }
    	 
    	 var payload = srcEvent.payload;
    	 if (!this._hasValue(srcEvent.payload)){            // Event is old style, so we know only one resource
    		 addNewContext(srcEvent,payload);          // Create new style payload to work with. Do not change source event.
    	 }
    	 
    	 if (!this._hasValue(srcEvent.payload)){            // Should not happen
    		 return null;
    	 }
    	 
    	 if (this._hasValue(srcEvent.payload.resources)){
 			var attrList={};
			for (var attr in srcEvent.payload) {
				if (attr != "resources" && attr!="framework"){
					attrList[attr]=srcEvent.payload[attr];
				}
			}
			dojo.mixin(attrList,srcEvent.payload.resources[index]);
			return attrList;
    	 } else {                                      // No payload.resources
    		 if (index == 0){
    			 return srcEvent.payload;
    		 } else {                                  // Should not occur
        		 console.error("EventUtil: Index error.");
        		 return null;
    		 }
    	 }
    	 
    	 return null;
     },
     
     // Returns the attributes for a given resource in the event as a list with the 'product'
     // attribute flattened.  This method takes the mssname and/or providerType as optional.
     // If you only want to specify the providerType, then pass null for the mssname
     // As an example, if the event looks like this:
     //
     // event=   {
     //       name: "http://ibm.com/tip#NodeClickedOn",
     //       resources: [{
     //       product: {
     //                   IBMTivoliMonitoringServices: { cdm:{
     //                                      MSSName: "ibm-cdm:///CDMMSS/Hostname=loco.raleigh.ibm.com+ManufacturerName=IBM+ProductName=IBM Tivoli Monitoring Services",
     //                                      SourceToken: "ip_address=9.27.144.150"
     //                                     },
     //                                     Color: "blue"
     //                                   }
     //                }}]
     //       uris: [{uri: "http://myserver:port/items/item1"}],
     //    }
     //
     // If this method is called with parameters (event, 0,  "ibm-cdm:///CDMMSS/Hostname=loco.raleigh.ibm.com+ManufacturerName=IBM+ProductName=IBM Tivoli Monitoring Services")
     // the returned result is
     //
     // {
     //   MSSName: "ibm-cdm:///CDMMSS/Hostname=loco.raleigh.ibm.com+ManufacturerName=IBM+ProductName=IBM Tivoli Monitoring Services",
     //   SourceToken: "ip_address=9.27.144.150",
     //   Color: "blue",
     //   uris: [{uri: "http://myserver:port/items/item1"}]
     // }
     //
     // If providerType or MSSName is not provided, then the product information will not be returned.  So in this example, the result 
     // for the parameters (event, 0) will be
     //
     // {
     //   uris: [{uri: "http://myserver:port/items/item1"}]
     // }
     //
     // null is returned on error
     
     getAttributesWithFlatProduct: function(srcEvent, index, /*optional*/ mssname, /*optional*/ providerType){
    	 if (!this._hasValue(srcEvent)){
    		 console.warn("no srcEvent passed to getAttributesWithFlatProduct");
    		 return null;
    	 }
    	 var attributes = this.getAttributes(srcEvent, index);
    	 if (attributes==null){
    		 return null;
    	 }
    	 var retAttrs={};
         for (var attr in attributes) {
        	 if (attr=='product'){
        		 this._addFlatProductData(retAttrs,attributes[attr],mssname,providerType);
        	 } else {
        		 retAttrs[attr] = attributes[attr];
        	 }
         }
    	 return retAttrs;
     },
     
     
     _addFlatProductData: function(target,productData,mssname,inputProviderType){
    	 var providerType = null;
    	 if (!this._hasValue(productData)){
    		 return;
    	 }
    	 if (!this._hasValue(mssname) && !this._hasValue(inputProviderType)){
    		 return;
    	 }
    	 if (this._hasValue(inputProviderType)){
    		 providerType = inputProviderType;
    	 }
    	 if (this._hasValue(mssname)){
    		 if (!this._hasValue(providerType)){
    			 providerType = this._parseProductKey(mssname);
    		 }
    	 }
    	 if (this._hasValue(productData) && this._hasValue(productData[providerType])){
    		 var providerAttrs = productData[providerType];
    		 if (this._hasValue(mssname) && this._hasValue(providerAttrs.cdm)){ // need to validate mssname value
    			 if (this._hasValue(providerAttrs.cdm.MSSName)){
    				 if (providerAttrs.cdm.MSSName != mssname){
    					 return;  // if the mssname specified does not match mssname, then do not include this product data
    				 }
    			 }
    		 }
        	 for (var attr in providerAttrs){
        		 if (attr == "cdm"){
            		 dojo.mixin(target,providerAttrs[attr]);
        		 } else {
        			 target[attr]=providerAttrs[attr];
        		 }
        	 }
    	 }
    	 return;
     },
     
     // This method will accept an input object and convert all the attribute values to
     // strings.  This is helpful for an object needing to send attributes as parameters
     // on a URL.
     // The returned object will contain all the same attributes as the input object, but
     // any dojo objects or arrays will have been converted to JSON strings.
     attributesToStrings: function(source){
    	 if (!this._hasValue(source)){
    		 console.warn("source is null in attributesToStrings");
    		 return null;
    	 }
    	 var output={};
    	 for (var attr in source){
    		 if (typeof source[attr] == "string"){
    			 output[attr]=source[attr];
    		 } else {
        		 console.log(attr+" is type "+typeof source[attr]);
    			 output[attr]=dojo.toJson(source[attr]);
    		 }
    	 }
    	 return output;
     },
     
     // This method is a utility to help the caller construct an event.
     // As parameters, the caller passes in an object that contains members
     // that control how the event is created.
     // Minimally, the caller passes in a member called 'valueMap'
     // that contains all the attribute names and values to be included in
     // the event.
     // {
     //   valueMap:   [{
     //                  MSSName: "ServiceInstanceID",
     //                  SourceToken: "12345",
     //                  bar: "http://somehost:8080/foo"
     //               }]
     // }
     // This method will construct the event based on the attribute names
     // given.  There should be one object in the 'attributes' array for
     // each unique resource to be represented in the event.
     //
     // Optionally, the caller may also include a mapping to assist
     // in the construction of the event.  This mapping uses the substitution
     // pattern of '%%variable%%' where this entire string will be replaced
     // with the value of the attribute that has the name 'variable' from the
     // 'attributes' list (provided above). This value MUST BE A STRING!!
     // The following is the resource, but it must be passed as a string.
     //
     //   resourceMap: {
     //       product: {
     //                   MyProviderType: {
     //                                      MSSName: %%MSSName%%,
     //                                      SourceToken: %%SourceToken%%
     //                                   }
     //                }
     //       uris: [{uri: %%bar%%}],
     //            }
     // 
     // param: eventName is simply the name of the event to send.
     createEvent: function(attributes,eventName){
    	 if (!this._hasValue(attributes)){
    		 console.warn("No attributes provided to EventUtil.createEvent");
    		 return null;
    	 }
    	 if (!this._hasValue(eventName)){
    		 console.warn("No eventName provided to EventUtil.createEvent");
    		 return null;
    	 }
    	 if (!this._hasValue(attributes.valueMap)){
    		 console.warn("No attributes.valueMap provided provided to EventUtil.createEvent");
    		 return null;
    	 }
    	 
    	 var event=this._buildEvent(attributes);
    	 event.name = eventName;
    	 return event;	 
     },
     
     _buildEvent: function(attributes){
    	 var event = {};
    	 event.payload={};
    	 event.payload.resources=[];
    	 
    	 // Loop through each set of attributes represented in the attributes array.
    	 for (var resourceAttrs in attributes.valueMap) {
    		 var payload = {};
    		 if (this._hasValue(attributes.resourceMap)){
            	 payload = this._contextFromMap(attributes.valueMap[resourceAttrs], attributes.resourceMap);
    		 } else {
            	 this.addNewContext(attributes.valueMap[resourceAttrs], payload);
    		 }
        	 event.payload.resources.push(payload);
    	 }
    	 return event;
     },
     
     // Construct the payload given a mapping
     _contextFromMap: function(values, mapStr){
    	 //var mapStr = dojo.toJson(map);  // convert to string to make substitutions easy
    	 for (var name in values){       // for each value in the values object, loop to replace
    		 var re = new RegExp("%%"+name+"%%","g");
    	     mapStr = mapStr.replace(re, values[name]);
    	 }
    	 return dojo.fromJson(mapStr);
     }
     
     
});

});

},
'ibm/tivoli/navmodel/LinkStyle':function(){
// wrapped by build app
define(["dijit","dojo","dojox"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.LinkStyle"); 
 
////////////////////////////////////
// LinkStyle
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.LinkStyle", null, {
	// private members       
	_version: null,
	_style: null,
	
	/*=====
	constructor_kwArgs: function(version, style) {
		// version: Float
		//		version used to communicate with rest service
		// style: Object
		//		json item style object
	},
	=====*/
	
	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary
		//		Constructs LinkStyle object.
		this._version = kwArgs.version;
		this._style = kwArgs.style;		
		if (!this._style) {
			this._style = {};
		}	
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	
	getAttribute: function(key) {
		return this._style[key];
	},
	
	setAttribute: function(key, value) {
		this._style[key] = value;
	},
	
	getId: function() {
		return this._style.id;
	},
	
	setId: function(id) {
		this._style = id;
	},
	
	getType: function() {
		return this._style.type;
	},
	
	setType: function(type) {
		this._style.type = type;
	},
	
	getLabel: function() {
		if (this._style.label) {
			// string
			if (typeof this._style.label === "string") {
				return this._style.label;
			}
			// object with key/value info
			else {
				return this._style.label.value;
			}
		}
		
		return null;
	},
	
	setLabel: function(label) {
		this._style.label = label;		
	},
	
	getLineStyle: function() {
		return this._style.lineStyle;
	},
	
	setLineStyle: function(lineStyle) {
		this._style.lineStyle = lineStyle;
	},
	
	getLineWidth: function() {
		return this._style.lineWidth;
	},
	
	setLineWidth: function(width) {
		this._style.lineWidth = width;
	},
	
	getLineColor: function() {
		return this._style.lineColor;
	},
	
	setLineColor: function(color) {
		this._style.lineColor = color;
	},
	
	getArrowColor: function() {
		return this._style.arrowColor;
	},

	setArrowColor: function(color) {
		this._style.arrowColor = color;
	},
	
	isLabelVisible: function() {
		return (this._style.labelVisible !== undefined ? this._style.labelVisible : false);
	},
	
	setLabelVisible: function(visible) {
		this._style.labelVisible = visible;
	},
	
	isLabelAutoRotate: function() {
		return (this._style.labelAutoRotate !== undefined ? this._style.labelAutoRotate : false);
	},
	
	setLabelAutoRotate: function(autoRotate) {
		this._style.labelAutoRotate = autoRotate;
	},

	getLabelParallelAlignment: function() {
		return (this._style.labelParallelAlignment !== undefined ? this._style.labelParallelAlignment : "middle");
	},
	
	setLabelParallelAlignment: function(parallelAlignment) {
		this._style.labelParallelAlignment = parallelAlignment;
	},
	
	getLabelOrthogonalAlignment: function() {
		return (this._style.labelOrthogonalAlignment !== undefined ? this._style.labelOrthogonalAlignment : "left");
	},
	
	setLabelOrthogonalAlignment: function(orthogonalAlignment) {
		this._style.labelOrthogonalAlignment = orthogonalAlignment;
	},

	getLabelDistance: function() {
		return (this._style.labelDistance !== undefined ? this._style.labelDistance : 5);
	},
	
	setLabelDistance: function(distance) {
		this._style.labelDistance = distance;
	},

	getLabelPositionType: function() {
		return (this._style.labelPositionType !== undefined ? this._style.labelPositionType : "relative");
	},
	
	setLabelPositionType: function(positionType) {
		this._style.labelPositionType = positionType;
	},

	getLabelPosition: function() {
		return (this._style.labelPosition !== undefined ? this._style.labelPosition : 0.5);
	},
	
	setLabelPosition: function(position) {
		this._style.labelPosition = position;
	},
	
	clone: function(widgetDef, objType) {
		var args = {
			style: {},
			version: this._version
		};		
		
		if (widgetDef) {
			// start with widgetDef default style
			var widgetDefaultStyle = null;
			switch(objType) {
				case "link":
					widgetDefaultStyle = widgetDef.getDefaultLinkStyle();
					break;
				case "relationship":
					widgetDefaultStyle = widgetDef.getDefaultRelationshipStyle();
			}
			if (widgetDefaultStyle) {
				dojo.mixin(args.style, widgetDefaultStyle);
			}
		
			// mixin our link style
			dojo.mixin(args.style, this._style);
			
			// mixin widgetDef style
			var widgetStyle = null;
			switch(objType) {
				case "link":
					widgetStyle = widgetDef.getLinkStyle(this.getType());
					break;
				case "relationship":
					widgetStyle = widgetDef.getRelationshipStyle(this.getType());
			}		
			if (widgetStyle) {
				dojo.mixin(args.style, widgetStyle);
			}
		}
		// just mixin our link style
		else {
			dojo.mixin(args.style, this._style);
		}
			
		return new ibm.tivoli.navmodel.LinkStyle(args);
	}
});

});

},
'gridx/core/util':function(){
define({
	stopEvent: function(evt){
		if(evt && evt.stopPropagation && evt.preventDefault){
			evt.stopPropagation();
			evt.preventDefault();
		}
	},

	biSearch: function(arr, comp){
		var i = 0, j = arr.length, k;
		for(k = Math.floor((i + j) / 2); i + 1 < j; k = Math.floor((i + j) / 2)){
			if(comp(arr[k]) > 0){
				j = k;
			}else{
				i = k;
			}
		}
		return arr.length && comp(arr[i]) >= 0 ? i : j;
	}
});

},
'dijit/form/_ListMouseMixin':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/_base/event", // event.stop
	"dojo/touch",
	"./_ListBase"
], function(declare, event, touch, _ListBase){

/*=====
var _ListBase = dijit.form._ListBase;
=====*/

// module:
//		dijit/form/_ListMouseMixin
// summary:
//		a mixin to handle mouse or touch events for a focus-less menu

return declare( "dijit.form._ListMouseMixin", _ListBase, {
	// summary:
	//		a Mixin to handle mouse or touch events for a focus-less menu
	//		Abstract methods that must be defined externally:
	//			onClick: item was chosen (mousedown somewhere on the menu and mouseup somewhere on the menu)
	// tags:
	//		private

	postCreate: function(){
		this.inherited(arguments);
		this.connect(this.domNode, touch.press, "_onMouseDown");
		this.connect(this.domNode, touch.release, "_onMouseUp");
		this.connect(this.domNode, "onmouseover", "_onMouseOver");
		this.connect(this.domNode, "onmouseout", "_onMouseOut");
	},

	_onMouseDown: function(/*Event*/ evt){
		event.stop(evt);
		if(this._hoveredNode){
			this.onUnhover(this._hoveredNode);
			this._hoveredNode = null;
		}
		this._isDragging = true;
		this._setSelectedAttr(this._getTarget(evt));
	},

	_onMouseUp: function(/*Event*/ evt){
		event.stop(evt);
		this._isDragging = false;
		var selectedNode = this._getSelectedAttr();
		var target = this._getTarget(evt);
		var hoveredNode = this._hoveredNode;
		if(selectedNode && target == selectedNode){
			this.onClick(selectedNode);
		}else if(hoveredNode && target == hoveredNode){ // drag to select
			this._setSelectedAttr(hoveredNode);
			this.onClick(hoveredNode);
		}
	},

	_onMouseOut: function(/*Event*/ /*===== evt ====*/){
		if(this._hoveredNode){
			this.onUnhover(this._hoveredNode);
			if(this._getSelectedAttr() == this._hoveredNode){
				this.onSelect(this._hoveredNode);
			}
			this._hoveredNode = null;
		}
		if(this._isDragging){
			this._cancelDrag = (new Date()).getTime() + 1000; // cancel in 1 second if no _onMouseOver fires
		}
	},

	_onMouseOver: function(/*Event*/ evt){
		if(this._cancelDrag){
			var time = (new Date()).getTime();
			if(time > this._cancelDrag){
				this._isDragging = false;
			}
			this._cancelDrag = null;
		}
		var node = this._getTarget(evt);
		if(!node){ return; }
		if(this._hoveredNode != node){
			if(this._hoveredNode){
				this._onMouseOut({ target: this._hoveredNode });
			}
			if(node && node.parentNode == this.containerNode){
				if(this._isDragging){
					this._setSelectedAttr(node);
				}else{
					this._hoveredNode = node;
					this.onHover(node);
				}
			}
		}
	}
});

});

},
'gridx/modules/VScroller':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/Deferred",
	"dojo/_base/event",
	"dojo/_base/sniff",
	"dojo/_base/query",
	"dojo/dom-geometry",
	"dojo/keys",
	"dojox/html/metrics",
	"../core/_Module"
], function(declare, Deferred, event, sniff, query, domGeo, keys, metrics, _Module){
	
	var st = 'scrollTop';

	return declare(/*===== "gridx.modules.VScroller", =====*/_Module, {
		// summary:
		//		This module provides basic vertical scrolling logic for grid.
		// description:
		//		This module will make the grid body render all rows without paging.
		//		So it is very fast for small client side store, and might be extremely slow
		//		for large server side store.

		name: 'vScroller',
	
		forced: ['body', 'vLayout', 'columnWidth'],

		optional: ['pagination'],
	
		getAPIPath: function(){
			// tags:
			//		protected extension
			return {
				vScroller: this
			};
		},

		constructor: function(){
			var t = this,
				g = t.grid,
				dn = t.domNode = g.vScrollerNode;
			t.stubNode = dn.firstChild;
			if(g.autoHeight){
				dn.style.display = 'none';
				if(sniff('ie') < 8){
					dn.style.width = '0px';
				}
			}else{
				var w = metrics.getScrollbar().w + 'px';
				dn.style.width = w;
				dn.style[g.isLeftToRight() ? 'right' : 'left'] = -metrics.getScrollbar().w + 'px';
				if(sniff('ie') < 8){
					t.stubNode.style.width = w;
				}
			}
		},

		preload: function(args){
			// tags:
			//		protected extension
			this.grid.hLayout.register(null, this.domNode, 1);
		},

		load: function(args, startup){
			// tags:
			//		protected extension
			var t = this,
				g = t.grid,
				bd = g.body,
				dn = t.domNode,
				bn = g.bodyNode;
			t.batchConnect(
				[t.domNode, 'onscroll', '_doScroll'],
				[bn, 'onmousewheel', '_onMouseWheel'],
				[g.mainNode, 'onkeypress', '_onKeyScroll'],
				sniff('ff') && [bn, 'DOMMouseScroll', '_onMouseWheel']);
			t.aspect(g, '_onResizeEnd', '_onBodyChange');
			t.aspect(bd, 'onForcedScroll', '_onForcedScroll');
			t.aspect(bd, 'onRender', '_onBodyChange');
			if(!g.autoHeight){
				t.aspect(bd, 'onEmpty', function(){
					var ds = dn.style;
					ds.display = 'none';
					ds.width = '';
					if(sniff('ie') < 8){
						ds.width = t.stub.style.width = '0px';
					}
					g.hLayout.reLayout();
					g.hScroller.refresh();
				});
			}
			startup.then(function(){
				Deferred.when(t._init(args), function(){
					t.domNode.style.width = '';
					t.loaded.callback();
				});
			});
		},
	
		//Public ----------------------------------------------------

		scrollToRow: function(rowVisualIndex, toTop){
			// summary:
			//		Scroll the grid until the required row is in view.
			// description:
			//		This job will be an asynchronous one if the lazy-loading and lazy-rendering are used.
			// rowVisualIndex: Integer
			//		The visual index of the row
			// toTop: Boolean?
			//		If set this to true, the grid will try to scroll the required row to the top of the view.
			//		Otherwise, the grid will stop scrolling as soon as the row is visible.
			// returns:
			//		A deferred object indicating when the scrolling process is finished. This will be useful
			//		when using lazy-loading and lazy-rendering.
			var d = new Deferred(),
				bn = this.grid.bodyNode,
				dn = this.domNode,
				dif = 0,
				n = query('[visualindex="' + rowVisualIndex + '"]', bn)[0],
				finish = function(success){
					setTimeout(function(){
						d.callback(success);
					}, 5);
				};
			if(n){
				var no = n.offsetTop,
					bs = bn[st];
				if(toTop){
					dn[st] = no;
					finish(true);
					return d;	//dojo.Deferred
				}else if(no < bs){
					dif = no - bs;
				}else if(no + n.offsetHeight > bs + bn.clientHeight){
					dif = no + n.offsetHeight - bs - bn.clientHeight;
				}else{
					finish(true);
					return d;	//dojo.Deferred
				}
				dn[st] += dif;
			}
			finish(!!n);
			return d;	//dojo.Deferred
		},
	
		//Protected -------------------------------------------------
		_init: function(){
			this._onForcedScroll();
		},

		_update: function(){
			var t = this,
				g = t.grid;
			if(!g.autoHeight){
				var bd = g.body,
					bn = g.bodyNode,
					toShow = bd.renderCount < bd.visualCount || bn.scrollHeight > bn.clientHeight,
					ds = t.domNode.style;
					scrollBarWidth = metrics.getScrollbar().w + (sniff('webkit') ? 1 : 0);//Fix a chrome RTL defect
				if(sniff('ie') < 8){
					var w = toShow ? scrollBarWidth + 'px' : '0px';
					t.stubNode.style.width = w;
					ds.width = w;
				}else{
					ds.width = '';
				}
				ds.display = toShow ? '' : 'none';
				ds[g.isLeftToRight() ? 'right' : 'left'] = -t.domNode.offsetWidth + 'px';
			}
			g.hLayout.reLayout();
		},

		_doScroll: function(){
			this.grid.bodyNode[st] = this.domNode[st];
		},
	
		_onMouseWheel: function(e){
			if(this.grid.vScrollerNode.style.display != 'none'){
				var rolled = typeof e.wheelDelta === "number" ? e.wheelDelta / 3 : (-40 * e.detail); 
				this.domNode[st] -= rolled;
				event.stop(e);
			}
		},
	
		_onBodyChange: function(){
			var t = this,
				g = t.grid;
			t._update();
			//IE7 Needs setTimeout
			setTimeout(function(){
				if(!g.bodyNode){
					//fix FF10 - g.bodyNode will be undefined during a quick recreation
					return;
				}				
				t.stubNode.style.height = g.bodyNode.scrollHeight + 'px';
				t._doScroll();
				//FIX IE7 problem:
				g.vScrollerNode[st] = g.vScrollerNode[st] || 0;
			}, 0);
		},

		_onForcedScroll: function(){
			var t = this,
				bd = t.grid.body;
			return t.model.when({
				start: bd.rootStart,
				count: bd.rootCount
			}, function(){
				bd.renderRows(0, bd.visualCount);
			});
		},
	
		_onKeyScroll: function(evt){
			var t = this,
				g = t.grid,
				bd = g.body,
				bn = g.bodyNode,
				focus = g.focus,
				sn = t.domNode,
				rowNode;
			if(bn.childNodes.length && (!focus || focus.currentArea() == 'body')){
				if(evt.keyCode == keys.HOME){
					sn[st] = 0;
					rowNode = bn.firstChild;
				}else if(evt.keyCode == keys.END){
					sn[st] = sn.scrollHeight - sn.offsetHeight;
					rowNode = bn.lastChild;
				}else if(evt.keyCode == keys.PAGE_UP){
					if(!sn[st]){
						rowNode = bn.firstChild;
					}else{
						sn[st] -= sn.offsetHeight;
					}
				}else if(evt.keyCode == keys.PAGE_DOWN){
					if(sn[st] >= sn.scrollHeight - sn.offsetHeight){
						rowNode = bn.lastChild;
					}else{
						sn[st] += sn.offsetHeight;
					}
				}else{
					return;
				}
				if(focus){
					if(rowNode){
						bd._focusCellRow = parseInt(rowNode.getAttribute('visualindex'), 10);
						focus.focusArea('body', 1);	//1 as true
					}else{
						setTimeout(function(){
							var rowNodes = bn.childNodes,
								start = 0,
								end = rowNodes.length - 1,
								containerPos = domGeo.position(bn),
								i, p,
								checkPos = function(idx){
									var rn = rowNodes[idx],
										pos = domGeo.position(rn);
									if(evt.keyCode == keys.PAGE_DOWN){
										var prev = rn.previousSibling;
										if((!prev && pos.y >= containerPos.y) || pos.y == containerPos.y){
											return 0;
										}else if(!prev){
											return -1;
										}else{
											var prevPos = domGeo.position(prev);
											if(prevPos.y < containerPos.y && prevPos.y + prevPos.h >= containerPos.y){
												return 0;
											}else if(prevPos.y > containerPos.y){
												return 1;
											}else{
												return -1;
											}
										}
									}else{
										var post = rn.nextSibling;
										if((!post && pos.y + pos.h <= containerPos.y + containerPos.h) ||
											pos.y + pos.h == containerPos.y + containerPos.h){
											return 0;
										}else if(!post){
											return 1;
										}else{
											var postPos = domGeo.position(post);
											if(postPos.y <= containerPos.y + containerPos.h &&
													postPos.y + postPos.h > containerPos.y + containerPos.h){
												return 0;
											}else if(postPos.y > containerPos.y + containerPos.h){
												return 1;
											}else{
												return -1;
											}
										}
									}
								};
							//Binary search the row to focus
							while(start <= end){
								i = Math.floor((start + end) / 2);
								p = checkPos(i);
								if(p < 0){
									start = i + 1;
								}else if(p > 0){
									end = i - 1;
								}else{
									rowNode = rowNodes[i];
									break;
								}
							}
							if(rowNode){
								bd._focusCellRow = parseInt(rowNode.getAttribute('visualindex'), 10);
								focus.focusArea('body', 1);	//1 as true
							}
						}, 0);
					}
				}
				event.stop(evt);
			}
		}
	});
});

},
'dojo/date/locale':function(){
define([
	"../_base/kernel",
	"../_base/lang",
	"../_base/array",
	"../date",
	"../cldr/supplemental",
	"../regexp",
	"../string",
	"../i18n!../cldr/nls/gregorian"
], function(dojo, lang, array, date, cldr, regexp, string, gregorian) {
	// module:
	//		dojo/date/locale
	// summary:
	//		This modules defines dojo.date.locale, localization methods for Date.

lang.getObject("date.locale", true, dojo);

// Localization methods for Date.   Honor local customs using locale-dependent dojo.cldr data.

// Load the bundles containing localization information for
// names and formats

//NOTE: Everything in this module assumes Gregorian calendars.
// Other calendars will be implemented in separate modules.

	// Format a pattern without literals
	function formatPattern(dateObject, bundle, options, pattern){
		return pattern.replace(/([a-z])\1*/ig, function(match){
			var s, pad,
				c = match.charAt(0),
				l = match.length,
				widthList = ["abbr", "wide", "narrow"];
			switch(c){
				case 'G':
					s = bundle[(l < 4) ? "eraAbbr" : "eraNames"][dateObject.getFullYear() < 0 ? 0 : 1];
					break;
				case 'y':
					s = dateObject.getFullYear();
					switch(l){
						case 1:
							break;
						case 2:
							if(!options.fullYear){
								s = String(s); s = s.substr(s.length - 2);
								break;
							}
							// fallthrough
						default:
							pad = true;
					}
					break;
				case 'Q':
				case 'q':
					s = Math.ceil((dateObject.getMonth()+1)/3);
//					switch(l){
//						case 1: case 2:
							pad = true;
//							break;
//						case 3: case 4: // unimplemented
//					}
					break;
				case 'M':
					var m = dateObject.getMonth();
					if(l<3){
						s = m+1; pad = true;
					}else{
						var propM = ["months", "format", widthList[l-3]].join("-");
						s = bundle[propM][m];
					}
					break;
				case 'w':
					var firstDay = 0;
					s = dojo.date.locale._getWeekOfYear(dateObject, firstDay); pad = true;
					break;
				case 'd':
					s = dateObject.getDate(); pad = true;
					break;
				case 'D':
					s = dojo.date.locale._getDayOfYear(dateObject); pad = true;
					break;
				case 'E':
					var d = dateObject.getDay();
					if(l<3){
						s = d+1; pad = true;
					}else{
						var propD = ["days", "format", widthList[l-3]].join("-");
						s = bundle[propD][d];
					}
					break;
				case 'a':
					var timePeriod = (dateObject.getHours() < 12) ? 'am' : 'pm';
					s = options[timePeriod] || bundle['dayPeriods-format-wide-' + timePeriod];
					break;
				case 'h':
				case 'H':
				case 'K':
				case 'k':
					var h = dateObject.getHours();
					// strange choices in the date format make it impossible to write this succinctly
					switch (c){
						case 'h': // 1-12
							s = (h % 12) || 12;
							break;
						case 'H': // 0-23
							s = h;
							break;
						case 'K': // 0-11
							s = (h % 12);
							break;
						case 'k': // 1-24
							s = h || 24;
							break;
					}
					pad = true;
					break;
				case 'm':
					s = dateObject.getMinutes(); pad = true;
					break;
				case 's':
					s = dateObject.getSeconds(); pad = true;
					break;
				case 'S':
					s = Math.round(dateObject.getMilliseconds() * Math.pow(10, l-3)); pad = true;
					break;
				case 'v': // FIXME: don't know what this is. seems to be same as z?
				case 'z':
					// We only have one timezone to offer; the one from the browser
					s = dojo.date.locale._getZone(dateObject, true, options);
					if(s){break;}
					l=4;
					// fallthrough... use GMT if tz not available
				case 'Z':
					var offset = dojo.date.locale._getZone(dateObject, false, options);
					var tz = [
						(offset<=0 ? "+" : "-"),
						string.pad(Math.floor(Math.abs(offset)/60), 2),
						string.pad(Math.abs(offset)% 60, 2)
					];
					if(l==4){
						tz.splice(0, 0, "GMT");
						tz.splice(3, 0, ":");
					}
					s = tz.join("");
					break;
//				case 'Y': case 'u': case 'W': case 'F': case 'g': case 'A': case 'e':
//					console.log(match+" modifier unimplemented");
				default:
					throw new Error("dojo.date.locale.format: invalid pattern char: "+pattern);
			}
			if(pad){ s = string.pad(s, l); }
			return s;
		});
	}

/*=====
	dojo.date.locale.__FormatOptions = function(){
	//	selector: String
	//		choice of 'time','date' (default: date and time)
	//	formatLength: String
	//		choice of long, short, medium or full (plus any custom additions).  Defaults to 'short'
	//	datePattern:String
	//		override pattern with this string
	//	timePattern:String
	//		override pattern with this string
	//	am: String
	//		override strings for am in times
	//	pm: String
	//		override strings for pm in times
	//	locale: String
	//		override the locale used to determine formatting rules
	//	fullYear: Boolean
	//		(format only) use 4 digit years whenever 2 digit years are called for
	//	strict: Boolean
	//		(parse only) strict parsing, off by default
		this.selector = selector;
		this.formatLength = formatLength;
		this.datePattern = datePattern;
		this.timePattern = timePattern;
		this.am = am;
		this.pm = pm;
		this.locale = locale;
		this.fullYear = fullYear;
		this.strict = strict;
	}
=====*/

dojo.date.locale._getZone = function(/*Date*/dateObject, /*boolean*/getName, /*dojo.date.locale.__FormatOptions?*/options){
	// summary:
	//		Returns the zone (or offset) for the given date and options.  This
	//		is broken out into a separate function so that it can be overridden
	//		by timezone-aware code.
	//
	// dateObject:
	//		the date and/or time being formatted.
	//
	// getName:
	//		Whether to return the timezone string (if true), or the offset (if false)
	//
	// options:
	//		The options being used for formatting
	if(getName){
		return date.getTimezoneName(dateObject);
	}else{
		return dateObject.getTimezoneOffset();
	}
};


dojo.date.locale.format = function(/*Date*/dateObject, /*dojo.date.locale.__FormatOptions?*/options){
	// summary:
	//		Format a Date object as a String, using locale-specific settings.
	//
	// description:
	//		Create a string from a Date object using a known localized pattern.
	//		By default, this method formats both date and time from dateObject.
	//		Formatting patterns are chosen appropriate to the locale.  Different
	//		formatting lengths may be chosen, with "full" used by default.
	//		Custom patterns may be used or registered with translations using
	//		the dojo.date.locale.addCustomFormats method.
	//		Formatting patterns are implemented using [the syntax described at
	//		unicode.org](http://www.unicode.org/reports/tr35/tr35-4.html#Date_Format_Patterns)
	//
	// dateObject:
	//		the date and/or time to be formatted.  If a time only is formatted,
	//		the values in the year, month, and day fields are irrelevant.  The
	//		opposite is true when formatting only dates.

	options = options || {};

	var locale = dojo.i18n.normalizeLocale(options.locale),
		formatLength = options.formatLength || 'short',
		bundle = dojo.date.locale._getGregorianBundle(locale),
		str = [],
		sauce = lang.hitch(this, formatPattern, dateObject, bundle, options);
	if(options.selector == "year"){
		return _processPattern(bundle["dateFormatItem-yyyy"] || "yyyy", sauce);
	}
	var pattern;
	if(options.selector != "date"){
		pattern = options.timePattern || bundle["timeFormat-"+formatLength];
		if(pattern){str.push(_processPattern(pattern, sauce));}
	}
	if(options.selector != "time"){
		pattern = options.datePattern || bundle["dateFormat-"+formatLength];
		if(pattern){str.push(_processPattern(pattern, sauce));}
	}

	return str.length == 1 ? str[0] : bundle["dateTimeFormat-"+formatLength].replace(/\{(\d+)\}/g,
		function(match, key){ return str[key]; }); // String
};

dojo.date.locale.regexp = function(/*dojo.date.locale.__FormatOptions?*/options){
	// summary:
	//		Builds the regular needed to parse a localized date

	return dojo.date.locale._parseInfo(options).regexp; // String
};

dojo.date.locale._parseInfo = function(/*dojo.date.locale.__FormatOptions?*/options){
	options = options || {};
	var locale = dojo.i18n.normalizeLocale(options.locale),
		bundle = dojo.date.locale._getGregorianBundle(locale),
		formatLength = options.formatLength || 'short',
		datePattern = options.datePattern || bundle["dateFormat-" + formatLength],
		timePattern = options.timePattern || bundle["timeFormat-" + formatLength],
		pattern;
	if(options.selector == 'date'){
		pattern = datePattern;
	}else if(options.selector == 'time'){
		pattern = timePattern;
	}else{
		pattern = bundle["dateTimeFormat-"+formatLength].replace(/\{(\d+)\}/g,
			function(match, key){ return [timePattern, datePattern][key]; });
	}

	var tokens = [],
		re = _processPattern(pattern, lang.hitch(this, _buildDateTimeRE, tokens, bundle, options));
	return {regexp: re, tokens: tokens, bundle: bundle};
};

dojo.date.locale.parse = function(/*String*/value, /*dojo.date.locale.__FormatOptions?*/options){
	// summary:
	//		Convert a properly formatted string to a primitive Date object,
	//		using locale-specific settings.
	//
	// description:
	//		Create a Date object from a string using a known localized pattern.
	//		By default, this method parses looking for both date and time in the string.
	//		Formatting patterns are chosen appropriate to the locale.  Different
	//		formatting lengths may be chosen, with "full" used by default.
	//		Custom patterns may be used or registered with translations using
	//		the dojo.date.locale.addCustomFormats method.
	//
	//		Formatting patterns are implemented using [the syntax described at
	//		unicode.org](http://www.unicode.org/reports/tr35/tr35-4.html#Date_Format_Patterns)
	//		When two digit years are used, a century is chosen according to a sliding
	//		window of 80 years before and 20 years after present year, for both `yy` and `yyyy` patterns.
	//		year < 100CE requires strict mode.
	//
	// value:
	//		A string representation of a date

	// remove non-printing bidi control chars from input and pattern
	var controlChars = /[\u200E\u200F\u202A\u202E]/g,
		info = dojo.date.locale._parseInfo(options),
		tokens = info.tokens, bundle = info.bundle,
		re = new RegExp("^" + info.regexp.replace(controlChars, "") + "$",
			info.strict ? "" : "i"),
		match = re.exec(value && value.replace(controlChars, ""));

	if(!match){ return null; } // null

	var widthList = ['abbr', 'wide', 'narrow'],
		result = [1970,0,1,0,0,0,0], // will get converted to a Date at the end
		amPm = "",
		valid = dojo.every(match, function(v, i){
		if(!i){return true;}
		var token=tokens[i-1];
		var l=token.length;
		switch(token.charAt(0)){
			case 'y':
				if(l != 2 && options.strict){
					//interpret year literally, so '5' would be 5 A.D.
					result[0] = v;
				}else{
					if(v<100){
						v = Number(v);
						//choose century to apply, according to a sliding window
						//of 80 years before and 20 years after present year
						var year = '' + new Date().getFullYear(),
							century = year.substring(0, 2) * 100,
							cutoff = Math.min(Number(year.substring(2, 4)) + 20, 99);
						result[0] = (v < cutoff) ? century + v : century - 100 + v;
					}else{
						//we expected 2 digits and got more...
						if(options.strict){
							return false;
						}
						//interpret literally, so '150' would be 150 A.D.
						//also tolerate '1950', if 'yyyy' input passed to 'yy' format
						result[0] = v;
					}
				}
				break;
			case 'M':
				if(l>2){
					var months = bundle['months-format-' + widthList[l-3]].concat();
					if(!options.strict){
						//Tolerate abbreviating period in month part
						//Case-insensitive comparison
						v = v.replace(".","").toLowerCase();
						months = dojo.map(months, function(s){ return s.replace(".","").toLowerCase(); } );
					}
					v = dojo.indexOf(months, v);
					if(v == -1){
//						console.log("dojo.date.locale.parse: Could not parse month name: '" + v + "'.");
						return false;
					}
				}else{
					v--;
				}
				result[1] = v;
				break;
			case 'E':
			case 'e':
				var days = bundle['days-format-' + widthList[l-3]].concat();
				if(!options.strict){
					//Case-insensitive comparison
					v = v.toLowerCase();
					days = dojo.map(days, function(d){return d.toLowerCase();});
				}
				v = dojo.indexOf(days, v);
				if(v == -1){
//					console.log("dojo.date.locale.parse: Could not parse weekday name: '" + v + "'.");
					return false;
				}

				//TODO: not sure what to actually do with this input,
				//in terms of setting something on the Date obj...?
				//without more context, can't affect the actual date
				//TODO: just validate?
				break;
			case 'D':
				result[1] = 0;
				// fallthrough...
			case 'd':
				result[2] = v;
				break;
			case 'a': //am/pm
				var am = options.am || bundle['dayPeriods-format-wide-am'],
					pm = options.pm || bundle['dayPeriods-format-wide-pm'];
				if(!options.strict){
					var period = /\./g;
					v = v.replace(period,'').toLowerCase();
					am = am.replace(period,'').toLowerCase();
					pm = pm.replace(period,'').toLowerCase();
				}
				if(options.strict && v != am && v != pm){
//					console.log("dojo.date.locale.parse: Could not parse am/pm part.");
					return false;
				}

				// we might not have seen the hours field yet, so store the state and apply hour change later
				amPm = (v == pm) ? 'p' : (v == am) ? 'a' : '';
				break;
			case 'K': //hour (1-24)
				if(v == 24){ v = 0; }
				// fallthrough...
			case 'h': //hour (1-12)
			case 'H': //hour (0-23)
			case 'k': //hour (0-11)
				//TODO: strict bounds checking, padding
				if(v > 23){
//					console.log("dojo.date.locale.parse: Illegal hours value");
					return false;
				}

				//in the 12-hour case, adjusting for am/pm requires the 'a' part
				//which could come before or after the hour, so we will adjust later
				result[3] = v;
				break;
			case 'm': //minutes
				result[4] = v;
				break;
			case 's': //seconds
				result[5] = v;
				break;
			case 'S': //milliseconds
				result[6] = v;
//				break;
//			case 'w':
//TODO				var firstDay = 0;
//			default:
//TODO: throw?
//				console.log("dojo.date.locale.parse: unsupported pattern char=" + token.charAt(0));
		}
		return true;
	});

	var hours = +result[3];
	if(amPm === 'p' && hours < 12){
		result[3] = hours + 12; //e.g., 3pm -> 15
	}else if(amPm === 'a' && hours == 12){
		result[3] = 0; //12am -> 0
	}

	//TODO: implement a getWeekday() method in order to test
	//validity of input strings containing 'EEE' or 'EEEE'...

	var dateObject = new Date(result[0], result[1], result[2], result[3], result[4], result[5], result[6]); // Date
	if(options.strict){
		dateObject.setFullYear(result[0]);
	}

	// Check for overflow.  The Date() constructor normalizes things like April 32nd...
	//TODO: why isn't this done for times as well?
	var allTokens = tokens.join(""),
		dateToken = allTokens.indexOf('d') != -1,
		monthToken = allTokens.indexOf('M') != -1;

	if(!valid ||
		(monthToken && dateObject.getMonth() > result[1]) ||
		(dateToken && dateObject.getDate() > result[2])){
		return null;
	}

	// Check for underflow, due to DST shifts.  See #9366
	// This assumes a 1 hour dst shift correction at midnight
	// We could compare the timezone offset after the shift and add the difference instead.
	if((monthToken && dateObject.getMonth() < result[1]) ||
		(dateToken && dateObject.getDate() < result[2])){
		dateObject = date.add(dateObject, "hour", 1);
	}

	return dateObject; // Date
};

function _processPattern(pattern, applyPattern, applyLiteral, applyAll){
	//summary: Process a pattern with literals in it

	// Break up on single quotes, treat every other one as a literal, except '' which becomes '
	var identity = function(x){return x;};
	applyPattern = applyPattern || identity;
	applyLiteral = applyLiteral || identity;
	applyAll = applyAll || identity;

	//split on single quotes (which escape literals in date format strings)
	//but preserve escaped single quotes (e.g., o''clock)
	var chunks = pattern.match(/(''|[^'])+/g),
		literal = pattern.charAt(0) == "'";

	dojo.forEach(chunks, function(chunk, i){
		if(!chunk){
			chunks[i]='';
		}else{
			chunks[i]=(literal ? applyLiteral : applyPattern)(chunk.replace(/''/g, "'"));
			literal = !literal;
		}
	});
	return applyAll(chunks.join(''));
}

function _buildDateTimeRE(tokens, bundle, options, pattern){
	pattern = regexp.escapeString(pattern);
	if(!options.strict){ pattern = pattern.replace(" a", " ?a"); } // kludge to tolerate no space before am/pm
	return pattern.replace(/([a-z])\1*/ig, function(match){
		// Build a simple regexp.  Avoid captures, which would ruin the tokens list
		var s,
			c = match.charAt(0),
			l = match.length,
			p2 = '', p3 = '';
		if(options.strict){
			if(l > 1){ p2 = '0' + '{'+(l-1)+'}'; }
			if(l > 2){ p3 = '0' + '{'+(l-2)+'}'; }
		}else{
			p2 = '0?'; p3 = '0{0,2}';
		}
		switch(c){
			case 'y':
				s = '\\d{2,4}';
				break;
			case 'M':
				s = (l>2) ? '\\S+?' : '1[0-2]|'+p2+'[1-9]';
				break;
			case 'D':
				s = '[12][0-9][0-9]|3[0-5][0-9]|36[0-6]|'+p2+'[1-9][0-9]|'+p3+'[1-9]';
				break;
			case 'd':
				s = '3[01]|[12]\\d|'+p2+'[1-9]';
				break;
			case 'w':
				s = '[1-4][0-9]|5[0-3]|'+p2+'[1-9]';
				break;
			case 'E':
				s = '\\S+';
				break;
			case 'h': //hour (1-12)
				s = '1[0-2]|'+p2+'[1-9]';
				break;
			case 'k': //hour (0-11)
				s = '1[01]|'+p2+'\\d';
				break;
			case 'H': //hour (0-23)
				s = '1\\d|2[0-3]|'+p2+'\\d';
				break;
			case 'K': //hour (1-24)
				s = '1\\d|2[0-4]|'+p2+'[1-9]';
				break;
			case 'm':
			case 's':
				s = '[0-5]\\d';
				break;
			case 'S':
				s = '\\d{'+l+'}';
				break;
			case 'a':
				var am = options.am || bundle['dayPeriods-format-wide-am'],
					pm = options.pm || bundle['dayPeriods-format-wide-pm'];
					s = am + '|' + pm;
				if(!options.strict){
					if(am != am.toLowerCase()){ s += '|' + am.toLowerCase(); }
					if(pm != pm.toLowerCase()){ s += '|' + pm.toLowerCase(); }
					if(s.indexOf('.') != -1){ s += '|' + s.replace(/\./g, ""); }
				}
				s = s.replace(/\./g, "\\.");
				break;
			default:
			// case 'v':
			// case 'z':
			// case 'Z':
				s = ".*";
//				console.log("parse of date format, pattern=" + pattern);
		}

		if(tokens){ tokens.push(match); }

		return "(" + s + ")"; // add capture
	}).replace(/[\xa0 ]/g, "[\\s\\xa0]"); // normalize whitespace.  Need explicit handling of \xa0 for IE.
}

var _customFormats = [];
dojo.date.locale.addCustomFormats = function(/*String*/packageName, /*String*/bundleName){
	// summary:
	//		Add a reference to a bundle containing localized custom formats to be
	//		used by date/time formatting and parsing routines.
	//
	// description:
	//		The user may add custom localized formats where the bundle has properties following the
	//		same naming convention used by dojo.cldr: `dateFormat-xxxx` / `timeFormat-xxxx`
	//		The pattern string should match the format used by the CLDR.
	//		See dojo.date.locale.format() for details.
	//		The resources must be loaded by dojo.requireLocalization() prior to use

	_customFormats.push({pkg:packageName,name:bundleName});
};

dojo.date.locale._getGregorianBundle = function(/*String*/locale){
	var gregorian = {};
	dojo.forEach(_customFormats, function(desc){
		var bundle = dojo.i18n.getLocalization(desc.pkg, desc.name, locale);
		gregorian = lang.mixin(gregorian, bundle);
	}, this);
	return gregorian; /*Object*/
};

dojo.date.locale.addCustomFormats("dojo.cldr","gregorian");

dojo.date.locale.getNames = function(/*String*/item, /*String*/type, /*String?*/context, /*String?*/locale){
	// summary:
	//		Used to get localized strings from dojo.cldr for day or month names.
	//
	// item:
	//	'months' || 'days'
	// type:
	//	'wide' || 'abbr' || 'narrow' (e.g. "Monday", "Mon", or "M" respectively, in English)
	// context:
	//	'standAlone' || 'format' (default)
	// locale:
	//	override locale used to find the names

	var label,
		lookup = dojo.date.locale._getGregorianBundle(locale),
		props = [item, context, type];
	if(context == 'standAlone'){
		var key = props.join('-');
		label = lookup[key];
		// Fall back to 'format' flavor of name
		if(label[0] == 1){ label = undefined; } // kludge, in the absence of real aliasing support in dojo.cldr
	}
	props[1] = 'format';

	// return by copy so changes won't be made accidentally to the in-memory model
	return (label || lookup[props.join('-')]).concat(); /*Array*/
};

dojo.date.locale.isWeekend = function(/*Date?*/dateObject, /*String?*/locale){
	// summary:
	//	Determines if the date falls on a weekend, according to local custom.

	var weekend = cldr.getWeekend(locale),
		day = (dateObject || new Date()).getDay();
	if(weekend.end < weekend.start){
		weekend.end += 7;
		if(day < weekend.start){ day += 7; }
	}
	return day >= weekend.start && day <= weekend.end; // Boolean
};

// These are used only by format and strftime.  Do they need to be public?  Which module should they go in?

dojo.date.locale._getDayOfYear = function(/*Date*/dateObject){
	// summary: gets the day of the year as represented by dateObject
	return date.difference(new Date(dateObject.getFullYear(), 0, 1, dateObject.getHours()), dateObject) + 1; // Number
};

dojo.date.locale._getWeekOfYear = function(/*Date*/dateObject, /*Number*/firstDayOfWeek){
	if(arguments.length == 1){ firstDayOfWeek = 0; } // Sunday

	var firstDayOfYear = new Date(dateObject.getFullYear(), 0, 1).getDay(),
		adj = (firstDayOfYear - firstDayOfWeek + 7) % 7,
		week = Math.floor((dojo.date.locale._getDayOfYear(dateObject) + adj - 1) / 7);

	// if year starts on the specified day, start counting weeks at 1
	if(firstDayOfYear == firstDayOfWeek){ week++; }

	return week; // Number
};

return dojo.date.locale;
});

},
'gridx/modules/extendedSelect/Row':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/query",
	"dojo/_base/lang",
	"dojo/_base/Deferred",
	"dojo/_base/sniff",
	"dojo/dom-class",
	"dojo/mouse",
	"dojo/keys",
	"../../core/_Module",
	"./_RowCellBase"
], function(declare, array, query, lang, Deferred, sniff, domClass, mouse, keys, _Module, _RowCellBase){

	return declare(/*===== "gridx.modules.extendedSelect.Row", =====*/_RowCellBase, {
		// summary:
		//		Provides advanced row selections.
		// description:
		//		This module provides an advanced way for selecting rows by clicking, swiping, SPACE key, or CTRL/SHIFT CLICK to select multiple rows.
		//
		// example:
		//		1. Use select api on grid row object obtained from grid.row(i)
		//		|	grid.row(1).select();
		//		|	grid.row(1).deselect();
		//		|	grid.row(1).isSelected();
		//
		//		2. Use select api on select.row module
		//		|	grid.select.row.selectById(rowId);
		//		|	grid.select.row.deSelectById(rowId);
		//		|	grid.select.row.isSelected(rowId);
		//		|	grid.select.row.getSelected();//[]
		//		|	grid.select.row.clear();

		// name: [readonly] String
		//		module name		
		name: 'selectRow',

		// rowMixin: Object
		//		A map of functions to be mixed into grid row object, so that we can use select api on row object directly
		//		- grid.row(1).select() | deselect() | isSelected();
		rowMixin: {
			select: function(){
				this.grid.select.row.selectById(this.id);
				return this;
			},

			deselect: function(){
				this.grid.select.row.deselectById(this.id);
				return this;
			},

			isSelected: function(){
				return this.model.getMark(this.id) === true;
			}
		},
		
		//Public-----------------------------------------------------------------
		
		// triggerOnCell: [readonly] Boolean
		//		Whether row will be selected by clicking on cell, false by default		
		triggerOnCell: false,

		// treeMode: Boolean
		//		Whether to apply tri-state selection for child rows.
		treeMode: true,

/*=====
		selectById: function(rowId){
			// summary:
			//		Select a row by id.
		},
		
		deselectById: function(rowId){
			// summary:
			//		Deselect a row by id.
		},
		
		selectByIndex: function(rowIndex){
			// summary:
			//		Select a row by index
		},
		
		deSelectByIndex: function(rowIndex){
			// summary:
			//		Deselect a row by index.
		},
		
=====*/
		getSelected: function(){
			// summary:
			//		Get id array of all selected rows
			return this.model.getMarkedIds();
		},

		isSelected: function(){
			// summary:
			//		Check if the given rows are all selected.
			return array.every(arguments, function(id){
				return this.model.getMark(id) === true;
			}, this);
		},

		clear: function(silent){
			// summary:
			//		Deselected all selected rows;			
			query(".gridxRowSelected", this.grid.mainNode).forEach(function(node){
				domClass.remove(node, 'gridxRowSelected');
				node.removeAttribute('aria-selected');
			});
			query(".gridxRowPartialSelected", this.grid.mainNode).forEach(function(node){
				domClass.remove(node, 'gridxRowPartialSelected');
			});
			this._clear();
			this.model.clearMark();
			if(!silent){
				this._onSelectionChange();
			}
		},

		onHighlightChange: function(){},
		
		//Private---------------------------------------------------------------------
		_type: 'row',

		_init: function(){
			var t = this,
				g = t.grid;
			t.model.treeMarkMode('', t.arg('treeMode'));
			t.inherited(arguments);
			//Use special types to make filtered out rows unselected
			t.model._spTypes.select = 1;	//1 as true
			t.batchConnect(
				g.rowHeader && [g.rowHeader, 'onMoveToRowHeaderCell', '_onMoveToRowHeaderCell'],
				[g, 'onRowMouseDown', function(e){
					if(mouse.isLeft(e) && ((t.arg('triggerOnCell') &&
						!domClass.contains(e.target, 'gridxTreeExpandoIcon') &&
						!domClass.contains(e.target, 'gridxTreeExpandoInner')) || !e.columnId)){
						t._isOnCell = e.columnId;
						if(t._isOnCell){
							g.body._focusCellCol = e.columnIndex;
						}
						t._start({row: e.visualIndex}, g._isCopyEvent(e), e.shiftKey);
					}
				}],
				[g, 'onRowMouseOver', function(e){
					if(t._selecting && t.arg('triggerOnCell') && e.columnId){
						g.body._focusCellCol = e.columnIndex;
					}
					t._highlight({row: e.visualIndex});
				}],
				[g, sniff('ff') < 4 ? 'onRowKeyUp' : 'onRowKeyDown', function(e){
					if(e.keyCode == keys.SPACE && (!e.columnId ||
							//When trigger on cell, check if we are navigating on body, reducing the odds of conflictions.
							(t.arg('triggerOnCell') && (!g.focus || g.focus.currentArea() == 'body')))){
						t._isOnCell = e.columnId;
						t._start({row: e.visualIndex}, g._isCopyEvent(e), e.shiftKey);
						t._end();
					}
				}]);
		},

		_markById: function(args, toSelect){
			var m = this.model;
			array.forEach(args, function(arg){
				m.markById(arg, toSelect);
			});
			m.when();
		},

		_markByIndex: function(args, toSelect){
			var g = this.grid,
				m = this.model,
				body = g.body,
				rowInfo;
			array.forEach(args, function(arg){
				if(lang.isArrayLike(arg)){
					var start = arg[0],
						end = arg[1],
						i, count;
					if(start >= 0 && start < Infinity){
						if(end >= start && end < Infinity){
							count = end - start + 1;
						}else{
							count = body.visualCount - start;
						}
						rowInfo = body.getRowInfo({visualIndex: start});
						start = rowInfo.rowIndex;
						for(i = 0; i < count; ++i){
							m.markByIndex(i + start, toSelect, '', rowInfo.parentId);
						}
					}
				}else if(arg >= 0 && arg < Infinity){
					rowInfo = body.getRowInfo({visualIndex: arg});
					m.markByIndex(rowInfo.rowIndex, toSelect, '', rowInfo.parentId);
				}
			});
			return m.when();
		},

		_onRender: function(start, count){
			var t = this, i, end = start + count;
			for(i = start; i < end; ++i){
				var item = {row: i};
				if(t._isSelected(item) || (t._selecting && t._toSelect && 
					t._inRange(i, t._startItem.row, t._currentItem.row, 1))){	//1 as true
					t._doHighlight(item, 1);	//1 as true
				}
			}
		},

		_onMark: function(id, toMark, oldState, type){
			if(type == 'select'){
				var nodes = query('[rowid="' + id + '"]', this.grid.mainNode);
				if(nodes.length){
					nodes.forEach(function(node){
						var selected = toMark && toMark != 'mixed';
						domClass.toggle(node, 'gridxRowSelected', selected);
						domClass.toggle(node, 'gridxRowPartialSelected', toMark == 'mixed');
						node.setAttribute('aria-selected', !!selected);
					});
					this.onHighlightChange({row: parseInt(nodes[0].getAttribute('visualindex'), 10)}, toMark);
				}
			}
		},

		_onMoveToCell: function(rowVisIndex, colIndex, e){
			var t = this;
			if(t.arg('triggerOnCell') && e.shiftKey && (e.keyCode == keys.UP_ARROW || e.keyCode == keys.DOWN_ARROW)){
				t._start({row: rowVisIndex}, t.grid._isCopyEvent(e), 1);	//1 as true
				t._end();
			}
		},

		_onMoveToRowHeaderCell: function(rowVisIndex, e){
			if(e.shiftKey){
				this._start({row: rowVisIndex}, this.grid._isCopyEvent(e), 1);	//1 as true
				this._end();
			}
		},

		_isSelected: function(target){
			var t = this,
				id = t._getRowId(target.row);
			return t._isRange ? array.indexOf(t._refSelectedIds, id) >= 0 : t.model.getMark(id);
		},

		_beginAutoScroll: function(){
			var autoScroll = this.grid.autoScroll;
			this._autoScrollH = autoScroll.horizontal;
			autoScroll.horizontal = false;
		},

		_endAutoScroll: function(){
			this.grid.autoScroll.horizontal = this._autoScrollH;
		},

		_doHighlight: function(target, toHighlight){
			query('[visualindex="' + target.row + '"]', this.grid.mainNode).forEach(function(node){
				domClass.toggle(node, 'gridxRowSelected', toHighlight);
				node.setAttribute('aria-selected', !!toHighlight);
			});
			this.onHighlightChange(target, toHighlight);
		},

		_end: function(){
			this.inherited(arguments);
			delete this._isOnCell;
		},

		_focus: function(target){
			var g = this.grid, focus = g.focus;
			if(focus){
				g.body._focusCellRow = target.row;
				focus.focusArea(this._isOnCell ? 'body' : 'rowHeader', true);
			}
		},

		_addToSelected: function(start, end, toSelect){
			var t = this,
				bd = t.grid.body,
				m = t.model,
				lastEndItem = t._lastEndItem,
				a, b, i, d;
			if(!t._isRange){
				t._refSelectedIds = m.getMarkedIds();
			}
			if(t._isRange && t._inRange(end.row, start.row, lastEndItem.row)){
				a = Math.min(end.row, lastEndItem.row);
				b = Math.max(end.row, lastEndItem.row);
				start = bd.getRowInfo({visualIndex: a}).rowIndex + 1;
				end = bd.getRowInfo({visualIndex: b}).rowIndex;
				d = new Deferred();
				m.when({
					start: start, 
					count: end - start + 1
				}, function(){
					for(i = start; i <= end; ++i){
						var id = m.indexToId(i),
							selected = array.indexOf(t._refSelectedIds, id) >= 0;
						m.markById(id, selected); 
					}
				}).then(function(){
					m.when(null, function(){
						d.callback();
					});
				});
				return d;
			}else{
				a = Math.min(start.row, end.row);
				b = Math.max(start.row, end.row);

				for(i = a; i <= b; ++i){
					var rowInfo = bd.getRowInfo({visualIndex: i});
					m.markByIndex(rowInfo.rowIndex, toSelect, '', rowInfo.parentId);
				}
				return m.when();
			}
		}
	});
});

},
'ibm/tivoli/tip/navwidget/dijit/DefaultMessageHandler':function(){
// wrapped by build app
define("ibm/tivoli/tip/navwidget/dijit/DefaultMessageHandler", ["dijit","dojo","dojox","dojo/i18n!ibm/tivoli/tip/navwidget/nls/resources","dojo/i18n!ibm/tivoli/tip/navwidget/nls/messages","dojo/require!ibm/tivoli/navmodel/Trace,ibm/tivoli/navmodel/Exception,dojoe/messagedialog/MessageDialog"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.dijit.DefaultMessageHandler");

dojo.require("ibm.tivoli.navmodel.Trace");
dojo.require("ibm.tivoli.navmodel.Exception");
dojo.require("dojoe.messagedialog.MessageDialog");
dojo.requireLocalization("ibm.tivoli.tip.navwidget", "resources");
dojo.requireLocalization("ibm.tivoli.tip.navwidget", "messages");

ibm.tivoli.tip.navwidget.dijit.DefaultMessageHandler._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.tip.navwidget.dijit.DefaultMessageHandler");

dojo.declare("ibm.tivoli.tip.navwidget.dijit.DefaultMessageHandler",
	      null,
{		
	
	// _trace: [private] ibm.tivoli.navmodel.Trace
	//		Utility for trace and logging
	_trace: ibm.tivoli.tip.navwidget.dijit.DefaultMessageHandler._trace,
	
	//Base constructor
	constructor: function () {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		
		//NLS
		this._resources = dojo.i18n.getLocalization("ibm.tivoli.tip.navwidget", "resources");
		this._messages = dojo.i18n.getLocalization("ibm.tivoli.tip.navwidget", "messages");
	},
	
	displayException: function (/*Object*/ ex, /*Object*/dontTraceMessage) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon

		var msgId = null;
		var msgSeverity = null;
		var msgText = null;

		//Checking for existence of isInstanceOf before calling it because normal JS exception won't have an isInstanceOf
		//Also check that msgId, msgSeverity and msgText exist otherwise popup may not display
		if (ex && ex.isInstanceOf && ex.isInstanceOf(ibm.tivoli.navmodel.Exception) && ex.getMsgId() && ex.getMsgSeverity() && ex.getMsgText()) { 
			msgId = ex.getMsgId();
			msgSeverity = ex.getMsgSeverity();
			msgText = ex.getMsgText();
		} else {
			
			//We have one of the following two messages to choose from. I don't think we're going to have enough in the exception to know if one of these is more appropriate than the
			//other. Also, I don't think we want to do the substitution for ${error_txt} since likely not translated.
			//
			//	- "TCTUI1002E":"An error has occurred.  The data provider could not return the data. ${error_txt}", // ${error_txt} will contain the error message returned from the server
			//	- "TCTUI1003E":"An error has occurred communicating with the server. ${error_txt}"  // ${error_txt} will contain the error message returned from the server
			
			msgId = "TCTUI1003E";
			msgSeverity = "Error";

			var messageTemplate = this._messages.TCTUI1003E;
			msgText = dojo.string.substitute(messageTemplate,
                    {error_txt: ""}); //Doing an empty substitution since message in generic exception probably won't be translated
		}

		//Display the message
		this.displayMessage(msgId, msgSeverity, msgText); 
		
		//Let's trace to the console window too
		if (!dontTraceMessage) {
			if (this._trace.isError()) {
				var methodName = "handleException";
				this._trace.error(ex, methodName);
			}
		}
	},
	
	displayMessage: function(/*Object*/ msgId, /*Object*/ msgSeverity, /*Object*/ msgText) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		
		var msgDlg = new dojoe.messagedialog.MessageDialog(
				{message: msgText,
				 messageId: msgId,
				 type: msgSeverity});
		msgDlg.show();
	}
});
});

},
'url:dijit/layout/templates/_ScrollingTabControllerButton.html':"<div data-dojo-attach-event=\"onclick:_onClick\">\n\t<div role=\"presentation\" class=\"dijitTabInnerDiv\" data-dojo-attach-point=\"innerDiv,focusNode\">\n\t\t<div role=\"presentation\" class=\"dijitTabContent dijitButtonContents\" data-dojo-attach-point=\"tabContent\">\n\t\t\t<img role=\"presentation\" alt=\"\" src=\"${_blankGif}\" class=\"dijitTabStripIcon\" data-dojo-attach-point=\"iconNode\"/>\n\t\t\t<span data-dojo-attach-point=\"containerNode,titleNode\" class=\"dijitButtonText\"></span>\n\t\t</div>\n\t</div>\n</div>",
'dojo/cldr/monetary':function(){
define(["../main"], function(dojo) {
	// module:
	//		dojo/cldr/monetary
	// summary:
	//		TODOC

dojo.getObject("cldr.monetary", true, dojo);

dojo.cldr.monetary.getData = function(/*String*/code){
// summary: A mapping of currency code to currency-specific formatting information. Returns a unique object with properties: places, round.
// code: an [ISO 4217](http://en.wikipedia.org/wiki/ISO_4217) currency code

// from http://www.unicode.org/cldr/data/common/supplemental/supplementalData.xml:supplementalData/currencyData/fractions

	var placesData = {
		ADP:0,AFN:0,ALL:0,AMD:0,BHD:3,BIF:0,BYR:0,CLF:0,CLP:0,
		COP:0,CRC:0,DJF:0,ESP:0,GNF:0,GYD:0,HUF:0,IDR:0,IQD:0,
		IRR:3,ISK:0,ITL:0,JOD:3,JPY:0,KMF:0,KPW:0,KRW:0,KWD:3,
		LAK:0,LBP:0,LUF:0,LYD:3,MGA:0,MGF:0,MMK:0,MNT:0,MRO:0,
		MUR:0,OMR:3,PKR:0,PYG:0,RSD:0,RWF:0,SLL:0,SOS:0,STD:0,
		SYP:0,TMM:0,TND:3,TRL:0,TZS:0,UGX:0,UZS:0,VND:0,VUV:0,
		XAF:0,XOF:0,XPF:0,YER:0,ZMK:0,ZWD:0
	};

	var roundingData = {CHF:5};

	var places = placesData[code], round = roundingData[code];
	if(typeof places == "undefined"){ places = 2; }
	if(typeof round == "undefined"){ round = 0; }

	return {places: places, round: round}; // Object
};

return dojo.cldr.monetary;
});

},
'url:gridx/templates/FilterDialog.html':"<form action=\"./\" onsubmit=\"return false;\">\n<label id=\"${id}_MatchOptionLabel\" for=\"${id}_MatchOptionSelect\">${i18n.relationMsgFront} </label>\n<select data-dojo-type=\"dijit.form.Select\" id=\"${id}_MatchOptionSelect\" aria-labelledby=\"${id}_MatchOptionLabel\">\n\t<option value=\"all\">${i18n.relationAll}</option>\n\t<option value=\"any\">${i18n.relationAny}</option>\n</select>\n\n<div class=\"gridxFilterAccordionWrapper\">\n\t<div data-dojo-type=\"dijit.layout.AccordionContainer\"></div>\n</div>\n\n<div class=\"gridxFilterDialogButtons\">\n\t<input type=\"button\" class=\"gridxFilterDialogBtnAdd\" data-dojo-type=\"dijit.form.Button\" \n\t\t   showLabel=\"false\" iconClass=\"gridxFilterBtnAddRule\" label=\"${i18n.addRuleButton}\"/>\n\t<input type=\"submit\" data-dojo-type=\"dijit.form.Button\" label=\"${i18n.filterButton}\"/>\n\t<input type=\"button\" data-dojo-type=\"dijit.form.Button\" label=\"${i18n.clearButton}\"/>\n\t<input type=\"button\" data-dojo-type=\"dijit.form.Button\" label=\"${i18n.cancelButton}\"/>\n</div>\n</form>\n",
'dijit/_MenuBase':function(){
define([
	"./popup",
	"dojo/window",
	"./_Widget",
	"./_KeyNavContainer",
	"./_TemplatedMixin",
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.isDescendant domClass.replace
	"dojo/dom-attr",
	"dojo/dom-class", // domClass.replace
	"dojo/_base/lang", // lang.hitch
	"dojo/_base/array"	// array.indexOf
], function(pm, winUtils, _Widget, _KeyNavContainer, _TemplatedMixin,
	declare, dom, domAttr, domClass, lang, array){

/*=====
	var _Widget = dijit._Widget;
	var _TemplatedMixin = dijit._TemplatedMixin;
	var _KeyNavContainer = dijit._KeyNavContainer;
=====*/

// module:
//		dijit/_MenuBase
// summary:
//		Base class for Menu and MenuBar

return declare("dijit._MenuBase",
	[_Widget, _TemplatedMixin, _KeyNavContainer],
{
	// summary:
	//		Base class for Menu and MenuBar

	// parentMenu: [readonly] Widget
	//		pointer to menu that displayed me
	parentMenu: null,

	// popupDelay: Integer
	//		number of milliseconds before hovering (without clicking) causes the popup to automatically open.
	popupDelay: 500,

	onExecute: function(){
		// summary:
		//		Attach point for notification about when a menu item has been executed.
		//		This is an internal mechanism used for Menus to signal to their parent to
		//		close them, because they are about to execute the onClick handler.  In
		//		general developers should not attach to or override this method.
		// tags:
		//		protected
	},

	onCancel: function(/*Boolean*/ /*===== closeAll =====*/){
		// summary:
		//		Attach point for notification about when the user cancels the current menu
		//		This is an internal mechanism used for Menus to signal to their parent to
		//		close them.  In general developers should not attach to or override this method.
		// tags:
		//		protected
	},

	_moveToPopup: function(/*Event*/ evt){
		// summary:
		//		This handles the right arrow key (left arrow key on RTL systems),
		//		which will either open a submenu, or move to the next item in the
		//		ancestor MenuBar
		// tags:
		//		private

		if(this.focusedChild && this.focusedChild.popup && !this.focusedChild.disabled){
			this.focusedChild._onClick(evt);
		}else{
			var topMenu = this._getTopMenu();
			if(topMenu && topMenu._isMenuBar){
				topMenu.focusNext();
			}
		}
	},

	_onPopupHover: function(/*Event*/ /*===== evt =====*/){
		// summary:
		//		This handler is called when the mouse moves over the popup.
		// tags:
		//		private

		// if the mouse hovers over a menu popup that is in pending-close state,
		// then stop the close operation.
		// This can't be done in onItemHover since some popup targets don't have MenuItems (e.g. ColorPicker)
		if(this.currentPopup && this.currentPopup._pendingClose_timer){
			var parentMenu = this.currentPopup.parentMenu;
			// highlight the parent menu item pointing to this popup
			if(parentMenu.focusedChild){
				parentMenu.focusedChild._setSelected(false);
			}
			parentMenu.focusedChild = this.currentPopup.from_item;
			parentMenu.focusedChild._setSelected(true);
			// cancel the pending close
			this._stopPendingCloseTimer(this.currentPopup);
		}
	},

	onItemHover: function(/*MenuItem*/ item){
		// summary:
		//		Called when cursor is over a MenuItem.
		// tags:
		//		protected

		// Don't do anything unless user has "activated" the menu by:
		//		1) clicking it
		//		2) opening it from a parent menu (which automatically focuses it)
		if(this.isActive){
			this.focusChild(item);
			if(this.focusedChild.popup && !this.focusedChild.disabled && !this.hover_timer){
				this.hover_timer = setTimeout(lang.hitch(this, "_openPopup"), this.popupDelay);
			}
		}
		// if the user is mixing mouse and keyboard navigation,
		// then the menu may not be active but a menu item has focus,
		// but it's not the item that the mouse just hovered over.
		// To avoid both keyboard and mouse selections, use the latest.
		if(this.focusedChild){
			this.focusChild(item);
		}
		this._hoveredChild = item;
	},

	_onChildBlur: function(item){
		// summary:
		//		Called when a child MenuItem becomes inactive because focus
		//		has been removed from the MenuItem *and* it's descendant menus.
		// tags:
		//		private
		this._stopPopupTimer();
		item._setSelected(false);
		// Close all popups that are open and descendants of this menu
		var itemPopup = item.popup;
		if(itemPopup){
			this._stopPendingCloseTimer(itemPopup);
			itemPopup._pendingClose_timer = setTimeout(function(){
				itemPopup._pendingClose_timer = null;
				if(itemPopup.parentMenu){
					itemPopup.parentMenu.currentPopup = null;
				}
				pm.close(itemPopup); // this calls onClose
			}, this.popupDelay);
		}
	},

	onItemUnhover: function(/*MenuItem*/ item){
		// summary:
		//		Callback fires when mouse exits a MenuItem
		// tags:
		//		protected

		if(this.isActive){
			this._stopPopupTimer();
		}
		if(this._hoveredChild == item){ this._hoveredChild = null; }
	},

	_stopPopupTimer: function(){
		// summary:
		//		Cancels the popup timer because the user has stop hovering
		//		on the MenuItem, etc.
		// tags:
		//		private
		if(this.hover_timer){
			clearTimeout(this.hover_timer);
			this.hover_timer = null;
		}
	},

	_stopPendingCloseTimer: function(/*dijit._Widget*/ popup){
		// summary:
		//		Cancels the pending-close timer because the close has been preempted
		// tags:
		//		private
		if(popup._pendingClose_timer){
			clearTimeout(popup._pendingClose_timer);
			popup._pendingClose_timer = null;
		}
	},

	_stopFocusTimer: function(){
		// summary:
		//		Cancels the pending-focus timer because the menu was closed before focus occured
		// tags:
		//		private
		if(this._focus_timer){
			clearTimeout(this._focus_timer);
			this._focus_timer = null;
		}
	},

	_getTopMenu: function(){
		// summary:
		//		Returns the top menu in this chain of Menus
		// tags:
		//		private
		for(var top=this; top.parentMenu; top=top.parentMenu);
		return top;
	},

	onItemClick: function(/*dijit._Widget*/ item, /*Event*/ evt){
		// summary:
		//		Handle clicks on an item.
		// tags:
		//		private

		// this can't be done in _onFocus since the _onFocus events occurs asynchronously
		if(typeof this.isShowingNow == 'undefined'){ // non-popup menu
			this._markActive();
		}

		this.focusChild(item);

		if(item.disabled){ return false; }

		if(item.popup){
			this._openPopup();
		}else{
			// before calling user defined handler, close hierarchy of menus
			// and restore focus to place it was when menu was opened
			this.onExecute();

			// user defined handler for click
			item.onClick(evt);
		}
	},

	_openPopup: function(){
		// summary:
		//		Open the popup to the side of/underneath the current menu item
		// tags:
		//		protected

		this._stopPopupTimer();
		var from_item = this.focusedChild;
		if(!from_item){ return; } // the focused child lost focus since the timer was started
		var popup = from_item.popup;
		if(popup.isShowingNow){ return; }
		if(this.currentPopup){
			this._stopPendingCloseTimer(this.currentPopup);
			pm.close(this.currentPopup);
		}
		popup.parentMenu = this;
		popup.from_item = from_item; // helps finding the parent item that should be focused for this popup
		var self = this;
		pm.open({
			parent: this,
			popup: popup,
			around: from_item.domNode,
			orient: this._orient || ["after", "before"],
			onCancel: function(){ // called when the child menu is canceled
				// set isActive=false (_closeChild vs _cleanUp) so that subsequent hovering will NOT open child menus
				// which seems aligned with the UX of most applications (e.g. notepad, wordpad, paint shop pro)
				self.focusChild(from_item);	// put focus back on my node
				self._cleanUp();			// close the submenu (be sure this is done _after_ focus is moved)
				from_item._setSelected(true); // oops, _cleanUp() deselected the item
				self.focusedChild = from_item;	// and unset focusedChild
			},
			onExecute: lang.hitch(this, "_cleanUp")
		});

		this.currentPopup = popup;
		// detect mouseovers to handle lazy mouse movements that temporarily focus other menu items
		popup.connect(popup.domNode, "onmouseenter", lang.hitch(self, "_onPopupHover")); // cleaned up when the popped-up widget is destroyed on close

		if(popup.focus){
			// If user is opening the popup via keyboard (right arrow, or down arrow for MenuBar),
			// if the cursor happens to collide with the popup, it will generate an onmouseover event
			// even though the mouse wasn't moved.  Use a setTimeout() to call popup.focus so that
			// our focus() call overrides the onmouseover event, rather than vice-versa.  (#8742)
			popup._focus_timer = setTimeout(lang.hitch(popup, function(){
				this._focus_timer = null;
				this.focus();
			}), 0);
		}
	},

	_markActive: function(){
		// summary:
		//		Mark this menu's state as active.
		//		Called when this Menu gets focus from:
		//			1) clicking it (mouse or via space/arrow key)
		//			2) being opened by a parent menu.
		//		This is not called just from mouse hover.
		//		Focusing a menu via TAB does NOT automatically set isActive
		//		since TAB is a navigation operation and not a selection one.
		//		For Windows apps, pressing the ALT key focuses the menubar
		//		menus (similar to TAB navigation) but the menu is not active
		//		(ie no dropdown) until an item is clicked.
		this.isActive = true;
		domClass.replace(this.domNode, "dijitMenuActive", "dijitMenuPassive");
	},

	onOpen: function(/*Event*/ /*===== e =====*/){
		// summary:
		//		Callback when this menu is opened.
		//		This is called by the popup manager as notification that the menu
		//		was opened.
		// tags:
		//		private

		this.isShowingNow = true;
		this._markActive();
	},

	_markInactive: function(){
		// summary:
		//		Mark this menu's state as inactive.
		this.isActive = false; // don't do this in _onBlur since the state is pending-close until we get here
		domClass.replace(this.domNode, "dijitMenuPassive", "dijitMenuActive");
	},

	onClose: function(){
		// summary:
		//		Callback when this menu is closed.
		//		This is called by the popup manager as notification that the menu
		//		was closed.
		// tags:
		//		private

		this._stopFocusTimer();
		this._markInactive();
		this.isShowingNow = false;
		this.parentMenu = null;
	},

	_closeChild: function(){
		// summary:
		//		Called when submenu is clicked or focus is lost.  Close hierarchy of menus.
		// tags:
		//		private
		this._stopPopupTimer();

		if(this.currentPopup){
			// If focus is on a descendant MenuItem then move focus to me,
			// because IE doesn't like it when you display:none a node with focus,
			// and also so keyboard users don't lose control.
			// Likely, immediately after a user defined onClick handler will move focus somewhere
			// else, like a Dialog.
			if(array.indexOf(this._focusManager.activeStack, this.id) >= 0){
				domAttr.set(this.focusedChild.focusNode, "tabIndex", this.tabIndex);
				this.focusedChild.focusNode.focus();
			}
			// Close all popups that are open and descendants of this menu
			pm.close(this.currentPopup);
			this.currentPopup = null;
		}

		if(this.focusedChild){ // unhighlight the focused item
			this.focusedChild._setSelected(false);
			this.focusedChild._onUnhover();
			this.focusedChild = null;
		}
	},

	_onItemFocus: function(/*MenuItem*/ item){
		// summary:
		//		Called when child of this Menu gets focus from:
		//			1) clicking it
		//			2) tabbing into it
		//			3) being opened by a parent menu.
		//		This is not called just from mouse hover.
		if(this._hoveredChild && this._hoveredChild != item){
			this._hoveredChild._onUnhover(); // any previous mouse movement is trumped by focus selection
		}
	},

	_onBlur: function(){
		// summary:
		//		Called when focus is moved away from this Menu and it's submenus.
		// tags:
		//		protected
		this._cleanUp();
		this.inherited(arguments);
	},

	_cleanUp: function(){
		// summary:
		//		Called when the user is done with this menu.  Closes hierarchy of menus.
		// tags:
		//		private

		this._closeChild(); // don't call this.onClose since that's incorrect for MenuBar's that never close
		if(typeof this.isShowingNow == 'undefined'){ // non-popup menu doesn't call onClose
			this._markInactive();
		}
	}
});

});

},
'dijit/layout/_ContentPaneResizeMixin':function(){
define([
	"dojo/_base/array", // array.filter array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom-attr",	// domAttr.has
	"dojo/dom-class",	// domClass.contains domClass.toggle
	"dojo/dom-geometry",// domGeometry.contentBox domGeometry.marginBox
	"dojo/_base/lang", // lang.mixin
	"dojo/query", // query
	"dojo/_base/sniff", // has("ie")
	"dojo/_base/window", // win.global
	"../registry",	// registry.byId
	"./utils",	// marginBox2contextBox
	"../_Contained"
], function(array, declare, domAttr, domClass, domGeometry, lang, query, has, win,
			registry, layoutUtils, _Contained){

/*=====
var _Contained = dijit._Contained;
=====*/

// module:
//		dijit/layout/_ContentPaneResizeMixin
// summary:
//		Resize() functionality of ContentPane.   If there's a single layout widget
//		child then it will call resize() with the same dimensions as the ContentPane.
//		Otherwise just calls resize on each child.


return declare("dijit.layout._ContentPaneResizeMixin", null, {
	// summary:
	//		Resize() functionality of ContentPane.   If there's a single layout widget
	//		child then it will call resize() with the same dimensions as the ContentPane.
	//		Otherwise just calls resize on each child.
	//
	//		Also implements basic startup() functionality, where starting the parent
	//		will start the children

	// doLayout: Boolean
	//		- false - don't adjust size of children
	//		- true - if there is a single visible child widget, set it's size to
	//				however big the ContentPane is
	doLayout: true,

	// isLayoutContainer: [protected] Boolean
	//		Indicates that this widget will call resize() on it's child widgets
	//		when they become visible.
	isLayoutContainer: true,

	startup: function(){
		// summary:
		//		See `dijit.layout._LayoutWidget.startup` for description.
		//		Although ContentPane doesn't extend _LayoutWidget, it does implement
		//		the same API.

		if(this._started){ return; }

		var parent = this.getParent();
		this._childOfLayoutWidget = parent && parent.isLayoutContainer;

		// I need to call resize() on my child/children (when I become visible), unless
		// I'm the child of a layout widget in which case my parent will call resize() on me and I'll do it then.
		this._needLayout = !this._childOfLayoutWidget;

		this.inherited(arguments);

		if(this._isShown()){
			this._onShow();
		}

		if(!this._childOfLayoutWidget){
			// If my parent isn't a layout container, since my style *may be* width=height=100%
			// or something similar (either set directly or via a CSS class),
			// monitor when my size changes so that I can re-layout.
			// For browsers where I can't directly monitor when my size changes,
			// monitor when the viewport changes size, which *may* indicate a size change for me.
			this.connect(has("ie") ? this.domNode : win.global, 'onresize', function(){
				// Using function(){} closure to ensure no arguments to resize.
				this._needLayout = !this._childOfLayoutWidget;
				this.resize();
			});
		}
	},

	_checkIfSingleChild: function(){
		// summary:
		//		Test if we have exactly one visible widget as a child,
		//		and if so assume that we are a container for that widget,
		//		and should propagate startup() and resize() calls to it.
		//		Skips over things like data stores since they aren't visible.

		var childNodes = query("> *", this.containerNode).filter(function(node){
				return node.tagName !== "SCRIPT"; // or a regexp for hidden elements like script|area|map|etc..
			}),
			childWidgetNodes = childNodes.filter(function(node){
				return domAttr.has(node, "data-dojo-type") || domAttr.has(node, "dojoType") || domAttr.has(node, "widgetId");
			}),
			candidateWidgets = array.filter(childWidgetNodes.map(registry.byNode), function(widget){
				return widget && widget.domNode && widget.resize;
			});

		if(
			// all child nodes are widgets
			childNodes.length == childWidgetNodes.length &&

			// all but one are invisible (like dojo.data)
			candidateWidgets.length == 1
		){
			this._singleChild = candidateWidgets[0];
		}else{
			delete this._singleChild;
		}

		// So we can set overflow: hidden to avoid a safari bug w/scrollbars showing up (#9449)
		domClass.toggle(this.containerNode, this.baseClass + "SingleChild", !!this._singleChild);
	},

	resize: function(changeSize, resultSize){
		// summary:
		//		See `dijit.layout._LayoutWidget.resize` for description.
		//		Although ContentPane doesn't extend _LayoutWidget, it does implement
		//		the same API.

		// For the TabContainer --> BorderContainer --> ContentPane case, _onShow() is
		// never called, so resize() is our trigger to do the initial href download (see [20099]).
		// However, don't load href for closed TitlePanes.
		if(!this._wasShown && this.open !== false){
			this._onShow();
		}

		this._resizeCalled = true;

		this._scheduleLayout(changeSize, resultSize);
	},

	_scheduleLayout: function(changeSize, resultSize){
		// summary:
		//		Resize myself, and call resize() on each of my child layout widgets, either now
		//		(if I'm currently visible) or when I become visible
		if(this._isShown()){
			this._layout(changeSize, resultSize);
		}else{
			this._needLayout = true;
			this._changeSize = changeSize;
			this._resultSize = resultSize;
		}
	},

	_layout: function(changeSize, resultSize){
		// summary:
		//		Resize myself according to optional changeSize/resultSize parameters, like a layout widget.
		//		Also, since I am a Container widget, each of my children expects me to
		//		call resize() or layout() on them.
		//
		//		Should be called on initialization and also whenever we get new content
		//		(from an href, or from set('content', ...))... but deferred until
		//		the ContentPane is visible

		// Set margin box size, unless it wasn't specified, in which case use current size.
		if(changeSize){
			domGeometry.setMarginBox(this.domNode, changeSize);
		}

		// Compute content box size of containerNode in case we [later] need to size our single child.
		var cn = this.containerNode;
		if(cn === this.domNode){
			// If changeSize or resultSize was passed to this method and this.containerNode ==
			// this.domNode then we can compute the content-box size without querying the node,
			// which is more reliable (similar to LayoutWidget.resize) (see for example #9449).
			var mb = resultSize || {};
			lang.mixin(mb, changeSize || {}); // changeSize overrides resultSize
			if(!("h" in mb) || !("w" in mb)){
				mb = lang.mixin(domGeometry.getMarginBox(cn), mb); // just use domGeometry.setMarginBox() to fill in missing values
			}
			this._contentBox = layoutUtils.marginBox2contentBox(cn, mb);
		}else{
			this._contentBox = domGeometry.getContentBox(cn);
		}

		this._layoutChildren();

		delete this._needLayout;
	},

	_layoutChildren: function(){
		// Call _checkIfSingleChild() again in case app has manually mucked w/the content
		// of the ContentPane (rather than changing it through the set("content", ...) API.
		if(this.doLayout){
			this._checkIfSingleChild();
		}

		if(this._singleChild && this._singleChild.resize){
			var cb = this._contentBox || domGeometry.getContentBox(this.containerNode);

			// note: if widget has padding this._contentBox will have l and t set,
			// but don't pass them to resize() or it will doubly-offset the child
			this._singleChild.resize({w: cb.w, h: cb.h});
		}else{
			// All my child widgets are independently sized (rather than matching my size),
			// but I still need to call resize() on each child to make it layout.
			array.forEach(this.getChildren(), function(widget){
				if(widget.resize){
					widget.resize();
				}
			});
		}
	},

	_isShown: function(){
		// summary:
		//		Returns true if the content is currently shown.
		// description:
		//		If I am a child of a layout widget then it actually returns true if I've ever been visible,
		//		not whether I'm currently visible, since that's much faster than tracing up the DOM/widget
		//		tree every call, and at least solves the performance problem on page load by deferring loading
		//		hidden ContentPanes until they are first shown

		if(this._childOfLayoutWidget){
			// If we are TitlePane, etc - we return that only *IF* we've been resized
			if(this._resizeCalled && "open" in this){
				return this.open;
			}
			return this._resizeCalled;
		}else if("open" in this){
			return this.open;		// for TitlePane, etc.
		}else{
			var node = this.domNode, parent = this.domNode.parentNode;
			return (node.style.display != 'none') && (node.style.visibility != 'hidden') && !domClass.contains(node, "dijitHidden") &&
					parent && parent.style && (parent.style.display != 'none');
		}
	},

	_onShow: function(){
		// summary:
		//		Called when the ContentPane is made visible
		// description:
		//		For a plain ContentPane, this is called on initialization, from startup().
		//		If the ContentPane is a hidden pane of a TabContainer etc., then it's
		//		called whenever the pane is made visible.
		//
		//		Does layout/resize of child widget(s)

		if(this._needLayout){
			// If a layout has been scheduled for when we become visible, do it now
			this._layout(this._changeSize, this._resultSize);
		}

		this.inherited(arguments);

		// Need to keep track of whether ContentPane has been shown (which is different than
		// whether or not it's currently visible).
		this._wasShown = true;
	}
});

});

},
'ibm/usmi/widgets/finder/FinderBaseDataStore':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
// Finder FinderBaseDataStore
define("ibm/usmi/widgets/finder/FinderBaseDataStore", [
	"dojo/_base/lang",
	"dojo/on",
	"dojo/dom",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dojo/i18n!ibm/usmi/nls/frameworkResources",
	"dijit",
	"dijit/Tooltip",
	"dojo/_base/declare",
	"dojox/data/AndOrReadStore",
	"ibm/usmi/util/Utils",
	"./TaskCategory"
], function (djLang, on, djDom, djClass, djDomC, MRI, dijit, tooltip, declare, djAndOrReadStore, Utils, TaskCategory) {

	return declare(null, {

		_dataStore : null,
		_type : null,
		_categories : null,
		_toolTips : null,

		constructor : function (type, domId) {
			this._type = type;
			this._toolTips = [];
		},
		getCategories : function () {
			return this._categories;
		},

		populateCategoriesWithSelectedItems : function (myitems) {
			var i = 0, item;
			for (i = 0; i < myitems.length; i++) {
				item = myitems[i];
				this._categories[item.categoryIdx].addSelected(item.objectIdx);
			}
			Utils.CMFinest('finder: populatedCategories for ' + i + ' items');
		},

		populateSearchResultItem : function (resultItemDomElement, finderObj, extraClass, notClickable) {
			var classname = 'SelectableCommonAction';
			if (notClickable) {
				classname = 'notSelectableCommonAction';
			}
			if (extraClass) {
				classname = extraClass + " " + classname;
			}
			djClass.add(resultItemDomElement, classname);
			if (notClickable !== true) {
				on(djDom.byId(resultItemDomElement), "mouseover", function () {
					this.className = classname + "Hover";
				});
				on(djDom.byId(resultItemDomElement), "mouseout", function () {
					this.className = classname;
				});
				on(djDom.byId(resultItemDomElement), "mousedown", djLang.hitch(finderObj, finderObj.doClick));

			}
		},

		createCategoryHTMLElements : function (numberOfResults, finderObj, moreId, moreText, notClickable) {
			var count, moreActionsDiv, sectionDomNode, sectionTitleDomNode, itemId, itemDiv;
			var maxNumberOfResults = 32;
			var numberOfItemsCurrentlyShowing = 0;
			var i = 1, j;
			for (i = 1; i < this._categories.length; i++) {
				count = this._categories[i].getSelectedCount();
				if (count === 0) {
					continue;
				}

				//create category section         
				//Utils.CMFinest('finder: creating category: ' + this._categories[i].getCategory());
				sectionDomNode = djDomC.create("div", {
					id : "categorySection" + this._type + "#_" + i
				}, djDom.byId('finder' + this._type + "_containerDiv"));
				sectionTitleDomNode = this.objHighlightSearchStr(sectionDomNode, dijit.byId('finder' + "searchBox")
						.get('value'), this._categories[i].getCategory(), 'true');
				//djClass.add(sectionTitle, "categorySectionTitle");
				for (j = 0; j < count; j++) {
					if (maxNumberOfResults <= 0) {
						break;
					}
					itemId = this._type + "#_" + this._categories[i].getSelectedId(j);
					var innerId = itemId + "innerDiv";
					var icon = this._categories[i].getSelectedIcon(j);
					var iconHtml = "";
					if (icon) {
						iconHtml = "<img class=\"finderIcon\" src=\"" + icon + "\"/>";
					}
					itemDiv = djDomC.create("div", {
						id : itemId,
						oncontextmenu : "return false;",
						innerHTML : iconHtml + "<div id=\"" + innerId + "\"/>"
					}, "categorySection" + this._type + "#_" + i);
					this.objHighlightSearchStr(djDom.byId(innerId), dijit.byId('finder' + "searchBox").get('value'),
							this._categories[i].getSelectedName(j));
					var toolTipLabel = this._constructTooltip(finderObj, this._categories[i].getSelectedId(j));
					var tp = new tooltip({
						connectId : [ itemDiv
						],
						label : toolTipLabel,
						closable : true,
						position : [ "before"
						]
					//id: overlayId+"_buttonTooltip"
					});
					on(itemDiv, "mouseover", djLang.hitch(finderObj, finderObj.setTooltip, tp));
					//djDom.byId('workAreaDiv').innerHTML = toolTipLabel;
					this._toolTips.push(tp);
					this.populateSearchResultItem(itemId, finderObj);
					maxNumberOfResults--;
					numberOfItemsCurrentlyShowing++;
				}
				if (maxNumberOfResults <= 0) {
					break;
				}

				maxNumberOfResults = maxNumberOfResults - 2;
			}
			if (maxNumberOfResults <= 0) {

				moreActionsDiv = djDomC.create("div", {
					id : moreId,
					innerHTML : '<BR/><span> ... ' + ' ' + moreText + '</span>'
				}, djDom.byId('finder' + this._type + "_containerDiv"));
				this.populateSearchResultItem(moreActionsDiv, finderObj, 'finderMoreTasks', notClickable);
			}

		},

		clearSelection : function () {
			var i;
			for (i = 0; i < this._categories.length; i++) {
				if (this._categories[i]) {
					this._categories[i].clearSelection();
				} else {
					break;
				}
			}
		},

		populateItems : function (list) {
			this._categories = [];
			if (!list || list === "") {
				return;
			}
			var size = list.length;
			var i = 0;
			var items = [];
			this._categories[0] = new TaskCategory("Uncategorized");
			for (i = 0; i < list.length; i++) {
				var object = list[i];
				var catIdx = 0;
				var objectIdx;
				for (catIdx = 0; catIdx < list.length + 1; catIdx++) {
					if (this._categories[catIdx]) {
						if (this._categories[catIdx].getCategory() === object.category) {
							var tempcategory = this._categories[catIdx];
							objectIdx = tempcategory.add(object.id, object.label, object.icon);
							break;
						}
					} else {
						this._categories[catIdx] = new TaskCategory(object.category);
						objectIdx = this._categories[catIdx].add(object.id, object.label, object.icon);
						break;
					}
				}
				items[i] = {
					'id' : object.id,
					'name' : object.label,
					'category' : object.category,
					'categoryIdx' : catIdx,
					'objectIdx' : objectIdx
				};
			}
			Utils.CMFinest('finder: parsed items, total: ' + i);
			var dataStore = {
				identifier : 'id',
				label : 'name',
				items : items
			};
			this._dataStore = new djAndOrReadStore({
				data : dataStore
			});
			Utils.CMFinest("finder: " + this._dataStore);

		},
		objHighlightSearchStr : function (targEl, searchStr, targetStr, isCategory) {
			targEl.innerHTML = "";
			var type = "span";
			var textClass = "";
			if (isCategory) {
				type = "h1";
				textClass = "categorySectionTitle";
			}
			var resultEl = djDomC.create(type, {
				className : textClass
			}, targEl);

			var re = new RegExp(searchStr, "i");
			var startPos = targetStr.search(re);

			//if no match just return the string as-is 
			if (startPos <= -1) {
				djDomC.create("span", {
					innerHTML : targetStr
				}, resultEl);
				return resultEl;
			}

			var s0 = djDomC.create("span", {
				innerHTML : targetStr.substring(0, startPos)
			}, resultEl);
			var s1 = djDomC.create("span", {
				innerHTML : targetStr.substring(startPos, startPos + searchStr.length)
			}, resultEl);
			djClass.add(s1, "SearchSubstringHighlight");
			djDomC.create("span", {
				innerHTML : targetStr.substring(startPos + searchStr.length, targetStr.length)
			}, resultEl);
			return resultEl;

		},
		_constructTooltip : function (finderObj, itemId) {
			var tooltip, clickHere, category, categoryLabel, label, labelHeader, additional;
			var item = finderObj._indexedItems[itemId];
			var itemName = item.getProperty('FINDER_NAME_COLUMN').getValue();
			var itemType = item.getProperty('FINDER_TYPE_COLUMN').getValue();
			var itemCategory = item.getProperty('FINDER_CATEGORY_COLUMN').getValue();
			var itemIcon /*= item.getProperty('FINDER_ICON_COLUMN').getValue()*/;
			if (item.getProperty('TOOLTIP_PROPS_COLUMN').getValue()
					&& item.getProperty('TOOLTIP_PROPS_COLUMN').getValue().length > 0) {
				additional = JSON.parse(item.getProperty('TOOLTIP_PROPS_COLUMN').getValue());
			}
			if (this._type === 'resources') {
				clickHere = MRI.FinderTooltipClickResources;
				labelHeader = djLang.replace(MRI.FinderTooltipEntryType, [ itemCategory
				]);
			} else if (this._type === 'tasks') {
				categoryLabel = MRI.FinderTooltipCategory;
				if (itemType === 'tasks') {
					labelHeader = MRI.FinderTooltipExternalTask;
					clickHere = MRI.FinderTooltipClickTask;
				} else if (itemType === 'jobs') {
					labelHeader = MRI.FinderTooltipJob;
					clickHere = MRI.FinderTooltipClickJob;
				} else if (itemType === 'urlTasks') {
					labelHeader = MRI.FinderTooltipUrl;
					clickHere = MRI.FinderTooltipClickUrl;
				} else if (itemType === 'ngenTask') {
					labelHeader = MRI.FinderTooltipPage;
					clickHere = MRI.FinderTooltipClickPage;
				}
				//description = 'Description: ' + item.getProperty('FINDER_DESCRIPTION_COLUMN').getValue() + "<BR/>";

				//var item = 

			} else if (this._type === 'groups') {
				clickHere = MRI.FinderTooltipClickGroup;
				labelHeader = djLang.replace(MRI.FinderTooltipEntryType, [ itemCategory
				]);
			}
			var iconHtml = "";
			if (itemIcon) {
				iconHtml = "<img class=\"finderToolTipIcon\" src=\"" + window.location.protocol + "//"
						+ window.location.host + itemIcon + "\"/>";
			}
			label = "<span>" + iconHtml + "<span class=\"finderTooltipHeaderText\">" + labelHeader + itemName
					+ "</span><BR/>";
			var categoryHtml = "";
			if (categoryLabel) {
				category = categoryLabel + itemCategory;
				categoryHtml = "<span class=\"finderTooltipCategoryText\">" + category + "</span><BR/>";
			}
			var additionalStr = '', idx = 0;
			if (additional) {
				for (; idx < additional.length; idx++) {
					var tPropLabel = additional[idx].LABEL;
					var tPropValue = additional[idx].VALUE;
					if (tPropValue && tPropValue.length > 0) {
						additionalStr += tPropLabel + ": " + tPropValue + "<BR/>";
					}
				}
				/*additionalStr += "<BR/>"*/
			}
			tooltip = label + categoryHtml + additionalStr + "<BR/><span class=\"finderToolTipClickHere\">" + clickHere
					+ "</span>";
			return tooltip;
		},
		destroy : function () {
			this.inherited(arguments);
			if (this._dataStore) {
				this._dataStore = null;
			}
			this.clearTooltips();

			if (this._categories) {
				var i = 0;
				for (; i < this._categories.length; i++) {
					this._categories[i].destroy;
				}
				this._categories.length = 0;
			}
		},
		clearTooltips : function () {
			if (this._toolTips) {
				var i = 0;
				for (; i < this._toolTips.length; i++) {
					this._toolTips[i].destroy();
				}
				this._toolTips.length = 0;
			}
		}
	});
});

},
'ibm/tivoli/tip/navwidget/dojo/NavWidgetBaseViewState':function(){
// wrapped by build app
define("ibm/tivoli/tip/navwidget/dojo/NavWidgetBaseViewState", ["dijit","dojo","dojox","dojo/require!dojox/collections/Dictionary"], function(dijit,dojo,dojox){
dojo.provide("ibm.tivoli.tip.navwidget.dojo.NavWidgetBaseViewState");

dojo.require("dojox.collections.Dictionary");

dojo.declare(
    "ibm.tivoli.tip.navwidget.dojo.NavWidgetBaseViewState",
    null,
    {		
  		dataset: null,
    	stateData: null,
  		_viewStates: null,
  		
  		constructor: function(kwArgs) {
			//Look at kwArgs
    		if (kwArgs) {
    			this.view = kwArgs.view;
				this.dataset = kwArgs.dataset;		
				this.stateData = kwArgs.stateData;
			}
		
    		//Create internal data structure for states
    		this._viewStates = new dojox.collections.Dictionary();
	 	},
	 	
	 	addPluggableViewState: function(viewId, viewState) {
			this._viewStates.add(viewId, viewState);
		},

		getPluggableViewState: function(viewId) {
			return this._viewStates.item(viewId);
		},
		
		removePluggableViewState: function(viewId) {
			return this._viewStates.remove(viewId);
		},
		
		destroy: function() {
			this.dataset = null;
			this.stateData = null;
			if (this._viewStates) {
				this._viewStates.clear();
				this._viewStates = null;
			}
		}
    }
);
});

},
'dijit/form/CurrencyTextBox':function(){
define([
	"dojo/currency", // currency._mixInDefaults currency.format currency.parse currency.regexp
	"dojo/_base/declare", // declare
	"dojo/_base/lang", // lang.hitch
	"./NumberTextBox"
], function(currency, declare, lang, NumberTextBox){

/*=====
	var NumberTextBox = dijit.form.NumberTextBox;
=====*/

	// module:
	//		dijit/form/CurrencyTextBox
	// summary:
	//		A validating currency textbox


	/*=====
	declare(
		"dijit.form.CurrencyTextBox.__Constraints",
		[dijit.form.NumberTextBox.__Constraints, currency.__FormatOptions, currency.__ParseOptions], {
		// summary:
		//		Specifies both the rules on valid/invalid values (minimum, maximum,
		//		number of required decimal places), and also formatting options for
		//		displaying the value when the field is not focused (currency symbol,
		//		etc.)
		// description:
		//		Follows the pattern of `dijit.form.NumberTextBox.constraints`.
		//		In general developers won't need to set this parameter
		// example:
		//		To ensure that the user types in the cents (for example, 1.00 instead of just 1):
		//	|		{fractional:true}
	});
	=====*/

	return declare("dijit.form.CurrencyTextBox", NumberTextBox, {
		// summary:
		//		A validating currency textbox
		// description:
		//		CurrencyTextBox is similar to `dijit.form.NumberTextBox` but has a few
		//		extra features related to currency:
		//
		//		1. After specifying the currency type (american dollars, euros, etc.) it automatically
		//			sets parse/format options such as how many decimal places to show.
		//		2. The currency mark (dollar sign, euro mark, etc.) is displayed when the field is blurred
		//			but erased during editing, so that the user can just enter a plain number.

		// currency: [const] String
		//		the [ISO4217](http://en.wikipedia.org/wiki/ISO_4217) currency code, a three letter sequence like "USD"
		currency: "",

		/*=====
		// constraints: dijit.form.CurrencyTextBox.__Constraints
		//		Despite the name, this parameter specifies both constraints on the input
		//		(including minimum/maximum allowed values) as well as
		//		formatting options.  See `dijit.form.CurrencyTextBox.__Constraints` for details.
		constraints: {},
		======*/

		baseClass: "dijitTextBox dijitCurrencyTextBox",

		// Override regExpGen ValidationTextBox.regExpGen().... we use a reg-ex generating function rather
		// than a straight regexp to deal with locale  (plus formatting options too?)
		regExpGen: function(constraints){
			// if focused, accept either currency data or NumberTextBox format
			return '(' + (this.focused ? this.inherited(arguments, [ lang.mixin({}, constraints, this.editOptions) ]) + '|' : '')
				+ currency.regexp(constraints) + ')';
		},

		// Override NumberTextBox._formatter to deal with currencies, ex: converts "123.45" to "$123.45"
		_formatter: currency.format,

		_parser: currency.parse,

		parse: function(/*String*/ value, /*Object*/ constraints){
			// summary:
			// 		Parses string value as a Currency, according to the constraints object
			// tags:
			// 		protected extension
			var v = this.inherited(arguments);
			if(isNaN(v) && /\d+/.test(value)){ // currency parse failed, but it could be because they are using NumberTextBox format so try its parse
				v = lang.hitch(lang.mixin({}, this, { _parser: NumberTextBox.prototype._parser }), "inherited")(arguments);
			}
			return v;
		},

		_setConstraintsAttr: function(/*Object*/ constraints){
			if(!constraints.currency && this.currency){
				constraints.currency = this.currency;
			}
			this.inherited(arguments, [ currency._mixInDefaults(lang.mixin(constraints, { exponent: false })) ]); // get places
		}
	});
});

},
'dijit/form/ComboBox':function(){
define([
	"dojo/_base/declare", // declare
	"./ValidationTextBox",
	"./ComboBoxMixin"
], function(declare, ValidationTextBox, ComboBoxMixin){

/*=====
	var ValidationTextBox = dijit.form.ValidationTextBox;
	var ComboBoxMixin = dijit.form.ComboBoxMixin;
=====*/

	// module:
	//		dijit/form/ComboBox
	// summary:
	//		Auto-completing text box

	return declare("dijit.form.ComboBox", [ValidationTextBox, ComboBoxMixin], {
		// summary:
		//		Auto-completing text box
		//
		// description:
		//		The drop down box's values are populated from an class called
		//		a data provider, which returns a list of values based on the characters
		//		that the user has typed into the input box.
		//		If OPTION tags are used as the data provider via markup,
		//		then the OPTION tag's child text node is used as the widget value
		//		when selected.  The OPTION tag's value attribute is ignored.
		//		To set the default value when using OPTION tags, specify the selected
		//		attribute on 1 of the child OPTION tags.
		//
		//		Some of the options to the ComboBox are actually arguments to the data
		//		provider.
	});
});

},
'dojox/grid/DataGrid':function(){
define([
	"../main",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/_base/json",
	"dojo/_base/sniff",
	"dojo/_base/declare",
	"./_Grid",
	"./DataSelection",
	"dojo/query",
	"dojo/_base/html"
], function(dojox, array, lang, json, has, declare, _Grid, DataSelection, query, html){

/*=====
declare("dojox.grid.__DataCellDef", dojox.grid.__CellDef, {
	constructor: function(){
		//	field: String?
		//		The attribute to read from the dojo.data item for the row.
		//  fields: String[]?
		//		An array of fields to grab the values of and pass as an array to the grid
		//	get: Function?
		//		function(rowIndex, item?){} rowIndex is of type Integer, item is of type
		//		Object.  This function will be called when a cell requests data.  Returns
		//		the unformatted data for the cell.
	}
});
=====*/

/*=====
declare("dojox.grid.__DataViewDef", dojox.grid.__ViewDef, {
	constructor: function(){
		//	cells: dojox.grid.__DataCellDef[]|Array[dojox.grid.__DataCellDef[]]?
		//		The structure of the cells within this grid.
		//	defaultCell: dojox.grid.__DataCellDef?
		//		A cell definition with default values for all cells in this view.  If
		//		a property is defined in a cell definition in the "cells" array and
		//		this property, the cell definition's property will override this
		//		property's property.
	}
});
=====*/

var DataGrid = declare("dojox.grid.DataGrid", _Grid, {
	store: null,
	query: null,
	queryOptions: null,
	fetchText: '...',
	sortFields: null,
	
	// updateDelay: int
	//		Time, in milliseconds, to delay updates automatically so that multiple
	//		calls to onSet/onNew/onDelete don't keep rerendering the grid.  Set
	//		to 0 to immediately cause updates.  A higher value will result in
	//		better performance at the expense of responsiveness of the grid.
	updateDelay: 1,

/*=====
	// structure: dojox.grid.__DataViewDef|dojox.grid.__DataViewDef[]|dojox.grid.__DataCellDef[]|Array[dojox.grid.__DataCellDef[]]
	//		View layout defintion.
	structure: '',
=====*/

	// You can specify items instead of a query, if you like.  They do not need
	// to be loaded - but the must be items in the store
	items: null,
	
	_store_connects: null,
	_by_idty: null,
	_by_idx: null,
	_cache: null,
	_pages: null,
	_pending_requests: null,
	_bop: -1,
	_eop: -1,
	_requests: 0,
	rowCount: 0,

	_isLoaded: false,
	_isLoading: false,
	
	//keepSelection: Boolean
	//		Whether keep selection after sort, filter etc.
	keepSelection: false,	
	
	postCreate: function(){
		this._pages = [];
		this._store_connects = [];
		this._by_idty = {};
		this._by_idx = [];
		this._cache = [];
		this._pending_requests = {};

		this._setStore(this.store);
		this.inherited(arguments);
	},
	
	destroy: function(){
		this.selection.destroy();
		this.inherited(arguments);
	},

	createSelection: function(){
		this.selection = new DataSelection(this);
	},

	get: function(inRowIndex, inItem){
		// summary: Default data getter.
		// description:
		//		Provides data to display in a grid cell. Called in grid cell context.
		//		So this.cell.index is the column index.
		// inRowIndex: Integer
		//		Row for which to provide data
		// returns:
		//		Data to display for a given grid cell.
		
		if(inItem && this.field == "_item" && !this.fields){
			return inItem;
		}else if(inItem && this.fields){
			var ret = [];
			var s = this.grid.store;
			array.forEach(this.fields, function(f){
				ret = ret.concat(s.getValues(inItem, f));
			});
			return ret;
		}else if(!inItem && typeof inRowIndex === "string"){
			return this.inherited(arguments);
		}
		return (!inItem ? this.defaultValue : (!this.field ? this.value : (this.field == "_item" ? inItem : this.grid.store.getValue(inItem, this.field))));
	},

	_checkUpdateStatus: function(){
		if(this.updateDelay > 0){
			var iStarted = false;
			if(this._endUpdateDelay){
				clearTimeout(this._endUpdateDelay);
				delete this._endUpdateDelay;
				iStarted = true;
			}
			if(!this.updating){
				this.beginUpdate();
				iStarted = true;
			}
			if(iStarted){
				var _this = this;
				this._endUpdateDelay = setTimeout(function(){
					delete _this._endUpdateDelay;
					_this.endUpdate();
				}, this.updateDelay);
			}
		}
	},
	
	_onSet: function(item, attribute, oldValue, newValue){
		this._checkUpdateStatus();
		var idx = this.getItemIndex(item);
		if(idx>-1){
			this.updateRow(idx);
		}
	},
	
	_createItem: function(item, index){
		var idty = this._hasIdentity ? this.store.getIdentity(item) : json.toJson(this.query) + ":idx:" + index + ":sort:" + json.toJson(this.getSortProps());
		var o = this._by_idty[idty] = { idty: idty, item: item };
		return o;
	},

	_addItem: function(item, index, noUpdate){
		this._by_idx[index] = this._createItem(item, index);
		if(!noUpdate){
			this.updateRow(index);
		}
	},

	_onNew: function(item, parentInfo){
		this._checkUpdateStatus();
		var rowCount = this.get('rowCount');
		this._addingItem = true;
		this.updateRowCount(rowCount+1);
		this._addingItem = false;
		this._addItem(item, rowCount);
		this.showMessage();
	},

	_onDelete: function(item){
		this._checkUpdateStatus();
		var idx = this._getItemIndex(item, true);

		if(idx >= 0){
			// When a row is deleted, all rest rows are shifted down,
			// and migrate from page to page. If some page is not
			// loaded yet empty rows can migrate to initialized pages
			// without refreshing. It causes empty rows in some pages, see:
			// http://bugs.dojotoolkit.org/ticket/6818
			// this code fix this problem by reseting loaded page info
			this._pages = [];
			this._bop = -1;
			this._eop = -1;

			var o = this._by_idx[idx];
			this._by_idx.splice(idx, 1);
			delete this._by_idty[o.idty];
			this.updateRowCount(this.get('rowCount')-1);
			if(this.get('rowCount') === 0){
				this.showMessage(this.noDataMessage);
			}
		}
		if(this.selection.isSelected(idx)){
			this.selection.deselect(idx);
			this.selection.selected.splice(idx, 1);
		}
	},

	_onRevert: function(){
		this._refresh();
	},

	setStore: function(store, query, queryOptions){
		if(this._requestsPending(0)){
			return;
		}
		this._setQuery(query, queryOptions);
		this._setStore(store);
		this._refresh(true);
	},
	
	setQuery: function(query, queryOptions){
		if(this._requestsPending(0)){
			return;
		}
		this._setQuery(query, queryOptions);
		this._refresh(true);
	},
	
	setItems: function(items){
		this.items = items;
		this._setStore(this.store);
		this._refresh(true);
	},
	
	_setQuery: function(query, queryOptions){
		this.query = query;
		this.queryOptions = queryOptions || this.queryOptions;
	},

	_setStore: function(store){
		if(this.store && this._store_connects){
			array.forEach(this._store_connects, this.disconnect, this);
		}
		this.store = store;

		if(this.store){
			var f = this.store.getFeatures();
			var h = [];

			this._canEdit = !!f["dojo.data.api.Write"] && !!f["dojo.data.api.Identity"];
			this._hasIdentity = !!f["dojo.data.api.Identity"];

			if(!!f["dojo.data.api.Notification"] && !this.items){
				h.push(this.connect(this.store, "onSet", "_onSet"));
				h.push(this.connect(this.store, "onNew", "_onNew"));
				h.push(this.connect(this.store, "onDelete", "_onDelete"));
			}
			if(this._canEdit){
				h.push(this.connect(this.store, "revert", "_onRevert"));
			}

			this._store_connects = h;
		}
	},

	_onFetchBegin: function(size, req){
		if(!this.scroller){ return; }
		if(this.rowCount != size){
			if(req.isRender){
				this.scroller.init(size, this.keepRows, this.rowsPerPage);
				this.rowCount = size;
				this._setAutoHeightAttr(this.autoHeight, true);
				this._skipRowRenormalize = true;
				this.prerender();
				this._skipRowRenormalize = false;
			}else{
				this.updateRowCount(size);
			}
		}
		if(!size){
			this.views.render();
			this._resize();
			this.showMessage(this.noDataMessage);
			this.focus.initFocusView();
		}else{
			this.showMessage();
		}
	},

	_onFetchComplete: function(items, req){
		if(!this.scroller){ return; }
		if(items && items.length > 0){
			//console.log(items);
			array.forEach(items, function(item, idx){
				this._addItem(item, req.start+idx, true);
			}, this);
			this.updateRows(req.start, items.length);
			if(req.isRender){
				this.setScrollTop(0);
				this.postrender();
			}else if(this._lastScrollTop){
				this.setScrollTop(this._lastScrollTop);
			}
			if(has("ie")){
				html.setSelectable(this.domNode, this.selectable);
				query("th.dojoxGridCell", this.viewsHeaderNode).forEach(function(node){
					dojo.setSelectable(node, true);
				});
			}	
		}
		delete this._lastScrollTop;
		if(!this._isLoaded){
			this._isLoading = false;
			this._isLoaded = true;
		}
		this._pending_requests[req.start] = false;
	},

	_onFetchError: function(err, req){
		console.log(err);
		delete this._lastScrollTop;
		if(!this._isLoaded){
			this._isLoading = false;
			this._isLoaded = true;
			this.showMessage(this.errorMessage);
		}
		this._pending_requests[req.start] = false;
		this.onFetchError(err, req);
	},

	onFetchError: function(err, req){
	},

	_fetch: function(start, isRender){
		start = start || 0;
		if(this.store && !this._pending_requests[start]){
			if(!this._isLoaded && !this._isLoading){
				this._isLoading = true;
				this.showMessage(this.loadingMessage);
			}
			this._pending_requests[start] = true;
			//console.log("fetch: ", start);
			try{
				if(this.items){
					var items = this.items;
					var store = this.store;
					this.rowsPerPage = items.length;
					var req = {
						start: start,
						count: this.rowsPerPage,
						isRender: isRender
					};
					this._onFetchBegin(items.length, req);
					
					// Load them if we need to
					var waitCount = 0;
					array.forEach(items, function(i){
						if(!store.isItemLoaded(i)){ waitCount++; }
					});
					if(waitCount === 0){
						this._onFetchComplete(items, req);
					}else{
						var onItem = function(item){
							waitCount--;
							if(waitCount === 0){
								this._onFetchComplete(items, req);
							}
						};
						array.forEach(items, function(i){
							if(!store.isItemLoaded(i)){
								store.loadItem({item: i, onItem: onItem, scope: this});
							}
						}, this);
					}
				}else{
					this.store.fetch({
						start: start,
						count: this.rowsPerPage,
						query: this.query,
						sort: this.getSortProps(),
						queryOptions: this.queryOptions,
						isRender: isRender,
						onBegin: lang.hitch(this, "_onFetchBegin"),
						onComplete: lang.hitch(this, "_onFetchComplete"),
						onError: lang.hitch(this, "_onFetchError")
					});
				}
			}catch(e){
				this._onFetchError(e, {start: start, count: this.rowsPerPage});
			}
		}
	},

	_clearData: function(){
		this.updateRowCount(0);
		this._by_idty = {};
		this._by_idx = [];
		this._pages = [];
		this._bop = this._eop = -1;
		this._isLoaded = false;
		this._isLoading = false;
	},

	getItem: function(idx){
		var data = this._by_idx[idx];
		if(!data||(data&&!data.item)){
			this._preparePage(idx);
			return null;
		}
		return data.item;
	},

	getItemIndex: function(item){
		return this._getItemIndex(item, false);
	},
	
	_getItemIndex: function(item, isDeleted){
		if(!isDeleted && !this.store.isItem(item)){
			return -1;
		}

		var idty = this._hasIdentity ? this.store.getIdentity(item) : null;

		for(var i=0, l=this._by_idx.length; i<l; i++){
			var d = this._by_idx[i];
			if(d && ((idty && d.idty == idty) || (d.item === item))){
				return i;
			}
		}
		return -1;
	},

	filter: function(query, reRender){
		this.query = query;
		if(reRender){
			this._clearData();
		}
		this._fetch();
	},

	_getItemAttr: function(idx, attr){
		var item = this.getItem(idx);
		return (!item ? this.fetchText : this.store.getValue(item, attr));
	},

	// rendering
	_render: function(){
		if(this.domNode.parentNode){
			this.scroller.init(this.get('rowCount'), this.keepRows, this.rowsPerPage);
			this.prerender();
			this._fetch(0, true);
		}
	},

	// paging
	_requestsPending: function(inRowIndex){
		return this._pending_requests[inRowIndex];
	},

	_rowToPage: function(inRowIndex){
		return (this.rowsPerPage ? Math.floor(inRowIndex / this.rowsPerPage) : inRowIndex);
	},

	_pageToRow: function(inPageIndex){
		return (this.rowsPerPage ? this.rowsPerPage * inPageIndex : inPageIndex);
	},

	_preparePage: function(inRowIndex){
		if((inRowIndex < this._bop || inRowIndex >= this._eop) && !this._addingItem){
			var pageIndex = this._rowToPage(inRowIndex);
			this._needPage(pageIndex);
			this._bop = pageIndex * this.rowsPerPage;
			this._eop = this._bop + (this.rowsPerPage || this.get('rowCount'));
		}
	},

	_needPage: function(inPageIndex){
		if(!this._pages[inPageIndex]){
			this._pages[inPageIndex] = true;
			this._requestPage(inPageIndex);
		}
	},

	_requestPage: function(inPageIndex){
		var row = this._pageToRow(inPageIndex);
		var count = Math.min(this.rowsPerPage, this.get('rowCount') - row);
		if(count > 0){
			this._requests++;
			if(!this._requestsPending(row)){
				setTimeout(lang.hitch(this, "_fetch", row, false), 1);
				//this.requestRows(row, count);
			}
		}
	},

	getCellName: function(inCell){
		return inCell.field;
		//console.log(inCell);
	},

	_refresh: function(isRender){
		this._clearData();
		this._fetch(0, isRender);
	},

	sort: function(){
		this.edit.apply();
		this._lastScrollTop = this.scrollTop;
		this._refresh();
	},

	canSort: function(){
		return (!this._isLoading);
	},

	getSortProps: function(){
		var c = this.getCell(this.getSortIndex());
		if(!c){
			if(this.sortFields){
				return this.sortFields;
			}
			return null;
		}else{
			var desc = c["sortDesc"];
			var si = !(this.sortInfo>0);
			if(typeof desc == "undefined"){
				desc = si;
			}else{
				desc = si ? !desc : desc;
			}
			return [{ attribute: c.field, descending: desc }];
		}
	},

	styleRowState: function(inRow){
		// summary: Perform row styling
		if(this.store && this.store.getState){
			var states=this.store.getState(inRow.index), c='';
			for(var i=0, ss=["inflight", "error", "inserting"], s; s=ss[i]; i++){
				if(states[s]){
					c = ' dojoxGridRow-' + s;
					break;
				}
			}
			inRow.customClasses += c;
		}
	},

	onStyleRow: function(inRow){
		this.styleRowState(inRow);
		this.inherited(arguments);
	},

	scrollToRow: function(inRowIndex){
		// summary:
		//		overriden
		var d = new dojo.Deferred(),
			_this = this,
			reqs = this._pending_requests,
			_scrollToRow = function(){
				_this.setScrollTop(_this.scroller.findScrollTop(inRowIndex) + 1);
			},
			check = function(){
				for(var i in reqs){
					if(reqs[i]){
						setTimeout(check, 10);
						return;
					}
				}
				_scrollToRow();
				d.callback();
			};
		_scrollToRow();
        setTimeout(check, 10);
		return d;
	},

	// editing
	canEdit: function(inCell, inRowIndex){
		return this._canEdit;
	},

	_copyAttr: function(idx, attr){
		var row = {};
		var backstop = {};
		var src = this.getItem(idx);
		return this.store.getValue(src, attr);
	},

	doStartEdit: function(inCell, inRowIndex){
		if(!this._cache[inRowIndex]){
			this._cache[inRowIndex] = this._copyAttr(inRowIndex, inCell.field);
		}
		this.onStartEdit(inCell, inRowIndex);
	},

	doApplyCellEdit: function(inValue, inRowIndex, inAttrName){
		this.store.fetchItemByIdentity({
			identity: this._by_idx[inRowIndex].idty,
			onItem: lang.hitch(this, function(item){
				var oldValue = this.store.getValue(item, inAttrName);
				if(typeof oldValue == 'number'){
					inValue = isNaN(inValue) ? inValue : parseFloat(inValue);
				}else if(typeof oldValue == 'boolean'){
					inValue = inValue == 'true' ? true : inValue == 'false' ? false : inValue;
				}else if(oldValue instanceof Date){
					var asDate = new Date(inValue);
					inValue = isNaN(asDate.getTime()) ? inValue : asDate;
				}
				this.store.setValue(item, inAttrName, inValue);
				this.onApplyCellEdit(inValue, inRowIndex, inAttrName);
			})
		});
	},

	doCancelEdit: function(inRowIndex){
		var cache = this._cache[inRowIndex];
		if(cache){
			this.updateRow(inRowIndex);
			delete this._cache[inRowIndex];
		}
		this.onCancelEdit.apply(this, arguments);
	},

	doApplyEdit: function(inRowIndex, inDataAttr){
		var cache = this._cache[inRowIndex];
		/*if(cache){
			var data = this.getItem(inRowIndex);
			if(this.store.getValue(data, inDataAttr) != cache){
				this.update(cache, data, inRowIndex);
			}
			delete this._cache[inRowIndex];
		}*/
		this.onApplyEdit(inRowIndex);
	},

	removeSelectedRows: function(){
		// summary:
		//		Remove the selected rows from the grid.
		if(this._canEdit){
			this.edit.apply();
			var fx = lang.hitch(this, function(items){
				if(items.length){
					array.forEach(items, this.store.deleteItem, this.store);
					this.selection.clear();
				}
			});
			if(this.allItemsSelected){
				this.store.fetch({
							query: this.query,
							queryOptions: this.queryOptions,
							onComplete: fx});
			}else{
				fx(this.selection.getSelected());
			}
		}
	}
});

DataGrid.cell_markupFactory = function(cellFunc, node, cellDef){
	var field = lang.trim(html.attr(node, "field")||"");
	if(field){
		cellDef.field = field;
	}
	cellDef.field = cellDef.field||cellDef.name;
	var fields = lang.trim(html.attr(node, "fields")||"");
	if(fields){
		cellDef.fields = fields.split(",");
	}
	if(cellFunc){
		cellFunc(node, cellDef);
	}
};

DataGrid.markupFactory = function(props, node, ctor, cellFunc){
	return _Grid.markupFactory(props, node, ctor,
					lang.partial(DataGrid.cell_markupFactory, cellFunc));
};

return DataGrid;

});

},
'dojo/data/util/sorter':function(){
define(["dojo/_base/lang"], function(lang) {
	// module:
	//		dojo/data/util/sorter
	// summary:
	//		TODOC

var sorter = lang.getObject("dojo.data.util.sorter", true);

sorter.basicComparator = function(	/*anything*/ a,
													/*anything*/ b){
	//	summary:
	//		Basic comparision function that compares if an item is greater or less than another item
	//	description:
	//		returns 1 if a > b, -1 if a < b, 0 if equal.
	//		'null' values (null, undefined) are treated as larger values so that they're pushed to the end of the list.
	//		And compared to each other, null is equivalent to undefined.

	//null is a problematic compare, so if null, we set to undefined.
	//Makes the check logic simple, compact, and consistent
	//And (null == undefined) === true, so the check later against null
	//works for undefined and is less bytes.
	var r = -1;
	if(a === null){
		a = undefined;
	}
	if(b === null){
		b = undefined;
	}
	if(a == b){
		r = 0;
	}else if(a > b || a == null){
		r = 1;
	}
	return r; //int {-1,0,1}
};

sorter.createSortFunction = function(	/* attributes array */sortSpec, /*dojo.data.core.Read*/ store){
	//	summary:
	//		Helper function to generate the sorting function based off the list of sort attributes.
	//	description:
	//		The sort function creation will look for a property on the store called 'comparatorMap'.  If it exists
	//		it will look in the mapping for comparisons function for the attributes.  If one is found, it will
	//		use it instead of the basic comparator, which is typically used for strings, ints, booleans, and dates.
	//		Returns the sorting function for this particular list of attributes and sorting directions.
	//
	//	sortSpec: array
	//		A JS object that array that defines out what attribute names to sort on and whether it should be descenting or asending.
	//		The objects should be formatted as follows:
	//		{
	//			attribute: "attributeName-string" || attribute,
	//			descending: true|false;   // Default is false.
	//		}
	//	store: object
	//		The datastore object to look up item values from.
	//
	var sortFunctions=[];

	function createSortFunction(attr, dir, comp, s){
		//Passing in comp and s (comparator and store), makes this
		//function much faster.
		return function(itemA, itemB){
			var a = s.getValue(itemA, attr);
			var b = s.getValue(itemB, attr);
			return dir * comp(a,b); //int
		};
	}
	var sortAttribute;
	var map = store.comparatorMap;
	var bc = sorter.basicComparator;
	for(var i = 0; i < sortSpec.length; i++){
		sortAttribute = sortSpec[i];
		var attr = sortAttribute.attribute;
		if(attr){
			var dir = (sortAttribute.descending) ? -1 : 1;
			var comp = bc;
			if(map){
				if(typeof attr !== "string" && ("toString" in attr)){
					 attr = attr.toString();
				}
				comp = map[attr] || bc;
			}
			sortFunctions.push(createSortFunction(attr,
				dir, comp, store));
		}
	}
	return function(rowA, rowB){
		var i=0;
		while(i < sortFunctions.length){
			var ret = sortFunctions[i++](rowA, rowB);
			if(ret !== 0){
				return ret;//int
			}
		}
		return 0; //int
	}; // Function
};

return sorter;
});

},
'gridx/modules/VLayout':function(){
define([
	"dojo/_base/declare",
	"dojo/DeferredList",
	"../core/_Module"
], function(declare, DeferredList, _Module){

	return declare(/*===== "gridx.modules.VLayout", =====*/_Module, {
		// summary:
		//		This module manages the vertical layout of all the grid UI parts.
		// description:
		//		When user creates a grid with a given height, it means the height of the whole grid,
		//		which includes grid body, toobar, pagination bar, headerbar, horizontal scrollerbar, etc.
		//		So the height of the grid body must be calculated out so as to layout the grid properly.
		//		This module calculates grid body height by collecting height from all the registered
		//		grid UI parts. The reLayout function in this module will be called everytime the
		//		grid size is changed.

		name: 'vLayout',

		getAPIPath: function(){
			// tags:
			//		protected extension
			return {
				vLayout: this
			};
		},

		preload: function(){
			// tags:
			//		protected extension
			var t = this,
				g = t.grid;
			t.connect(g, '_onResizeEnd', function(changeSize, ds){
				var d, dl = [];
				for(d in ds){
					dl.push(ds[d]);
				}
				new DeferredList(dl).then(function(){
					t.reLayout();
				});
			});
			if(g.autoHeight){
				t.connect(g.body, 'onRender', 'reLayout');
			}else{
				t.connect(g, 'setColumns', function(){
					setTimeout(function(){
						t.reLayout();
					}, 0);
				});
			}
		},
	
		load: function(args, startup){
			// tags:
			//		protected extension
			var t = this;
			startup.then(function(){
				if(t._defs && t._mods){
					new DeferredList(t._defs).then(function(){
						t._layout();
						t.loaded.callback();
					});
				}else{
					t.loaded.callback();
				}
			});
		},
	
		//Public ---------------------------------------------------------------------
		register: function(mod, nodeName, hookPoint, priority, deferReady){
			// summary:
			//		When the 'mod' is loaded or "ready", hook 'mod'['nodeName'] to grid['hookPoint'] with priority 'priority'
			// mod: Object
			//		The module object
			// nodeName: String
			//		The name of the node to be hooked. Must be able to be accessed by mod[nodeName]
			// hookPoint: String
			//		The name of a hook point in grid.
			// priority: Number?
			//		The priority of the hook node. If less than 0, then it's above the base node, larger than 0, below the base node.
			var t = this;
			t._defs = t._defs || [];
			t._mods = t._mods || {};
			t._mods[hookPoint] = t._mods[hookPoint] || [];
			t._defs.push(deferReady || mod.loaded);
			t._mods[hookPoint].push({
				p: priority || 0,
				mod: mod,
				nodeName: nodeName
			});
		},
		
		reLayout: function(){
			// summary:
			//		Virtically re-layout all the grid UI parts.
			var t = this,
				freeHeight = 0,
				hookPoint, n;
			for(hookPoint in t._mods){
				n = t.grid[hookPoint];
				if(n){
					freeHeight += n.offsetHeight;
				}
			}
			t._updateHeight(freeHeight);
		},

		//Private-------------------------------------------------------------------------------
		_layout: function(){
			var freeHeight = 0,
				t = this,
				mods = t._mods,
				hookPoint, n, i, hp, mod, nodeName;
			for(hookPoint in mods){
				n = t.grid[hookPoint];
				if(n){
					hp = mods[hookPoint];
					hp.sort(function(a, b){
						return a.p - b.p;
					});
					for(i = 0; i < hp.length; ++i){
						mod = hp[i].mod;
						nodeName = hp[i].nodeName;
						if(mod && mod[nodeName]){
							n.appendChild(mod[nodeName]);
						}
					}
					freeHeight += n.offsetHeight;
				}
			}
			t._updateHeight(freeHeight);
		},

		_updateHeight: function(freeHeight){
			var g = this.grid,
				dn = g.domNode,
				ms = g.mainNode.style;
			if(g.autoHeight){
				g.vScroller.loaded.then(function(){
					var lastRow = g.bodyNode.lastChild,
						bodyHeight = lastRow ? lastRow.offsetTop + lastRow.offsetHeight : g.emptyNode.offsetHeight;
					dn.style.height = (bodyHeight + freeHeight) + 'px';
					ms.height = bodyHeight + "px";
				});
			}else if(dn.clientHeight > freeHeight){
				//If grid height is smaller than freeHeight, IE will throw errer.
				ms.height = (dn.clientHeight - freeHeight) + "px";
			}
		}
	});
});

},
'url:dijit/layout/templates/ScrollingTabController.html':"<div class=\"dijitTabListContainer-${tabPosition}\" style=\"visibility:hidden\">\n\t<div data-dojo-type=\"dijit.layout._ScrollingTabControllerMenuButton\"\n\t\t\tclass=\"tabStripButton-${tabPosition}\"\n\t\t\tid=\"${id}_menuBtn\"\n\t\t\tdata-dojo-props=\"containerId: '${containerId}', iconClass: 'dijitTabStripMenuIcon',\n\t\t\t\t\tdropDownPosition: ['below-alt', 'above-alt']\"\n\t\t\tdata-dojo-attach-point=\"_menuBtn\" showLabel=\"false\" title=\"\">&#9660;</div>\n\t<div data-dojo-type=\"dijit.layout._ScrollingTabControllerButton\"\n\t\t\tclass=\"tabStripButton-${tabPosition}\"\n\t\t\tid=\"${id}_leftBtn\"\n\t\t\tdata-dojo-props=\"iconClass:'dijitTabStripSlideLeftIcon', showLabel:false, title:''\"\n\t\t\tdata-dojo-attach-point=\"_leftBtn\" data-dojo-attach-event=\"onClick: doSlideLeft\">&#9664;</div>\n\t<div data-dojo-type=\"dijit.layout._ScrollingTabControllerButton\"\n\t\t\tclass=\"tabStripButton-${tabPosition}\"\n\t\t\tid=\"${id}_rightBtn\"\n\t\t\tdata-dojo-props=\"iconClass:'dijitTabStripSlideRightIcon', showLabel:false, title:''\"\n\t\t\tdata-dojo-attach-point=\"_rightBtn\" data-dojo-attach-event=\"onClick: doSlideRight\">&#9654;</div>\n\t<div class='dijitTabListWrapper' data-dojo-attach-point='tablistWrapper'>\n\t\t<div role='tablist' data-dojo-attach-event='onkeypress:onkeypress'\n\t\t\t\tdata-dojo-attach-point='containerNode' class='nowrapTabStrip'></div>\n\t</div>\n</div>",
'dijit/form/_FormWidgetMixin':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.set
	"dojo/dom-style", // domStyle.get
	"dojo/_base/lang", // lang.hitch lang.isArray
	"dojo/mouse", // mouse.isLeft
	"dojo/_base/sniff", // has("webkit")
	"dojo/_base/window", // win.body
	"dojo/window", // winUtils.scrollIntoView
	"../a11y"	// a11y.hasDefaultTabStop
], function(array, declare, domAttr, domStyle, lang, mouse, has, win, winUtils, a11y){

// module:
//		dijit/form/_FormWidgetMixin
// summary:
//		Mixin for widgets corresponding to native HTML elements such as <checkbox> or <button>,
//		which can be children of a <form> node or a `dijit.form.Form` widget.

return declare("dijit.form._FormWidgetMixin", null, {
	// summary:
	//		Mixin for widgets corresponding to native HTML elements such as <checkbox> or <button>,
	//		which can be children of a <form> node or a `dijit.form.Form` widget.
	//
	// description:
	//		Represents a single HTML element.
	//		All these widgets should have these attributes just like native HTML input elements.
	//		You can set them during widget construction or afterwards, via `dijit._Widget.attr`.
	//
	//		They also share some common methods.

	// name: [const] String
	//		Name used when submitting form; same as "name" attribute or plain HTML elements
	name: "",

	// alt: String
	//		Corresponds to the native HTML <input> element's attribute.
	alt: "",

	// value: String
	//		Corresponds to the native HTML <input> element's attribute.
	value: "",

	// type: [const] String
	//		Corresponds to the native HTML <input> element's attribute.
	type: "text",

	// tabIndex: Integer
	//		Order fields are traversed when user hits the tab key
	tabIndex: "0",
	_setTabIndexAttr: "focusNode",	// force copy even when tabIndex default value, needed since Button is <span>

	// disabled: Boolean
	//		Should this widget respond to user input?
	//		In markup, this is specified as "disabled='disabled'", or just "disabled".
	disabled: false,

	// intermediateChanges: Boolean
	//		Fires onChange for each value change or only on demand
	intermediateChanges: false,

	// scrollOnFocus: Boolean
	//		On focus, should this widget scroll into view?
	scrollOnFocus: true,

	// Override _WidgetBase mapping id to this.domNode, needs to be on focusNode so <label> etc.
	// works with screen reader
	_setIdAttr: "focusNode",

	_setDisabledAttr: function(/*Boolean*/ value){
		this._set("disabled", value);
		domAttr.set(this.focusNode, 'disabled', value);
		if(this.valueNode){
			domAttr.set(this.valueNode, 'disabled', value);
		}
		this.focusNode.setAttribute("aria-disabled", value ? "true" : "false");

		if(value){
			// reset these, because after the domNode is disabled, we can no longer receive
			// mouse related events, see #4200
			this._set("hovering", false);
			this._set("active", false);

			// clear tab stop(s) on this widget's focusable node(s)  (ComboBox has two focusable nodes)
			var attachPointNames = "tabIndex" in this.attributeMap ? this.attributeMap.tabIndex :
				("_setTabIndexAttr" in this) ? this._setTabIndexAttr : "focusNode";
			array.forEach(lang.isArray(attachPointNames) ? attachPointNames : [attachPointNames], function(attachPointName){
				var node = this[attachPointName];
				// complex code because tabIndex=-1 on a <div> doesn't work on FF
				if(has("webkit") || a11y.hasDefaultTabStop(node)){	// see #11064 about webkit bug
					node.setAttribute('tabIndex', "-1");
				}else{
					node.removeAttribute('tabIndex');
				}
			}, this);
		}else{
			if(this.tabIndex != ""){
				this.set('tabIndex', this.tabIndex);
			}
		}
	},

	_onFocus: function(/*String*/ by){
		// If user clicks on the widget, even if the mouse is released outside of it,
		// this widget's focusNode should get focus (to mimic native browser hehavior).
		// Browsers often need help to make sure the focus via mouse actually gets to the focusNode.
		if(by == "mouse" && this.isFocusable()){
			// IE exhibits strange scrolling behavior when refocusing a node so only do it when !focused.
			var focusConnector = this.connect(this.focusNode, "onfocus", function(){
				this.disconnect(mouseUpConnector);
				this.disconnect(focusConnector);
			});
			// Set a global event to handle mouseup, so it fires properly
			// even if the cursor leaves this.domNode before the mouse up event.
			var mouseUpConnector = this.connect(win.body(), "onmouseup", function(){
				this.disconnect(mouseUpConnector);
				this.disconnect(focusConnector);
				// if here, then the mousedown did not focus the focusNode as the default action
				if(this.focused){
					this.focus();
				}
			});
		}
		if(this.scrollOnFocus){
			this.defer(function(){ winUtils.scrollIntoView(this.domNode); }); // without defer, the input caret position can change on mouse click
		}
		this.inherited(arguments);
	},

	isFocusable: function(){
		// summary:
		//		Tells if this widget is focusable or not.  Used internally by dijit.
		// tags:
		//		protected
		return !this.disabled && this.focusNode && (domStyle.get(this.domNode, "display") != "none");
	},

	focus: function(){
		// summary:
		//		Put focus on this widget
		if(!this.disabled && this.focusNode.focus){
			try{ this.focusNode.focus(); }catch(e){}/*squelch errors from hidden nodes*/
		}
	},

	compare: function(/*anything*/ val1, /*anything*/ val2){
		// summary:
		//		Compare 2 values (as returned by get('value') for this widget).
		// tags:
		//		protected
		if(typeof val1 == "number" && typeof val2 == "number"){
			return (isNaN(val1) && isNaN(val2)) ? 0 : val1 - val2;
		}else if(val1 > val2){
			return 1;
		}else if(val1 < val2){
			return -1;
		}else{
			return 0;
		}
	},

	onChange: function(/*===== newValue =====*/){
		// summary:
		//		Callback when this widget's value is changed.
		// tags:
		//		callback
	},

	// _onChangeActive: [private] Boolean
	//		Indicates that changes to the value should call onChange() callback.
	//		This is false during widget initialization, to avoid calling onChange()
	//		when the initial value is set.
	_onChangeActive: false,

	_handleOnChange: function(/*anything*/ newValue, /*Boolean?*/ priorityChange){
		// summary:
		//		Called when the value of the widget is set.  Calls onChange() if appropriate
		// newValue:
		//		the new value
		// priorityChange:
		//		For a slider, for example, dragging the slider is priorityChange==false,
		//		but on mouse up, it's priorityChange==true.  If intermediateChanges==false,
		//		onChange is only called form priorityChange=true events.
		// tags:
		//		private
		if(this._lastValueReported == undefined && (priorityChange === null || !this._onChangeActive)){
			// this block executes not for a change, but during initialization,
			// and is used to store away the original value (or for ToggleButton, the original checked state)
			this._resetValue = this._lastValueReported = newValue;
		}
		this._pendingOnChange = this._pendingOnChange
			|| (typeof newValue != typeof this._lastValueReported)
			|| (this.compare(newValue, this._lastValueReported) != 0);
		if((this.intermediateChanges || priorityChange || priorityChange === undefined) && this._pendingOnChange){
			this._lastValueReported = newValue;
			this._pendingOnChange = false;
			if(this._onChangeActive){
				if(this._onChangeHandle){
					this._onChangeHandle.remove();
				}
				// defer allows hidden value processing to run and
				// also the onChange handler can safely adjust focus, etc
				this._onChangeHandle = this.defer(
					function(){
						this._onChangeHandle = null;
						this.onChange(newValue);
					}); // try to collapse multiple onChange's fired faster than can be processed
			}
		}
	},

	create: function(){
		// Overrides _Widget.create()
		this.inherited(arguments);
		this._onChangeActive = true;
	},

	destroy: function(){
		if(this._onChangeHandle){ // destroy called before last onChange has fired
			this._onChangeHandle.remove();
			this.onChange(this._lastValueReported);
		}
		this.inherited(arguments);
	}
});

});

},
'gridx/modules/CellWidget':function(){
define([
	"dojo/_base/declare",	
	"dojo/_base/query",
	"dojo/_base/array",
	"dojo/_base/event",
	"dojo/_base/sniff",
	"dojo/dom-class",
	"dojo/keys",
	"dijit/registry",
	"dijit/a11y",
	"dijit/_WidgetBase",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	"../core/_Module"
], function(declare, query, array, event, sniff, domClass, keys, 
	registry, a11y, _WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin, _Module){

	/*=====
	var columnDefinitionCellDijitMixin = {
		// navigable: Boolean
		//      If a cell is navigable, that means the focusable elements in the cell can be focused.
		//      Use F2 to enter navigation mode, ESC to exit navigation mode and return focus back to cell.
		//      Pressing TAB during navigation mode will focus the next focusalbe element in all cells.
		//      That means if the current focused element is the last one in the current cell, then pressing TAB
		//      will focus the first focusable element in the next cell.
		navigable: true,

		// widgetsInCell: Boolean
		//		Indicating whether this column should use this CellDijit module.
		//		CellDijit module reuses widgets in cell, so if there is no widgets in cell, you don't need this module at all.
		widgetsInCell: false,

		// decorator: Function(data, rowId, rowIndex) return String
		//		This decorator function is slightly different from the one when this module is not used.
		//		This function should return a template string (see the doc for template string in dijit._TemplatedMixin
		//		and dijit._WidgetsInTemplateMixin). 
		//		In the template string, dijits or widgets can be used and they will be properly set value if they
		//		have the CSS class 'gridxHasGridCellValue' in their DOM node.
		//		Since setting value will be done automatically, and the created widgets will be reused between rows,
		//		so there's no arguments for this function.
		//		By default the dijits or widgets will be set value using the grid data (the result of the formatter function,
		//		if there is a formatter function for this column), not the store data (the raw data stored in store).
		//		If you'd like to use store data in some dijit, you can simly add a CSS class 'gridxUseStoreData' to it.
		decorator: null,
	
		// setCellValue: Function(gridData, storeData, cellWidget)
		//		If the settings in the decorator function can not meet your requirements, you use this function as a kind of complement.
		//		gridData: anything
		//				The data shown in grid cell. It's the result of formatter function if that function exists.
		//		storeData: anything
		//				The raw data in dojo store.
		//		cellWidget: CellWidget
		//				A widget representing the whole cell. This is the container of the templateString returned by decorator.
		//				So you can access any dojoAttachPoint from it (maybe your special dijit or node, and then set value for them).
		setCellValue: null
	};
	=====*/
	
	var dummyFunc = function(){ return ''; },

		CellWidget = declare([_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin], {
		
			content: '',
		
			setCellValue: null,

			cell: null,
		
			postMixInProperties: function(){
				this.templateString = ['<div class="gridxCellWidget">', this.content, '</div>'].join('');
			},

			postCreate: function(){
				this.connect(this.domNode, 'onmousedown', function(e){
					e.cancelBubble = true;
				});
			},
		
			setValue: function(gridData, storeData, isInit){
				try{
					var t = this;
					query('.gridxHasGridCellValue', t.domNode).map(function(node){
						return registry.byNode(node);
					}).forEach(function(widget){
						if(widget){
							var useStoreData = domClass.contains(widget.domNode, 'gridxUseStoreData'),
								data = useStoreData ? storeData : gridData,
								onChange = widget.onChange;
							//If we are just rendering this cell, setting widget value should not trigger onChange event,
							//which will then trigger edit apply. But things are complicated because onChange is
							//fired asynchronously, and maybe sometimes not fired.
							//FIXME: How to ensure the onChange event does not fire if isInit is true?
							if(isInit && widget.get('value') !== data){
								widget.onChange = function(){
									widget.onChange = onChange;
								};
							}
							if(!t.setCellValue){
								widget.set('value', data);
							}
						}
					});
					if(t.setCellValue){
						t.setCellValue(gridData, storeData, t, isInit);
					}
				}catch(e){
					console.error('Can not set cell value: ', e);
				}
			}
		});

	_Module._markupAttrs.push('!widgetsInCell', '!setCellValue');

	return declare(/*===== "gridx.modules.CellWidget", =====*/_Module, {
		// summary:
		//		This module makes it possible to efficiently show widgets within a grid cell.
		// description:
		//		Since widget declarations need to be parsed by dojo.parser, it can NOT be directly
		//		created by the decorator function. This module takes advantage of the _TemplatedMixin
		//		and the _WidgetInTemplateMixin so that users can write "templates" containing widgets
		//		in decorator function.
		//		This modules also limits the total number of widgets, so that the performance of grid
		//		can be configured to a tolerable level when there're lots of widgets in grid.

		name: 'cellWidget',
	
		getAPIPath: function(){
			// tags:
			//		protected extension
			return {
				cellWidget: this
			};
		},

		cellMixin: {
			widget: function(){
				// summary:
				//		Get the cell widget in this cell.
				return this.grid.cellWidget.getCellWidget(this.row.id, this.column.id);
			}
		},
	
		constructor: function(){
			this._init();
		},
	
		preload: function(){
			// tags:
			//		protected extension
			var t = this, body = t.grid.body;
			t.batchConnect(
				[body, 'onAfterRow', '_showDijits'],
				[body, 'onAfterCell', '_showDijit'],
				[body, 'onUnrender', '_onUnrenderRow']);
			t._initFocus();
		},
	
		destroy: function(){
			// tags:
			//		protected extension
			this.inherited(arguments);
			var i, id, col, cw, columns = this.grid._columns;
			for(i = columns.length - 1; i >= 0; --i){
				col = columns[i];
				cw = col._cellWidgets;
				if(cw){
					for(id in cw){
						cw[id].destroyRecursive();
					}
					delete col._cellWidgets;
				}
			}
		},
	
		//Public-----------------------------------------------------------------

		// backupCount: Integer
		//		The count of backup widgets for every column which contains widgets
		backupCount: 20,

		setCellDecorator: function(rowId, colId, decorator, setCellValue){
			// summary:
			//		This method is used to decorate a specific cell instead of a whole column.
			// rowId: String
			//		The row ID of the cell
			// colId: String
			//		The column ID of the cell
			// decorator: Function(data)
			//		The decorator function for this cell.
			// setCellValue: Function()?
			//		This function can be provided to customiz the way of setting widget value
			var rowDecs = this._decorators[rowId];
			if(!rowDecs){
				rowDecs = this._decorators[rowId] = {};
			}
			var cellDec = rowDecs[colId];
			if(!cellDec){
				cellDec = rowDecs[colId] = {};
			}
			cellDec.decorator = decorator;
			cellDec.setCellValue = setCellValue;
			cellDec.widget = null;
		},
	
		restoreCellDecorator: function(rowId, colId){
			// summary:
			//		Remove a cell decorator defined by the "setCellDecorator" method.
			// rowId: String
			//		The row ID of the cell
			// colId: String
			//		The column ID of the cell
			var rowDecs = this._decorators[rowId];
			if(rowDecs){
				var cellDec = rowDecs[colId];
				if(cellDec){
					if(cellDec.widget){
						//Because dijit.form.TextBox use setTimeout to fire onInput event, 
						//so we can not simply destroy the widget when ENTER key is pressed for an editing cell!!
						var parentNode = cellDec.widget.domNode.parentNode;
						if(parentNode){
							parentNode.innerHTML = null;
						}
						window.setTimeout(function(){
							cellDec.widget.destroyRecursive();
							cellDec.widget = null;
							cellDec.decorator = null;
							cellDec.setCellValue = null;
						}, 100);
					}
				}
				delete rowDecs[colId];
			}
		},
	
		getCellWidget: function(rowId, colId){
			// summary:
			//		Get the CellWidget displayed in the given cell.
			// description:
			//		When this module is used, the string returned from decorator function will be
			//		the template string of a CellWidget. This method gets this widget so that
			//		more control can be applied to it.
			// rowId: string
			//		The row ID of the cell
			// colId: string
			//		The column ID of the cell
			var cellNode = this.grid.body.getCellNode({
				rowId: rowId, 
				colId: colId
			});
			if(cellNode){
				var widgetNode = query('.gridxCellWidget', cellNode)[0];
				if(widgetNode){
					return registry.byNode(widgetNode);
				}
			}
			return null;
		},

		onCellWidgetCreated: function(/* widget, cell */){
			// summary:
			//		Fired when a cell widget is created.
			// widget: gridx.__CellWidget
			//		The created cell widget.
			// cell: gridx.core.Cell
			//		The cell object containing this widget.
			// tags:
			//		callback
		},
	
		//Private---------------------------------------------------------------
		_init: function(){
			this._decorators = {};
			var i, col, columns = this.grid._columns;
			for(i = columns.length - 1; i >= 0; --i){
				col = columns[i];
				if(col.decorator && col.widgetsInCell){
					col.userDecorator = col.decorator;
					col.decorator = dummyFunc;
					col._cellWidgets = {};
					col._backupWidgets = [];
				}
			}
		},

		_showDijits: function(row){
			var t = this;
			array.forEach(row.cells(), function(cell){
				var col = cell.column.def();
				if(col.userDecorator || t._getSpecialCellDec(cell.row.id, col.id)){
					var cellNode = cell.node();
					if(cellNode){
						var cellWidget = t._prepareCellWidget(cell);
						if(sniff('ie')){
							while(cellNode.childNodes.length){
								cellNode.removeChild(cellNode.firstChild);
							}
						}else{
							cellNode.innerHTML = "";
						}
						cellWidget.placeAt(cellNode);
						cellWidget.startup();
					}
				}
			});
		},

		_showDijit: function(cell){
			var col = cell.column.def();
			if(col.userDecorator || this._getSpecialCellDec(cell.row.id, col.id)){
				var cellWidget = this._prepareCellWidget(cell),
					cellNode = cell.node();
				cellNode.innerHTML = "";
				cellWidget.placeAt(cellNode);
				cellWidget.startup();
			}
		},
	
		_prepareCellWidget: function(cell){
			var col = cell.column.def(),
				widget = this._getSpecialWidget(cell);
			if(!widget){
				widget = col._backupWidgets.pop();
				if(!widget){
					widget = new CellWidget({
						content: col.userDecorator(),
						setCellValue: col.setCellValue
					});
					this.onCellWidgetCreated(widget, cell);
				}
				col._cellWidgets[cell.row.id] = widget;
			}
			widget.cell = cell;
			widget.setValue(cell.data(), cell.rawData(), true);
			return widget;
		},

		_onUnrenderRow: function(id){
			var cols = this.grid._columns,
				backupCount = this.arg('backupCount'),
				backup = function(col, rowId){
					var w = col._cellWidgets[rowId];
					if(col._backupWidgets.length < backupCount){
						col._backupWidgets.push(w);
					}else{
						w.destroyRecursive();
					}
				};
			for(var i = 0, len = cols.length; i < len; ++i){
				var col = cols[i],
					cellWidgets = col._cellWidgets;
				if(cellWidgets){
					if(this.model.isId(id) && cellWidgets[id]){
						backup(col, id);
						delete cellWidgets[id];
					}else{
						for(var j in cellWidgets){
							backup(col, j);
						}
						col._cellWidgets = {};
					}
				}
			}
		},
	
		_getSpecialCellDec: function(rowId, colId){
			var rowDecs = this._decorators[rowId];
			return rowDecs && rowDecs[colId];
		},
	
		_getSpecialWidget: function(cell){
			var rowDecs = this._decorators[cell.row.id];
			if(rowDecs){
				var cellDec = rowDecs[cell.column.id];
				if(cellDec){
					if(!cellDec.widget && cellDec.decorator){
						try{
							cellDec.widget = new CellWidget({
								content: cellDec.decorator(cell.data(), cell.row.id, cell.row.visualIndex()),
								setCellValue: cellDec.setCellValue
							});
						}catch(e){
							console.error('Edit:', e);
						}
					}
					return cellDec.widget;
				}
			}
			return null;
		},

		//Focus
		_initFocus: function(){
			var t = this, focus = t.grid.focus;
			if(focus){
				focus.registerArea({
					name: 'cellwidget',
					priority: 1,
					scope: t,
					doFocus: t._doFocus,
					doBlur: t._doBlur,
					onFocus: t._onFocus,
					onBlur: t._endNavigate,
					connects: [
						t.connect(t.grid, 'onCellKeyDown', '_onKey')
					]
				});
			}
		},

		_doFocus: function(evt, step){
			if(this._navigating){
				var elems = this._navElems,
					func = function(){
						var toFocus = step < 0 ? (elems.highest || elems.last) : (elems.lowest || elems.first);
						if(toFocus){
							toFocus.focus();
						}
					};
				if(sniff('webkit')){
					func();
				}else{
					setTimeout(func, 5);
				}
				return true;
			}
			return false;
		},

		_doBlur: function(evt, step){
			if(evt){
				var t = this,
					m = t.model,
					g = t.grid,
					body = g.body,
					elems = t._navElems,
					firstElem = elems.lowest || elems.first,
					lastElem = elems.last || elems.highest || firstElem,
					target = sniff('ie') ? evt.srcElement : evt.target;
				if(target == (step > 0 ? lastElem : firstElem)){
					event.stop(evt);
					m.when({id: t._focusRowId}, function(){
						var rowIndex = body.getRowInfo({
								parentId: m.treePath(t._focusRowId).pop(), 
								rowIndex: m.idToIndex(t._focusRowId)
							}).visualIndex,
							colIndex = g._columnsById[t._focusColId].index,
							dir = step > 0 ? 1 : -1,
							checker = function(r, c){
								return t._isNavigable(g._columns[c].id);
							};
						body._nextCell(rowIndex, colIndex, dir, checker).then(function(obj){
							t._focusColId = g._columns[obj.c].id;
							//This kind of breaks the encapsulation...
							var rowInfo = body.getRowInfo({visualIndex: obj.r});
							t._focusRowId = m.indexToId(rowInfo.rowIndex, rowInfo.parentId);
							body._focusCellCol = obj.c;
							body._focusCellRow = obj.r;
							t._beginNavigate(t._focusRowId, t._focusColId);
							t._doFocus(null, step);
						});
					});
				}
				return false;
			}else{
				this._navigating = false;
				return true;
			}
		},

		_isNavigable: function(colId){
			var col = this.grid._columnsById[colId];
			return col && (col.navigable || col.navigable === undefined) && col.decorator;
		},

		_beginNavigate: function(rowId, colId){
			var t = this;
			if(t._isNavigable(colId)){
				t._navigating = true;
				t._focusColId = colId;
				t._focusRowId = rowId;
				t._navElems = a11y._getTabNavigable(t.grid.body.getCellNode({
					rowId: rowId, 
					colId: colId
				}));
				return true;
			}
			return false;
		},

		_endNavigate: function(){
			this._navigating = false;
		},

		_onFocus: function(evt){
			var node = evt.target, dn = this.grid.domNode;
			while(node && node !== dn && !domClass.contains(node, 'gridxCell')){
				node = node.parentNode;
			}
			if(node && node !== dn){
				var cellNode = node,
					colId = node.getAttribute('colid');
				while(node && !domClass.contains(node, 'gridxRow')){
					node = node.parentNode;
				}
				if(node){
					var rowId = node.getAttribute('rowid');
					return cellNode != evt.target && this._beginNavigate(rowId, colId);
				}
			}
			return false;
		},
		
		_onKey: function(e){
			var t = this, focus = t.grid.focus;
			if(e.keyCode == keys.F2 && !t._navigating && focus.currentArea() == 'body'){
				if(t._beginNavigate(e.rowId, e.columnId)){
					event.stop(e);
					focus.focusArea('cellwidget');
				}
			}else if(e.keyCode == keys.ESCAPE && t._navigating && focus.currentArea() == 'cellwidget'){
				t._navigating = false;
				focus.focusArea('body');
			}
		}
	});
});

},
'gridx/core/model/Model':function(){
define([
	"require",
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/_base/Deferred",
	"dojo/DeferredList",
	"dojo/aspect"
], function(require, declare, array, lang, Deferred, DeferredList, aspect){

	var isArrayLike = lang.isArrayLike,
		isString = lang.isString;

	
	return declare(/*===== "gridx.core.model.Model", =====*/[], {
		// summary:
		//		This class handles all of the data logic in grid.
		// description:
		//		It provides a clean and useful set of APIs to encapsulate complicated data operations, 
		//		even for huge asynchronous (server side) data stores.
		//		It is built upon a simple extension mechanism, allowing new (even user defined) data operaions to be pluged in.
		//		An instance of this class can be regarded as a stand-alone logic grid providing consistent data processing 
		//		functionalities. This class can even be instanticated alone without any grid UI.

		
		constructor: function(args){
			var t = this,
				cacheClass = args.cacheClass;
			cacheClass = typeof cacheClass == 'string' ? require(cacheClass) : cacheClass;
			t.store = args.store;
			t._exts = {};
			t._cmdQueue = [];
			t._model = t._cache = new cacheClass(t, args);
			t._createExts(args.modelExtensions || [], args);
			var m = t._model;
			t._cnnts = [
				aspect.after(m, "onDelete", lang.hitch(t, "onDelete"), 1),
				aspect.after(m, "onNew", lang.hitch(t, "onNew"), 1),
				aspect.after(m, "onSet", lang.hitch(t, "onSet"), 1)
			];
		},
	
		destroy: function(){
			array.forEach(this._cnnts, function(cnnt){
				cnnt.remove();
			});
			for(var n in this._exts){
				this._exts[n].destroy();
			}
		},

		clearCache: function(){
			this._cache.clear();
		},

		isId: function(id){
			return id || id === 0;
		},

		setStore: function(store){
			this.store = store;
			this._cache.setStore(store);
		},
	
		//Public-------------------------------------------------------------------

		/*=====
		byIndex: function(index, parentId){
			// summary:
			//		Get the row cache by row index.
			// index: Integer
			//		The row index
			// parentId: String?
			//		If parentId is valid, the row index means the child index under this parent.
			// returns:
			//		The row cache
			return null;	//gridx.core.model.__RowCache
		},

		byId: function(id){
			// summary:
			//		Get the row cache by row id
			// id: String
			//		The row ID
			// returns:
			//		The row cache
			return null;	//gridx.core.model.__RowCache
		},

		indexToId: function(index, parentId){
			// summary:
			//		Transform row index to row ID
			// index: Integer
			//		The row index
			// parentId: String?
			//		If parentId is valid, the row index means the child index under this parent.
			// returns:
			//		The row ID
			return '';	//String
		},

		idToIndex: function(id){
			// summary:
			//		Transform row ID to row index
			// id: String
			//		The row ID
			// returns:
			//		The row index
			return -1;	//Integer
		},

		treePath: function(id){
			// summary:
			//		Get tree path of row by row ID
			// id: String
			//		The row ID
			// returns:
			//		An array of parent row IDs, from root to parent.
			//		Root level rows have parent of id ""(empty string).
			return [];	//String[]
		},

		parent: function(id){
			// summary:
			//		Get the parent ID of the given row.
			// id: String
			//		The row ID
			// returns:
			//		The parent ID.
			return [];
		},

		hasChildren: function(id){
			// summary:
			//		Check whether a row has children rows.
			// id: String
			//		The row ID
			// returns:
			//		Whether this row has child rows.
			return false;	//Boolean
		},

		children: function(id){
			// summary:
			//		Get IDs of children rows.
			// id: String
			//		The row ID
			// returns:
			//		An array of row IDs
			return [];	//Array
		},

		size: function(parentId){
			// summary:
			//		Get the count of rows under the given parent. 
			// parentId: String?
			//		The ID of a parent row. No parentId means root rows.
			// returns:
			//		The count of (child) rows
			return -1;	//Integer
		},

		keep: function(id){
			// summary:
			//		Lock up a row cache in memory, avoid clearing it out when cache size is reached.
			// id: String
			//		The row ID
		},

		free: function(id){
			// summary:
			//		Unlock a row cache in memory, so that it could be cleared out when cache size is reached.
			// id: String?
			//		The row ID. If omitted, all kept rows will be freed.
		},
		=====*/

		
		when: function(args, callback, scope){
			// summary:
			//		Call this method to make sure all the pending data operations are executed and
			//		all the needed rows are at client side.
			// description:
			//		This method makes it convenient to do various grid operations without worrying too much about server side
			//		or client side store. This method is the only asynchronous public method in grid model, so that most of
			//		the custom code can be written in synchronous way.
			// args: Object|null?
			//		Indicate what rows are needed by listing row IDs or row indexes.
			//		Acceptable args include: 
			//		1. A single row index.
			//		e.g.: model.when(1, ...)
			//		2. A single row index range object in form of: {start: ..., count: ...}.
			//		If count is omitted, means all remaining rows.
			//		e.g.: model.when({start: 10, count: 100}, ...)
			//		3. An array of row indexes and row index ranges.
			//		e.g.: model.when([0, 1, {start: 10, count: 3}, 100], ...)
			//		4. An object with property "index" set to the array defined in 3.
			//		e.g.: model.when({
			//			index: [0, 1, {start: 10, count: 3}, 100]
			//		}, ...)
			//		5. An object with property "id" set to an array of row IDs.
			//		e.g.: model.when({
			//		id: ['a', 'b', 'c']
			//		}, ...)
			//		6. An object containing both contents defined in 4 and 5.
			//		7. An empty object
			//		The model will fetch the store size. Currently it is implemented by fetching the first page of data.
			//		8. null or call this method without any arguments.
			//		This is useful when we only need to execute pending data operations but don't need to fetch rows.
			// callback: Function?
			//		The callback function is called when all the pending data operations are executed and all
			// returns:
			//		A Deferred object indicating when all this process is finished. Note that in this Deferred object,
			//		The needed rows might not be available since they might be cleared up to reduce memory usage.
			this._oldSize = this.size();
			this._addCmd({
				name: '_cmdRequest',
				scope: this,
				args: arguments,
				async: 1
			});
			return this._exec();	//dojo.Deferred
		},
	
		
		scan: function(args, callback){
			// summary:
			//		Go through all the rows in several batches from start to end (or according to given args),
			//		and execute the callback function for every batch of rows.
			// args: Object
			//		An object containing scan arguments
			// callback: Function(rows,startIndex)
			//		The callback function.
			// returns:
			//		If return true in this function, the scan process will end immediately.
			var d = new Deferred,
				start = args.start || 0,
				pageSize = args.pageSize || this._cache.pageSize || 1,
				count = args.count,
				end = count > 0 ? start + count : Infinity,
				scope = args.whenScope || this,
				whenFunc = args.whenFunc || scope.when;
			var f = function(s){
					d.progress(s / (count > 0 ? s + count : scope.size()));
					whenFunc.call(scope, {
						id: [],
						range: [{
							start: s,
							count: pageSize
						}]
					}, function(){
						var i, r, rows = [];
						for(i = s; i < s + pageSize && i < end; ++i){
							r = scope.byIndex(i);
							if(r){
								rows.push(r);
							}else{
								end = -1;
								break;
							}
						}
						if(callback(rows, s) || i == end){
							end = -1;
						}
					}).then(function(){
						if(end == -1){
							d.callback();
						}else{
							f(s + pageSize);
						}
					});
				};
			f(start);
			return d;	//dojo.Deferred
		},

		//Events---------------------------------------------------------------------------------
		onDelete: function(/*id, index*/){
			// summary:
			//		Fired when a row is deleted from store
			// tags:
			//		callback
		},

		onNew: function(/*id, index, row*/){
			// summary:
			//		Fired when a row is added to the store
			// tags:
			//		callback
		},

		onSet: function(/*id, index, row*/){
			// summary:
			//		Fired when a row's data is changed
			// tags:
			//		callback
		},

		onSizeChange: function(/*size, oldSize*/){
			// summary:
			//		Fired when the size of the grid model is changed
			// tags:
			//		callback
		},

		//Package----------------------------------------------------------------------------
		_msg: function(/* msg */){},

		_addCmd: function(args){
			//Add command to the command queue, and combine same kind of commands if possible.
			var cmds = this._cmdQueue,
				cmd = cmds[cmds.length - 1];
			if(cmd && cmd.name == args.name && cmd.scope == args.scope){
				cmd.args.push(args.args || []);
			}else{
				args.args = [args.args || []];
				cmds.push(args);
			}
		},

		//Private----------------------------------------------------------------------------
		_onSizeChange: function(){
			var t = this,
				oldSize = t._oldSize,
				size = t._oldSize = t.size();
			if(oldSize != size){
				t.onSizeChange(size, oldSize);
			}
		},

		_execEvents: function(scope, callback){
			this._onSizeChange();
			//TODO: fire events here
			if(callback){
				callback.call(scope);
			}
		},

		_cmdRequest: function(){
			var t = this;
			return new DeferredList(array.map(arguments, function(args){
				var arg = args[0],
					finish = lang.hitch(t, t._execEvents, args[2], args[1]);
				if(arg === null || !args.length){
					var d = new Deferred;
					finish();
					d.callback();
					return d;
				}
				return t._model._call('when', [t._normArgs(arg), finish]);
			}), 0, 1);
		},

		_exec: function(){
			//Execute commands one by one.
			var t = this,
				c = t._cache,
				d = new Deferred,
				cmds = t._cmdQueue,
				finish = function(d, err){
					t._busy = 0;
					c.skipCacheSizeCheck = 0;
					if(c._checkSize){
						c._checkSize();
					}
					if(err){
						d.errback(err);
					}else{
						d.callback();
					}
				},
				func = function(){
					if(array.some(cmds, function(cmd){
						return cmd.name == '_cmdRequest';
					})){
						try{
							while(cmds.length){
								var cmd = cmds.shift(),
									dd = cmd.scope[cmd.name].apply(cmd.scope, cmd.args);
								if(cmd.async){
									Deferred.when(dd, func, lang.partial(finish, d));
									return;
								}
							}
						}catch(e){
							finish(d, e);
							return;
						}
					}
					finish(d);
				};
			if(t._busy){
				return t._busy;
			}
			t._busy = d;
			c.skipCacheSizeCheck = 1;
			func();
			return d;
		},
	
		_createExts: function(exts, args){
			//Ensure the given extensions are valid
			exts = array.filter(exts, function(ext){
				ext = typeof ext == 'string' ? require(ext) : ext;
				return ext && ext.prototype;
			});
			//Sort the extensions by priority
			exts.sort(function(a, b){
				return a.prototype.priority - b.prototype.priority;
			});
			for(var i = 0, len = exts.length; i < len; ++i){
				//Avoid duplicated extensions
				//IMPORTANT: Assume extensions all have different priority values!
				if(i == exts.length - 1 || exts[i] != exts[i + 1]){
					var ext = new exts[i](this, args);
					this._exts[ext.name] = ext;
				}
			}
		},
	
		_normArgs: function(args){
			var i, rgs = [], ids = [],
			res = {
				range: rgs,
				id: ids 
			},
			isIndex = function(a){
				return typeof a == 'number' && a >= 0;
			},
			isRange = function(a){
				return a && isIndex(a.start);
			},
			f = function(a){
				if(isRange(a)){
					rgs.push(a);
				}else if(isIndex(a)){
					rgs.push({start: a, count: 1});
				}else if(isArrayLike(a)){
					for(i = a.length - 1; i >= 0; --i){
						if(isIndex(a[i])){
							rgs.push({
								start: a[i],
								count: 1
							});
						}else if(isRange(a[i])){
							rgs.push(a[i]);
						}else if(isString(a)){
							ids.push(a[i]);
						}
					}
				}else if(isString(a)){
					ids.push(a);
				}
			};
			if(args && (args.index || args.range || args.id)){
				f(args.index);
				f(args.range);
				if(isArrayLike(args.id)){
					for(i = args.id.length - 1; i >= 0; --i){
						ids.push(args.id[i]);
					}
				}else if(this.isId(args.id)){
					ids.push(args.id);
				}
			}else{
				f(args);
			}
			if(!rgs.length && !ids.length && this.size() < 0){
				//first time load, try to load a page
				rgs.push({start: 0, count: this._cache.pageSize || 1});
			}
			return res;
		}
	});
});

},
'ibm/usmi/util/DefaultDialogTask':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

define("ibm/usmi/util/DefaultDialogTask", [
	"dijit",
	"dijit/_WidgetBase",
	"dijit/Dialog",
	"dijit/form/Button",
	"ibm/tivoli/navmodel/Trace",
	"dojo/_base/declare",
	"dojox/data/dom",
	"dojo/topic",
	"dojo/_base/lang",
	"dijit/registry",
	"dojo/dom-style",
	"dojo/dom-construct",
	"dojo/i18n!ibm/usmi/nls/frameworkResources",
	"dojo/i18n!ibm/usmi/nls/defaultTaskResources",
	"dojo/window",
	"dojo/dom-geometry"
],

function (dijit, _WidgetBase, _Dialog, _Button, _Trace, declare, dom, topic, lang, registry, style, construct,
		frameworkmri, defaultmri, winUtils, domGeometry) {

	//ibm.usmi.util.DefaultDialogTask._trace = new ibm.tivoli.navmodel.Trace("ibm.usmi.util.DefaultDialogTask");

	return declare("ibm.usmi.util.DefaultDialogTask", [
		_WidgetBase,
		_Dialog
	], {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon

		//***************************
		// "Public"
		//***************************	

		//*************************** 
		// "Private"
		//***************************	

		// _trace: [private] ibm.tivoli.navmodel.Trace
		//		Utility for trace and logging
		_trace : new _Trace("ibm.usmi.util.DefaultDialogTask"),

		// _dialogDijit: [private]
		//		Coming soon
		_dialogDijit : null,
		
		_dijitDiv: null,
		
		_dialogDiv: null,

		_subscriptions : [],

		FrameworkResources : frameworkmri,
		DefaultResources : defaultmri,

		constructor : function () {
			// summary:
			//		Coming soon
			// description:
			//		Coming soon

			//set title
			this.title = this.DefaultResources.DELETE; // TODO - get from MRI bundle

			// Register for a DialogTask event
			var sub = topic.subscribe("DefaultDialogLaunch", lang.hitch(this, this._handleDialogLaunch));
			this._subscriptions.push(sub);
			sub = topic.subscribe("DefaultNoninteractiveTask", lang.hitch(this, this._handleDialogLaunch));
			this._subscriptions.push(sub);
		},

		postCreate : function () {
			// summary:
			//		Coming soon
			// description:
			//		Coming soon

			//Call superclass
			this.inherited(arguments);

			// Register for a Dialog dismissal event
			var sub = topic.subscribe("DefaultDialogDismissal", lang.hitch(this, this._close));
			this._subscriptions.push(sub);
		},

		startup : function () {
			// summary:
			//		Coming soon
			// description:
			//		Coming soon

			this.inherited(arguments);
		},

		destroy : function () {
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			if (this._dialogDijit) {
				this._dialogDijit.destroyRecursive();
			}
			if (this._subscriptions) {
				var i;
				for (i=0; i < this._subscriptions.length; i++) {
					this._subscriptions[i].remove();
				}
				this._subscriptions = null;
			}
			this.inherited(arguments);
		},

		//override the onCancel method to also destroy the whole widget.
		_close : function () {
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			//setTimeout('registry.byId(\"' + this.id + '\").destroyRecursive();',200); 
			dijit.byId(this.id).hide();
			this.inherited(arguments);
		},

	    _handleDialogLaunch : function (launchParms, menuItem, context) {
		    // summary:
		    //		Coming soon
		    // description:
		    //		Coming soon

		    var dialogContentWidget = null, divHeight, diijtHeight;
		    if (launchParms) {
			    // Handle undefined path and dijit, use utils.DefaultConfirmationDialogTask
			    if (!launchParms.modulePath && !launchParms.dijit) {
				    launchParms.modulePath = "ibm.usmi.tasks";
				    launchParms.dijit = "DefaultConfirmationDialogTask";
			    }
			    dialogContentWidget = launchParms.modulePath + "." + launchParms.dijit;
			   
			    if(!this._dijitDiv){
			    	this._dijitDiv = construct.create("div");
				    construct.place(this._dijitDiv, this.containerNode);
			    }
			    
				
			    
			    /*
				//the child dialog widget should implement getBodyHeight() to ensure the correct height
			    divHeight = null;
			    try {
				    divHeight = this._dialogDijit.getBodyHeight();
			    } catch (err) {
				    divHeight = 0.2 * this._getBodyHeight();
			    }
				if (!divHeight) {
					divHeight = 0.8 * this._getBodyHeight();
				}
			    style.set(dijitDiv, "height", divHeight);
			    
			    */

				//this.show();
				
			    // custom builds don't like passing variables into dojo.require
			    // so we have to do a little trick instead - RFB
			    // dojo.require(dialogContentWidget); // RFB
			    dojo["require"](dialogContentWidget);

			    var classname = eval(dialogContentWidget);
			    if (this._dialogDijit) {
				    this._dialogDijit.destroy();
				    this._dialogDijit = null;
			    }
			    if(this._dialogDiv){
			    	construct.destroy(this._dialogDiv);
			    	delete this._dialogDiv;
			    }
			    this._dialogDiv=construct.create("DIV");
			    this._dijitDiv.appendChild(this._dialogDiv);
			    
			    if (!this._dialogDijit) { //create the widget. 
				    this._dialogDijit = new classname({
				        // id causee a TaskID already exist error in Dojo
				        // id : launchParms.dijit + "_DialogContentWidget",
				        launchParms : launchParms,
				        menuItem : menuItem,
				        context : context
				    }, this._dialogDiv);
			    }

				this.show();

				this._dialogDijit.startup();

				
				if (this._dialogDijit.resize) {
					if (dialogContentWidget === "ibm.usmi.scheduler.JobLaunchWizard") {
						var bb = domGeometry.position(this.containerNode);
						diijtHeight=bb.h;
						diijtHeight-=32;
						//diijtHeight-=2; //remove two additional pixels
						this._dialogDijit.resize( {
							w : 700,
							h : diijtHeight
						}); 
					} else {
					this._dialogDijit.resize();
					}					
				}
				
				if (this._dialogDijit.updateUI) {
					// no longer need to call updateUI twice
					// this._dialogDijit.updateUI();
					this._dialogDijit.updateUI();
				}
				this.layout();
				
			    if (launchParms.dialogTitle) {
				    this.attr("title", launchParms.dialogTitle);
			    }

		    } else {
			    alert("undefined launch parms from server task");
			    // TODO log error and put up message?
		    }

	    },

		_getBodyHeight : function () {
			return ((window.innerHeight !== undefined) ? window.innerHeight : document.body.offsetHeight);
		}
	});
});

},
'ibm/usmi/statuspod/JobStatusPodMenuItem':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define(
		[
			"dojo/_base/declare",
			"./StatusPodMenuItem"
		],
		function (declare, StatusPodMenuItem) {
			return declare(
					[ StatusPodMenuItem
					],
					{
						onClick : function (/*Event*/) {
							var menuItemOnClickFunc = window["launchInContext"];
							//menuItemOnClickFunc('com.ibm.usmi.console.scheduler.navigationElement', this.objectId);
							var url = window.location.protocol
									+ '//'
									+ location.host
									+ '/ibm/console/xLaunch.do?pageID=com.ibm.usmi.console.scheduler.navigationElement&moduleRef=com.ibm.director.core.console.tasks&hideTaskArea=false&disableNavArea=false&hideBannerArea=false&hideTaskBarArea=false';
							url = url + '&DCMJobId=' + this.objectId;
							this._logger.finest("external launch url is:" + url, "onClick");
							menuItemOnClickFunc(null, null, url);
						}
					});
		});

},
'ibm/tivoli/navmodel/Dataset':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/AbstractDataset"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Dataset"); 
 
// includes
dojo.require("ibm.tivoli.navmodel.AbstractDataset");

ibm.tivoli.navmodel.Dataset._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.Dataset");

////////////////////////////////////
// Dataset
//
// NOTE: do not call any abstract methods directly;
// call the public version of the methods defined in
// AbstractDataset instead and it will in turn call
// the abstract overrides.
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Dataset", ibm.tivoli.navmodel.AbstractDataset, {
	// private members
	_optimize: true,
	_trace: ibm.tivoli.navmodel.Dataset._trace, 
    _oneTimeSetup: false,
    _createEventRouterFlag: false,
    
	
	/*=====
	constructor_onComplete: function(dataset, kwArgs){},
	=====*/
	
	/*=====
	constructor_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	constructor_onPage: function(start, count, ioArgs){},
	=====*/
	
	/*=====
	 constructor_kwArgs: function(onComplete, onError, onPage, baseUrl, provider, datasource, dataset, 
	 					parameters, defaultProperties, defaultLinkProperties, defaultRelProperties, 
	 					cacheThreshold, cacheValidationInterval, cacheExpirationTime, version, eventRouter) {
		// onComplete: constructor_onComplete
	 	//		callback to be invoked when the Dataset is initialized and ready for methods
	 	//		to be invoked  (i.e. asynchronous initialization must be performed). 
	 	//		Note: A null value will result in synchronous initialization.
	 	// onError: constructor_onError
	 	//		callback to be invoked if there is a problem constructing the Dataset
	 	// onPage:  constructor_onPage
	 	//		callback to be invoked when a new page is requested
		// baseUrl: String
		//		base url for identifying rest service
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// dataset: String
		//		dataset id
		// parameters: Object
		//		optional: extra parameters sent to dataset provider
		// defaultProperties: String[]
		//		optional: item property ids to fetch by default (can specify "all")
		// defaultLinkProperties: String[]
		//		optional: link property ids to fetch by default (can specify "all")
		// defaultRelProperties: String[]
		//		optional: relationship property ids to fetch by default (can specify "all")
		// cacheThreshold: int[]
		//		optional: cache objects if number of dataset items >= threshold
		// cacheValidationInterval: int[]
		//		optional: number of minutes in between cache validations
		// cacheExpirationTime: int[]
		//		optional: number of minutes objects can stay in cache while inactive
		// version: Float
		//		optional: version used to communicate with rest service (i.e. json version)
		// eventRouter: Object
		//		optional: object contains the following values (defaults shown)
		//		eventRouter: {
		//			timeout : 20,    		* number of seconds to wait for events  (optional)
		//			pollDelay : 0,			* number of ms to wait after an error before next poll  (optional)
		//			pollErrorDelay : 5000	* number of ms to wait after an error before next poll  (optional)
	   	//		},	   	
		//		Use With Caution:  If you are the first Dataset to create an EventRouter object, the parameters 
		// 		specified here will dictate timeout, pollDelay and pollErrorDelay for ALL Datasets in the browser. 
  		//		If a Dataset has already been constructed, these parameters will have NO effect.  If the Dataset 
  		//		is destroyed, the event router will shutdown and new Dataset instances and their settings will be used.	
	},
	=====*/

	constructor: function(/*constructor_kwArgs_kwArgs*/ kwArgs, dataset) {	
		// summary: 
		//		Constructs Dataset object.
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception		
	},

	//////////////////////////////////////////////////////
	// abstract method overrides
	//
	// NOTE: do not call any abstract methods directly;
	// call the public version of the methods defined in
	// AbstractDataset instead and it will in turn call
	// the abstract overrides.
	//////////////////////////////////////////////////////

	/*=====
	initDataset_kwArgs: function(createEventRouter, optimize) {
		// createEventRouter: Boolean
		//		boolean indicating whether new event router should be created
		// optimize: Boolean
		//		optimize json that is returned; remove items that can be calculated
		//		or can be retrieved via additional urls
	},
	=====*/
	
	_initDataset: function(/*initDataset_kwArgs*/ kwArgs) {
		// summary:
		//		Initialize internal dataset object
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		
		// start out with empty dataset
		this._initEmptyDataset();		
		this._optimize = kwArgs.optimize;
		
		// make synchronous call if no onComplete callback
		var synchronous = (kwArgs.onComplete ? false : true);
		var args = arguments;
		var exception = null;
		
		// dataset was NOT provided; fetch it
		if (!kwArgs.dataset) {			
			var content = ibm.tivoli.navmodel.Utils.getContent();
			this._mixinStandard(content, kwArgs);			
			ibm.tivoli.navmodel.Utils.mixinParams(content, this);
			
            if (kwArgs.fetchMetaData) {
                dojo.xhrGet({
                    url: this._getDatasetURL(),
                    headers: {
                        'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "dataset")
                    },
                    handleAs: "json",
                    sync: synchronous,
                    content: content,
                    load: dojo.hitch(this, function(result, ioArgs) {
                        // pull version from the Content-Type response header
                        this._setVersion(this._extractContentTypeVersion(ioArgs));
                        
                        // mixin all dataset attributes
                        this.mixinAttributes(result);
					
                        // listen for asynchronous events
                        if (kwArgs.createEventRouter && this.isEventsEnabled()) {
                            this._createEventRouter();
                        }
		
                        // call base class
                        this.inherited(args);
					
                        // call onComplete callback
                        if (kwArgs.onComplete) {
                            kwArgs.onComplete(this, kwArgs);
                        }							
                    }),
                    error: dojo.hitch(this, function(error, ioArgs){
                        exception = this._getException(error, ioArgs, "_initDataset");					
                        if (kwArgs.onError) {
                            kwArgs.onError(error, exception, ioArgs);
                        }						
                    })
                });	
            } else {
                this._createEventRouterFlag = kwArgs.createEventRouter;
				// call onComplete callback
				if (kwArgs.onComplete) {
                    // to support do not fetch Meta Data
					this._dataset.itemCount = -1;
					kwArgs.onComplete(this, kwArgs);
				} else {
                    console.log ("onComplete does not exist: " + kwArgs.onComplete);
                }
            }
			
			if (synchronous && exception) {
				throw exception;
			}						
		}
		
		// dataset was provided
		else {
			this.mixinAttributes(kwArgs.dataset);
			
			// listen for asynchronous events
			if (kwArgs.createEventRouter && this.isEventsEnabled()) {
				this._createEventRouter();
			}
		
			// call base class
			this.inherited(args);			
		}
	},
	
	_createEventRouter: function() {
		// summary:
		//		create event router object and set on the base class
		var baseUrl = this.getArg("baseUrl");
		var contextRoot = baseUrl.substring(0, baseUrl.lastIndexOf("/"));
		var eventRouterArgs = { 
			eventDestination: this.getAttribute("eventDestination"),
			contextRoot: contextRoot
		};		
		var evArgs = this.getArg("eventRouter");
		if (evArgs) {
			dojo.mixin(eventRouterArgs, evArgs);
		}
				
		var eventRouter = new ibm.tivoli.navmodel.EventRouter(eventRouterArgs);
			
		this._setEventRouter(eventRouter);		
	},

	_deleteDataset: function() {
		// summary:
		//		Dataset is no longer in use
		if (this._trace.isFinest()) {
			this._trace.finest("sending DELETE for dataset=" + this.getId(), "_deleteDataset");
		}
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		
		return dojo.xhrDelete({
			url: this._getDatasetURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion())
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){			
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				this._getException(error, ioArgs, "_deleteDataset");
			})
		});
	},
	
	
	_doKeepAlive: function(/*fetchVersion_kwArgs*/ kwArgs) {        // 255243
		// summary:
		//		Query the dataset attributes again to keep the connection with the provider alive
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception

      if (this._trace.isFine()) {
          this._trace.fine("sending keep-alive for remote dataset=" + this.getId(), "_doKeepAlive");
      }
		var exception = null;

		// need original "param_" values to locate cached dataset	
      var content = ibm.tivoli.navmodel.Utils.getContent();
      this._mixinStandard(content, kwArgs);			
      ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		
		dojo.xhrGet({
			url: this._getDatasetURL(),
			headers: {
				'Accept': "application/json"
			},
			handleAs: "json",
			sync: false,
         content: content,
			load: dojo.hitch(this, function(result, ioArgs){
                  // nothing to do
                  if (this._trace.isFine()) {
                      this._trace.fine("...keep-alive request returned for remote dataset=" + this.getId(), "_doKeepAlive");
                  }
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				exception = this._getException(error, ioArgs, "_doKeepAlive");
			})
		});
		
		if (exception) {
			throw exception;
		}
	},
	

	/*=====
	fetchItemsAndLinks_onComplete: function(items, links, datasetItems, toolbar, kwArgs){},
	=====*/
	
	/*=====
	_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	fetchItemsAndLinks_kwArgs: function(onComplete, onError, sync) {
		// onComplete: fetchItemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchItemsAndLinks: function(/*fetchItemsAndLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Returns both the items and links associated with the dataset
		var defaultProperties = this.getArg("defaultProperties");
		var defaultLinkProperties = this.getArg("defaultLinkProperties");
		var defaultRelProperties = this.getArg("defaultRelProperties");
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "itemProperties", defaultProperties);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "linkProperties", defaultLinkProperties);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "relationshipProperties", defaultRelProperties);		
		content.optimize = this._optimize;
		content.items = true;
		content.links = true;
		content.toolbarTasks = true;
		
		return dojo.xhrGet({
			url: this._getDatasetURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "dataset")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var expiration = this.getNewExpirationTime();
				var items = [];				
				var links = [];
				var toolbar = [];
				var styles = null;
				var status = null;
				var length;
				var index;
				var version = this.getVersion();
				
				this._setItemCount(result.itemCount);
				this._setLinkCount(result.linkCount);
				this._setRelationshipCount(result.relationshipCount);
										
				if (result.itemStyles) {
					styles = [];
					length = result.itemStyles.length;
					for (index = 0;  index < length;  ++index) {
						styles[index] = new ibm.tivoli.navmodel.ItemStyle({style: result.itemStyles[index], version: version});
					}
					this._cacheItemStyles(styles);
				}
				
				if (result.itemStatus) {
					status = [];
					length = result.itemStatus.length;
					for (index = 0;  index < length;  ++index) {
						status[index] = new ibm.tivoli.navmodel.Status({status: result.itemStatus[index], version: version});
					}
					this._cacheItemStatuses(status);
				}
				
				if (result.items) {
					var allProps = this._isAllProps(defaultProperties);					
					var propIds = this._getPropIds(defaultProperties, allProps);
					length = result.items.length;
					for (index = 0;  index < length;  ++index) {
						items[index] = new ibm.tivoli.navmodel.Item({item: result.items[index], dataset: this, propIds: propIds, allProperties: allProps, expiration: expiration});
					}
				}
				
				if (result.linkStyles) {
					styles = [];
					length = result.linkStyles.length;
					for (index = 0;  index < length;  ++index) {
						styles[index] = new ibm.tivoli.navmodel.LinkStyle({style: result.linkStyles[index], version: version});
					}
					this._cacheLinkStyles(styles);
				}
				
				if (result.relationshipStyles) {
					styles = [];
					length = result.relationshipStyles.length;
					for (index = 0;  index < length;  ++index) {
						styles[index] = new ibm.tivoli.navmodel.LinkStyle({style: result.relationshipStyles[index], version: version});
					}
					this._cacheRelationshipStyles(styles);
				}
				
				if (result.linkStatus) {
					status = [];
					length = result.linkStatus.length;
					for (index = 0;  index < length;  ++index) {
						status[index] = new ibm.tivoli.navmodel.Status({status: result.linkStatus[index], version: version});
					}
					this._cacheLinkStatuses(status);
				}
				
				if (result.relationshipStatus) {
					status = [];
					length = result.relationshipStatus.length;
					for (index = 0;  index < length;  ++index) {
						status[index] = new ibm.tivoli.navmodel.Status({status: result.relationshipStatus[index], version: version});
					}
					this._cacheRelationshipStatuses(status);
				}
				
				if (result.links) {
					var allLinkProps = this._isAllProps(defaultLinkProperties);
					var linkPropIds = this._getPropIds(defaultLinkProperties, allLinkProps);
					length = result.links.length;
					for (index = 0;  index < length;  ++index) {
						links[index] = new ibm.tivoli.navmodel.Link({link: result.links[index], dataset: this, propIds: linkPropIds, allProperties: allLinkProps, expiration: expiration});
					}
				}
				
				if (result.toolbarTasks) {
					length = result.toolbarTasks.length;
					for (index = 0;  index < length;  ++index) {
						toolbar[index] = new ibm.tivoli.navmodel.Task({task: result.toolbarTasks[index], dataset: this});
					}
				}
				
				kwArgs.onComplete(items, links, toolbar, kwArgs);				
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchItemsAndLinks"), ioArgs);
			})
		});
	},

	/*=====
	items_onComplete: function(items, kwArgs){},
	=====*/

	/*=====
	fetchItems_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error conditions
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// itemIds: String
		//		comma delimited list of item ids to fetch						
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	_fetchItems: function(/*fetchItems_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch items from rest service
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		
		// 24658
		// Check to see if a list of itemIds was supplied
		if (kwArgs.itemIds) {
			content.itemIds = kwArgs.itemIds;
		}
		
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);

		console.log ("url: " + this._getItemsURL());
		
		return dojo.xhrGet({
			url: this._getItemsURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemViewList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				console.log ("in Dataset._fetchItems.load");
                // Begin JSA Change for no Meta Data
				if (this._oneTimeSetup == false) {
				  this._oneTimeSetup = true;
				  // pull version from the Content-Type response header
				  this._setVersion(this._extractContentTypeVersion(ioArgs));
				
				  // listen for asynchronous events
				  if (this._createEventRouterFlag && this.isEventsEnabled()) {
					  this._createEventRouter();
				  }
				}            
                // End JSA Change for no Meta Data
            
				var items = [];				
				if (result.items) {
					var expiration = this.getNewExpirationTime();
					var allProps = this._isAllProps(kwArgs.properties);
					var propIds = this._getPropIds(kwArgs.properties, allProps);
					var itemsLength = result.items.length;
					for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
						var item = result.items[itemIndex];
						if (item.datasetId && item.datasetId !== this.getId()) {
							var dataset = this.getDescendentDataset(item.datasetId);
							items[itemIndex] = new ibm.tivoli.navmodel.Item({item: item, dataset: dataset, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});							
						}
						else {
							items[itemIndex] = new ibm.tivoli.navmodel.Item({item: item, dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});
						}
					}
				}
				
				// make sure our count is accurate
				this._setItemCount(result.totalRows);
					
				// place counts in the arguments map
				kwArgs.numRows = result.numRows;
				kwArgs.filteredRows = result.filteredRows;
				kwArgs.totalRows = result.totalRows;
				kwArgs.onComplete(items, kwArgs);	
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchItems"), ioArgs);
			})
		});
	},
	
	/*=====
	fetchItem_kwArgs: function(itemId, properties, metadata) {
		// itemId: String
		//		item id to get
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
	},
	=====*/
	
	_fetchItem: function(/*fetchItem_kwArg*/ kwArgs) {
		// summary:
		//		Synchronously fetch an individual item from the rest service
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var item = null;
		var exception = null;
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		
		dojo.xhrGet({
			url: this._getItemURL(kwArgs.itemId),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemView")
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var allProps = this._isAllProps(kwArgs.properties);
				var propIds = this._getPropIds(kwArgs.properties, allProps);
				item = new ibm.tivoli.navmodel.Item({item: result, dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata});											
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				exception = this._getException(error, ioArgs, "_fetchItem");
			})
		});
		
		if (exception) {
			throw exception;
		}
				
		return item;
	},

	/*=====
	fetchOriginItems_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metdata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchOriginItems: function(/*fetchOriginItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns orign items of the dataset
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		
		return dojo.xhrGet({
			url: this._getOriginItemsURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "originItemViewList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var expiration = this.getNewExpirationTime();				
				var items = [];
				if (result.items) {
					var allProps = this._isAllProps(kwArgs.properties);
					var propIds = this._getPropIds(kwArgs.properties, allProps);
					var itemsLength = result.items.length;
					for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
						items[itemIndex] = new ibm.tivoli.navmodel.Item({item: result.items[itemIndex], dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});
					}
				}
				
				// place counts in the arguments map
				kwArgs.numRows = result.numRows;
				kwArgs.filteredRows = result.filteredRows;
				kwArgs.totalRows = result.totalRows;
				
				kwArgs.onComplete(items, kwArgs);				
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchOriginItems"), ioArgs);
			})
		});
	},

	/*=====
	itemsAndLinks_onComplete: function(items, links, datasetItems, kwArgs){},
	=====*/
	
	/*=====
	fetchRelatedItems_kwArgs: function(onComplete, onError, itemId, relationships, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: itemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// itemId: String
		//		id of the base item
		// relationships: String
		//		comma delimited list of relationships to determine related items; 
		//		can also be keywords: child, parent, all
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchRelatedItems: function(/*fetchRelatedItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items that are related to the requested item
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		
		// relationships to explore
		if (kwArgs.relationships) {
			content.relationships = kwArgs.relationships;
		}
		
		return dojo.xhrGet({
			url: this._getRelatedItemsURL(kwArgs.itemId),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "relatedItemViewList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var expiration = this.getNewExpirationTime();
				var items = [];
				if (result.items) {
					if (result.datasetItemCount !== undefined) {
						this._setItemCount(result.datasetItemCount);
					}
					
					var allProps = this._isAllProps(kwArgs.properties);
					var propIds = this._getPropIds(kwArgs.properties, allProps);
					var itemsLength = result.items.length;
					for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
						items[itemIndex] = new ibm.tivoli.navmodel.Item({item: result.items[itemIndex], dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});							
					}
				}
				
				var links = [];
				if (result.links) {
					if (result.datasetLinkCount !== undefined) {
						this._setLinkCount(result.datasetLinkCount);
					}
					
					if (result.datasetRelationshipCount !== undefined) {
						this._setRelationshipCount(result.datasetRelationshipCount);
					}					
					
					var linksLength = result.links.length;
					for (var linkIndex = 0;  linkIndex < linksLength;  ++linkIndex) {
						links[linkIndex] = new ibm.tivoli.navmodel.Link({link: result.links[linkIndex], dataset: this, expiration: expiration});
					}
				}
				
				// place counts in the arguments map
				kwArgs.numRows = result.numRows;
				kwArgs.filteredRows = result.filteredRows;
				kwArgs.totalRows = result.totalRows;
				
				kwArgs.onComplete(items, links, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchRelatedItems"), ioArgs);
			})
		});
	},
	
	/*=====
	queryItems_kwArgs: function(onComplete, onError, query, start, count, properties, metadata, sync) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// query: Object
		//		custom query object to be sent as request body
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	 _queryItems: function(/*queryItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items associated with the dataset that match
		//		the provided custom query
		var json = dojo.toJson(kwArgs.query, false);
				
		return dojo.xhrPost({
			url: this._getItemsQueryURL(kwArgs),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemViewList"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "customQuery"),
				'Content-Length': json.length,
				'X-Method-Override': 'Custom-GET'
			},
			handleAs: "json",
			postData: json,
			sync: kwArgs.sync,
			load: dojo.hitch(this, function(result, ioArgs){
				var expiration = this.getNewExpirationTime();
				var items = [];				
				if (result.items) {
					var allProps = this._isAllProps(kwArgs.properties);				
					var propIds = this._getPropIds(kwArgs.properties, allProps);
					var itemsLength = result.items.length;
					for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
						items[itemIndex] = new ibm.tivoli.navmodel.Item({item: result.items[itemIndex], dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});
					}
				}
				
				kwArgs.onComplete(items, kwArgs);	
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_queryItems"), ioArgs);
			})
		});
	},
	
	/*=====
	links_onComplete: function(items, kwArgs){},
	=====*/

	/*=====
	fetchLinks_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: links_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error conditions
		// start: int
		//		start index to fetch
		// count: int
		//		number of links to fetch; null means all
		// linkIds: String
		//		comma delimited list of item ids to fetch						
		// properties: String[]
		//		link property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchLinks: function(/*fetchLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch links from rest service
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		
		// 24658
		// Check to see if a list of linkIds was supplied
		if (kwArgs.linkIds) {
			content.linkIds = kwArgs.linkIds;
		}
		
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		
		return dojo.xhrGet({
			url: this._getLinksURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "linkViewList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var expiration = this.getNewExpirationTime();
				var links = [];							
				if (result.items) {
					var allProps = this._isAllProps(kwArgs.properties);
					var propIds = this._getPropIds(kwArgs.properties, allProps);
					var linksLength = result.items.length;
					for (var linkIndex = 0;  linkIndex < linksLength;  ++linkIndex) {
						var link = result.items[linkIndex];
						if (link.datasetId && link.datasetId !== this.getId()) {
							var dataset = this.getDescendentDataset(link.datasetId);
							links[linkIndex] = new ibm.tivoli.navmodel.Link({link: link, dataset: dataset, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});
						}
						else {									
							links[linkIndex] = new ibm.tivoli.navmodel.Link({link: link, dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});
						}
					}
				}
				
				// make sure our count is accurate
				this._setLinkCount(result.totalRows);
					
				// place counts in the arguments map
				kwArgs.numRows = result.numRows;
				kwArgs.filteredRows = result.filteredRows;
				kwArgs.totalRows = result.totalRows;				
				kwArgs.onComplete(links, kwArgs);	
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchLinks"), ioArgs);
			})
		});
	},
	
	/*=====
	fetchLink_kwArgs: function(itemId, properties, metadata) {
		// linkId: String
		//		link id to get
		// properties: String[]
		//		array of property ids to pre-fetch
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
	},
	=====*/
	
	_fetchLink: function(/*fetchLink_kwArgs*/ kwArgs) {
		// summary:
		//		Synchronously fetch an individual link from the rest service
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var link = null;
		var exception = null;
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		
		dojo.xhrGet({
			url: this._getLinkURL(kwArgs.linkId),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "linkView")
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var allProps = this._isAllProps(kwArgs.properties);
				var propIds = this._getPropIds(kwArgs.properties, allProps);
				link = new ibm.tivoli.navmodel.Link({link: result, dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata});			
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				exception = this._getException(error, ioArgs, "_fetchLink");
			})
		});
		
		if (exception) {
			throw exception;
		}
		
		return link;
	},

	/*=====
	relationships_onComplete: function(relationships, kwArgs){},
	=====*/
	
	/*=====
	fetchRelationships_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: relationships_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error conditions
		// start: int
		//		start index to fetch
		// count: int
		//		number of relationships to fetch; null means all
		// relationshipIds: String
		//		comma delimited list of relationship ids to fetch								
		// properties: String[]
		//		relationship property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchRelationships: function(/*fetchRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch relationships from rest service
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		
		// 24658
		// Check to see if a list of relationshipIds was supplied
		if (kwArgs.relationshipIds) {
			content.relationshipIds = kwArgs.relationshipIds;
		}
		
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		
		return dojo.xhrGet({
			url: this._getRelationshipsURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "relationshipViewList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var expiration = this.getNewExpirationTime();
				var relationships = [];				
				if (result.items) {
					var allProps = this._isAllProps(kwArgs.properties);
					var propIds = this._getPropIds(kwArgs.properties, allProps);
					var relLength = result.items.length;
					for (var relIndex = 0;  relIndex < relLength;  ++relIndex) {
						var relationship = result.items[relIndex];
						if (relationship.datasetId && relationship.datasetId !== this.getId()) {
							var dataset = this.getDescendentDataset(relationship.datasetId);
							relationships[relIndex] = new ibm.tivoli.navmodel.Relationship({relationship: relationship, dataset: dataset, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});
						}
						else {									
							relationships[relIndex] = new ibm.tivoli.navmodel.Relationship({relationship: relationship, dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});
						}						
					}
				}
								
				// make sure our count is accurate
				this._setRelationshipCount(result.totalRows);
					
				// place counts in the arguments map
				kwArgs.numRows = result.numRows;
				kwArgs.filteredRows = result.filteredRows;
				kwArgs.totalRows = result.totalRows;				
				kwArgs.onComplete(relationships, kwArgs);	
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchRelationships"), ioArgs);
			})
		});
	},

	/*=====
	fetchRelationship_kwArgs: function(relationshipId, properties, metadata) {
		// relationshipId: String
		//		relationship id to fetch
		// properties: String[]
		//		array of property ids to pre-fetch
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
	},
	=====*/
	
	_fetchRelationship: function(/*fetchRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Synchronously fetch an individual relationship from the rest service
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var relationship = null;
		var exception = null;
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		
		dojo.xhrGet({
			url: this._getRelationshipURL(kwArgs.relationshipId),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "relationshipView")
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var allProps = this._isAllProps(kwArgs.properties);
				var propIds = this._getPropIds(kwArgs.properties, allProps);
				relationship = new ibm.tivoli.navmodel.Relationship({relationship: result, dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata});			
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				exception = this._getException(error, ioArgs, "_fetchRelationship");
			})
		});
		
		if (exception) {
			throw exception;
		}
		
		return relationship;
	},
	
	/*=====
	itemStyles_onComplete: function(itemStyles, kwArgs){},
	=====*/
	
	/*=====
	fetchItemStyles_kwArgs: function(onComplete, onError, start, count, sync) {
		// onComplete: itemStyle_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of item styles to fetch;  null means all
		// sync: Boolean
		//		indicates if fetch is synchronous		
	},
	=====*/
	
	_fetchItemStyles: function(/*fetchItemStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current item styles for the Dataset
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		
		return dojo.xhrGet({
			url: this._getItemStylesURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemStyleList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var styles = [];
				var stylesLength = result.items.length;
				for (var index = 0;  index < stylesLength;  ++index) {
					styles[index] = new ibm.tivoli.navmodel.ItemStyle({style: result.items[index], version: this.getVersion()});
				}
				
				kwArgs.onComplete(styles, kwArgs);				
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchItemStyles"), ioArgs);
			})
		});
	},

	/*=====
	linkStyles_onComplete: function(linkStyles, kwArgs){},
	=====*/
	
	/*=====
	fetchLinkStyles_kwArgs: function(onComplete, onError, start, count, sync) {
		// onComplete: linkStyles_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of link styles to fetch; null means all
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchLinkStyles: function(/*fetchLinkStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current link styles for the Dataset
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		
		return dojo.xhrGet({
			url: this._getLinkStylesURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "linkStyleList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var styles = [];
				var stylesLength = result.items.length;
				for (var index = 0;  index < stylesLength;  ++index) {
					styles[index] = new ibm.tivoli.navmodel.LinkStyle({style: result.items[index], version: this.getVersion()});
				}
				
				kwArgs.onComplete(styles, kwArgs);				
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchLinkStyles"), ioArgs);
			})
		});		
	},

	/*=====
	relStyles_onComplete: function(relStyles, kwArgs){},
	=====*/
	
	/*=====
	fetchRelationshipStyles_kwArgs: function(onComplete, onError, start, count, sync) {
		// onComplete: relStyles_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of relationship styles to fetch; null means all
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchRelationshipStyles: function(/*fetchRelationshipStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current relationship styles for the Dataset
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		
		return dojo.xhrGet({
			url: this._getRelationshipStylesURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "linkStyleList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var styles = [];
				var stylesLength = result.items.length;
				for (var index = 0;  index < stylesLength;  ++index) {
					styles[index] = new ibm.tivoli.navmodel.LinkStyle({style: result.items[index], version: this.getVersion()});
				}
				
				kwArgs.onComplete(styles, kwArgs);				
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchRelationshipStyles"), ioArgs);
			})
		});		
	},

	/*=====
	itemStatuses_onComplete: function(itemStatuses, kwArgs){},
	=====*/
	
	/*=====
	fetchStatuses_kwArgs: function(onComplete, onError, type, start, count, sync) {
		// onComplete: itemStatuses_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// type: String
		//		object type for columns: "item", "link", "relationship"
		// start: int
		//		start index to fetch
		// count: int
		//		number of status values to fetch; null means all
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	_fetchStatuses: function(/*fetchStatuses_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of possible item status values for the Dataset
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);		
		if (kwArgs.type) {
			content.type = kwArgs.type;
		}
		
		return dojo.xhrGet({
			url: this._getStatusURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "statusList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var status = [];
				var statusLength = result.items.length;
				for (var index = 0;  index < statusLength;  ++index) {
					status[index] = new ibm.tivoli.navmodel.Status({status: result.items[index], version: this.getVersion()});
				}
				
				kwArgs.onComplete(status, kwArgs);				
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchStatuses"), ioArgs);
			})
		});
	},

	/*=====
	tasks_onComplete: function(tasks, kwArgs){},
	=====*/
	
	/*=====
	fetchTasks_kwArgs: function(onComplete, onError, taskCategory, selection, itemIds, linkIds, itemIndexes, sync) {
		// onComplete: tasks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// taskCategory: String
		//		comma delimited list indicating type of tasks to fetch
		//		legal values are: menu, toolbar, and breadcrumb
		// selection: Object
		//		ibm.tivoli.navmodel.Selection object
		//		if selection is specified: itemIds, linkIds, itemIndexes is ignored
		// itemIds: String
		//		comma delimited list of item ids that are selected
		// linkIds, String
		//		comma delimited list of link ids that are selected
		// itemIndexes: String
		//		comma delimited list of item ids or ranges that are selected
		//		ex: "0,1,10,20" or "0-10,30,40-50"
		// sync: Boolean
		//		indicates if fetch is synchronous
		// failOK: Boolean
		//      indicates if errors should not be logged to the console by the request object
		//      (but onError is still called, if specified).  See dojo\_base\xhr.js		
	},
	=====*/
	
	_fetchTasks: function(/*fetchTasks_kwArgs*/ kwArgs) {
		// summary:
		//		Returns tasks associated with the dataset
		
		// use "selection"
		if (kwArgs.selection) {
			var json = dojo.toJson(kwArgs.selection.getAttributes(), false);
			
			return dojo.xhrPost ({
				url: this._getTasksParamURL(kwArgs),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "taskList"),
					'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "taskContext"),
					'Content-Length': json.length
				},
				handleAs: "json",
				postData: json,
				sync: kwArgs.sync,
				failOK: kwArgs.failOK,   // 37703
				load: dojo.hitch(this, function(result, ioArgs) {
					var tasks = [];
					var rawTasks = result.items;
					var rawTasksLength = rawTasks.length;
					for (var index = 0;  index < rawTasksLength;  ++index) {
						tasks[index] = new ibm.tivoli.navmodel.Task({task: rawTasks[index], dataset: this});
					}
					kwArgs.onComplete(tasks, kwArgs);
				}),
				error: dojo.hitch(this, function(error, ioArgs) {
					kwArgs.onError(error, this._getException(error, ioArgs, "getTasks"), ioArgs);	               
				})
			});
		}
		
		// use "itemIds", "linkIds", and "itemIndexes"
		else {
			// need original "param_" values to locate cached dataset
			var content = ibm.tivoli.navmodel.Utils.getContent();
			this._mixinStandard(content, kwArgs);
			ibm.tivoli.navmodel.Utils.mixinParams(content, this);
			
			// comma delimitted categories (i.e. "menu,toolbar,breadcrumb")
			content.category = kwArgs.taskCategory;

			// indicate what is selected
			if (kwArgs.itemIds) {
				content.itemIds = kwArgs.itemIds;
			}
			if (kwArgs.linkIds) {
				content.linkIds = kwArgs.linkIds;
			}
			if (kwArgs.itemIndexes) {
				content.itemIndexes = kwArgs.itemIndexes;
			}

			return dojo.xhrGet({
				url: this._getTasksURL(),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "taskList")
				},
				handleAs: "json",
				sync: kwArgs.sync,
				failOK: kwArgs.failOK,   // 37703
				content: content,
				load: dojo.hitch(this, function(result, ioArgs){
					var tasks = [];
					var rawTasks = result.items;
					var rawTasksLength = rawTasks.length;
					for (var index = 0;  index < rawTasksLength;  ++index) {
						tasks[index] = new ibm.tivoli.navmodel.Task({task: rawTasks[index], dataset: this});
					}
					kwArgs.onComplete(tasks, kwArgs);
				}),
				error: dojo.hitch(this, function(error, ioArgs){
					kwArgs.onError(error, this._getException(error, ioArgs, "getTasks()"), ioArgs);
				})
			});
		}
	},

	/*=====
	dataset_onComplete: function(dataset, kwArgs){},
	=====*/
	
	/*=====
	navigate_kwArgs: function(onComplete, onError, taskId, taskCategory, selection, sync) {
		// onComplete: dataset_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// taskId: String
		//		navigation task id
		// taskCategory: String
		//		navigation task category
		// selection: Object
		//		ibm.tivoli.navmodel.Selection object
		//		if selection is specified: itemIds, linkIds, itemIndexes is ignored
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_navigate: function(/*navigate_kwArgs*/ kwArgs) {
		// summary:
		//		Navigate to another Dataset from a navigation task
				
		// use "selection"
		var json = dojo.toJson(kwArgs.selection.getAttributes(), false);
			
		return dojo.xhrPost ({
			url: this._getNavigateParamURL(kwArgs),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "dataset"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "taskContext"),
				'Content-Length': json.length
			},
			handleAs: "json",
			postData: json,
			sync: kwArgs.sync,			
			load: dojo.hitch(this, function(result, ioArgs) {
				var navigateArgs = this.getArgs();
				navigateArgs.dataset = result.id;
				navigateArgs.parameters = {refId: ""};  // CURI compatability: include empty refId so we don't create one
				navigateArgs.defaultProperties = [];
				navigateArgs.defaultLinkProperties = [];
				navigateArgs.defaultRelProperties = [];
				navigateArgs.createEventRouter = false; // will share EventRouter
				navigateArgs.onComplete = null;
				navigateArgs.onError = null;
						
		  		// construct navigation dataset (share EventRouter)
				var dataset = new ibm.tivoli.navmodel.Dataset(navigateArgs, result);
				dataset._setEventRouter(this._getEventRouter());
			
				// pass new dataset back to caller
				kwArgs.onComplete(dataset, kwArgs);		
			}),
			error: dojo.hitch(this, function(error, ioArgs) {
				kwArgs.onError(error, this._getException(error, ioArgs, "navigate"), ioArgs);	               
			})
		});			      
	},

	/*=====
	cols_onComplete: function(cols, kwArgs){},
	=====*/
	
	/*=====
	fetchColumns_kwArgs: function(onComplete, onError, type, properties, start, count, metadata, sync) {
		// onComplete: cols_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// type: String
		//		object type: "item", "link", "relationship" 
		// properties: String[]
		//		array of column ids; or keyword "current", "default", or "all"
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// metadata: Boolean
		//		indicates if filter metadata should be returned with columns		
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchColumns: function(/*fetchColumns_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch columns from rest service
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);		
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		if (kwArgs.source) {
			content.source = kwArgs.source;
		}
		if (kwArgs.type) {
			content.type = kwArgs.type;
		}
		if (kwArgs.properties) {
			content.properties = kwArgs.properties.join(",");
		}		

		return dojo.xhrGet({
			url: this._getColumnsURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "columnList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var cols = [];
				var colsLength = result.items.length;
				for (var index = 0;  index < colsLength;  ++index) {
					cols[index] = new ibm.tivoli.navmodel.Column({column: result.items[index], objectType: kwArgs.type, dataset: this});
				}
				
				kwArgs.onComplete(cols, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchColumns"), ioArgs);
			})
		});
	},

	/*=====
	setColumns_kwArgs: function(onComplete, onError, columns, type, sync) {
		// onComplete: cols_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// columns: ibm.tivoli.navmodel.Column[]
		//		array of columns to be saved
		// type: String
		//		object type: "item", "link", "relationship" 
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_setColumns: function(/*setColumns_kwArgs*/ kwArgs) {
		// summary:
		//		Save columns to rest service
		
		// pull out raw javascript column data
		var columns = [];
		if (kwArgs.columns) {
			var count = kwArgs.columns.length;
			for (var index = 0;  index < count;  ++index) {
				var col = kwArgs.columns[index].getAttributes();
				delete col.filter;
				columns.push(col);
			}
		}		
		var json = dojo.toJson({columns: columns}, false);
		
		// need original "param_" values to locate cached dataset
		return dojo.xhrPost({
			url: this._getColumnsParamURL(kwArgs),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion()),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "columnList"),
				'Content-Length': json.length,
				'X-Method-Override': 'Multi-PUT'
			},
			handleAs: "json",
			postData: json,
			sync: kwArgs.sync,
			load: dojo.hitch(this, function(result, ioArgs){
				kwArgs.onComplete(kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_setColumns"), ioArgs);
			})
		});
	},

	/*=====
	item_onComplete: function(item, kwArgs){},
	=====*/
	
	/*=====
	createItem_kwArgs: function(onComplete, onError, item) {
		// onComplete: item_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// item: Object
		//		ibm.tivoli.navmodel.Item to be created on server
	},
	=====*/
	
	_createItem: function(/*createItem_kwArgs*/ kwArgs) {
		// summary:
		//			Asynchronously create a new item in the dataset	
		var item = kwArgs.item;
		var json = dojo.toJson(item.getAttributes(), false);
		
		return dojo.xhrPost({
			url: this._getItemParamURL(item.getId()),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemView"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemView"),
				'Content-Length': json.length
			},
			handleAs: "json",
			postData: json,
			sync: false,
			load: dojo.hitch(this, function(result, ioArgs){
				var newItem = new ibm.tivoli.navmodel.Item({item: result, dataset: this, allProperties: true, allMetadata: false});
				kwArgs.onComplete(newItem, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_createItem"), ioArgs);
			})
		});
	},

	/*=====
	createItems_kwArgs: function(onComplete, onError, item) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// item: Object[]
		//		array of ibm.tivoli.navmodel.Item to be created on server
	},
	=====*/
	
	_createItems: function(/*createItems_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new items in the dataset
		var itemsObj = {"items": []};
		if (kwArgs.items) {
			var itemsLength = kwArgs.items.length;
			for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
				itemsObj.items[itemIndex] = kwArgs.items[itemIndex].getAttributes();
			}
		}
	
		var json = dojo.toJson(itemsObj, false);
		
		return dojo.xhrPost({
			url: this._getItemsParamURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemViewList"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemViewList"),
				'Content-Length': json.length,
				'X-Method-Override': 'Multi-PUT'
			},
			handleAs: "json",
			postData: json,
			sync: false,
			load: dojo.hitch(this, function(result, ioArgs){
				var newItems = [];				
				if (result.items) {
					var itemsLength = result.items.length;
					for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
						newItems[itemIndex] = new ibm.tivoli.navmodel.Item({item: result.items[itemIndex], dataset: this, allProperties: true, allMetadata: false});
					}
				}
				kwArgs.onComplete(newItems, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_createItems"), ioArgs);
			})
		});
	},

	/*=====
	_onComplete: function(kwArgs){},
	=====*/
	
	/*=====
	deleteItem_kwArgs: function(onComplete, onError, itemId, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// itemId: String
		//		id of item to be deleted
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of item
	},
	=====*/
	
	_deleteItem: function(/*deleteItem_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a item from the dataset
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		if (kwArgs.remove) {
			content.remove = kwArgs.remove;
		}
		
		return dojo.xhrDelete({
			url: this._getItemURL(kwArgs.itemId),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion())
			},
			handleAs: "json",
			sync: false,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				kwArgs.onComplete(kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_deleteItem"), ioArgs);
			})
		});
	},
	
	/*=====
	deleteItems_kwArgs: function(onComplete, onError, itemIds, condition, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// itemIds: String[]
		//		array of item ids to be deleted
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of items
	},
	=====*/
	
	_deleteItems: function(/*deleteItems_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete items from the dataset
		
		// build up the delete filter
		var deleteFilter = {};
		if (kwArgs.itemIds && kwArgs.itemIds.length > 0) {
			deleteFilter.identifiers = kwArgs.itemIds;
		}
		if (kwArgs.condition) {
			deleteFilter.condition = kwArgs.condition;
		}
		if (kwArgs.remove) {
			deleteFilter.remove = kwArgs.remove;
		}
		
		var json = dojo.toJson(deleteFilter, false);
				
		return dojo.xhrPost({
			url: this._getItemsParamURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion()),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "deleteFilter"),
				'Content-Length': json.length,
				'X-Method-Override': 'Multi-DELETE'
			},
			handleAs: "json",
			postData: json,
			sync: false,			
			load: dojo.hitch(this, function(result, ioArgs){
				kwArgs.onComplete(kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_deleteItems"), ioArgs);
			})
		});
	},

	/*=====
	link_onComplete: function(link, kwArgs){},
	=====*/
	
	/*=====
	createLink_kwArgs: function(onComplete, onError, link) {
		// onComplete: link_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// link: Object
		//		ibm.tivoli.navmodel.Link to be created on server
	},
	=====*/
	
	_createLink: function(/*createLink_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new link in the dataset
		var link = kwArgs.link;
		var json = dojo.toJson(link.getAttributes(), false);
		
		return dojo.xhrPost({
			url: this._getLinkParamURL(link.getId()),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "linkView"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "linkView"),
				'Content-Length': json.length
			},
			handleAs: "json",
			postData: json,
			sync: false,
			load: dojo.hitch(this, function(result, ioArgs){
				var newLink = new ibm.tivoli.navmodel.Link({link: result, dataset: this, allProperties: true, allMetadata: false});
				kwArgs.onComplete(newLink, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_createLink"), ioArgs);
			})
		});
	},
	
	/*=====
	createLinks_kwArgs: function(onComplete, onError, links) {
		// onComplete: links_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// links: Object[]
		//		array of ibm.tivoli.navmodel.Link to be created on server
	},
	=====*/
	
	_createLinks: function(/*createLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new links in the dataset
		var linksObj = {"links": []};
		if (kwArgs.links) {
			var linksLength = kwArgs.links.length;
			for (var linkIndex = 0;  linkIndex < linksLength;  ++linkIndex) {
				linksObj.links[linkIndex] = kwArgs.links[linkIndex].getAttributes();
			}
		}
	
		var json = dojo.toJson(linksObj, false);
		
		return dojo.xhrPost({
			url: this._getLinksParamURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "linkViewList"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "linkViewList"),
				'Content-Length': json.length,
				'X-Method-Override': 'Multi-PUT'
			},
			handleAs: "json",
			postData: json,
			sync: false,
			load: dojo.hitch(this, function(result, ioArgs){
				var newLinks = [];				
				if (result.items) {
					var linksLength = result.items.length;
					for (var linkIndex = 0;  linkIndex < linksLength;  ++linkIndex) {
						newLinks[linkIndex] = new ibm.tivoli.navmodel.Link({link: result.items[linkIndex], dataset: this, allProperties: true, allMetadata: false});
					}
				}
				kwArgs.onComplete(newLinks, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_createLinks"), ioArgs);
			})
		});
	},

	/*=====
	deleteLink_kwArgs: function(onComplete, onError, linkId, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// linkId: String
		//		id of link to be deleted
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of link
	},
	=====*/
	
	_deleteLink: function(/*deleteLink_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a link from the dataset
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		if (kwArgs.remove) {
			content.remove = kwArgs.remove;
		}
		
		return dojo.xhrDelete({
			url: this._getLinkURL(kwArgs.linkId),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion())
			},
			handleAs: "json",
			sync: false,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				kwArgs.onComplete(kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_deleteLink"), ioArgs);
			})
		});
	},

	/*=====
	deleteLinks_kwArgs: function(onComplete, onError, linkIds, condition, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// linkIds: String[]
		//		array of link ids to be deleted
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of links
	},
	=====*/
	
	_deleteLinks: function(/*deleteLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete links from the dataset
		
		// build up the delete filter
		var deleteFilter = {};
		if (kwArgs.linkIds && kwArgs.linkIds.length > 0) {
			deleteFilter.identifiers = kwArgs.linkIds;
		}
		if (kwArgs.condition) {
			deleteFilter.condition = kwArgs.condition;
		}
		if (kwArgs.remove) {
			deleteFilter.remove = kwArgs.remove;
		}
		
		var json = dojo.toJson(deleteFilter, false);
				
		return dojo.xhrPost({
			url: this._getLinksParamURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion()),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "deleteFilter"),
				'Content-Length': json.length,
				'X-Method-Override': 'Multi-DELETE'
			},
			handleAs: "json",
			postData: json,
			sync: false,
			load: dojo.hitch(this, function(result, ioArgs){
				kwArgs.onComplete(kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_deleteLinks"), ioArgs);
			})
		});
	},
	
	/*=====
	relationship_onComplete: function(relationship, kwArgs){},
	=====*/
	
	/*=====
	createRelationship_kwArgs: function(onComplete, onError, relationship) {
		// onComplete: relationship_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationship: Object
		//		ibm.tivoli.navmodel.Relationship to be created on server
	},
	=====*/
	_createRelationship: function(/*createRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new relationship in the dataset
		var relationship = kwArgs.relationship;
		var json = dojo.toJson(relationship.getAttributes(), false);
		
		return dojo.xhrPost({
			url: this._getRelationshipParamURL(relationship.getId()),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "relationshipView"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "relationshipView"),
				'Content-Length': json.length
			},
			handleAs: "json",
			postData: json,
			sync: false,
			load: dojo.hitch(this, function(result, ioArgs){
				var newRelationship = new ibm.tivoli.navmodel.Relationship({relationship: result, dataset: this, allProperties: true, allMetadata: false});				
				kwArgs.onComplete(newRelationship, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_createRelationship"), ioArgs);
			})
		});
	},
	
	/*=====
	createRelationships_kwArgs: function(onComplete, onError, relationships) {
		// onComplete: relationships_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationship: Object[]
		//		array of ibm.tivoli.navmodel.Relationship to be created on server
	},
	=====*/
	
	_createRelationships: function(/*createRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new relationships in the dataset
		var relationshipsObj = {"relationships": []};
		if (kwArgs.relationships) {
			var relLength = kwArgs.relationships.length;
			for (var relIndex = 0;  relIndex < relLength;  ++relIndex) {
				relationshipsObj.relationships[relIndex] = kwArgs.relationships[relIndex].getAttributes();
			}
		}
	
		var json = dojo.toJson(relationshipsObj, false);
		
		return dojo.xhrPost({
			url: this._getRelationshipsParamURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "relationshipViewList"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "relationshipViewList"),
				'Content-Length': json.length,
				'X-Method-Override': 'Multi-PUT'
			},
			handleAs: "json",
			postData: json,
			sync: false,
			load: dojo.hitch(this, function(result, ioArgs){
				var newRelationships = [];				
				if (result.items) {
					var relLength = result.items.length;
					for (var relIndex = 0;  relIndex < relLength;  ++relIndex) {
						newRelationships[relIndex] = new ibm.tivoli.navmodel.Relationship({relationship: result.items[relIndex], dataset: this, allProperties: true, allMetadata: false});
					}
				}
				kwArgs.onComplete(newRelationships, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_createRelationships"), ioArgs);
			})
		});
	},

	/*=====
	deleteRelationship_kwArgs: function(onComplete, onError, relationshipId, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationshipId: String
		//		id of relationship to be deleted
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of relationship
	},
	=====*/
	
	_deleteRelationship: function(/*deleteRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a relationship from the dataset
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		if (kwArgs.remove) {
			content.remove = kwArgs.remove;
		}
				
		return dojo.xhrDelete({
			url: this._getRelationshipURL(kwArgs.relationshipId),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion())
			},
			handleAs: "json",
			sync: false,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				kwArgs.onComplete(kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_deleteRelationship"), ioArgs);
			})
		});
	},

	/*=====
	deleteRelationships_kwArgs: function(onComplete, onError, relationshipIds, condition, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationshipIds: String[]
		//		array of relationship ids to be deleted
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of relationships
	},
	=====*/
	
	_deleteRelationships: function(/*deleteRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete relationships from the dataset
		
		// build up the delete filter
		var deleteFilter = {};
		if (kwArgs.relationshipIds && kwArgs.relationshipIds.length > 0) {
			deleteFilter.identifiers = kwArgs.relationshipIds;
		}
		if (kwArgs.condition) {
			deleteFilter.condition = kwArgs.condition;
		}
		if (kwArgs.remove) {
			deleteFilter.remove = kwArgs.remove;
		}
		
		var json = dojo.toJson(deleteFilter, false);
			
		return dojo.xhrPost({
			url: this._getRelationshipsParamURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion()),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "deleteFilter"),
				'Content-Length': json.length,
				'X-Method-Override': 'Multi-DELETE'
			},
			handleAs: "json",
			postData: json,
			sync: false,			
			load: dojo.hitch(this, function(result, ioArgs){
				kwArgs.onComplete(kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_deleteRelationships"), ioArgs);
			})
		});
	},	
	
	/////////////////////////////
	// private methods
	/////////////////////////////
	
	_extractContentTypeVersion: function(ioArgs) {
		var version = this.getVersion();
		var contentType = ioArgs.xhr.getResponseHeader("Content-Type");
		if (contentType) {
			var parms = contentType.split(";");
			var length = parms.length;
			for (var index = 0;  index < length;  ++index) {
				if (parms[index].indexOf("version=") === 0) {								
					version = parms[index].substring(8)*1;
				}
			}
		}
		
		return version;		
	},
									
	_mixinStandard: function(content, kwArgs) {
		// summary:
		//		Add common arguments if provided in kwArgs
		if (kwArgs.clearCache) {
			content.clearCache = kwArgs.clearCache;
		}
		
		if (this._optimize) {
			content.optimize = this._optimize;
		}
		
		if (kwArgs.start && kwArgs.start > 0) {
			content.start = kwArgs.start;
		}
		
		if (kwArgs.count && kwArgs.count != Number.POSITIVE_INFINITY && kwArgs.count > 0) {
			content.count = kwArgs.count;
		}
		
		if (kwArgs.metadata) {
			content.metadata = true;
		}
		
		if (kwArgs.sort) {
			content.sort = kwArgs.sort; 
		}
		
		if (kwArgs.condition) {
			content.condition = kwArgs.condition;
		}
	},
	
	_getDatasetURL: function() {
		// summary:
		//		Returns url for for fetching dataset
		
		return this.getArg("baseUrl") + this.getAttribute("uri");
	},
	
	_getOriginItemsURL: function() {
		// summary:
		//		Returns url for fetching origin items
		return  this._getDatasetURL() + "/originItems";
	},

	_getItemsURL: function() {
		// summary:
		//		Returns url for fetching items for a dataset 
		return this._getDatasetURL() + "/items";		
	},

	_getItemsParamURL: function() {
		// summary:
		//		Returns url for fetching items and also includes
		//		all of the "param_" and "version" arguments
		return this._appendParams(this._getItemsURL());	
	},

	_getItemsQueryURL: function(kwArgs) {
		// summary:
		//		Returns url for custom query of items
		var url = this._getItemsParamURL();
		var questionMark = (url.indexOf('?') > 0);
		
		if (this._optimize) {
			if (!questionMark) {
				questionMark = true;
				url += "?optimize=" + this._optimize;
			}
			else {
				url += "&optimize=" + this._optimize;
			}			
		}
		
		if (kwArgs.start && kwArgs.start > 0) {
			if (!questionMark) {
				questionMark = true;
				url += "?start=" + kwArgs.start;
			}
			else {
				url += "&start=" + kwArgs.start;
			}
		}
		
		if (kwArgs.count && kwArgs.count > 0) {
			if (!questionMark) {
				questionMark = true;
				url += "?count=" + kwArgs.count;
			}
			else {
				url += "&count=" + kwArgs.count;
			}
		}
		
		if (kwArgs.properties && kwArgs.properties.length > 0) {
			if (!questionMark) {
				questionMark = true;
				url += "?properties=" + kwArgs.properties.join(",");
			}
			else {
				url += "&properties=" + kwArgs.properties.join(",");
			}
			
			if (kwArgs.metadata) {
				url += "&metadata=true";
			}
		}
				
		return url;
	},

	_getItemURL: function(itemId) {
		// summary:
		//		Returns url for fetching a specific item
		return this._getItemsURL() + "/" + encodeURIComponent(itemId);		
	},
	
	_getItemParamURL: function(itemId) {
		// summary:
		//		Returns url for item and also includes
		//		all of the "param_" and "version" arguments
		return this._appendParams(this._getItemURL(itemId));				
	},

	_getRelatedItemsURL: function(itemId) {
		// summary:
		//		Returns url for fetching related items
		return this._getItemURL(itemId) + "/relatedItems";	
	},

	_getLinksURL: function() {
		// summary:
		//		Returns url for fetching links for a dataset
		return this._getDatasetURL() + "/links";
	},

	_getLinksParamURL: function() {
		// summary:
		//		Returns url for fetching links and also includes
		//		all of the "param_" and "version" arguments
		return this._appendParams(this._getLinksURL()); 		
	},

	_getLinkURL: function(linkId) {
		// summary:
		//		Returns url for fetching a specific link
		return this._getLinksURL() + "/" + encodeURIComponent(linkId);		
	},

	_getLinkParamURL: function(linkId) {
		// summary:
		//		Returns url for link and also includes
		//		all of the "param_" and "version" arguments
		return this._appendParams(this._getLinkURL(linkId));				
	},

	_getRelationshipsURL: function() {
		// summary:
		//		Returns url for fetching relationships for a dataset
		return this._getDatasetURL() + "/relationships";		
	},

	_getRelationshipsParamURL: function() {
		// summary:
		//		Returns url for fetching relationships and also includes
		//		all of the "param_" and "version" arguments
		return this._appendParams(this._getRelationshipsURL()); 		
	},

	_getRelationshipURL: function(relationshipId) {
		// summary:
		//		Returns url for fetching a specific relationship
		return this._getRelationshipsURL() + "/" + encodeURIComponent(relationshipId);		
	},
	
	_getRelationshipParamURL: function(relationshipId) {
		// summary:
		//		Returns url for relationship and also includes
		//		all of the "param_" and "version" arguments
		return this._appendParams(this._getRelationshipURL(relationshipId));				
	},
	
	_getTasksURL: function() {
		// summary:
		//		Returns url for fetching tasks for a dataset
		return this._getDatasetURL() + "/tasks";		
	},

	_getTasksParamURL: function(kwArgs) {
		// summary:
		//		Returns url for fetching tasks for a dataset and also includes
		//		all of the "param_" and "version" arguments
		var url = this._appendParams(this._getTasksURL());		
		var questionMark = (url.indexOf('?') > 0);
		
		// optimize
		if (this._optimize) {
			if (!questionMark) {
				questionMark = true;
				url += "?optimize=" + this._optimize;
			}
			else {
				url += "&optimize=" + this._optimize;
			}			
		}
		
		// task category
		if (kwArgs.taskCategory) {
			if (!questionMark) {
				questionMark = true;
				url += "?category=" + kwArgs.taskCategory;
			}
			else {
				url += "&category=" + kwArgs.taskCategory;
			}			
		}
		
		return url;		
	},

	_getTaskURL: function(taskId) {
		// summary:
		//		Returns url for fetching a specific task
		return this._getTasksURL() + "/" + encodeURIComponent(taskId);		
	},

	_getNavigateParamURL: function(kwArgs) {
		// summary:
		//		Returns url for navigating a specific task and also includes
		//		all of the "param_" and "version" arguments
		var url = this._appendParams(this._getTaskURL(kwArgs.taskId));
		var questionMark = (url.indexOf('?') > 0);
		
		// optimize
		if (this._optimize) {
			if (!questionMark) {
				questionMark = true;
				url += "?optimize=" + this._optimize;
			}
			else {
				url += "&optimize=" + this._optimize;
			}			
		}
		
		// task category; should indicate navigation
		if (kwArgs.taskCategory) {
			if (!questionMark) {
				questionMark = true;
				url += "?category=" + kwArgs.taskCategory;
			}
			else {
				url += "&category=" + kwArgs.taskCategory;
			}			
		}
			
		return url;
	},
	
	_getItemStylesURL: function() {
		// summary:
		//		Returns url for fetching current item styles for a dataset
		return this._getDatasetURL() + "/itemStyles";	
	},

	_getLinkStylesURL: function() {
		// summary:
		//		Returns url for fetching current link styles for a dataset
		return this._getDatasetURL() + "/linkStyles";
	},

	_getRelationshipStylesURL: function() {
		// summary:
		//		Returns url for fetching current relationship styles for a dataset
		return this._getDatasetURL() + "/relationshipStyles";
	},

	_getStatusURL: function() {
		// summary:
		//		Returns url for fetching possible status values
		return this._getDatasetURL() + "/status";
	},

	_getColumnsURL: function() {
		// summary:
		//		Returns url for fetching current columns for a dataset
		return this._getDatasetURL() + "/columns";
	},
	
	_getColumnsParamURL: function(kwArgs) {
		// summary:
		//		Returns url for saving current tasks and also includes
		//		all of the "param_" and "version" arguments
		var url = this._appendParams(this._getColumnsURL());
		var questionMark = (url.indexOf('?') > 0);
		
		// type
		if (kwArgs.type) {
			if (!questionMark) {
				questionMark = true;
				url += "?type=" + kwArgs.type;
			}
			else {
				url += "&type=" + kwArgs.type;
			}			
		}
			
		return url;
	},

	_appendParams: function(url) {
		// summary:
		//		Append "param_" and "version" request parameters to the url
		//		so that dataset can be obtained from the REST service cache
		var questionMark = false;
		
		// add "param_" parms
		var parameters = this.getArg("parameters");
		for (var parm in parameters) {
			if (parameters.hasOwnProperty(parm)) {
				if (!questionMark) {
					questionMark = true;
					url += "?";
				}
				else {
					url += "&";
				}
				url += ("param_" + parm + "=" + parameters[parm]);				
			}
		}
		
		// IE will sometimes return http requests from cache without unique parms
		var now = new Date();
		if (!questionMark) {
			questionMark = true;
			url += "?d=" + now.getTime() + "&r=" + Math.random();
		}
		else {
			url += "&d=" + now.getTime() + "&r=" + Math.random();
		}	
		
		return url;
	},
	
	_getException: function(error, ioArgs, functionName) {
		return ibm.tivoli.navmodel.Utils.getException(error, ioArgs, functionName, this._trace, this.getVersion());
	},
	
	_isAllProps: function(properties) {
		return (properties && properties.length === 1 && properties[0] === "all");
	},
	
	_getPropIds: function(properties, allProps) {
		if (properties && properties.length > 0 && !allProps) {
			return properties;
		}
		
		return null;
	}
});

});

},
'dijit/layout/ContentPane':function(){
define([
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/lang", // lang.mixin lang.delegate lang.hitch lang.isFunction lang.isObject
	"../_Widget",
	"./_ContentPaneResizeMixin",
	"dojo/string", // string.substitute
	"dojo/html", // html._ContentSetter html._emptyNode
	"dojo/i18n!../nls/loading",
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/_base/Deferred", // Deferred
	"dojo/dom", // dom.byId
	"dojo/dom-attr", // domAttr.attr
	"dojo/_base/window", // win.body win.doc.createDocumentFragment
	"dojo/_base/xhr", // xhr.get
	"dojo/i18n" // i18n.getLocalization
], function(kernel, lang, _Widget, _ContentPaneResizeMixin, string, html, nlsLoading,
	array, declare, Deferred, dom, domAttr, win, xhr, i18n){

/*=====
	var _Widget = dijit._Widget;
	var _ContentPaneResizeMixin = dijit.layout._ContentPaneResizeMixin;
=====*/

// module:
//		dijit/layout/ContentPane
// summary:
//		A widget containing an HTML fragment, specified inline
//		or by uri.  Fragment may include widgets.


return declare("dijit.layout.ContentPane", [_Widget, _ContentPaneResizeMixin], {
	// summary:
	//		A widget containing an HTML fragment, specified inline
	//		or by uri.  Fragment may include widgets.
	//
	// description:
	//		This widget embeds a document fragment in the page, specified
	//		either by uri, javascript generated markup or DOM reference.
	//		Any widgets within this content are instantiated and managed,
	//		but laid out according to the HTML structure.  Unlike IFRAME,
	//		ContentPane embeds a document fragment as would be found
	//		inside the BODY tag of a full HTML document.  It should not
	//		contain the HTML, HEAD, or BODY tags.
	//		For more advanced functionality with scripts and
	//		stylesheets, see dojox.layout.ContentPane.  This widget may be
	//		used stand alone or as a base class for other widgets.
	//		ContentPane is useful as a child of other layout containers
	//		such as BorderContainer or TabContainer, but note that those
	//		widgets can contain any widget as a child.
	//
	// example:
	//		Some quick samples:
	//		To change the innerHTML: cp.set('content', '<b>new content</b>')
	//
	//		Or you can send it a NodeList: cp.set('content', dojo.query('div [class=selected]', userSelection))
	//
	//		To do an ajax update: cp.set('href', url)

	// href: String
	//		The href of the content that displays now.
	//		Set this at construction if you want to load data externally when the
	//		pane is shown.  (Set preload=true to load it immediately.)
	//		Changing href after creation doesn't have any effect; Use set('href', ...);
	href: "",

	// content: String || DomNode || NodeList || dijit._Widget
	//		The innerHTML of the ContentPane.
	//		Note that the initialization parameter / argument to set("content", ...)
	//		can be a String, DomNode, Nodelist, or _Widget.
	content: "",

	// extractContent: Boolean
	//		Extract visible content from inside of <body> .... </body>.
	//		I.e., strip <html> and <head> (and it's contents) from the href
	extractContent: false,

	// parseOnLoad: Boolean
	//		Parse content and create the widgets, if any.
	parseOnLoad: true,

	// parserScope: String
	//		Flag passed to parser.  Root for attribute names to search for.   If scopeName is dojo,
	//		will search for data-dojo-type (or dojoType).  For backwards compatibility
	//		reasons defaults to dojo._scopeName (which is "dojo" except when
	//		multi-version support is used, when it will be something like dojo16, dojo20, etc.)
	parserScope: kernel._scopeName,

	// preventCache: Boolean
	//		Prevent caching of data from href's by appending a timestamp to the href.
	preventCache: false,

	// preload: Boolean
	//		Force load of data on initialization even if pane is hidden.
	preload: false,

	// refreshOnShow: Boolean
	//		Refresh (re-download) content when pane goes from hidden to shown
	refreshOnShow: false,

	// loadingMessage: String
	//		Message that shows while downloading
	loadingMessage: "<span class='dijitContentPaneLoading'><span class='dijitInline dijitIconLoading'></span>${loadingState}</span>",

	// errorMessage: String
	//		Message that shows if an error occurs
	errorMessage: "<span class='dijitContentPaneError'><span class='dijitInline dijitIconError'></span>${errorState}</span>",

	// isLoaded: [readonly] Boolean
	//		True if the ContentPane has data in it, either specified
	//		during initialization (via href or inline content), or set
	//		via set('content', ...) / set('href', ...)
	//
	//		False if it doesn't have any content, or if ContentPane is
	//		still in the process of downloading href.
	isLoaded: false,

	baseClass: "dijitContentPane",

	/*======
	// ioMethod: dojo.xhrGet|dojo.xhrPost
	//		Function that should grab the content specified via href.
	ioMethod: dojo.xhrGet,
	======*/

	// ioArgs: Object
	//		Parameters to pass to xhrGet() request, for example:
	// |	<div data-dojo-type="dijit.layout.ContentPane" data-dojo-props="href: './bar', ioArgs: {timeout: 500}">
	ioArgs: {},

	// onLoadDeferred: [readonly] dojo.Deferred
	//		This is the `dojo.Deferred` returned by set('href', ...) and refresh().
	//		Calling onLoadDeferred.addCallback() or addErrback() registers your
	//		callback to be called only once, when the prior set('href', ...) call or
	//		the initial href parameter to the constructor finishes loading.
	//
	//		This is different than an onLoad() handler which gets called any time any href
	//		or content is loaded.
	onLoadDeferred: null,

	// Cancel _WidgetBase's _setTitleAttr because we don't want the title attribute (used to specify
	// tab labels) to be copied to ContentPane.domNode... otherwise a tooltip shows up over the
	// entire pane.
	_setTitleAttr: null,

	// Flag to parser that I'll parse my contents, so it shouldn't.
	stopParser: true,

	// template: [private] Boolean
	//		Flag from the parser that this ContentPane is inside a template
	//		so the contents are pre-parsed.
	// (TODO: this declaration can be commented out in 2.0)
	template: false,

	create: function(params, srcNodeRef){
		// Convert a srcNodeRef argument into a content parameter, so that the original contents are
		// processed in the same way as contents set via set("content", ...), calling the parser etc.
		// Avoid modifying original params object since that breaks NodeList instantiation, see #11906.
		if((!params || !params.template) && srcNodeRef && !("href" in params) && !("content" in params)){
			var df = win.doc.createDocumentFragment();
			srcNodeRef = dom.byId(srcNodeRef);
			while(srcNodeRef.firstChild){
				df.appendChild(srcNodeRef.firstChild);
			}
			params = lang.delegate(params, {content: df});
		}
		this.inherited(arguments, [params, srcNodeRef]);
	},

	postMixInProperties: function(){
		this.inherited(arguments);
		var messages = i18n.getLocalization("dijit", "loading", this.lang);
		this.loadingMessage = string.substitute(this.loadingMessage, messages);
		this.errorMessage = string.substitute(this.errorMessage, messages);
	},

	buildRendering: function(){
		this.inherited(arguments);

		// Since we have no template we need to set this.containerNode ourselves, to make getChildren() work.
		// For subclasses of ContentPane that do have a template, does nothing.
		if(!this.containerNode){
			this.containerNode = this.domNode;
		}

		// remove the title attribute so it doesn't show up when hovering
		// over a node  (TODO: remove in 2.0, no longer needed after #11490)
		this.domNode.title = "";

		if(!domAttr.get(this.domNode,"role")){
			this.domNode.setAttribute("role", "group");
		}
	},

	startup: function(){
		// summary:
		//		Call startup() on all children including non _Widget ones like dojo.dnd.Source objects

		// This starts all the widgets
		this.inherited(arguments);

		// And this catches stuff like dojo.dnd.Source
		if(this._contentSetter){
			array.forEach(this._contentSetter.parseResults, function(obj){
				if(!obj._started && !obj._destroyed && lang.isFunction(obj.startup)){
					obj.startup();
					obj._started = true;
				}
			}, this);
		}
	},

	setHref: function(/*String|Uri*/ href){
		// summary:
		//		Deprecated.   Use set('href', ...) instead.
		kernel.deprecated("dijit.layout.ContentPane.setHref() is deprecated. Use set('href', ...) instead.", "", "2.0");
		return this.set("href", href);
	},
	_setHrefAttr: function(/*String|Uri*/ href){
		// summary:
		//		Hook so set("href", ...) works.
		// description:
		//		Reset the (external defined) content of this pane and replace with new url
		//		Note: It delays the download until widget is shown if preload is false.
		//	href:
		//		url to the page you want to get, must be within the same domain as your mainpage

		// Cancel any in-flight requests (a set('href', ...) will cancel any in-flight set('href', ...))
		this.cancel();

		this.onLoadDeferred = new Deferred(lang.hitch(this, "cancel"));
		this.onLoadDeferred.addCallback(lang.hitch(this, "onLoad"));

		this._set("href", href);

		// _setHrefAttr() is called during creation and by the user, after creation.
		// Assuming preload == false, only in the second case do we actually load the URL;
		// otherwise it's done in startup(), and only if this widget is shown.
		if(this.preload || (this._created && this._isShown())){
			this._load();
		}else{
			// Set flag to indicate that href needs to be loaded the next time the
			// ContentPane is made visible
			this._hrefChanged = true;
		}

		return this.onLoadDeferred;		// Deferred
	},

	setContent: function(/*String|DomNode|Nodelist*/data){
		// summary:
		//		Deprecated.   Use set('content', ...) instead.
		kernel.deprecated("dijit.layout.ContentPane.setContent() is deprecated.  Use set('content', ...) instead.", "", "2.0");
		this.set("content", data);
	},
	_setContentAttr: function(/*String|DomNode|Nodelist*/data){
		// summary:
		//		Hook to make set("content", ...) work.
		//		Replaces old content with data content, include style classes from old content
		//	data:
		//		the new Content may be String, DomNode or NodeList
		//
		//		if data is a NodeList (or an array of nodes) nodes are copied
		//		so you can import nodes from another document implicitly

		// clear href so we can't run refresh and clear content
		// refresh should only work if we downloaded the content
		this._set("href", "");

		// Cancel any in-flight requests (a set('content', ...) will cancel any in-flight set('href', ...))
		this.cancel();

		// Even though user is just setting content directly, still need to define an onLoadDeferred
		// because the _onLoadHandler() handler is still getting called from setContent()
		this.onLoadDeferred = new Deferred(lang.hitch(this, "cancel"));
		if(this._created){
			// For back-compat reasons, call onLoad() for set('content', ...)
			// calls but not for content specified in srcNodeRef (ie: <div data-dojo-type=ContentPane>...</div>)
			// or as initialization parameter (ie: new ContentPane({content: ...})
			this.onLoadDeferred.addCallback(lang.hitch(this, "onLoad"));
		}

		this._setContent(data || "");

		this._isDownloaded = false; // mark that content is from a set('content') not a set('href')

		return this.onLoadDeferred; 	// Deferred
	},
	_getContentAttr: function(){
		// summary:
		//		Hook to make get("content") work
		return this.containerNode.innerHTML;
	},

	cancel: function(){
		// summary:
		//		Cancels an in-flight download of content
		if(this._xhrDfd && (this._xhrDfd.fired == -1)){
			this._xhrDfd.cancel();
		}
		delete this._xhrDfd; // garbage collect

		this.onLoadDeferred = null;
	},

	uninitialize: function(){
		if(this._beingDestroyed){
			this.cancel();
		}
		this.inherited(arguments);
	},

	destroyRecursive: function(/*Boolean*/ preserveDom){
		// summary:
		//		Destroy the ContentPane and its contents

		// if we have multiple controllers destroying us, bail after the first
		if(this._beingDestroyed){
			return;
		}
		this.inherited(arguments);
	},

	_onShow: function(){
		// summary:
		//		Called when the ContentPane is made visible
		// description:
		//		For a plain ContentPane, this is called on initialization, from startup().
		//		If the ContentPane is a hidden pane of a TabContainer etc., then it's
		//		called whenever the pane is made visible.
		//
		//		Does necessary processing, including href download and layout/resize of
		//		child widget(s)

		this.inherited(arguments);

		if(this.href){
			if(!this._xhrDfd && // if there's an href that isn't already being loaded
				(!this.isLoaded || this._hrefChanged || this.refreshOnShow)
			){
				return this.refresh();	// If child has an href, promise that fires when the load is complete
			}
		}
	},

	refresh: function(){
		// summary:
		//		[Re]download contents of href and display
		// description:
		//		1. cancels any currently in-flight requests
		//		2. posts "loading..." message
		//		3. sends XHR to download new data

		// Cancel possible prior in-flight request
		this.cancel();

		this.onLoadDeferred = new Deferred(lang.hitch(this, "cancel"));
		this.onLoadDeferred.addCallback(lang.hitch(this, "onLoad"));
		this._load();
		return this.onLoadDeferred;		// If child has an href, promise that fires when refresh is complete
	},

	_load: function(){
		// summary:
		//		Load/reload the href specified in this.href

		// display loading message
		this._setContent(this.onDownloadStart(), true);

		var self = this;
		var getArgs = {
			preventCache: (this.preventCache || this.refreshOnShow),
			url: this.href,
			handleAs: "text"
		};
		if(lang.isObject(this.ioArgs)){
			lang.mixin(getArgs, this.ioArgs);
		}

		var hand = (this._xhrDfd = (this.ioMethod || xhr.get)(getArgs));

		hand.addCallback(function(html){
			try{
				self._isDownloaded = true;
				self._setContent(html, false);
				self.onDownloadEnd();
			}catch(err){
				self._onError('Content', err); // onContentError
			}
			delete self._xhrDfd;
			return html;
		});

		hand.addErrback(function(err){
			if(!hand.canceled){
				// show error message in the pane
				self._onError('Download', err); // onDownloadError
			}
			delete self._xhrDfd;
			return err;
		});

		// Remove flag saying that a load is needed
		delete this._hrefChanged;
	},

	_onLoadHandler: function(data){
		// summary:
		//		This is called whenever new content is being loaded
		this._set("isLoaded", true);
		try{
			this.onLoadDeferred.callback(data);
		}catch(e){
			console.error('Error '+this.widgetId+' running custom onLoad code: ' + e.message);
		}
	},

	_onUnloadHandler: function(){
		// summary:
		//		This is called whenever the content is being unloaded
		this._set("isLoaded", false);
		try{
			this.onUnload();
		}catch(e){
			console.error('Error '+this.widgetId+' running custom onUnload code: ' + e.message);
		}
	},

	destroyDescendants: function(/*Boolean*/ preserveDom){
		// summary:
		//		Destroy all the widgets inside the ContentPane and empty containerNode

		// Make sure we call onUnload (but only when the ContentPane has real content)
		if(this.isLoaded){
			this._onUnloadHandler();
		}

		// Even if this.isLoaded == false there might still be a "Loading..." message
		// to erase, so continue...

		// For historical reasons we need to delete all widgets under this.containerNode,
		// even ones that the user has created manually.
		var setter = this._contentSetter;
		array.forEach(this.getChildren(), function(widget){
			if(widget.destroyRecursive){
				widget.destroyRecursive(preserveDom);
			}
		});
		if(setter){
			// Most of the widgets in setter.parseResults have already been destroyed, but
			// things like Menu that have been moved to <body> haven't yet
			array.forEach(setter.parseResults, function(widget){
				if(widget.destroyRecursive && widget.domNode && widget.domNode.parentNode == win.body()){
					widget.destroyRecursive(preserveDom);
				}
			});
			delete setter.parseResults;
		}

		// And then clear away all the DOM nodes
		if(!preserveDom){
			html._emptyNode(this.containerNode);
		}

		// Delete any state information we have about current contents
		delete this._singleChild;
	},

	_setContent: function(/*String|DocumentFragment*/ cont, /*Boolean*/ isFakeContent){
		// summary:
		//		Insert the content into the container node

		// first get rid of child widgets
		this.destroyDescendants();

		// html.set will take care of the rest of the details
		// we provide an override for the error handling to ensure the widget gets the errors
		// configure the setter instance with only the relevant widget instance properties
		// NOTE: unless we hook into attr, or provide property setters for each property,
		// we need to re-configure the ContentSetter with each use
		var setter = this._contentSetter;
		if(! (setter && setter instanceof html._ContentSetter)){
			setter = this._contentSetter = new html._ContentSetter({
				node: this.containerNode,
				_onError: lang.hitch(this, this._onError),
				onContentError: lang.hitch(this, function(e){
					// fires if a domfault occurs when we are appending this.errorMessage
					// like for instance if domNode is a UL and we try append a DIV
					var errMess = this.onContentError(e);
					try{
						this.containerNode.innerHTML = errMess;
					}catch(e){
						console.error('Fatal '+this.id+' could not change content due to '+e.message, e);
					}
				})/*,
				_onError */
			});
		}

		var setterParams = lang.mixin({
			cleanContent: this.cleanContent,
			extractContent: this.extractContent,
			parseContent: !cont.domNode && this.parseOnLoad,
			parserScope: this.parserScope,
			startup: false,
			dir: this.dir,
			lang: this.lang,
			textDir: this.textDir
		}, this._contentSetterParams || {});

		setter.set( (lang.isObject(cont) && cont.domNode) ? cont.domNode : cont, setterParams );

		// setter params must be pulled afresh from the ContentPane each time
		delete this._contentSetterParams;

		if(this.doLayout){
			this._checkIfSingleChild();
		}

		if(!isFakeContent){
			if(this._started){
				// Startup each top level child widget (and they will start their children, recursively)
				delete this._started;
				this.startup();

				// Call resize() on each of my child layout widgets,
				// or resize() on my single child layout widget...
				// either now (if I'm currently visible) or when I become visible
				this._scheduleLayout();
			}

			this._onLoadHandler(cont);
		}
	},

	_onError: function(type, err, consoleText){
		this.onLoadDeferred.errback(err);

		// shows user the string that is returned by on[type]Error
		// override on[type]Error and return your own string to customize
		var errText = this['on' + type + 'Error'].call(this, err);
		if(consoleText){
			console.error(consoleText, err);
		}else if(errText){// a empty string won't change current content
			this._setContent(errText, true);
		}
	},

	// EVENT's, should be overide-able
	onLoad: function(/*===== data =====*/){
		// summary:
		//		Event hook, is called after everything is loaded and widgetified
		// tags:
		//		callback
	},

	onUnload: function(){
		// summary:
		//		Event hook, is called before old content is cleared
		// tags:
		//		callback
	},

	onDownloadStart: function(){
		// summary:
		//		Called before download starts.
		// description:
		//		The string returned by this function will be the html
		//		that tells the user we are loading something.
		//		Override with your own function if you want to change text.
		// tags:
		//		extension
		return this.loadingMessage;
	},

	onContentError: function(/*Error*/ /*===== error =====*/){
		// summary:
		//		Called on DOM faults, require faults etc. in content.
		//
		//		In order to display an error message in the pane, return
		//		the error message from this method, as an HTML string.
		//
		//		By default (if this method is not overriden), it returns
		//		nothing, so the error message is just printed to the console.
		// tags:
		//		extension
	},

	onDownloadError: function(/*Error*/ /*===== error =====*/){
		// summary:
		//		Called when download error occurs.
		//
		//		In order to display an error message in the pane, return
		//		the error message from this method, as an HTML string.
		//
		//		Default behavior (if this method is not overriden) is to display
		//		the error message inside the pane.
		// tags:
		//		extension
		return this.errorMessage;
	},

	onDownloadEnd: function(){
		// summary:
		//		Called when download is finished.
		// tags:
		//		callback
	}
});

});

},
'dijit/dijit':function(){
define([
	".",
	"./_base",
	"dojo/parser",
	"./_Widget",
	"./_TemplatedMixin",
	"./_Container",
	"./layout/_LayoutWidget",
	"./form/_FormWidget",
	"./form/_FormValueWidget"
], function(dijit){

	// module:
	//		dijit/dijit
	// summary:
	//		A roll-up for common dijit methods
	//		All the stuff in _base (these are the function that are guaranteed available without an explicit dojo.require)
	//		And some other stuff that we tend to pull in all the time anyway

	return dijit;
});

},
'dojox/mvc/_patches':function(){
define([
	"dojo/_base/lang",
	"dojo/_base/array",
	"dijit/_WidgetBase",
	"./_DataBindingMixin",
	"dijit/form/ValidationTextBox",
	"dijit/form/NumberTextBox"
], function(lang, array, wb, dbm, vtb, ntb){
	/*=====
		vtb = dijit.form.ValidationTextBox;
		ntb = dijit.form.NumberTextBox;
		dbm = dojox.mvc._DataBindingMixin;
		wb = dijit._WidgetBase;
	=====*/

	//Apply the data binding mixin to all dijits, see mixin class description for details
	lang.extend(wb, new dbm());

	// monkey patch dijit._WidgetBase.startup to get data binds set up
	var oldWidgetBaseStartup = wb.prototype.startup;
	wb.prototype.startup = function(){
		this._dbstartup();
		oldWidgetBaseStartup.apply(this);
	};

	// monkey patch dijit._WidgetBase.destroy to remove watches setup in _DataBindingMixin
	var oldWidgetBaseDestroy = wb.prototype.destroy;
	wb.prototype.destroy = function(/*Boolean*/ preserveDom){
		if(this._modelWatchHandles){
			array.forEach(this._modelWatchHandles, function(h){ h.unwatch(); });
		}
		if(this._viewWatchHandles){
			array.forEach(this._viewWatchHandles, function(h){ h.unwatch(); });
		}
		oldWidgetBaseDestroy.apply(this, [preserveDom]);		
	};

	// monkey patch dijit.form.ValidationTextBox.isValid to check this.inherited for isValid
	var oldValidationTextBoxIsValid = vtb.prototype.isValid;
	vtb.prototype.isValid = function(/*Boolean*/ isFocused){
		return (this.inherited("isValid", arguments) !== false && oldValidationTextBoxIsValid.apply(this, [isFocused]));
	};

	// monkey patch dijit.form.NumberTextBox.isValid to check this.inherited for isValid
	var oldNumberTextBoxIsValid = ntb.prototype.isValid;
	ntb.prototype.isValid = function(/*Boolean*/ isFocused){
		return (this.inherited("isValid", arguments) !== false && oldNumberTextBoxIsValid.apply(this, [isFocused]));
	};
});

},
'ibm/tivoli/navmodel/Trace':function(){
// wrapped by build app
define("ibm/tivoli/navmodel/Trace", ["dijit","dojo","dojox"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Trace");
 
dojo.declare("ibm.tivoli.navmodel.Trace", null, {
    // private members     
	traceName: "",
	level: 0,  // error
        
	constructor: function(name) {
		this.traceName = name;
				
		if (dojo.config.trace) {
			// "all" overrides individual class entries
			if (dojo.config.trace.all) {
				this.level = this._translateLevel("all", dojo.config.trace.all);
			}
			else {
				var lastDot = this.traceName.lastIndexOf(".");
				if (lastDot > 0) {
					var className = this.traceName.substring(lastDot+1);
					if (dojo.config.trace[className]) {
						this.level = this._translateLevel(className, dojo.config.trace[className]);
					}
				}
			}
		}					
 	},
        
	/////////////////////////////
	// public methods
	/////////////////////////////
	
	isError: function() {
		return (this.level >= 0);
	},
	
	isWarn: function() {
		return (this.level >= 1);
	},
	
	isInfo: function() {
		return (this.level >= 2);
	},
	
	isFine: function() {
		return (this.level >= 3);
	},
	
	isFiner: function() {
		return (this.level >= 4);
	},
	
	isFinest: function() {
		return (this.level >= 5);
	},
	
	error: function(dbgString, methodName) {
		if (this.isError()) {
			if (dbgString && dbgString instanceof Error) {
                console.error(this._output("EXCEPTION", methodName, dbgString.toString()));
	    	} 
	    	else if (dbgString && dbgString.isInstanceOf && dbgString.isInstanceOf(ibm.tivoli.navmodel.Exception)) {
	    		console.error(this._output("EXCEPTION", methodName, dbgString.getMsgText()));
	    	} else {
	    		console.error(this._output("ERROR", methodName, dbgString));
	    	}
	    }
	},
	
	warn: function(dbgString, methodName) {
    	if (this.isWarn()) {
    		console.warn(this._output("WARN", methodName, dbgString));
    	}
    },
    
	info: function(dbgString, methodName) {
    	if (this.isInfo()) {
    		console.info(this._output("INFO", methodName, dbgString));
    	}
    },
    
    fine: function(dbgString, methodName) {
    	if (this.isFine()) {
    		console.info(this._output("FINE", methodName, dbgString));
    	}
    },
    
    finer: function(dbgString, methodName) {
    	if (this.isFiner()) {
    		console.info(this._output("FINER", methodName, dbgString));
    	}
    },
    
	finest: function(dbgString, methodName) {
    	if (this.isFinest()) {
    		console.info(this._output("FINEST", methodName, dbgString));
    	}
    },
	
	entering: function(dbgString, methodName) {
    	if (this.isFinest()) {
    		console.info(this._output("ENTERING", methodName, dbgString));
    	}
    },
    
    exiting: function(dbgString, methodName) {
    	if (this.isFinest()) {
    		console.info(this._output("EXITING", methodName, dbgString));
    	}
    },
    
	object: function(dbgObject, methodName) {
    	if (this.isFine()) {
    		console.log(this._output("OBJECT", methodName, dojo.toJson(dbgObject)));
    	}
    },
    
	assert: function(predicate, message, methodName) {
		if (this.isFine()) {
			if (predicate === false ) {
				console.log(this._output("ASSERT", methodName, "assertion failed: " + message));
			}
		}		
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////
	
	_translateLevel: function(className, value) {
		var level = 0;
		
		switch(value) {
			case "off":
				level = -1;
				break;
			case "error":
				level = 0;
				break;
			case "warn":
				level = 1;
				break;
			case "info":
				level = 2;
				break;
			case "fine":
				level = 3;
				break;
			case "finer":
				level = 4;
				break;
			case "finest":
				level = 5;
				break;
			default:
				console.log("className=" + className + " has invalid level=" + value);
		}
		
		return level;
	},
	
	//***** DEBUG::trace=ibm.tivoli.navmodel.Dataset, method=EXITING; event received, type = itemStatusChanged
	_output: function(level, methodName, message) {
		var outputMessage = "***** " + level;
		if (message) {
			outputMessage = outputMessage + ": " + message;
		}
		outputMessage = outputMessage + " ::trace=" + this.traceName;
		if (methodName) {
			outputMessage = outputMessage + ", method=" + methodName;
		}
		return outputMessage;
	}	
});
});

},
'ibm/tivoli/tip/navwidget/dojo/Timer':function(){
// wrapped by build app
define("ibm/tivoli/tip/navwidget/dojo/Timer", ["dijit","dojo","dojox"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.dojo.Timer");

dojo.declare(
    "ibm.tivoli.tip.navwidget.dojo.Timer",
    [],
    {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
    	
		// time: int
    	//		Coming soon
  		time: 0,
  		
  		// timing: Boolean
  		//		Coming soon
  		timing: false,
  
  		now: function(){ 		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			return (new Date()).getTime(); 
		},
		
  		start: function(){ 		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			this.timing = true; 
			this.time = this.now(); 
		},
		
  		stop: function(){ 		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			this.timing = false; 
			return this.now()-this.time; 
		},   
		
		stopAndGetTraceString: function(){ 		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			var since = this.stop();
			return "Time = " + since + " ms (or " + (since/1000) + " s)"; 
		},
		
  		isTiming: function(){ 		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			return this.timing; 
		}
    }
);


});

},
'url:dijit/layout/templates/TabContainer.html':"<div class=\"dijitTabContainer\">\n\t<div class=\"dijitTabListWrapper\" data-dojo-attach-point=\"tablistNode\"></div>\n\t<div data-dojo-attach-point=\"tablistSpacer\" class=\"dijitTabSpacer ${baseClass}-spacer\"></div>\n\t<div class=\"dijitTabPaneWrapper ${baseClass}-container\" data-dojo-attach-point=\"containerNode\"></div>\n</div>\n",
'ibm/usmi/widgets/util/DefaultMenuProviderHelper':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/widgets/util/DefaultMenuProviderHelper", ["dojo/_base/kernel",
        "dojo/_base/declare",
        "dojo/topic",
        // TWL imports
        "ibm/tivoli/tip/navwidget/topology/dijit/MenuProviderHelper",
        "ibm/tivoli/tip/navwidget/topology/dijit/MenuProviderHelperBase",
        "ibm/usmi/util/DefaultMessageHandler",
        "ibm/tivoli/navmodel/Dataset"],
    function (dojo, declare, topic,
            MenuProviderHelper, MenuProviderHelperBase, DefaultMessageHandler, Dataset) {

    return declare([MenuProviderHelper], {

        getMenuItems: function(/*Object*/ context) {
            // summary:
            //        Coming soon
            // description:
            //        Coming soon
            // context:
            //        Coming soon

            // Turn off properties menu item by saying selection does not support properties.
            // We probably could have just not inherited from MenuProviderHelper since that class
            // just sets up the properties menu item, but this seems safer in case TWL decides
            // to add to MenuProviderHelper in the future.
            var selectedObjects = context.selection;
            var showPropertiesInContextMenu = context.showPropertiesInContextMenu;

            if(( selectedObjects.getItemCount() + selectedObjects.getLinkCount() + selectedObjects.getRelationshipCount()) == 1){

                if(selectedObjects.getItemCount() == 1){

                    var object = selectedObjects.getItems()[0];
                    if(showPropertiesInContextMenu===true){
                    	 object.setSupportsProperties(true);
                    } else {
                    	object.setSupportsProperties(false);
                    }
                   
                }
            }

            var items = this.inherited(arguments);

            return items;
        },

        getMenuCommand: function(/*Object*/ commandName, /*Object*/ context, /*Object*/ menuItem) {
            // summary:
            //        Coming soon
            // description:
            //        Coming soon
            // commandName:
            //        Coming soon
            // context:
            //        Coming soon
            // menuItem:
            //        Coming soon
            var result = null, generalTab;

            // Note: add code here to check the commandName (which is also the actionID from
            // menuItem) and if it is one of our actions, do or call whatever is needed to do
            // the work and then set the result to a no-op command so the TIPContextMenu will
            // take no action on the returned result
            // Ex:
            //     result = {"launchType":"NOOP"};
            //  or
            //     result = menuItem;  (if the launchType="NOOP" was already an attribute in the menu item)
            //

            //command name is = to actionID...so if one of the special menu items added above, do special processing

            // make sure we have a menu item (default action/task processing can call with just a
            // command and no menuItem)
            if (menuItem && menuItem.launchType === "JavaScript") {
                launchParms = dojo.fromJson(menuItem.uri);
                if (launchParms.topic) {
                    topic.publish(launchParms.topic, launchParms, menuItem, context);
                    result = {"launchType":"NOOP"};
                }
                else {
                    alert("topic not defined in launch parms from server task");
                    // TODO put up error message or log error indicating task did not specify correct launch parms
                }
                return result;
            } //else if (menuItem && menuItem.launchType == "WEB_URL") {
    //            this.launchURL (menuItem);
    //        }
            else {
                // Note: If this is NOT one of our client-side actions, DO NOT set result and let
                // the parent class command be returned
                result = (result) ? result : this.inherited(arguments);
                switch (commandName) {
    				case "nwmi_PropertiesNotebookDialog":
    					if(!this._propDialog){
    						break;
    					}
    					if(!this._propDialog._propNotebook){
    						break;
    					}
    					generalTab=this._propDialog._propNotebook.getPluggableView('GENERAL_PROPERTIES_VIEW');
    					if(!generalTab){
    						break;
    					}
    					if(!generalTab._propertyRendererObj){
    						break;
    					}
    					if(context.daHelper){
    						generalTab._propertyRendererObj._daHelper=context.daHelper;
    					}
    					
                }
                return result;
            }
        },

           launchURL: function(data)
           {
               console.log ("enter launchURL id ");

               var left   = (screen.width  )/2,
                   top    = (screen.height)/2,
                   params = 'top='+top+', left='+left;

               if(data.browserwindowid !== null) {
                   window.open(data.uri,data.browserwindowid, params);
               }
               else {
                   window.open(data.uri,"_self", params);
               }
           }
    });
});

},
'dojo/dnd/Avatar':function(){
define(["../main", "./common"], function(dojo) {
	// module:
	//		dojo/dnd/Avatar
	// summary:
	//		TODOC


dojo.declare("dojo.dnd.Avatar", null, {
	// summary:
	//		Object that represents transferred DnD items visually
	// manager: Object
	//		a DnD manager object

	constructor: function(manager){
		this.manager = manager;
		this.construct();
	},

	// methods
	construct: function(){
		// summary:
		//		constructor function;
		//		it is separate so it can be (dynamically) overwritten in case of need
		this.isA11y = dojo.hasClass(dojo.body(),"dijit_a11y");
		var a = dojo.create("table", {
				"class": "dojoDndAvatar",
				style: {
					position: "absolute",
					zIndex:   "1999",
					margin:   "0px"
				}
			}),
			source = this.manager.source, node,
			b = dojo.create("tbody", null, a),
			tr = dojo.create("tr", null, b),
			td = dojo.create("td", null, tr),
			icon = this.isA11y ? dojo.create("span", {
						id : "a11yIcon",
						innerHTML : this.manager.copy ? '+' : "<"
					}, td) : null,
			span = dojo.create("span", {
				innerHTML: source.generateText ? this._generateText() : ""
			}, td),
			k = Math.min(5, this.manager.nodes.length), i = 0;
		// we have to set the opacity on IE only after the node is live
		dojo.attr(tr, {
			"class": "dojoDndAvatarHeader",
			style: {opacity: 0.9}
		});
		for(; i < k; ++i){
			if(source.creator){
				// create an avatar representation of the node
				node = source._normalizedCreator(source.getItem(this.manager.nodes[i].id).data, "avatar").node;
			}else{
				// or just clone the node and hope it works
				node = this.manager.nodes[i].cloneNode(true);
				if(node.tagName.toLowerCase() == "tr"){
					// insert extra table nodes
					var table = dojo.create("table"),
						tbody = dojo.create("tbody", null, table);
					tbody.appendChild(node);
					node = table;
				}
			}
			node.id = "";
			tr = dojo.create("tr", null, b);
			td = dojo.create("td", null, tr);
			td.appendChild(node);
			dojo.attr(tr, {
				"class": "dojoDndAvatarItem",
				style: {opacity: (9 - i) / 10}
			});
		}
		this.node = a;
	},
	destroy: function(){
		// summary:
		//		destructor for the avatar; called to remove all references so it can be garbage-collected
		dojo.destroy(this.node);
		this.node = false;
	},
	update: function(){
		// summary:
		//		updates the avatar to reflect the current DnD state
		dojo[(this.manager.canDropFlag ? "add" : "remove") + "Class"](this.node, "dojoDndAvatarCanDrop");
		if (this.isA11y){
			var icon = dojo.byId("a11yIcon");
			var text = '+';   // assume canDrop && copy
			if (this.manager.canDropFlag && !this.manager.copy) {
				text = '< '; // canDrop && move
			}else if (!this.manager.canDropFlag && !this.manager.copy) {
				text = "o"; //!canDrop && move
			}else if(!this.manager.canDropFlag){
				text = 'x';  // !canDrop && copy
			}
			icon.innerHTML=text;
		}
		// replace text
		dojo.query(("tr.dojoDndAvatarHeader td span" +(this.isA11y ? " span" : "")), this.node).forEach(
			function(node){
				node.innerHTML = this._generateText();
			}, this);
	},
	_generateText: function(){
		// summary: generates a proper text to reflect copying or moving of items
		return this.manager.nodes.length.toString();
	}
});

return dojo.dnd.Avatar;
});

},
'dijit/form/Form':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.set
	"dojo/_base/event", // event.stop
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/sniff", // has("ie")
	"../_Widget",
	"../_TemplatedMixin",
	"./_FormMixin",
	"../layout/_ContentPaneResizeMixin"
], function(declare, domAttr, event, kernel, has, _Widget, _TemplatedMixin, _FormMixin, _ContentPaneResizeMixin){

/*=====
	var _Widget = dijit._Widget;
	var _TemplatedMixin = dijit._TemplatedMixin;
	var _FormMixin = dijit.form._FormMixin;
	var _ContentPaneResizeMixin = dijit.layout._ContentPaneResizeMixin;
=====*/

	// module:
	//		dijit/form/Form
	// summary:
	//		Widget corresponding to HTML form tag, for validation and serialization


	return declare("dijit.form.Form", [_Widget, _TemplatedMixin, _FormMixin, _ContentPaneResizeMixin], {
		// summary:
		//		Widget corresponding to HTML form tag, for validation and serialization
		//
		// example:
		//	|	<form data-dojo-type="dijit.form.Form" id="myForm">
		//	|		Name: <input type="text" name="name" />
		//	|	</form>
		//	|	myObj = {name: "John Doe"};
		//	|	dijit.byId('myForm').set('value', myObj);
		//	|
		//	|	myObj=dijit.byId('myForm').get('value');

		// HTML <FORM> attributes

		// name: String?
		//		Name of form for scripting.
		name: "",

		// action: String?
		//		Server-side form handler.
		action: "",

		// method: String?
		//		HTTP method used to submit the form, either "GET" or "POST".
		method: "",

		// encType: String?
		//		Encoding type for the form, ex: application/x-www-form-urlencoded.
		encType: "",

		// accept-charset: String?
		//		List of supported charsets.
		"accept-charset": "",

		// accept: String?
		//		List of MIME types for file upload.
		accept: "",

		// target: String?
		//		Target frame for the document to be opened in.
		target: "",

		templateString: "<form data-dojo-attach-point='containerNode' data-dojo-attach-event='onreset:_onReset,onsubmit:_onSubmit' ${!nameAttrSetting}></form>",

		postMixInProperties: function(){
			// Setup name=foo string to be referenced from the template (but only if a name has been specified)
			// Unfortunately we can't use _setNameAttr to set the name due to IE limitations, see #8660
			this.nameAttrSetting = this.name ? ("name='" + this.name + "'") : "";
			this.inherited(arguments);
		},

		execute: function(/*Object*/ /*===== formContents =====*/){
			// summary:
			//		Deprecated: use submit()
			// tags:
			//		deprecated
		},

		onExecute: function(){
			// summary:
			//		Deprecated: use onSubmit()
			// tags:
			//		deprecated
		},

		_setEncTypeAttr: function(/*String*/ value){
			this.encType = value;
			domAttr.set(this.domNode, "encType", value);
			if(has("ie")){ this.domNode.encoding = value; }
		},

		reset: function(/*Event?*/ e){
			// summary:
			//		restores all widget values back to their init values,
			//		calls onReset() which can cancel the reset by returning false

			// create fake event so we can know if preventDefault() is called
			var faux = {
				returnValue: true, // the IE way
				preventDefault: function(){ // not IE
							this.returnValue = false;
						},
				stopPropagation: function(){},
				currentTarget: e ? e.target : this.domNode,
				target: e ? e.target : this.domNode
			};
			// if return value is not exactly false, and haven't called preventDefault(), then reset
			if(!(this.onReset(faux) === false) && faux.returnValue){
				this.inherited(arguments, []);
			}
		},

		onReset: function(/*Event?*/ /*===== e =====*/){
			// summary:
			//		Callback when user resets the form. This method is intended
			//		to be over-ridden. When the `reset` method is called
			//		programmatically, the return value from `onReset` is used
			//		to compute whether or not resetting should proceed
			// tags:
			//		callback
			return true; // Boolean
		},

		_onReset: function(e){
			this.reset(e);
			event.stop(e);
			return false;
		},

		_onSubmit: function(e){
			var fp = this.constructor.prototype;
			// TODO: remove this if statement beginning with 2.0
			if(this.execute != fp.execute || this.onExecute != fp.onExecute){
				kernel.deprecated("dijit.form.Form:execute()/onExecute() are deprecated. Use onSubmit() instead.", "", "2.0");
				this.onExecute();
				this.execute(this.getValues());
			}
			if(this.onSubmit(e) === false){ // only exactly false stops submit
				event.stop(e);
			}
		},

		onSubmit: function(/*Event?*/ /*===== e =====*/){
			// summary:
			//		Callback when user submits the form.
			// description:
			//		This method is intended to be over-ridden, but by default it checks and
			//		returns the validity of form elements. When the `submit`
			//		method is called programmatically, the return value from
			//		`onSubmit` is used to compute whether or not submission
			//		should proceed
			// tags:
			//		extension

			return this.isValid(); // Boolean
		},

		submit: function(){
			// summary:
			//		programmatically submit form if and only if the `onSubmit` returns true
			if(!(this.onSubmit() === false)){
				this.containerNode.submit();
			}
		}
	});
});

},
'gridx/modules/Persist':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/_base/json",
	"dojo/_base/unload",
	"dojo/cookie",
	"../core/_Module"
], function(declare, array, lang, json, unload, cookie, _Module){

	return declare(/*===== "gridx.modules.Persist", =====*/_Module, {
		// summary:
		//		Provide a mechanism to persist various grid features when the grid is destroyed,
		//		so that when a new grid with the same id (or the same persist key) is created,
		//		all these features will be restored.
		//		By default use cookie, but users can also provide custom put and get functions.
		//		Note: since dojox.storage is still experimental, and with HTML5 we will hardly need
		//		things like gears or behavior storage, so we aren't supporting dojox.storage by default.
		name: 'persist',
	
		constructor: function(grid){
			// summary:
			//		All initializations, if any, must be done in the constructor, instead of the load function.
			// grid: Object
			//		The grid itself.
			var t = this,
				gridDestroy = grid.destroy;
			//Initialize arguments
			t.arg('key', window.location + '/' + grid.id, function(arg){
				return arg;
			});
			t._persistedList = {};
			// Save states when grid destroy or window unload
			grid.destroy = function(){
				t.save();
				gridDestroy.call(grid);
			};
			//column state register/restore
			t._restoreColumnState();
			unload.addOnWindowUnload(function(){
				t.save();
			});
		},
	
		getAPIPath: function(){
			return {
				persist: this
			};
		},
		
		//Public---------------------------------------------------------
		
		// enabled: Boolean
		//		Whether this module is enabled (also means whether all registered features are persistable).
		enabled: true,

		// options:
		//		Options meaningful to the persist mechanism. By default it mean the cookie options.
		options: null,
		
		// key: String
		//		This is the storage key of this grid. If not provided (by default), the grid id is used as the key.
		//		This property is essential when a grid with a different id wants to load from this storage.
		key: '',
	
		put: function(key, value, options){
			// summary:
			//		This is NOT a public method, but users can provide their own to override it.
			//		This function is called when finally saving things into some kind of storage.
			// key: String
			//		The persist key of this grid.
			// value: Object
			//		A JSON object, containing everything we want to persist for this grid.
			if(value && lang.isObject(value)){
				cookie(key, json.toJson(value), options);
			}else{
				cookie(key, null, {expires: -1});
			}
		},
	
		get: function(key){
			// summary:
			//		This is NOT a public method, but users can provide their own to override it.
			//		This function is called when loading things from storage.
			// return:
			//		Then things we stored before.
			return json.fromJson(cookie(key));	//Object
		},
	
		registerAndLoad: function(name, saver, scope){
			// summary:
			//		Register a feature to be persisted, and then load (return) its contents.
			// name: String
			//		A unique name of the feature to be persisted.
			// saver: Function() return object
			//		A function to be called when persisting the grid.
			// return:
			//		The loaded contents of the given feature.
			this._persistedList[name] = {
				saver: saver,
				scope: scope,
				enabled: true
			};
			var content = this.get(this.arg('key'));
			return content ? content[name] : null;	//Object
		},

		features: function(){
			// summary:
			//		Get the names of all the persistable features.
			//		These names can be used in enable(), disable() or isEnabled() methods.
			// return:
			//		An array of persistable feature names.
			var list = this._persistedList,
				features = [],
				name;
			for(name in list){
				if(list.hasOwnProperty(name)){
					features.push(name);
				}
			}
			return features;	//String[]
		},
	
		enable: function(name){
			// summary:
			//		Enable persistance of the given feature, that is, will persist this feature when the save function
			//		is called. If name is not provided (undefined or null), then enable all registered features.
			// name: String
			//		Name of a feature.
			this._setEnable(name, 1);
		},
	
		disable: function(name){
			// summary:
			//		Disable persistance of the given feature, that is, will NOT persist this feature when the save
			//		function is called. If name is not provided (undefined or null), then disable all registered features.
			// name: String
			//		Name of a feature.
			this._setEnable(name, 0);
		},
	
		isEnabled: function(name){
			// summary:
			//		Check whether a feature is enabled or not.
			// name: String
			//		Name of a feature. If omitted, means every feature.
			// return:
			//		Whether this feature (or every feature) is enabled.
			var feature = this._persistedList[name];
			if(feature){
				return feature.enabled;	//Boolean
			}
			return name ? false : this.arg('enabled');	//Boolean
		},
	
		save: function(){
			// summary:
			//		Save all the enabled features.
			var t = this,
				contents = null;
			if(t.arg('enabled')){
				var name,
					feature,
					list = t._persistedList;
				contents = {};
				for(name in list){
					feature = list[name];
					if(feature.enabled){
						contents[name] = feature.saver.call(feature.scope || lang.global);
					}
				}
			}
			t.put(t.arg('key'), contents, t.arg('options'));
		},
	
		//Private--------------------------------------------------------
		//_persistedList: null,
		
		_setEnable: function(name, enabled){
			var list = this._persistedList;
			enabled = !!enabled;
			if(list[name]){
				list[name].enabled = enabled;
			}else if(!name){
				for(name in list){
					list[name].enabled = enabled;
				}
				this.enabled = enabled;
			}
		},
		
		//For column state restore---------------------
		_restoreColumnState: function(){
			var t = this,
				grid = t.grid,
				col, cols = [],
				columns = t.registerAndLoad('column', t._columnStateSaver, t);
			if(lang.isArray(columns)){
				array.forEach(columns, function(col){
					array.some(grid._columns, function(c, i){
						if(c.id == col.id){
							c = cols[col.index] = grid._columns[i];
							c.declaredWidth = c.width = col.width;
							return true;
						}
					});
				});
				grid.setColumns(cols);
			}
		},
		
		_columnStateSaver: function(){
			return array.map(this.grid._columns, function(c){
				return {
					id: c.id,
					index: c.index,
					width: c.width
				};
			});
		}
	});
});

},
'dijit/_base/popup':function(){
define([
	"dojo/dom-class", // domClass.contains
	"../popup",
	"../BackgroundIframe"	// just loading for back-compat, in case client code is referencing it
], function(domClass, popup){

// module:
//		dijit/_base/popup
// summary:
//		Old module for popups, new code should use dijit/popup directly


// Hack support for old API passing in node instead of a widget (to various methods)
var origCreateWrapper = popup._createWrapper;
popup._createWrapper = function(widget){
	if(!widget.declaredClass){
		// make fake widget to pass to new API
		widget = {
			_popupWrapper: (widget.parentNode && domClass.contains(widget.parentNode, "dijitPopup")) ?
				widget.parentNode : null,
			domNode: widget,
			destroy: function(){}
		};
	}
	return origCreateWrapper.call(this, widget);
};

// Support old format of orient parameter
var origOpen = popup.open;
popup.open = function(/*dijit.popup.__OpenArgs*/ args){
	// Convert old hash structure (ex: {"BL": "TL", ...}) of orient to format compatible w/new popup.open() API.
	// Don't do conversion for:
	//		- null parameter (that means to use the default positioning)
	//		- "R" or "L" strings used to indicate positioning for context menus (when there is no around node)
	//		- new format, ex: ["below", "above"]
	//		- return value from deprecated dijit.getPopupAroundAlignment() method,
	//			ex: ["below", "above"]
	if(args.orient && typeof args.orient != "string" && !("length" in args.orient)){
		var ary = [];
		for(var key in args.orient){
			ary.push({aroundCorner: key, corner: args.orient[key]});
		}
		args.orient = ary;
	}

	return origOpen.call(this, args);
};

return popup;
});

},
'url:dijit/form/templates/Spinner.html':"<div class=\"dijit dijitReset dijitInline dijitLeft\"\n\tid=\"widget_${id}\" role=\"presentation\"\n\t><div class=\"dijitReset dijitButtonNode dijitSpinnerButtonContainer\"\n\t\t><input class=\"dijitReset dijitInputField dijitSpinnerButtonInner\" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"presentation\"\n\t\t/><div class=\"dijitReset dijitLeft dijitButtonNode dijitArrowButton dijitUpArrowButton\"\n\t\t\tdata-dojo-attach-point=\"upArrowNode\"\n\t\t\t><div class=\"dijitArrowButtonInner\"\n\t\t\t\t><input class=\"dijitReset dijitInputField\" value=\"&#9650;\" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"presentation\"\n\t\t\t\t\t${_buttonInputDisabled}\n\t\t\t/></div\n\t\t></div\n\t\t><div class=\"dijitReset dijitLeft dijitButtonNode dijitArrowButton dijitDownArrowButton\"\n\t\t\tdata-dojo-attach-point=\"downArrowNode\"\n\t\t\t><div class=\"dijitArrowButtonInner\"\n\t\t\t\t><input class=\"dijitReset dijitInputField\" value=\"&#9660;\" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"presentation\"\n\t\t\t\t\t${_buttonInputDisabled}\n\t\t\t/></div\n\t\t></div\n\t></div\n\t><div class='dijitReset dijitValidationContainer'\n\t\t><input class=\"dijitReset dijitInputField dijitValidationIcon dijitValidationInner\" value=\"&#935;\" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"presentation\"\n\t/></div\n\t><div class=\"dijitReset dijitInputField dijitInputContainer\"\n\t\t><input class='dijitReset dijitInputInner' data-dojo-attach-point=\"textbox,focusNode\" type=\"${type}\" data-dojo-attach-event=\"onkeypress:_onKeyPress\"\n\t\t\trole=\"spinbutton\" autocomplete=\"off\" ${!nameAttrSetting}\n\t/></div\n></div>\n",
'ibm/usmi/widgets/wizard/DirectorWizard':function(){
require({cache:{
'url:ibm/usmi/widgets/wizard/templates/DirectorWizardNested.html':"<div class=\"tiv-wiz\" role=\"application\">\r\n<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n<div class=\"tiv-wiz\" role=\"application\">\r\n  <div dojoType=\"dijit.layout.BorderContainer\" design=\"headline\" gutters=\"false\" class=\"tiv-wiz-outer\" dojoAttachPoint=\"_outerContainer\">\r\n        <div dojoType=\"dijit.layout.BorderContainer\" gutters=\"false\" liveSplitters=\"false\" design=\"sidebar\" region=\"center\" style=\"width:100%;height:100%;\" dojoAttachPoint=\"_borderArea\">\r\n             <div dojoType=\"dijit.layout.ContentPane\" region=\"leading\" splitter=\"false\" class=\"tiv-wiz-toc\" dojoAttachPoint=\"_tocArea\">\r\n                  <ol dojoAttachPoint=\"_toc\"></ol>\r\n             </div>\r\n             <div dojoType=\"dijit.layout.StackContainer\" region=\"center\" splitter=\"false\" dojoAttachPoint=\"_pageContainer\" class=\"tiv-wiz-work-area\"> \r\n                  <div dojoAttachPoint=\"containerNode\"></div>\r\n             </div>\r\n        </div>\r\n\t\t<div dojoType=\"dijit.layout.ContentPane\" region=\"bottom\" splitter=\"false\" class=\"tiv-wiz-footer\" dojoAttachPoint=\"_buttonArea\">\r\n\t\t\t<!-- We want the director wizard step buttons to be left-justified -->\t\t\r\n            <div class=\"director-wiz-step-buttons\" style=\"display: inline;\" data-dojo-attach-point=\"_buttons\">\r\n\t\t\t\t<button role=\"button\" type=\"button\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickBack\" data-dojo-attach-point=\"_backButton\">${backLabel}</button>\r\n\t\t\t\t<button role=\"button\" type=\"button\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickNext\" data-dojo-attach-point=\"_nextButton\">${nextLabel}</button>\r\n\t\t\t</div>\t\t\t\r\n\t\t\t<!--  we want the director wizard commit buttons to be right-justified -->\r\n\t\t\t<div class=\"director-wiz-commit-buttons\" style=\"display:inline;\" data-dojo-attach-point=\"_buttons\">\r\n\t\t\t\t<button role=\"button\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickFinish\" data-dojo-props='type:\"button\"' data-dojo-attach-point=\"_finishButton\">${finishLabel}</button>\r\n\t\t\t\t<button role=\"button\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickCancel\"  data-dojo-props='type:\"button\"' data-dojo-attach-point=\"_cancelButton\">${cancelLabel}</button>\r\n\t\t\t</div> \r\n         </div>\r\n   </div>\r\n</div>\r\n",
'url:ibm/usmi/widgets/wizard/templates/DirectorWizard.html':"<div class=\"tiv-wiz\" role=\"application\">\r\n<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n\t<!-- we were using dijit.layout.BorderContainer -> now changed to DirectorBorderContainer -->\r\n<div class=\"tiv-wiz\" role=\"application\">\r\n\t<div dojoType=\"dijit.layout.BorderContainer\" design=\"headline\" gutters=\"false\" class=\"tiv-wiz-outer\" dojoAttachPoint=\"_outerContainer\">\r\n\t\t<div dojoType=\"dijit.layout.BorderContainer\" gutters=\"false\" liveSplitters=\"false\" design=\"sidebar\" region=\"center\" style=\"width:100%;height:100%;\" dojoAttachPoint=\"_borderArea\">\r\n\t\t\t<div dojoType=\"dijit.layout.ContentPane\" region=\"leading\" splitter=\"false\" class=\"tiv-wiz-toc\" dojoAttachPoint=\"_tocArea\">\r\n\t\t\t\t<ol dojoAttachPoint=\"_toc\"></ol>\r\n\t\t\t</div>\r\n\t\t\t<div dojoType=\"dijit.layout.StackContainer\" region=\"center\" splitter=\"false\" dojoAttachPoint=\"_pageContainer\" class=\"tiv-wiz-work-area\">\r\n\t\t\t<form id=\"${id}_form\" dojoAttachPoint=\"containerNode\" action=\"${action}\" method=\"${method}\"></form>              \r\n\t\t\t</div>\r\n\t\t</div>\r\n\t\t<div dojoType=\"dijit.layout.ContentPane\" region=\"bottom\" splitter=\"false\" class=\"tiv-wiz-footer\" dojoAttachPoint=\"_buttonArea\">\r\n\t\t\t<!-- We want the director wizard step buttons to be left-justified -->\r\n\t\t\t<div class=\"director-wiz-step-buttons\" style=\"display: inline;\" data-dojo-attach-point=\"_buttons\">\r\n\t\t\t\t<button role=\"button\" type=\"button\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickBack\" data-dojo-attach-point=\"_backButton\">${backLabel}</button>\r\n\t\t\t\t<button role=\"button\" type=\"button\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickNext\" data-dojo-attach-point=\"_nextButton\">${nextLabel}</button>\r\n\t\t\t</div>\t\t\t\r\n\t\t\t<!--  we want the director wizard commit buttons to be right-justified -->\r\n\t\t\t<div class=\"director-wiz-commit-buttons\" style=\"display:inline;\" data-dojo-attach-point=\"_buttons\">\r\n\t\t\t\t<button role=\"button\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickFinish\" data-dojo-props='type:\"button\"' data-dojo-attach-point=\"_finishButton\">${finishLabel}</button>\r\n\t\t\t\t<button role=\"button\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickCancel\" data-dojo-props='type:\"button\"' data-dojo-attach-point=\"_cancelButton\">${cancelLabel}</button>\r\n\t\t\t</div> \r\n\t\t</div>\r\n\t</div>\r\n</div>\r\n"}});
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
/*global define */
define("ibm/usmi/widgets/wizard/DirectorWizard", [ "dojo/_base/declare",
	"dojo/_base/window",
	"dojo/dom-construct",
	"dojoe/wizard/Wizard",
	"dojoe/wizard/PageContainer",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	"dijit/layout/StackContainer",
	"dijit/layout/BorderContainer",
	"dijit/layout/ContentPane",
	"dijit/form/Button",
	"dijit/form/RadioButton",
	"dijit/form/NumberSpinner",
	"dijit/form/CheckBox",
	"dijit/form/ValidationTextBox",
	"dijit/form/DateTextBox",
	"dijit/form/Select",
	"dijit/form/MultiSelect",
	"dojox/form/TimeSpinner",
	"dojo/text!./templates/DirectorWizardNested.html",
	"dojo/text!./templates/DirectorWizard.html",
	"ibm/usmi/widgets/wizard/DirectorBorderContainer",
	"require" ], function (
	declare, window, construct, Wizard, PageContainer, _TemplatedMixin, _WidgetsInTemplateMixin, StackContainer,
	BorderContainer, ContentPane, Button, RadioButton, NumberSpinner, Checkbox, ValidationTextBox, DateTextBox, Select,
	MultiSelect, TimeSpinner, DirectorWizardNestedTemplate, DirectorWizardTemplate, DirectorBorderContainer, require) {
	return declare([ Wizard,
		PageContainer,
		_TemplatedMixin,
		_WidgetsInTemplateMixin ], {
		// TODO:  it looks like the templateString needs to be set immediately....so the current logic I have in postMixInProperties does not work...Need
		//        to find some other way to set the templateString when using nested wizards....
		templateString : "", 

		// Image Path to Director Wizard css
		CSS_PATH_DIRECTOR_WIZARD : "js/ibm/usmi/widgets/wizard/themes/oneui/DirectorWizard.css", // DirectorWizard css path
		
		// Image Path to Wizard css
		CSS_PATH_WIZARD : "js/dojoe/wizard/themes/oneui/Wizard.css", // Wizard css path

		// override postMixInProperties to use our own Director-specific templates (navigation buttons split -- step buttons to the right/commit buttons to the left)
		postMixInProperties : function () {
			// first call my parent
			this.inherited(arguments);
			
			// do NOT load the wizard styles dynamically -- they are now part of the framework.css
			// this.load_wizardStyles(); // load the director wizard css 

			// Apply the Director specific wizard template....
			
			if (this.nested) {
				//templateString: DirectorWizardNested;
				this.templateString = DirectorWizardNestedTemplate;
			} else {
				//templateString: DirectorWizard;
				this.templateString = DirectorWizardTemplate;
			}
			

		},

		OnCancel : function (
			wiz) {
			
			// do NOT remove the wizard styles...
			/*
			var element = document.getElementById("DirectorWizardStyles");			
			element.parentNode.removeChild(element);

			element = document.getElementById("WizardStyles");
			element.parentNode.removeChild(element);
			*/
			
			this.inherited(arguments);
		},
		
		
		load_wizardStyles : function () {

			// jdwald 8/29 -- both of these CSS seem to be required....the top one allows the navigation buttons to have the director-specific values
			//                while the next one applies the wizard format styles....Not sure if we can merge these two....
			var styleSheetLink = document.createElement('link');
			styleSheetLink.id = 'DirectorWizardStyles';
			styleSheetLink.rel = 'stylesheet';
			styleSheetLink.type = 'text/css';
			styleSheetLink.href = this.CSS_PATH_DIRECTOR_WIZARD;
			window.doc.getElementsByTagName("head")[0].appendChild(styleSheetLink);

			styleSheetLink = document.createElement('link');
			styleSheetLink.id = 'WizardStyles';			
			styleSheetLink.rel = 'stylesheet';
			styleSheetLink.type = 'text/css';
			styleSheetLink.href = this.CSS_PATH_WIZARD;					
			window.doc.getElementsByTagName("head")[0].appendChild(styleSheetLink);
			
		}
	});
});

},
'ibm/usmi/widgets/TreeTopperLinks':function(){
require({cache:{
'url:ibm/usmi/widgets/templates/TreeTopperLink.html':"<a class=\"topperLink topperLinkAnchor\" data-dojo-attach-point=\"topperAnchor\" role=\"link\">\r\n<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n\t<div class=\"pad5t pad10l\">\r\n\t      <img class=\"topperLinkIcon sprite\" data-dojo-attach-point=\"iconAttachPoint\"></img>\r\n\t      <div class=\"topperLinkLabel\" data-dojo-attach-point=\"labelAttachPoint\"></div>\r\n\t</div>\r\n</a>\r\n"}});
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/widgets/TreeTopperLinks", ["dojo/_base/kernel",
        "dojo/_base/declare",
        "dojo/topic",
        "dijit/_WidgetBase",
        "dijit/_TemplatedMixin",
        "dojo/text!./templates/TreeTopperLink.html",
        "dijit/focus",
        // NextGen imports
        "ibm/usmi/util/BaseWidget",
        "ibm/usmi/widgets/util/TextAbbr",
        "ibm/usmi/widgets/util/Constants",
        "ibm/usmi/core/utils/Logger"],
    function (dojo, declare, topic, _WidgetBase, _TemplatedMixin, TreeTopperLinkTemplate, focus,
        BaseWidget, TextAbbr, Constants, Logger) {

    return declare([_WidgetBase, _TemplatedMixin, BaseWidget], {

        // Template info
        templateString: TreeTopperLinkTemplate,

        rootId: "",

        topperId: "",

        /**
         * NLS translated label for this node which the user will see
         * @param
         */
        label: "",


        /**
         * fully qualified class name of a 16x16 pixel icon to display by this node
         * @param
         */
        icon: "",


        /**
         * unique in conjunction with the node which will be sent out in
         * the topic.publish when the node is pressed
         * @param
         */
        taskWidgetId: "",


        textAbbr: null,

        _imagePath: null,

        // Image Path
        IMAGE_PATH: "ibm/usmi/widgets/themes/toppers/images/",

        /**
         * Whether this link will attempt to pre-select itself when initialzied.  Only one link
         * can be selected at a time but multiple can try... only one will succeed. Use with care
         * @public
         */
        selected: false,

        // the logger
        _treeTreeToppersLogger: new Logger("ibm.usmi.widgets", "TreeTopperLinks"),

        constructor: function() {
            this._imagePath = require.toUrl(this.IMAGE_PATH);
        },

        postCreate: function() {
            this.inherited(arguments);
            this.connect(this.topperAnchor, Constants.ON_CLICK, this._handleClick);
        },

        startup: function() {
            this.inherited(arguments);

            // check window location's hash to determine whether this link should be selected
            if (this.getUrlHashParm(Constants.HASH_TOPPER_SELECTION) === this.topperId) {
                this.selected = true;
            }

            this.update();
        },

        buildRendering: function() {
            this.inherited(arguments);
        },

        update: function(args) {
            dojo.mixin(this, args);
            this.updateLabel();
            this.updateIcon();
            this.selectionCheck();

        },

        updateIcon: function() {
           // dojo.attr(this.iconAttachPoint, "src", this._imagePath + this.icon);
            // currently setting complete path in dashbar.json, doing it that way for future
            // toppers can point to images outside topper path
            dojo.attr(this.iconAttachPoint, "src", this.icon);

        },

        updateLabel: function() {
            var formattedLabel = this.label;

            //update the textAbbr if it exists
            if(this.textAbbr){
                this.textAbbr.update(formattedLabel);
            } else {//create textAbbr
                this.textAbbr = new TextAbbr({text: formattedLabel, maxLength: 34, format: " (x)"}, this.labelAttachPoint);
            }
        },


        focus: function(){
            focus.focus(this.topperAnchor);
        },

        getLabel: function() {
            return this.label;
        },

        select: function(select) {
            this.selected = select;
            dojo.toggleClass(this.domNode, "taskLink-selected", select);
        },

        selectionCheck: function() {
            if(this.selected) {
             //   this.getParent().setLink(this);
                this._handleClick();
            }
        },

        _handleClick: function(evt)
        {
            if (this._treeTreeToppersLogger.isFinest()) {
                this._treeTreeToppersLogger.entering("", "_handleClick");
            }

            // call to parent to uncheck all others and then check me
            this.getParent().setLink(this);

            // tell tree I'm in charge
            topic.publish("TREETOPPERINCHARGE", "true"); //tell workarea to switch it up


            var hashObj = this.getUrlHashObj();
            hashObj.treeTopperId = this.topperId;

            // remove the sourcetoken from the hash object, there can be only one
            if(hashObj['sourceToken']) {
                delete hashObj.sourceToken;
            }

            dojo.hash(dojo.objectToQuery(hashObj), true); // set

            var retargetInfoObject = {};
            retargetInfoObject[Constants.HASH_TASK_SELECTION] = this.taskWidgetId;
            retargetInfoObject[Constants.COLLECTION] = this.collectionID;
            topic.publish(Constants.TOPIC_RETARGET_INFO, retargetInfoObject); //tell workarea to switch it up

            if (this._treeTreeToppersLogger.isFinest()) {
                this._treeTreeToppersLogger.exiting("", "_handleClick");
            }
          }
    });
});
},
'url:ibm/usmi/widgets/table/templates/ConfigureColumnsDialog.html':"<div class=\"oneui\">\r\n<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n\t<div data-dojo-type=\"idx.oneui.Dialog\" data-dojo-attach-point=\"dialog\"\r\n\t\tclass=\"configColumnsDialog\" title=\"${resources_.CONFIGURE_SETTINGS}\"\r\n\t\ttabindex=0 role=\"dialog\"\r\n\t\tdata-dojo-props='\r\n  \t\t\t\tcloseButtonLabel: this.nls.CANCEL,\r\n\t\t\t\tbuttons: this.actionButtons,\r\n\t\t\t\ttitle: this.nls.CONFIG_COLUMNS\r\n\t'>\r\n\t\t<table class=\"configColumnsDialogTable\" border=\"0\" cellspacing=\"0\"\r\n\t\t\tcellpadding=\"0\" role=\"presentation\" style=\"width: 670px;\">\r\n\t\t\t<tr>\r\n\t\t\t\t<td colspan='2'><label>${nls.CHOOSE_COLUMNS}</label> <br />\r\n\t\t\t\t\t<div style=\"display: inline;\">\r\n\t\t\t\t\t\t<table>\r\n\t\t\t\t\t\t\t<tr>\r\n\t\t\t\t\t\t\t\t<td><label>${nls.REORDER_COLUMNS}</label></td>\r\n\t\t\t\t\t\t\t</tr>\r\n\t\t\t\t\t\t</table>\r\n\t\t\t\t\t</div></td>\r\n\t\t\t</tr>\r\n\t\t\t<tr style=\"vertical-align: bottom;\">\r\n\t\t\t\t<td>\r\n\t\t\t\t\t<div class=\"columnsActionButtons tivTable\">\r\n\r\n\t\t\t\t\t\t<table border=\"0\" cellspacing=\"7\" cellpadding=\"0\"\r\n\t\t\t\t\t\t\trole=\"presentation\">\r\n\t\t\t\t\t\t\t<tbody>\r\n\t\t\t\t\t\t\t\t<tr>\r\n\t\t\t\t\t\t\t\t\t<!--<td><span data-dojo-attach-point=\"selectAllButton\"\r\n\t\t\t\t\t\t\t\t\t\trole=\"button\"></td>\r\n\t\t\t\t\t\t\t\t\t<td><span data-dojo-attach-point=\"deselectAllButton\"\r\n\t\t\t\t\t\t\t\t\t\trole=\"button\"></td>-->\r\n\t\t\t\t\t\t\t\t\t<td>\r\n\t\t\t\t\t\t\t\t\t\t<div class=\"resetToDefaultBtn\">\r\n\t\t\t\t\t\t\t\t\t\t\t<span data-dojo-attach-point=\"resetToDefaultsButton\"\r\n\t\t\t\t\t\t\t\t\t\t\t\trole=\"button\">\r\n\t\t\t\t\t\t\t\t\t\t</div>\r\n\t\t\t\t\t\t\t\t\t</td>\r\n\t\t\t\t\t\t\t\t</tr>\r\n\t\t\t\t\t\t\t</tbody>\r\n\t\t\t\t\t\t</table>\r\n\t\t\t\t\t\t<label class=\"availableColumnsLbl\">${nls.AVAILABLE_COLUMNS}</label>\r\n\t\t\t\t\t</div>\r\n\t\t\t\t</td>\r\n\t\t\t\t<td class=\"selectedColumnsLbl\">${nls.SELECTED_COLUMNS}</td>\r\n\t\t\t</tr>\r\n\t\t\t<tr>\r\n\t\t\t\t<td>\r\n\t\t\t\t\t<div class=\"availableColumns dialog\" tabStrip=\"false\"\r\n\t\t\t\t\t\tid=\"${id}tabs\" data-dojo-type=\"dijit.layout.LayoutContainer\">\r\n\t\t\t\t\t\t<div data-dojo-type=\"dijit.layout.ContentPane\"\r\n\t\t\t\t\t\t\tstyle=\"width: 96%; overflow-y: hidden;\">\r\n\t\t\t\t\t\t\t<div data-dojo-attach-point=\"loadingDiv\">${nls.LOADING}</div>\r\n\t\t\t\t\t\t\t<div id=\"${id}columnsVisibleDivContainer\"\r\n\t\t\t\t\t\t\t\tdata-dojo-attach-point=\"columnsDiv\" style=\"display: none;\">\r\n\t\t\t\t\t\t\t</div>\r\n\t\t\t\t\t\t</div>\r\n\t\t\t\t\t</div>\r\n\t\t\t\t</td>\r\n\t\t\t\t<td style=\"padding-left: 20px\">\r\n\t\t\t\t\t<div class=\"selectedColumns dialog\" tabStrip=\"false\"\r\n\t\t\t\t\t\tid=\"${id}_selected_tab\"\r\n\t\t\t\t\t\tdata-dojo-type=\"dijit.layout.LayoutContainer\">\r\n\t\t\t\t\t\t<div data-dojo-type=\"dijit.layout.ContentPane\"\r\n\t\t\t\t\t\t\tid=\"${id}selectedColumnsdiv\" style=\"width: 96%; height: 100%\"></div>\r\n\t\t\t\t\t</div>\r\n\t\t\t\t</td>\r\n\t\t\t</tr>\r\n\t\t</table>\r\n\t</div>\r\n</div>",
'dojox/validate/_base':function(){
define([
	"dojo/_base/lang",
	"dojo/regexp", // dojo core expressions
	"dojo/number", // dojo number expressions
	"./regexp" // additional expressions
], function(lang, regexp, number, xregexp) {

	var validate = lang.getObject("dojox.validate", true);
	/*=====
		validate = dojox.validate;
	=====*/

validate.isText = function(/*String*/value, /*Object?*/flags){
	// summary:
	//	Checks if a string has non whitespace characters.
	//	Parameters allow you to constrain the length.
	//
	// value: A string
	// flags: {length: Number, minlength: Number, maxlength: Number}
	//    flags.length  If set, checks if there are exactly flags.length number of characters.
	//    flags.minlength  If set, checks if there are at least flags.minlength number of characters.
	//    flags.maxlength  If set, checks if there are at most flags.maxlength number of characters.
	
	flags = (typeof flags == "object") ? flags : {};
	
	// test for text
	if(/^\s*$/.test(value)){ return false; } // Boolean
	
	// length tests
	if(typeof flags.length == "number" && flags.length != value.length){ return false; } // Boolean
	if(typeof flags.minlength == "number" && flags.minlength > value.length){ return false; } // Boolean
	if(typeof flags.maxlength == "number" && flags.maxlength < value.length){ return false; } // Boolean
	
	return true; // Boolean

};

validate._isInRangeCache = {};
validate.isInRange = function(/*String*/value, /*Object?*/flags){
	// summary:
	//	Validates whether a string denoting a number
	//	is between a max and min.
	//
	// value: A string
	// flags: {max:Number, min:Number, decimal:String}
	//    flags.max  A number, which the value must be less than or equal to for the validation to be true.
	//    flags.min  A number, which the value must be greater than or equal to for the validation to be true.
	//    flags.decimal  The character used for the decimal point.  Default is ".".
	
	value = number.parse(value, flags);
	if(isNaN(value)){
		return false; // Boolean
	}
    
	// assign default values to missing paramters
	flags = (typeof flags == "object") ? flags : {};
	var max = (typeof flags.max == "number") ? flags.max : Infinity,
		min = (typeof flags.min == "number") ? flags.min : -Infinity,
		dec = (typeof flags.decimal == "string") ? flags.decimal : ".",
	
		cache = validate._isInRangeCache,
		cacheIdx = value + "max" + max + "min" + min + "dec" + dec
	;
	if(typeof cache[cacheIdx] != "undefined"){
		return cache[cacheIdx];
	}

	cache[cacheIdx] = !(value < min || value > max);
	return cache[cacheIdx]; // Boolean

};

validate.isNumberFormat = function(/* String */value, /* Object? */flags){
	// summary: Validates any sort of number based format
	//
	// description:
	//		Validates any sort of number based format. Use it for phone numbers,
	//		social security numbers, zip-codes, etc. The value can be validated
	//		against one format or one of multiple formats.
	//
	// Format Definition
	// |   #        Stands for a digit, 0-9.
	// |   ?        Stands for an optional digit, 0-9 or nothing.
	//    All other characters must appear literally in the expression.
	//
	// example:
	// |  "(###) ###-####"       ->   (510) 542-9742
	// |  "(###) ###-#### x#???" ->   (510) 542-9742 x153
	// |  "###-##-####"          ->   506-82-1089       i.e. social security number
	// |  "#####-####"           ->   98225-1649        i.e. zip code
	//
	// value: A string
	//
	// flags: Object?
	//		FIXME: make pseudo-object for this
	//		format: String
	//
	//    flags.format  A string or an Array of strings for multiple formats.
	//
	// example:
	// | // returns true:
	// | dojox.validate.isNumberFormat("123-45", { format:"###-##" });
	//
	// example:
	// 		Check Multiple formats:
	// |	dojox.validate.isNumberFormat("123-45", {
	// |		format:["### ##","###-##","## ###"]
	// |	});
	//

	var re = new RegExp("^" + xregexp.numberFormat(flags) + "$", "i");
	return re.test(value); // Boolean
};

validate.isValidLuhn = function(/* String */value){
	// summary: Validate a String value against the Luhn algorithm.
	// description:
	//		Validate a String value against the Luhn algorithm to verify
	//		its integrity.
	
	var sum = 0, parity, curDigit;
	if(!lang.isString(value)){
		value = String(value);
	}
	value = value.replace(/[- ]/g,''); //ignore dashes and whitespaces
	parity = value.length % 2;

	for(var i = 0; i < value.length; i++){
		curDigit = parseInt(value.charAt(i));
		if(i % 2 == parity){
			curDigit *= 2;
		}
		if(curDigit > 9){
			curDigit -= 9;
		}
		sum += curDigit;
	}
	return !(sum % 10); // Boolean
};

return validate;

});

},
'dojo/uacss':function(){
define(["./dom-geometry", "./_base/lang", "./ready", "./_base/sniff", "./_base/window"],
	function(geometry, lang, ready, has, baseWindow){
	// module:
	//		dojo/uacss
	// summary:
	//		Applies pre-set CSS classes to the top-level HTML node, based on:
	//			- browser (ex: dj_ie)
	//			- browser version (ex: dj_ie6)
	//			- box model (ex: dj_contentBox)
	//			- text direction (ex: dijitRtl)
	//
	//		In addition, browser, browser version, and box model are
	//		combined with an RTL flag when browser text is RTL. ex: dj_ie-rtl.

	var
		html = baseWindow.doc.documentElement,
		ie = has("ie"),
		opera = has("opera"),
		maj = Math.floor,
		ff = has("ff"),
		boxModel = geometry.boxModel.replace(/-/,''),

		classes = {
			"dj_ie": ie,
			"dj_ie6": maj(ie) == 6,
			"dj_ie7": maj(ie) == 7,
			"dj_ie8": maj(ie) == 8,
			"dj_ie9": maj(ie) == 9,
			"dj_quirks": has("quirks"),
			"dj_iequirks": ie && has("quirks"),

			// NOTE: Opera not supported by dijit
			"dj_opera": opera,

			"dj_khtml": has("khtml"),

			"dj_webkit": has("webkit"),
			"dj_safari": has("safari"),
			"dj_chrome": has("chrome"),

			"dj_gecko": has("mozilla"),
			"dj_ff3": maj(ff) == 3
		}; // no dojo unsupported browsers

	classes["dj_" + boxModel] = true;

	// apply browser, browser version, and box model class names
	var classStr = "";
	for(var clz in classes){
		if(classes[clz]){
			classStr += clz + " ";
		}
	}
	html.className = lang.trim(html.className + " " + classStr);

	// If RTL mode, then add dj_rtl flag plus repeat existing classes with -rtl extension.
	// We can't run the code below until the <body> tag has loaded (so we can check for dir=rtl).
	// priority is 90 to run ahead of parser priority of 100
	ready(90, function(){
		if(!geometry.isBodyLtr()){
			var rtlClassStr = "dj_rtl dijitRtl " + classStr.replace(/ /g, "-rtl ");
			html.className = lang.trim(html.className + " " + rtlClassStr + "dj_rtl dijitRtl " + classStr.replace(/ /g, "-rtl "));
		}
	});
	return has;
});

},
'dojo/html':function(){
define(["./_base/kernel", "./_base/lang", "./_base/array", "./_base/declare", "./dom", "./dom-construct", "./parser"], function(dojo, lang, darray, declare, dom, domConstruct, parser) {
	// module:
	//		dojo/html
	// summary:
	//		TODOC

	lang.getObject("html", true, dojo);

	// the parser might be needed..

	// idCounter is incremented with each instantiation to allow asignment of a unique id for tracking, logging purposes
	var idCounter = 0;

	dojo.html._secureForInnerHtml = function(/*String*/ cont){
		// summary:
		//		removes !DOCTYPE and title elements from the html string.
		//
		//		khtml is picky about dom faults, you can't attach a style or <title> node as child of body
		//		must go into head, so we need to cut out those tags
		//	cont:
		//		An html string for insertion into the dom
		//
		return cont.replace(/(?:\s*<!DOCTYPE\s[^>]+>|<title[^>]*>[\s\S]*?<\/title>)/ig, ""); // String
	};

/*====
	dojo.html._emptyNode = function(node){
		// summary:
		//		removes all child nodes from the given node
		//	node: DOMNode
		//		the parent element
	};
=====*/
	dojo.html._emptyNode = domConstruct.empty;

	dojo.html._setNodeContent = function(/* DomNode */ node, /* String|DomNode|NodeList */ cont){
		// summary:
		//		inserts the given content into the given node
		//	node:
		//		the parent element
		//	content:
		//		the content to be set on the parent element.
		//		This can be an html string, a node reference or a NodeList, dojo.NodeList, Array or other enumerable list of nodes

		// always empty
		domConstruct.empty(node);

		if(cont) {
			if(typeof cont == "string") {
				cont = domConstruct.toDom(cont, node.ownerDocument);
			}
			if(!cont.nodeType && lang.isArrayLike(cont)) {
				// handle as enumerable, but it may shrink as we enumerate it
				for(var startlen=cont.length, i=0; i<cont.length; i=startlen==cont.length ? i+1 : 0) {
					domConstruct.place( cont[i], node, "last");
				}
			} else {
				// pass nodes, documentFragments and unknowns through to dojo.place
				domConstruct.place(cont, node, "last");
			}
		}

		// return DomNode
		return node;
	};

	// we wrap up the content-setting operation in a object
	declare("dojo.html._ContentSetter", null,
		{
			// node: DomNode|String
			//		An node which will be the parent element that we set content into
			node: "",

			// content: String|DomNode|DomNode[]
			//		The content to be placed in the node. Can be an HTML string, a node reference, or a enumerable list of nodes
			content: "",

			// id: String?
			//		Usually only used internally, and auto-generated with each instance
			id: "",

			// cleanContent: Boolean
			//		Should the content be treated as a full html document,
			//		and the real content stripped of <html>, <body> wrapper before injection
			cleanContent: false,

			// extractContent: Boolean
			//		Should the content be treated as a full html document, and the real content stripped of <html>, <body> wrapper before injection
			extractContent: false,

			// parseContent: Boolean
			//		Should the node by passed to the parser after the new content is set
			parseContent: false,

			// parserScope: String
			//		Flag passed to parser.	Root for attribute names to search for.	  If scopeName is dojo,
			//		will search for data-dojo-type (or dojoType).  For backwards compatibility
			//		reasons defaults to dojo._scopeName (which is "dojo" except when
			//		multi-version support is used, when it will be something like dojo16, dojo20, etc.)
			parserScope: dojo._scopeName,

			// startup: Boolean
			//		Start the child widgets after parsing them.	  Only obeyed if parseContent is true.
			startup: true,

			// lifecyle methods
			constructor: function(/* Object */params, /* String|DomNode */node){
				//	summary:
				//		Provides a configurable, extensible object to wrap the setting on content on a node
				//		call the set() method to actually set the content..

				// the original params are mixed directly into the instance "this"
				lang.mixin(this, params || {});

				// give precedence to params.node vs. the node argument
				// and ensure its a node, not an id string
				node = this.node = dom.byId( this.node || node );

				if(!this.id){
					this.id = [
						"Setter",
						(node) ? node.id || node.tagName : "",
						idCounter++
					].join("_");
				}
			},
			set: function(/* String|DomNode|NodeList? */ cont, /* Object? */ params){
				// summary:
				//		front-end to the set-content sequence
				//	cont:
				//		An html string, node or enumerable list of nodes for insertion into the dom
				//		If not provided, the object's content property will be used
				if(undefined !== cont){
					this.content = cont;
				}
				// in the re-use scenario, set needs to be able to mixin new configuration
				if(params){
					this._mixin(params);
				}

				this.onBegin();
				this.setContent();
				this.onEnd();

				return this.node;
			},
			setContent: function(){
				// summary:
				//		sets the content on the node

				var node = this.node;
				if(!node) {
					// can't proceed
					throw new Error(this.declaredClass + ": setContent given no node");
				}
				try{
					node = dojo.html._setNodeContent(node, this.content);
				}catch(e){
					// check if a domfault occurs when we are appending this.errorMessage
					// like for instance if domNode is a UL and we try append a DIV

					// FIXME: need to allow the user to provide a content error message string
					var errMess = this.onContentError(e);
					try{
						node.innerHTML = errMess;
					}catch(e){
						console.error('Fatal ' + this.declaredClass + '.setContent could not change content due to '+e.message, e);
					}
				}
				// always put back the node for the next method
				this.node = node; // DomNode
			},

			empty: function() {
				// summary
				//	cleanly empty out existing content

				// destroy any widgets from a previous run
				// NOTE: if you dont want this you'll need to empty
				// the parseResults array property yourself to avoid bad things happenning
				if(this.parseResults && this.parseResults.length) {
					darray.forEach(this.parseResults, function(w) {
						if(w.destroy){
							w.destroy();
						}
					});
					delete this.parseResults;
				}
				// this is fast, but if you know its already empty or safe, you could
				// override empty to skip this step
				dojo.html._emptyNode(this.node);
			},

			onBegin: function(){
				// summary
				//		Called after instantiation, but before set();
				//		It allows modification of any of the object properties
				//		- including the node and content provided - before the set operation actually takes place
				//		This default implementation checks for cleanContent and extractContent flags to
				//		optionally pre-process html string content
				var cont = this.content;

				if(lang.isString(cont)){
					if(this.cleanContent){
						cont = dojo.html._secureForInnerHtml(cont);
					}

					if(this.extractContent){
						var match = cont.match(/<body[^>]*>\s*([\s\S]+)\s*<\/body>/im);
						if(match){ cont = match[1]; }
					}
				}

				// clean out the node and any cruft associated with it - like widgets
				this.empty();

				this.content = cont;
				return this.node; /* DomNode */
			},

			onEnd: function(){
				// summary
				//		Called after set(), when the new content has been pushed into the node
				//		It provides an opportunity for post-processing before handing back the node to the caller
				//		This default implementation checks a parseContent flag to optionally run the dojo parser over the new content
				if(this.parseContent){
					// populates this.parseResults if you need those..
					this._parse();
				}
				return this.node; /* DomNode */
			},

			tearDown: function(){
				// summary
				//		manually reset the Setter instance if its being re-used for example for another set()
				// description
				//		tearDown() is not called automatically.
				//		In normal use, the Setter instance properties are simply allowed to fall out of scope
				//		but the tearDown method can be called to explicitly reset this instance.
				delete this.parseResults;
				delete this.node;
				delete this.content;
			},

			onContentError: function(err){
				return "Error occured setting content: " + err;
			},

			_mixin: function(params){
				// mix properties/methods into the instance
				// TODO: the intention with tearDown is to put the Setter's state
				// back to that of the original constructor (vs. deleting/resetting everything regardless of ctor params)
				// so we could do something here to move the original properties aside for later restoration
				var empty = {}, key;
				for(key in params){
					if(key in empty){ continue; }
					// TODO: here's our opportunity to mask the properties we dont consider configurable/overridable
					// .. but history shows we'll almost always guess wrong
					this[key] = params[key];
				}
			},
			_parse: function(){
				// summary:
				//		runs the dojo parser over the node contents, storing any results in this.parseResults
				//		Any errors resulting from parsing are passed to _onError for handling

				var rootNode = this.node;
				try{
					// store the results (widgets, whatever) for potential retrieval
					var inherited = {};
					darray.forEach(["dir", "lang", "textDir"], function(name){
						if(this[name]){
							inherited[name] = this[name];
						}
					}, this);
					this.parseResults = parser.parse({
						rootNode: rootNode,
						noStart: !this.startup,
						inherited: inherited,
						scope: this.parserScope
					});
				}catch(e){
					this._onError('Content', e, "Error parsing in _ContentSetter#"+this.id);
				}
			},

			_onError: function(type, err, consoleText){
				// summary:
				//		shows user the string that is returned by on[type]Error
				//		overide/implement on[type]Error and return your own string to customize
				var errText = this['on' + type + 'Error'].call(this, err);
				if(consoleText){
					console.error(consoleText, err);
				}else if(errText){ // a empty string won't change current content
					dojo.html._setNodeContent(this.node, errText, true);
				}
			}
	}); // end dojo.declare()

	dojo.html.set = function(/* DomNode */ node, /* String|DomNode|NodeList */ cont, /* Object? */ params){
			// summary:
			//		inserts (replaces) the given content into the given node. dojo.place(cont, node, "only")
			//		may be a better choice for simple HTML insertion.
			// description:
			//		Unless you need to use the params capabilities of this method, you should use
			//		dojo.place(cont, node, "only"). dojo.place() has more robust support for injecting
			//		an HTML string into the DOM, but it only handles inserting an HTML string as DOM
			//		elements, or inserting a DOM node. dojo.place does not handle NodeList insertions
			//		or the other capabilities as defined by the params object for this method.
			//	node:
			//		the parent element that will receive the content
			//	cont:
			//		the content to be set on the parent element.
			//		This can be an html string, a node reference or a NodeList, dojo.NodeList, Array or other enumerable list of nodes
			//	params:
			//		Optional flags/properties to configure the content-setting. See dojo.html._ContentSetter
			//	example:
			//		A safe string/node/nodelist content replacement/injection with hooks for extension
			//		Example Usage:
			//		dojo.html.set(node, "some string");
			//		dojo.html.set(node, contentNode, {options});
			//		dojo.html.set(node, myNode.childNodes, {options});
		if(undefined == cont){
			console.warn("dojo.html.set: no cont argument provided, using empty string");
			cont = "";
		}
		if(!params){
			// simple and fast
			return dojo.html._setNodeContent(node, cont, true);
		}else{
			// more options but slower
			// note the arguments are reversed in order, to match the convention for instantiation via the parser
			var op = new dojo.html._ContentSetter(lang.mixin(
					params,
					{ content: cont, node: node }
			));
			return op.set();
		}
	};

	return dojo.html;
});

},
'dijit/_base/place':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/_base/lang", // lang.isArray
	"dojo/window", // windowUtils.getBox
	"../place",
	".."	// export to dijit namespace
], function(array, lang, windowUtils, place, dijit){

	// module:
	//		dijit/_base/place
	// summary:
	//		Back compatibility module, new code should use dijit/place directly instead of using this module.

	dijit.getViewport = function(){
		// summary:
		//		Deprecated method to return the dimensions and scroll position of the viewable area of a browser window.
		//		New code should use windowUtils.getBox()

		return windowUtils.getBox();
	};

	/*=====
	dijit.placeOnScreen = function(node, pos, corners, padding){
		// summary:
		//		Positions one of the node's corners at specified position
		//		such that node is fully visible in viewport.
		//		Deprecated, new code should use dijit.place.at() instead.
	};
	=====*/
	dijit.placeOnScreen = place.at;

	/*=====
	dijit.placeOnScreenAroundElement = function(node, aroundElement, aroundCorners, layoutNode){
		// summary:
		//		Like dijit.placeOnScreenAroundNode(), except it accepts an arbitrary object
		//		for the "around" argument and finds a proper processor to place a node.
		//		Deprecated, new code should use dijit.place.around() instead.
	};
	====*/
	dijit.placeOnScreenAroundElement = function(node, aroundNode, aroundCorners, layoutNode){
		// Convert old style {"BL": "TL", "BR": "TR"} type argument
		// to style needed by dijit.place code:
		//		[
		// 			{aroundCorner: "BL", corner: "TL" },
		//			{aroundCorner: "BR", corner: "TR" }
		//		]
		var positions;
		if(lang.isArray(aroundCorners)){
			positions = aroundCorners;
		}else{
			positions = [];
			for(var key in aroundCorners){
				positions.push({aroundCorner: key, corner: aroundCorners[key]});
			}
		}

		return place.around(node, aroundNode, positions, true, layoutNode);
	};

	/*=====
	dijit.placeOnScreenAroundNode = function(node, aroundNode, aroundCorners, layoutNode){
		// summary:
		//		Position node adjacent or kitty-corner to aroundNode
		//		such that it's fully visible in viewport.
		//		Deprecated, new code should use dijit.place.around() instead.
	};
	=====*/
	dijit.placeOnScreenAroundNode = dijit.placeOnScreenAroundElement;

	/*=====
	dijit.placeOnScreenAroundRectangle = function(node, aroundRect, aroundCorners, layoutNode){
		// summary:
		//		Like dijit.placeOnScreenAroundNode(), except that the "around"
		//		parameter is an arbitrary rectangle on the screen (x, y, width, height)
		//		instead of a dom node.
		//		Deprecated, new code should use dijit.place.around() instead.
	};
	=====*/
	dijit.placeOnScreenAroundRectangle = dijit.placeOnScreenAroundElement;

	dijit.getPopupAroundAlignment = function(/*Array*/ position, /*Boolean*/ leftToRight){
		// summary:
		//		Deprecated method, unneeded when using dijit/place directly.
		//		Transforms the passed array of preferred positions into a format suitable for
		//		passing as the aroundCorners argument to dijit.placeOnScreenAroundElement.
		//
		// position: String[]
		//		This variable controls the position of the drop down.
		//		It's an array of strings with the following values:
		//
		//			* before: places drop down to the left of the target node/widget, or to the right in
		//			  the case of RTL scripts like Hebrew and Arabic
		//			* after: places drop down to the right of the target node/widget, or to the left in
		//			  the case of RTL scripts like Hebrew and Arabic
		//			* above: drop down goes above target node
		//			* below: drop down goes below target node
		//
		//		The list is positions is tried, in order, until a position is found where the drop down fits
		//		within the viewport.
		//
		// leftToRight: Boolean
		//		Whether the popup will be displaying in leftToRight mode.
		//
		var align = {};
		array.forEach(position, function(pos){
			var ltr = leftToRight;
			switch(pos){
				case "after":
					align[leftToRight ? "BR" : "BL"] = leftToRight ? "BL" : "BR";
					break;
				case "before":
					align[leftToRight ? "BL" : "BR"] = leftToRight ? "BR" : "BL";
					break;
				case "below-alt":
					ltr = !ltr;
					// fall through
				case "below":
					// first try to align left borders, next try to align right borders (or reverse for RTL mode)
					align[ltr ? "BL" : "BR"] = ltr ? "TL" : "TR";
					align[ltr ? "BR" : "BL"] = ltr ? "TR" : "TL";
					break;
				case "above-alt":
					ltr = !ltr;
					// fall through
				case "above":
				default:
					// first try to align left borders, next try to align right borders (or reverse for RTL mode)
					align[ltr ? "TL" : "TR"] = ltr ? "BL" : "BR";
					align[ltr ? "TR" : "TL"] = ltr ? "BR" : "BL";
					break;
			}
		});
		return align;
	};

	return dijit;
});

},
'gridx/modules/HScroller':function(){
define([
	"dojo/_base/declare",
	"dojo/dom-style",
	"dojo/_base/sniff",
	"dojo/_base/Deferred",
	"dojo/query",
	"dojox/html/metrics",
	"../core/_Module"
], function(declare, domStyle, sniff, Deferred, query, metrics, _Module){

	return declare(/*===== "gridx.modules.HScroller", =====*/_Module, {
		// summary:
		//		This module provides basic horizontal scrolling for grid

		name: 'hScroller',

		getAPIPath: function(){
			// tags:
			//		protected extension
			return {
				hScroller: this
			};
		},

		constructor: function(){
			var t = this,
				g = t.grid,
				n = g.hScrollerNode;
			g._initEvents(['H'], ['Scroll']);
			t.domNode = n;
			t.container = n.parentNode;
			t.stubNode = n.firstChild;
		},

		preload: function(){
			// tags:
			//		protected extension
			var t = this,
				g = t.grid,
				n = g.hScrollerNode;
			if(!g.autoWidth){
				g.vLayout.register(t, 'container', 'footerNode', 0);
				n.style.display = 'block';
				t.batchConnect(
					[g.columnWidth, 'onUpdate', 'refresh'],
					[n, 'onscroll', '_onScroll']);
				if(sniff('ie')){
					//In IE8 the horizontal scroller bar will disappear when grid.domNode's css classes are changed.
					//In IE6 this.domNode will become a bit taller than usual, still don't know why.
					n.style.height = metrics.getScrollbar().h + 'px';
				}
			}
		},
		
		//Public API-----------------------------------------------------------

		scroll: function(left){
			// summary:
			//		Scroll the grid horizontally
			// tags:
			//		package
			// left: Number
			//		The scrollLeft value
			
			var dn = this.domNode;
			if((sniff('webkit') || sniff('ie') < 8) && !this.grid.isLeftToRight()){
				left = dn.scrollWidth - dn.offsetWidth - left;
			}
			if((sniff('ff')) && !this.grid.isLeftToRight() && left > 0){
				left = -left;
			}
			dn.scrollLeft = left;
		},
		
		scrollToColumn: function(colId){
			// summary:
			//	Scroll the grid to make a column fully visible.
			var hNode = this.grid.header.innerNode,
				table = query('table', hNode)[0],
				cells = table.rows[0].cells,
				left = 0,
				right = 0,
				ltr = this.grid.isLeftToRight(),
				scrollLeft = this.domNode.scrollLeft;
			
			if(!ltr && (sniff('webkit') || sniff('ie') < 8)){
				scrollLeft = this.domNode.scrollWidth - scrollLeft - hNode.offsetWidth;//the value relative to col 0
			}
			scrollLeft = Math.abs(scrollLeft);
			//get cell's left border and right border position
			for(var i = 0; i < cells.length; i++){
				right += cells[i].offsetWidth;
				if(cells[i].getAttribute('colid') == colId){
					break;
				}
				left += cells[i].offsetWidth;
			}
			
			//if the cell is not visible, scroll to it
			if(left < scrollLeft){
				this.scroll(left);
			}else if(right > scrollLeft + hNode.offsetWidth){
				this.scroll(right - hNode.offsetWidth);
			}
			if(ltr)hNode.scrollLeft = 0;//force to 0, the header uses margin to scroll
			else hNode.scrollLeft = hNode.scrollWidth - hNode.offsetWidth;
		},
		
		refresh: function(){
			// summary:
			//		Refresh scroller itself to match grid body
			// tags:
			//		package
			var t = this,
				g = t.grid,
				ltr = g.isLeftToRight(),
				marginLead = ltr ? 'marginLeft' : 'marginRight',
				marginTail = ltr ? 'marginRight' : 'marginLeft',
				lead = g.hLayout.lead,
				tail = g.hLayout.tail,
				w = (g.domNode.clientWidth || domStyle.get(g.domNode, 'width')) - lead - tail,
				bn = g.header.innerNode,
				pl = domStyle.get(bn, ltr ? 'paddingLeft' : 'paddingRight') || 0,	//TODO: It is special for column lock now.
				s = t.domNode.style,
				sw = bn.firstChild.offsetWidth + pl,
				oldDisplay = s.display,
				newDisplay = (sw <= w) ? 'none' : 'block';
			s[marginLead] = lead + pl + 'px';
			s[marginTail] = tail + 'px';
			//Insure IE does not throw error...
			s.width = (w - pl < 0 ? 0 : w - pl) + 'px';
			t.stubNode.style.width = (sw - pl < 0 ? 0 : sw - pl) + 'px';
			s.display = newDisplay;
			if(oldDisplay != newDisplay){
				g.vLayout.reLayout();
			}
		},
		
		//Private-----------------------------------------------------------
		_lastLeft: 0,

		_onScroll: function(e){
			//	Fired by h-scroller's scrolling event
			var t = this,
				s = t.domNode.scrollLeft;
			if((sniff('webkit') || sniff('ie') < 8) && !t.grid.isLeftToRight()){
				s = t.domNode.scrollWidth - t.domNode.offsetWidth - s;
			}
			if(t._lastLeft != s){
				t._lastLeft = s;
				t._doScroll();
			}
		},

		_doScroll: function(rowNode){
			//	Sync the grid body with the scroller.
			var t = this,
				g = t.grid;
			g.bodyNode.scrollLeft = t.domNode.scrollLeft;
			g.onHScroll(t._lastLeft);
		}
	});
});

},
'ibm/usmi/widgets/DetailsSelector':function(){
require({cache:{
'url:ibm/usmi/widgets/templates/DetailsSelector.html':"<div class=\"detailsSelector maxSize\">\r\n<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n\t<div id=\"detailsSelectorContentPane\" style=\"overflow-y: auto;width: 98%;height: 100%;\" class=\"detailsSelectorMain\" data-dojo-type=\"dijit.layout.ContentPane\">\r\n\t\t<!-- Top & Rounded Corners -->\r\n\t    <div class=\"detailsSelectorTop\">\r\n\t        <div class=\"detailsSelectorTopLeft\"></div>\r\n\t        <div class=\"detailsSelectorTopCenter\"></div>\r\n\t        <div class=\"detailsSelectorTopRight\"></div>\r\n\t    </div>\r\n\t    \r\n\t    <!-- Center where all the good stuff goes -->   \r\n\t    <div class=\"detailsSelectorCenter\">\r\n            <div class=\"detailsSelectorCenterLeft\"></div>\r\n\t\t    <div class=\"detailsSelectorCenterCenter\">\r\n\t\t\t\t<!-- Main Image, High level properties and Actions Button -->\r\n\t    \t\t<div class=\"detailsSelectorTopPane\" data-dojo-attach-point=\"topPane\">\r\n\t    \t\t    <!-- Main Image -->\r\n\t\t\t\t\t<div id=\"TopImage\" class=\"detailsSelectorImage\">\r\n\t\t\t\t    \t<img class=\"mainImage\" data-dojo-attach-point=\"resourceImage\"></img>\r\n\t\t\t\t  \t</div>\r\n\t\t\t\t  \t<!-- Actions Dropdown Button -->\r\n\t\t\t\t  \t<div id=\"ActionsButton\" dojoAttachPoint=\"buttonDiv\" class=\"detailsButton pad5 hide\">\r\n\t\t\t\t      \t<button id=\"actBtn\" data-dojo-type=\"dijit.form.DropDownButton\" data-dojo-attach-point=\"actionListBtn\">\r\n\t\t\t\t          \t<span>${actionLabel}</span>\t          \t\r\n\t\t\t\t      \t</button>\r\n\t\t\t\t  \t</div>\t\t\t\t  \t\r\n\t\t\t\t  \t<!-- High Level Properties -->\r\n\t\t\t\t    <div class=\"detailsSelectorLabel pad15b\" role=\"heading\" aria-level=\"1\">\r\n\t\t\t\t        <div data-dojo-attach-point=\"propertiesList\"></div>\r\n\t\t\t\t    </div>\r\n\t\t\t\t\t\t\t\t\t\t\r\n\t\t\t\t</div>\r\n\t\t\t\t<!-- Tasks Section, categories and tasks -->\r\n\t\t\t\t<div id=\"tPane\" style=\"position: relative;\" class=\"detailsSelectorTaskPane\" data-dojo-type=\"ibm.usmi.widgets.TaskPane\" data-dojo-attach-point=\"taskPane\"></div>\r\n\t\t    </div>\r\n\t\t    <div class=\"detailsSelectorCenterRight\"></div>\r\n\t    </div>\r\n\t    \r\n\t    <!-- Bottom & Rounded Corners -->\r\n\t    <div class=\"detailsSelectorBottom\">\r\n\t        <div class=\"detailsSelectorBottomLeft\"></div>\r\n\t        <div class=\"detailsSelectorBottomCenter\"></div>\r\n\t        <div class=\"detailsSelectorBottomRight\"></div>\r\n\t    </div>\t\t  \r\n\t</div>\r\n</div>"}});
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

define(
		"ibm/usmi/widgets/DetailsSelector", [
			"dojo/_base/kernel",
			"dojo/_base/declare",
			"dojo/topic",
			"dijit/_WidgetBase",
			"dijit/_TemplatedMixin",
			"dijit/_WidgetsInTemplateMixin",
			"dojo/text!./templates/DetailsSelector.html",
			"dijit/form/Button",
			"dijit/Menu",
			"dijit/PopupMenuItem",
			"dijit/MenuItem",
			"dijit/MenuSeparator",
			"dijit/form/DropDownButton",
			"dijit/layout/BorderContainer",
			"require",
			// TWL imports
			"ibm/usmi/util/DefaultMessageHandler",
			"ibm/tivoli/tip/navwidget/dijit/NavWidgetPluggableView",
			"ibm/tivoli/tip/navwidget/dijit/NavWidget",
			"ibm/tivoli/tip/navwidget/topology/dijit/MenuProviderHelper",
			"ibm/tivoli/tip/navwidget/topology/dijit/MenuProviderHelperBase",
			"ibm/tivoli/tip/navwidget/properties/PropertyRenderer",
			"ibm/tivoli/tip/TCTTable",
			"ibm/tivoli/navmodel/Dataset",
			// NextGen imports
			"ibm/usmi/util/BaseWidget",
			"ibm/usmi/widgets/util/PropertyRenderer",
			"ibm/usmi/widgets/TaskCategory",
			"ibm/usmi/widgets/TaskLink",
			"ibm/usmi/widgets/TaskPane",
			"ibm/usmi/widgets/util/DefaultContextMenuProvider",
			"ibm/usmi/widgets/util/DefaultMenuProviderHelper",
			"ibm/usmi/widgets/util/Constants",
			"ibm/usmi/core/utils/Logger",
			"dojo/i18n!./nls/CommonWidgetsResources"
		],
		function (dojo, declare, topic, _WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin, DetailsSelectorTemplate,
				Button, Menu, PopupMenuItem, MenuItem, MenuSeparator, DropDownButton, BorderContainer, require,
				DefaultMessageHandler, NavWidgetPluggableView, NavWidget, MenuProviderHelper, MenuProviderHelperBase,
				PropertyRenderer, TCTTable, Dataset, BaseWidget, UsmiPropertyRenderer, TaskCategory, TaskLink,
				TaskPane, DefaultContextMenuProvider, DefaultMenuProviderHelper, Constants, Logger, mri) {

			return declare(
					[
						BorderContainer,
						_WidgetBase,
						_TemplatedMixin,
						_WidgetsInTemplateMixin,
						BaseWidget,
						NavWidgetPluggableView
					],
					{

						// Template info:  Path
						templateString : DetailsSelectorTemplate,

						// context menu provider that is passed off to pluggableview
						cmProvider : null,

						// NLS string to hold Processing String
						processingLabel : null,

						// NLS string to hold resource label
						resource : "",

						// attributes that can be set in constructor, html, or url tag
						selectorHome : "",

						// Defaults
						DEFAULT_COLLECTION : "RESOURCE_DETAILS_COLLECTION",

						// Image Path
						IMAGE_PATH : "ibm/usmi/widgets/themes/selector/images/",

						// actionLabel: [public]String
						//		label of action button.
						actionLabel : "",

						///////////////////////////////////////////////////////////////////
						// private members
						///////////////////////////////////////////////////////////////////

						// the logger
						_detailsSelectorLogger : new Logger("ibm.usmi.widgets", "DetailsSelector"),

						// tells navwidget pluggable view not to try and load a widgetdef
						_usesWidgetDefinition : false,

						// item backing selector widget
						_item : null,

						_imagePath : null,

						// actions button menu
						_dropDownMenu : null,

						// array to hold all items shown in the actions menu
						_menuItems : null,

						// array to hold all submenus / categories in actions menu
						_subMenus : null,

						// array to hold all task categories in task pane
						_subTasks : null,

						// array to hold all tasks in task pane
						_taskItems : null,

						// client constructed general task category (holds uncategorized tasks and properties)
						_generalTaskCategory : null,

						// properties to show in top pane, (default column descriptors)
						_defaultProps : null,

						// Passed to Task Categories so they are keyboard accessible
						tabIndex : "0",

						// used to render the properties shown in top pane, consistent with how TWL table properties would render
						_propertyRenderer : null,

						// load up the task utils for the different types of task launch
						_taskUtil : null,

						// Used for dataset events processnig
						_eventTimeout : null,
						_initialEventTime : null,

						//resource bundle
						_resources : mri,

						// This is called by NavWidgetPluggable View in its contructor
						_createCMProvider : function () {
							// summary:
							//        Create the menu provider
							// description:
							//        Create the menu provider that we'll pass off to the pluggable views

							//Create the menu provider that we'll pass off to the pluggable views
							if (!this.cmProvider) {
								var helper = new DefaultMenuProviderHelper();
								this.cmProvider = new DefaultContextMenuProvider({
									view : this,
									getMenuItemsCallback : dojo.hitch(helper, helper.getMenuItems),
									getMenuCommandCallback : dojo.hitch(helper, helper.getMenuCommand)
								});
							}

						},

						constructor : function () {
							// summary:
							//        Constructs Details Selector object.

							if (this._detailsSelectorLogger.isFinest()) {
								this._detailsSelectorLogger.entering("", "constructor");
							}

							this._propertyRenderer = new UsmiPropertyRenderer();//ibm.tivoli.tip.navwidget.properties.PropertyRenderer();      
							this._subMenus = [];
							this._menuItems = [];
							this._subTasks = [];
							this._taskItems = [];

							this.selectorHome = this.getInitArg(Constants.SELECTOR_HOME);
							this._imagePath = require.toUrl(this.IMAGE_PATH);

							if (this._detailsSelectorLogger.isFinest()) {
								this._detailsSelectorLogger.exiting("", "constructor");
							}
						},

						postMixInProperties : function () {
							// summary:  set nls strings
							// tags: private
							this.actionLabel = this._resources.ActionButtonLabel;
							this.processingLabel = this._resources.Processing;
							this.inherited(arguments);
						},

						postCreate : function () {
							// summary:
							//         Called after a widget's dom has been setup
							// tags:
							//        protected

							// Hide the actions button, if tasks are returned then will show
							dojo.addClass(this.buttonDiv, "hide");

							// add menu to dom element specified in template
							var dropDownArgs = {
								id : "_" + this.getUrlParm(Constants.SOURCE_TOKEN) + "_menu",
								style : Constants.CSS_DISPLAY_NONE,
								autoSize : true
							};

							this._dropDownMenu = new Menu(dropDownArgs);
							this.actionListBtn.set("dropDown", this._dropDownMenu);

							// construct the dataset, either passed in constructor or on url
							this.buildDataset();

							//Call superclass
							this.inherited(arguments);
						},

						buildDataset : function () {
							// summary:
							//        dataset that was passed in on URL or constructor, default to RESOURCE_DETAILS_COLLECTION

							// get dataset that was passed in on URL or constructor, default to RESOURCE_DETAILS_COLLECTION
							if (this._detailsSelectorLogger.isFinest()) {
								this._detailsSelectorLogger.entering("", "buildDataset");
							}

							if (!this.providerID) {
								// default
								this.providerID = Constants.DEFAULT_PROVIDER;
							}

							// get dataset that was passed in on URL or constructor, default to RESOURCE_DETAILS_COLLECTION
							if (!this.selectorHome) {
								// default
								this.selectorHome = this.DEFAULT_COLLECTION;
							}

							try {
								this.dataset = new Dataset({
									baseUrl : Constants.BASE_REST_URL,
									provider : this.providerID,
									datasource : "",
									dataset : this.selectorHome,
									parameters : {
										sourceToken : this.sourceToken
									},
									defaultProperties : [ Constants.PROPS_ALL
									]
								});
							} catch (err) {
								//There was a problem creating the dataset
								var msgHandler = new DefaultMessageHandler({});
								msgHandler.displayException(err);
								return;
							}

							if (this._detailsSelectorLogger.isFinest()) {
								this._detailsSelectorLogger.exiting("", "buildDataset");
							}
						},

						destroy : function () {
							// summary:
							//        Clean up any objects the details selctor is hanging onto
							if (this._detailsSelectorLogger.isFinest()) {
								this._detailsSelectorLogger.entering("", "destroy");
							}

							if (this.dataset) {
								this.dataset.removeEventListener(this);
								this.dataset.destroy();
								this.dataset = null;
							}

							this.inherited(arguments);

							if (this._detailsSelectorLogger.isFinest()) {
								this._detailsSelectorLogger.exiting("", "destroy");
							}
						},

						_setDatasetLoadData : function (/*ibm.tivoli.navmodel.Dataset*/dataset) {
							// summary:
							//        Called by NavWidgetPluggableView when the dataset is ready for rendering
							// description:
							//        Called by NavWidgetPluggableView when the dataset is ready for rendering.
							//        Kicks off the rendering and retrieval of all data to be shown in details selector
							// tags:
							//        private

							// call back functions to call once getItems completes
							var onItemsCompleteFunctions = {
								onComplete : dojo.hitch(this, this._onItemsRetrieval),
								onError : dojo.hitch(this, this._onItemsRetrievalFailed),
								properties : [ Constants.PROPS_DEFAULT
								]
							};

							// attach event listener so we get notified when the dataset updates
							dataset.addEventListener(this, dojo.hitch(this, this._eventHandler));

							// get items from dataset
							dataset.getItems(onItemsCompleteFunctions);
						},

						///////////////////////////////////////////////////////////////////////////////////////////
						// Successful on Complete Methods
						///////////////////////////////////////////////////////////////////////////////////////////
						_onItemsRetrieval : function (/*ibm.tivoli.navmodel.Item[]*/items) {
							// summary:
							//        Called when the items are returned from dataset
							// tags:
							//         callback
							if (this._detailsSelectorLogger.isFinest()) {
								this._detailsSelectorLogger.entering("", "_onItemsRetrieval");
								this._detailsSelectorLogger.finest("DETAILS SELECTOR- items retrieval RETURNED count: "
										+ items.length, "_onItemsRetrieval");
							}

							// check to see if items were returned from the dataset
							if (items) {
								this._item = items[0];
							}

							// if there is an item, use it
							if (this._item) {
								// set the selection for the context menu
								this.selection.setSelectedObjects([ this._item ], null, null);
								
								// set the resoure label
								this.resource = this._item._item.label;
								// Populate the properties to be shown in top pane, uses default column descriptors
								this._getTopPaneProperties();

							} // else use the collection to query further information
							else {
								// set the resource label to dataset label
								this.resource = this.dataset.getLabel();

								// TODO query dataset parms for icon, title
								this._getTopPaneContentFromCollection();
							}

							// Populate the actions button
							this._setActionsMenu();

							// Populate the task Pane
							this._fetchGlobalTasks();

							if (this._detailsSelectorLogger.isFinest()) {
								this._detailsSelectorLogger.exiting("", "_onItemsRetrieval");
							}
						},

						_onPropertiesDataRetrieved : function (/*ibm.tivoli.navmodel.Property[]*/properties, args) {
							// summary:
							//        Called when the properties are returned from item.   The properties returned
							//        are from the default columns
							// tags:
							//         callback
							if (this._detailsSelectorLogger.isFinest()) {
								this._detailsSelectorLogger.entering("", "_onPropertiesDataRetrieved");
								this._detailsSelectorLogger.finest("DETAILS SELECTOR- item: " + this._item.getLabel(),
										"_onPropertiesDataRetrieved");
							}

							var i, propDiv, prop;
							for (i = 0; i < properties.length; i += 1) {
								prop = properties[i];
								if (this._detailsSelectorLogger.isFinest()) {
									this._detailsSelectorLogger.finest("DETAILS SELECTOR- PropId: " + prop.getId()
											+ " displayValue: " + prop.getDisplayValue(), "_onPropertiesDataRetrieved");
								}

								propDiv = this._propertyRenderer.getRenderedPropertyNode(this._item, prop, this);
								propDiv.id = prop.getId();
								dojo.place(propDiv, this.propertiesList);
							}

							if (this._detailsSelectorLogger.isFinest()) {
								this._detailsSelectorLogger.exiting("", "_onPropertiesDataRetrieved");
							}
						},

						///////////////////////////////////////////////////////////////////////////////////////////
						// Action Button Methods                                                                 //
						///////////////////////////////////////////////////////////////////////////////////////////
						_setActionsMenu : function () {
							// summary:
							//        If the menu is currently empty (i.e., asked to load), return the placeholder
							//        menu item and tell the asynch provider to get the menu items asynchronously,
							//        otherwise, remove the placeholder and ask the provider for the menu items
							//        now that the asynch request has returned the menu items.

							if (this._detailsSelectorLogger.isFinest()) {
								this._detailsSelectorLogger.entering("", "_setActionsMenu");
							}
							var items = null, childItems = this._dropDownMenu.getChildren();
							if (childItems.length === 0) {
								items = [ {
									"label" : this.processingLabel,
									"iconClass" : "tipMenuIcon loadingIcon",
									"actionID" : this.loadingMenuItemID,
									"launchType" : "NOOP"
								}
								];
								this.cmProvider.getMenuItemsAsynch(this, dojo.hitch(this, this._menuItemsLoaded));
							} else {
								this._removeLoadingItem();
								items = this.cmProvider.getMenuItems(this);
							}
							this._addItemsToMenu(this._dropDownMenu, items);

							if (this._detailsSelectorLogger.isFinest()) {
								this._detailsSelectorLogger.exiting("", "_setActionsMenu");
							}
						},

						_menuItemsLoaded : function (/*Boolean*/success) {
							// summary:
							//        all to Context Menu Provider returned, populate actions menu
							// tags:
							//         callback

							if (success) {
								// repopulate menu
								this._setActionsMenu();
								// the menu is already marked as started and open, so loadDropDown skips these
								// steps, so do them again here
								dojo.forEach(this._dropDownMenu.getChildren(), function (child) {
									child.startup();
								});
								this._dropDownMenu.startupKeyNavChildren();
								// The dijit.Menu still remembers the "Loading..." item as focused so reset
								this._dropDownMenu.focusedChild = null;
								this._dropDownMenu.focus();
							} else {
								// There were either no tasks returned or an error occurred, so dismiss the dropDown
								//this.closeDropDown(true);
								this._menuItems = [];
								this._subMenus = [];
								this._subTasks = [];
								this._taskItems = [];
							}
						},

						_addItemsToMenu : function (/*dijit/Menu*/parent, /*ibm.tivoli.navmodel.Item[]*/items) {
							// summary:
							//        Recursive function to construct menu from array of items
							//        (adapted from ibm.tivoli.tip.dijit.ContextMenuAdapter)
							var params, subMenuIndex, menuItemIndex, i, menuItemOnClickMethod = function () {
								this.adapter._onMenuItemClick(this);
							};

							// if at least one action is returned, lets show the actions button
							if (items.length > 0) {
								// remove hide it
								dojo.removeClass(this.buttonDiv, "hide");
								// show the actions button
								dojo.addClass(this.buttonDiv, "show");
							} else {
								// add show it
								dojo.removeClass(this.buttonDiv, "show");
								// no action, remove button
								dojo.addClass(this.buttonDiv, "hide");
							}

							for (i = 0; i < items.length; i += 1) {
								if (items[i].submenu_items) {
									subMenuIndex = this._subMenus.length;
									this._subMenus[subMenuIndex] = new Menu();

									params = {
										popup : this._subMenus[subMenuIndex]
									};

									dojo.mixin(params, items[i]);
									parent.addChild(new PopupMenuItem(params));

									this._addItemsToMenu(this._subMenus[subMenuIndex], items[i].submenu_items);
								} else {
									// The following if clause added for backward compatibility
									// with the previous version of TIPContextMenu
									if (items[i].id && items[i].id.toLowerCase() === "separator") {
										// Previous TIPContextMenu users indicated separators via the id
										items[i].actionID = "_separator_";
									} else if (items[i].id && !items[i].actionID) {
										// Previous TIPContextMenu users didn't use actionID
										items[i].actionID = items[i].id;
									}

									if (items[i].actionID === "_separator_") {
										menuItemIndex = this._menuItems.length;
										this._menuItems[menuItemIndex] = new MenuSeparator();
										parent.addChild(this._menuItems[menuItemIndex]);
									} else {
										params = {
											adapter : this,
											onClick : menuItemOnClickMethod
										};
										dojo.mixin(params, items[i]);
										menuItemIndex = this._menuItems.length;
										this._menuItems[menuItemIndex] = new MenuItem(params);
										parent.addChild(this._menuItems[menuItemIndex]);
									}
								}
							}
						},

						_onMenuItemClick : function (/*dijit/MenuItem*/menuItem) {
							// summary:
							//        Invoke getMenuCommand on context menu provider when action is selected.
							// tags:
							//         callback
							this.cmProvider.getMenuCommand(menuItem.actionID, this, menuItem);
						},

						_removeLoadingItem : function () {
							// summary:
							//        Remove loading entry from context menu
							// description:
							//        Remove loading entry from context menu
							var childItems = this._dropDownMenu.getChildren(), loadingItem = childItems[0];
							this._dropDownMenu.removeChild(loadingItem);
							loadingItem.destroy();
						},

						///////////////////////////////////////////////////////////////////////////////////////////
						// TopPane Properties Methods                                                             //
						///////////////////////////////////////////////////////////////////////////////////////////
						_getTopPaneProperties : function () {
							// summary:
							//        Get the item style for icon, and the ids for the properties to be shown in the top pane
							// tags:
							//     private
							// load main image if one exists

							if (this._detailsSelectorLogger.isFinest()) {
								this._detailsSelectorLogger.entering("", "_getTopPaneProperties");
							}

							var style = this._item.getItemStyle({
								mergeWidgetDef : true,
								widgetDef : this.getNearestPluggableViewWidgetDefinition()
							}), path, columnIds = [], columnArgs, gotColumnsError, gotColumns, col, currcol, colData;

							if (style) {
								// ie shows a broken image, so need to remove widget from dom
								dojo.attr(this.resourceImage, "style", "display:inline");
								//create the image node
								path = this._cleanUpImagePath(style.getLargeImage());
								dojo.attr(this.resourceImage, "src", path);
								dojo.attr(this.resourceImage, "height", "48px;"); // TODO style.getAttribute("largeImageHeight"));
								dojo.attr(this.resourceImage, "width", "48px;");//TODO style.getAttribute("largeImageWidth"));

								if (style.getLabel()) {
									dojo.attr(this.resourceImage, "alt", style.getLabel());
								}

							} else {
								// ie shows a broken image, so need to remove widget from dom
								dojo.attr(this.resourceImage, "style", "display:none");
							}

							//Callback to get columns from dataset
							gotColumns = function (cols) {
								//Process the list of columns
								var columns = [], viewColumns = [], i;
								if (cols) {
									for (i = 0; i < cols.length; i += 1) {
										col = cols[i];
										// If the column is a "linkType" ignore it
										currcol = col.getValueType();
										if (currcol !== "linkType" && currcol !== "relationshipType") {
											colData = {};
											colData.id = col.getId();
											colData.label = {};
											colData.label.value = col.getLabel();
											colData.type = col.getValueType();
											colData.unsupportedOperators = col.getUnsupportedFilterOperators();

											// get column IDs to pass to store
											columnIds.push(colData.id);
											columns.push(colData);
											viewColumns.push(col);
										}
									}

									// store default properties in global var
									this._defaultProps = columnIds;

									// get the properties for the default columns to show in top pane
									this._fetchPropertiesForDefaultCols();
								}
							};

							gotColumnsError = function (error, request) {
								//Give the user some feedback
								if (this._detailsSelectorLogger.isFinest()) {
									this._detailsSelectorLogger.error("DETAILS SELECTOR- getcolumns failed.  " + error,
											"gotColumnsError");
								}
								var msgHandler = new DefaultMessageHandler({});
								msgHandler.displayException(error);
							};

							columnArgs = {
								type : Constants.ITEM,
								properties : [ Constants.PROPS_DEFAULT
								],
								onComplete : dojo.hitch(this, gotColumns),
								onError : dojo.hitch(this, gotColumnsError)
							};
							this.dataset.getColumns(columnArgs);

							if (this._detailsSelectorLogger.isFinest()) {
								this._detailsSelectorLogger.exiting("", "_getTopPaneProperties");
							}
						},

						_getTopPaneContentFromCollection : function () {
							// summary:
							//        Get the item style for icon, and the ids for the properties to be shown in the top pane
							// tags:
							//     private
							// load main image if one exists

							if (this._detailsSelectorLogger.isFinest()) {
								this._detailsSelectorLogger.entering("", "_getTopPaneProperties");
							}

							var path, label, propDiv;
							path = this.dataset.getAttribute("selectorIcon");
							label = this.dataset.getAttribute("selectorLabel");

							// if path is specified as collection attribute, get it and show
							if (path) {
								dojo.attr(this.resourceImage, "style", "display:inline");
								dojo.attr(this.resourceImage, "src", path);
								dojo.attr(this.resourceImage, "height", "48px;");
								dojo.attr(this.resourceImage, "width", "48px;");
								dojo.attr(this.resourceImage, "alt", label);
							} else {
								if (this._detailsSelectorLogger.isFinest()) {
									this._detailsSelectorLogger.exiting("no selectorIcon specified on collection",
											"_getTopPaneProperties");
								}
								
								// ie shows a broken image, so need to remove widget from dom
								dojo.attr(this.resourceImage, "style", "display:none");
							}

							// if label was specified as a collection attribute, get it and show
							if (label) {
								// get the properties for the default columns to show in top pane
								propDiv = dojo.doc.createElement("div");
								dojo.place(dojo.doc.createTextNode(label), propDiv, "last");

								propDiv.id = "DisplayName";
								dojo.place(propDiv, this.propertiesList);
							} else {
								if (this._detailsSelectorLogger.isFinest()) {
									this._detailsSelectorLogger.exiting("no selectorLabel specified on collection",
											"_getTopPaneProperties");
								}
							}

							if (this._detailsSelectorLogger.isFinest()) {
								this._detailsSelectorLogger.exiting("", "_getTopPaneProperties");
							}
						},

						_fetchPropertiesForDefaultCols : function () {
							// summary:
							//        Get properties from the item
							// tags:
							//     private
							if (this._detailsSelectorLogger.isFinest()) {
								this._detailsSelectorLogger.finest("DETAILS SELECTOR- item.id: " + this._item.getId()
										+ " , type: " + this._item.getType() + " , label: " + this._item.getLabel(),
										"_fetchPropertiesForDefaultCols");
							}

							var itemPropArgs = {
								onComplete : dojo.hitch(this, this._onPropertiesDataRetrieved),
								onError : dojo.hitch(this, this._onPropertiesDataRetrievedFailed),
								properties : this._defaultProps
							};

							this._item.getProperties(itemPropArgs);
						},

						handleLinkClickAction : function (viewId, objectId, propId, objectType) {
							// summary:
							//         this method is called by the base TWL PropertyRender when a property
							//         link is clicked on in the toppane section
							// tags:
							//     protected
							if (this._detailsSelectorLogger.isFinest()) {
								this._detailsSelectorLogger.entering("", "handleLinkClickAction");
							}

							var object, property;

							//ask the dataset for the correct object based on the objects id.
							//since we have already rendered these items, the value should be
							//cached and should be quick.
							switch (objectType) {
							case "ibm.tivoli.navmodel.Item":
								object = this.dataset.getItem({
									itemId : objectId
								});
								break;
							case "ibm.tivoli.navmodel.Link":
								object = this.dataset.getLink({
									itemId : objectId
								});
								break;
							case "ibm.tivoli.navmodel.Relationship":
								object = this.dataset.getRelationship({
									itemId : objectId
								});
								break;
							default:
								if (this._detailsSelectorLogger.isFinest()) {
									this._detailsSelectorLogger.finest("unrecognized object type!",
											"handleLinkClickAction");
								}

							}

							if (object) {
								property = object.getProperty(propId);
							}

							if (object && property) {
								this._propertyRenderer._handleLinkClick(property, this);
							}

							if (this._detailsSelectorLogger.isFinest()) {
								this._detailsSelectorLogger.exiting("", "handleLinkClickAction");
							}
						},
						///////////////////////////////////////////////////////////////////////////////////////////
						// TaskPane Methods                                                                         //
						///////////////////////////////////////////////////////////////////////////////////////////
						_fetchGlobalTasks : function () {
							// summary:
							//         Get the tasks from the collection to show as drawer tasks
							// tags:
							//     private
							this.dataset.getTasks({
								onComplete : dojo.hitch(this, this._onGlobalTasksComplete),
								onError : dojo.hitch(this, this._onTasksError),
								taskCategory : "toolbar"
							});
						},

						_onGlobalTasksComplete : function (/*ibm.tivoli.navmodel.Task[]*/tasks) {
							// summary:
							//         Process tasks returned into Task Categories and Task Links
							// tags:
							//     callback
							//Want to loop through tasks looking for
							if (tasks) {
								// process tasks and add to appropriate task categories
								this._populateTaskPane(this.taskPane, tasks);

								// after all tasks are added to taskpane, refresh to setup arrrays
								this.taskPane.refresh();
							}
						},

						_populateTaskPane : function (parent, /*ibm.tivoli.navmodel.Task[]*/tasks) {
							// summary:
							//        Recursive function to construct menu from array of tasks
							//        (adapted from ibm.tivoli.tip.dijit.ContextMenuAdapter)

							if (this._detailsSelectorLogger.isFinest()) {
								this._detailsSelectorLogger.entering("", "_populateTaskPane");
							}

							var cascadeIndex, taskIndex, i, tc, tl, firstTaskLink;

							for (i = 0; i < tasks.length; i += 1) {
								if (this._detailsSelectorLogger.isFinest()) {
									this._detailsSelectorLogger.finest("DETAILS SELECTOR Global Tasks- taskId: "
											+ tasks[i].getId() + " displayValue: " + tasks[i].getLabel()
											+ " LaunchUrl: " + tasks[i].getLaunchUrl(), "_populateTaskPane");
								}

								// check to see if task has child tasks (its a task category)
								if (tasks[i].isCascade()) {
									// current task categories count, used as index in array
									cascadeIndex = this._subTasks.length;

									// create a TaskCategory object
									tc = new TaskCategory({
										label : tasks[i].getLabel(),
										task : tasks[i]
									});

									// add TaskCategory to array of TaskCategories
									this._subTasks[cascadeIndex] = tc;

									// want to add sub categories up to main page level
									if (parent instanceof TaskCategory
											&& this._subTasks[cascadeIndex] instanceof TaskCategory) {
										this.taskPane.addCategory(this._subTasks[cascadeIndex]);
									} else if (parent instanceof TaskCategory) {
										// add link to category
										parent.addLink(this._subTasks[cascadeIndex]);
									} else if (parent instanceof TaskPane) {
										// add category to taskPane
										parent.addCategory(this._subTasks[cascadeIndex]);
									}

									// recursive call to process categories children tasks
									this._populateTaskPane(this._subTasks[cascadeIndex], tasks[i].getChildTasks());
								} else {
									// ignore separators if they exist in task returned from navmodel
									if (!tasks[i].isSeparator()) {
										// current task count, used as index
										taskIndex = this._taskItems.length;

										// create the task link
										tl = this._createTaskLink(tasks[i]);

										this._taskItems[taskIndex] = tl;

										// add link to category
										if (parent instanceof TaskCategory) {
											parent.addLink(this._taskItems[taskIndex]);
										} else if (parent instanceof TaskPane) {
											if (this._taskItems[taskIndex] instanceof TaskLink) {
												// tasks that isnt categorized, put in general task heading.
												if (this._detailsSelectorLogger.isWarn()) {
													this._detailsSelectorLogger.warn(
															"Task is not categorized, not showing: "
																	+ this._taskItems[taskIndex].link,
															"_populateTaskPane");
												}
											} else {
												// add category to taskPane
												parent.addCategory(this._taskItems[taskIndex]);
											}
										}
									}
								}
							}

							// if not bookmark, or hash task selection set, default to first in list.
							if (!this.getUrlHashParm(Constants.HASH_TASK_SELECTION)) {
								// check to make sure we have tasks
								if (this._taskItems && this._taskItems.length > 0) {
									firstTaskLink = this._taskItems[0];
									firstTaskLink.select(true);
									this._handleDefaultTaskSelection(firstTaskLink);
								}
							}
							if (this._detailsSelectorLogger.isFinest()) {
								this._detailsSelectorLogger.exiting("", "_populateTaskPane");
							}
						},

						///////////////////////////////////////////////////////////////////////////////////////////
						// Utililty Methods                                                                         //
						///////////////////////////////////////////////////////////////////////////////////////////
						_createTaskLink : function (/*ibm.tivoli.navmodel.Task*/task) {
							// summary:
							//        Utility method to query task and construct appropriate task link
							// tags:
							//     private
							var type = "WIDGET", relationship = "", tLink, icon, iconPath, taskJavascript, taskLaunchObj, taskId, path, pathKey, dijitId, linkId;
							// setup task icon informaton, check if provide, if not default
							if (task.getImage()) {
								icon = task.getImage();
								iconPath = "";
							} else {
								// default to task icon provided by details selector, and set path.
								icon = "task_16.gif";
								iconPath = this._imagePath;
							}

							taskJavascript = task.getLaunchJavascript();
							// Task's launch JavaScript must be valid json
							taskLaunchObj = dojo.fromJson(taskJavascript);

							// TaskLaunchObj expectation
							//{"launchType:'Widget',
							//"dijit:'discovery/Discover',
							//"path:'/com.ibm.usmi.console.samples/js/usmi-console-samples'}

							// get widget information to launch.
							if (taskLaunchObj) {
								if (taskLaunchObj.launchType === 'Widget') {
									// set task type
									type = "WIDGET";
									path = taskLaunchObj.path;
									pathKey = taskLaunchObj.pathKey;
									dijitId = taskLaunchObj.dijit;
									taskId = dijitId;
									linkId = taskId;
								} // if related resource, we need to gather some more information
								// to pass on the publish
								else if (taskLaunchObj.launchType === 'RelatedResource') {
									// set task type
									type = "RELATED_RESOURCE";
									path = taskLaunchObj.path;
									pathKey = taskLaunchObj.pathKey;
									dijitId = taskLaunchObj.dijit;
									taskId = dijitId;
									linkId = taskLaunchObj.relationship;
									relationship = taskLaunchObj.relationship;
								} else if (taskLaunchObj.launchType === 'URL') {
									// set task type
									type = "URL";
									taskId = taskLaunchObj.urlToLaunch;
									linkId = taskId;
								}
							}

							// default to taskId if wasnt specified
							if (!taskId) {
								taskId = task.getId();
								linkId = taskId;
							}

							// create a TaskLink
							tLink = new TaskLink({
								taskId : linkId,
								type : type,
								link : this.updateUrlHash(Constants.HASH_TASK_SELECTION, taskId),
								resourceLabel : this.resource,
								label : task.getLabel(),
								path : path,
								pathKey : pathKey,
								criteriaString : relationship,
								icon : icon,
								iconPath : iconPath
							});

							return tLink;
						},

						_handleDefaultTaskSelection : function (/*ibm.usmi.widgets.TaskLink*/dLink) {
							if (this._detailsSelectorLogger.isFinest()) {
								this._detailsSelectorLogger.entering("", "_handleDefaultTaskSelection");
							}
							var obj, launchParms, targetInfo;
							// if task type is widget, hash the link
							if (dLink.type.toUpperCase() === Constants.TASK_TYPE_WIDGET) {
								if (this._detailsSelectorLogger.isFiner()) {
									this._detailsSelectorLogger.finer(dLink.link, "_handleDefaultTaskSelection");
								}

								obj = dojo.queryToObject(dLink.link);

								// add the taskPath and taskPathKey if were set
								if (dLink.path && dLink.pathKey) {
									obj.taskPath = dLink.path;
									obj.taskPathKey = dLink.pathKey;
								}

								dojo.hash(dojo.objectToQuery(obj), true);
								targetInfo = {
									sourceToken : this.sourceToken,
									taskSelection : obj.taskSelection,
									taskPath : dLink.path,
									taskPathKey : dLink.pathKey
								};
								// publish the label, so workarea can render title
								dojo.publish(ibm.usmi.widgets.util.Constants.TOPIC_TASK_TITLE_CHANGE, {
									resource : dLink.getResourceLabel(),
									view : dLink.getLabel(),
									loading : false
								});
								topic.publish(Constants.TOPIC_RETARGET_INFO, targetInfo); //tell WorkArea the new info

							} else if (dLink.type.toUpperCase() === Constants.TASK_TYPE_RELATED_RESOURCE) {
								if (this._detailsSelectorLogger.isFiner()) {
									this._detailsSelectorLogger.finer(dLink, "_handleDefaultTaskSelection");
								}
								obj = dojo.queryToObject(dLink.link);
								obj.relationship = dLink.criteriaString;
								dojo.hash(dojo.objectToQuery(obj), true); // set
								targetInfo = {
									relationship : dLink.criteriaString,
									sourceToken : this.sourceToken,
									taskSelection : obj.taskSelection
								};
								topic.publish(Constants.TOPIC_RETARGET_INFO, targetInfo); //tell RRTable the new info

							}
							// if is URL, publish the URL
							else if (dLink.type.toUpperCase() === Constants.TASK_TYPE_URL) {
								launchParms = {
									topic : Constants.TOPIC_DEFAULT_URL_LAUNCH,
									launchType : Constants.TASK_TYPE_URL,
									urlToLaunch : dLink.link
								};

								topic.publish(Constants.TOPIC_DEFAULT_URL_LAUNCH, launchParms);
							}

							if (this._detailsSelectorLogger.isFinest()) {
								this._detailsSelectorLogger.exiting("", "_handleDefaultTaskSelection");
							}
						},

						// TODO this is a place holder for handling dataset events...
						// when an event is fired on a dataset we're listening to, refresh the view.
						// Note: many events may come through really quickly, so after 3 seconds of no more events, the refresh will fire.
						_eventHandler : function (listenerObject, event) {
							// summary:
							//         Handle dataset events
							// tags:
							//     callback
							if (this._detailsSelectorLogger.isFinest()) {
								this._detailsSelectorLogger.finest("DETAILS SELECTOR- event datasetId="
										+ event.getDataset().getId() + " event=" + event.getType() + " eventId="
										+ event.getId(), "_eventHandler");
							}

							var itemID, property;
							switch (event.getType()) {
							case "itemPropertiesChanged":
								// This is the event that is fired when a memeber is added / removed
								itemID = event.getItems()[0].getId();

								// get count of property, set new value		                	
								property = event.getProperties()[0][0];
								if (property.getId() === "DisplayName") {
									this._updateProperty(property, listenerObject);
								} else if (property.getId() === "STATUS") {
									this._updateProperty(property, listenerObject);
								}

								break;
							default:
								break;
							}

							if (this._detailsSelectorLogger.isFinest()) {
								this._detailsSelectorLogger.exiting("", "eventHandler");
							}

							//        clearTimeout(this._eventTimeout); // reset the timeout window.
							//        if ( this._initialEventTime == null ) {
							//            this._initialEventTime = new Date();
							//            this._eventTimeout = setTimeout(dojo.hitch(this, this._refreshView, "evented"), 5000);
							//        // FORCE a refresh if blocked for 30 seconds or more by constant incoming events.
							//        } else if ( this._initialEventTime.getTime() - (new Date()).getTime() > 30000 ){
							//            this._refreshView("evented");
							//        } else
							//            this._eventTimeout = setTimeout(dojo.hitch(this, this._refreshView, "evented"), 5000);
						},

						_updateProperty : function (changedProp, obj) {
							var propDiv = this._propertyRenderer.getRenderedPropertyNode(obj._item, changedProp, obj);
							propDiv.id = changedProp.getId();
							dojo.place(propDiv, propDiv.id, "replace");
						},

						///////////////////////////////////////////////////////////////////////////////////////////
						// Failure Methods                                                                         //
						///////////////////////////////////////////////////////////////////////////////////////////
						_onItemsRetrievalFailed : function (error, request) {
							if (this._detailsSelectorLogger.isError()) {
								this._detailsSelectorLogger.error("The items request to the datastore failed.  "
										+ error, "_onItemsRetrievalFailed");
								this._detailsSelectorLogger.error("request: " + request, "_onItemsRetrievalFailed");
							}

							var msgHandler = new DefaultMessageHandler({});
							msgHandler.displayException(error);
						},

						_onPropertiesDataRetrievedFailed : function (error, request) {
							if (this._detailsSelectorLogger.isError()) {
								this._detailsSelectorLogger.error("The properties request to the datastore failed.  "
										+ error, "_onPropertiesDataRetrievedFailed");
								this._detailsSelectorLogger.error("request: " + request,
										"_onPropertiesDataRetrievedFailed");
							}

							var msgHandler = new DefaultMessageHandler({});
							msgHandler.displayException(error);
						},

						// TYPE: ERROR HANDLING
						_onTasksError : function (error, request) {
							if (this._detailsSelectorLogger.isError()) {
								this._detailsSelectorLogger.error("The tasks request to the datastore failed.  "
										+ error, "_onTasksError");
								this._detailsSelectorLogger.error("request: " + request, "_onTasksError");
							}

							var msgHandler = new DefaultMessageHandler({});
							msgHandler.displayException(error);
						},

						/////////////////////////////////////////////////////////////////////////////////////
						// Gets for private vars
						////////////////////////////////////////////////////////////////////////////////////
						getDataSet : function () {
							return this.dataset;
						}
					});
		});

},
'dojo/touch':function(){
define(["./_base/kernel", "./on", "./has", "./mouse"], function(dojo, on, has, mouse){
// module:
//		dojo/touch

/*=====
	dojo.touch = {
		// summary:
		//		This module provides unified touch event handlers by exporting
		//		press, move, release and cancel which can also run well on desktop.
		//		Based on http://dvcs.w3.org/hg/webevents/raw-file/tip/touchevents.html
		//
		// example:
		//		1. Used with dojo.connect()
		//		|	dojo.connect(node, dojo.touch.press, function(e){});
		//		|	dojo.connect(node, dojo.touch.move, function(e){});
		//		|	dojo.connect(node, dojo.touch.release, function(e){});
		//		|	dojo.connect(node, dojo.touch.cancel, function(e){});
		//
		//		2. Used with dojo.on
		//		|	define(["dojo/on", "dojo/touch"], function(on, touch){
		//		|		on(node, touch.press, function(e){});
		//		|		on(node, touch.move, function(e){});
		//		|		on(node, touch.release, function(e){});
		//		|		on(node, touch.cancel, function(e){});
		//
		//		3. Used with dojo.touch.* directly
		//		|	dojo.touch.press(node, function(e){});
		//		|	dojo.touch.move(node, function(e){});
		//		|	dojo.touch.release(node, function(e){});
		//		|	dojo.touch.cancel(node, function(e){});
		
		press: function(node, listener){
			// summary:
			//		Register a listener to 'touchstart'|'mousedown' for the given node
			// node: Dom
			//		Target node to listen to
			// listener: Function
			//		Callback function
			// returns:
			//		A handle which will be used to remove the listener by handle.remove()
		},
		move: function(node, listener){
			// summary:
			//		Register a listener to 'touchmove'|'mousemove' for the given node
			// node: Dom
			//		Target node to listen to
			// listener: Function
			//		Callback function
			// returns:
			//		A handle which will be used to remove the listener by handle.remove()
		},
		release: function(node, listener){
			// summary:
			//		Register a listener to 'touchend'|'mouseup' for the given node
			// node: Dom
			//		Target node to listen to
			// listener: Function
			//		Callback function
			// returns:
			//		A handle which will be used to remove the listener by handle.remove()
		},
		cancel: function(node, listener){
			// summary:
			//		Register a listener to 'touchcancel'|'mouseleave' for the given node
			// node: Dom
			//		Target node to listen to
			// listener: Function
			//		Callback function
			// returns:
			//		A handle which will be used to remove the listener by handle.remove()
		}
	};
=====*/

	function _handle(/*String - press | move | release | cancel*/type){
		return function(node, listener){//called by on(), see dojo.on
			return on(node, type, listener);
		};
	}
	var touch = has("touch");
	//device neutral events - dojo.touch.press|move|release|cancel
	dojo.touch = {
		press: _handle(touch ? "touchstart": "mousedown"),
		move: _handle(touch ? "touchmove": "mousemove"),
		release: _handle(touch ? "touchend": "mouseup"),
		cancel: touch ? _handle("touchcancel") : mouse.leave
	};
	return dojo.touch;
});
},
'dojo/io/iframe':function(){
define(["../main", "require"], function(dojo, require) {
	// module:
	//		dojo/io/iframe
	// summary:
	//		TODOC

dojo.getObject("io", true, dojo);

/*=====
dojo.declare("dojo.io.iframe.__ioArgs", dojo.__IoArgs, {
	constructor: function(){
		//	summary:
		//		All the properties described in the dojo.__ioArgs type, apply
		//		to this type. The following additional properties are allowed
		//		for dojo.io.iframe.send():
		//	method: String?
		//		The HTTP method to use. "GET" or "POST" are the only supported
		//		values.  It will try to read the value from the form node's
		//		method, then try this argument. If neither one exists, then it
		//		defaults to POST.
		//	handleAs: String?
		//		Specifies what format the result data should be given to the
		//		load/handle callback. Valid values are: text, html, xml, json,
		//		javascript. IMPORTANT: For all values EXCEPT html and xml, The
		//		server response should be an HTML file with a textarea element.
		//		The response data should be inside the textarea element. Using an
		//		HTML document the only reliable, cross-browser way this
		//		transport can know when the response has loaded. For the html
		//		handleAs value, just return a normal HTML document.  NOTE: xml
		//		is now supported with this transport (as of 1.1+); a known issue
		//		is if the XML document in question is malformed, Internet Explorer
		//		will throw an uncatchable error.
		//	content: Object?
		//		If "form" is one of the other args properties, then the content
		//		object properties become hidden form form elements. For
		//		instance, a content object of {name1 : "value1"} is converted
		//		to a hidden form element with a name of "name1" and a value of
		//		"value1". If there is not a "form" property, then the content
		//		object is converted into a name=value&name=value string, by
		//		using dojo.objectToQuery().
		this.method = method;
		this.handleAs = handleAs;
		this.content = content;
	}
});
=====*/

dojo.io.iframe = {
	// summary:
	//		Sends an Ajax I/O call using and Iframe (for instance, to upload files)

	create: function(/*String*/fname, /*String*/onloadstr, /*String?*/uri){
		//	summary:
		//		Creates a hidden iframe in the page. Used mostly for IO
		//		transports.  You do not need to call this to start a
		//		dojo.io.iframe request. Just call send().
		//	fname: String
		//		The name of the iframe. Used for the name attribute on the
		//		iframe.
		//	onloadstr: String
		//		A string of JavaScript that will be executed when the content
		//		in the iframe loads.
		//	uri: String
		//		The value of the src attribute on the iframe element. If a
		//		value is not given, then dojo/resources/blank.html will be
		//		used.
		if(window[fname]){ return window[fname]; }
		if(window.frames[fname]){ return window.frames[fname]; }
		var turi = uri;
		if(!turi){
			if(dojo.config["useXDomain"] && !dojo.config["dojoBlankHtmlUrl"]){
				console.warn("dojo.io.iframe.create: When using cross-domain Dojo builds,"
					+ " please save dojo/resources/blank.html to your domain and set djConfig.dojoBlankHtmlUrl"
					+ " to the path on your domain to blank.html");
			}
			turi = (dojo.config["dojoBlankHtmlUrl"]||require.toUrl("../resources/blank.html"));
		}
		var cframe = dojo.place(
			'<iframe id="'+fname+'" name="'+fname+'" src="'+turi+'" onload="'+onloadstr+
			'" style="position: absolute; left: 1px; top: 1px; height: 1px; width: 1px; visibility: hidden">',
		dojo.body());

		window[fname] = cframe;

		return cframe;
	},

	setSrc: function(/*DOMNode*/iframe, /*String*/src, /*Boolean*/replace){
		//summary:
		//		Sets the URL that is loaded in an IFrame. The replace parameter
		//		indicates whether location.replace() should be used when
		//		changing the location of the iframe.
		try{
			if(!replace){
				if(dojo.isWebKit){
					iframe.location = src;
				}else{
					frames[iframe.name].location = src;
				}
			}else{
				// Fun with DOM 0 incompatibilities!
				var idoc;
				if(dojo.isIE || dojo.isWebKit){
					idoc = iframe.contentWindow.document;
				}else{ //  if(d.isMozilla){
					idoc = iframe.contentWindow;
				}

				//For Safari (at least 2.0.3) and Opera, if the iframe
				//has just been created but it doesn't have content
				//yet, then iframe.document may be null. In that case,
				//use iframe.location and return.
				if(!idoc){
					iframe.location = src;
				}else{
					idoc.location.replace(src);
				}
			}
		}catch(e){
			console.log("dojo.io.iframe.setSrc: ", e);
		}
	},

	doc: function(/*DOMNode*/iframeNode){
		//summary: Returns the document object associated with the iframe DOM Node argument.
		return iframeNode.contentDocument || // W3
			(
				(
					(iframeNode.name) && (iframeNode.document) &&
					(dojo.doc.getElementsByTagName("iframe")[iframeNode.name].contentWindow) &&
					(dojo.doc.getElementsByTagName("iframe")[iframeNode.name].contentWindow.document)
				)
			) ||  // IE
			(
				(iframeNode.name)&&(dojo.doc.frames[iframeNode.name])&&
				(dojo.doc.frames[iframeNode.name].document)
			) || null;
	},

	send: function(/*dojo.io.iframe.__ioArgs*/args){
		//summary:
		//		Function that sends the request to the server.
		//		This transport can only process one send() request at a time, so if send() is called
		//multiple times, it will queue up the calls and only process one at a time.
		if(!this["_frame"]){
			this._frame = this.create(this._iframeName, dojo._scopeName + ".io.iframe._iframeOnload();");
		}

		//Set up the deferred.
		var dfd = dojo._ioSetArgs(
			args,
			function(/*Deferred*/dfd){
				//summary: canceller function for dojo._ioSetArgs call.
				dfd.canceled = true;
				dfd.ioArgs._callNext();
			},
			function(/*Deferred*/dfd){
				//summary: okHandler function for dojo._ioSetArgs call.
				var value = null;
				try{
					var ioArgs = dfd.ioArgs;
					var dii = dojo.io.iframe;
					var ifd = dii.doc(dii._frame);
					var handleAs = ioArgs.handleAs;

					//Assign correct value based on handleAs value.
					value = ifd; //html
					if(handleAs != "html"){
						if(handleAs == "xml"){
							//	FF, Saf 3+ and Opera all seem to be fine with ifd being xml.  We have to
							//	do it manually for IE6-8.  Refs #6334.
							if(dojo.isIE < 9 || (dojo.isIE && dojo.isQuirks)){
								dojo.query("a", dii._frame.contentWindow.document.documentElement).orphan();
								var xmlText=(dii._frame.contentWindow.document).documentElement.innerText;
								xmlText=xmlText.replace(/>\s+</g, "><");
								xmlText=dojo.trim(xmlText);
								//Reusing some code in base dojo for handling XML content.  Simpler and keeps
								//Core from duplicating the effort needed to locate the XML Parser on IE.
								var fauxXhr = { responseText: xmlText };
								value = dojo._contentHandlers["xml"](fauxXhr); // DOMDocument
							}
						}else{
							value = ifd.getElementsByTagName("textarea")[0].value; //text
							if(handleAs == "json"){
								value = dojo.fromJson(value); //json
							}else if(handleAs == "javascript"){
								value = dojo.eval(value); //javascript
							}
						}
					}
				}catch(e){
					value = e;
				}finally{
					ioArgs._callNext();
				}
				return value;
			},
			function(/*Error*/error, /*Deferred*/dfd){
				//summary: errHandler function for dojo._ioSetArgs call.
				dfd.ioArgs._hasError = true;
				dfd.ioArgs._callNext();
				return error;
			}
		);

		//Set up a function that will fire the next iframe request. Make sure it only
		//happens once per deferred.
		dfd.ioArgs._callNext = function(){
			if(!this["_calledNext"]){
				this._calledNext = true;
				dojo.io.iframe._currentDfd = null;
				dojo.io.iframe._fireNextRequest();
			}
		};

		this._dfdQueue.push(dfd);
		this._fireNextRequest();

		//Add it the IO watch queue, to get things like timeout support.
		dojo._ioWatch(
			dfd,
			function(/*Deferred*/dfd){
				//validCheck
				return !dfd.ioArgs["_hasError"];
			},
			function(dfd){
				//ioCheck
				return (!!dfd.ioArgs["_finished"]);
			},
			function(dfd){
				//resHandle
				if(dfd.ioArgs._finished){
					dfd.callback(dfd);
				}else{
					dfd.errback(new Error("Invalid dojo.io.iframe request state"));
				}
			}
		);

		return dfd;
	},

	_currentDfd: null,
	_dfdQueue: [],
	_iframeName: dojo._scopeName + "IoIframe",

	_fireNextRequest: function(){
		//summary: Internal method used to fire the next request in the bind queue.
		try{
			if((this._currentDfd)||(this._dfdQueue.length == 0)){ return; }
			//Find next deferred, skip the canceled ones.
			do{
				var dfd = this._currentDfd = this._dfdQueue.shift();
			} while(dfd && dfd.canceled && this._dfdQueue.length);

			//If no more dfds, cancel.
			if(!dfd || dfd.canceled){
				this._currentDfd =  null;
				return;
			}

			var ioArgs = dfd.ioArgs;
			var args = ioArgs.args;

			ioArgs._contentToClean = [];
			var fn = dojo.byId(args["form"]);
			var content = args["content"] || {};
			if(fn){
				if(content){
					// if we have things in content, we need to add them to the form
					// before submission
					var pHandler = function(name, value) {
						dojo.create("input", {type: "hidden", name: name, value: value}, fn);
						ioArgs._contentToClean.push(name);
					};
					for(var x in content){
						var val = content[x];
						if(dojo.isArray(val) && val.length > 1){
							var i;
							for (i = 0; i < val.length; i++) {
								pHandler(x,val[i]);
							}
						}else{
							if(!fn[x]){
								pHandler(x,val);
							}else{
								fn[x].value = val;
							}
						}
					}
				}
				//IE requires going through getAttributeNode instead of just getAttribute in some form cases,
				//so use it for all.  See #2844
				var actnNode = fn.getAttributeNode("action");
				var mthdNode = fn.getAttributeNode("method");
				var trgtNode = fn.getAttributeNode("target");
				if(args["url"]){
					ioArgs._originalAction = actnNode ? actnNode.value : null;
					if(actnNode){
						actnNode.value = args.url;
					}else{
						fn.setAttribute("action",args.url);
					}
				}
				if(!mthdNode || !mthdNode.value){
					if(mthdNode){
						mthdNode.value= (args["method"]) ? args["method"] : "post";
					}else{
						fn.setAttribute("method", (args["method"]) ? args["method"] : "post");
					}
				}
				ioArgs._originalTarget = trgtNode ? trgtNode.value: null;
				if(trgtNode){
					trgtNode.value = this._iframeName;
				}else{
					fn.setAttribute("target", this._iframeName);
				}
				fn.target = this._iframeName;
				dojo._ioNotifyStart(dfd);
				fn.submit();
			}else{
				// otherwise we post a GET string by changing URL location for the
				// iframe
				var tmpUrl = args.url + (args.url.indexOf("?") > -1 ? "&" : "?") + ioArgs.query;
				dojo._ioNotifyStart(dfd);
				this.setSrc(this._frame, tmpUrl, true);
			}
		}catch(e){
			dfd.errback(e);
		}
	},

	_iframeOnload: function(){
		var dfd = this._currentDfd;
		if(!dfd){
			this._fireNextRequest();
			return;
		}

		var ioArgs = dfd.ioArgs;
		var args = ioArgs.args;
		var fNode = dojo.byId(args.form);

		if(fNode){
			// remove all the hidden content inputs
			var toClean = ioArgs._contentToClean;
			for(var i = 0; i < toClean.length; i++) {
				var key = toClean[i];
				//Need to cycle over all nodes since we may have added
				//an array value which means that more than one node could
				//have the same .name value.
				for(var j = 0; j < fNode.childNodes.length; j++){
					var chNode = fNode.childNodes[j];
					if(chNode.name == key){
						dojo.destroy(chNode);
						break;
					}
				}
			}

			// restore original action + target
			if(ioArgs["_originalAction"]){
				fNode.setAttribute("action", ioArgs._originalAction);
			}
			if(ioArgs["_originalTarget"]){
				fNode.setAttribute("target", ioArgs._originalTarget);
				fNode.target = ioArgs._originalTarget;
			}
		}

		ioArgs._finished = true;
	}
};

return dojo.io.iframe;
});

},
'dojox/grid/_ViewManager':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/sniff",
	"dojo/dom-class"
], function(declare, has, domClass){

return declare('dojox.grid._ViewManager', null, {
	// summary:
	//		A collection of grid views. Owned by grid and used internally for managing grid views.
	// description:
	//		Grid creates views automatically based on grid's layout structure.
	//		Users should typically not need to access individual views or the views collection directly.
	constructor: function(inGrid){
		this.grid = inGrid;
	},

	defaultWidth: 200,

	views: [],

	// operations
	resize: function(){
		this.onEach("resize");
	},

	render: function(){
		this.onEach("render");
	},

	// views
	addView: function(inView){
		inView.idx = this.views.length;
		this.views.push(inView);
	},

	destroyViews: function(){
		for(var i=0, v; v=this.views[i]; i++){
			v.destroy();
		}
		this.views = [];
	},

	getContentNodes: function(){
		var nodes = [];
		for(var i=0, v; v=this.views[i]; i++){
			nodes.push(v.contentNode);
		}
		return nodes;
	},

	forEach: function(inCallback){
		for(var i=0, v; v=this.views[i]; i++){
			inCallback(v, i);
		}
	},

	onEach: function(inMethod, inArgs){
		inArgs = inArgs || [];
		for(var i=0, v; v=this.views[i]; i++){
			if(inMethod in v){
				v[inMethod].apply(v, inArgs);
			}
		}
	},

	// layout
	normalizeHeaderNodeHeight: function(){
		var rowNodes = [];
		for(var i=0, v; (v=this.views[i]); i++){
			if(v.headerContentNode.firstChild){
				rowNodes.push(v.headerContentNode);
			}
		}
		this.normalizeRowNodeHeights(rowNodes);
	},

	normalizeRowNodeHeights: function(inRowNodes){
		var h = 0;
		var currHeights = [];
		if(this.grid.rowHeight){
			h = this.grid.rowHeight;
		}else{
			if(inRowNodes.length <= 1){
				// no need to normalize if we are the only one...
				return;
			}
			for(var i=0, n; (n=inRowNodes[i]); i++){
				// We only care about the height - so don't use marginBox.  This
				// depends on the container not having any margin (which it shouldn't)
				// Also - we only look up the height if the cell doesn't have the
				// dojoxGridNonNormalizedCell class (like for row selectors)
				if(!domClass.contains(n, "dojoxGridNonNormalizedCell")){
					currHeights[i] = n.firstChild.offsetHeight;
					h =  Math.max(h, currHeights[i]);
				}
			}
			h = (h >= 0 ? h : 0);
	
			//Work around odd FF3 rendering bug: #8864.
			//A one px increase fixes FireFox 3's rounding bug for fractional font sizes.
			if((has("mozilla") || has("ie") > 8 ) && h){h++;}
		}
		for(i=0; (n=inRowNodes[i]); i++){
			if(currHeights[i] != h){
				n.firstChild.style.height = h + "px";
			}
		}
	},
	
	resetHeaderNodeHeight: function(){
		for(var i=0, v, n; (v=this.views[i]); i++){
			n = v.headerContentNode.firstChild;
			if(n){
				n.style.height = "";
			}
		}
	},

	renormalizeRow: function(inRowIndex){
		var rowNodes = [];
		for(var i=0, v, n; (v=this.views[i])&&(n=v.getRowNode(inRowIndex)); i++){
			n.firstChild.style.height = '';
			rowNodes.push(n);
		}
		this.normalizeRowNodeHeights(rowNodes);
	},

	getViewWidth: function(inIndex){
		return this.views[inIndex].getWidth() || this.defaultWidth;
	},

	// must be called after view widths are properly set or height can be miscalculated
	// if there are flex columns
	measureHeader: function(){
		// need to reset view header heights so they are properly measured.
		this.resetHeaderNodeHeight();
		this.forEach(function(inView){
			inView.headerContentNode.style.height = '';
		});
		var h = 0;
		// calculate maximum view header height
		this.forEach(function(inView){
			h = Math.max(inView.headerNode.offsetHeight, h);
		});
		return h;
	},

	measureContent: function(){
		var h = 0;
		this.forEach(function(inView){
			h = Math.max(inView.domNode.offsetHeight, h);
		});
		return h;
	},

	findClient: function(inAutoWidth){
		// try to use user defined client
		var c = this.grid.elasticView || -1;
		// attempt to find implicit client
		if(c < 0){
			for(var i=1, v; (v=this.views[i]); i++){
				if(v.viewWidth){
					for(i=1; (v=this.views[i]); i++){
						if(!v.viewWidth){
							c = i;
							break;
						}
					}
					break;
				}
			}
		}
		// client is in the middle by default
		if(c < 0){
			c = Math.floor(this.views.length / 2);
		}
		return c;
	},

	arrange: function(l, w){
		var i, v, vw, len = this.views.length, self = this;
		// find the client
		var c = (w <= 0 ? len : this.findClient());
		// layout views
		var setPosition = function(v, l){
			var ds = v.domNode.style;
			var hs = v.headerNode.style;

			if(!self.grid.isLeftToRight()){
				ds.right = l + 'px';
				// fixed rtl, the scrollbar is on the right side in FF or WebKit
				if (has("ff") < 4 || has("webkit")){
					hs.right = l + v.getScrollbarWidth() + 'px';
					hs.width = parseInt(hs.width, 10) - v.getScrollbarWidth() + 'px';
				}else{
					hs.right = l + 'px';
				}
			}else{
				ds.left = l + 'px';
				hs.left = l + 'px';
			}
			ds.top = 0 + 'px';
			hs.top = 0;
		};
		// for views left of the client
		//BiDi TODO: The left and right should not appear in BIDI environment. Should be replaced with
		//leading and tailing concept.
		for(i=0; (v=this.views[i])&&(i<c); i++){
			// get width
			vw = this.getViewWidth(i);
			// process boxes
			v.setSize(vw, 0);
			setPosition(v, l);
			if(v.headerContentNode && v.headerContentNode.firstChild){
				vw = v.getColumnsWidth()+v.getScrollbarWidth();
			}else{
				vw = v.domNode.offsetWidth;
			}
			// update position
			l += vw;
		}
		// next view (is the client, i++ == c)
		i++;
		// start from the right edge
		var r = w;
		// for views right of the client (iterated from the right)
		for(var j=len-1; (v=this.views[j])&&(i<=j); j--){
			// get width
			vw = this.getViewWidth(j);
			// set size
			v.setSize(vw, 0);
			// measure in pixels
			vw = v.domNode.offsetWidth;
			// update position
			r -= vw;
			// set position
			setPosition(v, r);
		}
		if(c<len){
			v = this.views[c];
			// position the client box between left and right boxes
			vw = Math.max(1, r-l);
			// set size
			v.setSize(vw + 'px', 0);
			setPosition(v, l);
		}
		return l;
	},

	// rendering
	renderRow: function(inRowIndex, inNodes, skipRenorm){
		var rowNodes = [];
		for(var i=0, v, n, rowNode; (v=this.views[i])&&(n=inNodes[i]); i++){
			rowNode = v.renderRow(inRowIndex);
			n.appendChild(rowNode);
			rowNodes.push(rowNode);
		}
		if(!skipRenorm){
			this.normalizeRowNodeHeights(rowNodes);
		}
	},
	
	rowRemoved: function(inRowIndex){
		this.onEach("rowRemoved", [ inRowIndex ]);
	},
	
	// updating
	updateRow: function(inRowIndex, skipRenorm){
		for(var i=0, v; v=this.views[i]; i++){
			v.updateRow(inRowIndex);
		}
		if(!skipRenorm){
			this.renormalizeRow(inRowIndex);
		}
	},
	
	updateRowStyles: function(inRowIndex){
		this.onEach("updateRowStyles", [ inRowIndex ]);
	},
	
	// scrolling
	setScrollTop: function(inTop){
		var top = inTop;
		for(var i=0, v; v=this.views[i]; i++){
			top = v.setScrollTop(inTop);
			// Work around IE not firing scroll events that cause header offset
			// issues to occur.
			if(has("ie") && v.headerNode && v.scrollboxNode){
				v.headerNode.scrollLeft = v.scrollboxNode.scrollLeft;
			}
		}
		return top;
		//this.onEach("setScrollTop", [ inTop ]);
	},
	
	getFirstScrollingView: function(){
		// summary: Returns the first grid view with a scroll bar
		for(var i=0, v; (v=this.views[i]); i++){
			if(v.hasHScrollbar() || v.hasVScrollbar()){
				return v;
			}
		}
		return null;
	}
});
});
},
'dijit/registry':function(){
define([
	"dojo/_base/array", // array.forEach array.map
	"dojo/_base/sniff", // has("ie")
	"dojo/_base/unload", // unload.addOnWindowUnload
	"dojo/_base/window", // win.body
	"."	// dijit._scopeName
], function(array, has, unload, win, dijit){

	// module:
	//		dijit/registry
	// summary:
	//		Registry of existing widget on page, plus some utility methods.
	//		Must be accessed through AMD api, ex:
	//		require(["dijit/registry"], function(registry){ registry.byId("foo"); })

	var _widgetTypeCtr = {}, hash = {};

	var registry =  {
		// summary:
		//		A set of widgets indexed by id

		length: 0,

		add: function(/*dijit._Widget*/ widget){
			// summary:
			//		Add a widget to the registry. If a duplicate ID is detected, a error is thrown.
			//
			// widget: dijit._Widget
			//		Any dijit._Widget subclass.
			if(hash[widget.id]){
				throw new Error("Tried to register widget with id==" + widget.id + " but that id is already registered");
			}
			hash[widget.id] = widget;
			this.length++;
		},

		remove: function(/*String*/ id){
			// summary:
			//		Remove a widget from the registry. Does not destroy the widget; simply
			//		removes the reference.
			if(hash[id]){
				delete hash[id];
				this.length--;
			}
		},

		byId: function(/*String|Widget*/ id){
			// summary:
			//		Find a widget by it's id.
			//		If passed a widget then just returns the widget.
			return typeof id == "string" ? hash[id] : id;	// dijit._Widget
		},

		byNode: function(/*DOMNode*/ node){
			// summary:
			//		Returns the widget corresponding to the given DOMNode
			return hash[node.getAttribute("widgetId")]; // dijit._Widget
		},

		toArray: function(){
			// summary:
			//		Convert registry into a true Array
			//
			// example:
			//		Work with the widget .domNodes in a real Array
			//		|	array.map(dijit.registry.toArray(), function(w){ return w.domNode; });

			var ar = [];
			for(var id in hash){
				ar.push(hash[id]);
			}
			return ar;	// dijit._Widget[]
		},

		getUniqueId: function(/*String*/widgetType){
			// summary:
			//		Generates a unique id for a given widgetType

			var id;
			do{
				id = widgetType + "_" +
					(widgetType in _widgetTypeCtr ?
						++_widgetTypeCtr[widgetType] : _widgetTypeCtr[widgetType] = 0);
			}while(hash[id]);
			return dijit._scopeName == "dijit" ? id : dijit._scopeName + "_" + id; // String
		},

		findWidgets: function(/*DomNode*/ root){
			// summary:
			//		Search subtree under root returning widgets found.
			//		Doesn't search for nested widgets (ie, widgets inside other widgets).

			var outAry = [];

			function getChildrenHelper(root){
				for(var node = root.firstChild; node; node = node.nextSibling){
					if(node.nodeType == 1){
						var widgetId = node.getAttribute("widgetId");
						if(widgetId){
							var widget = hash[widgetId];
							if(widget){	// may be null on page w/multiple dojo's loaded
								outAry.push(widget);
							}
						}else{
							getChildrenHelper(node);
						}
					}
				}
			}

			getChildrenHelper(root);
			return outAry;
		},

		_destroyAll: function(){
			// summary:
			//		Code to destroy all widgets and do other cleanup on page unload

			// Clean up focus manager lingering references to widgets and nodes
			dijit._curFocus = null;
			dijit._prevFocus = null;
			dijit._activeStack = [];

			// Destroy all the widgets, top down
			array.forEach(registry.findWidgets(win.body()), function(widget){
				// Avoid double destroy of widgets like Menu that are attached to <body>
				// even though they are logically children of other widgets.
				if(!widget._destroyed){
					if(widget.destroyRecursive){
						widget.destroyRecursive();
					}else if(widget.destroy){
						widget.destroy();
					}
				}
			});
		},

		getEnclosingWidget: function(/*DOMNode*/ node){
			// summary:
			//		Returns the widget whose DOM tree contains the specified DOMNode, or null if
			//		the node is not contained within the DOM tree of any widget
			while(node){
				var id = node.getAttribute && node.getAttribute("widgetId");
				if(id){
					return hash[id];
				}
				node = node.parentNode;
			}
			return null;
		},

		// In case someone needs to access hash.
		// Actually, this is accessed from WidgetSet back-compatibility code
		_hash: hash
	};

	/*=====
	dijit.registry = {
		// summary:
		//		A list of widgets on a page.
	};
	=====*/
	dijit.registry = registry;

	return registry;
});

},
'ibm/tivoli/navmodel/Selection':function(){
// wrapped by build app
define("ibm/tivoli/navmodel/Selection", ["dijit","dojo","dojox"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Selection"); 
 
////////////////////////////////////
// Selection
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Selection", null, {
	// private members
	_version: null,
	_selection: null,
	
	/*=====
	constructor_kwArgs: function(version) {
		// version: Float
		//		version used to communicate with rest service
	},
	=====*/

	constructor: function(/*constructor_kwArgs*/ kwArgs) {		
		// summary:
		//		Constructs Selection object.
		this._version = kwArgs.version;
		this.clear();
	},
	
	/////////////////////////////
	// public methods
	/////////////////////////////

	getAttributes: function() {		
		// summary:
		//		Returns selection value
		return this._selection;
	},

	clear: function() {		
		// summary:
		//		Clear out selection contents
		this._selection = {selected: false, itemIds: [], linkIds: [], relationshipIds: []};
	},

	isEmpty: function() {		
		// summary:
		//		Returns true if selection is empty (i.e. no itemIds or linkIds)
		return this._isEmpty(this._selection);
	},

	addItem: function(itemId) {		
		// summary:
		//		Add an item id to the selection
		var selection = this._getSelection();		
		if (selection) {
			this._addItem(selection, itemId);
		}
	},

	addItems: function(itemIds) {		
		// summary:
		//		Add multiple item ids to the selection
		var idsLength = itemIds.length;
		for (var idIndex = 0;  idIndex < idsLength;  ++idIndex) {
			var selection = this._getSelection();		
			if (selection) {
				this._addItem(selection, itemIds[idIndex]);
			}
		}		
	},

	removeItem: function(itemId) {		
		// summary:
		//		Remove an item id from the selection
		var selection = this._getSelection();		
		if (selection) {
			this._removeItem(selection, itemId);						
		}
		
		// delete any empty selections in the hierarchy
		this._clean(this._selection);
	},

	removeItems: function(itemIds) {		
		// summary:
		//		Remove multiple item ids from the selection
		var idsLength = itemIds.length;
		for (var idIndex = 0;  idIndex < idsLength;  ++idIndex) {
			var selection = this._getSelection(ids);		
			if (selection) {
				this._removeItem(selection, itemIds[idIndex]);						
			}			
		}
		
		// delete any empty selections in the hierarchy
		this._clean(this._selection);
	},

	addLink: function(linkId) {		
		// summary:
		//		Add a link id to the selection
		var selection = this._getSelection();		
		if (selection) {
			this._addLink(selection, linkId);
		}
	},

	addLinks: function(linkIds) {		
		// summary:
		//		Add multiple link ids to the selection
		var idsLength = linkIds.length;
		for (var idIndex = 0;  idIndex < idsLength;  ++idIndex) {
			var selection = this._getSelection();		
			if (selection) {
				this._addLink(selection, linkIds[idIndex]);
			}
		}
	},

	removeLink: function(linkId) {		
		// summary:
		//		Remove a link id from the selection
		var selection = this._getSelection();		
		if (selection) {
			this._removeLink(selection, linkId);			
		}
		
		// delete any empty selections in the hierarchy
		this._clean(this._selection);
	},

	removeLinks: function(linkIds) {		
		// summary:
		//		Remove multiple link ids from the selection
		var idsLength = linkIds.length;
		for (var idIndex = 0;  idIndex < idsLength;  ++idIndex) {
			var selection = this._getSelection();		
			if (selection) {
				this._removeLink(selection, linkIds[idIndex]);			
			}
		}
		
		// delete any empty selections in the hierarchy
		this._clean(this._selection);
	},
	
	addRelationship: function(relationshipId) {		
		// summary:
		//		Add a relationship id to the selectio
		var selection = this._getSelection();		
		if (selection) {
			this._addRelationship(selection, relationshipId);
		}
	},

	addRelationships: function(relationshipIds) {		
		// summary:
		//		Add multiple relationship ids to the selection
		var idsLength = relationshipIds.length;
		for (var idIndex = 0;  idIndex < idsLength;  ++idIndex) {
			var selection = this._getSelection();		
			if (selection) {
				this._addRelationship(selection, relationshipIds[idIndex]);
			}
		}
	},

	removeRelationship: function(elationshipId) {		
		// summary:
		//		Remove a relationship id from the selection
		var selection = this._getSelection();		
		if (selection) {
			this._removeRelationship(selection, relationshipId);			
		}
		
		// delete any empty selections in the hierarchy
		this._clean(this._selection);
	},

	removeRelationships: function(relationshipIds) {		
		// summary:
		//		Remove multiple relationship ids from the selection
		var idsLength = relationshipIds.length;
		for (var idIndex = 0;  idIndex < idsLength;  ++idIndex) {
			var selection = this._getSelection();		
			if (selection) {
				this._removeRelatinship(selection, relationshipIds[idIndex]);			
			}
		}
		
		// delete any empty selections in the hierarchy
		this._clean(this._selection);
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////

	_getSelection: function(ids) {		
		// summary:
		//		Returns the selection object associated
		
		var selection = this._selection;
		return selection;	
	},

	_clean: function(selection) {		
		return this._isEmpty(selection);
	},

	_isEmpty: function(selection) {		
		// summary:
		//		Returns true if the selection is empty
		if (selection.selected) {
			return false;
		}
		
		if (selection.itemIds.length > 0) {
			return false;
		}
		
		if (selection.linkIds.length > 0) {
			return false;
		}
		
		if (selection.relationshipIds.length > 0) {
			return false;
		}
		
		return true;
	},

	_addItem: function(selection, itemId) {		
		// summary:
		//		Add item id to the selection object
		selection.itemIds.push(itemId);
	},
	
	_removeItem: function(selection, itemId) {		
		// summary:
		//		Remove item id from the selection object
		var itemIdsLength = selection.itemIds.length;
		for (var itemIdIndex = 0;  itemIdIndex < itemIdsLength;  ++itemIdIndex) {
			if (itemId === selection.itemIds[itemIdIndex]) {
				selection.itemIds.splice(itemIdIndex, 1);
				break;
			}
		}
	},

	_addLink: function(selection, linkId) {		
		// summary:
		//		Add link id to the selection object
		selection.linkIds.push(linkId);
	},

	_removeLink: function(selection, linkId) {		
		// summary:
		//		Remove link id from the selection object
		var linkIdsLength = selection.linkIds.length;
		for (var linkIdIndex = 0;  linkIdIndex < linkIdsLength;  ++linkIdIndex) {
			if (linkId === selection.linkIds[linkIdIndex]) {
				selection.linkIds.splice(linkIdIndex, 1);
				break;
			}
		}
	},

	_addRelationship: function(selection, relationshipId) {		
		// summary:
		//		Add relationship id to the selection object
		selection.relationshipIds.push(relationshipId);
	},

	_removeRelationship: function(selection, relationshipId) {		
		// summary:
		//		Remove relationship id from the selection object
		var relIdsLength = selection.relationshipIds.length;
		for (var relIdIndex = 0;  relIdIndex < relIdsLength;  ++relIdIndex) {
			if (relationshipId === selection.relationshipIds[relIdIndex]) {
				selection.relationshipIds.splice(relIdIndex, 1);
				break;
			}
		}
	}
});

});

},
'url:gridx/templates/PaginationBarDD.html':"<div class='gridxPager'\n\t><table class='gridxPagerInner' cellpadding=\"0\" cellspacing=\"0\" role=\"presentation\"\n\t\t><tr\n\t\t\t><td class=\"gridxPagerDescriptionTD\"\n\t\t\t\t><div class=\"gridxPagerDescription\"\n\t\t\t\t\tdata-dojo-attach-point=\"_descContainer\"\n\t\t\t\t></div\n\t\t\t></td\n\t\t\t><td class=\"gridxPagerStepperTD\"\n\t\t\t\t><div class=\"gridxPagerStepper\"\n\t\t\t\t\tdata-dojo-attach-point=\"_pageStepperContainer\"\n\t\t\t\t\t><label id=\"${id}-pagerStepperLabel\" class='gridxPagerLabel'>${pageLabel}</label\n\t\t\t\t></div\n\t\t\t></td\n\t\t\t><td class=\"gridxPagerSizeSwitchTD\"\n\t\t\t\t><div class=\"gridxPagerSizeSwitch\"\n\t\t\t\t\tdata-dojo-attach-point='_sizeSwitchContainer'\n\t\t\t\t\t><label id=\"${id}-pagerSizeSwitchLabel\" class='gridxPagerLabel'>${pageSizeLabel}</label\n\t\t\t\t></div\n\t\t\t></td\n\t\t></tr\n\t></table\n></div>\n",
'gridx/modules/pagination/_PagerBase':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/query",
	"dojo/string",
	"dijit/_WidgetBase",
	"dijit/_TemplatedMixin",
	"dojo/i18n!../../nls/PaginationBar"
], function(declare, lang, query, string, _WidgetBase, _TemplatedMixin, nls){

	return declare([_WidgetBase, _TemplatedMixin], {
		pagination: null,
	
		module: null,
		
		position: 'bottom',

		focusPriority: 0,

		constructor: function(){
			this._nls = nls;
			lang.mixin(this, nls);
		},

		postCreate: function(){
			var t = this,
				c = 'connect',
				p = t.pagination,
				m = t.module.model;
			t[c](p, 'onSwitchPage', '_onSwitchPage');
			t[c](p, 'onChangePageSize', '_onChangePageSize');
			t[c](m, 'onSizeChange', '_onSizeChange');
			t[c](m, 'onMarkChange', '_createDescription');
			t._initFocus();
			t.refresh();
		},

		_toggleNode: function(cls, toShow){
			query('.' + cls, this.domNode)[0].style.display = toShow ? '' : 'none';
			return toShow;
		},

		_createDescription: function(){
			var t = this, mod = t.module;
			if(t._toggleNode('gridxPagerDescription', mod._exist(t.position, 'description'))){
				var g = mod.grid,
					selectRow = g.select && g.select.row,
					selected = selectRow ? selectRow.getSelected().length : 0, 
					tpl = selectRow ? mod.arg('descriptionSelectionTemplate', nls.summaryWithSelection) : 
						mod.arg('descriptionTemplate', nls.summary);
				t._descContainer.innerHTML = string.substitute(tpl, [g.model.size(), selected]);
			}
		}
	});
});

},
'dojox/html/ellipsis':function(){
define("dojox/html/ellipsis",["dojo/_base/kernel", "dojo/_base/lang", "dojo/_base/array", "dojo/_base/Color", "dojo/colors"], function(d){
	/*=====
	dojox.html.ellipsis = {
		// summary: offers cross-browser support for text-overflow: ellipsis
		//
		// description: Add "dojoxEllipsis" on any node that you want to ellipsis-ize. In order to function properly,
		//	the node with the dojoxEllipsis class set on it should be a child of a node with a defined width.
		//	It should also be a block-level element (i.e. <div>) - it will not work on td elements.
		//	NOTE: When using the dojoxEllipsis class within tables, the table needs to have the table-layout: fixed style
	}
	=====*/
	
	if(d.isFF < 7){ //TODO: feature detect text-overflow in computed style?
		// The delay (in ms) to wait so that we don't keep querying when many
		// changes happen at once - set config "dojoxFFEllipsisDelay" if you
		// want a different value
		var delay = 1;
		if("dojoxFFEllipsisDelay" in d.config){
			delay = Number(d.config.dojoxFFEllipsisDelay);
			if(isNaN(delay)){
				delay = 1;
			}
		}
		try{
			var createXULEllipsis = (function(){
				// Create our stub XUL elements for cloning later
				// NOTE: this no longer works as of FF 4.0:
				// https://developer.mozilla.org/En/Firefox_4_for_developers#Remote_XUL_support_removed
				var sNS = 'http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul';
				var xml = document.createElementNS(sNS, 'window');
				var label = document.createElementNS(sNS, 'description');
				label.setAttribute('crop', 'end');
				xml.appendChild(label);

				return function(/* Node */ n){
					// Summary:
					//		Given a node, it creates the XUL and sets its
					//		content so that it will have an ellipsis
					var x = xml.cloneNode(true);
					x.firstChild.setAttribute('value', n.textContent);
					n.innerHTML = '';
					n.appendChild(x);
				};
			})();
		}catch(e){}
		
		// Create our iframe elements for cloning later
		var create = d.create;
		var dd = d.doc;
		var dp = d.place;
		var iFrame = create("iframe", {className: "dojoxEllipsisIFrame",
					src: "javascript:'<html><head><script>if(\"loadFirebugConsole\" in window){window.loadFirebugConsole();}</script></head><body></body></html>'"});
		var rollRange = function(/* W3C Range */ r, /* int? */ cnt){
			// Summary:
			//		Rolls the given range back one character from the end
			//
			//	r: W3C Range
			//		The range to roll back
			//	cnt: int?
			//		An optional number of times to roll back (defaults 1)
			if(r.collapsed){
				// Do nothing - we are already collapsed
				return;
			}
			if(cnt > 0){
				do{
					rollRange(r);
					cnt--;
				}while(cnt);
				return;
			}
			if(r.endContainer.nodeType == 3 && r.endOffset > 0){
				r.setEnd(r.endContainer, r.endOffset - 1);
			}else if(r.endContainer.nodeType == 3){
				r.setEndBefore(r.endContainer);
				rollRange(r);
				return;
			}else if(r.endOffset && r.endContainer.childNodes.length >= r.endOffset){
				var nCont = r.endContainer.childNodes[r.endOffset - 1];
				if(nCont.nodeType == 3){
					r.setEnd(nCont, nCont.length - 1);
				}else if(nCont.childNodes.length){
					r.setEnd(nCont, nCont.childNodes.length);
					rollRange(r);
					return;
				}else{
					r.setEndBefore(nCont);
					rollRange(r);
					return;
				}
			}else{
				r.setEndBefore(r.endContainer);
				rollRange(r);
				return;
			}
		};
		var createIFrameEllipsis = function(/* Node */ n){
			// Summary:
			//		Given a node, it creates an iframe and and ellipsis div and
			//		sets up the connections so that they will work correctly.
			//		This function is used when createXULEllipsis is not able
			//		to be used (because there is markup within the node) - it's
			//		a bit slower, but does the trick
			var c = create("div", {className: "dojoxEllipsisContainer"});
			var e = create("div", {className: "dojoxEllipsisShown", style: {display: "none"}});
			n.parentNode.replaceChild(c, n);
			c.appendChild(n);
			c.appendChild(e);
			var i = iFrame.cloneNode(true);
			var ns = n.style;
			var es = e.style;
			var ranges;
			var resizeNode = function(){
				ns.display = "";
				es.display = "none";
				if(n.scrollWidth <= n.offsetWidth){ return; }
				var r = dd.createRange();
				r.selectNodeContents(n);
				ns.display = "none";
				es.display = "";
				var done = false;
				do{
					var numRolls = 1;
					dp(r.cloneContents(), e, "only");
					var sw = e.scrollWidth, ow = e.offsetWidth;
					done = (sw <= ow);
					var pct = (1 - ((ow * 1) / sw));
					if(pct > 0){
						numRolls = Math.max(Math.round(e.textContent.length * pct) - 1, 1);
					}
					rollRange(r, numRolls);
				}while(!r.collapsed && !done);
			};
			i.onload = function(){
				i.contentWindow.onresize = resizeNode;
				resizeNode();
			};
			c.appendChild(i);
		};

		// Function for updating the ellipsis
		var hc = d.hasClass;
		var doc = d.doc;
		var s, fn, opt;
		if(doc.querySelectorAll){
			s = doc;
			fn = "querySelectorAll";
			opt = ".dojoxEllipsis";
		}else if(doc.getElementsByClassName){
			s = doc;
			fn = "getElementsByClassName";
			opt = "dojoxEllipsis";
		}else{
			s = d;
			fn = "query";
			opt = ".dojoxEllipsis";
		}
		fx = function(){
			d.forEach(s[fn].apply(s, [opt]), function(n){
				if(!n || n._djx_ellipsis_done){ return; }
				n._djx_ellipsis_done = true;
				if(createXULEllipsis && n.textContent == n.innerHTML && !hc(n, "dojoxEllipsisSelectable")){
					// We can do the faster XUL version, instead of calculating
					createXULEllipsis(n);
				}else{
					createIFrameEllipsis(n);
				}
			});
		};
		
		d.addOnLoad(function(){
			// Apply our initial stuff
			var t = null;
			var c = null;
			var connFx = function(){
				if(c){
					// disconnect us - so we don't fire anymore
					d.disconnect(c);
					c = null;
				}
				if(t){ clearTimeout(t); }
				t = setTimeout(function(){
					t = null;
					fx();
					// Connect to the modified function so that we can catch
					// our next change
					c = d.connect(d.body(), "DOMSubtreeModified", connFx);
				}, delay);
			};
			connFx();
		});
	}
});

},
'dijit/_editor/plugins/EnterKeyHandling':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom-construct", // domConstruct.destroy domConstruct.place
	"dojo/_base/event", // event.stop
	"dojo/keys", // keys.ENTER
	"dojo/_base/lang",
	"dojo/_base/sniff", // has("ie") has("mozilla") has("webkit")
	"dojo/_base/window", // win.global win.withGlobal
	"dojo/window", // winUtils.scrollIntoView
	"../_Plugin",
	"../RichText",
	"../range",
	"../selection"
], function(declare, domConstruct, event, keys, lang, has, win, winUtils, _Plugin, RichText, rangeapi, selectionapi){

/*=====
	var _Plugin = dijit._editor._Plugin;
=====*/

// module:
//		dijit/_editor/plugins/EnterKeyHandling
// summary:
//		This plugin tries to make all browsers behave consistently with regard to
//		how ENTER behaves in the editor window.  It traps the ENTER key and alters
//		the way DOM is constructed in certain cases to try to commonize the generated
//		DOM and behaviors across browsers.


return declare("dijit._editor.plugins.EnterKeyHandling", _Plugin, {
	// summary:
	//		This plugin tries to make all browsers behave consistently with regard to
	//		how ENTER behaves in the editor window.  It traps the ENTER key and alters
	//		the way DOM is constructed in certain cases to try to commonize the generated
	//		DOM and behaviors across browsers.
	//
	// description:
	//		This plugin has three modes:
	//
	//			* blockNodeForEnter=BR
	//			* blockNodeForEnter=DIV
	//			* blockNodeForEnter=P
	//
	//		In blockNodeForEnter=P, the ENTER key starts a new
	//		paragraph, and shift-ENTER starts a new line in the current paragraph.
	//		For example, the input:
	//
	//		|	first paragraph <shift-ENTER>
	//		|	second line of first paragraph <ENTER>
	//		|	second paragraph
	//
	//		will generate:
	//
	//		|	<p>
	//		|		first paragraph
	//		|		<br/>
	//		|		second line of first paragraph
	//		|	</p>
	//		|	<p>
	//		|		second paragraph
	//		|	</p>
	//
	//		In BR and DIV mode, the ENTER key conceptually goes to a new line in the
	//		current paragraph, and users conceptually create a new paragraph by pressing ENTER twice.
	//		For example, if the user enters text into an editor like this:
	//
	//		|		one <ENTER>
	//		|		two <ENTER>
	//		|		three <ENTER>
	//		|		<ENTER>
	//		|		four <ENTER>
	//		|		five <ENTER>
	//		|		six <ENTER>
	//
	//		It will appear on the screen as two 'paragraphs' of three lines each.  Markupwise, this generates:
	//
	//		BR:
	//		|		one<br/>
	//		|		two<br/>
	//		|		three<br/>
	//		|		<br/>
	//		|		four<br/>
	//		|		five<br/>
	//		|		six<br/>
	//
	//		DIV:
	//		|		<div>one</div>
	//		|		<div>two</div>
	//		|		<div>three</div>
	//		|		<div>&nbsp;</div>
	//		|		<div>four</div>
	//		|		<div>five</div>
	//		|		<div>six</div>

	// blockNodeForEnter: String
	//		This property decides the behavior of Enter key. It can be either P,
	//		DIV, BR, or empty (which means disable this feature). Anything else
	//		will trigger errors.  The default is 'BR'
	//
	//		See class description for more details.
	blockNodeForEnter: 'BR',

	constructor: function(args){
		if(args){
			if("blockNodeForEnter" in args){
				args.blockNodeForEnter = args.blockNodeForEnter.toUpperCase();
			}
			lang.mixin(this,args);
		}
	},

	setEditor: function(editor){
		// Overrides _Plugin.setEditor().
		if(this.editor === editor){ return; }
		this.editor = editor;
		if(this.blockNodeForEnter == 'BR'){
			// While Moz has a mode tht mostly works, it's still a little different,
			// So, try to just have a common mode and be consistent.  Which means
			// we need to enable customUndo, if not already enabled.
			this.editor.customUndo = true;
				editor.onLoadDeferred.then(lang.hitch(this,function(d){
					this.connect(editor.document, "onkeypress", function(e){
						if(e.charOrCode == keys.ENTER){
							// Just do it manually.  The handleEnterKey has a shift mode that
							// Always acts like <br>, so just use it.
							var ne = lang.mixin({},e);
							ne.shiftKey = true;
							if(!this.handleEnterKey(ne)){
								event.stop(e);
							}
						}
					});
					if(has("ie") == 9){
						this.connect(editor.document, "onpaste", function(e){
							setTimeout(dojo.hitch(this, function(){
								// Use the old range/selection code to kick IE 9 into updating
								// its range by moving it back, then forward, one 'character'.
								var r = this.editor.document.selection.createRange();
								r.move('character',-1);
								r.select();
								r.move('character',1);
								r.select();
							}),0);
						});
					}
					return d;
				}));
		}else if(this.blockNodeForEnter){
			// add enter key handler
			// FIXME: need to port to the new event code!!
			var h = lang.hitch(this,this.handleEnterKey);
			editor.addKeyHandler(13, 0, 0, h); //enter
			editor.addKeyHandler(13, 0, 1, h); //shift+enter
			this.connect(this.editor,'onKeyPressed','onKeyPressed');
		}
	},
	onKeyPressed: function(){
		// summary:
		//		Handler for keypress events.
		// tags:
		//		private
		if(this._checkListLater){
			if(win.withGlobal(this.editor.window, 'isCollapsed', dijit)){
				var liparent=win.withGlobal(this.editor.window, 'getAncestorElement', selectionapi, ['LI']);
				if(!liparent){
					// circulate the undo detection code by calling RichText::execCommand directly
					RichText.prototype.execCommand.call(this.editor, 'formatblock',this.blockNodeForEnter);
					// set the innerHTML of the new block node
					var block = win.withGlobal(this.editor.window, 'getAncestorElement', selectionapi, [this.blockNodeForEnter]);
					if(block){
						block.innerHTML=this.bogusHtmlContent;
						if(has("ie")){
							// move to the start by moving backwards one char
							var r = this.editor.document.selection.createRange();
							r.move('character',-1);
							r.select();
						}
					}else{
						console.error('onKeyPressed: Cannot find the new block node'); // FIXME
					}
				}else{
					if(has("mozilla")){
						if(liparent.parentNode.parentNode.nodeName == 'LI'){
							liparent=liparent.parentNode.parentNode;
						}
					}
					var fc=liparent.firstChild;
					if(fc && fc.nodeType == 1 && (fc.nodeName == 'UL' || fc.nodeName == 'OL')){
						liparent.insertBefore(fc.ownerDocument.createTextNode('\xA0'),fc);
						var newrange = rangeapi.create(this.editor.window);
						newrange.setStart(liparent.firstChild,0);
						var selection = rangeapi.getSelection(this.editor.window, true);
						selection.removeAllRanges();
						selection.addRange(newrange);
					}
				}
			}
			this._checkListLater = false;
		}
		if(this._pressedEnterInBlock){
			// the new created is the original current P, so we have previousSibling below
			if(this._pressedEnterInBlock.previousSibling){
				this.removeTrailingBr(this._pressedEnterInBlock.previousSibling);
			}
			delete this._pressedEnterInBlock;
		}
	},

	// bogusHtmlContent: [private] String
	//		HTML to stick into a new empty block
	bogusHtmlContent: '&#160;',		// &nbsp;

	// blockNodes: [private] Regex
	//		Regex for testing if a given tag is a block level (display:block) tag
	blockNodes: /^(?:P|H1|H2|H3|H4|H5|H6|LI)$/,

	handleEnterKey: function(e){
		// summary:
		//		Handler for enter key events when blockNodeForEnter is DIV or P.
		// description:
		//		Manually handle enter key event to make the behavior consistent across
		//		all supported browsers. See class description for details.
		// tags:
		//		private

		var selection, range, newrange, startNode, endNode, brNode, doc=this.editor.document,br,rs,txt;
		if(e.shiftKey){		// shift+enter always generates <br>
			var parent = win.withGlobal(this.editor.window, "getParentElement", selectionapi);
			var header = rangeapi.getAncestor(parent,this.blockNodes);
			if(header){
				if(header.tagName == 'LI'){
					return true; // let browser handle
				}
				selection = rangeapi.getSelection(this.editor.window);
				range = selection.getRangeAt(0);
				if(!range.collapsed){
					range.deleteContents();
					selection = rangeapi.getSelection(this.editor.window);
					range = selection.getRangeAt(0);
				}
				if(rangeapi.atBeginningOfContainer(header, range.startContainer, range.startOffset)){
						br=doc.createElement('br');
						newrange = rangeapi.create(this.editor.window);
						header.insertBefore(br,header.firstChild);
						newrange.setStartAfter(br);
						selection.removeAllRanges();
						selection.addRange(newrange);
				}else if(rangeapi.atEndOfContainer(header, range.startContainer, range.startOffset)){
					newrange = rangeapi.create(this.editor.window);
					br=doc.createElement('br');
						header.appendChild(br);
						header.appendChild(doc.createTextNode('\xA0'));
						newrange.setStart(header.lastChild,0);
					selection.removeAllRanges();
					selection.addRange(newrange);
				}else{
					rs = range.startContainer;
					if(rs && rs.nodeType == 3){
						// Text node, we have to split it.
						txt = rs.nodeValue;
						win.withGlobal(this.editor.window, function(){
							startNode = doc.createTextNode(txt.substring(0, range.startOffset));
							endNode = doc.createTextNode(txt.substring(range.startOffset));
							brNode = doc.createElement("br");

							if(endNode.nodeValue == "" && has("webkit")){
								endNode = doc.createTextNode('\xA0')
							}
							domConstruct.place(startNode, rs, "after");
							domConstruct.place(brNode, startNode, "after");
							domConstruct.place(endNode, brNode, "after");
							domConstruct.destroy(rs);
							newrange = rangeapi.create();
							newrange.setStart(endNode,0);
							selection.removeAllRanges();
							selection.addRange(newrange);
						});
						return false;
					}
					return true; // let browser handle
				}
			}else{
				selection = rangeapi.getSelection(this.editor.window);
				if(selection.rangeCount){
					range = selection.getRangeAt(0);
					if(range && range.startContainer){
						if(!range.collapsed){
							range.deleteContents();
							selection = rangeapi.getSelection(this.editor.window);
							range = selection.getRangeAt(0);
						}
						rs = range.startContainer;
						if(rs && rs.nodeType == 3){
							// Text node, we have to split it.
							win.withGlobal(this.editor.window, lang.hitch(this, function(){
								var endEmpty = false;

								var offset = range.startOffset;
								if(rs.length < offset){
									//We are not splitting the right node, try to locate the correct one
									ret = this._adjustNodeAndOffset(rs, offset);
									rs = ret.node;
									offset = ret.offset;
								}
								txt = rs.nodeValue;

								startNode = doc.createTextNode(txt.substring(0, offset));
								endNode = doc.createTextNode(txt.substring(offset));
								brNode = doc.createElement("br");

								if(!endNode.length){
									endNode = doc.createTextNode('\xA0');
									endEmpty = true;
								}

								if(startNode.length){
									domConstruct.place(startNode, rs, "after");
								}else{
									startNode = rs;
								}
								domConstruct.place(brNode, startNode, "after");
								domConstruct.place(endNode, brNode, "after");
								domConstruct.destroy(rs);
								newrange = rangeapi.create();
								newrange.setStart(endNode,0);
								newrange.setEnd(endNode, endNode.length);
								selection.removeAllRanges();
								selection.addRange(newrange);
								if(endEmpty && !has("webkit")){
									selectionapi.remove();
								}else{
									selectionapi.collapse(true);
								}
							}));
						}else{
							var targetNode;
							if(range.startOffset >= 0){
								targetNode = rs.childNodes[range.startOffset];
							}
							win.withGlobal(this.editor.window, lang.hitch(this, function(){
								var brNode = doc.createElement("br");
								var endNode = doc.createTextNode('\xA0');
								if(!targetNode){
									rs.appendChild(brNode);
									rs.appendChild(endNode);
								}else{
									domConstruct.place(brNode, targetNode, "before");
									domConstruct.place(endNode, brNode, "after");
								}
								newrange = rangeapi.create(win.global);
								newrange.setStart(endNode,0);
								newrange.setEnd(endNode, endNode.length);
								selection.removeAllRanges();
								selection.addRange(newrange);
								selectionapi.collapse(true);
							}));
						}
					}
				}else{
					// don't change this: do not call this.execCommand, as that may have other logic in subclass
					RichText.prototype.execCommand.call(this.editor, 'inserthtml', '<br>');
				}
			}
			return false;
		}
		var _letBrowserHandle = true;

		// first remove selection
		selection = rangeapi.getSelection(this.editor.window);
		range = selection.getRangeAt(0);
		if(!range.collapsed){
			range.deleteContents();
			selection = rangeapi.getSelection(this.editor.window);
			range = selection.getRangeAt(0);
		}

		var block = rangeapi.getBlockAncestor(range.endContainer, null, this.editor.editNode);
		var blockNode = block.blockNode;

		// if this is under a LI or the parent of the blockNode is LI, just let browser to handle it
		if((this._checkListLater = (blockNode && (blockNode.nodeName == 'LI' || blockNode.parentNode.nodeName == 'LI')))){
			if(has("mozilla")){
				// press enter in middle of P may leave a trailing <br/>, let's remove it later
				this._pressedEnterInBlock = blockNode;
			}
			// if this li only contains spaces, set the content to empty so the browser will outdent this item
			if(/^(\s|&nbsp;|&#160;|\xA0|<span\b[^>]*\bclass=['"]Apple-style-span['"][^>]*>(\s|&nbsp;|&#160;|\xA0)<\/span>)?(<br>)?$/.test(blockNode.innerHTML)){
				// empty LI node
				blockNode.innerHTML = '';
				if(has("webkit")){ // WebKit tosses the range when innerHTML is reset
					newrange = rangeapi.create(this.editor.window);
					newrange.setStart(blockNode, 0);
					selection.removeAllRanges();
					selection.addRange(newrange);
				}
				this._checkListLater = false; // nothing to check since the browser handles outdent
			}
			return true;
		}

		// text node directly under body, let's wrap them in a node
		if(!block.blockNode || block.blockNode===this.editor.editNode){
			try{
				RichText.prototype.execCommand.call(this.editor, 'formatblock',this.blockNodeForEnter);
			}catch(e2){ /*squelch FF3 exception bug when editor content is a single BR*/ }
			// get the newly created block node
			// FIXME
			block = {blockNode:win.withGlobal(this.editor.window, "getAncestorElement", selectionapi, [this.blockNodeForEnter]),
					blockContainer: this.editor.editNode};
			if(block.blockNode){
				if(block.blockNode != this.editor.editNode &&
					(!(block.blockNode.textContent || block.blockNode.innerHTML).replace(/^\s+|\s+$/g, "").length)){
					this.removeTrailingBr(block.blockNode);
					return false;
				}
			}else{	// we shouldn't be here if formatblock worked
				block.blockNode = this.editor.editNode;
			}
			selection = rangeapi.getSelection(this.editor.window);
			range = selection.getRangeAt(0);
		}

		var newblock = doc.createElement(this.blockNodeForEnter);
		newblock.innerHTML=this.bogusHtmlContent;
		this.removeTrailingBr(block.blockNode);
		var endOffset = range.endOffset;
		var node = range.endContainer;
		if(node.length < endOffset){
			//We are not checking the right node, try to locate the correct one
			var ret = this._adjustNodeAndOffset(node, endOffset);
			node = ret.node;
			endOffset = ret.offset;
		}
		if(rangeapi.atEndOfContainer(block.blockNode, node, endOffset)){
			if(block.blockNode === block.blockContainer){
				block.blockNode.appendChild(newblock);
			}else{
				domConstruct.place(newblock, block.blockNode, "after");
			}
			_letBrowserHandle = false;
			// lets move caret to the newly created block
			newrange = rangeapi.create(this.editor.window);
			newrange.setStart(newblock, 0);
			selection.removeAllRanges();
			selection.addRange(newrange);
			if(this.editor.height){
				winUtils.scrollIntoView(newblock);
			}
		}else if(rangeapi.atBeginningOfContainer(block.blockNode,
				range.startContainer, range.startOffset)){
			domConstruct.place(newblock, block.blockNode, block.blockNode === block.blockContainer ? "first" : "before");
			if(newblock.nextSibling && this.editor.height){
				// position input caret - mostly WebKit needs this
				newrange = rangeapi.create(this.editor.window);
				newrange.setStart(newblock.nextSibling, 0);
				selection.removeAllRanges();
				selection.addRange(newrange);
				// browser does not scroll the caret position into view, do it manually
				winUtils.scrollIntoView(newblock.nextSibling);
			}
			_letBrowserHandle = false;
		}else{ //press enter in the middle of P/DIV/Whatever/
			if(block.blockNode === block.blockContainer){
				block.blockNode.appendChild(newblock);
			}else{
				domConstruct.place(newblock, block.blockNode, "after");
			}
			_letBrowserHandle = false;

			// Clone any block level styles.
			if(block.blockNode.style){
				if(newblock.style){
					if(block.blockNode.style.cssText){
						newblock.style.cssText = block.blockNode.style.cssText;
					}
				}
			}

			// Okay, we probably have to split.
			rs = range.startContainer;
			var firstNodeMoved;
			if(rs && rs.nodeType == 3){
				// Text node, we have to split it.
				var nodeToMove, tNode;
				endOffset = range.endOffset;
				if(rs.length < endOffset){
					//We are not splitting the right node, try to locate the correct one
					ret = this._adjustNodeAndOffset(rs, endOffset);
					rs = ret.node;
					endOffset = ret.offset;
				}

				txt = rs.nodeValue;
				startNode = doc.createTextNode(txt.substring(0, endOffset));
				endNode = doc.createTextNode(txt.substring(endOffset, txt.length));

				// Place the split, then remove original nodes.
				domConstruct.place(startNode, rs, "before");
				domConstruct.place(endNode, rs, "after");
				domConstruct.destroy(rs);

				// Okay, we split the text.  Now we need to see if we're
				// parented to the block element we're splitting and if
				// not, we have to split all the way up.  Ugh.
				var parentC = startNode.parentNode;
				while(parentC !== block.blockNode){
					var tg = parentC.tagName;
					var newTg = doc.createElement(tg);
					// Clone over any 'style' data.
					if(parentC.style){
						if(newTg.style){
							if(parentC.style.cssText){
								newTg.style.cssText = parentC.style.cssText;
							}
						}
					}
					// If font also need to clone over any font data.
					if(parentC.tagName === "FONT"){
						if(parentC.color){
							newTg.color = parentC.color;
						}
						if(parentC.face){
							newTg.face = parentC.face;
						}
						if(parentC.size){  // this check was necessary on IE
							newTg.size = parentC.size;
						}
					}

					nodeToMove = endNode;
					while(nodeToMove){
						tNode = nodeToMove.nextSibling;
						newTg.appendChild(nodeToMove);
						nodeToMove = tNode;
					}
					domConstruct.place(newTg, parentC, "after");
					startNode = parentC;
					endNode = newTg;
					parentC = parentC.parentNode;
				}

				// Lastly, move the split out tags to the new block.
				// as they should now be split properly.
				nodeToMove = endNode;
				if(nodeToMove.nodeType == 1 || (nodeToMove.nodeType == 3 && nodeToMove.nodeValue)){
					// Non-blank text and non-text nodes need to clear out that blank space
					// before moving the contents.
					newblock.innerHTML = "";
				}
				firstNodeMoved = nodeToMove;
				while(nodeToMove){
					tNode = nodeToMove.nextSibling;
					newblock.appendChild(nodeToMove);
					nodeToMove = tNode;
				}
			}

			//lets move caret to the newly created block
			newrange = rangeapi.create(this.editor.window);
			var nodeForCursor;
			var innerMostFirstNodeMoved = firstNodeMoved;
			if(this.blockNodeForEnter !== 'BR'){
				while(innerMostFirstNodeMoved){
					nodeForCursor = innerMostFirstNodeMoved;
					tNode = innerMostFirstNodeMoved.firstChild;
					innerMostFirstNodeMoved = tNode;
				}
				if(nodeForCursor && nodeForCursor.parentNode){
					newblock = nodeForCursor.parentNode;
					newrange.setStart(newblock, 0);
					selection.removeAllRanges();
					selection.addRange(newrange);
					if(this.editor.height){
						winUtils.scrollIntoView(newblock);
					}
					if(has("mozilla")){
						// press enter in middle of P may leave a trailing <br/>, let's remove it later
						this._pressedEnterInBlock = block.blockNode;
					}
				}else{
					_letBrowserHandle = true;
				}
			}else{
				newrange.setStart(newblock, 0);
				selection.removeAllRanges();
				selection.addRange(newrange);
				if(this.editor.height){
					winUtils.scrollIntoView(newblock);
				}
				if(has("mozilla")){
					// press enter in middle of P may leave a trailing <br/>, let's remove it later
					this._pressedEnterInBlock = block.blockNode;
				}
			}
		}
		return _letBrowserHandle;
	},

	_adjustNodeAndOffset: function(/*DomNode*/node, /*Int*/offset){
		// summary:
		//              In the case there are multiple text nodes in a row the offset may not be within the node.  If the offset is larger than the node length, it will attempt to find
		//              the next text sibling until it locates the text node in which the offset refers to
		// node:
		//              The node to check.
		// offset:
		//              The position to find within the text node
		// tags:
		//              private.
		while(node.length < offset && node.nextSibling && node.nextSibling.nodeType==3){
			//Adjust the offset and node in the case of multiple text nodes in a row
			offset = offset - node.length;
			node = node.nextSibling;
		}
		return {"node": node, "offset": offset};
	},

	removeTrailingBr: function(container){
		// summary:
		//		If last child of container is a <br>, then remove it.
		// tags:
		//		private
		var para = /P|DIV|LI/i.test(container.tagName) ?
			container : selectionapi.getParentOfType(container,['P','DIV','LI']);

		if(!para){ return; }
		if(para.lastChild){
			if((para.childNodes.length > 1 && para.lastChild.nodeType == 3 && /^[\s\xAD]*$/.test(para.lastChild.nodeValue)) ||
				para.lastChild.tagName=='BR'){

				domConstruct.destroy(para.lastChild);
			}
		}
		if(!para.childNodes.length){
			para.innerHTML=this.bogusHtmlContent;
		}
	}
});

});

},
'dijit/typematic':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/_base/connect", // connect.connect
	"dojo/_base/event", // event.stop
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/lang", // lang.mixin, lang.hitch
	"dojo/on",
	"dojo/_base/sniff", // has("ie")
	"."		// setting dijit.typematic global
], function(array, connect, event, kernel, lang, on, has, dijit){

// module:
//		dijit/typematic
// summary:
//		These functions are used to repetitively call a user specified callback
//		method when a specific key or mouse click over a specific DOM node is
//		held down for a specific amount of time.
//		Only 1 such event is allowed to occur on the browser page at 1 time.

var typematic = (dijit.typematic = {
	// summary:
	//		These functions are used to repetitively call a user specified callback
	//		method when a specific key or mouse click over a specific DOM node is
	//		held down for a specific amount of time.
	//		Only 1 such event is allowed to occur on the browser page at 1 time.

	_fireEventAndReload: function(){
		this._timer = null;
		this._callback(++this._count, this._node, this._evt);

		// Schedule next event, timer is at most minDelay (default 10ms) to avoid
		// browser overload (particularly avoiding starving DOH robot so it never gets to send a mouseup)
		this._currentTimeout = Math.max(
			this._currentTimeout < 0 ? this._initialDelay :
				(this._subsequentDelay > 1 ? this._subsequentDelay : Math.round(this._currentTimeout * this._subsequentDelay)),
			this._minDelay);
		this._timer = setTimeout(lang.hitch(this, "_fireEventAndReload"), this._currentTimeout);
	},

	trigger: function(/*Event*/ evt, /*Object*/ _this, /*DOMNode*/ node, /*Function*/ callback, /*Object*/ obj, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
		// summary:
		//		Start a timed, repeating callback sequence.
		//		If already started, the function call is ignored.
		//		This method is not normally called by the user but can be
		//		when the normal listener code is insufficient.
		// evt:
		//		key or mouse event object to pass to the user callback
		// _this:
		//		pointer to the user's widget space.
		// node:
		//		the DOM node object to pass the the callback function
		// callback:
		//		function to call until the sequence is stopped called with 3 parameters:
		// count:
		//		integer representing number of repeated calls (0..n) with -1 indicating the iteration has stopped
		// node:
		//		the DOM node object passed in
		// evt:
		//		key or mouse event object
		// obj:
		//		user space object used to uniquely identify each typematic sequence
		// subsequentDelay (optional):
		//		if > 1, the number of milliseconds until the 3->n events occur
		//		or else the fractional time multiplier for the next event's delay, default=0.9
		// initialDelay (optional):
		//		the number of milliseconds until the 2nd event occurs, default=500ms
		// minDelay (optional):
		//		the maximum delay in milliseconds for event to fire, default=10ms
		if(obj != this._obj){
			this.stop();
			this._initialDelay = initialDelay || 500;
			this._subsequentDelay = subsequentDelay || 0.90;
			this._minDelay = minDelay || 10;
			this._obj = obj;
			this._evt = evt;
			this._node = node;
			this._currentTimeout = -1;
			this._count = -1;
			this._callback = lang.hitch(_this, callback);
			this._fireEventAndReload();
			this._evt = lang.mixin({faux: true}, evt);
		}
	},

	stop: function(){
		// summary:
		//		Stop an ongoing timed, repeating callback sequence.
		if(this._timer){
			clearTimeout(this._timer);
			this._timer = null;
		}
		if(this._obj){
			this._callback(-1, this._node, this._evt);
			this._obj = null;
		}
	},

	addKeyListener: function(/*DOMNode*/ node, /*Object*/ keyObject, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
		// summary:
		//		Start listening for a specific typematic key.
		//		See also the trigger method for other parameters.
		// keyObject:
		//		an object defining the key to listen for:
		// 		charOrCode:
		//			the printable character (string) or keyCode (number) to listen for.
		// 		keyCode:
		//			(deprecated - use charOrCode) the keyCode (number) to listen for (implies charCode = 0).
		// 		charCode:
		//			(deprecated - use charOrCode) the charCode (number) to listen for.
		// 		ctrlKey:
		//			desired ctrl key state to initiate the callback sequence:
		//			- pressed (true)
		//			- released (false)
		//			- either (unspecified)
		// 		altKey:
		//			same as ctrlKey but for the alt key
		// 		shiftKey:
		//			same as ctrlKey but for the shift key
		// returns:
		//		a connection handle
		if(keyObject.keyCode){
			keyObject.charOrCode = keyObject.keyCode;
			kernel.deprecated("keyCode attribute parameter for dijit.typematic.addKeyListener is deprecated. Use charOrCode instead.", "", "2.0");
		}else if(keyObject.charCode){
			keyObject.charOrCode = String.fromCharCode(keyObject.charCode);
			kernel.deprecated("charCode attribute parameter for dijit.typematic.addKeyListener is deprecated. Use charOrCode instead.", "", "2.0");
		}
		var handles = [
			on(node, connect._keypress, lang.hitch(this, function(evt){
				if(evt.charOrCode == keyObject.charOrCode &&
				(keyObject.ctrlKey === undefined || keyObject.ctrlKey == evt.ctrlKey) &&
				(keyObject.altKey === undefined || keyObject.altKey == evt.altKey) &&
				(keyObject.metaKey === undefined || keyObject.metaKey == (evt.metaKey || false)) && // IE doesn't even set metaKey
				(keyObject.shiftKey === undefined || keyObject.shiftKey == evt.shiftKey)){
					event.stop(evt);
					typematic.trigger(evt, _this, node, callback, keyObject, subsequentDelay, initialDelay, minDelay);
				}else if(typematic._obj == keyObject){
					typematic.stop();
				}
			})),
			on(node, "keyup", lang.hitch(this, function(){
				if(typematic._obj == keyObject){
					typematic.stop();
				}
			}))
		];
		return { remove: function(){ array.forEach(handles, function(h){ h.remove(); }); } };
	},

	addMouseListener: function(/*DOMNode*/ node, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
		// summary:
		//		Start listening for a typematic mouse click.
		//		See the trigger method for other parameters.
		// returns:
		//		a connection handle
		var handles =  [
			on(node, "mousedown", lang.hitch(this, function(evt){
				event.stop(evt);
				typematic.trigger(evt, _this, node, callback, node, subsequentDelay, initialDelay, minDelay);
			})),
			on(node, "mouseup", lang.hitch(this, function(evt){
				if(this._obj){
					event.stop(evt);
				}
				typematic.stop();
			})),
			on(node, "mouseout", lang.hitch(this, function(evt){
				event.stop(evt);
				typematic.stop();
			})),
			on(node, "mousemove", lang.hitch(this, function(evt){
				evt.preventDefault();
			})),
			on(node, "dblclick", lang.hitch(this, function(evt){
				event.stop(evt);
				if(has("ie")){
					typematic.trigger(evt, _this, node, callback, node, subsequentDelay, initialDelay, minDelay);
					setTimeout(lang.hitch(this, typematic.stop), 50);
				}
			}))
		];
		return { remove: function(){ array.forEach(handles, function(h){ h.remove(); }); } };
	},

	addListener: function(/*Node*/ mouseNode, /*Node*/ keyNode, /*Object*/ keyObject, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
		// summary:
		//		Start listening for a specific typematic key and mouseclick.
		//		This is a thin wrapper to addKeyListener and addMouseListener.
		//		See the addMouseListener and addKeyListener methods for other parameters.
		// mouseNode:
		//		the DOM node object to listen on for mouse events.
		// keyNode:
		//		the DOM node object to listen on for key events.
		// returns:
		//		a connection handle
		var handles = [
			this.addKeyListener(keyNode, keyObject, _this, callback, subsequentDelay, initialDelay, minDelay),
			this.addMouseListener(mouseNode, _this, callback, subsequentDelay, initialDelay, minDelay)
		];
		return { remove: function(){ array.forEach(handles, function(h){ h.remove(); }); } };
	}
});

return typematic;

});

},
'ibm/tivoli/tip/navwidget/properties/PropertyRenderer':function(){
// wrapped by build app
define("ibm/tivoli/tip/navwidget/properties/PropertyRenderer", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/Trace,ibm/tivoli/navmodel/Property,ibm/tivoli/navmodel/Status,ibm/tivoli/navmodel/ItemStyle,ibm/tivoli/navmodel/Item,ibm/tivoli/tip/navwidget/dojo/NavWidgetUtil,dijit/ProgressBar"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * “Restricted Materials of IBM”
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.properties.PropertyRenderer");

dojo.require("ibm.tivoli.navmodel.Trace");
dojo.require("ibm.tivoli.navmodel.Property");
dojo.require("ibm.tivoli.navmodel.Status");
dojo.require("ibm.tivoli.navmodel.ItemStyle");
dojo.require("ibm.tivoli.navmodel.Item");
dojo.require("ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil");
dojo.require("dijit.ProgressBar");

ibm.tivoli.tip.navwidget.properties.PropertyRenderer._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.tip.navwidget.properties.PropertyRenderer");

//declare global link handler
ibm.tivoli.tip.navwidget.properties.handleLinkClick = function(domNode) {		
	// summary:
	//		handle a link click and make sure it gets handled to right place
		
	//get the attributes from the domNode that are set when the link is set up
	var propID = dojo.attr(domNode, "propid");
	var objectID = dojo.attr(domNode, "objectid");
	var viewID = dojo.attr(domNode, "viewid");
	var objectType = dojo.attr(domNode, "objecttype");
	
	//get refrence to the original viewobject that created this property
	var viewObj = dijit.byId(viewID);
	
	//now that we found the view, let it handle the link click 
	// ( this method will get the proper object and prop objects and then call renders
	//   handle link class in order handle the action on the link click )
	viewObj.handleLinkClickAction(viewID, objectID, propID, objectType);
};

dojo.declare("ibm.tivoli.tip.navwidget.properties.PropertyRenderer",
    null,
    {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
	
        //***************************
        // "Public"
        //***************************  	
		//summary: 
		//		callback for determining if a property needs a threshold value. 
	    //description:
		//		This callback is a method to determine if a the given view/prop needs thresholding
		//		if this value is null, then it assumed that no thresholding is needed.
		propThresholdCallback: null,
		
        //***************************
        // "Private"
        //***************************
	
		// _trace: [private] ibm.tivoli.navmodel.Trace
		//		Utility for trace and logging
        _trace: ibm.tivoli.tip.navwidget.properties.PropertyRenderer._trace, 
// asahlin begin
        //_daHelper: null,
		_initArgs : null,
// asahlin end
        
        //***************************
        // "Public Methods"
        //***************************  	
// asahlin begin        
        constructor: function(kwArgs) {		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
        	this._initArgs = {
        		daHelper : null
        	};
        	
			dojo.mixin(this._initArgs, kwArgs);
// asahlin end
        },

	// rendered node for a simple string type
	getRenderedItemNode: function(/*ibm.tivoli.navmodel.Item*/ item, plugviewref) {		
    	// summary:
    	//		Coming soon
    	// description:
    	//		Coming soon
        // item:
        //		Coming soon
        // plugviewref:
        //		Coming soon
		var contentNode = dojo.doc.createElement("div");
		var label = "";
		if (item) {
			label = item.getLabel();

			var style = item.getItemStyle({
        			mergeWidgetDef: true,
        			widgetDef: plugviewref.getNearestPluggableViewWidgetDefinition()
        		});
        				
        		if (style) {
        			//create the image node
        			nameImageNode = dojo.doc.createElement("img");
 			    					       					
        			//add the attributes to the img node based on the item style
        			dojo.attr(nameImageNode, "src", plugviewref._cleanUpImagePath(style.getSmallImage()));
        			dojo.attr(nameImageNode, "height", style.getAttribute("smallImageHeight"));
        			dojo.attr(nameImageNode, "width", style.getAttribute("smallImageWidth"));
        			dojo.attr(nameImageNode, "style", "vertical-align:sub");
				dojo.attr(nameImageNode, "hspace", "3px");
        			dojo.addClass(nameImageNode, "tctGridCellImg"); //253021

				var altText = style.getLabel();
				if (altText) {
					dojo.attr(nameImageNode, "alt", altText);
				}
        			        					
        			//append the image node
        			dojo.place(nameImageNode, contentNode,"last");
        		} 
			else if (this._trace.isWarn()) {
        			this._trace.warn("Item style is NULL for item = " + item.getId(), "getRenderedPropertyNode");
        		}
		}       	

		dojo.place(dojo.doc.createTextNode(label), contentNode, "last");						
		return contentNode;
	},

	// rendered node for a simple string type
	getRenderedValueNode: function(value) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// value:
		//		Coming soon
		var contentNode = dojo.doc.createElement("div");
		dojo.place(dojo.doc.createTextNode(value), contentNode, "last");						
		return contentNode;
	},

    getRenderedPropertyNode: function(/*ibm.tivoli.navmodel.Item*/ object, prop, plugviewref, rowID, cell) {		
		// summary:
		//		Depending on the specific type of a property from the Property.getValueType() function
		//		return a node with the proper displayable content from the value.
		// description:
		//		|	values:	"itemName", "itemType", "linkName", "linkType", "relatonshipName", "relationshipType",
		//		|		"string", "string[]",
		//		|		"byte", "short", "int", "long", "byte[]", "short[]", "int[]", "long[]",
		//		|		"ubyte, "ushort", "uint", "ulong", "ubyte[]", "ushort[]", "uint[]", "ulong[]",
		//		|		"boolean", "boolean[]"
		//		|		"enum", "map",
		//		|		"status",
		//		|		"date", "datetime", "time", "date[]", "datetime[]", "time[]",
		//		|		"isodate", "isotime", "isodatetime", "isointerval", "isoduration", "isodate[]", "isotime[]", "isodatetime[]"
		//		|		"float", "double", "percentage", "float[]", "double[]"
		// object:
		//		Coming soon
		// prop:
		//		Coming soon
		// plugviewref:
		//		Coming soon

		//first check if there is a valid value
		switch(prop.getValueState()){
		case "ok":

				switch (prop.getValueType()) {
				case "itemName":

					var contentNode = dojo.doc.createElement("div");

					if(prop.getValueUnits()) {
						dojo.place(dojo.doc.createTextNode(prop.getDisplayValue() + " " + prop.getValueUnits()), contentNode,"last");
					} else {
						var style = object.getItemStyle({
							mergeWidgetDef: true,
							widgetDef: plugviewref.getNearestPluggableViewWidgetDefinition()
						});

						if (style) {
							//create the image node
							var nameImageNode = this._getImageContentNode( plugviewref, prop, object, style.getSmallImage(), style.getAttribute("smallImageHeight"), style.getAttribute("smallImageWidth"), style.getLabel());
							dojo.place(nameImageNode, contentNode,"last");
							
						} else {
							if (this._trace.isWarn()) {
								this._trace.warn("Item style is NULL for object = " + object.getId(), "getRenderedPropertyNode");
							}
						}
						
						//Add the text node        			    
						dojo.place(this._getLabelContentNode(prop.getDisplayValue(), prop, object, plugviewref ),contentNode,"last");

					}	
					
					//color background if thresholding needed
					this._thresholdBackground(cell, prop, object);
			
					//dojo.place(contentNode, outerNode, "last");
					return contentNode;

				case "status":


					var contentNode = dojo.doc.createElement("div");

					//the getStatusValue returns and object, which can be used to get the image
					//in order to use this we must mixin the widget def as well as the dataset and object type
					var sobject = prop.getStatusValue({
						widgetDef: plugviewref.getNearestPluggableViewWidgetDefinition(),
						dataset: plugviewref.dataset,
						objectType: object.getObjectType()
					});
					var statusImageNode = this._getImageContentNode( plugviewref, prop, object, sobject.getImage(), sobject.getImageHeight(), sobject.getImageWidth(), sobject.getLabel());
				
					//append the image node, and create then append the text node
					dojo.place(statusImageNode, contentNode,"last");

					var statusLabel = "";
					if(prop.getDisplayValue() && !prop.getDisplayValue().id){
						statusLabel = prop.getDisplayValue();
					}
					if (!statusLabel && cell) {
						cell.customStyles.push("text-align: center");
					}  
	
					dojo.place(this._getLabelContentNode(statusLabel, prop, object, plugviewref  ), contentNode,"last");
					
					//color background if thresholding needed
					this._thresholdBackground(cell, prop, object);

					return contentNode;
				case "percentage":
					var contentNode = dojo.doc.createElement("div");

					if(prop.getValueUnits()) {
						dojo.place(dojo.doc.createTextNode(prop.getDisplayValue() + " " + prop.getValueUnits()), contentNode,"last");
					} else {    					
                  // 255204
                  // Check if there is a maximum value specified on the property and if it exists and is non-zero, use it,
                  // otherwise use 1 which is the default assumption that percentage values will be between 0 and 1.
                  var maxValue = prop.getValueMax();    
                  maxValue = ((maxValue == null) || (maxValue <= 0)) ? 1 : maxValue;
						var pbar = new dijit.ProgressBar({ progress: prop.getValue(), 
															places: 0, 
															maximum: maxValue, 
															style:"width:100%"});
						
						dojo.query(".dijitProgressBarTile", pbar.domNode).addClass("tableCellDefaultProgressBarTile"); //SAPM desired a different color for the bar
						
						dojo.place(pbar.domNode, contentNode, "last");
					}
					
					//a threshold callback is defined, so use it to check if we need
					//to threshold this value and ADD an label, which we normally wouldnt render
					if(this.propThresholdCallback){
						var threshold = this.propThresholdCallback(prop, object);
						//only if a label is defined on the threshold for this value
						if(threshold && threshold.label){
							//render the node, and add it
							dojo.place(this._getLabelContentNode("", prop, object, plugviewref  ), contentNode, "last");	
						}
	
						if(threshold) {
							//set the background color
							if (threshold.color){
								//get the node for the progress bar portion that holds the color
								var barNode = dojo.query(".dijitProgressBarTile", pbar.domNode);
								dojo.style(barNode[0], "backgroundColor", threshold.color );
							}
							
							//set the foreground color
							if (threshold.fontColor){
								var labelNode = dojo.query(".dijitProgressBarLabel", pbar.domNode);
								dojo.style(labelNode[0], "color", threshold.fontColor );
							}
						}
					}
					
					//dont color background of progress bar since the bar itself is colored
					//this._thresholdBackground(cell, prop, object);
					
					return contentNode;

				default:
					var contentNode = dojo.doc.createElement("div");

					//a threshold callback is defined, so use it to check if we need
					//to threshold this value and ADD an image which we normally wouldnt render
					if(this.propThresholdCallback){
						var threshold = this.propThresholdCallback(prop, object);
						
						//only if an image is defined on the threshold for this value
						if(threshold && threshold.icon){
							var altText = (threshold.label ? threshold.label.value : null);
							if (!altText) {
								altText = prop.getDisplayValue();
							}

							//render the node, and add it
							dojo.place(this._getImageContentNode( plugviewref, prop, object, "", "16px", "16px"), contentNode, "last", altText);	
						}
					}
				
					if(prop.getValueUnits()) {
						dojo.place(this._getLabelContentNode(prop.getDisplayValue() + " " + prop.getValueUnits(), prop, object, plugviewref ), contentNode,"last");
					} else {
						dojo.place(this._getLabelContentNode(prop.getDisplayValue(), prop, object, plugviewref  ), contentNode,"last");
					}	
					
					//color background if thresholding needed
					this._thresholdBackground(cell, prop, object);
					
					return contentNode;
				//probably need to add some error handling in here in case the input value is invalid	
				}
			

		case "pending":
			var contentNode = dojo.doc.createElement("div");
		
			//get the image node			
			var nameImageNode = this._getImageContentNode( plugviewref, prop, object, "[widget]/images/processing-toolbar.gif", "16px", "16px", plugviewref.commonResources.PENDING);
			
			//append the image node
			dojo.place(nameImageNode, contentNode,"last");
			
			//a threshold callback is defined, so use it to check if we need
			//to threshold this value and ADD an label, which we normally wouldnt render
			if(this.propThresholdCallback){
				var threshold = this.propThresholdCallback(prop, object);
				//only if a label is defined on the threshold for this value
				if(threshold && threshold.label){
					//render the node, and add it
					dojo.place(this._getLabelContentNode("", prop, object, plugviewref  ), contentNode, "last");	
				}
			}
			
			//color background if thresholding needed
			this._thresholdBackground(cell, prop, object);
			
			return contentNode;
			
			
		case "error":
			var contentNode = dojo.doc.createElement("div");			
			var nameImageNode = this._getImageContentNode( plugviewref, prop, object, "[widget]/images/ac16_connstatnotoperational_24.png", "16px", "16px", plugviewref.commonResources.ERROR);
			
			//append the image node
			dojo.place(nameImageNode, contentNode,"last");
			
			//a threshold callback is defined, so use it to check if we need
			//to threshold this value and ADD an label, which we normally wouldnt render
			if(this.propThresholdCallback){
				var threshold = this.propThresholdCallback(prop, object);
				//only if a label is defined on the threshold for this value
				if(threshold && threshold.label){
					//render the node, and add it
					dojo.place(this._getLabelContentNode("", prop, object, plugviewref  ), contentNode, "last");	
				}
			}
			
			//color background if thresholding needed
			this._thresholdBackground(cell, prop, object);
				
			return contentNode;
		
		case "none":
			var contentNode = dojo.doc.createElement("div");
			contentNode.innerHTML = "&mdash;";
			return contentNode;

		default:
			
		}
	},
	
	getPropertyDisplayValue: function(object, prop, plugviewref) {		
		// summary:
		//		Depending on the specific type of a property from the Property.getValueType() function
		//		return a string suitable for the user. This method mostly intended for export.
		// description:

		// item:
		//		Coming soon
		// prop:
		//		Coming soon
		// plugviewref:
		//		Coming soon

		//first check if there is a valid value
		var displayValue = null;
		switch (prop.getValueState()){
			case "ok":
				if(prop.getValueUnits()) {
					displayValue = prop.getDisplayValue() + " " + prop.getValueUnits();
				} else {
					displayValue = prop.getDisplayValue();
				}	
				break;
			
			case "pending":
				displayValue = plugviewref.commonResources.PENDING;
				break;
	
			case "error":
				displayValue = plugviewref.commonResources.ERROR;
				break;
				
			default:
				break;
		}
		
		if (!displayValue) {
			displayValue = "";
		}
	
		return displayValue;
	},

	_thresholdBackground: function(cell, prop, object){
		//if a threshold is defined use the callback to determine if we need to color it
		if(this.propThresholdCallback && cell){
			var threshValue = this.propThresholdCallback(prop, object);
			if(threshValue) {
				if (threshValue.color){
					cell.customStyles.push("background-color: " + threshValue.color);
				}
				if (threshValue.fontColor){
					cell.customStyles.push("color: " + threshValue.fontColor);
				}
			}
		}
	},
	_getImageContentNode: function( plugviewref, prop, object, imagePath, height, width, altText){
		//create the image node
		var fullImageNode;
		var linkNode;
	
		var imageNode= dojo.doc.createElement("img");
		//var thresholded = false;
		
		//a threshold callback is defined,
		if(this.propThresholdCallback){
		//use it to check if we need to see if we need to threshold this image
			var threshold = this.propThresholdCallback(prop, object);
			if(threshold && threshold.icon){
				//threshold condition met, so use the threshold image instead
				//dojo.attr(imageNode, "src", plugviewref._cleanUpImagePath(threshold.icon));
				imagePath = plugviewref._cleanUpImagePath(threshold.icon);
			}
		}
			

		//add the attributes to the img node based on the object style
		dojo.attr(imageNode, "src", plugviewref._cleanUpImagePath(imagePath));
		// jbragiel begin
		if ((height !== undefined) && (height !== null)) {
		// jbragiel end
			dojo.attr(imageNode, "height", height);
		// jbragiel begin
		}
		if ((width !== undefined) && (width !== null)) {
		// jbragiel end
			dojo.attr(imageNode, "width", width);
		// jbragiel begin
		}
		// jbragiel end
		dojo.attr(imageNode, "style", "vertical-align:sub");
		dojo.attr(imageNode, "hspace", "3px");
		dojo.addClass(imageNode, "tctGridCellImg"); //253021
		if (altText) {
			dojo.attr(imageNode, "alt", altText);
		}

		if(prop.getTask()){
			linkNode = dojo.doc.createElement("a");
			
			dojo.attr(linkNode, "onclick", "ibm.tivoli.tip.navwidget.properties.handleLinkClick(this);return false;" );
			dojo.attr(linkNode, "id", object.getId() + "_"+ prop.getId()+".img.link");
			dojo.attr(linkNode, "propid", prop.getId());
			dojo.attr(linkNode, "objectid", object.getId());
			dojo.attr(linkNode, "viewid", plugviewref.id);
			//check to see which type of object this is and set on ojbecttype attr for link
			var objType = null;
			if(object instanceof ibm.tivoli.navmodel.Item){
				objType = "ibm.tivoli.navmodel.Item";
			}
			else if(object instanceof ibm.tivoli.navmodel.Link){
				objType = "ibm.tivoli.navmodel.Link";
			}
			else if(object instanceof ibm.tivoli.navmodel.Relationship){
				objType = "ibm.tivoli.navmodel.Relationship";
			}
			dojo.attr(linkNode, "objecttype", objType);
		
			
			// dojo.attr(linkNode, "href", "#");
			dojo.attr(linkNode, "href", "javascript:void(0)");  // 54309
			
			dojo.place(imageNode, linkNode, "last");
			
			fullImageNode = linkNode;
			
		}else{
			fullImageNode = imageNode;
		}
		
		
		return fullImageNode;

	},
	
	//create either a normal label, or a link if needed
	_getLabelContentNode: function(labelText, prop, object, plugviewref){
		//a threshold callback is defined,
		if(this.propThresholdCallback){
			//use it to check if we need to threshold this label
			var threshold = this.propThresholdCallback(prop, object);
			if(threshold && threshold.label && threshold.label.value){
				//use the threshold label instead of the normal label passed in
				labelText = threshold.label.value;
			}
		}
		
		if(prop.getTask()){
		
			var propTask = prop.getTask();
			//TODO:: ADM need better check for task, getCategory shuold work, but returns extra info

			//if this is a property task
			//this returns more than just "property" so lets use a diff check
			//if(propTask.getCategory() == "property"){

			//render SECOND! link for test
			var linkNode = dojo.doc.createElement("a");
				
			//var linkHelper2 = dojo.hitch(this, this._handleLinkClick, prop, plugviewref);
			dojo.attr(linkNode, "onclick", "ibm.tivoli.tip.navwidget.properties.handleLinkClick(this);return false;" );
			//render the link
			dojo.attr(linkNode, "id", object.getId() + "_"+ prop.getId()+".link");
			
			dojo.attr(linkNode, "propid", prop.getId());	
			dojo.attr(linkNode, "objectid", object.getId());	
			dojo.attr(linkNode, "viewid", plugviewref.id);
			//check to see which type of object this is and set on ojbecttype attr for link
			var objType = null;
			if(object instanceof ibm.tivoli.navmodel.Item){
				objType = "ibm.tivoli.navmodel.Item";
			}
			else if(object instanceof ibm.tivoli.navmodel.Link){
				objType = "ibm.tivoli.navmodel.Link";
			}
			else if(object instanceof ibm.tivoli.navmodel.Relationship){
				objType = "ibm.tivoli.navmodel.Relationship";
			}
			dojo.attr(linkNode, "objecttype", objType);
				
			// dojo.attr(linkNode, "href", "#");
			dojo.attr(linkNode, "href", "javascript:void(0)");  // 54309

			dojo.place(dojo.doc.createTextNode(labelText), linkNode, "last");
	
			return linkNode;
		//else render as normal	
		}else{	
			return dojo.doc.createTextNode(labelText);
		}
	},
	
	   //Event callback method to handle updating the tooltip content with updated content
	_handleLinkClick: function(property, objectContext, e){

		//first stop the default browser event:
		if(e){
			dojo.stopEvent(e);
		}
		//Create DefaultActionHelper
		//By default launches a link url else executes the task and launches it (via callback)
// asahlin begin
		if(!this._initArgs["daHelper"]){
			this._initArgs["daHelper"] = new ibm.tivoli.tip.navwidget.topology.dijit.DefaultActionHelper( 
					{
						defaultActionCallback: dojo.hitch(this, "_executeLink",objectContext, property)
					}
			);
		}
		//actually execute the default action/task (defined in constuctor)
		this._initArgs["daHelper"].executeDefaultAction(objectContext, property);
 		   
		//this.daHelper.destroy();
		//this.daHelper = null;
// asahlin end
		return false;
	 },
	 
	 _executeLink: function(objectContext, prop){
		//check if this is a url
		console.log("The default link task did not execute as expected");
	 }
			
	
});

});

},
'ibm/tivoli/tip/navwidget/table/dijit/ItemsTableView':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/i18n!ibm/tivoli/tip/navwidget/nls/resources","dojo/require!dojo/data/ItemFileReadStore,ibm/tivoli/navmodel/DatasetItemsStore,ibm/tivoli/tip/navwidget/table/dijit/AbstractTableView,ibm/tivoli/tip/navwidget/table/dijit/NavModelTable"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide ("ibm.tivoli.tip.navwidget.table.dijit.ItemsTableView");

dojo.require("dojo.data.ItemFileReadStore");   // 49426  Gridx 
dojo.require("ibm.tivoli.navmodel.DatasetItemsStore");

dojo.require ("ibm.tivoli.tip.navwidget.table.dijit.AbstractTableView");
dojo.require("ibm.tivoli.tip.navwidget.table.dijit.NavModelTable");

dojo.requireLocalization("ibm.tivoli.tip.navwidget", "resources");

ibm.tivoli.tip.navwidget.table.dijit.ItemsTableView._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.tip.navwidget.table.dijit.ItemsTableView");

dojo.declare ("ibm.tivoli.tip.navwidget.table.dijit.ItemsTableView",
	ibm.tivoli.tip.navwidget.table.dijit.AbstractTableView,
	{
	// summary:
	//		Coming soon
	// description:
	//		Coming soon
	
	//***************************
	// "Private"
	//***************************
	
	// _trace: [private] Trace
	//		Utility for trace and logging
	_trace: ibm.tivoli.tip.navwidget.table.dijit.ItemsTableView._trace,
	
	constructor: function () {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		//this._global = ibm.tivoli.tip.dojo.Global.setSingleton( "123", " ");
		this.title = this.commonResources.RESOURCE_VIEW;
		this.labelIconClass = "resourceViewIcon";
	    this._pDEFAULT_WIDGET_DEF_INFO = {
			className: "ibm.tivoli.navmodel.TableWidgetDef",
			baseUrl: "/ibm/tivoli/rest",
	    	provider: "any",
	    	datasource: "any",
	    	widget: "table",
	    	definition: "DefaultItemsTable"
		},
		
		this._pTableType = "item";
		this._pTableClassName = "ibm.tivoli.tip.navwidget.table.dijit.NavModelTable";

	},

	// overrides AbstractTableView  49426
	augmentGridParams: function(gridParams) {    
		// summary:
		//		Set up gridParams as needed
		// description:
		//		Modify the gridParams to compensate for what dojoe.tablex wants
		
        if (gridParams.filter) {
        	if (!gridParams.filter.isServerSide) {
        		gridParams.filter.isServerSide = true;
        	}
        }
        else {
        	gridParams.filter = {isServerSide : true}
        }
		
		// Reset the structure value to only the array of column descriptors and not the defaultCell info
		// Gridx does not support multiple or complicated nested layouts
 		if ((gridParams.structure.length == 1) && (gridParams.store instanceof dojo.data.ItemFileReadStore)) {						
			gridParams.structure = [{}];
		} else if ((gridParams.structure.length == 1) && ("rows" in gridParams.structure[0]) && (gridParams.structure[0].rows instanceof Array)) {
			gridParams.structure = gridParams.structure[0].rows; 
		} else if((gridParams.structure.length == 2) && ("rows" in gridParams.structure[0]) && (gridParams.structure[0].rows instanceof Array) && ("rows" in gridParams.structure[1]) && (gridParams.structure[1].rows instanceof Array)  ) {
	    	// if there are some defined locked columns, merge them into one array since
	    	// Gridx does not support true column locking, they will however appear
	    	// at the beginning of the table
	    	var layout1 = gridParams.structure[0].rows;
	    	var layout2 = gridParams.structure[1].rows;
	    	var layoutMerged = [];
    	
	    	layoutMerged = layout1.concat(layout2);

	    	gridParams.structure = layoutMerged;  	
	    }
 			    
		return gridParams;
	},	
	
	_createTableStore: function(columnIds){		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// columnIds:
		//		
		
		// create or update store
		if (!this._dataStore) {   // 49426  Gridx (split out to always return some store)
            if (this.dataset) {
            	this._dataStore = new ibm.tivoli.navmodel.DatasetItemsStore({dataset:this.dataset, defaultProperties:columnIds, returnItem:true});
            }
            else {
				//No dataset, so just create an empty store
				var emptyStore = new dojo.data.ItemFileReadStore({
										data: { items: []}
	                				});
				this._dataStore = emptyStore;
            }
		   this.inherited(arguments);
		}
	}	
	
});

});

},
'ibm/usmi/util/DefaultMessageHandler':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/util/DefaultMessageHandler", [
	"dijit",
	"dojo",
	"dojox"
], function (dijit, dojo, dojox) {
	dojo.provide("ibm.usmi.util.DefaultMessageHandler");
	dojo.require("ibm.tivoli.navmodel.Exception");
	dojo.require("ibm.usmi.core.utils.Logger");
	dojo.require("ibm.usmi.widgets.FrameworkMessage");

	dojo.declare("ibm.usmi.util.DefaultMessageHandler", null, {

		// the logger
		_defaultMessageHandlerLogger : new ibm.usmi.core.utils.Logger("ibm.usmi.util", "DefaultMessageHandler"),
		_messages : null,
		constructor : function () {
			//Let's trace to the console window too
			if (this._defaultMessageHandlerLogger.isInfo()) {
				this._defaultMessageHandlerLogger.info("Creating new instance.", "constructor");
			}
			this._messages = [];
			this._messageResources = dojo.i18n.getLocalization("ibm.tivoli.tip.navwidget", "messages");
			registerMessageHandler(this); //main.js
		},

		displayException : function (/*Object*/ex) {

			var msgId = null;
			var msgSeverity = null;
			var msgText = null;
			var msgDescription = null;
			var moreDetails = null;

			//Checking for existence of isInstanceOf before calling it because normal JS exception won't have an isInstanceOf
			//Also check that msgId, msgSeverity and msgText exist otherwise popup may not display
			//if (ex && ex.isInstanceOf && ex.isInstanceOf(ibm.tivoli.navmodel.Exception) && ex.getMsgId() && ex.getMsgSeverity() && ex.getMsgText()) {
			if (ex && ex.isInstanceOf && ex.isInstanceOf(ibm.tivoli.navmodel.Exception)) {
				msgId = ex.getMsgId();
				msgSeverity = ex.getMsgSeverity();
				msgText = ex.getMsgText();
				msgDescription = ex.getMsgDescription();
				moreDetails = ex.getMoreDetails();

				//HACK - strip the msgID off the string since our REST classes are tacking it onto the beginning, which is ugly in the UI
				var str = msgText.substring(0, 3);
				if ((str == "ATK") || (str == "DNZ") || (str == "TCT")) {
					msgText = msgText.substring(msgText.indexOf(' ') + 1, msgText.length);
				}
			} else {

				//We have one of the following two messages to choose from. I don't think we're going to have enough in the exception to know if one of these is more appropriate than the
				//other. Also, I don't think we want to do the substitution for ${error_txt} since likely not translated.
				//
				//	- "TCTUI1002E":"An error has occurred.  The data provider could not return the data. ${error_txt}", // ${error_txt} will contain the error message returned from the server
				//	- "TCTUI1003E":"An error has occurred communicating with the server. ${error_txt}"  // ${error_txt} will contain the error message returned from the server
				msgId = "TCTUI1003E";
				msgSeverity = "Error";

				var messageTemplate = this._messageResources.TCTUI1003E;
				msgText = dojo.string.substitute(messageTemplate,
						{error_txt: ""}); //Doing an empty substitution since message in generic exception probably won't be translated
				//msgText = "An error has occurred communicating with the server.";
			}

			//Display the message
			var msgType = this._translateSeverity(msgSeverity);

			if (moreDetails) {
				this.displayMessageWithExternalLaunch(msgId, msgType, msgText, msgDescription, moreDetails, true);
			} else {
				this.displayMessage(msgId, msgType, msgText, msgDescription, true);
			}
		},

		displayMessage : function (msgId, msgType, msgTitle, msgDescription, isCollapsed) {
			 var messageArea = dojo.byId('messageArea');

			 if (msgDescription == null) {
				 msgDescription = "";
			 }

			//var message = new idx.oneui.messaging.SingleMessage({
			var message = new ibm.usmi.widgets.FrameworkMessage({
				type : msgType,
				title : msgTitle,
				messageId : msgId,
				description : msgDescription,
				showViewDetails : false,
				collapsed : isCollapsed
			}).placeAt("messageArea", "last");
			this._messages.push(message);
			var container = dijit.byId("mainContentAreaLayout");
			container.resize();
			//Let's trace to the console window too
			if (this._defaultMessageHandlerLogger.isInfo()) {
				this._defaultMessageHandlerLogger.info(msgId + " " + msgType + " " + msgTitle, "displayTMSMessage");
			}
		},

/*		displayTMSMessage : function (msgId, msgType, msgTitle, msgDescription, opResp) {
			var messageArea = dojo.byId('messageArea');
			if (msgDescription == null) {
				msgDescription = "";
			}
			//var message = new idx.oneui.messaging.SingleMessage({
			var message = new ibm.usmi.core.framework.FrameworkMessage({
				type : msgType,
				title : msgTitle,
				messageId : msgId,
				description : msgDescription,
				operatorResponse : opResp
			}).placeAt("messageArea", "last");
			this._messages.push(message);
			var container = dijit.byId("mainContentAreaLayout");
			container.resize();
			//Let's trace to the console window too
			if (this._defaultMessageHandlerLogger.isInfo()) {
				this._defaultMessageHandlerLogger.info(msgId + " " + msgType + " " + msgTitle, "displayTMSMessage");
			}
		},*/
		displayTMSMessage : function (msgId, msgType, msgTitle, explanation, opResp) {
			var msgDescription = explanation + "<p>" + opResp + "</p>";
			this.displayMessage(msgId, msgType, msgTitle, msgDescription, true);
		},

		displayMessageWithExternalLaunch : function (msgId, msgType, msgTitle, msgDescription, url) {
			 this.displayMessageWithExternalLaunch(msgId, msgType, msgTitle, msgDescription, url, null);
		},

		displayMessageWithExternalLaunch : function (msgId, msgType, msgTitle, msgDescription, url, launchLabel, isCollapsed) {
			var messageArea = dojo.byId('messageArea');

			if (msgDescription == null) {
				msgDescription = "";
			}

			//var message = new idx.oneui.messaging.SingleMessage({
			var message = new ibm.usmi.widgets.FrameworkMessage({
				type : msgType,
				title : msgTitle,
				messageId : msgId,
				description : msgDescription,
				moreDetailsURL : url,
				alternateMoreDetailsLabel : launchLabel,
				collapsed : isCollapsed
			}).placeAt("messageArea", "last");
			this._messages.push(message);

			var container = dijit.byId("mainContentAreaLayout");
			container.resize();
			//Let's trace to the console window too
			if (this._defaultMessageHandlerLogger.isInfo()) {
				this._defaultMessageHandlerLogger.info(msgId + " " + msgType + " " + msgTitle + " " + url,
						"displayMessageWithExternalLaunch");
			}
		},

		displayMessageWithLaunchInContext : function (msgId, msgType, msgTitle, msgDescription, pageId, targets, url) {
			 this.displayMessageWithLaunchInContext(msgId, msgType, msgTitle, msgDescription, pageId, targets, url, null, true);
		},

		displayMessageWithLaunchInContext : function (msgId, msgType, msgTitle, msgDescription, pageId, targets, url, launchLabel, isCollapsed) {
			var messageArea = dojo.byId('messageArea');

			if (msgDescription == null) {
				msgDescription = "";
			}

			//var message = new idx.oneui.messaging.SingleMessage({
			var message = new ibm.usmi.widgets.FrameworkMessage({
				type : msgType,
				title : msgTitle,
				messageId : msgId,
				description : msgDescription,
		        internalLaunch_pageId : pageId,
			    internalLaunch_targets : targets,
				internalLaunch_url : url,
				alternateMoreDetailsLabel : launchLabel,
				collapsed : isCollapsed
			}).placeAt("messageArea", "last");
			this._messages.push(message);

			var container = dijit.byId("mainContentAreaLayout");
			container.resize();
			//Let's trace to the console window too
			if (this._defaultMessageHandlerLogger.isInfo()) {
				this._defaultMessageHandlerLogger.info(msgId + " " + msgType + " " + msgTitle + " " + url,
						"displayMessageWithLaunchInContext");
			}
		},


		/*displayTMSMessageInline : function (msgId, msgType, msgTitle, explanation, opResp) {
			var messageArea = dojo.byId('messageArea');

			if (msgDescription == null) {
				msgDescription = "";
			}

			var msgDescription = "<b>Explanation:</b><p>" + explanation + "</p><p>" + "<b>Operator Response</b><p>"
					+ opResp + "</p>";

			//var message = new idx.oneui.messaging.SingleMessage({
			var message = new ibm.usmi.core.framework.FrameworkMessage({
				type : msgType,
				title : msgTitle,
				messageId : msgId,
				description : msgDescription,
				showViewDetails : false
			}).placeAt("messageArea", "last");
			this._messages.push(message);

			var container = dijit.byId("mainContentAreaLayout");
			container.resize();
			//Let's trace to the console window too
			if (this._defaultMessageHandlerLogger.isInfo()) {
				this._defaultMessageHandlerLogger.info(msgId + " " + msgType + " " + msgTitle, "displayMessageInline");
			}
		},*/

		_translateSeverity : function (msgSeverity) {
			if (this._defaultMessageHandlerLogger.isFinest()) {
				this._defaultMessageHandlerLogger.fine("msgSeverity = " + msgSeverity, "_translateSeverity");
			}

			var type = "error";
			if (msgSeverity == "Error") {
				type = ibm.usmi.util.DefaultMessageHandler.ERROR;
			} else if (msgSeverity == "Warning") {
				type = ibm.usmi.util.DefaultMessageHandler.WARNING;
			} else if (msgSeverity == "Info") {
				type = ibm.usmi.util.DefaultMessageHandler.INFO;
			}
			return type;
		},
		destroy : function () {
			var i = 0;
			for (; i < this._messages.length; i++) {
				this._messages[i].destroyRecursive();
			}
			this._messages = [];
			this.inherited(arguments);
		}
	});

	/*
	 * Message type. Can be 'error', 'warning', 'success', 'information', 'critical', 'attention', 'compliance'
	 */
	ibm.usmi.util.DefaultMessageHandler.ERROR = "error";
	ibm.usmi.util.DefaultMessageHandler.WARNING = "warning";
	ibm.usmi.util.DefaultMessageHandler.INFO = "information";
	ibm.usmi.util.DefaultMessageHandler.SUCCESS = "success";
	ibm.usmi.util.DefaultMessageHandler.CRITICAL = "critical";
	ibm.usmi.util.DefaultMessageHandler.ATTENTION = "attention";
	ibm.usmi.util.DefaultMessageHandler.COMPLIANCE = "compliance";

});

},
'dojox/grid/enhanced/_PluginManager':function(){
define([
	"dojo/_base/kernel",
	"dojo/_base/lang",
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/connect",
	"./_Events",
	"./_FocusManager",
	"../util"
], function(dojo, lang, declare, array, connect, _Events, _FocusManager, util){

var _PluginManager = declare("dojox.grid.enhanced._PluginManager", null, {
	// summary:
	//		Singleton plugin manager
	//
	// description:
	//		Plugin manager is responsible for
	//		1. Loading required plugins
	//		2. Handling collaborat	ion and dependencies among plugins
	//
	//      Some plugin dependencies:
	//		- "columnReordering" attribute won't work when either DnD or Indirect Selections plugin is on.
		
	//_options: Object
	//		Normalized plugin options
	_options: null,

	//_plugins: Array
	//		Plugin list
	_plugins: null,

	//_connects: Array
	//		Connection list
	_connects: null,

	constructor: function(inGrid){
		this.grid = inGrid;
		this._store = inGrid.store;
		this._options = {};
		this._plugins = [];
		this._connects = [];
		this._parseProps(this.grid.plugins);
		
		inGrid.connect(inGrid, "_setStore", lang.hitch(this, function(store){
			if(this._store !== store){
				this.forEach('onSetStore', [store, this._store]);
				this._store = store;
			}
		}));
	},
	startup: function(){
		this.forEach('onStartUp');
	},
	preInit: function(){
		// summary:
		//		Load appropriate plugins before DataGrid.postCreate().
		//		See EnhancedGrid.postCreate()
		this.grid.focus.destroy();
		this.grid.focus = new _FocusManager(this.grid);
		new _Events(this.grid);//overwrite some default events of DataGrid
		this._init(true);
		this.forEach('onPreInit');
	},
	postInit: function(){
		// summary:
		//		Load plugins after DataGrid.postCreate() - the default phase when plugins are created
		//		See EnhancedGrid.postCreate()
		this._init(false);
		
		array.forEach(this.grid.views.views, this._initView, this);
		this._connects.push(connect.connect(this.grid.views, 'addView', lang.hitch(this, this._initView)));
			
		if(this._plugins.length > 0){
			var edit = this.grid.edit;
			if(edit){ edit.styleRow = function(inRow){}; }
		}
		this.forEach('onPostInit');
	},
	forEach: function(func, args){
		array.forEach(this._plugins, function(p){
			if(!p || !p[func]){ return; }
			p[func].apply(p, args ? args : []);
		});
	},
	_parseProps: function(plugins){
		// summary:
		//		Parse plugins properties
		// plugins: Object
		//		Plugin properties defined by user
		if(!plugins){ return; }
		
		var p, loading = {}, options = this._options, grid = this.grid;
		var registry = _PluginManager.registry;//global plugin registry
		for(p in plugins){
			if(plugins[p]){//filter out boolean false e.g. {p:false}
				this._normalize(p, plugins, registry, loading);
			}
		}
		//"columnReordering" attribute won't work when either DnD or Indirect Selections plugin is used.
		if(options.dnd || options.indirectSelection){
			options.columnReordering = false;
		}
		
		//mixin all plugin properties into Grid
		lang.mixin(grid, options);
	},
	_normalize: function(p, plugins, registry, loading){
		// summary:
		//		Normalize plugin properties especially the dependency chain
		// p: String
		//		Plugin name
		// plugins: Object
		//		Plugin properties set by user
		// registry: Object
		//		The global plugin registry
		// loading: Object
		//		Map for checking process state
		if(!registry[p]){ throw new Error('Plugin ' + p + ' is required.');}
		
		if(loading[p]){ throw new Error('Recursive cycle dependency is not supported.'); }
		
		var options = this._options;
		if(options[p]){ return options[p]; }
		
		loading[p] = true;
		//TBD - more strict conditions?
		options[p] = lang.mixin({}, registry[p], lang.isObject(plugins[p]) ? plugins[p] : {});
		
		var dependencies = options[p]['dependency'];
		if(dependencies){
			if(!lang.isArray(dependencies)){
				dependencies = options[p]['dependency'] = [dependencies];
			}
			array.forEach(dependencies, function(dependency){
				if(!this._normalize(dependency, plugins, registry, loading)){
					throw new Error('Plugin ' + dependency + ' is required.');
				}
			}, this);
		}
		delete loading[p];
		return options[p];
	},
	_init: function(pre){
		// summary:
		//		Find appropriate plugins and load them
		// pre: Boolean
		//		True - preInit | False - postInit(by default)
		var p, preInit, options = this._options;
		for(p in options){
			preInit = options[p]['preInit'];
			if((pre ? preInit : !preInit) && options[p]['class'] && !this.pluginExisted(p)){
				this.loadPlugin(p);
			}
		}
	},
	loadPlugin: function(name){
		// summary:
		//		Load required plugin("name")
		// name: String
		//		Plugin name
		// return: Object
		//		The newly loaded plugin
		var option = this._options[name];
		if(!option){ return null; } //return if no plugin option
		
		var plugin = this.getPlugin(name);
		if(plugin){ return plugin; } //return if plugin("name") already existed
		
		var dependencies = option['dependency'];
		array.forEach(dependencies, function(dependency){
			if(!this.loadPlugin(dependency)){
				throw new Error('Plugin ' + dependency + ' is required.');
			}
		}, this);
		var cls = option['class'];
		delete option['class'];//remove it for safety
		plugin = new this.getPluginClazz(cls)(this.grid, option);
		this._plugins.push(plugin);
		return plugin;
	},
	_initView: function(view){
		// summary:
		//		Overwrite several default behavior for each views(including _RowSelector view)
		if(!view){ return; }
		//add more events handler - _View
		util.funnelEvents(view.contentNode, view, "doContentEvent", ['mouseup', 'mousemove']);
		util.funnelEvents(view.headerNode, view, "doHeaderEvent", ['mouseup']);
	},
	pluginExisted: function(name){
		// summary:
		//		Check if plugin("name") existed
		// name: String
		//		Plugin name
		// return: Boolean
		//		True - existed | False - not existed
		return !!this.getPlugin(name);
	},
	getPlugin: function(name){
		// summary:
		//		Get plugin("name")
		// name: String
		//		Plugin name
		// return: Object
		//		Plugin instance
		var plugins = this._plugins;
		name = name.toLowerCase();
		for(var i = 0, len = plugins.length; i < len; i++){
			if(name == plugins[i]['name'].toLowerCase()){
				return plugins[i];
			}
		}
		return null;
	},
	getPluginClazz: function(clazz){
		// summary:
		//		Load target plugin which must be already required (require(..))
		// clazz: class | String
		//		Plugin class
		if(lang.isFunction(clazz)){
			return clazz;//return if it's already a clazz
		}
		var errorMsg = 'Please make sure Plugin "' + clazz + '" is existed.';
		try{
			var cls = lang.getObject(clazz);
			if(!cls){ throw new Error(errorMsg); }
			return cls;
		}catch(e){
			throw new Error(errorMsg);
		}
	},
	isFixedCell: function(cell){
		// summary:
		//		See if target cell(column) is fixed or not.
		// cell: Object
		//		Target cell(column)
		// return: Boolean
		//		True - fixed| False - not fixed

		//target cell can use Boolean attributes named "isRowSelector" or "fixedPos" to mark it's a fixed cell(column)
		return cell && (cell.isRowSelector || cell.fixedPos);
	},
	destroy: function(){
		// summary:
		//		Destroy all resources
		array.forEach(this._connects, connect.disconnect);
		this.forEach('destroy');
		if(this.grid.unwrap){
			this.grid.unwrap();
		}
		delete this._connects;
		delete this._plugins;
		delete this._options;
	}
});

_PluginManager.registerPlugin = function(clazz, props){
		// summary:
		//		Register plugins - TODO, a better way rather than global registry?
		// clazz: String
		//		Full class name, e.g. "dojox.grid.enhanced.plugins.DnD"
		// props: Object - Optional
		//		Plugin properties e.g. {"dependency": ["nestedSorting"], ...}
	if(!clazz){
		console.warn("Failed to register plugin, class missed!");
		return;
	}
	var cls = _PluginManager;
	cls.registry = cls.registry || {};
	cls.registry[clazz.prototype.name]/*plugin name*/ = lang.mixin({"class": clazz}, (props ? props : {}));
};

return _PluginManager;

});
},
'url:gridx/templates/FilterPane.html':"<ul class=\"gridxFilterPaneForm\">\n\t<li><label id=\"${id}_ColumnLabel\" for=\"${id}_ColumnSelect\">${i18n.columnSelectLabel}</label></li>\n\t<li name=\"sltColumn\">\n\t\t<div data-dojo-type=\"dijit.form.Select\" id=\"${id}_ColumnSelect\" aria-labelledby=\"${id}_ColumnLabel\" style=\"width:100%;\"></div>\n\t</li>\n\t<li><label id=\"${id}_ConditionLabel\" for=\"${id}_ConditionSelect\">${i18n.conditionSelectLabel}</label></li>\n\t<li name=\"sltCondition\">\n\t\t<div data-dojo-type=\"dijit.form.Select\" id=\"${id}_ConditionSelect\" style=\"width:100%;\" aria-labelledby=\"${id}_ConditionLabel\"></div>\n\t</li>\n\t<li><label id=\"${id}_ValueLabel\">${i18n.valueBoxLabel}</label></li>\n\t\n\t<li class=\"gridxFilterPaneFieldWrapper gridxFilterPaneTextWrapper\">\n\t\t<input type=\"text\" data-dojo-type=\"dijit.form.TextBox\" intermediateChanges=\"true\"\n\t\t style=\"width:100%;\" aria-labelledby=\"${id}_ValueLabel\"/>\n\t</li>\n\t\n<!--            dropDownClass=\"gridx.modules.filter.DistinctComboBoxMenu\"-->\n\t<li class=\"gridxFilterPaneFieldWrapper gridxFilterPaneComboWrapper\">\n\t\t<input type=\"text\" data-dojo-type=\"dijit.form.ComboBox\" \n\t\t\tintermediateChanges=\"true\" autoComplete=\"false\" queryExpr=\"*${i18n.startsWithExpr}\" \n\t\t\tstyle=\"width:100%;\"  aria-labelledby=\"${id}_ValueLabel\"/>\n\t</li>\n\t\n\t<li class=\"gridxFilterPaneFieldWrapper gridxFilterPaneNumberWrapper\">\n\t\t<input type=\"text\" data-dojo-type=\"dijit.form.NumberTextBox\"  aria-labelledby=\"${id}_ValueLabel\" intermediateChanges=\"true\" style=\"width:100%;\"/>\n\t</li>\n\t\n\t<li class=\"gridxFilterPaneFieldWrapper gridxFilterPaneDateWrapper\">\n\t\t<div data-dojo-type=\"dijit.form.DateTextBox\" aria-labelledby=\"${id}_ValueLabel\" intermediateChanges=\"true\" style=\"width: 100%\"></div>\n\t</li>\n\t\n\t<li class=\"gridxFilterPaneFieldWrapper gridxFilterPaneDateRangeWrapper\">\n\t\t<div data-dojo-type=\"dijit.form.DateTextBox\" style=\"width:44%; float: left;\" intermediateChanges=\"true\"\n\t\t\t aria-label=\"${i18n.beginDateRangeLabel}\"></div>\n\t\t<div style=\"width:10%; text-align: center; float: left;\">${i18n.rangeTo}</div>\n\t\t<div data-dojo-type=\"dijit.form.DateTextBox\" style=\"width:44%; float: right;\" intermediateChanges=\"true\"\n\t\t\t aria-label=\"${i18n.endDateRangeLabel}\"></div>\n\t</li>\n\t\n\t<li class=\"gridxFilterPaneFieldWrapper gridxFilterPaneTimeWrapper\">\n\t\t<div data-dojo-type=\"dijit.form.TimeTextBox\" aria-labelledby=\"${id}_ValueLabel\" style=\"width: 100%\" intermediateChanges=\"true\"></div>\n\t</li>\n\t\n\t<li class=\"gridxFilterPaneFieldWrapper gridxFilterPaneTimeRangeWrapper\">\n\t\t<div data-dojo-type=\"dijit.form.TimeTextBox\" style=\"width:44%; float: left;\" intermediateChanges=\"true\"\n\t\t\t aria-label=\"${i18n.beginTimeRangeLabel}\"></div>\n\t\t<div style=\"text-align: center; float: left; width: 10%;\">${i18n.rangeTo}</div>\n\t\t<div data-dojo-type=\"dijit.form.TimeTextBox\" style=\"width:44%; float: right;\" intermediateChanges=\"true\"\n\t\t\t aria-label=\"${i18n.endTimeRangeLabel}\"></div>\n\t</li>\n\t\n\t<li class=\"gridxFilterPaneFieldWrapper gridxFilterPaneRadioWrapper\">\n\t\t<span style=\"width:49%; float:left;\">\n\t\t\t<div data-dojo-type=\"dijit.form.RadioButton\" aria-label=\"${i18n.radioTrueLabel}\" checked=\"true\"></div><span>${i18n.trueLabel}</span>\n\t\t</span>\n\t\t<span style=\"width:49%; float: right;\">\n\t\t\t<div data-dojo-type=\"dijit.form.RadioButton\" aria-label=\"${i18n.radioFalseLabel}\"></div><span>${i18n.falseLabel}</span>\n\t\t</span>\n\t</li>\n\t\n\t<li class=\"gridxFilterPaneFieldWrapper gridxFilterPaneSelectWrapper\">\n\t\t<div data-dojo-type=\"dijit.form.Select\" aria-labelledby=\"${id}_ValueLabel\" style=\"width: 100%\"></div>\n\t</li>\n</ul>\n",
'gridx/modules/ColumnWidth':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/Deferred",
	"dojo/_base/query",
	"dojo/_base/sniff",
	"dojo/dom-geometry",
	"dojo/dom-class",
	"dojo/dom-style",
	"dojo/keys",
	"../core/_Module"
], function(declare, array, Deferred, query, sniff, domGeometry, domClass, domStyle, keys, _Module){

	return declare(/*===== "gridx.modules.ColumnWidth", =====*/_Module, {
		// summary:
		//		Manages column width distribution, allow grid autoWidth and column autoResize.

		name: 'columnWidth',
	
		forced: ['hLayout'],

		getAPIPath: function(){
			// tags:
			//		protected extension
			return {
				columnWidth: this
			};
		},

		constructor: function(){
			this._init();
		},

		preload: function(){
			// tags:
			//		protected extension
			var t = this,
				g = t.grid;
			t._ready = new Deferred();
			t.batchConnect(
				[g.hLayout, 'onUpdateWidth', '_onUpdateWidth'],
				[g, 'setColumns', '_onSetColumns']);
		},

		load: function(){
			this._adaptWidth();
			this.loaded.callback();
		},

		//Public-----------------------------------------------------------------------------

		// default: Number
		//		Default column width. Applied when it's not possible to decide accurate column width from user's config.
		'default': 60,

		// autoResize: Boolean
		//		If set to true, the column width can only be set to auto or percentage values (if not, it'll be regarded as auto),
		//		then the column will automatically resize when the grid width is changed (this is the default behavior of an
		//		HTML table).
		autoResize: false,

		onUpdate: function(){
			// summary:
			//		Fired when column widths are updated.
		},

		//Private-----------------------------------------------------------------------------
		_init: function(){
			var t = this,
				g = t.grid,
				dn = g.domNode,
				cols = g._columns;
			array.forEach(cols, function(col){
				if(!col.hasOwnProperty('declaredWidth')){
					col.declaredWidth = col.width = col.width || 'auto';
				}
			});
			if(g.autoWidth){
				array.forEach(cols, function(c){
					if(c.declaredWidth == 'auto'){
						c.width = t.arg('default') + 'px';
					}
				});
			}else if(t.arg('autoResize')){
				domClass.add(dn, 'gridxPercentColumnWidth');
				array.forEach(cols, function(c){
					if(!(/%$/).test(c.declaredWidth)){
						c.width = 'auto';
					}
				});
			}
		},

		_onUpdateWidth: function(){
			var t = this,
				g = t.grid;
			if(g.autoWidth){
				t._adaptWidth();
			}else{
				var noHScroller = g.hScrollerNode.style.display == 'none';
				t._adaptWidth(!noHScroller, 1);	//1 as true
				if(!t.arg('autoResize') && noHScroller){
					query('.gridxCell', g.bodyNode).forEach(function(cellNode){
						var col = g._columnsById[cellNode.getAttribute('colId')];
						if(t.arg('autoResize') ||
							!col.declaredWidth ||
							col.declaredWidth == 'auto' ||
							(/%$/).test(col.declaredWidth)){
							cellNode.style.width = col.width;
						}
					});
				}
				t.onUpdate();
			}
		},

		_adaptWidth: function(skip, noEvent){
			var t = this,
				g = t.grid,
				dn = g.domNode,
				header = g.header,
				ltr = g.isLeftToRight(),
				marginLead = ltr ? 'marginLeft' : 'marginRight',
				marginTail = ltr ? 'marginRight' : 'marginLeft',
				lead = g.hLayout.lead,
				tail = g.hLayout.tail,
				innerNode = header.innerNode,
				bs = g.bodyNode.style,
				hs = innerNode.style,
				bodyWidth = (dn.clientWidth || domStyle.get(dn, 'width')) - lead - tail,
				refNode = query('.gridxCell', innerNode)[0],
				padBorder = refNode ? domGeometry.getMarginBox(refNode).w - domGeometry.getContentBox(refNode).w : 0,
				isGridHidden = !dn.offsetHeight,
				isCollapse = refNode && domStyle.get(refNode, 'borderCollapse') == 'collapse',
				margin = isCollapse && lead > 0 ? (lead - 1) : lead;
			hs[marginLead] = margin + 'px';
			hs[marginTail] = ((t.arg('autoResize') || !isCollapse) && tail > 0 ? tail - 1 : 0)  + 'px';
			g.mainNode.style[marginLead] = margin + 'px';
			g.mainNode.style[marginTail] = tail + 'px';
			bodyWidth = bodyWidth < 0 ? 0 : bodyWidth;
			if(skip){
				t.onUpdate();
				return;
			}
			if(isCollapse){
				padBorder += isGridHidden ? -1 : 1;
			}
			if(g.autoWidth){
				var headers = query('th.gridxCell', innerNode),
					totalWidth = isCollapse ? 2 : 0;
				headers.forEach(function(node){
					var w = domStyle.get(node, 'width');
					if(!sniff('safari') || !isGridHidden){
						w += padBorder;
					}
					totalWidth += w;
					if(isCollapse){
						totalWidth--;
					}
					var c = g._columnsById[node.getAttribute('colid')];
					if(c.width == 'auto' || (/%$/).test(c.width)){
						node.style.width = c.width = w + 'px';
					}
				});
				bs.width = totalWidth + 'px';
				dn.style.width = (lead + tail + totalWidth) + 'px';
			}else if(!t.arg('autoResize')){
				var autoCols = [],
					cols = g._columns,
					fixedWidth = isCollapse ? 2 : 0;
				array.forEach(cols, function(c){
					if(c.declaredWidth == 'auto'){
						autoCols.push(c);
					}else if(/%$/.test(c.declaredWidth)){
						var w = parseInt(bodyWidth * parseFloat(c.declaredWidth, 10) / 100 -
							(sniff('safari') ? (isCollapse ? 1 : 0) : padBorder), 10);
						//Check if less than zero, prevent error in IE.
						if(w < 0){
							w = 0;
						}
						header.getHeaderNode(c.id).style.width = c.width = w + 'px';
					}
				});
				array.forEach(cols, function(c){
					if(c.declaredWidth != 'auto'){
						var w = sniff('safari') ?
							parseFloat(header.getHeaderNode(c.id).style.width, 10) :
							domStyle.get(header.getHeaderNode(c.id), 'width');
						if(/%$/.test(c.declaredWidth)){
							c.width = w + 'px';
						}
						if(!sniff('safari')){
							w += padBorder;
						}
						fixedWidth += w;
					}
				});
				if(autoCols.length){
					if(sniff('safari')){
						padBorder = 0;
					}
					var w = bodyWidth > fixedWidth ? ((bodyWidth - fixedWidth) / autoCols.length - padBorder) : t.arg('default'),
						ww = parseInt(w, 10);
					if(bodyWidth > fixedWidth){
						if(isCollapse){
							w += cols.length / autoCols.length;
							//FIXME:IE7 is strange here...
							if(sniff('ie') < 8){
								w += cols.length / autoCols.length;
							}
						}
						ww = bodyWidth - fixedWidth - (ww + padBorder) * (autoCols.length - 1) - padBorder;
					}
					w = parseInt(w, 10);
					//Check if less than zero, prevent error in IE.
					if(w < 0){
						w = 0;
					}
					if(ww < 0){
						ww = 0;
					}
					array.forEach(autoCols, function(c, i){
						header.getHeaderNode(c.id).style.width = c.width = (i < autoCols.length - 1 ? w : ww) + 'px';
					});
				}
			}
			g.hScroller.scroll(0);
			header._onHScroll(0);
			g.vLayout.reLayout();
			if(!noEvent){
				t.onUpdate();
			}
		},

		_onSetColumns: function(){
			var t = this,
				g = t.grid;
			t._init();
			g.header.refresh();
			t._adaptWidth();
			//FIXME: Is there any more elegant way to do this?
			if(g.cellWidget){
				g.cellWidget._init();
				if(g.edit){
					g.edit._init();
				}
			}
			if(g.tree){
				g.tree._initExpandLevel();
			}
			g.body.refresh();
		}
	});
});

},
'dojo/data/ItemFileReadStore':function(){
define(["../_base/kernel", "../_base/lang", "../_base/declare", "../_base/array", "../_base/xhr", 
	"../Evented", "../_base/window", "./util/filter", "./util/simpleFetch", "../date/stamp"
], function(kernel, lang, declare, array, xhr, Evented, window, filterUtil, simpleFetch, dateStamp) {
	// module:
	//		dojo/data/ItemFileReadStore
	// summary:
	//		TODOC


var ItemFileReadStore = declare("dojo.data.ItemFileReadStore", [Evented],{
	//	summary:
	//		The ItemFileReadStore implements the dojo.data.api.Read API and reads
	//		data from JSON files that have contents in this format --
	//		{ items: [
	//			{ name:'Kermit', color:'green', age:12, friends:['Gonzo', {_reference:{name:'Fozzie Bear'}}]},
	//			{ name:'Fozzie Bear', wears:['hat', 'tie']},
	//			{ name:'Miss Piggy', pets:'Foo-Foo'}
	//		]}
	//		Note that it can also contain an 'identifer' property that specified which attribute on the items
	//		in the array of items that acts as the unique identifier for that item.
	//
	constructor: function(/* Object */ keywordParameters){
		//	summary: constructor
		//	keywordParameters: {url: String}
		//	keywordParameters: {data: jsonObject}
		//	keywordParameters: {typeMap: object)
		//		The structure of the typeMap object is as follows:
		//		{
		//			type0: function || object,
		//			type1: function || object,
		//			...
		//			typeN: function || object
		//		}
		//		Where if it is a function, it is assumed to be an object constructor that takes the
		//		value of _value as the initialization parameters.  If it is an object, then it is assumed
		//		to be an object of general form:
		//		{
		//			type: function, //constructor.
		//			deserialize:	function(value) //The function that parses the value and constructs the object defined by type appropriately.
		//		}

		this._arrayOfAllItems = [];
		this._arrayOfTopLevelItems = [];
		this._loadFinished = false;
		this._jsonFileUrl = keywordParameters.url;
		this._ccUrl = keywordParameters.url;
		this.url = keywordParameters.url;
		this._jsonData = keywordParameters.data;
		this.data = null;
		this._datatypeMap = keywordParameters.typeMap || {};
		if(!this._datatypeMap['Date']){
			//If no default mapping for dates, then set this as default.
			//We use the dojo.date.stamp here because the ISO format is the 'dojo way'
			//of generically representing dates.
			this._datatypeMap['Date'] = {
											type: Date,
											deserialize: function(value){
												return dateStamp.fromISOString(value);
											}
										};
		}
		this._features = {'dojo.data.api.Read':true, 'dojo.data.api.Identity':true};
		this._itemsByIdentity = null;
		this._storeRefPropName = "_S"; // Default name for the store reference to attach to every item.
		this._itemNumPropName = "_0"; // Default Item Id for isItem to attach to every item.
		this._rootItemPropName = "_RI"; // Default Item Id for isItem to attach to every item.
		this._reverseRefMap = "_RRM"; // Default attribute for constructing a reverse reference map for use with reference integrity
		this._loadInProgress = false; //Got to track the initial load to prevent duelling loads of the dataset.
		this._queuedFetches = [];
		if(keywordParameters.urlPreventCache !== undefined){
			this.urlPreventCache = keywordParameters.urlPreventCache?true:false;
		}
		if(keywordParameters.hierarchical !== undefined){
			this.hierarchical = keywordParameters.hierarchical?true:false;
		}
		if(keywordParameters.clearOnClose){
			this.clearOnClose = true;
		}
		if("failOk" in keywordParameters){
			this.failOk = keywordParameters.failOk?true:false;
		}
	},

	url: "",	// use "" rather than undefined for the benefit of the parser (#3539)

	//Internal var, crossCheckUrl.  Used so that setting either url or _jsonFileUrl, can still trigger a reload
	//when clearOnClose and close is used.
	_ccUrl: "",

	data: null,	// define this so that the parser can populate it

	typeMap: null, //Define so parser can populate.

	//Parameter to allow users to specify if a close call should force a reload or not.
	//By default, it retains the old behavior of not clearing if close is called.  But
	//if set true, the store will be reset to default state.  Note that by doing this,
	//all item handles will become invalid and a new fetch must be issued.
	clearOnClose: false,

	//Parameter to allow specifying if preventCache should be passed to the xhrGet call or not when loading data from a url.
	//Note this does not mean the store calls the server on each fetch, only that the data load has preventCache set as an option.
	//Added for tracker: #6072
	urlPreventCache: false,

	//Parameter for specifying that it is OK for the xhrGet call to fail silently.
	failOk: false,

	//Parameter to indicate to process data from the url as hierarchical
	//(data items can contain other data items in js form).  Default is true
	//for backwards compatibility.  False means only root items are processed
	//as items, all child objects outside of type-mapped objects and those in
	//specific reference format, are left straight JS data objects.
	hierarchical: true,

	_assertIsItem: function(/* item */ item){
		//	summary:
		//		This function tests whether the item passed in is indeed an item in the store.
		//	item:
		//		The item to test for being contained by the store.
		if(!this.isItem(item)){
			throw new Error("dojo.data.ItemFileReadStore: Invalid item argument.");
		}
	},

	_assertIsAttribute: function(/* attribute-name-string */ attribute){
		//	summary:
		//		This function tests whether the item passed in is indeed a valid 'attribute' like type for the store.
		//	attribute:
		//		The attribute to test for being contained by the store.
		if(typeof attribute !== "string"){
			throw new Error("dojo.data.ItemFileReadStore: Invalid attribute argument.");
		}
	},

	getValue: function(	/* item */ item,
						/* attribute-name-string */ attribute,
						/* value? */ defaultValue){
		//	summary:
		//		See dojo.data.api.Read.getValue()
		var values = this.getValues(item, attribute);
		return (values.length > 0)?values[0]:defaultValue; // mixed
	},

	getValues: function(/* item */ item,
						/* attribute-name-string */ attribute){
		//	summary:
		//		See dojo.data.api.Read.getValues()

		this._assertIsItem(item);
		this._assertIsAttribute(attribute);
		// Clone it before returning.  refs: #10474
		return (item[attribute] || []).slice(0); // Array
	},

	getAttributes: function(/* item */ item){
		//	summary:
		//		See dojo.data.api.Read.getAttributes()
		this._assertIsItem(item);
		var attributes = [];
		for(var key in item){
			// Save off only the real item attributes, not the special id marks for O(1) isItem.
			if((key !== this._storeRefPropName) && (key !== this._itemNumPropName) && (key !== this._rootItemPropName) && (key !== this._reverseRefMap)){
				attributes.push(key);
			}
		}
		return attributes; // Array
	},

	hasAttribute: function(	/* item */ item,
							/* attribute-name-string */ attribute){
		//	summary:
		//		See dojo.data.api.Read.hasAttribute()
		this._assertIsItem(item);
		this._assertIsAttribute(attribute);
		return (attribute in item);
	},

	containsValue: function(/* item */ item,
							/* attribute-name-string */ attribute,
							/* anything */ value){
		//	summary:
		//		See dojo.data.api.Read.containsValue()
		var regexp = undefined;
		if(typeof value === "string"){
			regexp = filterUtil.patternToRegExp(value, false);
		}
		return this._containsValue(item, attribute, value, regexp); //boolean.
	},

	_containsValue: function(	/* item */ item,
								/* attribute-name-string */ attribute,
								/* anything */ value,
								/* RegExp?*/ regexp){
		//	summary:
		//		Internal function for looking at the values contained by the item.
		//	description:
		//		Internal function for looking at the values contained by the item.  This
		//		function allows for denoting if the comparison should be case sensitive for
		//		strings or not (for handling filtering cases where string case should not matter)
		//
		//	item:
		//		The data item to examine for attribute values.
		//	attribute:
		//		The attribute to inspect.
		//	value:
		//		The value to match.
		//	regexp:
		//		Optional regular expression generated off value if value was of string type to handle wildcarding.
		//		If present and attribute values are string, then it can be used for comparison instead of 'value'
		return array.some(this.getValues(item, attribute), function(possibleValue){
			if(possibleValue !== null && !lang.isObject(possibleValue) && regexp){
				if(possibleValue.toString().match(regexp)){
					return true; // Boolean
				}
			}else if(value === possibleValue){
				return true; // Boolean
			}
		});
	},

	isItem: function(/* anything */ something){
		//	summary:
		//		See dojo.data.api.Read.isItem()
		if(something && something[this._storeRefPropName] === this){
			if(this._arrayOfAllItems[something[this._itemNumPropName]] === something){
				return true;
			}
		}
		return false; // Boolean
	},

	isItemLoaded: function(/* anything */ something){
		//	summary:
		//		See dojo.data.api.Read.isItemLoaded()
		return this.isItem(something); //boolean
	},

	loadItem: function(/* object */ keywordArgs){
		//	summary:
		//		See dojo.data.api.Read.loadItem()
		this._assertIsItem(keywordArgs.item);
	},

	getFeatures: function(){
		//	summary:
		//		See dojo.data.api.Read.getFeatures()
		return this._features; //Object
	},

	getLabel: function(/* item */ item){
		//	summary:
		//		See dojo.data.api.Read.getLabel()
		if(this._labelAttr && this.isItem(item)){
			return this.getValue(item,this._labelAttr); //String
		}
		return undefined; //undefined
	},

	getLabelAttributes: function(/* item */ item){
		//	summary:
		//		See dojo.data.api.Read.getLabelAttributes()
		if(this._labelAttr){
			return [this._labelAttr]; //array
		}
		return null; //null
	},

	_fetchItems: function(	/* Object */ keywordArgs,
							/* Function */ findCallback,
							/* Function */ errorCallback){
		//	summary:
		//		See dojo.data.util.simpleFetch.fetch()
		var self = this,
		    filter = function(requestArgs, arrayOfItems){
			var items = [],
			    i, key;
			if(requestArgs.query){
				var value,
				    ignoreCase = requestArgs.queryOptions ? requestArgs.queryOptions.ignoreCase : false;

				//See if there are any string values that can be regexp parsed first to avoid multiple regexp gens on the
				//same value for each item examined.  Much more efficient.
				var regexpList = {};
				for(key in requestArgs.query){
					value = requestArgs.query[key];
					if(typeof value === "string"){
						regexpList[key] = filterUtil.patternToRegExp(value, ignoreCase);
					}else if(value instanceof RegExp){
						regexpList[key] = value;
					}
				}
				for(i = 0; i < arrayOfItems.length; ++i){
					var match = true;
					var candidateItem = arrayOfItems[i];
					if(candidateItem === null){
						match = false;
					}else{
						for(key in requestArgs.query){
							value = requestArgs.query[key];
							if(!self._containsValue(candidateItem, key, value, regexpList[key])){
								match = false;
							}
						}
					}
					if(match){
						items.push(candidateItem);
					}
				}
				findCallback(items, requestArgs);
			}else{
				// We want a copy to pass back in case the parent wishes to sort the array.
				// We shouldn't allow resort of the internal list, so that multiple callers
				// can get lists and sort without affecting each other.  We also need to
				// filter out any null values that have been left as a result of deleteItem()
				// calls in ItemFileWriteStore.
				for(i = 0; i < arrayOfItems.length; ++i){
					var item = arrayOfItems[i];
					if(item !== null){
						items.push(item);
					}
				}
				findCallback(items, requestArgs);
			}
		};

		if(this._loadFinished){
			filter(keywordArgs, this._getItemsArray(keywordArgs.queryOptions));
		}else{
			//Do a check on the JsonFileUrl and crosscheck it.
			//If it doesn't match the cross-check, it needs to be updated
			//This allows for either url or _jsonFileUrl to he changed to
			//reset the store load location.  Done this way for backwards
			//compatibility.  People use _jsonFileUrl (even though officially
			//private.
			if(this._jsonFileUrl !== this._ccUrl){
				kernel.deprecated("dojo.data.ItemFileReadStore: ",
					"To change the url, set the url property of the store," +
					" not _jsonFileUrl.  _jsonFileUrl support will be removed in 2.0");
				this._ccUrl = this._jsonFileUrl;
				this.url = this._jsonFileUrl;
			}else if(this.url !== this._ccUrl){
				this._jsonFileUrl = this.url;
				this._ccUrl = this.url;
			}

			//See if there was any forced reset of data.
			if(this.data != null){
				this._jsonData = this.data;
				this.data = null;
			}

			if(this._jsonFileUrl){
				//If fetches come in before the loading has finished, but while
				//a load is in progress, we have to defer the fetching to be
				//invoked in the callback.
				if(this._loadInProgress){
					this._queuedFetches.push({args: keywordArgs, filter: filter});
				}else{
					this._loadInProgress = true;
					var getArgs = {
							url: self._jsonFileUrl,
							handleAs: "json-comment-optional",
							preventCache: this.urlPreventCache,
							failOk: this.failOk
						};
					var getHandler = xhr.get(getArgs);
					getHandler.addCallback(function(data){
						try{
							self._getItemsFromLoadedData(data);
							self._loadFinished = true;
							self._loadInProgress = false;

							filter(keywordArgs, self._getItemsArray(keywordArgs.queryOptions));
							self._handleQueuedFetches();
						}catch(e){
							self._loadFinished = true;
							self._loadInProgress = false;
							errorCallback(e, keywordArgs);
						}
					});
					getHandler.addErrback(function(error){
						self._loadInProgress = false;
						errorCallback(error, keywordArgs);
					});

					//Wire up the cancel to abort of the request
					//This call cancel on the deferred if it hasn't been called
					//yet and then will chain to the simple abort of the
					//simpleFetch keywordArgs
					var oldAbort = null;
					if(keywordArgs.abort){
						oldAbort = keywordArgs.abort;
					}
					keywordArgs.abort = function(){
						var df = getHandler;
						if(df && df.fired === -1){
							df.cancel();
							df = null;
						}
						if(oldAbort){
							oldAbort.call(keywordArgs);
						}
					};
				}
			}else if(this._jsonData){
				try{
					this._loadFinished = true;
					this._getItemsFromLoadedData(this._jsonData);
					this._jsonData = null;
					filter(keywordArgs, this._getItemsArray(keywordArgs.queryOptions));
				}catch(e){
					errorCallback(e, keywordArgs);
				}
			}else{
				errorCallback(new Error("dojo.data.ItemFileReadStore: No JSON source data was provided as either URL or a nested Javascript object."), keywordArgs);
			}
		}
	},

	_handleQueuedFetches: function(){
		//	summary:
		//		Internal function to execute delayed request in the store.
		//Execute any deferred fetches now.
		if(this._queuedFetches.length > 0){
			for(var i = 0; i < this._queuedFetches.length; i++){
				var fData = this._queuedFetches[i],
				    delayedQuery = fData.args,
				    delayedFilter = fData.filter;
				if(delayedFilter){
					delayedFilter(delayedQuery, this._getItemsArray(delayedQuery.queryOptions));
				}else{
					this.fetchItemByIdentity(delayedQuery);
				}
			}
			this._queuedFetches = [];
		}
	},

	_getItemsArray: function(/*object?*/queryOptions){
		//	summary:
		//		Internal function to determine which list of items to search over.
		//	queryOptions: The query options parameter, if any.
		if(queryOptions && queryOptions.deep){
			return this._arrayOfAllItems;
		}
		return this._arrayOfTopLevelItems;
	},

	close: function(/*dojo.data.api.Request || keywordArgs || null */ request){
		 //	summary:
		 //		See dojo.data.api.Read.close()
		 if(this.clearOnClose &&
			this._loadFinished &&
			!this._loadInProgress){
			 //Reset all internalsback to default state.  This will force a reload
			 //on next fetch.  This also checks that the data or url param was set
			 //so that the store knows it can get data.  Without one of those being set,
			 //the next fetch will trigger an error.

			 if(((this._jsonFileUrl == "" || this._jsonFileUrl == null) &&
				 (this.url == "" || this.url == null)
				) && this.data == null){
				 console.debug("dojo.data.ItemFileReadStore: WARNING!  Data reload " +
					" information has not been provided." +
					"  Please set 'url' or 'data' to the appropriate value before" +
					" the next fetch");
			 }
			 this._arrayOfAllItems = [];
			 this._arrayOfTopLevelItems = [];
			 this._loadFinished = false;
			 this._itemsByIdentity = null;
			 this._loadInProgress = false;
			 this._queuedFetches = [];
		 }
	},

	_getItemsFromLoadedData: function(/* Object */ dataObject){
		//	summary:
		//		Function to parse the loaded data into item format and build the internal items array.
		//	description:
		//		Function to parse the loaded data into item format and build the internal items array.
		//
		//	dataObject:
		//		The JS data object containing the raw data to convery into item format.
		//
		// 	returns: array
		//		Array of items in store item format.

		// First, we define a couple little utility functions...
		var addingArrays = false,
		    self = this;

		function valueIsAnItem(/* anything */ aValue){
			// summary:
			//		Given any sort of value that could be in the raw json data,
			//		return true if we should interpret the value as being an
			//		item itself, rather than a literal value or a reference.
			// example:
			// 	|	false == valueIsAnItem("Kermit");
			// 	|	false == valueIsAnItem(42);
			// 	|	false == valueIsAnItem(new Date());
			// 	|	false == valueIsAnItem({_type:'Date', _value:'1802-05-14'});
			// 	|	false == valueIsAnItem({_reference:'Kermit'});
			// 	|	true == valueIsAnItem({name:'Kermit', color:'green'});
			// 	|	true == valueIsAnItem({iggy:'pop'});
			// 	|	true == valueIsAnItem({foo:42});
			return (aValue !== null) &&
				(typeof aValue === "object") &&
				(!lang.isArray(aValue) || addingArrays) &&
				(!lang.isFunction(aValue)) &&
				(aValue.constructor == Object || lang.isArray(aValue)) &&
				(typeof aValue._reference === "undefined") &&
				(typeof aValue._type === "undefined") &&
				(typeof aValue._value === "undefined") &&
				self.hierarchical;
		}

		function addItemAndSubItemsToArrayOfAllItems(/* Item */ anItem){
			self._arrayOfAllItems.push(anItem);
			for(var attribute in anItem){
				var valueForAttribute = anItem[attribute];
				if(valueForAttribute){
					if(lang.isArray(valueForAttribute)){
						var valueArray = valueForAttribute;
						for(var k = 0; k < valueArray.length; ++k){
							var singleValue = valueArray[k];
							if(valueIsAnItem(singleValue)){
								addItemAndSubItemsToArrayOfAllItems(singleValue);
							}
						}
					}else{
						if(valueIsAnItem(valueForAttribute)){
							addItemAndSubItemsToArrayOfAllItems(valueForAttribute);
						}
					}
				}
			}
		}

		this._labelAttr = dataObject.label;

		// We need to do some transformations to convert the data structure
		// that we read from the file into a format that will be convenient
		// to work with in memory.

		// Step 1: Walk through the object hierarchy and build a list of all items
		var i,
		    item;
		this._arrayOfAllItems = [];
		this._arrayOfTopLevelItems = dataObject.items;

		for(i = 0; i < this._arrayOfTopLevelItems.length; ++i){
			item = this._arrayOfTopLevelItems[i];
			if(lang.isArray(item)){
				addingArrays = true;
			}
			addItemAndSubItemsToArrayOfAllItems(item);
			item[this._rootItemPropName]=true;
		}

		// Step 2: Walk through all the attribute values of all the items,
		// and replace single values with arrays.  For example, we change this:
		//		{ name:'Miss Piggy', pets:'Foo-Foo'}
		// into this:
		//		{ name:['Miss Piggy'], pets:['Foo-Foo']}
		//
		// We also store the attribute names so we can validate our store
		// reference and item id special properties for the O(1) isItem
		var allAttributeNames = {},
		    key;

		for(i = 0; i < this._arrayOfAllItems.length; ++i){
			item = this._arrayOfAllItems[i];
			for(key in item){
				if(key !== this._rootItemPropName){
					var value = item[key];
					if(value !== null){
						if(!lang.isArray(value)){
							item[key] = [value];
						}
					}else{
						item[key] = [null];
					}
				}
				allAttributeNames[key]=key;
			}
		}

		// Step 3: Build unique property names to use for the _storeRefPropName and _itemNumPropName
		// This should go really fast, it will generally never even run the loop.
		while(allAttributeNames[this._storeRefPropName]){
			this._storeRefPropName += "_";
		}
		while(allAttributeNames[this._itemNumPropName]){
			this._itemNumPropName += "_";
		}
		while(allAttributeNames[this._reverseRefMap]){
			this._reverseRefMap += "_";
		}

		// Step 4: Some data files specify an optional 'identifier', which is
		// the name of an attribute that holds the identity of each item.
		// If this data file specified an identifier attribute, then build a
		// hash table of items keyed by the identity of the items.
		var arrayOfValues;

		var identifier = dataObject.identifier;
		if(identifier){
			this._itemsByIdentity = {};
			this._features['dojo.data.api.Identity'] = identifier;
			for(i = 0; i < this._arrayOfAllItems.length; ++i){
				item = this._arrayOfAllItems[i];
				arrayOfValues = item[identifier];
				var identity = arrayOfValues[0];
				if(!Object.hasOwnProperty.call(this._itemsByIdentity, identity)){
					this._itemsByIdentity[identity] = item;
				}else{
					if(this._jsonFileUrl){
						throw new Error("dojo.data.ItemFileReadStore:  The json data as specified by: [" + this._jsonFileUrl + "] is malformed.  Items within the list have identifier: [" + identifier + "].  Value collided: [" + identity + "]");
					}else if(this._jsonData){
						throw new Error("dojo.data.ItemFileReadStore:  The json data provided by the creation arguments is malformed.  Items within the list have identifier: [" + identifier + "].  Value collided: [" + identity + "]");
					}
				}
			}
		}else{
			this._features['dojo.data.api.Identity'] = Number;
		}

		// Step 5: Walk through all the items, and set each item's properties
		// for _storeRefPropName and _itemNumPropName, so that store.isItem() will return true.
		for(i = 0; i < this._arrayOfAllItems.length; ++i){
			item = this._arrayOfAllItems[i];
			item[this._storeRefPropName] = this;
			item[this._itemNumPropName] = i;
		}

		// Step 6: We walk through all the attribute values of all the items,
		// looking for type/value literals and item-references.
		//
		// We replace item-references with pointers to items.  For example, we change:
		//		{ name:['Kermit'], friends:[{_reference:{name:'Miss Piggy'}}] }
		// into this:
		//		{ name:['Kermit'], friends:[miss_piggy] }
		// (where miss_piggy is the object representing the 'Miss Piggy' item).
		//
		// We replace type/value pairs with typed-literals.  For example, we change:
		//		{ name:['Nelson Mandela'], born:[{_type:'Date', _value:'1918-07-18'}] }
		// into this:
		//		{ name:['Kermit'], born:(new Date(1918, 6, 18)) }
		//
		// We also generate the associate map for all items for the O(1) isItem function.
		for(i = 0; i < this._arrayOfAllItems.length; ++i){
			item = this._arrayOfAllItems[i]; // example: { name:['Kermit'], friends:[{_reference:{name:'Miss Piggy'}}] }
			for(key in item){
				arrayOfValues = item[key]; // example: [{_reference:{name:'Miss Piggy'}}]
				for(var j = 0; j < arrayOfValues.length; ++j){
					value = arrayOfValues[j]; // example: {_reference:{name:'Miss Piggy'}}
					if(value !== null && typeof value == "object"){
						if(("_type" in value) && ("_value" in value)){
							var type = value._type; // examples: 'Date', 'Color', or 'ComplexNumber'
							var mappingObj = this._datatypeMap[type]; // examples: Date, dojo.Color, foo.math.ComplexNumber, {type: dojo.Color, deserialize(value){ return new dojo.Color(value)}}
							if(!mappingObj){
								throw new Error("dojo.data.ItemFileReadStore: in the typeMap constructor arg, no object class was specified for the datatype '" + type + "'");
							}else if(lang.isFunction(mappingObj)){
								arrayOfValues[j] = new mappingObj(value._value);
							}else if(lang.isFunction(mappingObj.deserialize)){
								arrayOfValues[j] = mappingObj.deserialize(value._value);
							}else{
								throw new Error("dojo.data.ItemFileReadStore: Value provided in typeMap was neither a constructor, nor a an object with a deserialize function");
							}
						}
						if(value._reference){
							var referenceDescription = value._reference; // example: {name:'Miss Piggy'}
							if(!lang.isObject(referenceDescription)){
								// example: 'Miss Piggy'
								// from an item like: { name:['Kermit'], friends:[{_reference:'Miss Piggy'}]}
								arrayOfValues[j] = this._getItemByIdentity(referenceDescription);
							}else{
								// example: {name:'Miss Piggy'}
								// from an item like: { name:['Kermit'], friends:[{_reference:{name:'Miss Piggy'}}] }
								for(var k = 0; k < this._arrayOfAllItems.length; ++k){
									var candidateItem = this._arrayOfAllItems[k],
									    found = true;
									for(var refKey in referenceDescription){
										if(candidateItem[refKey] != referenceDescription[refKey]){
											found = false;
										}
									}
									if(found){
										arrayOfValues[j] = candidateItem;
									}
								}
							}
							if(this.referenceIntegrity){
								var refItem = arrayOfValues[j];
								if(this.isItem(refItem)){
									this._addReferenceToMap(refItem, item, key);
								}
							}
						}else if(this.isItem(value)){
							//It's a child item (not one referenced through _reference).
							//We need to treat this as a referenced item, so it can be cleaned up
							//in a write store easily.
							if(this.referenceIntegrity){
								this._addReferenceToMap(value, item, key);
							}
						}
					}
				}
			}
		}
	},

	_addReferenceToMap: function(/*item*/ refItem, /*item*/ parentItem, /*string*/ attribute){
		 //	summary:
		 //		Method to add an reference map entry for an item and attribute.
		 //	description:
		 //		Method to add an reference map entry for an item and attribute. 		 //
		 //	refItem:
		 //		The item that is referenced.
		 //	parentItem:
		 //		The item that holds the new reference to refItem.
		 //	attribute:
		 //		The attribute on parentItem that contains the new reference.

		 //Stub function, does nothing.  Real processing is in ItemFileWriteStore.
	},

	getIdentity: function(/* item */ item){
		//	summary:
		//		See dojo.data.api.Identity.getIdentity()
		var identifier = this._features['dojo.data.api.Identity'];
		if(identifier === Number){
			return item[this._itemNumPropName]; // Number
		}else{
			var arrayOfValues = item[identifier];
			if(arrayOfValues){
				return arrayOfValues[0]; // Object || String
			}
		}
		return null; // null
	},

	fetchItemByIdentity: function(/* Object */ keywordArgs){
		//	summary:
		//		See dojo.data.api.Identity.fetchItemByIdentity()

		// Hasn't loaded yet, we have to trigger the load.
		var item,
		    scope;
		if(!this._loadFinished){
			var self = this;
			//Do a check on the JsonFileUrl and crosscheck it.
			//If it doesn't match the cross-check, it needs to be updated
			//This allows for either url or _jsonFileUrl to he changed to
			//reset the store load location.  Done this way for backwards
			//compatibility.  People use _jsonFileUrl (even though officially
			//private.
			if(this._jsonFileUrl !== this._ccUrl){
				kernel.deprecated("dojo.data.ItemFileReadStore: ",
					"To change the url, set the url property of the store," +
					" not _jsonFileUrl.  _jsonFileUrl support will be removed in 2.0");
				this._ccUrl = this._jsonFileUrl;
				this.url = this._jsonFileUrl;
			}else if(this.url !== this._ccUrl){
				this._jsonFileUrl = this.url;
				this._ccUrl = this.url;
			}

			//See if there was any forced reset of data.
			if(this.data != null && this._jsonData == null){
				this._jsonData = this.data;
				this.data = null;
			}

			if(this._jsonFileUrl){

				if(this._loadInProgress){
					this._queuedFetches.push({args: keywordArgs});
				}else{
					this._loadInProgress = true;
					var getArgs = {
							url: self._jsonFileUrl,
							handleAs: "json-comment-optional",
							preventCache: this.urlPreventCache,
							failOk: this.failOk
					};
					var getHandler = xhr.get(getArgs);
					getHandler.addCallback(function(data){
						var scope = keywordArgs.scope?keywordArgs.scope:window.global;
						try{
							self._getItemsFromLoadedData(data);
							self._loadFinished = true;
							self._loadInProgress = false;
							item = self._getItemByIdentity(keywordArgs.identity);
							if(keywordArgs.onItem){
								keywordArgs.onItem.call(scope, item);
							}
							self._handleQueuedFetches();
						}catch(error){
							self._loadInProgress = false;
							if(keywordArgs.onError){
								keywordArgs.onError.call(scope, error);
							}
						}
					});
					getHandler.addErrback(function(error){
						self._loadInProgress = false;
						if(keywordArgs.onError){
							var scope = keywordArgs.scope?keywordArgs.scope:window.global;
							keywordArgs.onError.call(scope, error);
						}
					});
				}

			}else if(this._jsonData){
				// Passed in data, no need to xhr.
				self._getItemsFromLoadedData(self._jsonData);
				self._jsonData = null;
				self._loadFinished = true;
				item = self._getItemByIdentity(keywordArgs.identity);
				if(keywordArgs.onItem){
					scope = keywordArgs.scope?keywordArgs.scope:window.global;
					keywordArgs.onItem.call(scope, item);
				}
			}
		}else{
			// Already loaded.  We can just look it up and call back.
			item = this._getItemByIdentity(keywordArgs.identity);
			if(keywordArgs.onItem){
				scope = keywordArgs.scope?keywordArgs.scope:window.global;
				keywordArgs.onItem.call(scope, item);
			}
		}
	},

	_getItemByIdentity: function(/* Object */ identity){
		//	summary:
		//		Internal function to look an item up by its identity map.
		var item = null;
		if(this._itemsByIdentity){
			// If this map is defined, we need to just try to get it.  If it fails
			// the item does not exist.
			if(Object.hasOwnProperty.call(this._itemsByIdentity, identity)){
				item = this._itemsByIdentity[identity];
			}
		}else if (Object.hasOwnProperty.call(this._arrayOfAllItems, identity)){
			item = this._arrayOfAllItems[identity];
		}
		if(item === undefined){
			item = null;
		}
		return item; // Object
	},

	getIdentityAttributes: function(/* item */ item){
		//	summary:
		//		See dojo.data.api.Identity.getIdentityAttributes()

		var identifier = this._features['dojo.data.api.Identity'];
		if(identifier === Number){
			// If (identifier === Number) it means getIdentity() just returns
			// an integer item-number for each item.  The dojo.data.api.Identity
			// spec says we need to return null if the identity is not composed
			// of attributes
			return null; // null
		}else{
			return [identifier]; // Array
		}
	},

	_forceLoad: function(){
		//	summary:
		//		Internal function to force a load of the store if it hasn't occurred yet.  This is required
		//		for specific functions to work properly.
		var self = this;
		//Do a check on the JsonFileUrl and crosscheck it.
		//If it doesn't match the cross-check, it needs to be updated
		//This allows for either url or _jsonFileUrl to he changed to
		//reset the store load location.  Done this way for backwards
		//compatibility.  People use _jsonFileUrl (even though officially
		//private.
		if(this._jsonFileUrl !== this._ccUrl){
			kernel.deprecated("dojo.data.ItemFileReadStore: ",
				"To change the url, set the url property of the store," +
				" not _jsonFileUrl.  _jsonFileUrl support will be removed in 2.0");
			this._ccUrl = this._jsonFileUrl;
			this.url = this._jsonFileUrl;
		}else if(this.url !== this._ccUrl){
			this._jsonFileUrl = this.url;
			this._ccUrl = this.url;
		}

		//See if there was any forced reset of data.
		if(this.data != null){
			this._jsonData = this.data;
			this.data = null;
		}

		if(this._jsonFileUrl){
				var getArgs = {
					url: this._jsonFileUrl,
					handleAs: "json-comment-optional",
					preventCache: this.urlPreventCache,
					failOk: this.failOk,
					sync: true
				};
			var getHandler = xhr.get(getArgs);
			getHandler.addCallback(function(data){
				try{
					//Check to be sure there wasn't another load going on concurrently
					//So we don't clobber data that comes in on it.  If there is a load going on
					//then do not save this data.  It will potentially clobber current data.
					//We mainly wanted to sync/wait here.
					//TODO:  Revisit the loading scheme of this store to improve multi-initial
					//request handling.
					if(self._loadInProgress !== true && !self._loadFinished){
						self._getItemsFromLoadedData(data);
						self._loadFinished = true;
					}else if(self._loadInProgress){
						//Okay, we hit an error state we can't recover from.  A forced load occurred
						//while an async load was occurring.  Since we cannot block at this point, the best
						//that can be managed is to throw an error.
						throw new Error("dojo.data.ItemFileReadStore:  Unable to perform a synchronous load, an async load is in progress.");
					}
				}catch(e){
					console.log(e);
					throw e;
				}
			});
			getHandler.addErrback(function(error){
				throw error;
			});
		}else if(this._jsonData){
			self._getItemsFromLoadedData(self._jsonData);
			self._jsonData = null;
			self._loadFinished = true;
		}
	}
});
//Mix in the simple fetch implementation to this class.
lang.extend(ItemFileReadStore,simpleFetch);

return ItemFileReadStore;
});

},
'ibm/usmi/widgets/FrameworkMessage':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/widgets/FrameworkMessage", [
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dojo/_base/lang",
    "dojo/_base/sniff", // has("ie")
	"dojo/topic",
	"dojo/dom-class",
	"dojo/dom-attr",
	"dojo/dom-construct",
	"dijit/_WidgetBase",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	// IDX imports
	"idx/oneui/messaging/SingleMessage",
	// NextGen imports 
	"ibm/usmi/core/utils/Logger"
], function (dojo, declare, _lang, has, topic, domClass, domAttr, domconstruct, _WidgetBase, _TemplatedMixin,
		_WidgetsInTemplateMixin, SingleMessage, Logger) {

	return declare("ibm.usmi.widgets.FrameworkMessage", [
		_WidgetBase,
		_TemplatedMixin,
		_WidgetsInTemplateMixin,
		SingleMessage
	], {

		dateFormat : {
			datePattern : 'dd MMMM y, ',
			timePattern : 'hh:mm a'
		},

		style : "width: 98%;",

		showViewDetails : true,

		// external launch url (not Launch-in-Context)
		moreDetailsURL : null,

		// customize the "More Details" label (e.g. "View Job Properties")
		alternateMoreDetailsLabel : null,

		// Create a launch-in-context url w/ page id
		internalLaunch_pageId : null,

		// Create a launch-in-context url w/ targets
		internalLaunch_targets : null,

		// Launch a full URL in context (rather than creating a URL with pageId and targets)
		internalLaunch_url : null,

		/**
		 * More details isn't enough? Add another launch link then override
		 * onCustomLaunch() to do whatever you want Set this string for an
		 * additional launch point. Note: you really should use more details
		 * first.
		 */
		customLaunchLabel : null,

		postCreate : function () {
			this.inherited(arguments);
			if (this.alternateMoreDetailsLabel != null) {
				domAttr.set(this.viewDetailsNode, "innerHTML", this.alternateMoreDetailsLabel);
			}
			this._setShowActionAttr(false);
			this._setShowRefreshAttr(false);
		},

		buildRendering : function () {
			this.inherited(arguments);

			if (this.customLaunchLabel != null && this.bodyNode != null) {
				//this._injectTemplate(this.bodyNode, '<button id="testButton" dojoType="dijit.form.Button" dojoAttachPoint="testButton" dojoAttachEvent="onClick:onCustomLaunch()" name="test">Test</button>');
				this._injectTemplate(this.bodyNode,
				//'<span><div id="sep" class="idxMessageSeparator" dojoAttachPoint="sep">|</div></span>' + 
				'<a class="idxMessageViewDetails" href="javascript: void(0);"'
						+ ' target="_blank" dojoAttachPoint="testNode"'
						+ ' dojoAttachEvent="ondijitclick:onCustomLaunch" name="test">Test</a>');

				domAttr.set(this.testNode, "innerHTML", this.customLaunchLabel);

			}
		},

		/**
		 * Construct UI from a template, injecting the resulting DOM items as
		 * children on of the supplied container node.
		 * 
		 * @param {Object}
		 *            containerNode
		 * @param {Object}
		 *            templateString
		 */
		_injectTemplate : function (containerNode, templateString) {

			// this code is generalised from _Templated.buildRendering

			// Look up cached version of template, or download to cache.
			var cached = _TemplatedMixin.getCachedTemplate(templateString, true);

			var node;
			if (_lang.isString(cached)) {
				// if the cache returned a string, it contains replacement parameters,
				// so replace them and create DOM
				node = domconstruct.toDom(this._stringRepl(cached));
			} else {
				// if the cache returned a node, all we have to do is clone it
				node = cached.cloneNode(true);
			}

			// recurse through the node, looking for, and attaching to, our
			// attachment points and events, which should be defined on the template node.
			this._attachTemplateNodes(node, function (n, p) {
				return n.getAttribute(p);
			});

			// append resolved template as child of container
			containerNode.appendChild(node);
		},

		onClick : function (e) {
			this.inherited(arguments);
			this._refreshMainArea();
		},
		onClose : function () {
			this.inherited(arguments);
			this._refreshMainArea();
		},
		_refreshMainArea : function () {
			var container = dijit.byId("mainContentAreaLayout");
			if (container) {
				container.resize();
			}
		},

		onMoreDetails : function (e) {
			this.inherited(arguments);
			//if (this.operatorResponse != null) {
			//    this.showMoreDetailsDialog(this.operatorResponse);
			if (this.moreDetailsURL != null) {
				this._launchMoreDetails(this.moreDetailsURL);
			} else if (this.internalLaunch_pageId != null || this.internalLaunch_url != null) {
				launchInContext(this.internalLaunch_pageId, this.internalLaunch_targets, this.internalLaunch_url);
			}
	
			// eat the event, otherwise IE tries to reload the page, which fails
			// it onunloads the page but never onloads it
			if(has("ie")){
				e.preventDefault(); 
			}
		},

		destroy : function () {
			this.inherited(arguments);
			this._refreshMainArea();
		},

		/**
		 * Called by the customLaunch link. Override to provide funcion.
		 */
		onCustomLaunch : function (e) {
			topic.publish("message_CustomLaunch", {
				msgId : this.messageId,
				customLaunchLabel : this.customLaunchLabel
			});
		},

		/**
		 * Launch an external URL into a new browser window. Note: Use
		 * launchInContext() to launch internal URL's
		 */
		_launchMoreDetails : function (url) {
			window.open(url, "_blank", null);
		},

		/**
		 * Eventually this will be replaced by something similar in
		 * idx.oneui.messaging.SingleMessage. We are waiting on a fix pack from
		 * them, but I wanted to enable this funciton now.
		 */
		_setShowViewDetailsAttr : function (/*Boolean*/value) {
			this._set("showViewDetails", value);
			domClass.toggle(this.viewDetailsNode, "dijitHidden", !this.showViewDetails);
		}
	});

	/*
	 * Message type. Can be 'error', 'warning', 'success', 'information', 'critical', 'attention', 'compliance'
	 */
	ibm.usmi.widgets.FrameworkMessage.ERROR = "error";
	ibm.usmi.widgets.FrameworkMessage.WARNING = "warning";
	ibm.usmi.widgets.FrameworkMessage.INFO = "information";
	ibm.usmi.widgets.FrameworkMessage.SUCCESS = "success";
	ibm.usmi.widgets.FrameworkMessage.CRITICAL = "critical";
	ibm.usmi.widgets.FrameworkMessage.ATTENTION = "attention";
	ibm.usmi.widgets.FrameworkMessage.COMPLIANCE = "compliance";

});

},
'url:dijit/templates/TimePicker.html':"<div id=\"widget_${id}\" class=\"dijitMenu\"\n    ><div data-dojo-attach-point=\"upArrow\" class=\"dijitButtonNode dijitUpArrowButton\" data-dojo-attach-event=\"onmouseenter:_buttonMouse,onmouseleave:_buttonMouse\"\n\t\t><div class=\"dijitReset dijitInline dijitArrowButtonInner\" role=\"presentation\">&#160;</div\n\t\t><div class=\"dijitArrowButtonChar\">&#9650;</div></div\n    ><div data-dojo-attach-point=\"timeMenu,focusNode\" data-dojo-attach-event=\"onclick:_onOptionSelected,onmouseover,onmouseout\"></div\n    ><div data-dojo-attach-point=\"downArrow\" class=\"dijitButtonNode dijitDownArrowButton\" data-dojo-attach-event=\"onmouseenter:_buttonMouse,onmouseleave:_buttonMouse\"\n\t\t><div class=\"dijitReset dijitInline dijitArrowButtonInner\" role=\"presentation\">&#160;</div\n\t\t><div class=\"dijitArrowButtonChar\">&#9660;</div></div\n></div>\n",
'dojo/dnd/Selector':function(){
define(["../main", "./common", "./Container"], function(dojo) {
	// module:
	//		dojo/dnd/Selector
	// summary:
	//		TODOC


/*
	Container item states:
		""			- an item is not selected
		"Selected"	- an item is selected
		"Anchor"	- an item is selected, and is an anchor for a "shift" selection
*/

/*=====
dojo.declare("dojo.dnd.__SelectorArgs", [dojo.dnd.__ContainerArgs], {
	//	singular: Boolean
	//		allows selection of only one element, if true
	singular: false,

	//	autoSync: Boolean
	//		autosynchronizes the source with its list of DnD nodes,
	autoSync: false
});
=====*/

dojo.declare("dojo.dnd.Selector", dojo.dnd.Container, {
	// summary:
	//		a Selector object, which knows how to select its children

	/*=====
	// selection: Set<String>
	//		The set of id's that are currently selected, such that this.selection[id] == 1
	//		if the node w/that id is selected.  Can iterate over selected node's id's like:
	//	|		for(var id in this.selection)
	selection: {},
	=====*/

	constructor: function(node, params){
		// summary:
		//		constructor of the Selector
		// node: Node||String
		//		node or node's id to build the selector on
		// params: dojo.dnd.__SelectorArgs?
		//		a dictionary of parameters
		if(!params){ params = {}; }
		this.singular = params.singular;
		this.autoSync = params.autoSync;
		// class-specific variables
		this.selection = {};
		this.anchor = null;
		this.simpleSelection = false;
		// set up events
		this.events.push(
			dojo.connect(this.node, "onmousedown", this, "onMouseDown"),
			dojo.connect(this.node, "onmouseup",   this, "onMouseUp"));
	},

	// object attributes (for markup)
	singular: false,	// is singular property

	// methods
	getSelectedNodes: function(){
		// summary:
		//		returns a list (an array) of selected nodes
		var t = new dojo.NodeList();
		var e = dojo.dnd._empty;
		for(var i in this.selection){
			if(i in e){ continue; }
			t.push(dojo.byId(i));
		}
		return t;	// NodeList
	},
	selectNone: function(){
		// summary:
		//		unselects all items
		return this._removeSelection()._removeAnchor();	// self
	},
	selectAll: function(){
		// summary:
		//		selects all items
		this.forInItems(function(data, id){
			this._addItemClass(dojo.byId(id), "Selected");
			this.selection[id] = 1;
		}, this);
		return this._removeAnchor();	// self
	},
	deleteSelectedNodes: function(){
		// summary:
		//		deletes all selected items
		var e = dojo.dnd._empty;
		for(var i in this.selection){
			if(i in e){ continue; }
			var n = dojo.byId(i);
			this.delItem(i);
			dojo.destroy(n);
		}
		this.anchor = null;
		this.selection = {};
		return this;	// self
	},
	forInSelectedItems: function(/*Function*/ f, /*Object?*/ o){
		// summary:
		//		iterates over selected items;
		//		see `dojo.dnd.Container.forInItems()` for details
		o = o || dojo.global;
		var s = this.selection, e = dojo.dnd._empty;
		for(var i in s){
			if(i in e){ continue; }
			f.call(o, this.getItem(i), i, this);
		}
	},
	sync: function(){
		// summary:
		//		sync up the node list with the data map

		dojo.dnd.Selector.superclass.sync.call(this);

		// fix the anchor
		if(this.anchor){
			if(!this.getItem(this.anchor.id)){
				this.anchor = null;
			}
		}

		// fix the selection
		var t = [], e = dojo.dnd._empty;
		for(var i in this.selection){
			if(i in e){ continue; }
			if(!this.getItem(i)){
				t.push(i);
			}
		}
		dojo.forEach(t, function(i){
			delete this.selection[i];
		}, this);

		return this;	// self
	},
	insertNodes: function(addSelected, data, before, anchor){
		// summary:
		//		inserts new data items (see `dojo.dnd.Container.insertNodes()` method for details)
		// addSelected: Boolean
		//		all new nodes will be added to selected items, if true, no selection change otherwise
		// data: Array
		//		a list of data items, which should be processed by the creator function
		// before: Boolean
		//		insert before the anchor, if true, and after the anchor otherwise
		// anchor: Node
		//		the anchor node to be used as a point of insertion
		var oldCreator = this._normalizedCreator;
		this._normalizedCreator = function(item, hint){
			var t = oldCreator.call(this, item, hint);
			if(addSelected){
				if(!this.anchor){
					this.anchor = t.node;
					this._removeItemClass(t.node, "Selected");
					this._addItemClass(this.anchor, "Anchor");
				}else if(this.anchor != t.node){
					this._removeItemClass(t.node, "Anchor");
					this._addItemClass(t.node, "Selected");
				}
				this.selection[t.node.id] = 1;
			}else{
				this._removeItemClass(t.node, "Selected");
				this._removeItemClass(t.node, "Anchor");
			}
			return t;
		};
		dojo.dnd.Selector.superclass.insertNodes.call(this, data, before, anchor);
		this._normalizedCreator = oldCreator;
		return this;	// self
	},
	destroy: function(){
		// summary:
		//		prepares the object to be garbage-collected
		dojo.dnd.Selector.superclass.destroy.call(this);
		this.selection = this.anchor = null;
	},

	// mouse events
	onMouseDown: function(e){
		// summary:
		//		event processor for onmousedown
		// e: Event
		//		mouse event
		if(this.autoSync){ this.sync(); }
		if(!this.current){ return; }
		if(!this.singular && !dojo.isCopyKey(e) && !e.shiftKey && (this.current.id in this.selection)){
			this.simpleSelection = true;
			if(e.button === dojo.mouseButtons.LEFT){
				// accept the left button and stop the event
				// for IE we don't stop event when multiple buttons are pressed
				dojo.stopEvent(e);
			}
			return;
		}
		if(!this.singular && e.shiftKey){
			if(!dojo.isCopyKey(e)){
				this._removeSelection();
			}
			var c = this.getAllNodes();
			if(c.length){
				if(!this.anchor){
					this.anchor = c[0];
					this._addItemClass(this.anchor, "Anchor");
				}
				this.selection[this.anchor.id] = 1;
				if(this.anchor != this.current){
					var i = 0;
					for(; i < c.length; ++i){
						var node = c[i];
						if(node == this.anchor || node == this.current){ break; }
					}
					for(++i; i < c.length; ++i){
						var node = c[i];
						if(node == this.anchor || node == this.current){ break; }
						this._addItemClass(node, "Selected");
						this.selection[node.id] = 1;
					}
					this._addItemClass(this.current, "Selected");
					this.selection[this.current.id] = 1;
				}
			}
		}else{
			if(this.singular){
				if(this.anchor == this.current){
					if(dojo.isCopyKey(e)){
						this.selectNone();
					}
				}else{
					this.selectNone();
					this.anchor = this.current;
					this._addItemClass(this.anchor, "Anchor");
					this.selection[this.current.id] = 1;
				}
			}else{
				if(dojo.isCopyKey(e)){
					if(this.anchor == this.current){
						delete this.selection[this.anchor.id];
						this._removeAnchor();
					}else{
						if(this.current.id in this.selection){
							this._removeItemClass(this.current, "Selected");
							delete this.selection[this.current.id];
						}else{
							if(this.anchor){
								this._removeItemClass(this.anchor, "Anchor");
								this._addItemClass(this.anchor, "Selected");
							}
							this.anchor = this.current;
							this._addItemClass(this.current, "Anchor");
							this.selection[this.current.id] = 1;
						}
					}
				}else{
					if(!(this.current.id in this.selection)){
						this.selectNone();
						this.anchor = this.current;
						this._addItemClass(this.current, "Anchor");
						this.selection[this.current.id] = 1;
					}
				}
			}
		}
		dojo.stopEvent(e);
	},
	onMouseUp: function(e){
		// summary:
		//		event processor for onmouseup
		// e: Event
		//		mouse event
		if(!this.simpleSelection){ return; }
		this.simpleSelection = false;
		this.selectNone();
		if(this.current){
			this.anchor = this.current;
			this._addItemClass(this.anchor, "Anchor");
			this.selection[this.current.id] = 1;
		}
	},
	onMouseMove: function(e){
		// summary:
		//		event processor for onmousemove
		// e: Event
		//		mouse event
		this.simpleSelection = false;
	},

	// utilities
	onOverEvent: function(){
		// summary:
		//		this function is called once, when mouse is over our container
		this.onmousemoveEvent = dojo.connect(this.node, "onmousemove", this, "onMouseMove");
	},
	onOutEvent: function(){
		// summary:
		//		this function is called once, when mouse is out of our container
		dojo.disconnect(this.onmousemoveEvent);
		delete this.onmousemoveEvent;
	},
	_removeSelection: function(){
		// summary:
		//		unselects all items
		var e = dojo.dnd._empty;
		for(var i in this.selection){
			if(i in e){ continue; }
			var node = dojo.byId(i);
			if(node){ this._removeItemClass(node, "Selected"); }
		}
		this.selection = {};
		return this;	// self
	},
	_removeAnchor: function(){
		if(this.anchor){
			this._removeItemClass(this.anchor, "Anchor");
			this.anchor = null;
		}
		return this;	// self
	}
});

return dojo.dnd.Selector;
});

},
'ibm/usmi/widgets/util/PropertyRenderer':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

define(["dojo/_base/kernel",
        "dojo/_base/declare",
        "dijit/ProgressBar",
        //TWL imports
        "ibm/tivoli/navmodel/Property",
        "ibm/tivoli/navmodel/Status",
        "ibm/tivoli/navmodel/ItemStyle",
        "ibm/tivoli/navmodel/Item",
        "ibm/tivoli/navmodel/Link",
        "ibm/tivoli/navmodel/Relationship",
        "ibm/tivoli/tip/navwidget/dojo/NavWidgetUtil",
        "ibm/tivoli/tip/navwidget/properties/PropertyRenderer",
        // Next Gen
        "ibm/usmi/widgets/util/Constants"],

    function (dojo, declare, ProgressBar,
            Property, Status, ItemStyle, Item, Link, Relationship, NavWidgetUtil, PropertyRenderer,
            Constants) {

    return declare([PropertyRenderer], {

        //***************************
        // "Public Methods"
        //***************************

        constructor: function() {
            // summary:
            //        Coming soon
            // description:
            //        Coming soon
            //Call superclass
            this.inherited(arguments);
        },

        getRenderedPropertyNode: function(/*ibm.tivoli.navmodel.Item*/ object, prop, plugviewref, rowID, cell) {
            //first check if there is a valid value
            switch(prop.getValueState()){
            case "ok":

                    switch (prop.getValueType()) {
                    case "itemName":

                        var contentNode = dojo.doc.createElement("div"),
                            threshold, altText;

                        //a threshold callback is defined, so use it to check if we need
                        //to threshold this value and ADD an image which we normally wouldnt render
                        if(this.propThresholdCallback){
                            threshold = this.propThresholdCallback(prop, object);

                            //only if an image is defined on the threshold for this value
                            if(threshold && threshold.icon){
                                altText = (threshold.label ? threshold.label.value : null);
                                if (!altText) {
                                    altText = prop.getDisplayValue();
                                }

                                //render the node, and add it
                                dojo.place(this._getImageContentNode( plugviewref, prop, object, "", "16px", "16px"), contentNode, "last", altText);
                            }
                        }

                        if(prop.getValueUnits()) {
                            dojo.place(this._getLabelContentNode(prop.getDisplayValue() + " " + prop.getValueUnits(), prop, object, plugviewref ), contentNode,"last");
                        } else {
                            dojo.place(this._getLabelContentNode(prop.getDisplayValue(), prop, object, plugviewref  ), contentNode,"last");
                        }

                        //color background if thresholding needed
                        this._thresholdBackground(cell, prop, object);

                        return contentNode;

                        default:
                    //probably need to add some error handling in here in case the input value is invalid
                    }
                    break;
            default:

            }

            return this.inherited(arguments);
        },

        getResourceImage: function(plugviewref, prop, object, imagePath, height, width, altText){
            //create the image node
            var fullImageNode,
                linkNode,
                imageNode= dojo.doc.createElement("img"),
                threshold,
                objType;

            //a threshold callback is defined,
            if(this.propThresholdCallback){
            //use it to check if we need to see if we need to threshold this image
                threshold = this.propThresholdCallback(prop, object);
                if(threshold && threshold.icon){
                    //threshold condition met, so use the threshold image instead
                    //dojo.attr(imageNode, "src", plugviewref._cleanUpImagePath(threshold.icon));
                    imagePath = plugviewref._cleanUpImagePath(threshold.icon);
                }
            }


            //add the attributes to the img node based on the object style
            dojo.attr(imageNode, "src", plugviewref._cleanUpImagePath(imagePath));
            dojo.attr(imageNode, "height", height);
            dojo.attr(imageNode, "width", width);
            dojo.attr(imageNode, "style", "vertical-align:sub");
            dojo.attr(imageNode, "hspace", "3px");
            dojo.addClass(imageNode, "tctGridCellImg"); //253021
            if (altText) {
                dojo.attr(imageNode, "alt", altText);
            }

            if(prop.getTask()){
                linkNode = dojo.doc.createElement("a");

                dojo.attr(linkNode, Constants.ON_CLICK, "ibm.tivoli.tip.navwidget.properties.handleLinkClick(this);return false;" );
                dojo.attr(linkNode, "id", object.getId() + "_"+ prop.getId()+".img.link");
                dojo.attr(linkNode, "propid", prop.getId());
                dojo.attr(linkNode, "objectid", object.getId());
                dojo.attr(linkNode, "viewid", plugviewref.id);
                //check to see which type of object this is and set on ojbecttype attr for link
                objType = null;
                if(object instanceof Item){
                    objType = "ibm.tivoli.navmodel.Item";
                }
                else if(object instanceof Link){
                    objType = "ibm.tivoli.navmodel.Link";
                }
                else if(object instanceof Relationship){
                    objType = "ibm.tivoli.navmodel.Relationship";
                }
                dojo.attr(linkNode, "objecttype", objType);


                dojo.attr(linkNode, "href", "#");

                dojo.place(imageNode, linkNode, "last");

                fullImageNode = linkNode;

            }else{
                fullImageNode = imageNode;
            }


            return fullImageNode;

        }
    });
});

},
'ibm/tivoli/navmodel/Exception':function(){
//>>built
// wrapped by build app
define("ibm/tivoli/navmodel/Exception", ["dijit","dojo","dojox"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Exception"); 
 
////////////////////////////////////
// Exception
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Exception", null, {
	// private members
	_version: null,
	_exception: null,
	
	/*=====
	constructor_kwArgs: function(version, exception) {
		// version: Float
		//		version used to communicate with rest service
		// exception: Object
		//		json exception object
	},
	=====*/

	 constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary
		//		Constructs Exception object.
		this._version = kwArgs.version;
		this._exception = kwArgs.exception;		
	},

	getMsgId: function() {
		return this._exception.msgId;
	},
	
	getMsgSeverity: function() {
		return this._exception.msgSeverity;
	},
	
	getMsgText: function() {
		return this._exception.msgText;
	},

	getMsgDescription: function() {
		return this._exception.msgDescription;
	},

	getMoreDetails: function() {
		return this._exception.moreDetails;
	}

});

});

},
'url:ibm/usmi/widgets/templates/TaskCategory.html':"<div tabindex=\"${tabIndex}\" class=\"taskCategory\" data-dojo-attach-point=\"mainNode\">\r\n<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n\t<div class=\"taskCategoryBar\">\r\n\t   <div class=\"taskCategoryTitle\" data-dojo-attach-point=\"titlePoint\" role=\"heading\" aria-level=2 ></div>\r\n\t</div>\r\n<!--\t<div class=\"taskCategory-shadow\"></div> -->\r\n\t<div data-dojo-attach-point=\"containerNode\"></div>\r\n</div>",
'ibm/usmi/scheduler/wizard_pages/NotificationsPage':function(){
require({cache:{
'url:ibm/usmi/scheduler/wizard_pages/templates/NotificationsPage.html':"<div>\r\n<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n<!-- \r\n\t<div class=\"tiv-page-header\">\r\n\t\t<h2>Notifications</h2>\r\n\t\t<p>Receive an e-mail notification with the progress of this job.</p>\r\n\t</div>\r\n\t-->\r\n\t<label for=\"ReceiveNotifications\">${persistResources.RECEIVE_NOTIFICATIONS_LABEL}</label>\r\n\t<div id=\"ReceiveNotifications\">\r\n\t\t<input type=\"checkbox\" dojoAttachPoint=\"whenJobBegins\" data-dojo-type=\"dijit.form.CheckBox\" iconclass=\"dijitNoIcon\"\r\n\t\t\tdojoAttachEvent=\"onChange:doNotificationsPageRendering\" />${persistResources.JOB_BEGINS_VALUE}<br /> \r\n\t\t<input type=\"checkbox\" dojoAttachPoint=\"whenJobEndsSuccessfully\" data-dojo-type=\"dijit.form.CheckBox\" iconclass=\"dijitNoIcon\"\r\n\t\t\tdojoAttachEvent=\"onChange:doNotificationsPageRendering\" />${persistResources.JOB_COMPLETES_OK_VALUE}<br /> \r\n\t\t<input type=\"checkbox\" dojoAttachPoint=\"whenJobFails\" data-dojo-type=\"dijit.form.CheckBox\" iconclass=\"dijitNoIcon\"\r\n\t\t\tdojoAttachEvent=\"onChange:doNotificationsPageRendering\" />${persistResources.JOB_FAILS_VALUE}<br />\r\n\r\n\t\t<div dojoAttachPoint=\"errorSelectionStuff\" style=\"display: none; margin-left: 15px\">\r\n\t\t\t<input type=\"radio\" name=\"errorNotification\" data-dojo-type=\"dijit.form.RadioButton\"\r\n\t\t\t\tdojoAttachPoint=\"anyErrorRadio\" checked value=\"true\" dojoAttachEvent=\"onChange:doNextButtonCheck\" />${persistResources.JOB_FAILS_ANY_ERROR_RADIO_LABEL} <br />\r\n\t\t\t<input type=\"radio\" name=\"errorNotification\" data-dojo-type=\"dijit.form.RadioButton\"\r\n\t\t\t\tdojoAttachPoint=\"percentFailedRadio\" value=\"true\" dojoAttachEvent=\"onChange:doNextButtonCheck\" />${persistResources.JOB_FAILS_PERCENT_ERROR_RADIO_LABEL}\r\n\t\t\t\t\t<input type=\"text\"\tdojoAttachPoint=\"thresholdPercentFailed\"data-dojo-type=\"dijit.form.NumberSpinner\" \r\n\t\t\t\t\t\tdata-dojo-props=\"constraints:{min:1,max:100,places:0}\" value=\"1\" dojoAttachEvent=\"onChange:doNextButtonCheck\" intermediateChanges=\"true\" /><br />\r\n\t\t\t<input type=\"radio\" name=\"errorNotification\" data-dojo-type=\"dijit.form.RadioButton\"\r\n\t\t\t\tdojoAttachPoint=\"numberFailedRadio\" value=\"true\" dojoAttachEvent=\"onChange:doNextButtonCheck\" />${persistResources.JOB_FAILS_NUMBER_ERROR_RADIO_LABEL}\r\n\t\t\t\t\t<input type=\"text\"\tdojoAttachPoint=\"thresholdNumberFailed\"\tdata-dojo-type=\"dijit.form.NumberSpinner\" \r\n\t\t\t\t\t\tdata-dojo-props=\"constraints:{min:1,places:0}\" value=\"1\" dojoAttachEvent=\"onChange:doNextButtonCheck\" intermediateChanges=\"true\" /><br /> \r\n\t\t\t<br />\r\n\t\t</div>\r\n\t</div>\r\n\r\n\t<div dojoAttachPoint=\"emailNotificationStuff\" style=\"display: none;\">\r\n\t\t<label for=\"emailInformation\">${persistResources.EMAIL_INFORMATION_LABEL}</label>\r\n\t\t<div id=\"emailInformation\">\r\n\t\t\t<label for=\"notificationEmailAddress\" class=\"required\">${persistResources.EMAIL_ADDRESS_LABEL}</label> \r\n\t\t\t\t<input dojoAttachPoint=\"notificationEmailAddress\" type=\"text\" name=\"widget_emailAddr\" data-dojo-type=\"dijit.form.ValidationTextBox\" style=\"width: 60%;\"\r\n\t\t\t\t\trequired=\"true\" dojoAttachEvent=\"onChange:doNextButtonCheck\" intermediateChanges=\"true\" /> \r\n\t\t\t\t<br />\r\n\t\t\t<label for=\"notificationEmailServerName\" class=\"required\">${persistResources.EMAIL_SERVER_LABEL}</label> \r\n\t\t\t\t<input dojoAttachPoint=\"notificationEmailServerName\" type=\"text\" name=\"widget_emailServerName\" data-dojo-type=\"dijit.form.ValidationTextBox\" style=\"width: 60%;\"\r\n\t\t\t\t\trequired=\"true\" dojoAttachEvent=\"onChange:doNextButtonCheck\" intermediateChanges=\"true\" /> \r\n\t\t\t\t<br />\t\t\t\t\t\r\n\t\t\t<label for=\"notificationEmailServerPort\" class=\"required\">${persistResources.EMAIL_PORT_LABEL}</label> \r\n\t\t\t\t<input dojoAttachPoint=\"notificationEmailServerPort\" type=\"text\" name=\"widget_emailServerPort\" data-dojo-type=\"dijit.form.ValidationTextBox\" style=\"width: 60%;\"\r\n\t\t\t\trequired=\"true\" dojoAttachEvent=\"onChange:doNextButtonCheck\" intermediateChanges=\"true\" />\r\n\t\t</div>\r\n\t</div>\r\n</div>"}});
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
/*global define */
define("ibm/usmi/scheduler/wizard_pages/NotificationsPage", [ "dojo/_base/declare",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	"dijit/form/CheckBox",
	"dijit/form/RadioButton",
	"dijit/form/NumberSpinner",
	"dijit/form/ValidationTextBox",
	"ibm/tivoli/tip/navwidget/dijit/DefaultMessageHandler",
	"dojoe/wizard/Page",
	"dojo/text!./templates/NotificationsPage.html",
	// The resource bundle definition
	"dojo/i18n!./nls/JobLaunchWizardResources" ], function (
	declare, _TemplatedMixin, _WidgetsInTemplateMixin, CheckBox, RadioButton, NumberSpinner, ValidationTextBox,
	DefaultMessageHandler, Page, NotificationsPage, mri) {

	return declare([ Page,
		_TemplatedMixin,
		_WidgetsInTemplateMixin ], {

		templateString : NotificationsPage,
		persistResources : mri,

		title : "",

		postCreate : function () {
			// set the title of the page
			this.title = this.persistResources.NOTIFICATIONS_PAGE_TITLE;
		},

		allowNext : function () {
			return this.isValid();
		},

		allowFinish : function () {
			return this.isValid();
		},

		isValid : function () {
			return this.validate();
		},

		validate : function () {
			// TODO:  Need to figure out how to do validation of the fields at the right time....(when the user tries to leave the page)....right now, it's being
			//        done when the user starts to type values in the email fields (so, if we want to display an error message, it comes up several times!)

			var msgHandler = new DefaultMessageHandler();
			var msgId = ""; // not specifying a message ID
			var msgSeverity = "error";
			var theMessageTitle = "";

			// consider something like this:  This page is valid unless any of the check boxes are 
			// selected -- at which point we need to make sure there is email information...	
			var jobBeginsSelection = this.whenJobBegins.get('value');
			var jobEndsSelection = this.whenJobEndsSuccessfully.get('value');
			var jobFailsSelection = this.whenJobFails.get('value');

			if ((jobBeginsSelection === "on") || (jobEndsSelection === "on") || (jobFailsSelection === "on")) {
				// check to see if the user has entered email information....Retrieve the values
				var emailAddr = this.notificationEmailAddress.get('value');
				var serverAddr = this.notificationEmailServerName.get('value');
				var serverPort = this.notificationEmailServerPort.get('value');

				// All 3 fields must have a value....if not, disable the next button....
				if ((emailAddr === "") || (serverAddr === "") || (serverPort === "")) {
					theMessageTitle = this.persistResources.MISSING_EMAIL_INFORMATION;
					msgHandler.displayMessage(msgId, msgSeverity, theMessageTitle);
					return false;
				} else {// there's some value
					// now make sure that if 'when job fails' checkbox is selected....AND 
					if (this.whenJobFails.get('value')) {
						// if the user selected "percentage" or "number of targets" -- that the values are greater than 0:
						if (this.percentFailedRadio.get('value')) {
							var percentValue = this.thresholdPercentFailed.getValue('value');
							if ((!percentValue) || (percentValue <= 0) || (percentValue > 100)) {
								// the user is choosing 0 percent....that doesn't make sense
								theMessageTitle = this.persistResources.PERCENT_FAILED_VALUE_NOT_SET_ERROR;
								msgHandler.displayMessage(msgId, msgSeverity, theMessageTitle);
								return false;
							} else {
								return true;
							}
						}
						if (this.numberFailedRadio.get('value')) {
							var numberValue = this.thresholdNumberFailed.getValue('value');
							if ((!numberValue) || (numberValue <= 0)) {
								// the user is choosing 0 targets ... that doesn't make sense
								theMessageTitle = this.persistResources.NUMBER_FAILED_VALUE_NOT_SET_ERROR;
								msgHandler.displayMessage(msgId, msgSeverity, theMessageTitle);								
								return false;
							} else {
								return true;
							}
						}
					}

					return true;
				}
			} else { // no notifications are indicated....allow next
				return true;
			}
		},

		visited : function () {
			// indicating that this page has already been visited allow our submit button to be enabled initially...
			return true;
		},

		allowMarkAsComplete : function () {
			return true;
		},

		doNotificationsPageRendering : function () {
			var jobBeginsSelection = this.whenJobBegins.get('value');
			var jobEndsSelection = this.whenJobEndsSuccessfully.get('value');
			var jobFailsSelection = this.whenJobFails.get('value');

			// If any check box is checked...show the email stuff...
			if ((jobBeginsSelection === "on") || (jobEndsSelection === "on") || (jobFailsSelection === "on")) {
				// make sure that the email notification information is visible
				this.emailNotificationStuff.style.display = 'block';

				/*
				// check to see if the user has entered email information....Retrieve the values
				var emailAddr = this.notificationEmailAddress.get('value');
				var serverAddr = this.notificationEmailServerName.get('value');
				var serverPort = this.notificationEmailServerPort.get('value');
				*/
				// All 3 fields must have a value....if not, disable the next button....
				/*
				 * if ((emailAddr === "") || (serverAddr === "") || (serverPort === "")) {
					this.wizard.enableNextButton(false);
				} else {
					// all 3 fields have a value...we can enable next button
					this.wizard.enableNextButton(true);
				}
				*/
			} else {
				// None of the notification check boxes is checked, so we can go ahead and hide the email information, and make sure the next button
				// is enabled.
				this.emailNotificationStuff.style.display = 'none';
				this.wizard.enableNextButton(true);
			}

			// if the job fails check box is on, then display the fail block of information.
			if (jobFailsSelection === "on") {
				this.errorSelectionStuff.style.display = 'block';
			} else {
				this.errorSelectionStuff.style.display = 'none';
			}

		},

		doNextButtonCheck : function () {
			// Check if Next button is disabled first
			if (this.wizard._nextButton.attr("disabled") === true) {
				// Get the fields we need to check 

				// Get checkbox values
				var jobBeginsSelection = this.whenJobBegins.get('value');
				var jobEndsSelection = this.whenJobEndsSuccessfully.get('value');
				var jobFailsSelection = this.whenJobFails.get('value');
				var emailAddress = this.notificationEmailAddress.get('value');
				var emailServerName = this.notificationEmailServerName.get('value');
				var emailServerPort = this.notificationEmailServerPort.get('value');

				// Check which checkbox is selected
				if (jobFailsSelection === "on") {
					// Check if email address is set
					if ((emailAddress !== "") && (emailServerName !== "") && (emailServerPort !== "")) {
						// Check if selected radio field is valid
						if (this.percentFailedRadio.get('checked')) {
							var percentValue = this.thresholdPercentFailed.getValue('value');
							if ((percentValue > 0) && (percentValue <= 100)) {
								// Enable Next button
								this.wizard.enableNextButton(true);
							}
						} else {
							if (this.numberFailedRadio.get('checked')) {
								var numberValue = this.thresholdNumberFailed.getValue('value');
								if (numberValue > 0) {
									// Enable Next button
									this.wizard.enableNextButton(true);
								}
							} else { // any error radio must be selected
								// Enable Next button
								this.wizard.enableNextButton(true);
							}
						}
					}
				} else { // one or the other 2 must have been specified
					if ((jobBeginsSelection === "on") || (jobEndsSelection === "on")) {
						// Enable Next button if email fields are valid
						if ((emailAddress !== "") && (emailServerName !== "") && (emailServerPort !== "")) {
							this.wizard.enableNextButton(true);
						}
					}
				}
			}
		},
		
		reset : function () {}
	});
});

},
'dijit/MenuBarItem':function(){
define([
	"dojo/_base/declare", // declare
	"./MenuItem",
	"dojo/text!./templates/MenuBarItem.html"
], function(declare, MenuItem, template){

/*=====
	var MenuItem = dijit.MenuItem;
=====*/

	// module:
	//		dijit/MenuBarItem
	// summary:
	//		Item in a MenuBar that's clickable, and doesn't spawn a submenu when pressed (or hovered)


	var _MenuBarItemMixin = declare("dijit._MenuBarItemMixin", null, {
		templateString: template,

		// Map widget attributes to DOMNode attributes.
		_setIconClassAttr: null	// cancel MenuItem setter because we don't have a place for an icon
	});

	var MenuBarItem = declare("dijit.MenuBarItem", [MenuItem, _MenuBarItemMixin], {
		// summary:
		//		Item in a MenuBar that's clickable, and doesn't spawn a submenu when pressed (or hovered)

	});
	MenuBarItem._MenuBarItemMixin = _MenuBarItemMixin;	// dojox.mobile is accessing this


	return MenuBarItem;
});

},
'dojoe/treetable/filter/FilterLayer':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!dojoe/treetable/filter/_FilterExpr,dojoe/treetable/filter/_StoreLayer"], function(dijit,dojo,dojox){
/**
 * @author shakeel
 */
dojo.provide("dojoe.treetable.filter.FilterLayer");

dojo.require("dojoe.treetable.filter._FilterExpr");
dojo.require("dojoe.treetable.filter._StoreLayer");
(function(){
var ns = dojoe.treetable,
	cmdSetFilter = "filter",
	cmdClearFilter = "clear",
	hitchIfCan = function(scope, func){
		return func ? dojo.hitch(scope || dojo.global, func) : function(){};
	},
	shallowClone = function(obj){
		var res = {};
		if(obj && dojo.isObject(obj)){
			for(var name in obj){
				res[name] = obj[name];
			}
		}
		return res;
	},
	_findLongestSequences = function (/* Integer[] */numArray,/* Integer */startIdx){
		// summary:
		//		Find all the continuous sequences in an ascending number array, 
		//		and sort them from the longest to the shortest.
		// complexity: 
		//		2n + Complexity of sort
		
		var res = [], res_q = [], start = 0, len = 1, i;
		for(i=1;i<numArray.length;++i){
			if(numArray[i] == numArray[i-1]+1){
				++len;
			}else{
				res[start] = len;
				start = i;
				len = 1;
			}
		}
		res[start] = len;
		for(i=1;i<res.length;++i){
			if(res[i] != undefined){
				res_q.push({idx:i+startIdx,cnt:res[i]});
			}
		}
		res_q.sort(function(a,b){
			return b.cnt - a.cnt;
		});
		return res_q;
	};
	dojo.declare("dojoe.treetable.filter._FilterLayerMixin",null,{

		name: function(){
			// summary:
			//		override from _StoreLayer.name
			return "filter";	//string
		},
		onFilterDefined: function(filter){},
		
		onFiltered: function(filteredSize, totalSize){
			// summary:
			//		Called when store data is filtered. This event is before *onComplete*, after *onBegin*.
			// tags:
			//		callback extension
			// filteredSize: Integer
			//		The number of remaining fetched items after filtering.
			// totalSize: Integer
			//		The number of original fetched items.
		}
	});
	dojo.declare("dojoe.treetable.filter.ServerSideFilterLayer",
				[dojoe.treetable.filter._ServerSideLayer, ns.filter._FilterLayerMixin],
	{
		_filterText: null,
		constructor: function(args){
			this._onUserCommandLoad = args.setupFilterQuery || this._onUserCommandLoad;
			this.filterDef(null);
		},
		setFilterText: function(filterText){
			this._filterText = filterText;
		},
		filterDef: function(/* (_ConditionExpr|null)? */filter){
			// summary:
			// See _FilterLayerMixin.filterDef
			if(filter){
				this._filter = filter;
				var obj = filter.toObject();
				this.command(cmdSetFilter, obj); 
				this.onFilterDefined(filter);
			}else if(filter === null){
				this._filter = null;
				this.command(cmdSetFilter, null);
				this.onFilterDefined(null);
			}
			return this._filter; //_ConditionExpr
		},
		onCommandLoad: function(/* (in)string */responce, /* (in|out)keywordArgs */ userRequest){
			// summary:
			//		override from _ServerSideLayer.onCommandLoad
			this.inherited(arguments);
			var oldOnBegin = userRequest.onBegin;
			if(this._isStateful){
			var filteredSize;
			if(responce){
				this.command(cmdSetFilter, null);
				this.command(cmdClearFilter, null);
				this.useCommands(false);
				var sizes = responce.split(',');
				if(sizes.length >= 2){
					filteredSize = this._filteredSize = parseInt(sizes[0]);
					this.onFiltered(filteredSize, parseInt(sizes[1]));
				}else{
					//Error here.
					return;
				}
			}else{
				filteredSize = this._filteredSize;
			}
			if(this.enabled()){
					userRequest.onBegin = function(size, req){
						hitchIfCan(userRequest.scope, oldOnBegin)(filteredSize, req);
					};
				}
			}else{
				var _this = this;
				userRequest.onBegin = function(size, req){
					if(!_this._filter){
						_this._storeSize = size;
					}
					_this.onFiltered(size, _this._storeSize || size);
					req.onBegin = oldOnBegin;
					hitchIfCan(userRequest.scope, oldOnBegin)(size, req);
				};
			}
		},
		
		_translate: function(query, data, op, prevOp) { 
					if (dojo.isArray(data)) { 
					var length = data.length;

					if (length > 1) {
					query.expression += "(";
					}

					for (var index = 0; index < length; ++index) {
					if (!this._translate(query, data[index].data, data[index].op, op)) {
					if (data[index].isCol) {
					query.ids[data[index].data] = true;
					query.expression += data[index].data;
					query.expression += " ";
					query.expression += this._translateOperator(op, prevOp);
					}
					else {
					if (data[index].op == "string") {
					// replace single quotes with 2 consecutive single quotes
					query.expression += " '" + data[index].data.replace(/'/g, "''") + "'";
					}
					else {
					query.expression += " " + data[index].data;
					} 
					}
					}
					// and/or operator in between
					else if (index + 1 < length) {
					query.expression += " " + (op === "any" ? "or" : "and") + " "; 
					}
					}

					if (length > 1) {
					query.expression += ")";
					}

					return true;
					} 
					else {
					return false; 
					}
					},

					_translateOperator: function(op1, op2) {
					var newOp = ""; 
					switch(op1) {
					case "equal":
					newOp = (op2 === "not" ? "!=" : "=");
					break;
					case "less":
					newOp = (op2 === "not" ? ">=" : "<");
					break;
					case "lessEqual":
					newOp = (op2 === "not" ? ">" : "<=");
					break;
					case "larger":
					newOp = (op2 === "not" ? "<=" : ">");
					break;
					case "largerEqual":
					newOp = (op2 === "not" ? "<" : ">=");
					break;
					case "contains":
					case "containsignorecase":
					newOp = (op2 === "not" ? "!contains" : "contains");
					break;
					case "startsWith":
					newOp = (op2 === "not" ? "!starts" : "starts");
					break;
					case "endsWith":
					newOp = (op2 === "not" ? "!ends" : "ends");
					break;
					default:
					alert('unrecognized operator=' + op1);
					}

					return newOp;
		}
	});
	dojo.declare("dojoe.treetable.filter.ClientSideFilterLayer",
				[dojoe.treetable.filter._StoreLayer, ns.filter._FilterLayerMixin],
	{
		// summary:
		//		Add a client side filter layer on top of the data store,
		//		so any filter expression can be applied to the store.

/*=====
		//_items: Array,
		//		Cached items (may contain holes)
		_items: [],
		
		//_result: Array,
		//		Current fetch result
		_result: [],
		
		//_resultStartIdx: Integer,
		//		The index in cache of the first result item
		_resultStartIdx: 0,
		
		//_indexMap: Array,
		//		A map from the row index of this._items to the row index of the original store.
		_indexMap: null,
		
		//_cacheHoles: Array,
		//		Hole objects. e.g. {idx:0,cnt:3} means _items[0],items[1],items[2] are not valid.
		_cacheHoles: [],
		
		//_getter: function(datarow, colArg, rowIndex, store);
		//		A user defined way to get data from store
		_getter: null,
		
		// _cachedItemCount: Integer
		//		The number of valid cached items. Because of holes, _items.length != _cachedItemsCount
		_cachedItemCount: 0,
		
		// _nextUnfetchedIdx: Integer
		//		The index of the next item in the store that is never fetched.
		_nextUnfetchedIdx: 0,
=====*/
		// _storeSize: Integer
		//		The actual size of the original store
		_storeSize: -1,
		
		// _cacheSize: Integer
		//		A cache size limit. Should not be 0, less than 0 means cache all.
		_cacheSize: -1,
		
		// _fetchAll
		//		If the store is small or store size must be correct when onBegin is called,
		//		we should fetch and filter all the items on the first query.
		_fetchAll: true,
		
		//grid: the grid must be passed when the layer is created
		_grid: null,
		
		_filterText: null,
		
		children: null,
		
		constructor: function(args){
			this.filterDef(null);
			args = dojo.isObject(args) ? args : {};
			this.filterCacheSize(args.cacheSize);
			this.fetchAllOnFirstFilter(args.fetchAll);
			this._grid = args.grid;			
			this._getter = dojo.isFunction(args.getter) ? args.getter 
						: function(datarow,colName,rowIndex,store){
							return store.getValue(datarow,colName);
						};
		},
		filterDef: function(/* (_ConditionExpr|null)? */filter){
			// summary:
			//		See _FilterLayerMixin.filterDef
			if(filter === undefined){
				this._restore();
				return this._filter;	//_ConditionExpr
			}else if(!filter){
				this._restore();
				return this._filter;
				/*Changes to fix Client side filter issue with server stores
				Adding another string 'clear' to handle filter clear condition when filter text is removed from quick filter*/
			}else if(filter == "clear"){
				this._filter = null;
				this._restore();
				return this._filter;
			}else{
				this._filter = filter;
				this._restore();
			}

		},
		setFilterText: function(filterText){
			this._filterText = filterText;
		},
		setGetter: function(/* function */getter){
			// summary:
			//		Set the user defined way to retrieve data from store.
			// tags:
			//		public
			// getter: function(datarow, colArg, rowIndex, store);
			if (dojo.isFunction(getter)) {
				this._getter = getter;
			}
		},
		filterCacheSize: function(/* Integer? */size){
			// summary:
			//		The get/set function for filter cache size.
			// tags:
			//		public
			// size: Integer?
			//		if > 0, it's a reasonable cache size (hope so), if <=0, means cache all.
			//		if undefined, return the current cache size.
			// returns:
			//		The current filter cache size if it's a getter
			if (size === undefined) {
				return this._cacheSize;	//Integer
			}
			this._cacheSize = size > 0 ? size : -1;
		},
		fetchAllOnFirstFilter: function(/* bool? */toFetchAll){
			// summary:
			//		The get/set function for fetchAll.
			// tags:
			//		public
			// toFetchAll: boolean?
			//		If provided, it's a set function, otherwise it's a get function.
			// returns:
			//		Whether fetch all on first filter if this is a getter
			if (toFetchAll === undefined) {
				return this._fetchAll;	//Boolean
			}
			this._fetchAll = !!toFetchAll;
		},
		invalidate: function(){
			this._restore();
		},
		
		//----------------Private Functions-----------------------------
		_restore: function(){
			// summary:
			//		Clear all the status information of this layer
			// tags:
			//		private
			this._items = [];
			this._nextUnfetchedIdx = 0;
			this._result = [];
			this._indexMap = [];
			this._cacheHoles = [];
			this._cachedItemCount = 0;
			this._resultStartIdx = 0;
		},
		_fetch: function(userRequest,filterRequest){
			// summary:
			//		Implement _StoreLayer._fetch
			// tags:
			//		private callback
			// filterRequest: dojo.data.api.Request
			//		The actual request used in store.fetch. 
			//		This function is called recursively to fill the result store items 
			//		until the user specified item count is reached. Only in recursive calls,
			//		this parameter is valid.
			if(this._filter == null){
				//If we don't have any filter, use the original request and fetch.
				var old_onbegin = userRequest.onBegin, _this = this;
				userRequest.onBegin = function(size, r){
					hitchIfCan(userRequest.scope, old_onbegin)(size, r);
					_this.onFiltered(size, size);
				};
				dojo.hitch(this._store,this._originFetch)(userRequest);
				return userRequest;
			}
			try{
				//If the fetch is at the beginning, user's start position is used;
				//If we are in a recursion, our own request is used.
				var start = filterRequest ? filterRequest._nextResultItemIdx : userRequest.start;
				start = start || 0;
				this._grid.filterApplied = true;
				if(!filterRequest){
					//Initially, we have no results.
					this._result = [];
					this._resultStartIdx = start;
					var sortStr;
					if(dojo.isArray(userRequest.sort) && userRequest.sort.length > 0 
						//Sort info will stay here in every re-fetch, so remember it!
						&& (sortStr = dojo.toJson(userRequest.sort)) != this._lastSortInfo){
						//If we should sort data, all the old caches are no longer valid.
						//console.log("restore by sort!!!!");
						this._restore();
						this._lastSortInfo = sortStr;
					}
				}
				//this._result contains the current fetch result (of every recursion).
				var end = typeof userRequest.count == "number" ?
							start + userRequest.count - this._result.length
							: this._items.length;
				//Try to retrieve all the items from our cache.
				//Only need items after userRequest.start, test it in case start is smaller.
				if(this._result.length){
				this._result = this._result.concat(this._items.slice(start, end));
				}else{
					this._result = this._items.slice(userRequest.start, typeof userRequest.count == "number" ?
						userRequest.start + userRequest.count : this._items.length);
				}
				
				//console.log("result:",this._result);
				if(this._result.length >= userRequest.count || this._hasReachedStoreEnd()){
					//We already have had enough items, or we have to stop fetching
					//because there's nothing more to fetch.
					this._completeQuery(userRequest);
				}else{
					//User's request hasn't been finished yet. Fetch more. 
					if(!filterRequest) {
						//Initially, we've got to create a new request object.
						filterRequest = shallowClone(userRequest);
						//Use our own onBegin function to remember the total size of the original store.
						filterRequest.onBegin = dojo.hitch(this,this._onFetchBegin);
						filterRequest.onComplete = dojo.hitch(this, function(items, req){
							//We've fetched some more, so march ahead!
							this._nextUnfetchedIdx += items.length;
							//Actual filtering work goes here. Survived items are added to our cache.
							//req is our own request object.
							this._doFilter(items, req.start, userRequest);
							//Recursively call this function. Let's do this again!
							//console.log("refetch!");
							this._fetch(userRequest, req);
						});
					}
					//Fetch starts from the next unfetched item.
					filterRequest.start = this._nextUnfetchedIdx;
					//If store is small, we should only fetch once.
					if(this._fetchAll) delete filterRequest.count;
					//Remember we've (maybe) already added something to our result array,
					//so next time we should not start over again.
					filterRequest._nextResultItemIdx = end<this._items.length ? end:this._items.length;
					//Actual fetch work goes here.
					//console.log("user:",userRequest);
					//console.log("filter:",filterRequest);
					//console.log("FETCHING...");
					filterRequest.query = null;
					this.originFetch(filterRequest);
				}
			}catch(e){
				if(userRequest.onError){
					hitchIfCan(userRequest.scope, userRequest.onError)(e, userRequest);
				}else{
					throw e;
				}
			}
			return userRequest;
		},
		_hasReachedStoreEnd: function(){
			// summary:
			//		Check whether all the items in the original store have been fetched.
			// tags:
			//		private
			return this._storeSize >= 0 && this._nextUnfetchedIdx >= this._storeSize;	//Boolean
		},
		_applyFilter: function(/* data item */datarow,/* Integer */rowIndex){
			// summary:
			//		Apply the filter to a row of data
			// tags:
			//		private
			// returns:
			//		whether this row survived the filter.
			var g = this._getter, s = this._store;
			try{
				return !!(this._filter.applyRow(datarow, function(item, arg){
					return g(item, arg, rowIndex, s);
				}).getValue());	
			}catch(e){
				console.log(e);
				return false;
			}
		},
		_doFilter: function(/* Array */items,/* Integer */startIdx,/* object */userRequest){
			// summary:
			//		Use the filter expression to filter items. Survived items are stored in this._items.
			//		The given items start from "startIdx" in the original store.
			// tags:
			//		private
			for (var i = 0, cnt=0; i < items.length; ++i) {
				//items[i].itemFound = false;
				var queryObj = {};
				queryObj.start = 0;
				queryObj.count = 20;
				
				var result = this._applyFilter(items[i], startIdx + i);				
				if(result){
					/*changes to fix client filter issue with server stores
					add an attribute to filtered rows to identify filter is applied and expando should not be shown*/
					//if(items[i]){
						//items[i].removeAttribute('relationshipsKey');
						//items[i].removeAttribute('lazyLoad') ;
						
						//items[i].filterApplied = true;
						//this._grid.filterApplied = true;
					
					//}
					
					hitchIfCan(userRequest.scope, userRequest.onItem)(items[i], userRequest);
					cnt += this._addCachedItems(items[i], this._items.length);
					this._indexMap.push(startIdx + i);
				}
			}
			//console.log("Filtered! "+cnt+" out of "+items.length);
		},
		_setChildren: function(items, request, size){
			this.children = items;
		},
		_onFetchBegin: function(/* Integer */size,/* request object */req){
			// summary:
			//		This function is used to replace the user's onFetchBegin in store.fetch
			// tags:
			//		private
			this._storeSize = size;
		},
		_completeQuery: function(/* request object */userRequest){
			// summary:
			//		Logically, the user's query is completed here, i.e., all the filtered results are ready.
			//		(or their index mappings are ready)
			// tags:
			//		private
			
			//console.log("query complete.");
			var size = this._items.length;
			if(this._nextUnfetchedIdx < this._storeSize){
				//FIXME: There's still some items in the original store that are not fetched & filtered.
				//So we have to estimate a little bigger size to allow scrolling to these unfetched items.
				//However, this behavior is ONLY correct in Grid! Any better way to do this?
				size++;
			}
			hitchIfCan(userRequest.scope, userRequest.onBegin)(size,userRequest);
			this._fillUncachedResult(userRequest,0);
		},
		_fillUncachedResult: function(/* request object */userRequest,/* Integer */resultIdx){
			// summary:
			//		If the cache is not big enough to contain all the store data, there might be some "holes" in the cache
			//		that are only an index mapping. So we have to fill these holes here before give the result back to user.
			// tags:
			//		private callback
			// resultIdx: Integer
			//		Since this function is a callback when a "hole" is fixed, we have to remember where to start checking the
			//		result array again.
			
			//console.log("start _fillUncachedResult: result cnt:",this._result.length,", idx:",resultIdx);
			var i,filterIdx,j,m;
			if(this._cacheSize >= 0){
				//Not cache all, so maybe some holes.
				for (i = resultIdx; i < this._result.length; ++i) {
					if (!this._result[i]) {
						//Found a hole in the result, but maybe it's been fixed in the cache.
						filterIdx = i + this._resultStartIdx;
						if (this._result[i] = this._items[filterIdx]) {//intentional assignment
							//Yes, it's fixed.
							//console.log("fixed");
							continue;
						}
						//It's a real cache hole.
						//console.log("need fill hole");
						for (j = 0; j < this._cacheHoles.length; ++j) {
							//Try to find a hole object that contains this hole.
							m = this._cacheHoles[j];
							if (filterIdx >= m.idx && filterIdx < m.idx + m.cnt) {
								//Found the hole object. Fetch missing items and fill the holes.
								//console.log("Hole Found! ", filterIdx, ", ",m);
								dojo.hitch(this._store,this._originFetch)({
									start: this._indexMap[m.idx],
									count: m.cnt,
									_filterIdx: m.idx,	//Index in the cache.
									_resultIdx: i,		//Index in the result array
									_holeObjIdx: j,		//Index in the hole queue
									onComplete: dojo.hitch(this, function(items, req){
										//FOR debug
										if(items.length != req.count){
											throw new Error("_fillUncachedResult: fatal error");
										}
										//Fill the holes
										this._addCachedItems(items, req._filterIdx);
										//console.log("Hole Filled! ", req._filterIdx, ", ", items.length);
										//Remove hole object
										this._cacheHoles.splice(req._holeObjIdx, 1);
										//Go on to check remaining result items.
										this._fillUncachedResult(userRequest, req._resultIdx);
									})
								});
								//Following work is in the callback: onComplete.
								return;
							}
						}
					}
				}
				//here we have no holes.
				//console.log("All holes filled");
			}
			//Before giving the control back to user, clear some cache if it's too big.
			
			this._checkCacheSize(userRequest);
			this.onFiltered(this._items.length, this._storeSize);
			hitchIfCan(userRequest.scope, userRequest.onComplete)(this._result, userRequest);
		},
		_checkCacheSize: function(/* request object */userRequest){
			// summary:
			//		Try to delete some items from the cache in case we've fetched too much.
			// tags:
			//		private
			
			//console.log("_checkCacheSize: cur holes:",this._cacheHoles.length,",cached cnt:",this._cachedItemCount,",cache size:",this._cacheSize);
			//If we cache all, it's a no-op.
			if (this._cacheSize < 0) {
				return;
			}
			var dif, del_cnt = 0, start = userRequest.start || 0, end = start;
			if(userRequest.count){
				end += userRequest.count;
			}
			if((dif=this._cachedItemCount-this._cacheSize)>0){
				//We have cached too much.
				var func = dojo.hitch(this, function(cnt, slice_start, slice_end){
					var arr = slice_end ? this._indexMap.slice(slice_start,slice_end) : this._indexMap.slice(slice_start);
					var q = _findLongestSequences(arr,slice_start);
					//console.log("HOLE_Q_Begin: ",q.length,",indexMap:",this._indexMap.length,",start:",slice_start,",end:",slice_end);
					while(cnt < dif && (m = q.shift())){ //intentional assignment
						//console.log("HOLE_OBJ: ",m);
						cnt += this._delCachedItems(m.idx,m.cnt);
						this._cacheHoles.push(m);
					}
					return cnt;
				});
				if(userRequest.start>0){
					//Try to delete some cache before the user's request position.
					del_cnt = func(0,0,start);
				}
				if(del_cnt < dif && end < this._indexMap.length){
					//Try to delete some cache after the user's request position.
					del_cnt = func(del_cnt,end);
				}
			}
			console.log("DIF:",dif,",Deleted:",del_cnt,",cur holes:",this._cacheHoles.length,",cached cnt:",this._cachedItemCount,",cache size:",this._cacheSize);
		},
		_addCachedItems: function(/* Array */items,/* Integer */filterStartIdx){
			// summary:
			//		Add data items to the cache. The insert point is at *filterStartIdx*
			// tags:
			//		private
			// items: Array
			//		Data items to add.
			// filterStartIdx: Integer
			//		The start point to insert in the cache.
			if(!dojo.isArray(items)){
				items = [items];
			}
			for(var k = 0; k < items.length; ++k){
				this._items[filterStartIdx + k] = items[k];
			}
			this._cachedItemCount += items.length;
			//console.log("ADD CACHE: ",items.length,", cache:",this._cachedItemCount);
			return items.length;
		},
		_delCachedItems: function(/* Integer */filterStartIdx,/* Integer */count){
			// summary:
			//		Delete some data items from the cache.
			// tags:
			//		private
			// filterStartIdx: Integer
			//		The position in the cache to start deletion.
			// count: Integer
			//		The number of items to delete.
			for(var k = filterStartIdx, end = filterStartIdx + count; k < end && k < this._items.length; ++k){
				delete this._items[k];
			}
			this._cachedItemCount -= count;
			//console.log("DEL CACHE: ",count,", cache:",this._cachedItemCount);
			return count;
		},
		onRowMappingChange: function(mapping){
			//This function runs in FilterLayer scope!
			if(this._filter != null){
				//console.log("filter onRowMapChange begin:",mapping);
				var m = dojo.clone(mapping),
					alreadyUpdated = {};
				for(var r in m){
					r = parseInt(r, 10);
					mapping[this._indexMap[r]] = this._indexMap[m[r]];
					if(!alreadyUpdated[this._indexMap[r]]){
						alreadyUpdated[this._indexMap[r]] = true;
					}
					if(!alreadyUpdated[r]){
						alreadyUpdated[r] = true;
						delete mapping[r];
					}
				}
				//console.log("filter onRowMapChange end:",mapping);
			}
		}
	});
})();

});

},
'dijit/form/CheckBox':function(){
define([
	"require",
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.set
	"dojo/_base/kernel",
	"dojo/query", // query
	"dojo/ready",
	"./ToggleButton",
	"./_CheckBoxMixin",
	"dojo/text!./templates/CheckBox.html",
	"dojo/NodeList-dom" // NodeList.addClass/removeClass
], function(require, declare, domAttr, kernel, query, ready, ToggleButton, _CheckBoxMixin, template){

/*=====
	var ToggleButton = dijit.form.ToggleButton;
	var _CheckBoxMixin = dijit.form._CheckBoxMixin;
=====*/

	// module:
	//		dijit/form/CheckBox
	// summary:
	//		Checkbox widget

	// Back compat w/1.6, remove for 2.0
	if(!kernel.isAsync){
		ready(0, function(){
			var requires = ["dijit/form/RadioButton"];
			require(requires);	// use indirection so modules not rolled into a build
		});
	}

	return declare("dijit.form.CheckBox", [ToggleButton, _CheckBoxMixin], {
		// summary:
		// 		Same as an HTML checkbox, but with fancy styling.
		//
		// description:
		//		User interacts with real html inputs.
		//		On onclick (which occurs by mouse click, space-bar, or
		//		using the arrow keys to switch the selected radio button),
		//		we update the state of the checkbox/radio.
		//
		//		There are two modes:
		//			1. High contrast mode
		//			2. Normal mode
		//
		//		In case 1, the regular html inputs are shown and used by the user.
		//		In case 2, the regular html inputs are invisible but still used by
		//		the user. They are turned quasi-invisible and overlay the background-image.

		templateString: template,

		baseClass: "dijitCheckBox",

		_setValueAttr: function(/*String|Boolean*/ newValue, /*Boolean*/ priorityChange){
			// summary:
			//		Handler for value= attribute to constructor, and also calls to
			//		set('value', val).
			// description:
			//		During initialization, just saves as attribute to the <input type=checkbox>.
			//
			//		After initialization,
			//		when passed a boolean, controls whether or not the CheckBox is checked.
			//		If passed a string, changes the value attribute of the CheckBox (the one
			//		specified as "value" when the CheckBox was constructed (ex: <input
			//		data-dojo-type="dijit.CheckBox" value="chicken">)
			//		widget.set('value', string) will check the checkbox and change the value to the
			//		specified string
			//		widget.set('value', boolean) will change the checked state.
			if(typeof newValue == "string"){
				this._set("value", newValue);
				domAttr.set(this.focusNode, 'value', newValue);
				newValue = true;
			}
			if(this._created){
				this.set('checked', newValue, priorityChange);
			}
		},
		_getValueAttr: function(){
			// summary:
			//		Hook so get('value') works.
			// description:
			//		If the CheckBox is checked, returns the value attribute.
			//		Otherwise returns false.
			return (this.checked ? this.value : false);
		},

		// Override behavior from Button, since we don't have an iconNode
		_setIconClassAttr: null,

		postMixInProperties: function(){
			this.inherited(arguments);

			// Need to set initial checked state as part of template, so that form submit works.
			// domAttr.set(node, "checked", bool) doesn't work on IE until node has been attached
			// to <body>, see #8666
			this.checkedAttrSetting = this.checked ? "checked" : "";
		},

		 _fillContent: function(){
			// Override Button::_fillContent() since it doesn't make sense for CheckBox,
			// since CheckBox doesn't even have a container
		},

		_onFocus: function(){
			if(this.id){
				query("label[for='"+this.id+"']").addClass("dijitFocusedLabel");
			}
			this.inherited(arguments);
		},

		_onBlur: function(){
			if(this.id){
				query("label[for='"+this.id+"']").removeClass("dijitFocusedLabel");
			}
			this.inherited(arguments);
		}
	});
});

},
'gridx/core/model/extensions/Mark':function(){
define([
	'dojo/_base/declare',
	'dojo/_base/array',
	'../_Extension'
], function(declare, array, _Extension){

	return declare(/*===== "gridx.core.model.extensions.Mark", =====*/_Extension, {
		name: 'move',

		priority: 5,

		constructor: function(model){
			var t = this;
			t.mixed = 'mixed';
			t.states = {
				0: false,
				1: t.mixed,
				2: true
			};
			t.clear();
			t._mixinAPI('getMark', 'getMarkedIds', 'markById', 'markByIndex', 'clearMark', 'treeMarkMode');
			t.aspect(model, '_msg', '_receiveMsg');
			t.aspect(model._cache, 'onLoadRow', '_onLoadRow');
			t.aspect(model, 'setStore', 'clear');
			model.onMarkChange = function(){};
			model._spTypes = {};
		},

		//Public------------------------------------------------------------------
		clear: function(){
			this._byId = {};
			this._last = {};
			this._lazy = {};
			this._tree = {};
		},

		clearMark: function(type){
			this._byId[this._initMark(type)] = {};
		},

		getMarkedIds: function(type, includePartial){
			var t = this,
				ret = [], id,
				tp = t._initMark(type),
				ids = t._byId[tp];
			if(ids){
				for(id in ids){
					if(includePartial || ids[id] == 2){
						ret.push(id);
					}
				}
			}
			return ret;
		},

		isMarked: function(id, type){
			type = this._initMark(type);
			var state = this._byId[type][id];
			return state == 2;
		},

		isPartialMarked: function(id, type){
			return this._byId[this._initMark(type)][id] == 1;
		},

		getMark: function(id, type){
			var m = this._byId[this._initMark(type)][id] || 0;
			return {
				0: false,
				1: this.mixed,
				2: true
			}[m];
		},

		markById: function(id, toMark, type){
			this._cmd(id, toMark, type, 1); //Should we make this sync?
		},

		markByIndex: function(index, toMark, type, parentId){
			if(index >= 0 && index < Infinity){
				this._cmd(index, toMark, type, 0, parentId);
			}
		},

		treeMarkMode: function(type, toEnable){
			type = this._initMark(type);
			var tm = this._tree;
			return toEnable === undefined ? tm[type] : (tm[type] = toEnable);
		},

		//Private----------------------------------------------------------------
		_cmdMark: function(){
			var t = this,
				args = arguments,
				ranges = [],
				m = t.model._model;
			array.forEach(args, function(arg){
				if(!arg[3]){
					ranges.push({
						start: arg[0],
						count: 1
					});
				}
			});
			return m._call('when', [{
				id: [],
				range: ranges
			}, function(){
				array.forEach(args, function(arg){
					var id = arg[3] ? arg[0] : m._call('indexToId', [arg[0], arg[4]]),
						toMark = arg[1],
						type = t._initMark(arg[2]);
					if(toMark === t.mixed){
						toMark = 1;
					}else if(toMark){
						toMark = 2;
					}else{
						toMark = 0;
					}
					t._mark(id, toMark, type);
				});
			}]);
		},

		_onDelete: function(id){
			var t = this,
				tp,
				c = t._byId,
				s = t._last,
				z = t._lazy;
			for(tp in c){
				tp = t._initMark(tp);
				delete c[tp][id];
				delete s[tp][id];
				delete z[tp][id];
			}
			t.onDelete.apply(t, arguments);
		},

		_initMark: function(type){
			var t = this,
				c = t._byId,
				s = t._last,
				z = t._lazy,
				tp = type || 'select';
			c[tp] = c[tp] || {};
			z[tp] = z[tp] || {};
			s[tp] = s[tp] || {};
			return tp;
		},

		_cmd: function(){
			this.model._addCmd({
				name: "_cmdMark",
				scope: this,
				args: arguments,
				async: 1
			});
		},

		_receiveMsg: function(msg, filteredIds){
			if(msg == 'filter'){
				var t = this,
					tp, id,
					sp = t.model._spTypes;
				for(tp in sp){
					if(sp[tp]){
						for(id in t._byId[tp]){
							if(!filteredIds[id]){
								t._doMark(id, tp, 0);
							}
						}
					}
				}
			}
		},

		///////////////////////////////////////////////////////////////////////////////////////////////
		_mark: function(id, toMark, type){
			var t = this,
				tp = t._initMark(type),
				state = t._byId[tp][id] || 0;
			if(t.model.isId(id) && state != toMark){
				t._doMark(id, tp, toMark);
			}
		},

		_onLoadRow: function(id){
			var t = this,
				m = t.model,
				mm = m._model,
				lazy = t._lazy,
				type, lz, flag,
				pid = mm._call('treePath', [id]).pop();
			if(m.isId(pid)){
				for(type in lazy){
					lz = lazy[type];
					flag = lz[pid];
					if(typeof flag == 'number'){
						flag = lz[pid] = {
							toMark: flag,
							count: mm._call('size', [pid])
						};
					}
					if(flag){
						--flag.count;
						if(!flag.count){
							delete lz[pid];
						}
						t._doMark(id, type, flag.toMark, 1);
					}
				}
			}
		},

		_fireEvent: function(id, type, toMark, oldState){
			var t = this,
				m = t.model;
			if(toMark != oldState){
				if(!toMark){
					delete t._byId[type][id];
				}
				//console.log('mark change: ', id, ', state: ', oldState, ' => ', toMark);
				m.onMarkChange(id, t.states[toMark || 0], t.states[oldState || 0], type);
			}
		},

		_doMark: function(id, tp, toMark, skipParent){
			var i, ids, children, childId, pid, siblings, markCount, fullCount, treePath,
				t = this,
				m = t.model,
				mm = m._model,
				byId = t._byId[tp],
				last = t._last[tp],
				lazy = t._lazy[tp],
				oldState = byId[id] || 0,
				newState;
			if(t._tree[tp]){
				children = mm._call('children', [id]);
				if(toMark == 1 && array.every(children, function(childId){
					return (last[childId] || 0) == (last[children[0]] || 0);
				})){
					toMark = 2;
				}
			}
			byId[id] = last[id] = toMark;
			t._fireEvent(id, tp, toMark, oldState);
			if(t._tree[tp]){
				ids = [id];
				while(ids.length){
					childId = ids.shift();
					oldState = byId[childId] || 0;
					newState = byId[childId] = toMark == 1 ? last[childId] || 0 : toMark;
					t._fireEvent(childId, tp, newState, oldState);
					if(mm._call('hasChildren', [childId])){
						children = mm._call('children', [childId]);
						if(children.length){
							ids = ids.concat(children);
						}else{
							lazy[childId] = toMark;
						}
					}
				}
				if(!skipParent){
					treePath = mm._call('treePath', [id]);
					for(i = treePath.length - 1; i > 0; --i){
						pid = treePath[i];
						oldState = byId[pid];
						siblings = mm._call('children', [pid]);
						markCount = array.filter(siblings, function(childId){
							return last[childId] = byId[childId];
						}).length;
						fullCount = array.filter(siblings, function(childId){
							return byId[childId] == 2;
						}).length;
						if(fullCount == siblings.length && oldState != 2){
							byId[pid] = 2; //none|partial -> all
						}else if(!markCount && oldState){
							delete byId[pid]; //all|partial -> none
						}else if(markCount && fullCount < siblings.length && oldState != 1){
							byId[pid] = 1; //all|none -> partial
						}
						t._fireEvent(pid, tp, byId[pid], oldState);
					}
				}
			}
		}
	});
});

},
'url:ibm/usmi/statuspod/templates/alertMenuItemLabel.html':"<table>\r\n<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n\t<tbody>\r\n\t\t<tr>\r\n\t\t\t<td class=\"statusPod-sourceName\"><div style=\"width:330px;text-overflow:ellipsis;overflow: hidden;white-space:nowrap;\">${sourceName} - ${objectType}</div></td>\r\n\t\t\t<td class=\"statusPod-shortTime\">${shortTime}</td>\r\n\t\t</tr>\r\n\t<tbody>\r\n</table>",
'dijit/form/DateTextBox':function(){
define([
	"dojo/_base/declare", // declare
	"../Calendar",
	"./_DateTimeTextBox"
], function(declare, Calendar, _DateTimeTextBox){

/*=====
	var Calendar = dijit.Calendar;
	var _DateTimeTextBox = dijit.form._DateTimeTextBox;
=====*/

	// module:
	//		dijit/form/DateTextBox
	// summary:
	//		A validating, serializable, range-bound date text box with a drop down calendar


	return declare("dijit.form.DateTextBox", _DateTimeTextBox, {
		// summary:
		//		A validating, serializable, range-bound date text box with a drop down calendar
		//
		//		Example:
		// |	new dijit.form.DateTextBox({value: new Date(2009, 0, 20)})
		//
		//		Example:
		// |	<input data-dojo-type='dijit.form.DateTextBox' value='2009-01-20'>

		baseClass: "dijitTextBox dijitComboBox dijitDateTextBox",
		popupClass: Calendar,
		_selector: "date",

		// value: Date
		//		The value of this widget as a JavaScript Date object, with only year/month/day specified.
		//		If specified in markup, use the format specified in `stamp.fromISOString`.
		//		set("value", ...) accepts either a Date object or a string.
		value: new Date("")	// value.toString()="NaN"
	});
});

},
'dijit/Calendar':function(){
define([
	"dojo/_base/array", // array.map
	"dojo/date",
	"dojo/date/locale",
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.get
	"dojo/dom-class", // domClass.add domClass.contains domClass.remove domClass.toggle
	"dojo/_base/event", // event.stop
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/keys", // keys
	"dojo/_base/lang", // lang.hitch
	"dojo/_base/sniff", // has("ie")
	"./CalendarLite",
	"./_Widget",
	"./_CssStateMixin",
	"./_TemplatedMixin",
	"./form/DropDownButton",
	"./hccss"	// not used directly, but sets CSS class on <body>
], function(array, date, local, declare, domAttr, domClass, event, kernel, keys, lang, has,
			CalendarLite, _Widget, _CssStateMixin, _TemplatedMixin, DropDownButton){

/*=====
	var CalendarLite = dijit.CalendarLite;
	var _CssStateMixin = dijit._CssStateMixin;
	var _Widget = dijit._Widget;
	var _TemplatedMixin = dijit._TemplatedMixin;
	var DropDownButton = dijit.form.DropDownButton;
=====*/

	// module:
	//		dijit/Calendar
	// summary:
	//		A simple GUI for choosing a date in the context of a monthly calendar.

	var Calendar = declare("dijit.Calendar",
		[CalendarLite, _Widget, _CssStateMixin], // _Widget for deprecated methods like setAttribute()
		{
		// summary:
		//		A simple GUI for choosing a date in the context of a monthly calendar.
		//
		// description:
		//		See CalendarLite for general description.   Calendar extends CalendarLite, adding:
		//			- month drop down list
		//			- keyboard navigation
		//			- CSS classes for hover/mousepress on date, month, and year nodes
		//			- support of deprecated methods (will be removed in 2.0)

		// Set node classes for various mouse events, see dijit._CssStateMixin for more details
		cssStateNodes: {
			"decrementMonth": "dijitCalendarArrow",
			"incrementMonth": "dijitCalendarArrow",
			"previousYearLabelNode": "dijitCalendarPreviousYear",
			"nextYearLabelNode": "dijitCalendarNextYear"
		},

		setValue: function(/*Date*/ value){
			// summary:
			//      Deprecated.   Use set('value', ...) instead.
			// tags:
			//      deprecated
			kernel.deprecated("dijit.Calendar:setValue() is deprecated.  Use set('value', ...) instead.", "", "2.0");
			this.set('value', value);
		},

		_createMonthWidget: function(){
			// summary:
			//		Creates the drop down button that displays the current month and lets user pick a new one

			return new Calendar._MonthDropDownButton({
				id: this.id + "_mddb",
				tabIndex: -1,
				onMonthSelect: lang.hitch(this, "_onMonthSelect"),
				lang: this.lang,
				dateLocaleModule: this.dateLocaleModule
			}, this.monthNode);
		},

		buildRendering: function(){
			this.inherited(arguments);

			// Events specific to Calendar, not used in CalendarLite
			this.connect(this.domNode, "onkeypress", "_onKeyPress");
			this.connect(this.dateRowsNode, "onmouseover", "_onDayMouseOver");
			this.connect(this.dateRowsNode, "onmouseout", "_onDayMouseOut");
			this.connect(this.dateRowsNode, "onmousedown", "_onDayMouseDown");
			this.connect(this.dateRowsNode, "onmouseup", "_onDayMouseUp");
		},

		_onMonthSelect: function(/*Number*/ newMonth){
			// summary:
			//      Handler for when user selects a month from the drop down list
			// tags:
			//      protected

			// move to selected month, bounding by the number of days in the month
			// (ex: dec 31 --> jan 28, not jan 31)
			this._setCurrentFocusAttr(this.dateFuncObj.add(this.currentFocus, "month",
				newMonth - this.currentFocus.getMonth()));
		},

		_onDayMouseOver: function(/*Event*/ evt){
			// summary:
			//      Handler for mouse over events on days, sets hovered style
			// tags:
			//      protected

			// event can occur on <td> or the <span> inside the td,
			// set node to the <td>.
			var node =
				domClass.contains(evt.target, "dijitCalendarDateLabel") ?
				evt.target.parentNode :
				evt.target;

			if(node && (
				(node.dijitDateValue && !domClass.contains(node, "dijitCalendarDisabledDate"))
					|| node == this.previousYearLabelNode || node == this.nextYearLabelNode
				)){
				domClass.add(node, "dijitCalendarHoveredDate");
				this._currentNode = node;
			}
		},

		_onDayMouseOut: function(/*Event*/ evt){
			// summary:
			//      Handler for mouse out events on days, clears hovered style
			// tags:
			//      protected

			if(!this._currentNode){ return; }

			// if mouse out occurs moving from <td> to <span> inside <td>, ignore it
			if(evt.relatedTarget && evt.relatedTarget.parentNode == this._currentNode){ return; }
			var cls = "dijitCalendarHoveredDate";
			if(domClass.contains(this._currentNode, "dijitCalendarActiveDate")){
				cls += " dijitCalendarActiveDate";
			}
			domClass.remove(this._currentNode, cls);
			this._currentNode = null;
		},

		_onDayMouseDown: function(/*Event*/ evt){
			var node = evt.target.parentNode;
			if(node && node.dijitDateValue && !domClass.contains(node, "dijitCalendarDisabledDate")){
				domClass.add(node, "dijitCalendarActiveDate");
				this._currentNode = node;
			}
		},

		_onDayMouseUp: function(/*Event*/ evt){
			var node = evt.target.parentNode;
			if(node && node.dijitDateValue){
				domClass.remove(node, "dijitCalendarActiveDate");
			}
		},

		handleKey: function(/*Event*/ evt){
			// summary:
			//		Provides keyboard navigation of calendar.
			// description:
			//		Called from _onKeyPress() to handle keypress on a stand alone Calendar,
			//		and also from `dijit.form._DateTimeTextBox` to pass a keypress event
			//		from the `dijit.form.DateTextBox` to be handled in this widget
			// returns:
			//		False if the key was recognized as a navigation key,
			//		to indicate that the event was handled by Calendar and shouldn't be propogated
			// tags:
			//		protected
			var increment = -1,
				interval,
				newValue = this.currentFocus;
			switch(evt.charOrCode){
				case keys.RIGHT_ARROW:
					increment = 1;
					//fallthrough...
				case keys.LEFT_ARROW:
					interval = "day";
					if(!this.isLeftToRight()){ increment *= -1; }
					break;
				case keys.DOWN_ARROW:
					increment = 1;
					//fallthrough...
				case keys.UP_ARROW:
					interval = "week";
					break;
				case keys.PAGE_DOWN:
					increment = 1;
					//fallthrough...
				case keys.PAGE_UP:
					interval = evt.ctrlKey || evt.altKey ? "year" : "month";
					break;
				case keys.END:
					// go to the next month
					newValue = this.dateFuncObj.add(newValue, "month", 1);
					// subtract a day from the result when we're done
					interval = "day";
					//fallthrough...
				case keys.HOME:
					newValue = new this.dateClassObj(newValue);
					newValue.setDate(1);
					break;
				case keys.ENTER:
				case " ":
					this.set("value", this.currentFocus);
					break;
				default:
					return true;
			}

			if(interval){
				newValue = this.dateFuncObj.add(newValue, interval, increment);
			}

			this._setCurrentFocusAttr(newValue);

			return false;
		},

		_onKeyPress: function(/*Event*/ evt){
			// summary:
			//		For handling keypress events on a stand alone calendar
			if(!this.handleKey(evt)){
				event.stop(evt);
			}
		},

		onValueSelected: function(/*Date*/ /*===== date =====*/){
			// summary:
			//		Deprecated.   Notification that a date cell was selected.  It may be the same as the previous value.
			// description:
			//      Formerly used by `dijit.form._DateTimeTextBox` (and thus `dijit.form.DateTextBox`)
			//      to get notification when the user has clicked a date.  Now onExecute() (above) is used.
			// tags:
			//      protected
		},

		onChange: function(value){
			this.onValueSelected(value);	// remove in 2.0
		},

		getClassForDate: function(/*===== dateObject, locale =====*/){
			// summary:
			//		May be overridden to return CSS classes to associate with the date entry for the given dateObject,
			//		for example to indicate a holiday in specified locale.
			// dateObject: Date
			// locale: String?
			// tags:
			//      extension

/*=====
			return ""; // String
=====*/
		}
	});

	Calendar._MonthDropDownButton = declare("dijit.Calendar._MonthDropDownButton", DropDownButton, {
		// summary:
		//		DropDownButton for the current month.    Displays name of current month
		//		and a list of month names in the drop down

		onMonthSelect: function(){ },

		postCreate: function(){
			this.inherited(arguments);
			this.dropDown = new Calendar._MonthDropDown({
				id: this.id + "_mdd", //do not change this id because it is referenced in the template
				onChange: this.onMonthSelect
			});
		},
		_setMonthAttr: function(month){
			// summary:
			//		Set the current month to display as a label
			var monthNames = this.dateLocaleModule.getNames('months', 'wide', 'standAlone', this.lang, month);
			this.dropDown.set("months", monthNames);

			// Set name of current month and also fill in spacer element with all the month names
			// (invisible) so that the maximum width will affect layout.   But not on IE6 because then
			// the center <TH> overlaps the right <TH> (due to a browser bug).
			this.containerNode.innerHTML =
				(has("ie") == 6 ? "" : "<div class='dijitSpacer'>" + this.dropDown.domNode.innerHTML + "</div>") +
				"<div class='dijitCalendarMonthLabel dijitCalendarCurrentMonthLabel'>" +  monthNames[month.getMonth()] + "</div>";
		}
	});

	Calendar._MonthDropDown = declare("dijit.Calendar._MonthDropDown", [_Widget, _TemplatedMixin], {
		// summary:
		//		The list-of-months drop down from the MonthDropDownButton

		// months: String[]
		//		List of names of months, possibly w/some undefined entries for Hebrew leap months
		//		(ex: ["January", "February", undefined, "April", ...])
		months: [],

		templateString: "<div class='dijitCalendarMonthMenu dijitMenu' " +
			"data-dojo-attach-event='onclick:_onClick,onmouseover:_onMenuHover,onmouseout:_onMenuHover'></div>",

		_setMonthsAttr: function(/*String[]*/ months){
			this.domNode.innerHTML = array.map(months, function(month, idx){
					return month ? "<div class='dijitCalendarMonthLabel' month='" + idx +"'>" + month + "</div>" : "";
				}).join("");
		},

		_onClick: function(/*Event*/ evt){
			this.onChange(domAttr.get(evt.target, "month"));
		},

		onChange: function(/*Number*/ /*===== month =====*/){
			// summary:
			//		Callback when month is selected from drop down
		},

		_onMenuHover: function(evt){
			domClass.toggle(evt.target, "dijitCalendarMonthLabelHover", evt.type == "mouseover");
		}
	});

	return Calendar;
});

},
'ibm/tivoli/tip/navwidget/dijit/RESTContextMenuProvider':function(){
// wrapped by build app
define("ibm/tivoli/tip/navwidget/dijit/RESTContextMenuProvider", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/tip/dojo/TIPHelper,ibm/tivoli/navmodel/Trace,ibm/tivoli/tip/navwidget/dijit/AsynchContextMenuProvider"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * "Restricted Materials of IBM"
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.dijit.RESTContextMenuProvider");

dojo.require("ibm.tivoli.tip.dojo.TIPHelper");
dojo.require("ibm.tivoli.navmodel.Trace");
dojo.require("ibm.tivoli.tip.navwidget.dijit.AsynchContextMenuProvider");

ibm.tivoli.tip.navwidget.dijit.RESTContextMenuProvider._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.tip.navwidget.dijit.RESTContextMenuProvider");
dojo.declare("ibm.tivoli.tip.navwidget.dijit.RESTContextMenuProvider",
        ibm.tivoli.tip.navwidget.dijit.AsynchContextMenuProvider,
{		
	// summary:
	//		Coming soon
	// description:
	//		Coming soon

	// navigateTopic: "NavigateDataset",
	
	// _tasks:
	//		tasks retrieved from REST service
	_tasks: null,
	
	// _serverTasks:
	//		tasks on the menu that need to call back to server when clicked
	_serverTasks: null,
	
	// _tasksLoadedCallback:
	//		notification callback for when tasks are loaded
	_tasksLoadedCallback: null,
	
    // _pendingRequest:
    //     handle to dojo Deferred returned from xhr request
    _pendingRequest: null,      // 37703	
	
	 // _NOPORTLETNAMESPACE:
	 //     constant used as a flag for this.iscPortletNamespace if one is not provided and one cannot be found
	 //     Note that this string is not translated
	_NOPORTLETNAMESPACE: "NO_PORTLET_NAMESPACE",

	 // _NOPORTLETINSTANCEID:
	 //     constant used as a flag for this.iscPortletInstanceID if one is not provided and one cannot be found
	 //     Note that this string is not translated
	_NOPORTLETINSTANCEID: "NO_PORTLET_INSTANCE_ID",  // 252065
	
	// view: [public] ibm.tivoli.tip.navwidget.dijit.NavWidgetPluggableView
	//			The view that owns this provider. Must be passed in as an argument when this provider is created.
	view: null, 

	// _trace: [private] ibm.tivoli.navmodel.Trace
	//			Utility for trace and logging
	_trace: ibm.tivoli.tip.navwidget.dijit.RESTContextMenuProvider._trace,

	buildClientNavModelSelection: function(context, /*Float*/ version) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// context:
		//		
		// version:
		//		
		var currentSelection = null;
		var selectedThings = ("selection" in context) ? context.selection : null;
		if (selectedThings) {
			currentSelection = new ibm.tivoli.navmodel.Selection({version: version});
			currentSelection.addItems(selectedThings.getItemIds());
			currentSelection.addLinks(selectedThings.getLinkIds());
			currentSelection.addRelationships(selectedThings.getRelationshipIds());
		}
		return currentSelection;
	},

    cancelPendingRequest: function() {  // 37703
    	// Note: As of Dojo 1.7.2, calling cancel() on the Deferred object
    	// for the dojo.xhrPOST request will still cause the xhr code to 
    	// write a console.error( ) statement to the console log of "xhr canceled"
    	// even if the failOK flag on the request was set to true.  This is due
    	// to the following Dojo bug report.  http://bugs.dojotoolkit.org/ticket/11568
    	// This does not impact the proper operation of canceling the request, but only
    	// results in this error message being written to the log.
        if (this._pendingRequest) {
           this._pendingRequest.cancel();
           this._pendingRequest = null;
        }
    },	
	
	getMenuItemsAsynch: function(context, itemsLoadedCallback, category) {
		if (context.dataset) { //251075
			if (!this.iscPortletNamespace && context.domNode) {
				this.setPortletNamespace(context.domNode);
			}			
			
			var requestCategory = "menu";
			if (itemsLoadedCallback != null) {
				this._tasksLoadedCallback = itemsLoadedCallback;
			}
			if (category && (category != null) && (typeof category == "string" )) {
				requestCategory = category;
			}
			requestCategory = this.augmentRequestCategory(requestCategory);
			
			//if (selectedItemIds.length > 0) {   
			this._pendingRequest = context.dataset.getTasks(   // 37703
	    				{onComplete: dojo.hitch(this, this._processMenuItemTasks),
	    				onError: dojo.hitch(this, this._handleMenuItemTasksError),
	    				taskCategory: requestCategory,
	    				failOK: true,    // 37703
	    				selection: this.buildClientNavModelSelection(context, context.dataset.getVersion())});
	    	//}
		} else { 
			//Should still invoke the callback (and for consistency should be on different thread)
			if (itemsLoadedCallback != null) {
				setTimeout(dojo.hitch(this, itemsLoadedCallback), 0);
			}
		}
	},
	
	augmentRequestCategory: function(/*String*/ category) {
		// summary:
		//		Return the category to use for getMenuItemsAsynch( )
		// description:
		//		This implementation does not modify the category that is passed in.
		//      Subclasses may choose to provide additional logic by overriding this function.
		// category:
		//		
		return category;
	},
	
	getMenuItems: function(context) 
	{		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// context:
		//		
		if (this._tasks != null) {
			this.menus_ = this.tasksAsMenuJson(this._tasks);
			this._tasks = null;  // tasks processed for menu, so get rid of them
		} else {
			this.menus_ = "";
		}
		if (this.getMenuItemsCallback !== null) {
			var moreItems = this.getMenuItemsCallback(context);
	         // Check added items and set defaults for isServer and isResolved if not already set
	         dojo.forEach(moreItems, function(item) {
	            if (!("isServer" in item)) {
	               item.isServer = false;
	            }
	            if (!("isResolved" in item)) {
	               item.isResolved = true;
	            }
	         });
			this.menus_ = this.menus_.concat(moreItems);
		}

		return this.menus_;
	},
	
	// override to add selection to the result if it is a navigation or server task
	getMenuCommand: function (commandName, context, menuItem) 
	{		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// commandName:
		//		
		// context:
		//		
		// menuItem:
		//		
	   var result = null;
	   if (this.getMenuCommandCallback !== null) 
	   {
	      result = this.getMenuCommandCallback(commandName, context, menuItem);
	   }
	   else
	   {
	      result = this.getItem(commandName);
	   }
	   if ((result !== null) && ("isServer" in result) && result.isServer) {			   
		   result.selection = context.selection;
	   }
	   else if ((result !== null) && ("isResolved" in result) && !result.isResolved) {
		   result.selection = context.selection;
	   }
 
	   this.handleMenuAction(result);
	},	
	
	// override to handle navigation, server tasks, and 2-phase resolution tasks
	handleMenuAction: function(data, ioArgs) 
	{		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// data:
		//		
		// ioArgs:
		//		
	    if (data.type == "navigation") {
	    	this.handleNavigation(data, ioArgs);
	    }
	    else if (data.isServer) {
	    	this.handleExecuteOnServer(data, ioArgs);
	    }
	    // check if the action has parameters that need to be resolved (e.g., CMS action)
	    else if (("isResolved" in data) && !(data.isResolved)) {  
	    	this.resolveTaskParms(data, ioArgs);
	        this._formatLaunchParms(data); // 251855
	    	this.inherited(arguments);
	    }
	    else {
	    	this._formatLaunchParms(data); // 251855
	    	this.inherited(arguments);
	    }
	}, 
	
	handleNavigation: function(/*Object*/ data, ioArgs) {
		// summary:
		//		Function that should be overloaded to handle a navigation task.  If it
		//		is not overloaded, then this action is a NOOP.
		// data:
		//		Contains the key attributes of the task plus selection (the selection
		//		object from the context containing tagged arrays of IDs for
		//		itemIds, linkIds, relationshipIds)
		// ioArgs:
		//		
	},
	
	handleExecuteOnServer: function(data, ioArgs) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// data:
		//		
		// ioArgs:
		//		
		var id = data.actionID;
		
		// Check if a specific server task has been passed in with the ioArgs, if so, use it
		var matchTasks = [];
		if (ioArgs && ("task" in ioArgs) && (ioArgs.task != null) && (ioArgs.task.getType() != "navigation") && ioArgs.task.isServer()) {
			matchTasks.push(ioArgs.task);
		} else {
			matchTasks = dojo.filter(this._serverTasks, function(x) {return x.getId() == id;});			
		}
		if (matchTasks.length > 0) {
			matchTasks[0].execute({onError: dojo.hitch(this, this._handleExecuteOnServerError),
									selection: this.buildClientNavModelSelection(data, matchTasks[0].getVersion())
				});
		}
	},
	
	resolveTaskParms: function(data, ioArgs) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// data:
		//		
		// ioArgs:
		//		
		var id = data.actionID;
		
		// Check if a specific unresolved task has been passed in with the ioArgs, if so, use it
		var matchTasks = [];
		if (ioArgs && ("task" in ioArgs) && (ioArgs.task != null) && !(ioArgs.task.isResolved())) {
			matchTasks.push(ioArgs.task);
		} else {
			matchTasks = dojo.filter(this._serverTasks, function(x) {return x.getId() == id;});
		}
		if (matchTasks.length > 0) {
			matchTasks[0].resolve({selection: this.buildClientNavModelSelection(data, matchTasks[0].getVersion())});
		}
		data.isResolved = matchTasks[0].isResolved();
		// Fill in values and parameters that have been resolved
		if (matchTasks[0].isUrlLaunch()) {
			data.uri = matchTasks[0].getLaunchUrl();
			if (matchTasks[0].getLaunchBrowserWindowId()) {  // 32355
				data.browserwindowid = matchTasks[0].getLaunchBrowserWindowId(); 				
			}
		} else if (matchTasks[0].isPageLaunch()) {
			data.portletPageID = matchTasks[0].getLaunchPageId();
			data.parameters = matchTasks[0].getLaunchApplicationParms();
		} else if (matchTasks[0].isPortletLaunch()) {
			data.portletName = matchTasks[0].getLaunchPortletName();
			data.portletAppUID = matchTasks[0].getLaunchPortletAppUID();
			data.parameters = matchTasks[0].getLaunchApplicationParms();
		} else if (matchTasks[0].isClientEvent()) {
			data.event = matchTasks[0].getLaunchEvent();
			data.parameters = matchTasks[0].getLaunchApplicationParms();
		} else if (matchTasks[0].isJavascript()) {            // 254268
    		data.uri = matchTasks[0].getLaunchJavascript();
		}
	},
		
	filterTasks: function(/*ibm.tivoli.navmodel.Task[]*/ tasks) {
		// summary:
		//		Return a filtered collection of the tasks passed in
		// description:
		//		This implementation returns the collection of tasks unfiltered.
		//      Subclasses may choose to provide additional filtering.
		// tasks:
		
		return tasks;
	},

	filterNavigationTasks: function(/*ibm.tivoli.navmodel.Task[]*/ tasks) {
		// summary:
		//		Return a filtered collection of tasks (i.e. remove all navigation tasks)
		// description:
		//		This implementation removes all navigation tasks.
		//      	Subclasses may choose to override the removal of navigation tasks
		// tasks:
		
		if (tasks) {
			var length = tasks.length;
			for (var index = 0;  index < length;  ++index) {
				var remove = false;
				var task = tasks[index];

				if (task.isNavigation()) { 					
					remove = true;
				}
				else if (task.isCascade()) {
					var childTasks = task.getChildTasks();
					this.filterNavigationTasks(childTasks);
					childTasks = task.getChildTasks();
					if (!childTasks || childTasks.length === 0) {
						remove = true;
					}
				}

				if (remove) {
					tasks.splice(index, 1);
					--index;
					--length;
				}
			}
		}

		return tasks;
	},
	
	_processMenuItemTasks: function(/*ibm.tivoli.navmodel.Task[]*/ tasks, kwArgs) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// tasks:
		//		
		// kwArgs:
		//		
		
        // 37703
        // Tasks returned from xhr request, so drop handle to request object
        this._pendingRequest = null;
		
		this._tasks = (tasks.length > 0) ? tasks : [];
		this._tasks = this.filterTasks(this._tasks);
		this._tasks = this.filterNavigationTasks(this._tasks);
		this._serverTasks = this._getServerTasks(this._tasks);
		if (this._tasksLoadedCallback != null) {
			// report success only if any tasks were returned
			this._tasksLoadedCallback(this._tasks !== null);
		}
	},
	
	_handleMenuItemTasksError: function(error, exception, ioArgs) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// error:
		//		
		// exception:
		//		
		// ioArgs:
		//		
		this._tasks = null;
		this._serverTasks = null;
		
        // 37703
        // Check if getTasks request was canceled
        var canceled = (error.dojoType == "cancel");
        // Since this was either canceled or an error, drop the handle to request object
        this._pendingRequest = null;
        
		// log error here
		if (this._trace.isError() && !canceled) {  // 37703
			this._trace.error(">>> query menu tasks error = " + error, "_handleMenuItemTasksError");
		}
		if (this._tasksLoadedCallback != null) {
			this._tasksLoadedCallback(false);
		}

		//Give the user some feedback
		if (this.view && !canceled) {  // 37703
			this.view.displayException(error);
		}
	},
	
	_handleExecuteOnServerError: function(error, exception, ioArgs) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// error:
		//		
		// exception:
		//		
		// ioArgs:
		//		

		if (this._trace.isError()) {
			this._trace.error(">>> error executing server task = " + error, "_handleExecuteOnServerError");
		}
		
		//Give the user some feedback
		if (this.view) {
			this.view.displayException(error);
		}
	},
	
	// cache tasks that are to execute on server or require parameter resolution
	_getServerTasks: function(tasks) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// tasks:
		//		
		var serverTasks = [];
		var subMenuTasks = null;
		for (var i = 0; i < tasks.length; i++) {
			if (tasks[i].isServer() || !(tasks[i].isResolved())) {
				serverTasks.push(tasks[i]);
			}
			else if (tasks[i].isCascade) {
				subMenuTasks = tasks[i].getChildTasks();
				if (subMenuTasks) {
					var moreTasks = this._getServerTasks(subMenuTasks);
					for (var j = 0; j < moreTasks.length; j++) {
						serverTasks.push(moreTasks[j]);
					}
				}
			}
		}
		return serverTasks;
	},
	
	// cache tasks that are to execute on server or require parameter resolution
	// this was added to support toolbar server tasks.   The tasks are retrieved directly by
	// AbstractTableView, calling getTasks on the dataset, this was done to avoid due to
	// cross contamination between toolbar tasks and menu tasks.  
	_setServerTasks: function(serverTasks) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// tasks:
		//		
		this._serverTasks = serverTasks;
	},
	
	_formatLaunchParms: function(/* Object or Array */ data) {  // 251855
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// data:
		//
		if (data.parameters && !(data.parameters instanceof Array)) {
			var mappedParms = [];
			var parm;
			for (var key in data.parameters) {
				parm = {};
				parm.name = key;
				parm.value = data.parameters[key];
				mappedParms.push(parm);
			}
			if (mappedParms.length > 0) {
				data.parameters = mappedParms;
			}
		}
	},

	
	setPortletNamespace: function(/* Element */ element) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// tasks:
		//
		
		var tipHelper = new ibm.tivoli.tip.dojo.TIPHelper();
		// Add functions to this TIPHelper instance to lookup the portlet namespace
		// and whether this is an iWidget
		// (These may be contributed back to TIPHelper itself)
		dojo.safeMixin(tipHelper, {
			getPortletNamespace: function (element) {
				var node = this.getAncestorByName (element, "IscPortlet");
				if (!node) {  // 254337
					// Could be on a freeform page in TIP
					node = this.getAncestorByClass(element, "tip-portlet");
				}
				if (node) {
					var result = node.modelObject.portletData.namespace;
					return (result);
				}
			},
			getPortletInstanceID: function (element) {   // 252065
				var node = this.getAncestorByName (element, "IscPortlet");
				if (!node) {  // 254337
					// Could be on a freeform page in TIP
					node = this.getAncestorByClass(element, "tip-portlet");
				}
				if (node) {
					var result = node.modelObject.portletData.oid;
					return (result);
				}				
			},
			isiWidget: function (element) {     // 252065
				var node = this.getAncestorByName (element, "IscPortlet");
				if (!node) {  // 254337
					// Could be on a freeform page in TIP
					node = this.getAncestorByClass(element, "tip-portlet");
				}
				var result = false;
				if (node && ("iwidget" in node.modelObject.portletData)) {
					// Coerce the string value of the iwidget flag to a boolean
					result = (String(node.modelObject.portletData.iwidget) == "true");
				}
				return (result);
			}
		});
		
		// 252065  - various changes below for the rest of this function
		var frameElem = null;
		var namespace = tipHelper.getPortletNamespace(element);
		var portletInstanceID = tipHelper.getPortletInstanceID(element);
		var isiwidget = tipHelper.isiWidget(element);
		// No namespace so we may be inside an IFRAME
		if (!namespace) {
			frameElem = dojo.window.get(dojo.body().ownerDocument).frameElement;			
		}
		// Make sure something is set for namespace
		if (!namespace ) {
			if (frameElem && frameElem.tagName && frameElem.tagName.toLowerCase() == "iframe") {
				namespace = tipHelper.getPortletNamespace(frameElem);
			}
		}
		if (!namespace) {
			namespace = this._NOPORTLETNAMESPACE;
		}
		// Make sure something is set for portlet instance ID
		if (!portletInstanceID) {
			if (frameElem && frameElem.tagName && frameElem.tagName.toLowerCase() == "iframe") {
				porletInstanceID = tipHelper.getPortletInstanceID(frameElem);
			}			
		}
		if (!portletInstanceID) {
			portletInstanceID = this._NOPORTLETINSTANCEID;
		}
		// Check if we had to look in an IFRAME for whether we are an iWidget
		if (frameElem && !isiwidget) {
			if (frameElem.tagName && frameElem.tagName.toLowerCase() == "iframe") {
				isiwidget = tipHelper.isiWidget(frameElem);
			}						
		}
		
		this.is_iWidget = isiwidget;
		this.iscPortletNamespace = namespace;
		this.iscPortletInstanceID = portletInstanceID;
	},
	
	createIconClass: function(/*String*/ id, /*String*/ image) { 
		// summary:
		//		Creates an iconClass on the fly to accommodate provider URLs
		// description:
		//		Creates an iconClass on the fly to accommodate provider URLs
		// id:
		//		Unique identifier for the icon class
		// image:
		//		Image name and path
		var style = dojo.doc.createElement("style");
		dojo.attr(style, "type", "text/css");
		dojo.attr(style, "id", id);

		var iconClassContents = "." + id + " { background-image: url("+image+"); width: 22px; height: 22px; display: block; background-repeat: no-repeat; vertical-align: middle;}";
		// IE handles innerHTML differently
		if (dojo.isIE) {
			style.styleSheet.cssText = iconClassContents;
		} else {
			style.innerHTML = iconClassContents;
		}
		var head = dojo.doc.getElementsByTagName("head")[0];   
            head.appendChild(style);
	},

	tasksAsMenuJson: function(/*ibm.tivoli.navmodel.Task[]*/ tasks) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// tasks:
		//		
		
		// convert tasks to TIPContextMenu Json format
		var menuItems = [];
		var subMenu = null;
		var iconClass = "";
		for (var i = 0; i < tasks.length; i++) {
			if (dojo.doc.getElementById(ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.replaceSpecialChars('_',tasks[i].getId())) && 
				tasks[i].getCategory() != "toolbar") {
				iconClass = ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.replaceSpecialChars('_',tasks[i].getId());
			} else if (tasks[i].getCategory() == "menu" && tasks[i].getImage() != undefined) {
				this.createIconClass(ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.replaceSpecialChars('_',tasks[i].getId()), ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.cleanUpImagePath(tasks[i].getImage()));
				iconClass = ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.replaceSpecialChars('_',tasks[i].getId());
			} 
		    if (tasks[i].isSeparator()) {
		    	menuItems.push( { "actionID":"_separator_" });
	    	} 
	    	else if (tasks[i].isCascade()) {
	    		subMenu = this._getSubMenuTasksAsMenuJson(tasks[i]);
	    		menuItems.push({
	    						"label":tasks[i].getLabel(),
	    						"iconClass":iconClass,
	    						"submenu_items":subMenu
	    						});
	    	}
	    	else {
	    		menuItems.push({
	    							"label":tasks[i].getLabel(),
	    						      "iconClass":iconClass,
	    							"actionID":tasks[i].getId(),
	    							"disabled":!(tasks[i].isEnabled()),
	    							"category":tasks[i].getCategory(),
	    							"type":tasks[i].getType(),
	    							"isServer":tasks[i].isServer(),
	    							"isResolved":tasks[i].isResolved()
	    						});
	    		var thisItem = menuItems[menuItems.length - 1];
	    		if (tasks[i].isUrlLaunch()) {
	    			thisItem.launchType = "WEB_URL";
	    			thisItem.uri = tasks[i].getLaunchUrl();
	    			if (tasks[i].getLaunchBrowserWindowId()) {  // 32355
	    				thisItem.browserwindowid = tasks[i].getLaunchBrowserWindowId();  			
	    			}    			
	    		} else if (tasks[i].isPageLaunch()) {
	    			thisItem.launchType = "PORTAL_PAGE";
	    			thisItem.portletPageID = tasks[i].getLaunchPageId();
	    			thisItem.parameters = tasks[i].getLaunchApplicationParms();
	    		} else if (tasks[i].isPortletLaunch()) {
	    			thisItem.launchType = "PORTLET";
	    			thisItem.portletName = tasks[i].getLaunchPortletName();
	    			thisItem.portletAppUID = tasks[i].getLaunchPortletAppUID();
	    			thisItem.parameters = tasks[i].getLaunchApplicationParms();
	    		} else if (tasks[i].isClientEvent()) {
	    			thisItem.launchType = "CLIENT_EVENT";
	    			thisItem.event = tasks[i].getLaunchEvent();
	    			thisItem.parameters = tasks[i].getLaunchApplicationParms();
	    		} else if (tasks[i].isJavascript()) {
	    			thisItem.launchType = "JavaScript";
	    			thisItem.uri = tasks[i].getLaunchJavascript();
	    		}
	    	}
	    }
		return menuItems;
	},
	
	_getSubMenuTasksAsMenuJson: function(/*ibm.tivoli.navmodel.Task*/ task) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// task:
		//		
		
		// process submenu tasks
		var menuItems = [];
		var childTasks = task.getChildTasks();
		if (childTasks != null) {
			menuItems = this.tasksAsMenuJson(childTasks);
		}
		return menuItems;
	}

});
});

},
'dijit/form/Button':function(){
define([
	"require",
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.toggle
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/lang", // lang.trim
	"dojo/ready",
	"./_FormWidget",
	"./_ButtonMixin",
	"dojo/text!./templates/Button.html"
], function(require, declare, domClass, kernel, lang, ready, _FormWidget, _ButtonMixin, template){

/*=====
	var _FormWidget = dijit.form._FormWidget;
	var _ButtonMixin = dijit.form._ButtonMixin;
=====*/

// module:
//		dijit/form/Button
// summary:
//		Button widget

// Back compat w/1.6, remove for 2.0
if(!kernel.isAsync){
	ready(0, function(){
		var requires = ["dijit/form/DropDownButton", "dijit/form/ComboButton", "dijit/form/ToggleButton"];
		require(requires);	// use indirection so modules not rolled into a build
	});
}

return declare("dijit.form.Button", [_FormWidget, _ButtonMixin], {
	// summary:
	//		Basically the same thing as a normal HTML button, but with special styling.
	// description:
	//		Buttons can display a label, an icon, or both.
	//		A label should always be specified (through innerHTML) or the label
	//		attribute.  It can be hidden via showLabel=false.
	// example:
	// |	<button data-dojo-type="dijit.form.Button" onClick="...">Hello world</button>
	//
	// example:
	// |	var button1 = new dijit.form.Button({label: "hello world", onClick: foo});
	// |	dojo.body().appendChild(button1.domNode);

	// showLabel: Boolean
	//		Set this to true to hide the label text and display only the icon.
	//		(If showLabel=false then iconClass must be specified.)
	//		Especially useful for toolbars.
	//		If showLabel=true, the label will become the title (a.k.a. tooltip/hint) of the icon.
	//
	//		The exception case is for computers in high-contrast mode, where the label
	//		will still be displayed, since the icon doesn't appear.
	showLabel: true,

	// iconClass: String
	//		Class to apply to DOMNode in button to make it display an icon
	iconClass: "dijitNoIcon",
	_setIconClassAttr: { node: "iconNode", type: "class" },

	baseClass: "dijitButton",

	templateString: template,

	// Map widget attributes to DOMNode attributes.
	_setValueAttr: "valueNode",

	_onClick: function(/*Event*/ e){
		// summary:
		//		Internal function to handle click actions
		var ok = this.inherited(arguments);
		if(ok){
			if(this.valueNode){
				this.valueNode.click();
				e.preventDefault(); // cancel BUTTON click and continue with hidden INPUT click
				// leave ok = true so that subclasses can do what they need to do
			}
		}
		return ok;
	},

	_fillContent: function(/*DomNode*/ source){
		// Overrides _Templated._fillContent().
		// If button label is specified as srcNodeRef.innerHTML rather than
		// this.params.label, handle it here.
		// TODO: remove the method in 2.0, parser will do it all for me
		if(source && (!this.params || !("label" in this.params))){
			var sourceLabel = lang.trim(source.innerHTML);
			if(sourceLabel){
				this.label = sourceLabel; // _applyAttributes will be called after buildRendering completes to update the DOM
			}
		}
	},

	_setShowLabelAttr: function(val){
		if(this.containerNode){
			domClass.toggle(this.containerNode, "dijitDisplayNone", !val);
		}
		this._set("showLabel", val);
	},

	setLabel: function(/*String*/ content){
		// summary:
		//		Deprecated.  Use set('label', ...) instead.
		kernel.deprecated("dijit.form.Button.setLabel() is deprecated.  Use set('label', ...) instead.", "", "2.0");
		this.set("label", content);
	},

	_setLabelAttr: function(/*String*/ content){
		// summary:
		//		Hook for set('label', ...) to work.
		// description:
		//		Set the label (text) of the button; takes an HTML string.
		//		If the label is hidden (showLabel=false) then and no title has
		//		been specified, then label is also set as title attribute of icon.
		this.inherited(arguments);
		if(!this.showLabel && !("title" in this.params)){
			this.titleNode.title = lang.trim(this.containerNode.innerText || this.containerNode.textContent || '');
		}
	}
});


});


},
'gridx/modules/pagination/DropDownPager':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/store/Memory",
	"./_PagerBase",
	'../../core/util',
	"dojo/text!../../templates/PaginationBarDD.html"
], function(declare, lang, Store, _PagerBase, util, barTemplate){

	return declare(_PagerBase, {
		templateString: barTemplate,
	
		refresh: function(){
			this._createDescription();
			this._createPageStepper();
			this._createPageSizeSwitch();
		},
	
		_onSwitchPage: function(page){
			this._pageStepperSelect.set('value', page + 1);
		},

		_onChangePageSize: function(){
			this._createPageStepper();
		},

		_onSizeChange: function(){
			this._createDescription();
			this._createPageStepper();
		},
	
		_createPageStepper: function(){
			var t = this, mod = t.module;
			if(t._toggleNode('gridxPagerStepper', mod._exist(t.position, 'stepper'))){
				var items = [],
					selectedItem,
					p = t.pagination,
					pageCount = p.pageCount(),
					currentPage = p.currentPage(),
					stepper = t._pageStepperSelect,
					i, v, item;
				for(i = 0; i < pageCount; ++i){
					v = i + 1;
					item = {
						id: v,
						label: v,
						value: v
					};
					items.push(item);
					if(currentPage == i){
						selectedItem = item;
					}
				}
				var store = new Store({data: items});
				if(!stepper){
					var label = t._pageStepperContainer.firstChild,
						cls = mod.arg('stepperClass'),
						props = lang.mixin({
							store: store,
							searchAttr: 'label',
							item: selectedItem,
							'class': 'gridxPagerStepperWidget',
							onChange: function(page){
								p.gotoPage(page - 1);
							}
						}, mod.arg('stepperProps') || {});
					stepper = t._pageStepperSelect = new cls(props);
					stepper.placeAt(t._pageStepperContainer, "last");
					stepper.startup();
					label.setAttribute('for', stepper.focusNode.id);
				}else{
					stepper.set('store', store);
					stepper.set('value', currentPage + 1);
				}
				stepper.set('disabled', pageCount <= 1);
			}
		},
	
		_createPageSizeSwitch: function(){
			var t = this, mod = t.module;
			if(t._toggleNode('gridxPagerSizeSwitch', mod._exist(t.position, 'sizeSwitch'))){
				var options = [],
					p = t.pagination,
					currentSize = p.pageSize(), 
					nlsAll = mod.arg('pageSizeAllText', t.pageSizeAll),
					sizeSwitch = t._sizeSwitchSelect,
					sizes = mod.arg('sizes');
				for(var i = 0, len = sizes.length; i < len; ++i){
					var pageSize = sizes[i],
						isAll = !(pageSize > 0);
					options.push({
						label: isAll ? nlsAll : pageSize,
						value: isAll ? -1 : pageSize,
						selected: currentSize == pageSize || (isAll && p.isAll())
					});
				}
				if(!sizeSwitch){
					var label = t._sizeSwitchContainer.firstChild,
						cls = mod.arg('sizeSwitchClass'),
						props = lang.mixin({
							options: options,
							'class': 'gridxPagerSizeSwitchWidget',
							onChange: function(ps){
								p.setPageSize(ps < 0 ? 0 : ps);
							}
						}, mod.arg('sizeSwitchProps') || {});
					sizeSwitch = t._sizeSwitchSelect = new cls(props);
					sizeSwitch.placeAt(t._sizeSwitchContainer, "last");
					sizeSwitch.startup();
					label.setAttribute('for', sizeSwitch.focusNode.id);
				}else{
					sizeSwitch.removeOption(sizeSwitch.getOptions());
					sizeSwitch.addOption(options);
				}
			}
		},
	
		_initFocus: function(){
			var t = this,
				g = t.module.grid,
				focus = g.focus,
				pos = t.position,
				fp = t.focusPriority;
			if(focus){
				focus.registerArea({
					name: pos + 'PageStepper',
					priority: fp,
					focusNode: t._pageStepperContainer,
					doFocus: function(evt){
						util.stopEvent(evt);
						t._pageStepperSelect.focus();
						return true;
					}
				});
				focus.registerArea({
					name: pos + 'PageSizeSwitch',
					priority: fp + 0.001,
					focusNode: t._sizeSwitchContainer,
					doFocus: function(evt){
						util.stopEvent(evt);
						t._sizeSwitchSelect.focus();
						return true;
					}
				});
			}
		}
	});
});

},
'dijit/_Contained':function(){
define([
	"dojo/_base/declare", // declare
	"./registry"	// registry.getEnclosingWidget(), registry.byNode()
], function(declare, registry){

	// module:
	//		dijit/_Contained
	// summary:
	//		Mixin for widgets that are children of a container widget

	return declare("dijit._Contained", null, {
		// summary:
		//		Mixin for widgets that are children of a container widget
		//
		// example:
		// | 	// make a basic custom widget that knows about it's parents
		// |	declare("my.customClass",[dijit._Widget,dijit._Contained],{});

		_getSibling: function(/*String*/ which){
			// summary:
			//      Returns next or previous sibling
			// which:
			//      Either "next" or "previous"
			// tags:
			//      private
			var node = this.domNode;
			do{
				node = node[which+"Sibling"];
			}while(node && node.nodeType != 1);
			return node && registry.byNode(node);	// dijit._Widget
		},

		getPreviousSibling: function(){
			// summary:
			//		Returns null if this is the first child of the parent,
			//		otherwise returns the next element sibling to the "left".

			return this._getSibling("previous"); // dijit._Widget
		},

		getNextSibling: function(){
			// summary:
			//		Returns null if this is the last child of the parent,
			//		otherwise returns the next element sibling to the "right".

			return this._getSibling("next"); // dijit._Widget
		},

		getIndexInParent: function(){
			// summary:
			//		Returns the index of this widget within its container parent.
			//		It returns -1 if the parent does not exist, or if the parent
			//		is not a dijit._Container

			var p = this.getParent();
			if(!p || !p.getIndexOfChild){
				return -1; // int
			}
			return p.getIndexOfChild(this); // int
		}
	});
});

},
'dojox/grid/TreeGrid':function(){
define([
	"dojo/_base/kernel",
	"../main",
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/_base/event",
	"dojo/dom-attr",
	"dojo/dom-class",
	"dojo/query",
	"dojo/keys",
	"dijit/tree/ForestStoreModel",
	"./DataGrid",
	"./_Layout",
	"./_FocusManager",
	"./_RowManager",
	"./_EditManager",
	"./TreeSelection",
	"./cells/tree",
	"./_TreeView"
], function(dojo, dojox, declare, array, lang, event, domAttr, domClass, query, keys, ForestStoreModel, 
	DataGrid, _Layout, _FocusManager, _RowManager, _EditManager, TreeSelection, TreeCell){
		
dojo.experimental("dojox.grid.TreeGrid");

var _TreeAggregator = declare("dojox.grid._TreeAggregator", null, {
	cells: [],
	grid: null,
	childFields: [],
	
	constructor: function(kwArgs){
		this.cells = kwArgs.cells || [];
		this.childFields = kwArgs.childFields || [];
		this.grid = kwArgs.grid;
		this.store = this.grid.store;
	},
	_cacheValue: function(cache, id, value){
		cache[id] = value;
		return value;
	},
	clearSubtotalCache: function(){
		// summary:
		//		Clears the subtotal cache so that we are forced to recalc it
		//		(or reread it) again.  This is needed, for example, when
		//		column order is changed.
		if(this.store){
			delete this.store._cachedAggregates;
		}
	},
	
	cnt: function(cell, level, item){
		// summary:
		//		calculates the count of the children of item at the given level
		var total = 0;
		var store = this.store;
		var childFields = this.childFields;
		if(childFields[level]){
			var children = store.getValues(item, childFields[level]);
			if (cell.index <= level + 1){
				total = children.length;
			}else{
				array.forEach(children, function(c){
					total += this.getForCell(cell, level + 1, c, "cnt");
				}, this);
			}
		}else{
			total = 1;
		}
		return total;
	},
	sum: function(cell, level, item){
		// summary:
		//		calculates the sum of the children of item at the given level
		var total = 0;
		var store = this.store;
		var childFields = this.childFields;
		if(childFields[level]){
			array.forEach(store.getValues(item, childFields[level]), function(c){
				total += this.getForCell(cell, level + 1, c, "sum");
			}, this);
		}else{
			total += store.getValue(item, cell.field);
		}
		return total;
	},
	value: function(cell, level, item){
		// summary:
		//		Empty function so that we can set "aggregate='value'" to
		//		force loading from the data - and bypass calculating
	},
	getForCell: function(cell, level, item, type){
		// summary:
		//		Gets the value of the given cell at the given level and type.
		//		type can be one of "sum", "cnt", or "value".  If itemAggregates
		//		is set and can be used, it is used instead.  Values are also
		//		cached to prevent calculating them too often.
		var store = this.store;
		if(!store || !item || !store.isItem(item)){ return ""; }
		var storeCache = store._cachedAggregates = store._cachedAggregates || {};
		var id = store.getIdentity(item);
		var itemCache = storeCache[id] = storeCache[id] || [];
		if(!cell.getOpenState){
			cell = this.grid.getCell(cell.layoutIndex + level + 1);
		}
		var idx = cell.index;
		var idxCache = itemCache[idx] = itemCache[idx] || {};
		type = (type || (cell.parentCell ? cell.parentCell.aggregate : "sum"))||"sum";
		var attr = cell.field;
		if(attr == store.getLabelAttributes()[0]){
			// If our attribute is one of the label attributes, we should
			// use cnt instead (since it makes no sense to do a sum of labels)
			type = "cnt";
		}
		var typeCache = idxCache[type] = idxCache[type] || [];

		// See if we have it in our cache immediately for easy returning
		if(typeCache[level] != undefined){
			return typeCache[level];
		}

		// See if they have specified a valid field
		var field = ((cell.parentCell && cell.parentCell.itemAggregates) ?
							cell.parentCell.itemAggregates[cell.idxInParent] : "")||"";
		if(field && store.hasAttribute(item, field)){
			return this._cacheValue(typeCache, level, store.getValue(item, field));
		}else if(field){
			return this._cacheValue(typeCache, level, 0);
		}
		
		// Calculate it
		return this._cacheValue(typeCache, level, this[type](cell, level, item));
	}
});

var _TreeLayout = declare("dojox.grid._TreeLayout", _Layout, {
	// Whether or not we are collapsable - this is calculated when we
	// set our structure.
	_isCollapsable: false,
	
	_getInternalStructure: function(inStructure){
		//	Create a "Tree View" with 1 row containing references for
		//		each column (recursively)
		var g = this.grid;
		
		var s = inStructure;
		var cells = s[0].cells[0];
		var tree = {
			type: "dojox.grid._TreeView",
			cells: [[]]
		};
		var cFields = [];
		var maxLevels = 0;
		var getTreeCells = function(parentCell, level){
			var children = parentCell.children;
			var cloneTreeCell = function(originalCell, idx){
				var k, n = {};
				for(k in originalCell){
					n[k] = originalCell[k];
				}
				n = lang.mixin(n, {
					level: level,
					idxInParent: level > 0 ? idx : -1,
					parentCell: level > 0 ? parentCell : null
				});
				return n;
			};
			var ret = [];
			array.forEach(children, function(c, idx){
				if("children" in c){
					cFields.push(c.field);
					var last = ret[ret.length - 1];
					last.isCollapsable = true;
					c.level = level;
					ret = ret.concat(getTreeCells(c, level + 1));
				}else{
					ret.push(cloneTreeCell(c, idx));
				}
			});
			maxLevels = Math.max(maxLevels, level);
			return ret;
		};
		var tCell = {children: cells, itemAggregates: []};
		tree.cells[0] = getTreeCells(tCell, 0);
		g.aggregator = new _TreeAggregator({cells: tree.cells[0],
														grid: g,
														childFields: cFields});
		if(g.scroller && g.defaultOpen){
			g.scroller.defaultRowHeight = g.scroller._origDefaultRowHeight * (2 * maxLevels + 1);
		}
		return [ tree ];
	},

	setStructure: function(inStructure){
		// Mangle the structure a bit and make it work as desired
		var s = inStructure;
		var g = this.grid;
		// Only supporting single-view, single row or else we
		// are not collapsable
		if(g && g.treeModel && !array.every(s, function(i){
			return ("cells" in i);
		})){
			s = arguments[0] = [{cells:[s]}];
		}
		if(s.length == 1 && s[0].cells.length == 1){
			if(g && g.treeModel){
				s[0].type = "dojox.grid._TreeView";
				this._isCollapsable = true;
				s[0].cells[0][(this.grid.treeModel?this.grid.expandoCell:0)].isCollapsable = true;
			}else{
				var childCells = array.filter(s[0].cells[0], function(c){
					return ("children" in c);
				});
				if(childCells.length === 1){
					this._isCollapsable = true;
				}
			}
		}
		if(this._isCollapsable && (!g || !g.treeModel)){
			arguments[0] = this._getInternalStructure(s);
		}
		this.inherited(arguments);
	},

	addCellDef: function(inRowIndex, inCellIndex, inDef){
		var obj = this.inherited(arguments);
		return lang.mixin(obj, TreeCell);
	}
});

var TreePath = declare("dojox.grid.TreePath", null, {
	level: 0,
	_str: "",
	_arr: null,
	grid: null,
	store: null,
	cell: null,
	item: null,

	constructor: function(/*String|Integer[]|Integer|dojox.grid.TreePath*/ path, /*dojox.grid.TreeGrid*/ grid){
		if(lang.isString(path)){
			this._str = path;
			this._arr = array.map(path.split('/'), function(item){ return parseInt(item, 10); });
		}else if(lang.isArray(path)){
			this._str = path.join('/');
			this._arr = path.slice(0);
		}else if(typeof path == "number"){
			this._str = String(path);
			this._arr = [path];
		}else{
			this._str = path._str;
			this._arr = path._arr.slice(0);
		}
		this.level = this._arr.length-1;
		this.grid = grid;
		this.store = this.grid.store;
		if(grid.treeModel){
			this.cell = grid.layout.cells[grid.expandoCell];
		}else{
			this.cell = grid.layout.cells[this.level];
		}
	},
	item: function(){
		// summary:
		//	gets the dojo.data item associated with this path
		if(!this._item){
			this._item = this.grid.getItem(this._arr);
		}
		return this._item;
	},
	compare: function(path /*dojox.grid.TreePath|String|Array*/){
		// summary:
		//	compares two paths
		if(lang.isString(path) || lang.isArray(path)){
			if(this._str == path){ return 0; }
			if(path.join && this._str == path.join('/')){ return 0; }
			path = new TreePath(path, this.grid);
		}else if(path instanceof TreePath){
			if(this._str == path._str){ return 0; }
		}
		for(var i=0, l=(this._arr.length < path._arr.length ? this._arr.length : path._arr.length); i<l; i++){
			if(this._arr[i]<path._arr[i]){ return -1; }
			if(this._arr[i]>path._arr[i]){ return 1; }
		}
		if(this._arr.length<path._arr.length){ return -1; }
		if(this._arr.length>path._arr.length){ return 1; }
		return 0;
	},
	isOpen: function(){
		// summary:
		//	Returns the open state of this cell.
		return this.cell.openStates && this.cell.getOpenState(this.item());
	},
	previous: function(){
		// summary:
		//	Returns the path that is before this path in the
		//	grid. If no path is found, returns null.
		var new_path = this._arr.slice(0);

		if(this._str == "0"){
			return null;
		}

		var last = new_path.length-1;

		if(new_path[last] === 0){
			new_path.pop();
			return new TreePath(new_path, this.grid);
		}

		new_path[last]--;
		var path = new TreePath(new_path, this.grid);
		return path.lastChild(true);
	},
	next: function(){
		// summary:
		//	Returns the next path in the grid.  If no path
		//	is found, returns null.
		var new_path = this._arr.slice(0);

		if(this.isOpen()){
			new_path.push(0);
		}else{
			new_path[new_path.length-1]++;
			for(var i=this.level; i>=0; i--){
				var item = this.grid.getItem(new_path.slice(0, i+1));
				if(i>0){
					if(!item){
						new_path.pop();
						new_path[i-1]++;
					}
				}else{
					if(!item){
						return null;
					}
				}
			}
		}

		return new TreePath(new_path, this.grid);
	},
	children: function(alwaysReturn){
		// summary:
		//	Returns the child data items of this row.  If this
		//	row isn't open and alwaysReturn is falsey, returns null.
		if(!this.isOpen()&&!alwaysReturn){
			return null;
		}
		var items = [];
		var model = this.grid.treeModel;
		if(model){
			var item = this.item();
			var store = model.store;
			if(!model.mayHaveChildren(item)){
				return null;
			}
			array.forEach(model.childrenAttrs, function(attr){
				items = items.concat(store.getValues(item, attr));
			});
		}else{
			items = this.store.getValues(this.item(), this.grid.layout.cells[this.cell.level+1].parentCell.field);
			if(items.length>1&&this.grid.sortChildItems){
				var sortProps = this.grid.getSortProps();
				if(sortProps&&sortProps.length){
					var attr = sortProps[0].attribute,
						grid = this.grid;
					if(attr&&items[0][attr]){
						var desc = !!sortProps[0].descending;
						items = items.slice(0); // don't touch the array in the store, make a copy
						items.sort(function(a, b){
							return grid._childItemSorter(a, b, attr, desc);
						});
					}
				}
			}
		}
		return items;
	},
	childPaths: function(){
		var childItems = this.children();
		if(!childItems){
			return [];
		}
		return array.map(childItems, function(item, index){
			return new TreePath(this._str + '/' + index, this.grid);
		}, this);
	},
	parent: function(){
		// summary:
		//	Returns the parent path of this path.  If this is a
		//	top-level row, returns null.
		if(this.level === 0){
			return null;
		}
		return new TreePath(this._arr.slice(0, this.level), this.grid);
	},
	lastChild: function(/*Boolean?*/ traverse){
		// summary:
		//	Returns the last child row below this path.  If traverse
		//	is true, will traverse down to find the last child row
		//	of this branch.  If there are no children, returns itself.
		var children = this.children();
		if(!children || !children.length){
			return this;
		}
		var path = new TreePath(this._str + "/" + String(children.length-1), this.grid);
		if(!traverse){
			return path;
		}
		return path.lastChild(true);
	},
	toString: function(){
		return this._str;
	}
});

var _TreeFocusManager = declare("dojox.grid._TreeFocusManager", _FocusManager, {
	setFocusCell: function(inCell, inRowIndex){
		if(inCell && inCell.getNode(inRowIndex)){
			this.inherited(arguments);
		}
	},
	isLastFocusCell: function(){
		if(this.cell && this.cell.index == this.grid.layout.cellCount-1){
			var path = new TreePath(this.grid.rowCount-1, this.grid);
			path = path.lastChild(true);
			return this.rowIndex == path._str;
		}
		return false;
	},
	next: function(){
		// summary:
		//	focus next grid cell
		if(this.cell){
			var row=this.rowIndex, col=this.cell.index+1, cc=this.grid.layout.cellCount-1;
			var path = new TreePath(this.rowIndex, this.grid);
			if(col > cc){
				var new_path = path.next();
				if(!new_path){
					col--;
				}else{
					col = 0;
					path = new_path;
				}
			}
			if(this.grid.edit.isEditing()){ //when editing, only navigate to editable cells
				var nextCell = this.grid.getCell(col);
				if (!this.isLastFocusCell() && !nextCell.editable){
					this._focusifyCellNode(false);
					this.cell=nextCell;
					this.rowIndex=path._str;
					this.next();
					return;
				}
			}
			this.setFocusIndex(path._str, col);
		}
	},
	previous: function(){
		// summary:
		//	focus previous grid cell
		if(this.cell){
			var row=(this.rowIndex || 0), col=(this.cell.index || 0) - 1;
			var path = new TreePath(row, this.grid);
			if(col < 0){
				var new_path = path.previous();
				if(!new_path){
					col = 0;
				}else{
					col = this.grid.layout.cellCount-1;
					path = new_path;
				}
			}
			if(this.grid.edit.isEditing()){ //when editing, only navigate to editable cells
				var prevCell = this.grid.getCell(col);
				if (!this.isFirstFocusCell() && !prevCell.editable){
					this._focusifyCellNode(false);
					this.cell=prevCell;
					this.rowIndex=path._str;
					this.previous();
					return;
				}
			}
			this.setFocusIndex(path._str, col);
		}
	},
	move: function(inRowDelta, inColDelta){
		if(this.isNavHeader()){
			this.inherited(arguments);
			return;
		}
		if(!this.cell){ return; }
		// Handle grid proper.
		var sc = this.grid.scroller,
			r = this.rowIndex,
			rc = this.grid.rowCount-1,
			path = new TreePath(this.rowIndex, this.grid);
		if(inRowDelta){
			var row;
			if(inRowDelta>0){
				path = path.next();
				row = path._arr[0];
				if(row > sc.getLastPageRow(sc.page)){
					//need to load additional data, let scroller do that
					this.grid.setScrollTop(this.grid.scrollTop+sc.findScrollTop(row)-sc.findScrollTop(r));
				}
			}else if(inRowDelta<0){
				path = path.previous();
				row = path._arr[0];
				if(row <= sc.getPageRow(sc.page)){
					//need to load additional data, let scroller do that
					this.grid.setScrollTop(this.grid.scrollTop-sc.findScrollTop(r)-sc.findScrollTop(row));
				}
			}
		}
		var cc = this.grid.layout.cellCount-1,
		i = this.cell.index,
		col = Math.min(cc, Math.max(0, i+inColDelta));
		var cell = this.grid.getCell(col);
		var colDir = inColDelta < 0 ? -1 : 1;
		while(col>=0 && col < cc && cell && cell.hidden === true){
			// skip hidden cells
			col += colDir;
			cell = this.grid.getCell(col);
		}
		if (!cell || cell.hidden === true){
			// don't change col if would move to hidden
			col = i;
		}
		if(inRowDelta){
			this.grid.updateRow(r);
		}
		this.setFocusIndex(path._str, col);
	}
});

var TreeGrid = declare("dojox.grid.TreeGrid", DataGrid, {
	// summary:
	//		A grid that supports nesting rows - it provides an expando function
	//		similar to dijit.Tree.  It also provides mechanisms for aggregating
	//		the values of subrows
	//
	// description:
	//		TreeGrid currently only works on "simple" structures.  That is,
	//		single-view structures with a single row in them.
	//
	//		The TreeGrid works using the concept of "levels" - level 0 are the
	//		top-level items.
	
	// defaultOpen: Boolean
	//		Whether or not we default to open (all levels).  This defaults to
	//		false for grids with a treeModel.
	defaultOpen: true,

	// sortChildItems: Boolean
	// 		If true, child items will be returned sorted according to the sorting
	// 		properties of the grid.
	sortChildItems: false,

	// openAtLevels: Array
	//		Which levels we are open at (overrides defaultOpen for the values
	//		that exist here).  Its values can be a boolean (true/false) or an
	//		integer (for the # of children to be closed if there are more than
	//		that)
	openAtLevels: [],
	
	// treeModel: dijit.tree.ForestStoreModel
	//		A dijit.Tree model that will be used instead of using aggregates.
	//		Setting this value will make the TreeGrid behave like a columnar
	//		tree.  When setting this value, defaultOpen will default to false,
	//		and openAtLevels will be ignored.
	treeModel: null,
	
	// expandoCell: Integer
	//		When used in conjunction with a treeModel (see above), this is a 0-based
	//		index of the cell in which to place the actual expando
	expandoCell: 0,
	
	// private values
	// aggregator: Object
	//		The aggregator class - it will be populated automatically if we
	//		are a collapsable grid
	aggregator: null,


	// Override this to get our "magic" layout
	_layoutClass: _TreeLayout,

	createSelection: function(){
		this.selection = new TreeSelection(this);
	},

	_childItemSorter: function(a, b, attribute, descending){
		var av = this.store.getValue(a, attribute);
		var bv = this.store.getValue(b, attribute);
		if(av != bv){
			return av < bv == descending ? 1 : -1;
		}
		return 0;
	},

	_onNew: function(item, parentInfo){
		if(!parentInfo || !parentInfo.item){
			this.inherited(arguments);
		}else{
			var idx = this.getItemIndex(parentInfo.item);
			if(typeof idx == "string"){
				this.updateRow(idx.split('/')[0]);
			}else if(idx > -1){
				this.updateRow(idx);
			}
		}
	},

	_onSet: function(item, attribute, oldValue, newValue){
		this._checkUpdateStatus();
		if(this.aggregator){
			this.aggregator.clearSubtotalCache();
		}
		var idx = this.getItemIndex(item);
		if(typeof idx == "string"){
			this.updateRow(idx.split('/')[0]);
		}else if(idx > -1){
			this.updateRow(idx);
		}
	},

	_onDelete: function(item){
		this._cleanupExpandoCache(this._getItemIndex(item, true), this.store.getIdentity(item), item);
		this.inherited(arguments);
	},

	_cleanupExpandoCache: function(index, identity, item){},

	_addItem: function(item, index, noUpdate, dontUpdateRoot){
		// add our root items to the root of the model's children
		// list since we don't query the model
		if(!dontUpdateRoot && this.model && array.indexOf(this.model.root.children, item) == -1){
			this.model.root.children[index] = item;
		}
		this.inherited(arguments);
	},

	getItem: function(/*integer|Array|String*/ idx){
		// summary:
		//		overridden so that you can pass in a '/' delimited string of indexes to get the
		//		item based off its path...that is, passing in "1/3/2" will get the
		//		3rd (0-based) child from the 4th child of the 2nd top-level item.
		var isArray = lang.isArray(idx);
		if(lang.isString(idx) && idx.indexOf('/')){
			idx = idx.split('/');
			isArray = true;
		}
		if(isArray && idx.length == 1){
			idx = idx[0];
			isArray = false;
		}
		if(!isArray){
			return DataGrid.prototype.getItem.call(this, idx);
		}
		var s = this.store;
		var itm = DataGrid.prototype.getItem.call(this, idx[0]);
		var cf, i, j;
		if(this.aggregator){
			cf = this.aggregator.childFields||[];
			if(cf){
				for(i = 0; i < idx.length - 1 && itm; i++){
					if(cf[i]){
						itm = (s.getValues(itm, cf[i])||[])[idx[i + 1]];
					}else{
						itm = null;
					}
				}
			}
		}else if(this.treeModel){
			cf = this.treeModel.childrenAttrs||[];
			if(cf&&itm){
				for(i=1, il=idx.length; (i<il) && itm; i++) {
					for(j=0, jl=cf.length; j<jl; j++) {
						if(cf[j]){
							itm = (s.getValues(itm, cf[j])||[])[idx[i]];
						}else{
							itm = null;
						}
						if(itm){ break; }
					}
				}
			}
		}
		return itm || null;
	},

	_getItemIndex: function(item, isDeleted){
		if(!isDeleted && !this.store.isItem(item)){
			return -1;
		}
		var idx = this.inherited(arguments);
		if(idx == -1){
			var idty = this.store.getIdentity(item);
			return this._by_idty_paths[idty] || -1;
		}
		return idx;
	},
	
	postMixInProperties: function(){
		if(this.treeModel && !("defaultOpen" in this.params)){
			// Default open to false for tree models, true for other tree
			// grids.
			this.defaultOpen = false;
		}
		var def = this.defaultOpen;
		this.openAtLevels = array.map(this.openAtLevels, function(l){
			if(typeof l == "string"){
				switch(l.toLowerCase()){
					case "true":
						return true;
						break;
					case "false":
						return false;
						break;
					default:
						var r = parseInt(l, 10);
						if(isNaN(r)){
							return def;
						}
						return r;
						break;
				}
			}
			return l;
		});
		this._by_idty_paths = {};
		this.inherited(arguments);
	},
	
    postCreate: function(){
        this.inherited(arguments);
		if(this.treeModel){
			this._setModel(this.treeModel);
		}
    },

	setModel: function(treeModel){
		this._setModel(treeModel);
		this._refresh(true);
	},
	
	_setModel: function(treeModel){
		if(treeModel && (!ForestStoreModel || !(treeModel instanceof ForestStoreModel))){
			throw new Error("dojox.grid.TreeGrid: treeModel must be an instance of dijit.tree.ForestStoreModel");
		}
		this.treeModel = treeModel;
		domClass.toggle(this.domNode, "dojoxGridTreeModel", this.treeModel ? true : false);
		this._setQuery(treeModel ? treeModel.query : null);
		this._setStore(treeModel ? treeModel.store : null);
	},

	createScroller: function(){
		this.inherited(arguments);
		this.scroller._origDefaultRowHeight = this.scroller.defaultRowHeight;
	},
	
	createManagers: function(){
		// summary:
		//		create grid managers for various tasks including rows, focus, selection, editing

		// row manager
		this.rows = new _RowManager(this);
		// focus manager
		this.focus = new _TreeFocusManager(this);
		// edit manager
		this.edit = new _EditManager(this);
	},

	_setStore: function(store){
		this.inherited(arguments);
		if(this.treeModel&&!this.treeModel.root.children){
			this.treeModel.root.children = [];
		}
		if(this.aggregator){
			this.aggregator.store = store;
		}
	},
	
	getDefaultOpenState: function(cellDef, item){
		// summary:
		//		Returns the default open state for the given definition and item
		//		It reads from the openAtLevels and defaultOpen values of the
		//		grid to calculate if the given item should default to open or
		//		not.
		var cf;
		var store = this.store;
		if(this.treeModel){ return this.defaultOpen; }
		if(!cellDef || !store || !store.isItem(item) ||
				!(cf = this.aggregator.childFields[cellDef.level])){
			return this.defaultOpen;
		}
		if(this.openAtLevels.length > cellDef.level){
			var dVal = this.openAtLevels[cellDef.level];
			if(typeof dVal == "boolean"){
				return dVal;
			}else if(typeof dVal == "number"){
				return (store.getValues(item, cf).length <= dVal);
			}
		}
		return this.defaultOpen;
	},
	onStyleRow: function(row){
		if(!this.layout._isCollapsable){
			this.inherited(arguments);
			return;
		}
		var base = domAttr.get(row.node, 'dojoxTreeGridBaseClasses');
		if(base){
			row.customClasses = base;
		}
		var i = row;
		var tagName = i.node.tagName.toLowerCase();
		i.customClasses += (i.odd?" dojoxGridRowOdd":"") +
						   (i.selected&&tagName=='tr'?" dojoxGridRowSelected":"") +
						   (i.over&&tagName=='tr'?" dojoxGridRowOver":"");
		this.focus.styleRow(i);
		this.edit.styleRow(i);
	},
	styleRowNode: function(inRowIndex, inRowNode){
		if(inRowNode){
			if(inRowNode.tagName.toLowerCase() == 'div' && this.aggregator){
				query("tr[dojoxTreeGridPath]", inRowNode).forEach(function(rowNode){
					this.rows.styleRowNode(domAttr.get(rowNode, 'dojoxTreeGridPath'), rowNode);
				},this);
			}
			this.rows.styleRowNode(inRowIndex, inRowNode);
		}
	},
	onCanSelect: function(inRowIndex){
		var nodes = query("tr[dojoxTreeGridPath='" + inRowIndex + "']", this.domNode);
		if(nodes.length){
			if(domClass.contains(nodes[0], 'dojoxGridSummaryRow')){
				return false;
			}
		}
		return this.inherited(arguments);
	},
	onKeyDown: function(e){
		if(e.altKey || e.metaKey){
			return;
		}
		switch(e.keyCode){
			case keys.UP_ARROW:
				if(!this.edit.isEditing() && this.focus.rowIndex != "0"){
					event.stop(e);
					this.focus.move(-1, 0);
				}
				break;
			case keys.DOWN_ARROW:
				var currPath = new TreePath(this.focus.rowIndex, this);
				var lastPath = new TreePath(this.rowCount-1, this);
				lastPath = lastPath.lastChild(true);
				if(!this.edit.isEditing() && currPath.toString() != lastPath.toString()){
					event.stop(e);
					this.focus.move(1, 0);
				}
				break;
			default:
				this.inherited(arguments);
				break;
		}
	},
	canEdit: function(inCell, inRowIndex){
		var node = inCell.getNode(inRowIndex);
		return node && this._canEdit;
	},
	doApplyCellEdit: function(inValue, inRowIndex, inAttrName){
		var item = this.getItem(inRowIndex);
		var oldValue = this.store.getValue(item, inAttrName);
		if(typeof oldValue == 'number'){
			inValue = isNaN(inValue) ? inValue : parseFloat(inValue);
		}else if(typeof oldValue == 'boolean'){
			inValue = inValue == 'true' ? true : inValue == 'false' ? false : inValue;
		}else if(oldValue instanceof Date){
			var asDate = new Date(inValue);
			inValue = isNaN(asDate.getTime()) ? inValue : asDate;
		}
		this.store.setValue(item, inAttrName, inValue);
		this.onApplyCellEdit(inValue, inRowIndex, inAttrName);
	}
});
TreeGrid.markupFactory = function(props, node, ctor, cellFunc){
	var widthFromAttr = function(n){
		var w = domAttr.get(n, "width")||"auto";
		if((w != "auto")&&(w.slice(-2) != "em")&&(w.slice(-1) != "%")){
			w = parseInt(w, 10)+"px";
		}
		return w;
	};
	
	var cellsFromMarkup = function(table){
		var rows;
		// Don't support colgroup on our grid - single view, single row only
		if(table.nodeName.toLowerCase() == "table" &&
					query("> colgroup", table).length === 0 &&
					(rows = query("> thead > tr", table)).length == 1){
			var tr = rows[0];
			return query("> th", rows[0]).map(function(th){
				// Grab type and field (the only ones that are shared
				var cell = {
					type: lang.trim(domAttr.get(th, "cellType")||""),
					field: lang.trim(domAttr.get(th, "field")||"")
				};
				if(cell.type){
					cell.type = lang.getObject(cell.type);
				}
				
				var subTable = query("> table", th)[0];
				if(subTable){
					// If we have a subtable, we are an aggregate and a summary cell
					cell.name = "";
					cell.children = cellsFromMarkup(subTable);
					if(domAttr.has(th, "itemAggregates")){
						cell.itemAggregates = array.map(domAttr.get(th, "itemAggregates").split(","), function(v){
							return lang.trim(v);
						});
					}else{
						cell.itemAggregates = [];
					}
					if(domAttr.has(th, "aggregate")){
						cell.aggregate = domAttr.get(th, "aggregate");
					}
					cell.type = cell.type || dojox.grid.cells.SubtableCell;
				}else{
					// Grab our other stuff we need (mostly what's in the normal
					// Grid)
					cell.name = lang.trim(domAttr.get(th, "name")||th.innerHTML);
					if(domAttr.has(th, "width")){
						cell.width = widthFromAttr(th);
					}
					if(domAttr.has(th, "relWidth")){
						cell.relWidth = window.parseInt(domAttr.get(th, "relWidth"), 10);
					}
					if(domAttr.has(th, "hidden")){
						cell.hidden = domAttr.get(th, "hidden") == "true";
					}
					cell.field = cell.field||cell.name;
					DataGrid.cell_markupFactory(cellFunc, th, cell);
					cell.type = cell.type || dojox.grid.cells.Cell;
				}
				if(cell.type && cell.type.markupFactory){
					cell.type.markupFactory(th, cell);
				}
				return cell;
			});
		}
		return [];
	};
	
	var rows;
	if(	!props.structure ){
		var row = cellsFromMarkup(node);
		if(row.length){
			// Set our structure here - so that we don't try and set it in the
			// markup factory
			props.structure = [{__span: Infinity, cells:[row]}];
		}
	}
	return DataGrid.markupFactory(props, node, ctor, cellFunc);
};

return TreeGrid;

});
},
'dijit/_FocusMixin':function(){
define([
	"./focus",
	"./_WidgetBase",
	"dojo/_base/declare", // declare
	"dojo/_base/lang" // lang.extend
], function(focus, _WidgetBase, declare, lang){

/*=====
	var _WidgetBase = dijit._WidgetBase;
=====*/

	// module:
	//		dijit/_FocusMixin
	// summary:
	//		Mixin to widget to provide _onFocus() and _onBlur() methods that
	//		fire when a widget or it's descendants get/lose focus

	// We don't know where _FocusMixin will occur in the inheritance chain, but we need the _onFocus()/_onBlur() below
	// to be last in the inheritance chain, so mixin to _WidgetBase.
	lang.extend(_WidgetBase, {
		// focused: [readonly] Boolean
		//		This widget or a widget it contains has focus, or is "active" because
		//		it was recently clicked.
		focused: false,

		onFocus: function(){
			// summary:
			//		Called when the widget becomes "active" because
			//		it or a widget inside of it either has focus, or has recently
			//		been clicked.
			// tags:
			//		callback
		},

		onBlur: function(){
			// summary:
			//		Called when the widget stops being "active" because
			//		focus moved to something outside of it, or the user
			//		clicked somewhere outside of it, or the widget was
			//		hidden.
			// tags:
			//		callback
		},

		_onFocus: function(){
			// summary:
			//		This is where widgets do processing for when they are active,
			//		such as changing CSS classes.  See onFocus() for more details.
			// tags:
			//		protected
			this.onFocus();
		},

		_onBlur: function(){
			// summary:
			//		This is where widgets do processing for when they stop being active,
			//		such as changing CSS classes.  See onBlur() for more details.
			// tags:
			//		protected
			this.onBlur();
		}
	});

	return declare("dijit._FocusMixin", null, {
		// summary:
		//		Mixin to widget to provide _onFocus() and _onBlur() methods that
		//		fire when a widget or it's descendants get/lose focus

		// flag that I want _onFocus()/_onBlur() notifications from focus manager
		_focusManager: focus
	});

});

},
'ibm/tivoli/tip/navwidget/dojo/NavWidgetState':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!dojox/collections/Dictionary"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.dojo.NavWidgetState");
dojo.provide("ibm.tivoli.tip.navwidget.dojo.NavWidgetBaseViewState");

dojo.require("dojox.collections.Dictionary");

dojo.declare(
    "ibm.tivoli.tip.navwidget.dojo.NavWidgetState",
    [ibm.tivoli.tip.navwidget.dojo.NavWidgetBaseViewState],
    {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
  		
  		// selectedItems: ibm.tivoli.navmodel.Item[]
  		//		Coming soon
  		selectedItems: null,
  		
  		// selectedLinks: ibm.tivoli.navmodel.Link[]
  		//		Coming soon
  		selectedLinks: null,
  		
  		// selectedRelationships: ibm.tivoli.navmodel.Relationship[]
  		//		Coming soon
  		selectedRelationships: null,
  		
  		// isHome: Boolean
  		//		Coming soon
  		isHome: false,
  		
  		// hasStateBeenUsed: Boolean
  		//		Coming soon
  		hasStateBeenUsed: false,
  		
  		// breadcrumbEntry: Boolean
  		//		Coming soon
  		breadcrumbEntry: null,
  		
		/*=====
		constructor_kwArgs: function(dataset) {
			// dataset: ibm.tivoli.navmodel.Dataset
			//		Coming soon
		},
		=====*/
  		
  		constructor: function(/*constructor_kwArgs*/ kwArgs) {		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon

	 	},
    
    	destroy: function() {
			this.selectedItems = null;
			this.selectedLinks = null;
			this.selectedRelationships = null;
			this.isHome = false;
			this.hasStateBeenUsed = false;
			this.breadcrumbEntry = null;
	  		this.inherited(arguments);
		}
    }
);
});

},
'gridx/modules/move/Row':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/keys",
	"../../core/_Module",
	"../../core/model/extensions/Move"
], function(declare, array, keys, _Module, Move){

	return declare(/*===== "gridx.modules.move.Row", =====*/_Module, {
		// summary:
		//		This module provides some APIs to move rows within grid
		// description:
		//		This module requires the "Move" model extension.
		//		This module does not include any UI. So different kind of row dnd UI implementations can be built
		//		upon this module.

		name: 'moveRow',
		
		modelExtensions: [Move],

		constructor: function(){
			this.connect(this.model, 'onMoved', '_onMoved');
		},
	
		getAPIPath: function(){
			// tags:
			//		protected extension
			return {
				move: {
					row: this
				}
			};
		},

		preload: function(){
			this.aspect(this.grid, 'onRowKeyDown', '_onKeyDown');
		},
		
		rowMixin: {
			moveTo: function(target, skipUpdateBody){
				this.grid.move.row.move([this.index()], target, skipUpdateBody);
				return this;
			}
		},
		
		//Public-----------------------------------------------------------------

		//moveSelected: Boolean
		//		When moving using keyboard, whether to move all selected rows together.
		moveSelected: true,

		move: function(rowIndexes, target, skipUpdateBody){
			// summary:
			//		Move some rows to target position
			// rowIndexes: Integer[]
			//		An array of row indexes
			// target: Integer
			//		The rows will be moved to the position before the target row
			// skipUpdateBody: Boolean?
			//		If set to true, the grid will not refresh immediately, so that several
			//		grid operations can be done altogether.
			var m = this.model;
			m.moveIndexes(rowIndexes, target);
			if(!skipUpdateBody){
				m.when();
			}
		},
		
		moveRange: function(start, count, target, skipUpdateBody){
			// summary:
			//		Move a range of rows to target position
			// start: Integer
			//		The index of the first row to be moved
			// count: Integer
			//		The count of rows to be moved
			// skipUpdateBody: Boolean?
			//		
			var m = this.model;
			m.move(start, count, target);
			if(!skipUpdateBody){
				m.when();
			}
		},
		
		//Events------------------------------------------------------------------
		onMoved: function(/* rowIndexMapping */){
			// summary:
			//		Fired when row move is performed successfully
			// tags:
			//		callback
		},
		
		//Private-----------------------------------------------------------------
		_onMoved: function(){
			this.grid.body.refresh();
			this.onMoved();
		},

		_onKeyDown: function(e){
			var t = this,
				g = t.grid,
				selector = g.select && g.select.row;
			if(e.ctrlKey && !e.shiftKey && !e.altKey && (e.keyCode == keys.UP_ARROW || e.keyCode == keys.DOWN_ARROW)){
				var target = e.rowIndex,
					doMove = function(rowIdxes){
						if(e.keyCode == keys.UP_ARROW){
							while(array.indexOf(rowIdxes, target) >= 0){
								target--;
							}
							if(target >= 0){
								t.move(rowIdxes, target);
							}
						}else{
							while(array.indexOf(rowIdxes, target) >= 0){
								target++;
							}
							if(target < g.body.rootStart + g.body.rootCount){
								t.move(rowIdxes, target + 1);
							}
						}
					};
				if(t.arg('moveSelected') && selector && selector.isSelected(e.rowId)){
					var selected = selector.getSelected();
					g.model.when({id: selected}, function(){
						var rowIdxes = array.map(selected, function(id){
							return g.model.idToIndex(id);
						});
						doMove(rowIdxes);
					});
				}else{
					doMove([g.model.idToIndex(e.rowId)]);
				}
			}
		}
	});
});

},
'dijit/_Container':function(){
define([
	"dojo/_base/array", // array.forEach array.indexOf
	"dojo/_base/declare", // declare
	"dojo/dom-construct", // domConstruct.place
	"./registry"	// registry.byNode()
], function(array, declare, domConstruct, registry){

	// module:
	//		dijit/_Container
	// summary:
	//		Mixin for widgets that contain a set of widget children.

	return declare("dijit._Container", null, {
		// summary:
		//		Mixin for widgets that contain a set of widget children.
		// description:
		//		Use this mixin for widgets that needs to know about and
		//		keep track of their widget children. Suitable for widgets like BorderContainer
		//		and TabContainer which contain (only) a set of child widgets.
		//
		//		It's not suitable for widgets like ContentPane
		//		which contains mixed HTML (plain DOM nodes in addition to widgets),
		//		and where contained widgets are not necessarily directly below
		//		this.containerNode.   In that case calls like addChild(node, position)
		//		wouldn't make sense.

		buildRendering: function(){
			this.inherited(arguments);
			if(!this.containerNode){
				// all widgets with descendants must set containerNode
	 			this.containerNode = this.domNode;
			}
		},

		addChild: function(/*dijit._Widget*/ widget, /*int?*/ insertIndex){
			// summary:
			//		Makes the given widget a child of this widget.
			// description:
			//		Inserts specified child widget's dom node as a child of this widget's
			//		container node, and possibly does other processing (such as layout).

			var refNode = this.containerNode;
			if(insertIndex && typeof insertIndex == "number"){
				var children = this.getChildren();
				if(children && children.length >= insertIndex){
					refNode = children[insertIndex-1].domNode;
					insertIndex = "after";
				}
			}
			domConstruct.place(widget.domNode, refNode, insertIndex);

			// If I've been started but the child widget hasn't been started,
			// start it now.  Make sure to do this after widget has been
			// inserted into the DOM tree, so it can see that it's being controlled by me,
			// so it doesn't try to size itself.
			if(this._started && !widget._started){
				widget.startup();
			}
		},

		removeChild: function(/*Widget|int*/ widget){
			// summary:
			//		Removes the passed widget instance from this widget but does
			//		not destroy it.  You can also pass in an integer indicating
			//		the index within the container to remove

			if(typeof widget == "number"){
				widget = this.getChildren()[widget];
			}

			if(widget){
				var node = widget.domNode;
				if(node && node.parentNode){
					node.parentNode.removeChild(node); // detach but don't destroy
				}
			}
		},

		hasChildren: function(){
			// summary:
			//		Returns true if widget has children, i.e. if this.containerNode contains something.
			return this.getChildren().length > 0;	// Boolean
		},

		_getSiblingOfChild: function(/*dijit._Widget*/ child, /*int*/ dir){
			// summary:
			//		Get the next or previous widget sibling of child
			// dir:
			//		if 1, get the next sibling
			//		if -1, get the previous sibling
			// tags:
			//      private
			var node = child.domNode,
				which = (dir>0 ? "nextSibling" : "previousSibling");
			do{
				node = node[which];
			}while(node && (node.nodeType != 1 || !registry.byNode(node)));
			return node && registry.byNode(node);	// dijit._Widget
		},

		getIndexOfChild: function(/*dijit._Widget*/ child){
			// summary:
			//		Gets the index of the child in this container or -1 if not found
			return array.indexOf(this.getChildren(), child);	// int
		}
	});
});

},
'ibm/usmi/dashboard/DashBoardResourceNavigator':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/dashboard/DashBoardResourceNavigator", ["dojo/_base/declare", 
		"ibm/usmi/widgets/ResourceNavigator"], function (declare,
		ResourceNavigator) {
	return declare([ResourceNavigator], {
		
		//Create the items table view
		makeNavWidget: function() {
			// create a default context menu provider
			this._createCMProvider();
			this.inherited(arguments);
			/**
			var defaultActionHelper = new ResourceNavigatorDAHelper();


			this._navWidget = new DashBoardItemsTableView({
				id: this.navId,
				style: "width:100%;height:100%;", //Terry: Need such style because we do not add it to a reference dom node,							
				height: "100%", 
				// homeDataset: dataset,
				//widgetDefinitionId: "ResExpTable-0.1.0",
				daHelper: defaultActionHelper,
				cmProvider: this.cmProvider
			});
			this.addChild(this._navWidget);
	
			lang.hitch(this.makeDataset());
			*/
			
		},
		
				
		resize: function(){
			console.debug("resize resource navigator!");
			this.inherited(arguments);
		}
				
	});
});
},
'ibm/tivoli/tip/navwidget/dijit/NavWidgetMenuProvider':function(){
// wrapped by build app
define("ibm/tivoli/tip/navwidget/dijit/NavWidgetMenuProvider", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/tip/navwidget/dijit/RESTContextMenuProvider"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.dijit.NavWidgetMenuProvider");

dojo.require("ibm.tivoli.tip.navwidget.dijit.RESTContextMenuProvider");

dojo.declare("ibm.tivoli.tip.navwidget.dijit.NavWidgetMenuProvider",
        ibm.tivoli.tip.navwidget.dijit.RESTContextMenuProvider,
{		
	// summary:
	//		Coming soon
	// description:
	//		Coming soon
	
	// _handle:
	//		
	_handle: null,
	
	filterNavigationTasks: function(/*ibm.tivoli.navmodel.Task[]*/ tasks) {
		// summary:
		//	Return a filtered collection of tasks (i.e. remove all navigation tasks)
		//
		// Overriding this method because we do NOT want to filter out navigation tasks
		return tasks;
	},

	handleNavigation: function(data, ioArgs) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// data: 
		//		
		// ioArgs:
		//		
	
		// listen for event indicating new dataset is loaded and dismiss the busy pane
		this._handle = dojo.connect(this.view, "onSetDataset", this, function(dataset) {
																			this.view.hideBusyPane();
																			dojo.disconnect(this._handle);});
		this.view.showBusyPane();
		this.view.getCurrentDataset().navigate(
   	         {onComplete: dojo.hitch(this, this._processNavNewDataSet),
   	         onError: dojo.hitch(this, this._handleNavNewDataSetError),
   	         taskId: data.actionID,
   	         taskCategory: data.category,
   	         selection: this.buildClientNavModelSelection(data, this.view.dataset.getVersion())});
	},
	
	_processNavNewDataSet: function(dataset, kwArgs) {	
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// dataset: 
		//		
		// kwArgs:
		//		
		this.view.addDataset(dataset);
	},

	_handleNavNewDataSetError: function(error, exception, ioArgs) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// error: 
		//		
		// exception:
		//		
		// ioArgs:
		//		
		
		// in case of an error, dismiss the busy pane
		this.view.hideBusyPane();
		dojo.disconnect(this._handle);
		
		//Give the user some feedback
        this.view.displayException(error);
	}
	
});


});

},
'ibm/usmi/scheduler/JobLaunchWizard':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
/*global define */
define(
	[ "dojo/_base/declare",
		"dojo/_base/lang",
		"dojo/_base/array",
		"dojo/date/locale",
		"dojo/topic",
		"dojo/query",
		"dojoe/messagedialog/MessageDialog",
		"dijit/_WidgetBase",
		"dijit/_TemplatedMixin",
		"dijit/_WidgetsInTemplateMixin",
		"dijit/layout/StackContainer",
		"dijit/layout/BorderContainer",
		"dijit/layout/ContentPane",
		"dijit/form/Button",
		"dijit/form/RadioButton",
		"dijit/form/NumberSpinner",
		"dijit/form/CheckBox",
		"dijit/form/ValidationTextBox",
		"dijit/form/DateTextBox",
		"dijit/registry",
		"dijit/form/Select",
		"dijit/form/MultiSelect",
		"dojox/form/TimeSpinner",
		"ibm/usmi/core/utils/Logger",
		"ibm/usmi/util/BaseWidget",
		"ibm/usmi/widgets/util/Constants",
		"ibm/usmi/widgets/wizard/DirectorBorderContainer",
		"ibm/usmi/widgets/wizard/DirectorWizard",
		"ibm/usmi/widgets/wizard/DirectorPage",
		"./wizard_pages/SchedulePage",
		"./wizard_pages/NotificationsPage",
		"./wizard_pages/OptionsPage",
		"ibm/tivoli/navmodel/Dataset",
		"ibm/tivoli/navmodel/Selection",
		"ibm/usmi/util/DefaultMessageHandler",
		"dojo/i18n!./wizard_pages/nls/JobLaunchWizardResources"
	],
	function (
		declare, lang, array, locale, topic, query, MessageDialog, _WidgetBase, _TemplatedMixin,
		_WidgetsInTemplateMixin, StackContainer, BorderContainer, ContentPane, Button, RadioButton, NumberSpinner,
		CheckBox, ValidationTextBox, DateTextBox, registry, Select, MultiSelect, TimeSpinner, Logger, BaseWidget, Constants,
		DirectorBorderContainer, DirectorWizard, DirectorPage, SchedulePage, NotificationsPage, OptionsPage, Dataset,
		Selection, DefaultMessageHandler, mri) {

		// I declare that I am extending support.wizard.DirectorWizard, and BaseWidget
		return declare(
			"ibm.usmi.scheduler.JobLaunchWizard",
			[ DirectorWizard,
				BaseWidget ],
			{
				_dataset : null,
				persistResources : mri,

				iTaskId : null,
				jobName : null,

				//array of MEIDs to target
				MEIDs : null,

				taskSpecificParameters : null,

				publishReturn : null, // will set to the return we want to pass back to callers

				taskParms : null,
				tasks : [],

				// finishLabel : "**NLS**  Submit  **NLS**", // this setting appears to have no impact...I'd needed to do the this.resetFinishLabel call in the postCreate method....
				layout : "horizontal",
				//If noarrows is "false" wizard will be displayed with arrows, if it is "true" wizard will be displayed without arrows.	
				renderNoArrows : false,

				//If showCancelWarning is "true" confirmation message dialog will  be displayed on click of cancel, otherwise it will not be displayed. Default is false. 
				showCancelWarning : true,

				SCHEDULE_JOB_TASK_ID : "UsmiJobLauncherScheduleTask",
				RUN_JOB_NOW_TASK_ID : "UsmiJobLauncherRunNowTask",

				// Logger
				JobLaunchWizardLogger : new Logger("ibm.usmi.scheduler", "JobLaunchWizard"),

				// This method is called when a page in the table of contents is selected.  Only allow them to jump around if the current page 
				// is valid....
				jumpBackToPage : function (
					aPage) {

					// get the current page
					var curr_page = this._pageContainer.selectedChildWidget;

					// only allow us to jump to a different page if the current page is valid...
					if (curr_page.isValid()) {
						this.inherited(arguments);
					}
				},

				postCreate : function () {
					this.inherited(arguments);

					// retrieve the launch parameters...
					this.retrievelaunchParms();

					// retrieve the tasks related to the ITaskId
					this.retrieveTasks();

					this.resetFinishLabel(this.persistResources.SUBMIT_BUTTON_LABEL); // this was needed to set the button text since the "finishLabel" setting appeared to have no impact on the constructor above

					// If we don't yet have a job name (it wasn't passed in to us), we'll try to retrieve it from the task we retrieved
					if (!this.jobName) {
						// retrieve the job name from the first task we've got
						this.jobName = this.tasks[0].getLabel();
					}

					// Add the wizard pages....
					// First add the schedule page -- indicate it's required, and pass in the job name to be used as the base for the 
					// default job name....
					this.addPage(new SchedulePage({
						required : true,
						jobNameBase : this.jobName
					}));

					this.addPage(new NotificationsPage());
					this.addPage(new OptionsPage());

					// Attempt to set the title of the dialog to the job name
					if (this.launchParms) {
						this.launchParms.dialogTitle = this.jobName;
					}

					// Set the dialog size (so it displays)
					this.resize({
						w : 700,
						h : 700
					});

					// default our return value to "cancel" (assume the user will be exiting the 
					// dialog via the cancel button)
					this.publishReturn = Constants.TASK_LAUNCH_CANCELED;

				},

				retrievelaunchParms : function () {
					// get the values from the launch parameters 

					// First get the task ID -- REQUIRED:  we can't continue without one
					if (this.launchParms) {
						this.iTaskId = this.launchParms.taskID;
					} // else, NO launch parms -- iTaskId may have been passed directly to us, so we'll wait to display any error message....

					// set the target context for the task -- Optional:  this may not be a targeted task....or the MEIDs may have been passed directly to us on
					// the constructor....
					if (this.launchParms) {
						if (this.launchParms.MEIDS) {
							var mesSplit = this.launchParms.MEIDS.split(',');
							if (mesSplit.length > 0) {
								this.MEIDs = [];
								for ( var j = 0; j < mesSplit.length; j++) {
									this.MEIDs[j] = parseInt(mesSplit[j], 10);
								}
							}
						}
					}
				},

				retrieveTasks : function () {
					var moduleName = "retrieveTasks";
					var msgHandler = new DefaultMessageHandler();
					var msgId = ""; // not specifying a message ID
					var theMessageTitle = "";
					var theOperatorResponse = "";
					var msgSeverity = DefaultMessageHandler.ERROR;

					if (!this.iTaskId) {
						// This should never happen....but in case it does, we don't want to just not open and give no indication as to why...issue our generic error message.
						// First, log what's happened....
						if (this.JobLaunchWizardLogger.isError()) {
							this.JobLaunchWizardLogger
								.error(
									"JobLaunchWizard:  called without an iTaskId being passed in.  The job launch cannot continue.",
									moduleName);
						}

						// Build the error message.
						theMessageTitle = this.persistResources.GENERIC_ERROR_MESSAGE;
						theOperatorResponse = this.persistResources.GENERIC_ERROR_RESPONSE;
						msgHandler.displayMessage(msgId, msgSeverity, theMessageTitle, theOperatorResponse, true);
					} else {
						// First try to retrieve the task that the taskId is for:
						// Log the task ID that was passed in...
						if (this.JobLaunchWizardLogger.isFine()) {
							this.JobLaunchWizardLogger.fine("JobLaunchWizard:  The task ID that was passed in is:  " +
								this.iTaskId, moduleName);
						}

						var theDatasetValue = "JOB_LAUNCHER:" + this.iTaskId;
						try {
							this._dataset = new Dataset({
								baseUrl : "/ibm/tivoli/rest",
								provider : "USMI",
								datasource : "",
								dataset : theDatasetValue,
								parameters : {

								},
								defaultProperties : []
							});

							var tasks = this._dataset.getTasks({
								onComplete : lang.hitch(this, this._retrievedTasks),
								onError : lang.hitch(this, this._retrievedTasksError),
								taskCategory : "menu", //TODO: Could I specifically request nav hints instead of all tasks?
								itemIndexes : "0",
								sync : true
							});

						} catch (ex) {
							//There was a problem creating the dataset
							if (this.JobLaunchWizardLogger.isError()) {
								this.JobLaunchWizardLogger
									.error(
										"JobLaunchWizard:  A problem occurred creating our dataset threw an exception.  The task ID that was passed in is:  " +
											this.iTaskId, moduleName);
								this.JobLaunchWizardLogger.error(
									"JobLaunchWizard:  The exception is:  " + ex, moduleName);
							}

							msgHandler.displayException(ex);
						}

					}

				},

				_retrievedTasksError : function () {
					var msgHandler = new DefaultMessageHandler();
					var moduleName = "retrievedTasksError";
					var msgId = ""; // not specifying a message ID
					var theMessageTitle = "";
					var theOperatorResponse = "";
					var msgSeverity = DefaultMessageHandler.ERROR;

					// This should never happen....but in case it does, we don't want to just not open and give no indication as to why...issue our generic error message.
					if (this.JobLaunchWizardLogger.isError()) {
						this.JobLaunchWizardLogger
							.error(
								"JobLaunchWizard:  Action to retrieve the task was not successful.  The task ID that was passed in is:  " +
									this.iTaskId, moduleName);
					}
					theMessageTitle = this.persistResources.GENERIC_ERROR_MESSAGE;
					theOperatorResponse = this.persistResources.GENERIC_ERROR_RESPONSE;
					msgHandler.displayMessage(msgId, msgSeverity, theMessageTitle, theOperatorResponse, true);
				},

				_retrievedTasks : function (
					tasks) {
					// We got the appropriate tasks....save them away...
					this.tasks = tasks;
				},

				// Perform the activity.
				OnOK : function (
					wiz) {

					// build the full list of task parms needed to launch the task (will retrieve the user input from each of the schedule-related pages)
					this.buildTaskParameters();

					var execTask;

					// Retrieve the proper task -- Check to see if we are "schedule" or "run now"
					var taskToRetrieve = this.SCHEDULE_JOB_TASK_ID; // assume we are a scheduled job...
					// If the user chose the option to "run now", then we want a different task
					if (this.taskParms.StartDate === "RunNow") {
						taskToRetrieve = this.RUN_JOB_NOW_TASK_ID;
					}

					//Want to loop through the list of tasks we're initial retrieved looking for that correct task.
					if (this.tasks) {
						for ( var i = 0; i < this.tasks.length; i++) {
							var task = this.tasks[i];
							// found our task.
							if (task.getId() === taskToRetrieve) {

								execTask = task;

								break;
							}
						}
					}

					// execute the task....
					execTask.execute({
						onError : lang.hitch(this, this._handleExecuteOnServerError),
						onComplete : lang.hitch(this, this._getExecuteComplete),
						selection : new Selection({
							version : this._dataset.getVersion()
						}),
						selectionParms : this.taskParms,
						sync : true
					});

				},

				buildTaskParameters : function () {

					// define the values we will need to set to launch the task
					// schedule page information:
					var jobNameParm = null; // retrieve from schedule page.
					//var manageableElementsParm = null; // currently we do not support elements being passed in 
					//var startDateParm = null; // needs to be time in milliseconds from epoch that the job will begin running					
					var repeatOptionsParm = null;
					var frequency_option = null;
					var on_weekend_option = null;
					var forWeekDays_option = null;
					var forMonthDates_option = null;
					var customDates_option = null;
					var duration_option = null;

					// notification page information:
					var notificationOptionsParm = null;

					// additional options page options:
					var additionalOptionsParm = null;
					//var propertiesParm = null;

					// Retrieve the information from the Schedule page (page 0)....
					var schedulePage = this._pages[0];

					// retrieve the job name (required by every job)
					jobNameParm = schedulePage.jobName.get('value');

					var startDate = (new Date()).getTime(); // TODO:  initial value needed?  (perhaps just for this prototyping....)

					// retrieve the "runnow" radio button....if it's checked, all other scheduling information is moot:
					var runNow = schedulePage.runNowRadio.get('value');

					if (runNow === "now") {
						// If we know the user only wants to run the task "right now", then set that value for the start date
						startDate = "RunNow";
					} else {
						// retrieve the repeat option selected
						frequency_option = schedulePage.getRepeatSelectionValue();

						// retrieve the specific options based on the repeat option selected -- start with "custom" because 
						// it defines its own dates and times on which to run the job
						if (frequency_option === "CUSTOM") { // custom repeat option
							// "custom" appears to require only a list of custom dates/times
							// retrieve all of the custom dates
							var customDatesContainer = schedulePage.selectedCustomDates.containerNode;
							customDates_option = [];
							for ( var i = 0; i < customDatesContainer.options.length; i++) {
								customDates_option.push((new Date(customDatesContainer.options.item(i).value))
									.getTime());
							}
						} else { // we're something other than "CUSTOM"
							// the remaining repeat options require a date and time on which to begin the scheduled job...
							// retrieve the start date (scheduled date and time) -- built from the scheduleDate field and scheduleTime field...				
							var startDate_date = schedulePage.scheduleDate.get('value');
							var startDate_time = schedulePage.scheduleTime.get('value');

							// create our start date based off the date field.
							startDate = startDate_date;

							// retrieve the hours and minutes from the time field
							var hours = locale.format(startDate_time, {
								timePattern : "H",
								selector : "time"
							});
							var mins = locale.format(startDate_time, {
								timePattern : "mm",
								selector : "time"
							});
							// set the time field on our start date
							startDate.setHours(hours, mins);

							// need the time in milliseconds from epoch -- so get the time from startDate
							// you can verify that time via this Epoch 
							// converter: http://www.epochconverter.com/
							startDate = (startDate.getTime());

							if (frequency_option !== "ONCE") { // We're not on a repeat once schedule (and we know we're not a custom schedule)
								// Retrieve the Duration value (common to all repeat option except custom/once)					
								var theDuration = schedulePage.forDurationRadio.get('value');

								var durationType = null;
								var durationValue = null;

								if (theDuration === "for") {
									durationType = schedulePage.getDurationRepeatForUnitsValue(); // the units like weeks/months/years
									durationValue = schedulePage.getDurationRepeatForValue(); // the value (like "6")
								} else { // else we're "until" or "forever"
									theDuration = schedulePage.untilDurationRadio.get('value');
									if (theDuration === "until") { // we're "until"
										durationType = "UNTIL_ENDDATE";

										durationValue = ((new Date(schedulePage.repeatUntil.get('value'))).getTime());
									} else { // must be repeat forever (unlimited)....there is no additional duration information that needs to be retrieved.
										durationType = "FOREVER";
									}
								}

								// define the duration option
								duration_option = {
									Type : durationType,
									Value : durationValue
								};

								// the "weekly" repeat option is the only one that does not require value for what to do on weekends...
								if (frequency_option !== "WEEKLY") { // if we're not doing the weekly repeat option, retrieve the weekend choice
									on_weekend_option = schedulePage.getWeekEndOptionValue();
								} // end repeat weekly

								// the "monthly" repeat option also requires the additional "dates" value.
								if (frequency_option === "MONTHLY_DATES") { // monthly repeat options
									// retrieve the monthly dates values...(just like was done for custom dates)
									// "monthly" appears to require only a list of dates of the month
									// retrieve all of the monthly dates 
									var monthlyDatesContainer = schedulePage.selectedMonthlyDates.containerNode;
									forMonthDates_option = [];
									for ( var j = 0; j < monthlyDatesContainer.options.length; j++) {
										var aMonthlyDate = monthlyDatesContainer.options.item(j).value;
										var intMonthlyDate = parseInt(aMonthlyDate, 10);

										forMonthDates_option.push(intMonthlyDate);
									}
								} // end repeat monthly

								// If we're schedule to run weekly, determine on which days of the week we'll run
								if (frequency_option === "WEEKLY") {
									forWeekDays_option = [];
									if (schedulePage.sundayDay.get('value')) {
										forWeekDays_option.push("SUNDAY");
									}
									if (schedulePage.mondayDay.get('value')) {
										forWeekDays_option.push("MONDAY");
									}
									if (schedulePage.tuesdayDay.get('value')) {
										forWeekDays_option.push("TUESDAY");
									}
									if (schedulePage.wednesdayDay.get('value')) {
										forWeekDays_option.push("WEDNESDAY");
									}
									if (schedulePage.thursdayDay.get('value')) {
										forWeekDays_option.push("THURSDAY");
									}
									if (schedulePage.fridayDay.get('value')) {
										forWeekDays_option.push("FRIDAY");
									}
									if (schedulePage.saturdayDay.get('value')) {
										forWeekDays_option.push("SATURDAY");
									}
								}

								//else if (repeatOption === "YEARLY") {
								// Repeat yearly...there are no additional values that need to be retrieved.
								//} else if (repeatOption === "HOURLY") {
								// Repeat hourly....there are no additional values that need to be retrieved.						
								//} else if (repeatOption === "DAILY") {
								// Repeat daily....there are no additional values that need to be retrieved.
								//}

							} // end we are NOT a repeat "once" schedule 

						} // end we are not NOT a custom date

						// At this point, we should have all of the information from the schedule page properly set
						repeatOptionsParm = {
							Frequency : frequency_option,
							OnWeekEnds : on_weekend_option,
							ForWeekDays : forWeekDays_option,
							ForMonthDates : forMonthDates_option,
							CustomDates : customDates_option,
							Duration : duration_option
						};
					} // else we're not "run now" -- we've now set all our repeatOptions....

					// Retrieve the information from the Notifications page (page 1)
					var notificationsPage = this._pages[1];
					// Retrieve the notify flags
					var notifyTrigger = [];
					//var notifyTrigger = null;
					// retrieve the flag for notify when the task begins
					if (notificationsPage.whenJobBegins.get('value')) {
						notifyTrigger.push("JOB_BEGINS");
					}
					if (notificationsPage.whenJobEndsSuccessfully.get('value')) {
						notifyTrigger.push("JOB_COMPLETES");
					}
					if (notificationsPage.whenJobFails.get('value')) {
						notifyTrigger.push("JOB_FAILS");
					}

					// retrieve the email information...
					var emailAddr = notificationsPage.notificationEmailAddress.get('value');
					var serverAddr = notificationsPage.notificationEmailServerName.get('value');
					var serverPort = parseInt(notificationsPage.notificationEmailServerPort.get('value'), 10);

					// retrieve the errorThresholdType
					var thresholdTypeAny = notificationsPage.anyErrorRadio.getValue('value');
					var thresholdTypePercent = notificationsPage.percentFailedRadio.getValue('value');
					var thresholdTypeNumber = notificationsPage.numberFailedRadio.getValue('value');
					
					// retrieve the errorThreshold value
					var thresholdPercentFailed = notificationsPage.thresholdPercentFailed.getValue('value');
					var thresholdNumberFailed = notificationsPage.thresholdNumberFailed.getValue('value');

					// At this point, we should have all of the information from the notifications page properly set
					if (thresholdTypePercent === 'true') {
						notificationOptionsParm = {
						NotifyTrigger : notifyTrigger,
						PercentTargetsFailed : thresholdPercentFailed,
						EmailAddress : emailAddr,
						EmailServer : serverAddr,
						EmailServerPort : serverPort
						};
					} else if (thresholdTypeNumber) {
						notificationOptionsParm = {
						NotifyTrigger : notifyTrigger,
						NumberTargetsFailed : thresholdNumberFailed,
						EmailAddress : emailAddr,
						EmailServer : serverAddr,
						EmailServerPort : serverPort
						};
					} else {
						notificationOptionsParm = {
						NotifyTrigger : notifyTrigger,
						EmailAddress : emailAddr,
						EmailServer : serverAddr,
						EmailServerPort : serverPort
						};
					}
					/*
					notificationOptionsParm = {
						NotifyTrigger : notifyTrigger,
						PercentTargetsFailed : thresholdPercentFailed,
						NumberTargetsFailed : thresholdNumberFailed,
						EmailAddress : emailAddr,
						EmailServer : serverAddr,
						EmailServerPort : serverPort
					};
					*/

					// Retrieve the information from the Options page (page 2)
					var optionsPage = this._pages[2];

					// retrieve the flag using server time
					var useManagementServerTime = (optionsPage.useMgmtServerTime.get('value') === "true"); // boolean: true (use mgmt server time)/false (use endpoint time)

					// retrieve the flag for 
					var failUnavailableSystems = (optionsPage.failSystems.get('value') !== "true"); // boolean:  true (run when avail)/false (fail) 

					// retrieve the maximum number of job instances
					var numberOfInstances = optionsPage.maxiumJobInstances.get('value');

					// At this point, we should have all of the information from the options page properly set
					additionalOptionsParm = {
						ManagementServerTime : useManagementServerTime,
						RunWhenAvailable : failUnavailableSystems,
						MaxActivationHistory : numberOfInstances
					};

					// Now, build the taskParms....
					this.taskParms = {
						DisplayName : jobNameParm, // the name of the job (from the schedule page)

						ManageableElements : this.MEIDs, // the target context of the job (passed in to me)

						// date and time to start the task (from the schedule page).  
						// Will either be the number of milliseconds from epoch OR it will have a special value:  "RunNow"
						StartDate : startDate,

						// the task repeat information (from the schedule page) -- this could be null if the user chose to "RunNow"
						RepeatOptions : repeatOptionsParm,

						// the notification options for the task (from the notification page)
						NotificationOptions : notificationOptionsParm,

						// the additional options for the task (from the options page)
						AdditionalOptions : additionalOptionsParm,

						// the task specific parameters that were passed in to us
						Properties : this.taskSpecificParameters
					};

				},

				_getExecuteComplete : function (
					result, arg) {
					var r = result;
					var a = arg;
					var moduleName = "_getExecuteComplete";

					// TODO:  Consider how to add a link to the job details in the message -- based on the job ID....r.JobId
					var theMessageTitle = "";
					if (this.taskParms.StartDate === "RunNow") {
						theMessageTitle = this.persistResources.JOB_SUCCESSFULLY_RUN_NOW + this.taskParms.DisplayName;
						if (this.JobLaunchWizardLogger.isFine()) {
							this.JobLaunchWizardLogger.fine(
								"JobLaunchWizard:  Successfully The created and started job with name: " +
									this.taskParms.DisplayName + " and job id: " + r.JobId, moduleName);
						}
					} else {
						theMessageTitle = this.persistResources.JOB_SUCCESSFULLY_SCHEDULED + this.taskParms.DisplayName;
						if (this.JobLaunchWizardLogger.isFine()) {
							this.JobLaunchWizardLogger.fine(
								"JobLaunchWizard:  Successfully The created and scheduled job with name: " +
									this.taskParms.DisplayName + " and job id: " + r.JobId, moduleName);
						}
					}

					// Indicate how the execute attempt resulted for our return value
					if (this.taskParms.StartDate === "RunNow") {
						this.publishReturn = Constants.TASK_LAUNCH_SUBMIT_SUCCESSFUL;
					} else {
						this.publishReturn = Constants.TASK_LAUNCH_SUBMIT_SUCCESSFUL;
					}
					
					// Close the dialog...
					this.OnCancel();

					// Display the message...
					var msgHandler = new DefaultMessageHandler();
					var msgId = ""; // not specifying a message ID
					var msgSeverity = DefaultMessageHandler.SUCCESS;

					// Note that currently, this method on msgHandler can take a list of targets and prepends them with 
					// "&sourceToken=" -- this doesn't work for launching into
					// the jobs properties -- it needs &DCMJobId=xxxx...so I'll manually build my own URL.
					// msgHandler.displayMessageWithLaunchInContext(msgId, msgSeverity, theMessageTitle, null, null, null, url, this.persistResources.VIEW_JOB_LINK_TEXT, false);

					// build the URL for the link to launch the job properties:
					// TODO:  Is it okay to launch without the task area and nav area -- and to hide the banner/taskbar area?
					var url = window.location.protocol +
						'//' +
						location.host +
						'/ibm/console/xLaunch.do?pageID=com.ibm.usmi.console.scheduler.navigationElement&hideTaskArea=false&disableNavArea=false&hideBannerArea=false&hideTaskBarArea=false';
					url = url + '&DCMJobId=' + r.JobId;

					msgHandler.displayMessageWithLaunchInContext(
						msgId, msgSeverity, theMessageTitle, null, null, null, url,
						this.persistResources.VIEW_JOB_LINK_TEXT, false);
				},

				_handleExecuteOnServerError : function (
					error, exception, ioArgs) {
					var err = error;
					var exc = exception;
					var a = ioArgs;
					var moduleName = "_handleExecuteOnServerError";

					if (this.JobLaunchWizardLogger.error()) {
						this.JobLaunchWizardLogger.error("JobLaunchWizard:  Failed to create job with name: " +
							this.taskParms.DisplayName, moduleName);
						this.JobLaunchWizardLogger
							.error("JobLaunchWizard:  The error received is:  " + err, moduleName);
						this.JobLaunchWizardLogger.error("JobLaunchWizard:  The exception is:  " + exc, moduleName);
					}

					var theMessageTitle = this.persistResources.JOB_CREATION_FAILED;

					var theOperatorResponse = this.persistResources.JOB_FAILED_RESPONSE;
					var theMessageDesription = exc.getMsgText();

					var msgDesc = theOperatorResponse + "<p>" + theMessageDesription;

					// Indicate how the execute attempt resulted for our return value
					if (this.taskParms.StartDate === "RunNow") {
						this.publishReturn = Constants.TASK_LAUNCH_SUBMIT_FAILED;
					} else {
						this.publishReturn = Constants.TASK_LAUNCH_SUBMIT_FAILED;;
					}
					
					// Close the dialog...
					this.OnCancel();

					// Display the message...as a popup
					var msgHandler = new DefaultMessageHandler();
					var msgId = ""; // not specifying a message ID
					var msgSeverity = DefaultMessageHandler.ERROR;
					msgHandler.displayMessage(msgId, msgSeverity, theMessageTitle, msgDesc, true);
				},

				//overriding from Wizard
				OnCancel : function (
					wiz) {
					// publish my dismissal
					topic.publish("DefaultDialogDismissal", []);

					// publish my result -- passing the proper return (Constants.TASK_LAUNCH_CANCELED, Constants.TASK_LAUNCH_SUBMIT_SUCCESSFUL, Constants.TASK_LAUNCH_SUBMIT_FAILED)
					topic.publish(Constants.TOPIC_JOB_LAUNCH_RESULT, [this.publishReturn]);

					this.destroyRecursive();
					this.inherited(arguments);
				},

				/*
				destroy : function () {
					// summary:
					//		Coming soon
					// description:
					//		Coming soon
					var buttonId =  this.id + "_DialogCancelButton";  //  this._outerContainer.id;  
					if (registry.byId(buttonId)) {
						var widgets = dijit.findWidgets(this);
						dojo.forEach(widgets, function (
							w) {
							w.destroyRecursive(true);
						});
						registry.byId(this.id).destroyRecursive();

					}
					
					this.inherited(arguments);
				},
				*/

				destroyRecursive : function () {
					this.inherited(arguments);
				},

				//overriding from Wizard
				_onConfirm : function (
					buttonPressedId, messageId, checked, wiz) {
					if (buttonPressedId === 0) {
						this.OnCancel(wiz);
					}
				},

				getBodyHeight : function () {
					return (0.1 * ((window.innerHeight !== undefined) ? window.innerHeight : document.body.offsetHeight));
				}

				/*
				//force load the Wizard.css, not sure why this doesn't get loaded by the Wizard widget
				load_css : function (
					cssFile) {
					var styleSheetLink = document.createElement('link');
					styleSheetLink.rel = 'stylesheet';
					styleSheetLink.type = 'text/css';
					styleSheetLink.href = 'js/dojoe/wizard/themes/claro/Wizard.css';
					query("head")[0].appendChild(styleSheetLink);
								}
				*/
			});
	});

},
'url:dojoe/tablex/templates/Table.html':"\r\n<div class=\"tivTable\" tabIndex=0 aria-label=\"table\" role=\"application\">\r\n\t<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" width=\"100%\" height=\"100%\" \r\n\t\tstyle=\"width:100%;height:100%;\" role=\"presentation\">\t\t\r\n\t\t<tr valign=\"center\">\r\n\t\t\t<td width=\"100%\" role=\"toolbar\">\r\n\t\t\t\t<div dojoAttachPoint=\"wipeNode\">\r\n\t\t\t\t<div dojoAttachPoint=\"toolbarNode\"></div>\r\n\t\t\t\t</div>\r\n\t\t\t</td>\r\n\t\t</tr>\r\n\t\t<tr valign=\"center\"><td width=\"100%\" role=\"grid\">\r\n\t\t\t<div  dojoAttachPoint=\"enhancedGridNode\"></div></td></tr>\t\t\r\n\t</table>\r\n</div>\t",
'dijit/layout/_LayoutWidget':function(){
define([
	"dojo/_base/lang", // lang.mixin
	"../_Widget",
	"../_Container",
	"../_Contained",
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.add domClass.remove
	"dojo/dom-geometry", // domGeometry.marginBox
	"dojo/dom-style", // domStyle.getComputedStyle
	"dojo/_base/sniff", // has("ie")
	"dojo/_base/window" // win.global
], function(lang, _Widget, _Container, _Contained,
	declare, domClass, domGeometry, domStyle, has, win){

/*=====
	var _Widget = dijit._Widget;
	var _Container = dijit._Container;
	var _Contained = dijit._Contained;
=====*/

	// module:
	//		dijit/layout/_LayoutWidget
	// summary:
	//		_LayoutWidget Base class for a _Container widget which is responsible for laying out its children.
	//		Widgets which mixin this code must define layout() to manage placement and sizing of the children.


	return declare("dijit.layout._LayoutWidget", [_Widget, _Container, _Contained], {
		// summary:
		//		Base class for a _Container widget which is responsible for laying out its children.
		//		Widgets which mixin this code must define layout() to manage placement and sizing of the children.

		// baseClass: [protected extension] String
		//		This class name is applied to the widget's domNode
		//		and also may be used to generate names for sub nodes,
		//		for example dijitTabContainer-content.
		baseClass: "dijitLayoutContainer",

		// isLayoutContainer: [protected] Boolean
		//		Indicates that this widget is going to call resize() on its
		//		children widgets, setting their size, when they become visible.
		isLayoutContainer: true,

		buildRendering: function(){
			this.inherited(arguments);
			domClass.add(this.domNode, "dijitContainer");
		},

		startup: function(){
			// summary:
			//		Called after all the widgets have been instantiated and their
			//		dom nodes have been inserted somewhere under win.doc.body.
			//
			//		Widgets should override this method to do any initialization
			//		dependent on other widgets existing, and then call
			//		this superclass method to finish things off.
			//
			//		startup() in subclasses shouldn't do anything
			//		size related because the size of the widget hasn't been set yet.

			if(this._started){ return; }

			// Need to call inherited first - so that child widgets get started
			// up correctly
			this.inherited(arguments);

			// If I am a not being controlled by a parent layout widget...
			var parent = this.getParent && this.getParent();
			if(!(parent && parent.isLayoutContainer)){
				// Do recursive sizing and layout of all my descendants
				// (passing in no argument to resize means that it has to glean the size itself)
				this.resize();

				// Since my parent isn't a layout container, and my style *may be* width=height=100%
				// or something similar (either set directly or via a CSS class),
				// monitor when viewport size changes so that I can re-layout.
				this.connect(win.global, 'onresize', function(){
					// Using function(){} closure to ensure no arguments passed to resize().
					this.resize();
				});
			}
		},

		resize: function(changeSize, resultSize){
			// summary:
			//		Call this to resize a widget, or after its size has changed.
			// description:
			//		Change size mode:
			//			When changeSize is specified, changes the marginBox of this widget
			//			and forces it to relayout its contents accordingly.
			//			changeSize may specify height, width, or both.
			//
			//			If resultSize is specified it indicates the size the widget will
			//			become after changeSize has been applied.
			//
			//		Notification mode:
			//			When changeSize is null, indicates that the caller has already changed
			//			the size of the widget, or perhaps it changed because the browser
			//			window was resized.  Tells widget to relayout its contents accordingly.
			//
			//			If resultSize is also specified it indicates the size the widget has
			//			become.
			//
			//		In either mode, this method also:
			//			1. Sets this._borderBox and this._contentBox to the new size of
			//				the widget.  Queries the current domNode size if necessary.
			//			2. Calls layout() to resize contents (and maybe adjust child widgets).
			//
			// changeSize: Object?
			//		Sets the widget to this margin-box size and position.
			//		May include any/all of the following properties:
			//	|	{w: int, h: int, l: int, t: int}
			//
			// resultSize: Object?
			//		The margin-box size of this widget after applying changeSize (if
			//		changeSize is specified).  If caller knows this size and
			//		passes it in, we don't need to query the browser to get the size.
			//	|	{w: int, h: int}

			var node = this.domNode;

			// set margin box size, unless it wasn't specified, in which case use current size
			if(changeSize){
				domGeometry.setMarginBox(node, changeSize);
			}

			// If either height or width wasn't specified by the user, then query node for it.
			// But note that setting the margin box and then immediately querying dimensions may return
			// inaccurate results, so try not to depend on it.
			var mb = resultSize || {};
			lang.mixin(mb, changeSize || {});	// changeSize overrides resultSize
			if( !("h" in mb) || !("w" in mb) ){
				mb = lang.mixin(domGeometry.getMarginBox(node), mb);	// just use domGeometry.marginBox() to fill in missing values
			}

			// Compute and save the size of my border box and content box
			// (w/out calling domGeometry.getContentBox() since that may fail if size was recently set)
			var cs = domStyle.getComputedStyle(node);
			var me = domGeometry.getMarginExtents(node, cs);
			var be = domGeometry.getBorderExtents(node, cs);
			var bb = (this._borderBox = {
				w: mb.w - (me.w + be.w),
				h: mb.h - (me.h + be.h)
			});
			var pe = domGeometry.getPadExtents(node, cs);
			this._contentBox = {
				l: domStyle.toPixelValue(node, cs.paddingLeft),
				t: domStyle.toPixelValue(node, cs.paddingTop),
				w: bb.w - pe.w,
				h: bb.h - pe.h
			};

			// Callback for widget to adjust size of its children
			this.layout();
		},

		layout: function(){
			// summary:
			//		Widgets override this method to size and position their contents/children.
			//		When this is called this._contentBox is guaranteed to be set (see resize()).
			//
			//		This is called after startup(), and also when the widget's size has been
			//		changed.
			// tags:
			//		protected extension
		},

		_setupChild: function(/*dijit._Widget*/child){
			// summary:
			//		Common setup for initial children and children which are added after startup
			// tags:
			//		protected extension

			var cls = this.baseClass + "-child "
				+ (child.baseClass ? this.baseClass + "-" + child.baseClass : "");
			domClass.add(child.domNode, cls);
		},

		addChild: function(/*dijit._Widget*/ child, /*Integer?*/ insertIndex){
			// Overrides _Container.addChild() to call _setupChild()
			this.inherited(arguments);
			if(this._started){
				this._setupChild(child);
			}
		},

		removeChild: function(/*dijit._Widget*/ child){
			// Overrides _Container.removeChild() to remove class added by _setupChild()
			var cls = this.baseClass + "-child"
					+ (child.baseClass ?
						" " + this.baseClass + "-" + child.baseClass : "");
			domClass.remove(child.domNode, cls);

			this.inherited(arguments);
		}
	});
});

},
'gridx/Grid':function(){
define([
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/_base/sniff",
	"dojo/on",
	"dojo/dom-class",
	"dojo/dom-geometry",
	"dojo/_base/query",
	"dojox/html/metrics",
	"dijit/_WidgetBase",
	"dijit/_FocusMixin",
	"dijit/_TemplatedMixin",
	"dojo/text!./templates/Grid.html",
	"./core/Core",
	"./core/model/extensions/Query",
	"./core/_Module",
	"./modules/Header",
	"./modules/Body",
	"./modules/VLayout",
	"./modules/HLayout",
	"./modules/VScroller",
	"./modules/HScroller",
	"./modules/ColumnWidth"
], function(kernel, declare, array, lang, has, on, domClass, domGeometry, query, metrics,
	_WidgetBase, _FocusMixin, _TemplatedMixin, template,
	Core, Query, _Module, Header, Body, VLayout, HLayout, VScroller, HScroller, ColumnWidth){

	var forEach = array.forEach,
		dummyFunc = function(){};

	
	var Grid = declare('gridx.Grid', [_WidgetBase, _TemplatedMixin, _FocusMixin, Core], {
		// summary:
		//		Gridx is a highly extensible widget providing grid/table functionalities. 
		// description:
		//		Gridx is much smaller, faster, more reasonable designed, more powerful and more flexible 
		//		compared to the old dojo DataGrid/EnhancedGrid.
		//
		//		NOTE:
		//		=====
		//		The API documents will be updated from time to time. If you encountered an API whose doc is
		//		not sufficient enough, please refer to the following link for latest API docs:
		//		http://evanhw.github.com/gridx/doc/gridx.html

		
		templateString: template,

		coreModules: [
			//Put default modules here!
			Header,
			Body,
			VLayout,
			HLayout,
			VScroller,
			HScroller,
			ColumnWidth
		],

		coreExtensions: [
			//Put default extensions here!
			Query
		],
	
		postCreate: function(){
			// summary:
			//		Override to initialize grid modules
			// tags:
			//		protected extension
			var t = this;
			t.inherited(arguments);
			t._eventFlags = {};
			t.modules = t.coreModules.concat(t.modules || []);
			t.modelExtensions = t.coreExtensions.concat(t.modelExtensions || []);
			domClass.toggle(t.domNode, 'gridxRtl', !t.isLeftToRight());
			t.lastFocusNode.setAttribute('tabIndex', t.domNode.getAttribute('tabIndex'));
			t._initEvents(t._compNames, t._eventNames);
			t._reset(t);
			//resize the grid when zoomed in/out.
			t.connect(metrics, 'onFontResize', function(){
				t.resize();
			});
		},
	
		startup: function(){
			// summary:
			//		Startup this grid widget
			// tags:
			//		public extension
			if(!this._started){
				this.inherited(arguments);
				this._deferStartup.callback();
			}
		},
	
		destroy: function(){
			// summary:
			//		Destroy this grid widget
			// tags:
			//		public extension
			this._uninit();
			this.inherited(arguments);
		},

	/*=====
		// autoHeight: Boolean
		//		If true, the grid's height is determined by the total height of the rows in current body view,
		//		so that there will never be vertical scroller bar. And when scrolling the mouse wheel over grid body,
		//		the whole page will be scrolled. Note if this is false, only the grid body will be scrolled.
		autoHeight: false,
		// autoWidth: Boolean
		//		If true, the grid's width is determined by the total width of the columns, so that there will
		//		never be horizontal scroller bar.
		autoWidth: false,
	=====*/

		
		resize: function(changeSize){
			// summary:
			//		Resize the grid using given width and height.
			// tags:
			//		public
			// changeSize: Object?
			//		An object like {w: ..., h: ...}.
			//		If omitted, the grid will re-layout itself in current width/height.
			var t = this, ds = {};
			if(changeSize){
				if(t.autoWidth){
					changeSize.w = undefined;
				}
				if(t.autoHeight){
					changeSize.h = undefined;
				}
				domGeometry.setMarginBox(t.domNode, changeSize);
			}
			t._onResizeBegin(changeSize, ds);
			t._onResizeEnd(changeSize, ds);
		},

		//Private-------------------------------------------------------------------------------
		_onResizeBegin: function(){},
		_onResizeEnd: function(){},
		
		//event handling begin
		_compNames: ['Cell', 'HeaderCell', 'Row', 'Header'],
	
		_eventNames: [
			'Click', 'DblClick', 
			'MouseDown', 'MouseUp', 
			'MouseOver', 'MouseOut', 
			'MouseMove', 'ContextMenu',
			'KeyDown', 'KeyPress', 'KeyUp'
		],
	
		_initEvents: function(objNames, evtNames){
			var t = this;
			forEach(objNames, function(comp){
				forEach(evtNames, function(event){
					var evtName = 'on' + comp + event;
					t[evtName] = t[evtName] || dummyFunc;
				});
			});
		},
	
		_connectEvents: function(node, connector, scope){
			for(var t = this,
					m = t.model,
					eventName,
					eventNames = t._eventNames,
					len = eventNames.length,
					i = 0; i < len; ++i){
				eventName = eventNames[i];
				m._cnnts.push(on(node, eventName.toLowerCase(), lang.hitch(scope, connector, eventName)));
			}
		},
	
		_isConnected: function(eventName){
			return this[eventName] !== dummyFunc;
		},
		//event handling end

		_isCopyEvent: function(evt){
			// summary:
			//		On Mac Ctrl+click also opens a context menu. So call this to check ctrlKey instead of directly call evt.ctrlKey
			//		if you need to implement some handler for Ctrl+click.
			return has("mac") ? evt.metaKey : evt.ctrlKey;
		}
	});

	Grid.markupFactory = function(props, node, ctor){
		if(!props.structure && node.nodeName.toLowerCase() == "table"){
			kernel.deprecated('Column declaration in <th> elements is deprecated,', 'use "structure" attribute in data-dojo-props instead', '1.1');
			var s = props.structure = [];
			query("thead > tr > th", node).forEach(function(th){
				var col = {};
				forEach(_Module._markupAttrs, function(attr){
					if(attr[0] == '!'){
						attr = attr.slice(1);
						col[attr] = eval(th.getAttribute(attr));
					}else{
						col[attr] = th.getAttribute(attr);
					}
				});
				col.name = col.name || th.innerHTML;
				s.push(col);
			});
		}
		return new ctor(props, node);
	};
	
	return Grid;
});

},
'dojoe/treetable/_IndirectSelection':function(){
// wrapped by build app
define(["dijit","dojo","dojox"], function(dijit,dojo,dojox){

dojo.provide("dojoe.treetable._IndirectSelection");
//dojo.require("dojox.widget.Standby");

// define the widget class
dojo.declare("dojoe.treetable._IndirectSelection", null, {
	// Summary:
	//		Class for enabling Indirect selection for tree table
	// description:
	//		modifies grid structure to include a selection column to the very left of the tree table
	//		updates grid cache with selection status on grid events
	// tags: private
	
	//Tree table grid component - to be passed when indirect selection is instantiated	
	grid : null, 
	//private - temporary selection cache which is used to update the grid cache on events like refresh,resize etc..
	_selectionCache : [], 
	
	//_standBy : null,
	
	_updateSelectionByCacheRunning : false,
	
	_connectArr: null,
		
	constructor : function (/*object*/grid) {
		//Summary:
		//Lazy tree grid should be passed as parameter to constructor which is modified to include indirect selection
		//console.debug('here in constructor');
		this.grid = grid;		
		this._connectArr = new Array();		
		//add indirect selection column to the grid
		//this._selectionCache = (this.grid.cache.items).slice();
		this._addIndirectSelector();
		//handle grid events and update the selection accordingly
		this._connectSelectionHandlers();
		/*this._standBy  = new dojox.widget.Standby({
			target: this.grid.id
		});
        document.body.appendChild(this._standBy.domNode);
        this._standBy.startup();*/		
	}, 
	
	_addIndirectSelector : function () {
		//Summary:
		//Include selection column in Lazy tree grid
		//tags: Private		
		this.grid.expandoCell = 1;
		selector = [{
				//Bobby:28Mar2011
				//dojox.data.QueryReadStore.getValue(): Item does not have the attribute 'selector'
				//field: "selector", - disabling this
				//name: '<div dojoAttachPoint="select" align="right" class="treeTableCheckBox" dojoAttachEvent="onclick:selectAllRows">', 
				//Bobby: 09Mar2011, applying similar style as Table indirect selector
				//name: '<div id="selectDiv" align="center" class="treeTableCheckBox">', 
				//Bobby: 17Mar2011, removing selectAll from header as sort can't be disabled
				//Anupama: 12Sept2011
				// Defect 27514:: Safari:In Treetable, Selection column is truncated on initial load.
				//changed width to display selection column
				name : ' ', 
				width : "22px", 
				styles : 'text-align: center;', 
				formatter : this._createCheckbox
			}
		];
		this.grid.structure = dojo._toArray(this.grid.structure, 0, selector);
		this.grid.setStructure(this.grid.structure);
		this.grid._refresh(true);
	}, 
	
	_createCheckbox : function (val, inRowIndex, treePathArr, cell) {
		//Summary:
		//formatter for selection column
		//tags:private
		//return: formatted checkbox
		//var store=this.grid.store;
		//var item = this.grid.getItem(inRowIndex);
		var id = this.grid.store.getIdentity(this.grid.getItem(inRowIndex));
		var indimgid = "treeimgid" + id;
		return '<div align="right" id=' + indimgid + ' class="treeTableCheckBox">';
	}, 
	
	_updateCacheSelect : function (ridx, item, selStatus) {
		//Summary:
		//updates grid cache items with selection status - true,false,'partial'
		//tags:private
		var id = this.grid.store.getIdentity(item);
		var expandoStatus = this.grid.cache.getExpandoStatusByRowIndex(ridx);
		var itemIndex = this._getItemIndexById(id);
		//console.info("_updateCacheSelect");
		//console.info("selStatus:"+selStatus);
		if (itemIndex !== -1 && this._selectionCache[itemIndex]) {
			//console.info("IF - calling _updateCache");
			this._updateCache(itemIndex, {
					selStatus : selStatus,
					item : item,
					rowIdx : itemIndex,
					id : id,
					expandoStatus : expandoStatus
				});
		} else {
			//console.info("ELSE- calling _cacheItem");
			//dojo.mixin(this._selectionCache[ridx], {selStatus: selStatus, item: item, rowIdx: ridx, id: id});
			this._cacheItem({
					selStatus : selStatus,
					item : item,
					rowIdx : ridx,
					id : id,
					expandoStatus : expandoStatus
				});
		}
	}, 
	
	_cacheItem : function (cacheObj) {
		// Summary
		//		Add an item and its tree structure information to the cache.
		//console.info("_cacheItem --------- BEGIN");
		var idx = this._getItemIndexById(cacheObj.id);
		if (idx == -1) {
			this._selectionCache.push(dojo.mixin({
						id : null, 
						selStatus : false, 
						rowIdx : -1, 
						expandoStatus : false
					}, cacheObj));
		} else {
			//Bobby:27Mar11
			//_updateCache -> updateCache
			this._updateCache(idx, cacheObj);
		}
		//console.info("_cacheItem --------- END");
	}, 
	
	_updateCache : function (/*integer*/idx, cacheObj) {
		//Summary
		//update selection cache
		//console.info("_updateCache --------- BEGIN");
		if (this._selectionCache[idx]) {
			dojo.mixin(this._selectionCache[idx], cacheObj);
		}
		//console.info("_updateCache --------- END");		
	}, 
	
	_getItemIndexById : function (id) {
		// check to see item is already in and return the index	
		for (var i = 0; i < this._selectionCache.length; i++) {
			if (this._selectionCache[i]) {
				if (this._selectionCache[i].id == id) {
					return i;
				}
			}
		}
		return - 1;
	}, 
	
	_getItemById : function (/*id*/rowId) {
		//added new interface for grid cache to fetch item by row id for selection rather than index which is faulty
		//this takes care of dojo defect of lazytreegrid selection
		for (var i = 0, len = this._selectionCache.length; i < len; i++) {
			if (this._selectionCache[i] && this._selectionCache[i].id
				 && (this._selectionCache[i].id === rowId)) {
				return this._selectionCache[i];
			}
		}
		return null;
	}, 
	
	isItemSelectedInCacheByIdx : function (rowIdx) {
		//Summary:
		//returns if a particular row is selected in cache by row index
		//Row selection should be tagged to row id rather than row index(which might be incorrect when tree is expanded/collapsed)
		//takes care of dojo defect of wrong grid selection by row index
		//console.info("isItemSelectedInCacheByIdx --------- BEGIN");	
		var id, sel, item = this.grid.getItem(rowIdx);
		if (item) {
			id = this.grid.store.getIdentity(item);
			var cachedItem = this._getItemById(id);
			if (cachedItem) {
				sel = cachedItem.selStatus;
			}
		}
		//console.info("isItemSelectedInCacheByIdx --------- END");	
		return sel;
	}, 
	
	getSelectedStatusById : function (id) {
		// If in the cache, return the selection status, otherwise return undefined
		var i = this._getItemIndexById(id);
		if (i != -1) {
			return this._getSelectedStatusByIndex(i);
		}
	}, 
	
	_getSelectedStatusByIndex : function (/*integer*/idx) {
		return this._selectionCache[idx] ? this._selectionCache[idx].selStatus : null;
	}, 
	
	_connectSelectionHandlers : function () {
		//Summary:
		//handling grid events to update selection in grid cache
		//tags: private
		this.grid.selection.clickSelect = function(){}; //disabling default selection			
		this._connectArr.push(dojo.connect(this.grid, 'expandoFetch', dojo.hitch(this, "_updateCacheOnExpando"))); //expand or collapse
		//Bobby:16Mar2011. Indirect selection should happen only when clicked on selector cell
		//dojo.connect(this.grid, 'onRowClick', dojo.hitch(this, "onSelectionChanged"));					
		this._connectArr.push(dojo.connect(this.grid, 'onCellClick', dojo.hitch(this, "onSelectionChanged"))); //indirect selection					
		this._connectArr.push(dojo.connect(this.grid, 'onKeyDown', dojo.hitch(this, "onSelectionChanged"))); //indirect selection
		this._connectArr.push(dojo.connect(this.grid, 'onSelectionChanged', dojo.hitch(this, "onSelectionChanged")));
		//Bobby:15Mar2011
		this._connectArr.push(dojo.connect(this.grid, 'onHeaderClick', this, "_updateSelectionOnRefresh")); //sorting
		this._connectArr.push(dojo.connect(this.grid, 'onResizeColumn', this, "_updateSelectionOnRefresh")); //column resize			
		this._connectArr.push(dojo.connect(this.grid, '_refresh', dojo.hitch(this, "_updateSelectionOnFetch"))); //grid refresh	
		this._connectArr.push(dojo.connect(this.grid, '_onFetchComplete', dojo.hitch(this, "_updateSelectionOnFetch")));//on Fetch complete update selection
		//dojo.connect(this.grid, '_onExpandoComplete', dojo.hitch(this, "_updateSelectionOnFetch")); //on Fetch complete update selection
		this._connectArr.push(dojo.connect(this.grid, 'resize', dojo.hitch(this, "_updateSelectionOnRefresh"))); //resize				
		//Bobby:15Apr2011
		//14932: Selection goes off when a new row is added
		this._connectArr.push(dojo.connect(this.grid, 'updateRow', dojo.hitch(this, "_updateSelectionOnRefresh"))); //store events	
		this._connectArr.push(dojo.connect(this.grid, 'updateRowCount', dojo.hitch(this, "_updateSelectionOnRefresh"))); //store events	
	}, 
	
	_updateSelectionOnFetch : function (items, request, size) {
		//summary:
		//handles selection update on fetch
		//tags:private
		//console.log('_updateSelectionOnFetch ------------- BEGIN');
		//var tempCache = this._selectionCache;
		//var treePath = "",startRowIdx, count, start;
		//this._standBy.show();
		var count;
		if (request) {
			startRowIdx = request.startRowIdx;
			count = request.count;
			start = 0;
		}
		//for _refresh and _fetchOnComplete events there is a second call which happens when cache is empty and count 0, avoid updating on this scenario
		//also avoid refresh on expando fetch, expando fetch has its own handler 
		//console.info("count:"+count);
		//console.info("grid.rowCount:"+this.grid.rowCount);
		//Bobby: 28Mar11
		//using refresh Icon, this.count=0, so disabling the check
		//if(count &&  count > this.grid.rowCount){
		if (count &&  count > 0){
			this._updateSelectionOnRefresh();
		}
		//this._standBy.hide();
		//console.log('_updateSelectionOnFetch ------------- END');
	}, 
	
	selectAllRows : function () {
		//summary:
		//connected to toolbar select all action and selects all the rows in the grid
		var headerdiv = dojo.byId("selectDiv");
		var rowcnt = this.grid.rowCount;
		for (var i = 0; i < rowcnt; i++) {
			var item = this.grid.getItem(i);
			var id = this.grid.store.getIdentity(item);
			var indimgid = "treeimgid" + id;
			var rowimage = dojo.byId(indimgid);
			//updateCache
			this._updateCacheSelect(i, item, true);
			//update temp cache with grid cache status
			//this._selectionCache = (this.grid.cache.items).slice();
		}
		this._updateParent();
		this._updateSelectionByCache();
	}, 
	
	deSelectAllRows : function () {
		//Summary:
		//de-selects all rows connected to toolbar deselect action
		var headerdiv = dojo.byId("selectDiv");
		var rowcnt = this.grid.rowCount;
		for (var i = 0; i < rowcnt; i++) {
			var item = this.grid.getItem(i);
			this._updateCacheSelect(i, item, false);
		}
		this._updateParent();
		this._updateSelectionByCache();
		//this._selectionCache = (this.grid.cache.items).slice();
	}, 
	
	onSelectionChanged : function (e) {
		//Summary:
		//fired when row selection changes by clicking on indirect selection column
		//console.log('onSelectionChanged ------------- BEGIN');
		
		if (e) { //selection changed by user
			//Bobby:16Mar11, select only if enter pressed
			if (e.keyCode && e.keyCode != 13) {
				//console.log('Not Enter key');
				//console.log('onSelectionChanged ------------- END');
				return;
			}
			//Bobby:16Mar11, if not clicked on selector cell, do nothing
			if (e.cellIndex != 0) {
				//console.log('Not selector cell');
				//console.log('onSelectionChanged ------------- END');		 
				return;
			}
			//e.cancelBubble=true;
			//console.log("if (e) - true");
			var item = this.grid.getItem(e.rowIndex);
			var sel = this._isRowSelected(e.rowIndex);
			if (sel != undefined) { // only care about checkbox click, ignore other
				this._changeSelectionByRow(e);
			}
		} else { //selection changed via API
			//console.log("if (e) - false");
			//this._standBy.show();
			this._updateSelectionByCache();
			//this._standBy.hide();
		}
		
		//this._updateSelectionByCache();
		//console.log('onSelectionChanged ------------- END');
		//this._updateCache();
		//this._initCache();
		
		// this._endUpdate();
		// this.onSelectionDone();
		
	}, 
	
	_updateCacheOnExpando : function (rowIndex, openState) {
		//Summary:
		//fired on 'expandoFetch' to update the selection on fetch of child items
		//tags: Private
		//console.log('_updateCacheOnExpando ------------- BEGIN');
		//this._standBy.show();
		if (openState) {
			var item = this.grid.getItem(rowIndex);
			//console.log("item:"+item);
			var id = this.grid.store.getIdentity(item);
			var cachedItem = this._getItemById(id);
			//console.log("cachedItem -->"+ cachedItem);
			if (cachedItem) {
				var selectflag = cachedItem.selStatus;
				//console.log("selectflag -->"+ selectflag);
				if (selectflag !== 'partial') {
					//var children = this.grid.store.getValues(item, 'children');
					//console.log("going to call mayHaveChildren..");
					if (this.grid.treeModel.mayHaveChildren(item)) {
						//console.info("has children!");
						//var selectflag = cachedItem.selStatus;
						//call recursive child select with selection status of parent
						this._selectChildren(item, rowIndex, selectflag);
						
					}
								
				} else {
					this._updateSelectionByCache();
				}
			}else{
				this._updateSelectionByCache();
			}
			//var tempCache = (this.grid.cache.items).slice();
			//this._selectionCache = (this.grid.cache.items).slice();
		} else {
			this._updateSelectionByCache();
		}		
		//this._standBy.hide();
		//console.log('_updateCacheOnExpando ------------- END');
	}, 
	
	_changeSelectionByRow : function (e) {
		//changes the indirect selection checkboxes depending on the selection status
		
		//console.info('-_changeSelectionByRow ---------- BEGIN');
		//var store = e.grid.store;
		//var row = e.rowIndex;
		var rowIndex = e.rowIndex;
		var item = this.grid.getItem(rowIndex);
		
		//console.dir(item);
		var isParent = false;
		var selectflag = false;
		var id = this.grid.store.getIdentity(item);
		
		//console.info("expando Status:"+this.grid.cache.getExpandoStatusByRowIndex(rowIndex));
		//var children = this.grid.store.getValues(item, 'children');
		//if (children && children != true && children.length>0) {
		if (this.grid.cache.getExpandoStatusByRowIndex(rowIndex) && this.grid.treeModel.mayHaveChildren(item)) {
			isParent = true;
			//console.info("IF calling _updateSelection..");
			selectflag = this._updateSelection(item, id, rowIndex);
			this._selectChildren(item, rowIndex, selectflag);
			
		} else {
			//console.info("ELSE calling _updateSelection..");
			this._updateSelection(item, id, rowIndex);
			this._updateParent();
			this._updateSelectionByCache();
		
		}
		//console.info('_changeSelectionByRow ---------- END');
		
	}, 
	
	_updateSelectionOnRefresh : function () {
		//console.info('-_updateSelectionOnRefresh ---------- BEGIN');
		//this._standBy.show();
		this._updateSelectionByCache();
		//this._updateParent();
		//this._updateSelectionByCache();
		//this._standBy.hide();		
		//console.info('-_updateSelectionOnRefresh ---------- END');
	}, 
	
	_updateSelectionByCache : function () {
		//summary:
		//update grid selection by cache 
		//tags: private
		//console.info('_updateSelectionByCache ---------- BEGIN');
		
		for (var i = 0; i < this.grid.rowCount; i++) {
			var sel = this.isItemSelectedInCacheByIdx(i);
			var item = this.grid.getItem(i);
			//console.dir(item);
			//console.log("item "+i+":"+item);
			if(!item) continue;
			var id = this.grid.store.getIdentity(item);
			//var indimgid="treeimgid"+id;
			var rowimage = dojo.byId("treeimgid" + id);
			
			if (sel == true) {
				this.grid.selection.addToSelection(i);
				if (dojo.hasClass(rowimage, "treeTableCheckBox")) {
					dojo.removeClass(rowimage, ["treeTableCheckBox"]);
				} else if (dojo.hasClass(rowimage, "treeTableToggleButtonPartial")) {
					dojo.removeClass(rowimage, ["treeTableToggleButtonPartial"]);
				}
				dojo.addClass(rowimage, ["treeTableToggleButtonChecked"]);
			} else if (sel == false) {
				this.grid.selection.deselect(i);
				if (dojo.hasClass(rowimage, "treeTableToggleButtonChecked")) {
					dojo.removeClass(rowimage, ["treeTableToggleButtonChecked"]);
				} else if (dojo.hasClass(rowimage, "treeTableToggleButtonPartial")) {
					dojo.removeClass(rowimage, ["treeTableToggleButtonPartial"]);
				}
				dojo.addClass(rowimage, ["treeTableCheckBox"]);
			} else if (sel == 'partial') {
				//console.debug('selection is partial: '+ sel);	
				this.grid.selection.deselect(i);
				if (dojo.hasClass(rowimage, "treeTableToggleButtonChecked")) {
					dojo.removeClass(rowimage, ["treeTableToggleButtonChecked"]);
					
				} else if (dojo.hasClass(rowimage, "treeTableCheckBox")) {
					dojo.addClass(rowimage, ["treeTableToggleButtonPartial"]);
				}
				dojo.addClass(rowimage, ["treeTableToggleButtonPartial"]);
			} else {
				if (this.grid.selection.isSelected(i)) {
					this.grid.selection.deselect(i);
				}
			}
			//this._updateSelectionByCacheRunning = false;
		}
		//console.info('_updateSelectionByCache ---------- END');
	}, 
	
	_updateParent : function () {
		//Summary:
		//updates parent item when child is selected/deselected - totally/partially
		//tags: private
		//console.info('_updateParent ---------- BEGIN');
		//var rowcnt = this.grid.rowCount;
		for (var i = this.grid.rowCount-1; i >= 0; i--) {
			var numSel = 0;
			var item = this.grid.getItem(i);
			if(!item) continue;
			var id = this.grid.store.getIdentity(item);			
			//Bobby: 27Mar2011, following won't work in all stores.
			//var children = store.getValues(item, 'children');
			//if collapsed do nothing
			if (this.grid.cache.getExpandoStatusByRowIndex(i) === false) {
				continue;
			}
			//if expanded, go fetch the children and process further
			if (this.grid.treeModel.mayHaveChildren(item)) {
				var queryObj = {
					start : 0, 
					count : this.grid.keepRows, //or this.grid.rowCount?
					parentId : id, 
					parentRowIndex : i, 
					sort : this.grid.getSortProps() 
				};
				//console.info("Calling getChildren.....");
				if(this.grid.treeModel.serverStore){
					this.grid.treeModel.getChildren(item, dojo.hitch(this, "_updateParentRecursive"), dojo.hitch(this, "_onError"), queryObj);
				
				}
				else{
					//for clientstore treemodel does not set queryobj to callback
					var onComplete = dojo.hitch(this, "_updateParentRecursive");
					this.grid.treeModel.getChildren(item, 
													function(childItems){
														onComplete(childItems,queryObj);
													}, 
													dojo.hitch(this, "_onError"),
													queryObj
													);
				}
				//console.info("Called getChildren.....");
			}
		}
		//console.info('_updateParent ---------- END');
	}, 
	
	_updateParentRecursive : function (children, queryObj, size) {
		//Summary:
		//updates parent item when expanded
		//tags: private
		//console.info('_updateParentRecursive ---------- BEGIN');
		//console.info("children:"+children);
		//console.info("queryObj:"+queryObj);
		//console.info("size:"+size);		
		var item = this.grid.getItem(queryObj.parentRowIndex);
		var selStatus = false;
		//var indimgid="treeimgid"+id;
		var numSel = 0;		
		for (j = 0; j < children.length; j++) {
			var cId = this.grid.store.getIdentity(children[j]);
			var selStatus = this.getSelectedStatusById(cId);
			//console.debug('updateparent: selStatus of children: '+ selStatus);
			if (selStatus === true) {
				numSel++;
			}
			else if (selStatus === 'partial'){
				selStatus = 'partial';
				break;
			}
		}
		
		//console.info("numSel:"+numSel);
		//console.info("children.length:"+children.length);
		if (numSel > 0) {
			if (children.length == numSel) { // all children selected 
				this._updateCacheSelect(queryObj.parentRowIndex, item, true);
			} else {
				this._updateCacheSelect(queryObj.parentRowIndex, item, 'partial');
			}
		} else {
			if (selStatus == 'partial') {
				this._updateCacheSelect(queryObj.parentRowIndex,item, 'partial');
			}
			else {
				this._updateCacheSelect(queryObj.parentRowIndex,item, false);
			}
			//this._updateCacheSelect(queryObj.parentRowIndex, item, false);
		}
		this._updateSelectionByCache();
		//console.info('_updateParentRecursive ---------- END');
	}, 
	
	_isRowSelected : function (idx) {
		//Summary:
		// check the cache before checkbox, works like toggle
		//tags:private
		var sel = this.isItemSelectedInCacheByIdx(idx);
		var item = this.grid.getItem(idx);
		var id = this.grid.store.getIdentity(item);
		var indimgid = "treeimgid" + id;
		var rowimage = dojo.byId(indimgid);
		
		if (sel == undefined) {
			// checkbox.value hasn't not updated yet, only checked reflect the correct state
			if (dojo.hasClass(rowimage, "treeTableToggleButtonChecked")) {
				return true;
			} else {
				return false;
			}
		} else {
			//if (sel === true || sel === false) 
				return sel;
		}
	}, 
	
	_updateSelection : function (item, id, rowIndex) {
		//Summary:
		//update selection in cache on toggle
		//console.info('_updateSelection ------ BEGIN');
		var selectFlag;
		//console.dir(item);
		if (this.getSelectedStatusById(id) === false) {
			this._updateCacheSelect(rowIndex, item, true);
			selectFlag = true;
		} else if (this.getSelectedStatusById(id) === true) {
			this._updateCacheSelect(rowIndex, item, false);
			selectFlag = false;
		} else if (this.getSelectedStatusById(id) == 'partial') {
			this._updateCacheSelect(rowIndex, item, true);
			selectFlag = true;
		} else {
			this._updateCacheSelect(rowIndex, item, true);
			selectFlag = true;
		}
		//console.info('_updateSelection ------ END');
		return selectFlag;
	}, 
	
	_selectChildren : function (item, rowIndex, selectflag) {
		//console.info('_selectChildren ------ BEGIN');
		//this.parentItem = item;
		//this.parentRowIndex = rowIndex;
		//this.parentSelect = selectflag;
		/*if(!this.count){
		this.count = rowIndex + childItems.length;
		}*/		
		//console.dir(item);		
		var parentId = this.grid.store.getIdentity(item);
		var queryObj = {
			start : 0, 
			count : this.grid.keepRows, 
			parentId : parentId, 
			sort : this.grid.getSortProps(), 
			parentRowIndex : rowIndex, 
			parentSelectFlag : selectflag
		};
		if(this.grid.treeModel.serverStore){
			this.grid.treeModel.getChildren(item, dojo.hitch(this, "_recursiveChildSelect"), dojo.hitch(this, "_onError"), queryObj);
		}else{
			//for clientstore treemodel does not set queryobj to callback
			var onComplete = dojo.hitch(this, "_recursiveChildSelect");
			this.grid.treeModel.getChildren(item, 
											function(childItems){
												onComplete(childItems,queryObj);
											}, 
											dojo.hitch(this, "_onError"),
											queryObj
											);
		}
		//console.info('_selectChildren ------ END');
	}, 
	
	_getChildren : function (item) {
		var parentId = this.grid.store.getIdentity(item);
		var queryObj = {
			start : 0, 
			count : this.grid.keepRows, 
			parentId : parentId, 
			sort : this.grid.getSortProps() 
		};
		this.grid.treeModel.getChildren(item, dojo.hitch(this, "_setChildren"), dojo.hitch(this, "_onError"), queryObj);	
	}, 
	
	_setChildren : function (childItems, request, size) {
		this.children = childItems;
	}, 
	
	_onError : function (e) {
		//console.error('Fetch Children error: '+ e);
	}, 
	
	_recursiveChildSelect : function (childItems, request, size) {
		//Summary:
		//recursively select children if a parent is selected or deselected , expanded/collapsed
		//tag:private
		//console.info("_recursiveChildSelect ---------  BEGIN");
		//console.info("childItems:"+childItems);
		//console.info("request:"+request);
		//console.info("size:"+size);	
		
		var rowIndex = request.parentRowIndex;
		var selectflag = request.parentSelectFlag;
		var expand = this.grid.cache.getExpandoStatusByRowIndex(rowIndex);
		var childSelect = dojo.hitch(this, function (childItems, expand, rowIndex, selectflag) {
				if (childItems && expand === true) {
					for (var i =rowIndex+1, j=0; j < childItems.length; j++) {
						var item = childItems[j];
						var id = this.grid.store.getIdentity(item);
						var children = this.grid.treeModel.mayHaveChildren(item);
						var childExpand = this.grid.cache.getExpandoStatusByRowIndex(i);
						if (children && childExpand) {
							//var childSelStatus = selectflag;
							//childSelect(gchildren,childExpand,i,childSelStatus);
							this._selectChildren(item,i, selectflag);
							i= i+j;
							
						}else{
							i++;
						}
						if (selectflag === false) {
							this._updateCacheSelect(i,item, false);
						} else if (selectflag === true) {
							this._updateCacheSelect(i, item, true);
						}
					}
				}
				return;
			});
		//recursively call to select children if expandoStatus is true
		if (childItems && expand === true) {
			childSelect(childItems, expand, rowIndex, selectflag);
		}
		this._updateParent();
		this._updateSelectionByCache();
		//console.info("_recursiveChildSelect ---------  END");
	},
	
	destroy: function(){
		// summary:  Destroys the toolbar.
		// tags: public	
		//memory leak issue: custom destroy method to take care of removing all widgets					
		try{
			dojo.forEach(this._connectArr, dojo.disconnect);
			this._connectArr = [];
		}catch(ex){
			console.warn("Destroy failed: "+this.id+", ex="+ex.name+":"+ex.message);
		}
	}
});
 
});

},
'dojoe/tablex/Table':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/array",
    "dijit/_WidgetBase",
    "dijit/_TemplatedMixin",
    "dojo/text!./templates/Table.html",
    "gridx/Grid",
    "dojo/_base/lang",
	"dojo/dom-geometry",
	"dojo/window",
	"dojo/dom-style",
	"dojox/collections/ArrayList",
    "dojoe/tablex/_Toolbar",
    "dojo/i18n",
    "dojoe/slidingpane/SlidingPane",
    "dojo/i18n!dojoe/tablex/nls/Table",
	"gridx/core/model/cache/Sync",
	"gridx/core/model/cache/Async",
	"dojoe/tablex/plugins/_MapToModules",
	"gridx/modules/Persist"
		
        ], function(Declare, Array, _Widget, _TemplatedMixin, Template, Grid, Lang, DomGeometry,
        			Window, DomStyle, ArrayList, _Toolbar, Il8n, SlidingPane, NLS, Cache, ServerCache, _MapToModules, Persist ){
		
			var forEach = Array.forEach;
	
			Table = Declare('dojoe.tablex.Table', [_Widget, _TemplatedMixin], {
				// summary:  
				// 		Table widget is used to display potentially large, multivariate, structured data sets.
				//
				// description: 
				//	Table widget implementation is based on gridx
				//	Provides additional features upon gridx like toolbar with state control buttons (refresh)
				//	custom actions drop down and Filter(search) controls.
				//	Usage
				//	example:
				//		Usage example 1: Using Plugins (compatible way)
				//
				//		Main advantage of this format is to be able to use plugins as it is with the old EDG based table
				//		Differences from the EDG based Table configuration in comatibility mode: 
				//		layout: new gridx "structure" expects the layout info in different format (formatters are now called decorators, no hidden parameter support and editor support)
				//		Please refer to gridx documentation for layout information
				//		Please note that "gridParams_" holds gridx parameters and hence subject to change when dojo api/attributes changes
				//
				// |		var table = new Table({
				// |											  gridParams_ : {
				// |															rowsPerPage : 5,
				// |															store : jsonStore1,
				// |															structure : layout,
				// |															plugins :{
				// |																	  indirectSelection : true,
				// | 																	  nestedSorting : true,
				// |																	  dnd : true
				// |																     }
				// |															},
				// |															toolbarOptions : {},
				// |															enableResize : true
				// |											  }, domNode);	
				//	
				//		example:
				//		Usage example 2: Using Modules (new Gridx modules)
				// |		var grid = new Grid({
		        // |						cacheClass: Cache,
		        // |						store: store,
		        // |            			structure: columns,
		        // |            			modules: [
		        // |					   	 		Sort,       //Declare modules in a grid instance
		        // |					   	 		ColumnResizer,   //Declare modules in a grid instance
				// |								IndirectSelect,
				// |								NestedSort
		        // |						]
				// |					});
										
				// templateString:  String
				// Path to the template
				templateString : Template,
			
				// widgetsInTemplate: Boolean
				//		Set this to true if the widget contains other widgets
				widgetsInTemplate : true, 
				
				// toolbarVisible: Boolean
				//		Controls the visibility of the toolbar.
				toolbarVisible : true, 
				
				// toolbarOptions:  Object
				//		Holds the  information about the toolbar options passed by the user.
				toolbarOptions : null, 
				
				// filterOptions:  Object
				//		Holds all the filter information passed by the user.
				filterOptions : {}, 
				
				// enableResize: Boolean
				//		Controls the grid resize  on window resize event. 
				//		If true enables the table resize when the window resizes ,if false disable table resize on window resize event.
				enableResize : false,
				
				// gridParams:  Object
				//		Holds  parameters passed to grid by the user. These are dojo gridx parameters and subject to change when dojo api/attributes changes.	
				gridParams_ : null, 
				
				// grid: Object
				//		 gridx/Grid handle. Avoid using the underlying grid directly.
				// tags: private
				_grid : null, 
				
				//Enabled by default, set false to disable the default exportFormatter and set yours
				defaultExportFormatter : true, 
				
				// pageSizeArr:  array
				//		An array holds the size of the page.
				pageSizeArr : null, 
				
				// width: number  
				//		Holds the width of the grid set by user.  It can be set in px,percentage or em like '50px','50%' or '50em'.
				width : '', 
				
				// height: number
				//		Holds the height of the grid set by user. It can be set in px,percentage or em like '150px','150%' or '150em'.
				height : '', 
				
				// _toolbar: Object
				//		Reference to the toolbar
				// tags: private
				_toolbar : null, 
				
				// _defaultHeight:  number
				//		Holds the default height of the grid. 
				// tags: private	
				_defaultHeight : '70%', 
				
				// _defaultWidth: number
				//		Holds the default width of the grid
				// tags: private	
				_defaultWidth : '80%', 
				
				// footerdescription: boolean
				//		Description of the grid footer 
				footerDescription: true,
				
				// footerVisible: boolean
				//		Holds the visibility status of the footer
				footerVisible:true,
				
				// resources_:  Object
				//		Reference to the resources
				// tags: private
				resources_ : null, 
				
				// _previousSize: Object
				// Reference value for resize
				// tags: private
				_previousSize : null,
				
				// _resizing: boolean
				// flag to indicate resize is in progress
				// tags: private
				_resizing : false,
				
				// enableCollapsibleToolbar: boolean
				//To add collapsible toolbar  to the table
				enableCollapsibleToolbar : false,
				
				// toolbarExpand: boolean
				//To indicate(preset) the state of the toolbar on load
				toolbarExpand : true,
				
				// _originalStructure: original Structure set to the table
				// tags: private
				_origLayout : null,
				
				// _currentLayout: layout object after filtering columns with hidden parameters
				// tags: private
				_currentLayout : null,
				
				// persistOptions: Boolean - if enabled persists config options settings and column properties
				persistOptions : false,
				
				
				constructor : function () {
					this.resources_ = NLS;
					this.pageSizeArr = ["5", "10", "25", "50", "100", "All"];
				}, 
				
				postCreate : function () {
					// summary:  Creates and initializes the table.
					// tags: extension
					
					if (this.width === '' && !this.gridParams_.autoWidth) {
						this.width = this._defaultWidth;
					}
					if (this.height === '' && !this.gridParams_.autoHeight) {
						this.height = this._defaultHeight;
					}
					
					var size = Window.getBox();
					var heightPx = this.height;
					var sizeType = this.height.substring(this.height.length - 1, this.height.length);
					if (sizeType == "%") {
						//convert to px
						heightPx = size.h * ((parseInt(this.height.slice(0, -1))) / 100);
						heightPx = heightPx + "px";
					}
					if (!this.gridParams_.autoWidth) {
						DomStyle.set(this.domNode, {width: this.width});
					}
					DomStyle.set(this.domNode, "height", heightPx);
					
					this._makeGrid();
					if (this.toolbarVisible) {
						this._toolbar = new _Toolbar({
								tivTable : this
							}, this.toolbarNode);
							
						if (this.enableCollapsibleToolbar) {
							this.slidingPane = new SlidingPane({closeNodeClass: "slidingPaneClose", open: this.toolbarExpand}, this.wipeNode);
							this.connect(this.slidingPane, "onClick", Lang.hitch(this, "slidingPaneClicked"));//Connecting event onClick to sliding pane.
						}
					
					} else {
						if(this.filterOptions.hideFilterBar == true && this._grid.filterBar !== undefined){
							this._grid.filterBar.hide();
						} 
					} 
					if(this._grid.filterBar){
						// hack until dojo team fixes the issue on gridx or provides a new flag to disable autocomplete
						this._grid.filterBar.autoComplete = false;
					}
					this.resize();
					if (this.enableResize === true) {
						this.connect(window, "onresize", "resize");
					}
					if(this._toolbar && this._toolbar.actionListWidget){	
						this.connect(this._toolbar.actionListWidget, 'openDropDown', Lang.hitch(this,"onActionListOpen")); 
						this.connect(this._toolbar.actionListWidget, 'closeDropDown', Lang.hitch(this,"onActionListClose")); 			
					}
					
					setTimeout(Lang.hitch(this, "onTableLoad", this), 1000);
					
				}, 
				//Connect to this to monitor Table load complete.
				onTableLoad : function (table) {
					// summary: 
					//		Called after the Table loads
					// tags:
		   		    //      callback
					console.info("Table load complete....." + table.id);
				}, 
				
				//Connect to this to monitor Opening of actionList
				onActionListOpen : function(){
					// summary: 
					//		Called onClick of "Actions" Drop down menu
					// tags:
		   		    //      callback

					//console.info("actionListOpen.....");
				},
				
				//Connect to this to monitor Closing of actionList
				onActionListClose : function(){
					// summary: 
					//		Called onClose of "Actions" Drop down menu
					// tags:
		   		    //      callback
		   		    
					//console.info("actionListClosed.....");
				},		
				
				_makeGrid : function () {
					// summary:  Creates grid with the specified plugins and parameters.
					// tags: private	
					
					this._mapgridParams(); // this function is used to map ted plugins params with EnhancedGrid plugin params which may change frequently.
					
					this._mapToModules();
					if(!this.gridParams_.cacheClass){
						this.gridParams_.cacheClass = ServerCache;
					}
					//setting the Printer plugin when the Exporter plugin is set to make the Export to HTML work with css
					if (this.gridParams_.plugins && this.gridParams_.plugins.exporter === true && !this.gridParams_.plugins.printer) {
						var tempParams = this.gridParams_;
						tempParams.plugins.printer = true;
						this._grid = new grid(tempParams, this.enhancedGridNode);
						delete tempParams.plugins.printer;
					} else {
						this._grid = new Grid(this.gridParams_, this.enhancedGridNode);
					}
					//startup and resize has problems when grid is added to a custom widget _widget
					//so set appropriate height and width to grid domNode, update and parse the domNode after resize
					if (this.gridParams_.preFilter) {				
						this.setFilter(this.gridParams_.preFilter, this.gridParams_.preFilter.hideFilterBar);
					}
					//to display the default message when grid is empty or filter result is none
					if (!this.gridParams_.noDataMessage) {
						this._grid.noDataMessage = this.resources_.NO_DATA;
					} else {
						this._grid.noDataMessage = this.gridParams_.noDataMessage;
					}					
						
					this._grid.startup();
					// use onModulesLoaded callback to ensure the Persist module is loaded before operating on the columns
					this.connect(this._grid,"onModulesLoaded", Lang.hitch(this,"_persistColumns"));
					
					if(this.gridParams_.plugins && this.gridParams_.plugins.menus){
						var menus = this.gridParams_.plugins.menus;
						if(menus.headerMenu){
							var menuArgs = {hookPoint: "headercell"};
							this._bindMenu(menus.headerMenu,menuArgs );
						}
						if(menus.rowMenu){
							var menuArgs = {hookPoint: "row"};
							this._bindMenu(menus.rowMenu,menuArgs );
						}
						if(menus.selectedRegionMenu){
							var menuArgs = {hookPoint: "row", selected: true};
							this._bindMenu(menus.selectedRegionMenu,menuArgs );
						}
						
					}
					//not needed as there is a direct mapping available with gridx module
					/*if(this.gridParams_.plugins && this.gridParams_.plugins.pagination ){
						if(this.gridParams_.plugins.pagination.defaultPageSize && this._grid.pagination){
								this._grid.pagination.setPageSize(this.gridParams_.plugins.pagination.defaultPageSize );
						}
					}*/
					if(this.gridParams_.plugins && this.gridParams_.plugins.filter ){
						/*if(this.gridParams_.plugins.pagination.defaultPageSize && this._grid.pagination){
								this._grid.pagination.setPageSize(this.gridParams_.plugins.pagination.defaultPageSize );
						}*/
					}
				}, 
				_persistColumns : function () {
					// summary:   
					//	If persistOptions is enabled this is invoked to persist the column information in a cookie..	
					// tags: private
					// set filtered structure in case hidden parameters are present
					//LS - 10/6/12 - 54009: Need persistence support in gridx tables
					if( this.persistOptions && this._grid.persist ){
						// if hidden columns are configured using config columns dialog use the data from cookie
						var content = this._grid.persist.get(this._grid.id);
						if(content && content["configColumns"]){
							var visibleColsConfig = content["configColumns"];
							var col, cols =[], grid = this._grid;
							var structure = this._origLayout;
							if(Lang.isArray(visibleColsConfig.visibleCols)){
								Array.forEach(visibleColsConfig.visibleCols, function(col){
									Array.some(structure, function(c){
										if(c.field == col.field){
											cols.push(c);
											return true;
										}
										
									});
								});
								this._grid.setColumns(cols);
							}
							this._grid.body.refresh();
						}else{
							//LS - 10/11/2012 - 54009 - Persisted width and sort props are overwritten
							// in case persistOptions is not used and user is using gridx Persist directly... need to set back the persisted col info 
							// as the original structure while constructing the grid
							if(this._grid.persist){
								this._origLayout =  this._grid._columns;
							}
							this._filterHiddenColumns(this._origLayout);
						}
						
					}else{
						if(this._grid.persist){
							this._origLayout =  this._grid._columns;
						}
						this._filterHiddenColumns(this._origLayout);
					}
				},
				
				_mapgridParams : function () {
					// summary:   
					//		Maps ted plugins params with EnhancedGrid plugin params which may change frequently.
					// tags: private
					
					/* Parameters that could be defined for gridParams_.plugins
						filter:true, for grid filter
						dnd:true, for drag and drop - cell,row and column
						nestedSorting:true, for nested sorting feature
						indirectSelection:true, for checkboxes column
						menus:{headerMenu:"headerMenu", rowMenu:"rowMenu",  selectedRegionMenu:"selectedRegionMenu"}, for cell,row,header and selected region menu
						printer:true, for print feature
						exporter:true, for export data feature
						sortfields:[{ attribute:field name,ascending/descending:boolean...}]
						For pagination, by default pagination will be enabled, if it needs to be disabled please set "pagination:false" on this widget
						
						selector: true by default, set false to disable selector plugin.. else default selector config will be applied
						enableResize : true, to enable the table resize when the window resizes
						enableResize : false, to disable table resize on window resize event.
						To resize the table on a particular event, connect the event to table's resize() function
					*/
					
					if (this.gridParams_.plugins && this.gridParams_.plugins.pagination) {
						/*#37646: LS : Table: Rows per page property is not working
						 * there is a new attribute in Pagination plugin "defaultPagesize" which takes precedence over rowsPerPage
						 * value if Pagination is enabled
						 * mapping rowsPerPage value to defaultPagesize attribute only if the rowsPerPage setting is present in the PagesSizes array
						 * if not present first entry of the PageSizes array will be the default rowsPerPage..
						 * if user specifies for example: pagination:{defaultPageSize:10} this will take precedence over the rowsPerPage value*/
						
						var pagination = null;
						var defaultPageSize = (this.gridParams_.plugins.pagination.defaultPageSize > 0)?
								this.gridParams_.plugins.pagination.defaultPageSize : this.pageSizeArr[0];
						if(this.gridParams_.rowsPerPage > 0 && !this.gridParams_.plugins.pagination.defaultPageSize){
							for(var i=0; i < this.pageSizeArr.length; i++){
								if(this.pageSizeArr[i] == this.gridParams_.rowsPerPage){
									defaultPageSize = this.gridParams_.rowsPerPage;
									break;
								}
							}
						}
						pagination = {
							pageSizes : this.pageSizeArr, // Array, custom the items per page button
							// itemTitle: "entrys", 	// String, custom the item title of description
							description : this.footerDescription, // boolean, custom whether or not the description will be displayed
							sizeSwitch : true, // boolean, custom whether or not the page size switch will be displayed
							pageStepper : true, // boolean, custom whether or not the page step will be displayed
							gotoButton : true, 
							maxPageStep : 10, // Integer, custom how many page step will be displayed
							position : "bottom",// String, custom the position of the paginator bar
							defaultPageSize : defaultPageSize
							// there're three options: top, bottom, both
						};
						this.gridParams_.plugins.pagination = pagination;
					} else {
						if(this.footerVisible && this.gridParams_.plugins){ // checking if the footer need to be displayed
							pagination = {
								//changes to use SummaryBar instead of Pagination with just description
								// need to get support for description template for summary bar (to do)	
								descTemplate : "${1} ${0}",// A template of the current position description.
								onlyDesc : true // new attribute to identify pagination should not be loaded and just the pagination bar with desc should be displayed
							};
							this.gridParams_.plugins.pagination = pagination;
						}
						
					}
					
					if (this.gridParams_.plugins && this.gridParams_.plugins.indirectSelection ) {
						//always display the "select all" check box by default
						if (this.gridParams_.plugins.indirectSelection instanceof Object && this.gridParams_.plugins.indirectSelection.headerSelector == undefined){ 
							dojo.mixin(this.gridParams_.plugins.indirectSelection, {
								"headerSelector" : true
							});
						}else{
							this.gridParams_.plugins.indirectSelection = {"headerSelector" : true};
						}
						if(this.gridParams_.selectionMode && this.gridParams_.selectionMode == "single"){
							dojo.mixin(this.gridParams_.plugins.indirectSelection, {
								"multiple" : false
							});
						}
					}
					
					if (this.gridParams_.plugins && this.gridParams_.plugins.filter !== undefined) {
						//always display the closeFilterBarButton
						this.gridParams_.plugins.filter.closeFilterbarButton = true;
						//hide the filter bar by default
						if (this.gridParams_.preFilter ) {
							this.filterOptions.hideFilterBar = false;
						} else {
							this.filterOptions.hideFilterBar = true;
						}				
						if (this.gridParams_.plugins.filter.advancedFilter !== undefined) {
							this.filterOptions.advancedFilter = this.gridParams_.plugins.filter.advancedFilter;
							delete this.gridParams_.plugins.filter.advancedFilter;
						}
						if (this.gridParams_.plugins.filter.quickFilter !== undefined) {
							this.filterOptions.quickFilter = this.gridParams_.plugins.filter.quickFilter;
							delete this.gridParams_.plugins.filter.quickFilter;
						}
						if (this.gridParams_.plugins.filter.showQuickFilterButton !== undefined) {
							this.filterOptions.quickFilterButton = this.gridParams_.plugins.filter.showQuickFilterButton;
							delete this.gridParams_.plugins.filter.showQuickFilterButton;
						}	
						if (this.gridParams_.plugins.filter.searchDelay !== undefined) {
							this.filterOptions.searchDelay = this.gridParams_.plugins.filter.searchDelay;
						}	
						if (this.gridParams_.plugins.filter.retainSelection !== undefined) {
							this.filterOptions.retainSelection = this.gridParams_.plugins.filter.retainSelection;
						}					
								
					}
					//set original column structure to keep a reference of overall structure including hidden columns
					if(this.gridParams_.structure && !this._origLayout){
						this._origLayout = this.gridParams_.structure;
								
					}
										
				}, 
				
				_mapToModules : function() {
					// summary:   
					//		Maps plugins params to modules of gridx to be backward compatible
					// tags: private
					var userModules = (this.gridParams_.modules) || [];
					if(this.gridParams_.plugins){
						this.gridParams_.modules = userModules.concat(_MapToModules.convertToModules(this.gridParams_.plugins));
					}else{
						this.gridParams_.modules = (_MapToModules._getCoreModules()).concat(userModules);
					}

					if(this.persistOptions){
						//load persist module...conditionally
						this.gridParams_.modules.push(Persist);
					}
				},
				
				resize : function (/*event*/event) {
					// summary:  
					//		Resizes the table
					// description:
					//		Resizes the table with given width and height. Also takes care of window resize events.
					//		Connect to this function to resize table on change/resize of layout/container	 
					// tags:  public
					
					var resizeEvent = false;
					if(event && event.currentTarget){
						resizeEvent = true;
					}
					
					var size = Window.getBox();
					if(this._previousSize && this._previousSize.h == size.h && this._previousSize.w == size.w && resizeEvent == true){
						return;				
					}else{
						this._previousSize =  size;
					}			
					
					//if we have set up everything except the DOM, we cannot resize
					//if (!this.domNode.parentNode || this.domNode.parentNode.nodeType != 1) {
					if (!this._grid || !this.domNode || !this.domNode.parentNode || this.domNode.parentNode.nodeType != 1) {
						return;
					}
					/*supporting autoWidth feature of grid*/
					if (!this._grid.autoWidth) {
						DomStyle.set(this.domNode, "width", this.width);				
					} else {
						DomStyle.set(this.domNode, "width", DomGeometry.getMarginBox(this._grid.domNode).w + 'px');
					}
					
					var heightPx = this.height;
					if (!(this.domNode.parentNode.style && this.domNode.parentNode.style.height)) {
						var sizeType = this.height.substring(this.height.length - 1, this.height.length);
						//if the height is given in % then translate it to pixel
						if (sizeType == "%") {
							//convert to px
							heightPx = size.h * ((parseInt(this.height.slice(0, -1))) / 100);
							heightPx = heightPx + "px";
						}
					}
					
					DomStyle.set(this.domNode, "height", heightPx);
					setTimeout(Lang.hitch(this, function(){this._delayResize(resizeEvent);}), 100);
				}, 
				
				_delayResize: function (resizeEvent) {
					// summary:  
					//		invoked with delay for grid resize
					// tags:
					//		 private
								
					/*using marginBox to include margins/borders of the domNode*/					
					// 53485:  KDM 10/12/12:   Sizing issue in IE8(nodeType null error)
					if(this.domNode){
						var tableSize = DomGeometry.getMarginBox(this.domNode);
						if(tableSize){
							if (this._toolbar) {
								/*calculating toolbarsize including margins and borders as they contibute to some height*/
								var toolbarSize = DomGeometry.getMarginBox(this._toolbar.domNode);
								//removing grid padding height which will be later added when pagination bar is placed and grid view node is resized.
								tableSize.h = tableSize.h - toolbarSize.h ;				
							}
							/*Resize issue with tables in multiple tabs, 
								Table should not attempt to resize when hidden */
							if((tableSize.h <= 0 || tableSize.w <= 0 )){
								return;
							}

							if(resizeEvent){
								this._grid.resize({
									h : tableSize.h,
									w : tableSize.w
								});
							}else{
								//This check has been added for rendering issue of table inside wizard.In IE7 resize is happening only on passing both height and width. 
								// 53485: KDM 10/12/12:  Vertical scrollbar error in IE8 when no content (Dojo defect, but try/catch put in place for 1.0.2)
								if(dojo.isIE == 7) {
									try {
										this._grid.resize({
											h : tableSize.h,
											w : tableSize.w
										});
									}catch(ex){
										// Catch Dojo bug with VScroller in IE7 until Dojo can be fixed									
									}
								} else { 
									this._grid.resize({
										h : tableSize.h,
										w : tableSize.w //189737: also resize the width of the gridx if it's not resize event
									});
								}
							}
							var heightPx = this.height;
							if(this.enableCollapsibleToolbar) {
								if(this.slidingPane!=null && !this.slidingPane.open) {
									tableSize.h = parseInt(heightPx) - 8;
									this._grid.resize({
										h : tableSize.h
									});
								}
							}
						}	
					}
					this._resizing = false;  		
				},
				
				getRowCount : function () {
					// summary:   
					//		Returns the total number of rows present in the grid.
					// tags:  public	
					// returns: total number of rows.
					
					return this._grid.rowCount();
				}, 
				
				getSelectedRowCount : function () {
					// summary:   
					//		Returns the number of selected rows in the grid.
					// tags: public	
					// returns: total number of selected rows.
					
					return this._grid.select.row.getSelected().length;
				}, 
				
				getSelectedRows : function () {
					// summary:   
					// 		Returns the selected rows from the grid.
					// tags: public
					// returns: selected rows.
					
					return this._grid.select.row.getSelected();
				}, 
				
				getFilter : function () {
					// summary:
					//		Returns filter data in json format if fiterBar is enabled
					// tags: private
					// returns: filter data object
					return this._getFilterData();
				}, 
				
				_getFilterFunc : function (){
					// summary:
					//		Returns current Checker function from Filter module. Please refer gridx Filter module
					// tags: private
					// returns:Function|null|undefined (or anything that is invalid)
					if(this._grid.filter){
						return this._grid.filter.getFilter();
					}
					return null;
				},
				
				_getFilterData : function (){
					// summary:
					//		Returns filter data in json format if fiterBar is enabled
					// tags: private
					// returns: filter data object
					if(this._grid.filterBar){
						return this._grid.filterBar.filterData;
					}
					return null;
				},
				
				getFilterRelation : function () {
					// summary:  
					// 		Returns  the relationship between filter rules specified.
					// tags: public
					// returns:
					//		Filter Rule Relations
					if (this._grid.filterBar) {
						return this._grid.filterBar.filterData.type;
					}
				}, 
				
				_getColumnName : function (index) {
					// summary:  
					//		Returns the column name for the specified column index
					// tags: private
					// index: number
					// returns:  	
					//		Name of the column
					
					return this._grid.column(index);
				}, 
				
				_getColumnField : function (index) {
					// summary:  
					//		Returns the column field for the specified column index
					// tags: private
					// index: number
					// returns: 
					//		Field of the column
					return (this._grid.column(index)).field();
				}, 
				
				_isInt : function (value) {
					// summary:   Returns whether the passed value is int or not
					// tags: private
					// returns: boolean
					if ((parseFloat(value) == parseInt(value)) && !isNaN(value)) {
						return true;
					} else {
						return false;
					}
				}, 
				
				_getColumnIndex : function (colField) {
					// summary: Returns the column index for the column field.
					// tags: private
					// returns: Index of the column
					var cells = this._grid.structure;
					for (var i = cells.length - 1; i >= 0; --i) {
						var cell = cells[i];
						if (cell.field == colField) {
							return i;
						}
					}
					return null;
				}, 
				
				_filterHiddenColumns: function (structure){
					// summary: Returns the layout after filtering hidden column definition
					// tags: private
					// returns: layout object
					var filteredColumns = [];
					if(Lang.isArray(structure)){
						for(var i in structure) {
							var column = structure[i];
							if(!(column.hidden && column.hidden == true) ){
								filteredColumns.push(column);
							}
						}
					}
					if(filteredColumns.length >0 )this._currentLayout = filteredColumns;
					this._grid.setColumns(this._currentLayout);
					
				},
				refresh : function (updateBody) {
					// summary:	
					//		Calls in Grid refresh and handles filter applied
					// tags: public
					
					if (this._grid) {
						//Satanik: 5Apr2011: added for CURI store notification of refresh in progress
						if (this._grid.model.store.CURIRefresh) {
							this._grid.store.CURIRefresh();
							this._grid.model.clearCache();  // 49426
                            this._grid.body.refresh();   // 49426
						}
						
						//var scrollpos = this.grid.scrollTop;
						//filter should be applied only if previously applied
						/*if(this.grid.getFilter){
							//console.log("Grid has filter applied");
							//20610: Refresh on Table prompt for clear filters dialog when there are more than 2 filters set
							//Bobby:25May11, using new Grid API to suppress the Clear Filter warning on refresh 
							//LS - #24957 - 22/07/2011 - Problems with ruleCountToConfirmClearFilter - retaining the value set by user if any
							var preVal = this.grid.pluginMgr.getPlugin('filter').ruleCountToConfirmClearFilter;
							this.grid.pluginMgr.getPlugin('filter').ruleCountToConfirmClearFilter = Infinity; //Now the confirm dialog will never show
							var filterArray = this.getFilter();
							var ruleRelation = this.getFilterRelation();
							this.grid.setFilter(filterArray, ruleRelation);
							//LS - #24957 - 22/07/2011 - Problems with ruleCountToConfirmClearFilter - retaining the value set by user if any
							this.grid.pluginMgr.getPlugin('filter').ruleCountToConfirmClearFilter = preVal; //Now the confirm dialog will show
						}else {//as filter calls the grid refresh so adding the _refresh if filter is not there
							//console.log("Grid has NO filter applied");
							this.grid._refresh(true);
						}*/
						else {   // 49426
							//this.grid.setScrollTop(scrollpos);
							this._grid.model.clearCache();
							if(updateBody){
								this._grid.body.refresh();
							}
						}   // 49426
						
					}
				}, 
				
				removeSelectedRows : function () {
					// summary:   
					//		Removes the selected rows from the grid
					// tags: public
					
					this._grid.select.row.clear();
				}, 
				
				deleteAllRows : function () {
					// summary:   
					//		Deletes all rows from the grid.
					// tags: public
					
					var rowCount = this._grid.rowCount();
					for (var i = 0; i < rowCount; i++) {
						this.deleteRow(i);
					}
				}, 
				
				deleteRow : function (rowIndex) {
					// summary:  
					//		Delete a row with the  rowindex  specified.
					// rowIndex: number
					// tags: public
					
					var jsonStore = this._grid.store;
					var gridRef = this._grid;
					jsonStore.fetch({
						query : {},
						onComplete : function (items, request) {
							for (i = 0; i < items.length; i++) {
								var item = items[i];
								
								if (item) {
									jsonStore.deleteItem(item);
								}
							}
							//gridRef.body.refresh();
						}
					});
				}, 
				
				addToActionList : function (item) {
					// summary:  
					//		Add a new item to the toolbar action menu
					// tags: public
					// item: Object
					//		dijit MenuItem object	
					
					if (this._toolbar) {
						this._toolbar._addToActionList(item);
					}
				}, 
				
				addToToolbar : function (item, position, row, noResize) {
					// summary:  
					//		Add an item to the toolbar
					// item:  object   
					//		The item that needs to be added
					// position:  number 
					//		The position of the item in number
					// row:  number 
					//		The row number to which the item needs to be added
					// noResize: boolean 
					//		To control whether a table resize should be done or not
					// tags: public
	
					if (this._toolbar) {
						if (item) {
							this._toolbar._addCustomControl(item, position, row);
						}
						/* #8414: height issue - call resize on this widget whenever toolbar layout is modified, 
						this way new toolbar height can be calculated and grid can be resized accordingly*/
						//20May2011: Bobby
						//performance issue when multiple items are added to toolbar (Sunset/Analytics)
						if(noResize == false){}
						else this.resize();						
					}
				}, 
				
				addItemsToToolbar : function (items, position, row, noResize) {
					// summary:  
					//		Add list of items to toolbar
					// items:  array   
					//		The items that needs to be added
					// position:  number 
					//		The position of the item in number
					// row:  number 
					//		The row number to which the item needs to be added
					// noResize: boolean 
					//		To control whether a table resize should be done or not
					// tags: public
					
					if (this._toolbar) {
						for (var i = 0; i < items.length; i++) {		
							if (items[i]) {
								this._toolbar._addCustomControl(items[i], position + i, row);
							}
						}
						/* #8414: height issue - call resize on this widget whenever toolbar layout is modified, 
						this way new toolbar height can be calculated and grid can be resized accordingly*/
						//20May2011: Bobby
						//performance issue when multiple items are added to toolbar (Sunset/Analytics)
						if(noResize == false){}
						else this.resize();				
					}
				}, 		
				
				addCustomMenuToTable : function (position, customMenu) {
					// summary:  
					//		Add custom menu to the table
					// position: number 
					//		The position of the menu in number
					// customMenu: object  
					//		dijit menu
					// tags: public
					
					if (position == "headerMenu" || position == "headercell") {
						var menuArgs = {hookPoint: "headercell"};
						this.bindMenu(customMenu.id,menuArgs );
						
					} else if (position == "rowMenu" || position == "row") {
						var menuArgs = {hookPoint: "row"};
						this.bindMenu(customMenu.id,menuArgs );
						
					} else if (position == "cellMenu" || position == "cell" ) {
						var menuArgs = {hookPoint: "cell"};
						this.bindMenu(customMenu.id,menuArgs );
						
					} else if (position == "selectedRegionMenu") {
						
						var menuArgs = {hookPoint: "row", selected: true};
						this.bindMenu(customMenu.id,menuArgs );
					}
				}, 
				
				removeFromActionList : function (menuItemId) {
					// summary:    
					//		Removes the given menu item from the action list
					// menuItemId: string
					//		id of the menu item of type dijit MenuItem to be removed form action list
					//tags: public
					
					if (this._toolbar) {
						this._toolbar._removeFromActionList(menuItemId);
					}
				}, 
				
				enableMenuItem : function (menuItemId) {
					// summary:	
					//		Enables the given menu item
					// menuItemId: string
					//		id of the menu item of type dijit MenuItem to be enabled
					//tags: public
					
					if (this._toolbar) {
						this._toolbar._enableMenuItem(menuItemId);
					}
				}, 
				
				disableMenuItem : function (menuItemId) {
					// summary:	
					//		Disables the given menu item
					// menuItemId: string
					//		id of the menu item of type dijit MenuItem to be disabled
					//tags: public
					
					if (this._toolbar) {
						this._toolbar._disableMenuItem(menuItemId);
					}
				}, 
				
				removeItemFromToolbar : function (itemId) {
					// summary:	
					//		Removes an item  with the given item id from the toolbar
					//itemId: number  
					//	id of the item
					//tags: public
					
					if (this._toolbar) {
						this._toolbar._removeItem(itemId);
					}
				}, 
			
				addRow : function (item) {
					// summary:  
					//		Adds a row to the table and scroll to it
					// item: object
					//		row item/object to be added to the end of the table
					// tags:   public
					
					if (this._grid) {
						this.clearSort();
						this.clearFilter();
						var g = this._grid;
						setTimeout(function () {
							var itemAdded = g.store.newItem(item);
							var rowIndex = g.rowCount() - 1;
							if(g.vScroller)
								g.vScroller.scrollToRow(rowIndex);
							}, 10);
					}
				},
				
				clearSort : function (updateBody) {
					// summary:  
					//		Clears sorting for all columns
					// tags: public
					
					if(this._grid.sort && this._grid.sort.clearSort){
						this._grid.sort.clearSort(!updateBody);
					}else if(this._grid.sort && this._grid.sort.clear){
						//nested sort
						this._grid.sort.clear();
						
					}
				}, 
				
				slidingPaneClicked : function () {
					// summary:
					//		callback when collapsible tool bar sliding pane is clicked
					// tags: callback
					
					this.resize();
				},
				
				//Satanik:5May2011: destroy method is changed for mem leak issue 
				destroy : function () {
					// summary:  
					//		Destroys the grid.
					// tags: public
					try{
						delete this.resources_;
						//this is for destroying child templated widgets 
						//this should happen by itself but need to figure out why dijit._Templated is not taking care of it...
						var widgets = dijit.findWidgets(this.domNode);
						dojo.forEach(widgets, function(w) {
							if(w){
								if(w.destroyRecursive){
									w.destroyRecursive();
								} else if(w.destroy){
									w.destroy();
								}
							}
						});
						// this is for widgets added as child to this widget and their children..
						//not present in template
						var widgetArr = this.getChildren();
						dojo.forEach(widgetArr, function (child) {
							if(child){
								if(child.destroyRecursive){
									child.destroyRecursive();
								} else if(child.destroy){
									child.destroy();
								}
							}
						});
						this.inherited(arguments);
					}catch(ex){
						console.warn("Widget destroy failed: "+this.id+", ex="+ex.name+":"+ex.message);
					}
				},
				setFilter : function (filterObjArray, hideFilterBar) {
					// summary:  
					//		Sets the filter for the grid.
					// filterObjArray: Array
					//		 an array of filter objects
					// hideFilterBar: boolean 
					//		to indicate Fitler bar should be hidden
					// tags: public
								
					if (this._grid.filter) {
						var toFilterList = new ArrayList();
						if (filterObjArray) {
							for (var i = 0; i < filterObjArray.length; i++) {
								var filterObj = filterObjArray[i];
								if (filterObj.value) {
									if ((filterObj.column) && (this._isInt(filterObj.column) == false)) {
										filterObj.column = this._getColumnIndex(filterObj.column);
									}	
									var objToSet = {};
									if(filterObj.column && filterObj.column != "anycolumn"){
										objToSet = {
											"type" : filterObj.type ? filterObj.type : "string", 
											"colId" : filterObj.column ? filterObj.column : "anycolumn", 
											"value" : filterObj.value, 
											"condition" : filterObj.condition ? filterObj.condition : "contain"
										};
									}else{
										//anyColumn do not set colId
										objToSet = {
											"type" : filterObj.type ? filterObj.type : "string", 
											"value" : filterObj.value, 
											"condition" : filterObj.condition ? filterObj.condition : "contain"
										};
									}
									
									toFilterList.add(objToSet);
								}
							}
							var toFilterArray = toFilterList.toArray();
							var ruleRelation = filterObjArray.ruleRelation;
							var filter = {};
							if(ruleRelation == "logicAll"){
								filter = {"type":"all","conditions":toFilterArray };
							}else {
								filter = {"type":"any","conditions":toFilterArray };
							} 
							this._grid.filterBar.applyFilter(filter);
						} else {
							this.clearFilter()
						}
						if(hideFilterBar == true){
							this._grid.filterBar.hide();
						}else this._grid.filterBar.show();
					}
				}, 
				
				_setFilterData: function(filterData, hideFilterBar){
					// summary:  
					//		Sets the filter for the grid. This invokes gridx api to apply filter, refer gridx FilterBar module for data format
					// filterData: Object
					//		 Filter data 
					// hideFilterBar: boolean 
					//		to indicate Fitler bar should be hidden
					// tags: public
					if (this._grid.filter && this._grid.filterBar) {
						this._grid.filterBar.applyFilter(filterData);	
					}
					if(hideFilterBar == true){
						this._grid.filterBar.hide();
					}else this._grid.filterBar.show();
				},
				
				clearFilter : function (hideFilterBar) {
					// summary:   
					//		Clears the filter applied 	
					// hideFilterBar: boolean
					//		to hide filterbar when filter is cleared
					// tags: public
					
					if (this._toolbar && this._toolbar._quickFilter) {
						//console.warn('clearing quick filter....');
						this._toolbar._quickFilter._clearQuickFilter();
					} 
					if (this._grid.filter && this._grid.filter.getFilter()) {
						//console.warn('clearing grid filter....');
						this._grid.filter.setFilter(null);				
					}
					if(hideFilterBar == true){
						this._grid.filterBar.hide();
					}
				},
					
				_exportGrid : function (writer, exporterArgs) {
					var deferred = null;
					if(this._grid.exporter){
						if(writer == "csv"){
							deferred = this._grid.exporter.toCSV(exporterArgs);
						}else{
							deferred = this._grid.exporter.toTable(exporterArgs);
						}
					}
					return deferred;
				},
				_printGrid : function (printerArgs) {
					var deferred = null;
					if(this._grid.printer){
						deferred = this._grid.printer.print(printerArgs);
						
					}
					return deferred;
				},
				_printPreviewGrid : function (printerArgs) {
					var deferred = null;
					if(this._grid.printer){
						deferred = this._grid.printer.toHTML(printerArgs);
						
					}
					return deferred;
				},
				
				_bindMenu: function(/* dijit.Menu|ID */ menu, /* __MenuArgs? */ args) {
					
					this._grid.menu.bind(menu, args);
				},
				
				_unbindMenu: function(/* dijit.Menu|ID */ menu) {
					this._grid.menu.unbind(menu);
				}
				
				
			});
			
			
		return Table;
});
        
       
        
},
'dojox/main':function(){
define(["dojo/_base/kernel"], function(dojo) {
	// module:
	//		dojox/main
	// summary:
	//		The dojox package main module; dojox package is somewhat unusual in that the main module currently just provides an empty object.

	return dojo.dojox;
});
},
'ibm/tivoli/navmodel/WidgetDef':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/AbstractWidgetDef"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.WidgetDef"); 
 
// includes
dojo.require("ibm.tivoli.navmodel.AbstractWidgetDef");


////////////////////////////////////
// WidgetDef
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.WidgetDef", ibm.tivoli.navmodel.AbstractWidgetDef, {
	// private members
	_itemStyles: null,
	_itemStatuses: null,
	_linkStyles: null,
	_linkStatuses: null,
	_relationshipStyles: null,
	_relationshipStatuses: null,
	
	// default styles and status
	_defaultItemStyle: null,
	_defaultItemStatus: null,
	_defaultLinkStyle: null,
	_defaultLinkStatus: null,
	_defaultRelationshipStyle: null,
	_defaultRelationshipStatus: null,
	
	/*=====
	constructor_kwArgs: function(baseUrl, provider, datasource, widget, definition, type, version) {
		// baseUrl: String
		//		base url for identifying rest service
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// widget: String
		//		widget type in form <type>-<version>
		// definition: String
		//		widget definition id in form <id>-<version>
		// type: String
		//		definition type:  "user", "shared", or "base"
		// version: Float
		//		version used to communicate with rest service
	},
	=====*/ 
	
	constructor: function(/*constructor_kwArgs*/ kwArgs) {		
		// summary:
		//		Constructs WidgetDef object.
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		this._itemStyles = {};
		this._itemStatuses = {};
		this._linkStyles = {};
		this._linkStatuses = {};
		this._relationshipStyles = {};
		this._relationshipStatuses = {};
		
		this._defaultItemStyle = {
			"scalarImage": "[widget]/resources/common_assets/common_resource_icons/re64_unknownserver1.swf",
           	"scalarImageWidth": 61.011,
           	"scalarImageHeight": 53.596,
           	"smallImage": "[widget]/resources/common_assets/common_resource_icons/re64_unknownserver1_8.png",
           	"smallImageHeight": 16,
         	"smallImageWidth": 16,
         	"largeImage": "[widget]/resources/common_assets/common_resource_icons/re64_unknownserver1_8.png",
           	"largeImageHeight": 32,
         	"largeImageWidth": 32          	
		};
			
		this._defaultItemStatus = {
			"backgroundScalarImage": "[widget]\/resources\/common_assets\/status_icons\/BK_SWF\/unknownBk141x117gradient.swf",
            "backgroundScalarImageHeight": 117,
            "backgroundScalarImageWidth": 141,
            "backgroundScalarImageVisible": true,            
            "scalarImageHeight": 25,
            "scalarImageWidth": 25,
            "scalarImageVisible": true,
            "imageHeight": 16,
            "imageWidth": 16,
            "imageVisible": true
		};
		
		this._defaultLinkStyle = {
			"lineStyle": "solid",
			"lineWidth": 5, //For best results with link selection and seeing link flyovers on roll over, this is the minimum recommended width for links
			"lineColor": "#000000",
			"labelVisible": false,
			"labelAutoRotate": false,
			"labelParallelAlignment": "middle",
			"labelOrthogonalAlignment": "left",
			"labelDistance": 5,
			"labelPositionType": "relative",
			"labelPosition": 0.5
		};
		
		this._defaultLinkStatus = {            
            "scalarImageHeight": 25,
            "scalarImageWidth": 25,
            "scalarImageVisible": true,
            "imageHeight": 16,
            "imageWidth": 16,
            "imageVisible": true,
            "color": "#000000"
		};
		
		this._defaultRelationshipStyle = this._defaultLinkStyle;
		
		this._defaultRelationshipStatus = this._defaultLinkStatus;
	},

	/////////////////////////////
	// public methods
	/////////////////////////////

	destroy: function() {		
		// summary:
		//		Do any necessary clean up
		if (!this.isDestroyed()) {
			// call base class
			this.inherited(arguments);
		}
	},
	
	
	//////////////////
	// pluggableview
	//////////////////
	getPluggableView: function() {
		if (!this._widgetDef.pluggableview) {
			this._widgetDef.pluggableview = {};
		}
		
		return this._widgetDef.pluggableview;
	},
	
	//////////
	// views 
	//////////
	getViews: function(){
		var pluggableview = this.getPluggableView();
		return pluggableview.views;
	},
	
	////////////
	// toolbar 
	////////////
	getToolbar: function() {
		var pluggableview = this.getPluggableView();		
		var returnValue = pluggableview.toolbar;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getToolbar) {
				returnValue = parent.getToolbar();
			}
		}		
		return returnValue;	
	},
	
	getToolbarRows: function(){
		var toolbar = this.getToolbar();
		return (toolbar ? toolbar.rows : []);
	},
	
	isToolbarHidden: function() {
		var toolbar = this.getToolbar();
		var returnValue = (toolbar ? toolbar.hidden : undefined);
		if(returnValue!==undefined){
			return returnValue;
		} else {
			return false;
		}	
	},
	
	//////////////////
	// selectionMode 
	/////////////////
	getSelectionMode: function(){
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.selectionMode;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getSelectionMode){
			returnValue = this._parentWidgetDef.getSelectionMode();
		}		
		return returnValue;			
	},
	
	///////////////
	// labelStyle 
	//////////////
	getLabelStyle: function(){
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.labelStyle;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getLabelStyle){
			returnValue = this._parentWidgetDef.getLabelStyle();
		}		
		return returnValue;
	},
	
	///////////////
	// itemStyles 
	//////////////
	getItemStyles: function() {
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.itemStyles;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getItemStyles){
			returnValue = this._parentWidgetDef.getItemStyles();
		}		
		return returnValue;			
	},
	
	getItemStyle: function(type) {
		var style = this._itemStyles[type];
		
		if (!style) {
			var styles = this.getItemStyles();
			if (styles) {
				var styleLength = styles.length;
				for (var styleIndex = 0;  styleIndex < styleLength;  ++styleIndex) {
					if (styles[styleIndex].type === type) {
						style = styles[styleIndex];
						this._itemStyles[type] = style;
						break;
					}
				}
			}
		}
		
		return style;		
	},
	
	////////////////
	// itemStatuses 
	////////////////
	getItemStatuses: function() {
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.itemStatuses;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getItemStatuses){
			returnValue = this._parentWidgetDef.getItemStatuses();
		}		
		return returnValue;	
	},
	
	getItemStatus: function(id, value) {
		var key = id + "_" + value;
		var status = this._itemStatuses[key];
		
		if (!status) {
			var statuses = this.getItemStatuses();
			if (statuses) {
				var statusLength = statuses.length;
				for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
					if (statuses[statusIndex].id === id &&  statuses[statusIndex].value === value) {
						status = statuses[statusIndex];
						this._itemStatuses[key] = status;
						break;
					}
				}
			}
		}
		
		return status;
	},
	
	///////////////
	// linkStyles 
	///////////////
	getLinkStyles: function() {
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.linkStyles;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getLinkStyles){
			returnValue = this._parentWidgetDef.getLinkStyles();
		}		
		return returnValue;			
	},
	
	getLinkStyle: function(type) {
		var style = this._linkStyles[type];
		
		if (!style) {
			var styles = this.getLinkStyles();
			if (styles) {
				var styleLength = styles.length;
				for (var styleIndex = 0;  styleIndex < styleLength;  ++styleIndex) {
					if (styles[styleIndex].type === type) {
						style = styles[styleIndex];
						this._linkStyles[type] = style;
						break;
					}
				}
			}
		}
		
		return style;		
	},
	
	/////////////////
	// linkStatuses 
	////////////////
	getLinkStatuses: function() {
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.linkStatuses;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getLinkStatuses){
			returnValue = this._parentWidgetDef.getLinkStatuses();
		}		
		return returnValue;			
	},
	
	getLinkStatus: function(id, value) {
		var key = id + "_" + value;
		var status = this._linkStatuses[key];
		
		if (!status) {
			var statuses = this.getLinkStatuses();
			if (statuses) {
				var statusLength = statuses.length;
				for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
					if (statuses[statusIndex].id === id && statuses[statusIndex].value === value) {
						status = statuses[statusIndex];
						this._linkStatuses[key] = status;
						break;
					}
				}
			}
		}
		
		return status;
	},
		
	///////////////////////
	// relationshipStyles 
	///////////////////////
	getRelationshipStyles: function() {
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.relationshipStyles;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getRelationshipStyles){
			returnValue = this._parentWidgetDef.getRelationshipStyles();
		}		
		return returnValue;
	},
	
	getRelationshipStyle: function(type) {
		var style = this._relationshipStyles[type];
		
		if (!style) {
			var styles = this.getRelationshipStyles();
			if (styles) {
				var styleLength = styles.length;
				for (var styleIndex = 0;  styleIndex < styleLength;  ++styleIndex) {
					if (styles[styleIndex].type === type) {
						style = styles[styleIndex];
						this._relationshipStyles[type] = style;
						break;
					}
				}
			}
		}
		
		return style;		
	},
	
	/////////////////////////
	// relationshipStatuses 
	/////////////////////////
	getRelationshipStatuses: function() {
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.relationshipStatuses;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getRelationshipStatuses){
			returnValue = this._parentWidgetDef.getRelationshipStatuses();
		}		
		return returnValue;
	},
	
	getRelationshipStatus: function(id, value) {
		var key = id + "_" + value;
		var status = this._relationshipStatuses[key];
		
		if (!status) {
			var statuses = this.getRelationshipStatuses();
			if (statuses) {
				var statusLength = statuses.length;
				for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
					if (statuses[statusIndex].id === id && statuses[statusIndex].value === value) {
						status = statuses[statusIndex];
						this._relationshipStatuses[key] = status;
						break;
					}
				}
			}
		}
		
		return status;
	},

	getDefaultItemStyle: function() {
		return this._defaultItemStyle;
	},
	
	getDefaultLinkStyle: function() {
		return this._defaultLinkStyle;
	},
	
	getDefaultRelationshipStyle: function() {
		return this._defaultRelationshipStyle;
	},
		
	getDefaultItemStatus: function() {
		return this._defaultItemStatus;
	},
	
	getDefaultLinkStatus: function() {
		return this._defaultLinkStatus;
	},
	
	getDefaultRelationshipStatus: function() {
		return this._defaultRelationshipStatus;
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////
	
	_getTypeSection: function() {
		var type = this.getWidgetType();
		if (type.indexOf("-") > 0) {
			var typeInfo = type.split("-");
			type = typeInfo[0];    		        
        }
        
        return this.getSection(type);
	}
});

});

},
'dojox/grid/_Scroller':function(){
define([
	"dijit/registry",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"./util",
	"dojo/_base/html"
], function(dijitRegistry, declare, lang, util, html){

	var indexInParent = function(inNode){
		var i=0, n, p=inNode.parentNode;
		while((n = p.childNodes[i++])){
			if(n == inNode){
				return i - 1;
			}
		}
		return -1;
	};
	
	var cleanNode = function(inNode){
		if(!inNode){
			return;
		}
		dojo.forEach(dijitRegistry.toArray(), function(w){
			if(w.domNode && html.isDescendant(w.domNode, inNode, true)){
				w.destroy();
			}
		});
	};

	var getTagName = function(inNodeOrId){
		var node = html.byId(inNodeOrId);
		return (node && node.tagName ? node.tagName.toLowerCase() : '');
	};
	
	var nodeKids = function(inNode, inTag){
		var result = [];
		var i=0, n;
		while((n = inNode.childNodes[i])){
			i++;
			if(getTagName(n) == inTag){
				result.push(n);
			}
		}
		return result;
	};
	
	var divkids = function(inNode){
		return nodeKids(inNode, 'div');
	};

	return declare("dojox.grid._Scroller", null, {
		constructor: function(inContentNodes){
			this.setContentNodes(inContentNodes);
			this.pageHeights = [];
			this.pageNodes = [];
			this.stack = [];
		},
		// specified
		rowCount: 0, // total number of rows to manage
		defaultRowHeight: 32, // default height of a row
		keepRows: 100, // maximum number of rows that should exist at one time
		contentNode: null, // node to contain pages
		scrollboxNode: null, // node that controls scrolling
		// calculated
		defaultPageHeight: 0, // default height of a page
		keepPages: 10, // maximum number of pages that should exists at one time
		pageCount: 0,
		windowHeight: 0,
		firstVisibleRow: 0,
		lastVisibleRow: 0,
		averageRowHeight: 0, // the average height of a row
		// private
		page: 0,
		pageTop: 0,
		// init
		init: function(inRowCount, inKeepRows, inRowsPerPage){
			switch(arguments.length){
				case 3: this.rowsPerPage = inRowsPerPage;
				case 2: this.keepRows = inKeepRows;
				case 1: this.rowCount = inRowCount;
				default: break;
			}
			this.defaultPageHeight = this.defaultRowHeight * this.rowsPerPage;
			this.pageCount = this._getPageCount(this.rowCount, this.rowsPerPage);
			this.setKeepInfo(this.keepRows);
			this.invalidate();
			if(this.scrollboxNode){
				this.scrollboxNode.scrollTop = 0;
				this.scroll(0);
				this.scrollboxNode.onscroll = lang.hitch(this, 'onscroll');
			}
		},
		_getPageCount: function(rowCount, rowsPerPage){
			return rowCount ? (Math.ceil(rowCount / rowsPerPage) || 1) : 0;
		},
		destroy: function(){
			this.invalidateNodes();
			delete this.contentNodes;
			delete this.contentNode;
			delete this.scrollboxNode;
		},
		setKeepInfo: function(inKeepRows){
			this.keepRows = inKeepRows;
			this.keepPages = !this.keepRows ? this.keepPages : Math.max(Math.ceil(this.keepRows / this.rowsPerPage), 2);
		},
		// nodes
		setContentNodes: function(inNodes){
			this.contentNodes = inNodes;
			this.colCount = (this.contentNodes ? this.contentNodes.length : 0);
			this.pageNodes = [];
			for(var i=0; i<this.colCount; i++){
				this.pageNodes[i] = [];
			}
		},
		getDefaultNodes: function(){
			return this.pageNodes[0] || [];
		},
		// updating
		invalidate: function(){
			this._invalidating = true;
			this.invalidateNodes();
			this.pageHeights = [];
			this.height = (this.pageCount ? (this.pageCount - 1)* this.defaultPageHeight + this.calcLastPageHeight() : 0);
			this.resize();
			this._invalidating = false;
		},
		updateRowCount: function(inRowCount){
			this.invalidateNodes();
			this.rowCount = inRowCount;
			// update page count, adjust document height
			var oldPageCount = this.pageCount;
			if(oldPageCount === 0){
				//We want to have at least 1px in height to keep scroller.  Otherwise with an
				//empty grid you can't scroll to see the header.
				this.height = 1;
			}
			this.pageCount = this._getPageCount(this.rowCount, this.rowsPerPage);
			if(this.pageCount < oldPageCount){
				for(var i=oldPageCount-1; i>=this.pageCount; i--){
					this.height -= this.getPageHeight(i);
					delete this.pageHeights[i];
				}
			}else if(this.pageCount > oldPageCount){
				this.height += this.defaultPageHeight * (this.pageCount - oldPageCount - 1) + this.calcLastPageHeight();
			}
			this.resize();
		},
		// implementation for page manager
		pageExists: function(inPageIndex){
			return Boolean(this.getDefaultPageNode(inPageIndex));
		},
		measurePage: function(inPageIndex){
			if(this.grid.rowHeight){
				var height = this.grid.rowHeight + 1;
				return ((inPageIndex + 1) * this.rowsPerPage > this.rowCount ?
					this.rowCount - inPageIndex * this.rowsPerPage :
					this.rowsPerPage) * height;
					 
			}
			var n = this.getDefaultPageNode(inPageIndex);
			return (n && n.innerHTML) ? n.offsetHeight : undefined;
		},
		positionPage: function(inPageIndex, inPos){
			for(var i=0; i<this.colCount; i++){
				this.pageNodes[i][inPageIndex].style.top = inPos + 'px';
			}
		},
		repositionPages: function(inPageIndex){
			var nodes = this.getDefaultNodes();
			var last = 0;

			for(var i=0; i<this.stack.length; i++){
				last = Math.max(this.stack[i], last);
			}
			//
			var n = nodes[inPageIndex];
			var y = (n ? this.getPageNodePosition(n) + this.getPageHeight(inPageIndex) : 0);
			for(var p=inPageIndex+1; p<=last; p++){
				n = nodes[p];
				if(n){
					if(this.getPageNodePosition(n) == y){
						return;
					}
					this.positionPage(p, y);
				}
				y += this.getPageHeight(p);
			}
		},
		installPage: function(inPageIndex){
			for(var i=0; i<this.colCount; i++){
				this.contentNodes[i].appendChild(this.pageNodes[i][inPageIndex]);
			}
		},
		preparePage: function(inPageIndex, inReuseNode){
			var p = (inReuseNode ? this.popPage() : null);
			for(var i=0; i<this.colCount; i++){
				var nodes = this.pageNodes[i];
				var new_p = (p === null ? this.createPageNode() : this.invalidatePageNode(p, nodes));
				new_p.pageIndex = inPageIndex;
				nodes[inPageIndex] = new_p;
			}
		},
		// rendering implementation
		renderPage: function(inPageIndex){
			var nodes = [];
			var i, j;
			for(i=0; i<this.colCount; i++){
				nodes[i] = this.pageNodes[i][inPageIndex];
			}
			for(i=0, j=inPageIndex*this.rowsPerPage; (i<this.rowsPerPage)&&(j<this.rowCount); i++, j++){
				this.renderRow(j, nodes);
			}
		},
		removePage: function(inPageIndex){
			for(var i=0, j=inPageIndex*this.rowsPerPage; i<this.rowsPerPage; i++, j++){
				this.removeRow(j);
			}
		},
		destroyPage: function(inPageIndex){
			for(var i=0; i<this.colCount; i++){
				var n = this.invalidatePageNode(inPageIndex, this.pageNodes[i]);
				if(n){
					html.destroy(n);
				}
			}
		},
		pacify: function(inShouldPacify){
		},
		// pacification
		pacifying: false,
		pacifyTicks: 200,
		setPacifying: function(inPacifying){
			if(this.pacifying != inPacifying){
				this.pacifying = inPacifying;
				this.pacify(this.pacifying);
			}
		},
		startPacify: function(){
			this.startPacifyTicks = new Date().getTime();
		},
		doPacify: function(){
			var result = (new Date().getTime() - this.startPacifyTicks) > this.pacifyTicks;
			this.setPacifying(true);
			this.startPacify();
			return result;
		},
		endPacify: function(){
			this.setPacifying(false);
		},
		// default sizing implementation
		resize: function(){
			if(this.scrollboxNode){
				this.windowHeight = this.scrollboxNode.clientHeight;
			}
			for(var i=0; i<this.colCount; i++){
				//We want to have 1px in height min to keep scroller.  Otherwise can't scroll
				//and see header in empty grid.
				util.setStyleHeightPx(this.contentNodes[i], Math.max(1,this.height));
			}
			
			// Calculate the average row height and update the defaults (row and page).
			var needPage = (!this._invalidating);
			if(!needPage){
				var ah = this.grid.get("autoHeight");
				if(typeof ah == "number" && ah <= Math.min(this.rowsPerPage, this.rowCount)){
					needPage = true;
				}
			}
			if(needPage){
				this.needPage(this.page, this.pageTop);
			}
			var rowsOnPage = (this.page < this.pageCount - 1) ? this.rowsPerPage : ((this.rowCount % this.rowsPerPage) || this.rowsPerPage);
			var pageHeight = this.getPageHeight(this.page);
			this.averageRowHeight = (pageHeight > 0 && rowsOnPage > 0) ? (pageHeight / rowsOnPage) : 0;
		},
		calcLastPageHeight: function(){
			if(!this.pageCount){
				return 0;
			}
			var lastPage = this.pageCount - 1;
			var lastPageHeight = ((this.rowCount % this.rowsPerPage)||(this.rowsPerPage)) * this.defaultRowHeight;
			this.pageHeights[lastPage] = lastPageHeight;
			return lastPageHeight;
		},
		updateContentHeight: function(inDh){
			this.height += inDh;
			this.resize();
		},
		updatePageHeight: function(inPageIndex, fromBuild, fromAsynRendering){
			if(this.pageExists(inPageIndex)){
				var oh = this.getPageHeight(inPageIndex);
				var h = (this.measurePage(inPageIndex));
				if(h === undefined){
					h = oh;
				}
				this.pageHeights[inPageIndex] = h;
				if(oh != h){
					this.updateContentHeight(h - oh);
					var ah = this.grid.get("autoHeight");
					if((typeof ah == "number" && ah > this.rowCount)||(ah === true && !fromBuild)){
						if(!fromAsynRendering){
							this.grid.sizeChange();
						}else{//fix #11101 by using fromAsynRendering to avoid deadlock
							var ns = this.grid.viewsNode.style;
							ns.height = parseInt(ns.height) + h - oh + 'px';
							this.repositionPages(inPageIndex);
						}
					}else{
						this.repositionPages(inPageIndex);
					}
				}
				return h;
			}
			return 0;
		},
		rowHeightChanged: function(inRowIndex, fromAsynRendering){
			this.updatePageHeight(Math.floor(inRowIndex / this.rowsPerPage), false, fromAsynRendering);
		},
		// scroller core
		invalidateNodes: function(){
			while(this.stack.length){
				this.destroyPage(this.popPage());
			}
		},
		createPageNode: function(){
			var p = document.createElement('div');
			html.attr(p,"role","presentation");
			p.style.position = 'absolute';
			//p.style.width = '100%';
			p.style[this.grid.isLeftToRight() ? "left" : "right"] = '0';
			return p;
		},
		getPageHeight: function(inPageIndex){
			var ph = this.pageHeights[inPageIndex];
			return (ph !== undefined ? ph : this.defaultPageHeight);
		},
		// FIXME: this is not a stack, it's a FIFO list
		pushPage: function(inPageIndex){
			return this.stack.push(inPageIndex);
		},
		popPage: function(){
			return this.stack.shift();
		},
		findPage: function(inTop){
			var i = 0, h = 0;
			for(var ph = 0; i<this.pageCount; i++, h += ph){
				ph = this.getPageHeight(i);
				if(h + ph >= inTop){
					break;
				}
			}
			this.page = i;
			this.pageTop = h;
		},
		buildPage: function(inPageIndex, inReuseNode, inPos){
			this.preparePage(inPageIndex, inReuseNode);
			this.positionPage(inPageIndex, inPos);
			// order of operations is key below
			this.installPage(inPageIndex);
			this.renderPage(inPageIndex);
			// order of operations is key above
			this.pushPage(inPageIndex);
		},
		needPage: function(inPageIndex, inPos){
			var h = this.getPageHeight(inPageIndex), oh = h;
			if(!this.pageExists(inPageIndex)){
				this.buildPage(inPageIndex, (!this.grid._autoHeight/*fix #10543*/ && this.keepPages&&(this.stack.length >= this.keepPages)), inPos);
				h = this.updatePageHeight(inPageIndex, true);
			}else{
				this.positionPage(inPageIndex, inPos);
			}
			return h;
		},
		onscroll: function(){
			this.scroll(this.scrollboxNode.scrollTop);
		},
		scroll: function(inTop){
			this.grid.scrollTop = inTop;
			if(this.colCount){
				this.startPacify();
				this.findPage(inTop);
				var h = this.height;
				var b = this.getScrollBottom(inTop);
				for(var p=this.page, y=this.pageTop; (p<this.pageCount)&&((b<0)||(y<b)); p++){
					y += this.needPage(p, y);
				}
				this.firstVisibleRow = this.getFirstVisibleRow(this.page, this.pageTop, inTop);
				this.lastVisibleRow = this.getLastVisibleRow(p - 1, y, b);
				// indicates some page size has been updated
				if(h != this.height){
					this.repositionPages(p-1);
				}
				this.endPacify();
			}
		},
		getScrollBottom: function(inTop){
			return (this.windowHeight >= 0 ? inTop + this.windowHeight : -1);
		},
		// events
		processNodeEvent: function(e, inNode){
			var t = e.target;
			while(t && (t != inNode) && t.parentNode && (t.parentNode.parentNode != inNode)){
				t = t.parentNode;
			}
			if(!t || !t.parentNode || (t.parentNode.parentNode != inNode)){
				return false;
			}
			var page = t.parentNode;
			e.topRowIndex = page.pageIndex * this.rowsPerPage;
			e.rowIndex = e.topRowIndex + indexInParent(t);
			e.rowTarget = t;
			return true;
		},
		processEvent: function(e){
			return this.processNodeEvent(e, this.contentNode);
		},
		// virtual rendering interface
		renderRow: function(inRowIndex, inPageNode){
		},
		removeRow: function(inRowIndex){
		},
		// page node operations
		getDefaultPageNode: function(inPageIndex){
			return this.getDefaultNodes()[inPageIndex];
		},
		positionPageNode: function(inNode, inPos){
		},
		getPageNodePosition: function(inNode){
			return inNode.offsetTop;
		},
		invalidatePageNode: function(inPageIndex, inNodes){
			var p = inNodes[inPageIndex];
			if(p){
				delete inNodes[inPageIndex];
				this.removePage(inPageIndex, p);
				cleanNode(p);
				p.innerHTML = '';
			}
			return p;
		},
		// scroll control
		getPageRow: function(inPage){
			return inPage * this.rowsPerPage;
		},
		getLastPageRow: function(inPage){
			return Math.min(this.rowCount, this.getPageRow(inPage + 1)) - 1;
		},
		getFirstVisibleRow: function(inPage, inPageTop, inScrollTop){
			if(!this.pageExists(inPage)){
				return 0;
			}
			var row = this.getPageRow(inPage);
			var nodes = this.getDefaultNodes();
			var rows = divkids(nodes[inPage]);
			for(var i=0,l=rows.length; i<l && inPageTop<inScrollTop; i++, row++){
				inPageTop += rows[i].offsetHeight;
			}
			return (row ? row - 1 : row);
		},
		getLastVisibleRow: function(inPage, inBottom, inScrollBottom){
			if(!this.pageExists(inPage)){
				return 0;
			}
			var nodes = this.getDefaultNodes();
			var row = this.getLastPageRow(inPage);
			var rows = divkids(nodes[inPage]);
			for(var i=rows.length-1; i>=0 && inBottom>inScrollBottom; i--, row--){
				inBottom -= rows[i].offsetHeight;
			}
			return row + 1;
		},
		findTopRow: function(inScrollTop){
			var nodes = this.getDefaultNodes();
			var rows = divkids(nodes[this.page]);
			for(var i=0,l=rows.length,t=this.pageTop,h; i<l; i++){
				h = rows[i].offsetHeight;
				t += h;
				if(t >= inScrollTop){
					this.offset = h - (t - inScrollTop);
					return i + this.page * this.rowsPerPage;
				}
			}
			return -1;
		},
		findScrollTop: function(inRow){
			var rowPage = Math.floor(inRow / this.rowsPerPage);
			var t = 0;
			var i, l;
			for(i=0; i<rowPage; i++){
				t += this.getPageHeight(i);
			}
			this.pageTop = t;
			this.page = rowPage;//fix #10543
			this.needPage(rowPage, this.pageTop);

			var nodes = this.getDefaultNodes();
			var rows = divkids(nodes[rowPage]);
			var r = inRow - this.rowsPerPage * rowPage;
			for(i=0,l=rows.length; i<l && i<r; i++){
				t += rows[i].offsetHeight;
			}
			return t;
		},
		dummy: 0
	});
});

},
'dijit/Editor':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/_base/Deferred", // Deferred
	"dojo/i18n", // i18n.getLocalization
	"dojo/dom-attr", // domAttr.set
	"dojo/dom-class", // domClass.add
	"dojo/dom-geometry",
	"dojo/dom-style", // domStyle.set, get
	"dojo/_base/event", // event.stop
	"dojo/keys", // keys.F1 keys.F15 keys.TAB
	"dojo/_base/lang", // lang.getObject lang.hitch
	"dojo/_base/sniff", // has("ie") has("mac") has("webkit")
	"dojo/string", // string.substitute
	"dojo/topic", // topic.publish()
	"dojo/_base/window", // win.withGlobal
	"./_base/focus",	// dijit.getBookmark()
	"./_Container",
	"./Toolbar",
	"./ToolbarSeparator",
	"./layout/_LayoutWidget",
	"./form/ToggleButton",
	"./_editor/_Plugin",
	"./_editor/plugins/EnterKeyHandling",
	"./_editor/html",
	"./_editor/range",
	"./_editor/RichText",
	".",	// dijit._scopeName
	"dojo/i18n!./_editor/nls/commands"
], function(array, declare, Deferred, i18n, domAttr, domClass, domGeometry, domStyle,
			event, keys, lang, has, string, topic, win,
			focusBase, _Container, Toolbar, ToolbarSeparator, _LayoutWidget, ToggleButton,
			_Plugin, EnterKeyHandling, html, rangeapi, RichText, dijit){

	// module:
	//		dijit/Editor
	// summary:
	//		A rich text Editing widget

	var Editor = declare("dijit.Editor", RichText, {
		// summary:
		//		A rich text Editing widget
		//
		// description:
		//		This widget provides basic WYSIWYG editing features, based on the browser's
		//		underlying rich text editing capability, accompanied by a toolbar (`dijit.Toolbar`).
		//		A plugin model is available to extend the editor's capabilities as well as the
		//		the options available in the toolbar.  Content generation may vary across
		//		browsers, and clipboard operations may have different results, to name
		//		a few limitations.  Note: this widget should not be used with the HTML
		//		&lt;TEXTAREA&gt; tag -- see dijit._editor.RichText for details.

		// plugins: [const] Object[]
		//		A list of plugin names (as strings) or instances (as objects)
		//		for this widget.
		//
		//		When declared in markup, it might look like:
		//	|	plugins="['bold',{name:'dijit._editor.plugins.FontChoice', command:'fontName', generic:true}]"
		plugins: null,

		// extraPlugins: [const] Object[]
		//		A list of extra plugin names which will be appended to plugins array
		extraPlugins: null,

		constructor: function(){
			// summary:
			//		Runs on widget initialization to setup arrays etc.
			// tags:
			//		private

			if(!lang.isArray(this.plugins)){
				this.plugins=["undo","redo","|","cut","copy","paste","|","bold","italic","underline","strikethrough","|",
				"insertOrderedList","insertUnorderedList","indent","outdent","|","justifyLeft","justifyRight","justifyCenter","justifyFull",
				EnterKeyHandling /*, "createLink"*/];
			}

			this._plugins=[];
			this._editInterval = this.editActionInterval * 1000;

			//IE will always lose focus when other element gets focus, while for FF and safari,
			//when no iframe is used, focus will be lost whenever another element gets focus.
			//For IE, we can connect to onBeforeDeactivate, which will be called right before
			//the focus is lost, so we can obtain the selected range. For other browsers,
			//no equivalent of onBeforeDeactivate, so we need to do two things to make sure
			//selection is properly saved before focus is lost: 1) when user clicks another
			//element in the page, in which case we listen to mousedown on the entire page and
			//see whether user clicks out of a focus editor, if so, save selection (focus will
			//only lost after onmousedown event is fired, so we can obtain correct caret pos.)
			//2) when user tabs away from the editor, which is handled in onKeyDown below.
			if(has("ie")){
				this.events.push("onBeforeDeactivate");
				this.events.push("onBeforeActivate");
			}
		},

		postMixInProperties: function(){
			// summary:
			//	Extension to make sure a deferred is in place before certain functions
			//	execute, like making sure all the plugins are properly inserted.

			// Set up a deferred so that the value isn't applied to the editor
			// until all the plugins load, needed to avoid timing condition
			// reported in #10537.
			this.setValueDeferred = new Deferred();
			this.inherited(arguments);
		},

		postCreate: function(){
			//for custom undo/redo, if enabled.
			this._steps=this._steps.slice(0);
			this._undoedSteps=this._undoedSteps.slice(0);

			if(lang.isArray(this.extraPlugins)){
				this.plugins=this.plugins.concat(this.extraPlugins);
			}

			this.inherited(arguments);

			this.commands = i18n.getLocalization("dijit._editor", "commands", this.lang);

			if(!this.toolbar){
				// if we haven't been assigned a toolbar, create one
				this.toolbar = new Toolbar({
					dir: this.dir,
					lang: this.lang
				});
				this.header.appendChild(this.toolbar.domNode);
			}

			array.forEach(this.plugins, this.addPlugin, this);

			// Okay, denote the value can now be set.
			this.setValueDeferred.callback(true);

			domClass.add(this.iframe.parentNode, "dijitEditorIFrameContainer");
			domClass.add(this.iframe, "dijitEditorIFrame");
			domAttr.set(this.iframe, "allowTransparency", true);

			if(has("webkit")){
				// Disable selecting the entire editor by inadvertent double-clicks.
				// on buttons, title bar, etc.  Otherwise clicking too fast on
				// a button such as undo/redo selects the entire editor.
				domStyle.set(this.domNode, "KhtmlUserSelect", "none");
			}
			this.toolbar.startup();
			this.onNormalizedDisplayChanged(); //update toolbar button status
		},
		destroy: function(){
			array.forEach(this._plugins, function(p){
				if(p && p.destroy){
					p.destroy();
				}
			});
			this._plugins=[];
			this.toolbar.destroyRecursive();
			delete this.toolbar;
			this.inherited(arguments);
		},
		addPlugin: function(/*String||Object||Function*/plugin, /*Integer?*/index){
			// summary:
			//		takes a plugin name as a string or a plugin instance and
			//		adds it to the toolbar and associates it with this editor
			//		instance. The resulting plugin is added to the Editor's
			//		plugins array. If index is passed, it's placed in the plugins
			//		array at that index. No big magic, but a nice helper for
			//		passing in plugin names via markup.
			//
			// plugin: String, args object, plugin instance, or plugin constructor
			//
			// args:
			//		This object will be passed to the plugin constructor
			//
			// index: Integer
			//		Used when creating an instance from
			//		something already in this.plugins. Ensures that the new
			//		instance is assigned to this.plugins at that index.
			var args=lang.isString(plugin)?{name:plugin}:lang.isFunction(plugin)?{ctor:plugin}:plugin;
			if(!args.setEditor){
				var o={"args":args,"plugin":null,"editor":this};
				if(args.name){
					// search registry for a plugin factory matching args.name, if it's not there then
					// fallback to 1.0 API:
					// ask all loaded plugin modules to fill in o.plugin if they can (ie, if they implement args.name)
					// remove fallback for 2.0.
					if(_Plugin.registry[args.name]){
						o.plugin = _Plugin.registry[args.name](args);
					}else{
						topic.publish(dijit._scopeName + ".Editor.getPlugin", o);	// publish
					}
				}
				if(!o.plugin){
					var pc = args.ctor || lang.getObject(args.name);
					if(pc){
						o.plugin=new pc(args);
					}
				}
				if(!o.plugin){
					console.warn('Cannot find plugin',plugin);
					return;
				}
				plugin=o.plugin;
			}
			if(arguments.length > 1){
				this._plugins[index] = plugin;
			}else{
				this._plugins.push(plugin);
			}
			plugin.setEditor(this);
			if(lang.isFunction(plugin.setToolbar)){
				plugin.setToolbar(this.toolbar);
			}
		},

		//the following 2 functions are required to make the editor play nice under a layout widget, see #4070

		resize: function(size){
			// summary:
			//		Resize the editor to the specified size, see `dijit.layout._LayoutWidget.resize`
			if(size){
				// we've been given a height/width for the entire editor (toolbar + contents), calls layout()
				// to split the allocated size between the toolbar and the contents
				_LayoutWidget.prototype.resize.apply(this, arguments);
			}
			/*
			else{
				// do nothing, the editor is already laid out correctly.   The user has probably specified
				// the height parameter, which was used to set a size on the iframe
			}
			*/
		},
		layout: function(){
			// summary:
			//		Called from `dijit.layout._LayoutWidget.resize`.  This shouldn't be called directly
			// tags:
			//		protected

			// Converts the iframe (or rather the <div> surrounding it) to take all the available space
			// except what's needed for the header (toolbars) and footer (breadcrumbs, etc).
			// A class was added to the iframe container and some themes style it, so we have to
			// calc off the added margins and padding too. See tracker: #10662
			var areaHeight = (this._contentBox.h -
				(this.getHeaderHeight() + this.getFooterHeight() +
				 domGeometry.getPadBorderExtents(this.iframe.parentNode).h +
				 domGeometry.getMarginExtents(this.iframe.parentNode).h));
			this.editingArea.style.height = areaHeight + "px";
			if(this.iframe){
				this.iframe.style.height="100%";
			}
			this._layoutMode = true;
		},

		_onIEMouseDown: function(/*Event*/ e){
			// summary:
			//		IE only to prevent 2 clicks to focus
			// tags:
			//		private
			var outsideClientArea;
			// IE 8's componentFromPoint is broken, which is a shame since it
			// was smaller code, but oh well.  We have to do this brute force
			// to detect if the click was scroller or not.
			var b = this.document.body;
			var clientWidth = b.clientWidth;
			var clientHeight = b.clientHeight;
			var clientLeft = b.clientLeft;
			var offsetWidth = b.offsetWidth;
			var offsetHeight = b.offsetHeight;
			var offsetLeft = b.offsetLeft;

			//Check for vertical scroller click.
			if(/^rtl$/i.test(b.dir || "")){
				if(clientWidth < offsetWidth && e.x > clientWidth && e.x < offsetWidth){
					// Check the click was between width and offset width, if so, scroller
					outsideClientArea = true;
				}
			}else{
				// RTL mode, we have to go by the left offsets.
				if(e.x < clientLeft && e.x > offsetLeft){
					// Check the click was between width and offset width, if so, scroller
					outsideClientArea = true;
				}
			}
			if(!outsideClientArea){
				// Okay, might be horiz scroller, check that.
				if(clientHeight < offsetHeight && e.y > clientHeight && e.y < offsetHeight){
					// Horizontal scroller.
					outsideClientArea = true;
				}
			}
			if(!outsideClientArea){
				delete this._cursorToStart; // Remove the force to cursor to start position.
				delete this._savedSelection; // new mouse position overrides old selection
				if(e.target.tagName == "BODY"){
					setTimeout(lang.hitch(this, "placeCursorAtEnd"), 0);
				}
				this.inherited(arguments);
			}
		},
		onBeforeActivate: function(){
			this._restoreSelection();
		},
		onBeforeDeactivate: function(e){
			// summary:
			//		Called on IE right before focus is lost.   Saves the selected range.
			// tags:
			//		private
			if(this.customUndo){
				this.endEditing(true);
			}
			//in IE, the selection will be lost when other elements get focus,
			//let's save focus before the editor is deactivated
			if(e.target.tagName != "BODY"){
				this._saveSelection();
			}
			//console.log('onBeforeDeactivate',this);
		},

		/* beginning of custom undo/redo support */

		// customUndo: Boolean
		//		Whether we shall use custom undo/redo support instead of the native
		//		browser support. By default, we now use custom undo.  It works better
		//		than native browser support and provides a consistent behavior across
		//		browsers with a minimal performance hit.  We already had the hit on
		//		the slowest browser, IE, anyway.
		customUndo: true,

		// editActionInterval: Integer
		//		When using customUndo, not every keystroke will be saved as a step.
		//		Instead typing (including delete) will be grouped together: after
		//		a user stops typing for editActionInterval seconds, a step will be
		//		saved; if a user resume typing within editActionInterval seconds,
		//		the timeout will be restarted. By default, editActionInterval is 3
		//		seconds.
		editActionInterval: 3,

		beginEditing: function(cmd){
			// summary:
			//		Called to note that the user has started typing alphanumeric characters, if it's not already noted.
			//		Deals with saving undo; see editActionInterval parameter.
			// tags:
			//		private
			if(!this._inEditing){
				this._inEditing=true;
				this._beginEditing(cmd);
			}
			if(this.editActionInterval>0){
				if(this._editTimer){
					clearTimeout(this._editTimer);
				}
				this._editTimer = setTimeout(lang.hitch(this, this.endEditing), this._editInterval);
			}
		},

		// TODO: declaring these in the prototype is meaningless, just create in the constructor/postCreate
		_steps:[],
		_undoedSteps:[],

		execCommand: function(cmd){
			// summary:
			//		Main handler for executing any commands to the editor, like paste, bold, etc.
			//      Called by plugins, but not meant to be called by end users.
			// tags:
			//		protected
			if(this.customUndo && (cmd == 'undo' || cmd == 'redo')){
				return this[cmd]();
			}else{
				if(this.customUndo){
					this.endEditing();
					this._beginEditing();
				}
				var r = this.inherited(arguments);
				if(this.customUndo){
					this._endEditing();
				}
				return r;
			}
		},

		_pasteImpl: function(){
			// summary:
			//		Over-ride of paste command control to make execCommand cleaner
			// tags:
			//		Protected
			return this._clipboardCommand("paste");
		},

		_cutImpl: function(){
			// summary:
			//		Over-ride of cut command control to make execCommand cleaner
			// tags:
			//		Protected
			return this._clipboardCommand("cut");
		},

		_copyImpl: function(){
			// summary:
			//		Over-ride of copy command control to make execCommand cleaner
			// tags:
			//		Protected
			return this._clipboardCommand("copy");
		},

		_clipboardCommand: function(cmd){
			// summary:
			//		Function to handle processing clipboard commands (or at least try to).
			// tags:
			//		Private
			var r;
			try{
				// Try to exec the superclass exec-command and see if it works.
				r = this.document.execCommand(cmd, false, null);
				if(has("webkit") && !r){ //see #4598: webkit does not guarantee clipboard support from js
					throw { code: 1011 }; // throw an object like Mozilla's error
				}
			}catch(e){
				//TODO: when else might we get an exception?  Do we need the Mozilla test below?
				if(e.code == 1011 /* Mozilla: service denied */){
					// Warn user of platform limitation.  Cannot programmatically access clipboard. See ticket #4136
					var sub = string.substitute,
						accel = {cut:'X', copy:'C', paste:'V'};
					alert(sub(this.commands.systemShortcut,
						[this.commands[cmd], sub(this.commands[has("mac") ? 'appleKey' : 'ctrlKey'], [accel[cmd]])]));
				}
				r = false;
			}
			return r;
		},

		queryCommandEnabled: function(cmd){
			// summary:
			//		Returns true if specified editor command is enabled.
			//      Used by the plugins to know when to highlight/not highlight buttons.
			// tags:
			//		protected
			if(this.customUndo && (cmd == 'undo' || cmd == 'redo')){
				return cmd == 'undo' ? (this._steps.length > 1) : (this._undoedSteps.length > 0);
			}else{
				return this.inherited(arguments);
			}
		},
		_moveToBookmark: function(b){
			// summary:
			//		Selects the text specified in bookmark b
			// tags:
			//		private
			var bookmark = b.mark;
			var mark = b.mark;
			var col = b.isCollapsed;
			var r, sNode, eNode, sel;
			if(mark){
				if(has("ie") < 9){
					if(lang.isArray(mark)){
						//IE CONTROL, have to use the native bookmark.
						bookmark = [];
						array.forEach(mark,function(n){
							bookmark.push(rangeapi.getNode(n,this.editNode));
						},this);
						win.withGlobal(this.window,'moveToBookmark',dijit,[{mark: bookmark, isCollapsed: col}]);
					}else{
						if(mark.startContainer && mark.endContainer){
							// Use the pseudo WC3 range API.  This works better for positions
							// than the IE native bookmark code.
							sel = rangeapi.getSelection(this.window);
							if(sel && sel.removeAllRanges){
								sel.removeAllRanges();
								r = rangeapi.create(this.window);
								sNode = rangeapi.getNode(mark.startContainer,this.editNode);
								eNode = rangeapi.getNode(mark.endContainer,this.editNode);
								if(sNode && eNode){
									// Okay, we believe we found the position, so add it into the selection
									// There are cases where it may not be found, particularly in undo/redo, when
									// IE changes the underlying DOM on us (wraps text in a <p> tag or similar.
									// So, in those cases, don't bother restoring selection.
									r.setStart(sNode,mark.startOffset);
									r.setEnd(eNode,mark.endOffset);
									sel.addRange(r);
								}
							}
						}
					}
				}else{//w3c range
					sel = rangeapi.getSelection(this.window);
					if(sel && sel.removeAllRanges){
						sel.removeAllRanges();
						r = rangeapi.create(this.window);
						sNode = rangeapi.getNode(mark.startContainer,this.editNode);
						eNode = rangeapi.getNode(mark.endContainer,this.editNode);
						if(sNode && eNode){
							// Okay, we believe we found the position, so add it into the selection
							// There are cases where it may not be found, particularly in undo/redo, when
							// formatting as been done and so on, so don't restore selection then.
							r.setStart(sNode,mark.startOffset);
							r.setEnd(eNode,mark.endOffset);
							sel.addRange(r);
						}
					}
				}
			}
		},
		_changeToStep: function(from, to){
			// summary:
			//		Reverts editor to "to" setting, from the undo stack.
			// tags:
			//		private
			this.setValue(to.text);
			var b=to.bookmark;
			if(!b){ return; }
			this._moveToBookmark(b);
		},
		undo: function(){
			// summary:
			//		Handler for editor undo (ex: ctrl-z) operation
			// tags:
			//		private
			//console.log('undo');
			var ret = false;
			if(!this._undoRedoActive){
				this._undoRedoActive = true;
				this.endEditing(true);
				var s=this._steps.pop();
				if(s && this._steps.length>0){
					this.focus();
					this._changeToStep(s,this._steps[this._steps.length-1]);
					this._undoedSteps.push(s);
					this.onDisplayChanged();
					delete this._undoRedoActive;
					ret = true;
				}
				delete this._undoRedoActive;
			}
			return ret;
		},
		redo: function(){
			// summary:
			//		Handler for editor redo (ex: ctrl-y) operation
			// tags:
			//		private
			//console.log('redo');
			var ret = false;
			if(!this._undoRedoActive){
				this._undoRedoActive = true;
				this.endEditing(true);
				var s=this._undoedSteps.pop();
				if(s && this._steps.length>0){
					this.focus();
					this._changeToStep(this._steps[this._steps.length-1],s);
					this._steps.push(s);
					this.onDisplayChanged();
					ret = true;
				}
				delete this._undoRedoActive;
			}
			return ret;
		},
		endEditing: function(ignore_caret){
			// summary:
			//		Called to note that the user has stopped typing alphanumeric characters, if it's not already noted.
			//		Deals with saving undo; see editActionInterval parameter.
			// tags:
			//		private
			if(this._editTimer){
				clearTimeout(this._editTimer);
			}
			if(this._inEditing){
				this._endEditing(ignore_caret);
				this._inEditing=false;
			}
		},

		_getBookmark: function(){
			// summary:
			//		Get the currently selected text
			// tags:
			//		protected
			var b=win.withGlobal(this.window,focusBase.getBookmark);
			var tmp=[];
			if(b && b.mark){
				var mark = b.mark;
				if(has("ie") < 9){
					// Try to use the pseudo range API on IE for better accuracy.
					var sel = rangeapi.getSelection(this.window);
					if(!lang.isArray(mark)){
						if(sel){
							var range;
							if(sel.rangeCount){
								range = sel.getRangeAt(0);
							}
							if(range){
								b.mark = range.cloneRange();
							}else{
								b.mark = win.withGlobal(this.window,focusBase.getBookmark);
							}
						}
					}else{
						// Control ranges (img, table, etc), handle differently.
						array.forEach(b.mark,function(n){
							tmp.push(rangeapi.getIndex(n,this.editNode).o);
						},this);
						b.mark = tmp;
					}
				}
				try{
					if(b.mark && b.mark.startContainer){
						tmp=rangeapi.getIndex(b.mark.startContainer,this.editNode).o;
						b.mark={startContainer:tmp,
							startOffset:b.mark.startOffset,
							endContainer:b.mark.endContainer===b.mark.startContainer?tmp:rangeapi.getIndex(b.mark.endContainer,this.editNode).o,
							endOffset:b.mark.endOffset};
					}
				}catch(e){
					b.mark = null;
				}
			}
			return b;
		},
		_beginEditing: function(){
			// summary:
			//		Called when the user starts typing alphanumeric characters.
			//		Deals with saving undo; see editActionInterval parameter.
			// tags:
			//		private
			if(this._steps.length === 0){
				// You want to use the editor content without post filtering
				// to make sure selection restores right for the 'initial' state.
				// and undo is called.  So not using this.value, as it was 'processed'
				// and the line-up for selections may have been altered.
				this._steps.push({'text':html.getChildrenHtml(this.editNode),'bookmark':this._getBookmark()});
			}
		},
		_endEditing: function(){
			// summary:
			//		Called when the user stops typing alphanumeric characters.
			//		Deals with saving undo; see editActionInterval parameter.
			// tags:
			//		private
			// Avoid filtering to make sure selections restore.
			var v = html.getChildrenHtml(this.editNode);

			this._undoedSteps=[];//clear undoed steps
			this._steps.push({text: v, bookmark: this._getBookmark()});
		},
		onKeyDown: function(e){
			// summary:
			//		Handler for onkeydown event.
			// tags:
			//		private

			//We need to save selection if the user TAB away from this editor
			//no need to call _saveSelection for IE, as that will be taken care of in onBeforeDeactivate
			if(!has("ie") && !this.iframe && e.keyCode == keys.TAB && !this.tabIndent){
				this._saveSelection();
			}
			if(!this.customUndo){
				this.inherited(arguments);
				return;
			}
			var k = e.keyCode;
			if(e.ctrlKey && !e.altKey){//undo and redo only if the special right Alt + z/y are not pressed #5892
				if(k == 90 || k == 122){ //z
					event.stop(e);
					this.undo();
					return;
				}else if(k == 89 || k == 121){ //y
					event.stop(e);
					this.redo();
					return;
				}
			}
			this.inherited(arguments);

			switch(k){
					case keys.ENTER:
					case keys.BACKSPACE:
					case keys.DELETE:
						this.beginEditing();
						break;
					case 88: //x
					case 86: //v
						if(e.ctrlKey && !e.altKey && !e.metaKey){
							this.endEditing();//end current typing step if any
							if(e.keyCode == 88){
								this.beginEditing('cut');
								//use timeout to trigger after the cut is complete
								setTimeout(lang.hitch(this, this.endEditing), 1);
							}else{
								this.beginEditing('paste');
								//use timeout to trigger after the paste is complete
								setTimeout(lang.hitch(this, this.endEditing), 1);
							}
							break;
						}
						//pass through
					default:
						if(!e.ctrlKey && !e.altKey && !e.metaKey && (e.keyCode<keys.F1 || e.keyCode>keys.F15)){
							this.beginEditing();
							break;
						}
						//pass through
					case keys.ALT:
						this.endEditing();
						break;
					case keys.UP_ARROW:
					case keys.DOWN_ARROW:
					case keys.LEFT_ARROW:
					case keys.RIGHT_ARROW:
					case keys.HOME:
					case keys.END:
					case keys.PAGE_UP:
					case keys.PAGE_DOWN:
						this.endEditing(true);
						break;
					//maybe ctrl+backspace/delete, so don't endEditing when ctrl is pressed
					case keys.CTRL:
					case keys.SHIFT:
					case keys.TAB:
						break;
				}
		},
		_onBlur: function(){
			// summary:
			//		Called from focus manager when focus has moved away from this editor
			// tags:
			//		protected

			//this._saveSelection();
			this.inherited(arguments);
			this.endEditing(true);
		},
		_saveSelection: function(){
			// summary:
			//		Save the currently selected text in _savedSelection attribute
			// tags:
			//		private
			try{
				this._savedSelection=this._getBookmark();
			}catch(e){ /* Squelch any errors that occur if selection save occurs due to being hidden simultaneously. */}
		},
		_restoreSelection: function(){
			// summary:
			//		Re-select the text specified in _savedSelection attribute;
			//		see _saveSelection().
			// tags:
			//		private
			if(this._savedSelection){
				// Clear off cursor to start, we're deliberately going to a selection.
				delete this._cursorToStart;
				// only restore the selection if the current range is collapsed
				// if not collapsed, then it means the editor does not lose
				// selection and there is no need to restore it
				if(win.withGlobal(this.window,'isCollapsed',dijit)){
					this._moveToBookmark(this._savedSelection);
				}
				delete this._savedSelection;
			}
		},

		onClick: function(){
			// summary:
			//		Handler for when editor is clicked
			// tags:
			//		protected
			this.endEditing(true);
			this.inherited(arguments);
		},

		replaceValue: function(/*String*/ html){
			// summary:
			//		over-ride of replaceValue to support custom undo and stack maintenance.
			// tags:
			//		protected
			if(!this.customUndo){
				this.inherited(arguments);
			}else{
				if(this.isClosed){
					this.setValue(html);
				}else{
					this.beginEditing();
					if(!html){
						html = "&#160;";	// &nbsp;
					}
					this.setValue(html);
					this.endEditing();
				}
			}
		},

		_setDisabledAttr: function(/*Boolean*/ value){
			var disableFunc = lang.hitch(this, function(){
				if((!this.disabled && value) || (!this._buttonEnabledPlugins && value)){
				// Disable editor: disable all enabled buttons and remember that list
					array.forEach(this._plugins, function(p){
						p.set("disabled", true);
				});
			}else if(this.disabled && !value){
					// Restore plugins to being active.
					array.forEach(this._plugins, function(p){
						p.set("disabled", false);
				});
			}
			});
			this.setValueDeferred.addCallback(disableFunc);
			this.inherited(arguments);
		},

		_setStateClass: function(){
			try{
				this.inherited(arguments);

				// Let theme set the editor's text color based on editor enabled/disabled state.
				// We need to jump through hoops because the main document (where the theme CSS is)
				// is separate from the iframe's document.
				if(this.document && this.document.body){
					domStyle.set(this.document.body, "color", domStyle.get(this.iframe, "color"));
				}
			}catch(e){ /* Squelch any errors caused by focus change if hidden during a state change */}
		}
	});

	// Register the "default plugins", ie, the built-in editor commands
	function simplePluginFactory(args){
		return new _Plugin({ command: args.name });
	}
	function togglePluginFactory(args){
		return new _Plugin({ buttonClass: ToggleButton, command: args.name });
	}
	lang.mixin(_Plugin.registry, {
		"undo": simplePluginFactory,
		"redo": simplePluginFactory,
		"cut": simplePluginFactory,
		"copy": simplePluginFactory,
		"paste": simplePluginFactory,
		"insertOrderedList": simplePluginFactory,
		"insertUnorderedList": simplePluginFactory,
		"indent": simplePluginFactory,
		"outdent": simplePluginFactory,
		"justifyCenter": simplePluginFactory,
		"justifyFull": simplePluginFactory,
		"justifyLeft": simplePluginFactory,
		"justifyRight": simplePluginFactory,
		"delete": simplePluginFactory,
		"selectAll": simplePluginFactory,
		"removeFormat": simplePluginFactory,
		"unlink": simplePluginFactory,
		"insertHorizontalRule": simplePluginFactory,

		"bold": togglePluginFactory,
		"italic": togglePluginFactory,
		"underline": togglePluginFactory,
		"strikethrough": togglePluginFactory,
		"subscript": togglePluginFactory,
		"superscript": togglePluginFactory,

		"|": function(){
			return new _Plugin({ button: new ToolbarSeparator(), setEditor: function(editor){this.editor = editor;}});
		}
	});

	return Editor;
});

},
'ibm/usmi/widgets/table/ConfigureColumnsDialog':function(){
require({cache:{
'url:ibm/usmi/widgets/table/templates/ConfigureColumnsDialog.html':"<div class=\"oneui\">\r\n<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n\t<div data-dojo-type=\"idx.oneui.Dialog\" data-dojo-attach-point=\"dialog\"\r\n\t\tclass=\"configColumnsDialog\" title=\"${resources_.CONFIGURE_SETTINGS}\"\r\n\t\ttabindex=0 role=\"dialog\"\r\n\t\tdata-dojo-props='\r\n  \t\t\t\tcloseButtonLabel: this.nls.CANCEL,\r\n\t\t\t\tbuttons: this.actionButtons,\r\n\t\t\t\ttitle: this.nls.CONFIG_COLUMNS\r\n\t'>\r\n\t\t<table class=\"configColumnsDialogTable\" border=\"0\" cellspacing=\"0\"\r\n\t\t\tcellpadding=\"0\" role=\"presentation\" style=\"width: 670px;\">\r\n\t\t\t<tr>\r\n\t\t\t\t<td colspan='2'><label>${nls.CHOOSE_COLUMNS}</label> <br />\r\n\t\t\t\t\t<div style=\"display: inline;\">\r\n\t\t\t\t\t\t<table>\r\n\t\t\t\t\t\t\t<tr>\r\n\t\t\t\t\t\t\t\t<td><label>${nls.REORDER_COLUMNS}</label></td>\r\n\t\t\t\t\t\t\t</tr>\r\n\t\t\t\t\t\t</table>\r\n\t\t\t\t\t</div></td>\r\n\t\t\t</tr>\r\n\t\t\t<tr style=\"vertical-align: bottom;\">\r\n\t\t\t\t<td>\r\n\t\t\t\t\t<div class=\"columnsActionButtons tivTable\">\r\n\r\n\t\t\t\t\t\t<table border=\"0\" cellspacing=\"7\" cellpadding=\"0\"\r\n\t\t\t\t\t\t\trole=\"presentation\">\r\n\t\t\t\t\t\t\t<tbody>\r\n\t\t\t\t\t\t\t\t<tr>\r\n\t\t\t\t\t\t\t\t\t<!--<td><span data-dojo-attach-point=\"selectAllButton\"\r\n\t\t\t\t\t\t\t\t\t\trole=\"button\"></td>\r\n\t\t\t\t\t\t\t\t\t<td><span data-dojo-attach-point=\"deselectAllButton\"\r\n\t\t\t\t\t\t\t\t\t\trole=\"button\"></td>-->\r\n\t\t\t\t\t\t\t\t\t<td>\r\n\t\t\t\t\t\t\t\t\t\t<div class=\"resetToDefaultBtn\">\r\n\t\t\t\t\t\t\t\t\t\t\t<span data-dojo-attach-point=\"resetToDefaultsButton\"\r\n\t\t\t\t\t\t\t\t\t\t\t\trole=\"button\">\r\n\t\t\t\t\t\t\t\t\t\t</div>\r\n\t\t\t\t\t\t\t\t\t</td>\r\n\t\t\t\t\t\t\t\t</tr>\r\n\t\t\t\t\t\t\t</tbody>\r\n\t\t\t\t\t\t</table>\r\n\t\t\t\t\t\t<label class=\"availableColumnsLbl\">${nls.AVAILABLE_COLUMNS}</label>\r\n\t\t\t\t\t</div>\r\n\t\t\t\t</td>\r\n\t\t\t\t<td class=\"selectedColumnsLbl\">${nls.SELECTED_COLUMNS}</td>\r\n\t\t\t</tr>\r\n\t\t\t<tr>\r\n\t\t\t\t<td>\r\n\t\t\t\t\t<div class=\"availableColumns dialog\" tabStrip=\"false\"\r\n\t\t\t\t\t\tid=\"${id}tabs\" data-dojo-type=\"dijit.layout.LayoutContainer\">\r\n\t\t\t\t\t\t<div data-dojo-type=\"dijit.layout.ContentPane\"\r\n\t\t\t\t\t\t\tstyle=\"width: 96%; overflow-y: hidden;\">\r\n\t\t\t\t\t\t\t<div data-dojo-attach-point=\"loadingDiv\">${nls.LOADING}</div>\r\n\t\t\t\t\t\t\t<div id=\"${id}columnsVisibleDivContainer\"\r\n\t\t\t\t\t\t\t\tdata-dojo-attach-point=\"columnsDiv\" style=\"display: none;\">\r\n\t\t\t\t\t\t\t</div>\r\n\t\t\t\t\t\t</div>\r\n\t\t\t\t\t</div>\r\n\t\t\t\t</td>\r\n\t\t\t\t<td style=\"padding-left: 20px\">\r\n\t\t\t\t\t<div class=\"selectedColumns dialog\" tabStrip=\"false\"\r\n\t\t\t\t\t\tid=\"${id}_selected_tab\"\r\n\t\t\t\t\t\tdata-dojo-type=\"dijit.layout.LayoutContainer\">\r\n\t\t\t\t\t\t<div data-dojo-type=\"dijit.layout.ContentPane\"\r\n\t\t\t\t\t\t\tid=\"${id}selectedColumnsdiv\" style=\"width: 96%; height: 100%\"></div>\r\n\t\t\t\t\t</div>\r\n\t\t\t\t</td>\r\n\t\t\t</tr>\r\n\t\t</table>\r\n\t</div>\r\n</div>"}});
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

define("ibm/usmi/widgets/table/ConfigureColumnsDialog", [
	"dojo/dom",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/dom-construct",
	"dijit/_WidgetBase",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	"dijit/form/Button",
	"ibm/usmi/util/DataStore",
	"ibm/usmi/util/Utils",
	"dojo/i18n!./nls/ColumnChooser",
	"dojo/text!./templates/ConfigureColumnsDialog.html",
	"ibm/tivoli/navmodel/Dataset",
	"dojoe/tablex/_ConfigureColumnsDialog",
	"ibm/usmi/widgets/table/_TableOptions",
	"idx/oneui/Dialog",
	"dojo/_base/window",
	"dojo/dom-class",
	"dojo/_base/event" // event.stop
], function (dom, declare, lang, domConstruct, _WidgetBase, TemplatedMixin, WidgetsInTemplateMixin, Button, DataStore, Utils, mri,
		template, CuriDataset, _ConfigureColumnsDialog, _TableOptions, Dialog, win, domClass, event) {
	return declare([
		_WidgetBase,
		TemplatedMixin,
		WidgetsInTemplateMixin,
		_ConfigureColumnsDialog
	], {
		templateString : template,
		nls : mri,
		
		constructor: function() {
			if(this.actionButtons){
				delete this.actionButtons;
			}
			this.actionButtons=[];
			var actionButton=new Button({
				label: this.resources_.OK,
				onClick: lang.hitch(this, this.onOK),
				onKeypress: lang.hitch(this, this.onOK)
			});
			this.actionButtons.push(actionButton);
		},

		postCreate : function () {
			/*
			this.dialog=new Dialog({
				closeButtonLabel: this.nls.CANCEL,
				buttons: this.actionButtons,
				title: this.resources_.CONFIGURE_SETTINGS,
				style: "width:670px;height:370px;overflow:auto;;",
				content: this.dialogContents
			});
			*/
			
			this.dialog.closeButton.onClick=lang.hitch(this, function(evt){
				this.dialog.onCancel();
				event.stop(evt);	
				setTimeout (dojo.hitch (this, this.kill), 100);	
			});
			
			this.optionsWidget_ = new _TableOptions({
				grid : this.grid,
				_structure: this._structure,
				configColumnsWidget : this
			}, dom.byId(this.id + "columnsVisibleDivContainer"));
			this.optionsWidget_.startup();

			var selectIcon = new Button({
				id : this.id + "selectAllIcon",
				showLabel : false,
				iconClass : "selectAllIcon",
				label : this.nls.SELECT_ALL
			}, this.selectAllButton);

			selectIcon.onClick = lang.hitch(this, "onSelectAll");
			selectIcon.onKeypress = lang.hitch(this, "onSelectAll");

			var deSelectIcon = new Button({
				id : this.id + "deselectAllIcon",
				showLabel : false,
				iconClass : "deselectAllIcon",
				label : this.nls.DESELECT_ALL
			}, this.deselectAllButton);

			console.log("Terry reset to default text: "+this.nls.RESET_DEFAULTS);
			var resetIcon = new Button({
				id : this.id + "resetToDefaultsIcon",
				showLabel : true,
				iconClass : "resetToDefaultsIcon",
				label : this.nls.RESET_DEFAULTS
			}, this.resetToDefaultsButton);

			deSelectIcon.onClick = lang.hitch(this, "onDeselectAll");
			deSelectIcon.onKeypress = lang.hitch(this, "onDeselectAll");

			resetIcon.onClick = lang.hitch(this, "onRestDefaults");
			resetIcon.onKeypress = lang.hitch(this, "onRestDefaults");
			
		},
		
		destroy : function () {
			if (this.optionsWidget) {
				this.optionsWidget.destroy();
			}
			if(this.columnsReorderHelpTooltip){
				this.columnsReorderHelpTooltip.destroy();
			}
			domConstruct.destroy("columnsReorderHelpImg");
			this.inherited(arguments);
		},
		
		onRestDefaults : function () {
			this.optionsWidget_.onRestDefaults();
		},
		
		onOK: function(){
			domClass.add(win.body(), 'configureColumnsWaiting');
			this.optionsWidget_.onSave();
			this.hide();
			domClass.remove(win.body(), 'configureColumnsWaiting');
		}
	});
});

},
'ibm/tivoli/tip/dojo/_Server':function(){
// wrapped by build app
define("ibm/tivoli/tip/dojo/_Server", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/tip/dojo/util"], function(dijit,dojo,dojox){
/****************************************************************************
* Provides I/O access to the controller servlet.
* Handles TIP protocol of returning server-side exceptions in a valid
* data response besides Dojo's error and timeout conditions.
****************************************************************************/
dojo.require("ibm.tivoli.tip.dojo.util");

dojo.provide("ibm.tivoli.tip.dojo._Server");

dojo.declare("ibm.tivoli.tip.dojo._Server", 
             null,
{	 
	_totalRequests:   0,        // total server requests made
  _activeRequests:  0,        // # server request(s) currently active
	
  /************************************************************************
  * Object constructor
  ************************************************************************/
	constructor: function( mixinData )
	{
    dojo.mixin( this, mixinData );
	},
	
	/************************************************************************
	* Perform a HTTP GET request.
	************************************************************************/
	/* void */ get: function( /* Object */ requestKW, checkTimeout )
	{
    this._preRequest( requestKW, checkTimeout );
	
    //console.debug( "Ready to xhrGet", requestKW );
    //console.debug( ibm.tivoli.tip.dojo.util.fieldsOf(requestKW) );
    //console.debug( "Timeout", requestKW.timeout );

	  dojo.xhrGet( requestKW ); 
	},
	
	/************************************************************************
	* Perform a HTTP POST request.  Preferred for data forms that do
  * NOT contain any file-input fields...
	************************************************************************/
	/* void */ post: function( /* Object */ requestKW, checkTimeout )
	{
    this._preRequest( requestKW, checkTimeout );
	
	  dojo.xhrPost( requestKW ); 
	},

  /************************************************************************
  * Common PRE request processing of the request KW object.
  ************************************************************************/
  /* Object */ _preRequest: function( /* Object */ requestKW, checkTimeout )
  {
  	if (checkTimeout === undefined) { checkTimeout = true; }
    this._requestStarting( ( requestKW.showBusyDialog !== undefined ) ? requestKW.showBusyDialog : true );                 

    requestKW._server      = this;                 // save self reference for use in load/error handlers below   
    requestKW._requestID   = this._totalRequests;  // unofficial request ID

    requestKW.url          = requestKW.appendURL ? 
      this.global.controller + requestKW.appendURL :
      this.global.controller;

    requestKW.handleAs	   = "json";  
    requestKW.mimetype     = "application/json";
    
    if (checkTimeout) {
    	requestKW.content.requestType  = "global";     // indicates that the request was sent here
    }

    requestKW.preventCache = true;     // fix IE cache bug

    // provide default timeout if none specified

    if ( ! requestKW.timeout ) {
           requestKW.timeout = 0; 
    }

    // our code handles it all!  NOTE: trying to set callbacks on "deferred" object did NOT work

    requestKW.load  = this._loadHandler;
    requestKW.error = this._errorHandler;

    return( requestKW ); 
  },
	
	/************************************************************************
	* Standard JSON request "load" handler.
  *
  * Use the appropriate "how to handle" directions from the request:
  * - "onExc"      for server-side errors returned in the 'exc' field of response
  * - "onResponse" for response data
  *
  *	NOTE "this" will be the object passed as argument to dojo.xhrXxxx() used (aka requestKW)
	************************************************************************/
  /* response */ _loadHandler: function( response, ioArgs ) 
	{
    //console.debug( "************************ Server._loadHandler ************************" );

//  console.debug( ibm.tivoli.tip.dojo.util.fieldsOf(this) );      proved its requestKW
//  console.debug( ibm.tivoli.tip.dojo.util.fieldsOf(response) );
//  console.debug( ibm.tivoli.tip.dojo.util.fieldsOf(ioArgs) );    ioArgs.args == requestKW

    //console.debug( "  ** response", dojo.toJson(response) );

    var server = this._server;

    server._requestCompleted();

	  // was an exception detected/reported from the server-side?

    var exc = response.exc;

	  if ( exc )
	  { 
      //console.debug( "    !!!!! handling EXC !!!!!" );

      // ISC session timeout?
      if ( exc == "ISCSessionTimeoutException" ) 
      {
        ibm.tivoli.tip.dojo.util.refreshPage();
        return;
      }

      var howTo  = this.onExc;

      if ( ! howTo ) {
        howTo = { showMsg : true };    // use default
      }

      //console.debug( "  ?? howTo",  dojo.toJson(howTo) );

      // Crop the exc message to maximum size
      server.cropExcMsg( response );

      //TODO: handle new NLS enabled exceptions...
      var msgKW = null;
      
      if (response.excID) {
	      msgKW = { 
                   messageId : response.excID,
                   message   : response.excMsg
                  };
      } else {
    	  msgKW = { 
                   messageId : server.global.tipnls( "BIND_ERROR_ID" ),
                   message   : server.global.tipnls( "BIND_ERROR_MSG", response )
                  };
	  }

      //console.debug( "  ?? msgKW",  dojo.toJson(msgKW) );

      if ( howTo.showMsg ) {
       // server.global.showErrorMessage( msgKW );
	   server.global.showMessage( msgKW);
      }

      if ( howTo.callback ) {
        howTo.callback( msgKW, this /* requestKW */ );
      }
	  }
	
	  // No exception --> forward to callback
	  else
	  {
      howTo = this.onResponse;

      if ( howTo && howTo.callback ) {
	      howTo.callback( response, this /* requestKW */ );
      }
	  }

    return( response ); // Dojo best practice 
	},

	/************************************************************************
	* Standard JSON request error handler
  *
  * How to handle directions are contained in the request's "onExc" field.
  *
  *	NOTE "this" will be the object passed as argument to dojo.xhrXxxx() used
  *
  * error.message:    specific error message, "too much recursion"
  * error.fileName:   "https://localhost:16316/ibm/console/secure/isclite/scripts/dojo/dojo.js"
  * error.lineNumber: 20
  * error.stack:      callback stack (can be very deep )
  * error.name:       "InternalError"
  *
  * error.dojoType:   "timeout" or "cancel", but other error types are possible.
  * dojoType=timeout message=timeout exceeded
  *
  * Error: bad http response code:404 message=bad http response code:404
  * 
  * console.error("HTTP status code: ", ioArgs.xhr.status); 
	************************************************************************/
	/* data */ _errorHandler: function( error, ioArgs ) 
	{
	//alert ("in Server._errorHandler with error: " + dojo.toJson(error));
    console.error( "************************ Server._errorHandler ************************" );

    console.error( "  >> this",   ibm.tivoli.tip.dojo.util.fieldsOf(this) );
    console.error( "  >> error",  ibm.tivoli.tip.dojo.util.fieldsOf(error) );
    console.error( "  >> ioArgs", ibm.tivoli.tip.dojo.util.fieldsOf(ioArgs) );
    console.error( "  >> ioArgs.args", ibm.tivoli.tip.dojo.util.fieldsOf(ioArgs.args) );

    console.error( "  -- error",  dojo.toJson(error) );

    var server = this._server;

    server._requestCompleted();

    var howTo = this.onError;
    var msgKW = null;

    if ( ! howTo ) {
      howTo = { showMsg : true };    // use default
    }

    if ( error.dojoType && ( error.dojoType === "timeout" ) ) 
    {
      msgKW = { 
                messageId : server.global.tipnls( "BIND_TIMEOUT_ID" ),
                message   : server.global.tipnls( "BIND_TIMEOUT_MSG", error )
              }; 
    }
    else if ( error.name && ( error.name === "InternalError" ) ) 
    {
      msgKW = { 
                messageId : server.global.tipnls( "JS_ERROR_ID" ),
                message   : server.global.tipnls( "JS_ERROR_MSG", error )
              }; 
    }
    else
    {
      msgKW = { 
                messageId : server.global.tipnls( "XHR_ERROR_ID" ),
                message   : server.global.tipnls( "XHR_ERROR_MSG", error )
              }; 
    }

    if ( howTo.showMsg ) {
	    server.global.showErrorMessage( msgKW );
    }

    if ( howTo.callback ) {
      howTo.callback( msgKW, this /* requestKW */ );
    }

    return( error ); // Dojo best practice 
	},

  /************************************************************************
  * Starting a new server request.
  ************************************************************************/
  _requestStarting: function( showBusyDialog ) 
  {
    // increment total server requests 
    this._totalRequests++;

    // increment outstanding server requests 
    this._activeRequests++;

    // show wait dialog on first active request
    if ( showBusyDialog === true && this._activeRequests == 1 )
    {
      this.wait_timer = setInterval( dojo.hitch( this, "_showBusyPane" ), 100 );
    }
  },
  
  /************************************************************************
  * Show Busy Pane
  *    
  * Constructing a ibm.tivoli.tip.dijit.BusyPane and passing its ID on 
  * busyPaneID when constructing Server object will provide feedback
  * when requests are made
  ************************************************************************/
  _showBusyPane: function()
  {
  	// console.log ("enter _showBusyPane");
	// commenting out console.logs - refer TWL RTC 42745 
    clearInterval( this.wait_timer );
    // in case of race condition, make sure that active requests > 0
    if ( this._activeRequests > 0 ) {
	    var busyPane = dijit.byId( this.busyPaneID );
	    if ( busyPane ) {
	    	//console.log ("showing busypane");
	      busyPane.show(true); // size to portlet
	    } else {
	    	//console.log ("busypane does not exist");
			// do nothing
	    }
    } else {
    	 //console.log ("active requests ! > 0 so not showing busypane");
    }
  },

  /************************************************************************
  * Completing a server request.
  ************************************************************************/
  _requestCompleted: function() 
  {
	  //decrement the # outstanding requests 
	  this._activeRequests--;

	  // if no more active requests --> close wait dialog
	  if ( this._activeRequests === 0 )
	  {
		  clearInterval( this.wait_timer );
      
          var busyPane = dijit.byId( this.busyPaneID );
          if ( busyPane )
          {
            busyPane.hide();
          }
	  }  	
  },

  /***************************************************************************
  * Crop the message in the 'excMsg' field to the maximum allowed length.
  ***************************************************************************/
  cropExcMsg: function( response )
  {
    // provide a default exc message if field isn't already available
    if ( ! response.excMsg ) {
      response.excMsg = "-?-";
    }

    // access externalized MAX length
    var msgMax = this.global.nlsTIPResources.MAX_WIDTH_EXC_MSG;

    // exc message too long?
    if ( response.excMsg.length > msgMax ) 
    {
      response.excMsg = response.excMsg.substring( 0, msgMax ) + this.global.tipnls( "TEXT_CROPPED" );
    }
  }

} );
});

},
'ibm/tivoli/navmodel/ProviderUtils':function(){
// wrapped by build app
define("ibm/tivoli/navmodel/ProviderUtils", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/Utils,ibm/tivoli/navmodel/Trace,ibm/tivoli/navmodel/Provider,ibm/tivoli/navmodel/Datasource,ibm/tivoli/navmodel/DatasetInfo"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.ProviderUtils"); 

// includes
dojo.require("ibm.tivoli.navmodel.Utils");
dojo.require("ibm.tivoli.navmodel.Trace");
dojo.require("ibm.tivoli.navmodel.Provider");
dojo.require("ibm.tivoli.navmodel.Datasource");
dojo.require("ibm.tivoli.navmodel.DatasetInfo");

ibm.tivoli.navmodel.ProviderUtils._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.ProviderUtils");

ibm.tivoli.navmodel.ProviderUtils.createRemoteProvider = function(kwArgs) {
	// summary: 
	//		Create a remote provider definition on the server.
	// baseUrl: String
	//		base url for identifying rest service
	// provider: ibm.tivoli.navmodel.Provider
	//		provider definition to create
	// remoteAttributes: Boolean
	//		Indicates whether a request should be sent to the remote
	//		provider to retrieve additional attributes.
	// version: Float

	var args = {
		baseUrl: null,
		provider: null,
		remoteAttributes: false,
		version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion()
	};
	dojo.mixin(args, kwArgs);

	var exception = null;		
	var url = args.baseUrl + "/providers/" + encodeURIComponent(args.provider.getId());
	if (args.remoteAttributes) {
		url += "?remoteAttributes=true";
	}
	
	var json = dojo.toJson(args.provider.getAttributes(), false);
	
	dojo.xhrPut({
		url: url,
		headers: {
			'Accept': ibm.tivoli.navmodel.Utils.getContentType(args.version),
			'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(args.version, "provider"), 	
			'Content-Length': json.length		
		},
		handleAs: "json",
		putData: json,
		sync: true,
		load: dojo.hitch(this, function(result, ioArgs){														
		}),
		error: dojo.hitch(this, function(error, ioArgs){
			exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "createRemoteProvider", ibm.tivoli.navmodel.ProviderUtils._trace, args.version);
		})
	});
	
	if (exception) {
		throw exception;
	}					
};


ibm.tivoli.navmodel.ProviderUtils.deleteRemoteProvider = function(kwArgs) {
	// summary: 
	//		Delete a remote provider definition on the server.
	// baseUrl: String
	//		base url for identifying rest service
	// provider: String
	//		provider id
	// version: Float

	var args = {
		baseUrl: null,
		provider: null,
		version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion()
	};
	dojo.mixin(args, kwArgs);

	var exception = null;		
	var url = args.baseUrl + "/providers/" + encodeURIComponent(args.provider);
	
	dojo.xhrDelete({
		url: url,
		headers: {
			'Accept': ibm.tivoli.navmodel.Utils.getContentType(args.version)	
		},
		handleAs: "json",
		sync: true,
		load: dojo.hitch(this, function(result, ioArgs){														
		}),
		error: dojo.hitch(this, function(error, ioArgs){
			exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "deleteRemoteProvider", ibm.tivoli.navmodel.ProviderUtils._trace, args.version);
		})
	});
	
	if (exception) {
		throw exception;
	}					
};


ibm.tivoli.navmodel.ProviderUtils.getProviders = function(kwArgs) {
	// summary: 
	//		Returns list of ibm.tivoli.navmodel.Provider instances
	// onComplete: function(providers, providerCount, kwArgs)
	//		callback for successful fetch
	// onError: function(error, exception, ioArgs)
	//		callback for error conditions
	// provider: ibm.tivoli.navmodel.Provider
	//		Optional: if specified will be used as passthru connection information
	//		to fetch providers from remote CURI.  If NOT specified; the providers from
	//		the local CURI will be returned
	// baseUrl: String
	//		base url for identifying rest service
	// start: int
	//		start index to fetch
	// count: int
	//		number of providers to fetch; null means all
	// sort: String
	//		comma deliminated attribute string (preceded with optional +/- indicating ascending/descending)
	//		example:  label,type
	// condition: String
	//		condition expression string 
	//		example: description contains 'abcd'
	// sync: Boolean
	//		indicates if fetch is synchronous
	// version: Float

	var args = {
		onComplete: function(providers, providerCount, kwArgs){
		},
		onError: function(error, exception, ioArgs){				
		},
		provider: null,
		baseUrl: null,
		start: 0,
		count: null,
		sort: null,
		condition: null,
		sync: false,
		version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion()
	};
	dojo.mixin(args, kwArgs);

	var url = args.baseUrl + "/providers";
	
	// returning providers from local CURI
	if (!args.provider) {
		var content = ibm.tivoli.navmodel.Utils.getContent();
		if (args.start) {
			content.start = args.start;
		}
		if (args.count) {
			content.count = args.count;
		}
		if (args.sort) {
			content.sort = args.sort;
		}
		if (args.condition) {
			content.condition = args.condition;
		}
	
		return dojo.xhrGet({
			url: url,
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(args.version, "providerList")			
			},
			handleAs: "json",
			sync: args.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var providers = [];			
				if (result.items) {
					var length = result.items.length;
					for (index = 0;  index < length;  ++index) {
						providers[index] = new ibm.tivoli.navmodel.Provider({provider: result.items[index], baseUrl: args.baseUrl, version: args.version});
					}
				}
					
				args.onComplete(providers, result.filteredRows, args);										
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				var exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "getProviders", ibm.tivoli.navmodel.ProviderUtils._trace, args.version);
				args.onError(error, exception, ioArgs);
			})
		});
	}
	
	// returning providers from remote CURI
	else {
		var qm = false;
		if (args.start) {
			url += (qm ? "&start=" : "?start=") + args.start;
			qm = true;
		}
		if (args.count) {
			url += (qm ? "&count=" : "?count=") + args.count;
			qm = true;			
		}
		if (args.sort) {
			url += (qm ? "&sort=" : "?sort=") + args.sort;
			qm = true;
		}
		
		if (args.condition) {
			url += (qm ? "&condition=" : "?condition=") + args.condition;			
		}
		
		var json = dojo.toJson(args.provider.getAttributes(), false);
			
		return dojo.xhrPost({
			url: url,
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(args.version, "providerList"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(args.version, "provider"), 	
				'Content-Length': json.length		
			},
			handleAs: "json",
			postData: json,
			sync: args.sync,
			load: dojo.hitch(this, function(result, ioArgs){
				var providers = [];			
				if (result.items) {
					var length = result.items.length;
					for (index = 0;  index < length;  ++index) {
						providers[index] = new ibm.tivoli.navmodel.Provider({provider: result.items[index], baseUrl: args.baseUrl, version: args.version});
					}
				}
					
				args.onComplete(providers, result.filteredRows, args);										
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				var exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "getProviders", ibm.tivoli.navmodel.ProviderUtils._trace, args.version);
				args.onError(error, exception, ioArgs);
			})
		});
	}	
};

ibm.tivoli.navmodel.ProviderUtils.getProvider = function(kwArgs) {
	// summary: 
	//		Returns the ibm.tivoli.navmodel.Provider matching the given provider id.
	// onComplete: function(provider, kwArgs)
	//		callback for successful fetch
	// onError: function(error, exception, ioArgs)
	//		callback for error conditions
	// baseUrl: String
	//		base url for identifying rest service
	// provider: String
	//		provider id
	// sync: Boolean
	//		indicates if fetch is synchronous	
	// version: Float

	var args = {
		onComplete: function(provider, kwArgs){
		},
		onError: function(error, exception, ioArgs){
		},
		baseUrl: null,
		provider: null,
		sync: false,
		version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion()
	};
	dojo.mixin(args, kwArgs);

	var url = args.baseUrl + "/providers/" + encodeURIComponent(args.provider);
	var content = ibm.tivoli.navmodel.Utils.getContent();

	return dojo.xhrGet({
		url: url,
		headers: {
			'Accept': ibm.tivoli.navmodel.Utils.getContentType(args.version, "provider")			
		},
		handleAs: "json",
		sync: args.sync,
		content: content,
		load: dojo.hitch(this, function(result, ioArgs){			
			var provider = new ibm.tivoli.navmodel.Provider({provider: result});
			args.onComplete(provider, args);											
		}),
		error: dojo.hitch(this, function(error, ioArgs){
			var exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "getProvider", ibm.tivoli.navmodel.ProviderUtils._trace, args.version);
			args.onError(error, exception, ioArgs);
		})
	});
};

ibm.tivoli.navmodel.ProviderUtils.getProviderById = function(kwArgs) {
	// summary: 
	//		Returns the ibm.tivoli.navmodel.Provider matching the given provider id.
	// baseUrl: String
	//		base url for identifying rest service
	// provider: String
	//		provider id
	// version: Float

	var args = {
		baseUrl: null,
		provider: null,
		version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion()
	};
	dojo.mixin(args, kwArgs);

	var provider = null;
	var exception = null;		
	var url = args.baseUrl + "/providers/" + encodeURIComponent(args.provider);
	var content = ibm.tivoli.navmodel.Utils.getContent();

	dojo.xhrGet({
		url: url,
		headers: {
			'Accept': ibm.tivoli.navmodel.Utils.getContentType(args.version, "provider")			
		},
		handleAs: "json",
		sync: true,
		content: content,
		load: dojo.hitch(this, function(result, ioArgs){			
			provider = new ibm.tivoli.navmodel.Provider({provider: result});											
		}),
		error: dojo.hitch(this, function(error, ioArgs){
			exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "getProviderById", ibm.tivoli.navmodel.ProviderUtils._trace, args.version);
		})
	});
	
	if (exception) {
		throw exception;
	}				

	return provider;
};


ibm.tivoli.navmodel.ProviderUtils.getDatasources = function(kwArgs) {
	// summary: 
	//		Returns the ibm.tivoli.navmodel.Datasource instances for a provider
	// onComplete: function(datasources, datasourceCount, kwArgs)
	//		callback for successful fetch
	// onError: function(error, exception, ioArgs)
	//		callback for error conditions
	// baseUrl: String
	//		base url for identifying rest service
	// provider: String
	//		provider id 
	// start: int
	//		start index to fetch
	// count: int
	//		number of datasources to fetch; null means all
	// sort: String
	//		comma deliminated attribute string (preceded with optional +/- indicating ascending/descending)
	//		example:  label,type
	// condition: String
	//		condition expression string 
	//		example: description contains 'abcd'
	// sync: Boolean
	//		indicates if fetch is synchronous
	// version: Float

	var args = {
		onComplete: function(datasources, datasourceCount, kwArgs){
		},
		onError: function(error, exception, ioArgs){				
		},
		baseUrl: null,
		provider: null,
		start: 0,
		count: null,
		sort: null,
		condition: null,
		sync: false,
		version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion()
	};
	dojo.mixin(args, kwArgs);

	var url = args.baseUrl + "/providers/" + encodeURIComponent(args.provider) + "/datasources";
	var content = ibm.tivoli.navmodel.Utils.getContent();
	if (args.start) {
		content.start = args.start;
	}
	if (args.count) {
		content.count = args.count;
	}
	if (args.sort) {
		content.sort = args.sort;
	}
	if (args.condition) {
		content.condition = args.condition;
	}
	
	return dojo.xhrGet({
		url: url,
		headers: {
			'Accept': ibm.tivoli.navmodel.Utils.getContentType(args.version, "datasourceList")			
		},
		handleAs: "json",
		sync: args.sync,
		content: content,
		load: dojo.hitch(this, function(result, ioArgs){
			var datasources = [];			
			if (result.items) {
				var length = result.items.length;
				for (index = 0;  index < length;  ++index) {
					datasources[index] = new ibm.tivoli.navmodel.Datasource({datasource: result.items[index], baseUrl: args.baseUrl, version: args.version});
				}
			}
					
			args.onComplete(datasources, result.filteredRows, args);										
		}),
		error: dojo.hitch(this, function(error, ioArgs){
			var exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "getDatasources", ibm.tivoli.navmodel.ProviderUtils._trace, args.version);
			args.onError(error, exception, ioArgs);
		})
	});	
};


ibm.tivoli.navmodel.ProviderUtils.getDatasets = function(kwArgs) {
	// summary: 
	//		Returns the ibm.tivoli.navmodel.DatasetInfo instances for a provider/datasource
	// onComplete: function(datasets, datasetCount, kwArgs)
	//		callback for successful fetch
	// onError: function(error, exception, ioArgs)
	//		callback for error conditions
	// baseUrl: String
	//		base url for identifying rest service
	// provider: String
	//		provider id 
	// datasource: String
	//		datasourceId or null 
	// start: int
	//		start index to fetch
	// count: int
	//		number of datasources to fetch; null means all
	// sort: String
	//		comma deliminated attribute string (preceded with optional +/- indicating ascending/descending)
	//		example:  label,type
	// condition: String
	//		condition expression string 
	//		example: description contains 'abcd'
	// sync: Boolean
	//		indicates if fetch is synchronous
	// version: Float

	var args = {
		onComplete: function(datasets, datasetCount, kwArgs){
		},
		onError: function(error, exception, ioArgs){				
		},
		baseUrl: null,
		provider: null,
		datasource: null,
		start: 0,
		count: null,
		sort: null,
		condition: null,
		sync: false,
		version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion()
	};
	dojo.mixin(args, kwArgs);

	var url = args.baseUrl + "/providers/" + encodeURIComponent(args.provider);
	url += (args.datasource ? "/datasources/" + encodeURIComponent(args.datasource) : "");
	url += "/datasets";
	
	var content = ibm.tivoli.navmodel.Utils.getContent();
	if (args.start) {
		content.start = args.start;
	}
	if (args.count) {
		content.count = args.count;
	}
	if (args.sort) {
		content.sort = args.sort;
	}
	if (args.condition) {
		content.condition = args.condition;
	}
	
	return dojo.xhrGet({
		url: url,
		headers: {
			'Accept': ibm.tivoli.navmodel.Utils.getContentType(args.version, "datasetList")			
		},
		handleAs: "json",
		sync: args.sync,
		content: content,
		load: dojo.hitch(this, function(result, ioArgs){
			var datasets = [];			
			if (result.items) {
				var length = result.items.length;
				for (index = 0;  index < length;  ++index) {
					datasets[index] = new ibm.tivoli.navmodel.DatasetInfo({dataset: result.items[index], version: args.version});
				}
			}
					
			args.onComplete(datasets, result.filteredRows, args);										
		}),
		error: dojo.hitch(this, function(error, ioArgs){
			var exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "getDatasets", ibm.tivoli.navmodel.ProviderUtils._trace, args.version);
			args.onError(error, exception, ioArgs);
		})
	});	
};

});

},
'dijit/form/_ComboBoxMenuMixin':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.set
	"dojo/i18n", // i18n.getLocalization
	"dojo/_base/window", // win.doc.createTextNode
	"dojo/i18n!./nls/ComboBox"
], function(array, declare, domAttr, i18n, win){

// module:
//		dijit/form/_ComboBoxMenuMixin
// summary:
//		Focus-less menu for internal use in `dijit.form.ComboBox`

return declare( "dijit.form._ComboBoxMenuMixin", null, {
	// summary:
	//		Focus-less menu for internal use in `dijit.form.ComboBox`
	// tags:
	//		private

	// _messages: Object
	//		Holds "next" and "previous" text for paging buttons on drop down
	_messages: null,

	postMixInProperties: function(){
		this.inherited(arguments);
		this._messages = i18n.getLocalization("dijit.form", "ComboBox", this.lang);
	},

	buildRendering: function(){
		this.inherited(arguments);

		// fill in template with i18n messages
		this.previousButton.innerHTML = this._messages["previousMessage"];
		this.nextButton.innerHTML = this._messages["nextMessage"];
	},

	_setValueAttr: function(/*Object*/ value){
		this.value = value;
		this.onChange(value);
	},

	onClick: function(/*DomNode*/ node){
		if(node == this.previousButton){
			this._setSelectedAttr(null);
			this.onPage(-1);
		}else if(node == this.nextButton){
			this._setSelectedAttr(null);
			this.onPage(1);
		}else{
			this.onChange(node);
		}
	},

	// stubs
	onChange: function(/*Number*/ /*===== direction =====*/){
		// summary:
		//		Notifies ComboBox/FilteringSelect that user selected an option.
		// tags:
		//		callback
	},

	onPage: function(/*Number*/ /*===== direction =====*/){
		// summary:
		//		Notifies ComboBox/FilteringSelect that user clicked to advance to next/previous page.
		// tags:
		//		callback
	},

	onClose: function(){
		// summary:
		//		Callback from dijit.popup code to this widget, notifying it that it closed
		// tags:
		//		private
		this._setSelectedAttr(null);
	},

	_createOption: function(/*Object*/ item, labelFunc){
		// summary:
		//		Creates an option to appear on the popup menu subclassed by
		//		`dijit.form.FilteringSelect`.

		var menuitem = this._createMenuItem();
		var labelObject = labelFunc(item);
		if(labelObject.html){
			menuitem.innerHTML = labelObject.label;
		}else{
			menuitem.appendChild(
				win.doc.createTextNode(labelObject.label)
			);
		}
		// #3250: in blank options, assign a normal height
		if(menuitem.innerHTML == ""){
			menuitem.innerHTML = "&#160;";	// &nbsp;
		}

		// update menuitem.dir if BidiSupport was required
		this.applyTextDir(menuitem, (menuitem.innerText || menuitem.textContent || ""));

		menuitem.item=item;
		return menuitem;
	},

	createOptions: function(results, options, labelFunc){
		// summary:
		//		Fills in the items in the drop down list
		// results:
		//		Array of items
		// options:
		//		The options to the query function of the store
		//
		// labelFunc:
		//		Function to produce a label in the drop down list from a dojo.data item

		this.items = results;

		// display "Previous . . ." button
		this.previousButton.style.display = (options.start == 0) ? "none" : "";
		domAttr.set(this.previousButton, "id", this.id + "_prev");
		// create options using _createOption function defined by parent
		// ComboBox (or FilteringSelect) class
		// #2309:
		//		iterate over cache nondestructively
		array.forEach(results, function(item, i){
			var menuitem = this._createOption(item, labelFunc);
			menuitem.setAttribute("item", i);	// index to this.items; use indirection to avoid mem leak
			domAttr.set(menuitem, "id", this.id + i);
			this.nextButton.parentNode.insertBefore(menuitem, this.nextButton);
		}, this);
		// display "Next . . ." button
		var displayMore = false;
		// Try to determine if we should show 'more'...
		if(results.total && !results.total.then && results.total != -1){
			if((options.start + options.count) < results.total){
				displayMore = true;
			}else if((options.start + options.count) > results.total && options.count == results.length){
				// Weird return from a data store, where a start + count > maxOptions
				// implies maxOptions isn't really valid and we have to go into faking it.
				// And more or less assume more if count == results.length
				displayMore = true;
			}
		}else if(options.count == results.length){
			//Don't know the size, so we do the best we can based off count alone.
			//So, if we have an exact match to count, assume more.
			displayMore = true;
		}

		this.nextButton.style.display = displayMore ? "" : "none";
		domAttr.set(this.nextButton,"id", this.id + "_next");
	},

	clearResultList: function(){
		// summary:
		//		Clears the entries in the drop down list, but of course keeps the previous and next buttons.
		var container = this.containerNode;
		while(container.childNodes.length > 2){
			container.removeChild(container.childNodes[container.childNodes.length-2]);
		}
		this._setSelectedAttr(null);
	},

	highlightFirstOption: function(){
		// summary:
		//		Highlight the first real item in the list (not Previous Choices).
		this.selectFirstNode();
	},

	highlightLastOption: function(){
		// summary:
		//		Highlight the last real item in the list (not More Choices).
		this.selectLastNode();
	},

	selectFirstNode: function(){
		this.inherited(arguments);
		if(this.getHighlightedOption() == this.previousButton){
			this.selectNextNode();
		}
	},

	selectLastNode: function(){
		this.inherited(arguments);
		if(this.getHighlightedOption() == this.nextButton){
			this.selectPreviousNode();
		}
	},

	getHighlightedOption: function(){
		return this._getSelectedAttr();
	}
});

});

},
'dijit/form/ToggleButton':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/_base/kernel", // kernel.deprecated
	"./Button",
	"./_ToggleButtonMixin"
], function(declare, kernel, Button, _ToggleButtonMixin){

/*=====
	var Button = dijit.form.Button;
	var _ToggleButtonMixin = dijit.form._ToggleButtonMixin;
=====*/

	// module:
	//		dijit/form/ToggleButton
	// summary:
	//		A templated button widget that can be in two states (checked or not).


	return declare("dijit.form.ToggleButton", [Button, _ToggleButtonMixin], {
		// summary:
		//		A templated button widget that can be in two states (checked or not).
		//		Can be base class for things like tabs or checkbox or radio buttons

		baseClass: "dijitToggleButton",

		setChecked: function(/*Boolean*/ checked){
			// summary:
			//		Deprecated.  Use set('checked', true/false) instead.
			kernel.deprecated("setChecked("+checked+") is deprecated. Use set('checked',"+checked+") instead.", "", "2.0");
			this.set('checked', checked);
		}
	});
});

},
'url:dijit/templates/MenuSeparator.html':"<tr class=\"dijitMenuSeparator\">\n\t<td class=\"dijitMenuSeparatorIconCell\">\n\t\t<div class=\"dijitMenuSeparatorTop\"></div>\n\t\t<div class=\"dijitMenuSeparatorBottom\"></div>\n\t</td>\n\t<td colspan=\"3\" class=\"dijitMenuSeparatorLabelCell\">\n\t\t<div class=\"dijitMenuSeparatorTop dijitMenuSeparatorLabel\"></div>\n\t\t<div class=\"dijitMenuSeparatorBottom\"></div>\n\t</td>\n</tr>",
'dijit/MenuItem':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.setSelectable
	"dojo/dom-attr", // domAttr.set
	"dojo/dom-class", // domClass.toggle
	"dojo/_base/event", // event.stop
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/sniff", // has("ie")
	"./_Widget",
	"./_TemplatedMixin",
	"./_Contained",
	"./_CssStateMixin",
	"dojo/text!./templates/MenuItem.html"
], function(declare, dom, domAttr, domClass, event, kernel, has,
			_Widget, _TemplatedMixin, _Contained, _CssStateMixin, template){

/*=====
	var _Widget = dijit._Widget;
	var _TemplatedMixin = dijit._TemplatedMixin;
	var _Contained = dijit._Contained;
	var _CssStateMixin = dijit._CssStateMixin;
=====*/

	// module:
	//		dijit/MenuItem
	// summary:
	//		A line item in a Menu Widget


	return declare("dijit.MenuItem",
		[_Widget, _TemplatedMixin, _Contained, _CssStateMixin],
		{
		// summary:
		//		A line item in a Menu Widget

		// Make 3 columns
		// icon, label, and expand arrow (BiDi-dependent) indicating sub-menu
		templateString: template,

		baseClass: "dijitMenuItem",

		// label: String
		//		Menu text
		label: '',
		_setLabelAttr: { node: "containerNode", type: "innerHTML" },

		// iconClass: String
		//		Class to apply to DOMNode to make it display an icon.
		iconClass: "dijitNoIcon",
		_setIconClassAttr: { node: "iconNode", type: "class" },

		// accelKey: String
		//		Text for the accelerator (shortcut) key combination.
		//		Note that although Menu can display accelerator keys there
		//		is no infrastructure to actually catch and execute these
		//		accelerators.
		accelKey: "",

		// disabled: Boolean
		//		If true, the menu item is disabled.
		//		If false, the menu item is enabled.
		disabled: false,
		
		// dynamic: Boolean
		//this is applicable in case of dynamic ComboButton and defaults to true
		//if set to false in case of dynamic ComboButton, onClick of this item, action on the button does not change to this item action
		dynamic: true,

		_fillContent: function(/*DomNode*/ source){
			// If button label is specified as srcNodeRef.innerHTML rather than
			// this.params.label, handle it here.
			if(source && !("label" in this.params)){
				this.set('label', source.innerHTML);
			}
		},

		buildRendering: function(){
			this.inherited(arguments);
			var label = this.id+"_text";
			domAttr.set(this.containerNode, "id", label);
			if(this.accelKeyNode){
				domAttr.set(this.accelKeyNode, "id", this.id + "_accel");
				label += " " + this.id + "_accel";
			}
			this.domNode.setAttribute("aria-labelledby", label);
			dom.setSelectable(this.domNode, false);
		},

		_onHover: function(){
			// summary:
			//		Handler when mouse is moved onto menu item
			// tags:
			//		protected
			this.getParent().onItemHover(this);
		},

		_onUnhover: function(){
			// summary:
			//		Handler when mouse is moved off of menu item,
			//		possibly to a child menu, or maybe to a sibling
			//		menuitem or somewhere else entirely.
			// tags:
			//		protected

			// if we are unhovering the currently selected item
			// then unselect it
			this.getParent().onItemUnhover(this);

			// When menu is hidden (collapsed) due to clicking a MenuItem and having it execute,
			// FF and IE don't generate an onmouseout event for the MenuItem.
			// So, help out _CssStateMixin in this case.
			this._set("hovering", false);
		},

		_onClick: function(evt){
			// summary:
			//		Internal handler for click events on MenuItem.
			// tags:
			//		private
			this.getParent().onItemClick(this, evt);
			event.stop(evt);
		},

		onClick: function(/*Event*/){
			// summary:
			//		User defined function to handle clicks
			// tags:
			//		callback
		},

		focus: function(){
			// summary:
			//		Focus on this MenuItem
			try{
				if(has("ie") == 8){
					// needed for IE8 which won't scroll TR tags into view on focus yet calling scrollIntoView creates flicker (#10275)
					this.containerNode.focus();
				}
				this.focusNode.focus();
			}catch(e){
				// this throws on IE (at least) in some scenarios
			}
		},

		_onFocus: function(){
			// summary:
			//		This is called by the focus manager when focus
			//		goes to this MenuItem or a child menu.
			// tags:
			//		protected
			this._setSelected(true);
			this.getParent()._onItemFocus(this);

			this.inherited(arguments);
		},

		_setSelected: function(selected){
			// summary:
			//		Indicate that this node is the currently selected one
			// tags:
			//		private

			/***
			 * TODO: remove this method and calls to it, when _onBlur() is working for MenuItem.
			 * Currently _onBlur() gets called when focus is moved from the MenuItem to a child menu.
			 * That's not supposed to happen, but the problem is:
			 * In order to allow dijit.popup's getTopPopup() to work,a sub menu's popupParent
			 * points to the parent Menu, bypassing the parent MenuItem... thus the
			 * MenuItem is not in the chain of active widgets and gets a premature call to
			 * _onBlur()
			 */

			domClass.toggle(this.domNode, "dijitMenuItemSelected", selected);
		},

		setLabel: function(/*String*/ content){
			// summary:
			//		Deprecated.   Use set('label', ...) instead.
			// tags:
			//		deprecated
			kernel.deprecated("dijit.MenuItem.setLabel() is deprecated.  Use set('label', ...) instead.", "", "2.0");
			this.set("label", content);
		},

		setDisabled: function(/*Boolean*/ disabled){
			// summary:
			//		Deprecated.   Use set('disabled', bool) instead.
			// tags:
			//		deprecated
			kernel.deprecated("dijit.Menu.setDisabled() is deprecated.  Use set('disabled', bool) instead.", "", "2.0");
			this.set('disabled', disabled);
		},
		_setDisabledAttr: function(/*Boolean*/ value){
			// summary:
			//		Hook for attr('disabled', ...) to work.
			//		Enable or disable this menu item.

			this.focusNode.setAttribute('aria-disabled', value ? 'true' : 'false');
			this._set("disabled", value);
		},
		_setAccelKeyAttr: function(/*String*/ value){
			// summary:
			//		Hook for attr('accelKey', ...) to work.
			//		Set accelKey on this menu item.

			this.accelKeyNode.style.display=value?"":"none";
			this.accelKeyNode.innerHTML=value;
			//have to use colSpan to make it work in IE
			domAttr.set(this.containerNode,'colSpan',value?"1":"2");

			this._set("accelKey", value);
		}
	});
});

},
'dijit/_WidgetBase':function(){
define([
	"require",			// require.toUrl
	"dojo/_base/array", // array.forEach array.map
	"dojo/aspect",
	"dojo/_base/config", // config.blankGif
	"dojo/_base/connect", // connect.connect
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.byId
	"dojo/dom-attr", // domAttr.set domAttr.remove
	"dojo/dom-class", // domClass.add domClass.replace
	"dojo/dom-construct", // domConstruct.create domConstruct.destroy domConstruct.place
	"dojo/dom-geometry",	// isBodyLtr
	"dojo/dom-style", // domStyle.set, domStyle.get
	"dojo/_base/kernel",
	"dojo/_base/lang", // mixin(), isArray(), etc.
	"dojo/on",
	"dojo/ready",
	"dojo/Stateful", // Stateful
	"dojo/topic",
	"dojo/_base/window", // win.doc.createTextNode
	"./registry"	// registry.getUniqueId(), registry.findWidgets()
], function(require, array, aspect, config, connect, declare,
			dom, domAttr, domClass, domConstruct, domGeometry, domStyle, kernel,
			lang, on, ready, Stateful, topic, win, registry){

/*=====
var Stateful = dojo.Stateful;
=====*/

// module:
//		dijit/_WidgetBase
// summary:
//		Future base class for all Dijit widgets.

// For back-compat, remove in 2.0.
if(!kernel.isAsync){
	ready(0, function(){
		var requires = ["dijit/_base/manager"];
		require(requires);	// use indirection so modules not rolled into a build
	});
}

// Nested hash listing attributes for each tag, all strings in lowercase.
// ex: {"div": {"style": true, "tabindex" true}, "form": { ...
var tagAttrs = {};
function getAttrs(obj){
	var ret = {};
	for(var attr in obj){
		ret[attr.toLowerCase()] = true;
	}
	return ret;
}

function nonEmptyAttrToDom(attr){
	// summary:
	//		Returns a setter function that copies the attribute to this.domNode,
	//		or removes the attribute from this.domNode, depending on whether the
	//		value is defined or not.
	return function(val){
		domAttr[val ? "set" : "remove"](this.domNode, attr, val);
		this._set(attr, val);
	};
}

return declare("dijit._WidgetBase", Stateful, {
	// summary:
	//		Future base class for all Dijit widgets.
	// description:
	//		Future base class for all Dijit widgets.
	//		_Widget extends this class adding support for various features needed by desktop.
	//
	//		Provides stubs for widget lifecycle methods for subclasses to extend, like postMixInProperties(), buildRendering(),
	//		postCreate(), startup(), and destroy(), and also public API methods like set(), get(), and watch().
	//
	//		Widgets can provide custom setters/getters for widget attributes, which are called automatically by set(name, value).
	//		For an attribute XXX, define methods _setXXXAttr() and/or _getXXXAttr().
	//
	//		_setXXXAttr can also be a string/hash/array mapping from a widget attribute XXX to the widget's DOMNodes:
	//
	//		- DOM node attribute
	// |		_setFocusAttr: {node: "focusNode", type: "attribute"}
	// |		_setFocusAttr: "focusNode"	(shorthand)
	// |		_setFocusAttr: ""		(shorthand, maps to this.domNode)
	// 		Maps this.focus to this.focusNode.focus, or (last example) this.domNode.focus
	//
	//		- DOM node innerHTML
	//	|		_setTitleAttr: { node: "titleNode", type: "innerHTML" }
	//		Maps this.title to this.titleNode.innerHTML
	//
	//		- DOM node innerText
	//	|		_setTitleAttr: { node: "titleNode", type: "innerText" }
	//		Maps this.title to this.titleNode.innerText
	//
	//		- DOM node CSS class
	// |		_setMyClassAttr: { node: "domNode", type: "class" }
	//		Maps this.myClass to this.domNode.className
	//
	//		If the value of _setXXXAttr is an array, then each element in the array matches one of the
	//		formats of the above list.
	//
	//		If the custom setter is null, no action is performed other than saving the new value
	//		in the widget (in this).
	//
	//		If no custom setter is defined for an attribute, then it will be copied
	//		to this.focusNode (if the widget defines a focusNode), or this.domNode otherwise.
	//		That's only done though for attributes that match DOMNode attributes (title,
	//		alt, aria-labelledby, etc.)

	// id: [const] String
	//		A unique, opaque ID string that can be assigned by users or by the
	//		system. If the developer passes an ID which is known not to be
	//		unique, the specified ID is ignored and the system-generated ID is
	//		used instead.
	id: "",
	_setIdAttr: "domNode",	// to copy to this.domNode even for auto-generated id's

	// lang: [const] String
	//		Rarely used.  Overrides the default Dojo locale used to render this widget,
	//		as defined by the [HTML LANG](http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang) attribute.
	//		Value must be among the list of locales specified during by the Dojo bootstrap,
	//		formatted according to [RFC 3066](http://www.ietf.org/rfc/rfc3066.txt) (like en-us).
	lang: "",
	// set on domNode even when there's a focus node.   but don't set lang="", since that's invalid.
	_setLangAttr: nonEmptyAttrToDom("lang"),

	// dir: [const] String
	//		Bi-directional support, as defined by the [HTML DIR](http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir)
	//		attribute. Either left-to-right "ltr" or right-to-left "rtl".  If undefined, widgets renders in page's
	//		default direction.
	dir: "",
	// set on domNode even when there's a focus node.   but don't set dir="", since that's invalid.
	_setDirAttr: nonEmptyAttrToDom("dir"),	// to set on domNode even when there's a focus node

	// textDir: String
	//		Bi-directional support,	the main variable which is responsible for the direction of the text.
	//		The text direction can be different than the GUI direction by using this parameter in creation
	//		of a widget.
	// 		Allowed values:
	//			1. "ltr"
	//			2. "rtl"
	//			3. "auto" - contextual the direction of a text defined by first strong letter.
	//		By default is as the page direction.
	textDir: "",

	// class: String
	//		HTML class attribute
	"class": "",
	_setClassAttr: { node: "domNode", type: "class" },

	// style: String||Object
	//		HTML style attributes as cssText string or name/value hash
	style: "",

	// title: String
	//		HTML title attribute.
	//
	//		For form widgets this specifies a tooltip to display when hovering over
	//		the widget (just like the native HTML title attribute).
	//
	//		For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,
	//		etc., it's used to specify the tab label, accordion pane title, etc.
	title: "",

	// tooltip: String
	//		When this widget's title attribute is used to for a tab label, accordion pane title, etc.,
	//		this specifies the tooltip to appear when the mouse is hovered over that text.
	tooltip: "",

	// baseClass: [protected] String
	//		Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate
	//		widget state.
	baseClass: "",

	// srcNodeRef: [readonly] DomNode
	//		pointer to original DOM node
	srcNodeRef: null,

	// domNode: [readonly] DomNode
	//		This is our visible representation of the widget! Other DOM
	//		Nodes may by assigned to other properties, usually through the
	//		template system's data-dojo-attach-point syntax, but the domNode
	//		property is the canonical "top level" node in widget UI.
	domNode: null,

	// containerNode: [readonly] DomNode
	//		Designates where children of the source DOM node will be placed.
	//		"Children" in this case refers to both DOM nodes and widgets.
	//		For example, for myWidget:
	//
	//		|	<div data-dojo-type=myWidget>
	//		|		<b> here's a plain DOM node
	//		|		<span data-dojo-type=subWidget>and a widget</span>
	//		|		<i> and another plain DOM node </i>
	//		|	</div>
	//
	//		containerNode would point to:
	//
	//		|		<b> here's a plain DOM node
	//		|		<span data-dojo-type=subWidget>and a widget</span>
	//		|		<i> and another plain DOM node </i>
	//
	//		In templated widgets, "containerNode" is set via a
	//		data-dojo-attach-point assignment.
	//
	//		containerNode must be defined for any widget that accepts innerHTML
	//		(like ContentPane or BorderContainer or even Button), and conversely
	//		is null for widgets that don't, like TextBox.
	containerNode: null,

/*=====
	// _started: Boolean
	//		startup() has completed.
	_started: false,
=====*/

	// attributeMap: [protected] Object
	//		Deprecated.   Instead of attributeMap, widget should have a _setXXXAttr attribute
	//		for each XXX attribute to be mapped to the DOM.
	//
	//		attributeMap sets up a "binding" between attributes (aka properties)
	//		of the widget and the widget's DOM.
	//		Changes to widget attributes listed in attributeMap will be
	//		reflected into the DOM.
	//
	//		For example, calling set('title', 'hello')
	//		on a TitlePane will automatically cause the TitlePane's DOM to update
	//		with the new title.
	//
	//		attributeMap is a hash where the key is an attribute of the widget,
	//		and the value reflects a binding to a:
	//
	//		- DOM node attribute
	// |		focus: {node: "focusNode", type: "attribute"}
	// 		Maps this.focus to this.focusNode.focus
	//
	//		- DOM node innerHTML
	//	|		title: { node: "titleNode", type: "innerHTML" }
	//		Maps this.title to this.titleNode.innerHTML
	//
	//		- DOM node innerText
	//	|		title: { node: "titleNode", type: "innerText" }
	//		Maps this.title to this.titleNode.innerText
	//
	//		- DOM node CSS class
	// |		myClass: { node: "domNode", type: "class" }
	//		Maps this.myClass to this.domNode.className
	//
	//		If the value is an array, then each element in the array matches one of the
	//		formats of the above list.
	//
	//		There are also some shorthands for backwards compatibility:
	//		- string --> { node: string, type: "attribute" }, for example:
	//	|	"focusNode" ---> { node: "focusNode", type: "attribute" }
	//		- "" --> { node: "domNode", type: "attribute" }
	attributeMap: {},

	// _blankGif: [protected] String
	//		Path to a blank 1x1 image.
	//		Used by <img> nodes in templates that really get their image via CSS background-image.
	_blankGif: config.blankGif || require.toUrl("dojo/resources/blank.gif"),

	//////////// INITIALIZATION METHODS ///////////////////////////////////////

	postscript: function(/*Object?*/params, /*DomNode|String*/srcNodeRef){
		// summary:
		//		Kicks off widget instantiation.  See create() for details.
		// tags:
		//		private
		this.create(params, srcNodeRef);
	},

	create: function(/*Object?*/params, /*DomNode|String?*/srcNodeRef){
		// summary:
		//		Kick off the life-cycle of a widget
		// params:
		//		Hash of initialization parameters for widget, including
		//		scalar values (like title, duration etc.) and functions,
		//		typically callbacks like onClick.
		// srcNodeRef:
		//		If a srcNodeRef (DOM node) is specified:
		//			- use srcNodeRef.innerHTML as my contents
		//			- if this is a behavioral widget then apply behavior
		//			  to that srcNodeRef
		//			- otherwise, replace srcNodeRef with my generated DOM
		//			  tree
		// description:
		//		Create calls a number of widget methods (postMixInProperties, buildRendering, postCreate,
		//		etc.), some of which of you'll want to override. See http://dojotoolkit.org/reference-guide/dijit/_WidgetBase.html
		//		for a discussion of the widget creation lifecycle.
		//
		//		Of course, adventurous developers could override create entirely, but this should
		//		only be done as a last resort.
		// tags:
		//		private

		// store pointer to original DOM tree
		this.srcNodeRef = dom.byId(srcNodeRef);

		// For garbage collection.  An array of listener handles returned by this.connect() / this.subscribe()
		this._connects = [];

		// For widgets internal to this widget, invisible to calling code
		this._supportingWidgets = [];

		// this is here for back-compat, remove in 2.0 (but check NodeList-instantiate.html test)
		if(this.srcNodeRef && (typeof this.srcNodeRef.id == "string")){ this.id = this.srcNodeRef.id; }

		// mix in our passed parameters
		if(params){
			this.params = params;
			lang.mixin(this, params);
		}
		this.postMixInProperties();

		// generate an id for the widget if one wasn't specified
		// (be sure to do this before buildRendering() because that function might
		// expect the id to be there.)
		if(!this.id){
			this.id = registry.getUniqueId(this.declaredClass.replace(/\./g,"_"));
		}
		registry.add(this);

		this.buildRendering();

		if(this.domNode){
			// Copy attributes listed in attributeMap into the [newly created] DOM for the widget.
			// Also calls custom setters for all attributes with custom setters.
			this._applyAttributes();

			// If srcNodeRef was specified, then swap out original srcNode for this widget's DOM tree.
			// For 2.0, move this after postCreate().  postCreate() shouldn't depend on the
			// widget being attached to the DOM since it isn't when a widget is created programmatically like
			// new MyWidget({}).   See #11635.
			var source = this.srcNodeRef;
			if(source && source.parentNode && this.domNode !== source){
				source.parentNode.replaceChild(this.domNode, source);
			}
		}

		if(this.domNode){
			// Note: for 2.0 may want to rename widgetId to dojo._scopeName + "_widgetId",
			// assuming that dojo._scopeName even exists in 2.0
			this.domNode.setAttribute("widgetId", this.id);
		}
		this.postCreate();

		// If srcNodeRef has been processed and removed from the DOM (e.g. TemplatedWidget) then delete it to allow GC.
		if(this.srcNodeRef && !this.srcNodeRef.parentNode){
			delete this.srcNodeRef;
		}

		this._created = true;
	},

	_applyAttributes: function(){
		// summary:
		//		Step during widget creation to copy  widget attributes to the
		//		DOM according to attributeMap and _setXXXAttr objects, and also to call
		//		custom _setXXXAttr() methods.
		//
		//		Skips over blank/false attribute values, unless they were explicitly specified
		//		as parameters to the widget, since those are the default anyway,
		//		and setting tabIndex="" is different than not setting tabIndex at all.
		//
		//		For backwards-compatibility reasons attributeMap overrides _setXXXAttr when
		//		_setXXXAttr is a hash/string/array, but _setXXXAttr as a functions override attributeMap.
		// tags:
		//		private

		// Get list of attributes where this.set(name, value) will do something beyond
		// setting this[name] = value.  Specifically, attributes that have:
		//		- associated _setXXXAttr() method/hash/string/array
		//		- entries in attributeMap.
		var ctor = this.constructor,
			list = ctor._setterAttrs;
		if(!list){
			list = (ctor._setterAttrs = []);
			for(var attr in this.attributeMap){
				list.push(attr);
			}

			var proto = ctor.prototype;
			for(var fxName in proto){
				if(fxName in this.attributeMap){ continue; }
				var setterName = "_set" + fxName.replace(/^[a-z]|-[a-zA-Z]/g, function(c){ return c.charAt(c.length-1).toUpperCase(); }) + "Attr";
				if(setterName in proto){
					list.push(fxName);
				}
			}
		}

		// Call this.set() for each attribute that was either specified as parameter to constructor,
		// or was found above and has a default non-null value.   For correlated attributes like value and displayedValue, the one
		// specified as a parameter should take precedence, so apply attributes in this.params last.
		// Particularly important for new DateTextBox({displayedValue: ...}) since DateTextBox's default value is
		// NaN and thus is not ignored like a default value of "".
		array.forEach(list, function(attr){
			if(this.params && attr in this.params){
				// skip this one, do it below
			}else if(this[attr]){
				this.set(attr, this[attr]);
			}
		}, this);
		for(var param in this.params){
			this.set(param, this[param]);
		}
	},

	postMixInProperties: function(){
		// summary:
		//		Called after the parameters to the widget have been read-in,
		//		but before the widget template is instantiated. Especially
		//		useful to set properties that are referenced in the widget
		//		template.
		// tags:
		//		protected
	},

	buildRendering: function(){
		// summary:
		//		Construct the UI for this widget, setting this.domNode.
		//		Most widgets will mixin `dijit._TemplatedMixin`, which implements this method.
		// tags:
		//		protected

		if(!this.domNode){
			// Create root node if it wasn't created by _Templated
			this.domNode = this.srcNodeRef || domConstruct.create('div');
		}

		// baseClass is a single class name or occasionally a space-separated list of names.
		// Add those classes to the DOMNode.  If RTL mode then also add with Rtl suffix.
		// TODO: make baseClass custom setter
		if(this.baseClass){
			var classes = this.baseClass.split(" ");
			if(!this.isLeftToRight()){
				classes = classes.concat( array.map(classes, function(name){ return name+"Rtl"; }));
			}
			domClass.add(this.domNode, classes);
		}
	},

	postCreate: function(){
		// summary:
		//		Processing after the DOM fragment is created
		// description:
		//		Called after the DOM fragment has been created, but not necessarily
		//		added to the document.  Do not include any operations which rely on
		//		node dimensions or placement.
		// tags:
		//		protected
	},

	startup: function(){
		// summary:
		//		Processing after the DOM fragment is added to the document
		// description:
		//		Called after a widget and its children have been created and added to the page,
		//		and all related widgets have finished their create() cycle, up through postCreate().
		//		This is useful for composite widgets that need to control or layout sub-widgets.
		//		Many layout widgets can use this as a wiring phase.
		if(this._started){ return; }
		this._started = true;
		array.forEach(this.getChildren(), function(obj){
			if(!obj._started && !obj._destroyed && lang.isFunction(obj.startup)){
				obj.startup();
				obj._started = true;
			}
		});
	},

	//////////// DESTROY FUNCTIONS ////////////////////////////////

	destroyRecursive: function(/*Boolean?*/ preserveDom){
		// summary:
		// 		Destroy this widget and its descendants
		// description:
		//		This is the generic "destructor" function that all widget users
		// 		should call to cleanly discard with a widget. Once a widget is
		// 		destroyed, it is removed from the manager object.
		// preserveDom:
		//		If true, this method will leave the original DOM structure
		//		alone of descendant Widgets. Note: This will NOT work with
		//		dijit._Templated widgets.

		this._beingDestroyed = true;
		this.destroyDescendants(preserveDom);
		this.destroy(preserveDom);
	},

	destroy: function(/*Boolean*/ preserveDom){
		// summary:
		// 		Destroy this widget, but not its descendants.
		//		This method will, however, destroy internal widgets such as those used within a template.
		// preserveDom: Boolean
		//		If true, this method will leave the original DOM structure alone.
		//		Note: This will not yet work with _Templated widgets

		this._beingDestroyed = true;
		this.uninitialize();

		// remove this.connect() and this.subscribe() listeners
		var c;
		while((c = this._connects.pop())){
			c.remove();
		}

		// destroy widgets created as part of template, etc.
		var w;
		while((w = this._supportingWidgets.pop())){
			if(w.destroyRecursive){
				w.destroyRecursive();
			}else if(w.destroy){
				w.destroy();
			}
		}

		this.destroyRendering(preserveDom);
		registry.remove(this.id);
		this._destroyed = true;
	},

	destroyRendering: function(/*Boolean?*/ preserveDom){
		// summary:
		//		Destroys the DOM nodes associated with this widget
		// preserveDom:
		//		If true, this method will leave the original DOM structure alone
		//		during tear-down. Note: this will not work with _Templated
		//		widgets yet.
		// tags:
		//		protected

		if(this.bgIframe){
			this.bgIframe.destroy(preserveDom);
			delete this.bgIframe;
		}

		if(this.domNode){
			if(preserveDom){
				domAttr.remove(this.domNode, "widgetId");
			}else{
				domConstruct.destroy(this.domNode);
			}
			delete this.domNode;
		}

		if(this.srcNodeRef){
			if(!preserveDom){
				domConstruct.destroy(this.srcNodeRef);
			}
			delete this.srcNodeRef;
		}
	},

	destroyDescendants: function(/*Boolean?*/ preserveDom){
		// summary:
		//		Recursively destroy the children of this widget and their
		//		descendants.
		// preserveDom:
		//		If true, the preserveDom attribute is passed to all descendant
		//		widget's .destroy() method. Not for use with _Templated
		//		widgets.

		// get all direct descendants and destroy them recursively
		array.forEach(this.getChildren(), function(widget){
			if(widget.destroyRecursive){
				widget.destroyRecursive(preserveDom);
			}
		});
	},

	uninitialize: function(){
		// summary:
		//		Stub function. Override to implement custom widget tear-down
		//		behavior.
		// tags:
		//		protected
		return false;
	},

	////////////////// GET/SET, CUSTOM SETTERS, ETC. ///////////////////

	_setStyleAttr: function(/*String||Object*/ value){
		// summary:
		//		Sets the style attribute of the widget according to value,
		//		which is either a hash like {height: "5px", width: "3px"}
		//		or a plain string
		// description:
		//		Determines which node to set the style on based on style setting
		//		in attributeMap.
		// tags:
		//		protected

		var mapNode = this.domNode;

		// Note: technically we should revert any style setting made in a previous call
		// to his method, but that's difficult to keep track of.

		if(lang.isObject(value)){
			domStyle.set(mapNode, value);
		}else{
			if(mapNode.style.cssText){
				mapNode.style.cssText += "; " + value;
			}else{
				mapNode.style.cssText = value;
			}
		}

		this._set("style", value);
	},

	_attrToDom: function(/*String*/ attr, /*String*/ value, /*Object?*/ commands){
		// summary:
		//		Reflect a widget attribute (title, tabIndex, duration etc.) to
		//		the widget DOM, as specified by commands parameter.
		//		If commands isn't specified then it's looked up from attributeMap.
		//		Note some attributes like "type"
		//		cannot be processed this way as they are not mutable.
		//
		// tags:
		//		private

		commands = arguments.length >= 3 ? commands : this.attributeMap[attr];

		array.forEach(lang.isArray(commands) ? commands : [commands], function(command){

			// Get target node and what we are doing to that node
			var mapNode = this[command.node || command || "domNode"];	// DOM node
			var type = command.type || "attribute";	// class, innerHTML, innerText, or attribute

			switch(type){
				case "attribute":
					if(lang.isFunction(value)){ // functions execute in the context of the widget
						value = lang.hitch(this, value);
					}

					// Get the name of the DOM node attribute; usually it's the same
					// as the name of the attribute in the widget (attr), but can be overridden.
					// Also maps handler names to lowercase, like onSubmit --> onsubmit
					var attrName = command.attribute ? command.attribute :
						(/^on[A-Z][a-zA-Z]*$/.test(attr) ? attr.toLowerCase() : attr);

					domAttr.set(mapNode, attrName, value);
					break;
				case "innerText":
					mapNode.innerHTML = "";
					mapNode.appendChild(win.doc.createTextNode(value));
					break;
				case "innerHTML":
					mapNode.innerHTML = value;
					break;
				case "class":
					domClass.replace(mapNode, value, this[attr]);
					break;
			}
		}, this);
	},

	get: function(name){
		// summary:
		//		Get a property from a widget.
		//	name:
		//		The property to get.
		// description:
		//		Get a named property from a widget. The property may
		//		potentially be retrieved via a getter method. If no getter is defined, this
		// 		just retrieves the object's property.
		//
		// 		For example, if the widget has properties `foo` and `bar`
		//		and a method named `_getFooAttr()`, calling:
		//		`myWidget.get("foo")` would be equivalent to calling
		//		`widget._getFooAttr()` and `myWidget.get("bar")`
		//		would be equivalent to the expression
		//		`widget.bar2`
		var names = this._getAttrNames(name);
		return this[names.g] ? this[names.g]() : this[name];
	},

	set: function(name, value){
		// summary:
		//		Set a property on a widget
		//	name:
		//		The property to set.
		//	value:
		//		The value to set in the property.
		// description:
		//		Sets named properties on a widget which may potentially be handled by a
		// 		setter in the widget.
		//
		// 		For example, if the widget has properties `foo` and `bar`
		//		and a method named `_setFooAttr()`, calling
		//		`myWidget.set("foo", "Howdy!")` would be equivalent to calling
		//		`widget._setFooAttr("Howdy!")` and `myWidget.set("bar", 3)`
		//		would be equivalent to the statement `widget.bar = 3;`
		//
		//		set() may also be called with a hash of name/value pairs, ex:
		//
		//	|	myWidget.set({
		//	|		foo: "Howdy",
		//	|		bar: 3
		//	|	});
		//
		//	This is equivalent to calling `set(foo, "Howdy")` and `set(bar, 3)`

		if(typeof name === "object"){
			for(var x in name){
				this.set(x, name[x]);
			}
			return this;
		}
		var names = this._getAttrNames(name),
			setter = this[names.s];
		if(lang.isFunction(setter)){
			// use the explicit setter
			var result = setter.apply(this, Array.prototype.slice.call(arguments, 1));
		}else{
			// Mapping from widget attribute to DOMNode attribute/value/etc.
			// Map according to:
			//		1. attributeMap setting, if one exists (TODO: attributeMap deprecated, remove in 2.0)
			//		2. _setFooAttr: {...} type attribute in the widget (if one exists)
			//		3. apply to focusNode or domNode if standard attribute name, excluding funcs like onClick.
			// Checks if an attribute is a "standard attribute" by whether the DOMNode JS object has a similar
			// attribute name (ex: accept-charset attribute matches jsObject.acceptCharset).
			// Note also that Tree.focusNode() is a function not a DOMNode, so test for that.
			var defaultNode = this.focusNode && !lang.isFunction(this.focusNode) ? "focusNode" : "domNode",
				tag = this[defaultNode].tagName,
				attrsForTag = tagAttrs[tag] || (tagAttrs[tag] = getAttrs(this[defaultNode])),
				map =	name in this.attributeMap ? this.attributeMap[name] :
						names.s in this ? this[names.s] :
						((names.l in attrsForTag && typeof value != "function") ||
							/^aria-|^data-|^role$/.test(name)) ? defaultNode : null;
			if(map != null){
				this._attrToDom(name, value, map);
			}
			this._set(name, value);
		}
		return result || this;
	},

	_attrPairNames: {},		// shared between all widgets
	_getAttrNames: function(name){
		// summary:
		//		Helper function for get() and set().
		//		Caches attribute name values so we don't do the string ops every time.
		// tags:
		//		private

		var apn = this._attrPairNames;
		if(apn[name]){ return apn[name]; }
		var uc = name.replace(/^[a-z]|-[a-zA-Z]/g, function(c){ return c.charAt(c.length-1).toUpperCase(); });
		return (apn[name] = {
			n: name+"Node",
			s: "_set"+uc+"Attr",	// converts dashes to camel case, ex: accept-charset --> _setAcceptCharsetAttr
			g: "_get"+uc+"Attr",
			l: uc.toLowerCase()		// lowercase name w/out dashes, ex: acceptcharset
		});
	},

	_set: function(/*String*/ name, /*anything*/ value){
		// summary:
		//		Helper function to set new value for specified attribute, and call handlers
		//		registered with watch() if the value has changed.
		var oldValue = this[name];
		this[name] = value;
		if(this._watchCallbacks && this._created && value !== oldValue){
			this._watchCallbacks(name, oldValue, value);
		}
	},

	on: function(/*String*/ type, /*Function*/ func){
		// summary:
		//		Call specified function when event occurs, ex: myWidget.on("click", function(){ ... }).
		// description:
		//		Call specified function when event `type` occurs, ex: `myWidget.on("click", function(){ ... })`.
		//		Note that the function is not run in any particular scope, so if (for example) you want it to run in the
		//		widget's scope you must do `myWidget.on("click", lang.hitch(myWidget, func))`.

		return aspect.after(this, this._onMap(type), func, true);
	},

	_onMap: function(/*String*/ type){
		// summary:
		//		Maps on() type parameter (ex: "mousemove") to method name (ex: "onMouseMove")
		var ctor = this.constructor, map = ctor._onMap;
		if(!map){
			map = (ctor._onMap = {});
			for(var attr in ctor.prototype){
				if(/^on/.test(attr)){
					map[attr.replace(/^on/, "").toLowerCase()] = attr;
				}
			}
		}
		return map[type.toLowerCase()];	// String
	},

	toString: function(){
		// summary:
		//		Returns a string that represents the widget
		// description:
		//		When a widget is cast to a string, this method will be used to generate the
		//		output. Currently, it does not implement any sort of reversible
		//		serialization.
		return '[Widget ' + this.declaredClass + ', ' + (this.id || 'NO ID') + ']'; // String
	},

	getChildren: function(){
		// summary:
		//		Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.
		//		Does not return nested widgets, nor widgets that are part of this widget's template.
		return this.containerNode ? registry.findWidgets(this.containerNode) : []; // dijit._Widget[]
	},

	getParent: function(){
		// summary:
		//		Returns the parent widget of this widget
		return registry.getEnclosingWidget(this.domNode.parentNode);
	},

	connect: function(
			/*Object|null*/ obj,
			/*String|Function*/ event,
			/*String|Function*/ method){
		// summary:
		//		Connects specified obj/event to specified method of this object
		//		and registers for disconnect() on widget destroy.
		// description:
		//		Provide widget-specific analog to dojo.connect, except with the
		//		implicit use of this widget as the target object.
		//		Events connected with `this.connect` are disconnected upon
		//		destruction.
		// returns:
		//		A handle that can be passed to `disconnect` in order to disconnect before
		//		the widget is destroyed.
		// example:
		//	|	var btn = new dijit.form.Button();
		//	|	// when foo.bar() is called, call the listener we're going to
		//	|	// provide in the scope of btn
		//	|	btn.connect(foo, "bar", function(){
		//	|		console.debug(this.toString());
		//	|	});
		// tags:
		//		protected

		var handle = connect.connect(obj, event, this, method);
		this._connects.push(handle);
		return handle;		// _Widget.Handle
	},

	disconnect: function(handle){
		// summary:
		//		Disconnects handle created by `connect`.
		//		Also removes handle from this widget's list of connects.
		// tags:
		//		protected
		var i = array.indexOf(this._connects, handle);
		if(i != -1){
			handle.remove();
			this._connects.splice(i, 1);
		}
	},

	subscribe: function(t, method){
		// summary:
		//		Subscribes to the specified topic and calls the specified method
		//		of this object and registers for unsubscribe() on widget destroy.
		// description:
		//		Provide widget-specific analog to dojo.subscribe, except with the
		//		implicit use of this widget as the target object.
		// t: String
		//		The topic
		// method: Function
		//		The callback
		// example:
		//	|	var btn = new dijit.form.Button();
		//	|	// when /my/topic is published, this button changes its label to
		//	|   // be the parameter of the topic.
		//	|	btn.subscribe("/my/topic", function(v){
		//	|		this.set("label", v);
		//	|	});
		// tags:
		//		protected
		var handle = topic.subscribe(t, lang.hitch(this, method));
		this._connects.push(handle);
		return handle;		// _Widget.Handle
	},

	unsubscribe: function(/*Object*/ handle){
		// summary:
		//		Unsubscribes handle created by this.subscribe.
		//		Also removes handle from this widget's list of subscriptions
		// tags:
		//		protected
		this.disconnect(handle);
	},

	isLeftToRight: function(){
		// summary:
		//		Return this widget's explicit or implicit orientation (true for LTR, false for RTL)
		// tags:
		//		protected
		return this.dir ? (this.dir == "ltr") : domGeometry.isBodyLtr(); //Boolean
	},

	isFocusable: function(){
		// summary:
		//		Return true if this widget can currently be focused
		//		and false if not
		return this.focus && (domStyle.get(this.domNode, "display") != "none");
	},

	placeAt: function(/* String|DomNode|_Widget */reference, /* String?|Int? */position){
		// summary:
		//		Place this widget's domNode reference somewhere in the DOM based
		//		on standard domConstruct.place conventions, or passing a Widget reference that
		//		contains and addChild member.
		//
		// description:
		//		A convenience function provided in all _Widgets, providing a simple
		//		shorthand mechanism to put an existing (or newly created) Widget
		//		somewhere in the dom, and allow chaining.
		//
		// reference:
		//		The String id of a domNode, a domNode reference, or a reference to a Widget possessing
		//		an addChild method.
		//
		// position:
		//		If passed a string or domNode reference, the position argument
		//		accepts a string just as domConstruct.place does, one of: "first", "last",
		//		"before", or "after".
		//
		//		If passed a _Widget reference, and that widget reference has an ".addChild" method,
		//		it will be called passing this widget instance into that method, supplying the optional
		//		position index passed.
		//
		// returns:
		//		dijit._Widget
		//		Provides a useful return of the newly created dijit._Widget instance so you
		//		can "chain" this function by instantiating, placing, then saving the return value
		//		to a variable.
		//
		// example:
		// | 	// create a Button with no srcNodeRef, and place it in the body:
		// | 	var button = new dijit.form.Button({ label:"click" }).placeAt(win.body());
		// | 	// now, 'button' is still the widget reference to the newly created button
		// | 	button.on("click", function(e){ console.log('click'); }));
		//
		// example:
		// |	// create a button out of a node with id="src" and append it to id="wrapper":
		// | 	var button = new dijit.form.Button({},"src").placeAt("wrapper");
		//
		// example:
		// |	// place a new button as the first element of some div
		// |	var button = new dijit.form.Button({ label:"click" }).placeAt("wrapper","first");
		//
		// example:
		// |	// create a contentpane and add it to a TabContainer
		// |	var tc = dijit.byId("myTabs");
		// |	new dijit.layout.ContentPane({ href:"foo.html", title:"Wow!" }).placeAt(tc)

		if(reference.declaredClass && reference.addChild){
			reference.addChild(this, position);
		}else{
			domConstruct.place(this.domNode, reference, position);
		}
		return this;
	},

	getTextDir: function(/*String*/ text,/*String*/ originalDir){
		// summary:
		//		Return direction of the text.
		//		The function overridden in the _BidiSupport module,
		//		its main purpose is to calculate the direction of the
		//		text, if was defined by the programmer through textDir.
		//	tags:
		//		protected.
		return originalDir;
	},

	applyTextDir: function(/*===== element, text =====*/){
		// summary:
		//		The function overridden in the _BidiSupport module,
		//		originally used for setting element.dir according to this.textDir.
		//		In this case does nothing.
		// element: DOMNode
		// text: String
		// tags:
		//		protected.
	},

	defer: function(fcn, delay){ 
		// summary:
		//		Wrapper to setTimeout to avoid deferred functions executing
		//		after the originating widget has been destroyed.
		//		Returns an object handle with a remove method (that returns null) (replaces clearTimeout).
		// fcn: function reference
		// delay: Optional number (defaults to 0)
		// tags:
		//		protected.
		var timer = setTimeout(lang.hitch(this, 
			function(){ 
				timer = null;
				if(!this._destroyed){ 
					lang.hitch(this, fcn)(); 
				} 
			}),
			delay || 0
		);
		return {
			remove:	function(){
					if(timer){
						clearTimeout(timer);
						timer = null;
					}
					return null; // so this works well: handle = handle.remove();
				}
		};
	}
});

});

},
'gridx/modules/AutoScroll':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/window",
	"dojo/dom-geometry",
	"../core/_Module"
], function(declare, win, domGeometry, _Module){

	return _Module.register(
	declare(_Module, {

		name: 'autoScroll',

		constructor: function(){
			this.connect(win.doc, 'mousemove', '_onMouseMove');
		},

		getAPIPath: function(){
			return {
				autoScroll: this
			};
		},
	
		//Public ---------------------------------------------------------------------
		enabled: false,

		vertical: true,

		horizontal: true,

		margin: 20,

		//Private ---------------------------------------------------------------------

		_timeout: 100,

		_step: 10,

		_maxMargin: 100,

		_onMouseMove: function(e){
			var t = this;
			if(t.arg('enabled')){
				var d1, d2, g = t.grid, m = t.arg('margin'), 
					pos = domGeometry.position(g.bodyNode);
				if(t.arg('vertical') && g.vScroller){
					d1 = e.clientY - pos.y - m;
					d2 = d1 + 2 * m - pos.h;
					t._vdir = d1 < 0 ? d1 : (d2 > 0 ? d2 : 0);
				}
				if(t.arg('horizontal') && g.hScroller){
					d1 = e.clientX - pos.x - m;
					d2 = d1 + 2 * m - pos.w;
					t._hdir = d1 < 0 ? d1 : (d2 > 0 ? d2 : 0);
				}
				if(!t._handler){
					t._scroll();
				}
			}
		},

		_scroll: function(){
			var t = this;
			if(t.arg('enabled')){
				var dir, a, needScroll, g = t.grid,
					m = t._maxMargin, s = t._step,
					v = t._vdir, h = t._hdir;
				if(t.arg('vertical') && v){
					dir = v > 0 ? 1 : -1;
					a = Math.min(m, Math.abs(v)) / s;
					a = (a < 1 ? 1 : a) * s * dir;
					g.vScroller.domNode.scrollTop += a;
					needScroll = 1;
				}
				if(t.arg('horizontal') && h){
					dir = h > 0 ? 1 : -1;
					a = Math.min(m, Math.abs(h)) / s;
					a = (a < 1 ? 1 : a) * s * dir;
					g.hScroller.domNode.scrollLeft += a;
					needScroll = 1;
				}
				if(needScroll){
					t._handler = setTimeout(function(){
						t._scroll();
					}, t._timeout);
					return;
				}
			}
			delete t._handler;
		}
	}));
});

},
'ibm/tivoli/tip/dijit/BusyPane':function(){
define("ibm/tivoli/tip/dijit/BusyPane", ["dojo/_base/declare","dojo", "dijit", "dojoe/progressbar/EnhancedProgressBar"], function(declare,dojo, dijit, epb) {

console.warn("'ibm.tivoli.tip.dijit.BusyPane' is deprecated, use 'dojoe.progressbar.EnhancedProgressBar'(TED) instead.");
console.log("redirecting to dojoe.progressbar.EnhancedProgressBar");

	return declare("ibm.tivoli.tip.dijit.BusyPane", [epb], {
	
		postCreate: function()
		{
			/*override: to set portlet frame/document body as the target
			 * New EPB impl needs a target node to overlay which can be any dom node */
		  this.inherited( arguments );
		  var portletBodyElem = null;
		  var cNode = this.domNode.parentNode;
		  while(cNode ){
		  		if(cNode == dojo.body()){
					portletBodyElem = cNode;
		  			break;
		  		}
		  		cNode = cNode.parentNode;
		  }
		
		  if(portletBodyElem)this.target = cNode;
		}
	
	});

});

},
'dijit/popup':function(){
define([
	"dojo/_base/array", // array.forEach array.some
	"dojo/aspect",
	"dojo/_base/connect",	// connect._keypress
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.isDescendant
	"dojo/dom-attr", // domAttr.set
	"dojo/dom-construct", // domConstruct.create domConstruct.destroy
	"dojo/dom-geometry", // domGeometry.isBodyLtr
	"dojo/dom-style", // domStyle.set
	"dojo/_base/event", // event.stop
	"dojo/keys",
	"dojo/_base/lang", // lang.hitch
	"dojo/on",
	"dojo/_base/sniff", // has("ie") has("mozilla")
	"dojo/_base/window", // win.body
	"./place",
	"./BackgroundIframe",
	"."	// dijit (defining dijit.popup to match API doc)
], function(array, aspect, connect, declare, dom, domAttr, domConstruct, domGeometry, domStyle, event, keys, lang, on, has, win,
			place, BackgroundIframe, dijit){

	// module:
	//		dijit/popup
	// summary:
	//		Used to show drop downs (ex: the select list of a ComboBox)
	//		or popups (ex: right-click context menus)


	/*=====
	dijit.popup.__OpenArgs = function(){
		// popup: Widget
		//		widget to display
		// parent: Widget
		//		the button etc. that is displaying this popup
		// around: DomNode
		//		DOM node (typically a button); place popup relative to this node.  (Specify this *or* "x" and "y" parameters.)
		// x: Integer
		//		Absolute horizontal position (in pixels) to place node at.  (Specify this *or* "around" parameter.)
		// y: Integer
		//		Absolute vertical position (in pixels) to place node at.  (Specify this *or* "around" parameter.)
		// orient: Object|String
		//		When the around parameter is specified, orient should be a list of positions to try, ex:
		//	|	[ "below", "above" ]
		//		For backwards compatibility it can also be an (ordered) hash of tuples of the form
		//		(around-node-corner, popup-node-corner), ex:
		//	|	{ "BL": "TL", "TL": "BL" }
		//		where BL means "bottom left" and "TL" means "top left", etc.
		//
		//		dijit.popup.open() tries to position the popup according to each specified position, in order,
		//		until the popup appears fully within the viewport.
		//
		//		The default value is ["below", "above"]
		//
		//		When an (x,y) position is specified rather than an around node, orient is either
		//		"R" or "L".  R (for right) means that it tries to put the popup to the right of the mouse,
		//		specifically positioning the popup's top-right corner at the mouse position, and if that doesn't
		//		fit in the viewport, then it tries, in order, the bottom-right corner, the top left corner,
		//		and the top-right corner.
		// onCancel: Function
		//		callback when user has canceled the popup by
		//			1. hitting ESC or
		//			2. by using the popup widget's proprietary cancel mechanism (like a cancel button in a dialog);
		//			   i.e. whenever popupWidget.onCancel() is called, args.onCancel is called
		// onClose: Function
		//		callback whenever this popup is closed
		// onExecute: Function
		//		callback when user "executed" on the popup/sub-popup by selecting a menu choice, etc. (top menu only)
		// padding: dijit.__Position
		//		adding a buffer around the opening position. This is only useful when around is not set.
		this.popup = popup;
		this.parent = parent;
		this.around = around;
		this.x = x;
		this.y = y;
		this.orient = orient;
		this.onCancel = onCancel;
		this.onClose = onClose;
		this.onExecute = onExecute;
		this.padding = padding;
	}
	=====*/

	/*=====
	dijit.popup = {
		// summary:
		//		Used to show drop downs (ex: the select list of a ComboBox)
		//		or popups (ex: right-click context menus).
		//
		//		Access via require(["dijit/popup"], function(popup){ ... }).

		moveOffScreen: function(widget){
			// summary:
			//		Moves the popup widget off-screen.
			//		Do not use this method to hide popups when not in use, because
			//		that will create an accessibility issue: the offscreen popup is
			//		still in the tabbing order.
			// widget: dijit._WidgetBase
			//		The widget
		},

		hide: function(widget){
			// summary:
			//		Hide this popup widget (until it is ready to be shown).
			//		Initialization for widgets that will be used as popups
			//
			// 		Also puts widget inside a wrapper DIV (if not already in one)
			//
			//		If popup widget needs to layout it should
			//		do so when it is made visible, and popup._onShow() is called.
			// widget: dijit._WidgetBase
			//		The widget
		},

		open: function(args){
			// summary:
			//		Popup the widget at the specified position
			// example:
			//		opening at the mouse position
			//		|		popup.open({popup: menuWidget, x: evt.pageX, y: evt.pageY});
			// example:
			//		opening the widget as a dropdown
			//		|		popup.open({parent: this, popup: menuWidget, around: this.domNode, onClose: function(){...}});
			//
			//		Note that whatever widget called dijit.popup.open() should also listen to its own _onBlur callback
			//		(fired from _base/focus.js) to know that focus has moved somewhere else and thus the popup should be closed.
			// args: dijit.popup.__OpenArgs
			//		Parameters
			return {};	// Object specifying which position was chosen
		},

		close: function(popup){
			// summary:
			//		Close specified popup and any popups that it parented.
			//		If no popup is specified, closes all popups.
			// widget: dijit._WidgetBase?
			//		The widget, optional
		}
	};
	=====*/

	function destroyWrapper(){
		// summary:
		//		Function to destroy wrapper when popup widget is destroyed.
		//		Left in this scope to avoid memory leak on IE8 on refresh page, see #15206.
		if(this._popupWrapper){
			domConstruct.destroy(this._popupWrapper);
			delete this._popupWrapper;
		}
	}

	var PopupManager = declare(null, {
		// _stack: dijit._Widget[]
		//		Stack of currently popped up widgets.
		//		(someone opened _stack[0], and then it opened _stack[1], etc.)
		_stack: [],

		// _beginZIndex: Number
		//		Z-index of the first popup.   (If first popup opens other
		//		popups they get a higher z-index.)
		_beginZIndex: 1000,

		_idGen: 1,

		_createWrapper: function(/*Widget*/ widget){
			// summary:
			//		Initialization for widgets that will be used as popups.
			//		Puts widget inside a wrapper DIV (if not already in one),
			//		and returns pointer to that wrapper DIV.

			var wrapper = widget._popupWrapper,
				node = widget.domNode;

			if(!wrapper){
				// Create wrapper <div> for when this widget [in the future] will be used as a popup.
				// This is done early because of IE bugs where creating/moving DOM nodes causes focus
				// to go wonky, see tests/robot/Toolbar.html to reproduce
				wrapper = domConstruct.create("div", {
					"class":"dijitPopup",
					style:{ display: "none"},
					role: "presentation"
				}, win.body());
				wrapper.appendChild(node);

				var s = node.style;
				s.display = "";
				s.visibility = "";
				s.position = "";
				s.top = "0px";

				widget._popupWrapper = wrapper;
				aspect.after(widget, "destroy", destroyWrapper, true);
			}

			return wrapper;
		},

		moveOffScreen: function(/*Widget*/ widget){
			// summary:
			//		Moves the popup widget off-screen.
			//		Do not use this method to hide popups when not in use, because
			//		that will create an accessibility issue: the offscreen popup is
			//		still in the tabbing order.

			// Create wrapper if not already there
			var wrapper = this._createWrapper(widget);

			domStyle.set(wrapper, {
				visibility: "hidden",
				top: "-9999px",		// prevent transient scrollbar causing misalign (#5776), and initial flash in upper left (#10111)
				display: ""
			});
		},

		hide: function(/*Widget*/ widget){
			// summary:
			//		Hide this popup widget (until it is ready to be shown).
			//		Initialization for widgets that will be used as popups
			//
			// 		Also puts widget inside a wrapper DIV (if not already in one)
			//
			//		If popup widget needs to layout it should
			//		do so when it is made visible, and popup._onShow() is called.

			// Create wrapper if not already there
			var wrapper = this._createWrapper(widget);

			domStyle.set(wrapper, "display", "none");
		},

		getTopPopup: function(){
			// summary:
			//		Compute the closest ancestor popup that's *not* a child of another popup.
			//		Ex: For a TooltipDialog with a button that spawns a tree of menus, find the popup of the button.
			var stack = this._stack;
			for(var pi=stack.length-1; pi > 0 && stack[pi].parent === stack[pi-1].widget; pi--){
				/* do nothing, just trying to get right value for pi */
			}
			return stack[pi];
		},

		open: function(/*dijit.popup.__OpenArgs*/ args){
			// summary:
			//		Popup the widget at the specified position
			//
			// example:
			//		opening at the mouse position
			//		|		popup.open({popup: menuWidget, x: evt.pageX, y: evt.pageY});
			//
			// example:
			//		opening the widget as a dropdown
			//		|		popup.open({parent: this, popup: menuWidget, around: this.domNode, onClose: function(){...}});
			//
			//		Note that whatever widget called dijit.popup.open() should also listen to its own _onBlur callback
			//		(fired from _base/focus.js) to know that focus has moved somewhere else and thus the popup should be closed.

			var stack = this._stack,
				widget = args.popup,
				orient = args.orient || ["below", "below-alt", "above", "above-alt"],
				ltr = args.parent ? args.parent.isLeftToRight() : domGeometry.isBodyLtr(),
				around = args.around,
				id = (args.around && args.around.id) ? (args.around.id+"_dropdown") : ("popup_"+this._idGen++);

			// If we are opening a new popup that isn't a child of a currently opened popup, then
			// close currently opened popup(s).   This should happen automatically when the old popups
			// gets the _onBlur() event, except that the _onBlur() event isn't reliable on IE, see [22198].
			while(stack.length && (!args.parent || !dom.isDescendant(args.parent.domNode, stack[stack.length-1].widget.domNode))){
				this.close(stack[stack.length-1].widget);
			}

			// Get pointer to popup wrapper, and create wrapper if it doesn't exist
			var wrapper = this._createWrapper(widget);


			domAttr.set(wrapper, {
				id: id,
				style: {
					zIndex: this._beginZIndex + stack.length
				},
				"class": "dijitPopup " + (widget.baseClass || widget["class"] || "").split(" ")[0] +"Popup",
				dijitPopupParent: args.parent ? args.parent.id : ""
			});

			if(has("ie") || has("mozilla")){
				if(!widget.bgIframe){
					// setting widget.bgIframe triggers cleanup in _Widget.destroy()
					widget.bgIframe = new BackgroundIframe(wrapper);
				}
			}

			// position the wrapper node and make it visible
			var best = around ?
				place.around(wrapper, around, orient, ltr, widget.orient ? lang.hitch(widget, "orient") : null) :
				place.at(wrapper, args, orient == 'R' ? ['TR','BR','TL','BL'] : ['TL','BL','TR','BR'], args.padding);

			wrapper.style.display = "";
			wrapper.style.visibility = "visible";
			widget.domNode.style.visibility = "visible";	// counteract effects from _HasDropDown

			var handlers = [];

			// provide default escape and tab key handling
			// (this will work for any widget, not just menu)
			handlers.push(on(wrapper, connect._keypress, lang.hitch(this, function(evt){
				if(evt.charOrCode == keys.ESCAPE && args.onCancel){
					event.stop(evt);
					args.onCancel();
				}else if(evt.charOrCode === keys.TAB){
					event.stop(evt);
					var topPopup = this.getTopPopup();
					if(topPopup && topPopup.onCancel){
						topPopup.onCancel();
					}
				}
			})));

			// watch for cancel/execute events on the popup and notify the caller
			// (for a menu, "execute" means clicking an item)
			if(widget.onCancel && args.onCancel){
				handlers.push(widget.on("cancel", args.onCancel));
			}

			handlers.push(widget.on(widget.onExecute ? "execute" : "change", lang.hitch(this, function(){
				var topPopup = this.getTopPopup();
				if(topPopup && topPopup.onExecute){
					topPopup.onExecute();
				}
			})));

			stack.push({
				widget: widget,
				parent: args.parent,
				onExecute: args.onExecute,
				onCancel: args.onCancel,
				onClose: args.onClose,
				handlers: handlers
			});

			if(widget.onOpen){
				// TODO: in 2.0 standardize onShow() (used by StackContainer) and onOpen() (used here)
				widget.onOpen(best);
			}

			return best;
		},

		close: function(/*Widget?*/ popup){
			// summary:
			//		Close specified popup and any popups that it parented.
			//		If no popup is specified, closes all popups.

			var stack = this._stack;

			// Basically work backwards from the top of the stack closing popups
			// until we hit the specified popup, but IIRC there was some issue where closing
			// a popup would cause others to close too.  Thus if we are trying to close B in [A,B,C]
			// closing C might close B indirectly and then the while() condition will run where stack==[A]...
			// so the while condition is constructed defensively.
			while((popup && array.some(stack, function(elem){return elem.widget == popup;})) ||
				(!popup && stack.length)){
				var top = stack.pop(),
					widget = top.widget,
					onClose = top.onClose;

				if(widget.onClose){
					// TODO: in 2.0 standardize onHide() (used by StackContainer) and onClose() (used here)
					widget.onClose();
				}

				var h;
				while(h = top.handlers.pop()){ h.remove(); }

				// Hide the widget and it's wrapper unless it has already been destroyed in above onClose() etc.
				if(widget && widget.domNode){
					this.hide(widget);
				}

				if(onClose){
					onClose();
				}
			}
		}
	});

	return (dijit.popup = new PopupManager());
});

},
'dijit/layout/_TabContainerBase':function(){
define([
	"dojo/text!./templates/TabContainer.html",
	"./StackContainer",
	"./utils",	// marginBox2contextBox, layoutChildren
	"../_TemplatedMixin",
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.add
	"dojo/dom-geometry", // domGeometry.contentBox
	"dojo/dom-style" // domStyle.style
], function(template, StackContainer, layoutUtils, _TemplatedMixin, declare, domClass, domGeometry, domStyle){


/*=====
	var StackContainer = dijit.layout.StackContainer;
	var _TemplatedMixin = dijit._TemplatedMixin;
=====*/

// module:
//		dijit/layout/_TabContainerBase
// summary:
//		Abstract base class for TabContainer.   Must define _makeController() to instantiate
//		and return the widget that displays the tab labels


return declare("dijit.layout._TabContainerBase", [StackContainer, _TemplatedMixin], {
	// summary:
	//		Abstract base class for TabContainer.   Must define _makeController() to instantiate
	//		and return the widget that displays the tab labels
	// description:
	//		A TabContainer is a container that has multiple panes, but shows only
	//		one pane at a time.  There are a set of tabs corresponding to each pane,
	//		where each tab has the name (aka title) of the pane, and optionally a close button.

	// tabPosition: String
	//		Defines where tabs go relative to tab content.
	//		"top", "bottom", "left-h", "right-h"
	tabPosition: "top",

	baseClass: "dijitTabContainer",

	// tabStrip: [const] Boolean
	//		Defines whether the tablist gets an extra class for layouting, putting a border/shading
	//		around the set of tabs.   Not supported by claro theme.
	tabStrip: false,

	// nested: [const] Boolean
	//		If true, use styling for a TabContainer nested inside another TabContainer.
	//		For tundra etc., makes tabs look like links, and hides the outer
	//		border since the outer TabContainer already has a border.
	nested: false,

	templateString: template,

	postMixInProperties: function(){
		// set class name according to tab position, ex: dijitTabContainerTop
		this.baseClass += this.tabPosition.charAt(0).toUpperCase() + this.tabPosition.substr(1).replace(/-.*/, "");

		this.srcNodeRef && domStyle.set(this.srcNodeRef, "visibility", "hidden");

		this.inherited(arguments);
	},

	buildRendering: function(){
		this.inherited(arguments);

		// Create the tab list that will have a tab (a.k.a. tab button) for each tab panel
		this.tablist = this._makeController(this.tablistNode);

		if(!this.doLayout){ domClass.add(this.domNode, "dijitTabContainerNoLayout"); }

		if(this.nested){
			/* workaround IE's lack of support for "a > b" selectors by
			 * tagging each node in the template.
			 */
			domClass.add(this.domNode, "dijitTabContainerNested");
			domClass.add(this.tablist.containerNode, "dijitTabContainerTabListNested");
			domClass.add(this.tablistSpacer, "dijitTabContainerSpacerNested");
			domClass.add(this.containerNode, "dijitTabPaneWrapperNested");
		}else{
			domClass.add(this.domNode, "tabStrip-" + (this.tabStrip ? "enabled" : "disabled"));
		}
	},

	_setupChild: function(/*dijit._Widget*/ tab){
		// Overrides StackContainer._setupChild().
		domClass.add(tab.domNode, "dijitTabPane");
		this.inherited(arguments);
	},

	startup: function(){
		if(this._started){ return; }

		// wire up the tablist and its tabs
		this.tablist.startup();

		this.inherited(arguments);
	},

	layout: function(){
		// Overrides StackContainer.layout().
		// Configure the content pane to take up all the space except for where the tabs are

		if(!this._contentBox || typeof(this._contentBox.l) == "undefined"){return;}

		var sc = this.selectedChildWidget;

		if(this.doLayout){
			// position and size the titles and the container node
			var titleAlign = this.tabPosition.replace(/-h/, "");
			this.tablist.layoutAlign = titleAlign;
			var children = [this.tablist, {
				domNode: this.tablistSpacer,
				layoutAlign: titleAlign
			}, {
				domNode: this.containerNode,
				layoutAlign: "client"
			}];
			layoutUtils.layoutChildren(this.domNode, this._contentBox, children);

			// Compute size to make each of my children.
			// children[2] is the margin-box size of this.containerNode, set by layoutChildren() call above
			this._containerContentBox = layoutUtils.marginBox2contentBox(this.containerNode, children[2]);

			if(sc && sc.resize){
				sc.resize(this._containerContentBox);
			}
		}else{
			// just layout the tab controller, so it can position left/right buttons etc.
			if(this.tablist.resize){
				//make the tabs zero width so that they don't interfere with width calc, then reset
				var s = this.tablist.domNode.style;
				s.width="0";
				var width = domGeometry.getContentBox(this.domNode).w;
				s.width="";
				this.tablist.resize({w: width});
			}

			// and call resize() on the selected pane just to tell it that it's been made visible
			if(sc && sc.resize){
				sc.resize();
			}
		}
	},

	destroy: function(){
		if(this.tablist){
			this.tablist.destroy();
		}
		this.inherited(arguments);
	}
});

});

},
'dijit/Dialog':function(){
define([
	"require",
	"dojo/_base/array", // array.forEach array.indexOf array.map
	"dojo/_base/connect", // connect._keypress
	"dojo/_base/declare", // declare
	"dojo/_base/Deferred", // Deferred
	"dojo/dom", // dom.isDescendant
	"dojo/dom-class", // domClass.add domClass.contains
	"dojo/dom-geometry", // domGeometry.position
	"dojo/dom-style", // domStyle.set
	"dojo/_base/event", // event.stop
	"dojo/_base/fx", // fx.fadeIn fx.fadeOut
	"dojo/i18n", // i18n.getLocalization
	"dojo/_base/kernel", // kernel.isAsync
	"dojo/keys",
	"dojo/_base/lang", // lang.mixin lang.hitch
	"dojo/on",
	"dojo/ready",
	"dojo/_base/sniff", // has("ie") has("opera")
	"dojo/_base/window", // win.body
	"dojo/window", // winUtils.getBox
	"dojo/dnd/Moveable", // Moveable
	"dojo/dnd/TimedMoveable", // TimedMoveable
	"./focus",
	"./_base/manager",	// manager.defaultDuration
	"./_Widget",
	"./_TemplatedMixin",
	"./_CssStateMixin",
	"./form/_FormMixin",
	"./_DialogMixin",
	"./DialogUnderlay",
	"./layout/ContentPane",
	"dojo/text!./templates/Dialog.html",
	".",			// for back-compat, exporting dijit._underlay (remove in 2.0)
	"dojo/i18n!./nls/common"
], function(require, array, connect, declare, Deferred,
			dom, domClass, domGeometry, domStyle, event, fx, i18n, kernel, keys, lang, on, ready, has, win, winUtils,
			Moveable, TimedMoveable, focus, manager, _Widget, _TemplatedMixin, _CssStateMixin, _FormMixin, _DialogMixin,
			DialogUnderlay, ContentPane, template, dijit){
	
/*=====
	var _Widget = dijit._Widget;
	var _TemplatedMixin = dijit._TemplatedMixin;
	var _CssStateMixin = dijit._CssStateMixin;
	var _FormMixin = dijit.form._FormMixin;
	var _DialogMixin = dijit._DialogMixin;
=====*/	


	// module:
	//		dijit/Dialog
	// summary:
	//		A modal dialog Widget


	/*=====
	dijit._underlay = function(kwArgs){
		// summary:
		//		A shared instance of a `dijit.DialogUnderlay`
		//
		// description:
		//		A shared instance of a `dijit.DialogUnderlay` created and
		//		used by `dijit.Dialog`, though never created until some Dialog
		//		or subclass thereof is shown.
	};
	=====*/

	var _DialogBase = declare("dijit._DialogBase", [_TemplatedMixin, _FormMixin, _DialogMixin, _CssStateMixin], {
		// summary:
		//		A modal dialog Widget
		//
		// description:
		//		Pops up a modal dialog window, blocking access to the screen
		//		and also graying out the screen Dialog is extended from
		//		ContentPane so it supports all the same parameters (href, etc.)
		//
		// example:
		// |	<div data-dojo-type="dijit.Dialog" data-dojo-props="href: 'test.html'"></div>
		//
		// example:
		// |	var foo = new dijit.Dialog({ title: "test dialog", content: "test content" };
		// |	dojo.body().appendChild(foo.domNode);
		// |	foo.startup();

		templateString: template,

		baseClass: "dijitDialog",

		cssStateNodes: {
			closeButtonNode: "dijitDialogCloseIcon"
		},

		// Map widget attributes to DOMNode attributes.
		_setTitleAttr: [
			{ node: "titleNode", type: "innerHTML" },
			{ node: "titleBar", type: "attribute" }
		],

		// open: [readonly] Boolean
		//		True if Dialog is currently displayed on screen.
		open: false,

		// duration: Integer
		//		The time in milliseconds it takes the dialog to fade in and out
		duration: manager.defaultDuration,

		// refocus: Boolean
		// 		A Toggle to modify the default focus behavior of a Dialog, which
		// 		is to re-focus the element which had focus before being opened.
		//		False will disable refocusing. Default: true
		refocus: true,

		// autofocus: Boolean
		// 		A Toggle to modify the default focus behavior of a Dialog, which
		// 		is to focus on the first dialog element after opening the dialog.
		//		False will disable autofocusing. Default: true
		autofocus: true,

		// _firstFocusItem: [private readonly] DomNode
		//		The pointer to the first focusable node in the dialog.
		//		Set by `dijit._DialogMixin._getFocusItems`.
		_firstFocusItem: null,

		// _lastFocusItem: [private readonly] DomNode
		//		The pointer to which node has focus prior to our dialog.
		//		Set by `dijit._DialogMixin._getFocusItems`.
		_lastFocusItem: null,

		// doLayout: [protected] Boolean
		//		Don't change this parameter from the default value.
		//		This ContentPane parameter doesn't make sense for Dialog, since Dialog
		//		is never a child of a layout container, nor can you specify the size of
		//		Dialog in order to control the size of an inner widget.
		doLayout: false,

		// draggable: Boolean
		//		Toggles the moveable aspect of the Dialog. If true, Dialog
		//		can be dragged by it's title. If false it will remain centered
		//		in the viewport.
		draggable: true,

		//aria-describedby: String
		//		Allows the user to add an aria-describedby attribute onto the dialog.   The value should
		//		be the id of the container element of text that describes the dialog purpose (usually
		//		the first text in the dialog).
		//		<div data-dojo-type="dijit.Dialog" aria-describedby="intro" .....>
		//			<div id="intro">Introductory text</div>
		//			<div>rest of dialog contents</div>
		//		</div>
		"aria-describedby":"",

		postMixInProperties: function(){
			var _nlsResources = i18n.getLocalization("dijit", "common");
			lang.mixin(this, _nlsResources);
			this.inherited(arguments);
		},

		postCreate: function(){
			domStyle.set(this.domNode, {
				display: "none",
				position:"absolute"
			});
			win.body().appendChild(this.domNode);

			this.inherited(arguments);

			this.connect(this, "onExecute", "hide");
			this.connect(this, "onCancel", "hide");
			this._modalconnects = [];
		},

		onLoad: function(){
			// summary:
			//		Called when data has been loaded from an href.
			//		Unlike most other callbacks, this function can be connected to (via `dojo.connect`)
			//		but should *not* be overridden.
			// tags:
			//		callback

			// when href is specified we need to reposition the dialog after the data is loaded
			// and find the focusable elements
			this._position();
			if(this.autofocus && DialogLevelManager.isTop(this)){
				this._getFocusItems(this.domNode);
				focus.focus(this._firstFocusItem);
			}
			this.inherited(arguments);
		},

		_endDrag: function(){
			// summary:
			//		Called after dragging the Dialog. Saves the position of the dialog in the viewport,
			//		and also adjust position to be fully within the viewport, so user doesn't lose access to handle
			var nodePosition = domGeometry.position(this.domNode),
				viewport = winUtils.getBox();
			nodePosition.y = Math.min(Math.max(nodePosition.y, 0), (viewport.h - nodePosition.h));
			nodePosition.x = Math.min(Math.max(nodePosition.x, 0), (viewport.w - nodePosition.w));
			this._relativePosition = nodePosition;
			this._position();
		},

		_setup: function(){
			// summary:
			//		Stuff we need to do before showing the Dialog for the first
			//		time (but we defer it until right beforehand, for
			//		performance reasons).
			// tags:
			//		private

			var node = this.domNode;

			if(this.titleBar && this.draggable){
				this._moveable = new ((has("ie") == 6) ? TimedMoveable // prevent overload, see #5285
					: Moveable)(node, { handle: this.titleBar });
				this.connect(this._moveable, "onMoveStop", "_endDrag");
			}else{
				domClass.add(node,"dijitDialogFixed");
			}

			this.underlayAttrs = {
				dialogId: this.id,
				"class": array.map(this["class"].split(/\s/), function(s){ return s+"_underlay"; }).join(" ")
			};
		},

		_size: function(){
			// summary:
			// 		If necessary, shrink dialog contents so dialog fits in viewport
			// tags:
			//		private

			this._checkIfSingleChild();

			// If we resized the dialog contents earlier, reset them back to original size, so
			// that if the user later increases the viewport size, the dialog can display w/out a scrollbar.
			// Need to do this before the domGeometry.position(this.domNode) call below.
			if(this._singleChild){
				if(this._singleChildOriginalStyle){
					this._singleChild.domNode.style.cssText = this._singleChildOriginalStyle;
				}
				delete this._singleChildOriginalStyle;
			}else{
				domStyle.set(this.containerNode, {
					width:"auto",
					height:"auto"
				});
			}

			var bb = domGeometry.position(this.domNode);
			var viewport = winUtils.getBox();
			if(bb.w >= viewport.w || bb.h >= viewport.h){
				// Reduce size of dialog contents so that dialog fits in viewport

				var w = Math.min(bb.w, Math.floor(viewport.w * 0.75)),
					h = Math.min(bb.h, Math.floor(viewport.h * 0.75));

				if(this._singleChild && this._singleChild.resize){
					this._singleChildOriginalStyle = this._singleChild.domNode.style.cssText;
					this._singleChild.resize({w: w, h: h});
				}else{
					domStyle.set(this.containerNode, {
						width: w + "px",
						height: h + "px",
						overflow: "auto",
						position: "relative"	// workaround IE bug moving scrollbar or dragging dialog
					});
				}
			}else{
				if(this._singleChild && this._singleChild.resize){
					this._singleChild.resize();
				}
			}
		},

		_position: function(){
			// summary:
			//		Position modal dialog in the viewport. If no relative offset
			//		in the viewport has been determined (by dragging, for instance),
			//		center the node. Otherwise, use the Dialog's stored relative offset,
			//		and position the node to top: left: values based on the viewport.
			if(!domClass.contains(win.body(), "dojoMove")){	// don't do anything if called during auto-scroll
				var node = this.domNode,
					viewport = winUtils.getBox(),
					p = this._relativePosition,
					bb = p ? null : domGeometry.position(node),
					l = Math.floor(viewport.l + (p ? p.x : (viewport.w - bb.w) / 2)),
					t = Math.floor(viewport.t + (p ? p.y : (viewport.h - bb.h) / 2))
				;
				domStyle.set(node,{
					left: l + "px",
					top: t + "px"
				});
			}
		},

		_onKey: function(/*Event*/ evt){
			// summary:
			//		Handles the keyboard events for accessibility reasons
			// tags:
			//		private

			if(evt.charOrCode){
				var node = evt.target;
				if(evt.charOrCode === keys.TAB){
					this._getFocusItems(this.domNode);
				}
				var singleFocusItem = (this._firstFocusItem == this._lastFocusItem);
				// see if we are shift-tabbing from first focusable item on dialog
				if(node == this._firstFocusItem && evt.shiftKey && evt.charOrCode === keys.TAB){
					if(!singleFocusItem){
						focus.focus(this._lastFocusItem); // send focus to last item in dialog
					}
					event.stop(evt);
				}else if(node == this._lastFocusItem && evt.charOrCode === keys.TAB && !evt.shiftKey){
					if(!singleFocusItem){
						focus.focus(this._firstFocusItem); // send focus to first item in dialog
					}
					event.stop(evt);
				}else{
					// see if the key is for the dialog
					while(node){
						if(node == this.domNode || domClass.contains(node, "dijitPopup")){
							if(evt.charOrCode == keys.ESCAPE){
								this.onCancel();
							}else{
								return; // just let it go
							}
						}
						node = node.parentNode;
					}
					// this key is for the disabled document window
					if(evt.charOrCode !== keys.TAB){ // allow tabbing into the dialog for a11y
						event.stop(evt);
					// opera won't tab to a div
					}else if(!has("opera")){
						try{
							this._firstFocusItem.focus();
						}catch(e){ /*squelch*/ }
					}
				}
			}
		},

		show: function(){
			// summary:
			//		Display the dialog
			// returns: dojo.Deferred
			//		Deferred object that resolves when the display animation is complete

			if(this.open){ return; }

			if(!this._started){
				this.startup();
			}

			// first time we show the dialog, there's some initialization stuff to do
			if(!this._alreadyInitialized){
				this._setup();
				this._alreadyInitialized=true;
			}

			if(this._fadeOutDeferred){
				this._fadeOutDeferred.cancel();
			}

			this._modalconnects.push(on(window, "scroll", lang.hitch(this, "layout")));
			this._modalconnects.push(on(window, "resize", lang.hitch(this, function(){
				// IE gives spurious resize events and can actually get stuck
				// in an infinite loop if we don't ignore them
				var viewport = winUtils.getBox();
				if(!this._oldViewport ||
						viewport.h != this._oldViewport.h ||
						viewport.w != this._oldViewport.w){
					this.layout();
					this._oldViewport = viewport;
				}
			})));
			this._modalconnects.push(on(this.domNode, connect._keypress, lang.hitch(this, "_onKey")));

			domStyle.set(this.domNode, {
				opacity:0,
				display:""
			});

			this._set("open", true);
			this._onShow(); // lazy load trigger

			this._size();
			this._position();

			// fade-in Animation object, setup below
			var fadeIn;

			this._fadeInDeferred = new Deferred(lang.hitch(this, function(){
				fadeIn.stop();
				delete this._fadeInDeferred;
			}));

			fadeIn = fx.fadeIn({
				node: this.domNode,
				duration: this.duration,
				beforeBegin: lang.hitch(this, function(){
					DialogLevelManager.show(this, this.underlayAttrs);
				}),
				onEnd: lang.hitch(this, function(){
					if(this.autofocus && DialogLevelManager.isTop(this)){
						// find focusable items each time dialog is shown since if dialog contains a widget the
						// first focusable items can change
						this._getFocusItems(this.domNode);
						focus.focus(this._firstFocusItem);
					}
					this._fadeInDeferred.callback(true);
					delete this._fadeInDeferred;
				})
			}).play();

			return this._fadeInDeferred;
		},

		hide: function(){
			// summary:
			//		Hide the dialog
			// returns: dojo.Deferred
			//		Deferred object that resolves when the hide animation is complete

			// if we haven't been initialized yet then we aren't showing and we can just return
			if(!this._alreadyInitialized){
				return;
			}
			if(this._fadeInDeferred){
				this._fadeInDeferred.cancel();
			}

			// fade-in Animation object, setup below
			var fadeOut;

			this._fadeOutDeferred = new Deferred(lang.hitch(this, function(){
				fadeOut.stop();
				delete this._fadeOutDeferred;
			}));
			// fire onHide when the promise resolves.
			this._fadeOutDeferred.then(lang.hitch(this, 'onHide'));

			fadeOut = fx.fadeOut({
				node: this.domNode,
				duration: this.duration,
				onEnd: lang.hitch(this, function(){
					this.domNode.style.display = "none";
					DialogLevelManager.hide(this);
					this._fadeOutDeferred.callback(true);
					delete this._fadeOutDeferred;
				})
			 }).play();

			if(this._scrollConnected){
				this._scrollConnected = false;
			}
			var h;
			while(h = this._modalconnects.pop()){
				h.remove();
			}

			if(this._relativePosition){
				delete this._relativePosition;
			}
			this._set("open", false);

			return this._fadeOutDeferred;
		},

		layout: function(){
			// summary:
			//		Position the Dialog and the underlay
			// tags:
			//		private
			if(this.domNode.style.display != "none"){
				if(dijit._underlay){	// avoid race condition during show()
					dijit._underlay.layout();
				}
				this._position();
			}
		},

		destroy: function(){
			if(this._fadeInDeferred){
				this._fadeInDeferred.cancel();
			}
			if(this._fadeOutDeferred){
				this._fadeOutDeferred.cancel();
			}
			if(this._moveable){
				this._moveable.destroy();
			}
			var h;
			while(h = this._modalconnects.pop()){
				h.remove();
			}

			DialogLevelManager.hide(this);

			this.inherited(arguments);
		}
	});

	var Dialog = declare("dijit.Dialog", [ContentPane, _DialogBase], {});
	Dialog._DialogBase = _DialogBase;	// for monkey patching

	var DialogLevelManager = Dialog._DialogLevelManager = {
		// summary:
		//		Controls the various active "levels" on the page, starting with the
		//		stuff initially visible on the page (at z-index 0), and then having an entry for
		//		each Dialog shown.

		_beginZIndex: 950,

		show: function(/*dijit._Widget*/ dialog, /*Object*/ underlayAttrs){
			// summary:
			//		Call right before fade-in animation for new dialog.
			//		Saves current focus, displays/adjusts underlay for new dialog,
			//		and sets the z-index of the dialog itself.
			//
			//		New dialog will be displayed on top of all currently displayed dialogs.
			//
			//		Caller is responsible for setting focus in new dialog after the fade-in
			//		animation completes.

			// Save current focus
			ds[ds.length-1].focus = focus.curNode;

			// Display the underlay, or if already displayed then adjust for this new dialog
			var underlay = dijit._underlay;
			if(!underlay || underlay._destroyed){
				underlay = dijit._underlay = new DialogUnderlay(underlayAttrs);
			}else{
				underlay.set(dialog.underlayAttrs);
			}

			// Set z-index a bit above previous dialog
			var zIndex = ds[ds.length-1].dialog ? ds[ds.length-1].zIndex + 2 : Dialog._DialogLevelManager._beginZIndex;
			if(ds.length == 1){	// first dialog
				underlay.show();
			}
			domStyle.set(dijit._underlay.domNode, 'zIndex', zIndex - 1);

			// Dialog
			domStyle.set(dialog.domNode, 'zIndex', zIndex);

			ds.push({dialog: dialog, underlayAttrs: underlayAttrs, zIndex: zIndex});
		},

		hide: function(/*dijit._Widget*/ dialog){
			// summary:
			//		Called when the specified dialog is hidden/destroyed, after the fade-out
			//		animation ends, in order to reset page focus, fix the underlay, etc.
			//		If the specified dialog isn't open then does nothing.
			//
			//		Caller is responsible for either setting display:none on the dialog domNode,
			//		or calling dijit.popup.hide(), or removing it from the page DOM.

			if(ds[ds.length-1].dialog == dialog){
				// Removing the top (or only) dialog in the stack, return focus
				// to previous dialog

				ds.pop();

				var pd = ds[ds.length-1];	// the new active dialog (or the base page itself)

				// Adjust underlay
				if(ds.length == 1){
					// Returning to original page.
					// Hide the underlay, unless the underlay widget has already been destroyed
					// because we are being called during page unload (when all widgets are destroyed)
					if(!dijit._underlay._destroyed){
						dijit._underlay.hide();
					}
				}else{
					// Popping back to previous dialog, adjust underlay
					domStyle.set(dijit._underlay.domNode, 'zIndex', pd.zIndex - 1);
					dijit._underlay.set(pd.underlayAttrs);
				}

				// Adjust focus
				if(dialog.refocus){
					// If we are returning control to a previous dialog but for some reason
					// that dialog didn't have a focused field, set focus to first focusable item.
					// This situation could happen if two dialogs appeared at nearly the same time,
					// since a dialog doesn't set it's focus until the fade-in is finished.
					var focus = pd.focus;
					if(pd.dialog && (!focus || !dom.isDescendant(focus, pd.dialog.domNode))){
						pd.dialog._getFocusItems(pd.dialog.domNode);
						focus = pd.dialog._firstFocusItem;
					}

					if(focus){
						// Refocus the button that spawned the Dialog.   This will fail in corner cases including
						// page unload on IE, because the dijit/form/Button that launched the Dialog may get destroyed
						// before this code runs.  (#15058)
						try{
							focus.focus();
						}catch(e){}
					}
				}
			}else{
				// Removing a dialog out of order (#9944, #10705).
				// Don't need to mess with underlay or z-index or anything.
				var idx = array.indexOf(array.map(ds, function(elem){return elem.dialog}), dialog);
				if(idx != -1){
					ds.splice(idx, 1);
				}
			}
		},

		isTop: function(/*dijit._Widget*/ dialog){
			// summary:
			//		Returns true if specified Dialog is the top in the task
			return ds[ds.length-1].dialog == dialog;
		}
	};

	// Stack representing the various active "levels" on the page, starting with the
	// stuff initially visible on the page (at z-index 0), and then having an entry for
	// each Dialog shown.
	// Each element in stack has form {
	//		dialog: dialogWidget,
	//		focus: returnFromGetFocus(),
	//		underlayAttrs: attributes to set on underlay (when this widget is active)
	// }
	var ds = Dialog._dialogStack = [
		{dialog: null, focus: null, underlayAttrs: null}	// entry for stuff at z-index: 0
	];

	// Back compat w/1.6, remove for 2.0
	if(!kernel.isAsync){
		ready(0, function(){
			var requires = ["dijit/TooltipDialog"];
			require(requires);	// use indirection so modules not rolled into a build
		});
	}

	return Dialog;
});

},
'idx/oneui/_EventTriggerMixin':function(){
/*
 * Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

// This mix-in provides support for binding a widget to one or more DOM nodes 
// and being notified when specified events occur. Support for a "hover" trigger
// is also provided.

define(["dojo/_base/array",
        "dojo/_base/declare",
        "dojo/_base/event",
        "dojo/_base/lang",
        "dojo/_base/window",
        "dojo/dom",
		"dojo/io/iframe",
        "dojo/mouse",
        "dojo/on",
        "dojo/window",
        "dijit/_MenuBase"],
        function(array,
                 declare,
                 event,
                 lang,
                 window,
                 dom,
				 iframe,
                 mouse,
                 on,
                 winUtils,
                 _MenuBase){
    
	// Ensure we're not relying on the old globals, ready for 2.0.
	var dojo = {}, dijit = {};

	/**
	 * Creates a new idx.oneui._EventTriggerMixin
	 * @name idx.oneui._EventTriggerMixin
	 * @class This mix-in provides support for binding a widget to one or more
	 * DOM nodes and being notified when specified events occur. Support for a
	 * "hover" trigger is also provided.
	 */
	return declare("idx.oneui._EventTriggerMixin", null, 
	/** @lends idx.oneui._EventTriggerMixin.prototype */
	{
		/**
		 * An array of "binding" objects that store details of the event
		 * triggers that are currently bound to trigger nodes.
		 * @type Object[]
		 */
		_bindings: null,
    
		/**
		 * A timer that is started at the beginning of a hover event and
		 * triggers the calling of onTrigger() after hoverDuration, unless it
		 * is cancelled.
		 * @type timer handle
		 */
		_hoverTimer: null,
    
		/**
		 * The time in milliseconds that the mouse needs to remain within a DOM
		 * node in order to cause a "hover" event to be reported. Initialised
		 * to match the hover delay used by menus.
		 * @type number
		 */
		hoverDuration: _MenuBase.prototype.popupDelay,
    
		/**
		 * Constructor.
		 * @private
		 */
		constructor: function(){
			this._bindings = [];
		},

		/**
		 * Register a DOM node to act as an event trigger. When the specified
		 * event is detected, the "_onTrigger" method is called, passing a
		 * trigger object containing contextual and event information as the
		 * single argument. The fields in the trigger object are as follows:
		 * <ul>
		 * <li>
		 * triggerNode: {DOMNode} the DOM node that triggered the event
		 * </li>
		 * <li>
		 * eventName: {string} the name of the event (as passed as eventName)
		 * </li>
		 * <li>
		 * event: {Event} the event object (in the case of "hover", this will be
		 * the most recent mouse.enter or mousemove event received
		 * </li>
		 * <li>
		 * additionalData: {Object} the additional data object supplied as additionalData
		 * </li>
		 * </ul>
		 * @param {string | DOMNode} triggerNode The DOM node or node ID that
		 * trigger events are to be detected on.
		 * @param {string} eventName The name of the event to be detected.
		 * As well as regular dojo events the value "hover" can also be used
		 * to configure a hover trigger.
		 * @param {function(params)} filterCallback An optional filter callback
		 * function that can be used to limit the events that are detected.
		 * If the function returns true the event is considered to be a
		 * trigger otherwise it is ignored. The same params object is supplied
		 * to the filter callback that will be passed to the _onTrigger method
		 * if the filter callback returns true. 
		 * @param {Object} additionalData Optional application-specific data
		 * that will be returned when an event is detected.
		 */
		_addEventTrigger: function(triggerNode, eventName, filterCallback, additionalData) {
 			// Resolve node ID (if passed as parameter) and validate DOM node.
			triggerNode = dom.byId(triggerNode);
			if(!triggerNode){
				require.log("ERROR: oneui._EventTriggerMixin._addEventTrigger(): Invalid triggerNode parameter.");
				return;
			}

			// a utility function to check the filter and call onTrigger
			// using data in the closure, hitched this, and a passed event			
			var trigger = lang.hitch(this, function(event){
				var params = { triggerNode: triggerNode, eventName: eventName, event: event, additionalData: additionalData }
				
				// If a filter callback is not defined or it returns true then
				// trigger the event. Otherwise ignore it.
				if (!filterCallback || filterCallback(params)) {
					this._onTrigger(params);
				}
			});
			
			// A utility function to create a 'hover' pseudo-event from real
			// mouse events.
			var createHoverEvent = function(event){
				return {
					type: "hover",
					pageX: event.pageX,
					pageY: event.pageY,
					screenX: event.screenX,
					screenY: event.screenY,
					clientX: event.clientX,
					clientY: event.clientY
				};
			};
      
			// store a binding object per call, containing:
			//   triggerNode: the node we attach a listener/listeners to
			//   connectHandles: an array of currently attached listeners
			//   hoverDuration: if eventName=="hover", the hover duration in ms
			//   hoverTimer: the handle to any currently-running hover timer
			//   bindFunction: a function that attaches the required listener(s)
			//   unbindFunction: a function that removes the listener(s)
			//   iframeOnLoadHandler: if triggerNode is an iframe, a function
			//                         that reattaches the listeners on iframe load
			var binding = { triggerNode: triggerNode, connectHandles: [] }
			
			if(eventName == "hover"){
				binding.hoverDuration = this.hoverDuration;
				binding.hoverTimer = null;
			} 
			
			// Create a function that implements the necessary binding logic.
			binding.bindFunction = function(){
      
				// Determine which node to connect event listeners to, which may need to 
				// be different to the specified trigger node.
				var connectNode;
				if(triggerNode.tagName == "IFRAME"){
					// If the trigger node is an iFrame then we need to attach event 
					// listeners to the <body> element of the iFrame's contents. 
					try{
						var iframeDocNode = iframe.doc(triggerNode);
						connectNode = iframeDocNode ? iframeDocNode.body : null;
					}catch(e){
						require.log("ERROR: oneui._EventTriggerMixin._addEventTrigger(): Error accessing body of document within iframe. " + e);
					}
				}else{
					// Normal trigger node specified - just attach event listeners to it.
					connectNode = triggerNode;
				}
				
				if(!connectNode){
					require.log("ERROR: oneui._EventTriggerMixin._addEventTrigger(): Unable to determine node to attach event listener(s) to.");
					return;
				}
				
				// If the event is "hover" then attach specific event listeners and 
				// callbacks to synthesise it, otherwise just attach the specified
				// event listener.
				if(eventName == "hover"){
					var hoverEvent = null;
					
					// Connect mouseenter and mouseleave event listeners to the 
					// connection node to start/clear the hover timer. Trigger
					// if/when the timer expires. Keep track of the mouse recent
					// mouse event too.
					binding.connectHandles.push(on(connectNode, mouse.enter, lang.hitch(this, function(event){
						hoverEvent = createHoverEvent(event);
						
						// there shouldn't be a running timer, but clear it if there is
						if(binding.hoverTimer){
							clearTimeout(binding.hoverTimer);
						} 
						
						// the hoverEvent may be replaced by the time the hover timer fires
						binding.hoverTimer = setTimeout(function(){ trigger(hoverEvent); }, binding.hoverDuration);
					})));
					
					binding.connectHandles.push(on(connectNode, mouse.leave, lang.hitch(this, function(event){
						if(binding.hoverTimer){
							clearTimeout(binding.hoverTimer);
							binding.hoverTimer = null;
						}
						hoverEvent = undefined;
					})));
					
					binding.connectHandles.push(on(connectNode, "mousemove", function(event){
						hoverEvent = createHoverEvent(event);
					}));
					
				}else{
					// Connect the appropriate event listener to the connect node.
					binding.connectHandles.push(on(connectNode, eventName, function(event){ trigger(event); }));
				}
			}
			
			// Create a disconnect function that removes all the event listeners.
			binding.unbindFunction = function(){
				array.forEach(binding.connectHandles, function(conn){ conn.remove(); });
				
				if(binding.hoverTimer){
					clearTimeout(binding.hoverTimer);
					binding.hoverTimer = null;
				}
			}
			
			// If the trigger node is an iframe then add an onload listener to it, 
			// with a callback that rebinds the handlers to the iframe content if 
			// it changes. Store the connect handle in the binding, so that it can
			// be cleaned up.
			if(triggerNode.tagName === "IFRAME"){
				binding.iframeOnLoadHandler = function(event){
					// Remove all existing event listeners.
					try{
						// This may fail because the iframe content node that the event
						// listeners are bound to has been destroyed.
						binding.unbindFunction();
					}catch(e){
						// Ignore failures - 'best efforts' cleanup.
					}

					// Rebind all of the event listeners to the iframe and new content 
					// document.
					binding.bindFunction();
				}
				
				if(triggerNode.addEventListener){
					triggerNode.addEventListener("load", binding.iframeOnLoadHandler, false);
				}else{
					triggerNode.attachEvent("onload", binding.iframeOnLoadHandler);
				}
			}

			// store the binding data, and call the bind function once now
			this._bindings.push(binding);
			binding.bindFunction(); 
		},

		/**
		 * Callback that is called whenever one of the specified trigger
		 * events (as configured via _addEventTrigger()) is detected on
		 * a DOM node.
		 * @param {Object} trigger An object that describes the triggering 
		 * event. The fields in the trigger object are as follows:
		 * <ul>
		 * <li>
		 * triggerNode: {DOMNode} the DOM node that triggered the event
		 * </li>
		 * <li>
		 * eventName: {string} the name of the event
		 * </li>
		 * <li>
		 * event: {Event} the event object (in the case of "hover", this will be
		 * a pseudo-event with properties of clientX, clientY, pageX, pageY, 
		 * screenX, screenY set when the data is available from the original 
		 * event, and type set to 'hover').
		 * </li>
		 * <li>
		 * additionalData: {Object} the additional data object supplied as
		 * additionalData via _addEventTrigger()
		 * </li>
		 * </ul>
		 */
		_onTrigger: function(trigger) {
			// summary:
			//   Callback that is called whenever one of the specified trigger events
			//   (as configured via _addEventTrigger()) is detected on a DOM node.
			//
			// parameters:
			//   trigger: Object
			//     an object containing data about the event.
			//
			//         triggerNode: DOMNode
			//           the DOM node that the trigger event was detected on.
			//
			//         eventName: String
			//           the name of the event that was detected.
			//
			//         event: Event
			//           the dojo event object describing the event.
			//
			//         additionalData: Object
			//           optional application-specific data that may have been 
			//           supplied when the event trigger was configured via
			//           _addEventTrigger().
			//
			// returns: nothing.
		},
    
		/**
		 * Deregisters a DOM node as an event trigger, by removing all the 
		 * event triggers that have been configured via _addEventTrigger().
		 * @param {string | DOMNode} triggerNode The DOM node or node ID that
		 * trigger events are no longer to be detected on. If undefined or
		 * null all event triggers for all nodes are unbound.
		 */
		_removeEventTriggers: function(triggerNode) {

			// Resolve node ID (if passed as parameter) and validate DOM node.
			if(triggerNode){
				triggerNode = dom.byId(triggerNode);
			}
      
			// Iterate through the array of bindings, unbind the appropriate ones,
			// and remove the binding records.
			for(var i = this._bindings.length - 1; i >= 0; i--){
				var binding = this._bindings[i];
				
				if(!triggerNode || (triggerNode === binding.triggerNode)){
					binding.unbindFunction();
					if(binding.iframeOnLoadHandler){
						if(binding.triggerNode.removeEventListener){
							binding.triggerNode.removeEventListener("load", binding.iframeOnLoadHandler, false);
						}else{
							binding.triggerNode.detachEvent("onload", binding.iframeOnLoadHandler);
						}
					}
					this._bindings.splice(i, 1);
				}
			}
		}
		
	});
});
	
},
'url:dijit/form/templates/ComboButton.html':"<table class=\"dijit dijitReset dijitInline dijitLeft\"\n\tcellspacing='0' cellpadding='0' role=\"presentation\"\n\t><tbody role=\"presentation\"><tr role=\"presentation\"\n\t\t><td class=\"dijitReset dijitStretch dijitButtonNode\" data-dojo-attach-point=\"buttonNode\" data-dojo-attach-event=\"ondijitclick:_onClick,onkeypress:_onButtonKeyPress\"\n\t\t><div id=\"${id}_button\" class=\"dijitReset dijitButtonContents\"\n\t\t\tdata-dojo-attach-point=\"titleNode\"\n\t\t\trole=\"button\" aria-labelledby=\"${id}_label\"\n\t\t\t><div class=\"dijitReset dijitInline dijitIcon\" data-dojo-attach-point=\"iconNode\" role=\"presentation\"></div\n\t\t\t><div class=\"dijitReset dijitInline dijitButtonText\" id=\"${id}_label\" data-dojo-attach-point=\"containerNode\" role=\"presentation\"></div\n\t\t></div\n\t\t></td\n\t\t><td id=\"${id}_arrow\" class='dijitReset dijitRight dijitButtonNode dijitArrowButton'\n\t\t\tdata-dojo-attach-point=\"_popupStateNode,focusNode,_buttonNode\"\n\t\t\tdata-dojo-attach-event=\"onkeypress:_onArrowKeyPress\"\n\t\t\ttitle=\"${optionsTitle}\"\n\t\t\trole=\"button\" aria-haspopup=\"true\"\n\t\t\t><div class=\"dijitReset dijitArrowButtonInner\" role=\"presentation\"></div\n\t\t\t><div class=\"dijitReset dijitArrowButtonChar\" role=\"presentation\">&#9660;</div\n\t\t></td\n\t\t><td style=\"display:none !important;\"\n\t\t\t><input ${!nameAttrSetting} type=\"${type}\" value=\"${value}\" data-dojo-attach-point=\"valueNode\"\n\t\t/></td></tr></tbody\n></table>\n",
'url:ibm/usmi/widgets/templates/TreeTopper.html':"<div data-dojo-attach-point=\"mainNode\">\r\n<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n\t<div class=\"treeTopperCategoryBar\">\r\n\t   <div class=\"treeTopperCategoryTitle\" data-dojo-attach-point=\"titlePoint\" role=\"heading\" aria-level=2 ></div>\r\n\t</div>\r\n\t<div class=\"treeTopper\" data-dojo-attach-point=\"containerNode\"></div>\r\n</div>",
'ibm/tivoli/navmodel/WidgetTypeInfo':function(){
// wrapped by build app
define("ibm/tivoli/navmodel/WidgetTypeInfo", ["dijit","dojo","dojox"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */
dojo.provide("ibm.tivoli.navmodel.WidgetTypeInfo"); 

////////////////////////////////////
// WidgetTypeInfo
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.WidgetTypeInfo", null, {
	// private members       
	_widgetType: null,
	_version: null,
	
	/*=====
	constructor_kwArgs: function(widgetType, version) {
		// widgetType: Object
		//		json widget type information object
		// version: Float
		//		optional: version used to communicate with rest service (i.e. json version)
	},
	=====*/
	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs Provider object.
		this._version = kwArgs.version;
		this._widgetType = kwArgs.widgetType;
		if (!this._widgetType) {
			this._widgetType = {};
		}
	},

	getAttribute: function(key) {
		// summary:
		//		Returns general attribute for requested key
		return this._widgetType[key];
	},	
	
	getId: function() {
		// summary:
		//		Returns the id of this definition
		return this._widgetType.id;
	},
	
	getType: function() {
		// summary:
		//		Returns the widget type
		return this._widgetType.type;
	},
	
	getVersion: function() {
		// summary:
		//		Returns the schema version
		return this._widgetType.version;
	}
});

});

},
'dijit/TooltipDialog':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.replace
	"dojo/_base/event", // event.stop
	"dojo/keys", // keys
	"dojo/_base/lang", // lang.hitch
	"./focus",
	"./layout/ContentPane",
	"./_DialogMixin",
	"./form/_FormMixin",
	"./_TemplatedMixin",
	"dojo/text!./templates/TooltipDialog.html",
	"."		// exports methods to dijit global
], function(declare, domClass, event, keys, lang,
			focus, ContentPane, _DialogMixin, _FormMixin, _TemplatedMixin, template, dijit){

/*=====
	var ContentPane = dijit.layout.ContentPane;
	var _DialogMixin = dijit._DialogMixin;
	var _FormMixin = dijit.form._FormMixin;
	var _TemplatedMixin = dijit._TemplatedMixin;
=====*/

	// module:
	//		dijit/TooltipDialog
	// summary:
	//		Pops up a dialog that appears like a Tooltip


	return declare("dijit.TooltipDialog",
		[ContentPane, _TemplatedMixin, _FormMixin, _DialogMixin], {
		// summary:
		//		Pops up a dialog that appears like a Tooltip

		// title: String
		// 		Description of tooltip dialog (required for a11y)
		title: "",

		// doLayout: [protected] Boolean
		//		Don't change this parameter from the default value.
		//		This ContentPane parameter doesn't make sense for TooltipDialog, since TooltipDialog
		//		is never a child of a layout container, nor can you specify the size of
		//		TooltipDialog in order to control the size of an inner widget.
		doLayout: false,

		// autofocus: Boolean
		// 		A Toggle to modify the default focus behavior of a Dialog, which
		// 		is to focus on the first dialog element after opening the dialog.
		//		False will disable autofocusing. Default: true
		autofocus: true,

		// baseClass: [protected] String
		//		The root className to use for the various states of this widget
		baseClass: "dijitTooltipDialog",

		// _firstFocusItem: [private] [readonly] DomNode
		//		The pointer to the first focusable node in the dialog.
		//		Set by `dijit._DialogMixin._getFocusItems`.
		_firstFocusItem: null,

		// _lastFocusItem: [private] [readonly] DomNode
		//		The pointer to which node has focus prior to our dialog.
		//		Set by `dijit._DialogMixin._getFocusItems`.
		_lastFocusItem: null,

		templateString: template,

		_setTitleAttr: function(/*String*/ title){
			this.containerNode.title = title;
			this._set("title", title)
		},

		postCreate: function(){
			this.inherited(arguments);
			this.connect(this.containerNode, "onkeypress", "_onKey");
		},

		orient: function(/*DomNode*/ node, /*String*/ aroundCorner, /*String*/ corner){
			// summary:
			//		Configure widget to be displayed in given position relative to the button.
			//		This is called from the dijit.popup code, and should not be called
			//		directly.
			// tags:
			//		protected
			var newC = "dijitTooltipAB" + (corner.charAt(1) == 'L' ? "Left" : "Right")
					+ " dijitTooltip"
					+ (corner.charAt(0) == 'T' ? "Below" : "Above");

			domClass.replace(this.domNode, newC, this._currentOrientClass || "");
			this._currentOrientClass = newC;
		},

		focus: function(){
			// summary:
			//		Focus on first field
			this._getFocusItems(this.containerNode);
			focus.focus(this._firstFocusItem);
		},

		onOpen: function(/*Object*/ pos){
			// summary:
			//		Called when dialog is displayed.
			//		This is called from the dijit.popup code, and should not be called directly.
			// tags:
			//		protected

			this.orient(this.domNode,pos.aroundCorner, pos.corner);
			this._onShow(); // lazy load trigger
		},

		onClose: function(){
			// summary:
			//		Called when dialog is hidden.
			//		This is called from the dijit.popup code, and should not be called directly.
			// tags:
			//		protected
			this.onHide();
		},

		_onKey: function(/*Event*/ evt){
			// summary:
			//		Handler for keyboard events
			// description:
			//		Keep keyboard focus in dialog; close dialog on escape key
			// tags:
			//		private

			var node = evt.target;
			if(evt.charOrCode === keys.TAB){
				this._getFocusItems(this.containerNode);
			}
			var singleFocusItem = (this._firstFocusItem == this._lastFocusItem);
			if(evt.charOrCode == keys.ESCAPE){
				// Use setTimeout to avoid crash on IE, see #10396.
				setTimeout(lang.hitch(this, "onCancel"), 0);
				event.stop(evt);
			}else if(node == this._firstFocusItem && evt.shiftKey && evt.charOrCode === keys.TAB){
				if(!singleFocusItem){
					focus.focus(this._lastFocusItem); // send focus to last item in dialog
				}
				event.stop(evt);
			}else if(node == this._lastFocusItem && evt.charOrCode === keys.TAB && !evt.shiftKey){
				if(!singleFocusItem){
					focus.focus(this._firstFocusItem); // send focus to first item in dialog
				}
				event.stop(evt);
			}else if(evt.charOrCode === keys.TAB){
				// we want the browser's default tab handling to move focus
				// but we don't want the tab to propagate upwards
				evt.stopPropagation();
			}
		}
	});
});

},
'ibm/tivoli/navmodel/EventRouter':function(){
// wrapped by build app
define("ibm/tivoli/navmodel/EventRouter", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/Trace"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.EventRouter"); 

dojo.require("ibm.tivoli.navmodel.Trace");

ibm.tivoli.navmodel.EventRouter._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.EventRouter");

////////////////////////////////////
// EventRouter
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.EventRouter", null, {
	// private members
	_args: null,
	_initialized: false,
	_globalObj: null,
	_trace: ibm.tivoli.navmodel.EventRouter._trace,
	
	/*=====
	constructor_kwArgs: function(eventDestination, timeout, pollDelay, pollErrorDelay, contextRoot) {
		// eventDestination: String
		//		destination to listen to (i.e. provided by Dataset)
		// timeout: int?
		//		number of seconds to wait for events  (optional)
		// pollDelay: int?
		//		number of ms to wait in between poll (optional)
		// pollErrorDelay: int?
		//		number of ms to wait after an error before next poll  (optional)
		// contextRoot: String?
		//		context root for requests to event servlet  (optional)		
	},
	=====*/

	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs EventRouter object.
		this._args = kwArgs;
		
		// determine the document that we want to anchor the javascript under
		try {
			this._globalObj = ISOLATION_INIFRAME ? parent : window;
		}
		catch(e) {
			this._globalObj = parent;
		}		
		
		// load javascript if we haven't already				
		this._loadEventRouterJavascript();	
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	
	/*=====
	addListener_event: function(event){},
	=====*/

	addListener: function(/*String*/ eventId, /*addListener_event*/ eventCallback) {
		// summary:
		//		Adds a listener for the requested eventId
		// eventId:
		//		id of events (i.e. provided by Dataset)
		// eventCallback:
		//		callback function to invoke when events are received
		if (this._initialized) {
			if (this._trace.isFinest()) {
				this._trace.finest("adding listener for eventId=" + eventId, "addListener");
			}
			
			this._getGlobalEventRouter().addListener(this, eventId, eventCallback);
		}
		else {
			if (this._trace.isFinest()) {
				this._trace.finest("Waiting on event_router.js to finish loading", "addListener");
			}
					
			setTimeout(dojo.hitch(this, function() {this.addListener(eventId, eventCallback);}), 50);
		}
	},

	removeListener: function(/*String*/ eventId) {
		// summary:
		//		Removes a listener for the requested eventId
		// eventId:
		//		id of events (i.e. provided by Dataset)
		if (this._initialized) {
			return this._getGlobalEventRouter().removeListener(this, eventId);
		}
		
		return 0;
	},

	ping: function() {
		// summary:
		//		Check if the event router is still alive
		console.assert(true);
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////

	_init: function() {	
		// summary:
		//		Initializes the event router
		var globalEventRouter = this._getGlobalEventRouter();
		
		if (globalEventRouter) {
			if (!globalEventRouter.isInitialized()) {
				if (this._trace.isFinest()) {
					this._trace.finest("initializing global event router", "_init");
				}
			
				globalEventRouter.init(this._args);
			}
			
			this._initialized = true;
		}
		else {		
			if (this._trace.isFinest()) {
				this._trace.finest("Waiting on event_router.js to finish loading", "_init");
			}
					
			setTimeout(dojo.hitch(this, function() {this._init();}), 50);
		}	
	},

	_loadEventRouterJavascript: function() {
		// summary:
		//		Load event_router.js using script tag
		var id = "ibm_tivoli_navmodel_event_router";
		var script = this._globalObj.document.getElementById(id);
		if (!script && !this._getGlobalEventRouter()) {
			if (this._trace.isFinest()) {
				this._trace.finest("Inserting event_router.js", "_loadEventRouterJavascript");
			}
			
			// so we can use inside onreadystatechange
			var that = this;
			
			// create the script tag
			script = this._globalObj.document.createElement('script');
			script.id = id;
			script.type = "text/javascript";
			// Dojo 1.7 migration
			// script.src = dojo.moduleUrl("ibm.tivoli.navmodel", "event_router.js").uri;
			script.src = require.toUrl("ibm/tivoli/navmodel")+ "/event_router.js";
			// Check if presence of cacheBust messed up the path to the .js file
			if (dojo.config.cacheBust) {
			    script.src = script.src.slice(0,script.src.indexOf("?")) + "/event_router.js?" + dojo.config.cacheBust;
			}
			script.onload = dojo.hitch(this, this._init);
			script.onreadystatechange = function () { 
				if (this.readyState == 'loaded' || this.readyState == 'complete') {
					that._init();				
				} 
			};
			
			// insert the script tag		
			this._globalObj.document.body.appendChild(script);			
		}
		// script has loaded or in the process of loading
		else {	
			if (this._trace.isFinest()) {
				this._trace.finest("event_router.js is already loaded", "_loadEventRouterJavascript");
			}
			this._init();				
		}		
	},

	_getGlobalEventRouter: function() {
		// summary:
		//		Returns instance of global event router; depending on how javascript was loaded
		var eventRouter = this._globalObj.ibm_tivoli_navmodel_EventRouter;		
		if (!eventRouter) {
			try { 
				eventRouter = ibm_tivoli_navmodel_EventRouter;
			}
			catch (e) {
				// event_router.js has not been loaded
			}
		}
		
		return eventRouter;
	}
});


ibm.tivoli.navmodel.EventRouter.getPageId = function() {	
	// determine the document that we want to anchor the javascript under
	var globalObj = null;
	try {
		globalObj = ISOLATION_INIFRAME ? parent : window;		
	}
	catch(e) {
		globalObj = parent;
	}
	
	if (!globalObj.ibm_tivoli_navmodel_EventRouter_pageId) {
		var now = new Date();
		globalObj.ibm_tivoli_navmodel_EventRouter_pageId = now.getTime().toString();
	}
	
	return globalObj.ibm_tivoli_navmodel_EventRouter_pageId;
};
});

},
'url:dijit/form/templates/HorizontalSlider.html':"<table class=\"dijit dijitReset dijitSlider dijitSliderH\" cellspacing=\"0\" cellpadding=\"0\" border=\"0\" rules=\"none\" data-dojo-attach-event=\"onkeypress:_onKeyPress,onkeyup:_onKeyUp\"\n\trole=\"presentation\"\n\t><tr class=\"dijitReset\"\n\t\t><td class=\"dijitReset\" colspan=\"2\"></td\n\t\t><td data-dojo-attach-point=\"topDecoration\" class=\"dijitReset dijitSliderDecoration dijitSliderDecorationT dijitSliderDecorationH\"></td\n\t\t><td class=\"dijitReset\" colspan=\"2\"></td\n\t></tr\n\t><tr class=\"dijitReset\"\n\t\t><td class=\"dijitReset dijitSliderButtonContainer dijitSliderButtonContainerH\"\n\t\t\t><div class=\"dijitSliderDecrementIconH\" style=\"display:none\" data-dojo-attach-point=\"decrementButton\"><span class=\"dijitSliderButtonInner\">-</span></div\n\t\t></td\n\t\t><td class=\"dijitReset\"\n\t\t\t><div class=\"dijitSliderBar dijitSliderBumper dijitSliderBumperH dijitSliderLeftBumper\" data-dojo-attach-event=\"press:_onClkDecBumper\"></div\n\t\t></td\n\t\t><td class=\"dijitReset\"\n\t\t\t><input data-dojo-attach-point=\"valueNode\" type=\"hidden\" ${!nameAttrSetting}\n\t\t\t/><div class=\"dijitReset dijitSliderBarContainerH\" role=\"presentation\" data-dojo-attach-point=\"sliderBarContainer\"\n\t\t\t\t><div role=\"presentation\" data-dojo-attach-point=\"progressBar\" class=\"dijitSliderBar dijitSliderBarH dijitSliderProgressBar dijitSliderProgressBarH\" data-dojo-attach-event=\"press:_onBarClick\"\n\t\t\t\t\t><div class=\"dijitSliderMoveable dijitSliderMoveableH\"\n\t\t\t\t\t\t><div data-dojo-attach-point=\"sliderHandle,focusNode\" class=\"dijitSliderImageHandle dijitSliderImageHandleH\" data-dojo-attach-event=\"press:_onHandleClick\" role=\"slider\" valuemin=\"${minimum}\" valuemax=\"${maximum}\"></div\n\t\t\t\t\t></div\n\t\t\t\t></div\n\t\t\t\t><div role=\"presentation\" data-dojo-attach-point=\"remainingBar\" class=\"dijitSliderBar dijitSliderBarH dijitSliderRemainingBar dijitSliderRemainingBarH\" data-dojo-attach-event=\"press:_onBarClick\"></div\n\t\t\t></div\n\t\t></td\n\t\t><td class=\"dijitReset\"\n\t\t\t><div class=\"dijitSliderBar dijitSliderBumper dijitSliderBumperH dijitSliderRightBumper\" data-dojo-attach-event=\"press:_onClkIncBumper\"></div\n\t\t></td\n\t\t><td class=\"dijitReset dijitSliderButtonContainer dijitSliderButtonContainerH\"\n\t\t\t><div class=\"dijitSliderIncrementIconH\" style=\"display:none\" data-dojo-attach-point=\"incrementButton\"><span class=\"dijitSliderButtonInner\">+</span></div\n\t\t></td\n\t></tr\n\t><tr class=\"dijitReset\"\n\t\t><td class=\"dijitReset\" colspan=\"2\"></td\n\t\t><td data-dojo-attach-point=\"containerNode,bottomDecoration\" class=\"dijitReset dijitSliderDecoration dijitSliderDecorationB dijitSliderDecorationH\"></td\n\t\t><td class=\"dijitReset\" colspan=\"2\"></td\n\t></tr\n></table>\n",
'dojoe/treetable/_ConfigureColumnsDialog':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/i18n!dojoe/treetable/nls/TreeTable","dojo/require!dijit/_Widget,dijit/_Templated,dijit/Dialog,dijit/Toolbar,dijit/form/Form,dijit/form/Button,dijit/form/CheckBox,dijit/layout/TabContainer,dijit/layout/BorderContainer,dijit/layout/ContentPane,dojo/data/ItemFileWriteStore,dijit/layout/ContentPane,dojox/grid/EnhancedGrid,dojox/grid/enhanced/plugins/NestedSorting,dijit/_Container,dojo/i18n,dojox/collections/ArrayList"], function(dijit,dojo,dojox){
dojo.provide("dojoe.treetable._ConfigureColumnsDialog");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Dialog");
dojo.require("dijit.Toolbar");
dojo.require("dijit.form.Form");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.CheckBox");
dojo.require("dijit.layout.TabContainer");
dojo.require("dijit.layout.BorderContainer");
dojo.require("dijit.layout.ContentPane");

dojo.require("dojo.data.ItemFileWriteStore");
dojo.require("dijit.layout.ContentPane");
dojo.require("dojox.grid.EnhancedGrid");
dojo.require("dojox.grid.enhanced.plugins.NestedSorting");
dojo.require("dijit._Container");
dojo.require("dojo.i18n");
dojo.require("dojox.collections.ArrayList");

// NLS
dojo.requireLocalization("dojoe.treetable", "TreeTable");

// define the widget class
dojo.declare ("dojoe.treetable._ConfigureColumnsDialog", [ dijit._Widget, dijit._Templated ],{
		// summary: 		This class handles all UI rendering for configure column dialog options.
		// description: ConfigureColumnsDialog is a widget to control all the operations  
        //              on the dialog.
		// tags: private
	
		widgetsInTemplate: true,
		templateString: dojo.cache("dojoe.treetable", "templates/ConfigureColumnsDialog.html", "<div>\r\n\t<div dojoType=\"dijit.Dialog\" dojoAttachPoint=\"dialog\"\ttitle=\"${resources_.CONFIGURE_TREETABLE}\" tabindex=0 role=\"dialog\">\r\n\t\t<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" role=\"presentation\" style=\"width:100%height:100%\">\r\n\t\t\t<tr height=\"20%\" style=\"vertical-align:top;\"><td><label>${resources_.COLUMNS_VISIBLE}</label></td><tr>\r\n\t\t\t<tr><td>\r\n\t\t\t\t<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" role=\"presentation\">\r\n\t\t\t\t\t<tbody>\r\n\t\t\t\t\t\t<tr><td>\r\n\t\t\t\t\t\t\t\t<span dojoAttachPoint=\"selectAllButton\" role=\"button\">\r\n\t\t\t\t\t\t\t</td>\r\n\t\t\t\t\t\t\t<td>\r\n\t\t\t\t\t\t\t\t<span dojoAttachPoint=\"deselectAllButton\" role=\"button\">\r\n\t\t\t\t\t\t\t</td>\r\n\t\t\t\t\t\t</tr>\r\n\t\t\t\t\t</tbody>\r\n\t\t\t\t</table>\r\n\t\t\t</td></tr>\r\n\t\t\t<tr><td>\r\n\t\t\t\t<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" width=\"100%\"  role=\"presentation\">\r\n\t\t\t\t\t<tbody>\r\n\t\t\t\t\t\t<tr valign=\"center\">\r\n\t\t\t\t\t\t\t<td width=\"90%\" height=\"100%\">\t\t\t\t\t\r\n\t\t\t\t\t\t\t\t<div id=\"${id}tabs\" dojoType=\"dijit.layout.BorderContainer\" class=\"dialog\" style=\"height:200px;width:260px;\">\r\n\t\t\t\t\t\t\t\t\t<div dojoType=\"dijit.layout.ContentPane\" region=\"top\" >\t\t\t\t\r\n\t\t\t\t\t\t\t\t\t<div  dojoAttachPoint=\"enhancedGridNode\" style=\"height:170px;width:200px;\" role=\"grid\"></div>\r\n\t\t\t\t\t\t\t\t\t</div>\r\n\t\t\t\t\t\t\t\t</div>\r\n\t\t\t\t\t\t\t</td>\r\n\t\t\t\t\t\t\t<td width=\"10%\" height=\"100%\">\r\n\t\t\t\t\t\t\t\t<table role=\"presentation\">\r\n\t\t\t\t\t\t\t\t\t<tr><td><span dojoAttachPoint=\"upButton\" role=\"button\"></td></tr>  \r\n\t\t\t\t\t\t\t\t\t<tr><td><span dojoAttachPoint=\"downButton\" role=\"button\"></td></tr> \r\n\t\t\t\t\t\t\t\t</table>\r\n\t\t\t\t\t\t</td></tr>\t\r\n\t\t\t\t\t</tbody>\t\t\r\n\t\t\t\t</table>\t\t\t\t\t\t\t\r\n\t\t\t</td></tr>\r\n\t\t\t<tr><td>\r\n\t\t\t\t<button value=\"${resources_.OK}\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:onOK,onKeypress:onOK\" role=\"button\">${resources_.OK}</button>\r\n\t\t\t\t&nbsp;&nbsp;<button value=\"${resources_.CANCEL}\" dojoType=\"dijit.form.Button\" dojoAttachPoint=\"cancelButton\" dojoAttachEvent=\"onClick:onCancel,onKeypress:onCancel\" role=\"button\">${resources_.CANCEL}</button>\r\n\t\t\t</td></tr>\r\n\t\t</table>\r\n\t</div>\r\n</div>\r\n"),
		optionsWidget_: null,
		grid: null,
		indirectSelection: null,
		enableColumnReorder:null,
		enhancedGrid: null,
		columndata:null,
		
		constructor: function() {
			this.resources_ = dojo.i18n.getLocalization("dojoe.treetable", "TreeTable", this.lang);
		},
		
		postCreate: function() { 
		//summary: inherited method from the _Widget
		//description: Instantiate the TreeTableOptions for the the operations of configurecolumn.
			this.inherited ("postCreate", arguments);
			var columns=this.grid.layout.cells;
			var names=new Array();
			for (var i=0 in columns) {
				var column=columns[i];
				//Satanik:06Apr2011 checking for indirectSelection column
				var size=dojo.trim(column.name).length;
				if(size > 0){
				//Satanik:06Apr2011 checking for expandable column
					if(!column.isCollapsable){
						names.push(column);
					}
				}					
			}				
		    columndata = {
				identifier: 'id',
				label: 'id',
				items: []
			};
			var len = names.length;
			for(var i=0; i < names.length; ++i){
				//var row ={id : (i+1), 'name': names[i].name, 'hidden':!names[i].hidden, 'index': i} ;
				var row ={id : (i+1), 'name': names[i].name, 'hidden':!names[i].hidden} ;
				columndata.items.push(row);
			}
			var grid_layout = [{
				rows:[
					//22283: Configure Options dialog, IE - 'Visible' column should be a bit wider
					//Bobby: 20Jun2011, changing 15% & 85% width to 30% & 70%
					{ field: "hidden", width: "30%", name: this.resources_.VISIBLE, editable:true, cellType: dojox.grid.cells.Bool},
					//{ field: "index", width: "auto", name: 'Index'},
					{ field: "name", width: "70%", name: this.resources_.COLUMNS}					
				]
			}];
			var pluginsForGrid = {};	
			var itemStore = new dojo.data.ItemFileWriteStore({data:columndata});			
			var gridParams ={
				store : itemStore,
				structure : grid_layout,
				plugins : pluginsForGrid,
				selectionMode:'single',
				canSort: function(){return false;},
				keepSelection: false
			};
			this.enhancedGrid = new dojox.grid.EnhancedGrid(gridParams, this.enhancedGridNode);
			dojo.connect(this, "resize",this.enhancedGrid, "resize");
			
			var selectIcon = new dijit.form.Button({
				id: this.id+"selectAllIcon",
				showLabel: false,
				iconClass: "selectAllIcon",
				label: this.resources_.SELECT_ALL
			},this.selectAllButton);
				
			selectIcon.onClick = dojo.hitch (this, "onSelectAll");
			selectIcon.onKeypress = dojo.hitch (this, "onSelectAll");		
			
			var deSelectIcon = new dijit.form.Button({
				id: this.id+"deselectAllIcon",
				showLabel: false,
				iconClass: "deselectAllIcon",
				label: this.resources_.DESELECT_ALL
			},this.deselectAllButton);
				
			deSelectIcon.onClick = dojo.hitch (this, "onDeselectAll");
			deSelectIcon.onKeypress = dojo.hitch (this, "onDeselectAll");
				
			//this.enhancedGrid.setSortIndex('1', true);
			//Satanik:25May2011: changes made to address ILMT requiremnt 19890
			if(this.enableColumnReorder){
				var upIcon = new dijit.form.Button({
					id: this.id+"upIcon",
					showLabel: false,
					iconClass: "upIcon",
					label: this.resources_.UP
				},this.upButton);
					
				upIcon.onClick = dojo.hitch (this, "onUp");
				upIcon.onKeypress = dojo.hitch (this, "onUp");		
				
				var downIcon = new dijit.form.Button({
					id: this.id+"downIcon",
					showLabel: false,
					iconClass: "downIcon",
					label: this.resources_.DOWN
				},this.downButton);
					
				downIcon.onClick = dojo.hitch (this, "onDown");
				downIcon.onKeypress = dojo.hitch (this, "onDown");
			}else {
				dojo.style(this.upButton.parentNode, "width", "0px");
				dojo.style(this.downButton.parentNode, "width", "0px");
			}
			/*LS - 12/11/2011 - Opening and closing of Configure columns dialog results in dijit.registry to grow */
			//this change was made to make sure destroy is called even when dialog's close button is hit
			this.connect(this.dialog, "onHide",dojo.hitch (this, this.kill));
			
		},		
		
		onSelectAll: function(){
		//summary: selects all the column shown in the ConfigureColumnsDialog		
			var rowCount = this.enhancedGrid.rowCount;
			for (var i = 0; i < rowCount; i++) {
				columndata.items[i].hidden[0]=true;
			}
			this.enhancedGrid._refresh();	
		},	
		
		onDeselectAll: function(){		
		//summary: deselects all the column shown in the ConfigureColumnsDialog
			var rowCount = this.enhancedGrid.rowCount;
			for (var i = 0; i < rowCount; i++) {
				columndata.items[i].hidden[0]=false;
			}
			this.enhancedGrid._refresh();
		},
		
		/*Satanik:09May2011: This is not required any more*/
		/*Satanik:12May2011: This code is needed for the up/down arrow with current impl will review and refactor later */
		_reorderColumn: function(indirectSelection){
			var rowCount = this.enhancedGrid.rowCount;
			var columns=this.grid.structure;
			var reorderedcolumn=new Array();
			if(this.indirectSelection){
				reorderedcolumn[0]=columns[0];
				reorderedcolumn[1]=columns[1];
				for (var i = 0; i < rowCount; i++) {
					var selectedrow=this.enhancedGrid.getItem(i);
					for (var j = 0; j < rowCount+2; j++) {
						if(selectedrow.name[0] === columns[j].name){
							reorderedcolumn[i+2]=columns[j];
						}
					}
				}
			}else {
				reorderedcolumn[0]=columns[0];
				for (var i = 0; i < rowCount; i++) {
					var selectedrow=this.enhancedGrid.getItem(i);
					for (var j = 0; j < rowCount+1; j++) {
						if(selectedrow.name[0] === columns[j].name){
							reorderedcolumn[i+1]=columns[j];
						}
					}
				}
			}
			this.grid.setStructure(reorderedcolumn);
			//Satanik:12May2011: ILMT requested this to be removed as this is not required.
			//this.grid._refresh();
		},
		
		onOK: function () {
			//this.optionsWidget_.onSave();
			var rowCount = this.enhancedGrid.rowCount;
			this._reorderColumn();
			//console.dir(columndata.items);
			if(this.indirectSelection){
				for (var i = 0; i < rowCount; i++) {
						//if(this.enhancedGrid.selection.selected[i] !== true){
						if(columndata.items[i].hidden[0] == false){
							this.grid.layout.setColumnVisibility(i+2, false);
						} else{
							this.grid.layout.setColumnVisibility(i+2, true);
						}
				}
			}else{
				for (var i = 0; i < rowCount; i++) {
						//if(this.enhancedGrid.selection.selected[i] !== false){
						if(columndata.items[i].hidden[0] == false){
							this.grid.layout.setColumnVisibility(i+1, false);
						} else{
							this.grid.layout.setColumnVisibility(i+1, true);
						}
				}
			}
			this.hide();
		},
		
		onCancel: function () {
			this.hide();
		},		
		
		clearSelection: function () {
			this.enhancedGrid.selection.clear();
		},			
		
		onUp: function () {
			//console.log("onUp");
			//console.dir(this.enhancedGrid.selection.getSelected());
			//console.log("selectedIndex:"+this.enhancedGrid.selection.selectedIndex);
			//console.dir(columndata.items);
			var selectedIndex = this.enhancedGrid.selection.selectedIndex;
			if(selectedIndex <= 0) return;
			var itemsNew = new Array();
			//console.log("columndata.items.length:"+columndata.items.length);
			if (selectedIndex-1 >= 0)
				itemsNew = itemsNew.concat(columndata.items.slice(0,selectedIndex-1));
			itemsNew = itemsNew.concat(this.enhancedGrid.selection.getSelected());
			if (selectedIndex-1 >= 0)
				itemsNew.push(columndata.items[selectedIndex-1]);				
			itemsNew = itemsNew.concat(columndata.items.slice(selectedIndex+1, columndata.items.length));				
			this.enhancedGrid.store.close();
			columndata.items = itemsNew;
			//console.dir(columndata.items);
			this.enhancedGrid.store = new dojo.data.ItemFileWriteStore({data:columndata});
			this.enhancedGrid.selection.clear();
			//24425: In TreeTable Configure Columns popup, A column disappears from the list we try to change the position of column.
			//Satanik:13Jul11 - Selection is not correctly retained on refresh in latest grid, so doing after refresh		
			//this.enhancedGrid.selection.addToSelection(selectedIndex-1);
			this.enhancedGrid._refresh();
			this.enhancedGrid.selection.addToSelection(selectedIndex-1);
		},	

		onDown: function () {
			//console.log("onDown");
			var selectedIndex = this.enhancedGrid.selection.selectedIndex;
			if(selectedIndex < 0) return;
			if(selectedIndex == columndata.items.length-1) return;
			var itemsNew = new Array();
			//console.log("selectedIndex:"+selectedIndex);
			//console.log("columndata.items.length:"+columndata.items.length);
			//console.dir(columndata.items);
			itemsNew = itemsNew.concat(columndata.items.slice(0,selectedIndex));
			if(columndata.items[selectedIndex+1] !== 'undefined')
				itemsNew.push(columndata.items[selectedIndex+1]);			
			itemsNew = itemsNew.concat(this.enhancedGrid.selection.getSelected());
			if(columndata.items[selectedIndex+2] !== 'undefined')
				itemsNew = itemsNew.concat(columndata.items.slice(selectedIndex+2, columndata.items.length));				
			this.enhancedGrid.store.close();
			columndata.items = itemsNew;
			//console.dir(columndata.items);
			this.enhancedGrid.store = new dojo.data.ItemFileWriteStore({data:columndata});
			this.enhancedGrid.selection.clear();
			//24425: In TreeTable Configure Columns popup, A column disappears from the list we try to change the position of column.
			//Satanik:13Jul11  - Selection is not correctly retained on refresh in latest grid, so doing after refresh
			//this.enhancedGrid.selection.addToSelection(selectedIndex+1);
			this.enhancedGrid._refresh();			
			this.enhancedGrid.selection.addToSelection(selectedIndex+1);
		},			
		
		_selectColumns: function(){
			var rowCount = this.enhancedGrid.rowCount;
			for (var i = 0; i < rowCount; i++) {
				var selectedrow=this.enhancedGrid.getItem(i);
				if(selectedrow.hidden[0] === false){
					//this.enhancedGrid.rowSelectCell.toggleRow(selectedrow, true);
					this.enhancedGrid.selection.addToSelection(i);
				}
			}    
		},
		
		show: function () {
			this.dialog.show();
			this.enhancedGrid.startup();
			//this._selectColumns();
			dijit.byId (this.id + "tabs").resize();
		},
		
		hide: function () {
			/*LS - 12/11/2011 - Opening and closing of Configure columns dialog results in dijit.registry to grow */
			//just hide the dialog the destroy would take place with a connect to onhide of dijit.dialog 
			//this change was made to make sure destroy is called even when dialog's close button is hit
			this.dialog.hide();
		},	
		
		kill: function () {
		/*LS - 12/11/2011 - Opening and closing of Configure columns dialog results in dijit.registry to grow */
		//call destroy to completely destroy the widget...
			/*var id = this.id;
			var dId=this.dialog.id;
			this.dialog.destroyRecursive();
			dijit.registry.remove (dId);
			this.destroyRecursive();
			dijit.registry.remove (id);*/
			setTimeout (dojo.hitch (this, this.destroy), 100);
		},
		
		getTableOptions: function () {
			return this.optionsWidget_;
		},
		
		destroy: function(){
			// summary:  Destroys the toolbar.
			// tags: public	
			//memory leak issue: custom destroy method to take care of removing all widgets					
			try{
				delete this.resources_;
				//this is for destroying child templated widgets 
				//this should happen by itself but need to figure out why dijit._Templated is not taking care of it...
				/*LS - 12/11/2011 - Opening and closing of Configure columns dialog results in dijit.registry to grow */
				//destroy the dialog and bordercontainer separately as template is not finding it properly
				//this must be a bug with bordercontainer in template it needs to be destroyed separately
				var dijitDialog = dijit.byId(this.dialog);
				var borderContID = this.id +'tabs';
				var borderContainer = dijit.byId(borderContID );
				if(borderContainer)borderContainer.destroyRecursive();
				if(dijitDialog)dijitDialog.destroyRecursive();
				var widgets = dijit.findWidgets(this.domNode);
				dojo.forEach(widgets, function(w) {
					if(w){
						if(w.destroyRecursive){
							w.destroyRecursive();
						} else if(w.destroy){
							w.destroy();
						}
					}
				});
				// this is for widgets added as child to this widget and their children..
				//not present in template
				var widgetArr = this.getChildren();
				dojo.forEach(widgetArr, function (child) {
					if(child){
						if(child.destroyRecursive){
							child.destroyRecursive();
						} else if(child.destroy){
							child.destroy();
						}
					}
				});
				this.inherited(arguments);
			}catch(ex){
				console.warn("Widget destroy failed: "+this.id+", ex="+ex.name+":"+ex.message);
			}
		}
	}
);
});

},
'dijit/place':function(){
define([
	"dojo/_base/array", // array.forEach array.map array.some
	"dojo/dom-geometry", // domGeometry.getMarginBox domGeometry.position
	"dojo/dom-style", // domStyle.getComputedStyle
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/window", // win.body
	"dojo/window", // winUtils.getBox
	"."	// dijit (defining dijit.place to match API doc)
], function(array, domGeometry, domStyle, kernel, win, winUtils, dijit){

	// module:
	//		dijit/place
	// summary:
	//		Code to place a popup relative to another node


	function _place(/*DomNode*/ node, choices, layoutNode, aroundNodeCoords){
		// summary:
		//		Given a list of spots to put node, put it at the first spot where it fits,
		//		of if it doesn't fit anywhere then the place with the least overflow
		// choices: Array
		//		Array of elements like: {corner: 'TL', pos: {x: 10, y: 20} }
		//		Above example says to put the top-left corner of the node at (10,20)
		// layoutNode: Function(node, aroundNodeCorner, nodeCorner, size)
		//		for things like tooltip, they are displayed differently (and have different dimensions)
		//		based on their orientation relative to the parent.	 This adjusts the popup based on orientation.
		//		It also passes in the available size for the popup, which is useful for tooltips to
		//		tell them that their width is limited to a certain amount.	 layoutNode() may return a value expressing
		//		how much the popup had to be modified to fit into the available space.	 This is used to determine
		//		what the best placement is.
		// aroundNodeCoords: Object
		//		Size of aroundNode, ex: {w: 200, h: 50}

		// get {x: 10, y: 10, w: 100, h:100} type obj representing position of
		// viewport over document
		var view = winUtils.getBox();

		// This won't work if the node is inside a <div style="position: relative">,
		// so reattach it to win.doc.body.	 (Otherwise, the positioning will be wrong
		// and also it might get cutoff)
		if(!node.parentNode || String(node.parentNode.tagName).toLowerCase() != "body"){
			win.body().appendChild(node);
		}

		var best = null;
		array.some(choices, function(choice){
			var corner = choice.corner;
			var pos = choice.pos;
			var overflow = 0;

			// calculate amount of space available given specified position of node
			var spaceAvailable = {
				w: {
					'L': view.l + view.w - pos.x,
					'R': pos.x - view.l,
					'M': view.w
				   }[corner.charAt(1)],
				h: {
					'T': view.t + view.h - pos.y,
					'B': pos.y - view.t,
					'M': view.h
				   }[corner.charAt(0)]
			};

			// configure node to be displayed in given position relative to button
			// (need to do this in order to get an accurate size for the node, because
			// a tooltip's size changes based on position, due to triangle)
			if(layoutNode){
				var res = layoutNode(node, choice.aroundCorner, corner, spaceAvailable, aroundNodeCoords);
				overflow = typeof res == "undefined" ? 0 : res;
			}

			// get node's size
			var style = node.style;
			var oldDisplay = style.display;
			var oldVis = style.visibility;
			if(style.display == "none"){
				style.visibility = "hidden";
				style.display = "";
			}
			var mb = domGeometry. getMarginBox(node);
			style.display = oldDisplay;
			style.visibility = oldVis;

			// coordinates and size of node with specified corner placed at pos,
			// and clipped by viewport
			var
				startXpos = {
					'L': pos.x,
					'R': pos.x - mb.w,
					'M': Math.max(view.l, Math.min(view.l + view.w, pos.x + (mb.w >> 1)) - mb.w) // M orientation is more flexible
				}[corner.charAt(1)],
				startYpos = {
					'T': pos.y,
					'B': pos.y - mb.h,
					'M': Math.max(view.t, Math.min(view.t + view.h, pos.y + (mb.h >> 1)) - mb.h)
				}[corner.charAt(0)],
				startX = Math.max(view.l, startXpos),
				startY = Math.max(view.t, startYpos),
				endX = Math.min(view.l + view.w, startXpos + mb.w),
				endY = Math.min(view.t + view.h, startYpos + mb.h),
				width = endX - startX,
				height = endY - startY;

			overflow += (mb.w - width) + (mb.h - height);

			if(best == null || overflow < best.overflow){
				best = {
					corner: corner,
					aroundCorner: choice.aroundCorner,
					x: startX,
					y: startY,
					w: width,
					h: height,
					overflow: overflow,
					spaceAvailable: spaceAvailable
				};
			}

			return !overflow;
		});

		// In case the best position is not the last one we checked, need to call
		// layoutNode() again.
		if(best.overflow && layoutNode){
			layoutNode(node, best.aroundCorner, best.corner, best.spaceAvailable, aroundNodeCoords);
		}

		// And then position the node.  Do this last, after the layoutNode() above
		// has sized the node, due to browser quirks when the viewport is scrolled
		// (specifically that a Tooltip will shrink to fit as though the window was
		// scrolled to the left).
		//
		// In RTL mode, set style.right rather than style.left so in the common case,
		// window resizes move the popup along with the aroundNode.
		var l = domGeometry.isBodyLtr(),
			s = node.style;
		s.top = best.y + "px";
		s[l ? "left" : "right"] = (l ? best.x : view.w - best.x - best.w) + "px";
		s[l ? "right" : "left"] = "auto";	// needed for FF or else tooltip goes to far left

		return best;
	}

	/*=====
	dijit.place.__Position = function(){
		// x: Integer
		//		horizontal coordinate in pixels, relative to document body
		// y: Integer
		//		vertical coordinate in pixels, relative to document body

		this.x = x;
		this.y = y;
	};
	=====*/

	/*=====
	dijit.place.__Rectangle = function(){
		// x: Integer
		//		horizontal offset in pixels, relative to document body
		// y: Integer
		//		vertical offset in pixels, relative to document body
		// w: Integer
		//		width in pixels.   Can also be specified as "width" for backwards-compatibility.
		// h: Integer
		//		height in pixels.   Can also be specified as "height" from backwards-compatibility.

		this.x = x;
		this.y = y;
		this.w = w;
		this.h = h;
	};
	=====*/

	return (dijit.place = {
		// summary:
		//		Code to place a DOMNode relative to another DOMNode.
		//		Load using require(["dijit/place"], function(place){ ... }).

		at: function(node, pos, corners, padding){
			// summary:
			//		Positions one of the node's corners at specified position
			//		such that node is fully visible in viewport.
			// description:
			//		NOTE: node is assumed to be absolutely or relatively positioned.
			// node: DOMNode
			//		The node to position
			// pos: dijit.place.__Position
			//		Object like {x: 10, y: 20}
			// corners: String[]
			//		Array of Strings representing order to try corners in, like ["TR", "BL"].
			//		Possible values are:
			//			* "BL" - bottom left
			//			* "BR" - bottom right
			//			* "TL" - top left
			//			* "TR" - top right
			// padding: dijit.place.__Position?
			//		optional param to set padding, to put some buffer around the element you want to position.
			// example:
			//		Try to place node's top right corner at (10,20).
			//		If that makes node go (partially) off screen, then try placing
			//		bottom left corner at (10,20).
			//	|	place(node, {x: 10, y: 20}, ["TR", "BL"])
			var choices = array.map(corners, function(corner){
				var c = { corner: corner, pos: {x:pos.x,y:pos.y} };
				if(padding){
					c.pos.x += corner.charAt(1) == 'L' ? padding.x : -padding.x;
					c.pos.y += corner.charAt(0) == 'T' ? padding.y : -padding.y;
				}
				return c;
			});

			return _place(node, choices);
		},

		around: function(
			/*DomNode*/		node,
			/*DomNode || dijit.place.__Rectangle*/ anchor,
			/*String[]*/	positions,
			/*Boolean*/		leftToRight,
			/*Function?*/	layoutNode){

			// summary:
			//		Position node adjacent or kitty-corner to anchor
			//		such that it's fully visible in viewport.
			//
			// description:
			//		Place node such that corner of node touches a corner of
			//		aroundNode, and that node is fully visible.
			//
			// anchor:
			//		Either a DOMNode or a __Rectangle (object with x, y, width, height).
			//
			// positions:
			//		Ordered list of positions to try matching up.
			//			* before: places drop down to the left of the anchor node/widget, or to the right in the case
			//				of RTL scripts like Hebrew and Arabic; aligns either the top of the drop down
			//				with the top of the anchor, or the bottom of the drop down with bottom of the anchor.
			//			* after: places drop down to the right of the anchor node/widget, or to the left in the case
			//				of RTL scripts like Hebrew and Arabic; aligns either the top of the drop down
			//				with the top of the anchor, or the bottom of the drop down with bottom of the anchor.
			//			* before-centered: centers drop down to the left of the anchor node/widget, or to the right
			//				 in the case of RTL scripts like Hebrew and Arabic
			//			* after-centered: centers drop down to the right of the anchor node/widget, or to the left
			//				 in the case of RTL scripts like Hebrew and Arabic
			//			* above-centered: drop down is centered above anchor node
			//			* above: drop down goes above anchor node, left sides aligned
			//			* above-alt: drop down goes above anchor node, right sides aligned
			//			* below-centered: drop down is centered above anchor node
			//			* below: drop down goes below anchor node
			//			* below-alt: drop down goes below anchor node, right sides aligned
			//
			// layoutNode: Function(node, aroundNodeCorner, nodeCorner)
			//		For things like tooltip, they are displayed differently (and have different dimensions)
			//		based on their orientation relative to the parent.	 This adjusts the popup based on orientation.
			//
			// leftToRight:
			//		True if widget is LTR, false if widget is RTL.   Affects the behavior of "above" and "below"
			//		positions slightly.
			//
			// example:
			//	|	placeAroundNode(node, aroundNode, {'BL':'TL', 'TR':'BR'});
			//		This will try to position node such that node's top-left corner is at the same position
			//		as the bottom left corner of the aroundNode (ie, put node below
			//		aroundNode, with left edges aligned).	If that fails it will try to put
			// 		the bottom-right corner of node where the top right corner of aroundNode is
			//		(ie, put node above aroundNode, with right edges aligned)
			//

			// if around is a DOMNode (or DOMNode id), convert to coordinates
			var aroundNodePos = (typeof anchor == "string" || "offsetWidth" in anchor)
				? domGeometry.position(anchor, true)
				: anchor;

			// Compute position and size of visible part of anchor (it may be partially hidden by ancestor nodes w/scrollbars)
			if(anchor.parentNode){
				// ignore nodes between position:relative and position:absolute
				var sawPosAbsolute = domStyle.getComputedStyle(anchor).position == "absolute";
				var parent = anchor.parentNode;
				while(parent && parent.nodeType == 1 && parent.nodeName != "BODY"){  //ignoring the body will help performance
					var parentPos = domGeometry.position(parent, true),
						pcs = domStyle.getComputedStyle(parent);
					if(/relative|absolute/.test(pcs.position)){
						sawPosAbsolute = false;
					}
					if(!sawPosAbsolute && /hidden|auto|scroll/.test(pcs.overflow)){
						var bottomYCoord = Math.min(aroundNodePos.y + aroundNodePos.h, parentPos.y + parentPos.h);
						var rightXCoord = Math.min(aroundNodePos.x + aroundNodePos.w, parentPos.x + parentPos.w);
						aroundNodePos.x = Math.max(aroundNodePos.x, parentPos.x);
						aroundNodePos.y = Math.max(aroundNodePos.y, parentPos.y);
						aroundNodePos.h = bottomYCoord - aroundNodePos.y;
						aroundNodePos.w = rightXCoord - aroundNodePos.x;
					}
					if(pcs.position == "absolute"){
						sawPosAbsolute = true;
					}
					parent = parent.parentNode;
				}
			}


			var x = aroundNodePos.x,
				y = aroundNodePos.y,
				width = "w" in aroundNodePos ? aroundNodePos.w : (aroundNodePos.w = aroundNodePos.width),
				height = "h" in aroundNodePos ? aroundNodePos.h : (kernel.deprecated("place.around: dijit.place.__Rectangle: { x:"+x+", y:"+y+", height:"+aroundNodePos.height+", width:"+width+" } has been deprecated.  Please use { x:"+x+", y:"+y+", h:"+aroundNodePos.height+", w:"+width+" }", "", "2.0"), aroundNodePos.h = aroundNodePos.height);

			// Convert positions arguments into choices argument for _place()
			var choices = [];
			function push(aroundCorner, corner){
				choices.push({
					aroundCorner: aroundCorner,
					corner: corner,
					pos: {
						x: {
							'L': x,
							'R': x + width,
							'M': x + (width >> 1)
						   }[aroundCorner.charAt(1)],
						y: {
							'T': y,
							'B': y + height,
							'M': y + (height >> 1)
						   }[aroundCorner.charAt(0)]
					}
				})
			}
			array.forEach(positions, function(pos){
				var ltr =  leftToRight;
				switch(pos){
					case "above-centered":
						push("TM", "BM");
						break;
					case "below-centered":
						push("BM", "TM");
						break;
					case "after-centered":
						ltr = !ltr;
						// fall through
					case "before-centered":
						push(ltr ? "ML" : "MR", ltr ? "MR" : "ML");
						break;
					case "after":
						ltr = !ltr;
						// fall through
					case "before":
						push(ltr ? "TL" : "TR", ltr ? "TR" : "TL");
						push(ltr ? "BL" : "BR", ltr ? "BR" : "BL");
						break;
					case "below-alt":
						ltr = !ltr;
						// fall through
					case "below":
						// first try to align left borders, next try to align right borders (or reverse for RTL mode)
						push(ltr ? "BL" : "BR", ltr ? "TL" : "TR");
						push(ltr ? "BR" : "BL", ltr ? "TR" : "TL");
						break;
					case "above-alt":
						ltr = !ltr;
						// fall through
					case "above":
						// first try to align left borders, next try to align right borders (or reverse for RTL mode)
						push(ltr ? "TL" : "TR", ltr ? "BL" : "BR");
						push(ltr ? "TR" : "TL", ltr ? "BR" : "BL");
						break;
					default:
						// To assist dijit/_base/place, accept arguments of type {aroundCorner: "BL", corner: "TL"}.
						// Not meant to be used directly.
						push(pos.aroundCorner, pos.corner);
				}
			});

			var position = _place(node, choices, layoutNode, {w: width, h: height});
			position.aroundNodePos = aroundNodePos;

			return position;
		}
	});
});

},
'dijit/_editor/_Plugin':function(){
define([
	"dojo/_base/connect", // connect.connect
	"dojo/_base/declare", // declare
	"dojo/_base/lang", // lang.mixin, lang.hitch
	"../form/Button"
], function(connect, declare, lang, Button){

// module:
//		dijit/_editor/_Plugin
// summary:
//		Base class for a "plugin" to the editor, which is usually
//		a single button on the Toolbar and some associated code


var _Plugin = declare("dijit._editor._Plugin", null, {
	// summary:
	//		Base class for a "plugin" to the editor, which is usually
	//		a single button on the Toolbar and some associated code

	constructor: function(/*Object?*/args){
		this.params = args || {};
		lang.mixin(this, this.params);
		this._connects=[];
		this._attrPairNames = {};
	},

	// editor: [const] dijit.Editor
	//		Points to the parent editor
	editor: null,

	// iconClassPrefix: [const] String
	//		The CSS class name for the button node is formed from `iconClassPrefix` and `command`
	iconClassPrefix: "dijitEditorIcon",

	// button: dijit._Widget?
	//		Pointer to `dijit.form.Button` or other widget (ex: `dijit.form.FilteringSelect`)
	//		that is added to the toolbar to control this plugin.
	//		If not specified, will be created on initialization according to `buttonClass`
	button: null,

	// command: String
	//		String like "insertUnorderedList", "outdent", "justifyCenter", etc. that represents an editor command.
	//		Passed to editor.execCommand() if `useDefaultCommand` is true.
	command: "",

	// useDefaultCommand: Boolean
	//		If true, this plugin executes by calling Editor.execCommand() with the argument specified in `command`.
	useDefaultCommand: true,

	// buttonClass: Widget Class
	//		Class of widget (ex: dijit.form.Button or dijit.form.FilteringSelect)
	//		that is added to the toolbar to control this plugin.
	//		This is used to instantiate the button, unless `button` itself is specified directly.
	buttonClass: Button,

	// disabled: Boolean
	//		Flag to indicate if this plugin has been disabled and should do nothing
	//		helps control button state, among other things.  Set via the setter api.
	disabled: false,

	getLabel: function(/*String*/key){
		// summary:
		//		Returns the label to use for the button
		// tags:
		//		private
		return this.editor.commands[key];		// String
	},

	_initButton: function(){
		// summary:
		//		Initialize the button or other widget that will control this plugin.
		//		This code only works for plugins controlling built-in commands in the editor.
		// tags:
		//		protected extension
		if(this.command.length){
			var label = this.getLabel(this.command),
				editor = this.editor,
				className = this.iconClassPrefix+" "+this.iconClassPrefix + this.command.charAt(0).toUpperCase() + this.command.substr(1);
			if(!this.button){
				var props = lang.mixin({
					label: label,
					dir: editor.dir,
					lang: editor.lang,
					showLabel: false,
					iconClass: className,
					dropDown: this.dropDown,
					tabIndex: "-1"
				}, this.params || {});
				this.button = new this.buttonClass(props);
			}
		}
		if(this.get("disabled") && this.button){
			this.button.set("disabled", this.get("disabled"));
		}
	},

	destroy: function(){
		// summary:
		//		Destroy this plugin

		var h;
		while(h = this._connects.pop()){ h.remove(); }
		if(this.dropDown){
			this.dropDown.destroyRecursive();
		}
	},

	connect: function(o, f, tf){
		// summary:
		//		Make a connect.connect() that is automatically disconnected when this plugin is destroyed.
		//		Similar to `dijit._Widget.connect`.
		// tags:
		//		protected
		this._connects.push(connect.connect(o, f, this, tf));
	},

	updateState: function(){
		// summary:
		//		Change state of the plugin to respond to events in the editor.
		// description:
		//		This is called on meaningful events in the editor, such as change of selection
		//		or caret position (but not simple typing of alphanumeric keys).   It gives the
		//		plugin a chance to update the CSS of its button.
		//
		//		For example, the "bold" plugin will highlight/unhighlight the bold button depending on whether the
		//		characters next to the caret are bold or not.
		//
		//		Only makes sense when `useDefaultCommand` is true, as it calls Editor.queryCommandEnabled(`command`).
		var e = this.editor,
			c = this.command,
			checked, enabled;
		if(!e || !e.isLoaded || !c.length){ return; }
		var disabled = this.get("disabled");
		if(this.button){
			try{
				enabled = !disabled && e.queryCommandEnabled(c);
				if(this.enabled !== enabled){
					this.enabled = enabled;
					this.button.set('disabled', !enabled);
				}
				if(typeof this.button.checked == 'boolean'){
					checked = e.queryCommandState(c);
					if(this.checked !== checked){
						this.checked = checked;
						this.button.set('checked', e.queryCommandState(c));
					}
				}
			}catch(e){
				console.log(e); // FIXME: we shouldn't have debug statements in our code.  Log as an error?
			}
		}
	},

	setEditor: function(/*dijit.Editor*/ editor){
		// summary:
		//		Tell the plugin which Editor it is associated with.

		// TODO: refactor code to just pass editor to constructor.

		// FIXME: detach from previous editor!!
		this.editor = editor;

		// FIXME: prevent creating this if we don't need to (i.e., editor can't handle our command)
		this._initButton();

		// Processing for buttons that execute by calling editor.execCommand()
		if(this.button && this.useDefaultCommand){
			if(this.editor.queryCommandAvailable(this.command)){
				this.connect(this.button, "onClick",
					lang.hitch(this.editor, "execCommand", this.command, this.commandArg)
				);
			}else{
				// hide button because editor doesn't support command (due to browser limitations)
				this.button.domNode.style.display = "none";
			}
		}

		this.connect(this.editor, "onNormalizedDisplayChanged", "updateState");
	},

	setToolbar: function(/*dijit.Toolbar*/ toolbar){
		// summary:
		//		Tell the plugin to add it's controller widget (often a button)
		//		to the toolbar.  Does nothing if there is no controller widget.

		// TODO: refactor code to just pass toolbar to constructor.

		if(this.button){
			toolbar.addChild(this.button);
		}
		// console.debug("adding", this.button, "to:", toolbar);
	},

	set: function(/* attribute */ name, /* anything */ value){
		// summary:
		//		Set a property on a plugin
		//	name:
		//		The property to set.
		//	value:
		//		The value to set in the property.
		// description:
		//		Sets named properties on a plugin which may potentially be handled by a
		// 		setter in the plugin.
		// 		For example, if the plugin has a properties "foo"
		//		and "bar" and a method named "_setFooAttr", calling:
		//	|	plugin.set("foo", "Howdy!");
		//		would be equivalent to writing:
		//	|	plugin._setFooAttr("Howdy!");
		//		and:
		//	|	plugin.set("bar", 3);
		//		would be equivalent to writing:
		//	|	plugin.bar = 3;
		//
		//	set() may also be called with a hash of name/value pairs, ex:
		//	|	plugin.set({
		//	|		foo: "Howdy",
		//	|		bar: 3
		//	|	})
		//	This is equivalent to calling set(foo, "Howdy") and set(bar, 3)
		if(typeof name === "object"){
			for(var x in name){
				this.set(x, name[x]);
	}
			return this;
		}
		var names = this._getAttrNames(name);
		if(this[names.s]){
			// use the explicit setter
			var result = this[names.s].apply(this, Array.prototype.slice.call(arguments, 1));
		}else{
			this._set(name, value);
		}
		return result || this;
	},

	get: function(name){
		// summary:
		//		Get a property from a plugin.
		//	name:
		//		The property to get.
		// description:
		//		Get a named property from a plugin. The property may
		//		potentially be retrieved via a getter method. If no getter is defined, this
		// 		just retrieves the object's property.
		// 		For example, if the plugin has a properties "foo"
		//		and "bar" and a method named "_getFooAttr", calling:
		//	|	plugin.get("foo");
		//		would be equivalent to writing:
		//	|	plugin._getFooAttr();
		//		and:
		//	|	plugin.get("bar");
		//		would be equivalent to writing:
		//	|	plugin.bar;
		var names = this._getAttrNames(name);
		return this[names.g] ? this[names.g]() : this[name];
	},

	_setDisabledAttr: function(disabled){
		// summary:
		//		Function to set the plugin state and call updateState to make sure the
		//		button is updated appropriately.
		this.disabled = disabled;
		this.updateState();
	},

	_getAttrNames: function(name){
		// summary:
		//		Helper function for get() and set().
		//		Caches attribute name values so we don't do the string ops every time.
		// tags:
		//		private

		var apn = this._attrPairNames;
		if(apn[name]){ return apn[name]; }
		var uc = name.charAt(0).toUpperCase() + name.substr(1);
		return (apn[name] = {
			s: "_set"+uc+"Attr",
			g: "_get"+uc+"Attr"
		});
	},

	_set: function(/*String*/ name, /*anything*/ value){
		// summary:
		//		Helper function to set new value for specified attribute
		this[name] = value;
	}
});

// Hash mapping plugin name to factory, used for registering plugins
_Plugin.registry = {};

return _Plugin;

});

},
'dojox/grid/cells/dijit':function(){
define([
	"dojo/_base/kernel",
	"../../main",
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/_base/json",
	"dojo/_base/connect",
	"dojo/_base/sniff",
	"dojo/dom",
	"dojo/dom-attr",
	"dojo/dom-construct",
	"dojo/dom-geometry",
	"dojo/data/ItemFileReadStore",
	"dijit/form/DateTextBox",
	"dijit/form/TimeTextBox",
	"dijit/form/ComboBox",
	"dijit/form/CheckBox",
	"dijit/form/TextBox",
	"dijit/form/NumberSpinner",
	"dijit/form/NumberTextBox",
	"dijit/form/CurrencyTextBox",
	"dijit/form/HorizontalSlider",
	"dijit/Editor",
	"../util",
	"./_base"
], function(dojo, dojox, declare, array, lang, json, connect, has, dom, domAttr, domConstruct,
	domGeometry, ItemFileReadStore, DateTextBox, TimeTextBox, ComboBox, CheckBox, TextBox,
	NumberSpinner, NumberTextBox, CurrencyTextBox, HorizontalSlider, Editor, util, BaseCell){
		
// TODO: shouldn't it be the test file's job to require these modules,
// if it is using them?  Most of these modules aren't referenced by this file.
	
	var _Widget = declare("dojox.grid.cells._Widget", BaseCell, {
		widgetClass: TextBox,
		constructor: function(inCell){
			this.widget = null;
			if(typeof this.widgetClass == "string"){
				dojo.deprecated("Passing a string to widgetClass is deprecated", "pass the widget class object instead", "2.0");
				this.widgetClass = lang.getObject(this.widgetClass);
			}
		},
		formatEditing: function(inDatum, inRowIndex){
			this.needFormatNode(inDatum, inRowIndex);
			return "<div></div>";
		},
		getValue: function(inRowIndex){
			return this.widget.get('value');
		},
		_unescapeHTML: function(value){
			return (value && value.replace && this.grid.escapeHTMLInData) ? 
					value.replace(/&lt;/g, '<').replace(/&amp;/g, '&') : value;
		},
		setValue: function(inRowIndex, inValue){
			if(this.widget&&this.widget.set){
				inValue = this._unescapeHTML(inValue);
				//Look for lazy-loading editor and handle it via its deferred.
				if(this.widget.onLoadDeferred){
					var self = this;
					this.widget.onLoadDeferred.addCallback(function(){
						self.widget.set("value",inValue===null?"":inValue);
					});
				}else{
					this.widget.set("value", inValue);
				}
			}else{
				this.inherited(arguments);
			}
		},
		getWidgetProps: function(inDatum){
			return lang.mixin(
				{
					dir: this.dir,
					lang: this.lang
				},
				this.widgetProps||{},
				{
					constraints: lang.mixin({}, this.constraint) || {}, //TODO: really just for ValidationTextBoxes
					value: this._unescapeHTML(inDatum)
				}
			);
		},
		createWidget: function(inNode, inDatum, inRowIndex){
			return new this.widgetClass(this.getWidgetProps(inDatum), inNode);
		},
		attachWidget: function(inNode, inDatum, inRowIndex){
			inNode.appendChild(this.widget.domNode);
			this.setValue(inRowIndex, inDatum);
		},
		formatNode: function(inNode, inDatum, inRowIndex){
			if(!this.widgetClass){
				return inDatum;
			}
			if(!this.widget){
				this.widget = this.createWidget.apply(this, arguments);
			}else{
				this.attachWidget.apply(this, arguments);
			}
			this.sizeWidget.apply(this, arguments);
			this.grid.views.renormalizeRow(inRowIndex);
			this.grid.scroller.rowHeightChanged(inRowIndex, true/*fix #11101*/);
			this.focus();
			return undefined;
		},
		sizeWidget: function(inNode, inDatum, inRowIndex){
			var
				p = this.getNode(inRowIndex),
				box = dojo.contentBox(p);
			dojo.marginBox(this.widget.domNode, {w: box.w});
		},
		focus: function(inRowIndex, inNode){
			if(this.widget){
				setTimeout(lang.hitch(this.widget, function(){
					util.fire(this, "focus");
				}), 0);
			}
		},
		_finish: function(inRowIndex){
			this.inherited(arguments);
			util.removeNode(this.widget.domNode);
			if(has("ie")){
				dom.setSelectable(this.widget.domNode, true);
			}
		}
	});
	_Widget.markupFactory = function(node, cell){
		BaseCell.markupFactory(node, cell);
		var widgetProps = lang.trim(domAttr.get(node, "widgetProps")||"");
		var constraint = lang.trim(domAttr.get(node, "constraint")||"");
		var widgetClass = lang.trim(domAttr.get(node, "widgetClass")||"");
		if(widgetProps){
			cell.widgetProps = json.fromJson(widgetProps);
		}
		if(constraint){
			cell.constraint = json.fromJson(constraint);
		}
		if(widgetClass){
			cell.widgetClass = lang.getObject(widgetClass);
		}
	};

	var ComboBox = declare("dojox.grid.cells.ComboBox", _Widget, {
		widgetClass: ComboBox,
		getWidgetProps: function(inDatum){
			var items=[];
			array.forEach(this.options, function(o){
				items.push({name: o, value: o});
			});
			var store = new ItemFileReadStore({data: {identifier:"name", items: items}});
			return lang.mixin({}, this.widgetProps||{}, {
				value: inDatum,
				store: store
			});
		},
		getValue: function(){
			var e = this.widget;
			// make sure to apply the displayed value
			e.set('displayedValue', e.get('displayedValue'));
			return e.get('value');
		}
	});
	ComboBox.markupFactory = function(node, cell){
		_Widget.markupFactory(node, cell);
		var options = lang.trim(domAttr.get(node, "options")||"");
		if(options){
			var o = options.split(',');
			if(o[0] != options){
				cell.options = o;
			}
		}
	};

	var DateTextBox = declare("dojox.grid.cells.DateTextBox", _Widget, {
		widgetClass: DateTextBox,
		setValue: function(inRowIndex, inValue){
			if(this.widget){
				this.widget.set('value', new Date(inValue));
			}else{
				this.inherited(arguments);
			}
		},
		getWidgetProps: function(inDatum){
			return lang.mixin(this.inherited(arguments), {
				value: new Date(inDatum)
			});
		}
	});
	DateTextBox.markupFactory = function(node, cell){
		_Widget.markupFactory(node, cell);
	};

	var CheckBox = declare("dojox.grid.cells.CheckBox", _Widget, {
		widgetClass: CheckBox,
		getValue: function(){
			return this.widget.checked;
		},
		setValue: function(inRowIndex, inValue){
			if(this.widget&&this.widget.attributeMap.checked){
				this.widget.set("checked", inValue);
			}else{
				this.inherited(arguments);
			}
		},
		sizeWidget: function(inNode, inDatum, inRowIndex){
			return;
		}
	});
	CheckBox.markupFactory = function(node, cell){
		_Widget.markupFactory(node, cell);
	};

	var Editor = declare("dojox.grid.cells.Editor", _Widget, {
		widgetClass: Editor,
		getWidgetProps: function(inDatum){
			return lang.mixin({}, this.widgetProps||{}, {
				height: this.widgetHeight || "100px"
			});
		},
		createWidget: function(inNode, inDatum, inRowIndex){
			// widget needs its value set after creation
			var widget = new this.widgetClass(this.getWidgetProps(inDatum), inNode);
			connect.connect(widget, 'onLoad', lang.hitch(this, 'populateEditor'));
			return widget;
		},
		formatNode: function(inNode, inDatum, inRowIndex){
			this.content = inDatum;
			this.inherited(arguments);
			if(has("mozilla")){
				// FIXME: seem to need to reopen the editor and display the toolbar
				var e = this.widget;
				e.open();
				if(this.widgetToolbar){
					domConstruct.place(e.toolbar.domNode, e.editingArea, "before");
				}
			}
		},
		populateEditor: function(){
			this.widget.set('value', this.content);
			this.widget.placeCursorAtEnd();
		}
	});
	Editor.markupFactory = function(node, cell){
		_Widget.markupFactory(node, cell);
		var h = lang.trim(domAttr.get(node, "widgetHeight")||"");
		if(h){
			if((h != "auto")&&(h.substr(-2) != "em")){
				h = parseInt(h, 10)+"px";
			}
			cell.widgetHeight = h;
		}
	};

	return dojox.grid.cells.dijit;

});
},
'gridx/core/Row':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/Deferred"
], function(declare, lang, Deferred){

	
	return declare(/*===== "gridx.core.Row", =====*/[], {
		// summary:
		//		Represents a row of a grid
		// description:
		//		An instance of this class represents a grid row.
		//		This class should not be directly instantiated by users. It should be returned by grid APIs.

		/*=====
		// id: [readonly] String
		//		The ID of this row
		id: null,

		// grid: [readonly] gridx.Grid
		//		Reference to the grid
		grid: null,

		// model: [readonly] grid.core.model.Model
		//		Reference to this grid model
		model: null,
		=====*/

		

		constructor: function(grid, id){
			this.grid = grid;
			this.model = grid.model;
			this.id = id;
		},

		
		index: function(){
			// summary:
			//		Get the index of this row
			// returns:
			//		The row index
			return this.model.idToIndex(this.id);	//Integer
		},

		parent: function(){
			return this.grid.row(this.model.treePath(this.id).pop(), 1);	//gridx.core.Row
		},

		
		cell: function(column, isId){
			// summary:
			//		Get a cell object in this row
			// column: gridx.core.Column|Integer|String
			//		Column index or column ID or a column object
			// isId: Boolean?
			//		If the column parameter is a numeric ID, set this to true
			// returns:
			//		If the params are valid return the cell object, else return null.
			return this.grid.cell(this, column, isId);	//gridx.core.Cell|null
		},

		
		cells: function(start, count){
			// summary:
			//		Get cells in this row.
			// start: Integer?
			//		The column index of the first cell in the returned array.
			//		If omitted, defaults to 0, so row.cells() gets all the cells.
			// count: Integer?
			//		The number of cells to return.
			//		If omitted, all the cells starting from column 'start' will be returned.
			// returns:
			//		An array of cells in this row
			var t = this,
				g = t.grid,
				cells = [],
				cols = g._columns,
				total = cols.length,
				i = start || 0,
				end = count >= 0 ? start + count : total;
			for(; i < end && i < total; ++i){
				cells.push(g.cell(t.id, cols[i].id, 1));	//1 as true
			}
			return cells;	//gridx.core.Cell[]
		},

		
		data: function(){
			// summary:
			//		Get the grid data in this row.
			// description:
			//		Grid data means the result of the formatter functions (if exist).
			//		It can be different from store data (a.k.a. raw data).
			// returns:
			//		An associative array using column IDs as keys and grid data as values
			return this.model.byId(this.id).data;	//Object
		},

		
		rawData: function(){
			// summary:
			//		Get the store data in this row.
			// description:
			//		Store data means the data defined in store. It is the data before applying the formatter functions.
			//		It can be different from grid data (a.k.a. formatted data)
			// returns:
			//		An associative array using store fields as keys and store data as values
			return this.model.byId(this.id).rawData;	//Object
		},

		
		item: function(){
			// summary:
			//		Get the store item of this row
			// description:
			//		If using the old dojo.data store, store items usually have complicated structures,
			//		and they are also useful when doing store operations.
			// returns:
			//		A store item
			return this.model.byId(this.id).item;	//Object
		},

		
		setRawData: function(rawData){
			// summary:
			//		Set new raw data of this row into the store
			// rawData: Object
			//		The new data to be set. It can be incomplete, only providing a few fields.
			// returns:
			//		If using server side store, a Deferred object is returned to indicate when the operation is finished.
			var t = this, 
				s = t.grid.store,
				item = t.item(),
				field, d;
			if(s.setValue){
				d = new Deferred();
				try{
					for(field in rawData){
						s.setValue(item, field, rawData[field]);
					}
					s.save({
						onComplete: lang.hitch(d, d.callback),
						onError: lang.hitch(d, d.errback)
					});
				}catch(e){
					d.errback(e);
				}
			}
			return d || Deferred.when(s.put(lang.mixin(lang.clone(item), rawData)));	//dojo.Deferred
		}
	});
});

},
'dojo/dnd/Moveable':function(){
define(["../main", "../Evented", "../touch", "./Mover"], function(dojo, Evented, touch) {
	// module:
	//		dojo/dnd/Moveable
	// summary:
	//		TODOC


/*=====
dojo.declare("dojo.dnd.__MoveableArgs", [], {
	// handle: Node||String
	//		A node (or node's id), which is used as a mouse handle.
	//		If omitted, the node itself is used as a handle.
	handle: null,

	// delay: Number
	//		delay move by this number of pixels
	delay: 0,

	// skip: Boolean
	//		skip move of form elements
	skip: false,

	// mover: Object
	//		a constructor of custom Mover
	mover: dojo.dnd.Mover
});
=====*/

dojo.declare("dojo.dnd.Moveable", [Evented], {
	// object attributes (for markup)
	handle: "",
	delay: 0,
	skip: false,

	constructor: function(node, params){
		// summary:
		//		an object, which makes a node moveable
		// node: Node
		//		a node (or node's id) to be moved
		// params: dojo.dnd.__MoveableArgs?
		//		optional parameters
		this.node = dojo.byId(node);
		if(!params){ params = {}; }
		this.handle = params.handle ? dojo.byId(params.handle) : null;
		if(!this.handle){ this.handle = this.node; }
		this.delay = params.delay > 0 ? params.delay : 0;
		this.skip  = params.skip;
		this.mover = params.mover ? params.mover : dojo.dnd.Mover;
		this.events = [
			dojo.connect(this.handle, touch.press, this, "onMouseDown"),
			// cancel text selection and text dragging
			dojo.connect(this.handle, "ondragstart",   this, "onSelectStart"),
			dojo.connect(this.handle, "onselectstart", this, "onSelectStart")
		];
	},

	// markup methods
	markupFactory: function(params, node, ctor){
		return new ctor(node, params);
	},

	// methods
	destroy: function(){
		// summary:
		//		stops watching for possible move, deletes all references, so the object can be garbage-collected
		dojo.forEach(this.events, dojo.disconnect);
		this.events = this.node = this.handle = null;
	},

	// mouse event processors
	onMouseDown: function(e){
		// summary:
		//		event processor for onmousedown/ontouchstart, creates a Mover for the node
		// e: Event
		//		mouse/touch event
		if(this.skip && dojo.dnd.isFormElement(e)){ return; }
		if(this.delay){
			this.events.push(
				dojo.connect(this.handle, touch.move, this, "onMouseMove"),
				dojo.connect(this.handle, touch.release, this, "onMouseUp")
			);
			this._lastX = e.pageX;
			this._lastY = e.pageY;
		}else{
			this.onDragDetected(e);
		}
		dojo.stopEvent(e);
	},
	onMouseMove: function(e){
		// summary:
		//		event processor for onmousemove/ontouchmove, used only for delayed drags
		// e: Event
		//		mouse/touch event
		if(Math.abs(e.pageX - this._lastX) > this.delay || Math.abs(e.pageY - this._lastY) > this.delay){
			this.onMouseUp(e);
			this.onDragDetected(e);
		}
		dojo.stopEvent(e);
	},
	onMouseUp: function(e){
		// summary:
		//		event processor for onmouseup, used only for delayed drags
		// e: Event
		//		mouse event
		for(var i = 0; i < 2; ++i){
			dojo.disconnect(this.events.pop());
		}
		dojo.stopEvent(e);
	},
	onSelectStart: function(e){
		// summary:
		//		event processor for onselectevent and ondragevent
		// e: Event
		//		mouse event
		if(!this.skip || !dojo.dnd.isFormElement(e)){
			dojo.stopEvent(e);
		}
	},

	// local events
	onDragDetected: function(/* Event */ e){
		// summary:
		//		called when the drag is detected;
		//		responsible for creation of the mover
		new this.mover(this.node, e, this);
	},
	onMoveStart: function(/* dojo.dnd.Mover */ mover){
		// summary:
		//		called before every move operation
		dojo.publish("/dnd/move/start", [mover]);
		dojo.addClass(dojo.body(), "dojoMove");
		dojo.addClass(this.node, "dojoMoveItem");
	},
	onMoveStop: function(/* dojo.dnd.Mover */ mover){
		// summary:
		//		called after every move operation
		dojo.publish("/dnd/move/stop", [mover]);
		dojo.removeClass(dojo.body(), "dojoMove");
		dojo.removeClass(this.node, "dojoMoveItem");
	},
	onFirstMove: function(/* dojo.dnd.Mover */ mover, /* Event */ e){
		// summary:
		//		called during the very first move notification;
		//		can be used to initialize coordinates, can be overwritten.

		// default implementation does nothing
	},
	onMove: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop, /* Event */ e){
		// summary:
		//		called during every move notification;
		//		should actually move the node; can be overwritten.
		this.onMoving(mover, leftTop);
		var s = mover.node.style;
		s.left = leftTop.l + "px";
		s.top  = leftTop.t + "px";
		this.onMoved(mover, leftTop);
	},
	onMoving: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop){
		// summary:
		//		called before every incremental move; can be overwritten.

		// default implementation does nothing
	},
	onMoved: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop){
		// summary:
		//		called after every incremental move; can be overwritten.

		// default implementation does nothing
	}
});

return dojo.dnd.Moveable;
});

},
'ibm/tivoli/tip/dojo/Global':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/i18n!ibm/tivoli/tip/dojo/nls/resources","dojo/require!dojoe/messagedialog/MessageDialog,ibm/tivoli/tip/dojo/InvalidContextError,ibm/tivoli/tip/dojo/_Global"], function(dijit,dojo,dojox){
/****************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* xxxx-xxx 
*
* (C) Copyright IBM Corp. 2008
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
********************************************************** {COPYRIGHT-END} **/
dojo.require("dojoe.messagedialog.MessageDialog");
dojo.require("ibm.tivoli.tip.dojo.InvalidContextError");
dojo.require("ibm.tivoli.tip.dojo._Global");
dojo.requireLocalization("ibm.tivoli.tip.dojo", "resources");

dojo.provide("ibm.tivoli.tip.dojo.Global");


/****************************************************************************
* Global 
*
* (Q) What is this?
* (A) Since the TIP applications will be running inside a portal environment
*     it is possible that multiple instances of the application could be
*     running on one browser.  Therefore, it is necessary that our global
*     data and variables are referenced by Portlet Namespace to avoid
*     collisions between multipile instances.
*
*     This object is also used to provide common functions that are namespace
*     dependant.
****************************************************************************/

ibm.tivoli.tip.dojo.Global.IFrame        = "IframeTransport";
ibm.tivoli.tip.dojo.Global.instances     = [];
ibm.tivoli.tip.dojo.Global.anyException  = "*"; 

/****************************************************************************
* Global getSingleton function
*
* namespace (required) -- the namespace of the global object space
* context   (optional: used only if global object does NOT exist) an object of 
*			properties that will get mixed into the Global object for storage
* contentIsolation (optional) -- to show MessageDialog in separate iFrame
*
*  When creating the context, you create it like this:
*  var context = {
*                  controllerURL: "/SimpleExample/AjaxController",
*                  namespace: myNamespace,
*                  nlsResources : dojo.i18n.getLocalization( "ibm.tivoli.tbsm", "resources" )
*                  contextPath: "<%=rootContextPath%>"
*                };
*   controllerURL (required) - specifies the URL to use to send requests to.
*                 this is the URL which handles the requests usually using the
*                 TIP CommandHandler.
*   namespace (optional) - Should always be specified.  If not specified, then
*             this routine will set it on the context.
*   nlsResources (optional) - Must be specified if you intend to use the nls 
*                facility provided in this object
*   contextPath (optional) - Should always be specified but not required. 
*               Must be specified if you intend to use the helper
*               function "contextualURL".
*   ===========
*   any other properties you add into the context will be avaible to your app.
****************************************************************************/
ibm.tivoli.tip.dojo.Global.getSingleton = function( namespace, context, contentIsolation )
{
  var singleton = ibm.tivoli.tip.dojo.Global.instances[ namespace ];
  
  //if a singleton was not found then assume we need to create one now
  if ( singleton === undefined )
  {
  	//check to make sure a context object was passed before creating a Global object
  	//if the context object wasn't given then assume that an attempt was made to 
  	//access a Global object that hasn't been created -- throw an error
  	if ( context === undefined )
  	{
  		throw new ibm.tivoli.tip.dojo.InvalidContextError( "An attempt was made to access a " + 
  		"singleton ibm.tivoli.tip.dojo.Global object that has not been created yet.  Or the context " + 
  		"object was not provided on creation." );
  	}
  	
  	//make sure the namespace is saved on the context object
  	if ( context.namespace === undefined )
  	{
  		context.namespace = namespace;	
  	}  	
	//conditionally setting contentIsolation variable
    if(contentIsolation === null || contentIsolation === undefined)
		contentIsolation = false;
  	
    singleton = new ibm.tivoli.tip.dojo._Global( context, contentIsolation );
   
   	//save the new singleton instance on the Global memory space
    ibm.tivoli.tip.dojo.Global.instances[ namespace ] = singleton;
  }  
  
  return( singleton );	
};

/****************************************************************************
* Global setSingleton function
*
* namespace (required) -- the namespace of the global object space
* context   (required) an object of 
   properties that will get mixed into the Global object for storage
* contentIsolation (optional) -- to show MessageDialog in separate iFrame
****************************************************************************/
ibm.tivoli.tip.dojo.Global.setSingleton = function (namespace, context, contentIsolation) 
{
   //check to make sure a context object was passed before creating a Global object
   //if the context object wasn't given then assume that an attempt was made to 
   //access a Global object that hasn't been created -- throw an error
   if ( context === undefined )
   {
    throw new ibm.tivoli.tip.dojo.InvalidContextError( "An attempt was made to access a " + 
    "singleton ibm.tivoli.tip.dojo.Global object that has not been created yet.  Or the context " + 
    "object was not provided on creation." );
   }
   
   //make sure the namespace is saved on the context object
   if ( context.namespace === undefined )
   {
    context.namespace = namespace; 
   }
   //conditionally setting contentIsolation variable
   if(contentIsolation === null || contentIsolation === undefined)
		contentIsolation = false;
   var singleton = new ibm.tivoli.tip.dojo._Global( context, contentIsolation );
   
   //save the new singleton instance on the Global memory space
   ibm.tivoli.tip.dojo.Global.instances[ namespace ] = singleton; 
   
   return (singleton);
};

});

},
'ibm/tivoli/tip/navwidget/table/dijit/NavModelTable':function(){
// wrapped by build app
define("ibm/tivoli/tip/navwidget/table/dijit/NavModelTable", ["dijit","dojo","dojox","dojo/i18n!ibm/tivoli/tip/navwidget/nls/resources","dojo/require!ibm/tivoli/tip/navwidget/dijit/CustomContentTooltip,ibm/tivoli/tip/navwidget/properties/dijit/PropertiesNotebookGeneralTab,dojoe/tablex/TableAll"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.require("ibm.tivoli.tip.navwidget.dijit.CustomContentTooltip");
dojo.require("ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebookGeneralTab");

dojo.require("dojoe.tablex.TableAll");  // 49426  Gridx

dojo.requireLocalization("ibm.tivoli.tip.navwidget", "resources");

dojo.provide("ibm.tivoli.tip.navwidget.table.dijit.NavModelTable");

dojo.declare ("ibm.tivoli.tip.navwidget.table.dijit.NavModelTable",
	[dojoe.tablex.Table],   // 49426  Gridx
	{

	// viewLoaded: boolean
	//		Coming soon
	viewLoaded : false,
	
	// showTooltip: boolean
	//		Optionally hide tooltip
	showTooltip: true,
	
	// _propOnSetObjectsConnection: [private] Object
	//		Coming soon
	_propOnSetObjectsConnection: null,
	
	// _propOnSetDatasetConnection: [private] Object
	//		Coming soon
	_propOnSetDatasetConnection: null,
	
	// _tooltip: [private] Object
	//      Coming soon
	_tooltip : null,
	
	// _resizing: [private] Number
	//      Coming soon
	_resizing : 0,

	postCreate : function() {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon

		//first let the grid get generated as normal
		this.inherited(arguments);
	
		// 22993: KDM  Optionally hide tooltips
		if (this.showTooltip) {	
			// create tooltip onto the grid domNode
			this._tooltip = new ibm.tivoli.tip.navwidget.dijit.CustomContentTooltip(
					{
						connectId : [ this._grid.domNode.id ],  // 49426
						getCustomContentCallback : dojo.hitch(this, this._customTooltip),
						position : ["below", "after"]
					});
		
			// Override the tooltip behavior specifically for
			// topology
			dojo.safeMixin(this._tooltip, {
		
				// Override open to set tooltip position over the
				// menu layer DIV near table row
				open : function(target) {
				// Create a rectangle for positioning the
				// tooltip, otherwise default
				// position is outside the DOM node of the menu
				// layer DIV
				var aroundRect = target.rolloverTargetRect;
		
				// make sure the rolloverTargetRect has been set
				// to actally display the tooltip
				// (it may not be if we got a rollover while the
				// context menu is displayed)
				if (aroundRect != null) {
					// replace the target node, the menu layer
					// DIV, with this rectangle
					arguments[0] = aroundRect;
					this.inherited(arguments);
				}
			},
			_onTargetMouseEnter : function(e) {
				// ignore since menu layer div is not the real
				// target (we handle the tooltip)
			},
			_onTargetMouseLeave : function(e) {
				// ignore since menu layer div is not the real
				// target (we handle the tooltip)
			},
			_onTargetBlur : function(e) {
				// ignore since menu layer div is not the real
				// target (we handle the tooltip)
			},
			_onTargetFocus : function(e) {
				// ignore since menu layer div is not the real
				// target (we handle the tooltip)
			},
			// added helper method to trigger the timer to display
			// the tooltip
			hover : function(target) {
				this._onHover( {
					target : target
				});
			},
			// added helper method to dismiss the tooltip
			unHover : function(target) {
				this._onUnHover( {
					target : target
				});
			}
			});
			this._tooltip.startup();

			dojo.connect( this._grid, "onCellMouseOver", this, "_onCellMouseOver");  // 49426
			dojo.connect( this._grid, "onCellMouseOut", this, "_onCellMouseOut");  // 49426
			dojo.connect( this, "onTableLoad", this, "notifyLoadComplete");
		}
	},
	
	notifyLoadComplete: function(){
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		
		this.viewLoaded = true;
		
  		//Send notification we're done loading the dataset/prepping the view now that the table has loaded
  		var parentPluggableView = this.tableView;
  		parentPluggableView._onSetDatasetLoadComplete(parentPluggableView.dataset);
	
	},
	
	resize : function() {
		// summary:
		//		Handle special resize case for IE8
		// description:
		//		Resize events that can occur on IE8 while processing the CURI REST
		//      callback to populate the context menu can sometimes interfere with
		//      the menu rendering the returned new task items.  (The resize event
		//      processing is dispatched after the menu has been repopulated but
		//      before it can actually render to the screen in IE8.  The result is
		//      the menu can be dismissed before it renders correctly.)  This
		//      workaround delays the processing of the resize event slightly thus
		//      allowing the menu to render.

        if (dojo.isIE) {
        	if (this.tableView && this.tableView.isMenuOpen() && (this._resizing < 1)) {
        		this._resizing++;
        		setTimeout(dojo.hitch(this, this._doResize), 0);
            }
            else {
            	this.inherited(arguments);
            }
        } 
        else {
        	this.inherited(arguments);
        }
    },

    _doResize: function() {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon

    	this.resize();
    	this._resizing--;
    },

	_customTooltip: function(/*Object*/ target) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// target:
		//		Coming soon

		if (this._rolloverTarget) {
			// target is this._layerDiv, so we use this._rolloverTarget
			var content = "";
			if (typeof this._rolloverTarget == "string") {
				content = content + this._rolloverTarget;
			} else {
				content = content + this._rolloverTarget.getLabel();
			}

			//first check if a server side cell property tooltip is defined, use that instead of other defaults
			if(this._rolloverCellTarget && this._rolloverCellTarget.getTooltip){
				var tooltip = this._rolloverCellTarget.getTooltip();
				if(tooltip){
					return tooltip;
				}
			}
			
			//if the server side tooltip content is set, use this instead of defaults.
			if(this._rolloverTarget.getTooltip) { //Need first test because getTooltip is not a function on child datasets right now
				var tooltip = this._rolloverTarget.getTooltip();
				if (tooltip){ 
					return tooltip;
				}
			}

			parentPluggableView = this.tableView;
			
			if(this._tooltipProperties){
				return this._tooltipProperties.getContent();
			}
			
			//create properties content object
			this._tooltipProperties = new ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebookGeneralTab({
				id: this.id + "_tooltipProperties",
				managingPluggableView: parentPluggableView, 
				maxProperties: 3,  //TODO: ADM should probably be something a widgetDef sets.
				_isTooltipProperties: true
			});

			//Call back for when the dataset it set
			var objectType = (this._rolloverTarget.getObjectType) ? this._rolloverTarget.getObjectType() : null;
			var setDatasetComplete = function(dataset) {
				//connect to the onSetObjects Event
				this._propOnSetObjectsConnection = dojo.connect(this._tooltipProperties, "onSetObjects", dojo.hitch(this, this._updateTooltipContent));

				if (objectType === "item"){
					this._tooltipProperties.setObjects([this._rolloverTarget], null, null, null);
				} else if(objectType === "link"){
					this._tooltipProperties.setObjects(null, [this._rolloverTarget], null, null);
				} else if(objectType === "relationship"){
					this._tooltipProperties.setObjects(null, null, [this._rolloverTarget], null);
				}
			};
			this._propOnSetDatasetConnection = dojo.connect(this._tooltipProperties, "onSetDataset", dojo.hitch(this, setDatasetComplete));
			this._tooltipProperties.setDataset(parentPluggableView.dataset);

			//return content;
			return this._createTemporaryContentDiv(content, objectType);
		}
	   },
	   
	   //Event callback method to handle updating the tooltip content with updated content
	   _updateTooltipContent: function(){
		
		this._tooltip.updateContent(this._tooltipProperties.getContent(), this._grid.domNode);  // 49426
	   },
	   
	   _createTemporaryContentDiv: function(content, objectType) {		   
		   //Background div
		   var backgroundDiv = dojo.doc.createElement("div");
		   var processingPane = new dijit.layout.ContentPane({
			   content: backgroundDiv
		   });
			
			// text div		
		   var textDiv = dojo.create("div",{
			   "innerHTML": content
		   });
		   backgroundDiv.appendChild(textDiv);
		
		   if (objectType) {
			   // processing image div		
			   var processDiv = dojo.create("div",{
					"class": "processingImage"
			   });
			   backgroundDiv.appendChild(processDiv);
		   }

		   //Return
		   return processingPane;
	   },
	   
	   
	   //function to close tooltips
		closeTooltip: function() {
			if (this._rolloverTarget) {
				dojo.disconnect(this._propOnSetObjectsConnection);
				dojo.disconnect(this._propOnSetDatasetConnection);
				
				// Clear out the reference in the tooltip to the content
				this._tooltip.updateContent("", this._grid.domNode);  // 49426
				if (this._tooltipProperties){
					this._tooltipProperties.destroyRecursive();
					this._tooltipProperties = null;
				}
				
				// this._tooltip.close();
				this._tooltip.unHover(this._grid.domNode);  // 49426
				this._rolloverTarget = null;
				this._rolloverCellTarget = null;
				this._grid.domNode.rolloverTargetRect = null;  // 49426
		    }
		},

	_onCellMouseOut: function( e ){
		//console.log("mouse rolled out of row");
		
		this.closeTooltip();	
		dojo.stopEvent(e);
		
	},
			
	_onCellMouseOver: function( e ){
		//console.log("rolled onto row ");
		
		dojo.stopEvent(e);
		
		// ignore if the menu is open or if we already have a rollover target
		// or we are temporarily suppressing the tooltip (e.g. , during refresh) 
		if(!this.tableView.suppressTooltip() && !this.tableView.isMenuOpen() && !this._rolloverTarget ){  // 52876
	    	var aroundRect = {type: "rect"};
	    	var field = this._grid.column(e.columnId,false).field;   // 49426  Gridx

	    	// 49426  this._rolloverTarget = e.grid.getItem(e.rowNode.gridRowIndex);
	    	this._rolloverTarget = this._grid.model.byId(e.rowId).item;   // 49426  Gridx
	    	
	    	if (this._rolloverTarget.hasCachedProperty(field)) {  // 49426  Gridx
	    		this._rolloverCellTarget = this._rolloverTarget.getProperty(field);  // 49426 Gridx
	    	}
	    	else {
	    		this._rolloverCellTarget = null;
	    	}

	    	var boundingInfo = dojo.position(e.target, true);  // 49426  Gridx
	    	aroundRect.x = boundingInfo.x; 
	    	aroundRect.y = boundingInfo.y;

	    	aroundRect.width = e.target.clientWidth;  // 49426  Gridx
	    	aroundRect.height = e.target.clientHeight;  // 49426  Gridx
	    
	    	// adjust relative coordinates to absolute, and remove fractions
	    	var lt = dojo.coords(this._grid.domNode, true);  // 49426
	    	aroundRect.x = Math.round(aroundRect.x);
	    	aroundRect.y = Math.round(aroundRect.y);
			
	    	this._grid.domNode.rolloverTargetRect = aroundRect;  // 49426
	    	this._tooltip.hover(this._grid.domNode);  // 49426

		
	    }
			
	}	

});	
	

});

},
'dijit/form/RangeBoundTextBox':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/i18n", // i18n.getLocalization
	"./MappedTextBox"
], function(declare, i18n, MappedTextBox){

/*=====
	var MappedTextBox = dijit.form.MappedTextBox;
=====*/

	// module:
	//		dijit/form/RangeBoundTextBox
	// summary:
	//		Base class for textbox form widgets which defines a range of valid values.

	/*=====
		dijit.form.RangeBoundTextBox.__Constraints = function(){
			// min: Number
			//		Minimum signed value.  Default is -Infinity
			// max: Number
			//		Maximum signed value.  Default is +Infinity
			this.min = min;
			this.max = max;
		}
	=====*/

	return declare("dijit.form.RangeBoundTextBox", MappedTextBox, {
		// summary:
		//		Base class for textbox form widgets which defines a range of valid values.

		// rangeMessage: String
		//		The message to display if value is out-of-range
		rangeMessage: "",

		/*=====
		// constraints: dijit.form.RangeBoundTextBox.__Constraints
		constraints: {},
		======*/

		rangeCheck: function(/*Number*/ primitive, /*dijit.form.RangeBoundTextBox.__Constraints*/ constraints){
			// summary:
			//		Overridable function used to validate the range of the numeric input value.
			// tags:
			//		protected
			return	("min" in constraints? (this.compare(primitive,constraints.min) >= 0) : true) &&
				("max" in constraints? (this.compare(primitive,constraints.max) <= 0) : true); // Boolean
		},

		isInRange: function(/*Boolean*/ /*===== isFocused =====*/){
			// summary:
			//		Tests if the value is in the min/max range specified in constraints
			// tags:
			//		protected
			return this.rangeCheck(this.get('value'), this.constraints);
		},

		_isDefinitelyOutOfRange: function(){
			// summary:
			//		Returns true if the value is out of range and will remain
			//		out of range even if the user types more characters
			var val = this.get('value');
			var isTooLittle = false;
			var isTooMuch = false;
			if("min" in this.constraints){
				var min = this.constraints.min;
				min = this.compare(val, ((typeof min == "number") && min >= 0 && val !=0) ? 0 : min);
				isTooLittle = (typeof min == "number") && min < 0;
			}
			if("max" in this.constraints){
				var max = this.constraints.max;
				max = this.compare(val, ((typeof max != "number") || max > 0) ? max : 0);
				isTooMuch = (typeof max == "number") && max > 0;
			}
			return isTooLittle || isTooMuch;
		},

		_isValidSubset: function(){
			// summary:
			//		Overrides `dijit.form.ValidationTextBox._isValidSubset`.
			//		Returns true if the input is syntactically valid, and either within
			//		range or could be made in range by more typing.
			return this.inherited(arguments) && !this._isDefinitelyOutOfRange();
		},

		isValid: function(/*Boolean*/ isFocused){
			// Overrides dijit.form.ValidationTextBox.isValid to check that the value is also in range.
			return this.inherited(arguments) &&
				((this._isEmpty(this.textbox.value) && !this.required) || this.isInRange(isFocused)); // Boolean
		},

		getErrorMessage: function(/*Boolean*/ isFocused){
			// Overrides dijit.form.ValidationTextBox.getErrorMessage to print "out of range" message if appropriate
			var v = this.get('value');
			if(v !== null && v !== '' && v !== undefined && (typeof v != "number" || !isNaN(v)) && !this.isInRange(isFocused)){ // don't check isInRange w/o a real value
				return this.rangeMessage; // String
			}
			return this.inherited(arguments);
		},

		postMixInProperties: function(){
			this.inherited(arguments);
			if(!this.rangeMessage){
				this.messages = i18n.getLocalization("dijit.form", "validate", this.lang);
				this.rangeMessage = this.messages.rangeMessage;
			}
		},

		_setConstraintsAttr: function(/*Object*/ constraints){
			this.inherited(arguments);
			if(this.focusNode){ // not set when called from postMixInProperties
				if(this.constraints.min !== undefined){
					this.focusNode.setAttribute("aria-valuemin", this.constraints.min);
				}else{
					this.focusNode.removeAttribute("aria-valuemin");
				}
				if(this.constraints.max !== undefined){
					this.focusNode.setAttribute("aria-valuemax", this.constraints.max);
				}else{
					this.focusNode.removeAttribute("aria-valuemax");
				}
			}
		},

		_setValueAttr: function(/*Number*/ value, /*Boolean?*/ priorityChange){
			// summary:
			//		Hook so set('value', ...) works.

			this.focusNode.setAttribute("aria-valuenow", value);
			this.inherited(arguments);
		},

		applyTextDir: function(/*===== element, text =====*/){
			// summary:
			//		The function overridden in the _BidiSupport module,
			//		originally used for setting element.dir according to this.textDir.
			//		In this case does nothing.
			// element: Object
			// text: String
			// tags:
			//		protected.
		}
	});
});

},
'dojoe/treetable/filter/_ConditionExpr':function(){
// wrapped by build app
define(["dijit","dojo","dojox"], function(dijit,dojo,dojox){
/**
 * @author shakeel
 */
dojo.provide("dojoe.treetable.filter._ConditionExpr");

(function(){
	var f_ns = dojoe.treetable.filter;
	
dojo.declare("dojoe.treetable.filter._ConditionExpr",null,{
	// summary:
	//		The most abstract class for all condition expressions.
	//		A condition expression can be applied on a data row (e.g. an item in a store)
	//		and generate a result condition expression.
	// tags:
	//		abstract
	applyRow: function(/* data item */datarow,/* function(row,colArg) */getter){
		// summary:
		//		*Unimplemented Interface*
		//		Apply this condition expression on the given datarow, return a result expression.
		// taqs:
		//		public extension
		// datarow: object
		//		A data item of a store.
		// getter: function(datarow, colArg)
		//		A user defined function that extract cell data from *datarow*.
		//		*colArg* is an argument that provides a kind of column information.
		//		It is defined by user in the constructor of a _DataExpr object.
		// returns: 
		//		MUST return a _ConditionExpr object
		throw new Error("_ConditionExpr.applyRow: unimplemented interface");
	},
	toObject: function(){
		// summary:
		//		Convert this data expression to a simple object. Mainly used for serialization.
		// tags:
		//		public extension
		// returns:
		//		An object for serialization.
		return {};	//Object
	},
	getName: function(){
		// summary:
		//		Get the name of this kind of expression.
		// tags:
		//		public extension
		// returns:
		//		the name of this kind of expression
		return "expr";	//String
	}
});
dojo.declare("dojoe.treetable.filter._DataExpr",f_ns._ConditionExpr,{
	// summary:
	//		The most abstract class for all data expressions.
	//		A _DataExpr is a condition expression for a single data value.
	//		If the data value to be represent is a pure value (literal value, like string/number/Date/...)
	//		this _DataExpr is nothing more than a simple wrapper.
	//		If the data value to be represent is in a store, then _DataExpr is responsible to extract it 
	//		from the store when this condition is applied to a data row.
	// private fields:
			//_value: null,
			//_colArg: null,
	constructor: function(/* anything */dataValue,/* bool */isColumn, /* object */convertArgs){
		// summary:
		//		If a _DataExpr is constructed with only one argument, this argument is regarded as a pure value.
		//		If the second argument is exactly a boolean true (no implict type transformation, 
		//		so as to allow derived classes accept more arguments while retain *isColumn* to be optional), 
		//		then this _DataExpr represents a column, and it's applyRow method is not a no-op.
		// dataValue: anything
		//		If *isColumn* is a boolean true, then it should be a kind of column information, like field name
		//		or column index. Otherwise, it is regarded as a pure value, and the getValue method will simply
		//		return it.
		// isColumn: boolean?
		//		Optional. To specify whether this _DataExpr represents a column or a pure value.
		try{
			this._convertArgs = convertArgs || {};
			if(dojo.isFunction(this._convertArgs.convert)){
				this._convertData = dojo.hitch(this._convertArgs.scope, this._convertArgs.convert);
			}
			if(isColumn){
				this._colArg = dataValue;
			}else{
				this._value = this._convertData(dataValue, this._convertArgs);
			}
		}catch(ex){
			console.log(ex.message);
		}
	},
	getValue: function(){
		// summary:
		//		If this is a pure value wrapper, simply return the value.
		//		Otherwise (it's a column), return is undefined.
		// tags:
		//		public
		// returns:
		//		the value of this data expression.
		return this._value;	//String
	},
	applyRow: function(/* data item */datarow,/* function(row,colIdx) */getter){
		// summary:
		//		Implement _ConditionExpr.applyRow.
		//		If this is a pure value, simply return self.
		//		Otherwise, extract the cell data from datarow using the given getter function,
		//		and then convert this cell data to a _DataExpr and return the expression.
		return typeof this._colArg == "undefined" ? this :			//_ConditionExpr 
			new (dojo.getObject(this.declaredClass))(
				this._convertData(getter(datarow, this._colArg), this._convertArgs)
			);
	},
	_convertData: function(/* anything */dataValue){
		// summary:
		//		
		// tags:
		//		protected extension
		// dataValue: anything
		//		This argument should come from a store.
		// returns: 
		return dataValue;
	},
	toObject: function(){
		// summary:
		//		Overrided from _ConditionExpr.toObject
		return {															//String
			op: this.getName(),
			data: this._colArg === undefined ? this._value : this._colArg,
			isCol: this._colArg !== undefined
		};
	},
	getName: function(){
		// summary:
		//		Overrided from _ConditionExpr.getName.
		return "data";	//String
	}
});
dojo.declare("dojoe.treetable.filter._OperatorExpr",f_ns._ConditionExpr,{
	// summary:
	//		The most abstract class for all operator expressions.
	//		An operator expression is a _ConditionExpr that represents an operation.
	constructor: function(/* Array | operand1,operand2,... */){
		// summary:
		//		The arguments are operands (or an array of operands, if the first argument
		//		is an Array) of this operator, ordering from left to right.
		//		Every operand should be a _ConditionExpr.
		if(dojo.isArray(arguments[0])){
			this._operands = arguments[0];
		}else{
			this._operands = [];
			for (var i = 0; i < arguments.length; ++i) {
				this._operands.push(arguments[i]);
			}
		}
	},
	toObject: function(){
		// summary:
		//		Overrided from _ConditionExpr.toObject
		return {											//Object
			op: this.getName(),
			data: dojo.map(this._operands,function(operand){
				return operand.toObject();
			})
		};
	},
	getName: function(){
		// summary:
		//		Overrided from _ConditionExpr.getName.
		return "operator";	//String
	}
});
dojo.declare("dojoe.treetable.filter._UniOpExpr",f_ns._OperatorExpr,{
	// summary:
	//		The most abstract class for all uni-operator expressions.
	//		A uni-operator expression is an _OperatorExpr that only allow one operand.
	applyRow: function(/* data item */datarow,/* function(row,colArg) */getter){
		// summary:
		//		Implement _ConditionExpr.applyRow.
		//		Apply the restriction of "only one operand" and confirm the operand is a valid _ConditionExpr.
		//		Then do the calculation of this operator.
		if(!(this._operands[0] instanceof f_ns._ConditionExpr)){
			throw new Error("_UniOpExpr: operand is not expression.");
		}
		return this._calculate(this._operands[0],datarow,getter);	//_ConditionExpr
	},
	_calculate: function(/* _ConditionExpr */operand,/* data item*/datarow,/* function(row,colArg) */getter){
		// summary:
		//		*Unimplemented Interface*
		//		Do the actrual work of applyRow here.
		// tags:
		//		protected extension
		// operand: _ConditionExpr
		// datarow: object
		// getter: function(row,colArg)
		// returns:
		//		MUST return a _ConditionExpr object.
		throw new Error("_UniOpExpr._calculate: unimplemented interface");
	},
	getName: function(){
		// summary:
		//		Overrided from _ConditionExpr.getName.
		return "uniOperator";	//String
	}
});
dojo.declare("dojoe.treetable.filter._BiOpExpr",f_ns._OperatorExpr,{
	// summary:
	//		The most abstract class for all bi-operator expressions.
	//		A bi-operator expression is an _OperatorExpr that allow and only allow two operands.
	applyRow: function(/* data item */datarow,/* function(row,colArg) */getter){
		// summary:
		//		Implement _ConditionExpr.applyRow.
		//		Apply the restriction of "two operands" and confirm operands are valid _ConditionExpr's.
		if(!(this._operands[0] instanceof f_ns._ConditionExpr)){
			throw new Error("_BiOpExpr: left operand is not expression.");
		}else if(!(this._operands[1] instanceof f_ns._ConditionExpr)){
			throw new Error("_BiOpExpr: right operand is not expression.");
		}
		return this._calculate(this._operands[0],this._operands[1],datarow,getter);
	},
	_calculate: function(/* _ConditionExpr */left_operand,/* _ConditionExpr */right_operand,/* data item*/datarow,/* function(row,colArg) */getter){
		// summary:
		//		*Unimplemented Interface*
		//		Do the actrual work of applyRow here.
		// tags:
		//		protected extension
		// left_operand: _ConditionExpr
		// right_operand: _ConditionExpr
		// datarow: object
		// getter: function(row,colArg)
		// returns:
		//		MUST return a _ConditionExpr object.
		throw new Error("_BiOpExpr._calculate: unimplemented interface");
	},
	getName: function(){
		// summary:
		//		Overrided from _ConditionExpr.getName.
		return "biOperator";	//String
	}
});
})();

});

},
'ibm/tivoli/navmodel/Status':function(){
// wrapped by build app
define("ibm/tivoli/navmodel/Status", ["dijit","dojo","dojox"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Status"); 
 
////////////////////////////////////
// Status
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Status", null, {
	// private members
	_version: null,
	_status: null,

	/*=====
	constructor_kwArgs: function(version, style) {
		// version: Float
		//		version used to communicate with rest service
		// style: Object
		//		json item style object
	},
	=====*/
	
	 constructor: function(/*constructor_kwArgs*/ kwArgs) {		
		// summary:
		//		Constructs Status object.
		this._version = kwArgs.version;
		this._status = kwArgs.status;		
		if (!this._status) {
			this._status = {};
		}
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	
	getAttribute: function(key) {
		return this._status[key];
	},
	
	setAttribute: function(key, value) {
		this._status[key] = value;
	},
	
	getId: function() {
		return this._status.id;
	},
	
	setId: function(id) {
		this._status.id = id;		
	},
	
	getValue: function() {
		return this._status.value;
	},
	
	setValue: function(value) {
		this._status.value = value;
	},
	
	getLabel: function() {
		if (this._status.label) {
			// string
			if (typeof this._status.label === "string") {
				return this._status.label;
			}
			// object with key/value info
			else {
				return this._status.label.value;
			}
		}
		
		return null;
	},
	
	setLabel: function(label) {
		this._status.label = label;
	},
	
	getImage: function() {
		return this._status.image;
	},
	
	setImage: function(image) {
		this._status.image = image;
	},
	
	getImageWidth: function() {
		return this._status.imageWidth;
	},
	
	setImageWidth: function(width) {
		this._status.imageWidth = width;
	},
	
	getImageHeight: function() {
		return this._status.imageHeight;
	},
	
	setImageHeight: function(height) {
		this._status.imageHeight = height;
	},
	
	isImageVisible: function() {
		return (this._status.imageVisible !== undefined ? this._status.imageVisible : true);
	},
	
	setImageVisible: function(visible) {
		this._status.imageVisible = visible;
	},
	
	getScalarImage: function() {
		return this._status.scalarImage;
	},
	
	setScalarImage: function(image) {
		this._status.scalarImage = image;
	},
	
	getScalarImageWidth: function() {
		return this._status.scalarImageWidth;
	},
	
	setScalarImageWidth: function(width) {
		this._status.scalarImageWidth = width;
	},
	
	getScalarImageHeight: function() {
		return this._status.scalarImageHeight;
	},
	
	setScalarImageHeight: function(height) {
		this._status.scalarImageHeight = height;
	},
	
	isScalarImageVisible: function() {
		return (this._status.scalarImageVisible !== undefined ? this._status.scalarImageVisible : true);
	},
	
	setScalarImageVisible: function(visible) {
		this._status.scalarImageVisible = visible;
	},
	
	getBackgroundScalarImage: function() {
		return this._status.backgroundScalarImage;
	},
	
	setBackgroundScalarImage: function(image) {
		this._status.backgroundScalarImage = image;
	},
	
	getBackgroundScalarImageWidth: function() {
		return this._status.backgroundScalarImageWidth;
	},
	
	setBackgroundScalarImageWidth: function(width) {
		this._status.backgroundScalarImageWidth = width;
	},
	
	getBackgroundScalarImageHeight: function() {
		return this._status.backgroundScalarImageHeight;
	},
	
	setBackgroundScalarImageHeight: function(height) {
		this._status.backgroundScalarImageHeight = height;
	},
	
	isBackgroundScalarImageVisible: function() {
		return (this._status.backgroundScalarImageVisible !== undefined ? this._status.backgroundScalarImageVisible : true);
	},
	
	setBackgroundScalarImageVisible: function(visible) {
		this._status.backgroundScalarImageVisible = visible;
	},
	
	getTooltip: function() {
		if (this._status.tooltip) {
			// string
			if (typeof this._status.tooltip === "string") {
				return this._status.tooltip;
			}
			// object with key/value info
			else {
				return this._status.tooltip.value;
			}
		}
		
		return null;
	},
	
	setTooltip: function(tooltip) {
		this._status.tooltip = tooltip;
	},
	
	getMessage: function() {
		return this._status.message;
	},
	
	setMessage: function(message) {
		this._status.message = message;
	},
	
	getColor: function() {
		return this._status.color;
	},
	
	setColor: function(color) {
		this._status.color = color;
	},
	
	clone: function(widgetDef, objectType) {
		var args = {
			status: {},
			version: this._version
		};		
		
		if (widgetDef) {
			// start with widgetDef default status
			var widgetDefaultStatus = null;
			switch(objectType) {
				case "item":
					widgetDefaultStatus = widgetDef.getDefaultItemStatus();
					break;
				case "link":
					widgetDefaultStatus = widgetDef.getDefaultLinkStatus();
					break;
				case "relationship":
					widgetDefaultStatus = widgetDef.getDefaultRelationshipStatus();					
			}			
			if (widgetDefaultStatus) {
				dojo.mixin(args.status, widgetDefaultStatus);
			}
		
			// mixin our status
			dojo.mixin(args.status, this._status);
			
			// mixin widgetDef status
			var widgetStatus = null;
			switch(objectType) {
				case "item":
					widgetStatus = widgetDef.getItemStatus(this.getId(), this.getValue());
					break;
				case "link":
					widgetStatus = widgetDef.getLinkStatus(this.getId(), this.getValue());
					break;
				case "relationship":
					widgetStatus = widgetDef.getRelationshipStatus(this.getId(), this.getValue());					
			}					
			if (widgetStatus) {
				dojo.mixin(args.status, widgetStatus);
			}
		}
		// just mixin our status
		else {
			dojo.mixin(args.status, this._status);
		}
			
		return new ibm.tivoli.navmodel.Status(args);
	}
});

});

},
'ibm/tivoli/tip/navwidget/dijit/NavWidgetPluggableView':function(){
// wrapped by build app
define("ibm/tivoli/tip/navwidget/dijit/NavWidgetPluggableView", ["dijit","dojo","dojox","dojo/i18n!ibm/tivoli/tip/navwidget/nls/resources","dojo/require!ibm/tivoli/tip/navwidget/dijit/DefaultMessageHandler,ibm/tivoli/navmodel/Trace,dijit/_Widget,dojox/collections/SortedList,dojox/collections/ArrayList,ibm/tivoli/tip/dijit/BusyPane,ibm/tivoli/tip/navwidget/dojo/NavWidgetUtil,ibm/tivoli/tip/navwidget/dojo/NavWidgetPluggableViewSelection,ibm/tivoli/tip/navwidget/dojo/NavWidgetBaseViewState"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.dijit.NavWidgetPluggableView"); 

// includes
dojo.require("ibm.tivoli.tip.navwidget.dijit.DefaultMessageHandler");
dojo.require("ibm.tivoli.navmodel.Trace");
dojo.require("dijit._Widget");
dojo.require("dojox.collections.SortedList");
dojo.require("dojox.collections.ArrayList");
dojo.require("ibm.tivoli.tip.dijit.BusyPane");
dojo.require("ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil");
dojo.require("ibm.tivoli.tip.navwidget.dojo.NavWidgetPluggableViewSelection");
dojo.require("ibm.tivoli.tip.navwidget.dojo.NavWidgetBaseViewState");
ibm.tivoli.tip.navwidget.dijit.NavWidgetPluggableView._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.tip.navwidget.dijit.NavWidgetPluggableView");
 
dojo.requireLocalization("ibm.tivoli.tip.navwidget", "resources");


dojo.declare("ibm.tivoli.tip.navwidget.dijit.NavWidgetPluggableView", 
	[dijit._Widget], {
	// summary:
	//		Base class for a view that plugged into a NavWidget
	// description:
	//		Coming soon
	
    //**************************************************************************************/
	
	// viewId: [const] String
	//		Data model
	viewId: null,
	
	// viewTypeSpec: [const] String
	//		Coming soon
	viewTypeSpec: null,
	
	// dataset: [const] ibm.tivoli.navmodel.Dataset
	//		Data model
	dataset: null,
	
	// initializeWithNullDataset: [const] ibm.tivoli.navmodel.Dataset
	//		Determines whether setDataset(null) will be called during initialization. Generally, this can be left to true.
	//
	//		The typical scenario is to pass in null on the constructor, and then when the Dataset to be rendered finishes it's
	//		asynchronous initialization manually calling setDataset with the initialized dataset. In those cases, it is 
	//		often desirable to allow the UI to start rendering while the dataset initializes. 
	//
	//		This works pretty well if you expect the widgetDefinition passed in to the constructor to remain active 
	//		when the initialized dataset is later passed in via setDataset. However, in cases where you expect the
	//		dataset to specify a custom widget definition to override the default, it is less desirable because the
	//		UI will be set-up using the default def and then entirely rebuilt whenever the intialized dataset is passed in
	initializeWithNullDataset: true,
	
	// parentPluggableView: [const] ibm.tivoli.tip.navwidget.dijit.NavWidgetPluggableView
	//		The parent pluggable view. If null, this instance is a stand-alone view not part of a larger view. This
	//		should only be set when this view is a true child of the parent (e.g., has been added to the parent
	//		with addPluggableView). This will typically be the case, but occasionally there are more "transient"
	//		pluggable views (like dialogs and tooltips) that aren't truly child views.
	parentPluggableView: null,
	
	// managingPluggableView: [const] ibm.tivoli.tip.navwidget.dijit.NavWidgetPluggableView
	//		The managing pluggable view. This is intended for use in the special cases of more "transient" pluggable 
	//		views that aren't truly children (such as dialogs and tooltips), but might have a need to for some
	//		support from a higher-level view to help manage busy pane's, etc.
	managingPluggableView: null,
	
	// widgetDefinitionId: [const] String
	//		The id for the current widget definition used in this view
	widgetDefinitionId: "",		
	
	// widgetDefinitionId: [const] String
	//		The current widget definition used in this view
	widgetDefinition : null,
	
	// labelIconClass: [const] String
	//		An appropriate iconClass for use next to a label in a menu for switching views, etc.
	labelIconClass : null,
	
	// _originalWidgetDefinition: [private] String
	//		The first widget definition used in this view
	_originalWidgetDefinition : null,

	// cmProvider: [const] TIPContextMenuProvider
	//		Custom context menu provider (must be provided on construction to be used)
	cmProvider: null,
	
	//daHelper: [const] DefaultActionHandler
	//      Custom overwritten DefaultActionHandler (must be provided on construction to be used)
	daHelper: null,
	
	//defaultMessageHandler: [const] defaultMessageHandler
	//      Custom overwritten DefaultActionHandler (must be provided on construction to be used)
	defaultMessageHandler: null,
	
	// destroyDatasetWhenDone: [const] Boolean
	//		Determine whether to destroy the dataset when we clean ourselves up (but only if no parent view and no one else listening to dataset)
	destroyDatasetWhenDone: true,
	
	// _pluggableViews: [private] Dictionary
	//		Data structure of pluggable views
    _childPluggableViews: null,
    
    // selection: [const] ibm.tivoli.tip.navwidget.dojo.NavWidgetPluggableViewSelection
	//		Manages the selection of the view
	selection: null,
	
	// _setDatasetLoadDataCompletesSynchronously: [protected] Boolean
	//		Allows subclasses to specify whether they load data asynchronously or not
	_setDatasetLoadDataCompletesSynchronously: true,
	
	// _usesWidgetDefinition: [protected] Boolean
	//		Allows subclasses to specify whether they use a widget definition or not
	_usesWidgetDefinition: true,
	
	// _busyPane: [private] ibm.tivoli.tip.dijit.BusyPane
	//      Reference to the BusyPane for this view.  Note, only standalone views
	//      or the topmost parent pluggable view have a BusyPane to ensure only
	//      one per NavWidget
	_busyPane: null,
	
	// _busyPaneCount: [private] Number (integer)
	//      A counter that represents the number of times showBusyPane() has
	//      been called before hideBusyPane() is called.  Calling hideBusyPane()
	//      decrements the count and will only truly hide the busy pane when the
	//      count returns to zero.  This allows multiple child pluggable views to
	//      request the busy pane, but it will only be displayed on the first
	//      request and only hidden once the last request cycle calls hideBusyPane()
	_busyPaneCount: 0,

	// commonResources: [protected] dojo.i18n
	//		Shared resource bundle

	commonResources: null,
	
	// _isDestroyed: [private] Boolean
	//  	Flag as to whether we've been destroyed or not
	_isDestroyed: false,
	
	// _trace: [private] Trace
	//		An instance of Trace customized for this class
	_trace: ibm.tivoli.tip.navwidget.dijit.NavWidgetPluggableView._trace, 

	//***********************************************************************************************/
	
	constructor: function () {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon

        	this.commonResources = dojo.i18n.getLocalization("ibm.tivoli.tip.navwidget", "resources");	

		//The sorted list can suffer from performance issues if there are a lot of insertions. But, the number of views managed should typically be less
		//than a handful.
		this._childPluggableViews = new dojox.collections.SortedList();
	},
	
	postMixInProperties: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		
		//Set up the widgetDef
		this._createWidgetDefinition();
		
		this.inherited(arguments);
	},

	buildRendering: function(){				
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		
		//Create the selection manager
		this.selection = this._createSelection();
		
		//Call superclass
		this.inherited(arguments);
	},
	
	postCreate: function(){
		// summary:
		//		Override's _Widgets postCreate to call setDataset if a dataset was set in the initial 
		//		parms.
		// dataset:
		//		The Client Nav Model Dataset that the pluggable view should render.
		// description:
		//		Coming soon
		// tags:
		//		protected
		
		//Create menu provider
		this._createCMProvider();
		
		//Create supporting widgets (like toolbars, etc.)
		this._createSupportingWidgets();
		
		//Create the busy pane
		if (!this.parentPluggableView && !this.managingPluggableView) {
			this._createBusyPane();
		}

		//Initialize ourselves with the dataset. 
		this._initializeWithDataset();
	
		//Call superclass
		this.inherited(arguments);
	},
	
	startup: function(){
		this.inherited(arguments);
	},
	
	_createSelection: function(){
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// tags:
		//		protected
		
		var kwArgs = {
			pluggableView: this
		};
		return new ibm.tivoli.tip.navwidget.dojo.NavWidgetPluggableViewSelection(kwArgs); 
	},
	
	_createWidgetDefinition: function(){
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// tags:
		//		protected

		if (this.widgetDefinition) {
			//If a widget def instance was passed in, let's use that
			this.widgetDefinitionId = this.widgetDefinition.getWidgetId();
			
		} else if (this._usesWidgetDefinition) {
			var defaultWidgetDefInfo = this._getDefaultWidgetDefInfo();
			if (defaultWidgetDefInfo) {
				if(!this.widgetDefinitionId){
					this.widgetDefinitionId = defaultWidgetDefInfo.definition;
				}
				
				this.widgetDefinition 
					= ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.createWidgetDefinition(defaultWidgetDefInfo.className, 
							this.widgetDefinitionId, defaultWidgetDefInfo, this.dataset, this.getParentPluggableViewWidgetDefinition());
			} else {
				if (this._trace.isError()) {
					this._trace.error("Missing default widget def info for view = " + this.id, "_createWidgetDefinition");
				}
			}
		}
		
		//Let's store away in case we need to revert later
		this._originalWidgetDefinition = this.widgetDefinition;
	},
	
	_getDefaultWidgetDefInfo: function(){
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// tags:
		//		protected
	
		return null;
	},
	
	_createCMProvider: function(){
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// tags:
		//		protected
		
		
	},
	
	_createSupportingWidgets: function(){
		// summary:
		//		Give subclass a chance to create some widgets (like toolbars, etc.) during the setup of the view.
		// description:
		//		Give subclass a chance to create some widgets (like toolbars, etc.). This is called in postCreate BEFORE the view 
		//		has been initialized with the Dataset, so the widgets created here would typically not be dependent on info in the
		//		dataset.
		// tags:
		//		protected
		
		
	},
	
	_createBusyPane: function() {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// tags:
		//		protected
		
        this._busyPane = new ibm.tivoli.tip.dijit.BusyPane({id: this.id + "_busyPane"});
        dojo.style(this._busyPane.domNode, "zIndex", "10");
                
		//Bobby Joseph, 16Nov2011 - BEGIN
		//254806 Dojo1.6: nav...WithWidgetDefProcessing doesnot work with dojo16
		//http://bugs.dojotoolkit.org/ticket/12795
		//Need to set region for all BorderContainer children from dojo 1.6
		this._busyPane.region = "center";
		//Bobby Joseph, 16Nov2011 - END
		
		this.domNode.appendChild(this._busyPane.domNode);
		this._busyPane.startup();
		// Be sure it is initially hidden
		this.forceHideBusyPane();
	},
	
	_initializeWithDataset: function(){
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// tags:
		//		protected
		
		//Initialize ourselves with the dataset. If the parentPluggableView has been specified, we can skip
		//because setDataset will be called when parentPluggableView initializes its views.
		if (!this.parentPluggableView) {
			if (this.dataset || this.initializeWithNullDataset) {
				this.setDataset(this.dataset);
			}
		}
	},
		
	forceHideBusyPane: function() {
		// summary:
		//		Hides the busy pane if it is currently displayed regardless of the count
		// description:
		//		If this view has the busy pane, it forces the count to 1 and then calls
		//      hideBusyPane() to hide the busy pane.
       //      If this view does not have the busy pane, the request is passed on to the parent 
       //      pluggable view or managing pluggable view.
		// tags:
		//		public
		if (this._busyPane) {
			this._busyPaneCount = 1;
			this.hideBusyPane();
		}		
		else {
			// Send to parent view, if one exists
			if (this.parentPluggableView) {
				this.parentPluggableView.forceHideBusyPane();
			// Otherwise, send to managingPluggableView if one exists
			} else if (this.managingPluggableView) {
				this.managingPluggableView.forceHideBusyPane();
			}
		}
	},
	
	hideBusyPane: function() {
		// summary:
		//		Hides the busy pane if it is currently displayed and the decremented count is zero
		// description:
		//		If this view has the busy pane, it decrements the count and it checks if the busy 
		//      pane's current state is displayed, and if the count is back to zero, hides it.  
		//      If this view does not have the busy pane, the request is passed on to the parent 
		//      pluggable view or managing pluggable view.
		// tags:
		//		public
		
		// Does this view have the busy pane?
		if (this._busyPane) {
			// decrement the count, but not past zero
			if (this._busyPaneCount > 0) {
				this._busyPaneCount--;
			}
			// Is the busy pane currently visible? (ibm.tivoli.tip.dijit.BusyPane uses
			// display = "none" when it is hidden)
			if (dojo.style(this._busyPane.domNode, "display") != "none") {
	            if (this._busyPaneCount == 0) {
	            	this._busyPane.hide();
	            }
			}
		}
		else {
			// Send to parent view, if one exists
			if (this.parentPluggableView) {
				this.parentPluggableView.hideBusyPane();
			// Otherwise, send to managingPluggableView if one exists
			} else if (this.managingPluggableView) {
				this.managingPluggableView.hideBusyPane();
			}
		}
	},

	showBusyPane: function() {
		// summary:
		//		Shows the busy pane if it is not currently displayed
		// description:
		//		If this view has the busy pane, it increments the count and checks if the 
		//      busy pane's current state is hidden, and if so, shows it.  If this view does 
		//      not have the busy pane, the request is passed on to the parent pluggable view
      //      or managing pluggable view
		// tags:
		//		public
		
		// Does this view have the busy pane?
		if (this._busyPane) {
			this._busyPaneCount++;
			// Is the busy pane currently hidden? (ibm.tivoli.tip.dijit.BusyPane uses
			// display = "none" when it is hidden)
			if (dojo.style(this._busyPane.domNode, "display") == "none") {
				// this._busyPane.show(true);
				this._busyPane.show();
			}
		}
		else {
			// Send to the parent view, if one exists
			if (this.parentPluggableView) {
				this.parentPluggableView.showBusyPane();
			}
			// Otherwise, send to the managing pluggable view, if one exists
			else if (this.managingPluggableView) {
				this.managingPluggableView.showBusyPane();
			}
		}
	},
	
	displayException: function (/*Object*/ ex, /*Object*/dontTraceMessage) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon

		// Send to the parent view, if one exists
		if (this.parentPluggableView) {
			this.parentPluggableView.displayException(ex, dontTraceMessage);
		}
		// Otherwise, send to the managing pluggable view, if one exists
		else if (this.managingPluggableView) {
			this.managingPluggableView.displayException(ex, dontTraceMessage);
		} else {
			//No where else to turn, so we need to deal with it
			if (!this.defaultMessageHandler) {
				//Create our own handler if necessary
				this.defaultMessageHandler = new ibm.tivoli.tip.navwidget.dijit.DefaultMessageHandler({});
			}
			this.defaultMessageHandler.displayException(ex, dontTraceMessage);
		}
	},
	
	displayMessage: function(/*Object*/ msgId, /*Object*/ msgSeverity, /*Object*/ msgText) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		
		// Send to the parent view, if one exists
		if (this.parentPluggableView) {
			this.parentPluggableView.displayMessage(msgId, msgSeverity, msgText);
		}
		// Otherwise, send to the managing pluggable view, if one exists
		else if (this.managingPluggableView) {
			this.managingPluggableView.displayMessage(msgId, msgSeverity, msgText);
		} else {
			//No where else to turn, so we need to deal with it
			if (!this.defaultMessageHandler) {
				//Create our own handler if necessary
				this.defaultMessageHandler = new ibm.tivoli.tip.navwidget.dijit.DefaultMessageHandler({});
			}
			this.defaultMessageHandler.displayMessage(msgId, msgSeverity, msgText);
		}
	},
	
	isApplicable: function(/*ibm.tivoli.navmodel.Dataset*/dataset){
		// summary:
		//		Analyzes dataset data and/or metadata to decide if this view makes sense for the given dataset.
		// dataset:
		//		The Client Nav Model Dataset that the pluggable view should render.
		// description:
		//		Analyzes dataset data and/or metadata to decide if this view makes for the given dataset. For
		//		example, if this view represents a topology, it might not make sense for it to render a dataset
		//		without relationships. Default implementation checks the dataset's viewType against the explicitly
		//		excluded viewType's.
		// tags:
		//		public
	
		var applicable = true;
		
		//Let's see if the datseet viewType matches one of the view primaryViewType's
		if (dataset) {
			var datasetViewType = dataset.getViewType();
			var excludedViewTypes = this.getExcludedViewTypes();
			for (var i = 0; i < excludedViewTypes.length; i++) {
				var exViewType = excludedViewTypes[i];
				if (exViewType == datasetViewType) {
					applicable = false;
					break;
				}
			}
		} else {
			applicable = false;
		}
		
		return applicable;
	},
	
	getPrimaryViewTypes: function(){
		// summary:
		//		Coming soon
		// dataset:
		//		Coming soon
		// description:
		//		Coming soon
		// tags:
		//		public
	
		var primaryViewTypes = [];
		if (this.viewTypeSpec) {
			//Split apart the spec string
			var breakdown = this.viewTypeSpec.split(",");
			
			//Examine the individual viewTypes
			for (var i = 0; i < breakdown.length; i++) {
				var viewType = breakdown[i];
				var firstChar = viewType.substring(0, 1);
				if (firstChar != "!") { //If starts with "!" it would indicate an excluded viewType, so skip
					primaryViewTypes.push(viewType);
				}
			}
		}
		return primaryViewTypes;
	},
	
	getExcludedViewTypes: function(){
		// summary:
		//		Coming soon
		// dataset:
		//		Coming soon
		// description:
		//		Coming soon
		// tags:
		//		public
	
		var excludedViewTypes = [];
		if (this.viewTypeSpec) {
			//Split apart the spec string
			var breakdown = this.viewTypeSpec.split(",");
			
			//Examine the individual viewTypes
			for (var i = 0; i < breakdown.length; i++) {
				var viewType = breakdown[i];
				var firstChar = viewType.substring(0, 1);
				if (firstChar == "!") { //If starts with "!" we've found an excluded viewType
					excludedViewTypes.push(viewType.substring(1));
				}
			}
		}
		return excludedViewTypes;
	},

	getPreferredPluggableView: function(/*ibm.tivoli.navmodel.Dataset*/dataset){
		// summary:
		//		Coming soon
		// dataset:
		//		Coming soon
		// description:
		//		Comming soon
		// tags:
		//		public
	
		var datasetViewType = dataset;
		var preferredView = null;
		var iterator = this.getPluggableViewIterator();
		while (!iterator.atEnd()) {
			var view = iterator.get().value;
			if (view.isApplicable(dataset)) {
				//Let's see if the dataset viewType matches one of the view primaryViewType's
				var datasetViewType = dataset.getViewType();
				var matchesPrimaryViewType = false;
				var primaryViewTypes = view.getPrimaryViewTypes();
				for (var i = 0; i < primaryViewTypes.length; i++) {
					var primViewType = primaryViewTypes[i];
					if (primViewType == datasetViewType) {
						matchesPrimaryViewType = true;
						break;
					}
				}
				
				if (matchesPrimaryViewType) {
					//We've found an applicable view matching a primary view type, so we can quit looking
					preferredView = view;
					break;
				} else {
					//The view is applicable, but primary view type isn't a match... so if we haven't found another
					//applicable view along the way, let's save as best match so far
					if (!preferredView) {
						preferredView = view;
					}
				}
			}
		}
		return preferredView;
	},
	
	getLabel: function(){
		// summary:
		//		Provides an appropriate label appropriate for use in a menu for switching views, etc.
		// description:
		//		Provides an appropriate label appropriate for use in a menu for switching views, etc.
		// tags:
		//		public
	
		return this.title;
	},
	
	getLabelIconClass: function(){
		// summary:
		//		Provides an appropriate iconClass for use next to a label in a menu for switching views, etc.
		// description:
		//		Provides an appropriate iconClass for use next to a label in a menu for switching views, etc.
		// tags:
		//		public
	
		return this.labelIconClass;
	},
	
	saveWidgetDelta: function(){
		// summary:
		//		Save any changes to the user delta of the current widget def's for the view.
		// description:
		//		Save any changes to the user delta of the current widget def's for the view.
		// tags:
		//		public
		//
		// sub-class would want to save any information to the delta and then call base class method
		
		if (this.widgetDefinition && this.widgetDefinition.hasPendingChanges()) {
			this.widgetDefinition.saveWidgetDelta();
		}
	},

	clearWidgetDelta: function(){
		// summary:
		//		Clear information from user delta of the current widget def's for the view.
		// description:
		//		Clear information from user delta of the current widget def's for the view.
		// tags:
		//		public
		//		
		// sub-class would want to remove any information from the delta and then call base class method

		if (this.widgetDefinition) {
			this.widgetDefinition.saveWidgetDelta();
		}
	},
	
	hasWidgetDeltaPendingChanges: function(){
		// summary:
		//		Indicates whether the user delta has any updates to be saved.
		// description:
		//		Indicates whether the user delta has any updates to be saved. Intended to be 
		//		used as a convenience for a toolbar or menu providers (e.g., help to control enablement
		//		of "Save" action)
		// tags:
		//		public
		
		return (this.widgetDefinition && this.widgetDefinition.hasPendingChanges()); 
	},

	_setDatasetActive: false,
	setDataset: function(/*ibm.tivoli.navmodel.Dataset*/dataset, /*ibm.tivoli.navmodel.AbstractWidgetDef?*/datasetWidgetDefinition, /*ibm.tivoli.tip.navwidget.dojo.NavWidgetBaseViewState?*/state){
		// summary:
		//		Kick off the setDataset life-cycle of a widget. Default implementation will call (in order) the
		//      following protected functions:
		//			- _setDatasetCleanUpAfterLastDataset
		//			- _setDatasetInit
		//			- _setDatasetSetWidgetDefs
		//			- _setDatasetCreatePluggableViews();
		//			- _setDatasetProcessWidgetDefs
		//			- _setDatasetLoadData
		//			- _setDatasetComplete
		//
		//		It is NOT recommended for subclasses to override setDataset, but instead one or more of the
		//		protected methods that get called. In particular, it is expected subclasses would override 
		//		_setDatasetLoadData. Be sure to call super on any of the overridden methods.
		// dataset:
		//		The Client Nav Model Dataset that the pluggable view should render.
		// datasetWidgetDefinition:
		//		Optional widget definition to use while visualizing this particular dataset
		// description:
		//		Tells this pluggable view it's time to render a new dataset
		// tags:
		//		public
	
		this._setDatasetCleanUpAfterLastDataset();
		this._setDatasetInit(dataset);
		this._setDatasetSetWidgetDefs(dataset, datasetWidgetDefinition, state);
		this._setDatasetCreatePluggableViews();
		this._setDatasetProcessWidgetDefs(dataset);
		this._setDatasetLoadData(dataset);
	},
	
	_setDatasetCleanUpAfterLastDataset: function(){
		// summary:
		//		Called during the setDataset lifecycle to allow clean-up related to previous dataset to occur. The base class implementation
		//		takes care of deregistering listening for event changes. Subclasses will probably NOT need to override.
		// description:
		//		Tells the pluggable view it's time to clean up before the new dataset is used.
		// tags:
		//		protected

		//Stop listening for changes to the old dataset
		this._unregisterDatasetEventListener(this.dataset);
	},
	
	_setDatasetInit: function(/*ibm.tivoli.navmodel.Dataset*/ dataset){
		// summary:
		//		Called during the setDataset lifecycle for any initialization that should take place in the view before
		//		starting to pull information out of the dataset for rendering. Subclasses will probably NOT need to override.
		// dataset:
		//		The Client Nav Model Dataset that the pluggable view should render.
		// description:
		//		Tells the pluggable view it's time do initialization for the new dataset
		// tags:
		//		protected
		
		//Remember we're in setDataset
		this._setDatasetActive = true;
		
		//Store away the dataset
		this.dataset = dataset;
		
		//Start listening for changes to the new Dataset. If the parentPluggableView has been specified, we don't
		//need to explicitly listen as the parentPluggableView will pass changes along to us.
		if (!this.parentPluggableView && this.dataset) {
			this._registerDatasetEventListener();
		}
		
		//Let selection know there's a new dataset; also clears the selection
		this.selection.setDataset(this.dataset);
	},
	
	_setDatasetNewWidgetDef: false,
	_setDatasetState: null,
	_setDatasetSetWidgetDefs: function(/*ibm.tivoli.navmodel.Dataset*/ dataset, /*ibm.tivoli.navmodel.AbstractWidgetDef?*/datasetWidgetDefinition, /*ibm.tivoli.tip.navwidget.dojo.NavWidgetBaseViewState?*/state){
		// summary:
		//		Called during the setDataset lifecycle for setting widget defs specified in the dataset to the view
		//		Subclasses will probably NOT need to override.
		// dataset:
		//		The Client Nav Model Dataset that the pluggable view should render.
		// datasetWidgetDefinition:
		//		Optional widget definition to use while visualizing this particular dataset
		// description:
		//		Sets widget defs specified in the dataset to the view
		// tags:
		//		protected
		
		//Remember the current def id
		var oldDefId = this.widgetDefinitionId;
		var oldDefIdContainsVersion = false;
		if (oldDefId && oldDefId.indexOf("-") > 0) {
			oldDefIdContainsVersion = true;
		}
 
		if (datasetWidgetDefinition) {
			//We have a widget def explicitly set to go with this dataset, so respect above everything else
			this.widgetDefinitionId = datasetWidgetDefinition.getWidgetId() + (oldDefIdContainsVersion ? ("-" + datasetWidgetDefinition.getWidgetVersion()) : "");
			this.widgetDefinition = datasetWidgetDefinition;
		} else if (dataset && this._usesWidgetDefinition) {
			//See if the dataset specifies it's own widget def
			var defaultWidgetDefInfo = this._getDefaultWidgetDefInfo();
			if (defaultWidgetDefInfo) {
				var kwArgs = {
						widgetType: defaultWidgetDefInfo.widget
					};
				var datasetWidgetDef = dataset.getWidgetDef(kwArgs);
				if(datasetWidgetDef){
					this.widgetDefinitionId = datasetWidgetDef.getWidgetId() + (oldDefIdContainsVersion ? ("-" + datasetWidgetDef.getWidgetVersion()) : "");
					this.widgetDefinition = datasetWidgetDef;
				} else {
					// We didn't get a widget def from the dataset, let's revert to the original widget deft
					if (this._originalWidgetDefinition) {
						this.widgetDefinitionId = this._originalWidgetDefinition.getWidgetId() + (oldDefIdContainsVersion ? ("-" + this._originalWidgetDefinition.getWidgetVersion()) : "");
						this.widgetDefinition = this._originalWidgetDefinition;
					}
				}
			}
		}
		
		//Let's store the state away as well in case it has somethng in it that should override the widget def
		this._setDatasetState = state;
		
		//Let's see if the widget def is different than before
		this._setDatasetNewWidgetDef = (this.widgetDefinitionId != oldDefId);
	},
	
	_setDatasetProcessWidgetDefs: function(/*ibm.tivoli.navmodel.Dataset*/ dataset){
		// summary:
		//		Called during the setDataset lifecycle for processing widet definitions specified in the dataset.
		// dataset:
		//		The Client Nav Model Dataset that the pluggable view should render.
		// description:
		//		Processes widget definition specified in the datasets. This widget definition is already apllied to the view.
		// tags:
		//		protected
		//
		
		//Delegate impl to subclass.
	},
	
	_setDatasetLoadDataActive: false,
	_setDatasetLoadData: function(/*ibm.tivoli.navmodel.Dataset*/ dataset){
		// summary:
		//		Called during the setDataset lifecycle to tell the implementor to process the dataset to set up
		//		rendering (e.g., start pulling info out of dataset, set data models on any children, etc.). The 
		//		base implementation does nothing, so all work delegated to subclass.
		// dataset:
		//		The Client Nav Model Dataset that the pluggable view should render.
		// description:
		//		Tells the pluggable view it's time to process the new dataset
		// tags:
		//		protected
		
		this._setDatasetLoadDataActive = true;
		if (this._setDatasetLoadDataCompletesSynchronously) {
			this._onSetDatasetLoadComplete(dataset);
		}
	},
	
	_onSetDatasetLoadComplete: function(/*ibm.tivoli.navmodel.Dataset*/ dataset){ 
		// summary:
		//		Event fired when an the load of the dataset is completed (potentially asynchronously). It is recommended to use dojo.connect rather than overriding this method.
		// dataset: Dataset
		//		The client nav model dataset that this view is now rendering
		// tags:
		//		protected callback
		
		this._setDatasetLoadDataActive = false;
		this._setDatasetComplete(dataset);
	},
	
	isSetDatasetActive: function() {
		return this._setDatasetActive;
	},
	
	isSetDatasetNewWidgetDef: function() {
		return this._setDatasetNewWidgetDef;
	},
	
	getSetDatasetState: function() {
		return this._setDatasetState;
	},
	
	isSetDatasetLoadDataActive: function() {
		return this._setDatasetLoadDataActive;
	},
	
	_setDatasetComplete: function(/*ibm.tivoli.navmodel.Dataset*/dataset){
		// summary:
		//		Called during the setDataset lifecycle as the last step. Base implementation will make sure
		//		this view is setup to receive data change events from the Dataset. Subclasses will probably NOT need to override.
		// dataset:
		//		The Client Nav Model Dataset that the pluggable view should render.
		// description:
		//		Tells the pluggable view we're done setting up the dataset
		// tags:
		//		protected
		
		//Create an "inner class" to help us determine if all of the subviews have called back with onSetDatasetComplete
		var mainOnSetDatasetHandlerInnerClass = {
				view: this,
				callbackCounter: 0,
				
				//Should be invoked in the scope of mainOnSetDatasetHandlerInnerClass, so "this" refers to this inner class rather than the pluggable view
				onSetDatasetHandler: 
					function(dataset) {
						//Increment counter
						this.callbackCounter++;
						
						if (this.callbackCounter >= this.view.getPluggableViewCount()) {
							//Fire event
							dojo.hitch(this.view, this.view.onSetDataset, dataset).call();
						}
					}
		};
		var mainDataSetHandlerFunc = dojo.hitch(mainOnSetDatasetHandlerInnerClass, mainOnSetDatasetHandlerInnerClass.onSetDatasetHandler, dataset);
		
		//Notify the sub-views of the new dataset
		var iterator = this.getPluggableViewIterator();
		if (iterator.atEnd()) {
			//There are no pluggable views, so we let's complete the setDataset process
			mainDataSetHandlerFunc.call();
		} else {
			//We want to iterate through the pluggable views and listen for when they hit onSetDatasetComplete
			while (!iterator.atEnd()) {
				var view = iterator.get().value;

					//Provide an "inner class" to support a handler for the "onSetDataset" event that can remove itself as a listener when done
					var subHandlerInnerClass = {
						mainHandler: mainDataSetHandlerFunc,
						handle: null,
						
						//Should be invoked in the scope of subHandlerInnerClass, so "this" refers to this inner class
						handler: function(dataset){
								this.mainHandler.call();
								
							     //Disconnect the handler
							     dojo.disconnect(this.handle);
							}
					};
					var handle = dojo.connect(view, "onSetDataset", dojo.hitch(subHandlerInnerClass, subHandlerInnerClass.handler));
					subHandlerInnerClass.handle = handle;
						
					if (view.isApplicable(dataset)) {
						try {
							//Since views are pluggable... let's take some care with try/catch, so that a custom one doesn't take us out if it craps
							//out while handling the dataset
							var state = this.getSetDatasetState();
							var viewState = (state) ? state.getPluggableViewState(view.viewId) : null;
							view.setDataset(dataset, null, viewState);
						} catch (ex) {
							if (this._trace.isError()) {
								this._trace.error("Cannot set dataset on view = " + view.id + "; ex = " + ex.name + ": " + ex.message, "_setDatasetComplete");
							}
						}
					} else {
						//View not applicable, so let's clear out the view
						try {
							view.setDataset(null);
						} catch (ex) {
							if (this._trace.isError()) {
								this._trace.error("Unable to clear dataset on view = " + view.id + "; ex = " + ex.name + ": " + ex.message, "_setDatasetComplete");
							}
						}
					}
			}
		}
	},
	
	destroyRecursive: function(/*Boolean?*/ preserveDom){		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// preserveDom:
		//		
		
		//Overriding simply because on more than one occasion we've had trouble tracking down why this.destroy isn't being called on this
		//widget. The superclass's implementation is to do the following:
		//
		//			this._beingDestroyed = true;
		//			this.destroyDescendants(preserveDom);
		//			this.destroy(preserveDom); 
		//
		//If this.destroyDescendants craps out, then this.destroy doesn't get reached.
		
		var methodName = "destroyRecursive";
		
		this._beingDestroyed = true;
		try {
			this.destroyDescendants(preserveDom);
		} catch (ex) {
			if (this._trace.isError()) {
				this._trace.error("problem destroyingDescendants of " + this.id + "; ex = " + ex.name + ": " + ex.message, methodName);
			}
		}
		
		try {
			this.destroy(preserveDom);
		} catch (ex) {
			if (this._trace.isError()) {
				this._trace.error("problem destroying " + this.id  + "; ex = " + ex.name + ": " + ex.message, methodName);
			}
		}
	}, 
	
	destroy: function(){
		// summary:
		//		Override's the widget's destroy to clean up some view-specific things.
		// description:
		//		Cleans up some view-specific things. For example, will destroy the dataset if
		//		this view is NOT part of a Nav Widget.
		// tags:
		//		public
		
		if (this._isDestroyed) {
			return;
		} else {
			this._isDestroyed = true;
		}
		
		// Destroy the busy pane if it exists
		if (this._busyPane) {
			this._busyPane.destroyRecursive(false);
			this._busyPane = null;
		}
		
		//Do clean up related to dataset
		if (this.dataset) {
			//Stop listening for changes to the old dataset
			this._unregisterDatasetEventListener(this.dataset);
			
			//destroy the dataset if this view is NOT a subview of another view, AND no one else is listening for changes
			if (!this.parentPluggableView && this.dataset.getEventListenerCount() <= 0 && this.destroyDatasetWhenDone) {
				this.dataset.destroy();
			}
		}
		
		//Do clean up related to widget def
		if(this.widgetDefinition) {
			this.widgetDefinition.destroy();
		}
		
		this.inherited(arguments);
	},
	
	isDestroyed: function() {
		return this._isDestroyed;
	},
	
	getParentPluggableViewWidgetDefinition: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		var parent = (this.parentPluggableView) ? this.parentPluggableView : this.managingPluggableView;
		if (parent) { 
			return parent.widgetDefinition;
		}
		return null;
	},
	
	getTopLevelPluggableViewWidgetDefinition: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		var topLevelPluggableView = this.getTopLevelPluggableView();
		return topLevelPluggableView.widgetDefinition;
	},
	
	getTopLevelPluggableView: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		var topLevelPluggableView = this;
		
		var parent = (this.parentPluggableView) ? this.parentPluggableView : this.managingPluggableView;
		while (parent != null) {
			topLevelPluggableView = parent;
			parent = parent.parentPluggableView;
		}
		return topLevelPluggableView;
	},
	
	getNearestPluggableViewWidgetDefinition: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		var nearestPluggableViewWidgetDefinition = this.widgetDefinition;
		
		var parent = (this.parentPluggableView) ? this.parentPluggableView : this.managingPluggableView;
		while (parent != null && nearestPluggableViewWidgetDefinition == null) {
			nearestPluggableViewWidgetDefinition = parent.widgetDefinition;
			parent = parent.parentPluggableView;
		}
		return nearestPluggableViewWidgetDefinition;
	},
	
	isRootLevelPluggableView: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		return this.parentPluggableView == null && this.managingPluggableView == null;
	},
	
	isAncestorOfPluggableView: function(viewToFind) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// viewToFind:
		//		
		var isAnscestorOf = (this.viewId == viewToFind.viewId) || this.getPluggableView(viewToFind.viewId);
		var iterator = this.getPluggableViewIterator();
		while (!isAnscestorOf && !iterator.atEnd()) {
			var view = iterator.get().value;
			isAnscestorOf = view.isAncestorOfPluggableView(viewToFind);
		}
		return isAnscestorOf;
	},
	
	//*********************************************************************
	
	_cleanUpImagePath: function(imagePath) {
		// summary:
		//		Convenience for subclasses to clean up image path (e.g., look for [widget] tag
		// 		in path and replace appropriately).
		// imagePath:
		//		Coming soon
		// description:
		//		Convenience for subclasses to clean up image path (e.g., look for [widget] tag
		// 		in path and replace appropriately).
		// tags:
		//		protected
		
		return ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.cleanUpImagePath(imagePath);
	},
	
	//******************************************************************************************
	// Functions for pluggable subview management
	//******************************************************************************************
    addPluggableView: function(view) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// view:
		//		
		this._childPluggableViews.add(view.viewId, view);
		
		this._addPluggableViewToContainer(view);  
	},
	
	removeAllPluggableViews: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// view:
		//	
		
		var iterator = this.getPluggableViewIterator();
		while (!iterator.atEnd()) {
			var view = iterator.get().value;
			this.removePluggableView(view);
		}
	},
	
	removePluggableView: function(view) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// view:
		//	
		
		//Remove from the list of views
		this._childPluggableViews.remove(view.viewId);
		
		//Remove from the Dojo container
		this._removePluggableViewFromContainer(view); 
		
		//Destroy the view
		view.destroyRecursive();
	},

	getPluggableView: function(viewId) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// viewId:
		//		
		return this._childPluggableViews.item(viewId);
	},
	
	isPluggableViewApplicable: function(viewId, dataset) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// viewId:
		//		
		
		var applicable = false;
		
		var view = this.getPluggableView(viewId);
		if (view) {
			applicable = view.isApplicable(dataset);
		}
		
		return applicable;
	},
	
	makePluggableViewVisible: function(viewId) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// viewId:
		//		
		
		//Delegate to subclass
	},
	
	getVisiblePluggableView: function(viewId) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// viewId:
		//		

		//Delegate to subclass
		return null;
	},
	
    getPluggableViewIterator: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		return this._childPluggableViews.getIterator();
	},
	
	forEachPluggableView: function(fn, scope) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// fn:
		//		
		// scope:
		//		
		this._childPluggableViews.forEach(fn, scope);
	},
	
	getPluggableViewCount: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		return this._childPluggableViews.count;
	},
	
	getState: function() {
		// summary:
		//		Return an instance of NavWidgetBaseViewState representing the current state of the view. 
		// description:
		//		Return an instance of NavWidgetBaseViewState representing the current state of the view. 
		//
		//		It is NOT recommended for subclasses to override getState, but instead override
		//		_getStateData (which will be called by this function).
		// tags:
		//		public

		//Create a base state 
		var stateData = this._getStateData();
		var state = new ibm.tivoli.tip.navwidget.dojo.NavWidgetBaseViewState({
			view: this,
			dataset: this.dataset,
			stateData: stateData
		});
		
		//Get child states
		this._populatePluggableViewStates(state);
		
		//Return the state
		return state;
	},
	
	_getStateData: function() {
		// summary:
		//		Returns an object of the implementer's choosing which will be put in the 
		//		NavWidgetBaseViewState returned by getState.
		// description:
		//		Coming soon
		// tags:
		//		protected
		
		return null;
	},
	
	_populatePluggableViewStates: function(state) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// tags:
		//		protected
		
		var iterator = this.getPluggableViewIterator();
		while (!iterator.atEnd()) {
			var view = iterator.get().value;
			state.addPluggableViewState(view.viewId, view.getState());
		}
	},
	
	getPluggableViewState: function(viewId) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		
		var viewState = null;
		var view = this.getState(viewId);
		if (view) {
			viewState = view.getState();
		}
		return state;
	},
	
	restoreFromState: function(state) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon

		this._restorePluggableViewsFromStates(state);
	},
	
	_restorePluggableViewsFromStates: function(state) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// tags:
		//		protected
		
		var iterator = this.getPluggableViewIterator();
		while (!iterator.atEnd()) {
			var view = iterator.get().value;
			if (state) {
				view.restoreFromState(state.getPluggableViewState(view.viewId));
			} else {
				view.restoreFromState(null);
			}
		}
	},
	
	getViewSpecificTasks: function() {		
		// summary:
		//		Get tasks / actions specific to the pluggable view.
		// description:
		//		Get tasks / actions specific to the pluggable view.
		//		Subclasses should override this function.
	},

	executeViewSpecificTask: function(commandName) {		
		// summary:
		//		Executes the commandName in the pluggable view.
		// description:
		//		Executes the commandName in the pluggable view.
		//		Subclasses should override this function.
	},
	
	getSiblings: function(){
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		var siblings = [];
		if(this.parentPluggableView){
			var iterator = this.parentPluggableView.getPluggableViewIterator();
			while (!iterator.atEnd()) {
				siblings.push(iterator.get().value);
			}
		}
		return siblings;
	},	
	
	_getPluggableViewDefinitions: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		var childPluggableViewDefinitions = [];
		if (this.widgetDefinition) {
			childPluggableViewDefinitions = this.widgetDefinition.getViews();
		}
		return childPluggableViewDefinitions;
	},
	
	_setDatasetCreatePluggableViewsPreviouslyCalled: false, //Flag to keep track of whether we've created views before or not
	_setDatasetCreatePluggableViews: function() {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// tags:
		//		protected
		
		//If this is the first time being called or we have a new widget def, then go ahead with creation
		if (!this._setDatasetCreatePluggableViewsPreviouslyCalled || this.isSetDatasetNewWidgetDef()) {
			//Let's brute force it and remove all existing pluggable views. NOTE: Could potentially be smarter and try to reuse
			//views that already exist (e.g., have same ID, etc. as on of the pluggable view defs we'll be processing)
			this.removeAllPluggableViews(); 

			//Process the new pluggable views
			var childPluggableViewDefinitions = this._getPluggableViewDefinitions();
			if (childPluggableViewDefinitions) {
				for (var i = 0; i < childPluggableViewDefinitions.length; i++) {
					var viewDef = childPluggableViewDefinitions[i];
					
					var viewArgs = {
							id: this.id + "_" + viewDef.viewId,
							viewId: viewDef.viewId,
							viewTypeSpec: viewDef.viewType,
							dataset: this.dataset,
							cmProvider: this.cmProvider,
							daHelper: this.daHelper,
							widgetDefinitionId: viewDef.definition,
							parentPluggableView: this
						};
	
					var viewInstance = ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getObjectInstance(viewDef.className, viewArgs);
					this._setDatasetCreatePluggableViewsAugmentViewAttributes(viewInstance, viewDef);
					if (viewInstance) {
						this.addPluggableView(viewInstance);
					}
				}
			}
			
			//Set up to listener for selection changes on all the views
			var iterator = this.getPluggableViewIterator();
			while (!iterator.atEnd()) {
				var view = iterator.get().value;
				var chainSelectionChanged = function (objectContext) {
					if (!this.selection.isChanging()) {
						this.selection.chainSelectionChanged(objectContext);
					}
				};
				dojo.connect(view.selection, "onSelectionChanged", this, chainSelectionChanged);
			}
		}
		
		//Remember that we've created pluggable views before
		this._setDatasetCreatePluggableViewsPreviouslyCalled = true;
	},
	
	_setDatasetCreatePluggableViewsAugmentViewAttributes: function(view, viewDef) {
		// summary:
		//		TODO
		// description:
		//		TODO
		// tags:
		//		protected
		
	},
	
	_addPluggableViewToContainer: function(view) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// tags:
		//		protected
		
		this.addChild(view);
	},
	
	_removePluggableViewFromContainer: function(view) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// tags:
		//		protected
		
		this.removeChild(view);
	},
	
	//*****************************************************************************************************
	// Events we fire
	//*****************************************************************************************************
	onFilterChange: function(){
		// summary:
		//		Event fired when an filter is changed
		// tags:
		//		callback
	},
	
	onSetDataset: function(/*ibm.tivoli.navmodel.Dataset*/ dataset){
		// summary:
		//		Event fired when an the dataset is set
		// dataset: Dataset
		//		The client nav model dataset that this view is now rendering
		// tags:
		//		callback
		
		//Mark we're done setting the dataset
		this._setDatasetNewWidgetDef = false;
		this._setDatasetActive = false;
		
		//Delegate anything else to listeners
	},
	
	//*****************************************************************************************************
	// Data change event handling management
	//*****************************************************************************************************
	// summary:
	//		Base class for a handler to deal with Dataset change events. By itself, it doesn't maintain
	//		any state to speak of, but it more of a convenience to mixin to another class.
	// description:
	//		Coming soon
	//
	
	//***********************************************************************************************/
	
	_registerDatasetEventListener: function(){
		// summary:
		//		Called during the setDataset lifecycle. Base implementation does... Subclasses may not have a need to override.
		// description:
		//		Tells the pluggable view it's time to register an event listener for the dataset
		// tags:
		//		private
		
		//Start listening for changes
		if (this.dataset) {
			this.dataset.addEventListener(this, dojo.hitch(this, this._datasetEventListenerCallback));
		}
	},
	
	_unregisterDatasetEventListener: function(){
		// summary:
		//		Called during the setDataset lifecycle. Base implementation does... Subclasses may not have a need to override.
		// description:
		//		Tells the pluggable view it's time to register an event listener for the dataset
		// tags:
		//		private
		
		//Stop listening for dataset changes
		if (this.dataset) {
			this.dataset.removeEventListener(this);
		}
	},
	
	//***************************************************************************************************************
	_datasetEventListenerCallback: function (/*Object*/ listenerObj, /*Object*/ event) {
		// summary:
		//		Callback for when there are changes within the Dataset we've registered to listen for updates to.
		// listenerObj:
		//		An object representing the listener registered with the Dataset.
		// event:
		//		An object containing information about what changed in the Dataset.
		// description:
		//		Callback for when there are changes within the Dataset currently being rendered. It will call one of the 
		//		specific handlers based on the change type in the Event. It's not anticipated any subclass should need to 
		//		override this callback.
		// tags:
		//		private
		var type = event.getType();
		if (this._trace.isFinest()) {
			this._trace.entering("event received, type = " + type, "_datasetEventListenerCallback");
		}
		//Handle the different event types
		switch (type) {
			case "changed":
				this._handleChangedEvent(event);
				break;
			case "labelChanged":
				this._handleLabelChangedEvent(event);
				break;
			case "statusChanged":
				this._handleStatusChangedEvent(event);
				break;	
			case "itemsAdded":
				this._handleItemsAddedEvent(event);
				break;
			case "itemsRemoved":
				this._handleItemsRemovedEvent(event);
				break;
			case "itemStatusChanged":
				this._handleItemStatusChangedEvent(event);
				break;
			case "itemPropertiesChanged":
				this._handleItemPropertiesChangedEvent(event);
				break;
			case "linksAdded":
				this._handleLinksAddedEvent(event);
				break;
			case "linksRemoved":
				this._handleLinksRemovedEvent(event);
				break;
			case "linkStatusChanged":
				this._handleLinkStatusChangedEvent(event);
				break;
			case "linkPropertiesChanged":
				this._handleLinkPropertiesChangedEvent(event);
				break;
			case "linkChanged":
				this._handleLinkChangedEvent(event);
				break;
			case "relationshipStatusChanged":
				this._handleRelationshipStatusChangedEvent(event);
				break;
			case "relationshipPropertiesChanged":
				this._handleRelationshipPropertiesChangedEvent(event);
				break;
			case "exception":
				this._handleExceptionEvent(event);
				break;
			case "eventRouter":
				this._handleEventRouterEvent(event);
				break;
			default:
				if (this._trace.isError()) {
					this._trace.error("Invalid event type = " + type, "_datasetEventListenerCallback");
				}
				break;
		}
		
		//Notify pluggable views
		var iterator = this.getPluggableViewIterator();
		while (!iterator.atEnd()) {
			var view = iterator.get().value;
			view._datasetEventListenerCallback(listenerObj, event);
		}
		
		if (this._trace.isFinest()) {
			this._trace.exiting("event received, type = " + type, "_datasetEventListenerCallback");
		}
	},

	_handleChangedEvent: function(/*Object*/ event) {
		// summary:
		//		Handle "changed" event type.
		// event:
		//		An object containing information about what changed in the Dataset.
		// description:
		//		Subclass should override to if it wishes to deal with "changed" events.
		// tags:
		//		protected
		
	},

	_handleLabelChangedEvent: function(/*Object*/ event) {
		// summary:
		//		Handle "labelChanged" event type.
		// event:
		//		An object containing information about what changed in the Dataset.
		// description:
		//		Subclass should override to if it wishes to deal with "labelChanged" events.
		// tags:
		//		protected
		
	},
	
	_handleStatusChangedEvent: function(/*Object*/ event) {
		// summary:
		//		Handle "statusChanged" event type.
		// event:
		//		An object containing information about what changed in the Dataset.
		// description:
		//		Subclass should override to if it wishes to deal with "statusChanged" events.
		// tags:
		//		protected
		
	},
	
	_handleItemsAddedEvent: function(/*Object*/ event) {
		// summary:
		//		Handle "itemsAdded" event type.
		// event:
		//		An object containing information about what changed in the Dataset.
		// description:
		//		Subclass should override to if it wishes to deal with "itemsAdded" events.
		// tags:
		//		protected

	},

	_handleItemsRemovedEvent: function(/*Object*/ event) {
		// summary:
		//		Handle "itemsRemoved" event type.
		// event:
		//		An object containing information about what changed in the Dataset.
		// description:
		//		Subclass should override to if it wishes to deal with "itemsRemoved" events.
		// tags:
		//		protected
		
	},

	_handleItemStatusChangedEvent: function(/*Object*/ event) {
		// summary:
		//		Handle "itemStatusChanged" event type.
		// event:
		//		An object containing information about what changed in the Dataset.
		// description:
		//		Subclass should override to if it wishes to deal with "itemStatusChanged" events.
		// tags:
		//		protected
		
	},
	
	_handleItemPropertiesChangedEvent: function(/*Object*/ event) {
		// summary:
		//		Handle "itemPropertiesChanged" event type.
		// event:
		//		An object containing information about what changed in the Dataset.
		// description:
		//		Subclass should override to if it wishes to deal with "itemPropertiesChanged" events.
		// tags:
		//		protected
		
	},

	_handleLinksAddedEvent: function(/*Object*/ event) {
		// summary:
		//		Handle "linksAdded" event type.
		// event:
		//		An object containing information about what changed in the Dataset.
		// description:
		//		Subclass should override to if it wishes to deal with "linksAdded" events.
		// tags:
		//		protected
		

	},
	
	_handleLinksRemovedEvent: function(/*Object*/ event) {
		// summary:
		//		Handle "linksRemoved" event type.
		// event:
		//		An object containing information about what changed in the Dataset.
		// description:
		//		Subclass should override to if it wishes to deal with "linksRemoved" events.
		// tags:
		//		protected
		
	},
	
	_handleLinkStatusChangedEvent: function(/*Object*/ event) {
		// summary:
		//		Handle "linkStatusChanged" event type.
		// event:
		//		An object containing information about what changed in the Dataset.
		// description:
		//		Subclass should override to if it wishes to deal with "linkStatusChanged" events.
		// tags:
		//		protected
		
	},
	
	_handleLinkPropertiesChangedEvent: function(/*Object*/ event) {		
		// summary:
		//		Handle "linkPropertiesChanged" event type.
		// event:
		//		An object containing information about what changed in the Dataset.
		// description:
		//		Subclass should override to if it wishes to deal with "linkPropertiesChanged" events.
		// tags:
		//		protected
		
	},

	_handleLinkChangedEvent: function(/*Object*/ event) {
		// summary:
		//		Handle "linkChanged" event type.
		// event:
		//		An object containing information about what changed in the Dataset.
		// description:
		//		Subclass should override to if it wishes to deal with "linkChanged" events.
		// tags:
		//		protected
		
	},
	
	_handleRelationshipPropertiesChangedEvent: function(/*Object*/ event) {
		// summary:
		//		Handle "relationshipPropertiesChanged" event type.
		// event:
		//		An object containing information about what changed in the Dataset.
		// description:
		//		Subclass should override to if it wishes to deal with "relationshipPropertiesChanged" events.
		// tags:
		//		protected
		
	},
	
	_handleRelationshipStatusChangedEvent: function(/*Object*/ event) {
		// summary:
		//		Handle "relationshipStatusChanged" event type.
		// event:
		//		An object containing information about what changed in the Dataset.
		// description:
		//		Subclass should override to if it wishes to deal with "relationshipStatusChanged" events.
		// tags:
		//		protected

	},
	
	_handleExceptionEvent: function(/*Object*/ event) {
		// summary:
		//		Handle "exception" event type.
		// event:
		//		An object containing information the "exception" event
		// description:
		//		Subclass should override to if it wishes to deal with "exception" events.
		// tags:
		//		protected

	},
	
	_handleEventRouterEvent: function(/*Object*/ event) {
		// summary:
		//		Handle "eventRouter" event type.
		// event:
		//		An object containing information the "eventRouter" event
		// description:
		//		Subclass should override to if it wishes to deal with "eventRouter" events.
		// tags:
		//		protected

	}
});
});

},
'dojo/dnd/Container':function(){
define(["../main", "../Evented", "./common", "../parser"], function(dojo, Evented) {
	// module:
	//		dojo/dnd/Container
	// summary:
	//		TODOC


/*
	Container states:
		""		- normal state
		"Over"	- mouse over a container
	Container item states:
		""		- normal state
		"Over"	- mouse over a container item
*/

/*=====
dojo.declare("dojo.dnd.__ContainerArgs", [], {
	creator: function(){
		// summary:
		//		a creator function, which takes a data item, and returns an object like that:
		//		{node: newNode, data: usedData, type: arrayOfStrings}
	},

	// skipForm: Boolean
	//		don't start the drag operation, if clicked on form elements
	skipForm: false,

	// dropParent: Node||String
	//		node or node's id to use as the parent node for dropped items
	//		(must be underneath the 'node' parameter in the DOM)
	dropParent: null,

	// _skipStartup: Boolean
	//		skip startup(), which collects children, for deferred initialization
	//		(this is used in the markup mode)
	_skipStartup: false
});

dojo.dnd.Item = function(){
	// summary:
	//		Represents (one of) the source node(s) being dragged.
	//		Contains (at least) the "type" and "data" attributes.
	// type: String[]
	//		Type(s) of this item, by default this is ["text"]
	// data: Object
	//		Logical representation of the object being dragged.
	//		If the drag object's type is "text" then data is a String,
	//		if it's another type then data could be a different Object,
	//		perhaps a name/value hash.

	this.type = type;
	this.data = data;
}
=====*/

dojo.declare("dojo.dnd.Container", Evented, {
	// summary:
	//		a Container object, which knows when mouse hovers over it,
	//		and over which element it hovers

	// object attributes (for markup)
	skipForm: false,

	/*=====
	// current: DomNode
	//		The DOM node the mouse is currently hovered over
	current: null,

	// map: Hash<String, dojo.dnd.Item>
	//		Map from an item's id (which is also the DOMNode's id) to
	//		the dojo.dnd.Item itself.
	map: {},
	=====*/

	constructor: function(node, params){
		// summary:
		//		a constructor of the Container
		// node: Node
		//		node or node's id to build the container on
		// params: dojo.dnd.__ContainerArgs
		//		a dictionary of parameters
		this.node = dojo.byId(node);
		if(!params){ params = {}; }
		this.creator = params.creator || null;
		this.skipForm = params.skipForm;
		this.parent = params.dropParent && dojo.byId(params.dropParent);

		// class-specific variables
		this.map = {};
		this.current = null;

		// states
		this.containerState = "";
		dojo.addClass(this.node, "dojoDndContainer");

		// mark up children
		if(!(params && params._skipStartup)){
			this.startup();
		}

		// set up events
		this.events = [
			dojo.connect(this.node, "onmouseover", this, "onMouseOver"),
			dojo.connect(this.node, "onmouseout",  this, "onMouseOut"),
			// cancel text selection and text dragging
			dojo.connect(this.node, "ondragstart",   this, "onSelectStart"),
			dojo.connect(this.node, "onselectstart", this, "onSelectStart")
		];
	},

	// object attributes (for markup)
	creator: function(){
		// summary:
		//		creator function, dummy at the moment
	},

	// abstract access to the map
	getItem: function(/*String*/ key){
		// summary:
		//		returns a data item by its key (id)
		return this.map[key];	// dojo.dnd.Item
	},
	setItem: function(/*String*/ key, /*dojo.dnd.Item*/ data){
		// summary:
		//		associates a data item with its key (id)
		this.map[key] = data;
	},
	delItem: function(/*String*/ key){
		// summary:
		//		removes a data item from the map by its key (id)
		delete this.map[key];
	},
	forInItems: function(/*Function*/ f, /*Object?*/ o){
		// summary:
		//		iterates over a data map skipping members that
		//		are present in the empty object (IE and/or 3rd-party libraries).
		o = o || dojo.global;
		var m = this.map, e = dojo.dnd._empty;
		for(var i in m){
			if(i in e){ continue; }
			f.call(o, m[i], i, this);
		}
		return o;	// Object
	},
	clearItems: function(){
		// summary:
		//		removes all data items from the map
		this.map = {};
	},

	// methods
	getAllNodes: function(){
		// summary:
		//		returns a list (an array) of all valid child nodes
		return dojo.query("> .dojoDndItem", this.parent);	// NodeList
	},
	sync: function(){
		// summary:
		//		sync up the node list with the data map
		var map = {};
		this.getAllNodes().forEach(function(node){
			if(node.id){
				var item = this.getItem(node.id);
				if(item){
					map[node.id] = item;
					return;
				}
			}else{
				node.id = dojo.dnd.getUniqueId();
			}
			var type = node.getAttribute("dndType"),
				data = node.getAttribute("dndData");
			map[node.id] = {
				data: data || node.innerHTML,
				type: type ? type.split(/\s*,\s*/) : ["text"]
			};
		}, this);
		this.map = map;
		return this;	// self
	},
	insertNodes: function(data, before, anchor){
		// summary:
		//		inserts an array of new nodes before/after an anchor node
		// data: Array
		//		a list of data items, which should be processed by the creator function
		// before: Boolean
		//		insert before the anchor, if true, and after the anchor otherwise
		// anchor: Node
		//		the anchor node to be used as a point of insertion
		if(!this.parent.firstChild){
			anchor = null;
		}else if(before){
			if(!anchor){
				anchor = this.parent.firstChild;
			}
		}else{
			if(anchor){
				anchor = anchor.nextSibling;
			}
		}
		if(anchor){
			for(var i = 0; i < data.length; ++i){
				var t = this._normalizedCreator(data[i]);
				this.setItem(t.node.id, {data: t.data, type: t.type});
				this.parent.insertBefore(t.node, anchor);
			}
		}else{
			for(var i = 0; i < data.length; ++i){
				var t = this._normalizedCreator(data[i]);
				this.setItem(t.node.id, {data: t.data, type: t.type});
				this.parent.appendChild(t.node);
			}
		}
		return this;	// self
	},
	destroy: function(){
		// summary:
		//		prepares this object to be garbage-collected
		dojo.forEach(this.events, dojo.disconnect);
		this.clearItems();
		this.node = this.parent = this.current = null;
	},

	// markup methods
	markupFactory: function(params, node, ctor){
		params._skipStartup = true;
		return new ctor(node, params);
	},
	startup: function(){
		// summary:
		//		collects valid child items and populate the map

		// set up the real parent node
		if(!this.parent){
			// use the standard algorithm, if not assigned
			this.parent = this.node;
			if(this.parent.tagName.toLowerCase() == "table"){
				var c = this.parent.getElementsByTagName("tbody");
				if(c && c.length){ this.parent = c[0]; }
			}
		}
		this.defaultCreator = dojo.dnd._defaultCreator(this.parent);

		// process specially marked children
		this.sync();
	},

	// mouse events
	onMouseOver: function(e){
		// summary:
		//		event processor for onmouseover
		// e: Event
		//		mouse event
		var n = e.relatedTarget;
		while(n){
			if(n == this.node){ break; }
			try{
				n = n.parentNode;
			}catch(x){
				n = null;
			}
		}
		if(!n){
			this._changeState("Container", "Over");
			this.onOverEvent();
		}
		n = this._getChildByEvent(e);
		if(this.current == n){ return; }
		if(this.current){ this._removeItemClass(this.current, "Over"); }
		if(n){ this._addItemClass(n, "Over"); }
		this.current = n;
	},
	onMouseOut: function(e){
		// summary:
		//		event processor for onmouseout
		// e: Event
		//		mouse event
		for(var n = e.relatedTarget; n;){
			if(n == this.node){ return; }
			try{
				n = n.parentNode;
			}catch(x){
				n = null;
			}
		}
		if(this.current){
			this._removeItemClass(this.current, "Over");
			this.current = null;
		}
		this._changeState("Container", "");
		this.onOutEvent();
	},
	onSelectStart: function(e){
		// summary:
		//		event processor for onselectevent and ondragevent
		// e: Event
		//		mouse event
		if(!this.skipForm || !dojo.dnd.isFormElement(e)){
			dojo.stopEvent(e);
		}
	},

	// utilities
	onOverEvent: function(){
		// summary:
		//		this function is called once, when mouse is over our container
	},
	onOutEvent: function(){
		// summary:
		//		this function is called once, when mouse is out of our container
	},
	_changeState: function(type, newState){
		// summary:
		//		changes a named state to new state value
		// type: String
		//		a name of the state to change
		// newState: String
		//		new state
		var prefix = "dojoDnd" + type;
		var state  = type.toLowerCase() + "State";
		//dojo.replaceClass(this.node, prefix + newState, prefix + this[state]);
		dojo.replaceClass(this.node, prefix + newState, prefix + this[state]);
		this[state] = newState;
	},
	_addItemClass: function(node, type){
		// summary:
		//		adds a class with prefix "dojoDndItem"
		// node: Node
		//		a node
		// type: String
		//		a variable suffix for a class name
		dojo.addClass(node, "dojoDndItem" + type);
	},
	_removeItemClass: function(node, type){
		// summary:
		//		removes a class with prefix "dojoDndItem"
		// node: Node
		//		a node
		// type: String
		//		a variable suffix for a class name
		dojo.removeClass(node, "dojoDndItem" + type);
	},
	_getChildByEvent: function(e){
		// summary:
		//		gets a child, which is under the mouse at the moment, or null
		// e: Event
		//		a mouse event
		var node = e.target;
		if(node){
			for(var parent = node.parentNode; parent; node = parent, parent = node.parentNode){
				if(parent == this.parent && dojo.hasClass(node, "dojoDndItem")){ return node; }
			}
		}
		return null;
	},
	_normalizedCreator: function(/*dojo.dnd.Item*/ item, /*String*/ hint){
		// summary:
		//		adds all necessary data to the output of the user-supplied creator function
		var t = (this.creator || this.defaultCreator).call(this, item, hint);
		if(!dojo.isArray(t.type)){ t.type = ["text"]; }
		if(!t.node.id){ t.node.id = dojo.dnd.getUniqueId(); }
		dojo.addClass(t.node, "dojoDndItem");
		return t;
	}
});

dojo.dnd._createNode = function(tag){
	// summary:
	//		returns a function, which creates an element of given tag
	//		(SPAN by default) and sets its innerHTML to given text
	// tag: String
	//		a tag name or empty for SPAN
	if(!tag){ return dojo.dnd._createSpan; }
	return function(text){	// Function
		return dojo.create(tag, {innerHTML: text});	// Node
	};
};

dojo.dnd._createTrTd = function(text){
	// summary:
	//		creates a TR/TD structure with given text as an innerHTML of TD
	// text: String
	//		a text for TD
	var tr = dojo.create("tr");
	dojo.create("td", {innerHTML: text}, tr);
	return tr;	// Node
};

dojo.dnd._createSpan = function(text){
	// summary:
	//		creates a SPAN element with given text as its innerHTML
	// text: String
	//		a text for SPAN
	return dojo.create("span", {innerHTML: text});	// Node
};

// dojo.dnd._defaultCreatorNodes: Object
//		a dictionary that maps container tag names to child tag names
dojo.dnd._defaultCreatorNodes = {ul: "li", ol: "li", div: "div", p: "div"};

dojo.dnd._defaultCreator = function(node){
	// summary:
	//		takes a parent node, and returns an appropriate creator function
	// node: Node
	//		a container node
	var tag = node.tagName.toLowerCase();
	var c = tag == "tbody" || tag == "thead" ? dojo.dnd._createTrTd :
			dojo.dnd._createNode(dojo.dnd._defaultCreatorNodes[tag]);
	return function(item, hint){	// Function
		var isObj = item && dojo.isObject(item), data, type, n;
		if(isObj && item.tagName && item.nodeType && item.getAttribute){
			// process a DOM node
			data = item.getAttribute("dndData") || item.innerHTML;
			type = item.getAttribute("dndType");
			type = type ? type.split(/\s*,\s*/) : ["text"];
			n = item;	// this node is going to be moved rather than copied
		}else{
			// process a DnD item object or a string
			data = (isObj && item.data) ? item.data : item;
			type = (isObj && item.type) ? item.type : ["text"];
			n = (hint == "avatar" ? dojo.dnd._createSpan : c)(String(data));
		}
		if(!n.id){
			n.id = dojo.dnd.getUniqueId();
		}
		return {node: n, data: data, type: type};
	};
};

return dojo.dnd.Container;
});

},
'gridx/modules/select/Column':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/query",
	"dojo/_base/array",
	"dojo/_base/sniff",
	"dojo/dom-class",
	"dojo/keys",
	"./_Base",
	"../../core/_Module"
], function(declare, query, array, sniff, domClass, keys, _Base, _Module){

	return declare(/*===== "gridx.modules.select.Column", =====*/_Base, {
		// summary:
		//		Provides simple column selection.
		// description:
		//		This module provides a simple way for selecting columns by clicking or SPACE key, 
		//		or CTRL + Click to select multiple columns.
		//
		// example:
		//		1. Use select api on grid column object obtained from grid.column(i)
		//		|	grid.column(1).select();
		//		|	grid.column(1).deselect();
		//		|	grid.column(1).isSelected();
		//
		//		2. Use select api on select.column module
		//		|	grid.select.column.selectById(columnId);
		//		|	grid.select.column.deSelectById(columnId);
		//		|	grid.select.column.isSelected(columnId);
		//		|	grid.select.column.getSelected();//[]
		//		|	grid.select.column.clear();

		// name: [readonly] String
		//		module name
		name: "selectColumn",

		// columnMixin: Object
		//		A map of functions to be mixed into grid column object, so that we can use select api on column object directly
		//		- grid.column(1).select() | deselect() | isSelected();
		columnMixin: {
			select: function(){
				this.grid.select.column._markById(this.id, 1);
				return this;
			},
			deselect: function(){
				this.grid.select.column._markById(this.id, 0);
				return this;
			},
			isSelected: function(){
				return this.grid.select.column.isSelected(this.id);
			}
		},
		
		//Public API----------------------------------------------------------------------
		selectById: function(/*String*/id){
			// summary:
			//		Select target column by id
			this._markById(id, 1);
		},
		
		deselectById: function(/*String*/id){
			// summary:
			//		Deselect target column by id
			this._markById(id, 0);
		},
		
		isSelected: function(/*String*/id){
			// summary:
			//		Check if a column is selected 
			var c = this.grid._columnsById[id];
			return !!(c && c._selected);
		},
		
		getSelected: function(){
			// summary:
			//		Get array of column id of all selected columns
			var ids = [], i, c,
				g = this.grid,
				cols = g._columns,
				count = cols.length;
			for(i = 0; i < count; ++i){
				c = cols[i];
				if(c._selected){
					ids.push(c.id);
				}
			}
			return ids;
		},
		
		clear: function(){
			// summary:
			//		Clear all column selections
			var columns = this.grid._columns, i, count = columns.length;
			for(i = 0; i < count; ++i){
				this._markById(columns[i].id, 0);
			}
		},

	/*=====
		onSelected: function(col){
			// summary:
			//		Fired when a column is selected.
			// col: gridx.core.Column
			//		The column object
		},

		onDeselected: function(col){
			// summary:
			//		Fired when a column is deselected.
			// col: gridx.core.Column
			//		The column object
		},

		onHighlightChange: function(){
			// summary:
			//		Fired when a column's highlight is changed.
			// tags:
			//		private package
		},
	=====*/

		//Private-------------------------------------------------------------------------------
		_type: 'column',

		_init: function(){
			var t = this,
				g = t.grid;
			t.batchConnect(
				[g, 'onHeaderCellClick', function(e){
					if(!domClass.contains(e.target, 'gridxArrowButtonNode')){
						t._select(e.columnId, g._isCopyEvent(e));
					}
				}],
				[g, sniff('ff') < 4 ? 'onHeaderCellKeyUp' : 'onHeaderCellKeyDown', function(e){
					if(e.keyCode == keys.SPACE || e.keyCode == keys.ENTER){
						t._select(e.columnId, g._isCopyEvent(e));
					}
				}]
			);
		},

		_markById: function(id, toSelect){
			var t = this, c = t.grid._columnsById[id];
			if(t.arg('enabled')){
				toSelect = !!toSelect;
				if(c && !c._selected == toSelect){
					c._selected = toSelect;
					t._highlight(id, toSelect);
					t[toSelect ? "onSelected" : "onDeselected"](t.grid.column(id, 1));
				}
			}
		},
		
		_highlight: function(id, toHighlight){
			var t = this, g = t.grid;
			query("[colid='" + id + "']", g.bodyNode).forEach(function(node){
				domClass.toggle(node, 'gridxColumnSelected', toHighlight);
				t.onHighlightChange({column: g._columnsById[id].index}, toHighlight);
			});
		},

		_onRender: function(start, count){
			var i, j, node,
				end = start + count,
				g = this.grid,
				bn = g.bodyNode,
				cols = array.filter(g._columns, function(col){
					return col._selected;
				});
			for(i = cols.length - 1; i >= 0; --i){
				for(j = start; j < end; ++j){
					node = query(['[visualindex="', j, '"] [colid="', cols[i].id, '"]'].join(''), bn)[0];
					domClass.add(node, 'gridxColumnSelected');
				}
			}
		}
	});
});

},
'url:idx/oneui/templates/MenuDialog.html':"<div role=\"presentation\">\r\n\t<div class=\"dijitTooltipContainer\" role=\"presentation\">\r\n\t\t<div class =\"dijitTooltipContents dijitTooltipFocusNode\" data-dojo-attach-point=\"containerNode\" role=\"dialog\" tabIndex=\"-1\"></div>\r\n\t</div>\r\n\t<div class=\"dijitTooltipConnector\" role=\"presentation\" data-dojo-attach-point=\"connectorNode\"></div>\r\n</div>\r\n",
'dojo/dnd/common':function(){
define(["../main"], function(dojo) {
	// module:
	//		dojo/dnd/common
	// summary:
	//		TODOC

dojo.getObject("dnd", true, dojo);

dojo.dnd.getCopyKeyState = dojo.isCopyKey;

dojo.dnd._uniqueId = 0;
dojo.dnd.getUniqueId = function(){
	// summary:
	//		returns a unique string for use with any DOM element
	var id;
	do{
		id = dojo._scopeName + "Unique" + (++dojo.dnd._uniqueId);
	}while(dojo.byId(id));
	return id;
};

dojo.dnd._empty = {};

dojo.dnd.isFormElement = function(/*Event*/ e){
	// summary:
	//		returns true if user clicked on a form element
	var t = e.target;
	if(t.nodeType == 3 /*TEXT_NODE*/){
		t = t.parentNode;
	}
	return " button textarea input select option ".indexOf(" " + t.tagName.toLowerCase() + " ") >= 0;	// Boolean
};

return dojo.dnd;
});

},
'dojoe/tablex/_Toolbar':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/i18n!dojoe/tablex/nls/Table","dojo/require!dojoe/common/toolbar/_Toolbar,dijit/form/Button,dijit/ToolbarSeparator,dijit/form/DropDownButton,dijit/form/ComboButton,dijit/Menu,dijit/Dialog,dijit/form/SimpleTextarea,dojo/i18n,dojoe/tablex/_ConfigureColumnsDialog,dojoe/tablex/_QuickFilter,dijit/_TemplatedMixin"], function(dijit,dojo,dojox){
// dojo.provide allows pages to use all of the types declared in this resource.
dojo.provide("dojoe.tablex._Toolbar");

dojo.require("dojoe.common.toolbar._Toolbar");
dojo.require("dijit.form.Button");
dojo.require("dijit.ToolbarSeparator");
dojo.require("dijit.form.DropDownButton");
dojo.require("dijit.form.ComboButton");
dojo.require("dijit.Menu");
dojo.require("dijit.Dialog");
dojo.require("dijit.form.SimpleTextarea");
dojo.require("dojo.i18n");
dojo.require("dojoe.tablex._ConfigureColumnsDialog");
dojo.require("dojoe.tablex._QuickFilter");
dojo.require("dijit._TemplatedMixin");

dojo.requireLocalization("dojoe.tablex", "Table");

dojo.declare("dojoe.tablex._Toolbar", [ dojoe.common.toolbar._Toolbar], {
		// summary:
		//		Table toolbar class which is based on common toolbar widget and is a private class
		// description:
		//		This class provides ability to enable/disable Table specific toolbar controls
		//		like refresh icon, action menu, print, export, configure columns and filter/search options
		//		this is internally used by Table class and can be configured on Table widget class
		
		// widgetsInTemplate: boolean
		//		this is used to indicate there are widgets in template
		widgetsInTemplate:true,
		
		// plugins: object
		//		array of plugins associated with the grid instance
		plugins:null,
		
		// inGrid: object
		//		Grid instance
		inGrid:null,
		
		// actionListWidget: object
		//		Action list widget instance
		actionListWidget:null,
		
		// resources reference
		resources_:null,
		
		// toolbarOptions from table
		toolbarOptions:null,
		
		// filterOptions: object
		//		To hide/show filter bar, to hide/show Quick Filter, hide/show Advnaced filter
		filterOptions:null,
		
		// refreshIcon: boolean
		//		To enable/disable refresh icon on toolbar, defaults to true 
		refreshIcon:true,
		
		// refreshMenu: boolean
		//		To enable/disable refresh item in action list, defaults to true 
		refreshMenu:true,
		
		// actionMenu: boolean
		//		To enable/disable action list in the toolbar, defaults to true 
		actionMenu:true,
		
		// configTableIcon: boolean
		//		To enable/disable configure options Icon in the toolbar, defaults to true
		configTableIcon:true,
		
		// configTableMenu: boolean
		//		To indicate whether configure options should appear in action list, defautls to true
		configTableMenu:true,
		
		//table instance
		tivTable:null,
		
		// configure dialog isntance
		configureTableDialog:null,
		
		// just the quickFilter instance
		_quickFilter:null,

		_origStructure: null,
		
		// Constructor function. Called when instance of this class is created
		constructor : function() {
			this.resources_ = dojo.i18n.getLocalization("dojoe.tablex", "Table", this.lang);			
		},
 		
		postCreate:function(){		
			this.inGrid = this.tivTable._grid;
			this._origStructure = this.tivTable._origLayout;
			this.plugins = this.tivTable.gridParams_.plugins;
			this.toolbarOptions = this.tivTable.toolbarOptions;
			this.filterOptions = this.tivTable.filterOptions;
			//commenting as this not required
			//this.toolbar = new dijit.Toolbar({id: "toolbar"+this.id});			
			
			//setting the toolbar options passed from UI
			this._mapToolbarOptions();
			
			//adding the refresh icon
			this._addRefreshIcon();			
			
			//adding the export icon
			this._addExporterIcon();
						
			//adding the printer icon
			this._addPrintIcon();				
			
			//adding the configure settings icon
			this._addConfigTableIcon();				
			
			//Action List Menu
			this._addActionMenu();			

			//adding filters
			this._addFilters();
		},
		
		//16716: table toolbar has browser popup menu on right click
		_stopEvent: function(e){
			// summary:
			//		dojo stop event
			//Since event is oncontextmenu, just stop it!
			dojo.stopEvent(e);
		},
		
		
		_addRefreshIcon: function() {
			// summary:
			//		internal function to add refresh icon to the table toolbar
			
			if(this.refreshIcon) {
				var refreshTableIcon = new dijit.form.Button({
						showLabel: false,
						id: "refreshIcon"+this.id,
						iconClass: "refreshIcon",
						label: this.resources_.REFRESH,
						title: this.resources_.REFRESH
					});
					
				refreshTableIcon.onClick = dojo.hitch (this, "_refreshTable");
				refreshTableIcon.onKeypress = dojo.hitch (this, "_refreshTable");
				var seperator = new dijit.ToolbarSeparator({id: "refSeperator"+this.id});
				
				this._addStateControl(refreshTableIcon,0);
				this._addStateControl(seperator);
			}
		},
		_addExporterIcon: function() {
			// summary:
			//		internal function to add Export icon to the table toolbar
			
			if(this.toolbarOptions.exporterIcon) {
				var exportDataMenu = new dijit.Menu({ style: "display: none;"});
				var exportHTMLMenuItem = new dijit.MenuItem({
					title: this.resources_.EXPORT_HTML,
					label: this.resources_.EXPORT_HTML
				});
				exportHTMLMenuItem.onClick = dojo.hitch (this, "_exportHTML");
				exportHTMLMenuItem.onKeypress = dojo.hitch (this, "_exportHTML");
				exportDataMenu.addChild(exportHTMLMenuItem);	
				
				//Bobby: 07Jan2011, adding CSV Export 
				var exportCSVMenuItem = new dijit.MenuItem({
					title: this.resources_.EXPORT_ALL_CSV,
					label: this.resources_.EXPORT_ALL_CSV
				});
				exportCSVMenuItem.onClick = dojo.hitch (this, "_exportCSV");
				exportCSVMenuItem.onKeypress = dojo.hitch (this, "_exportCSV");
				exportDataMenu.addChild(exportCSVMenuItem);				
			
				//Combo Button for All Export Related Actions
				var exportDataCombo = new dijit.form.DropDownButton({
					showLabel: false,
					iconClass: "exportDataIcon",
					id: "exportDataIcon"+this.id,
					label: this.resources_.EXPORT,
					title: this.resources_.EXPORT,
					dropDown: exportDataMenu
				});
				this._addFastPathControl(exportDataCombo);
			}

		},
		
		_addPrintIcon: function() {
			// summary:
			//		internal function to add print icon to the table toolbar
			
			//Check whether we need to show the print icon or not
			if( this.toolbarOptions.printerIcon) {
				var printDataMenu = new dijit.Menu({ style: "display: none;"});		    
				var printAllItem = new dijit.MenuItem({
					title: this.resources_.PRINT_ALL,
					label: this.resources_.PRINT_ALL
				});
				printAllItem.onClick = dojo.hitch (this, "_printAll");
				printAllItem.onKeypress = dojo.hitch (this, "_printAll");			     
				printDataMenu.addChild(printAllItem);
				var printSelectedMenuItem = new dijit.MenuItem({
					title: this.resources_.PRINT_SELECTED,
					label: this.resources_.PRINT_SELECTED
				});
				printSelectedMenuItem.onClick = dojo.hitch (this, "_printSelected");
				printSelectedMenuItem.onKeypress = dojo.hitch (this, "_printSelected");
				printDataMenu.addChild(printSelectedMenuItem);		     
				var printPreviewMenuItem = new dijit.MenuItem({
					title: this.resources_.PRINT_PREVIEW,
					label: this.resources_.PRINT_PREVIEW
				});
				printPreviewMenuItem.onClick = dojo.hitch (this, "_printPreview");
				printPreviewMenuItem.onKeypress = dojo.hitch (this, "_printPreview");
				
				printDataMenu.addChild(printPreviewMenuItem);		     
				//Combo Button for All Print Related Actions
				var printDataCombo = new dijit.form.DropDownButton({
					showLabel: false,
					iconClass: "printIcon",
					id: "printIcon"+this.id,
					label: this.resources_.PRINT,
					title: this.resources_.PRINT,
					dropDown: printDataMenu
				});
				this._addFastPathControl(printDataCombo);
			}
		},
		_addConfigTableIcon: function() {
			// summary:
			//		internal function to add config icon to the table toolbar
			
			if(this.configTableIcon) {
				var configTableIcon = new dijit.form.Button({
						showLabel: false,
						iconClass: "configureColIcon",
						id: "configureColIcon"+this.id,
						label: this.resources_.CONFIGURE_SETTINGS,
						title: this.resources_.CONFIGURE_SETTINGS
					});
					
				configTableIcon.onClick = dojo.hitch (this, "_configureTable");
				configTableIcon.onKeypress = dojo.hitch (this, "_configureTable");
				this._addFastPathControl(configTableIcon);
			}
		},
		
		_addActionMenu: function() {
			// summary:
			//		internal function to add actionlist icon to the table toolbar
			
			//Action List Menu
			if(this.actionMenu) {			
				//var seperator = new dijit.ToolbarSeparator({id: "actionMenuSeperator"+this.id},this.actionSeperator);
				var menu = new dijit.Menu({ style: "display: none;"});
				//menu.addChild(new dijit.MenuSeparator());
				
				//23789: Refresh should be part of Actions Menu also - OID
				//Bobby:29Jun11				
				if(this.refreshMenu) {
					//Menu Item for Configure Columns action
					var refreshMenuItem = new dijit.MenuItem({
						//Bobby:19Jul11, title is reqd by JAWS to anounce the menu item.					
						//24709: JAWS does not read the items in the Actions, Export or Print menus.
						title: this.resources_.REFRESH,
						label: this.resources_.REFRESH,
						iconClass: "refreshIcon"
					});
					refreshMenuItem.onClick = dojo.hitch (this, "_refreshTable");
					refreshMenuItem.onKeypress = dojo.hitch (this, "_refreshTable");					 
					menu.addChild(refreshMenuItem);
				}

				if(this.filterOptions.showAdvancedFilter){
					//Menu Item for Advanced Filter action
					var filterMenuItem = new dijit.MenuItem({
						title: this.resources_.ADVANCED_FILTER,
						label: this.resources_.ADVANCED_FILTER,
						//20508: Advanced Filter menu item on Actions menu has tooltip while other menu items do not
						//Bobby:25May11, removing unnecessary title, not reqd for Menu
						//title: this.resources_.ADVANCED_FILTER,
						iconClass: "filterIcon"
					});
					filterMenuItem.onClick = dojo.hitch (this, "_advancedFilter");
					filterMenuItem.onKeypress = dojo.hitch (this, "_advancedFilter");					 
					menu.addChild(filterMenuItem);
				}				
				
				
				if(this.toolbarOptions.exporterMenu) {
					//Sub-menu item for All Export Related Actions
					var exportDataSubMenu = new dijit.Menu();
					exportDataSubMenu.addChild(new dijit.MenuItem({
						title: this.resources_.EXPORT_HTML,
						label: this.resources_.EXPORT_HTML,
						onClick: dojo.hitch (this, this._exportHTML),
						onKeypress: dojo.hitch (this, this._exportHTML)
					}));	
					
					//Bobby: 07Jan2011, Adding export as CSV
					exportDataSubMenu.addChild(new dijit.MenuItem({
						title: this.resources_.EXPORT_ALL_CSV,
						label: this.resources_.EXPORT_ALL_CSV,
						onClick: dojo.hitch (this, this._exportCSV),
						onKeypress: dojo.hitch (this, this._exportCSV)
					}));					
						
					var exportDataItem = new dijit.PopupMenuItem({
						title: this.resources_.EXPORT,
						label: this.resources_.EXPORT,
						iconClass: "exportDataIcon",
						popup: exportDataSubMenu
					});
				    menu.addChild(exportDataItem);
				}
				
				if( this.toolbarOptions.printerMenu) {
					//Sub-menu items for All Print Related Actions
					var printDataSubMenu = new dijit.Menu();
					printDataSubMenu.addChild(new dijit.MenuItem({
						title: this.resources_.PRINT_ALL,
						label: this.resources_.PRINT_ALL,
						onClick: dojo.hitch (this, this._printAll),
						onKeypress: dojo.hitch (this, this._printAll)
					}));
					printDataSubMenu.addChild(new dijit.MenuItem({
						title: this.resources_.PRINT_SELECTED,
						label: this.resources_.PRINT_SELECTED,
						onClick: dojo.hitch (this, this._printSelected),
						onKeypress: dojo.hitch (this, this._printSelected)
					}));
					printDataSubMenu.addChild(new dijit.MenuItem({
						title: this.resources_.PRINT_PREVIEW,
						label: this.resources_.PRINT_PREVIEW,
						onClick: dojo.hitch (this, this._printPreview),
						onKeypress: dojo.hitch (this, this._printPreview)
					}));
					var printMenuItem = new dijit.PopupMenuItem({
						title: this.resources_.PRINT,
						label: this.resources_.PRINT,
						iconClass: "printIcon",
						popup: printDataSubMenu
					});
					menu.addChild(printMenuItem);
				}
				
				if(this.configTableMenu) {
					//Menu Item for Configure Columns action
					var configureColMenuItem = new dijit.MenuItem({
						title: this.resources_.CONFIGURE_SETTINGS,
						label: this.resources_.CONFIGURE_SETTINGS,
						iconClass: "configureColIcon"
					});
					configureColMenuItem.onClick = dojo.hitch (this, "_configureTable");
					configureColMenuItem.onKeypress = dojo.hitch (this, "_configureTable");					 
					menu.addChild(configureColMenuItem);
				}				
				
			    menu.startup();
				//Initializing the Action List
			    var actionListMenu = new dijit.form.DropDownButton({
					optionsTitle : this.resources_.ACTIONS,
			        label: this.resources_.ACTIONS,
					title: this.resources_.ACTIONS,
					id: "actionListMenu"+this.id,
			        dropDown: menu
			    });				
				this.actionListWidget = actionListMenu;
				this._addFastPathControl(actionListMenu);
			}
			
		},
		
		// To create quick filter
		_createQuickFilter: function() {
			// summary:
			//		internal function to create quick filter
			this._quickFilter = new dojoe.tablex._QuickFilter({
						grid:this.inGrid,
						id: "quickFilter"+this.id,
						filterOptions: this.filterOptions,
						placeHolder: this.resources_.QF_DEFAULT_TEXT,
						clearIconText:this.resources_.CLEAR_FILTER
			}, this.quickFilterNode);
		},
		
		// To create advanced filter
		_createAdvanceFilter: function() {
			// summary:
			//		internal function to create advance filter 
			
			var advancedFilterIcon = new dijit.form.Button({
				showLabel: false,
				iconClass: "filterIcon",
				id: "advancedFilterIcon"+this.id,
				//label: this.resources_.ADVANCED_FILTER,
				title: this.resources_.ADVANCED_FILTER
			},this.filterButton);				
			advancedFilterIcon.onClick = dojo.hitch (this, "_advancedFilter");
			advancedFilterIcon.onKeypress = dojo.hitch (this, "_advancedFilter");
			//advancedFilterIcon.domNode.title = this.resources_.ADVANCED_FILTER;
		},
		
		_addFilters: function() {
			// summary:
			//		internal function to add filter options to the toolbar
			
			//adding filters
			if(this.filterOptions.hideFilterBar == true && this.inGrid.filterBar !== undefined){
				//hide Advanced Filter bar
				this.inGrid.filterBar.hide();
			} else if(this.filterOptions.hideFilterBar == false){
				//show Advanced Filter bar
				this.inGrid.filterBar.show();
			}			
			
			/*If 
				quickFilter 		- YES
				quickFilterButton 	- YES
				advancedFilter		- NO
			  then show only quickFilterButton
			*/
			if(this.filterOptions.quickFilter && this.filterOptions.showQuickFilterButton && !this.filterOptions.advancedFilter){
				this._createQuickFilter();
				var quickFilterIcon = new dijit.form.Button({
						showLabel: false,
						iconClass: "filterIcon",
						id: "filterIcon"+this.id,
						//label: this.resources_.QUICK_FILTER,
						title: this.resources_.QUICK_FILTER
					},this.filterButton);				
				quickFilterIcon.onClick = dojo.hitch (this, "_setQuickFilter");
				quickFilterIcon.onKeypress = dojo.hitch (this, "_setQuickFilter");
				quickFilterIcon.domNode.title = this.resources_.QUICK_FILTER;				
			}			
			/*If 
				quickFilter 		- YES
				quickFilterButton 	- YES
				advancedFilter		- YES
			  then show filter combobutton
			*/			
			else if(this.filterOptions.quickFilter && this.filterOptions.showQuickFilterButton && this.filterOptions.advancedFilter){
				this._createQuickFilter();
				var menu = new dijit.Menu({ });
				menu.domNode.style.display="none";
				var qfMenuItem = new dijit.MenuItem({
					label: this.resources_.QF_DEFAULT_TEXT,
					//iconClass: "filterIcon",
					onClick: dojo.hitch (this, "_setQuickFilter"),
					onKeypress: dojo.hitch (this, "_setQuickFilter")
				});
				var afMenuItem = new dijit.MenuItem({
					label: this.resources_.ADVANCED_FILTER,
					//iconClass: "filterIcon",
					onClick: dojo.hitch (this, "_advancedFilter"),
					onKeypress: dojo.hitch (this, "_advancedFilter")
				});				
				menu.addChild(qfMenuItem);
				menu.addChild(afMenuItem);
				
				var comboFilterIcon = new dijit.form.ComboButton({
						showLabel: false,
						iconClass: "filterIcon",
						id: "comboFilterIcon"+this.id,
						label: this.resources_.QUICK_FILTER,
						title: this.resources_.QUICK_FILTER,
						dropDown: menu,
						onClick: dojo.hitch (this, "_setQuickFilter"),
						onKeypress: dojo.hitch (this, "_setQuickFilter")
					},this.filterButton);
			}
			/*If 
				quickFilter 		- YES
				quickFilterButton 	- NO
				advancedFilter		- NO
			  then show filter textbox only
			*/				
			else if(this.filterOptions.quickFilter && !this.filterOptions.showQuickFilterButton && !this.filterOptions.advancedFilter){
				this._createQuickFilter();
			}
			/*If 
				quickFilter 		- YES
				quickFilterButton 	- NO
				advancedFilter		- YES
			  then show filter textbox and AdvancedFilter button
			*/			
			
			else if(this.filterOptions.advancedFilter && !this.filterOptions.showQuickFilterButton && this.filterOptions.advancedFilter){
				this._createQuickFilter();
				this._createAdvanceFilter();
			}
			/*If 
				quickFilter 		- NO
				quickFilterButton 	- NO
				advancedFilter		- YES
			  then show AdvancedFilter button only
			*/				
			else if(this.filterOptions.advancedFilter) {
				this._createAdvanceFilter();					
			}//else don't show qf or af.
			
		},
		
		_refreshTable: function(){
			// summary:
			//		internal function to refresh table
			this.tivTable.refresh();
		},
		
			
		_exportHTML: function(){
			var url = dojo.moduleUrl("dojoe", "table/themes/print_style.css");
			var urlStr = url.toString();
			
			var printArgs = {title: this.resources_.EXPORT_HTML, styleSrc:urlStr};
			if(this.inGrid.select.row.getSelected() > 0) {
				printArgs.selectedOnly = true;
				var deferred = this.tivTable._printPreviewGrid( printArgs);
				deferred.addCallback(function(exportedString) {
					var win = window.open();
					win.document.open();
					win.document.write(exportedString);
					//Do adjustments to fix the problems of preview here.					
					//grid.normalizePrintedGrid(win.document);
					win.document.close();
				});
			}
			else {			
				var deferred = this.tivTable._printPreviewGrid(printArgs);
				deferred.addCallback(
					function(str){
						var win = window.open();
						win.document.open();
						win.document.write(str);
						//Do adjustments to fix the problems of preview here.
						//grid.normalizePrintedGrid(win.document);
						win.document.close();
					}
				);
			}
		},		
		
	   //Bobby: 07Jan2011, adding CSV Export 
		_exportCSV: function(){
		 // this._customizeData();	
		  var deferred = this.tivTable._exportGrid("csv");
		  deferred.addCallback(
			function(str){
				//Bobby:16Apr2011. Browser messes up the format of CSV
			    //16850: TBSM: Table CSV Export not usable by Excel
				//var win = window.open();
				//win.document.open();
				//win.document.write(str);
				//win.document.close();
				resources_ = dojo.i18n.getLocalization("dojoe.tablex", "Table", this.lang);		
				var exportTextArea = new dijit.form.SimpleTextarea({
					value: str,
					rows: 20,
					cols: 100,
					//style: "width:80%; height:80%",
					selectOnClick: true
				});
				exportTextArea.selected = true;
				var exportDialog = new dijit.Dialog({
					title: resources_.EXPORT_ALL_CSV+ " - " + resources_.EXPORT_ALL_CSV_SAVE
				});	
				exportDialog.attr("content",exportTextArea);
				exportDialog.show();				
			}
		  );
		},			
		
		_printAll: function(){
			//this._customizeData();
			/*#24000 - 11 Jul 2011 -dojo grid is now accepting only string uri for loading .css, now sure why */
			var url = dojo.moduleUrl("dojoe", "table/themes/print_style.css");
			var urlStr = url.toString();
			var printArgs = {title: this.resources_.PRINT_ALL, styleSrc: urlStr};
			//this.tivTable.printGrid(printArgs);
			var deferred = this.tivTable._printPreviewGrid(printArgs);
			deferred.addCallback(function(exportedString) {
				var win = window.open();
				win.document.open();
				win.document.write(exportedString);
				//Do adjustments to fix the problems of preview here.					
				//grid.normalizePrintedGrid(win.document);
				win.print();
				win.document.close();
			});
			
		},		
		_printSelected: function(){
			//this._customizeData();
			/*#24000 - 11 Jul 2011 -dojo grid is now accepting only string uri for loading .css, now sure why */
			var url = dojo.moduleUrl("dojoe", "table/themes/print_style.css");
			var urlStr = url.toString();
			var printArgs = {selectedOnly: true, title: this.resources_.PRINT_SELECTED, styleSrc: urlStr};
			//this.tivTable.printGrid(printArgs);
			var deferred = this.tivTable._printPreviewGrid(printArgs);
			deferred.addCallback(function(exportedString) {
				var win = window.open();
				win.document.open();
				win.document.write(exportedString);
				//Do adjustments to fix the problems of preview here.					
				//grid.normalizePrintedGrid(win.document);
				win.print();
				win.document.close();
			});
			
		},		
		_printPreview: function(){
			//this._customizeData();
			var url = dojo.moduleUrl("dojoe", "table/themes/print_style.css");
			var urlStr = url.toString();
			var printArgs = {title: this.resources_.PRINT_PREVIEW, styleSrc: urlStr};
			if(this.inGrid.select.row.getSelected() > 0) {
				printArgs.selectedOnly = true;
				var deferred = this.tivTable._printPreviewGrid(printArgs);
				deferred.addCallback(function(exportedString) {
					var win = window.open();
					win.document.open();
					win.document.write(exportedString);
					//Do adjustments to fix the problems of preview here.					
					//grid.normalizePrintedGrid(win.document);
					win.document.close();
				});
			}
			else {			
				var deferred = this.tivTable._printPreviewGrid(printArgs);
				deferred.then(function(exportedString) {
					var win = window.open();
					win.document.open();
					win.document.write(exportedString);
					//Do adjustments to fix the problems of preview here.					
					//grid.normalizePrintedGrid(win.document);
					win.document.close();
				});
			}
		},
		_setQuickFilter: function(){
			//console.info("In _setQuickFilter.....");
			this._quickFilter._setFilter();
		},
		_advancedFilter: function(){
		    this.inGrid.filterBar.show();
		    this.inGrid.filterBar.showFilterDialog(); 
			
		},
		_configureTable: function(){
			this.configureTableDialog = new dojoe.tablex._ConfigureColumnsDialog(
				{ grid: this.inGrid , _structure: this._origStructure}
			);	
			this.configureTableDialog.show();
		},
		
		//do not call this method directly, use table.addToToolbar(item,position,row) method to add a custom item to the toolbar before default buttons
		_addCustomControl: function(item, position,row){
			this.inherited(arguments);
		},		
		
		//do not call this method directly, use table.removeItemFromToolbar(itemId) method to remove a custom item from the toolbar
		_removeItem: function(itemId){
			var button= dijit.byId(itemId);
			if(button !== null && button !== undefined) {		
				if (button.domNode.parentNode) {
					//remove td which is the parent of the button
					buttonParent = button.domNode.parentNode;
					buttonParent.parentNode.removeChild(buttonParent);
				}
				button.destroyRecursive();
			}
		},
			
		_mapToolbarOptions: function(){
			if(this.toolbarOptions) {
				if(this.toolbarOptions.refreshIcon == false){
					this.refreshIcon = this.toolbarOptions.refreshIcon;
				}
				//23789: Refresh should be part of Actions Menu also - OID
				//Bobby:29Jun11
				if(this.toolbarOptions.refreshMenu == false){
					this.refreshMenu = this.toolbarOptions.refreshMenu;
				}				
				if(this.toolbarOptions.configTableIcon == false){
					this.configTableIcon = this.toolbarOptions.configTableIcon;
				}
				if(this.toolbarOptions.configTableMenu == false){
					this.configTableMenu = this.toolbarOptions.configTableMenu;
				}
				if(this.toolbarOptions.actionMenu == false){
					this.actionMenu = this.toolbarOptions.actionMenu;
				}
			}
		},
		
		_addToActionList: function(item) {
			if(this.actionMenu) {
				//always add new menu items to the top of the menu
				this.actionListWidget.dropDown.addChild(item, 0);
			}
		},
		
		_removeFromActionList: function (menuItemId){		
			var menuItem = dijit.byId(menuItemId);
			if(menuItem !== null && menuItem !== undefined) {
				menuItem.destroyRecursive();
			}
		},
		
		_enableMenuItem : function(menuItemId){
			var menuItem= dijit.byId(menuItemId);
			if(menuItem !== null && menuItem !== undefined) {
				menuItem.set('disabled', false);
			}
		},
		
		_disableMenuItem : function(menuItemId){
			var menuItem= dijit.byId(menuItemId);
			if(menuItem !== null && menuItem !== undefined) {
				menuItem.set('disabled', true);
			}
		},
		
		destroy: function(){
			// summary:  Destroys the toolbar.
			// tags: public	
			//memory leak issue: custom destroy method to take care of removing all widgets					
			try{
				delete this.resources_;
				//this is for destroying child templated widgets 
				//this should happen by itself but need to figure out why dijit._Templated is not taking care of it...
				var widgets = dijit.findWidgets(this.domNode);
				dojo.forEach(widgets, function(w) {
					if(w){
						if(w.destroyRecursive){
							w.destroyRecursive();
						} else if(w.destroy){
							w.destroy();
						}
					}
				});
				// this is for widgets added as child to this widget and their children..
				//not present in template
				var widgetArr = this.getChildren();
				dojo.forEach(widgetArr, function (child) {
					if(child){
						if(child.destroyRecursive){
							child.destroyRecursive();
						} else if(child.destroy){
							child.destroy();
						}
					}
				});
				this.inherited(arguments);
			}catch(ex){
				console.warn("Widget destroy failed: "+this.id+", ex="+ex.name+":"+ex.message);
			}
		}
		
	});
});

},
'gridx/modules/filter/FilterDialog':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/array",
	"dojo/dom-class",
	"dojo/string",
	"dojo/query",
	"dojo/keys",
	"dijit/registry",
	"dijit/Dialog",
	"dojox/html/metrics",
	"./FilterPane",
	"dojo/text!../../templates/FilterDialog.html",
	"dojo/i18n!../../nls/FilterBar",
	"dijit/form/Select",
	"dijit/form/Button",
	"dijit/layout/AccordionContainer"
], function(declare, lang, array, css, string, query, keys, registry, Dialog, metrics, FilterPane, template, i18n){
	return declare(Dialog, {
		title: i18n.filterDefDialogTitle,
		cssClass: 'gridxFilterDialog',
		grid: null,
		autofocus: false,
		//TWL #55300: this is a temporary fix until dojo team provides a flag
		autoComplete: true,
		postCreate: function(){
			this.inherited(arguments);
			this.i18n = i18n;
			this.set('content', string.substitute(template, this));
			this._initWidgets();
			css.add(this.domNode, 'gridxFilterDialog');
		},
		
		done: function(){
			// summary:
			//	Apply the filter.
			this.hide();
			this.grid.filterBar.applyFilter(this.getData());
		},
		
		getData: function(){
			// summary:
			//	Get filter data.
			return {
				type: this._sltMatch.get('value'),
				conditions: array.map(this._accordionContainer.getChildren(), function(p){
					return p.getData();
				})
			};
		},
		
		setData: function(data){
			// summary:
			//	Set filter data.
			this.removeChildren();
			if(!data || !data.conditions.length){
				return;
			}
			this._sltMatch.set('value', 'all' && data && data.type);
			array.forEach(data.conditions, function(d){
				this.addRule().setData(d);
			}, this);
		},
		
		removeChildren: function(){
			// summary:
			//	Remove all child of the accodion container.
			array.forEach(this._accordionContainer.getChildren(), function(child){
				this._accordionContainer.removeChild(child);
				child.destroy();
			}, this);
		
		},
		
		clear: function(){
			this.grid.filterBar.confirmToExecute(function(){
				this.grid.filterBar.clearFilter(true);
				this.hide();
			}, this);
		},
		
		cancel: function(){
			this.hide();
		},
		
		show: function(){
			this.inherited(arguments);
			if(!this._accordionContainer.hasChildren()){
				this.addRule();
			}
		},
		
		addRule: function(){
			var ac = this._accordionContainer;
			if(ac.getChildren().length === 3){
				ac._contentBox.w -= metrics.getScrollbar().w;
			}
			var nextRuleNumber = ac.getChildren().length + 1;
			var ruleTitle = string.substitute(this.i18n.ruleTitleTemplate, {ruleNumber: nextRuleNumber});
			//TWL #55300: this is a temporary fix until dojo team provides a flag
			var fp = new FilterPane({grid: this.grid, title: ruleTitle, autoComplete: this.autoComplete});
			ac.addChild(fp);
			ac.selectChild(fp);
			
			if(!this._titlePaneHeight){
				this._titlePaneHeight = fp._buttonWidget.domNode.offsetHeight + 3;
			}
			fp._initCloseButton();
			fp._onColumnChange();
			try{
				fp.tbSingle.focus();//TODO: this doesn't work now.
			}catch(e){}
			css.toggle(ac.domNode, 'gridxFilterSingleRule', ac.getChildren().length === 1);
			
			this.connect(fp, 'onChange', '_updateButtons');
			this._updateButtons();
			this._updateAccordionContainerHeight();
			//scroll to bottom when add a rule
			ac.domNode.parentNode.scrollTop = 100000;
			return fp;
		},
		
		_initWidgets: function(){
			var form = dojo.query('form', this.domNode)[0], _this = this;
			form.onsubmit = function(){
				_this.done();
				return false;
			}
			this._accordionContainer = registry.byNode(query('.dijitAccordionContainer', this.domNode)[0]);
			this._sltMatch = registry.byNode(query('.dijitSelect', this.domNode)[0]);
			var btns = query('.dijitButton', this.domNode);
			this._btnAdd = registry.byNode(btns[0]);
			this._btnFilter = registry.byNode(btns[1]);
			this._btnClear = registry.byNode(btns[2]);
			this._btnCancel = registry.byNode(btns[3]);
			this.connect(this._btnAdd, 'onClick', 'addRule');
			this.connect(this._btnClear, 'onClick', 'clear');
			this.connect(this._btnCancel, 'onClick', 'cancel');
			this.connect(this._accordionContainer, 'removeChild', '_updateButtons');
			this.connect(this._accordionContainer, 'removeChild', '_updatePaneTitle');
		},
		
		_updatePaneTitle: function(){
			// summary:
			//		Update each pane title. Only called after remove a RULE pane.
			array.forEach(this._accordionContainer.getChildren(), function(pane){
				pane._updateTitle();
			});
		},
		
		_updateButtons: function(){
			var children = this._accordionContainer.getChildren();
			//toggle filter button disable
			if(array.some(children, function(c){return c.getData() === null;})){
				this._btnFilter.set('disabled', true);
			}else{
				this._btnFilter.set('disabled', false);
			}
			//toggle add rule button disable
			var c = this.grid.filterBar.arg('maxRuleCount');
			this._btnAdd.set('disabled', children.length >= c && c > 0);
			this._btnClear.set('disabled', !this.grid.filterBar.filterData);
		},
		
		_updateAccordionContainerHeight: function(){
			// summary:
			//	Update the height of the accordion container to ensure consistent height of each accordion pane.
			var ac = this._accordionContainer, len = ac.getChildren().length;
			ac.domNode.style.height = 145 + len * this._titlePaneHeight + 'px';
			ac.resize();
		},
		uninitialize: function(){
			console.log('bbb');
			this.inherited(arguments);
		}
		
	});
});

},
'url:idx/oneui/templates/Dialog.html':"<div class=\"idxDialog\" role=\"dialog\" aria-labelledby=\"${id}_title\" aria-describedby=\"${id}_desc\">\r\n\t<div class=\"dijitDialogTitle\">\r\n\t\t<span id=\"${id}_title\" dojoAttachPoint=\"titleNode\" tabindex=\"0\">${title}</span>\r\n\t</div>\r\n\t<div class=\"dijitDialogInstruction\">\r\n\t\t<span id=\"${id}_desc\" role=\"document\" dojoAttachPoint=\"instructionNode\" tabindex=\"0\">${instruction}</span>\r\n\t</div>\t\r\n\t<div dojoAttachPoint=\"contentWrapper\" class=\"dijitDialogPaneContentWrapper compact\">\r\n\t\t<div dojoAttachPoint=\"containerNode\" class=\"dijitDialogPaneContent\" tabindex=\"0\">\r\n\t\t</div>\r\n\t</div>\r\n\t<div class=\"dijitDialogPaneActionBar\">\r\n\t\t<span class=\"idxDialogActionBarLead\">\r\n\t\t\t<a dojoAttachPoint=\"reference\" href=\"${referenceLink}\">${referenceName}</a>\r\n\t\t</span>\r\n\t\t<span class=\"idxDialogActionBarEnd\">\r\n\t\t<input dojoAttachPoint=\"closeButtonNode\"/>\r\n\t\t</span>\r\n\t</div>\r\n</div>",
'gridx/modules/filter/FilterTooltip':function(){
define([
	"dojo",
	"dijit",
	"dojo/_base/declare",
	"dojo/string",
	"dojo/i18n!../../nls/FilterBar",
	"./Filter",
	"./FilterDialog",
	"dijit/TooltipDialog",
	"dijit/popup",
	"dijit/Tooltip",
	"dojo/_base/array",
	"dojo/_base/event",
	"dojo/_base/html"
], function(dojo, dijit, declare, string, i18n){
	
	return declare(dijit.TooltipDialog, {
		// summary:
		//	Show status dialog of filter.
		grid: null,
		filterBar: null,
		postCreate: function(){
			this.inherited(arguments);
			this.filterBar = this.grid.filterBar;
			this.connect(this, 'onClick', '_onClick');
			this.connect(this, 'onMouseEnter', '_onMouseEnter');
			this.connect(this, 'onMouseLeave', '_onMouseLeave');
			dojo.addClass(this.domNode, 'gridxFilterTooltip');
			dojo.addClass(this.domNode, 'dijitTooltipBelow');
		},
		show: function(evt){
			this.inherited(arguments);
			dijit.popup.open({
				popup: this,
				x: evt.pageX,
				y: evt.pageY,
				padding: {x: -6, y: -3}
			});
		},
		hide: function(){
			this.inherited(arguments);
			dijit.popup.close(this);
		},

		buildContent: function(){
			// summary:
			//	Build the status of current filter.
			
			var fb = this.filterBar, nls = fb._nls, data = fb.filterData;
			if(!data || !data.conditions.length){return;}
			
			var typeString = data.type === 'all' ? nls.statusTipHeaderAll : nls.statusTipHeaderAny;
			var arr = ['<div class="gridxFilterTooltipTitle"><b>${i18n.statusTipTitleHasFilter}</b> ', 
				typeString, '</div><table><tr><th>${i18n.statusTipHeaderColumn}</th><th>${i18n.statusTipHeaderCondition}</th></tr>'
			];
			
			dojo.forEach(data.conditions, function(d, idx){
				var odd = idx%2 ? ' class="gridxFilterTooltipOddRow"' : '';
				arr.push('<tr', odd, '><td>', (d.colId ? this.grid.column(d.colId).name() : '${i18n.anyColumnOption}'), 
					'</td><td class="gridxFilterTooltipValueCell">', 
					'<div>',
					fb._getRuleString(d.condition, d.value, d.type),
					'<span action="remove-rule" title="${i18n.removeRuleButton}"',
					' class="gridxFilterTooltipRemoveBtn"><span class="gridxFilterTooltipRemoveBtnText">x</span></span></div></td></tr>');
			}, this);
			arr.push('</table>');
			this.i18n = i18n;
			this.set('content', string.substitute(arr.join(''), this));
			dojo.toggleClass(this.domNode, 'gridxFilterTooltipSingleRule', data.conditions.length === 1);
		},
		_onMouseEnter: function(e){
			this.isMouseOn = true;
		},
		_onMouseLeave: function(e){
			this.isMouseOn = false;
			this.hide();
		},
		_onClick: function(e){
			var tr = this._getTr(e), fb = this.filterBar;
			if(tr && /^span$/i.test(e.target.tagName)){
				//remove the rule
				fb.filterData.conditions.splice(tr.rowIndex - 1, 1);
				tr.parentNode.removeChild(tr);
				fb.applyFilter(fb.filterData);
				dojo.stopEvent(e);
			}else{
				this.filterBar.showFilterDialog();
				this.hide();
			}
		},
		_getTr: function(e){
			// summary:
			//	Get table row of status
			var tr = e.target;
			while(tr && !/^tr$/i.test(tr.tagName) && tr !== this.domNode){
				tr = tr.parentNode;
			}
			return (tr && /^tr$/i.test(tr.tagName)) ? tr : null;
		}
	});
});

},
'gridx/modules/Body':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/query",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/dom-construct",
	"dojo/dom-class",
	"dojo/_base/Deferred",
	"dojo/_base/sniff",
	"dojo/keys",
	"../core/_Module",
	"../core/util",
	"dojo/i18n!../nls/Body"
], function(declare, query, array, lang, domConstruct, domClass, Deferred, sniff, keys, _Module, util, nls){

	/*=====
	gridx._RowCellInfo = function(){
		// summary:
		//		This structure includes all possible information that can be used to identify a row or a cell, it is used
		//		to retrieve a row or a cell in grid body.
		//		Usually user only need to provide some of them that is sufficient to uniquely identify a row or a cell,
		//		e.g. rowId, or rowIndex and parentId, or visualIndex.
		// rowId: String|Number
		//		The ID of a row.
		// rowIndex: Integer
		//		The index of a row. It is the index below the parent of this row. The parent of root rows is an imaginary row
		//		with id "" (empty string).
		// visualIndex: Integer
		//		The visual index of a row. It represents the visual position of this row in the current body view.
		//		If there are no pagination, no filtering, no tree structure data, this value is equal to the row index.
		// colId: String|Number
		//		The ID of a column (should not be false values)
		// colIndex: Integer
		//		The index of a column.
		this.rowId = '';
		this.rowIndex = 0;
		this.visualIndex = 0;
		this.parentId = '';
		this.colId = 1;
		this.colIndex = 0;
	};
	=====*/

	return declare(/*===== "gridx.modules.Body", =====*/_Module, {
		// summary:
		//		The body UI of grid.
		// description:
		//		This module is in charge of row rendering. It should be compatible with virtual/non-virtual scroll, 
		//		pagination, details on demand, and even tree structure.

		name: "body",

		optional: ['tree'],

		getAPIPath: function(){
			// tags:
			//		protected extended
			return {
				body: this
			};
		},

		constructor: function(){
			var t = this,
				m = t.model,
				g = t.grid,
				dn = t.domNode = g.bodyNode,
				refresh = function(){ t.refresh(); };
			if(t.arg('rowHoverEffect')){
				domClass.add(dn, 'gridxBodyRowHoverEffect');
			}
			g.emptyNode.innerHTML = t.arg('loadingInfo', nls.loadingInfo);
			g._connectEvents(dn, '_onMouseEvent', t);
			t.aspect(m, 'onDelete', '_onDelete');
			t.aspect(m, 'onSet', '_onSet');
			t.aspect(g, 'onRowMouseOver', '_onRowMouseOver');
			t.aspect(g, 'onCellMouseOver', '_onCellMouseOver');
			t.aspect(g, 'onCellMouseOut', '_onCellMouseOver');
			t.connect(g.bodyNode, 'onmouseleave', function(){
				query('> .gridxRowOver', t.domNode).removeClass('gridxRowOver');
			});
			t.connect(g.bodyNode, 'onmouseover', function(e){
				if(e.target == g.bodyNode){
					query('> .gridxRowOver', t.domNode).removeClass('gridxRowOver');
				}
			});
			t.aspect(g, 'setStore', refresh);
		},

		preload: function(){
			// tags:
			//		protected extended
			this._initFocus();
		},

		load: function(args){
			// tags:
			//		protected extended
			var t = this,
				m = t.model,
				g = t.grid,
				finish = function(){
					t.aspect(m, 'onSizeChange', '_onSizeChange');
					t.loaded.callback();
				};
			//Load the store size
			m.when({}, function(){
				t.rootCount = t.rootCount || m.size();
				t.visualCount = g.tree ? g.tree.getVisualSize(t.rootStart, t.rootCount) : t.rootCount;
				finish();
			}).then(null, function(e){
				t._loadFail(e);
				finish();
			});
		},

		destroy: function(){
			// tags:
			//		protected extended
			this.inherited(arguments);
			this.domNode.innerHTML = '';
		},
	
		rowMixin: {
			node: function(){
				// summary:
				//		Get the dom node of this row.
				// return:
				//		DOMNode|null
				return this.grid.body.getRowNode({
					rowId: this.id
				});
			},

			visualIndex: function(){
				// summary:
				//		Get the visual index of this row.
				var t = this,
					id = t.id;
				return t.grid.body.getRowInfo({
					rowId: id,
					rowIndex: t.index(),
					parentId: t.model.parentId(id)
				}).visualIndex;
			}
		},

		cellMixin: {
			node: function(){
				// summary:
				//		Get the dom node of this cell.
				// return:
				//		DOMNode|null
				return this.grid.body.getCellNode({
					rowId: this.row.id,
					colId: this.column.id
				});
			}
		},

		//Public-----------------------------------------------------------------------------

		// rowHoverEffect: Boolean
		//		Whether to show a visual effect when mouse hovering a row.
		rowHoverEffect: true,

		// stuffEmptyCell: Boolean
		//		Whether to stuff a cell with &nbsp; if it is empty.
		stuffEmptyCell: true,

		// renderWholeRowOnSet: Boolean
		//		If true, the whole row will be re-rendered even if only one field has changed.
		//		Default to false, so that only one cell will be re-rendered editing that cell.
		renderWholeRowOnSet: false,

		getRowNode: function(args){
			// summary:
			//		Get the DOM node of a row
			// args: gridx.__RowCellInfo
			//		A row info object containing row index or row id
			// returns:
			//		The DOM node of the row. Null if not found.
			var rowQuery = this._getRowNodeQuery(args);
			return rowQuery && query('> ' + rowQuery, this.domNode)[0] || null;	//DOMNode|null
		},

		getCellNode: function(args){
			// summary:
			//		Get the DOM node of a cell
			// args: gridx.__RowCellInfo
			//		A cell info object containing sufficient info
			// returns:
			//		The DOM node of the cell. Null if not found.
			var t = this,
				colId = args.colId,
				cols = t.grid._columns,
				r = t._getRowNodeQuery(args);
			if(r){
				if(!colId && cols[args.colIndex]){
					colId = cols[args.colIndex].id;
				}
				r += " [colid='" + colId + "'].gridxCell";
				return query(r, t.domNode)[0] || null;	//DOMNode|null
			}
			return null;	//null
		},

		getRowInfo: function(args){
			// summary:
			//		Get complete row info by partial row info
			// args: gridx.__RowCellInfo
			//		A row info object containing partial row info
			// returns:
			//		A row info object containing as complete as possible row info.
			var t = this,
				m = t.model,
				g = t.grid,
				id = args.rowId;
			if(m.isId(id)){
				args.rowIndex = m.idToIndex(id);
				args.parentId = m.parentId(id);
			}
			if(typeof args.rowIndex == 'number' && args.rowIndex >= 0){
				args.visualIndex = g.tree ? 
					g.tree.getVisualIndexByRowInfo(args.parentId, args.rowIndex, t.rootStart) : 
					args.rowIndex - t.rootStart;
			}else if(typeof args.visualIndex == 'number' && args.visualIndex >= 0){
				if(g.tree){
					var info = g.tree.getRowInfoByVisualIndex(args.visualIndex, t.rootStart);
					args.rowIndex = info.start;
					args.parentId = info.parentId;
				}else{
					args.rowIndex = t.rootStart + args.visualIndex;
				} 
			}else{
				return args;	//gridx.__RowCellInfo
			}
			args.rowId = m.isId(id) ? id : m.indexToId(args.rowIndex, args.parentId);
			return args;	//gridx.__RowCellInfo
		},
	
		refresh: function(start){
			// summary:
			//		Refresh the grid body
			// start: Integer?
			//		The visual row index to start refresh. If omitted, default to 0.
			// returns:
			//		A deferred object indicating when the refreshing process is finished.
			var t = this;
			delete t._err;
			//Call when to make sure all pending commands are executed
			return t.model.when({}).then(function(){	//dojo.Deferred
				var rs = t.renderStart,
					rc = t.renderCount;
				if(typeof start == 'number' && start >= 0){
					start = rs > start ? rs : start;
					var count = rs + rc - start,
						n = query('> [visualindex="' + start + '"]', t.domNode)[0],
						uncachedRows = [],
						renderedRows = [];
					if(n){
						var rows = t._buildRows(start, count, uncachedRows, renderedRows);
						if(rows){
							domConstruct.place(rows, n, 'before');
							array.forEach(renderedRows, t.onAfterRow, t);
						}
					}
					while(n){
						var tmp = n.nextSibling,
							vidx = parseInt(n.getAttribute('visualindex'), 10),
							id = n.getAttribute('rowid');
						domConstruct.destroy(n);
						if(vidx >= start + count){
							t.onUnrender(id);
						}
						n = tmp;
					}
					Deferred.when(t._buildUncachedRows(uncachedRows), function(){
						t.onRender(start, count);
						t.onForcedScroll();
					});
				}else{
					t.renderRows(rs, rc, 0, 1);
					t.onForcedScroll();
				}
			}, function(e){
				t._loadFail(e);
			});
		},
	
		refreshCell: function(rowVisualIndex, columnIndex){
			// summary:
			//		Refresh a single cell
			// rowVisualIndex: Integer
			//		The visual index of the row of this cell
			// columnIndex: Integer
			//		The index of the column of this cell
			// returns:
			//		A deferred object indicating when this refreshing process is finished.
			var d = new Deferred(),
				t = this,
				m = t.model,
				g = t.grid,
				col = g._columns[columnIndex],
				cellNode = col && t.getCellNode({
					visualIndex: rowVisualIndex,
					colId: col.id
				});
			if(cellNode){
				var rowCache,
					rowInfo = t.getRowInfo({visualIndex: rowVisualIndex}),
					idx = rowInfo.rowIndex, pid = rowInfo.parentId;
				m.when({
					start: idx,
					count: 1,
					parentId: pid
				}, function(){
					rowCache = m.byIndex(idx, pid);
					if(rowCache){
						rowInfo.rowId = m.indexToId(idx, pid);
						var isPadding = g.tree && rowCache.data[col.id] === undefined;
						var cell = g.cell(rowInfo.rowId, col.id, 1);
						cellNode.innerHTML = t._buildCellContent(cell, isPadding);
						t.onAfterCell(cell);
					}
				}).then(function(){
					d.callback(!!rowCache);
				});
				return d;	//dojo.Deferred
			}
			d.callback(false);
			return d;	//dojo.Deferred
		},
		
		//Package--------------------------------------------------------------------------------
		
		// rootStart: [readonly] Integer
		//		The row index of the first root row that logically exists in the current body
		rootStart: 0,

		// rootCount: [readonly] Integer
		//		The count of root rows that logically exist in thi current body
		rootCount: 0,
	
		// renderStart: [readonly] Integer
		//		The visual row index of the first renderred row in the current body
		renderStart: 0,
		// renderCount: [readonly] Integer
		//		The count of renderred rows in the current body.
		renderCount: 0,
	
		// visualStart: [readonly] Integer
		//		The visual row index of the first row that is logically visible in the current body.
		//		This should be always zero.
		visualStart: 0, 
		// visualCount: [readonly] Integer
		//		The count of rows that are logically visible in the current body
		visualCount: 0,
	
		// autoUpdate: [read|write] Boolean
		//		Update grid body automatically when onNew/onSet/onDelete is fired
		autoUpdate: true,
	
		// autoChangeSize: [read|write] Boolean
		//		Whether to change rootStart and rootCount automatically when store size is changed.
		//		This need to be turned off when pagination is used.
		autoChangeSize: true,

		updateRootRange: function(start, count){
			// tags:
			//		private package
			var t = this, tree = t.grid.tree,
				vc = t.visualCount = tree ? tree.getVisualSize(start, count) : count;
			t.rootStart = start;
			t.rootCount = count;
			if(t.renderStart + t.renderCount > vc){
				t.renderStart = vc - t.renderCount;
				if(t.renderStart < 0){
					t.renderStart = 0;
					t.renderCount = vc;
				}
			}
			//If there was nothing shown in the body, should force the scroller to check again.
			if(!t.renderCount && vc){
				t.onForcedScroll();
			}
		},

		renderRows: function(start, count, position/*?top|bottom*/, isRefresh){
			// tags:
			//		private package
			var t = this,
				g = t.grid,
				str = '',
				uncachedRows = [], 
				renderedRows = [],
				n = t.domNode,
				en = g.emptyNode,
				emptyInfo = t.arg('emptyInfo', nls.emptyInfo),
				finalInfo = '';
			if(t._err){
				return;
			}
			if(count > 0){
				en.innerHTML = t.arg('loadingInfo', nls.loadingInfo);
				en.style.zIndex = '';
				if(position != 'top' && position != 'bottom'){
					t.model.free();
				}
				str = t._buildRows(start, count, uncachedRows, renderedRows);
				if(position == 'top'){
					t.renderCount += t.renderStart - start;
					t.renderStart = start;
					domConstruct.place(str, n, 'first');
				}else if(position == 'bottom'){
					t.renderCount = start + count - t.renderStart;
					domConstruct.place(str, n, 'last');
				}else{
					t.renderStart = start;
					t.renderCount = count;
					var scrollTop = isRefresh ? n.scrollTop : 0;
					n.scrollTop = 0;
					if(sniff('ie')){
						//In IE, setting innerHTML will completely destroy the node,
						//But CellWidget still need it.
						while(n.childNodes.length){
							n.removeChild(n.firstChild);
						}
					}
					n.innerHTML = str;
					if(scrollTop){
						n.scrollTop = scrollTop;
					}
					n.scrollLeft = g.hScrollerNode.scrollLeft;
					finalInfo = str ? "" : emptyInfo;
					if(!str){
						en.style.zIndex = 1;
					}
					t.onUnrender();
				}
				array.forEach(renderedRows, t.onAfterRow, t);
				Deferred.when(t._buildUncachedRows(uncachedRows), function(){
					en.innerHTML = finalInfo;
					t.onRender(start, count);
				});
			}else if(!{top: 1, bottom: 1}[position]){
				n.scrollTop = 0;
				n.innerHTML = '';
				en.innerHTML = emptyInfo;
				en.style.zIndex = 1;
				t.onUnrender();
				t.onEmpty();
				t.model.free();
			}
		},
	
		unrenderRows: function(count, preOrPost){
			// tags:
			//		private package
			if(count > 0){
				//Just remove the nodes from DOM tree instead of destroying them,
				//in case other logic still needs these nodes.
				var t = this, i = 0, id, bn = t.domNode;
				if(preOrPost == 'post'){
					for(; i < count && bn.lastChild; ++i){
						id = bn.lastChild.getAttribute('rowid');
						t.model.free(id);
						bn.removeChild(bn.lastChild);
						t.onUnrender(id);
					}
				}else{
					var tp = bn.scrollTop;
					for(; i < count && bn.firstChild; ++i){
						id = bn.firstChild.getAttribute('rowid');
						t.model.free(id);
						tp -= bn.firstChild.offsetHeight;
						bn.removeChild(bn.firstChild);
						t.onUnrender(id);
					}
					t.renderStart += i;
					bn.scrollTop = tp > 0 ? tp : 0;
				}
				t.renderCount -= i;
				//Force check cache size
				t.model.when();
			}
		},

		//Events--------------------------------------------------------------------------------

		onAfterRow: function(/* Row */){
			// summary:
			//		Fired when a row is created, data is filled in, and its node is inserted into the dom tree.
			// row: gridx.core.Row
			//		A row object representing this row.
		},

		onAfterCell: function(/* Cell */){
			// summary:
			//		Fired when a cell is updated by cell editor (or store data change), or by cell refreshing.
			//		Note this is not fired when rendering the whole grid. Use onAfterRow in that case.
			// cell: grid.core.Cell
			//		A cell object representing this cell
		},

		onRender: function(/*start, count*/){
			// summary:
			//		Fired everytime the grid body content is rendered or updated.
			// start: Integer
			//		The visual index of the start row that is affected by this rendering. If omitted, all rows are affected.
			// count: Integer
			//		The count of rows that is affected by this rendering. If omitted, all rows from start are affected.
		},

		onUnrender: function(/* id */){
			// summary:
			//		Fired when a row is unrendered (removed from the grid dom tree).
			//		Usually, this event is only useful when using virtual scrolling.
			// id: String|Number
			//		The ID of the row that is unrendered.
		},

		onDelete: function(/*id, index*/){
			// summary:
			//		Fired when a row in current view is deleted from the store.
			//		Note if the deleted row is not visible in current view, this event will not fire.
			// id: String|Number
			//		The ID of the deleted row.
			// index: Integer
			//		The index of the deleted row.
		},

		onSet: function(/* Row */){
			// summary:
			//		Fired when a row in current view is updated in store.
			// row: gridx.core.Row
			//		A row object representing the updated row.
		},

		onMoveToCell: function(){
			// summary:
			//		Fired when the focus is moved to a body cell by keyboard.
		},

		onEmpty: function(){
			// summary:
			//		Fired when there's no rows in current body view.
		},

		onForcedScroll: function(){
			// summary:
			//		Fired when the body needs to fetch more data, but there's no trigger to the scroller.
			//		This is an inner mechanism to solve some problems when using virtual scrolling or pagination.
			//		This event should not be used by grid users.
			// tags:
			//		private package
		},


		collectCellWrapper: function(/* wrappers, rowId, colId */){
			// summary:
			//		Fired when a cell is being rendered, so as to collect wrappers for the content in this cell.
			//		This is currently an inner mechanism used to implement widgets in cell and tree node.
			// tags:
			//		package
			// wrappers: Array
			//		An array of functions with signature function(cellData, rowId, colId) and should return a string to replace
			//		cell data. The connectors of this event should push a new wrapper function in this array.
			//		The functions in this array can also carry a number typed "priority" property.
			//		The wrappers will be executed in ascending order of this "priority" function.
			// rowId: String|Number
			//		The row ID of this cell
			// colId: String|Number
			//		The column ID of this cell.
		},

		//Private---------------------------------------------------------------------------
		_getRowNodeQuery: function(args){
			var r;
			if(this.model.isId(args.rowId)){
				r = "[rowid='" + args.rowId + "']";
			}else if(typeof args.rowIndex == 'number' && args.rowIndex >= 0){
				r = "[rowindex='" + args.rowIndex + "']";
				if(args.parentId){
					r += "[parentid='" + args.parentId + "']";
				}
			}else if(typeof args.visualIndex == 'number' && args.visualIndex >= 0){
				r = "[visualindex='" + args.visualIndex + "']";
			}
			return r && r + '.gridxRow';
		},

		_buildRows: function(start, count, uncachedRows, renderedRows){
			var t = this,
				i,
				end = start + count,
				s = [],
				g = t.grid,
				m = t.model,
				w = t.domNode.scrollWidth;
			for(i = start; i < end; ++i){
				var rowInfo = t.getRowInfo({visualIndex: i}),
					row = g.row(rowInfo.rowId, 1);
				s.push('<div class="gridxRow ', i % 2 ? 'gridxRowOdd' : '',
					'" role="row" visualindex="', i);
				if(row){
					m.keep(row.id);
					s.push('" rowid="', row.id,
						'" rowindex="', rowInfo.rowIndex,
						'" parentid="', rowInfo.parentId,
						'">', t._buildCells(row),
					'</div>');
					renderedRows.push(row);
				}else{
					s.push('"><div class="gridxRowDummy" style="width:', w, 'px;"></div></div>');
					rowInfo.start = rowInfo.rowIndex;
					rowInfo.count = 1;
					uncachedRows.push(rowInfo);
				}
			}
			return s.join('');
		},

		_buildUncachedRows: function(uncachedRows){
			var t = this;
			return uncachedRows.length && t.model.when(uncachedRows, function(){
				try{
					array.forEach(uncachedRows, t._buildRowContent, t);
				}catch(e){
					t._loadFail(e);
				}
			}).then(null, function(e){
				t._loadFail(e);
			});
		},

		_loadFail: function(e){
			console.error(e);
			var en = this.grid.emptyNode;
			en.innerHTML = this.arg('loadFailInfo', nls.loadFailInfo);
			en.style.zIndex = 1;
			this.domNode.innerHTML = '';
			this._err = 1;	//1 as true;
		},
	
		_buildRowContent: function(rowInfo){
			var t = this,
				n = query('> [visualindex="' + rowInfo.visualIndex + '"]', t.domNode)[0];
			if(n){
				var row = t.grid.row(rowInfo.rowIndex, 0, rowInfo.parentId);
				if(row){
					t.model.keep(row.id);
					n.setAttribute('rowid', row.id);
					n.setAttribute('rowindex', rowInfo.rowIndex);
					n.setAttribute('parentid', rowInfo.parentId || '');
					n.innerHTML = t._buildCells(row);
					t.onAfterRow(row);
				}else{
					throw new Error('Row is not in cache:' + rowInfo.rowIndex);
				}
			}
		},
	
		_buildCells: function(row){
			var col, cell, isPadding, cls, style, i, len,
				t = this,
				g = t.grid,
				columns = g._columns,
				rowData = row.data(),
				isFocusArea = g.focus && (g.focus.currentArea() == 'body'),
				sb = ['<table class="gridxRowTable" role="presentation" border="0" cellpadding="0" cellspacing="0"><tr>'];
			for(i = 0, len = columns.length; i < len; ++i){
				col = columns[i];
				isPadding = g.tree && rowData[col.id] === undefined;
				cell = g.cell(row.id, col.id, 1);
				cls = (lang.isFunction(col['class']) ? col['class'](cell) : col['class']) || '';
				style = (lang.isFunction(col.style) ? col.style(cell) : col.style) || '';
				sb.push('<td aria-describedby="', g.id, '-', col.id, '" class="gridxCell ');
				if(isPadding){
					sb.push('gridxPaddingCell');
				}
				if(isFocusArea && t._focusCellRow === row.visualIndex() && t._focusCellCol === i){
					sb.push('gridxCellFocus');
				}
				sb.push(cls,
					'" aria-readonly="true" role="gridcell" tabindex="-1" colid="', col.id, 
					'" style="width: ', col.width,
					'; ', style,
					'">', t._buildCellContent(cell, isPadding),
				'</td>');
			}
			sb.push('</tr></table>');
			return sb.join('');
		}, 
	
		_buildCellContent: function(cell, isPadding){
			var r = '',
				col = cell.column,
				row = cell.row,
				data = cell.data();
			if(!isPadding){
				var s = col.decorator ? col.decorator(data, row.id, row.visualIndex()) : data;
				r = this._wrapCellData(s, row.id, col.id);
			}
			return (r === '' || r === null || r === undefined) && (sniff('ie') < 8 || this.arg('stuffEmptyCell')) ? '&nbsp;' : r;
		},

		_wrapCellData: function(cellData, rowId, colId){
			var wrappers = [];
			this.collectCellWrapper(wrappers, rowId, colId);
			var i = wrappers.length - 1;
			if(i > 0){
				wrappers.sort(function(a, b){
					a.priority = a.priority || 0;
					b.priority = b.priority || 0;
					return a.priority - b.priority;
				});
			}
			for(; i >= 0; --i){
				cellData = wrappers[i].wrap(cellData, rowId, colId);
			}
			return cellData;
		},
	
		//Events-------------------------------------------------------------
		_onMouseEvent: function(eventName, e){
			var g = this.grid,
				evtCell = 'onCell' + eventName,
				evtRow = 'onRow' + eventName;
			if(g._isConnected(evtCell) || g._isConnected(evtRow)){
				this._decorateEvent(e);
				if(e.rowId){
					if(e.columnId){
						g[evtCell](e);
					}
					g[evtRow](e);
				}
			}
		},
	
		_decorateEvent: function(e){
			var n = e.target || e.originalTarget,
				g = this.grid,
				tag;
			for(; n && n != g.bodyNode; n = n.parentNode){
				tag = n.tagName.toLowerCase();
				if(tag == 'td' && domClass.contains(n, 'gridxCell')){
					var col = g._columnsById[n.getAttribute('colid')];
					e.cellNode = n;
					e.columnId = col.id;
					e.columnIndex = col.index;
				}
				if(tag == 'div' && domClass.contains(n, 'gridxRow')){
					e.rowId = n.getAttribute('rowid');
					e.parentId = n.getAttribute('parentid');
					e.rowIndex = parseInt(n.getAttribute('rowindex'), 10);
					e.visualIndex = parseInt(n.getAttribute('visualindex'), 10);
					return;
				}
			}
		},
	
		//Store Notification-------------------------------------------------------------------
		_onSet: function(id, index, rowCache, oldCache){
			var t = this;
			if(t.autoUpdate && rowCache){
				var g = t.grid,
					row = g.row(id, 1),
					rowNode = row && row.node();
				if(rowNode){
					var curData = rowCache.data,
						oldData = oldCache.data,
						cols = g._columns,
						renderWhole = t.arg('renderWholeRowOnSet'),
						changedCols = [];
					if(!renderWhole){
						array.some(cols, function(col){
							if(curData[col.id] !== oldData[col.id]){
								changedCols.push(col);
							}
							return changedCols.length > 1;
						});
					}
					if(renderWhole || changedCols.length > 1){
						rowNode.innerHTML = t._buildCells(row);
						t.onAfterRow(row);
						t.onSet(row);
						t.onRender(index, 1);
					}else if(changedCols.length == 1){
						var col = changedCols[0],
							isPadding = g.tree && curData[col.id] === undefined,
							cell = row.cell(col.id, 1);
						cell.node().innerHTML = t._buildCellContent(cell, isPadding);
						t.onAfterCell(cell);
					}
				}
			}
		},

		_onDelete: function(id){
			var t = this;
			if(t.autoUpdate){
				var node = t.getRowNode({rowId: id});
				if(node){
					var sn, count = 0,
						start = parseInt(node.getAttribute('rowindex'), 10),
						pid = node.getAttribute('parentid'),
						pids = {},
						toDelete = [node],
						rid, ids = [id],
						vidx;
					pids[id] = 1;
					for(sn = node.nextSibling; sn && pids[sn.getAttribute('parentid')]; sn = sn.nextSibling){
						rid = sn.getAttribute('rowid');
						ids.push(rid);
						toDelete.push(sn);
						pids[rid] = 1;
					}
					for(; sn; sn = sn.nextSibling){
						if(sn.getAttribute('parentid') == pid){
							sn.setAttribute('rowindex', parseInt(sn.getAttribute('rowindex'), 10) - 1);
						}
						vidx = parseInt(sn.getAttribute('visualindex'), 10) - toDelete.length;
						sn.setAttribute('visualindex', vidx);
						domClass.toggle(sn, 'gridxRowOdd', vidx % 2);
						++count;
					}
					t.renderCount -= toDelete.length;
					array.forEach(toDelete, domConstruct.destroy);
					array.forEach(ids, t.onUnrender, t);
					if(t.autoChangeSize && t.rootStart === 0 && !pid){
						t.updateRootRange(0, t.rootCount - 1);
					}
					t.onDelete(id, start);
					t.onRender(start, count);
				}
			}
		},
	
		_onSizeChange: function(size, oldSize){
			var t = this;
			if(t.autoChangeSize && t.rootStart === 0 && (t.rootCount === oldSize || oldSize < 0)){
				t.updateRootRange(0, size);
				t.refresh();
			}
		},
		
		//-------------------------------------------------------------------------------------
		_onRowMouseOver: function(e){
			var preNode = query('> div.gridxRowOver', this.domNode)[0],
				rowNode = this.getRowNode({rowId: e.rowId});
			if(preNode != rowNode){
				if(preNode){
					domClass.remove(preNode, 'gridxRowOver');
				}
				if(rowNode){
					domClass.add(rowNode, 'gridxRowOver');
				}
			}
		},
		
		_onCellMouseOver: function(e){
			domClass.toggle(e.cellNode, 'gridxCellOver', e.type == 'mouseover');
		},
	
		//Focus------------------------------------------------------------------------------------------
		_focusCellCol: 0,
		_focusCellRow: 0,

		_initFocus: function(){
			var t = this,
				g = t.grid,
				ltr = g.isLeftToRight(),
				bn = g.bodyNode,
				focus = g.focus,
				c = 'connect';
			if(focus){
				focus.registerArea({
					name: 'body',
					priority: 1,
					focusNode: bn,
					scope: t,
					doFocus: t._doFocus,
					doBlur: t._blurCell,
					onFocus: t._onFocus,
					onBlur: t._blurCell
				});
				t[c](g.mainNode, 'onkeypress', function(evt){
					if(focus.currentArea() == 'body' && (!g.tree || !evt.ctrlKey)){
						focus._noBlur = 1;	//1 as true
						var dk = keys, arr = {}, dir = ltr ? 1 : -1;
						arr[dk.LEFT_ARROW] = [0, -dir, evt];
						arr[dk.RIGHT_ARROW] = [0, dir, evt];
						arr[dk.UP_ARROW] = [-1, 0, evt];
						arr[dk.DOWN_ARROW] = [1, 0, evt];
						t._moveFocus.apply(t, arr[evt.keyCode] || []);
						focus._noBlur = 0;	//0 as false
					}
				});
				t[c](g, 'onCellClick', function(evt){
					t._focusCellRow = evt.visualIndex;
					t._focusCellCol = evt.columnIndex;
				});
				t[c](t, 'onRender', function(start, count){
					if(t._focusCellRow >= start &&
						t._focusCellRow < start + count &&
						focus.currentArea() == 'body'){
						t._focusCell();
					}
				});
				t[c](g.emptyNode, 'onfocus', function(){
					focus.focusArea('body');
				});
			}
		},

		_doFocus: function(evt){
			return this._focusCell(evt) || this._focusCell(0, -1, -1);
		},

		_focusCell: function(evt, rowVisIdx, colIdx){
			util.stopEvent(evt);
			var t = this,
				g = t.grid;
			colIdx = colIdx >= 0 ? colIdx : t._focusCellCol;
			rowVisIdx = rowVisIdx >= 0 ? rowVisIdx : t._focusCellRow;
			var colId = g._columns[colIdx].id,
				n = t.getCellNode({
					visualIndex: rowVisIdx,
					colId: colId
				});
			if(n){
				var preNode = query('.gridxCellFocus', t.domNode)[0];
				if(n != preNode){
					if(preNode){
						domClass.remove(preNode, 'gridxCellFocus');
					}
					domClass.add(n, 'gridxCellFocus');
					t._focusCellRow = rowVisIdx;
					t._focusCellCol = colIdx;
					g.header._focusHeaderId = colId;
				}
				g.hScroller.scrollToColumn(colId);
				if(sniff('ie') < 8){
					//In IE7 focus cell node will scroll grid to the left most.
					//So save the scrollLeft first and then set it back.
					//FIXME: this still makes the grid body shake, any better solution?
					var scrollLeft = g.bodyNode.scrollLeft;
					n.focus();
					g.bodyNode.scrollLeft = scrollLeft;
				}else{
					n.focus();
				}
			}else if(!g.rowCount()){
				g.emptyNode.focus();
				return true;
			}
			return n;
		},

		_moveFocus: function(rowStep, colStep, evt){
			if(rowStep || colStep){
				util.stopEvent(evt); //Prevent scrolling the whole page.
				var r, c,
					t = this,
					g = t.grid, 
					cols = g._columns,
					vc = t.visualCount;
				r = t._focusCellRow + rowStep;
				r = r < 0 ? 0 : (r >= vc ? vc - 1 : r);
				c = t._focusCellCol + colStep;
				c = c < 0 ? 0 : (c >= cols.length ? cols.length - 1 : c);
				g.vScroller.scrollToRow(r).then(function(){
					t._focusCell(0, r, c);
					t.onMoveToCell(r, c, evt);
				});
			}
		},

		_nextCell: function(r, c, dir, checker){
			var d = new Deferred(),
				g = this.grid,
				cc = g._columns.length,
				rc = this.visualCount;
			do{
				c += dir;
				if(c < 0 || c >= cc){
					r += dir;
					c = c < 0 ? cc - 1 : 0;
					if(r < 0){
						r = rc - 1;
						c = cc - 1;
					}else if(r >= rc){
						r = 0;
						c = 0;
					}
				}
			}while(!checker(r, c));
			g.vScroller.scrollToRow(r).then(function(){
				d.callback({r: r, c: c});
			});
			return d;
		},

		_blurCell: function(){
			var n = query('.gridxCellFocus', this.domNode)[0];
			if(n){
				domClass.remove(n, 'gridxCellFocus');
			}
			return true;
		},

		_onFocus: function(evt){
			for(var n = evt.target, t = this; n && n != t.domNode; n = n.parentNode){
				if(domClass.contains(n, 'gridxCell')){
					var colIndex = t.grid._columnsById[n.getAttribute('colid')].index;
					while(!domClass.contains(n, 'gridxRow')){
						n = n.parentNode;
					}
					return t._focusCell(0, parseInt(n.getAttribute('visualindex'), 10), colIndex);
				}
			}
			return false;
		}
	});
});

},
'ibm/tivoli/navmodel/Datasource':function(){
// wrapped by build app
define("ibm/tivoli/navmodel/Datasource", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/ProviderUtils"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */
dojo.provide("ibm.tivoli.navmodel.Datasource"); 

// includes
dojo.require("ibm.tivoli.navmodel.ProviderUtils");

////////////////////////////////////
// Datasource
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Datasource", null, {
	// private members       
	_datasource: null,
	_baseUrl: null,
	_version: null,
	
	/*=====
	constructor_kwArgs: function(datasource, baseUrl, version) {
		// datasource: Object
		//		json datasource object
		// baseUrl: String
		//		base url for identifying rest service
		// version: Float
		//		optional: version used to communicate with rest service (i.e. json version)
	},
	=====*/
	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs Provider object.
		this._baseUrl = kwArgs.baseUrl;
		this._version = kwArgs.version;
		this._datasource = kwArgs.datasource;
		if (!this._datasource) {
			this._datasource = {};
		}
	},

	getAttribute: function(key) {
		// summary:
		//		Returns general attribute for requested key
		return this._datasource[key];
	},

	setAttribute: function(key, value) {		
		// summary:
		//		Set general attribute for requested key
		this._datasource[key] = value;
	},
	
	removeAttribute: function(key) {
		// summary:
		//		Remove general attribute for requested key
		delete this._datasource[key];
	},
	
	getProviderId: function() {
		// summary:
		//		Returns the providerId of this datasource
		return this._datasource.providerId;
	},
	
	getId: function() {
		// summary:
		//		Returns the id of this datasource
		return this._datasource.id;
	},
	
	getLabel: function() {
		// summary:
		//		Returns the label of this datasource
		return this._datasource.label;
	},
	
	getDescription: function() {
		// summary:
		//		Returns the description of this datasource
		return this._datasource.description;
	},
	
	getType: function() {
		// summary:
		//		Returns the type of this datasource.
		return this._datasource.type;
	},
	
	/*=====
	getDatasets_kwArgs: function(onComplete, onError, start, count, condition, sync) {
		// onComplete: function(datasets, datasetCount, kwArgs)
		//		callback for successful fetch
		// onError: function(error, exception, ioArgs)
		//		callback for error conditions
		// start: int
		//		start index to fetch
		// count: int
		//		number of datasources to fetch; null means all
		// sort: String
		//		comma deliminated attribute string (preceded with optional +/- indicating ascending/descending)
		//		example:  label,type
		// condition: String
		//		condition expression string 
		//		example: description contains 'abcd'
		// sync: Boolean
		//		indicates if fetch is synchronous		
	},
	=====*/
	
	getDatasets: function(/*getDatasets_kwArgs*/ kwArgs) {
		var args = {
			onComplete: function(datasets, datasetCount, kwArgs) {
			},
			onError: function(error, exception, ioArgs) {
			},
			baseUrl: this._baseUrl,
			provider: this._datasource.providerId,
			datasource: this._datasource.id,		
			start: 0,
			count: null,
			sourt: null,
			condition: null,
			sync: false,
			version: this._version		
		};
		dojo.mixin(args, kwArgs);
		
		return ibm.tivoli.navmodel.ProviderUtils.getDatasets(args);
	}
});

});

},
'ibm/tivoli/tip/navwidget/dojo/ToolbarUtil':function(){
// wrapped by build app
define("ibm/tivoli/tip/navwidget/dojo/ToolbarUtil", ["dijit","dojo","dojox","dojo/require!dijit/Menu,dijit/MenuItem,dijit/form/Button,dijit/form/ComboButton,dijit/form/DropDownButton,dijit/form/TextBox,dojo/data/ItemFileWriteStore,ibm/tivoli/tip/navwidget/topology/dijit/TopologyLabel"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.require("dijit.Menu");
dojo.require("dijit.MenuItem");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.ComboButton");
dojo.require("dijit.form.DropDownButton");
dojo.require("dijit.form.TextBox");
dojo.require("dojo.data.ItemFileWriteStore");
dojo.require("ibm.tivoli.tip.navwidget.topology.dijit.TopologyLabel");

dojo.provide("ibm.tivoli.tip.navwidget.dojo.ToolbarUtil");

/*=====
ibm.tivoli.tip.navwidget.dojo.ToolbarUtil = {
	// summary: Toolbar creation utilities
};
=====*/

ibm.tivoli.tip.navwidget.dojo.ToolbarUtil._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.tip.navwidget.dojo.ToolbarUtil");

ibm.tivoli.tip.navwidget.dojo.ToolbarUtil.customLabel = function(/*Object*/ kwArgs) {
	// summary:
	//		Create a label object for the toolbar item	
	// id: String
	//		base id value for the component; will be appended to
	// item: Object
	//		the toolbar item from the widget definition
	// tags:
	//		public
	var label = null;
	var item = kwArgs.item;

	if (item.label && item.label.showLabel){
		label = new ibm.tivoli.tip.navwidget.topology.dijit.TopologyLabel({ 
			labelFor : kwArgs.id + "_" + item.id,
			labelText: item.label.value});
	}

	return label;
};

ibm.tivoli.tip.navwidget.dojo.ToolbarUtil.customText = function(/*Object*/ kwArgs) {
	// summary:
	//		Create a text object for the toolbar item	
	// id: String
	//		base id value for the component; will be appended to
	// item: Object
	//		the toolbar item from the widget definition
	// tags:
	//		public
	var item = kwArgs.item;
	return new dijit.form.TextBox({id: kwArgs.id + "_" + item.id});				
};

ibm.tivoli.tip.navwidget.dojo.ToolbarUtil.customButton = function(/*Object*/ kwArgs) {
	// summary:
	//		Create a button object for the toolbar item	
	// id: String
	//		base id value for the component; will be appended to
	// item: Object
	//		the toolbar item from the widget definition
	// tags:
	//		public
	var item = kwArgs.item;
	var buttonArgs = {
		id: kwArgs.id + "_" +  item.id,
		label: item.label.value,
		showLabel: item.label.showLabel,
		iconClass:item.iconClass				
	};

	var button = new dijit.form.Button(buttonArgs);
	if (item.alt) {
		button.attr("alt", item.alt.value); 
		button.attr("title", item.alt.value); 
	}
				
	if (item.action) {		
		ibm.tivoli.tip.navwidget.dojo.ToolbarUtil._connect(button, item.action.event, item.action.handler);
	};

	return button;
};


ibm.tivoli.tip.navwidget.dojo.ToolbarUtil.customComboButton = function(/*Object*/ kwArgs) {
	// summary:
	//		Create a combo button object for the toolbar item	
	// id: String
	//		base id value for the component; will be appended to
	// item: Object
	//		the toolbar item from the widget definition
	// tags:
	//		public
	var item = kwArgs.item;

	var options = new dojo.data.ItemFileWriteStore({data: {identifier: 'name', items:[]}});
	if (item.options) {
		for (var i = 0; i < item.options.length; ++i) {
			options.newItem({name: item.options[i].name, value: item.options[i].value});
		}
	};	

	var buttonArgs = {
		id: kwArgs.id + "_" + item.id,
		name: kwArgs.id + "_" + item.id,
		store: options,
		searchAttr: "name",
		value:item.options[0].value //TODO add possibility of specifying selected value
	};
							
	return new dijit.form.ComboBox(buttonArgs);
};

ibm.tivoli.tip.navwidget.dojo.ToolbarUtil.customDropDown = function(/*Object*/ kwArgs) {
	// summary:
	//		Create a drop down object for the toolbar item	
	// id: String
	//		base id value for the component; will be appended to
	// item: Object
	//		the toolbar item from the widget definition
	// tags:
	//		public
	var item = kwArgs.item;
			
	var dropDownArgs = {
		id: kwArgs.id + "_" +  item.id + "_menu",
		style: "display: none;",
		autoSize: true
	};
	var dropDownMenu = new dijit.Menu(dropDownArgs);
				
	if (item.options) {
		for(var i = 0; i < item.options.length; ++i) {
			var menuArgs = {
				id: kwArgs.id + "_" +  item.id + "_menu_" + i,
				label: item.options[i].label.value,
				iconClass: item.options[i].iconClass
			};
						
			var menuItem = new dijit.MenuItem(menuArgs);
			ibm.tivoli.tip.navwidget.dojo.ToolbarUtil._connect(menuItem, "onClick", item.options[i].action.handler);
			dropDownMenu.addChild(menuItem);
		}
	}

	var buttonArgs = {
		id: kwArgs.id + "_" +  item.id,
		label: item.label.value,
		title: item.label.value,
		showLabel: item.label.showLabel,					
		dropDown: dropDownMenu,
		iconClass: item.iconClass
	};
	var button = new dijit.form.DropDownButton(buttonArgs);
			
	if (item.alt){
		button.attr("alt", item.alt.value);
		button.attr("title", item.alt.value); 
	}
		
	return button;
};

ibm.tivoli.tip.navwidget.dojo.ToolbarUtil.viewToggler = function(/*Object*/ kwArgs) {
	// summary:
	//		Create a view toggler for the toolbar item	
	// view: ibm.tivoli.tip.navwidget.dijit.NavWidgetPluggableView
	//		pluggable view associated with the toolbar
	// item: Object
	//		the toolbar item from the widget definition
	// tags:
	//		public
	var item = kwArgs.item;
			
	var pulldownArgs = {
		id: kwArgs.view.id + "_viewMenu",
		style: "display: none;",
		autoSize: true
	};
	var viewPulldown = new dijit.Menu(pulldownArgs);
			
	var pluggableViews = kwArgs.view.getSiblings();
	var defaultLabel = null;
	var defaultIconClass = null;
	if (pluggableViews) {
		for (var i = 0; i < pluggableViews.length; ++i) {
			var label = pluggableViews[i].getLabel();
			var iconClass = pluggableViews[i].getLabelIconClass();
			var viewMenuItem = new dijit.MenuItem({
 				id: kwArgs.view.id + "_viewMenuItem_" + pluggableViews[i].viewId,
 				label: label,
 				viewId: pluggableViews[i].viewId,
       			iconClass: iconClass
    			});

			ibm.tivoli.tip.navwidget.dojo.ToolbarUtil._connect(viewMenuItem, "onClick", kwArgs.view, function(/*Object*/ event) {							
				var viewMenuItem = event.target;
				while (!viewMenuItem.parentNode.parentNode.id || viewMenuItem.parentNode.parentNode.id != this.id + "_viewMenu") {
					viewMenuItem = viewMenuItem.parentNode;
				};
				viewMenuItem = dijit.byId(viewMenuItem.id);
				if (this.parentPluggableView) {
					this.parentPluggableView.makePluggableViewVisible(viewMenuItem.viewId);
				}							
			});
			viewPulldown.addChild(viewMenuItem);	
						
			// mark default view as selected
			if (pluggableViews[i].viewId == kwArgs.view.viewId) {
				defaultLabel = label;
				defaultIconClass = iconClass;
			}
		}
	};

	ibm.tivoli.tip.navwidget.dojo.ToolbarUtil._connect(kwArgs.view, "onSetDataset", function() {
		var pluggableViews = this.getSiblings();
		if (pluggableViews) {
			for (var i = 0; i < pluggableViews.length; ++i) {
				var menuItem = dijit.byId(this.id + "_viewMenuItem_" + pluggableViews[i].viewId);
				if (menuItem) {
					var applicable = this.parentPluggableView.isPluggableViewApplicable(pluggableViews[i].viewId, this.dataset);
					menuItem.attr("disabled", !applicable);
				}
			}
		}
	});
	    		
	var buttonArgs = {
		label: defaultLabel,
		dropDown: viewPulldown,
		title:  defaultLabel,
		iconClass: defaultIconClass,
		showLabel: false //OID requests we don't show the label in the toolbar
	};
	return new dijit.form.ComboButton(buttonArgs);
};				

ibm.tivoli.tip.navwidget.dojo.ToolbarUtil._connect = function(object, event, context, handler) {
	try {
		dojo.connect(object, event, context, handler);
	} catch (error) {
		var msg = "Failed to dojo.connect(" + object + ", " + event + ", " + context + ", " + handler + ")";
		ibm.tivoli.tip.navwidget.dojo.ToolbarUtil._trace.error(msg, "ibm.tivoli.tip.navwidget.dojo.ToolbarUtil._connect");
	}
};



});

},
'gridx/modules/pagination/PaginationBar':function(){
define([
	"dojo/_base/declare",
	"./_PaginationBarBase",
	"./LinkPager",
	"../../core/_Module",
	"./GotoPagePane",
	"dijit/Dialog",
	"dijit/form/Button",
	"dijit/form/NumberTextBox"
], function(declare, _PaginationBarBase, Pager, _Module, GotoPagePane, Dialog, Button, NumberTextBox){

	return declare(/*===== "gridx.modules.pagination.PaginationBar", =====*/_PaginationBarBase, {
		// summary:
		//		This module implements a pagination bar UI that uses link buttons for pages and page sizes.

		// gotoButton: Boolean|String
		gotoButton: true,

		// visibleSteppers: Integer
		visibleSteppers: 3,

		// sizeSeparator: String
		sizeSeparator: '|',

		// gotoPagePane: [private]
		gotoPagePane: GotoPagePane,

		// dialogClass: [private]
		dialogClass: Dialog,

		// buttonClass: [private]
		buttonClass: Button,

		// numberTextBoxClass: [private]
		numberTextBoxClass: NumberTextBox,

	/*=====
		// Configurable texts on the pagination bar:
		pageIndexTitleTemplate: '',
		pageIndexWaiTemplate: '',
		pageIndexTemplate: '',
		pageSizeTitleTemplate: '',
		pageSizeWaiTemplate: '',
		pageSizeTemplate: '',
		pageSizeAllTitleText: '',
		pageSizeAllWaiText: '',
		pageSizeAllText: '',
		descriptionTemplate: '',
		descriptionSelectionTemplate: '',
	=====*/

		// pagerClass: [private]
		pagerClass: Pager
	});	
});

},
'ibm/tivoli/tip/navwidget/dijit/NavWidgetObjectPluggableView':function(){
// wrapped by build app
define("ibm/tivoli/tip/navwidget/dijit/NavWidgetObjectPluggableView", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/Trace,ibm/tivoli/tip/navwidget/dijit/NavWidgetPluggableView,dijit/_Widget"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.dijit.NavWidgetObjectPluggableView"); 

// includes
dojo.require("ibm.tivoli.navmodel.Trace");
dojo.require("ibm.tivoli.tip.navwidget.dijit.NavWidgetPluggableView");
dojo.require("dijit._Widget");
ibm.tivoli.tip.navwidget.dijit.NavWidgetObjectPluggableView._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.tip.navwidget.dijit.NavWidgetObjectPluggableView");
 
dojo.declare("ibm.tivoli.tip.navwidget.dijit.NavWidgetObjectPluggableView", [ibm.tivoli.tip.navwidget.dijit.NavWidgetPluggableView], {
	// summary:
	//		Base class for an object-specific view that plugs into a NavWidget
	// description:
	//		Coming soon
	
    //**************************************************************************************/
	
	// _items: [private] ibm.tivoli.navmodel.Item[]
	//		The items within the dataset to render
	_items: null,
	
	// _links: [private] ibm.tivoli.navmodel.Link[]
	//		The links within the dataset to render
	_links: null,
	
	// _trace: [private] Trace
	//		Utility for trace and logging
	_trace: ibm.tivoli.tip.navwidget.dijit.NavWidgetObjectPluggableView._trace,
	
	// _setObjectsLoadDataCompletesSynchronously: [protected] Boolean
	//		Allows subclasses to specify whether they load data asynchronously or not
	_setObjectsLoadDataCompletesSynchronously: true,
	
	isApplicableForObjects: function(/*ibm.tivoli.navmodel.Item[]*/ items, /*ibm.tivoli.navmodel.Link[]*/ links, /*ibm.tivoli.navmodel.Link[]*/ relationships){
		// summary:
		//		Analyzes object data and/or metadata to decide if this view makes sense for the given objects.
		// tags:
		//		public

		return true;
	},
	
	_setObjectsActive: false,
	setObjects: function(/*ibm.tivoli.navmodel.Item[]*/ items, /*ibm.tivoli.navmodel.Link[]*/ links, /*ibm.tivoli.navmodel.Link[]*/ relationships){
		// summary:
		//		Kick off the setObjects life-cycle of a widget. Default implementation will call (in order) the
		//      following protected functions:
		//			- _setObjectsInit
		//			- _setObjectsLoadData
		//			- _setObjectsComplete
		//
		//		It is NOT recommended for subclasses to override setObjects, but instead one or more of the
		//		protected methods that get called. In particular, it is expected subclasses would override 
		//		_setObjectsLoadData. Be sure to call super on any of the overridden methods.
		// items:
		//		An array of items
		// links:
		//		An array of links
		// relationships:
		//		An array of relationships
		// description:
		//		Tells this pluggable view it's time to render new objects
		// tags:
		//		public
	
		if (this._setObjectsActive) {
			if (this._trace.isFinest()) {
				this._trace.finest("Previous request to setObject still active, so ignoring; view = " + this.id, "setObjects");
			}
			return;
		}
		
		this._setObjectsInit(items, links, relationships);
		this._setObjectsLoadData(items, links, relationships);
	},
	
	_setObjectsInit: function(/*ibm.tivoli.navmodel.Item[]*/ items, /*ibm.tivoli.navmodel.Link[]*/ links, /*ibm.tivoli.navmodel.Relationship[]*/ relationships){
		// summary:
		//		Called during the setObjects lifecycle for any initialization that should take place in the view before
		//		starting to pull information out of the objects for rendering. Subclasses will probably NOT need to override.
		// items:
		//		An array of items
		// links:
		//		An array of links
		// relationships:
		//		An array of relationships
		// description:
		//		Tells this pluggable view it's time to initialize based on the new objects
		// tags:
		//		protected
		
		//Remember we're in setDataset
		this._setObjectsActive = true;
		
		//Store away the object
		this._items = items;
		this._links = links;
		this._relationships = relationships;
	},
	
	_setObjectsLoadData: function(/*ibm.tivoli.navmodel.Item[]*/ items, /*ibm.tivoli.navmodel.Link[]*/ links,  /*ibm.tivoli.navmodel.Relationship[]*/ relationships){
		// summary:
		//		Called during the setObjects lifecycle to tell the implementor to process the objects to set up
		//		rendering (e.g., start pulling info out of objects, set data models on any children, etc.). The 
		//		base implementation does nothing, so all work delegated to subclass.
		// items:
		//		An array of items
		// links:
		//		An array of links
		// relationships:
		//		An array of relationships
		// description:
		//		Tells the pluggable view it's time to process the new dataset
		// tags:
		//		protected
		
		//Delegate impl to subclass.
	},
	
	_setObjectsLoadDataActive: false,
	_setObjectsLoadData: function(/*ibm.tivoli.navmodel.Item[]*/ items, /*ibm.tivoli.navmodel.Link[]*/ links, /*ibm.tivoli.navmodel.Relationship[]*/ relationships){
		// summary:
		//		Called during the setObjects lifecycle to tell the implementor to process the dataset to set up
		//		rendering (e.g., start pulling info out of dataset, set data models on any children, etc.). The 
		//		base implementation does nothing, so all work delegated to subclass.
		// dataset:
		//		The Client Nav Model Dataset that the pluggable view should render.
		// description:
		//		Tells the pluggable view it's time to process the new objects
		// tags:
		//		protected
		
		this._setObjectsLoadDataActive = true;
		if (this._setObjectsLoadDataCompletesSynchronously) {
			this._onSetObjectsLoadComplete(items, links, relationships);
		}
	},
	
	_onSetObjectsLoadComplete: function(/*ibm.tivoli.navmodel.Item[]*/ items, /*ibm.tivoli.navmodel.Link[]*/ links, /*ibm.tivoli.navmodel.Relationship[]*/ relationships){ 
		// summary:
		//		Event fired when an the load of the dataset is completed (potentially asynchronously). It is recommended to use dojo.connect rather than overriding this method.
		// dataset: Dataset
		//		The client nav model dataset that this view is now rendering
		// tags:
		//		protected callback
		
		this._setObjectsLoadDataActive = false;
		this._setObjectsComplete(items, links, relationships);
	},
	
	_setObjectsComplete: function(/*ibm.tivoli.navmodel.Item[]*/ items, /*ibm.tivoli.navmodel.Link[]*/ links, /*ibm.tivoli.navmodel.Relationship[]*/ relationships){
		// summary:
		//		Called during the setObjects lifecycle as the last step. Subclasses will probably NOT need to override.
		// items:
		//		An array of items
		// links:
		//		An array of links
		// relationships:
		//		An array of relationships
		// description:
		//		Tells the pluggable view we're done setting up the new objects
		// tags:
		//		protected
		
		//Create an "inner class" to help us determine if all of the subviews have called back with onSetObjectsComplete
		var mainOnSetObjectsHandlerInnerClass = {
				view: this,
				callbackCounter: 0,
				
				//Should be invoked in the scope of mainOnSetObjectsHandlerInnerClass, so "this" refers to this inner class rather than the pluggable view
				onSetObjectsHandler: 
					function(items, links, relationships) {
						//Increment counter
						this.callbackCounter++;
						
						if (this.callbackCounter >= this.view.getPluggableViewCount()) {
							//Fire event
							dojo.hitch(this.view, this.view.onSetObjects, items, links, relationships).call();
						}
					}
		};
		var mainDataSetObjectsHandlerFunc = dojo.hitch( mainOnSetObjectsHandlerInnerClass, mainOnSetObjectsHandlerInnerClass.onSetObjectsHandler, items, links, relationships);
		
		//Notify the sub-views of the new objects (if they are object pluggable views)
		var iterator = this.getPluggableViewIterator();
		if (iterator.atEnd()) {
			//There are no pluggable views, so we let's complete the setDataset process
			mainDataSetObjectsHandlerFunc.call();
		} else {
			//We want to iterate through the pluggable views and listen for when they hit onSetDatasetComplete
			while (!iterator.atEnd()) {
				var view = iterator.get().value;

					//Provide an "inner class" to support a handler for the "onSetDataset" event that can remove itself as a listener when done
					var subHandlerInnerClass = {
						mainHandler: mainDataSetObjectsHandlerFunc,
						handle: null,
						
						//Should be invoked in the scope of subHandlerInnerClass, so "this" refers to this inner class
						handler: function(items, links, relationships){
								this.mainHandler.call();
								
							     //Disconnect the handler
							     dojo.disconnect(this.handle);
							}
					};
					var handle = dojo.connect(view, "onSetObjects", dojo.hitch(subHandlerInnerClass, subHandlerInnerClass.handler));
					subHandlerInnerClass.handle = handle;
						
					if (view.isInstanceOf(ibm.tivoli.tip.navwidget.dijit.NavWidgetObjectPluggableView) &&
							view.isApplicableForObjects(items, links, relationships)) {
						try {
							//Since views are pluggable... let's take some care so that a custom one doesn't take us out if it craps
							//out handling the new objects
							view.setObjects(items, links, relationships);
						} catch (ex) {
							if (this._trace.isError()) {
								this._trace.error("Cannot set objects on view = " + view.id + "; ex = " + ex.name + ": " + ex.message, "_setObjectsComplete");
							}
						}
					} else {
						//View not applicable, so let's clear out the view
						try {
							view.setObjects(null, null, null, null);
						} catch (ex) {
							if (this._trace.isError()) {
								this._trace.error("Unable to clear objects on view = " + view.id + "; ex = " + ex.name + ": " + ex.message, "_setObjectsComplete");
							}
						}
					}
			}
		}
	},

	_setDatasetLoadData: function(/*ibm.tivoli.navmodel.Dataset*/ dataset){
		// summary:
		//		Overriden to clear out any objects that may have previously been set with setObjects.
		// dataset:
		//		The Client Nav Model Dataset that the pluggable view should render.
		// description:
		//		Tells the pluggable view we're done setting up the dataset
		// tags:
		//		protected
		
		this.clearObjects();
		this.inherited(arguments)
	},
	
	clearObjects: function(){
		// summary:
		//		Overriden to clear out any objects that may have previously been set with setObjects.
		// dataset:
		//		The Client Nav Model Dataset that the pluggable view should render.
		// description:
		//		Tells the pluggable view we're done setting up the dataset
		// tags:
		//		protected
		
		this._items = null;
		this._links = null;
		this._relationships = null;

		//Clear out the views too
		var iterator = this.getPluggableViewIterator();
		while (!iterator.atEnd()) {
			var view = iterator.get().value;
			view.clearObjects();
		}
	},
	
	getItemCount: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
 		if (this._items) {
 			return this._items.length;
 		}
 	},
 	
 	getLinkCount: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
 		if (this._links) {
 			return this._links.length;
 		}
 	},
 	
 	getRelationshipCount: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
 		if (this._relationships) {
 			return this._relationships.length;
 		}
 	},
 	
 	
 	isMixedObjects: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
 		var objectTypeCount = 0;
 		
 		if (this.getItemCount() > 0) {
 			//We have items
 			objectTypeCount++;
 		}
 		if (this.getLinkCount() > 0) {
 			//We have items
 			objectTypeCount++;
 		}
 		if (this.getRelationshipCount() > 0) {
 			//We have items
 			objectTypeCount++;
 		}
 		return (objectTypeCount > 1);
 	},
 	
 	//*****************************************************************************************************
	// Events we fire
	//*****************************************************************************************************
	onSetObjects: function(/*ibm.tivoli.navmodel.Item[]*/ items, /*ibm.tivoli.navmodel.Link[]*/ links, /*ibm.tivoli.navmodel.Link[]*/ relationships){
		// summary:
		//		Event fired when the objects are set
		// dataset: Dataset
		//		The client nav model objects that this view is now rendering
		// tags:
		//		callback
		
		//Mark that we're done setting the objects
		this._setObjectsActive = false;
		
		//Delegate anything else to listeners
	}
});
});

},
'ibm/usmi/widgets/TaskPane':function(){
// wrapped by build app
define("ibm/usmi/widgets/TaskPane", ["dijit","dojo","dojox","dojo/require!dojo/hash,dijit/_WidgetBase,dijit/_Container,ibm/usmi/util/BaseWidget,ibm/usmi/widgets/TaskLink,ibm/usmi/widgets/TaskCategory,ibm/usmi/widgets/util/Constants"], function(dijit,dojo,dojox){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

/* define(["dojo/_base/kernel",
        "dojo/_base/declare",
        "dijit/_WidgetBase",
        "dijit/_Container",
 //       "dojo/_base/Deferred",
        "dojo/hash",
        // NextGen imports
        "ibm/usmi/widgets/TaskLink",
        "ibm/usmi/widgets/TaskCategory"],
    function (dojo, declare, _WidgetBase, _Container, //Deferred,
            TaskLink, TaskCategory) {

    return declare([_WidgetBase, _Container], {*/

dojo.provide("ibm.usmi.widgets.TaskPane");

dojo.require("dojo.hash");
dojo.require("dijit._WidgetBase");
dojo.require("dijit._Container");
dojo.require("ibm.usmi.util.BaseWidget");
dojo.require("ibm.usmi.widgets.TaskLink");
dojo.require("ibm.usmi.widgets.TaskCategory");
dojo.require("ibm.usmi.widgets.util.Constants");

dojo.declare("ibm.usmi.widgets.TaskPane",[dijit._WidgetBase, dijit._Container, ibm.usmi.util.BaseWidget],
{

	topic : "",
	subject : "",

	_deferredEvent : null,
	_categories : [],
	_links : [],
	_type : "",
	subscription : null,
	tabIndex : "0", // propagate tabIndex to TaskCategorys from declaration or invocation, default to zero

	postCreate : function () {
		this.inherited(arguments);
		this._deferredEvent = new dojo.Deferred();
	},

	startup : function () {
		this.inherited(arguments);
		this._categories = this.getChildren();
		this._links = [];
		var _this = this;
		dojo.forEach(this._categories, function (category) {
			_this._links = _this._links.concat(category.getChildren());
		});

		this.subscription = this.subscribe(ibm.usmi.widgets.util.Constants.TOPIC_HASH_CHANGE,
				this._hashChanged);
	},

	refresh : function () {
		this._categories = this.getChildren();
		this._links = [];
		var _this = this;
		dojo.forEach(this._categories, function (category) {
			_this._links = _this._links.concat(category.getChildren());
		});
	},

	addCategory : function (category) {
		var g;
		if (category instanceof ibm.usmi.widgets.TaskCategory) {
			g = dojo.mixin(category, {
				tabIndex : this.tabIndex
			});
		} else {
			g = new ibm.usmi.widgets.TaskCategory(dojo.mixin(category, {
				tabIndex : this.tabIndex
			}));
		}

		this.addChild(g);
	},

	updateCategories : function (updates, type, typeExt) {
		this._categories = this.getChildren();

		dojo.forEach(this._categories, function (category) {
			var visible = this._categoryContainsDevType(category, type, typeExt), src;

			category.set("visible", visible);
			// If category.category explicitly appears in updates and is not null, then use it,
			// else check whether category.sourceCategory was specified and use this one instead.
			src = updates[category.category];
			if (!src && category.sourceCategory) {
				src = updates[category.sourceCategory];
			}
			category.updateLinks(src);
			if (visible) {
				category.setNoTaskMessage();
			}
		}, this);

		if (this._deferredEvent && updates.general) {
			this._deferredEvent.callback(type);
		}
	},

	_categoryContainsDevType : function (category, type, typeExt) {
		// Allow to specify an extension for the type, e.g.
		// 'BlockOnly', 'FileOnly', 'BlockAndFile', so in the
		// devTypes or excludedDevTypes list you may specify,
		// for example, 'netapp' (meaning netapp in any configuration,
		// or 'netappBlockAndFile' (meaning netapp configured for
		// BlockAndFile data).

		type = type || "";
		var extendedType = type ? type + (typeExt || "") : "";

		if (category.devTypes) {
			return (dojo.indexOf(category.devTypes, type) > -1)
					|| (dojo.indexOf(category.devTypes, extendedType) > -1);
		}

		if (category.excludedDevTypes) {
			return (dojo.indexOf(category.excludedDevTypes, type) === -1)
					&& (dojo.indexOf(category.excludedDevTypes, extendedType) === -1);
		}

		return true;
	},

	setLink : function (/*ibm.usmi.widgets.TaskLink*/link) {
		if (link.getParent().get("visible")) {
			dojo.forEach(this._links, function (item) {
				item.select(false);
			});
			link.select(true);
		}
	},

	_hashChanged : function (/*dojo.hash*/hash) {
		dojo.forEach(this._links, function (link) {
			// if criteria exists, trying to render a related resource view, compare task id versus
			// hash releationship
			if (link.criteriaString) {
				if (this.getUrlHashParm(ibm.usmi.widgets.util.Constants.HASH_TASK_RELATIONSHIP) === link.taskId) {
					this._linkClicked(null, link);
				}
			}
			// if not no criteria string, compare task id versus hash task selection parm
			else {
				if (this.getUrlHashParm(ibm.usmi.widgets.util.Constants.HASH_TASK_SELECTION) === link.taskId) {
					this._linkClicked(null, link);
				}
			}
		}, this);
	},

	_linkClicked : function (event, link) {
		this.setLink(link);

		var category = link.getParent(), args = [
			category,
			link,
			this.subject
		],
		// the following publish should be deferred to "update" completed
		_this = this;

		if (this._deferredEvent) {
			this._deferredEvent.addCallback(function (extraInfo) {
				dojo.publish(_this.topic, (extraInfo ? args.concat([ extraInfo
				]) : args));
				_this._type = extraInfo ? extraInfo : "";
				// do not need to defer next time
				delete _this._deferredEvent;
			});
		} else {
			dojo.publish(this.topic, this._type ? args.concat([ this._type
			]) : args);
		}
		dojo.publish(this.topic + "/" + this.subject, args);

		// publish the label, so workarea can render title
		dojo.publish(ibm.usmi.widgets.util.Constants.TOPIC_TASK_TITLE_CHANGE, {
			resource : link.getResourceLabel(),
			view : link.getLabel(),
			loading : false
		});
		
		// if this was kicked off by a link selection, must have been a bookmark or direct call,
		// need to kick off retarget event
		if(!link.getWasClicked()) {
			link.fakeClick();
		}
		
		// reset variable
		link.resetWasClicked();
	},

	selectTask : function (/*String*/taskName) {
		var link, idx = 0;
		while ((idx < this._links.length)) {
			if (this._links[idx].link === taskName) {
				link = this._links[idx];
				break;
			}
		}
		if (link) {
			this._linkClicked(null, link);
		}
	},

	getCategory : function (/*String*/category) {
		var subset = dojo.filter(this.getChildren(), function (/*Widget*/child) {
			return child.isInstanceOf(ibm.usmi.widgets.TaskCategory) && child.category === category;
		}, this);
		return (subset && subset[0]) || null;
	},
	destroy : function () {
		if (this.subscription) {
			this.subscription.remove();
			this.subscription = null;
		}
		this.inherited(arguments);
	}
});
});

},
'gridx/modules/filter/Filter':function(){
define([
	"../../core/_Module",
	"../../core/model/extensions/ClientFilter",
	"../../core/model/extensions/Query",
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/lang"
], function(_Module, ClientFilter, Query, declare, array, lang){

	var module = declare(/*===== "gridx.modules.filter.Filter", =====*/_Module, {
		// summary:
		//		This module makes it possible for user to set arbitrary filter condition to grid.
		
		name: 'filter',
		
		modelExtensions: [ClientFilter, Query],
	
		getAPIPath: function(){
			return {
				filter: this
			};
		},

		constructor: function(){
			this.setFilter(this.arg('preCondition'), 1);
		},
	
		//Public---------------------------------------------------------
		serverMode: false,

		setupFilterQuery: function(obj){
			return obj;
		},
		
		setFilter: function(/* Function|null */checker, /* Boolean? */skipUpdateBody){
			// summary:
			//		Apply function *checker* as the filter condition to filter every row.
			// checker: Function|null|undefined (or anything that is invalid)
			//		A function only returning TRUE or FALSE. It is used to decide whether a row should survive.
			//		If it is null (or anything invalid here), then clear the current filter.
			// skipUpdateBody: Boolean
			//		Whether to immediately update grid UI after filtering.
			// return:
			//		undefined
			// throw:
			//		If *skipUpdateBody* is not TRUE, then must not throw, else, only allowed to throw the exceptions
			//		that are generated from the *checker* function.
			//		If *checker* is not a function, or null, should not throw.
			var t = this, m = t.model;
			if(checker != t._checker){
				t._checker = checker;
				if(t.arg('serverMode')){
					m.query(t.setupFilterQuery(checker && checker.expr));
				}else{
					m.filter(checker);
				}
				if(!skipUpdateBody){
					t.grid.body.refresh();
				}
			}
			//this.model.query({_filter: 'xxx'});//{_filter: 'xxx'});//checker.expr);
		},
	
		getFilter: function(){
			// summary:
			//		Return the current checker function.
			// return: Function|null|undefined (or anything that is invalid)
			//		The current checker function
			return this._checker;
		}
	});
	
	//Util
	function valueConvert(d, type, converter){
		if(lang.isFunction(converter)){
			d = converter(d);
		}
		switch(type){
			case 'number':
				return parseFloat(d, 10);
			case 'boolean':
				return !!d;
			case 'date':
				d = new Date(d);
				d.setHours(0);
				d.setMinutes(0);
				d.setSeconds(0);
				d.setMilliseconds(0);
				return d.getTime();
			case 'time':
				d = new Date(d);
				d.setDate(1);
				d.setMonth(0);
				d.setFullYear(2000);
				return d.getTime();
			default: //string
				return String(d);
		}
	}
	
	function wrap(checker, op, operands, options){
		if(lang.isArray(operands)){
			operands = array.map(operands, function(operand){
				return operand.expr;
			});
		}
		return lang.mixin(checker, {
			expr: lang.mixin({
				op: op,
				data: operands
			}, options || {})
		});
	}

	function stringOperator(args, expr1, expr2, caseSensitive, op){
		var str1 = String(expr1.apply(0, args)),
			str2 = String(expr2.apply(0, args));
		if(!caseSensitive){
			str1 = str1.toLowerCase();
			str2 = str2.toLowerCase();
		}
		return op(str1, str2);
	}

// Filter Expression:
//		A filter expression is just a function returning TRUE/FALSE.
//		Here provides a set of useful expression tools to help construct complicated filter expressions for grid.
//		These expressions are not bound to any specific grid instance, so they can be directly reused for many grids.
//
//		For example: 
//			var expr = and(
//					and(
//						startWith(column('colA', 'string'), value('123abc', 'string')),
//						greater(column('colB', 'number'), value(456, 'number'))
//					),
//					or(
//						lessEqual(column('colC', 'number'), value(89, 'number')),
//						not(
//							endWith(column('colD', 'string'), value('xyz', 'string'))
//						)
//					)
//				);
	
	return lang.mixin(module, {
		//Data
		column: function(/* String|Number */colId, /* String? */type, /* Function? */converter, /* Boolean? */useRawData){
			// summary:
			//		Used in filter condition expression to identify a grid column
			// colId: String|Number
			//		The id of the grid column, usually string.
			// type: String?
			//		The data type of the grid column. If omitted, default to "string".
			// converter: Function?
			//		A data type converter function, used when the default converting does not work.
			//		For date or time type, converting to a Date object is enough.
			// useRawData: Boolean?
			//		To filter the store data (raw data) or the grid data (formatted data).
			// return: Function
			//		A filter expression.
			// example:
			//		//Omitting namespace here.
			//		var c = column('ISO Date Column', 'date', function(data){
			//			return dojo.date.stamp.fromISOString(data);
			//		});
			//		//Create a filter expression to checker whether the cell data equals today's date.
			//		var expr = equal(c, value(new Date(), 'date'));
			type = String(type || 'string').toLowerCase();
			return wrap(function(row){
				return valueConvert(row[useRawData ? 'rawData' : 'data'][colId], type, converter);
			}, type, colId, {isCol: true});
		},
	
		value: function(v, type, converter){
			// summary:
			//		Used in filter condition expression to represent a pure value.
			// v: Anything
			//		The value to be used in the filter condition
			// type: String?
			//		The data type of the value. If omitted, this function tris to infer by itself using typeof.
			// converter: Function?
			//		A data type converter function, used when the default converting does not work.
			//		For date or time type, converting to a Date object is enough.
			// return: Function
			//		A filter expression
			type = String(type || typeof v).toLowerCase();
			v = valueConvert(v, type, converter);
			return wrap(function(){
				return v;
			}, type, v);
		},
		
		//Empty check
		isEmpty: function(expr, emptyValues){
			// summary:
			//		A filter condition operaton to check whether an expression's result is empty.
			// expr: Function
			//		A filter condition expression.
			// emptyValue: Array?
			//		An array of values to be regarded as empty. If omitted, default to ""(empty string), null and undefined.
			// return: Function
			//		A filter expression
			return wrap(function(){
				var v = expr.apply(0, arguments);
				if(emptyValues){
					return array.indexOf(emptyValues, v) >= 0;
				}else{
					return v === "" || v === null || v === undefined;
				}
			}, "isEmpty", [expr]);
		},
		
		//Logic operations
		and: function(/* filter expressions */){
			// summary:
			//		A filter operation to check whether all the filter expressions passed in have TRUE results.
			// return: Function
			//		A filter expression
			var parts = array.filter(arguments, function(arg){
				return lang.isFunction(arg);
			});
			return wrap(function(){
				var args = arguments;
				return array.every(parts, function(part){
					return part.apply(0, args);
				});
			}, "and", parts);
		},
		
		or: function(){
			// summary:
			//		A filter operation to check whether any of the filter expressions passed in have TRUE result.
			// return: Function
			//		A filter expression
			var parts = array.filter(arguments, function(arg){
				return lang.isFunction(arg);
			});
			return wrap(function(){
				var args = arguments;
				return array.some(parts, function(part){
					return part.apply(0, args);
				});
			}, "or", parts);
		},
		
		not: function(predicate){
			// summary:
			//		A filter operation to check whether the result of the *predicate* is FALSE.
			// predicate: Function
			//		A filter expression.
			// return: Function
			//		A filter expression.
			return wrap(function(){
				return !predicate.apply(0, arguments);
			}, "not", [predicate]);
		},
		
		//Compare operations
		equal: function(expr1, expr2){
			// summary:
			//		A filter operation to check whether the given two expressions have the same result.
			// expr1: Function
			//		A filter expression.
			// expr2: Function
			//		A filter expression.
			// return: Function
			//		A filter expression.
			return wrap(function(){
				return expr1.apply(0, arguments) === expr2.apply(0, arguments);
			}, "equal", [expr1, expr2]);
		},
		
		greater: function(expr1, expr2){
			// summary:
			//		A filter operation to check whether the result of *expr1* is greater than that of *expr2*.
			// expr1: Function
			//		A filter expression.
			// expr2: Function
			//		A filter expression.
			// return: Function
			//		A filter expression.
			return wrap(function(){
				return expr1.apply(0, arguments) > expr2.apply(0, arguments);
			}, "greater", [expr1, expr2]);
		},
		
		less: function(expr1, expr2){
			// summary:
			//		A filter operation to check whether the result of *expr1* is less than that of *expr2*.
			// expr1: Function
			//		A filter expression.
			// expr2: Function
			//		A filter expression.
			// return: Function
			//		A filter expression.
			return wrap(function(){
				return expr1.apply(0, arguments) < expr2.apply(0, arguments);
			}, "less", [expr1, expr2]);
		},
		
		greaterEqual: function(expr1, expr2){
			// summary:
			//		A filter operation to check whether the result of *expr1* is greater than or equal to that of *expr2*.
			// expr1: Function
			//		A filter expression.
			// expr2: Function
			//		A filter expression.
			// return: Function
			//		A filter expression.
			return wrap(function(){
				return expr1.apply(0, arguments) >= expr2.apply(0, arguments);
			}, "greaterEqual", [expr1, expr2]);
		},
		
		lessEqual: function(expr1, expr2){
			// summary:
			//		A filter operation to check whether the result of *expr1* is less than or equal to that of *expr2*.
			// expr1: Function
			//		A filter expression.
			// expr2: Function
			//		A filter expression.
			// return: Function
			//		A filter expression.
			return wrap(function(){
				return expr1.apply(0, arguments) <= expr2.apply(0, arguments);
			}, "lessEqual", [expr1, expr2]);
		},
		
		//String operations
		match: function(expr, regExpr){
			// summary:
			//		A filter operation to check whether the result of *expr* matches the given regular expression *regExpr*.
			// expr: Function
			//		A filter expression returning string value.
			// regExpr: RegEx
			//		A regular expression.
			// return: Function
			//		A filter expression.
			return wrap(function(){
				return String(expr.apply(0, arguments)).search(regExpr) >= 0;
			}, "match", [expr, {
				expr: {
					op: "regex",
					data: regExpr
				}
			}]);
		},
		
		contain: function(expr1, expr2, caseSensitive){
			// summary:
			//		A filter operation to check wheter the string result of *expr1* contains that of *expr2*.
			// expr1: Function
			//		A filter expression.
			// expr2: Function
			//		A filter expression.
			// return: Function
			//		A filter expression.
			return wrap(function(){
				return stringOperator(arguments, expr1, expr2, caseSensitive, function(str1, str2){
					return str1.indexOf(str2) >= 0;
				});
			}, "contain", [expr1, expr2]);
		},
		
		startWith: function(expr1, expr2, caseSensitive){
			// summary:
			//		A filter operation to check wheter the string result of *expr1* starts with that of *expr2*.
			// expr1: Function
			//		A filter expression.
			// expr2: Function
			//		A filter expression.
			// return: Function
			//		A filter expression.
			return wrap(function(){
				return stringOperator(arguments, expr1, expr2, caseSensitive, function(str1, str2){
					return str1.substring(0, str2.length) === str2;
				});
			}, "startWith", [expr1, expr2]);
		},
		
		endWith: function(expr1, expr2, caseSensitive){
			// summary:
			//		A filter operation to check wheter the string result of *expr1* ends with that of *expr2*.
			// expr1: Function
			//		A filter expression.
			// expr2: Function
			//		A filter expression.
			// return: Function
			//		A filter expression.
			return wrap(function(){
				return stringOperator(arguments, expr1, expr2, caseSensitive, function(str1, str2){
					return str1.substring(str1.length - str2.length) === str2;
				});
			}, "endWith", [expr1, expr2]);
		}
	});
});

},
'dojox/grid/LazyTreeGrid':function(){
define([
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/event",
	"dojo/_base/array",
	"dojo/query",
	"dojo/parser",
	"dojo/dom-construct",
	"dojo/dom-class",
	"dojo/dom-style",
	"dojo/dom-geometry",
	"dojo/dom",
	"dojo/keys",	
	"dojo/text!./resources/Expando.html",
	"dijit/_Widget",
	"dijit/_TemplatedMixin",
	"./TreeGrid",
	"./_Builder",
	"./_View",
	"./_Layout",
	"./cells/tree",
	"./_RowManager",
	"./_FocusManager",
	"./_EditManager",
	"./DataSelection",
	"./util"
], function(dojo, declare, lang, event, array, query, parser, domConstruct,
	domClass, domStyle, domGeometry, dom, keys, template, _widget, _templatedMixin,
	TreeGrid, _Builder, _View, _Layout, TreeCell, _RowManager, _FocusManager, _EditManager, DataSelection, util){
		
var _LazyExpando = declare("dojox.grid._LazyExpando", [_widget, _templatedMixin], {
	grid: null,
	view: null,
	rowIdx: -1,
	cellIdx: -1,
	level: 0,
	itemId: "",
	templateString: template,
	onToggle: function(evt){
		// summary:
		//		The onclick handler of expando, expand/collapse a tree node if has children.
		if(this.grid._treeCache.items[this.rowIdx]){
			this.grid.focus.setFocusIndex(this.rowIdx, this.cellIdx);
			this.setOpen(!this.grid._treeCache.items[this.rowIdx].opened);
			try{
				event.stop(evt);
			}catch(e){}
		}
	},
	setOpen: function(open){
		// summary:
		//		expand/collapse the row where the expando is in.
		var g = this.grid,
			item = g._by_idx[this.rowIdx].item;
		if(item && g.treeModel.mayHaveChildren(item) && !g._loading && g._treeCache.items[this.rowIdx].opened !== open){
			g._treeCache.items[this.rowIdx].opened = open;
			g.expandoFetch(this.rowIdx, open);
			this._updateOpenState(item);
		}
	},
	_updateOpenState: function(item){
		var g = this.grid, state;
		if(item && g.treeModel.mayHaveChildren(item)){
			state = g._treeCache.items[this.rowIdx].opened;
			this.expandoInner.innerHTML = state ? "-" : "+";
			domClass.toggle(this.domNode, "dojoxGridExpandoOpened", state);
			this.domNode.parentNode.parentNode.setAttribute("aria-expanded", state);
		}else{
			domClass.remove(this.domNode, "dojoxGridExpandoOpened");
		}
	},
	setRowNode: function(rowIdx, rowNode, view){
		if(this.cellIdx < 0 || !this.itemId){
			return false;
		}
		this.view = view;
		this.grid = view.grid;
		this.rowIdx = rowIdx;
		var marginPos = this.grid.isLeftToRight() ? "marginLeft" : "marginRight";
		domStyle.set(this.domNode.parentNode, marginPos, (this.level * 1.125) + "em");
		this._updateOpenState(this.grid._by_idx[this.rowIdx].item);
		return true;
	}
});

var _TreeGridContentBuilder = declare("dojox.grid._TreeGridContentBuilder", _Builder._ContentBuilder, {
	generateHtml: function(inDataIndex, rowIndex){
		var html = this.getTableArray(),
			grid = this.grid,
			v = this.view,
			cells = v.structure.cells,
			item = grid.getItem(rowIndex),
			level = 0,
			toggleClass = "",
			treePath = grid._treeCache.items[rowIndex] ? grid._treeCache.items[rowIndex].treePath : null;
		util.fire(this.view, "onBeforeRow", [rowIndex, cells]);
		if(item && lang.isArray(treePath)){
			level = treePath.length;
			toggleClass = grid.treeModel.mayHaveChildren(item) ? "" : "dojoxGridNoChildren";
		}
		var i = 0, j = 0, row, cell,
			mergedCells, totalWidth = 0, totalWidthes = [];
		for(; row = cells[j]; j++){
			if(row.hidden || row.header){
				continue;
			}
			html.push('<tr class="' + toggleClass + '">');
			// cell merge
			mergedCells = this._getColSpans(level);
			if(mergedCells){
				array.forEach(mergedCells, function(c){
					for(i = 0; cell = row[i]; i++){
						if(i >= c.start && i <= c.end){
							totalWidth += this._getCellWidth(row, i);
						}
					}
					totalWidthes.push(totalWidth);
					totalWidth = 0;
				}, this);
			}
			var m, cc, cs, pbm, k = 0;
			for(i = 0; cell = row[i]; i++){
				m = cell.markup;
				cc = cell.customClasses = [];
				cs = cell.customStyles = [];
				if(mergedCells && mergedCells[k] && (i >= mergedCells[k].start && i <= mergedCells[k].end)){
					var primaryIdx = mergedCells[k].primary || mergedCells[k].start;
					if(i == primaryIdx){
						m[5] = cell.formatAtLevel(item, level, rowIndex);
						m[1] = cc.join(' ');
						pbm = domGeometry.getMarginBox(cell.getHeaderNode()).w - domGeometry.getContentBox(cell.getHeaderNode()).w;
						cs = cell.customStyles = ['width:' + (totalWidthes[k] - pbm) + "px"];
						m[3] = cs.join(';');
						html.push.apply(html, m);
					}else if(i == mergedCells[k].end){
						k++;
						continue;
					}else{
						continue;
					}
				}else{
					m[5] = cell.formatAtLevel(item, level, rowIndex);
					m[1] = cc.join(' ');
					m[3] = cs.join(';');
					html.push.apply(html, m);
				}
			}
			html.push('</tr>');
		}
		html.push('</table>');
		return html.join(''); // String
	},
	_getColSpans: function(level){
		var colSpans = this.grid.colSpans;
		return colSpans && colSpans[level] ? colSpans[level] : null;
	},
	_getCellWidth: function(cells, colIndex){
		var curCell = cells[colIndex], node = curCell.getHeaderNode();
		if(curCell.hidden){
			return 0;
		}
		if(colIndex == cells.length - 1 || array.every(cells.slice(colIndex + 1), function(cell){
			return cell.hidden;
		})){
			var headerNodePos = domGeometry.position(cells[colIndex].view.headerContentNode.firstChild);
			return headerNodePos.x + headerNodePos.w - domGeometry.position(node).x;
		}else{
			var nextCell;
			do{
				nextCell = cells[++colIndex];
			}while(nextCell.hidden);
			return domGeometry.position(nextCell.getHeaderNode()).x - domGeometry.position(node).x;
		}
	}
});

declare("dojox.grid._TreeGridView", _View, {
	_contentBuilderClass: _TreeGridContentBuilder,
	postCreate: function(){
		this.inherited(arguments);
		this._expandos = {};
		this.connect(this.grid, '_onCleanupExpandoCache', '_cleanupExpandoCache');
	},
	destroy: function(){
		this._cleanupExpandoCache();
		this.inherited(arguments);
	},
	_cleanupExpandoCache: function(identity){
		if(identity && this._expandos[identity]){
			this._expandos[identity].destroy();
			delete this._expandos[identity];
		}else{
			var i;
			for(i in this._expandos){
				this._expandos[i].destroy();
			}
			this._expandos = {};
		}
	},
	onAfterRow: function(rowIndex, cells, rowNode){
		query("span.dojoxGridExpando", rowNode).forEach(function(n){
			if(n && n.parentNode){
				var idty, expando, _byIdx = this.grid._by_idx;
				if(_byIdx && _byIdx[rowIndex] && _byIdx[rowIndex].idty){
					idty = _byIdx[rowIndex].idty;
					expando = this._expandos[idty];
				}
				if(expando){
					domConstruct.place(expando.domNode, n, "replace");
					expando.itemId = n.getAttribute("itemId");
					expando.cellIdx = parseInt(n.getAttribute("cellIdx"), 10);
					if(isNaN(expando.cellIdx)){
						expando.cellIdx = -1;
					}
				}else{
					expando = parser.parse(n.parentNode)[0];
					if(idty){
						this._expandos[idty] = expando;
					}
				}
				if(!expando.setRowNode(rowIndex, rowNode, this)){
					expando.domNode.parentNode.removeChild(expando.domNode);
				}
				domConstruct.destroy(n);
			}
		}, this);
		this.inherited(arguments);
	},
	updateRow: function(rowIndex){
		var grid = this.grid, item;
		if(grid.keepSelection){
			item = grid.getItem(rowIndex);
			if(item){
				grid.selection.preserver._reSelectById(item, rowIndex);
			}
		}
		this.inherited(arguments);
	}
});

var LazyTreeCell = lang.mixin(lang.clone(TreeCell), {
	formatAtLevel: function(item, level, rowIndex){
		if(!item){
			return this.formatIndexes(rowIndex, item, level);
		}
		var result = "", ret = "", content;
		if(this.isCollapsable && this.grid.store.isItem(item)){
			ret = '<span ' + dojo._scopeName + 'Type="dojox.grid._LazyExpando" level="' + level + '" class="dojoxGridExpando"' +
					' itemId="' + this.grid.store.getIdentity(item) + '" cellIdx="' + this.index + '"></span>';
		}
		content = this.formatIndexes(rowIndex, item, level);
		result = ret !== "" ? '<div>' + ret + content + '</div>' : content;
		return result;
	},
	formatIndexes: function(rowIndex, item, level){
		var info = this.grid.edit.info,
			d = this.get ? this.get(rowIndex, item) : (this.value || this.defaultValue);
		if(this.editable && (this.alwaysEditing || (info.rowIndex === rowIndex && info.cell === this))){
			return this.formatEditing(d, rowIndex);
		}else{
			return this._defaultFormat(d, [d, rowIndex, level, this]);
		}
	}
});

var _LazyTreeLayout = declare("dojox.grid._LazyTreeLayout", _Layout, {
	// summary:
	//		Override the dojox.grid._TreeLayout to modify the _TreeGridView and cell formatter
	setStructure: function(structure){
		var g = this.grid, s = structure;
		if(g && !array.every(s, function(i){
			return !!i.cells;
		})){
			s = arguments[0] = [{cells:[s]}];//intentionally change arguments[0]
		}
		if(s.length === 1 && s[0].cells.length === 1){
			s[0].type = "dojox.grid._TreeGridView";
			this._isCollapsable = true;
			s[0].cells[0][this.grid.expandoCell].isCollapsable = true;
		}
		this.inherited(arguments);
	},
	addCellDef: function(rowIndex, cellIndex, def){
		var obj = this.inherited(arguments);
		return lang.mixin(obj, LazyTreeCell);
	}
});

var _LazyTreeGridCache = declare("dojox.grid._LazyTreeGridCache", null, {
	// summary:
	//		An internal object used to cache the tree path and open state of each item.
	//		The form of the cache items would be an object array: 
	//		[{opened: true/false, treePath: [level0 parent id, level1 parent id, ...]}]
	// example:
	//		| [{opened: true, treePath: []},
	//		|  {opened: false, treePath: ["root0"]},
	//		|  {opened: false, treePath: ["root0"]},
	//		|  {opened: false, treePath: []},
	//		|  ...]
	constructor: function(){
		this.items = [];
	},
	getSiblingIndex: function(rowIndex, treePath){
		var i = rowIndex - 1, indexCount = 0, tp;
		for(; i >=0; i--){
			tp = this.items[i] ? this.items[i].treePath : [];
			if(tp.join('/') === treePath.join('/')){
				indexCount++;
			}else if(tp.length < treePath.length){
				break;
			}
		}
		return indexCount;
	},
	removeChildren: function(rowIndex){
		// find next sibling index
		var i = rowIndex + 1, count, tp,
			treePath = this.items[rowIndex] ? this.items[rowIndex].treePath : [];
		for(; i < this.items.length; i++){
			tp = this.items[i] ? this.items[i].treePath : [];
			if(tp.join('/') === treePath.join('/') || tp.length <= treePath.length){
				break;
			}
		}
		count = i - (rowIndex + 1);
		this.items.splice(rowIndex + 1, count);
		return count;
	}
});

var LazyTreeGrid = declare("dojox.grid.LazyTreeGrid", TreeGrid, {
	// summary:
	//		An enhanced TreeGrid widget which supports lazy-loading for nested children items
	//
	// description:
	//		LazyTreeGrid inherits from dojo.grid.TreeGrid and applies virtual scrolling mechanism
	//		to nested children rows so that it's possible to deal with complex tree structure data set
	//		with nested and huge children rows. It's also compatible with dijit.tree.ForestStoreModel
	//
	//		Most methods and properties pertaining to dojox.grid.DataGrid
	//		and dojox.grid.TreeGrid also apply here
	//
	//		LazyTreeGrid does not support summary row/items aggregate due to the lazy-loading rationale.
	_layoutClass: _LazyTreeLayout,
	_size: 0,
	// treeModel: dijit.tree.ForestStoreModel | dojox.grid.LazyTreeGridStoreModel
	//		A tree store model object.
	treeModel: null,
	// defaultState: Object
	//		Used to restore the state of LazyTreeGrid.
	//		This object should ONLY be obtained from `LazyTreeGrid.getState()`.
	defaultState: null,
	// colSpans: Object
	//		a json object that defines column span of each level rows
	//		attributes:
	//			0/1/..: which level need to colspan
	//			start: start column index of colspan
	//			end: end column index of colspan
	//			primary: index of column which content will be displayed (default is value of start).
	//		example:
	//		|	colSpans = {
	//		|	0:	[
	//		|			{start: 0, end: 1, primary: 0},
	//		|			{start: 2, end: 4, primary: 3}
	//		|		],
	//		|	1:	[
	//		|			{start: 0, end: 3, primary: 1}
	//		|		]
	//		|	};
	colSpans: null,
	
	postCreate: function(){
		this._setState();
		this.inherited(arguments);
		if(this.defaultState){
			if(this.keepSelection){
				this.selection.preserver._selectedById = this.defaultState.selection;
			}else{
				this.selection.selected = this.defaultState.selection || [];
			}
		}
		if(!this._treeCache){
			this._treeCache = new _LazyTreeGridCache();
		}
		if(!this.treeModel || !(this.treeModel instanceof dijit.tree.ForestStoreModel)){
			throw new Error("dojox.grid.LazyTreeGrid: must be used with a treeModel which is an instance of dijit.tree.ForestStoreModel");
		}
		domClass.add(this.domNode, "dojoxGridTreeModel");
		dom.setSelectable(this.domNode, this.selectable);
	},
	createManagers: function(){
		this.rows = new _RowManager(this);
		this.focus = new _FocusManager(this);
		this.edit = new _EditManager(this);
	},
	createSelection: function(){
		this.selection = new DataSelection(this);
	},
	setModel: function(treeModel){
		if(!treeModel){
			return;
		}
		this._setModel(treeModel);
		this._cleanup();
		this._refresh(true);
	},
	setStore: function(store, query, queryOptions){
		if(!store){
			return;
		}
		this._setQuery(query, queryOptions);
		this.treeModel.query = query;
		this.treeModel.store = store;
		this.treeModel.root.children = [];
		this.setModel(this.treeModel);
	},
	onSetState: function(){
		// summary:
		//		Event fired when a default state being set.
	},
	_setState: function(){
		if(this.defaultState){
			var cache = this._treeCache = this.defaultState.cache;
			if(cache){
				//Honor this cached size so that it won't be reset in onBegin event
				this._size = cache.items.length;
			}
			this.sortInfo = this.defaultState.sortInfo || 0;
			this.query = this.defaultState.query || this.query;
			this._lastScrollTop = this.defaultState.scrollTop;
			this.onSetState();
		}
	},
	getState: function(){
		// summary:
		//		Get the current state of LazyTreeGrid including expanding, sorting, selection and scroll top state.
		var _this = this, 
			selection = this.keepSelection ? this.selection.preserver._selectedById : this.selection.selected;
		return {
			cache: lang.clone(_this._treeCache),
			query: lang.clone(_this.query),
			sortInfo: lang.clone(_this.sortInfo),
			scrollTop: lang.clone(_this.scrollTop),
			selection: lang.clone(selection)
		};
	},
	_setQuery: function(query, queryOptions){
		this.inherited(arguments);
		this.treeModel.query = query;
	},
	canSort: function(){
		return (!this._loading);
	},
	filter: function(query, reRender){
		this._cleanup();
		this.inherited(arguments);
    },
	destroy: function(){
		this._cleanup();
		this.inherited(arguments);
	},
	expand: function(itemId){
		//	summary:
		//		Expand the row with the given itemId.
		//	id: string?
		this._fold(itemId, true);
	},
	collapse: function(itemId){
		//	summary:
		//		Collapse the row with the given itemId.
		//	id: string?
		this._fold(itemId, false);
	},
	refresh: function(keepState){
		//	summary:
		//		Refresh, and persist the expand/collapse state when keepState equals true
		//	keepState: boolean
		if(!keepState){
			this._cleanup();
		}
		this._refresh(true);
	},
	_cleanup: function(){
		this._treeCache.items = [];
		this._onCleanupExpandoCache();
	},
	setSortIndex: function(inIndex, inAsc){
		// Need to clean up the cache before sorting
		if(this.canSort(inIndex + 1)){
			this._cleanup();
		}
		this.inherited(arguments);
	},
	_refresh: function(isRender){
		this._clearData();
		this.updateRowCount(this._size);
		this._fetch(0, true);
	},
	render: function(){
		this.inherited(arguments);
		this.setScrollTop(this.scrollTop);
	},
	_onNew: function(item, parentInfo){
		var addingChild = parentInfo && this.store.isItem(parentInfo.item) && array.some(this.treeModel.childrenAttrs, function(c){
			return c === parentInfo.attribute;
		});
		var items = this._treeCache.items, byIdx = this._by_idx;
		if(!addingChild){
			items.push({opened: false, treePath: []});
			this._size += 1;
			this.inherited(arguments);
		}else{
			var parentItem = parentInfo.item, 
				parentIdty = this.store.getIdentity(parentItem),
				rowIndex = -1, i = 0;
			for(; i < byIdx.length; i++){
				if(parentIdty === byIdx[i].idty){
					rowIndex = i;
					break;
				}
			}
			if(rowIndex >= 0){
				if(items[rowIndex] && items[rowIndex].opened){
					var parentTreePath = items[rowIndex].treePath, pos = rowIndex + 1;
					for(; pos < items.length; pos++){
						if(items[pos].treePath.length <= parentTreePath.length){
							break;
						}
					}
					var treePath = parentTreePath.slice();
					treePath.push(parentIdty);
					this._treeCache.items.splice(pos, 0, {opened: false, treePath: treePath});
					// update grid._by_idx
					var idty = this.store.getIdentity(item);
					this._by_idty[idty] = { idty: idty, item: item };
					byIdx.splice(pos, 0, this._by_idty[idty]);
					// update grid
					this._size += 1;
					this.updateRowCount(this._size);
					this._updateRenderedRows(pos);
				}else{
					this.updateRow(rowIndex);
				}
			}
		}
	},
	_onDelete: function(item){
		var i = 0, rowIndex = -1, idty = this.store.getIdentity(item);
		for(; i < this._by_idx.length; i++){
			if(idty === this._by_idx[i].idty){
				rowIndex = i;
				break;
			}
		}
		if(rowIndex >= 0){
			var items = this._treeCache.items, treePath = items[rowIndex] ? items[rowIndex].treePath : [], tp, count = 1;
			i = rowIndex + 1;
			for(; i < this._size; i++, count++){
				tp = items[i] ? items[i].treePath : [];
				if(items[i].treePath.length <= treePath.length){
					break;
				}
			}
			items.splice(rowIndex, count);
			this._onCleanupExpandoCache(idty);
			this._by_idx.splice(rowIndex, count);
			this._size -= count;
			this.updateRowCount(this._size);
			this._updateRenderedRows(rowIndex);
		}
	},
	_onCleanupExpandoCache: function(identity){},
	_fetch: function(start, isRender){
		if(!this._loading){
			this._loading = true;
		}
		start = start || 0;
		var count = this._size - start > 0 ? Math.min(this.rowsPerPage, this._size - start) : this.rowsPerPage;
		var i = 0;
		var fetchedItems = [];
		this._reqQueueLen = 0;
		for(; i < count; i++){
			if(this._by_idx[start + i]){
				fetchedItems.push(this._by_idx[start + i].item);
			}else{
				break;
			}
		}
		if(fetchedItems.length === count){
			this._reqQueueLen = 1;
			this._onFetchBegin(this._size, {startRowIdx: start, count: count});
			this._onFetchComplete(fetchedItems, {startRowIdx: start, count: count});
		}else{
			var level, nextLevel, len = 1, items = this._treeCache.items, 
				treePath = items[start] ? items[start].treePath : [];
			for(i = 1; i < count; i++){
				level = items[start + len - 1] ? items[start + len - 1].treePath.length : 0;
				nextLevel = items[start + len] ? items[start + len].treePath.length : 0;
				if(level !== nextLevel){
					this._reqQueueLen++;
					this._fetchItems({startRowIdx: start, count: len, treePath: treePath});
					start = start + len;
					len = 1;
					treePath = items[start] ? items[start].treePath : 0;
				}else{
					len++;
				}
			}
			this._reqQueueLen++;
			this._fetchItems({startRowIdx: start, count: len, treePath: treePath});
		}
	},
	_fetchItems: function(req){
		if(this._pending_requests[req.startRowIdx]){
			return;
		}
		this.showMessage(this.loadingMessage);
		this._pending_requests[req.startRowIdx] = true;
		var	onError = lang.hitch(this, '_onFetchError'),
			start = this._treeCache.getSiblingIndex(req.startRowIdx, req.treePath);
		if(req.treePath.length === 0){
			this.store.fetch({
				start: start,
				startRowIdx: req.startRowIdx,
				treePath: req.treePath,
				count: req.count,
				query: this.query,
				sort: this.getSortProps(),
				queryOptions: this.queryOptions,
				onBegin: lang.hitch(this, '_onFetchBegin'),
				onComplete: lang.hitch(this, '_onFetchComplete'),
				onError: lang.hitch(this, '_onFetchError')
			});
		}else{
			var parentId = req.treePath[req.treePath.length - 1], parentItem;
			var queryObj = {
				start: start,
				startRowIdx: req.startRowIdx,
				treePath: req.treePath,
				count: req.count,
				parentId: parentId,
				sort: this.getSortProps()
			};
			var _this = this;
			var onComplete = function(){
				var f = lang.hitch(_this, '_onFetchComplete');
				if(arguments.length == 1){
					f.apply(_this, [arguments[0], queryObj]);
				}else{
					f.apply(_this, arguments);
				}
			};
			if(this._by_idty[parentId]){
				parentItem = this._by_idty[parentId].item;
				this.treeModel.getChildren(parentItem, onComplete, onError, queryObj);
			}else{
				this.store.fetchItemByIdentity({
					identity: parentId,
					onItem: function(item){
						_this.treeModel.getChildren(item, onComplete, onError, queryObj);
					},
					onError: onError
				});
			}
		}
	},
	_onFetchBegin: function(size, request){
		if(this._treeCache.items.length === 0){
			this._size = parseInt(size, 10);
		}
		size = this._size;
		// this._size = size = this._treeCache.items.length;
		this.inherited(arguments);
	},
	_onFetchComplete: function(items, request){
		var startRowIdx = request.startRowIdx,
			count = request.count,
			start = items.length <= count ? 0: request.start,
			treePath = request.treePath || [];
		if(lang.isArray(items) && items.length > 0){
			var i = 0, len = Math.min(count, items.length);
			for(; i < len; i++){
				if(!this._treeCache.items[startRowIdx + i]){
					this._treeCache.items[startRowIdx + i] = {opened: false, treePath: treePath};
				}
				if(!this._by_idx[startRowIdx + i]){
					this._addItem(items[start + i], startRowIdx + i, true, treePath.length == 0);
				}
				// this._treeCache.items.splice(startRowIdx + i, 0, {opened: false, treePath: treePath});
			}
			this.updateRows(startRowIdx, len);
		}
		if(this._size == 0){
			this.showMessage(this.noDataMessage);
		}else{
			this.showMessage();
		}
		this._pending_requests[startRowIdx] = false;
		this._reqQueueLen--;
		if(this._loading && this._reqQueueLen === 0){
			this._loading = false;
			if(this._lastScrollTop){
				this.setScrollTop(this._lastScrollTop);
			}
		}
	},
	expandoFetch: function(rowIndex, open){
		// summary:
		//		Function for fetch children of a given row
		if(this._loading || !this._by_idx[rowIndex]){return;}
		this._loading = true;
		this._toggleLoadingClass(rowIndex, true);
		this.expandoRowIndex = rowIndex;
		var item = this._by_idx[rowIndex].item;
		//This is necessary because expanding node changes page structure, so reset this to trigger refetch.
		this._pages = [];
		if(open){
			var queryObj = {
				start: 0,
				count: this.rowsPerPage,
				parentId: this.store.getIdentity(this._by_idx[rowIndex].item),
				sort: this.getSortProps()
			};
			this.treeModel.getChildren(item, lang.hitch(this, "_onExpandoComplete"), lang.hitch(this, "_onFetchError"), queryObj);
		}else{
			// get the whole children number when clear the children from cache
			var num = this._treeCache.removeChildren(rowIndex);
			// remove the items from grid._by_idx
			this._by_idx.splice(rowIndex + 1, num);
			this._bop = this._eop = -1;
			//update grid
			this._size -= num;
			this.updateRowCount(this._size);
			this._updateRenderedRows(rowIndex + 1);
			this._toggleLoadingClass(rowIndex, false);
			if(this._loading){
				this._loading = false;
			}
			this.focus._delayedCellFocus();
		}
	},
	_onExpandoComplete: function(childItems, request, size){
		size = isNaN(size) ? childItems.length : parseInt(size, 10);
		var treePath = this._treeCache.items[this.expandoRowIndex].treePath.slice(0);
		treePath.push(this.store.getIdentity(this._by_idx[this.expandoRowIndex].item));
		var i = 1, idty;
		for(; i <= size; i++){
			this._treeCache.items.splice(this.expandoRowIndex + i, 0, {treePath: treePath, opened: false});
		}
		this._size += size;
		this.updateRowCount(this._size);
		for(i = 0; i < size; i++){
			if(childItems[i]){
				idty = this.store.getIdentity(childItems[i]);
				this._by_idty[idty] = { idty: idty, item: childItems[i] };
				this._by_idx.splice(this.expandoRowIndex + 1 + i, 0, this._by_idty[idty]);
			}else{
				this._by_idx.splice(this.expandoRowIndex + 1 + i, 0, null);
			}
		}
		this._updateRenderedRows(this.expandoRowIndex + 1);
		this._toggleLoadingClass(this.expandoRowIndex, false);
		this.stateChangeNode = null;
		if(this._loading){
			this._loading = false;
		}
		if(this.autoHeight === true){
			this._resize();
		}
		this.focus._delayedCellFocus();
	},
	styleRowNode: function(rowIndex, rowNode){
		if(rowNode){
			this.rows.styleRowNode(rowIndex, rowNode);
		}
	},
	onStyleRow: function(row){
		if(!this.layout._isCollapsable){
			this.inherited(arguments);
			return;
		}
		row.customClasses = (row.odd ? " dojoxGridRowOdd" : "") + (row.selected ? " dojoxGridRowSelected" : "") + (row.over ? " dojoxGridRowOver" : "");
		this.focus.styleRow(row);
		this.edit.styleRow(row);
	},
	onKeyDown: function(e){
		if(e.altKey || e.metaKey){
			return;
		}
		var expando = dijit.findWidgets(e.target)[0];
		if(e.keyCode === keys.ENTER && expando instanceof _LazyExpando){
			expando.onToggle();
		}
		this.inherited(arguments);
	},
	_toggleLoadingClass: function(rowIndex, flag){
		var views = this.views.views, node,
			rowNode = views[views.length - 1].getRowNode(rowIndex);
		if(rowNode){
			node = query('.dojoxGridExpando', rowNode)[0];
			if(node){
				domClass.toggle(node, "dojoxGridExpandoLoading", flag);
			}
		}
	},
	_updateRenderedRows: function(start){
		var rowsPerPage = this.scroller.rowsPerPage;
		array.forEach(this.scroller.stack, function(p){
			if(p * rowsPerPage >= start){
				this.updateRows(p * rowsPerPage, rowsPerPage);
			}else if((p + 1) * rowsPerPage >=  start){
				this.updateRows(start, (p + 1) * rowsPerPage - start + 1);
			}
		}, this);
	},
	_fold: function(itemId, open){
		var rowIndex = -1, i = 0, byIdx = this._by_idx, idty = this._by_idty[itemId];
		if(idty && idty.item && this.treeModel.mayHaveChildren(idty.item)){
			for(; i < byIdx.length; i++){
				if(byIdx[i] && byIdx[i].idty === itemId){
					rowIndex = i;
					break;
				}
			}
			if(rowIndex >= 0){
				var rowNode = this.views.views[this.views.views.length - 1].getRowNode(rowIndex);
				if(rowNode){
					var expando = dijit.findWidgets(rowNode)[0];
					if(expando){
						expando.setOpen(open);
					}
				}
			}
		}
	}
});

LazyTreeGrid.markupFactory = function(props, node, ctor, cellFunc){
	return TreeGrid.markupFactory(props, node, ctor, cellFunc);
};

return LazyTreeGrid;

});

},
'dijit/_base/wai':function(){
define([
	"dojo/dom-attr", // domAttr.attr
	"dojo/_base/lang", // lang.mixin
	"..",	// export symbols to dijit
	"../hccss"			// not using this module directly, but loading it sets CSS flag on <html>
], function(domAttr, lang, dijit){

	// module:
	//		dijit/_base/wai
	// summary:
	//		Deprecated methods for setting/getting wai roles and states.
	//		New code should call setAttribute()/getAttribute() directly.
	//
	//		Also loads hccss to apply dijit_a11y class to root node if machine is in high-contrast mode.

	lang.mixin(dijit, {
		hasWaiRole: function(/*Element*/ elem, /*String?*/ role){
			// summary:
			//		Determines if an element has a particular role.
			// returns:
			//		True if elem has the specific role attribute and false if not.
			// 		For backwards compatibility if role parameter not provided,
			// 		returns true if has a role
			var waiRole = this.getWaiRole(elem);
			return role ? (waiRole.indexOf(role) > -1) : (waiRole.length > 0);
		},

		getWaiRole: function(/*Element*/ elem){
			// summary:
			//		Gets the role for an element (which should be a wai role).
			// returns:
			//		The role of elem or an empty string if elem
			//		does not have a role.
			 return lang.trim((domAttr.get(elem, "role") || "").replace("wairole:",""));
		},

		setWaiRole: function(/*Element*/ elem, /*String*/ role){
			// summary:
			//		Sets the role on an element.
			// description:
			//		Replace existing role attribute with new role.

			domAttr.set(elem, "role", role);
		},

		removeWaiRole: function(/*Element*/ elem, /*String*/ role){
			// summary:
			//		Removes the specified role from an element.
			// 		Removes role attribute if no specific role provided (for backwards compat.)

			var roleValue = domAttr.get(elem, "role");
			if(!roleValue){ return; }
			if(role){
				var t = lang.trim((" " + roleValue + " ").replace(" " + role + " ", " "));
				domAttr.set(elem, "role", t);
			}else{
				elem.removeAttribute("role");
			}
		},

		hasWaiState: function(/*Element*/ elem, /*String*/ state){
			// summary:
			//		Determines if an element has a given state.
			// description:
			//		Checks for an attribute called "aria-"+state.
			// returns:
			//		true if elem has a value for the given state and
			//		false if it does not.

			return elem.hasAttribute ? elem.hasAttribute("aria-"+state) : !!elem.getAttribute("aria-"+state);
		},

		getWaiState: function(/*Element*/ elem, /*String*/ state){
			// summary:
			//		Gets the value of a state on an element.
			// description:
			//		Checks for an attribute called "aria-"+state.
			// returns:
			//		The value of the requested state on elem
			//		or an empty string if elem has no value for state.

			return elem.getAttribute("aria-"+state) || "";
		},

		setWaiState: function(/*Element*/ elem, /*String*/ state, /*String*/ value){
			// summary:
			//		Sets a state on an element.
			// description:
			//		Sets an attribute called "aria-"+state.

			elem.setAttribute("aria-"+state, value);
		},

		removeWaiState: function(/*Element*/ elem, /*String*/ state){
			// summary:
			//		Removes a state from an element.
			// description:
			//		Sets an attribute called "aria-"+state.

			elem.removeAttribute("aria-"+state);
		}
	});

	return dijit;
});

},
'dijit/form/_TextBoxMixin':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.byId
	"dojo/_base/event", // event.stop
	"dojo/keys", // keys.ALT keys.CAPS_LOCK keys.CTRL keys.META keys.SHIFT
	"dojo/_base/lang", // lang.mixin
	".."	// for exporting dijit._setSelectionRange, dijit.selectInputText
], function(array, declare, dom, event, keys, lang, dijit){

// module:
//		dijit/form/_TextBoxMixin
// summary:
//		A mixin for textbox form input widgets

var _TextBoxMixin = declare("dijit.form._TextBoxMixin", null, {
	// summary:
	//		A mixin for textbox form input widgets

	// trim: Boolean
	//		Removes leading and trailing whitespace if true.  Default is false.
	trim: false,

	// uppercase: Boolean
	//		Converts all characters to uppercase if true.  Default is false.
	uppercase: false,

	// lowercase: Boolean
	//		Converts all characters to lowercase if true.  Default is false.
	lowercase: false,

	// propercase: Boolean
	//		Converts the first character of each word to uppercase if true.
	propercase: false,

	// maxLength: String
	//		HTML INPUT tag maxLength declaration.
	maxLength: "",

	// selectOnClick: [const] Boolean
	//		If true, all text will be selected when focused with mouse
	selectOnClick: false,

	// placeHolder: String
	//		Defines a hint to help users fill out the input field (as defined in HTML 5).
	//		This should only contain plain text (no html markup).
	placeHolder: "",

	_getValueAttr: function(){
		// summary:
		//		Hook so get('value') works as we like.
		// description:
		//		For `dijit.form.TextBox` this basically returns the value of the <input>.
		//
		//		For `dijit.form.MappedTextBox` subclasses, which have both
		//		a "displayed value" and a separate "submit value",
		//		This treats the "displayed value" as the master value, computing the
		//		submit value from it via this.parse().
		return this.parse(this.get('displayedValue'), this.constraints);
	},

	_setValueAttr: function(value, /*Boolean?*/ priorityChange, /*String?*/ formattedValue){
		// summary:
		//		Hook so set('value', ...) works.
		//
		// description:
		//		Sets the value of the widget to "value" which can be of
		//		any type as determined by the widget.
		//
		// value:
		//		The visual element value is also set to a corresponding,
		//		but not necessarily the same, value.
		//
		// formattedValue:
		//		If specified, used to set the visual element value,
		//		otherwise a computed visual value is used.
		//
		// priorityChange:
		//		If true, an onChange event is fired immediately instead of
		//		waiting for the next blur event.

		var filteredValue;
		if(value !== undefined){
			// TODO: this is calling filter() on both the display value and the actual value.
			// I added a comment to the filter() definition about this, but it should be changed.
			filteredValue = this.filter(value);
			if(typeof formattedValue != "string"){
				if(filteredValue !== null && ((typeof filteredValue != "number") || !isNaN(filteredValue))){
					formattedValue = this.filter(this.format(filteredValue, this.constraints));
				}else{ formattedValue = ''; }
			}
		}
		if(formattedValue != null && formattedValue != undefined && ((typeof formattedValue) != "number" || !isNaN(formattedValue)) && this.textbox.value != formattedValue){
			this.textbox.value = formattedValue;
			this._set("displayedValue", this.get("displayedValue"));
		}

		if(this.textDir == "auto"){
			this.applyTextDir(this.focusNode, formattedValue);
		}

		this.inherited(arguments, [filteredValue, priorityChange]);
	},

	// displayedValue: String
	//		For subclasses like ComboBox where the displayed value
	//		(ex: Kentucky) and the serialized value (ex: KY) are different,
	//		this represents the displayed value.
	//
	//		Setting 'displayedValue' through set('displayedValue', ...)
	//		updates 'value', and vice-versa.  Otherwise 'value' is updated
	//		from 'displayedValue' periodically, like onBlur etc.
	//
	//		TODO: move declaration to MappedTextBox?
	//		Problem is that ComboBox references displayedValue,
	//		for benefit of FilteringSelect.
	displayedValue: "",

	_getDisplayedValueAttr: function(){
		// summary:
		//		Hook so get('displayedValue') works.
		// description:
		//		Returns the displayed value (what the user sees on the screen),
		// 		after filtering (ie, trimming spaces etc.).
		//
		//		For some subclasses of TextBox (like ComboBox), the displayed value
		//		is different from the serialized value that's actually
		//		sent to the server (see dijit.form.ValidationTextBox.serialize)

		// TODO: maybe we should update this.displayedValue on every keystroke so that we don't need
		// this method
		// TODO: this isn't really the displayed value when the user is typing
		return this.filter(this.textbox.value);
	},

	_setDisplayedValueAttr: function(/*String*/ value){
		// summary:
		//		Hook so set('displayedValue', ...) works.
		// description:
		//		Sets the value of the visual element to the string "value".
		//		The widget value is also set to a corresponding,
		//		but not necessarily the same, value.

		if(value === null || value === undefined){ value = '' }
		else if(typeof value != "string"){ value = String(value) }

		this.textbox.value = value;

		// sets the serialized value to something corresponding to specified displayedValue
		// (if possible), and also updates the textbox.value, for example converting "123"
		// to "123.00"
		this._setValueAttr(this.get('value'), undefined);

		this._set("displayedValue", this.get('displayedValue'));

		// textDir support
		if(this.textDir == "auto"){
			this.applyTextDir(this.focusNode, value);
		}
	},

	format: function(value /*=====, constraints =====*/){
		// summary:
		//		Replaceable function to convert a value to a properly formatted string.
		// value: String
		// constraints: Object
		// tags:
		//		protected extension
		return ((value == null || value == undefined) ? "" : (value.toString ? value.toString() : value));
	},

	parse: function(value /*=====, constraints =====*/){
		// summary:
		//		Replaceable function to convert a formatted string to a value
		// value: String
		// constraints: Object
		// tags:
		//		protected extension

		return value;	// String
	},

	_refreshState: function(){
		// summary:
		//		After the user types some characters, etc., this method is
		//		called to check the field for validity etc.  The base method
		//		in `dijit.form.TextBox` does nothing, but subclasses override.
		// tags:
		//		protected
	},

	/*=====
	onInput: function(event){
		// summary:
		//		Connect to this function to receive notifications of various user data-input events.
		//		Return false to cancel the event and prevent it from being processed.
		// event:
		//		keydown | keypress | cut | paste | input
		// tags:
		//		callback
	},
	=====*/
	onInput: function(){},

	__skipInputEvent: false,
	_onInput: function(){
		// summary:
		//		Called AFTER the input event has happened
		// set text direction according to textDir that was defined in creation
		if(this.textDir == "auto"){
			this.applyTextDir(this.focusNode, this.focusNode.value);
		}

		this._refreshState();

		// In case someone is watch()'ing for changes to displayedValue
		this._set("displayedValue", this.get("displayedValue"));
	},

	postCreate: function(){
		// setting the value here is needed since value="" in the template causes "undefined"
		// and setting in the DOM (instead of the JS object) helps with form reset actions
		this.textbox.setAttribute("value", this.textbox.value); // DOM and JS values should be the same

		this.inherited(arguments);

		// normalize input events to reduce spurious event processing
		//	onkeydown: do not forward modifier keys
		//	           set charOrCode to numeric keycode
		//	onkeypress: do not forward numeric charOrCode keys (already sent through onkeydown)
		//	onpaste & oncut: set charOrCode to 229 (IME)
		//	oninput: if primary event not already processed, set charOrCode to 229 (IME), else do not forward
		var handleEvent = function(e){
			var charCode = e.charOrCode || e.keyCode || 229;
			if(e.type == "keydown"){
				switch(charCode){ // ignore "state" keys
					case keys.SHIFT:
					case keys.ALT:
					case keys.CTRL:
					case keys.META:
					case keys.CAPS_LOCK:
						return;
					default:
						if(charCode >= 65 && charCode <= 90){ return; } // keydown for A-Z can be processed with keypress
				}
			}
			if(e.type == "keypress" && typeof charCode != "string"){ return; }
			if(e.type == "input"){
				if(this.__skipInputEvent){ // duplicate event
					this.__skipInputEvent = false;
					return;
				}
			}else{
				this.__skipInputEvent = true;
			}
			// create fake event to set charOrCode and to know if preventDefault() was called
			var faux = lang.mixin({}, e, {
				charOrCode: charCode,
				wasConsumed: false,
				preventDefault: function(){
					faux.wasConsumed = true;
					e.preventDefault();
				},
				stopPropagation: function(){ e.stopPropagation(); }
			});
			// give web page author a chance to consume the event
			if(this.onInput(faux) === false){
				event.stop(faux); // return false means stop
			}
			if(faux.wasConsumed){ return; } // if preventDefault was called
			setTimeout(lang.hitch(this, "_onInput", faux), 0); // widget notification after key has posted
		};
		array.forEach([ "onkeydown", "onkeypress", "onpaste", "oncut", "oninput", "oncompositionend" ], function(event){
			this.connect(this.textbox, event, handleEvent);
		}, this);
	},

	_blankValue: '', // if the textbox is blank, what value should be reported
	filter: function(val){
		// summary:
		//		Auto-corrections (such as trimming) that are applied to textbox
		//		value on blur or form submit.
		// description:
		//		For MappedTextBox subclasses, this is called twice
		// 			- once with the display value
		//			- once the value as set/returned by set('value', ...)
		//		and get('value'), ex: a Number for NumberTextBox.
		//
		//		In the latter case it does corrections like converting null to NaN.  In
		//		the former case the NumberTextBox.filter() method calls this.inherited()
		//		to execute standard trimming code in TextBox.filter().
		//
		//		TODO: break this into two methods in 2.0
		//
		// tags:
		//		protected extension
		if(val === null){ return this._blankValue; }
		if(typeof val != "string"){ return val; }
		if(this.trim){
			val = lang.trim(val);
		}
		if(this.uppercase){
			val = val.toUpperCase();
		}
		if(this.lowercase){
			val = val.toLowerCase();
		}
		if(this.propercase){
			val = val.replace(/[^\s]+/g, function(word){
				return word.substring(0,1).toUpperCase() + word.substring(1);
			});
		}
		return val;
	},

	_setBlurValue: function(){
		this._setValueAttr(this.get('value'), true);
	},

	_onBlur: function(e){
		if(this.disabled){ return; }
		this._setBlurValue();
		this.inherited(arguments);

		if(this._selectOnClickHandle){
			this.disconnect(this._selectOnClickHandle);
		}
	},

	_isTextSelected: function(){
		return this.textbox.selectionStart == this.textbox.selectionEnd;
	},

	_onFocus: function(/*String*/ by){
		if(this.disabled || this.readOnly){ return; }

		// Select all text on focus via click if nothing already selected.
		// Since mouse-up will clear the selection need to defer selection until after mouse-up.
		// Don't do anything on focus by tabbing into the widget since there's no associated mouse-up event.
		if(this.selectOnClick && by == "mouse"){
			this._selectOnClickHandle = this.connect(this.domNode, "onmouseup", function(){
				// Only select all text on first click; otherwise users would have no way to clear
				// the selection.
				this.disconnect(this._selectOnClickHandle);

				// Check if the user selected some text manually (mouse-down, mouse-move, mouse-up)
				// and if not, then select all the text
				if(this._isTextSelected()){
					_TextBoxMixin.selectInputText(this.textbox);
				}
			});
		}
		// call this.inherited() before refreshState(), since this.inherited() will possibly scroll the viewport
		// (to scroll the TextBox into view), which will affect how _refreshState() positions the tooltip
		this.inherited(arguments);

		this._refreshState();
	},

	reset: function(){
		// Overrides dijit._FormWidget.reset().
		// Additionally resets the displayed textbox value to ''
		this.textbox.value = '';
		this.inherited(arguments);
	},
	_setTextDirAttr: function(/*String*/ textDir){
		// summary:
		//		Setter for textDir.
		// description:
		//		Users shouldn't call this function; they should be calling
		//		set('textDir', value)
		// tags:
		//		private

		// only if new textDir is different from the old one
		// and on widgets creation.
		if(!this._created
			|| this.textDir != textDir){
				this._set("textDir", textDir);
				// so the change of the textDir will take place immediately.
				this.applyTextDir(this.focusNode, this.focusNode.value);
		}
	}
});


_TextBoxMixin._setSelectionRange = dijit._setSelectionRange = function(/*DomNode*/ element, /*Number?*/ start, /*Number?*/ stop){
	if(element.setSelectionRange){
		element.setSelectionRange(start, stop);
	}
};

_TextBoxMixin.selectInputText = dijit.selectInputText = function(/*DomNode*/ element, /*Number?*/ start, /*Number?*/ stop){
	// summary:
	//		Select text in the input element argument, from start (default 0), to stop (default end).

	// TODO: use functions in _editor/selection.js?
	element = dom.byId(element);
	if(isNaN(start)){ start = 0; }
	if(isNaN(stop)){ stop = element.value ? element.value.length : 0; }
	try{
		element.focus();
		_TextBoxMixin._setSelectionRange(element, start, stop);
	}catch(e){ /* squelch random errors (esp. on IE) from unexpected focus changes or DOM nodes being hidden */ }
};

return _TextBoxMixin;
});

},
'dijit/form/_ToggleButtonMixin':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom-attr" // domAttr.set
], function(declare, domAttr){

// module:
//		dijit/form/_ToggleButtonMixin
// summary:
//		A mixin to provide functionality to allow a button that can be in two states (checked or not).

return declare("dijit.form._ToggleButtonMixin", null, {
	// summary:
	//		A mixin to provide functionality to allow a button that can be in two states (checked or not).

	// checked: Boolean
	//		Corresponds to the native HTML <input> element's attribute.
	//		In markup, specified as "checked='checked'" or just "checked".
	//		True if the button is depressed, or the checkbox is checked,
	//		or the radio button is selected, etc.
	checked: false,

	// aria-pressed for toggle buttons, and aria-checked for checkboxes
	_aria_attr: "aria-pressed",

	_onClick: function(/*Event*/ evt){
		var original = this.checked;
		this._set('checked', !original); // partially set the toggled value, assuming the toggle will work, so it can be overridden in the onclick handler
		var ret = this.inherited(arguments); // the user could reset the value here
		this.set('checked', ret ? this.checked : original); // officially set the toggled or user value, or reset it back
		return ret;
	},

	_setCheckedAttr: function(/*Boolean*/ value, /*Boolean?*/ priorityChange){
		this._set("checked", value);
		domAttr.set(this.focusNode || this.domNode, "checked", value);
		(this.focusNode || this.domNode).setAttribute(this._aria_attr, value ? "true" : "false"); // aria values should be strings
		this._handleOnChange(value, priorityChange);
	},

	reset: function(){
		// summary:
		//		Reset the widget's value to what it was at initialization time

		this._hasBeenBlurred = false;

		// set checked state to original setting
		this.set('checked', this.params.checked || false);
	}
});

});

},
'dojox/collections/ArrayList':function(){
define(["dojo/_base/kernel", "dojo/_base/array", "./_base"], function(dojo, darray, dxc){
/*=====
var dxc = dojox.collections;
=====*/
	dxc.ArrayList=function(/* array? */arr){
		//	summary
		//	Returns a new object of type dojox.collections.ArrayList
		var items=[];
		if(arr) items=items.concat(arr);
		this.count=items.length;
		this.add=function(/* object */obj){
			//	summary
			//	Add an element to the collection.
			items.push(obj);
			this.count=items.length;
		};
		this.addRange=function(/* array */a){
			//	summary
			//	Add a range of objects to the ArrayList
			if(a.getIterator){
				var e=a.getIterator();
				while(!e.atEnd()){
					this.add(e.get());
				}
				this.count=items.length;
			}else{
				for(var i=0; i<a.length; i++){
					items.push(a[i]);
				}
				this.count=items.length;
			}
		};
		this.clear=function(){
			//	summary
			//	Clear all elements out of the collection, and reset the count.
			items.splice(0, items.length);
			this.count=0;
		};
		this.clone=function(){
			//	summary
			//	Clone the array list
			return new dxc.ArrayList(items);	//	dojox.collections.ArrayList
		};
		this.contains=function(/* object */obj){
			//	summary
			//	Check to see if the passed object is a member in the ArrayList
			for(var i=0; i < items.length; i++){
				if(items[i] == obj) {
					return true;	//	bool
				}
			}
			return false;	//	bool
		};
		this.forEach=function(/* function */ fn, /* object? */ scope){
			//	summary
			//	functional iterator, following the mozilla spec.
			dojo.forEach(items, fn, scope);
		};
		this.getIterator=function(){
			//	summary
			//	Get an Iterator for this object
			return new dxc.Iterator(items);	//	dojox.collections.Iterator
		};
		this.indexOf=function(/* object */obj){
			//	summary
			//	Return the numeric index of the passed object; will return -1 if not found.
			for(var i=0; i < items.length; i++){
				if(items[i] == obj) {
					return i;	//	int
				}
			}
			return -1;	// int
		};
		this.insert=function(/* int */ i, /* object */ obj){
			//	summary
			//	Insert the passed object at index i
			items.splice(i,0,obj);
			this.count=items.length;
		};
		this.item=function(/* int */ i){
			//	summary
			//	return the element at index i
			return items[i];	//	object
		};
		this.remove=function(/* object */obj){
			//	summary
			//	Look for the passed object, and if found, remove it from the internal array.
			var i=this.indexOf(obj);
			if(i >=0) {
				items.splice(i,1);
			}
			this.count=items.length;
		};
		this.removeAt=function(/* int */ i){
			//	summary
			//	return an array with function applied to all elements
			items.splice(i,1);
			this.count=items.length;
		};
		this.reverse=function(){
			//	summary
			//	Reverse the internal array
			items.reverse();
		};
		this.sort=function(/* function? */ fn){
			//	summary
			//	sort the internal array
			if(fn){
				items.sort(fn);
			}else{
				items.sort();
			}
		};
		this.setByIndex=function(/* int */ i, /* object */ obj){
			//	summary
			//	Set an element in the array by the passed index.
			items[i]=obj;
			this.count=items.length;
		};
		this.toArray=function(){
			//	summary
			//	Return a new array with all of the items of the internal array concatenated.
			return [].concat(items);
		}
		this.toString=function(/* string */ delim){
			//	summary
			//	implementation of toString, follows [].toString();
			return items.join((delim||","));
		};
	};
	return dxc.ArrayList;
});

},
'ibm/usmi/dashboard/Constants':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define({
	resourceGroupSelected: "/nextgen/dashboard/topics/resourceGroupSelected",
	resourceGroupHoved: "/nextgen/dashboard/topics/resourceGroupHovered"
	
//	resourceGroupNames: ["chassis","virtualservers", "network", "storage", "favorites", "systems"],
	
/*	resourceGroupLabels: {
		"chassis": "Chassis",
		"virtualservers": "Hosts and VSs",
		"network": "Network",
		"storage": "Storage",
		"favorites": "Favorites",
		"systems": "All Systems"
	},
*/		
/*	
	resourceGroupDataSets: {
		"favorites": "GROUP:PERSONAL_GROUPS",
		"systems": "GROUP:ALL_ENDPOINTS"		
	},
*/
/*
	resourceGroupSelector: {
		"chassis": "GROUP:BLADECENTER_AND_MEMBERS",
		"virtualservers": "RESOURCE_VIEWS_VIRTUAL_TREE",
		"network": "RESOURCE_VIEWS_NETWORK_TREE",
		"storage": "RESOURCE_VIEWS_STORAGE_TREE"
	},
*/
/*	treeToppers: {
	    "virtualservers": [
	        {
		        "label": "All Virtual Servers",
		        "collectionID": "GROUP:VIRTUAL_SERVERS",
		        "taskWidgetId": "ibm/usmi/widgets/ResourceNavigator",
		        "icon" : "virtualServer_obj.gif"
	        }
	    ],
	    "chassis": [
            {
                "label": "All Chassis",
                "collectionID": "NEXT_GEN_ALL_CHASSIS",
                "taskWidgetId": "ibm/usmi/widgets/ResourceNavigator",
                "icon": "virtualServer_obj.gif"
            },
            {
                "label": "All Compute Nodes",
                "collectionID": "NEXT_GEN_ALL_COMPUTE_NODES",
                "taskWidgetId": "ibm/usmi/widgets/ResourceNavigator",
                "icon": "task_16.gif"
            }
        ],
	    "storage": [
	        {
		        "label": "All Storage Pools",
		        "collectionID": "NEXT_GEN_ALL_STORAGE_NODES",
		        "taskWidgetId": "ibm/usmi/widgets/ResourceNavigator",
		        "icon" : "virtualServer_obj.gif"
	        }
	    ]
	},
*/		
/*	resourceGroupSourceTokens:{
		"chassis": {
			parentGroupId: "BLADECENTER_SYSTEMS",
			description: "Contains Chassis"
			},
		"virtualservers": {
			parentGroupId: "VIRTUALIZATION_GROUPS",
			description: "Contains virtual servers"
			},	
		"network": {
			parentGroupId: "ROOT",
			description: "Contains All Network Systems"
			},
		"storage": {
			parentGroupId: "ROOT",
			description: "Contains all storage systems"
			},
		"favorites": {
			parentGroupId: "ROOT",
			description: "Contains groups optimized for each individual user"
				},
		"systems": {
			parentGroupId: "ROOT",
			description: "Contains all systems that can be managed including servers, chassis, operating systems, switches, blades, and storage systems"
			}
	}*/
});

},
'dojox/grid/_RowSelector':function(){
define([
	"dojo/_base/declare",
	"./_View"
], function(declare, _View){

return declare('dojox.grid._RowSelector', _View, {
	// summary:
	//	Custom grid view. If used in a grid structure, provides a small selectable region for grid rows.
	defaultWidth: "2em",
	noscroll: true,
	padBorderWidth: 2,
	buildRendering: function(){
		this.inherited('buildRendering', arguments);
		this.scrollboxNode.style.overflow = "hidden";
		this.headerNode.style.visibility = "hidden";
	},
	getWidth: function(){
		return this.viewWidth || this.defaultWidth;
	},
	buildRowContent: function(inRowIndex, inRowNode){
		var w = this.contentWidth || 0;
		inRowNode.innerHTML = '<table class="dojoxGridRowbarTable" style="width:' + w + 'px;height:1px;" border="0" cellspacing="0" cellpadding="0" role="presentation"><tr><td class="dojoxGridRowbarInner" tabindex="-1">&nbsp;</td></tr></table>';
	},
	renderHeader: function(){
	},
	updateRow: function(){
	},
	resize: function(){
		this.adaptHeight();
	},
	adaptWidth: function(){
		// Only calculate this here - rather than every call to buildRowContent
		if(!("contentWidth" in this) && this.contentNode){
			this.contentWidth = this.contentNode.offsetWidth - this.padBorderWidth;
		}
	},
	// styling
	doStyleRowNode: function(inRowIndex, inRowNode){
		var n = [ "dojoxGridRowbar dojoxGridNonNormalizedCell" ];
		if(this.grid.rows.isOver(inRowIndex)){
			n.push("dojoxGridRowbarOver");
		}
		if(this.grid.selection.isSelected(inRowIndex)){
			n.push("dojoxGridRowbarSelected");
		}
		inRowNode.className = n.join(" ");
	},
	// event handlers
	domouseover: function(e){
		this.grid.onMouseOverRow(e);
	},
	domouseout: function(e){
		if(!this.isIntraRowEvent(e)){
			this.grid.onMouseOutRow(e);
		}
	}
});
});
},
'dijit/form/MappedTextBox':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom-construct", // domConstruct.place
	"./ValidationTextBox"
], function(declare, domConstruct, ValidationTextBox){

/*=====
	var ValidationTextBox = dijit.form.ValidationTextBox;
=====*/

	// module:
	//		dijit/form/MappedTextBox
	// summary:
	//		A dijit.form.ValidationTextBox subclass which provides a base class for widgets that have
	//		a visible formatted display value, and a serializable
	//		value in a hidden input field which is actually sent to the server.

	return declare("dijit.form.MappedTextBox", ValidationTextBox, {
		// summary:
		//		A dijit.form.ValidationTextBox subclass which provides a base class for widgets that have
		//		a visible formatted display value, and a serializable
		//		value in a hidden input field which is actually sent to the server.
		// description:
		//		The visible display may
		//		be locale-dependent and interactive.  The value sent to the server is stored in a hidden
		//		input field which uses the `name` attribute declared by the original widget.  That value sent
		//		to the server is defined by the dijit.form.MappedTextBox.serialize method and is typically
		//		locale-neutral.
		// tags:
		//		protected

		postMixInProperties: function(){
			this.inherited(arguments);

			// we want the name attribute to go to the hidden <input>, not the displayed <input>,
			// so override _FormWidget.postMixInProperties() setting of nameAttrSetting
			this.nameAttrSetting = "";
		},

		// Override default behavior to assign name to focusNode
		_setNameAttr: null,

		serialize: function(val /*=====, options =====*/){
			// summary:
			//		Overridable function used to convert the get('value') result to a canonical
			//		(non-localized) string.  For example, will print dates in ISO format, and
			//		numbers the same way as they are represented in javascript.
			// val: anything
			// options: Object?
			// tags:
			//		protected extension
			return val.toString ? val.toString() : ""; // String
		},

		toString: function(){
			// summary:
			//		Returns widget as a printable string using the widget's value
			// tags:
			//		protected
			var val = this.filter(this.get('value')); // call filter in case value is nonstring and filter has been customized
			return val != null ? (typeof val == "string" ? val : this.serialize(val, this.constraints)) : ""; // String
		},

		validate: function(){
			// Overrides `dijit.form.TextBox.validate`
			this.valueNode.value = this.toString();
			return this.inherited(arguments);
		},

		buildRendering: function(){
			// Overrides `dijit._TemplatedMixin.buildRendering`

			this.inherited(arguments);

			// Create a hidden <input> node with the serialized value used for submit
			// (as opposed to the displayed value).
			// Passing in name as markup rather than calling domConstruct.create() with an attrs argument
			// to make query(input[name=...]) work on IE. (see #8660)
			this.valueNode = domConstruct.place("<input type='hidden'" + (this.name ? " name='" + this.name.replace(/'/g, "&quot;") + "'" : "") + "/>", this.textbox, "after");
		},

		reset: function(){
			// Overrides `dijit.form.ValidationTextBox.reset` to
			// reset the hidden textbox value to ''
			this.valueNode.value = '';
			this.inherited(arguments);
		}
	});
});

},
'ibm/tivoli/navmodel/DatasetUtils':function(){
// wrapped by build app
define("ibm/tivoli/navmodel/DatasetUtils", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/DatasetParameter,ibm/tivoli/navmodel/Utils,ibm/tivoli/navmodel/Trace"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.DatasetUtils"); 

// includes
dojo.require("ibm.tivoli.navmodel.DatasetParameter");
dojo.require("ibm.tivoli.navmodel.Utils");
dojo.require("ibm.tivoli.navmodel.Trace");

ibm.tivoli.navmodel.DatasetUtils._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.DatasetUtils");

ibm.tivoli.navmodel.DatasetUtils.getParameters = function(kwArgs) {
	// summary: 
	//		Retrieve available parameters that can be used when creating a dataset instance
	// onComplete: onComplete(parameters, kwArgs)
	//		callback for successful fetch
	// onError: _onError(error, exception, ioArgs)
	//		callback for error condition
	// baseUrl: String
	//		base url for identifying rest service
	// provider: String
	//		provider id
	// datasource: String
	//		datasource id
	// dataset: String
	//		datasetId
	// start: int
	//		start index to fetch
	// count: int
	//		number of items to fetch; null means all
	// sync: Boolean
	//		indicates if fetch is synchronous	
	// version: Float?
	//		optional: version used to communicate with rest service (i.e. json version)
	var args = {
		onComplete: function(parameters, parameterCount, kwArgs) {},
		onError: function(error, exception, ioArgs) {},
		baseUrl: null,
		provider: null,
		datasource: null,
		dataset: null,
		start: 0,
		count: null,
		sync: false,
		version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion()
	};
	dojo.mixin(args, kwArgs);

	// url needed to fetch parameters
	var url = args.baseUrl;
	url += "/providers/" + encodeURIComponent(args.provider);
	url += args.datasource ? ("/datasources/" + encodeURIComponent(args.datasource)) : "";
	url += "/datasets/"+ encodeURIComponent(args.dataset); 
	url += "/parameters";
		
	var content = ibm.tivoli.navmodel.Utils.getContent();
	if (args.start) {
		content.start = args.start;
	}
	if (args.count) {
		content.count = args.count;
	}
			
	return dojo.xhrGet({
		url: url,
		headers: {
			'Accept': ibm.tivoli.navmodel.Utils.getContentType(args.version, "datasetParmList")	
		},
		handleAs: "json",
		sync: args.sync,
		content: content,
		load: dojo.hitch(this, function(result, ioArgs) {
			var parms = [];			
			if (result.items) {
				var length = result.items.length;
				for (var index = 0;  index < length;  ++index) {
					var parm = new ibm.tivoli.navmodel.DatasetParameter({parameter: result.items[index], baseUrl: args.baseUrl, version: args.version});
					parms.push(parm);
				}
			}
			
		 	args.onComplete(parms, result.totalRows, result.iargs);							
		}),
		error: dojo.hitch (this, function(error, ioArgs){
			var exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "getParameters", ibm.tivoli.navmodel.DatasetUtils._trace, args.version);	
			args.onError(error, exception, ioArgs);			
		})
	});
};


ibm.tivoli.navmodel.DatasetUtils.getParameterById = function(kwArgs) {
	// summary: 
	//		Retrieve a particular parameter that can be used to create a dataset	
	// baseUrl: String
	//		base url for identifying rest service
	// provider: String
	//		provider id
	// datasource: String
	//		datasource id
	// dataset: String
	//		datasetId
	// parameter: String
	//		parameter id
	// version: Float?
	//		optional: version used to communicate with rest service (i.e. json version)
	
	var args = {
		baseUrl: null,
		provider: null,
		datasource: null,		
		dataset: null,
		parameter: null,
		version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion()
	};
	dojo.mixin(args, kwArgs);

	var parameter = null;
	var exception = null;
		
	// url needed to fetch parameters
	var url = args.baseUrl;
	url += "/providers/" + encodeURIComponent(args.provider);
	url += args.datasource ? ("/datasources/" + encodeURIComponent(args.datasource)) : "";
	url += "/datasets/"+ encodeURIComponent(args.dataset); 
	url += "/parameters/" + encodeURIComponent(args.parameter);
		
	dojo.xhrGet({
		url: url,
		headers: {
			'Accept': ibm.tivoli.navmodel.Utils.getContentType(args.version, "datasetParm")	
		},
		handleAs: "json",
		sync: true,
		load: dojo.hitch(this, function(result, ioArgs) {
			parameter = new ibm.tivoli.navmodel.DatasetParameter({parameter: result, baseUrl: args.baseUrl, version: args.version});										
		}),
		error: dojo.hitch (this, function(error, ioArgs){
			exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "getParameters", ibm.tivoli.navmodel.DatasetUtils._trace, args.version);				
		})
	});
	
	if (exception) {
		throw exception;
	}
	
	return parameter;
};
});

},
'gridx/core/Cell':function(){
define([
	"dojo/_base/declare"
], function(declare){

	
	return declare(/*===== "gridx.core.Cell", =====*/[], {
		// summary:
		//		Represents a cell of a grid
		// description:
		//		An instance of this class represents a grid cell.
		//		This class should not be directly instantiated by users. It should be returned by grid APIs.

		/*=====
		// row: [readonly] gridx.core.Row
		//		Reference to the row of this cell
		row: null,

		// column [readonly] gridx.core.Column
		//		Reference to the column of this cell
		column: null,

		// grid: [readonly] gridx.Grid
		//		Reference to the grid
		grid: null,

		// model: [readonly] grid.core.model.Model
		//		Reference to this grid model
		model: null,
		=====*/

		
		constructor: function(grid, row, column){
			var t = this;
			t.grid = grid;
			t.model = grid.model;
			t.row = row;
			t.column = column;
		},

		
		data: function(){
			// summary:
			//		Get the grid data of this cell.
			// description:
			//		Grid data means the result of the formatter functions (if exist).
			//		It can be different from store data (a.k.a. raw data).
			// returns:
			//		The grid data in this cell
			return this.model.byId(this.row.id).data[this.column.id];	//String|Number
		},

		
		rawData: function(){
			// summary:
			//		Get the store data of this cell.
			// description:
			//		If the column of this cell has a store field, then this method can return the store data of this cell.
			// returns:
			//		The store data of this cell
			var t = this, f = t.column.field();
			return f && t.model.byId(t.row.id).rawData[f];	//anything
		},

		
		setRawData: function(rawData){
			// summary:
			//		Set new raw data to this cell.
			// rawData:
			//		Anything that store can recognize as data
			// returns:
			//		If using server side store, a Deferred object is returned to indicate when the operation is finished.
			var obj = {},
				field = this.column.field();
			if(field){
				obj[field] = rawData;
				return this.row.setRawData(obj);	//dojo.Deferred
			}
		}
	});
});

},
'url:ibm/usmi/dashboard/templates/GroupHealthTip.html':"<div class=\"healthTip\">\r\n<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n    <div data-dojo-attach-point=\"containerNode\" style=\"display: none;\">\r\n        <div class=\"healthTip-all\">\r\n\t       <!-- <div class=\"entityTypeName\">${entityType}</div> -->\r\n\t        <div data-dojo-attach-point=\"healthTipBody\" class=\"healthTipBody\">\r\n\t\t\t\t<div id=\"recentEventLogs\" data-dojo-attach-point=\"recentEventLogs\">\r\n\t\t\t\t    <span class=\"healthTip-title\">${_resources.RecentEvents} </span>\r\n\t\t\t\t\t<span data-dojo-attach-point=\"noEvents\" class=\"healthTip-title-no-contents\">${_resources.NoRecentEvents}</span>\r\n\t\t\t\t\t<div id=\"eventLogTable\" data-dojo-attach-point=\"eventLogTable\">\r\n\t\t\t\t\t</div>\r\n\t\t\t\t</div>\r\n\t\t\t\t<div id=\"recentActiveAlerts\" data-dojo-attach-point=\"recentActiveAlerts\">\r\n\t\t\t\t\t<span class=\"healthTip-title\">${_resources.RecentAlerts} </span>\r\n\t\t\t\t\t<span data-dojo-attach-point=\"noAlerts\" class=\"healthTip-title-no-contents\">${_resources.NoRecentAlerts}</span>\r\n\t\t\t\t\t<div id=\"activeAlertsTable\" data-dojo-attach-point=\"activeAlertsTable\">\r\n\t\t\t\t</div>\r\n            </div>\r\n        </div>\r\n    </div>\r\n</div>\r\n",
'gridx/modules/NestedSort':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/connect",
	"dojo/_base/event",
	"dojo/_base/query",
	"dojo/_base/window",
	"dojo/_base/sniff",
	"dojo/string",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dojo/dom-style",
	"dojo/dom-geometry",
	"dojo/keys",
	"../core/_Module",
	"../core/model/extensions/Sort",
	"dojo/i18n!../nls/NestedSort"
], function(declare, array, connect, event, query, win, sniff, string, domClass, domConstruct, domStyle, domGeometry, keys, _Module, Sort, nls){
	
	var forEach = array.forEach,
		filter = array.filter,
		indexOf = array.indexOf,
		hasClass = domClass.contains,
		removeClass = domClass.remove,
		addClass = domClass.add;

	return declare(/*===== "gridx.modules.NestedSort", =====*/_Module, {
		name: 'sort',

		forced: ['header'],

//		required: ['vLayout'],

		modelExtensions: [Sort],

		_a11yText: {
			'dojoxGridDescending': '<span class="gridxNestedSortBtnText">&#9662;</span>',
			'dojoxGridAscending': '<span class="gridxNestedSortBtnText">&#9652;</span>',
			'dojoxGridAscendingTip': '<span class="gridxNestedSortBtnText">&#1784;</span>',
			'dojoxGridDescendingTip': '<span class="gridxNestedSortBtnText">&#1783;</span>',
			'dojoxGridUnsortedTip': '<span class="gridxNestedSortBtnText">x</span>' //'&#10006;'
		},

		constructor: function(){
			this._sortData = [];
		},

		getAPIPath: function(){
			return {
				sort: this
			};
		},

		preload: function(args){
			var t = this;
			t._sortData = t.arg('initialOrder') || t._sortData;
			//persistence support
			if(t.grid.persist){
				var d = t.grid.persist.registerAndLoad('sort', function(){
					return t._sortData;
				});
				if(d){
					t._sortData = d;
				}
			}
			t._sortData = filter(t._sortData, function(d){
				return t.isSortable(d.colId);
			});
			if(t._sortData.length){
				t.grid.model.sort(t._sortData);
			}
		},

		load: function(args){
			this._init();
			this.loaded.callback();
		},

		columnMixin: {
			isSorted: function(){
				return this.grid.sort.isSorted(this.id);
			},
			isSortable: function(){
				return this.grid.sort.isSortable(this.id);
			}
		},
		
		getSortData: function(){
			return this._sortData;
		},
		
		sort: function(sortData){
			var t = this;
			t._sortData = filter(sortData, function(d){
				return t.isSortable(d.colId);
			});
			t._doSort();
			t._updateUI();
		},
		
		isSorted: function(colId){
			return array.some(this._sortData, function(d){
				return d.colId == colId;
			});
		},

		_doSort: function(){
			var g = this.grid,
				d = this._sortData;
			g.model.sort(d.length ? d : null);
			g.body.refresh();
		},
		
		clear: function(){
			// summary:
			//	Clear the sorting state
			this._sortData.length = 0;
			this._doSort();
			this._updateUI();
		},

		isSortable: function(colId){
			var col = this.grid._columnsById[colId];
			return col && (col.sortable || col.sortable === undefined);
		},

		//Private---------------------------------------------------------------------------
		_init: function(){
			var t = this,
				n = t.grid.header.domNode,
				f = function(){
					t._initHeader();
					t._initFocus();
					t._updateUI();
				};
			t.connect(n, 'onclick', '_onHeaderClick');
			t.connect(n, 'onmouseover', '_onMouseOver');
			t.connect(n, 'onmouseout', '_onMouseOver');
			t.connect(t.grid.header, 'onRender', f);
			f();
		},
		
		_initHeader: function(){
			var t = this,
				table = t.grid.header.domNode.firstChild.firstChild,
				tds = table.rows[0].cells;
			if(query('.gridxSortBtn', table).length){
				return;
			}
			forEach(table.rows[0].cells, function(td){
				var colid = td.getAttribute('colid');
				if(t.isSortable(colid)){
					domConstruct.create('div', {
						className: 'gridxSortBtn gridxSortBtnNested'
					}, td, 'first');
					domConstruct.create('div', {
						className: 'gridxSortBtn gridxSortBtnSingle'
					}, td, 'first');
				}
			});
		},
		
		_onHeaderClick: function(e){
			var t = this,
				btn = e.target,
				sortData = t._sortData,
				colid;
			if(hasClass(btn, 'gridxNestedSortBtnText')){
				btn = btn.parentNode;
			}
			t._markFocus(e);
			if(hasClass(btn, 'gridxSortBtn')){
				colid = btn.parentNode.getAttribute('colid');
			}else{
				return;
			}
			if(hasClass(btn, 'gridxSortBtnSingle')){
				//single sort
				if(sortData.length > 1){
					sortData.length = 0;
				}
				var d = filter(sortData, function(data){
					return data.colId === colid;
				})[0];
				sortData.length = 0;
				if(d){
					sortData.push(d);
				}
				t._sortColumn(colid);
			}else if(hasClass(btn, 'gridxSortBtnNested')){
				//nested sort
				t._sortColumn(colid);
			}
			event.stop(e);
		},
		
		_onMouseOver: function(e){
			var g = this.grid;
			domClass.toggle(g.header.domNode, 'gridxHeaderHover', e.type == 'mouseover');
			//FIXME: this is ugly...
			if(g.autoHeight){
				g.vLayout.reLayout();
			}
		},

		_sortColumn: function(colid){
			// summary:
			//	Sort one column in nested sorting state
			var t = this,
				sortData = t._sortData;
			if(t.isSortable(colid)){
				var d = filter(sortData, function(d){
					return d.colId === colid;
				})[0];
				if(d){
					if(d.descending){
						sortData.splice(indexOf(sortData, d), 1);
					}
					d.descending = !d.descending;
				}else{
					sortData.push({
						colId: colid,
						descending: false
					});
				}
				t._doSort();
				t._updateUI();
			}
		},
		
		_updateUI: function(){
			var t = this,
				g = t.grid,
				dn = g.domNode,
				sortData = t._sortData;
			removeClass(dn, 'gridxSingleSorted');
			removeClass(dn, 'gridxNestedSorted');
			query('th', g.header.domNode).forEach(function(cell){
				var colid = cell.getAttribute('colid');
				if(t.isSortable(colid)){
					forEach(['', 'Desc', 'Asc', 'Main'], function(s){
						removeClass(cell, 'gridxCellSorted' + s);
					});
					var singleBtn = cell.childNodes[0],
						nestedBtn = cell.childNodes[1],
						a11yText = t._a11yText;
					singleBtn.title = nls.singleSort + ' - ' + nls.ascending;
					nestedBtn.title = nls.nestedSort + ' - ' + nls.ascending;
					singleBtn.innerHTML = a11yText.dojoxGridAscendingTip + '&nbsp;';
					nestedBtn.innerHTML = sortData.length + 1 + a11yText.dojoxGridAscendingTip;
					var d = filter(sortData, function(data){
						return data.colId === colid;
					})[0];
					t._setWaiState(cell, colid, d);
					if(d){
						nestedBtn.innerHTML = indexOf(sortData, d) + 1;
						addClass(cell, 'gridxCellSorted');
						if(d == sortData[0]){
							addClass(cell, 'gridxCellSortedMain');
						}
						var len = sortData.length;
						if(d.descending){
							addClass(cell, 'gridxCellSortedDesc');
							if(len == 1){
								singleBtn.title = nls.singleSort + ' - ' + nls.unsorted;
								singleBtn.innerHTML = a11yText.dojoxGridDescending + '&nbsp;';
							}else{
								nestedBtn.title = nls.nestedSort + ' - ' + nls.unsorted;
								nestedBtn.innerHTML += a11yText.dojoxGridDescending;
							}
						}else{
							addClass(cell, 'gridxCellSortedAsc');
							if(len == 1){
								singleBtn.title = nls.singleSort + ': ' + nls.descending;
								singleBtn.innerHTML = a11yText.dojoxGridAscending + '&nbsp;';
							}else{
								nestedBtn.title = nls.nestedSort + ' - ' + nls.descending;
								nestedBtn.innerHTML += a11yText.dojoxGridAscending;
							}
						}
					}
				}
			});
			if(sortData.length == 1){
				addClass(dn, 'gridxSingleSorted');
			}else if(sortData.length > 1){
				addClass(dn, 'gridxNestedSorted');
			}
		},
		
		//Focus and keyboard support---------------------------------------------------------------------------
		_initFocus: function(){
			var t = this,
				g = t.grid,
				headerNode = g.header.domNode;
			if(g.focus){
				t._initRegions();
				g.focus.registerArea({
					name: 'header',
					priority: 0,
					focusNode: headerNode,
					scope: t,
					doFocus: t._doFocus,
					doBlur: t._blurNode,
					onBlur: t._blurNode,
					connects: [
						t.connect(headerNode, 'onkeypress', '_onKeyPress')
					]
				});
			}
		},

		_doFocus: function(e){
			this._focusRegion(this._getCurrentRegion() || this._focusRegions[0]);
			return true;
		},

		_blurNode: function(e){
			return true;
		},

		_onKeyPress: function(e){
			var t = this,
				ltr = t.grid.isLeftToRight(),
				nextKey = ltr ? keys.RIGHT_ARROW : keys.LEFT_ARROW,
				previousKey = ltr ? keys.LEFT_ARROW : keys.RIGHT_ARROW;
			switch(e.keyCode){
				case previousKey:
					t._focusPrevious();
					break;
				case nextKey:
					t._focusNext();
					break;
				case keys.ENTER:
				case keys.SPACE:
					t._onHeaderClick(e);
					break;
			}
			event.stop(e);
		},

		_onBlur: function(e){
			this._blurRegion(e.target);
		},

		_focusNext: function(){
			var t = this,
				i = t._currRegionIdx,
				rs = t._focusRegions;
			while(rs[i+1] && domStyle.get(rs[++i], 'display') === 'none'){}
			if(rs[i]){
				t._focusRegion(rs[i]);
			}
		},

		_focusPrevious: function(){
			var t = this,
				i = t._currRegionIdx,
				rs = t._focusRegions;
			while(rs[i-1] && (domStyle.get(rs[--i], 'display') === 'none')){}
			if(rs[i]){
				t._focusRegion(rs[i]);
			}
		},

		_markFocus: function(e){
			var region = e.target,
				i = indexOf(this._focusRegions, region);
			if(i >= 0){
				this._focusRegion(region);
			}
		},

		_initRegions: function(){
			var t = this;
			forEach(t._nconns, connect.disconnect);
			t._focusRegions = [];
			t._nconns = [];
			query('.gridxCell', t.grid.header.domNode).forEach(function(cell){
				var children = cell.childNodes;
				forEach([2, 1, 0], function(i){
					if(children[i]){
						children[i].setAttribute('tabindex', '-1');
						t._focusRegions.push(children[i]);
						t._nconns.push(t.connect(children[i], 'onblur', '_onBlur'));
						return;
					}
				});
			});
			t._currRegionIdx = -1;
		},

		_focusRegion: function(region){
			// summary
			//		Focus the given region
			if(region){
				var t = this,
					g = t.grid,
					header = t._getRegionHeader(region);
				region.focus();
				
				
				window.setTimeout(function(){
					//make it asnyc so that IE will not lost focus
					addClass(header, 'gridxCellSortFocus');
					if(hasClass(region, 'gridxSortNode')){
						addClass(region, 'gridxSortNodeFocus');
					}else if(hasClass(region, 'gridxSortBtn')){
						addClass(region, 'gridxSortBtnFocus');
					}
					addClass(t.grid.header.domNode, 'gridxHeaderFocus');
					t._currRegionIdx = indexOf(t._focusRegions, region);
					
					//firefox and ie will lost focus when region is invisible, focus it again.
					region.focus();
					
					if(g.hScroller){
						g.hScroller.scrollToColumn(header.getAttribute('colid'));
					}
				}, 0);
				
			}
		},

		_blurRegion: function(region){
			if(region){
				var header = this._getRegionHeader(region);
				removeClass(header, 'gridxCellSortFocus');
				removeClass(region, 'gridxSortNodeFocus');
				removeClass(region, 'gridxSortBtnFocus');
				removeClass(this.grid.header.domNode, 'gridxHeaderFocus');
			}
		},

		_getCurrentRegion: function(){
			return this._currRegionIdx === -1 ? null : this._focusRegions[this._currRegionIdx];
		},

		_getRegionHeader: function(region){
			while(region && !hasClass(region, 'gridxCell')){
				region = region.parentNode;
			}
			return region;
		},
		
		//a11y support ----------------------------
		_setWaiState: function(cell, colid, data){
			var col = this.grid.column(colid),
				columnInfo = 'Column ' + col.name(),
				orderState = 'none', orderAction = 'ascending';
			if(data){
				orderState = data.descending ? 'descending' : 'ascending';
				orderAction = data.descending ? 'none' : 'descending';
			}
			var a11ySingleLabel = string.substitute(nls.waiSingleSortLabel, [columnInfo, orderState, orderAction]),
				a11yNestedLabel = string.substitute(nls.waiNestedSortLabel, [columnInfo, orderState, orderAction]);
			cell.childNodes[0].setAttribute("aria-label", a11ySingleLabel);
			cell.childNodes[1].setAttribute("aria-label", a11yNestedLabel);
		}
	});
});

},
'dojoe/tablex/_TableOptions':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/i18n!dojoe/tablex/nls/Table","dojo/require!dijit/_Widget,dijit/_TemplatedMixin,dijit/_Container,dijit/form/CheckBox,dojoe/tablex/_ColumnVisible"], function(dijit,dojo,dojox){
dojo.provide("dojoe.tablex._TableOptions");

dojo.require("dijit._Widget");
dojo.require("dijit._TemplatedMixin");
dojo.require("dijit._Container");
dojo.require("dijit.form.CheckBox");
dojo.require("dojoe.tablex._ColumnVisible");

// NLS
dojo.requireLocalization("dojoe.tablex", "Table");
// define the widget class
dojo.declare (
	"dojoe.tablex._TableOptions",[ dijit._Widget, dijit._Container ],{
	
	// summary:
	//		Private class for Configure columns widget
	
	grid: null,
	_structure: [],
	constructor: function() {
		// summary: A reusable widget for displaying a dynamic set of parameters to be 
		//  filled in at runtime.
	    this.resources_ = dojo.i18n.getLocalization("dojoe.tablex", "Table", this.lang);
	},
	postCreate: function() { 
		this.inherited ("postCreate", arguments);	
		/*LS - 13/01/2012 - 30956 - Unresponsive script msg when using configure options */	
		//removing the indirect way of fetching columns through grid structure and getting it through layout.cells directly
		// this is improve performance of this feature reducing no of iterations...
		this.addColumnsVisible(this._structure);
	},
	addColumnsVisible: function (columns) {		
		var widget = null;
		var column = null;
		for (var i in columns) {
			if (i) {
				column = columns[i];
				
				if (column.field) {
					var visible = this._checkColumnVisible(column.field);
					widget = new dojoe.tablex._ColumnVisible ({
						id: this.id + "column_" + column.field,
						columnid: this.id + "column_" + column.field,
						checked: visible,  
						label: column.name
					});
					this.addChild(widget);
					//console.log ("added column widget");
				} else {
					//console.log ("this column was created internally by data store.  do not show it.");
				}
			}
		}
	},
	
	_checkColumnVisible: function(colField) {
		var cols = this.grid.structure;
		for (var i in cols) {
			if(i){
				if(cols[i].field == colField){
					return true;
				}
			}
		}
		return false;
	},
		
	onSelectAll: function(){		
		//for(var j=0; j<this.grid.structure.length; j++) {
			//var columns = this.grid.structure[j].rows;	
			//Anupama :24May11, replacing for..in construct with regular for loop since for..in construct interfering with changes in Array prototype in IE7.
			//26600: select all,de-select all feature in Table configure options dialog not working on IE7 browser.
			//for (var i in columns) {
			/*LS - 13/01/2012 - 30956 - Unresponsive script msg when using configure options */	
			//removing the indirect way of fetching columns through grid structure and getting it through layout.cells directly
			// this is improve performance of this feature reducing no of iterations...
			for(var k=0; k<this._structure.length; k++) {
				var column = this._structure[k];
				if (column && column.field) {
					var columnWidget = dijit.byId (this.id + "column_" +column.field);
					columnWidget.setVisible(true);	 
				}
			}		
		//}
	},
	onDeselectAll: function(){
		//for(var j=0; j<this.grid.structure.length; j++) {
			//var columns = this.grid.structure[j].rows;
			//Anupama :24May11, replacing for..in construct with regular for loop since for..in construct interfering with changes in Array prototype in IE7.
			//26600: select all,de-select all feature in Table configure options dialog not working on IE7 browser.
			//for (var i in columns) {
			/*LS - 13/01/2012 - 30956 - Unresponsive script msg when using configure options */	
			//removing the indirect way of fetching columns through grid structure and getting it through layout.cells directly
			// this is improve performance of this feature reducing no of iterations...
			for(var k=0; k<this._structure.length; k++) {
				var column = this._structure[k];
				if (column && column.field) {
					var columnWidget = dijit.byId (this.id + "column_" + column.field);
					columnWidget.setVisible(false);	 
				}
			}
		//}
	},
	onSave: function(){
	/*LS - 13/01/2012 - 30956 - Unresponsive script msg when using configure options */	
		//removing the indirect way of fetching columns through grid structure and getting it through layout.cells directly
		// this is improve performance of this feature reducing no of iterations...	
		var visibleCols = [];
		for(var k=0; k<this._structure.length; k++) {
			var column = this._structure[k];
			if(column && column.field) {
				var tblColumn = column;
				tblColumn.width = tblColumn.unitWidth;
				column.width = tblColumn.unitWidth;
				var columnWidget = dijit.byId (this.id + "column_" + column.field);
				column.hidden = !(columnWidget.isVisible());
				column.id = column.field
				//do not use setColumnVisibility, as each call for each cell makes a view update which is a overkill...
				//this.grid.layout.setColumnVisibility(k, columnWidget.isVisible());
				this._structure[k] = column;
				if(!column.hidden)visibleCols.push(this._structure[k]);	
				
			}
		}
		this.grid.setColumns(visibleCols);
		// persist the visible column information
		if(this.grid.persist){
			var config = {'configColumns':visibleCols };
			this.grid.persist.key = this.grid.id;
			this.grid.persist.put(this.grid.persist.key, config);
			this.grid.persist.registerAndLoad('configColumns', function(){
						return {
							visibleCols: visibleCols
						};
					});
			this.grid.persist.save();
		}
		
		
	},
	
	isColumnVisible: function (columnID) {
		var columnWidget = dijit.byId (this.id + "column_" + columnID);
		return (columnWidget.isVisible());
	}	
});

});

},
'dijit/form/_AutoCompleterMixin':function(){
define([
	"dojo/_base/connect", // keys keys.SHIFT
	"dojo/data/util/filter", // patternToRegExp
	"dojo/_base/declare", // declare
	"dojo/_base/Deferred", // Deferred.when
	"dojo/dom-attr", // domAttr.get
	"dojo/_base/event", // event.stop
	"dojo/keys",
	"dojo/_base/lang", // lang.clone lang.hitch
	"dojo/query", // query
	"dojo/regexp", // regexp.escapeString
	"dojo/_base/sniff", // has("ie")
	"dojo/string", // string.substitute
	"dojo/_base/window", // win.doc.selection.createRange
	"./DataList",
	"../registry",	// registry.byId
	"./_TextBoxMixin"	// defines _TextBoxMixin.selectInputText
], function(connect, filter, declare, Deferred, domAttr, event, keys, lang, query, regexp, has, string, win,
			DataList, registry, _TextBoxMixin){

	// module:
	//		dijit/form/_AutoCompleterMixin
	// summary:
	//		A mixin that implements the base functionality for `dijit.form.ComboBox`/`dijit.form.FilteringSelect`


	return declare("dijit.form._AutoCompleterMixin", null, {
		// summary:
		//		A mixin that implements the base functionality for `dijit.form.ComboBox`/`dijit.form.FilteringSelect`
		// description:
		//		All widgets that mix in dijit.form._AutoCompleterMixin must extend `dijit.form._FormValueWidget`.
		// tags:
		//		protected

		// item: Object
		//		This is the item returned by the dojo.data.store implementation that
		//		provides the data for this ComboBox, it's the currently selected item.
		item: null,

		// pageSize: Integer
		//		Argument to data provider.
		//		Specifies number of search results per page (before hitting "next" button)
		pageSize: Infinity,

		// store: [const] dojo.store.api.Store
		//		Reference to data provider object used by this ComboBox
		store: null,

		// fetchProperties: Object
		//		Mixin to the store's fetch.
		//		For example, to set the sort order of the ComboBox menu, pass:
		//	|	{ sort: [{attribute:"name",descending: true}] }
		//		To override the default queryOptions so that deep=false, do:
		//	|	{ queryOptions: {ignoreCase: true, deep: false} }
		fetchProperties:{},

		// query: Object
		//		A query that can be passed to 'store' to initially filter the items,
		//		before doing further filtering based on `searchAttr` and the key.
		//		Any reference to the `searchAttr` is ignored.
		query: {},

		// autoComplete: Boolean
		//		If user types in a partial string, and then tab out of the `<input>` box,
		//		automatically copy the first entry displayed in the drop down list to
		//		the `<input>` field
		autoComplete: true,

		// highlightMatch: String
		// 		One of: "first", "all" or "none".
		//
		//		If the ComboBox/FilteringSelect opens with the search results and the searched
		//		string can be found, it will be highlighted.  If set to "all"
		//		then will probably want to change `queryExpr` parameter to '*${0}*'
		//
		//		Highlighting is only performed when `labelType` is "text", so as to not
		//		interfere with any HTML markup an HTML label might contain.
		highlightMatch: "first",

		// searchDelay: Integer
		//		Delay in milliseconds between when user types something and we start
		//		searching based on that value
		searchDelay: 100,

		// searchAttr: String
		//		Search for items in the data store where this attribute (in the item)
		//		matches what the user typed
		searchAttr: "name",

		// labelAttr: String?
		//		The entries in the drop down list come from this attribute in the
		//		dojo.data items.
		//		If not specified, the searchAttr attribute is used instead.
		labelAttr: "",

		// labelType: String
		//		Specifies how to interpret the labelAttr in the data store items.
		//		Can be "html" or "text".
		labelType: "text",

		// queryExpr: String
		//		This specifies what query ComboBox/FilteringSelect sends to the data store,
		//		based on what the user has typed.  Changing this expression will modify
		//		whether the drop down shows only exact matches, a "starting with" match,
		//		etc.  Use it in conjunction with highlightMatch.
		//		dojo.data query expression pattern.
		//		`${0}` will be substituted for the user text.
		//		`*` is used for wildcards.
		//		`${0}*` means "starts with", `*${0}*` means "contains", `${0}` means "is"
		queryExpr: "${0}*",

		// ignoreCase: Boolean
		//		Set true if the ComboBox/FilteringSelect should ignore case when matching possible items
		ignoreCase: true,

		// Flags to _HasDropDown to limit height of drop down to make it fit in viewport
		maxHeight: -1,

		// For backwards compatibility let onClick events propagate, even clicks on the down arrow button
		_stopClickEvents: false,

		_getCaretPos: function(/*DomNode*/ element){
			// khtml 3.5.2 has selection* methods as does webkit nightlies from 2005-06-22
			var pos = 0;
			if(typeof(element.selectionStart) == "number"){
				// FIXME: this is totally borked on Moz < 1.3. Any recourse?
				pos = element.selectionStart;
			}else if(has("ie")){
				// in the case of a mouse click in a popup being handled,
				// then the win.doc.selection is not the textarea, but the popup
				// var r = win.doc.selection.createRange();
				// hack to get IE 6 to play nice. What a POS browser.
				var tr = win.doc.selection.createRange().duplicate();
				var ntr = element.createTextRange();
				tr.move("character",0);
				ntr.move("character",0);
				try{
					// If control doesn't have focus, you get an exception.
					// Seems to happen on reverse-tab, but can also happen on tab (seems to be a race condition - only happens sometimes).
					// There appears to be no workaround for this - googled for quite a while.
					ntr.setEndPoint("EndToEnd", tr);
					pos = String(ntr.text).replace(/\r/g,"").length;
				}catch(e){
					// If focus has shifted, 0 is fine for caret pos.
				}
			}
			return pos;
		},

		_setCaretPos: function(/*DomNode*/ element, /*Number*/ location){
			location = parseInt(location);
			_TextBoxMixin.selectInputText(element, location, location);
		},

		_setDisabledAttr: function(/*Boolean*/ value){
			// Additional code to set disabled state of ComboBox node.
			// Overrides _FormValueWidget._setDisabledAttr() or ValidationTextBox._setDisabledAttr().
			this.inherited(arguments);
			this.domNode.setAttribute("aria-disabled", value);
		},

		_abortQuery: function(){
			// stop in-progress query
			if(this.searchTimer){
				clearTimeout(this.searchTimer);
				this.searchTimer = null;
			}
			if(this._fetchHandle){
				if(this._fetchHandle.cancel){
					this._cancelingQuery = true;
					this._fetchHandle.cancel();
					this._cancelingQuery = false;
				}
				this._fetchHandle = null;
			}
		},

		_onInput: function(/*Event*/ evt){
			// summary:
			//		Handles paste events
			this.inherited(arguments);
			if(evt.charOrCode == 229){ // IME or cut/paste event
				this._onKey(evt);
			}
		},

		_onKey: function(/*Event*/ evt){
			// summary:
			//		Handles keyboard events

			if(this.disabled || this.readOnly){ return; }
			var key = evt.charOrCode;

			// except for cutting/pasting case - ctrl + x/v
			if(evt.altKey || ((evt.ctrlKey || evt.metaKey) && (key != 'x' && key != 'v')) || key == keys.SHIFT){
				return; // throw out weird key combinations and spurious events
			}

			var doSearch = false;
			var pw = this.dropDown;
			var highlighted = null;
			this._prev_key_backspace = false;
			this._abortQuery();

			// _HasDropDown will do some of the work:
			//		1. when drop down is not yet shown:
			//			- if user presses the down arrow key, call loadDropDown()
			//		2. when drop down is already displayed:
			//			- on ESC key, call closeDropDown()
			//			- otherwise, call dropDown.handleKey() to process the keystroke
			this.inherited(arguments);

			if(this._opened){
				highlighted = pw.getHighlightedOption();
			}
			switch(key){
				case keys.PAGE_DOWN:
				case keys.DOWN_ARROW:
				case keys.PAGE_UP:
				case keys.UP_ARROW:
					// Keystroke caused ComboBox_menu to move to a different item.
					// Copy new item to <input> box.
					if(this._opened){
						this._announceOption(highlighted);
					}
					event.stop(evt);
					break;

				case keys.ENTER:
					// prevent submitting form if user presses enter. Also
					// prevent accepting the value if either Next or Previous
					// are selected
					if(highlighted){
						// only stop event on prev/next
						if(highlighted == pw.nextButton){
							this._nextSearch(1);
							event.stop(evt);
							break;
						}else if(highlighted == pw.previousButton){
							this._nextSearch(-1);
							event.stop(evt);
							break;
						}
					}else{
						// Update 'value' (ex: KY) according to currently displayed text
						this._setBlurValue(); // set value if needed
						this._setCaretPos(this.focusNode, this.focusNode.value.length); // move cursor to end and cancel highlighting
					}
					// default case:
					// if enter pressed while drop down is open, or for FilteringSelect,
					// if we are in the middle of a query to convert a directly typed in value to an item,
					// prevent submit
					if(this._opened || this._fetchHandle){
						event.stop(evt);
					}
					// fall through

				case keys.TAB:
					var newvalue = this.get('displayedValue');
					//	if the user had More Choices selected fall into the
					//	_onBlur handler
					if(pw && (
						newvalue == pw._messages["previousMessage"] ||
						newvalue == pw._messages["nextMessage"])
					){
						break;
					}
					if(highlighted){
						this._selectOption(highlighted);
					}
					// fall through

				case keys.ESCAPE:
					if(this._opened){
						this._lastQuery = null; // in case results come back later
						this.closeDropDown();
					}
					break;

				case ' ':
					if(highlighted){
						// user is effectively clicking a choice in the drop down menu
						event.stop(evt);
						this._selectOption(highlighted);
						this.closeDropDown();
					}else{
						// user typed a space into the input box, treat as normal character
						doSearch = true;
					}
					break;

				case keys.DELETE:
				case keys.BACKSPACE:
					this._prev_key_backspace = true;
					doSearch = true;
					break;

				default:
					// Non char keys (F1-F12 etc..)  shouldn't open list.
					// Ascii characters and IME input (Chinese, Japanese etc.) should.
					//IME input produces keycode == 229.
					doSearch = typeof key == 'string' || key == 229;
			}
			if(doSearch){
				// need to wait a tad before start search so that the event
				// bubbles through DOM and we have value visible
				this.item = undefined; // undefined means item needs to be set
				this.searchTimer = setTimeout(lang.hitch(this, "_startSearchFromInput"),1);
			}
		},

		_autoCompleteText: function(/*String*/ text){
			// summary:
			// 		Fill in the textbox with the first item from the drop down
			// 		list, and highlight the characters that were
			// 		auto-completed. For example, if user typed "CA" and the
			// 		drop down list appeared, the textbox would be changed to
			// 		"California" and "ifornia" would be highlighted.

			var fn = this.focusNode;

			// IE7: clear selection so next highlight works all the time
			_TextBoxMixin.selectInputText(fn, fn.value.length);
			// does text autoComplete the value in the textbox?
			var caseFilter = this.ignoreCase? 'toLowerCase' : 'substr';
			if(text[caseFilter](0).indexOf(this.focusNode.value[caseFilter](0)) == 0){
				var cpos = this.autoComplete ? this._getCaretPos(fn) : fn.value.length;
				// only try to extend if we added the last character at the end of the input
				if((cpos+1) > fn.value.length){
					// only add to input node as we would overwrite Capitalisation of chars
					// actually, that is ok
					fn.value = text;//.substr(cpos);
					// visually highlight the autocompleted characters
					_TextBoxMixin.selectInputText(fn, cpos);
				}
			}else{
				// text does not autoComplete; replace the whole value and highlight
				fn.value = text;
				_TextBoxMixin.selectInputText(fn);
			}
		},

		_openResultList: function(/*Object*/ results, /*Object*/ query, /*Object*/ options){
			// summary:
			//		Callback when a search completes.
			// description:
			//		1. generates drop-down list and calls _showResultList() to display it
			//		2. if this result list is from user pressing "more choices"/"previous choices"
			//			then tell screen reader to announce new option
			this._fetchHandle = null;
			if(	this.disabled ||
				this.readOnly ||
				(query[this.searchAttr] !== this._lastQuery)	// TODO: better way to avoid getting unwanted notify
			){
				return;
			}
			var wasSelected = this.dropDown.getHighlightedOption();
			this.dropDown.clearResultList();
			if(!results.length && options.start == 0){ // if no results and not just the previous choices button
				this.closeDropDown();
				return;
			}

			// Fill in the textbox with the first item from the drop down list,
			// and highlight the characters that were auto-completed. For
			// example, if user typed "CA" and the drop down list appeared, the
			// textbox would be changed to "California" and "ifornia" would be
			// highlighted.

			this.dropDown.createOptions(
				results,
				options,
				lang.hitch(this, "_getMenuLabelFromItem")
			);

			// show our list (only if we have content, else nothing)
			this._showResultList();

			// #4091:
			//		tell the screen reader that the paging callback finished by
			//		shouting the next choice
			if(options.direction){
				if(1 == options.direction){
					this.dropDown.highlightFirstOption();
				}else if(-1 == options.direction){
					this.dropDown.highlightLastOption();
				}
				if(wasSelected){
					this._announceOption(this.dropDown.getHighlightedOption());
				}
			}else if(this.autoComplete && !this._prev_key_backspace
				// when the user clicks the arrow button to show the full list,
				// startSearch looks for "*".
				// it does not make sense to autocomplete
				// if they are just previewing the options available.
				&& !/^[*]+$/.test(query[this.searchAttr].toString())){
				this._announceOption(this.dropDown.containerNode.firstChild.nextSibling); // 1st real item
			}
		},

		_showResultList: function(){
			// summary:
			//		Display the drop down if not already displayed, or if it is displayed, then
			//		reposition it if necessary (reposition may be necessary if drop down's height changed).
			this.closeDropDown(true);
			this.openDropDown();
			this.domNode.setAttribute("aria-expanded", "true");
		},

		loadDropDown: function(/*Function*/ /*===== callback =====*/){
			// Overrides _HasDropDown.loadDropDown().
			// This is called when user has pressed button icon or pressed the down arrow key
			// to open the drop down.

			this._startSearchAll();
		},

		isLoaded: function(){
			// signal to _HasDropDown that it needs to call loadDropDown() to load the
			// drop down asynchronously before displaying it
			return false;
		},

		closeDropDown: function(){
			// Overrides _HasDropDown.closeDropDown().  Closes the drop down (assuming that it's open).
			// This method is the callback when the user types ESC or clicking
			// the button icon while the drop down is open.  It's also called by other code.
			this._abortQuery();
			if(this._opened){
				this.inherited(arguments);
				this.domNode.setAttribute("aria-expanded", "false");
				this.focusNode.removeAttribute("aria-activedescendant");
			}
		},

		_setBlurValue: function(){
			// if the user clicks away from the textbox OR tabs away, set the
			// value to the textbox value
			// #4617:
			//		if value is now more choices or previous choices, revert
			//		the value
			var newvalue = this.get('displayedValue');
			var pw = this.dropDown;
			if(pw && (
				newvalue == pw._messages["previousMessage"] ||
				newvalue == pw._messages["nextMessage"]
				)
			){
				this._setValueAttr(this._lastValueReported, true);
			}else if(typeof this.item == "undefined"){
				// Update 'value' (ex: KY) according to currently displayed text
				this.item = null;
				this.set('displayedValue', newvalue);
			}else{
				if(this.value != this._lastValueReported){
					this._handleOnChange(this.value, true);
				}
				this._refreshState();
			}
		},

		_setItemAttr: function(/*item*/ item, /*Boolean?*/ priorityChange, /*String?*/ displayedValue){
			// summary:
			//		Set the displayed valued in the input box, and the hidden value
			//		that gets submitted, based on a dojo.data store item.
			// description:
			//		Users shouldn't call this function; they should be calling
			//		set('item', value)
			// tags:
			//		private
			var value = '';
			if(item){
				if(!displayedValue){
					displayedValue = this.store._oldAPI ?	// remove getValue() for 2.0 (old dojo.data API)
						this.store.getValue(item, this.searchAttr) : item[this.searchAttr];
				}
				value = this._getValueField() != this.searchAttr ? this.store.getIdentity(item) : displayedValue;
			}
			this.set('value', value, priorityChange, displayedValue, item);
		},

		_announceOption: function(/*Node*/ node){
			// summary:
			//		a11y code that puts the highlighted option in the textbox.
			//		This way screen readers will know what is happening in the
			//		menu.

			if(!node){
				return;
			}
			// pull the text value from the item attached to the DOM node
			var newValue;
			if(node == this.dropDown.nextButton ||
				node == this.dropDown.previousButton){
				newValue = node.innerHTML;
				this.item = undefined;
				this.value = '';
			}else{
				var item = this.dropDown.items[node.getAttribute("item")];
				newValue = (this.store._oldAPI ? 	// remove getValue() for 2.0 (old dojo.data API)
					this.store.getValue(item, this.searchAttr) : item[this.searchAttr]).toString();
				this.set('item', item, false, newValue);
			}
			// get the text that the user manually entered (cut off autocompleted text)
			this.focusNode.value = this.focusNode.value.substring(0, this._lastInput.length);
			// set up ARIA activedescendant
			this.focusNode.setAttribute("aria-activedescendant", domAttr.get(node, "id"));
			// autocomplete the rest of the option to announce change
			this._autoCompleteText(newValue);
		},

		_selectOption: function(/*DomNode*/ target){
			// summary:
			//		Menu callback function, called when an item in the menu is selected.
			this.closeDropDown();
			if(target){
				this._announceOption(target);
			}
			this._setCaretPos(this.focusNode, this.focusNode.value.length);
			this._handleOnChange(this.value, true);
		},

		_startSearchAll: function(){
			this._startSearch('');
		},

		_startSearchFromInput: function(){
			this._startSearch(this.focusNode.value.replace(/([\\\*\?])/g, "\\$1"));
		},

		_getQueryString: function(/*String*/ text){
			return string.substitute(this.queryExpr, [text]);
		},

		_startSearch: function(/*String*/ key){
			// summary:
			//		Starts a search for elements matching key (key=="" means to return all items),
			//		and calls _openResultList() when the search completes, to display the results.
			if(!this.dropDown){
				var popupId = this.id + "_popup",
					dropDownConstructor = lang.isString(this.dropDownClass) ?
						lang.getObject(this.dropDownClass, false) : this.dropDownClass;
				this.dropDown = new dropDownConstructor({
					onChange: lang.hitch(this, this._selectOption),
					id: popupId,
					dir: this.dir,
					textDir: this.textDir
				});
				this.focusNode.removeAttribute("aria-activedescendant");
				this.textbox.setAttribute("aria-owns",popupId); // associate popup with textbox
			}
			this._lastInput = key; // Store exactly what was entered by the user.

			// Setup parameters to be passed to store.query().
			// Create a new query to prevent accidentally querying for a hidden
			// value from FilteringSelect's keyField
			var query = lang.clone(this.query); // #5970
			var options = {
				start: 0,
				count: this.pageSize,
				queryOptions: {		// remove for 2.0
					ignoreCase: this.ignoreCase,
					deep: true
				}
			};
			lang.mixin(options, this.fetchProperties);

			// Generate query
			var qs = this._getQueryString(key), q;
			if(this.store._oldAPI){
				// remove this branch for 2.0
				q = qs;
			}else{
				// Query on searchAttr is a regex for benefit of dojo.store.Memory,
				// but with a toString() method to help dojo.store.JsonRest.
				// Search string like "Co*" converted to regex like /^Co.*$/i.
				q = filter.patternToRegExp(qs, this.ignoreCase);
				q.toString = function(){ return qs; };
			}
			this._lastQuery = query[this.searchAttr] = q;

			// Function to run the query, wait for the results, and then call _openResultList()
			var _this = this,
				startQuery = function(){
					var resPromise = _this._fetchHandle = _this.store.query(query, options);
					Deferred.when(resPromise, function(res){
						_this._fetchHandle = null;
						res.total = resPromise.total;
						_this._openResultList(res, query, options);
					}, function(err){
						_this._fetchHandle = null;
						if(!_this._cancelingQuery){	// don't treat canceled query as an error
							console.error(_this.declaredClass + ' ' + err.toString());
							_this.closeDropDown();
						}
					});
				};

			// #5970: set _lastQuery, *then* start the timeout
			// otherwise, if the user types and the last query returns before the timeout,
			// _lastQuery won't be set and their input gets rewritten

			this.searchTimer = setTimeout(lang.hitch(this, function(query, _this){
				this.searchTimer = null;

				startQuery();

				// Setup method to handle clicking next/previous buttons to page through results
				this._nextSearch = this.dropDown.onPage = function(direction){
					options.start += options.count * direction;
					//	tell callback the direction of the paging so the screen
					//	reader knows which menu option to shout
					options.direction = direction;
					startQuery();
					_this.focus();
				};
			}, query, this), this.searchDelay);
		},

		_getValueField: function(){
			// summary:
			//		Helper for postMixInProperties() to set this.value based on data inlined into the markup.
			//		Returns the attribute name in the item (in dijit.form._ComboBoxDataStore) to use as the value.
			return this.searchAttr;
		},

		//////////// INITIALIZATION METHODS ///////////////////////////////////////

		constructor: function(){
			this.query={};
			this.fetchProperties={};
		},

		postMixInProperties: function(){
			if(!this.store){
				var srcNodeRef = this.srcNodeRef;
				var list = this.list;
				if(list){
					this.store = registry.byId(list);
				}else{
					// if user didn't specify store, then assume there are option tags
					this.store = new DataList({}, srcNodeRef);
				}

				// if there is no value set and there is an option list, set
				// the value to the first value to be consistent with native Select
				// Firefox and Safari set value
				// IE6 and Opera set selectedIndex, which is automatically set
				// by the selected attribute of an option tag
				// IE6 does not set value, Opera sets value = selectedIndex
				if(!("value" in this.params)){
					var item = (this.item = this.store.fetchSelectedItem());
					if(item){
						var valueField = this._getValueField();
						// remove getValue() for 2.0 (old dojo.data API)
						this.value = this.store._oldAPI ? this.store.getValue(item, valueField) : item[valueField];
					}
				}
			}

			this.inherited(arguments);
		},

		postCreate: function(){
			// summary:
			//		Subclasses must call this method from their postCreate() methods
			// tags:
			//		protected

			// find any associated label element and add to ComboBox node.
			var label=query('label[for="'+this.id+'"]');
			if(label.length){
				label[0].id = (this.id+"_label");
				this.domNode.setAttribute("aria-labelledby", label[0].id);

			}
			this.inherited(arguments);
		},

		_getMenuLabelFromItem: function(/*Item*/ item){
			var label = this.labelFunc(item, this.store),
				labelType = this.labelType;
			// If labelType is not "text" we don't want to screw any markup ot whatever.
			if(this.highlightMatch != "none" && this.labelType == "text" && this._lastInput){
				label = this.doHighlight(label, this._escapeHtml(this._lastInput));
				labelType = "html";
			}
			return {html: labelType == "html", label: label};
		},

		doHighlight: function(/*String*/ label, /*String*/ find){
			// summary:
			//		Highlights the string entered by the user in the menu.  By default this
			//		highlights the first occurrence found. Override this method
			//		to implement your custom highlighting.
			// tags:
			//		protected

			var
				// Add (g)lobal modifier when this.highlightMatch == "all" and (i)gnorecase when this.ignoreCase == true
				modifiers = (this.ignoreCase ? "i" : "") + (this.highlightMatch == "all" ? "g" : ""),
				i = this.queryExpr.indexOf("${0}");
			find = regexp.escapeString(find); // escape regexp special chars
			return this._escapeHtml(label).replace(
				// prepend ^ when this.queryExpr == "${0}*" and append $ when this.queryExpr == "*${0}"
				new RegExp((i == 0 ? "^" : "") + "("+ find +")" + (i == (this.queryExpr.length - 4) ? "$" : ""), modifiers),
				'<span class="dijitComboBoxHighlightMatch">$1</span>'
			); // returns String, (almost) valid HTML (entities encoded)
		},

		_escapeHtml: function(/*String*/ str){
			// TODO Should become dojo.html.entities(), when exists use instead
			// summary:
			//		Adds escape sequences for special characters in XML: &<>"'
			str = String(str).replace(/&/gm, "&amp;").replace(/</gm, "&lt;")
				.replace(/>/gm, "&gt;").replace(/"/gm, "&quot;"); //balance"
			return str; // string
		},

		reset: function(){
			// Overrides the _FormWidget.reset().
			// Additionally reset the .item (to clean up).
			this.item = null;
			this.inherited(arguments);
		},

		labelFunc: function(/*item*/ item, /*dojo.store.api.Store*/ store){
			// summary:
			//		Computes the label to display based on the dojo.data store item.
			// returns:
			//		The label that the ComboBox should display
			// tags:
			//		private

			// Use toString() because XMLStore returns an XMLItem whereas this
			// method is expected to return a String (#9354).
			// Remove getValue() for 2.0 (old dojo.data API)
			return (store._oldAPI ? store.getValue(item, this.labelAttr || this.searchAttr) :
				item[this.labelAttr || this.searchAttr]).toString(); // String
		},

		_setValueAttr: function(/*String*/ value, /*Boolean?*/ priorityChange, /*String?*/ displayedValue, /*item?*/ item){
			// summary:
			//		Hook so set('value', value) works.
			// description:
			//		Sets the value of the select.
			this._set("item", item||null); // value not looked up in store
			if(!value){ value = ''; } // null translates to blank
			this.inherited(arguments);
		},
		_setTextDirAttr: function(/*String*/ textDir){
			// summary:
			//		Setter for textDir, needed for the dropDown's textDir update.
			// description:
			//		Users shouldn't call this function; they should be calling
			//		set('textDir', value)
			// tags:
			//		private
			this.inherited(arguments);
			// update the drop down also (_ComboBoxMenuMixin)
			if(this.dropDown){
				this.dropDown._set("textDir", textDir);
			}
		}
	});
});

},
'gridx/modules/extendedSelect/Cell':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/query",
	"dojo/_base/lang",
	"dojo/_base/Deferred",
	"dojo/_base/sniff",
	"dojo/dom-class",
	"dojo/mouse",
	"dojo/keys",
	"../../core/_Module",
	"./_RowCellBase"
], function(declare, array, query, lang, Deferred, sniff, domClass, mouse, keys, _Module, _RowCellBase){

	var isArrayLike = lang.isArrayLike;

	function createItem(rowId, visualIndex, columnId, columnIndex){
		return {
			rid: rowId,
			r: visualIndex,
			cid: columnId,
			c: columnIndex
		};
	}

	return declare(/*===== "gridx.modules.extendedSelect.Cell", =====*/_RowCellBase, {
		// summary:
		//		Provides advanced cell selections.
		// description:
		//		This module provides an advanced way for selecting cells by clicking, swiping, SPACE key, or CTRL/SHIFT CLICK to select multiple cell.
		//
		// example:
		//		1. Use select api on cell object obtained from grid.cell(i,j)
		//		|	grid.cell(1,1).select();
		//		|	grid.cell(1,1).deselect();
		//		|	grid.cell(1,1).isSelected();
		//
		//		2. Use select api on select.cell module
		//		|	grid.select.cell.selectById(columnId);
		//		|	grid.select.cell.deSelectById(columnId);
		//		|	grid.select.cell.isSelected(columnId);
		//		|	grid.select.cell.getSelected();//[]
		//		|	grid.select.cell.clear();

		// name: [readonly] String
		//		module name
		name: 'selectCell',

		// cellMixin: Object
		//		A map of functions to be mixed into grid cell object, so that we can use select api on column object directly
		//		- grid.cell(1,1).select() | deselect() | isSelected();
		cellMixin: {
			select: function(){
				this.grid.select.cell.selectByIndex(this.row.index(), this.column.index());
				return this;
			},
			deselect: function(){
				this.grid.select.cell.deselectByIndex(this.row.index(), this.column.index());
				return this;
			},
			isSelected: function(){
				return this.grid.select.cell.isSelected(this.row.id, this.column.id);
			}
		},
		
		//Public-----------------------------------------------------------------
/*=====
		selectById: function(rowId, columnId){
			// summary:
			//		Select a cell by (rowId, columnId)
		},
		
		deselectById: function(columnId){
			// summary:
			//		Deselect a cell by (rowId, columnId)
		},
		
		selectByIndex: function(rowIndex, columnIndex){
			// summary:
			//		Select a cess by (rowIndex, columnIndex)
		},
		
		deSelectByIndex: function(rowIndex, columnIndex){
			// summary:
			//		Deselect a cell by (rowIndex, columnIndex)
		},		
=====*/
		getSelected: function(){
			// summary:
			//		Get an array of selected cells e.g.[['row1', 'col1'], ['row2', 'col2']]
			var t = this, res = [];
			array.forEach(t.grid._columns, function(col){
				var ids = t.model.getMarkedIds(t._getMarkType(col.id));
				res.push.apply(res, array.map(ids, function(rid){
					return [rid, col.id];
				}));
			});
			return res;
		},

		clear: function(silent){
			// summary:
			//		Deselected all selected cells	
			var t = this;
			query(".gridxCellSelected", t.grid.bodyNode).forEach(function(node){
				domClass.remove(node, 'gridxCellSelected');
				node.removeAttribute('aria-selected');
			});
			array.forEach(t.grid._columns, function(col){
				t.model.clearMark(t._getMarkType(col.id));
			});
			t._clear();
			if(!silent){
				t._onSelectionChange();
			}
		},

		isSelected: function(rowId, columnId){
			// summary:
			//		Check if the given cell is selected.			
			return this.model.getMark(rowId, this._getMarkType(columnId));
		},
		
		//Private---------------------------------------------------------------------
		_type: 'cell',

		_markTypePrefix: "select_",

		_getMarkType: function(colId){
			var type = this._markTypePrefix + colId;
			this.model._spTypes[type] = 1;
			return type;
		},

		_markById: function(args, toSelect){
			if(!isArrayLike(args[0])){
				args = [args];
			}
			var t = this, columns = t.grid._columnsById, model = t.model;
			array.forEach(args, function(cell){
				var rowId = cell[0], colId = cell[1];
				if(rowId && columns[colId]){
					model.markById(rowId, toSelect, t._getMarkType(colId));
				}
			});
			model.when();
		},

		_markByIndex: function(args, toSelect){
			if(!isArrayLike(args[0])){
				args = [args];
			}
			args = array.filter(args, function(arg){
				if(isArrayLike(arg) && arg.length >= 2 && 
					arg[0] >= 0 && arg[0] < Infinity && arg[1] >= 0 && arg[1] < Infinity){
					if(arg.length >= 4 && arg[2] >= 0 && arg[2] < Infinity && arg[3] >= 0 && arg[3] < Infinity){
						arg._range = 1;	//1 as true
					}
					return true;
				}
			});
			var t = this,
				m = t.model,
				g = t.grid,
				columns = g._columns,
				body = g.body,
				i, j, col, type, rowInfo;
			array.forEach(args, function(arg){
				if(arg._range){
					var a = Math.min(arg[0], arg[2]),
						b = Math.max(arg[0], arg[2]),
						n = b - a + 1,
						c1 = Math.min(arg[1], arg[3]),
						c2 = Math.max(arg[1], arg[3]);
					for(i = c1; i <= c2; ++i){
						col = columns[i];
						if(col){
							rowInfo = body.getRowInfo({visualIndex: a});
							a = rowInfo.rowIndex;
							type = t._getMarkType(col.id);
							for(j = 0; j < n; ++j){
								m.markByIndex(a + j, toSelect, type, rowInfo.parentId);
							}
						}
					}
				}else{
					col = columns[arg[1]];
					if(col){
						rowInfo = body.getRowInfo({visualIndex: arg[0]});
						i = rowInfo.rowIndex;
						m.markByIndex(i, toSelect, t._getMarkType(col.id), rowInfo.parentId);
					}
				}
			});
			return m.when();
		},

		_init: function(){
			var t = this, g = t.grid;
			t.inherited(arguments);
			t.batchConnect(
				[g, 'onCellMouseDown', function(e){
					if(mouse.isLeft(e)){
						t._start(createItem(e.rowId, e.visualIndex, e.columnId, e.columnIndex), g._isCopyEvent(e), e.shiftKey);
					}
				}],
				[g, 'onCellMouseOver', function(e){
					t._highlight(createItem(e.rowId, e.visualIndex, e.columnId, e.columnIndex));
				}],
				[g, sniff('ff') < 4 ? 'onCellKeyUp' : 'onCellKeyDown', function(e){
					if(e.keyCode === keys.SPACE){
						t._start(createItem(e.rowId, e.visualIndex, e.columnId, e.columnIndex), g._isCopyEvent(e), e.shiftKey);
						t._end();
					}
				}]
			);
		},

		_onRender: function(start, count){
			var t = this, i, j,
				m = t.model,
				g = t.grid,
				cols = g._columns,
				end = start + count;
			for(i = 0; i < cols.length; ++i){
				var cid = cols[i].id,
					type = t._getMarkType(cid);
				if(m.getMarkedIds(type).length){
					for(j = start; j < end; ++j){
						var rid = t._getRowId(j);
						if(m.getMark(rid, type) || (t._selecting && t._toSelect &&
							t._inRange(i, t._startItem.c, t._currentItem.c, 1) && //1 as true
							t._inRange(j, t._startItem.r, t._currentItem.r, 1))){	//1 as true
							domClass.add(query('[visualindex="' + j + '"] [colid="' + cid + '"]', g.bodyNode)[0], 'gridxCellSelected');
						}
					}
				}
			}
		},

		_onMark: function(id, toMark, oldState, type){
			var t = this;
			if(lang.isString(type) && !t._marking && type.indexOf(t._markTypePrefix) === 0){
				var rowNode = query('[rowid="' + id + '"]', t.grid.bodyNode)[0];
				if(rowNode){
					var cid = type.substr(t._markTypePrefix.length),
						node = query('[colid="' + cid + '"]', rowNode)[0];
					if(node){
						domClass.toggle(node, 'gridxCellSelected', toMark);
					}
				}
			}
		},

		_onMoveToCell: function(rowVisIndex, colIndex, e){
			if(e.shiftKey){
				var t = this,
					g = t.grid,
					rid = t._getRowId(rowVisIndex),
					cid = g._columns[colIndex].id;
				t._start(createItem(rid, rowVisIndex, cid, colIndex), g._isCopyEvent(e), 1);	//1 as true
				t._end();
			}
		},

		_isSelected: function(item){
			var t = this;
			if(!item.rid){
				item.rid = t._getRowId(item.r);
			}
			if(t._isRange){
				var rids = t._refSelectedIds[item.cid];
				return rids && array.indexOf(rids, item.rid) >= 0;
			}else{
				return t.model.getMark(item.rid, t._getMarkType(item.cid));
			}
		},

		_highlight: function(target){
			var t = this,
				current = t._currentItem;
			if(t._selecting){
				if(current === null){
					//First time select.
					t._highlightSingle(target, 1);	//1 as true
					//In IE, when setSelectable(false), the onfocusin event will not fire on doc, so the focus border is gone.
					//So refocus it here.
					if(sniff('ie')){
						t._focus(target);
					}
				}else{
					var start = t._startItem,
						highlight = function(from, to, toHL){
							var colDir = to.c > from.c ? 1 : -1,
								rowDir = to.r > from.r ? 1 : -1,
								i, j, p, q, rids = {};
							if(!toHL){
								for(j = from.r, p = to.r + rowDir; j != p; j += rowDir){
									rids[j] = t.model.indexToId(j);
								}
							}
							for(i = from.c, q = to.c + colDir; i != q; i += colDir){
								var cid = t.grid._columns[i].id;
								for(j = from.r, p = to.r + rowDir; j != p; j += rowDir){
									t._highlightSingle(createItem(rids[j], j, cid, i), toHL);
								}
							}
						};
					if(t._inRange(target.r, start.r, current.r) ||
						t._inRange(target.c, start.c, current.c) ||
						t._inRange(start.r, target.r, current.r) ||
						t._inRange(start.c, target.c, current.c)){
						highlight(start, current, 0);	//0 as false
					}
					highlight(start, target, 1);	//1 as true
					t._focus(target);
				}
				t._currentItem = target;
			}
		},

		_doHighlight: function(item, toHighlight){
			var i, j, rowNodes = this.grid.bodyNode.childNodes;
			for(i = rowNodes.length - 1; i >= 0; --i){
				if(rowNodes[i].getAttribute('visualindex') == item.r){
					var cellNodes = rowNodes[i].getElementsByTagName('td');
					for(j = cellNodes.length - 1; j >= 0; --j){
						if(cellNodes[j].getAttribute('colid') == item.cid){
							domClass.toggle(cellNodes[j], 'gridxCellSelected', toHighlight);
							cellNodes[j].setAttribute('aria-selected', !!toHighlight);
							return;
						}
					}
					return;
				}
			}
		},

		_focus: function(target){
			if(this.grid.focus){
				this.grid.body._focusCell(null, target.r, target.c);
			}
		},

		_getSelectedIds: function(){
			var t = this, res = {};
			array.forEach(t.grid._columns, function(col){
				var ids = t.model.getMarkedIds(t._getMarkType(col.id));
				if(ids.length){
					res[col.id] = ids;
				}
			});
			return res;
		},
		
		_beginAutoScroll: function(){},

		_endAutoScroll: function(){},

		_addToSelected: function(start, end, toSelect){
			var t = this,
				model = t.model,
				d = new Deferred(),
				lastEndItem = t._lastEndItem,
				a, b, colDir, i, j,
				packs = [],
				finish = function(){
					model.when().then(function(){
						d.callback();
					});
				};
			if(!t._isRange){
				t._refSelectedIds = t._getSelectedIds();
			}
			if(t._isRange){
				if(t._inRange(end.r, start.r, lastEndItem.r)){
					a = Math.min(end.r, lastEndItem.r);
					b = Math.max(end.r, lastEndItem.r);
					packs.push({
						start: a + 1,
						count: b - a,
						columnStart: start.c,
						columnEnd: lastEndItem.c
					});
				}
				if(t._inRange(end.c, start.c, lastEndItem.c)){
					colDir = end.c < lastEndItem.c ? 1 : -1;
					a = Math.min(start.r, end.r);
					b = Math.max(start.r, end.r);
					packs.push({
						start: a,
						count: b - a + 1,
						columnStart: end.c + colDir,
						columnEnd: lastEndItem.c
					});
				}
			}
			colDir = start.c < end.c ? 1 : -1;
			for(i = start.c; i != end.c + colDir; i += colDir){
				var cid = t.grid._columns[i].id,
					type = t._getMarkType(cid);
				a = Math.min(start.r, end.r);
				b = Math.max(start.r, end.r);
				for(j = a; j <= b; ++j){
					model.markByIndex(j, toSelect, type);
				}
			}
			if(packs.length){
				model.when(packs, function(){
					var i, j, k, pack;
					for(i = 0; i < packs.length; ++i){
						pack = packs[i];
						var colDir = pack.columnStart < pack.columnEnd ? 1 : -1;
						for(j = pack.columnStart; j != pack.columnEnd + colDir; j += colDir){
							var cid = t.grid._columns[j].id,
								type = t._getMarkType(cid),
								rids = t._refSelectedIds[cid];
							for(k = pack.start; k < pack.start + pack.count; ++k){
								var rid = model.indexToId(k),
									selected = rids && rids[rid];
								model.markById(rid, selected, type);
							}
						}
					}
				}).then(finish);
			}else{
				finish();
			}
			return d;
		}
	});
});

},
'url:ibm/usmi/statuspod/templates/jobStatuspodDialog.html':"<div class=\"statuspod\">\r\n<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n\t<div class=\"statuspodTabContainer\" id=\"jobStatusPodTabContainer\" data-dojo-attach-point=\"jobStatusPodTabContainer\" data-dojo-type=\"dijit.layout.TabContainer\" style=\"width: 100%; height: 100%;\" doLayout=\"false\" controllerWidget=\"dijit.layout.TabController\">\r\n\t\t<div id=\"errorJobs_pane\" data-dojo-attach-point=\"errorJobs_pane\" data-dojo-type=\"dijit.layout.ContentPane\" title=\"${ErrorJobsTabTitle}(${totalErrorJobsCount})\" selected=\"true\">\r\n\t\t\t<div data-dojo-type=\"idx.oneui.Menu\" class=\"menu-errorJobs\" data-dojo-attach-point=\"errorJobsMenu\">\r\n\t\t\t</div>\r\n\t\t\t\r\n\t\t\t<div class=\"statusPodPaneSpace\" data-dojo-attach-point=\"errorJobsSpace\">\r\n\t\t\t</div>\r\n\t\t\t\r\n\t\t\t<div class=\"footerHeader\">\r\n\t\t\t\t<span class=\"statusPodSummary\" data-dojo-attach-point=\"errorJobsSummary\"></span>\r\n\t\t\t</div>\r\n\t\t\t<div class=\"footerBottom\">\r\n\t\t\t\t<a data-dojo-attach-point=\"errorJobsMenuLink\" href=\"javascript:launchInContext('com.ibm.usmi.console.scheduler.navigationElement');\">${ViewAllJobsLabel}</a>\r\n\t\t\t\t<img src=\"js/ibm/usmi/core/framework/themes/images/ac10_opennewtab_24.png\" alt=\"launch in context image\" style=\"padding-left:5px; display:none;\"/>\r\n\t\t\t</div>\r\n\t\t</div>\r\n\t\t\r\n\t\t<div id=\"activeJobs_pane\" data-dojo-attach-point=\"activeJobs_pane\" data-dojo-type=\"dijit.layout.ContentPane\" title=\"${ActiveJobsTabTitle}(${totalActiveJobsCount})\" >\r\n\t\t   <div data-dojo-type=\"idx.oneui.Menu\" class=\"menu-activeJobs\" data-dojo-attach-point=\"activeJobsMenu\">\r\n\t\t\t</div>\r\n\t\t\t\r\n\t\t\t<div class=\"statusPodPaneSpace\" data-dojo-attach-point=\"activeJobsSpace\">\r\n\t\t\t</div>\r\n\r\n\t\t\t<div class=\"footerHeader\">\r\n\t\t\t\t<span class=\"statusPodSummary\" data-dojo-attach-point=\"activeJobsSummary\"></span>\r\n\t\t\t</div>\r\n\t\t\t<div class=\"footerBottom\">\r\n\t\t\t\t<a data-dojo-attach-point=\"activeJobsMenuLink\" href=\"javascript:launchInContext('com.ibm.usmi.console.scheduler.navigationElement');\">${ViewAllJobsLabel}</a>\r\n\t\t\t\t<img src=\"js/ibm/usmi/core/framework/themes/images/ac10_opennewtab_24.png\" alt=\"launch in context image\" style=\"padding-left:5px; display:none;\"/>\r\n\t\t\t</div>\r\n        </div>\r\n        \r\n        <div id=\"completedJobs_pane\" data-dojo-attach-point=\"completedJobs_pane\" data-dojo-type=\"dijit.layout.ContentPane\" title=\"${CompletedJobsTabTitle}(${totalCompletedJobsCount})\" >\r\n\t\t   <div data-dojo-type=\"idx.oneui.Menu\" class=\"menu-completedJobs\" data-dojo-attach-point=\"completedJobsMenu\">\r\n\t\t\t</div>\r\n\t\t\t\r\n\t\t\t<div class=\"statusPodPaneSpace\" data-dojo-attach-point=\"completedJobsSpace\">\r\n\t\t\t</div>\r\n\r\n\t\t\t<div class=\"footerHeader\">\r\n\t\t\t\t<span class=\"statusPodSummary\" data-dojo-attach-point=\"completedJobsSummary\"></span>\r\n\t\t\t</div>\r\n\t\t\t<div class=\"footerBottom\">\r\n\t\t\t\t<a data-dojo-attach-point=\"completedJobsMenuLink\" href=\"javascript:launchInContext('com.ibm.usmi.console.scheduler.navigationElement');\">${ViewAllJobsLabel}</a>\r\n\t\t\t\t<img src=\"js/ibm/usmi/core/framework/themes/images/ac10_opennewtab_24.png\" alt=\"launch in context image\" style=\"padding-left:5px; display:none;\"/>\r\n\t\t\t</div>\r\n        </div>\r\n\t</div>\r\n\t\t\r\n</div>",
'dijit/layout/StackController':function(){
define([
	"dojo/_base/array", // array.forEach array.indexOf array.map
	"dojo/_base/declare", // declare
	"dojo/_base/event", // event.stop
	"dojo/keys", // keys
	"dojo/_base/lang", // lang.getObject
	"dojo/_base/sniff", // has("ie")
	"../focus",		// focus.focus()
	"../registry",	// registry.byId
	"../_Widget",
	"../_TemplatedMixin",
	"../_Container",
	"../form/ToggleButton",
	"dojo/i18n!../nls/common"
], function(array, declare, event, keys, lang, has,
			focus, registry, _Widget, _TemplatedMixin, _Container, ToggleButton){

/*=====
	var _Widget = dijit._Widget;
	var _TemplatedMixin = dijit._TemplatedMixin;
	var _Container = dijit._Container;
	var ToggleButton = dijit.form.ToggleButton;
=====*/

	// module:
	//		dijit/layout/StackController
	// summary:
	//		Set of buttons to select a page in a `dijit.layout.StackContainer`

	var StackButton = declare("dijit.layout._StackButton", ToggleButton, {
		// summary:
		//		Internal widget used by StackContainer.
		// description:
		//		The button-like or tab-like object you click to select or delete a page
		// tags:
		//		private

		// Override _FormWidget.tabIndex.
		// StackContainer buttons are not in the tab order by default.
		// Probably we should be calling this.startupKeyNavChildren() instead.
		tabIndex: "-1",

		// closeButton: Boolean
		//		When true, display close button for this tab
		closeButton: false,
		
		_setCheckedAttr: function(/*Boolean*/ value, /*Boolean?*/ priorityChange){
			this.inherited(arguments);
			this.focusNode.removeAttribute("aria-pressed");
		},

		buildRendering: function(/*Event*/ evt){
			this.inherited(arguments);
			(this.focusNode || this.domNode).setAttribute("role", "tab");
		},

		onClick: function(/*Event*/ /*===== evt =====*/){
			// summary:
			//		This is for TabContainer where the tabs are <span> rather than button,
			//		so need to set focus explicitly (on some browsers)
			//		Note that you shouldn't override this method, but you can connect to it.
			focus.focus(this.focusNode);

			// ... now let StackController catch the event and tell me what to do
		},

		onClickCloseButton: function(/*Event*/ evt){
			// summary:
			//		StackContainer connects to this function; if your widget contains a close button
			//		then clicking it should call this function.
			//		Note that you shouldn't override this method, but you can connect to it.
			evt.stopPropagation();
		}
	});


	var StackController = declare("dijit.layout.StackController", [_Widget, _TemplatedMixin, _Container], {
		// summary:
		//		Set of buttons to select a page in a `dijit.layout.StackContainer`
		// description:
		//		Monitors the specified StackContainer, and whenever a page is
		//		added, deleted, or selected, updates itself accordingly.

		baseClass: "dijitStackController",

		templateString: "<span role='tablist' data-dojo-attach-event='onkeypress'></span>",

		// containerId: [const] String
		//		The id of the page container that I point to
		containerId: "",

		// buttonWidget: [const] Constructor
		//		The button widget to create to correspond to each page
		buttonWidget: StackButton,

		constructor: function(){
			this.pane2button = {};		// mapping from pane id to buttons
			this.pane2connects = {};	// mapping from pane id to this.connect() handles
			this.pane2watches = {};		// mapping from pane id to watch() handles
		},

		postCreate: function(){
			this.inherited(arguments);

			// Listen to notifications from StackContainer
			this.subscribe(this.containerId+"-startup", "onStartup");
			this.subscribe(this.containerId+"-addChild", "onAddChild");
			this.subscribe(this.containerId+"-removeChild", "onRemoveChild");
			this.subscribe(this.containerId+"-selectChild", "onSelectChild");
			this.subscribe(this.containerId+"-containerKeyPress", "onContainerKeyPress");
		},

		onStartup: function(/*Object*/ info){
			// summary:
			//		Called after StackContainer has finished initializing
			// tags:
			//		private
			array.forEach(info.children, this.onAddChild, this);
			if(info.selected){
				// Show button corresponding to selected pane (unless selected
				// is null because there are no panes)
				this.onSelectChild(info.selected);
			}
		},

		destroy: function(){
			for(var pane in this.pane2button){
				this.onRemoveChild(registry.byId(pane));
			}
			this.inherited(arguments);
		},

		onAddChild: function(/*dijit._Widget*/ page, /*Integer?*/ insertIndex){
			// summary:
			//		Called whenever a page is added to the container.
			//		Create button corresponding to the page.
			// tags:
			//		private

			// create an instance of the button widget
			// (remove typeof buttonWidget == string support in 2.0)
			var cls = lang.isString(this.buttonWidget) ? lang.getObject(this.buttonWidget) : this.buttonWidget;
			var button = new cls({
				id: this.id + "_" + page.id,
				label: page.title,
				dir: page.dir,
				lang: page.lang,
				textDir: page.textDir,
				showLabel: page.showTitle,
				iconClass: page.iconClass,
				closeButton: page.closable,
				title: page.tooltip
			});
			button.focusNode.setAttribute("aria-selected", "false");


			// map from page attribute to corresponding tab button attribute
			var pageAttrList = ["title", "showTitle", "iconClass", "closable", "tooltip"],
				buttonAttrList = ["label", "showLabel", "iconClass", "closeButton", "title"];

			// watch() so events like page title changes are reflected in tab button
			this.pane2watches[page.id] = array.map(pageAttrList, function(pageAttr, idx){
				return page.watch(pageAttr, function(name, oldVal, newVal){
					button.set(buttonAttrList[idx], newVal);
				});
			});

			// connections so that clicking a tab button selects the corresponding page
			this.pane2connects[page.id] = [
				this.connect(button, 'onClick', lang.hitch(this,"onButtonClick", page)),
				this.connect(button, 'onClickCloseButton', lang.hitch(this,"onCloseButtonClick", page))
			];

			this.addChild(button, insertIndex);
			this.pane2button[page.id] = button;
			page.controlButton = button;	// this value might be overwritten if two tabs point to same container
			if(!this._currentChild){ // put the first child into the tab order
				button.focusNode.setAttribute("tabIndex", "0");
				button.focusNode.setAttribute("aria-selected", "true");
				this._currentChild = page;
			}
			// make sure all tabs have the same length
			if(!this.isLeftToRight() && has("ie") && this._rectifyRtlTabList){
				this._rectifyRtlTabList();
			}
		},

		onRemoveChild: function(/*dijit._Widget*/ page){
			// summary:
			//		Called whenever a page is removed from the container.
			//		Remove the button corresponding to the page.
			// tags:
			//		private

			if(this._currentChild === page){ this._currentChild = null; }

			// disconnect/unwatch connections/watches related to page being removed
			array.forEach(this.pane2connects[page.id], lang.hitch(this, "disconnect"));
			delete this.pane2connects[page.id];
			array.forEach(this.pane2watches[page.id], function(w){ w.unwatch(); });
			delete this.pane2watches[page.id];

			var button = this.pane2button[page.id];
			if(button){
				this.removeChild(button);
				delete this.pane2button[page.id];
				button.destroy();
			}
			delete page.controlButton;
		},

		onSelectChild: function(/*dijit._Widget*/ page){
			// summary:
			//		Called when a page has been selected in the StackContainer, either by me or by another StackController
			// tags:
			//		private

			if(!page){ return; }

			if(this._currentChild){
				var oldButton=this.pane2button[this._currentChild.id];
				oldButton.set('checked', false);
				oldButton.focusNode.setAttribute("aria-selected", "false");
				oldButton.focusNode.setAttribute("tabIndex", "-1");
			}

			var newButton=this.pane2button[page.id];
			newButton.set('checked', true);
			newButton.focusNode.setAttribute("aria-selected", "true");
			this._currentChild = page;
			newButton.focusNode.setAttribute("tabIndex", "0");
			var container = registry.byId(this.containerId);
			container.containerNode.setAttribute("aria-labelledby", newButton.id);
		},

		onButtonClick: function(/*dijit._Widget*/ page){
			// summary:
			//		Called whenever one of my child buttons is pressed in an attempt to select a page
			// tags:
			//		private

			if(this._currentChild.id === page.id) {
				//In case the user clicked the checked button, keep it in the checked state because it remains to be the selected stack page.
				var button=this.pane2button[page.id];
				button.set('checked', true);
			}
			var container = registry.byId(this.containerId);
			container.selectChild(page);
		},

		onCloseButtonClick: function(/*dijit._Widget*/ page){
			// summary:
			//		Called whenever one of my child buttons [X] is pressed in an attempt to close a page
			// tags:
			//		private

			var container = registry.byId(this.containerId);
			container.closeChild(page);
			if(this._currentChild){
				var b = this.pane2button[this._currentChild.id];
				if(b){
					focus.focus(b.focusNode || b.domNode);
				}
			}
		},

		// TODO: this is a bit redundant with forward, back api in StackContainer
		adjacent: function(/*Boolean*/ forward){
			// summary:
			//		Helper for onkeypress to find next/previous button
			// tags:
			//		private

			if(!this.isLeftToRight() && (!this.tabPosition || /top|bottom/.test(this.tabPosition))){ forward = !forward; }
			// find currently focused button in children array
			var children = this.getChildren();
			var current = array.indexOf(children, this.pane2button[this._currentChild.id]);
			// pick next button to focus on
			var offset = forward ? 1 : children.length - 1;
			return children[ (current + offset) % children.length ]; // dijit._Widget
		},

		onkeypress: function(/*Event*/ e){
			// summary:
			//		Handle keystrokes on the page list, for advancing to next/previous button
			//		and closing the current page if the page is closable.
			// tags:
			//		private

			if(this.disabled || e.altKey ){ return; }
			var forward = null;
			if(e.ctrlKey || !e._djpage){
				switch(e.charOrCode){
					case keys.LEFT_ARROW:
					case keys.UP_ARROW:
						if(!e._djpage){ forward = false; }
						break;
					case keys.PAGE_UP:
						if(e.ctrlKey){ forward = false; }
						break;
					case keys.RIGHT_ARROW:
					case keys.DOWN_ARROW:
						if(!e._djpage){ forward = true; }
						break;
					case keys.PAGE_DOWN:
						if(e.ctrlKey){ forward = true; }
						break;
					case keys.HOME:
					case keys.END:
						var children = this.getChildren();
						if(children && children.length){
							children[e.charOrCode == keys.HOME ? 0 : children.length-1].onClick();
						}
						event.stop(e);
						break;
					case keys.DELETE:
						if(this._currentChild.closable){
							this.onCloseButtonClick(this._currentChild);
						}
						event.stop(e);
						break;
					default:
						if(e.ctrlKey){
							if(e.charOrCode === keys.TAB){
								this.adjacent(!e.shiftKey).onClick();
								event.stop(e);
							}else if(e.charOrCode == "w"){
								if(this._currentChild.closable){
									this.onCloseButtonClick(this._currentChild);
								}
								event.stop(e); // avoid browser tab closing.
							}
						}
				}
				// handle next/previous page navigation (left/right arrow, etc.)
				if(forward !== null){
					this.adjacent(forward).onClick();
					event.stop(e);
				}
			}
		},

		onContainerKeyPress: function(/*Object*/ info){
			// summary:
			//		Called when there was a keypress on the container
			// tags:
			//		private
			info.e._djpage = info.page;
			this.onkeypress(info.e);
		}
	});

	StackController.StackButton = StackButton;	// for monkey patching

	return StackController;
});

},
'url:gridx/templates/FilterBar.html':"<input type=\"button\" data-dojo-type=\"dijit.form.Button\" data-dojo-props=\"\n\ticonClass: 'gridxFilterBarBtnIcon',\n\tlabel: '...',\n\ttitle: '${defineFilter}'\" aria-label='${defineFilter}'\n/><div class=\"gridxFilterBarStatus\"\n\t><span>${noFilterApplied}</span\n\t><span class=\"gridxFilterBarCloseBtn\" tabindex=\"-1\" title=\"${closeFilterBarBtn}\"><span class=\"gridxFilterBarCloseBtnText\">x</span></span\n></div>\n",
'dojo/number':function(){
define(["./_base/kernel", "./_base/lang", "./i18n", "./i18n!./cldr/nls/number", "./string", "./regexp"],
	function(dojo, lang, i18n, nlsNumber, dstring, dregexp) {

	// module:
	//		dojo/number
	// summary:
	//		TODOC

lang.getObject("number", true, dojo);

/*=====
dojo.number = {
	// summary: localized formatting and parsing routines for Number
}

dojo.number.__FormatOptions = function(){
	//	pattern: String?
	//		override [formatting pattern](http://www.unicode.org/reports/tr35/#Number_Format_Patterns)
	//		with this string.  Default value is based on locale.  Overriding this property will defeat
	//		localization.  Literal characters in patterns are not supported.
	//	type: String?
	//		choose a format type based on the locale from the following:
	//		decimal, scientific (not yet supported), percent, currency. decimal by default.
	//	places: Number?
	//		fixed number of decimal places to show.  This overrides any
	//		information in the provided pattern.
	//	round: Number?
	//		5 rounds to nearest .5; 0 rounds to nearest whole (default). -1
	//		means do not round.
	//	locale: String?
	//		override the locale used to determine formatting rules
	//	fractional: Boolean?
	//		If false, show no decimal places, overriding places and pattern settings.
	this.pattern = pattern;
	this.type = type;
	this.places = places;
	this.round = round;
	this.locale = locale;
	this.fractional = fractional;
}
=====*/

dojo.number.format = function(/*Number*/value, /*dojo.number.__FormatOptions?*/options){
	// summary:
	//		Format a Number as a String, using locale-specific settings
	// description:
	//		Create a string from a Number using a known localized pattern.
	//		Formatting patterns appropriate to the locale are chosen from the
	//		[Common Locale Data Repository](http://unicode.org/cldr) as well as the appropriate symbols and
	//		delimiters.
	//		If value is Infinity, -Infinity, or is not a valid JavaScript number, return null.
	// value:
	//		the number to be formatted

	options = lang.mixin({}, options || {});
	var locale = i18n.normalizeLocale(options.locale),
		bundle = i18n.getLocalization("dojo.cldr", "number", locale);
	options.customs = bundle;
	var pattern = options.pattern || bundle[(options.type || "decimal") + "Format"];
	if(isNaN(value) || Math.abs(value) == Infinity){ return null; } // null
	return dojo.number._applyPattern(value, pattern, options); // String
};

//dojo.number._numberPatternRE = /(?:[#0]*,?)*[#0](?:\.0*#*)?/; // not precise, but good enough
dojo.number._numberPatternRE = /[#0,]*[#0](?:\.0*#*)?/; // not precise, but good enough

dojo.number._applyPattern = function(/*Number*/value, /*String*/pattern, /*dojo.number.__FormatOptions?*/options){
	// summary:
	//		Apply pattern to format value as a string using options. Gives no
	//		consideration to local customs.
	// value:
	//		the number to be formatted.
	// pattern:
	//		a pattern string as described by
	//		[unicode.org TR35](http://www.unicode.org/reports/tr35/#Number_Format_Patterns)
	// options: dojo.number.__FormatOptions?
	//		_applyPattern is usually called via `dojo.number.format()` which
	//		populates an extra property in the options parameter, "customs".
	//		The customs object specifies group and decimal parameters if set.

	//TODO: support escapes
	options = options || {};
	var group = options.customs.group,
		decimal = options.customs.decimal,
		patternList = pattern.split(';'),
		positivePattern = patternList[0];
	pattern = patternList[(value < 0) ? 1 : 0] || ("-" + positivePattern);

	//TODO: only test against unescaped
	if(pattern.indexOf('%') != -1){
		value *= 100;
	}else if(pattern.indexOf('\u2030') != -1){
		value *= 1000; // per mille
	}else if(pattern.indexOf('\u00a4') != -1){
		group = options.customs.currencyGroup || group;//mixins instead?
		decimal = options.customs.currencyDecimal || decimal;// Should these be mixins instead?
		pattern = pattern.replace(/\u00a4{1,3}/, function(match){
			var prop = ["symbol", "currency", "displayName"][match.length-1];
			return options[prop] || options.currency || "";
		});
	}else if(pattern.indexOf('E') != -1){
		throw new Error("exponential notation not supported");
	}

	//TODO: support @ sig figs?
	var numberPatternRE = dojo.number._numberPatternRE;
	var numberPattern = positivePattern.match(numberPatternRE);
	if(!numberPattern){
		throw new Error("unable to find a number expression in pattern: "+pattern);
	}
	if(options.fractional === false){ options.places = 0; }
	return pattern.replace(numberPatternRE,
		dojo.number._formatAbsolute(value, numberPattern[0], {decimal: decimal, group: group, places: options.places, round: options.round}));
};

dojo.number.round = function(/*Number*/value, /*Number?*/places, /*Number?*/increment){
	//	summary:
	//		Rounds to the nearest value with the given number of decimal places, away from zero
	//	description:
	//		Rounds to the nearest value with the given number of decimal places, away from zero if equal.
	//		Similar to Number.toFixed(), but compensates for browser quirks. Rounding can be done by
	//		fractional increments also, such as the nearest quarter.
	//		NOTE: Subject to floating point errors.  See dojox.math.round for experimental workaround.
	//	value:
	//		The number to round
	//	places:
	//		The number of decimal places where rounding takes place.  Defaults to 0 for whole rounding.
	//		Must be non-negative.
	//	increment:
	//		Rounds next place to nearest value of increment/10.  10 by default.
	//	example:
	//		>>> dojo.number.round(-0.5)
	//		-1
	//		>>> dojo.number.round(162.295, 2)
	//		162.29  // note floating point error.  Should be 162.3
	//		>>> dojo.number.round(10.71, 0, 2.5)
	//		10.75
	var factor = 10 / (increment || 10);
	return (factor * +value).toFixed(places) / factor; // Number
};

if((0.9).toFixed() == 0){
	// (isIE) toFixed() bug workaround: Rounding fails on IE when most significant digit
	// is just after the rounding place and is >=5
	var round = dojo.number.round;
	dojo.number.round = function(v, p, m){
		var d = Math.pow(10, -p || 0), a = Math.abs(v);
		if(!v || a >= d || a * Math.pow(10, p + 1) < 5){
			d = 0;
		}
		return round(v, p, m) + (v > 0 ? d : -d);
	};
}

/*=====
dojo.number.__FormatAbsoluteOptions = function(){
	//	decimal: String?
	//		the decimal separator
	//	group: String?
	//		the group separator
	//	places: Number?|String?
	//		number of decimal places.  the range "n,m" will format to m places.
	//	round: Number?
	//		5 rounds to nearest .5; 0 rounds to nearest whole (default). -1
	//		means don't round.
	this.decimal = decimal;
	this.group = group;
	this.places = places;
	this.round = round;
}
=====*/

dojo.number._formatAbsolute = function(/*Number*/value, /*String*/pattern, /*dojo.number.__FormatAbsoluteOptions?*/options){
	// summary:
	//		Apply numeric pattern to absolute value using options. Gives no
	//		consideration to local customs.
	// value:
	//		the number to be formatted, ignores sign
	// pattern:
	//		the number portion of a pattern (e.g. `#,##0.00`)
	options = options || {};
	if(options.places === true){options.places=0;}
	if(options.places === Infinity){options.places=6;} // avoid a loop; pick a limit

	var patternParts = pattern.split("."),
		comma = typeof options.places == "string" && options.places.indexOf(","),
		maxPlaces = options.places;
	if(comma){
		maxPlaces = options.places.substring(comma + 1);
	}else if(!(maxPlaces >= 0)){
		maxPlaces = (patternParts[1] || []).length;
	}
	if(!(options.round < 0)){
		value = dojo.number.round(value, maxPlaces, options.round);
	}

	var valueParts = String(Math.abs(value)).split("."),
		fractional = valueParts[1] || "";
	if(patternParts[1] || options.places){
		if(comma){
			options.places = options.places.substring(0, comma);
		}
		// Pad fractional with trailing zeros
		var pad = options.places !== undefined ? options.places : (patternParts[1] && patternParts[1].lastIndexOf("0") + 1);
		if(pad > fractional.length){
			valueParts[1] = dstring.pad(fractional, pad, '0', true);
		}

		// Truncate fractional
		if(maxPlaces < fractional.length){
			valueParts[1] = fractional.substr(0, maxPlaces);
		}
	}else{
		if(valueParts[1]){ valueParts.pop(); }
	}

	// Pad whole with leading zeros
	var patternDigits = patternParts[0].replace(',', '');
	pad = patternDigits.indexOf("0");
	if(pad != -1){
		pad = patternDigits.length - pad;
		if(pad > valueParts[0].length){
			valueParts[0] = dstring.pad(valueParts[0], pad);
		}

		// Truncate whole
		if(patternDigits.indexOf("#") == -1){
			valueParts[0] = valueParts[0].substr(valueParts[0].length - pad);
		}
	}

	// Add group separators
	var index = patternParts[0].lastIndexOf(','),
		groupSize, groupSize2;
	if(index != -1){
		groupSize = patternParts[0].length - index - 1;
		var remainder = patternParts[0].substr(0, index);
		index = remainder.lastIndexOf(',');
		if(index != -1){
			groupSize2 = remainder.length - index - 1;
		}
	}
	var pieces = [];
	for(var whole = valueParts[0]; whole;){
		var off = whole.length - groupSize;
		pieces.push((off > 0) ? whole.substr(off) : whole);
		whole = (off > 0) ? whole.slice(0, off) : "";
		if(groupSize2){
			groupSize = groupSize2;
			delete groupSize2;
		}
	}
	valueParts[0] = pieces.reverse().join(options.group || ",");

	return valueParts.join(options.decimal || ".");
};

/*=====
dojo.number.__RegexpOptions = function(){
	//	pattern: String?
	//		override [formatting pattern](http://www.unicode.org/reports/tr35/#Number_Format_Patterns)
	//		with this string.  Default value is based on locale.  Overriding this property will defeat
	//		localization.
	//	type: String?
	//		choose a format type based on the locale from the following:
	//		decimal, scientific (not yet supported), percent, currency. decimal by default.
	//	locale: String?
	//		override the locale used to determine formatting rules
	//	strict: Boolean?
	//		strict parsing, false by default.  Strict parsing requires input as produced by the format() method.
	//		Non-strict is more permissive, e.g. flexible on white space, omitting thousands separators
	//	places: Number|String?
	//		number of decimal places to accept: Infinity, a positive number, or
	//		a range "n,m".  Defined by pattern or Infinity if pattern not provided.
	this.pattern = pattern;
	this.type = type;
	this.locale = locale;
	this.strict = strict;
	this.places = places;
}
=====*/
dojo.number.regexp = function(/*dojo.number.__RegexpOptions?*/options){
	//	summary:
	//		Builds the regular needed to parse a number
	//	description:
	//		Returns regular expression with positive and negative match, group
	//		and decimal separators
	return dojo.number._parseInfo(options).regexp; // String
};

dojo.number._parseInfo = function(/*Object?*/options){
	options = options || {};
	var locale = i18n.normalizeLocale(options.locale),
		bundle = i18n.getLocalization("dojo.cldr", "number", locale),
		pattern = options.pattern || bundle[(options.type || "decimal") + "Format"],
//TODO: memoize?
		group = bundle.group,
		decimal = bundle.decimal,
		factor = 1;

	if(pattern.indexOf('%') != -1){
		factor /= 100;
	}else if(pattern.indexOf('\u2030') != -1){
		factor /= 1000; // per mille
	}else{
		var isCurrency = pattern.indexOf('\u00a4') != -1;
		if(isCurrency){
			group = bundle.currencyGroup || group;
			decimal = bundle.currencyDecimal || decimal;
		}
	}

	//TODO: handle quoted escapes
	var patternList = pattern.split(';');
	if(patternList.length == 1){
		patternList.push("-" + patternList[0]);
	}

	var re = dregexp.buildGroupRE(patternList, function(pattern){
		pattern = "(?:"+dregexp.escapeString(pattern, '.')+")";
		return pattern.replace(dojo.number._numberPatternRE, function(format){
			var flags = {
				signed: false,
				separator: options.strict ? group : [group,""],
				fractional: options.fractional,
				decimal: decimal,
				exponent: false
				},

				parts = format.split('.'),
				places = options.places;

			// special condition for percent (factor != 1)
			// allow decimal places even if not specified in pattern
			if(parts.length == 1 && factor != 1){
			    parts[1] = "###";
			}
			if(parts.length == 1 || places === 0){
				flags.fractional = false;
			}else{
				if(places === undefined){ places = options.pattern ? parts[1].lastIndexOf('0') + 1 : Infinity; }
				if(places && options.fractional == undefined){flags.fractional = true;} // required fractional, unless otherwise specified
				if(!options.places && (places < parts[1].length)){ places += "," + parts[1].length; }
				flags.places = places;
			}
			var groups = parts[0].split(',');
			if(groups.length > 1){
				flags.groupSize = groups.pop().length;
				if(groups.length > 1){
					flags.groupSize2 = groups.pop().length;
				}
			}
			return "("+dojo.number._realNumberRegexp(flags)+")";
		});
	}, true);

	if(isCurrency){
		// substitute the currency symbol for the placeholder in the pattern
		re = re.replace(/([\s\xa0]*)(\u00a4{1,3})([\s\xa0]*)/g, function(match, before, target, after){
			var prop = ["symbol", "currency", "displayName"][target.length-1],
				symbol = dregexp.escapeString(options[prop] || options.currency || "");
			before = before ? "[\\s\\xa0]" : "";
			after = after ? "[\\s\\xa0]" : "";
			if(!options.strict){
				if(before){before += "*";}
				if(after){after += "*";}
				return "(?:"+before+symbol+after+")?";
			}
			return before+symbol+after;
		});
	}

//TODO: substitute localized sign/percent/permille/etc.?

	// normalize whitespace and return
	return {regexp: re.replace(/[\xa0 ]/g, "[\\s\\xa0]"), group: group, decimal: decimal, factor: factor}; // Object
};

/*=====
dojo.number.__ParseOptions = function(){
	//	pattern: String?
	//		override [formatting pattern](http://www.unicode.org/reports/tr35/#Number_Format_Patterns)
	//		with this string.  Default value is based on locale.  Overriding this property will defeat
	//		localization.  Literal characters in patterns are not supported.
	//	type: String?
	//		choose a format type based on the locale from the following:
	//		decimal, scientific (not yet supported), percent, currency. decimal by default.
	//	locale: String?
	//		override the locale used to determine formatting rules
	//	strict: Boolean?
	//		strict parsing, false by default.  Strict parsing requires input as produced by the format() method.
	//		Non-strict is more permissive, e.g. flexible on white space, omitting thousands separators
	//	fractional: Boolean?|Array?
	//		Whether to include the fractional portion, where the number of decimal places are implied by pattern
	//		or explicit 'places' parameter.  The value [true,false] makes the fractional portion optional.
	this.pattern = pattern;
	this.type = type;
	this.locale = locale;
	this.strict = strict;
	this.fractional = fractional;
}
=====*/
dojo.number.parse = function(/*String*/expression, /*dojo.number.__ParseOptions?*/options){
	// summary:
	//		Convert a properly formatted string to a primitive Number, using
	//		locale-specific settings.
	// description:
	//		Create a Number from a string using a known localized pattern.
	//		Formatting patterns are chosen appropriate to the locale
	//		and follow the syntax described by
	//		[unicode.org TR35](http://www.unicode.org/reports/tr35/#Number_Format_Patterns)
    	//		Note that literal characters in patterns are not supported.
	// expression:
	//		A string representation of a Number
	var info = dojo.number._parseInfo(options),
		results = (new RegExp("^"+info.regexp+"$")).exec(expression);
	if(!results){
		return NaN; //NaN
	}
	var absoluteMatch = results[1]; // match for the positive expression
	if(!results[1]){
		if(!results[2]){
			return NaN; //NaN
		}
		// matched the negative pattern
		absoluteMatch =results[2];
		info.factor *= -1;
	}

	// Transform it to something Javascript can parse as a number.  Normalize
	// decimal point and strip out group separators or alternate forms of whitespace
	absoluteMatch = absoluteMatch.
		replace(new RegExp("["+info.group + "\\s\\xa0"+"]", "g"), "").
		replace(info.decimal, ".");
	// Adjust for negative sign, percent, etc. as necessary
	return absoluteMatch * info.factor; //Number
};

/*=====
dojo.number.__RealNumberRegexpFlags = function(){
	//	places: Number?
	//		The integer number of decimal places or a range given as "n,m".  If
	//		not given, the decimal part is optional and the number of places is
	//		unlimited.
	//	decimal: String?
	//		A string for the character used as the decimal point.  Default
	//		is ".".
	//	fractional: Boolean?|Array?
	//		Whether decimal places are used.  Can be true, false, or [true,
	//		false].  Default is [true, false] which means optional.
	//	exponent: Boolean?|Array?
	//		Express in exponential notation.  Can be true, false, or [true,
	//		false]. Default is [true, false], (i.e. will match if the
	//		exponential part is present are not).
	//	eSigned: Boolean?|Array?
	//		The leading plus-or-minus sign on the exponent.  Can be true,
	//		false, or [true, false].  Default is [true, false], (i.e. will
	//		match if it is signed or unsigned).  flags in regexp.integer can be
	//		applied.
	this.places = places;
	this.decimal = decimal;
	this.fractional = fractional;
	this.exponent = exponent;
	this.eSigned = eSigned;
}
=====*/

dojo.number._realNumberRegexp = function(/*dojo.number.__RealNumberRegexpFlags?*/flags){
	// summary:
	//		Builds a regular expression to match a real number in exponential
	//		notation

	// assign default values to missing parameters
	flags = flags || {};
	//TODO: use mixin instead?
	if(!("places" in flags)){ flags.places = Infinity; }
	if(typeof flags.decimal != "string"){ flags.decimal = "."; }
	if(!("fractional" in flags) || /^0/.test(flags.places)){ flags.fractional = [true, false]; }
	if(!("exponent" in flags)){ flags.exponent = [true, false]; }
	if(!("eSigned" in flags)){ flags.eSigned = [true, false]; }

	var integerRE = dojo.number._integerRegexp(flags),
		decimalRE = dregexp.buildGroupRE(flags.fractional,
		function(q){
			var re = "";
			if(q && (flags.places!==0)){
				re = "\\" + flags.decimal;
				if(flags.places == Infinity){
					re = "(?:" + re + "\\d+)?";
				}else{
					re += "\\d{" + flags.places + "}";
				}
			}
			return re;
		},
		true
	);

	var exponentRE = dregexp.buildGroupRE(flags.exponent,
		function(q){
			if(q){ return "([eE]" + dojo.number._integerRegexp({ signed: flags.eSigned}) + ")"; }
			return "";
		}
	);

	var realRE = integerRE + decimalRE;
	// allow for decimals without integers, e.g. .25
	if(decimalRE){realRE = "(?:(?:"+ realRE + ")|(?:" + decimalRE + "))";}
	return realRE + exponentRE; // String
};

/*=====
dojo.number.__IntegerRegexpFlags = function(){
	//	signed: Boolean?
	//		The leading plus-or-minus sign. Can be true, false, or `[true,false]`.
	//		Default is `[true, false]`, (i.e. will match if it is signed
	//		or unsigned).
	//	separator: String?
	//		The character used as the thousands separator. Default is no
	//		separator. For more than one symbol use an array, e.g. `[",", ""]`,
	//		makes ',' optional.
	//	groupSize: Number?
	//		group size between separators
	//	groupSize2: Number?
	//		second grouping, where separators 2..n have a different interval than the first separator (for India)
	this.signed = signed;
	this.separator = separator;
	this.groupSize = groupSize;
	this.groupSize2 = groupSize2;
}
=====*/

dojo.number._integerRegexp = function(/*dojo.number.__IntegerRegexpFlags?*/flags){
	// summary:
	//		Builds a regular expression that matches an integer

	// assign default values to missing parameters
	flags = flags || {};
	if(!("signed" in flags)){ flags.signed = [true, false]; }
	if(!("separator" in flags)){
		flags.separator = "";
	}else if(!("groupSize" in flags)){
		flags.groupSize = 3;
	}

	var signRE = dregexp.buildGroupRE(flags.signed,
		function(q){ return q ? "[-+]" : ""; },
		true
	);

	var numberRE = dregexp.buildGroupRE(flags.separator,
		function(sep){
			if(!sep){
				return "(?:\\d+)";
			}

			sep = dregexp.escapeString(sep);
			if(sep == " "){ sep = "\\s"; }
			else if(sep == "\xa0"){ sep = "\\s\\xa0"; }

			var grp = flags.groupSize, grp2 = flags.groupSize2;
			//TODO: should we continue to enforce that numbers with separators begin with 1-9?  See #6933
			if(grp2){
				var grp2RE = "(?:0|[1-9]\\d{0," + (grp2-1) + "}(?:[" + sep + "]\\d{" + grp2 + "})*[" + sep + "]\\d{" + grp + "})";
				return ((grp-grp2) > 0) ? "(?:" + grp2RE + "|(?:0|[1-9]\\d{0," + (grp-1) + "}))" : grp2RE;
			}
			return "(?:0|[1-9]\\d{0," + (grp-1) + "}(?:[" + sep + "]\\d{" + grp + "})*)";
		},
		true
	);

	return signRE + numberRE; // String
};

return dojo.number;
});

},
'dijit/DialogUnderlay':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.set
	"dojo/_base/window", // win.body
	"dojo/window", // winUtils.getBox
	"./_Widget",
	"./_TemplatedMixin",
	"./BackgroundIframe"
], function(declare, domAttr, win, winUtils, _Widget, _TemplatedMixin, BackgroundIframe){

/*=====
	var _Widget = dijit._Widget;
	var _TemplatedMixin = dijit._TemplatedMixin;
=====*/

	// module:
	//		dijit/DialogUnderlay
	// summary:
	//		The component that blocks the screen behind a `dijit.Dialog`

	return declare("dijit.DialogUnderlay", [_Widget, _TemplatedMixin], {
		// summary:
		//		The component that blocks the screen behind a `dijit.Dialog`
		//
		// description:
		// 		A component used to block input behind a `dijit.Dialog`. Only a single
		//		instance of this widget is created by `dijit.Dialog`, and saved as
		//		a reference to be shared between all Dialogs as `dijit._underlay`
		//
		//		The underlay itself can be styled based on and id:
		//	|	#myDialog_underlay { background-color:red; }
		//
		//		In the case of `dijit.Dialog`, this id is based on the id of the Dialog,
		//		suffixed with _underlay.

		// Template has two divs; outer div is used for fade-in/fade-out, and also to hold background iframe.
		// Inner div has opacity specified in CSS file.
		templateString: "<div class='dijitDialogUnderlayWrapper'><div class='dijitDialogUnderlay' data-dojo-attach-point='node'></div></div>",

		// Parameters on creation or updatable later

		// dialogId: String
		//		Id of the dialog.... DialogUnderlay's id is based on this id
		dialogId: "",

		// class: String
		//		This class name is used on the DialogUnderlay node, in addition to dijitDialogUnderlay
		"class": "",

		_setDialogIdAttr: function(id){
			domAttr.set(this.node, "id", id + "_underlay");
			this._set("dialogId", id);
		},

		_setClassAttr: function(clazz){
			this.node.className = "dijitDialogUnderlay " + clazz;
			this._set("class", clazz);
		},

		postCreate: function(){
			// summary:
			//		Append the underlay to the body
			win.body().appendChild(this.domNode);
		},

		layout: function(){
			// summary:
			//		Sets the background to the size of the viewport
			//
			// description:
			//		Sets the background to the size of the viewport (rather than the size
			//		of the document) since we need to cover the whole browser window, even
			//		if the document is only a few lines long.
			// tags:
			//		private

			var is = this.node.style,
				os = this.domNode.style;

			// hide the background temporarily, so that the background itself isn't
			// causing scrollbars to appear (might happen when user shrinks browser
			// window and then we are called to resize)
			os.display = "none";

			// then resize and show
			var viewport = winUtils.getBox();
			os.top = viewport.t + "px";
			os.left = viewport.l + "px";
			is.width = viewport.w + "px";
			is.height = viewport.h + "px";
			os.display = "block";
		},

		show: function(){
			// summary:
			//		Show the dialog underlay
			this.domNode.style.display = "block";
			this.layout();
			this.bgIframe = new BackgroundIframe(this.domNode);
		},

		hide: function(){
			// summary:
			//		Hides the dialog underlay
			this.bgIframe.destroy();
			delete this.bgIframe;
			this.domNode.style.display = "none";
		}
	});
});

},
'gridx/modules/select/_Base':function(){
define([
	'dojo/_base/declare',
	'dojo/_base/connect',
	'../../core/_Module'
], function(declare, connect, _Module){

	return declare(_Module, {
		getAPIPath: function(){
			var path = {
				select: {}
			};
			path.select[this._type] = this;
			return path;
		},

		preload: function(){
			var t = this, g = t.grid;
			t.subscribe('gridClearSelection_' + g.id, function(type){
				if(type != t._type){
					t.clear();
				}
			});
			t.connect(g.body, 'onRender', '_onRender');
			if(t.arg('multiple')){
				g.domNode.setAttribute('aria-multiselectable', true);
			}
			t._init();
		},

		//Public--------------------------------------------------------------------

		// enabled: Boolean
		//		Whether this module is enabled.
		enabled: true,
	
		// multiple: Boolean
		//		Whether multiple selectionis allowe.
		multiple: true,
	
		// holdingCtrl: Boolean
		//		Whether to add to selection all the time (as if the CTRL key is always held).
		holdingCtrl: false,

		//Events----------------------------------------------------------------------
		onSelected: function(){},

		onDeselected: function(){},

		onHighlightChange: function(){},

		//Private---------------------------------------------------------------------
		
		_getMarkType: function(){},

		_isSelected: function(){
			return this.isSelected.apply(this, arguments);
		},

		_select: function(item, extending){
			var t = this, toSelect = 1;
			if(t.arg('enabled')){
				if(t.arg('multiple') && (extending || t.arg('holdingCtrl'))){
					toSelect = !t._isSelected(item);
				}else{
					t.clear();
				}
				connect.publish('gridClearSelection_' + t.grid.id, [t._type]);
				t._markById(item, toSelect);
			}
		}
	});
});

},
'dijit/_WidgetsInTemplateMixin':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/parser", // parser.parse
	"dijit/registry"	// registry.findWidgets
], function(array, declare, parser, registry){

	// module:
	//		dijit/_WidgetsInTemplateMixin
	// summary:
	//		Mixin to supplement _TemplatedMixin when template contains widgets

	return declare("dijit._WidgetsInTemplateMixin", null, {
		// summary:
		//		Mixin to supplement _TemplatedMixin when template contains widgets

		// _earlyTemplatedStartup: Boolean
		//		A fallback to preserve the 1.0 - 1.3 behavior of children in
		//		templates having their startup called before the parent widget
		//		fires postCreate. Defaults to 'false', causing child widgets to
		//		have their .startup() called immediately before a parent widget
		//		.startup(), but always after the parent .postCreate(). Set to
		//		'true' to re-enable to previous, arguably broken, behavior.
		_earlyTemplatedStartup: false,

		// widgetsInTemplate: [protected] Boolean
		//		Should we parse the template to find widgets that might be
		//		declared in markup inside it?  (Remove for 2.0 and assume true)
		widgetsInTemplate: true,

		_beforeFillContent: function(){
			if(this.widgetsInTemplate){
				// Before copying over content, instantiate widgets in template
				var node = this.domNode;

				var cw = (this._startupWidgets = parser.parse(node, {
					noStart: !this._earlyTemplatedStartup,
					template: true,
					inherited: {dir: this.dir, lang: this.lang, textDir: this.textDir},
					propsThis: this,	// so data-dojo-props of widgets in the template can reference "this" to refer to me
					scope: "dojo"	// even in multi-version mode templates use dojoType/data-dojo-type
				}));

				this._supportingWidgets = registry.findWidgets(node);

				this._attachTemplateNodes(cw, function(n,p){
					return n[p];
				});
			}
		},

		startup: function(){
			array.forEach(this._startupWidgets, function(w){
				if(w && !w._started && w.startup){
					w.startup();
				}
			});
			this.inherited(arguments);
		}
	});
});

},
'url:gridx/templates/PaginationBar.html':"<div class='gridxPager' role=\"navigation\" aria-label='${id} ${pagerWai}'\n\t><table class='gridxPagerInner' cellpadding=\"0\" cellspacing=\"0\" role=\"presentation\"\n\t\t><tr\n\t\t\t><td class=\"gridxPagerDescriptionTD\"\n\t\t\t\t><div class=\"gridxPagerDescription\"\n\t\t\t\t\tdata-dojo-attach-point=\"_descContainer\"\n\t\t\t\t></div\n\t\t\t></td\n\t\t\t><td class=\"gridxPagerStepperTD\"\n\t\t\t\t><div class=\"gridxPagerStepper\"\n\t\t\t\t\trole=\"toolbar\"\n\t\t\t\t\tdata-dojo-attach-point=\"_pageStepperContainer\"\n\t\t\t\t\t><span class='gridxPagerStepperBtn gridxPagerPrevPage'\n\t\t\t\t\t\ttabindex='${_tabIndex}'\n\t\t\t\t\t\ttitle='${prevPageTitle}'\n\t\t\t\t\t\taria-label='${prevPageTitle}'\n\t\t\t\t\t\trole=\"button\"\n\t\t\t\t\t\tpageindex='Prev'\n\t\t\t\t\t\tdata-dojo-attach-point='_prevPageBtn'\n\t\t\t\t\t\tdata-dojo-attach-event='onclick: _gotoPrevPage'\n\t\t\t\t\t\t><span class='gridxPagerA11yInner'>&nbsp;&lt;</span\n\t\t\t\t\t></span\n\t\t\t\t\t><span class='gridxPagerPages'\n\t\t\t\t\t\tdata-dojo-attach-point='_pageBtnContainer'\n\t\t\t\t\t\tdata-dojo-attach-event='onclick: _gotoPage, onmouseover: _onHoverPageBtn, onmouseout: _onHoverPageBtn'\n\t\t\t\t\t></span\n\t\t\t\t\t><span class='gridxPagerStepperBtn gridxPagerNextPage'\n\t\t\t\t\t\ttabindex='${_tabIndex}'\n\t\t\t\t\t\ttitle='${nextPageTitle}'\n\t\t\t\t\t\taria-label='${nextPageTitle}'\n\t\t\t\t\t\tpageindex='Next'\n\t\t\t\t\t\tdata-dojo-attach-point='_nextPageBtn'\n\t\t\t\t\t\tdata-dojo-attach-event='onclick: _gotoNextPage'\n\t\t\t\t\t\t><span class='gridxPagerA11yInner'>&nbsp;&gt;</span\n\t\t\t\t\t></span\n\t\t\t\t></div\n\t\t\t></td\n\t\t\t><td class=\"gridxPagerSizeSwitchTD\"\n\t\t\t\t><div class=\"gridxPagerSizeSwitch\"\n\t\t\t\t\trole=\"toolbar\"\n\t\t\t\t\tdata-dojo-attach-point='_sizeSwitchContainer'\n\t\t\t\t\tdata-dojo-attach-event='onclick: _switchPageSize, onmouseover: _onHoverSizeBtn, onmouseout: _onHoverSizeBtn'\n\t\t\t\t></div\n\t\t\t></td\n\t\t\t><td class='gridxPagerGoto'\n\t\t\t\t><span class='gridxPagerGotoBtn'\n\t\t\t\t\ttabindex='${_tabIndex}'\n\t\t\t\t\ttitle='${gotoBtnTitle}'\n\t\t\t\t\trole='button'\n\t\t\t\t\taria-label='${gotoBtnTitle}'\n\t\t\t\t\tdata-dojo-attach-point='_gotoBtn'\n\t\t\t\t\tdata-dojo-attach-event='onclick: _showGotoDialog'\n\t\t\t\t\t><span class='gridxPagerA11yInner'>&#9650;</span\n\t\t\t\t></span\n\t\t\t></td\n\t\t></tr\n\t></table\n></div>\n",
'dojo/DeferredList':function(){
define(["./_base/kernel", "./_base/Deferred", "./_base/array"], function(dojo, Deferred, darray) {
	// module:
	//		dojo/DeferredList
	// summary:
	//		TODOC


dojo.DeferredList = function(/*Array*/ list, /*Boolean?*/ fireOnOneCallback, /*Boolean?*/ fireOnOneErrback, /*Boolean?*/ consumeErrors, /*Function?*/ canceller){
	// summary:
	//		Provides event handling for a group of Deferred objects.
	// description:
	//		DeferredList takes an array of existing deferreds and returns a new deferred of its own
	//		this new deferred will typically have its callback fired when all of the deferreds in
	//		the given list have fired their own deferreds.  The parameters `fireOnOneCallback` and
	//		fireOnOneErrback, will fire before all the deferreds as appropriate
	//
	// list:
	//		The list of deferreds to be synchronizied with this DeferredList
	// fireOnOneCallback:
	//		Will cause the DeferredLists callback to be fired as soon as any
	//		of the deferreds in its list have been fired instead of waiting until
	//		the entire list has finished
	// fireonOneErrback:
	//		Will cause the errback to fire upon any of the deferreds errback
	// canceller:
	//		A deferred canceller function, see dojo.Deferred
	var resultList = [];
	Deferred.call(this);
	var self = this;
	if(list.length === 0 && !fireOnOneCallback){
		this.resolve([0, []]);
	}
	var finished = 0;
	darray.forEach(list, function(item, i){
		item.then(function(result){
			if(fireOnOneCallback){
				self.resolve([i, result]);
			}else{
				addResult(true, result);
			}
		},function(error){
			if(fireOnOneErrback){
				self.reject(error);
			}else{
				addResult(false, error);
			}
			if(consumeErrors){
				return null;
			}
			throw error;
		});
		function addResult(succeeded, result){
			resultList[i] = [succeeded, result];
			finished++;
			if(finished === list.length){
				self.resolve(resultList);
			}

		}
	});
};
dojo.DeferredList.prototype = new Deferred();

dojo.DeferredList.prototype.gatherResults = function(deferredList){
	// summary:
	//		Gathers the results of the deferreds for packaging
	//		as the parameters to the Deferred Lists' callback
	// deferredList: dojo.DeferredList
	//		The deferred list from which this function gathers results.
	// returns: dojo.DeferredList
	//		The newly created deferred list which packs results as
	//		parameters to its callback.

	var d = new dojo.DeferredList(deferredList, false, true, false);
	d.addCallback(function(results){
		var ret = [];
		darray.forEach(results, function(result){
			ret.push(result[1]);
		});
		return ret;
	});
	return d;
};

return dojo.DeferredList;
});

},
'url:dijit/form/templates/TextBox.html':"<div class=\"dijit dijitReset dijitInline dijitLeft\" id=\"widget_${id}\" role=\"presentation\"\n\t><div class=\"dijitReset dijitInputField dijitInputContainer\"\n\t\t><input class=\"dijitReset dijitInputInner\" data-dojo-attach-point='textbox,focusNode' autocomplete=\"off\"\n\t\t\t${!nameAttrSetting} type='${type}'\n\t/></div\n></div>\n",
'url:ibm/usmi/dashboard/templates/ResourceGroupWidget.html':"<div class=\"dashboard-group-component\">\r\n<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n\t<div id=\"groupPod\" data-dojo-attach-point=\"groupPod\">\r\n\t\t<div data-dojo-attach-point=\"image\" role=\"button\"></div>\r\n\t\t<div data-dojo-attach-point=\"footer\">\r\n\t\t\t<span data-dojo-attach-point=\"status\"></span>\r\n\t\t\t<span data-dojo-attach-point=\"label\" role=\"button\" class=\"dashboard-head-componentLabel\">\r\n\t\t\t</span>\r\n\t\t</div>\r\n\t</div>\r\n</div>\r\n",
'dojo/_base/url':function(){
define(["./kernel"], function(dojo) {
	// module:
	//		dojo/url
	// summary:
	//		This module contains dojo._Url

	var
		ore = new RegExp("^(([^:/?#]+):)?(//([^/?#]*))?([^?#]*)(\\?([^#]*))?(#(.*))?$"),
		ire = new RegExp("^((([^\\[:]+):)?([^@]+)@)?(\\[([^\\]]+)\\]|([^\\[:]*))(:([0-9]+))?$"),
		_Url = function(){
			var n = null,
				_a = arguments,
				uri = [_a[0]];
			// resolve uri components relative to each other
			for(var i = 1; i<_a.length; i++){
				if(!_a[i]){ continue; }

				// Safari doesn't support this.constructor so we have to be explicit
				// FIXME: Tracked (and fixed) in Webkit bug 3537.
				//		http://bugs.webkit.org/show_bug.cgi?id=3537
				var relobj = new _Url(_a[i]+""),
					uriobj = new _Url(uri[0]+"");

				if(
					relobj.path == "" &&
					!relobj.scheme &&
					!relobj.authority &&
					!relobj.query
				){
					if(relobj.fragment != n){
						uriobj.fragment = relobj.fragment;
					}
					relobj = uriobj;
				}else if(!relobj.scheme){
					relobj.scheme = uriobj.scheme;

					if(!relobj.authority){
						relobj.authority = uriobj.authority;

						if(relobj.path.charAt(0) != "/"){
							var path = uriobj.path.substring(0,
								uriobj.path.lastIndexOf("/") + 1) + relobj.path;

							var segs = path.split("/");
							for(var j = 0; j < segs.length; j++){
								if(segs[j] == "."){
									// flatten "./" references
									if(j == segs.length - 1){
										segs[j] = "";
									}else{
										segs.splice(j, 1);
										j--;
									}
								}else if(j > 0 && !(j == 1 && segs[0] == "") &&
									segs[j] == ".." && segs[j-1] != ".."){
									// flatten "../" references
									if(j == (segs.length - 1)){
										segs.splice(j, 1);
										segs[j - 1] = "";
									}else{
										segs.splice(j - 1, 2);
										j -= 2;
									}
								}
							}
							relobj.path = segs.join("/");
						}
					}
				}

				uri = [];
				if(relobj.scheme){
					uri.push(relobj.scheme, ":");
				}
				if(relobj.authority){
					uri.push("//", relobj.authority);
				}
				uri.push(relobj.path);
				if(relobj.query){
					uri.push("?", relobj.query);
				}
				if(relobj.fragment){
					uri.push("#", relobj.fragment);
				}
			}

			this.uri = uri.join("");

			// break the uri into its main components
			var r = this.uri.match(ore);

			this.scheme = r[2] || (r[1] ? "" : n);
			this.authority = r[4] || (r[3] ? "" : n);
			this.path = r[5]; // can never be undefined
			this.query = r[7] || (r[6] ? "" : n);
			this.fragment	 = r[9] || (r[8] ? "" : n);

			if(this.authority != n){
				// server based naming authority
				r = this.authority.match(ire);

				this.user = r[3] || n;
				this.password = r[4] || n;
				this.host = r[6] || r[7]; // ipv6 || ipv4
				this.port = r[9] || n;
			}
		};
	_Url.prototype.toString = function(){ return this.uri; };

	return dojo._Url = _Url;
});

},
'dojo/hash':function(){
define(["./_base/kernel", "require", "./_base/connect", "./_base/lang", "./ready", "./_base/sniff"],
	function(dojo, require, connect, lang, ready, has) {
	// module:
	//		dojo/hash
	// summary:
	//		TODOC


//TODOC: where does this go?
// summary:
//		Methods for monitoring and updating the hash in the browser URL.
//
// example:
//		dojo.subscribe("/dojo/hashchange", context, callback);
//
//		function callback (hashValue){
//			// do something based on the hash value.
//		}

	dojo.hash = function(/* String? */ hash, /* Boolean? */ replace){
		//	summary:
		//		Gets or sets the hash string.
		//	description:
		//		Handles getting and setting of location.hash.
		//		 - If no arguments are passed, acts as a getter.
		//		 - If a string is passed, acts as a setter.
		//	hash:
		//		the hash is set - #string.
		//	replace:
		//		If true, updates the hash value in the current history
		//		state instead of creating a new history state.
		//	returns:
		//		when used as a getter, returns the current hash string.
		//		when used as a setter, returns the new hash string.

		// getter
		if(!arguments.length){
			return _getHash();
		}
		// setter
		if(hash.charAt(0) == "#"){
			hash = hash.substring(1);
		}
		if(replace){
			_replace(hash);
		}else{
			location.href = "#" + hash;
		}
		return hash; // String
	};

	// Global vars
	var _recentHash, _ieUriMonitor, _connect,
		_pollFrequency = dojo.config.hashPollFrequency || 100;

	//Internal functions
	function _getSegment(str, delimiter){
		var i = str.indexOf(delimiter);
		return (i >= 0) ? str.substring(i+1) : "";
	}

	function _getHash(){
		return _getSegment(location.href, "#");
	}

	function _dispatchEvent(){
		connect.publish("/dojo/hashchange", [_getHash()]);
	}

	function _pollLocation(){
		if(_getHash() === _recentHash){
			return;
		}
		_recentHash = _getHash();
		_dispatchEvent();
	}

	function _replace(hash){
		if(_ieUriMonitor){
			if(_ieUriMonitor.isTransitioning()){
				setTimeout(lang.hitch(null,_replace,hash), _pollFrequency);
				return;
			}
			var href = _ieUriMonitor.iframe.location.href;
			var index = href.indexOf('?');
			// main frame will detect and update itself
			_ieUriMonitor.iframe.location.replace(href.substring(0, index) + "?" + hash);
			return;
		}
		location.replace("#"+hash);
		!_connect && _pollLocation();
	}

	function IEUriMonitor(){
		// summary:
		//		Determine if the browser's URI has changed or if the user has pressed the
		//		back or forward button. If so, call _dispatchEvent.
		//
		//	description:
		//		IE doesn't add changes to the URI's hash into the history unless the hash
		//		value corresponds to an actual named anchor in the document. To get around
		//		this IE difference, we use a background IFrame to maintain a back-forward
		//		history, by updating the IFrame's query string to correspond to the
		//		value of the main browser location's hash value.
		//
		//		E.g. if the value of the browser window's location changes to
		//
		//		#action=someAction
		//
		//		... then we'd update the IFrame's source to:
		//
		//		?action=someAction
		//
		//		This design leads to a somewhat complex state machine, which is
		//		described below:
		//
		//		s1: Stable state - neither the window's location has changed nor
		//			has the IFrame's location. Note that this is the 99.9% case, so
		//			we optimize for it.
		//			Transitions: s1, s2, s3
		//		s2: Window's location changed - when a user clicks a hyperlink or
		//			code programmatically changes the window's URI.
		//			Transitions: s4
		//		s3: Iframe's location changed as a result of user pressing back or
		//			forward - when the user presses back or forward, the location of
		//			the background's iframe changes to the previous or next value in
		//			its history.
		//			Transitions: s1
		//		s4: IEUriMonitor has programmatically changed the location of the
		//			background iframe, but it's location hasn't yet changed. In this
		//			case we do nothing because we need to wait for the iframe's
		//			location to reflect its actual state.
		//			Transitions: s4, s5
		//		s5: IEUriMonitor has programmatically changed the location of the
		//			background iframe, and the iframe's location has caught up with
		//			reality. In this case we need to transition to s1.
		//			Transitions: s1
		//
		//		The hashchange event is always dispatched on the transition back to s1.
		//

		// create and append iframe
		var ifr = document.createElement("iframe"),
			IFRAME_ID = "dojo-hash-iframe",
			ifrSrc = dojo.config.dojoBlankHtmlUrl || require.toUrl("./resources/blank.html");

		if(dojo.config.useXDomain && !dojo.config.dojoBlankHtmlUrl){
			console.warn("dojo.hash: When using cross-domain Dojo builds,"
				+ " please save dojo/resources/blank.html to your domain and set djConfig.dojoBlankHtmlUrl"
				+ " to the path on your domain to blank.html");
		}

		ifr.id = IFRAME_ID;
		ifr.src = ifrSrc + "?" + _getHash();
		ifr.style.display = "none";
		document.body.appendChild(ifr);

		this.iframe = dojo.global[IFRAME_ID];
		var recentIframeQuery, transitioning, expectedIFrameQuery, docTitle, ifrOffline,
			iframeLoc = this.iframe.location;

		function resetState(){
			_recentHash = _getHash();
			recentIframeQuery = ifrOffline ? _recentHash : _getSegment(iframeLoc.href, "?");
			transitioning = false;
			expectedIFrameQuery = null;
		}

		this.isTransitioning = function(){
			return transitioning;
		};

		this.pollLocation = function(){
			if(!ifrOffline) {
				try{
					//see if we can access the iframe's location without a permission denied error
					var iframeSearch = _getSegment(iframeLoc.href, "?");
					//good, the iframe is same origin (no thrown exception)
					if(document.title != docTitle){ //sync title of main window with title of iframe.
						docTitle = this.iframe.document.title = document.title;
					}
				}catch(e){
					//permission denied - server cannot be reached.
					ifrOffline = true;
					console.error("dojo.hash: Error adding history entry. Server unreachable.");
				}
			}
			var hash = _getHash();
			if(transitioning && _recentHash === hash){
				// we're in an iframe transition (s4 or s5)
				if(ifrOffline || iframeSearch === expectedIFrameQuery){
					// s5 (iframe caught up to main window or iframe offline), transition back to s1
					resetState();
					_dispatchEvent();
				}else{
					// s4 (waiting for iframe to catch up to main window)
					setTimeout(lang.hitch(this,this.pollLocation),0);
					return;
				}
			}else if(_recentHash === hash && (ifrOffline || recentIframeQuery === iframeSearch)){
				// we're in stable state (s1, iframe query == main window hash), do nothing
			}else{
				// the user has initiated a URL change somehow.
				// sync iframe query <-> main window hash
				if(_recentHash !== hash){
					// s2 (main window location changed), set iframe url and transition to s4
					_recentHash = hash;
					transitioning = true;
					expectedIFrameQuery = hash;
					ifr.src = ifrSrc + "?" + expectedIFrameQuery;
					ifrOffline = false; //we're updating the iframe src - set offline to false so we can check again on next poll.
					setTimeout(lang.hitch(this,this.pollLocation),0); //yielded transition to s4 while iframe reloads.
					return;
				}else if(!ifrOffline){
					// s3 (iframe location changed via back/forward button), set main window url and transition to s1.
					location.href = "#" + iframeLoc.search.substring(1);
					resetState();
					_dispatchEvent();
				}
			}
			setTimeout(lang.hitch(this,this.pollLocation), _pollFrequency);
		};
		resetState(); // initialize state (transition to s1)
		setTimeout(lang.hitch(this,this.pollLocation), _pollFrequency);
	}
	ready(function(){
		if("onhashchange" in dojo.global && (!has("ie") || (has("ie") >= 8 && document.compatMode != "BackCompat"))){	//need this IE browser test because "onhashchange" exists in IE8 in IE7 mode
			_connect = connect.connect(dojo.global,"onhashchange",_dispatchEvent);
		}else{
			if(document.addEventListener){ // Non-IE
				_recentHash = _getHash();
				setInterval(_pollLocation, _pollFrequency); //Poll the window location for changes
			}else if(document.attachEvent){ // IE7-
				//Use hidden iframe in versions of IE that don't have onhashchange event
				_ieUriMonitor = new IEUriMonitor();
			}
			// else non-supported browser, do nothing.
		}
	});

	return dojo.hash;

});

},
'url:dojoe/treetablex/templates/QuickSearchFilter.html':"<div class=\"dijitReset  dijitTextBox quickFilter\" dojoAttachEvent='onmouseover:_onHover,mouseleave:_onUnHover,onblur:_onBlur,onclick: _onClick'>\r\n\t<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" width=\"100%\" role=\"presentation\">\r\n\t\t<tbody>\r\n\t\t\t<tr valign=\"center\">\r\n\t\t\t\t<td width=\"98%\" height=\"100%\">\r\n\t\t\t\t\t<div class=\"dijitReset  dijitInputContainer\">\t\t    \r\n\t\t\t\t\t\t<input dojoAttachPoint=\"searchField\" title=\"${_resources.QUICK_FILTER}\" class=\"dijitInputInner qf-textfield\" type=\"text\" \r\n\t\t\t\t\t\t\tvalue=\"${_resources.QF_DEFAULT_TEXT}\" valign=\"middle\" tabindex=\"0\"\r\n\t\t\t\t\t\t\tdojoAttachEvent='onfocus:_onFocus,onclick:_onClick,onkeyup:_onKeyUp,onpaste:_onKeyUp,oncut:_onKeyUp,onKeyPress:_onKeyUp' role=\"textbox\"/>\t\r\n\t\t\t\t\t</div>\r\n\t\t\t\t</td>\t\r\n\t\t\t\t<td width=\"2%\" height=\"100%\">\r\n\t\t\t\t\t<div class=\"dijitReset  dijitArrowButtonContainer \">\r\n\t\t\t\t\t\t<button title=\"${_resources.CLEAR_FILTER}\" class=\"dijitReset quickFilterClearInner\" type=\"button\" tabindex=\"0\" \r\n\t\t\t\t\t\t\tvalign=\"middle\"  dojoAttachPoint='clearFilterNode' dojoAttachEvent='onclick:_onCancel,onkeypress:_onCancel' role=\"button\"></button>\t\t\t\r\n\t\t\t\t\t</div>\r\n\t\t\t\t</td>\r\n\t\t\t</tr>\r\n\t\t</tbody>\t\t\r\n\t</table>\r\n</div>",
'dojox/layout/ResizeHandle':function(){
define(["dojo/_base/kernel","dojo/_base/lang","dojo/_base/connect","dojo/_base/array","dojo/_base/event",
	"dojo/_base/fx","dojo/_base/window","dojo/fx","dojo/window","dojo/dom","dojo/dom-class",
	"dojo/dom-geometry","dojo/dom-style","dijit/_base/manager","dijit/_Widget","dijit/_TemplatedMixin",
	"dojo/_base/declare"], function (
	kernel, lang, connect, arrayUtil, eventUtil, fxBase, windowBase, fxUtil, windowUtil, 
	domUtil, domClass, domGeometry, domStyle, manager, Widget, TemplatedMixin, declare) {

kernel.experimental("dojox.layout.ResizeHandle");

/*===== 
	var Widget = dijit._Widget;
	var TemplatedMixin = dijit._TemplatedMixin;
=====*/
var ResizeHandle = declare("dojox.layout.ResizeHandle",[Widget, TemplatedMixin],
	{
	// summary: A dragable handle used to resize an attached node.
	//
	// description:
	//	The handle on the bottom-right corner of FloatingPane or other widgets that allows
	//	the widget to be resized.
	//	Typically not used directly.
	//
	// targetId: String
	//	id of the Widget OR DomNode that I will size
	targetId: "",
	
	// targetContainer: DomNode
	//	over-ride targetId and attch this handle directly to a reference of a DomNode
	targetContainer: null,
	
	// resizeAxis: String
	//	one of: x|y|xy limit resizing to a single axis, default to xy ...
	resizeAxis: "xy",
	
	// activeResize: Boolean
	// 	if true, node will size realtime with mouse movement,
	//	if false, node will create virtual node, and only resize target on mouseUp
	activeResize: false,
	
	// activeResizeClass: String
	//	css class applied to virtual resize node.
	activeResizeClass: "dojoxResizeHandleClone",
	
	// animateSizing: Boolean
	//	only applicable if activeResize = false. onMouseup, animate the node to the
	//	new size
	animateSizing: true,
	
	// animateMethod: String
	// 	one of "chain" or "combine" ... visual effect only. combine will "scale"
	// 	node to size, "chain" will alter width, then height
	animateMethod: "chain",

	// animateDuration: Integer
	//	time in MS to run sizing animation. if animateMethod="chain", total animation
	//	playtime is 2*animateDuration
	animateDuration: 225,

	// minHeight: Integer
	//	smallest height in px resized node can be
	minHeight: 100,

	// minWidth: Integer
	//	smallest width in px resize node can be
	minWidth: 100,

	// constrainMax: Boolean
	//	Toggle if this widget cares about the maxHeight and maxWidth
	//	parameters.
	constrainMax: false,

	// maxHeight: Integer
	//	Largest height size in px the resize node can become.
	maxHeight:0,
	
	// maxWidth: Integer
	//	Largest width size in px the reize node can become.
	maxWidth:0,

	// fixedAspect: Boolean
	//		Toggle to enable this widget to maintain the aspect
	//		ratio of the attached node.
	fixedAspect: false,

	// intermediateChanges: Boolean
	//		Toggle to enable/disable this widget from firing onResize
	//		events at every step of a resize. If `activeResize` is true,
	//		and this is false, onResize only fires _after_ the drop
	//		operation. Animated resizing is not affected by this setting.
	intermediateChanges: false,

	// startTopic: String
	//		The name of the topic this resizehandle publishes when resize is starting
	startTopic: "/dojo/resize/start",
	
	// endTopic: String
	//		The name of the topic this resizehandle publishes when resize is complete
	endTopic:"/dojo/resize/stop",

	templateString: '<div dojoAttachPoint="resizeHandle" class="dojoxResizeHandle"><div></div></div>',

	postCreate: function(){
		// summary: setup our one major listener upon creation
		this.connect(this.resizeHandle, "onmousedown", "_beginSizing");
		if(!this.activeResize){
			// there shall be only a single resize rubberbox that at the top
			// level so that we can overlay it on anything whenever the user
			// resizes something. Since there is only one mouse pointer he
			// can't at once resize multiple things interactively.
			this._resizeHelper = manager.byId('dojoxGlobalResizeHelper');
			if(!this._resizeHelper){
				this._resizeHelper = new _ResizeHelper({
						id: 'dojoxGlobalResizeHelper'
				}).placeAt(windowBase.body());
				domClass.add(this._resizeHelper.domNode, this.activeResizeClass);
			}
		}else{ this.animateSizing = false; }

		if(!this.minSize){
			this.minSize = { w: this.minWidth, h: this.minHeight };
		}
		
		if(this.constrainMax){
			this.maxSize = { w: this.maxWidth, h: this.maxHeight }
		}
		
		// should we modify the css for the cursor hover to n-resize nw-resize and w-resize?
		this._resizeX = this._resizeY = false;
		var addClass = lang.partial(domClass.add, this.resizeHandle);
		switch(this.resizeAxis.toLowerCase()){
			case "xy" :
				this._resizeX = this._resizeY = true;
				// FIXME: need logic to determine NW or NE class to see
				// based on which [todo] corner is clicked
				addClass("dojoxResizeNW");
				break;
			case "x" :
				this._resizeX = true;
				addClass("dojoxResizeW");
				break;
			case "y" :
				this._resizeY = true;
				addClass("dojoxResizeN");
				break;
		}
	},

	_beginSizing: function(/*Event*/ e){
		// summary: setup movement listeners and calculate initial size
		
		if(this._isSizing){ return; }

		connect.publish(this.startTopic, [ this ]);
		this.targetWidget = manager.byId(this.targetId);

		this.targetDomNode = this.targetWidget ? this.targetWidget.domNode : domUtil.byId(this.targetId);
		if(this.targetContainer){ this.targetDomNode = this.targetContainer; }
		if(!this.targetDomNode){ return; }

		if(!this.activeResize){
			var c = domGeometry.position(this.targetDomNode, true);
			this._resizeHelper.resize({l: c.x, t: c.y, w: c.w, h: c.h});
			this._resizeHelper.show();
			if(!this.isLeftToRight()){
				this._resizeHelper.startPosition = {l: c.x, t: c.y};
			}
		}

		this._isSizing = true;
		this.startPoint  = { x:e.clientX, y:e.clientY };

		// widget.resize() or setting style.width/height expects native box model dimension 
		// (in most cases content-box, but it may be border-box if in backcompact mode)
		var style = domStyle.getComputedStyle(this.targetDomNode), 
			borderModel = domGeometry.boxModel==='border-model',
			padborder = borderModel?{w:0,h:0}:domGeometry.getPadBorderExtents(this.targetDomNode, style),
			margin = domGeometry.getMarginExtents(this.targetDomNode, style),
			mb;
		mb = this.startSize = { 
				w: domStyle.get(this.targetDomNode, 'width', style), 
				h: domStyle.get(this.targetDomNode, 'height', style),
				//ResizeHelper.resize expects a bounding box of the
				//border box, so let's keep track of padding/border
				//width/height as well
				pbw: padborder.w, pbh: padborder.h,
				mw: margin.w, mh: margin.h};
		if(!this.isLeftToRight() && dojo.style(this.targetDomNode, "position") == "absolute"){
			var p = domGeometry.position(this.targetDomNode, true);
			this.startPosition = {l: p.x, t: p.y};
		}
		
		this._pconnects = [
			connect.connect(windowBase.doc,"onmousemove",this,"_updateSizing"),
			connect.connect(windowBase.doc,"onmouseup", this, "_endSizing")
		];
		
		eventUtil.stop(e);
	},

	_updateSizing: function(/*Event*/ e){
		// summary: called when moving the ResizeHandle ... determines
		//	new size based on settings/position and sets styles.

		if(this.activeResize){
			this._changeSizing(e);
		}else{
			var tmp = this._getNewCoords(e, 'border', this._resizeHelper.startPosition);
			if(tmp === false){ return; }
			this._resizeHelper.resize(tmp);
		}
		e.preventDefault();
	},

	_getNewCoords: function(/* Event */ e, /* String */ box, /* Object */startPosition){
		
		// On IE, if you move the mouse above/to the left of the object being resized,
		// sometimes clientX/Y aren't set, apparently.  Just ignore the event.
		try{
			if(!e.clientX  || !e.clientY){ return false; }
		}catch(e){
			// sometimes you get an exception accessing above fields...
			return false;
		}
		this._activeResizeLastEvent = e;

		var dx = (this.isLeftToRight()?1:-1) * (this.startPoint.x - e.clientX),
			dy = this.startPoint.y - e.clientY,
			newW = this.startSize.w - (this._resizeX ? dx : 0),
			newH = this.startSize.h - (this._resizeY ? dy : 0),
			r = this._checkConstraints(newW, newH)
		;
		
		startPosition = (startPosition || this.startPosition);
		if(startPosition && this._resizeX){
			// adjust x position for RtoL
			r.l = startPosition.l + dx;
			if(r.w != newW){
				r.l += (newW - r.w);
			}
			r.t = startPosition.t;
		}

		switch(box){
			case 'margin':
				r.w += this.startSize.mw;
				r.h += this.startSize.mh;
				//pass through
			case "border":
				r.w += this.startSize.pbw;
				r.h += this.startSize.pbh;
				break;
			//default: //native, do nothing
		}

		return r; // Object
	},
	
	_checkConstraints: function(newW, newH){
		// summary: filter through the various possible constaint possibilities.
				
		// minimum size check
		if(this.minSize){
			var tm = this.minSize;
			if(newW < tm.w){
				newW = tm.w;
			}
			if(newH < tm.h){
				newH = tm.h;
			}
		}
		
		// maximum size check:
		if(this.constrainMax && this.maxSize){
			var ms = this.maxSize;
			if(newW > ms.w){
				newW = ms.w;
			}
			if(newH > ms.h){
				newH = ms.h;
			}
		}
		
		if(this.fixedAspect){
			var w = this.startSize.w, h = this.startSize.h,
				delta = w * newH - h * newW;
			if(delta<0){
				newW = newH * w / h;
			}else if(delta>0){
				newH = newW * h / w;
			}
		}
		
		return { w: newW, h: newH }; // Object
	},
		
	_changeSizing: function(/*Event*/ e){
		// summary: apply sizing information based on information in (e) to attached node
		
		var isWidget = this.targetWidget && lang.isFunction(this.targetWidget.resize),
			tmp = this._getNewCoords(e, isWidget && 'margin');
		if(tmp === false){ return; }

		if(isWidget){
			this.targetWidget.resize(tmp);
		}else{
			if(this.animateSizing){
				var anim = fxUtil[this.animateMethod]([
					fxBase.animateProperty({
						node: this.targetDomNode,
						properties: {
							width: { start: this.startSize.w, end: tmp.w }
						},
						duration: this.animateDuration
					}),
					fxBase.animateProperty({
						node: this.targetDomNode,
						properties: {
							height: { start: this.startSize.h, end: tmp.h }
						},
						duration: this.animateDuration
					})
				]);
				anim.play();
			}else{
				domStyle.set(this.targetDomNode,{
					width: tmp.w + "px",
					height: tmp.h + "px"
				});
			}
		}
		if(this.intermediateChanges){
			this.onResize(e);
		}
	},

	_endSizing: function(/*Event*/ e){
		// summary: disconnect listenrs and cleanup sizing
		arrayUtil.forEach(this._pconnects, connect.disconnect);
		var pub = lang.partial(connect.publish, this.endTopic, [ this ]);
		if(!this.activeResize){
			this._resizeHelper.hide();
			this._changeSizing(e);
			setTimeout(pub, this.animateDuration + 15);
		}else{
			pub();
		}
		this._isSizing = false;
		this.onResize(e);
	},
	
	onResize: function(e){
		// summary: Stub fired when sizing is done. Fired once
		//	after resize, or often when `intermediateChanges` is
		//	set to true.
	}
	
});

var _ResizeHelper = dojo.declare("dojox.layout._ResizeHelper", Widget, {
	// summary: A global private resize helper shared between any
	//		`dojox.layout.ResizeHandle` with activeSizing off.
	
	show: function(){
		// summary: show helper to start resizing
		domStyle.set(this.domNode, "display", "");
	},
	
	hide: function(){
		// summary: hide helper after resizing is complete
		domStyle.set(this.domNode, "display", "none");
	},
	
	resize: function(/* Object */dim){
		// summary: size the widget and place accordingly
		domGeometry.setMarginBox(this.domNode, dim);
	}
	
});
return ResizeHandle;
});

},
'dojoe/treetablex/_ConfigureColumnsDialog':function(){
define("dojoe/treetablex/_ConfigureColumnsDialog", [
	"dojo/_base/declare",
	"dijit/_Widget",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	"dijit/Dialog",
	"dijit/Toolbar",
	"dijit/form/Form",
	"dijit/form/Button",
	"dijit/form/CheckBox",
	"dijit/layout/TabContainer",
	"dijit/layout/BorderContainer",
	"dijit/layout/ContentPane",
	"dojo/data/ItemFileWriteStore",
	"dojox/grid/EnhancedGrid",
	"dijit/_Container",
	"dojo/_base/lang",
	"dojo/_base/array",
	"dojo/dom-style",
	"dojo/i18n!dojoe/treetablex/nls/TreeTable",
	"dojo/text!./templates/ConfigureColumnsDialog.html"
	], function (declare, _Widget, _TemplatedMixin, _WidgetsInTemplateMixin, Dialog, Toolbar, Form, Button, CheckBox, TabContainer, BorderContainer, ContentPane, ItemFileWriteStore, EnhancedGrid, _Container, Lang, Array, domStyle, _resources, template) {

	_ConfigureColumnsDialog = declare('dojoe.treetablex._ConfigureColumnsDialog', [ _Widget, _TemplatedMixin, _WidgetsInTemplateMixin ], {	

		// summary: 		This class handles all UI rendering for configure column dialog options.
		// description: ConfigureColumnsDialog is a widget to control all the operations  
        //              on the dialog.
		// tags: private
	
		widgetsInTemplate: true,
		templateString: template,
		optionsWidget_: null,
		grid: null,
		indirectSelection: null,
		enableColumnReorder:null,
		enhancedGrid: null,
		columndata:null,
		gridStructure: null,
		resources_: null,
		
		constructor: function() {
			this.resources_ = _resources;
		},
		
		postCreate: function() { 
		//summary: inherited method from the _Widget
		//description: Instantiate the TreeTableOptions for the the operations of configurecolumn.
			this.inherited ("postCreate", arguments);
			var columns=this.gridStructure;
			var names=[];
			for (var i=0 in columns) {
				var column=columns[i];
				var size=dojo.trim(column.name).length;
				if(size > 0){
					if(column.expandLevel != 'all'){
						names.push(column);
					}
				}					
			}				
		    columndata = {
				identifier: 'id',
				label: 'id',
				items: []
			};
			var len = names.length;
			for(var i=0; i < names.length; ++i){
				var row ={id : (i+1), 'name': names[i].name, 'hidden':!names[i].hidden} ;
				columndata.items.push(row);
			}
			var grid_layout = [{
				rows:[
					{ field: "hidden", width: "30%", name: this.resources_.VISIBLE, editable:true, cellType: dojox.grid.cells.Bool},
					{ field: "name", width: "70%", name: this.resources_.COLUMNS}					
				]
			}];
			var pluginsForGrid = {};	
			var itemStore = new ItemFileWriteStore({data:columndata});			
			var gridParams ={
				store : itemStore,
				structure : grid_layout,
				plugins : pluginsForGrid,
				selectionMode:'single',
				canSort: function(){return false;},
				keepSelection: false
			};
			this.enhancedGrid = new EnhancedGrid(gridParams, this.enhancedGridNode);
			this.connect(this, "resize",this.enhancedGrid, "resize");
			
			var selectIcon = new Button({
				id: this.id+"selectAllIcon",
				showLabel: false,
				iconClass: "selectAllIcon",
				label: this.resources_.SELECT_ALL
			},this.selectAllButton);
				
			selectIcon.onClick = Lang.hitch (this, "onSelectAll");
			selectIcon.onKeypress = Lang.hitch (this, "onSelectAll");		
			
			var deSelectIcon = new Button({
				id: this.id+"deselectAllIcon",
				showLabel: false,
				iconClass: "deselectAllIcon",
				label: this.resources_.DESELECT_ALL
			},this.deselectAllButton);
				
			deSelectIcon.onClick = Lang.hitch (this, "onDeselectAll");
			deSelectIcon.onKeypress = Lang.hitch (this, "onDeselectAll");
				
			if(this.enableColumnReorder){
				var upIcon = new Button({
					id: this.id+"upIcon",
					showLabel: false,
					iconClass: "upIcon",
					label: this.resources_.UP
				},this.upButton);
					
				upIcon.onClick = Lang.hitch (this, "onUp");
				upIcon.onKeypress = Lang.hitch (this, "onUp");		
				
				var downIcon = new Button({
					id: this.id+"downIcon",
					showLabel: false,
					iconClass: "downIcon",
					label: this.resources_.DOWN
				},this.downButton);
					
				downIcon.onClick = Lang.hitch (this, "onDown");
				downIcon.onKeypress = Lang.hitch (this, "onDown");
			}else {
				domStyle(this.upButton.parentNode, "width", "0px");
				domStyle(this.downButton.parentNode, "width", "0px");
			}
			this.connect(this.dialog, "onHide",Lang.hitch (this, this.kill));
			
		},		
		
		onSelectAll: function(){
		//summary: selects all the column shown in the ConfigureColumnsDialog		
			var rowCount = this.enhancedGrid.rowCount;
			for (var i = 0; i < rowCount; i++) {
				columndata.items[i].hidden[0]=true;
			}
			this.enhancedGrid._refresh();	
		},	
		
		onDeselectAll: function(){		
		//summary: deselects all the column shown in the ConfigureColumnsDialog
			var rowCount = this.enhancedGrid.rowCount;
			for (var i = 0; i < rowCount; i++) {
				columndata.items[i].hidden[0]=false;
			}
			this.enhancedGrid._refresh();
		},
		
		onOK: function () {
			var rowCount = this.enhancedGrid.rowCount;
			var columns=this.gridStructure;
			var reorderedcolumn=[];
			reorderedcolumn[0]=columns[0];
			for (var i = 0; i < rowCount; i++) {
				var item=this.enhancedGrid.getItem(i);
				console.debug("item...");
				console.debug(item);
				for (var j = 0; j < columns.length ; j++) {
						if(item.name[0] === columns[j].name && item.hidden[0]==true){
							reorderedcolumn.push(columns[j]);
						}
				}
			}
			console.debug("reorderedcolumn ...");
			console.debug(reorderedcolumn);
			this.grid.setColumns(reorderedcolumn);
			this.hide();
		},
		
		onCancel: function () {
			this.hide();
		},		
		
		clearSelection: function () {
			this.enhancedGrid.selection.clear();
		},			
		
		onUp: function () {
			var selectedIndex = this.enhancedGrid.selection.selectedIndex;
			if(selectedIndex <= 0) return;
			var itemsNew = [];
			if (selectedIndex-1 >= 0)
				itemsNew = itemsNew.concat(columndata.items.slice(0,selectedIndex-1));
			itemsNew = itemsNew.concat(this.enhancedGrid.selection.getSelected());
			if (selectedIndex-1 >= 0)
				itemsNew.push(columndata.items[selectedIndex-1]);				
			itemsNew = itemsNew.concat(columndata.items.slice(selectedIndex+1, columndata.items.length));				
			this.enhancedGrid.store.close();
			columndata.items = itemsNew;
			this.enhancedGrid.store = new ItemFileWriteStore({data:columndata});
			this.enhancedGrid.selection.clear();
			this.enhancedGrid._refresh();
			this.enhancedGrid.selection.addToSelection(selectedIndex-1);
		},	

		onDown: function () {
			var selectedIndex = this.enhancedGrid.selection.selectedIndex;
			if(selectedIndex < 0) return;
			if(selectedIndex == columndata.items.length-1) return;
			var itemsNew = [];
			itemsNew = itemsNew.concat(columndata.items.slice(0,selectedIndex));
			if(columndata.items[selectedIndex+1] !== 'undefined')
				itemsNew.push(columndata.items[selectedIndex+1]);			
			itemsNew = itemsNew.concat(this.enhancedGrid.selection.getSelected());
			if(columndata.items[selectedIndex+2] !== 'undefined')
				itemsNew = itemsNew.concat(columndata.items.slice(selectedIndex+2, columndata.items.length));				
			this.enhancedGrid.store.close();
			columndata.items = itemsNew;
			this.enhancedGrid.store = new ItemFileWriteStore({data:columndata});
			this.enhancedGrid.selection.clear();
			this.enhancedGrid._refresh();			
			this.enhancedGrid.selection.addToSelection(selectedIndex+1);
		},			
		
		_selectColumns: function(){
			var rowCount = this.enhancedGrid.rowCount;
			for (var i = 1; i < rowCount; i++) {
				var item=this.enhancedGrid.getItem(i);
				console.debug("item...");
				console.debug(item);
				var status = this._checkColumnVisible(item.name[0]);
				if(status == false) {
					item.hidden[0]=false;
				}
				
			}
		},
		
		_checkColumnVisible: function(colField) {
			var cols = this.grid.structure;
			for (var i in cols) {
				if(i){
					if(cols[i].name == colField){
						return true;
					}
				}
			}
			return false;
		},
		
		show: function () {
			this.dialog.show();
			this.enhancedGrid._refresh();
			//	this.enhancedGrid.startup();
			this._selectColumns();
			dijit.byId (this.id + "tabs").resize();
		},
		
		hide: function () {
			this.dialog.hide();
		},	
		
		kill: function () {
			//call destroy to completely destroy the widget...
			/*var id = this.id;
			var dId=this.dialog.id;
			this.dialog.destroyRecursive();
			dijit.registry.remove (dId);
			this.destroyRecursive();
			dijit.registry.remove (id);*/
			setTimeout (Lang.hitch (this, this.destroy), 100);
		},
		
		destroy: function(){
			// summary:  Destroys the toolbar.
			// tags: public	
			//memory leak issue: custom destroy method to take care of removing all widgets					
			try{
				delete this.resources_;
				//this is for destroying child templated widgets 
				//this should happen by itself but need to figure out why dijit._Templated is not taking care of it...
				var dijitDialog = dijit.byId(this.dialog);
				var borderContID = this.id +'tabs';
				var borderContainer = dijit.byId(borderContID );
				if(borderContainer)borderContainer.destroyRecursive();
				if(dijitDialog)dijitDialog.destroyRecursive();
				var widgets = dijit.findWidgets(this.domNode);
				dojo.forEach(widgets, function(w) {
					if(w){
						if(w.destroyRecursive){
							w.destroyRecursive();
						} else if(w.destroy){
							w.destroy();
						}
					}
				});
				// this is for widgets added as child to this widget and their children..
				//not present in template
				var widgetArr = this.getChildren();
				dojo.forEach(widgetArr, function (child) {
					if(child){
						if(child.destroyRecursive){
							child.destroyRecursive();
						} else if(child.destroy){
							child.destroy();
						}
					}
				});
				this.inherited(arguments);
			}catch(ex){
				console.warn("Widget destroy failed: "+this.id+", ex="+ex.name+":"+ex.message);
			}
		}
	});
	return _ConfigureColumnsDialog;
});
},
'dijit/_base/typematic':function(){
define(["../typematic"], function(){
	// for back-compat, just loads top level module
});

},
'ibm/tivoli/tip/dijit/BreadcrumbBar':function(){
// wrapped by build app
define("ibm/tivoli/tip/dijit/BreadcrumbBar", ["dijit","dojo","dojox","dojo/require!dijit/_Widget,dijit/_Container,dijit/_Templated,dijit/_KeyNavContainer,dijit/form/Button,ibm/tivoli/tip/dijit/Breadcrumb,ibm/tivoli/tip/dijit/BreadcrumbSeparator"], function(dijit,dojo,dojox){
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
dojo.provide("ibm.tivoli.tip.dijit.BreadcrumbBar");

dojo.require("dijit._Widget");
dojo.require("dijit._Container");
dojo.require("dijit._Templated");
dojo.require("dijit._KeyNavContainer");
dojo.require("dijit.form.Button");
dojo.require("ibm.tivoli.tip.dijit.Breadcrumb");
dojo.require("ibm.tivoli.tip.dijit.BreadcrumbSeparator");

dojo.declare(
	"ibm.tivoli.tip.dijit.BreadcrumbBar",
	[dijit._Widget, dijit._Templated, 
	dijit._KeyNavContainer],
{
	templateString:
		'<div class="dijit dijitToolbar tipBreadcrumbBar" waiRole="toolbar" tabIndex="${tabIndex}" dojoAttachPoint="containerNode">' +
		'</div>',

	tabIndex: "0",
	breadcrumbWidget: [],
	breadcrumbsHidden: false,

	postCreate: function(){
		this.connectKeyNavHandlers(
			this.isLeftToRight() ? [dojo.keys.LEFT_ARROW] : [dojo.keys.RIGHT_ARROW],
			this.isLeftToRight() ? [dojo.keys.RIGHT_ARROW] : [dojo.keys.LEFT_ARROW]
		);
	},

	startup: function(){
		this.startupKeyNavChildren();
	},
	
	destroy: function () {
		this.destroyDescendants();
		this.inherited (arguments);
	},
	
	addChild: function(widget, insertIndex){
		var children = this.getChildren();
		var childToUpdate = null;
		if(children.length>0 && children[children.length -1]._setStateClass){
			childToUpdate=children[children.length -1]; //child which style class needs to be updated after addition
		}
		this.inherited (arguments);	//adding child
		if(childToUpdate!=null){
			childToUpdate._setStateClass(); //updating style class
		}
		//Breadcrumbs Tivoli Guidelines implementation
		var size = dojo.contentBox(this.domNode);
		this.breadcrumbWidget.push(widget);
		var i = 1;
		var isRtL = false;
		// for RtL languages, we need to manipulate the space left in breadcrumbbar considering that it is filled from right.
		// breadcrumbWidth  and  breadcrumbTrailWidth are calculated differently for R to L languages.  isRtL holds if the locale is Right to Left or otherwise.
		if(dojo.locale=="iw" || dojo.locale=="he"|| dojo.locale.indexOf("ar")>-1)
			isRtL = true;
		var breadcrumbWidth = this.breadcrumbWidget[0].containerNode.offsetLeft;
		if(isRtL)
			breadcrumbWidth = size.w - breadcrumbWidth;
		while(i < this.breadcrumbWidget.length){
			if(this.breadcrumbWidget[i].declaredClass == "ibm.tivoli.tip.dijit.BreadcrumbSeparator")
				breadcrumbWidth = breadcrumbWidth + this.breadcrumbWidget[i-1].containerNode.offsetWidth + this.breadcrumbWidget[i].domNode.offsetWidth;
			else 
				breadcrumbWidth = breadcrumbWidth + this.breadcrumbWidget[i-1].domNode.offsetWidth + this.breadcrumbWidget[i].containerNode.offsetWidth;
			if(breadcrumbWidth > size.w){
				this.breadcrumbsHidden = true;
				var breadcrumbTrailWidth = breadcrumbWidth;
				//Vyshantha: Mar31,2011: #907: Long breadcrumb trail puts ellipses in bad location
				//for(j = i - 2; j > 0 && breadcrumbTrailWidth > size.w ; j = j - 1){
				for(j = i - 4; j > 0 && breadcrumbTrailWidth > size.w ; j = j - 1){
					breadcrumbTrailWidth = 0;
					if(this.breadcrumbWidget[j].declaredClass == "ibm.tivoli.tip.dijit.BreadcrumbSeparator"){
						dojo.byId(this.breadcrumbWidget[j].id).style.display = "none";
						breadcrumbTrailWidth = this.breadcrumbWidget[j-1].containerNode.offsetLeft + this.breadcrumbWidget[j-1].containerNode.offsetWidth;
					} else {
						dojo.byId(this.breadcrumbWidget[j].id).style.display = "none";
						dojo.byId(this.breadcrumbWidget[j-1].id).style.display = "none";
						breadcrumbTrailWidth = this.breadcrumbWidget[j-2].domNode.offsetLeft + this.breadcrumbWidget[j-2].domNode.offsetWidth;
					}
					if(isRtL)
						breadcrumbTrailWidth = size.w - breadcrumbTrailWidth;
				}
				//Adding Ellipsis to make long breadcrumb trails guideline compliant
				//Vyshantha: Mar31,2011: #907: Long breadcrumb trail puts ellipses in bad location
				//if(this.breadcrumbWidget[i - 1].declaredClass == "ibm.tivoli.tip.dijit.BreadcrumbSeparator"){
				if(this.breadcrumbWidget[i - 3] !== undefined && this.breadcrumbWidget[i - 3].declaredClass == "ibm.tivoli.tip.dijit.BreadcrumbSeparator"){ // added check for undefined as it would throw exception in R to L languages.
					//dojo.byId(this.breadcrumbWidget[i - 1].id).innerHTML = "&gt;&#160;...&#160;&#160;&gt;";
					dojo.byId(this.breadcrumbWidget[i - 3].id).innerHTML = "&gt;&#160;...&#160;&#160;&gt;";
				} else {
					//dojo.byId(this.breadcrumbWidget[i].id).innerHTML = "&gt;&#160;...&#160;&#160;&gt;";
					if(this.breadcrumbWidget[i - 2] !== undefined) {
						dojo.byId(this.breadcrumbWidget[i - 2].id).innerHTML = "&gt;&#160;...&#160;&#160;&gt;";
						}
				}
			} else {
				this.breadcrumbsHidden = false;
			}
			i = i + 1;
		}
	},
	
	areBreadcrumbsHidden: function(){
		return this.breadcrumbsHidden;
	}
}
);


});

},
'dijit/form/FilteringSelect':function(){
define([
	"dojo/data/util/filter", // filter.patternToRegExp
	"dojo/_base/declare", // declare
	"dojo/_base/Deferred", // Deferred.when
	"dojo/_base/lang", // lang.mixin
	"./MappedTextBox",
	"./ComboBoxMixin"
], function(filter, declare, Deferred, lang, MappedTextBox, ComboBoxMixin){

/*=====
	var MappedTextBox = dijit.form.MappedTextBox;
	var ComboBoxMixin = dijit.form.ComboBoxMixin;
=====*/

	// module:
	//		dijit/form/FilteringSelect
	// summary:
	//		An enhanced version of the HTML SELECT tag, populated dynamically


	return declare("dijit.form.FilteringSelect", [MappedTextBox, ComboBoxMixin], {
		// summary:
		//		An enhanced version of the HTML SELECT tag, populated dynamically
		//
		// description:
		//		An enhanced version of the HTML SELECT tag, populated dynamically. It works
		//		very nicely with very large data sets because it can load and page data as needed.
		//		It also resembles ComboBox, but does not allow values outside of the provided ones.
		//		If OPTION tags are used as the data provider via markup, then the
		//		OPTION tag's child text node is used as the displayed value when selected
		//		while the OPTION tag's value attribute is used as the widget value on form submit.
		//		To set the default value when using OPTION tags, specify the selected
		//		attribute on 1 of the child OPTION tags.
		//
		//		Similar features:
		//			- There is a drop down list of possible values.
		//			- You can only enter a value from the drop down list.  (You can't
		//				enter an arbitrary value.)
		//			- The value submitted with the form is the hidden value (ex: CA),
		//				not the displayed value a.k.a. label (ex: California)
		//
		//		Enhancements over plain HTML version:
		//			- If you type in some text then it will filter down the list of
		//				possible values in the drop down list.
		//			- List can be specified either as a static list or via a javascript
		//				function (that can get the list from a server)

		// required: Boolean
		//		True (default) if user is required to enter a value into this field.
		required: true,

		_lastDisplayedValue: "",

		_isValidSubset: function(){
			return this._opened;
		},

		isValid: function(){
			// Overrides ValidationTextBox.isValid()
			return !!this.item || (!this.required && this.get('displayedValue') == ""); // #5974
		},

		_refreshState: function(){
			if(!this.searchTimer){ // state will be refreshed after results are returned
				this.inherited(arguments);
			}
		},

		_callbackSetLabel: function(
						/*Array*/ result,
						/*Object*/ query,
						/*Object*/ options,
						/*Boolean?*/ priorityChange){
			// summary:
			//		Callback from dojo.store after lookup of user entered value finishes

			// setValue does a synchronous lookup,
			// so it calls _callbackSetLabel directly,
			// and so does not pass dataObject
			// still need to test against _lastQuery in case it came too late
			if((query && query[this.searchAttr] !== this._lastQuery) || (!query && result.length && this.store.getIdentity(result[0]) != this._lastQuery)){
				return;
			}
			if(!result.length){
				//#3268: don't modify display value on bad input
				//#3285: change CSS to indicate error
				this.set("value", '', priorityChange || (priorityChange === undefined && !this.focused), this.textbox.value, null);
			}else{
				this.set('item', result[0], priorityChange);
			}
		},

		_openResultList: function(/*Object*/ results, /*Object*/ query, /*Object*/ options){
			// Callback when a data store query completes.
			// Overrides ComboBox._openResultList()

			// #3285: tap into search callback to see if user's query resembles a match
			if(query[this.searchAttr] !== this._lastQuery){
				return;
			}
			this.inherited(arguments);

			if(this.item === undefined){ // item == undefined for keyboard search
				// If the search returned no items that means that the user typed
				// in something invalid (and they can't make it valid by typing more characters),
				// so flag the FilteringSelect as being in an invalid state
				this.validate(true);
			}
		},

		_getValueAttr: function(){
			// summary:
			//		Hook for get('value') to work.

			// don't get the textbox value but rather the previously set hidden value.
			// Use this.valueNode.value which isn't always set for other MappedTextBox widgets until blur
			return this.valueNode.value;
		},

		_getValueField: function(){
			// Overrides ComboBox._getValueField()
			return "value";
		},

		_setValueAttr: function(/*String*/ value, /*Boolean?*/ priorityChange, /*String?*/ displayedValue, /*item?*/ item){
			// summary:
			//		Hook so set('value', value) works.
			// description:
			//		Sets the value of the select.
			//		Also sets the label to the corresponding value by reverse lookup.
			if(!this._onChangeActive){ priorityChange = null; }

			if(item === undefined){
				if(value === null || value === ''){
					value = '';
					if(!lang.isString(displayedValue)){
						this._setDisplayedValueAttr(displayedValue||'', priorityChange);
						return;
					}
				}

				var self = this;
				this._lastQuery = value;
				Deferred.when(this.store.get(value), function(item){
					self._callbackSetLabel(item? [item] : [], undefined, undefined, priorityChange);
				});
			}else{
				this.valueNode.value = value;
				this.inherited(arguments);
			}
		},

		_setItemAttr: function(/*item*/ item, /*Boolean?*/ priorityChange, /*String?*/ displayedValue){
			// summary:
			//		Set the displayed valued in the input box, and the hidden value
			//		that gets submitted, based on a dojo.data store item.
			// description:
			//		Users shouldn't call this function; they should be calling
			//		set('item', value)
			// tags:
			//		private
			this.inherited(arguments);
			this._lastDisplayedValue = this.textbox.value;
		},

		_getDisplayQueryString: function(/*String*/ text){
			return text.replace(/([\\\*\?])/g, "\\$1");
		},

		_setDisplayedValueAttr: function(/*String*/ label, /*Boolean?*/ priorityChange){
			// summary:
			//		Hook so set('displayedValue', label) works.
			// description:
			//		Sets textbox to display label. Also performs reverse lookup
			//		to set the hidden value.  label should corresponding to item.searchAttr.

			if(label == null){ label = ''; }

			// This is called at initialization along with every custom setter.
			// Usually (or always?) the call can be ignored.   If it needs to be
			// processed then at least make sure that the XHR request doesn't trigger an onChange()
			// event, even if it returns after creation has finished
			if(!this._created){
				if(!("displayedValue" in this.params)){
					return;
				}
				priorityChange = false;
			}

			// Do a reverse lookup to map the specified displayedValue to the hidden value.
			// Note that if there's a custom labelFunc() this code
			if(this.store){
				this.closeDropDown();
				var query = lang.clone(this.query); // #6196: populate query with user-specifics

				// Generate query
				var qs = this._getDisplayQueryString(label), q;
				if(this.store._oldAPI){
					// remove this branch for 2.0
					q = qs;
				}else{
					// Query on searchAttr is a regex for benefit of dojo.store.Memory,
					// but with a toString() method to help dojo.store.JsonRest.
					// Search string like "Co*" converted to regex like /^Co.*$/i.
					q = filter.patternToRegExp(qs, this.ignoreCase);
					q.toString = function(){ return qs; };
				}
				this._lastQuery = query[this.searchAttr] = q;

				// If the label is not valid, the callback will never set it,
				// so the last valid value will get the warning textbox.   Set the
				// textbox value now so that the impending warning will make
				// sense to the user
				this.textbox.value = label;
				this._lastDisplayedValue = label;
				this._set("displayedValue", label);	// for watch("displayedValue") notification
				var _this = this;
				var options = {
					ignoreCase: this.ignoreCase,
					deep: true
				};
				lang.mixin(options, this.fetchProperties);
				this._fetchHandle = this.store.query(query, options);
				Deferred.when(this._fetchHandle, function(result){
					_this._fetchHandle = null;
					_this._callbackSetLabel(result || [], query, options, priorityChange);
				}, function(err){
					_this._fetchHandle = null;
					if(!_this._cancelingQuery){	// don't treat canceled query as an error
						console.error('dijit.form.FilteringSelect: ' + err.toString());
					}
				});
			}
		},

		undo: function(){
			this.set('displayedValue', this._lastDisplayedValue);
		}
	});
});

},
'dijit/layout/TabContainer':function(){
define([
	"dojo/_base/lang", // lang.getObject
	"dojo/_base/declare", // declare
	"./_TabContainerBase",
	"./TabController",
	"./ScrollingTabController"
], function(lang, declare, _TabContainerBase, TabController, ScrollingTabController){

/*=====
	var _TabContainerBase = dijit.layout._TabContainerBase;
	var TabController = dijit.layout.TabController;
	var ScrollingTabController = dijit.layout.ScrollingTabController;
=====*/

	// module:
	//		dijit/layout/TabContainer
	// summary:
	//		A Container with tabs to select each child (only one of which is displayed at a time).


	return declare("dijit.layout.TabContainer", _TabContainerBase, {
		// summary:
		//		A Container with tabs to select each child (only one of which is displayed at a time).
		// description:
		//		A TabContainer is a container that has multiple panes, but shows only
		//		one pane at a time.  There are a set of tabs corresponding to each pane,
		//		where each tab has the name (aka title) of the pane, and optionally a close button.

		// useMenu: [const] Boolean
		//		True if a menu should be used to select tabs when they are too
		//		wide to fit the TabContainer, false otherwise.
		useMenu: true,

		// useSlider: [const] Boolean
		//		True if a slider should be used to select tabs when they are too
		//		wide to fit the TabContainer, false otherwise.
		useSlider: true,

		// controllerWidget: String
		//		An optional parameter to override the widget used to display the tab labels
		controllerWidget: "",

		_makeController: function(/*DomNode*/ srcNode){
			// summary:
			//		Instantiate tablist controller widget and return reference to it.
			//		Callback from _TabContainerBase.postCreate().
			// tags:
			//		protected extension

			var cls = this.baseClass + "-tabs" + (this.doLayout ? "" : " dijitTabNoLayout"),
				TabController = lang.getObject(this.controllerWidget);

			return new TabController({
				id: this.id + "_tablist",
				dir: this.dir,
				lang: this.lang,
				textDir: this.textDir,
				tabPosition: this.tabPosition,
				doLayout: this.doLayout,
				containerId: this.id,
				"class": cls,
				nested: this.nested,
				useMenu: this.useMenu,
				useSlider: this.useSlider,
				tabStripClass: this.tabStrip ? this.baseClass + (this.tabStrip ? "":"No") + "Strip": null
			}, srcNode);
		},

		postMixInProperties: function(){
			this.inherited(arguments);

			// Scrolling controller only works for horizontal non-nested tabs
			if(!this.controllerWidget){
				this.controllerWidget = (this.tabPosition == "top" || this.tabPosition == "bottom") && !this.nested ?
							"dijit.layout.ScrollingTabController" : "dijit.layout.TabController";
			}
		}
	});
});

},
'ibm/usmi/widgets/table/_TableOptions':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

define("ibm/usmi/widgets/table/_TableOptions", [
    "dojo/dom",
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dijit/_WidgetBase",
	"dijit/registry",
	"ibm/usmi/util/DataStore",
	"ibm/tivoli/navmodel/Dataset",
	"dojoe/tablex/_TableOptions",
	"ibm/usmi/widgets/table/_ColumnVisible",
	"dojo/store/Memory",
	"gridx/core/model/cache/Sync",
	"gridx/Grid",
	"gridx/modules/move/Row",
	"gridx/modules/dnd/Row",
	"gridx/modules/CellWidget"
], function (dom, declare, array, lang, _WidgetBase, registry, DataStore, Dataset, TableOptions, ColumnVisible, Memory,
		Cache, Grid, MoveRow, DndRow, CellWidget) {
	return declare([
		_WidgetBase,
		TableOptions
	], {
		allColumns : null,
		columnMap : {},
		defaultColumns : null,
		//checkedColumns : [],
		structure : null,
		store : null,
		gridSelected : null,
		nextGridValue : 0,

		postCreate : function () {
			this.structure = [
				{
					id : 'name',
					field : 'name',
					name : '',
					//width : '200px'
				},
				{
					id : 'remove',
					field : 'remove',
					name : '',
					width : '20px',
					widgetsInCell : true,
					decorator : function () {
						//Generate cell widget template string
						return [
							'<span ',
							'data-dojo-attach-point="btn" ',
							'class="dijitReset dijitInline dijitIcon dijitIconDelete" ',
							'></span>'
						].join('');
					},
					setCellValue : function (data) {
						//"this" is the cell widget
						this.btn.onclick = lang.hitch(this, function (evt) {
							var thisContext = registry.byId(data.context);
							thisContext.removeUsingButton(data.tableId, data.columnId);
						});
					}
				}
			];
			this.store = new Memory({
				data : []
			});

			this.initializeDataStores();
			this.gridSelected = Grid({
				id : this.configColumnsWidget.id + 'selectedColumnsGrid',
				cacheClass : Cache,
				store : this.store,
				structure : this.structure,
				headerHidden : true
			});

			this.gridSelected.placeAt(this.configColumnsWidget.id + 'selectedColumnsdiv');
			this.gridSelected.startup();
		},
		destroy : function () {
			//this is for destroying child templated widgets 
			//this should happen by itself but need to figure out why dijit._Templated is not taking care of it...
			var widgets = registry.findWidgets(this.domNode);
			array.forEach(widgets, function (w) {
				if (w) {
					if (w.destroyRecursive) {
						w.destroyRecursive();
					} else if (w.destroy) {
						w.destroy();
					}
				}
			});
			// this is for widgets added as child to this widget and their children..
			//not present in template
			var widgetArr = this.getChildren();
			array.forEach(widgetArr, function (child) {
				if (child) {
					if (child.destroyRecursive) {
						child.destroyRecursive();
					} else if (child.destroy) {
						child.destroy();
					}
				}
			});
			this.allColumns = null;
			this.columnMap = null;
			this.defaultColumns = null;
			this.checkedColumns = null;
			this.gridSelected = null;
			this.store = null;
			this.structure = null;
			this.inherited(arguments);
		},
		removeUsingButton : function (tableId, columnId) {
			this.store.remove(tableId);
			columnWidget = registry.byId(this.id + "column_" + columnId);
			columnWidget.setVisible(false);
		},
		addColumnsVisible : function (columns) {
			var widget = null;
			var column = null;
			var tableColumns = this.grid._columns;
			console.info(this.grid.getParent().getParent()._dataStore);
			//columns = this._columnUnion(columns, this.grid.layout.cells);
			var i;
			for (i in columns) {
				if (i) {
					column = columns[i];
					/*this.grid.layout.cells.push(column);
					if (column.field && !dom.byId(this.id + "column_" + column.field)) {
						console.info(column.visible);
						widget = new ColumnVisible({
							id : this.id + "column_" + column.field,
							columnid : this.id + "column_" + column.field,
							checked : column.visible === undefined ? !column.hidden : column.visible,
							label : column.name,
							columnChooser : this
						});
						this.addChild(widget);
						if (widget.isVisible() && this.checkedColumns.indexOf(widget.label) === -1) {
							this.checkedColumns.push(widget.label);
						}
					} else */

					if (column.getId && column.getId() && !dom.byId(this.id + "column_" + column.getId())) {
						widget = new ColumnVisible({
							id : this.id + "column_" + column.getId(),
							columnid : this.id + "column_" + column.getId(),
							checked : this._isColumnVisibleInTable(column, tableColumns),//column.hidden,
							label : column.getLabel(),
							columnChooser : this
						});
						this.addChild(widget);
					}
				}
			}

			for ( var k in tableColumns) {
				column = tableColumns[k];
				var temp = this.store.query({
					name : column.name
				});
				if ((!this.store.query({
					name : column.name
				}) || temp.length === 0)) {
					var array = [];
					array["tableId"] = this.nextGridValue;
					array["columnId"] = column.field;
					array["context"] = this;
					this.store.put({
						id : this.nextGridValue,
						order : this.nextGridValue,
						name : registry.byId(this.id + "column_" + column.field).label,
						remove : {
							tableId : this.nextGridValue,
							columnId : column.field,
							context : this.id
						}
					});
					this.nextGridValue++;
				}
			}
			this.configColumnsWidget.loadingDiv.style.display = "none";
			this.configColumnsWidget.columnsDiv.style.display = "block";
			this.configColumnsWidget.columnsDiv.style.paddingLeft = "30px";
			this.configColumnsWidget.columnsDiv.style.textIndent = "-30px"; 
		},

		_isColumnVisibleInTable : function (column, tableColumns) {
			var tableColumn;
			for ( var i = 0; tableColumns && i < tableColumns.length; i++) {
				tableColumn = tableColumns[i];
				if (!tableColumn.field) {
					continue;
				}
				if (!column.getId) {
					break;
				}
				if (column.getId() === tableColumn.field) {
					this.columnMap[tableColumn.field] = column;//put the raw column to cache
					if (tableColumn.visible !== undefined) {
						return tableColumn.visible;
					} else {
						return !tableColumn.hidden;
					}
				}
			}
			if (column.visible !== undefined) {
				return column.visible;
			}
			return false;
		},

		_columnUnion : function (array1, array2) {
			var array3 = array1;
			var element = null;
			var element1 = null;
			var element2 = null;
			var noMatch = true;
			for ( var i in array2) {
				element = array2[i];
				for ( var j in array1) {
					if (array1[j].getId && element.field) {
						element1 = array1[j].getId();
						element2 = element.field;
					} else if (element.getId && array1[j].field) {
						element1 = element.getId();
						element2 = array1[j].field;
					}
					if (element1 === element2) {
						noMatch = false;
						this.columnMap[element1] = array3[j];
						array3[j] = element;
					}
				}
				if (noMatch) {
					array3.push(element);
				}
				noMatch = true;
			}
			return array3;
		},

		onSelectAll : function () {
			//for(var j=0; j<this.grid.structure.length; j++) {
			//var columns = this.grid.structure[j].rows;    
			//Anupama :24May11, replacing for..in construct with regular for loop since for..in construct interfering with changes in Array prototype in IE7.
			//26600: select all,de-select all feature in Table configure options dialog not working on IE7 browser.
			//for (var i in columns) {
			/*LS - 13/01/2012 - 30956 - Unresponsive script msg when using configure options */
			//removing the indirect way of fetching columns through grid structure and getting it through layout.cells directly
			// this is improve performance of this feature reducing no of iterations...
			var k;
			for (k = 0; k < this.allColumns.length; k++) {
				var column = this.allColumns[k];
				var columnWidget;
				if (column && column.field) {
					columnWidget = registry.byId(this.id + "column_" + column.field);
					columnWidget.setVisible(true);
				} else {
					columnWidget = registry.byId(this.id + "column_" + column.getId());
					columnWidget.setVisible(true);
				}
			}
			//}
		},
		onDeselectAll : function () {
			//for(var j=0; j<this.grid.structure.length; j++) {
			//var columns = this.grid.structure[j].rows;
			//Anupama :24May11, replacing for..in construct with regular for loop since for..in construct interfering with changes in Array prototype in IE7.
			//26600: select all,de-select all feature in Table configure options dialog not working on IE7 browser.
			//for (var i in columns) {
			/*LS - 13/01/2012 - 30956 - Unresponsive script msg when using configure options */
			//removing the indirect way of fetching columns through grid structure and getting it through layout.cells directly
			// this is improve performance of this feature reducing no of iterations...
			for ( var k = 0; k < this.allColumns.length; k++) {
				var column = this.allColumns[k];
				var columnWidget;
				if (column && column.field) {
					columnWidget = registry.byId(this.id + "column_" + column.field);
					columnWidget.setVisible(false);
				} else {
					columnWidget = registry.byId(this.id + "column_" + column.getId());
					columnWidget.setVisible(false);
				}
			}
			this.store.destroy();
			this.store = new Memory({
				data : []
			});
			//this.checkedColumns = [];
			//}
		},
		onRestDefaults : function () {
			var columnWidget;
			for (var k = 0; k < this.nextGridValue; k++){
				if(this.store.get(k)){
					columnWidget = registry.byId(this.id + "column_" + this.store.get(k).remove.columnId);
					columnWidget.setVisible(false);
				}
			}
			this.store = new Memory({
                data : []
			});
			this.nextGridValue = 0;
			for ( var j in this.defaultColumns) {
				if (j) {
					if (this.defaultColumns[j].getId()) {
						columnWidget = registry.byId(this.id + "column_" + this.defaultColumns[j].getId());
						columnWidget.setVisible(true);
					}
				}
			}
		},
		updatedCheck : function (elementWidget) {
			var inStore = this.store.query({
				name : elementWidget.label
			});
			if (elementWidget.isVisible() && (!this.store.query({
				name : elementWidget.label
			}) || inStore.length === 0)) {
				//this.checkedColumns.push(elementWidget.label);
				this.store.put({
					id : this.nextGridValue,
					order : this.nextGridValue,
					name : elementWidget.label,
					remove : {
						tableId : this.nextGridValue,
						columnId : elementWidget.id.replace(this.id + "column_", ""),
						context : this.id
					//this
					}
				});
				this.nextGridValue++;
			} else {
				if (!elementWidget.isVisible() && this.store.query({
					name : elementWidget.label
				}) && inStore.length !== 0) {
					this.store.remove(this.store.query({
						name : elementWidget.label
					})[0].id);
				}
			}
			//this.checkedColumns.sort();
			//var list = '<table>';
			this.gridSelected.destroy();
			/*this.store = new Memory({
				data : []
			});
			for ( var i in this.checkedColumns) {
				if (i) {
					this.store.put({
						id : i,
						order : i,
						name : this.checkedColumns[i]
					});
					//	list += '<tr><td>' + this.checkedColumns[i] + '</td></tr>';
				}
			}*/
			//list += '</table>';
			//Create a grid to display the currently selected columns, and allowing reorder of the columns
			this.gridSelected = Grid({
				id : this.configColumnsWidget.id + 'selectedColumnsGrid',
				cacheClass : Cache,
				store : this.store,
				structure : this.structure,
				style : 'width: 100%; height: 98%; overflow: auto',
				selectRowTriggerOnCell : true,
				headerHidden : true,
				modules : [
					MoveRow,
					DndRow,
					CellWidget
				]
			});
			this.gridSelected.placeAt(this.configColumnsWidget.id + 'selectedColumnsdiv');
			this.gridSelected.startup();

			//Adjust the formatting so it shows up correctly instead of being presented with multiple layers of scroll bars,
			//This is a *hack*, but will do for the short term
			dom.byId(this.configColumnsWidget.id + 'selectedColumnsdiv').setAttribute('style',
					'width: 95%; height: 98%; left: 10px; top: 2px; position: absolute;');
			//dom.byId(this.configColumnsWidget.id + 'selectedColumnsdiv').innerHTML = list;
		},
		orderColumns : function (selectedColumns) {
			var orderedColumns = [];
			var orderedRows = [];
			var counter;
			//Get the list of selected columns from the grid and then sort using the order field, as that is what the table uses for it's sorting
			//and it will be accurate
			for ( var i in this.gridSelected.store.data) {
				var dataPoint = this.gridSelected.store.data[i];
				counter = 0;
				while (counter < orderedRows.length && orderedRows[counter].order < dataPoint.order) {
					counter++;
				}
				orderedRows.splice(counter, 0, dataPoint);
			}
			for ( var i in orderedRows) {
				if (selectedColumns[orderedRows[i].name]) {
					orderedColumns.push(selectedColumns[orderedRows[i].name]);
				}
			}
			return orderedColumns;
		},

		_isColumnVisible : function (column) {
			var columnWidget;
			if (column && column.field) {
				columnWidget = registry.byId(this.id + "column_" + column.field);
				if (columnWidget) {
					return columnWidget.isVisible();
				}
			} else if (column && column.getId) {
				columnWidget = registry.byId(this.id + "column_" + column.getId());
				if (columnWidget) {
					return columnWidget.isVisible();
				}
			}
			return false;
		},

		_getRawColumn : function (column) {
			if (column && column.field) {
				return this.columnMap[column.field];
			} else {
				return column;
			}
		},

		onSave : function () {
			/*LS - 13/01/2012 - 30956 - Unresponsive script msg when using configure options */
			//removing the indirect way of fetching columns through grid structure and getting it through layout.cells directly
			// this is improve performance of this feature reducing no of iterations...     
			var selectedColumns = {};
			var tableColumns = this.grid._columns;
			var column, rawColumn;
			//retain current presents columns
			for ( var i = 0; i < tableColumns.length; i++) {
				column = tableColumns[i];
				rawColumn = this._getRawColumn(column);
				if (!rawColumn) {
					continue;
				}
				if (this._isColumnVisible(column) && !selectedColumns[rawColumn.getLabel()]) {
					selectedColumns[rawColumn.getLabel()] = rawColumn;
				}
			}

			console.log(selectedColumns);

			//add new selected columns
			for ( var k = 0; k < this.allColumns.length; k++) {
				column = this.allColumns[k];
				rawColumn = this._getRawColumn(column);
				if (column && !selectedColumns[rawColumn.getLabel()]) {
					if (this._isColumnVisible(column)) {
						selectedColumns[rawColumn.getLabel()] = rawColumn;
					}
				}
			}

			if (selectedColumns === {}) {
				console.log("selected items is zero, use default columns instead");
				selectedColumns = this.defaultColumns;
				this.onRestDefaults();
			}
			if (selectedColumns !== this.defaultColumns) {
				selectedColumns = this.orderColumns(selectedColumns);
			}

			var setColumnsForMixin = function () {
				//console.info('XYZZY - This Got called!!!!!');
				this.grid.getParent().getParent()._dataStore.dataset.destroy();
				//if the table is using widget definition, we need to reset the table.view defintion
				if (this.grid.getParent().getParent()._tableWidgetDefinition
						&& this.grid.getParent().getParent()._tableWidgetDefinition._widgetDef
						&& this.grid.getParent().getParent()._tableWidgetDefinition._widgetDef.table
						&& this.grid.getParent().getParent()._tableWidgetDefinition._widgetDef.table.view) {
					this.grid.getParent().getParent()._tableWidgetDefinition._widgetDef.table.view = [];
				}
				this.grid.getParent().getParent()._dataStore.dataset = new Dataset(
						this.grid.getParent().getParent()._dataStore.dataset.getArgs());
				//this.destroy();
			};

			console.info(selectedColumns);
			this.grid.store.dataset.setColumns({
				columns : selectedColumns,
				onComplete : lang.hitch(this, setColumnsForMixin)
			});
			/*Ls - 17/01/2012 - 30956 - further reducing the time taken to hide/show columns*/
			/*this.grid.views.forEach(function (v, i) {
				var w = v.width;
				if (w && w != "auto") {
					v._togglingColumn = dojo.marginBox(v.structure.cells[0].getHeaderNode()).w || 0;
				}
				v.update();
			});*/
			//this.checkedColumns = [];
		},
		initializeDataStores : function (domId) {
			var gotAllColumns = function (cols) {
				this.allColumns = cols;
				this.addColumnsVisible(cols);
			};
			var gotColumnsError = function () {
				console.info('Error on get columns');
			};
			var dataset = this.grid.store.dataset;
			var columnArgsForMixin = {
				type : this._pTableType,
				onComplete : lang.hitch(this, gotAllColumns),
				onError : lang.hitch(this, gotColumnsError),
				properties : [ "all"
				]
			};
			dataset.getColumns(columnArgsForMixin);

			var gotDefaultColumns = function (cols) {
				this.defaultColumns = cols;
			};

			columnArgsForMixin = {
				type : this._pTableType,
				onComplete : lang.hitch(this, gotDefaultColumns),
				onError : lang.hitch(this, gotColumnsError),
				properties : [ "default"
				]
			};
			dataset.getColumns(columnArgsForMixin);

		}
	});
});

},
'dijit/form/_FormValueWidget':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/_base/sniff", // has("ie")
	"./_FormWidget",
	"./_FormValueMixin"
], function(declare, has, _FormWidget, _FormValueMixin){

/*=====
var _FormWidget = dijit.form._FormWidget;
var _FormValueMixin = dijit.form._FormValueMixin;
=====*/

// module:
//		dijit/form/_FormValueWidget
// summary:
//		FormValueWidget


return declare("dijit.form._FormValueWidget", [_FormWidget, _FormValueMixin],
{
	// summary:
	//		Base class for widgets corresponding to native HTML elements such as <input> or <select> that have user changeable values.
	// description:
	//		Each _FormValueWidget represents a single input value, and has a (possibly hidden) <input> element,
	//		to which it serializes it's input value, so that form submission (either normal submission or via FormBind?)
	//		works as expected.

	// Don't attempt to mixin the 'type', 'name' attributes here programatically -- they must be declared
	// directly in the template as read by the parser in order to function. IE is known to specifically
	// require the 'name' attribute at element creation time.  See #8484, #8660.

	_layoutHackIE7: function(){
		// summary:
		//		Work around table sizing bugs on IE7 by forcing redraw

		if(has("ie") == 7){ // fix IE7 layout bug when the widget is scrolled out of sight
			var domNode = this.domNode;
			var parent = domNode.parentNode;
			var pingNode = domNode.firstChild || domNode; // target node most unlikely to have a custom filter
			var origFilter = pingNode.style.filter; // save custom filter, most likely nothing
			var _this = this;
			while(parent && parent.clientHeight == 0){ // search for parents that haven't rendered yet
				(function ping(){
					var disconnectHandle = _this.connect(parent, "onscroll",
						function(){
							_this.disconnect(disconnectHandle); // only call once
							pingNode.style.filter = (new Date()).getMilliseconds(); // set to anything that's unique
							setTimeout(function(){ pingNode.style.filter = origFilter }, 0); // restore custom filter, if any
						}
					);
				})();
				parent = parent.parentNode;
			}
		}
	}
});

});

},
'ibm/usmi/widgets/finder/TaskDataStore':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
// Finder TaskDataStore
define([
	"dojo/_base/lang",
	"dojo/_base/declare",
	"ibm/usmi/util/Utils",
	"./FinderBaseDataStore"
], function (djLang, declare, Utils, FinderBaseDataStore) {

	return declare([ FinderBaseDataStore
	], {

		constructor : function (type, cache) {
			this.populateItems(cache);
		},
		populateItems : function () {
			this.inherited(arguments);
		},

		initiateTaskSearch : function (searchString, finderObj) {
			var query = "name:'*" + searchString + "*' OR category:'*" + searchString + "*'";
			Utils.CMTraceTimeDelta("finderTimer", "before initiating search", true);
			this._dataStore.fetch({
				query : {
					complexQuery : query
				},
				queryOptions : {
					ignoreCase : true
				},
				onError : djLang.hitch(finderObj, finderObj.fetchFailed),
				onComplete : djLang.hitch(finderObj, finderObj.doTaskSearch)
			});
		},
		destroy : function () {
			this.inherited(arguments);
		}
	});
});

},
'ibm/usmi/widgets/util/Constants':function(){
// wrapped by build app
define(["dijit","dojo","dojox"], function(dijit,dojo,dojox){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
dojo.provide("ibm.usmi.widgets.util.Constants");

ibm.usmi.widgets.util.Constants = {
    // URLs
    CONTEXT_ROOT: "/ibm/console",
    LAYOUT_PAGE:  "/ibm/console/xLaunch.do?pageID=",
    BASE_REST_URL: "/ibm/tivoli/rest",

    // Widgets Parms
    DEFAULT_PROVIDER: "USMI",
    SOURCE_TOKEN: "sourceToken",
    COLLECTION: "collectionID",
    PROVIDER: "providerID",
    SELECTOR_HOME: "selectorHome",
    TREE_SELECTOR_GROUP: "groupID",

    // Topics
    TOPIC_HASH_CHANGE: "/dojo/hashchange",
    TOPIC_RETARGET: "reTarget",
    TOPIC_RETARGET_INFO: "RETARGET_INFO",
    TOPIC_DEFAULT_URL_LAUNCH: "DefaultURLLaunch",
    TOPIC_JOB_LAUNCH_RESULT: "JobLaunchResult",
    TOPIC_TASK_TITLE_CHANGE: "updateTitle",
    TOPIC_READY_WORKAREA: "workAreaReady",
    TOPIC_LOG_JS_TO_SERVER: "log-js-to-server",

    // Hash Parms
    HASH_TASK_SELECTION: "taskSelection",
    HASH_TOPPER_SELECTION: "treeTopperId",
    HASH_TASK_PATH: "taskPath",
    HASH_TASK_PATH_KEY: "taskPathKey",
    HASH_TASK_RELATIONSHIP: "relationship",

    // Events
    SELECTION_CHANGE: "onSelectionChanged",
    SET_DATASET: "onSetDataset",
    ON_CLICK: "onclick",

    // CSS & HTML
    CSS_DISPLAY_NONE: "display: none;",
    CSS_MAX_SIZE: "height:100%;width:100%;",
    HTML_LIST_ITEM: "li",

    // TWL
    PROPS_ALL: "all",
    PROPS_DEFAULT: "default",
    PROPS_CURRENT: "current",
    ITEM: "item",

    // Tasks
    TASK_TYPE_URL: "URL",
    TASK_TYPE_WIDGET: "WIDGET",
    TASK_TYPE_RELATED_RESOURCE: "RELATED_RESOURCE",
    TASK_LAUNCH_CANCELED: "CANCEL",
    TASK_LAUNCH_SUBMIT_SUCCESSFUL: "SUCCESS",
    TASK_LAUNCH_SUBMIT_FAILED:  "FAIL",
    
    //RETARGET_INFO
    RETARGET_INFO_DATASET: "dataset",
    TARGET_INFO: "targetInfo"
};
});

},
'dojox/grid/_Events':function(){
define([
	"dojo/keys",
	"dojo/dom-class",
	"dojo/_base/declare",
	"dojo/_base/event",
	"dojo/_base/sniff"
], function(keys, domClass, declare, event, has){

return declare("dojox.grid._Events", null, {
	// summary:
	//		_Grid mixin that provides default implementations for grid events.
	// description:
	//		Default synthetic events dispatched for _Grid. dojo.connect to events to
	//		retain default implementation or override them for custom handling.
	
	// cellOverClass: String
	// 		css class to apply to grid cells over which the cursor is placed.
	cellOverClass: "dojoxGridCellOver",
	
	onKeyEvent: function(e){
		// summary: top level handler for Key Events
		this.dispatchKeyEvent(e);
	},

	onContentEvent: function(e){
		// summary: Top level handler for Content events
		this.dispatchContentEvent(e);
	},

	onHeaderEvent: function(e){
		// summary: Top level handler for header events
		this.dispatchHeaderEvent(e);
	},

	onStyleRow: function(inRow){
		// summary:
		//		Perform row styling on a given row. Called whenever row styling is updated.
		//
		// inRow: Object
		// 		Object containing row state information: selected, true if the row is selcted; over:
		// 		true of the mouse is over the row; odd: true if the row is odd. Use customClasses and
		// 		customStyles to control row css classes and styles; both properties are strings.
		//
		// example: onStyleRow({ selected: true, over:true, odd:false })
		var i = inRow;
		i.customClasses += (i.odd?" dojoxGridRowOdd":"") + (i.selected?" dojoxGridRowSelected":"") + (i.over?" dojoxGridRowOver":"");
		this.focus.styleRow(inRow);
		this.edit.styleRow(inRow);
	},
	
	onKeyDown: function(e){
		// summary:
		// 		Grid key event handler. By default enter begins editing and applies edits, escape cancels an edit,
		// 		tab, shift-tab, and arrow keys move grid cell focus.
		if(e.altKey || e.metaKey){
			return;
		}
		var colIdx;
		switch(e.keyCode){
			case keys.ESCAPE:
				this.edit.cancel();
				break;
			case keys.ENTER:
				if(!this.edit.isEditing()){
					colIdx = this.focus.getHeaderIndex();
					if(colIdx >= 0) {
						this.setSortIndex(colIdx);
						break;
					}else {
						this.selection.clickSelect(this.focus.rowIndex, dojo.isCopyKey(e), e.shiftKey);
					}
					event.stop(e);
				}
				if(!e.shiftKey){
					var isEditing = this.edit.isEditing();
					this.edit.apply();
					if(!isEditing){
						this.edit.setEditCell(this.focus.cell, this.focus.rowIndex);
					}
				}
				if (!this.edit.isEditing()){
					var curView = this.focus.focusView || this.views.views[0];  //if no focusView than only one view
					curView.content.decorateEvent(e);
					this.onRowClick(e);
					event.stop(e);
				}
				break;
			case keys.SPACE:
				if(!this.edit.isEditing()){
					colIdx = this.focus.getHeaderIndex();
					if(colIdx >= 0) {
						this.setSortIndex(colIdx);
						break;
					}else {
						this.selection.clickSelect(this.focus.rowIndex, dojo.isCopyKey(e), e.shiftKey);
					}
					//If there's a text box in the cell, we can not type SPACE there if the event is stopped.
					//event.stop(e);
				}
				break;
			case keys.TAB:
				this.focus[e.shiftKey ? 'previousKey' : 'nextKey'](e);
				break;
			case keys.LEFT_ARROW:
			case keys.RIGHT_ARROW:
				if(!this.edit.isEditing()){
					var keyCode = e.keyCode;  // IE seems to lose after stopEvent when modifier keys
					event.stop(e);
					colIdx = this.focus.getHeaderIndex();
					if (colIdx >= 0 && (e.shiftKey && e.ctrlKey)){
						this.focus.colSizeAdjust(e, colIdx, (keyCode == keys.LEFT_ARROW ? -1 : 1)*5);
					}
					else{
						var offset = (keyCode == keys.LEFT_ARROW) ? 1 : -1;
						if(this.isLeftToRight()){ offset *= -1; }
						this.focus.move(0, offset);
					}
				}
				break;
			case keys.UP_ARROW:
				if(!this.edit.isEditing() && this.focus.rowIndex !== 0){
					event.stop(e);
					this.focus.move(-1, 0);
				}
				break;
			case keys.DOWN_ARROW:
				if(!this.edit.isEditing() && this.focus.rowIndex+1 != this.rowCount){
					event.stop(e);
					this.focus.move(1, 0);
				}
				break;
			case keys.PAGE_UP:
				if(!this.edit.isEditing() && this.focus.rowIndex !== 0){
					event.stop(e);
					if(this.focus.rowIndex != this.scroller.firstVisibleRow+1){
						this.focus.move(this.scroller.firstVisibleRow-this.focus.rowIndex, 0);
					}else{
						this.setScrollTop(this.scroller.findScrollTop(this.focus.rowIndex-1));
						this.focus.move(this.scroller.firstVisibleRow-this.scroller.lastVisibleRow+1, 0);
					}
				}
				break;
			case keys.PAGE_DOWN:
				if(!this.edit.isEditing() && this.focus.rowIndex+1 != this.rowCount){
					event.stop(e);
					if(this.focus.rowIndex != this.scroller.lastVisibleRow-1){
						this.focus.move(this.scroller.lastVisibleRow-this.focus.rowIndex-1, 0);
					}else{
						this.setScrollTop(this.scroller.findScrollTop(this.focus.rowIndex+1));
						this.focus.move(this.scroller.lastVisibleRow-this.scroller.firstVisibleRow-1, 0);
					}
				}
				break;
			default:
				break;
		}
	},
	
	onMouseOver: function(e){
		// summary:
		//		Event fired when mouse is over the grid.
		// e: Event
		//		Decorated event object contains reference to grid, cell, and rowIndex
		e.rowIndex == -1 ? this.onHeaderCellMouseOver(e) : this.onCellMouseOver(e);
	},
	
	onMouseOut: function(e){
		// summary:
		//		Event fired when mouse moves out of the grid.
		// e: Event
		//		Decorated event object that contains reference to grid, cell, and rowIndex
		e.rowIndex == -1 ? this.onHeaderCellMouseOut(e) : this.onCellMouseOut(e);
	},
	
	onMouseDown: function(e){
		// summary:
		//		Event fired when mouse is down inside grid.
		// e: Event
		//		Decorated event object that contains reference to grid, cell, and rowIndex
		e.rowIndex == -1 ? this.onHeaderCellMouseDown(e) : this.onCellMouseDown(e);
	},
	
	onMouseOverRow: function(e){
		// summary:
		//		Event fired when mouse is over any row (data or header).
		// e: Event
		//		Decorated event object contains reference to grid, cell, and rowIndex
		if(!this.rows.isOver(e.rowIndex)){
			this.rows.setOverRow(e.rowIndex);
			e.rowIndex == -1 ? this.onHeaderMouseOver(e) : this.onRowMouseOver(e);
		}
	},
	onMouseOutRow: function(e){
		// summary:
		//		Event fired when mouse moves out of any row (data or header).
		// e: Event
		//		Decorated event object contains reference to grid, cell, and rowIndex
		if(this.rows.isOver(-1)){
			this.onHeaderMouseOut(e);
		}else if(!this.rows.isOver(-2)){
			this.rows.setOverRow(-2);
			this.onRowMouseOut(e);
		}
	},
	
	onMouseDownRow: function(e){
		// summary:
		//		Event fired when mouse is down inside grid row
		// e: Event
		//		Decorated event object that contains reference to grid, cell, and rowIndex
		if(e.rowIndex != -1)
			this.onRowMouseDown(e);
	},

	// cell events
	onCellMouseOver: function(e){
		// summary:
		//		Event fired when mouse is over a cell.
		// e: Event
		//		Decorated event object contains reference to grid, cell, and rowIndex
		if(e.cellNode){
			domClass.add(e.cellNode, this.cellOverClass);
		}
	},
	
	onCellMouseOut: function(e){
		// summary:
		//		Event fired when mouse moves out of a cell.
		// e: Event
		//		Decorated event object which contains reference to grid, cell, and rowIndex
		if(e.cellNode){
			domClass.remove(e.cellNode, this.cellOverClass);
		}
	},
	
	onCellMouseDown: function(e){
		// summary:
		//		Event fired when mouse is down in a header cell.
		// e: Event
		// 		Decorated event object which contains reference to grid, cell, and rowIndex
	},

	onCellClick: function(e){
		// summary:
		//		Event fired when a cell is clicked.
		// e: Event
		//		Decorated event object which contains reference to grid, cell, and rowIndex
		this._click[0] = this._click[1];
		this._click[1] = e;
		if(!this.edit.isEditCell(e.rowIndex, e.cellIndex)){
			this.focus.setFocusCell(e.cell, e.rowIndex);
		}
		// in some cases click[0] is null which causes false doubeClicks. Fixes #100703
		if(this._click.length > 1 && this._click[0] == null){
			this._click.shift();
		}
		this.onRowClick(e);
	},

	onCellDblClick: function(e){
		// summary:
		//		Event fired when a cell is double-clicked.
		// e: Event
		//		Decorated event object contains reference to grid, cell, and rowIndex
		var event;
		if(this._click.length > 1 && has("ie")){
			event = this._click[1];
		}else if(this._click.length > 1 && this._click[0].rowIndex != this._click[1].rowIndex){
			event = this._click[0];
		}else{
			event = e;
		}
		this.focus.setFocusCell(event.cell, event.rowIndex);
		this.onRowClick(event);
		this.edit.setEditCell(event.cell, event.rowIndex);
		this.onRowDblClick(e);
	},

	onCellContextMenu: function(e){
		// summary:
		//		Event fired when a cell context menu is accessed via mouse right click.
		// e: Event
		//		Decorated event object which contains reference to grid, cell, and rowIndex
		this.onRowContextMenu(e);
	},

	onCellFocus: function(inCell, inRowIndex){
		// summary:
		//		Event fired when a cell receives focus.
		// inCell: Object
		//		Cell object containing properties of the grid column.
		// inRowIndex: Integer
		//		Index of the grid row
		this.edit.cellFocus(inCell, inRowIndex);
	},

	// row events
	onRowClick: function(e){
		// summary:
		//		Event fired when a row is clicked.
		// e: Event
		//		Decorated event object which contains reference to grid, cell, and rowIndex
		this.edit.rowClick(e);
		this.selection.clickSelectEvent(e);
	},

	onRowDblClick: function(e){
		// summary:
		//		Event fired when a row is double clicked.
		// e: Event
		//		decorated event object which contains reference to grid, cell, and rowIndex
	},

	onRowMouseOver: function(e){
		// summary:
		//		Event fired when mouse moves over a data row.
		// e: Event
		//		Decorated event object which contains reference to grid, cell, and rowIndex
	},

	onRowMouseOut: function(e){
		// summary:
		//		Event fired when mouse moves out of a data row.
		// e: Event
		// 		Decorated event object contains reference to grid, cell, and rowIndex
	},
	
	onRowMouseDown: function(e){
		// summary:
		//		Event fired when mouse is down in a row.
		// e: Event
		// 		Decorated event object which contains reference to grid, cell, and rowIndex
	},

	onRowContextMenu: function(e){
		// summary:
		//		Event fired when a row context menu is accessed via mouse right click.
		// e: Event
		// 		Decorated event object which contains reference to grid, cell, and rowIndex
		event.stop(e);
	},

	// header events
	onHeaderMouseOver: function(e){
		// summary:
		//		Event fired when mouse moves over the grid header.
		// e: Event
		// 		Decorated event object contains reference to grid, cell, and rowIndex
	},

	onHeaderMouseOut: function(e){
		// summary:
		//		Event fired when mouse moves out of the grid header.
		// e: Event
		// 		Decorated event object which contains reference to grid, cell, and rowIndex
	},

	onHeaderCellMouseOver: function(e){
		// summary:
		//		Event fired when mouse moves over a header cell.
		// e: Event
		// 		Decorated event object which contains reference to grid, cell, and rowIndex
		if(e.cellNode){
			domClass.add(e.cellNode, this.cellOverClass);
		}
	},

	onHeaderCellMouseOut: function(e){
		// summary:
		//		Event fired when mouse moves out of a header cell.
		// e: Event
		// 		Decorated event object which contains reference to grid, cell, and rowIndex
		if(e.cellNode){
			domClass.remove(e.cellNode, this.cellOverClass);
		}
	},
	
	onHeaderCellMouseDown: function(e) {
		// summary:
		//		Event fired when mouse is down in a header cell.
		// e: Event
		// 		Decorated event object which contains reference to grid, cell, and rowIndex
	},

	onHeaderClick: function(e){
		// summary:
		//		Event fired when the grid header is clicked.
		// e: Event
		// Decorated event object which contains reference to grid, cell, and rowIndex
	},

	onHeaderCellClick: function(e){
		// summary:
		//		Event fired when a header cell is clicked.
		// e: Event
		//		Decorated event object which contains reference to grid, cell, and rowIndex
		this.setSortIndex(e.cell.index);
		this.onHeaderClick(e);
	},

	onHeaderDblClick: function(e){
		// summary:
		//		Event fired when the grid header is double clicked.
		// e: Event
		//		Decorated event object which contains reference to grid, cell, and rowIndex
	},

	onHeaderCellDblClick: function(e){
		// summary:
		//		Event fired when a header cell is double clicked.
		// e: Event
		//		Decorated event object which contains reference to grid, cell, and rowIndex
		this.onHeaderDblClick(e);
	},

	onHeaderCellContextMenu: function(e){
		// summary:
		//		Event fired when a header cell context menu is accessed via mouse right click.
		// e: Event
		//		Decorated event object which contains reference to grid, cell, and rowIndex
		this.onHeaderContextMenu(e);
	},

	onHeaderContextMenu: function(e){
		// summary:
		//		Event fired when the grid header context menu is accessed via mouse right click.
		// e: Event
		//		Decorated event object which contains reference to grid, cell, and rowIndex
		if(!this.headerMenu){
			event.stop(e);
		}
	},

	// editing
	onStartEdit: function(inCell, inRowIndex){
		// summary:
		//		Event fired when editing is started for a given grid cell
		// inCell: Object
		//		Cell object containing properties of the grid column.
		// inRowIndex: Integer
		//		Index of the grid row
	},

	onApplyCellEdit: function(inValue, inRowIndex, inFieldIndex){
		// summary:
		//		Event fired when editing is applied for a given grid cell
		// inValue: String
		//		Value from cell editor
		// inRowIndex: Integer
		//		Index of the grid row
		// inFieldIndex: Integer
		//		Index in the grid's data store
	},

	onCancelEdit: function(inRowIndex){
		// summary:
		//		Event fired when editing is cancelled for a given grid cell
		// inRowIndex: Integer
		//		Index of the grid row
	},

	onApplyEdit: function(inRowIndex){
		// summary:
		//		Event fired when editing is applied for a given grid row
		// inRowIndex: Integer
		//		Index of the grid row
	},

	onCanSelect: function(inRowIndex){
		// summary:
		//		Event to determine if a grid row may be selected
		// inRowIndex: Integer
		//		Index of the grid row
		// returns: Boolean
		//		true if the row can be selected
		return true;
	},

	onCanDeselect: function(inRowIndex){
		// summary:
		//		Event to determine if a grid row may be deselected
		// inRowIndex: Integer
		//		Index of the grid row
		// returns: Boolean
		//		true if the row can be deselected
		return true;
	},

	onSelected: function(inRowIndex){
		// summary:
		//		Event fired when a grid row is selected
		// inRowIndex: Integer
		//		Index of the grid row
		this.updateRowStyles(inRowIndex);
	},

	onDeselected: function(inRowIndex){
		// summary:
		//		Event fired when a grid row is deselected
		// inRowIndex: Integer
		//		Index of the grid row
		this.updateRowStyles(inRowIndex);
	},

	onSelectionChanged: function(){
	}
});
});
},
'dojo/regexp':function(){
define(["./_base/kernel", "./_base/lang"], function(dojo, lang) {
	// module:
	//		dojo/regexp
	// summary:
	//		TODOC

lang.getObject("regexp", true, dojo);

/*=====
dojo.regexp = {
	// summary: Regular expressions and Builder resources
};
=====*/

dojo.regexp.escapeString = function(/*String*/str, /*String?*/except){
	//	summary:
	//		Adds escape sequences for special characters in regular expressions
	// except:
	//		a String with special characters to be left unescaped

	return str.replace(/([\.$?*|{}\(\)\[\]\\\/\+^])/g, function(ch){
		if(except && except.indexOf(ch) != -1){
			return ch;
		}
		return "\\" + ch;
	}); // String
};

dojo.regexp.buildGroupRE = function(/*Object|Array*/arr, /*Function*/re, /*Boolean?*/nonCapture){
	//	summary:
	//		Builds a regular expression that groups subexpressions
	//	description:
	//		A utility function used by some of the RE generators. The
	//		subexpressions are constructed by the function, re, in the second
	//		parameter.  re builds one subexpression for each elem in the array
	//		a, in the first parameter. Returns a string for a regular
	//		expression that groups all the subexpressions.
	// arr:
	//		A single value or an array of values.
	// re:
	//		A function. Takes one parameter and converts it to a regular
	//		expression.
	// nonCapture:
	//		If true, uses non-capturing match, otherwise matches are retained
	//		by regular expression. Defaults to false

	// case 1: a is a single value.
	if(!(arr instanceof Array)){
		return re(arr); // String
	}

	// case 2: a is an array
	var b = [];
	for(var i = 0; i < arr.length; i++){
		// convert each elem to a RE
		b.push(re(arr[i]));
	}

	 // join the REs as alternatives in a RE group.
	return dojo.regexp.group(b.join("|"), nonCapture); // String
};

dojo.regexp.group = function(/*String*/expression, /*Boolean?*/nonCapture){
	// summary:
	//		adds group match to expression
	// nonCapture:
	//		If true, uses non-capturing match, otherwise matches are retained
	//		by regular expression.
	return "(" + (nonCapture ? "?:":"") + expression + ")"; // String
};

return dojo.regexp;
});

},
'ibm/usmi/statuspod/AlertStatusPod':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/statuspod/AlertStatusPod", [
	"ibm/tivoli/tip/TCTTable",
	"ibm/tivoli/tip/TCTNavModel",
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dijit/PopupMenuBarItem",
	"./StatusPodMenuDialog",
	"./AlertStatusPodDialog",
	"ibm/usmi/util/DataStore",
	"dojo/i18n!./nls/statuspodResources",
	"ibm/usmi/core/utils/_UsmiBrowserPersistence",
	"dojo/DeferredList",
	"dojo/_base/Deferred",
	'dojo/_base/unload',
	"./Constants" ], function (TCTTable, TCTNavModel, dojo, declare, lang, PopupMenuBarItem, StatusPodMenuDialog,
	AlertStatusPodDialog, DataStore, MRI, _UsmiBrowserPersistence, DeferredList, Deferred, baseUnload, Constants) {
	return declare("ibm.usmi.statuspod.AlertStatusPod", [
		PopupMenuBarItem,
		_UsmiBrowserPersistence ],
		{
			label : MRI.StatusPodLoading,

			dialog : null,
			listeningDataset : null,
			problemsDataset : null,
			complianceDataset : null,
			problemHandler : null,
			complianceHandler : null,
			menuParent : null,
			blockEvents : true,
			pendingEvents : false,
			_PERSISTENCE_ALERTS_ID : "ibm.usmi.statuspod.AlertStatusPod.contents",
			_PERSISTENCE_ALERTS_COUNT : "ibm.usmi.statuspod.AlertStatusPod.contents.count",

			problemsGroupProperties : [
				"DisplayName",
				"ObjectType",
				"STATUS",
				"STATUS_CAT:ComplianceStatusCategory",
				"STATUS_CAT:Problems",
				"STATUSPOD_SORT:Problems"],
				
			compliancesGroupProperties : [
				 "DisplayName",
				 "ObjectType",
				 "STATUS",
				 "STATUS_CAT:ComplianceStatusCategory",
				 "STATUS_CAT:Problems",
				 "STATUSPOD_SORT:ComplianceStatusCategory"],

			problemsGroupSort : [ {
				attribute : "STATUSPOD_SORT:Problems",
				descending : true
			} ],
			
			nonCompliancesGroupSort : [ {
				attribute : "STATUSPOD_SORT:ComplianceStatusCategory",
				descending : true
			} ],


			postCreate : function () {
				this.inherited(arguments);
				this.dialog = new AlertStatusPodDialog();
				this.popup = new StatusPodMenuDialog({
					content : this.dialog
				});
			},
			startup : function () {
				this.inherited(arguments);
				this._createDatasetListener();
				this._refreshDatasets();
			},
			
	        destroy : function () {
	            // clean up the datastore
				if (this.listeningDataset) {
					this.listeningDataset.getDataset().removeEventListener(this);
					this.listeningDataset.destroy();
					this.listeningDataset = null;
				}
			
				if (this.problemsDataset) {
					this.problemsDataset.destroy();
					this.problemsDataset = null;
				}
				if (this.complianceDataset) {
					this.complianceDataset.destroy();
					this.complianceDataset = null;
				}
					
	            this.inherited(arguments);
	        },
			_refreshDatasets : function () {
				var cache = this.getData(this._PERSISTENCE_ALERTS_ID, true);
				if (cache) {
					this._refreshProblems(cache.problems, true);
					this._refreshCompliance(cache.compliance, true);
					this._updateTopElement(true);
					return;
				}
				if (this.problemsDataset) {
					this.problemsDataset.destroy();
				}
				if (this.complianceDataset) {
					this.complianceDataset.destroy();
				}
				this.problemHandler = new Deferred();
				this.complianceHandler = new Deferred();
				var handlers = new DeferredList([
					this.problemHandler,
					this.complianceHandler ]);
				handlers.then(lang.hitch(this, function (result) {
					// only remove on Destroy so we don't do this at logout. 
					// deleting datasets at logout casues redirect popups
					//baseUnload.addOnUnload(this.problemsDataset.getDataset(), "destroy");
					//baseUnload.addOnUnload(this.complianceDataset.getDataset(), "destroy");
					this.setData(this._PERSISTENCE_ALERTS_ID, this.dialog.cache, true);
					this._updateTopElement();
					this.dialog.clearCache();

				}));
				this.problemsDataset = this.makeDataSet("SYSTEM_STATUSPOD", lang
					.hitch(this, this._refreshProblems), this.problemsGroupProperties, {
					groupId:"STATUS_PROBLEMS",
					STATUS_CAT: "Problems"
				}, this.problemsGroupSort);
				this.complianceDataset = this.makeDataSet("SYSTEM_STATUSPOD", lang.hitch(this,
					this._refreshCompliance), this.compliancesGroupProperties, {
					groupId:"STATUS_COMPLIANCE",
					STATUS_CAT: "ComplianceStatusCategory"
				}, this.nonCompliancesGroupSort);

			},
			_updateTopElement : function (fromCache) {
				var problemItems = this.dialog.statusItems;
				var complianceItems = this.dialog.complianceItems;
				var idx = 0, idx2 = 0, itemId, duplicates = 0;
				if (complianceItems) {
					for (idx=0; idx < complianceItems.length; idx++) {
						itemId = complianceItems[idx].itemId;
						for (idx2 = 0; idx2 < problemItems.length; idx2++) {
							if (itemId === problemItems[idx2].itemId) {
								duplicates++;
								break;
							}
						}
					}

				}
				var totalCounter = problemItems.length + complianceItems.length - duplicates;
				dojo.byId('alertStatusPodHeaderText').innerHTML = lang.replace(MRI.StatusHeaderText, [ totalCounter ]);
				var overallStatusIcon = this.determineOverallStatusIcon(this.determineStatusIcon(problemItems), this
					.determineStatusIcon(complianceItems));
				this.label = "<span class='" + overallStatusIcon + "'></span>"
					+ lang.replace(MRI.Status, [ totalCounter ]);
				this.containerNode.innerHTML = this.label;
				window.setTimeout(dojo.hitch(this, this._unblockEvents), '5000');
			},
			_refreshProblems : function (items, fromCache) {
				this.dialog.updateProblems(items, fromCache);
				if (this.problemHandler) {
					this.problemHandler.resolve();
				}
			},
			_refreshCompliance : function (items, fromCache) {
				this.dialog.updateCompliances(items, fromCache);
				if (this.complianceHandler) {
					this.complianceHandler.resolve();
				}
			},
			makeDataSet : function (datasetId, callback, groupProperties, parameters, sortProp) {
				return new DataStore({
					datasetId : datasetId,
					getTasks : false,
					initReadyCallback : callback					
				}, {
					sort : sortProp,
					scope : this,
					defaultProperties : groupProperties,
					createEventRouter : false,
					clearCacheOnRefresh : true,
					parameters: parameters
				//do not create event router
				});
			},

			determineStatusIcon : function (items) {
				var overallStatusIcon = Constants.STATUS_CLASS_NORMAL;
				if (!items || items.length === 0) {
					return overallStatusIcon;
				}
				var item = items[0];
				//var statusVal = item.getProperty("STATUS").getValue().getValue();
				var statusVal = item.severityId;
				if (statusVal === 1) {
					overallStatusIcon = Constants.STATUS_CLASS_WARNING;
				} else if (statusVal === 0) {
					overallStatusIcon = Constants.STATUS_CLASS_ERROR;
				} else if (statusVal === 5) {
					overallStatusIcon = Constants.STATUS_CLASS_UNKNOWN;
				}
				return overallStatusIcon;
			},

			determineOverallStatusIcon : function (statusProblems, statusCompliances) {
				if (statusProblems === Constants.STATUS_CLASS_ERROR
					|| statusCompliances === Constants.STATUS_CLASS_ERROR) {
					return Constants.STATUS_CLASS_ERROR;
				}
				if (statusProblems === Constants.STATUS_CLASS_WARNING
					|| statusCompliances === Constants.STATUS_CLASS_WARNING) {
					return Constants.STATUS_CLASS_WARNING;
				}
				
				if (statusProblems === Constants.STATUS_CLASS_NORMAL
						&& statusCompliances === Constants.STATUS_CLASS_UNKNOWN) {
						return Constants.STATUS_CLASS_UNKNOWN;
				}
				
				return Constants.STATUS_CLASS_NORMAL;
			},
			_setEventHandler : function () {
				this.listeningDataset.getDataset().addEventListener(this, lang.hitch(this, this._cleanDialog));
				var cachedCount = this.getData(this._PERSISTENCE_ALERTS_COUNT, false);
				var currentCount = this.listeningDataset.getDatasetAttribute("STATUS_COUNT");

				if (cachedCount) {
					if (cachedCount != currentCount) {
						this._cleanDialog();
					}
				} else {
					this.setData(this._PERSISTENCE_ALERTS_COUNT, currentCount, false);
				}
			},
			_createDatasetListener : function () {
				if (this.listeningDataset) {
					this.listeningDataset.destroy();
				}
				this.listeningDataset = new DataStore({
					datasetId : 'StatusPodCollection',
					getItems : false,
					retrieveItemProperties : false,
					getTasks : false,
					initReadyCallback : lang.hitch(this, this._setEventHandler)
				});
			},
			_cleanDialog : function () {
				if (this.blockEvents) { //Refresh is in progress, don't refresh
					this.pendingEvent = true; //save the need to refresh again later
					return;
				}
				this.blockEvents = true;
				this.pendingEvent = false;
				this.removeData(this._PERSISTENCE_ALERTS_ID);
				this.removeData(this._PERSISTENCE_ALERTS_COUNT);
				this._createDatasetListener();
				this.dialog.clearDescendants();
				this.label = MRI.StatusPodRefreshing;
				this.containerNode.innerHTML = this.label;
				dojo.byId('alertStatusPodHeaderText').innerHTML = MRI.StatusHeaderTextRefreshing;
				this._refreshDatasets();

			},
			_unblockEvents : function () {
				this.blockEvents = false; //Refresh is done, we can now listen for events again
				if (this.pendingEvent) { //did any more events come in during our refresh? if so, refresh again
					this._cleanDialog();
				}
			}
		});
});

},
'dijit/_editor/RichText':function(){
define([
	"dojo/_base/array", // array.forEach array.indexOf array.some
	"dojo/_base/config", // config
	"dojo/_base/declare", // declare
	"dojo/_base/Deferred", // Deferred
	"dojo/dom", // dom.byId
	"dojo/dom-attr", // domAttr.set or get
	"dojo/dom-class", // domClass.add domClass.remove
	"dojo/dom-construct", // domConstruct.create domConstruct.destroy domConstruct.place
	"dojo/dom-geometry", // domGeometry.getMarginBox domGeometry.position
	"dojo/dom-style", // domStyle.getComputedStyle domStyle.set
	"dojo/_base/event", // event.stop
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/keys", // keys.BACKSPACE keys.TAB
	"dojo/_base/lang", // lang.clone lang.hitch lang.isArray lang.isFunction lang.isString lang.trim
	"dojo/on", // on()
	"dojo/query", // query
	"dojo/ready", // ready
	"dojo/_base/sniff", // has("ie") has("mozilla") has("opera") has("safari") has("webkit")
	"dojo/topic",	// topic.publish() (publish)
	"dojo/_base/unload", // unload
	"dojo/_base/url", // url
	"dojo/_base/window", // win.body win.doc.body.focus win.doc.createElement win.global.location win.withGlobal
	"../_Widget",
	"../_CssStateMixin",
	"./selection",
	"./range",
	"./html",
	"../focus",
	".."	// dijit._scopeName
], function(array, config, declare, Deferred, dom, domAttr, domClass, domConstruct, domGeometry, domStyle,
	event, kernel, keys, lang, on, query, ready, has, topic, unload, _Url, win,
	_Widget, _CssStateMixin, selectionapi, rangeapi, htmlapi, focus, dijit){

/*=====
	var _Widget = dijit._Widget;
	var _CssStateMixin = dijit._CssStateMixin;
=====*/

// module:
//		dijit/_editor/RichText
// summary:
//		dijit._editor.RichText is the core of dijit.Editor, which provides basic
//		WYSIWYG editing features.

// if you want to allow for rich text saving with back/forward actions, you must add a text area to your page with
// the id==dijit._scopeName + "._editor.RichText.value" (typically "dijit._editor.RichText.value). For example,
// something like this will work:
//
//	<textarea id="dijit._editor.RichText.value" style="display:none;position:absolute;top:-100px;left:-100px;height:3px;width:3px;overflow:hidden;"></textarea>
//

var RichText = declare("dijit._editor.RichText", [_Widget, _CssStateMixin], {
	// summary:
	//		dijit._editor.RichText is the core of dijit.Editor, which provides basic
	//		WYSIWYG editing features.
	//
	// description:
	//		dijit._editor.RichText is the core of dijit.Editor, which provides basic
	//		WYSIWYG editing features. It also encapsulates the differences
	//		of different js engines for various browsers.  Do not use this widget
	//		with an HTML &lt;TEXTAREA&gt; tag, since the browser unescapes XML escape characters,
	//		like &lt;.  This can have unexpected behavior and lead to security issues
	//		such as scripting attacks.
	//
	// tags:
	//		private

	constructor: function(params){
		// contentPreFilters: Function(String)[]
		//		Pre content filter function register array.
		//		these filters will be executed before the actual
		//		editing area gets the html content.
		this.contentPreFilters = [];

		// contentPostFilters: Function(String)[]
		//		post content filter function register array.
		//		These will be used on the resulting html
		//		from contentDomPostFilters. The resulting
		//		content is the final html (returned by getValue()).
		this.contentPostFilters = [];

		// contentDomPreFilters: Function(DomNode)[]
		//		Pre content dom filter function register array.
		//		These filters are applied after the result from
		//		contentPreFilters are set to the editing area.
		this.contentDomPreFilters = [];

		// contentDomPostFilters: Function(DomNode)[]
		//		Post content dom filter function register array.
		//		These filters are executed on the editing area dom.
		//		The result from these will be passed to contentPostFilters.
		this.contentDomPostFilters = [];

		// editingAreaStyleSheets: dojo._URL[]
		//		array to store all the stylesheets applied to the editing area
		this.editingAreaStyleSheets = [];

		// Make a copy of this.events before we start writing into it, otherwise we
		// will modify the prototype which leads to bad things on pages w/multiple editors
		this.events = [].concat(this.events);

		this._keyHandlers = {};

		if(params && lang.isString(params.value)){
			this.value = params.value;
		}

		this.onLoadDeferred = new Deferred();
	},

	baseClass: "dijitEditor",

	// inheritWidth: Boolean
	//		whether to inherit the parent's width or simply use 100%
	inheritWidth: false,

	// focusOnLoad: [deprecated] Boolean
	//		Focus into this widget when the page is loaded
	focusOnLoad: false,

	// name: String?
	//		Specifies the name of a (hidden) <textarea> node on the page that's used to save
	//		the editor content on page leave.   Used to restore editor contents after navigating
	//		to a new page and then hitting the back button.
	name: "",

	// styleSheets: [const] String
	//		semicolon (";") separated list of css files for the editing area
	styleSheets: "",

	// height: String
	//		Set height to fix the editor at a specific height, with scrolling.
	//		By default, this is 300px.  If you want to have the editor always
	//		resizes to accommodate the content, use AlwaysShowToolbar plugin
	//		and set height="".  If this editor is used within a layout widget,
	//		set height="100%".
	height: "300px",

	// minHeight: String
	//		The minimum height that the editor should have.
	minHeight: "1em",

	// isClosed: [private] Boolean
	isClosed: true,

	// isLoaded: [private] Boolean
	isLoaded: false,

	// _SEPARATOR: [private] String
	//		Used to concat contents from multiple editors into a single string,
	//		so they can be saved into a single <textarea> node.  See "name" attribute.
	_SEPARATOR: "@@**%%__RICHTEXTBOUNDRY__%%**@@",

	// _NAME_CONTENT_SEP: [private] String
	//		USed to separate name from content.  Just a colon isn't safe.
	_NAME_CONTENT_SEP: "@@**%%:%%**@@",

	// onLoadDeferred: [readonly] dojo.Deferred
	//		Deferred which is fired when the editor finishes loading.
	//		Call myEditor.onLoadDeferred.then(callback) it to be informed
	//		when the rich-text area initialization is finalized.
	onLoadDeferred: null,

	// isTabIndent: Boolean
	//		Make tab key and shift-tab indent and outdent rather than navigating.
	//		Caution: sing this makes web pages inaccessible to users unable to use a mouse.
	isTabIndent: false,

	// disableSpellCheck: [const] Boolean
	//		When true, disables the browser's native spell checking, if supported.
	//		Works only in Firefox.
	disableSpellCheck: false,

	postCreate: function(){
		if("textarea" === this.domNode.tagName.toLowerCase()){
			console.warn("RichText should not be used with the TEXTAREA tag.  See dijit._editor.RichText docs.");
		}

		// Push in the builtin filters now, making them the first executed, but not over-riding anything
		// users passed in.  See: #6062
		this.contentPreFilters = [lang.hitch(this, "_preFixUrlAttributes")].concat(this.contentPreFilters);
		if(has("mozilla")){
			this.contentPreFilters = [this._normalizeFontStyle].concat(this.contentPreFilters);
			this.contentPostFilters = [this._removeMozBogus].concat(this.contentPostFilters);
		}
		if(has("webkit")){
			// Try to clean up WebKit bogus artifacts.  The inserted classes
			// made by WebKit sometimes messes things up.
			this.contentPreFilters = [this._removeWebkitBogus].concat(this.contentPreFilters);
			this.contentPostFilters = [this._removeWebkitBogus].concat(this.contentPostFilters);
		}
		if(has("ie")){
			// IE generates <strong> and <em> but we want to normalize to <b> and <i>
			this.contentPostFilters = [this._normalizeFontStyle].concat(this.contentPostFilters);
			this.contentDomPostFilters = [lang.hitch(this, this._stripBreakerNodes)].concat(this.contentDomPostFilters);
		}
		this.inherited(arguments);

		topic.publish(dijit._scopeName + "._editor.RichText::init", this);
		this.open();
		this.setupDefaultShortcuts();
	},

	setupDefaultShortcuts: function(){
		// summary:
		//		Add some default key handlers
		// description:
		//		Overwrite this to setup your own handlers. The default
		//		implementation does not use Editor commands, but directly
		//		executes the builtin commands within the underlying browser
		//		support.
		// tags:
		//		protected
		var exec = lang.hitch(this, function(cmd, arg){
			return function(){
				return !this.execCommand(cmd,arg);
			};
		});

		var ctrlKeyHandlers = {
			b: exec("bold"),
			i: exec("italic"),
			u: exec("underline"),
			a: exec("selectall"),
			s: function(){ this.save(true); },
			m: function(){ this.isTabIndent = !this.isTabIndent; },

			"1": exec("formatblock", "h1"),
			"2": exec("formatblock", "h2"),
			"3": exec("formatblock", "h3"),
			"4": exec("formatblock", "h4"),

			"\\": exec("insertunorderedlist")
		};

		if(!has("ie")){
			ctrlKeyHandlers.Z = exec("redo"); //FIXME: undo?
		}

		var key;
		for(key in ctrlKeyHandlers){
			this.addKeyHandler(key, true, false, ctrlKeyHandlers[key]);
		}
	},

	// events: [private] String[]
	//		 events which should be connected to the underlying editing area
	events: ["onKeyPress", "onKeyDown", "onKeyUp"], // onClick handled specially

	// captureEvents: [deprecated] String[]
	//		 Events which should be connected to the underlying editing
	//		 area, events in this array will be addListener with
	//		 capture=true.
	// TODO: looking at the code I don't see any distinction between events and captureEvents,
	// so get rid of this for 2.0 if not sooner
	captureEvents: [],

	_editorCommandsLocalized: false,
	_localizeEditorCommands: function(){
		// summary:
		//		When IE is running in a non-English locale, the API actually changes,
		//		so that we have to say (for example) danraku instead of p (for paragraph).
		//		Handle that here.
		// tags:
		//		private
		if(RichText._editorCommandsLocalized){
			// Use the already generate cache of mappings.
			this._local2NativeFormatNames = RichText._local2NativeFormatNames;
			this._native2LocalFormatNames = RichText._native2LocalFormatNames;
			return;
		}
		RichText._editorCommandsLocalized = true;
		RichText._local2NativeFormatNames = {};
		RichText._native2LocalFormatNames = {};
		this._local2NativeFormatNames = RichText._local2NativeFormatNames;
		this._native2LocalFormatNames = RichText._native2LocalFormatNames;
		//in IE, names for blockformat is locale dependent, so we cache the values here

		//put p after div, so if IE returns Normal, we show it as paragraph
		//We can distinguish p and div if IE returns Normal, however, in order to detect that,
		//we have to call this.document.selection.createRange().parentElement() or such, which
		//could slow things down. Leave it as it is for now
		var formats = ['div', 'p', 'pre', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'ol', 'ul', 'address'];
		var localhtml = "", format, i=0;
		while((format=formats[i++])){
			//append a <br> after each element to separate the elements more reliably
			if(format.charAt(1) !== 'l'){
				localhtml += "<"+format+"><span>content</span></"+format+"><br/>";
			}else{
				localhtml += "<"+format+"><li>content</li></"+format+"><br/>";
			}
		}
		// queryCommandValue returns empty if we hide editNode, so move it out of screen temporary
		// Also, IE9 does weird stuff unless we do it inside the editor iframe.
		var style = { position: "absolute", top: "0px", zIndex: 10, opacity: 0.01 };
		var div = domConstruct.create('div', {style: style, innerHTML: localhtml});
		win.body().appendChild(div);

		// IE9 has a timing issue with doing this right after setting
		// the inner HTML, so put a delay in.
		var inject = lang.hitch(this, function(){
			var node = div.firstChild;
			while(node){
				try{
					selectionapi.selectElement(node.firstChild);
					var nativename = node.tagName.toLowerCase();
					this._local2NativeFormatNames[nativename] = document.queryCommandValue("formatblock");
					this._native2LocalFormatNames[this._local2NativeFormatNames[nativename]] = nativename;
					node = node.nextSibling.nextSibling;
					//console.log("Mapped: ", nativename, " to: ", this._local2NativeFormatNames[nativename]);
				}catch(e){ /*Sqelch the occasional IE9 error */ }
			}
			div.parentNode.removeChild(div);
			div.innerHTML = "";
		});
		setTimeout(inject, 0);
	},

	open: function(/*DomNode?*/ element){
		// summary:
		//		Transforms the node referenced in this.domNode into a rich text editing
		//		node.
		// description:
		//		Sets up the editing area asynchronously. This will result in
		//		the creation and replacement with an iframe.
		// tags:
		//		private

		if(!this.onLoadDeferred || this.onLoadDeferred.fired >= 0){
			this.onLoadDeferred = new Deferred();
		}

		if(!this.isClosed){ this.close(); }
		topic.publish(dijit._scopeName + "._editor.RichText::open", this);

		if(arguments.length === 1 && element.nodeName){ // else unchanged
			this.domNode = element;
		}

		var dn = this.domNode;

		// "html" will hold the innerHTML of the srcNodeRef and will be used to
		// initialize the editor.
		var html;

		if(lang.isString(this.value)){
			// Allow setting the editor content programmatically instead of
			// relying on the initial content being contained within the target
			// domNode.
			html = this.value;
			delete this.value;
			dn.innerHTML = "";
		}else if(dn.nodeName && dn.nodeName.toLowerCase() == "textarea"){
			// if we were created from a textarea, then we need to create a
			// new editing harness node.
			var ta = (this.textarea = dn);
			this.name = ta.name;
			html = ta.value;
			dn = this.domNode = win.doc.createElement("div");
			dn.setAttribute('widgetId', this.id);
			ta.removeAttribute('widgetId');
			dn.cssText = ta.cssText;
			dn.className += " " + ta.className;
			domConstruct.place(dn, ta, "before");
			var tmpFunc = lang.hitch(this, function(){
				//some browsers refuse to submit display=none textarea, so
				//move the textarea off screen instead
				domStyle.set(ta, {
					display: "block",
					position: "absolute",
					top: "-1000px"
				});

				if(has("ie")){ //nasty IE bug: abnormal formatting if overflow is not hidden
					var s = ta.style;
					this.__overflow = s.overflow;
					s.overflow = "hidden";
				}
			});
			if(has("ie")){
				setTimeout(tmpFunc, 10);
			}else{
				tmpFunc();
			}

			if(ta.form){
				var resetValue = ta.value;
				this.reset = function(){
					var current = this.getValue();
					if(current !== resetValue){
						this.replaceValue(resetValue);
					}
				};
				on(ta.form, "submit", lang.hitch(this, function(){
					// Copy value to the <textarea> so it gets submitted along with form.
					// FIXME: should we be calling close() here instead?
					domAttr.set(ta, 'disabled', this.disabled); // don't submit the value if disabled
					ta.value = this.getValue();
				}));
			}
		}else{
			html = htmlapi.getChildrenHtml(dn);
			dn.innerHTML = "";
		}

		this.value = html;

		// If we're a list item we have to put in a blank line to force the
		// bullet to nicely align at the top of text
		if(dn.nodeName && dn.nodeName === "LI"){
			dn.innerHTML = " <br>";
		}

		// Construct the editor div structure.
		this.header = dn.ownerDocument.createElement("div");
		dn.appendChild(this.header);
		this.editingArea = dn.ownerDocument.createElement("div");
		dn.appendChild(this.editingArea);
		this.footer = dn.ownerDocument.createElement("div");
		dn.appendChild(this.footer);

		if(!this.name){
			this.name = this.id + "_AUTOGEN";
		}

		// User has pressed back/forward button so we lost the text in the editor, but it's saved
		// in a hidden <textarea> (which contains the data for all the editors on this page),
		// so get editor value from there
		if(this.name !== "" && (!config["useXDomain"] || config["allowXdRichTextSave"])){
			var saveTextarea = dom.byId(dijit._scopeName + "._editor.RichText.value");
			if(saveTextarea && saveTextarea.value !== ""){
				var datas = saveTextarea.value.split(this._SEPARATOR), i=0, dat;
				while((dat=datas[i++])){
					var data = dat.split(this._NAME_CONTENT_SEP);
					if(data[0] === this.name){
						html = data[1];
						datas = datas.splice(i, 1);
						saveTextarea.value = datas.join(this._SEPARATOR);
						break;
					}
				}
			}

			if(!RichText._globalSaveHandler){
				RichText._globalSaveHandler = {};
				unload.addOnUnload(function(){
					var id;
					for(id in RichText._globalSaveHandler){
						var f = RichText._globalSaveHandler[id];
						if(lang.isFunction(f)){
							f();
						}
					}
				});
			}
			RichText._globalSaveHandler[this.id] = lang.hitch(this, "_saveContent");
		}

		this.isClosed = false;

		var ifr = (this.editorObject = this.iframe = win.doc.createElement('iframe'));
		ifr.id = this.id+"_iframe";
		this._iframeSrc = this._getIframeDocTxt();
		ifr.style.border = "none";
		ifr.style.width = "100%";
		if(this._layoutMode){
			// iframe should be 100% height, thus getting it's height from surrounding
			// <div> (which has the correct height set by Editor)
			ifr.style.height = "100%";
		}else{
			if(has("ie") >= 7){
				if(this.height){
					ifr.style.height = this.height;
				}
				if(this.minHeight){
					ifr.style.minHeight = this.minHeight;
				}
			}else{
				ifr.style.height = this.height ? this.height : this.minHeight;
			}
		}
		ifr.frameBorder = 0;
		ifr._loadFunc = lang.hitch( this, function(w){
			this.window = w;
			this.document = this.window.document;

			if(has("ie")){
				this._localizeEditorCommands();
			}

			// Do final setup and set initial contents of editor
			this.onLoad(html);
		});

		// Set the iframe's initial (blank) content.
		var iframeSrcRef = 'parent.' + dijit._scopeName + '.byId("'+this.id+'")._iframeSrc';
		var s = 'javascript:(function(){try{return ' + iframeSrcRef + '}catch(e){document.open();document.domain="' +
				document.domain + '";document.write(' + iframeSrcRef + ');document.close();}})()';
		ifr.setAttribute('src', s);
		this.editingArea.appendChild(ifr);

		if(has("safari") <= 4){
			var src = ifr.getAttribute("src");
			if(!src || src.indexOf("javascript") === -1){
				// Safari 4 and earlier sometimes act oddly
				// So we have to set it again.
				setTimeout(function(){ifr.setAttribute('src', s);},0);
			}
		}

		// TODO: this is a guess at the default line-height, kinda works
		if(dn.nodeName === "LI"){
			dn.lastChild.style.marginTop = "-1.2em";
		}

		domClass.add(this.domNode, this.baseClass);
	},

	//static cache variables shared among all instance of this class
	_local2NativeFormatNames: {},
	_native2LocalFormatNames: {},

	_getIframeDocTxt: function(){
		// summary:
		//		Generates the boilerplate text of the document inside the iframe (ie, <html><head>...</head><body/></html>).
		//		Editor content (if not blank) should be added afterwards.
		// tags:
		//		private
		var _cs = domStyle.getComputedStyle(this.domNode);

		// The contents inside of <body>.  The real contents are set later via a call to setValue().
		var html = "";
		var setBodyId = true;
		if(has("ie") || has("webkit") || (!this.height && !has("mozilla"))){
			// In auto-expand mode, need a wrapper div for AlwaysShowToolbar plugin to correctly
			// expand/contract the editor as the content changes.
			html = "<div id='dijitEditorBody'></div>";
			setBodyId = false;
		}else if(has("mozilla")){
			// workaround bug where can't select then delete text (until user types something
			// into the editor)... and/or issue where typing doesn't erase selected text
			this._cursorToStart = true;
			html = "&#160;";	// &nbsp;
		}

		var font = [ _cs.fontWeight, _cs.fontSize, _cs.fontFamily ].join(" ");

		// line height is tricky - applying a units value will mess things up.
		// if we can't get a non-units value, bail out.
		var lineHeight = _cs.lineHeight;
		if(lineHeight.indexOf("px") >= 0){
			lineHeight = parseFloat(lineHeight)/parseFloat(_cs.fontSize);
			// console.debug(lineHeight);
		}else if(lineHeight.indexOf("em")>=0){
			lineHeight = parseFloat(lineHeight);
		}else{
			// If we can't get a non-units value, just default
			// it to the CSS spec default of 'normal'.  Seems to
			// work better, esp on IE, than '1.0'
			lineHeight = "normal";
		}
		var userStyle = "";
		var self = this;
		this.style.replace(/(^|;)\s*(line-|font-?)[^;]+/ig, function(match){
			match = match.replace(/^;/ig,"") + ';';
			var s = match.split(":")[0];
			if(s){
				s = lang.trim(s);
				s = s.toLowerCase();
				var i;
				var sC = "";
				for(i = 0; i < s.length; i++){
					var c = s.charAt(i);
					switch(c){
						case "-":
							i++;
							c = s.charAt(i).toUpperCase();
						default:
							sC += c;
					}
				}
				domStyle.set(self.domNode, sC, "");
			}
			userStyle += match + ';';
		});


		// need to find any associated label element and update iframe document title
		var label=query('label[for="'+this.id+'"]');

		return [
			this.isLeftToRight() ? "<html>\n<head>\n" : "<html dir='rtl'>\n<head>\n",
			(has("mozilla") && label.length ? "<title>" + label[0].innerHTML + "</title>\n" : ""),
			"<meta http-equiv='Content-Type' content='text/html'>\n",
			"<style>\n",
			"\tbody,html {\n",
			"\t\tbackground:transparent;\n",
			"\t\tpadding: 1px 0 0 0;\n",
			"\t\tmargin: -1px 0 0 0;\n", // remove extraneous vertical scrollbar on safari and firefox

			// Set the html/body sizing.  Webkit always needs this, other browsers
			// only set it when height is defined (not auto-expanding), otherwise
			// scrollers do not appear.
			((has("webkit"))?"\t\twidth: 100%;\n":""),
			((has("webkit"))?"\t\theight: 100%;\n":""),
			"\t}\n",

			// TODO: left positioning will cause contents to disappear out of view
			//	   if it gets too wide for the visible area
			"\tbody{\n",
			"\t\ttop:0px;\n",
			"\t\tleft:0px;\n",
			"\t\tright:0px;\n",
			"\t\tfont:", font, ";\n",
				((this.height||has("opera")) ? "" : "\t\tposition: fixed;\n"),
			// FIXME: IE 6 won't understand min-height?
			"\t\tmin-height:", this.minHeight, ";\n",
			"\t\tline-height:", lineHeight,";\n",
			"\t}\n",
			"\tp{ margin: 1em 0; }\n",

			// Determine how scrollers should be applied.  In autoexpand mode (height = "") no scrollers on y at all.
			// But in fixed height mode we want both x/y scrollers.  Also, if it's using wrapping div and in auto-expand
			// (Mainly IE) we need to kill the y scroller on body and html.
			(!setBodyId && !this.height ? "\tbody,html {overflow-y: hidden;}\n" : ""),
			"\t#dijitEditorBody{overflow-x: auto; overflow-y:" + (this.height ? "auto;" : "hidden;") + " outline: 0px;}\n",
			"\tli > ul:-moz-first-node, li > ol:-moz-first-node{ padding-top: 1.2em; }\n",
			// Can't set min-height in IE9, it puts layout on li, which puts move/resize handles.
			(!has("ie") ? "\tli{ min-height:1.2em; }\n" : ""),
			"</style>\n",
			this._applyEditingAreaStyleSheets(),"\n",
			"</head>\n<body ",
			(setBodyId?"id='dijitEditorBody' ":""),
			"onload='frameElement._loadFunc(window,document)' style='"+userStyle+"'>", html, "</body>\n</html>"
		].join(""); // String
	},

	_applyEditingAreaStyleSheets: function(){
		// summary:
		//		apply the specified css files in styleSheets
		// tags:
		//		private
		var files = [];
		if(this.styleSheets){
			files = this.styleSheets.split(';');
			this.styleSheets = '';
		}

		//empty this.editingAreaStyleSheets here, as it will be filled in addStyleSheet
		files = files.concat(this.editingAreaStyleSheets);
		this.editingAreaStyleSheets = [];

		var text='', i=0, url;
		while((url=files[i++])){
			var abstring = (new _Url(win.global.location, url)).toString();
			this.editingAreaStyleSheets.push(abstring);
			text += '<link rel="stylesheet" type="text/css" href="'+abstring+'"/>';
		}
		return text;
	},

	addStyleSheet: function(/*dojo._Url*/ uri){
		// summary:
		//		add an external stylesheet for the editing area
		// uri:
		//		A dojo.uri.Uri pointing to the url of the external css file
		var url=uri.toString();

		//if uri is relative, then convert it to absolute so that it can be resolved correctly in iframe
		if(url.charAt(0) === '.' || (url.charAt(0) !== '/' && !uri.host)){
			url = (new _Url(win.global.location, url)).toString();
		}

		if(array.indexOf(this.editingAreaStyleSheets, url) > -1){
//			console.debug("dijit._editor.RichText.addStyleSheet: Style sheet "+url+" is already applied");
			return;
		}

		this.editingAreaStyleSheets.push(url);
		this.onLoadDeferred.addCallback(lang.hitch(this, function(){
			if(this.document.createStyleSheet){ //IE
				this.document.createStyleSheet(url);
			}else{ //other browser
				var head = this.document.getElementsByTagName("head")[0];
				var stylesheet = this.document.createElement("link");
				stylesheet.rel="stylesheet";
				stylesheet.type="text/css";
				stylesheet.href=url;
				head.appendChild(stylesheet);
			}
		}));
	},

	removeStyleSheet: function(/*dojo._Url*/ uri){
		// summary:
		//		remove an external stylesheet for the editing area
		var url=uri.toString();
		//if uri is relative, then convert it to absolute so that it can be resolved correctly in iframe
		if(url.charAt(0) === '.' || (url.charAt(0) !== '/' && !uri.host)){
			url = (new _Url(win.global.location, url)).toString();
		}
		var index = array.indexOf(this.editingAreaStyleSheets, url);
		if(index === -1){
//			console.debug("dijit._editor.RichText.removeStyleSheet: Style sheet "+url+" has not been applied");
			return;
		}
		delete this.editingAreaStyleSheets[index];
		win.withGlobal(this.window,'query', dojo, ['link:[href="'+url+'"]']).orphan();
	},

	// disabled: Boolean
	//		The editor is disabled; the text cannot be changed.
	disabled: false,

	_mozSettingProps: {'styleWithCSS':false},
	_setDisabledAttr: function(/*Boolean*/ value){
		value = !!value;
		this._set("disabled", value);
		if(!this.isLoaded){ return; } // this method requires init to be complete
		if(has("ie") || has("webkit") || has("opera")){
			var preventIEfocus = has("ie") && (this.isLoaded || !this.focusOnLoad);
			if(preventIEfocus){ this.editNode.unselectable = "on"; }
			this.editNode.contentEditable = !value;
			if(preventIEfocus){
				var _this = this;
				setTimeout(function(){
					if(_this.editNode){		// guard in case widget destroyed before timeout
						_this.editNode.unselectable = "off";
					}
				}, 0);
			}
		}else{ //moz
			try{
				this.document.designMode=(value?'off':'on');
			}catch(e){ return; } // ! _disabledOK
			if(!value && this._mozSettingProps){
				var ps = this._mozSettingProps;
				var n;
				for(n in ps){
					if(ps.hasOwnProperty(n)){
						try{
							this.document.execCommand(n,false,ps[n]);
						}catch(e2){}
					}
				}
			}
//			this.document.execCommand('contentReadOnly', false, value);
//				if(value){
//					this.blur(); //to remove the blinking caret
//				}
		}
		this._disabledOK = true;
	},

/* Event handlers
 *****************/

	onLoad: function(/*String*/ html){
		// summary:
		//		Handler after the iframe finishes loading.
		// html: String
		//		Editor contents should be set to this value
		// tags:
		//		protected

		// TODO: rename this to _onLoad, make empty public onLoad() method, deprecate/make protected onLoadDeferred handler?

		if(!this.window.__registeredWindow){
			this.window.__registeredWindow = true;
			this._iframeRegHandle = focus.registerIframe(this.iframe);
		}
		if(!has("ie") && !has("webkit") && (this.height || has("mozilla"))){
			this.editNode=this.document.body;
		}else{
			// there's a wrapper div around the content, see _getIframeDocTxt().
			this.editNode=this.document.body.firstChild;
			var _this = this;
			if(has("ie")){ // #4996 IE wants to focus the BODY tag
				this.tabStop = domConstruct.create('div', { tabIndex: -1 }, this.editingArea);
				this.iframe.onfocus = function(){ _this.editNode.setActive(); };
			}
		}
		this.focusNode = this.editNode; // for InlineEditBox


		var events = this.events.concat(this.captureEvents);
		var ap = this.iframe ? this.document : this.editNode;
		array.forEach(events, function(item){
			this.connect(ap, item.toLowerCase(), item);
		}, this);

		this.connect(ap, "onmouseup", "onClick"); // mouseup in the margin does not generate an onclick event

		if(has("ie")){ // IE contentEditable
			this.connect(this.document, "onmousedown", "_onIEMouseDown"); // #4996 fix focus

			// give the node Layout on IE
			// TODO: this may no longer be needed, since we've reverted IE to using an iframe,
			// not contentEditable.   Removing it would also probably remove the need for creating
			// the extra <div> in _getIframeDocTxt()
			this.editNode.style.zoom = 1.0;
		}else{
			this.connect(this.document, "onmousedown", function(){
				// Clear the moveToStart focus, as mouse
				// down will set cursor point.  Required to properly
				// work with selection/position driven plugins and clicks in
				// the window. refs: #10678
				delete this._cursorToStart;
			});
		}

		if(has("webkit")){
			//WebKit sometimes doesn't fire right on selections, so the toolbar
			//doesn't update right.  Therefore, help it out a bit with an additional
			//listener.  A mouse up will typically indicate a display change, so fire this
			//and get the toolbar to adapt.  Reference: #9532
			this._webkitListener = this.connect(this.document, "onmouseup", "onDisplayChanged");
			this.connect(this.document, "onmousedown", function(e){
				var t = e.target;
				if(t && (t === this.document.body || t === this.document)){
					// Since WebKit uses the inner DIV, we need to check and set position.
					// See: #12024 as to why the change was made.
					setTimeout(lang.hitch(this, "placeCursorAtEnd"), 0);
				}
			});
		}

		if(has("ie")){
			// Try to make sure 'hidden' elements aren't visible in edit mode (like browsers other than IE
			// do).  See #9103
			try{
				this.document.execCommand('RespectVisibilityInDesign', true, null);
			}catch(e){/* squelch */}
		}

		this.isLoaded = true;

		this.set('disabled', this.disabled); // initialize content to editable (or not)

		// Note that setValue() call will only work after isLoaded is set to true (above)

		// Set up a function to allow delaying the setValue until a callback is fired
		// This ensures extensions like dijit.Editor have a way to hold the value set
		// until plugins load (and do things like register filters).
		var setContent = lang.hitch(this, function(){
			this.setValue(html);
			if(this.onLoadDeferred){
				this.onLoadDeferred.callback(true);
			}
			this.onDisplayChanged();
			if(this.focusOnLoad){
				// after the document loads, then set focus after updateInterval expires so that
				// onNormalizedDisplayChanged has run to avoid input caret issues
				ready(lang.hitch(this, function(){ setTimeout(lang.hitch(this, "focus"), this.updateInterval); }));
			}
			// Save off the initial content now
			this.value = this.getValue(true);
		});
		if(this.setValueDeferred){
			this.setValueDeferred.addCallback(setContent);
		}else{
			setContent();
		}
	},

	onKeyDown: function(/* Event */ e){
		// summary:
		//		Handler for onkeydown event
		// tags:
		//		protected

		// we need this event at the moment to get the events from control keys
		// such as the backspace. It might be possible to add this to Dojo, so that
		// keyPress events can be emulated by the keyDown and keyUp detection.

		if(e.keyCode === keys.TAB && this.isTabIndent ){
			event.stop(e); //prevent tab from moving focus out of editor

			// FIXME: this is a poor-man's indent/outdent. It would be
			// better if it added 4 "&nbsp;" chars in an undoable way.
			// Unfortunately pasteHTML does not prove to be undoable
			if(this.queryCommandEnabled((e.shiftKey ? "outdent" : "indent"))){
				this.execCommand((e.shiftKey ? "outdent" : "indent"));
			}
		}
		if(has("ie")){
			if(e.keyCode == keys.TAB && !this.isTabIndent){
				if(e.shiftKey && !e.ctrlKey && !e.altKey){
					// focus the BODY so the browser will tab away from it instead
					this.iframe.focus();
				}else if(!e.shiftKey && !e.ctrlKey && !e.altKey){
					// focus the BODY so the browser will tab away from it instead
					this.tabStop.focus();
				}
			}else if(e.keyCode === keys.BACKSPACE && this.document.selection.type === "Control"){
				// IE has a bug where if a non-text object is selected in the editor,
				// hitting backspace would act as if the browser's back button was
				// clicked instead of deleting the object. see #1069
				event.stop(e);
				this.execCommand("delete");
			}else if((65 <= e.keyCode && e.keyCode <= 90) ||
				(e.keyCode>=37 && e.keyCode<=40) // FIXME: get this from connect() instead!
			){ //arrow keys
				e.charCode = e.keyCode;
				this.onKeyPress(e);
			}
		}
		if(has("ff")){
			if(e.keyCode === keys.PAGE_UP || e.keyCode === keys.PAGE_DOWN ){
				if(this.editNode.clientHeight >= this.editNode.scrollHeight){
					// Stop the event to prevent firefox from trapping the cursor when there is no scroll bar.
					e.preventDefault();
				}
			}
		}
		return true;
	},

	onKeyUp: function(/*===== e =====*/){
		// summary:
		//		Handler for onkeyup event
		// tags:
		//      callback
	},

	setDisabled: function(/*Boolean*/ disabled){
		// summary:
		//		Deprecated, use set('disabled', ...) instead.
		// tags:
		//		deprecated
		kernel.deprecated('dijit.Editor::setDisabled is deprecated','use dijit.Editor::attr("disabled",boolean) instead', 2.0);
		this.set('disabled',disabled);
	},
	_setValueAttr: function(/*String*/ value){
		// summary:
		//      Registers that attr("value", foo) should call setValue(foo)
		this.setValue(value);
	},
	_setDisableSpellCheckAttr: function(/*Boolean*/ disabled){
		if(this.document){
			domAttr.set(this.document.body, "spellcheck", !disabled);
		}else{
			// try again after the editor is finished loading
			this.onLoadDeferred.addCallback(lang.hitch(this, function(){
				domAttr.set(this.document.body, "spellcheck", !disabled);
			}));
		}
		this._set("disableSpellCheck", disabled);
	},

	onKeyPress: function(e){
		// summary:
		//		Handle the various key events
		// tags:
		//		protected

		var c = (e.keyChar && e.keyChar.toLowerCase()) || e.keyCode,
			handlers = this._keyHandlers[c],
			args = arguments;
			
		if(handlers && !e.altKey){
			array.some(handlers, function(h){
				// treat meta- same as ctrl-, for benefit of mac users
				if(!(h.shift ^ e.shiftKey) && !(h.ctrl ^ (e.ctrlKey||e.metaKey))){ 
					if(!h.handler.apply(this, args)){
						e.preventDefault();
					}
					return true;
				}
			}, this);
		}

		// function call after the character has been inserted
		if(!this._onKeyHitch){
			this._onKeyHitch = lang.hitch(this, "onKeyPressed");
		}
		setTimeout(this._onKeyHitch, 1);
		return true;
	},

	addKeyHandler: function(/*String*/ key, /*Boolean*/ ctrl, /*Boolean*/ shift, /*Function*/ handler){
		// summary:
		//		Add a handler for a keyboard shortcut
		// description:
		//		The key argument should be in lowercase if it is a letter character
		// tags:
		//		protected
		if(!lang.isArray(this._keyHandlers[key])){
			this._keyHandlers[key] = [];
		}
		//TODO: would be nice to make this a hash instead of an array for quick lookups
		this._keyHandlers[key].push({
			shift: shift || false,
			ctrl: ctrl || false,
			handler: handler
		});
	},

	onKeyPressed: function(){
		// summary:
		//		Handler for after the user has pressed a key, and the display has been updated.
		//		(Runs on a timer so that it runs after the display is updated)
		// tags:
		//		private
		this.onDisplayChanged(/*e*/); // can't pass in e
	},

	onClick: function(/*Event*/ e){
		// summary:
		//		Handler for when the user clicks.
		// tags:
		//		private

		// console.info('onClick',this._tryDesignModeOn);
		this.onDisplayChanged(e);
	},

	_onIEMouseDown: function(){
		// summary:
		//		IE only to prevent 2 clicks to focus
		// tags:
		//		protected

		if(!this.focused && !this.disabled){
			this.focus();
		}
	},

	_onBlur: function(e){
		// summary:
		//		Called from focus manager when focus has moved away from this editor
		// tags:
		//		protected

		// console.info('_onBlur')

		this.inherited(arguments);

		var newValue = this.getValue(true);
		if(newValue !== this.value){
			this.onChange(newValue);
		}
		this._set("value", newValue);
	},

	_onFocus: function(/*Event*/ e){
		// summary:
		//		Called from focus manager when focus has moved into this editor
		// tags:
		//		protected

		// console.info('_onFocus')
		if(!this.disabled){
			if(!this._disabledOK){
				this.set('disabled', false);
			}
			this.inherited(arguments);
		}
	},

	// TODO: remove in 2.0
	blur: function(){
		// summary:
		//		Remove focus from this instance.
		// tags:
		//		deprecated
		if(!has("ie") && this.window.document.documentElement && this.window.document.documentElement.focus){
			this.window.document.documentElement.focus();
		}else if(win.doc.body.focus){
			win.doc.body.focus();
		}
	},

	focus: function(){
		// summary:
		//		Move focus to this editor
		if(!this.isLoaded){
			this.focusOnLoad = true;
			return;
		}
		if(this._cursorToStart){
			delete this._cursorToStart;
			if(this.editNode.childNodes){
				this.placeCursorAtStart(); // this calls focus() so return
				return;
			}
		}
		if(!has("ie")){
			focus.focus(this.iframe);
		}else if(this.editNode && this.editNode.focus){
			// editNode may be hidden in display:none div, lets just punt in this case
			//this.editNode.focus(); -> causes IE to scroll always (strict and quirks mode) to the top the Iframe
			// if we fire the event manually and let the browser handle the focusing, the latest
			// cursor position is focused like in FF
			this.iframe.fireEvent('onfocus', document.createEventObject()); // createEventObject only in IE
		//	}else{
		// TODO: should we throw here?
		// console.debug("Have no idea how to focus into the editor!");
		}
	},

	// _lastUpdate: 0,
	updateInterval: 200,
	_updateTimer: null,
	onDisplayChanged: function(/*Event*/ /*===== e =====*/){
		// summary:
		//		This event will be fired every time the display context
		//		changes and the result needs to be reflected in the UI.
		// description:
		//		If you don't want to have update too often,
		//		onNormalizedDisplayChanged should be used instead
		// tags:
		//		private

		// var _t=new Date();
		if(this._updateTimer){
			clearTimeout(this._updateTimer);
		}
		if(!this._updateHandler){
			this._updateHandler = lang.hitch(this,"onNormalizedDisplayChanged");
		}
		this._updateTimer = setTimeout(this._updateHandler, this.updateInterval);

		// Technically this should trigger a call to watch("value", ...) registered handlers,
		// but getValue() is too slow to call on every keystroke so we don't.
	},
	onNormalizedDisplayChanged: function(){
		// summary:
		//		This event is fired every updateInterval ms or more
		// description:
		//		If something needs to happen immediately after a
		//		user change, please use onDisplayChanged instead.
		// tags:
		//		private
		delete this._updateTimer;
	},
	onChange: function(/*===== newContent =====*/){
		// summary:
		//		This is fired if and only if the editor loses focus and
		//		the content is changed.
	},
	_normalizeCommand: function(/*String*/ cmd, /*Anything?*/argument){
		// summary:
		//		Used as the advice function to map our
		//		normalized set of commands to those supported by the target
		//		browser.
		// tags:
		//		private

		var command = cmd.toLowerCase();
		if(command === "formatblock"){
			if(has("safari") && argument === undefined){ command = "heading"; }
		}else if(command === "hilitecolor" && !has("mozilla")){
			command = "backcolor";
		}

		return command;
	},

	_qcaCache: {},
	queryCommandAvailable: function(/*String*/ command){
		// summary:
		//		Tests whether a command is supported by the host. Clients
		//		SHOULD check whether a command is supported before attempting
		//		to use it, behaviour for unsupported commands is undefined.
		// command:
		//		The command to test for
		// tags:
		//		private

		// memoizing version. See _queryCommandAvailable for computing version
		var ca = this._qcaCache[command];
		if(ca !== undefined){ return ca; }
		return (this._qcaCache[command] = this._queryCommandAvailable(command));
	},

	_queryCommandAvailable: function(/*String*/ command){
		// summary:
		//		See queryCommandAvailable().
		// tags:
		//		private

		var ie = 1;
		var mozilla = 1 << 1;
		var webkit = 1 << 2;
		var opera = 1 << 3;

		function isSupportedBy(browsers){
			return {
				ie: Boolean(browsers & ie),
				mozilla: Boolean(browsers & mozilla),
				webkit: Boolean(browsers & webkit),
				opera: Boolean(browsers & opera)
			};
		}

		var supportedBy = null;

		switch(command.toLowerCase()){
			case "bold": case "italic": case "underline":
			case "subscript": case "superscript":
			case "fontname": case "fontsize":
			case "forecolor": case "hilitecolor":
			case "justifycenter": case "justifyfull": case "justifyleft":
			case "justifyright": case "delete": case "selectall": case "toggledir":
				supportedBy = isSupportedBy(mozilla | ie | webkit | opera);
				break;

			case "createlink": case "unlink": case "removeformat":
			case "inserthorizontalrule": case "insertimage":
			case "insertorderedlist": case "insertunorderedlist":
			case "indent": case "outdent": case "formatblock":
			case "inserthtml": case "undo": case "redo": case "strikethrough": case "tabindent":
				supportedBy = isSupportedBy(mozilla | ie | opera | webkit);
				break;

			case "blockdirltr": case "blockdirrtl":
			case "dirltr": case "dirrtl":
			case "inlinedirltr": case "inlinedirrtl":
				supportedBy = isSupportedBy(ie);
				break;
			case "cut": case "copy": case "paste":
				supportedBy = isSupportedBy( ie | mozilla | webkit);
				break;

			case "inserttable":
				supportedBy = isSupportedBy(mozilla | ie);
				break;

			case "insertcell": case "insertcol": case "insertrow":
			case "deletecells": case "deletecols": case "deleterows":
			case "mergecells": case "splitcell":
				supportedBy = isSupportedBy(ie | mozilla);
				break;

			default: return false;
		}

		return (has("ie") && supportedBy.ie) ||
			(has("mozilla") && supportedBy.mozilla) ||
			(has("webkit") && supportedBy.webkit) ||
			(has("opera") && supportedBy.opera);	// Boolean return true if the command is supported, false otherwise
	},

	execCommand: function(/*String*/ command, argument){
		// summary:
		//		Executes a command in the Rich Text area
		// command:
		//		The command to execute
		// argument:
		//		An optional argument to the command
		// tags:
		//		protected
		var returnValue;

		//focus() is required for IE to work
		//In addition, focus() makes sure after the execution of
		//the command, the editor receives the focus as expected
		this.focus();

		command = this._normalizeCommand(command, argument);
		
		if(argument !== undefined){
			if(command === "heading"){
				throw new Error("unimplemented");
			}else if((command === "formatblock") && has("ie")){
				argument = '<'+argument+'>';
			}
		}

		//Check to see if we have any over-rides for commands, they will be functions on this
		//widget of the form _commandImpl.  If we don't, fall through to the basic native
		//exec command of the browser.
		var implFunc = "_" + command + "Impl";
		if(this[implFunc]){
			returnValue = this[implFunc](argument);
		}else{
			argument = arguments.length > 1 ? argument : null;
			if(argument || command !== "createlink"){
				returnValue = this.document.execCommand(command, false, argument);
			}
		}

		this.onDisplayChanged();
		return returnValue;
	},

	queryCommandEnabled: function(/*String*/ command){
		// summary:
		//		Check whether a command is enabled or not.
		// command:
		//		The command to execute
		// tags:
		//		protected
		if(this.disabled || !this._disabledOK){ return false; }

		command = this._normalizeCommand(command);

		//Check to see if we have any over-rides for commands, they will be functions on this
		//widget of the form _commandEnabledImpl.  If we don't, fall through to the basic native
		//command of the browser.
		var implFunc = "_" + command + "EnabledImpl";

		if(this[implFunc]){
			return  this[implFunc](command);
		}else{
			return this._browserQueryCommandEnabled(command);
		}
	},

	queryCommandState: function(command){
		// summary:
		//		Check the state of a given command and returns true or false.
		// tags:
		//		protected

		if(this.disabled || !this._disabledOK){ return false; }
		command = this._normalizeCommand(command);
		try{
			return this.document.queryCommandState(command);
		}catch(e){
			//Squelch, occurs if editor is hidden on FF 3 (and maybe others.)
			return false;
		}
	},

	queryCommandValue: function(command){
		// summary:
		//		Check the value of a given command. This matters most for
		//		custom selections and complex values like font value setting.
		// tags:
		//		protected

		if(this.disabled || !this._disabledOK){ return false; }
		var r;
		command = this._normalizeCommand(command);
		if(has("ie") && command === "formatblock"){
			r = this._native2LocalFormatNames[this.document.queryCommandValue(command)];
		}else if(has("mozilla") && command === "hilitecolor"){
			var oldValue;
			try{
				oldValue = this.document.queryCommandValue("styleWithCSS");
			}catch(e){
				oldValue = false;
			}
			this.document.execCommand("styleWithCSS", false, true);
			r = this.document.queryCommandValue(command);
			this.document.execCommand("styleWithCSS", false, oldValue);
		}else{
			r = this.document.queryCommandValue(command);
		}
		return r;
	},

	// Misc.

	_sCall: function(name, args){
		// summary:
		//		Run the named method of dijit._editor.selection over the
		//		current editor instance's window, with the passed args.
		// tags:
		//		private
		return win.withGlobal(this.window, name, selectionapi, args);
	},

	// FIXME: this is a TON of code duplication. Why?

	placeCursorAtStart: function(){
		// summary:
		//		Place the cursor at the start of the editing area.
		// tags:
		//		private

		this.focus();

		//see comments in placeCursorAtEnd
		var isvalid=false;
		if(has("mozilla")){
			// TODO:  Is this branch even necessary?
			var first=this.editNode.firstChild;
			while(first){
				if(first.nodeType === 3){
					if(first.nodeValue.replace(/^\s+|\s+$/g, "").length>0){
						isvalid=true;
						this._sCall("selectElement", [ first ]);
						break;
					}
				}else if(first.nodeType === 1){
					isvalid=true;
					var tg = first.tagName ? first.tagName.toLowerCase() : "";
					// Collapse before childless tags.
					if(/br|input|img|base|meta|area|basefont|hr|link/.test(tg)){
						this._sCall("selectElement", [ first ]);
					}else{
						// Collapse inside tags with children.
						this._sCall("selectElementChildren", [ first ]);
					}
					break;
				}
				first = first.nextSibling;
			}
		}else{
			isvalid=true;
			this._sCall("selectElementChildren", [ this.editNode ]);
		}
		if(isvalid){
			this._sCall("collapse", [ true ]);
		}
	},

	placeCursorAtEnd: function(){
		// summary:
		//		Place the cursor at the end of the editing area.
		// tags:
		//		private

		this.focus();

		//In mozilla, if last child is not a text node, we have to use
		// selectElementChildren on this.editNode.lastChild otherwise the
		// cursor would be placed at the end of the closing tag of
		//this.editNode.lastChild
		var isvalid=false;
		if(has("mozilla")){
			var last=this.editNode.lastChild;
			while(last){
				if(last.nodeType === 3){
					if(last.nodeValue.replace(/^\s+|\s+$/g, "").length>0){
						isvalid=true;
						this._sCall("selectElement", [ last ]);
						break;
					}
				}else if(last.nodeType === 1){
					isvalid=true;
					if(last.lastChild){
						this._sCall("selectElement", [ last.lastChild ]);
					}else{
						this._sCall("selectElement", [ last ]);
					}
					break;
				}
				last = last.previousSibling;
			}
		}else{
			isvalid=true;
			this._sCall("selectElementChildren", [ this.editNode ]);
		}
		if(isvalid){
			this._sCall("collapse", [ false ]);
		}
	},

	getValue: function(/*Boolean?*/ nonDestructive){
		// summary:
		//		Return the current content of the editing area (post filters
		//		are applied).  Users should call get('value') instead.
		//	nonDestructive:
		//		defaults to false. Should the post-filtering be run over a copy
		//		of the live DOM? Most users should pass "true" here unless they
		//		*really* know that none of the installed filters are going to
		//		mess up the editing session.
		// tags:
		//		private
		if(this.textarea){
			if(this.isClosed || !this.isLoaded){
				return this.textarea.value;
			}
		}

		return this._postFilterContent(null, nonDestructive);
	},
	_getValueAttr: function(){
		// summary:
		//		Hook to make attr("value") work
		return this.getValue(true);
	},

	setValue: function(/*String*/ html){
		// summary:
		//		This function sets the content. No undo history is preserved.
		//		Users should use set('value', ...) instead.
		// tags:
		//		deprecated

		// TODO: remove this and getValue() for 2.0, and move code to _setValueAttr()

		if(!this.isLoaded){
			// try again after the editor is finished loading
			this.onLoadDeferred.addCallback(lang.hitch(this, function(){
				this.setValue(html);
			}));
			return;
		}
		this._cursorToStart = true;
		if(this.textarea && (this.isClosed || !this.isLoaded)){
			this.textarea.value=html;
		}else{
			html = this._preFilterContent(html);
			var node = this.isClosed ? this.domNode : this.editNode;
			if(html && has("mozilla") && html.toLowerCase() === "<p></p>"){
				html = "<p>&#160;</p>";	// &nbsp;
			}

			// Use &nbsp; to avoid webkit problems where editor is disabled until the user clicks it
			if(!html && has("webkit")){
				html = "&#160;";	// &nbsp;
			}
			node.innerHTML = html;
			this._preDomFilterContent(node);
		}

		this.onDisplayChanged();
		this._set("value", this.getValue(true));
	},

	replaceValue: function(/*String*/ html){
		// summary:
		//		This function set the content while trying to maintain the undo stack
		//		(now only works fine with Moz, this is identical to setValue in all
		//		other browsers)
		// tags:
		//		protected

		if(this.isClosed){
			this.setValue(html);
		}else if(this.window && this.window.getSelection && !has("mozilla")){ // Safari
			// look ma! it's a totally f'd browser!
			this.setValue(html);
		}else if(this.window && this.window.getSelection){ // Moz
			html = this._preFilterContent(html);
			this.execCommand("selectall");
			if(!html){
				this._cursorToStart = true;
				html = "&#160;";	// &nbsp;
			}
			this.execCommand("inserthtml", html);
			this._preDomFilterContent(this.editNode);
		}else if(this.document && this.document.selection){//IE
			//In IE, when the first element is not a text node, say
			//an <a> tag, when replacing the content of the editing
			//area, the <a> tag will be around all the content
			//so for now, use setValue for IE too
			this.setValue(html);
		}

		this._set("value", this.getValue(true));
	},

	_preFilterContent: function(/*String*/ html){
		// summary:
		//		Filter the input before setting the content of the editing
		//		area. DOM pre-filtering may happen after this
		//		string-based filtering takes place but as of 1.2, this is not
		//		guaranteed for operations such as the inserthtml command.
		// tags:
		//		private

		var ec = html;
		array.forEach(this.contentPreFilters, function(ef){ if(ef){ ec = ef(ec); } });
		return ec;
	},
	_preDomFilterContent: function(/*DomNode*/ dom){
		// summary:
		//		filter the input's live DOM. All filter operations should be
		//		considered to be "live" and operating on the DOM that the user
		//		will be interacting with in their editing session.
		// tags:
		//		private
		dom = dom || this.editNode;
		array.forEach(this.contentDomPreFilters, function(ef){
			if(ef && lang.isFunction(ef)){
				ef(dom);
			}
		}, this);
	},

	_postFilterContent: function(
		/*DomNode|DomNode[]|String?*/ dom,
		/*Boolean?*/ nonDestructive){
		// summary:
		//		filter the output after getting the content of the editing area
		//
		// description:
		//		post-filtering allows plug-ins and users to specify any number
		//		of transforms over the editor's content, enabling many common
		//		use-cases such as transforming absolute to relative URLs (and
		//		vice-versa), ensuring conformance with a particular DTD, etc.
		//		The filters are registered in the contentDomPostFilters and
		//		contentPostFilters arrays. Each item in the
		//		contentDomPostFilters array is a function which takes a DOM
		//		Node or array of nodes as its only argument and returns the
		//		same. It is then passed down the chain for further filtering.
		//		The contentPostFilters array behaves the same way, except each
		//		member operates on strings. Together, the DOM and string-based
		//		filtering allow the full range of post-processing that should
		//		be necessaray to enable even the most agressive of post-editing
		//		conversions to take place.
		//
		//		If nonDestructive is set to "true", the nodes are cloned before
		//		filtering proceeds to avoid potentially destructive transforms
		//		to the content which may still needed to be edited further.
		//		Once DOM filtering has taken place, the serialized version of
		//		the DOM which is passed is run through each of the
		//		contentPostFilters functions.
		//
		//	dom:
		//		a node, set of nodes, which to filter using each of the current
		//		members of the contentDomPostFilters and contentPostFilters arrays.
		//
		//	nonDestructive:
		//		defaults to "false". If true, ensures that filtering happens on
		//		a clone of the passed-in content and not the actual node
		//		itself.
		//
		// tags:
		//		private

		var ec;
		if(!lang.isString(dom)){
			dom = dom || this.editNode;
			if(this.contentDomPostFilters.length){
				if(nonDestructive){
					dom = lang.clone(dom);
				}
				array.forEach(this.contentDomPostFilters, function(ef){
					dom = ef(dom);
				});
			}
			ec = htmlapi.getChildrenHtml(dom);
		}else{
			ec = dom;
		}

		if(!lang.trim(ec.replace(/^\xA0\xA0*/, '').replace(/\xA0\xA0*$/, '')).length){
			ec = "";
		}

		//	if(has("ie")){
		//		//removing appended <P>&nbsp;</P> for IE
		//		ec = ec.replace(/(?:<p>&nbsp;</p>[\n\r]*)+$/i,"");
		//	}
		array.forEach(this.contentPostFilters, function(ef){
			ec = ef(ec);
		});

		return ec;
	},

	_saveContent: function(){
		// summary:
		//		Saves the content in an onunload event if the editor has not been closed
		// tags:
		//		private

		var saveTextarea = dom.byId(dijit._scopeName + "._editor.RichText.value");
		if(saveTextarea){
			if(saveTextarea.value){
				saveTextarea.value += this._SEPARATOR;
			}
			saveTextarea.value += this.name + this._NAME_CONTENT_SEP + this.getValue(true);
		}
	},


	escapeXml: function(/*String*/ str, /*Boolean*/ noSingleQuotes){
		// summary:
		//		Adds escape sequences for special characters in XML.
		//		Optionally skips escapes for single quotes
		// tags:
		//		private

		str = str.replace(/&/gm, "&amp;").replace(/</gm, "&lt;").replace(/>/gm, "&gt;").replace(/"/gm, "&quot;");
		if(!noSingleQuotes){
			str = str.replace(/'/gm, "&#39;");
		}
		return str; // string
	},

	getNodeHtml: function(/* DomNode */ node){
		// summary:
		//		Deprecated.   Use dijit/_editor/html::_getNodeHtml() instead.
		// tags:
		//		deprecated
		kernel.deprecated('dijit.Editor::getNodeHtml is deprecated','use dijit/_editor/html::getNodeHtml instead', 2);
		return htmlapi.getNodeHtml(node); // String
	},

	getNodeChildrenHtml: function(/* DomNode */ dom){
		// summary:
		//		Deprecated.   Use dijit/_editor/html::getChildrenHtml() instead.
		// tags:
		//		deprecated
		kernel.deprecated('dijit.Editor::getNodeChildrenHtml is deprecated','use dijit/_editor/html::getChildrenHtml instead', 2);
		return htmlapi.getChildrenHtml(dom);
	},

	close: function(/*Boolean?*/ save){
		// summary:
		//		Kills the editor and optionally writes back the modified contents to the
		//		element from which it originated.
		// save:
		//		Whether or not to save the changes. If false, the changes are discarded.
		// tags:
		//		private

		if(this.isClosed){ return; }

		if(!arguments.length){ save = true; }
		if(save){
			this._set("value", this.getValue(true));
		}

		// line height is squashed for iframes
		// FIXME: why was this here? if(this.iframe){ this.domNode.style.lineHeight = null; }

		if(this.interval){ clearInterval(this.interval); }

		if(this._webkitListener){
			//Cleaup of WebKit fix: #9532
			this.disconnect(this._webkitListener);
			delete this._webkitListener;
		}

		// Guard against memory leaks on IE (see #9268)
		if(has("ie")){
			 this.iframe.onfocus = null;
		}
		this.iframe._loadFunc = null;

		if(this._iframeRegHandle){
			this._iframeRegHandle.remove();
			delete this._iframeRegHandle;
		}

		if(this.textarea){
			var s = this.textarea.style;
			s.position = "";
			s.left = s.top = "";
			if(has("ie")){
				s.overflow = this.__overflow;
				this.__overflow = null;
			}
			this.textarea.value = this.value;
			domConstruct.destroy(this.domNode);
			this.domNode = this.textarea;
		}else{
			// Note that this destroys the iframe
			this.domNode.innerHTML = this.value;
		}
		delete this.iframe;

		domClass.remove(this.domNode, this.baseClass);
		this.isClosed = true;
		this.isLoaded = false;

		delete this.editNode;
		delete this.focusNode;

		if(this.window && this.window._frameElement){
			this.window._frameElement = null;
		}

		this.window = null;
		this.document = null;
		this.editingArea = null;
		this.editorObject = null;
	},

	destroy: function(){
		if(!this.isClosed){ this.close(false); }
		if(this._updateTimer){
			clearTimeout(this._updateTimer);
		}
		this.inherited(arguments);
		if(RichText._globalSaveHandler){
			delete RichText._globalSaveHandler[this.id];
		}
	},

	_removeMozBogus: function(/* String */ html){
		// summary:
		//		Post filter to remove unwanted HTML attributes generated by mozilla
		// tags:
		//		private
		return html.replace(/\stype="_moz"/gi, '').replace(/\s_moz_dirty=""/gi, '').replace(/_moz_resizing="(true|false)"/gi,''); // String
	},
	_removeWebkitBogus: function(/* String */ html){
		// summary:
		//		Post filter to remove unwanted HTML attributes generated by webkit
		// tags:
		//		private
		html = html.replace(/\sclass="webkit-block-placeholder"/gi, '');
		html = html.replace(/\sclass="apple-style-span"/gi, '');
		// For some reason copy/paste sometime adds extra meta tags for charset on
		// webkit (chrome) on mac.They need to be removed.  See: #12007"
		html = html.replace(/<meta charset=\"utf-8\" \/>/gi, '');
		return html; // String
	},
	_normalizeFontStyle: function(/* String */ html){
		// summary:
		//		Convert 'strong' and 'em' to 'b' and 'i'.
		// description:
		//		Moz can not handle strong/em tags correctly, so to help
		//		mozilla and also to normalize output, convert them to 'b' and 'i'.
		//
		//		Note the IE generates 'strong' and 'em' rather than 'b' and 'i'
		// tags:
		//		private
		return html.replace(/<(\/)?strong([ \>])/gi, '<$1b$2')
			.replace(/<(\/)?em([ \>])/gi, '<$1i$2' ); // String
	},

	_preFixUrlAttributes: function(/* String */ html){
		// summary:
		//		Pre-filter to do fixing to href attributes on <a> and <img> tags
		// tags:
		//		private
		return html.replace(/(?:(<a(?=\s).*?\shref=)("|')(.*?)\2)|(?:(<a\s.*?href=)([^"'][^ >]+))/gi,
				'$1$4$2$3$5$2 _djrealurl=$2$3$5$2')
			.replace(/(?:(<img(?=\s).*?\ssrc=)("|')(.*?)\2)|(?:(<img\s.*?src=)([^"'][^ >]+))/gi,
				'$1$4$2$3$5$2 _djrealurl=$2$3$5$2'); // String
	},

	/*****************************************************************************
		The following functions implement HTML manipulation commands for various
		browser/contentEditable implementations.  The goal of them is to enforce
		standard behaviors of them.
	******************************************************************************/

	/*** queryCommandEnabled implementations ***/

	_browserQueryCommandEnabled: function(command){
		// summary:
		//		Implementation to call to the native queryCommandEnabled of the browser.
		// command:
		//		The command to check.
		// tags:
		//		protected
		if(!command) { return false; }
		var elem = has("ie") ? this.document.selection.createRange() : this.document;
		try{
			return elem.queryCommandEnabled(command);
		}catch(e){
			return false;
		}
	},

	_createlinkEnabledImpl: function(/*===== argument =====*/){
		// summary:
		//		This function implements the test for if the create link
		//		command should be enabled or not.
		// argument:
		//		arguments to the exec command, if any.
		// tags:
		//		protected
		var enabled = true;
		if(has("opera")){
			var sel = this.window.getSelection();
			if(sel.isCollapsed){
				enabled = true;
			}else{
				enabled = this.document.queryCommandEnabled("createlink");
			}
		}else{
			enabled = this._browserQueryCommandEnabled("createlink");
		}
		return enabled;
	},

	_unlinkEnabledImpl: function(/*===== argument =====*/){
		// summary:
		//		This function implements the test for if the unlink
		//		command should be enabled or not.
		// argument:
		//		arguments to the exec command, if any.
		// tags:
		//		protected
		var enabled = true;
		if(has("mozilla") || has("webkit")){
			enabled = this._sCall("hasAncestorElement", ["a"]);
		}else{
			enabled = this._browserQueryCommandEnabled("unlink");
		}
		return enabled;
	},

	_inserttableEnabledImpl: function(/*===== argument =====*/){
		// summary:
		//		This function implements the test for if the inserttable
		//		command should be enabled or not.
		// argument:
		//		arguments to the exec command, if any.
		// tags:
		//		protected
		var enabled = true;
		if(has("mozilla") || has("webkit")){
			enabled = true;
		}else{
			enabled = this._browserQueryCommandEnabled("inserttable");
		}
		return enabled;
	},

	_cutEnabledImpl: function(/*===== argument =====*/){
		// summary:
		//		This function implements the test for if the cut
		//		command should be enabled or not.
		// argument:
		//		arguments to the exec command, if any.
		// tags:
		//		protected
		var enabled = true;
		if(has("webkit")){
			// WebKit deems clipboard activity as a security threat and natively would return false
			var sel = this.window.getSelection();
			if(sel){ sel = sel.toString(); }
			enabled = !!sel;
		}else{
			enabled = this._browserQueryCommandEnabled("cut");
		}
		return enabled;
	},

	_copyEnabledImpl: function(/*===== argument =====*/){
		// summary:
		//		This function implements the test for if the copy
		//		command should be enabled or not.
		// argument:
		//		arguments to the exec command, if any.
		// tags:
		//		protected
		var enabled = true;
		if(has("webkit")){
			// WebKit deems clipboard activity as a security threat and natively would return false
			var sel = this.window.getSelection();
			if(sel){ sel = sel.toString(); }
			enabled = !!sel;
		}else{
			enabled = this._browserQueryCommandEnabled("copy");
		}
		return enabled;
	},

	_pasteEnabledImpl: function(/*===== argument =====*/){
		// summary:c
		//		This function implements the test for if the paste
		//		command should be enabled or not.
		// argument:
		//		arguments to the exec command, if any.
		// tags:
		//		protected
		var enabled = true;
		if(has("webkit")){
			return true;
		}else{
			enabled = this._browserQueryCommandEnabled("paste");
		}
		return enabled;
	},

	/*** execCommand implementations ***/

	_inserthorizontalruleImpl: function(argument){
		// summary:
		//		This function implements the insertion of HTML 'HR' tags.
		//		into a point on the page.  IE doesn't to it right, so
		//		we have to use an alternate form
		// argument:
		//		arguments to the exec command, if any.
		// tags:
		//		protected
		if(has("ie")){
			return this._inserthtmlImpl("<hr>");
		}
		return this.document.execCommand("inserthorizontalrule", false, argument);
	},

	_unlinkImpl: function(argument){
		// summary:
		//		This function implements the unlink of an 'a' tag.
		// argument:
		//		arguments to the exec command, if any.
		// tags:
		//		protected
		if((this.queryCommandEnabled("unlink")) && (has("mozilla") || has("webkit"))){
			var a = this._sCall("getAncestorElement", [ "a" ]);
			this._sCall("selectElement", [ a ]);
			return this.document.execCommand("unlink", false, null);
		}
		return this.document.execCommand("unlink", false, argument);
	},

	_hilitecolorImpl: function(argument){
		// summary:
		//		This function implements the hilitecolor command
		// argument:
		//		arguments to the exec command, if any.
		// tags:
		//		protected
		var returnValue;
		var isApplied = this._handleTextColorOrProperties("hilitecolor", argument);
		if(!isApplied){
			if(has("mozilla")){
				// mozilla doesn't support hilitecolor properly when useCSS is
				// set to false (bugzilla #279330)
				this.document.execCommand("styleWithCSS", false, true);
				console.log("Executing color command.");
				returnValue = this.document.execCommand("hilitecolor", false, argument);
				this.document.execCommand("styleWithCSS", false, false);
			}else{
				returnValue = this.document.execCommand("hilitecolor", false, argument);
			}
		}
		return returnValue;
	},

	_backcolorImpl: function(argument){
		// summary:
		//		This function implements the backcolor command
		// argument:
		//		arguments to the exec command, if any.
		// tags:
		//		protected
		if(has("ie")){
			// Tested under IE 6 XP2, no problem here, comment out
			// IE weirdly collapses ranges when we exec these commands, so prevent it
			//	var tr = this.document.selection.createRange();
			argument = argument ? argument : null;
		}
		var isApplied = this._handleTextColorOrProperties("backcolor", argument);
		if(!isApplied){
			isApplied = this.document.execCommand("backcolor", false, argument);
		}
		return isApplied;
	},

	_forecolorImpl: function(argument){
		// summary:
		//		This function implements the forecolor command
		// argument:
		//		arguments to the exec command, if any.
		// tags:
		//		protected
		if(has("ie")){
			// Tested under IE 6 XP2, no problem here, comment out
			// IE weirdly collapses ranges when we exec these commands, so prevent it
			//	var tr = this.document.selection.createRange();
			argument = argument? argument : null;
		}
		var isApplied = false;
		isApplied = this._handleTextColorOrProperties("forecolor", argument);
		if(!isApplied){
			isApplied = this.document.execCommand("forecolor", false, argument);
		}
		return isApplied;
	},

	_inserthtmlImpl: function(argument){
		// summary:
		//		This function implements the insertion of HTML content into
		//		a point on the page.
		// argument:
		//		The content to insert, if any.
		// tags:
		//		protected
		argument = this._preFilterContent(argument);
		var rv = true;
		if(has("ie")){
			var insertRange = this.document.selection.createRange();
			if(this.document.selection.type.toUpperCase() === 'CONTROL'){
				var n=insertRange.item(0);
				while(insertRange.length){
					insertRange.remove(insertRange.item(0));
				}
				n.outerHTML=argument;
			}else{
				insertRange.pasteHTML(argument);
			}
			insertRange.select();
			//insertRange.collapse(true);
		}else if(has("mozilla") && !argument.length){
			//mozilla can not inserthtml an empty html to delete current selection
			//so we delete the selection instead in this case
			this._sCall("remove"); // FIXME
		}else{
			rv = this.document.execCommand("inserthtml", false, argument);
		}
		return rv;
	},

	_boldImpl: function(argument){
		// summary:
		//		This function implements an over-ride of the bold command.
		// argument:
		//		Not used, operates by selection.
		// tags:
		//		protected
		var applied = false;
		if(has("ie")){
			this._adaptIESelection();		
			applied = this._adaptIEFormatAreaAndExec("bold");
		}
		if(!applied){
			applied = this.document.execCommand("bold", false, argument);
		}
		return applied;
	},

	_italicImpl: function(argument){
		// summary:
		//		This function implements an over-ride of the italic command.
		// argument:
		//		Not used, operates by selection.
		// tags:
		//		protected
		var applied = false;
		if(has("ie")){
			this._adaptIESelection();			
			applied = this._adaptIEFormatAreaAndExec("italic");
		}
		if(!applied){
			applied = this.document.execCommand("italic", false, argument);
		}
		return applied;
	},

	_underlineImpl: function(argument){
		// summary:
		//		This function implements an over-ride of the underline command.
		// argument:
		//		Not used, operates by selection.
		// tags:
		//		protected
		var applied = false;
		if(has("ie")){
			this._adaptIESelection();			
			applied = this._adaptIEFormatAreaAndExec("underline");
		}
		if(!applied){
			applied = this.document.execCommand("underline", false, argument);
		}
		return applied;
	},

	_strikethroughImpl: function(argument){
		// summary:
		//		This function implements an over-ride of the strikethrough command.
		// argument:
		//		Not used, operates by selection.
		// tags:
		//		protected
		var applied = false;
		if(has("ie")){
			this._adaptIESelection();			
			applied = this._adaptIEFormatAreaAndExec("strikethrough");
		}
		if(!applied){
			applied = this.document.execCommand("strikethrough", false, argument);
		}
		return applied;
	},

	_superscriptImpl: function(argument){
		// summary:
		//		This function implements an over-ride of the superscript command.
		// argument:
		//		Not used, operates by selection.
		// tags:
		//		protected
		var applied = false;
		if(has("ie")){
			this._adaptIESelection();			
			applied = this._adaptIEFormatAreaAndExec("superscript");
		}
		if(!applied){
			applied = this.document.execCommand("superscript", false, argument);
		}
		return applied;
	},

	_subscriptImpl: function(argument){
		// summary:
		//		This function implements an over-ride of the superscript command.
		// argument:
		//		Not used, operates by selection.
		// tags:
		//		protected
		var applied = false;
		if(has("ie")){
			this._adaptIESelection();			
			applied = this._adaptIEFormatAreaAndExec("subscript");
			
		}
		if(!applied){
			applied = this.document.execCommand("subscript", false, argument);
		}
		return applied;
	},
	
	_fontnameImpl: function(argument){
		// summary:
		//		This function implements the fontname command
		// argument:
		//		arguments to the exec command, if any.
		// tags:
		//		protected
		var isApplied;
		if(has("ie")){
			isApplied = this._handleTextColorOrProperties("fontname", argument);
		}
		if(!isApplied){
			isApplied = this.document.execCommand("fontname", false, argument);
		}
		return isApplied;
	},

	_fontsizeImpl: function(argument){
		// summary:
		//		This function implements the fontsize command
		// argument:
		//		arguments to the exec command, if any.
		// tags:
		//		protected
		var isApplied;
		if(has("ie")){
			isApplied = this._handleTextColorOrProperties("fontsize", argument);
		}
		if(!isApplied){
			isApplied = this.document.execCommand("fontsize", false, argument);
		}
		return isApplied;
	},
	
	_insertorderedlistImpl: function(argument){
		// summary:
		//		This function implements the insertorderedlist command
		// argument:
		//		arguments to the exec command, if any.
		// tags:
		//		protected
		var applied = false;
		if(has("ie")){
			applied = this._adaptIEList("insertorderedlist", argument);
		}
		if(!applied){
			applied = this.document.execCommand("insertorderedlist", false, argument);
		}
		return applied;
	},
	
	_insertunorderedlistImpl: function(argument){
		// summary:
		//		This function implements the insertunorderedlist command
		// argument:
		//		arguments to the exec command, if any.
		// tags:
		//		protected
		var applied = false;
		if(has("ie")){
			applied = this._adaptIEList("insertunorderedlist", argument);
		}
		if(!applied){
			applied = this.document.execCommand("insertunorderedlist", false, argument);
		}
		return applied;
	},
	
	getHeaderHeight: function(){
		// summary:
		//		A function for obtaining the height of the header node
		return this._getNodeChildrenHeight(this.header); // Number
	},

	getFooterHeight: function(){
		// summary:
		//		A function for obtaining the height of the footer node
		return this._getNodeChildrenHeight(this.footer); // Number
	},

	_getNodeChildrenHeight: function(node){
		// summary:
		//		An internal function for computing the cumulative height of all child nodes of 'node'
		// node:
		//		The node to process the children of;
		var h = 0;
		if(node && node.childNodes){
			// IE didn't compute it right when position was obtained on the node directly is some cases,
			// so we have to walk over all the children manually.
			var i;
			for(i = 0; i < node.childNodes.length; i++){
				var size = domGeometry.position(node.childNodes[i]);
				h += size.h;
			}
		}
		return h; // Number
	},

	_isNodeEmpty: function(node, startOffset){
		// summary:
		//		Function to test if a node is devoid of real content.
		// node:
		//		The node to check.
		// tags:
		//		private.
		if(node.nodeType === 1/*element*/){
			if(node.childNodes.length > 0){
				return this._isNodeEmpty(node.childNodes[0], startOffset);
	}
			return true;
		}else if(node.nodeType === 3/*text*/){
			return (node.nodeValue.substring(startOffset) === "");
		}
		return false;
	},

	_removeStartingRangeFromRange: function(node, range){
		// summary:
		//		Function to adjust selection range by removing the current
		//		start node.
		// node:
		//		The node to remove from the starting range.
		// range:
		//		The range to adapt.
		// tags:
		//		private
		if(node.nextSibling){
			range.setStart(node.nextSibling,0);
		}else{
			var parent = node.parentNode;
			while(parent && parent.nextSibling == null){
				//move up the tree until we find a parent that has another node, that node will be the next node
				parent = parent.parentNode;
			}
			if(parent){
				range.setStart(parent.nextSibling,0);
			}
		}
		return range;
	},

	_adaptIESelection: function(){
		// summary:
		//		Function to adapt the IE range by removing leading 'newlines'
		//		Needed to fix issue with bold/italics/underline not working if
		//		range included leading 'newlines'.
		//		In IE, if a user starts a selection at the very end of a line,
		//		then the native browser commands will fail to execute correctly.
		//		To work around the issue,  we can remove all empty nodes from
		//		the start of the range selection.
		var selection = rangeapi.getSelection(this.window);
		if(selection && selection.rangeCount && !selection.isCollapsed){
			var range = selection.getRangeAt(0);
			var firstNode = range.startContainer;
			var startOffset = range.startOffset;

			while(firstNode.nodeType === 3/*text*/ && startOffset >= firstNode.length && firstNode.nextSibling){
				//traverse the text nodes until we get to the one that is actually highlighted
				startOffset = startOffset - firstNode.length;
				firstNode = firstNode.nextSibling;
			}

			//Remove the starting ranges until the range does not start with an empty node.
			var lastNode=null;
			while(this._isNodeEmpty(firstNode, startOffset) && firstNode !== lastNode){
				lastNode =firstNode; //this will break the loop in case we can't find the next sibling
				range = this._removeStartingRangeFromRange(firstNode, range); //move the start container to the next node in the range
				firstNode = range.startContainer;
				startOffset = 0; //start at the beginning of the new starting range
			}
			selection.removeAllRanges();// this will work as long as users cannot select multiple ranges. I have not been able to do that in the editor.
			selection.addRange(range);
		}
	},
	
	_adaptIEFormatAreaAndExec: function(command){
		// summary:
		//		Function to handle IE's quirkiness regarding how it handles
		//		format commands on a word.  This involves a lit of node splitting
		//		and format cloning.
		// command:
		//		The format command, needed to check if the desired
		//		command is true or not.
		var selection = rangeapi.getSelection(this.window);
		var doc = this.document;
		var rs, ret, range, txt, startNode, endNode, breaker, sNode;
		if(command && selection && selection.isCollapsed){
			var isApplied = this.queryCommandValue(command);
			if(isApplied){
				
				// We have to split backwards until we hit the format
				var nNames = this._tagNamesForCommand(command);
				range = selection.getRangeAt(0);
				var fs = range.startContainer;
				if(fs.nodeType === 3){
					var offset = range.endOffset;
					if(fs.length < offset){
						//We are not looking from the right node, try to locate the correct one
						ret = this._adjustNodeAndOffset(rs, offset);
						fs = ret.node;
						offset = ret.offset;
					}
				}									
				var topNode;
				while(fs && fs !== this.editNode){
					// We have to walk back and see if this is still a format or not.
					// Hm, how do I do this?
					var tName = fs.tagName? fs.tagName.toLowerCase() : "";
					if(array.indexOf(nNames, tName) > -1){
						topNode = fs;
						break;
					}
					fs = fs.parentNode;
				}

				// Okay, we have a stopping place, time to split things apart.
				if(topNode){
					// Okay, we know how far we have to split backwards, so we have to split now.
					rs = range.startContainer;
					var newblock = doc.createElement(topNode.tagName);
					domConstruct.place(newblock, topNode, "after");
					if(rs && rs.nodeType === 3){
						// Text node, we have to split it.
						var nodeToMove, tNode;
						var endOffset = range.endOffset;
						if(rs.length < endOffset){
							//We are not splitting the right node, try to locate the correct one
							ret = this._adjustNodeAndOffset(rs, endOffset);
							rs = ret.node;
							endOffset = ret.offset;
						}
		
						txt = rs.nodeValue;
						startNode = doc.createTextNode(txt.substring(0, endOffset));
						var endText = txt.substring(endOffset, txt.length);
						if(endText){
							endNode = doc.createTextNode(endText);
						}
						// Place the split, then remove original nodes.
						domConstruct.place(startNode, rs, "before");
						if(endNode){
							breaker = doc.createElement("span");
							breaker.className = "ieFormatBreakerSpan";
							domConstruct.place(breaker, rs, "after");
							domConstruct.place(endNode, breaker, "after");
							endNode = breaker;
						}
						domConstruct.destroy(rs);
						
						// Okay, we split the text.  Now we need to see if we're
						// parented to the block element we're splitting and if
						// not, we have to split all the way up.  Ugh.
						var parentC = startNode.parentNode;
						var tagList = [];
						var tagData;
						while(parentC !== topNode){
							var tg = parentC.tagName;
							tagData = {tagName: tg};
							tagList.push(tagData);
														
							var newTg = doc.createElement(tg);
							// Clone over any 'style' data.
							if(parentC.style){
								if(newTg.style){
									if(parentC.style.cssText){
										newTg.style.cssText = parentC.style.cssText;
										tagData.cssText = parentC.style.cssText;
									}
								}
							}
							// If font also need to clone over any font data.
							if(parentC.tagName === "FONT"){
								if(parentC.color){
									newTg.color = parentC.color;
									tagData.color = parentC.color;
								}
								if(parentC.face){
									newTg.face = parentC.face;
									tagData.face = parentC.face;
								}
								if(parentC.size){  // this check was necessary on IE
									newTg.size = parentC.size;
									tagData.size = parentC.size;
								}
							}
							if(parentC.className){
								newTg.className = parentC.className;
								tagData.className = parentC.className;
							}
							
							// Now move end node and every sibling 
							// after it over into the new tag.
							if(endNode){
								nodeToMove = endNode;
								while(nodeToMove){
									tNode = nodeToMove.nextSibling;
									newTg.appendChild(nodeToMove);
									nodeToMove = tNode;
								}
							}
							if(newTg.tagName == parentC.tagName){
								breaker = doc.createElement("span");
								breaker.className = "ieFormatBreakerSpan";
								domConstruct.place(breaker, parentC, "after");
								domConstruct.place(newTg, breaker, "after");
							}else{
								domConstruct.place(newTg, parentC, "after");
							}
							startNode = parentC;
							endNode = newTg;
							parentC = parentC.parentNode;
						}

						// Lastly, move the split out all the split tags 
						// to the new block as they should now be split properly.
						if(endNode){
							nodeToMove = endNode;
							if(nodeToMove.nodeType === 1 || (nodeToMove.nodeType === 3 && nodeToMove.nodeValue)){
								// Non-blank text and non-text nodes need to clear out that blank space
								// before moving the contents.
								newblock.innerHTML = "";
							}
							while(nodeToMove){
								tNode = nodeToMove.nextSibling;
								newblock.appendChild(nodeToMove);
								nodeToMove = tNode;
							}
						}
						
						// We had intermediate tags, we have to now recreate them inbetween the split
						// and restore what styles, classnames, etc, we can.  
						if(tagList.length){
							tagData = tagList.pop();
							var newContTag = doc.createElement(tagData.tagName);
							if(tagData.cssText && newContTag.style){
								newContTag.style.cssText = tagData.cssText;
							}
							if(tagData.className){
								newContTag.className = tagData.className;
							}
							if(tagData.tagName === "FONT"){
								if(tagData.color){
									newContTag.color = tagData.color;
								}
								if(tagData.face){
									newContTag.face = tagData.face;
								}
								if(tagData.size){ 
									newContTag.size = tagData.size;
								}
							}								
							domConstruct.place(newContTag, newblock, "before");
							while(tagList.length){
								tagData = tagList.pop();
								var newTgNode = doc.createElement(tagData.tagName);
								if(tagData.cssText && newTgNode.style){
									newTgNode.style.cssText = tagData.cssText;
								}
								if(tagData.className){
									newTgNode.className = tagData.className;
								}
								if(tagData.tagName === "FONT"){
									if(tagData.color){
										newTgNode.color = tagData.color;
									}
									if(tagData.face){
										newTgNode.face = tagData.face;
									}
									if(tagData.size){ 
										newTgNode.size = tagData.size;
									}
								}	
								newContTag.appendChild(newTgNode);
								newContTag = newTgNode;
							}							
							
							// Okay, everything is theoretically split apart and removed from the content
							// so insert the dummy text to select, select it, then
							// clear to position cursor.
							sNode = doc.createTextNode(".");
							breaker.appendChild(sNode);
							newContTag.appendChild(sNode);
							win.withGlobal(this.window, lang.hitch(this, function(){
								var newrange = rangeapi.create();
								newrange.setStart(sNode, 0);
								newrange.setEnd(sNode, sNode.length);
								selection.removeAllRanges();
								selection.addRange(newrange);
								selectionapi.collapse(false);
								sNode.parentNode.innerHTML = "";
							}));							
						}else{
							// No extra tags, so we have to insert a breaker point and rely
							// on filters to remove it later.
							breaker = doc.createElement("span");
							breaker.className="ieFormatBreakerSpan";
							sNode = doc.createTextNode(".");
							breaker.appendChild(sNode);
							domConstruct.place(breaker, newblock, "before");
							win.withGlobal(this.window, lang.hitch(this, function(){
								var newrange = rangeapi.create();
								newrange.setStart(sNode, 0);
								newrange.setEnd(sNode, sNode.length);
								selection.removeAllRanges();
								selection.addRange(newrange);
								selectionapi.collapse(false);
								sNode.parentNode.innerHTML = "";
							}));
						}
						if(!newblock.firstChild){
							// Empty, we don't need it.  Split was at end or similar
							// So, remove it.
							domConstruct.destroy(newblock);
						}					
						return true;
					}
				}
				return false;
			}else{
				range = selection.getRangeAt(0);
				rs = range.startContainer;
				if(rs && rs.nodeType === 3){
					// Text node, we have to split it.
					win.withGlobal(this.window, lang.hitch(this, function(){
						var offset = range.startOffset;
						if(rs.length < offset){
							//We are not splitting the right node, try to locate the correct one
							ret = this._adjustNodeAndOffset(rs, offset);
							rs = ret.node;
							offset = ret.offset;
						}
						txt = rs.nodeValue;
						startNode = doc.createTextNode(txt.substring(0, offset));
						var endText = txt.substring(offset);
						if(endText !== ""){
							endNode = doc.createTextNode(txt.substring(offset));
						}
						// Create a space, we'll select and bold it, so 
						// the whole word doesn't get bolded
						breaker = doc.createElement("span");
						sNode = doc.createTextNode(".");
						breaker.appendChild(sNode);
						if(startNode.length){
							domConstruct.place(startNode, rs, "after");
						}else{
							startNode = rs;
						}
						domConstruct.place(breaker, startNode, "after");
						if(endNode){
							domConstruct.place(endNode, breaker, "after");
						}
						domConstruct.destroy(rs);
						var newrange = rangeapi.create();
						newrange.setStart(sNode, 0);
						newrange.setEnd(sNode, sNode.length);
						selection.removeAllRanges();
						selection.addRange(newrange);
						doc.execCommand(command);
						domConstruct.place(breaker.firstChild, breaker, "before");
						domConstruct.destroy(breaker);
						newrange.setStart(sNode, 0);
						newrange.setEnd(sNode, sNode.length);
						selection.removeAllRanges();
						selection.addRange(newrange);
						selectionapi.collapse(false);
						sNode.parentNode.innerHTML = "";
					}));
					return true;
				}
			}
		}else{
			return false;
		}
	},
	
	_adaptIEList: function(command /*===== , argument =====*/){
		// summary:
		//		This function handles normalizing the IE list behavior as 
		//		much as possible.
		// command:
		//		The list command to execute.
		// argument:
		//		Any additional argument.
		// tags:
		//		private
		var selection = rangeapi.getSelection(this.window);
		if(selection.isCollapsed){
			// In the case of no selection, lets commonize the behavior and
			// make sure that it indents if needed.
			if(selection.rangeCount && !this.queryCommandValue(command)){
				var range = selection.getRangeAt(0);
				var sc = range.startContainer;
				if(sc && sc.nodeType == 3){
					// text node.  Lets see if there is a node before it that isn't
					// some sort of breaker.
					if(!range.startOffset){
						// We're at the beginning of a text area.  It may have been br split
						// Who knows?  In any event, we must create the list manually
						// or IE may shove too much into the list element.  It seems to
						// grab content before the text node too if it's br split.
						// Why can't IE work like everyone else?
						win.withGlobal(this.window, lang.hitch(this, function(){
							// Create a space, we'll select and bold it, so 
							// the whole word doesn't get bolded
							var lType = "ul";
							if(command === "insertorderedlist"){
								lType = "ol";
							}
							var list = domConstruct.create(lType);
							var li = domConstruct.create("li", null, list);
							domConstruct.place(list, sc, "before");
							// Move in the text node as part of the li.
							li.appendChild(sc);
							// We need a br after it or the enter key handler
							// sometimes throws errors.
							domConstruct.create("br", null, list, "after");
							// Okay, now lets move our cursor to the beginning.
							var newrange = rangeapi.create();
							newrange.setStart(sc, 0);
							newrange.setEnd(sc, sc.length);
							selection.removeAllRanges();
							selection.addRange(newrange);
							selectionapi.collapse(true);
						}));
						return true;
					}
				}
			}
		}
		return false;
	},
	
	_handleTextColorOrProperties: function(command, argument){
		// summary:
		//		This function handles appplying text color as best it is 
		//		able to do so when the selection is collapsed, making the
		//		behavior cross-browser consistent. It also handles the name
		//		and size for IE.
		// command:
		//		The command.
		// argument:
		//		Any additional arguments.
		// tags:
		//		private
		var selection = rangeapi.getSelection(this.window);
		var doc = this.document;
		var rs, ret, range, txt, startNode, endNode, breaker, sNode;
		argument = argument || null;
		if(command && selection && selection.isCollapsed){
			if(selection.rangeCount){
				range = selection.getRangeAt(0);
				rs = range.startContainer;
				if(rs && rs.nodeType === 3){
					// Text node, we have to split it.
					win.withGlobal(this.window, lang.hitch(this, function(){
						var offset = range.startOffset;
						if(rs.length < offset){
							//We are not splitting the right node, try to locate the correct one
							ret = this._adjustNodeAndOffset(rs, offset);
							rs = ret.node;
							offset = ret.offset;
						}
						txt = rs.nodeValue;
						startNode = doc.createTextNode(txt.substring(0, offset));
						var endText = txt.substring(offset);
						if(endText !== ""){
							endNode = doc.createTextNode(txt.substring(offset));
						}
						// Create a space, we'll select and bold it, so 
						// the whole word doesn't get bolded
						breaker = domConstruct.create("span");
						sNode = doc.createTextNode(".");
						breaker.appendChild(sNode);
						// Create a junk node to avoid it trying to stlye the breaker.
						// This will get destroyed later.
						var extraSpan = domConstruct.create("span");
						breaker.appendChild(extraSpan);
						if(startNode.length){
							domConstruct.place(startNode, rs, "after");
						}else{
							startNode = rs;
						}
						domConstruct.place(breaker, startNode, "after");
						if(endNode){
							domConstruct.place(endNode, breaker, "after");
						}
						domConstruct.destroy(rs);
						var newrange = rangeapi.create();
						newrange.setStart(sNode, 0);
						newrange.setEnd(sNode, sNode.length);
						selection.removeAllRanges();
						selection.addRange(newrange);
						if(has("webkit")){
							// WebKit is frustrating with positioning the cursor. 
							// It stinks to have a selected space, but there really
							// isn't much choice here.
							var style = "color";
							if(command === "hilitecolor" || command === "backcolor"){
								style = "backgroundColor";
							}
							domStyle.set(breaker, style, argument);
							selectionapi.remove();
							domConstruct.destroy(extraSpan);
							breaker.innerHTML = "&#160;";	// &nbsp;
							selectionapi.selectElement(breaker);
							this.focus();
						}else{
							this.execCommand(command, argument);
							domConstruct.place(breaker.firstChild, breaker, "before");
							domConstruct.destroy(breaker);
							newrange.setStart(sNode, 0);
							newrange.setEnd(sNode, sNode.length);
							selection.removeAllRanges();
							selection.addRange(newrange);
							selectionapi.collapse(false);
							sNode.parentNode.removeChild(sNode);
						}
					}));
					return true;
				}
			}				
		}
		return false;
	},
	
	_adjustNodeAndOffset: function(/*DomNode*/node, /*Int*/offset){
		// summary:
		//		In the case there are multiple text nodes in a row the offset may not be within the node.  
		//		If the offset is larger than the node length, it will attempt to find
		//		the next text sibling until it locates the text node in which the offset refers to
		// node:
		//		The node to check.
		// offset:
		//		The position to find within the text node
		// tags:
		//		private.
		while(node.length < offset && node.nextSibling && node.nextSibling.nodeType === 3){
			//Adjust the offset and node in the case of multiple text nodes in a row
			offset = offset - node.length;
			node = node.nextSibling;
		}
		return {"node": node, "offset": offset};
	},
	
	_tagNamesForCommand: function(command){
		// summary:
		//		Function to return the tab names that are associated
		//		with a particular style.
		// command: String
		//		The command to return tags for.
		// tags:
		//		private
		if(command === "bold"){
			return ["b", "strong"];
		}else if(command === "italic"){
			return ["i","em"];
		}else if(command === "strikethrough"){
			return ["s", "strike"];
		}else if(command === "superscript"){
			return ["sup"];
		}else if(command === "subscript"){
			return ["sub"];
		}else if(command === "underline"){
			return ["u"];
		}	
		return [];
	},

	_stripBreakerNodes: function(node){
		// summary:
		//		Function for stripping out the breaker spans inserted by the formatting command.
		//		Registered as a filter for IE, handles the breaker spans needed to fix up
		//		How bold/italic/etc, work when selection is collapsed (single cursor).
		win.withGlobal(this.window, lang.hitch(this, function(){
			var breakers = query(".ieFormatBreakerSpan", node);
			var i;
			for(i = 0; i < breakers.length; i++){
				var b = breakers[i];
				while(b.firstChild){
					domConstruct.place(b.firstChild, b, "before");
				}
				domConstruct.destroy(b);
			}		
		}));
		return node;
	}
});

return RichText;

});

},
'url:gridx/templates/FilterConfirmDialog.html':"<div class=\"gridxFilterConfirmDialogMessage\">${clearFilterMsg}</div\n><div class=\"gridxFilterConfirmDialogButtons\"\n\t><input type=\"button\" data-dojo-type=\"dijit.form.Button\" label=\"${clearButton}\"\n\t/><input type=\"button\" data-dojo-type=\"dijit.form.Button\" label=\"${cancelButton}\"\n/></div>\n",
'url:dijit/form/templates/ValidationTextBox.html':"<div class=\"dijit dijitReset dijitInline dijitLeft\"\n\tid=\"widget_${id}\" role=\"presentation\"\n\t><div class='dijitReset dijitValidationContainer'\n\t\t><input class=\"dijitReset dijitInputField dijitValidationIcon dijitValidationInner\" value=\"&#935; \" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"presentation\"\n\t/></div\n\t><div class=\"dijitReset dijitInputField dijitInputContainer\"\n\t\t><input class=\"dijitReset dijitInputInner\" data-dojo-attach-point='textbox,focusNode' autocomplete=\"off\"\n\t\t\t${!nameAttrSetting} type='${type}'\n\t/></div\n></div>\n",
'dojox/html/metrics':function(){
define(["dojo/_base/kernel","dojo/_base/lang", "dojo/_base/sniff", "dojo/ready", "dojo/_base/unload",
		"dojo/_base/window", "dojo/dom-geometry"],
  function(kernel,lang,has,ready,UnloadUtil,Window,DOMGeom){
	var dhm = lang.getObject("dojox.html.metrics",true);
	var dojox = lang.getObject("dojox");

	//	derived from Morris John's emResized measurer
	dhm.getFontMeasurements = function(){
		//	summary
		//	Returns an object that has pixel equivilents of standard font size values.
		var heights = {
			'1em':0, '1ex':0, '100%':0, '12pt':0, '16px':0, 'xx-small':0, 'x-small':0,
			'small':0, 'medium':0, 'large':0, 'x-large':0, 'xx-large':0
		};
	
		if(has("ie")){
			//	we do a font-size fix if and only if one isn't applied already.
			//	NOTE: If someone set the fontSize on the HTML Element, this will kill it.
			Window.doc.documentElement.style.fontSize="100%";
		}
	
		//	set up the measuring node.
		var div=Window.doc.createElement("div");
		var ds = div.style;
		ds.position="absolute";
		ds.left="-100px";
		ds.top="0";
		ds.width="30px";
		ds.height="1000em";
		ds.borderWidth="0";
		ds.margin="0";
		ds.padding="0";
		ds.outline="0";
		ds.lineHeight="1";
		ds.overflow="hidden";
		Window.body().appendChild(div);
	
		//	do the measurements.
		for(var p in heights){
			ds.fontSize = p;
			heights[p] = Math.round(div.offsetHeight * 12/16) * 16/12 / 1000;
		}
		
		Window.body().removeChild(div);
		div = null;
		return heights; 	//	object
	};

	var fontMeasurements = null;
	
	dhm.getCachedFontMeasurements = function(recalculate){
		if(recalculate || !fontMeasurements){
			fontMeasurements = dhm.getFontMeasurements();
		}
		return fontMeasurements;
	};

	var measuringNode = null, empty = {};
	dhm.getTextBox = function(/* String */ text, /* Object */ style, /* String? */ className){
		var m, s;
		if(!measuringNode){
			m = measuringNode = Window.doc.createElement("div");
			// Container that we can set contraints on so that it doesn't
			// trigger a scrollbar.
			var c = Window.doc.createElement("div");
			c.appendChild(m);
			s = c.style;
			s.overflow='scroll';
			s.position = "absolute";
			s.left = "0px";
			s.top = "-10000px";
			s.width = "1px";
			s.height = "1px";
			s.visibility = "hidden";
			s.borderWidth = "0";
			s.margin = "0";
			s.padding = "0";
			s.outline = "0";
			Window.body().appendChild(c);
		}else{
			m = measuringNode;
		}
		// reset styles
		m.className = "";
		s = m.style;
		s.borderWidth = "0";
		s.margin = "0";
		s.padding = "0";
		s.outline = "0";
		// set new style
		if(arguments.length > 1 && style){
			for(var i in style){
				if(i in empty){ continue; }
				s[i] = style[i];
			}
		}
		// set classes
		if(arguments.length > 2 && className){
			m.className = className;
		}
		// take a measure
		m.innerHTML = text;
		var box = DOMGeom.position(m);
		// position doesn't report right (reports 1, since parent is 1)
		// So we have to look at the scrollWidth to get the real width
		// Height is right.
		box.w = m.parentNode.scrollWidth;
		return box;
	};

	//	determine the scrollbar sizes on load.
	var scroll={ w:16, h:16 };
	dhm.getScrollbar=function(){ return { w:scroll.w, h:scroll.h }; };

	dhm._fontResizeNode = null;

	dhm.initOnFontResize = function(interval){
		var f = dhm._fontResizeNode = Window.doc.createElement("iframe");
		var fs = f.style;
		fs.position = "absolute";
		fs.width = "5em";
		fs.height = "10em";
		fs.top = "-10000px";
		if(has("ie")){
			f.onreadystatechange = function(){
				if(f.contentWindow.document.readyState == "complete"){
					f.onresize = f.contentWindow.parent[dojox._scopeName].html.metrics._fontresize;
				}
			};
		}else{
			f.onload = function(){
				f.contentWindow.onresize = f.contentWindow.parent[dojox._scopeName].html.metrics._fontresize;
			};
		}
		//The script tag is to work around a known firebug race condition.  See comments in bug #9046
		f.setAttribute("src", "javascript:'<html><head><script>if(\"loadFirebugConsole\" in window){window.loadFirebugConsole();}</script></head><body></body></html>'");
		Window.body().appendChild(f);
		dhm.initOnFontResize = function(){};
	};

	dhm.onFontResize = function(){};
	dhm._fontresize = function(){
		dhm.onFontResize();
	}

	UnloadUtil.addOnUnload(function(){
		// destroy our font resize iframe if we have one
		var f = dhm._fontResizeNode;
		if(f){
			if(has("ie") && f.onresize){
				f.onresize = null;
			}else if(f.contentWindow && f.contentWindow.onresize){
				f.contentWindow.onresize = null;
			}
			dhm._fontResizeNode = null;
		}
	});

	ready(function(){
		// getScrollbar metrics node
		try{
			var n=Window.doc.createElement("div");
			n.style.cssText = "top:0;left:0;width:100px;height:100px;overflow:scroll;position:absolute;visibility:hidden;";
			Window.body().appendChild(n);
			scroll.w = n.offsetWidth - n.clientWidth;
			scroll.h = n.offsetHeight - n.clientHeight;
			Window.body().removeChild(n);
			//console.log("Scroll bar dimensions: ", scroll);
			delete n;
		}catch(e){}

		// text size poll setup
		if("fontSizeWatch" in kernel.config && !!kernel.config.fontSizeWatch){
			dhm.initOnFontResize();
		}
	});
	return dhm;
});
},
'dojo/fx':function(){
define([
	"./_base/lang",
	"./Evented",
	"./_base/kernel",
	"./_base/array",
	"./_base/connect",
	"./_base/fx",
	"./dom",
	"./dom-style",
	"./dom-geometry",
	"./ready",
	"require" // for context sensitive loading of Toggler
], function(lang, Evented, dojo, arrayUtil, connect, baseFx, dom, domStyle, geom, ready, require) {

	// module:
	//		dojo/fx
	// summary:
	//		TODOC


	/*=====
	dojo.fx = {
		// summary: Effects library on top of Base animations
	};
	var coreFx = dojo.fx;
	=====*/
	
// For back-compat, remove in 2.0.
if(!dojo.isAsync){
	ready(0, function(){
		var requires = ["./fx/Toggler"];
		require(requires);	// use indirection so modules not rolled into a build
	});
}

	var coreFx = dojo.fx = {};

	var _baseObj = {
			_fire: function(evt, args){
				if(this[evt]){
					this[evt].apply(this, args||[]);
				}
				return this;
			}
		};

	var _chain = function(animations){
		this._index = -1;
		this._animations = animations||[];
		this._current = this._onAnimateCtx = this._onEndCtx = null;

		this.duration = 0;
		arrayUtil.forEach(this._animations, function(a){
			this.duration += a.duration;
			if(a.delay){ this.duration += a.delay; }
		}, this);
	};
	_chain.prototype = new Evented();
	lang.extend(_chain, {
		_onAnimate: function(){
			this._fire("onAnimate", arguments);
		},
		_onEnd: function(){
			connect.disconnect(this._onAnimateCtx);
			connect.disconnect(this._onEndCtx);
			this._onAnimateCtx = this._onEndCtx = null;
			if(this._index + 1 == this._animations.length){
				this._fire("onEnd");
			}else{
				// switch animations
				this._current = this._animations[++this._index];
				this._onAnimateCtx = connect.connect(this._current, "onAnimate", this, "_onAnimate");
				this._onEndCtx = connect.connect(this._current, "onEnd", this, "_onEnd");
				this._current.play(0, true);
			}
		},
		play: function(/*int?*/ delay, /*Boolean?*/ gotoStart){
			if(!this._current){ this._current = this._animations[this._index = 0]; }
			if(!gotoStart && this._current.status() == "playing"){ return this; }
			var beforeBegin = connect.connect(this._current, "beforeBegin", this, function(){
					this._fire("beforeBegin");
				}),
				onBegin = connect.connect(this._current, "onBegin", this, function(arg){
					this._fire("onBegin", arguments);
				}),
				onPlay = connect.connect(this._current, "onPlay", this, function(arg){
					this._fire("onPlay", arguments);
					connect.disconnect(beforeBegin);
					connect.disconnect(onBegin);
					connect.disconnect(onPlay);
				});
			if(this._onAnimateCtx){
				connect.disconnect(this._onAnimateCtx);
			}
			this._onAnimateCtx = connect.connect(this._current, "onAnimate", this, "_onAnimate");
			if(this._onEndCtx){
				connect.disconnect(this._onEndCtx);
			}
			this._onEndCtx = connect.connect(this._current, "onEnd", this, "_onEnd");
			this._current.play.apply(this._current, arguments);
			return this;
		},
		pause: function(){
			if(this._current){
				var e = connect.connect(this._current, "onPause", this, function(arg){
						this._fire("onPause", arguments);
						connect.disconnect(e);
					});
				this._current.pause();
			}
			return this;
		},
		gotoPercent: function(/*Decimal*/percent, /*Boolean?*/ andPlay){
			this.pause();
			var offset = this.duration * percent;
			this._current = null;
			arrayUtil.some(this._animations, function(a){
				if(a.duration <= offset){
					this._current = a;
					return true;
				}
				offset -= a.duration;
				return false;
			});
			if(this._current){
				this._current.gotoPercent(offset / this._current.duration, andPlay);
			}
			return this;
		},
		stop: function(/*boolean?*/ gotoEnd){
			if(this._current){
				if(gotoEnd){
					for(; this._index + 1 < this._animations.length; ++this._index){
						this._animations[this._index].stop(true);
					}
					this._current = this._animations[this._index];
				}
				var e = connect.connect(this._current, "onStop", this, function(arg){
						this._fire("onStop", arguments);
						connect.disconnect(e);
					});
				this._current.stop();
			}
			return this;
		},
		status: function(){
			return this._current ? this._current.status() : "stopped";
		},
		destroy: function(){
			if(this._onAnimateCtx){ connect.disconnect(this._onAnimateCtx); }
			if(this._onEndCtx){ connect.disconnect(this._onEndCtx); }
		}
	});
	lang.extend(_chain, _baseObj);

	coreFx.chain = /*===== dojo.fx.chain = =====*/ function(/*dojo.Animation[]*/ animations){
		// summary:
		//		Chain a list of `dojo.Animation`s to run in sequence
		//
		// description:
		//		Return a `dojo.Animation` which will play all passed
		//		`dojo.Animation` instances in sequence, firing its own
		//		synthesized events simulating a single animation. (eg:
		//		onEnd of this animation means the end of the chain,
		//		not the individual animations within)
		//
		// example:
		//	Once `node` is faded out, fade in `otherNode`
		//	|	dojo.fx.chain([
		//	|		dojo.fadeIn({ node:node }),
		//	|		dojo.fadeOut({ node:otherNode })
		//	|	]).play();
		//
		return new _chain(animations); // dojo.Animation
	};

	var _combine = function(animations){
		this._animations = animations||[];
		this._connects = [];
		this._finished = 0;

		this.duration = 0;
		arrayUtil.forEach(animations, function(a){
			var duration = a.duration;
			if(a.delay){ duration += a.delay; }
			if(this.duration < duration){ this.duration = duration; }
			this._connects.push(connect.connect(a, "onEnd", this, "_onEnd"));
		}, this);

		this._pseudoAnimation = new baseFx.Animation({curve: [0, 1], duration: this.duration});
		var self = this;
		arrayUtil.forEach(["beforeBegin", "onBegin", "onPlay", "onAnimate", "onPause", "onStop", "onEnd"],
			function(evt){
				self._connects.push(connect.connect(self._pseudoAnimation, evt,
					function(){ self._fire(evt, arguments); }
				));
			}
		);
	};
	lang.extend(_combine, {
		_doAction: function(action, args){
			arrayUtil.forEach(this._animations, function(a){
				a[action].apply(a, args);
			});
			return this;
		},
		_onEnd: function(){
			if(++this._finished > this._animations.length){
				this._fire("onEnd");
			}
		},
		_call: function(action, args){
			var t = this._pseudoAnimation;
			t[action].apply(t, args);
		},
		play: function(/*int?*/ delay, /*Boolean?*/ gotoStart){
			this._finished = 0;
			this._doAction("play", arguments);
			this._call("play", arguments);
			return this;
		},
		pause: function(){
			this._doAction("pause", arguments);
			this._call("pause", arguments);
			return this;
		},
		gotoPercent: function(/*Decimal*/percent, /*Boolean?*/ andPlay){
			var ms = this.duration * percent;
			arrayUtil.forEach(this._animations, function(a){
				a.gotoPercent(a.duration < ms ? 1 : (ms / a.duration), andPlay);
			});
			this._call("gotoPercent", arguments);
			return this;
		},
		stop: function(/*boolean?*/ gotoEnd){
			this._doAction("stop", arguments);
			this._call("stop", arguments);
			return this;
		},
		status: function(){
			return this._pseudoAnimation.status();
		},
		destroy: function(){
			arrayUtil.forEach(this._connects, connect.disconnect);
		}
	});
	lang.extend(_combine, _baseObj);

	coreFx.combine = /*===== dojo.fx.combine = =====*/ function(/*dojo.Animation[]*/ animations){
		// summary:
		//		Combine a list of `dojo.Animation`s to run in parallel
		//
		// description:
		//		Combine an array of `dojo.Animation`s to run in parallel,
		//		providing a new `dojo.Animation` instance encompasing each
		//		animation, firing standard animation events.
		//
		// example:
		//	Fade out `node` while fading in `otherNode` simultaneously
		//	|	dojo.fx.combine([
		//	|		dojo.fadeIn({ node:node }),
		//	|		dojo.fadeOut({ node:otherNode })
		//	|	]).play();
		//
		// example:
		//	When the longest animation ends, execute a function:
		//	|	var anim = dojo.fx.combine([
		//	|		dojo.fadeIn({ node: n, duration:700 }),
		//	|		dojo.fadeOut({ node: otherNode, duration: 300 })
		//	|	]);
		//	|	dojo.connect(anim, "onEnd", function(){
		//	|		// overall animation is done.
		//	|	});
		//	|	anim.play(); // play the animation
		//
		return new _combine(animations); // dojo.Animation
	};

	coreFx.wipeIn = /*===== dojo.fx.wipeIn = =====*/ function(/*Object*/ args){
		// summary:
		//		Expand a node to it's natural height.
		//
		// description:
		//		Returns an animation that will expand the
		//		node defined in 'args' object from it's current height to
		//		it's natural height (with no scrollbar).
		//		Node must have no margin/border/padding.
		//
		// args: Object
		//		A hash-map of standard `dojo.Animation` constructor properties
		//		(such as easing: node: duration: and so on)
		//
		// example:
		//	|	dojo.fx.wipeIn({
		//	|		node:"someId"
		//	|	}).play()
		var node = args.node = dom.byId(args.node), s = node.style, o;

		var anim = baseFx.animateProperty(lang.mixin({
			properties: {
				height: {
					// wrapped in functions so we wait till the last second to query (in case value has changed)
					start: function(){
						// start at current [computed] height, but use 1px rather than 0
						// because 0 causes IE to display the whole panel
						o = s.overflow;
						s.overflow = "hidden";
						if(s.visibility == "hidden" || s.display == "none"){
							s.height = "1px";
							s.display = "";
							s.visibility = "";
							return 1;
						}else{
							var height = domStyle.get(node, "height");
							return Math.max(height, 1);
						}
					},
					end: function(){
						return node.scrollHeight;
					}
				}
			}
		}, args));

		var fini = function(){
			s.height = "auto";
			s.overflow = o;
		};
		connect.connect(anim, "onStop", fini);
		connect.connect(anim, "onEnd", fini);

		return anim; // dojo.Animation
	};

	coreFx.wipeOut = /*===== dojo.fx.wipeOut = =====*/ function(/*Object*/ args){
		// summary:
		//		Shrink a node to nothing and hide it.
		//
		// description:
		//		Returns an animation that will shrink node defined in "args"
		//		from it's current height to 1px, and then hide it.
		//
		// args: Object
		//		A hash-map of standard `dojo.Animation` constructor properties
		//		(such as easing: node: duration: and so on)
		//
		// example:
		//	|	dojo.fx.wipeOut({ node:"someId" }).play()

		var node = args.node = dom.byId(args.node), s = node.style, o;

		var anim = baseFx.animateProperty(lang.mixin({
			properties: {
				height: {
					end: 1 // 0 causes IE to display the whole panel
				}
			}
		}, args));

		connect.connect(anim, "beforeBegin", function(){
			o = s.overflow;
			s.overflow = "hidden";
			s.display = "";
		});
		var fini = function(){
			s.overflow = o;
			s.height = "auto";
			s.display = "none";
		};
		connect.connect(anim, "onStop", fini);
		connect.connect(anim, "onEnd", fini);

		return anim; // dojo.Animation
	};

	coreFx.slideTo = /*===== dojo.fx.slideTo = =====*/ function(/*Object*/ args){
		// summary:
		//		Slide a node to a new top/left position
		//
		// description:
		//		Returns an animation that will slide "node"
		//		defined in args Object from its current position to
		//		the position defined by (args.left, args.top).
		//
		// args: Object
		//		A hash-map of standard `dojo.Animation` constructor properties
		//		(such as easing: node: duration: and so on). Special args members
		//		are `top` and `left`, which indicate the new position to slide to.
		//
		// example:
		//	|	.slideTo({ node: node, left:"40", top:"50", units:"px" }).play()

		var node = args.node = dom.byId(args.node),
			top = null, left = null;

		var init = (function(n){
			return function(){
				var cs = domStyle.getComputedStyle(n);
				var pos = cs.position;
				top = (pos == 'absolute' ? n.offsetTop : parseInt(cs.top) || 0);
				left = (pos == 'absolute' ? n.offsetLeft : parseInt(cs.left) || 0);
				if(pos != 'absolute' && pos != 'relative'){
					var ret = geom.position(n, true);
					top = ret.y;
					left = ret.x;
					n.style.position="absolute";
					n.style.top=top+"px";
					n.style.left=left+"px";
				}
			};
		})(node);
		init();

		var anim = baseFx.animateProperty(lang.mixin({
			properties: {
				top: args.top || 0,
				left: args.left || 0
			}
		}, args));
		connect.connect(anim, "beforeBegin", anim, init);

		return anim; // dojo.Animation
	};

	return coreFx;
});

},
'ibm/usmi/util/Utils':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define([ "dojo/dom",
	"dojo/dom-construct",
	"dojo/dom-class",
	"dojo/dom-attr",
	"dojo/dom-geometry" ], function (
	djDom, djDomC, djClass, djDomAttr, djDomG) {
	return {

		FSMFlightRecorder : {
			curSize : 0,
			startIndex : 0,
			entries : [],
			maxSize : 75,
			add : function (
				traceEntry) {
				var now, timeString;
				now = new Date();
				timeString = now.getFullYear() + "/" + now.getMonth() + 1 + "/" + now.getDate() + " " + now.getHours() +
					":" + now.getMinutes() + ":" + now.getSeconds() + "." + now.getMilliseconds();
				//var timeString = now.format("yyyy/mm/dd HH:MM:ss.l");
				if (this.curSize < this.maxSize) {
					this.entries[this.curSize] = timeString + " - " + traceEntry;
					this.curSize++;
				} else {
					this.entries[this.startIndex] = timeString + " - " + traceEntry;
					this.startIndex += 1;
					if (this.startIndex >= this.maxSize) {
						this.startIndex = 0;
					}
				}
			},
			test : function () {
				var i;
				try {
					console.info("DEBUG -- BEGIN -- TEST FLIGHT RECORDER DATA");
					for (i = 0; i < this.curSize; i++) {
						if (this.startIndex + i >= this.curSize) {
							console.info(this.entries[this.startIndex + i - this.curSize]);
						} else {
							console.info(this.entries[this.startIndex + i]);
						}
					}
					console.info("DEBUG -- END -- TEST FLIGHT RECORDER DATA");
				} catch (e) {}
			},

			toString : function (
				newLine) {
				var i, outputString = "";
				if (!newLine) {
					newLine = "<br>";
				}
				try {
					outputString = outputString +
						" -- BEGIN -- FSM SCRIPT FLIGHT RECORDER DATA (time values are from the browsers system)" +
						newLine;
					for (i = 0; i < this.curSize; i++) {
						if ((this.startIndex) + i >= this.curSize) {
							outputString = outputString + this.entries[this.startIndex + i - this.curSize] + newLine;
						} else {
							outputString = outputString + this.entries[this.startIndex + i] + newLine;
						}
					}
					outputString = outputString + " -- END -- FSM SCRIPT FLIGHT RECORDER DATA";
				} catch (e) {}
				return outputString;
			}

		},

		// --- TRACING --- 

		TraceLevel : 20, // 10 = DEBUG , 15 = FINEST, 20 = TRACE , 25 = INFO ,  30 = WARN , 40 = ERROR 

		// REMOVE ANY USE WHEN DONE
		CMDEBUG : function (
			outputText) {
			if (this.TraceLevel > 10) {
				return;
			}
			var out = "DEBUG (" + arguments.callee.caller.nom + ") : " + outputText;
			console.log(out);
			this.FSMFlightRecorder.add(out);
		},

		CMFinest : function (
			outputText) {
			if (this.TraceLevel > 15) {
				return;
			}
			var out = "FINEST (" + arguments.callee.caller.nom + ") : " + outputText;
			console.log(out);
			this.FSMFlightRecorder.add(out);
		},

		// FINE level tracing
		CMTrace : function (
			outputText) {
			if (this.TraceLevel > 20) {
				return;
			}
			var out = "TRACE (" + arguments.callee.caller.nom + ") : " + outputText;
			console.log(out);
			this.FSMFlightRecorder.add(out);
		},
		// Info level tracing
		CMInfo : function (
			outputText) {
			if (this.TraceLevel > 25) {
				return;
			}
			var out = "INFO (" + arguments.callee.caller.nom + ") : " + outputText;
			console.info(out);
			this.FSMFlightRecorder.add(out);
		},
		CMWarn : function (
			outputText) {
			if (this.TraceLevel > 30) {
				return;
			}
			var out = "WARN (" + arguments.callee.caller.nom + ") : " + outputText;
			console.warn(out);
			this.FSMFlightRecorder.add(out);
		},
		CMError : function (
			outputText) {
			if (this.TraceLevel > 40) {
				return;
			}
			var out = "ERROR (" + arguments.callee.caller.nom + ") : " + outputText;
			console.error(out);
			this.FSMFlightRecorder.add(out);
		},

		_traceTimers : [],
		CMTraceTimeInit : function (
			timerName) {
			if (this.TraceLevel > 20) {
				return;
			}
			this._traceTimers[timerName] = new Date();
		},

		CMTraceTimeDelta : function (
			timerName, message, restartTimer) {
			if (this.TraceLevel > 20) {
				return;
			}

			if (this._traceTimers[timerName]) {
				var out = "TRACE TIME (" + arguments.callee.caller.nom + ") [ " +
					((new Date()) - this._traceTimers[timerName]) + "ms ] : " + message;
				console.log(out);
				this.FSMFlightRecorder.add(out);
			}
			if (restartTimer) {
				this._traceTimers[timerName] = new Date();
			}
		},

		// make a page element visible
		showPageElement : function (
			elName) {
			var tbl = document.getElementById(elName);
			tbl.style.display = 'block'; // consider if using for text elements "inline"
		},

		// hide a page element
		hidePageElement : function (
			elName) {
			var tbl = document.getElementById(elName);
			tbl.style.display = 'none';
		},

		getProperty : function (
			item, propertyName) {
			var propValue = null, prop;
			try {
				prop = item.getProperty(propertyName);
				if (prop) {
					propValue = prop.getValue();
				}
			} catch (err) {
				this.CMError("an error occurred getting properties for " + item.getLabel() + " : " + err);
			}
			return propValue;
		},

		isChrome : function () {
			return (navigator.userAgent.toLowerCase().indexOf('chrome') > -1);
		},

		// creates and returns the element
		/* parmameters:
		 * elementType: html element type "span", "div", etc.. (DEFAULT: DIV)
		 * styleClass: "one or more style classes seperated by a space"
		 * style: object containing html style info.  Used as dojo.attr(<>,"Style",style)
		 * parentNode: append this element to...
		 * baseDimensionsOnNode: set default dimensions based on node...:
		 * innerHTML: a string that will be put within the elements opening and closing tags
		 */
		createElement : function (
			elementId, parentNode, parms) {
			var newElement = null, elementType, sizeToElementCBox;
			if (!parms) {
				parms = {};
			}
			elementType = (parms.elementType) ? parms.elementType : "div";

			if (djDom.byId(elementId)) {
				return djDom.byId(elementId);
			}
			try {
				newElement = document.createElement(elementType);
				djDomAttr.set(newElement, "id", elementId);
				if (parms.styleClass) {
					djDomAttr.set(newElement, "class", parms.styleClass);
				}
				if (parms.style) {
					djDomAttr.set(newElement, "style", parms.style);
				}
				// IE hack, we have to clone the node and do future work from the clone
				if (parentNode) {
					parentNode.appendChild(newElement.cloneNode(true));
				} else {
					newElement.cloneNode(true);
				}
				newElement = djDom.byId(elementId);
			} catch (e) {
				this.CMError("error appending new element (" + elementId + "):" + e.message);
			}

			if (!newElement) {
				return null;
			}

			if (parms.baseDimensionsOnNode) {
				sizeToElementCBox = djDomG.getContentBox(parms.baseDimensionsOnNode);
				djDomG.setContentSize(newElement, {
					w : sizeToElementCBox.w,
					h : sizeToElementCBox.h
				});
			}

			if (parms.innerHTML) {
				newElement.innerHTML = parms.innerHTML;
			}

			return newElement;
		},

		myJoin : function (
			theArray, seperator) {
			var first = true, retString = "", elementIndex;
			if (!seperator) {
				seperator = ",";
			}
			for (elementIndex in theArray) {
				if (theArray.hasOwnProperty(elementIndex)) {
					if (first === false) {
						retString += seperator;
					}
					retString += theArray[elementIndex];
					first = false;
				}
			}
			return retString;
		},

		associativeArrayLength : function (
			theArray) {
			var size = 0, elementIndex;
			for (elementIndex in theArray) {
				if (theArray.hasOwnProperty(elementIndex)) {
					size++;
				}
			}
			return size;
		},

		//nice way to attach events with params; from http://stackoverflow.com/questions/95731/why-does-an-onclick-property-set-with-setattribute-fail-to-work-in-ie
		makeEvent1Param : function (
			element, callback, param1, event) {
			function local (
				e) {
				var ret = callback(param1, e);
				return ret;
			}

			if (element.addEventListener) {
				//Mozilla
				element.addEventListener(event, local, false);
			} else if (element.attachEvent) {
				//IE
				element.attachEvent("on" + event, local);
			}
		},

		// returns true if control or shift was pressed during the mouseEvent
		isMultiSelectKeyUsed : function (
			mouseEvent) {
			if (mouseEvent.ctrlKey === true || mouseEvent.shiftKey === true) {
				return true;
			}
			return false;
		},

		isArray : function (
			someVar) {
			if (Object.prototype.toString.call(someVar) === '[object Array]') {
				return true;
			}
			return false;
		},

		/*
		 * Will create a new span element as a child of 'element' parameter containing 'text' ellipsed if necessary
		 * parms { // all optional parms
		 *  styleClass: will append this style to the new span element
		 *  ellipseAt: int index to insert the ellipse at, use for inserting an ellipse at a point within the 'text'
		 *  ellipseAfter: string to insert ellipse after, use to insert ellipse at a point within the 'text'
		 *  returnOnly: true if we just want the string
		 *  }
		 */
		insertEllipsedTextSpanIntoElement : function (
			element, text, parms) {
			var elementCount = 0, elementId, width, wi, i, txtSpan, returnStr;
			if (!parms) {
				parms = {};
			}
			elementId = element.id + "_txtSpan" + elementCount;
			while (djDom.byId(elementId)) {
				elementCount++;
				elementId = element.id + "_txtSpan" + elementCount;
			}

			txtSpan = djDomC.create("span", {
				innerHTML : text
			}, element);
			djClass.add(txtSpan, parms.styleClass);

			width = djDomG.getContentBox(element).w;

			wi = djDomG.getContentBox(txtSpan).w;
			if (wi > width) {
				i = 1;
				txtSpan.innerHTML = '';
				wi = djDomG.getContentBox(txtSpan).w;
				while (wi < (width) && i < text.length) {
					txtSpan.innerHTML = text.substr(0, i) + '...';
					i++;
					wi = djDomG.getContentBox(txtSpan).w;
				}

				if (parms.insertAfter) {
					parms.insertAt = text.indexOf(parms.insertAfter) + parms.insertAfter.length;
				}
				if (parms.insertAt) {
					txtSpan.innerHTML = text.substr(0, parms.insertAt) + "..." + text.substr(parms.insertAt, i - 2);
				} else {
					txtSpan.innerHTML = text.substr(0, (i - 2)) + '...'; // 1 less than what sent us over the size limit
				}
			}

			if (parms.returnOnly) {
				returnStr = txtSpan.innerHTML;
				djDomC.destroy(txtSpan);
				return returnStr;
			}
		},

		splitFRMDividedProperty : function (
			item, propName) {
			var i, dataArray = [], property, propList;
			property = this.getProperty(item, propName);
			if (property) {
				this.CMTrace("OVERLAY", propName + ": " + property);
				propList = property.split("-,-");
				for (i = 0; i < propList.length; i++) {
					if (propList[i] !== "") {
						this.CMTrace("OVERLAY", "Property : " + propList[i]);
						var keyValuePair = propList[i].split("=");
						if (keyValuePair[1]) {
							dataArray[keyValuePair[0]] = keyValuePair[1];
						}
					}
				}
			} else {
				this.CMTrace("OVERLAY", propName + ": was empty or not passed");
			}
			return dataArray;
		},

		unSetBadPrototypes : function () {
			// json*.js sets the below prototypes.  This breaks dojo's tooltip.  Deleting the prototype
			//   appears to fix the problem.  Note however that dynamic refresh won't work without these 
			//   defined prototypes.
			delete Object.prototype.toJSONString;
			//delete Array.prototype.toJSONString;
			//String.prototype.parseJSON = "";
		}

	};
});
},
'ibm/tivoli/navmodel/TreeTableWidgetDef':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/WidgetDef"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.TreeTableWidgetDef"); 
 
// includes
dojo.require("ibm.tivoli.navmodel.WidgetDef");

////////////////////////////////////
// TreeTableWidgetDef
//
// This currently inherits all its behavior from TableWidgetDef.
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.TreeTableWidgetDef", ibm.tivoli.navmodel.TableWidgetDef, {

	/*=====
	constructor_kwArgs: function(baseUrl, provider, datasource, widget, definition, type, version) {
		// baseUrl: String
		//		base url for identifying rest service
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// widget: String
		//		widget type in form <type>-<version>
		// definition: String
		//		widget definition id in form <id>-<version>
		// type: String
		//		definition type:  "user", "shared", or "base"
		// version: Float
		//		version used to communicate with rest service
	},
	=====*/ 
	
	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs TreeTableWidgetDef object.
	}

	/////////////////////////////
	// public methods
	/////////////////////////////

		
	/////////////////////////////
	// dataset section data
	/////////////////////////////	
	

	/////////////////////////////
	// tables section data
	/////////////////////////////	


	/////////////////////////////
	// toolbars section data
	/////////////////////////////

});

});

},
'dojoe/treetablex/_QuickSearchFilter':function(){
define("dojoe/treetablex/_QuickSearchFilter", [
	"dojo/_base/declare",
	"dijit/_Widget",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	"dojo/dom-style",
	"dojo/dom-class",
	"dojo/i18n!dojoe/treetablex/nls/TreeTable",
	"dojo/text!./templates/QuickSearchFilter.html"
	], function (declare, _Widget, _TemplatedMixin,_WidgetsInTemplateMixin, domStyle, domClass, resourceBundle , template) {

		_QuickSearchFilter = declare('dojoe.treetablex._QuickSearchFilter', [_Widget, _TemplatedMixin, _WidgetsInTemplateMixin], {	

		// summary:
		//		Private class to create and handle Quick Search Filter on the Tree table toolbar
		// tags: private
	
		// Path to the template
		templateString : template, 
		
		// Set this to true if your widget contains other widgets
		widgetsInTemplate : true, 
		grid : null, 
		handle : null, 
		filterText : null, 
		filterLayer : null, 
		filterOptions : null, 
		_quickFilterButton: false, //change filter on type based on this	
		_quickFilterDelay: 700,	//delay in ms before quick filter is applied as typed	
		_ruleCntClearFilter: null,		
		_resources: null,
		
		// Override this method to perform custom behavior during dijit construction.
		// Common operations for constructor:
		// 1) Initialize non-primitive types (i.e. objects and arrays)
		// 2) Add additional properties needed by succeeding lifecycle methods
		constructor : function () {
			this._resources = resourceBundle;
		}, 
		
		postCreate : function () {
			if(this.filterOptions){
				if(this.filterOptions.showQuickFilterButton !== undefined) {
					this._quickFilterButton = this.filterOptions.showQuickFilterButton;
				}
				if(this.filterOptions.quickFilterDelay) {
					this._quickFilterDelay = this.filterOptions.quickFilterDelay;
				}
			}
			//disabling clear filter icon when there is no filter applied to avoid above confusion	
			if(this.filterText !== null && this.filterText !== "" )	{
				domStyle.set(this.clearFilterNode, "display", "block");
			}else{
				domStyle.set(this.clearFilterNode, "display", "none");
			}		
		}, 		
		
		// When this method is called, all variables inherited from superclasses are 'mixed in'.
		// Common operations for postMixInProperties
		// 1) Modify or assign values for widget property variables defined in the template HTML file
		postMixInProperties : function () {
			 
		}, 
		
		_parseDecimal : function (v) {
			return parseInt(v, 10);
		}, 
		
		_onFocus : function () {
			//return if grid._loading a variable indicating the grid loading is true
			if(this.grid._loading) return;
			domClass.add(this.domNode, 'dijitTextBoxFocused');
			if (this.searchField.value == this._resources.QF_DEFAULT_TEXT) {
				//Richa- Color settings should be there in css files
				//this.searchField.style.color = "#000000";
				this.searchField.value = "";
			}
		}, 
		
		_onClick : function () {
			//return if grid._loading a variable indicating the grid loading is true
			if(this.grid._loading) return;
			domClass.add(this.domNode, 'dijitTextBoxFocused');
			this.searchField.focus();
			//Richa- Color settings should be there in css files
			//this.searchField.style.color = "#000000";
			if (this.searchField.value == this._resources.QF_DEFAULT_TEXT) {
				this.searchField.value = "";
			}
		},
		
		_onBlur : function () {
			domClass.remove(this.domNode, 'dijitTextBoxFocused');
			domClass.remove(this.domNode, 'dijitTextBoxHover');
			if (!this.searchField.value) {
				this.searchField.style.color = "#999999";
				this.searchField.value = this._resources.QF_DEFAULT_TEXT;
			}
		}, 
		
		_onHover : function () {
			//LS- 26/10/2011 - # 30377 - Treetable - incorrect results when setting filter or sorting while data is being loaded
			//return if grid._loading a variable indicating the grid loading is true
			if(this.grid._loading) return;
			domClass.add(this.domNode, 'dijitTextBoxHover');
		}, 
		
		_onUnHover : function () {
			domClass.remove(this.domNode, 'dijitTextBoxHover');
		}, 
		
		_onKeyUp : function (event) {	
			//return if grid._loading a variable indicating the grid loading is true
			if(this.grid._loading) return;
			//tab & shift keys		
			//console.info("key -"+ event.keyCode);	
			this.filterText = this.searchField.value;
			if(event){
				if (event.keyCode == 9 || event.keyCode == 16) {
					return;
				}else if(this._quickFilterButton && event.keyCode !== 13){
					return;
				}			
				//console.log("filter ---"+this.filterText);
				if (event.keyCode == 13) {
					dojo.stopEvent(event);
					this._setFilter();
					return;
				}
			}
			this.onChangeFilterText();
		}, 
		
		_onCancel : function (event) {			
			//tab & shift keys	
			if(event.keyCode == 9 || event.keyCode == 16) {
				//Bobby:17Apr2011, tab gets stuck if stopped
				//dojo.stopEvent(event);
				return;
			}else{
				this._clearFilter();			
			}			
		}, 
		
		_clearFilter : function () {
			if(this.grid.filterBar && this.grid.filterBar.filterData && this._savedCriteria) {
			 var filterExp = this.grid.filterBar.filterData;
			 var filterArr = filterExp.conditions;
			 console.info(this._savedCriteria.conditions[0].value);
	 		 for(var i=0;i<filterArr.length;i++){
					
				if(!(filterArr[i].colId )&& (this._savedCriteria.conditions[0].value == filterArr[i].value) ){
					filterArr.splice(i,1);
					break;
			 	}
	 		 }
			 if(filterArr.length > 0){
				filterExp.conditions = filterArr;
				this.grid.filterBar.applyFilter(filterExp);
			 }else{
				this.grid.filterBar.clearFilter();
			 }
			 	
		}	
		this.searchField.style.color = "#999999";
		this.searchField.value = this._resources.QF_DEFAULT_TEXT;
		dojo.style(this.clearFilterNode, "display", "none");
		}, 
		_setFilter : function (filterText){
			if(filterText !== undefined && filterText !==''){
				this.filterText=filterText;
				this.searchField.value=filterText;
			}
			var input = this.searchField.value;
	   		if (input == this._resources.QF_DEFAULT_TEXT || input == "") {
				input = "";
				this._clearFilter();
				return;
			}
			var filterData  = {"type":"any", "conditions":
			[{  "type": "string",
			  "condition": "contain",
			  "value": input,
			  "colTxt": "Any Column",
			  "condTxt": "contain",
			  "formattedVal": input
			  }]};
			if(this._savedCriteria){
				var previousCriteria = this._savedCriteria;
			}
			this._savedCriteria = filterData;
			if(this.grid.filterBar && this.grid.filterBar.filterData){
				var filterSet = false;
				var filterObj = this.grid.filterBar.filterData;
				if(previousCriteria){
					for(var i=0; i< filterObj.conditions.length; i++){
						if(!(filterObj.conditions[i].colId)  && (previousCriteria.conditions[0].value == filterObj.conditions[i].value) ){
							filterObj.conditions[i].value = filterData.conditions[0].value;
							filterSet = true;
							break;
						}
					}
					if(filterSet)filterData = filterObj;
				} else if(filterObj){
					filterObj.conditions = filterObj.conditions.concat(filterData.conditions);
					console.dir(filterObj);
					filterData = filterObj;
				}
			}
			this._ruleCntClearFilter = this.grid.filterBar.ruleCountToConfirmClearFilter;
			this.grid.filterBar.ruleCountToConfirmClearFilter = 'Infinity';
			this.grid.filterBar.applyFilter(filterData);	
			dojo.style(this.clearFilterNode, "display", "block");	
		}, 
		
		_getFilter : function(){
			return this.filterText;
		},
			
		_enableQuickFilter: function(fitleredSize, totalSize) {
			if(this.clearFilterNode && this.clearFilterNode.style.display == "none" &&
				(this.filterText && this.filterText !=="" && this.searchField.value != this._resources.QF_DEFAULT_TEXT )){
					dojo.style(this.clearFilterNode, "display", "block");
				}
			if(this.searchField && this.searchField.disabled == true){
				this.searchField.disabled = false;
				if(this.filterText && this.filterText !=="" && this.searchField.value != this._resources.QF_DEFAULT_TEXT ){
					this.searchField.focus();
					//Richa- Color settings should be there in css files
					//this.searchField.style.color = "#000000";
				}else if (!this.searchField.value || this.searchField.value == this._resources.QF_DEFAULT_TEXT ){
					this.searchField.style.color = "#999999";
					this.searchField.value = this._resources.QF_DEFAULT_TEXT;
				}
			}
			//this.disconnect(this.fetchEnableFilterHandler);
			
		},
		
		_disableQuickFilter: function() {
			if(this.clearFilterNode && this.clearFilterNode.style.display == "block"){
				dojo.style(this.clearFilterNode, "display", "none");
			}
			if(this.searchField && this.searchField.disabled == false){
				this.searchField.disabled = true;
				this._onBlur();
				this.searchField.style.color = "#999999";
			}
		},
		
		onChangeFilterText : function () {
			if (this.handle) {
				clearTimeout(this.handle);
				this.handle = null;
			}
			var fnc = dojo.hitch(this, this._setFilter, this.filterText);
			this.handle = setTimeout(fnc, this._quickFilterDelay);
		}, 
		
		destroy: function(){
			// summary:  Destroys the toolbar.
			// tags: public	
			//memory leak issue: custom destroy method to take care of removing all widgets					
			try{
				delete this._resources;
				//this is for destroying child templated widgets 
				//this should happen by itself but need to figure out why dijit._Templated is not taking care of it...
				var widgets = dijit.findWidgets(this.domNode);
				dojo.forEach(widgets, function(w) {
					if(w){
						if(w.destroyRecursive){
							w.destroyRecursive();
						} else if(w.destroy){
							w.destroy();
						}
					}
				});
				// this is for widgets added as child to this widget and their children..
				//not present in template
				var widgetArr = this.getChildren();
				dojo.forEach(widgetArr, function (child) {
					if(child){
						if(child.destroyRecursive){
							child.destroyRecursive();
						} else if(child.destroy){
							child.destroy();
						}
					}
				});
				this.inherited(arguments);
			}catch(ex){
				console.warn("Widget destroy failed: "+this.id+", ex="+ex.name+":"+ex.message);
			}
		} 	
	});
	return _QuickSearchFilter;
	
});

},
'dojox/grid/enhanced/_Events':function(){
define([
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dojo/keys",
	"dojo/_base/html",
	"dojo/_base/event",
	"dojox/grid/_Events"
], function(dojo, declare, keys, html, event, _Events){

return declare("dojox.grid.enhanced._Events", null, {
	// summary:
	//		Overwrite some default events of DataGrid
	//
	// description:
	//		Methods are copied or replaced for overwriting, this might be refined once
	//		an overall plugin architecture is set up for DataGrid.

	//_events: Object
	//		Method map cached from dojox.grid._Events().
	_events: null,

	// headerCellActiveClass: String
	//		css class to apply to grid header cells when activated(mouse down)
	headerCellActiveClass: 'dojoxGridHeaderActive',
	
	// cellActiveClass: String
	//		css class to apply to grid content cells when activated(mouse down)
	cellActiveClass: 'dojoxGridCellActive',
	
	// rowActiveClass: String
	//		css class to apply to grid rows when activated(mouse down)
	rowActiveClass: 'dojoxGridRowActive',

	constructor: function(inGrid){
		//TODO - extend dojox.grid._Events rather than mixin for 1.8
		this._events = new _Events();
		//mixin "this" to Grid
		inGrid.mixin(inGrid, this);
	},
	dokeyup: function(e){
		// summary:
		//		Grid key up event handler.
		// e: Event
		//		Un-decorated event object
		this.focus.currentArea().keyup(e);
	},
	onKeyDown: function(e){
		// summary:
		//		Overwritten, see dojox.grid._Events.onKeyDown();
		if(e.altKey || e.metaKey){ return; }
		var focus = this.focus;
		var editing = this.edit.isEditing();
		switch(e.keyCode){
			case keys.TAB:
				if(e.ctrlKey){ return; }
				focus.tab(e.shiftKey ? -1:1,e);
				break;
			case keys.UP_ARROW:
			case keys.DOWN_ARROW:
				if(editing){ return; }
				focus.currentArea().move(e.keyCode == keys.UP_ARROW ? -1 : 1, 0, e);
				break;
			case keys.LEFT_ARROW:
			case keys.RIGHT_ARROW:
				if(editing){ return; }
				var offset = (e.keyCode == keys.LEFT_ARROW) ? 1 : -1;
				if(html._isBodyLtr()){ offset *= -1; }
				focus.currentArea().move(0, offset, e);
				break;
			case keys.F10:
				if(this.menus && e.shiftKey){
					this.onRowContextMenu(e);
				}
				break;
			default:
				focus.currentArea().keydown(e);
				break;
		}
	},
	//TODO - make the following events more reasonalble - e.g. more accurate conditions
	//events for row selectors
	domouseup: function(e){
		if(e.cellNode){
			this.onMouseUp(e);
		}else{
			this.onRowSelectorMouseUp(e);
		}
	},
	domousedown: function(e){
		if(!e.cellNode){
			this.onRowSelectorMouseDown(e);
		}
	},
	onMouseUp: function(e){
		// summary:
		//		New - Event fired when mouse is up inside grid.
		// e: Event
		//		Decorated event object that contains reference to grid, cell, and rowIndex
		this[e.rowIndex == -1 ? "onHeaderCellMouseUp" : "onCellMouseUp"](e);
	},
	onCellMouseDown: function(e){
		// summary:
		//		Overwritten, see dojox.grid._Events.onCellMouseDown()
		html.addClass(e.cellNode, this.cellActiveClass);
		html.addClass(e.rowNode, this.rowActiveClass);
	},
	onCellMouseUp: function(e){
		// summary:
		//		New - Event fired when mouse is up inside content cell.
		// e: Event
		//		Decorated event object that contains reference to grid, cell, and rowIndex
		html.removeClass(e.cellNode, this.cellActiveClass);
		html.removeClass(e.rowNode, this.rowActiveClass);
	},
	onCellClick: function(e){
		// summary:
		//		Overwritten, see dojox.grid._Events.onCellClick()

		//invoke dojox.grid._Events.onCellClick()
		this._events.onCellClick.call(this, e);
		//move mouse events to the focus manager.
		this.focus.contentMouseEvent(e);//TODO
	},
	onCellDblClick: function(e){
		// summary:
		//		Overwritten, see dojox.grid._Events.onCellDblClick()
		if(this.pluginMgr.isFixedCell(e.cell)){ return; }
		if(this._click.length > 1 && (!this._click[0] || !this._click[1])){
			this._click[0] = this._click[1] = e;
		}
		//invoke dojox.grid._Events.onCellDblClick()
		this._events.onCellDblClick.call(this, e);
	},
	onRowClick: function(e){
		// summary:
		//		Overwritten, see dojox.grid._Events.onRowClick()
		this.edit.rowClick(e);
		if(!e.cell || !this.plugin('indirectSelection')){
			this.selection.clickSelectEvent(e);
		}
	},
	onRowContextMenu: function(e){
		// summary:
		//		Overwritten, see dojox.grid._Events.onRowContextMenu()
		if(/*!this.edit.isEditing() && */this.menus){
			this.showMenu(e);
		}
	},
	onSelectedRegionContextMenu: function(e){
		// summary:
		//		New - Event fired when a selected region context menu is accessed via mouse right click.
		// e: Event
		//		Decorated event object which contains reference to grid and info of selected
		//		regions(selection type - row|column, selected index - [...])
		if(this.selectedRegionMenu){
			this.selectedRegionMenu._openMyself({
				target: e.target,
				coords: e.keyCode !== keys.F10 && "pageX" in e ? {
					x: e.pageX,
					y: e.pageY
				} : null
			});
			event.stop(e);
		}
	},
	onHeaderCellMouseOut: function(e){
		// summary:
		//		Overwritten, see dojox.grid._Events.onHeaderCellMouseOut()
		if(e.cellNode){
			html.removeClass(e.cellNode, this.cellOverClass);
			html.removeClass(e.cellNode, this.headerCellActiveClass);
		}
	},
	onHeaderCellMouseDown: function(e){
		// summary:
		//		Overwritten, see dojox.grid._Events.onHeaderCellMouseDown()
		if(e.cellNode){//TBD - apply to selection region for nested sorting?
			html.addClass(e.cellNode, this.headerCellActiveClass);
		}
	},
	onHeaderCellMouseUp: function(e){
		// summary:
		//		New event
		if(e.cellNode){
			html.removeClass(e.cellNode, this.headerCellActiveClass);
		}
	},
	onHeaderCellClick: function(e){
		// summary:
		//		Overwritten, see dojox.grid._Events.onHeaderCellClick()
		//move focus to header.
		this.focus.currentArea("header");
		//invoke dojox.grid._Events.onHeaderCellClick()
		if(!e.cell.isRowSelector){
			this._events.onHeaderCellClick.call(this, e);
		}
		//move mouse events to the focus manager.
		this.focus.headerMouseEvent(e);
	},
	onRowSelectorMouseDown: function(e){
		this.focus.focusArea("rowHeader", e);
	},
	
	onRowSelectorMouseUp: function(e){},
	
	//triggered in _View, see Selector plugin
	onMouseUpRow: function(e){
		if(e.rowIndex != -1){
			this.onRowMouseUp(e);
		}
	},
	onRowMouseUp: function(e){}
});
});

},
'gridx/modules/Focus':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/connect",
	"dojo/_base/lang",
	"dojo/_base/sniff",
	"dojo/_base/window",
	"dojo/keys",
	"../core/_Module",
	"../core/util"
], function(declare, array, connect, lang, sniff, win, keys, _Module, util){

	/*=====
		gridx.modules.Focus.__FocusArea = function(){
			// name: String (mandatory)
			//		The name of this area. Must be unique. Must not be empty.
			//
			// priority: Number (mandatory)
			//		This number decides the position of this area in the TAB sequence.
			//		Areas with bigger priority number, their position in TAB sequence comes later.
			//		If two areas have the same priority, then the later registered area is put *above* the earlier one.
			//		That is, no matter TAB or SHIFT-TAB, the upper area is accessed first.
			//
			// focusNode: DOM-Node?
			//		If provided, this is the node of this area. 
			//		When this area is focused, *onFocus* will be called. When blurred, *onBlur* will be called.
			//
			// scope: anything?
			//		If provided, all area functions are called on this scope.
			//
			// doFocus: Function(evt, step)?
			//		If provided, will be called when TABing to this area.
			//		If not provided, default to successful focus.
			//		Return TRUE if successfully focused. FALSE if not.
			//
			// doBlur: Function(evt, step)?
			//		If provided, will be called when TABing out of this area.
			//		If not provided, default to successful blur.
			//		Return TRUE if successfully blurred. FALSE if not.
			//
			// onFocus: function(evt)?
			//		If provided, will be called when the *focusNode* of this area is focused.
			//		If return TRUE, later areas on this node will be skipped and this area becomes the current focused area.
			//		If return FALSE, call later areas on this same node.
			//
			// onBlur: function(evt)?
			//		If provided, will be called when the *focusNode* of this area is blurred.
			//		When *focusNode* is blurred, only the currently focused area will be called.
		};
	=====*/

	return declare(/*===== "gridx.modules.Focus", =====*/_Module, {
		// summary
		//		This module controls the TAB sequence of all the UI modules.
		//		But this module is (or at least can be) a non-UI module, because it does not handle the actual focus job.
		
		name: 'focus',
		
		getAPIPath: function(){
			return {
				focus: this
			};
		},

		constructor: function(){
			var t = this,
				g = t.grid;
			t._areas = {};
			t._tabQueue = [];
			t._focusNodes = [];
			t._onDocFocus = function(evt){
				if(!t._noBlur){
					if(sniff('ie')){
						evt.target = evt.srcElement;
					}
					t._onFocus(evt);
				}
			};
			t.batchConnect(
				[g.domNode, 'onkeydown', '_onTabDown'],
				[g.domNode, 'onfocus', '_focus'],
				[g.lastFocusNode, 'onfocus', '_focus'],
				[g, 'onBlur', '_doBlur']);
			if(sniff('ie')){
				win.doc.attachEvent('onfocusin', t._onDocFocus);
			}else{
				win.doc.addEventListener('focus', t._onDocFocus, true);
			}
		},

		destroy: function(){
			var t = this;
			t._areas = null;
			t._areaQueue = null;
			t._focusNodes = [];
			t._queueIdx = -1;
			if(sniff('ie')){
				win.doc.detachEvent('onfocusin', t._onDocFocus);
			}else{
				win.doc.removeEventListener('focus', t._onDocFocus, true);
			}
			t.inherited(arguments);
		},
	
		//Public----------------------------------------------------------
		registerArea: function(/* __FocusArea */ area){
			// summary:
			//		Register a new focus area, so this area will be included in the TAB sequence.
			//		If there's an existing area with the same name, it is removed and replaced by the new area.
			//		This function always succeed. No exception.
			// tags:
			//		package
			// area: __FocusArea
			//		A focus area definition.
			if(area && area.name && typeof area.priority == 'number'){
				var t = this,
					tq = t._tabQueue,
					init = function(fn){
						area[fn] = area[fn] ? lang.hitch(area.scope || area, area[fn]) : function(){ return true; };
					};
				if(t._areas[area.name]){
					t.removeArea(area.name);
				}
				init('doFocus');
				init('doBlur');
				init('onFocus');
				init('onBlur');
				area.connects = area.connects || [];

				t._areas[area.name] = area;
				var i = util.biSearch(tq, function(a){
					return a.p - area.priority;
				});
				//If the priority is the same, put this area above the current one.
				if(tq[i] && tq[i].p === area.priority){
					tq[i].stack.unshift(area.name);
					//Assuming areas with same priority must have same focusNode.
					t._focusNodes[i] = area.focusNode || t._focusNodes[i];
				}else{
					tq.splice(i, 0, {
						p: area.priority,
						stack: [area.name]
					});
					t._focusNodes.splice(i, 0, area.focusNode);
				}
			}
		},

		focusArea: function(/* String */ areaName, forced){
			// summary:
			//		Focus the area with name of *areaName*.
			//		If the current area is not this area, blur the current area.
			//		If the current area is this area, this is a no-op and return TRUE.
			//		If the area with this name does not exist, this is a no-op and return FALSE.
			// tags:
			//		package
			// return: Boolean
			//		TRUE if the focus is successful, FALSE if not.	
			var t = this, area = t._areas[areaName];
			if(area){
				var curArea = t._areas[t.currentArea()];
				if(curArea && curArea.name === areaName){
					if(forced){
						curArea.doFocus();
					}
					return true;
				}else if(!curArea || curArea.doBlur()){
					if(curArea){
						t.onBlurArea(curArea.name);
					}
					if(area.doFocus()){
						t.onFocusArea(area.name);
						t._updateCurrentArea(area);
						return true;
					}
					t._updateCurrentArea();
				}
			}
			return false;
		},

		currentArea: function(){
			// summary:
			//		Get the name of the current focus area. 
			// tags:
			//		package
			// return: String
			//		The name of the current Area. Return "" if no area is focused.
			var a = this._tabQueue[this._queueIdx];
			return a ? a.stack[this._stackIdx] : '';
		},

		tab: function(step, evt){
			// summary:
			//		Move focus from one area to another.
			// tags:
			//		package
			// step: Integer
			//		If positive, then move forward along the TAB sequence.
			//		If negative, then move backward along the TAB sequence (SHIFT-TAB).
			//		If zero or other invalid values, this is a no-op.
			//		The absolute value of *step* is the distance between the target area and the current area
			//		in the whole TAB sequence.
			// evt: Object
			//		This can either be a real Event object or a mock object with same information .
			// return: String
			//		The name of currently focused area. Return "" if no area is focused.
			var t = this,
				areas = t._areas,
				curArea = areas[t.currentArea()];
			if(!step){
				return curArea ? curArea.name : '';
			}
			var cai = t._queueIdx + step,
				dir = step > 0 ? 1 : -1,
				tq = t._tabQueue;
			if(curArea){
				var blurResult = curArea.doBlur(evt, step),
					nextArea = areas[blurResult];
				if(blurResult){
					t.onBlurArea(curArea.name);
				}
				if(nextArea && nextArea.doFocus(evt, step)){
					t.onFocusArea(nextArea.name);
					t._updateCurrentArea(nextArea);
					return nextArea.name;
				}else if(!blurResult){
					return curArea.name;
				}
			}
			for(; cai >= 0 && cai < tq.length; cai += dir){
				var i, stack = tq[cai].stack;
				for(i = 0; i < stack.length; ++i){
					var areaName = stack[i];
					if(areas[areaName].doFocus(evt, step)){
						t.onFocusArea(areaName);
						t._queueIdx = cai;
						t._stackIdx = i;
						return areaName;
					}
				}
			}
			t._tabingOut = 1;
			if(step < 0){
				t._queueIdx = -1;
				t.grid.domNode.focus();
			}else{
				t._queueIdx = tq.length;
				t.grid.lastFocusNode.focus();
			}
			return "";
		},

		removeArea: function(areaName){
			// summary:
			//		Remove the area with name of *areaName*.
			//		If there's no such area, this is a no-op and return FALSE.
			//		If currently focused area is removed, then current area becomes empty.
			// tags:
			//		package
			// areaName: String
			//		The name of the area to be removed.
			// return: Boolean
			//		TRUE if this operation is successful, FALSE if not.
			var t = this, area = t._areas[areaName];
			if(area){
				if(t.currentArea() === areaName){
					t._updateCurrentArea();
				}
				var i = util.biSearch(t._tabQueue, function(a){
						return a.p - area.priority;
					}), j, 
					stack = t._tabQueue[i].stack;
				for(j = stack.length - 1; j >= 0; --j){
					if(stack[j] === area.name){
						stack.splice(j, 1);
						break;
					}
				}
				if(!stack.length){
					t._tabQueue.splice(i, 1);
					t._focusNodes.splice(i, 1);
				}
				array.forEach(area.connects, connect.disconnect);
				delete t._areas[areaName];
				return true;
			}
			return false;
		},

		onFocusArea: function(/* String areaName*/){
			// summary:
			//		Fired when an area is focused.
			// tags:
			//		callback
		},

		onBlurArea: function(/* String areaName */){
			// summary:
			//		Fired when an area is blurred.
			// tags:
			//		callback
		},

		//Private----------------------------------------------------------
		//_areas: null,
		//_tabQueue: null,
		//_focusNodes: null,
		_queueIdx: -1,
		_stackIdx: 0,

		_onTabDown: function(evt){
			if(evt.keyCode === keys.TAB){
				this.tab(evt.shiftKey ? -1 : 1, evt);
			}
		},

		//-----------------------------------------------------------------------
		_onFocus: function(evt){
			var t = this, i, j, stack, area,
				dn = t.grid.domNode,
				n = evt.target,
				currentArea = t._areas[t.currentArea()];
			while(n && n !== dn){
				i = array.indexOf(t._focusNodes, n);
				if(i >= 0){
					stack = t._tabQueue[i].stack;
					for(j = 0; j < stack.length; ++j){
						area = t._areas[stack[j]];
						if(area.onFocus(evt)){
							if(currentArea && currentArea.name !== area.name){
								currentArea.onBlur(evt);
								t.onBlurArea(currentArea.name);
							}
							t.onFocusArea(area.name);
							t._queueIdx = i;
							t._stackIdx = j;
							return;
						}
					}
					return;
				}
				n = n.parentNode;
			}
			if(n == dn && currentArea){
				t._doBlur(evt, currentArea);
			}
		},

		_focus: function(evt){
			var t = this;
			if(t._tabingOut){
				t._tabingOut = 0;
			}else if(evt.target == t.grid.domNode){
				t._queueIdx = -1;
				t.tab(1);
			}else if(evt.target === t.grid.lastFocusNode){
				t._queueIdx = t._tabQueue.length;
				t.tab(-1);
			}
		},

		_doBlur: function(evt, area){
			var t = this;
			if(!area && t.currentArea()){
				area = t._areas[t.currentArea()];
			}
			if(area){
				area.onBlur(evt);
				t.onBlurArea(area.name);
				t._updateCurrentArea();
			}
		},

		_updateCurrentArea: function(area){
			var t = this, tq = t._tabQueue;
			if(area){
				var i = t._queueIdx = util.biSearch(tq, function(a){
						return a.p - area.priority;
					}),
					stack = tq[i].stack;
				t._stackIdx = array.indexOf(stack, area.name);
			}else{
				t._queueIdx = null;
				t._stackIdx = 0;
			}
		}
	});
});

},
'url:ibm/usmi/statuspod/templates/alertStatusTipTableRow.html':"<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n\t\t<tr>\r\n\t\t\t<td style=\"${padding}\" colspan=\"2\">\r\n\t\t\t\t<img src=\"${alertSev}\"/> \r\n\t\t\t\t<span class=\"statusPodTipAlertText\" >${alertText}\r\n\t\t\t\t</span>\r\n\t\t\t</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<td>\r\n\t\t\t\t<span class=\"statusPodTipAlert\">${StatusTipCategory} ${alertCat}</span>\r\n\t\t\t</td>\r\n\t\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<td>\r\n\t\t\t\t<span class=\"statusPodTipAlert\">${StatusTipDate} ${alertDateTime}</span>\r\n\t\t\t</td>\r\n\t\t</tr>",
'dijit/_TimePicker':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/date", // date.compare
	"dojo/date/locale", // locale.format
	"dojo/date/stamp", // stamp.fromISOString stamp.toISOString
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.add domClass.contains domClass.toggle
	"dojo/dom-construct", // domConstruct.create
	"dojo/_base/event", // event.stop
	"dojo/_base/kernel", // deprecated
	"dojo/keys", // keys
	"dojo/_base/lang", // lang.mixin
	"dojo/_base/sniff", // has(...)
	"dojo/query", // query
	"./typematic",
	"./_Widget",
	"./_TemplatedMixin",
	"./form/_FormValueWidget",
	"dojo/text!./templates/TimePicker.html"
], function(array, ddate, locale, stamp, declare, domClass, domConstruct, event, kernel, keys, lang, has, query,
			typematic, _Widget, _TemplatedMixin, _FormValueWidget, template){

/*=====
	var _Widget = dijit._Widget;
	var _TemplatedMixin = dijit._TemplatedMixin;
	var _FormValueWidget = dijit.form._FormValueWidget;
=====*/

	// module:
	//		dijit/_TimePicker
	// summary:
	//		A graphical time picker.


	/*=====
	declare(
		"dijit._TimePicker.__Constraints",
		locale.__FormatOptions,
		{
			// clickableIncrement: String
			//		See `dijit._TimePicker.clickableIncrement`
			clickableIncrement: "T00:15:00",

			// visibleIncrement: String
			//		See `dijit._TimePicker.visibleIncrement`
			visibleIncrement: "T01:00:00",

			// visibleRange: String
			//		See `dijit._TimePicker.visibleRange`
			visibleRange: "T05:00:00"
		}
	);
	=====*/

	return declare("dijit._TimePicker", [_Widget, _TemplatedMixin], {
		// summary:
		//		A graphical time picker.
		//		This widget is used internally by other widgets and is not available
		//		as a standalone widget due to lack of accessibility support.

		templateString: template,

		// baseClass: [protected] String
		//		The root className to use for the various states of this widget
		baseClass: "dijitTimePicker",

		// clickableIncrement: String
		//		ISO-8601 string representing the amount by which
		//		every clickable element in the time picker increases.
		//		Set in local time, without a time zone.
		//		Example: `T00:15:00` creates 15 minute increments
		//		Must divide dijit._TimePicker.visibleIncrement evenly
		clickableIncrement: "T00:15:00",

		// visibleIncrement: String
		//		ISO-8601 string representing the amount by which
		//		every element with a visible time in the time picker increases.
		//		Set in local time, without a time zone.
		//		Example: `T01:00:00` creates text in every 1 hour increment
		visibleIncrement: "T01:00:00",

		// visibleRange: String
		//		ISO-8601 string representing the range of this TimePicker.
		//		The TimePicker will only display times in this range.
		//		Example: `T05:00:00` displays 5 hours of options
		visibleRange: "T05:00:00",

		// value: String
		//		Date to display.
		//		Defaults to current time and date.
		//		Can be a Date object or an ISO-8601 string.
		//		If you specify the GMT time zone (`-01:00`),
		//		the time will be converted to the local time in the local time zone.
		//		Otherwise, the time is considered to be in the local time zone.
		//		If you specify the date and isDate is true, the date is used.
		//		Example: if your local time zone is `GMT -05:00`,
		//		`T10:00:00` becomes `T10:00:00-05:00` (considered to be local time),
		//		`T10:00:00-01:00` becomes `T06:00:00-05:00` (4 hour difference),
		//		`T10:00:00Z` becomes `T05:00:00-05:00` (5 hour difference between Zulu and local time)
		//		`yyyy-mm-ddThh:mm:ss` is the format to set the date and time
		//		Example: `2007-06-01T09:00:00`
		value: new Date(),

		_visibleIncrement:2,
		_clickableIncrement:1,
		_totalIncrements:10,

		// constraints: dijit._TimePicker.__Constraints
		//		Specifies valid range of times (start time, end time)
		constraints:{},

/*=====
		serialize: function(val, options){
			// summary:
			//		User overridable function used to convert the attr('value') result to a String
			// val: Date
			//		The current value
			// options: Object?
			// tags:
			//		protected
		},
=====*/
		serialize: stamp.toISOString,

/*=====
		// filterString: string
		//		The string to filter by
		filterString: "",
=====*/

		setValue: function(/*Date*/ value){
			// summary:
			//		Deprecated.  Used set('value') instead.
			// tags:
			//		deprecated
			kernel.deprecated("dijit._TimePicker:setValue() is deprecated.  Use set('value', ...) instead.", "", "2.0");
			this.set('value', value);
		},

		_setValueAttr: function(/*Date*/ date){
			// summary:
			//		Hook so set('value', ...) works.
			// description:
			//		Set the value of the TimePicker.
			//		Redraws the TimePicker around the new date.
			// tags:
			//		protected
			this._set("value", date);
			this._showText();
		},

		_setFilterStringAttr: function(val){
			// summary:
			//		Called by TimeTextBox to filter the values shown in my list
			this._set("filterString", val);
			this._showText();
		},

		isDisabledDate: function(/*===== dateObject, locale =====*/){
			// summary:
			//		May be overridden to disable certain dates in the TimePicker e.g. `isDisabledDate=locale.isWeekend`
			// dateObject: Date
			// locale: String?
			// type:
			//		extension
			return false; // Boolean
		},

		_getFilteredNodes: function(/*number*/ start, /*number*/ maxNum, /*Boolean*/ before, /*DOMnode*/ lastNode){
			// summary:
			//		Returns an array of nodes with the filter applied.  At most maxNum nodes
			//		will be returned - but fewer may be returned as well.  If the
			//		before parameter is set to true, then it will return the elements
			//		before the given index
			// tags:
			//		private
			var
				nodes = [],
				lastValue = lastNode ? lastNode.date : this._refDate,
				n,
				i = start,
				max = this._maxIncrement + Math.abs(i),
				chk = before ? -1 : 1,
				dec = before ? 1 : 0,
				inc = 1 - dec;
			do{
				i -= dec;
				n = this._createOption(i);
				if(n){
					if((before && n.date > lastValue) || (!before && n.date < lastValue)){
						break; // don't wrap
					}
					nodes[before ? "unshift" : "push"](n);
					lastValue = n.date;
				}
				i += inc;
			}while(nodes.length < maxNum && (i*chk) < max);
			return nodes;
		},

		_showText: function(){
			// summary:
			//		Displays the relevant choices in the drop down list
			// tags:
			//		private
			var fromIso = stamp.fromISOString;
			this.timeMenu.innerHTML = "";
			this._clickableIncrementDate=fromIso(this.clickableIncrement);
			this._visibleIncrementDate=fromIso(this.visibleIncrement);
			this._visibleRangeDate=fromIso(this.visibleRange);
			// get the value of the increments and the range in seconds (since 00:00:00) to find out how many divs to create
			var
				sinceMidnight = function(/*Date*/ date){
					return date.getHours() * 60 * 60 + date.getMinutes() * 60 + date.getSeconds();
				},
				clickableIncrementSeconds = sinceMidnight(this._clickableIncrementDate),
				visibleIncrementSeconds = sinceMidnight(this._visibleIncrementDate),
				visibleRangeSeconds = sinceMidnight(this._visibleRangeDate),
				// round reference date to previous visible increment
				time = (this.value || this.currentFocus).getTime();

			this._refDate = new Date(time - time % (clickableIncrementSeconds*1000));
			this._refDate.setFullYear(1970,0,1); // match parse defaults

			// assume clickable increment is the smallest unit
			this._clickableIncrement = 1;
			// divide the visible range by the clickable increment to get the number of divs to create
			// example: 10:00:00/00:15:00 -> display 40 divs
			this._totalIncrements = visibleRangeSeconds / clickableIncrementSeconds;
			// divide the visible increments by the clickable increments to get how often to display the time inline
			// example: 01:00:00/00:15:00 -> display the time every 4 divs
			this._visibleIncrement = visibleIncrementSeconds / clickableIncrementSeconds;
			// divide the number of seconds in a day by the clickable increment in seconds to get the
			// absolute max number of increments.
			this._maxIncrement = (60 * 60 * 24) / clickableIncrementSeconds;

			var
				// Find the nodes we should display based on our filter.
				// Limit to 10 nodes displayed as a half-hearted attempt to stop drop down from overlapping <input>.
				count = Math.min(this._totalIncrements, 10),
				after = this._getFilteredNodes(0, (count >> 1) + 1, false),
				moreAfter = [],
				estBeforeLength = count - after.length,
				before = this._getFilteredNodes(0, estBeforeLength, true, after[0]);
				if(before.length < estBeforeLength && after.length > 0){
					moreAfter = this._getFilteredNodes(after.length, estBeforeLength - before.length, false, after[after.length-1]);
				}
			array.forEach(before.concat(after, moreAfter), function(n){ this.timeMenu.appendChild(n); }, this);
		},

		constructor: function(){
			this.constraints = {}; // create instance object
		},

		postMixInProperties: function(){
			this.inherited(arguments);
			this._setConstraintsAttr(this.constraints); // this needs to happen now (and later) due to codependency on _set*Attr calls
		},

		_setConstraintsAttr: function(/* Object */ constraints){
			// brings in visibleRange, increments, etc.
			lang.mixin(this, constraints);

			// locale needs the lang in the constraints as locale
			if(!constraints.locale){
				constraints.locale = this.lang;
			}
		},

		postCreate: function(){
			// assign typematic mouse listeners to the arrow buttons
			this.connect(this.timeMenu, has("mozilla") ? 'DOMMouseScroll' : "onmousewheel", "_mouseWheeled");
			this._connects.push(typematic.addMouseListener(this.upArrow, this, "_onArrowUp", 33, 250));
			this._connects.push(typematic.addMouseListener(this.downArrow, this, "_onArrowDown", 33, 250));

			this.inherited(arguments);
		},

		_buttonMouse: function(/*Event*/ e){
			// summary:
			//		Handler for hover (and unhover) on up/down arrows
			// tags:
			//		private

			// in non-IE browser the "mouseenter" event will become "mouseover",
			// but in IE it's still "mouseenter"
			domClass.toggle(e.currentTarget, e.currentTarget == this.upArrow ? "dijitUpArrowHover" : "dijitDownArrowHover",
				e.type == "mouseenter" || e.type == "mouseover");
		},

		_createOption: function(/*Number*/ index){
			// summary:
			//		Creates a clickable time option
			// tags:
			//		private
			var date = new Date(this._refDate);
			var incrementDate = this._clickableIncrementDate;
			date.setTime(date.getTime()
				+ incrementDate.getHours() * index * 3600000
				+ incrementDate.getMinutes() * index * 60000
				+ incrementDate.getSeconds() * index * 1000);
			if(this.constraints.selector == "time"){
				date.setFullYear(1970,0,1); // make sure each time is for the same date
			}
			var dateString = locale.format(date, this.constraints);
			if(this.filterString && dateString.toLowerCase().indexOf(this.filterString) !== 0){
				// Doesn't match the filter - return null
				return null;
			}

			var div = domConstruct.create("div", {"class": this.baseClass+"Item"});
			div.date = date;
			div.idx = index;
			domConstruct.create('div',{
				"class": this.baseClass + "ItemInner",
				innerHTML: dateString
			}, div);

			if(index%this._visibleIncrement<1 && index%this._visibleIncrement>-1){
				domClass.add(div, this.baseClass+"Marker");
			}else if(!(index%this._clickableIncrement)){
				domClass.add(div, this.baseClass+"Tick");
			}

			if(this.isDisabledDate(date)){
				// set disabled
				domClass.add(div, this.baseClass+"ItemDisabled");
			}
			if(this.value && !ddate.compare(this.value, date, this.constraints.selector)){
				div.selected = true;
				domClass.add(div, this.baseClass+"ItemSelected");
				if(domClass.contains(div, this.baseClass+"Marker")){
					domClass.add(div, this.baseClass+"MarkerSelected");
				}else{
					domClass.add(div, this.baseClass+"TickSelected");
				}

				// Initially highlight the current value.   User can change highlight by up/down arrow keys
				// or mouse movement.
				this._highlightOption(div, true);
			}
			return div;
		},

		_onOptionSelected: function(/*Object*/ tgt){
			// summary:
			//		Called when user clicks an option in the drop down list
			// tags:
			//		private
			var tdate = tgt.target.date || tgt.target.parentNode.date;
			if(!tdate || this.isDisabledDate(tdate)){ return; }
			this._highlighted_option = null;
			this.set('value', tdate);
			this.onChange(tdate);
		},

		onChange: function(/*Date*/ /*===== time =====*/){
			// summary:
			//		Notification that a time was selected.  It may be the same as the previous value.
			// tags:
			//      public
		},

		_highlightOption: function(/*node*/ node, /*Boolean*/ highlight){
			// summary:
			//		Turns on/off highlight effect on a node based on mouse out/over event
			// tags:
			//		private
			if(!node){return;}
			if(highlight){
				if(this._highlighted_option){
					this._highlightOption(this._highlighted_option, false);
				}
				this._highlighted_option = node;
			}else if(this._highlighted_option !== node){
				return;
			}else{
				this._highlighted_option = null;
			}
			domClass.toggle(node, this.baseClass+"ItemHover", highlight);
			if(domClass.contains(node, this.baseClass+"Marker")){
				domClass.toggle(node, this.baseClass+"MarkerHover", highlight);
			}else{
				domClass.toggle(node, this.baseClass+"TickHover", highlight);
			}
		},

		onmouseover: function(/*Event*/ e){
			// summary:
			//		Handler for onmouseover event
			// tags:
			//		private
			this._keyboardSelected = null;
			var tgr = (e.target.parentNode === this.timeMenu) ? e.target : e.target.parentNode;
			// if we aren't targeting an item, then we return
			if(!domClass.contains(tgr, this.baseClass+"Item")){return;}
			this._highlightOption(tgr, true);
		},

		onmouseout: function(/*Event*/ e){
			// summary:
			//		Handler for onmouseout event
			// tags:
			//		private
			this._keyboardSelected = null;
			var tgr = (e.target.parentNode === this.timeMenu) ? e.target : e.target.parentNode;
			this._highlightOption(tgr, false);
		},

		_mouseWheeled: function(/*Event*/ e){
			// summary:
			//		Handle the mouse wheel events
			// tags:
			//		private
			this._keyboardSelected = null;
			event.stop(e);
			// we're not _measuring_ the scroll amount, just direction
			var scrollAmount = has("mozilla") ? -e.detail : e.wheelDelta;
			this[(scrollAmount>0 ? "_onArrowUp" : "_onArrowDown")](); // yes, we're making a new dom node every time you mousewheel, or click
		},

		_onArrowUp: function(count){
			// summary:
			//		Handler for up arrow key.
			// description:
			//		Removes the bottom time and add one to the top
			// tags:
			//		private
			if(count === -1){
				domClass.remove(this.upArrow, "dijitUpArrowActive");
				return;
			}else if(count === 0){
				domClass.add(this.upArrow, "dijitUpArrowActive");

			} // typematic end
			if(!this.timeMenu.childNodes.length){ return; }
			var index = this.timeMenu.childNodes[0].idx;
			var divs = this._getFilteredNodes(index, 1, true, this.timeMenu.childNodes[0]);
			if(divs.length){
				this.timeMenu.removeChild(this.timeMenu.childNodes[this.timeMenu.childNodes.length - 1]);
				this.timeMenu.insertBefore(divs[0], this.timeMenu.childNodes[0]);
			}
		},

		_onArrowDown: function(count){
			// summary:
			//		Handler for up arrow key.
			// description:
			//		Remove the top time and add one to the bottom
			// tags:
			//		private
			if(count === -1){
				domClass.remove(this.downArrow, "dijitDownArrowActive");
				return;
			}else if(count === 0){
				domClass.add(this.downArrow, "dijitDownArrowActive");
			} // typematic end
			if(!this.timeMenu.childNodes.length){ return; }
			var index = this.timeMenu.childNodes[this.timeMenu.childNodes.length - 1].idx + 1;
			var divs = this._getFilteredNodes(index, 1, false, this.timeMenu.childNodes[this.timeMenu.childNodes.length - 1]);
			if(divs.length){
				this.timeMenu.removeChild(this.timeMenu.childNodes[0]);
				this.timeMenu.appendChild(divs[0]);
			}
		},

		handleKey: function(/*Event*/ e){
			// summary:
			//		Called from `dijit.form._DateTimeTextBox` to pass a keypress event
			//		from the `dijit.form.TimeTextBox` to be handled in this widget
			// tags:
			//		protected
			if(e.charOrCode == keys.DOWN_ARROW || e.charOrCode == keys.UP_ARROW){
				event.stop(e);
				// Figure out which option to highlight now and then highlight it
				if(this._highlighted_option && !this._highlighted_option.parentNode){
					this._highlighted_option = null;
				}
				var timeMenu = this.timeMenu,
					tgt = this._highlighted_option || query("." + this.baseClass + "ItemSelected", timeMenu)[0];
				if(!tgt){
					tgt = timeMenu.childNodes[0];
				}else if(timeMenu.childNodes.length){
					if(e.charOrCode == keys.DOWN_ARROW && !tgt.nextSibling){
						this._onArrowDown();
					}else if(e.charOrCode == keys.UP_ARROW && !tgt.previousSibling){
						this._onArrowUp();
					}
					if(e.charOrCode == keys.DOWN_ARROW){
						tgt = tgt.nextSibling;
					}else{
						tgt = tgt.previousSibling;
					}
				}
				this._highlightOption(tgt, true);
				this._keyboardSelected = tgt;
				return false;
			}else if(e.charOrCode == keys.ENTER || e.charOrCode === keys.TAB){
				// mouse hover followed by TAB is NO selection
				if(!this._keyboardSelected && e.charOrCode === keys.TAB){
					return true;	// true means don't call stopEvent()
				}

				// Accept the currently-highlighted option as the value
				if(this._highlighted_option){
					this._onOptionSelected({target: this._highlighted_option});
				}

				// Call stopEvent() for ENTER key so that form doesn't submit,
				// but not for TAB, so that TAB does switch focus
				return e.charOrCode === keys.TAB;
			}
			return undefined;
		}
	});
});

},
'dijit/WidgetSet':function(){
define([
	"dojo/_base/array", // array.forEach array.map
	"dojo/_base/declare", // declare
	"dojo/_base/window", // win.global
	"./registry"	// to add functions to dijit.registry
], function(array, declare, win, registry){

	// module:
	//		dijit/WidgetSet
	// summary:
	//		Legacy registry code.   New modules should just use registry.
	//		Will be removed in 2.0.

	var WidgetSet = declare("dijit.WidgetSet", null, {
		// summary:
		//		A set of widgets indexed by id. A default instance of this class is
		//		available as `dijit.registry`
		//
		// example:
		//		Create a small list of widgets:
		//		|	var ws = new dijit.WidgetSet();
		//		|	ws.add(dijit.byId("one"));
		//		| 	ws.add(dijit.byId("two"));
		//		|	// destroy both:
		//		|	ws.forEach(function(w){ w.destroy(); });
		//
		// example:
		//		Using dijit.registry:
		//		|	dijit.registry.forEach(function(w){ /* do something */ });

		constructor: function(){
			this._hash = {};
			this.length = 0;
		},

		add: function(/*dijit._Widget*/ widget){
			// summary:
			//		Add a widget to this list. If a duplicate ID is detected, a error is thrown.
			//
			// widget: dijit._Widget
			//		Any dijit._Widget subclass.
			if(this._hash[widget.id]){
				throw new Error("Tried to register widget with id==" + widget.id + " but that id is already registered");
			}
			this._hash[widget.id] = widget;
			this.length++;
		},

		remove: function(/*String*/ id){
			// summary:
			//		Remove a widget from this WidgetSet. Does not destroy the widget; simply
			//		removes the reference.
			if(this._hash[id]){
				delete this._hash[id];
				this.length--;
			}
		},

		forEach: function(/*Function*/ func, /* Object? */thisObj){
			// summary:
			//		Call specified function for each widget in this set.
			//
			// func:
			//		A callback function to run for each item. Is passed the widget, the index
			//		in the iteration, and the full hash, similar to `array.forEach`.
			//
			// thisObj:
			//		An optional scope parameter
			//
			// example:
			//		Using the default `dijit.registry` instance:
			//		|	dijit.registry.forEach(function(widget){
			//		|		console.log(widget.declaredClass);
			//		|	});
			//
			// returns:
			//		Returns self, in order to allow for further chaining.

			thisObj = thisObj || win.global;
			var i = 0, id;
			for(id in this._hash){
				func.call(thisObj, this._hash[id], i++, this._hash);
			}
			return this;	// dijit.WidgetSet
		},

		filter: function(/*Function*/ filter, /* Object? */thisObj){
			// summary:
			//		Filter down this WidgetSet to a smaller new WidgetSet
			//		Works the same as `array.filter` and `NodeList.filter`
			//
			// filter:
			//		Callback function to test truthiness. Is passed the widget
			//		reference and the pseudo-index in the object.
			//
			// thisObj: Object?
			//		Option scope to use for the filter function.
			//
			// example:
			//		Arbitrary: select the odd widgets in this list
			//		|	dijit.registry.filter(function(w, i){
			//		|		return i % 2 == 0;
			//		|	}).forEach(function(w){ /* odd ones */ });

			thisObj = thisObj || win.global;
			var res = new WidgetSet(), i = 0, id;
			for(id in this._hash){
				var w = this._hash[id];
				if(filter.call(thisObj, w, i++, this._hash)){
					res.add(w);
				}
			}
			return res; // dijit.WidgetSet
		},

		byId: function(/*String*/ id){
			// summary:
			//		Find a widget in this list by it's id.
			// example:
			//		Test if an id is in a particular WidgetSet
			//		| var ws = new dijit.WidgetSet();
			//		| ws.add(dijit.byId("bar"));
			//		| var t = ws.byId("bar") // returns a widget
			//		| var x = ws.byId("foo"); // returns undefined

			return this._hash[id];	// dijit._Widget
		},

		byClass: function(/*String*/ cls){
			// summary:
			//		Reduce this widgetset to a new WidgetSet of a particular `declaredClass`
			//
			// cls: String
			//		The Class to scan for. Full dot-notated string.
			//
			// example:
			//		Find all `dijit.TitlePane`s in a page:
			//		|	dijit.registry.byClass("dijit.TitlePane").forEach(function(tp){ tp.close(); });

			var res = new WidgetSet(), id, widget;
			for(id in this._hash){
				widget = this._hash[id];
				if(widget.declaredClass == cls){
					res.add(widget);
				}
			 }
			 return res; // dijit.WidgetSet
		},

		toArray: function(){
			// summary:
			//		Convert this WidgetSet into a true Array
			//
			// example:
			//		Work with the widget .domNodes in a real Array
			//		|	array.map(dijit.registry.toArray(), function(w){ return w.domNode; });

			var ar = [];
			for(var id in this._hash){
				ar.push(this._hash[id]);
			}
			return ar;	// dijit._Widget[]
		},

		map: function(/* Function */func, /* Object? */thisObj){
			// summary:
			//		Create a new Array from this WidgetSet, following the same rules as `array.map`
			// example:
			//		|	var nodes = dijit.registry.map(function(w){ return w.domNode; });
			//
			// returns:
			//		A new array of the returned values.
			return array.map(this.toArray(), func, thisObj); // Array
		},

		every: function(func, thisObj){
			// summary:
			// 		A synthetic clone of `array.every` acting explicitly on this WidgetSet
			//
			// func: Function
			//		A callback function run for every widget in this list. Exits loop
			//		when the first false return is encountered.
			//
			// thisObj: Object?
			//		Optional scope parameter to use for the callback

			thisObj = thisObj || win.global;
			var x = 0, i;
			for(i in this._hash){
				if(!func.call(thisObj, this._hash[i], x++, this._hash)){
					return false; // Boolean
				}
			}
			return true; // Boolean
		},

		some: function(func, thisObj){
			// summary:
			// 		A synthetic clone of `array.some` acting explicitly on this WidgetSet
			//
			// func: Function
			//		A callback function run for every widget in this list. Exits loop
			//		when the first true return is encountered.
			//
			// thisObj: Object?
			//		Optional scope parameter to use for the callback

			thisObj = thisObj || win.global;
			var x = 0, i;
			for(i in this._hash){
				if(func.call(thisObj, this._hash[i], x++, this._hash)){
					return true; // Boolean
				}
			}
			return false; // Boolean
		}

	});

	// Add in 1.x compatibility methods to dijit.registry.
	// These functions won't show up in the API doc but since they are deprecated anyway,
	// that's probably for the best.
	array.forEach(["forEach", "filter", "byClass", "map", "every", "some"], function(func){
		registry[func] = WidgetSet.prototype[func];
	});


	return WidgetSet;
});

},
'ibm/usmi/core/utils/_UsmiBrowserPersistence':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define(
    "ibm/usmi/core/utils/_UsmiBrowserPersistence", [
        "dojo/_base/lang",
        "dojo/_base/declare",
        "dojo/cookie",
        "dojox/mvc",
        "dijit/registry",
        "ibm/usmi/core/utils/Logger"
    ],
    function (lang, declare, cookie, mvc, registry, Logger) {
        "use strict";
        return declare(
            null,
            {
                // the logger
                _usmiBrowserPersistenceLogger: new Logger("ibm.usmi.core.utils", "_UsmiBrowserPersistence"),

                constructor: function () {
                    if (this._usmiBrowserPersistenceLogger.isFinest()) {
                        this._usmiBrowserPersistenceLogger.entering("", "constructor");
                    }
                    if (this._usmiBrowserPersistenceLogger.isFinest()) {
                        this._usmiBrowserPersistenceLogger.exiting("", "constructor");
                    }
                },

                //set up watchers and persistence for the model passed to us
                initPersistence: function (model) {
                    if (this._usmiBrowserPersistenceLogger.isFinest()) {
                        this._usmiBrowserPersistenceLogger.entering("", "initPersistence");
                    }
                    var modelObj = model.toPlainObject(),
                        property = null,
                        ss = null;
                    for (property in modelObj) {
                        if (modelObj.hasOwnProperty(property)) {
                            // look for saved value for model property
                            ss = this.getData(property);
                            if (ss !== null) {
                                if (this._usmiBrowserPersistenceLogger.isFiner()) {
                                    this._usmiBrowserPersistenceLogger.finer("STORAGE:  Found '" + property + "' in session storage.  Value is " + ss,
                                        "initPersistence");
                                }
                                model[property].set("value", ss);
                            }

                            // set up watcher for value changes
                            //javascripty & clean
                            model[property].watch("value", this.propertyChangeListener(property));

                            //dojo'y & dirty
                            /*model[property].watch("value",
                                lang.hitch({property: property,
                                        setData: this.setData,
                                        validateUser: this.validateUser,
                                        getSessionIdFromLocalStorage: this.getSessionIdFromLocalStorage,
                                        getSessionIdFromSessionStorage: this.getSessionIdFromSessionStorage },
                                            function (name, oldValue, newValue) {
                                                console.log(this.property + " was updated from: " + oldValue + " to: " + newValue);
                                                this.setData(this.property, newValue);
                        }));*/
                        }
                    }
                    if (this._usmiBrowserPersistenceLogger.isFinest()) {
                        this._usmiBrowserPersistenceLogger.entering("", "initPersistence");
                    }
                },

                propertyChangeListener: function (property) {
                    var myObject = this; //for scope on the callback
                    return function (name, oldValue, newValue) {
                        var newType = Object.prototype.toString.call(newValue);
                        if (myObject._usmiBrowserPersistenceLogger.isFiner()) {
                            myObject._usmiBrowserPersistenceLogger.finer("NOTIFICATION:  '" + property + "' was updated from: " + oldValue + " to: " + newValue,
                                "propertyChangeListener");
                        }
                        myObject.setData(property, newValue);
                        if (myObject._usmiBrowserPersistenceLogger.isFiner()) {
                            myObject._usmiBrowserPersistenceLogger.finer("STORAGE:  Updated session storage for '" + property + "':"  + myObject.getData(property),
                                "propertyChangeListener");
                        }
                    };
                },

                radioButtonChangeListener: function (modelEntry, domValue) {
                    return function (isChecked) {
                        if (isChecked) {
                            modelEntry.set("value", domValue);
                        }
                    };
                },

                checkboxChangeListener: function (modelEntry, domValue) {
                    return function (isChecked) {
                        modelEntry.set("value", isChecked ? domValue : "");
                    };
                },

                getSessionIdFromLocalStorage: function () {
                    return localStorage.getItem("usmiLocalStorageKey");
                },

                getSessionIdFromSessionStorage: function () {
                    return sessionStorage.getItem("usmiSessionStorageKey");
                },

                validateUser: function () {
                    var sessionIdFromLocalStorage = this.getSessionIdFromLocalStorage(),
                        sessionIdFromSessionStorage = this.getSessionIdFromSessionStorage();

                    if ((!sessionIdFromLocalStorage) ||
                            (!sessionIdFromSessionStorage) ||
                            (sessionIdFromLocalStorage !== sessionIdFromSessionStorage)) {
                        // Local storage session id is different.  The most probable
                        // cause of this is due to a logout in another window or tab.
                        // The data stored in the session is now invalid.
                        if (this._usmiBrowserPersistenceLogger.isInfo()) {
                            this._usmiBrowserPersistenceLogger.info("STORAGE:  Session storage session id DOES NOT MATCH local storage session id.",
                                "validateUser");
                            this._usmiBrowserPersistenceLogger.info("STORAGE:  Clearing session storage.",
                                "validateUser");
                        }

                        // Delete old saved data and save new session id.
                        sessionStorage.clear();
                        sessionStorage.setItem("usmiSessionStorageKey", sessionIdFromLocalStorage);
                    }
                    return true;
                },

                getFromStorage: function (storage, key, storedAsJSON) {
                    var valueFromStorage = null,
                        valueToReturn = null;
                    if (this.validateUser()) {
                        valueFromStorage = storage.getItem(key);
                        if (valueFromStorage !== null) {
                            if (storedAsJSON === false) {
                                valueToReturn = valueFromStorage;
                            } else {
                                return JSON.parse(valueFromStorage).valueData;
                            }
                        }
                    }
                    return valueToReturn;
                },

                writeToStorage: function (storage, key, value, storeAsJSON) {
                    var valueToStore = null,
                        valueType = null;
                    if (this.validateUser()) {
                        if (storeAsJSON === false) {
                            valueToStore = value;
                        } else {
                            valueType = Object.prototype.toString.call(value);
                            valueToStore = JSON.stringify({valueType: valueType, valueData: value});
                        }
                        storage.setItem(key, valueToStore);
                    }
                },

                removeFromStorage: function (storage, key) {
                    if (this.validateUser()) {
                        storage.removeItem(key);
                    }
                },

                // storedAsJSON is an optional parameter.  The default is true.
                getData: function (key, storedAsJSON) {
                    return this.getFromStorage(sessionStorage, key, storedAsJSON);
                },

                // storeAsJSON is an optional parameter.  The default is true.
                setData: function (key, value, storeAsJSON) {
                    this.writeToStorage(sessionStorage, key, value, storeAsJSON);
                },

                removeData: function (key) {
                    this.removeFromStorage(sessionStorage, key);
                },

                // storedAsJSON is an optional parameter.  The default is true.
                getGlobalData: function (key, storedAsJSON) {
                    return this.getFromStorage(localStorage, key, storedAsJSON);
                },

                // storedAsJSON is an optional parameter.  The default is true.
                setGlobalData: function (key, value, storeAsJSON) {
                    this.writeToStorage(localStorage, key, value, storeAsJSON);
                },

                removeGlobalData: function (key) {
                    this.removeFromStorage(localStorage, key);
                },

                initializeRadioButtons: function (ids, modelEntry) {
                    var i,
                        domValue;
                    for (i = 0; i < ids.length; i += 1) {
                        domValue = registry.byId(ids[i]).value;
                        if (modelEntry.get("value") === domValue) {
                            registry.byId(ids[i]).set("checked", "true");
                        } else {
                            registry.byId(ids[i]).set("checked", "");
                        }
                        registry.byId(ids[i]).on("change", this.radioButtonChangeListener(modelEntry, domValue), true);
                    }
                },

                initializeCheckbox: function (id, modelEntry) {
                    var domValue = registry.byId(id).value;
                    if (modelEntry.get("value") === domValue) {
                        registry.byId(id).set("checked", "true");
                    } else {
                        registry.byId(id).set("checked", "");
                    }

                    registry.byId(id).on("change", this.checkboxChangeListener(modelEntry, domValue), true);
                }
            }
        );
    }
);

},
'gridx/core/model/extensions/Sort':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/json",
	'../_Extension'
], function(declare, lang, json, _Extension){

	return declare(/*===== "gridx.core.model.extensions.Sort", =====*/_Extension, {
		name: 'sort',

		priority: 30,

		constructor: function(model, args){
			var t = this, bs = args.baseSort;
			t.clear();
			t._mixinAPI('sort');
			if(bs && bs.length){
				t._baseSort = bs;
				t._sort();
			}
		},

		//Public--------------------------------------------------------------
		clear: function(){},

		sort: function(/* sortSpec */){
			this.model._addCmd({
				name: '_cmdSort',
				scope: this,
				args: arguments
			});
		},

		//Private--------------------------------------------------------------
		_cmdSort: function(){
			var a = arguments;
			this._sort.apply(this, a[a.length - 1]);
		},

		_sort: function(sortSpec){
			var t = this, m = t.model, bs = t._baseSort, c = m._cache,
				op = c.options = c.options || {}, i, s, toSort;
			if(lang.isArrayLike(sortSpec)){
				for(i = 0; i < sortSpec.length; ++i){
					s = sortSpec[i];
					if(s.colId){
						s.attribute = c.columns ? (c.columns[s.colId].field || s.colId) : s.colId;
					}else{
						s.colId = s.attribute;
					}
				}
				if(bs){
					sortSpec = sortSpec.concat(bs);
				}
			}else{
				sortSpec = bs;
			}
			if(op.sort && op.sort.length){
				if(json.toJson(op.sort) !== json.toJson(sortSpec)){
					toSort = 1;	//1 as true
				}
			}else if(sortSpec && sortSpec.length){
				toSort = 1;
			}
			op.sort = lang.clone(sortSpec);
			if(toSort){
				c.clear();
			}
			m._msg('storeChange');
		}
	});
});

},
'ibm/tivoli/tip/navwidget/dijit/AsynchContextMenuProvider':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!ibm/tivoli/tip/dijit/TIPContextMenuProvider"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * "Restricted Materials of IBM"
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.dijit.AsynchContextMenuProvider");

dojo.require("ibm.tivoli.tip.dijit.TIPContextMenuProvider");

dojo.declare("ibm.tivoli.tip.navwidget.dijit.AsynchContextMenuProvider",
        ibm.tivoli.tip.dijit.TIPContextMenuProvider,
{		
	// summary:
	//		Coming soon
	// description:
	//		Coming soon
	
	getMenuItemsAsynch: function(context, itemsLoadedCallback, /*String?*/ category) { 
		// summary:
		//		Callback that should be overloaded to make the asynch request for the menu
		//		items and that calls the callback when the asynch request is complete.
		//		The optional category parameter can override the default category of "menu"
		//		for the asynch request.
	
		/*required overload*/ 
	}
	
});


});

},
'ibm/tivoli/tip/navwidget/dijit/CustomContentTooltip':function(){
// wrapped by build app
define("ibm/tivoli/tip/navwidget/dijit/CustomContentTooltip", ["dijit","dojo","dojox","dojo/require!dijit/Tooltip"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.dijit.CustomContentTooltip");

dojo.require("dijit.Tooltip");

dojo.declare("ibm.tivoli.tip.navwidget.dijit.CustomContentTooltip",dijit.Tooltip,
{
	// summary:
	//		A tooltip that allows specification of a callback to provide custom
	//      content for the tooltip at the time the tooltip is shown.
	//      This widget extends dijit.Tooltip.
	// description:
	//		Coming soon
	
	// getCustomContentCallback: function(target)
	//		Callback function that is called just before the tooltip is displayed
	//      to supply custom content.  If provided, this callback should return
	//      either 
	//          - a string of valid HTML text
	//          - a DOM node or page fragment
	//          - a dijit._Widget
	getCustomContentCallback: null,
	
	// position: String[]
	//		defaultPosition: ["above"],
	position: ["below","above"],
	
	// _openInProcess: Boolean
	//		
	_openInProcess: false,
	
	// _pendingNewContent:
	//		
	_pendingNewContent: null,
	
	constructor: function(){		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
	
		// Placeholder in case we need something here later
	},

	isOpening: function() {
		// summary:
		//		Answers true while tooltip is in the process of being displayed
		//		and false once the tooltip is visible
		// description:
		//		Coming soon
		return this._openInProcess;	
	},

	isVisible: function() {
		// summary:
		//		Answers true if the tooltip is currently displayed
		// description:
		//		Coming soon
		return (!this._connectNode) ? false : true;
	},
	
	// Tooltip is now visible, so open process is done
	onShow: function(/*DomNode*/ target, position){
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// target:
		//		
		// position:
		//		
		this._openInProcess = false;
		if (this._pendingNewContent) {
			this._putContentInLabel(this._pendingNewContent);
			this._pendingNewContent = null;
			this._replaceDisplayedTootipContents();
		}
		this.inherited(arguments);
	},
	
	// Override open(target) to allow the callback to supply custom content
	open: function(/*DomNode*/ target){		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		this._openInProcess = true;
		if (this.getCustomContentCallback && (typeof this.getCustomContentCallback == "function")) {
			var content = this.getCustomContentCallback(target);
			this._putContentInLabel(content);
		}
		// Now that the content is set, let parent display the tooltip
		this.inherited(arguments);
	},

	updateContent: function(/*DomNode|widget|String*/ content, target) {
		// summary:
		//		Use this function to update the content of this tooltip when it is displayed
		//		or about to be displayed.
		// description:
		//		Coming soon
		
		// Is the tooltip currently showing or waiting on the timer to show?
		if ((this.isVisible() || this._showTimer) && (this.isOpening() == false)) {
			this._putContentInLabel(content);
			this.unHover(target);	
			this.hover(target);
		} else if (this.isOpening()) {
			this._pendingNewContent = content;
		}
	},
	
	_putContentInLabel: function(/*DomNode|widget|String*/ content) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		if (content) {
			if (typeof content == "string") { // string
				this.label = content;
			} else if (content.domNode) { // widget
				this.label = content.domNode.innerHTML;
			} else if (content.innerHTML) { // domNode, page fragment
				this.label = content.innerHTML;
			} else { // not sure, but convert to string to be safe
				this.label = content.toString();
			}
		}
	},
	
	_replaceDisplayedTootipContents: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		dijit._masterTT.containerNode.innerHTML = this.label;
		// Firefox bug. when innerHTML changes to be shorter than previous
		// one, the node size will not be updated until it moves.
		dijit._masterTT.domNode.style.top = (dijit._masterTT.domNode.offsetTop + 1) + "px";		
	}
	
});
});

},
'ibm/tivoli/navmodel/Column':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/Utils"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Column"); 

// includes
dojo.require("ibm.tivoli.navmodel.Utils");

////////////////////////////////////
// Column
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Column", null, {
	// private members
	_col: null,
	_objectType: null,
	_dataset: null,
	_condition: null,
	
	/*=====
	constructor_kwArgs: function(column, objectType, dataset) {
		// column: Object
		//		json column object
		// objectType: String
		//		object type: "item", "link", or "relationship"
		// dataset: Object
		//		ibm.tivoli.navmodel.Dataset that owns this column
	},
	=====*/

	constructor: function(/*constructor_kwArgs*/ kwArgs) {		
		// summary:
		//		Constructs Column object.		
		this._col = kwArgs.column;
		if (!this._col) {
			this._col = {};
		}
		
		this._objectType = kwArgs.objectType;
		this._dataset = kwArgs.dataset;
	},

	getAttribute: function(key) {
		// summary:
		//		Returns general attribute for requested key
		return this._col[key];
	},

	setAttribute: function(key, value) {
		// summary:
		//		Set general attribute for requested key
		this._col[key] = value;
	},
	
	getAttributes: function() {		
		// summary:
		//		Get all general attributes for the column
		return dojo.clone(this._col);			
	},

	getDataset: function() {
		// summary:
		//		Returns the dataset that owns the column
		return this._dataset;
	},
	
	getId: function() {
		// summary:
		//		Returns column id
		return this._col.id;
	},

	setId: function(id) {
		// summary:
		//		Set column id
		this._col.id = id;
	},

	getLabel: function() {
		// summary:
		//		Returns globalized column header label
		if (this._col.label) {
			// string
			if (typeof this._col.label === "string") {
				return this._col.label;
			}
			// object with key/value info
			else {
				return this._col.label.value;
			}
		}
		
		return null;		
	},

	setLabel: function(label) {
		// summary:
		//		Set globalized column header label
		this._col.label = label;		
	},	
	
	getDescription: function() {
		// summary:
		//		Returns globalized description
		if (this._col.description) {
			// string
			if (typeof this._col.description === "string") {
				return this._col.description;
			}
			// object with key/value info
			else {
				return this._col.description.value;
			}
		}
		
		return null;
	},

	setDescription: function(description) {
		// summary:
		//		Set globalized description
		this._col.description = description;		
	},	
	
	getValueType: function() {
		// summary:
		//		Returns the type associated with the column
		//		
		//		|	values:	"itemName", "itemType", "linkName", "linkType", "relationshipName", "relationshipType",
		//		|			string", "string[]",
		//		|			"byte", "short", "int", "long", "byte[]", "short[]", "int[]", "long[]",
		//		|			"ubyte, "ushort", "uint", "ulong", "ubyte[]", "ushort[]", "uint[]", "ulong[]",
		//		|			"boolean", "boolean[]"
		//		|			"enum",
		//		|			"status"
		//		|			"date", "datetime", "time", "date[]", "datetime[]", "time[]"
		//		|			"isodate", isodatetime", "isotime", "isodate[]", "isodatetime[], "isotime[]", "isointerval", "isoduration"
		//		|			"float", "double", "percentage", "float[]", "double[]"
		return this._col.valueType;
	},

	setValueType: function(valueType) {
		// summary:
		//		Set the type associated with the column	
		this._col.valueType = valueType;
	},
	
	getWidth: function() {
		// summary:
		//		Returns column width string as: pixels, percentage, or em value
		return this._col.width;
	},

	setWidth: function(width) {
		// summary:
		//		Set the column width string as: pixels, percentage, or em value
		// width: String
		//		Column width value
		this._col.width = width;
	},
	
	isResizeable: function() {
		// summary:
		//		Returns true if column is resizeable by the user
		return (this._col.resizeable !== undefined ? this._col.resizeable : true);
	},
	
	setResizeable: function(resizeable) {
		// summary:
		//		Set if the column is to be resizeable by the user
		this._col.resizeable = resizeable;
	},
	
	isLineWrap: function() {
		// summary:
		//		Returns true if column allows line wrap to multiple lines
		return (this._col.lineWrap !== undefined ? this._col.lineWrap : false);
	},
	
	setLineWrap: function(lineWrap) {
		// summary:
		//		Set if the column allows line wrap to multiple lines
		this._col.lineWrap = lineWrap;
	},
	
	isLocked: function() {
		// summary:
		//		Returns true if column locked in non-scrollable region
		return (this._col.locked !== undefined ? this._col.locked : false);
	},
	
	setLocked: function(Locked) {
		// summary:
		//		Set whether the column is locked in non-scrollable region
		this._col.locked = locked;
	},
		
	isHidden: function() {
		// summary:
		//		Returns true if the column is to be hidden from the user
		return (this._col.hidden !== undefined ? this._col.hidden : false);
	},
	
	setHidden: function(hidden) {
		// summary:
		//		Set if the column is to be hidden from the user
		this._col.hidden = hidden;
	},
	
	getImage: function() {
		// summary:
		//		Returns optional image for the column
		return this._col.image;
	},
	
	setImage: function(image) {
		// summary:
		//		Set optional image for the column
		this._col.image = image;
	},
	
	getSortOrder: function() {
		// summary:
		//		Returns sort order priority (value is 0 based and should be -1 if not sorted)
		return (this._col.sortOrder !== undefined ? this._col.sortOrder : -1);
	},

	setSortOrder: function(sortOrder) {
		// summary:
		//		Set sort order priority (value is 0 based and should be -1 if not sorted)
		this._col.sortOrder = sortOrder;
	},

	isSortable: function() {
		// summary:
		//		Returns true if the column is sortable
		return (this._col.sortable !== undefined ? this._col.sortable : true);
	},

	setSortable: function(sortable) {
		// summary:
		//		Set if the column is sortable
		this._col.sortable = sortable;
	},

	isSortAscending: function() {
		// summary:
		//		Returns true if the sort is ascending or descending
		return (this._col.sortAscending !== undefined ? this._col.sortAscending : true);
	},

	setSortAscending: function(ascending) {
		// summary:
		//		Set if the sort is ascending or descending
		this._col.sortAscending = ascending;
	},

	isSearchable: function() {
		// summary:
		//		Returns true if the column is searchable
		return (this._col.searchable !== undefined ? this._col.searchable : true);
	},

	setSearchable: function(searchable) {
		// summary:
		//		Set if the column is searchable
		this._col.searchable = searchable;
	},

	isFilterable: function() {
		// summary:
		//		Returns true if the column is filterable
		return (this._col.filterable !== undefined ? this._col.filterable : true);
	},
	
	setFilterable: function(filterable) {
		// summary:
		//		Set if the column is filterable
		this._col.filterable = filterable;
	},
	
	isFiltered: function() {
		// summary:
		//		Returns true if the column is currently filtered
		return (this._col.filtered !== undefined ? this._col.filtered : false);
	},

	setFiltered: function(filtered) {
		// summary:
		//		Set if the column is currently filtered
		this._col.filtered = filtered;
	},
	
	getFilter: function() {
		// summary:
		//		Returns filter object for the column		
		return this._col.filter;
	},
	
	getUnsupportedFilterOperators: function() {
		// summary:
		//		Returns String[] of operators that are not supported for this
		//		column or null if all operators are supported.
		return (this._col.filter ? this._col.filter.unsupportedOperators : null);		
	},
	
	hasPossibleValues: function() {
		// summary:
		//		Returns true if the column supports a list of possible values		
		var filter = this._col.filter;
		return ((filter && (filter.possibleValues || filter.possibleValuesUri)) ? true : false);
	},
	
	hasInlinedPossibleValues: function() {
		// summary:
		//		Returns true if the column supports possible values and
		//		all of those values have been inlined in the json
		var filter = this._col.filter;
		return (filter && filter.possibleValues ? true : false);
	},
	
	hasUrlAddressablePossibleValues: function() {
		// summary:
		//		Returns true if the column supports possible values and
		//		the values are addressable via a url
		var filter = this._col.filter;
		return (filter && filter.possibleValuesUri ? true : false);
	},
	
	getPossibleValues: function(kwArgs) {
		// summary:
		//		Retrieves a list of possible values for columns with a valueType of
		//		enum, map, itemType, linkType, or relationshipType
		var args = {
			onComplete: function(values, totalCount, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			start: 0,
			count: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
	
		var filter = this._col.filter;
		
		// possible values were inlined; just return them
		if (filter && filter.possibleValues) {
			if (args.start === 0 && args.count === null) {
				args.onComplete(filter.possibleValues, filter.possibleValues.length, args);
			}
			else {
				var values = [];
				var length = filter.possibleValues.length;
				var count = args.count ? args.count : length;
				for (var index = args.start;  index < length && count > 0;  ++index, --count) {
					values.push(filter.possibleValues[index]);
				}
				args.onComplete(values, length, args);
			}
		}
		// possible values are available via uri
		else if (filter && filter.possibleValuesUri) {
			var content = ibm.tivoli.navmodel.Utils.getContent();
			ibm.tivoli.navmodel.Utils.mixinParams(content, this._dataset);
			if (args.start) {
				content.start = args.start;
			}
			if (args.count) {
				content.count = args.count;
			}
			if (this._objectType) {
				content.type = this._objectType;
			}
			
			dojo.xhrGet({
				url: this._dataset.getArg("baseUrl") + filter.possibleValuesUri,
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "columnValueList")
				},
				handleAs: "json",
				sync: args.sync,
				content: content,		
				load: dojo.hitch(this, function(result, ioArgs){
					args.onComplete(result.items, result.totalRows, args);					
				}),
				error: dojo.hitch(this, function(error, ioArgs){
					args.onError(error, ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "getPossibleValues", this._trace, this._version()), ioArgs);
				})
			});	
		}
		// doesn't have any
		else {
			args.onComplete(null, 0, args);
		}		
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////

	_version: function() {		
		// summary:
		//		Returns rest version
		return this._dataset.getVersion();
	}
});

});

},
'dojox/collections/Dictionary':function(){
define(["dojo/_base/kernel", "dojo/_base/array", "./_base"], function(dojo, darray, dxc){
/*=====
var dxc = dojox.collections;
=====*/
	dxc.Dictionary=function(/* dojox.collections.Dictionary? */dictionary){
		//	summary
		//	Returns an object of type dojox.collections.Dictionary
		var items={};
		this.count=0;

		//	comparator for property addition and access.
		var testObject={};

		this.add=function(/* string */k, /* object */v){
			//	summary
			//	Add a new item to the Dictionary.
			var b=(k in items);
			items[k]=new dxc.DictionaryEntry(k,v);
			if(!b){
				this.count++;
			}
		};
		this.clear=function(){
			//	summary
			//	Clears the internal dictionary.
			items={};
			this.count=0;
		};
		this.clone=function(){
			//	summary
			//	Returns a new instance of dojox.collections.Dictionary; note the the dictionary is a clone but items might not be.
			return new dxc.Dictionary(this);	//	dojox.collections.Dictionary
		};
		this.contains=this.containsKey=function(/* string */k){
			//	summary
			//	Check to see if the dictionary has an entry at key "k".
			if(testObject[k]){
				return false;			// bool
			}
			return (items[k]!=null);	//	bool
		};
		this.containsValue=function(/* object */v){
			//	summary
			//	Check to see if the dictionary has an entry with value "v".
			var e=this.getIterator();
			while(e.get()){
				if(e.element.value==v){
					return true;	//	bool
				}
			}
			return false;	//	bool
		};
		this.entry=function(/* string */k){
			//	summary
			//	Accessor method; similar to dojox.collections.Dictionary.item but returns the actual Entry object.
			return items[k];	//	dojox.collections.DictionaryEntry
		};
		this.forEach=function(/* function */ fn, /* object? */ scope){
			//	summary
			//	functional iterator, following the mozilla spec.
			var a=[];	//	Create an indexing array
			for(var p in items) {
				if(!testObject[p]){
					a.push(items[p]);	//	fill it up
				}
			}
			dojo.forEach(a, fn, scope);
		};
		this.getKeyList=function(){
			//	summary
			//	Returns an array of the keys in the dictionary.
			return (this.getIterator()).map(function(entry){
				return entry.key;
			});	//	array
		};
		this.getValueList=function(){
			//	summary
			//	Returns an array of the values in the dictionary.
			return (this.getIterator()).map(function(entry){
				return entry.value;
			});	//	array
		};
		this.item=function(/* string */k){
			//	summary
			//	Accessor method.
			if(k in items){
				return items[k].valueOf();	//	object
			}
			return undefined;	//	object
		};
		this.getIterator=function(){
			//	summary
			//	Gets a dojox.collections.DictionaryIterator for iteration purposes.
			return new dxc.DictionaryIterator(items);	//	dojox.collections.DictionaryIterator
		};
		this.remove=function(/* string */k){
			//	summary
			//	Removes the item at k from the internal collection.
			if(k in items && !testObject[k]){
				delete items[k];
				this.count--;
				return true;	//	bool
			}
			return false;	//	bool
		};

		if (dictionary){
			var e=dictionary.getIterator();
			while(e.get()) {
				 this.add(e.element.key, e.element.value);
			}
		}
	};
	return dxc.Dictionary;
});

},
'url:ibm/usmi/scheduler/wizard_pages/templates/OptionsPage.html':"<div>\r\n<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n<!-- \r\n\t<div class=\"tiv-page-header\">\r\n\t\t<h2>Options</h2>\r\n\t\t<p>Additional options</p>\r\n\t</div>\r\n\t-->\r\n\t<label for=\"SystemTimes\">${persistResources.SYSTEM_TIMES_LABEL}</label>\r\n\t<div id=\"SystemTimes\">\r\n\t\t<input type=\"radio\" name=\"timeToUse\" data-dojo-type=\"dijit.form.RadioButton\" dojoAttachPoint=\"useMgmtServerTime\" checked value=\"true\" />${persistResources.MANAGEMENT_SERVER_TIME_RADIO}<br /> \r\n\t\t<input type=\"radio\" name=\"timeToUse\" data-dojo-type=\"dijit.form.RadioButton\" id=\"useLocalSystemTime\" value=\"localSystem\" />${persistResources.LOCAL_SYSTEM_TIME_RADIO}<br />\r\n\t</div>\r\n\t<label for=\"UnavailableSystems\">${persistResources.UNAVAILABLE_SYSTEMS_LABEL}</label>\r\n\t<div id=\"UnavailableSystems\">\r\n\t\t<input type=\"radio\" name=\"unavailableSystems\" data-dojo-type=\"dijit.form.RadioButton\" dojoAttachPoint=\"failSystems\"\tchecked value=\"true\" />${persistResources.FAIL_IF_UNAVAILABLE_RADIO}<br /> \r\n\t\t<input type=\"radio\" name=\"unavailableSystems\" data-dojo-type=\"dijit.form.RadioButton\" id=\"runSystems\" value=\"run\" />${persistResources.RUN_WHEN_AVAILABLE_RADIO}<br />\r\n\t</div>\r\n\t<label for=\"MaximumJobInstances\">${persistResources.MAXIMUM_JOBS_LABEL}</label>\r\n\t<div id=\"MaximumJobInstances\">\r\n\t\t<input type=\"text\" dojoAttachPoint=\"maxiumJobInstances\" dojotype=\"dijit.form.NumberSpinner\" data-dojo-props=\"constraints:{min:10,max:100,places:0}\" value=\"100\" \r\n\t\t    dojoAttachEvent=\"onChange:doSubmitButtonCheck\" intermediateChanges=\"true\" /><br />\r\n\t</div>\r\n</div>\r\n",
'url:ibm/usmi/statuspod/templates/jobStatusTip.html':"<div data-dojo-attach-point=\"tipContent\" class=\"statusPodTipTable\">\r\n<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n\r\n\r\n\t<table data-dojo-attach-point=\"jobStatusTipTable\" class=\"statusPodToolTipTable\">\r\n\r\n        <tr>\r\n\t\t\t<td>\r\n\t\t\t\t<span class=\"statusPodToolTipCell\">${JobNameLabel}:</span>\r\n\t\t\t</td>\r\n\t\t\t<td colspan='3'  style=\"white-space: normal; word-break:break-all; word-wrap:break-word;\">\r\n\t\t\t\t<span class=\"statusPodToolTipCell\">${jobName}</span>\r\n\t\t\t</td>\r\n\t\t\t\r\n\t\t</tr>\r\n\t\t\r\n\r\n\t\t\r\n\t\t<tr>\r\n\t\t\t<td>\r\n\t\t\t\t<span class=\"statusPodToolTipCell\">${StatusLabel}:</span>\r\n\t\t\t</td>\r\n\t\t\t<td>\r\n\t\t\t\t<span class=\"statusPodToolTipCell\">${status}</span>\r\n\t\t\t</td>\r\n\t\t\t<td>\r\n\t\t\t\t<span>${ProgressLabel}:</span>\r\n\t\t\t</td>\r\n\t\t\t<td>\r\n\t\t\t\t<div data-dojo-type=\"dijit.ProgressBar\" data-dojo-attach-point=\"jobProgressBar\" style=\"width:80px\" data-dojo-props=\"maximum:1\"></div>\r\n\t\t\t</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<td>\r\n\t\t\t\t<span class=\"statusPodToolTipCell\">${LastRunStatusLabel}:</span>\r\n\t\t\t</td>\r\n\t\t\t<td>\r\n\t\t\t\t<span class=\"statusPodToolTipCell\">${lastRunStatus}</span>\r\n\t\t\t</td>\r\n\t\t\t<td>\r\n\t\t\t\t<span class=\"statusPodToolTipCell\">${LastRunLabel}:</span>\r\n\t\t\t</td>\r\n\t\t\t<td>\r\n\t\t\t\t<span class=\"statusPodToolTipCell\">${lastRun}</span>\r\n\t\t\t</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t\t\t\r\n\t\t\t<td>\r\n\t\t\t\t<span class=\"statusPodToolTipCell\">${CreatedByLabel}:</span>\r\n\t\t\t</td>\r\n\t\t\t<td>\r\n\t\t\t\t<span class=\"statusPodToolTipCell\">${createdBy}</span>\r\n\t\t\t</td>\r\n\t\t\t<td>\r\n\t\t\t\t<span class=\"statusPodToolTipCell\">${NextRunLabel}:</span>\r\n\t\t\t</td>\r\n\t\t\t<td>\r\n\t\t\t\t<span class=\"statusPodToolTipCell\">${nextRun}</span>\r\n\t\t\t</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<td>\r\n\t\t\t\t<span class=\"statusPodToolTipCell\">${TaskLabel}:</span>\r\n\t\t\t</td>\r\n\t\t\t<td colspan='3'  style=\"white-space: normal; word-break:break-all; word-wrap:break-word;\">\r\n\t\t\t\t<span class=\"statusPodToolTipCell\">${task}</span>\r\n\t\t\t</td>\r\n\t\t</tr>\r\n\t\t\r\n\t\t<tr>\r\n\t\t\t<td>\r\n\t\t\t\t<span class=\"statusPodToolTipCell\">${DescriptionLabel}:</span>\r\n\t\t\t</td>\r\n\t\t\t<td colspan=\"3\" style=\"white-space: normal; word-break:break-all; word-wrap:break-word;\">\r\n\t\t\t\t<span class=\"statusPodToolTipCell\">${description}</span>\r\n\t\t\t</td>\r\n\t\t\t\r\n\t\t</tr>\r\n\t</table>\r\n</div>",
'ibm/tivoli/tip/navwidget/table/dijit/TableView':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/i18n!ibm/tivoli/tip/navwidget/nls/resources","dojo/require!dijit/layout/BorderContainer,dojo/data/ItemFileReadStore,ibm/tivoli/tip/dojo/Global,ibm/tivoli/navmodel/Trace,ibm/tivoli/tip/navwidget/dijit/RESTContextMenuProvider,ibm/tivoli/tip/navwidget/topology/dijit/MenuProviderHelper,ibm/tivoli/tip/navwidget/dijit/NavWidgetPluggableView,ibm/tivoli/tip/navwidget/dijit/NavWidgetTabContainerPluggableView,dijit/dijit,dijit/layout/TabContainer,dijit/layout/ContentPane,ibm/tivoli/tip/dijit/BusyPane"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide ("ibm.tivoli.tip.navwidget.table.dijit.TableView");

dojo.require("dijit.layout.BorderContainer");
// dojo.require("dojox.grid.DataGrid");   // 49426  Gridx
dojo.require("dojo.data.ItemFileReadStore");
dojo.require("ibm.tivoli.tip.dojo.Global");
dojo.require("ibm.tivoli.navmodel.Trace");
dojo.require("ibm.tivoli.tip.navwidget.dijit.RESTContextMenuProvider");
dojo.require("ibm.tivoli.tip.navwidget.topology.dijit.MenuProviderHelper");
dojo.require("ibm.tivoli.tip.navwidget.dijit.NavWidgetPluggableView");
dojo.require("ibm.tivoli.tip.navwidget.dijit.NavWidgetTabContainerPluggableView");

dojo.require("dijit.dijit");
dojo.require("dijit.layout.TabContainer");
dojo.require("dijit.layout.ContentPane");
dojo.require("ibm.tivoli.tip.dijit.BusyPane");
dojo.requireLocalization("ibm.tivoli.tip.navwidget", "resources");

ibm.tivoli.tip.navwidget.table.dijit.TableView._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.tip.navwidget.table.dijit.TableView");

dojo.declare ("ibm.tivoli.tip.navwidget.table.dijit.TableView",
	[ibm.tivoli.tip.navwidget.dijit.NavWidgetTabContainerPluggableView],
	{		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
	
		// cmProvider: Object
		//		Coming soon
		cmProvider:null, 
		
		//*************************** 
		// "Private"
		//***************************
		
		// _trace: [private] Trace
		//		Utilitiy for trace and logging
		_trace: ibm.tivoli.tip.navwidget.table.dijit.TableView._trace,
		
		// _DEFAULT_WIDGET_DEF_INFO: [private] Object
		//		Coming soon
		_DEFAULT_WIDGET_DEF_INFO: {
			className: "ibm.tivoli.navmodel.WidgetDef", //NOTE: Think there is supposed to eventually be a TableViewDef
			baseUrl: "/ibm/tivoli/rest",
	    	provider: "any",
	    	datasource: "any",
	    	widget: "tableview",
	    	definition: "DefaultTableView"
		},

		constructor: function (container, div, cmProvider) {		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			// container:
			//		
			// div: 
			//		
			// cmProvider:	
			//		
	
			// Use the supplied context menu provider if passed in
		        if (cmProvider) {
		        	this.cmProvider = cmProvider;
		        }
		        else {
			        // create a default context menu provider
		            var helper = new ibm.tivoli.tip.navwidget.topology.dijit.MenuProviderHelper();
			        this.cmProvider = new ibm.tivoli.tip.navwidget.dijit.RESTContextMenuProvider(
			        			{
			        				view: this,
			        				getMenuItemsCallback: dojo.hitch(helper, helper.getMenuItems),
			        				getMenuCommandCallback: dojo.hitch(helper, helper.getMenuCommand)
			        			}
			        		);	        	
		        }

		},

		
		postCreate: function(){		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			
			//Call superclass
			this.inherited(arguments);
		},
		
		startup: function(){		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			
			this.inherited(arguments);
		},
		
		destroy: function(){		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			
			//TODO
            this.inherited(arguments);
		},
		
		_getDefaultWidgetDefInfo: function(){
			return this._DEFAULT_WIDGET_DEF_INFO;
		},
		
		isApplicable: function(/*ibm.tivoli.navmodel.Dataset*/dataset){
			return this.inherited(arguments);
		}
});

});

},
'dojo/dnd/Mover':function(){
define(["../main", "../Evented", "../touch", "./common", "./autoscroll"], function(dojo, Evented, touch) {
	// module:
	//		dojo/dnd/Mover
	// summary:
	//		TODOC


dojo.declare("dojo.dnd.Mover", [Evented], {
	constructor: function(node, e, host){
		// summary:
		//		an object which makes a node follow the mouse, or touch-drag on touch devices.
		//		Used as a default mover, and as a base class for custom movers.
		// node: Node
		//		a node (or node's id) to be moved
		// e: Event
		//		a mouse event, which started the move;
		//		only pageX and pageY properties are used
		// host: Object?
		//		object which implements the functionality of the move,
		//	 	and defines proper events (onMoveStart and onMoveStop)
		this.node = dojo.byId(node);
		this.marginBox = {l: e.pageX, t: e.pageY};
		this.mouseButton = e.button;
		var h = (this.host = host), d = node.ownerDocument;
		this.events = [
			// At the start of a drag, onFirstMove is called, and then the following two
			// connects are disconnected
			dojo.connect(d, touch.move, this, "onFirstMove"),

			// These are called continually during the drag
			dojo.connect(d, touch.move, this, "onMouseMove"),

			// And these are called at the end of the drag
			dojo.connect(d, touch.release,   this, "onMouseUp"),

			// cancel text selection and text dragging
			dojo.connect(d, "ondragstart",   dojo.stopEvent),
			dojo.connect(d.body, "onselectstart", dojo.stopEvent)
		];
		// notify that the move has started
		if(h && h.onMoveStart){
			h.onMoveStart(this);
		}
	},
	// mouse event processors
	onMouseMove: function(e){
		// summary:
		//		event processor for onmousemove/ontouchmove
		// e: Event
		//		mouse/touch event
		dojo.dnd.autoScroll(e);
		var m = this.marginBox;
		this.host.onMove(this, {l: m.l + e.pageX, t: m.t + e.pageY}, e);
		dojo.stopEvent(e);
	},
	onMouseUp: function(e){
		if(dojo.isWebKit && dojo.isMac && this.mouseButton == 2 ?
				e.button == 0 : this.mouseButton == e.button){ // TODO Should condition be met for touch devices, too?
			this.destroy();
		}
		dojo.stopEvent(e);
	},
	// utilities
	onFirstMove: function(e){
		// summary:
		//		makes the node absolute; it is meant to be called only once.
		// 		relative and absolutely positioned nodes are assumed to use pixel units
		var s = this.node.style, l, t, h = this.host;
		switch(s.position){
			case "relative":
			case "absolute":
				// assume that left and top values are in pixels already
				l = Math.round(parseFloat(s.left)) || 0;
				t = Math.round(parseFloat(s.top)) || 0;
				break;
			default:
				s.position = "absolute";	// enforcing the absolute mode
				var m = dojo.marginBox(this.node);
				// event.pageX/pageY (which we used to generate the initial
				// margin box) includes padding and margin set on the body.
				// However, setting the node's position to absolute and then
				// doing dojo.marginBox on it *doesn't* take that additional
				// space into account - so we need to subtract the combined
				// padding and margin.  We use getComputedStyle and
				// _getMarginBox/_getContentBox to avoid the extra lookup of
				// the computed style.
				var b = dojo.doc.body;
				var bs = dojo.getComputedStyle(b);
				var bm = dojo._getMarginBox(b, bs);
				var bc = dojo._getContentBox(b, bs);
				l = m.l - (bc.l - bm.l);
				t = m.t - (bc.t - bm.t);
				break;
		}
		this.marginBox.l = l - this.marginBox.l;
		this.marginBox.t = t - this.marginBox.t;
		if(h && h.onFirstMove){
			h.onFirstMove(this, e);
		}

		// Disconnect onmousemove and ontouchmove events that call this function
		dojo.disconnect(this.events.shift());
	},
	destroy: function(){
		// summary:
		//		stops the move, deletes all references, so the object can be garbage-collected
		dojo.forEach(this.events, dojo.disconnect);
		// undo global settings
		var h = this.host;
		if(h && h.onMoveStop){
			h.onMoveStop(this);
		}
		// destroy objects
		this.events = this.node = this.host = null;
	}
});

return dojo.dnd.Mover;
});

},
'ibm/tivoli/tip/dojo/InvalidContextError':function(){
// wrapped by build app
define("ibm/tivoli/tip/dojo/InvalidContextError", ["dijit","dojo","dojox","dojo/i18n!ibm/tivoli/tip/dojo/nls/resources"], function(dijit,dojo,dojox){

dojo.requireLocalization("ibm.tivoli.tip.dojo", "resources");
dojo.provide("ibm.tivoli.tip.dojo.InvalidContextError");

dojo.declare("ibm.tivoli.tip.dojo.InvalidContextError", Error, {});
});

},
'dojo/store/util/SimpleQueryEngine':function(){
define(["../../_base/array"], function(arrayUtil) {
  //  module:
  //    dojo/store/util/SimpleQueryEngine
  //  summary:
  //    The module defines a simple filtering query engine for object stores. 

return function(query, options){
	// summary:
	//		Simple query engine that matches using filter functions, named filter
	//		functions or objects by name-value on a query object hash
	//
	// description:
	//		The SimpleQueryEngine provides a way of getting a QueryResults through
	//		the use of a simple object hash as a filter.  The hash will be used to
	//		match properties on data objects with the corresponding value given. In
	//		other words, only exact matches will be returned.
	//
	//		This function can be used as a template for more complex query engines;
	//		for example, an engine can be created that accepts an object hash that
	//		contains filtering functions, or a string that gets evaluated, etc.
	//
	//		When creating a new dojo.store, simply set the store's queryEngine
	//		field as a reference to this function.
	//
	// query: Object
	//		An object hash with fields that may match fields of items in the store.
	//		Values in the hash will be compared by normal == operator, but regular expressions
	//		or any object that provides a test() method are also supported and can be
	// 		used to match strings by more complex expressions
	// 		(and then the regex's or object's test() method will be used to match values).
	//
	// options: dojo.store.util.SimpleQueryEngine.__queryOptions?
	//		An object that contains optional information such as sort, start, and count.
	//
	// returns: Function
	//		A function that caches the passed query under the field "matches".  See any
	//		of the "query" methods on dojo.stores.
	//
	// example:
	//		Define a store with a reference to this engine, and set up a query method.
	//
	//	|	var myStore = function(options){
	//	|		//	...more properties here
	//	|		this.queryEngine = dojo.store.util.SimpleQueryEngine;
	//	|		//	define our query method
	//	|		this.query = function(query, options){
	//	|			return dojo.store.util.QueryResults(this.queryEngine(query, options)(this.data));
	//	|		};
	//	|	};

	// create our matching query function
	switch(typeof query){
		default:
			throw new Error("Can not query with a " + typeof query);
		case "object": case "undefined":
			var queryObject = query;
			query = function(object){
				for(var key in queryObject){
					var required = queryObject[key];
					if(required && required.test){
						if(!required.test(object[key])){
							return false;
						}
					}else if(required != object[key]){
						return false;
					}
				}
				return true;
			};
			break;
		case "string":
			// named query
			if(!this[query]){
				throw new Error("No filter function " + query + " was found in store");
			}
			query = this[query];
			// fall through
		case "function":
			// fall through
	}
	function execute(array){
		// execute the whole query, first we filter
		var results = arrayUtil.filter(array, query);
		// next we sort
		if(options && options.sort){
			results.sort(function(a, b){
				for(var sort, i=0; sort = options.sort[i]; i++){
					var aValue = a[sort.attribute];
					var bValue = b[sort.attribute];
					if (aValue != bValue) {
						return !!sort.descending == aValue > bValue ? -1 : 1;
					}
				}
				return 0;
			});
		}
		// now we paginate
		if(options && (options.start || options.count)){
			var total = results.length;
			results = results.slice(options.start || 0, (options.start || 0) + (options.count || Infinity));
			results.total = total;
		}
		return results;
	}
	execute.matches = query;
	return execute;
};
});

},
'dijit/_HasDropDown':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/_base/Deferred",
	"dojo/_base/event", // event.stop
	"dojo/dom", // dom.isDescendant
	"dojo/dom-attr", // domAttr.set
	"dojo/dom-class", // domClass.add domClass.contains domClass.remove
	"dojo/dom-geometry", // domGeometry.marginBox domGeometry.position
	"dojo/dom-style", // domStyle.set
	"dojo/has",
	"dojo/keys", // keys.DOWN_ARROW keys.ENTER keys.ESCAPE
	"dojo/_base/lang", // lang.hitch lang.isFunction
	"dojo/touch",
	"dojo/_base/window", // win.doc
	"dojo/window", // winUtils.getBox
	"./registry",	// registry.byNode()
	"./focus",
	"./popup",
	"./_FocusMixin"
], function(declare, Deferred, event,dom, domAttr, domClass, domGeometry, domStyle, has, keys, lang, touch,
			win, winUtils, registry, focus, popup, _FocusMixin){

/*=====
	var _FocusMixin = dijit._FocusMixin;
=====*/

	// module:
	//		dijit/_HasDropDown
	// summary:
	//		Mixin for widgets that need drop down ability.

	return declare("dijit._HasDropDown", _FocusMixin, {
		// summary:
		//		Mixin for widgets that need drop down ability.

		// _buttonNode: [protected] DomNode
		//		The button/icon/node to click to display the drop down.
		//		Can be set via a data-dojo-attach-point assignment.
		//		If missing, then either focusNode or domNode (if focusNode is also missing) will be used.
		_buttonNode: null,

		// _arrowWrapperNode: [protected] DomNode
		//		Will set CSS class dijitUpArrow, dijitDownArrow, dijitRightArrow etc. on this node depending
		//		on where the drop down is set to be positioned.
		//		Can be set via a data-dojo-attach-point assignment.
		//		If missing, then _buttonNode will be used.
		_arrowWrapperNode: null,

		// _popupStateNode: [protected] DomNode
		//		The node to set the popupActive class on.
		//		Can be set via a data-dojo-attach-point assignment.
		//		If missing, then focusNode or _buttonNode (if focusNode is missing) will be used.
		_popupStateNode: null,

		// _aroundNode: [protected] DomNode
		//		The node to display the popup around.
		//		Can be set via a data-dojo-attach-point assignment.
		//		If missing, then domNode will be used.
		_aroundNode: null,

		// dropDown: [protected] Widget
		//		The widget to display as a popup.  This widget *must* be
		//		defined before the startup function is called.
		dropDown: null,

		// autoWidth: [protected] Boolean
		//		Set to true to make the drop down at least as wide as this
		//		widget.  Set to false if the drop down should just be its
		//		default width
		autoWidth: true,

		// forceWidth: [protected] Boolean
		//		Set to true to make the drop down exactly as wide as this
		//		widget.  Overrides autoWidth.
		forceWidth: false,

		// maxHeight: [protected] Integer
		//		The max height for our dropdown.
		//		Any dropdown taller than this will have scrollbars.
		//		Set to 0 for no max height, or -1 to limit height to available space in viewport
		maxHeight: 0,

		// dropDownPosition: [const] String[]
		//		This variable controls the position of the drop down.
		//		It's an array of strings with the following values:
		//
		//			* before: places drop down to the left of the target node/widget, or to the right in
		//			  the case of RTL scripts like Hebrew and Arabic
		//			* after: places drop down to the right of the target node/widget, or to the left in
		//			  the case of RTL scripts like Hebrew and Arabic
		//			* above: drop down goes above target node
		//			* below: drop down goes below target node
		//
		//		The list is positions is tried, in order, until a position is found where the drop down fits
		//		within the viewport.
		//
		dropDownPosition: ["below","above"],

		// _stopClickEvents: Boolean
		//		When set to false, the click events will not be stopped, in
		//		case you want to use them in your subwidget
		_stopClickEvents: true,

		_onDropDownMouseDown: function(/*Event*/ e){
			// summary:
			//		Callback when the user mousedown's on the arrow icon
			if(this.disabled || this.readOnly){ return; }

			// Prevent default to stop things like text selection, but don't stop propogation, so that:
			//		1. TimeTextBox etc. can focusthe <input> on mousedown
			//		2. dropDownButtonActive class applied by _CssStateMixin (on button depress)
			//		3. user defined onMouseDown handler fires
			e.preventDefault();

			this._docHandler = this.connect(win.doc, touch.release, "_onDropDownMouseUp");

			this.toggleDropDown();
		},

		_onDropDownMouseUp: function(/*Event?*/ e){
			// summary:
			//		Callback when the user lifts their mouse after mouse down on the arrow icon.
			//		If the drop down is a simple menu and the mouse is over the menu, we execute it, otherwise, we focus our
			//		drop down widget.  If the event is missing, then we are not
			//		a mouseup event.
			//
			//		This is useful for the common mouse movement pattern
			//		with native browser <select> nodes:
			//			1. mouse down on the select node (probably on the arrow)
			//			2. move mouse to a menu item while holding down the mouse button
			//			3. mouse up.  this selects the menu item as though the user had clicked it.
			if(e && this._docHandler){
				this.disconnect(this._docHandler);
			}
			var dropDown = this.dropDown, overMenu = false;

			if(e && this._opened){
				// This code deals with the corner-case when the drop down covers the original widget,
				// because it's so large.  In that case mouse-up shouldn't select a value from the menu.
				// Find out if our target is somewhere in our dropdown widget,
				// but not over our _buttonNode (the clickable node)
				var c = domGeometry.position(this._buttonNode, true);
				if(!(e.pageX >= c.x && e.pageX <= c.x + c.w) ||
					!(e.pageY >= c.y && e.pageY <= c.y + c.h)){
					var t = e.target;
					while(t && !overMenu){
						if(domClass.contains(t, "dijitPopup")){
							overMenu = true;
						}else{
							t = t.parentNode;
						}
					}
					if(overMenu){
						t = e.target;
						if(dropDown.onItemClick){
							var menuItem;
							while(t && !(menuItem = registry.byNode(t))){
								t = t.parentNode;
							}
							if(menuItem && menuItem.onClick && menuItem.getParent){
								menuItem.getParent().onItemClick(menuItem, e);
							}
						}
						return;
					}
				}
			}
			if(this._opened){
				if(dropDown.focus && dropDown.autoFocus !== false){
					// Focus the dropdown widget - do it on a delay so that we
					// don't steal our own focus.
					window.setTimeout(lang.hitch(dropDown, "focus"), 1);
				}
			}else{
				// The drop down arrow icon probably can't receive focus, but widget itself should get focus.
				// setTimeout() needed to make it work on IE (test DateTextBox)
				setTimeout(lang.hitch(this, "focus"), 0);
			}

			if(has("ios")){
				this._justGotMouseUp = true;
				setTimeout(lang.hitch(this, function(){
					this._justGotMouseUp = false;
				}), 0);
			}
		},

		_onDropDownClick: function(/*Event*/ e){
			if(has("ios") && !this._justGotMouseUp){
				// This branch fires on iPhone for ComboBox, because the button node is an <input> and doesn't
				// generate touchstart/touchend events.   Pretend we just got a mouse down / mouse up.
				// The if(has("ios") is necessary since IE and desktop safari get spurious onclick events
				// when there are nested tables (specifically, clicking on a table that holds a dijit.form.Select,
				// but not on the Select itself, causes an onclick event on the Select)
				this._onDropDownMouseDown(e);
				this._onDropDownMouseUp(e);
			}

			// The drop down was already opened on mousedown/keydown; just need to call stopEvent().
			if(this._stopClickEvents){
				event.stop(e);
			}
		},

		buildRendering: function(){
			this.inherited(arguments);

			this._buttonNode = this._buttonNode || this.focusNode || this.domNode;
			this._popupStateNode = this._popupStateNode || this.focusNode || this._buttonNode;

			// Add a class to the "dijitDownArrowButton" type class to _buttonNode so theme can set direction of arrow
			// based on where drop down will normally appear
			var defaultPos = {
					"after" : this.isLeftToRight() ? "Right" : "Left",
					"before" : this.isLeftToRight() ? "Left" : "Right",
					"above" : "Up",
					"below" : "Down",
					"left" : "Left",
					"right" : "Right"
			}[this.dropDownPosition[0]] || this.dropDownPosition[0] || "Down";
			domClass.add(this._arrowWrapperNode || this._buttonNode, "dijit" + defaultPos + "ArrowButton");
		},

		postCreate: function(){
			// summary:
			//		set up nodes and connect our mouse and keypress events

			this.inherited(arguments);

			this.connect(this._buttonNode, touch.press, "_onDropDownMouseDown");
			this.connect(this._buttonNode, "onclick", "_onDropDownClick");
			this.connect(this.focusNode, "onkeypress", "_onKey");
			this.connect(this.focusNode, "onkeyup", "_onKeyUp");
		},

		destroy: function(){
			if(this.dropDown){
				// Destroy the drop down, unless it's already been destroyed.  This can happen because
				// the drop down is a direct child of <body> even though it's logically my child.
				if(!this.dropDown._destroyed){
					this.dropDown.destroyRecursive();
				}
				delete this.dropDown;
			}
			this.inherited(arguments);
		},

		_onKey: function(/*Event*/ e){
			// summary:
			//		Callback when the user presses a key while focused on the button node

			if(this.disabled || this.readOnly){ return; }

			var d = this.dropDown, target = e.target;
			if(d && this._opened && d.handleKey){
				if(d.handleKey(e) === false){
					/* false return code means that the drop down handled the key */
					event.stop(e);
					return;
				}
			}
			if(d && this._opened && e.charOrCode == keys.ESCAPE){
				this.closeDropDown();
				event.stop(e);
			}else if(!this._opened &&
					(e.charOrCode == keys.DOWN_ARROW ||
						( (e.charOrCode == keys.ENTER || e.charOrCode == " ") &&
						  //ignore enter and space if the event is for a text input
						  ((target.tagName || "").toLowerCase() !== 'input' ||
						     (target.type && target.type.toLowerCase() !== 'text'))))){
				// Toggle the drop down, but wait until keyup so that the drop down doesn't
				// get a stray keyup event, or in the case of key-repeat (because user held
				// down key for too long), stray keydown events
				this._toggleOnKeyUp = true;
				event.stop(e);
			}
		},

		_onKeyUp: function(){
			if(this._toggleOnKeyUp){
				delete this._toggleOnKeyUp;
				this.toggleDropDown();
				var d = this.dropDown;	// drop down may not exist until toggleDropDown() call
				if(d && d.focus){
					setTimeout(lang.hitch(d, "focus"), 1);
				}
			}
		},

		_onBlur: function(){
			// summary:
			//		Called magically when focus has shifted away from this widget and it's dropdown

			// Don't focus on button if the user has explicitly focused on something else (happens
			// when user clicks another control causing the current popup to close)..
			// But if focus is inside of the drop down then reset focus to me, because IE doesn't like
			// it when you display:none a node with focus.
			var focusMe = focus.curNode && this.dropDown && dom.isDescendant(focus.curNode, this.dropDown.domNode);

			this.closeDropDown(focusMe);

			this.inherited(arguments);
		},

		isLoaded: function(){
			// summary:
			//		Returns true if the dropdown exists and it's data is loaded.  This can
			//		be overridden in order to force a call to loadDropDown().
			// tags:
			//		protected

			return true;
		},

		loadDropDown: function(/*Function*/ loadCallback){
			// summary:
			//		Creates the drop down if it doesn't exist, loads the data
			//		if there's an href and it hasn't been loaded yet, and then calls
			//		the given callback.
			// tags:
			//		protected

			// TODO: for 2.0, change API to return a Deferred, instead of calling loadCallback?
			loadCallback();
		},

		loadAndOpenDropDown: function(){
			// summary:
			//		Creates the drop down if it doesn't exist, loads the data
			//		if there's an href and it hasn't been loaded yet, and
			//		then opens the drop down.  This is basically a callback when the
			//		user presses the down arrow button to open the drop down.
			// returns: Deferred
			//		Deferred for the drop down widget that
			//		fires when drop down is created and loaded
			// tags:
			//		protected
			var d = new Deferred(),
				afterLoad = lang.hitch(this, function(){
					this.openDropDown();
					d.resolve(this.dropDown);
				});
			if(!this.isLoaded()){
				this.loadDropDown(afterLoad);
			}else{
				afterLoad();
			}
			return d;
		},

		toggleDropDown: function(){
			// summary:
			//		Callback when the user presses the down arrow button or presses
			//		the down arrow key to open/close the drop down.
			//		Toggle the drop-down widget; if it is up, close it, if not, open it
			// tags:
			//		protected

			if(this.disabled || this.readOnly){ return; }
			if(!this._opened){
				this.loadAndOpenDropDown();
			}else{
				this.closeDropDown();
			}
		},

		openDropDown: function(){
			// summary:
			//		Opens the dropdown for this widget.   To be called only when this.dropDown
			//		has been created and is ready to display (ie, it's data is loaded).
			// returns:
			//		return value of dijit.popup.open()
			// tags:
			//		protected

			var dropDown = this.dropDown,
				ddNode = dropDown.domNode,
				aroundNode = this._aroundNode || this.domNode,
				self = this;

			// Prepare our popup's height and honor maxHeight if it exists.

			// TODO: isn't maxHeight dependent on the return value from dijit.popup.open(),
			// ie, dependent on how much space is available (BK)

			if(!this._preparedNode){
				this._preparedNode = true;
				// Check if we have explicitly set width and height on the dropdown widget dom node
				if(ddNode.style.width){
					this._explicitDDWidth = true;
				}
				if(ddNode.style.height){
					this._explicitDDHeight = true;
				}
			}

			// Code for resizing dropdown (height limitation, or increasing width to match my width)
			if(this.maxHeight || this.forceWidth || this.autoWidth){
				var myStyle = {
					display: "",
					visibility: "hidden"
				};
				if(!this._explicitDDWidth){
					myStyle.width = "";
				}
				if(!this._explicitDDHeight){
					myStyle.height = "";
				}
				domStyle.set(ddNode, myStyle);

				// Figure out maximum height allowed (if there is a height restriction)
				var maxHeight = this.maxHeight;
				if(maxHeight == -1){
					// limit height to space available in viewport either above or below my domNode
					// (whichever side has more room)
					var viewport = winUtils.getBox(),
						position = domGeometry.position(aroundNode, false);
					maxHeight = Math.floor(Math.max(position.y, viewport.h - (position.y + position.h)));
				}

				// Attach dropDown to DOM and make make visibility:hidden rather than display:none
				// so we call startup() and also get the size
				popup.moveOffScreen(dropDown);

				if(dropDown.startup && !dropDown._started){
					dropDown.startup(); // this has to be done after being added to the DOM
				}
				// Get size of drop down, and determine if vertical scroll bar needed
				var mb = domGeometry.getMarginSize(ddNode);
				var overHeight = (maxHeight && mb.h > maxHeight);
				domStyle.set(ddNode, {
					overflowX: "hidden",
					overflowY: overHeight ? "auto" : "hidden"
				});
				if(overHeight){
					mb.h = maxHeight;
					if("w" in mb){
						mb.w += 16;	// room for vertical scrollbar
					}
				}else{
					delete mb.h;
				}

				// Adjust dropdown width to match or be larger than my width
				if(this.forceWidth){
					mb.w = aroundNode.offsetWidth;
				}else if(this.autoWidth){
					mb.w = Math.max(mb.w, aroundNode.offsetWidth);
				}else{
					delete mb.w;
				}

				// And finally, resize the dropdown to calculated height and width
				if(lang.isFunction(dropDown.resize)){
					dropDown.resize(mb);
				}else{
					domGeometry.setMarginBox(ddNode, mb);
				}
			}

			var retVal = popup.open({
				parent: this,
				popup: dropDown,
				around: aroundNode,
				orient: this.dropDownPosition,
				onExecute: function(){
					self.closeDropDown(true);
				},
				onCancel: function(){
					self.closeDropDown(true);
				},
				onClose: function(){
					domAttr.set(self._popupStateNode, "popupActive", false);
					domClass.remove(self._popupStateNode, "dijitHasDropDownOpen");
					self._opened = false;
				}
			});
			domAttr.set(this._popupStateNode, "popupActive", "true");
			domClass.add(self._popupStateNode, "dijitHasDropDownOpen");
			this._opened=true;

			// TODO: set this.checked and call setStateClass(), to affect button look while drop down is shown
			return retVal;
		},

		closeDropDown: function(/*Boolean*/ focus){
			// summary:
			//		Closes the drop down on this widget
			// focus:
			//		If true, refocuses the button widget
			// tags:
			//		protected

			if(this._opened){
				if(focus){ this.focus(); }
				popup.close(this.dropDown);
				this._opened = false;
			}
		}

	});
});

},
'dijit/_Widget':function(){
define([
	"dojo/aspect",	// aspect.around
	"dojo/_base/config",	// config.isDebug
	"dojo/_base/connect",	// connect.connect
	"dojo/_base/declare", // declare
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/lang", // lang.hitch
	"dojo/query",
	"dojo/ready",
	"./registry",	// registry.byNode
	"./_WidgetBase",
	"./_OnDijitClickMixin",
	"./_FocusMixin",
	"dojo/uacss",		// browser sniffing (included for back-compat; subclasses may be using)
	"./hccss"		// high contrast mode sniffing (included to set CSS classes on <body>, module ret value unused)
], function(aspect, config, connect, declare, kernel, lang, query, ready,
			registry, _WidgetBase, _OnDijitClickMixin, _FocusMixin){

/*=====
	var _WidgetBase = dijit._WidgetBase;
	var _OnDijitClickMixin = dijit._OnDijitClickMixin;
	var _FocusMixin = dijit._FocusMixin;
=====*/


// module:
//		dijit/_Widget
// summary:
//		Old base for widgets.   New widgets should extend _WidgetBase instead


function connectToDomNode(){
	// summary:
	//		If user connects to a widget method === this function, then they will
	//		instead actually be connecting the equivalent event on this.domNode
}

// Trap dojo.connect() calls to connectToDomNode methods, and redirect to _Widget.on()
function aroundAdvice(originalConnect){
	return function(obj, event, scope, method){
		if(obj && typeof event == "string" && obj[event] == connectToDomNode){
			return obj.on(event.substring(2).toLowerCase(), lang.hitch(scope, method));
		}
		return originalConnect.apply(connect, arguments);
	};
}
aspect.around(connect, "connect", aroundAdvice);
if(kernel.connect){
	aspect.around(kernel, "connect", aroundAdvice);
}

var _Widget = declare("dijit._Widget", [_WidgetBase, _OnDijitClickMixin, _FocusMixin], {
	// summary:
	//		Base class for all Dijit widgets.
	//
	//		Extends _WidgetBase, adding support for:
	//			- declaratively/programatically specifying widget initialization parameters like
	//				onMouseMove="foo" that call foo when this.domNode gets a mousemove event
	//			- ondijitclick
	//				Support new data-dojo-attach-event="ondijitclick: ..." that is triggered by a mouse click or a SPACE/ENTER keypress
	//			- focus related functions
	//				In particular, the onFocus()/onBlur() callbacks.   Driven internally by
	//				dijit/_base/focus.js.
	//			- deprecated methods
	//			- onShow(), onHide(), onClose()
	//
	//		Also, by loading code in dijit/_base, turns on:
	//			- browser sniffing (putting browser id like .dj_ie on <html> node)
	//			- high contrast mode sniffing (add .dijit_a11y class to <body> if machine is in high contrast mode)


	////////////////// DEFERRED CONNECTS ///////////////////

	onClick: connectToDomNode,
	/*=====
	onClick: function(event){
		// summary:
		//		Connect to this function to receive notifications of mouse click events.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onDblClick: connectToDomNode,
	/*=====
	onDblClick: function(event){
		// summary:
		//		Connect to this function to receive notifications of mouse double click events.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onKeyDown: connectToDomNode,
	/*=====
	onKeyDown: function(event){
		// summary:
		//		Connect to this function to receive notifications of keys being pressed down.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onKeyPress: connectToDomNode,
	/*=====
	onKeyPress: function(event){
		// summary:
		//		Connect to this function to receive notifications of printable keys being typed.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onKeyUp: connectToDomNode,
	/*=====
	onKeyUp: function(event){
		// summary:
		//		Connect to this function to receive notifications of keys being released.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onMouseDown: connectToDomNode,
	/*=====
	onMouseDown: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse button is pressed down.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseMove: connectToDomNode,
	/*=====
	onMouseMove: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves over nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseOut: connectToDomNode,
	/*=====
	onMouseOut: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves off of nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseOver: connectToDomNode,
	/*=====
	onMouseOver: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves onto nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseLeave: connectToDomNode,
	/*=====
	onMouseLeave: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves off of this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseEnter: connectToDomNode,
	/*=====
	onMouseEnter: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves onto this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseUp: connectToDomNode,
	/*=====
	onMouseUp: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse button is released.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/

	constructor: function(params){
		// extract parameters like onMouseMove that should connect directly to this.domNode
		this._toConnect = {};
		for(var name in params){
			if(this[name] === connectToDomNode){
				this._toConnect[name.replace(/^on/, "").toLowerCase()] = params[name];
				delete params[name];
			}
		}
	},

	postCreate: function(){
		this.inherited(arguments);

		// perform connection from this.domNode to user specified handlers (ex: onMouseMove)
		for(var name in this._toConnect){
			this.on(name, this._toConnect[name]);
		}
		delete this._toConnect;
	},

	on: function(/*String*/ type, /*Function*/ func){
		if(this[this._onMap(type)] === connectToDomNode){
			// Use connect.connect() rather than on() to get handling for "onmouseenter" on non-IE, etc.
			// Also, need to specify context as "this" rather than the default context of the DOMNode
			return connect.connect(this.domNode, type.toLowerCase(), this, func);
		}
		return this.inherited(arguments);
	},

	_setFocusedAttr: function(val){
		// Remove this method in 2.0 (or sooner), just here to set _focused == focused, for back compat
		// (but since it's a private variable we aren't required to keep supporting it).
		this._focused = val;
		this._set("focused", val);
	},

	////////////////// DEPRECATED METHODS ///////////////////

	setAttribute: function(/*String*/ attr, /*anything*/ value){
		// summary:
		//		Deprecated.  Use set() instead.
		// tags:
		//		deprecated
		kernel.deprecated(this.declaredClass+"::setAttribute(attr, value) is deprecated. Use set() instead.", "", "2.0");
		this.set(attr, value);
	},

	attr: function(/*String|Object*/name, /*Object?*/value){
		// summary:
		//		Set or get properties on a widget instance.
		//	name:
		//		The property to get or set. If an object is passed here and not
		//		a string, its keys are used as names of attributes to be set
		//		and the value of the object as values to set in the widget.
		//	value:
		//		Optional. If provided, attr() operates as a setter. If omitted,
		//		the current value of the named property is returned.
		// description:
		//		This method is deprecated, use get() or set() directly.

		// Print deprecation warning but only once per calling function
		if(config.isDebug){
			var alreadyCalledHash = arguments.callee._ach || (arguments.callee._ach = {}),
				caller = (arguments.callee.caller || "unknown caller").toString();
			if(!alreadyCalledHash[caller]){
				kernel.deprecated(this.declaredClass + "::attr() is deprecated. Use get() or set() instead, called from " +
				caller, "", "2.0");
				alreadyCalledHash[caller] = true;
			}
		}

		var args = arguments.length;
		if(args >= 2 || typeof name === "object"){ // setter
			return this.set.apply(this, arguments);
		}else{ // getter
			return this.get(name);
		}
	},

	getDescendants: function(){
		// summary:
		//		Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.
		//		This method should generally be avoided as it returns widgets declared in templates, which are
		//		supposed to be internal/hidden, but it's left here for back-compat reasons.

		kernel.deprecated(this.declaredClass+"::getDescendants() is deprecated. Use getChildren() instead.", "", "2.0");
		return this.containerNode ? query('[widgetId]', this.containerNode).map(registry.byNode) : []; // dijit._Widget[]
	},

	////////////////// MISCELLANEOUS METHODS ///////////////////

	_onShow: function(){
		// summary:
		//		Internal method called when this widget is made visible.
		//		See `onShow` for details.
		this.onShow();
	},

	onShow: function(){
		// summary:
		//		Called when this widget becomes the selected pane in a
		//		`dijit.layout.TabContainer`, `dijit.layout.StackContainer`,
		//		`dijit.layout.AccordionContainer`, etc.
		//
		//		Also called to indicate display of a `dijit.Dialog`, `dijit.TooltipDialog`, or `dijit.TitlePane`.
		// tags:
		//		callback
	},

	onHide: function(){
		// summary:
			//		Called when another widget becomes the selected pane in a
			//		`dijit.layout.TabContainer`, `dijit.layout.StackContainer`,
			//		`dijit.layout.AccordionContainer`, etc.
			//
			//		Also called to indicate hide of a `dijit.Dialog`, `dijit.TooltipDialog`, or `dijit.TitlePane`.
			// tags:
			//		callback
	},

	onClose: function(){
		// summary:
		//		Called when this widget is being displayed as a popup (ex: a Calendar popped
		//		up from a DateTextBox), and it is hidden.
		//		This is called from the dijit.popup code, and should not be called directly.
		//
		//		Also used as a parameter for children of `dijit.layout.StackContainer` or subclasses.
		//		Callback if a user tries to close the child.   Child will be closed if this function returns true.
		// tags:
		//		extension

		return true;		// Boolean
	}
});

// For back-compat, remove in 2.0.
if(!kernel.isAsync){
	ready(0, function(){
		var requires = ["dijit/_base"];
		require(requires);	// use indirection so modules not rolled into a build
	});
}
return _Widget;
});

},
'dojox/grid/DataSelection':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/array",
	"./_SelectionPreserver",
	"./Selection",
	"dojo/_base/Deferred",
	"dojo/DeferredList"
], function(declare, lang, array, _SelectionPreserver, Selection, Deferred, DeferredList){
	
return declare("dojox.grid.DataSelection", Selection, {
	constructor: function(grid){
		if(grid.keepSelection){
			this.preserver = new _SelectionPreserver(this);
		}
	},
	
	destroy: function(){
		if(this.preserver){
			this.preserver.destroy();
		}
	},
	
	getFirstSelected: function(){
		var idx = Selection.prototype.getFirstSelected.call(this);

		if(idx == -1){ return null; }
		return this.grid.getItem(idx);
	},

	getNextSelected: function(inPrev){
		var old_idx = this.grid.getItemIndex(inPrev);
		var idx = Selection.prototype.getNextSelected.call(this, old_idx);

		if(idx == -1){ return null; }
		return this.grid.getItem(idx);
	},

	getSelected: function(){
		var result = [];
		for(var i=0, l=this.selected.length; i<l; i++){
			if(this.selected[i]){
				result.push(this.grid.getItem(i));
			}
		}
		return result;
	},

	getSelectedItems: function(pageSize){
		// summary:
		//		This function is an asynchronous implementation of getSelected().
		//		Unlike getSelected, we won't get any null values in the result of this function.
		// pageSize: Integer
		//		To load all selected rows, the grid needs to fetch a lot of data. So it is necessary
		//		to break a huge request into several small requests, so as not to hold up the browser
		//		for too long.
		//		This pageSize parameter is used to specifiy how many rows to fetch in one request.
		//		If not provided or equal to zero, grid.rowsPerPage is used.
		//		If less than zero, then there will be no upper limit to the request size.
		//		If client side store is used, it is recommented to provide a negative pageSize such as -1.
		// return: Deferred Object
		//		The Deferred object representing the result item array.
		// example:
		// |	grid.selection.getSelectedItems().then(function(items){
		// |		//We get all the selected items here.
		// |	});
		var res = [], d = new Deferred(), defs = [],
			start, i, len, item, toFetch = [], 
			grid = this.grid, _this = this, 
			ps = pageSize || grid.rowsPerPage,
			//Prepare the constant options.
			options = {
				query: grid.query,
				sort: grid.getSortProps(),
				queryOptions: grid.queryOptions,
				onError: lang.hitch(d, d.errback)
			},
			onComplete = function(def, start, items){
				//Save the fetched items to the result array.
				for(var i = 0, len = items.length; i < len; ++i){
					res[toFetch[start] + i] = items[i];
				}
				def.callback();
			};
		for(i = 0, len = this.selected.length; i < len; ++i){
			if(this.selected[i]){
				item = grid._by_idx[i];
				res[i] = item && item.item;
				if(!item){
					toFetch.push(i);
				}
			}
		}
		for(start = 0, len = toFetch.length; start < len; start = i){
			//Find how many items can we get by one fetch.
			for(i = start + 1; 
					i < len &&
					toFetch[i] === toFetch[i - 1] + 1 &&	//Ensure the item indexes are continuous
					ps < 0 || i - start < ps;		//Paging the request, so we can handle huge stores. 
				++i){}
			var def = new Deferred();
			var req = lang.mixin({
				start: toFetch[start],
				count: i - start,
				onComplete: lang.partial(onComplete, def, start)
			}, options);
			defs.push(def);
			//Check whether we are using EDG
            if(grid._storeLayerFetch){
                grid._storeLayerFetch(req);
            }else{
                grid.store.fetch(req);
            }
		}
		(new DeferredList(defs)).then(function(){
			res = array.filter(res, function(item){
				return item;
			});
			if(_this.preserver){
				//Since the grid is paged, some previously selected rows may not be loaded after
				//sorting or filtering, etc. So get these rows from selectedById.
				//Note: the row order can not be honored here! 
				var id, dl = [],
					onItem = function(defer, item){
						res.push(item);
						defer.callback();
					},
					selectedIds = array.map(res, function(item){
						return grid.store.getIdentity(item);
					});
				for(id in _this.preserver._selectedById){
					if(_this.preserver._selectedById[id] === true && array.indexOf(selectedIds, id) < 0){
						var dd = new Deferred();
						grid.store.fetchItemByIdentity({
							identity: id,
							onItem: lang.partial(onItem, dd)
						});
						dl.push(dd);
					}
				}
				if(dl.length){
					(new DeferredList(dl)).then(function(){
						//Get back to the user.
						d.callback(res);
					});
					return;
				}
			}
			//Get back to the user.
			d.callback(res);
		}, lang.hitch(d, d.errback));
		return d;
	},

	addToSelection: function(inItemOrIndex){
		if(this.mode == 'none'){ return; }
		var idx = null;
		if(typeof inItemOrIndex == "number" || typeof inItemOrIndex == "string"){
			idx = inItemOrIndex;
		}else{
			idx = this.grid.getItemIndex(inItemOrIndex);
		}
		Selection.prototype.addToSelection.call(this, idx);
	},

	deselect: function(inItemOrIndex){
		if(this.mode == 'none'){ return; }
		var idx = null;
		if(typeof inItemOrIndex == "number" || typeof inItemOrIndex == "string"){
			idx = inItemOrIndex;
		}else{
			idx = this.grid.getItemIndex(inItemOrIndex);
		}
		Selection.prototype.deselect.call(this, idx);
	},

	deselectAll: function(inItemOrIndex){
		var idx = null;
		if(inItemOrIndex || typeof inItemOrIndex == "number"){
			if(typeof inItemOrIndex == "number" || typeof inItemOrIndex == "string"){
				idx = inItemOrIndex;
			}else{
				idx = this.grid.getItemIndex(inItemOrIndex);
			}
			Selection.prototype.deselectAll.call(this, idx);
		}else{
			this.inherited(arguments);
		}
	}
});
});

},
'gridx/modules/move/Column':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/query",
	"dojo/_base/array",
	"dojo/keys",
	"../../core/_Module"
], function(declare, query, array, keys, _Module){

	return declare(/*===== "gridx.modules.move.Column", =====*/_Module, {
		// summary:
		//		This module provides several APIs to move columns within grid.
		// description:
		//		This module does not include any UI. So different kind of column dnd UI implementations can be built
		//		upon this module.
		//		But this module does provide a keyboard support for reordering columns. When focus is on a column header,
		//		pressing CTRL+LEFT/RIGHT ARROW will move the column around within grid.

		name: 'moveColumn',
		
		getAPIPath: function(){
			// tags:
			//		protected extension
			return {
				move: {
					column: this
				}
			};
		},

		preload: function(){
			this.aspect(this.grid, 'onHeaderCellKeyDown', '_onKeyDown');
		},

		columnMixin: {
			moveTo: function(target){
				// summary:
				//		Move this column to the position before the column with index "target"
				// target: Integer
				//		The target index
				this.grid.move.column.moveRange(this.index(), 1, target);
				return this;
			}
		},
		
		//public---------------------------------------------------------------

		//moveSelected: Boolean
		//		When moving using keyboard, whether to move all selected columns together.
		moveSelected: true,

		move: function(columnIndexes, target){
			// summary:
			//		Move some columns to the given target position
			// columnIndexes: Integer[]
			//		The current indexes of columns to move
			// target: Integer
			//		The moved columns will be inserted before the column with this index.
			if(typeof columnIndexes === 'number'){
				columnIndexes = [columnIndexes];
			}
			var map = [], i, len, columns = this.grid._columns, pos, movedCols = [];
			for(i = 0, len = columnIndexes.length; i < len; ++i){
				map[columnIndexes[i]] = true;
			}
			for(i = map.length - 1; i >= 0; --i){
				if(map[i]){
					movedCols.unshift(columns[i]);
					columns.splice(i, 1);
				}
			}
			for(i = 0, len = columns.length; i < len; ++i){
				if(columns[i].index >= target){
					pos = i;
					break;
				}
			}
			if(pos === undefined){
				pos = columns.length;
			}
			this._moveComplete(movedCols, pos);
		},
	
		moveRange: function(start, count, target){
			// summary:
			//		Move a range of columns to a given target position
			// start: Integer
			//		The index of the first column to move
			// count: Integer
			//		The count of columns to move
			if(target < start || target > start + count){
				if(target > start + count){
					target -= count;
				}
				this._moveComplete(this.grid._columns.splice(start, count), target);
			}
		},
		
		//Events--------------------------------------------------------------------
		onMoved: function(){
			// summary:
			//		Fired when column move is performed successfully
			// tags:
			//		callback
		},
		
		//Private-------------------------------------------------------------------
		_moveComplete: function(movedCols, target){
			var g = this.grid,
				map = {},
				columns = g._columns,
				i, movedColIds = {},
				targetId = target < columns.length ? columns[target].id : null,
				update = function(tr){
					var cells = query('> .gridxCell', tr).filter(function(cellNode){
						return movedColIds[cellNode.getAttribute('colid')];
					});
					if(targetId === null){
						cells.place(tr);
					}else{
						cells.place(query('> [colid="' + targetId + '"]', tr)[0], 'before');
					}
				};
			for(i = movedCols.length - 1; i >= 0; --i){
				map[movedCols[i].index] = target + i;
				movedColIds[movedCols[i].id] = 1;
			}
			[].splice.apply(columns, [target, 0].concat(movedCols));
			for(i = columns.length - 1; i >= 0; --i){
				columns[i].index = i;
			}
			update(query('.gridxHeaderRowInner > table > tbody > tr', g.headerNode)[0]);
			query('.gridxRow > table > tbody > tr', g.bodyNode).forEach(update);
			this.onMoved(map);
		},

		_onKeyDown: function(e){
			var t = this,
				g = t.grid,
				selector = t.arg('moveSelected') && g.select && g.select.column,
				ltr = g.isLeftToRight(),
				preKey = ltr ? keys.LEFT_ARROW : keys.RIGHT_ARROW,
				postKey = ltr ? keys.RIGHT_ARROW : keys.LEFT_ARROW;
			if(e.ctrlKey && !e.shiftKey && !e.altKey && (e.keyCode == preKey || e.keyCode == postKey)){
				var target = e.columnIndex,
					colIdxes = selector && selector.isSelected(e.columnId) ?
						array.map(selector.getSelected(), function(id){
							return g._columnsById[id].index;
						}) : [e.columnIndex],
					node = g.header.getHeaderNode(e.columnId);
				if(e.keyCode == preKey){
					while(array.indexOf(colIdxes, target) >= 0){
						target--;
					}
					if(target >= 0){
						t.move(colIdxes, target);
						g.header._focusNode(node);
					}
				}else if(e.keyCode == postKey){
					while(array.indexOf(colIdxes, target) >= 0){
						target++;
					}
					if(target < g._columns.length){
						t.move(colIdxes, target + 1);
						g.header._focusNode(node);
					}
				}
			}
		}
	});
});

},
'dojo/currency':function(){
define(["./_base/kernel", "./_base/lang", "./_base/array", "./number", "./i18n", "./i18n!./cldr/nls/currency", "./cldr/monetary"], function(dojo, lang, darray, dnumber, i18n, nlsCurrency, cldrMonetary) {
	// module:
	//		dojo/currency
	// summary:
	//		TODOC

lang.getObject("currency", true, dojo);

/*=====
dojo.currency = {
	// summary: localized formatting and parsing routines for currencies
	//
	// description: extends dojo.number to provide culturally-appropriate formatting of values
	//	in various world currencies, including use of a currency symbol.  The currencies are specified
	//	by a three-letter international symbol in all uppercase, and support for the currencies is
	//	provided by the data in `dojo.cldr`.  The scripts generating dojo.cldr specify which
	//	currency support is included.  A fixed number of decimal places is determined based
	//	on the currency type and is not determined by the 'pattern' argument.  The fractional
	//	portion is optional, by default, and variable length decimals are not supported.
}
=====*/

dojo.currency._mixInDefaults = function(options){
	options = options || {};
	options.type = "currency";

	// Get locale-dependent currency data, like the symbol
	var bundle = i18n.getLocalization("dojo.cldr", "currency", options.locale) || {};

	// Mixin locale-independent currency data, like # of places
	var iso = options.currency;
	var data = cldrMonetary.getData(iso);

	darray.forEach(["displayName","symbol","group","decimal"], function(prop){
		data[prop] = bundle[iso+"_"+prop];
	});

	data.fractional = [true, false];

	// Mixin with provided options
	return lang.mixin(data, options);
};

/*=====
dojo.declare("dojo.currency.__FormatOptions", [dojo.number.__FormatOptions], {
	//	type: String?
	//		Should not be set.  Value is assumed to be "currency".
	//	symbol: String?
	//		localized currency symbol. The default will be looked up in table of supported currencies in `dojo.cldr`
	//		A [ISO4217](http://en.wikipedia.org/wiki/ISO_4217) currency code will be used if not found.
	//	currency: String?
	//		an [ISO4217](http://en.wikipedia.org/wiki/ISO_4217) currency code, a three letter sequence like "USD".
	//		For use with dojo.currency only.
	//	places: Number?
	//		number of decimal places to show.  Default is defined based on which currency is used.
	type: "",
	symbol: "",
	currency: "",
	places: ""
});
=====*/

dojo.currency.format = function(/*Number*/value, /*dojo.currency.__FormatOptions?*/options){
// summary:
//		Format a Number as a currency, using locale-specific settings
//
// description:
//		Create a string from a Number using a known, localized pattern.
//		[Formatting patterns](http://www.unicode.org/reports/tr35/#Number_Elements)
//		appropriate to the locale are chosen from the [CLDR](http://unicode.org/cldr)
//		as well as the appropriate symbols and delimiters and number of decimal places.
//
// value:
//		the number to be formatted.

	return dnumber.format(value, dojo.currency._mixInDefaults(options));
};

dojo.currency.regexp = function(/*dojo.number.__RegexpOptions?*/options){
//
// summary:
//		Builds the regular needed to parse a currency value
//
// description:
//		Returns regular expression with positive and negative match, group and decimal separators
//		Note: the options.places default, the number of decimal places to accept, is defined by the currency type.
	return dnumber.regexp(dojo.currency._mixInDefaults(options)); // String
};

/*=====
dojo.declare("dojo.currency.__ParseOptions", [dojo.number.__ParseOptions], {
	//	type: String?
	//		Should not be set.  Value is assumed to be currency.
	//	currency: String?
	//		an [ISO4217](http://en.wikipedia.org/wiki/ISO_4217) currency code, a three letter sequence like "USD".
	//		For use with dojo.currency only.
	//	symbol: String?
	//		localized currency symbol. The default will be looked up in table of supported currencies in `dojo.cldr`
	//		A [ISO4217](http://en.wikipedia.org/wiki/ISO_4217) currency code will be used if not found.
	//	places: Number?
	//		fixed number of decimal places to accept.  The default is determined based on which currency is used.
	//	fractional: Boolean?|Array?
	//		Whether to include the fractional portion, where the number of decimal places are implied by the currency
	//		or explicit 'places' parameter.  The value [true,false] makes the fractional portion optional.
	//		By default for currencies, it the fractional portion is optional.
	type: "",
	currency: "",
	symbol: "",
	places: "",
	fractional: ""
});
=====*/

dojo.currency.parse = function(/*String*/expression, /*dojo.currency.__ParseOptions?*/options){
	//
	// summary:
	//		Convert a properly formatted currency string to a primitive Number,
	//		using locale-specific settings.
	//
	// description:
	//		Create a Number from a string using a known, localized pattern.
	//		[Formatting patterns](http://www.unicode.org/reports/tr35/#Number_Format_Patterns)
	//		are chosen appropriate to the locale, as well as the appropriate symbols and delimiters
	//		and number of decimal places.
	//
	// expression: A string representation of a currency value

	return dnumber.parse(expression, dojo.currency._mixInDefaults(options));
};

return dojo.currency;
});

},
'ibm/tivoli/navmodel/Property':function(){
// wrapped by build app
define("ibm/tivoli/navmodel/Property", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/Status,dojo/date/stamp"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Property");
dojo.require("ibm.tivoli.navmodel.Status"); 
dojo.require("dojo.date.stamp");

////////////////////////////////////
// Property
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Property", null, {
	// private members
	_prop: null,
	_dataset: null,
	_objectType: null,
	_value: null,
	_task: null,

	/*=====
	constructor_kwArgs: function(property, dataset) {
		// property: Object
		//		json property object
		// dataset: Object
		//		ibm.tivoli.navmodel.Dataset that owns this item
		// objectType: String
		//		objectType of the owning object "item", "link", or "relationship"
	},
	=====*/

	 constructor: function(/*constructor_kwArgs*/ kwArgs) {		
		// summary:
		//		Constructs Property object.
		this._dataset = kwArgs.dataset;
		this._objectType = (kwArgs.objectType ? kwArgs.objectType : "item");
		this._prop = kwArgs.property;
		
		// optimization of json on the server can leave out:
		// "displayValue"
		if ((typeof this._prop.displayValue === "undefined") || (this._prop.displayValue === null)) {  // 54308
			// be careful not to set displayValue to a non-scalar
			if (typeof this._prop.value == "object") {
				this._prop.displayValue = "";
			} else {
				this._prop.displayValue = this._prop.value;											
			}		
		}		
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	 
	clone: function() {		
		// summary:
		//		Clones the property object and returns a new one with the same values
		var args = {
			property: dojo.clone(this._prop),
			dataset: this._dataset,
			objectType: this._objectType
		};		
		
		return new ibm.tivoli.navmodel.Property(args);
	},

	mixin: function(property) {		
		// summary:
		//		Mixin contents of property with ours
		if (property !== this && property.getId() === this.getId()) {
			this._prop = property.getAttributes();
			
			// re-cache values on next request
			this._value = null;
			this._task = null;		
		}
	},

	getAttribute: function(key) {		
		// summary:
		//		Returns general attribute for requested key
		return this._prop[key];
	},

	setAttribute: function(key, value) {		
		// summary:
		//		Set general attribute for requested key
		this._prop[key] = value;
	},
	
	getAttributes: function() {		
		// summary:
		//		Get all general attributes for the property
		return dojo.clone(this._prop);		
	},

	mixinAttributes: function(attr) {		
		// summary:
		//		Mixin general attributes for the property
		dojo.mixin(this._prop, attr);
		
		// re-cache values on next request
		this._value = null;
		this._task = null;
	},

	getId: function() {		
		// summary:
		//		Returns the property id
		return this._prop.id;
	},

	setId: function(id) {		
		// summary:
		//		Set the property id
		this._prop.id = id;
	},

	getLabel: function() {		
		// summary:
		//		Returns the property label
		return this._prop.label;
	},

	setLabel: function(label) {		
		// summary:
		//		Set the property label
		this._prop.label = label;
	},
	
	getTooltip: function() {
		// summary:
		//		Returns the property tooltip
		return this._prop.tooltip;
	},
	
	setTooltip: function(tooltip) {
		// summary:
		//		Set the property tooltip
		this._prop.tooltip = tooltip;
	},

	getDisplayValue: function() {		
		// summary:
		//		Returns the globalized display value for the property
		return this._prop.displayValue;
	},

	setDisplayValue: function(displayValue) {		
		// summary:
		//		Set the globalized display value for the property
		this._prop.displayValue = displayValue;
	},

	getValue: function() {		
		// summary:
		//		Returns the actual value object for the property
		if (!this._value && this._prop.value !== null && this._prop.value !== undefined) {
			var valueObj = this._prop.value;
			
			switch (this.getValueType()) {
				case "date":
				case "datetime":
				case "time": 
					if (typeof valueObj === "number") {
						this._value = new Date(valueObj);
					}
					else if (typeof valueObj === "string") {
						this._value = dojo.date.stamp.fromISOString(valueObj);										
					}
					break;
				
				case "date[]":
				case "datetime[]":
				case "time[]":
					var dates = [];
					var length = valueObj.length;
					for (var index = 0;  index < length;  ++index) {
						if (typeof valueObj[index] === "number") {
							dates[index] = new Date(valueObj[index]);
						}
						else if (typeof valueObj[index] === "string") {
							dates[index] = dojo.date.stamp.fromISOString(valueObj[index]);						
						}										
					}
					this._value = dates;
					break;
				
				case "status":
					// full status (i.e. was not optimize)
					if (valueObj.label && valueObj.image) {
						this._value = new ibm.tivoli.navmodel.Status({status: valueObj, version: this._dataset.getVersion()});
					}
					// optimized status; look up with dataset
					else {
						var statusArgs = {
							type: this._objectType,
							statusId: valueObj.id,
							statusValue: valueObj.value				
						};
						this._value = this._dataset.getStatus(statusArgs);
					}
					break;
					
				default:
					this._value = valueObj;
			}
		}
		
		return this._value;
	},
	
	/*=====
	getStatusValue_kwArgs: function(widgetDef) {
		// widgetDef: Object
		//		WidgetDefinition to use for the merge		
	},
	=====*/
	
	getStatusValue: function(/*getStatusValue_kwArgs*/ kwArgs) {		
		// summary:
		//		Returns the "status" value object for the property but
		//		also merges in information from widget definition
		var args = {
			widgetDef: null,
			objectType: this._objectType
		};
		dojo.mixin(args, kwArgs);
		
		var status = null;
		if (this.getValueType() === "status") {
			status = this.getValue();
			if (status && args.widgetDef) {
				status = status.clone(args.widgetDef, args.objectType);
			}
		}
		
		return status;
	},

	setValue: function(value) {		
		// summary:
		//		Modifies the value object for the property
		this._value = value;
				
		switch (this.getValueType()) {
			case "date":
			case "datetime":
			case "time": 
				this._prop.value = value.getTime();
				break;
				
			case "date[]":
			case "datetime[]":
			case "time[]":
				this._prop.value = [];
				var length = value.length;
				for (var index = 0;  index < length;  ++index) {
					this._prop.value[index] = value[index].getTime();
				}
				break;
				
			default:
				this._prop.value = value;
		}
	},

	getValueType: function() {		
		// summary:
		//		Returns the type associated with the property value
		//		|	values:	"itemName", "itemType", "linkName", "linkType", "relationshipName", "relationshipType"
		//		|		"string", "string[]",
		//		|		"byte", "short", "int", "long", "byte[]", "short[]", "int[]", "long[]",
		//		|		"ubyte, "ushort", "uint", "ulong", "ubyte[]", "ushort[]", "uint[]", "ulong[]",	
		//		|		"boolean", "boolean[]"	
		//		|		"enum", "map", "json"
		//		|		"status",
		//		|		"date", "datetime", "time", "date[]", "datetime[]", "time[]"
		//		|		"isodate", isodatetime", "isotime", "isodate[]", "isodatetime[], "isotime[]", "isointerval", "isoduration"
		//		|		"float", "double", "percentage", "float[]", "double[]"
		return this._prop.valueType;
	},

	setValueType: function(valueType) {		
		// summary:
		//		Set the type associated with the property value
		this._prop.valueType = valueType;
	},

	getValueState: function() {		
		// summary:
		//		Returns the state assoicated with the property value
		//		|	values:	"ok", "none", "pending", "error"
		return this._prop.valueState ? this._prop.valueState : "ok";
	},

	setValueState: function(valueState) {		
		// summary:
		//		Set the state associated with the property value
		this._prop.valueState = valueState;
	},

	hasMetadata: function() {		
		// summary:
		//		Returns true if the metadata is present in this property object
		return (this._prop.sortWeight !== undefined);
	},

	getValueUnits: function() {		
		// summary:
		//		Returns a units qualifier that can be assoicated with the property value
		//		example:  "seconds", "hours", "mph"
		return this._prop.units;
	},

	setValueUnits: function(units) {		
		// summary:
		//		Set units qualifier
		this._prop.units = units;
	},

	getValueMin: function() {		
		// summary:
		//		Returns minimum value;  for string properties this is minimum length
		return this._getMetadataObj(this._prop.valueMin);
	},

	setValueMin: function(valueMin) {		
		// summary:
		//		Set minimum value
		this._prop.valueMin = valueMin;
	},

	getValueMax: function() {		
		// summary:
		//		Returns maximum value;  for string properties this is maximum length
		return this._getMetadataObj(this._prop.valueMax);
	},

	setValueMax: function(valueMax) {		
		// summary:
		//		Set maximum value
		this._prop.valueMax = valueMax;
	},

	getValueDefault: function() {		
		// summary:
		//		Returns default value for property (i.e. if has no value and 
		//		a set defaults operation is chosen
		return this._getMetadataObj(this._prop.valueDefault);
	},

	setValueDefault: function(valueDefault) {		
		// summary:
		//		Set default value
		this._prop.valueDefault = valueDefault;
	},

	getSortWeight: function() {		
		// summary:
		//		Returns the sort weight for the property;  when sorting a diverse list of properties
		//		the sortWeight can be used to determine sort order.  Properties with equal sortWeight
		//		should be sorted alphabetically by the label.
		return this._prop.sortWeight;
	},

	setSortWeight: function(sortWeight) {		
		// summary:
		//		Set the sort weight
		this._prop.sortWeight = sortWeight;
	},

	isHidden: function() {		
		// summary:
		//		Returns true if the property should be hidden from the user
		return (this._prop.hidden !== undefined ? this._prop.hidden : false);
	},
	
	setHidden: function(hidden) {		
		this._prop.hidden = hidden;
	},

	isEditable: function() {		
		// summary:
		//		Returns true if the property is editable by the user
		return (this._prop.editable !== undefined ? this._prop.editable : true);
	},
	
	setEditable: function(editable) {
		this._prop.editable = editable;
	},

	isRequired: function() {		
		// summary:
		//		Returns true if the property is required (i.e. must have a value if edited)
		return (this._prop.required !== undefined ? this._prop.required : false);
	},
	
	setRequired: function(required) {
		this._prop.required = required;
	},

	isSensitive: function() {		
		// summary:
		//		Returns true if the property is sensitive and should not 
		//		be displayed in the clear
		return (this._prop.sensitive !== undefined ? this._prop.sensitive : false);
	},
	
	setSensitivie: function(sensitive) {
		this._prop.sensitive = sensitive;
	},
	
	getTask: function() {
		if (!this._task && this._prop.task) {
			this._task = new ibm.tivoli.navmodel.Task({task: this._prop.task, dataset: this._dataset});
		}
		
		return this._task;
	},
	
	setTask: function(task) {
		this._task = task;
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////

	_getMetadataObj: function(obj) {		
		// summary:
		//		Makes necessary conversion on objects for
		//		metadata functions like min,max,default
		if (obj) {
			switch (this.getValueType()) {
				case "date":
				case "datetime":
				case "time":
				case "date[]":
				case "datetime[]":
				case "time[]":
					if (typeof obj === "number") {
						return new Date(obj);
					}
					else if (typeof obj === "string") {
						return dojo.date.stamp.fromISOString(obj);						
					}									
			}		
		}
		
		return obj;
	}
});

});

},
'dojo/colors':function(){
define(["./_base/kernel", "./_base/lang", "./_base/Color", "./_base/array"], function(dojo, lang, Color, ArrayUtil) {
	// module:
	//		dojo/colors
	// summary:
	//		TODOC

	var ColorExt = lang.getObject("dojo.colors", true);

//TODO: this module appears to break naming conventions

/*=====
	lang.mixin(dojo, {
		colors: {
			// summary: Color utilities, extending Base dojo.Color
		}
	});
=====*/

	// this is a standard conversion prescribed by the CSS3 Color Module
	var hue2rgb = function(m1, m2, h){
		if(h < 0){ ++h; }
		if(h > 1){ --h; }
		var h6 = 6 * h;
		if(h6 < 1){ return m1 + (m2 - m1) * h6; }
		if(2 * h < 1){ return m2; }
		if(3 * h < 2){ return m1 + (m2 - m1) * (2 / 3 - h) * 6; }
		return m1;
	};
	// Override base Color.fromRgb with the impl in this module
	dojo.colorFromRgb = Color.fromRgb = function(/*String*/ color, /*dojo.Color?*/ obj){
		// summary:
		//		get rgb(a) array from css-style color declarations
		// description:
		//		this function can handle all 4 CSS3 Color Module formats: rgb,
		//		rgba, hsl, hsla, including rgb(a) with percentage values.
		var m = color.toLowerCase().match(/^(rgba?|hsla?)\(([\s\.\-,%0-9]+)\)/);
		if(m){
			var c = m[2].split(/\s*,\s*/), l = c.length, t = m[1], a;
			if((t == "rgb" && l == 3) || (t == "rgba" && l == 4)){
				var r = c[0];
				if(r.charAt(r.length - 1) == "%"){
					// 3 rgb percentage values
					a = ArrayUtil.map(c, function(x){
						return parseFloat(x) * 2.56;
					});
					if(l == 4){ a[3] = c[3]; }
					return Color.fromArray(a, obj); // dojo.Color
				}
				return Color.fromArray(c, obj); // dojo.Color
			}
			if((t == "hsl" && l == 3) || (t == "hsla" && l == 4)){
				// normalize hsl values
				var H = ((parseFloat(c[0]) % 360) + 360) % 360 / 360,
					S = parseFloat(c[1]) / 100,
					L = parseFloat(c[2]) / 100,
					// calculate rgb according to the algorithm
					// recommended by the CSS3 Color Module
					m2 = L <= 0.5 ? L * (S + 1) : L + S - L * S,
					m1 = 2 * L - m2;
				a = [
					hue2rgb(m1, m2, H + 1 / 3) * 256,
					hue2rgb(m1, m2, H) * 256,
					hue2rgb(m1, m2, H - 1 / 3) * 256,
					1
				];
				if(l == 4){ a[3] = c[3]; }
				return Color.fromArray(a, obj); // dojo.Color
			}
		}
		return null;	// dojo.Color
	};

	var confine = function(c, low, high){
		// summary:
		//		sanitize a color component by making sure it is a number,
		//		and clamping it to valid values
		c = Number(c);
		return isNaN(c) ? high : c < low ? low : c > high ? high : c;	// Number
	};

	Color.prototype.sanitize = function(){
		// summary: makes sure that the object has correct attributes
		var t = this;
		t.r = Math.round(confine(t.r, 0, 255));
		t.g = Math.round(confine(t.g, 0, 255));
		t.b = Math.round(confine(t.b, 0, 255));
		t.a = confine(t.a, 0, 1);
		return this;	// dojo.Color
	};

	ColorExt.makeGrey = Color.makeGrey = function(/*Number*/ g, /*Number?*/ a){
		// summary: creates a greyscale color with an optional alpha
		return Color.fromArray([g, g, g, a]);	// dojo.Color
	};

	// mixin all CSS3 named colors not already in _base, along with SVG 1.0 variant spellings
	lang.mixin(Color.named, {
		"aliceblue":	[240,248,255],
		"antiquewhite": [250,235,215],
		"aquamarine":	[127,255,212],
		"azure":	[240,255,255],
		"beige":	[245,245,220],
		"bisque":	[255,228,196],
		"blanchedalmond":	[255,235,205],
		"blueviolet":	[138,43,226],
		"brown":	[165,42,42],
		"burlywood":	[222,184,135],
		"cadetblue":	[95,158,160],
		"chartreuse":	[127,255,0],
		"chocolate":	[210,105,30],
		"coral":	[255,127,80],
		"cornflowerblue":	[100,149,237],
		"cornsilk": [255,248,220],
		"crimson":	[220,20,60],
		"cyan": [0,255,255],
		"darkblue": [0,0,139],
		"darkcyan": [0,139,139],
		"darkgoldenrod":	[184,134,11],
		"darkgray": [169,169,169],
		"darkgreen":	[0,100,0],
		"darkgrey": [169,169,169],
		"darkkhaki":	[189,183,107],
		"darkmagenta":	[139,0,139],
		"darkolivegreen":	[85,107,47],
		"darkorange":	[255,140,0],
		"darkorchid":	[153,50,204],
		"darkred":	[139,0,0],
		"darksalmon":	[233,150,122],
		"darkseagreen": [143,188,143],
		"darkslateblue":	[72,61,139],
		"darkslategray":	[47,79,79],
		"darkslategrey":	[47,79,79],
		"darkturquoise":	[0,206,209],
		"darkviolet":	[148,0,211],
		"deeppink": [255,20,147],
		"deepskyblue":	[0,191,255],
		"dimgray":	[105,105,105],
		"dimgrey":	[105,105,105],
		"dodgerblue":	[30,144,255],
		"firebrick":	[178,34,34],
		"floralwhite":	[255,250,240],
		"forestgreen":	[34,139,34],
		"gainsboro":	[220,220,220],
		"ghostwhite":	[248,248,255],
		"gold": [255,215,0],
		"goldenrod":	[218,165,32],
		"greenyellow":	[173,255,47],
		"grey": [128,128,128],
		"honeydew": [240,255,240],
		"hotpink":	[255,105,180],
		"indianred":	[205,92,92],
		"indigo":	[75,0,130],
		"ivory":	[255,255,240],
		"khaki":	[240,230,140],
		"lavender": [230,230,250],
		"lavenderblush":	[255,240,245],
		"lawngreen":	[124,252,0],
		"lemonchiffon": [255,250,205],
		"lightblue":	[173,216,230],
		"lightcoral":	[240,128,128],
		"lightcyan":	[224,255,255],
		"lightgoldenrodyellow": [250,250,210],
		"lightgray":	[211,211,211],
		"lightgreen":	[144,238,144],
		"lightgrey":	[211,211,211],
		"lightpink":	[255,182,193],
		"lightsalmon":	[255,160,122],
		"lightseagreen":	[32,178,170],
		"lightskyblue": [135,206,250],
		"lightslategray":	[119,136,153],
		"lightslategrey":	[119,136,153],
		"lightsteelblue":	[176,196,222],
		"lightyellow":	[255,255,224],
		"limegreen":	[50,205,50],
		"linen":	[250,240,230],
		"magenta":	[255,0,255],
		"mediumaquamarine": [102,205,170],
		"mediumblue":	[0,0,205],
		"mediumorchid": [186,85,211],
		"mediumpurple": [147,112,219],
		"mediumseagreen":	[60,179,113],
		"mediumslateblue":	[123,104,238],
		"mediumspringgreen":	[0,250,154],
		"mediumturquoise":	[72,209,204],
		"mediumvioletred":	[199,21,133],
		"midnightblue": [25,25,112],
		"mintcream":	[245,255,250],
		"mistyrose":	[255,228,225],
		"moccasin": [255,228,181],
		"navajowhite":	[255,222,173],
		"oldlace":	[253,245,230],
		"olivedrab":	[107,142,35],
		"orange":	[255,165,0],
		"orangered":	[255,69,0],
		"orchid":	[218,112,214],
		"palegoldenrod":	[238,232,170],
		"palegreen":	[152,251,152],
		"paleturquoise":	[175,238,238],
		"palevioletred":	[219,112,147],
		"papayawhip":	[255,239,213],
		"peachpuff":	[255,218,185],
		"peru": [205,133,63],
		"pink": [255,192,203],
		"plum": [221,160,221],
		"powderblue":	[176,224,230],
		"rosybrown":	[188,143,143],
		"royalblue":	[65,105,225],
		"saddlebrown":	[139,69,19],
		"salmon":	[250,128,114],
		"sandybrown":	[244,164,96],
		"seagreen": [46,139,87],
		"seashell": [255,245,238],
		"sienna":	[160,82,45],
		"skyblue":	[135,206,235],
		"slateblue":	[106,90,205],
		"slategray":	[112,128,144],
		"slategrey":	[112,128,144],
		"snow": [255,250,250],
		"springgreen":	[0,255,127],
		"steelblue":	[70,130,180],
		"tan":	[210,180,140],
		"thistle":	[216,191,216],
		"tomato":	[255,99,71],
		"turquoise":	[64,224,208],
		"violet":	[238,130,238],
		"wheat":	[245,222,179],
		"whitesmoke":	[245,245,245],
		"yellowgreen":	[154,205,50]
	});

	return Color;
});

},
'dojo/dnd/move':function(){
define(["../main", "./Mover", "./Moveable"], function(dojo) {
	// module:
	//		dojo/dnd/move
	// summary:
	//		TODOC


/*=====
dojo.declare("dojo.dnd.move.__constrainedMoveableArgs", [dojo.dnd.__MoveableArgs], {
	// constraints: Function
	//		Calculates a constraint box.
	//		It is called in a context of the moveable object.
	constraints: function(){},

	// within: Boolean
	//		restrict move within boundaries.
	within: false
});
=====*/

dojo.declare("dojo.dnd.move.constrainedMoveable", dojo.dnd.Moveable, {
	// object attributes (for markup)
	constraints: function(){},
	within: false,

	constructor: function(node, params){
		// summary:
		//		an object that makes a node moveable
		// node: Node
		//		a node (or node's id) to be moved
		// params: dojo.dnd.move.__constrainedMoveableArgs?
		//		an optional object with additional parameters;
		//		the rest is passed to the base class
		if(!params){ params = {}; }
		this.constraints = params.constraints;
		this.within = params.within;
	},
	onFirstMove: function(/* dojo.dnd.Mover */ mover){
		// summary:
		//		called during the very first move notification;
		//		can be used to initialize coordinates, can be overwritten.
		var c = this.constraintBox = this.constraints.call(this, mover);
		c.r = c.l + c.w;
		c.b = c.t + c.h;
		if(this.within){
			var mb = dojo._getMarginSize(mover.node);
			c.r -= mb.w;
			c.b -= mb.h;
		}
	},
	onMove: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop){
		// summary:
		//		called during every move notification;
		//		should actually move the node; can be overwritten.
		var c = this.constraintBox, s = mover.node.style;
		this.onMoving(mover, leftTop);
		leftTop.l = leftTop.l < c.l ? c.l : c.r < leftTop.l ? c.r : leftTop.l;
		leftTop.t = leftTop.t < c.t ? c.t : c.b < leftTop.t ? c.b : leftTop.t;
		s.left = leftTop.l + "px";
		s.top  = leftTop.t + "px";
		this.onMoved(mover, leftTop);
	}
});

/*=====
dojo.declare("dojo.dnd.move.__boxConstrainedMoveableArgs", [dojo.dnd.move.__constrainedMoveableArgs], {
	// box: Object
	//		a constraint box
	box: {}
});
=====*/

dojo.declare("dojo.dnd.move.boxConstrainedMoveable", dojo.dnd.move.constrainedMoveable, {
	// box:
	//		object attributes (for markup)
	box: {},

	constructor: function(node, params){
		// summary:
		//		an object, which makes a node moveable
		// node: Node
		//		a node (or node's id) to be moved
		// params: dojo.dnd.move.__boxConstrainedMoveableArgs?
		//		an optional object with parameters
		var box = params && params.box;
		this.constraints = function(){ return box; };
	}
});

/*=====
dojo.declare("dojo.dnd.move.__parentConstrainedMoveableArgs", [dojo.dnd.move.__constrainedMoveableArgs], {
	// area: String
	//		A parent's area to restrict the move.
	//		Can be "margin", "border", "padding", or "content".
	area: ""
});
=====*/

dojo.declare("dojo.dnd.move.parentConstrainedMoveable", dojo.dnd.move.constrainedMoveable, {
	// area:
	//		object attributes (for markup)
	area: "content",

	constructor: function(node, params){
		// summary:
		//		an object, which makes a node moveable
		// node: Node
		//		a node (or node's id) to be moved
		// params: dojo.dnd.move.__parentConstrainedMoveableArgs?
		//		an optional object with parameters
		var area = params && params.area;
		this.constraints = function(){
			var n = this.node.parentNode,
				s = dojo.getComputedStyle(n),
				mb = dojo._getMarginBox(n, s);
			if(area == "margin"){
				return mb;	// Object
			}
			var t = dojo._getMarginExtents(n, s);
			mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
			if(area == "border"){
				return mb;	// Object
			}
			t = dojo._getBorderExtents(n, s);
			mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
			if(area == "padding"){
				return mb;	// Object
			}
			t = dojo._getPadExtents(n, s);
			mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
			return mb;	// Object
		};
	}
});

// patching functions one level up for compatibility

dojo.dnd.constrainedMover = dojo.dnd.move.constrainedMover;
dojo.dnd.boxConstrainedMover = dojo.dnd.move.boxConstrainedMover;
dojo.dnd.parentConstrainedMover = dojo.dnd.move.parentConstrainedMover;

return dojo.dnd.move;
});

},
'url:ibm/usmi/widgets/templates/DetailsSelector.html':"<div class=\"detailsSelector maxSize\">\r\n<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n\t<div id=\"detailsSelectorContentPane\" style=\"overflow-y: auto;width: 98%;height: 100%;\" class=\"detailsSelectorMain\" data-dojo-type=\"dijit.layout.ContentPane\">\r\n\t\t<!-- Top & Rounded Corners -->\r\n\t    <div class=\"detailsSelectorTop\">\r\n\t        <div class=\"detailsSelectorTopLeft\"></div>\r\n\t        <div class=\"detailsSelectorTopCenter\"></div>\r\n\t        <div class=\"detailsSelectorTopRight\"></div>\r\n\t    </div>\r\n\t    \r\n\t    <!-- Center where all the good stuff goes -->   \r\n\t    <div class=\"detailsSelectorCenter\">\r\n            <div class=\"detailsSelectorCenterLeft\"></div>\r\n\t\t    <div class=\"detailsSelectorCenterCenter\">\r\n\t\t\t\t<!-- Main Image, High level properties and Actions Button -->\r\n\t    \t\t<div class=\"detailsSelectorTopPane\" data-dojo-attach-point=\"topPane\">\r\n\t    \t\t    <!-- Main Image -->\r\n\t\t\t\t\t<div id=\"TopImage\" class=\"detailsSelectorImage\">\r\n\t\t\t\t    \t<img class=\"mainImage\" data-dojo-attach-point=\"resourceImage\"></img>\r\n\t\t\t\t  \t</div>\r\n\t\t\t\t  \t<!-- Actions Dropdown Button -->\r\n\t\t\t\t  \t<div id=\"ActionsButton\" dojoAttachPoint=\"buttonDiv\" class=\"detailsButton pad5 hide\">\r\n\t\t\t\t      \t<button id=\"actBtn\" data-dojo-type=\"dijit.form.DropDownButton\" data-dojo-attach-point=\"actionListBtn\">\r\n\t\t\t\t          \t<span>${actionLabel}</span>\t          \t\r\n\t\t\t\t      \t</button>\r\n\t\t\t\t  \t</div>\t\t\t\t  \t\r\n\t\t\t\t  \t<!-- High Level Properties -->\r\n\t\t\t\t    <div class=\"detailsSelectorLabel pad15b\" role=\"heading\" aria-level=\"1\">\r\n\t\t\t\t        <div data-dojo-attach-point=\"propertiesList\"></div>\r\n\t\t\t\t    </div>\r\n\t\t\t\t\t\t\t\t\t\t\r\n\t\t\t\t</div>\r\n\t\t\t\t<!-- Tasks Section, categories and tasks -->\r\n\t\t\t\t<div id=\"tPane\" style=\"position: relative;\" class=\"detailsSelectorTaskPane\" data-dojo-type=\"ibm.usmi.widgets.TaskPane\" data-dojo-attach-point=\"taskPane\"></div>\r\n\t\t    </div>\r\n\t\t    <div class=\"detailsSelectorCenterRight\"></div>\r\n\t    </div>\r\n\t    \r\n\t    <!-- Bottom & Rounded Corners -->\r\n\t    <div class=\"detailsSelectorBottom\">\r\n\t        <div class=\"detailsSelectorBottomLeft\"></div>\r\n\t        <div class=\"detailsSelectorBottomCenter\"></div>\r\n\t        <div class=\"detailsSelectorBottomRight\"></div>\r\n\t    </div>\t\t  \r\n\t</div>\r\n</div>",
'dijit/PopupMenuBarItem':function(){
define([
	"dojo/_base/declare", // declare
	"./PopupMenuItem",
	"./MenuBarItem"
], function(declare, PopupMenuItem, MenuBarItem){

	// module:
	//		dijit/PopupMenuBarItem
	// summary:
	//		Item in a MenuBar like "File" or "Edit", that spawns a submenu when pressed (or hovered)

	var _MenuBarItemMixin = MenuBarItem._MenuBarItemMixin;

/*=====
	var PopupMenuItem = dijit.PopupMenuItem;
	var _MenuBarItemMixin = dijit._MenuBarItemMixin;
=====*/

	return declare("dijit.PopupMenuBarItem", [PopupMenuItem, _MenuBarItemMixin], {
		// summary:
		//		Item in a MenuBar like "File" or "Edit", that spawns a submenu when pressed (or hovered)
	});
});

},
'ibm/usmi/widgets/table/DefaultTreeTableView':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define([
	"dojo/_base/declare",
	"dojo/_base/kernel",	
	"dojo/on",
	"dojo/_base/lang",
	"dojo/_base/array",
	"dojo/aspect",	
	"ibm/tivoli/tip/navwidget/tree/dijit/TreeTableView",
	"ibm/usmi/widgets/table/ConfigureColumnsDialog",
	"ibm/usmi/core/utils/Logger",
	"ibm/usmi/widgets/table/DefaultTablePersistenceMixin"
], function (declare, dojo, on, lang, array, aspect, TreeTableView, ConfigureColumnsDialog, 
		Logger, DefaultTablePersistenceMixin) {

	return declare([ TreeTableView //, DefaultTablePersistenceMixin
	], {

		// the logger
		_defaultTreeTableViewLogger : new Logger("ibm.usmi.widgets.table", "DefaultTreeTableView"),
		
		//@188637 START	uncomment to enable tree table persistence		
//		augmentGridParams: function (gridParams) {
//			gridParams = this.inherited(arguments);
//			gridParams = this.persistenceGridParams(this._dataStore, gridParams);
//			return gridParams;
//		},
		//@188637 START	

		// AbstractTableView._initTableBilder connects to onTableLoad, and inadvertently shows the filter bar.
		// This aspect is called afterward, and hides the filter bar again.
		_initTableBuilder : function () {
			this.inherited(arguments);		

			//@188637 START	uncomment to enable tree table persistence			
//			if (this._table) {
//				this._table.setFilter(null, true);
//				var searchField = null;
//				if (this._table._toolbar && this._table._toolbar.quickFilter) {
//					searchField = this._table._toolbar.quickFilter;
//				}
//				this.initPersistence(this, this._grid, searchField);
//			}
			//@188637 END
			
			var changeStyle = function (row) {
				if (!this.layout._isCollapsable) {
					this.inherited(arguments);
					return;
				}
				row.customClasses = (row.selected ? " SelectorTreedojoxGridRowSelected" : "")
						+ (row.over ? " SelectorTreedojoxGridRowOver" : "");
				this.focus.styleRow(row);
				this.edit.styleRow(row);
			};
			on(this._table._grid, "styleRow", changeStyle);

			//Hack in our column chooser
			if (this._table && this._table._toolbar) {
				//console.log("setting the custom Column Chooser");
				this._table._toolbar._configureColumn = function () {
					this.configureTableDialog = new ConfigureColumnsDialog({
						grid : this.treeTable._grid
					});
					this.configureTableDialog.show();
				};
			}
		},
				
		handleLinkClickAction : function (viewId, objectId, propId, objectType) {
			// summary:
			//		Method to override base implementation so we get notified and can
			//      handle when a link is clicked in a table cell
			if (this._defaultTreeTableViewLogger.isFinest()) {
				this._defaultTreeTableViewLogger.entering("", "handleLinkClickAction");
				this._defaultTreeTableViewLogger.finest("handling link click for table view - prop: " + propId,
						"handleLinkClickAction");
			}
			var object, property;

			//ask the dataset for the correct object based on the objects id.
			//since we have already rendered these items, the value should be
			//cached and should be quick.
			switch (objectType) {
			case "ibm.tivoli.navmodel.Item":
				object = this.dataset.getItem({
					itemId : objectId
				});
				break;
			case "ibm.tivoli.navmodel.Link":
				object = this.dataset.getLink({
					itemId : objectId
				});
				break;
			case "ibm.tivoli.navmodel.Relationship":
				object = this.dataset.getRelationship({
					itemId : objectId
				});
				break;
			default:
				if (this._defaultTreeTableViewLogger.isFinest()) {
					this._defaultTreeTableViewLogger.finest("unknown objectType: " + objectType,
							"handleLinkClickAction");
				}
			}

			if (object) {
				property = object.getProperty(propId);
			}

			if (object && property) {
				this.daHelper.executeDefaultAction(this, property);
			}

			if (this._defaultTreeTableViewLogger.isFinest()) {
				this._defaultTreeTableViewLogger.exiting("", "handleLinkClickAction");
			}
		}
	});
});

},
'url:dijit/templates/MenuItem.html':"<tr class=\"dijitReset dijitMenuItem\" data-dojo-attach-point=\"focusNode\" role=\"menuitem\" tabIndex=\"-1\"\n\t\tdata-dojo-attach-event=\"onmouseenter:_onHover,onmouseleave:_onUnhover,ondijitclick:_onClick\">\n\t<td class=\"dijitReset dijitMenuItemIconCell\" role=\"presentation\">\n\t\t<img src=\"${_blankGif}\" alt=\"\" class=\"dijitIcon dijitMenuItemIcon\" data-dojo-attach-point=\"iconNode\"/>\n\t</td>\n\t<td class=\"dijitReset dijitMenuItemLabel\" colspan=\"2\" data-dojo-attach-point=\"containerNode\"></td>\n\t<td class=\"dijitReset dijitMenuItemAccelKey\" style=\"display: none\" data-dojo-attach-point=\"accelKeyNode\"></td>\n\t<td class=\"dijitReset dijitMenuArrowCell\" role=\"presentation\">\n\t\t<div data-dojo-attach-point=\"arrowWrapper\" style=\"visibility: hidden\">\n\t\t\t<img src=\"${_blankGif}\" alt=\"\" class=\"dijitMenuExpand\"/>\n\t\t\t<span class=\"dijitMenuExpandA11y\">+</span>\n\t\t</div>\n\t</td>\n</tr>\n",
'dijit/form/TextBox':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom-construct", // domConstruct.create
	"dojo/dom-style", // domStyle.getComputedStyle
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/lang", // lang.hitch
	"dojo/_base/sniff", // has("ie") has("mozilla")
	"dojo/_base/window", // win.doc.selection.createRange
	"./_FormValueWidget",
	"./_TextBoxMixin",
	"dojo/text!./templates/TextBox.html",
	".."	// to export dijit._setSelectionRange, remove in 2.0
], function(declare, domConstruct, domStyle, kernel, lang, has, win,
			_FormValueWidget, _TextBoxMixin, template, dijit){

/*=====
	var _FormValueWidget = dijit.form._FormValueWidget;
	var _TextBoxMixin = dijit.form._TextBoxMixin;
=====*/

	// module:
	//		dijit/form/TextBox
	// summary:
	//		A base class for textbox form inputs

	var TextBox = declare(/*====="dijit.form.TextBox", =====*/ [_FormValueWidget, _TextBoxMixin], {
		// summary:
		//		A base class for textbox form inputs

		templateString: template,
		_singleNodeTemplate: '<input class="dijit dijitReset dijitLeft dijitInputField" data-dojo-attach-point="textbox,focusNode" autocomplete="off" type="${type}" ${!nameAttrSetting} />',

		_buttonInputDisabled: has("ie") ? "disabled" : "", // allows IE to disallow focus, but Firefox cannot be disabled for mousedown events

		baseClass: "dijitTextBox",

		postMixInProperties: function(){
			var type = this.type.toLowerCase();
			if(this.templateString && this.templateString.toLowerCase() == "input" || ((type == "hidden" || type == "file") && this.templateString == this.constructor.prototype.templateString)){
				this.templateString = this._singleNodeTemplate;
			}
			this.inherited(arguments);
		},

		_onInput: function(e){
			this.inherited(arguments);
			if(this.intermediateChanges){ // _TextBoxMixin uses onInput
				var _this = this;
				// the setTimeout allows the key to post to the widget input box
				setTimeout(function(){ _this._handleOnChange(_this.get('value'), false); }, 0);
			}
		},

		_setPlaceHolderAttr: function(v){
			this._set("placeHolder", v);
			if(!this._phspan){
				this._attachPoints.push('_phspan');
				// dijitInputField class gives placeHolder same padding as the input field
				// parent node already has dijitInputField class but it doesn't affect this <span>
				// since it's position: absolute.
				this._phspan = domConstruct.create('span',{className:'dijitPlaceHolder dijitInputField'},this.textbox,'after');
			}
			this._phspan.innerHTML="";
			this._phspan.appendChild(document.createTextNode(v));
			this._updatePlaceHolder();
		},

		_updatePlaceHolder: function(){
			if(this._phspan){
				this._phspan.style.display=(this.placeHolder&&!this.focused&&!this.textbox.value)?"":"none";
			}
		},

		_setValueAttr: function(value, /*Boolean?*/ priorityChange, /*String?*/ formattedValue){
			this.inherited(arguments);
			this._updatePlaceHolder();
		},

		getDisplayedValue: function(){
			// summary:
			//		Deprecated.  Use get('displayedValue') instead.
			// tags:
			//		deprecated
			kernel.deprecated(this.declaredClass+"::getDisplayedValue() is deprecated. Use set('displayedValue') instead.", "", "2.0");
			return this.get('displayedValue');
		},

		setDisplayedValue: function(/*String*/ value){
			// summary:
			//		Deprecated.  Use set('displayedValue', ...) instead.
			// tags:
			//		deprecated
			kernel.deprecated(this.declaredClass+"::setDisplayedValue() is deprecated. Use set('displayedValue', ...) instead.", "", "2.0");
			this.set('displayedValue', value);
		},

		_onBlur: function(e){
			if(this.disabled){ return; }
			this.inherited(arguments);
			this._updatePlaceHolder();
		},

		_onFocus: function(/*String*/ by){
			if(this.disabled || this.readOnly){ return; }
			this.inherited(arguments);
			this._updatePlaceHolder();
		}
	});

	if(has("ie")){
		TextBox = declare(/*===== "dijit.form.TextBox.IEMixin", =====*/ TextBox, {
			declaredClass: "dijit.form.TextBox",	// for user code referencing declaredClass

			_isTextSelected: function(){
				var range = win.doc.selection.createRange();
				var parent = range.parentElement();
				return parent == this.textbox && range.text.length == 0;
			},

			postCreate: function(){
				this.inherited(arguments);
				// IE INPUT tag fontFamily has to be set directly using STYLE
				// the setTimeout gives IE a chance to render the TextBox and to deal with font inheritance
				setTimeout(lang.hitch(this, function(){
					try{
						var s = domStyle.getComputedStyle(this.domNode); // can throw an exception if widget is immediately destroyed
						if(s){
							var ff = s.fontFamily;
							if(ff){
								var inputs = this.domNode.getElementsByTagName("INPUT");
								if(inputs){
									for(var i=0; i < inputs.length; i++){
										inputs[i].style.fontFamily = ff;
									}
								}
							}
						}
					}catch(e){/*when used in a Dialog, and this is called before the dialog is
						shown, s.fontFamily would trigger "Invalid Argument" error.*/}
				}), 0);
			}
		});

		// Overrides definition of _setSelectionRange from _TextBoxMixin (TODO: move to _TextBoxMixin.js?)
		dijit._setSelectionRange = _TextBoxMixin._setSelectionRange = function(/*DomNode*/ element, /*Number?*/ start, /*Number?*/ stop){
			if(element.createTextRange){
				var r = element.createTextRange();
				r.collapse(true);
				r.moveStart("character", -99999); // move to 0
				r.moveStart("character", start); // delta from 0 is the correct position
				r.moveEnd("character", stop-start);
				r.select();
			}
		}
	}else if(has("mozilla")){
		TextBox = declare(/*===== "dijit.form.TextBox.MozMixin", =====*/TextBox, {
			declaredClass: "dijit.form.TextBox",	// for user code referencing declaredClass

			_onBlur: function(e){
				this.inherited(arguments);
				if(this.selectOnClick){
						// clear selection so that the next mouse click doesn't reselect
					this.textbox.selectionStart = this.textbox.selectionEnd = undefined;
				}
			}
		});
	}else{
		TextBox.prototype.declaredClass = "dijit.form.TextBox";
	}
	lang.setObject("dijit.form.TextBox", TextBox);	// don't do direct assignment, it confuses API doc parser

	return TextBox;
});

},
'dojoe/wizard/WizardToCSlider':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!dojoe/wizard/Wizard,dijit/_WidgetBase"], function(dijit,dojo,dojox){
dojo.provide("dojoe.wizard.WizardToCSlider");

dojo.require("dojoe.wizard.Wizard");
dojo.require("dijit._WidgetBase");

/**
 * Slider support for horizontal navigation in Wizard.
 */
 
dojo.declare("dojoe.wizard.WizardToCSlider", [dijit._WidgetBase], 
	{
		toc: null,        // ordered list for ToC
		tocSteps: null,   // div containing toc main steps
		ltr: true, 
		tocStepsMargin: 10,
		tocMarginLeft: 2,
		oldTocStepsWidth: 0,
		totalWidthToCItems: 0,
		oneButtonWidth: 29,
		visibleButtonWidth: 0,  // width plus margin of visible buttons 
		sliderOuterDiv: null,
		sliderLeft: null,
		sliderRight: null,
		sliderLeftImg: dojo.moduleUrl("dojoe.common", "themes/claro/images/slider_left.png"),
		sliderRightImg: dojo.moduleUrl("dojoe.common", "themes/claro/images/slider_right.png"),
		t: null,
		slideDelay: 2,  // 2 milliseconds
		slidePixel: 2,
		pos: 0,
		slideDirection: "",
		slideRightLabel: "",
		slideLeftLabel: "",
  
		/**
		* Constructor which adds nodes to ToC content pane for the slider, e.g.
		* <div for main steps - passed in>
		*   <img for left slider button />
		*   <div for containing ToC (outer) >
		*     <ol for ToC (inner) - passed in />
		*   </div>
		*   <img for right slider button>
		* </div>
		* 
		* Takes three arguments:
		*   toc       <ol> for ToC
		*   tocSteps  div for main steps
		*   ltr       flag indicating whether container text direction is left-to-right
		*             or not (right-to-left)
		*/
		constructor: function (args) {
			this.toc = args.toc;
			this.tocSteps = args.tocSteps;
			this.ltr = args.ltr;
			// strings for slider buttons
			this.resources = dojo.i18n.getLocalization("dojoe.wizard", "Wizard", this.lang);
			this.slideRightLabel = this.resources.NEXT;
			this.slideLeftLabel = this.resources.PREVIOUS;
			this.sliderOuterDiv = dojo.create("div", { className: "slider-outer" });          
			dojo.place(this.toc, this.sliderOuterDiv, 0);
			dojo.addClass(this.toc, "slider-inner");        
		
			this.sliderLeft = dojo.create("img", {
				src: this._blankGif || this.sliderLeftImg.toString(),
				alt: this.slideLeftLabel,
				title: this.slideLeftLabel,
				className: "slider-left"
			});
			this.sliderRight = dojo.create("img", {
				src: this._blankGif || this.sliderRightImg.toString(),
				alt: this.slideRightLabel,
				title: this.slideRightLabel,
				className: "slider-right"
			});
			dojo.place(this.sliderLeft, this.tocSteps, 0);
			dojo.place(this.sliderOuterDiv, this.tocSteps, 1);
			dojo.place(this.sliderRight, this.tocSteps, 2);
			// initially hide buttons
			dojo.style(this.sliderLeft, "display", "none");        
			dojo.style(this.sliderRight, "display", "none");        
			// connect button handlers
			var slideLeftListener = dojo.connect(this.sliderLeft, "onclick", this, this.slideLeft);
			var slideRightListener = dojo.connect(this.sliderRight, "onclick", this, this.slideRight);
		},
	  
		/**
		* Re-calculate total width of all ToC items (step arrows).
		*/
		recalcToCWidth: function () {
			// With IE arrow image parts may wrap as the arrow styles change.  
			// To ensure we have the correct total width, we temporarily increase
			// the width of the ToC list to prevent any truncation of parts and 
			// then add up the widths of the arrow parts, i.e. front, middle and 
			// tail.
			dojo.style(this.toc, "width", (this.totalWidthToCItems * 2) + "px");  // should be sufficient   
			this.totalWidthToCItems = 0;
			dojo.query("li", this.toc).forEach( function( item ){
				this.totalWidthToCItems += dojo.getMarginBox( item ).w; 
			}, this );			
			
			//dojo.style(this.toc, "width", this.totalWidthToCItems + "px");
			//55135: STG: Wizard usability/layout issues in OneUI
			//Bobby: 12Oct2012, ToC is wrapping and need additional 1px
			if (this.tocMarginLeft > 0) {
				dojo.style(this.toc, "width", (this.totalWidthToCItems + this.tocMarginLeft) + "px");
			} else {
				dojo.style(this.toc, "width", (this.totalWidthToCItems + 2) + "px");
			}
		},
		
		/**
		* Right slider button is made visible if not all step arrows can
		* be displayed in the pane.  Once user clicks on button and slides
		* the arrows, then the left slider button is added.  Both buttons
		* remain visible unless the pane is resized so that all steps can
		* be displayed.
		*/
		updateSliderUI: function (newWidth) {
			// newWidth will be passed in when toc content pane is being resized;
			// use it instead of scrollWidth since that value may not be updated yet
			var slideWidth = (newWidth != null) ? newWidth : dojo.style(this.tocSteps, "width");
			if (this.totalWidthToCItems >= slideWidth - this.tocStepsMargin) {
				if (this.ltr) {
				// Show right slider button only if no buttons currently showing
					if (this.sliderRight.style.display == "none") {
						dojo.style(this.sliderRight, "display", "inline");
						this.visibleButtonWidth = this.oneButtonWidth;        
					} else {
						this.finishSlide(this.sliderLeft, slideWidth);
					}
				} else {
					// Show left slider button only if no buttons currently showing
					if (this.sliderLeft.style.display == "none") {
						dojo.style(this.sliderLeft, "display", "inline");
						this.visibleButtonWidth = this.oneButtonWidth;        
					} else {
						this.finishSlide(this.sliderRight, slideWidth);
					}
				}
				//55564: NaN error causes wizard launch to fail in IE 8
				//Bobby: 17Oct2012, adding check to avoid setting NaN as width
				if (slideWidth >= this.visibleButtonWidth) {
					dojo.style(this.sliderOuterDiv, "width", (slideWidth - this.visibleButtonWidth) + "px");
					this.oldTocStepsWidth = slideWidth;
				}
			} else if (this.visibleButtonWidth > 0) {
				// no need for slider, hide everything
				dojo.style(this.sliderLeft, "display", "none");        
				dojo.style(this.sliderRight, "display", "none");        
				dojo.style(this.sliderOuterDiv, "width", "100%");
				dojo.style(this.toc, "left", "0px");
				this.pos = 0;
				this.slideDirection = "";
				this.visibleButtonWidth = 0;
			}    
		},

		/**
		* Handles case where both slider buttons are visible, toc has been 
		* scrolled so first and last step(s) are no longer visible and window 
		* is being resized.  As the window width is increased, once the last 
		* step is visible the toc needs to be slid so that the first step(s) 
		* become visible too.
		*/
		finishSlide: function (button, slideWidth) {
			var defaultToc = 0;
			if (button.style.display != "none") {
				var tocLeft = dojo.style(this.toc, "left");
				if (this.ltr) {
					// left-to-right direction
					if (tocLeft < 0) {
						var tocWidthShowing = dojo.style(this.toc, "width") + tocLeft;
						if (slideWidth > (tocWidthShowing + this.visibleButtonWidth + this.tocMarginLeft)) {
							this.pos = tocLeft + slideWidth - this.oldTocStepsWidth;
							dojo.style(this.toc, "left", this.pos + "px");
						}                  
					}
				} else {
					// right-to=left direction
					if (tocLeft > 0) {
						var tocWidthShowing1 = defaultToc - (dojo.style(this.toc, "width") - tocLeft);
						if (slideWidth > (tocWidthShowing1 + this.visibleButtonWidth + this.tocMarginLeft)) {
							this.pos = tocLeft - slideWidth + this.oldTocStepsWidth;
							dojo.style(this.toc, "left", this.pos + "px");
						}                            
					}
				}
			}
		},
	  
		/**
		* Called to show the second slider button after first slider button
		* has been clicked.
		*/  
		showSecondButton: function (button, marginStyle, multiplier) {
			this.visibleButtonWidth *= 2;
			this.pos = multiplier * this.oneButtonWidth;
			dojo.style(button, "display", "inline");
			dojo.style(this.toc, "left", this.pos + "px");
			dojo.style(this.sliderOuterDiv, "width", (dojo.style(this.tocSteps, "width") - this.visibleButtonWidth) + "px");
			dojo.style(this.toc, marginStyle, this.tocMarginLeft + "px");    
		},
	  
		/**
		* Returns the number of pixels that have been scrolled due to
		* tabbing.
		*/
		getScrollAmount: function () {
			var scrollAmt = 0;
			if (this.sliderOuterDiv.scrollLeft > 0) { // IE
				scrollAmt = this.sliderOuterDiv.scrollWidth - this.sliderOuterDiv.scrollLeft - this.sliderOuterDiv.offsetWidth;
			} else if (this.sliderOuterDiv.scrollLeft < 0) {  // FF
				scrollAmt = scrollAmt - this.sliderOuterDiv.scrollLeft;
			}
			return scrollAmt;
		},
	  
		/**
		* Invoked when user clicks on right slider button
		*/
		slideRight: function () {
			if (this.slideDirection == "") {
				this.showSecondButton(this.sliderLeft, "marginLeft", -1);
			} else {
				this.slideStop();
			}
			this.slideDirection = "R";
			// determine number of pixels to slide adjusting for any scrolling
			// from tabbing if necessary
			var slideWidth = dojo.style(this.sliderOuterDiv, "width") - this.tocMarginLeft;    
			var leftToSlide;
			if (this.ltr) {
				leftToSlide = this.totalWidthToCItems - slideWidth + this.pos - this.sliderOuterDiv.scrollLeft;
			} else {
				leftToSlide = this.pos + this.getScrollAmount();
			}
			// don't slide too much
			if (leftToSlide > slideWidth) {
				leftToSlide = slideWidth;
			}
			// call recursive function to do actual slide
			if (leftToSlide > 0) {
				this._slideRight(leftToSlide - this.pos);
			}
		},
	  
		_slideRight: function (totalToSlide) {    
			if (this.pos == 0) {
				this.pos = -1;
			}
			this.pos -= this.slidePixel;
			if (this.pos < -totalToSlide) {
				var diff = -this.pos - totalToSlide;
				this.pos += this.slidePixel;
				if (diff < this.slidePixel) {
					this.pos -= diff;  // any last pixels if not multiple of slidePixel number
					dojo.style(this.toc, "left", this.pos + "px");
				}
				return;
			}
			dojo.style(this.toc, "left", this.pos + "px");
			this.t = setTimeout(dojo.hitch(this, this._slideRight, totalToSlide), this.slideDelay);    
		},
	  
		/**
		* Invoked when user clicks on left slider button
		*/
		slideLeft: function () {    
			if (this.slideDirection == "") {
				this.showSecondButton(this.sliderRight, "marginRight", 1);
			} else {
				this.slideStop();
			}
			this.slideDirection = "L";
				
			// determine number of pixels to slide adjusting for any scrolling
			// from tabbing
			var slideWidth = dojo.style(this.sliderOuterDiv, "width") - this.tocMarginLeft;    
			var leftToSlide;
			if (this.ltr) {
				leftToSlide = -this.pos + this.sliderOuterDiv.scrollLeft;
			} else {
				leftToSlide = this.totalWidthToCItems - slideWidth - this.pos - this.getScrollAmount();
			}
			// don't slide too much
			if (leftToSlide > slideWidth) {
				leftToSlide = slideWidth;
			}
			// call recursive function to do actual slide
			if (leftToSlide > 0) {
				this._slideLeft(leftToSlide);
			}
		},
	  
		_slideLeft: function (totalToSlide) {
			if (this.pos == 0) {
				this.pos = 1;
			}
			this.pos += this.slidePixel;
			totalToSlide -= this.slidePixel;
			if (totalToSlide < 0) {
				this.pos -= this.slidePixel;
				if (-totalToSlide < this.slidePixel) {
					this.pos += -totalToSlide;  // any last pixels if not multiple of slidePixel number
					dojo.style(this.toc, "left", this.pos + "px");
				}
				return;
			}
			dojo.style(this.toc, "left", this.pos + "px");
			this.t = setTimeout(dojo.hitch(this, this._slideLeft, totalToSlide), this.slideDelay);    
		},

		/**
		* Cancels the set timer so sliding is stopped
		*/
		slideStop: function () {
			clearTimeout(this.t);
		},
	  
		/**
		* Resets slider so first arrow is visible.
		*/
		reset: function () {
			this.pos = 0;
			dojo.style(this.toc, "left", "0px");
			if (this.ltr) {
				this.sliderOuterDiv.scrollLeft = 0;
			}
		}
	});

});

},
'idx/oneui/MenuBar':function(){
/*
 * Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

define(["dojo/_base/declare",
		"dijit/MenuBar",
    "idx/oneui/_MenuOpenOnHoverMixin"],
		function(declare,
				 MenuBar,
         _MenuOpenOnHoverMixin){
	/**
	 * Creates an idx.oneui.MenuBar
	 * @name idx.oneui.MenuBar
	 * @class The MenuBar widget provides a menu bar with open-on-hover behaviour.
	 * @augments dijit.MenuBar
	 * @augments idx.oneui._MenuOpenOnHoverMixin
	 * @borrows idx.oneui._MenuOpenOnHoverMixin#openOnHover as this.openOnHover
	 * @example &lt;div data-dojo-type="idx.oneui.MenuBar"&gt;
  &lt;div data-dojo-type="dijit.PopupMenuBarItem"&gt;
    &lt;span&gt;Edit&lt;/span&gt;
    &lt;div data-dojo-type="idx.oneui.Menu"&gt;
      &lt;div data-dojo-type="dijit.MenuItem" onclick="..."&gt;Cut&lt;/div&gt;
      &lt;div data-dojo-type="dijit.MenuItem" onclick="..."&gt;Copy&lt;/div&gt;
      &lt;div data-dojo-type="dijit.MenuItem" onclick="..."&gt;Paste&lt;/div&gt;
    &lt;/div&gt;
  &lt;/div&gt;
&lt;/div&gt;
	 */
	return declare("idx.oneui.MenuBar", [MenuBar, _MenuOpenOnHoverMixin], 
	/** @lends idx.oneui.MenuBar.prototype */
	{
	});
	
});

},
'url:idx/oneui/templates/HoverHelpTooltip.html':"<div class=\"idxOneuiHoverHelpTooltip idxOneuiHoverHelpTooltipLeft\" role=\"dialog\"\r\n\t><div role=\"document\"\r\n\t\t><span data-dojo-attach-point=\"closeButtonNode\" class=\"idxOneuiHoverHelpTooltipCloseIcon\" data-dojo-attach-event=\"ondijitclick: hideOnClickClose\" role=\"button\" tabIndex=\"0\"></span\r\n\t></div\r\n\t><div data-dojo-attach-point=\"outerContainerNode\" class=\"idxOneuiHoverHelpTooltipContainer idxOneuiHoverHelpTooltipContents\"\r\n\t\t><div data-dojo-attach-point=\"containerNode\" role=\"document\" tabindex=0></div\r\n\t\t><a target=\"_blank\" href=\"#updateme\" class=\"idxOneuiHoverHelpTooltipLearnLink\" data-dojo-attach-point=\"learnMoreNode\"><span>${learnMoreLabel}</span></a\r\n\t></div\r\n\t><div class=\"idxOneuiHoverHelpTooltipConnector\" data-dojo-attach-point=\"connectorNode\"></div\r\n></div>",
'dijit/layout/LayoutContainer':function(){
define([
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/lang",
	"dojo/_base/declare", // declare
	"../_WidgetBase",
	"./_LayoutWidget",
	"./utils"		// layoutUtils.layoutChildren
], function(kernel, lang, declare, _WidgetBase, _LayoutWidget, layoutUtils){

/*=====
	var _WidgetBase = dijit._WidgetBase;
	var _LayoutWidget = dijit.layout._LayoutWidget;
=====*/

// module:
//		dijit/layout/LayoutContainer
// summary:
//		Deprecated.  Use `dijit.layout.BorderContainer` instead.


// This argument can be specified for the children of a LayoutContainer.
// Since any widget can be specified as a LayoutContainer child, mix it
// into the base widget class.  (This is a hack, but it's effective.)
lang.extend(_WidgetBase, {
	// layoutAlign: String
	//		"none", "left", "right", "bottom", "top", and "client".
	//		See the LayoutContainer description for details on this parameter.
	layoutAlign: 'none'
});

return declare("dijit.layout.LayoutContainer", _LayoutWidget, {
	// summary:
	//		Deprecated.  Use `dijit.layout.BorderContainer` instead.
	//
	// description:
	//		Provides Delphi-style panel layout semantics.
	//
	//		A LayoutContainer is a box with a specified size (like style="width: 500px; height: 500px;"),
	//		that contains children widgets marked with "layoutAlign" of "left", "right", "bottom", "top", and "client".
	//		It takes it's children marked as left/top/bottom/right, and lays them out along the edges of the box,
	//		and then it takes the child marked "client" and puts it into the remaining space in the middle.
	//
	//		Left/right positioning is similar to CSS's "float: left" and "float: right",
	//		and top/bottom positioning would be similar to "float: top" and "float: bottom", if there were such
	//		CSS.
	//
	//		Note that there can only be one client element, but there can be multiple left, right, top,
	//		or bottom elements.
	//
	// example:
	// |	<style>
	// |		html, body{ height: 100%; width: 100%; }
	// |	</style>
	// |	<div data-dojo-type="dijit.layout.LayoutContainer" style="width: 100%; height: 100%">
	// |		<div data-dojo-type="dijit.layout.ContentPane" data-dojo-props="layoutAlign: 'top'">header text</div>
	// |		<div data-dojo-type="dijit.layout.ContentPane" data-dojo-props="layoutAlign: 'left'" style="width: 200px;">table of contents</div>
	// |		<div data-dojo-type="dijit.layout.ContentPane" data-dojo-props="layoutAlign: 'client'">client area</div>
	// |	</div>
	//
	//		Lays out each child in the natural order the children occur in.
	//		Basically each child is laid out into the "remaining space", where "remaining space" is initially
	//		the content area of this widget, but is reduced to a smaller rectangle each time a child is added.
	// tags:
	//		deprecated

	baseClass: "dijitLayoutContainer",

	constructor: function(){
		kernel.deprecated("dijit.layout.LayoutContainer is deprecated", "use BorderContainer instead", 2.0);
	},

	layout: function(){
		layoutUtils.layoutChildren(this.domNode, this._contentBox, this.getChildren());
	},

	addChild: function(/*dijit._Widget*/ child, /*Integer?*/ insertIndex){
		this.inherited(arguments);
		if(this._started){
			layoutUtils.layoutChildren(this.domNode, this._contentBox, this.getChildren());
		}
	},

	removeChild: function(/*dijit._Widget*/ widget){
		this.inherited(arguments);
		if(this._started){
			layoutUtils.layoutChildren(this.domNode, this._contentBox, this.getChildren());
		}
	}
});

});

},
'ibm/usmi/statuspod/JobStatusDataSet':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/statuspod/JobStatusDataSet", [
	"ibm/tivoli/navmodel/Dataset",
	"dojo/_base/declare"
], function (Dataset, declare) {
	return declare("ibm.usmi.statuspod.JobStatusDataSet", [ Dataset
	], {
		_handleChangedEvent: function(event) {
			//do nothing at all, our job status pod would recreate a dataset while receive the event.
		}
		
	});
});

},
'dojo/cldr/supplemental':function(){
define(["../_base/kernel", "../_base/lang", "../i18n"], function(dojo, lang) {
	// module:
	//		dojo/cldr/supplemental
	// summary:
	//		TODOC

lang.getObject("cldr.supplemental", true, dojo);

dojo.cldr.supplemental.getFirstDayOfWeek = function(/*String?*/locale){
// summary: Returns a zero-based index for first day of the week
// description:
//		Returns a zero-based index for first day of the week, as used by the local (Gregorian) calendar.
//		e.g. Sunday (returns 0), or Monday (returns 1)

	// from http://www.unicode.org/cldr/data/common/supplemental/supplementalData.xml:supplementalData/weekData/firstDay
	var firstDay = {/*default is 1=Monday*/
		mv:5,
		ae:6,af:6,bh:6,dj:6,dz:6,eg:6,er:6,et:6,iq:6,ir:6,jo:6,ke:6,kw:6,
		ly:6,ma:6,om:6,qa:6,sa:6,sd:6,so:6,sy:6,tn:6,ye:6,
		ar:0,as:0,az:0,bw:0,ca:0,cn:0,fo:0,ge:0,gl:0,gu:0,hk:0,
		il:0,'in':0,jm:0,jp:0,kg:0,kr:0,la:0,mh:0,mn:0,mo:0,mp:0,
		mt:0,nz:0,ph:0,pk:0,sg:0,th:0,tt:0,tw:0,um:0,us:0,uz:0,
		vi:0,zw:0
// variant. do not use?		gb:0,
	};

	var country = dojo.cldr.supplemental._region(locale);
	var dow = firstDay[country];
	return (dow === undefined) ? 1 : dow; /*Number*/
};

dojo.cldr.supplemental._region = function(/*String?*/locale){
	locale = dojo.i18n.normalizeLocale(locale);
	var tags = locale.split('-');
	var region = tags[1];
	if(!region){
		// IE often gives language only (#2269)
		// Arbitrary mappings of language-only locales to a country:
		region = {de:"de", en:"us", es:"es", fi:"fi", fr:"fr", he:"il", hu:"hu", it:"it",
			ja:"jp", ko:"kr", nl:"nl", pt:"br", sv:"se", zh:"cn"}[tags[0]];
	}else if(region.length == 4){
		// The ISO 3166 country code is usually in the second position, unless a
		// 4-letter script is given. See http://www.ietf.org/rfc/rfc4646.txt
		region = tags[2];
	}
	return region;
};

dojo.cldr.supplemental.getWeekend = function(/*String?*/locale){
// summary: Returns a hash containing the start and end days of the weekend
// description:
//		Returns a hash containing the start and end days of the weekend according to local custom using locale,
//		or by default in the user's locale.
//		e.g. {start:6, end:0}

	// from http://www.unicode.org/cldr/data/common/supplemental/supplementalData.xml:supplementalData/weekData/weekend{Start,End}
	var weekendStart = {/*default is 6=Saturday*/
		'in':0,
		af:4,dz:4,ir:4,om:4,sa:4,ye:4,
		ae:5,bh:5,eg:5,il:5,iq:5,jo:5,kw:5,ly:5,ma:5,qa:5,sd:5,sy:5,tn:5
	};

	var weekendEnd = {/*default is 0=Sunday*/
		af:5,dz:5,ir:5,om:5,sa:5,ye:5,
		ae:6,bh:5,eg:6,il:6,iq:6,jo:6,kw:6,ly:6,ma:6,qa:6,sd:6,sy:6,tn:6
	};

	var country = dojo.cldr.supplemental._region(locale);
	var start = weekendStart[country];
	var end = weekendEnd[country];
	if(start === undefined){start=6;}
	if(end === undefined){end=0;}
	return {start:start, end:end}; /*Object {start,end}*/
};

return dojo.cldr.supplemental;
});

},
'dijit/_base/manager':function(){
define([
	"dojo/_base/array",
	"dojo/_base/config", // defaultDuration
	"../registry",
	".."	// for setting exports to dijit namespace
], function(array, config, registry, dijit){

	// module:
	//		dijit/_base/manager
	// summary:
	//		Shim to methods on registry, plus a few other declarations.
	//		New code should access dijit/registry directly when possible.

	/*=====
	dijit.byId = function(id){
		// summary:
		//		Returns a widget by it's id, or if passed a widget, no-op (like dom.byId())
		// id: String|dijit._Widget
		return registry.byId(id); // dijit._Widget
	};

	dijit.getUniqueId = function(widgetType){
		// summary:
		//		Generates a unique id for a given widgetType
		// widgetType: String
		return registry.getUniqueId(widgetType); // String
	};

	dijit.findWidgets = function(root){
		// summary:
		//		Search subtree under root returning widgets found.
		//		Doesn't search for nested widgets (ie, widgets inside other widgets).
		// root: DOMNode
		return registry.findWidgets(root);
	};

	dijit._destroyAll = function(){
		// summary:
		//		Code to destroy all widgets and do other cleanup on page unload

		return registry._destroyAll();
	};

	dijit.byNode = function(node){
		// summary:
		//		Returns the widget corresponding to the given DOMNode
		// node: DOMNode
		return registry.byNode(node); // dijit._Widget
	};

	dijit.getEnclosingWidget = function(node){
		// summary:
		//		Returns the widget whose DOM tree contains the specified DOMNode, or null if
		//		the node is not contained within the DOM tree of any widget
		// node: DOMNode
		return registry.getEnclosingWidget(node);
	};
	=====*/
	array.forEach(["byId", "getUniqueId", "findWidgets", "_destroyAll", "byNode", "getEnclosingWidget"], function(name){
		dijit[name] = registry[name];
	});

	/*=====
	dojo.mixin(dijit, {
		// defaultDuration: Integer
		//		The default fx.animation speed (in ms) to use for all Dijit
		//		transitional fx.animations, unless otherwise specified
		//		on a per-instance basis. Defaults to 200, overrided by
		//		`djConfig.defaultDuration`
		defaultDuration: 200
	});
	=====*/
	dijit.defaultDuration = config["defaultDuration"] || 200;

	return dijit;
});

},
'gridx/core/model/extensions/Query':function(){
define([
	"dojo/_base/declare",
	'../_Extension'
], function(declare, _Extension){

	return declare(/*===== "gridx.core.model.extensions.Query", =====*/_Extension, {
		name: 'query',

		priority: 40,

		constructor: function(model, args){
			this.clear();
			this._mixinAPI('query');
			if(args.query){
				this.query(args.query, args.queryOptions);
			}
		},

		//Public--------------------------------------------------------------
		clear: function(){},

		query: function(/* query, queryOptions */){
			this.model._addCmd({
				name: '_cmdQuery',
				scope: this,
				args: arguments
			});
		},
	
		//Private--------------------------------------------------------------
		_cmdQuery: function(){
			var a = arguments,
				args = a[a.length - 1],
				m = this.model,
				c = m._cache, 
				op = c.options = c.options || {};
			op.query = args[0];
			op.queryOptions = args[1];
			m._msg('storeChange');
			c.clear();
		}
	});
});

},
'dijit/layout/AccordionContainer':function(){
define([
	"require",
	"dojo/_base/array", // array.forEach array.map
	"dojo/_base/declare", // declare
	"dojo/_base/event", // event.stop
	"dojo/_base/fx", // fx.Animation
	"dojo/dom", // dom.setSelectable
	"dojo/dom-attr", // domAttr.attr
	"dojo/dom-class", // domClass.remove
	"dojo/dom-construct", // domConstruct.place
	"dojo/dom-geometry",
	"dojo/_base/kernel",
	"dojo/keys", // keys
	"dojo/_base/lang", // lang.getObject lang.hitch
	"dojo/_base/sniff", // has("ie")
	"dojo/topic", // publish
	"../focus",			// focus.focus()
	"../_base/manager",	// manager.defaultDuration
	"dojo/ready",
	"../_Widget",
	"../_Container",
	"../_TemplatedMixin",
	"../_CssStateMixin",
	"./StackContainer",
	"./ContentPane",
	"dojo/text!./templates/AccordionButton.html"
], function(require, array, declare, event, fx, dom, domAttr, domClass, domConstruct, domGeometry,
			kernel, keys, lang, has, topic, focus, manager, ready,
			_Widget, _Container, _TemplatedMixin, _CssStateMixin, StackContainer, ContentPane, template){

/*=====
	var _Widget = dijit._Widget;
	var _Container = dijit._Container;
	var _TemplatedMixin = dijit._TemplatedMixin;
	var _CssStateMixin = dijit._CssStateMixin;
	var StackContainer = dijit.layout.StackContainer;
	var ContentPane = dijit.layout.ContentPane;
=====*/

	// module:
	//		dijit/layout/AccordionContainer
	// summary:
	//		Holds a set of panes where every pane's title is visible, but only one pane's content is visible at a time,
	//		and switching between panes is visualized by sliding the other panes up/down.


	// Design notes:
	//
	// An AccordionContainer is a StackContainer, but each child (typically ContentPane)
	// is wrapped in a _AccordionInnerContainer.   This is hidden from the caller.
	//
	// The resulting markup will look like:
	//
	//	<div class=dijitAccordionContainer>
	//		<div class=dijitAccordionInnerContainer>	(one pane)
	//				<div class=dijitAccordionTitle>		(title bar) ... </div>
	//				<div class=dijtAccordionChildWrapper>   (content pane) </div>
	//		</div>
	//	</div>
	//
	// Normally the dijtAccordionChildWrapper is hidden for all but one child (the shown
	// child), so the space for the content pane is all the title bars + the one dijtAccordionChildWrapper,
	// which on claro has a 1px border plus a 2px bottom margin.
	//
	// During animation there are two dijtAccordionChildWrapper's shown, so we need
	// to compensate for that.


	var AccordionButton = declare("dijit.layout._AccordionButton", [_Widget, _TemplatedMixin, _CssStateMixin], {
		// summary:
		//		The title bar to click to open up an accordion pane.
		//		Internal widget used by AccordionContainer.
		// tags:
		//		private

		templateString: template,

		// label: String
		//		Title of the pane
		label: "",
		_setLabelAttr: {node: "titleTextNode", type: "innerHTML" },

		// title: String
		//		Tooltip that appears on hover
		title: "",
		_setTitleAttr: {node: "titleTextNode", type: "attribute", attribute: "title"},

		// iconClassAttr: String
		//		CSS class for icon to left of label
		iconClassAttr: "",
		_setIconClassAttr: { node: "iconNode", type: "class" },

		baseClass: "dijitAccordionTitle",

		getParent: function(){
			// summary:
			//		Returns the AccordionContainer parent.
			// tags:
			//		private
			return this.parent;
		},

		buildRendering: function(){
			this.inherited(arguments);
			var titleTextNodeId = this.id.replace(' ','_');
			domAttr.set(this.titleTextNode, "id", titleTextNodeId+"_title");
			this.focusNode.setAttribute("aria-labelledby", domAttr.get(this.titleTextNode, "id"));
			dom.setSelectable(this.domNode, false);
		},

		getTitleHeight: function(){
			// summary:
			//		Returns the height of the title dom node.
			return domGeometry.getMarginSize(this.domNode).h;	// Integer
		},

		// TODO: maybe the parent should set these methods directly rather than forcing the code
		// into the button widget?
		_onTitleClick: function(){
			// summary:
			//		Callback when someone clicks my title.
			var parent = this.getParent();
				parent.selectChild(this.contentWidget, true);
				focus.focus(this.focusNode);
		},

		_onTitleKeyPress: function(/*Event*/ evt){
			return this.getParent()._onKeyPress(evt, this.contentWidget);
		},

		_setSelectedAttr: function(/*Boolean*/ isSelected){
			this._set("selected", isSelected);
			this.focusNode.setAttribute("aria-expanded", isSelected);
			this.focusNode.setAttribute("aria-selected", isSelected);
			this.focusNode.setAttribute("tabIndex", isSelected ? "0" : "-1");
		}
	});

	var AccordionInnerContainer = declare("dijit.layout._AccordionInnerContainer", [_Widget, _CssStateMixin], {
		// summary:
		//		Internal widget placed as direct child of AccordionContainer.containerNode.
		//		When other widgets are added as children to an AccordionContainer they are wrapped in
		//		this widget.

/*=====
		// buttonWidget: Function || String
		//		Class to use to instantiate title
		//		(Wish we didn't have a separate widget for just the title but maintaining it
		//		for backwards compatibility, is it worth it?)
		 buttonWidget: null,
=====*/

/*=====
		// contentWidget: dijit._Widget
		//		Pointer to the real child widget
	 	contentWidget: null,
=====*/

		baseClass: "dijitAccordionInnerContainer",

		// tell nested layout widget that we will take care of sizing
		isLayoutContainer: true,

		buildRendering: function(){
			// Builds a template like:
			//	<div class=dijitAccordionInnerContainer>
			//		Button
			//		<div class=dijitAccordionChildWrapper>
			//			ContentPane
			//		</div>
			//	</div>

			// Create wrapper div, placed where the child is now
			this.domNode = domConstruct.place("<div class='" + this.baseClass +
				"' role='presentation'>", this.contentWidget.domNode, "after");

			// wrapper div's first child is the button widget (ie, the title bar)
			var child = this.contentWidget,
				cls = lang.isString(this.buttonWidget) ? lang.getObject(this.buttonWidget) : this.buttonWidget;
			this.button = child._buttonWidget = (new cls({
				contentWidget: child,
				label: child.title,
				title: child.tooltip,
				dir: child.dir,
				lang: child.lang,
				textDir: child.textDir,
				iconClass: child.iconClass,
				id: child.id + "_button",
				parent: this.parent
			})).placeAt(this.domNode);

			// and then the actual content widget (changing it from prior-sibling to last-child),
			// wrapped by a <div class=dijitAccordionChildWrapper>
			this.containerNode = domConstruct.place("<div class='dijitAccordionChildWrapper' style='display:none'>", this.domNode);
			domConstruct.place(this.contentWidget.domNode, this.containerNode);
		},

		postCreate: function(){
			this.inherited(arguments);

			// Map changes in content widget's title etc. to changes in the button
			var button = this.button;
			this._contentWidgetWatches = [
				this.contentWidget.watch('title', lang.hitch(this, function(name, oldValue, newValue){
					button.set("label", newValue);
				})),
				this.contentWidget.watch('tooltip', lang.hitch(this, function(name, oldValue, newValue){
					button.set("title", newValue);
				})),
				this.contentWidget.watch('iconClass', lang.hitch(this, function(name, oldValue, newValue){
					button.set("iconClass", newValue);
				}))
			];
		},

		_setSelectedAttr: function(/*Boolean*/ isSelected){
			this._set("selected", isSelected);
			this.button.set("selected", isSelected);
			if(isSelected){
				var cw = this.contentWidget;
				if(cw.onSelected){ cw.onSelected(); }
			}
		},

		startup: function(){
			// Called by _Container.addChild()
			this.contentWidget.startup();
		},

		destroy: function(){
			this.button.destroyRecursive();

			array.forEach(this._contentWidgetWatches || [], function(w){ w.unwatch(); });

			delete this.contentWidget._buttonWidget;
			delete this.contentWidget._wrapperWidget;

			this.inherited(arguments);
		},

		destroyDescendants: function(/*Boolean*/ preserveDom){
			// since getChildren isn't working for me, have to code this manually
			this.contentWidget.destroyRecursive(preserveDom);
		}
	});

	var AccordionContainer = declare("dijit.layout.AccordionContainer", StackContainer, {
		// summary:
		//		Holds a set of panes where every pane's title is visible, but only one pane's content is visible at a time,
		//		and switching between panes is visualized by sliding the other panes up/down.
		// example:
		//	| 	<div data-dojo-type="dijit.layout.AccordionContainer">
		//	|		<div data-dojo-type="dijit.layout.ContentPane" title="pane 1">
		//	|		</div>
		//	|		<div data-dojo-type="dijit.layout.ContentPane" title="pane 2">
		//	|			<p>This is some text</p>
		//	|		</div>
		//	|	</div>

		// duration: Integer
		//		Amount of time (in ms) it takes to slide panes
		duration: manager.defaultDuration,

		// buttonWidget: [const] String
		//		The name of the widget used to display the title of each pane
		buttonWidget: AccordionButton,

/*=====
		// _verticalSpace: Number
		//		Pixels of space available for the open pane
		//		(my content box size minus the cumulative size of all the title bars)
		_verticalSpace: 0,
=====*/
		baseClass: "dijitAccordionContainer",

		buildRendering: function(){
			this.inherited(arguments);
			this.domNode.style.overflow = "hidden";		// TODO: put this in dijit.css
			this.domNode.setAttribute("role", "tablist");	// TODO: put this in template
		},

		startup: function(){
			if(this._started){ return; }
			this.inherited(arguments);
			if(this.selectedChildWidget){
				var style = this.selectedChildWidget.containerNode.style;
				style.display = "";
				style.overflow = "auto";
				this.selectedChildWidget._wrapperWidget.set("selected", true);
			}
		},

		layout: function(){
			// Implement _LayoutWidget.layout() virtual method.
			// Set the height of the open pane based on what room remains.

			var openPane = this.selectedChildWidget;

			if(!openPane){ return;}

			// space taken up by title, plus wrapper div (with border/margin) for open pane
			var wrapperDomNode = openPane._wrapperWidget.domNode,
				wrapperDomNodeMargin = domGeometry.getMarginExtents(wrapperDomNode),
				wrapperDomNodePadBorder = domGeometry.getPadBorderExtents(wrapperDomNode),
				wrapperContainerNode = openPane._wrapperWidget.containerNode,
				wrapperContainerNodeMargin = domGeometry.getMarginExtents(wrapperContainerNode),
				wrapperContainerNodePadBorder = domGeometry.getPadBorderExtents(wrapperContainerNode),
				mySize = this._contentBox;

			// get cumulative height of all the unselected title bars
			var totalCollapsedHeight = 0;
			array.forEach(this.getChildren(), function(child){
	            if(child != openPane){
					// Using domGeometry.getMarginSize() rather than domGeometry.position() since claro has 1px bottom margin
					// to separate accordion panes.  Not sure that works perfectly, it's probably putting a 1px
					// margin below the bottom pane (even though we don't want one).
					totalCollapsedHeight += domGeometry.getMarginSize(child._wrapperWidget.domNode).h;
				}
			});
			this._verticalSpace = mySize.h - totalCollapsedHeight - wrapperDomNodeMargin.h
			 	- wrapperDomNodePadBorder.h - wrapperContainerNodeMargin.h - wrapperContainerNodePadBorder.h
				- openPane._buttonWidget.getTitleHeight();

			// Memo size to make displayed child
			this._containerContentBox = {
				h: this._verticalSpace,
				w: this._contentBox.w - wrapperDomNodeMargin.w - wrapperDomNodePadBorder.w
					- wrapperContainerNodeMargin.w - wrapperContainerNodePadBorder.w
			};

			if(openPane){
				openPane.resize(this._containerContentBox);
			}
		},

		_setupChild: function(child){
			// Overrides _LayoutWidget._setupChild().
			// Put wrapper widget around the child widget, showing title

			child._wrapperWidget = AccordionInnerContainer({
				contentWidget: child,
				buttonWidget: this.buttonWidget,
				id: child.id + "_wrapper",
				dir: child.dir,
				lang: child.lang,
				textDir: child.textDir,
				parent: this
			});

			this.inherited(arguments);
		},

		addChild: function(/*dijit._Widget*/ child, /*Integer?*/ insertIndex){
			if(this._started){
				// Adding a child to a started Accordion is complicated because children have
				// wrapper widgets.  Default code path (calling this.inherited()) would add
				// the new child inside another child's wrapper.

				// First add in child as a direct child of this AccordionContainer
				var refNode = this.containerNode;
				if(insertIndex && typeof insertIndex == "number"){
					var children = _Widget.prototype.getChildren.call(this);	// get wrapper panes
					if(children && children.length >= insertIndex){
						refNode = children[insertIndex-1].domNode;
						insertIndex = "after";
					}
				}
				domConstruct.place(child.domNode, refNode, insertIndex);

				if(!child._started){
					child.startup();
				}

				// Then stick the wrapper widget around the child widget
				this._setupChild(child);

				// Code below copied from StackContainer
				topic.publish(this.id+"-addChild", child, insertIndex);	// publish
				this.layout();
				if(!this.selectedChildWidget){
					this.selectChild(child);
				}
			}else{
				// We haven't been started yet so just add in the child widget directly,
				// and the wrapper will be created on startup()
				this.inherited(arguments);
			}
		},

		removeChild: function(child){
			// Overrides _LayoutWidget.removeChild().

			// Destroy wrapper widget first, before StackContainer.getChildren() call.
			// Replace wrapper widget with true child widget (ContentPane etc.).
			// This step only happens if the AccordionContainer has been started; otherwise there's no wrapper.
			if(child._wrapperWidget){
				domConstruct.place(child.domNode, child._wrapperWidget.domNode, "after");
				child._wrapperWidget.destroy();
				delete child._wrapperWidget;
			}

			domClass.remove(child.domNode, "dijitHidden");

			this.inherited(arguments);
		},

		getChildren: function(){
			// Overrides _Container.getChildren() to return content panes rather than internal AccordionInnerContainer panes
			return array.map(this.inherited(arguments), function(child){
				return child.declaredClass == "dijit.layout._AccordionInnerContainer" ? child.contentWidget : child;
			}, this);
		},

		destroy: function(){
			if(this._animation){
				this._animation.stop();
			}
			array.forEach(this.getChildren(), function(child){
				// If AccordionContainer has been started, then each child has a wrapper widget which
				// also needs to be destroyed.
				if(child._wrapperWidget){
					child._wrapperWidget.destroy();
				}else{
					child.destroyRecursive();
				}
			});
			this.inherited(arguments);
		},

		_showChild: function(child){
			// Override StackContainer._showChild() to set visibility of _wrapperWidget.containerNode
			child._wrapperWidget.containerNode.style.display="block";
			return this.inherited(arguments);
		},

		_hideChild: function(child){
			// Override StackContainer._showChild() to set visibility of _wrapperWidget.containerNode
			child._wrapperWidget.containerNode.style.display="none";
			this.inherited(arguments);
		},

		_transition: function(/*dijit._Widget?*/ newWidget, /*dijit._Widget?*/ oldWidget, /*Boolean*/ animate){
			// Overrides StackContainer._transition() to provide sliding of title bars etc.

			if(has("ie") < 8){
				// workaround animation bugs by not animating; not worth supporting animation for IE6 & 7
				animate = false;
			}

			if(this._animation){
				// there's an in-progress animation.  speedily end it so we can do the newly requested one
				this._animation.stop(true);
				delete this._animation;
			}

			var self = this;

			if(newWidget){
				newWidget._wrapperWidget.set("selected", true);

				var d = this._showChild(newWidget);	// prepare widget to be slid in

				// Size the new widget, in case this is the first time it's being shown,
				// or I have been resized since the last time it was shown.
				// Note that page must be visible for resizing to work.
				if(this.doLayout && newWidget.resize){
					newWidget.resize(this._containerContentBox);
				}
			}

			if(oldWidget){
				oldWidget._wrapperWidget.set("selected", false);
				if(!animate){
					this._hideChild(oldWidget);
				}
			}

			if(animate){
				var newContents = newWidget._wrapperWidget.containerNode,
					oldContents = oldWidget._wrapperWidget.containerNode;

				// During the animation we will be showing two dijitAccordionChildWrapper nodes at once,
				// which on claro takes up 4px extra space (compared to stable AccordionContainer).
				// Have to compensate for that by immediately shrinking the pane being closed.
				var wrapperContainerNode = newWidget._wrapperWidget.containerNode,
					wrapperContainerNodeMargin = domGeometry.getMarginExtents(wrapperContainerNode),
					wrapperContainerNodePadBorder = domGeometry.getPadBorderExtents(wrapperContainerNode),
					animationHeightOverhead = wrapperContainerNodeMargin.h + wrapperContainerNodePadBorder.h;

				oldContents.style.height = (self._verticalSpace - animationHeightOverhead) + "px";

				this._animation = new fx.Animation({
					node: newContents,
					duration: this.duration,
					curve: [1, this._verticalSpace - animationHeightOverhead - 1],
					onAnimate: function(value){
						value = Math.floor(value);	// avoid fractional values
						newContents.style.height = value + "px";
						oldContents.style.height = (self._verticalSpace - animationHeightOverhead - value) + "px";
					},
					onEnd: function(){
						delete self._animation;
						newContents.style.height = "auto";
						oldWidget._wrapperWidget.containerNode.style.display = "none";
						oldContents.style.height = "auto";
						self._hideChild(oldWidget);
					}
				});
				this._animation.onStop = this._animation.onEnd;
				this._animation.play();
			}

			return d;	// If child has an href, promise that fires when the widget has finished loading
		},

		// note: we are treating the container as controller here
		_onKeyPress: function(/*Event*/ e, /*dijit._Widget*/ fromTitle){
			// summary:
			//		Handle keypress events
			// description:
			//		This is called from a handler on AccordionContainer.domNode
			//		(setup in StackContainer), and is also called directly from
			//		the click handler for accordion labels
			if(this.disabled || e.altKey || !(fromTitle || e.ctrlKey)){
				return;
			}
			var c = e.charOrCode;
			if((fromTitle && (c == keys.LEFT_ARROW || c == keys.UP_ARROW)) ||
					(e.ctrlKey && c == keys.PAGE_UP)){
				this._adjacent(false)._buttonWidget._onTitleClick();
				event.stop(e);
			}else if((fromTitle && (c == keys.RIGHT_ARROW || c == keys.DOWN_ARROW)) ||
					(e.ctrlKey && (c == keys.PAGE_DOWN || c == keys.TAB))){
				this._adjacent(true)._buttonWidget._onTitleClick();
				event.stop(e);
			}
		}
	});

	// Back compat w/1.6, remove for 2.0
	if(!kernel.isAsync){
		ready(0, function(){
			var requires = ["dijit/layout/AccordionPane"];
			require(requires);	// use indirection so modules not rolled into a build
		});
	}

	// For monkey patching
	AccordionContainer._InnerContainer = AccordionInnerContainer;
	AccordionContainer._Button = AccordionButton;

	return AccordionContainer;
});

},
'dojo/cache':function(){
define(["./_base/kernel", "./text"], function(dojo, text){
	// module:
	//		dojo/cache
	// summary:
	//		The module defines dojo.cache by loading dojo/text.

	//dojo.cache is defined in dojo/text
	return dojo.cache;
});

},
'url:ibm/usmi/scheduler/wizard_pages/templates/NotificationsPage.html':"<div>\r\n<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n<!-- \r\n\t<div class=\"tiv-page-header\">\r\n\t\t<h2>Notifications</h2>\r\n\t\t<p>Receive an e-mail notification with the progress of this job.</p>\r\n\t</div>\r\n\t-->\r\n\t<label for=\"ReceiveNotifications\">${persistResources.RECEIVE_NOTIFICATIONS_LABEL}</label>\r\n\t<div id=\"ReceiveNotifications\">\r\n\t\t<input type=\"checkbox\" dojoAttachPoint=\"whenJobBegins\" data-dojo-type=\"dijit.form.CheckBox\" iconclass=\"dijitNoIcon\"\r\n\t\t\tdojoAttachEvent=\"onChange:doNotificationsPageRendering\" />${persistResources.JOB_BEGINS_VALUE}<br /> \r\n\t\t<input type=\"checkbox\" dojoAttachPoint=\"whenJobEndsSuccessfully\" data-dojo-type=\"dijit.form.CheckBox\" iconclass=\"dijitNoIcon\"\r\n\t\t\tdojoAttachEvent=\"onChange:doNotificationsPageRendering\" />${persistResources.JOB_COMPLETES_OK_VALUE}<br /> \r\n\t\t<input type=\"checkbox\" dojoAttachPoint=\"whenJobFails\" data-dojo-type=\"dijit.form.CheckBox\" iconclass=\"dijitNoIcon\"\r\n\t\t\tdojoAttachEvent=\"onChange:doNotificationsPageRendering\" />${persistResources.JOB_FAILS_VALUE}<br />\r\n\r\n\t\t<div dojoAttachPoint=\"errorSelectionStuff\" style=\"display: none; margin-left: 15px\">\r\n\t\t\t<input type=\"radio\" name=\"errorNotification\" data-dojo-type=\"dijit.form.RadioButton\"\r\n\t\t\t\tdojoAttachPoint=\"anyErrorRadio\" checked value=\"true\" dojoAttachEvent=\"onChange:doNextButtonCheck\" />${persistResources.JOB_FAILS_ANY_ERROR_RADIO_LABEL} <br />\r\n\t\t\t<input type=\"radio\" name=\"errorNotification\" data-dojo-type=\"dijit.form.RadioButton\"\r\n\t\t\t\tdojoAttachPoint=\"percentFailedRadio\" value=\"true\" dojoAttachEvent=\"onChange:doNextButtonCheck\" />${persistResources.JOB_FAILS_PERCENT_ERROR_RADIO_LABEL}\r\n\t\t\t\t\t<input type=\"text\"\tdojoAttachPoint=\"thresholdPercentFailed\"data-dojo-type=\"dijit.form.NumberSpinner\" \r\n\t\t\t\t\t\tdata-dojo-props=\"constraints:{min:1,max:100,places:0}\" value=\"1\" dojoAttachEvent=\"onChange:doNextButtonCheck\" intermediateChanges=\"true\" /><br />\r\n\t\t\t<input type=\"radio\" name=\"errorNotification\" data-dojo-type=\"dijit.form.RadioButton\"\r\n\t\t\t\tdojoAttachPoint=\"numberFailedRadio\" value=\"true\" dojoAttachEvent=\"onChange:doNextButtonCheck\" />${persistResources.JOB_FAILS_NUMBER_ERROR_RADIO_LABEL}\r\n\t\t\t\t\t<input type=\"text\"\tdojoAttachPoint=\"thresholdNumberFailed\"\tdata-dojo-type=\"dijit.form.NumberSpinner\" \r\n\t\t\t\t\t\tdata-dojo-props=\"constraints:{min:1,places:0}\" value=\"1\" dojoAttachEvent=\"onChange:doNextButtonCheck\" intermediateChanges=\"true\" /><br /> \r\n\t\t\t<br />\r\n\t\t</div>\r\n\t</div>\r\n\r\n\t<div dojoAttachPoint=\"emailNotificationStuff\" style=\"display: none;\">\r\n\t\t<label for=\"emailInformation\">${persistResources.EMAIL_INFORMATION_LABEL}</label>\r\n\t\t<div id=\"emailInformation\">\r\n\t\t\t<label for=\"notificationEmailAddress\" class=\"required\">${persistResources.EMAIL_ADDRESS_LABEL}</label> \r\n\t\t\t\t<input dojoAttachPoint=\"notificationEmailAddress\" type=\"text\" name=\"widget_emailAddr\" data-dojo-type=\"dijit.form.ValidationTextBox\" style=\"width: 60%;\"\r\n\t\t\t\t\trequired=\"true\" dojoAttachEvent=\"onChange:doNextButtonCheck\" intermediateChanges=\"true\" /> \r\n\t\t\t\t<br />\r\n\t\t\t<label for=\"notificationEmailServerName\" class=\"required\">${persistResources.EMAIL_SERVER_LABEL}</label> \r\n\t\t\t\t<input dojoAttachPoint=\"notificationEmailServerName\" type=\"text\" name=\"widget_emailServerName\" data-dojo-type=\"dijit.form.ValidationTextBox\" style=\"width: 60%;\"\r\n\t\t\t\t\trequired=\"true\" dojoAttachEvent=\"onChange:doNextButtonCheck\" intermediateChanges=\"true\" /> \r\n\t\t\t\t<br />\t\t\t\t\t\r\n\t\t\t<label for=\"notificationEmailServerPort\" class=\"required\">${persistResources.EMAIL_PORT_LABEL}</label> \r\n\t\t\t\t<input dojoAttachPoint=\"notificationEmailServerPort\" type=\"text\" name=\"widget_emailServerPort\" data-dojo-type=\"dijit.form.ValidationTextBox\" style=\"width: 60%;\"\r\n\t\t\t\trequired=\"true\" dojoAttachEvent=\"onChange:doNextButtonCheck\" intermediateChanges=\"true\" />\r\n\t\t</div>\r\n\t</div>\r\n</div>",
'url:dijit/templates/Menu.html':"<table class=\"dijit dijitMenu dijitMenuPassive dijitReset dijitMenuTable\" role=\"menu\" tabIndex=\"${tabIndex}\" data-dojo-attach-event=\"onkeypress:_onKeyPress\" cellspacing=\"0\">\n\t<tbody class=\"dijitReset\" data-dojo-attach-point=\"containerNode\"></tbody>\n</table>\n",
'dijit/_DialogMixin':function(){
define([
	"dojo/_base/declare", // declare
	"./a11y"	// _getTabNavigable
], function(declare, a11y){

	// module:
	//		dijit/_DialogMixin
	// summary:
	//		_DialogMixin provides functions useful to Dialog and TooltipDialog

	return declare("dijit._DialogMixin", null, {
		// summary:
		//		This provides functions useful to Dialog and TooltipDialog

		execute: function(/*Object*/ /*===== formContents =====*/){
			// summary:
			//		Callback when the user hits the submit button.
			//		Override this method to handle Dialog execution.
			// description:
			//		After the user has pressed the submit button, the Dialog
			//		first calls onExecute() to notify the container to hide the
			//		dialog and restore focus to wherever it used to be.
			//
			//		*Then* this method is called.
			// type:
			//		callback
		},

		onCancel: function(){
			// summary:
			//	    Called when user has pressed the Dialog's cancel button, to notify container.
			// description:
			//	    Developer shouldn't override or connect to this method;
			//		it's a private communication device between the TooltipDialog
			//		and the thing that opened it (ex: `dijit.form.DropDownButton`)
			// type:
			//		protected
		},

		onExecute: function(){
			// summary:
			//	    Called when user has pressed the dialog's OK button, to notify container.
			// description:
			//	    Developer shouldn't override or connect to this method;
			//		it's a private communication device between the TooltipDialog
			//		and the thing that opened it (ex: `dijit.form.DropDownButton`)
			// type:
			//		protected
		},

		_onSubmit: function(){
			// summary:
			//		Callback when user hits submit button
			// type:
			//		protected
			this.onExecute();	// notify container that we are about to execute
			this.execute(this.get('value'));
		},

		_getFocusItems: function(){
			// summary:
			//		Finds focusable items in dialog,
			//		and sets this._firstFocusItem and this._lastFocusItem
			// tags:
			//		protected

			var elems = a11y._getTabNavigable(this.containerNode);
			this._firstFocusItem = elems.lowest || elems.first || this.closeButtonNode || this.domNode;
			this._lastFocusItem = elems.last || elems.highest || this._firstFocusItem;
		}
	});
});

},
'url:ibm/usmi/widgets/templates/TreeTopperLink.html':"<a class=\"topperLink topperLinkAnchor\" data-dojo-attach-point=\"topperAnchor\" role=\"link\">\r\n<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n\t<div class=\"pad5t pad10l\">\r\n\t      <img class=\"topperLinkIcon sprite\" data-dojo-attach-point=\"iconAttachPoint\"></img>\r\n\t      <div class=\"topperLinkLabel\" data-dojo-attach-point=\"labelAttachPoint\"></div>\r\n\t</div>\r\n</a>\r\n",
'dijit/form/_CheckBoxMixin':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.set
	"dojo/_base/event" // event.stop
], function(declare, domAttr, event){

	// module:
	//		dijit/form/_CheckBoxMixin
	// summary:
	// 		Mixin to provide widget functionality corresponding to an HTML checkbox

	return declare("dijit.form._CheckBoxMixin", null, {
		// summary:
		// 		Mixin to provide widget functionality corresponding to an HTML checkbox
		//
		// description:
		//		User interacts with real html inputs.
		//		On onclick (which occurs by mouse click, space-bar, or
		//		using the arrow keys to switch the selected radio button),
		//		we update the state of the checkbox/radio.
		//

		// type: [private] String
		//		type attribute on <input> node.
		//		Overrides `dijit.form.Button.type`.  Users should not change this value.
		type: "checkbox",

		// value: String
		//		As an initialization parameter, equivalent to value field on normal checkbox
		//		(if checked, the value is passed as the value when form is submitted).
		value: "on",

		// readOnly: Boolean
		//		Should this widget respond to user input?
		//		In markup, this is specified as "readOnly".
		//		Similar to disabled except readOnly form values are submitted.
		readOnly: false,
		
		// aria-pressed for toggle buttons, and aria-checked for checkboxes
		_aria_attr: "aria-checked",

		_setReadOnlyAttr: function(/*Boolean*/ value){
			this._set("readOnly", value);
			domAttr.set(this.focusNode, 'readOnly', value);
			this.focusNode.setAttribute("aria-readonly", value);
		},

		// Override dijit.form.Button._setLabelAttr() since we don't even have a containerNode.
		// Normally users won't try to set label, except when CheckBox or RadioButton is the child of a dojox.layout.TabContainer
		_setLabelAttr: undefined,

		postMixInProperties: function(){
			if(this.value == ""){
				this.value = "on";
			}
			this.inherited(arguments);
		},

		reset: function(){
			this.inherited(arguments);
			// Handle unlikely event that the <input type=checkbox> value attribute has changed
			this._set("value", this.params.value || "on");
			domAttr.set(this.focusNode, 'value', this.value);
		},

		_onClick: function(/*Event*/ e){
			// summary:
			//		Internal function to handle click actions - need to check
			//		readOnly, since button no longer does that check.
			if(this.readOnly){
				event.stop(e);
				return false;
			}
			return this.inherited(arguments);
		}
	});
});

},
'ibm/usmi/widgets/wizard/DirectorPage':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
/*global define */
define("ibm/usmi/widgets/wizard/DirectorPage", [ "dojo/_base/declare",
	"dojo/dom-class",
	"dojoe/wizard/Page" ], function (
	declare, domclass, Page) {
	// declare that I'm extending dojoe.wizard.Page
	return declare([ Page ], {
		// override buildRendering to use our own Director-specific class (proper padding)
		buildRendering : function () {
			this.inherited(arguments);
			domclass.add(this.domNode, "director-tiv-page");
		}
	});
});

},
'ibm/tivoli/navmodel/DatasetRelationshipsStore':function(){
// wrapped by build app
define("ibm/tivoli/navmodel/DatasetRelationshipsStore", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/DatasetStore,ibm/tivoli/navmodel/Dataset"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */
 
dojo.provide("ibm.tivoli.navmodel.DatasetRelationshipsStore");

dojo.require("ibm.tivoli.navmodel.DatasetStore");
dojo.require("ibm.tivoli.navmodel.Dataset");
	
ibm.tivoli.navmodel.DatasetRelationshipsStore._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.DatasetRelationshipsStore");
	
dojo.declare("ibm.tivoli.navmodel.DatasetRelationshipsStore", ibm.tivoli.navmodel.DatasetStore, {
	// private members
	_clearCache: false,	
	_linkIdCondition: null,
	_trace: ibm.tivoli.navmodel.DatasetRelationshipsStore._trace, 
    
	/*=====
	constructor_kwArgs: function(dataset, defaultProperties, returnItem) {
		// summary:
		//		An anonymous object used to configure the datastore
		// dataset: Object
		//		A ibm.tivoli.navmodel.Dataset instance
		// defaultProperties: String[]
		//		An array of property id values that are pre-fetched with the items
		// returnItem: Boolean
		//		A boolean indicating if the getValue() method should just return the actual
		//		item so that it can be used for custom formatting of the value
	},
	=====*/
	constructor: function(/*constructor_kwArgs*/ kwArgs){
		// summary:
		//		Constructor for dojo datastore;
		//		that receives relationship data from an ibm.tivoli.navmodel.Dataset
		//		Base class will mixin kwArgs
		
        // create a filter condition to get relationships
        // associated with the provided links
        if (kwArgs.links && kwArgs.links.length > 0) {
        	this._linkIdCondition = "(";
        	var linksLength = this.links.length;
        	for (var linkIndex = 0;  linkIndex < linksLength;  ++linkIndex) {
        		if (linkIndex > 0) {
        			this._linkIdCondition += " or ";
        		}
        		this._linkIdCondition += ("linkId = '" + kwArgs.links[linkIndex].getId() + "'");        		
        	}        	
        	this._linkIdCondition += ")";        	
        }      
    },
    
    //////////////////////////////////////////////////////////////
	// dojo.data.api.Identity
	//////////////////////////////////////////////////////////////
	
	getIdentity: function(/*Object*/ item){
		// summary:
		//		Returns a unique identifier for an item. The return value will be
		//		either a string or something that has a toString() method.
		// item:
		//		The item from the store from which to obtain its identifier.
		// exceptions:
		//		Conforming implementations may throw an exception or return null if
		//		item is not an item.
		if (!this.isItem(item)){
  			throw new Error("Specified item argument is not a valid item.");
  		}
  		
  		var identity = item.getId();
		return identity;		
    },
    
	/*=====
	_onItem: function(item) {},
	=====*/
	
	/*=====
	_onError: function(error) {},
	=====*/
    
    /*=====
	fetchItemByIdentity_kwArgs: function(identity, onItem, onError, scope) {
		// summary:
		//		An anonymous object that defines the item to locate and callbacks to invoke when the
		//		item has been located and load has completed.
		// identity: String|Object
		//		The identity parameter is the identity of the item you wish to locate and load
		//		This attribute is required. It should be a string or an object that toString()
		//		can be called on.
		// onItem: _onItem
		//		The onItem parameter is the callback to invoke when the item has been loaded.
		//		It takes only one parameter, the item located, or null if none found.
		// onError: _onError
		//		The onError parameter is the callback to invoke when the item load encountered an error.
		//		It takes only one parameter, the error object.
		// scope: Object
		//		If a scope object is provided, all of the callback functions (onItem, onError, etc)
		//		will be invoked in the context of the scope object.
	},
	=====*/
	
 	fetchItemByIdentity: function(/*fetchItemByIdentity_kwArgs*/ kwArgs) {
 		// summary:
		//		Given the identity of an item, this method returns the item that has
		//		that identity through the onItem callback. Conforming implementations
		//		should return null if there is no item with the given identity.
		//		Implementations of fetchItemByIdentity() may sometimes return an item
		//		from a local cache and may sometimes fetch an item from a remote server.
    	var args = {
 			identity: null,
 			onItem: function(item) {},
 			onError: function(error) {},
 			scope: null
 		};
 		dojo.mixin(args, kwArgs);
 		
 		var item = null;
 		var error = null;
 		if (args.identity) {
 			try {
 				if (this.dataset) {
 					item = this.dataset.getRelationship({relationshipId: args.identity, properties: this.defaultProperties});
 				}
  			}
  			catch(err) {
  				error = err; 				
  			}  			
  		}  		
  		
 		if (item) {
 			args.onItem.call(args.scope, item);
 		}
 		else {
 			args.onError.call(args.scope, error ? error : "Could not fetch item for identity=" + args.identity);
 		}		
	},
	
	
	//////////////////////////////////////////////////////////////
	// dojo.data.api.Read
	//////////////////////////////////////////////////////////////
	
	/*=====
	fetch_onBegin: function(size, request){},
	=====*/
	
	/*=====
	fetch_onItem: function(item, request){},
	=====*/
	
	/*=====
	fetch_onComplete: function(items, request){},
	=====*/
	
	/*=====
	fetch_onError: function(errorData, request){},
	=====*/
	
	/*=====
	fetch_kwArgs: function(query, queryOptions, onBegin, onItem, onComplete, onError, scope,
							start, count, sort) {
		// summary:
		//		The kwArgs parameter may either be an instance of conforming to 
		//		dojo.data.api.Request or may be a simple anonymous object
		//		containing any of the above parameters.
		// query: String|Object
		//		The query parameter is used to specify search and filter condition information for the fetch.
		//		Only those items that match the search and/or filter will be returned.
		//		|	{	ids: {
		//		|			propId: boolean		// indicates propIds involved in the expression
		//		|		},
		//		|		expression: string,		// expression involiving propId's and values  (and/or together)
		//		|								// example:  ((Name contains 'abc' || Type contains 'abc)) and (Size > 100)
		//		|
		//		|		// the following is for compatability with legacy datastore
		//		| 		navModel: {
		//		|			search: {
		//		|				term: string,		the search text
		//		|				attributes: array	["property id1", "property id2", ...]
		//		|			}
		//		|		}
		//		|	}
		// queryOptions: Object?
		//		The queryOptions parameter is an optional parameter that can be used to modify the query.  The
		//		ignoreCase case can be specified to globally make all of the filter conditions case sensitive/insensitive.
		//		|	{ ignoreCase: boolean
		//		|	}
		// onBegin: fetch_onBegin
		//		If an onBegin callback function is provided, the callback function will be called just once, 
		//		before the first onItem callback is called. The onBegin callback function will be passed two arguments, 
		//		the the total number of items identified and the Request object.  If the total number is
		//		unknown, then size will be -1.  Note that size is not necessarily the size of the collection of items 
		//		returned from the query, as the request may have specified to return only a subset of the total set 
		//		of items through the use of the start and count parameters.
		// onItem: fetch_onItem
		//		If an onItem callback function is provided, the callback function will be called as each item 
		//		in the result is received. The callback function will be passed two arguments: the item itself, 
		//		and the Request object.
		// onComplete: fetch_onComplete
		//		If an onComplete callback function is provided, the callback function will be called just once,
		//		after the last onItem callback is called. Note that if the onItem callback is not present,
		//		then onComplete will be passed an array containing all items which matched the query and the 
		//		request object. If the onItem callback is present, then onComplete is called as: onComplete(null, request).
		// onError: fetch_onError
		//		If an onError callback function is provided, the callback function will be called if there is any 
		//		sort of error while attempting to execute the query. The onError callback function will be passed two arguments:
		//		an Error object and the Request object.
		// scope: Object
		//		If a scope object is provided, all of the callback functions (onItem, onComplete, onError, etc) 
		//		will be invoked in the context of the scope object.
		// start: int
		//		If a start parameter is specified, this is a indication to the datastore to
		//		only start returning items once the start number of items have been located and
		//		skipped.  When this parameter is paired with 'count', the store should be able
		//		to page across queries with millions of hits by only returning subsets of the
		//		hits for each query
		// count: int
		//		 If a count parameter is specified, this is a indication to the datastore to
		//		 only return up to that many items.  This allows a fetch call that may have
		//		millions of item matches to be paired down to something reasonable.
		// sort: Object[]
		//		If a sort parameter is specified, this is a indication to the datastore to
		//		sort the items in some manner before returning the items.  The array is an array of
		//		javascript objects that must conform to the following format to be applied to the
		//		fetching of items:
		//		|	{
		//		|		attribute: attribute || attribute-name-string,
		//		|		descending: true|false;   // Optional.  Default is false.
		//		|	}
	},
	=====*/
	
    fetch: function(/*fetch_kwArgs*/ kwArgs){
		// summary:
		//		Given a query and set of defined options, such as a start and count of items to return,
		//		this method executes the query and makes the results available as data items.
		//		The format and expectations of stores is that they operate in a generally asynchronous
		//		manner, therefore callbacks are always used to return items located by the fetch parameters.
		// description:
		//		A Request object will always be returned and is returned immediately.
		//		The basic request is nothing more than the keyword args passed to fetch and
		//		an additional function attached, abort().  The returned request object may then be used
		//		to cancel a fetch.  All data items returns are passed through the callbacks defined in the
		//		fetch parameters and are not present on the 'request' object.
		// returns:
		//		The fetch() method will return a javascript object conforming to the API
		//		defined in dojo.data.api.Request.  In general, it will be the keywordArgs
		//		object returned with the required functions in Request.js attached.
		//		Its general purpose is to provide a convenient way for a caller to abort an
		//		ongoing fetch.
		// exceptions:
		//		Throws an exception if the query is not valid, or if the query is required but was not supplied.
		var args = {
			query: null,
			queryOptions: null,
			onBegin: function(size, request) {},
			onItem: function(item, request) {},
			onComplete: function(items, request){},
			onError: function(errorData, request){},
			scope: null,			
			start: 0,
			count: null, //default should be to get all
			sort: [],
			abort: function() {}
		};
		dojo.mixin(args, kwArgs);
		
		if (this._trace.isFinest()) {
			this._trace.finest("dataset=" + this.dataset.getId() + " start=" + args.start + " count=" + args.count, "fetch");
		}
		
		// make sure event handling is set up
		this._validateEventHandler();
		
		// TED refresh is occurring
		this._clearCache = false;
		if (this.isRefresh()) {
			this._clearCache = this.dataset.isClearCacheOnRefresh();						
		}
		
		// translate information from the args
		var sort = this._getSort(args.sort);
		var condition = this._getCondition(args.query);				
				
		// append the linkId filter condition
		if (this._linkIdCondition) {
			if (condition) {
				condition += (this._linkIdCondition + " and (" + condition + ")");				
			}
			else {
				condition = this._linkIdCondition;
			}
		}
		
		// if relationships are being viewed without context that includes items;
		// then we will have to fetch all the items to avoid fetching them one at
		// a time when the source and target items are evaluated
		if (!this.dataset.hasCachedItems()) {
			this.dataset.getItems({sync: true});
		}
		
		var relArgs = {
			onComplete: dojo.hitch(this, function(relationships, relArgs) {
				args.onBegin.call(args.scope, relArgs.filteredRows, args);
				
				if (relationships) {
					var relsLength = relationships.length;
					for(var relIndex = 0;  relIndex < relsLength;  ++relIndex) {                 	                	
                		args.onItem.call(args.scope, relationships[relIndex], args);                	
					}
				}
				
				args.onComplete.call(args.scope, relationships, args);				
			}),
			
			onError: dojo.hitch(this, function(error, exception, ioArgs) {
				args.onError.call(args.scope, error, args);
			}),
			
			clearCache: this._clearCache,
			start: args.start,
			count: args.count,	
			properties: this.defaultProperties,
			sort: sort,
			condition: condition
		}; 		
		this.dataset.getRelationships(relArgs);	
		       
        return args;
    },
    
    getAttributes: function(/*Object*/ item) {
		// summary:
		//		Returns an array with all the attributes that this item has.  This
		//		method will always return an array; if the item has no attributes
		//		at all, getAttributes() will return an empty array: [].
		//		Throws an exception if *item* is not an item
		// item:
		//		The item to access attributes on.
		//		ibm.tivoli.navmodel.Item object
		var attributes = this.inherited(arguments);  		
		attributes.push("label");
 		attributes.push("sourceItem");
		attributes.push("targetItem");
		attributes.push("identity1");
		attributes.push("identity2");

		return attributes;		
    },
    
    getValue: function(/*Object*/ item, /*String*/ attribute, /*Object?*/ defaultValue) {
		// summary:
		//		Returns a single attribute value. Returns defaultValue if and only if item does not 
		//		have a value for attribute. Returns null if and only if null was explicitly set as the 
		//		attribute value. Returns undefined if and only if the item does not have a value for the 
		//		given attribute (which is the same as saying the item does not have the attribute).
		// item:
		//		The item to access attributes on.
		//		ibm.tivoli.navmodel.Item object
		// attribute:
		//		The attribute to access represented as a string.
		// defaultValue:
		//		Optional. A default value to use for the getValue return if the attribute does not exist  
		//		or has no value.
		// exceptions:		
		//		Throws an exception if *item* is not an item, or *attribute* is not a string 
        if (!this.isItem(item)) {
            throw new Error("Specified item argument is not a valid item.");
        }
        
        if (!dojo.isString(attribute)) {
            throw new Error("Specified attribute argument is not a string.");
        }  
        
        if (this.returnItem) {
        	return item;
        }
        
        var value = defaultValue;       
        switch(attribute) {
        	case "label":
        		value = item.getLabel();
        		break;
        		
        	case "sourceItem":
        		var sourceItem = item.getSourceItem();
				value = (sourceItem ? sourceItem.getLabel() : "");
				break;
        		
        	case "targetItem":
        		var targetItem = item.getTargetItem();
				value = (targetItem ? targetItem.getLabel() : "");
				break;
    		
    		case "identity1":
    			value = item.getSourceItemId();
        		break;
        		
        	case "identity2":
        		value = item.getTargetItemId();
        		break;
        		
    		default:
    			var prop = item.getProperty(attribute);
        		if (prop) {
        			value = prop.getDisplayValue();
        		}        		
        }
        
        return value;
    },
    
    /*=====
	loadItem_kwArgs: function(item, onItem, onError, scope) {
		// summary:
		//		An anonymous object that defines the item to load and callbacks to invoke when the
		//		load has completed.
		// item: Object
		//		The item parameter is an object that represents the item in question that should be
		//		contained by the store.  This attribute is required.
		//		ibm.tivoli.navmodel.Item object
		// onItem: _onItem
		//		The onItem parameter is the callback to invoke when the item has been loaded.  
		//		It takes only one parameter, the error object
		// onError: _onError
		//		The onError parameter is the callback to invoke when the item load encountered an error.
		//		It takes only one parameter, the error object.
		// scope: Object
		//		If a scope object is provided, all of the callback functions (onItem, onError, etc)
		//		will be invoked in the context of the scope object.
	},
	=====*/
	
 	loadItem: function(/*loadItem_kwArgs*/ kwArgs) {
		// summary:
		//		Given an item, this method loads the item so that a subsequent call to store.isItemLoaded(item) 
		//		will return true.  If a call to isItemLoaded() returns true before loadItem() is even called, 
		//		then loadItem() need not do any work at all and will not even invoke the callback handlers.  
		//		So, before invoking this method, check that the item has not already been loaded.
		var args = {
			item : null,
			onItem: function(item) {},
			onError: function(error){},
			scope: null
		};		
		dojo.advancedMixin(args, kwArgs);
	
		if (!this.isItem(args.item)){
  			throw new Error("Specified item argument is not a valid item.");
		}
  		
		if (this.isItemLoaded(args.item)){
			return args.item;
		}
		
		try {
			var dataset = args.item.getDataset();
			var item = dataset.getRelationship({relationshipId: args.item.getId(), properties: this.defaultProperties});
			args.onItem.call(args.scope, item);
		}
		catch(err) {
			args.onError.call(args.scope, err);
		}
	},
	
	
	//////////////////////////////////////////////////////////////
	// private methods
	///////////////////////////////////////////////////////////////
	
     _getObjectTypes: function() {
		// summary:
		//		Returns String[] of object types that can in the datastore
    	return {"relationship" : true};
    },
        
    _handleRelationshipPropertiesChangedEvent: function(/*Object*/ event) {
		var relationships = event.getRelationships();    	
    	var propsArray = event.getProperties();			
		if (relationships && propsArray) {
			var refresh = false;
    		var length = relationships.length;			
			for (var index = 0;  index < length;  ++index) {
				var properties = propsArray[index];				
				if (properties) {
					var propsLength = properties.length;
					for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
						var relationship = relationships[index];
						var property = properties[propIndex];
						var propId = property.getId();
						if (this._trace.isFinest()) {
							this._trace.finest("datasetId=" + event.getDataset().getId() + " relationshipId=" + relationship.getId() + " prop=" + propId + " value=" + property.getDisplayValue(), "_handleRelationshipPropertiesChangedEvent");
						}
						
						this.onSet(relationship, propId, null, property.getDisplayValue());
						
						if (!refresh && (this._affectsSort(propId) || this._affectsCondition(propId))) {
							refresh = true;							
						}
					}										
				}				
			}
			
			if (refresh) {
				this.onRefresh();
			}	
		}
	},
	
	_handleLinksAddedEvent:function(/*Object*/ event) {
		var links = event.getLinks();
		if (links) {
			var linksLength = links.length;
			for (var linkIndex = 0;  linkIndex < linksLength;  ++linkIndex) {
				var link = links[linkIndex];
				var relationships = link.getRelationships();
				if (relationships) {
					relsLength = relationships.length;			
					for (var relIndex = 0;  relIndex < relsLength;  ++relIndex) {
						var relationship = relationships[relIndex];
						if (this._trace.isFinest()) {
							this._trace.finest("datasetId=" + event.getDataset().getId() + " relationshipId=" + relationship.getId(), "_handleLinksAddedEvent");
						}
						this.onNew(relationship);
					}
				}
			}
			
			if (this._hasSort() || this._hasCondition()) {
				this.onRefresh();
			}
		}
	},
	
	_handleLinksRemovedEvent: function(/*Object*/ event) {
		var links = event.getLinks();
		if (links) {
			var linksLength = links.length;
			for (var linkIndex = 0;  linkIndex < linksLength;  ++linkIndex) {
				var link = links[linkIndex];
				var relationships = link.getRelationships();
				if (relationships) {
					relsLength = relationships.length;			
					for (var relIndex = 0;  relIndex < relsLength;  ++relIndex) {
						var relationship = relationships[relIndex];
						if (this._trace.isFinest()) {
							this._trace.finest("datasetId=" + event.getDataset().getId() + " relationshipId=" + relationship.getId(), "_handleLinksRemovedEvent");
						}
						this.onDelete(relationship);
					}
				}
			}			
		}
	},
	
	_handleLinkChangedEvent:function(/*Object*/ event){		
		var relationships = event.getLinkChangedRelationships();	
		if (relationships) {
			// reason can be "relationshipAdded" or "relationshipRemoved"
			var reason = event.getLinkChangedReason();			
			var relsLength = relationships.length;
			for (var relIndex = 0;  relIndex < relsLength;  ++relIndex) {			
				if (reason === "relationshipAdded") {
					this.onNew(relationships[relIndex]);
				}
				else if (reason === "relationshipRemoved") {
					this.onDelete(relationships[relIndex]);
				}
			}
			
			if (this._hasSort() || this._hasCondition()) {
				this.onRefresh();
			}			
		}
	},
	
	_handleItemPropertiesChangedEvent: function(/*Object*/ event) {
		var items = event.getItems();
		var propsArray = event.getProperties();			
		if (items && propsArray) {
			var itemIds = [];
			var itemValues = [];
			
			// see if any "itemName" properties changed;  we use these to 
			// display the "sourceItem" and "targetItem" values
    		var length = items.length;			
			for (var index = 0;  index < length;  ++index) {
				var properties = propsArray[index];				
				if (properties) {
					var propsLength = properties.length;
					for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
						if (properties[propIndex].getValueType() === "itemName") {
							itemIds.push(items[index].getId());
							itemValues.push(properties[propIndex].getDisplayValue());
							if (this._trace.isFinest()) {
								this._trace.finest("datasetId=" + event.getDataset().getId() + " itemName property changed for itemId=" + items[index].getId(), "_handleItemPropertiesChangedEvent");
							}
							break;
						}
					}										
				}				
			}	
			
			// go check relationships that might be affected
			if (itemIds.length > 0) {
				var fetchArgs = {
					onComplete: dojo.hitch(this, function(relationships, args) {
						if (relationships) {
							var relsLength = relationships.length;
							for (var relIndex = 0;  relIndex < relsLength;  ++relIndex) {
								var sourceItemId = relationships[relIndex].getSourceItemId();
								var targetItemId = relationships[relIndex].getTargetItemId();
								var idLength = itemIds.length;
								
								for (var idIndex = 0;  idIndex < idLength;  ++idIndex) {
									if (sourceItemId && sourceItemId === itemIds[idIndex]) {
										this.onSet(relationships[relIndex], "sourceItem", null, itemValues[idIndex]);
									}
									else if (targetItemId && targetItemId === itemIds[idIndex]) {
										this.onSet(relationships[relIndex], "targetItem", null, itemValues[idIndex]);
									}
								}																
							}
						}
					}),
					start: 0,
					count: null
				};
				this.fetch(fetchArgs);
			}			
		}
	}
});
});

},
'dojox/collections/_base':function(){
define(["dojo/_base/kernel", "dojo/_base/lang", "dojo/_base/array"], 
  function(dojo, lang, arr){
	var collections = lang.getObject("dojox.collections", true);

/*=====
	collections = dojox.collections;
=====*/

	collections.DictionaryEntry=function(/* string */k, /* object */v){
		//	summary
		//	return an object of type dojox.collections.DictionaryEntry
		this.key=k;
		this.value=v;
		this.valueOf=function(){
			return this.value; 	//	object
		};
		this.toString=function(){
			return String(this.value);	//	string
		};
	}

	/*	Iterators
	 *	The collections.Iterators (Iterator and DictionaryIterator) are built to
	 *	work with the Collections included in this module.  However, they *can*
	 *	be used with arrays and objects, respectively, should one choose to do so.
	 */
	collections.Iterator=function(/* array */a){
		//	summary
		//	return an object of type dojox.collections.Iterator
		var position=0;
		this.element=a[position]||null;
		this.atEnd=function(){
			//	summary
			//	Test to see if the internal cursor has reached the end of the internal collection.
			return (position>=a.length);	//	bool
		};
		this.get=function(){
			//	summary
			//	Get the next member in the collection.
			if(this.atEnd()){
				return null;		//	object
			}
			this.element=a[position++];
			return this.element;	//	object
		};
		this.map=function(/* function */fn, /* object? */scope){
			//	summary
			//	Functional iteration with optional scope.
			return arr.map(a, fn, scope);
		};
		this.reset=function(){
			//	summary
			//	reset the internal cursor.
			position=0;
			this.element=a[position];
		};
	}

	/*	Notes:
	 *	The DictionaryIterator no longer supports a key and value property;
	 *	the reality is that you can use this to iterate over a JS object
	 *	being used as a hashtable.
	 */
	collections.DictionaryIterator=function(/* object */obj){
		//	summary
		//	return an object of type dojox.collections.DictionaryIterator
		var a=[];	//	Create an indexing array
		var testObject={};
		for(var p in obj){
			if(!testObject[p]){
				a.push(obj[p]);	//	fill it up
			}
		}
		var position=0;
		this.element=a[position]||null;
		this.atEnd=function(){
			//	summary
			//	Test to see if the internal cursor has reached the end of the internal collection.
			return (position>=a.length);	//	bool
		};
		this.get=function(){
			//	summary
			//	Get the next member in the collection.
			if(this.atEnd()){
				return null;		//	object
			}
			this.element=a[position++];
			return this.element;	//	object
		};
		this.map=function(/* function */fn, /* object? */scope){
			//	summary
			//	Functional iteration with optional scope.
			return arr.map(a, fn, scope);
		};
		this.reset=function() {
			//	summary
			//	reset the internal cursor.
			position=0;
			this.element=a[position];
		};
	};

	return collections;
});

},
'ibm/tivoli/navmodel/AbstractWidgetDef':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/WidgetDelta,ibm/tivoli/navmodel/Utils,ibm/tivoli/navmodel/Exception,ibm/tivoli/navmodel/Trace"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.AbstractWidgetDef"); 
 
// includes
dojo.require("ibm.tivoli.navmodel.WidgetDelta");
dojo.require("ibm.tivoli.navmodel.Utils");
dojo.require("ibm.tivoli.navmodel.Exception");
dojo.require("ibm.tivoli.navmodel.Trace");

ibm.tivoli.navmodel.AbstractWidgetDef._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.AbstractWidgetDef");

////////////////////////////////////
// AbstractWidgetDef
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.AbstractWidgetDef", null, {
	// private members       
	_args: null,
	_parentWidgetDef: null,	
	_widgetDef: null,
	_userDelta: null,
	_sharedDelta: null,
	_destroyed: false,
	_trace: ibm.tivoli.navmodel.AbstractWidgetDef._trace,

	/*=====
	 constructor_kwArgs: function(baseUrl, provider, datasource, widget, definition, type, profileID, parentWidgetDef, registryLocation, version) {
		// baseUrl: String
		//		base url for identifying rest service
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// widget: String
		//		widget type in form <type>-<version>
		// definition: String
		//		widget definition id in form <id>-<version>
		// type: String
		//		definition type:  "user", "shared", or "base"
		// profileID: String
		//		optional unique id to associate with reading and writing the
		//		"user" and "shared" deltas to the widget definition.  If not
		//		supplied; a uniqueId is created consisting of all the related
		//		information (i.e. providerType, datasourceType, widgetType, etc)
		// parentWidgetDef: ibm.tivoli.navmodel.AbstractWidgetDef  (or descendent)
		//		optional parent widget definition
		// registryLocation: String
		//		Indicates if "remote" provider definition is in local registry or is
		//		located at the remote rest service.  
		//		values are: "local" or "remote"  (Default for TIP is "local")
		// version: Float
		//		version used to communicate with rest service
	},
	=====*/

	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs AbstractWidgetDef object.
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		this._args = {
			baseUrl: null,
			provider: null,
			datasource: null,
			widget: null,
			definition: null,
			type: "user",
			profileID: null,
			parentWidgetDef: null,
			// TIP policy is to have remote provider widget definitions in local database
			registryLocation: (ibm.tivoli.navmodel.Utils.isTIP() ? "local" : null),
			version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion()
		};
		dojo.mixin(this._args, kwArgs);
		
		// convert datasource of "" to null
		if (this._args.datasource && this._args.datasource.length === 0) {
			this._args.datasource = null;
		}
		
		this._parentWidgetDef = this._args.parentWidgetDef;
		this._fetchWidgetDef();
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	
	validateDefaults: function() {
		// summary:
		//		This method is called after the widget def has been read from the server.
		//		Sub-class can override this method and validate/set any defaults that were
		//		not present in the widget def contents
	},
	
	getParentWidgetDef: function() {
		// summary:
		//		Returns parent widget definition 
		return this._parentWidgetDef;
	},
	
	setParentWidgetDef: function(parent) {
		// summary:
		//		Set parent widget definition; will force a
		//		re-fetch of the widget definition from server if
		//		parent has a profileID set and this definition does not.
		this._parentWidgetDef = parent;
		
		// if we do NOT have a profileID; and the new parent does;
		// then we need to re-fetch with the parents profileID
		if (parent && !this._args.profileID && parent.getProfileID()) {
			this._fetchWidgetDef();
		} 
	},
	
	getProfileID: function() {
		// summary:
		//		Returns delta id value if specified;  will inherit value from
		//		parent widget definition chain if not explicitly set
		var profileID = null;
		
		// profileID explicitly declared		
		if (this._args.profileID) {
			profileID = this._args.profileID;
		}
		// inherit profileID from parent
		else if (this._parentWidgetDef) {
			profileID = this._parentWidgetDef.getProfileID();			
		}
		
		return profileID;
	},
	
	setProfileID: function(/*String*/ profileID) {
		// summary:
		//		Set new profileID for the widget definition;
		//		will force a re-fetch of the widget definition from server
		this._args.profileID = profileID;
		this._fetchWidgetDef();
	},	
			
	getArg: function(key) {
		// summary:
		//		Returns a specific initial argument passed to the widget def
		return this._args[key];
	},
	
	destroy: function() {
		// summary:
		//		Do any necessary clean up 
		this._destroyed = true;
	},

	isDestroyed: function() {
		// summary:
		//		Returns true if the widget def has already been destroyed
		return this._destroyed;
	},

	getDef: function() {
		// summary:
		//		Returns the entire widget definition
		if (this._widgetDef) {
			return this._widgetDef;
		}

		return null;
	},

	getSection: function(section) {
		// summary:
		//		Returns the requested section of the widget definition
		if (this._widgetDef) {
			return this._widgetDef[section];
		}

		return null;
	},
	
	/////////////////////////////
	// meta section data
	/////////////////////////////
	
	getMeta: function() {
		return this._widgetDef.meta;		
	},
	
	getWidgetId: function() {
		return this._widgetDef.meta.id;
	},
	
	getWidgetType: function() {
		return this._widgetDef.meta.type;
	},
	
	getWidgetVersion: function() {
		return this._widgetDef.meta.version;
	},
	
	getWidgetLabel: function() {
		return this._widgetDef.meta.label.value;
	},
	
	getWidgetDescription: function() {
		return this._widgetDef.meta.description.value;
	},
	
	setMeta: function(meta) {
		this._widgetDef.meta = meta;		
	},
	
	setWidgetId: function(id) {
		this._widgetDef.meta.id = id;
	},
	
	setWidgetType: function(type) {
		this._widgetDef.meta.type = type;
	},
	
	setWidgetVersion: function(version) {
		this._widgetDef.meta.version = version;
	},
	
	setWidgetLabel: function(value) {
		this._widgetDef.meta.label.value = value;
	},
	
	setWidgetDescription: function(value) {
		this._widgetDef.meta.description.value = value;
	},

	/////////////////////////////
	// provider section data
	/////////////////////////////
	
	getProvider: function() {
		return this._widgetDef.provider;		
	},

	getProviderType: function() {
		return this._widgetDef.provider.type;
	},
	
	getProviderLabel: function() {
		return this._widgetDef.provider.label.value;
	},
	
	getProviderDescription: function() {
		return this._widgetDef.provider.description.value;
	},
	
	setProvider: function(provider) {
		this._widgetDef.provider = provider;		
	},

	setProviderType: function(type) {
		this._widgetDef.provider.type = type;
	},
	
	setProviderLabel: function(value) {
		this._widgetDef.provider.label.value = value;
	},
	
	setProviderDescription: function(value) {
		this._widgetDef.provider.description.value = value;
	},
	
	/////////////////////////////
	// datasource section data
	/////////////////////////////
	
	getDatasource: function() {
		return this._widgetDef.datasource;		
	},
	
	getDatasourceType: function() {
		if (this._widgetDef.datasource) {
			return this._widgetDef.datasource.type;
		}
		
		return null;
	},
	
	getDatasourceLabel: function() {
		if (this._widgetDef.datasource) {
			return this._widgetDef.datasource.label.value;
		}
		
		return null;
	},
	
	getDatasourceDescription: function() {
		if (this._widgetDef.datasource) {
			return this._widgetDef.datasource.description.value;
		}
		
		return null;
	},
	
	setDatasource: function(datasource) {
		this._widgetDef.datasource = datasource;		
	},
	
	setDataSetDatasourceType: function(type) {
		this._widgetDef.datasets[0].datasourceType = type;
	}, 
	
	setDatasourceType: function(type) {
		this._widgetDef.datasource.type = type;
	},
	
	setDatasourceLabel: function(value) {
		this._widgetDef.datasource.label.value = value;
	},
	
	setDatasourceDescription: function(value) {
		this._widgetDef.datasource.description.value = value;
	},	

	/////////////////////////////
	// details section data
	/////////////////////////////
	
	getDetails: function() {
		return this._widgetDef.details;		
	},
	
	getDetailsAttribute: function(attr) {
		return this._widgetDef.details[attr];
	},
	
	setDetails: function(details) {
		this._widgetDef.details = details;
	},
	
	setDetailsAttribute: function(key, value) {
		if (!this._widgetDef.details) {
			this._widgetDef.details = {};
		}
		
		this._widgetDef.details[key] = value;
	},
		
	/////////////////////////////
	// WidgetDelta
	/////////////////////////////
	getWidgetDelta: function(/*String*/ type) {
		// summary:
		//		Get ibm.tivoli.navmodel.WidgetDelta object
		//	type: String
		//		The delta type: "user" or "shared";  defaults to the 
		//		type passed to widget def constructor if not supplied.		
		if (!type) {
			type = this._args.type;			
		}
		
		if (!type || type === "base") {
			type = "user";
		}
		
		var delta = null;
		switch(type) {
			case "user":
				if (!this._userDelta) {
					this._userDelta = new ibm.tivoli.navmodel.WidgetDelta({widgetDef: this, type: type});
				}
				delta = this._userDelta;
				break;
			case "shared":
				if (!this._sharedDelta) {
					this._sharedDelta = new ibm.tivoli.navmodel.WidgetDelta({widgetDef: this, type: type});
				}
				delta = this._sharedDelta;
		}
				
		return delta;
	},
	
	saveWidgetDelta: function() {
		// summary:
		//		Save pending changes on ibm.tivoli.navmodel.WidgetDelta object(s)
		if (this._userDelta && this._userDelta.hasPendingChanges()) {
			this._userDelta.save();
		}
		
		if (this._sharedDelta && this._sharedDelta.hasPendingChanges()) {
			this._sharedDelta.save();
		}
	},
	
	hasPendingChanges: function() {
		// summary:
		//		Returns true if ibm.tivoli.navmodel.WidgetDelta object(s) have pending changes
		return ((this._userDelta && this._userDelta.hasPendingChanges()) || (this._sharedDelta && this._sharedDelta.hasPendingChanges()));
	},
	
	/////////////////////////////
	// raw user delta
	/////////////////////////////
	hasUserDelta: function() {
		// summary:
		//		indicates if raw user delta json object currently exists
		if (this._widgetDef.user) {
			return true;
		}
		
		return false;
	},
	
	getUserDelta: function() {
		// summary:
		//		Get raw user delta json object		
		return this._widgetDef.user;
	},
	
	setUserDelta: function(userDelta) {
		// summary:
		//		PUT raw user delta json object to the server	
		this._putDelta(userDelta, "user");
		this._fetchWidgetDef();
	},
	
	deleteUserDelta: function() {
		// summary:
		//		DELETE raw user delta json object on server
		this._deleteDelta("user");
		this._fetchWidgetDef();
	},
	
	/////////////////////////////
	// raw shared delta
	/////////////////////////////
	hasSharedDelta: function() {
		// summary:
		//		indicates if raw shared delta json object currently exists
		if (this._widgetDef.shared) {
			return true;
		}
		
		return false;
	},
	
	getSharedDelta: function() {
		// summary:
		//		Get raw shared delta json object		
		return this._widgetDef.shared;
	},
	
	setSharedDelta: function(sharedDelta) {
		// summary:
		//		PUT raw shared delta json object to the server	
		this._putDelta(sharedDelta, "shared");
		this._fetchWidgetDef();
	},
	
	deleteSharedDelta: function() {
		// summary:
		//		DELETE raw shared delta json object on server
		this._deleteDelta("shared");
		this._fetchWidgetDef();
	},
	
	/////////////////////////////
	// abstract methods
	/////////////////////////////
	getSelectionMode: function() {
		return null;
	},
	
	getItemStyles: function() {
		return null;		
	},
	
	getItemStyle: function(type) {
		return null;
	},
	
	getDefaultItemStyle: function() {
		return null;
	},
	
	getLinkStyles: function() {
		return null;		
	},
	
	getLinkStyle: function(type) {
		return null;		
	},

	getDefaultLinkStyle: function() {
		return null;
	},
	
	getItemStatuses: function() {
		return null;
	},
	
	getItemStatus: function(id) {
		return null;
	},
	
	getDefaultItemStatus: function() {
		return null;
	},
	
	getLinkStatuses: function() {
		return null;	
	},
	
	getLinkStatus: function(id) {
		return null;
	},
	
	getDefaultLinkStatus: function() {
		return null;
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////
	
	_version: function() {
		return this._args.version;
	},
	
	_fetchWidgetDef: function() {
		var exception = null;
		var profileID = this.getProfileID();
		var content = this._content();
		content.type = this._args.type;
		if (profileID) {
			content.profileID = profileID;
		}
		
		dojo.xhrGet({
			url: this._getWidgetDefinitionURL(),
			headers: {
				'Accept': "application/json"
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs) {
					this._widgetDef = result;
					this._userDelta = null;
					this._sharedDelta = null;
					this.validateDefaults();		
			}),
			error: dojo.hitch (this, function(error, ioArgs){
				exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_fetchWidgetDef", this._trace, this._version());				
			})
		});
		
		if (exception) {
			throw exception;
		}
	},
	
	_putDelta: function(/*Object*/ delta, /*String*/ deltaType) {
		// summary:
		//		Save delta information for the widget definition
		// delta:
		//		contains the delta to the definition
		// deltaType:
		//		indicates the delta type:  "user" or "shared"
		var exception = null;		
		var json = dojo.toJson(delta, false);
		
		dojo.xhrPut({
			url: this._getPutURL(deltaType),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version()),
				'Content-Type': 'application/json',
				'Content-Length': json.length
			},
			handleAs: "json",
			putData: json,
			sync: true,
			load: dojo.hitch(this, function(result, ioArgs) {									
			}),
			error: dojo.hitch (this, function(error, ioArgs){
				exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_putDelta", this._trace, this._version());				
			})
		});
		
		if (exception) {
			throw exception;
		}
	},
	
	_deleteDelta: function(/*String*/ deltaType) {
		// summary:
		//		Delete delta information for the widget definition
		// deltaType:
		//		indicates the delta type:  "user" or "shared"
		var exception = null;
		var profileID = this.getProfileID();
		var content = this._content();
		content.type = deltaType;
		if (profileID) {
			content.profileID = profileID;
		}
		
		dojo.xhrDelete({
			url: this._getWidgetDefinitionURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version())
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: function(result, ioArgs) {									
			},
			error: dojo.hitch (this, function(error, ioArgs){
				exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_deleteDelta", this._trace, this._version());				
			})
		});
		
		if (exception) {
			throw exception;
		}
	},
	
	_content: function() {
		// summary:
		//		Construct a "content" object for sending an xdr request;
		//		IE will sometimes return http requests from cache without unique parms
		var content = {};
		
		// add date and random
		var now = new Date();		
		content.d = now.getTime();
		content.r = Math.random();
		
		// registryLocation if specified
		if (this._args.registryLocation) {
			content.registryLocation = this._args.registryLocation;
		}
		
		return content;
	},

	_getDatasourceURL: function() {
		// summary:
		//		Returns url for fetching a datasource
		var url = null;
		
		if (this._widgetDef) {
			var providerId = this._args.provider;
			if (this.getProviderType() === "any") {
				providerId = "any";
			}
			
			var datasourceId = this._args.datasource;
			if (this.getDatasourceType() === "any") {
				datasourceId = "any";
			}
			
			url = this._args.baseUrl + "/providers/" + encodeURIComponent(providerId);
			url += (datasourceId ? ("/datasources/" + encodeURIComponent(datasourceId)) : "");
		}
		else {
			url = this._args.baseUrl + "/providers/" + encodeURIComponent(this._args.provider);
			url += (this._args.datasource ? ("/datasources/" + encodeURIComponent(this._args.datasource)) : "");
		}
		
		return url;	
	},

	_getWidgetDefinitionURL: function() {
		// summary:
		//		Returns url for fetching a widget definition
		var url = this._getDatasourceURL();
		
		if (this._widgetDef) {
			url+= "/widgets/"+ this.getWidgetType() + "/definitions/" + this.getWidgetId() + "-" + this.getWidgetVersion();
		}
		else {
			url+= "/widgets/"+ this._args.widget + "/definitions/" + this._args.definition;			
		}
		
		return url;
	},
	
	_getPutURL: function(deltaType) {
		// summary:
		//		Build url necessary to do a put or post operation;
		//		have to add on the "type" and "version" ourself
		//		since the dojo.xhr function doesn't include the content
		var url = this._getWidgetDefinitionURL();
		var questionMark = false;
		
		// delta type
		if (!questionMark) {
			questionMark = true;
			url += "?type=" + deltaType;
		}
		else {
			url += "&type=" + deltaType;
		}
		
		// profileID if specified
		var profileID = this.getProfileID();
		if (profileID) {
			if (!questionMark) {
				questionMark = true;
				url += "?profileID=" + profileID;
			}
			else {
				url += "&profileID=" + profileID;
			}
		}
		
		// registryLocation if specified
		if (this._args.registryLocation) {
			if (!questionMark) {
				questionMark = true;
				url += "?registryLocation=" + this._args.registryLocation;
			}
			else {
				url += "&registryLocation=" + this._args.registryLocation;
			}
		}
						
		// IE will sometimes return http requests from cache without unique parms
		var now = new Date();
		if (!questionMark) {
			questionMark = true;
			url += "?d=" + now.getTime() + "&r=" + Math.random();
		}
		else {
			url += "&d=" + now.getTime() + "&r=" + Math.random();
		}
		
		return url;
	}
});

});

},
'idx/oneui/HoverHelpTooltip':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
/**
 * @name idx.oneui.HoverHelpTooltip
 * @class HoverHelpTooltip provides pop-up information that displays when users hover the mouse pointer over an help indicator.
 * @augments dijit.Tooltip
 * @see dijit.Tooltip
 **/
define(["dojo/_base/declare", "dojo/_base/fx", // fx.fadeIn fx.fadeOut
 "dojo/keys", // keys
 "dojo/_base/array", // array.forEach array.indexOf array.map
 "dojo/dom", // dom.byId
 "dojo/_base/lang", // lang.hitch lang.isArrayLike
 "dojo/_base/sniff", // has("ie")
 "dijit/focus", "dojo/_base/event", // event.stop
 "dojo/dom-geometry", // domGeometry.getMarginBox domGeometry.position
 "dijit/place", "dijit/a11y", // _getTabNavigable
 "dijit/BackgroundIframe", "dojo/dom-style", // domStyle.set, domStyle.get
 "dojo/_base/window", // win.body
 "dijit/_base/manager",	// manager.defaultDuration
	"dijit/_Widget",
	"dijit/_TemplatedMixin",
 "dijit/Tooltip", "dojo/text!./templates/HoverHelpTooltip.html", "dijit/dijit", 
 "dojo/i18n",
	"dojo/i18n!./nls/HoverHelpTooltip"], function(declare, fx, keys, array, dom, lang, has, dijitfocus, event, domGeometry, place, a11y, BackgroundIframe, domStyle, win, manager, _Widget, _TemplatedMixin, Tooltip, template, dijit, i18n){

    var HoverHelpTooltip = declare("idx.oneui.HoverHelpTooltip", Tooltip, {
        /** @lends idx.oneui.HoverHelpTooltip.prototype */
        
        showDelay: 500,
        hideDelay: 800,
        
        /**
         * Whether to show Learn more link
         * @type Boolean
         */
        showLearnMore: false,
        
        /**
         * Learn more link value
         * @type String
         */
        learnMoreLinkValue: "#updateme",      
                
        
        showCloseIcon: true,
		
		/**
		 * Focus HoverHelpTooltip once it shown.
		 * @type Boolean
		 */
		forceFocus: false,
		
        _onHover: function(/*Event*/e){
            // summary:
            //		Despite the name of this method, it actually handles both hover and focus
            //		events on the target node, setting a timer to show the HoverHelpTooltip.
            // tags:
            //		private
            if (!this._showTimer) {
                var target = e.target;
                this._showTimer = setTimeout(lang.hitch(this, function(){
                    this.open(target)
                }), this.showDelay);
            }
            
            if (this._hideTimer) {
                clearTimeout(this._hideTimer);
                delete this._hideTimer;
            }
        },
        
        _onUnHover: function(/*Event*/ /*===== e =====*/){
            // summary:
            //		Despite the name of this method, it actually handles both mouseleave and blur
            //		events on the target node, hiding the HoverHelpTooltip.
            // tags:
            //		private
            
            // keep a HoverHelpTooltip open if the associated element still has focus (even though the
            // mouse moved away)
            if (this._focus) {
                return;
            }
            
            if (this._showTimer) {
                clearTimeout(this._showTimer);
                delete this._showTimer;
            }
            
            if (!this._hideTimer) {
                this._hideTimer = setTimeout(lang.hitch(this, function(){
                    this.close()
                }), this.hideDelay);
            }
        },
        
        
        
        /**
         * Display the HoverHelpTooltip
         * @private
         */
        open: function(/*DomNode*/target){
            // summary:
            //		
            // tags:
            //		private
            
            if (this._showTimer) {
                clearTimeout(this._showTimer);
                delete this._showTimer;
            }
            HoverHelpTooltip.show(this.label || this.domNode.innerHTML, target, this.position, 
			!this.isLeftToRight(), this.textDir, this.showLearnMore, this.learnMoreLinkValue, 
			this.showCloseIcon, this.forceFocus);
            
            this._connectNode = target;
            this.onShow(target, this.position);
        },
        close: function(){
			// summary:
			//		Hide the tooltip or cancel timer for show of tooltip
			// tags:
			//		private

			if(this._connectNode){
				// if tooltip is currently shown
				HoverHelpTooltip.hide(this._connectNode);
				delete this._connectNode;
				this.onHide();
			}
			if(this._showTimer){
				// if tooltip is scheduled to be shown (after a brief delay)
				clearTimeout(this._showTimer);
				delete this._showTimer;
			}
		},
		_setConnectIdAttr: function(/*String|String[]*/ newId){
			// summary:
			//		Connect to specified node(s)

			// Remove connections to old nodes (if there are any)
			array.forEach(this._connections || [], function(nested){
				array.forEach(nested, lang.hitch(this, "disconnect"));
			}, this);

			// Make array of id's to connect to, excluding entries for nodes that don't exist yet, see startup()
			this._connectIds = array.filter(lang.isArrayLike(newId) ? newId : (newId ? [newId] : []),
					function(id){ return dom.byId(id); });

			// Make connections
			this._connections = array.map(this._connectIds, function(id){
				var node = dom.byId(id);
				return [
					this.connect(node, "onmouseenter", "_onHover"),
					this.connect(node, "onmouseleave", "_onUnHover"),
					this.connect(node, "onclick", "_onHover"),
					//this.connect(node, "onfocus", "_onHover"),
					this.connect(node, "onblur", "_onUnHover"),
					this.connect(node, "onkeypress", "_onConnectIdKey")
				];
			}, this);

			this._set("connectId", newId);
		},
		_onConnectIdKey: function(/*Event*/evt){
            // summary:
            //		Handler for keyboard events
            // description:
            // tags:
            //		private
            var node = evt.target;
            
            if (evt.charOrCode == keys.ENTER || evt.charOrCode == keys.SPACE || evt.charOrCode == " " || evt.charOrCode == keys.F1) {
                // Use setTimeout to avoid crash on IE, see #10396.
                 this._showTimer = setTimeout(lang.hitch(this, function(){
                    this.open(node)
                }), this.showDelay);
                
                event.stop(evt);     
            }
        }
        
    });
    
    var MasterHoverHelpTooltip = declare("idx.oneui._MasterHoverHelpTooltip", [_Widget, _TemplatedMixin], {
    //lang.extend(Tooltip._MasterTooltip, {    
    	// duration: Integer
		//		Milliseconds to fade in/fade out
		duration: manager.defaultDuration,
		
        templateString: template,
        learnMoreLabel: "", 
        
        /**
         * draggable: Boolean
         *		Toggles the moveable aspect of the HoverHelpTooltip. If true, HoverHelpTooltip
         *		can be dragged by it's grippy bar. If false it will remain positioned
         *		relative to the attached node
         *		@type boolean
         **/
        draggable: true,
        
        _firstFocusItem: null,
        _lastFocusItem: null,
        
        /**
		 * Focus HoverHelpTooltip once it hovered.
		 * @type Boolean
		 */
        hoverFocus: true,
        
				postMixInProperties: function(){
					this.learnMoreLabel = i18n.getLocalization("idx.oneui", "HoverHelpTooltip", this.lang).learnMoreLabel;
				},
        postCreate: function(){
        	
        		
        		
            win.body().appendChild(this.domNode);
            
            this.bgIframe = new BackgroundIframe(this.domNode);
            
            // Setup fade-in and fade-out functions.
            this.fadeIn = fx.fadeIn({
                node: this.domNode,
                duration: this.duration,
                onEnd: lang.hitch(this, "_onShow")
            });
            this.fadeOut = fx.fadeOut({
                node: this.domNode,
                duration: this.duration,
                onEnd: lang.hitch(this, "_onHide")
            });
            this.connect(this.domNode, "onkeypress", "_onKey");
			this.connect(this.outerContainerNode, "onmouseenter", lang.hitch(this, function(e){
				if(this.hoverFocus){this.focus();}}
			));
            
        },
        show: function(innerHTML, aroundNode, position, rtl, textDir, showLearnMore, learnMoreLinkValue, showCloseIcon, forceFocus){
        	if(forceFocus){this.focus();}
			
            if (showLearnMore) {
                this.learnMoreNode.style.display = "inline";
                this.learnMoreNode.href = learnMoreLinkValue;
            }
            else{
				this.learnMoreNode.style.display = "none";
			}
            
            if (showCloseIcon || showCloseIcon == null) 
                this.closeButtonNode.style.display = "inline";
            else {
                this.closeButtonNode.style.display = "none";
            }
            //in case connectorNode was hidden on a previous call to hide
            this.connectorNode.hidden = false;
            
            if (this.aroundNode && this.aroundNode === aroundNode && this.containerNode.innerHTML == innerHTML) {
                return;
            }
            
            
            // reset width; it may have been set by orient() on a previous HoverHelpTooltip show()
            this.domNode.width = "auto";
            
            if (this.fadeOut.status() == "playing") {
                // previous HoverHelpTooltip is being hidden; wait until the hide completes then show new one
                this._onDeck = arguments;
                return;
            }
            
            this.containerNode.innerHTML = innerHTML;
            
            this.set("textDir", textDir);
            this.containerNode.align = rtl ? "right" : "left"; //fix the text alignment
            var pos = place.around(this.domNode, aroundNode, position && position.length ? position : HoverHelpTooltip.defaultPosition, !rtl, lang.hitch(this, "orient"));
            
            // Position the HoverHelpTooltip connector for middle alignment.
            // This could not have been done in orient() since the HoverHelpTooltip wasn't positioned at that time.
            var aroundNodeCoords = pos.aroundNodePos;
            if (pos.corner.charAt(0) == 'M' && pos.aroundCorner.charAt(0) == 'M') {
                this.connectorNode.style.top = aroundNodeCoords.y + ((aroundNodeCoords.h - this.connectorNode.offsetHeight) >> 1) - pos.y + "px";
                this.connectorNode.style.left = "";
            }
            else if (pos.corner.charAt(1) == 'M' && pos.aroundCorner.charAt(1) == 'M') {
                this.connectorNode.style.left = aroundNodeCoords.x + ((aroundNodeCoords.w - this.connectorNode.offsetWidth) >> 1) - pos.x + "px";
            }
            // show it
            domStyle.set(this.domNode, "opacity", 0);
            this.fadeIn.play();
            this.isShowingNow = true;
            this.aroundNode = aroundNode;
        },
        
        
        orient: function(/*DomNode*/node, /*String*/ aroundCorner, /*String*/ HoverHelpTooltipCorner, /*Object*/ spaceAvailable, /*Object*/ aroundNodeCoords){
            // summary:
            //		Private function to set CSS for HoverHelpTooltip node based on which position it's in.
            //		This is called by the dijit popup code.   It will also reduce the HoverHelpTooltip's
            //		width to whatever width is available
            // tags:
            //		protected
            this.connectorNode.style.top = ""; //reset to default
            //Adjust the spaceAvailable width, without changing the spaceAvailable object
            var HoverHelpTooltipSpaceAvaliableWidth = spaceAvailable.w - this.connectorNode.offsetWidth;
            
            node.className = "idxOneuiHoverHelpTooltip " +
            {
                "MR-ML": "idxOneuiHoverHelpTooltipRight",
                "ML-MR": "idxOneuiHoverHelpTooltipLeft",
                "TM-BM": "idxOneuiHoverHelpTooltipAbove",
                "BM-TM": "idxOneuiHoverHelpTooltipBelow",
                "BL-TL": "idxOneuiHoverHelpTooltipBelow idxOneuiHoverHelpTooltipABLeft",
                "TL-BL": "idxOneuiHoverHelpTooltipAbove idxOneuiHoverHelpTooltipABLeft",
                "BR-TR": "idxOneuiHoverHelpTooltipBelow idxOneuiHoverHelpTooltipABRight",
                "TR-BR": "idxOneuiHoverHelpTooltipAbove idxOneuiHoverHelpTooltipABRight",
                "BR-BL": "idxOneuiHoverHelpTooltipRight",
                "BL-BR": "idxOneuiHoverHelpTooltipLeft",
                "TR-TL": "idxOneuiHoverHelpTooltipRight"
            }[aroundCorner + "-" + HoverHelpTooltipCorner];
            
            // reduce HoverHelpTooltip's width to the amount of width available, so that it doesn't overflow screen
            this.domNode.style.width = "auto";
            var size = domGeometry.getContentBox(this.domNode);
            
            var width = Math.min((Math.max(HoverHelpTooltipSpaceAvaliableWidth, 1)), size.w);
            var widthWasReduced = width < size.w;
            
            this.domNode.style.width = width + "px";
            
            //Adjust width for HoverHelpTooltips that have a really long word or a nowrap setting
            if (widthWasReduced) {
                this.containerNode.style.overflow = "auto"; //temp change to overflow to detect if our HoverHelpTooltip needs to be wider to support the content
                var scrollWidth = this.containerNode.scrollWidth;
                this.containerNode.style.overflow = "visible"; //change it back
                if (scrollWidth > width) {
                    scrollWidth = scrollWidth + domStyle.get(this.domNode, "paddingLeft") + domStyle.get(this.domNode, "paddingRight");
                    this.domNode.style.width = scrollWidth + "px";
                }
            }
            
            // Reposition the HoverHelpTooltip connector.
            if (HoverHelpTooltipCorner.charAt(0) == 'B' && aroundCorner.charAt(0) == 'B') {
                var mb = domGeometry.getMarginBox(node);
                var HoverHelpTooltipConnectorHeight = this.connectorNode.offsetHeight;
                if (mb.h > spaceAvailable.h) {
                    // The HoverHelpTooltip starts at the top of the page and will extend past the aroundNode
                    var aroundNodePlacement = spaceAvailable.h - ((aroundNodeCoords.h + HoverHelpTooltipConnectorHeight) >> 1);
                    this.connectorNode.style.top = aroundNodePlacement + "px";
                    this.connectorNode.style.bottom = "";
                }
                else {
                    // Align center of connector with center of aroundNode, except don't let bottom
                    // of connector extend below bottom of HoverHelpTooltip content, or top of connector
                    // extend past top of HoverHelpTooltip content
                    this.connectorNode.style.bottom = Math.min(Math.max(aroundNodeCoords.h / 2 - HoverHelpTooltipConnectorHeight / 2, 0), mb.h - HoverHelpTooltipConnectorHeight) +
                    "px";
                    this.connectorNode.style.top = "";
                }
            }
            else {
                // reset the HoverHelpTooltip back to the defaults
                this.connectorNode.style.top = "";
                this.connectorNode.style.bottom = "";
            }
            
            return Math.max(0, size.w - HoverHelpTooltipSpaceAvaliableWidth);
        },
        
        focus: function(){
            this._getFocusItems(this.outerContainerNode);
            this._focus = true;
            dijitfocus.focus(this._firstFocusItem);            
        },
        
        _onShow: function(){
					// summary:
					//		Called at end of fade-in operation
					// tags:
					//		protected
					if(has("ie")){
						// the arrow won't show up on a node w/an opacity filter
						this.domNode.style.filter="";
					}
				},
        
        hide: function(aroundNode){
            // summary:
            //		Hide the HoverHelpTooltip
            
            if (this._onDeck && this._onDeck[1] == aroundNode /*|| this._focus*/) {
            
                // this hide request is for a show() that hasn't even started yet;
                // just cancel the pending show()
                this._onDeck = null;
            }else if (this.aroundNode === aroundNode || this.isShowingNow) {
                // this hide request is for the currently displayed HoverHelpTooltip
                this.fadeIn.stop();
                this.isShowingNow = false;
                this.aroundNode = null;
                this.fadeOut.play();
            }
            else {
            // just ignore the call, it's for a HoverHelpTooltip that has already been erased
            }
        }, 
		
        hideOnClickClose: function(){
            // summary:
            //		Hide the HoverHelpTooltip
            // this hide request is for the currently displayed HoverHelpTooltip            
            dijitfocus.focus(this.aroundNode);
            this.fadeIn.stop();
            this.isShowingNow = false;
            this.aroundNode = null;
            this.fadeOut.play();            
        },
        _getFocusItems: function(){
            // summary:
            //		Finds focusable items in dialog,
            //		and sets this._firstFocusItem and this._lastFocusItem
            // tags:
            //		protected
            
            var elems = a11y._getTabNavigable(this.outerContainerNode.parentNode);
            this._firstFocusItem = elems.lowest || elems.first || this.closeButtonNode || this.domNode;
            this._lastFocusItem = elems.last || elems.highest || this._firstFocusItem;
        },
        _onKey: function(/*Event*/evt){
            // summary:
            //		Handler for keyboard events
            // description:
            // tags:
            //		private
            
            var node = evt.target;
            if (evt.charOrCode === keys.TAB) {
                this._getFocusItems(this.outerContainerNode);
            }
            var singleFocusItem = (this._firstFocusItem == this._lastFocusItem);
            if (evt.charOrCode == keys.ESCAPE) {
                // Use setTimeout to avoid crash on IE, see #10396.
                setTimeout(lang.hitch(this, "hideOnClickClose"), 0);
                event.stop(evt);
            }else if (node == this._firstFocusItem && evt.shiftKey && evt.charOrCode === keys.TAB) {
                if (!singleFocusItem) {
                    dijitfocus.focus(this._lastFocusItem); // send focus to last item in dialog
                }
                event.stop(evt);
            }else if (node == this._lastFocusItem && evt.charOrCode === keys.TAB && !evt.shiftKey) {
                if (!singleFocusItem) {
                    dijitfocus.focus(this._firstFocusItem); // send focus to first item in dialog
                }
                event.stop(evt);
            }else if (evt.charOrCode === keys.TAB) {
                // we want the browser's default tab handling to move focus
                // but we don't want the tab to propagate upwards
                evt.stopPropagation();
            }
        },
        _onHide: function(){
			// summary:
			//		Called at end of fade-out operation
			// tags:
			//		protected

			this.domNode.style.cssText="";	// to position offscreen again
			this.containerNode.innerHTML="";
			if(this._onDeck){
				// a show request has been queued up; do it now
				this.show.apply(this, this._onDeck);
				this._onDeck=null;
			}
		},
		
		_setAutoTextDir: function(/*Object*/node){
		    // summary:
		    //	    Resolve "auto" text direction for children nodes
		    // tags:
		    //		private

            this.applyTextDir(node, has("ie") ? node.outerText : node.textContent);
            array.forEach(node.children, function(child){this._setAutoTextDir(child); }, this);
		},
		
		_setTextDirAttr: function(/*String*/ textDir){
		    // summary:
		    //		Setter for textDir.
		    // description:
		    //		Users shouldn't call this function; they should be calling
		    //		set('textDir', value)
		    // tags:
		    //		private
	
            this._set("textDir", typeof textDir != 'undefined'? textDir : "");
    	    if (textDir == "auto"){
    	        this._setAutoTextDir(this.containerNode);
    	    }else{
    	    	if(this.textDir){
    	    		this.containerNode.dir = this.textDir;
    	    	}
    	    }  		             		        
        }
        
    }); //end declare

//    var MasterHoverHelpTooltip = Tooltip._MasterTooltip;
       
    HoverHelpTooltip._MasterHoverHelpTooltip = MasterHoverHelpTooltip;		// for monkey patching
    
    
    
    // summary:
    //		Static method to display HoverHelpTooltip w/specified contents in specified position.
    //		See description of idx.oneui.HoverHelpTooltip.defaultPosition for details on position parameter.
    //		If position is not specified then idx.oneui.HoverHelpTooltip.defaultPosition is used.
    // innerHTML: String
    //		Contents of the HoverHelpTooltip
    // aroundNode: dijit.__Rectangle
    //		Specifies that HoverHelpTooltip should be next to this node / area
    // position: String[]?
    //		List of positions to try to position HoverHelpTooltip (ex: ["right", "above"])
    // rtl: Boolean?
    //		Corresponds to `WidgetBase.dir` attribute, where false means "ltr" and true
    //		means "rtl"; specifies GUI direction, not text direction.
    // textDir: String?
    //		Corresponds to `WidgetBase.textdir` attribute; specifies direction of text.	
    HoverHelpTooltip.show = idx.oneui.showHoverHelpTooltip = function(innerHTML, aroundNode, position, rtl, textDir, showLearnMore, learnMoreLinkValue, showCloseIcon, forceFocus){
    
        if (!HoverHelpTooltip._masterTT) {
            idx.oneui._masterTT = HoverHelpTooltip._masterTT = new MasterHoverHelpTooltip();
        }
        return HoverHelpTooltip._masterTT.show(innerHTML, aroundNode, position, rtl, textDir, showLearnMore, learnMoreLinkValue, showCloseIcon, forceFocus);
    };
    
    // summary:
    //		Static method to hide the HoverHelpTooltip displayed via showHoverHelpTooltip()
    HoverHelpTooltip.hide = idx.oneui.hideHoverHelpTooltip = function(aroundNode){
    
        return HoverHelpTooltip._masterTT && HoverHelpTooltip._masterTT.hide(aroundNode);
    };
    
    HoverHelpTooltip.defaultPosition = ["after-centered", "before-centered", "below", "above"];
    
    
    return HoverHelpTooltip;
    
});

},
'gridx/modules/select/Cell':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/sniff",
	"dojo/dom-class",
	"dojo/keys",
	"./_RowCellBase",
	"../../core/_Module"
], function(declare, array, sniff, domClass, keys, _RowCellBase, _Module){

	return declare(/*===== "gridx.modules.select.Cell", =====*/_RowCellBase, {
		// summary:
		//		Provides simple cell selection.
		// description:
		//		This module provides a simple way for selecting cells by clicking or SPACE key, or CTRL + Click to select multiple cells.
		//
		// example:
		//		1. Use select api on grid cell object obtained from grid.cell(i, j)
		//		|	grid.cell(1,1).select();
		//		|	grid.cell(1,1).deselect();
		//		|	grid.cell(1,1).isSelected();
		//
		//		2. Use select api on select.cell module
		//		|	grid.select.cell.selectById(rowId, columnId);
		//		|	grid.select.cell.deSelectById(rowId, columnId);
		//		|	grid.select.cell.isSelected(rowId, columnId);		
		//		|	grid.select.cell.getSelected();//[]
		//		|	grid.select.cell.clear();

		// name: [readonly] String
		//		module name
		name: "selectCell",
		
		// cellMixin: Object
		//		A map of functions to be mixed into grid cell object, so that we can use select api on cell object directly
		//		- grid.cell(1,1).select() | deselect() | isSelected();
		cellMixin: {
			select: function(){
				this.grid.select.cell.selectById(this.row.id, this.column.id);
				return this;
			},
			
			deselect: function(){
				this.grid.select.cell.deselectById(this.row.id, this.column.id);
				return this;
			},
			
			isSelected: function(){
				return this.grid.select.cell.isSelected(this.row.id, this.column.id);
			}
		},
		
		//Public API--------------------------------------------------------------------------------
/*=====
		selectById: function(rowId, columnId){
			// summary:
			//		Select a cell by [rowId, columnId].
		},
		
		deselectById: function(rowId, columnId){
			// summary:
			//		Deselect a cell by [rowId, columnId].
		},
		
		isSelected: function(rowId, columnId){
			// summary:
			//		Check if a cell is already selected.
		},
=====*/
		
		getSelected: function(){
			// summary:
			//		Get arrays of [rowId, columnId] of all the selected cells
			var t = this, res = [];
			array.forEach(t.grid._columns, function(col){
				var ids = t.model.getMarkedIds(t._getMarkType(col.id));
				res.push.apply(res, array.map(ids, function(rid){
					return [rid, col.id];
				}));
			});
			return res;
		},
		
		clear: function(){
			// summary:
			//		Deselected all the selected cells;
			var t = this, m = t.model;
			if(t.arg('enabled')){
				array.forEach(t.grid._columns, function(col){
					var markType = t._getMarkType(col.id),
						selected = m.getMarkedIds(markType), 
						len = selected.length, i;
					for(i = 0; i < len; ++i){
						m.markById(selected[i], 0, markType);
					}
				});
				m.when();
			}
		},

	/*=====
		onSelected: function(cell){
			// summary:
			//		Fired when a cell is selected.
			// cell: gridx.core.Cell
			//		The cell object
		},

		onDeselected: function(cell){
			// summary:
			//		Fired when a cell is deselected.
			// cell: gridx.core.Cell
			//		The cell object
		},

		onHighlightChange: function(){
			// summary:
			//		Fired when a cell's highlight is changed.
			// tags:
			//		private package
		},
	=====*/

		//Private--------------------------------------------------------------------------------
		_type: 'cell',

		_markTypePrefix: "select_",
	
		_getMarkType: function(colId){
			var type = this._markTypePrefix + colId;
			this.model._spTypes[type] = 1;
			return type;
		},

		_init: function(){
			var t = this,
				g = t.grid;
			t.inherited(arguments);
			t.batchConnect(
				[g, 'onCellMouseDown', function(e){
					t._select([e.rowId, e.columnId], g._isCopyEvent(e));
				}],
				[g, sniff('ff') < 4 ? 'onCellKeyUp' : 'onCellKeyDown', function(e){
					if(e.keyCode == keys.SPACE){
						t._select([e.rowId, e.columnId], g._isCopyEvent(e));
					}
				}]);
		},

		_isSelected: function(cell){
			return this.isSelected(cell[0], cell[1]);
		},

		_onMark: function(rowId, toMark, oldState, type){
			var t = this;
			if(type.indexOf(t._markTypePrefix) === 0){
				var colId = type.substr(t._markTypePrefix.length);
				if(t.grid._columnsById[colId]){
					t._highlight(rowId, colId, toMark);
					//Fire event when the row is loaded, so users can use the cell directly.
					t.model.when({id: rowId}, function(){
						t[toMark ? 'onSelected' : 'onDeselected'](t.grid.cell(rowId, colId, 1));
					});
				}
			}
		},
		
		_highlight: function(rowId, colId, toHighlight){
			var node = this.grid.body.getCellNode({
				rowId: rowId, 
				colId: colId
			});
			if(node){
				domClass.toggle(node, "gridxCellSelected", toHighlight);
				node.setAttribute('aria-selected', !!toHighlight);
				this.onHighlightChange();
			}
		},

		_markById: function(item, toMark){
			var t = this, m = this.model;
			m.markById(item[0], toMark, t._getMarkType(item[1]));
			m.when();
		},
		
		_onRender: function(start, count){
			var t = this, model = t.model, end = start + count, i, j, rowId, colId,
				columns = t.grid._columns; 
			for(i = start; i < end; ++i){
				rowId = model.indexToId(i);
				for(j = columns.length - 1; j >= 0; --j){
					colId = columns[j].id;
					if(model.getMark(rowId, t._getMarkType(colId))){
						t._highlight(rowId, colId, 1);
					}
				}
			}
		}
	});
});

},
'ibm/usmi/widgets/util/ResourceMenuProviderHelper':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

define(
		"ibm/usmi/widgets/util/ResourceMenuProviderHelper",
		[
			"dojo/_base/kernel",
			"dojo/_base/declare",
			"dojo/topic",
			"dojo/i18n!./nls/ResourceMenuProvider",
			// TWL imports
			"ibm/tivoli/tip/navwidget/topology/dijit/MenuProviderHelper",
			"ibm/tivoli/tip/navwidget/topology/dijit/MenuProviderHelperBase",
			"ibm/usmi/widgets/util/DefaultMenuProviderHelper",
			"ibm/tivoli/tip/navwidget/dijit/DefaultMessageHandler",
			"ibm/tivoli/navmodel/Dataset"
		],
		function (dojo, declare, topic, mri, MenuProviderHelper, MenuProviderHelperBase, DefaultMenuProviderHelper,
				DefaultMessageHandler, Dataset) {

			return declare(
					[ DefaultMenuProviderHelper
					],
					{
						nls : mri,

						getMenuItems : function (/* Object */context) {
							// summary:
							// Coming soon
							// description:
							// Coming soon
							// context:
							// Coming soon
							var items = this.inherited(arguments);
							var selectedObjects = context.selection;
							if ((selectedObjects.getItemCount() + selectedObjects.getLinkCount() + selectedObjects
									.getRelationshipCount()) == 1) {

								var object = null;

								if (selectedObjects.getItemCount() == 1) {
									object = selectedObjects.getItems()[0];
								} else if (selectedObjects.getRelationshipCount() == 1) {
									object = selectedObjects.getRelationships()[0];
								} else if (selectedObjects.getLinkCount() == 1) {
									object = selectedObjects.getLinks()[0];
								} else {
									console.log("Cannot handle non link, rel, or item objects. No action preformed.");
								}

								// if the object doesnt support properties, dont
								// add prop menu item.
								if (object && object.supportsProperties()) {
									// Add the PropertiesNotebook to the Menu
									// list of items
									items
											.push({
												"actionID" : "nwmi_PropertiesNotebookDialogISC",

												"label" : ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil
														.getStringWithEllipsis(this.nls.LEGACY_PROPERTIES),
												"launchType" : "WEB_URL",
												"uri" : window.location.protocol
														+ '//'
														+ window.location.hostname
														+ ':'
														+ window.location.port
														+ '/ibm/action/launch?pageID=com.ibm.usmi.console.resnav.navigationElement&root='
														+ object.getId()
														+ '&hideTaskArea=false&disableNavArea=false&hideBannerArea=false&hideTaskBarArea=false&hint=PROPERTIES'
											});
								}

							}

							return items;
						}
					});
		});
},
'dijit/form/_Spinner':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/_base/event", // event.stop
	"dojo/keys", // keys keys.DOWN_ARROW keys.PAGE_DOWN keys.PAGE_UP keys.UP_ARROW
	"dojo/_base/lang", // lang.hitch
	"dojo/_base/sniff", // has("mozilla")
	"dijit/typematic",
	"./RangeBoundTextBox",
	"dojo/text!./templates/Spinner.html",
	"./_TextBoxMixin"	// selectInputText
], function(declare, event, keys, lang, has, typematic, RangeBoundTextBox, template, _TextBoxMixin){

/*=====
	var RangeBoundTextBox = dijit.form.RangeBoundTextBox;
=====*/

	// module:
	//		dijit/form/_Spinner
	// summary:
	//		Mixin for validation widgets with a spinner.


	return declare("dijit.form._Spinner", RangeBoundTextBox, {
		// summary:
		//		Mixin for validation widgets with a spinner.
		// description:
		//		This class basically (conceptually) extends `dijit.form.ValidationTextBox`.
		//		It modifies the template to have up/down arrows, and provides related handling code.

		// defaultTimeout: Number
		//		Number of milliseconds before a held arrow key or up/down button becomes typematic
		defaultTimeout: 500,

		// minimumTimeout: Number
		//		minimum number of milliseconds that typematic event fires when held key or button is held
		minimumTimeout: 10,

		// timeoutChangeRate: Number
		//		Fraction of time used to change the typematic timer between events.
		//		1.0 means that each typematic event fires at defaultTimeout intervals.
		//		< 1.0 means that each typematic event fires at an increasing faster rate.
		timeoutChangeRate: 0.90,

		// smallDelta: Number
		//		Adjust the value by this much when spinning using the arrow keys/buttons
		smallDelta: 1,

		// largeDelta: Number
		//		Adjust the value by this much when spinning using the PgUp/Dn keys
		largeDelta: 10,

		templateString: template,

		baseClass: "dijitTextBox dijitSpinner",

		// Set classes like dijitUpArrowButtonHover or dijitDownArrowButtonActive depending on
		// mouse action over specified node
		cssStateNodes: {
			"upArrowNode": "dijitUpArrowButton",
			"downArrowNode": "dijitDownArrowButton"
		},

		adjust: function(val /*=====, delta =====*/){
			// summary:
			//		Overridable function used to adjust a primitive value(Number/Date/...) by the delta amount specified.
			// 		The val is adjusted in a way that makes sense to the object type.
			// val: Object
			// delta: Number
			// tags:
			//		protected extension
			return val;
		},

		_arrowPressed: function(/*Node*/ nodePressed, /*Number*/ direction, /*Number*/ increment){
			// summary:
			//		Handler for arrow button or arrow key being pressed
			if(this.disabled || this.readOnly){ return; }
			this._setValueAttr(this.adjust(this.get('value'), direction*increment), false);
			_TextBoxMixin.selectInputText(this.textbox, this.textbox.value.length);
		},

		_arrowReleased: function(/*Node*/ /*===== node =====*/){
			// summary:
			//		Handler for arrow button or arrow key being released
			this._wheelTimer = null;
		},

		_typematicCallback: function(/*Number*/ count, /*DOMNode*/ node, /*Event*/ evt){
			var inc=this.smallDelta;
			if(node == this.textbox){
				var key = evt.charOrCode;
				inc = (key == keys.PAGE_UP || key == keys.PAGE_DOWN) ? this.largeDelta : this.smallDelta;
				node = (key == keys.UP_ARROW || key == keys.PAGE_UP) ? this.upArrowNode : this.downArrowNode;
			}
			if(count == -1){ this._arrowReleased(node); }
			else{ this._arrowPressed(node, (node == this.upArrowNode) ? 1 : -1, inc); }
		},

		_wheelTimer: null,
		_mouseWheeled: function(/*Event*/ evt){
			// summary:
			//		Mouse wheel listener where supported

			event.stop(evt);
			// FIXME: Safari bubbles

			// be nice to DOH and scroll as much as the event says to
			var wheelDelta = evt.wheelDelta / 120;
			if(Math.floor(wheelDelta) != wheelDelta){
				// If not an int multiple of 120, then its touchpad scrolling.
				// This can change very fast so just assume 1 wheel click to make it more manageable.
				wheelDelta = evt.wheelDelta > 0 ? 1 : -1;
			}
			var scrollAmount = evt.detail ? (evt.detail * -1) : wheelDelta;
			if(scrollAmount !== 0){
				var node = this[(scrollAmount > 0 ? "upArrowNode" : "downArrowNode" )];

				this._arrowPressed(node, scrollAmount, this.smallDelta);

				if(!this._wheelTimer){
					clearTimeout(this._wheelTimer);
				}
				this._wheelTimer = setTimeout(lang.hitch(this,"_arrowReleased",node), 50);
			}

		},

		postCreate: function(){
			this.inherited(arguments);

			// extra listeners
			this.connect(this.domNode, !has("mozilla") ? "onmousewheel" : 'DOMMouseScroll', "_mouseWheeled");
			this._connects.push(typematic.addListener(this.upArrowNode, this.textbox, {charOrCode:keys.UP_ARROW,ctrlKey:false,altKey:false,shiftKey:false,metaKey:false}, this, "_typematicCallback", this.timeoutChangeRate, this.defaultTimeout, this.minimumTimeout));
			this._connects.push(typematic.addListener(this.downArrowNode, this.textbox, {charOrCode:keys.DOWN_ARROW,ctrlKey:false,altKey:false,shiftKey:false,metaKey:false}, this, "_typematicCallback", this.timeoutChangeRate, this.defaultTimeout, this.minimumTimeout));
			this._connects.push(typematic.addListener(this.upArrowNode, this.textbox, {charOrCode:keys.PAGE_UP,ctrlKey:false,altKey:false,shiftKey:false,metaKey:false}, this, "_typematicCallback", this.timeoutChangeRate, this.defaultTimeout, this.minimumTimeout));
			this._connects.push(typematic.addListener(this.downArrowNode, this.textbox, {charOrCode:keys.PAGE_DOWN,ctrlKey:false,altKey:false,shiftKey:false,metaKey:false}, this, "_typematicCallback", this.timeoutChangeRate, this.defaultTimeout, this.minimumTimeout));
		}
	});
});

},
'dijit/Menu':function(){
define([
	"require",
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/_base/event", // event.stop
	"dojo/dom", // dom.byId dom.isDescendant
	"dojo/dom-attr", // domAttr.get domAttr.set domAttr.has domAttr.remove
	"dojo/dom-geometry", // domStyle.getComputedStyle domGeometry.position
	"dojo/dom-style", // domStyle.getComputedStyle
	"dojo/_base/kernel",
	"dojo/keys",	// keys.F10
	"dojo/_base/lang", // lang.hitch
	"dojo/on",
	"dojo/_base/sniff", // has("ie"), has("quirks")
	"dojo/_base/window", // win.body win.doc.documentElement win.doc.frames win.withGlobal
	"dojo/window", // winUtils.get
	"./popup",
	"./DropDownMenu",
	"dojo/ready"
], function(require, array, declare, event, dom, domAttr, domGeometry, domStyle, kernel, keys, lang, on,
			has, win, winUtils, pm, DropDownMenu, ready){

/*=====
	var DropDownMenu = dijit.DropDownMenu;
=====*/

// module:
//		dijit/Menu
// summary:
//		Includes dijit.Menu widget and base class dijit._MenuBase

// Back compat w/1.6, remove for 2.0
if(!kernel.isAsync){
	ready(0, function(){
		var requires = ["dijit/MenuItem", "dijit/PopupMenuItem", "dijit/CheckedMenuItem", "dijit/MenuSeparator"];
		require(requires);	// use indirection so modules not rolled into a build
	});
}

return declare("dijit.Menu", DropDownMenu, {
	// summary:
	//		A context menu you can assign to multiple elements

	constructor: function(){
		this._bindings = [];
	},

	// targetNodeIds: [const] String[]
	//		Array of dom node ids of nodes to attach to.
	//		Fill this with nodeIds upon widget creation and it becomes context menu for those nodes.
	targetNodeIds: [],

	// contextMenuForWindow: [const] Boolean
	//		If true, right clicking anywhere on the window will cause this context menu to open.
	//		If false, must specify targetNodeIds.
	contextMenuForWindow: false,

	// leftClickToOpen: [const] Boolean
	//		If true, menu will open on left click instead of right click, similar to a file menu.
	leftClickToOpen: false,

	// refocus: Boolean
	// 		When this menu closes, re-focus the element which had focus before it was opened.
	refocus: true,

	postCreate: function(){
		if(this.contextMenuForWindow){
			this.bindDomNode(win.body());
		}else{
			// TODO: should have _setTargetNodeIds() method to handle initialization and a possible
			// later set('targetNodeIds', ...) call.  There's also a problem that targetNodeIds[]
			// gets stale after calls to bindDomNode()/unBindDomNode() as it still is just the original list (see #9610)
			array.forEach(this.targetNodeIds, this.bindDomNode, this);
		}
		this.inherited(arguments);
	},

	// thanks burstlib!
	_iframeContentWindow: function(/* HTMLIFrameElement */iframe_el){
		// summary:
		//		Returns the window reference of the passed iframe
		// tags:
		//		private
		return winUtils.get(this._iframeContentDocument(iframe_el)) ||
			// Moz. TODO: is this available when defaultView isn't?
			this._iframeContentDocument(iframe_el)['__parent__'] ||
			(iframe_el.name && win.doc.frames[iframe_el.name]) || null;	//	Window
	},

	_iframeContentDocument: function(/* HTMLIFrameElement */iframe_el){
		// summary:
		//		Returns a reference to the document object inside iframe_el
		// tags:
		//		protected
		return iframe_el.contentDocument // W3
			|| (iframe_el.contentWindow && iframe_el.contentWindow.document) // IE
			|| (iframe_el.name && win.doc.frames[iframe_el.name] && win.doc.frames[iframe_el.name].document)
			|| null;	//	HTMLDocument
	},

	bindDomNode: function(/*String|DomNode*/ node){
		// summary:
		//		Attach menu to given node
		node = dom.byId(node);

		var cn;	// Connect node

		// Support context menus on iframes.  Rather than binding to the iframe itself we need
		// to bind to the <body> node inside the iframe.
		if(node.tagName.toLowerCase() == "iframe"){
			var iframe = node,
				window = this._iframeContentWindow(iframe);
			cn = win.withGlobal(window, win.body);
		}else{

			// To capture these events at the top level, attach to <html>, not <body>.
			// Otherwise right-click context menu just doesn't work.
			cn = (node == win.body() ? win.doc.documentElement : node);
		}


		// "binding" is the object to track our connection to the node (ie, the parameter to bindDomNode())
		var binding = {
			node: node,
			iframe: iframe
		};

		// Save info about binding in _bindings[], and make node itself record index(+1) into
		// _bindings[] array.  Prefix w/_dijitMenu to avoid setting an attribute that may
		// start with a number, which fails on FF/safari.
		domAttr.set(node, "_dijitMenu" + this.id, this._bindings.push(binding));

		// Setup the connections to monitor click etc., unless we are connecting to an iframe which hasn't finished
		// loading yet, in which case we need to wait for the onload event first, and then connect
		// On linux Shift-F10 produces the oncontextmenu event, but on Windows it doesn't, so
		// we need to monitor keyboard events in addition to the oncontextmenu event.
		var doConnects = lang.hitch(this, function(cn){
			return [
				// TODO: when leftClickToOpen is true then shouldn't space/enter key trigger the menu,
				// rather than shift-F10?
				on(cn, this.leftClickToOpen ? "click" : "contextmenu", lang.hitch(this, function(evt){
					// Schedule context menu to be opened unless it's already been scheduled from onkeydown handler
					event.stop(evt);
					this._scheduleOpen(evt.target, iframe, {x: evt.pageX, y: evt.pageY});
				})),
				on(cn, "keydown", lang.hitch(this, function(evt){
					if(evt.shiftKey && evt.keyCode == keys.F10){
						event.stop(evt);
						this._scheduleOpen(evt.target, iframe);	// no coords - open near target node
					}
				}))
			];
		});
		binding.connects = cn ? doConnects(cn) : [];

		if(iframe){
			// Setup handler to [re]bind to the iframe when the contents are initially loaded,
			// and every time the contents change.
			// Need to do this b/c we are actually binding to the iframe's <body> node.
			// Note: can't use connect.connect(), see #9609.

			binding.onloadHandler = lang.hitch(this, function(){
				// want to remove old connections, but IE throws exceptions when trying to
				// access the <body> node because it's already gone, or at least in a state of limbo

				var window = this._iframeContentWindow(iframe);
					cn = win.withGlobal(window, win.body);
				binding.connects = doConnects(cn);
			});
			if(iframe.addEventListener){
				iframe.addEventListener("load", binding.onloadHandler, false);
			}else{
				iframe.attachEvent("onload", binding.onloadHandler);
			}
		}
	},

	unBindDomNode: function(/*String|DomNode*/ nodeName){
		// summary:
		//		Detach menu from given node

		var node;
		try{
			node = dom.byId(nodeName);
		}catch(e){
			// On IE the dom.byId() call will get an exception if the attach point was
			// the <body> node of an <iframe> that has since been reloaded (and thus the
			// <body> node is in a limbo state of destruction.
			return;
		}

		// node["_dijitMenu" + this.id] contains index(+1) into my _bindings[] array
		var attrName = "_dijitMenu" + this.id;
		if(node && domAttr.has(node, attrName)){
			var bid = domAttr.get(node, attrName)-1, b = this._bindings[bid], h;
			while(h = b.connects.pop()){
				h.remove();
			}

			// Remove listener for iframe onload events
			var iframe = b.iframe;
			if(iframe){
				if(iframe.removeEventListener){
					iframe.removeEventListener("load", b.onloadHandler, false);
				}else{
					iframe.detachEvent("onload", b.onloadHandler);
				}
			}

			domAttr.remove(node, attrName);
			delete this._bindings[bid];
		}
	},

	_scheduleOpen: function(/*DomNode?*/ target, /*DomNode?*/ iframe, /*Object?*/ coords){
		// summary:
		//		Set timer to display myself.  Using a timer rather than displaying immediately solves
		//		two problems:
		//
		//		1. IE: without the delay, focus work in "open" causes the system
		//		context menu to appear in spite of stopEvent.
		//
		//		2. Avoid double-shows on linux, where shift-F10 generates an oncontextmenu event
		//		even after a event.stop(e).  (Shift-F10 on windows doesn't generate the
		//		oncontextmenu event.)

		if(!this._openTimer){
			this._openTimer = setTimeout(lang.hitch(this, function(){
				delete this._openTimer;
				this._openMyself({
					target: target,
					iframe: iframe,
					coords: coords
				});
			}), 1);
		}
	},

	_openMyself: function(args){
		// summary:
		//		Internal function for opening myself when the user does a right-click or something similar.
		// args:
		//		This is an Object containing:
		//		* target:
		//			The node that is being clicked
		//		* iframe:
		//			If an <iframe> is being clicked, iframe points to that iframe
		//		* coords:
		//			Put menu at specified x/y position in viewport, or if iframe is
		//			specified, then relative to iframe.
		//
		//		_openMyself() formerly took the event object, and since various code references
		//		evt.target (after connecting to _openMyself()), using an Object for parameters
		//		(so that old code still works).

		var target = args.target,
			iframe = args.iframe,
			coords = args.coords;

		// Get coordinates to open menu, either at specified (mouse) position or (if triggered via keyboard)
		// then near the node the menu is assigned to.
		if(coords){
			if(iframe){
				// Specified coordinates are on <body> node of an <iframe>, convert to match main document
				var ifc = domGeometry.position(iframe, true),
					window = this._iframeContentWindow(iframe),
					scroll = win.withGlobal(window, "_docScroll", dojo);

				var cs = domStyle.getComputedStyle(iframe),
					tp = domStyle.toPixelValue,
					left = (has("ie") && has("quirks") ? 0 : tp(iframe, cs.paddingLeft)) + (has("ie") && has("quirks") ? tp(iframe, cs.borderLeftWidth) : 0),
					top = (has("ie") && has("quirks") ? 0 : tp(iframe, cs.paddingTop)) + (has("ie") && has("quirks") ? tp(iframe, cs.borderTopWidth) : 0);

				coords.x += ifc.x + left - scroll.x;
				coords.y += ifc.y + top - scroll.y;
			}
		}else{
			coords = domGeometry.position(target, true);
			coords.x += 10;
			coords.y += 10;
		}

		var self=this;
		var prevFocusNode = this._focusManager.get("prevNode");
		var curFocusNode = this._focusManager.get("curNode");
		var savedFocusNode = !curFocusNode || (dom.isDescendant(curFocusNode, this.domNode)) ? prevFocusNode : curFocusNode;

		function closeAndRestoreFocus(){
			// user has clicked on a menu or popup
			if(self.refocus && savedFocusNode){
				savedFocusNode.focus();
			}
			pm.close(self);
		}
		pm.open({
			popup: this,
			x: coords.x,
			y: coords.y,
			onExecute: closeAndRestoreFocus,
			onCancel: closeAndRestoreFocus,
			orient: this.isLeftToRight() ? 'L' : 'R'
		});
		this.focus();

		this._onBlur = function(){
			this.inherited('_onBlur', arguments);
			// Usually the parent closes the child widget but if this is a context
			// menu then there is no parent
			pm.close(this);
			// don't try to restore focus; user has clicked another part of the screen
			// and set focus there
		};
	},

	uninitialize: function(){
 		array.forEach(this._bindings, function(b){ if(b){ this.unBindDomNode(b.node); } }, this);
 		this.inherited(arguments);
	}
});

});

},
'ibm/tivoli/tip/navwidget/properties/dijit/PropertiesNotebookDialog':function(){
// wrapped by build app
define("ibm/tivoli/tip/navwidget/properties/dijit/PropertiesNotebookDialog", ["dijit","dojo","dojox","dojo/require!dijit/dijit,dijit/Dialog,dijit/form/Button,ibm/tivoli/navmodel/Trace,ibm/tivoli/tip/navwidget/properties/PropertyRenderer,ibm/tivoli/tip/navwidget/properties/dijit/PropertiesNotebook,ibm/tivoli/tip/navwidget/dijit/NavWidgetPluggableView"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebookDialog");

dojo.require("dijit.dijit");
dojo.require("dijit.Dialog");
dojo.require("dijit.form.Button");
dojo.require("ibm.tivoli.navmodel.Trace");
// 41517 remove dojo.require("dojox.data.dom");
dojo.require("ibm.tivoli.tip.navwidget.properties.PropertyRenderer");
dojo.require("ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebook");
dojo.require("ibm.tivoli.tip.navwidget.dijit.NavWidgetPluggableView");

ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebookDialog._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebookDialog");

dojo.declare("ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebookDialog",
		[dijit.Dialog, ibm.tivoli.tip.navwidget.dijit.NavWidgetObjectPluggableView],
	{		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
	
		//***************************
		// "Public"
		//***************************	
  	

		//*************************** 
		// "Private"
		//***************************	
	
		// _trace: [private] ibm.tivoli.navmodel.Trace
		//		Utility for trace and logging
	    _trace: ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebookDialog._trace,
	    
	    // _propNotebook: [private]
	    //		Coming soon
	    _propNotebook: null,

		constructor: function() {		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon

			//set title
			this.title =  this.commonResources.PROPERTIES;
			
	        //We don't have a widget def specific to this view, so let's signal to 
	        //super class not to try and load any widget defs on our behalf
			this._usesWidgetDefinition = false;
			
			//tab index
			this._tabIndexValueProperties=ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.tabIndexFunc();
		},
		
        
		postCreate: function(){
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			
			//Call superclass
			this.inherited(arguments);
		},
	
		_addPluggableViewToContainer: function(view) {
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		protected
			
			dojo.attr(this, "content", view.domNode);
		},
		
		_removePluggableViewFromContainer: function(view) {
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		protected
			
	//		this.setContent("nada");
		},
		
		startup: function(){		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			
			this.inherited(arguments);	
		},

		//override the onCancel method to also destroy the whole widget.
		//we always want to destroy the properties notebook when Dialog closed.
		onCancel: function(){		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			setTimeout('dijit.byId(\"' + this.id + '\").destroyRecursive();',200); 
			
			// the above line was not deleting the button id and so additional step
			var buttonId=this.id+"_DialogCloseButton";
			if(buttonId) {
			dijit.byId(this.id+"_DialogCloseButton").destroyRecursive();
			}
			this.inherited(arguments);			
		},
	
		destroy: function(){		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon

			if (this._propNotebook) {
				this._propNotebook.destroyRecursive();
				this._propNotebook = null;
			}
            
            this.inherited(arguments);
		},		

		_setDatasetCreatePluggableViews: function(items, links, relationships){	
			// summary:
			//		Coming soon
			// description:
			//		Coming soon

			if (!this._propNotebook) {
				this._propNotebook = new ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebook({
					id: this.id + "_myProperties",
					parentPluggableView: this,
					widgetDefinitionId: this.widgetDefinitionId
				});
				this.addPluggableView(this._propNotebook);
				
				//Richa 10th oct - Reducing the height of the dialog
				divHeight = 0.6 * this._getBodyHeight();
				dojo.style(this._propNotebook.domNode, "height", divHeight + "px");
				
				//adding a close button for accessibility purpose
				var closeDiv=dojo.create("div");
				
				var closeButton=new dijit.form.Button({
					id: this.id+"_DialogCloseButton",
					label:this.commonResources.CLOSE, 
					disabled: false
				},closeDiv.appendChild(dojo.create("div")));
			
			    closeDiv.className="dijitDialogPaneActionBar";
				
				dojo.connect(closeButton, "onClick",dojo.hitch(this,"onCancel"));
				dojo.place(closeDiv,this.containerNode,"last");
			
			}	
		},
		
		_getBodyHeight: function() {
			return ((window.innerHeight != undefined) ? window.innerHeight : document.body.offsetHeight);
		}
});
});

},
'ibm/tivoli/tip/navwidget/tree/dijit/NavModelTreeTable':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/i18n!ibm/tivoli/tip/navwidget/nls/resources","dojo/require!ibm/tivoli/tip/navwidget/dijit/CustomContentTooltip,ibm/tivoli/tip/navwidget/properties/dijit/PropertiesNotebookGeneralTab,dojoe/treetablex/TreeTable"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.tree.dijit.NavModelTreeTable");

dojo.require("ibm.tivoli.tip.navwidget.dijit.CustomContentTooltip");
dojo.require("ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebookGeneralTab");

// dojo.require("dojoe.treetable.TreeTableAll");
dojo.require("dojoe.treetablex.TreeTable");

dojo.requireLocalization("ibm.tivoli.tip.navwidget", "resources");

dojo.declare ("ibm.tivoli.tip.navwidget.tree.dijit.NavModelTreeTable",
	[dojoe.treetablex.TreeTable],
{

	// viewLoaded: boolean
	//		Coming soon
	viewLoaded : false,
	
	// showTooltip: boolean
	//		Optionally hide tooltip
	showTooltip: true,
	
	// _propOnSetObjectsConnection: [private] Object
	//		Coming soon
	_propOnSetObjectsConnection: null,
	
	// _propOnSetDatasetConnection: [private] Object
	//		Coming soon
	_propOnSetDatasetConnection: null,

	// _tooltip: [private] Object
	//      Coming soon
	_tooltip : null,
	
	// _resizing: [private] Number
	//      Coming soon
	_resizing : 0,

	postCreate : function() {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon

		//first let the tree grid get generated as normal
		this.inherited(arguments);
	
		// 22993: KDM  Optionally hide tooltips
		if (this.showTooltip) {		
			// create tooltip onto the tree grid domNode
			this._tooltip = new ibm.tivoli.tip.navwidget.dijit.CustomContentTooltip(
						{
							connectId : [ this._grid.domNode.id ],
							getCustomContentCallback : dojo.hitch(this, this._customTooltip),
							position : ["below", "after"]
						});
			
			// Override the tooltip behavior specifically for the tree grid
			dojo.safeMixin(this._tooltip, {
				
				// Override open to set tooltip position over the tree grid
				// near table row
				open : function(target) {
					// Create a rectangle for positioning the tooltip, otherwise default
					// position is outside the DOM node of the tree grid
					var aroundRect = target.rolloverTargetRect;
			
					// make sure the rolloverTargetRect has been set to actually display the tooltip
					// (it may not be if we got a rollover while the context menu is displayed)
					if (aroundRect != null) {
						// replace the target node, the tree grid DOM node, with this rectangle
						arguments[0] = aroundRect;
						this.inherited(arguments);
					}
				},
				_onTargetMouseEnter : function(e) {
					// ignore since the tree grid DOM node is not the real
					// target (we handle the tooltip)
				},
				_onTargetMouseLeave : function(e) {
					// ignore since the tree grid DOM node is not the real
					// target (we handle the tooltip)
				},
				_onTargetBlur : function(e) {
					// ignore since the tree grid DOM node is not the real
					// target (we handle the tooltip)
				},
				_onTargetFocus : function(e) {
					// ignore since the tree grid DOM node is not the real
					// target (we handle the tooltip)
				},
				// added helper method to trigger the timer to display the tooltip
				hover : function(target) {
					this._onHover( {
						target : target
					});
				},
				// added helper method to dismiss the tooltip
				unHover : function(target) {
					this._onUnHover( {
						target : target
					});
				}
			});
			this._tooltip.startup();

			dojo.connect( this._grid, "onCellMouseOver", this, "_onCellMouseOver");
			dojo.connect( this._grid, "onCellMouseOut", this, "_onCellMouseOut");
			dojo.connect( this, "onTableLoad", this, "notifyLoadComplete");	  	
		}
	},
	
	notifyLoadComplete: function(){
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		
		this.viewLoaded = true;
		
  		//Send notification we're done loading the dataset/prepping the view now that the table has loaded
  		var parentPluggableView = this.tableView;
  		parentPluggableView._onSetDatasetLoadComplete(parentPluggableView.dataset);
		
	},
	
	resize : function() {
		// summary:
		//		Handle special resize case for IE8
		// description:
		//		Resize events that can occur on IE8 while processing the CURI REST
		//      callback to populate the context menu can sometimes interfere with
		//      the menu rendering the returned new task items.  (The resize event
		//      processing is dispatched after the menu has been repopulated but
		//      before it can actually render to the screen in IE8.  The result is
		//      the menu can be dismissed before it renders correctly.)  This
		//      workaround delays the processing of the resize event slightly thus
		//      allowing the menu to render.

        if (dojo.isIE) {
        	if (this.tableView && this.tableView.isMenuOpen() && (this._resizing < 1)) {
        		this._resizing++;
        		setTimeout(dojo.hitch(this, this._doResize), 0);
            }
            else {
            	this.inherited(arguments);
            }
        } 
        else {
        	this.inherited(arguments);
        }
    },

    _doResize: function() {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon

    	this.resize();
    	this._resizing--;
    },
	
	_customTooltip: function(/*Object*/ target) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// target:
		//		Coming soon

		if (this._rolloverTarget) {
			// target is this._grid.domNode, so we use this._rolloverTarget
			var content = "";
			if (typeof this._rolloverTarget == "string") {
				content = content + this._rolloverTarget;
			} else {
				content = content + this._rolloverTarget.getLabel();
			}

			// first check if a server side cell property tooltip is defined, use that instead of other defaults
			if(this._rolloverCellTarget && this._rolloverCellTarget.getTooltip){
				var tooltip = this._rolloverCellTarget.getTooltip();
				if(tooltip){
					return tooltip;
				}
			}
			
			//if the server side tooltip content is set, use this instead of defaults.
			if(this._rolloverTarget.getTooltip) { //Need first test because getTooltip is not a function on child datasets right now
				var tooltip = this._rolloverTarget.getTooltip();
				if (tooltip){ 
					return tooltip;
				}
			}

			parentPluggableView = this.tableView;
			
			if(this._tooltipProperties){
				return this._tooltipProperties.getContent();
			}
			
			//create properties content object
			this._tooltipProperties = new ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebookGeneralTab({
				id: this.id + "_tooltipProperties",
				managingPluggableView: parentPluggableView, 
				maxProperties: 3,  //TODO: ADM should probably be something a widgetDef sets.
				_isTooltipProperties: true
			});

			//Call back for when the dataset it set
			var objectType = (this._rolloverTarget.getObjectType) ? this._rolloverTarget.getObjectType() : null;
			var setDatasetComplete = function(dataset) {
				//connect to the onSetObjects Event
				this._propOnSetObjectsConnection = dojo.connect(this._tooltipProperties, "onSetObjects", dojo.hitch(this, this._updateTooltipContent));

				if (objectType === "item"){
					this._tooltipProperties.setObjects([this._rolloverTarget], null, null, null);
				} else if(objectType === "link"){
					this._tooltipProperties.setObjects(null, [this._rolloverTarget], null, null);
				} else if(objectType === "relationship"){
					this._tooltipProperties.setObjects(null, null, [this._rolloverTarget], null);
				}
			};
			this._propOnSetDatasetConnection = dojo.connect(this._tooltipProperties, "onSetDataset", dojo.hitch(this, setDatasetComplete));
			this._tooltipProperties.setDataset(parentPluggableView.dataset);

			return this._createTemporaryContentDiv(content, objectType);
		}
	   },
	   
	   //Event callback method to handle updating the tooltip content with updated content
	   _updateTooltipContent: function(){
		
		this._tooltip.updateContent(this._tooltipProperties.getContent(), this._grid.domNode);
	   },
	   
	   _createTemporaryContentDiv: function(content, objectType) {		   
		   //Background div
		   var backgroundDiv = dojo.doc.createElement("div");
		   var processingPane = new dijit.layout.ContentPane({
			   content: backgroundDiv
		   });
			
			// text div		
		   var textDiv = dojo.create("div",{
			   "innerHTML": content
		   });
		   backgroundDiv.appendChild(textDiv);
		
		   if (objectType) {
			   // processing image div		
			   var processDiv = dojo.create("div",{
					"class": "processingImage"
			   });
			   backgroundDiv.appendChild(processDiv);
		   }

		   //Return
		   return processingPane;
	   },
	   
	   //function to close tooltips
	   closeTooltip: function() {
			if (this._rolloverTarget) {
				dojo.disconnect(this._propOnSetObjectsConnection);
				dojo.disconnect(this._propOnSetDatasetConnection);
				
				// Clear out the reference in the tooltip to the content
				this._tooltip.updateContent("", this._grid.domNode); 
				if (this._tooltipProperties){
					this._tooltipProperties.destroyRecursive();
					this._tooltipProperties = null;
				}
				
				// this._tooltip.close();
				this._tooltip.unHover(this._grid.domNode);
				this._rolloverTarget = null;
				this._rolloverCellTarget = null;
				this._grid.domNode.rolloverTargetRect = null;
		    }
		},

	_onCellMouseOut: function( e ){
		// console.log("mouse rolled out of row");
		this.closeTooltip();
		dojo.stopEvent(e);  
	},
		
	_onCellMouseOver: function( e ){
		// console.log("rolled onto row ");
		
		dojo.stopEvent(e);
		
		// ignore if the menu is open or if we already have a rollover target
		if (!this.tableView.suppressTooltip() && !this.tableView.isMenuOpen() && !this._rolloverTarget) {  // 52876
	    	var aroundRect = {type: "rect"};
	    	var field = this._grid.column(e.columnId,false).field;

	    	// this._rolloverTarget = e.grid.getItem(e.rowNode.gridRowIndex);
	    	this._rolloverTarget = this._grid.model.byId(e.rowId).item;
	    	
	    	if (this._rolloverTarget.hasCachedProperty(field)) {
	    		this._rolloverCellTarget = this._rolloverTarget.getProperty(field);
	    	}
	    	else {
	    		this._rolloverCellTarget = null;
	    	}

	    	var boundingInfo = dojo.position(e.target, true);
	    	aroundRect.x = boundingInfo.x; 
	    	aroundRect.y = boundingInfo.y;

	    	aroundRect.width = e.target.clientWidth;
	    	aroundRect.height = e.target.clientHeight;
	    
	    	// adjust relative coordinates to absolute, and remove fractions
	    	var lt = dojo.coords(this._grid.domNode, true);
	    	aroundRect.x = Math.round(aroundRect.x);
	    	aroundRect.y = Math.round(aroundRect.y);
			
	    	this._grid.domNode.rolloverTargetRect = aroundRect;
	    	this._tooltip.hover(this._grid.domNode);		
	    }			
	}
});	
	

});

},
'dijit/layout/utils':function(){
define([
	"dojo/_base/array", // array.filter array.forEach
	"dojo/dom-class", // domClass.add domClass.remove
	"dojo/dom-geometry", // domGeometry.marginBox
	"dojo/dom-style", // domStyle.getComputedStyle
	"dojo/_base/lang", // lang.mixin
	".."	// for exporting symbols to dijit, remove in 2.0
], function(array, domClass, domGeometry, domStyle, lang, dijit){

	// module:
	//		dijit/layout/utils
	// summary:
	//		marginBox2contentBox() and layoutChildren()

	var layout = lang.getObject("layout", true, dijit);
	/*===== layout = dijit.layout =====*/

	layout.marginBox2contentBox = function(/*DomNode*/ node, /*Object*/ mb){
		// summary:
		//		Given the margin-box size of a node, return its content box size.
		//		Functions like domGeometry.contentBox() but is more reliable since it doesn't have
		//		to wait for the browser to compute sizes.
		var cs = domStyle.getComputedStyle(node);
		var me = domGeometry.getMarginExtents(node, cs);
		var pb = domGeometry.getPadBorderExtents(node, cs);
		return {
			l: domStyle.toPixelValue(node, cs.paddingLeft),
			t: domStyle.toPixelValue(node, cs.paddingTop),
			w: mb.w - (me.w + pb.w),
			h: mb.h - (me.h + pb.h)
		};
	};

	function capitalize(word){
		return word.substring(0,1).toUpperCase() + word.substring(1);
	}

	function size(widget, dim){
		// size the child
		var newSize = widget.resize ? widget.resize(dim) : domGeometry.setMarginBox(widget.domNode, dim);

		// record child's size
		if(newSize){
			// if the child returned it's new size then use that
			lang.mixin(widget, newSize);
		}else{
			// otherwise, call getMarginBox(), but favor our own numbers when we have them.
			// the browser lies sometimes
			lang.mixin(widget, domGeometry.getMarginBox(widget.domNode));
			lang.mixin(widget, dim);
		}
	}

	layout.layoutChildren = function(/*DomNode*/ container, /*Object*/ dim, /*Widget[]*/ children,
			/*String?*/ changedRegionId, /*Number?*/ changedRegionSize){
		// summary:
		//		Layout a bunch of child dom nodes within a parent dom node
		// container:
		//		parent node
		// dim:
		//		{l, t, w, h} object specifying dimensions of container into which to place children
		// children:
		//		an array of Widgets or at least objects containing:
		//			* domNode: pointer to DOM node to position
		//			* region or layoutAlign: position to place DOM node
		//			* resize(): (optional) method to set size of node
		//			* id: (optional) Id of widgets, referenced from resize object, below.
		// changedRegionId:
		//		If specified, the slider for the region with the specified id has been dragged, and thus
		//		the region's height or width should be adjusted according to changedRegionSize
		// changedRegionSize:
		//		See changedRegionId.

		// copy dim because we are going to modify it
		dim = lang.mixin({}, dim);

		domClass.add(container, "dijitLayoutContainer");

		// Move "client" elements to the end of the array for layout.  a11y dictates that the author
		// needs to be able to put them in the document in tab-order, but this algorithm requires that
		// client be last.    TODO: move these lines to LayoutContainer?   Unneeded other places I think.
		children = array.filter(children, function(item){ return item.region != "center" && item.layoutAlign != "client"; })
			.concat(array.filter(children, function(item){ return item.region == "center" || item.layoutAlign == "client"; }));

		// set positions/sizes
		array.forEach(children, function(child){
			var elm = child.domNode,
				pos = (child.region || child.layoutAlign);
			if(!pos){
				throw new Error("No region setting for " + child.id)
			}

			// set elem to upper left corner of unused space; may move it later
			var elmStyle = elm.style;
			elmStyle.left = dim.l+"px";
			elmStyle.top = dim.t+"px";
			elmStyle.position = "absolute";

			domClass.add(elm, "dijitAlign" + capitalize(pos));

			// Size adjustments to make to this child widget
			var sizeSetting = {};

			// Check for optional size adjustment due to splitter drag (height adjustment for top/bottom align
			// panes and width adjustment for left/right align panes.
			if(changedRegionId && changedRegionId == child.id){
				sizeSetting[child.region == "top" || child.region == "bottom" ? "h" : "w"] = changedRegionSize;
			}

			// set size && adjust record of remaining space.
			// note that setting the width of a <div> may affect its height.
			if(pos == "top" || pos == "bottom"){
				sizeSetting.w = dim.w;
				size(child, sizeSetting);
				dim.h -= child.h;
				if(pos == "top"){
					dim.t += child.h;
				}else{
					elmStyle.top = dim.t + dim.h + "px";
				}
			}else if(pos == "left" || pos == "right"){
				sizeSetting.h = dim.h;
				size(child, sizeSetting);
				dim.w -= child.w;
				if(pos == "left"){
					dim.l += child.w;
				}else{
					elmStyle.left = dim.l + dim.w + "px";
				}
			}else if(pos == "client" || pos == "center"){
				size(child, dim);
			}
		});
	};


	return {
		marginBox2contentBox: layout.marginBox2contentBox,
		layoutChildren: layout.layoutChildren
	};
});

},
'dojox/date/php':function(){
define(["dojo/_base/kernel", "dojo/_base/lang","dojo/date","dojox/string/tokenize"], function(dojo,dlang,ddate,dxst){
dojo.getObject("date.php", true, dojox);

dojox.date.php.format = function(/*Date*/ date, /*String*/ format){
	// summary: Get a formatted string for a given date object
	var df = new dojox.date.php.DateFormat(format);
	return df.format(date);
}

dojox.date.php.DateFormat = function(/*String*/ format){
	// summary: Format the internal date object
	if(!this.regex){
		var keys = [];
		for(var key in this.constructor.prototype){
			if(dojo.isString(key) && key.length == 1 && dojo.isFunction(this[key])){
				keys.push(key);
			}
		}
		this.constructor.prototype.regex = new RegExp("(?:(\\\\.)|([" + keys.join("") + "]))", "g");
	}

	var replacements = [];

	this.tokens = dxst(format, this.regex, function(escape, token, i){
		if(token){
			replacements.push([i, token]);
			return token;
		}
		if(escape){
			return escape.charAt(1);
		}
	});

	this.replacements = replacements;
}
dojo.extend(dojox.date.php.DateFormat, {
	weekdays: ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"],
	weekdays_3: ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"],
	months: ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"],
	months_3: ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"],
	monthdays: [31,28,31,30,31,30,31,31,30,31,30,31],

	format: function(/*Date*/ date){
		this.date = date;
		for(var i = 0, replacement; replacement = this.replacements[i]; i++){
			this.tokens[replacement[0]] = this[replacement[1]]();
		}
		return this.tokens.join("");
	},

	// Day

	d: function(){
		// summary: Day of the month, 2 digits with leading zeros
		var j = this.j();
		return (j.length == 1) ? "0" + j : j;
	},

	D: function(){
		// summary: A textual representation of a day, three letters
		return this.weekdays_3[this.date.getDay()];
	},

	j: function(){
		// summary: Day of the month without leading zeros
		return this.date.getDate() + "";
	},

	l: function(){
		// summary: A full textual representation of the day of the week
		return this.weekdays[this.date.getDay()];
	},
	
	N: function(){
		// summary: ISO-8601 numeric representation of the day of the week (added in PHP 5.1.0)
		var w = this.w();
		return (!w) ? 7 : w;
	},

	S: function(){
		// summary: English ordinal suffix for the day of the month, 2 characters
		switch(this.date.getDate()){
			case 11: case 12: case 13: return "th";
			case 1: case 21: case 31: return "st";
			case 2: case 22: return "nd";
			case 3: case 23: return "rd";
			default: return "th";
		}
	},

	w: function(){
		// summary: Numeric representation of the day of the week
		return this.date.getDay() + "";
	},

	z: function(){
		// summary: The day of the year (starting from 0)
		var millis = this.date.getTime() - new Date(this.date.getFullYear(), 0, 1).getTime();
		return Math.floor(millis/86400000) + "";
	},

	// Week

	W: function(){
		// summary: ISO-8601 week number of year, weeks starting on Monday (added in PHP 4.1.0)
		var week;
		var jan1_w = new Date(this.date.getFullYear(), 0, 1).getDay() + 1;
		var w = this.date.getDay() + 1;
		var z = parseInt(this.z());

		if(z <= (8 - jan1_w) && jan1_w > 4){
			var last_year = new Date(this.date.getFullYear() - 1, this.date.getMonth(), this.date.getDate());
			if(jan1_w == 5 || (jan1_w == 6 && ddate.isLeapYear(last_year))){
				week = 53;
			}else{
				week = 52;
			}
		}else{
			var i;
			if(Boolean(this.L())){
				i = 366;
			}else{
				i = 365;
			}
			if((i - z) < (4 - w)){
				week = 1;
			}else{
				var j = z + (7 - w) + (jan1_w - 1);
				week = Math.ceil(j / 7);
				if(jan1_w > 4){
					--week;
				}
			}
		}
		
		return week;
	},

	// Month

	F: function(){
		// summary: A full textual representation of a month, such as January or March
		return this.months[this.date.getMonth()];
	},

	m: function(){
		// summary: Numeric representation of a month, with leading zeros
		var n = this.n();
		return (n.length == 1) ? "0" + n : n;
	},

	M: function(){
		// summary: A short textual representation of a month, three letters
		return this.months_3[this.date.getMonth()];
	},

	n: function(){
		// summary: Numeric representation of a month, without leading zeros
		return this.date.getMonth() + 1 + "";
	},

	t: function(){
		// summary: Number of days in the given month
		return (Boolean(this.L()) && this.date.getMonth() == 1) ? 29 : this.monthdays[this.getMonth()];
	},

	// Year

	L: function(){
		// summary: Whether it's a leap year
		return (ddate.isLeapYear(this.date)) ? "1" : "0";
	},

	o: function(){
		// summary:
		//		ISO-8601 year number. This has the same value as Y, except that if
		//		the ISO week number (W) belongs to the previous or next year, that year is used instead. (added in PHP 5.1.0)
		// TODO: Figure out what this means
	},

	Y: function(){
		// summary: A full numeric representation of a year, 4 digits
		return this.date.getFullYear() + "";
	},

	y: function(){
		// summary: A two digit representation of a year
		return this.Y().slice(-2);
	},

	// Time

	a: function(){
		// summary: Lowercase Ante meridiem and Post meridiem
		return this.date.getHours() >= 12 ? "pm" : "am";
	},

	b: function(){
		// summary: Uppercase Ante meridiem and Post meridiem
		return this.a().toUpperCase();
	},

	B: function(){
		// summary:
		//	Swatch Internet time
		//	A day is 1,000 beats. All time is measured from GMT + 1
		var off = this.date.getTimezoneOffset() + 60;
		var secs = (this.date.getHours() * 3600) + (this.date.getMinutes() * 60) + this.getSeconds() + (off * 60);
		var beat = Math.abs(Math.floor(secs / 86.4) % 1000) + "";
		while(beat.length <  2) beat = "0" + beat;
		return beat;
	},

	g: function(){
		// summary: 12-hour format of an hour without leading zeros
		return (this.date.getHours() > 12) ? this.date.getHours() - 12 + "" : this.date.getHours() + "";
	},

	G: function(){
		// summary: 24-hour format of an hour without leading zeros
		return this.date.getHours() + "";
	},

	h: function(){
		// summary: 12-hour format of an hour with leading zeros
		var g = this.g();
		return (g.length == 1) ? "0" + g : g;
	},

	H: function(){
		// summary: 24-hour format of an hour with leading zeros
		var G = this.G();
		return (G.length == 1) ? "0" + G : G;
	},

	i: function(){
		// summary: Minutes with leading zeros
		var mins = this.date.getMinutes() + "";
		return (mins.length == 1) ? "0" + mins : mins;
	},

	s: function(){
		// summary: Seconds, with leading zeros
		var secs = this.date.getSeconds() + "";
		return (secs.length == 1) ? "0" + secs : secs;
	},

	// Timezone

	e: function(){
		// summary: Timezone identifier (added in PHP 5.1.0)
		return ddate.getTimezoneName(this.date);
	},

	I: function(){
		// summary: Whether or not the date is in daylight saving time
		// TODO: Can dojo.date do this?
	},

	O: function(){
		// summary: Difference to Greenwich time (GMT) in hours
		var off = Math.abs(this.date.getTimezoneOffset());
		var hours = Math.floor(off / 60) + "";
		var mins = (off % 60) + "";
		if(hours.length == 1) hours = "0" + hours;
		if(mins.length == 1) hours = "0" + mins;
		return ((this.date.getTimezoneOffset() < 0) ? "+" : "-") + hours + mins;
	},

	P: function(){
		// summary: Difference to Greenwich time (GMT) with colon between hours and minutes (added in PHP 5.1.3)
		var O = this.O();
		return O.substring(0, 2) + ":" + O.substring(2, 4);
	},

	T: function(){
		// summary: Timezone abbreviation

		// Guess...
		return this.e().substring(0, 3);
	},

	Z: function(){
		// summary:
		//		Timezone offset in seconds. The offset for timezones west of UTC is always negative,
		//		and for those east of UTC is always positive.
		return this.date.getTimezoneOffset() * -60;
	},

	// Full Date/Time

	c: function(){
		// summary: ISO 8601 date (added in PHP 5)
		return this.Y() + "-" + this.m() + "-" + this.d() + "T" + this.h() + ":" + this.i() + ":" + this.s() + this.P();
	},

	r: function(){
		// summary: RFC 2822 formatted date
		return this.D() + ", " + this.d() + " " + this.M() + " " + this.Y() + " " + this.H() + ":" + this.i() + ":" + this.s() + " " + this.O();
	},

	U: function(){
		// summary: Seconds since the Unix Epoch (January 1 1970 00:00:00 GMT)
		return Math.floor(this.date.getTime() / 1000);
	}

});
return dojox.date.php;
});
},
'dojo/i18n':function(){
define(["./_base/kernel", "require", "./has", "./_base/array", "./_base/config", "./_base/lang", "./_base/xhr", "./json"],
	function(dojo, require, has, array, config, lang, xhr, json) {
	// module:
	//		dojo/i18n
	// summary:
	//		This module implements the !dojo/i18n plugin and the v1.6- i18n API
	// description:
	//		We choose to include our own plugin to leverage functionality already contained in dojo
	//		and thereby reduce the size of the plugin compared to various loader implementations. Also, this
	//		allows foreign AMD loaders to be used without their plugins.


	has.add("dojo-preload-i18n-Api",
		// if true, define the preload localizations machinery
		1
	);

	true || has.add("dojo-v1x-i18n-Api",
		// if true, define the v1.x i18n functions
		1
	);

	var
		thisModule= dojo.i18n=
			// the dojo.i18n module
			{},

		nlsRe=
			// regexp for reconstructing the master bundle name from parts of the regexp match
			// nlsRe.exec("foo/bar/baz/nls/en-ca/foo") gives:
			// ["foo/bar/baz/nls/en-ca/foo", "foo/bar/baz/nls/", "/", "/", "en-ca", "foo"]
			// nlsRe.exec("foo/bar/baz/nls/foo") gives:
			// ["foo/bar/baz/nls/foo", "foo/bar/baz/nls/", "/", "/", "foo", ""]
			// so, if match[5] is blank, it means this is the top bundle definition.
			// courtesy of http://requirejs.org
			/(^.*(^|\/)nls)(\/|$)([^\/]*)\/?([^\/]*)/,

		getAvailableLocales= function(
			root,
			locale,
			bundlePath,
			bundleName
		){
			// return a vector of module ids containing all available locales with respect to the target locale
			// For example, assuming:
			//	 * the root bundle indicates specific bundles for "fr" and "fr-ca",
			//	 * bundlePath is "myPackage/nls"
			//	 * bundleName is "myBundle"
			// Then a locale argument of "fr-ca" would return
			//	 ["myPackage/nls/myBundle", "myPackage/nls/fr/myBundle", "myPackage/nls/fr-ca/myBundle"]
			// Notice that bundles are returned least-specific to most-specific, starting with the root.
			//
			// If root===false indicates we're working with a pre-AMD i18n bundle that doesn't tell about the available locales;
			// therefore, assume everything is available and get 404 errors that indicate a particular localization is not available
			//

			for(var result= [bundlePath + bundleName], localeParts= locale.split("-"), current= "", i= 0; i<localeParts.length; i++){
				current+= (current ? "-" : "") + localeParts[i];
				if(!root || root[current]){
					result.push(bundlePath + current + "/" + bundleName);
				}
			}
			return result;
		},

		cache= {},

		getL10nName= dojo.getL10nName = function(moduleName, bundleName, locale){
			locale = locale ? locale.toLowerCase() : dojo.locale;
			moduleName = "dojo/i18n!" + moduleName.replace(/\./g, "/");
			bundleName = bundleName.replace(/\./g, "/");
			return (/root/i.test(locale)) ?
				(moduleName + "/nls/" + bundleName) :
				(moduleName + "/nls/" + locale + "/" + bundleName);
		},

		doLoad = function(require, bundlePathAndName, bundlePath, bundleName, locale, load){
			// get the root bundle which instructs which other bundles are required to construct the localized bundle
			require([bundlePathAndName], function(root){
				var current= lang.clone(root.root),
					availableLocales= getAvailableLocales(!root._v1x && root, locale, bundlePath, bundleName);
				require(availableLocales, function(){
					for (var i= 1; i<availableLocales.length; i++){
						current= lang.mixin(lang.clone(current), arguments[i]);
					}
					// target may not have been resolve (e.g., maybe only "fr" exists when "fr-ca" was requested)
					var target= bundlePathAndName + "/" + locale;
					cache[target]= current;
					load();
				});
			});
		},

		normalize = function(id, toAbsMid){
			// id may be relative
			// preload has form *preload*<path>/nls/<module>*<flattened locales> and
			// therefore never looks like a relative
			return /^\./.test(id) ? toAbsMid(id) : id;
		},

		getLocalesToLoad = function(targetLocale){
			var list = config.extraLocale || [];
			list = lang.isArray(list) ? list : [list];
			list.push(targetLocale);
			return list;
		},

		load = function(id, require, load){
			//
			// id is in one of the following formats
			//
			//	1. <path>/nls/<bundle>
			//		=> load the bundle, localized to config.locale; load all bundles localized to
			//      config.extraLocale (if any); return the loaded bundle localized to config.locale.
			//
			//  2. <path>/nls/<locale>/<bundle>
			//		=> load then return the bundle localized to <locale>
			//
			//  3. *preload*<path>/nls/<module>*<JSON array of available locales>
			//		=> for config.locale and all config.extraLocale, load all bundles found
			//		   in the best-matching bundle rollup. A value of 1 is returned, which
			//         is meaningless other than to say the plugin is executing the requested
			//         preloads
			//
			// In cases 1 and 2, <path> is always normalized to an absolute module id upon entry; see
			// normalize. In case 3, it <path> is assumed to be absolue; this is arranged by the builder.
			//
			// To load a bundle means to insert the bundle into the plugin's cache and publish the bundle
			// value to the loader. Given <path>, <bundle>, and a particular <locale>, the cache key
			//
			//   <path>/nls/<bundle>/<locale>
			//
			// will hold the value. Similarly, then plugin will publish this value to the loader by
			//
			//   define("<path>/nls/<bundle>/<locale>", <bundle-value>);
			//
			// Given this algorithm, other machinery can provide fast load paths be preplacing
			// values in the plugin's cache, which is public. When a load is demanded the
			// cache is inspected before starting any loading. Explicitly placing values in the plugin
			// cache is an advanced/experimental feature that should not be needed; use at your own risk.
			//
			// For the normal AMD algorithm, the root bundle is loaded first, which instructs the
			// plugin what additional localized bundles are required for a particular locale. These
			// additional locales are loaded and a mix of the root and each progressively-specific
			// locale is returned. For example:
			//
			// 1. The client demands "dojo/i18n!some/path/nls/someBundle
			//
			// 2. The loader demands load(some/path/nls/someBundle)
			//
			// 3. This plugin require's "some/path/nls/someBundle", which is the root bundle.
			//
			// 4. Assuming config.locale is "ab-cd-ef" and the root bundle indicates that localizations
			//    are available for "ab" and "ab-cd-ef" (note the missing "ab-cd", then the plugin
			//    requires "some/path/nls/ab/someBundle" and "some/path/nls/ab-cd-ef/someBundle"
			//
			// 5. Upon receiving all required bundles, the plugin constructs the value of the bundle
			//    ab-cd-ef as...
			//
			//      mixin(mixin(mixin({}, require("some/path/nls/someBundle"),
			//        require("some/path/nls/ab/someBundle")),
			//          require("some/path/nls/ab-cd-ef/someBundle"));
			//
			//    This value is inserted into the cache and published to the loader at the
			//    key/module-id some/path/nls/someBundle/ab-cd-ef.
			//
			// The special preload signature (case 3) instructs the plugin to stop servicing all normal requests
			// (further preload requests will be serviced) until all ongoing preloading has completed.
			//
			// The preload signature instructs the plugin that a special rollup module is available that contains
			// one or more flattened, localized bundles. The JSON array of available locales indicates which locales
			// are available. Here is an example:
			//
			//   *preload*some/path/nls/someModule*["root", "ab", "ab-cd-ef"]
			//
			// This indicates the following rollup modules are available:
			//
			//   some/path/nls/someModule_ROOT
			//   some/path/nls/someModule_ab
			//   some/path/nls/someModule_ab-cd-ef
			//
			// Each of these modules is a normal AMD module that contains one or more flattened bundles in a hash.
			// For example, assume someModule contained the bundles some/bundle/path/someBundle and
			// some/bundle/path/someOtherBundle, then some/path/nls/someModule_ab would be expressed as folllows:
			//
			// define({
			//   some/bundle/path/someBundle:<value of someBundle, flattened with respect to locale ab>,
			//   some/bundle/path/someOtherBundle:<value of someOtherBundle, flattened with respect to locale ab>,
			// });
			//
			// E.g., given this design, preloading for locale=="ab" can execute the following algorithm:
			//
			// require(["some/path/nls/someModule_ab"], function(rollup){
			//   for(var p in rollup){
			//     var id = p + "/ab",
			//     cache[id] = rollup[p];
			//     define(id, rollup[p]);
			//   }
			// });
			//
			// Similarly, if "ab-cd" is requested, the algorithm can determine that "ab" is the best available and
			// load accordingly.
			//
			// The builder will write such rollups for every layer if a non-empty localeList  profile property is
			// provided. Further, the builder will include the following cache entry in the cache associated with
			// any layer.
			//
			//   "*now":function(r){r(['dojo/i18n!*preload*<path>/nls/<module>*<JSON array of available locales>']);}
			//
			// The *now special cache module instructs the loader to apply the provided function to context-require
			// with respect to the particular layer being defined. This causes the plugin to hold all normal service
			// requests until all preloading is complete.
			//
			// Notice that this algorithm is rarely better than the standard AMD load algorithm. Consider the normal case
			// where the target locale has a single segment and a layer depends on a single bundle:
			//
			// Without Preloads:
			//
			//   1. Layer loads root bundle.
			//   2. bundle is demanded; plugin loads single localized bundle.
			//
			// With Preloads:
			//
			//   1. Layer causes preloading of target bundle.
			//   2. bundle is demanded; service is delayed until preloading complete; bundle is returned.
			//
			// In each case a single transaction is required to load the target bundle. In cases where multiple bundles
			// are required and/or the locale has multiple segments, preloads still requires a single transaction whereas
			// the normal path requires an additional transaction for each additional bundle/locale-segment. However all
			// of these additional transactions can be done concurrently. Owing to this analysis, the entire preloading
			// algorithm can be discard during a build by setting the has feature dojo-preload-i18n-Api to false.
			//
			if(has("dojo-preload-i18n-Api")){
				var split = id.split("*"),
					preloadDemand = split[1]=="preload";
				if(preloadDemand){
					if(!cache[id]){
						// use cache[id] to prevent multiple preloads of the same preload; this shouldn't happen, but
						// who knows what over-aggressive human optimizers may attempt
						cache[id] = 1;
						preloadL10n(split[2], json.parse(split[3]), 1);
					}
					// don't stall the loader!
					load(1);
				}
				if(preloadDemand || waitForPreloads(id, require, load)){
					return;
				}
			}

			var match= nlsRe.exec(id),
				bundlePath= match[1] + "/",
				bundleName= match[5] || match[4],
				bundlePathAndName= bundlePath + bundleName,
				localeSpecified = (match[5] && match[4]),
				targetLocale=  localeSpecified || dojo.locale,
				loadTarget= bundlePathAndName + "/" + targetLocale,
				loadList = localeSpecified ? [targetLocale] : getLocalesToLoad(targetLocale),
				remaining = loadList.length,
				finish = function(){
					if(!--remaining){
						load(lang.delegate(cache[loadTarget]));
					}
				};
			array.forEach(loadList, function(locale){
				var target = bundlePathAndName + "/" + locale;
				if(has("dojo-preload-i18n-Api")){
					checkForLegacyModules(target);
				}
				if(!cache[target]){
					doLoad(require, bundlePathAndName, bundlePath, bundleName, locale, finish);
				}else{
					finish();
				}
			});
		};

	if(has("dojo-unit-tests")){
		var unitTests = thisModule.unitTests = [];
	}

	if(has("dojo-preload-i18n-Api") || 1){
		var normalizeLocale = thisModule.normalizeLocale= function(locale){
				var result = locale ? locale.toLowerCase() : dojo.locale;
				return result == "root" ? "ROOT" : result;
			},

			isXd = function(mid){
				return (1 && 1) ?
					require.isXdUrl(require.toUrl(mid + ".js")) :
					true;
			},

			preloading = 0,

			preloadWaitQueue = [],

			preloadL10n = thisModule._preloadLocalizations = function(/*String*/bundlePrefix, /*Array*/localesGenerated, /*boolean*/ guaranteedAmdFormat){
				//	summary:
				//		Load available flattened resource bundles associated with a particular module for dojo.locale and all dojo.config.extraLocale (if any)
				//
				//  descirption:
				//		Only called by built layer files. The entire locale hierarchy is loaded. For example,
				//		if locale=="ab-cd", then ROOT, "ab", and "ab-cd" are loaded. This is different than v1.6-
				//		in that the v1.6- would lonly load ab-cd...which was *always* flattened.
				//
				//		If guaranteedAmdFormat is true, then the module can be loaded with require thereby circumventing the detection algorithm
				//		and the extra possible extra transaction.
				//

				function forEachLocale(locale, func){
					// given locale= "ab-cd-ef", calls func on "ab-cd-ef", "ab-cd", "ab", "ROOT"; stops calling the first time func returns truthy
					var parts = locale.split("-");
					while(parts.length){
						if(func(parts.join("-"))){
							return true;
						}
						parts.pop();
					}
					return func("ROOT");
				}

				function preload(locale){
					locale = normalizeLocale(locale);
					forEachLocale(locale, function(loc){
						if(array.indexOf(localesGenerated, loc)>=0){
							var mid = bundlePrefix.replace(/\./g, "/")+"_"+loc;
							preloading++;
							(isXd(mid) || guaranteedAmdFormat ? require : syncRequire)([mid], function(rollup){
								for(var p in rollup){
									cache[p + "/" + loc] = rollup[p];
								}
								--preloading;
								while(!preloading && preloadWaitQueue.length){
									load.apply(null, preloadWaitQueue.shift());
								}
							});
							return true;
						}
						return false;
					});
				}

				preload();
				array.forEach(dojo.config.extraLocale, preload);
			},

			waitForPreloads = function(id, require, load){
				if(preloading){
					preloadWaitQueue.push([id, require, load]);
				}
				return preloading;
			};
	}

	if(1){
		// this code path assumes the dojo loader and won't work with a standard AMD loader
		var evalBundle=
				// use the function ctor to keep the minifiers away (also come close to global scope, but this is secondary)
				new Function(
					"__bundle",                // the bundle to evalutate
					"__checkForLegacyModules", // a function that checks if __bundle defined __mid in the global space
					"__mid",                   // the mid that __bundle is intended to define

					// returns one of:
					//		1 => the bundle was an AMD bundle
					//		a legacy bundle object that is the value of __mid
					//		instance of Error => could not figure out how to evaluate bundle

					  // used to detect when __bundle calls define
					  "var define = function(){define.called = 1;},"
					+ "    require = function(){define.called = 1;};"

					+ "try{"
					+		"define.called = 0;"
					+		"eval(__bundle);"
					+		"if(define.called==1)"
								// bundle called define; therefore signal it's an AMD bundle
					+			"return 1;"

					+		"if((__checkForLegacyModules = __checkForLegacyModules(__mid)))"
								// bundle was probably a v1.6- built NLS flattened NLS bundle that defined __mid in the global space
					+			"return __checkForLegacyModules;"

					+ "}catch(e){}"
					// evaulating the bundle was *neither* an AMD *nor* a legacy flattened bundle
					// either way, re-eval *after* surrounding with parentheses

					+ "try{"
					+ 		"return eval('('+__bundle+')');"
					+ "}catch(e){"
					+ 		"return e;"
					+ "}"
				),

			syncRequire= function(deps, callback){
				var results= [];
				array.forEach(deps, function(mid){
					var url= require.toUrl(mid + ".js");

					function load(text){
						var result = evalBundle(text, checkForLegacyModules, mid);
						if(result===1){
							// the bundle was an AMD module; re-inject it through the normal AMD path
							// we gotta do this since it could be an anonymous module and simply evaluating
							// the text here won't provide the loader with the context to know what
							// module is being defined()'d. With browser caching, this should be free; further
							// this entire code path can be circumvented by using the AMD format to begin with
							require([mid], function(bundle){
								results.push(cache[url]= bundle);
							});
						}else{
							if(result instanceof Error){
								console.error("failed to evaluate i18n bundle; url=" + url, result);
								result = {};
							}
							// nls/<locale>/<bundle-name> indicates not the root.
							results.push(cache[url] = (/nls\/[^\/]+\/[^\/]+$/.test(url) ? result : {root:result, _v1x:1}));
						}
					}

					if(cache[url]){
						results.push(cache[url]);
					}else{
						var bundle= require.syncLoadNls(mid);
						// don't need to check for legacy since syncLoadNls returns a module if the module
						// (1) was already loaded, or (2) was in the cache. In case 1, if syncRequire is called
						// from getLocalization --> load, then load will have called checkForLegacyModules() before
						// calling syncRequire; if syncRequire is called from preloadLocalizations, then we
						// don't care about checkForLegacyModules() because that will be done when a particular
						// bundle is actually demanded. In case 2, checkForLegacyModules() is never relevant
						// because cached modules are always v1.7+ built modules.
						if(bundle){
							results.push(bundle);
						}else{
							if(!xhr){
								try{
									require.getText(url, true, load);
								}catch(e){
									results.push(cache[url]= {});
								}
							}else{
								xhr.get({
									url:url,
									sync:true,
									load:load,
									error:function(){
										results.push(cache[url]= {});
									}
								});
							}
						}
					}
				});
				callback && callback.apply(null, results);
			},

			checkForLegacyModules = function(target){
				// legacy code may have already loaded [e.g] the raw bundle x/y/z at x.y.z; when true, push into the cache
				for(var result, names = target.split("/"), object = dojo.global[names[0]], i = 1; object && i<names.length-1; object = object[names[i++]]){}
				if(object){
					result = object[names[i]];
					if(!result){
						// fallback for incorrect bundle build of 1.6
						result = object[names[i].replace(/-/g,"_")];
					}
					if(result){
						cache[target] = result;
					}
				}
				return result;
			};

		thisModule.getLocalization= function(moduleName, bundleName, locale){
			var result,
				l10nName= getL10nName(moduleName, bundleName, locale).substring(10);
			load(l10nName, (!isXd(l10nName) ? syncRequire : require), function(result_){ result= result_; });
			return result;
		};

		if(has("dojo-unit-tests")){
			unitTests.push(function(doh){
				doh.register("tests.i18n.unit", function(t){
					var check;

					check = evalBundle("{prop:1}");
					t.is({prop:1}, check); t.is(undefined, check[1]);

					check = evalBundle("({prop:1})");
					t.is({prop:1}, check); t.is(undefined, check[1]);

					check = evalBundle("{'prop-x':1}");
					t.is({'prop-x':1}, check); t.is(undefined, check[1]);

					check = evalBundle("({'prop-x':1})");
					t.is({'prop-x':1}, check); t.is(undefined, check[1]);

					check = evalBundle("define({'prop-x':1})");
					t.is(1, check);

					check = evalBundle("this is total nonsense and should throw an error");
					t.is(check instanceof Error, true);
				});
			});
		}
	}

	return lang.mixin(thisModule, {
		dynamic:true,
		normalize:normalize,
		load:load,
		cache:cache
	});
});

},
'url:ibm/usmi/statuspod/templates/alertStatuspodDialog.html':"<div class=\"statuspod\">\r\n<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n<span id=\"alertStatusPodHeaderText\" class=\"alertStatusPodHeaderText\"></span>\r\n\t<div class=\"statuspodTabContainer\" id=\"statusPodTabContainer\" data-dojo-attach-point=\"statusPodTabContainer\" data-dojo-type=\"dijit.layout.TabContainer\" style=\"width: 100%; height: 100%;\" data-dojo-props='doLayout:false,controllerWidget:\"dijit.layout.TabController\"'>\r\n\t\t<div id=\"problems_pane\" data-dojo-attach-point=\"problems_pane\" data-dojo-type=\"dijit.layout.ContentPane\" title=\"${templateMRI.StatusProblemsPlaceholder}\" selected=\"true\">\r\n\t\t\t<div data-dojo-type=\"idx.oneui.Menu\" class=\"menu-problems\" data-dojo-attach-point=\"problemsMenu\">\r\n\t\t\t</div>\r\n\t\t\t\r\n\t\t\t<div class=\"statusPodPaneSpace\" data-dojo-attach-point=\"problemAlertsSpace\">\r\n\t\t\t</div>\r\n\t\t\t\r\n\t\t\t<div  class=\"footerHeader\">\r\n\t\t\t\t<span class=\"statusPodSummary\" data-dojo-attach-point=\"problemsCatSummary\">${templateMRI.StatusShowingPartialPlaceholder}</span>\r\n\t\t\t</div>\r\n\t\t\t<div  class=\"footerBottom\">\r\n\t\t\t\t<span class=\"firstLink\">\r\n\t\t\t\t\t<a href=\"javascript:menuClicked('workAreaDijit=ibm/usmi/activealerts/ActiveAlerts&alertFilter=problems');\">${templateMRI.StatusViewAllStatus}</a>\r\n\t\t\t\t</span>\r\n\t\t\t\t<span class=\"lastLink\">\r\n\t\t\t\t\t<a href=\"javascript:menuClicked('workAreaDijit=ibm/usmi/widgets/ResourceNavigator&dataset=GROUP:STATUS_PROBLEMS&forceRefresh=true');\">${templateMRI.StatusViewAll}</a>\r\n\t\t\t\t</span>\r\n\t\t\t</div>\r\n\t\t</div>\r\n\t\t\r\n\t\t<div id=\"compliances_pane\" data-dojo-attach-point=\"compliances_pane\" data-dojo-type=\"dijit.layout.ContentPane\" title=\"${templateMRI.StatusCompliancePlaceholder}\" >\r\n\t\t   <div data-dojo-type=\"idx.oneui.Menu\" class=\"menu-compliances\" data-dojo-attach-point=\"compliancesMenu\">\r\n\t\t\t</div>\r\n\t\t\t\r\n\t\t\t<div class=\"statusPodPaneSpace\" data-dojo-attach-point=\"complianceAlertsSpace\">\r\n\t\t\t</div>\r\n\r\n\t\t\t<div  class=\"footerHeader\">\r\n\t\t\t\t<span class=\"statusPodSummary\" data-dojo-attach-point=\"compliancesCatSummary\">${templateMRI.StatusShowingPartialPlaceholder}</span>\r\n\t\t\t\t\r\n\t\t\t</div>\r\n\t\t\t<div  class=\"footerBottom\">\r\n\t\t\t\t<span class=\"firstLink\">\r\n\t\t\t\t\t<a href=\"javascript:menuClicked('workAreaDijit=ibm/usmi/activealerts/ActiveAlerts&alertFilter=nonCompliances');\">${templateMRI.StatusViewAllCompliance}</a>\r\n\t\t\t\t</span>\r\n\t\t\t\t<span class=\"lastLink\">\r\n\t\t\t\t\t<a href=\"javascript:menuClicked('workAreaDijit=ibm/usmi/widgets/ResourceNavigator&dataset=GROUP:STATUS_COMPLIANCE&forceRefresh=true');\">${templateMRI.StatusViewAllSystemWithCompliance}</a>\r\n\t\t\t\t</span>\r\n\t\t\t</div>\r\n        </div>\r\n\t</div>\r\n\t\t\r\n</div>",
'dojo/store/Memory':function(){
define(["../_base/declare", "./util/QueryResults", "./util/SimpleQueryEngine"], function(declare, QueryResults, SimpleQueryEngine) {
  //  module:
  //    dojo/store/Memory
  //  summary:
  //    The module defines an in-memory object store.


return declare("dojo.store.Memory", null, {
	// summary:
	//		This is a basic in-memory object store. It implements dojo.store.api.Store.
	constructor: function(/*dojo.store.Memory*/ options){
		// summary:
		//		Creates a memory object store.
		// options:
		//		This provides any configuration information that will be mixed into the store.
		// 		This should generally include the data property to provide the starting set of data.
		for(var i in options){
			this[i] = options[i];
		}
		this.setData(this.data || []);
	},
	// data: Array
	//		The array of all the objects in the memory store
	data:null,

	// idProperty: String
	//		Indicates the property to use as the identity property. The values of this
	//		property should be unique.
	idProperty: "id",

	// index: Object
	//		An index of data indices into the data array by id
	index:null,

	// queryEngine: Function
	//		Defines the query engine to use for querying the data store
	queryEngine: SimpleQueryEngine,
	get: function(id){
		//	summary:
		//		Retrieves an object by its identity
		//	id: Number
		//		The identity to use to lookup the object
		//	returns: Object
		//		The object in the store that matches the given id.
		return this.data[this.index[id]];
	},
	getIdentity: function(object){
		// 	summary:
		//		Returns an object's identity
		// 	object: Object
		//		The object to get the identity from
		//	returns: Number
		return object[this.idProperty];
	},
	put: function(object, options){
		// 	summary:
		//		Stores an object
		// 	object: Object
		//		The object to store.
		// 	options: dojo.store.api.Store.PutDirectives??
		//		Additional metadata for storing the data.  Includes an "id"
		//		property if a specific id is to be used.
		//	returns: Number
		var data = this.data,
			index = this.index,
			idProperty = this.idProperty;
		var id = (options && "id" in options) ? options.id : idProperty in object ? object[idProperty] : Math.random();
		if(id in index){
			// object exists
			if(options && options.overwrite === false){
				throw new Error("Object already exists");
			}
			// replace the entry in data
			data[index[id]] = object;
		}else{
			// add the new object
			index[id] = data.push(object) - 1;
		}
		return id;
	},
	add: function(object, options){
		// 	summary:
		//		Creates an object, throws an error if the object already exists
		// 	object: Object
		//		The object to store.
		// 	options: dojo.store.api.Store.PutDirectives??
		//		Additional metadata for storing the data.  Includes an "id"
		//		property if a specific id is to be used.
		//	returns: Number
		(options = options || {}).overwrite = false;
		// call put with overwrite being false
		return this.put(object, options);
	},
	remove: function(id){
		// 	summary:
		//		Deletes an object by its identity
		// 	id: Number
		//		The identity to use to delete the object
		// returns: Boolean
		// 		Returns true if an object was removed, falsy (undefined) if no object matched the id
		var index = this.index;
		var data = this.data;
		if(id in index){
			data.splice(index[id], 1);
			// now we have to reindex
			this.setData(data);
			return true;
		}
	},
	query: function(query, options){
		// 	summary:
		//		Queries the store for objects.
		// 	query: Object
		//		The query to use for retrieving objects from the store.
		//	options: dojo.store.api.Store.QueryOptions?
		//		The optional arguments to apply to the resultset.
		//	returns: dojo.store.api.Store.QueryResults
		//		The results of the query, extended with iterative methods.
		//
		// 	example:
		// 		Given the following store:
		//
		// 	|	var store = new dojo.store.Memory({
		// 	|		data: [
		// 	|			{id: 1, name: "one", prime: false },
		//	|			{id: 2, name: "two", even: true, prime: true},
		//	|			{id: 3, name: "three", prime: true},
		//	|			{id: 4, name: "four", even: true, prime: false},
		//	|			{id: 5, name: "five", prime: true}
		//	|		]
		//	|	});
		//
		//	...find all items where "prime" is true:
		//
		//	|	var results = store.query({ prime: true });
		//
		//	...or find all items where "even" is true:
		//
		//	|	var results = store.query({ even: true });
		return QueryResults(this.queryEngine(query, options)(this.data));
	},
	setData: function(data){
		// 	summary:
		//		Sets the given data as the source for this store, and indexes it
		//	data: Object[]
		//		An array of objects to use as the source of data.
		if(data.items){
			// just for convenience with the data format IFRS expects
			this.idProperty = data.identifier;
			data = this.data = data.items;
		}else{
			this.data = data;
		}
		this.index = {};
		for(var i = 0, l = data.length; i < l; i++){
			this.index[data[i][this.idProperty]] = i;
		}
	}
});

});

},
'gridx/core/model/_Extension':function(){
define([
	'dojo/_base/declare',
	"dojo/_base/lang",
	'dojo/_base/array',
	'dojo/aspect'
], function(declare, lang, array, aspect){

	return declare([], {
		// summary:
		//		Abstract base class for all model components (including cache)
		constructor: function(model){
			var t = this,
				i = t.inner = model._model;
			t._cnnts = [];
			t.model = model;
			model._model = t;
			if(i){
				t.aspect(i, 'onDelete', '_onDelete');
				t.aspect(i, 'onNew', '_onNew');
				t.aspect(i, 'onSet', '_onSet');
			}
		},

		destroy: function(){
			array.forEach(this._cnnts, function(cnnt){
				cnnt.remove();
			});
		},

		aspect: function(obj, e, method, scope, pos){
			var cnnt = aspect[pos || 'after'](obj, e, lang.hitch(scope || this, method), 1);
			this._cnnts.push(cnnt);
			return cnnt;
		},

		//Events----------------------------------------------------------------------
		//Make sure every extension has the oppotunity to decide when to fire an event at its level.
		_onNew: function(){
			this.onNew.apply(this, arguments);
		},

		_onSet: function(){
			this.onSet.apply(this, arguments);
		},

		_onDelete: function(){
			this.onDelete.apply(this, arguments);
		},

		onNew: function(){},
		onDelete: function(){},
		onSet: function(){},

		//Protected-----------------------------------------------------------------
		_call: function(method, args){
			var t = this,
				m = t[method],
				n = t.inner;
			return m ? m.apply(t, args || []) : n && n._call(method, args);
		},

		_mixinAPI: function(){
			var i,
				m = this.model,
				args = arguments,
				api = function(method){
					return function(){
						return m._model._call(method, arguments);
					};
				};
			for(i = args.length - 1; i >= 0; --i){
				m[args[i]] = api(args[i]);
			}
		}
	});
});

},
'ibm/usmi/widgets/finder/Finder':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
// Finder Base
define("ibm/usmi/widgets/finder/Finder", [
	"dojo/_base/lang",
	"dojo/dom",
	"dojo/_base/array",
	"dojo/on",
	"dojo/_base/connect",
	"dojo/dom-construct",
	"dojo/dom-class",
	"dijit",
	"dojo/_base/declare",
	"dijit/_WidgetBase",
	// Need this because we are declaring a widget in our template
	"dojo/cookie",
	"dijit/form/TextBox",
	"dijit/TooltipDialog",
	"dijit/popup",
	"dijit/layout/TabContainer",
	"dijit/layout/ContentPane",
	"dojo/i18n!ibm/usmi/nls/frameworkResources",
	"ibm/usmi/util/DataStore",
	"ibm/usmi/util/Utils",
	"ibm/usmi/scheduler/JobLaunchWizard",
	"./TaskDataStore",
	"./ResourceDataStore",
	"./GroupDataStore",
	"ibm/tivoli/navmodel/Dataset"
], function (djLang, djDom, djArray, on, connect, djDomC, djClass, dijit, declare, _WidgetBase, djCookie, djTextBox,
		djTooltipDialog, djPopup, djTabContainer, djContentPane, MRI, DataStore, Utils, JobLaunchWizard, TaskDataStore,
		ResourceDataStore, GroupDataStore, CuriDataset) {
	return declare([ _WidgetBase
	], {
		// templateString : template,
		finderHelpTooltip : MRI.FinderTooltipHelp,
		searchTextHintString : "hint",
		tooltipHelp : MRI.FinderTooltipHelp,

		_searchTextHintString : null,
		_tabbedContainer : null,

		// references to finder elements
		_finderDomObj : null, // finder main dom container
		_searchDetailDlg : null, // finder results tooltip
		// dialog
		tooltipShown : null,
		// finder state variables
		_searchDlgIsOpen : false,
		_searchDlgInitialized : false,
		_latestSearchEntry : null,
		_currentView : '0',
		_refreshImage : null,
		_currentResultCountArray : null,

		_dataStore : null,

		//Refresh variables
		blockEvents : true,
		pendingEvents : false,
		initialized : false,
		searchOpen : false,
		origStyle : "dijitReset dijitInline dijitButtonNode",
		refreshStyle : "dijitReset dijitInline finderRefresh",
		lastRefresh : 0,
		manualRefresh : false,

		// Data stores containing cached lists
		_taskDataStore : null,
		_resourceDataStore : null,
		_groupDataStore : null,
		_indexedItems : null,
		// _keyPressed : 0,

		constructor : function () {
			this.inherited(arguments);
			this._indexedItems = [];
			Utils.CMTraceTimeInit("finderTimer");
			this._searchTextHintString = MRI.FinderInitialText;
			this.searchTextHintString = this._searchTextHintString;
			this._currentSingleResultArray = {
				taskId : null,
				resourceId : null,
				groupId : null
			};
		},
		postCreate : function () {
			this.inherited(arguments);
			this.initializeFieldReferences();
			this.initializeFinderHtml();

			this._dataStore = new DataStore({
				datasetId : 'FinderCollection',
				initReadyCallback : djLang.hitch(this, this.initializeDataStores)
			});

		},
		/*
		 * 
			// setTimeout(djLang.hitch(this, this.foo), 50);
		 * foo : function() { this._dataStore = new DataStore({
		 * datasetId : 'FinderCollection', homeParms : 150, getItems :
		 * true, retrieveItemProperties : true, getTasks : true,
		 * initReadyCallback : djLang.hitch(this,
		 * this.initializeDataStores) }); },
		 */
		incrementRefreshMinute : function () {
			if (this.lastRefresh === 15) {
				this._refreshCache();
			} else {
				this.lastRefresh = this.lastRefresh + 1;
				if (this._refreshImage) {
					this.changeRefreshCount();
				}
			}
			setTimeout(djLang.hitch(this, 'incrementRefreshMinute'), 60000);
		},
		changeRefreshCount : function () {
			this._refreshImage.title = djLang.replace(MRI.RefreshCache, [
				this.lastRefresh,
				MRI.Minutes
			]);

		},
		initializeDataStores : function () {
			/*this._taskDataStore = new TaskDataStore('tasks', this.retrieveDatasetCache([
				'tasks',
				'jobs',
				'ngenTask'
			]));*/
			this.initializeResourcesAndGroups(true);
			setTimeout(djLang.hitch(this, 'incrementRefreshMinute'), 60000);
		},
		_doRefreshProcessing : function () {
			if (this._searchDlgIsOpen) {
				this.searchOpen = true;
			}
			djDom.byId("widget_findersearchBox").nextSibling.childNodes[0].className = this.refreshStyle;
			if (this._latestSearchEntry) {
				djCookie("finder.lastSearch", this._latestSearchEntry, {
					expires : 1
				});
			}
			var inputField = dijit.byId("findersearchBox").domNode.childNodes[0].childNodes[0];
			this.closeSearchDetailDlg();
			inputField.value = MRI.FinderRefreshing;
			inputField.disabled = true;
		},
		_manualRefresh : function () {
			if (this._searchDlgIsOpen) {
				this.searchOpen = true;
			}

			this.manualRefresh = true;
			this._doRefreshProcessing();
			this._refreshCache();

		},
		_refreshCache : function () {
			if (this.blockEvents) { //Refresh is in progress, don't refresh
				this.pendingEvent = true; //save the need to refresh again later
				return;
			}

			this.lastRefresh = 0;
			this.blockEvents = true;
			this.pendingEvent = false;
			if (!this.manualRefresh) {
				this._doRefreshProcessing();
			} else {
				this.manualRefresh = false;
			}
			this._dataStore.destroy();
			if (this._resourceDataStore) {
				this._resourceDataStore.destroy();
				this._resourceDataStore = null;
			}
			if (this._groupDataStore) {
				this._groupDataStore.destroy();
				this._groupDataStore = null;
			}
			this._dataStore = new DataStore({
				datasetId : 'FinderCollection',
				initReadyCallback : djLang.hitch(this, this.initializeResourcesAndGroups)
			});
		},
		initializeResourcesAndGroups : function (initial) {
			this._dataStore.getDataset().addEventListener(this, djLang.hitch(this, this._refreshCache));
			this._taskDataStore = new TaskDataStore('tasks', this.retrieveDatasetCache([
				'tasks',
				'jobs',
				'ngenTask'
			]));
			this._resourceDataStore = new ResourceDataStore('resources', this.retrieveDatasetCache('resources'));
			this._groupDataStore = new GroupDataStore('groups', this.retrieveDatasetCache('groups'));
			window.setTimeout(djLang.hitch(this, this._unblockEvents), '10000');
			if (initial === true) {} else {
				this.initializeFinderHtml();
				djDom.byId("widget_findersearchBox").nextSibling.childNodes[0].className = this.origStyle;
				dijit.byId("findersearchBox").domNode.childNodes[0].childNodes[0].disabled = false;
				if (this.searchOpen) {
					this.searchOpen = false;
					this.finderKeyPressed({});
				}
			}

		},
		_unblockEvents : function () {
			this.blockEvents = false; //Refresh is done, we can now listen for events again
			if (this.pendingEvent) { //did any more events come in during our refresh? if so, refresh again
				this._refreshCache();
			}
		},
		retrieveDatasetCache : function (
		/* String or array of strings*/types) {
			var items = [], idx;
			djArray.forEach(this._dataStore.getItems(), djLang.hitch(this, function (entry, i) {
				var matches = false;
				var type = entry.getProperty('FINDER_TYPE_COLUMN').getValue();
				if (types instanceof Array) {
					for (idx = 0; idx < types.length; idx++) {
						if (types[idx] === type) {
							matches = true;
						}
					}
				} else {
					matches = type === types;
				}
				if (matches) {
					var iconString = null;
					if (entry.getProperty('FINDER_ICON_COLUMN')) {
						iconString = entry.getProperty('FINDER_ICON_COLUMN').getValue();
					}
					var itId = entry.getProperty('FINDER_ID_COLUMN').getValue();
					items.push({
						id : itId,
						label : entry.getProperty('FINDER_NAME_COLUMN').getValue(),
						category : entry.getProperty('FINDER_CATEGORY_COLUMN').getValue(),
						icon : iconString
					});
					this._indexedItems[itId] = entry;
				}
			}));
			return items;
		},

		initializeFieldReferences : function (domId) {
			this._finderDomObj = djDom.byId('finder');
		},
		initializeFinderHtml : function () {
			var inputFieldWidget = dijit.byId("findersearchBox");
			var inputField = inputFieldWidget.domNode.childNodes[0].childNodes[0];

			var lastEnteredSearch = djCookie("finder.lastSearch");
			if (lastEnteredSearch) {
				inputField.value = lastEnteredSearch;
				djCookie("finder.lastSearch", null, {
					expires : -1
				});
			} else {
				inputField.value = this._searchTextHintString;
			}
			if (!this.initialized) {
				on(inputField, "blur", djLang.hitch(this, this.doSearchBoxBlur));
				on(inputField, "focus", djLang.hitch(this, this.doSearchBoxFocus));
				on(inputField, "keyup", djLang.hitch(this, this.finderKeyPressed));
				this.initialized = true;
			}

			if (dijit.byId("finderdialog")) {
				dijit.byId("finderdialog").destroy();
			}
			this._searchDetailDlg = new djTooltipDialog({
				id : "finderdialog",
				style : "width:450px;height:150px;",
				resize : function () {}/* HACK, figure out why resize kills the dialog*/
			});
			this._searchDetailDlg.startup();

		},
		fetchFailed : function (error, request) {
			console.error("finder: lookup failed.");
			Utils.CMFinest('finder: ' + error);
		},
		doSearchBoxBlur : function () {
			if (dijit.byId("findersearchBox").get('value') === "") {
				dijit.byId("findersearchBox").set('value', this._searchTextHintString);
			}
		},
		doSearchBoxFocus : function () {
			if (dijit.byId("findersearchBox").get('value') === this._searchTextHintString) {
				dijit.byId("findersearchBox").set('value', "");
			}
			dijit.byId("findersearchBox").focus();
		},
		closeSearchDetailDlg : function (keepFocusOnInput) {
			if (this._searchDetailDlg) {
				djPopup.close(this._searchDetailDlg);
				this._searchDetailDlg.destroyRecursive();
				this._searchDetailDlg = new djTooltipDialog({
					id : "finderdialog",
					style : "width:450px;height:150px;",
					resize : function () {}/* HACK, figure out why resize kills the dialog*/
				});
				this._searchDetailDlg.startup();
			}
			// dijit.byId("findersearchBox").attr("value",
			// this._searchTextHintString)
			this._searchDlgIsOpen = false;
			this._latestSearchEntry = '';
			var inputFieldWidget = dijit.byId("findersearchBox");
			var inputField = inputFieldWidget.domNode.childNodes[0].childNodes[0];
			if (keepFocusOnInput !== true) {
				inputField.value = this._searchTextHintString;
				inputField.blur();
			}
			this._taskDataStore.clearTooltips();
			this._resourceDataStore.clearTooltips();
			this._groupDataStore.clearTooltips();

			// var obj = dijit.byId("findersearchBox").domNode;
			// obj.blur();
		},
		doTaskSearch : function (myitems, myrequest) {
			if (myitems.length === 1) {
				this._currentSingleResultArray.taskId = myitems[0].id;
			} else {
				this._currentSingleResultArray.taskId = null;
			}
			this.doSearch(myitems, myrequest, this._taskDataStore,
			/*'special#_com.ibm.usmi.console.taskmap'*/'special#_none', MRI.MoreTasks, 'tasks', true);
		},
		doResourceSearch : function (myitems, myrequest) {
			if (myitems.length === 1) {
				this._currentSingleResultArray.resourceId = myitems[0].id;
			} else {
				this._currentSingleResultArray.resourceId = null;
			}
			this.doSearch(myitems, myrequest, this._resourceDataStore,
					'special#_com.ibm.usmi.console.tasks.QuickFindTask', MRI.MoreResources, 'resources', false);
		},
		doGroupSearch : function (myitems, myrequest) {
			if (myitems.length === 1) {
				this._currentSingleResultArray.groupId = myitems[0].id;
			} else {
				this._currentSingleResultArray.groupId = null;
			}
			this.doSearch(myitems, myrequest, this._groupDataStore, 'special#_none', MRI.MoreGroups, 'groups', true);
		},
		doSearch : function (myitems, myrequest, dataStore, moreItemsTask, moreItemsText, typeOfSearch,
				moreItemsNotClickable) {
			Utils.CMTraceTimeDelta("finderTimer", "doSearch" + typeOfSearch, true);
			var searchString = dijit.byId("findersearchBox").get('value');
			if (searchString === "" || searchString === this._searchTextHintString) {
				return;
			}
			if (myitems) {
				if (myitems.length > 0) {
					dataStore.clearSelection();
					Utils.CMTraceTimeDelta("finderTimer", "processing results", true);
					dataStore.populateCategoriesWithSelectedItems(myitems);
					Utils.CMTraceTimeDelta("finderTimer", "results populated", true);
					this.initializeDialogConstruct(typeOfSearch, myitems.length);
					Utils.CMTraceTimeDelta("finderTimer", "initialized dialog", true);
					dataStore.createCategoryHTMLElements(myitems.length, this, moreItemsTask, moreItemsText,
							moreItemsNotClickable);
					Utils.CMTraceTimeDelta("finderTimer", "results UI created", true);

				} else {
					this.initializeDialogConstruct(typeOfSearch);
					var noMatchesFoundText;
					if (typeOfSearch === 'resources') {
						noMatchesFoundText = MRI.NoResourcesFound;
					} else if (typeOfSearch === 'tasks') {
						noMatchesFoundText = MRI.NoTasksFound;
					} else if (typeOfSearch === 'groups') {
						noMatchesFoundText = MRI.NoGroupsFound;
					}
					djDomC.create("div", {
						innerHTML : noMatchesFoundText
					}, djDom.byId('finder' + typeOfSearch + "_containerDiv"));
				}
				// djDom.byId('finder' + typeOfSearch +
				// "Menu_count").innerHTML = " ("
				// +myitems.length + ")";
			}
			if (typeOfSearch === 'resources') {
				this._tabbedContainer.startup();
				var selectedTab = this._tabbedContainer.selectedChildWidget;
				djClass.remove(selectedTab.domNode, "dijitHidden");
				djClass.add(selectedTab.domNode, "dijitVisible");
			}
		},
		doClick : function (event, id) {
			var selectedTaskId, clickType = 1;
			if (!event) {
				selectedTaskId = id;
			} else {
				clickType = event.which;
				selectedTaskId = event.currentTarget.id;
			}
			this.closeSearchDetailDlg();
			Utils.CMFinest('finder: doClick' + selectedTaskId);
			var type = selectedTaskId.split('#_')[0];
			var itemId = selectedTaskId.split('#_')[1];
			if (type.indexOf('categorySection') !== -1) {
				var category = this._resourceDataStore.getCategories()[itemId];
				var ids = category.getIds();
				selectedTaskId = 'resourceList#_';
				var i;
				for (i = 0; i < ids.length; i++) {
					if (i !== 0) {
						selectedTaskId = selectedTaskId + ",";
					}
					selectedTaskId = selectedTaskId + ids[i];
				}
			}

			// get item from dataset, since our cache doesn't
			// have all the info

			if (itemId.indexOf("com.ibm.usmi.console.taskmap") !== -1) {
				launchNextGenPage("workAreaDijit=ibm/usmi/widgets/ResourceNavigator&dataset=GROUP:USMI_TASK_MAP");
				return;
			} else if (itemId.indexOf("com.ibm.usmi.console.tasks.QuickFindTask") !== -1) {
				launchNextGenPage("workAreaDijit=ibm/usmi/widgets/ResourceNavigator&dataset=GROUP:ALL_ENDPOINTS");
				return;
			}
			var item = this._dataStore.getItemFromProperty("FINDER_ID_COLUMN", itemId);
			if (type === 'tasks') {
				if (item) {
					var taskType = item.getProperty('FINDER_TYPE_COLUMN').getValue();
					if (taskType === 'tasks') {
						var pageid = item.getProperty('FINDER_PAGE_ID_COLUMN').getValue();
						launchInContext(pageid);
						// interactive ISC task, launch it
					} else if (taskType === 'jobs') {
						// non interactive task, launch launcher
						// dialog widget
						this.showSchedulerForJob(itemId, item.getProperty('FINDER_NAME_COLUMN').getValue());
					} else if (taskType === 'ngenTask') {
						// non interactive task, launch launcher
						// dialog widget

						var pageid = item.getProperty('FINDER_PAGE_ID_COLUMN').getValue();
						if (clickType > 1) {
							window.open(window.location.href.split("#")[0] + "#" + pageid);
						} else {
							launchNextGenPage(pageid);
						}
					}
				}
			} else if (type === 'resources') {
				var resourceTypeProperty = item.getProperty('FINDER_RESOURCE_TYPE_COLUMN');
				var url = "workAreaDijit=ibm/usmi/widgets/WorkArea&sourceToken=" + itemId + "&selector=details";
				if (resourceTypeProperty) {
					var resourceType = (resourceTypeProperty.getValue());
					if (resourceType === 'System Chassis' || resourceType === 'Server' || resourceType === 'Switch'
							|| resourceType === 'Generic System') {
						url = "workAreaDijit=fsm/Pages/fsmChassis/chassisMapPage&sourceToken=" + itemId
								+ "&selector=ibm/usmi/dashboard/ResourceDashBoard&selectedGroup=chassis";

					}

				}
				if (clickType > 1) {
					window.open(window.location.href.split("#")[0] + "#" + url);
				} else {
					launchNextGenPage(url);

				}
			} else if (type === 'groups') {
				var url = "workAreaDijit=ibm/usmi/widgets/ResourceNavigator&dataset=GROUP:" + itemId;
				if (clickType > 1) {
					window.open(window.location.href.split("#")[0] + "#" + url);
				} else {
					launchNextGenPage(url);
				}
			}
		},
		setTooltip : function (id) {
			this.tooltipShown = id;
		},
		showSchedulerForJob : function (jobId, jobTitle) {
			var wizard = new JobLaunchWizard({
				id : "launchFinderJobScheduler",
				iTaskId : jobId,
				jobName : jobTitle
			});
			launchSchedulerDialog(jobTitle, wizard);

		},
		finderKeyPressed : function (event) {
			Utils.CMFinest('finder: keyPressed');
			Utils.CMTraceTimeDelta("finderTimer2", "keyPressed", true);
			if (this.tooltipShown) {
				this.tooltipShown.close();

			}
			if (event.keyCode === 27) {
				dijit.byId("findersearchBox").set("value", "");
			} else if (event.keyCode === 13) {
				var resultId;
				if (this._currentView === 'resources') {
					resultId = this._currentSingleResultArray.resourceId;
				} else if (this._currentView === 'groups') {
					resultId = this._currentSingleResultArray.groupId;
				} else if (this._currentView === 'tasks') {
					resultId = this._currentSingleResultArray.taskId;
				}
				if (resultId) {
					this.doClick(null, this._currentView + "#_" + resultId);
				}
				return;

			}
			var searchStringEntered = dijit.byId("findersearchBox").get('value');
			if (this._latestSearchEntry === searchStringEntered) {
				return;
			} else {
				this._latestSearchEntry = searchStringEntered;
			}
			if ((searchStringEntered === "") || (searchStringEntered === this._searchTextHintString)) { // ||
				// searchStringEntered.length
				// == 1
				this.closeSearchDetailDlg(true);
				return;
			}
			// this._keyPressed++;
			if (!this._searchDlgIsOpen || this._searchDetailDlg._isShown() === false) {
				djPopup.open({
					popup : this._searchDetailDlg,
					around : dijit.byId("findersearchBox").domNode,
					onCancel : djLang.hitch(this, this.closeSearchDetailDlg),
					parent : dijit.byId("findersearchBox")
				});
			}

			this._searchDlgIsOpen = true;
			dijit.byId("findersearchBox").focus();
			var searchTerm = searchStringEntered;
			Utils.CMFinest('finder: query: ' + searchTerm);
			this._searchDlgInitialized = false;
			this.initiateSearch(searchTerm, 0);
			// window.setTimeout(djLang.hitch(this,
			// this.initiateSearch, searchTerm, 0,
			// this._keyPressed), 500);

		},
		initiateSearch : function (searchTerm, state/* ,keyPressedNumber*/) {
			/*
			 * if(this._keyPressed !== keyPressedNumber) { return; }
			 */
			if (this._taskDataStore && this._groupDataStore && this._resourceDataStore) {
				if (djDom.byId('searchingText')) {
					djDomC.destroy('searchingText');
				}
				this._taskDataStore.initiateTaskSearch(searchTerm, this);
				this._groupDataStore.initiateTaskSearch(searchTerm, this);
				this._resourceDataStore.initiateTaskSearch(searchTerm, this);
			} else {
				var stateNumber = state % 3;
				if (!djDom.byId('searchingText')) {
					djDomC.create("div", {
						id : 'searchingText',
						innerHTML : MRI['Searching' + stateNumber]
					}, this._searchDetailDlg.containerNode);
				} else {
					djDom.byId('searchingText').innerHTML = MRI['Searching' + stateNumber];
				}
				setTimeout(djLang.hitch(this, 'initiateSearch', searchTerm, state + 1 /* , keyPressedNumber*/), 200);
			}
		},

		initializeDialogConstruct : function (typeOfSearch, items) {
			if (this._searchDlgInitialized === false) {
				this._searchDlgInitialized = true;
				djDomC.empty(this._searchDetailDlg.containerNode);
				djDomC.create("img", {
					onclick : djLang.hitch(this, this.closeSearchDetailDlg),
					src : "js/ibm/usmi/core/framework/themes/images/title_close.gif",
					title : MRI.Close,
					className : "closeImg"
				}, this._searchDetailDlg.containerNode);
				this._refreshImage = djDomC.create("img", {
					onclick : djLang.hitch(this, this._manualRefresh),
					src : "js/ibm/usmi/core/framework/themes/images/ac16_refresh.png",
					className : "refreshImg"
				}, this._searchDetailDlg.containerNode);
				this.changeRefreshCount();

				var header = djDomC.create("span", {
					id : "finderfinderMenu",
					style : "display:inline;"
				}, this._searchDetailDlg.containerNode);

				var mainDivTabContainer = djDomC.create("div", {
					id : 'mainDivTabContainer'
				}, this._searchDetailDlg.containerNode);
				if (dijit.byId("findertab")) {
					dijit.byId("findertab").destroy();
				}
				this._tabbedContainer = new djTabContainer({
					id : "findertab",
					style : "width:100%;",
					doLayout : true,
					useMenu : false,
					useSlider : false
				}, mainDivTabContainer);
				var lastSelectedTab = djCookie("finder.currentView");
				if (lastSelectedTab && lastSelectedTab.length > 0) {
					this._currentView = lastSelectedTab;
				}
				this._tabbedContainer.addChild(this.initializeMenu('resources', MRI.FinderResourceMenu, header,
						this._tabbedContainer, (this._currentView === 'resources')));
				this._tabbedContainer.addChild(this.initializeMenu('groups', MRI.FinderGroupMenu, header,
						this._tabbedContainer, (this._currentView === 'groups')));
				this._tabbedContainer.addChild(this.initializeMenu('tasks', MRI.FinderTaskMenu, header,
						this._tabbedContainer, (this._currentView === 'tasks')));
				connect.connect(this._tabbedContainer, "selectChild", djLang.hitch(this, this.tabSelected));
			}
			// add the count in parenthesis to the tabs
			var numItems;
			if (items) {
				numItems = items;
			} else {
				numItems = 0;
			}
			if (this._tabbedContainer) {
				var children = this._tabbedContainer.getChildren();
				var idx = 0;
				if (typeOfSearch === 'groups') {
					idx = 1;
				} else if (typeOfSearch === 'tasks') {
					idx = 2;
				}
				children[idx].title = children[idx].title + "(" + numItems + ")";
			}
		},

		tabSelected : function (child) {
			if (child.content.id.indexOf('tasks') !== -1) {
				this._currentView = 'tasks';
			} else if (child.content.id.indexOf('resources') !== -1) {
				this._currentView = 'resources';
			} else if (child.content.id.indexOf('groups') !== -1) {
				this._currentView = 'groups';
			}
			djCookie("finder.currentView", this._currentView, {
				expires : 30
			});
			Utils.CMFinest('finder: ' + child);
			Utils.CMFinest('finder: ' + this._currentView);
		},

		initializeMenu : function (type, headerText, header, tabbedContainer, isSelected) {
			var div = djDomC.create("div", {
				id : 'finder' + type + "_containerDiv"
			});
			var cp1 = new djContentPane({
				title : headerText,
				content : div,
				refreshOnShow : true,
				selected : isSelected
			});
			return cp1;
		},
		destroy : function () {
			this.inherited(arguments);
			if (this._dataStore) {
				this._dataStore.destroy();
				this._dataStore = null;
			}
			if (this._resourceDataStore) {
				this._resourceDataStore.destroy();
				this._resourceDataStore = null;
			}
			if (this._groupDataStore) {
				this._groupDataStore.destroy();
				this._groupDataStore = null;
			}
			if (this._taskDataStore) {
				this._taskDataStore.destroy();
				this._taskDataStore = null;
			}

		}
	});
});

},
'gridx/modules/Tree':function(){
define([
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/dom-class",
	"dojo/dom-geometry",
	"dojo/_base/lang",
	"dojo/_base/Deferred",
	"dojo/DeferredList",
	"dojo/query",
	"dojo/keys",
	"../core/util",
	"../core/_Module"
], function(kernel, declare, array, domClass, domGeometry, lang, Deferred, DeferredList, query, keys, util, _Module){
	kernel.experimental('gridx/modules/Tree');

	function isExpando(cellNode){
		var n = cellNode.firstChild;
		return n && n.className && domClass.contains(n, 'gridxTreeExpandoCell') &&
			!domClass.contains(n, 'gridxTreeExpandoLoading');
	}

	_Module._markupAttrs.push('!expandLevel');

	return declare(/*===== "gridx.modules.Tree", =====*/_Module, {
		// summary:
		//		Tree Grid module.
		// description:
		//		This module is used for creation, destruction and management of the Tree Grid.
		//		There are two kind of Tree Grid: columnar or nested, it will be indicated by
		//		the argument `type`, and the layout of the TreeGrid will be defined by extended
		//		`structure` argument.
		// example:
		//		For the columnar Tree Grid, the column which is expandable is indicated by the 
		//		new added attribute `expandField`, and the value of the `expandField` is one 
		//		or more attribute names (attributes in the meta data) that specify that item's 
		//		children.
		//		See the quick sample below:
		//
		//		sample of data source
		//	|	{ identifier: 'name',
		//	|	  label: 'name',
		//	|	  items: [
		//	|		{ name:'Africa', type:'continent', children: [
		//	|			{ name:'Egypt', type:'country' }, 
		//	|			{ name:'Kenya', type:'country', children:[
		//	|				{ name:'Nairobi', type:'city', adults: 70400, popnum: 2940911 },
		//	|				{ name:'Mombasa', type:'city', adults: 294091, popnum: 707400 } ]
		//	|			},
		//	|			{ name:'Sudan', type:'country', children:
		//	|				{ name:'Khartoum', type:'city', adults: 480293, popnum: 1200394 } 
		//	|			} ]
		//	|		},
		//	|		{ name:'Asia', type:'continent', children:[
		//	|			{ name:'China', type:'country' },
		//	|			{ name:'India', type:'country' },
		//	|			{ name:'Russia', type:'country' },
		//	|			{ name:'Mongolia', type:'country' } ]
		//	|		},
		//	|		{ name:'Australia', type:'continent', population:'21 million', children:
		//	|			{ name:'Commonwealth of Australia', type:'country', population:'21 million'}
		//	|		} ]
		//	|	}
		//
		//		define the grid structure
		//	|	var structure = [
		//	|		{name: "Name", field: "name", expandLevel: 'all'},
		//	|		{name: "Type", field: "type"},
		//	|		{name: "Population", field: "population"}
		//	|	];
		//		
		//		For the nested TreeGrid, there could be more than one column can be expanded, 
		//		so the structure might be a little more complicated. There is ONLY one attribute
		//		name can be assigned to the `expandField` as children, and there would be a 
		//		attribute called `nestedLevel` to specify the hierarchy of the column.
		//		A quick sample:
		//
		//		sample of data source
		//		
		//	|	{ identifier: 'id',
		//	|	  items: [
		//	|		{ id: "1", playername: "Player 1", seasons: [
		//	|			{ id: "2", seasonindex: "Season 1", games: [
		//	|				{ id: "3", gameindex: "Game 1", quarters: [
		//	|					{id: "4", point: "3", rebound: "3", assistant: "1"},
		//	|					{id: "5", point: "5", rebound: "0", assistant: "0"},
		//	|					{id: "6", point: "0", rebound: "1", assistant: "3"},
		//	|					{id: "7", point: "2", rebound: "2", assistant: "0"} ]
		//	|				},
		//	|				{ id: "8", gameindex: "Game 2", quarters: [
		//	|					{id: "9", point: "3", rebound: "0", assistant: "2"},
		//	|					{id: "10", point: "0", rebound: "4", assistant: "1"},
		//	|					{id: "11", point: "5", rebound: "0", assistant: "1"},
		//	|					{id: "12", point: "10", rebound: "2", assistant: "0"} ]
		//	|				} ]
		//	|			},
		//	|			{ id: "13", seasonindex: "Season 2" } ]
		//	|		} ]
		//	|	}
		//
		//		define the tree grid type
		//	|	treeNested: true;
		//
		//		define the grid structure
		//	|	var structure = [
		//	|		{name: "Player", field: "playername", expandLevel: 1},
		//	|		{name: "Season", field: "seasonindex", expandLevel: 2},
		//	|		{name: "Game", field: "gameindex", expandLevel: 3},
		//	|		{name: "Point", field: "point"},
		//	|		{name: "Rebound", field: "rebound"},
		//	|		{name: "Assistant", field: "assistant"}
		//	|	];
		
		name: "tree",

		constructor: function(){
			this._clear();
		},
	
		getAPIPath: function(){
			// tags:
			//		protected extension
			return {
				tree: this
			};
		},
	
		preload: function(){
			// tags:
			//		protected extension
			var t = this,
				g = t.grid;
			if(t.model.treeMarkMode){
				t.model.treeMarkMode('', true);
			}
			g.domNode.setAttribute('role', 'treegrid');
			t.batchConnect(
				[g.body, 'collectCellWrapper', '_createCellWrapper'],
				[g.body, 'onAfterRow', '_onAfterRow'],
				[g, 'onCellClick', '_onCellClick'],
				[g, 'setStore', '_clear']);
			t._initExpandLevel();
			t._initFocus();
			if(g.persist){
				var id,
					data = g.persist.registerAndLoad('tree', function(){
						return {
							openInfo: this._openInfo, 
							parentOpenInfo: this._parentOpenInfo
						};
					});
				if(data){
					var openInfo = t._openInfo = data.openInfo,
						parentOpenInfo = t._parentOpenInfo = data.parentOpenInfo;
					for(id in openInfo){
						openInfo[id].openned = parentOpenInfo[id];
					}
					t._persisted = 1;
				}
			}
		},

		load: function(args){
			// tags:
			//		protected extension
			var t = this;
			if(t._persisted){
				t.loaded.callback();
			}else{
				t.model.when({}, function(){
					t._openInfo[''].count = t.model.size();
				}).then(function(){
					t.loaded.callback();
				});
			}
		},

		rowMixin: {
			canExpand: function(){
				return this.grid.tree.canExpand(this.id);
			},
			isExpanded: function(){
				return this.grid.tree.isExpanded(this.id);
			},
			expand: function(){
				return this.grid.tree.expand(this.id);
			},
			collapse: function(){
				return this.grid.tree.collapse(this.id);
			},
			expandRecursive: function(){
				return this.grid.expandRecursive(this.id);
			},
			collapseRecursive: function(){
				return this.grid.collapseRecursive(this.id);
			}
		},
	
		//Public--------------------------------------------------------------------------------

		// nested: Boolean
		//		If set to true, the tree nodes can be shown in nested mode.
		nested: false,

		// expandoPadding: Integer
		//		The padding added for each level of expando. Unit is pixel. Default to 18.
		expandoPadding: 18,

		// expandLevel: Integer
		//		The maximum allowed expand level of this tree grid.
		//		If less than 1, then this is not a tree grid at all.
		expandLevel: 1 / 0,

		onExpand: function(id){
			// summary:
			//		Fired when a row is expanded.
			// tags:
			//		callback
			// id: String
			//		The ID of the expanded row
		},

		onCollapse: function(id){
			// summary:
			//		Fired when a row is collapsed.
			// tags:
			//		callback
			// id: String
			//		The ID of the collapsed row.
		},

		canExpand: function(id){
			// summary:
			//		Check whether a row can be expanded.
			// id: String
			//		The row ID
			// returns:
			//		Whether the row can be expanded.
			var t = this,
				m = t.model,
				level = m.treePath(id).length,
				expandLevel = t.arg('expandLevel');
			return m.hasChildren(id) && (!(expandLevel > 0) || level <= expandLevel);
		},
	
		isExpanded: function(id){
			// summary:
			//		Check whether a row is already expanded.
			// id: String
			//		The row ID
			// returns:
			//		Whether the row is expanded.
			return !!this._openInfo[id];	//Boolean
		},

		expand: function(id, skipUpdateBody){
			// summary:
			//		Expand the row.
			// id: String
			//		The row ID
			// skipUpdateBody: Boolean
			//		If set to true the grid will not automatically refresh itself after this method,
			//		so that several grid operations can be executed altogether.
			// returns:
			//		A deferred object indicating whether this expanding process has completed.
			var d = new Deferred(),
				t = this;
			if(id && !t.isExpanded(id)){
				t.model.when({
					parentId: id, 
					start: 0
				}, function(){
					t._logicExpand(id);
				}).then(function(){
					Deferred.when(skipUpdateBody || t._updateBody(id), function(){
						d.callback();
						t.onExpand(id);
					});
				});
			}else{
				d.callback();
			}
			return d;	//dojo.Deferred
		},
	
		collapse: function(id, skipUpdateBody){
			// summary:
			//		Collapse a row.
			// id: String
			//		The row ID
			// skipUpdateBody: Boolean
			//		If set to true the grid will not automatically refresh itself after this method,
			//		so that several grid operations can be executed altogether.
			// returns:
			//		A deferred object indicating whether this collapsing process has completed.
			var d = new Deferred(),
				t = this;
			if(id && t.isExpanded(id)){
				t._logicCollapse(id);
				Deferred.when(skipUpdateBody || t._updateBody(id), function(){
					d.callback();
					t.onCollapse(id);
				});
			}else{
				d.callback();
			}
			return d;	//dojo.Deferred
		},
	
		expandRecursive: function(id, skipUpdateBody){
			// summary:
			//		Recursively expand a row and all its descendants.
			// id: String
			//		The row ID
			// skipUpdateBody: Boolean
			//		If set to true the grid will not automatically refresh itself after this method,
			//		so that several grid operations can be executed altogether.
			// returns:
			//		A deferred object indicating whether this expanding process has completed.
			var t = this,
				m = t.model,
				d = new Deferred();
			t.expand(id, 1).then(function(){
				var i, dl = [], size = m.size(id);
				m.when({start: 0, parentId: id}, function(){
					for(i = 0; i < size; ++i){
						var childId = m.indexToId(i, id);
						dl.push(t.expandRecursive(childId, 1));
					}
				}).then(function(){
					new DeferredList(dl).then(function(){
						Deferred.when(skipUpdateBody || t._updateBody(id), function(){
							d.callback();
						});
					});
				});
			});
			return d;
		},
	
		collapseRecursive: function(id, skipUpdateBody){
			// summary:
			//		Recursively collapse a row recursively and all its descendants.
			// id: String
			//		The row ID
			// skipUpdateBody: Boolean
			//		If set to true the grid will not automatically refresh itself after this method,
			//		so that several grid operations can be executed altogether.
			// returns:
			//		A deferred object indicating whether this collapsing process has completed.
			var d = new Deferred(),
				success = lang.hitch(d, d.callback),
				fail = lang.hitch(d, d.errback),
				t = this,
				info = t._openInfo[id || ''],
				i, dl = [];
			if(info){
				for(i = info.openned.length - 1; i >= 0; --i){
					dl.push(t.collapseRecursive(info.openned[i], 1));
				}
				new DeferredList(dl).then(function(){
					if(id){
						t.collapse(id, skipUpdateBody).then(success, fail);
					}else if(!skipUpdateBody){
						t._updateBody().then(success, fail);
					}
				});
			}else{
				success();
			}
			return d;
		},
	
		//Package------------------------------------------------------------------------------
		getRowInfoByVisualIndex: function(visualIndex, rootStart){
			// summary:
			//		Get row info (including row index, row id, parent id, etc) by row visual index.
			// tags:
			//		package
			// visualIndex: Integer
			// rootStart: Integer
			// returns:
			//		A row info object
			var t = this,
				rootOpenned = t._openInfo[''].openned,
				root, i;
			for(i = 0; i < rootOpenned.length; ++i){
				root = t._openInfo[rootOpenned[i]];
				if(root.index < rootStart){
					visualIndex += root.count + 1;
				}else{
					break;
				}
			}
			var info = {
				parentId: '',
				preCount: 0
			};
			while(!info.found){
				info = t._getChild(visualIndex, info);
			}
			return info;	//Object
		},
	
		getVisualIndexByRowInfo: function(parentId, rowIndex, rootStart){
			// tags:
			//		package
			var index = this._getAbsoluteVisualIndex(parentId, rowIndex);
			return index >= 0 ? index - this._getAbsoluteVisualIndex('', rootStart) : null;
		},
	
		getVisualSize: function(start, count, parentId){
			// tags:
			//		package
			var info = this._openInfo[parentId || ''];
			if(info){
				var i, len = info.openned.length, child, size = count;
				for(i = 0; i < len; ++i){
					child = this._openInfo[info.openned[i]];
					if(child.index >= start && child.index < start + count){
						size += child.count;
					}
				}
				return size;
			}
			return 0;
		},
	
		//Private-------------------------------------------------------------------------------
		//_parentOpenInfo: null,
	
		//_openInfo: null,
	
		_clear: function(){
			var openned = [];
			this._openInfo = {
				'': {
					id: '',
					parentId: null,
					index: -1,
					count: 0,
					openned: openned
				}
			};
			this._parentOpenInfo = {
				'': openned
			};
		},

		_initExpandLevel: function(){
			var cols = this.grid._columns;
			if(!array.some(cols, function(col){
				return col.expandLevel;
			})){
				if(this.arg('nested')){
					array.forEach(cols, function(col, i){
						col.expandLevel = i + 1;
					});
				}else{
					cols[0].expandLevel = 1;
				}
			}
		},

		_createCellWrapper: function(wrappers, rowId, colId){
			var t = this,
				col = t.grid._columnsById[colId];
			if(col.expandLevel){
				var isNested = t.arg('nested'),
					level = t.model.treePath(rowId).length,
					expandLevel = t.arg('expandLevel');
				if((!isNested || col.expandLevel == level) && 
						(!(expandLevel > 0) || level <= expandLevel + 1)){
					var hasChildren = t.model.hasChildren(rowId),
						isOpen = t.isExpanded(rowId),
						pad = 0,
						singlePad = t.arg('expandoPadding'),
						ltr = t.grid.isLeftToRight();
					if(!isNested){
						pad = (level - 1) * singlePad;
					}
					if(level == expandLevel + 1){
						//This is one level beyond the last level, there should not be expando
						if(isNested){
							//If nested, no indent needed
							return;
						}
						//If not nested, this level still needs indent
						hasChildren = false;
					}
					wrappers.push({
						priority: 0,
						wrap: function(cellData){
							return ["<div class='gridxTreeExpandoCell ",
								isOpen ? "gridxTreeExpandoCellOpen" : "",
								"' style='padding-", ltr ? 'left' : 'right', ": ", pad + singlePad, "px;'>",
								"<span class='gridxTreeExpandoIcon ",
								hasChildren ? '' : 'gridxTreeExpandoIconNoChildren',
								"' ",
								"style='margin-", ltr ? 'left' : 'right', ": ", pad, "px;'>",
								"<span class='gridxTreeExpandoInner'>",
								isOpen ? "-" : "+",
								"</span></span><span class='gridxTreeExpandoContent'>",
								cellData,
								"</span></span>"
							].join('');
						}
					});
				}
			}
		},
	
		_onCellClick: function(e){
			if(isExpando(e.cellNode)){
				var t = this,
					pos = domGeometry.position(query('.gridxTreeExpandoIcon', e.cellNode)[0]);
				if(e.clientX >= pos.x && e.clientX <= pos.x + pos.w && e.clientY >= pos.y && e.clientY <= pos.y + pos.h){
					if(t.isExpanded(e.rowId)){
						t.collapse(e.rowId);
					}else{
						t.expand(e.rowId);
					}
				}
			}
		},
	
		_updateBody: function(id){
			var t = this,
				body = t.grid.body;
			if(body){
				body.updateRootRange(body.rootStart, body.rootCount);
				var rowNode = body.getRowNode({rowId: id}), n, expando,
					isOpen = t.isExpanded(id);
				if(rowNode){
					n = query('.gridxTreeExpandoCell', rowNode)[0];
					if(n){
						expando = query('.gridxTreeExpandoIcon', n)[0];
						expando.firstChild.innerHTML = 'o';
						domClass.add(n, 'gridxTreeExpandoLoading');
					}
				}
				var visualIndex = id ? t.getVisualIndexByRowInfo(t.model.treePath(id).pop(), t.model.idToIndex(id), body.rootStart) : -1;
				return body.refresh(visualIndex + 1).then(function(){
					if(n){
						rowNode.setAttribute('aria-expanded', isOpen ? 'true' : 'false');
						expando.firstChild.innerHTML = isOpen ? '-' : '+';
						domClass.remove(n, 'gridxTreeExpandoLoading');
						domClass.toggle(n, 'gridxTreeExpandoCellOpen', isOpen);
					}
				});
			}
			return null;
		},

		_getAbsoluteVisualIndex: function(parentId, rowIndex){
			var info = this._openInfo[parentId || ''];
			if(info){
				var preCount = 0,
					openInfo = this._openInfo,
					func = function(info){
						preCount += rowIndex;
						var child, i;
						for(i = 0; i < info.openned.length; ++i){
							child = openInfo[info.openned[i]];
							if(child.index < rowIndex){
								preCount += child.count;
							}else{
								break;
							}
						}
						rowIndex = info.index;
						if(info.id){
							preCount++;
						}
						return openInfo[info.parentId];
					};
				while(info){
					info = func(info);
				}
				return preCount;
			}
			return -1;
		},
	
		_logicExpand: function(id){
			var t = this,
				m = t.model,
				treePath = m.treePath(id),
				level = treePath.length,
				expandLevel = t.arg('expandLevel');
			if(m.hasChildren(id) && (!(expandLevel > 0) || level <= expandLevel)){
				var parentId = treePath.pop(),
					openInfo = t._openInfo,
					poi = t._parentOpenInfo,
					parentOpenInfo = poi[parentId] = poi[parentId] || [];
				poi[id] = poi[id] || [];
				if(!openInfo[id]){
					var index = m.idToIndex(id),
						childCount = m.size(id),
						i = util.biSearch(parentOpenInfo, function(childId){
							return openInfo[childId].index - index;
						});
					if(parentOpenInfo[i] !== id){
						parentOpenInfo.splice(i, 0, id);
					}
					for(i = poi[id].length - 1; i >= 0; --i){
						childCount += openInfo[poi[id][i]].count;
					}
					openInfo[id] = {
						id: id,
						parentId: parentId,
						index: index,
						count: childCount,
						openned: poi[id]
					};
					var info = openInfo[parentId];
					while(info){
						info.count += childCount;
						info = openInfo[info.parentId];
					}
				}
			}
			//console.log('after expand:', id, dojo.clone(this._openInfo), dojo.clone(this._parentOpenInfo));
		},
	
		_logicCollapse: function(id){
			var t = this,
				info = t._openInfo[id];
			if(info){
				var openInfo = t._openInfo,
					parentId = t.model.treePath(id).pop(),
					parentOpenInfo = t._parentOpenInfo[parentId],
					i = util.biSearch(parentOpenInfo, function(childId){
						return openInfo[childId].index - info.index; 
					}),
					childCount = info.count;
				parentOpenInfo.splice(i, 1);
				info = openInfo[parentId];
				while(info){
					info.count -= childCount;
					info = openInfo[info.parentId];
				}
				delete openInfo[id];
			}
			//console.log('after collapse:', id,  dojo.clone(this._openInfo), dojo.clone(this._parentOpenInfo));
		},
	
		_getChild: function(visualIndex, info){
			var item = this._openInfo[info.parentId],
				i, len, preCount = info.preCount + item.index + 1,
				commonMixin = {
					found: true,
					visualIndex: visualIndex,
					count: 1
				};
	
			for(i = 0, len = item.openned.length; i < len; ++i){
				var childId = item.openned[i],
					child = this._openInfo[childId],
					vidx = child.index + preCount;
				if(vidx === visualIndex){
					return lang.mixin({
						parentId: item.id,
						start: child.index
					}, commonMixin);
				}else if(vidx > visualIndex){
					break;
				}else if(vidx + child.count >= visualIndex){
					return {
						parentId: childId, 
						preCount: preCount 
					};
				}
				preCount += child.count;
			}
			return lang.mixin({
				parentId: item.id,
				start: visualIndex - preCount
			}, commonMixin);
		},

		_onAfterRow: function(row){
			var hasChildren = this.model.hasChildren(row.id);
			if(hasChildren){
				row.node().setAttribute('aria-expanded', this.isExpanded(row.id));
			}
		},

		//Focus------------------------------------------------------------------
		_initFocus: function(){
			this.connect(this.grid, 'onCellKeyPress', '_onKey'); 
		},

		_onKey: function(e){
			var t = this;
			if(e.keyCode == keys.ESCAPE){
				var m = t.model,
					treePath = m.treePath(e.rowId),
					parentId = treePath.pop(),
					parentLevel = treePath.length,
					grid = t.grid;
				if(parentId){
					var i, col, visualIndex;
					for(i = grid._columns.length - 1; i >= 0; --i){
						col = grid._columns[i];
						if(col.expandLevel && (!t.arg('nested') || col.expandLevel == parentLevel)){
							break;
						}
					}
					m.when({id: parentId}, function(){
						visualIndex = grid.body.getVisualIndex({
							parentId: treePath.pop(), 
							rowIndex: m.idToIndex(parentId)
						}).visualIndex;
					}).then(function(){
						grid.vScroller.scrollToRow(visualIndex).then(function(){
							grid.body._focusCell(null, visualIndex, col.index);
						});
					});
				}
			}else if(e.ctrlKey && isExpando(e.cellNode)){
				var ltr = t.grid.isLeftToRight();
				if(e.keyCode == (ltr ? keys.LEFT_ARROW : keys.RIGHT_ARROW) && t._openInfo[e.rowId]){
					t.collapse(e.rowId);
				}else if(e.keyCode == (ltr ? keys.RIGHT_ARROW : keys.LEFT_ARROW) && !t._openInfo[e.rowId]){
					t.expand(e.rowId);
				}
			}
		}
	});
});

},
'dojoe/treetable/filter/_StoreLayer':function(){
// wrapped by build app
define(["dijit","dojo","dojox"], function(dijit,dojo,dojox){
dojo.provide("dojoe.treetable.filter._StoreLayer");
// summary:
//		The dojo.data.api.Read API is powerful, but it's difficult to give the store some special commands before 
//		fetch, so that the store content can be temporarily modified or transformed, and acts as another store. The 
//		parameter *query* or *queryOptions* in keywordArgs for *fetch* is not enough because:
//		1.	users do not have the opportunity to response to the store actions when these options or queries are applied,
//			especially when the real store is at server side.
//		2.	the store implementation must be changed to support any new options in 'query' or 'queryOptions', so it'll be
//			difficult if this implementation is not able to or very hard to be changed, or some new options are required to
//			be valid for all stores. 
//		This *StoreLayer* framework is dedicated to provide a uniform way for configuring an existing store, so that
//		it can be easily extended to have special behaviors or act like a totally different store.
//		The major approach is to wrap the *fetch* function of store, layer by layer. Every layer treats the incoming 
//		store.fetch as a 'black box', thus maintaining the independence between layers.
//		*fetch* is the most important data retriever in the Read API, almost all other functions are used for a single
//		item, and require that this item is already retrieved (by and only by *fetch*). So once we've controlled this 
//		*fetch* function, we've controlled almost the whole store. This fact simplifies our implementation of StoreLayer.
// example:
//		//ns is for namespace, i.e.:dojox.grid.enhanced.plugins
//		ns.wrap(ns.wrap(ns.wrap(store, new ns.FilterLayer()), new ns.UniqueLayer()), new ns.TransformLayer());
//
//		//every layer has a name, it should be given in the document of this layer.
//		//if you don't know it's name, you can get it by: ns.SomeLayer.prototype.name();
//		store.layer("filter").filterDef(...);
//		store.layer("unique").setUniqueColumns(...);
//		store.layer("transform").setScheme(...);
//
//		//now use the store as usual...
//
//		store.unwrap("transform"); //remove the transform layer but retain the other two.
//
//		//now use the store as usual...
//
//		store.unwrap(); //remove all the layers, get the original store back.
(function(){
	var ns = dojoe.treetable.filter,
	
	getPrevTags = function(tags){
		var tagList = ["reorder", "sizeChange", "normal", "presentation"];
		var idx = tagList.length;
		for(var i = tags.length - 1; i >= 0; --i){
			var p = dojo.indexOf(tagList, tags[i]);
			if(p >= 0 && p <= idx){
				idx = p;
			}
		}
		if(idx < tagList.length - 1){
			return tagList.slice(0, idx + 1);
		}else{
			return tagList;
		}
	},
	
	unwrap = function(/* string? */layerName){
		// summary:
		//		Unwrap the layers of the store
		// tags:
		//		public
		// returns:
		//		The unwrapped store, for nested use only.
		var i, layers = this._layers, len = layers.length;
		if(layerName){
			for(i = len-1; i >= 0; --i){
				if(layers[i].name() == layerName){
					layers[i]._unwrap(layers[i + 1]);
					break;
				}
			}
			layers.splice(i, 1);
		}else{
			for(i = len - 1; i >= 0; --i){
				layers[i]._unwrap();
			}
		}
		if(!layers.length){
			delete this._layers;
			delete this.layer;
			delete this.unwrap;
			delete this.forEachLayer;
		}
		//console.log("layers:",this._layers);
		return this;	//Read-store
	},
	
	getLayer = function(layerName){
		// summary:
		//		Get a layer of the store, so we can configure that layer.
		// tags:
		//		public (scope is store)
		// layerName: string
		//		the name of the layer
		// returns:
		//		the store layer object
		var i, layers = this._layers;
		if(typeof layerName == "undefined"){
			return layers.length;	//Integer
		}
		if(typeof layerName == "number"){
			return layers[layerName];	//_StoreLayer
		}
		for(i = layers.length - 1; i >= 0; --i){
			if(layers[i].name() == layerName){
				return layers[i];	//_StoreLayer
			}
		}
		return null;	//_StoreLayer
	},
	
	forEachLayer = function(callback, isInnerToOuter){
		// summary:
		//		Visit the layers one by one. From the outer most to inner most by default.
		// callback: Function
		//		The function to callback.
		//		If return false, break the loop.
		// isInnerToOuter: Boolean
		//		Whether visit from the inner most layer to the outer most layer.
		var len = this._layers.length, start, end, dir;
		if(isInnerToOuter){
			start = 0;
			end = len;
			dir = 1;
		}else{
			start = len - 1;
			end = -1;
			dir = -1;
		}
		for(var i = start; i != end; i += dir){
			if(callback(this._layers[i], i) === false){
				return i;
			}
		}
		return end;
	};
	ns.wrap = function(store, funcName, layer, layerFuncName){
		// summary:
		//		Wrap the store with the given layer.
		// tags:
		//		public
		// store: Read-store
		//		The store to be wrapped.
		// layer: _StoreLayer
		//		The layer to be used
		// returns
		//		The wrapped store, for nested use only.
		if(!store._layers){
			store._layers = [];
			store.layer = dojo.hitch(store, getLayer);
			store.unwrap = dojo.hitch(store, unwrap);
			store.forEachLayer = dojo.hitch(store, forEachLayer);
		}
		var prevTags = getPrevTags(layer.tags);
		if(!dojo.some(store._layers, function(lyr, i){
			if(dojo.some(lyr.tags, function(tag){
				return dojo.indexOf(prevTags, tag) >= 0;
			})){
				return false;
			}else{
				store._layers.splice(i, 0, layer);
				layer._wrap(store, funcName, layerFuncName, lyr);
				return true;
			}
		})){
			store._layers.push(layer);
			layer._wrap(store, funcName, layerFuncName);
		}
		//console.log("wrapped layers:", dojo.map(store._layers, function(lyr){return lyr.name();}));
		return store;	//Read-store
	};

	dojo.declare("dojoe.treetable.filter._StoreLayer",null,{
		// summary:
		//		The most abstract class of store layers, provides basic utilities and some interfaces.
		// tags:
		//		abstract
/*=====
		// _store: [protected] Read-store
		//		The wrapped store.
		_store: null,
		
		// _originFetch: [protected] function
		//		The original fetch function of the store.
		_originFetch: null,
		
		// __enabled: [private] Boolean
		//		To control whether this layer is valid.
		__enabled: true,
=====*/
		tags: ["normal"],
		
		layerFuncName: "_fetch",
		
		constructor: function(){
			this._store = null;
			this._originFetch = null;
			this.__enabled = true;
		},
		initialize: function(store){
			// summary:
			//		
		},
		uninitialize: function(store){
			// summary:
			//		
		},
		invalidate: function(){
			
		},
		_wrap: function(store, funcName, layerFuncName, nextLayer){
			// summary:
			//		Do the actual wrapping (or 'hacking' if you like) to the store. 
			// tags:
			//		internal
			// store: Read-store
			//		The store to be wrapped.
			this._store = store;
			this._funcName = funcName;
			var fetchFunc = dojo.hitch(this, function(){
				return (this.enabled() ? this[layerFuncName || this.layerFuncName] : this.originFetch).apply(this, arguments);
			});
			if(nextLayer){
				this._originFetch = nextLayer._originFetch;
				nextLayer._originFetch = fetchFunc;
			}else{
				this._originFetch = store[funcName] || function(){};
				store[funcName] = fetchFunc;
			}
			this.initialize(store);
		},
		_unwrap: function(nextLayer){
			// summary:
			//		Do the actual unwrapping to the store.
			// tags:
			//		internal
			// store: Read-store
			//		The store to be unwrapped.
			this.uninitialize(this._store);
			if(nextLayer){
				nextLayer._originFetch = this._originFetch;
			}else{
				this._store[this._funcName] = this._originFetch;
			}
			this._originFetch = null;
			this._store = null;
		},
		enabled: function(/* bool? */toEnable){
			// summary:
			//		The get/set function of the enabled status of this layer
			// tags:
			//		public
			// toEnable: Boolean?
			//		If given, is a setter, otherwise, it's getter.
			if(typeof toEnable != "undefined"){
				this.__enabled = !!toEnable;
			}
			return this.__enabled;	//Boolean
		},
		name: function(){
			// summary:
			//		Get the name of this store layer.
			//		The default name retrieved from class name, which should have a pattern of "{name}Layer".
			//		If this pattern does not exist, the whole class name will be this layer's name.
			//		It's better to override this method if your class name is too complicated.
			// tags:
			//		public extension
			// returns:
			//		The name of this layer.
			if(!this.__name){
				var m = this.declaredClass.match(/(?:\.([^\.]+)Layer$)|(?:\.([^\.]+)$)/i);
				this.__name = m ? (m[1] || m[2]).toLowerCase() : this.declaredClass;
			}
			return this.__name;
		},
		originFetch: function(){
			return (dojo.hitch(this._store, this._originFetch)).apply(this, arguments);
		}
	});
	dojo.declare("dojoe.treetable.filter._ServerSideLayer",ns._StoreLayer,{
		// summary:
		//		The most abstract class for all server side store layers.
		// tags:
		//		abstract
/*=====
		// _url: [protected] string
		//		The url of the server
		_url: "",
		// __cmds [private] object
		//		The command object to be sent to server.
		__cmds: {},
=====*/
		constructor: function(args){
			args = args || {};
			this._url = args.url || "";
			this._isStateful = !!args.isStateful;
			this._onUserCommandLoad = args.onCommandLoad || function(){};
			this.__cmds = {cmdlayer:this.name(), enable:true};
			
			//Only for stateful server, sending commands before fetch makes sense.
			this.useCommands(this._isStateful);
		},
		enabled: function(/* bool? */toEnable){
			// summary:
			//		Overrided from _StoreLayer.enabled
			var res = this.inherited(arguments);
			this.__cmds.enable = this.__enabled;
			return res;
		},
		useCommands: function(/* bool? */toUse){
			// summary:
			//		If you only want to modify the user request, instead of sending a separate command
			//		to server before fetch, just call:
			//			this.useCommand(false);
			// tags:
			//		public
			// toUse: Boolean?
			//		If provided, it's a setter, otherwise, it's a getter
			if(typeof toUse != "undefined"){
				this.__cmds.cmdlayer = (toUse && this._isStateful) ? this.name() : null;
			}
			return !!(this.__cmds.cmdlayer);	//Boolean
		},
		_fetch: function(/* keywordArgs */userRequest){
			// summary:
			//		Implementation of _StoreLayer._fetch
			if(this.__cmds.cmdlayer){
				//We're gonna send command to server before fetch.
				dojo.xhrPost({
					url: this._url || this._store.url,
					content: this.__cmds,
					load: dojo.hitch(this, function(responce){
						this.onCommandLoad(responce, userRequest);
						this.originFetch(userRequest);
					}),
					error: dojo.hitch(this, this.onCommandError)
				});		
			}else{
				//The user only wants to modify the request object.
				this.onCommandLoad("", userRequest);
				this.originFetch(userRequest);
			}
			return userRequest;	//dojo.data.api.Request
		},
		command: function(/* string */cmdName,/* (string|number|bool|...)? */cmdContent){
			// summary:
			//		get/set a command (a name-value pair)
			// tags:
			//		public
			// cmdName: string
			//		The name of the command
			// cmdContent: anything
			//		The content of the command
			// returns:
			//		The content of the command if cmdContent is undefined
			var cmds = this.__cmds;
			if(cmdContent === null){
				delete cmds[cmdName];
			}else if(typeof cmdContent !== "undefined"){
				cmds[cmdName] = cmdContent;
			}
			return cmds[cmdName];	//anything
		},
		onCommandLoad: function(/* string */response, /* keywordArgs */userRequest){
			// summary:
			//		When the server gives back *response* for the commands, you can do something here.
			// tags:
			//		callback extension
			// response: string
			//		server response
			// userRequest: [in|out] dojo.data.api.Request
			//		The request object for *fetch*. You can modify this object according to the *response*
			//		so as to change the behavior of *fetch*
			this._onUserCommandLoad(this.__cmds, userRequest, response);
		},
		onCommandError: function(error){
			// summary:
			//		handle errors when sending commands.
			// tags:
			//		callback extension
			// error: Error
			console.log(error);
			throw error;
		}
	});
})();

});

},
'dijit/form/_DateTimeTextBox':function(){
define([
	"dojo/date", // date date.compare
	"dojo/date/locale", // locale.regexp
	"dojo/date/stamp", // stamp.fromISOString stamp.toISOString
	"dojo/_base/declare", // declare
	"dojo/_base/lang", // lang.getObject
	"./RangeBoundTextBox",
	"../_HasDropDown",
	"dojo/text!./templates/DropDownBox.html"
], function(date, locale, stamp, declare, lang, RangeBoundTextBox, _HasDropDown, template){

/*=====
	var _HasDropDown = dijit._HasDropDown;
	var RangeBoundTextBox = dijit.form.RangeBoundTextBox;
=====*/

	// module:
	//		dijit/form/_DateTimeTextBox
	// summary:
	//		Base class for validating, serializable, range-bound date or time text box.


	new Date("X"); // workaround for #11279, new Date("") == NaN

	/*=====
	declare(
		"dijit.form._DateTimeTextBox.__Constraints",
		[RangeBoundTextBox.__Constraints, locale.__FormatOptions], {
		// summary:
		//		Specifies both the rules on valid/invalid values (first/last date/time allowed),
		//		and also formatting options for how the date/time is displayed.
		// example:
		//		To restrict to dates within 2004, displayed in a long format like "December 25, 2005":
		//	|		{min:'2004-01-01',max:'2004-12-31', formatLength:'long'}
	});
	=====*/

	var _DateTimeTextBox = declare("dijit.form._DateTimeTextBox", [RangeBoundTextBox, _HasDropDown], {
		// summary:
		//		Base class for validating, serializable, range-bound date or time text box.

		templateString: template,

		// hasDownArrow: [const] Boolean
		//		Set this textbox to display a down arrow button, to open the drop down list.
		hasDownArrow: true,

		// openOnClick: [const] Boolean
		//		Set to true to open drop down upon clicking anywhere on the textbox.
		openOnClick: true,

		/*=====
		// constraints: dijit.form._DateTimeTextBox.__Constraints
		//		Despite the name, this parameter specifies both constraints on the input
		//		(including starting/ending dates/times allowed) as well as
		//		formatting options like whether the date is displayed in long (ex: December 25, 2005)
		//		or short (ex: 12/25/2005) format.  See `dijit.form._DateTimeTextBox.__Constraints` for details.
		constraints: {},
		======*/

		// Override ValidationTextBox.regExpGen().... we use a reg-ex generating function rather
		// than a straight regexp to deal with locale  (plus formatting options too?)
		regExpGen: locale.regexp,

		// datePackage: String
		//		JavaScript namespace to find calendar routines.	 Uses Gregorian calendar routines
		//		at dojo.date, by default.
		datePackage: date,

		postMixInProperties: function(){
			this.inherited(arguments);
			this._set("type", "text"); // in case type="date"|"time" was specified which messes up parse/format
		},

		// Override _FormWidget.compare() to work for dates/times
		compare: function(/*Date*/ val1, /*Date*/ val2){
			var isInvalid1 = this._isInvalidDate(val1);
			var isInvalid2 = this._isInvalidDate(val2);
			return isInvalid1 ? (isInvalid2 ? 0 : -1) : (isInvalid2 ? 1 : date.compare(val1, val2, this._selector));
		},

		// flag to _HasDropDown to make drop down Calendar width == <input> width
		forceWidth: true,

		format: function(/*Date*/ value, /*dojo.date.locale.__FormatOptions*/ constraints){
			// summary:
			//		Formats the value as a Date, according to specified locale (second argument)
			// tags:
			//		protected
			if(!value){ return ''; }
			return this.dateLocaleModule.format(value, constraints);
		},

		"parse": function(/*String*/ value, /*dojo.date.locale.__FormatOptions*/ constraints){
			// summary:
			//		Parses as string as a Date, according to constraints
			// tags:
			//		protected

			return this.dateLocaleModule.parse(value, constraints) || (this._isEmpty(value) ? null : undefined);	 // Date
		},

		// Overrides ValidationTextBox.serialize() to serialize a date in canonical ISO format.
		serialize: function(/*anything*/ val, /*Object?*/ options){
			if(val.toGregorian){
				val = val.toGregorian();
			}
			return stamp.toISOString(val, options);
		},

		// dropDownDefaultValue: Date
		//		The default value to focus in the popupClass widget when the textbox value is empty.
		dropDownDefaultValue : new Date(),

		// value: Date
		//		The value of this widget as a JavaScript Date object.  Use get("value") / set("value", val) to manipulate.
		//		When passed to the parser in markup, must be specified according to `dojo.date.stamp.fromISOString`
		value: new Date(""),	// value.toString()="NaN"

		_blankValue: null,	// used by filter() when the textbox is blank

		// popupClass: [protected extension] String
		//		Name of the popup widget class used to select a date/time.
		//		Subclasses should specify this.
		popupClass: "", // default is no popup = text only


		// _selector: [protected extension] String
		//		Specifies constraints.selector passed to dojo.date functions, should be either
		//		"date" or "time".
		//		Subclass must specify this.
		_selector: "",

		constructor: function(/*Object*/ args){
			this.datePackage = args.datePackage || this.datePackage;
			this.dateFuncObj = typeof this.datePackage == "string" ?
				lang.getObject(this.datePackage, false) :// "string" part for back-compat, remove for 2.0
				this.datePackage;
			this.dateClassObj = this.dateFuncObj.Date || Date;
			this.dateLocaleModule = lang.getObject("locale", false, this.dateFuncObj);
			this.regExpGen = this.dateLocaleModule.regexp;
			this._invalidDate = this.constructor.prototype.value.toString();
		},

		buildRendering: function(){
			this.inherited(arguments);

			if(!this.hasDownArrow){
				this._buttonNode.style.display = "none";
			}

			// If openOnClick is true, we basically just want to treat the whole widget as the
			// button.  We need to do that also if the actual drop down button will be hidden,
			// so that there's a mouse method for opening the drop down.
			if(this.openOnClick || !this.hasDownArrow){
				this._buttonNode = this.domNode;
				this.baseClass += " dijitComboBoxOpenOnClick";
			}
		},

		_setConstraintsAttr: function(/*Object*/ constraints){
			constraints.selector = this._selector;
			constraints.fullYear = true; // see #5465 - always format with 4-digit years
			var fromISO = stamp.fromISOString;
			if(typeof constraints.min == "string"){ constraints.min = fromISO(constraints.min); }
 			if(typeof constraints.max == "string"){ constraints.max = fromISO(constraints.max); }
			this.inherited(arguments);
		},

		_isInvalidDate: function(/*Date*/ value){
			// summary:
			//		Runs various tests on the value, checking for invalid conditions
			// tags:
			//		private
			return !value || isNaN(value) || typeof value != "object" || value.toString() == this._invalidDate;
		},

		_setValueAttr: function(/*Date|String*/ value, /*Boolean?*/ priorityChange, /*String?*/ formattedValue){
			// summary:
			//		Sets the date on this textbox. Note: value can be a JavaScript Date literal or a string to be parsed.
			if(value !== undefined){
				if(typeof value == "string"){
					value = stamp.fromISOString(value);
				}
				if(this._isInvalidDate(value)){
					value = null;
				}
				if(value instanceof Date && !(this.dateClassObj instanceof Date)){
					value = new this.dateClassObj(value);
				}
			}
			this.inherited(arguments);
			if(this.value instanceof Date){
				this.filterString = "";
			}
			if(this.dropDown){
				this.dropDown.set('value', value, false);
			}
		},

		_set: function(attr, value){
			// Avoid spurious watch() notifications when value is changed to new Date object w/the same value
			if(attr == "value" && this.value instanceof Date && this.compare(value, this.value) == 0){
				return;
			}
			this.inherited(arguments);
		},

		_setDropDownDefaultValueAttr: function(/*Date*/ val){
			if(this._isInvalidDate(val)){
				// convert null setting into today's date, since there needs to be *some* default at all times.
				 val = new this.dateClassObj();
			}
			this.dropDownDefaultValue = val;
		},

		openDropDown: function(/*Function*/ callback){
			// rebuild drop down every time, so that constraints get copied (#6002)
			if(this.dropDown){
				this.dropDown.destroy();
			}
			var PopupProto = lang.isString(this.popupClass) ? lang.getObject(this.popupClass, false) : this.popupClass,
				textBox = this,
				value = this.get("value");
			this.dropDown = new PopupProto({
				onChange: function(value){
					// this will cause InlineEditBox and other handlers to do stuff so make sure it's last
					textBox.set('value', value, true);
				},
				id: this.id + "_popup",
				dir: textBox.dir,
				lang: textBox.lang,
				value: value,
				currentFocus: !this._isInvalidDate(value) ? value : this.dropDownDefaultValue,
					constraints: textBox.constraints,
				filterString: textBox.filterString, // for TimeTextBox, to filter times shown

					datePackage: textBox.datePackage,

					isDisabledDate: function(/*Date*/ date){
						// summary:
						// 	disables dates outside of the min/max of the _DateTimeTextBox
						return !textBox.rangeCheck(date, textBox.constraints);
					}
				});

			this.inherited(arguments);
		},

		_getDisplayedValueAttr: function(){
			return this.textbox.value;
		},

		_setDisplayedValueAttr: function(/*String*/ value, /*Boolean?*/ priorityChange){
			this._setValueAttr(this.parse(value, this.constraints), priorityChange, value);
		}
	});

	return _DateTimeTextBox;
});

},
'url:ibm/usmi/widgets/wizard/templates/DirectorWizard.html':"<div class=\"tiv-wiz\" role=\"application\">\r\n<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n\t<!-- we were using dijit.layout.BorderContainer -> now changed to DirectorBorderContainer -->\r\n<div class=\"tiv-wiz\" role=\"application\">\r\n\t<div dojoType=\"dijit.layout.BorderContainer\" design=\"headline\" gutters=\"false\" class=\"tiv-wiz-outer\" dojoAttachPoint=\"_outerContainer\">\r\n\t\t<div dojoType=\"dijit.layout.BorderContainer\" gutters=\"false\" liveSplitters=\"false\" design=\"sidebar\" region=\"center\" style=\"width:100%;height:100%;\" dojoAttachPoint=\"_borderArea\">\r\n\t\t\t<div dojoType=\"dijit.layout.ContentPane\" region=\"leading\" splitter=\"false\" class=\"tiv-wiz-toc\" dojoAttachPoint=\"_tocArea\">\r\n\t\t\t\t<ol dojoAttachPoint=\"_toc\"></ol>\r\n\t\t\t</div>\r\n\t\t\t<div dojoType=\"dijit.layout.StackContainer\" region=\"center\" splitter=\"false\" dojoAttachPoint=\"_pageContainer\" class=\"tiv-wiz-work-area\">\r\n\t\t\t<form id=\"${id}_form\" dojoAttachPoint=\"containerNode\" action=\"${action}\" method=\"${method}\"></form>              \r\n\t\t\t</div>\r\n\t\t</div>\r\n\t\t<div dojoType=\"dijit.layout.ContentPane\" region=\"bottom\" splitter=\"false\" class=\"tiv-wiz-footer\" dojoAttachPoint=\"_buttonArea\">\r\n\t\t\t<!-- We want the director wizard step buttons to be left-justified -->\r\n\t\t\t<div class=\"director-wiz-step-buttons\" style=\"display: inline;\" data-dojo-attach-point=\"_buttons\">\r\n\t\t\t\t<button role=\"button\" type=\"button\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickBack\" data-dojo-attach-point=\"_backButton\">${backLabel}</button>\r\n\t\t\t\t<button role=\"button\" type=\"button\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickNext\" data-dojo-attach-point=\"_nextButton\">${nextLabel}</button>\r\n\t\t\t</div>\t\t\t\r\n\t\t\t<!--  we want the director wizard commit buttons to be right-justified -->\r\n\t\t\t<div class=\"director-wiz-commit-buttons\" style=\"display:inline;\" data-dojo-attach-point=\"_buttons\">\r\n\t\t\t\t<button role=\"button\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickFinish\" data-dojo-props='type:\"button\"' data-dojo-attach-point=\"_finishButton\">${finishLabel}</button>\r\n\t\t\t\t<button role=\"button\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickCancel\" data-dojo-props='type:\"button\"' data-dojo-attach-point=\"_cancelButton\">${cancelLabel}</button>\r\n\t\t\t</div> \r\n\t\t</div>\r\n\t</div>\r\n</div>\r\n",
'gridx/modules/extendedSelect/_RowCellBase':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/query",
	"./_Base",
	"../../core/model/extensions/Mark"
], function(declare, lang, query, _Base, Mark){

	return declare(_Base, {
		modelExtensions: [Mark],

		_getRowId: function(visualIndex){
			var node = query('[visualindex="' + visualIndex + '"]', this.grid.bodyNode)[0];
			return node && node.getAttribute('rowid');
		},

		_init: function(){
			var t = this, m = t.model;
			t.batchConnect(
				[t.grid.body, 'onMoveToCell', '_onMoveToCell'],
				[m, 'onMarkChange', '_onMark']
			);
		}
	});
});


},
'dojoe/wizard/TivSubstep':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!dojoe/wizard/Page"], function(dijit,dojo,dojox){
dojo.provide("dojoe.wizard.TivSubstep");

dojo.require("dojoe.wizard.Page");

/**
 * An extension that is a substep
 * For the wizard, it handles substeps.
 */
dojo.declare("dojoe.wizard.TivSubstep", [dojoe.wizard.Page], 
	{
		mainPage: null,  // main page for this substep
		
		constructor: function (args) {
		},

		isMainStep: function () {
			return (false);
		},
  
		isPartOfLoop: function () {
			if (this.mainPage.isPartOfLoop()) {
				return (true);
			} else {
				return (false);
			}
		},
  
		/**
		* Nested substeps are not supported
		*/
		hasSubsteps: function () {
			return (false);
		},
  
		getMainPage: function () {
			return (this.mainPage);
		},
  
		/** 
		* These items are created with some minor differences in how they are
		* displayed
		*/
		buildTOCRendring: function () {
			var listItemToAdd = this.listItem;
			// if the list entry hasn't been created yet
			if (this.listItem == null) {
				// create it
				this.inherited(arguments);
				// save main page for this substep
				this.mainPage = this.wizard.getPreviousMainPage(this);
				// if vertical toc
				if ('v' === this.wizard.layout) {
					listItemToAdd = this.listItem;
					// make it invisible
					dojo.style(this.listItem, "display", "none");
					// and indented
					dojo.addClass(this.listItem, "tiv-wiz-toc-substep");
				} else {
					// horizontal mode - add to substep list of main page
					listItemToAdd = null;
					if (this.mainPage != null) {
						if (this.mainPage.substepsList == null) {
							this.mainPage.substepsList = dojo.create("ol", {});
						}
						this.mainPage.substepsList.appendChild(this.listItem);
					}        
				}
			}
			return listItemToAdd;
		}
	});
});

},
'url:idx/oneui/templates/_MenuColumn.html':"<td class=\"dijitReset oneuiMenuColumn\" data-dojo-attach-point=\"columnNodes\">\r\n\t<table class=\"dijitReset\" cellspacing=\"0\" width=\"100%\" role=\"presentation\">\r\n\t\t<tbody class=\"dijitReset\" data-dojo-attach-point=\"_containerNodes\">\r\n<!-- this must be kept in synch with column 0 included in Menu.html -->\r\n\t\t</tbody>\r\n\t</table>\r\n</td>",
'dijit/form/Select':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.set
	"dojo/dom-class", // domClass.add domClass.remove domClass.toggle
	"dojo/dom-construct", // domConstruct.create
	"dojo/dom-geometry", // domGeometry.setMarginBox
	"dojo/_base/event", // event.stop
	"dojo/i18n", // i18n.getLocalization
	"dojo/_base/lang", // lang.hitch
	"./_FormSelectWidget",
	"../_HasDropDown",
	"../Menu",
	"../MenuItem",
	"../MenuSeparator",
	"../Tooltip",
	"dojo/text!./templates/Select.html",
	"dojo/i18n!./nls/validate"
], function(array, declare, domAttr, domClass, domConstruct, domGeometry, event, i18n, lang,
			_FormSelectWidget, _HasDropDown, Menu, MenuItem, MenuSeparator, Tooltip, template){

/*=====
	var _FormSelectWidget = dijit.form._FormSelectWidget;
	var _HasDropDown = dijit._HasDropDown;
	var _FormSelectWidget = dijit._FormSelectWidget;
	var Menu = dijit.Menu;
	var MenuItem = dijit.MenuItem;
	var MenuSeparator = dijit.MenuSeparator;
	var Tooltip = dijit.Tooltip;
=====*/

// module:
//		dijit/form/Select
// summary:
//		This is a "styleable" select box - it is basically a DropDownButton which
//		can take a <select> as its input.


var _SelectMenu = declare("dijit.form._SelectMenu", Menu, {
	// summary:
	//		An internally-used menu for dropdown that allows us a vertical scrollbar
	buildRendering: function(){
		// summary:
		//		Stub in our own changes, so that our domNode is not a table
		//		otherwise, we won't respond correctly to heights/overflows
		this.inherited(arguments);
		var o = (this.menuTableNode = this.domNode);
		var n = (this.domNode = domConstruct.create("div", {style: {overflowX: "hidden", overflowY: "scroll"}}));
		if(o.parentNode){
			o.parentNode.replaceChild(n, o);
		}
		domClass.remove(o, "dijitMenuTable");
		n.className = o.className + " dijitSelectMenu";
		o.className = "dijitReset dijitMenuTable";
		o.setAttribute("role", "listbox");
		n.setAttribute("role", "presentation");
		n.appendChild(o);
	},

	postCreate: function(){
		// summary:
		//		stop mousemove from selecting text on IE to be consistent with other browsers

		this.inherited(arguments);

		this.connect(this.domNode, "onmousemove", event.stop);
	},

	resize: function(/*Object*/ mb){
		// summary:
		//		Overridden so that we are able to handle resizing our
		//		internal widget.  Note that this is not a "full" resize
		//		implementation - it only works correctly if you pass it a
		//		marginBox.
		//
		// mb: Object
		//		The margin box to set this dropdown to.
		if(mb){
			domGeometry.setMarginBox(this.domNode, mb);
			if("w" in mb){
				// We've explicitly set the wrapper <div>'s width, so set <table> width to match.
				// 100% is safer than a pixel value because there may be a scroll bar with
				// browser/OS specific width.
				this.menuTableNode.style.width = "100%";
			}
		}
	}
});

var Select = declare("dijit.form.Select", [_FormSelectWidget, _HasDropDown], {
	// summary:
	//		This is a "styleable" select box - it is basically a DropDownButton which
	//		can take a <select> as its input.

	baseClass: "dijitSelect",

	templateString: template,

	// required: Boolean
	//		Can be true or false, default is false.
	required: false,

	// state: [readonly] String
	//		"Incomplete" if this select is required but unset (i.e. blank value), "" otherwise
	state: "",

	// message: String
	//		Currently displayed error/prompt message
	message: "",

	//	tooltipPosition: String[]
	//		See description of dijit.Tooltip.defaultPosition for details on this parameter.
	tooltipPosition: [],

	// emptyLabel: string
	//		What to display in an "empty" dropdown
	emptyLabel: "&#160;",	// &nbsp;

	// _isLoaded: Boolean
	//		Whether or not we have been loaded
	_isLoaded: false,

	// _childrenLoaded: Boolean
	//		Whether or not our children have been loaded
	_childrenLoaded: false,

	_fillContent: function(){
		// summary:
		//		Set the value to be the first, or the selected index
		this.inherited(arguments);
		// set value from selected option
		if(this.options.length && !this.value && this.srcNodeRef){
			var si = this.srcNodeRef.selectedIndex || 0; // || 0 needed for when srcNodeRef is not a SELECT
			this.value = this.options[si >= 0 ? si : 0].value;
		}
		// Create the dropDown widget
		this.dropDown = new _SelectMenu({id: this.id + "_menu"});
		domClass.add(this.dropDown.domNode, this.baseClass + "Menu");
	},

	_getMenuItemForOption: function(/*dijit.form.__SelectOption*/ option){
		// summary:
		//		For the given option, return the menu item that should be
		//		used to display it.  This can be overridden as needed
		if(!option.value && !option.label){
			// We are a separator (no label set for it)
			return new MenuSeparator();
		}else{
			// Just a regular menu option
			var click = lang.hitch(this, "_setValueAttr", option);
			var item = new MenuItem({
				option: option,
				label: option.label || this.emptyLabel,
				onClick: click,
				disabled: option.disabled || false
			});
			item.focusNode.setAttribute("role", "listitem");
			return item;
		}
	},

	_addOptionItem: function(/*dijit.form.__SelectOption*/ option){
		// summary:
		//		For the given option, add an option to our dropdown.
		//		If the option doesn't have a value, then a separator is added
		//		in that place.
		if(this.dropDown){
			this.dropDown.addChild(this._getMenuItemForOption(option));
		}
	},

	_getChildren: function(){
		if(!this.dropDown){
			return [];
		}
		return this.dropDown.getChildren();
	},

	_loadChildren: function(/*Boolean*/ loadMenuItems){
		// summary:
		//		Resets the menu and the length attribute of the button - and
		//		ensures that the label is appropriately set.
		//	loadMenuItems: Boolean
		//		actually loads the child menu items - we only do this when we are
		//		populating for showing the dropdown.

		if(loadMenuItems === true){
			// this.inherited destroys this.dropDown's child widgets (MenuItems).
			// Avoid this.dropDown (Menu widget) having a pointer to a destroyed widget (which will cause
			// issues later in _setSelected). (see #10296)
			if(this.dropDown){
				delete this.dropDown.focusedChild;
			}
			if(this.options.length){
				this.inherited(arguments);
			}else{
				// Drop down menu is blank but add one blank entry just so something appears on the screen
				// to let users know that they are no choices (mimicing native select behavior)
				array.forEach(this._getChildren(), function(child){ child.destroyRecursive(); });
				var item = new MenuItem({label: "&#160;"});
				this.dropDown.addChild(item);
			}
		}else{
			this._updateSelection();
		}

		this._isLoaded = false;
		this._childrenLoaded = true;

		if(!this._loadingStore){
			// Don't call this if we are loading - since we will handle it later
			this._setValueAttr(this.value);
		}
	},

	_setValueAttr: function(value){
		this.inherited(arguments);
		domAttr.set(this.valueNode, "value", this.get("value"));
		this.validate(this.focused);	// to update this.state
	},

	_setDisabledAttr: function(/*Boolean*/ value){
		this.inherited(arguments);
		this.validate(this.focused);	// to update this.state
	},

	_setRequiredAttr: function(/*Boolean*/ value){
		this._set("required", value);
		this.focusNode.setAttribute("aria-required", value);
		this.validate(this.focused);	// to update this.state
	},

	_setDisplay: function(/*String*/ newDisplay){
		// summary:
		//		sets the display for the given value (or values)
		var lbl = newDisplay || this.emptyLabel;
		this.containerNode.innerHTML = '<span class="dijitReset dijitInline ' + this.baseClass + 'Label">' + lbl + '</span>';
		this.focusNode.setAttribute("aria-valuetext", lbl);
	},

	validate: function(/*Boolean*/ isFocused){
		// summary:
		//		Called by oninit, onblur, and onkeypress, and whenever required/disabled state changes
		// description:
		//		Show missing or invalid messages if appropriate, and highlight textbox field.
		//		Used when a select is initially set to no value and the user is required to
		//		set the value.

		var isValid = this.disabled || this.isValid(isFocused);
		this._set("state", isValid ? "" : "Incomplete");
		this.focusNode.setAttribute("aria-invalid", isValid ? "false" : "true");
		var message = isValid ? "" : this._missingMsg;
		if(message && this.focused && this._hasBeenBlurred){
			Tooltip.show(message, this.domNode, this.tooltipPosition, !this.isLeftToRight());
		}else{
			Tooltip.hide(this.domNode);
		}
		this._set("message", message);
		return isValid;
	},

	isValid: function(/*Boolean*/ /*===== isFocused =====*/){
		// summary:
		//		Whether or not this is a valid value.  The only way a Select
		//		can be invalid is when it's required but nothing is selected.
		return (!this.required || this.value === 0 || !(/^\s*$/.test(this.value || ""))); // handle value is null or undefined
	},

	reset: function(){
		// summary:
		//		Overridden so that the state will be cleared.
		this.inherited(arguments);
		Tooltip.hide(this.domNode);
		this.validate(this.focused);	// to update this.state
	},

	postMixInProperties: function(){
		// summary:
		//		set the missing message
		this.inherited(arguments);
		this._missingMsg = i18n.getLocalization("dijit.form", "validate",
									this.lang).missingMessage;
	},

	postCreate: function(){
		// summary:
		//		stop mousemove from selecting text on IE to be consistent with other browsers

		this.inherited(arguments);

		this.connect(this.domNode, "onmousemove", event.stop);
	},

	_setStyleAttr: function(/*String||Object*/ value){
		this.inherited(arguments);
		domClass.toggle(this.domNode, this.baseClass + "FixedWidth", !!this.domNode.style.width);
	},

	isLoaded: function(){
		return this._isLoaded;
	},

	loadDropDown: function(/*Function*/ loadCallback){
		// summary:
		//		populates the menu
		this._loadChildren(true);
		this._isLoaded = true;
		loadCallback();
	},

	closeDropDown: function(){
		// overriding _HasDropDown.closeDropDown()
		this.inherited(arguments);

		if(this.dropDown && this.dropDown.menuTableNode){
			// Erase possible width: 100% setting from _SelectMenu.resize().
			// Leaving it would interfere with the next openDropDown() call, which
			// queries the natural size of the drop down.
			this.dropDown.menuTableNode.style.width = "";
		}
	},

	uninitialize: function(preserveDom){
		if(this.dropDown && !this.dropDown._destroyed){
			this.dropDown.destroyRecursive(preserveDom);
			delete this.dropDown;
		}
		this.inherited(arguments);
	},

	_onFocus: function(){
		this.validate(true);	// show tooltip if second focus of required tooltip, but no selection
		this.inherited(arguments);
	},

	_onBlur: function(){
		Tooltip.hide(this.domNode);
		this.inherited(arguments);
	}
});

Select._Menu = _SelectMenu;	// for monkey patching

return Select;
});

},
'url:dijit/form/templates/Select.html':"<table class=\"dijit dijitReset dijitInline dijitLeft\"\n\tdata-dojo-attach-point=\"_buttonNode,tableNode,focusNode\" cellspacing='0' cellpadding='0'\n\trole=\"combobox\" aria-haspopup=\"true\"\n\t><tbody role=\"presentation\"><tr role=\"presentation\"\n\t\t><td class=\"dijitReset dijitStretch dijitButtonContents dijitButtonNode\" role=\"presentation\"\n\t\t\t><span class=\"dijitReset dijitInline dijitButtonText\"  data-dojo-attach-point=\"containerNode,_popupStateNode\"></span\n\t\t\t><input type=\"hidden\" ${!nameAttrSetting} data-dojo-attach-point=\"valueNode\" value=\"${value}\" aria-hidden=\"true\"\n\t\t/></td><td class=\"dijitReset dijitRight dijitButtonNode dijitArrowButton dijitDownArrowButton\"\n\t\t\t\tdata-dojo-attach-point=\"titleNode\" role=\"presentation\"\n\t\t\t><div class=\"dijitReset dijitArrowButtonInner\" role=\"presentation\"></div\n\t\t\t><div class=\"dijitReset dijitArrowButtonChar\" role=\"presentation\">&#9660;</div\n\t\t></td\n\t></tr></tbody\n></table>\n",
'dojo/data/ItemFileWriteStore':function(){
define(["../_base/lang", "../_base/declare", "../_base/array", "../_base/json", "../_base/window", 
	"./ItemFileReadStore", "../date/stamp"
], function(lang, declare, arrayUtil, jsonUtil, window, ItemFileReadStore, dateStamp) {
	// module:
	//		dojo/data/ItemFileWriteStore
	// summary:
	//		TODOC

/*===== var ItemFileReadStore = dojo.data.ItemFileReadStore; =====*/
return declare("dojo.data.ItemFileWriteStore", ItemFileReadStore, {
	constructor: function(/* object */ keywordParameters){
		//	keywordParameters: {typeMap: object)
		//		The structure of the typeMap object is as follows:
		//		{
		//			type0: function || object,
		//			type1: function || object,
		//			...
		//			typeN: function || object
		//		}
		//		Where if it is a function, it is assumed to be an object constructor that takes the
		//		value of _value as the initialization parameters.  It is serialized assuming object.toString()
		//		serialization.  If it is an object, then it is assumed
		//		to be an object of general form:
		//		{
		//			type: function, //constructor.
		//			deserialize:	function(value) //The function that parses the value and constructs the object defined by type appropriately.
		//			serialize:	function(object) //The function that converts the object back into the proper file format form.
		//		}

		// ItemFileWriteStore extends ItemFileReadStore to implement these additional dojo.data APIs
		this._features['dojo.data.api.Write'] = true;
		this._features['dojo.data.api.Notification'] = true;

		// For keeping track of changes so that we can implement isDirty and revert
		this._pending = {
			_newItems:{},
			_modifiedItems:{},
			_deletedItems:{}
		};

		if(!this._datatypeMap['Date'].serialize){
			this._datatypeMap['Date'].serialize = function(obj){
				return dateStamp.toISOString(obj, {zulu:true});
			};
		}
		//Disable only if explicitly set to false.
		if(keywordParameters && (keywordParameters.referenceIntegrity === false)){
			this.referenceIntegrity = false;
		}

		// this._saveInProgress is set to true, briefly, from when save() is first called to when it completes
		this._saveInProgress = false;
	},

	referenceIntegrity: true, //Flag that defaultly enabled reference integrity tracking.  This way it can also be disabled pogrammatially or declaratively.

	_assert: function(/* boolean */ condition){
		if(!condition){
			throw new Error("assertion failed in ItemFileWriteStore");
		}
	},

	_getIdentifierAttribute: function(){
		// this._assert((identifierAttribute === Number) || (dojo.isString(identifierAttribute)));
		return this.getFeatures()['dojo.data.api.Identity'];
	},


/* dojo.data.api.Write */

	newItem: function(/* Object? */ keywordArgs, /* Object? */ parentInfo){
		// summary: See dojo.data.api.Write.newItem()

		this._assert(!this._saveInProgress);

		if(!this._loadFinished){
			// We need to do this here so that we'll be able to find out what
			// identifierAttribute was specified in the data file.
			this._forceLoad();
		}

		if(typeof keywordArgs != "object" && typeof keywordArgs != "undefined"){
			throw new Error("newItem() was passed something other than an object");
		}
		var newIdentity = null;
		var identifierAttribute = this._getIdentifierAttribute();
		if(identifierAttribute === Number){
			newIdentity = this._arrayOfAllItems.length;
		}else{
			newIdentity = keywordArgs[identifierAttribute];
			if(typeof newIdentity === "undefined"){
				throw new Error("newItem() was not passed an identity for the new item");
			}
			if(lang.isArray(newIdentity)){
				throw new Error("newItem() was not passed an single-valued identity");
			}
		}

		// make sure this identity is not already in use by another item, if identifiers were
		// defined in the file.  Otherwise it would be the item count,
		// which should always be unique in this case.
		if(this._itemsByIdentity){
			this._assert(typeof this._itemsByIdentity[newIdentity] === "undefined");
		}
		this._assert(typeof this._pending._newItems[newIdentity] === "undefined");
		this._assert(typeof this._pending._deletedItems[newIdentity] === "undefined");

		var newItem = {};
		newItem[this._storeRefPropName] = this;
		newItem[this._itemNumPropName] = this._arrayOfAllItems.length;
		if(this._itemsByIdentity){
			this._itemsByIdentity[newIdentity] = newItem;
			//We have to set the identifier now, otherwise we can't look it
			//up at calls to setValueorValues in parentInfo handling.
			newItem[identifierAttribute] = [newIdentity];
		}
		this._arrayOfAllItems.push(newItem);

		//We need to construct some data for the onNew call too...
		var pInfo = null;

		// Now we need to check to see where we want to assign this thingm if any.
		if(parentInfo && parentInfo.parent && parentInfo.attribute){
			pInfo = {
				item: parentInfo.parent,
				attribute: parentInfo.attribute,
				oldValue: undefined
			};

			//See if it is multi-valued or not and handle appropriately
			//Generally, all attributes are multi-valued for this store
			//So, we only need to append if there are already values present.
			var values = this.getValues(parentInfo.parent, parentInfo.attribute);
			if(values && values.length > 0){
				var tempValues = values.slice(0, values.length);
				if(values.length === 1){
					pInfo.oldValue = values[0];
				}else{
					pInfo.oldValue = values.slice(0, values.length);
				}
				tempValues.push(newItem);
				this._setValueOrValues(parentInfo.parent, parentInfo.attribute, tempValues, false);
				pInfo.newValue = this.getValues(parentInfo.parent, parentInfo.attribute);
			}else{
				this._setValueOrValues(parentInfo.parent, parentInfo.attribute, newItem, false);
				pInfo.newValue = newItem;
			}
		}else{
			//Toplevel item, add to both top list as well as all list.
			newItem[this._rootItemPropName]=true;
			this._arrayOfTopLevelItems.push(newItem);
		}

		this._pending._newItems[newIdentity] = newItem;

		//Clone over the properties to the new item
		for(var key in keywordArgs){
			if(key === this._storeRefPropName || key === this._itemNumPropName){
				// Bummer, the user is trying to do something like
				// newItem({_S:"foo"}).  Unfortunately, our superclass,
				// ItemFileReadStore, is already using _S in each of our items
				// to hold private info.  To avoid a naming collision, we
				// need to move all our private info to some other property
				// of all the items/objects.  So, we need to iterate over all
				// the items and do something like:
				//    item.__S = item._S;
				//    item._S = undefined;
				// But first we have to make sure the new "__S" variable is
				// not in use, which means we have to iterate over all the
				// items checking for that.
				throw new Error("encountered bug in ItemFileWriteStore.newItem");
			}
			var value = keywordArgs[key];
			if(!lang.isArray(value)){
				value = [value];
			}
			newItem[key] = value;
			if(this.referenceIntegrity){
				for(var i = 0; i < value.length; i++){
					var val = value[i];
					if(this.isItem(val)){
						this._addReferenceToMap(val, newItem, key);
					}
				}
			}
		}
		this.onNew(newItem, pInfo); // dojo.data.api.Notification call
		return newItem; // item
	},

	_removeArrayElement: function(/* Array */ array, /* anything */ element){
		var index = arrayUtil.indexOf(array, element);
		if(index != -1){
			array.splice(index, 1);
			return true;
		}
		return false;
	},

	deleteItem: function(/* item */ item){
		// summary: See dojo.data.api.Write.deleteItem()
		this._assert(!this._saveInProgress);
		this._assertIsItem(item);

		// Remove this item from the _arrayOfAllItems, but leave a null value in place
		// of the item, so as not to change the length of the array, so that in newItem()
		// we can still safely do: newIdentity = this._arrayOfAllItems.length;
		var indexInArrayOfAllItems = item[this._itemNumPropName];
		var identity = this.getIdentity(item);

		//If we have reference integrity on, we need to do reference cleanup for the deleted item
		if(this.referenceIntegrity){
			//First scan all the attributes of this items for references and clean them up in the map
			//As this item is going away, no need to track its references anymore.

			//Get the attributes list before we generate the backup so it
			//doesn't pollute the attributes list.
			var attributes = this.getAttributes(item);

			//Backup the map, we'll have to restore it potentially, in a revert.
			if(item[this._reverseRefMap]){
				item["backup_" + this._reverseRefMap] = lang.clone(item[this._reverseRefMap]);
			}

			//TODO:  This causes a reversion problem.  This list won't be restored on revert since it is
			//attached to the 'value'. item, not ours.  Need to back tese up somehow too.
			//Maybe build a map of the backup of the entries and attach it to the deleted item to be restored
			//later.  Or just record them and call _addReferenceToMap on them in revert.
			arrayUtil.forEach(attributes, function(attribute){
				arrayUtil.forEach(this.getValues(item, attribute), function(value){
					if(this.isItem(value)){
						//We have to back up all the references we had to others so they can be restored on a revert.
						if(!item["backupRefs_" + this._reverseRefMap]){
							item["backupRefs_" + this._reverseRefMap] = [];
						}
						item["backupRefs_" + this._reverseRefMap].push({id: this.getIdentity(value), attr: attribute});
						this._removeReferenceFromMap(value, item, attribute);
					}
				}, this);
			}, this);

			//Next, see if we have references to this item, if we do, we have to clean them up too.
			var references = item[this._reverseRefMap];
			if(references){
				//Look through all the items noted as references to clean them up.
				for(var itemId in references){
					var containingItem = null;
					if(this._itemsByIdentity){
						containingItem = this._itemsByIdentity[itemId];
					}else{
						containingItem = this._arrayOfAllItems[itemId];
					}
					//We have a reference to a containing item, now we have to process the
					//attributes and clear all references to the item being deleted.
					if(containingItem){
						for(var attribute in references[itemId]){
							var oldValues = this.getValues(containingItem, attribute) || [];
							var newValues = arrayUtil.filter(oldValues, function(possibleItem){
								return !(this.isItem(possibleItem) && this.getIdentity(possibleItem) == identity);
							}, this);
							//Remove the note of the reference to the item and set the values on the modified attribute.
							this._removeReferenceFromMap(item, containingItem, attribute);
							if(newValues.length < oldValues.length){
								this._setValueOrValues(containingItem, attribute, newValues, true);
							}
						}
					}
				}
			}
		}

		this._arrayOfAllItems[indexInArrayOfAllItems] = null;

		item[this._storeRefPropName] = null;
		if(this._itemsByIdentity){
			delete this._itemsByIdentity[identity];
		}
		this._pending._deletedItems[identity] = item;

		//Remove from the toplevel items, if necessary...
		if(item[this._rootItemPropName]){
			this._removeArrayElement(this._arrayOfTopLevelItems, item);
		}
		this.onDelete(item); // dojo.data.api.Notification call
		return true;
	},

	setValue: function(/* item */ item, /* attribute-name-string */ attribute, /* almost anything */ value){
		// summary: See dojo.data.api.Write.set()
		return this._setValueOrValues(item, attribute, value, true); // boolean
	},

	setValues: function(/* item */ item, /* attribute-name-string */ attribute, /* array */ values){
		// summary: See dojo.data.api.Write.setValues()
		return this._setValueOrValues(item, attribute, values, true); // boolean
	},

	unsetAttribute: function(/* item */ item, /* attribute-name-string */ attribute){
		// summary: See dojo.data.api.Write.unsetAttribute()
		return this._setValueOrValues(item, attribute, [], true);
	},

	_setValueOrValues: function(/* item */ item, /* attribute-name-string */ attribute, /* anything */ newValueOrValues, /*boolean?*/ callOnSet){
		this._assert(!this._saveInProgress);

		// Check for valid arguments
		this._assertIsItem(item);
		this._assert(lang.isString(attribute));
		this._assert(typeof newValueOrValues !== "undefined");

		// Make sure the user isn't trying to change the item's identity
		var identifierAttribute = this._getIdentifierAttribute();
		if(attribute == identifierAttribute){
			throw new Error("ItemFileWriteStore does not have support for changing the value of an item's identifier.");
		}

		// To implement the Notification API, we need to make a note of what
		// the old attribute value was, so that we can pass that info when
		// we call the onSet method.
		var oldValueOrValues = this._getValueOrValues(item, attribute);

		var identity = this.getIdentity(item);
		if(!this._pending._modifiedItems[identity]){
			// Before we actually change the item, we make a copy of it to
			// record the original state, so that we'll be able to revert if
			// the revert method gets called.  If the item has already been
			// modified then there's no need to do this now, since we already
			// have a record of the original state.
			var copyOfItemState = {};
			for(var key in item){
				if((key === this._storeRefPropName) || (key === this._itemNumPropName) || (key === this._rootItemPropName)){
					copyOfItemState[key] = item[key];
				}else if(key === this._reverseRefMap){
					copyOfItemState[key] = lang.clone(item[key]);
				}else{
					copyOfItemState[key] = item[key].slice(0, item[key].length);
				}
			}
			// Now mark the item as dirty, and save the copy of the original state
			this._pending._modifiedItems[identity] = copyOfItemState;
		}

		// Okay, now we can actually change this attribute on the item
		var success = false;

		if(lang.isArray(newValueOrValues) && newValueOrValues.length === 0){

			// If we were passed an empty array as the value, that counts
			// as "unsetting" the attribute, so we need to remove this
			// attribute from the item.
			success = delete item[attribute];
			newValueOrValues = undefined; // used in the onSet Notification call below

			if(this.referenceIntegrity && oldValueOrValues){
				var oldValues = oldValueOrValues;
				if(!lang.isArray(oldValues)){
					oldValues = [oldValues];
				}
				for(var i = 0; i < oldValues.length; i++){
					var value = oldValues[i];
					if(this.isItem(value)){
						this._removeReferenceFromMap(value, item, attribute);
					}
				}
			}
		}else{
			var newValueArray;
			if(lang.isArray(newValueOrValues)){
				// Unfortunately, it's not safe to just do this:
				//    newValueArray = newValueOrValues;
				// Instead, we need to copy the array, which slice() does very nicely.
				// This is so that our internal data structure won't
				// get corrupted if the user mucks with the values array *after*
				// calling setValues().
				newValueArray = newValueOrValues.slice(0, newValueOrValues.length);
			}else{
				newValueArray = [newValueOrValues];
			}

			//We need to handle reference integrity if this is on.
			//In the case of set, we need to see if references were added or removed
			//and update the reference tracking map accordingly.
			if(this.referenceIntegrity){
				if(oldValueOrValues){
					var oldValues = oldValueOrValues;
					if(!lang.isArray(oldValues)){
						oldValues = [oldValues];
					}
					//Use an associative map to determine what was added/removed from the list.
					//Should be O(n) performant.  First look at all the old values and make a list of them
					//Then for any item not in the old list, we add it.  If it was already present, we remove it.
					//Then we pass over the map and any references left it it need to be removed (IE, no match in
					//the new values list).
					var map = {};
					arrayUtil.forEach(oldValues, function(possibleItem){
						if(this.isItem(possibleItem)){
							var id = this.getIdentity(possibleItem);
							map[id.toString()] = true;
						}
					}, this);
					arrayUtil.forEach(newValueArray, function(possibleItem){
						if(this.isItem(possibleItem)){
							var id = this.getIdentity(possibleItem);
							if(map[id.toString()]){
								delete map[id.toString()];
							}else{
								this._addReferenceToMap(possibleItem, item, attribute);
							}
						}
					}, this);
					for(var rId in map){
						var removedItem;
						if(this._itemsByIdentity){
							removedItem = this._itemsByIdentity[rId];
						}else{
							removedItem = this._arrayOfAllItems[rId];
						}
						this._removeReferenceFromMap(removedItem, item, attribute);
					}
				}else{
					//Everything is new (no old values) so we have to just
					//insert all the references, if any.
					for(var i = 0; i < newValueArray.length; i++){
						var value = newValueArray[i];
						if(this.isItem(value)){
							this._addReferenceToMap(value, item, attribute);
						}
					}
				}
			}
			item[attribute] = newValueArray;
			success = true;
		}

		// Now we make the dojo.data.api.Notification call
		if(callOnSet){
			this.onSet(item, attribute, oldValueOrValues, newValueOrValues);
		}
		return success; // boolean
	},

	_addReferenceToMap: function(/* item */ refItem, /* item */ parentItem, /* string */ attribute){
		//	summary:
		//		Method to add an reference map entry for an item and attribute.
		//	description:
		//		Method to add an reference map entry for an item and attribute. 		 //
		//	refItem:
		//		The item that is referenced.
		//	parentItem:
		//		The item that holds the new reference to refItem.
		//	attribute:
		//		The attribute on parentItem that contains the new reference.

		var parentId = this.getIdentity(parentItem);
		var references = refItem[this._reverseRefMap];

		if(!references){
			references = refItem[this._reverseRefMap] = {};
		}
		var itemRef = references[parentId];
		if(!itemRef){
			itemRef = references[parentId] = {};
		}
		itemRef[attribute] = true;
	},

	_removeReferenceFromMap: function(/* item */ refItem, /* item */ parentItem, /* string */ attribute){
		//	summary:
		//		Method to remove an reference map entry for an item and attribute.
		//	description:
		//		Method to remove an reference map entry for an item and attribute.  This will
		//		also perform cleanup on the map such that if there are no more references at all to
		//		the item, its reference object and entry are removed.
		//
		//	refItem:
		//		The item that is referenced.
		//	parentItem:
		//		The item holding a reference to refItem.
		//	attribute:
		//		The attribute on parentItem that contains the reference.
		var identity = this.getIdentity(parentItem);
		var references = refItem[this._reverseRefMap];
		var itemId;
		if(references){
			for(itemId in references){
				if(itemId == identity){
					delete references[itemId][attribute];
					if(this._isEmpty(references[itemId])){
						delete references[itemId];
					}
				}
			}
			if(this._isEmpty(references)){
				delete refItem[this._reverseRefMap];
			}
		}
	},

	_dumpReferenceMap: function(){
		//	summary:
		//		Function to dump the reverse reference map of all items in the store for debug purposes.
		//	description:
		//		Function to dump the reverse reference map of all items in the store for debug purposes.
		var i;
		for(i = 0; i < this._arrayOfAllItems.length; i++){
			var item = this._arrayOfAllItems[i];
			if(item && item[this._reverseRefMap]){
				console.log("Item: [" + this.getIdentity(item) + "] is referenced by: " + jsonUtil.toJson(item[this._reverseRefMap]));
			}
		}
	},

	_getValueOrValues: function(/* item */ item, /* attribute-name-string */ attribute){
		var valueOrValues = undefined;
		if(this.hasAttribute(item, attribute)){
			var valueArray = this.getValues(item, attribute);
			if(valueArray.length == 1){
				valueOrValues = valueArray[0];
			}else{
				valueOrValues = valueArray;
			}
		}
		return valueOrValues;
	},

	_flatten: function(/* anything */ value){
		if(this.isItem(value)){
			// Given an item, return an serializable object that provides a
			// reference to the item.
			// For example, given kermit:
			//    var kermit = store.newItem({id:2, name:"Kermit"});
			// we want to return
			//    {_reference:2}
			return {_reference: this.getIdentity(value)};
		}else{
			if(typeof value === "object"){
				for(var type in this._datatypeMap){
					var typeMap = this._datatypeMap[type];
					if(lang.isObject(typeMap) && !lang.isFunction(typeMap)){
						if(value instanceof typeMap.type){
							if(!typeMap.serialize){
								throw new Error("ItemFileWriteStore:  No serializer defined for type mapping: [" + type + "]");
							}
							return {_type: type, _value: typeMap.serialize(value)};
						}
					} else if(value instanceof typeMap){
						//SImple mapping, therefore, return as a toString serialization.
						return {_type: type, _value: value.toString()};
					}
				}
			}
			return value;
		}
	},

	_getNewFileContentString: function(){
		// summary:
		//		Generate a string that can be saved to a file.
		//		The result should look similar to:
		//		http://trac.dojotoolkit.org/browser/dojo/trunk/tests/data/countries.json
		var serializableStructure = {};

		var identifierAttribute = this._getIdentifierAttribute();
		if(identifierAttribute !== Number){
			serializableStructure.identifier = identifierAttribute;
		}
		if(this._labelAttr){
			serializableStructure.label = this._labelAttr;
		}
		serializableStructure.items = [];
		for(var i = 0; i < this._arrayOfAllItems.length; ++i){
			var item = this._arrayOfAllItems[i];
			if(item !== null){
				var serializableItem = {};
				for(var key in item){
					if(key !== this._storeRefPropName && key !== this._itemNumPropName && key !== this._reverseRefMap && key !== this._rootItemPropName){
						var valueArray = this.getValues(item, key);
						if(valueArray.length == 1){
							serializableItem[key] = this._flatten(valueArray[0]);
						}else{
							var serializableArray = [];
							for(var j = 0; j < valueArray.length; ++j){
								serializableArray.push(this._flatten(valueArray[j]));
								serializableItem[key] = serializableArray;
							}
						}
					}
				}
				serializableStructure.items.push(serializableItem);
			}
		}
		var prettyPrint = true;
		return jsonUtil.toJson(serializableStructure, prettyPrint);
	},

	_isEmpty: function(something){
		//	summary:
		//		Function to determine if an array or object has no properties or values.
		//	something:
		//		The array or object to examine.
		var empty = true;
		if(lang.isObject(something)){
			var i;
			for(i in something){
				empty = false;
				break;
			}
		}else if(lang.isArray(something)){
			if(something.length > 0){
				empty = false;
			}
		}
		return empty; //boolean
	},

	save: function(/* object */ keywordArgs){
		// summary: See dojo.data.api.Write.save()
		this._assert(!this._saveInProgress);

		// this._saveInProgress is set to true, briefly, from when save is first called to when it completes
		this._saveInProgress = true;

		var self = this;
		var saveCompleteCallback = function(){
			self._pending = {
				_newItems:{},
				_modifiedItems:{},
				_deletedItems:{}
			};

			self._saveInProgress = false; // must come after this._pending is cleared, but before any callbacks
			if(keywordArgs && keywordArgs.onComplete){
				var scope = keywordArgs.scope || window.global;
				keywordArgs.onComplete.call(scope);
			}
		};
		var saveFailedCallback = function(err){
			self._saveInProgress = false;
			if(keywordArgs && keywordArgs.onError){
				var scope = keywordArgs.scope || window.global;
				keywordArgs.onError.call(scope, err);
			}
		};

		if(this._saveEverything){
			var newFileContentString = this._getNewFileContentString();
			this._saveEverything(saveCompleteCallback, saveFailedCallback, newFileContentString);
		}
		if(this._saveCustom){
			this._saveCustom(saveCompleteCallback, saveFailedCallback);
		}
		if(!this._saveEverything && !this._saveCustom){
			// Looks like there is no user-defined save-handler function.
			// That's fine, it just means the datastore is acting as a "mock-write"
			// store -- changes get saved in memory but don't get saved to disk.
			saveCompleteCallback();
		}
	},

	revert: function(){
		// summary: See dojo.data.api.Write.revert()
		this._assert(!this._saveInProgress);

		var identity;
		for(identity in this._pending._modifiedItems){
			// find the original item and the modified item that replaced it
			var copyOfItemState = this._pending._modifiedItems[identity];
			var modifiedItem = null;
			if(this._itemsByIdentity){
				modifiedItem = this._itemsByIdentity[identity];
			}else{
				modifiedItem = this._arrayOfAllItems[identity];
			}

			// Restore the original item into a full-fledged item again, we want to try to
			// keep the same object instance as if we don't it, causes bugs like #9022.
			copyOfItemState[this._storeRefPropName] = this;
			for(var key in modifiedItem){
				delete modifiedItem[key];
			}
			lang.mixin(modifiedItem, copyOfItemState);
		}
		var deletedItem;
		for(identity in this._pending._deletedItems){
			deletedItem = this._pending._deletedItems[identity];
			deletedItem[this._storeRefPropName] = this;
			var index = deletedItem[this._itemNumPropName];

			//Restore the reverse refererence map, if any.
			if(deletedItem["backup_" + this._reverseRefMap]){
				deletedItem[this._reverseRefMap] = deletedItem["backup_" + this._reverseRefMap];
				delete deletedItem["backup_" + this._reverseRefMap];
			}
			this._arrayOfAllItems[index] = deletedItem;
			if(this._itemsByIdentity){
				this._itemsByIdentity[identity] = deletedItem;
			}
			if(deletedItem[this._rootItemPropName]){
				this._arrayOfTopLevelItems.push(deletedItem);
			}
		}
		//We have to pass through it again and restore the reference maps after all the
		//undeletes have occurred.
		for(identity in this._pending._deletedItems){
			deletedItem = this._pending._deletedItems[identity];
			if(deletedItem["backupRefs_" + this._reverseRefMap]){
				arrayUtil.forEach(deletedItem["backupRefs_" + this._reverseRefMap], function(reference){
					var refItem;
					if(this._itemsByIdentity){
						refItem = this._itemsByIdentity[reference.id];
					}else{
						refItem = this._arrayOfAllItems[reference.id];
					}
					this._addReferenceToMap(refItem, deletedItem, reference.attr);
				}, this);
				delete deletedItem["backupRefs_" + this._reverseRefMap];
			}
		}

		for(identity in this._pending._newItems){
			var newItem = this._pending._newItems[identity];
			newItem[this._storeRefPropName] = null;
			// null out the new item, but don't change the array index so
			// so we can keep using _arrayOfAllItems.length.
			this._arrayOfAllItems[newItem[this._itemNumPropName]] = null;
			if(newItem[this._rootItemPropName]){
				this._removeArrayElement(this._arrayOfTopLevelItems, newItem);
			}
			if(this._itemsByIdentity){
				delete this._itemsByIdentity[identity];
			}
		}

		this._pending = {
			_newItems:{},
			_modifiedItems:{},
			_deletedItems:{}
		};
		return true; // boolean
	},

	isDirty: function(/* item? */ item){
		// summary: See dojo.data.api.Write.isDirty()
		if(item){
			// return true if the item is dirty
			var identity = this.getIdentity(item);
			return new Boolean(this._pending._newItems[identity] ||
				this._pending._modifiedItems[identity] ||
				this._pending._deletedItems[identity]).valueOf(); // boolean
		}else{
			// return true if the store is dirty -- which means return true
			// if there are any new items, dirty items, or modified items
			return !this._isEmpty(this._pending._newItems) ||
				!this._isEmpty(this._pending._modifiedItems) ||
				!this._isEmpty(this._pending._deletedItems); // boolean
		}
	},

/* dojo.data.api.Notification */

	onSet: function(/* item */ item,
					/*attribute-name-string*/ attribute,
					/*object|array*/ oldValue,
					/*object|array*/ newValue){
		// summary: See dojo.data.api.Notification.onSet()

		// No need to do anything. This method is here just so that the
		// client code can connect observers to it.
	},

	onNew: function(/* item */ newItem, /*object?*/ parentInfo){
		// summary: See dojo.data.api.Notification.onNew()

		// No need to do anything. This method is here just so that the
		// client code can connect observers to it.
	},

	onDelete: function(/* item */ deletedItem){
		// summary: See dojo.data.api.Notification.onDelete()

		// No need to do anything. This method is here just so that the
		// client code can connect observers to it.
	},

	close: function(/* object? */ request){
		 // summary:
		 //		Over-ride of base close function of ItemFileReadStore to add in check for store state.
		 // description:
		 //		Over-ride of base close function of ItemFileReadStore to add in check for store state.
		 //		If the store is still dirty (unsaved changes), then an error will be thrown instead of
		 //		clearing the internal state for reload from the url.

		 //Clear if not dirty ... or throw an error
		 if(this.clearOnClose){
			 if(!this.isDirty()){
				 this.inherited(arguments);
			 }else{
				 //Only throw an error if the store was dirty and we were loading from a url (cannot reload from url until state is saved).
				 throw new Error("dojo.data.ItemFileWriteStore: There are unsaved changes present in the store.  Please save or revert the changes before invoking close.");
			 }
		 }
	}
});

});

},
'dojoe/treetable/filter/_FilterExpr':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!dojoe/treetable/filter/_DataExprs,dojo/date"], function(dijit,dojo,dojox){
/**
 * @author shakeel
 */
//This is the main file that should be 'required' if filter expression facility is necessary.
dojo.provide("dojoe.treetable.filter._FilterExpr");

dojo.require("dojoe.treetable.filter._DataExprs");
dojo.require("dojo.date");

(function(){
	var f_ns = dojoe.treetable.filter;
	/* Logic Operations */
	dojo.declare("dojoe.treetable.filter.LogicAND", f_ns._BiOpExpr, {
		// summary:
		//		A logic AND condition expression.
		_calculate: function(/* _ConditionExpr */left_operand,/* _ConditionExpr */right_operand,
							/* data item*/datarow,/* function(row,colIdx) */getter){
			// summary:
			//		Override from _BiOpExpr
			var res = left_operand.applyRow(datarow, getter).getValue() 
					&& right_operand.applyRow(datarow, getter).getValue();
			return new f_ns.BooleanExpr(res);	//_ConditionExpr
		},
		getName: function(){
			// summary:
			//		override from _ConditionExpr
			return "and";	//string
		}
	});
	dojo.declare("dojoe.treetable.filter.LogicOR", f_ns._BiOpExpr, {
		// summary:
		//		A logic OR condition expression.
		_calculate: function(/* _ConditionExpr */left_operand,/* _ConditionExpr */right_operand,
							/* data item*/datarow,/* function(row,colIdx) */getter){
			// summary:
			//		Override from _BiOpExpr
			var res = left_operand.applyRow(datarow, getter).getValue()
					|| right_operand.applyRow(datarow, getter).getValue();
			return new f_ns.BooleanExpr(res);	//_ConditionExpr
		},
		getName: function(){
			// summary:
			//		override from _ConditionExpr
			return "or";	//string
		}
	});
	dojo.declare("dojoe.treetable.filter.LogicXOR", f_ns._BiOpExpr, {
		// summary:
		//		A logic XOR condition expression.
		_calculate: function(/* _ConditionExpr */left_operand,/* _ConditionExpr */right_operand,
							/* data item*/datarow,/* function(row,colIdx) */getter){
			// summary:
			//		Override from _BiOpExpr
			var left_res = left_operand.applyRow(datarow, getter).getValue();
			var right_res = right_operand.applyRow(datarow, getter).getValue();
			return new f_ns.BooleanExpr((!!left_res) != (!!right_res));	//_ConditionExpr
		},
		getName: function(){
			// summary:
			//		override from _ConditionExpr
			return "xor";	//String
		}
	});
	dojo.declare("dojoe.treetable.filter.LogicNOT", f_ns._UniOpExpr, {
		// summary:
		//		A logic NOT condition expression.
		_calculate: function(/* _ConditionExpr */operand,/* data item*/datarow,/* function(row,colIdx) */getter){
			// summary:
			//		Override from _UniOpExpr
			return new f_ns.BooleanExpr(!operand.applyRow(datarow, getter).getValue());	//_ConditionExpr
		},
		getName: function(){
			// summary:
			//		override from _ConditionExpr
			return "not";	//String
		}
	});
	dojo.declare("dojoe.treetable.filter.LogicALL", f_ns._OperatorExpr, {
		// summary:
		//		A logic ALL condition expression, equals a sequence of logic ANDs
		applyRow: function(/* data item */datarow,/* function(row,colIdx) */ getter){
			// summary:
			//		Override from _ConditionExpr
			for(var i=0,res=true;
				res && (this._operands[i] instanceof f_ns._ConditionExpr);
				(res = this._operands[i].applyRow(datarow,getter).getValue()), ++i);
			return new f_ns.BooleanExpr(res);	//_ConditionExpr
		},
		getName: function(){
			// summary:
			//		override from _ConditionExpr
			return "all";	//String
		}
	});
	dojo.declare("dojoe.treetable.filter.LogicANY", f_ns._OperatorExpr, {
		// summary:
		//		A logic ANY condition expression, equals a sequence of logic ORs
		applyRow: function(/* data item */datarow,/* function(row,colIdx) */ getter){
			for(var i=0,res=false;
				!res && (this._operands[i] instanceof f_ns._ConditionExpr);
				(res = this._operands[i].applyRow(datarow,getter).getValue()), ++i);
			return new f_ns.BooleanExpr(res);	//_ConditionExpr
		},
		getName: function(){
			// summary:
			//		override from _ConditionExpr
			return "any";	//string
		}
	});
	
	/* Comparison Operations */
	function compareFunc(left,right,row,getter){
		var left = left.applyRow(row, getter);
		var right = right.applyRow(row, getter);
		var left_res = left.getValue();
		var right_res = right.getValue();
		if(left instanceof f_ns.TimeExpr){
			return dojo.date.compare(left_res,right_res,"time");
		}else if(left instanceof f_ns.DateExpr){
			return dojo.date.compare(left_res,right_res,"date");
		}else{
			return left_res == right_res ? 0 : (left_res < right_res ? -1 : 1);
		}
	}
	dojo.declare("dojoe.treetable.filter.EqualTo", f_ns._BiOpExpr, {
		// summary:
		//		An "equal to" condition expression.
		_calculate: function(/* _ConditionExpr */left_operand,/* _ConditionExpr */right_operand,
							/* data item*/datarow,/* function(row,colIdx) */getter){
			// summary:
			//		Override from _BiOpExpr
			var res = compareFunc(left_operand,right_operand,datarow,getter);
			return new f_ns.BooleanExpr(res == 0);	//_ConditionExpr
		},
		getName: function(){
			// summary:
			//		override from _ConditionExpr
			return "equal";	//string
		}
	});
	dojo.declare("dojoe.treetable.filter.LessThan", f_ns._BiOpExpr, {
		// summary:
		//		A "less than" condition expression.		
		_calculate: function(/* _ConditionExpr */left_operand,/* _ConditionExpr */right_operand,
							/* data item*/datarow,/* function(row,colIdx) */getter){
			// summary:
			//		Override from _BiOpExpr
			var res = compareFunc(left_operand,right_operand,datarow,getter);
			return new f_ns.BooleanExpr(res < 0);	//_ConditionExpr
		},
		getName: function(){
			// summary:
			//		override from _ConditionExpr
			return "less";	//string
		}
	});
	dojo.declare("dojoe.treetable.filter.LessThanOrEqualTo", f_ns._BiOpExpr, {
		// summary:
		//		A "less than or equal to" condition expression.	
		_calculate: function(/* _ConditionExpr */left_operand,/* _ConditionExpr */right_operand,
							/* data item*/datarow,/* function(row,colIdx) */getter){
			// summary:
			//		Override from _BiOpExpr
			var res = compareFunc(left_operand,right_operand,datarow,getter);
			return new f_ns.BooleanExpr(res <= 0);	//_ConditionExpr
		},
		getName: function(){
			// summary:
			//		override from _ConditionExpr
			return "lessEqual";	//string
		}
	});
	dojo.declare("dojoe.treetable.filter.LargerThan", f_ns._BiOpExpr, {
		// summary:
		//		A "larger than" condition expression.	
		_calculate: function(/* _ConditionExpr */left_operand,/* _ConditionExpr */right_operand,
							/* data item*/datarow,/* function(row,colIdx) */getter){
			// summary:
			//		Override from _BiOpExpr
			var res = compareFunc(left_operand,right_operand,datarow,getter);
			return new f_ns.BooleanExpr(res > 0);	//_ConditionExpr
		},
		getName: function(){
			// summary:
			//		override from _ConditionExpr
			return "larger";	//string
		}
	});
	dojo.declare("dojoe.treetable.filter.LargerThanOrEqualTo", f_ns._BiOpExpr, {
		// summary:
		//		A "larger than or equal to" condition expression.	
		_calculate: function(/* _ConditionExpr */left_operand,/* _ConditionExpr */right_operand,
							/* data item*/datarow,/* function(row,colIdx) */getter){
			// summary:
			//		Override from _BiOpExpr
			var res = compareFunc(left_operand,right_operand,datarow,getter);
			return new f_ns.BooleanExpr(res >= 0);	//_ConditionExpr
		},
		getName: function(){
			// summary:
			//		override from _ConditionExpr
			return "largerEqual";	//string
		}
	});
	
	/* String Operations */
	dojo.declare("dojoe.treetable.filter.Contains", f_ns._BiOpExpr, {
		// summary:
		//		A "contains" condition expression.
		_calculate: function(/* _ConditionExpr */left_operand,/* _ConditionExpr */right_operand,
							/* data item*/datarow,/* function(row,colIdx) */getter){
			// summary:
			//		Override from _BiOpExpr
			var left_res = String(left_operand.applyRow(datarow, getter).getValue());
			var right_res = String(right_operand.applyRow(datarow, getter).getValue());
			return new f_ns.BooleanExpr(left_res.indexOf(right_res) >= 0);	//_ConditionExpr
		},
		getName: function(){
			// summary:
			//		override from _ConditionExpr
			return "contains";	//string
		}
	});
	dojo.declare("dojoe.treetable.filter.ContainsIgnoreCase", f_ns._BiOpExpr, {
		// summary:
		//		A "contains" condition expression.
		fields : [],
		_calculate: function(/* _ConditionExpr */left_operand,/* _ConditionExpr */right_operand,
				/* data item*/datarow,/* function(row,colIdx) */getter){
		// summary:
		//		Override from _BiOpExpr
		var left_res = String(left_operand.applyRow(datarow, getter).getValue()).toLowerCase();
		var right_res = String(right_operand.applyRow(datarow, getter).getValue()).toLowerCase();
		var result = new f_ns.BooleanExpr(left_res.indexOf(right_res) >= 0);	//_ConditionExpr
		if(!result._value && left_operand._colArg.parentCell) {
			//might need to traverse the children if any
			var field = left_operand._colArg.parentCell.field;
			//traverse more if the field does not match the datarow children column 
			if(datarow[field] === undefined && left_operand._colArg.parentCell.children) {
				this.fields = [];
				//this.fields is also populated
				field = this._getDatarowField(left_operand._colArg.parentCell.children, datarow);
			}

			//working on the last child 
			if(dojo.isArray(this.fields) && this.fields.length > 0) {
				for(var i=0; i<this.fields.length; i++) {
					var field = this.fields[i];
					if(datarow[field].length == 1) {
						result = new f_ns.BooleanExpr(datarow[field][0].toString().indexOf(right_res) >= 0);	//_ConditionExpr
						if(result._value) {
							datarow.itemFound = true;
							return result;
						}
					}
				}
			}
			
			//console.log("field :", field);
			var datarowChildren = datarow[field];
			var tempArray = [];
			for(var i in datarowChildren) {
				result = this._calculate(left_operand,right_operand,datarowChildren[i],getter);
				if(datarowChildren[i].itemFound) {
					datarow.itemFound = true;
				}
				if(result._value ) {
                    tempArray.push(datarowChildren[i]);
					datarow[field] = tempArray;
				    datarow.itemFound = true;
				}				
			}
		}
		else {
			if(result._value) {
				//Bobby:25Mar2011, removing console out
				//console.log("--------------------result found : ", left_res ," ", right_res);				
			}
		}
		return result;
		},
		_getDatarowField: function(children, datarow) {
			if(children === undefined) {
				//no more children
				return;
			}
			
			for ( var chld=0; chld<children.length; chld++ ) {				
				var field = children[chld].field;
				if(field !== "label") {
					if(datarow[field] === undefined) {
						field = this._getDatarowField(children[chld].children, datarow);
					}
					if(field != null) {
						this.fields.push(field);
					}					
				}
			}
			return field;
		},
		getName: function(){
			// summary:
			//		override from _ConditionExpr
			return "containsignorecase";	//string
		}
	});
	dojo.declare("dojoe.treetable.filter.StartsWith", f_ns._BiOpExpr, {
		// summary:
		//		A "starts with" condition expression.
		_calculate: function(/* _ConditionExpr */left_operand,/* _ConditionExpr */right_operand,
							/* data item*/datarow,/* function(row,colIdx) */getter){
			// summary:
			//		Override from _BiOpExpr
			var left_res = String(left_operand.applyRow(datarow, getter).getValue());
			var right_res = String(right_operand.applyRow(datarow, getter).getValue());
			return new f_ns.BooleanExpr(left_res.substring(0, right_res.length) == right_res);	//_ConditionExpr
		},
		getName: function(){
			// summary:
			//		override from _ConditionExpr
			return "startsWith";	//string
		}
	});
	dojo.declare("dojoe.treetable.filter.EndsWith", f_ns._BiOpExpr, {
		// summary:
		//		An "ends with" condition expression.
		_calculate: function(/* _ConditionExpr */left_operand,/* _ConditionExpr */right_operand,
							/* data item*/datarow,/* function(row,colIdx) */getter){
			// summary:
			//		Override from _BiOpExpr
			var left_res = String(left_operand.applyRow(datarow, getter).getValue());
			var right_res = String(right_operand.applyRow(datarow, getter).getValue());
			return new f_ns.BooleanExpr(left_res.substring(left_res.length - right_res.length) == right_res);	//_ConditionExpr
		},
		getName: function(){
			// summary:
			//		override from _ConditionExpr
			return "endsWith";	//string
		}
	});
	dojo.declare("dojoe.treetable.filter.Matches", f_ns._BiOpExpr, {
		// summary:
		//		A "regular expression match" condition expression.
		//		The second operand's value will be regarded as an regular expression string.
		_calculate: function(/* _ConditionExpr */left_operand,/* _ConditionExpr */right_operand,
							/* data item*/datarow,/* function(row,colIdx) */getter){
			// summary:
			//		Override from _BiOpExpr
			var left_res = String(left_operand.applyRow(datarow, getter).getValue());
			var right_res = new RegExp(right_operand.applyRow(datarow, getter).getValue());
			return new f_ns.BooleanExpr(left_res.search(right_res) >=0 );	//_ConditionExpr
		},
		getName: function(){
			// summary:
			//		override from _ConditionExpr
			return "matches";	//string
		}
	});
})();

});

},
'gridx/modules/SummaryBar':function(){
define([
	"dojo/_base/declare",
	"dojo/dom-construct",
	"dojo/string",
	"../core/_Module",
	"dojo/i18n!../nls/SummaryBar"
], function(declare, domConstruct, string, _Module, nls){
	
	return declare(/*===== "gridx.modules.SummaryBar", =====*/_Module, {
		name: 'summaryBar',

//        required: ['vLayout'],

		getAPIPath: function(){
			return {
				summaryBar: this
			};
		},

		preload: function(){
			var t = this, m = t.model;
			t.domNode = domConstruct.create('div', {'class': 'gridxSummaryBar'});
			t.grid.vLayout.register(t, 'domNode', 'footerNode', 5);
			t.connect(m, 'onSizeChange', '_update');
			t.connect(m, 'onMarkChange', '_update');
			t._update();
		},
		destroy: function(){
			domConstruct.destroy(this.domNode);
			this.inherited(arguments);
		},
		_update: function(){
			var t = this,
				g = t.grid,
				sr = g.select && g.select.row,
				size = t.model.size(),
				selected = sr ? sr.getSelected().length : 0,
				tpl = sr ? nls.summaryWithSelection : nls.summary;
			t.domNode.innerHTML = string.substitute(tpl, [size, selected]);
		}
	});
});

},
'gridx/modules/IndirectSelect':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/event",
	"dojo/_base/query",
	"dojo/_base/lang",
	"dojo/dom-class",
	"dojo/_base/Deferred",
	"dojo/keys",
	"../core/_Module",
	"../core/util",
	"./RowHeader"
], function(declare, array, event, query, lang, domClass, Deferred, keys, _Module, util){

	return declare(/*===== "gridx.modules.IndirectSelect", =====*/_Module, {
		// summary:
		//		This module shows a checkbox(or radiobutton) on the row header when row selection is used.
		// description:
		//		This module relies on an implementation of the RowHeader module, and an implementation of
		//		the SelectRow module.
		//		This module will check whether the SelectRow module provides the functionality of "select rows by index" 
		//		(which means the "selectByIndex" method exists). If so, a "select all" checkbox can be provided 
		//		in the header node of the row header column.
		//		This module will also check whether the SelectRow module is configured to "single selection" mode
		//		(which means the "multiple" attribute is set to false). If so, radio button instead of checkbox
		//		will be used in row headers.

		name: 'indirectSelect',

		required: ['rowHeader', 'selectRow'],

		preload: function(){
			// tags:
			//		protected extension
			var t = this,
				g = t.grid,
				focus = g.focus,
				sr = g.select.row,
				rowHeader = g.rowHeader;
			rowHeader.cellProvider = lang.hitch(t, t._createSelector);
			t.batchConnect(
				[sr,'onHighlightChange', '_onHighlightChange' ],
				[sr,'clear', '_onClear' ],
				[sr, 'onSelectionChange', '_onSelectionChange'],
				[g, 'onRowMouseOver', '_onMouseOver'],
				[g, 'onRowMouseOut', '_onMouseOut'],
				[g, 'onRowKeyDown', '_onKeyDown'],
				[g, 'onHeaderKeyDown', '_onKeyDown'],
				focus && [focus, 'onFocusArea', function(name){
					if(name == 'rowHeader'){
						t._onMouseOver();
					}
				}],
				focus && [focus, 'onBlurArea', function(name){
					if(name == 'rowHeader'){
						t._onMouseOut();
					}
				}]);
			if(sr.selectByIndex && t.arg('all')){
				t._allSelected = {};
				rowHeader.headerProvider = lang.hitch(t, t._createSelectAllBox);
				rowHeader.loaded.then(function(){
					if(focus){
						t._initFocus();
					}
					t.connect(g, 'onRowHeaderHeaderMouseDown', '_onSelectAll');
				});
			}
		},

		// all: Boolean
		//		Whether the "select all" checkbox is allowed to appear.
		all: true,

		//Private----------------------------------------------------------
		_createSelector: function(row){
			var rowNode = row.node(),
				selected = rowNode && domClass.contains(rowNode, 'gridxRowSelected'),
				partial = rowNode && domClass.contains(rowNode, 'gridxRowPartialSelected');
			return this._createCheckBox(selected, partial);
		},

		_createCheckBox: function(selected, partial){
			var dijitClass = this._getDijitClass();
			return ['<span role="', this._isSingle() ? 'radio' : 'checkbox',
				'" class="gridxIndirectSelectionCheckBox dijitReset dijitInline ',
				dijitClass, ' ',
				selected ? dijitClass + 'Checked' : '',
				partial ? dijitClass + 'Partial' : '',
				'" aria-checked="', selected ? 'true' : partial ? 'mixed' : 'false',
				'"><span class="gridxIndirectSelectionCheckBoxInner">',
				selected ? '&#10003;' : partial ? '&#9646;' : '&#9744;',
				'</span></span>'
			].join('');
		},

		_createSelectAllBox: function(){
			return this._createCheckBox(this._allSelected[this._getPageId()]);
		},

		_getPageId: function(){
			return this.grid.body.rootStart + ',' + this.grid.body.rootCount;
		},

		_onClear: function(){
			var cls = this._getDijitClass() + 'Checked';
			query('.' + cls, this.grid.rowHeader.bodyNode).forEach(function(node){
				domClass.remove(node, cls);
			});
		},

		_onHighlightChange: function(target, toHighlight){
			var node = query('[visualindex="' + target.row + '"].gridxRowHeaderRow .gridxIndirectSelectionCheckBox', this.grid.rowHeader.bodyNode)[0];
			if(node){
				var dijitClass = this._getDijitClass(),
					partial = toHighlight == 'mixed',
					selected = toHighlight && !partial;
				domClass.toggle(node, dijitClass + 'Checked', selected);
				domClass.toggle(node, dijitClass + 'Partial', partial);
				node.setAttribute('aria-checked', selected ? 'true' : partial ? 'mixed' : 'false');
				node.firstChild.innerHTML = selected ? '&#10003;' : partial ? '&#9646;' : '&#9744;';
			}
		},

		_onMouseOver: function(){
			var sr = this.grid.select.row;
			if(!sr.holdingCtrl){
				this._holdingCtrl = false;
				sr.holdingCtrl = true;
			}
		},

		_onMouseOut: function(){
			if(this.hasOwnProperty('_holdingCtrl')){
				this.grid.select.row.holdingCtrl = false;
				delete this._holdingCtrl;
			}
		},

		_getDijitClass: function(){
			return this._isSingle() ? 'dijitRadio' : 'dijitCheckBox';
		},

		_isSingle: function(){
			var select = this.grid.select.row;
			return select.hasOwnProperty('multiple') && !select.arg('multiple');
		},

		_onSelectAll: function(){
			var t = this,
				g = t.grid,
				body = g.body;
			g.select.row[t._allSelected[t._getPageId()] ? 
				'deselectByIndex' :
				'selectByIndex'
			]([0, body.visualCount - 1]);
		},

		_onSelectionChange: function(selected){
			var t = this, d,
				allSelected,
				body = t.grid.body,
				model = t.model,
				start = body.rootStart,
				count = body.rootCount;
			var selectedRoot = array.filter(selected, function(id){
				return !model.treePath(id).pop();
			});
			if(count === model.size()){
				allSelected = count == selectedRoot.length;
			}else{
				d = new Deferred();
				model.when({
					start: start,
					count: count
				}, function(){
					var indexes = array.filter(array.map(selectedRoot, function(id){
						return model.idToIndex(id);
					}), function(index){
						return index >= start && index < start + count;
					});
					allSelected = count == indexes.length;
					d.callback();
				});
			}
			Deferred.when(d, function(){
				t._allSelected[t._getPageId()] = allSelected;
				var node = t.grid.rowHeader.headerCellNode.firstChild;
				domClass.toggle(node, t._getDijitClass() + 'Checked', allSelected);
				node.setAttribute('aria-checked', allSelected ? 'true' : 'false');
			});
		},

		//Focus------------------------------------------------------
		_initFocus: function(){
			var g = this.grid,
				rowHeader = g.rowHeader,
				headerCellNode = rowHeader.headerCellNode,
				focus = function(evt){
					util.stopEvent(evt);
					domClass.add(headerCellNode, 'gridxHeaderCellFocus');
					headerCellNode.focus();
					return true;
				},
				blur = function(){
					domClass.remove(headerCellNode, 'gridxHeaderCellFocus');
					return true;
				};
			g.focus.registerArea({
				name: 'selectAll',
				priority: 0.89,
				focusNode: rowHeader.headerNode,
				doFocus: focus,
				doBlur: blur,
				onFocus: focus,
				onBlur: blur
			});
		},
		_onKeyDown: function(evt){
			if(evt.keyCode == 65 && evt.ctrlKey && !evt.shiftKey){
				if(!this._allSelected[this._getPageId()]){
					this._onSelectAll();
				}
				event.stop(evt);
			}
		}		
	});
});

},
'ibm/tivoli/navmodel/Link':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/Dataset,ibm/tivoli/navmodel/Relationship,ibm/tivoli/navmodel/Property,ibm/tivoli/navmodel/Status,ibm/tivoli/navmodel/LinkStyle,ibm/tivoli/navmodel/Exception,ibm/tivoli/navmodel/Utils,ibm/tivoli/navmodel/Trace"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Link"); 
 
// includes
dojo.require("ibm.tivoli.navmodel.Dataset");
dojo.require("ibm.tivoli.navmodel.Relationship");
dojo.require("ibm.tivoli.navmodel.Property");
dojo.require("ibm.tivoli.navmodel.Status");
dojo.require("ibm.tivoli.navmodel.LinkStyle");
dojo.require("ibm.tivoli.navmodel.Exception");
dojo.require("ibm.tivoli.navmodel.Utils");
dojo.require("ibm.tivoli.navmodel.Trace");

ibm.tivoli.navmodel.Link._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.Link");

////////////////////////////////////
// Link
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Link", null, {
	// private members       
	_link: null,
	_dataset: null,
	_relationships: null,
	_props: null,
	_propsById: null,
	_nullPropIds: null,
	_allProps: false,
	_allMetadata: false,
	_status: null,
	_style: null,
	_defaultTask: null,
	_expiration: null,
	_trace: ibm.tivoli.navmodel.Link._trace,
	
	/*=====
	constructor_kwArgs: function(link, dataset, propIds, allProperties, allMetadata, expiration) {
		// link: Object
		//		json link object
		// dataset: Object
		//		ibm.tivoli.navmodel.Dataset that owns this link
		// propIds: String[]
		//		optional array indicating the property Id's that were requested with 
		//		the item; so it can be determined if any properties are null
		// allProperties: Boolean
		//		optional: indicates if the link already has "all" properties
		// allMetadata: Boolean
		//		optional: indicates if link properties already have metadata
		// expiration: Long
		//		optional: expiration time;  will use (current time + interval) if not supplied
	},
	=====*/

	 constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs Link object.
		this._link = kwArgs.link;
		if (this._link) {
			// optimization of json on the server can leave out the "id"
			// but we know the default format
			if (!this._link.id) {
				if (this._link.item1 < this._link.item2) {
					this._link.id = this._link.item1 + "_" + this._link.item2;
				}
				else {
					this._link.id = this._link.item2 + "_" + this._link.item1;
				}
			}
		}
		else {
			this._link = {};
		}
		
		this._dataset = kwArgs.dataset;
		this._props = [];
		this._propsById = {};
		this._nullPropIds = {};
		this._allProps = kwArgs.allProperties;		
		this._allMetadata = kwArgs.allMetadata;
		
		if (this._link.properties) {
			var propsLength = this._link.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var prop = new ibm.tivoli.navmodel.Property({property: this._link.properties[propIndex], dataset: this._dataset, objectType: "link"});
				this._props[propIndex] = prop;
				this._propsById[prop.getId()] = prop;
			}
			
			// see if we should mark any properties as null because they were not returned
			if (kwArgs.propIds) {
				propsLength = kwArgs.propIds.length;
				for (propIndex = 0;  propIndex < propsLength;  ++propIndex) {
					var propId = kwArgs.propIds[propIndex];				
					if (!this._propsById[propId]) {
						this._nullPropIds[propId] = true;
					}
				}
			}
		}
				
		// expiration was supplied
		if (kwArgs.expiration) {
			this._expiration = kwArgs.expiration;
		}
		// reset to current time + interval
		else {
			this.resetExpiration();
		}
	},

	/////////////////////////////
	// public methods
	/////////////////////////////

	mixin: function(link) {		
		// summary:
		//		Mixin properties and status from this Link object
		if (link && link !== this) {
			this.removeAttribute("style");
			this._style = null;
			
			this.mixinAttributes(link.getAttributes());
			this.mixinProperties(link._props, link._nullPropIds);
			this._status = link.getStatuses({widgetDef: null});
			this._defaultTask = link.getDefaultTask();
			this._relationships = link.getRelationships();
			this._allProps = (this._allProps || link._allProps);
			this._allMetadata = (link._allProps && link._allMetadata);			
		}		
	},
	
	getDataset: function() {		
		// summary:
		//		Returns the dataset that owns the link
		return this._dataset;
	},

	getObjectType: function() {		
		// summary:
		//		Indicate that this object is a "link"
		return "link";
	},

	getAttribute: function(key) {		
		// summary:
		//		Returns general attribute for requested key
		return this._link[key];
	},

	setAttribute: function(key, value) {		
		// summary:
		//		Set general attribute for requested key
		this._link[key] = value;
	},

	removeAttribute: function(key) {
		// summary:
		//		Remove general attribute for requested key
		delete this._link[key];
	},
	
	getAttributes: function() {		
		// summary:
		//		Get all general attributes for the link
		var link = dojo.clone(this._link);
		
		// make sure "properties" are current as we may have received events
		if (this._props && this._props.length > 0) {
			link.properties = [];
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				link.properties[propIndex] = this._props[propIndex].getAttributes();
			}
		}
		
		return link;
	},

	mixinAttributes: function(attr) {		
		// summary:
		//		Mixin general attributes for the link
		dojo.mixin(this._link, attr);		
	},

	getUri: function() {
		if (!this._link.uri) {
			this._link.uri = this._dataset.getAttribute("uri") + "/links/" + encodeURIComponent(this.getId());
		}
	
		return this._dataset.getArg("baseUrl") + this._link.uri;
	},
	
	getId: function() {		
		// summary:
		//		Returns link id
		return this._link.id;
	},

	setId: function(id) {		
		// summary:
		//		Set the link id
		this._link.id = id;		
	},

	getType: function() {		
		// summary:
		//		Returns the type of link
		return this._link.type;
	},

	setType: function(type) {		
		// summary:
		//		Set the link type
		this._link.type = type;		
	},

	getLabel: function() {		
		// summary:
		//		Returns globalized label for the link
		return this._link.label;
	},

	setLabel: function(label) {		
		// summary:
		//		Set the globalized label for the link
		this._link.label = label;		
	},

	getTooltip: function() {		
		// summary:
		//		Returns globalized tooltip for the link
		return this._link.tooltip;
	},

	setTooltip: function(tooltip) {		
		// summary:
		//		Set the globalized tooltip for the link
		this._link.tooltip = tooltip;
	},
	
	isContainment: function() {		
		// summary:
		//		Returns true if this link represents containment instead of
		//		connection via normal lines
		return (this._link.containment ? this._link.containment : false);
	},
	
	setContainment: function(isContainment) {		
		// summary:
		//		Set the value indicating containment
		this._link.containment = isContainment;
	},

	getDirection: function() {		
		// summary:
		//		Returns the direction associated with the link
		//		
		//		|	values:	"1_2"   	means direction of arrow flows from item1 to item2
		//		|		"2_1"		means direction of arrow flows from item2 to item1
		//		|		"both"	means that arrow flows in both directions
		return (this._link.direction ? this._link.direction : "1_2");
	},

	setDirection: function(direction) {		
		// summary:
		//		Set the direction associated with the link
		//		
		//		|	values:	"1_2"   	means direction of arrow flows from item1 to item2
		//		|		"2_1"		means direction of arrow flows from item2 to item1
		//		|		"both"	means that arrow flows in both directions
		this._link.direction = direction;
	},
	
	/*=====
	getStatuses_kwArgs: function(widgetDef) {
		// widgetDef: Object
		//		widget definition can override attributes
	},
	=====*/

	getStatuses: function(/*getStatuses_kwArgs*/ kwArgs) {		
		// summary:
		//		Returns ibm.tivoli.navmodel.Status[] objects indicating the link status values
		var args = {
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		// if expired mixin from dataset cache
		this._validateExpiration();
		
		if (!this._status && this._link.status) {
			this._status = [];
			var statusLength = this._link.status.length;
			for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
				if (this._link.status.label && this._link.status.image) {
					this._status.push(new ibm.tivoli.navmodel.Status({status: this._link.status[statusIndex], version: this._version()}));				
				}				
				else {
					var statusArgs = {
						type: "link",
						statusId: this._link.status[statusIndex].id,
						statusValue: this._link.status[statusIndex].value				
					};
				
					var statusObj = this._dataset.getStatus(statusArgs);
					if (statusObj) {				
						this._status.push(statusObj);
					}
					else {
						this._trace.error("Could not locate status with id=" + statusArgs.statusId + " value=" + statusArgs.statusValue, "getStatuses");
					}
				}
			}			
		}
		
		// merge widget definition status
		var status = this._status;
		if (status && args.widgetDef) {
			status = this._status.slice(0);
			var statusLength2 = this._status.length;
			for (var statusIndex2 = 0;  statusIndex2 < statusLength2;  ++statusIndex2) {
				status[statusIndex2] = this._status[statusIndex2].clone(args.widgetDef, "link");
			}
		}					
		
		return status;		
	},

	setStatuses: function(statuses) {		
		// summary:
		//		Set ibm.tivoli.navmodel.Status[] objects indicating the link status values
		this._status = statuses;
	},


	getItem1: function() {
		// summary:
		//		Returns the item1 involved in the link		
		var item = this._dataset.getItem({itemId: this.getItem1Id()});		
		return item;
	},
	
	getItem1Id: function() {		
		// summary:
		//		Returns the id of item1 involved in the link;  this value can be undefined
		//		if the link is between two different datasets and the connection point
		//		is to the dataset and not a link within it
		return this._link.item1;
	},

	setItem1Id: function(item1) {		
		// summary:
		//		Set the id of item1 involved in the link
		this._link.item1 = item1;
	},

	getItem2: function() {
		// summary:
		//		Returns the item2 involved in the link		
		var item = this._dataset.getItem({itemId: this.getItem2Id()});	
		return item;
	},
		
	getItem2Id: function() {		
		// summary:
		//		Returns the id of item2 involved in the link;  this value can be undefined
		//		if the link is between two different datasets and the connection point
		//		is to the dataset and not a link within it
		return this._link.item2;
	},

	setItem2Id: function(item2) {		
		// summary:
		//		Set the id of item1 involved in the link
		this._link.item2 = item2;
	},

	getRelationshipCount: function() {		
		// summary:
		//		Returns the number of relationships involved in the link
		return this.getRelationships().length;
	},

	getRelationship: function(relationshipId) {		
		// summary:
		//		Returns requested relationship if it exists
		
		// if expired mixin from dataset cache
		this._validateExpiration();
		
		var relationships = this.getRelationships();
		if (relationships) {
			var relLength = relationships.length;
			for (var relIndex = 0;  relIndex < relLength;  ++relIndex) {
				if (relationships[relIndex].getId() === relationshipId) {
					return relationships[relIndex];
				}
			}
		}
		
		return null;
	},

	getRelationships: function() {		
		// summary:
		//		Returns the ibm.tivoli.navmodel.Relationship[] objects that are involved in the link
		
		// if expired mixin from dataset cache
		this._validateExpiration();
		
		if (!this._relationships) {
			var direction = this.getDirection();
			this._relationships = [];
			var relationshipsLength = this._link.relationships.length;
			for (var relIndex = 0;  relIndex < relationshipsLength;  ++relIndex) {			
				var relationshipObj = this._link.relationships[relIndex];
				
				if (!relationshipObj.linkId) {
					relationshipObj.linkId = this.getId();
				}
				
				if (!relationshipObj.label) {
					relationshipObj.label = this.getLabel();
				}
				
				if (!relationshipObj.type) {
					relationshipObj.type = this.getType();
				}
				
				if (!relationshipObj.sourceItem) {
					switch(direction) {
						case "1_2":
						case "none":
							if (this.getItem1Id()) {
								relationshipObj.sourceItem = this.getItem1Id();
							}
							break;
						case "2_1":
							if (this.getItem2Id()) {
								relationshipObj.sourceItem = this.getItem2Id();
							}
					}	
				}
				
				if (!relationshipObj.targetItem) {
					switch(direction) {
						case "1_2":
						case "none":
							if (this.getItem2Id()) {
								relationshipObj.targetItem = this.getItem2Id();
							}
							break;
						case "2_1":
							if (this.getItem1Id()) {
								relationshipObj.targetItem = this.getItem1Id();
							}
					}					
				}
				
				var relationship = new ibm.tivoli.navmodel.Relationship({relationship: relationshipObj, dataset: this._dataset});
				this._relationships[relIndex] = this._dataset._cacheRelationship(relationship);
			}
		}

		return this._relationships;
	},

	setRelationships: function(relationships) {		
		// summary:
		//		Set the ibm.tivoli.navmodel.Relationship[] objects that are involved in the link
		this._relationships = relationships;
	},

	supportsProperties: function() {		
		// summary:
		//		Indicates if "Properties" is supported for the link
		return (this._link.supportsProperties !== undefined ? this._link.supportsProperties : true);
	},

	setSupportsProperties: function(supportsProperties) {		
		// summary:
		//		Set whether link  supports "Properties"
		this._link.supportsProperties = supportsProperties;
	},

	hasCachedMetadata: function() {		
		// summary:
		//		Returns true if link has all properties and associated metadata cached
		return this._allMetadata;
	},

	hasAllCachedProperties: function() {		
		// summary:
		//		Returns true if link has all proprties cached
		return this._allProps;
	},

	hasCachedProperty: function(/*String*/ propId) {
		// summary:
		//		Returns true if we have the requested property in the cache
		// propId
		//		property id
		return (this._propsById[propId] ? true : false);
	},
	
	hasCachedProperties: function(/*String[]*/ properties) {		
		// summary:
		//		Returns true if we have all the requested properties in the cache
		// properties:
		//		array of property ids
		if (properties) {
			var propsLength = properties.length;
			
			// asking if we have "all" properties
			if (propsLength === 1 && properties[0] === "all") {
				return this._allProps;
			}
			
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				if (!this._propsById[properties[propIndex]]) {
					return false;
				}
			}
		}
		
		return true;
	},

	getCachedPropertyIds: function(expandAll) {		
		// summary:
		//		Returns array of property Ids currently cached in the item
		var propIds = [];
		
		// have all the properties
		if (!expandAll && this._allProps) {
			propIds.push("all");
		}
		// get individual property ids	
		else if (this._props) {
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				propIds.push(this._props[propIndex].getId());
			}
		}
		return propIds;
	},
	
	/*=====
	properties_onComplete: function(properties, kwArgs){},
	=====*/
	
	/*=====
	_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	getProperties_kwArgs: function(onComplete, onError, properties, metadata, sync) {
		// onComplete: properties_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		property ids to fetch or "all"
		// metadata: Boolean
		//		indicates whether metadata is required
		// sync: Boolean
		//		indicates if call is synchronous or asynchronous
	},
	=====*/

	getProperties: function(/*getProperties_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously returns properties for the link
		var args = {                
			onComplete: function(properties, args){
			},
			onError: function(error, exception, ioArgs){				
			},                
			properties: ["all"],
			metadata: false,
			sync: false
		};   
		dojo.mixin(args, kwArgs);         
		
		// if expired mixin from dataset cache
		this._validateExpiration();
		
		// fetch all properties
		if (args.properties.length === 1 && args.properties[0] === "all") {
			if (this._allProps && (!args.metadata || this._allMetadata)) {				
				if (args.sync) {
					args.onComplete(this._props, args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, this._props, args), 0);
				}		
			}
			else {
				this._fetchProperties(args);
			}
		}		
		// fetch selective properties
		else {
			// look up properties in cache
			var props = [];
			var propsLength = args.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var propId = args.properties[propIndex];
				var prop = this._propsById[propId];				
				if (prop && (!args.metadata || prop.hasMetadata())) {
					props[propIndex] = prop;					
				}
				else {
					break;
				}
			}
			
			// if we found all properties in cache then return them
			if (props.length === propsLength) {
				if (args.sync) {
					args.onComplete(props, args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, props, args), 0);
				}
			}
			// otherwise we need to fetch
			else {
				this._fetchProperties(args);
			}
		}		
	},
	
	setProperties: function(properties) {		
		// summary:
		//		Set the ibm.tivoli.navmodel.Property[] objects for the link
		//		
		//		Note: client side only operation;  no request to rest service
		this._propsById = {};
		this._nullPropIds = {};
		this._props = [];
		
		if (properties) {
			var propsLength = properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var prop = properties[propIndex];
				this._propsById[prop.getId()] = prop;
			}
		}		
	},

	mixinProperties: function(properties, nullPropIds) {		
		// summary:
		//		Mixin the ibm.tivoli.navmodel.Property[] objects for the link
		//		
		//		Note: client side only operation;  no request to rest service
		
		// mixin property objects
		if (properties) {
			var propsLength = properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				this.mixinProperty(properties[propIndex]);
			}
		}
		
		// mixin any propIds that were null
		if (nullPropIds) {
			for (var propId in nullPropIds) {
				if (nullPropIds.hasOwnProperty(propId)) {
					this._nullPropIds[propId] = true;
				}
			}
		}
	},

	 getProperty: function(/*String*/ propId) {		
		// summary:
		//		Synchronously returns requested property
		// propId:
		//		property id to return
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		
		// if expired mixin from dataset cache
		this._validateExpiration();
		
		// check prop cache
		var prop = this._propsById[propId];
			
		if (!prop) {
			// we have asked for this property before and it wasn't found
			if (this._nullPropIds[propId]) {
				return null;
			}
			
			// check default properties on link	
			if (this._link.properties) {
				var propsLength = this._link.properties.length;
				for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
					var rawProp = this._link.properties[propIndex];
					if (rawProp.id === propId) {
						prop = new ibm.tivoli.navmodel.Property({property: rawProp, dataset: this._dataset, objectType: "link"});
						
						// cache property
						this._propsById[propId] = prop;
						break;
					}
				}				
			}
		}	
		
		// fetch it
		if (!prop && !this._allProps) {
			prop = this._fetchProperty(propId);
			
			// cache property
			if (prop) {
				this._propsById[propId] = prop;
			}
		}		
		
		return prop;
	},

	setProperty: function(property) {		
		// summary:
		//		Set ibm.tivoli.navmodel.Property for the item
		//		
		//		Note: client side only operation;  no request to rest service
		var found = this._propsById[property.getId()];		
		if (found) {
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				if (this._props[propIndex].getId() === property.getId()) {
					this._props[propIndex] = property;
					break;
				}
			}
		}
		else {
			this._props.push(property);
		}		
		
		this._propsById[property.getId()] = property;
	},

	mixinProperty: function(property) {		
		// summary:
		//		Mixin ibm.tivoli.navmodel.Property for the link
		//		
		//		Note: client side only operation;  no request to rest service
		var prop = this._propsById[property.getId()];
		if (prop) {
			prop.mixin(property);
		}
		else {
			this._props.push(property);
			this._propsById[property.getId()] = property;
		}	
	},
	
	/*=====
	_onComplete: function(kwArgs){},
	=====*/
	
	/*=====
	editProperties_kwArgs: function(onComplete, onError, properties) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		array of property ids to set
	},
	=====*/

	editProperties: function(/*editProperties_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously modify properties for the link
		var args = {                
			onComplete: function(properties, args){
			},
			onError: function(error, exception, ioArgs){				
			},                
			properties: null
		};
		dojo.mixin(args, kwArgs);
		
		if (args.properties && args.properties.length > 0) {
			var link = dojo.clone(this._link);
			link.properties = [];
			
			var propsLength = args.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				link.properties[propIndex] = args.properties[propIndex].getAttributes();
			}
			
			var json = dojo.toJson(link, false);
			
			dojo.xhrPost ({
				url: ibm.tivoli.navmodel.Utils.getParamUrl(this.getUri(), this._dataset),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version()),
					'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "linkView"),
					'Content-Length': json.length
				},
				handleAs: "json",
				postData: json,
				sync: false,			
				load: dojo.hitch(this, function(result, ioArgs) {
					args.onComplete(args);						
				}),
				error: dojo.hitch(function(error, ioArgs) {
					exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "setProperties", this._trace, this._version());				               
				})
			});	
		}
		// nothing to set
		else {
			setTimeout(dojo.hitch(this, args.onComplete, args), 0);
		}		
	},

	 editProperty: function(/*String*/ property) {		
		// summary:
		//		Synchronously modify property for the link
		// propId:
		//		property id to return
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var exception = null;
		
		if (property) {
			var link = dojo.clone(this._link);
			link.properties = [property.getAttributes()];
			
			var json = dojo.toJson(link, false);
			
			dojo.xhrPost ({
				url: ibm.tivoli.navmodel.Utils.getParamUrl(this.getUri(), this._dataset),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version()),
					'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "linkView"),
					'Content-Length': json.length
				},
				handleAs: "json",
				postData: json,
				sync: true,			
				load: dojo.hitch(this, function(result, ioArgs) {						
				}),
				error: dojo.hitch(this, function(error, ioArgs) {
					exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "setProperty", this._trace, this._version());									              
				})
			});				
		}
		
		if (exception) {
			throw exception;
		}
	},

	hasOwnLinkStyle: function() {		
		// summary:
		//		Returns true if the link has a LinkStyle that overrides the default
		//		link style for the link type  (i.e. different than the one retrieved
		//		from the dataset)
		return (this._link.style ? true: false);
	},
	
	/*=====
	getLinkStyle_kwArgs: function(widgetDef) {
		// widgetDef: Object
		//		widget definition can override
	},
	=====*/
	
	getLinkStyle: function(/*getLinkStyle_kwArgs*/ kwArgs) {		
		// summary:
		//		Returns the ibm.tivoli.navmodel.LinkStyle for this link
		var args = {
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		if (!this._style) {
			if (this._link.style) {
				this._style = new ibm.tivoli.navmodel.LinkStyle({style: this._link.style, version: this._version()});
			}
			else {
				this._style = this._dataset.getLinkStyle({linkType: this.getType(), widgetDef: null});
			}
		}
		
		// merge widget definition styles 
		var style = this._style;
		if (style && args.widgetDef) {
			// clone and merge widget definition attributes				
			if (style) {
				style = style.clone(args.widgetDef, "link");
			}
			// unknown style; see if it exists in widget definition
			else {
				var widgetStyle = args.widgetDef.getLinkStyle(this.getType());
				if (widgetStyle) {
					style = this._dataset._createLinkStyle(args.widgetDef, widgetStyle, "link");
				}
			}							
		}
		
		return style;
	},

	setLinkStyle: function(style) {		
		// summary:
		//		Set link style for the link
		this._style = style;
	},

	getDefaultTask: function() {		
		// summary:
		//		Returns optional default task associated with the link
		if (!this._defaultTask && this._link.defaultTask) {
			this._defaultTask = new ibm.tivoli.navmodel.Task({task: this._link.defaultTask, dataset: this._dataset});
		}
		
		return this._defaultTask;
	},

	setDefaultTask: function(defaultTask) {		
		// summary:
		//		Set optional default task associated with the link
		this._defaultTask = defaultTask;
	},

	isExpired: function() {		
		// summary:
		//		Returns true if the item is currently expired
		return (this._expiration ? false : true);
	},

	checkExpiration: function(nowTime) {		
		// summary:
		//		Check the expiration time on the item; return true if expired
		var expired = false;
		
		// already set to null from a previous check
		if (!this._expiration) {
			expired = true;
		}
		// check agaist the time to see if expired
		else if (nowTime >= this._expiration) {
			this._expiration = null;
			expired = true;
		}
		
		return expired;		
	},

	resetExpiration: function() {		
		// summary:
		//		Rest the expiration time for this item
		if (this._dataset.cacheExpires()) {
			this._expiration = this._dataset.getNewExpirationTime();
		}
		else {
			this._expiration = -1;
		}
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////

	_version: function() {		
		// summary:
		//		Returns rest version
		return this._dataset.getVersion();
	},

	_validateExpiration: function(kwArgs) {		
		// summary:
		//		If item is expired; mixin with a current item from the dataset cache
		if (this._dataset.cacheExpires()) {
			// expired
			if (this.isExpired()) {
				var properties = null;
				var metadata = false;
				if (kwArgs) {
					properties = kwArgs.properties;
					metadata = kwArgs.metadata;
				}			
				if (!properties) {
					properties = this.getCachedPropertyIds();
				}
		
				var link = this._dataset.getLink({linkId: this.getId(), properties: properties, metadata: metadata});
				if (link) {
					this.mixin(link);
				}			
			}
			// NOT expired; reset expiration since touched
			else {
				this.resetExpiration();
			}
		}
	},	
        
	/*=====
	fetchProperties_kwArgs: function(onComplete, onError, properties, metadata, sync) {
		// onComplete: properties_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		property ids to fetch or "all"
		// metadata: Boolean
		//		indicates whether metadata is required
		// sync: Boolean
		//		indicates if call is synchronous or asynchronous	
	},
	=====*/
	
	_fetchProperties: function(/*fetchProperties_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously returns properties for the link
		this._props = [];
		this._allProps = false;
		this._allMetadata = false;
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this._dataset);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		content.optimize = true;
		content.metadata = kwArgs.metadata;
		
		// are we fetching "all" properties
		var fetchAll = (kwArgs.properties.length === 1 && kwArgs.properties[0] === "all");
		
		return dojo.xhrGet({
			url: this.getUri(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "linkView")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var props = [];
				var rawProps = result.properties;
				var rawPropsLength = rawProps.length;
				for (var propIndex = 0;  propIndex < rawPropsLength;  ++propIndex) {
					var prop = new ibm.tivoli.navmodel.Property({property: rawProps[propIndex], dataset: this._dataset, objectType: "link"});
					props[propIndex] = prop;
					
					if (fetchAll) {
						this._props[propIndex] = prop;
					}
					
					this._propsById[prop.getId()] = prop;
				}
				
				if (fetchAll) {
					this._allProps = true;
					this._allMetadata = kwArgs.metadata;
				}
				
				kwArgs.onComplete(props, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_fetchProperties", this._trace, this._version()), ioArgs);			
			})
		});		
	},

	_fetchProperty: function(/*String*/ propId) {		
		// summary:
		//		Synchronously returns requested property
		// propId:
		//		property id to return
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var exception = null;
		var prop = null;
			
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this._dataset);
		content.optimize = true;
		content.properties = propId;
		
		dojo.xhrGet({
			url: this.getUri(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "linkView")
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				if (result.properties && result.properties.length > 0) {
					prop = new ibm.tivoli.navmodel.Property({property: result.properties[0], dataset: this._dataset, objectType: "link"});
				
					// cache property
					this._propsById[propId] = prop;
				}
				// property was not found; remember so we don't ask again
				else {
					this._nullPropIds[propId] = true;
				}
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_fetchProperty", this._trace, this._version());				
			})
		});
		
		if (exception) {
			throw exception;
		}
		
		return prop;
	}
});
});

},
'gridx/modules/dnd/Column':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/dom-geometry",
	"dojo/dom-class",
	"dojo/_base/query",
	"./_Base",
	"../../core/_Module"
], function(declare, array, domGeometry, domClass, query, _Base, _Module){

	return declare(/*===== "gridx.modules.dnd.Column", =====*/_Base, {
		// summary:
		//		This module provides an implementation of column drag & drop.
		//		It supports column reordering within grid, dragging out of grid, and dragging into grid.

		name: 'dndColumn',

		required: ['_dnd', 'selectColumn', 'moveColumn'],

		getAPIPath: function(){
			return {
				dnd: {
					column: this
				}
			};
		},

		preload: function(){
			var t = this,
				g = t.grid;
			t.inherited(arguments);
			t._selector = g.select.column;
			t.connect(g.header, 'onRender', '_initHeader');
		},

		load: function(){
			this._initHeader();
			this.loaded.callback();
		},
	
		//Public---------------------------------------------------------------------------------------

		//accept: String[]
		//		Can drag out what kind of stuff.
		//		For now can not drag in any columns.
		accept: [],

		//provide: String[]
		//		Can drag out what kind of stuff
		provide: ['grid/columns'],

		//Package--------------------------------------------------------------------------------------
		_checkDndReady: function(evt){
			var t = this;
			if(t._selector.isSelected(evt.columnId)){
				t._selectedColIds = t._selector.getSelected();
				t.grid.dnd._dnd.profile = t;
				return true;
			}
			return false;
		},

		onDraggedOut: function(/*source*/){
			//TODO: Support drag columns out (remove columns).
		},

		//Private--------------------------------------------------------------------------------------
		_cssName: "Column",

		_initHeader: function(){
			query('.gridxCell', this.grid.header.domNode).attr('aria-dragged', 'false');
		},

		_onBeginDnd: function(source){
			var t = this;
			source.delay = t.arg('delay');
			array.forEach(t._selectedColIds, function(id){
				query('[colid="' + id + '"].gridxCell', t.grid.header.domNode).attr('aria-dragged', 'true');
			});
		},

		_getDndCount: function(){
			return this._selectedColIds.length;
		},

		_onEndDnd: function(){
			query('[aria-dragged="true"].gridxCell', this.grid.header.domNode).attr('aria-dragged', 'false');
		},

		_buildDndNodes: function(){
			var gid = this.grid.id;
			return array.map(this._selectedColIds, function(colId){
				return ["<div id='", gid, "_dndcolumn_", colId, "' gridid='", gid, "' columnid='", colId, "'></div>"].join('');
			}).join('');
		},
	
		_onBeginAutoScroll: function(){
			var autoScroll = this.grid.autoScroll;
			this._autoScrollV = autoScroll.vertical;
			autoScroll.vertical = false;
		},

		_onEndAutoScroll: function(){
			this.grid.autoScroll.vertical = this._autoScrollV;
		},

		_getItemData: function(id){
			return id.substring((this.grid.id + '_dndcolumn_').length);
		},
		
		//---------------------------------------------------------------------------------------------
		_calcTargetAnchorPos: function(evt, containerPos){
			var node = evt.target,
				t = this,
				g = t.grid,
				ltr = g.isLeftToRight(),
				columns = g._columns,
				ret = {
					height: containerPos.h + "px",
					width: '',
					top: ''
				},
				func = function(n){
					var id = n.getAttribute('colid'),
						index = g._columnsById[id].index,
						first = n,
						last = n,
						firstIdx = index,
						lastIdx = index;
					if(t._selector.isSelected(id)){
						firstIdx = index;
						while(firstIdx > 0 && t._selector.isSelected(columns[firstIdx - 1].id)){
							--firstIdx;
						}
						first = query(".gridxHeaderRow [colid='" + columns[firstIdx].id + "']", g.headerNode)[0];
						lastIdx = index;
						while(lastIdx < columns.length - 1 && t._selector.isSelected(columns[lastIdx + 1].id)){
							++lastIdx;
						}
						last = query(".gridxHeaderRow [colid='" + columns[lastIdx].id + "']", g.headerNode)[0];
					}
					if(first && last){
						var firstPos = domGeometry.position(first),
							lastPos = domGeometry.position(last),
							middle = (firstPos.x + lastPos.x + lastPos.w) / 2,
							pre = evt.clientX < middle;
						if(pre){
							ret.left = (firstPos.x - containerPos.x - 1) + "px";
						}else{
							ret.left = (lastPos.x + lastPos.w - containerPos.x - 1) + "px";
						}
						t._target = pre ^ ltr ? lastIdx + 1 : firstIdx;
					}else{
						delete t._target;
					}
					return ret;
				};
			while(node){
				if(domClass.contains(node, 'gridxCell')){
					return func(node);
				}
				node = node.parentNode;
			}
			//For FF, when dragging from another grid, the evt.target is always grid.bodyNode!
			// so have to get the cell node by position, which is relatively slow.
			var rowNode = query(".gridxRow", g.bodyNode)[0],
				rowPos = domGeometry.position(rowNode.firstChild);
			if(rowPos.x + rowPos.w <= evt.clientX){
				ret.left = (rowPos.x + rowPos.w - containerPos.x - 1) + 'px';
				t._target = columns.length;
			}else if(rowPos.x >= evt.clientX){
				ret.left = (rowPos.x - containerPos.x - 1) + 'px';
				t._target = 0;
			}else if(query(".gridxCell", rowNode).some(function(cellNode){
				var cellPos = domGeometry.position(cellNode);
				if(cellPos.x <= evt.clientX && cellPos.x + cellPos.w >= evt.clientX){
					node = cellNode;
					return true;
				}
			})){
				return func(node);
			}
			return ret;
		},
		
		_onDropInternal: function(nodes, copy){
			var t = this;
			if(t._target >= 0){
				var indexes = array.map(t._selectedColIds, function(colId){
					return t.grid._columnsById[colId].index;
				});
				t.grid.move.column.move(indexes, t._target);
			}
		},
		
		_onDropExternal: function(/*source, nodes, copy*/){
			//TODO: Support drag in columns from another grid or non-grid source
		}
	});
});

},
'dijit/_base/focus':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/dom", // dom.isDescendant
	"dojo/_base/lang", // lang.isArray
	"dojo/topic", // publish
	"dojo/_base/window", // win.doc win.doc.selection win.global win.global.getSelection win.withGlobal
	"../focus",
	".."	// for exporting symbols to dijit
], function(array, dom, lang, topic, win, focus, dijit){

	// module:
	//		dijit/_base/focus
	// summary:
	//		Deprecated module to monitor currently focused node and stack of currently focused widgets.
	//		New code should access dijit/focus directly.

	lang.mixin(dijit, {
		// _curFocus: DomNode
		//		Currently focused item on screen
		_curFocus: null,

		// _prevFocus: DomNode
		//		Previously focused item on screen
		_prevFocus: null,

		isCollapsed: function(){
			// summary:
			//		Returns true if there is no text selected
			return dijit.getBookmark().isCollapsed;
		},

		getBookmark: function(){
			// summary:
			//		Retrieves a bookmark that can be used with moveToBookmark to return to the same range
			var bm, rg, tg, sel = win.doc.selection, cf = focus.curNode;

			if(win.global.getSelection){
				//W3C Range API for selections.
				sel = win.global.getSelection();
				if(sel){
					if(sel.isCollapsed){
						tg = cf? cf.tagName : "";
						if(tg){
							//Create a fake rangelike item to restore selections.
							tg = tg.toLowerCase();
							if(tg == "textarea" ||
									(tg == "input" && (!cf.type || cf.type.toLowerCase() == "text"))){
								sel = {
									start: cf.selectionStart,
									end: cf.selectionEnd,
									node: cf,
									pRange: true
								};
								return {isCollapsed: (sel.end <= sel.start), mark: sel}; //Object.
							}
						}
						bm = {isCollapsed:true};
						if(sel.rangeCount){
							bm.mark = sel.getRangeAt(0).cloneRange();
						}
					}else{
						rg = sel.getRangeAt(0);
						bm = {isCollapsed: false, mark: rg.cloneRange()};
					}
				}
			}else if(sel){
				// If the current focus was a input of some sort and no selection, don't bother saving
				// a native bookmark.  This is because it causes issues with dialog/page selection restore.
				// So, we need to create psuedo bookmarks to work with.
				tg = cf ? cf.tagName : "";
				tg = tg.toLowerCase();
				if(cf && tg && (tg == "button" || tg == "textarea" || tg == "input")){
					if(sel.type && sel.type.toLowerCase() == "none"){
						return {
							isCollapsed: true,
							mark: null
						}
					}else{
						rg = sel.createRange();
						return {
							isCollapsed: rg.text && rg.text.length?false:true,
							mark: {
								range: rg,
								pRange: true
							}
						};
					}
				}
				bm = {};

				//'IE' way for selections.
				try{
					// createRange() throws exception when dojo in iframe
					//and nothing selected, see #9632
					rg = sel.createRange();
					bm.isCollapsed = !(sel.type == 'Text' ? rg.htmlText.length : rg.length);
				}catch(e){
					bm.isCollapsed = true;
					return bm;
				}
				if(sel.type.toUpperCase() == 'CONTROL'){
					if(rg.length){
						bm.mark=[];
						var i=0,len=rg.length;
						while(i<len){
							bm.mark.push(rg.item(i++));
						}
					}else{
						bm.isCollapsed = true;
						bm.mark = null;
					}
				}else{
					bm.mark = rg.getBookmark();
				}
			}else{
				console.warn("No idea how to store the current selection for this browser!");
			}
			return bm; // Object
		},

		moveToBookmark: function(/*Object*/ bookmark){
			// summary:
			//		Moves current selection to a bookmark
			// bookmark:
			//		This should be a returned object from dijit.getBookmark()

			var _doc = win.doc,
				mark = bookmark.mark;
			if(mark){
				if(win.global.getSelection){
					//W3C Rangi API (FF, WebKit, Opera, etc)
					var sel = win.global.getSelection();
					if(sel && sel.removeAllRanges){
						if(mark.pRange){
							var n = mark.node;
							n.selectionStart = mark.start;
							n.selectionEnd = mark.end;
						}else{
							sel.removeAllRanges();
							sel.addRange(mark);
						}
					}else{
						console.warn("No idea how to restore selection for this browser!");
					}
				}else if(_doc.selection && mark){
					//'IE' way.
					var rg;
					if(mark.pRange){
						rg = mark.range;
					}else if(lang.isArray(mark)){
						rg = _doc.body.createControlRange();
						//rg.addElement does not have call/apply method, so can not call it directly
						//rg is not available in "range.addElement(item)", so can't use that either
						array.forEach(mark, function(n){
							rg.addElement(n);
						});
					}else{
						rg = _doc.body.createTextRange();
						rg.moveToBookmark(mark);
					}
					rg.select();
				}
			}
		},

		getFocus: function(/*Widget?*/ menu, /*Window?*/ openedForWindow){
			// summary:
			//		Called as getFocus(), this returns an Object showing the current focus
			//		and selected text.
			//
			//		Called as getFocus(widget), where widget is a (widget representing) a button
			//		that was just pressed, it returns where focus was before that button
			//		was pressed.   (Pressing the button may have either shifted focus to the button,
			//		or removed focus altogether.)   In this case the selected text is not returned,
			//		since it can't be accurately determined.
			//
			// menu: dijit._Widget or {domNode: DomNode} structure
			//		The button that was just pressed.  If focus has disappeared or moved
			//		to this button, returns the previous focus.  In this case the bookmark
			//		information is already lost, and null is returned.
			//
			// openedForWindow:
			//		iframe in which menu was opened
			//
			// returns:
			//		A handle to restore focus/selection, to be passed to `dijit.focus`
			var node = !focus.curNode || (menu && dom.isDescendant(focus.curNode, menu.domNode)) ? dijit._prevFocus : focus.curNode;
			return {
				node: node,
				bookmark: node && (node == focus.curNode) && win.withGlobal(openedForWindow || win.global, dijit.getBookmark),
				openedForWindow: openedForWindow
			}; // Object
		},

		// _activeStack: dijit._Widget[]
		//		List of currently active widgets (focused widget and it's ancestors)
		_activeStack: [],

		registerIframe: function(/*DomNode*/ iframe){
			// summary:
			//		Registers listeners on the specified iframe so that any click
			//		or focus event on that iframe (or anything in it) is reported
			//		as a focus/click event on the <iframe> itself.
			// description:
			//		Currently only used by editor.
			// returns:
			//		Handle to pass to unregisterIframe()
			return focus.registerIframe(iframe);
		},

		unregisterIframe: function(/*Object*/ handle){
			// summary:
			//		Unregisters listeners on the specified iframe created by registerIframe.
			//		After calling be sure to delete or null out the handle itself.
			// handle:
			//		Handle returned by registerIframe()

			handle && handle.remove();
		},

		registerWin: function(/*Window?*/targetWindow, /*DomNode?*/ effectiveNode){
			// summary:
			//		Registers listeners on the specified window (either the main
			//		window or an iframe's window) to detect when the user has clicked somewhere
			//		or focused somewhere.
			// description:
			//		Users should call registerIframe() instead of this method.
			// targetWindow:
			//		If specified this is the window associated with the iframe,
			//		i.e. iframe.contentWindow.
			// effectiveNode:
			//		If specified, report any focus events inside targetWindow as
			//		an event on effectiveNode, rather than on evt.target.
			// returns:
			//		Handle to pass to unregisterWin()

			return focus.registerWin(targetWindow, effectiveNode);
		},

		unregisterWin: function(/*Handle*/ handle){
			// summary:
			//		Unregisters listeners on the specified window (either the main
			//		window or an iframe's window) according to handle returned from registerWin().
			//		After calling be sure to delete or null out the handle itself.

			handle && handle.remove();
		}
	});

	// Override focus singleton's focus function so that dijit.focus()
	// has backwards compatible behavior of restoring selection (although
	// probably no one is using that).
	focus.focus = function(/*Object || DomNode */ handle){
		// summary:
		//		Sets the focused node and the selection according to argument.
		//		To set focus to an iframe's content, pass in the iframe itself.
		// handle:
		//		object returned by get(), or a DomNode

		if(!handle){ return; }

		var node = "node" in handle ? handle.node : handle,		// because handle is either DomNode or a composite object
			bookmark = handle.bookmark,
			openedForWindow = handle.openedForWindow,
			collapsed = bookmark ? bookmark.isCollapsed : false;

		// Set the focus
		// Note that for iframe's we need to use the <iframe> to follow the parentNode chain,
		// but we need to set focus to iframe.contentWindow
		if(node){
			var focusNode = (node.tagName.toLowerCase() == "iframe") ? node.contentWindow : node;
			if(focusNode && focusNode.focus){
				try{
					// Gecko throws sometimes if setting focus is impossible,
					// node not displayed or something like that
					focusNode.focus();
				}catch(e){/*quiet*/}
			}
			focus._onFocusNode(node);
		}

		// set the selection
		// do not need to restore if current selection is not empty
		// (use keyboard to select a menu item) or if previous selection was collapsed
		// as it may cause focus shift (Esp in IE).
		if(bookmark && win.withGlobal(openedForWindow || win.global, dijit.isCollapsed) && !collapsed){
			if(openedForWindow){
				openedForWindow.focus();
			}
			try{
				win.withGlobal(openedForWindow || win.global, dijit.moveToBookmark, null, [bookmark]);
			}catch(e2){
				/*squelch IE internal error, see http://trac.dojotoolkit.org/ticket/1984 */
			}
		}
	};

	// For back compatibility, monitor changes to focused node and active widget stack,
	// publishing events and copying changes from focus manager variables into dijit (top level) variables
	focus.watch("curNode", function(name, oldVal, newVal){
		dijit._curFocus = newVal;
		dijit._prevFocus = oldVal;
		if(newVal){
			topic.publish("focusNode", newVal);	// publish
		}
	});
	focus.watch("activeStack", function(name, oldVal, newVal){
		dijit._activeStack = newVal;
	});

	focus.on("widget-blur", function(widget, by){
		topic.publish("widgetBlur", widget, by);	// publish
	});
	focus.on("widget-focus", function(widget, by){
		topic.publish("widgetFocus", widget, by);	// publish
	});

	return dijit;
});

},
'dojoe/treetable/filter/_DataExprs':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!dojoe/treetable/filter/_ConditionExpr,dojo/date/locale"], function(dijit,dojo,dojox){
/**
 * @author shakeel
 */
dojo.provide("dojoe.treetable.filter._DataExprs");

dojo.require("dojoe.treetable.filter._ConditionExpr");
dojo.require("dojo.date.locale");

(function(){
	var f_ns = dojoe.treetable.filter;

	dojo.declare("dojoe.treetable.filter.BooleanExpr", f_ns._DataExpr, {
		// summary:
		//		A condition expression wrapper for boolean values
		_convertData: function(/* anything */dataValue){
			// summary:
			//		override from _DataExpr
			return !!dataValue;	//Boolean
		},
		getName: function(){
			return "bool";	//String
		}
	});
	dojo.declare("dojoe.treetable.filter.StringExpr", f_ns._DataExpr, {
		// summary:
		//		A condition expression wrapper for string values
		_convertData: function(/* anything */dataValue){
			// summary:
			//		override from _DataExpr
			return String(dataValue);	//String
		},
		getName: function(){
			// summary:
			//		override from _ConditionExpr
			return "string";	//String
		}
	});
	dojo.declare("dojoe.treetable.filter.NumberExpr", f_ns._DataExpr, {
		// summary:
		//		A condition expression wrapper for number values
		_convertDataToExpr: function(/* anything */dataValue){
			// summary:
			//		override from _DataExpr
			return parseFloat(dataValue);	//Number
		},
		getName: function(){
			// summary:
			//		override from _ConditionExpr
			return "number";	//String
		}
	});
	dojo.declare("dojoe.treetable.filter.DateExpr", f_ns._DataExpr, {
		// summary:
		//		A condition expression wrapper for date values
		_selector: "date",
		_convertData: function(/* anything */dataValue){
			// summary:
			//		override from _DataExpr
			if(dataValue instanceof Date){
				return dataValue;
			}else if(typeof dataValue == "number"){
				return new Date(dataValue);
			}else{
				var res = dojo.date.locale.parse(String(dataValue), dojo.mixin({selector: this._selector}, this._convertArgs));
				if(!res){
					throw new Error("Datetime parse failed: " + dataValue);
				}
				return res;
			}
		},
		toObject: function(){
			// summary:
			//		Overrided from _DataExpr.toObject
			if(this._value instanceof Date){
				var tmp = this._value;
				this._value = this._value.valueOf();
				var res = this.inherited(arguments);
				this._value = tmp;
				return res;
			}else{
				return this.inherited(arguments);
			}
		},
		getName: function(){
			// summary:
			//		override from _ConditionExpr
			return this._selector;	//String
		}
	});
	dojo.declare("dojoe.treetable.filter.TimeExpr", f_ns.DateExpr, {
		// summary:
		//		A condition expression wrapper for time values
		_selector: "time"
	});
})();


});

},
'ibm/tivoli/navmodel/WidgetDefUtils':function(){
// wrapped by build app
define("ibm/tivoli/navmodel/WidgetDefUtils", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/WidgetTypeInfo,ibm/tivoli/navmodel/WidgetDefInfo,ibm/tivoli/navmodel/WidgetGroupInfo,ibm/tivoli/navmodel/Utils,ibm/tivoli/navmodel/Trace"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.WidgetDefUtils"); 

// includes
dojo.require("ibm.tivoli.navmodel.WidgetTypeInfo");
dojo.require("ibm.tivoli.navmodel.WidgetDefInfo");
dojo.require("ibm.tivoli.navmodel.WidgetGroupInfo");
dojo.require("ibm.tivoli.navmodel.Utils");
dojo.require("ibm.tivoli.navmodel.Trace");

ibm.tivoli.navmodel.WidgetDefUtils._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.WidgetDefUtils");

ibm.tivoli.navmodel.WidgetDefUtils.createWidgetDef = function(/*createWidgetDef_kwArgs*/ kwArgs) {
	// summary: 
	//		Create a widget definition in the server database and then construct the
	//		requested widget definition object (i.e. sub-class of AbstractWidgetDef or WidgetDef)
	// widgetClass: String
	//		widget definition class to construct (ex: ibm.tivoli.navmodel.TopoWidgetDef)
	// baseUrl: String
	//		base url for identifying rest service
	// registryLocation: String
	//		(optional) Indicates if "remote" provider definition is in local registry or is
	//		located at the remote rest service.  
	//		values are: "local" or "remote"  (Default for TIP is "local")
	// provider: String
	//		provider id
	// datasource: String
	//		datasource id
	// widgetType: String
	//		widget type in form <type>-<version>
	// widgetId: String
	//		widget definition id in form <id>-<version>
	// definitionObj: Object
	//		javascript object representing the widget definition contents
	// overwrite: Boolean
	//		indicates if widget definition should be overwritten if it already exists
	// profileID: String
	//		optional unique id to associate with reading and writing the
	//		"user" and "shared" deltas to the widget definition.  If not
	//		supplied; a uniqueId is created consisting of all the related
	//		information (i.e. providerType, datasourceType, widgetType, etc)
	// parentWidgetDef: ibm.tivoli.navmodel.AbstractWidgetDef  (or descendent)
	//		optional parent widget definition
	// version: Float

	var args = {
		widgetClass: null,
		baseUrl: null,
		registryLocation: (ibm.tivoli.navmodel.Utils.isTIP() ? "local" : null),
		provider: null,
		datasource: null,
		widgetType: null,
		widgetId: null,
		definitionObj: null,
		overwrite: false,
		profileID: null,
		parentWidgetDef: null,
		version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion()
	};
	dojo.mixin(args, kwArgs);

	// url needed to create the widget definition
	var url = args.baseUrl;
	url += "/providers/" + encodeURIComponent(args.provider);
	url += args.datasource ? ("/datasources/" + encodeURIComponent(args.datasource)) : "";
	url += "/widgets/"+ args.widgetType; 
	url += "/definitions/" + args.widgetId;
	url += "?type=base";
	if (args.overwrite) {
		url += "&overwrite=true";
	}
	if (args.registryLocation && args.registryLocation === "local") {
		url += "&registryLocation=local";
	}

	var exception = null;		
	var json = dojo.toJson(args.definitionObj, false);

	dojo.xhrPut({
		url: url,
		headers: {
			'Accept': ibm.tivoli.navmodel.Utils.getContentType(args.version),
			'Content-Type': 'application/json',
			'Content-Length': json.length
		},
		handleAs: "json",
		putData: json,
		sync: true,
		load: dojo.hitch(this, function(result, ioArgs) {									
		}),
		error: dojo.hitch (this, function(error, ioArgs){
			exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "createWidgetDef", ibm.tivoli.navmodel.WidgetDefUtils._trace, args.version);				
		})
	});

	if (exception) {
		throw exception;
	}

	var widget = null;
	var objectClass = dojo.getObject(args.widgetClass);
	if (objectClass) {
		var widgetArgs = {
			baseUrl: args.baseUrl,
			provider: args.provider,
			datasource: args.datasource,
			widget: args.widgetType,
			definition: args.widgetId,
			type: "user",
			profileID: args.profileID,
			parentWidgetDef: args.parentWidgetDef,
			version: args.version
		};

		try {
			widget = new objectClass(widgetArgs);
		}
		catch(ex) {
			ibm.tivoli.navmodel.WidgetDefUtils._trace.error("Exception constructing widget definition: ex=" + ex, "createWidgetDef");					
		}											
	}
	else {
		ibm.tivoli.navmodel.WidgetDefUtils._trace.error("Cannot instantiate widget def class=" + args.widgetClass, "createWidgetDef");				
	}

	return widget;
};

ibm.tivoli.navmodel.WidgetDefUtils.getWidgetTypes = function(kwArgs) {
	// summary: 
	//		Returns list of ibm.tivoli.navmodel.WidgetTypeInfo instances
	// onComplete: function(widgetTypes, widgetTypeCount, kwArgs)
	//		callback for successful fetch
	// onError: function(error, exception, ioArgs)
	//		callback for error conditions
	// baseUrl: String
	//		base url for identifying rest service
	// registryLocation: String
	//		(optional) Indicates if "remote" provider definition is in local registry or is
	//		located at the remote rest service.  
	//		values are: "local" or "remote"  (Default for TIP is "local")
	// provider: String
	//		provider id (can also be null or key words "any" and "all")
	// datasource: String
	//		datasource id (can also be null or key words "any" and "all")
	// start: int
	//		start index to fetch
	// count: int
	//		number of definitions to fetch; null means all
	// sort: String
	//		comma deliminated attribute string (preceded with optional +/- indicating ascending/descending)
	//		example:  label,type
	// condition: String
	//		condition expression string 
	//		example: description contains 'abcd'
	// sync: Boolean
	//		indicates if fetch is synchronous
	// version: Float

	var args = {
		onComplete: function(widgetTypes, widgetTypeCount, kwArgs){
		},
		onError: function(error, exception, ioArgs){				
		},
		baseUrl: null,
		registryLocation: (ibm.tivoli.navmodel.Utils.isTIP() ? "local" : null),
		provider: null,
		datasource: null,
		start: 0,
		count: null,
		sort: null,
		condition: null,
		sync: false,
		version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion()
	};
	dojo.mixin(args, kwArgs);

	var url = null;	
	if (args.provider) {
		url = args.baseUrl + "/providers/" + encodeURIComponent(args.provider);
		url += (args.datasource ? "/datasources/" + encodeURIComponent(args.datasource) : "");
		url += "/widgets";
	}
	else {
		url = args.baseUrl + "/widgets";
	}
	
	var content = ibm.tivoli.navmodel.Utils.getContent();
	if (args.registryLocation && args.registryLocation === "local") {
		content.registryLocation = args.registryLocation;
	}
	if (args.start) {
		content.start = args.start;
	}
	if (args.count) {
		content.count = args.count;
	}
	if (args.sort) {
		content.sort = args.sort;
	}
	if (args.condition) {
		content.condition = args.condition;
	}
	
	return dojo.xhrGet({
		url: url,
		headers: {
			'Accept': ibm.tivoli.navmodel.Utils.getContentType(args.version, "widgetTypeList")			
		},
		handleAs: "json",
		sync: args.sync,
		content: content,
		load: dojo.hitch(this, function(result, ioArgs){
			var widgetTypes = [];			
			if (result.items) {
				var length = result.items.length;
				for (index = 0;  index < length;  ++index) {
					widgetTypes[index] = new ibm.tivoli.navmodel.WidgetTypeInfo({widgetType: result.items[index], version: args.version});
				}
			}
					
			kwArgs.onComplete(widgetTypes, result.filteredRows, kwArgs);										
		}),
		error: dojo.hitch(this, function(error, ioArgs){
			var exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "getWidgetTypes", ibm.tivoli.navmodel.WidgetDefUtils._trace, args.version);
			kwArgs.onError(error, exception, ioArgs);
		})
	});	
};


ibm.tivoli.navmodel.WidgetDefUtils.getWidgetDefinitions = function(kwArgs) {
	// summary: 
	//		Returns list of ibm.tivoli.navmodel.WidgetDefInfo instances
	// onComplete: function(widgetDefs, widgetDefCount, kwArgs)
	//		callback for successful fetch
	// onError: function(error, exception, ioArgs)
	//		callback for error conditions
	// baseUrl: String
	//		base url for identifying rest service
	// registryLocation: String
	//		(optional) Indicates if "remote" provider definition is in local registry or is
	//		located at the remote rest service.  
	//		values are: "local" or "remote"  (Default for TIP is "local")
	// provider: String
	//		provider id (can also use key words "any" and "all")
	// datasource: String
	//		datasource id (can also use key words "any" and "all")
	// widgetType: String
	//		widget type (i.e. topology-0.1.0;  can also use key word "all")
	// groupIds: String[]
	//		array of groupIds used to filter the returned definitions
	// start: int
	//		start index to fetch
	// count: int
	//		number of definitions to fetch; null means all
	// sort: String
	//		comma deliminated attribute string (preceded with optional +/- indicating ascending/descending)
	//		example:  label,type
	// condition: String
	//		condition expression string 
	//		example: description contains 'abcd'
	// sync: Boolean
	//		indicates if fetch is synchronous
	// version: Float

	var args = {
		onComplete: function(widgetDefs, widgetDefCount, kwArgs){
		},
		onError: function(error, exception, ioArgs){				
		},
		baseUrl: null,
		registryLocation: (ibm.tivoli.navmodel.Utils.isTIP() ? "local" : null),
		provider: null,
		datasource: null,
		widgetType: null,
		groupIds: null,
		start: 0,
		count: null,
		sort: null,
		condition: null,
		sync: false,
		version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion()
	};
	dojo.mixin(args, kwArgs);

	var url = args.baseUrl + "/providers/" + encodeURIComponent(args.provider);
	url += (args.datasource ? "/datasources/" + encodeURIComponent(args.datasource) : "");
	url += "/widgets/" + args.widgetType;
	url += "/definitions";
	
	var content = ibm.tivoli.navmodel.Utils.getContent();
	if (args.registryLocation && args.registryLocation === "local") {
		content.registryLocation = args.registryLocation;
	}
	if (args.start) {
		content.start = args.start;
	}
	if (args.count) {
		content.count = args.count;
	}
	if (args.sort) {
		content.sort = args.sort;
	}
	if (args.condition) {
		content.condition = args.condition;
	}
	if (args.groupIds) {
		content.groupIds = args.groupIds.join(",");
	}
	
	return dojo.xhrGet({
		url: url,
		headers: {
			'Accept': ibm.tivoli.navmodel.Utils.getContentType(args.version, "widgetDefList")			
		},
		handleAs: "json",
		sync: args.sync,
		content: content,
		load: dojo.hitch(this, function(result, ioArgs){
			var widgetDefs = [];			
			if (result.items) {
				var length = result.items.length;
				for (index = 0;  index < length;  ++index) {
					widgetDefs[index] = new ibm.tivoli.navmodel.WidgetDefInfo({widgetDef: result.items[index], version: args.version});
				}
			}
					
			kwArgs.onComplete(widgetDefs, result.filteredRows, kwArgs);										
		}),
		error: dojo.hitch(this, function(error, ioArgs){
			var exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "getWidgetDefinitions", ibm.tivoli.navmodel.WidgetDefUtils._trace, args.version);
			kwArgs.onError(error, exception, ioArgs);
		})
	});	
};


ibm.tivoli.navmodel.WidgetDefUtils.getWidgetGroups = function(kwArgs) {
	// summary: 
	//		Returns list of ibm.tivoli.navmodel.WidgetGroupInfo instances
	// onComplete: function(widgetGroups, widgetGroupCount, kwArgs)
	//		callback for successful fetch
	// onError: function(error, exception, ioArgs)
	//		callback for error conditions
	// baseUrl: String
	//		base url for identifying rest service
	// registryLocation: String
	//		(optional) Indicates if "remote" provider definition is in local registry or is
	//		located at the remote rest service.  
	//		values are: "local" or "remote"  (Default for TIP is "local")
	// provider: String
	//		provider id (can also be key words "any" and "all")
	// datasource: String
	//		datasource id (can also be key words "any" and "all")
	// start: int
	//		start index to fetch
	// count: int
	//		number of groups to fetch; null means all
	// sort: String
	//		comma deliminated attribute string (preceded with optional +/- indicating ascending/descending)
	//		example:  label,type
	// condition: String
	//		condition expression string 
	//		example: description contains 'abcd'
	// sync: Boolean
	//		indicates if fetch is synchronous
	// version: Float

	var args = {
		onComplete: function(widgetGroups, widgetGroupCount, kwArgs){
		},
		onError: function(error, exception, ioArgs){				
		},
		baseUrl: null,
		registryLocation: (ibm.tivoli.navmodel.Utils.isTIP() ? "local" : null),
		provider: null,
		datasource: null,
		start: 0,
		count: null,
		sort: null,
		condition: null,
		sync: false,
		version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion()
	};
	dojo.mixin(args, kwArgs);

	var url = args.baseUrl + "/providers/" + encodeURIComponent(args.provider);
	url += (args.datasource ? "/datasources/" + encodeURIComponent(args.datasource) : "");
	url += "/widgets/groups";	
	
	var content = ibm.tivoli.navmodel.Utils.getContent();
	if (args.registryLocation && args.registryLocation === "local") {
		content.registryLocation = args.registryLocation;
	}
	if (args.start) {
		content.start = args.start;
	}
	if (args.count) {
		content.count = args.count;
	}
	if (args.sort) {
		content.sort = args.sort;
	}
	if (args.condition) {
		content.condition = args.condition;
	}
	
	return dojo.xhrGet({
		url: url,
		headers: {
			'Accept': ibm.tivoli.navmodel.Utils.getContentType(args.version, "widgetGroupList")			
		},
		handleAs: "json",
		sync: args.sync,
		content: content,
		load: dojo.hitch(this, function(result, ioArgs){
			var widgetGroups = [];			
			if (result.items) {
				var length = result.items.length;
				for (index = 0;  index < length;  ++index) {
					widgetGroups[index] = new ibm.tivoli.navmodel.WidgetGroupInfo({widgetGroup: result.items[index], version: args.version});
				}
			}
					
			kwArgs.onComplete(widgetGroups, result.filteredRows, kwArgs);										
		}),
		error: dojo.hitch(this, function(error, ioArgs){
			var exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "getWidgetGroups", ibm.tivoli.navmodel.WidgetDefUtils._trace, args.version);
			kwArgs.onError(error, exception, ioArgs);
		})
	});	
};

});

},
'ibm/usmi/widgets/TreeTopper':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define(["dojo/_base/kernel",
        "dojo/_base/declare",
        "dijit/_WidgetBase",
        "dijit/_TemplatedMixin",
        "dojo/topic",
        "dojo/text!./templates/TreeTopper.html",
        "dijit/_KeyNavContainer",
        "dijit/_Contained",
        // NextGen imports
        "ibm/usmi/widgets/TreeTopperLinks",
        "ibm/usmi/widgets/util/Constants"],
    function (dojo, declare, _WidgetBase, _TemplatedMixin, topic, TreeTopperTemplate, _KeyNavContainer, _Contained,
            TreeTopperLinks, Constants) {

    return declare([_WidgetBase, _TemplatedMixin, _KeyNavContainer, _Contained], {

        // Template info
        templateString: TreeTopperTemplate,

        // topper label
        label: "",

        // topper links
        _links : [],

        //Subscriptions to clean up
        _subscriptions : null,
		
		// workarea is ready
		_workAreaReady : false,

        constructor: function() {
			this._subscriptions = [];
			
            // listen for when Tree is in charge, and unselecto tree topper selection
            this._subscriptions.push(topic.subscribe("TREEINCHARGE", dojo.hitch(this, this.removeSelection)));
			
			// subscribe to workarea isReady event
 //           this._subscriptions.push(topic.subscribe(Constants.TOPIC_READY_WORKAREA, dojo.hitch(this, this.workAreaIsActive)));
			
			// subscrib to hash updates
			this._subscriptions.push(topic.subscribe(Constants.TOPIC_HASH_CHANGE, dojo.hitch(this, this._hashChanged)));
        },

        startup: function() {
            this.inherited(arguments);

            // initialize array
            this._links = [];

            // key nav container
            this.startupKeyNavChildren();
  
        },

        postCreate: function() {
            this.inherited(arguments);

            // key nav container
            this.connectKeyNavHandlers([dojo.keys.UP_ARROW],[dojo.keys.DOWN_ARROW]);
        },

        buildRendering: function() {
            this.inherited(arguments);
            this.titlePoint.innerHTML = this.label;
        },

        addTreeTopperLink: function(tLink) {
            this.addChild(tLink);
            this._links.push(tLink);
        },

        /*
         * Returns TreeTopperLinks widget.
         */
        getRoot: function(/*String*/ id) {
            var subset = dojo.filter(this.getChildren(), function(/*Widget*/ child) {
                return child.isInstanceOf(TreeTopperLinks) && child.rootId === id;
            }, this);
            return (subset && subset[0]) || null;
        },

        removeLink: function(/*String*/id) {
            var widget = this.getRoot(id);
            if(widget) {
                this.removeChild(widget);
            }
        },

        setLink: function(/*ibm.usmi.widgets.TreeTopperLinks*/link) {
            dojo.forEach(this.getChildren(), function(item) {
                item.select(false);
            });
            link.select(true);
        },

        removeSelection: function() {
            dojo.forEach(this.getChildren(), function(item) {
                if(item) {
                    item.select(false);
                }
            });
        },

        _hashChanged : function (/*dojo.hash*/hash) {
            dojo.forEach(this._links, function (link) {
                if (this.getUrlHashParm(ibm.usmi.widgets.util.Constants.HASH_TOPPER_SELECTION) === link.topperId) {
                    this.setLink(link);
                }
            }, this);
        },


        // workAreaIsActive : function () {
		//	when tree topper link is bookmarked or direct launched it publish retarget event before workArea is ready
			// dojo.forEach(this.getChildren(), function(item) {
				// if(item.selected) {
					// item._handClick();
				// }
            // });
			// this._workAreaReady = true;
        // },

        destroy : function () {
            var idx = 0;
            for (; idx < this._subscriptions.length; idx++) {
                this._subscriptions[idx].remove();
            }
            this._subscriptions.length = 0;
            this.inherited(arguments);
        }

    });
});
},
'ibm/tivoli/tip/dojo/Server':function(){
// wrapped by build app
define("ibm/tivoli/tip/dojo/Server", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/tip/dojo/util,ibm/tivoli/tip/dojo/_Server"], function(dijit,dojo,dojox){
/****************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* 5724-Q87 
*
* (C) Copyright IBM Corp. 2008
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
********************************************************** {COPYRIGHT-END} **/
dojo.require("ibm.tivoli.tip.dojo.util");
dojo.require("ibm.tivoli.tip.dojo._Server");

dojo.provide("ibm.tivoli.tip.dojo.Server");

ibm.tivoli.tip.dojo.Server.anyException  = "*"; 

ibm.tivoli.tip.dojo.Server._singleton    = null;

/****************************************************************************
* Get the "singleton" Server instance.
****************************************************************************/
/* "ibm.tivoli.tip.dojo._Server" */
ibm.tivoli.tip.dojo.Server.getSingleton = function( /* required: ibm.tivoli.tip.dojo._Global */ global )
{
  if ( global === undefined ) {
  	throw new Error( "ibm.tivoli.tip.dojo.Server requires a 'global' object reference" );
  }
  		
  //if the singleton doesn't exist yet --> create it now
  //there must be a unique singleton per global instance
  if ( global._serverSingleton === undefined || global._serverSingleton === null )
  {
    global._serverSingleton = new ibm.tivoli.tip.dojo._Server( { global : global } );
  }  
  
  return global._serverSingleton;	
};


});

},
'dojo/window':function(){
define(["./_base/lang", "./_base/sniff", "./_base/window", "./dom", "./dom-geometry", "./dom-style"],
	function(lang, has, baseWindow, dom, geom, style) {

// module:
//		dojo/window
// summary:
//		TODOC

var window = lang.getObject("dojo.window", true);

/*=====
dojo.window = {
	// summary:
	//		TODO
};
window = dojo.window;
=====*/

window.getBox = function(){
	// summary:
	//		Returns the dimensions and scroll position of the viewable area of a browser window

	var
		scrollRoot = (baseWindow.doc.compatMode == 'BackCompat') ? baseWindow.body() : baseWindow.doc.documentElement,
		// get scroll position
		scroll = geom.docScroll(), // scrollRoot.scrollTop/Left should work
		w, h;

	if(has("touch")){ // if(scrollbars not supported)
		var uiWindow = baseWindow.doc.parentWindow || baseWindow.doc.defaultView;   // use UI window, not dojo.global window. baseWindow.doc.parentWindow probably not needed since it's not defined for webkit
		// on mobile, scrollRoot.clientHeight <= uiWindow.innerHeight <= scrollRoot.offsetHeight, return uiWindow.innerHeight
		w = uiWindow.innerWidth || scrollRoot.clientWidth; // || scrollRoot.clientXXX probably never evaluated
		h = uiWindow.innerHeight || scrollRoot.clientHeight;
	}else{
		// on desktops, scrollRoot.clientHeight <= scrollRoot.offsetHeight <= uiWindow.innerHeight, return scrollRoot.clientHeight
		// uiWindow.innerWidth/Height includes the scrollbar and cannot be used
		w = scrollRoot.clientWidth;
		h = scrollRoot.clientHeight;
	}
	return {
		l: scroll.x,
		t: scroll.y,
		w: w,
		h: h
	};
};

window.get = function(doc){
	// summary:
	// 		Get window object associated with document doc

	// In some IE versions (at least 6.0), document.parentWindow does not return a
	// reference to the real window object (maybe a copy), so we must fix it as well
	// We use IE specific execScript to attach the real window reference to
	// document._parentWindow for later use
	if(has("ie") && window !== document.parentWindow){
		/*
		In IE 6, only the variable "window" can be used to connect events (others
		may be only copies).
		*/
		doc.parentWindow.execScript("document._parentWindow = window;", "Javascript");
		//to prevent memory leak, unset it after use
		//another possibility is to add an onUnload handler which seems overkill to me (liucougar)
		var win = doc._parentWindow;
		doc._parentWindow = null;
		return win;	//	Window
	}

	return doc.parentWindow || doc.defaultView;	//	Window
};

window.scrollIntoView = function(/*DomNode*/ node, /*Object?*/ pos){
	// summary:
	//		Scroll the passed node into view, if it is not already.

	// don't rely on node.scrollIntoView working just because the function is there

	try{ // catch unexpected/unrecreatable errors (#7808) since we can recover using a semi-acceptable native method
		node = dom.byId(node);
		var doc = node.ownerDocument || baseWindow.doc,
			body = doc.body || baseWindow.body(),
			html = doc.documentElement || body.parentNode,
			isIE = has("ie"), isWK = has("webkit");
		// if an untested browser, then use the native method
		if((!(has("mozilla") || isIE || isWK || has("opera")) || node == body || node == html) && (typeof node.scrollIntoView != "undefined")){
			node.scrollIntoView(false); // short-circuit to native if possible
			return;
		}
		var backCompat = doc.compatMode == 'BackCompat',
			clientAreaRoot = (isIE >= 9 && node.ownerDocument.parentWindow.frameElement)
				? ((html.clientHeight > 0 && html.clientWidth > 0 && (body.clientHeight == 0 || body.clientWidth == 0 || body.clientHeight > html.clientHeight || body.clientWidth > html.clientWidth)) ? html : body)
				: (backCompat ? body : html),
			scrollRoot = isWK ? body : clientAreaRoot,
			rootWidth = clientAreaRoot.clientWidth,
			rootHeight = clientAreaRoot.clientHeight,
			rtl = !geom.isBodyLtr(),
			nodePos = pos || geom.position(node),
			el = node.parentNode,
			isFixed = function(el){
				return ((isIE <= 6 || (isIE && backCompat))? false : (style.get(el, 'position').toLowerCase() == "fixed"));
			};
		if(isFixed(node)){ return; } // nothing to do

		while(el){
			if(el == body){ el = scrollRoot; }
			var elPos = geom.position(el),
				fixedPos = isFixed(el);

			if(el == scrollRoot){
				elPos.w = rootWidth; elPos.h = rootHeight;
				if(scrollRoot == html && isIE && rtl){ elPos.x += scrollRoot.offsetWidth-elPos.w; } // IE workaround where scrollbar causes negative x
				if(elPos.x < 0 || !isIE){ elPos.x = 0; } // IE can have values > 0
				if(elPos.y < 0 || !isIE){ elPos.y = 0; }
			}else{
				var pb = geom.getPadBorderExtents(el);
				elPos.w -= pb.w; elPos.h -= pb.h; elPos.x += pb.l; elPos.y += pb.t;
				var clientSize = el.clientWidth,
					scrollBarSize = elPos.w - clientSize;
				if(clientSize > 0 && scrollBarSize > 0){
					elPos.w = clientSize;
					elPos.x += (rtl && (isIE || el.clientLeft > pb.l/*Chrome*/)) ? scrollBarSize : 0;
				}
				clientSize = el.clientHeight;
				scrollBarSize = elPos.h - clientSize;
				if(clientSize > 0 && scrollBarSize > 0){
					elPos.h = clientSize;
				}
			}
			if(fixedPos){ // bounded by viewport, not parents
				if(elPos.y < 0){
					elPos.h += elPos.y; elPos.y = 0;
				}
				if(elPos.x < 0){
					elPos.w += elPos.x; elPos.x = 0;
				}
				if(elPos.y + elPos.h > rootHeight){
					elPos.h = rootHeight - elPos.y;
				}
				if(elPos.x + elPos.w > rootWidth){
					elPos.w = rootWidth - elPos.x;
				}
			}
			// calculate overflow in all 4 directions
			var l = nodePos.x - elPos.x, // beyond left: < 0
				t = nodePos.y - Math.max(elPos.y, 0), // beyond top: < 0
				r = l + nodePos.w - elPos.w, // beyond right: > 0
				bot = t + nodePos.h - elPos.h; // beyond bottom: > 0
			if(r * l > 0){
				var s = Math[l < 0? "max" : "min"](l, r);
				if(rtl && ((isIE == 8 && !backCompat) || isIE >= 9)){ s = -s; }
				nodePos.x += el.scrollLeft;
				el.scrollLeft += s;
				nodePos.x -= el.scrollLeft;
			}
			if(bot * t > 0){
				nodePos.y += el.scrollTop;
				el.scrollTop += Math[t < 0? "max" : "min"](t, bot);
				nodePos.y -= el.scrollTop;
			}
			el = (el != scrollRoot) && !fixedPos && el.parentNode;
		}
	}catch(error){
		console.error('scrollIntoView: ' + error);
		node.scrollIntoView(false);
	}
};

return window;
});

},
'ibm/usmi/util/BaseWidget':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

define("ibm/usmi/util/BaseWidget", [
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dojo/hash",
	"ibm/usmi/widgets/util/Constants"
],

function (dojo, declare, hash, Constants) {

	return declare([], {

		// private members
		// object to hold initial arguments, pass in via constructor, html tag, or url
		_initArgs : null,

		// common arguments
		providerID : "",
		collectionID : "",
		sourceToken : "",

		subscription : null,

		// update target method, subclasses override
		updateTargetMethod : "_updateTargets",

		constructor : function (/*constructor_kwArgs*/kwArgs) {

			this._initArgs = {
				sourceToken : null,
				collectionID : null,
				providerID : null
			};

			// mixin parameters that were passed in on the constructor, html tag attributes...
			dojo.mixin(this._initArgs, kwArgs);

			// mixin parameters that were specified on the URL, URL parms override all.
			dojo.mixin(this._initArgs, this.getUrlParmsObj());

			// mixin hash parameters
			dojo.mixin(this._initArgs, this.getUrlHashObj());

			// set provider, collection, source token passed in via constructor, html attribute, url
			this.providerID = this.getInitArg(Constants.PROVIDER);
			this.collectionID = this.getInitArg(Constants.COLLECTION);
			this.sourceToken = this.getInitArg(Constants.SOURCE_TOKEN);

			// Register for a re-target event
			this.subscription = dojo.subscribe(Constants.TOPIC_RETARGET, this, this.updateTargetMethod);
		},

		getInitArg : function (/*String*/key) {
			// summary:
			//        Returns a specific initial argument passed to the widget
			return this._initArgs[key];
		},

		getUrlParmsObj : function () {
			return dojo.queryToObject(dojo.doc.location.search.substr((dojo.doc.location.search[0] === "?" ? 1 : 0)));
		},

		getUrlParm : function (/*String*/key) {
			var urlObj = this.getUrlParmsObj();
			return urlObj[key];

		},

		getUrlHashObj : function () {
			return dojo.queryToObject(hash());
		},

		getUrlHashParm : function (/*String*/key) {
			var hashObj = this.getUrlHashObj();
			return hashObj[key];
		},

		setUrlHashObj : function (hashObj) {
			dojo.hash(dojo.objectToQuery(hashObj));
		},

		setUrlHash : function (/*String*/hashString, /*boolean*/replace) {
			dojo.hash(hashString, replace);
		},

		updateUrlHash : function (/*String*/hashkey, /*String*/hashValue) {
			var obj = this.getUrlHashObj(); // get
			obj[hashkey] = hashValue;
			return dojo.objectToQuery(obj); // set
		},

		//accept an object, take all the key/values and append/replace to current hash, return the string
		//doesn't actually call dojo.hash, just gives back the updated string
		updateHashValues : function (/*Object*/hashObject) {
			var key, obj = this.getUrlHashObj();
			for (key in hashObject) {
				obj[key] = hashObject[key];
			}
			return dojo.objectToQuery(obj);
		},
		
		//accept an object, take all the key/values and append/replace to current hash, return the string
		//doesn't actually call dojo.hash, just gives back the updated string
		updateHashObjectValues : function (/*Object*/hashObject) {
			var key, obj = this.getUrlHashObj();
			for (key in hashObject) {
				obj[key] = hashObject[key];
			}
			return obj;
		},

		_updateTargets : function (/*ibm.tivoli.navmodel.Dataset*/dataset, /*ibm.tivoli.navmodel.Item[]*/items) {
			console.log("Base Widget _updateTargets called, no subclass is listening");
		},
		destroy : function () {
			if (this.subscription) {
				this.subscription.remove();
				this.subscription = null;
			}
			this.inherited(arguments);
		}
	});
});

},
'url:dojoe/treetablex/templates/TreeTable.html':"\r\n<div class=\"tivTreeTable\" tabIndex=0 aria-label=\"tree table\" role=\"application\">\r\n\t<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\"  style=\"width:100%;height:100%;\"  role=\"presentation\">\t\t\r\n\t\t<tr valign=\"center\">\r\n\t\t\t<td width=\"100%\" role=\"toolbar\">\r\n\t\t\t\t<div dojoAttachPoint=\"wipeNode\">\r\n\t\t\t\t\t<div dojoAttachPoint=\"toolbarNode\"></div>\r\n\t\t\t\t</div>\r\n\t\t\t</td>\r\n\t\t</tr>\r\n\t\t<tr valign=\"center\">\r\n\t\t\t<td width=\"100%\" role=\"treegrid\">\r\n\t\t\t\t<div dojoAttachPoint=\"treeGridNode\"></div>\t\r\n\t\t\t</td>\r\n\t\t</tr>\t\t\r\n\t</table>\r\n</div>\t",
'dojox/string/tokenize':function(){
define([
	"dojo/_base/lang",
	"dojo/_base/sniff"	
], function(lang, has){
	var tokenize = lang.getObject("dojox.string", true).tokenize;

	tokenize = function(/*String*/ str, /*RegExp*/ re, /*Function?*/ parseDelim, /*Object?*/ instance){
		// summary:
		//		Split a string by a regular expression with the ability to capture the delimeters
		// parseDelim:
		//		Each group (excluding the 0 group) is passed as a parameter. If the function returns
		//		a value, it's added to the list of tokens.
		// instance:
		//		Used as the "this" instance when calling parseDelim
		var tokens = [];
		var match, content, lastIndex = 0;
		while(match = re.exec(str)){
			content = str.slice(lastIndex, re.lastIndex - match[0].length);
			if(content.length){
				tokens.push(content);
			}
			if(parseDelim){
				if(has("opera")){
					var copy = match.slice(0);
					while(copy.length < match.length){
						copy.push(null);
					}
					match = copy;
				}
				var parsed = parseDelim.apply(instance, match.slice(1).concat(tokens.length));
				if(typeof parsed != "undefined"){
					tokens.push(parsed);
				}
			}
			lastIndex = re.lastIndex;
		}
		content = str.slice(lastIndex);
		if(content.length){
			tokens.push(content);
		}
		return tokens;
	};
	return tokenize;
});

},
'dojox/mvc':function(){
define(["./mvc/_base"], function(dxmvc){
	// module:
	//		dojox/mvc
	// summary:
	//		Adds elements of MVC support to Dojo.

	return dxmvc;
});

},
'dojoe/treetable/TreeTableAll':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!dojoe/treetable/_QuickSearchFilter,dojoe/treetable/_ConfigureColumnsDialog,dojoe/treetable/_Toolbar,dojoe/treetable/TreeTable"], function(dijit,dojo,dojox){
dojo.provide("dojoe.treetable.TreeTableAll");
dojo.require("dojoe.treetable._QuickSearchFilter" );
dojo.require("dojoe.treetable._ConfigureColumnsDialog" );  
dojo.require("dojoe.treetable._Toolbar" );
dojo.require("dojoe.treetable.TreeTable" );    


});

},
'ibm/usmi/statuspod/AlertStatusPodMenuItem':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define([
	"dojo/_base/declare",
	"./StatusPodMenuItem"
], function (declare, StatusPodMenuItem) {
	return declare([ StatusPodMenuItem
	], {
		onClick : function (/*Event*/) {
			//var currentLocation = dojo.doc.location.href.substr(0, dojo.doc.location.href.indexOf("?"));
			launchNextGenPage("workAreaDijit=ibm/usmi/widgets/WorkArea&sourceToken=" + this.objectId
					+ "&selector=details&taskSelection=ibm/usmi/activealerts/ActiveAlerts");
			/*var URL = currentLocation + "#workAreaDijit=ibm/usmi/widgets/WorkArea&sourceToken=" + this.objectId
					+ "&selector=details&taskSelection=ibm/usmi/activealerts/ActiveAlerts";
			this._logger.finest("launch alert status pod item URL:" + URL, "onClick");
			var left = (screen.width) / 2, top = (screen.height) / 2, params = 'top=' + top + ', left=' + left;
			window.open(URL, "_self", params);*/

		}
	});
});

},
'ibm/tivoli/navmodel/TableWidgetDef':function(){
// wrapped by build app
define("ibm/tivoli/navmodel/TableWidgetDef", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/WidgetDef"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.TableWidgetDef"); 
 
// includes
dojo.require("ibm.tivoli.navmodel.WidgetDef");

////////////////////////////////////
// TableWidgetDef
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.TableWidgetDef", ibm.tivoli.navmodel.WidgetDef, {

	/*=====
	constructor_kwArgs: function(baseUrl, provider, datasource, widget, definition, type, version) {
		// baseUrl: String
		//		base url for identifying rest service
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// widget: String
		//		widget type in form <type>-<version>
		// definition: String
		//		widget definition id in form <id>-<version>
		// type: String
		//		definition type:  "user", "shared", or "base"
		// version: Float
		//		version used to communicate with rest service
	},
	=====*/ 
	
	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs TopoWidgetDef object.
	},

	/////////////////////////////
	// public methods
	/////////////////////////////

	destroy: function() {
		// summary:
		//		Do any necessary clean up 
		if (!this.isDestroyed()) {
			// call base class
			this.inherited(arguments);
		}
	},
		
	/////////////////////////////
	// dataset section data
	/////////////////////////////	
	getDataSet: function() {
		if (!this._widgetDef.datasets) {
			this._widgetDef.datasets = [];
		}
		
		if (this._widgetDef.datasets.length === 0) {
			this._widgetDef.datasets[0] = {};
		}
		
		return this._widgetDef.datasets;		
	},

	getDataSetID: function() {
		var datasets = this.getDataSet();
		return datasets[0].id;
	},

	getDataSetLabelKey: function() {
		var datasets = this.getDataSet();
		return datasets[0].label.key;
	},
	
	getDataSetLabelValue: function() {
		var datasets = this.getDataSet();
		return datasets[0].label.value;
	},
	
	getDataSetQuery: function() {
		var datasets = this.getDataSet();
		return datasets[0].query;
	},
	
	getDataSetColumns: function() {
		var datasets = this.getDataSet();
		return datasets[0].columns;		
	},
	
	getDataSetThresholds: function(){
		var datasets = this.getDataSet();
		return datasets[0].thresholds;
	},

	getDataSetParameters: function() {
		var datasets = this.getDataSet();
		return datasets[0].parameters;
	},
	
	getDataSetResourceBundle: function() {
		var datasets = this.getDataSet();
		return datasets[0].label.resourceBundle;
	},
	
	
	getDataSetDataSource: function() {
		var datasets = this.getDataSet();
		return datasets[0].datasource;
	},
	
	getDataSetDatasourceType: function() {
		var datasets = this.getDataSet();
		datasets[0].datasourceType = type;
	},
	
	setDataSet: function(datasets) {
		this._widgetDef.datasets = datasets;		
	},

	setDataSetID: function(id) {
		var datasets = this.getDataSet();
		datasets[0].id = id;
	},

	setDataSetLabelKey: function(key) {
		var datasets = this.getDataSet();
		datasets[0].label.key = key;
	},
	
	setDataSetLabelValue: function(value) {
		var datasets = this.getDataSet();
		datasets[0].label.value = value;
	},
	
	setDataSetQuery: function(query) {
		var datasets = this.getDataSet();
		datasets[0].query = query;
	},
	
	setDataSetColumns: function(columns) {
		var datasets = this.getDataSet();
		datasets[0].columns = columns;
	},

	setDataSetParameters: function(parameters) {
		var datasets = this.getDataSet();
		datasets[0].parameters = parameters;
	},
	
	setDataSetResourceBundle: function(bundle) {
		var datasets = this.getDataSet();
		datasets[0].label.resourceBundle = bundle;
	},
	
	/////////////////////////////
	// table section data
	/////////////////////////////	
	getTable: function() {
		if (!this._widgetDef.table) {
			this._widgetDef.table = {};
		}
		
		return this._widgetDef.table;		
	},
	
	getTableID: function() {
		var table = this.getTable();
		return table.id;		
	},
	
	getTableView: function() {
		var table = this.getTable();
		if (!table.view) {
			table.view = [];
		}
		
		return table.view;		
	},
	
	getTableViewElement: function(id) {
		var entry = null;
		var view = this.getTableView();
		if (view) {
			var length = view.length;
			for (var index = 0;  index < length;  ++index) {
				if (view[index].id === id) {
					entry = view[index];
					break;
				}
			}
		}
		
		return entry;
	},
	
	getTableCommon: function() {
		var table = this.getTable();
		return table.common;		
	},

	getTableCommonAutoUpdate: function() {
		var table = this.getTable();
		return table.common.autoUpdate;		
	},
	
	getTableCommonShowLastUpdateTime: function() {
		var table = this.getTable();
		return table.common.showLastUpdateTime;		
	},
	
	getTableCommonPaging: function() {
		var table = this.getTable();
		return table.common.paging;		
	},
	
	getTableCommonPagingSize: function() {
		var table = this.getTable();
		return table.common.pagingSize;		
	},
	
	getTableShowAdvancedFilter: function() {
		var table = this.getTable();
		return table.showAdvancedFilter;
	},
	
	getTableShowQuickFilter: function() {
		var table = this.getTable();
		return table.showQuickFilter;
	},
	
	getTableShowClearFilterConfirmation: function() {
		var table = this.getTable();
		return table.showClearFilterConfirmation;
	},
		
	getTableFilter: function() {
		var table = this.getTable();
		return table.filter;
	},
	
	getTableLabelKey: function() {
		var table = this.getTable();
		if (!table.label) {
			table.label = {};
		}
		
		return table.label.key;		
	},
	
	getTableLabelValue: function() {
		var table = this.getTable();
		if (!table.label) {
			table.label = {};
		}
		
		return table.label.value;		
	},
	
	getTableDescriptionKey: function() {
		var table = this.getTable();
		if (!table.description) {
			table.description = {};
		}
		
		return table.description.key;		
	},
	
	getTableDescriptionValue: function() {
		var table = this.getTable();
		if (!table.description) {
			table.description = {};
		}
		
		return table.description.value;		
	},
	
	getTableColumnReordering: function() {
		var table = this.getTable();
		return table.columnReordering;		
	},

	setTable: function(table) {
		this._widgetDef.table = table;		
	},
	
	setTableID: function(id) {
		var table = this.getTable();
		table.id = id;		
	},
	
	setTableView: function(view) {
		var table = this.getTable();
		table.view = view;
	},
	
	setTableCommon: function(common) {
		var table = this.getTable();
		table.common = common;		
	},

	setTableCommonAutoUpdate: function(autoUpdate) {
		var table = this.getTable();
		table.common.autoUpdate = autoUpdate;		
	},
	
	setTableCommonShowLastUpdateTime: function(showLastUpdateTime) {
		var table = this.getTable();
		table.common.showLastUpdateTime = showLastUpdateTime;		
	},
	
	setTableCommonPaging: function(paging) {
		var table = this.getTable();
		table.common.paging = paging;		
	},
	
	setTableCommonPagingSize: function(pagingSize) {
		var table = this.getTable();
		table.common.pagingSize = pagingSize;		
	},
	
	setTableShowAdvancedFilter: function(showAdvancedFilter) {
		var table = this.getTable();
		table.showAdvancedFilter = showAdvancedFilter;
	},
	
	setTableFilter: function(filter) {
		var table = this.getTable();
		table.filter = filter;
	},
	
	setTableLabelKey: function(key) {
		var table = this.getTable();
		if (!table.label) {
			table.label = {};
		}
		table.label.key = key;		
	},
	
	setTableLabelValue: function(value) {
		var table = this.getTable();
		if (!table.label) {
			table.label = {};
		}
		table.label.value = value;		
	},
	
	setTableDescriptionKey: function(key) {
		var table = this.getTable();
		if (!table.description) {
			table.description = {};
		}
		table.description.key = key;		
	},
	
	setTableDescriptionValue: function(value) {
		var table = this.getTable();
		if (!table.description) {
			table.description = {};
		}
		table.description.value = value;		
	},
	
	setTableColumnReordering: function(columnReordering) {
		var table = this.getTable();
		table.columnReordering = columnReordering;		
	}
});

});

},
'ibm/usmi/widgets/table/DefaultItemsTableViewPersist':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/widgets/table/DefaultItemsTableViewPersist", [
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/_base/json",
	"dojo/_base/unload",
	"dojo/cookie",
	"ibm/usmi/core/utils/Logger",
	"gridx/modules/Persist",
	"ibm/usmi/core/utils/_UsmiBrowserPersistence"
], function (declare, array, lang, json, unload, cookie, Logger, Persist, _UsmiBrowserPersistence) {

	return declare([
		Persist,
		_UsmiBrowserPersistence
	], {

		put : function (key, value, options) {
			var newkey = key + this.model.store.declaredClass;
			// only persist if there are columns in the table...  helps avoids empty table persistence
			if (value && value.column && value.column.length > 0) {
				this.setGlobalData(newkey, value);
			}
		},

		get : function (key) {
			var newkey = key + this.model.store.declaredClass;
			return this.getGlobalData(newkey);
		},
		
		//For column state restore---------------------
		_restoreColumnState: function(){
			var t = this,
				grid = t.grid,
				col, cols = [],
				found=false,
				columns = t.registerAndLoad('column', t._columnStateSaver, t);
			if(!lang.isArray(columns)){
					return;
			}
			
			//1. we do not change the order of the columns as current structure, columns chooser can sort the order
			//2. we need to set the col width via the width in persist storage
			cols=array.map(this.grid._columns, function(colInGrid){
				col=colInGrid;
				array.some(columns, function(colInPersist){
					if(colInPersist.id === col.id){
						col.declaredWidth=col.width = colInPersist.width;
						return true;
					}
				});
				return col;
			});
	
			grid.setColumns(cols);
		}

	});
});
},
'dojox/data/dom':function(){
define(["dojo/_base/kernel", "dojo/_base/lang", "dojox/xml/parser"], 
  function(kernel, lang, xmlParser) {

//DOM type to int value for reference.
//Ints make for more compact code than full constant names.
//ELEMENT_NODE                  = 1;
//ATTRIBUTE_NODE                = 2;
//TEXT_NODE                     = 3;
//CDATA_SECTION_NODE            = 4;
//ENTITY_REFERENCE_NODE         = 5;
//ENTITY_NODE                   = 6;
//PROCESSING_INSTRUCTION_NODE   = 7;
//COMMENT_NODE                  = 8;
//DOCUMENT_NODE                 = 9;
//DOCUMENT_TYPE_NODE            = 10;
//DOCUMENT_FRAGMENT_NODE        = 11;
//NOTATION_NODE                 = 12;

//This file contains internal/helper APIs as holders for people who used them.  They have been migrated to
//a better project, dojox.xml and experimental has been removed there.  Please update usage to the new package.
dojo.deprecated("dojox.data.dom", "Use dojox.xml.parser instead.", "2.0");

var dataDom = lang.getObject("dojox.data.dom",true);

dataDom.createDocument = function(/*string?*/ str, /*string?*/ mimetype){
	//	summary:
	//		cross-browser implementation of creating an XML document object.
	//
	//	str:
	//		Optional text to create the document from.  If not provided, an empty XML document will be created.
	//		If str is empty string "", then a new empty document will be created.
	//	mimetype:
	//		Optional mimetype of the text.  Typically, this is text/xml.  Will be defaulted to text/xml if not provided.
	dojo.deprecated("dojox.data.dom.createDocument()", "Use dojox.xml.parser.parse() instead.", "2.0");
	try{
		return xmlParser.parse(str,mimetype); //DOMDocument.
	}catch(e){
		/*Squeltch errors like the old parser did.*/
		return null;
	}
};

dataDom.textContent = function(/*Node*/node, /*string?*/text){
	//	summary:
	//		Implementation of the DOM Level 3 attribute; scan node for text
	//	description:
	//		Implementation of the DOM Level 3 attribute; scan node for text
	//		This function can also update the text of a node by replacing all child
	//		content of the node.
	//	node:
	//		The node to get the text off of or set the text on.
	//	text:
	//		Optional argument of the text to apply to the node.
	dojo.deprecated("dojox.data.dom.textContent()", "Use dojox.xml.parser.textContent() instead.", "2.0");
	if(arguments.length> 1){
		return xmlParser.textContent(node, text); //string
	}else{
		return xmlParser.textContent(node); //string
	}
};

dataDom.replaceChildren = function(/*Element*/node, /*Node || array*/ newChildren){
	//	summary:
	//		Removes all children of node and appends newChild. All the existing
	//		children will be destroyed.
	//	description:
	//		Removes all children of node and appends newChild. All the existing
	//		children will be destroyed.
	// 	node:
	//		The node to modify the children on
	//	newChildren:
	//		The children to add to the node.  It can either be a single Node or an
	//		array of Nodes.
	dojo.deprecated("dojox.data.dom.replaceChildren()", "Use dojox.xml.parser.replaceChildren() instead.", "2.0");
	xmlParser.replaceChildren(node, newChildren);
};

dataDom.removeChildren = function(/*Element*/node){
	//	summary:
	//		removes all children from node and returns the count of children removed.
	//		The children nodes are not destroyed. Be sure to call dojo._destroyElement on them
	//		after they are not used anymore.
	//	node:
	//		The node to remove all the children from.
	dojo.deprecated("dojox.data.dom.removeChildren()", "Use dojox.xml.parser.removeChildren() instead.", "2.0");
	return dojox.xml.parser.removeChildren(node); //int
};

dataDom.innerXML = function(/*Node*/node){
	//	summary:
	//		Implementation of MS's innerXML function.
	//	node:
	//		The node from which to generate the XML text representation.
	dojo.deprecated("dojox.data.dom.innerXML()", "Use dojox.xml.parser.innerXML() instead.", "2.0");
	return xmlParser.innerXML(node); //string||null
};

return dataDom;

});


},
'ibm/tivoli/tip/navwidget/table/dijit/RelationshipTableViewSingleLink':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!ibm/tivoli/tip/navwidget/dijit/NavWidgetObjectPluggableView,ibm/tivoli/navmodel/DatasetRelationshipsStore,ibm/tivoli/tip/navwidget/table/dijit/RelationshipsTableView,ibm/tivoli/tip/navwidget/table/dijit/NavModelTable"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide ("ibm.tivoli.tip.navwidget.table.dijit.RelationshipTableViewSingleLink");
dojo.require("ibm.tivoli.tip.navwidget.dijit.NavWidgetObjectPluggableView");

dojo.require("ibm.tivoli.navmodel.DatasetRelationshipsStore");
dojo.require("ibm.tivoli.tip.navwidget.table.dijit.RelationshipsTableView");
dojo.require("ibm.tivoli.tip.navwidget.table.dijit.NavModelTable");

ibm.tivoli.tip.navwidget.table.dijit.RelationshipTableViewSingleLink._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.tip.navwidget.table.dijit.RelationshipTableViewSingleLink");
dojo.declare ("ibm.tivoli.tip.navwidget.table.dijit.RelationshipTableViewSingleLink",
	ibm.tivoli.tip.navwidget.table.dijit.RelationshipsTableView,
	{ 		
//dojo.declare ("ibm.tivoli.tip.navwidget.table.dijit.RelationshipTableViewSingleLink",null,
//	{ 
	// summary:
	//		Coming soon
	// description:
	//		Coming soon

	//BEGIN  "NEW" stuff that differs from LinkTableView
	// _trace: [private] Trace
	//		Utility for trace and logging
	_trace: ibm.tivoli.tip.navwidget.table.dijit.RelationshipTableViewSingleLink._trace,
	constructor: function () {	
	
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
	//	this._global = ibm.tivoli.tip.dojo.Global.setSingleton( "666", " ");
		this.title = this.commonResources.RELATIONSHIP_VIEW;
		this.removeContextMenu = true;
		this.width = "99%";
		this.height = "50%";
	},

	_createTableStore: function(columnIds){		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// columnIds:
		//		
		
		// create or update store

		if(!this._dataStore && this.dataset) {		
			this._dataStore = new ibm.tivoli.navmodel.DatasetRelationshipsStore({dataset:this.dataset, defaultProperties:columnIds, links:this.links, returnItem:true});			
			this.inherited(arguments);
		}
	},	

	_setObjectsLoadData: function(/*ibm.tivoli.navmodel.Item[]*/ items, /*ibm.tivoli.navmodel.Links[]*/ links, /*ibm.tivoli.navmodel.Relationships[]*/ relationships){
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// items:
		//		
		// links:
		//		
		// relationships:
		//		

		if (links && links.length > 0){
			this.links = links;
			// OK, we have the link(s), so go ahead and create the table
			this._buildTDLTable();
			
			// fire _onSetDatasetLoadComplete
			setTimeout(dojo.hitch(this, this._onSetObjectsLoadComplete, items, links, relationships), 0);
		}	
		
		this.inherited(arguments);
	},	

	_setDatasetLoadData: function(dataset){			
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// dataset:
		//		
		if (this._trace.isFinest()) {
			this._trace.entering("setting dataset for " + this.id, "_setDatasetLoadData");
		}
		
		this.dataset = dataset;
		
		if(!this._tableWidgetDefinition) {
			this._createTableDef();
		}
		// LinksTableView calls buildTDLTable here. We can't because the view has not yet been informed
		// which object (link) has been selected. So set the dataset and create the widgetdef and go on.
		// SetObjects will be called explicitly by the creator of this object when it gets the link object(s) 
		
		if (!this.dataset) {
			//No dataset, so fire _onSetDatasetLoadComplete
			setTimeout(dojo.hitch(this, this._onSetDatasetLoadComplete, dataset), 0);
		}
		
		if (this._trace.isFinest()) {
			this._trace.exiting("setting dataset for " + this.id, "_setDatasetLoadData");
		}

	},
	_processDoubleClick: function(evt) {	
		//Single Link table should NOT take any action on double click
		//OVERRIDDING double click listener from abstact table view to ignore the event.
	}

});
});

},
'idx/oneui/Header':function(){
/*
 * Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

define("idx/oneui/Header", ["dojo/_base/array",
        "dojo/_base/declare",
		"dojo/_base/lang",
		"dojo/_base/window",
		"dojo/aspect",
		"dojo/dom-attr",
		"dojo/dom-class",
		"dojo/dom-construct",
		"dojo/dom-style",
		"dojo/i18n",
		"dojo/keys",
		"dojo/string",
		"dijit/_base/popup",
		"dijit/place",
		"dijit/registry",
		"dijit/_Widget",
		"dijit/_TemplatedMixin",
		"dojo/i18n!./nls/Header" ],
        function(_array,
		         declare,
				 _lang,
				 _window,
				 aspect,
				 domattr,
				 domclass,
				 domconstruct,
				 domstyle,
				 i18n,
				 keys,
				 string,
				 popup,
				 place,
				 registry,
				 _Widget,
				 _TemplatedMixin){
		
	// ensure we're not relying on the old globals, ready for 2.0
	var dojo = {}, dijit = {};
	
	// these widgets will be loaded later if needed
	var Button = function(){ log.error("dijit/form/Button has been used without being loaded"); }
	var FinderWidget = function(){ log.error("ibm/usmi/widgets/finder/Finder has been used without being loaded"); }	
	var TextBox = function(){ log.error("dijit/form/TextBox has been used without being loaded"); }
	var MenuTabController = function(){ log.error("idx/oneui/layout/MenuTabController has been used without being loaded"); }

	var prepareMenu = function(menu, cssclasses, trigger, around, trailing){
		// prepare a menu or menu bar to be presented from the One UI header widget.
		//
		// if cssclasses (array of strings) is specified, the open logic of the
		// menu is overridden to ensure that if it is used as a popup then there
		// is an outer wrapper DIV element always in place carrying the specified
		// CSS class and containing the popup DOM elements. The menu is recursively
		// processed to ensure this is true of all cascaded/popup menu items
		// within the menu. Successive elements of the array are applied to each
		// successive level of cascaded menu, and the last element is applied to 
		// all subsequent cascade levels.
		//
		// if trigger (DOM Node) is specified, the menu is assumed to be a popup
		// or cascaded menu, and is bound to the trigger.
		//
		// if around (DOM Node) is specified, the open logic of the popup is
		// overridden to position the popup to that node.
		
		if(cssclasses){
			if(cssclasses[0]){
				aspect.after(menu, "onOpen", function(){
					if(menu._popupWrapper){
						if(!menu._oneuiWrapper){
							// Create another wrapper <div> for our outermost oneui marker class.
							menu._oneuiWrapper = domconstruct.create("div", { "class": "idxHeaderContainer " + cssclasses[0] }, _window.body());

							aspect.after(menu, "destroy", function(){
								domconstruct.destroy(menu._oneuiWrapper);
								delete menu._oneuiWrapper;
							});
						}
						
						menu._oneuiWrapper.appendChild(menu._popupWrapper);
					}
				});
			}
			
			var nextcssclasses = (cssclasses.length > 1) ? cssclasses.slice(1) : cssclasses;
			_array.forEach(menu.getChildren(), function(child){
				if(child.popup){				
					prepareMenu(child.popup, nextcssclasses);
				}
				if(child.currentPage){
					domclass.add(child.domNode, "idxHeaderNavCurrentPage");
				}
			});
		}
		
		if(around){
			var _around = around;
			menu._scheduleOpen = function(/*DomNode?*/ target, /*DomNode?*/ iframe, /*Object?*/ coords){
				if(!this._openTimer){
					var ltr = menu.isLeftToRight(),
						where = place.around(//placeOnScreenAroundElement(
							popup._createWrapper(menu),
							_around,
							/*(menu.isLeftToRight() == (trailing ? false : true)) ? 
								{'BL':'TL', 'BR':'TR', 'TL':'BL', 'TR':'BR'} :
								{'BR':'TR', 'BL':'TL', 'TR':'BR', 'TL':'BL'},*/
							trailing ?
								[ "below-alt", "below", "above-alt", "above" ] : 
								[ "below", "below-alt", "above", "above-alt" ],
							ltr,
							menu.orient ? _lang.hitch(menu, "orient") : null);
						
					if(!ltr){
						where.x = where.x + where.w;
					}
					
					this._openTimer = setTimeout(_lang.hitch(this, function(){
						delete this._openTimer;
						this._openMyself({
							target: target,
							iframe: iframe,
							coords: where
						});
					}), 1);
				}
			}

			menu.leftClickToOpen = true;
		
			if(trigger){
				menu.bindDomNode(trigger);
			}
		}
	}
	
	/**
	 * Creates a new idx.oneui.Header
	 * @name idx.oneui.Header
	 * @class The Header widget generates the HTML and CSS to provide an
	 * IBM One UI header according to the design specification and templates.
	 * <p>
	 * To construct a header, initialise the widget with the required
	 * properties. The appropriate HTML and CSS is created immediately, and
	 * subsidiary dijit components may be created and marshalled. No dynamic
	 * layout is performed: once the HTML has been injected into the DOM,
	 * all layout is delegated to the renderer and associated CSS rules.
	 * </p>
	 * @augments dijit._Widget
	 * @augments dijit._TemplatedMixin
	 * @example
	 * var hdr = new idx.oneui.Header({ primaryTitle: "Hello" }, "myHeader");
	 */
	declare("idx.oneui.Header", [_Widget, _TemplatedMixin], 
	/** @lends idx.oneui.Header.prototype */
	{
		/**
		 * The IBM Brand/product name.
		 * @type string
		 */
		primaryTitle: "",
		
		/**
		 * make the primary title clickable by providing a primary title link
		 */
		primaryTitleLink: "",
		
		/**
		 * The desired style of primary (black) banner: "thick" or "thin".
		 * @type string
		 * @default "thin"
		 */
		primaryBannerType: "thin",
		
		/**
		 * A menu bar, which can contain items and popup menu items, which
		 * will be displayed as navigation actions/menus in the header. The
		 * menu bar may be supplied as an instance or by id or as a DOM node.
		 * @type string | dijit.MenuBar | DOMNode
		 */
		navigation: undefined,
		
		//jlc
		//  secondaryNavigation: String | dijit.MenuBar | DOMNode
		//      A menu bar, which can contain items and popup menu items, which
		//      will be displayed as navigation actions/menus in the second header. The
		//      menu bar may be supplied as an instance or by id or as a DOM node.
		secondaryNavigation: undefined,
		
		/**
		 * True (the default) if navigation menu items that have a popup
		 * menu associated with them are to show a drop-down arrow affordance.
		 * If false, drop-down arrows are not shown on navigation items.
		 * @type boolean
		 */
		showNavigationDropDownArrows: true,

		/**
		 * True (the default) if navigation menu items that have a popup
		 * menu associated with them are to show a drop-down arrow affordance.
		 * If false, drop-down arrows are not shown on navigation items.
		 * @type boolean
		 */
		showSecondaryNavigationDropDownArrows: true,
		
		/**
		 * Specifies that a primary search box should be included in the
		 * header, and supplies the parameters for it. All the properties are
		 * optional:
		 * <ul>
		 * <li>
		 * entryPrompt: {string | function} A string containing the prompt
		 * text for entering the search terms, or a function (which will be
		 * called with no arguments) which returns the prompt text.
		 * </li>
		 * <li>
		 * submitPrompt: {string | function} A string containing the prompt
		 * text for submitting the search, or a function (which will be called
		 * with no arguments) which returns the prompt text.
		 * </li>
		 * <li>onChange: {function} A function which will be called whenever
		 * the text in the search box changes. The function will receive one
		 * argument, which is the text currently in the search box.
		 * </li>
		 * <li>
		 * onSubmit: {function} A function which will be called whenever
		 * the user submits a search (eg, by pressing enter, or activating a
		 * search affordance). The function will receive one argument, which
		 * is the text currently in the search box.
		 * </li>
		 * </ul>
		 * @type Object
		 */
		primarySearch: undefined,
		
		/**
		 * The identity of the user to be included in the header. All
		 * properties are optional.
		 * <ul>
		 * <li>
		 * displayName: {string | function} A string containing the displayable
		 * name of the current user, or a function (which will be called with
		 * no arguments) which returns the displayable name of the current
		 * user. The displayable name may include mark-up (for example,
		 * entities for accented characters, etc). A displayName should always
		 * be supplied whenever feedback of the user's identity is required.
		 * The displayName can be modified after construction by setting the
		 * "userDisplayName" property of the header.
		 * Examples: "Clark, D. J. (Dave)"
		 * </li>
		 * <li>
		 * displayImage: {string | Object | function} A string containing the
		 * URI of an image to be displayed alongside the user name or welcome
		 * message, or an HTML image object (or other suitable mark-up object)
		 * to be used as the image alongside the user name or welcome message,
		 * or a function (which will be called with no arguments) which returns
		 * either a string or an object to specify the image to use. If omitted,
		 * null or undefined, no image is shown. The displayImage can be
		 * modified after construction by setting the "userDisplayName" property
		 * of the header.
		 * </li>
		 * <li>
		 * messageName: {string | function} A string containing the displayable
		 * name of the current user as it should appear in the message shown
		 * in the header to confirm the user's identity, if that is different
		 * from the displayName (for example, a shortened or simplified form
		 * of the user's name might be used as the messageName). If messageName
		 * is not supplied, the displayName is used. Note that displayName
		 * should still be supplied as well as messageName, because although it
		 * is the messageName that is substituted into the message for display,
		 * the displayName is also added as alternative text/title to add clarity
		 * for the user. The messageName can be set or modified after construction
		 * by setting the "userMessageName" property of the header.
		 * Examples: "Dave", "No&euml;l"
		 * </li>
		 * <li>
		 * message: {string | function} A string containing the message to be
		 * shown in the header to confirm the current user's identity, or a
		 * function (which will be called with no arguments) which returns
		 * the message to be shown. The string pattern will have the following
		 * substitutions applied:
		 * 	<ul>
		 * 	<li>
		 * 	${messageName} - the message name of the current user, if supplied,
		 * 	othewise the display name is used, if supplied
		 * 	</li>
		 * 	<li>
		 * 	$(displayName} - the display name of the current user
		 * 	</li>
		 * 	</ul>
		 * The message may include mark-up (for example, entities for accented
		 * characters, etc). If message is not supplied, the message that is
		 * used is "${messageName}". The message can be set or modified after
		 * construction by setting the "userMessage" property of the header.
		 * Examples: "Welcome back, ${messageName}",
		 * "Welcome, new user"
		 * </li>
		 * <li>
		 * actions: {string | dijit.Menu | dijit.MenuItem} A dijit.Menu to be
		 * used as the popup of available actions for the current user. If a
		 * single dijit.MenuItem is supplied, the current user name will be
		 * presented as a simple action and onClick will be triggered on the
		 * menu item when that action is selected. The menu or item may be
		 * supplied as an instance or by id.
		 * </li>
		 * </ul>
		 * @type Object
		 */
		user: undefined,
		
		/**
		 * True (the default) if a drop-down arrow affordance is to be shown
		 * on the user identification when a popup menu of user actions is supplied.
		 * If false, a drop-down arrow is not shown on the user identification.
		 * @type boolean
		 */
		showUserDropDownArrow: true,

		/**
		 * A dijit.Menu to be used as the popup of available site settings
		 * actions. If a single dijit.MenuItem is supplied, a simple site
		 * settings action will be presented and onClick will be triggered on
		 * the menu item when that action is selected.
		 * @type dijit.Menu | dijit.MenuItem
		 */
		settings: undefined,
	
		/**
		 * True (the default) if a drop-down arrow affordance is to be shown on the
		 * site settings icon when a popup menu of site settings items is supplied.
		 * If false, a drop-down arrow is not shown on the site settings icon.
		 * @type boolean
		 */
		showSettingsDropDownArrow: true,

		/**
		 * A dijit.Menu to be used as the popup of available site help actions. 
		 * If a single dijit.MenuItem is supplied, a simple site help action
		 * will be presented and onClick will be triggered on the menu item when
		 * that action is selected.
		 * @type dijit.MenuBarItem | dijit.MenuBarPopupItem
		 */
		help: undefined,
	
		/**
		 * True (the default) if a drop-down arrow affordance is to be shown on the
		 * site help icon when a popup menu of site help items is supplied.
		 * If false, a drop-down arrow is not shown on the site help icon.
		 * @type boolean
		 */
		showHelpDropDownArrow: true,

		/**
		 * The context title which shows users where they are, for example
		 * if they have arrived here by selecting a menu item.
		 * @type string
		 */
		secondaryTitle: "",
		
		/**
		 * The desired style of secondary (context) banner: "blue" or "lightgrey".
		 * @default "blue"
		 * @type string
		 */
		secondaryBannerType: "blue",
		
		/**
		 * A subtitle which gives additional context information.
		 * @type string
		 */
		secondarySubtitle: "",
		
		/**
		 * Text containing additional context information, such as when page
		 * content was last updated and by whom.
		 * @type string
		 */
		additionalContext: "",

		/**
		 * An array of objects defining actions which will be displayed as
		 * action buttons in the context part of the header. Each object
		 * must contain the following properties:
		 * <ul>
		 * <li>
		 * label: text label for the action
		 * </li>
		 * <li>
		 * onClick: click handler for the action button
		 * </li>
		 * </ul>
		 * @type Object[]
		 */
		actions: undefined,
		
		/**
		 * The id of a content container which is to be controlled by tabs
		 * included in the header, or the widget itself. Each ContentPane in
		 * the StackContainer may additionally include the following properties
		 * (all optional):
		 * <ul>
		 * <li>
		 * closable: {boolean} If true, a close affordance will be displayed on
		 * the corresponding tab and will close the content pane when activated.
		 * If false, or if this property is not set, no close affordance is shown.
		 * </li>
		 * <li>
		 * actions: {dijit.Menu} A menu of items to be presented when the
		 * drop-down affordance on the tab is activated. The drop-down
		 * affordance will be displayed on the tab if this property is set and
		 * either the tab is selected or alwaysShowMenu is true.
		 * </li>
		 * <li>
		 * alwaysShowMenu: {boolean} If true, a drop-down affordance will be
		 * displayed on the tab if the actions property has been set,
		 * regardless of whether the tab is currently selected. If false, a
		 * drop-down affordance will only be displayed on the tab if the
		 * actions property has been set AND the tab is currently selected.
		 * </li>
		 * </ul>
		 * @type string | dijit.StackContainer
		 */
		contentContainer: "",
		
		/**
		 * If true, content tabs will be placed on the same line as a context
		 * title and/or other secondary banner content. If false, the tabs will
		 * occupy their own row within the secondary banner. The default value is false.
		 * @type boolean
		 */
		contentTabsInline: false,
		
		/**
		 * Specifies that a secondary search box should be included in the
		 * header, and supplies the parameters for it. All the properties are
		 * optional:
		 * <ul>
		 * <li>
		 * entryPrompt: {string | function} A string containing the prompt text
		 * for entering the search terms, or a function (which will be called
		 * with no arguments) which returns the prompt text.
		 * </li>
		 * <li>
		 * submitPrompt: {string | function} A string containing the prompt
		 * text for submitting the search, or a function (which will be called
		 * with no arguments) which returns the prompt text.
		 * </li>
		 * <li>
		 * onChange: {function} A function which will be called whenever the
		 * text in the search box changes. The function will receive one
		 * argument, which is the text currently in the search box.
		 * </li>
		 * <li>
		 * onSubmit: {function} A function which will be called whenever user
		 * submits a search (eg, by pressing enter, or activating a 
		 * search affordance). The function will receive one argument, which
		 * is the text currently in the search box.
		 * </li>
		 * </ul>
		 * @type Object
		 */
		secondarySearch: undefined,
		
		/**
		 * Specifies the desired layout mode, which can be "fixed" for a
		 * fixed-width layout independent of the browser width (extra space
		 * will be left at the side margins, and a scroll bar will appear if
		 * the browser window is too narrow) or "variable" for a variable-width
		 * layout that exploits the full browser window width (extra space will
		 * be left within the layout, which will change as the browser window
		 * is resized). 
		 * @default "variable".
		 * @type string
		 */
		layoutType: "variable",
		
		/**
		 * make the logo clickable by providing a logo link
		 */
		logoLink: "",
		
		// The following properties (read-only) can be used to obtain the DOM
		// nodes of key elements of the constructed UI. These properties will
		// not be defined unless the corresponding UI element is used/required.
		//
		// Container nodes:
		//  domNode: outer containing DOM node
		//  primaryBannerNode: container (div) for all primary banner content
		//  navigationNode: contains a menu bar of navigation action items
		//  userNode: contains the user identity and actions display
		//  secondaryBannerNode: container (div) for all secondary banner content
		//
		// Widget and content nodes:
		//  primaryTitleTextNode: contains primary title text/markup
		//  userTextNode: contains user identity text/markup
		//  primarySearchTextNode: the text field for primary search
		//  primarySearchButtonNode: the submit button for primary search
		//  secondaryTitleTextNode: contains secondary title text/markup
		//  secondarySubtitleTextNode: contains secondary subtitle text/markup
		//  contextActionNodes: array of action button nodes
		//  contentControllerNode: the content controller (tab bar)
		//  secondarySearchTextNode: the text field for secondary search
		//  secondarySearchButtonNode: the submit button for secondary search
		//  
		// Other nodes used internally:
		//  _mainContainerNode: container for the primary and secondary banners
		//  _globalActionsNode: container for all global actions
		//  _secondaryTitleSeparatorNode: text separating secondary title and subtitle
		//  _contextActionsNode: container for all context actions
		//
		/**
	 	 * The template HTML for the widget.
		 * @constant
		 * @type string
		 * @private
		 */
		templateString: '<div>' +
		                '<div dojoAttachPoint="_mainContainerNode">' +
						'</div>' +
						'</div>',

		/**
		 * Return the user identity display name, calling supplied functions
		 * where applicable.
		 * @private
		 */		
		_getComputedUserName: function(){
			return (this.user && (typeof this.user.displayName == "function")) ? this.user.displayName() : (this.user.displayName || "");
		},
		
		/**
		 * Return the user identity display image, calling supplied functions
		 * where applicable.
		 * @private
		 */		
		_getComputedUserImage: function(){
			return (this.user && (typeof this.user.displayImage == "function")) ? this.user.displayImage() : this.user.displayImage;
		},
		
		/**
		 * Return the user identity message, taking into account any custom
		 * message template and calling supplied functions where applicable.
		 * @private
		 */		
		_getComputedUserMessage: function(){
			// name to use in message: if no message name, use display name
			var displayname = this._getComputedUserName(),
				messagename = ((typeof this.user.messageName == "function") ? this.user.messageName() : this.user.messageName) || displayname,
				result = messagename;
			
			if(this.user && this.user.message){
				var message = (typeof this.user.message == "function") ? this.user.message() : this.user.message;
				
				result = string.substitute(message, this.user, function(value, key){
					switch(key){
						case "messageName": return messagename;
						case "displayName": return displayname;
						default: return value || "";
					}
				});
			}
			
			return result;
		},
		
		_setUserDisplayNameAttr: function(value){
			this.user = this.user || {};
			this.user.displayName = value;
			this._refreshUser();
		},
		
		_setUserDisplayImageAttr: function(value){
			this.user = this.user || {};
			this.user.displayImage = value;
			this._refreshUser();
		},
		
		_setUserMessageNameAttr: function(value){
			this.user = this.user || {};
			this.user.messageName = value;
			this._refreshUser();
		},
		
		_setUserMessageAttr: function(value){
			this.user = this.user || {};
			this.user.message = value;
			this._refreshUser();
		},
		
		_refreshUser: function(){
			var name = this._getComputedUserName(),
				imgsrc = this._getComputedUserImage(),
				msg = this._getComputedUserMessage();
				
			domattr.set(this.userNode, "title", name);
			domattr.set(this.userImageNode, "alt", name);
			
			domattr.set(this.userImageNode, "src", imgsrc || "");
			domstyle.set(this.userImageNode, "display", imgsrc ? "block" : "none");
			
			this.userTextNode.innerHTML = msg;
			domclass.replace(this.userNode, msg ? "idxHeaderUserName" : "idxHeaderUserNameNoText", "idxHeaderUserName idxHeaderUserNameNoText");
		},
		
		/**
		 * Construct UI from a template, injecting the resulting DOM items
		 * as children on of the supplied container node.
		 * @param {Object} containerNode
		 * @param {Object} templateString
		 */
		_injectTemplate: function(containerNode, templateString){
			
			// this code is generalised from _Templated.buildRendering

			// Look up cached version of template, or download to cache.
			var cached = _TemplatedMixin.getCachedTemplate(templateString, true);

			var node;
			if(_lang.isString(cached)){
				// if the cache returned a string, it contains replacement parameters,
				// so replace them and create DOM
				node = domconstruct.toDom(this._stringRepl(cached));
			}else{
				// if the cache returned a node, all we have to do is clone it
				node = cached.cloneNode(true);
			}

			// recurse through the node, looking for, and attaching to, our
			// attachment points and events, which should be defined on the template node.
			this._attachTemplateNodes(node, function(n,p){ return n.getAttribute(p); });
			
			// append resolved template as child of container
			containerNode.appendChild(node);
		},

		/**
		 * Standard widget lifecycle postMixInProperties() method.
		 * @private
		 */
		postMixInProperties: function(){
			this._nls = i18n.getLocalization("idx.oneui", "Header");
		
			if(this.primarySearch){
				this.primarySearch = _lang.mixin({
					entryPrompt: this._nls.searchEntry,
					submitPrompt: this._nls.searchSubmit
				}, this.primarySearch);
			}

			if(this.secondarySearch){
				this.secondarySearch = _lang.mixin({
					entryPrompt: this._nls.searchEntry,
					submitPrompt: this._nls.searchSubmit
				}, this.secondarySearch);
			}
		},
		
		/**
		 * Standard widget lifecycle buildRendering() method.
		 * @private
		 */
		buildRendering: function(){
			// summary:
			//     Generate the HTML and CSS for the header.

			// call down to apply the template and base widget handling
			this.inherited(arguments);
			
			// The following logic allocates all header items into either a
			// primary or a secondary banner, both of which are optional.
			//
			// The primary banner will accommodate:
			//  - primary title (if any)
			//  - navigation links/menus (if any)
			//  - search (if any, and if there's room)
			//  - user identity (if any)
			// The search will only be accommodated if at least one of the
			// other items is omitted; if a primary title, navigation
			// links/menus and user identity are all provided then the search
			// (if required) will be placed into the secondary banner.
			//
			// The primary banner will place the primary title at the top left
			// and the user identity at the top right. The navigation
			// links/menus and search will be placed between them, flowing onto
			// a second line if necessary.
			//
			// The secondary banner will accommodate:
			//  - secondary title (if any)
			//  - action links/menus (if any)
			//  - content controller (if any)
			//  - search (if any, and if not accommodated in primary banner)
			//
			// The secondary banner will place the secondary title at the top
			// left and the search at the top right. The action links/menus and
			// content controller will be placed between them, flowing onto a
			// second line if necessary.
			// 
			// Other layout schemes (eg for mobile) will require separate logic
			// not yet provided here.
			
			// First issue warnings for situations that may not be intended.
			if(this.contentContainer && this.secondaryBannerType && this.secondaryBannerType.toLowerCase() == "white"){
				// content tabs in "white" style are not supported
				require.log('*** Warning: Header will not display content tabs when secondaryBannerType is "white". Specify a different type to see content tabs.');
			}
			
			var show_primary_title = this.primaryTitle,
				show_primary_logo = true,
				show_primary_help = this.help,
				show_primary_settings = this.settings,
				show_primary_user = this.user,
				show_primary_navigation = this.navigation,
				show_primary_search = this.primarySearch,
				show_secondary_navigation =this.secondaryNavigation,
			    show_secondary_title = this.secondaryTitle || this.secondarySubtitle,
			    show_secondary_actions = this.contextActions,
			    show_secondary_search = this.secondarySearch,
			    show_content = this.contentContainer && (!this.secondaryBannerType || (this.secondaryBannerType.toLowerCase() != "white")),  // never show content tabs in "white" style (not supported)
				show_secondary_content = show_content && (this.contentTabsInline || !show_secondary_title),
				show_secondary_border = this.secondaryBannerType && (this.secondaryBannerType.toLowerCase() == "white"),
				show_tertiary_content = show_content && !show_secondary_content,
			    show_primary_items = show_primary_title || show_primary_logo || show_primary_help || show_primary_settings || show_primary_user || show_primary_navigation || show_primary_search,  
			    show_secondary_items = show_secondary_title || show_secondary_actions || show_secondary_search || show_secondary_content,
				show_tertiary_items = show_tertiary_content,
				show_lip;
			
			if(show_primary_items || show_secondary_items || show_tertiary_items){
				domclass.add(this.domNode, "idxHeaderContainer");
				
				if(this.primaryBannerType && (this.primaryBannerType.toLowerCase() == "thick")){
					domclass.add(this._mainContainerNode, "idxHeaderPrimaryThick");
				}else{
					domclass.add(this._mainContainerNode, "idxHeaderPrimaryThin");
				}
				
				if(this.secondaryBannerType && ((this.secondaryBannerType.toLowerCase() == "lightgrey") || (this.secondaryBannerType.toLowerCase() == "lightgray"))){
					domclass.add(this._mainContainerNode, "idxHeaderSecondaryGray");
					domclass.add(this._mainContainerNode, show_tertiary_items ? "idxHeaderSecondaryGrayDoubleRow" : "idxHeaderSecondaryGraySingleRow");
					show_lip = show_primary_items;
				}else if(this.secondaryBannerType && (this.secondaryBannerType.toLowerCase() == "white")){
					domclass.add(this._mainContainerNode, "idxHeaderSecondaryWhite");
					domclass.add(this._mainContainerNode, show_tertiary_items ? "idxHeaderSecondaryWhiteDoubleRow" : "idxHeaderSecondaryWhiteSingleRow");
					show_lip = show_primary_items;
				}else{
					domclass.add(this._mainContainerNode, "idxHeaderSecondaryBlue");
					domclass.add(this._mainContainerNode, (show_tertiary_items) ? "idxHeaderSecondaryBlueDoubleRow" : "idxHeaderSecondaryBlueSingleRow");
					show_lip = show_primary_items && !show_secondary_items && !show_tertiary_items;
				}
				domclass.add(this._mainContainerNode, show_tertiary_items ? "idxHeaderSecondaryDoubleRow" : "idxHeaderSecondarySingleRow");
				
				if(this.layoutType && (this.layoutType.toLowerCase() == "fixed")){
					domclass.add(this._mainContainerNode, "idxHeaderWidthFixed");
				}else{
					domclass.add(this._mainContainerNode, "idxHeaderWidthLiquid");
				}				
			}
			
			// now load any additional modules we know we need
			var modules = [],
			    assigns = [],
				me = this;
			
			if(show_primary_search || show_secondary_search || show_secondary_actions){
				modules.push("dijit/form/Button");
				assigns.push(function(obj){ Button = obj; });
				if(this.secondarySearch.finderWidget) {
					modules.push(this.secondarySearch.finderWidget);
					assigns.push(function(obj){ FinderWidget = obj; });				
				}
			}
			
			if(show_primary_search || show_secondary_search){
				modules.push("dijit/form/TextBox");
				assigns.push(function(obj){ TextBox = obj; });
			}
			
			if(show_content){
				modules.push("idx/oneui/layout/MenuTabController");
				assigns.push(function(obj){ MenuTabController = obj; });
			}
			
			require(modules, function(){
			
				for(var i=0; i<assigns.length; i++){
					assigns[i](arguments[i]);
				}
			
				// create the primary bar
				
				if(show_primary_items){
					me._injectTemplate(me._mainContainerNode,
										 '<div class="idxHeaderPrimary">' +
										 '<div class="idxHeaderPrimaryInner" dojoAttachPoint="primaryBannerNode">' +
										 '<ul dojoAttachPoint="_globalActionsNode">' +
										 '</ul>' +
										 '</div>' +
										 '</div>');
				}
				
				if(show_primary_title){
					me._renderPrimaryTitle(me._globalActionsNode);
				}
				
				if(show_primary_logo){
					me._renderLogo(me._globalActionsNode);
				}
				
				if(show_primary_help){
					me._renderHelp(me._globalActionsNode, show_primary_settings || show_primary_user);
				}
				
				if(show_primary_settings){
					me._renderSettings(me._globalActionsNode, show_primary_user);
				}
				
				if(show_primary_user){
					me._renderUser(me._globalActionsNode);
				}
				
				if(show_primary_search){
					me._renderPrimarySearch(me._globalActionsNode);
				}			
				
				if(show_primary_navigation){
					me._renderNavigation(me.primaryBannerNode);
				}
				
				// create the blue lip
				
				if(show_lip){
					me._injectTemplate(me._mainContainerNode,
										 '<div class="idxHeaderBlueLip">' + 
										 '</div>');
				}
				
				// create the secondary bar
				
				if(show_secondary_items){
					me._injectTemplate(me._mainContainerNode,
										 '<div class="idxHeaderSecondary"> ' +
										 '<div class="idxHeaderSecondaryInner" dojoAttachPoint="secondaryBannerNode">' + 
										 '</div>' + 
										 '</div>');
				}
				
				if(show_secondary_search){
					me._renderSecondarySearch(me.secondaryBannerNode);
				}			

				if(show_secondary_title){
					me._renderSecondaryTitle(me.secondaryBannerNode);
				}
				
				if(show_secondary_content){
					me._renderContent(me.secondaryBannerNode, false);
				}

				if(show_secondary_actions){
					me._renderContextActions(me.secondaryBannerNode);
				}
				
				if(show_secondary_border){
					me._renderSecondaryInnerBorder(me.secondaryBannerNode);
				}
				
				//jlc
				if(show_secondary_navigation){
					me._renderSecondaryNavigation(me.secondaryBannerNode);
				}
				
				// create the tertiary bar
				
				if(show_tertiary_content){
					me._renderContent(me._mainContainerNode, true);
				}
			});
		},
		
		_renderPrimaryTitle: function(domNode){
			var primaryTitleDom = '<li>';
			if(this.primaryTitleLink)
			{
				primaryTitleDom = primaryTitleDom + '<span onclick="' + this.primaryTitleLink + '">';
			} else {
				primaryTitleDom = primaryTitleDom + '<span>';
			}
			primaryTitleDom = primaryTitleDom + '<div class="idxHeaderPrimaryTitle">' + '${primaryTitle}' +  '</div>' + '</span>' + '</li>';
			
			console.log("***primary title dom is: " + primaryTitleDom);
			this._injectTemplate(domNode, primaryTitleDom); 
		},
		
		_renderLogo: function(domNode){
			var logoDom = '<li class="idxHeaderPrimaryAction end">';
			
			if(this.logoLink)
			{
				logoDom = logoDom + '<span onclick="' + this.logoLink + '">';
			} else {
				logoDom = logoDom + '<span>';
			}
			
			logoDom = logoDom +	'<div class="idxHeaderLogoBox">' +
			         			 '<div class="idxHeaderLogo" alt="${_nls.ibmlogo}">' +
			         			 '</div>' +
			         			 '</div>' +
			         			 '</span>' +
			         			 '</li>'; 
			this._injectTemplate(domNode, logoDom);
		},
		
		_renderHelp: function(domNode, addSeparator){
			this._injectTemplate(domNode,
			         			 '<li class="idxHeaderPrimaryAction idxHeaderHelp">' +
			         			 '<span dojoAttachPoint="_helpNode" alt="${_nls.actionHelp}" title="${_nls.actionHelp}">' +
									 '<span class="idxHeaderHelpIcon">' +
									 '</span>' +
							         '<span class="idxHeaderDropDownArrow">' +
							         '</span>' +
			         			 '</span>' +
			         			 '</li>');
			
			if(addSeparator){
				this._injectTemplate(domNode,
				                     '<li class="idxHeaderPrimaryAction idxHeaderSeparator"><span></span></li>');
			}

			if(this.help){
				this.help = registry.byId(this.help);
				prepareMenu(this.help, [ "oneuiHeaderGlobalActionsMenu", "oneuiHeaderGlobalActionsSubmenu" ], this._helpNode, this._helpNode, true);
				domclass.toggle(this._helpNode, "idxHeaderDropDown", this.showHelpDropDownArrow);
			}
		},
		
		_renderSettings: function(domNode, addSeparator){
			this._injectTemplate(domNode,
			         			 '<li class="idxHeaderPrimaryAction idxHeaderTools">' +
			         			 '<span dojoAttachPoint="_settingsNode" alt="${_nls.actionShare}" title="${_nls.actionShare}">' +
									 '<span class="idxHeaderShareIcon">' +
									 '</span>' +
							         '<span class="idxHeaderDropDownArrow">' +
							         '</span>' +
			         			 '</span>' +
			         			 '</li>'); 
			
			if(addSeparator){
				this._injectTemplate(domNode,
				                     '<li class="idxHeaderPrimaryAction idxHeaderSeparator"><span></span></li>');
			}

			if(this.settings){
				this.settings = registry.byId(this.settings);
				prepareMenu(this.settings, [ "oneuiHeaderGlobalActionsMenu", "oneuiHeaderGlobalActionsSubmenu" ], this._settingsNode, this._settingsNode, true);
				domclass.toggle(this._settingsNode, "idxHeaderDropDown", this.showSettingsDropDownArrow);
			}
		},
		
		_renderUser: function(domNode){
			this._injectTemplate(domNode,
			                     '<li class="idxHeaderPrimaryAction">' + 
									'<span dojoAttachPoint="userNode" class="idxHeaderUserNameNoText">' +
										'<span class="idxHeaderUserIcon">' +
											'<img dojoAttachPoint="userImageNode" class="idxHeaderUserIcon" />' +
										'</span>' +
										'<span class="idxHeaderUserText" dojoAttachPoint="userTextNode">' +
										'</span>' +
								        '<span class="idxHeaderDropDownArrow">' +
								        '</span>' +
									'</span>' +
			                     '</li>');			                     
			
			this._refreshUser();

			if(this.user.actions){
				this.user.actions = registry.byId(this.user.actions);
				prepareMenu(this.user.actions, [ "oneuiHeaderGlobalActionsMenu", "oneuiHeaderGlobalActionsSubmenu" ], this.userNode, this.userNode, true);
				domclass.toggle(this.userNode, "idxHeaderDropDown", this.showUserDropDownArrow);
			}
		},
		
		_renderNavigation: function(domNode){
			this.navigation = ((typeof this.navigation == "object") && ('nodeType' in this.navigation)) ? registry.byNode(this.navigation) : registry.byId(this.navigation);
			
			if(!this.navigation){
				require.log("WARNING: navigation widget not found");
			}else{
				this.navigation.placeAt(domNode);
				this.navigation.startup();
				
				var children = this.navigation.getChildren();
				if((children.length == 1) && (children[0].label == "")){
					// if there is just a single menu bar item with no text, make it a "home" icon
					domclass.toggle(children[0].containerNode, "idxHeaderNavigationHome", true);
				}else if(this.showNavigationDropDownArrows){
					for(var i = 0; i < children.length; i++){
						if(children[i].popup){
							this._injectTemplate(children[i].focusNode, '<span class="idxHeaderDropDownArrow"></span>'); 
							domclass.toggle(children[i].domNode, "idxHeaderDropDown", true);
						}
					}
				}
				
				// remove whitespace-only text nodes in the menu-bar, as these
				// can disrupt precise layout of the menu items
				var node = this.navigation.domNode.firstChild, del;
				while(node){
					del = node;
					node = node.nextSibling;
					if((del.nodeType == 3) && (!del.nodeValue.match(/\S/))){
						this.navigation.domNode.removeChild(del);
					}
				}
				
				prepareMenu(this.navigation, [null, "oneuiHeaderNavigationMenu", "oneuiHeaderNavigationSubmenu"]);
			}
		},
		
		//jlc: added rendering for navigation on second header bar
		_renderSecondaryNavigation: function(domNode){
			this.secondaryNavigation = ((typeof this.secondaryNavigation == "object") && ('nodeType' in this.secondaryNavigation)) ? registry.byNode(this.secondaryNavigation) : registry.byId(this.secondaryNavigation);
			
			if(!this.secondaryNavigation){
				require.log("WARNING: secondaryNavigation widget not found");
			}else{
				this.secondaryNavigation.placeAt(domNode);
				this.secondaryNavigation.startup();
				
				this._showSecondaryNavigationInHead();
				this.secondaryNavigationInHeadShowed=true;
				
				
			}
		},
		
		_showSecondaryNavigationInHead: function(){
			var children = this.secondaryNavigation.getChildren();
			if((children.length == 1) && (children[0].label == "")){
				// if there is just a single menu bar item with no text, make it a "home" icon
				domclass.toggle(children[0].containerNode, "idxHeaderNavigationHome", true);
			}else if(this.showSecondaryNavigationDropDownArrows){
				for(var i = 0; i < children.length; i++){
					if(children[i].popup){
						this._injectTemplate(children[i].focusNode, '<span class="idxHeaderDropDownArrow"></span>'); 
						domclass.toggle(children[i].domNode, "idxHeaderDropDown", true);
					} 
				}
			}
			
			// remove whitespace-only text nodes in the menu-bar, as these
			// can disrupt precise layout of the menu items
			var node = this.secondaryNavigation.domNode.firstChild, del;
			while(node){
				del = node;
				node = node.nextSibling;
				if((del.nodeType == 3) && (!del.nodeValue.match(/\S/))){
					this.secondaryNavigation.domNode.removeChild(del);
				}
			}
			prepareMenu(this.secondaryNavigation, [ null, "oneuiHeaderNavigationMenu", "oneuiHeaderNavigationSubmenu" ]);
		},
		
		_renderPrimarySearch: function(domNode){
			this._injectTemplate(domNode,
			                     '<li role="search" class="idxHeaderSearchContainer">' +
			                     '<input type="text" dojoAttachPoint="primarySearchTextNode" />' + 
			                     '<input type="image" dojoAttachPoint="primarySearchButtonNode" />' +
			                     '</li>');
								 
			this.primarySearch.onChange = _lang.isFunction(this.primarySearch.onChange) ? this.primarySearch.onChange : new Function("value", this.primarySearch.onChange);
			this.primarySearch.onSubmit = _lang.isFunction(this.primarySearch.onSubmit) ? this.primarySearch.onSubmit : new Function("value", this.primarySearch.onSubmit);
			
			var me = this;
			
			var text = new TextBox({
				trim: true,
				placeHolder: this.primarySearch.entryPrompt,
				intermediateChanges: true,
				title: this.primarySearch.entryPrompt,
				onChange: function(){ me._onPrimarySearchChange(text.attr("value")); },
				onKeyUp: function(event){ if(event.keyCode == keys.ENTER){ me._onPrimarySearchSubmit(text.attr("value")); } }
			},
			this.primarySearchTextNode);
			
			new Button({
				label: this.primarySearch.submitPrompt,
				showLabel: false,
				iconClass: "idxHeaderSearchButton",
				onClick: function(){ me._onPrimarySearchSubmit(text.attr("value")); }				
			},
			this.primarySearchButtonNode);
		},
		
		_renderSecondaryTitle: function(domNode){
			this._injectTemplate(domNode,
			                     '<span class="idxHeaderSecondaryTitleContainer">' +
								 '<span class="idxHeaderSecondaryTitle" dojoAttachPoint="secondaryTitleTextNode">' +
			                     '${secondaryTitle}' +
			                     '</span>' +
			                     '<span class="idxHeaderSecondarySubtitle" dojoAttachPoint="_secondaryTitleSeparatorNode">' +
			                     '&nbsp;&ndash;&nbsp;' +
			                     '</span>' +
			                     '<span class="idxHeaderSecondarySubtitle" dojoAttachPoint="secondarySubtitleTextNode">' +
			                     '${secondarySubtitle}' +
			                     '</span>' +
								 '&nbsp;&nbsp;' +
			                     '<span class="idxHeaderSecondaryAdditionalContext" dojoAttachPoint="additionalContextTextNode">' +
			                     '${additionalContext}' +
			                     '</span>' +
								 '</span>');
								 
			domstyle.set(this._secondaryTitleSeparatorNode, "display", (this.secondaryTitle && this.secondarySubtitle) ? "" : "none");
		},
		
		_renderContextActions: function(domNode){
			this._injectTemplate(domNode,
			                     '<div class="idxHeaderSecondaryActions" dojoAttachPoint="_contextActionsNode"></div>');
			this.contextActionNodes = [];
			
			for(var i=0; i<this.contextActions.length; i++){
				this._injectTemplate(this._contextActionsNode,
									 '<button type="button" dojoAttachPoint="_nextActionNode"></button>');
				new Button(this.contextActions[i], this._nextActionNode);
				this.contextActionNodes.push(this._nextActionNode);
				delete this._nextActionNode;
			}
		},
		
		_renderSecondarySearch: function(domNode){
			this._injectTemplate(domNode,
			                     '<div id="finder" role="search" class="idxHeaderSearchContainer">' +
			                     '<input type="text" dojoAttachPoint="secondarySearchTextNode" />' + 
			                     '<input type="image" dojoAttachPoint="secondarySearchButtonNode" />' +
			                     '</div>');
			
			this.secondarySearch.onChange = _lang.isFunction(this.secondarySearch.onChange) ? this.secondarySearch.onChange : new Function("value", this.secondarySearch.onChange);
			this.secondarySearch.onSubmit = _lang.isFunction(this.secondarySearch.onSubmit) ? this.secondarySearch.onSubmit : new Function("value", this.secondarySearch.onSubmit);
			
			var me = this;
			
			var text = new TextBox({
				trim: true,
				id: "findersearchBox",
				intermediateChanges: true,
				title: this.secondarySearch.entryPrompt,
				onChange: function(){ me._onSecondarySearchChange(text.attr("value")); },
				onKeyUp: function(event){ if(event.keyCode == keys.ENTER){ me._onSecondarySearchSubmit(text.attr("value")); } }
			},
			this.secondarySearchTextNode);
			
			new Button({
				label: this.secondarySearch.submitPrompt,
				showLabel: false,
				iconClass: "idxHeaderSearchButton",
				onClick: function(){ me._onSecondarySearchSubmit(text.attr("value")); }				
			},
			this.secondarySearchButtonNode);
			new FinderWidget({id:'finder'}, 'finder');
		},
		
		_renderSecondaryInnerBorder: function(domNode){
			this._injectTemplate(domNode,
			                     '<div role="presentation" class="idxHeaderSecondaryInnerBorder">' +
			                     '</div>');
		},
		
		_renderContent: function(domNode, includeInnerDiv){
			this._injectTemplate(domNode,
			                     '<div class="oneuiContentContainer">' +
								 (includeInnerDiv ? '<div class="oneuiContentContainerInner">' : '') +
			                     '<div dojoAttachPoint="contentControllerNode"></div>' +
								 (includeInnerDiv ? '</div>' : '') +
			                     '</div>');	
			
			var controller = new MenuTabController({
				containerId: (typeof this.contentContainer === "string") ? this.contentContainer : this.contentContainer.id,
				"class": "dijitTabContainerTop-tabs",
				useMenu: this._tabMenu,
				useSlider: this._tabSlider,
				buttonWidget: _lang.extend(idx.oneui.layout._PopupTabButton, { tabDropDownText: "", tabSeparatorText: "|" })
			},
			this.contentControllerNode);
			
			prepareMenu(controller._menuBtn, [ "oneuiHeader2ndLevMenu", "oneuiHeader2ndLevSubmenu" ]);
			aspect.after(controller, "_bindPopup", function(page, tabNode, popupNode, popup){
				prepareMenu(popup, [ "oneuiHeader2ndLevMenu", "oneuiHeader2ndLevSubmenu" ], popupNode, tabNode);
			}, true);
			
			controller.startup();
				
			// if the content container is already started, ensure the controller initialises correctly
			var container = registry.byId(this.contentContainer);
			if(container && container._started){
				controller.onStartup({ children: container.getChildren(), selected: container.selectedChildWidget });
			}
		},
		
		_onPrimarySearchChange: function(value){
			this.primarySearch.onChange(value);
		},
		
		_onPrimarySearchSubmit: function(value){
			this.primarySearch.onSubmit(value);
		},
		
		_onSecondarySearchChange: function(value){
			this.secondarySearch.onChange(value);
		},
		
		_onSecondarySearchSubmit: function(value){
			this.secondarySearch.onSubmit(value);
		}
		
	});
	
	return idx.oneui.Header;
		
});

},
'dijit/_base':function(){
define([
	".",
	"./a11y",	// used to be in dijit/_base/manager
	"./WidgetSet",	// used to be in dijit/_base/manager
	"./_base/focus",
	"./_base/manager",
	"./_base/place",
	"./_base/popup",
	"./_base/scroll",
	"./_base/sniff",
	"./_base/typematic",
	"./_base/wai",
	"./_base/window"
], function(dijit){

	// module:
	//		dijit/_base
	// summary:
	//		Includes all the modules in dijit/_base

	return dijit._base;
});

},
'dojoe/wizard/Wizard':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/i18n!dojoe/wizard/nls/Wizard","dojo/require!dijit/_Widget,dijit/_Templated,dojo/i18n,dojoe/wizard/Page,dojoe/wizard/WizardToCSlider,dojoe/wizard/TivSubstep,dojoe/messagedialog/MessageDialog,dijit/layout/BorderContainer,dijit/layout/StackContainer,dijit/layout/ContentPane,dojoe/wizard/PageContainer"], function(dijit,dojo,dojox){
dojo.provide("dojoe.wizard.Wizard");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.requireLocalization("dojoe.wizard", "Wizard");
dojo.require("dojoe.wizard.Page");
dojo.require("dojoe.wizard.WizardToCSlider");
dojo.require("dojoe.wizard.TivSubstep");
dojo.require("dojoe.messagedialog.MessageDialog");
dojo.require("dijit.layout.BorderContainer");
dojo.require("dijit.layout.StackContainer");
dojo.require("dijit.layout.ContentPane");

/**
 * Inherit from PageContainer which contains the common functions of both
 * the notebook and wizard.
 */
dojo.require("dojoe.wizard.PageContainer");

/**
 * 
 * This class is an extension of PageContainer that handles all the
 * wizard specific functions as specified in the UI guidelines.  The table
 * of contents layout may be vertical or horizontal.  It also includes the 
 * advanced navigation features such as the ability to have sub-steps, 
 * sub-steps that loop and to jump backwards to a previous page by clicking 
 * on an item in the table of contents.
 * 
 * Extra capabilities that have been added include the ability to skip from 
 * whatever page is displayed to the next main step, enable and disable steps
 * and buttons, hide the button area, indicate a step is required and restart 
 * the wizard. 
 * 
 * Creating a Wizard in markup
 * <div id="note" dojoType="dojoe.wizard.Wizard">
 *    <div dojoType="dojoe.wizard.tests.Pages.Welcome"></div>
 *    <div dojoType="dojoe.wizard.tests.Pages.BaseWidget"></div>
 *    <div dojoType="dojoe.wizard.tests.Pages.General"></div>
 *    <div dojoType="dojoe.wizard.tests.Pages.Security"></div>
 *    <div dojoType="dojoe.wizard.tests.Pages.Summary" ></div>
 *  </div> 
 *
 * Events:
 *  OnPageSelected: when a page is made active
 *  OnBack: when the Back button is clicked
 *  OnNext: when the Next button is clicked
 *  OnOK: when the Finish/OK button is clicked
 *  OnCancel: when the Cancel button is clicked
 */
dojo.declare("dojoe.wizard.Wizard", 
            [dijit._Widget, dijit._Templated, dojoe.wizard.PageContainer], 
	{  

	// summary:  The wizard is used in the context when a user performs infrequent or complex tasks that include a precise series of steps that require the entry of large amounts of interdependent information
	// description: Wizard widget uses the dojoe.wizard.PageContainer for its implementation.
	//		Pages can be created by extending dojoe.wizard.Page.
	
	// widgetsInTemplate: [public]Boolean
	//		Set this to true if the widget contains other widgets
		widgetsInTemplate: true,
	
	// templatePath:  [private]String
	//		Path to the template
		templatePath: "",
	
	// mode:  [private]String
	//		creation mode default is w for wizard
		mode: "w",               
	
	// renderNoArrows: [public]boolean
	//		if true render ToC without arrow graphics, only applicable in horizontal layout. default is false.		
		renderNoArrows: false,   
	
	// slider:  [private]Object
	//		for scrolling ToC, only applicable in horizontal layout
		slider: null,        

	// tocStepsArea:  [private]Object
	//		for main steps of ToC, only applicable in horizontal layout
		tocStepsArea: null,     
	
	// tocSubstepsArea:  [private]Object
	//		for sub steps of ToC, only applicable in horizontal layout
		tocSubstepsArea: null,   
	
	// lastPageSelected:  [private]Object
	//		keep track of the last page that was selected
		lastPageSelected: null, 
	
	// backLabel: [public]String
	//		label of back button.
		backLabel: "",
	
	// nextLabel: [public]String
	//		label of next button.
		nextLabel: "",
	
	// finishLabel: [public]String
	//		label of finish button.
		finishLabel: "",
	
	// cancelLabel: [public]String
	//		label of cancel button.
		cancelLabel: "",
		
	
	// showCancelWarning: [public]boolean
	//		if true renders the confirmation message on click of cancel otherwise confirmation message will not be displayed. Default is false.	
		showCancelWarning: false, //This attribute is added on request of TIP to make the confirmation message display as optional on click of cancel button. 
		
		constructor: function () {
			this._resources = dojo.i18n.getLocalization("dojoe.wizard", "Wizard", this.lang);
		},
  
		postMixInProperties: function () {
			// summary:  Creates and initialises the Wizard.
			// tags: private
  
			if (this.nested) {
				this.templateString="<div class=\"tiv-wiz\" role=\"application\">\r\n  <div dojoType=\"dijit.layout.BorderContainer\" design=\"headline\" gutters=\"false\" class=\"tiv-wiz-outer\" dojoAttachPoint=\"_outerContainer\">\r\n    <div dojoType=\"dijit.layout.BorderContainer\" gutters=\"false\" liveSplitters=\"false\" design=\"sidebar\" region=\"center\" style=\"width:100%;height:100%;\" dojoAttachPoint=\"_borderArea\">\r\n      <div dojoType=\"dijit.layout.ContentPane\" region=\"leading\" splitter=\"false\" class=\"tiv-wiz-toc\" dojoAttachPoint=\"_tocArea\">\r\n        <ol dojoAttachPoint=\"_toc\"></ol>\r\n      </div>\r\n      <div dojoType=\"dijit.layout.StackContainer\" region=\"center\" splitter=\"false\" dojoAttachPoint=\"_pageContainer\" class=\"tiv-wiz-work-area\"> \r\n        <div dojoAttachPoint=\"containerNode\"></div>\r\n      </div>\r\n    </div>\r\n    <div dojoType=\"dijit.layout.ContentPane\" region=\"bottom\" splitter=\"false\" class=\"tiv-wiz-footer\" dojoAttachPoint=\"_buttonArea\">\r\n      <div class=\"tiv-wiz-buttons\" dojoAttachPoint=\"_buttons\">\r\n        <button role=\"button\" type=\"button\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickBack\" dojoAttachPoint=\"_backButton\">${backLabel}</button>\r\n        <button role=\"button\" type=\"button\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickNext\" dojoAttachPoint=\"_nextButton\">${nextLabel}</button>\r\n        <button role=\"button\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickFinish\" dojoAttachPoint=\"_finishButton\">${finishLabel}</button>\r\n        <button role=\"button\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickCancel\" dojoAttachPoint=\"_cancelButton\">${cancelLabel}</button>\r\n      </div>\r\n    </div>\r\n  </div>\r\n</div>\r\n";
			} else {
				this.templateString="<div class=\"tiv-wiz\" role=\"application\">\r\n  <div dojoType=\"dijit.layout.BorderContainer\" design=\"headline\" gutters=\"false\" class=\"tiv-wiz-outer\" dojoAttachPoint=\"_outerContainer\">\r\n    <div dojoType=\"dijit.layout.BorderContainer\" gutters=\"false\" liveSplitters=\"false\" design=\"sidebar\" region=\"center\" style=\"width:100%;height:100%;\" dojoAttachPoint=\"_borderArea\">\r\n      <div dojoType=\"dijit.layout.ContentPane\" region=\"leading\" splitter=\"false\" class=\"tiv-wiz-toc\" dojoAttachPoint=\"_tocArea\">\r\n        <ol dojoAttachPoint=\"_toc\"></ol>\r\n      </div>\r\n      <div dojoType=\"dijit.layout.StackContainer\" region=\"center\" splitter=\"false\" dojoAttachPoint=\"_pageContainer\" class=\"tiv-wiz-work-area\">\r\n        <form id=\"${id}_form\" dojoAttachPoint=\"containerNode\" action=\"${action}\" method=\"${method}\"></form>              \r\n      </div>\r\n    </div>\r\n    <div dojoType=\"dijit.layout.ContentPane\" region=\"bottom\" splitter=\"false\" class=\"tiv-wiz-footer\" dojoAttachPoint=\"_buttonArea\">\r\n      <div class=\"tiv-wiz-buttons\" dojoAttachPoint=\"_buttons\">\r\n        <button role=\"button\" type=\"button\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickBack\" dojoAttachPoint=\"_backButton\">${backLabel}</button>\r\n        <button role=\"button\" type=\"button\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickNext\" dojoAttachPoint=\"_nextButton\">${nextLabel}</button>\r\n        <button role=\"button\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickFinish\" dojoAttachPoint=\"_finishButton\">${finishLabel}</button>\r\n        <button role=\"button\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickCancel\" dojoAttachPoint=\"_cancelButton\">${cancelLabel}</button>\r\n      </div>\r\n    </div>\r\n  </div>\r\n</div>\r\n";
			}
			this.backLabel = this._resources.BACK;
			this.nextLabel = this._resources.NEXT;
			this.finishLabel = this._resources.FINISH;
			this.cancelLabel = this._resources.CANCEL;
			this.inherited(arguments);
		},
  
		buildRendering: function () {
			// summary:  Builds and renders the container whenever changes occur.
			// tags: private
		
			this.inherited(arguments);
			// if ToC layout is horizontal... 
			if ('h' === this.layout) {
				var child = this._tocArea;
				// add main step area
				this.tocStepsArea = dojo.create("div", {});          
				dojo.place(this.tocStepsArea, child.domNode, 0);
				if (this.renderNoArrows) {
					dojo.addClass(this.tocStepsArea, "steps-area-noarrows");            
					dojo.place(this._toc, this.tocStepsArea, 0);
				} else {
					dojo.addClass(this.tocStepsArea, "steps-area");            
					// create slider components which are hidden until needed
					this.slider = new dojoe.wizard.WizardToCSlider({
						toc: this._toc, 
						tocSteps: this.tocStepsArea, 
						ltr: this.isLeftToRight()
					}, null);
				}
				// add hidden sub-step area
				this.tocSubstepsArea = dojo.create("div", {className: "substeps-area"});
				dojo.place(this.tocSubstepsArea, child.domNode, 1);          
			}    
		},
  
		_setContainerType: function (page) {
			// summary:  Sets the container type for Wizard
			// tags: private
		
			page.attr("wizard", this);
		},
	  
		updateUI: function () {        
			// summary:  updates UI whenever changes occurs.
			// tags: public
			
			// disable the Back/Next buttons if we are on the first/last page
			var curr_page = this._pageContainer.selectedChildWidget;
			var page_count = this._pages.length;
			if (curr_page != null) {
				var isFirst = true, isLast = true;
				if (page_count > 0) {
					isFirst = curr_page === this._pages[0];
					isLast =  curr_page === this._pages[this._pages.length - 1];
				}
				this._backButton.attr("disabled", isFirst);
				this._nextButton.attr("disabled", isLast);
				// disable the Next button if the selected page does not validate
				var isCurrentPageValid = curr_page && curr_page.isValid(); 
				if (!isCurrentPageValid) {
					this._nextButton.attr("disabled", true);    
				}
				// disable the Finish/OK button until all pages are valid
				var allValid = dojo.every(this._pages, function (p) {
					var v = p.isValid();
					v = v && (p.visited || p === curr_page);
					return v;
				}, this);
				this._finishButton.attr("disabled", !allValid);
				this.inherited(arguments);
				if ('h' === this.layout) {     
					//55135: STG: Wizard usability/layout issues in OneUI
					/* Bobby: 12Oct2012, ToC part of the wizard needs to get laid down before the bottom part. 
					   Otherwise, the bottom part is positioned too low, and several other problems occur*/
					if (this.slider != null) {
						// calculate required width for all step arrows in ToC
						this.slider.recalcToCWidth();
						// update slider
						this.slider.updateSliderUI(null);      
					}					
					
					// update UI for all substeps
					// substeps may be hidden due to jump back and need to be updated too
					dojo.forEach(this._pages, function (aPage) {
						if (aPage.hasSubsteps()) {
							dojo.query("li[pageID]", aPage.getSubstepsList()).forEach(function (n, i) {
								var pid = dojo.attr(n, "pageID");
								var p = dijit.byId(pid);
								if (p.visited) {
									dojo.query("a",n).forEach(function (a) {
										a.setAttribute("title","visited");
									},this);
								}
								dojo[(p && p.visited) ? "addClass" : "removeClass"](n, "tiv-page-visited");
								dojo[(p && (!p.visited || p.validated)) ? "removeClass" : "addClass"](n, "tiv-page-invalid");
								dojo[(p == curr_page) ? "addClass" : "removeClass"](n, "tiv-page-selected");  
								dojo[(p == curr_page) ? "removeClass" : "addClass"](n, "tiv-page-disabled");
							}, this);
						}
					}, this);
		  
					// if current page is a substep update main step to look selected
					if (!curr_page.isMainStep()) {
						var mainPage = curr_page.getMainPage();
						dojo.query("li[pageID]", this._toc).every(function (n, i) {
							var pid = dojo.attr(n, "pageID");
							if (pid == mainPage.id) {
								dojo.addClass(n, "tiv-page-selected");
								return false;
							} else {
								return true;
							}
						});              
					}
					// show or hide substeps area
					var newHeight = 0;
					if (curr_page.hasSubsteps() || !curr_page.isMainStep()) {
						dojo.setStyle( this.tocSubstepsArea, "display", "block" );
						newHeight = dojo.getMarginBox( this.tocStepsArea ).h + dojo.getMarginBox( this.tocSubstepsArea ).h;				
					} else {
						dojo.setStyle( this.tocSubstepsArea, "display", "none" );
						newHeight = dojo.getMarginBox( this.tocStepsArea ).h;
					}
					if (dojo.style(this._pageContainer.domNode, "top") != newHeight) {
						dojo.style(this._pageContainer.domNode, "top", newHeight + "px");        
						dojo.style(this._tocArea.domNode, "height", newHeight + "px");
					}
				}
			}
		},

		resize: function (newSize) {
			// summary:  Overrides base function to also resize the slider.  Resizes the Wizard according to the new dimensions.
			//newSize: array	newSize  is the new dimensions with properties w for width and h for height
			// tags: public
			
			if (newSize != null) {
				this.inherited(arguments);
				if (this.slider && newSize.w != null) {
					//Note: for updating the slider, cannot rely on connecting a handler to the resize event for the _tocArea div.  Problems occur in Firefox where the event is not fired often enough so sometimes the slider button will disappear
					this.slider.updateSliderUI(newSize.w); 
				}
			}
		},
	  
		restartWizard: function () {
			// summary:   Restarts the Wizard
			// tags: public
			
			if (this.slider != null) {
				this.slider.reset();
			}
		
			// the bottom button area might have been hidden
			this.showAllButtons(); //Show the buttons (may have been hidden)
			// individual buttons might have been hidden
			this.hideBackButton(false);
			this.hideNextButton(false);
			this.hideCancelButton(false);
			this.hideFinishButton(false);
			// or maybe disabled
			this.enableNextButton(true);
			this.enableCancelButton(true);
			this.enableFinishButton(false);
			// get all of the children
			var theKids = this._pageContainer.getChildren();
			// do for all children
			var i;
			for (i = 0; i < theKids.length; i++) {
				// mark as not visited
				theKids[i].visited = false; //Remove the "visited" flag
				// hide any kids
				theKids[i].hideSubsteps();	//Collapse substeps
				// if the child supports the "reset" method
				if ("function" == typeof theKids[i].reset) {
					// give the kid a chance to reset
					theKids[i].reset();  //Give each page a chance to reset to the original values
				}
			}
			// the TOC might have been disabled
			this.enableJumpBack();  //Enable jumping backwards (may have been disabled)
			// and then select the first page
			this.selectPage(theKids[0]); //Reset to the first page
		},

		resetFinishLabel: function (newName) {
			// summary:   Resets the label for the "Finish" button
			// tags: public
			
			dojo.attr(this._finishButton, "label", newName);
		},

		markPreviousPagesVisited: function (aPage) {
			// summary:   Marks the  pages as having been visited up to the one specified
			// tags: public
			
			// get all of the children
			var theKids = this._pageContainer.getChildren();
			// do for all children
			var foundPage = false;
			var i;
			for (i = 0; !foundPage && i < theKids.length; i++) {
				// if this is not the page specified
				if (aPage != theKids[i]) {
					// if the kid is not disabled
					if (theKids[i].isDisabled() != true) {
						// mark as visited
						theKids[i].visited = true;
					}
					// else this is the end of the line
				} else {
					// mark as found
					foundPage = true;
				}
			}
		},

		disableJumpBack: function (exceptFor) {
			// summary:   Disable jumping back to every page except for the one specified
			// tags: public
		
			// get all of the children
			var theKids = this._pageContainer.getChildren();
			// do for all children
			var i;
			for (i = 0; i < theKids.length; i++) {
				// if this is not the exception)
				if (exceptFor != theKids[i]) {
					// disable it
					theKids[i].disableJumpBack();
				}
			}
		},

		enableJumpBack: function () {
			// summary:   Enable jumping back to every page
			// tags: public
			
			// get all of the children
			var theKids = this._pageContainer.getChildren();
			// do for all children
			var i;
			for (i = 0; i < theKids.length; i++) {
				// enable it
				theKids[i].enableJumpBack();
			}
		},

		_onPageAdded: function (page, insertIndex) {
			// summary:   When a page is being added it might be a substep. If so it needs to be added to the previous main step
			// tags: private
			
			this.inherited(arguments);
			// if this is a substep
			if (page.isMainStep() == false) {
				// get the substep's main page
				var mainPage = page.getMainPage();
				if (mainPage != null) {
					// add as a substep
					mainPage.addSubstep(page);
				} else {
					// message
					console.error("failed to add substep to main page");
				}
			}
		},
	  
		getPreviousMainPage: function (page) {
			// summary:   Gets the previous main page.  If page is a substep, gets the main page for this substep.  Note the page may be disabled.
			// tags: public
		
			// get the previous page
			var previousPage = this._pageContainer._getSiblingOfChild(page, -1);
			// while the previous page is a substep
			while (previousPage.isMainStep() == false) {
				// get the previous page
				previousPage = this._pageContainer._getSiblingOfChild(previousPage, -1);
			}
		   // if the previous page is a main page
			if (previousPage.isMainStep() == false) {
				previousPage = null;
			}
			return previousPage;
		},
	  
		getNextMainPage: function (page) {
			// summary:   Gets the next main page.  Note the page may be disabled.
			// tags: public
			
			// get the next page
			var nextPage = this._pageContainer._getSiblingOfChild(page, 1);
			// while the next page is a substep
			while (nextPage.isMainStep() == false) {
				// get the next page
				nextPage = this._pageContainer._getSiblingOfChild(nextPage, 1);
			}
			// if the next page is a main page
			if (nextPage.isMainStep() == false) {
				nextPage = null;
			}
			return nextPage;    
		},
		
		jumpToNextLoopStep: function () {
			// summary:   Jumps to the next enabled step in the loop which may be a substep or the main step.
			// tags: public

			var nextPage = null;
			// get the current page
			var tempPage = this.lastPageSelected;
			if (tempPage.isMainStep()) {
				// next step must be a substep so return first enabled substep
				if (tempPage.hasSubsteps()) {
					dojo.every(tempPage.getSubsteps(), function (substep) {
						if (!substep.isDisabled()) {
							nextPage = substep;
							return false;
						} else {
							return true;
						}
					}, this);
				}
			} else {
				// current is a substep, get the next substep
				var nextPossible = this._pageContainer._getSiblingOfChild(tempPage, 1);
				// while the next page is a substep and disabled
				while (nextPossible.isMainStep() == false && nextPossible.isDisabled()) {
					// get the next page
					nextPossible = this._pageContainer._getSiblingOfChild(nextPossible, 1);
				}
				// if next page is a substep save it for selection
				if (nextPossible != null && !nextPossible.isMainStep()) {
					nextPage = nextPossible;
				} else {
					// otherwise next page is a main page, make sure we
					// loop around and load the main page in this loop
					nextPage = tempPage.getMainPage();
				}	
			}
			if (nextPage != null) {
				// select the new page
				this.selectPage(nextPage);
			}
		},
	  
		jumpToPreviousLoopStep: function () {
			// summary:   Jumps to the previous enabled step in the loop which may be a substep or the main step.  Note if the current page is the main step, the jump  will not occur.
			// tags: public
			
			// get the current page
			var tempPage = this.lastPageSelected;
			if (!tempPage.isMainStep()) {
				// get the previous page
				var previousPossible = this._pageContainer._getSiblingOfChild(tempPage, -1);
				// while the next page is a substep and disabled
				while (previousPossible.isMainStep() == false && previousPossible.isDisabled()) {
					// get the previous page
					previousPossible = this._pageContainer._getSiblingOfChild(previousPossible, -1);
				}		
				if (previousPossible != null) {
					// select the new page
					this.selectPage(previousPossible);
				}
			}
		},

		skipToNextMainStep: function () {
			// summary:  Using the skip function takes you to the next main page that is enabled. But you still want to hide any substeps if you happen to be in a main  page with substeps, and you want to hide all of the substeps for a main page if you happen to be in a substep. Also if the next main step contains substeps but the main step should not show its page, the first enabled substep page will be selected.
			// tags: public
			
			// get the current page
			var tempPage = this.lastPageSelected;
			// assume it is a main step
			var mainPage = tempPage;
			// while still on a substep
			while (mainPage.isMainStep() == false) {
				// back up another page
				mainPage = this._pageContainer._getSiblingOfChild(mainPage, -1);
			}
			// hide any substeps
			mainPage.hideSubsteps();
			// mark it as visited
			tempPage.visited = true;
			// find the next page
			var whoIsNext = this._pageContainer._getSiblingOfChild(tempPage, 1);
			// while that page is a substep
			while (whoIsNext != null && (whoIsNext.isMainStep() == false || whoIsNext.isDisabled())) {
				// find the next page
				whoIsNext = this._pageContainer._getSiblingOfChild(whoIsNext, 1);
			}
			// select the new page
			if (whoIsNext != null) {
				// check if we need to show substep directly
				if (whoIsNext.showTitleOnly()) {
					this.goToSubstep(whoIsNext);
				} else {
					this.selectPage(whoIsNext);
				}
			}     
		},	
		
		jumpBackToPage: function (aPage) {
			// summary:  The user has clicked on something in the table of contents, so the container needs to display that page.Hide any substeps of the jump off page if the page is not a substep of the jump off page. Let the developer decide when to mark the pages invalid. This also means that the page where the jump originated needs to be marked as visited in order to return to that page.  This can be overridden if allowMarkAsComplete returns false.
			// tags: public
			
			// the jump off point might have substeps or be a substep
			// hide them as long as page to jump to isn't one of those substeps
			var jumpOffMainPage = (this.lastPageSelected.isMainStep()) ? this.lastPageSelected
																   : this.lastPageSelected.getMainPage();
			var jumpToMainPage = (aPage.isMainStep()) ? aPage : aPage.getMainPage();    
			if (jumpOffMainPage != jumpToMainPage) {
				jumpOffMainPage.hideSubsteps();    
			}
			// mark where we were as having been visited
			if (this.lastPageSelected.allowMarkAsComplete()) {
				this.lastPageSelected.visited = true;
			}
			// check if we need to show substep directly
			if (aPage.showTitleOnly()) {
				this.goToSubstep(aPage);
			} else {
				// then let the parent class handle selecting the page
				this.selectPage(aPage);
			}
		
		},
	  
		goToSubstep: function (mainPage) {
			// summary:   Selects the first enabled substep for the given main page.
			// tags: public
			
			mainPage.showSubsteps();
			dojo.every(mainPage.getSubsteps(), function (substep) {
				if (!substep.isDisabled()) {
					this.selectPage(substep);
					return false;
				} else {
					return true;
				}
			}, this);    
		},

		hideAllButtons: function () {
			// summary:   Hide all of the buttons by adding a class
			// tags: public
			
			dojo.addClass(this._buttonArea.domNode, "tiv-wiz-footer-hidden");
			this._outerContainer.resize();
		},

		showAllButtons: function () {
			// summary:  Show all of the buttons by removing a class
			// tags: public
			
			dojo.removeClass(this._buttonArea.domNode, "tiv-wiz-footer-hidden");
			this._outerContainer.resize();
		},

		hideBackButton: function (shouldHide) {
			// summary:   Hide (or show) the "back" button 
			// tags: public
			
			if (shouldHide) {
				dojo.style(this._backButton.domNode, "display", "none");
			} else {
				dojo.style(this._backButton.domNode, "display", "inline-block");
			}
		},

		hideNextButton: function (shouldHide) {
			// summary:   Hide (or show) the "next" button
			// tags: public
			
			if (shouldHide) {
				dojo.style(this._nextButton.domNode, "display", "none");
			} else {
				dojo.style(this._nextButton.domNode, "display", "inline-block");
			}
		},

		enableBackButton: function (shouldEnable) {
			// summary:   Enable (or disable) the "back" button
			// tags: public
			
			this._backButton.attr("disabled", !shouldEnable);
		},

		enableNextButton: function (shouldEnable) {
			// summary:    Enable (or disable) the "next" button
			// tags: public
			
			this._nextButton.attr("disabled", !shouldEnable);
		},

		unmarkVisitedPages: function (aPage) {
			// summary:   If a change is made to a page that was reached by jumping back, that might cause subsequent pages to have information or choices  that are no longer valid. Call this function to invalidate the selected page and all subsequent pages.
			// tags: public
			
			while (aPage != null) {
				aPage.visited = false;
				aPage = this._pageContainer._getSiblingOfChild(aPage, 1);
			}
			this.updateUI();
		},

		_onPageSelected: function (page) {
			// summary:   There are some differences when a page is selected. This is where the last page selected is reset. The selected page should not be marked as having been visited, a change from the parent class.
			// tags: public
			
			// keep track of which page was last selected
			this.lastPageSelected = page;
			// let the parent class handle this
			this.OnPageSelected(this, page);
			// if the page can handle this function
			if ("function" == typeof page.OnPageSelected) {
				// let the page know it has been selected
				page.OnPageSelected(this);
			}
			// update the display
			this.updateUI();
			// if the page can handle this function
			if ("function" == typeof page.AfterPageSelected) {
				// let the page get final control
				page.AfterPageSelected(this);
			}
		},

		_onClickFinish: function (event) {
			// summary:   When "Finish" is clicked any substeps that are open should be  hidden
			// tags: public
			
			// get the page current in view
			var tempPage = this.lastPageSelected;
			// see if the page wants to allow the customer to move on
			var shouldAllow = tempPage.allowFinish();
			// if so
			if (shouldAllow) {
				// while this is a substep
				while (tempPage.isMainStep() == false) {
					// get the page preceding this one
					tempPage = this._pageContainer._getSiblingOfChild(tempPage, -1);
				}
				// then hide any substeps
				tempPage.hideSubsteps();
				// and allow any additional processing to occur
				
				
				this.OnOK(this);
			}
		},
		

		/**
		* This has a number of differences from the parent class.
		* 
		* If a page has substeps, then they should be hidden from view at
		* this time.
		* 
		* If the previous page is really a substep then the page that should
		* be shown is the parent of that substep.
		* 
		* When backing up to the parent of a substep the substeps should be
		* marked as not having been visited.
		* 
		* Disabled pages should be avoided.
		* 
		* Changing behavior:
		* - To completely override the behavior of the Back button, have
		* allowBack() of current page return false and connect customized 
		* handler function to _onClickBack event.
		* - To perform additional processing after previous page has been
		* selected, connect customized handler function to OnBack event.
		*/
		_onClickBack: function (evt) {
			// summary:   Enable jumping back to every page
			// tags: public
				
			// allow any additional processing to occur
			this.OnBack(this);
			// get the current position
			var tempPage = this.lastPageSelected;
			// see if the page wants to allow the customer to move back
			var shouldAllow = tempPage.allowBack();
			// if so
			if (shouldAllow) {
				// get the next page
				var whoIsNext = tempPage;
				whoIsNext = this._pageContainer._getSiblingOfChild(whoIsNext, 1);   
				// if the next page is marked as unvisited (no jump back)
				if (whoIsNext != null && !whoIsNext.visited) {
				// mark current page as unvisited
					tempPage.visited = false;      
				}
				var startingPage = tempPage;
				// if the current page is not a substep
				if (tempPage.isMainStep() == true) {
					// hide any substeps for this page
					tempPage.hideSubsteps();         
				} else {
					// current page is a substep
					var mainPage = tempPage.getMainPage();
					if (whoIsNext != null && !whoIsNext.visited) {
						// mark main step as unvisited as long as no jump back
						mainPage.visited = false;
					}
					if (mainPage.showTitleOnly()) {
						// get the previous enabled page
						var previous = this._pageContainer._getSiblingOfChild(tempPage, -1);
						while (previous != null && previous.isDisabled()) {
							previous = this._pageContainer._getSiblingOfChild(previous, -1);      
						}
					  
					  // if previous page is this substep's main page, then skip it
						if (previous == mainPage) {
							startingPage = previous;
							mainPage.hideSubsteps();
						}
					}
				}
			  
				// get the previous page that is not disabled
				var whoIsPrevious = this._pageContainer._getSiblingOfChild(startingPage, -1);
				  
				// while that page is marked disabled or if page is a substep and its
				// main page is disabled
				while (whoIsPrevious != null && (whoIsPrevious.isDisabled() || 
						!whoIsPrevious.isMainStep() && whoIsPrevious.getMainPage().isDisabled())) {
					// get the previous page
					whoIsPrevious = this._pageContainer._getSiblingOfChild(whoIsPrevious, -1);      
				}
				  
			  // if there is still a page to load
				if (whoIsPrevious != null) {      
				
					// if that page is a substep 
					if (whoIsPrevious.isMainStep() == false) {
						// keep track of substep
						var subStepToSelect = whoIsPrevious;
						// get previous main page
						whoIsPrevious = subStepToSelect.getMainPage();
						// if main page is not part of a loop
						// (otherwise select main page)
						if (!whoIsPrevious.isPartOfLoop()) {
							// make all substeps for this page visible then the substep can be selected
							whoIsPrevious.OnPageSelected(this);
							// reset page to be selected to our substep
							whoIsPrevious = subStepToSelect;
						}        
					}
					// select that page
					this.selectPage(whoIsPrevious);
				}
		  		// mark this page widget as not having been visited if
				// we didn't get here from jumping back
				if (!tempPage.visited) {
					this._pageContainer.selectedChildWidget.visited = false; 
				}
			  
			}
		},

		/** 
		* Moving to the next page is slightly different if there are substeps
		* involved.
		* 
		* Substeps should only be seen when their parent is the last main
		* step to be visited. They need to disappear when the next main step
		* is entered.
		* 
		* Disabled pages should be avoided.
		* 
		* Changing behavior:
		* - To completely override the behavior of the Next button, have
		* allowNext() of current page return false and connect customized 
		* handler function to _onClickNext event.
		* - To perform additional processing after next page has been
		* selected, connect customized handler function to OnNext event.
		*/
		_onClickNext: function (evt) {
			// summary:   Enable jumping next to every page
			// tags: public
			
			// allow any additional processing to occur
			this.OnNext(this);
			// get the page current in view
			var tempPage = this.lastPageSelected;
			// see if the page wants to allow the customer to move on
			var shouldAllow = tempPage.allowNext();
			// if so
			if (shouldAllow) {
				// if current page in view has no substeps, mark the page as 
				// having been visited
				if (!tempPage.hasSubsteps()) {
					tempPage.visited = true;
				}
				// get the next page
				var nextPossible;    
				if (tempPage.isPartOfLoop()) {
					// current page is part of a loop so get next main page
					nextPossible = this.getNextMainPage(tempPage);
				} else {
					nextPossible = this._pageContainer._getSiblingOfChild(tempPage, 1);
				}

				// while that page is marked disabled
				while (nextPossible != null && nextPossible.isDisabled()) {
				// if disabled page has substeps
					if (nextPossible.hasSubsteps()) {
						// get next main page
						nextPossible = this.getNextMainPage(nextPossible);
					} else {
				  		// get the next page
						nextPossible = this._pageContainer._getSiblingOfChild(nextPossible, 1);
					}
				}
				// if there is still a next page to load
				if (nextPossible != null) {
					// if next page is a main page and current page is a substep or main page for loop
					if (nextPossible.isMainStep() && (!tempPage.isMainStep() || tempPage.isPartOfLoop())) {
						// get the current main page
						var currentMainPage;
						if (tempPage.isPartOfLoop()) {
							if (tempPage.isMainStep()) {
								currentMainPage = tempPage;
							} else {
								currentMainPage = tempPage.getMainPage();
							}
							// mark all loop substeps as visited
							dojo.forEach(currentMainPage.getSubsteps(), function (page) {
								page.visited = true;
							}, this);
						} else {
							currentMainPage = tempPage.getMainPage();          
						}
						// hide substeps for that main step
						currentMainPage.hideSubsteps();
					    // mark the main step as having been visited
						currentMainPage.visited = true;
					}       
					// check if we need to show substep directly
					if (nextPossible.showTitleOnly()) {
						this.goToSubstep(nextPossible);
					} else {
						// select that page
						this.selectPage(nextPossible);
					}
				}      
			}        
		},
		
		_onClickCancel: function (event) {
			// summary:   Called on click of cancel button. If showCancelWarning is true displays Confirmation Message otherwise calls the default OnCancel method.
			// tags: private
			
			if(this.showCancelWarning == true) {
				this._showCancelMessage(this);
			} else {
				this.OnCancel(this);
			}
		}, 
	  
		OnBack: function (wiz) {
		},
	  
		OnNext: function (wiz) {
		},
	  
		//methods added by TED team.
		//Latha 5 July 2011 : Basing on the requirement from TIP team , removing the display of Confirmation messages after click on OK button
		OnOK: function (wiz) {
		/*	var dialog = new dojoe.messagedialog.MessageDialog({ message: this._resources.FINISH_MSG,
																messageId: "",	
																type: "Confirm",
																callback:  dojo.hitch(this, this.onOkClick),
																callbackParms: wiz,
																buttons: [this._resources.CONFIRM_YES, this._resources.CONFIRM_NO],
																showCheckbox: false                                         
																});
			dialog.show(); */
		},
		
		/*onOkClick : function (buttonPressedId, messageId, checked, wiz) {
		
			if (buttonPressedId == 0) {
				pages = wiz.getChildren(); 
				dojo.forEach(pages, function (p, i) {
					wiz.removePage(p);
					//p.destroy();
				}, this);
				this.hideAllButtons();
				this._outerContainer.destroy();
			}
		}, */
		
		//Override this method to execute the user defined implementation on click of cancel button in the wizard.
	  
		OnCancel: function (wiz) {
			// override or connect to me
			
		},
		
		_showCancelMessage : function(wiz) {
			// summary:   Displays the Confirmation Message.
			// tags: private
		
			var dialog = new dojoe.messagedialog.MessageDialog({message: this._resources.CANCEL_MSG,
																messageId: "",
																type: "Confirm",
																callback:  dojo.hitch(this, this._onConfirm),
																callbackParms: wiz,
																buttons: [this._resources.CONFIRM_YES, this._resources.CONFIRM_NO],
																showCheckbox: false                                         
																});
			dialog.show();
		},
		
		_onConfirm : function (buttonPressedId, messageId, checked, wiz) {
			// summary:   Call back method for Confirmation Message. Called once the user selects the option on the confirmation messgage dialog.
			// tags: private
		
			if (buttonPressedId == 0) {
				var pages = wiz.getChildren(); 
				dojo.forEach(pages, function (p, i) {
					wiz.removePage(p);
					//p.destroy();
				}, this);
				this.hideAllButtons();
				this._outerContainer.destroy();
				//dijit.byId(wiz).destroy();
			}
		}, 
		getChildren : function () {
			// summary:   To get children of the  wizard
			// tags: public
			
			var page = [];
			page = this.inherited(arguments);
			return page;
		},
	  
		removePage: function (p) { //p is a widget
			// summary:   To remove a page from Wizard
			// tags: public
		
			this.inherited(arguments);
		},
		
		//Latha:06May2011: updating destroy for complete cleanup
		destroy : function () {
			// summary:  Destroys the wizard.
			// tags: public
			try {
				delete this._resources;
				var widgetArr = this.getChildren();
				dojo.forEach(widgetArr, function (child) {
					if(child){
						if(child.destroyRecursive){
							child.destroyRecursive();
						} else if(child.destroy){
							child.destroy();
						}
					}
				});
				this.inherited(arguments);
			} catch (ex) {
				console.warn("Widget destroy failed: "+this.id+", ex="+ex.name+":"+ex.message);
			}
		}
	});


});

},
'dojox/grid/enhanced/plugins/Menu':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/_base/html",
	"dojo/_base/event",
	"dojo/keys",
	"../_Plugin",
	"../../EnhancedGrid"
], function(declare, array, lang, html, evt, keys, _Plugin, EnhancedGrid){

var Menu = declare("dojox.grid.enhanced.plugins.Menu", _Plugin, {
	// summary:
	//		 Provides context menu support, including header menu, row menu, cell menu and selected region menu
	// example:
	//		<div dojoType="dojox.grid.EnhancedGrid"
	//			plugins="{menus:{headerMenu:"headerMenuId", rowMenu:"rowMenuId", cellMenu:"cellMenuId",
	//							   selectedRegionMenu:"selectedRegionMenuId"}}" ...>
	//		</div>
	
	//name: String
	//		Plugin name
	name: "menus",

	//name: [const] Array
	//		menu types
	types: ['headerMenu', 'rowMenu', 'cellMenu', 'selectedRegionMenu'],
	
	constructor: function(){
		var g = this.grid;
		g.showMenu = lang.hitch(g, this.showMenu);
		g._setRowMenuAttr = lang.hitch(this, '_setRowMenuAttr');
		g._setCellMenuAttr = lang.hitch(this, '_setCellMenuAttr');
		g._setSelectedRegionMenuAttr = lang.hitch(this, '_setSelectedRegionMenuAttr');
	},
	onStartUp: function(){
		var type, option = this.option;
		for(type in option){
			if(array.indexOf(this.types, type) >= 0 && option[type]){
				this._initMenu(type, option[type]);
			}
		}
	},
	_initMenu: function(/*String*/menuType, /*String | Widget(dijit.Menu)*/menu){
		var g = this.grid;
		if(!g[menuType]){//in case already created in _Grid.postCreate()
			var m = this._getMenuWidget(menu);
			if(!m){return;}
			g.set(menuType, m);
			if(menuType != "headerMenu"){
				m._scheduleOpen = function(){return;};
			}else{
				g.setupHeaderMenu();
			}
		}
	},
	_getMenuWidget: function(/*String|Widget(dijit.Menu)*/menu){
		// summary:
		//		Fetch the required menu widget(should already been created)
		return (menu instanceof dijit.Menu) ? menu : dijit.byId(menu);
	},
	_setRowMenuAttr: function(/*Widget(dijit.Menu)*/menu){
		// summary:
		//		Set row menu widget
		this._setMenuAttr(menu, 'rowMenu');
	},
	_setCellMenuAttr: function(/*Widget(dijit.Menu)*/menu){
		// summary:
		//		Set cell menu widget
		this._setMenuAttr(menu, 'cellMenu');
	},
	_setSelectedRegionMenuAttr: function(/*Widget(dijit.Menu)*/menu){
		// summary:
		//		Set row menu widget
		this._setMenuAttr(menu, 'selectedRegionMenu');
	},
	_setMenuAttr: function(/*Widget(dijit.Menu)*/menu, /*String*/menuType){
		// summary:
		//		Bind menus to Grid.
		var g = this.grid, n = g.domNode;
		if(!menu || !(menu instanceof dijit.Menu)){
			console.warn(menuType, " of Grid ", g.id, " is not existed!");
			return;
		}
		if(g[menuType]){
			g[menuType].unBindDomNode(n);
		}
		g[menuType] = menu;
		g[menuType].bindDomNode(n);
	},
	showMenu: function(/*Event*/e){
		// summary:
		//		Show appropriate context menu
		//		Fired from dojox.grid.enhanced._Events.onRowContextMenu, 'this' scope - Grid
		//		TODO: test Shift-F10
		var inSelectedRegion = (e.cellNode && html.hasClass(e.cellNode, 'dojoxGridRowSelected') ||
			e.rowNode && (html.hasClass(e.rowNode, 'dojoxGridRowSelected') || html.hasClass(e.rowNode, 'dojoxGridRowbarSelected')));
		
		if(inSelectedRegion && this.selectedRegionMenu){
			this.onSelectedRegionContextMenu(e);
			return;
		}
		
		var info = {target: e.target, coords: e.keyCode !== keys.F10 && "pageX" in e ? {x: e.pageX, y: e.pageY } : null};
		if(this.rowMenu && (!this.cellMenu || this.selection.isSelected(e.rowIndex) || e.rowNode && html.hasClass(e.rowNode, 'dojoxGridRowbar'))){
			this.rowMenu._openMyself(info);
			evt.stop(e);
			return;
		}

		if(this.cellMenu){
			this.cellMenu._openMyself(info);
		}
		evt.stop(e);
	},
	destroy: function(){
		// summary:
		//		Destroy all resources.
		//		_Grid.destroy() will unbind headerMenu
		var g = this.grid;
		if(g.headerMenu){g.headerMenu.unBindDomNode(g.viewsHeaderNode);}
		if(g.rowMenu){g.rowMenu.unBindDomNode(g.domNode);}
		if(g.cellMenu){g.cellMenu.unBindDomNode(g.domNode);}
		if(g.selectedRegionMenu){g.selectedRegionMenu.destroy();}
		this.inherited(arguments);
	}
});

EnhancedGrid.registerPlugin(Menu/*name:'menus'*/);

return Menu;

});

},
'ibm/usmi/statuspod/JobStatusPod':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define([
	"ibm/tivoli/tip/TCTTable",
	"ibm/tivoli/tip/TCTNavModel",
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dijit/PopupMenuBarItem",
	"ibm/tivoli/tip/navwidget/dijit/DefaultMessageHandler",
	"./JobStatusDataSet",
	"idx/oneui/MenuDialog",
	"./JobStatusPodDialog",
	"dojo/DeferredList",
	"dojo/_base/Deferred",
	'dojo/_base/unload',
	"./Constants",
	"dojo/_base/connect",
	"dojo/string",
	"dojo/i18n!./nls/statuspodResources",
	"ibm/usmi/core/utils/Logger"
], function (TCTTable, TCTNavModel, dojo, declare, lang, PopupMenuBarItem, DefaultMessageHandler, Dataset,
		MenuDialog, JobStatusPodDialog, DeferredList, Deferred, baseUnload, Constants, connect, string, MRI, Logger) {
	return declare("ibm.usmi.statuspod.JobStatusPod", [ PopupMenuBarItem
	], {
		label : MRI.JobMenuBarItemLoadingLabel,

		dialog : null,

		dataset : null,

		jobsCallbacks : null,
		
		jobsCallbackHanlders: null,
		
		originCallback: null,

		parentIds : [
			"Jobs",
			"Errors",
			"Active",
			"Completed"
		],
		
		jobDatasetName: "SCHEDULER_STATUSPOD",
		
		sortProp : [ {
			attribute : "LAST_RUN",
			descending : true
		}
		],
		
		totalCount: 0,
		
		iconClass: "status-msgs-icon-confirmation",
		
		_logger : new Logger("ibm.usmi.statuspod", "JobStatusPod"),
		
		blockEvents : true,
		pendingEvents : 0,

		postCreate : function () {
			this.inherited(arguments);
			this.dialog = new JobStatusPodDialog();
			this.popup = new MenuDialog({
				content : this.dialog
			});
		},

		startup : function () {
			this._logger.finest("startup job status pod", "startup");
			this.inherited(arguments);
			connect.subscribe(Constants.updateJobsLabel, this, function (message) {
				this.updateLabel(message);
			});
			this.jobsCallbacks = [
				lang.hitch(this.dialog, this.dialog.updateParentsCountNumber),
				lang.hitch(this.dialog, this.dialog.updateErrorJobs),
				lang.hitch(this.dialog, this.dialog.updateActiveJobs),
				lang.hitch(this.dialog, this.dialog.updateCompletedJobs)
			];
			this.jobsCallbackHanlders = [
			    new Deferred(),  new Deferred(),  new Deferred(),  new Deferred()
			];
			this.originCallback=new Deferred();
			this.makeDataSet();

		},

		refreshCallback : function () {
			var log=this._logger;
			log.finest("job status pod refresh callback was called", "refreshCallback");
			if (this.blockEvents) { //Refresh is in progress, don't refresh
				this._logger.finest("job status pod refresh callback is blocking, set pending state", "refreshCallback");
				this.pendingEvents = this.pendingEvents+1; //save the need to refresh again later
				return;
			}
			this.blockEvents = true;
			this.pendingEvents = 0;
			this.label = MRI.JobPodRefreshing;
			this.containerNode.innerHTML = this.label;
			this.dialog.clearDescendants();
			if(this.dataset){
				this.dataset.destroy();
			}
			delete this.jobsCallbackHanlders;
			delete this.originCallback;
			this.jobsCallbackHanlders = [
			     new Deferred(),  new Deferred(),  new Deferred(),  new Deferred()
			];
			this.originCallback=new Deferred();
			this.makeDataSet();
		},
		
		_unblockEvents : function () {
			this._logger.finest("job status pod unblock the event refreshing", "_unblockEvents");
			this.blockEvents = false; //Refresh is done, we can now listen for events again
			if (this.pendingEvents > 0) { //did any more events come in during our refresh? if so, refresh again
				this._logger.finest("there are pending events, begine to refresh", "_unblockEvents");
				this.refreshCallback();
			}
		},

		makeDataSet : function () {
			var i;
			var dataset = new Dataset({
				baseUrl : "/ibm/tivoli/rest",
				provider : "USMI",
				datasource : "",
				dataset : this.jobDatasetName,
				clearCacheOnRefresh : true,
				onComplete : lang.hitch(this, function (dataset, kwArgs) {
					this._logger.finest("dataset loading completed ", "makeDataSet");
					this.dataset=dataset;
					this.fetchOriginals();
					var parentIds=this.parentIds;
					var callbacks = this.jobsCallbacks;
					var handlers = this.jobsCallbackHanlders;
					for (i = 0; i < parentIds.length; i++) {
						this.fetchItems(parentIds[i], callbacks[i], handlers[i]);
					}

				}),
				onError : function (error, exception, ioArgs) {
					var msgHandler = new DefaultMessageHandler({});
					msgHandler.displayException(exception);
				},
				parameters : {
					maxSPElementsToken : 10
				},
				defaultProperties : []
			});
			this.dataset=dataset;
			this.dialog.dataset=dataset;
			dataset.addEventListener(this, lang.hitch(this, this.refreshCallback));

			// only remove on Destroy so we don't do this at logout. 
			// deleting datasets at logout casues redirect popups
			//baseUnload.addOnUnload(dataset, "destroy");

			if(!this.jobsCallbackHanlders || !this.originCallback){
				return;
			}
			var deferreds = lang.clone(this.jobsCallbackHanlders);
			deferreds.push(this.originCallback);
			var handlers = new DeferredList(deferreds);
			handlers.then(lang.hitch(this, function (result) {
				var delay='5000';
				if(this.pendingEvents >= 3){
					//if too many pending events, then use large delay
					delay='15000';
				}
				this._logger.finest("completed all deferreds", "makeDataSet");
				this.updateLabel();
				window.setTimeout(dojo.hitch(this, this._unblockEvents), delay);
			}));
		},

		destroy : function () {
			 if (this.dataset) {
				 this.dataset.destroy();
				 this.dataset = null;
			 }
			 this.inherited(arguments);
		},
		fetchItems : function (parentId, callback, handler) {
			this.dataset.getRelatedItems({
				onComplete : lang.hitch(this, function (items, kwArgs) {
					callback(items);
					if(handler){
						handler.resolve();
					}
				}),
				onError : function (error, exception, ioArgs) {
					var msgHandler = new DefaultMessageHandler({});
					msgHandler.displayException(exception);
					if(handler){
						handler.resolve();
					}
				},
				itemId: parentId,
				start : 0,
				count : 10,
				scope : this,
				sort : this.sortProp,
				properties: ["all"],
				relationships : "child"
			});
		},
		
		fetchOriginals : function () {
			this.dataset.getOriginItems({
				onComplete : lang.hitch(this, function (items, kwArgs) {
					items = items || [];
					var itemIndex = 0;
					var itemsLength = items.length;
					if(itemsLength>0) {
						var item = items[0];
						this.totalCount = item.getProperty("CATEGORY_COUNT").getValue();
						
					}
					this.originCallback.resolve();
				}),
				onError : function (error, exception, ioArgs) {
					var msgHandler = new DefaultMessageHandler({});
					msgHandler.displayException(exception);
					if(this.originCallback){
						this.originCallback.resolve();
					}
				},
				start : 0,
				count : 1,
				properties: ["all"],
				scope : this
			});
		},

		updateLabel : function (count) {
			if (count && count > 0) {
				this.iconClass="status-msgs-icon-error";
			}
			if(count===0){
				this.iconClass="status-msgs-icon-confirmation";
			}
			this.label=string.substitute("<span class='${iconClass}'></span>", {
				iconClass: this.iconClass
			});
			this.label=this.label+lang.replace(MRI.JobMenuBarItemLabel, [this.totalCount]);
			this.containerNode.innerHTML = this.label;
		}
		
	});
});

},
'dijit/form/_FormValueMixin':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.set
	"dojo/keys", // keys.ESCAPE
	"dojo/_base/sniff", // has("ie"), has("quirks")
	"./_FormWidgetMixin"
], function(declare, domAttr, keys, has, _FormWidgetMixin){

/*=====
	var _FormWidgetMixin = dijit.form._FormWidgetMixin;
=====*/

	// module:
	//		dijit/form/_FormValueMixin
	// summary:
	//		Mixin for widgets corresponding to native HTML elements such as <input> or <select> that have user changeable values.

	return declare("dijit.form._FormValueMixin", _FormWidgetMixin, {
		// summary:
		//		Mixin for widgets corresponding to native HTML elements such as <input> or <select> that have user changeable values.
		// description:
		//		Each _FormValueMixin represents a single input value, and has a (possibly hidden) <input> element,
		//		to which it serializes it's input value, so that form submission (either normal submission or via FormBind?)
		//		works as expected.

		// readOnly: Boolean
		//		Should this widget respond to user input?
		//		In markup, this is specified as "readOnly".
		//		Similar to disabled except readOnly form values are submitted.
		readOnly: false,

		_setReadOnlyAttr: function(/*Boolean*/ value){
			domAttr.set(this.focusNode, 'readOnly', value);
			this.focusNode.setAttribute("aria-readonly", value);
			this._set("readOnly", value);
		},

		postCreate: function(){
			this.inherited(arguments);

			if(has("ie")){ // IE won't stop the event with keypress
				this.connect(this.focusNode || this.domNode, "onkeydown", this._onKeyDown);
			}
			// Update our reset value if it hasn't yet been set (because this.set()
			// is only called when there *is* a value)
			if(this._resetValue === undefined){
				this._lastValueReported = this._resetValue = this.value;
			}
		},

		_setValueAttr: function(/*anything*/ newValue, /*Boolean?*/ priorityChange){
			// summary:
			//		Hook so set('value', value) works.
			// description:
			//		Sets the value of the widget.
			//		If the value has changed, then fire onChange event, unless priorityChange
			//		is specified as null (or false?)
			this._handleOnChange(newValue, priorityChange);
		},

		_handleOnChange: function(/*anything*/ newValue, /*Boolean?*/ priorityChange){
			// summary:
			//		Called when the value of the widget has changed.  Saves the new value in this.value,
			//		and calls onChange() if appropriate.   See _FormWidget._handleOnChange() for details.
			this._set("value", newValue);
			this.inherited(arguments);
		},

		undo: function(){
			// summary:
			//		Restore the value to the last value passed to onChange
			this._setValueAttr(this._lastValueReported, false);
		},

		reset: function(){
			// summary:
			//		Reset the widget's value to what it was at initialization time
			this._hasBeenBlurred = false;
			this._setValueAttr(this._resetValue, true);
		},

		_onKeyDown: function(e){
			if(e.keyCode == keys.ESCAPE && !(e.ctrlKey || e.altKey || e.metaKey)){
				var te;
				if(has("ie") < 9 || (has("ie") && has("quirks"))){
					e.preventDefault(); // default behavior needs to be stopped here since keypress is too late
					te = document.createEventObject();
					te.keyCode = keys.ESCAPE;
					te.shiftKey = e.shiftKey;
					e.srcElement.fireEvent('onkeypress', te);
				}
			}
		}
	});
});

},
'dijit/ToolbarSeparator':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.setSelectable
	"./_Widget",
	"./_TemplatedMixin"
], function(declare, dom, _Widget, _TemplatedMixin){

/*=====
	var _Widget = dijit._Widget;
	var _TemplatedMixin = dijit._TemplatedMixin;
=====*/

	// module:
	//		dijit/ToolbarSeparator
	// summary:
	//		A spacer between two `dijit.Toolbar` items


	return declare("dijit.ToolbarSeparator", [_Widget, _TemplatedMixin], {
		// summary:
		//		A spacer between two `dijit.Toolbar` items

		templateString: '<div class="dijitToolbarSeparator dijitInline" role="presentation"></div>',

		buildRendering: function(){
			this.inherited(arguments);
			dom.setSelectable(this.domNode, false);
		},

		isFocusable: function(){
			// summary:
			//		This widget isn't focusable, so pass along that fact.
			// tags:
			//		protected
			return false;
		}
	});
});

},
'dijit/ProgressBar':function(){
define([
	"require",			// require.toUrl
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.toggle
	"dojo/_base/lang", // lang.mixin
	"dojo/number", // number.format
	"./_Widget",
	"./_TemplatedMixin",
	"dojo/text!./templates/ProgressBar.html"
], function(require, declare, domClass, lang, number, _Widget, _TemplatedMixin, template){

/*=====
	var _Widget = dijit._Widget;
	var _TemplatedMixin = dijit._TemplatedMixin;
=====*/

// module:
//		dijit/ProgressBar
// summary:
//		A progress indication widget, showing the amount completed
//		(often the percentage completed) of a task.


return declare("dijit.ProgressBar", [_Widget, _TemplatedMixin], {
	// summary:
	//		A progress indication widget, showing the amount completed
	//		(often the percentage completed) of a task.
	//
	// example:
	// |	<div data-dojo-type="ProgressBar"
	// |		 places="0"
	// |		 value="..." maximum="...">
	// |	</div>

	// progress: [const] String (Percentage or Number)
	//		Number or percentage indicating amount of task completed.
	// 		Deprecated.   Use "value" instead.
	progress: "0",

	// value: String (Percentage or Number)
	//		Number or percentage indicating amount of task completed.
	// 		With "%": percentage value, 0% <= progress <= 100%, or
	// 		without "%": absolute value, 0 <= progress <= maximum.
	//		Infinity means that the progress bar is indeterminate.
	value: "",

	// maximum: [const] Float
	//		Max sample number
	maximum: 100,

	// places: [const] Number
	//		Number of places to show in values; 0 by default
	places: 0,

	// indeterminate: [const] Boolean
	// 		If false: show progress value (number or percentage).
	// 		If true: show that a process is underway but that the amount completed is unknown.
	// 		Deprecated.   Use "value" instead.
	indeterminate: false,

	// label: String?
	//		Label on progress bar.   Defaults to percentage for determinate progress bar and
	//		blank for indeterminate progress bar.
	label:"",

	// name: String
	//		this is the field name (for a form) if set. This needs to be set if you want to use
	//		this widget in a dijit.form.Form widget (such as dijit.Dialog)
	name: '',

	templateString: template,

	// _indeterminateHighContrastImagePath: [private] URL
	//		URL to image to use for indeterminate progress bar when display is in high contrast mode
	_indeterminateHighContrastImagePath:
		require.toUrl("./themes/a11y/indeterminate_progress.gif"),

	postMixInProperties: function(){
		this.inherited(arguments);
		if(!("value" in this.params)){
			this.value = this.indeterminate ? Infinity : this.progress;
		}
	},

	buildRendering: function(){
		this.inherited(arguments);
		this.indeterminateHighContrastImage.setAttribute("src",
			this._indeterminateHighContrastImagePath.toString());
		this.update();
	},

	update: function(/*Object?*/attributes){
		// summary:
		//		Internal method to change attributes of ProgressBar, similar to set(hash).  Users should call
		//		set("value", ...) rather than calling this method directly.
		// attributes:
		//		May provide progress and/or maximum properties on this parameter;
		//		see attribute specs for details.
		// example:
		//	|	myProgressBar.update({'indeterminate': true});
		//	|	myProgressBar.update({'progress': 80});
		//	|	myProgressBar.update({'indeterminate': true, label:"Loading ..." })
		// tags:
		//		private

		// TODO: deprecate this method and use set() instead

		lang.mixin(this, attributes || {});
		var tip = this.internalProgress, ap = this.domNode;
		var percent = 1;
		if(this.indeterminate){
			ap.removeAttribute("aria-valuenow");
			ap.removeAttribute("aria-valuemin");
			ap.removeAttribute("aria-valuemax");
		}else{
			if(String(this.progress).indexOf("%") != -1){
				percent = Math.min(parseFloat(this.progress)/100, 1);
				this.progress = percent * this.maximum;
			}else{
				this.progress = Math.min(this.progress, this.maximum);
				percent = this.maximum ? this.progress / this.maximum : 0;
			}

			ap.setAttribute("aria-describedby", this.labelNode.id);
			ap.setAttribute("aria-valuenow", this.progress);
			ap.setAttribute("aria-valuemin", 0);
			ap.setAttribute("aria-valuemax", this.maximum);
		}
		this.labelNode.innerHTML = this.report(percent);

		domClass.toggle(this.domNode, "dijitProgressBarIndeterminate", this.indeterminate);
		tip.style.width = (percent * 100) + "%";
		this.onChange();
	},

	_setValueAttr: function(v){
		this._set("value", v);
		if(v == Infinity){
			this.update({indeterminate:true});
		}else{
			this.update({indeterminate:false, progress:v});
		}
	},

	_setLabelAttr: function(label){
		this._set("label", label);
		this.update();
	},

	_setIndeterminateAttr: function(indeterminate){
		// Deprecated, use set("value", ...) instead
		this.indeterminate = indeterminate;
		this.update();
	},

	report: function(/*float*/percent){
		// summary:
		//		Generates message to show inside progress bar (normally indicating amount of task completed).
		//		May be overridden.
		// tags:
		//		extension

		return this.label ? this.label :
				(this.indeterminate ? "&#160;" : number.format(percent, { type: "percent", places: this.places, locale: this.lang }));
	},

	onChange: function(){
		// summary:
		//		Callback fired when progress updates.
		// tags:
		//		extension
	}
});

});

},
'url:dijit/form/templates/DropDownButton.html':"<span class=\"dijit dijitReset dijitInline\"\n\t><span class='dijitReset dijitInline dijitButtonNode'\n\t\tdata-dojo-attach-event=\"ondijitclick:_onClick\" data-dojo-attach-point=\"_buttonNode\"\n\t\t><span class=\"dijitReset dijitStretch dijitButtonContents\"\n\t\t\tdata-dojo-attach-point=\"focusNode,titleNode,_arrowWrapperNode\"\n\t\t\trole=\"button\" aria-haspopup=\"true\" aria-labelledby=\"${id}_label\"\n\t\t\t><span class=\"dijitReset dijitInline dijitIcon\"\n\t\t\t\tdata-dojo-attach-point=\"iconNode\"\n\t\t\t></span\n\t\t\t><span class=\"dijitReset dijitInline dijitButtonText\"\n\t\t\t\tdata-dojo-attach-point=\"containerNode,_popupStateNode\"\n\t\t\t\tid=\"${id}_label\"\n\t\t\t></span\n\t\t\t><span class=\"dijitReset dijitInline dijitArrowButtonInner\"></span\n\t\t\t><span class=\"dijitReset dijitInline dijitArrowButtonChar\">&#9660;</span\n\t\t></span\n\t></span\n\t><input ${!nameAttrSetting} type=\"${type}\" value=\"${value}\" class=\"dijitOffScreen\" tabIndex=\"-1\"\n\t\tdata-dojo-attach-point=\"valueNode\"\n/></span>\n",
'dijit/Toolbar':function(){
define([
	"require",
	"dojo/_base/declare", // declare
	"dojo/_base/kernel",
	"dojo/keys", // keys.LEFT_ARROW keys.RIGHT_ARROW
	"dojo/ready",
	"./_Widget",
	"./_KeyNavContainer",
	"./_TemplatedMixin"
], function(require, declare, kernel, keys, ready, _Widget, _KeyNavContainer, _TemplatedMixin){

/*=====
	var _Widget = dijit._Widget;
	var _KeyNavContainer = dijit._KeyNavContainer;
	var _TemplatedMixin = dijit._TemplatedMixin;
=====*/

	// module:
	//		dijit/Toolbar
	// summary:
	//		A Toolbar widget, used to hold things like `dijit.Editor` buttons


	// Back compat w/1.6, remove for 2.0
	if(!kernel.isAsync){
		ready(0, function(){
			var requires = ["dijit/ToolbarSeparator"];
			require(requires);	// use indirection so modules not rolled into a build
		});
	}

	return declare("dijit.Toolbar", [_Widget, _TemplatedMixin, _KeyNavContainer], {
		// summary:
		//		A Toolbar widget, used to hold things like `dijit.Editor` buttons

		templateString:
			'<div class="dijit" role="toolbar" tabIndex="${tabIndex}" data-dojo-attach-point="containerNode">' +
			'</div>',

		baseClass: "dijitToolbar",

		postCreate: function(){
			this.inherited(arguments);

			this.connectKeyNavHandlers(
				this.isLeftToRight() ? [keys.LEFT_ARROW] : [keys.RIGHT_ARROW],
				this.isLeftToRight() ? [keys.RIGHT_ARROW] : [keys.LEFT_ARROW]
			);
		}
	});
});

},
'ibm/tivoli/tip/navwidget/topology/dijit/TopologyMenuLayer':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!ibm/tivoli/tip/navwidget/dijit/CustomContentTooltip,ibm/tivoli/tip/navwidget/dijit/AsynchContextMenu,ibm/tivoli/tip/navwidget/dijit/RESTContextMenuProvider,ibm/tivoli/tip/navwidget/topology/dijit/MenuProviderHelper,ibm/tivoli/tip/navwidget/topology/dijit/DefaultActionHelper,ibm/tivoli/tip/navwidget/properties/dijit/PropertiesNotebookGeneralTab,ibm/tivoli/tip/navwidget/dojo/NavWidgetUtil,ibm/tivoli/navmodel/Selection,ibm/tivoli/navmodel/Trace"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.topology.dijit.TopologyMenuLayer");

dojo.require("ibm.tivoli.tip.navwidget.dijit.CustomContentTooltip");
dojo.require("ibm.tivoli.tip.navwidget.dijit.AsynchContextMenu");
dojo.require("ibm.tivoli.tip.navwidget.dijit.RESTContextMenuProvider");
dojo.require("ibm.tivoli.tip.navwidget.topology.dijit.MenuProviderHelper");
dojo.require("ibm.tivoli.tip.navwidget.topology.dijit.DefaultActionHelper");
dojo.require("ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebookGeneralTab");
dojo.require("ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil");
dojo.require("ibm.tivoli.navmodel.Selection");
dojo.require("ibm.tivoli.navmodel.Trace");

ibm.tivoli.tip.navwidget.topology.dijit.TopologyMenuLayer._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.tip.navwidget.topology.dijit.TopologyMenuLayer");

dojo.declare("ibm.tivoli.tip.navwidget.topology.dijit.TopologyMenuLayer",null,
	{
	
	// summary:
	//		Coming soon
	// description:
	//		Coming soon
	
	// contextMenu: [public] Object
	//		Coming soon
	contextMenu: null,

	//*******************************************************************************
	// "Private"
	//********************************************************************************
	
	// _trace: [private] ibm.tivoli.navmodel.Trace
	//		Utility for trace and logging
	_trace: ibm.tivoli.tip.navwidget.topology.dijit.TopologyMenuLayer._trace,
	
	// _topologyContainer: [private] Object
	//		Coming soon
	_topologyContainer: null,
	
	// _flexBridge: [private] Object
	//		Coming soon
	_flexBridge: null,
	
	// _layerDiv: [private] Object
	//		Coming soon
	_layerDiv: null,
	
	// _cmProvider: [private] Object
	//		Coming soon
	_cmProvider: null,
	
	// _daHelper: [private] Object
	//		Coming soon
	_daHelper: null,
	
	// _cMenu: [private] Object
	//		Coming soon
	_cMenu: null,
	
	// _mouseConnectionHandles: [private] Object[]
	//		Coming soon
	_mouseConnectionHandles: null,
	
	// _rightButtonTargetCache: [private] Object
	//		Coming soon
	_rightButtonTargetCache: null,
	
	// _rolloverTarget: [private] Object
	//		Coming soon
	_rolloverTarget: null,
	
	// _tooltip: [private] Object
	//		Coming soon
	_tooltip: null,
	
	// _tooltipProperties: [private] Object
	//		Coming soon
	_tooltipProperties: null,
	
	// _propOnSetObjectsConnection: [private] Object
	//		Coming soon
	_propOnSetObjectsConnection: null,
	
	// _propOnSetDatasetConnection: [private] Object
	//		Coming soon
	_propOnSetDatasetConnection: null,
	
	// _activatedForMenu: [private] Boolean
	//		Coming soon
	_activatedForMenu: false,
	
	// _forceMenuOnMouseUp: [private] Boolean
	//		Coming soon
	_forceMenuOnMouseUp: false,
	
	// _menuCloseHandle: [private] Object
	//		Coming soon
	_menuCloseHandle: null,
	
	// _openingNewMenu: [private] Boolean
	//		Coming soon
	_openingNewMenu: false,
	
	// _cxtMenuEvt: [private] Object
	//		Coming soon
	_cxtMenuEvt: null,
	
	// _loadingMenuId: [private] String
	//		Coming soon
	_loadingMenuID: "LOADING_ID",
	
	// _layerIDSuffix: [private] String
	//		Coming soon
	_layerIDSuffix: "_menuLayer",
	
	constructor: function (/*Object*/ container, /*Object*/ cmProvider, /*Object*/ daHelper) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// container:
		//		Coming soon
		// cmProvider:
		//		Coming soon
		// daHelper:
		//		Coming soon
		if (this._trace.isFinest()) {
			this._trace.entering(">> In layer constructor", "constructor");
		}
		
		//Initialize data structure for handles
		this._mouseConnectionHandles = [];
		
		if (container) {
			this._topologyContainer = container;
			
	        // add layered div above the topology container
	        var containerBox = dojo.marginBox(container.domNode);
	        if (this._trace.isFinest()) {
				this._trace.finest("   container size w=" + containerBox.w + " h=" + containerBox.h, "constructor");
	        }
	        this._layerDiv = dojo.doc.createElement("div");
	        this._layerDiv.id = container.id + this._layerIDSuffix;
	        this._layerDiv.rolloverTargetRect = null;
	        // initially create the DIV with "visibility: hidden" so it acts as if it is not there
	        dojo.style(this._layerDiv, {position: "absolute",
	        						top: "0px",
	        	                    left: "0px",
	        	                    width: "100%",
	        	                    height: "100%",
	        	                    padding: "0",
	        	                    border: "0",
	        	                    zIndex: "5",
	        	                    visibility: "hidden",
	        	                    //In IE, the div doesn't seem to actually sit over top of Flash unless the background color is set. So, set background
	        	                    //to white and set the opacity to 0 to make transparent.
	        	                    background: "#ffffff",
	        	                    opacity: "0"});
	        container.domNode.appendChild(this._layerDiv);
	        this._flexBridge = container.getFlexBridge();

	        // Use the supplied context menu provider if passed in
	        if (cmProvider) {
	        	this._cmProvider = cmProvider;
	        }
	        else {
		        // create a default context menu provider
	            var helper = new ibm.tivoli.tip.navwidget.topology.dijit.MenuProviderHelper();
		        this._cmProvider = new ibm.tivoli.tip.navwidget.dijit.RESTContextMenuProvider(
		        			{
		        				view: this,
		        				getMenuItemsCallback: dojo.hitch(helper, helper.getMenuItems),
		        				getMenuCommandCallback: dojo.hitch(helper, helper.getMenuCommand)
		        			}
		        		);	        	
	        }

	        // create the context menu using the provider
	        this._cMenu = new ibm.tivoli.tip.navwidget.dijit.AsynchContextMenu({cmProvider: this._cmProvider});
	        
	        //create the defaultActionHandler if defined
	        // Use the supplied default action handler if passed in
	        if (daHelper) {
	        	this._daHelper = daHelper;
	        }
	        	        
	        // create tooltip
	        this._tooltip = new ibm.tivoli.tip.navwidget.dijit.CustomContentTooltip({connectId: [ this._layerDiv.id ],
	                                                                                 getCustomContentCallback: dojo.hitch(this, this._customTooltip)
											//position : ["after", "below"]

										});
	        
	        // Override the tooltip behavior specifically for topology
	        dojo.safeMixin(this._tooltip, {
	        	
	           // Override open to set tooltip position over the menu layer DIV near the node or link
	           open: function(target) {
	        	  // (temporary coordinates until flex bridge can supply mouse position)
	        	  // Create a rectangle for positioning the tooltip, otherwise default
	        	  // position is outside the DOM node of the menu layer DIV
	              var aroundRect = target.rolloverTargetRect;
	              
	              // make sure the rolloverTargetRect has been set to actally display the tooltip
	              // (it may not be if we got a rollover while the context menu is displayed)
	              if (aroundRect != null) {
		              // replace the target node, the menu layer DIV, with this rectangle
		              arguments[0] = aroundRect;
		              this.inherited(arguments);
	              }
	           },
	           _onTargetMouseEnter: function(e) {
	        	   // ignore since menu layer div is not the real target (we handle the tooltip)
	           },
	           _onTargetMouseLeave: function(e) {
	        	   // ignore since menu layer div is not the real target (we handle the tooltip)
	           },
	           _onTargetBlur: function(e) {
	        	   // ignore since menu layer div is not the real target (we handle the tooltip)
	           },
	           _onTargetFocus: function(e) {
	        	   // ignore since menu layer div is not the real target (we handle the tooltip)
	           },
	           // added helper method to trigger the timer to display the tooltip
	           hover: function(target) {
	               this._onHover({target: target});
	           },
	           // added helper method to dismiss the tooltip
	           unHover: function(target) {
	               this._onUnHover({target: target});
	           }	           
	        });
	        this._tooltip.startup();
          
	        // Register window-level mousedown handler to catch potential right-clicks before
	        // they get to the DOM
	        this._rightButtonTargetCache = container.id;
	        if (dojo.isIE) {
	        	// set up mouse handlers on mouse up and mouse down to check for right click events
	        	// and a handler on the document for oncontextmenu so it can be stopped if on the Flash canvas
				this._mouseConnectionHandles.push(dojo.connect(this._topologyContainer.domNode, "onmousedown", this, function(evt) {this._menuLayerHandlerIE(evt);}));
				this._mouseConnectionHandles.push(dojo.connect(this._topologyContainer.domNode, "onmouseup", this, function(evt) {this._topologyContainer.domNode.releaseCapture();}));
				this._mouseConnectionHandles.push(dojo.connect(dojo.doc, "oncontextmenu", 
															   this, function(evt) {
																			var swfSuffixPos = evt.srcElement.id.indexOf("_embeddedSWFObjectId");
																			if ( swfSuffixPos > 0) {
																				// since this event is caught at the document level, find
																				// the actual canvas related to the Flash canvas
																				var actualCanvas = dijit.byId(evt.srcElement.id.substring(0, swfSuffixPos));
																				actualCanvas._menuLayer._activateLayerDiv(true);
																				actualCanvas._menuLayer._processContextMenu(evt);
																				return false;
																			} else {
																				this._rightButtonTargetCache = "nan";
																			}					
																}));
	        }
	        else {  // Firefox and other browsers
	        	// add mousedown listener to window to filter all right mousedown events
		        dojo.global.addEventListener("mousedown", dojo.hitch(this, this._menuLayerHandler), true);	        	
	        }
	        
	        //Register for object doubleClick/rollOver/rollOut events
	        dojo.connect(this._topologyContainer, "onObjectDoubleClick", this, this._objectDoubleClickHandler);
	        dojo.connect(this._topologyContainer, "onObjectRollOver", this, this._objectRollOverHandler);
	        dojo.connect(this._topologyContainer, "onObjectRollOut", this, this._objectRollOutHandler);
	        dojo.connect(this._topologyContainer, "_onPanStart", this, this._canvasPanStartHandler);
	        dojo.connect(this._topologyContainer, "_onPanEnd", this, this._canvasPanEndHandler);
		        
	        /* These are the different MouseEvent types in ActionScript (the ones currently sending on from div are marked)
		        
			**CLICK : String = "click"
				[static] Defines the value of the type property of a click event object. MouseEvent 
     		**CONTEXT_MENU : String = "contextMenu"
				[static] The MouseEvent.CONTEXT_MENU constant defines the value of the type property of a contextMenu event object. MouseEvent 
    		**DOUBLE_CLICK : String = "doubleClick"
				[static] Defines the value of the type property of a doubleClick event object. MouseEvent 
     		MIDDLE_CLICK : String = "middleClick"
				[static] Defines the value of the type property of a middleClick event object. MouseEvent 
     		MIDDLE_MOUSE_DOWN : String = "middleMouseDown"
				[static] Defines the value of the type property of a middleMouseDown event object. MouseEvent 
     		MIDDLE_MOUSE_UP : String = "middleMouseUp"
				[static] Defines the value of the type property of a middleMouseUp event object. MouseEvent 
	    	**MOUSE_DOWN : String = "mouseDown"
				[static] Defines the value of the type property of a mouseDown event object. MouseEvent 
	    	**MOUSE_MOVE : String = "mouseMove"
				[static] Defines the value of the type property of a mouseMove event object. MouseEvent 
	    	**MOUSE_OUT : String = "mouseOut"
				[static] Defines the value of the type property of a mouseOut event object. MouseEvent 
	    	**MOUSE_OVER : String = "mouseOver"
				[static] Defines the value of the type property of a mouseOver event object. MouseEvent 
	    	**MOUSE_UP : String = "mouseUp"
				[static] Defines the value of the type property of a mouseUp event object. MouseEvent 
	    	**MOUSE_WHEEL : String = "mouseWheel"
				[static] Defines the value of the type property of a mouseWheel event object. MouseEvent 
	     	RIGHT_CLICK : String = "rightClick"
				[static] Defines the value of the type property of a rightClick event object. MouseEvent 
	     	RIGHT_MOUSE_DOWN : String = "rightMouseDown"
				[static] Defines the value of the type property of a rightMouseDown event object. MouseEvent 
	     	RIGHT_MOUSE_UP : String = "rightMouseUp"
				[static] Defines the value of the type property of a rightMouseUp event object. MouseEvent 
	    	ROLL_OUT : String = "rollOut"
				[static] Defines the value of the type property of a rollOut event object. MouseEvent 
	    	ROLL_OVER : String = "rollOver"
				[static] Defines the value of the type property of a rollOver event object. MouseEvent 
	        */
	        
	        // Register to listen for the different mouse events of importance
			this._mouseConnectionHandles.push(dojo.connect(this._layerDiv, "onclick", this, function(evt) {this._processClick(evt);}));
			this._mouseConnectionHandles.push(dojo.connect(this._layerDiv, "oncontextmenu", this, function(evt) {this._processContextMenu(evt);}));
			this._mouseConnectionHandles.push(dojo.connect(this._layerDiv, "ondblclick", this, function(evt) {this._processDoubleClick(evt);}));
			this._mouseConnectionHandles.push(dojo.connect(this._layerDiv, "onmousedown", this, function(evt) {this._processMouseDown(evt);}));
			this._mouseConnectionHandles.push(dojo.connect(this._layerDiv, "onmousemove", this, function(evt) {this._processMouseMove(evt);}));
			this._mouseConnectionHandles.push(dojo.connect(this._layerDiv, "onmouseout", this, function(evt) {this._processMouseOut(evt);}));
			this._mouseConnectionHandles.push(dojo.connect(this._layerDiv, "onmouseover", this, function(evt) {this._processMouseOver(evt);}));
			this._mouseConnectionHandles.push(dojo.connect(this._layerDiv, "onmouseup", this, function(evt) {this._processMouseUp(evt);}));
			this._mouseConnectionHandles.push(dojo.connect(this._layerDiv, "onmousewheel", this, function(evt) {this._processMouseWheel(evt);}));
			
			if (this._trace.isFinest()) {
				this._trace.exiting(">> In layer constructor", "constructor");
			}
		}
	},
	
	destroy: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		dojo.forEach(this._mouseConnectionHandles, function(h) {dojo.disconnect(h);});
		this._mouseConnectionHandles = [];
		
		if (this._tooltip) {
			this._tooltip.removeTarget(this._layerDiv.id);
			this._tooltip.destroyRecursive();
		}
		
		// Remove window level listener (IE specific ones were removed above from 
		// _mouseConnectionHandles
		if (!(dojo.isIE)) {
			/*
			try {
				dojo.global.addEventListener("mousedown", null, true); //MAA TODO: This seems to die so added try/catch for now.
			} catch (ex) {
				if (this._trace.isError()) {
					var methodName = "destroy";
					this._trace.error("problem destroying " + this._layerDiv.id + "; " + ex, methodName);
				}
			}
			*/
			dojo.global.removeEventListener("mousedown", dojo.hitch(this, this._menuLayerHandler), true);			
		}
	},
	
	getRolloverTarget: function() {
		// summary:
		//		Returns the graph entity under the mouse cursor in the topology container
		// description:
		//		This accessor returns the item (node), link, or relationship under the mouse
		//      cursor in the topology graph.  This can be used when overriding what is
		//      displayed in the tooltip for the entity.
		return this._rolloverTarget;
	},

	getTooltip: function() {
		// summary:
		//		Returns the tooltip instance used for the topology container.
		// description:
		//		This accessor returns the ibm.tivoli.tip.navwidget.dijit.CustomContentTooltip instance
		//      used to display tooltips for all entities in the topology graph.
		return this._tooltip;
	},
	
	_menuLayerHandler: function(/*Object*/ evt) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// evt:
		//		Coming soon
		
		// Is this a mouse button 2 down event?
		if (dojo.mouseButtons.isRight(evt)) {
			if(this._cmProvider.viewWithToolbar){
				this._cmProvider.viewWithToolbar.setPostponeToolbarTasksRefresh(true);
			}
			var nodeId = evt.target.id;
			var swfSuffixPos = nodeId.indexOf("_embeddedSWFObjectId");
			var layerSuffixPos = nodeId.indexOf(this._layerIDSuffix);
			var actualCanvas = null;

			if (swfSuffixPos > 0) { // Is it over the Flex canvas?
				// console.log("### right click is over Flex canvas");
				// console.dir(evt);
				actualCanvas = dijit.byId(nodeId.substring(0, swfSuffixPos));
	            actualCanvas._menuLayer._forceMenuOnMouseUp = true;  // flag it as caught by window event handler
				actualCanvas._menuLayer._activateLayerDiv(true);  // flag it as being activated for the menu
				actualCanvas._menuLayer._processMouseDown(evt);
				dojo.stopEvent(evt);
	        } else if (layerSuffixPos > 0) { // Is it over the "visible" menu layer DIV?
	        	// let it go through
	        	// console.log("### right click is over menu layer");
	        	actualCanvas = dijit.byId(nodeId.substring(0, layerSuffixPos));
	        	if (actualCanvas._menuLayer._cMenu.getMenu()) {
	                // If we currently have a menu displayed, it will be dismissed, but
	                // set flag so we do not deactivate the DIV layer
	                actualCanvas._menuLayer._openingNewMenu = true;
	            }
	         	// console.dir(evt);	         	
	        } else { // It is over some other element, ignore it
	        	// let it go through
	         	// console.log("### right click is NOT over topology");
	         	// console.dir(evt);
	        }
	    } else {
	    	if(this._cmProvider.viewWithToolbar){
	    		this._cmProvider.viewWithToolbar.setPostponeToolbarTasksRefresh(false);
	    	}
	    }
	 },
	 
	 _menuLayerHandlerIE: function(/*Object*/ evt) {
		 // summary:
		 //		Coming soon
		 // description:
		 //		Coming soon
		 // evt:
		 //		Coming soon
		 if (dojo.mouseButtons.isRight(evt)) {
			 if(this._cmProvider.viewWithToolbar){
				 this._cmProvider.viewWithToolbar.setPostponeToolbarTasksRefresh(true);
			 }
			 var flashCanvasId = this._topologyContainer.id + "_embeddedSWFObjectId";
			 if (evt.srcElement.id && evt.srcElement.id == flashCanvasId && this._rightButtonTargetCache == flashCanvasId) {
				 this._processMouseDown(evt); 
			 }
			 this._topologyContainer.domNode.setCapture();
			 if (evt.srcElement.id) {
				 this._rightButtonTargetCache = evt.srcElement.id;
			 }
		 } else {
			 if(this._cmProvider.viewWithToolbar){
				 this._cmProvider.viewWithToolbar.setPostponeToolbarTasksRefresh(false);
			 }
		 }
	 },

	
    _closeMenu: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		 
		// close a context menu if it is open
		var menu = this._cMenu.getMenu();
		if (menu) {
			// Clear the focus, otherwise IE has problems on _onBlur
			menu.focusedChild = null;
			dijit.popup.close(menu);			
	    }
	},
	
	_isMenuOpen: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		
		return (this._cMenu.getMenu()) ? true : false;
	},
	
	_activateLayerDiv: function(/*Object*/ isRightClick) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// isRightClick:
		//		Coming soon
		this._activatedForMenu = isRightClick;
		if (dojo.style(this._layerDiv, "visibility") == "hidden") {
	        //console.log("   >>> activating layer");
			dojo.style(this._layerDiv, "visibility", "visible");
		}
	},
	
	_deactivateLayerDiv: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		if (dojo.style(this._layerDiv, "visibility") == "visible") {
			this._activatedForMenu = false;
	        this._forceMenuOnMouseUp = false;
			//console.log(">>> deactivating layer");
			dojo.style(this._layerDiv, "visibility", "hidden");
		}		
	},
		
	_processClick: function(/*Object*/ evt) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// evt:
		//		Coming soon
		
		// just for debug to see (x,y) before passing to flex bridge
		if (this._trace.isFinest()) {
			this._trace.finest("*** layer detected CLICK: x=" + evt.layerX + " y=" + evt.layerY);
		}
		
		// NOTE: *no* need pass event on to canvas flex bridge here

		//Close the menu 
		//  (not needed here as we handle now on mousedown)
		// this._closeMenu();
		// this._closeTooltip();
		// this._deactivateLayerDiv();
		
		//Stop propagation of the event
		dojo.stopEvent(evt);
	},
	
	_processContextMenu: function(/*Object*/ evt) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// evt:
		//		Coming soon
		
		// just for debug to see (x,y) before passing to flex bridge
		if (this._trace.isFinest()) {
			this._trace.finest("*** layer detected CONTEXT_MENU: x=" + evt.layerX + " y=" + evt.layerY);
		}
		
	    this._activatedForMenu = true;
	    this._closeTooltip();
	    if (this._forceMenuOnMouseUp) { // if initiated by window event handler, clear it here
	    	this._forceMenuOnMouseUp = false;
	    }
		
	    // Copy (only on IE versions less than IE9 or if using quirks mode) and cache the event
	    this._cxtMenuEvt = (dojo.isIE < 9 || (dojo.isIE && dojo.isQuirks)) ? dojo.mixin({}, evt): evt;   // 254234
	    
		var flexBridgeCallback = function(objectContext) {
			//Get the selected items for menu context
			if (this._trace.isFinest()) {
				this._trace.finest("****** after right-click the selection for canvas = " + this._topologyContainer.id + "; itemIds = " +
						dojo.toJson(objectContext.getItemIds()) + 
							", linkIds = " + dojo.toJson(objectContext.getLinkIds()) +
							", relationshipIds = " + dojo.toJson(objectContext.getRelationshipIds()));
			}
			
		    // If a context menu is already open, close it because the selected context is likely different
			if (this._cMenu.getMenu()) {
				this._openingNewMenu = true;
				this._closeMenu();
		    }
	
			//Open up the Dojo dijit.Menu
			this._cMenu.contextMenuOpen(this._cxtMenuEvt, this._topologyContainer);
			
			// Listen for the onClose event to know when to hide the menu layer DIV
			this._menuCloseHandle = dojo.connect(this._cMenu.getMenu(), "onClose", this,
													function() {
														dojo.disconnect(this._menuCloseHandle);
														if (this._openingNewMenu) {
															this._openingNewMenu = false;
														} else {
															this._deactivateLayerDiv();
														}
													});
			this._cxtMenuEvt = null;
		};
		
		// Pass (x,y) along to canvas flex bridge here
		var kwArgs = {
				onComplete: dojo.hitch(this, flexBridgeCallback),
				evt: evt
		};
		this._topologyContainer._divLayerOnContextMenu(kwArgs);
		
		//Stop propagation of the event
		dojo.stopEvent(evt);
	},
	
	_processDoubleClick: function(/*Object*/ evt) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// evt:
		//		Coming soon
		
		// just for debug to see (x,y) before passing to flex bridge
		if (this._trace.isFinest()) {
			this._trace.finest("*** layer detected DOUBLE_CLICK: x=" + evt.layerX + " y=" + evt.layerY);
		}
		
		// pass event on to canvas flex bridge here
		this._topologyContainer._divLayerOnDoubleClick(evt);

		//Close the menu
		this._closeMenu();
		this._closeTooltip();
		this._deactivateLayerDiv();
		
		//Stop propagation of the event
		dojo.stopEvent(evt);
	},
	
	_processMouseDown: function(/*Object*/ evt) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// evt:
		//		Coming soon
		
		// just for debug to see (x,y) before passing to flex bridge
		if (this._trace.isFinest()) {
			this._trace.finest("*** layer detected MOUSE_DOWN: x=" + evt.layerX + " y=" + evt.layerY);
		}
		
		// pass event on to canvas flex bridge here
		this._topologyContainer._divLayerOnMouseDown(evt);
		
		if (dojo.mouseButtons.isLeft(evt)) {
	         this._closeMenu();
	         this._closeTooltip();
	         this._deactivateLayerDiv();
		}
		
		//Get the selected items
		if (this._trace.isFinest()) {
			var selection = this._topologyContainer.selection;
			this._trace.finest("******after mouse down the selection for canvas = " + this._topologyContainer.id + "; itemIds = " +
					dojo.toJson(selection.getItemIds()) + 
							", linkIds = " + dojo.toJson(selection.getLinkIds()) + 
							", relationshipIds = " + dojo.toJson(selection.getRelationshipIds()));
		}
				
		//Stop propagation of the event
		dojo.stopEvent(evt);
	},
	
	_processMouseMove: function(/*Object*/ evt) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// evt:
		//		Coming soon
		
		// just for debug to see (x,y) before passing to flex bridge
		//Commenting out trace to see if performance is improved...
		//if (this._trace.isFinest()) {
		//	this._trace.finest("*** layer detected MOUSE_MOVE: x=" + evt.layerX + " y=" + evt.layerY);
		//}
		
		// only pass the event on if the menu layer DIV is visible/active
	    if (dojo.style(this._layerDiv, "visibility") == "visible") {
			// pass event on to canvas flex bridge here
			this._topologyContainer._divLayerOnMouseMove(evt);
	    }
		
		//Stop propagation of the event
		dojo.stopEvent(evt);
	},
	
	_processMouseOut: function(/*Object*/ evt) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// evt:
		//		Coming soon
		if (this._trace.isFinest()) {
			this._trace.finest("*** layer detected MOUSE_OUT");  // just for debug for now
		}

		// only pass the event on if the menu layer DIV is visible/active
	    if (dojo.style(this._layerDiv, "visibility") == "visible") {
	    	// Can get mouseout events when mouse is over the menu, so
	    	// ignore these if the menu is currently displayed
	    	var menu = this._cMenu.getMenu();
	    	if (!menu) {
	    		//Passing along allows Flash side to determine if we have a rollOut event for the current
	    		//rollOver object
			    this._topologyContainer._divLayerOnMouseOut(evt);
			}
	    }

		//Stop propagation of the event
		dojo.stopEvent(evt);
	},
	
	_processMouseOver: function(/*Object*/ evt) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// evt:
		//		Coming soon
		
		if (this._trace.isFinest()) {
			this._trace.finest("*** layer detected MOUSE_OVER");  // just for debug for now
		}

		// Can get mouseover events when mouse is over the menu and leaves it, so
		// ignore these if the menu is currently displayed
		var menu = this._cMenu.getMenu();
		if (!menu) {
			// NOTE: *no* need pass event on to canvas flex bridge here
		}
		
		//Stop propagation of the event
		dojo.stopEvent(evt);
	},
	
	_processMouseUp: function(/*Object*/ evt) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// evt:
		//		Coming soon
		
		// just for debug to see (x,y) before passing to flex bridge
		if (this._trace.isFinest()) {
			this._trace.finest("*** layer detected MOUSE_UP: x=" + evt.layerX + " y=" + evt.layerY);
		}
		
		// check for first time since we will not get oncontextmenu since we did not get the
		// mousedown
		if (this._forceMenuOnMouseUp && dojo.mouseButtons.isRight(evt)) {
			// coerce this into oncontextmenu processing for the first time only
			this._processContextMenu(evt);
		} else {
			// NOTE: *no* need pass event on to canvas flex bridge here
		}
		
		//Stop propagation of the event
		dojo.stopEvent(evt);
	},
	
	_processMouseWheel: function(/*Object*/ evt) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// evt:
		//		Coming soon
		
		// just for debug to see (x,y) before passing to flex bridge
		if (this._trace.isFinest()) {
			this._trace.finest("*** layer detected MOUSE_WHEEL: x=" + evt.layerX + " y=" + evt.layerY);
		}
		
		// pass event on to canvas flex bridge here
		this._topologyContainer._divLayerOnMouseWheel(evt);
		
		//Stop propagation of the event
		dojo.stopEvent(evt);
	},
	
	//***********************************************************************************
	//Called by Canvas as Flash determines object that's roller over or rolled out of
	//***********************************************************************************
	
	_objectRollOverHandler: function(/*Object*/ objectContext, /*Object*/ boundingInfo) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// objectContext:
		//		Coming soon
		// boundingInfo:
		//		Coming soon
		if (this._trace.isFinest()) {
			this._trace.finest("****** ROLL_OVER!!! itemIds = " + dojo.toJson(objectContext.getItemIds()) + 
					", linkIds = " + dojo.toJson(objectContext.getLinkIds()) + 
					", relationshipIds = " + dojo.toJson(objectContext.getRelationshipIds()) +
					"; boundingInfo = " + dojo.toJson(boundingInfo));
		}
		
		// ignore if the menu is open or if we already havea rollover target
	    if (!this._isMenuOpen() && !this._rolloverTarget) {
	    	var aroundRect = {type: "rect"};
	    	if (objectContext.getItemCount() == 1) {
	    		this._rolloverTarget = objectContext.getItems()[0];
	    		aroundRect.x = boundingInfo.objectX;
	    		aroundRect.y = boundingInfo.objectY;
	    		aroundRect.width = boundingInfo.objectWidth;
	    		aroundRect.height = boundingInfo.objectHeight;
	    	} else if (objectContext.getLinkCount() == 1) {
	    		this._rolloverTarget = objectContext.getLinks()[0];
	    		aroundRect.x = boundingInfo.mouseX;
	    		aroundRect.y = boundingInfo.mouseY;
	    		aroundRect.width = aroundRect.height = 2;
	    	} else if (objectContext.getRelationshipCount() == 1) {
	    		this._rolloverTarget = objectContext.getRelationships()[0];
	    		aroundRect.x = boundingInfo.mouseX;
	    		aroundRect.y = boundingInfo.mouseY;
	    		aroundRect.width = aroundRect.height = 2;
	    	} else {
	    		this._rolloverTarget = "Unknown node or link";
	    		aroundRect.x = boundingInfo.mouseX;
	    		aroundRect.y = boundingInfo.mouseY;
	    		aroundRect.width = aroundRect.height = 1;
	    	}
	    	// adjust relative coordinates to absolute, and remove fractions
	    	var lt = dojo.coords(this._layerDiv, true);
	    	aroundRect.x += lt.x;
	    	aroundRect.y += lt.y;
	    	aroundRect.x = Math.round(aroundRect.x);
	    	aroundRect.y = Math.round(aroundRect.y);
	    	aroundRect.width = Math.ceil(aroundRect.width);
	    	aroundRect.height = Math.ceil(aroundRect.height);
			
	    	this._layerDiv.rolloverTargetRect = aroundRect;
	    	this._activateLayerDiv(false);
	    	this._tooltip.hover(this._layerDiv);
	    }
	},
	
	_objectRollOutHandler: function(/*Object*/ objectContext) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// objectContext:
		//		Coming soon
		if (this._trace.isFinest()) {
			this._trace.finest("****** ROLL_OUT!!! itemIds = " + dojo.toJson(objectContext.getItemIds()) + 
					", linkIds = " + dojo.toJson(objectContext.getLinkIds()) + 
					", relationshipIds = " + dojo.toJson(objectContext.getRelationshipIds()));
		}
		
		this._closeTooltip();
	},
	
	_canvasPanStartHandler: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		
		//TODO: Mark needs to do work here
		if (this._trace.isFinest()) {
			this._trace.finest("PAN START!!!!");
		}
	},
	
	_canvasPanEndHandler: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		
		//TODO: Mark needs to do work here
		if (this._trace.isFinest()) {
			this._trace.finest("PAN END!!!!");
		}
	},
	
	_closeTooltip: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		if (this._rolloverTarget) {
			dojo.disconnect(this._propOnSetDatasetConnection);
			dojo.disconnect(this._propOnSetObjectsConnection);
			
			// Clear out the reference in the tooltip to the content
			this._tooltip.updateContent(""); 
			if (this._tooltipProperties){
				this._tooltipProperties.destroyRecursive();
				this._tooltipProperties = null;
			}
			
			// this._tooltip.close();
			this._tooltip.unHover(this._layerDiv);
			this._rolloverTarget = null;
			this._layerDiv.rolloverTargetRect = null;
			if (!this._activatedForMenu) {
				this._deactivateLayerDiv();
			}
	    }
	},
	
	_objectDoubleClickHandler: function(/*Object*/ objectContext) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// objectContext:
		//		Coming soon
		if (this._trace.isFinest()) {
			this._trace.finest("****** DOUBLE-CLICK!!! itemIds = " + dojo.toJson(objectContext.getItemIds()) + 
					", linkIds = " + dojo.toJson(objectContext.getLinkIds()) + 
					", relationshipIds = " + dojo.toJson(objectContext.getRelationshipIds()));
		}


		if(!this._daHelper){
			//Create DefaultActionHelper
			//By default creates the properties Dialog and launches it (via callback)
			this._daHelper = new ibm.tivoli.tip.navwidget.topology.dijit.DefaultActionHelper( 
					{
						defaultActionCallback: dojo.hitch(this, "_launchPropertiesDialog",objectContext)
					}
			);
		}
		//actually execute the default action/task (defined in constuctor)
		this._daHelper.executeDefaultAction(objectContext.getPluggableView());
	},
	
	_launchPropertiesDialog: function(/*Object*/ objectContext){
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// objectContext:
		//		Coming soon
		
		//NOTE: Reusing Anthony's MenuProviderHelper to launch the dialog to avoid duplicate logic. This is a temporary solution. 
		var menuProviderHelper = new ibm.tivoli.tip.navwidget.topology.dijit.MenuProviderHelper();
		menuProviderHelper.getMenuCommand("nwmi_PropertiesNotebookDialog", objectContext.getPluggableView(), null);
	},
	
	_customTooltip: function(/*Object*/ target) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// target:
		//		Coming soon

		if (this._rolloverTarget) {
			// target is this._layerDiv, so we use this._rolloverTarget
			var content = "";
			if (typeof this._rolloverTarget == "string") {
				content = content + this._rolloverTarget;
			} else {
				content = content + this._rolloverTarget.getLabel();
			}

			//if the server side tooltip content is set, use this instead of defaults.
			if(this._rolloverTarget.getTooltip) { //Need first test because getTooltip is not a function on child datasets right now
				var tooltip = this._rolloverTarget.getTooltip();
				if (tooltip){ 
					content = tooltip;
					return content;
				}
			}

			if(this._tooltipProperties){
				return this._tooltipProperties.getContent();
			}

			//create properties content object
			this._tooltipProperties = new ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebookGeneralTab({
				id: this._topologyContainer.id + "_tooltipProperties",
				managingPluggableView: this._topologyContainer, 
				maxProperties: 3,  //TODO: ADM should probably be something a widgetDef sets.
				_isTooltipProperties: true
			});

			//Call back for when the dataset it set
			var objectType = (this._rolloverTarget.getObjectType) ? this._rolloverTarget.getObjectType() : null;
			var setDatasetComplete = function(dataset) {
				//connect to the onSetObjects Event
				this._propOnSetObjectsConnection = dojo.connect(this._tooltipProperties, "onSetObjects", dojo.hitch(this, this._updateTooltipContent));

				if (objectType === "item"){
					this._tooltipProperties.setObjects([this._rolloverTarget], null, null, null);
				} else if(objectType === "link"){
					this._tooltipProperties.setObjects(null, [this._rolloverTarget], null, null);
				} else if(objectType === "relationship"){
					this._tooltipProperties.setObjects(null, null, [this._rolloverTarget], null);
				} 
			};
			this._propOnSetDatasetConnection = dojo.connect(this._tooltipProperties, "onSetDataset", dojo.hitch(this, setDatasetComplete));
			this._tooltipProperties.setDataset(this._topologyContainer.dataset);

			//return content;
			return this._createTemporaryContentDiv(content, objectType);
		}
	   },
	   
	   //Event callback method to handle updating the tooltip content with updated content
	   _updateTooltipContent: function(){
		   this._tooltip.updateContent(this._tooltipProperties.getContent(), this._layerDiv);
	   },
	   
	   _createTemporaryContentDiv: function(content, objectType) {		   
		   //Background div
		   var backgroundDiv = dojo.create("div",{
		   });
		   var processingPane = new dijit.layout.ContentPane({
			   content: backgroundDiv
		   });
			
			// text div		
		   var textDiv = dojo.create("div",{
			   "innerHTML": content
		   });
		   backgroundDiv.appendChild(textDiv);
		
		   if (objectType) {
			   // processing image div		
			   var processDiv = dojo.create("div",{
					"class": "processingImage"
			   });
			   backgroundDiv.appendChild(processDiv);
		   }

		   //Return
		   return processingPane;
	   }
});

});

},
'dojoe/treetable/_Toolbar':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/i18n!dojoe/treetable/nls/TreeTable","dojo/require!dojoe/common/toolbar/_Toolbar,dijit/form/Button,dijit/ToolbarSeparator,dijit/form/DropDownButton,dijit/Menu,dojo/i18n,dojoe/treetable/_ConfigureColumnsDialog,dojoe/treetable/_QuickSearchFilter"], function(dijit,dojo,dojox){
// dojo.provide allows pages to use all of the types declared in this resource.
dojo.provide("dojoe.treetable._Toolbar");

//dojo.require("dijit.Toolbar");
dojo.require("dojoe.common.toolbar._Toolbar");
dojo.require("dijit.form.Button");
dojo.require("dijit.ToolbarSeparator");
dojo.require("dijit.form.DropDownButton");
dojo.require("dijit.Menu");
dojo.require("dojo.i18n");
dojo.require("dojoe.treetable._ConfigureColumnsDialog");
dojo.require("dojoe.treetable._QuickSearchFilter");

dojo.requireLocalization("dojoe.treetable", "TreeTable");

dojo.declare("dojoe.treetable._Toolbar", [ dojoe.common.toolbar._Toolbar], {	
	
		// summary: 
		//		A private class to create tree table toolbar with icons, action lists and fitler options
		// description: 
		//		This is based on a common toolbar widget which supports both table and tree table widgets.
		//		This class is internally used to enable/disable toolbar icons, actions and search controls in the tree table
		//		Please refer dojoe.treetable.TreeTable.toolbarOptions property.
		// tags: private
		
		// widgetsInTemplate: boolean
		//		to indicate widgets in template
		widgetsInTemplate:true,
		//toolbar:null,
		
		// treeTable: object
		//		Reference to tree table instance
		treeTable:null,
		
		// actionListWidget: object
		//		Reference to Action List menu in toolbar
		actionListWidget:null,
		
		// quickFilter: object
		//		Reference to quick filter widget in the toolbar
		quickFilter:null,
		
		// filterOptions: object
		//		filter options to set to quick filter
		filterOptions: null,
		
		// refreshIcon: boolean
		//		To enable/disable refresh icon on toolbar, defaults to true
		refreshIcon:true,
		
		// refreshMenu: boolean
		//		To enable/disable refresh menu item in the action list 
		refreshMenu:true,
		
		// showSelectAllIcon: boolean
		//		to enable/disable select All icon in the toolbar
		showSelectAllIcon:true,
		
		// onlySelectAllIcon: boolean
		//		To provide a option to use only select All without de-select All option
		onlySelectAllIcon:false,
		
		// onlyDeselectAllIcon: boolean
		//		To provide a option to use only de-select All without select All option	
		onlyDeselectAllIcon:false,
		
		// expandAllIcon: boolean
		//		to indicate the state of items expand/collapse when the tree table loads
		expandAllIcon: true,
		
		// clearSortIcon: boolean
		//		To enable/disable clear sort icon on the toolbar, defaults to true
		clearSortIcon: true,
		
		// configureTreeTableIcon: boolean
		//		To enable/disable Configure options in the toolbar, defaults to true
		configureTreeTableIcon:true,
		
		// actionMenu: boolean
		//		To enable/disable Action list in the toolbar, defaults to true
		actionMenu:true,
		
		// selectAllMenu: boolean
		//		To enable/disable select All menu item in the Action list, defaults to true
		selectAllMenu: true,
		
		// expandAllMenu: boolean
		//		To enable/disable expandAll menu item in Action list, defaults to true
		expandAllMenu : true,
		
		// clearSortMenu: boolean
		//		To enable/disable clear sort menu item in Action list, defaults to true
		clearSortMenu: true,
		
		// configureTreeTableMenu: boolean
		//		To enable/disable configure options menu item in Action list, defaults to true
		configureTreeTableMenu:true,
		
		// toolbarOptions: object
		//		local variable which holds the tree table toolbarOptions values 
		toolbarOptions : null,
		
		// quickFilterVisible: boolean
		//		To enable/disable quick search filter on the toolbar, defaults to true
		quickFilterVisible : true,
		
		// quickFilterVisible: boolean
		//		To enable/disable quick filter button on the toolbar, defaults to false
		showQuickFilterButton: false,		
	
		_mapToolbarControls : function() {
			// summary: 
			//		Internal function to map toolbar options from dojoe.treetable.TreeTable to properties of this class
	
			if(this.toolbarOptions) {
				if(this.toolbarOptions.refreshIcon == false){
					this.refreshIcon = this.toolbarOptions.refreshIcon;
				}
				//23789: Refresh should be part of Actions Menu also - OID
				//Bobby:29Jun11
				if(this.toolbarOptions.refreshMenu == false){
					this.refreshMenu = this.toolbarOptions.refreshMenu;
				}				
				if(this.toolbarOptions.selectAllIcon == false){
					this.showSelectAllIcon = this.toolbarOptions.selectAllIcon;
					
				}else{
					if(this.toolbarOptions.onlySelectAllIcon == true){
						this.onlySelectAllIcon=this.toolbarOptions.onlySelectAllIcon;
					}
					if(this.toolbarOptions.onlyDeselectAllIcon == true){
						this.onlyDeselectAllIcon=this.toolbarOptions.onlyDeselectAllIcon;
					}
				}
				if(this.toolbarOptions.selectAllMenu == false){
					this.selectAllMenu = this.toolbarOptions.selectAllMenu;
				}				
				//Only collapseAll is supported in TED 1.1 and TED 1.0.
				if(this.toolbarOptions.expandAllIcon == false){
					this.expandAllIcon = this.toolbarOptions.expandAllIcon;
				}
				if(this.toolbarOptions.expandAllMenu == false){
					this.expandAllMenu = this.toolbarOptions.expandAllMenu;
				}
				
				if(this.toolbarOptions.clearSortIcon == false){
					this.clearSortIcon = this.toolbarOptions.clearSortIcon;
				}
				if(this.toolbarOptions.clearSortMenu == false){
					this.clearSortMenu = this.toolbarOptions.clearSortMenu;
				}
				if(this.toolbarOptions.configureTreeTableIcon == false) 
				{
					this.configureTreeTableIcon = this.toolbarOptions.configureTreeTableIcon;
				}
				if(this.toolbarOptions.configureTreeTableMenu == false){
					this.configureTreeTableMenu = this.toolbarOptions.configureTreeTableMenu;
				}
				if(this.toolbarOptions.actionMenu == false){
					this.actionMenu = this.toolbarOptions.actionMenu;
				}				
				//Keep for backward compatibility?
				//console.log('_mapToolbarControls: this.toolbarOptions.quickFilterVisible:'+this.toolbarOptions.quickFilterVisible);
				if(this.toolbarOptions.quickFilterVisible == false){
					this.quickFilterVisible = this.toolbarOptions.quickFilterVisible;
				}	
				//LS- 26/10/2011 - # 30377 - Treetable - incorrect results when setting filter or sorting while data is being loaded
				//this was missed in mapping
				if(this.toolbarOptions.quickFilterVisible && this.toolbarOptions.showQuickFilterButton){
					this.showQuickFilterButton = this.toolbarOptions.showQuickFilterButton;
				}
				//console.log('_mapToolbarControls: this.quickFilterVisible:'+this.quickFilterVisible);				
			}			
		},
		
		// Place comma-separated class attributes here. Note, instance attributes 
		// should be initialized in the constructor. Variables initialized here
		// will be treated as 'static' class variables.

		// Constructor function. Called when instance of this class is created
		constructor : function() {
			// summary:
			//		constructor
			this.resources_ = dojo.i18n.getLocalization("dojoe.treetable", "TreeTable", this.lang);	
		},
		
		_refreshTreeTable: function(){
			// summary:
			//		Internal function to invoke tree table refresh
			
			/*LA - 09/01/2012 - TWL - 16212- TreeTable CURI refresh not clearing cache*/
			//setting forceRefresh true to clear the LTG cache so that data from stores is reloaded...
			this.treeTable.refresh(true);
		},
		
		_selectAll: function () 
		{
			// summary:
			//		Internal function to select all the rows in the tree table, provided Indirect Selection is enabled
			
			if(this.treeTable.indirectSelection === true){
				//moving this to indirect selection class
				this.treeTable._loadIndSelection.selectAllRows();
			}else{
				var count = this.treeTable.grid.rowCount;
				for (var i=0; i<count; i++) {
					this.treeTable.grid.selection.addToSelection(i);
				}
			}
		},
		
		_deselectAll: function () 
		{
			// summary:
			//		Internal function to deselect all the rows in the tree table, provided Indirect selection is enabled
			
			if(this.treeTable.indirectSelection === true){
				this.treeTable._loadIndSelection.deSelectAllRows();
			}else{
				this.treeTable.grid.selection.deselectAll();
			}			
		},
		
		/* Not supported in the base widget, dojo LazyTreeGrid.
		_expandAll : function(){
			this.treeTable.grid.foldingAll(true);
		},
		*/
		_collapseAll : function()
		{
			// summary:
			//		Internal function to collapse all the parent items in the tree
			
			//this.treeTable.grid.foldingAll(false);
			//Satanik:07Jun2011: changing the collapse all
			this.treeTable.grid._cleanup();
			/*Lakshmi - 15/07/2011 - 21173 - In Treetable, Collapse all is not working */
			//normal grid refresh solves this issue and even is suffice
			this.treeTable.grid._refresh(true);
		},
		
		_clearSort : function()
		{
			// summary:
			//		Internal function to clear sorting applied to all the columns
			
			//LS - 21/10/2011 - #30979 - Clear sort does not work for treetable
			//since LTG refresh has changed adding cleanup before clear sort to make sure the applied sort is cleared.
			this.treeTable.grid._cleanup();
			this.treeTable.grid.setSortInfo(0);
			this.treeTable.grid._refresh(true);
		},
		
		_configureColumn : function()
		{ 
			// summary:
			//		Internal function to show configure options dialog, which provides ability to show/hide columns in the tree table
			
		    //Satanik:25May2011: changes made to address ILMT requiremnt 19890
			this.tableOptionsDialog = new dojoe.treetable._ConfigureColumnsDialog ({ grid:this.treeTable.grid,indirectSelection:this.treeTable.indirectSelection,enableColumnReorder:this.treeTable.enableColumnReorder});
			this.tableOptionsDialog.show();
		},
		
		postCreate:function(){
			// summary:
			//		To handle stuff when the toolbar is rendered
			// tags: extension
			
			//this.inherited(arguments);
			//22694: Hiding the quickfilter in TreeTable toolbar doesn't work - Bjørn Stadheim
			//Bobby: 22Jun2011, correcting the validation below, undefined doesn't need quotes too.
			//Also changing filterOptions.quickFilter -> filterOptions.showQuickFilter to be consistent with Table
			if(this.filterOptions && this.filterOptions.showQuickFilter !== undefined){
				this.quickFilterVisible = this.filterOptions.showQuickFilter;
			}	
			if(this.filterOptions && this.filterOptions.showQuickFilterButton !== undefined){
				this.showQuickFilterButton = this.filterOptions.showQuickFilterButton;
			}			
			//console.log("postCreate: this.quickFilterVisible:"+this.quickFilterVisible);
			
			//commenting as this not required
			//this.toolbar = new dijit.Toolbar({id: "toolbar"+this.id});
			this.toolbarOptions = this.treeTable.toolbarOptions;
			this._mapToolbarControls();
			this._createToolBarControls();			
		},
		
		//16716: table toolbar has browser popup menu on right click
		_stopEvent: function(e){
			// summary:
			//		to stop event propagation
			
			//Since event is oncontextmenu, just stop it!
			dojo.stopEvent(e);				
		},
		
		_createToolBarControls: function(){
			// summary:
			//		Internal function specific to tree table used to create required toolbar options/controls
			
			if(this.refreshIcon) {
				var refreshTableIcon = new dijit.form.Button({
					id: this.id+"refreshIcon",
					showLabel: false,
					iconClass: "refreshIcon",
					label: this.resources_.REFRESH
				});
					
				refreshTableIcon.onClick = dojo.hitch (this, "_refreshTreeTable");
				refreshTableIcon.onKeypress = dojo.hitch (this, "_refreshTreeTable");
				var seperator = new dijit.ToolbarSeparator({id: "refSeperator"+this.id});
				
				this._addStateControl(refreshTableIcon,0);
				this._addStateControl(seperator);
				
				
			}
			if(this.showSelectAllIcon) {	
				//Satanik:25May2011: changes made to address ILMT requiremnt of showing only select all or deselect all icon
				if(this.onlySelectAllIcon){
					var selectAllIcon = new dijit.form.Button({
						id: this.id+"selectAllIcon",
						showLabel: false,
						iconClass: "selectAllIcon",
						label: this.resources_.SELECT_ALL
					});
						
					selectAllIcon.onClick = dojo.hitch (this, "_selectAll");
					selectAllIcon.onKeypress = dojo.hitch (this, "_selectAll");
					
					this._addFastPathControl(selectAllIcon);
					
					
				}else if(this.onlyDeselectAllIcon){
					var deselectAllIcon = new dijit.form.Button({
						id: this.id+"deselectAllIcon",
						showLabel: false,
						iconClass: "deselectAllIcon",
						label: this.resources_.DESELECT_ALL
					});
						
					deselectAllIcon.onClick = dojo.hitch (this, "_deselectAll");
					deselectAllIcon.onKeypress = dojo.hitch (this, "_deselectAll");
					
					this._addFastPathControl(deselectAllIcon);
				}else {
					var selectAllIcon = new dijit.form.Button({
						id: this.id+"selectAllIcon",
						showLabel: false,
						iconClass: "selectAllIcon",
						label: this.resources_.SELECT_ALL
					});
						
					selectAllIcon.onClick = dojo.hitch (this, "_selectAll");
					selectAllIcon.onKeypress = dojo.hitch (this, "_selectAll");
					
					this._addFastPathControl(selectAllIcon);
					
					var deselectAllIcon = new dijit.form.Button({
						id: this.id+"deselectAllIcon",
						showLabel: false,
						iconClass: "deselectAllIcon",
						label: this.resources_.DESELECT_ALL
					});
						
					deselectAllIcon.onClick = dojo.hitch (this, "_deselectAll");
					deselectAllIcon.onKeypress = dojo.hitch (this, "_deselectAll");
					this._addFastPathControl(deselectAllIcon);
				}
			}
			
			
			if(this.expandAllIcon) {
				/* Not supported in the base widget, dojo LazyTreeGrid
				var expandtAllIcon = new dijit.form.Button({
						id: this.id+"expandableAllIcon",
						showLabel: false,
						iconClass: "expandtAllIcon",
						label: this.resources_.EXPAND_ALL
					},this.expandtAllIcon);
					
				expandtAllIcon.onClick = dojo.hitch (this, "_expandAll");
				expandtAllIcon.onKeypress = dojo.hitch (this, "_expandAll");*/
				
				var collapseAllIcon = new dijit.form.Button({
					id: this.id+"collapseAllIcon",
					showLabel: false,
					iconClass: "collapseAllIcon",
					label: this.resources_.COLLAPSE_ALL
				});				
				//Bobby:18Mar2011. Workaround for now since refresh collapses the grid!	
				//Satanik:07Jun2011: ORIA fixed the refresh code so it does not collapse the grid so calling different api
				collapseAllIcon.onClick = dojo.hitch (this, "_collapseAll");
				collapseAllIcon.onKeypress = dojo.hitch (this, "_collapseAll");			
				this._addFastPathControl(collapseAllIcon);
				//collapseAllIcon.onClick = dojo.hitch (this, "_refreshTreeTable");
				//collapseAllIcon.onKeypress = dojo.hitch (this, "_refreshTreeTable");
			}
					
				
			if(this.clearSortIcon) {
				var removeSortIcon = new dijit.form.Button({
					id: this.id+"clearSortIcon",
					showLabel: false,
					iconClass: "removeSortIcon",
					label: this.resources_.CLEAR_SORT
				});
					
				removeSortIcon.onClick = dojo.hitch (this, "_clearSort");
				removeSortIcon.onKeypress = dojo.hitch (this, "_clearSort");
				this._addFastPathControl(removeSortIcon);
			}
			
			if(this.configureTreeTableIcon) {
				var configureColIcon = new dijit.form.Button({
					id: this.id+"configureTreeTableIcon",
					showLabel: false,
					iconClass: "configureColIcon",
					label: this.resources_.CONFIGURE_TREETABLE
				});
					
				configureColIcon.onClick = dojo.hitch (this, "_configureColumn");
				configureColIcon.onKeypress = dojo.hitch (this, "_configureColumn");
				this._addFastPathControl(configureColIcon);
			}
					
			//console.log("this.quickFilterVisible:"+this.quickFilterVisible);
			
			if(this.quickFilterVisible && this.showQuickFilterButton) {
				this.quickFilter = new dojoe.treetable._QuickSearchFilter({
					id: this.id+"quickFilter", 
					grid: this.treeTable.grid,
					_quickFilterButton: true,
					filterOptions: this.filterOptions },this.quickFilterNode);	
				var quickFilterIcon = new dijit.form.Button({
					showLabel: false,
					iconClass: "filterIcon",
					id: "quickFilterIcon"+this.id,
					label: this.resources_.QUICK_FILTER,
					title: this.resources_.QUICK_FILTER
				},this.filterButton);				
				quickFilterIcon.onClick = dojo.hitch (this, "_setQuickFilter");
				quickFilterIcon.onKeypress = dojo.hitch (this, "_setQuickFilter");
				quickFilterIcon.domNode.title = this.resources_.QUICK_FILTER;	
			}else if(this.quickFilterVisible && !this.showQuickFilterButton) {
				this.quickFilter = new dojoe.treetable._QuickSearchFilter({
					id: this.id+"quickFilter", 
					grid: this.treeTable.grid,
					_quickFilterButton: false,
					filterOptions: this.filterOptions },this.quickFilterNode);				
			}
			
			if(this.actionMenu){
				var menu = new dijit.Menu({ style: "display: none;"});
				//menu.addChild(new dijit.MenuSeparator());
				
				//23789: Refresh should be part of Actions Menu also - OID
				//Bobby:29Jun11				
				if(this.refreshMenu) {
					//Menu Item for Configure Columns action
					var refreshMenuItem = new dijit.MenuItem({
						//Bobby:19Jul11, title is reqd by JAWS to anounce the menu item.					
						//24709: JAWS does not read the items in the Actions, Export or Print menus.					
						title: this.resources_.REFRESH,
						label: this.resources_.REFRESH,
						iconClass: "refreshIcon"
					});
					refreshMenuItem.onClick = dojo.hitch (this, "_refreshTreeTable");
					refreshMenuItem.onKeypress = dojo.hitch (this, "_refreshTreeTable");					 
					menu.addChild(refreshMenuItem);
				}				
				
				if(this.selectAllMenu){
					var selectAllMenuItem = new dijit.MenuItem({
						 //id: this.id+"selectAllMenuItem",
						 title: this.resources_.SELECT_ALL,
						 label: this.resources_.SELECT_ALL,
						 iconClass: "selectAllIcon"			         
					 });
					 selectAllMenuItem.onClick = dojo.hitch (this, "_selectAll");
					 selectAllMenuItem.onKeypress = dojo.hitch (this, "_selectAll");	
					 menu.addChild(selectAllMenuItem);
					
					 var deselectAllMenuItem = new dijit.MenuItem({
						 //id: this.id+"deselectAllMenuItem",
						 title: this.resources_.DESELECT_ALL,
						 label: this.resources_.DESELECT_ALL,
						 iconClass: "deselectAllIcon"
					  });
					 deselectAllMenuItem.onClick = dojo.hitch (this, "_deselectAll");
					 deselectAllMenuItem.onKeypress = dojo.hitch (this, "_deselectAll");
					 menu.addChild(deselectAllMenuItem);
				}
				
				if(this.expandAllMenu){
					/* Not supported in the base widget, dojo LazyTreeGrid
					var expandAllMenuItem = new dijit.MenuItem({
						 id: this.id+"expandAllMenuItem",
						 label: this.resources_.EXPAND_ALL,
						 iconClass: "expandtAllIcon"
					 });
					 expandAllMenuItem.onClick = dojo.hitch (this, "_expandAll");
					 expandAllMenuItem.onKeypress = dojo.hitch (this, "_expandAll");
					 menu.addChild(expandAllMenuItem);*/
					 
					 var collapseAllMenuItem = new dijit.MenuItem({
						 //id: this.id+"collapeAllMenuItem",
						 title: this.resources_.COLLAPSE_ALL,						 
						 label: this.resources_.COLLAPSE_ALL,
						 iconClass: "collapseAllIcon"
					 });
					 collapseAllMenuItem.onClick = dojo.hitch (this, "_collapseAll");
					 collapseAllMenuItem.onKeypress = dojo.hitch (this, "_collapseAll");
					 //collapseAllMenuItem.onClick = dojo.hitch (this, "_refreshTreeTable");
					 //collapseAllMenuItem.onKeypress = dojo.hitch (this, "_refreshTreeTable");					 
					 menu.addChild(collapseAllMenuItem);
				}
					
				if(this.clearSortMenu){
					 var clearSortMenuItem = new dijit.MenuItem({
						 //id: this.id+"clearSortMenuItem",
						 title: this.resources_.CLEAR_SORT,
						 label: this.resources_.CLEAR_SORT,
						 iconClass: "removeSortIcon"
					 });
					 clearSortMenuItem.onClick = dojo.hitch (this, "_clearSort");
					 clearSortMenuItem.onKeypress = dojo.hitch (this, "_clearSort");
					 menu.addChild(clearSortMenuItem);
				}
				
				if(this.configureTreeTableMenu){
					 var configureColMenuItem = new dijit.MenuItem({
						 //id: this.id+"configureColMenuItem",
						 title: this.resources_.CONFIGURE_TREETABLE,
						 label: this.resources_.CONFIGURE_TREETABLE,
						 iconClass: "configureColIcon"
					 });
					 configureColMenuItem.onClick = dojo.hitch (this, "_configureColumn");
					 configureColMenuItem.onKeypress = dojo.hitch (this, "_configureColumn");
					 menu.addChild(configureColMenuItem);
				}
				menu.startup();
				var actionList = new dijit.form.DropDownButton({
					optionsTitle : this.resources_.ACTIONS,
			        label: this.resources_.ACTIONS,
			        dropDown: menu,
					scrollOnFocus : true
			    });				
				this.actionListWidget = actionList;		
				this._addFastPathControl(actionList);	
			}
		},
		
		_setQuickFilter: function(){
			// summary:
			//		Internal function to set filter criteria for quick filter
			
			//console.info("In _setQuickFilter.....");
			//this.quickFilter.filterObj = this.filterOptions;
			this.quickFilter._setFilter();
		},		
		
		_removeFromActionList: function (menuItemId){
			// summary:
			//		Internal function invoked by dojoe.treetable.TreeTable to remove items form Action list on the toolbar
			
			var menuItem = dijit.byId(menuItemId);
			if(menuItem !== null && menuItem !== undefined) {
				menuItem.destroyRecursive();
			}
		},
		
		//do not call this method directly, use table.removeItemFromToolbar(itemId) method to remove a custom item from the toolbar
		_removeItem: function(itemId){
			// summary:
			//		Internal function invoked by dojoe.treetable.TreeTable to remove controls on the toolbar
			
			var button= dijit.byId(itemId);
			if(button !== null && button !== undefined) {		
				if (button.domNode.parentNode) {
					//remove td which is the parent of the button
					buttonParent = button.domNode.parentNode;
					buttonParent.parentNode.removeChild(buttonParent);
				}
				button.destroyRecursive();
			}
		},
		
		//do not call this method directly, use treetable.addToToolbar(item,position,row) method to add a custom item to the toolbar before default buttons
		_addCustomControl: function(item, position, row){
			// summary:
			//		Internal function invoked by dojoe.treetable.TreeTable to add custom buttons, actions to the toolbar
			
			this.inherited(arguments);
		},	

		destroy: function(){
			// summary:  Destroys the toolbar.
			// tags: public	
			
			//memory leak issue: custom destroy method to take care of removing all widgets					
			try{
				delete this.resources_;
				//this is for destroying child templated widgets 
				//this should happen by itself but need to figure out why dijit._Templated is not taking care of it...
				var widgets = dijit.findWidgets(this.domNode);
				dojo.forEach(widgets, function(w) {
					if(w){
						if(w.destroyRecursive){
							w.destroyRecursive();
						} else if(w.destroy){
							w.destroy();
						}
					}
				});
				// this is for widgets added as child to this widget and their children..
				//not present in template
				var widgetArr = this.getChildren();
				dojo.forEach(widgetArr, function (child) {
					if(child){
						if(child.destroyRecursive){
							child.destroyRecursive();
						} else if(child.destroy){
							child.destroy();
						}
					}
				});
				this.inherited(arguments);
			}catch(ex){
				console.warn("Widget destroy failed: "+this.id+", ex="+ex.name+":"+ex.message);
			}
		}		
	//Uncomment above comma and add comma-separated functions here. Do not leave a 
	// trailing comma after last element.			
});
});

},
'ibm/tivoli/tip/dijit/TIPContextMenuProvider':function(){
// wrapped by build app
define(["dijit","dojo","dojox"], function(dijit,dojo,dojox){
/**
 *  You provide a context menu provider to do things like populate the context menu with
 *  items, and to handle the actions when one of those items are clicked.
 *  You provide:
 *  - getMenuItemsCallback (required)   (optional alternative: override getMenuItems)
 *      Parameters: (context)
 *                 context (that was passed on the contextMenuOpen call (in your code)
 *
 *      Summary: This callback provides the menu items to be inserted into the context menu.          
 *      The following defines the callback that is used to return menu items to appear
 *      in the list.  This method should return JSON data in the form:
 *       [ 
 *       { 
 *            "label": "testAction",
 *            "iconClass": "tipMenuIcon tipPropertiesIcon", 
 *            "actionID": "testActionID" 
 *       },
 *       { 
 *            "actionID": "_separator_" 
 *       },
 *       { 
 *            "label": "submenu", 
 *            "iconClass": "tipMenuIcon", 
 *            "submenu_items": 
 *            [ 
 *             { 
 *                  "label": "Delete", 
 *                 "iconClass": "tipMenuIcon tipDeleteIcon", 
 *                 "actionID": "testDeleteActionID" 
 *             }
 *            ] 
 *       } 
 *       ]
 *
 *  - getMenuCommandCallback (optional)  (optional alternative: override getMenuCommand)
 *      Parameters: (commandName, context, menuItem)
 *                 commandName - the command name of the context menu item.  Provided on the
 *                               'actionID' item above. 
 *                 context - (that was passed on the contextMenuOpen call (in your code)
 *                 menuItem - the actual menu item object.
 *      This callback provides the function needed to process a menu item click.
 *      The response depends largely on the action you wish to take.  A sample response:
 *
 *      {
 *            "launchType":"JavaScript",
 *            "uri":"alert('Hello World');"
 *      }
 *      Launch types include: JavaScript, WEB_URL, CLIENT_EVENT, PORTLET, PORTAL_PAGE, NOOP
 *
 *      The actions listed above may be included in the original information returned by 
 *      the getMenuItemsCallback.  If these actions are included, you need not specify
 *      a getMenuCommandCallback, and the provider will process these actions for you
 *      automatically.  For example:
 *       { 
 *            "label": "testAction",
 *            "iconClass": "tipMenuIcon tipPropertiesIcon",
 *            "actionID": "testActionID", 
 *            "launchType":"JavaScript",
 *            "uri":"alert('Hello World');"
 *       },
 */
 
dojo.provide("ibm.tivoli.tip.dijit.TIPContextMenuProvider");


dojo.declare("ibm.tivoli.tip.dijit.TIPContextMenuProvider", null,
{
   iscPortletNamespace: null,   // the namespace of this UI's portlet
                                // (must be set to enable launching of portlet or portlet page from a UI in
                                // a non-iWidget portlet)
   
   iscPortletInstanceID: null,  // the instance ID of this UI's portlet (also called "oid" in portletData)
                                // (must be set to enable launching of portlet or portlet page from a UI in
                                // an iWidget)
   
   is_iWidget: false,  // indicates if this menu provider is being used on a UI in an iWidget
   
   eventFormat: "mixed", // Event context format: "new"=new context, "old"=old context, "mixed"=new and old
   
   iWidgetIDPrefix: "_iw",  // prefix needed for iscPortletInstanceID if UI is an iWidget

   getMenuItemsCallback: null,
   
   getMenuCommandCallback: null,
   
   launchNewWindowOnNullId :true,

   menus_: null, // List of menus saved from getMenuItems

   constructor: function( args ) 
   {
      // summary:
      //      The base class for all providers that populate the TIPContextMenu
      dojo.mixin( this, args );

      this.id_ = "1_";
   },

   getMenuItems: function(context)
   {
      this.menus_ = this.getMenuItemsCallback(context);
      return this.menus_;
   },

   getItem: function(/*String*/ id) 
   {
      var submenus = [];
      var currentMenu = null;
      var x = null;
      submenus.push(this.menus_);
      
      currentMenu = submenus.pop();
      while (currentMenu)
      {
         x = currentMenu.pop();
         while (x)
         {
            if (x) 
            {
               if (x.submenu_items)
               {
                  submenus.push(x.submenu_items);
               }
               else
               {
                  if (x.id == id || x.actionID == id)
                  {
                     return x;
                  }
               }
            }
            x = currentMenu.pop();
         }
         currentMenu = submenus.pop();
      }
   },
   
   isArray: function (obj) 
   {
      if (obj.constructor.toString().indexOf("Array") == -1) 
      {
         return false;
      } else {
         return true;      
      }
   },

   getMenuCommand: function (commandName, context, menuItem) 
   {
      var result = null;
      if (this.getMenuCommandCallback !== null) 
      {
         result = this.getMenuCommandCallback(commandName, context, menuItem);
      }
      else
      {
         result = this.getItem(commandName);
      }

      // console.debug("------------ handleMenuAction result",result);
      this.handleMenuAction(result);
   },

   handleMenuAction: function (data, ioArgs) 
   {
      // console.info ("handleMenuAction: ");
      // console.info (data);

      if (data.launchType == "JavaScript") {
         this.handleJavascript (data);

      } else if (data.launchType == "WEB_URL") {
         this.launchURL (data);

      } else if (data.launchType == "CLIENT_EVENT") {
         this.sendEvent (data);

      } else if (data.launchType == "PORTLET") {
         this.launchPortlet (data);

      } else if (data.launchType == "PORTAL_PAGE") {
         this.launchPage(data);

      } else if (data.launchType == "NOOP") {
         // Do nothing in the case of a no operation type

      } else {
         console.error ("unsupported launchtype: " + data.launchType);
      }
   },

   handleJavascript: function (data) 
   {
      // console.log ("enter handleJavascript");
      var fnc = new Function ("eval (\"" + data.uri + "\");");
      fnc();
   },

   launchURL: function(data) 
   {
	   console.log ("enter launchURL id ");
		  console.info("browserwindowid :"+data.browserwindowid + " launchNewWindowOnNullId :"+this.launchNewWindowOnNullId);
		 var width  = 300;
		 var height = 200;
		 var left   = (screen.width  )/2;
		 var top    = (screen.height)/2;
		 var params = 'top='+top+', left='+left +', resizable=yes';
		
		 if(data.browserwindowid != null) {
		   window.open(data.uri,data.browserwindowid, params);
		 }
		 else {
		   if(this.launchNewWindowOnNullId == true) {
		      window.open(data.uri,"_blank", params);
		   }
		   else {
		      window.open(data.uri,"_self", params);
		   
		   }
		   
		 }
   },

   launchPortlet: function (data) 
   {
	   var portletEvent = {
                  name: "http://ibm.com/isclite#launchPortlet",
                  portletName: data.portletName,
                  portletApplicationID: data.portletAppUID
               };
	      
	   if (data.parameters)
	   {
		   for (var j=0; j<data.parameters.length; j++)
		   {
			   portletEvent[data.parameters[j].name] = data.parameters[j].value.toString();
		   }
	   }
	      
	   try
	   {
		   EventBroker.sendEvent (this.iscPortletNamespace, this.formatOutboundEvent(portletEvent));
	   } catch (e) {
		   console.error ("Caught javascript exception in TIPContextMenuProvider.launchPortlet: " + e);
		   alert (e);
	   }		  
   },
   
   launchPage: function (data) 
   {
	  if (this.is_iWidget)
	  {
		  // Get the TIP iWidget event service
		  var eventService = com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.iwidget.services.EventService.SERVICE_NAME);
		  var eventArray = new Array();
		  var eventData = {};
		  eventData.targetEvent = (("event" in data) && (data.event != null)) ? data.event : "http://ibm.com/isclite#launchPage"; 

		  // Set up the launch parameters as the payload
		  eventData.payload = {};
		  eventData.payloadType = "String";
		  if (data.parameters)
		  {
			  for (var k=0; k<data.parameters.length; k++)
			  {
				  eventData.payload[data.parameters[k].name] = data.parameters[k].value.toString();
			  }
		  }
		  eventData.payload = this.formatOutboundEvent(eventData.payload);
		  eventArray.push(eventData);
		  // Make sure the portletInstanceID has the iWidget prefix
		  var portletInstanceID = this.iscPortletInstanceID;
		  if (portletInstanceID && (portletInstanceID.indexOf(this.iWidgetIDPrefix) == 0))
		  {
			  portletInstanceID = this.iWidgetIDPrefix + portletInstanceID;
		  }
		  try 
		  {
			  eventService.broadcastEvents(eventArray, this.iscPortletInstanceID, data.portletPageID);			  
		  } catch (ex) {
			  console.error ("Caught javascript exception in TIPContextMenuProvider.launchPage: " + ex);
			  alert (ex);
		  }		  
	  }
	  else
	  {
		  var pageEvent = {
                         name: "http://ibm.com/isclite#launchPage",
                         NavigationNode: data.portletPageID,
                         pageInstanceRef: null,
                         switchPage: true
          };
		  
		  if (data.parameters)
		  {
			  for (var k=0; k<data.parameters.length; k++)
			  {
				  pageEvent[data.parameters[k].name] = data.parameters[k].value.toString();
			  }
		  }
		  
		  try
		  {
			  EventBroker.sendEvent (this.iscPortletNamespace, this.formatOutboundEvent(pageEvent));
		  } catch (ex) {
			  console.error ("Caught javascript exception in TIPContextMenuProvider.launchPage: " + ex);
			  alert (ex);
		  }
	  }
   },

   /* Formats an iWidget payload or porlet event.  The input parameter will be the   */
   /* entire event for portlets, and it will be the payload for iwidgets.            */
   /* This method will format the context according to the flag set in this provider */
   formatOutboundEvent: function(payload)
   {
      var newPayload=payload;
      if (this.eventFormat != "old"){ // CMS builds old context by default
         var util = new ibm.tivoli.tip.dojo.EventUtil();
         if (this.eventFormat == "mixed"){
            util.addNewContext(newPayload);
         } else if (this.eventFormat == "new"){
            newPayload={};
            if (typeof payload.name != "undefined"){ // Portlets have the event name coded in
               newPayload.name = payload.name;
            }
            if (typeof payload.source != "undefined"){ // Portlets have the event name coded in
               newPayload.source = payload.source;
            }
            newPayload.payload={};
            util.addNewContext(payload,newPayload.payload);
         }
      }
      return newPayload;
   },
   
   sendEvent: function (data) 
   {
   
   	  var standardEvent = this.createCMSEventCallback ? this.createCMSEventCallback(data) : null;   	  
   
      var parms = data.parameters;
	  if (this.is_iWidget)
	  {
		  // Get the TIP iWidget event service.  aasinha : the below service would not be required. This refers to 
		  // another overloaded fireEvent function which needs source and taget widget IDs.
		  // var eventService = com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.iwidget.services.EventService.SERVICE_NAME);
		  var iwId=this.iWidgetIDPrefix + this.iscPortletInstanceID;
		  var iWidgetContainer = com.ibm.mashups.iwidget.model.Factory.getGlobalWidgetModel();
		  var widgetObj = iWidgetContainer.getWidgetById(iwId);
		  if(widgetObj == null) {
			widgetObj = iWBridge.findIframedWidget(iwId);
		  }
		  var widget_iContext = widgetObj.iScope.iContext;
		  if(!standardEvent){
		  	var payload = {};
		  	// Set up the event parameters as the payload
		  	if (parms)
		  	{
			  	for (var i=0; i<parms.length; i++)
			  	{
				  	payload[parms[i].name] = parms[i].value.toString();
			  	}
		  	}
			//	eventService.fireEvent(data.event, "String", this.formatOutboundEvent(payload));  aasinha1 : this call goes to another method which needs source and target widget.
			widget_iContext.iEvents.fireEvent(data.event, "String", this.formatOutboundEvent(payload));
		  }else{
		  	//	eventService.fireEvent(standardEvent.name, "String", this.formatOutboundEvent(standardEvent.payload));
			widget_iContext.iEvents.fireEvent(standardEvent.name, "String", this.formatOutboundEvent(standardEvent.payload));
		  }			  
	  }
	  else
	  {
	  	if(!standardEvent){
		  var event = {
				  name: data.event
                  };

		  if (parms)
		  {
			  for (var i=0; i<parms.length; i++) 
			  {
				  event[parms[i].name] = parms[i].value.toString();
			  }
		  }		  
		  // console.log ("TIPContextMenu: sending event: ");
		  // console.log (event);
		  EventBroker.sendEvent (this.iscPortletNamespace, this.formatOutboundEvent(event));
		  }else{
		  	EventBroker.sendEvent (this.iscPortletNamespace, this.formatOutboundEvent(standardEvent));
		  }
	  }
   },

   destroy: function()
   {
   		this.inherited (arguments);
      // Placeholder: for overrides.  Exploiters should call this as best practice. 
   }
      
});
});

},
'ibm/tivoli/navmodel/DatasetItemsStore':function(){
// wrapped by build app
define("ibm/tivoli/navmodel/DatasetItemsStore", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/DatasetStore,ibm/tivoli/navmodel/Dataset"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */
 
dojo.provide("ibm.tivoli.navmodel.DatasetItemsStore");

// includes
dojo.require("ibm.tivoli.navmodel.DatasetStore");
dojo.require("ibm.tivoli.navmodel.Dataset");
	
ibm.tivoli.navmodel.DatasetItemsStore._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.DatasetItemsStore");
	
dojo.declare("ibm.tivoli.navmodel.DatasetItemsStore", ibm.tivoli.navmodel.DatasetStore, {
	// private members
	_clearCache: false,
	_trace: ibm.tivoli.navmodel.DatasetItemsStore._trace,
	
	/*=====
	constructor_kwArgs: function(dataset, defaultProperties, returnItem, returnValue) {
		// dataset: Object
		//		A ibm.tivoli.navmodel.Dataset instance
		// defaultProperties: String[]
		//		An array of property id values that are pre-fetched with the items
		// returnItem: Boolean
		//		A boolean indicating if the getValue() method should just return the actual
		//		item so that it can be used for custom formatting of the value
		// returnValue: Boolean
		//		A boolean indicating if the getValue() method should just return the actual
		//		raw value so that it can be used for custom formatting of the value
	},
	=====*/
	
	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructor for dojo datastore;
		//		that receives item data from an ibm.tivoli.navmodel.Dataset
		//		Base class will mixin kwArgs		
	},
	
	
	//////////////////////////////////////////////////////////////
	// dojo.data.api.Identity
	//////////////////////////////////////////////////////////////
	
	getIdentity: function(/*Object*/ item){
		// summary:
    	//		Returns a unique identifier for an item. The return value will be
    	//		either a string or something that has a toString() method.
    	// item:
    	//		The item from the store from which to obtain its identifier.
		// exceptions:
    	//		Conforming implementations may throw an exception or return null if
    	//		item is not an item.
		if (!this.isItem(item)){
  			throw new Error("Specified item argument is not a valid item.");
  		}
  		
		return item.getId();		
    },
    
	/*=====
	_onItem: function(item) {},
	=====*/
	
	/*=====
	_onError: function(error) {},
	=====*/
	
	/*=====
	fetchItemByIdentity_kwArgs: function(identity, onItem, onError, scope) {
		// identity: String|Object
		//		The identity parameter is the identity of the item you wish to locate and load
		//		This attribute is required. It should be a string or an object that toString()
		//		can be called on.
		// onItem: _onItem
		//		The onItem parameter is the callback to invoke when the item has been loaded. It takes only one
		//		parameter, the item located, or null if none found.
		// onError: _onError
		//		The onError parameter is the callback to invoke when the item load encountered an error. It takes 
		//		only one parameter, the error object
		// scope: Object
		//		If a scope object is provided, all of the callback functions (onItem,
		//		onError, etc) will be invoked in the context of the scope object.
	},
	=====*/
	
 	fetchItemByIdentity: function(/*fetchItemByIdentity_kwArgs*/ kwArgs) {
		// summary:
		//		Given the identity of an item, this method returns the item that has
		//		that identity through the onItem callback. Conforming implementations
		//		should return null if there is no item with the given identity.
		//		Implementations of fetchItemByIdentity() may sometimes return an item
		//		from a local cache and may sometimes fetch an item from a remote server
	 	var args = {
 			identity: null,
 			onItem: function(item) {},
 			onError: function(error) {},
 			scope: null
 		};
 		dojo.mixin(args, kwArgs);
 		
 		var item = null;
 		var error = null;
 		if (args.identity) {
 			try { 			
  				item = this.dataset.getItem({itemId: args.identity, properties: this.defaultProperties});
  			}
  			catch(err) {
  				error = err;
  			}  			
  		}  		
  		
 		if (item) {
 			args.onItem.call(args.scope, item);
 		}
 		else {
 			args.onError.call(args.scope, error ? error : "Could not fetch item for identity=" + args.identity);
 		}		
	},
	
	
	//////////////////////////////////////////////////////////////
	// dojo.data.api.Read
	//////////////////////////////////////////////////////////////
	
	/*=====
	fetch_onBegin: function(size, request){},
	=====*/
	
	/*=====
	fetch_onItem: function(item, request){},
	=====*/
	
	/*=====
	fetch_onComplete: function(items, request){},
	=====*/
	
	/*=====
	fetch_onError: function(errorData, request){},
	=====*/
	
	/*=====
	fetch_kwArgs: function(query, queryOptions, onBegin, onItem, onComplete, onError, scope,
							start, count, sort) {
		// summary:
		//		The kwArgs parameter may either be an instance of conforming to 
		//		dojo.data.api.Request or may be a simple anonymous object
		//		containing any of the above parameters.
		// query: String|Object
		//		The query parameter is used to specify search and filter condition information for the fetch.
		//		Only those items that match the search and/or filter will be returned.
		//		|	{	ids: {
		//		|			propId: boolean		// indicates propIds involved in the expression
		//		|		},
		//		|		expression: string,		// expression involiving propId's and values  (and/or together)
		//		|								// example:  ((Name contains 'abc' || Type contains 'abc)) and (Size > 100)
		//		|
		//		|		// the following is for compatability with legacy datastore
		//		| 		navModel: {
		//		|			search: {
		//		|				term: string,		the search text
		//		|				attributes: array	["property id1", "property id2", ...]
		//		|			}
		//		|		}
		//		|	}
		// queryOptions: Object?
		//		The queryOptions parameter is an optional parameter that can be used to modify the query.  The
		//		ignoreCase case can be specified to globally make all of the filter conditions case sensitive/insensitive.
		//		|	{ ignoreCase: boolean
		//		|	}
		// onBegin: fetch_onBegin
		//		If an onBegin callback function is provided, the callback function will be called just once, 
		//		before the first onItem callback is called. The onBegin callback function will be passed two arguments, 
		//		the the total number of items identified and the Request object.  If the total number is
		//		unknown, then size will be -1.  Note that size is not necessarily the size of the collection of items 
		//		returned from the query, as the request may have specified to return only a subset of the total set 
		//		of items through the use of the start and count parameters.
		// onItem: fetch_onItem
		//		If an onItem callback function is provided, the callback function will be called just once, 
		//		after the last onItem callback is called. Note that if the onItem callback is not present, 
		//		then onComplete will be passed an array containing all items which matched the query and the 
		//		request object. If the onItem callback is present, then onComplete is called as: onComplete(null, request).
		// onComplete: fetch_onComplete
		//		If an onComplete callback function is provided, the callback function will be called just once,
		//		after the last onItem callback is called. Note that if the onItem callback is not present,
		//		then onComplete will be passed an array containing all items which matched the query and the 
		//		request object. If the onItem callback is present, then onComplete is called as: onComplete(null, request).
		// onError: fetch_onError
		//		If an onError callback function is provided, the callback function will be called if there is any 
		//		sort of error while attempting to execute the query. The onError callback function will be passed two arguments:
		//		an Error object and the Request object.
		// scope: Object
		//		If a scope object is provided, all of the callback functions (onItem, onComplete, onError, etc) 
		//		will be invoked in the context of the scope object.
		// start: int
		//		If a start parameter is specified, this is a indication to the datastore to
		//		only start returning items once the start number of items have been located and
		//		skipped.  When this parameter is paired with 'count', the store should be able
		//		to page across queries with millions of hits by only returning subsets of the
		//		hits for each query
		// count: int
		//		 If a count parameter is specified, this is a indication to the datastore to
		//		 only return up to that many items.  This allows a fetch call that may have
		//		millions of item matches to be paired down to something reasonable.
		// sort: Object[]
		//		If a sort parameter is specified, this is a indication to the datastore to
		//		sort the items in some manner before returning the items.  The array is an array of
		//		javascript objects that must conform to the following format to be applied to the
		//		fetching of items:
		//		|	{
		//		|		attribute: attribute || attribute-name-string,
		//		|		descending: true|false;   // Optional.  Default is false.
		//		|	}
	},
	=====*/
	
	fetch: function(/*fetch_kwArgs*/ kwArgs) {
		// summary:
		//		Given a query and set of defined options, such as a start and count of items to return,
		//		this method executes the query and makes the results available as data items.
		//		The format and expectations of stores is that they operate in a generally asynchronous
		//		manner, therefore callbacks are always used to return items located by the fetch parameters.
		// description:
		//		A Request object will always be returned and is returned immediately.
		//		The basic request is nothing more than the keyword args passed to fetch and
		// 		an additional function attached, abort().  The returned request object may then be used
		//		to cancel a fetch.  All data items returns are passed through the callbacks defined in the
		//		fetch parameters and are not present on the 'request' object.
		// returns:
		//		The fetch() method will return a javascript object conforming to the API
		//		defined in dojo.data.api.Request.  In general, it will be the keywordArgs
		//		object returned with the required functions in Request.js attached.
		//		Its general purpose is to provide a convenient way for a caller to abort an
		//		ongoing fetch.
		// exceptions:
		//		Throws an exception if the query is not valid, or if the query is required but was not supplied.
		var args = {
			query: {},
			queryOptions: {},
			onBegin: function(size, request) {},
			onItem: function(item, request) {},
			onComplete: function(items, request){},
			onError: function(errorData, request){},
			scope: null,
			start: 0,
			count: null, //default should be to get all
			sort: null,
			abort: function() {}
		};
		dojo.mixin(args, kwArgs);
		
		if (this._trace.isFinest()) {
			this._trace.finest("dataset=" + this.dataset.getId() + " start=" + args.start + " count=" + args.count, "fetch");
		}

		if (this.dataset._args.onPage) {
			this.dataset._args.onPage.call(args.scope, args.start, args.count, args);
		}		

		// make sure event handling is set up
		this._validateEventHandler();
		
		// TED refresh is occurring
		this._clearCache = false;
		if (this.isRefresh()) {
			this._clearCache = this.dataset.isClearCacheOnRefresh();						
		}
		
		// translate information from the args
		var sort = this._getSort(args.sort);
		var condition = this._getCondition(args.query);	
			
		var itemArgs = {
			onComplete: dojo.hitch(this, function(items, itemArgs) {
				args.onBegin.call(args.scope, itemArgs.filteredRows, args);
				
				if (items) {
					var itemsLength = items.length;
					for(var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {                 	                	
                		args.onItem.call(args.scope, items[itemIndex], args);                	
					}
				}
				
				args.onComplete.call(args.scope, items, args);				
			}),
			
			onError: dojo.hitch(this, function(error, exception, ioArgs) {
				args.onError.call(args.scope, error, args);
			}),
			
			clearCache: this._clearCache,
			start: args.start,
			count: args.count,	
			properties: this.defaultProperties,
			sort: sort,
			condition: condition
		}; 		
		this.dataset.getItems(itemArgs);	
		       
        return args;
    },
    
    
    getValue: function(/*Object*/ item, /*String*/ attribute, /*Object?*/ defaultValue, /*Object*/ useValue) {
		// summary:
    	//		Returns a single attribute value. Returns defaultValue if and only if item does not have a 
    	//		value for attribute. Returns null if and only if null was explicitly set as the attribute value. 
    	//		Returns undefined if and only if the item does not have a value for the given attribute 
    	//		(which is the same as saying the item does not have the attribute).
    	// item:
    	//		The item to access value on.
    	//		ibm.tivoli.navmodel.Item object
    	// attribute:
    	//		The attribute to access represented as a string.
    	// defaultValue:
    	//		Optional. A default value to use for the getValue return if the attribute does not exist or 
    	//		has no value.
    	// useValue: Boolean
	//		A boolean indicating if the getValue() method should just return the actual
	//		raw value so that it can be used for custom formatting of the value
    	// exceptions:
    	//		Throws an exception if *item* is not an item, or *attribute* is not a string
        if (arguments.length < 2) {
            throw new Error("Incorrect number of arguments specified: expecting 2 (item, attribute)");
        }
        
        if (!this.isItem(item)) {
            throw new Error("Specified item argument is not a valid item.");
        }
        
        if (!dojo.isString(attribute)) {
            throw new Error("Specified attribute argument is not a string.");
        }  
          	
        if (this.returnItem) {
        	return item;
        }
        
        var value = defaultValue;
        var prop = item.getProperty(attribute);
        if (prop) {
	  	if (this.returnValue || useValue)
			value = prop.getValue();
		else 
			value = prop.getDisplayValue();            
        }        	        
                        
        return value;
    },
    
    /*=====
	loadItem_kwArgs: function(item, onItem, onError, scope) {
		// summary:
		//		An anonymous object that defines the item to load and callbacks to invoke when the
		//		load has completed.
		// item: Object
		//		The item parameter is an object that represents the item in question that should be
		//		ontained by the store.  This attribute is required.
		//		ibm.tivoli.navmodel.Item object
		// onItem: _onItem
		//		The onItem parameter is the callback to invoke when the item has been loaded.
		//		It takes only one parameter, the fully loaded item.
		// onError: _onError
		//		The onError parameter is the callback to invoke when the item load encountered an error.
		//		It takes only one parameter, the error object.
		// scope: Object
		//		If a scope object is provided, all of the callback functions (onItem, onError, etc) 
		//		will be invoked in the context of the scope object. 
		//		onItem.call(dojo.global(), item, request)
	},
	=====*/
    
	loadItem: function(/*loadItem_kwArgs*/ kwArgs) {
		// summary:
		//		Given an item, this method loads the item so that a subsequent call to 
		//		store.isItemLoaded(item) will return true.  If a call to isItemLoaded() returns true 
		//		before loadItem() is even called, then loadItem() need not do any work at all and will 
		//		not even invoke the callback handlers.  So, before invoking this method, check that
		//		the item has not already been loaded.
		var args = {
			item : null,
			onItem: function(item) {},
			onError: function(error){},
			scope: null
		};		
		dojo.advancedMixin(args, kwArgs);
	
		if (!this.isItem(args.item)){
  			throw new Error("Specified item argument is not a valid item.");
  		}
  		
		if (this.isItemLoaded(args.item)){
			return args.item;
		}
		
		try {
			var dataset = args.item.getDataset();
			var item = dataset.getItem({itemId: args.item.getId(), properties: this.defaultProperties});
			args.onItem.call(args.scope, item);
		}
		catch(err) {
			args.onError.call(args.scope, err);
		}
	},
	
	//////////////////////////////////////////////////////////////
	// private methods
	///////////////////////////////////////////////////////////////
	
	_getObjectTypes: function() {
		// summary:
		//		Returns String[] of object types that can in the datastore
    	return {"item" : true};
    },
    
    _handleItemPropertiesChangedEvent: function(/*Object*/ event) {
		var items = event.getItems();    	
		var propsArray = event.getProperties();			
		if (items && propsArray) {
			var refresh = false;
			var length = items.length;			
			for (var index = 0;  index < length;  ++index) {
				var properties = propsArray[index];				
				if (properties) {
					var propsLength = properties.length;
					for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
						var item = items[index];
						var property = properties[propIndex];
						var propId = property.getId();
						if (this._trace.isFinest()) {
							this._trace.finest("datasetId=" + event.getDataset().getId() + " itemId=" + item.getId() + " prop=" + propId + " value=" + property.getDisplayValue(), "_handleItemPropertiesChangedEvent");
						}
						
						this.onSet(item, propId, null, property.getDisplayValue());
						
						if (!refresh && (this._affectsSort(propId) || this._affectsCondition(propId))) {
							refresh = true;							
						}
					}										
				}				
			}
			
			if (refresh) {
				this.onRefresh();
			}	
		}	
	},
	
    _handleItemsAddedEvent: function(/*Object*/ event) {
		var items = event.getItems();
		if (items) {
    		var length = items.length;			
			for (var index = 0;  index < length;  ++index) {
				var item = items[index];				
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + event.getDataset().getId() + " itemId=" + item.getId(), "_handleItemsAddedEvent");
				}
				this.onNew(item);
			}
			
			if (this._hasSort() || this._hasCondition()) {
				this.onRefresh();
			}
		}		    		
	},
	
	_handleItemsRemovedEvent: function(/*Object*/ event) {
		var items = event.getItems();
		if (items) {
    		var length = items.length;			
			for (var index = 0;  index < length;  ++index) {
				var item = items[index];				
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + event.getDataset().getId() + " itemId=" + item.getId(), "_handleItemsRemovedEvent");
				}
				this.onDelete(item);
			}			
		}
	}
});
});

},
'gridx/core/model/cache/Async':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/_base/Deferred",
	"dojo/DeferredList",
	"./_Cache"
], function(declare, array, lang, Deferred, DeferredList, _Cache){

	var hitch = lang.hitch;

	function minus(rangesA, rangesB){
		//Minus index range list B from index range list A, 
		//assuming A and B do not have overlapped ranges.
		//This is a set operation
		if(!rangesB.length || !rangesA.length){
			return rangesA;
		}
		var indexes = [], f = 0, r, res = [],
			mark = function(idx, flag){
				indexes[idx] = indexes[idx] || 0;
				indexes[idx] += flag;
			},
			markRanges = function(ranges, flag){
				var i, r;
				for(i = ranges.length - 1; i >= 0; --i){
					r = ranges[i];
					mark(r.start, flag);
					if(r.count){
						mark(r.start + r.count, -flag);
					}
				}
			};
		markRanges(rangesA, 1);
		markRanges(rangesB, 2);
		for(var i = 0, len = indexes.length; i < len; ++i){
			if(indexes[i]){
				f += indexes[i];
				if(f === 1){
					res.push({
						start: i
					});
				}else{
					if(f === 3){
						res._overlap = true;
					}
					r = res[res.length - 1];
					if(r && !r.count){
						r.count = i - r.start;
					}
				}
			}
		}
		return res;
	}

	function mergeRanges(args){
		//Merge index ranges into separate ones.
		var ranges = [], r = args.range, i, t, a, b, c, merged;
		while(r.length > 0){
			c = a = r.pop();
			merged = 0;
			for(i = r.length - 1; i >= 0; --i){
				b = r[i];
				if(a.start < b.start){
					//make sure a is always after b, so the logic can be simplified
					t = b;
					b = a;
					a = t;
				}
				//If b is an open range, and starts before a, then b must include a.
				if(b.count){
					//b is a closed range, it's possible to overlap.
					if(a.start <= b.start + b.count){
						//overlap
						if(a.count && a.start + a.count > b.start + b.count){
							b.count = a.start + a.count - b.start;
						}else if(!a.count){
							b.count = null;
						}
						//otherwise, b includes a
					}else{
						//not overlap, try next range
						a = c;
						continue;
					}
				}
				//now n is a merged range
				r[i] = b;
				merged = 1;
				break;
			}
			if(!merged){
				//Can not merge, this is a sperate range
				ranges.push(c);
			}
		}
		args.range = ranges;
		return args;
	}

	function connectRanges(args, ps){
		//Connect small ranges into big ones to reduce request count
		//FIXME: find a better way to do this!
		var r = args.range, ranges = [], a, b;
		r.sort(function(a, b){
			return a.start - b.start;
		});
		while(r.length){
			a = r.shift();
			if(r.length){
				b = r[0];
				if(b.count && b.count + b.start - a.start <= ps){
					b.count = b.count + b.start - a.start;
					b.start = a.start;
					continue;
				}else if(!b.count && b.start - a.start < ps){
					b.start = a.start;
					continue;
				}
			}
			ranges.push(a);
		}
		//Improve performance for most cases
		if(ranges.length == 1 && ranges[0].count < ps){
			ranges[0].count = ps;
		}
		args.range = ranges;
		return args;
	}

	function isNumber(n){
		return typeof n == 'number' && !isNaN(n);
	}

	return declare(/*===== "gridx.core.model.cache.Async", =====*/_Cache, {
		// summary:
		//		
		isAsync: true,
		//By default, do not clear cache when scrolling, this is the same with DataGrid
		//cacheSize: -1,
		//pageSize: 100,
		
		constructor: function(model, args){
			var cs = args.cacheSize,
				ps = args.pageSize;
			this.cacheSize = isNumber(cs) ? cs : -1;
			this.pageSize = isNumber(ps) && ps > 0 ? ps : 100;
		},

		when: function(args, callback){
			var t = this,
				d = args._def = new Deferred(),
				fail = hitch(d, d.errback),
				innerFail = function(e){
					t._requests.pop();
					fail(e);
				};
			t._fetchById(args).then(function(args){
				t._fetchByIndex(args).then(function(args){
					t._fetchByParentId(args).then(function(args){
						Deferred.when(args._req, function(){
							var err;
							if(callback){
								try{
									callback();
								}catch(e){
									err = e;
								}
							}
							t._requests.shift();
							//this is meaningless given current model impl
//                            if(!t.skipCacheSizeCheck && !t._requests.length){
//                                t._checkSize();
//                            }
							if(err){
								d.errback(err);
							}else{
								d.callback();
							}
						}, innerFail);
					}, innerFail);
				}, innerFail);
			}, fail);
			return d;
		},
	
		keep: function(id){
			var t = this,
				k = t._kept;
			if(t._cache[id] && t._struct[id] && !k[id]){
				k[id] = 1;
				++t._keptSize;
			}
		},
	
		free: function(id){
			var t = this;
			if(!t.model.isId(id)){
				t._kept = {};
				t._keptSize = 0;
			}else if(t._kept[id]){
				delete t._kept[id];
				--t._keptSize;
			}
		},

		clear: function(){
			var t = this;
			if(t._requests && t._requests.length){
				t._clearLock = 1;	//1 as true
				return;
			}
			t.inherited(arguments);
			t._requests = [];
			t._priority = [];
			t._kept = {};
			t._keptSize = 0;
		},

		//-----------------------------------------------------------------------------------------------------------
		_init: function(){},

		_findMissingIds: function(ids){
			var c = this._cache;
			return array.filter(ids, function(id){
				return !c[id];
			});
		},

		_searchRootLevel: function(ids){
			//search root level for missing ids
			var t = this,
				d = new Deferred(),
				fail = hitch(d, d.errback),
				indexMap = t._struct[''],
				ranges = [],
				lastRange,
				premissing; //Whether the previous item is missing
			if(ids.length){
				for(var i = 1, len = indexMap.length; i < len; ++i){
					if(!indexMap[i]){
						if(premissing){
							lastRange.count++;
						}else{
							premissing = 1;
							ranges.push(lastRange = {
								start: i - 1,
								count: 1
							});
						}
					}
				}
				ranges.push({
					start: indexMap.length < 2 ? 0 : indexMap.length - 2
				});
			}
			var func = function(ids){
				if(ids.length && ranges.length){
					t._storeFetch(ranges.shift()).then(function(){
						func(t._findMissingIds(ids));
					}, fail);
				}else{
					d.callback(ids);
				}
			};
			func(ids);
			return d;
		},

		_searchChildLevel: function(ids){
			//Search children level of current level for missing ids
			var t = this,
				d = new Deferred(),
				fail = hitch(d, d.errback),
				st = t._struct,
				parentIds = st[''].slice(1),
				func = function(ids){
					if(ids.length && parentIds.length){
						var pid = parentIds.shift();
						t._loadChildren(pid).then(function(){
							[].push.apply(parentIds, st[pid].slice(1));
							func(t._findMissingIds(ids));
						}, fail);
					}else{
						d.callback(ids);
					}
				};
			func(ids);
			return d;
		},
	
		_fetchById: function(args){
			//Although store supports query by id, it does not support get index by id, so must find the index by ourselves.
			var t = this,
				d = new Deferred(), 
				i, r, len, pid,
				success = hitch(d, d.callback),
				fail = hitch(d, d.errback),
				ranges = args.range,
				isTree = t.store.getChildren;
			args.pids = [];
			if(isTree){
				for(i = ranges.length - 1; i >= 0; --i){
					r = ranges[i];
					pid = r.parentId;
					if(t.model.isId(pid)){
						args.id.push(pid);
						args.pids.push(pid);
						ranges.splice(i, 1);
					}
				}
			}
			var ids = t._findMissingIds(args.id),
				mis = [];
			if(ids.length){
				array.forEach(ids, function(id){
					var idx = t.idToIndex(id);
					if(idx >= 0 && !t.treePath(id).pop()){
						ranges.push({
							start: idx,
							count: 1
						});
					}else{
						mis.push(id);
					}
				});
				t._searchRootLevel(mis).then(function(ids){
					if(ids.length && isTree){
						t._searchChildLevel(ids).then(function(ids){
							if(ids.length){
								console.warn('Requested row ids are not found: ', ids);
							}
							success(args);
						}, fail);
					}else{
						success(args);
					}
				}, fail);
			}else{
				success(args);
			}
			return d;
		},

		_fetchByParentId: function(args){
			var t = this,
				d = new Deferred();
			new DeferredList(array.map(args.pids, function(pid){
				return t._loadChildren(pid);
			}), 0, 1).then(hitch(d, d.callback, args), hitch(d, d.errback));
			return d;
		},

		_fetchByIndex: function(args){
			var t = this,
				d = new Deferred(),
				size = t._size[''];
			args = connectRanges(
					t._mergePendingRequests(
						t._findMissingIndexes(mergeRanges(args))), t.pageSize);
			var ranges = size > 0 ? array.filter(args.range, function(r){
				if(r.count > 0 && size < r.start + r.count){
					r.count = size - r.start;
				}
				return r.start < size;
			}) : args.range;
			new DeferredList(array.map(ranges, function(r){
				return t._storeFetch(r);
			}), 0, 1).then(hitch(d, d.callback, args), hitch(d, d.errback));
			return d;
		},
	
		_findMissingIndexes: function(args){
			//Removed loaded rows from the request index ranges.
			//generate unsorted range list.
			var i, j, r, end, newRange,
				t = this,
				ranges = [],
				indexMap = t._struct[''],
				totalSize = t._size[''];
			for(i = args.range.length - 1; i >= 0; --i){
				r = args.range[i];
				end = r.count ? r.start + r.count : indexMap.length - 1;
				newRange = 1;
				for(j = r.start; j < end; ++j){
					var id = indexMap[j + 1];
					if(!id || !t._cache[id]){
						if(newRange){
							ranges.push({
								start: j,
								count: 1
							});
						}else{
							++ranges[ranges.length - 1].count;
						}
						newRange = 0;
					}else{
						newRange = 1;
					}
				}
				if(!r.count){
					if(!newRange){
						delete ranges[ranges.length - 1].count;
					}else if(totalSize < 0 || j < totalSize){
						ranges.push({
							start: j
						});
					}
				}
			}
			args.range = ranges;
			return args;
		},

		_mergePendingRequests: function(args){
			var i, req,
				defs = [],
				reqs = this._requests;
			for(i = reqs.length - 1; i >= 0; --i){
				req = reqs[i];
				args.range = minus(args.range, req.range);
				if(args.range._overlap){
					defs.push(req._def);
				}
			}
			args._req = defs.length && new DeferredList(defs, 0, 1);
			reqs.push(args);
			return args;
		},
	
		_checkSize: function(){
			var t = this, id,
				cs = t.cacheSize,
				p = t._priority;
			if(t._clearLock){
				t._clearLock = 0;	//0 as false
				t.clear();
			}else if(cs >= 0){
				cs += t._keptSize;
//                console.warn("### Cache size:", p.length,
//                        ", To release: ", p.length - cs,
//                        ", Keep size: ", this._keptSize);
				while(p.length > cs){
					id = p.shift();
					if(t._kept[id]){
						p.push(id);
					}else{
						delete t._cache[id];
					}
				}
			}
		}
	});
});

},
'ibm/usmi/widgets/util/TextAbbr':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

define(["dojo/_base/kernel",
        "dojo/_base/declare",
        "dijit/_WidgetBase",
        "dijit/_TemplatedMixin",
        "dijit/Tooltip",
        // NextGen imports
        "ibm/usmi/widgets/util/Utils"],
    function (dojo, declare, _WidgetBase, _TemplatedMixin, Tooltip, Utils) {

    return declare([_WidgetBase, _TemplatedMixin], {

/** copied from TPC Util.js **/


    widgetsInTemplate: true,
    templateString: '<span ></span>',

    text: "",
    maxLength: 23, //default value, can be passed in as a parameter
    _minLength: 10, //the minimum length that maxLength can be
    tooltip: null,
    textContainer: null,
    center: true,
    format: null,

    postCreate: function _postCreate(){
        this.inherited(arguments);
        if(this.maxLength < this._minLength){
            this.maxLength = this._minLength;
        }
        //setup the widget
        this._setupTextAbbr();
    },

    _setupTextAbbr: function __setupTextAbbr(){
        //destroy the tooltip if it exists
        if(this.tooltip){ this.tooltip.destroyRecursive(); }

        //if the textContainer already exists replace it with the new textContainer
        if(this.textContainer){
            var newTextContainer = this._createTextContainer(this.text);
            dojo.place(newTextContainer, this.textContainer, "replace");
            this.textContainer = newTextContainer;
        } else {
            this.textContainer = this._createTextContainer(this.text);
            this.domNode.appendChild(this.textContainer); //add node to this domNode
        }
        if(this.textContainer.connectTooltip){
           this.tooltip = new Tooltip({connectId: [this.textContainer], label: this.text}); //create tooltip
        }
    },

    _createTextContainer: function __createTextContainer(/*String*/text){
        if(text.length <= this.maxLength){
            return dojo.create("span", { innerHTML: text,
                                         "connectTooltip": false });
        } else {
            return dojo.create("span", { innerHTML: this.abbreviate(text),
                                         "connectTooltip": true,
                                         "aria-label": text});
        }
    },

    update: function(/*String*/newText, /*int*/maxLength){
        this.text = newText;
        this.maxLength = maxLength || this.maxLength;

        //setup the widget with new text
        this._setupTextAbbr();
    },

    /* Abbreviate the string */
    abbreviate: function _abbreviate(/*String*/st){
        return Utils.abbreviate(st, {min: this._minLength, max: this.maxLength, center: this.center, format: this.format});
    },

    /* Destroy the tooltip if it exists */
    destroy: function _destroy(){
        if(this.tooltip){ this.tooltip.destroyRecursive(); }
        this.inherited(arguments);
    }

});
});

},
'dojox/grid/Selection':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/dom-attr"
], function(declare, array, lang, domAttr){

return declare("dojox.grid.Selection", null, {
	// summary:
	//		Manages row selection for grid. Owned by grid and used internally
	//		for selection. Override to implement custom selection.

	constructor: function(inGrid){
		this.grid = inGrid;
		this.selected = [];

		this.setMode(inGrid.selectionMode);
	},

	mode: 'extended',

	selected: null,
	updating: 0,
	selectedIndex: -1,

	setMode: function(mode){
		if(this.selected.length){
			this.deselectAll();
		}
		if(mode != 'extended' && mode != 'multiple' && mode != 'single' && mode != 'none'){
			this.mode = 'extended';
		}else{
			this.mode = mode;
		}
	},

	onCanSelect: function(inIndex){
		return this.grid.onCanSelect(inIndex);
	},

	onCanDeselect: function(inIndex){
		return this.grid.onCanDeselect(inIndex);
	},

	onSelected: function(inIndex){
	},

	onDeselected: function(inIndex){
	},

	//onSetSelected: function(inIndex, inSelect) { };
	onChanging: function(){
	},

	onChanged: function(){
	},

	isSelected: function(inIndex){
		if(this.mode == 'none'){
			return false;
		}
		return this.selected[inIndex];
	},

	getFirstSelected: function(){
		if(!this.selected.length||this.mode == 'none'){ return -1; }
		for(var i=0, l=this.selected.length; i<l; i++){
			if(this.selected[i]){
				return i;
			}
		}
		return -1;
	},

	getNextSelected: function(inPrev){
		if(this.mode == 'none'){ return -1; }
		for(var i=inPrev+1, l=this.selected.length; i<l; i++){
			if(this.selected[i]){
				return i;
			}
		}
		return -1;
	},

	getSelected: function(){
		var result = [];
		for(var i=0, l=this.selected.length; i<l; i++){
			if(this.selected[i]){
				result.push(i);
			}
		}
		return result;
	},

	getSelectedCount: function(){
		var c = 0;
		for(var i=0; i<this.selected.length; i++){
			if(this.selected[i]){
				c++;
			}
		}
		return c;
	},

	_beginUpdate: function(){
		if(this.updating === 0){
			this.onChanging();
		}
		this.updating++;
	},

	_endUpdate: function(){
		this.updating--;
		if(this.updating === 0){
			this.onChanged();
		}
	},

	select: function(inIndex){
		if(this.mode == 'none'){ return; }
		if(this.mode != 'multiple'){
			this.deselectAll(inIndex);
			this.addToSelection(inIndex);
		}else{
			this.toggleSelect(inIndex);
		}
	},

	addToSelection: function(inIndex){
		if(this.mode == 'none'){ return; }
		if(lang.isArray(inIndex)){
			array.forEach(inIndex, this.addToSelection, this);
			return;
		}
		inIndex = Number(inIndex);
		if(this.selected[inIndex]){
			this.selectedIndex = inIndex;
		}else{
			if(this.onCanSelect(inIndex) !== false){
				this.selectedIndex = inIndex;
				var rowNode = this.grid.getRowNode(inIndex);
				if(rowNode){
					domAttr.set(rowNode, "aria-selected", "true");
				}
				this._beginUpdate();
				this.selected[inIndex] = true;
				//this.grid.onSelected(inIndex);
				this.onSelected(inIndex);
				//this.onSetSelected(inIndex, true);
				this._endUpdate();
			}
		}
	},

	deselect: function(inIndex){
		if(this.mode == 'none'){ return; }
		if(lang.isArray(inIndex)){
			array.forEach(inIndex, this.deselect, this);
			return;
		}
		inIndex = Number(inIndex);
		if(this.selectedIndex == inIndex){
			this.selectedIndex = -1;
		}
		if(this.selected[inIndex]){
			if(this.onCanDeselect(inIndex) === false){
				return;
			}
			var rowNode = this.grid.getRowNode(inIndex);
			if(rowNode){
				domAttr.set(rowNode, "aria-selected", "false");
			}
			this._beginUpdate();
			delete this.selected[inIndex];
			//this.grid.onDeselected(inIndex);
			this.onDeselected(inIndex);
			//this.onSetSelected(inIndex, false);
			this._endUpdate();
		}
	},

	setSelected: function(inIndex, inSelect){
		this[(inSelect ? 'addToSelection' : 'deselect')](inIndex);
	},

	toggleSelect: function(inIndex){
		if(lang.isArray(inIndex)){
			array.forEach(inIndex, this.toggleSelect, this);
			return;
		}
		this.setSelected(inIndex, !this.selected[inIndex]);
	},

	_range: function(inFrom, inTo, func){
		var s = (inFrom >= 0 ? inFrom : inTo), e = inTo;
		if(s > e){
			e = s;
			s = inTo;
		}
		for(var i=s; i<=e; i++){
			func(i);
		}
	},

	selectRange: function(inFrom, inTo){
		this._range(inFrom, inTo, lang.hitch(this, "addToSelection"));
	},

	deselectRange: function(inFrom, inTo){
		this._range(inFrom, inTo, lang.hitch(this, "deselect"));
	},

	insert: function(inIndex){
		this.selected.splice(inIndex, 0, false);
		if(this.selectedIndex >= inIndex){
			this.selectedIndex++;
		}
	},

	remove: function(inIndex){
		this.selected.splice(inIndex, 1);
		if(this.selectedIndex >= inIndex){
			this.selectedIndex--;
		}
	},

	deselectAll: function(inExcept){
		for(var i in this.selected){
			if((i!=inExcept)&&(this.selected[i]===true)){
				this.deselect(i);
			}
		}
	},

	clickSelect: function(inIndex, inCtrlKey, inShiftKey){
		if(this.mode == 'none'){ return; }
		this._beginUpdate();
		if(this.mode != 'extended'){
			this.select(inIndex);
		}else{
			var lastSelected = this.selectedIndex;
			if(!inCtrlKey){
				this.deselectAll(inIndex);
			}
			if(inShiftKey){
				this.selectRange(lastSelected, inIndex);
			}else if(inCtrlKey){
				this.toggleSelect(inIndex);
			}else{
				this.addToSelection(inIndex);
			}
		}
		this._endUpdate();
	},

	clickSelectEvent: function(e){
		this.clickSelect(e.rowIndex, dojo.isCopyKey(e), e.shiftKey);
	},

	clear: function(){
		this._beginUpdate();
		this.deselectAll();
		this._endUpdate();
	}
});
});
},
'dijit/_base/scroll':function(){
define([
	"dojo/window", // windowUtils.scrollIntoView
	".."	// export symbol to dijit
], function(windowUtils, dijit){
	// module:
	//		dijit/_base/scroll
	// summary:
	//		Back compatibility module, new code should use windowUtils directly instead of using this module.

	dijit.scrollIntoView = function(/*DomNode*/ node, /*Object?*/ pos){
		// summary:
		//		Scroll the passed node into view, if it is not already.
		//		Deprecated, use `windowUtils.scrollIntoView` instead.

		windowUtils.scrollIntoView(node, pos);
	};
});

},
'url:dijit/templates/MenuBarItem.html':"<div class=\"dijitReset dijitInline dijitMenuItem dijitMenuItemLabel\" data-dojo-attach-point=\"focusNode\" role=\"menuitem\" tabIndex=\"-1\"\n\t\tdata-dojo-attach-event=\"onmouseenter:_onHover,onmouseleave:_onUnhover,ondijitclick:_onClick\">\n\t<span data-dojo-attach-point=\"containerNode\"></span>\n</div>\n",
'ibm/usmi/widgets/table/_ColumnVisible':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

define([
    "dojo/on",
	"dojo/_base/declare",
	"dijit/_WidgetBase",
	"dijit/registry",
	"dojo/_base/lang",
	"dojoe/tablex/_ColumnVisible"
], function (on, declare, _WidgetBase, registry, lang, ColumnVisible) {
	return declare([
		_WidgetBase,
		ColumnVisible
	], {
		postCreate : function () {
			this.inherited("postCreate", arguments);
			on(this.checkbox, 'change', lang.hitch(this, this.checkChanged));
		},
		setVisible : function (checked) {
			this.checkbox.setChecked(checked);
			//Defect 27350: Safari:In Table, Select All/ Deselect All for config columns popup does not work as expected for safari browser.
			//Anupama -  workaround, adding a fake event
			registry.byId(this.checkbox.id.toString()).set("disabled", true);
			registry.byId(this.checkbox.id.toString()).set("disabled", false);
		},
		checkChanged : function () {
			if (this.columnChooser) {
				this.columnChooser.updatedCheck(this);
			}
		}
	});
});
},
'ibm/tivoli/tip/TCTTable':function(){
// wrapped by build app
define("ibm/tivoli/tip/TCTTable", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/tip/navwidget/table/dijit/TableView,ibm/tivoli/tip/navwidget/table/dijit/ItemsTableView,ibm/tivoli/tip/navwidget/table/dijit/LinksTableView,ibm/tivoli/tip/navwidget/table/dijit/RelationshipsTableView,ibm/tivoli/navmodel/Dataset,ibm/tivoli/navmodel/TableWidgetDef,ibm/tivoli/navmodel/NotebookWidgetDef,ibm/tivoli/navmodel/DatasetItemsStore,ibm/tivoli/navmodel/DatasetLinksStore,ibm/tivoli/navmodel/DatasetRelationshipsStore,ibm/tivoli/navmodel/ProviderUtils,ibm/tivoli/navmodel/WidgetDefUtils,ibm/tivoli/navmodel/DatasetUtils"], function(dijit,dojo,dojox){
dojo.provide("ibm.tivoli.tip.TCTTable");
dojo.require("ibm.tivoli.tip.navwidget.table.dijit.TableView");
dojo.require("ibm.tivoli.tip.navwidget.table.dijit.ItemsTableView");
dojo.require("ibm.tivoli.tip.navwidget.table.dijit.LinksTableView");
dojo.require("ibm.tivoli.tip.navwidget.table.dijit.RelationshipsTableView");
dojo.require("ibm.tivoli.navmodel.Dataset");
dojo.require("ibm.tivoli.navmodel.TableWidgetDef");
dojo.require("ibm.tivoli.navmodel.NotebookWidgetDef");
dojo.require("ibm.tivoli.navmodel.DatasetItemsStore");
dojo.require("ibm.tivoli.navmodel.DatasetLinksStore");
dojo.require("ibm.tivoli.navmodel.DatasetRelationshipsStore");
dojo.require("ibm.tivoli.navmodel.ProviderUtils");
dojo.require("ibm.tivoli.navmodel.WidgetDefUtils");
dojo.require("ibm.tivoli.navmodel.DatasetUtils");


});

},
'ibm/usmi/menus/NextGenUIMenuUtils':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
/**
 * NextGenUIMenuUtils is a group of utility functions to use during the creation
 * of a dojo menu
 */
define("ibm/usmi/menus/NextGenUIMenuUtils", [
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dijit/_WidgetBase",
	"dijit/MenuItem",
	"idx/oneui/MenuHeading",
	"dojo/dom-construct",
	"dijit/registry",
	"dojo/_base/lang",
	"ibm/usmi/core/utils/Logger",
	"dijit/Tooltip",
	"ibm/usmi/core/utils/_UsmiBrowserPersistence",
	"dojo/on",
	"dijit/focus",
	"dojo/dom"
], function (dojo, declare, WidgetBase, MenuItem, MenuHeading, domConstruct, registry, lang, Logger, Tooltip,
		_UsmiBrowserPersistence, on, focus, dom) {
	return declare([
		WidgetBase,
		_UsmiBrowserPersistence
	], {

		_nextGenUIMenuUtilsLogger : new Logger("ibm.usmi.menus", "NextGenUIMenuUtils"),

		getFunctionName : function (
		/*String*/pFunctionAndParams) {
			var startOfArgs;
			startOfArgs = pFunctionAndParams.indexOf('(');
			return pFunctionAndParams.slice(0, startOfArgs);
		},

		/**
		 * gets the function arguments from a string in the form of
		 * 'functionName(args)'
		 */
		getFunctionArgs : function (
		/*String*/pFunctionAndParams) {
			var startOfArgs, endOfArgs, args, i, returnArgs;
			returnArgs = [];
			startOfArgs = pFunctionAndParams.indexOf('(');
			endOfArgs = pFunctionAndParams.lastIndexOf(')');
			args = (pFunctionAndParams.slice(startOfArgs + 1, endOfArgs)).split(",");
			for (i = 0; i < args.length; i++) {
				returnArgs.push(args[i].slice(1, args[i].length - 1));
			}
			return returnArgs;
		},

		/**
		 * gets the inner function and parameters from a string in the form of
		 * 'functionName(innerFunction(args))'
		 */
		getInnerFunctionAndParams : function (
		/*String*/pFunctionAndParams) {
			var startOfArgs, endOfArgs;
			startOfArgs = pFunctionAndParams.indexOf('(');
			endOfArgs = pFunctionAndParams.lastIndexOf(')');
			return pFunctionAndParams.slice(startOfArgs + 1, endOfArgs);
		},

		/**
		 * creates a menu item for the menu param pName: the title of the menu
		 * item param pColumn: the menu column to put the menu item in param
		 * pTask: the task link the menu item launches. The task link will be
		 * either in the form of function(params) or
		 * function(innerFunction(params))
		 */
		createMenuItem : function (
		/*String*/
		pName, /*String*/
		pColumn, /*String*/
		pTask, /*Resource bundle*/MRI) {
			var functionName, func, args, menuItem, innerFunctionCall, innerFunction, menuLabel;

			if (MRI[pName]) {
				menuLabel = MRI[pName];
			} else {
				menuLabel = pName;
			}
			menuItem = new MenuItem({
				"label" : menuLabel,
				"column" : pColumn,
				"class" : "mi-workitem-item"
			});

			if (pTask) {
				functionName = this.getFunctionName(pTask);
				func = window[functionName];
				args = this.getFunctionArgs(pTask);
				//check to make sure the args don't contain another function that must be executed
				if (args[0].indexOf('(') === -1) {
					menuItem.set("onClick", lang.hitch(menuItem, function () {
						if (this.tooltip) {
							this.tooltip.close();
						}
						// [ Defect 187346
						// Close the drop down menu dialog, it stays open sometimes.
						if (menuItem.getParent()) {
							if (menuItem.getParent().getParent()) {
								if (menuItem.getParent().getParent().close) {
									menuItem.getParent().getParent().close();
								}
							}
						}
						// Defect 187346]
						func.apply(null, args);
					}));
					if (functionName.length >= 15 && functionName.substr(0, 15) === 'launchInContext') {
						menuItem.set("class", "mi-workitem-item launchInContext");
						menuItem.set("label", menuLabel + "  ");
						menuItem.set("onBlur", lang.hitch(menuItem, function () {
							if (focus.curNode === menuItem.domNode) {
								//for FF/Chrome, while the MenuBarItem is hover and opened, then the focus manager
								//would set the focus to the parent menu node, not the menu item node, and in focus manager
								//if we got a blur event, normally the focus.curNode is null, here it's not null and equal
								//the menu item node, refocus it to the menu item node, and does not remove the launch in context
								//icon
								focus.focus(this.focusNode);
								return;
							}
							if (this.tooltip) {
								this.tooltip.close();
							}
							if (this.imgElement) {
								domConstruct.destroy(this.imgElement);
								this.imgElement = null;
							}
						}));
						menuItem.productName = this.getGlobalData("brandedProductKey", false);
						var tooltip = new Tooltip({
							connectId : [ menuItem.containerNode
							],
							label : lang.replace(MRI.LiCTooltip, [ menuItem.productName
							])
						});
						menuItem.set("tooltip", tooltip);
						menuItem.set("onFocus", lang.hitch(menuItem, function () {
							if (!this.imgElement) {
								var containerNode = this.get("containerNode");
								var img = dojo.doc.createElement('img');
								dojo.attr(img, {
									"src" : "js/ibm/usmi/core/framework/themes/images/ac10_opennewtab_24.png",
									"alt" : "launch in context image"
								});
								domConstruct.place(img, containerNode);
								this.set("imgElement", img);
							}

						}));
					}
				}
				//the params are another function that needs to be resolved
				else {
					innerFunctionCall = this.getInnerFunctionAndParams(pTask);
					innerFunction = window[this.getFunctionName(innerFunctionCall)];
					args = this.getFunctionArgs(innerFunctionCall);
					menuItem.set("onClick", function () {
						var itemLink = innerFunction(args[0]);
						func(itemLink);
					});
				}
			}

			return menuItem;
		},

		/**
		 * creates a menu heading for the menu param pName: the header title
		 * param pColumn: the menu column to put the header in
		 */
		createMenuHeading : function (
		/*String*/
		pName, /*String*/
		pColumn, /*Resource Bundle*/MRI) {
			var headingName = MRI[pName];
			return new MenuHeading({
				"label" : headingName,
				"column" : pColumn,
				"class" : "mi-workitem-category"
			});
		},

		/**
		 * create an image for the menu. It's created as a menu heading so that
		 * it's not hoverable or clickable param pImagePath: the path to the
		 * image param pColumn: the menu column to put the image in
		 */
		createMenuHeadingImage : function (
		/*String*/
		pImagePath, /*String*/
		pColumn) {
			var heading = new MenuHeading({
				"column" : pColumn,
				"class" : "megaMenuIcon"

			}), img = dojo.doc.createElement('img');

			dojo.attr(img, {
				"src" : pImagePath,
				"alt" : "Menu image"
			});
			domConstruct.place(img, heading.containerNode);

			return heading;
		},

		/**
		 * creats a line of links to add to the menu param pLinks: an array of
		 * link objects that contain the properties "link" and "text". The
		 * "link" property is the hyperlink, the "text" is the text that should
		 * be displayed for the link
		 */
		createLinks : function (
		/*array*/
		pLinks, /*Resource Bundle*/MRI) {
			var linksDiv, hrElement, verticalRuleElement, hrefElement, linksTitle, i;
			linksDiv = dojo.doc.createElement('div');
			dojo.attr(linksDiv, {
				"class" : "nextGenMenuLinks"
			});
			hrElement = dojo.doc.createElement('hr');
			dojo.attr(hrElement, {
				"class" : "nextGenMenuHorizontalRule"
			});
			domConstruct.place(hrElement, linksDiv);
			if (pLinks) {
				var productName = this.getGlobalData("brandedProductKey", false);
				for (i = 0; i < pLinks.length; i++) {
					if (i !== 0) {
						verticalRuleElement = dojo.doc.createElement('span');
						dojo.attr(verticalRuleElement, {
							"class" : "nextGenMenuVerticalRule"
						});
						domConstruct.place(verticalRuleElement, linksDiv);

					}
					linksTitle = pLinks[i].title;
					hrefElement = dojo.doc.createElement('span');
					hrefElement.setAttribute("class", "nextGenMenuLink");
					on(hrefElement, "click", lang.hitch(pLinks[i], function () {
						eval(this.task);
					}));

					if (MRI[linksTitle]) {
						hrefElement.innerHTML = MRI[linksTitle];
					} else {
						hrefElement.innerHTML = linksTitle;
					}
					domConstruct.place(hrefElement, linksDiv);

					if (pLinks[i].task.length >= 15 && pLinks[i].task.substr(0, 15) === 'launchInContext') {
						var img = dojo.doc.createElement('img');
						dojo.attr(img, {
							"src" : "js/ibm/usmi/core/framework/themes/images/ac10_opennewtab_24.png",
							"alt" : "launch in context image",
							"style" : "padding-left:5px; display:none;"
						});
						var tooltip = new Tooltip({
							connectId : [ hrefElement
							],
							position : [ "below"
							],
							label : lang.replace(MRI.LiCTooltip, [ productName
							])
						});
						on(hrefElement, "mouseover", function () {
							this.nextSibling.style.display = "inline";
						});
						on(hrefElement, "mouseout", function () {
							this.nextSibling.style.display = "none";
						});
						domConstruct.place(img, linksDiv);
					}
				}
			}
			return linksDiv;
		},

		/**
		 * should just be able to use dijit.byId(pId) but for some reason it is
		 * returning null. So using own function for now...
		 */
		getWidgetFromRegistry : function (
		/*String*/
		pId) {
			var widgetArray, i, wid;
			if (this._nextGenUIMenuUtilsLogger.isFinest()) {
				this._nextGenUIMenuUtilsLogger.entering("pId: " + pId, "getWidgetFromRegistry");
			}
			widgetArray = registry.toArray();

			for (i = 0; i < widgetArray.length; i++) {
				wid = widgetArray[i];
				if (wid.id === pId) {
					if (this._nextGenUIMenuUtilsLogger.isFinest()) {
						this._nextGenUIMenuUtilsLogger.exiting("widget found, returning widget",
								"getWidgetFromRegistry");
					}
					return wid;
				}
			}

			if (this._nextGenUIMenuUtilsLogger.isFinest()) {
				this._nextGenUIMenuUtilsLogger.exiting("widget not found, returning null", "getWidgetFromRegistry");
			}
			return null;
		},

		constructor : function (args) {
			dojo.safeMixin(this, args);
		}
	});

});

},
'dojox/grid/enhanced/plugins/NestedSorting':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/connect",
	"dojo/_base/lang",
	"dojo/_base/html",
	"dojo/_base/event",
	"dojo/_base/window",
	"dojo/keys",
	"dojo/query",
	"dojo/string",
	"../_Plugin",
	"../../EnhancedGrid"
], function(declare, array, connect, lang, html, evt, win, keys, query, string, _Plugin, EnhancedGrid){

var NestedSorting = declare("dojox.grid.enhanced.plugins.NestedSorting", _Plugin, {
	// summary:
	//		Provides nested sorting feature
	//
	// description:
	//		A flexible way to control multiple column sorting, including
	//		1. Set default sorting order
	//		2. Disable sorting for certain columns
	//		3. Set sorting order dynamically with JS API
	//
	// example:
	// |	<script type="text/javascript">
	// |		var grid = new dojox.grid.EnhancedGrid({plugins : {nestedSorting: true}},
	// |	               sortFields: [{attribute: 'col4', descending: false},...],//set default sorting order
	// |			       canSort: function(index, field){ return true},//disable sorting for a column
	// |				   ... }, dojo.byId('gridDiv'));
	// |		grid.startup();
	// |		//set new sorting order
	// |		grid.setSortIndex([{attribute: 'col3', descending: true},...])
	// |	</script>
	
	// name: String
	//		Plugin name
	name: "nestedSorting",
	
	_currMainSort: 'none',//'none'|'asc'|'desc'
	
	_currRegionIdx: -1,
	
	_a11yText: {
		'dojoxGridDescending'   : '&#9662;',
		'dojoxGridAscending'    : '&#9652;',
		'dojoxGridAscendingTip' : '&#1784;',
		'dojoxGridDescendingTip': '&#1783;',
		'dojoxGridUnsortedTip'  : 'x' //'&#10006;'
	},
	
	constructor: function(){
		this._sortDef = [];
		this._sortData = {};
		this._headerNodes = {};
		//column index that are hidden, un-sortable or indirect selection etc.
		this._excludedColIdx = [];
		this.nls = this.grid._nls;
		this.grid.setSortInfo = function(){};
		this.grid.setSortIndex = lang.hitch(this, '_setGridSortIndex');
		this.grid.getSortIndex = function(){};
		this.grid.getSortProps = lang.hitch(this, 'getSortProps');
		if(this.grid.sortFields){
			this._setGridSortIndex(this.grid.sortFields, null, true);
		}
		this.connect(this.grid.views, 'render', '_initSort');//including column resize
		this.initCookieHandler();
		this.subscribe("dojox/grid/rearrange/move/" + this.grid.id, lang.hitch(this, '_onColumnDnD'));
	},
	onStartUp: function(){
		//overwrite base Grid functions
		this.inherited(arguments);
		this.connect(this.grid, 'onHeaderCellClick', '_onHeaderCellClick');
		this.connect(this.grid, 'onHeaderCellMouseOver', '_onHeaderCellMouseOver');
		this.connect(this.grid, 'onHeaderCellMouseOut', '_onHeaderCellMouseOut');
	},
	_onColumnDnD: function(type, mapping){
		// summary:
		//		Update nested sorting after column moved		
		if(type !== 'col'){return;}
		var m = mapping, obj = {}, d = this._sortData, p;
		var cr = this._getCurrentRegion();
		this._blurRegion(cr);
		var idx = this._getRegionHeader(cr).getAttribute('idx');
		for(p in m){
			if(d[p]){
				obj[m[p]] = d[p];
				delete d[p];
			}
			if(p === idx){
				idx = m[p];
			}
		}
		for(p in obj){
			d[p] = obj[p];
		}
		var c = this._headerNodes[idx];
		this._currRegionIdx = array.indexOf(this._getRegions(), c.firstChild);
		this._initSort(false);
	},
	_setGridSortIndex: function(inIndex, inAsc, noRefresh){
		if(lang.isArray(inIndex)){
			var i, d, cell;
			for(i = 0; i < inIndex.length; i++){
				d = inIndex[i];
				cell = this.grid.getCellByField(d.attribute);
				if(!cell){
					console.warn('Invalid sorting option, column ', d.attribute, ' not found.');
					return;
				}
				if(cell['nosort'] || !this.grid.canSort(cell.index, cell.field)){
					console.warn('Invalid sorting option, column ', d.attribute, ' is unsortable.');
					return;
				}
			}
			this.clearSort();
			array.forEach(inIndex, function(d, i){
				cell = this.grid.getCellByField(d.attribute);
				this.setSortData(cell.index, 'index', i);
				this.setSortData(cell.index, 'order', d.descending ? 'desc': 'asc');
			}, this);
		}else if(!isNaN(inIndex)){
			if(inAsc === undefined){ return; }//header click from base DataGrid
			this.setSortData(inIndex, 'order', inAsc ? 'asc' : 'desc');
		}else{
			return;
		}
		this._updateSortDef();
		if(!noRefresh){
			this.grid.sort();
		}
	},
	getSortProps: function(){
		// summary:
		//		Overwritten, see DataGrid.getSortProps()
		return this._sortDef.length ? this._sortDef : null;
	},
	_initSort: function(postSort){
		// summary:
		//		Initiate sorting
		var g = this.grid, n = g.domNode, len = this._sortDef.length;
		html.toggleClass(n, 'dojoxGridSorted', !!len);
		html.toggleClass(n, 'dojoxGridSingleSorted', len === 1);
		html.toggleClass(n, 'dojoxGridNestSorted', len > 1);
		if(len > 0){
			this._currMainSort = this._sortDef[0].descending ? 'desc' : 'asc';
		}
		var idx, excluded = this._excludedCoIdx = [];//reset it
		//cache column index of hidden, un-sortable or indirect selection
		this._headerNodes = query("th", g.viewsHeaderNode).forEach(function(n){
			idx = parseInt(n.getAttribute('idx'), 10);
			if(html.style(n, 'display') === 'none' || g.layout.cells[idx]['nosort'] || (g.canSort && !g.canSort(idx, g.layout.cells[idx]['field']))){
				excluded.push(idx);
			}
		});
		this._headerNodes.forEach(this._initHeaderNode, this);
		this._initFocus();
		if(postSort){
			this._focusHeader();
		}
	},
	_initHeaderNode: function(node){
		// summary:
		//		Initiate sort for each header cell node
		html.toggleClass(node, 'dojoxGridSortNoWrap', true);
		var sortNode = query('.dojoxGridSortNode', node)[0];
		if(sortNode){
			html.toggleClass(sortNode, 'dojoxGridSortNoWrap', true);
		}
		if(array.indexOf(this._excludedCoIdx, node.getAttribute('idx')) >= 0){
			html.addClass(node, 'dojoxGridNoSort');
			return;
		}
		if(!query('.dojoxGridSortBtn', node).length){
			//clear any previous connects
			this._connects = array.filter(this._connects, function(conn){
				if(conn._sort){
					connect.disconnect(conn);
					return false;
				}
				return true;
			});
			var n = html.create('a', {
				className: 'dojoxGridSortBtn dojoxGridSortBtnNested',
				title: string.substitute(this.nls.sortingState, [this.nls.nestedSort, this.nls.ascending]),
				innerHTML: '1'
			}, node.firstChild, 'last');
			n.onmousedown = evt.stop;
			n = html.create('a', {
				className: 'dojoxGridSortBtn dojoxGridSortBtnSingle',
				title: string.substitute(this.nls.sortingState, [this.nls.singleSort, this.nls.ascending])
			}, node.firstChild, 'last');
			n.onmousedown = evt.stop;
		}else{
			//deal with small height grid which doesn't re-render the grid after refresh
			var a1 = query('.dojoxGridSortBtnSingle', node)[0];
			var a2 = query('.dojoxGridSortBtnNested', node)[0];
			a1.className = 'dojoxGridSortBtn dojoxGridSortBtnSingle';
			a2.className = 'dojoxGridSortBtn dojoxGridSortBtnNested';
			a2.innerHTML = '1';
			html.removeClass(node, 'dojoxGridCellShowIndex');
			html.removeClass(node.firstChild, 'dojoxGridSortNodeSorted');
			html.removeClass(node.firstChild, 'dojoxGridSortNodeAsc');
			html.removeClass(node.firstChild, 'dojoxGridSortNodeDesc');
			html.removeClass(node.firstChild, 'dojoxGridSortNodeMain');
			html.removeClass(node.firstChild, 'dojoxGridSortNodeSub');
		}
		this._updateHeaderNodeUI(node);
	},
	_onHeaderCellClick: function(e){
		// summary
		//		See dojox.grid.enhanced._Events._onHeaderCellClick()
		this._focusRegion(e.target);
		if(html.hasClass(e.target, 'dojoxGridSortBtn')){
			this._onSortBtnClick(e);
			evt.stop(e);
			this._focusRegion(this._getCurrentRegion());
		}
	},
	_onHeaderCellMouseOver: function(e){
		// summary
		//		See dojox.grid._Events._onHeaderCellMouseOver()
		//		When user mouseover other columns than sorted column in a single sorted grid,
		//		We need to show 1 in the sorted column
		if(!e.cell){return; }
		if(this._sortDef.length > 1){ return; }
		if(this._sortData[e.cellIndex] && this._sortData[e.cellIndex].index === 0){ return; }
		var p;
		for(p in this._sortData){
			if(this._sortData[p] && this._sortData[p].index === 0){
				html.addClass(this._headerNodes[p], 'dojoxGridCellShowIndex');
				break;
			}
		}
		if(!html.hasClass(win.body(), 'dijit_a11y')){ return; }
		//a11y support
		var i = e.cell.index, node = e.cellNode;
		var singleSortBtn = query('.dojoxGridSortBtnSingle', node)[0];
		var nestedSortBtn = query('.dojoxGridSortBtnNested', node)[0];
		
		var sortMode = 'none';
		if(html.hasClass(this.grid.domNode, 'dojoxGridSingleSorted')){
			sortMode = 'single';
		}else if(html.hasClass(this.grid.domNode, 'dojoxGridNestSorted')){
			sortMode = 'nested';
		}
		var nestedIndex = nestedSortBtn.getAttribute('orderIndex');
		if(nestedIndex === null || nestedIndex === undefined){
			nestedSortBtn.setAttribute('orderIndex', nestedSortBtn.innerHTML);
			nestedIndex = nestedSortBtn.innerHTML;
		}
		if(this.isAsc(i)){
			nestedSortBtn.innerHTML = nestedIndex + this._a11yText.dojoxGridDescendingTip;
		}else if(this.isDesc(i)){
			nestedSortBtn.innerHTML = nestedIndex + this._a11yText.dojoxGridUnsortedTip;
		}else{
			nestedSortBtn.innerHTML = nestedIndex + this._a11yText.dojoxGridAscendingTip;
		}
		if(this._currMainSort === 'none'){
			singleSortBtn.innerHTML = this._a11yText.dojoxGridAscendingTip;
		}else if(this._currMainSort === 'asc'){
			singleSortBtn.innerHTML = this._a11yText.dojoxGridDescendingTip;
		}else if(this._currMainSort === 'desc'){
			singleSortBtn.innerHTML = this._a11yText.dojoxGridUnsortedTip;
		}
		
		this._updateA11y(node);
	},
	_onHeaderCellMouseOut: function(e){
		// summary
		//		See dojox.grid.enhanced._Events._onHeaderCellMouseOut()
		var p;
		for(p in this._sortData){
			if(this._sortData[p] && this._sortData[p].index === 0){
				html.removeClass(this._headerNodes[p], 'dojoxGridCellShowIndex');
				break;
			}
		}
		if(e.cellNode){this._updateA11y(e.cellNode);}
	},
	_onSortBtnClick: function(e){
		// summary:
		//		If the click target is single sort button, do single sort.
		//		Else if the click target is nested sort button, do nest sort.
		//		Otherwise return.
		var cellIdx = e.cell.index;
		if(html.hasClass(e.target, 'dojoxGridSortBtnSingle')){
			this._prepareSingleSort(cellIdx);
		}else if(html.hasClass(e.target, 'dojoxGridSortBtnNested')){
			this._prepareNestedSort(cellIdx);
		}else{
			return;
		}
		evt.stop(e);
		this._doSort(cellIdx);
	},
	_doSort: function(cellIdx){
		if(!this._sortData[cellIdx] || !this._sortData[cellIdx].order){
			this.setSortData(cellIdx, 'order', 'asc');	//no sorting data
		}else if(this.isAsc(cellIdx)){
			this.setSortData(cellIdx, 'order', 'desc');	//change to 'desc'
		}else if(this.isDesc(cellIdx)){
			this.removeSortData(cellIdx); //remove from sorting sequence
		}
		this._updateSortDef();
		this.grid.sort();
		this._initSort(true);
	},
	setSortData: function(cellIdx, attr, value){
		// summary:
		//		Set sorting data for a column.
		var sd = this._sortData[cellIdx];
		if(!sd){
			sd = this._sortData[cellIdx] = {};
		}
		sd[attr] = value;
	},
	removeSortData: function(cellIdx){
		var d = this._sortData, i = d[cellIdx].index, p;
		delete d[cellIdx];
		for(p in d){
			if(d[p].index > i){
				d[p].index--;
			}
		}
	},
	_prepareSingleSort: function(cellIdx){
		// summary:
		//		Prepare the single sort, also called main sort, this will clear any existing sorting and just sort the grid by current column.
		var d = this._sortData, p;
		for(p in d){
			delete d[p];
		}
		this.setSortData(cellIdx, 'index', 0);
		this.setSortData(cellIdx, 'order', this._currMainSort === 'none' ? null : this._currMainSort);
		if(!this._sortData[cellIdx] || !this._sortData[cellIdx].order){
			this._currMainSort = 'asc';
		}else if(this.isAsc(cellIdx)){
			this._currMainSort = 'desc';
		}else if(this.isDesc(cellIdx)){
			this._currMainSort = 'none';
		}
	},
	_prepareNestedSort: function(cellIdx){
		// summary
		//		Prepare the nested sorting, this will order the column on existing sorting result.
		var i = this._sortData[cellIdx] ? this._sortData[cellIdx].index : null;
		if(i === 0 || !!i){ return; }
		this.setSortData(cellIdx, 'index', this._sortDef.length);
	},
	_updateSortDef: function(){
		this._sortDef.length = 0;
		var d = this._sortData, p;
		for(p in d){
			this._sortDef[d[p].index] = {
				attribute: this.grid.layout.cells[p].field,
				descending: d[p].order === 'desc'
			};
		}
	},
	_updateHeaderNodeUI: function(node){
		// summary:
		//		Update the column header UI based on current sorting state.
		//		Show indicator of the sorting order of the column, no order no indicator
		var cell = this._getCellByNode(node);
		var cellIdx = cell.index;
		var data = this._sortData[cellIdx];
		var sortNode = query('.dojoxGridSortNode', node)[0];
		var singleSortBtn = query('.dojoxGridSortBtnSingle', node)[0];
		var nestedSortBtn = query('.dojoxGridSortBtnNested', node)[0];
		
		html.toggleClass(singleSortBtn, 'dojoxGridSortBtnAsc', this._currMainSort === 'asc');
		html.toggleClass(singleSortBtn, 'dojoxGridSortBtnDesc', this._currMainSort === 'desc');
		if(this._currMainSort === 'asc'){
			singleSortBtn.title = string.substitute(this.nls.sortingState, [this.nls.singleSort, this.nls.descending]);
		}else if(this._currMainSort === 'desc'){
			singleSortBtn.title = string.substitute(this.nls.sortingState, [this.nls.singleSort, this.nls.unsorted]);
		}else{
			singleSortBtn.title = string.substitute(this.nls.sortingState, [this.nls.singleSort, this.nls.ascending]);
		}
		
		var _this = this;
		function setWaiState(){
			var columnInfo = 'Column ' + (cell.index + 1) + ' ' + cell.field;
			var orderState = 'none';
			var orderAction = 'ascending';
			if(data){
				orderState = data.order === 'asc' ? 'ascending' : 'descending';
				orderAction = data.order === 'asc' ? 'descending' : 'none';
			}
			var a11ySingleLabel = columnInfo + ' - is sorted by ' + orderState;
			var a11yNestedLabel = columnInfo + ' - is nested sorted by ' + orderState;
			var a11ySingleLabelHover = columnInfo + ' - choose to sort by ' + orderAction;
			var a11yNestedLabelHover = columnInfo + ' - choose to nested sort by ' + orderAction;
			
			singleSortBtn.setAttribute("aria-label", a11ySingleLabel);
			nestedSortBtn.setAttribute("aria-label", a11yNestedLabel);
			
			var handles = [
				_this.connect(singleSortBtn, "onmouseover", function(){
					singleSortBtn.setAttribute("aria-label", a11ySingleLabelHover);
				}),
				_this.connect(singleSortBtn, "onmouseout", function(){
					singleSortBtn.setAttribute("aria-label", a11ySingleLabel);
				}),
				_this.connect(nestedSortBtn, "onmouseover", function(){
					nestedSortBtn.setAttribute("aria-label", a11yNestedLabelHover);
				}),
				_this.connect(nestedSortBtn, "onmouseout", function(){
					nestedSortBtn.setAttribute("aria-label", a11yNestedLabel);
				})
			];
			array.forEach(handles, function(handle){ handle._sort = true; });
		}
		setWaiState();

		var a11y = html.hasClass(win.body(), "dijit_a11y");
		if(!data){
			nestedSortBtn.innerHTML = this._sortDef.length + 1;
			nestedSortBtn.title = string.substitute(this.nls.sortingState, [this.nls.nestedSort, this.nls.ascending]);
		}else{
			if(data.index || (data.index === 0 && this._sortDef.length > 1)){
				nestedSortBtn.innerHTML = data.index + 1;
			}
			html.addClass(sortNode, 'dojoxGridSortNodeSorted');
			if(this.isAsc(cellIdx)){
				html.addClass(sortNode, 'dojoxGridSortNodeAsc');
				nestedSortBtn.title = string.substitute(this.nls.sortingState, [this.nls.nestedSort, this.nls.descending]);
			}else if(this.isDesc(cellIdx)){
				html.addClass(sortNode, 'dojoxGridSortNodeDesc');
				nestedSortBtn.title = string.substitute(this.nls.sortingState, [this.nls.nestedSort, this.nls.unsorted]);
			}
			html.addClass(sortNode, (data.index === 0 ? 'dojoxGridSortNodeMain' : 'dojoxGridSortNodeSub'));
		}
		this._updateA11y(node);
	},
	isAsc: function(cellIndex){
		return this._sortData[cellIndex] && this._sortData[cellIndex].order === 'asc';
	},
	isDesc: function(cellIndex){
		return this._sortData[cellIndex] && this._sortData[cellIndex].order === 'desc';
	},
	
	_updateA11y: function(node){
		if(!html.hasClass(win.body(), "dijit_a11y")){return;}
		var cell = this._getCellByNode(node), cellIdx = cell.index, data = this._sortData[cellIdx];
		//Unsortable columns do not have these nodes.
		if(cell.nosort){return;}
		var nls = this.nls, txt = this._a11yText;
		var sortNode = query('.dojoxGridSortNode', node)[0];
		var singleSortBtn = query('.dojoxGridSortBtnSingle', node)[0];
		var nestedSortBtn = query('.dojoxGridSortBtnNested', node)[0];
		var isActive = html.hasClass(node, 'dojoxGridCellOver') || html.hasClass(node, 'dojoxGridCellSortFocus');
		var idx = (data ? data.index : this._sortDef.length) + 1;
		var s1, s2;
		
		if(this.isAsc(cellIdx)){
			s2 = idx + '&nbsp;' + (isActive ? txt.dojoxGridDescendingTip : txt.dojoxGridAscending);
		}else if(this.isDesc(cellIdx)){
			s2 = idx + '&nbsp;' + (isActive ? txt.dojoxGridUnsortedTip : txt.dojoxGridDescending);
		}else{
			s2 = idx + '&nbsp;' + (isActive ? txt.dojoxGridAscendingTip : '');
		}
		
		if(this._currMainSort === 'asc'){
			s1 = isActive ? txt.dojoxGridDescendingTip : txt.dojoxGridAscending;
		}else if(this._currMainSort === 'desc'){
			s1 = isActive ? txt.dojoxGridUnsortedTip : txt.dojoxGridDescending;
		}else{
			s1 = isActive ? txt.dojoxGridAscendingTip : '';
		}
		
		singleSortBtn.innerHTML = s1;
		nestedSortBtn.innerHTML = s2;
	},
	
	_getCellByNode: function(node){
		var i;
		for(i = 0; i < this._headerNodes.length; i++){
			if(this._headerNodes[i] === node){
				return this.grid.layout.cells[i];
			}
		}
		return null;
	},
	clearSort: function(){
		this._sortData = {};
		this._sortDef.length = 0;
	},
	
	//persistence
	initCookieHandler: function(){
		if(this.grid.addCookieHandler){
			this.grid.addCookieHandler({
				name: "sortOrder",
				onLoad: lang.hitch(this, '_loadNestedSortingProps'),
				onSave: lang.hitch(this, '_saveNestedSortingProps')
			});
		}
	},
	_loadNestedSortingProps: function(sortInfo, grid){
		this._setGridSortIndex(sortInfo, null, true);
	},
	_saveNestedSortingProps: function(grid){
		return this.getSortProps();
	},
	
	//focus & keyboard
	_initFocus: function(){
		var f = this.focus = this.grid.focus;
		this._focusRegions = this._getRegions();
		if(!this._headerArea){
			var area = this._headerArea = f.getArea('header');
			area.onFocus = f.focusHeader = lang.hitch(this, '_focusHeader');
			area.onBlur = f.blurHeader = f._blurHeader = lang.hitch(this, '_blurHeader');
			area.onMove = lang.hitch(this, '_onMove');
			area.onKeyDown = lang.hitch(this, '_onKeyDown');
			area._regions = [];
			area.getRegions = null;
			this.connect(this.grid, 'onBlur', '_blurHeader');
		}
	},
	_focusHeader: function(e){
		// summary:
		//		Overwritten, see _FocusManager.focusHeader()
		//delayed: Boolean
		//		If called from "this.focus._delayedHeaderFocus()"
		if(this._currRegionIdx === -1){
			this._onMove(0, 1, null);
		}else{
			this._focusRegion(this._getCurrentRegion());
		}
		try{
			evt.stop(e);
		}catch(e){}
		return true;
	},
	_blurHeader: function(e){
		this._blurRegion(this._getCurrentRegion());
		return true;
	},
	_onMove: function(rowStep, colStep, e){
		var curr = this._currRegionIdx || 0, regions = this._focusRegions;
		var region = regions[curr + colStep];
		if(!region){
			return;
		}else if(html.style(region, 'display') === 'none' || html.style(region, 'visibility') === 'hidden'){
			//if the region is invisible, keep finding next
			this._onMove(rowStep, colStep + (colStep > 0 ? 1 : -1), e);
			return;
		}
		this._focusRegion(region);
		//keep grid body scrolled by header
		var view = this._getRegionView(region);
		view.scrollboxNode.scrollLeft = view.headerNode.scrollLeft;
	},
	_onKeyDown: function(e, isBubble){
		if(isBubble){
			switch(e.keyCode){
				case keys.ENTER:
				case keys.SPACE:
					if(html.hasClass(e.target, 'dojoxGridSortBtnSingle') ||
						html.hasClass(e.target, 'dojoxGridSortBtnNested')){
						this._onSortBtnClick(e);
					}
			}
		}
	},
	_getRegionView: function(region){
		var header = region;
		while(header && !html.hasClass(header, 'dojoxGridHeader')){ header = header.parentNode; }
		if(header){
			return array.filter(this.grid.views.views, function(view){
				return view.headerNode === header;
			})[0] || null;
		}
		return null;
	},
	_getRegions: function(){
		var regions = [], cells = this.grid.layout.cells;
		this._headerNodes.forEach(function(n, i){
			if(html.style(n, 'display') === 'none'){return;}
			if(cells[i]['isRowSelector']){
				regions.push(n);
				return;
			}
			query('.dojoxGridSortNode,.dojoxGridSortBtnNested,.dojoxGridSortBtnSingle', n).forEach(function(node){
					node.setAttribute('tabindex', 0);
					regions.push(node);
			});
		},this);
		return regions;
	},
	_focusRegion: function(region){
		// summary
		//		Focus the given region
		if(!region){return;}
		var currRegion = this._getCurrentRegion();
		if(currRegion && region !== currRegion){
			this._blurRegion(currRegion);
		}
		var header = this._getRegionHeader(region);
		html.addClass(header, 'dojoxGridCellSortFocus');
		if(html.hasClass(region, 'dojoxGridSortNode')){
			html.addClass(region, 'dojoxGridSortNodeFocus');
		}else if(html.hasClass(region, 'dojoxGridSortBtn')){
			html.addClass(region, 'dojoxGridSortBtnFocus');
		}
		region.focus();
		this.focus.currentArea('header');
		this._currRegionIdx = array.indexOf(this._focusRegions, region);
	},
	_blurRegion: function(region){
		if(!region){return;}
		var header = this._getRegionHeader(region);
		html.removeClass(header, 'dojoxGridCellSortFocus');
		if(html.hasClass(region, 'dojoxGridSortNode')){
			html.removeClass(region, 'dojoxGridSortNodeFocus');
		}else if(html.hasClass(region, 'dojoxGridSortBtn')){
			html.removeClass(region, 'dojoxGridSortBtnFocus');
		}
		region.blur();
		this._updateA11y(header);
	},
	_getCurrentRegion: function(){
		return this._focusRegions ? this._focusRegions[this._currRegionIdx] : null;
	},
	_getRegionHeader: function(region){
		while(region && !html.hasClass(region, 'dojoxGridCell')){
			region = region.parentNode;
		}
		return region;
	},
	destroy: function(){
		this._sortDef = this._sortData = null;
		this._headerNodes = this._focusRegions = null;
		this.inherited(arguments);
	}
});

EnhancedGrid.registerPlugin(NestedSorting);

return NestedSorting;
});

},
'dojoe/tablex/_ConfigureColumnsDialog':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/i18n!dojoe/tablex/nls/Table","dojo/require!dijit/_Widget,dijit/_TemplatedMixin,dijit/Dialog,dijit/Toolbar,dijit/form/Form,dijit/form/Button,dijit/form/CheckBox,dijit/layout/TabContainer,dijit/layout/LayoutContainer,dijit/layout/ContentPane,dijit/_Container,dijit/_WidgetsInTemplateMixin,dojoe/tablex/_TableOptions"], function(dijit,dojo,dojox){
dojo.provide("dojoe.tablex._ConfigureColumnsDialog");

dojo.require("dijit._Widget");
dojo.require("dijit._TemplatedMixin");
dojo.require("dijit.Dialog");
dojo.require("dijit.Toolbar");
dojo.require("dijit.form.Form");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.CheckBox");
dojo.require("dijit.layout.TabContainer");
dojo.require("dijit.layout.LayoutContainer");
dojo.require("dijit.layout.ContentPane");
dojo.require("dijit._Container");
dojo.require("dijit._WidgetsInTemplateMixin");

dojo.require("dojoe.tablex._TableOptions");

// NLS
dojo.requireLocalization("dojoe.tablex", "Table");
// define the widget class
dojo.declare ("dojoe.tablex._ConfigureColumnsDialog", [ dijit._Widget, dijit._TemplatedMixin, dijit._WidgetsInTemplateMixin ],{

		// summary:
		//		Private class - Configure Columns Dialog
		
		widgetsInTemplate: true,
		// Path to the template
		templateString : dojo.cache("dojoe.tablex", "templates/ConfigureColumnsDialog.html", "\r\n<div>\r\n  <div dojoType=\"dijit.Dialog\" dojoAttachPoint=\"dialog\"\ttitle=\"${resources_.CONFIGURE_SETTINGS}\" tabindex=0 role=\"dialog\" style=\"width:350px;height:auto;overflow:auto;\" >\r\n\t\t<table class=\"dijitDialogPaneContentArea\" role=\"presentation\">\r\n\t\t\t<tr><td>\r\n\t\t\t\t<div  style=\"width:100%;height:100%;overflow:auto;\" class=\"tivTable\">\r\n\t\t\t\t\t<label>${resources_.COLUMNS_VISIBLE}</label>\t\t\r\n\t\t\t\t\t \t\r\n\t                  <table border=\"0\" cellspacing=\"7\" cellpadding=\"0\" role=\"presentation\">\r\n\t\t\t\t\t\t<tbody>\r\n\t\t\t\t\t\t\t<tr>\t\t\t\t\t\t\r\n\t\t\t\t\t\t\t\t<td>\r\n\t\t\t\t\t\t\t\t\t<span dojoAttachPoint=\"selectAllButton\" role=\"button\">\r\n\t\t\t\t\t\t\t\t</td>\r\n\t\t\t\t\t\t\t\t<td>\r\n\t\t\t\t\t\t\t\t\t<span dojoAttachPoint=\"deselectAllButton\" role=\"button\">\r\n\t\t\t\t\t\t\t\t</td>\r\n\t\t\t\t\t\t\t</tr>\r\n\t\t\t\t\t\t</tbody>\r\n\t\t\t\t\t</table>\r\n\t\t\t\t</div>\r\n\t\t\t</td></tr>\r\n\t\t\t<tr><td>\t\t\r\n\t\t\t\t<div tabStrip=\"false\" id=\"${id}tabs\" dojoType=\"dijit.layout.LayoutContainer\"  class=\"dialog\" style=\"vertical-align:top;width:315px;height:110px;overflow-x:hidden;overflow-y:auto;padding:2px 0px 0px 10px;\">\r\n\t\t\t\t\t<div id=\"${id}columnsVisibleDivContainer\"></div>\r\n\t\t\t\t</div>\r\n\t\t\t</td></tr>\r\n\t\t</table>\t\r\n\t\t<div class=\"dijitDialogPaneActionBar\">\t\t\r\n\t\t\t<button value=\"${resources_.OK}\" dojoType=\"dijit.form.Button\" role=\"button\"\r\n\t\t\t\tdojoAttachEvent=\"onClick:onOK,onKeypress:onOK\">${resources_.OK}</button>&nbsp;\r\n\t\t\t<button value=\"${resources_.CANCEL}\" dojoType=\"dijit.form.Button\" role=\"button\"\r\n\t\t\tdojoAttachPoint=\"cancelButton\" dojoAttachEvent=\"onClick:onCancel,onKeypress:onCancel\">${resources_.CANCEL}</button>\t\r\n\t\t</div>\r\n  </div>\r\n</div>\r\n"),		
		optionsWidget_: null,
		_structure: null,
		constructor: function() {
			this.resources_ = dojo.i18n.getLocalization("dojoe.tablex", "Table", this.lang);
		},		
		postCreate: function() { 
			this.inherited ("postCreate", arguments);
			this.optionsWidget_=new dojoe.tablex._TableOptions({
				grid: this.grid, _structure: this._structure
			}, dojo.byId(this.id + "columnsVisibleDivContainer"));
			this.optionsWidget_.startup();	

			var selectIcon = new dijit.form.Button({
				id: this.id+"selectAllIcon",
				showLabel: false,
				iconClass: "selectAllIcon",
				label: this.resources_.SELECT_ALL
			},this.selectAllButton);
				
			selectIcon.onClick = dojo.hitch (this, "onSelectAll");
			selectIcon.onKeypress = dojo.hitch (this, "onSelectAll");		
			
			var deSelectIcon = new dijit.form.Button({
				id: this.id+"deselectAllIcon",
				showLabel: false,
				iconClass: "deselectAllIcon",
				label: this.resources_.DESELECT_ALL
			},this.deselectAllButton);
				
			deSelectIcon.onClick = dojo.hitch (this, "onDeselectAll");
			deSelectIcon.onKeypress = dojo.hitch (this, "onDeselectAll");
						
		},		
		onSelectAll: function(){		
			this.optionsWidget_.onSelectAll();
		},		
		onDeselectAll: function(){		
			this.optionsWidget_.onDeselectAll();
		},		
		onOK: function () {
		/*Ls - 17/01/2012 - 30956 - showing cursor busy when saving the column configuration*/
			window.document.body.style.cursor = 'wait';
			this.optionsWidget_.onSave();
			this.hide();
			window.document.body.style.cursor = 'default';
		},		
		onCancel: function () {
			this.hide();
		},		
		show: function () {
			this.dialog.show();
			dijit.byId (this.id + "tabs").resize();
		},		
		hide: function () {
			this.dialog.hide();
			setTimeout (dojo.hitch (this, this.kill), 100);
		},		
		kill: function () {
			var id = this.id;
			this.destroyRecursive();
			dijit.registry.remove (id);
		},		
		getTableOptions: function () {
			return this.optionsWidget_;
		},
		destroy: function () {
			//memory leak issue: custom destroy method to take care of removing all widgets					
			try{
				delete this.resources_;
				//this is for destroying child templated widgets 
				//this should happen by itself but need to figure out why dijit._Templated is not taking care of it...
				var widgets = dijit.findWidgets(this.domNode);
				dojo.forEach(widgets, function(w) {
					if(w){
						if(w.destroyRecursive){
							w.destroyRecursive();
						} else if(w.destroy){
							w.destroy();
						}
					}
				});
				// this is for widgets added as child to this widget and their children..
				//not present in template
				var widgetArr = this.getChildren();
				dojo.forEach(widgetArr, function (child) {
					if(child){
						if(child.destroyRecursive){
							child.destroyRecursive();
						} else if(child.destroy){
							child.destroy();
						}
					}
				});
				this.inherited(arguments);
			}catch(ex){
				console.warn("Widget destroy failed: "+this.id+", ex="+ex.name+":"+ex.message);
			}
		}
	}
);
});

},
'dijit/CheckedMenuItem':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.toggle
	"./MenuItem",
	"dojo/text!./templates/CheckedMenuItem.html",
	"./hccss"
], function(declare, domClass, MenuItem, template){

/*=====
	var MenuItem = dijit.MenuItem;
=====*/

	// module:
	//		dijit/CheckedMenuItem
	// summary:
	//		A checkbox-like menu item for toggling on and off

	return declare("dijit.CheckedMenuItem", MenuItem, {
		// summary:
		//		A checkbox-like menu item for toggling on and off

		templateString: template,

		// checked: Boolean
		//		Our checked state
		checked: false,
		_setCheckedAttr: function(/*Boolean*/ checked){
			// summary:
			//		Hook so attr('checked', bool) works.
			//		Sets the class and state for the check box.
			domClass.toggle(this.domNode, "dijitCheckedMenuItemChecked", checked);
			this.domNode.setAttribute("aria-checked", checked);
			this._set("checked", checked);
		},

		iconClass: "",	// override dijitNoIcon

		onChange: function(/*Boolean*/ /*===== checked =====*/){
			// summary:
			//		User defined function to handle check/uncheck events
			// tags:
			//		callback
		},

		_onClick: function(/*Event*/ e){
			// summary:
			//		Clicking this item just toggles its state
			// tags:
			//		private
			if(!this.disabled){
				this.set("checked", !this.checked);
				this.onChange(this.checked);
			}
			this.inherited(arguments);
		}
	});
});

},
'ibm/tivoli/tip/navwidget/table/dijit/AbstractTableView':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/i18n!ibm/tivoli/tip/dijit/nls/resources","dojo/require!dijit/layout/BorderContainer,dijit/ToolbarSeparator,dijit/form/Button,dijit/Menu,dijit/MenuItem,dijit/PopupMenuItem,dojox/collections/Dictionary,dojox/grid/DataGrid,ibm/tivoli/tip/dijit/TIPContextMenu,ibm/tivoli/tip/navwidget/dijit/NavWidgetObjectPluggableView,ibm/tivoli/tip/navwidget/dijit/AsynchContextMenu,ibm/tivoli/tip/navwidget/dijit/RESTContextMenuProvider,ibm/tivoli/tip/navwidget/dojo/NavWidgetUtil,ibm/tivoli/tip/navwidget/table/dijit/NavModelTableSelection,ibm/tivoli/tip/navwidget/dojo/ToolbarUtil,ibm/tivoli/tip/navwidget/properties/PropertyRenderer,ibm/tivoli/tip/navwidget/properties/dijit/PropertiesNotebookDialog,ibm/tivoli/tip/navwidget/topology/dijit/MenuProviderHelper,ibm/tivoli/tip/navwidget/topology/dijit/TopologyLabel,ibm/tivoli/tip/navwidget/topology/dijit/DefaultActionHelper,ibm/tivoli/navmodel/TableWidgetDef,ibm/tivoli/navmodel/Exception,ibm/tivoli/navmodel/Trace,dojoe/tablex/TableAll"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide ("ibm.tivoli.tip.navwidget.table.dijit.AbstractTableView");

dojo.require("dijit.layout.BorderContainer");
dojo.require("dijit.ToolbarSeparator");
dojo.require("dijit.form.Button");
dojo.require("dijit.Menu");
dojo.require("dijit.MenuItem");
dojo.require("dijit.PopupMenuItem");
dojo.require("dojox.collections.Dictionary");
dojo.require("dojox.grid.DataGrid");

dojo.require("ibm.tivoli.tip.dijit.TIPContextMenu");
dojo.require("ibm.tivoli.tip.navwidget.dijit.NavWidgetObjectPluggableView");
dojo.require("ibm.tivoli.tip.navwidget.dijit.AsynchContextMenu");
dojo.require("ibm.tivoli.tip.navwidget.dijit.RESTContextMenuProvider");
dojo.require("ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil");
dojo.require("ibm.tivoli.tip.navwidget.table.dijit.NavModelTableSelection");

dojo.require("ibm.tivoli.tip.navwidget.dojo.ToolbarUtil");

dojo.require("ibm.tivoli.tip.navwidget.properties.PropertyRenderer");
dojo.require("ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebookDialog");
dojo.require("ibm.tivoli.tip.navwidget.topology.dijit.MenuProviderHelper");
dojo.require("ibm.tivoli.tip.navwidget.topology.dijit.TopologyLabel");
dojo.require("ibm.tivoli.tip.navwidget.topology.dijit.DefaultActionHelper");

dojo.require("ibm.tivoli.navmodel.TableWidgetDef");
dojo.require("ibm.tivoli.navmodel.Exception");
dojo.require("ibm.tivoli.navmodel.Trace");

dojo.require("dojoe.tablex.TableAll");   // 49426 Gridx change

dojo.requireLocalization("ibm.tivoli.tip.dijit", "resources");

ibm.tivoli.tip.navwidget.table.dijit.AbstractTableView._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.tip.navwidget.table.dijit.AbstractTableView");

dojo.declare ("ibm.tivoli.tip.navwidget.table.dijit.AbstractTableView",
	[dijit.layout.ContentPane, ibm.tivoli.tip.navwidget.dijit.NavWidgetObjectPluggableView],
	{
	// summary:
	//		Coming soon
	// description:
	//		Coming soon
	
	// cmProvider: Object
	//		Coming soon
	cmProvider: null,
		
	// removeToolbar: Object
	//		Coming soon
	removeToolbar: null,
	// removeContextMenu: Object
	//		Coming soon
	removeContextMenu: false,
	// daHelper: Object
	//		Coming soon
	daHelper: null,
		
	// height: Object
	//		Coming soon
	height: "100%",
	
	// width: Object
	//		Coming soon
	width: "100%",
	
	// enableResize: Object
	//		Coming soon
	enableResize: true,
	
	insertOnTop : false,
	
	// showClearFilterConfirmation: Object
	//		Coming soon
	showClearFilterConfirmation: true,
	
	// title: [private] Object
	//		Coming soon
	title: null,

	// showTooltip: boolean
	//		Optionally hide tooltip
	showTooltip: true,

	// widgetState: Object
	//	Convenience method to manage the underlying table widget state
	//	Not to be confused with state object, which is used to persist state in a widget def
	widgetState: null,

	//***************************
	// "Private"
	//***************************

		// asahlin begin
	// _toolbarProviderEntries: [private] Object
	//		Coming soon
	_toolbarProviderEntries: null,
	
	// _toolbarButtonExist: [private] Object
	//		Coming soon
	_toolbarButtonExist: false,
	
	// _loadingToolbarTasks: [private] Object
	//		Coming soon
	_loadingToolbarTasks: false,
	
	// _tasksLoadedCallback:
	//		notification callback for when tasks are loaded
	_toolBarTasksLoadedCallback: null,
	
	// _rightClickClearEvent: [private] Object
	//		Coming soon
	_rightClickClearEvent: false,
	
	// _rightClickClearEvent: [private] Object
	//		Coming soon
	_toolbarServerTasks: null,
	
	// asahlin end

	
	// _tableContentPane: [private] Object
	//		Coming soon
	_tableContentPane: null,
	
	// _tableWidgetDefinition: [private] Object
	//		Coming soon
	_tableWidgetDefinition: null,
	
	// _propertyRenderer: [private] Object
	//		Coming soon
	_propertyRenderer: null,
	
	// _grid: [private] Object
	//		Coming soon
    _grid: null,
    	
    // _table: [private] Object
    //		Coming soon
	_table: null,
	
	// _preFilter: Object
	//		Coming soon
	_preFilter: null,
	
	// _state: Object
	//		Coming soon
	_state: null,
	
	// _condition: Object
	//		Coming soon
	_condition: null,
	
	// _tableConnections: Object
	//		Coming soon
	_tableConnections: null,

	// _widgetDefColumns: Object
	//		Coming soon
	_widgetDefColumns: false,

	
	// _dataStore: [private] Object
	//		Coming soon
	_dataStore: null,
	_dataStoreConnects: null,

	_tableDiv: null,
	_outerDiv: null,
	
	//_tabIndexValueTable: [private] integer
	_tabIndexValueTable:0,
	
	// _defaultSelectionClassName: [private] String
	_defaultSelctionClassName: "ibm.tivoli.tip.navwidget.table.dijit.NavModelTableSelection",
	
	_actionMenuDefaultCount: 0,
	
	// 54947
	// _suppressTooltip: [private] boolean
	//    Used to temporarily suppress tooltip during table refresh, etc.
	_suppressTooltip: false,
	
	// _trace: [private] Trace
	//		Utility for trace and logging
	_trace: ibm.tivoli.tip.navwidget.table.dijit.AbstractTableView._trace,

	//********************************************
	// "Protected" - to be overridden by subclasses
	//********************************************
	
	// _pTableDef: [protected] Object
	//		Coming soon
	_pTableDef: null,
	
	// _pTableClassName: [protected] String
	//		Coming soon
	_pTableClassName: null,
	
	// _pSelectionClassName: [protected] String
	//		Coming soon
	_pSelectionClassName: null,	
	
	// _pTableId: [protected] Object
	//		Coming soon
	_pTableID:null,
	
	// _pTableType: [protected] Object
	//		Coming soon
	_pTableType:null,
	
	// _pDefaultColumns: [protected] Object
	//		Coming soon
	_pDefaultColumns:null,
		
	// _DEFAULT_WIDGET_DEF_INFO:[protected] Object
	//      Coming soon
    _pDEFAULT_WIDGET_DEF_INFO: {
		className: "ibm.tivoli.navmodel.TableWidgetDef",
		baseUrl: "/ibm/tivoli/rest",
    	provider: "any",
    	datasource: "any",
    	widget: "tables",
    	definition: "DefaultTable"
	},
	
	_toolbarItemsToDestroy: null,

	// Used to help identify when an asterisk should be applied to the condition statement
	// when using enumerated/possible values.    Only used for simple table model.
	_stringFieldsWithPossibleValues: null,
	
	postCreate: function(){
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		this.inherited(arguments);
	},

	constructor: function (container, div, cmProvider) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		this._tableConnections = [];
		this._dataStoreConnects = [];
		this._toolbarItemsToDestroy = [];
		
		this._propertyRenderer = new ibm.tivoli.tip.navwidget.properties.PropertyRenderer();
		this._propertyRenderer.propThresholdCallback = dojo.hitch(this, this._evaluatePropThreshold);
		
		this.title = this.commonResources.RELATIONSHIP_VIEW;
		this.labelIconClass = "relationshipViewIcon";
		
		//Signal to superclass we'll be firing onSetDataSetLoadComplete manually
		this._setDatasetLoadDataCompletesSynchronously = false;

		// Use the supplied context menu provider if passed in
	        if (cmProvider) {
	        	this.cmProvider = cmProvider;
	        }
	        else {
		        // create a default context menu provider
	            var helper = new ibm.tivoli.tip.navwidget.topology.dijit.MenuProviderHelper();
		        this.cmProvider = new ibm.tivoli.tip.navwidget.dijit.RESTContextMenuProvider(
		        			{
		        				view: this,
		        				getMenuItemsCallback: dojo.hitch(helper, helper.getMenuItems),
		        				getMenuCommandCallback: dojo.hitch(helper, helper.getMenuCommand)
		        			}
		        		);	        	
	        }
	        //tab index
			this._tabIndexValueTable=ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.tabIndexFunc();

	},
	
	startup: function(){
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
	
		this.inherited(arguments);
	},
	
	destroy: function(){
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		
		this._destroyTable();
		
		this._destroyTableStore();

		this.inherited(arguments);
	},
	
	_destroyTable: function() {
		if (this._table){
			try {
				//Hitting a case where the TED destroy processing is dying in their _Toolbar class because of an undefined variable. We will open a defect 
				//against TED. But, working around with a try/catch for now.
				if (this._grid) {
					this._grid = null;
				}				
				this._table.destroyRecursive();
			} catch (ex) {
				if (this._trace.isError()) {
					this._trace.error("problem destroying this._table in AbstractTableView = " + this.id  + "; ex = " + ex.name + ": " + ex.message, "destroy");
				}
			}
			this._table = null;
			
			dojo.forEach(this._tableConnections, dojo.disconnect);
			this._tableConnections = [];
			
			//TED not good about cleaning up custom toolbar entries, so we'll do it here.
			  if (this._toolbarItemsToDestroy) {
				  for(var i = 0; i < this._toolbarItemsToDestroy.length; ++i) {
					  this._toolbarItemsToDestroy[i].destroyRecursive();
				  }
			  }
			  this._toolbarItemsToDestroy = [];
		}
	},
	
	setFilter: function(filter){
		// summary:
		//		Sets the specified filter into the table filter
		// description:
		//		filter to set on the table, the .ruleRelation value contains the relationship
		
		//as long as table exists, call set Filter
		if(this._table){	
			if (filter) {
				this._table.setFilter(filter);
				try {
					this._grid.showFilterBar(true);
				} catch (ex) {
					// Filter bar not supported for all table types
				}
			} else {
				this._table.clearFilter();
			}
		}
	},

	// Allow programmatic hide of filter
	setFilter: function(filter, hideFilterBar){
		// summary:
		//		Sets the specified filter into the table filter
		// description:
		//		filter to set on the table, the .ruleRelation value contains the relationship
		
		//as long as table exists, call set Filter
		if(this._table){	
			if (filter) {
				this._table.setFilter(filter, hideFilterBar);
				try {
					this._grid.showFilterBar(true);
				} catch (ex) {
					// Filter bar not supported for all table types
				}
			} else {
				this._table.clearFilter(hideFilterBar);
			}
		}
	},	

	getFilter: function(){
		// summary:
		//		Return the filter data from what is set in the table filter
		// description:
		//		returns an array of conditions and a .rulRelation value that
		//      contains the relationship between the conditions.
		var retvalue;
		if(this._table ){
			retvalue = this._table.getFilter(true);
		}
		if(retvalue && retvalue.length > 0){
			for(var j = 0; j < retvalue.length; j++){
				//go thru each index and cull out the data we cant use
				var entry = retvalue[j];
				if(entry.value != undefined) {
					// 2/8/12: JCHAN: Modifications needed to date objs to msecs
					if (this.isDateObject(entry.value)){
						// date object. Convert value to msecs
						entry.value = ""+entry.value.getTime();
					} else if (typeof entry.value === "object" ) {
						// might be a range object. Convert to start and end
						if (entry.value.start&&this.isDateObject(entry.value.start)) {
							entry.value.start = ""+entry.value.start.getTime();
						}
						if (entry.value.end&&this.isDateObject(entry.value.end)) {
							entry.value.end = ""+entry.value.end.getTime();
						}
						// else the entry.value should be string value.
						if (!entry.value.end && !entry.value.start){
							entry.value = entry.value.toString();
						}
					}
					// JCHAN - END
				}
				if(entry.formattedVal != undefined){
					entry.formattedVal = entry.formattedVal.toString();
				}
			}
		}
		return retvalue;
	},

	isDateObject: function(obj) {
		return(typeof(obj)==='date')?true:(typeof(obj)==='object')?obj.constructor.toString().match(/date/i)!==null:false;
	},

	refresh: function() {
		this._table.refresh();
	},
	
	getDatasetCondition: function(){
		
		return this._condition;
		
	},
	
	getTooltip: function(){
		if(this._table  && this._table._tooltip){
			return this._table._tooltip;
		}else{
			return null;	
		}
		
	},
	
	getTooltipRolloverTarget: function(){
	
		if(this._table && this._table._rolloverTarget){
			return this._table._rolloverTarget;
		}else{
			return null;
		}
	},
	
	getTooltipRolloverCellTarget: function(){
		if(this._table && this._table._rolloverCellTarget){
			return this._table._rolloverCellTarget;
		}else{
			return null;
		}
			
	},
	
	augmentGridParams: function(gridParams) {
		// summary:
		//		Modify or add to the gridParams before creating the grid
		// description:
		//		The implementation here returns the gridParams unmodified.
		//      Subclasses may override this to add to or modify the gridParams.
		
		return gridParams;
	},
	
	_getDefaultWidgetDefInfo: function(){
		return this._pDEFAULT_WIDGET_DEF_INFO;
	},
	
	_createSelection: function(){
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		
		//Need our own selection object
		if (!this._pSelectionClassName) {
			this._pSelectionClassName = this._defaultSelctionClassName;
		}
		var selectionClass = eval(this._pSelectionClassName);
		var kwArgs = {
				pluggableView: this
			};
		return new selectionClass(kwArgs); 
	},
	

	
	_buildTDLTable: function(){
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		if (this.dataset) {			
			var providerId = this.dataset.getProviderId();
			var datasourceId = this.dataset.getDatasourceId();
			if (!datasourceId) {
				datasourceId = "any";
			}
			var datasetId = this.dataset.getId();
			
			// update widget def
			if(datasetId) {
				this._tableWidgetDefinition.setDataSetID(datasetId);
			}
			
			// column ids that we need to fetch for each item
			var columnIds = [];
			
			// Initialize string field array
			this._stringFieldsWithPossibleValues = [];
    
			//Callback to get columns from dataset
			var gotColumns = function(cols){				
				//Process the list of columns 
				var columns = [];				
				var extraColumnData = [];	
				var viewColumns = [];
				if (cols) {
					for (var index = 0;  index < cols.length;  ++index) {
						var col = cols[index];						
						// If the column is a "linkType" ignore it
						var currcol = col.getValueType();
						if (currcol !== "linkType" && currcol !== "relationshipType"){
							var colData = {};
							var extraColData = {};
							colData.id = col.getId();
							colData.label = {};
							colData.label.value = col.getLabel();
							colData.type = col.getValueType();
				

							// Add key attributes for unsupported operators, keyboard navigation
							extraColData.filterable = col.isFilterable();
							//51538: Sortable property of column in nav model is ignored by ItemsTableView
							//Bobby: 09Oct2012, adding sortable attribute from dataset column							
							extraColData.sortable = col.isSortable();
							extraColData.unsupportedOperators = (col.getUnsupportedFilterOperators() ? col.getUnsupportedFilterOperators() : []);
							extraColData.cellNavigatable = (col.getAttribute("cellNavigatable") ? col.getAttribute("cellNavigatable") : false);
							extraColData.possibleValues = [];
							if (col.hasInlinedPossibleValues()) {
								extraColData.possibleValues = col._col.filter.possibleValues;		
								if (this.dataset.getDatasetImplementation() == ibm.tivoli.navmodel.AbstractDataset.implementation.SIMPLE
									&& col.getValueType() == "string")
									this._stringFieldsWithPossibleValues.push(colData.id);
							}
							
							// get column IDs to pass to store
							columnIds.push(colData.id);
							columns.push(colData);
							extraColumnData.push(extraColData);
							viewColumns.push(col);
						}
					}
					
					// default columns added on the front
					if(this._pDefaultColumns){
						for (var j = 0;  j < this._pDefaultColumns.length;  ++j) {
							var defColumn = this._pDefaultColumns[j];
							this._createDefaultColumnObject(columnIds,columns,viewColumns, defColumn);
						}
					}					
					
					// update the datasets.columns in the widget def
					this._tableWidgetDefinition.setDataSetColumns(columns);
					
					// update the tables.view in the widget def
					this._setTableView(viewColumns);
					
					this._widgetDefColumns = false;
					
					// init table builder	
					this._initTableBuilder(providerId, datasourceId, columnIds, this._tableWidgetDefinition.getDataSetColumns(), extraColumnData);
				}							       
			};

			// Callback to get columns from dataset for supplementing with globalized label, unsupportedOperator, 
			// cellNavigatable (keyboard/accessibility) params
			var gotColumnsForMixin = function(cols){				
				//Process the list of columns 
				var columns = [];	
				columnIds = [];
				var extraColumnData = [];				
				if (cols) {
					// widget def already has columns			
					var dsCols = this._tableWidgetDefinition.getDataSetColumns();
					for( var i = 0; i < dsCols.length; ++i) {	
						for (var j=0; j < cols.length; j++) {
							var extraColData = {};
							if (dsCols[i].id == cols[j].getId()) {
								// When label.value is empty string, replace with translated value from dataset
				  				if (dsCols[i].label.value == "") {
									dsCols[i].label.runtimeValue = cols[j].getLabel();		
								}
								// Add key attributes for unsupported operators, keyboard navigation
								extraColData.filterable = cols[j].isFilterable();
								//51538: Sortable property of column in nav model is ignored by ItemsTableView
								//Bobby: 09Oct2012, adding sortable attribute from dataset column								
								extraColData.sortable = cols[j].isSortable();
								extraColData.unsupportedOperators = (cols[j].getUnsupportedFilterOperators() ? cols[j].getUnsupportedFilterOperators() : []);
								extraColData.cellNavigatable = (cols[j].getAttribute("cellNavigatable") ? cols[j].getAttribute("cellNavigatable") : false);
								extraColData.possibleValues = [];
								if (cols[j].hasInlinedPossibleValues()) {
									extraColData.possibleValues = cols[j]._col.filter.possibleValues;
									if (this.dataset.getDatasetImplementation() == ibm.tivoli.navmodel.AbstractDataset.implementation.SIMPLE
												&& cols[j].getValueType() == "string")
										this._stringFieldsWithPossibleValues.push(cols[j].id);
								}
							
								break;
							}
						}
						// get column IDs to pass to store
						columnIds.push(dsCols[i].id);
						extraColumnData.push(extraColData);
					}				
					
					// update the datasets.columns in the widget def
					this._tableWidgetDefinition.setDataSetColumns(dsCols);
									
					// init table builder	
					this._initTableBuilder(providerId, datasourceId, columnIds, this._tableWidgetDefinition.getDataSetColumns(), extraColumnData);
				}							       
			};

			var gotColumnsError = function(error, exception, ioArgs){
				//Give the user some feedback
		            this.displayException(exception);
			};
			
			// see if columns are specified in the widget definition
			var tableView = this._tableWidgetDefinition.getTableView();
			if (tableView && tableView.length > 0) {							
				var viewLength = tableView.length;
				for (var viewIndex = 0;  viewIndex < viewLength;  ++ viewIndex) {
					columnIds.push(tableView[viewIndex].id);
				}	
				this._widgetDefColumns = true;
			}		    

			// KDM 12/6/2011:   No longer bypassing getting column information from dataset, necessary for key attributes 
			// such as translated label, unsupported operators (advanced filter), cellNavigatable (keyboard/accessibility)
			// if (columnIds.length > 0 && translated) {		
				//setTimeout(dojo.hitch(this, this._initTableBuilder, providerId, datasourceId, columnIds, this._tableWidgetDefinition.getDataSetColumns()), 0);
			// Widget def columns set, supplement including possible translate of column names
			if (columnIds.length > 0) {
				var columnArgsForMixin = {
				    type: this._pTableType,		
					onComplete: dojo.hitch(this, gotColumnsForMixin),
					onError: dojo.hitch(this, gotColumnsError),
					properties: ["all"]
				};
				this.dataset.getColumns(columnArgsForMixin);
			}
			// get columns from the dataset
			else {
				
				var columnArgs = {
				    type: this._pTableType,		
					onComplete: dojo.hitch(this, gotColumns),
					onError: dojo.hitch(this, gotColumnsError)
				};
				this.dataset.getColumns(columnArgs);
			}			
		} else {
			//Dataset is null, but we can still render an empty table
			setTimeout(dojo.hitch(this, this._initTableBuilder, null, null, [], this._tableWidgetDefinition.getDataSetColumns(), null), 0);
		}
	},
	
	_initTableBuilder: function(/*String*/ providerId, /*String*/ datasourceId, columnIds, datasetColumns, extraColData) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// providerId:
		//		
		// datasourceId:
		//		
		// columnIds:
		//		
		
		//We're going to be making a new table... let's destroy the current one
		this._destroyTable();
		
		//Destroy the old container div
		if (this._outerDiv) {
			var domNode = this.domNode;
			if(domNode){
	        		domNode.removeChild(this._outerDiv);
			}
			this._outerDiv = null;
		}
		
		//Destroy the old store; and create new one
		this._destroyTableStore();		
		this._createTableStore(columnIds);

			
		//var layout = [
		//  			{ field: "NAME", name: "Name", Width: "auto"},
		//  			{ field: "DESCRIPTION", name: "Description", Width: "auto" }					
		//  		];
		var tableDef = this._tableWidgetDefinition.getTableView();
		var sortFieldsForTable = null;
		var layout = [];
		
	
			//generate column layouts

			var regularColumns = [];
			var lockedColumns = [];
			var columnIdsSorted = [];
			
			if(this._state){
				//only process the column orders if returning from a previous state
				for( var k = 0; k < columnIds.length; ++k){		
					var order = 0;
					//first check if the state contains column orders
					if(this._state.stateData && this._state.stateData.layoutData){
						var weightEntry  = this._state.stateData.layoutData.item(columnIds[k]);
						if(weightEntry){
							var weight = weightEntry.stateColumnWeight;
							if(weight){
								//save the weight if it exists in the state for this item
								order = weight;
							}
						}
					}
					var entry = null;
					if (datasetColumns[k].label.value == "" && datasetColumns[k].label.runtimeValue) {
						entry = { id: columnIds[k], sortOrder: order, label: datasetColumns[k].label.runtimeValue, type: this._mapDataType(datasetColumns[k].type) };
					} else {
						entry = { id: columnIds[k], sortOrder: order, label: datasetColumns[k].label.value, type: this._mapDataType(datasetColumns[k].type) };
					}
					columnIdsSorted.push(entry);
				}	
				//actually sort the array now based on the weights in state
				columnIdsSorted = columnIdsSorted.sort(this._customSort);

			
			}else{
				//make a sorted object directly from each column, since already in proper order, no need to sort
				for( var k = 0; k < columnIds.length; ++k){
					var entry = null;
					if (datasetColumns[k].label.value == "" && datasetColumns[k].label.runtimeValue) {
						entry = { id: columnIds[k], sortOrder: 0, label: datasetColumns[k].label.runtimeValue, type: this._mapDataType(datasetColumns[k].type)};
					} else {
						entry = { id: columnIds[k], sortOrder: 0, label: datasetColumns[k].label.value, type: this._mapDataType(datasetColumns[k].type)};
					}
					columnIdsSorted.push(entry);
				}
			}
			
			for (var i = 0;  i < columnIdsSorted.length;  ++i) {
				//get this particular column and set into colView
				// (which either comes from widgetDef or mapped widgetDef object from dataset)
				var colView = null;
				for(var j = 0; j< tableDef.length; ++j){
					if( tableDef[j].id === columnIdsSorted[i].id){
						colView = tableDef[j];
						break;
					}
				}
				
				//defaults come from colView object 
				var hiddenFlag = colView.hidden;
				var lockedFlag = colView.lockColumn;
				
				//if we have a state with layout data, we need to check if
				//hidden flag or locked flag need to be overwritten with state value
				if(this._state && this._state.stateData && this._state.stateData.layoutData){
					var layoutEntry = this._state.stateData.layoutData.item(columnIdsSorted[i].id);
					if (layoutEntry) {
						var stateHiddenFlag = layoutEntry.hidden;
						//ensure the state value is valid
						if((stateHiddenFlag != null) && (stateHiddenFlag != undefined)){
							hiddenFlag = stateHiddenFlag;
						}		
					
						var stateLockedFlag = layoutEntry.isLocked;
						if((stateLockedFlag != null) && (stateLockedFlag != undefined)){
							lockedFlag = stateLockedFlag;
						}
					}
				}
				
				//logic is reversed from TED flag and widgetDef
				var noResizeFlag = false;
				//only if defined as not resizeable change flag
				//this means it will be resizable if not specified
				if(!colView.resizeable && (colView.resizeable != undefined)){
					noResizeFlag = true;
				}
			
				var unsupportedArr = new Array();
				//the column descriptor that is sent to TED table
				//First, check to see if any unsupported operators to exclude
                if (extraColData[i]) {
					if (extraColData[i].unsupportedOperators) {
						var arrIdx = 0;
						for (var j = 0; j < extraColData[i].unsupportedOperators.length; j++) {
    						if (this._translateOperatorInverse(extraColData[i].unsupportedOperators[j])) {
     							unsupportedArr[arrIdx++] = this._translateOperatorInverse(extraColData[i].unsupportedOperators[j]);
            				}
						}
						// 255892: KDM 2/8/12:   Due to widget def schema issues, range operator currently unsupported
						if (this._mapDataType(datasetColumns[i].type) == "date" || this._mapDataType(datasetColumns[i].type) == "time") {
							unsupportedArr[arrIdx++] = "range";
						}
						//255236: Cannot invoke table cell task using the keyboard  
						//Bobby: 08Dec11, adding navigatable attribute to column definition - BEGIN
						//255562/KDM: Filterable dataset flag not recognized when filtering table columns
						//Due to use of searchable in widget def, the table filterable flag used it and ignored filterable from the dataset
						//Cannot remove searchable, so instead recognize and check both values in determining if column is filterable
						var columnDescriptor = { 
							field: columnIdsSorted[i].id, 
							width: colView.width , 
							name: columnIdsSorted[i].label, 
							filterable: (this._isColumnFilterSupported(datasetColumns[i].type) && colView.searchable && extraColData[i].filterable), 
							hidden: hiddenFlag,  
							noresize: noResizeFlag, 
							formatter: dojo.hitch(this, this._formatter, colView), 
							datatype: this._mapDataType(datasetColumns[i].type), 
							disabledConditions: unsupportedArr, 
							navigatable: extraColData[i].cellNavigatable,
							//51538: Sortable property of column in nav model is ignored by ItemsTableView
							//Bobby: 09Oct2012, adding sortable attribute to column descriptor
							sortable: extraColData[i].sortable,							
							dataTypeArgs:{ enumConst:extraColData[i].possibleValues}
						};
					} else {				
						var columnDescriptor = { 
							field: columnIdsSorted[i].id, 
							width: colView.width , 
							name: columnIdsSorted[i].label, 
							filterable: (this._isColumnFilterSupported(datasetColumns[i].type) && colView.searchable && extraColData[i].filterable), 
							hidden: hiddenFlag,  
							noresize: noResizeFlag, 
							formatter: dojo.hitch(this, this._formatter, colView), 
							datatype: this._mapDataType(datasetColumns[i].type),
							navigatable: extraColData[i].cellNavigatable, 
							//51538: Sortable property of column in nav model is ignored by ItemsTableView
							//Bobby: 09Oct2012, adding sortable attribute to column descriptor							
							sortable: extraColData[i].sortable,	
							dataTypeArgs:{ enumConst:extraColData[i].possibleValues}
						};
						//Bobby: 08Dec11, adding navigatable attribute to column definition - END
					}
				} else {
					// 255892: KDM 2/8/12:   Due to widget def schema issues, range operator currently unsupported
					if (this._mapDataType(datasetColumns[i].type) == "date" || this._mapDataType(datasetColumns[i].type) == "time") {
						unsupportedArr[0] = "range";
					}
					var columnDescriptor = { field: columnIdsSorted[i].id, width: colView.width , name: columnIdsSorted[i].label, filterable: (this._isColumnFilterSupported(datasetColumns[i].type) && colView.searchable), hidden: hiddenFlag,  noresize: noResizeFlag, formatter: dojo.hitch(this, this._formatter, colView), datatype: this._mapDataType(datasetColumns[i].type), disabledConditions: unsupportedArr};
				} 
             
                // gridx uses decorator like DataGrid used to use formatter
                // 49426  if (this._pTableClassName.indexOf("tree") > 0) {  // hack until both table and tree table are using gridx
                columnDescriptor.decorator = dojo.hitch(this, this._formatter, colView);
                // 51355
                // gridx uses the formatter function to get the actual data value
                // for data comparisons, sorting, etc.
                columnDescriptor.formatter = dojo.hitch(this, this._dataFormatter, colView);
                // 49426  }
			
				//check if the lockedFlag is set for any of these columns...if so put in proper layout
				if(lockedFlag){
					//locked column, add to that array instead
					lockedColumns.push(columnDescriptor);
				}else{
					//normal column that is not locked
					regularColumns.push(columnDescriptor);
				}
				
					
				//do not allow initial sort if the column is hidden
				if(!hiddenFlag){

					//first check if we are coming from a previous state to return to
					if(this._state){
						if(this._state.stateData && this._state.stateData.sortData){
							//use the state sort data instead of the normal sort data from the widgetDef
							sortFieldsForTable = this._state.stateData.sortData;
						}
					}else{
						//check if sorted in widgetDef
						if(colView.sorted && (colView.sorted !== "none") && colView.sortOrder != -1){

							//init the array if it is not already done
							//to work around TED bug if we pass an empty array object
							if(!sortFieldsForTable){
								sortFieldsForTable = [];
							}
							//setup the flag based on sorted type
							var sortFlag = true;  //descending unless specified
							if(colView.sorted === "ascending"){
								sortFlag = false;
							}

							//@189518 i+1 as the index is 1 (not 0) based
							var sortParam = { colId: i+1, attribute: columnIdsSorted[i].id, descending: sortFlag, ascending: !sortFlag, sortOrder: colView.sortOrder};
							sortFieldsForTable.push(sortParam);
						}
					}
				}
				
			}
			
			//now make sure we sort the presort fields before setting them in TED
			if(sortFieldsForTable && (sortFieldsForTable.length > 0) ){
				sortFieldsForTable.sort(this._customSort);
			}
			
			//check if we have any locked columns, if so put them in the locked column part of the layout
			//we are hardcoding layout to only allow two sections, one for locked columns if any, and one for
			//the normal set of columns. This means we are hardcoding the structure and locked will always
			//appear before the normal columns in the table.
			if(lockedColumns.length > 0){
				layout = [{noscroll: true, defaultCell:{ width: 8, editable: false, type: dojox.grid.cells._Widget },rows: lockedColumns}, {defaultCell:{ width: 8, editable: false, type: dojox.grid.cells._Widget },rows: regularColumns}];
			}else{
				layout = [{defaultCell:{ width: 8, editable: false, type: dojox.grid.cells._Widget },rows: regularColumns}];
			}
			
		menuOptions = {rowMenu: dojo.hitch(this, this.cmProvider)};
		
		// Check if removeToolbar is passed on constructor, if not use widgetDef
		// Need a separate hideToolbar var should a reset on the widget def occur
		var hideToolbar;
		if(this.removeToolbar == undefined){
			this.hideToolbar = this._tableWidgetDefinition.isToolbarHidden();	
		} else {
			this.hideToolbar = this.removeToolbar;
		}

		this._outerDiv = document.createElement("div");
		this._outerDiv.id = this.id + "_" + this._pTableType + "OuterContentDiv";
		dojo.style(this._outerDiv, {height: this.height, width:this.width});
		dojo.place(this._outerDiv, this.domNode, "first");

		this._tableDiv = document.createElement("div");
		this._tableDiv.id = this.id + "_" + this._pTableType + "ContentDiv";
		dojo.style(this._tableDiv, {height: this.height, width:this.width});

		dojo.place(this._tableDiv, this._outerDiv,"first");
		
		//Setting tab index for table
		dojo.attr(this._tableDiv,"tabIndex",this._tabIndexValueTable+1);
		
		var widgetDef = this._tableWidgetDefinition.getDef();
		
		//if there is state filter data, then use this
		if(this._state && this._state.stateData && this._state.stateData.filterData){
			
			this._preFilter = this._state.stateData.filterData.ruleArray;
			this._preFilter.ruleRelation = this._state.stateData.filterData.ruleRelation;

		}else{
			//check to see if a preFilter is defined in the widgetDef
			var filterDefValue = this._tableWidgetDefinition.getTableFilter();			
			if(filterDefValue) {				
				this._preFilter = filterDefValue.ruleArray;
				// 2/8/12: JCHAN: Modifications needed to convert strings to date objects
				for ( var i = 0; i < this._preFilter.length; i++ ) {
					if ( this._preFilter[i].type.match(/date|time/gi) !== null ) {
						// convert string value to Date object for date/time type
						if ( typeof this._preFilter[i].value === "object" ) {
							// start and end is for range operator
							if ( this._preFilter[i].value.start ) {
								this._preFilter[i].value.start = new Date(parseInt(this._preFilter[i].value.start));
							}
							if ( this._preFilter[i].value.end ) {
								this._preFilter[i].value.end = new Date(parseInt(this._preFilter[i].value.end));
							}
						} else {
							// for is, after and before operator
							this._preFilter[i].value = new Date(parseInt(this._preFilter[i].value));
						}
					}
				}
				// JCHAN - END
				this._preFilter.ruleRelation = filterDefValue.ruleRelation;
				// get dataset condition string
				var dataset = this._tableWidgetDefinition.getDataSet()[0];
				if (dataset.conditions) {
					this._condition = dataset.conditions.filterString;
				}
			}else{
				this._preFilter = null;
			}
		}
		
		
		var filterObj;
		var toolbarOptions;

		filterObj = {					
				advancedFilter: this._tableWidgetDefinition.getTableShowAdvancedFilter(), 
				quickFilter: this._tableWidgetDefinition.getTableShowQuickFilter(),
				isServerSide: true,				
				setupFilterQuery: dojo.hitch(this, this._translateQuery)												
		};
		
		//set filter confirmation flag if specified in widgetDef
		var cfFlag = this._tableWidgetDefinition.getTableShowClearFilterConfirmation();
		
		if( (cfFlag != undefined || cfFlag != null) && (cfFlag == false)){
			this.showClearFilterConfirmation = false;
		}

		if(!this.hideToolbar){
			this.toolbarRows = this._tableWidgetDefinition.getToolbarRows();		
			toolbarOptions = {
					refreshIcon:true,
					actionMenu:true,
					configTableIcon:true, configTableMenu: true,
					configureTreeTableIcon:true, configureTreeTableMenu:true,
					exporterIcon: true, exporterMenu: true,
					printerIcon:true, printerMenu: true											
				};
			if(this.toolbarRows){
				//check which toolbar options are disabled via widget defs
				var widgetDefsToolbarOptions = {
					refreshIcon:false,
					actionMenu:false,
					configTableIcon:false, configTableMenu: false,
					configureTreeTableIcon:false, configureTreeTableMenu:false,
					exporterIcon: false, exporterMenu: false,
					printerIcon:false, printerMenu: false											
				};
				for(var i=0;i<this.toolbarRows.length && i<2;i++){ //i<2 because TED table allows having only 2 rows (TED_1000_src_20101119_2326)
					if(!this.toolbarRows[i].hidden){
						var toolbarContents = this.toolbarRows[i].contents;
						for (var j=0; j<toolbarContents.length; j++){
							switch(toolbarContents[j].type){
								case ibm.tivoli.tip.navwidget.table.dijit.AbstractTableView.toolbarItemTypes.PRINT_BUTTON:
									widgetDefsToolbarOptions.printerIcon = true;
									widgetDefsToolbarOptions.printerMenu = true;
									break;
								case ibm.tivoli.tip.navwidget.table.dijit.AbstractTableView.toolbarItemTypes.CONFIG_TABLE:
									widgetDefsToolbarOptions.configTableIcon = true;
									widgetDefsToolbarOptions.configTableMenu = true;
									widgetDefsToolbarOptions.configureTreeTableIcon = true;
									widgetDefsToolbarOptions.configureTreeTableMenu = true;
									break;
								case ibm.tivoli.tip.navwidget.table.dijit.AbstractTableView.toolbarItemTypes.EXPORT_BUTTON:
									widgetDefsToolbarOptions.exporterIcon = true;
									widgetDefsToolbarOptions.exporterMenu = true;
									break;
								case ibm.tivoli.tip.navwidget.table.dijit.AbstractTableView.toolbarItemTypes.REFRESH_BUTTON:
									widgetDefsToolbarOptions.refreshIcon = true;
									break;
								case ibm.tivoli.tip.navwidget.table.dijit.AbstractTableView.toolbarItemTypes.ACTION_MENU:
									widgetDefsToolbarOptions.actionMenu = true;
									break;	
								default:
									break;			
							}									
						}	
					}
				}	
				toolbarOptions = widgetDefsToolbarOptions;	
			}			
		}
		var indirectSelectFlag;
		//check if we need to disable the inderct selectoin (ie if selection type is "none")
		if(this._tableWidgetDefinition.getSelectionMode() === "none"){
			indirectSelectFlag = false;
		}else{
			indirectSelectFlag = true;
		}
		
		var gridParams = {
			columnReordering: this._tableWidgetDefinition.getTableColumnReordering(),
			rowsPerPage: this._tableWidgetDefinition.getTableCommonPagingSize(),	
			pagination: this._tableWidgetDefinition.getTableCommonPaging(),
			store: this._dataStore,
			toolbarVisible: !this.hideToolbar,
			toolbarOptions: toolbarOptions,
			footerVisible: true,
			persistOptions: false,        // 54615
			sortFields: sortFieldsForTable,
			selectionMode: this._tableWidgetDefinition.getSelectionMode(),
			plugins: {
				nestedSorting: true, 
				exporter: true, 
				printer: true, 
				filter:  filterObj,	
				sortFields: sortFieldsForTable,					//@189518
				indirectSelection: indirectSelectFlag,
				selector: {row: this._tableWidgetDefinition.getSelectionMode(), cell: "disabled"},            
				dnd: this._tableWidgetDefinition.getTableColumnReordering(),
				pagination: this._tableWidgetDefinition.getTableCommonPaging()
			}, 

			structure:layout,		
			parentPluggableView: this
		};

		gridParams = this.augmentGridParams(gridParams);
		
        // 49707
        // Add style function callback to each column definition
        if (gridParams.structure instanceof Array) {
        	dojo.forEach(gridParams.structure, function(colDef) {colDef.style = dojo.hitch(this, this._styleCell);}, this);
        }
        
        // 52591
        // Ensure hideFilterBar is true if the toolbar is hidden and
        // there is no preFilter set
        var filterOptions = {};
        if (!(gridParams.toolbarVisible)) {
        	if (!("preFilter" in gridParams)) {
        		filterOptions.hideFilterBar = true;
        	}
        }

		var tableClass = eval (this._pTableClassName);

		var tivTable = new tableClass (
			{
				gridParams_: gridParams,  // 49426
				filterOptions: filterOptions,  // 52591
				toolbarOptions: gridParams.toolbarOptions,
				toolbarVisible: gridParams.toolbarVisible,
				persistOptions: gridParams.persistOptions,  // 54615
				plugins: gridParams.plugins,
				Pagination: gridParams.pagination,
				widgetID: tableDef.id,
				height: this.height,
				width: this.width,
				tableView: this,
				cmProvider: this.cmProvider,
				div: this._tableDiv,
				enableResize: this.enableResize,
				insertOnTop : this.insertOnTop,
				state: this.widgetState,
				showTooltip:  this.showTooltip
			}, this._tableDiv);
		
		// Set filter only when dataset (and columns) configured
		// Do not set on empty dataset 
		// and only if a preFilter is defined 
		if (this.dataset && this._preFilter) {      // 36324
			// set filter when table is done loading
			var handle = dojo.connect(tivTable, "onTableLoad", this, dojo.hitch(this, function() {
				tivTable.setFilter(this._preFilter);
        	    dojo.disconnect(handle);
        	}));
		}

		//Store away the grid for use in dblclick, selection, etc.
		// this._grid = tivTable.grid ? tivTable.grid : tivTable._grid;  // GRIDX
		this._grid = tivTable._grid;  // 49426
		this._table = tivTable;
		
		if (this._grid.pluginMgr) {
			var testFilterVal = this._grid.pluginMgr.getPlugin('filter').ruleCountToConfirmClearFilter;
		
			//disable the clear filter confirmation if needed:
			if(!this.showClearFilterConfirmation){
				this._grid.pluginMgr.getPlugin('filter').ruleCountToConfirmClearFilter = Infinity;
			}
		}				
		
		//if we have toolbar rows, render them
		if(this.toolbarRows){
			this._fillToolbar(this.toolbarRows);
		}		
		
		// asahlin begin
		// if the toolbar is shown, check to see if provider provided toolbar tasks 
		if(!this.hideToolbar){
			this._loadToolbarTasks(dojo.hitch(this, this._toolbarTasksLoaded));
		}				
		// asahlin end	

		//make sure to remove the filter bar if toolbar is not shown at all
		if(this.hideToolbar){
			try {
				this._grid.showFilterBar(false);
			} catch (ex) {
				// Filter bar not supported for all table types
			}				
		}
		
		
		//add our cm menus?
        cmadapter = new ibm.tivoli.tip.navwidget.dijit.AsynchContextMenu({cmProvider: this._cmProvider});

		//Set our content to be the tableDiv       
	//	dojo.place(tivTable.domNode, this._tableDiv, "first");
				
	
		if (this._trace.isFinest()) {
			this._trace.finest ("created tableBuilder", "_initTableBuilder");
		}
				
		tivTable.startup();
	
			
        // 54947
		// register grid onRender event to temporarily suppress tooltip
		var render = dojo.connect(this._grid.body, "onRender", this, function(start, count){
			                                 this._suppressTooltip = true;
			                                 // wait 1 second for table to finish rendering, then clear flag
			                                 setTimeout(dojo.hitch(this, function() { this._suppressTooltip = false; }), 1000);
			                                 });
		this._tableConnections.push(render);
		if (this._grid.filterBar !== undefined) {
			var af = dojo.connect(this._grid.filterBar, "applyFilter", this, function(filterData){
                this._suppressTooltip = true;
                // wait 1 second for table to finish applying filer (possibly from filter dialog)
                setTimeout(dojo.hitch(this, function() { this._suppressTooltip = false; }), 1000);
                });
			this._tableConnections.push(af);
		}
		
		//register double click event
		var dblConn = dojo.connect (this._grid, "onRowDblClick", this, "_processDoubleClick");
		this._tableConnections.push(dblConn);
		//register menu event
		var menuConn = dojo.connect( this._grid, "onRowContextMenu", this, "_onRowContextMenu" );
		this._tableConnections.push(menuConn);
		//Register for user-initiated selection changes to the grid
        // 49426  if (this._grid.selection) {  // DataGrid case
 		// 49426   this._tableConnections.push(dojo.connect(this._grid.selection, "onChanged", this, this._gridSelectionChangedCallback));
 		// 49426  }
        // 49426  else {  // gridx case
       	this._tableConnections.push(dojo.connect(this._grid.select.row, "onSelected", this, this._gridXSelectionChangedCallback));
       	this._tableConnections.push(dojo.connect(this._grid.select.row, "onDeselected", this, this._gridXSelectionChangedCallback));
       	// 49426  GridX uses onSelectionChange event for IndirectSelection
        if ((typeof this._grid._modules["indirectSelect"] !== "undefined") || ("onSelectionChange" in this._grid.select.row)) {  // 53683
            this._tableConnections.push(dojo.connect(this._grid.select.row, "onSelectionChange", this, this._gridXSelectionChangedCallback));
        }
        // 49426  }
		
		//check that the actions menu is there
		if(this._table._toolbar && this._table._toolbar.actionListWidget && this._table._toolbar.actionListWidget.dropDown){
			//if so, connect to the open/close events
			this._actionMenuDefaultCount = this._table._toolbar.actionListWidget.dropDown.getChildren().length;
			var openActMenuConn = dojo.connect(this._table._toolbar.actionListWidget, "openDropDown", this, this._openActionMenu);
			this._tableConnections.push(openActMenuConn);
		
			var closeActMenuConn = dojo.connect(this._table._toolbar.actionListWidget, "closeDropDown", this, this._closeActionMenu);
			this._tableConnections.push(closeActMenuConn);
		}
		
	//  	this.viewLoaded = true;
	  	
	  	//Send notification we're done loading the dataset/prepping the view
	//  	this._onSetDatasetLoadComplete(this.dataset);
	
		
		//Custom exporter stuff.
		if (tivTable._toolbar) {
			tivTable.defaultExportFormatter = false;
			if (this._grid.setExportFormatter) { //Check for existence of setExportFormatter function as doesn't currently exist in tree table
				this._grid.setExportFormatter(dojo.hitch(this, this._exportFormatter));
			}
		}
	},
	
	_fillToolbar: function(/*Object[]*/ toolbars) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon		
		// toolbars:
		//		Coming soon
		
		for(var i=0;i<toolbars.length && i<2;i++){ //i<2 because TED table allows having only 2 rows (TED_1000_src_20101119_2326)
			if(!toolbars[i].hidden){
				var item = null;
				var toolbarContents = toolbars[i].contents;
				for (var j = 0; j < toolbarContents.length; j++){ // KDM:6/24/2011 Reversed order of input from previous due to TED 1.1.0.1 mods
					this._addItemToTheToolbar(toolbarContents[j], j, i+1); //i+1 because TED starts row enumeration from 1 :/ (TED_1000_src_20101119_2326)
				}
			}
		}
	},
	
	// asahlin begin
 	_loadToolbarTasks: function(callback) {
 		if (!this._loadingToolbarTasks) {
 		//	this.cmProvider.getMenuItemsAsynch(this, callback, "toolbar");
 			if (callback != null) {
 				this._toolBarTasksLoadedCallback = callback;
 			}
 			
 			if (this.dataset) {
	 			this.dataset.getTasks(  
	    				{onComplete: dojo.hitch(this, callback),
	    				onError: dojo.hitch(this, this._handleToolbarTasksError),
	    				taskCategory: "toolbar",
	    				failOK: true,
	    				selection: this.buildClientNavModelSelection(this, this.dataset.getVersion())});
	 			
	 			this._loadingToolbarTasks = true;
 			} else {
 				//Should still invoke the callback (and for consistency should be on different thread)
 				if (callback != null) {
 					setTimeout(dojo.hitch(this, callback), 0);
 				}
 			}
 		} else {
 			this._loadingToolbarTasksNextCallback = callback;
 		}
 	},
 	
	buildClientNavModelSelection: function(context, /*Float*/ version) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// context:
		//		
		// version:
		//		
		var currentSelection = null;
		var selectedThings = ("selection" in context) ? context.selection : null;
		if (selectedThings) {
			currentSelection = new ibm.tivoli.navmodel.Selection({version: version});
			currentSelection.addItems(selectedThings.getItemIds());
			currentSelection.addLinks(selectedThings.getLinkIds());
			currentSelection.addRelationships(selectedThings.getRelationshipIds());
		}
		return currentSelection;
	},
 	
 	_handleToolbarTasksError: function(error, exception, ioArgs) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// error:
		//		
		// exception:
		//		
		// ioArgs:
		//		

		// log error here
		if (this._trace.isError()) {
			this._trace.error(">>> query toolbar tasks error = " + error, "_handleToolbarTasksError");
		}
		if (this._toolBarTasksLoadedCallback != null) {
			this._toolBarTasksLoadedCallback(false);
		}
		
	},
 	
	_toolbarTasksLoaded: function(/*Boolean*/ success) {			
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// success:
		//		Coming soon
		// tags:
		//		private		
 		if (!success) {
 	    	this._trace.warn("Toolbar tasks not loaded", "_toolbarTasksLoaded");
 		}
 		
 		// clean out old toolbar items, then create
 		if(this._toolbarProviderEntries) {
			for (var i=0; i<this._toolbarProviderEntries.length; i++) {
				// this._table can be null if daHelper function navigates away from page before callback 
				// from getGlobalTasks processes after selection change
	 			if(this._table != null) {
	 				this._table.removeItemFromToolbar(this._toolbarProviderEntries[i].id);

	 			}
			}
 			
 			//Clear out the array
			this._toolbarProviderEntries.splice(0); 
 		}
 		
 		//add to the "toolbar"
 	    this._createProviderPortionOfToolbar(success);

 	},
 	
	_createProviderPortionOfToolbar: function(/*Object*/ providerTasksReceived){		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// providerTasksReceived:
		//		Coming soon
		// tags:
		//		private
 		if(providerTasksReceived){
 			// store off any tasks that maybe serverTasks
 			if(this.cmProvider) { 				
 				this._toolbarServerTasks = this.cmProvider._getServerTasks(providerTasksReceived);
 			}
 			
 			//adding provider tasks
			var toolbarTasks = this.cmProvider.tasksAsMenuJson(providerTasksReceived);
			//adding provider tasks to the custom toolbar
			var addChildCount = 0, tBTask;
			
			// check to see if array to hold toolbar buttons exists
			if (!this._toolbarProviderEntries) {
				//Create toolbar entries data struct
				this._toolbarProviderEntries = [];
			} else {
				this._toolbarProviderEntries.splice(0); //Clear out the array
			}
			
			for (var i=0; i<toolbarTasks.length; i++) {
				tBTask = toolbarTasks[i];
				// Do not add view specific tasks to custom toolbar
				if (tBTask.launchType == "VIEW_COMMAND")
					break;
				
				// Do not process seperator tasks
				if (tBTask.actionID === "_separator_") {
					break;
				}

				if (tBTask.category === "toolbar") {
		    		var button = null;
		    		if(tBTask.submenu_items){
		    			var dropdownMenu = new dijit.Menu();
		    			var params = {
		    	        		provider: this.cmProvider,
		    					dropDown: dropdownMenu					
		    			};
		    			dojo.mixin( params, tBTask );
		    			button = new dijit.form.DropDownButton(params);
		    			this._addItemsToMenu(dropdownMenu,tBTask.submenu_items);
		    		} else {
		    	        var params = {
		    	        		adapter: this,
		    	        		onClick: function() {
		    	        			// set the serverTasks on cmProvider, need to do it when button
		    	        			// is clicked, becuasea right click could have cleared out cmProvider,
		    	        			// serverTasks arrary	    	    			
		    	     				this.adapter.cmProvider._setServerTasks(this.adapter._toolbarServerTasks);
		    	     				
		    	        			// need to get to the view that has the selection
		    	        			var ctx = this.getParent().getParent().getParent();
		    	        			this.adapter.cmProvider.getMenuCommand(this.actionID, ctx , this);
		    	        		}
		    	        };
		    	        dojo.mixin( params, tBTask );
		    	        button = new dijit.form.Button(params);
		    		}
					// this._table can be null if daHelper function navigates away from page before callback 
					// from getGlobalTasks processes after selection change
		    		if(this._table != null) {
		    			this._table.addToToolbar(button, addChildCount++, 0);	
		    			this._toolbarProviderEntries.push(button); //Keep track of button
		    		}
		    	}
			}
			
			if(this._toolbarProviderEntries != null && this._toolbarProviderEntries.length > 0) {
				// mark that we have button, tells selection whether it should check on selection
				this._toolbarButtonExist = true;
			} else {
				// mark that we have button, tells selection whether it should check on selection
				this._toolbarButtonExist = false;
			}
 		}
 		
 		//Mark we're done loading toolbar tasks and make another request if necessary
 		this._dealWithLoadingToolbarTasksNextCallback();
	},
	
 	//Mark we're done loading toolbar tasks and make another request if necessary
	_dealWithLoadingToolbarTasksNextCallback: function() {
 		if (this._loadingToolbarTasks) {
 			this._loadingToolbarTasks = false;
 			//Selection may have changed while we were loading provider tasks and building up toolbar, and someone
 			//else may have wanted to make the request
 			if (this._loadingToolbarTasksNextCallback) {
	 			var nextCallback = this._loadingToolbarTasksNextCallback;
	 			this._loadingToolbarTasksNextCallback = null;
	 			this._loadToolbarTasks(dojo.hitch(this, nextCallback));
 			}
 		}
	},
	// asahlin end
	
	_addDefaultToolbarButton: function(/*String*/ buttonType, /*Number*/ position, /*Number*/ rowNumber) {
		var button = null;
		if(this._table._toolbar && this._table._toolbar.id){
			button = dijit.byId(buttonType + this._table._toolbar.id);
		}
		else if(this._table.toolbar && this._table.toolbar.id){
			button = dijit.byId(buttonType + this._table.toolbar.id);
		}
				
		if(button){
			this._table.addToToolbar(button, position, rowNumber);	
		}
		else {
			console.log("No toolbar object defined...skipping adding this button" + buttonType);
		}

		return button;				
	},

	_addItemToTheToolbar: function(/*Object*/ item,/*Number*/ position, /*Number*/ rowNumber) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon		
		// item:
		//		Coming soon
		// position:
		//      Coming soon
		// rowNumber:
		//		Coming soon
		switch(item.type){
			case ibm.tivoli.tip.navwidget.table.dijit.AbstractTableView.toolbarItemTypes.PRINT_BUTTON:
				this._addDefaultToolbarButton("printIcon", position, rowNumber);				
				break;
			case ibm.tivoli.tip.navwidget.table.dijit.AbstractTableView.toolbarItemTypes.CONFIG_TABLE:
				this._addDefaultToolbarButton("configureColIcon", position, rowNumber);
				break;
			case ibm.tivoli.tip.navwidget.table.dijit.AbstractTableView.toolbarItemTypes.EXPORT_BUTTON:
				this._addDefaultToolbarButton("exportDataIcon", position, rowNumber);
				break;
			case ibm.tivoli.tip.navwidget.table.dijit.AbstractTableView.toolbarItemTypes.REFRESH_BUTTON:
				// Do not add refresh bar explicitly, will be handled in toolbarOptions, refreshIcon: true
				break;
			case ibm.tivoli.tip.navwidget.table.dijit.AbstractTableView.toolbarItemTypes.ACTION_MENU:
				this._addDefaultToolbarButton("actionListMenu", position, rowNumber);
				break;
			case ibm.tivoli.tip.navwidget.table.dijit.AbstractTableView.toolbarItemTypes.SEPARATOR:
				var separator = new dijit.ToolbarSeparator();
				this._table.addToToolbar(separator, position, rowNumber);
				this._toolbarItemsToDestroy.push(separator); //Hack, but TED Toolbar not good about cleaning up custom toolbar entries
				break;
			case ibm.tivoli.tip.navwidget.table.dijit.AbstractTableView.toolbarItemTypes.CUSTOM_BUTTON:
				var button = ibm.tivoli.tip.navwidget.dojo.ToolbarUtil.customButton({id: this.id, item: item});
				if (button) {
					this._table.addToToolbar(button, position, rowNumber);
					this._toolbarItemsToDestroy.push(button); //Hack, but TED Toolbar not good about cleaning up custom toolbar entries
				}
				break;
			case ibm.tivoli.tip.navwidget.table.dijit.AbstractTableView.toolbarItemTypes.CUSTOM_COMBO:
				var combo = ibm.tivoli.tip.navwidget.dojo.ToolbarUtil.customComboButton({id: this.id, item: item});
				if (combo) {
					this._table.addToToolbar(combo, position, rowNumber);
					this._toolbarItemsToDestroy.push(combo); //Hack, but TED Toolbar not good about cleaning up custom toolbar entries
				}

				var label = ibm.tivoli.tip.navwidget.dojo.ToolbarUtil.customLabel({id: this.id, item: item});
				if (label) {
					this._table.addToToolbar(label, position, rowNumber);
					this._toolbarItemsToDestroy.push(label); //Hack, but TED Toolbar not good about cleaning up custom toolbar entries
				};				
				break;
			case ibm.tivoli.tip.navwidget.table.dijit.AbstractTableView.toolbarItemTypes.CUSTOM_TEXTBOX:
				var text = ibm.tivoli.tip.navwidget.dojo.ToolbarUtil.customText({id: this.id, item: item});
				if (text) {
					this._table.addToToolbar(text, position, rowNumber);
					this._toolbarItemsToDestroy.push(text); //Hack, but TED Toolbar not good about cleaning up custom toolbar entries
				}

				var label = ibm.tivoli.tip.navwidget.dojo.ToolbarUtil.customLabel({id: this.id, item: item});
				if (label) {
					this._table.addToToolbar(label, position, rowNumber);
					this._toolbarItemsToDestroy.push(label); //Hack, but TED Toolbar not good about cleaning up custom toolbar entries
				};
				break;
			case ibm.tivoli.tip.navwidget.table.dijit.AbstractTableView.toolbarItemTypes.CUSTOM_DROPDOWN_BUTTON:
				var dropDown = ibm.tivoli.tip.navwidget.dojo.ToolbarUtil.customDropButton({id: this.id, item: item});
				if (dropDown) {
					this._table.addToToolbar(dropDown, position, rowNumber);
					this._toolbarItemsToDestroy.push(dropDown); //Hack, but TED Toolbar not good about cleaning up custom toolbar entries
					
					//Further hack, but in IE we also need to keep track of menu items within dropDown so we can destroy later
					if (dojo.ieIE && dropDown.dropDown) {
						var childMenuItems = dropDown.dropDown.getChildren();
						for (var i = 0; i < childMenuItems.length; i++) {
							this._toolbarItemsToDestroy.push(childMenuItems[i]);
						}
					}
				}	
				break;
			case ibm.tivoli.tip.navwidget.table.dijit.AbstractTableView.toolbarItemTypes.VIEW_TOGGLER:
				var viewToggler = ibm.tivoli.tip.navwidget.dojo.ToolbarUtil.viewToggler({view: this, item: item});
				if (viewToggler) {
					this._table.addToToolbar(viewToggler, position, rowNumber);
					this._toolbarItemsToDestroy.push(viewToggler); //Hack, but TED Toolbar not good about cleaning up custom toolbar entries
					
					//Further hack, but in IE we also need to keep track of menu items within dropDown so we can destroy later
					if (dojo.ieIE && viewToggler.dropDown) {
						var childMenuItems = viewToggler.dropDown.getChildren();
						for (var i = 0; i < childMenuItems.length; i++) {
							this._toolbarItemsToDestroy.push(childMenuItems[i]);
						}
					}
				}
				break;
	
			default:
				this._trace.warn("Invalid toolbar item type provided", "_addItemToTheToolbar");
		}
	},	

    	_customSort: function( a, b ) {		
		// summary:
		//		Function used to determine how to sort an array of properties.
    	//		This function assumes two Properties objects passed in and
    	//		it will then sort based on the Weight of the item
    	//		if weights are equal it will sort lexigraphically
		// description:
		//		Coming soon
    		// a:
    		//		Coming soon
    		// b:
    		//		Coming soon
    		var retval = 0;    	
    		retval = a.sortOrder - b.sortOrder;    	
    		return retval;
	},
	
	
	//To be implemented by subclass in order to create the datastore
	_createTableStore:function(columnIds){
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// columnIds:
		//

		// connect to datastore onRefresh event
		if (this._dataStore) {						
			this._dataStoreConnects.push(dojo.connect(this._dataStore, "onRefresh", this, "_onDatastoreRefresh"));
		}	
	},

	_destroyTableStore: function() {
		if (this._dataStore) {
			dojo.forEach(this._dataStoreConnects, dojo.disconnect);
			this._dataStoreConnects = [];
			this._dataStore.close();
			this._dataStore = null;
		}
	},

	//dataset refresh
	_onDatastoreRefresh:function() {
		this._table.refresh();
	},


	resize: function(newSize){
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// newSize:
		//		
		if (newSize && this._table) {
			this._table.resize (newSize);
		}
		
		//Call super
		this.inherited(arguments);
		
	},
	getContentPane:function(){
		return this;
	},

	//Override of NavWidgetPluggableView
	_setDatasetSetWidgetDefs: function(/*ibm.tivoli.navmodel.Dataset*/ dataset, /*ibm.tivoli.navmodel.AbstractWidgetDef?*/datasetWidgetDefinition, /*ibm.tivoli.tip.navwidget.dojo.NavWidgetBaseViewState?*/state){
		// summary:
		//		Called during the setDataset lifecycle for setting widget defs specified in the dataset to the view
		//		Subclasses will probably NOT need to override.
		// dataset:
		//		The Client Nav Model Dataset that the pluggable view should render.
		// description:
		//		Sets widget defs specified in the dataset to the view
		// tags:
		//		protected

		this.inherited(arguments);
		this._tableWidgetDefinition = this.widgetDefinition;
		
		if (datasetWidgetDefinition !== undefined) {
			this._tableWidgetDefinition = this.datasetWidgetDefinition;
		}
			
		//save the state that was passed in
		this._state = state;
	},
	
	//Override of NavWidgetPluggableView
	_setDatasetLoadData: function(dataset){
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// dataset:
		//		
		if (this._trace.isFinest()) {
			this._trace.entering("setting dataset for " + this.id, "_setDatasetLoadData");
		}
		
		this.dataset = dataset;
		
		if(!this._tableWidgetDefinition) {
			this._createTableDef();
		}
		
		this._buildTDLTable();
		
		if (!this.dataset) {
			//No dataset, so fire _onSetDatasetLoadComplete
			setTimeout(dojo.hitch(this, this._onSetDatasetLoadComplete, dataset), 0);
		}
		
		if (this._trace.isFinest()) {
			this._trace.exiting("setting dataset for " + this.id, "_setDatasetLoadData");
		}
	},
	
	_createTableDef: function() {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		
		if(!this.widgetDefinitionId){
			this.widgetDefinitionId = this._pDEFAULT_WIDGET_DEF_INFO.definition;
		}
		
		this._tableWidgetDefinition
			= ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.createWidgetDefinition(this._pDEFAULT_WIDGET_DEF_INFO.className, 
					this.widgetDefinitionId, this._pDEFAULT_WIDGET_DEF_INFO, this.dataset, this.getParentPluggableViewWidgetDefinition());
		
		//Call superclass
		this.inherited(arguments);

	},  
	
    // 49707
    addCustomStylesToCell: function(columnDefObj, item, property, cell) {
        // subclasses can override this to add to the customStyles array on the cell
        // as long as they call the inherited implementation also

        //Wrapping enabled in TED by default, so turn off if disabled in widgetDef or provider
	    if (!columnDefObj.lineWrap) {
	    	if (cell && cell.customStyles && cell.customStyles instanceof Array) {
	    		cell.customStyles.push("white-space: nowrap");
	    		cell.customStyles.push("word-wrap: normal");
	    	}
        }
    },

    // 49707
    _styleCell: function(cell) {
        // Callback to add customStyles to cell
        // todo - put here
        if (!cell.customStyles) { 
            cell.customStyles = []; 
        }  
        var style = "";   // 52584
        var item = cell.rawData();
		// 52584
		// Dataset should never provide a null property object for a defined column,
		// but some data providers may have this edge case, so ensure we avoid an "object
		// undefined" error
        if (item) { 
			var prop = item.getProperty(cell.column.field());
			// pick up any threshold customStyles
			//    NOTE: To be more efficient, should add function to renderer to just
			//    set the styles for thresholds and not render the property and 
			//    throw away that rendering just to get the customStyles set.
			this._propertyRenderer.getRenderedPropertyNode(item, prop, this,
					cell.row.id, cell);
			// allow for additional customStyles
			this.addCustomStylesToCell(cell.column, item, prop, cell);
			if (cell && cell.customStyles && cell.customStyles instanceof Array) {
				style = cell.customStyles.join(";");
			}
		}
		return style;
    },	

	_formatter: function(/*String*/ columnDefObj, object, rowId, cellOrDepth, treeCell){
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// propId:
		//		
		// object:
		//		
		var cellData = null;
		//console.log("formatting "+propId + "_"+rowId);
		var propId = columnDefObj.id;
		
		// 51355
        // Make sure we get the row item
        var item = object;
        if ((item == null) || !(typeof item == "object")) {
            item = this._grid.model.byId(rowId).item;
        }		
		
		// 52063
		// Dataset should never provide a null property object for a defined column,
		// but some data providers may have this edge case, so ensure we avoid an "object
		// undefined" error
		if (item) {
			switch (propId) {

			case "label":
				cellData = this._propertyRenderer.getRenderedValueNode(item.getLabel());
				break;

			case "sourceItem":
				if (item.getObjectType() === "relationship") {
					var sourceItem = item.getSourceItem();
					if (sourceItem) {
						cellData = this._propertyRenderer.getRenderedItemNode(sourceItem, this);
					}
				}
				break;

			case "targetItem":
				if (item.getObjectType() === "relationship") {
					var targetItem = item.getTargetItem();
					if (targetItem) {
						cellData = this._propertyRenderer.getRenderedItemNode(targetItem, this);
					}
				}
				break;

			case "item1":
				if (item.getObjectType() === "link") {
					var item1 = item.getItem1();
					if (item1) {
						cellData = this._propertyRenderer.getRenderedItemNode(item1, this);
					}
				}
				break;

			case "item2":
				if (item.getObjectType() === "link") {
					var item2 = item.getItem2();
					if (item2) {
						cellData = this._propertyRenderer.getRenderedItemNode(item2, this);
					}
				}
				break;

			case "dataset1":
				if (item.getObjectType() === "link") {
					var d1 = item.getDataset1();
					var value = (d1 ? d1.getLabel() : "");
					cellData = this._propertyRenderer.getRenderedValueNode(value, this);
				}
				break;

			case "dataset2":
				if (item.getObjectType() === "link") {
					var d2 = item.getDataset2();
					var value = (d2 ? d2.getLabel() : "");
					cellData = this._propertyRenderer.getRenderedValueNode(value, this);
				}
				break;

			default:
				var prop = item.getProperty(propId);
				if (prop) {
					if (typeof (cellOrDepth) == "number") {
						//passed a depth so nee to use treeCell param
						cellData = this._propertyRenderer.getRenderedPropertyNode(item, prop, this, rowId, treeCell);
					} else {
						//normal cell passed in so use it
						cellData = this._propertyRenderer.getRenderedPropertyNode(item, prop, this, rowId, cellOrDepth);
					}
				}
			}
			// 49707 customStyle setting moved to addCustomStylesToCell(cell)
		}

		if (!cellData) {
			cellData = this._propertyRenderer.getRenderedValueNode("", this);
		}
        else if ((dojo.isIE < 9 || (dojo.isIE && dojo.isQuirks)) && !columnDefObj.lineWrap) {  // 51856
            // If this is IE8 (or less) and column should be nowrap, set styles on DIV and preserve DIV by wrapping it
            // before returning
            var wrapperDiv = dojo.doc.createElement("div");
            dojo.style(cellData, "overflow", "hidden");
            dojo.style(cellData, "white-space", "nowrap");
            dojo.style(cellData, "word-wrap", "normal");
            dojo.place(cellData, wrapperDiv, "last");
            cellData = wrapperDiv;
        }
		
		return cellData.innerHTML;

	},
	
	// 51355
    _dataFormatter: function(/*String*/ columnDefObj, rawData) {
    	// summary:
    	//		Coming soon
    	// description:
    	//		Coming soon
    	// columnDefObj:
    	//
    	// object:
    	//
    	
    	var cellDataValue = "";
    	var propId = columnDefObj.id;
    	var object = (propId in rawData) ? rawData[propId] : null; 
        
  		// Dataset should never provide a null property object for a defined column,
  		// but some data providers may have this edge case, so ensure we avoid an "object
  		// undefined" error
  		if (object) {
			switch (propId) {

				case "label":
					cellDataValue = object.getLabel();
					break;

				case "sourceItem":
					if (object.getObjectType() === "relationship") {
						var sourceItem = object.getSourceItem();
						cellDataValue = (sourceItem ? sourceItem.getLabel() : "");
					}
					break;

				case "targetItem":
					if (object.getObjectType() === "relationship") {
						var targetItem = object.getTargetItem();
						cellDataValue = (targetItem ? targetItem.getLabel() : "");
					}
					break;

				case "item1":
					if (object.getObjectType() === "link") {
						var item1 = object.getItem1();
						cellDataValue = (item1 ? item1.getLabel() : "");
					}
					break;

				case "item2":
					if (object.getObjectType() === "link") {
						var item2 = object.getItem2();
						cellDataValue = (item2 ? item2.getLabel() : "");
					}
					break;

				case "dataset1":
					if (object.getObjectType() === "link") {
						var d1 = object.getDataset1();
						cellDataValue = (d1 ? d1.getLabel() : "");
					}
					break;

				case "dataset2":
					if (object.getObjectType() === "link") {
						var d2 = object.getDataset2();
						cellDataValue = (d2 ? d2.getLabel() : "");
					}
					break;

				default:
					var prop = object.getProperty(propId);
					if (prop) {
						switch (prop.getValueState()) {
							case "ok":
								switch (prop.getValueType()) {
									case "itemName":
									case "itemType":
										cellDataValue = prop.getDisplayValue();
										break;
									case "status":
										cellDataValue = prop.getValue().getValue();
										break;
									case "percentage":
										cellDataValue = prop.getValue();
										break;
									default:
										// 53750  Check for non-scalar property value
										if (typeof prop.getValue() == "object") {
											cellDataValue = prop.getDisplayValue();
										} else {
											cellDataValue = prop.getValue();											
										}
										break;
								}
								break;
							case "pending":
								break;
							case "error":
								break;
							case "none":
								break;
							default:
						}
					}
				}
        }
        return cellDataValue;
    },	

	_evaluatePropThreshold: function(prop, object){
		var thresholds = null;
		var retThreshold = null;
		var threshold = null;
		
		var propId = prop.getId();
		
		// get the thresholds for this property
		thresholds = this._getPropThresholds(propId);
		
		// process them if thresholds for this property are returned
		if(thresholds && prop){
			var propType = prop.getValueType();
			// make a "mock" threshold obj for prop value so we can reuse
			// compare method
			var propValue = {"start": prop.getValue()};
			
			// find highest threshold that matches this property value
			for(var i = 0; i < thresholds.length; i++){
				//Check to see if the property matches the current threshold
				if(this._compareThresholds(propValue, thresholds[i], propType, true)){
				//if(true){
					//we know this property meets the current threshold, so now see if it is the highest of this thresh type
					
					//first check if retThreshold is defined.  If not, then this is the first match, so dont need to compare
					if(!retThreshold){
						retThreshold = thresholds[i];
					}
					//one match already, so we need to get the highest threshold value
					else if(this._compareThresholds(thresholds[i],retThreshold, propType)){
						retThreshold = thresholds[i];
					}
				}
			}
			
		}	
		
		return retThreshold;
	},
	
	//returns true if thresh 1 is >= thresh 2
	_compareThresholds: function(thresh1, thresh2, type, propComp){
		//  FROM Property.getValueType():
		//		|	values:	"itemName", "itemType", "linkName", "linkType", "relationshipName", "relationshipType"
		//		|		"string", "string[]",
		//		|		"byte", "short", "int", "long", "byte[]", "short[]", "int[]", "long[]",
		//		|	x->	"ubyte, "ushort", "uint", "ulong", "ubyte[]", "ushort[]", "uint[]", "ulong[]",	
		//		|		"boolean", "boolean[]"	
		//		|	x->	"enum", "map", "json"
		//		|	x->	"status",
		//		|		"date", "datetime", "time", "date[]", "datetime[]", "time[]"
		//		|		"isodate", isodatetime", "isotime", "isodate[]", "isodatetime[], "isotime[]", "isointerval", "isoduration"
		//		|		"float", "double", "percentage", "float[]", "double[]"
		//  x-> means we dont support it...also we dont support array types for thresholding
	
		var retVal = false;
		
		switch(type) {
			case "int": case "short": case "byte": case "long": case "float": case "double": case "percentage":
				//parseFloat handles floats and non floats and returns Numbers
				var val1 = parseFloat(thresh1.start);
				var val2 = parseFloat(thresh2.start);
				if(!(isNaN(val1)) && !(isNaN(val2))){
					retVal = val1 >= val2;				
				}
				break;
	/*		case "status":
				//retVal = (thresh1.start === thresh2.start);
				var statusVal = thresh1.start;
				
				//If we get a status object (prop) then get the status objects value
				//otherwise its a normal comparison of two thresholds
				try{
					statusVal = thresh1.start.getValue();
				}catch(e){
					statusVal = thresh1.start;
				}
				retVal = (statusVal == thresh2.start);
				break;
	*/     case "string": case "itemName": case "itemType": case "linkName": case "linkType": case "relationshipName": case "relationshipType":				//if we 
				//if comparing property to threshold then check if it matches

				if(propComp){
					//retVal = (thresh1.start.match(thresh2.start));
					var val1 = thresh1.start;
					var val2 = thresh2.start;
					if(val1 && val2){
						var strContains = thresh1.start.indexOf(thresh2.start);
						if(strContains == -1){
							retVal = false;
						}else{
							retVal = true;
						}
					}else{
						//no value for one of the thresholds, so we say it doesnt match
						retVal = false;
					}
				}else{
					//else we are just comparing thresholds so whichever is last
					retVal = true;
				}
				break;			
			case "date": case "datetime": case "time":
				//we are comparing js Date objects
				var val1 = null;
				if(propComp){
					val1 = thresh1.start;
				}else{
					val1 = new Date(thresh1.start);
				}
				var val2 = new Date(thresh2.start);
				
				if(val1 && val2){
					retVal = val1 >= val2;
				}
				break;
			case "isodate": case "isodatetime": case "isotime":
				//we get iso string date formats, so first need to get the date objects
				var val1 = dojo.date.stamp.fromISOString(thresh1.start);
				var val2 = dojo.date.stamp.fromISOString(thresh2.start);
				
				if(val1 && val2){
					retVal = val1 >= val2;
				}else{
					retVale = false;
				}
				
				break;
			//default: basically what we dont support, always return false
			default:
				retVal = false;
		}

		return retVal;
	},
	
	
	//gets the threshold for a particular property
	_getPropThresholds: function(propId){
		var threshArray = this._tableWidgetDefinition.getDataSetThresholds();
		var thresholds = null;
		if(threshArray){
			thresholds = [];
			for(var i = 0; i< threshArray.length; i++){
				if(threshArray[i].attribute === propId){
					thresholds.push(threshArray[i]);
				}
			}
		}
		return thresholds;
	},
	
	//override pluggable view getState in order for pluggable view be able to return
	//the state of the table
	_getStateData: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		
		//Create data structure to hold state data like the visible view rect, etc.
		var stateData = {};
		if (this._grid) {
			var gLayout = this._grid.layout;
			var layoutData = new dojox.collections.Dictionary();
			
			if(gLayout.structure && (gLayout.structure.length > 1)){
	
				if(gLayout.structure[0] && gLayout.structure[1]){
					
					if(gLayout.structure[0].cells[0] && gLayout.structure[0].cells[0].length){
						//first column is selection column
						for(var i = 1; i < gLayout.structure[0].cells[0].length; i++){
							//for each column, get the field and add it to the sorted list
							//the columns are in order from the current table, so their index becomes our sort weight for the columns
							var k = gLayout.structure[0].cells[0][i].field;
							var val = gLayout.structure[0].cells[0][i];
							val.stateColumnWeight = i;
							val.isLocked = true;
							layoutData.add(k,val);
						}
					}
					
					if(gLayout.structure[1].cells[0] && gLayout.structure[1].cells[0].length){
						//first column is NOT selection column on second layout
						for(var j = 0; j < gLayout.structure[1].cells[0].length; j++){
							//for each column, get the field and add it to the sorted list
							//the columns are in order from the current table, so their index becomes our sort weight for the columns
							var k = gLayout.structure[1].cells[0][j].field;
							var val = gLayout.structure[1].cells[0][j];
							val.stateColumnWeight = j;
							val.isLocked = false;
							layoutData.add(k,val);
						}
					}
				}
	
			}else{
				if(gLayout.structure[0] && gLayout.structure[0].cells[0] && gLayout.structure[0].cells[0].length){
					//first column is selection column
					for(var i = 1; i < gLayout.structure[0].cells[0].length; i++){
						//for each column, get the field and add it to the sorted list
						//the columns are in order from the current table, so their index becomes our sort weight for the columns
						var k = gLayout.structure[0].cells[0][i].field;
						var val = gLayout.structure[0].cells[0][i];
						val.stateColumnWeight = i;
						val.isLocked = false;
						layoutData.add(k,val);
					}
				}
				
			}
			
			
			stateData.layoutData = layoutData;
			
			var filterData = this.getFilter();
			//create filter data with only valid schema data
			//since the TED object contains additional values
			var cullFilterData = null;
			if(filterData && filterData.length > 0){
				cullFilterData = {};
				cullFilterData.ruleArray = [];
				for(var j = 0; j < filterData.length; j++){
					//go thru each index and cull out the data we cant use
					var entry = filterData[j];
					var cullEntry = {};
					if(entry.column){
						cullEntry.column = entry.column;
					}
					if(entry.condition){
						cullEntry.condition = entry.condition;
					}
					if(entry.value){
						cullEntry.value = entry.value;
					}
					if(entry.type){
						cullEntry.type = entry.type;
					}
					//overwrite the current entry with the culled entry
					cullFilterData.ruleArray.push(cullEntry);
				}
				
				cullFilterData.ruleRelation = filterData.ruleRelation;
				
			}
			
			stateData.filterData = cullFilterData;
			
			var nestPlug = this._grid.plugin("NestedSorting");
			if(nestPlug){
				stateData.sortData = nestPlug.getSortProps();
			}
		}
			
		//Return state
		return stateData;
	},
	
	//public method to allow writing to widgetDef
	saveWidgetDelta: function(){
		

		
		if(this._widgetDefColumns){

			var stateData = this._getStateData();
			var tableWidgetDef = this._tableWidgetDefinition;

			//parse the data into the widgetDef reference if one exists
			if(this._tableWidgetDefinition && stateData){
				//id to use to find correct tables entry
				var delta = this._tableWidgetDefinition.getWidgetDelta("user");

				if(stateData.filterData && stateData.filterData.ruleArray && (stateData.filterData.ruleArray.length > 0 ) && delta){
					//set the filter in the delta
					var filterDefData = stateData.filterData;
					delta.setSectionAttribute({sectionKey: "table", attrKey: "filter", attrValue: filterDefData});
				}else{
					//remove this from the delta since there is no longer a filter
					delta.removeSectionAttribute({sectionKey: "table", attrKey: "filter"});
				}

				//always start with an empty array since we are providing a delta for each column
				//view in order to maintain the order of the columns
				var viewDelta = [];

				//for each column, check if we need to update any layout properties
				if(stateData.layoutData){
	
					//iterate on the layoutData
					var layouts = stateData.layoutData.getIterator();

					while(!layouts.atEnd()){
						var entry = layouts.get();
						
						//first get the existing widget def values for this view column
						var currEntry = tableWidgetDef.getTableViewElement(entry.key);
						//this will consolidate any previous merged delta data and allow for reordering
						
						if(currEntry){
							currEntry.hidden = entry.value.hidden;
							currEntry.width = entry.value.unitWidth;
							currEntry.lockColumn = entry.value.isLocked;
						
							//add the new merged entry, this resolves the order as well and is the reason we 
							//need to fully generate the each delta view each time
							viewDelta.push(currEntry);
						}
					}
			
					
					//now overwrite those into the view delta
					delta.setSectionAttribute({sectionKey: "table", attrKey: "view", attrValue: viewDelta});				
				}	

				//now handle the sort data and place into the delta
				if(stateData.sortData && (stateData.sortData.length > 0)){
					
					if(viewDelta && (viewDelta.length > 0)){
						//for each column in delta (note all columns)
						dojo.forEach(viewDelta, function(entryDelta, m){
					
							var matchFound = false;
							//find if there is a matching sort data
							dojo.forEach(stateData.sortData, function(entry, l){
								//first check for sort order
								if(entry.attribute === entryDelta.id){
									matchFound = true;
									//we found a matching entry, so update it
									if(entry.descending){
										viewDelta[m].sorted = "descending";
									}else{
										viewDelta[m].sorted = "ascending";
									}
									viewDelta[m].sortOrder = l;
								
								}
							});
							//check for no match...if no match then we need to remove the sorted data
							if(!matchFound){
								delete viewDelta[m].sorted;
								delete viewDelta[m].sortOrder;
							}
			
						});		
					}
					//now overwrite those into the view delta
					delta.setSectionAttribute({sectionKey: "table", attrKey: "view", attrValue: viewDelta});				

				}else{
					//NO sort data defined, so we need to remove the sort attributes from each of the view entries
					dojo.forEach(viewDelta, function(entry, n){
						if(entry.sorted){
							delete viewDelta[n].sorted;
						}
					});
				}

				//Call super class to actually do the save
				this.inherited(arguments);			
			}
		}else{
			console.log("Columns are not based on the widget definition.  Save is unecessary.");
		}
		
	},
	
	_exportFormatter: function(data, cell, rowIndex, object) {
		// NOTE: It would be nice to delegate the formatting to our property renderer like so:
		//
		//		return this._formatter(cell.field, item);
		//
		// But, unfortunately, we don't really know what Exporter is active (e.g., the CSV exporter might be calling us, and
		// we don't really want to spit out HTML tags to go into CSV.
		
		var cellData = null;
		
		// 52063
		// Dataset should never provide a null property object for a defined column,
		// but some data providers may have this edge case, so ensure we avoid an "object
		// undefined" error	
		if (object) {
			var propId = cell.field;
			switch(propId) {
			case "label":
				cellData = object.getLabel();
				break;

			case "sourceItem":
				if (object.getObjectType() === "relationship") {
					var sourceItem = object.getSourceItem();
					if (sourceItem) {
						cellData = sourceItem.getLabel();
					}
				}				
				break;

			case "targetItem":
				if (object.getObjectType() === "relationship") {
					var targetItem = object.getTargetItem();
					if (targetItem) {
						cellData = targetItem.getLabel();
					}
				}        			
				break;

			case "item1":
				if (object.getObjectType() === "link") {
					var item1 = object.getItem1();
					if (item1) {
						cellData = item1.getLabel();
					}
				}	
				break;

			case "item2":
				if (object.getObjectType() === "link") {
					var item2 = object.getItem2();
					if (item2) {
						cellData = item2.getLabel();
					}
				}			
				break;

			case "dataset1":
				if (object.getObjectType() === "link") {
					var d1 = object.getDataset1();
					var value = (d1 ? d1.getLabel() : "");
					cellData = value;
				}
				break;

			case "dataset2":
				if (object.getObjectType() === "link") {
					var d2 = object.getDataset2();
					var value = (d2 ? d2.getLabel() : "");
					cellData = value;
				}				
				break;

			default:
				var prop = object.getProperty(cell.field);
				if (prop){
					cellData = this._propertyRenderer.getPropertyDisplayValue(cell, prop, this);
				}
				break;
			}
		}
		
		if (!cellData) {
			cellData = "";
		}
		
		return cellData;
	},
	
	handleLinkClickAction: function(viewId, objectId, propId, objectType){
		
		console.log("handling link click for table view");
		var object;
		var property;
		
		//ask the dataset for the correct object based on the objects id.
		//since we have already rendered these items, the value should be
		//cached and should be quick.
		switch(objectType) {
			case "ibm.tivoli.navmodel.Item":
				object = this.dataset.getItem({itemId:objectId});
				break;
			case "ibm.tivoli.navmodel.Link":
				object = this.dataset.getLink({itemId:objectId});
				break;
			case "ibm.tivoli.navmodel.Relationship":
				object = this.dataset.getRelationship({itemId:objectId});
				break;
			default:
				console.log("unrecognized object type!");
		}	

		//object = this._grid.getItem(objectId);
		if(object){
			property = object.getProperty(propId);
		}
		
		
		if(object && property){
			this._propertyRenderer._handleLinkClick(property, this);
		}
	},

	_setTableView: function(columns) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// columns:
		//		
		
		//Process the columns to set the table view
		var view = [];

		//Process the list of view 
		if (columns) {
			for (var index = 0;  index < columns.length;  ++index) {
				var col = columns[index];
				
					//Deal with columns...
					var vData = {};
					if(col.isSortable()){		
						vData.sorted = (col.isSortAscending() ? "ascending" : "descending");
						vData.sortOrder =  col.getSortOrder();
					}else{
						vData.sorted = "none";
					}
					
					vData.searchable = col.isSearchable();
					vData.hidden = col.isHidden();
					vData.id = col.getId();
					vData.resizeable = col.isResizeable();
					vData.lineWrap = col.isLineWrap();
					
					if(!col.getWidth()) {
						vData.width = "10em";
					} else {
						vData.width = col.getWidth();
						//just in case the with still comes back with -1
						if(vData.width == -1){
							vData.width = "10em";
						}
					}
					vData.lockColumn = col.isLocked();
	
					//Store the object to pass to the bridge later
					view.push(vData);
			} 
			
			this._tableWidgetDefinition.setTableView(view);
		}
	},

	
	_processDoubleClick: function(evt) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// evt:
		//		
		if (this._trace.isFinest()) {
			this._trace.finest("*** row was double clicked");
		}
		
	    var row = this._grid.model.byId(evt.rowId).item;    // 49426 Gridx
        var rowIndex = evt.rowId;   // 49426  Gridx
	    // console.log ("row is: " + row);
	    // console.log ("evt.rowIndex: " + rowIndex);

	    if ( row ) {
	    	//select the row that the context menu was initiated on (unless already selected)
	    	// 49426  if (this._pTableClassName.indexOf("tree") > 0) {  // hack until both table and tree table are using gridx
	    	if (!this._grid.select.row.isSelected(rowIndex)) {
    			this._rightClickClearEvent = true;
	    		this._grid.select.row.clear();
	    		this._grid.select.row.selectById(rowIndex);
	    	}
	    	// 49426  }
	    	// 49426  else {
	    	// 49426  	if (!evt.grid.selection.isSelected(rowIndex)) {
	    	// 49426		evt.grid.selection.deselectAll( rowIndex );
	    	// 49426		evt.grid.selection.setSelected( rowIndex, true );
	    	// 49426	}
	    	// 49426  }
	    }
	    
		//need to set the selection on the row
		var selection = null;
		
		   //Use the DefaultActionHelper to create the launch the default double click action
			if(!this.daHelper){
				//Create DefaultActionHelper
				//By default creates the properties Dialog and launches it (via callback)
				this.daHelper = new ibm.tivoli.tip.navwidget.topology.dijit.DefaultActionHelper( 
					{
						defaultActionCallback: dojo.hitch(this, "_launchPropertiesDialog", this)
					}
			);

		}
		//actually execute the default action/task (defined in constuctor)
		this.daHelper.executeDefaultAction(this);
	},
	
	_launchPropertiesDialog: function(pluggableView){
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// pluggableView:
		//		
		
		//NOTE: Reusing Anthony's MenuProviderHelper to launch the dialog to avoid duplicate logic. This is a temporary solution. 
		var menuProviderHelper = new ibm.tivoli.tip.navwidget.topology.dijit.MenuProviderHelper();
		menuProviderHelper.getMenuCommand("nwmi_PropertiesNotebookDialog", pluggableView, null);

	},
	//Latha: 12Oct2012 : 52876 :TBSM 35062: Table: Filter doesn't work in JazzSM Driver16
	_translateQuery: function(commands) {
		// console.debug("_translateQuery method Start");
		this.query = {ids: {}, expression: ""};	
		if (commands) {
			this._translate(this.query, commands.data, commands.op, "");	
			this._condition = this.query.expression;
			// console.debug("this._condition : ....");
			// console.debug(this._condition); 
		}
		else {
			this._condition = null;
		}

		//fire onFilterChange event in NavWidgetPluggableView
		this.onFilterChange();
		// console.debug("_translateQuery method End"); 
		// console.debug("this.query : ....");
		// console.debug(this.query);
		return this.query;
	},

	_translate: function(query, data, op, prevOp) {
			if (dojo.isArray(data)) {			
				var length = data.length;

				if (length > 1) {
					query.expression += "(";
				}
				
				var columns = this._grid.structure;
				var fields=[];
				for (var i=0 in columns) {
					var column=columns[i];
					var size=dojo.trim(column.field).length;
					if(size > 0){
						fields.push(column.field);
					}					
				}
				// console.debug("fields ...");
				// console.debug(fields);
				for (var index = 0;  index < length;  ++index) {
					if (!this._translate(query, data[index].data, data[index].op, op)) {
						if (data[index].isCol) {
							var field;
							if(fields.indexOf(data[index].data)>-1) {
								field=data[index].data;
							} else {
								field = fields[parseInt(data[index].data-1)]
							}
								query.ids[field] = true;
								// If field is enumerated string, prepend condition with *	
								for (var i = 0; i < this._stringFieldsWithPossibleValues.length; i++) {
									if (this._stringFieldsWithPossibleValues[i] === data[index].data) {
										data[index].data = "*"+data[index].data;
										break;
									}
								}					
								query.expression += field;
								query.expression += " ";
								query.expression += this._translateOperator(op, prevOp);
						}
						else {
							if (data[index].op == "string") {
								// replace single quotes with 2 consecutive single quotes
								query.expression += " '" + data[index].data.replace(/'/g, "''") + "'";
							}
							else {
								if (data[index].data) {
									query.expression += " " + data[index].data;
								} else {
									query.expression += " " + data[index];
								}
							}						
						}
					}
					// and/or operator in between
					else if (index + 1 < length) {
						query.expression += " " + (op) + " ";					
					}
				}

				if (length > 1) {
					query.expression += ")";
				}

				return true;
			}	
		else {
			return false;	
		}
	},

	_translateOperator: function(op1, op2) {
		var newOp = "";		
		switch(op1) {
			case "equal":
				newOp = (op2 === "not" ? "!=" : "=");
				break;
			case "less":
				newOp = (op2 === "not" ? ">=" : "<");
				break;
			case "lessEqual":
				newOp = (op2 === "not" ? ">" : "<=");
				break;
			case "greater":
				newOp = (op2 === "not" ? "<=" : ">");
				break;
			case "greaterEqual":
				newOp = (op2 === "not" ? "<" : ">=");
				break;
			case "contain":
				newOp = (op2 === "not" ? "!contains" : "contains");
				break;
			//LS- 55245 - This is not supported by gridx	
			/*case "containsignorecase":
				newOp = (op2 === "not" ? "!contains" : "contains");
				break;*/
			case "startWith":
				newOp = (op2 === "not" ? "!starts" : "starts");
				break;
			case "endWith":
				newOp = (op2 === "not" ? "!ends" : "ends");
				break;
			case "isEmpty":
				newOp = (op2 === "not" ? "!isnull" : "isnull");
				break;
			default:
				alert('unrecognized operator=' + op1);
		}

		return newOp;
	},

	_translateOperatorInverse: function(op1) {
		var newOp = "";		
		switch(op1) {
			case "contains":
				newOp = "contains";
				break;
			case "!contains":
				newOp = "notContains";
				break;
			case "starts":
				newOp = "startsWith";
				break;
			case "!starts":
				newOp = "notStartsWith";
				break;
			case "ends":
				newOp = "endsWith";
				break;
			case "!ends":
				newOp = "notEndsWith";
				break;
			case "isnull":
				newOp = "isEmpty";
				break;
			case "!isnull":
				console.log('Operator is not supported in filter dialog: ' + op1);
				break;
			case "=":
				newOp = "equalTo";
				break;
			case "!=":
				newOp = "notEqualTo";
				break;
			case "<":
				newOp = "lessThan";
				break;
			case ">":
				newOp = "largerThan";
				break;
			case ">=":
				newOp = "largerThanOrEqualTo";
				break;
			case "<=":
				newOp = "lessThanOrEqualTo";
				break;
			default:
				alert('unrecognized operator=' + op1);
		}

		return newOp;
	},

	_createDefaultColumnObject: function(columnIds, columns, cols, defColumn) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// columnIds:
		//		
		// columns:
		//		
		// cols:
		//		
		// defColumn:		
		//		
		var targetColData = {};							
		targetColData.id = defColumn.id;
		targetColData.label = {};
		targetColData.label.value = defColumn.label; 
		targetColData.type = "java.lang.String";						
		// get column IDs to pass to store
		columnIds.unshift(targetColData.id);
		columns.unshift(targetColData);

		// Create a column object and add it to the returned cols
		var newTargetColumnArgs = {
				version: 0.1
		};
		var newColumn = new ibm.tivoli.navmodel.Column(newTargetColumnArgs);

		newColumn.setId(defColumn.id);
		newColumn.setLabel(defColumn.label);
		newColumn.setValueType(defColumn.type);
		newColumn.setSortOrder(defColumn.sortOrder !== undefined ? defColumn.sortOrder : -1);
		newColumn.setSortable(true);
		newColumn.setSearchable(true);							
		newColumn.setFilterable(true);
		newColumn.isFiltered(false);
		newColumn.setSortAscending(true);							
		newColumn.setWidth(-1);

		cols.unshift(newColumn);							

	},
	
	isMenuOpen: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		if(this._contextMenuAdapter){
			return (this._contextMenuAdapter.getMenu()) ? true : false;
		}else{
			return false;
		}
	},
	
	// 54947
	suppressTooltip: function() {
		return this._suppressTooltip;  // return boolean flag
	},
	
	_openActionMenu: function(e){
		var childItems = this._table._toolbar.actionListWidget.dropDown.getChildren();
	
		if (childItems.length === this._actionMenuDefaultCount) {
			var processingLabel = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources").PROCESSING;
			items = [
						//new dijit.MenuSeparator(),
						{ 
	        				"label": processingLabel,
	        				"iconClass": "tipMenuIcon loadingIcon", 
	        				"actionID": "LOADING_ID",
	        				"launchType": "NOOP"
	        			}
        			];
			this.cmProvider.getMenuItemsAsynch(this, dojo.hitch(this, this._menuItemsLoaded), "menu,toolbar");
		}
		else {
			this._removeLoadingItem();
			items = this.cmProvider.getMenuItems(this);
		}
		
		this._addItemsToMenu(this._table, items);
		
	},
	
	_addItemsToMenu: function(parent, items) {		
		// summary:
		//		Recursive function to construct menu from array of items
		//		(adapted from ibm.tivoli.tip.dijit.ContextMenuAdapter)
		// description:
		//		Coming soon
		// parent:
		//		Coming soon
		// items:
		//		Coming soon

		// Note:  actions menu items get added in bottom to top order.  
	    for (var i = items.length-1; i >= 0; --i) {
	    	this._handleAddingMenuItem(parent, items[i]);  // 53516      
	    }
	},	
	
	_addItemsToSubmenu:function(parent, items) {	// 53516
		// summary:
		//  	Recursive function to construct a submenu from array of items
		//  	(adapted from ibm.tivoli.tip.dijit.ContextMenuAdapter)
		// description:
		//  	Coming soon
		// parent:
		//  	Coming soon
		// items:
		//  	Coming soon
		  
		// Note:  submenu items get added in top to bottom order.
		for (var i in items) {
			this._handleAddingMenuItem(parent, items[i]);
		}
	},	
	
	_handleAddingMenuItem: function(parent, item) {  // 53516
		// summary:
		//  	Function to build the menu item to add to a parent, either menu or
		//      submenu
		//  	(adapted from ibm.tivoli.tip.dijit.ContextMenuAdapter)
		// description:
		//  	Coming soon
		// parent:
		//  	Coming soon
		// item:
		//  	Coming soon		
		var params;

		if (item.submenu_items) {
			//subMenuIndex = this._subMenus.length;
			//this._subMenus[subMenuIndex] = new dijit.Menu();
			var subMenuItem = new dijit.Menu();
			params = 
			{
				popup: subMenuItem
			};

			dojo.mixin( params, item );
			var popMenu = new dijit.PopupMenuItem(params);

			this._addItemToMenu(parent, popMenu);
			this._addItemsToSubmenu(subMenuItem, item.submenu_items);   		
		}
		else {
			// The following if clause added for backward compatibility
			// with the previous version of TIPContextMenu
			if (item.id && item.id.toLowerCase() === "separator") {
				// Previous TIPContextMenu users indicated separators via the id
				item.actionID = "_separator_";
			} 
			else if (item.id && !item.actionID) {
				// Previous TIPContextMenu users didn't use actionID
				item.actionID = item.id;
			}

			if (item.actionID === "_separator_") {
				this._addItemToMenu(parent, new dijit.MenuSeparator());
			} 
			else {
				params = {
					adapter: this,
					onClick: function() {this.adapter._onMenuItemClick(this); }
				};  			 
				dojo.mixin(params, item);
				this._addItemToMenu(parent, new dijit.MenuItem(params));
			}
		}
	},	
	
	_addItemToMenu: function(parent, menuItem) {   // 53516
		// summary:
		//  	Function to add a menu item to a parent, either menu or
		//      submenu
		//  	(adapted from ibm.tivoli.tip.dijit.ContextMenuAdapter)
		// description:
		//  	Coming soon
		// parent:
		//  	Coming soon
		// menuItem:
		//  	Coming soon	
		
		if((parent instanceof dijit.Menu)) {  
			//passed a recursive menu item, so add it to that directly
			parent.addChild(menuItem); 
		} else {
			//need to add to the table toolbar
			this._table.addToActionList(menuItem);
		}
	},	
	
	_onMenuItemClick: function(menuItem) { 		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// menuItem:
		//		Coming soon
	   this.cmProvider.getMenuCommand(menuItem.actionID, this, menuItem);
	},
	
 	_removeLoadingItem: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
	    var childItems = this._table._toolbar.actionListWidget.dropDown.getChildren();
 	    var loadingItem = childItems[0];
 	    this._table._toolbar.actionListWidget.dropDown.removeChild(loadingItem);
 	 	loadingItem.destroy();
 	},
 	_menuItemsLoaded: function(/*Boolean*/ success) {		

 		if (success) {
 			// repopulate menu
 			this._openActionMenu();
 			// the menu is already marked as started and open, so loadDropDown skips these
 			// steps, so do them again here
 			dojo.forEach(this._table._toolbar.actionListWidget.dropDown.getChildren(), function(child){ child.startup(); });
 			this._table._toolbar.actionListWidget.dropDown.startupKeyNavChildren();
 			// The dijit.Menu still remembers the "Loading..." item as focused so reset
 			this._table._toolbar.actionListWidget.dropDown.focusedChild = null;
 			this._table._toolbar.actionListWidget.dropDown.focus();
 		}
 	    else {
 	    	// There were either no tasks returned or an error occurred, so dismiss the dropDown
 	    	//this.closeDropDown(true);
 	    }
 	},
	
	_closeActionMenu: function(e){

		var childItems = this._table._toolbar.actionListWidget.dropDown.getChildren();
		
		while(childItems.length > this._actionMenuDefaultCount) {
			this._table.removeFromActionList(childItems[0]);
			childItems = this._table._toolbar.actionListWidget.dropDown.getChildren();
		}
	},
	
    _onRowContextMenu: function( e ) {
		// summary:
		//		This method is called each time a context menu is required on a row
		// description:
		//		Coming soon
		// e: 
		//		
    	var row = this._grid.model.byId(e.rowId).item;  // 49426  Gridx
    	var rowIndex = e.rowId;    // 49426  Gridx
    	// console.log ("row is: " + row);
    	// console.log ("e.rowIndex: " + e.rowIndex);
    	// console.log ("e.rowId: " + e.rowId);

    	if ( row ) {
    		//select the row that the context menu was initiated on (unless already selected)
    		// 49426  if (this._pTableClassName.indexOf("tree") > 0) {  // hack until both table and tree table are using gridx
    		if (!this._grid.select.row.isSelected(rowIndex)) {
    			this._rightClickClearEvent = true;
    			this._grid.select.row.clear();
    			this._grid.select.row.selectById(rowIndex);
    		}
    		// 49426  }
    		// 49426  else {
    		// 49426	if (!e.grid.selection.isSelected(rowIndex)) {
    		// 49426		e.grid.selection.deselectAll( rowIndex );
    		// 49426		e.grid.selection.setSelected( rowIndex, true );
    		// 49426	}
    		// 49426  }
    		
    		if(!this.removeContextMenu){
    			//dismiss tooltip if there is one
    			this._table.closeTooltip();
    			this.getContextMenuAdapter().contextMenuOpen( e, this);
    		}
    		
    		/*      // Get the parentPluggableView if it has not been initialized before
				var parentPluggableView = this.tableBuilder.params.gridParms.parentPluggableView;
   	 			if(!parentPluggableView.removeContextMenu){
   		 			//only open the menu if we are not told to remove it
   		 			this.getContextMenuAdapter().contextMenuOpen( e, parentPluggableView);        
   	 			}
    		*/
    	}
    },      
    
    getContextMenuAdapter: function() {	  		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
      if (!this._contextMenuAdapter) {
         // Create the contextMenuAdapter.
         this._contextMenuAdapter =         	 
             new ibm.tivoli.tip.navwidget.dijit.AsynchContextMenu({cmProvider: this.cmProvider});
      }
      return this._contextMenuAdapter;
    },


	_mapDataType: function(dataType){
		switch( dataType ){
			case "byte": case "short":  case "int": case "long": case "ushort": case "ubyte": case "uint": case "ulong": case "float": case "double": case "percentage" : case "java.lang.Integer": case "java.lang.Double": case "java.lang.Long": case "java.lang.Decimal":
				return "number";
			case "status":
				return "number";
			case "boolean": case "java.lang.Boolean": 
				return "boolean";
			case "date": case "datetime": case "isodate": case "isodatetime": case "java.lang.Date": case "java.util.Date": 
				return "date";
			case "time": case "isotime" :
				return "time";
			default :
				return "string";
			


		}
	},

	_isColumnFilterSupported: function(dataType) {
		switch( dataType ){
			case "isoduration": case "isointerval":
				return false;
			default:
				return true;
		}
	},
	
	/////////////////////////////////////////////////
	// Selection change handling
	/////////////////////////////////////////////////
	_handlingSelectOrDeselectAll: false,
	_handlingSelectionChangedCallback: false,
	
/*	49426  No longer needed...unless used by some deployer subclass (remove?)
	_gridSelectionChangedCallback: function() {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		this._handlingSelectionChangedCallback = true;
		
		if (!this.selection.isChanging() && ! this._handlingSelectOrDeselectAll) {
			//Get the selected rows... will be an array of Client Dataset Items
            var selectedRows = [];
            var rowIds = [];
            if (this._grid.selection) {  // DataGrid case
            	selectedRows = this._grid.selection.getSelected();
            }
            else {  // gridx case
               rowIds = this._grid.select.row.getSelected(); 
               dojo.forEach(rowIds, function(rowId) {
            	   		selectedRows.push(this._grid.model.byId(rowId).item);
            	   }, this);
            }

			//Set the selection on the view's selection object
            //ADM TODO: maybe add a case statement here based on type? where we can decide where to set what type of objects selected here...not sure
			this.selection.setSelectedObjects(selectedRows, null, null, rowIds);
		}
		
		this._handlingSelectionChangedCallback = false;
	},
*/
	
    // 49426  Gridx
    _gridXSelectionChangedCallback: function() {
		this._handlingSelectionChangedCallback = true;

		if (!this.selection.isChanging() && ! this._handlingSelectOrDeselectAll) {
			//Get the selected rows... will be an array of Client Dataset Items
			var selectedRows = [];
			var rowIds = [];
			rowIds = this._grid.select.row.getSelected();
			dojo.forEach(rowIds, function(rowId) {
					selectedRows.push(this._grid.model.byId(rowId).item);
				}, this);
			// Update the selection on the view's selection object
			this.selection.updateSelectedObjects(selectedRows, rowIds);
			
			//asahlin begin
			// if the toolbar is shown, check for updates to toolbar tasks
			if(!this.hideToolbar && this._toolbarButtonExist && this._rightClickClearEvent == false){
				this._loadToolbarTasks(dojo.hitch(this, this._toolbarTasksLoaded));
			} else {
				this._rightClickClearEvent = false;
			}
			//asahlin end
		}

		this._handlingSelectionChangedCallback = false;
    }
});


ibm.tivoli.tip.navwidget.table.dijit.AbstractTableView.toolbarItemTypes = {		
		// summary:
		//		definitions for common toolbar item types values
		// description:
		//		Coming soon		
		PRINT_BUTTON:		"PRINT_BUTTON",
		REFRESH_BUTTON:		"REFRESH_BUTTON",
		SEPARATOR:			"SEPARATOR",
		CUSTOM_BUTTON:		"CUSTOM_BUTTON",
		CUSTOM_TEXTBOX:		"CUSTOM_TEXTBOX",
		CUSTOM_COMBO: 		"CUSTOM_COMBO",
		CUSTOM_DROPDOWN_BUTTON: "CUSTOM_DROPDOWN_BUTTON",
		ACTION_MENU:		"ACTION_MENU",		
		VIEW_TOGGLER:		"VIEW_TOGGLER",
		CONFIG_TABLE:		"CONFIG_TABLE",
		EXPORT_BUTTON: 		"EXPORT_BUTTON"
	  };
});

},
'ibm/tivoli/tip/navwidget/dijit/NavWidget':function(){
// wrapped by build app
define("ibm/tivoli/tip/navwidget/dijit/NavWidget", ["dijit","dojo","dojox","dojo/require!dijit/_Widget,dijit/_Container,dijit/_Templated,dijit/form/Button,dijit/Toolbar,dijit/layout/BorderContainer,dijit/layout/StackContainer,ibm/tivoli/tip/dijit/BreadcrumbBar,ibm/tivoli/tip/dijit/Breadcrumb,ibm/tivoli/tip/dijit/BreadcrumbSeparator,ibm/tivoli/navmodel/NavWidgetDef,ibm/tivoli/tip/navwidget/topology/dijit/MenuProviderHelper,ibm/tivoli/tip/navwidget/dijit/NavWidgetMenuProvider,ibm/tivoli/tip/navwidget/dijit/NavWidgetPluggableView,ibm/tivoli/tip/navwidget/dojo/NavWidgetState,ibm/tivoli/tip/navwidget/dojo/NavWidgetUtil,ibm/tivoli/navmodel/Trace,dojox/collections/Dictionary,ibm/tivoli/tip/navwidget/dojo/Timer"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.dijit.NavWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Container");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("dijit.Toolbar");
dojo.require("dijit.layout.BorderContainer");
dojo.require("dijit.layout.StackContainer");
dojo.require("ibm.tivoli.tip.dijit.BreadcrumbBar");
dojo.require("ibm.tivoli.tip.dijit.Breadcrumb");
dojo.require("ibm.tivoli.tip.dijit.BreadcrumbSeparator");

dojo.require("ibm.tivoli.navmodel.NavWidgetDef");
dojo.require("ibm.tivoli.tip.navwidget.topology.dijit.MenuProviderHelper");
dojo.require("ibm.tivoli.tip.navwidget.dijit.NavWidgetMenuProvider");    // 25005  For Dojo 1.7, this was split out to its own file
dojo.require("ibm.tivoli.tip.navwidget.dijit.NavWidgetPluggableView");
dojo.require("ibm.tivoli.tip.navwidget.dojo.NavWidgetState");
dojo.require("ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil");
dojo.require("ibm.tivoli.navmodel.Trace");
dojo.require("dojox.collections.Dictionary");
dojo.require("ibm.tivoli.tip.navwidget.dojo.Timer");
ibm.tivoli.tip.navwidget.dijit.NavWidget._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.tip.navwidget.dijit.NavWidget");


dojo.declare ("ibm.tivoli.tip.navwidget.dijit.NavWidget",
	[dijit.layout.BorderContainer, ibm.tivoli.tip.navwidget.dijit.NavWidgetPluggableView],
	{		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
	
		//***************************
		// "Public"
		//***************************
		
		// homeDataset:
		//		Home dataset
		homeDataset: null, 

		// cmProvider:
		//		Custom context menu provider (must be provided on construction to be used)
		cmProvider: null,	
		
		// daHelper:
		//		
		daHelper: null,
		
		// includeBreadcrumbs: Boolean
		//		
		includeBreadcrumbs: true,
		
		gutters: false,
		
		//***************************
		// "Private"
		//***************************
		
		// _navWidgetStateStack: [private]
		//		A stack of datasets
		_navWidgetStateStack: null,

		// _breadcrumbBar: [private]
		//		
		_breadcrumbBar: null,
		
		// _centerContainer: [private]
		//		
		_centerContainer: null,

		// _trace: [private] Trace
		//		Utility for trace and logging
		_trace: ibm.tivoli.tip.navwidget.dijit.NavWidget._trace, 
				
		// _constants: [private const] Object
		//		
		_constants: 
			{
				DEFAULT_WIDGET_DEF_INFO: {
					className: "ibm.tivoli.navmodel.NavWidgetDef",
					baseUrl: "/ibm/tivoli/rest",
			    	provider: "any",
			    	datasource: "any",
			    	widget: "navwidget",
	            	definition: "DefaultNavWidget"
				}
			},

		constructor: function () {		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
								
			this._navWidgetStateStack = [];
		},
		
		destroy: function(){		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			
			//Destroy all of the dataset's on the dataset stack
			this._clearStateStack(true);

			//Call superclass
			this.inherited(arguments);		
		},
		
		_changeEventListeners: [],
		
		_dispatchChangeEvent: function (viewId) {
			for (var i = 0; i < this._changeEventListeners.length; i++) {
				var listener = this._changeEventListeners[i];
				if (listener.handle) {
					listener.handle(viewId);
				}
			}
		},
		
		addChangeEventListener: function (listener) {
			// listener should be an object with handle(viewId) method
			this._changeEventListeners.push(listener);
		},
		
		_clearStateStack: function(inDestroy){		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			
			//Destroy all of the state on the state stack
			for (var i = this._navWidgetStateStack.length; i > 0; i--) {
				this._removeLastState(inDestroy);
			}		
		},

		postCreate: function(){		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			
			//Call superclass
			this.inherited(arguments);
		},
		
		startup: function(){		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			
			//Call superclass
			this.inherited(arguments);
		},
		
		_createCMProvider: function(){		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			
			//Create the menu provider that we'll pass off to the pluggable views
			if (!this.cmProvider) {
				var helper = new ibm.tivoli.tip.navwidget.topology.dijit.MenuProviderHelper();
				this.cmProvider = new ibm.tivoli.tip.navwidget.dijit.NavWidgetMenuProvider({
	                				view: this,
	                				getMenuItemsCallback: dojo.hitch(helper, helper.getMenuItems),
	                				getMenuCommandCallback: dojo.hitch(helper, helper.getMenuCommand)
				});
			}
			
			this.inherited(arguments);
		},
		
		_createSupportingWidgets: function(){
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		protected
			
			//Create the breadcrumbs
			if (this.includeBreadcrumbs) {
				this._createBreadcrumbBar();
				//Add breadcrumbs to the container
				this.addChild( this._breadcrumbBar );
			}
		},
		
		_setDatasetCreatePluggableViewsAugmentViewAttributes: function(view, viewDef) {	
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		protected
			
			if (this.widgetDefinition) {
				var viewsLayout = this.widgetDefinition.getViewsLayout();
				if (!viewsLayout) {  //We'll assume "border" is the default if viewsLayout is undefined
					view.attr("region", "center");
				} else if (viewsLayout == "border") {
					if (viewDef.layoutStyle) {
						//region
						if (viewDef.layoutStyle.region) {
							view.attr("region", viewDef.layoutStyle.region);
						} else {
							view.attr("region", "center");
						}
						//splitter
						if (viewDef.layoutStyle.splitter) {
							view.attr("splitter", viewDef.layoutStyle.splitter);
						}
						//height
						if (viewDef.layoutStyle.height) {
							view._setStyleAttr({height: viewDef.layoutStyle.height});
						}
						//width
						if (viewDef.layoutStyle.width) {
							view._setStyleAttr({width: viewDef.layoutStyle.width});
						}
					};
				} else { //Stack
					view._setStyleAttr({height: "100%", width: "100%"});
				}
			}
		},
		
		_addPluggableViewToContainer: function(view) {
			if (!this._centerContainer) {
				//Create a BorderContainer or StackContainer for the center region, depending on the viewsLayout
				var viewsLayout = this.widgetDefinition ? this.widgetDefinition.getViewsLayout() : "border"; //We'll assume "border" is the default if viewsLayout is undefined
				var centerContainerArgs = {
						id: this.id + "_centerContainer",
						region: "center",
						gutters: false
						//style: "height:100%;width:100%" //Remove per 250692
				};
				if (viewsLayout == "border") {  
					this._centerContainer = new dijit.layout.BorderContainer(centerContainerArgs);
				} else {
					this._centerContainer = new dijit.layout.StackContainer(centerContainerArgs);
				}
				this.addChild(this._centerContainer);	
			}
			this._centerContainer.addChild(view);

		},
		
		_removePluggableViewFromContainer: function(view) {
			if (this._centerContainer) {
				this._centerContainer.removeChild(view);
			}
		},
		
		makePluggableViewVisible: function(viewId) {
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		public
			
			this._dispatchChangeEvent(viewId);
			
			if (this._centerContainer && this._centerContainer.isInstanceOf(dijit.layout.StackContainer)) {
				var pluggableView = this.getPluggableView(viewId);
				if (pluggableView) { //Make sure pluggable view exists
					var dataset = this.getCurrentDataset();
					if (pluggableView.isApplicable(dataset)) {
						this._centerContainer.selectChild(this.id + "_" + viewId);
					} else {
						if (this._trace.isError()) {
							this._trace.error("View is not applicable for dataset; viewId = " + viewId, "makePluggableViewVisible");
						}
					}
				} else {
					if (this._trace.isError()) {
						this._trace.error("Cannot find pluggable view; viewId = " + viewId, "makePluggableViewVisible");
					}
				}
			} else {
				this.inherited(arguments);
			}
		},
		
		getVisiblePluggableView: function(viewId) {
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			// viewId:
			//		
			
			if (this._centerContainer && this._centerContainer.isInstanceOf(dijit.layout.StackContainer)) {
				this._centerContainer.selectedChildWidget;
			} else {
				return this.inherited(arguments);
			}
		},
		
		_initializeWithDataset: function(){
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		protected
			
			//Initialize the home dataset
			if (this.homeDataset || this.initializeWithNullDataset) {
				this.setHomeDataset(this.homeDataset);
			}
			
			//Special case for nav widget, don't call superclass
			//this.inherited(arguments);
		},
		
		//***************************
		// "Public"
		//***************************
		setDataset: function(dataset, datasetWidgetDefinition) {		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			// dataset:
			//		
			
			if (this._navWidgetStateStack.length == 1) {
				this.setHomeDataset(dataset, datasetWidgetDefinition);
			} else { 
				this._addOrNavigateHelper(dataset, datasetWidgetDefinition, true, true);
			}
		},
		
		setHomeDataset: function(dataset, datasetWidgetDefinition) {		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			// dataset:
			//		

			//Clear out the state stack
			this._clearStateStack();
			
			//Store ref to home dataset and add to stack
			this.homeDataset = dataset;
			this.addDataset(this.homeDataset, datasetWidgetDefinition);
		},
		
		addDataset: function(dataset, datasetWidgetDefinition){	
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			// dataset:
			//

			//Let's do some timing
			if (this._trace.isFinest()) {
				//Start a timer to see how long it takes
				var addDatasetTimer = new ibm.tivoli.tip.navwidget.dojo.Timer();
				addDatasetTimer.start();
			
				//Set up callback that will hide the busy pane
		    	this._handle = dojo.connect(this, "onSetDataset", this, function(dataset) {
		    		this._trace.finest("addDataset took ; " + addDatasetTimer.stopAndGetTraceString(), "addDataset");
					dojo.disconnect(this._handle);});
			}
			
			//Kick off the adding of the dataset
			this._addOrNavigateHelper(dataset, datasetWidgetDefinition, false, true);
		},
		
		_addOrNavigateHelper: function(dataset, datasetWidgetDefinition, removeLastState, createNewState){	
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			// dataset:
			//

			//I want to basically do what setDataset in superclass does, but need to consider a couple of 
			//special cases related to whether we need to remove the last state or create a new state
			this._setDatasetCleanUpAfterLastDataset(removeLastState, createNewState);
			this._setDatasetInit(dataset);
			this._setDatasetSetWidgetDefs(dataset, datasetWidgetDefinition);
			this._setDatasetCreatePluggableViews();
			this._setDatasetProcessWidgetDefs(dataset);
			this._setDatasetLoadData(dataset, removeLastState, createNewState);
		},
		
		_setDatasetCleanUpAfterLastDataset: function(removeLastState, createNewState) {
			if (!removeLastState && createNewState) { //Really only need to save the previous view state if we're going to be creating a new state
				var oldState = this._getCurrentNavWidgetState();
				if (oldState) {
					//Tuck away some information we'll want to restore if we come back to this state
					oldState.selectedItems = this.selection.getItems();
					oldState.selectedLinks = this.selection.getLinks();
					
					//Get states from all the views, so that we can restore them if user comes back
					var iterator = this.getPluggableViewIterator();
					while (!iterator.atEnd()) {
						var view = iterator.get().value;
						
						var viewState = view.getState();
						if (viewState) {
							oldState.addPluggableViewState(view.viewId, viewState);
						}
					}
				}
			}
			
			//Call superclass
			this.inherited(arguments);
		},
		
		_setDatasetInit: function(dataset) {
			//Call superclass
			this.inherited(arguments);
		},
		
		_setDatasetSetWidgetDefs: function(dataset) {
			//Call superclass
			this.inherited(arguments);
		},
		
		_setDatasetProcessWidgetDefs: function(dataset) {
			//Call superclass
			this.inherited(arguments);
		},
		
		
		_setDatasetLoadData: function(dataset, removeLastState, createNewState) {
			//Remove the current state if not doing an addDataset

			if (removeLastState) {
				this._removeLastState();
			}
			
			if (createNewState) {
				//Create a new state to add to the stack based on the dataset
				var navWidgetState = new ibm.tivoli.tip.navwidget.dojo.NavWidgetState({
					dataset: dataset
				});
				
				//Push the new dataset onto the end of the stack
				this._navWidgetStateStack.push(navWidgetState);
				
				//Add new dataset to breadcrumb bar
				if (this._breadcrumbBar) {
					//Add separator before next step
					if (this._navWidgetStateStack.length > 1) {
						this._breadcrumbBar.addChild(new ibm.tivoli.tip.dijit.BreadcrumbSeparator());
					}
					
					//Add new step
					var newStep = new ibm.tivoli.tip.dijit.Breadcrumb({label: this._getBreadcrumbLabel(dataset, true),
																		title: this._getBreadcrumbAltTitle(dataset)});
					navWidgetState.breadcrumbEntry = newStep;
					this._breadcrumbBar.addChild(newStep);
					this._cleanBreadcrumbLabels();
					var currentNavWidgetStateStackIndex = this._navWidgetStateStack.length - 1;
					dojo.connect(newStep, 'onClick', dojo.hitch(this, function() {
															this.navigateToDatasetInStack(currentNavWidgetStateStackIndex);
														}));
				}
			}
			
			//Call superclass
        	this.inherited(arguments);
		},
		
		_getBreadcrumbLabel: function(dataset, isLastEntry) {
			var label = null;
			if (dataset) {
				label = dataset.getLabel();
				if (isLastEntry) {
					var labelQualifier = dataset.getLabelQualifier();
					if (labelQualifier) {
						var messageTemplate = this.commonResources.PARENTHETICAL_ENTRY;
				        label = dojo.string.substitute(messageTemplate,
	                                                   {label:label,
	                                                   	parens:labelQualifier});
					}
				}
			}
			return label;
		},
		
		_getBreadcrumbAltTitle: function(dataset) {
			return this._getBreadcrumbLabel(dataset, true);
		},
		
		_cleanBreadcrumbLabels: function() {
			var navWidgetStateStackSize = this._navWidgetStateStack.length;
			for (var i = 0; i < navWidgetStateStackSize; i++) {
				var state = this._navWidgetStateStack[i];
				var isLast = (i == navWidgetStateStackSize - 1);
				var newLabel = this._getBreadcrumbLabel(state.dataset, isLast);
				var newAltTitle = this._getBreadcrumbAltTitle(state.dataset);
				state.breadcrumbEntry.set("label", newLabel);
				state.breadcrumbEntry.set("title", newAltTitle);
			}
		},
		
		_setDatasetComplete: function(/*ibm.tivoli.navmodel.Dataset*/dataset){
			//Deal with the dataset in the (now) current nav state
			this._applyCurrentNavState();
		},
		
		getCurrentDataset: function(){		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			var navState = this._getCurrentNavWidgetState();
			if (navState) {
				return navState.dataset;
			} else {
				return null;
			}
		},
		
		_getCurrentNavWidgetState: function(){		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			var navWidgetStateStackSize = this._navWidgetStateStack.length;
			if (navWidgetStateStackSize > 0) {
				return this._navWidgetStateStack[navWidgetStateStackSize - 1];
			} else {
				return null;
			}
		},

		navigateToDatasetInStack: function(datasetIndex) {		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			// datasetIndex: int
			//		
			
			//Do some checking to make sure datasetIndex is "inbounds"
			var stateStackLength = this._navWidgetStateStack.length;
			if (datasetIndex == stateStackLength - 1) {
	            //this would just be navigating to the current dataset, so return
	            return;
	        } else if (datasetIndex < 0) {
	        	if (this._trace.isError()) {
					this._trace.error("trying to go back too far: datasetIndex = " + datasetIndex + ", state stack size = " + this._navWidgetStateStack.length,
							"navigateToDatasetInStack");
				}
	            return;
	        } else if (datasetIndex >= this._navWidgetStateStack.length) {
	        	if (this._trace.isError()) {
					this._trace.error("trying to navigate past last dataset: datasetIndex = " + datasetIndex + ", state stack size = " + this._navWidgetStateStack.length,
							"navigateToDatasetInStack");
				}
	            return;
	        }
			
			// Fix for 250777
			var fixBreadcrumbBarEllipses = false;
			if (datasetIndex == (stateStackLength - 2)) {
				if (this._breadcrumbBar && this._breadcrumbBar.areBreadcrumbsHidden()) {
					fixBreadcrumbBarEllipses = true;
				}
			}
			// End this part of fix for 250777
			
			//Let's do some timing
			if (this._trace.isFinest()) {
				//Start a timer to see how long it takes
				var navigateToDatasetInStackTimer = new ibm.tivoli.tip.navwidget.dojo.Timer();
				navigateToDatasetInStackTimer.start();
			}

	    	//Show busy pane
	    	this.showBusyPane();
	    	
	    	//Set up callback that will hide the busy pane
	    	this._handle = dojo.connect(this, "onSetDataset", this, function(dataset) {
				this.hideBusyPane();
				if (this._trace.isFinest()) {
					this._trace.finest("navigateToDatasetInStack took ; " + navigateToDatasetInStackTimer.stopAndGetTraceString(), "navigateToDatasetInStack");
				}
				dojo.disconnect(this._handle);});
			
			//Pop off the relevant number of states
			var navWidgetStateStackSize = this._navWidgetStateStack.length;
			var lastDatasetIndex = navWidgetStateStackSize - 1;
			for (var i = lastDatasetIndex; i > datasetIndex; i--) {
				this._removeLastState();
			}
			
			// Fix for 250777
			// After popping the states, does the breadcrumb bar still show ellipses?
			if (this._breadcrumbBar && this._breadcrumbBar.areBreadcrumbsHidden() && fixBreadcrumbBarEllipses) {
				this._rebuildBreadcrumbTrail();
			}
			// End this part of fix for 250777
			
			//Deal with the dataset in the (now) current nav state
			this._addOrNavigateHelper(this._getCurrentNavWidgetState().dataset, false, false);
		},
		
		_rebuildBreadcrumbTrail: function() {   // 250777
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			
			if (this._breadcrumbBar) {
				// Clean out and destroy all dijits of the breadcrumb trail
				var trailDijit;
				for (var i = this._breadcrumbBar.getChildren().length - 1; i >= 0; i--) {
					trailDijit = this._breadcrumbBar.getChildren()[i];
					this._breadcrumbBar.removeChild(trailDijit);
					this._breadcrumbBar.breadcrumbWidget.pop(); // remove dijit from the bar's internal array
					trailDijit.destroy();
				}
				// Rebuild the trail now from the state stack				
				for (var j = 0; j < this._navWidgetStateStack.length; j++) {
					// Add separator before next step if this is not the first step
					if ((this._navWidgetStateStack.length > 1) && (j > 0)) {
						this._breadcrumbBar.addChild(new ibm.tivoli.tip.dijit.BreadcrumbSeparator());
					}
					// Have to split this out to separate method since index j needs to be in hitched function
					this._addBreadcrumbTrailStep(j);
				}
				this._cleanBreadcrumbLabels();				
			}
		},
		
		_addBreadcrumbTrailStep: function(stateIndex) {  // 250777
			// summary:
			//		Coming soon
			// description:
			//		Coming soon

			// Add next step
			var stateDataset = (this._navWidgetStateStack[stateIndex]).dataset;
			var newStep = new ibm.tivoli.tip.dijit.Breadcrumb({label: this._getBreadcrumbLabel(stateDataset, stateIndex == (this._navWidgetStateStack.length - 1)),
															title: this._getBreadcrumbAltTitle(stateDataset)});
			// Reset the state item in the stack to the new breadcrumb and add breadcrumb to the trail
			(this._navWidgetStateStack[stateIndex]).breadcrumbEntry = newStep;
			this._breadcrumbBar.addChild(newStep);
			var index = Number(stateIndex);  // Copy the value before using in hitch statement
			dojo.connect(newStep, 'onClick', dojo.hitch(this, function() { this.navigateToDatasetInStack(index); }));			
		},
		
		_applyCurrentNavState: function() {		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			
			//The current dataset after popping things off stack is what we want to see on view
			var currentNavState = this._getCurrentNavWidgetState();
			if (currentNavState) {
				//Create an "inner class" to help us determine if all of the subviews have called back with onSetDatasetComplete
				var mainOnSetDatasetHandlerInnerClass = {
						navWidget: this,
						callbackCounter: 0,
						
						//Should be invoked in the scope of mainOnSetDatasetHandlerInnerClass, so "this" refers to this inner class rather than the pluggable view
						onSetDatasetHandler: 
							function(dataset) {
								//Increment counter
								this.callbackCounter++;
								
								if (this.callbackCounter >= this.navWidget.getPluggableViewCount(dataset)) {
									var currentNavState = this.navWidget._getCurrentNavWidgetState();
									
									///Start listening for data changes
									if (dataset) {
										dataset.addEventListener(this.navWidget, dojo.hitch(this.navWidget, this.navWidget._datasetEventListenerCallback));
									}
									
									//Set dataset on the selection
									dojo.hitch(this.navWidget.selection, this.navWidget.selection.setDataset, dataset).call();
									
									if (currentNavState.hasStateBeenUsed) {
										//Set selection based on what had been stored in the state... 
										dojo.hitch(this.navWidget.selection, this.navWidget.selection.setSelectedObjects, currentNavState.selectedItems, 
												currentNavState.selectedLinks, currentNavState.selectedRelationships).call();
									} else {
											//This is the first time using this state, so let's also select the origin items...
										
											//Define a callback that fires when all the origin items
											var gotOriginItems = function(items){
												if (items) {
													this.navWidget.selection.setSelectedObjects(items);
												}
											};
							
											var gotOriginItemsError = function(error, request){
												//Give the user some feedback
					        			        this.displayException(error);
											};
							
											//Fetch the origin
											var itemArgs = {
													onComplete: dojo.hitch(this, gotOriginItems),
													onError: dojo.hitch(this, gotOriginItemsError)
											};
											
											if (dataset) {
												dataset.getOriginItems(itemArgs);
											}
									}
									
									//Try to restore previous view state
									var iterator = this.navWidget.getPluggableViewIterator();
									while (!iterator.atEnd()) {
										var view = iterator.get().value;

										var viewState = currentNavState.getPluggableViewState(view.viewId);
										if (viewState) {
											currentNavState.removePluggableViewState(view.viewId);
											view.restoreFromState(viewState);
										} else {
											view.restoreFromState(null); //It's fine to pass null here
										}
									}
									
									//Fire set dataset event
									dojo.hitch(this.navWidget, this.navWidget.onSetDataset, dataset).call();
									
									//Mark state as having previously been used
									currentNavState.hasStateBeenUsed = true;
								}
							}
				};
				var mainDataSetHandlerFunc = dojo.hitch(mainOnSetDatasetHandlerInnerClass, mainOnSetDatasetHandlerInnerClass.onSetDatasetHandler, currentNavState.dataset);
				
				//Pick the desired view to make visible
				var viewsLayout = this.widgetDefinition.getViewsLayout();
				if (viewsLayout && viewsLayout == "stack") { 
					var preferredView = this.getPreferredPluggableView(currentNavState.dataset);
					if (preferredView) {
						this.makePluggableViewVisible(preferredView.viewId);
					}
				} 
				
				//Give dataset to the views
				var iterator = this.getPluggableViewIterator();
				while (!iterator.atEnd()) {
					var view = iterator.get().value;
					
					//Provide an "inner class" to support a handler for the "onSetDataset" event that can remove itself as a listener when done
					var subHandlerInnerClass = {
						mainHandler: mainDataSetHandlerFunc,
						handle: null,
						
						//Should be invoked in the scope of subHandlerInnerClass, so "this" refers to this inner class
						handler: function(dataset){
								this.mainHandler.call();
								
							     //Disconnect the handler
							     dojo.disconnect(this.handle);
							}
					};
					var handle = dojo.connect(view, "onSetDataset", dojo.hitch(subHandlerInnerClass, subHandlerInnerClass.handler));
					subHandlerInnerClass.handle = handle;
						
					if (view.isApplicable(currentNavState.dataset)) {
						try {
							//Since views are pluggable... let's take some care with try/catch, so that a custom one doesn't take us out if it craps
							//out while handling the dataset
							view.setDataset(currentNavState.dataset, null, currentNavState.getPluggableViewState(view.viewId));
						} catch (ex) {
							if (this._trace.isError()) {
								this._trace.error("Cannot set dataset on view = " + view.id + "; ex = " + ex.name + ": " + ex.message, "_applyCurrentNavState");
							}
						}
					} else {
						//View not applicable, so let's clear out the view
						try {
							view.setDataset(null);
						} catch (ex) {
							if (this._trace.isError()) {
								this._trace.error("Unable to clear dataset on view = " + view.id + "; ex = " + ex.name + ": " + ex.message, "_applyCurrentNavState");
							}
						}
					}
				}
			}
		},
		
		_removeLastState: function(inDestroy) {
			if (this._navWidgetStateStack.length > 0) {
				//Remove associated breadcrumb bar entry
				if (this._breadcrumbBar) {
					var lastBreadcrumbBarElementIndex = this._breadcrumbBar.getChildren().length - 1;
					var lastBreadcrumbButton = this._breadcrumbBar.getChildren()[lastBreadcrumbBarElementIndex];
					this._breadcrumbBar.removeChild(lastBreadcrumbButton); // remove breadcrumb entry
					this._breadcrumbBar.breadcrumbWidget.pop(); // remove dijit from the bar's internal array  (250777)
					if (lastBreadcrumbBarElementIndex > 1) {
						//also a separator we need to clean up
						var lastSeparator = this._breadcrumbBar.getChildren()[lastBreadcrumbBarElementIndex - 1];  // 250777
						this._breadcrumbBar.removeChild(lastSeparator);  // 250777
						this._breadcrumbBar.breadcrumbWidget.pop(); // remove dijit from the bar's internal array  (250777)
						lastSeparator.destroy();   // clean up separator dijit  (250777)
					}
				}
				
				//Pop and destroy the state
				var poppedState = this._navWidgetStateStack.pop(); 
            if (poppedState.breadcrumbEntry) {     // breadcrumb bar might be hidden in view so entry may not exist (254914)
               poppedState.breadcrumbEntry.destroy();  // clean up breadcrumb dijit we removed above  (250777)
               poppedState.breadcrumbEntry = null;     // 250777
            }
				this._destroyDataset(poppedState.dataset);
				poppedState.destroy();
				
				//clean up the breadcrumbs
				if (!inDestroy) { //Calling this while we're destorying ourselves causes an exception in Button.js
					this._cleanBreadcrumbLabels();
				}
			}
		},
		
		_destroyDataset: function(dataset) {
			if (dataset) {
				dataset.destroy();
			}
		},
		
		//****************************************************************************
		//* DatasetEventListener stuff
		//****************************************************************************
		_handleChangedEvent: function(/*Object*/ event) {
			// summary:
			//		Handle "changed" event type.
			// event:
			//		An object containing information about what changed in the Dataset.
			// description:
			//		Handle "changed" event type.
			// tags:
			//		protected
			
			//Call superclass
			this.inherited(arguments);
		},
		
		_handleLabelChangedEvent: function(/*Object*/ event) {
			// summary:
			//		Handle "labelChanged" event type.
			// event:
			//		An object containing information about what changed in the Dataset.
			// description:
			//		Handle "labelChanged" event type.
			// tags:
			//		protected
			
			var dataset = event.getDataset();
			var state = this._getCurrentNavWidgetState();
			if (state) {
				var label = this._getBreadcrumbLabel(dataset, true);
				var altTitle = this._getBreadcrumbAltTitle(dataset);
				state.breadcrumbEntry.set("label", label);
				state.breadcrumbEntry.set("title", altTitle);
			}
			
			//Call superclass
			this.inherited(arguments);
		},
		
		_handleStatusChangedEvent: function(/*Object*/ event) {
			// summary:
			//		Handle "statusChanged" event type.
			// event:
			//		An object containing information about what changed in the Dataset.
			// description:
			//		Subclass should override to if it wishes to deal with "statusChanged" events.
			// tags:
			//		protected
			
			//Call superclass
			this.inherited(arguments);
		},
		
		_handleItemsAddedEvent: function(/*Object*/ event) {
			// summary:
			//		Handle "itemsAdded" event type.
			// event:
			//		An object containing information about what changed in the Dataset.
			// description:
			//		Handle "itemsAdded" event type.
			// tags:
			//		protected
			
			//Call superclass
			this.inherited(arguments);
		},

		_handleItemsRemovedEvent: function(/*Object*/ event) {
			// summary:
			//		Handle "itemsRemoved" event type.
			// event:
			//		An object containing information about what changed in the Dataset.
			// description:
			//		Handle "itemsRemoved" event type.
			// tags:
			//		protected
			
			//Call superclass
			this.inherited(arguments);
		},

		_handleItemStatusChangedEvent: function(/*Object*/ event) {
			// summary:
			//		Handle "itemStatusChanged" event type.
			// event:
			//		An object containing information about what changed in the Dataset.
			// description:
			//		Handle "itemStatusChanged" event type.
			// tags:
			//		protected
			
			//Call superclass
			this.inherited(arguments);
		},
		
		_handleItemPropertiesChangedEvent: function(/*Object*/ event) {
			// summary:
			//		Handle "itemPropertiesChanged" event type.
			// event:
			//		An object containing information about what changed in the Dataset.
			// description:
			//		Handle "itemPropertiesChanged" event type.
			// tags:
			//		protected
			
			//Call superclass
			this.inherited(arguments);
		},

		_handleLinksAddedEvent: function(/*Object*/ event) {
			// summary:
			//		Handle "linksAdded" event type.
			// event:
			//		An object containing information about what changed in the Dataset.
			// description:
			//		Handle "linksAdded" event type.
			// tags:
			//		protected
			
			//Call superclass
			this.inherited(arguments);
		},
		
		_handleLinksRemovedEvent: function(/*Object*/ event) {
			// summary:
			//		Handle "linksRemoved" event type.
			// event:
			//		An object containing information about what changed in the Dataset.
			// description:
			//		Handle "linksRemoved" event type.
			// tags:
			//		protected
			
			//Call superclass
			this.inherited(arguments);
		},
		
		_handleLinkStatusChangedEvent: function(/*Object*/ event) {
			// summary:
			//		Handle "linkStatusChanged" event type.
			// event:
			//		An object containing information about what changed in the Dataset.
			// description:
			//		Handle "linkStatusChanged" event type.
			// tags:
			//		protected
			
			//Call superclass
			this.inherited(arguments);
		},
		
		_handleLinkPropertiesChangedEvent: function(/*Object*/ event) {		
			// summary:
			//		Handle "linkPropertiesChanged" event type.
			// event:
			//		An object containing information about what changed in the Dataset.
			// description:
			//		Handle "linkPropertiesChanged" event type.
			// tags:
			//		protected
			
			//Call superclass
			this.inherited(arguments);	
		},

		_handleLinkChangedEvent: function(/*Object*/ event) {
			// summary:
			//		Handle "linkChanged" event type.
			// event:
			//		An object containing information about what changed in the Dataset.
			// description:
			//		Handle "linkChanged" event type.
			// tags:
			//		protected
			
			//Call superclass
			this.inherited(arguments);		
		},
		
		_handleRelationshipStatusChangedEvent: function(/*Object*/ event) {
			// summary:
			//		Handle "relationshipStatusChanged" event type.
			// event:
			//		An object containing information about what changed in the Dataset.
			// description:
			//		Handle "relationshipStatusChanged" event type.
			// tags:
			//		protected
			
			//Call superclass
			this.inherited(arguments);
		},
		
		_handleRelationshipPropertiesChangedEvent: function(/*Object*/ event) {		
			// summary:
			//		Handle "relationshipPropertiesChanged" event type.
			// event:
			//		An object containing information about what changed in the Dataset.
			// description:
			//		Handle "relationshipPropertiesChanged" event type.
			// tags:
			//		protected
			
			//Call superclass
			this.inherited(arguments);	
		},
		
		_handleExceptionEvent: function(/*Object*/ event) {
			// summary:
			//		Handle "exception" event type.
			// event:
			//		An object containing information the "exception" event
			// description:
			//		Subclass should override to if it wishes to deal with "exception" events.
			// tags:
			//		protected

			//Call superclass
			this.inherited(arguments);
		},
		
		_handleEventRouterEvent: function(/*Object*/ event) {
			// summary:
			//		Handle "eventRouter" event type.
			// event:
			//		An object containing information the "eventRouter" event
			// description:
			//		Subclass should override to if it wishes to deal with "eventRouter" events.
			// tags:
			//		protected

			//Call superclass
			this.inherited(arguments);
		},
		
		//*********************************************************************************************
		// "Private"
		//*********************************************************************************************
		_createBreadcrumbBar: function(){		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			var breadcrumbBar = new ibm.tivoli.tip.dijit.BreadcrumbBar({
				region: "top"
			});
			this._breadcrumbBar = breadcrumbBar;
		},
		
		_createWidgetDefinition: function(){		
			//Call super class which will load widget def
			this.inherited(arguments);
			
			//Initialize based on widget def
			this._updateNavWidgetParamsFromWidgetDefs();
		},
		
		_getDefaultWidgetDefInfo: function(){
			return this._constants.DEFAULT_WIDGET_DEF_INFO;
		},
		
		_updateNavWidgetParamsFromWidgetDefs: function(){		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			if (this.widgetDefinition){				
				this.includeBreadcrumbs = !this.widgetDefinition.getBreadcrumbs().hidden;
			}
		}		
		
	});

});

},
'dojox/grid/_Grid':function(){
define([
	"dojo/_base/kernel",
	"../main",
	"dojo/_base/declare",
	"./_Events",
	"./_Scroller",
	"./_Layout",
	"./_View",
	"./_ViewManager",
	"./_RowManager",
	"./_FocusManager",
	"./_EditManager",
	"./Selection",
	"./_RowSelector",
	"./util",
	"dijit/_Widget",
	"dijit/_TemplatedMixin",
	"dijit/CheckedMenuItem",
	"dojo/text!./resources/_Grid.html",
	"dojo/string",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/_base/sniff",
	"dojox/html/metrics",
	"dojo/_base/html",
	"dojo/query",
	"dojo/dnd/common",
	"dojo/i18n!dijit/nls/loading"
], function(dojo, dojox, declare, _Events, _Scroller, _Layout, _View, _ViewManager,
	_RowManager, _FocusManager, _EditManager, Selection, _RowSelector, util, _Widget,
	 _TemplatedMixin, CheckedMenuItem, template, string, array, lang, has, metrics, html, query){

	// NOTE: this is for backwards compatibility with Dojo 1.3
	if(!dojo.isCopyKey){
		dojo.isCopyKey = dojo.dnd.getCopyKeyState;
	}
	/*=====
	dojox.grid.__CellDef = function(){
		//	name: String?
		//		The text to use in the header of the grid for this cell.
		//	get: Function?
		//		function(rowIndex){} rowIndex is of type Integer.  This
		//		function will be called when a cell	requests data.  Returns the
		//		unformatted data for the cell.
		//	value: String?
		//		If "get" is not specified, this is used as the data for the cell.
		//	defaultValue: String?
		//		If "get" and "value" aren't specified or if "get" returns an undefined
		//		value, this is used as the data for the cell.  "formatter" is not run
		//		on this if "get" returns an undefined value.
		//	formatter: Function?
		//		function(data, rowIndex){} data is of type anything, rowIndex
		//		is of type Integer.  This function will be called after the cell
		//		has its data but before it passes it back to the grid to render.
		//		Returns the formatted version of the cell's data.
		//	type: dojox.grid.cells._Base|Function?
		//		TODO
		//	editable: Boolean?
		//		Whether this cell should be editable or not.
		//	hidden: Boolean?
		//		If true, the cell will not be displayed.
		//	noresize: Boolean?
		//		If true, the cell will not be able to be resized.
		//	width: Integer|String?
		//		A CSS size.  If it's an Integer, the width will be in em's.
		//	colSpan: Integer?
		//		How many columns to span this cell.  Will not work in the first
		//		sub-row of cells.
		//	rowSpan: Integer?
		//		How many sub-rows to span this cell.
		//	styles: String?
		//		A string of styles to apply to both the header cell and main
		//		grid cells.  Must end in a ';'.
		//	headerStyles: String?
		//		A string of styles to apply to just the header cell.  Must end
		//		in a ';'
		//	cellStyles: String?
		//		A string of styles to apply to just the main grid cells.  Must
		//		end in a ';'
		//	classes: String?
		//		A space separated list of classes to apply to both the header
		//		cell and the main grid cells.
		//	headerClasses: String?
		//		A space separated list of classes to apply to just the header
		//		cell.
		//	cellClasses: String?
		//		A space separated list of classes to apply to just the main
		//		grid cells.
		//	attrs: String?
		//		A space separated string of attribute='value' pairs to add to
		//		the header cell element and main grid cell elements.
		this.name = name;
		this.value = value;
		this.get = get;
		this.formatter = formatter;
		this.type = type;
		this.editable = editable;
		this.hidden = hidden;
		this.width = width;
		this.colSpan = colSpan;
		this.rowSpan = rowSpan;
		this.styles = styles;
		this.headerStyles = headerStyles;
		this.cellStyles = cellStyles;
		this.classes = classes;
		this.headerClasses = headerClasses;
		this.cellClasses = cellClasses;
		this.attrs = attrs;
	}
	=====*/

	/*=====
	dojox.grid.__ViewDef = function(){
		//	noscroll: Boolean?
		//		If true, no scrollbars will be rendered without scrollbars.
		//	width: Integer|String?
		//		A CSS size.  If it's an Integer, the width will be in em's. If
		//		"noscroll" is true, this value is ignored.
		//	cells: dojox.grid.__CellDef[]|Array[dojox.grid.__CellDef[]]?
		//		The structure of the cells within this grid.
		//	type: String?
		//		A string containing the constructor of a subclass of
		//		dojox.grid._View.  If this is not specified, dojox.grid._View
		//		is used.
		//	defaultCell: dojox.grid.__CellDef?
		//		A cell definition with default values for all cells in this view.  If
		//		a property is defined in a cell definition in the "cells" array and
		//		this property, the cell definition's property will override this
		//		property's property.
		//	onBeforeRow: Function?
		//		function(rowIndex, cells){} rowIndex is of type Integer, cells
		//		is of type Array[dojox.grid.__CellDef[]].  This function is called
		//		before each row of data is rendered.  Before the header is
		//		rendered, rowIndex will be -1.  "cells" is a reference to the
		//		internal structure of this view's cells so any changes you make to
		//		it will persist between calls.
		//	onAfterRow: Function?
		//		function(rowIndex, cells, rowNode){} rowIndex is of type Integer, cells
		//		is of type Array[dojox.grid.__CellDef[]], rowNode is of type DOMNode.
		//		This function is called	after each row of data is rendered.  After the
		//		header is rendered, rowIndex will be -1.  "cells" is a reference to the
		//		internal structure of this view's cells so any changes you make to
		//		it will persist between calls.
		this.noscroll = noscroll;
		this.width = width;
		this.cells = cells;
		this.type = type;
		this.defaultCell = defaultCell;
		this.onBeforeRow = onBeforeRow;
		this.onAfterRow = onAfterRow;
	}
	=====*/

	var _Grid = declare('dojox.grid._Grid',
		[ _Widget, _TemplatedMixin, _Events ],
		{
		// summary:
		// 		A grid widget with virtual scrolling, cell editing, complex rows,
		// 		sorting, fixed columns, sizeable columns, etc.
		//
		//	description:
		//		_Grid provides the full set of grid features without any
		//		direct connection to a data store.
		//
		//		The grid exposes a get function for the grid, or optionally
		//		individual columns, to populate cell contents.
		//
		//		The grid is rendered based on its structure, an object describing
		//		column and cell layout.
		//
		//	example:
		//		A quick sample:
		//
		//		define a get function
		//	|	function get(inRowIndex){ // called in cell context
		//	|		return [this.index, inRowIndex].join(', ');
		//	|	}
		//
		//		define the grid structure:
		//	|	var structure = [ // array of view objects
		//	|		{ cells: [// array of rows, a row is an array of cells
		//	|			[
		//	|				{ name: "Alpha", width: 6 },
		//	|				{ name: "Beta" },
		//	|				{ name: "Gamma", get: get }]
		//	|		]}
		//	|	];
		//
		//	|	<div id="grid"
		//	|		rowCount="100" get="get"
		//	|		structure="structure"
		//	|		dojoType="dojox.grid._Grid"></div>

		templateString: template,

		// classTag: String
		// 		CSS class applied to the grid's domNode
		classTag: 'dojoxGrid',

		// settings
		// rowCount: Integer
		//		Number of rows to display.
		rowCount: 5,

		// keepRows: Integer
		//		Number of rows to keep in the rendering cache.
		keepRows: 75,

		// rowsPerPage: Integer
		//		Number of rows to render at a time.
		rowsPerPage: 25,

		// autoWidth: Boolean
		//		If autoWidth is true, grid width is automatically set to fit the data.
		autoWidth: false,
		
		// initialWidth: String
		//		A css string to use to set our initial width (only used if autoWidth
		//		is true).  The first rendering of the grid will be this width, any
		//		resizing of columns, etc will result in the grid switching to
		//		autoWidth mode.  Note, this width will override any styling in a
		//		stylesheet or directly on the node.
		initialWidth: "",

		// autoHeight: Boolean|Integer
		//		If autoHeight is true, grid height is automatically set to fit the data.
		//		If it is an integer, the height will be automatically set to fit the data
		//		if there are fewer than that many rows - and the height will be set to show
		//		that many rows if there are more
		autoHeight: '',

		// rowHeight: Integer
		//		If rowHeight is set to a positive number, it will define the height of the rows
		//		in pixels. This can provide a significant performance advantage, since it
		//		eliminates the need to measure row sizes during rendering, which is one
		// 		the primary bottlenecks in the DataGrid's performance.
		rowHeight: 0,
		
		// autoRender: Boolean
		//		If autoRender is true, grid will render itself after initialization.
		autoRender: true,

		// defaultHeight: String
		//		default height of the grid, measured in any valid css unit.
		defaultHeight: '15em',
		
		// height: String
		//		explicit height of the grid, measured in any valid css unit.  This will be populated (and overridden)
		//		if the height: css attribute exists on the source node.
		height: '',

		// structure: dojox.grid.__ViewDef|dojox.grid.__ViewDef[]|dojox.grid.__CellDef[]|Array[dojox.grid.__CellDef[]]
		//		View layout defintion.
		structure: null,

		// elasticView: Integer
		//	Override defaults and make the indexed grid view elastic, thus filling available horizontal space.
		elasticView: -1,

		// singleClickEdit: boolean
		//		Single-click starts editing. Default is double-click
		singleClickEdit: false,

		// selectionMode: String
		//		Set the selection mode of grid's Selection.  Value must be 'single', 'multiple',
		//		or 'extended'.  Default is 'extended'.
		selectionMode: 'extended',

		// rowSelector: Boolean|String
		// 		If set to true, will add a row selector view to this grid.  If set to a CSS width, will add
		// 		a row selector of that width to this grid.
		rowSelector: '',

		// columnReordering: Boolean
		// 		If set to true, will add drag and drop reordering to views with one row of columns.
		columnReordering: false,

		// headerMenu: dijit.Menu
		// 		If set to a dijit.Menu, will use this as a context menu for the grid headers.
		headerMenu: null,

		// placeholderLabel: String
		// 		Label of placeholders to search for in the header menu to replace with column toggling
		// 		menu items.
		placeholderLabel: "GridColumns",
		
		// selectable: Boolean
		//		Set to true if you want to be able to select the text within the grid.
		selectable: false,
		
		// Used to store the last two clicks, to ensure double-clicking occurs based on the intended row
		_click: null,
		
		// loadingMessage: String
		//  Message that shows while the grid is loading
		loadingMessage: "<span class='dojoxGridLoading'>${loadingState}</span>",

		// errorMessage: String
		//  Message that shows when the grid encounters an error loading
		errorMessage: "<span class='dojoxGridError'>${errorState}</span>",

		// noDataMessage: String
		//  Message that shows if the grid has no data - wrap it in a
		//  span with class 'dojoxGridNoData' if you want it to be
		//  styled similar to the loading and error messages
		noDataMessage: "",
		
		// escapeHTMLInData: Boolean
		//		This will escape HTML brackets from the data to prevent HTML from
		// 		user-inputted data being rendered with may contain JavaScript and result in
		// 		XSS attacks. This is true by default, and it is recommended that it remain
		// 		true. Setting this to false will allow data to be displayed in the grid without
		// 		filtering, and should be only used if it is known that the data won't contain
		// 		malicious scripts. If HTML is needed in grid cells, it is recommended that
		// 		you use the formatter function to generate the HTML (the output of
		// 		formatter functions is not filtered, even with escapeHTMLInData set to true).
		escapeHTMLInData: true,
		
		// formatterScope: Object
		//		An object to execute format functions within.  If not set, the
		//		format functions will execute within the scope of the cell that
		//		has a format function.
		formatterScope: null,
		
		// editable: boolean
		// indicates if the grid contains editable cells, default is false
		// set to true if editable cell encountered during rendering
		editable: false,
		
		// private
		sortInfo: 0,
		themeable: true,
		_placeholders: null,

		// _layoutClass: Object
		//	The class to use for our layout - can be overridden by grid subclasses
		_layoutClass: _Layout,

		// initialization
		buildRendering: function(){
			this.inherited(arguments);
			if(!this.domNode.getAttribute('tabIndex')){
				this.domNode.tabIndex = "0";
			}
			this.createScroller();
			this.createLayout();
			this.createViews();
			this.createManagers();

			this.createSelection();

			this.connect(this.selection, "onSelected", "onSelected");
			this.connect(this.selection, "onDeselected", "onDeselected");
			this.connect(this.selection, "onChanged", "onSelectionChanged");

			metrics.initOnFontResize();
			this.connect(metrics, "onFontResize", "textSizeChanged");
			util.funnelEvents(this.domNode, this, 'doKeyEvent', util.keyEvents);
			if (this.selectionMode != "none") {
				this.domNode.setAttribute("aria-multiselectable", this.selectionMode == "single" ? "false" : "true");
			}

			html.addClass(this.domNode, this.classTag);
			if(!this.isLeftToRight()){
				html.addClass(this.domNode, this.classTag+"Rtl");
			}
		},
		
		postMixInProperties: function(){
			this.inherited(arguments);
			var messages = dojo.i18n.getLocalization("dijit", "loading", this.lang);
			this.loadingMessage = string.substitute(this.loadingMessage, messages);
			this.errorMessage = string.substitute(this.errorMessage, messages);
			if(this.srcNodeRef && this.srcNodeRef.style.height){
				this.height = this.srcNodeRef.style.height;
			}
			// Call this to update our autoheight to start out
			this._setAutoHeightAttr(this.autoHeight, true);
			this.lastScrollTop = this.scrollTop = 0;
		},
		
		postCreate: function(){
			this._placeholders = [];
			this._setHeaderMenuAttr(this.headerMenu);
			this._setStructureAttr(this.structure);
			this._click = [];
			this.inherited(arguments);
			if(this.domNode && this.autoWidth && this.initialWidth){
				this.domNode.style.width = this.initialWidth;
			}
			if (this.domNode && !this.editable){
				// default value for aria-readonly is false, set to true if grid is not editable
				html.attr(this.domNode,"aria-readonly", "true");
			}
		},

		destroy: function(){
			this.domNode.onReveal = null;
			this.domNode.onSizeChange = null;

			// Fixes IE domNode leak
			delete this._click;

			if(this.scroller){
				this.scroller.destroy();
				delete this.scroller;
			}
			this.edit.destroy();
			delete this.edit;
			this.views.destroyViews();
			if(this.focus){
				this.focus.destroy();
				delete this.focus;
			}
			if(this.headerMenu&&this._placeholders.length){
				array.forEach(this._placeholders, function(p){ p.unReplace(true); });
				this.headerMenu.unBindDomNode(this.viewsHeaderNode);
			}
			this.inherited(arguments);
		},

		_setAutoHeightAttr: function(ah, skipRender){
			// Calculate our autoheight - turn it into a boolean or an integer
			if(typeof ah == "string"){
				if(!ah || ah == "false"){
					ah = false;
				}else if (ah == "true"){
					ah = true;
				}else{
					ah = window.parseInt(ah, 10);
				}
			}
			if(typeof ah == "number"){
				if(isNaN(ah)){
					ah = false;
				}
				// Autoheight must be at least 1, if it's a number.  If it's
				// less than 0, we'll take that to mean "all" rows (same as
				// autoHeight=true - if it is equal to zero, we'll take that
				// to mean autoHeight=false
				if(ah < 0){
					ah = true;
				}else if (ah === 0){
					ah = false;
				}
			}
			this.autoHeight = ah;
			if(typeof ah == "boolean"){
				this._autoHeight = ah;
			}else if(typeof ah == "number"){
				this._autoHeight = (ah >= this.get('rowCount'));
			}else{
				this._autoHeight = false;
			}
			if(this._started && !skipRender){
				this.render();
			}
		},

		_getRowCountAttr: function(){
			return this.updating && this.invalidated && this.invalidated.rowCount != undefined ?
				this.invalidated.rowCount : this.rowCount;
		},
		
		textSizeChanged: function(){
			this.render();
		},

		sizeChange: function(){
			this.update();
		},

		createManagers: function(){
			// summary:
			//		create grid managers for various tasks including rows, focus, selection, editing

			// row manager
			this.rows = new _RowManager(this);
			// focus manager
			this.focus = new _FocusManager(this);
			// edit manager
			this.edit = new _EditManager(this);
		},

		createSelection: function(){
			// summary:	Creates a new Grid selection manager.

			// selection manager
			this.selection = new Selection(this);
		},

		createScroller: function(){
			// summary: Creates a new virtual scroller
			this.scroller = new _Scroller();
			this.scroller.grid = this;
			this.scroller.renderRow = lang.hitch(this, "renderRow");
			this.scroller.removeRow = lang.hitch(this, "rowRemoved");
		},

		createLayout: function(){
			// summary: Creates a new Grid layout
			this.layout = new this._layoutClass(this);
			this.connect(this.layout, "moveColumn", "onMoveColumn");
		},

		onMoveColumn: function(){
			this.render();
		},
		
		onResizeColumn: function(/*int*/ cellIdx){
			// Called when a column is resized.
		},

		// views
		createViews: function(){
			this.views = new _ViewManager(this);
			this.views.createView = lang.hitch(this, "createView");
		},

		createView: function(inClass, idx){
			var c = lang.getObject(inClass);
			var view = new c({ grid: this, index: idx });
			this.viewsNode.appendChild(view.domNode);
			this.viewsHeaderNode.appendChild(view.headerNode);
			this.views.addView(view);
			html.attr(this.domNode, "align", this.isLeftToRight() ? 'left' : 'right');
			return view;
		},

		buildViews: function(){
			for(var i=0, vs; (vs=this.layout.structure[i]); i++){
				this.createView(vs.type || dojox._scopeName + ".grid._View", i).setStructure(vs);
			}
			this.scroller.setContentNodes(this.views.getContentNodes());
		},

		_setStructureAttr: function(structure){
			var s = structure;
			if(s && lang.isString(s)){
				dojo.deprecated("dojox.grid._Grid.set('structure', 'objVar')", "use dojox.grid._Grid.set('structure', objVar) instead", "2.0");
				s=lang.getObject(s);
			}
			this.structure = s;
			if(!s){
				if(this.layout.structure){
					s = this.layout.structure;
				}else{
					return;
				}
			}
			this.views.destroyViews();
			this.focus.focusView = null;
			if(s !== this.layout.structure){
				this.layout.setStructure(s);
			}
			this._structureChanged();
		},

		setStructure: function(/* dojox.grid.__ViewDef|dojox.grid.__ViewDef[]|dojox.grid.__CellDef[]|Array[dojox.grid.__CellDef[]] */ inStructure){
			// summary:
			//		Install a new structure and rebuild the grid.
			dojo.deprecated("dojox.grid._Grid.setStructure(obj)", "use dojox.grid._Grid.set('structure', obj) instead.", "2.0");
			this._setStructureAttr(inStructure);
		},
		
		getColumnTogglingItems: function(){
			// Summary: returns an array of dijit.CheckedMenuItem widgets that can be
			//		added to a menu for toggling columns on and off.
			var items, checkedItems = [];
			items = array.map(this.layout.cells, function(cell){
				if(!cell.menuItems){ cell.menuItems = []; }

				var self = this;
				var item = new CheckedMenuItem({
					label: cell.name,
					checked: !cell.hidden,
					_gridCell: cell,
					onChange: function(checked){
						if(self.layout.setColumnVisibility(this._gridCell.index, checked)){
							var items = this._gridCell.menuItems;
							if(items.length > 1){
								array.forEach(items, function(item){
									if(item !== this){
										item.setAttribute("checked", checked);
									}
								}, this);
							}
							checked = array.filter(self.layout.cells, function(c){
								if(c.menuItems.length > 1){
									array.forEach(c.menuItems, "item.set('disabled', false);");
								}else{
									c.menuItems[0].set('disabled', false);
								}
								return !c.hidden;
							});
							if(checked.length == 1){
								array.forEach(checked[0].menuItems, "item.set('disabled', true);");
							}
						}
					},
					destroy: function(){
						var index = array.indexOf(this._gridCell.menuItems, this);
						this._gridCell.menuItems.splice(index, 1);
						delete this._gridCell;
						CheckedMenuItem.prototype.destroy.apply(this, arguments);
					}
				});
				cell.menuItems.push(item);
				if(!cell.hidden) {
					checkedItems.push(item);
				}
				return item;
			}, this); // dijit.CheckedMenuItem[]
			if(checkedItems.length == 1) {
				checkedItems[0].set('disabled', true);
			}
			return items;
		},

		_setHeaderMenuAttr: function(menu){
			if(this._placeholders && this._placeholders.length){
				array.forEach(this._placeholders, function(p){
					p.unReplace(true);
				});
				this._placeholders = [];
			}
			if(this.headerMenu){
				this.headerMenu.unBindDomNode(this.viewsHeaderNode);
			}
			this.headerMenu = menu;
			if(!menu){ return; }

			this.headerMenu.bindDomNode(this.viewsHeaderNode);
			if(this.headerMenu.getPlaceholders){
				this._placeholders = this.headerMenu.getPlaceholders(this.placeholderLabel);
			}
		},

		setHeaderMenu: function(/* dijit.Menu */ menu){
			dojo.deprecated("dojox.grid._Grid.setHeaderMenu(obj)", "use dojox.grid._Grid.set('headerMenu', obj) instead.", "2.0");
			this._setHeaderMenuAttr(menu);
		},
		
		setupHeaderMenu: function(){
			if(this._placeholders && this._placeholders.length){
				array.forEach(this._placeholders, function(p){
					if(p._replaced){
						p.unReplace(true);
					}
					p.replace(this.getColumnTogglingItems());
				}, this);
			}
		},

		_fetch: function(start){
			this.setScrollTop(0);
		},

		getItem: function(inRowIndex){
			return null;
		},
		
		showMessage: function(message){
			if(message){
				this.messagesNode.innerHTML = message;
				this.messagesNode.style.display = "";
			}else{
				this.messagesNode.innerHTML = "";
				this.messagesNode.style.display = "none";
			}
		},

		_structureChanged: function() {
			this.buildViews();
			if(this.autoRender && this._started){
				this.render();
			}
		},

		hasLayout: function() {
			return this.layout.cells.length;
		},

		// sizing
		resize: function(changeSize, resultSize){
			// summary:
			//		Update the grid's rendering dimensions and resize it
			
			// Calling sizeChange calls update() which calls _resize...so let's
			// save our input values, if any, and use them there when it gets
			// called.  This saves us an extra call to _resize(), which can
			// get kind of heavy.
			
			// fixes #11101, should ignore resize when in autoheight mode(IE) to avoid a deadlock
			// e.g when an autoheight editable grid put in dijit.form.Form or other similar containers,
			// grid switch to editing mode --> grid height change --> From height change
			// ---> Form call grid.resize() ---> grid height change  --> deaklock
			if(dojo.isIE && !changeSize && !resultSize && this._autoHeight){
				return;
			}
			this._pendingChangeSize = changeSize;
			this._pendingResultSize = resultSize;
			this.sizeChange();
		},

		_getPadBorder: function() {
			this._padBorder = this._padBorder || html._getPadBorderExtents(this.domNode);
			return this._padBorder;
		},

		_getHeaderHeight: function(){
			var vns = this.viewsHeaderNode.style, t = vns.display == "none" ? 0 : this.views.measureHeader();
			vns.height = t + 'px';
			// header heights are reset during measuring so must be normalized after measuring.
			this.views.normalizeHeaderNodeHeight();
			return t;
		},
		
		_resize: function(changeSize, resultSize){
			// Restore our pending values, if any
			changeSize = changeSize || this._pendingChangeSize;
			resultSize = resultSize || this._pendingResultSize;
			delete this._pendingChangeSize;
			delete this._pendingResultSize;
			// if we have set up everything except the DOM, we cannot resize
			if(!this.domNode){ return; }
			var pn = this.domNode.parentNode;
			if(!pn || pn.nodeType != 1 || !this.hasLayout() || pn.style.visibility == "hidden" || pn.style.display == "none"){
				return;
			}
			// useful measurement
			var padBorder = this._getPadBorder();
			var hh = undefined;
			var h;
			// grid height
			if(this._autoHeight){
				this.domNode.style.height = 'auto';
			}else if(typeof this.autoHeight == "number"){
				h = hh = this._getHeaderHeight();
				h += (this.scroller.averageRowHeight * this.autoHeight);
				this.domNode.style.height = h + "px";
			}else if(this.domNode.clientHeight <= padBorder.h){
				if(pn == document.body){
					this.domNode.style.height = this.defaultHeight;
				}else if(this.height){
					this.domNode.style.height = this.height;
				}else{
					this.fitTo = "parent";
				}
			}
			// if we are given dimensions, size the grid's domNode to those dimensions
			if(resultSize){
				changeSize = resultSize;
			}
			if(!this._autoHeight && changeSize){
				html.marginBox(this.domNode, changeSize);
				this.height = this.domNode.style.height;
				delete this.fitTo;
			}else if(this.fitTo == "parent"){
				h = this._parentContentBoxHeight = this._parentContentBoxHeight || html._getContentBox(pn).h;
				this.domNode.style.height = Math.max(0, h) + "px";
			}
			
			var hasFlex = array.some(this.views.views, function(v){ return v.flexCells; });

			if(!this._autoHeight && (h || html._getContentBox(this.domNode).h) === 0){
				// We need to hide the header, since the Grid is essentially hidden.
				this.viewsHeaderNode.style.display = "none";
			}else{
				// Otherwise, show the header and give it an appropriate height.
				this.viewsHeaderNode.style.display = "block";
				if(!hasFlex && hh === undefined){
					hh = this._getHeaderHeight();
				}
			}
			if(hasFlex){
				hh = undefined;
			}

			// NOTE: it is essential that width be applied before height
			// Header height can only be calculated properly after view widths have been set.
			// This is because flex column width is naturally 0 in Firefox.
			// Therefore prior to width sizing flex columns with spaces are maximally wrapped
			// and calculated to be too tall.
			this.adaptWidth();
			this.adaptHeight(hh);

			this.postresize();
		},

		adaptWidth: function() {
			// private: sets width and position for views and update grid width if necessary
			var doAutoWidth = (!this.initialWidth && this.autoWidth);
			var w = doAutoWidth ? 0 : this.domNode.clientWidth || (this.domNode.offsetWidth - this._getPadBorder().w),
				vw = this.views.arrange(1, w);
			this.views.onEach("adaptWidth");
			if(doAutoWidth){
				this.domNode.style.width = vw + "px";
			}
		},

		adaptHeight: function(inHeaderHeight){
			// private: measures and normalizes header height, then sets view heights, and then updates scroller
			// content extent
			var t = inHeaderHeight === undefined ? this._getHeaderHeight() : inHeaderHeight;
			var h = (this._autoHeight ? -1 : Math.max(this.domNode.clientHeight - t, 0) || 0);
			this.views.onEach('setSize', [0, h]);
			this.views.onEach('adaptHeight');
			if(!this._autoHeight){
				var numScroll = 0, numNoScroll = 0;
				var noScrolls = array.filter(this.views.views, function(v){
					var has = v.hasHScrollbar();
					if(has){ numScroll++; }else{ numNoScroll++; }
					return (!has);
				});
				if(numScroll > 0 && numNoScroll > 0){
					array.forEach(noScrolls, function(v){
						v.adaptHeight(true);
					});
				}
			}
			if(this.autoHeight === true || h != -1 || (typeof this.autoHeight == "number" && this.autoHeight >= this.get('rowCount'))){
				this.scroller.windowHeight = h;
			}else{
				this.scroller.windowHeight = Math.max(this.domNode.clientHeight - t, 0);
			}
		},

		// startup
		startup: function(){
			if(this._started){return;}
			this.inherited(arguments);
			if(this.autoRender){
				this.render();
			}
		},

		// render
		render: function(){
			// summary:
			//	Render the grid, headers, and views. Edit and scrolling states are reset. To retain edit and
			// scrolling states, see Update.

			if(!this.domNode){return;}
			if(!this._started){return;}

			if(!this.hasLayout()) {
				this.scroller.init(0, this.keepRows, this.rowsPerPage);
				return;
			}
			//
			this.update = this.defaultUpdate;
			this._render();
		},

		_render: function(){
			this.scroller.init(this.get('rowCount'), this.keepRows, this.rowsPerPage);
			this.prerender();
			this.setScrollTop(0);
			this.postrender();
		},

		prerender: function(){
			// if autoHeight, make sure scroller knows not to virtualize; everything must be rendered.
			this.keepRows = this._autoHeight ? 0 : this.keepRows;
			this.scroller.setKeepInfo(this.keepRows);
			this.views.render();
			this._resize();
		},

		postrender: function(){
			this.postresize();
			this.focus.initFocusView();
			// make rows unselectable
			html.setSelectable(this.domNode, this.selectable);
			if(dojo.isIE){
				query("th.dojoxGridCell", this.viewsHeaderNode).forEach(function(node){
					dojo.setSelectable(node, true);
				});	
			}
		},

		postresize: function(){
			// views are position absolute, so they do not inflate the parent
			if(this._autoHeight){
				var size = Math.max(this.views.measureContent()) + 'px';
				
				this.viewsNode.style.height = size;
			}
		},

		renderRow: function(inRowIndex, inNodes){
			// summary: private, used internally to render rows
			this.views.renderRow(inRowIndex, inNodes, this._skipRowRenormalize);
		},

		rowRemoved: function(inRowIndex){
			// summary: private, used internally to remove rows
			this.views.rowRemoved(inRowIndex);
		},

		invalidated: null,

		updating: false,

		beginUpdate: function(){
			// summary:
			//		Use to make multiple changes to rows while queueing row updating.
			// NOTE: not currently supporting nested begin/endUpdate calls
			this.invalidated = [];
			this.updating = true;
		},

		endUpdate: function(){
			// summary:
			//		Use after calling beginUpdate to render any changes made to rows.
			this.updating = false;
			var i = this.invalidated, r;
			if(i.all){
				this.update();
			}else if(i.rowCount != undefined){
				this.updateRowCount(i.rowCount);
			}else{
				for(r in i){
					this.updateRow(Number(r));
				}
			}
			this.invalidated = [];
		},

		// update
		defaultUpdate: function(){
			// note: initial update calls render and subsequently this function.
			if(!this.domNode){return;}
			if(this.updating){
				this.invalidated.all = true;
				return;
			}
			//this.edit.saveState(inRowIndex);
			this.lastScrollTop = this.scrollTop;
			this.prerender();
			this.scroller.invalidateNodes();
			this.setScrollTop(this.lastScrollTop);
			this.postrender();
			//this.edit.restoreState(inRowIndex);
		},

		update: function(){
			// summary:
			//		Update the grid, retaining edit and scrolling states.
			this.render();
		},

		updateRow: function(inRowIndex){
			// summary:
			//		Render a single row.
			// inRowIndex: Integer
			//		Index of the row to render
			inRowIndex = Number(inRowIndex);
			if(this.updating){
				this.invalidated[inRowIndex]=true;
			}else{
				this.views.updateRow(inRowIndex);
				this.scroller.rowHeightChanged(inRowIndex);
			}
		},

		updateRows: function(startIndex, howMany){
			// summary:
			//		Render consecutive rows at once.
			// startIndex: Integer
			//		Index of the starting row to render
			// howMany: Integer
			//		How many rows to update.
			startIndex = Number(startIndex);
			howMany = Number(howMany);
			var i;
			if(this.updating){
				for(i=0; i<howMany; i++){
					this.invalidated[i+startIndex]=true;
				}
			}else{
				for(i=0; i<howMany; i++){
					this.views.updateRow(i+startIndex, this._skipRowRenormalize);
				}
				this.scroller.rowHeightChanged(startIndex);
			}
		},

		updateRowCount: function(inRowCount){
			//summary:
			//	Change the number of rows.
			// inRowCount: int
			//	Number of rows in the grid.
			if(this.updating){
				this.invalidated.rowCount = inRowCount;
			}else{
				this.rowCount = inRowCount;
				this._setAutoHeightAttr(this.autoHeight, true);
				if(this.layout.cells.length){
					this.scroller.updateRowCount(inRowCount);
				}
				this._resize();
				if(this.layout.cells.length){
					this.setScrollTop(this.scrollTop);
				}
			}
		},

		updateRowStyles: function(inRowIndex){
			// summary:
			//		Update the styles for a row after it's state has changed.
			this.views.updateRowStyles(inRowIndex);
		},
		getRowNode: function(inRowIndex){
			// summary:
			//		find the rowNode that is not a rowSelector
			if (this.focus.focusView && !(this.focus.focusView instanceof _RowSelector)){
					return this.focus.focusView.rowNodes[inRowIndex];
			}else{ // search through views
				for (var i = 0, cView; (cView = this.views.views[i]); i++) {
					if (!(cView instanceof _RowSelector)) {
						return cView.rowNodes[inRowIndex];
					}
				}
			}
			return null;
		},
		rowHeightChanged: function(inRowIndex){
			// summary:
			//		Update grid when the height of a row has changed. Row height is handled automatically as rows
			//		are rendered. Use this function only to update a row's height outside the normal rendering process.
			// inRowIndex: Integer
			// 		index of the row that has changed height

			this.views.renormalizeRow(inRowIndex);
			this.scroller.rowHeightChanged(inRowIndex);
		},

		// fastScroll: Boolean
		//		flag modifies vertical scrolling behavior. Defaults to true but set to false for slower
		//		scroll performance but more immediate scrolling feedback
		fastScroll: true,

		delayScroll: false,

		// scrollRedrawThreshold: int
		//	pixel distance a user must scroll vertically to trigger grid scrolling.
		scrollRedrawThreshold: (has("ie") ? 100 : 50),

		// scroll methods
		scrollTo: function(inTop){
			// summary:
			//		Vertically scroll the grid to a given pixel position
			// inTop: Integer
			//		vertical position of the grid in pixels
			if(!this.fastScroll){
				this.setScrollTop(inTop);
				return;
			}
			var delta = Math.abs(this.lastScrollTop - inTop);
			this.lastScrollTop = inTop;
			if(delta > this.scrollRedrawThreshold || this.delayScroll){
				this.delayScroll = true;
				this.scrollTop = inTop;
				this.views.setScrollTop(inTop);
				if(this._pendingScroll){
					window.clearTimeout(this._pendingScroll);
				}
				var _this = this;
				this._pendingScroll = window.setTimeout(function(){
					delete _this._pendingScroll;
					_this.finishScrollJob();
				}, 200);
			}else{
				this.setScrollTop(inTop);
			}
		},

		finishScrollJob: function(){
			this.delayScroll = false;
			this.setScrollTop(this.scrollTop);
		},

		setScrollTop: function(inTop){
			this.scroller.scroll(this.views.setScrollTop(inTop));
		},

		scrollToRow: function(inRowIndex){
			// summary:
			//		Scroll the grid to a specific row.
			// inRowIndex: Integer
			// 		grid row index
			this.setScrollTop(this.scroller.findScrollTop(inRowIndex) + 1);
		},

		// styling (private, used internally to style individual parts of a row)
		styleRowNode: function(inRowIndex, inRowNode){
			if(inRowNode){
				this.rows.styleRowNode(inRowIndex, inRowNode);
			}
		},
		
		// called when the mouse leaves the grid so we can deselect all hover rows
		_mouseOut: function(e){
			this.rows.setOverRow(-2);
		},
	
		// cells
		getCell: function(inIndex){
			// summary:
			//		Retrieves the cell object for a given grid column.
			// inIndex: Integer
			// 		Grid column index of cell to retrieve
			// returns:
			//		a grid cell
			return this.layout.cells[inIndex];
		},

		setCellWidth: function(inIndex, inUnitWidth){
			this.getCell(inIndex).unitWidth = inUnitWidth;
		},

		getCellName: function(inCell){
			// summary: Returns the cell name of a passed cell
			return "Cell " + inCell.index; // String
		},

		// sorting
		canSort: function(inSortInfo){
			// summary:
			//		Determines if the grid can be sorted
			// inSortInfo: Integer
			//		Sort information, 1-based index of column on which to sort, positive for an ascending sort
			// 		and negative for a descending sort
			// returns: Boolean
			//		True if grid can be sorted on the given column in the given direction
		},

		sort: function(){
		},

		getSortAsc: function(inSortInfo){
			// summary:
			//		Returns true if grid is sorted in an ascending direction.
			inSortInfo = inSortInfo == undefined ? this.sortInfo : inSortInfo;
			return Boolean(inSortInfo > 0); // Boolean
		},

		getSortIndex: function(inSortInfo){
			// summary:
			//		Returns the index of the column on which the grid is sorted
			if(this.sortInfo === 0 && this.sortFields){
				var columnField = this.sortFields[0].attribute;
				var sortIdx;
				array.some(this.layout.cells, function(c, idx){
					if(c.field === columnField){
						sortIdx = idx + 1;
						return true;
					}
					return false;
				});
				this.sortInfo = this.sortFields[0].descending ? -sortIdx : sortIdx;
			}
			inSortInfo = inSortInfo == undefined ? this.sortInfo : inSortInfo;
			return Math.abs(inSortInfo) - 1; // Integer
		},

		setSortIndex: function(inIndex, inAsc){
			// summary:
			// 		Sort the grid on a column in a specified direction
			// inIndex: Integer
			// 		Column index on which to sort.
			// inAsc: Boolean
			// 		If true, sort the grid in ascending order, otherwise in descending order
			var si = inIndex +1;
			if(inAsc != undefined){
				si *= (inAsc ? 1 : -1);
			} else if(this.getSortIndex() == inIndex){
				si = -this.sortInfo;
			}
			this.setSortInfo(si);
		},

		setSortInfo: function(inSortInfo){
			if(this.canSort(inSortInfo)){
				this.sortInfo = inSortInfo;
				this.sort();
				this.update();
			}
		},

		// DOM event handler
		doKeyEvent: function(e){
			e.dispatch = 'do' + e.type;
			this.onKeyEvent(e);
		},

		// event dispatch
		//: protected
		_dispatch: function(m, e){
			if(m in this){
				return this[m](e);
			}
			return false;
		},

		dispatchKeyEvent: function(e){
			this._dispatch(e.dispatch, e);
		},

		dispatchContentEvent: function(e){
			this.edit.dispatchEvent(e) || e.sourceView.dispatchContentEvent(e) || this._dispatch(e.dispatch, e);
		},

		dispatchHeaderEvent: function(e){
			e.sourceView.dispatchHeaderEvent(e) || this._dispatch('doheader' + e.type, e);
		},

		dokeydown: function(e){
			this.onKeyDown(e);
		},

		doclick: function(e){
			if(e.cellNode){
				this.onCellClick(e);
			}else{
				this.onRowClick(e);
			}
		},

		dodblclick: function(e){
			if(e.cellNode){
				this.onCellDblClick(e);
			}else{
				this.onRowDblClick(e);
			}
		},

		docontextmenu: function(e){
			if(e.cellNode){
				this.onCellContextMenu(e);
			}else{
				this.onRowContextMenu(e);
			}
		},

		doheaderclick: function(e){
			if(e.cellNode){
				this.onHeaderCellClick(e);
			}else{
				this.onHeaderClick(e);
			}
		},

		doheaderdblclick: function(e){
			if(e.cellNode){
				this.onHeaderCellDblClick(e);
			}else{
				this.onHeaderDblClick(e);
			}
		},

		doheadercontextmenu: function(e){
			if(e.cellNode){
				this.onHeaderCellContextMenu(e);
			}else{
				this.onHeaderContextMenu(e);
			}
		},

		// override to modify editing process
		doStartEdit: function(inCell, inRowIndex){
			this.onStartEdit(inCell, inRowIndex);
		},

		doApplyCellEdit: function(inValue, inRowIndex, inFieldIndex){
			this.onApplyCellEdit(inValue, inRowIndex, inFieldIndex);
		},

		doCancelEdit: function(inRowIndex){
			this.onCancelEdit(inRowIndex);
		},

		doApplyEdit: function(inRowIndex){
			this.onApplyEdit(inRowIndex);
		},

		// row editing
		addRow: function(){
			// summary:
			//		Add a row to the grid.
			this.updateRowCount(this.get('rowCount')+1);
		},

		removeSelectedRows: function(){
			// summary:
			//		Remove the selected rows from the grid.
			if(this.allItemsSelected){
				this.updateRowCount(0);
			}else{
				this.updateRowCount(Math.max(0, this.get('rowCount') - this.selection.getSelected().length));
			}
			this.selection.clear();
		}

	});

	_Grid.markupFactory = function(props, node, ctor, cellFunc){
		var widthFromAttr = function(n){
			var w = html.attr(n, "width")||"auto";
			if((w != "auto")&&(w.slice(-2) != "em")&&(w.slice(-1) != "%")){
				w = parseInt(w, 10)+"px";
			}
			return w;
		};
		// if(!props.store){ console.debug("no store!"); }
		// if a structure isn't referenced, do we have enough
		// data to try to build one automatically?
		if(	!props.structure &&
			node.nodeName.toLowerCase() == "table"){

			// try to discover a structure
			props.structure = query("> colgroup", node).map(function(cg){
				var sv = html.attr(cg, "span");
				var v = {
					noscroll: (html.attr(cg, "noscroll") == "true") ? true : false,
					__span: (!!sv ? parseInt(sv, 10) : 1),
					cells: []
				};
				if(html.hasAttr(cg, "width")){
					v.width = widthFromAttr(cg);
				}
				return v; // for vendetta
			});
			if(!props.structure.length){
				props.structure.push({
					__span: Infinity,
					cells: [] // catch-all view
				});
			}
			// check to see if we're gonna have more than one view

			// for each tr in our th, create a row of cells
			query("thead > tr", node).forEach(function(tr, tr_idx){
				var cellCount = 0;
				var viewIdx = 0;
				var lastViewIdx;
				var cView = null;
				query("> th", tr).map(function(th){
					// what view will this cell go into?

					// NOTE:
					//		to prevent extraneous iteration, we start counters over
					//		for each row, incrementing over the surface area of the
					//		structure that colgroup processing generates and
					//		creating cell objects for each <th> to place into those
					//		cell groups.  There's a lot of state-keepking logic
					//		here, but it is what it has to be.
					if(!cView){ // current view book keeping
						lastViewIdx = 0;
						cView = props.structure[0];
					}else if(cellCount >= (lastViewIdx+cView.__span)){
						viewIdx++;
						// move to allocating things into the next view
						lastViewIdx += cView.__span;
						var lastView = cView;
						cView = props.structure[viewIdx];
					}

					// actually define the cell from what markup hands us
					var cell = {
						name: lang.trim(html.attr(th, "name")||th.innerHTML),
						colSpan: parseInt(html.attr(th, "colspan")||1, 10),
						type: lang.trim(html.attr(th, "cellType")||""),
						id: lang.trim(html.attr(th,"id")||"")
					};
					cellCount += cell.colSpan;
					var rowSpan = html.attr(th, "rowspan");
					if(rowSpan){
						cell.rowSpan = rowSpan;
					}
					if(html.hasAttr(th, "width")){
						cell.width = widthFromAttr(th);
					}
					if(html.hasAttr(th, "relWidth")){
						cell.relWidth = window.parseInt(html.attr(th, "relWidth"), 10);
					}
					if(html.hasAttr(th, "hidden")){
						cell.hidden = (html.attr(th, "hidden") == "true" || html.attr(th, "hidden") === true/*always boolean true in Chrome*/);
					}

					if(cellFunc){
						cellFunc(th, cell);
					}

					cell.type = cell.type ? lang.getObject(cell.type) : dojox.grid.cells.Cell;

					if(cell.type && cell.type.markupFactory){
						cell.type.markupFactory(th, cell);
					}

					if(!cView.cells[tr_idx]){
						cView.cells[tr_idx] = [];
					}
					cView.cells[tr_idx].push(cell);
				});
			});
		}

		return new ctor(props, node);
	};

	return _Grid;

});

},
'url:dojox/grid/resources/Expando.html':"<div class=\"dojoxGridExpando\"\r\n\t><div class=\"dojoxGridExpandoNode\" dojoAttachEvent=\"onclick:onToggle\"\r\n\t\t><div class=\"dojoxGridExpandoNodeInner\" dojoAttachPoint=\"expandoInner\"></div\r\n\t></div\r\n></div>\r\n",
'dijit/_editor/html':function(){
define([
	"dojo/_base/lang", // lang.isString
	"dojo/_base/sniff", // has("ie")
	".."		// for exporting symbols to dijit._editor (remove for 2.0)
], function(lang, has, dijit){

// module:
//		dijit/_editor/html
// summary:
//		Utility functions used by editor

lang.getObject("_editor", true, dijit);

dijit._editor.escapeXml=function(/*String*/str, /*Boolean?*/noSingleQuotes){
	// summary:
	//		Adds escape sequences for special characters in XML: &<>"'
	//		Optionally skips escapes for single quotes
	str = str.replace(/&/gm, "&amp;").replace(/</gm, "&lt;").replace(/>/gm, "&gt;").replace(/"/gm, "&quot;");
	if(!noSingleQuotes){
		str = str.replace(/'/gm, "&#39;");
	}
	return str; // string
};

dijit._editor.getNodeHtml=function(/* DomNode */node){
	var output;
	switch(node.nodeType){
		case 1: //element node
			var lName = node.nodeName.toLowerCase();
			if(!lName || lName.charAt(0) == "/"){
				// IE does some strange things with malformed HTML input, like
				// treating a close tag </span> without an open tag <span>, as
				// a new tag with tagName of /span.  Corrupts output HTML, remove
				// them.  Other browsers don't prefix tags that way, so will
				// never show up.
				return "";
			}
			output = '<' + lName;

			//store the list of attributes and sort it to have the
			//attributes appear in the dictionary order
			var attrarray = [];
			var attr;
			if(has("ie") && node.outerHTML){
				var s = node.outerHTML;
				s = s.substr(0, s.indexOf('>'))
					.replace(/(['"])[^"']*\1/g, ''); //to make the following regexp safe
				var reg = /(\b\w+)\s?=/g;
				var m, key;
				while((m = reg.exec(s))){
					key = m[1];
					if(key.substr(0,3) != '_dj'){
						if(key == 'src' || key == 'href'){
							if(node.getAttribute('_djrealurl')){
								attrarray.push([key,node.getAttribute('_djrealurl')]);
								continue;
							}
						}
						var val, match;
						switch(key){
							case 'style':
								val = node.style.cssText.toLowerCase();
								break;
							case 'class':
								val = node.className;
								break;
							case 'width':
								if(lName === "img"){
									// This somehow gets lost on IE for IMG tags and the like
									// and we have to find it in outerHTML, known IE oddity.
									match=/width=(\S+)/i.exec(s);
									if(match){
										val = match[1];
									}
									break;
								}
							case 'height':
								if(lName === "img"){
									// This somehow gets lost on IE for IMG tags and the like
									// and we have to find it in outerHTML, known IE oddity.
									match=/height=(\S+)/i.exec(s);
									if(match){
										val = match[1];
									}
									break;
								}
							default:
								val = node.getAttribute(key);
						}
						if(val != null){
							attrarray.push([key, val.toString()]);
						}
					}
				}
			}else{
				var i = 0;
				while((attr = node.attributes[i++])){
					//ignore all attributes starting with _dj which are
					//internal temporary attributes used by the editor
					var n = attr.name;
					if(n.substr(0,3) != '_dj' /*&&
						(attr.specified == undefined || attr.specified)*/){
						var v = attr.value;
						if(n == 'src' || n == 'href'){
							if(node.getAttribute('_djrealurl')){
								v = node.getAttribute('_djrealurl');
							}
						}
						attrarray.push([n,v]);
					}
				}
			}
			attrarray.sort(function(a,b){
				return a[0] < b[0] ? -1 : (a[0] == b[0] ? 0 : 1);
			});
			var j = 0;
			while((attr = attrarray[j++])){
				output += ' ' + attr[0] + '="' +
					(lang.isString(attr[1]) ? dijit._editor.escapeXml(attr[1], true) : attr[1]) + '"';
			}
			if(lName === "script"){
				// Browsers handle script tags differently in how you get content,
				// but innerHTML always seems to work, so insert its content that way
				// Yes, it's bad to allow script tags in the editor code, but some people
				// seem to want to do it, so we need to at least return them right.
				// other plugins/filters can strip them.
				output += '>' + node.innerHTML +'</' + lName + '>';
			}else{
				if(node.childNodes.length){
					output += '>' + dijit._editor.getChildrenHtml(node)+'</' + lName +'>';
				}else{
					switch(lName){
						case 'br':
						case 'hr':
						case 'img':
						case 'input':
						case 'base':
						case 'meta':
						case 'area':
						case 'basefont':
							// These should all be singly closed
							output += ' />';
							break;
						default:
							// Assume XML style separate closure for everything else.
							output += '></' + lName + '>';
					}
				}
			}
			break;
		case 4: // cdata
		case 3: // text
			// FIXME:
			output = dijit._editor.escapeXml(node.nodeValue, true);
			break;
		case 8: //comment
			// FIXME:
			output = '<!--' + dijit._editor.escapeXml(node.nodeValue, true) + '-->';
			break;
		default:
			output = "<!-- Element not recognized - Type: " + node.nodeType + " Name: " + node.nodeName + "-->";
	}
	return output;
};

dijit._editor.getChildrenHtml = function(/* DomNode */dom){
	// summary:
	//		Returns the html content of a DomNode and children
	var out = "";
	if(!dom){ return out; }
	var nodes = dom["childNodes"] || dom;

	//IE issue.
	//If we have an actual node we can check parent relationships on for IE,
	//We should check, as IE sometimes builds invalid DOMS.  If no parent, we can't check
	//And should just process it and hope for the best.
	var checkParent = !has("ie") || nodes !== dom;

	var node, i = 0;
	while((node = nodes[i++])){
		//IE is broken.  DOMs are supposed to be a tree.  But in the case of malformed HTML, IE generates a graph
		//meaning one node ends up with multiple references (multiple parents).  This is totally wrong and invalid, but
		//such is what it is.  We have to keep track and check for this because otherise the source output HTML will have dups.
		//No other browser generates a graph.  Leave it to IE to break a fundamental DOM rule.  So, we check the parent if we can
		//If we can't, nothing more we can do other than walk it.
		if(!checkParent || node.parentNode == dom){
			out += dijit._editor.getNodeHtml(node);
		}
	}
	return out; // String
};

return dijit._editor;
});

},
'url:dijit/templates/ProgressBar.html':"<div class=\"dijitProgressBar dijitProgressBarEmpty\" role=\"progressbar\"\n\t><div  data-dojo-attach-point=\"internalProgress\" class=\"dijitProgressBarFull\"\n\t\t><div class=\"dijitProgressBarTile\" role=\"presentation\"></div\n\t\t><span style=\"visibility:hidden\">&#160;</span\n\t></div\n\t><div data-dojo-attach-point=\"labelNode\" class=\"dijitProgressBarLabel\" id=\"${id}_label\"></div\n\t><img data-dojo-attach-point=\"indeterminateHighContrastImage\" class=\"dijitProgressBarIndeterminateHighContrastImage\" alt=\"\"\n/></div>\n",
'dijit/layout/TabController':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.setSelectable
	"dojo/dom-attr", // domAttr.attr
	"dojo/dom-class", // domClass.toggle
	"dojo/i18n", // i18n.getLocalization
	"dojo/_base/lang", // lang.hitch lang.trim
	"./StackController",
	"../Menu",
	"../MenuItem",
	"dojo/text!./templates/_TabButton.html",
	"dojo/i18n!../nls/common"
], function(declare, dom, domAttr, domClass, i18n, lang, StackController, Menu, MenuItem, template){

/*=====
	var StackController = dijit.layout.StackController;
	var Menu = dijit.Menu;
	var MenuItem = dijit.MenuItem;
=====*/

	// module:
	//		dijit/layout/TabController
	// summary:
	// 		Set of tabs (the things with titles and a close button, that you click to show a tab panel).
	//		Used internally by `dijit.layout.TabContainer`.

	var TabButton = declare("dijit.layout._TabButton", StackController.StackButton, {
		// summary:
		//		A tab (the thing you click to select a pane).
		// description:
		//		Contains the title of the pane, and optionally a close-button to destroy the pane.
		//		This is an internal widget and should not be instantiated directly.
		// tags:
		//		private

		// baseClass: String
		//		The CSS class applied to the domNode.
		baseClass: "dijitTab",

		// Apply dijitTabCloseButtonHover when close button is hovered
		cssStateNodes: {
			closeNode: "dijitTabCloseButton"
		},

		templateString: template,

		// Override _FormWidget.scrollOnFocus.
		// Don't scroll the whole tab container into view when the button is focused.
		scrollOnFocus: false,

		buildRendering: function(){
			this.inherited(arguments);

			dom.setSelectable(this.containerNode, false);
		},

		startup: function(){
			this.inherited(arguments);
			var n = this.domNode;

			// Required to give IE6 a kick, as it initially hides the
			// tabs until they are focused on.
			setTimeout(function(){
				n.className = n.className;
			}, 1);
		},

		_setCloseButtonAttr: function(/*Boolean*/ disp){
			// summary:
			//		Hide/show close button
			this._set("closeButton", disp);
			domClass.toggle(this.innerDiv, "dijitClosable", disp);
			this.closeNode.style.display = disp ? "" : "none";
			if(disp){
				var _nlsResources = i18n.getLocalization("dijit", "common");
				if(this.closeNode){
					domAttr.set(this.closeNode,"title", _nlsResources.itemClose);
				}
				// add context menu onto title button
				this._closeMenu = new Menu({
					id: this.id+"_Menu",
					dir: this.dir,
					lang: this.lang,
					textDir: this.textDir,
					targetNodeIds: [this.domNode]
				});

				this._closeMenu.addChild(new MenuItem({
					label: _nlsResources.itemClose,
					dir: this.dir,
					lang: this.lang,
					textDir: this.textDir,
					onClick: lang.hitch(this, "onClickCloseButton")
				}));
			}else{
				if(this._closeMenu){
					this._closeMenu.destroyRecursive();
					delete this._closeMenu;
				}
			}
		},
		_setLabelAttr: function(/*String*/ content){
			// summary:
			//		Hook for set('label', ...) to work.
			// description:
			//		takes an HTML string.
			//		Inherited ToggleButton implementation will Set the label (text) of the button;
			//		Need to set the alt attribute of icon on tab buttons if no label displayed
			this.inherited(arguments);
			if(!this.showLabel && !this.params.title){
				this.iconNode.alt = lang.trim(this.containerNode.innerText || this.containerNode.textContent || '');
			}
		},

		destroy: function(){
			if(this._closeMenu){
				this._closeMenu.destroyRecursive();
				delete this._closeMenu;
			}
			this.inherited(arguments);
		}
	});

	var TabController = declare("dijit.layout.TabController", StackController, {
		// summary:
		// 		Set of tabs (the things with titles and a close button, that you click to show a tab panel).
		//		Used internally by `dijit.layout.TabContainer`.
		// description:
		//		Lets the user select the currently shown pane in a TabContainer or StackContainer.
		//		TabController also monitors the TabContainer, and whenever a pane is
		//		added or deleted updates itself accordingly.
		// tags:
		//		private

		baseClass: "dijitTabController",

		templateString: "<div role='tablist' data-dojo-attach-event='onkeypress:onkeypress'></div>",

		// tabPosition: String
		//		Defines where tabs go relative to the content.
		//		"top", "bottom", "left-h", "right-h"
		tabPosition: "top",

		// buttonWidget: Constructor
		//		The tab widget to create to correspond to each page
		buttonWidget: TabButton,

		_rectifyRtlTabList: function(){
			// summary:
			//		For left/right TabContainer when page is RTL mode, rectify the width of all tabs to be equal, otherwise the tab widths are different in IE

			if(0 >= this.tabPosition.indexOf('-h')){ return; }
			if(!this.pane2button){ return; }

			var maxWidth = 0;
			for(var pane in this.pane2button){
				var ow = this.pane2button[pane].innerDiv.scrollWidth;
				maxWidth = Math.max(maxWidth, ow);
			}
			//unify the length of all the tabs
			for(pane in this.pane2button){
				this.pane2button[pane].innerDiv.style.width = maxWidth + 'px';
			}
		}
	});

	TabController.TabButton = TabButton;	// for monkey patching

	return TabController;
});

},
'ibm/tivoli/navmodel/NotebookWidgetDef':function(){
// wrapped by build app
define("ibm/tivoli/navmodel/NotebookWidgetDef", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/WidgetDef"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.NotebookWidgetDef"); 
 
// includes
dojo.require("ibm.tivoli.navmodel.WidgetDef");

////////////////////////////////////
// NavWidgetDef
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.NotebookWidgetDef", ibm.tivoli.navmodel.WidgetDef, {

	/*=====
	constructor_kwArgs: function(baseUrl, provider, datasource, widget, definition, type, version) {
		// baseUrl: String
		//		base url for identifying rest service
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// widget: String
		//		widget type in form <type>-<version>
		// definition: String
		//		widget definition id in form <id>-<version>
		// type: String
		//		definition type:  "user", "shared", or "base"
		// version: Float
		//		version used to communicate with rest service
	},
	=====*/
	
	constructor: function(/*constructor_kwArgs*/ kwArgs) {		
		// summary:
		//		Constructs NotebookWidgetDef object.
	},

	/////////////////////////////
	// public methods
	/////////////////////////////

	destroy: function() {		
		// summary:
		//		Do any necessary clean up
		if (!this.isDestroyed()) {
			// call base class
			this.inherited(arguments);
		}
	},
	
	getTabOrientation: function() {
		var returnValue = this._widgetDef.notebook.tabOrientation;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
		 	if (parent && parent.getTabOrientation) {
				returnValue = parent.getTabOrientation();
			}
		}		
		return returnValue;			
	}
	
	/////////////////////////////
	// private methods
	/////////////////////////////	

});

});

},
'ibm/tivoli/navmodel/NavWidgetDef':function(){
// wrapped by build app
define("ibm/tivoli/navmodel/NavWidgetDef", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/WidgetDef"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.NavWidgetDef"); 
 
// includes
dojo.require("ibm.tivoli.navmodel.WidgetDef");

////////////////////////////////////
// NavWidgetDef
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.NavWidgetDef", ibm.tivoli.navmodel.WidgetDef, {
	
	/*=====
	constructor_kwArgs: function(baseUrl, provider, datasource, widget, definition, type, version) {
		// baseUrl: String
		//		base url for identifying rest service
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// widget: String
		//		widget type in form <type>-<version>
		// definition: String
		//		widget definition id in form <id>-<version>
		// type: String
		//		definition type:  "user", "shared", or "base"
		// version: Float
		//		version used to communicate with rest service
	},
	=====*/ 
	
	constructor: function(/*constructor_kwArgs*/ kwArgs) {		
		// summary:
		//		
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	getBreadcrumbs: function(){
		var returnValue = this._widgetDef.navwidget.breadcrumbs;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
		 	if (parent && parent.getBreadcrumbs) {
				returnValue = parent.getBreadcrumbs();
			}
		}		
		return returnValue;			
	},
	
	getNotebook: function(){
		var returnValue = this._widgetDef.navwidget.notebook;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
		 	if (parent && parent.getNotebook) {
				returnValue = parent.getNotebook();
			}
		}		
		return returnValue;			
	},
	
	getViewsLayout: function(){
		var returnValue = this._widgetDef.navwidget.viewsLayout;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
		 	if (parent && parent.getViewsLayout) {
				returnValue = parent.getViewsLayout();
			}
		}		
		return returnValue;					
	},
	

	destroy: function() {		
		// summary:
		//		Do any necessary clean up 
		if (!this.isDestroyed()) {
			// call base class
			this.inherited(arguments);
		}
	}
	
	/////////////////////////////
	// private methods
	/////////////////////////////

});

});

},
'ibm/usmi/widgets/TreeSelector':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/widgets/TreeSelector", [
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/array",
	"dojo/topic",
	"dojo/_base/connect",
	"dijit/_WidgetBase",
	"dijit/layout/ContentPane",
	"dijit/layout/BorderContainer",
	// TWL imports
	"ibm/tivoli/navmodel/Dataset",
	"ibm/usmi/util/DefaultMessageHandler",
	"ibm/tivoli/tip/TCTTable",
	"ibm/tivoli/tip/navwidget/tree/dijit/TreeTableView",
	"ibm/tivoli/tip/navwidget/dijit/NavWidgetPluggableView",
	"ibm/tivoli/navmodel/Selection",
	// NextGen imports
	"ibm/usmi/util/BaseWidget",
	"ibm/usmi/widgets/util/Constants",
	"ibm/usmi/core/utils/_UsmiBrowserPersistence",
	"ibm/usmi/core/utils/Logger",
	"ibm/usmi/widgets/util/ResourceNavigatorDAHelper",
	"ibm/usmi/widgets/util/DefaultContextMenuProvider",
	"ibm/usmi/widgets/util/ResourceMenuProviderHelper"
], function (dojo, declare, lang, array, topic, connect, _WidgetBase, ContentPane, BorderContainer, Dataset,
		DefaultMessageHandler, TCTTable, TreeTableView, NavWidgetPluggableView, Selection, BaseWidget, Constants,
		_UsmiBrowserPersistence, Logger, ResourceNavigatorDAHelper, DefaultContextMenuProvider, ResourceMenuProviderHelper) {

	return declare([
		_WidgetBase,
		BaseWidget,
		BorderContainer,
		NavWidgetPluggableView,
		_UsmiBrowserPersistence
	], {

        _attempts: 0,
		treemodel : null,

		// attributes that can be set in constructor, html, or url tag
        style : "width:100%;height:100%;",

		// Defaults
		DEFAULT_COLLECTION : "RESOURCES_TREE_SELECTOR",

		///////////////////////////////////////////////////////////////////
		// private members
		///////////////////////////////////////////////////////////////////

		// the logger
		_treeSelectorLogger : null,

		// handle to TWL TreeTableView
		_tree : null,

		selectorTitle : null,

		selectorHome : null,

		_navRules : null,

		includeGroupNode : true,
		
		defaultActionHelper : null,
		
		// cmProvider: Custom context menu provider (must be provided on construction to be used)
		cmProvider : null,
		
        	//Subscriptions to clean up
        	_subscriptions : null,
        
        	// Id of resource group currently selected
       		_prevSourceToken : "",

		// default widget definition: found com.ibm.usmi.console.dojo.data\src\main\resources\widgetdefs\widgets
		_DEFAULT_WIDGET_DEF_INFO : {
			className : "ibm.tivoli.navmodel.TreeTableWidgetDef",
			baseUrl : Constants.BASE_REST_URL,
			provider : "any",
			datasource : "any",
			widget : "treetable-0.1.0",
			definition : "UsmiTreeTable-0.1.0"
		},

		constructor : function (/*constructor_kwArgs*/kwArgs) {
			// summary:
			//        Constructs Tree Selector object.
			
			//enabling logger for convenience. uncomment to use
			//dojoConfig.trace["ibm.usmi.widgets"]="finest";
			this._treeSelectorLogger = new Logger("ibm.usmi.widgets", "TreeSelector");
			
			
			if (this._treeSelectorLogger.isFinest()) {
				this._treeSelectorLogger.entering("", "constructor");
			}
			
			//        this.loadWidgetDefs();
			//We don't have a widget def specific to this view, so let's signal to
			//super class not to try and load any widget defs on our behalf
			this._usesWidgetDefinition = true;
			this.groupID = this.getInitArg(Constants.TREE_SELECTOR_GROUP);
			//only overwrite if defined, should make a baseWidget method for this
			if (this.getInitArg("INCLUDE_GROUP_NODE") !== undefined) {
				this.includeGroupNode = this.getInitArg("INCLUDE_GROUP_NODE");
			}

			if (this._treeSelectorLogger.isFinest()) {
				this._treeSelectorLogger.exiting("", "constructor");
			}
		},

		postCreate : function () {
			// summary:
			//         Called after a widget's dom has been setup
			// tags:
			//        protected
			//        this.initPersistence();
			
			this._subscriptions = [];
			
			this.inherited(arguments);
			
			var selectorTitleArg = this.getInitArg("selectorTitle"), selectorHomeArg = this.getInitArg("selectorHome");
			if (selectorTitleArg) {
				this.selectorTitle = selectorTitleArg;
			}

			if (selectorHomeArg) {
				this.selectorHome = selectorHomeArg;
			}
			
			// construct dataset, which once oncomplete, tree creation is kicked off
			this._makeDataset();

		},

		startup : function () {
			// summary:
			//         Called after any DOM fragments have been actually added to the document
			// tags:
			//        protected
			this.inherited(arguments);
		},

		destroy : function () {
			// summary:
			//        Clean up any objects the tree selector is hanging onto

			if (this._treeSelectorLogger.isFinest()) {
				this._treeSelectorLogger.entering("", "destroy");
			}
            		var idx = 0;
            		for (; idx < this._subscriptions.length; idx++) {
                		this._subscriptions[idx].remove();
            		}
            		this._subscriptions.length = 0;
            
			this.inherited(arguments);

			if (this._treeSelectorLogger.isFinest()) {
				this._treeSelectorLogger.exiting("", "destroy");
			}
		},

		_parseTreeNavRules : function (dataset) {
			if (this._treeSelectorLogger.isFinest()) {
				this._treeSelectorLogger.entering("", "_parseTreeNavRules");
			}

			var navstring, navobject;
			navstring = dataset.getAttribute("TreeSelectorRules");
            if (navstring) {
                navobject = JSON.parse(navstring);
                this._navRules = navobject;
            }

			if (this._treeSelectorLogger.isFinest()) {
				this._treeSelectorLogger.exiting("", "_parseTreeNavRules");
			}

		},

		_makeTree : function (dataset) {
			// summary:  
			//        Construct TWL TreeTable view with passed in dataset, process sourcetoken for 
			//		  pre-selection

			if (this._treeSelectorLogger.isFinest()) {
				this._treeSelectorLogger.entering("", "_makeTree");
			}

			//Destroy old tree
			if (this._tree) {
				this.removeChild(this._tree);
				this._tree.destroyRecursive();
				this._tree = null;
			}

			var stateString, state;

			if (this.widgetDefinition) {
				if (this.selectorTitle) {
					// Set the title of tree from the parameter passed in
					this.widgetDefinition.getDataSet()[0].columns[0].label.value = this.selectorTitle;
				} // If the dataset returns a title, set it to the tree.
				else if (dataset._dataset.label) {
					this.widgetDefinition.getDataSet()[0].columns[0].label.value = dataset._dataset.label;
				}
			}
			
			// create a default context menu provider
			this.defaultActionHelper = new ResourceNavigatorDAHelper({
				noOp: true
			});

			
			// get state information out of sessin persistence and set on tree
			// TODO once fix in in from TWL do state stuff
			//stateString = this.getData("treeKey", true);
			if (stateString) {
				state = dojo.fromJson(stateString);
				this._tree = new TreeTableView({
					id : "_treeTableView" + this.id,
					style : "width:100%;height:100%;",
					dataset : dataset,
					removeContextMenu: true,
					daHelper : this.defaultActionHelper,
					widgetDefinition : this.widgetDefinition,
					state : state
				});
			} else {
				this._tree = new TreeTableView({
					id : "_treeTableView" + this.id,
					style : "width:100%;height:100%;",
					dataset : dataset,
					removeContextMenu: true,
					daHelper : this.defaultActionHelper,
					widgetDefinition : this.widgetDefinition
				});
			}

			this.addChild(this._tree);
			
			// listen for when Tree is in charge, and unselecto tree topper selection
			this._subscriptions.push(topic.subscribe("TREETOPPERINCHARGE", dojo.hitch(this, this._removeSelection)));   

			// Listen for hash updates
			this._subscriptions.push(topic.subscribe(Constants.TOPIC_HASH_CHANGE, dojo.hitch(this, this._hashChanged)));

			//Listen for selection change
			dojo.connect(this._tree.selection, Constants.SELECTION_CHANGE, dojo.hitch(this,
					this._treeSelectionChangedHandler));

						
			// get sourceToken from hash, and reopen / seelct branch if passed in
			hashSoureToken = this.getUrlHashParm("sourceToken");
			
			// Override _initTableBuilder so that we can connect to onTreeTableLoad event.
            // This is needed so we can know when the data is there, in order to auto-expand nodes.
            var old = this._tree._initTableBuilder;
            this._tree._initTableBuilder = lang.hitch(this, function (/*String*/providerId, /*String*/
            datasourceId, columnIds, datasetColumns, extraColData) {
                old.call(this._tree, providerId, datasourceId, columnIds, datasetColumns, extraColData);
                
                // set the prevSourceToken so we dont kick off another by hashlistener
                this._prevSourceToken = hashSoureToken;
                // expand/preselect the tree if sourcetoken was specified on url 
                this._initialSelection(dataset, hashSoureToken);

            });

			this._parseTreeNavRules(dataset);
			
			if (this._treeSelectorLogger.isFinest()) {
				this._treeSelectorLogger.exiting("", "_makeTree");
			}
		},
		
		_makeDataset : function () {
			// summary:
			//        Get the dataset that was passed in on URL or constructor, default to RESOURCES_TREE_SELECTOR
			//        Construct TWL TreeTable view and set dataset

			if (this._treeSelectorLogger.isFinest()) {
				this._treeSelectorLogger.entering("", "_makeDataset");
			}

			var msgHandler, hashSoureToken, _processDataSet = function (dataset) {			
				// now we have the dataset, lets create the tree
				this._makeTree(dataset);
			};

			if (!this.providerID) {
				// default
				this.providerID = Constants.DEFAULT_PROVIDER;
			}

			if (!this.selectorHome) {
				// default
				this.selectorHome = this.DEFAULT_COLLECTION;
			}

			if (!this.groupID) {
				// default
				this.groupID = "ROOT";
			}

			try {
				this.dataset = new Dataset({
					onComplete : dojo.hitch(this, _processDataSet),
					baseUrl : Constants.BASE_REST_URL,
					provider : this.providerID,
					datasource : "",
					dataset : this.selectorHome,
					parameters : {
						INCLUDE_GROUP_NODE : this.includeGroupNode
					},
					//parameters: {},
					defaultProperties : []
				});
			} catch (ex) {
				//There was a problem creating the dataset
				msgHandler = new DefaultMessageHandler({});
				msgHandler.displayException(ex);
			}


			if (this._treeSelectorLogger.isFinest()) {
				this._treeSelectorLogger.exiting("", "_makeDataset");
			}
		},

		_treeSelectionChangedHandler : function (objectContext) {
            //this selection handler gets called twice, once with 0 selection and 
            //once with real selection
            //this if guards the real function b/c settimeout always passes the latter parm
            //TODO: learn a better way to control what gets passed thru setTimeout             
            if (objectContext.getItems() && objectContext.getItems().length>0){
			// get of render thread so mouse click event can complete
                setTimeout(dojo.hitch(this, this._selectionChangedHandler, objectContext), 1);
            }
		},
		
		_selectionChangedHandler : function (objectContext) {
			// summary:
			//        Called when a node is selected in the the tree selector
			// tags:
			//        callback

			if (this._treeSelectorLogger.isFiner()) {
				this._treeSelectorLogger.entering("", "_treeSelectionChangedHandler");
			}
			
			var i, task, _processNavNewDataSet, _handleNavNewDataSetError, getTasksComplete, getTasksError, dataset;
			var sourceToken, stateObj, myHashUpdate, myHashUpdateObject, nodeType, item, numRules, foundMatch, _processNavRule;
			var localHashObject = {}, taskParmsString, retargetInfoObject = {}, datasetPublished=false;

            // with the communication between the topper and the tree, when topper tells the tree
            // to deselect, it fires of a selectionChange, so we're getting called sayin the tree is in
            // charge

            if(this._tree._table && this._tree._table.getSelectedRowCount() > 0) {
                // tell tree I'm in charge
                topic.publish("TREEINCHARGE", "true"); //tell tree topper I am in charge

			_processNavNewDataSet = function (dataset) {
				if (this._treeSelectorLogger.isFiner()) {
					this._treeSelectorLogger.finer("publish dataset: " + dataset.id, "_treeSelectionChangedHandler");
				}
				retargetInfoObject[Constants.RETARGET_INFO_DATASET] = dataset;
				topic.publish(Constants.TOPIC_RETARGET_INFO, retargetInfoObject); //tell workarea to switch it up
				//topic.publish(Constants.TOPIC_RETARGET, dataset);
			};

			_handleNavNewDataSetError = function (error, request) {
				//There was a problem creating the dataset
				var msgHandler = new DefaultMessageHandler({});
				msgHandler.displayException(error);
			};

			foundMatch = false;
			item = objectContext.getItems()[0];
			nodeType = item.getAttribute("type");

			_processNavRule = function (navRule) {
				//could almost use for/in here, 
				//but the mapping to java to js isn't exact science
				retargetInfoObject[Constants.HASH_TASK_SELECTION] = navRule.taskWidgetId;

				if (navRule.taskParms) {
					if (lang.isObject(navRule.taskParms)) {
						//need to cycle through hashparms and add individually to the hash
						for ( var parmKey in navRule.taskParms) {
							retargetInfoObject[parmKey] = navRule.taskParms[parmKey];
						}
					} else {
						this._treeSelectorLogger
								.severe("TreeSelectorRules taskparms must be an object with key value pairs");
						//show error here too?
					}
				}
				if (navRule[Constants.HASH_TASK_PATH_KEY] && navRule[Constants.HASH_TASK_PATH]) {
					retargetInfoObject[Constants.HASH_TASK_PATH_KEY] = navRule[Constants.HASH_TASK_PATH_KEY];
					retargetInfoObject[Constants.HASH_TASK_PATH] = navRule[Constants.HASH_TASK_PATH];
				}
			};

			if (this._navRules && lang.isArrayLike(this._navRules)) {
				array.forEach(this._navRules, function (navRule) {
					if (!foundMatch && navRule.nodeType === nodeType) {
						foundMatch = true;
						_processNavRule(navRule);
						//enf not visible. see if i can pass it in
						//enf this._treeSelectorLogger.finest("***found nodetype in our map from array loop");
						//TODO: also need to break, could use array.some but may not be worth it
					}
				});
			} else if (this._navRules && this._navRules.nodeType === nodeType) {
				this._treeSelectorLogger.finest("***found nodetype in our map");
				foundMatch = true;
				_processNavRule(this._navRules);
			} else {
				this._treeSelectorLogger.finest("***nodetype NOT in map. Will check default nav hint");
			}

			if (!foundMatch) {
				var oid, theNode = objectContext.getItems()[0];
				if (theNode.getType() === "Group"){
					this._treeSelectorLogger.finest("new group check and publishing of oid");
					// lets publish the groupOID: string
					oid=theNode.getId();
				    retargetInfoObject[Constants.HASH_TASK_SELECTION] = "ibm/usmi/widgets/ResourceNavigator";
					retargetInfoObject[Constants.COLLECTION] = "GROUP_OID:"+oid;				
				}
				else{
				//Retrieve tasks
				dataset = objectContext.getDataset();
				//get default task more efficiently
				task = objectContext.getItems()[0].getDefaultTask();
				//use task info to set the hash so workArea can do work
				if (task && task.isNavigation()) {
					//This should be handled by the generic navigate below,
					//but if the RELATIONSHIP isn't in the related resources list, it will fail. badly.
					//see UsmiNavigationHint.java for the differences in processing of baserules vs general rules 
					if (task.getId().startWith("RELATIONSHIP:")) {
						this._treeSelectorLogger.finest("***Found default nav hint. " + task.getId());
						//not entirely sure this is needed var subbedString=task.getId().replace(/:/g, "_");
						retargetInfoObject[Constants.HASH_TASK_SELECTION] = "ibm/usmi/widgets/RelatedResourcesTable";
						retargetInfoObject[Constants.HASH_TASK_RELATIONSHIP] = task.getId();
					} else {
						this._treeSelectorLogger.finest("***Found default nav hint. navigate and publish dataset");
						retargetInfoObject[Constants.HASH_TASK_SELECTION] = "ibm/usmi/widgets/ResourceNavigator";
						datasetPublished=true;
						//navigate will give us back the new dataset. 
						//We'll in turn set that on the resource navigator
						objectContext.getDataset().navigate(
								{
									onComplete : dojo.hitch(this, _processNavNewDataSet),
									onError : dojo.hitch(this, _handleNavNewDataSetError),
									taskId : task.getId(),
									taskCategory : task.getCategory(),
									selection : this._buildClientNavModelSelection(objectContext, objectContext
											.getDataset().getVersion())
								});
					}
				}
			}
			}
			// update sourceToken hash with selection
			sourceToken = objectContext.getItemIds()[0];
			this._prevSourceToken = sourceToken;
			localHashObject[Constants.SOURCE_TOKEN] = sourceToken;
			retargetInfoObject[Constants.SOURCE_TOKEN] = sourceToken;
			//append all our items to the hash, and get a string back that represents it
			myHashUpdateObject = this.updateHashObjectValues(localHashObject);
			
			// check to see if treeTopperId is set in hash, if so remove
			if(myHashUpdateObject['treeTopperId']) {
				delete myHashUpdateObject.treeTopperId;
			}
			
			//actually set the hash
			this.setUrlHash(dojo.objectToQuery(myHashUpdateObject), true);

			//hackaround to handle the callbacks above that handled dataset publish to resnav
			if (!datasetPublished){
				topic.publish(Constants.TOPIC_RETARGET_INFO, retargetInfoObject);
			}
			// store the current selection off into persistence so it can be retrieved later.
			// TODO once fix in in from TWL do state stuff
			//            stateObj = this._tree._table.getState();
			//            if(stateObj) {
			//                if (this._treeSelectorLogger.isFiner()) {
			//                    this._treeSelectorLogger.finer("save off stateObj", "_treeSelectionChangedHandler");
			//                }
			//                this.setData("treeKey", stateObj, true);
                //            }
            }
		},

		_hashChanged : function (hash) {
			var _sourceToken = this.getUrlHashParm(Constants.SOURCE_TOKEN);
			// should only get in this if statement in if the hash update was external 
			// i.e bookmark, external link, manual url change...
			if (_sourceToken && (this._prevSourceToken !== _sourceToken)) {
				// clear previous selection
				this._removeSelection();
				// store the _sourceToken
				this._prevSourceToken = _sourceToken;
				// change treeSelection
				this._initialSelection(this.dataset, _sourceToken);
			}
		},
        
		//***********************************************************
		// override NavWidgetPluggableView methods
		//***********************************************************
		_createWidgetDefinition : function () {
			// summary:
			//        Called by NavWidgetPluggableView, modify behavior
			// tags:
			//        private

			//Call super class which will load widget def
			this.inherited(arguments);

			//Look at widget def for attributes we need
			if (this.widgetDefinition) {
				this.widgetDefinition.getToolbar().hidden = true;
			}
		},

		_getDefaultWidgetDefInfo : function () {
			// summary:
			//        Load the default widget definition
			// tags:
			//        private
			return this._DEFAULT_WIDGET_DEF_INFO;
		},


		_onClickHandler : function (/*ibm.tivoli.navmodel.Item*/item) {
			// summary:
			//        Called when a node is selected in the the tree selector
			// tags:
			//        callback
			this._handlingSelectionChangedCallback = true;

			if (!this.selection.isChanging()) {
				this.selection.setSelectedItem(item);
				this._treeSelectionChangedHandler(item);
			}

			this._handlingSelectionChangedCallback = false;
		},

		//NOTE: _buildClientNavModelSelection is copied from AsynchContextMenu... could be a utility
		_buildClientNavModelSelection : function (objectContext, /*Float*/version) {
			// summary:
			//        Coming soon

			var currentSelection;
			if (objectContext) {
				currentSelection = new Selection({
					version : version
				});
				currentSelection.addItems(objectContext.getItemIds());
				currentSelection.addLinks(objectContext.getLinkIds());
				currentSelection.addRelationships(objectContext.getRelationshipIds());
			}
			return currentSelection;
        },


        //NOTE: _initialSelection
        _initialSelection : function (/*ibm.tivoli.navmodel.dataset*/dataset, /*String*/srcToken) {
            // summary:
            //        Coming soon
            //dataset.getOriginItems({
        	dataset.getItems({
                onComplete : dojo.hitch(this, this.setItems, srcToken),
                onError : function (error, exception, ioArgs) {
                    alert(error);
                }
            });

        },

        setItems : function (/*String*/srcToken, /*ibm.tivoli.navmodel.Item[]*/items) {
            if (items) {
    			// Return if grid is null
    			if (!this._tree._grid) {
    				if (this._treeSelectorLogger.isWarn()) {
    					this._treeSelectorLogger.warn("Grid is null", "setItems");
    				}
    				return;
    			}

            	try {
        			var grid = this._tree._grid, mytree=this._tree;

                    if(srcToken) {
                    	// need to see if the src token is from this dataset
                    	var item = this._matchItem(items, srcToken);

                    	// when toggling between dashhbar items, the previos dashbarItems source
                    	// token is on the url, in that case lets default to the first item in tree
                    	if(item != null ) {
                    		if(item.isOrigin()) {
		            			// since is origin, grid has it; get the row to expand and select it
	                    		var rowIthink = grid.model.byId(item.getId());
	                    		if(rowIthink) {
	    							var rItem = rowIthink.item;
	    							mytree._grid.tree.expand(rItem.getId());
	    							
	    							// Tell the table to select the items
	    							mytree.selection.setSelectedObjects([rItem], null, null, [rItem.getId()]);
	                    			
	                    		}
	    					} else {
	    						// now we have a child selected, lets find out who his origin is
	    						// to do so we have to get each origin item get his children and see
	    						// if he is the parent of the source token passed in
	    						var originItem = null;
	    						for (var j=0; j < items.length; j++) {
	    							originItem = items[j];
	    							if(originItem.isOrigin()) {
		    								originItem.getRelatedItems({
			    								onComplete:  function (relatedItems, kwArgs, originItem) {	
			    									for (var i=0; i < relatedItems.length; i++) {
					    								if(relatedItems[i].getId() == srcToken) {
					    									// we found the parent, lets expand him
					    									mytree._grid.tree.expand(originItem.itemId);
					    									mytree.selection.setSelectedObjects([relatedItems[i]], null, null, [relatedItems[i].getId()]);
					    			    					break;
					    								}
					    							}
			    								}
		    								});			    							
	    							}
	    						}
	                    	}

                    	} else {
		                    // source token from url, was from previous dashbar; select / expan the first one
							var row = this._tree._grid.model.byIndex(0);
							var rItem = row.item;
							this._tree._grid.tree.expand(rItem.getId());
							
							// Tell the table to select the items
							this._tree.selection.setSelectedObjects([rItem], null, null, [rItem.getId()]);
                    	}
                    	
                    } else {
	                    // no source token, select / expand the first one
						var row = this._tree._grid.model.byIndex(0);
						var rItem = row.item;
						this._tree._grid.tree.expand(rItem.getId());
						
						// only select if treeTopperId is NOT on url.   If treeTopperId is on url
						// then initial selection will be one of the treetoppers
						// Tell the table to select the items
						if(this.getUrlHashParm('treeTopperId') == null) {
							this._tree.selection.setSelectedObjects([rItem], null, null, [rItem.getId()]);
						}
                    }

            	} catch(err) {
            		this._treeSelectorLogger.warn("***attempt to preselect tree node: " + err);
            	}
            }
        },
        
        _matchItem: function (items, srcToken) {
        	// first lets check the dataset to see if the sourcetoken matches an item id
        	for (var i=0; i < items.length; i++) {
        		if(items[i].getId() == srcToken) {
        			return items[i];       			
        		}
        	}
        	
        	// Secondary check, if the item is lazyload && is an origin node, we need to get the related items
        	// then check.. 
        	var oItem = null;
        	var foundItem = null;
        	for (var j=0; j < items.length; j++) {
        		oItem = items[j];
        		if(oItem.isOrigin() && oItem.isLazyLoad())
        			oItem.getRelatedItems({
					onComplete:  function (relatedItems, kwArgs) {	
						for (var k=0; k < relatedItems.length; k++) {
							if(relatedItems[k].getId() == srcToken) {
								// we found the node
								// todo we know the orgin so should we return it, to avoid another call?
								foundItem =  relatedItems[k];
								break;
							}
						}
					},
					sync: true
				});
        	}
        	
        	return foundItem;
        },

        _removeSelection: function () {
            if (this._tree._table) {
                this._tree._grid.select.row.clear();
            }
        }
    });
});
},
'dijit/form/DropDownButton':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/_base/lang",	// hitch
	"dojo/query", // query
	"../registry",	// registry.byNode
	"../popup",		// dijit.popup2.hide
	"./Button",
	"../_Container",
	"../_HasDropDown",
	"dojo/text!./templates/DropDownButton.html"
], function(declare, lang, query, registry, popup, Button, _Container, _HasDropDown, template){

/*=====
	Button = dijit.form.Button;
	_Container = dijit._Container;
	_HasDropDown = dijit._HasDropDown;
=====*/

// module:
//		dijit/form/DropDownButton
// summary:
//		A button with a drop down


return declare("dijit.form.DropDownButton", [Button, _Container, _HasDropDown], {
	// summary:
	//		A button with a drop down
	//
	// example:
	// |	<button data-dojo-type="dijit.form.DropDownButton">
	// |		Hello world
	// |		<div data-dojo-type="dijit.Menu">...</div>
	// |	</button>
	//
	// example:
	// |	var button1 = new dijit.form.DropDownButton({ label: "hi", dropDown: new dijit.Menu(...) });
	// |	win.body().appendChild(button1);
	//

	baseClass : "dijitDropDownButton",

	templateString: template,

	_fillContent: function(){
		// Overrides Button._fillContent().
		//
		// My inner HTML contains both the button contents and a drop down widget, like
		// <DropDownButton>  <span>push me</span>  <Menu> ... </Menu> </DropDownButton>
		// The first node is assumed to be the button content. The widget is the popup.

		if(this.srcNodeRef){ // programatically created buttons might not define srcNodeRef
			//FIXME: figure out how to filter out the widget and use all remaining nodes as button
			//	content, not just nodes[0]
			var nodes = query("*", this.srcNodeRef);
			this.inherited(arguments, [nodes[0]]);

			// save pointer to srcNode so we can grab the drop down widget after it's instantiated
			this.dropDownContainer = this.srcNodeRef;
		}
	},

	startup: function(){
		if(this._started){ return; }

		// the child widget from srcNodeRef is the dropdown widget.  Insert it in the page DOM,
		// make it invisible, and store a reference to pass to the popup code.
		if(!this.dropDown && this.dropDownContainer){
			var dropDownNode = query("[widgetId]", this.dropDownContainer)[0];
			this.dropDown = registry.byNode(dropDownNode);
			delete this.dropDownContainer;
		}
		if(this.dropDown){
			popup.hide(this.dropDown);
		}

		this.inherited(arguments);
	},

	isLoaded: function(){
		// Returns whether or not we are loaded - if our dropdown has an href,
		// then we want to check that.
		var dropDown = this.dropDown;
		return (!!dropDown && (!dropDown.href || dropDown.isLoaded));
	},

	loadDropDown: function(/*Function*/ callback){
		// Default implementation assumes that drop down already exists,
		// but hasn't loaded it's data (ex: ContentPane w/href).
		// App must override if the drop down is lazy-created.
		var dropDown = this.dropDown;
		var handler = dropDown.on("load", lang.hitch(this, function(){
			handler.remove();
			callback();
		}));
		dropDown.refresh();		// tell it to load
	},

	isFocusable: function(){
		// Overridden so that focus is handled by the _HasDropDown mixin, not by
		// the _FormWidget mixin.
		return this.inherited(arguments) && !this._mouseDown;
	}
});

});

},
'url:idx/oneui/templates/MenuHeading.html':"<tr class=\"dijitReset dijitMenuItem oneuiMenuHeading\" role=\"presentation\" tabindex=\"-1\">\r\n\t<td class=\"dijitReset dijitMenuItemLabel oneuiMenuHeadingLabel\" colspan=\"4\" data-dojo-attach-point=\"containerNode\"></td>\r\n</tr>",
'dijit/form/TimeTextBox':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/keys", // keys.DOWN_ARROW keys.ENTER keys.ESCAPE keys.TAB keys.UP_ARROW
	"dojo/_base/lang", // lang.hitch
	"../_TimePicker",
	"./_DateTimeTextBox"
], function(declare, keys, lang, _TimePicker, _DateTimeTextBox){

/*=====
	var _TimePicker = dijit._TimePicker;
	var _DateTimeTextBox = dijit.form._DateTimeTextBox;
=====*/

	// module:
	//		dijit/form/TimeTextBox
	// summary:
	//		A validating, serializable, range-bound time text box with a drop down time picker


	/*=====
	declare(
		"dijit.form.TimeTextBox.__Constraints",
		[dijit.form._DateTimeTextBox.__Constraints, dijit._TimePicker.__Constraints]
	);
	=====*/

	return declare("dijit.form.TimeTextBox", _DateTimeTextBox, {
		// summary:
		//		A validating, serializable, range-bound time text box with a drop down time picker

		baseClass: "dijitTextBox dijitComboBox dijitTimeTextBox",
		popupClass: _TimePicker,
		_selector: "time",

/*=====
		// constraints: dijit.form.TimeTextBox.__Constraints
		constraints:{},
=====*/

		// value: Date
		//		The value of this widget as a JavaScript Date object.  Note that the date portion implies time zone and daylight savings rules.
		//
		//		Example:
		// |	new dijit.form.TimeTextBox({value: stamp.fromISOString("T12:59:59", new Date())})
		//
		//		When passed to the parser in markup, must be specified according to locale-independent
		//		`stamp.fromISOString` format.
		//
		//		Example:
		// |	<input data-dojo-type='dijit.form.TimeTextBox' value='T12:34:00'>
		value: new Date(""),		// value.toString()="NaN"
		//FIXME: in markup, you have no control over daylight savings

		_onKey: function(evt){
			if(this.disabled || this.readOnly){ return; }
			this.inherited(arguments);

			// If the user has backspaced or typed some numbers, then filter the result list
			// by what they typed.  Maybe there's a better way to detect this, like _handleOnChange()?
			switch(evt.keyCode){
				case keys.ENTER:
				case keys.TAB:
				case keys.ESCAPE:
				case keys.DOWN_ARROW:
				case keys.UP_ARROW:
					// these keys have special meaning
					break;
				default:
					// setTimeout() because the keystroke hasn't yet appeared in the <input>,
					// so the get('displayedValue') call below won't give the result we want.
					setTimeout(lang.hitch(this, function(){
						// set this.filterString to the filter to apply to the drop down list;
						// it will be used in openDropDown()
						var val = this.get('displayedValue');
						this.filterString = (val && !this.parse(val, this.constraints)) ? val.toLowerCase() : "";

						// close the drop down and reopen it, in order to filter the items shown in the list
						// and also since the drop down may need to be repositioned if the number of list items has changed
						// and it's being displayed above the <input>
						if(this._opened){
							this.closeDropDown();
						}
						this.openDropDown();
					}), 0);
			}
		}
	});
});

},
'url:dijit/form/templates/CheckBox.html':"<div class=\"dijit dijitReset dijitInline\" role=\"presentation\"\n\t><input\n\t \t${!nameAttrSetting} type=\"${type}\" ${checkedAttrSetting}\n\t\tclass=\"dijitReset dijitCheckBoxInput\"\n\t\tdata-dojo-attach-point=\"focusNode\"\n\t \tdata-dojo-attach-event=\"onclick:_onClick\"\n/></div>\n",
'ibm/usmi/widgets/WorkArea':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define([
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dojo/hash",
	"dojo/topic",
	"dijit/_WidgetBase",
	"dijit/layout/ContentPane",
	"dijit/layout/StackContainer",
	// TWL imports
	"ibm/tivoli/navmodel/Dataset",
	"ibm/usmi/util/DefaultMessageHandler",
	// NextGen imports
	"ibm/usmi/util/BaseWidget",
	"ibm/usmi/widgets/util/Constants",
	"ibm/usmi/core/utils/Logger"
], function (dojo, declare, hash, topic, _WidgetBase, ContentPane, StackContainer, Dataset, DefaultMessageHandler,
		BaseWidget, Constants, Logger) {

	return declare([
		_WidgetBase,
		BaseWidget,
		StackContainer
	],
			{

				// Defaults
				DEFAULT_COLLECTION : "RESOURCE_DETAILS_COLLECTION",

				///////////////////////////////////////////////////////////////////
				// private members
				///////////////////////////////////////////////////////////////////

				// array holding constructed widgets
				_widgets : null,

				// Id of widget currently shown in work area
				_currentTaskWidgetID : "",

				// relationship string, for resource table if shown in work area
				_isRelatedResource : false,

				// the logger
				_workAreaLogger : null,

				// kind of a hack, to see if selectorHome is set, use vs default
				selectorHome : null,

				// item gotten from the dataset, another pairing to detailsSelector
				item : null,

				//save topic handle so we can clean up
				topicHandle : null,

				// dataset from url
				dataset : null,

				constructor : function (/*constructor_kwArgs*/kwArgs) {
					//enabling logger for convenience. uncomment to use
					//dojoConfig.trace["ibm.usmi.widgets"] = "finest";
					this._workAreaLogger = new Logger("ibm.usmi.widgets", "WorkArea");
					if (this._workAreaLogger.isFinest()) {
						this._workAreaLogger.entering("", "constructor");
					}

					//this._resExplorerResources = 
					//dojo.i18n.getLocalization("resexp", "resources");
					//TODO: MRI this._resExplorerResources.NAV_WIDGET_CUSTOM_VIEW1;

					this._widgets = [];

					// check to see if selectorHome is specified, if so we want to use that
					this.selectorHome = this.getInitArg(Constants.SELECTOR_HOME);

					if (this._workAreaLogger.isFinest()) {
						this._workAreaLogger.exiting("", "constructor");
					}
				},

				startup : function () {
					//Call superclass
					this.inherited(arguments);
					//call hashchanged to trigger initial load
					//get correct parms and do initial load
					//enf TODO
					//	this._hashChanged();
					//what do you really want to happen initially?

					//retarget info events
					this.topicHandle = topic
							.subscribe(Constants.TOPIC_RETARGET_INFO, dojo.hitch(this, this.retargeted));

					// let everyone we are open for business
					topic.publish(Constants.TOPIC_READY_WORKAREA, "");

				},

				postCreate : function () {
					//Call superclass
					this.inherited(arguments);

				},

				destroy : function () {
					// summary:
					//        Clean up any objects the workarea is hanging onto
					if (this._workAreaLogger.isFinest()) {
						this._workAreaLogger.entering("", "destroy");
					}
					if (this.topicHandle) {
						this.topicHandle.remove();
						this.topicHandle = null;
					}
					if (this._widgets) {
						var i = 0;
						for ( var key in this._widgets) {
							this._widgets[key].destroyRecursive;
							if (dijit.byId(key)) {
								dijit.byId(key).destroyRecursive();
							}

						}
						this._widgets.length = 0;
					}
					this.inherited(arguments);

					if (this._workAreaLogger.isFinest()) {
						this._workAreaLogger.entering("", "destroy");
					}
				},

				_makeWidget : function (/*String*/widgetID, /*object*/infoObj) {
					// summary:
					//Called in response to a hash update.   
					//Checks to see if widget already has been created
					//and exists in _widgets array, if not creates, adds then displays it.
					var loadDone, widgetKey, currentWidget, pathKey, pathMod;
					if (this._workAreaLogger.isFinest()) {
						this._workAreaLogger.entering("", "_makeWidget");
					}

					widgetKey = this._getWidgetKey(widgetID, infoObj);
					currentWidget = this._widgets[widgetKey];
					//Do we have the widget already?
					if (!currentWidget) {
						if (this._workAreaLogger.isFiner()) {
							this._workAreaLogger.finer("Not the current widget, need to construct: " + widgetID,
									"_makeWidget");
						}

						//create widget function after requires are ready
						loadDone = function (currentWidget) {
							currentWidget = new currentWidget({
								id : widgetKey,
								style : Constants.CSS_MAX_SIZE,
								targetInfo : infoObj
							});
							if (!currentWidget) {
								currentWidget = new ContentPane({
									title : widgetID,
									content : "### PAGE CONTENT ###  " + widgetID
								});
							}

							// add widget to stack container and_widgets array
							this.addChild(currentWidget);
							this._widgets[widgetKey] = currentWidget;

							// set selected to current shown
							this.selectChild(currentWidget, true);
							//currentWidget.startup();
							//TODO: This startup is being really funky
							//on one box, this workarea isn't started yet, so need to call startup
							//on most its not needed.
							//Do we have a big timing issue, or did something change from the old box?
							//looks like _started didnt exist on one of parents back then
						};

						try {
							//check to see if taskPath was specified, 
							//if so need to add it to require statement
							// so can load module path
							if (infoObj[Constants.HASH_TASK_PATH] && infoObj[Constants.HASH_TASK_PATH_KEY]) {
								pathKey = infoObj[Constants.HASH_TASK_PATH_KEY];
								pathMod = infoObj[Constants.HASH_TASK_PATH];

								// add path to require
								dojo.registerModulePath(pathKey, pathMod);
								require([ widgetID
								], dojo.hitch(this, loadDone));
								//dojo isnt resolving the variable, its just passing it in
								//require({paths:{pathKey:
								//"/com.ibm.usmi.console.samples/js/usmi-console-samples"}}
								//,[widgetID], dojo.hitch(this, loadDone));
							} else {
								//require classes as needed on URL and create dijits
								require([ widgetID
								], dojo.hitch(this, loadDone));
							}
						} catch (err) {
							// for development, should not happen in production
							console.error(err);
							currentWidget = new ContentPane({
								title : widgetID,
								content : "### PAGE CONTENT ###  " + widgetID
							});

							// add widget to stack container and_widgets array
							this.addChild(currentWidget);
							this._widgets[widgetKey] = currentWidget;

							// set selected to current shown
							this.selectChild(currentWidget, true);
						} //end catch
					} else {
						//We already had the widget.
						//set selected to current shown
						this.selectChild(currentWidget, true);
						//just show him. He'll figure the rest out based on hash, or the retarget coming elsewhere
						this._workAreaLogger.fine("already had the widget! showing him now: " + widgetID);

					}

					if (this._workAreaLogger.isFinest()) {
						this._workAreaLogger.exiting("", "_makeWidget");
					}
				},

				_getWidgetKey : function (/*String*/widgetID, /*dojo.hash*/infoObj) {
					// summary:
					//        Create unique key based off widget ID, and hash parms

					var key = widgetID;
					// check to see if relationship was specified on the hash, if so use it to
					// construct unique id.   If it was not specified, just return the widget id as key
					if (infoObj[Constants.HASH_TASK_RELATIONSHIP]) {
						this._isRelatedResource = true;
						//	key = key + infoObj[Constants.HASH_TASK_RELATIONSHIP] + infoObj[Constants.SOURCE_TOKEN];
					}

					// set showing releated resource to false
					else {
						this._isRelatedResource = false;
						//key = key + infoObj["collectionID"] + infoObj[Constants.SOURCE_TOKEN];
					}

					//making the key just the widget ID. This determines if you're re-creating it, 
					///or just caching it in the stack. For things like related resourcs, 
					//just cache one copy and change the dataset
					return key;
				},

				//event contains info on what to create as child
				retargeted : function (retargetInfo) {
					this._currentTaskWidgetID = retargetInfo[Constants.HASH_TASK_SELECTION];
					this._makeWidget(retargetInfo[Constants.HASH_TASK_SELECTION], retargetInfo);
				}

			});
});

},
'dojo/dnd/autoscroll':function(){
define(["../main", "../window"], function(dojo) {
	// module:
	//		dojo/dnd/autoscroll
	// summary:
	//		TODOC

dojo.getObject("dnd", true, dojo);

dojo.dnd.getViewport = dojo.window.getBox;

dojo.dnd.V_TRIGGER_AUTOSCROLL = 32;
dojo.dnd.H_TRIGGER_AUTOSCROLL = 32;

dojo.dnd.V_AUTOSCROLL_VALUE = 16;
dojo.dnd.H_AUTOSCROLL_VALUE = 16;

dojo.dnd.autoScroll = function(e){
	// summary:
	//		a handler for onmousemove event, which scrolls the window, if
	//		necesary
	// e: Event
	//		onmousemove event

	// FIXME: needs more docs!
	var v = dojo.window.getBox(), dx = 0, dy = 0;
	if(e.clientX < dojo.dnd.H_TRIGGER_AUTOSCROLL){
		dx = -dojo.dnd.H_AUTOSCROLL_VALUE;
	}else if(e.clientX > v.w - dojo.dnd.H_TRIGGER_AUTOSCROLL){
		dx = dojo.dnd.H_AUTOSCROLL_VALUE;
	}
	if(e.clientY < dojo.dnd.V_TRIGGER_AUTOSCROLL){
		dy = -dojo.dnd.V_AUTOSCROLL_VALUE;
	}else if(e.clientY > v.h - dojo.dnd.V_TRIGGER_AUTOSCROLL){
		dy = dojo.dnd.V_AUTOSCROLL_VALUE;
	}
	window.scrollBy(dx, dy);
};

dojo.dnd._validNodes = {"div": 1, "p": 1, "td": 1};
dojo.dnd._validOverflow = {"auto": 1, "scroll": 1};

dojo.dnd.autoScrollNodes = function(e){
	// summary:
	//		a handler for onmousemove event, which scrolls the first avaialble
	//		Dom element, it falls back to dojo.dnd.autoScroll()
	// e: Event
	//		onmousemove event

	// FIXME: needs more docs!

	var b, t, w, h, rx, ry, dx = 0, dy = 0, oldLeft, oldTop;

	for(var n = e.target; n;){
		if(n.nodeType == 1 && (n.tagName.toLowerCase() in dojo.dnd._validNodes)){
			var s = dojo.getComputedStyle(n),
				overflow = (s.overflow.toLowerCase() in dojo.dnd._validOverflow),
				overflowX = (s.overflowX.toLowerCase() in dojo.dnd._validOverflow),
				overflowY = (s.overflowY.toLowerCase() in dojo.dnd._validOverflow);
			if(overflow || overflowX || overflowY){
				b = dojo._getContentBox(n, s);
				t = dojo.position(n, true);
			}
			// overflow-x
			if(overflow || overflowX){
				w = Math.min(dojo.dnd.H_TRIGGER_AUTOSCROLL, b.w / 2);
				rx = e.pageX - t.x;
				if(dojo.isWebKit || dojo.isOpera){
					// FIXME: this code should not be here, it should be taken into account
					// either by the event fixing code, or the dojo.position()
					// FIXME: this code doesn't work on Opera 9.5 Beta
					rx += dojo.body().scrollLeft;
				}
				dx = 0;
				if(rx > 0 && rx < b.w){
					if(rx < w){
						dx = -w;
					}else if(rx > b.w - w){
						dx = w;
					}
					oldLeft = n.scrollLeft;
					n.scrollLeft = n.scrollLeft + dx;
				}
			}
			// overflow-y
			if(overflow || overflowY){
				//console.log(b.l, b.t, t.x, t.y, n.scrollLeft, n.scrollTop);
				h = Math.min(dojo.dnd.V_TRIGGER_AUTOSCROLL, b.h / 2);
				ry = e.pageY - t.y;
				if(dojo.isWebKit || dojo.isOpera){
					// FIXME: this code should not be here, it should be taken into account
					// either by the event fixing code, or the dojo.position()
					// FIXME: this code doesn't work on Opera 9.5 Beta
					ry += dojo.body().scrollTop;
				}
				dy = 0;
				if(ry > 0 && ry < b.h){
					if(ry < h){
						dy = -h;
					}else if(ry > b.h - h){
						dy = h;
					}
					oldTop = n.scrollTop;
					n.scrollTop  = n.scrollTop  + dy;
				}
			}
			if(dx || dy){ return; }
		}
		try{
			n = n.parentNode;
		}catch(x){
			n = null;
		}
	}
	dojo.dnd.autoScroll(e);
};

	return dojo.dnd;
});

},
'gridx/modules/pagination/GotoPagePane':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/event",
	"dojo/keys",
	"dijit/_WidgetBase",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	"dojo/text!../../templates/GotoPagePane.html"
], function(declare, lang, event, keys, _WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin, goToTemplate){

	return declare([_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin], {
		templateString: goToTemplate,
	
		pager: null,

		postMixInProperties: function(){
			var t = this,
				p = t.pager,
				mod = p.module;
			lang.mixin(t, p._nls);
			t.numberTextBoxClass = mod.arg('numberTextBoxClass').prototype.declaredClass;
			t.buttonClass = mod.arg('buttonClass').prototype.declaredClass;
		},
	
		postCreate: function(){
			this._updateStatus();
		},
	
		_updateStatus: function(){
			var b = this.pageInputBox;
			this.okBtn.set('disabled', !b.isValid() || b.get('displayedValue') === "");
		},
	
		_onOK: function(){
			var p = this.pager;
			p.pagination.gotoPage(this.pageInputBox.get('value') - 1);
			p._gotoDialog.hide();
		},
	
		_onCancel: function(){
			this.pager._gotoDialog.hide();
		},
		
		_onKeyDown: function(evt){
			if(!this.okBtn.get('disabled') && keys.ENTER == evt.keyCode){
				this._onOK();
				event.stop(evt);
			}
		}
	});
});

},
'ibm/tivoli/navmodel/DatasetLinksStore':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/DatasetStore,ibm/tivoli/navmodel/Dataset"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */
 
dojo.provide("ibm.tivoli.navmodel.DatasetLinksStore");

dojo.require("ibm.tivoli.navmodel.DatasetStore");
dojo.require("ibm.tivoli.navmodel.Dataset");
	
ibm.tivoli.navmodel.DatasetLinksStore._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.DatasetLinksStore");
	
dojo.declare("ibm.tivoli.navmodel.DatasetLinksStore", ibm.tivoli.navmodel.DatasetStore, {
	// private members
	_clearCache: false,
	_trace: ibm.tivoli.navmodel.DatasetLinksStore._trace, 
	
	/*=====
	constructor_kwArgs: function(dataset, defaultProperties, returnItem) {
		// summary:
		//		An anonymous object used to configure the datastore
		// dataset: Object
		//		A ibm.tivoli.navmodel.Dataset instance
		// defaultProperties: String[]
		//		An array of property id values that are pre-fetched with the items
		// returnItem: Boolean
		//		A boolean indicating if the getValue() method should just return the actual
		//		item so that it can be used for custom formatting of the value
	},
	=====*/
	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructor for dojo datastore;
		//		that receives link data from an ibm.tivoli.navmodel.Dataset
		//		Base class will mixin kwArgs      
	},


	//////////////////////////////////////////////////////////////
	// dojo.data.api.Identity
	//////////////////////////////////////////////////////////////
	
	getIdentity: function(/*Object*/ item){
		// summary:
    	//		Returns a unique identifier for an item. The return value will be
    	//		either a string or something that has a toString() method.
    	// item:
    	//		The item from the store from which to obtain its identifier.
		// exceptions:
    	//		Conforming implementations may throw an exception or return null if
    	//		item is not an item.
		if (!this.isItem(item)){
  			throw new Error("Specified item argument is not a valid item.");
  		}
  		
  		var identity = item.getId();
		
		return identity;		
    },
    
	/*=====
	_onItem: function(item) {},
	=====*/
	
	/*=====
	_onError: function(error) {},
	=====*/
	
	/*=====
	fetchItemByIdentity_kwArgs: function(identity, onItem, onError, scope) {
		// summary:
		//		An anonymous object that defines the item to locate and callbacks to invoke when the
		//		item has been located and load has completed.
		// identity: String|Object
		//		The identity parameter is the identity of the item you wish to locate and load
		//		This attribute is required. It should be a string or an object that toString()
		//		can be called on.
		// onItem: _onItem
		//		The onItem parameter is the callback to invoke when the item has been loaded.
		//		It takes only one parameter, the item located, or null if none found.
		// onError: _onError
		//		The onError parameter is the callback to invoke when the item load encountered an error.
		//		It takes only one parameter, the error object
		// scope: Object
		//		If a scope object is provided, all of the callback functions (onItem, onError, etc)
		//		will be invoked in the context of the scope object.
	},
	=====*/

 	fetchItemByIdentity: function(/*fetchItemByIdentity_kwArgs*/ kwArgs) {
		// summary:
		//		Given the identity of an item, this method returns the item that has
		//		that identity through the onItem callback. Conforming implementations
		//		should return null if there is no item with the given identity.
		//		Implementations of fetchItemByIdentity() may sometimes return an item
		//		from a local cache and may sometimes fetch an item from a remote server.
		var args = {
 			identity: null,
 			onItem: function(item) {},
 			onError: function(error) {},
 			scope: null
 		};
 		dojo.mixin(args, kwArgs);
 		
 		var item = null;
 		var error = null;
 		if (args.identity) {
 			try {
 				var linkId  = args.identity;
 				if (this.dataset) {
 					item = this.dataset.getLink({linkId: linkId, properties: this.defaultProperties});
 				}
  			}
  			catch(err) {
  				error = err;  				
  			}  			
  		}  		
  		
 		if (item) {
 			args.onItem.call(args.scope, item);
 		}
 		else {
 			args.onError.call(args.scope, error ? error : "Could not fetch item for identity=" + args.identity);
 		}		
	},
	
	
	//////////////////////////////////////////////////////////////
	// dojo.data.api.Read
	//////////////////////////////////////////////////////////////
	
	/*=====
	fetch_onBegin: function(size, request){},
	=====*/
	
	/*=====
	fetch_onItem: function(item, request){},
	=====*/
	
	/*=====
	fetch_onComplete: function(items, request){},
	=====*/
	
	/*=====
	fetch_onError: function(errorData, request){},
	=====*/
	
	/*=====
	fetch_kwArgs: function(query, queryOptions, onBegin, onItem, onComplete, onError, scope,
							start, count, sort) {
		// summary:
		//		The kwArgs parameter may either be an instance of conforming to 
		//		dojo.data.api.Request or may be a simple anonymous object
		//		containing any of the above parameters.
		// query: String|Object
		//		The query parameter is used to specify search and filter condition information for the fetch.
		//		Only those items that match the search and/or filter will be returned.
		//		|	{	ids: {
		//		|			propId: boolean		// indicates propIds involved in the expression
		//		|		},
		//		|		expression: string,		// expression involiving propId's and values  (and/or together)
		//		|								// example:  ((Name contains 'abc' || Type contains 'abc)) and (Size > 100)
		//		|
		//		|		// the following is for compatability with legacy datastore
		//		| 		navModel: {
		//		|			search: {
		//		|				term: string,		the search text
		//		|				attributes: array	["property id1", "property id2", ...]
		//		|			}
		//		|		}
		//		|	}
		// queryOptions: Object?
		//		The queryOptions parameter is an optional parameter that can be used to modify the query.  The
		//		ignoreCase case can be specified to globally make all of the filter conditions case sensitive/insensitive.
		//		|	{ ignoreCase: boolean
		//		|	}
		// onBegin: fetch_onBegin
		//		If an onBegin callback function is provided, the callback function will be called just once, 
		//		before the first onItem callback is called. The onBegin callback function will be passed two arguments, 
		//		the the total number of items identified and the Request object.  If the total number is
		//		unknown, then size will be -1.  Note that size is not necessarily the size of the collection of items 
		//		returned from the query, as the request may have specified to return only a subset of the total set 
		//		of items through the use of the start and count parameters.
		// onItem: fetch_onItem
		//		If an onItem callback function is provided, the callback function will be called as each item 
		//		in the result is received. The callback function will be passed two arguments: the item itself, 
		//		and the Request object.
		// onComplete: fetch_onComplete
		//		If an onComplete callback function is provided, the callback function will be called just once,
		//		after the last onItem callback is called. Note that if the onItem callback is not present,
		//		then onComplete will be passed an array containing all items which matched the query and the 
		//		request object. If the onItem callback is present, then onComplete is called as: onComplete(null, request).
		// onError: fetch_onError
		//		If an onError callback function is provided, the callback function will be called if there is any 
		//		sort of error while attempting to execute the query. The onError callback function will be passed two arguments:
		//		an Error object and the Request object.
		// scope: Object
		//		If a scope object is provided, all of the callback functions (onItem, onComplete, onError, etc) 
		//		will be invoked in the context of the scope object.
		// start: int
		//		If a start parameter is specified, this is a indication to the datastore to
		//		only start returning items once the start number of items have been located and
		//		skipped.  When this parameter is paired with 'count', the store should be able
		//		to page across queries with millions of hits by only returning subsets of the
		//		hits for each query
		// count: int
		//		 If a count parameter is specified, this is a indication to the datastore to
		//		 only return up to that many items.  This allows a fetch call that may have
		//		millions of item matches to be paired down to something reasonable.
		// sort: Object[]
		//		If a sort parameter is specified, this is a indication to the datastore to
		//		sort the items in some manner before returning the items.  The array is an array of
		//		javascript objects that must conform to the following format to be applied to the
		//		fetching of items:
		//		|	{
		//		|		attribute: attribute || attribute-name-string,
		//		|		descending: true|false;   // Optional.  Default is false.
		//		|	}
	},
	=====*/
	
	fetch: function(/*fetch_kwArgs*/ kwArgs) {
		// summary:
		//		Given a query and set of defined options, such as a start and count of items to return,
		//		this method executes the query and makes the results available as data items.
		//		The format and expectations of stores is that they operate in a generally asynchronous
		//		manner, therefore callbacks are always used to return items located by the fetch parameters.
		// description:
		//		A Request object will always be returned and is returned immediately.
		//		The basic request is nothing more than the keyword args passed to fetch and
		//		an additional function attached, abort().  The returned request object may then be used
		//		to cancel a fetch.  All data items returns are passed through the callbacks defined in the
		//		fetch parameters and are not present on the 'request' object.
		// returns:
		//		The fetch() method will return a javascript object conforming to the API
		//		defined in dojo.data.api.Request.  In general, it will be the keywordArgs
		//		object returned with the required functions in Request.js attached.
		//		Its general purpose is to provide a convenient way for a caller to abort an
		//		ongoing fetch.
		// exceptions:
		//		Throws an exception if the query is not valid, or if the query is required but was not supplied.
		var args = {
			query: null,
			queryOptions: null,
			onBegin: function(size, request) {},
			onItem: function(item, request) {},
			onComplete: function(items, request){},
			onError: function(errorData, request){},
			scope: null,
			start: 0,
			count: null, //default should be to get all
			sort: [],
			abort: function() {}
		};
		dojo.mixin(args, kwArgs);
		
		if (this._trace.isFinest()) {
			this._trace.finest("dataset=" + this.dataset.getId() + " start=" + args.start + " count=" + args.count, "fetch");
		}
		
		// make sure event handling is set up
		this._validateEventHandler();
		
		// TED refresh is occurring
		this._clearCache = false;
		if (this.isRefresh()) {
			this._clearCache = this.dataset.isClearCacheOnRefresh();						
		}
		
		// translate information from the args
		var sort = this._getSort(args.sort);
		var condition = this._getCondition(args.query);		
					
		// if links are being viewed without context that includes items;
		// then we will have to fetch all the items to avoid fetching them one at
		// a time when the item1 and item2 are evaluated
		if (!this.dataset.hasCachedItems()) {
			this.dataset.getItems({sync: true});
		}
			
		var linkArgs = {
			onComplete: dojo.hitch(this, function(links, linkArgs) {
				args.onBegin.call(args.scope, linkArgs.filteredRows, args);
				
				if (links) {
					var linksLength = links.length;
					for (var linkIndex = 0;  linkIndex < linksLength;  ++linkIndex) {
						args.onItem.call(args.scope, links[linkIndex], args);                	
					}
				}
				
				args.onComplete.call(args.scope, links, args);				
			}),
			
			onError: dojo.hitch(this, function(error, exception, ioArgs) {
				args.onError.call(args.scope, error, args);
			}),
			
			clearCache: this._clearCache,
			start: args.start,
			count: args.count,	
			properties: this.defaultProperties,
			sort: sort,
			condition: condition
		}; 		
		this.dataset.getLinks(linkArgs);	
		       
        return args;
    },
    
	getAttributes: function(/*Object*/ item) {
		// summary:
		//		Returns an array with all the attributes that this item has.  This
		//		method will always return an array; if the item has no attributes
		//		at all, getAttributes() will return an empty array: [].
		//		Throws an exception if *item* is not an item
		// item:
		//		The item to access attributes on. Type: ibm.tivoli.navmodel.Item    
		var attributes = this.inherited(arguments);
		attributes.push("label");
		attributes.push("item1");
		attributes.push("item2");
		attributes.push("dataset1");
		attributes.push("dataset2");
		attributes.push("identity1");
		attributes.push("identity2");
		
		return attributes;		
    },
    
    getValue: function(/*Object*/ item, /*String*/ attribute, /*Object?*/ defaultValue) {
		// summary:
		//		Returns a single attribute value. Returns defaultValue if and only if item does not 
		//		have a value for attribute. Returns null if and only if null was explicitly set as the 
		//		attribute value. Returns undefined if and only if the item does not have a value for the 
		//		given attribute (which is the same as saying the item does not have the attribute).
		// item:
		//		The item to access attributes on.
		//		ibm.tivoli.navmodel.Item object
		// attribute:
		//		The attribute to access represented as a string.
		// defaultValue:
		//		Optional. A default value to use for the getValue return if the attribute does not exist 
		//		or has no value.
		// exceptions:
		//		Throws an exception if *item* is not an item, or *attribute* is not a string 
		if (!this.isItem(item)) {
			throw new Error("Specified item argument is not a valid item.");
        }
        
 		if (!dojo.isString(attribute)) {
			throw new Error("Specified attribute argument is not a string.");
		}  
		
		if (this.returnItem) {
			return item;
		}
		
		var value = defaultValue;    
        switch(attribute) {
        	case "label":
        		value = item.getLabel();
        		break;
        		
        	case "item1":
        		var item1 = item.getItem1();
				value = (item1 ? item1.getLabel() : "");
				break;
        		
        	case "item2":
        		var item2 = item.getItem2();
				value = (item2 ? item2.getLabel() : "");
				break;
        		
        	case "dataset1":
        		var d1 = item.getDataset1();
    			value = (d1 ? d1.getLabel() : "");
        		break;
        		
        	case "dataset2":
        		var d2 = item.getDataset2();
    			value = (d2 ? d2.getLabel() : "");
    			break;
    			
    		case "identity1":
    			value = item.getItem1Id();
        		break;
        		
        	case "identity2":
        		value = item.getItem2Id();
        		break;
        		
    		default:
    			var prop = item.getProperty(attribute);
        		if (prop) {
        			value = prop.getDisplayValue();
        		}
        }
        
        return value;        
    },
    
	/*=====
	loadItem_kwArgs: function(item, onItem, onError, scope) {
		// summary:
		//		An anonymous object that defines the item to load and callbacks to invoke when the
		//		load has completed.
		// item: Object
		//		The item parameter is an object that represents the item in question that should be
		//		contained by the store.  This attribute is required.
    	//		ibm.tivoli.navmodel.Item object
		// onItem: _onItem
		//		The onItem parameter is the callback to invoke when the item has been loaded.  
		//		It takes only one parameter, the error object
		// onError: _onError
		//		The onError parameter is the callback to invoke when the item load encountered an error.
		//		It takes only one parameter, the error object.
		// scope: Object
		//		If a scope object is provided, all of the callback functions (onItem, onError, etc)
		//		will be invoked in the context of the scope object.
	},
	=====*/
	
 	loadItem: function(/*loadItem_kwArgs*/ kwArgs) {
		// summary:
		//		Given an item, this method loads the item so that a subsequent call to store.isItemLoaded(item) 
		//		will return true.  If a call to isItemLoaded() returns true before loadItem() is even called,
		//		then loadItem() need not do any work at all and will not even invoke the callback handlers.  
		//		So, before invoking this method, check that the item has not already been loaded.
		var args = {
			item : null,
			onItem: function(item) {},
			onError: function(error){},
			scope: null
		};		
		dojo.advancedMixin(args, kwArgs);
	
		if (!this.isItem(args.item)){
  			throw new Error("Specified item argument is not a valid item.");
  		}
  		
		if (this.isItemLoaded(args.item)){
			return args.item;
		}
		
		try {
			var dataset = args.item.getDataset();
			var item = dataset.getLink({linkId: args.item.getId(), properties: this.defaultProperties});
			args.onItem.call(args.scope, item);
		}
		catch(err) {
			args.onError.call(args.scope, err);
		}
	},
	
	
	//////////////////////////////////////////////////////////////
	// private methods
	///////////////////////////////////////////////////////////////
	
    _getObjectTypes: function() {
		// summary:
		//		Returns String[] of object types that can in the datastore
    	return {"link" : true};
    },
    
    _handleLinkPropertiesChangedEvent: function(/*Object*/ event) {
		var links = event.getLinks();    	
    	var propsArray = event.getProperties();			
		if (links && propsArray) {
			var refresh = false;
    		var length = links.length;			
			for (var index = 0;  index < length;  ++index) {
				var properties = propsArray[index];				
				if (properties) {
					var propsLength = properties.length;
					for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
						var link = links[index];
						var property = properties[propIndex];
						var propId = property.getId();
						if (this._trace.isFinest()) {
							this._trace.finest("datasetId=" + event.getDataset().getId() + " linkId=" + link.getId() + " prop=" + propId + " value=" + property.getDisplayValue(), "_handleLinkPropertiesChangedEvent");
						}
						
						this.onSet(link, propId, null, property.getDisplayValue());
						
						if (!refresh && (this._affectsSort(propId) || this._affectsCondition(propId))) {
							refresh = true;							
						}
					}										
				}				
			}
			
			if (refresh) {
				this.onRefresh();
			}	
		}
	},
	
	_handleLinksAddedEvent:function(/*Object*/ event) {
		var links = event.getLinks();
		if (links) {
			var length = links.length;
			for (var index = 0;  index < length;  ++index) {
				var link = links[index];
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + event.getDataset().getId() + " linkId=" + link.getId(), "_handleLinksAddedEvent");
				}
				this.onNew(link);
			}
			
			if (this._hasSort() || this._hasCondition()) {
				this.onRefresh();
			}
		}
	},
	
	_handleLinksRemovedEvent: function(/*Object*/ event) {
		var links = event.getLinks();
		if (links) {
			var length = links.length;
			for (var index = 0;  index < length;  ++index) {
				var link = links[index];
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + event.getDataset().getId() + " linkId=" + link.getId(), "_handleLinksRemovedEvent");
				}
				this.onDelete(link);
			}			
		}
	},
	
	_handleItemPropertiesChangedEvent: function(/*Object*/ event) {
		var items = event.getItems();
		var propsArray = event.getProperties();			
		if (items && propsArray) {
			var itemIds = [];
			var itemValues = [];
			
			// see if any "itemName" properties changed;  we use these to 
			// display the "item1" and "item2" values
    		var length = items.length;			
			for (var index = 0;  index < length;  ++index) {
				var properties = propsArray[index];				
				if (properties) {
					var propsLength = properties.length;
					for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
						if (properties[propIndex].getValueType() === "itemName") {						
							itemIds.push(items[index].getId());
							itemValues.push(properties[propIndex].getDisplayValue());
							if (this._trace.isFinest()) {
								this._trace.finest("datasetId=" + event.getDataset().getId() + " itemName property changed for itemId=" + items[index].getId(), "_handleItemPropertiesChangedEvent");
							}
							break;
						}
					}										
				}				
			}	
			
			// go check links that might be affected
			if (itemIds.length > 0) {
				var fetchArgs = {
					onComplete: dojo.hitch(this, function(links, args) {
						if (links) {
							var linksLength = links.length;
							for (var linkIndex = 0;  linkIndex < linksLength;  ++linkIndex) {
								var item1Id = links[linkIndex].getItem1Id();
								var item2Id = links[linkIndex].getItem2Id();
								var idLength = itemIds.length;
								
								for (var idIndex = 0;  idIndex < idLength;  ++idIndex) {
									if (item1Id && item1Id === itemIds[idIndex]) {
										this.onSet(links[linkIndex], "item1", null, itemValues[idIndex]);
									}
									else if (item2Id && item2Id === itemIds[idIndex]) {
										this.onSet(links[linkIndex], "item2", null, itemValues[idIndex]);
									}
								}																
							}
						}
					}),
					start: 0,
					count: null
				};
				this.fetch(fetchArgs);
			}			
		}
	}
});
});

},
'url:dijit/form/templates/DropDownBox.html':"<div class=\"dijit dijitReset dijitInline dijitLeft\"\n\tid=\"widget_${id}\"\n\trole=\"combobox\"\n\t><div class='dijitReset dijitRight dijitButtonNode dijitArrowButton dijitDownArrowButton dijitArrowButtonContainer'\n\t\tdata-dojo-attach-point=\"_buttonNode, _popupStateNode\" role=\"presentation\"\n\t\t><input class=\"dijitReset dijitInputField dijitArrowButtonInner\" value=\"&#9660; \" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"presentation\"\n\t\t\t${_buttonInputDisabled}\n\t/></div\n\t><div class='dijitReset dijitValidationContainer'\n\t\t><input class=\"dijitReset dijitInputField dijitValidationIcon dijitValidationInner\" value=\"&#935; \" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"presentation\"\n\t/></div\n\t><div class=\"dijitReset dijitInputField dijitInputContainer\"\n\t\t><input class='dijitReset dijitInputInner' ${!nameAttrSetting} type=\"text\" autocomplete=\"off\"\n\t\t\tdata-dojo-attach-point=\"textbox,focusNode\" role=\"textbox\" aria-haspopup=\"true\"\n\t/></div\n></div>\n",
'ibm/usmi/statuspod/AlertStatusPodDialog':function(){
require({cache:{
'url:ibm/usmi/statuspod/templates/alertStatuspodDialog.html':"<div class=\"statuspod\">\r\n<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n<span id=\"alertStatusPodHeaderText\" class=\"alertStatusPodHeaderText\"></span>\r\n\t<div class=\"statuspodTabContainer\" id=\"statusPodTabContainer\" data-dojo-attach-point=\"statusPodTabContainer\" data-dojo-type=\"dijit.layout.TabContainer\" style=\"width: 100%; height: 100%;\" data-dojo-props='doLayout:false,controllerWidget:\"dijit.layout.TabController\"'>\r\n\t\t<div id=\"problems_pane\" data-dojo-attach-point=\"problems_pane\" data-dojo-type=\"dijit.layout.ContentPane\" title=\"${templateMRI.StatusProblemsPlaceholder}\" selected=\"true\">\r\n\t\t\t<div data-dojo-type=\"idx.oneui.Menu\" class=\"menu-problems\" data-dojo-attach-point=\"problemsMenu\">\r\n\t\t\t</div>\r\n\t\t\t\r\n\t\t\t<div class=\"statusPodPaneSpace\" data-dojo-attach-point=\"problemAlertsSpace\">\r\n\t\t\t</div>\r\n\t\t\t\r\n\t\t\t<div  class=\"footerHeader\">\r\n\t\t\t\t<span class=\"statusPodSummary\" data-dojo-attach-point=\"problemsCatSummary\">${templateMRI.StatusShowingPartialPlaceholder}</span>\r\n\t\t\t</div>\r\n\t\t\t<div  class=\"footerBottom\">\r\n\t\t\t\t<span class=\"firstLink\">\r\n\t\t\t\t\t<a href=\"javascript:menuClicked('workAreaDijit=ibm/usmi/activealerts/ActiveAlerts&alertFilter=problems');\">${templateMRI.StatusViewAllStatus}</a>\r\n\t\t\t\t</span>\r\n\t\t\t\t<span class=\"lastLink\">\r\n\t\t\t\t\t<a href=\"javascript:menuClicked('workAreaDijit=ibm/usmi/widgets/ResourceNavigator&dataset=GROUP:STATUS_PROBLEMS&forceRefresh=true');\">${templateMRI.StatusViewAll}</a>\r\n\t\t\t\t</span>\r\n\t\t\t</div>\r\n\t\t</div>\r\n\t\t\r\n\t\t<div id=\"compliances_pane\" data-dojo-attach-point=\"compliances_pane\" data-dojo-type=\"dijit.layout.ContentPane\" title=\"${templateMRI.StatusCompliancePlaceholder}\" >\r\n\t\t   <div data-dojo-type=\"idx.oneui.Menu\" class=\"menu-compliances\" data-dojo-attach-point=\"compliancesMenu\">\r\n\t\t\t</div>\r\n\t\t\t\r\n\t\t\t<div class=\"statusPodPaneSpace\" data-dojo-attach-point=\"complianceAlertsSpace\">\r\n\t\t\t</div>\r\n\r\n\t\t\t<div  class=\"footerHeader\">\r\n\t\t\t\t<span class=\"statusPodSummary\" data-dojo-attach-point=\"compliancesCatSummary\">${templateMRI.StatusShowingPartialPlaceholder}</span>\r\n\t\t\t\t\r\n\t\t\t</div>\r\n\t\t\t<div  class=\"footerBottom\">\r\n\t\t\t\t<span class=\"firstLink\">\r\n\t\t\t\t\t<a href=\"javascript:menuClicked('workAreaDijit=ibm/usmi/activealerts/ActiveAlerts&alertFilter=nonCompliances');\">${templateMRI.StatusViewAllCompliance}</a>\r\n\t\t\t\t</span>\r\n\t\t\t\t<span class=\"lastLink\">\r\n\t\t\t\t\t<a href=\"javascript:menuClicked('workAreaDijit=ibm/usmi/widgets/ResourceNavigator&dataset=GROUP:STATUS_COMPLIANCE&forceRefresh=true');\">${templateMRI.StatusViewAllSystemWithCompliance}</a>\r\n\t\t\t\t</span>\r\n\t\t\t</div>\r\n        </div>\r\n\t</div>\r\n\t\t\r\n</div>",
'url:ibm/usmi/statuspod/templates/alertMenuItemLabel.html':"<table>\r\n<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n\t<tbody>\r\n\t\t<tr>\r\n\t\t\t<td class=\"statusPod-sourceName\"><div style=\"width:330px;text-overflow:ellipsis;overflow: hidden;white-space:nowrap;\">${sourceName} - ${objectType}</div></td>\r\n\t\t\t<td class=\"statusPod-shortTime\">${shortTime}</td>\r\n\t\t</tr>\r\n\t<tbody>\r\n</table>"}});
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/statuspod/AlertStatusPodDialog", [
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/array",
	"dijit/_WidgetBase",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	"dojo/text!./templates/alertStatuspodDialog.html",
	"dojo/text!./templates/alertMenuItemLabel.html",
	"dojo/string",
	"./AlertStatusPodMenuItem",
	"./AlertStatusTip",
	"./_StatusPodPopupMixin",
	"dojo/i18n!./nls/statuspodResources",
	"dojo/dom-geometry",
	"dijit/MenuSeparator",
	"idx/oneui/Menu",
	"dijit/layout/TabContainer",
	"dijit/layout/ContentPane" ], function (dojo, declare, lang, array, WidgetBase, TemplatedMixin,
	WidgetsInTemplateMixin, template, menuItemLabel, string, AlertStatusPodMenuItem, StatusTip, _StatusPodPopupMixin,
	MRI, domGeom, MenuSeparator, Menu, TabContainer, ContentPane) {
	return declare([
		WidgetBase,
		TemplatedMixin,
		WidgetsInTemplateMixin,
		_StatusPodPopupMixin ], {
		templateString : template,
		templateMRI : MRI,
		recentProblemsCount : 0,
		totalProblemsCount : 0,

		recentCompliancesCount : 0,
		totalCompliancesCount : 0,

		complianceItems : null,
		statusItems : null,
		cache : null,

		postCreate : function () {
			this.menuItems = [];
			this.complianceItems = [], this.statusItems = [], this.cache = {
				problems : [],
				compliance : []
			};
			this.inherited(arguments);
		},

		startup : function () {
			this.inherited(arguments);
			//change the style of the TabContainer
			if(this.statusPodTabContainer.tablist.pane2button){
				this.statusPodTabContainer.tablist.pane2button["compliances_pane"].tabContent.style.borderRightStyle="none";
			}
			

		},

		updateAlerts : function (items, useCache, menuNode) {
			var itemList = [];
			items = items || [];
			var itemIndex = 0, filteredItems = 0, totalItems = 0;
			var itemsLength = items.length;
			for (itemIndex = 0; itemIndex < itemsLength; ++itemIndex) {
				var item = items[itemIndex], severityId, severityValue, sourceName, objectType, image, itemId, statuscatName;
				if (useCache) {
					severityId = item.severityId;
					sourceName = item.sourceName;
					objectType = item.objectType;
					itemId = item.itemId;
					severityValue = item.severityValue;
					image = item.image;
				} else {
					if (menuNode === this.problemsMenu) {
						statuscatName="STATUS_CAT:Problems";
					} else{
						statuscatName="STATUS_CAT:ComplianceStatusCategory";
					}
					severityId = item.getProperty(statuscatName).getValue().getId();
					severityValue = item.getProperty(statuscatName).getValue().getValue();
					sourceName = item.getProperty("DisplayName").getValue();
					objectType = item.getProperty("ObjectType").getDisplayValue();
					problemImage = item.getProperty(statuscatName).getValue().getImage();
					itemId = item.getId();
					complianceImage = item.getProperty(statuscatName).getValue().getImage();
					var cacheItem = {
						severityId : severityId,
						sourceName : sourceName,
						objectType : objectType,
						severityValue : severityValue,
						itemId : itemId
					};
					if (menuNode === this.problemsMenu) {
						cacheItem.image = problemImage;
						image = problemImage;
						this.cache.problems.push(cacheItem);
					} else {
						cacheItem.image = complianceImage;
						image = complianceImage;
						this.cache.compliance.push(cacheItem);
					}
				}
				if (menuNode === this.problemsMenu) {
					if (severityId < 0 || severityId > 3) {
						continue;
					} else {
						totalItems++;
					}
				} else {
					if (severityId < 0 || (severityId > 3 && severityId != 5)) {
						continue;
					} else {
						totalItems++;
					}
				}

				itemList.push({
					itemId : itemId,
					severityId : severityValue
				});
				if (filteredItems >= 10) {
					continue;
				}
				filteredItems++;
				//var image = item.getProperty("STATUS").getValue().getImage();
				//var dateTime=item.getProperty("DateTime").getValue();
				//var shortTime= this._getTime(dateTime);
				var shortTime = "";
				//dateTime=this._formatDateStr(dateTime);

				var menuLabel = string.substitute(menuItemLabel, {
					//image: image,
					sourceName : sourceName,
					shortTime : shortTime,
					objectType : objectType
				});

				var menuItem = new AlertStatusPodMenuItem({
					label : menuLabel,
					column : 0,
					iconClass : "statusPodMenuItemIcon",
					objectId : itemId
				});

				menuItem.iconNode.src = image;

				var menuItemHoverTip = new StatusTip({
					entityNode : menuItem.domNode,
					objectId : itemId,
					isProblems : menuNode === this.problemsMenu
				});

				menuItem.setHoverTip(menuItemHoverTip);

				menuNode.addChild(menuItem, 2 * itemIndex);

				this.addDescendant(menuItem);

				var menuSep = new MenuSeparator();
				menuNode.addChild(menuSep, 2 * itemIndex + 1);
			}
			
			if(filteredItems === 0){
				if (menuNode === this.problemsMenu) {
					this.problemAlertsSpace.style.display="block";
				} else if (menuNode === this.compliancesMenu) {
					this.complianceAlertsSpace.style.display="block";
				} 
			} else{
				if (menuNode === this.problemsMenu) {
					this.problemAlertsSpace.style.display="none";
				} else if (menuNode === this.compliancesMenu) {
					this.complianceAlertsSpace.style.display="none";
				} 
			}
			
			return {
				filterCount : filteredItems,
				totalCount : totalItems,
				itemList : itemList
			};
		},
		clearCache : function () {
			this.cache = {
				problems : [],
				compliance : []
			};
		},
		updateProblems : function (items, fromCache) {
			var result = this.updateAlerts(items, fromCache, this.problemsMenu);

			this.set("totalProblemsCount", result.totalCount);
			this.set("recentProblemsCount", result.filterCount);
			this.statusItems = result.itemList;
		},

		updateCompliances : function (items, fromCache) {
			var result = this.updateAlerts(items, fromCache, this.compliancesMenu);

			this.set("totalCompliancesCount", result.totalCount);
			this.set("recentCompliancesCount", result.filterCount);
			this.complianceItems = result.itemList;

		},

		_formatDateStr : function (dateTime) {
			var formatedStr;
			var dateStrArr;
			if (dateTime) {
				dateStrArr = dateTime.toString().split(" ");
				formatedStr = dateStrArr.slice(1, 5).join(" ");
			}
			//console.debug("shorten date is:" + formatedStr);
			return formatedStr;
		},

		_getTime : function (dateTime) {
			var timeStr;
			var dateStrArr;
			if (dateTime) {
				dateStrArr = dateTime.toString().split(" ");
				timeStr = dateStrArr[4].split(":").slice(0, 2).join(":");
			}
			return timeStr;
		},

		_setRecentProblemsCountAttr : function (problemsCount) {
			this._logger.finest("set recently problems", "_setRecentProblemsCountAttr");
			this.recentProblemsCount = problemsCount;
			var text = lang.replace(MRI.StatusShowingPartial, [
				this.recentProblemsCount,
				this.totalProblemsCount ]);
			this.problemsCatSummary.innerHTML = text;
		},

		_setRecentCompliancesCountAttr : function (problemsCount) {
			this._logger.finest("set recently compliances", "_setRecentCompliancesCountAttr");
			this.recentCompliancesCount = problemsCount;
			var text = lang.replace(MRI.StatusShowingPartial, [
				this.recentCompliancesCount,
				this.totalCompliancesCount ]);
			this.compliancesCatSummary.innerHTML = text;

		},

		_setTotalProblemsCountAttr : function (problemsCount) {
			this._logger.finest("set total problems", "_setTotalProblemsCountAttr");
			this.totalProblemsCount = problemsCount;
			if (this.problems_pane.controlButton) {
				this.problems_pane.controlButton.set("label", lang.replace(MRI.StatusProblems,
					[ this.totalProblemsCount ]));
			} else {
				this._logger.finest("no controlButton for the page", "_setTotalProblemsCountAttr");
			}
		},

		_setTotalCompliancesCountAttr : function (problemsCount) {
			this._logger.finest("set total compliances", "_setTotalCompliancesCountAttr");
			this.totalCompliancesCount = problemsCount;
			if (this.compliances_pane.controlButton) {
				this.compliances_pane.controlButton.set("label", lang.replace(MRI.StatusCompliance,
					[ this.totalCompliancesCount ]));
			} else {
				this._logger.finest("no controlButton for the page", "_setTotalCompliancesCountAttr");
			}
		},
		destroy : function () {
			this.inherited(arguments);
			if (this.menuItems && this.menuItems.length > 0) {
				var i = 0;
				for (; i < this.menuItems.length; i++) {
					this.menuItems[i].destroyDescendants();
				}
				this.menuItems = [];

			}
		},

		clearDescendants : function () {
			this._logger.finest("clear descendants of alert status pod dialog", "clearDescendants");
			if (this.menuItems) {
				array.forEach(this.menuItems, function (menuItem) {
					if (menuItem) {
						menuItem.destroy();
					}
				});
				this.menuItems = [];
			}
			this.problemsMenu.destroyDescendants();
			this.compliancesMenu.destroyDescendants();

		}
	});
});

},
'dijit/form/NumberTextBox':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/_base/lang", // lang.hitch lang.mixin
	"dojo/number", // number._realNumberRegexp number.format number.parse number.regexp
	"./RangeBoundTextBox"
], function(declare, lang, number, RangeBoundTextBox){

/*=====
	var RangeBoundTextBox = dijit.form.RangeBoundTextBox;
=====*/

	// module:
	//		dijit/form/NumberTextBox
	// summary:
	//		A TextBox for entering numbers, with formatting and range checking


	/*=====
	declare(
		"dijit.form.NumberTextBox.__Constraints",
		[dijit.form.RangeBoundTextBox.__Constraints, number.__FormatOptions, number.__ParseOptions], {
		// summary:
		//		Specifies both the rules on valid/invalid values (minimum, maximum,
		//		number of required decimal places), and also formatting options for
		//		displaying the value when the field is not focused.
		// example:
		//		Minimum/maximum:
		//		To specify a field between 0 and 120:
		//	|		{min:0,max:120}
		//		To specify a field that must be an integer:
		//	|		{fractional:false}
		//		To specify a field where 0 to 3 decimal places are allowed on input:
		//	|		{places:'0,3'}
	});
	=====*/

	var NumberTextBoxMixin = declare("dijit.form.NumberTextBoxMixin", null, {
		// summary:
		//		A mixin for all number textboxes
		// tags:
		//		protected

		// Override ValidationTextBox.regExpGen().... we use a reg-ex generating function rather
		// than a straight regexp to deal with locale (plus formatting options too?)
		regExpGen: number.regexp,

		/*=====
		// constraints: dijit.form.NumberTextBox.__Constraints
		//		Despite the name, this parameter specifies both constraints on the input
		//		(including minimum/maximum allowed values) as well as
		//		formatting options like places (the number of digits to display after
		//		the decimal point).  See `dijit.form.NumberTextBox.__Constraints` for details.
		constraints: {},
		======*/

		// value: Number
		//		The value of this NumberTextBox as a Javascript Number (i.e., not a String).
		//		If the displayed value is blank, the value is NaN, and if the user types in
		//		an gibberish value (like "hello world"), the value is undefined
		//		(i.e. get('value') returns undefined).
		//
		//		Symmetrically, set('value', NaN) will clear the displayed value,
		//		whereas set('value', undefined) will have no effect.
		value: NaN,

		// editOptions: [protected] Object
		//		Properties to mix into constraints when the value is being edited.
		//		This is here because we edit the number in the format "12345", which is
		//		different than the display value (ex: "12,345")
		editOptions: { pattern: '#.######' },

		/*=====
		_formatter: function(value, options){
			// summary:
			//		_formatter() is called by format().  It's the base routine for formatting a number,
			//		as a string, for example converting 12345 into "12,345".
			// value: Number
			//		The number to be converted into a string.
			// options: dojo.number.__FormatOptions?
			//		Formatting options
			// tags:
			//		protected extension

			return "12345";		// String
		},
		 =====*/
		_formatter: number.format,

		postMixInProperties: function(){
			this.inherited(arguments);
			this._set("type", "text"); // in case type="number" was specified which messes up parse/format
		},

		_setConstraintsAttr: function(/*Object*/ constraints){
			var places = typeof constraints.places == "number"? constraints.places : 0;
			if(places){ places++; } // decimal rounding errors take away another digit of precision
			if(typeof constraints.max != "number"){
				constraints.max = 9 * Math.pow(10, 15-places);
			}
			if(typeof constraints.min != "number"){
				constraints.min = -9 * Math.pow(10, 15-places);
			}
			this.inherited(arguments, [ constraints ]);
			if(this.focusNode && this.focusNode.value && !isNaN(this.value)){
				this.set('value', this.value);
			}
		},

		_onFocus: function(){
			if(this.disabled){ return; }
			var val = this.get('value');
			if(typeof val == "number" && !isNaN(val)){
				var formattedValue = this.format(val, this.constraints);
				if(formattedValue !== undefined){
					this.textbox.value = formattedValue;
				}
			}
			this.inherited(arguments);
		},

		format: function(/*Number*/ value, /*dojo.number.__FormatOptions*/ constraints){
			// summary:
			//		Formats the value as a Number, according to constraints.
			// tags:
			//		protected

			var formattedValue = String(value);
			if(typeof value != "number"){ return formattedValue; }
			if(isNaN(value)){ return ""; }
			// check for exponential notation that dojo.number.format chokes on
			if(!("rangeCheck" in this && this.rangeCheck(value, constraints)) && constraints.exponent !== false && /\de[-+]?\d/i.test(formattedValue)){
				return formattedValue;
			}
			if(this.editOptions && this.focused){
				constraints = lang.mixin({}, constraints, this.editOptions);
			}
			return this._formatter(value, constraints);
		},

		/*=====
		_parser: function(value, constraints){
			// summary:
			//		Parses the string value as a Number, according to constraints.
			// value: String
			//		String representing a number
			// constraints: dojo.number.__ParseOptions
			//		Formatting options
			// tags:
			//		protected

			return 123.45;		// Number
		},
		=====*/
		_parser: number.parse,

		parse: function(/*String*/ value, /*number.__FormatOptions*/ constraints){
			// summary:
			//		Replaceable function to convert a formatted string to a number value
			// tags:
			//		protected extension

			var v = this._parser(value, lang.mixin({}, constraints, (this.editOptions && this.focused) ? this.editOptions : {}));
			if(this.editOptions && this.focused && isNaN(v)){
				v = this._parser(value, constraints); // parse w/o editOptions: not technically needed but is nice for the user
			}
			return v;
		},

		_getDisplayedValueAttr: function(){
			var v = this.inherited(arguments);
			return isNaN(v) ? this.textbox.value : v;
		},

		filter: function(/*Number*/ value){
			// summary:
			//		This is called with both the display value (string), and the actual value (a number).
			//		When called with the actual value it does corrections so that '' etc. are represented as NaN.
			//		Otherwise it dispatches to the superclass's filter() method.
			//
			//		See `dijit.form.TextBox.filter` for more details.
			return (value === null || value === '' || value === undefined) ? NaN : this.inherited(arguments); // set('value', null||''||undefined) should fire onChange(NaN)
		},

		serialize: function(/*Number*/ value, /*Object?*/ options){
			// summary:
			//		Convert value (a Number) into a canonical string (ie, how the number literal is written in javascript/java/C/etc.)
			// tags:
			//		protected
			return (typeof value != "number" || isNaN(value)) ? '' : this.inherited(arguments);
		},

		_setBlurValue: function(){
			var val = lang.hitch(lang.mixin({}, this, { focused: true }), "get")('value'); // parse with editOptions
			this._setValueAttr(val, true);
		},

		_setValueAttr: function(/*Number*/ value, /*Boolean?*/ priorityChange, /*String?*/ formattedValue){
			// summary:
			//		Hook so set('value', ...) works.
			if(value !== undefined && formattedValue === undefined){
				formattedValue = String(value);
				if(typeof value == "number"){
					if(isNaN(value)){ formattedValue = '' }
					// check for exponential notation that number.format chokes on
					else if(("rangeCheck" in this && this.rangeCheck(value, this.constraints)) || this.constraints.exponent === false || !/\de[-+]?\d/i.test(formattedValue)){
						formattedValue = undefined; // lets format compute a real string value
					}
				}else if(!value){ // 0 processed in if branch above, ''|null|undefined flows through here
					formattedValue = '';
					value = NaN;
				}else{ // non-numeric values
					value = undefined;
				}
			}
			this.inherited(arguments, [value, priorityChange, formattedValue]);
		},

		_getValueAttr: function(){
			// summary:
			//		Hook so get('value') works.
			//		Returns Number, NaN for '', or undefined for unparseable text
			var v = this.inherited(arguments); // returns Number for all values accepted by parse() or NaN for all other displayed values

			// If the displayed value of the textbox is gibberish (ex: "hello world"), this.inherited() above
			// returns NaN; this if() branch converts the return value to undefined.
			// Returning undefined prevents user text from being overwritten when doing _setValueAttr(_getValueAttr()).
			// A blank displayed value is still returned as NaN.
			if(isNaN(v) && this.textbox.value !== ''){
				if(this.constraints.exponent !== false && /\de[-+]?\d/i.test(this.textbox.value) && (new RegExp("^"+number._realNumberRegexp(lang.mixin({}, this.constraints))+"$").test(this.textbox.value))){	// check for exponential notation that parse() rejected (erroneously?)
					var n = Number(this.textbox.value);
					return isNaN(n) ? undefined : n; // return exponential Number or undefined for random text (may not be possible to do with the above RegExp check)
				}else{
					return undefined; // gibberish
				}
			}else{
				return v; // Number or NaN for ''
			}
		},

		isValid: function(/*Boolean*/ isFocused){
			// Overrides dijit.form.RangeBoundTextBox.isValid to check that the editing-mode value is valid since
			// it may not be formatted according to the regExp validation rules
			if(!this.focused || this._isEmpty(this.textbox.value)){
				return this.inherited(arguments);
			}else{
				var v = this.get('value');
				if(!isNaN(v) && this.rangeCheck(v, this.constraints)){
					if(this.constraints.exponent !== false && /\de[-+]?\d/i.test(this.textbox.value)){ // exponential, parse doesn't like it
						return true; // valid exponential number in range
					}else{
						return this.inherited(arguments);
					}
				}else{
					return false;
				}
			}
		}
	});
/*=====
	NumberTextBoxMixin = dijit.form.NumberTextBoxMixin;
=====*/

	var NumberTextBox = declare("dijit.form.NumberTextBox", [RangeBoundTextBox,NumberTextBoxMixin], {
		// summary:
		//		A TextBox for entering numbers, with formatting and range checking
		// description:
		//		NumberTextBox is a textbox for entering and displaying numbers, supporting
		//		the following main features:
		//
		//			1. Enforce minimum/maximum allowed values (as well as enforcing that the user types
		//				a number rather than a random string)
		//			2. NLS support (altering roles of comma and dot as "thousands-separator" and "decimal-point"
		//				depending on locale).
		//			3. Separate modes for editing the value and displaying it, specifically that
		//				the thousands separator character (typically comma) disappears when editing
		//				but reappears after the field is blurred.
		//			4. Formatting and constraints regarding the number of places (digits after the decimal point)
		//				allowed on input, and number of places displayed when blurred (see `constraints` parameter).

		baseClass: "dijitTextBox dijitNumberTextBox"
	});

	NumberTextBox.Mixin = NumberTextBoxMixin;	// for monkey patching

	return NumberTextBox;
});

},
'ibm/tivoli/navmodel/WidgetDelta':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/Trace"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.WidgetDelta"); 
 
// includes
dojo.require("ibm.tivoli.navmodel.Trace");

ibm.tivoli.navmodel.WidgetDelta._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.WidgetDelta");

////////////////////////////////////
// WidgetDelta
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.WidgetDelta", null, {
	// private members       
	_widgetDef: null,
	_type: null,
	_delta: null,
	_pending: false,	
	_trace: ibm.tivoli.navmodel.WidgetDelta._trace,

	/*=====
	 constructor_kwArgs: function(widgetDef, type) {
		// widgetDef: ibm.tivoli.navmodel.WidgetDef (or descendent)
		//		widget definition object to manipulate delta
		// type: String
		//		the delta type:  "user" or "shared"		
	},
	=====*/

	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		var args = {
			widgetDef: null,
			type: null
		};
		dojo.mixin(args, kwArgs);
		
		// summary:
		//		Constructs WidgetDelta object.
		this._widgetDef = args.widgetDef;
		this._type = (args.type ? args.type : this._widgetDef.getArg("type"));
		
		switch(this._type) {
			case "user":
				this._delta = this._widgetDef.getUserDelta();
				break;
			case "shared":
				this._delta = this._widgetDef.getSharedDelta();
				break;
			default:
				this._trace.error("Unknown delta type=" + this._type, "constructor");
		}
		
		if (this._delta) {
			this._delta = dojo.clone(this._delta);
		}
		else {
			this._delta = {};
		}		
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
		
	getType: function() {
		// summary:
		//		Return delta type: either "user" or "shared"
		return this._type;
	},
	
	hasPendingChanges: function() {
		// summary:
		//		Returns true if changes are pending
		return this._pending;
	},
	
	save: function() {
		// summary:
		//		Save current delta changes to the widget registry;
		//		delete the delta if it is empty.
		if (this.hasPendingChanges()) {
			switch(this._type) {
				case "user":
					if (this._hasKeys(this._delta)) {
						this._widgetDef.setUserDelta(this._delta);
					}
					else {
						this._widgetDef.deleteUserDelta();
					}
					break;
				case "shared":			
					if (this._hasKeys(this._delta)) {
						this._widgetDef.setSharedDelta(this._delta);
					}
					else {
						this._widgetDef.deleteSharedDeleta();
					}
			}
			this._pending = false;
		}
	},
	
	clear: function() {
		// summary:
		//		Clear the entire delta
		this._delta = {};
		this._pending = true;
	},
	
	/*=====
	 clearSection_kwArgs: function(sectionKey) {
	 	//	sectionKey: String
		//		The key that identifies the section		
	},
	=====*/
	
	clearSection: function(/*clearSection_kwArgs*/ kwArgs) {
		if (this._delta[kwArgs.sectionKey]) {
			delete this._delta[kwArgs.sectionKey];			
		}
		
		this._pending = true;
	},
	
	/*=====
	 sectionAttribute_kwArgs: function(sectionKey, attrKey) {
	 	//	sectionKey: String
		//		The key that identifies the section
		//	attrKey: String
		//		The attribute in the section
	},
	=====*/
	
	getSectionAttribute: function(/*sectionAttribute_kwArgs*/ kwArgs) {
		// summary:
		//		Get a key/value in a specified section
		//
		//	Example:
		//	{	...
		//		"topology: {
		//			"backgroundColor": "white",
		//			...
		//		},
		//		...
		//	}
		//	Get "backgroundColor" attribute in "topology" section 
		//	var mode = delta.getSectionAttribute({sectionKey: "topology", attrKey: "backgroundColor"});		
		var value = null;
		var section = this._delta[kwArgs.sectionKey];
		
		// section exists in delta
		if (section) {
			value = section[kwArgs.attrKey];						
		}
		
		return value;	
	},
	
	/*=====
	 setSectionAttribute_kwArgs: function(sectionKey, attrKey, attrValue) {
	 	//	sectionKey: String
		//		The key that identifies the section
		//	attrKey: String
		//		The attribute in the section
		//	attrValue: any
		//		The value to be set on the attribute
	},
	=====*/
		
	setSectionAttribute: function(/*setSectionAttribute_kwArgs*/ kwArgs) {
		// summary:
		//		Set a key/value in a specified section
		//
		//	Example:
		//	{	...
		//		"topology: {
		//			"backgroundColor": "white",
		//			...
		//		},
		//		...
		//	}
		//	Set "backgroundColor" attribute in "topology" section 
		//	delta.setSectionAttribute({sectionKey: "topology", attrKey: "backgroundColor", attrValue: "gray"});		
		var rc = false;
		var section = this._delta[kwArgs.sectionKey];
		
		// section exists in delta
		if (section) {
			section[kwArgs.attrKey] = kwArgs.attrValue;			
			rc = true;			
		}
		// section does not exist in delta
		else {
			section = {};
			section[kwArgs.attrKey] = kwArgs.attrValue;
			this._delta[kwArgs.sectionKey] = section;
			rc = true;
		}
		
		this._pending = true;
		return rc;		
	},
	
	removeSectionAttribute: function(/*sectionAttribute_kwArgs*/ kwArgs) {
		// summary:
		//		 Remove a key from a specified array element (where section is an array)
		//
		//	Example:
		//	{	...
		//		"topology: {
		//			"backgroundColor": "white",
		//			...
		//		},
		//		...
		//	}
		//	Remove "backgroundColor" attribute from "topology" section
		//	delta.removeSectionAttribute({sectionKey: "topology", attrKey: "backgroundColor"});				
		var rc = false;
		var section = this._delta[kwArgs.sectionKey];
		
		// section exists in delta
		if (section) {
			delete section[kwArgs.attrKey];
			rc = true;
			
			// if section is empty; we can remove from delta
			if (!this._hasKeys(section)) {
				delete this._delta[kwArgs.sectionKey];
			}					
		}
			
		this._pending = true;					
		return rc;		
	},
	
	/*=====
	 arraySectionAttribute_kwArgs: function(sectionKey, elementKey, elementValue, attrKey) {
	 	//	sectionKey: String
		//		The key that identifies the section
		//	elementKey: String
		//		The key used to identify the array element
		//	elementValue: String
		//		The value used to identify the array element (i.e. compare to elementKey)
		//	attrKey: String
		//		The attribute in the array element			
	},
	=====*/
		
	getArraySectionAttribute: function(/*arraySectionAttribute_kwArgs*/ kwArgs) {
		// summary:
		//		Get a key/value in a specified array element (where section is an array)
		//
		//	Example:
		//	{	...
		//		"tables: [
		//			{	"id": "ItemsTable",
		//				"filter": {
		//					...
		//				},
		//				...
		//			},
		//			...
		//		],
		//		...
		//	}
		//	Get "filter" attribute in "tables" section (i.e. the element where "id" is "ItemsTable")		
		//	var filter = delta.getArraySectionAttribute({sectionKey: "tables", elementKey: "id", elementValue: "ItemsTable", attrKey: "filter"});
		
		var value = null;
		var section = this._delta[kwArgs.sectionKey];
		
		// array section exists in delta
		if (section && dojo.isArray(section)) {
			var count = section.length;
			for (var index = 0;  index < count;  ++index) {
				var element = section[index];
				if (element[kwArgs.elementKey] === kwArgs.elementValue) {
					value = element[kwArgs.attrKey];
					break;
				}
			}				
		}
		
		return value;	
	},
		
	/*=====
	 setArraySectionAttribute_kwArgs: function(sectionKey, elementKey, elementValue, attrKey, attrValue) {
	 	//	sectionKey: String
		//		The key that identifies the section
		//	elementKey: String
		//		The key used to identify the array element
		//	elementValue: String
		//		The value used to identify the array element (i.e. compare to elementKey)
		//	attrKey: String
		//		The attribute to be modified in the array element
		//	attrValue: any
		//		The value to be set on the array element attribute
	},
	=====*/
	
	setArraySectionAttribute: function(/*setArraySectionAttribute_kwArgs*/ kwArgs) {
		// summary:
		//		Set a key/value in a specified array element (where section is an array)
		//
		//	Example:
		//	{	...
		//		"tables: [
		//			{	"id": "ItemsTable",
		//				"filter": {
		//					...
		//				},
		//				...
		//			},
		//			...
		//		],
		//		...
		//	}
		//	Set "filter" attribute in "tables" section (i.e. the element that has "id" = "ItemsTable")		
		//	delta.setArraySectionAttribute({sectionKey: "tables", elementKey: "id", elementValue: "ItemsTable", attrKey: "filter", attrValue: value});
		
		var rc = false;
		var section = this._delta[kwArgs.sectionKey];
		
		// array section exists in delta
		if (section && dojo.isArray(section)) {
			var count = section.length;
			for (var index = 0;  index < count;  ++index) {
				var element = section[index];
				if (element[kwArgs.elementKey] === kwArgs.elementValue) {
					element[kwArgs.attrKey] = kwArgs.attrValue;
					rc = true;
					break;
				}
			}				
		}
		// array section does not exist in delta
		else {
			var newElement = {};
			newElement[kwArgs.elementKey] = kwArgs.elementValue;
			newElement[kwArgs.attrKey] = kwArgs.attrValue;
			this._delta[kwArgs.sectionKey + "Identifier"] = kwArgs.elementKey;			
			this._delta[kwArgs.sectionKey] = [newElement];
			rc = true;
		}
		
		this._pending = true;					
		return rc;		
	},
	
	removeArraySectionAttribute: function(/*arraySectionAttribute_kwArgs*/ kwArgs) {
		// summary:
		//		 Remove a key from a specified array element (where section is an array)
		//
		//	Example:
		//	{	...
		//		"tables: [
		//			{	"id": "ItemsTable",
		//				"filter": {
		//					...
		//				},
		//				...
		//			},
		//			...
		//		],
		//		...
		//	}
		//	Remove "filter" attribute from "tables" section (i.e. the element that has "id" = "ItemsTable")		
		//	delta.removeArraySectionAttribute({sectionKey: "tables", elementKey: "id", elementValue: "ItemsTable", attrKey: "filter"});
				
		var rc = false;
		var section = this._delta[kwArgs.sectionKey];
		
		// array section exists in delta
		if (section && dojo.isArray(section)) {
			var count = section.length;
			for (var index = 0;  index < count;  ++index) {
				var element = section[index];
				if (element[kwArgs.elementKey] === kwArgs.elementValue) {					
					delete element[kwArgs.attrKey];
					rc = true;
					
					// if the only key left in the array element is the
					// elementKey; then we can remove this element
					var keys = this._getKeys(element);
					if (keys.length === 1 && keys[0] === kwArgs.elementKey) {
						section.splice(index,1);
					}
					
					// if array is empty; we can remove from the delta
					if (section.length === 0) {
						delete this._delta[kwArgs.sectionKey];
						delete this._delta[kwArgs.sectionKey + "Identifier"];
					}					
					break;
				}
			}				
		}
				
		this._pending = true;						
		return rc;		
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////
	_hasKeys: function(obj) {
		for (var key in obj) {
			if (obj.hasOwnProperty(key)) {
				return true;
			}
		}
		
		return false;
	},
	
	_getKeys: function(obj) {
		var keys = [];		
		for (var key in obj) {
			if (obj.hasOwnProperty(key)) {
				keys.push(key);
			}
		}
		return keys;
	}
});

});

},
'ibm/tivoli/tip/navwidget/topology/dijit/DefaultActionHelper':function(){
// wrapped by build app
define("ibm/tivoli/tip/navwidget/topology/dijit/DefaultActionHelper", ["dijit","dojo","dojox","dojo/i18n!ibm/tivoli/tip/navwidget/nls/resources","dojo/require!ibm/tivoli/tip/navwidget/properties/dijit/PropertiesNotebookDialog,ibm/tivoli/navmodel/NavWidgetDef,ibm/tivoli/navmodel/Trace"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.topology.dijit.DefaultActionHelper");

dojo.require("ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebookDialog");
dojo.require("ibm.tivoli.navmodel.NavWidgetDef");
dojo.require("ibm.tivoli.navmodel.Trace");
dojo.requireLocalization("ibm.tivoli.tip.navwidget", "resources");


dojo.declare("ibm.tivoli.tip.navwidget.topology.dijit.DefaultActionHelper",
	      null,
{		
	// summary:
	//		Coming soon
	// description:
	//		Coming soon

	// defaultActionCallback: function
	//		callback method for the default action
	defaultActionCallback: null,
	
	constructor: function( /*Object*/ args ){
		// summary:
		//		The base class for all providers that populate the TIPContextMenu
		// args:
		//		Coming soon
		
	      dojo.mixin( this, args );
	},

	executeDefaultAction: function(/*Object*/ objectContext, /*Property*/ property){
		// summary:
		//		Method which checks context for default server task and executes if needed
		//		else executes the callback method
		// description:
		//		Coming soon
		// objectContext:
		//		Coming soon
		var object = null;
		
		if(property){
			
			object = property;
			
		}else{
			//First check to ensure a server side task is not defined.
			//if so execute it
			var selectedObjects = objectContext.selection;
			if(( selectedObjects.getRelationshipCount() + selectedObjects.getLinkCount() + selectedObjects.getItemCount() ) != 1){
				//if there is more than one item then just return and do nothing for now

				//TODO: ADM - add case for additional objects.  It may be possible some
				//default action is needed on multiple items/links/relationships
				//not sure what the use case is at this moment tho
				return;
			}


			if(selectedObjects.getItemCount() == 1){
				object = selectedObjects.getItems()[0];            
			}else if(selectedObjects.getRelationshipCount() == 1){
				object = selectedObjects.getRelationships()[0];
			}else if(selectedObjects.getLinkCount() == 1){
				object = selectedObjects.getLinks()[0];
			}else{
				console.log("Cannot handle non link, rel, or item objects. No action preformed.");
			}
			
			
			if(!object.supportsProperties()){
				//if the object doesnt support properties, we shouldnt render the prop notebook
				console.log("object doesnt support properties, preforming no action");
				return;
			}

		}
        

		
        // if a default task is defined for the item or property, execute it
		// 251346
		var defaultTask = property ? object.getTask() : object.getDefaultTask();
		if (defaultTask) {
			var menuProvider = this._getCMProvider(objectContext);
			if (menuProvider) {
				// 251771
				// portlet namespace might not be initialized yet for the menu provider
				// if no context menu has yet been displayed, so check
				if (!menuProvider.iscPortletNamespace && objectContext.domNode) {
					menuProvider.setPortletNamespace(objectContext.domNode);
				}
				// Menu provider's method to map tasks to JSON expects an array
				// var tasks = [defaultTask];
				var jsonMenuItem = (menuProvider.tasksAsMenuJson([defaultTask]))[0];
				jsonMenuItem.selection = objectContext.selection;
				menuProvider.handleMenuAction(jsonMenuItem, {task: defaultTask});
				return;
			}			
		}
	
		//If the default action callback is set, then execute it
		if(this.defaultActionCallback){
			return this.defaultActionCallback(objectContext, property);
		}
		
	},
	
	_getCMProvider: function(objectContext){
		var cmProvider = null;
		var topView = objectContext.getTopLevelPluggableView();
		
		if(objectContext){
			if(objectContext.cmProvider){
				cmProvider = objectContext.cmProvider;
			
			}else if(topView && topView.cmProvider){
			
				cmProvider = topView.cmProvider;
	
			}else if(topView && topView.managingPluggableView && topView.managingPluggableView.cmProvider){
			
				cmProvider = topView.managingPluggableView.cmProvider;
			}
			else{
				console.log("unable to find an appropriate cmProvider");
			}
		}
		return cmProvider;
	
	}

});
});

},
'gridx/modules/SingleSort':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/keys",
	"../core/model/extensions/Sort",
	"../core/_Module"
], function(declare, lang, keys, Sort, _Module){

	return declare(/*===== "gridx.modules.SingleSort", =====*/_Module, {
		// summary:
		//		This module provides the single column sorting functionality for grid.

		name: 'sort',

		forced: ['header'],

		modelExtensions: [Sort],
		
		getAPIPath: function(){
			// tags:
			//		protected extension
			return {
				sort: this
			};
		},

		preload: function(){
			// tags:
			//		protected extension
			var t = this,
				g = t.grid, sort;
			t.connect(g, 'onHeaderCellClick', '_onClick');
			t.connect(g, 'onHeaderCellKeyDown', '_onKeyDown');
			//persistence support
			if(g.persist){
				sort = g.persist.registerAndLoad('sort', function(){
					return [{
						colId: t._sortId,
						descending: t._sortDescend 
					}];
				});
			}
			//Presort...
			sort = sort || t.arg('initialOrder');
			if(lang.isArrayLike(sort)){
				sort = sort[0];
			}
			if(sort && sort.colId){
				t._sortId = sort.colId;
				t._sortDescend = sort.descending;
				//sort here so the body can render correctly.
				t.model.sort([sort]);
			}
		},
	
		load: function(){
			// tags:
			//		protected extension
			var t = this,
				colId,
				f = function(){
					if(t._sortId){
						t._updateHeader(t._sortId, t._sortDescend);
					}
				};
			t.connect(t.grid.header, 'onRender', f);
			for(colId in t.grid._columnsById){
				t._initHeader(colId);
			}
			//If presorted, update header UI
			f();
			t.loaded.callback();
		},
	
		columnMixin: {
			sort: function(isDescending, skipUpdateBody){
				// summary:
				//		Sort this column.
				this.grid.sort.sort(this.id, isDescending, skipUpdateBody);
				return this;
			},
	
			isSorted: function(){
				// summary:
				//		Check wheter this column is sorted.
				return this.grid.sort.isSorted(this.id);
			},
	
			clearSort: function(skipUpdateBody){
				// summary:
				//		Clear sort on this column
				if(this.isSorted()){
					this.grid.sort.clear(skipUpdateBody);
				}
				return this;
			},
	
			isSortable: function(){
				// summary:
				//		Check whether this column is sortable.
				var col = this.grid._columnsById[this.id];
				return col.sortable || col.sortable === undefined;
			},
	
			setSortable: function(isSortable){
				// summary:
				//		Set sortable for this column
				this.grid._columnsById[this.id].sortable = !!isSortable;
				return this;
			}
		},
	
		//Public--------------------------------------------------------------

	/*=====
		// initialOrder: Object|Array
		//		The initial sort order when grid is created.
		//		This is of the same format of the sort argument of the store fetch function.
		//		If an array of sort orders is provided, only the first will be used.
		initialOrder: null,
	=====*/

		sort: function(colId, isDescending, skipUpdateBody){
			// summary:
			//		Sort the grid on given column.
			// colId: String
			//		The column ID
			// isDescending: Boolean?
			//		Whether to sort the column descendingly
			// skipUpdateBody: Boolean?
			//		If set to true, the grid body will not automatically be refreshed after this call,
			//		so that several grid operations could be done altogether
			//		without refreshing the grid over and over.
			var t = this, g = t.grid, col = g._columnsById[colId];
			if(col && (col.sortable || col.sortable === undefined)){
				if(t._sortId != colId || t._sortDescend == !isDescending){
					t._updateHeader(colId, isDescending);
				}
				t.model.sort([{colId: colId, descending: isDescending}]);
				if(!skipUpdateBody){
					g.body.refresh();
				}
			}
		},
	
		isSorted: function(colId){
			// summary:
			//		Check wheter (and how) the grid is sorted on the given column.
			// colId: String
			//		The columnn ID
			// returns:
			//		Positive number if the column is sorted ascendingly;
			//		Negative number if the column is sorted descendingly;
			//		Zero if the column is not sorted.
			if(colId == this._sortId){
				return this._sortDescend ? -1 : 1;	//Number
			}
			return 0;	//Number
		},
	
		clear: function(skipUpdateBody){
			// summary:
			//		Clear sort.
			// skipUpdateBody:
			//		If set to true, the grid body will not automatically be refreshed after this call,
			//		so that several grid operations could be done altogether
			//		without refreshing the grid over and over.
			var t = this;
			if(t._sortId !== null){
				t._initHeader(t._sortId);
				t._sortId = t._sortDescend = null;
				t.model.sort();
				if(!skipUpdateBody){
					t.grid.body.refresh();
				}
			}
		},

		getSortData: function(){
			// summary:
			//		Get an array of objects that can be accepted by the store's "sort" argument.	
			// returns:
			//		An array containing the sort info
			return this._sortId ? [{	//Object[]|null
				colId: this._sortId, 
				descending: this._sortDescend
			}] : null;
		},
	
		//Private--------------------------------------------------------------
		_sortId: null,

		_sortDescend: null,
		
		_initHeader: function(colId){
			var g = this.grid,
				headerCell = g.header.getHeaderNode(colId),
				col = g.column(colId, 1);	//1 as true
			headerCell.innerHTML = ["<div class='gridxSortNode'>", col.name(), "</div>"].join('');
			if(col.isSortable()){
				headerCell.setAttribute('aria-sort', 'none');
			}else{
				headerCell.removeAttribute('aria-sort');
			}
		},
	
		_updateHeader: function(colId, isDescending){
			//Change the structure of sorted header
			var t = this;
			if(t._sortId && t._sortId != colId){
				t._initHeader(t._sortId);
			}
			t._sortId = colId;
			t._sortDescend = !!isDescending;
			var g = t.grid,
				headerCell = g.header.getHeaderNode(colId);
			headerCell.innerHTML = ["<div class='gridxSortNode ",
				isDescending ? 'gridxSortDown' : 'gridxSortUp',
				"'><div class='gridxArrowButtonChar'>",
				isDescending ? "&#9662;" : "&#9652;",
				"</div><div role='presentation' class='gridxArrowButtonNode'>&nbsp;</div><div class='gridxColCaption'>",
				g.column(colId, 1).name(),	//1 as true
				"</div></div>"
			].join('');
			headerCell.setAttribute('aria-sort', isDescending ? 'descending' : 'ascending');
			g.vLayout.reLayout();
		},
	
		_onClick: function(e){
			this.sort(e.columnId, this._sortId != e.columnId ? 0 : !this._sortDescend);
		},
		
		_onKeyDown: function(e){
			if(e.keyCode == keys.ENTER){
				this._onClick(e);
			}
		}
	});
});

},
'dojo/_base/query':function(){
define(["./kernel", "../query", "./NodeList"], function(dojo){
	return dojo.query;
});

},
'dijit/form/NumberSpinner':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/_base/event", // event.stop
	"dojo/keys", // keys.END keys.HOME
	"./_Spinner",
	"./NumberTextBox"
], function(declare, event, keys, _Spinner, NumberTextBox){

/*=====
	var _Spinner = dijit.form._Spinner;
	var NumberTextBox = dijit.form.NumberTextBox;
=====*/

// module:
//		dijit/form/NumberSpinner
// summary:
//		Extends NumberTextBox to add up/down arrows and pageup/pagedown for incremental change to the value


return declare("dijit.form.NumberSpinner", [_Spinner, NumberTextBox.Mixin], {
	// summary:
	//		Extends NumberTextBox to add up/down arrows and pageup/pagedown for incremental change to the value
	//
	// description:
	//		A `dijit.form.NumberTextBox` extension to provide keyboard accessible value selection
	//		as well as icons for spinning direction. When using the keyboard, the typematic rules
	//		apply, meaning holding the key will gradually increase or decrease the value and
	// 		accelerate.
	//
	// example:
	//	| new dijit.form.NumberSpinner({ constraints:{ max:300, min:100 }}, "someInput");

	adjust: function(/*Object*/ val, /*Number*/ delta){
		// summary:
		//		Change Number val by the given amount
		// tags:
		//		protected

		var tc = this.constraints,
			v = isNaN(val),
			gotMax = !isNaN(tc.max),
			gotMin = !isNaN(tc.min)
		;
		if(v && delta != 0){ // blank or invalid value and they want to spin, so create defaults
			val = (delta > 0) ?
				gotMin ? tc.min : gotMax ? tc.max : 0 :
				gotMax ? this.constraints.max : gotMin ? tc.min : 0
			;
		}
		var newval = val + delta;
		if(v || isNaN(newval)){ return val; }
		if(gotMax && (newval > tc.max)){
			newval = tc.max;
		}
		if(gotMin && (newval < tc.min)){
			newval = tc.min;
		}
		return newval;
	},

	_onKeyPress: function(e){
		if((e.charOrCode == keys.HOME || e.charOrCode == keys.END) && !(e.ctrlKey || e.altKey || e.metaKey)
		&& typeof this.get('value') != 'undefined' /* gibberish, so HOME and END are default editing keys*/){
			var value = this.constraints[(e.charOrCode == keys.HOME ? "min" : "max")];
			if(typeof value == "number"){
				this._setValueAttr(value, false);
			}
			// eat home or end key whether we change the value or not
			event.stop(e);
		}
	}
});

});

},
'url:dijit/templates/CheckedMenuItem.html':"<tr class=\"dijitReset dijitMenuItem\" data-dojo-attach-point=\"focusNode\" role=\"menuitemcheckbox\" tabIndex=\"-1\"\n\t\tdata-dojo-attach-event=\"onmouseenter:_onHover,onmouseleave:_onUnhover,ondijitclick:_onClick\">\n\t<td class=\"dijitReset dijitMenuItemIconCell\" role=\"presentation\">\n\t\t<img src=\"${_blankGif}\" alt=\"\" class=\"dijitMenuItemIcon dijitCheckedMenuItemIcon\" data-dojo-attach-point=\"iconNode\"/>\n\t\t<span class=\"dijitCheckedMenuItemIconChar\">&#10003;</span>\n\t</td>\n\t<td class=\"dijitReset dijitMenuItemLabel\" colspan=\"2\" data-dojo-attach-point=\"containerNode,labelNode\"></td>\n\t<td class=\"dijitReset dijitMenuItemAccelKey\" style=\"display: none\" data-dojo-attach-point=\"accelKeyNode\"></td>\n\t<td class=\"dijitReset dijitMenuArrowCell\" role=\"presentation\">&#160;</td>\n</tr>\n",
'dojoe/tablex/_QuickFilter':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/i18n!dojoe/tablex/nls/Table","dojo/require!dijit/_Widget,dijit/_TemplatedMixin"], function(dijit,dojo,dojox){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
// dojo.provide allows pages to use all of the types declared in this resource.
dojo.provide("dojoe.tablex._QuickFilter");

//dojo.require the necessary dijit hierarchy
dojo.require("dijit._Widget");
dojo.require("dijit._TemplatedMixin");

//NLS
dojo.requireLocalization("dojoe.tablex", "Table");

dojo.declare("dojoe.tablex._QuickFilter", [ dijit._Widget, dijit._TemplatedMixin ], {

	// summary:
	//		Private class - Quick Filter widget option in toolbar
	
	// Path to the template
	templateString : dojo.cache("dojoe.tablex", "templates/QuickFilter.html", "<div class=\"dijitReset  dijitTextBox quickFilter\" dojoAttachEvent='onmouseover:_onHover,mouseleave:_onUnHover,onblur:_onBlur,onclick: _onClick'>\r\n\t<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" width=\"100%\" role=\"presentation\">\r\n\t\t<tbody>\r\n\t\t\t<tr valign=\"center\">\r\n\t\t\t\t<td width=\"98%\" height=\"100%\">\r\n\t\t\t\t  <div>\t\t    \r\n\t\t\t\t\t\t<input dojoAttachPoint=\"searchField\" title=\"${resources_.QUICK_FILTER}\" class=\"dijitInputInner qf-textfield\" type=\"text\" \r\n\t\t\t\t\t\t\tvalue=\"${resources_.QF_DEFAULT_TEXT}\" valign=\"middle\" tabindex=\"0\" role=\"textbox\" \r\n\t\t\t\t\t\t\tdojoAttachEvent='onfocus:_onFocus,onkeyup:_onKeyUp,onpaste:_onKeyUp,oncut:_onKeyUp'/>\t\r\n\t\t\t\t  </div>\r\n\t\t\t\t</td>\t\r\n\t\t\t\t<td width=\"2%\" height=\"100%\">\r\n\t\t\t\t  <div class=\"dijitReset  dijitArrowButtonContainer \">\r\n\t\t\t\t\t\t<button title=\"${resources_.CLEAR_FILTER}\" class=\"dijitReset dijitInline dijitIcon quickFilterClearInner\" type=\"button\" tabindex=\"0\" \r\n\t\t\t\t\t\t\tvalign=\"middle\" dojoAttachPoint='clearFilterNode' dojoAttachEvent='onclick:_onCancel,onkeypress:_onCancel' role=\"button\"></button>\t\t\r\n\t\t\t\t  </div>\r\n\t\t\t\t</td>\r\n\t\t\t</tr>\r\n\t\t</tbody>\t\t\r\n\t</table>\r\n</div>"),
	// Set this to true if your widget contains other widgets
	widgetsInTemplate : false,	
	grid:null,	
	timer:null,
	//amortiz begin XB132526 - 189503
	dontClearTextBox : false,
	//amortiz end XB132526 - 189503
	filterOptions: null,
	_quickFilterDelay: 700,	//delay in ms before quick filter is applied as typed
	_quickFilterButton: false, //change filter on type based on this
	/*#22195: 29/06/11 Using Quick Filter in table replaces an existing advanced filter*/
	//adding a private variable to keep track of previous QF string if any
	_savedCriteria: null,
	//LS - #24957 - 22/07/2011 - Problems with ruleCountToConfirmClearFilter - retaining the value set by user if any
	_ruleCntClearFilter: null,
	_retainSelection: false,//if true selection won't be cleared on filter
	
	// Override this method to perform custom behavior during dijit construction.
	// Common operations for constructor:
	// 1) Initialize non-primitive types (i.e. objects and arrays)
	// 2) Add additional properties needed by succeeding lifecycle methods
	constructor : function() {
	   this.resources_ = dojo.i18n.getLocalization("dojoe.tablex", "Table", this.lang);   
	},
	
	postCreate : function () {
	    //console.dir(this.filterOptions);
		if(this.filterOptions){
			if(this.filterOptions.showQuickFilterButton !== undefined) this._quickFilterButton = this.filterOptions.showQuickFilterButton;
			if(this.filterOptions.quickFilterDelay) this._quickFilterDelay = this.filterOptions.quickFilterDelay;
			if(this.filterOptions.retainSelection) this._retainSelection = this.filterOptions.retainSelection;
		}
		/*LS - 2/08/2011 - 25076 - Treetable - when clicking clear filter 'x' button few times, data is not re-fetched (server-side store) */
		//disabling clear filter icon when there is no filter applied to avoid above confusion, to have same behavior in table QF
		var filterText = this.searchField.value;
	   	if(this.filterText !== null && this.filterText !== "" && filterText !== this.resources_.QF_DEFAULT_TEXT  )	{
			dojo.style(this.clearFilterNode, "display", "block");
		}else{
			dojo.style(this.clearFilterNode, "display", "none");
		}				
	}, 		
	
	// When this method is called, all variables inherited from superclasses are 'mixed in'.
	// Common operations for postMixInProperties
	// 1) Modify or assign values for widget property variables defined in the template HTML file
	postMixInProperties : function() {	    
	    this.connect(this.grid, "onFilterDefined",  "_onFilterDefined");
	    /*#22195: 29/06/11 Using Quick Filter in table replaces an existing advanced filter*/
		if(this.grid.filterBar){
			//clearing quick filter text (not filter)when advanced filter is opened..
		   	this.connect(this.grid.filterBar, "showFilterDialog", "_onShowDialog");
			//Defect:50665 - Simple Filter in the grid header does not get cleared, after clearing it from within the grid
			//this.connect(this.grid.filterBar, "onDomClick", "_clearQuickFilterText");			
			this.connect(this.grid.filterBar, "clearFilter", "_clearQuickFilterText");			
	    }
	},
	
	_clearQuickFilterText: function() {
		//amortiz begin XB132526 - 189503
		if (this.dontClearTextBox) {
			this._savedCriteria = null;
		} else {
			this._clearText();
		}
		//amortiz end XB132526 - 189503
	},
	
	_onFocus : function() {
		dojo.addClass(this.domNode, 'dijitTextBoxFocused');
		if (this.searchField.value == this.resources_.QF_DEFAULT_TEXT) {
			//Richa- Color settings should be there in css files
			//this.searchField.style.color = "#000000";
			this.searchField.value = "";
		}
	},
	
	//Latha : 14Mar2011 Fix for defect 5523. Setting Focus to filter box
	_onClick : function () {		
		dojo.addClass(this.domNode, 'dijitTextBoxFocused');
		this.searchField.focus();
		//Richa- Color settings should be there in css files
		//this.searchField.style.color = "#000000";
	},

	_onBlur : function() {
		dojo.removeClass( this.domNode, 'dijitTextBoxFocused' );
		dojo.removeClass( this.domNode, 'dijitTextBoxHover' );
		if (this.searchField.value === "") {
			this.searchField.style.color = "#999999";
			this.searchField.value = this.resources_.QF_DEFAULT_TEXT;
		}
	},

    _onHover : function() {
		dojo.addClass(this.domNode, 'dijitTextBoxHover');
	},

	_onUnHover : function(){
		dojo.removeClass( this.domNode, 'dijitTextBoxHover' );
	},

	_onKeyUp: function(event){		
		//enter key
		if (event.keyCode == 13) {
			//event.cancelBubble = true;
			//event.returnValue = false;        				
			dojo.stopEvent(event);
			this._setFilter();
			return;
		}//Bobby: 23Feb2011, don't do filter when tabbed thru
		//tab & shift keys
		else if(event.keyCode == 9 || event.keyCode == 16) {
			//event.cancelBubble = true;
			//event.returnValue = false; 
			//Bobby:17Apr2011, tab may get stuck if stopped
			//dojo.stopEvent(event);			
			//console.info("tab key -"+ event.keyCode);
			//clearTimeout(this.timer);
			//this.timer = null;
			return;
		}else if(this._quickFilterButton){
			return;
		}
		//Reset the timer since a key was pressed!
		else if(this.timer){
			clearTimeout(this.timer);
			this.timer = null;
		}		
		//Add delay before setting filter. 
		this.timer = setTimeout(dojo.hitch(this, "_setFilter"), this._quickFilterDelay);		
	},	
	
	_onShowDialog: function() {
		 /*#22195: 29/06/11 Using Quick Filter in table replaces an existing advanced filter*/
		 //clearing quick filter text (not filter)when advanced filter dialog is opened..
		this._clearText();
	},
        	
	_setFilter: function(){
	    var input = this.searchField.value;
	    /*#22195: 29/06/11 Using Quick Filter in table replaces an existing advanced filter*/
	    //connecting clear text and enter/afbutton press to clear QF criteria
		if (input == this.resources_.QF_DEFAULT_TEXT || input == "") {
			input = "";
			//amortiz begin XB132526 - 189503
			this.dontClearTextBox = true;
			//amortiz end XB132526 - 189503
			
			this._clearQuickFilter();
			return;
		}
		var filterData  = {"type":"any", "conditions":
		[{  "type": "string",
		  "condition": "contain",
		  "value": input
		  }]};
		//taking previous criteria to check if filter has to be added or replaced  
		if(this._savedCriteria){
			var previousCriteria = this._savedCriteria;}
		this._savedCriteria = filterData;
		if(this.grid.filterBar ){
			var filterSet = false;
			var filterObj = null;
			if(this.grid.filterBar.filterData){
				filterObj = this.grid.filterBar.filterData;
			}
			if(previousCriteria){
				for(var i=0; i< filterObj.conditions.length; i++){
					if(!(filterObj.conditions[i].colId)  && (previousCriteria.conditions[0].value == filterObj.conditions[i].value) ){
						filterObj.conditions[i].value = filterData.conditions[0].value;
						filterSet = true;
						break;
					}
				}
				if(filterSet)filterData = filterObj;
			}else if(filterObj){
				filterObj.conditions = filterObj.conditions.concat(filterData.conditions);
				console.dir(filterObj);
				filterData = filterObj;
			}
			
		}
		//setting ruleCountToConfirmClearFilter to infinity so that AF clearFilter confirm popup never shows while setting filter
		//dojo has this attribute default as 2 and when fitler rules exceeds 2, on setFilter also this pop up starts showing up instead of clear filter 
		//LS - #24957 - 22/07/2011 - Problems with ruleCountToConfirmClearFilter - retaining the value set by user if any
		this._ruleCntClearFilter = this.grid.filterBar.ruleCountToConfirmClearFilter;
		this.grid.filterBar.ruleCountToConfirmClearFilter = 'Infinity';
		this.grid.filterBar.applyFilter(filterData);	
		/*LS - 2/08/2011 - 25076 - Treetable - when clicking clear filter 'x' button few times, data is not re-fetched (server-side store) */
			//disabling clear filter icon when there is no filter applied to avoid above confusion, applying to table also to maintain similar behavior	
		dojo.style(this.clearFilterNode, "display", "block");	
	},
	
	_onFilterDefined: function(rules){
		//Bobby:18Aug2011, adding option to prevent clearing selection on Filter
		//27171: Need option to allow selection after filtering
		if(this._retainSelection == false){
			//Bobby:31Mar2011, clearing selection on Filter
			//13285: DOJO:Allow for selection to be cleared when filter occurs
			this.grid.selection.clear();
		}
	

		/*#22195: 29/06/11 Using Quick Filter in table replaces an existing advanced filter*/
		//commenting following lines as this would set QF with anycolumn value even if defined by advanced filter
		//also commenting timer fix, because focus should not go to QF after filterdefined, becuase filter defined can be from AF
	    //if(rules.length == 1 && rules[0].column == "anycolumn" && rules[0].condition == "contains"){
		//			this.searchField.value = rules[0].value;
		//}
		//Latha : 29Mar2011 Fix for defect 7284. Table Quick Filter focus changes while typing into the filter box.
		//var timer = setTimeout(dojo.hitch(this, "_setFocus"), 700);
		//Now the confirm dialog will show
		//this has to be done since clearfilter is called upon setFilter in dojo Grid, causes unnecessary popup's
		//LS - #24957 - 22/07/2011 - Problems with ruleCountToConfirmClearFilter - retaining the value set by user if any
		if(this._ruleCntClearFilter)
			this.grid.filterBar.ruleCountToConfirmClearFilter = this._ruleCntClearFilter;
	},
	
	_setFocus: function(){
	    this.searchField.focus();
	},
		
	_onCancel: function(event){
		//Bobby: 23Feb2011, don't do anything when tabbed thru
		//tab & shift keys	
		if(event.keyCode == 9 || event.keyCode == 16) {
			//event.cancelBubble = true;
			//event.returnValue = false;
			//Bobby:17Apr2011, tab gets stuck if stopped
			//dojo.stopEvent(event);
			//console.info("tab key -"+ event.keyCode);
			return;
		}else{
			/*this.searchField.style.color = "#999999";
			this.searchField.value = this.resources_.QF_DEFAULT_TEXT;	  
			this.grid.setFilter(null);*/
			this._clearQuickFilter();			
		}		
	},
	
	//Bobby:04Mar2011
	//QuickFilter textbox also needs to be cleared	
	_clearQuickFilter: function(){
		/*#22195: 29/06/11 Using Quick Filter in table replaces an existing advanced filter*/
		if(this.grid.filterBar && this.grid.filterBar.filterData && this._savedCriteria) {
			 var filterExp = this.grid.filterBar.filterData;
			 var filterArr = filterExp.conditions;
			 console.info(this._savedCriteria.conditions[0].value);
	 		 for(var i=0;i<filterArr.length;i++){
					
				if(!(filterArr[i].colId )&& (this._savedCriteria.conditions[0].value == filterArr[i].value) ){
					console.info("here inside the if...");
					filterArr.splice(i,1);
					break;
			 	}
	 		 }
			 if(filterArr.length > 0){
				filterExp.conditions = filterArr;
				this.grid.filterBar.applyFilter(filterExp);
			 }else{
				this.grid.filterBar.clearFilter();
			 }
			 	
		}	
		//amortiz begin XB132526 - 189503
		if (!this.dontClearTextBox) {
			//amortiz end XB132526 - 189503
			this.searchField.style.color = "#999999";
			this.searchField.value = this.resources_.QF_DEFAULT_TEXT;
			/*LS - 2/08/2011 - 25076 - Treetable - when clicking clear filter 'x' button few times, data is not re-fetched (server-side store) */
			//disabling clear filter icon when there is no filter applied to avoid above confusion, applying to table also to maintain similar behavior	
			dojo.style(this.clearFilterNode, "display", "none");
			//amortiz begin XB132526 - 189503			
		}
		this.dontClearTextBox = false;
		//amortiz end XB132526 - 189503

	},
	
    _clearText: function() {
    	/*#22195: 29/06/11 Using Quick Filter in table replaces an existing advanced filter*/
    	//adding to just clear text of QF and not filter as such
		this.searchField.style.color = "#999999";
		this.searchField.value = this.resources_.QF_DEFAULT_TEXT;
		this._savedCriteria = null;
		/*LS - 2/08/2011 - 25076 - Treetable - when clicking clear filter 'x' button few times, data is not re-fetched (server-side store) */
		//disabling clear filter icon when there is no filter applied to avoid above confusion, applying to table also to maintain similar behavior	
		dojo.style(this.clearFilterNode, "display", "none");	
	},
	
	
	destroy: function(){
		//memory leak issue: custom destroy method to take care of removing all widgets					
		try{
			delete this.resources_;
			//this is for destroying child templated widgets 
			//this should happen by itself but need to figure out why dijit._Templated is not taking care of it...
			var widgets = dijit.findWidgets(this.domNode);
			dojo.forEach(widgets, function(w) {
				if(w){
					if(w.destroyRecursive){
						w.destroyRecursive();
					} else if(w.destroy){
						w.destroy();
					}
				}
			});
			// this is for widgets added as child to this widget and their children..
			//not present in template
			var widgetArr = this.getChildren();
			dojo.forEach(widgetArr, function (child) {
				if(child){
					if(child.destroyRecursive){
						child.destroyRecursive();
					} else if(child.destroy){
						child.destroy();
					}
				}
			});
			this.inherited(arguments);
		}catch(ex){
			console.warn("Widget destroy failed: "+this.id+", ex="+ex.name+":"+ex.message);
		}
	}
	
});
});

},
'ibm/tivoli/tip/dijit/TIPContextMenu':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!dijit/Menu,ibm/tivoli/tip/dijit/TIPMessageDialog,dijit/_Widget,dijit/_Templated,ibm/tivoli/tip/dojo/Global,ibm/tivoli/tip/dojo/Server,ibm/tivoli/tip/dijit/ContextMenuAdapter,ibm/tivoli/tip/dojo/EventUtil,ibm/tivoli/tip/dijit/TIPContextMenuProvider,ibm/tivoli/tip/dijit/CMSMenuProvider,ibm/tivoli/tip/dijit/CMSMenuProviderV2"], function(dijit,dojo,dojox){
/*
	Copyright (c) 2004-2011, The Dojo Foundation All Rights Reserved.
	Available via Academic Free License >= 2.1 OR the modified BSD license.
	see: http://dojotoolkit.org/license for details
*/


if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPContextMenu"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPContextMenu"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2007 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
dojo.require("dijit.Menu");
dojo.require("ibm.tivoli.tip.dijit.TIPMessageDialog");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("ibm.tivoli.tip.dojo.Global");
dojo.require("ibm.tivoli.tip.dojo.Server");
dojo.require("ibm.tivoli.tip.dijit.ContextMenuAdapter");
dojo.require("ibm.tivoli.tip.dojo.EventUtil");
dojo.require("ibm.tivoli.tip.dijit.TIPContextMenuProvider");
dojo.require("ibm.tivoli.tip.dijit.CMSMenuProvider");
dojo.require("ibm.tivoli.tip.dijit.CMSMenuProviderV2");


dojo.provide("ibm.tivoli.tip.dijit.TIPContextMenu");



/**
 *  Provides dynamic context menu support.
 *  How do you use this?
 *  First, you create an instance of this context menu similar to:
 *     this._contextMenuAdapter = new ibm.tivoli.tip.dijit.TIPContextMenu({cmProvider: this.cmProvider});
 *  Note: you pass in an instance of a context menu provider (see objects lower in this file)
 *
 *  Second, you latch whatever event you want to create the context menu.  For 
 *  Grids, you might do something like this:
 *     this._onRowContextMenuListener = dojo.connect( this.grid, "onRowContextMenu", this, "_onRowContextMenu" );
 *  Don't forget to disconnect this listener when your object is destroyed.
 *
 *  Third, you create the method that gets called by the above connect, and call
 *  the contextMenuOpen on this context menu with whatever context you want 
 *  processed.  Again, in a Grid example:
 *   _onRowContextMenu: function( e )
 *   { 
 *     var row = e.grid.model.getRow( e.rowIndex );
 *     
 *     if ( row )
 *     {
 *       //select the row that the context menu was initiated on
 *       e.grid.selection.unselectAll( e.rowIndex );
 *       e.grid.selection.setSelected( e.rowIndex, true );
 *               
 *       //tell the context menu adapter to show the menu
 *       // no context is provided because we overload the _loadMenuItems method
 *       this._contextMenuAdapter.contextMenuOpen( e, {selectedData: this.getSelectionData()} );
 *     }
 *   },
 *
 *  
 */
dojo.declare("ibm.tivoli.tip.dijit.TIPContextMenu",
              ibm.tivoli.tip.dijit.ContextMenuAdapter,
{
   cmProvider: null,       // TIPContextMenuProvider
   cmClientProvider: null,
   url: "no_url_needed",   // Note we have to pass something to ContextMenuAdapter
                           // for the url so that we can override _loadMenuItems
   
   /****************************************************************************
   * Callback that should be overloaded to be told when a menu item is clicked
   * and an action should occur
   ****************************************************************************/
   onMenuItemClick: function( menuItem )
   { 
      console.debug("-------------- menuItem :",menuItem);
      try{
    	  this.cmClientProvider.getMenuCommand(menuItem.actionID, this._context, menuItem);
      }catch(e){
    	  this.cmProvider.getMenuCommand(menuItem.actionID, this._context, menuItem);
      }
   },

   /****************************************************************************
   * Synchronously Call a JSON Url to get the menu items
   *
   ****************************************************************************/
   _loadMenuItems: function()
   {
	   return this.cmClientProvider ? this.cmClientProvider.getMenuItems( this._context ).concat (this.cmProvider.getMenuItems( this._context )) : this.cmProvider.getMenuItems( this._context );
   }

});

}

});

},
'dijit/_base/window':function(){
define([
	"dojo/window", // windowUtils.get
	".."	// export symbol to dijit
], function(windowUtils, dijit){
	// module:
	//		dijit/_base/window
	// summary:
	//		Back compatibility module, new code should use windowUtils directly instead of using this module.

	dijit.getDocumentWindow = function(doc){
		return windowUtils.get(doc);
	};
});

},
'dojox/grid/enhanced/plugins/_SelectionPreserver':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/connect",
	'../../_SelectionPreserver'
], function(declare, lang, connect, _SelectionPreserver){

return declare("dojox.grid.enhanced.plugins._SelectionPreserver", _SelectionPreserver, {
	// summary:
	//		Preserve selections across various user actions.
	//
	// description:
	//		Extends dojox.grid._SelectionPreserver adding a bit more support to make selection persistence working well
	//		with various EnhancedGrid features, e.g. filtering, nested sorting, pagination, select all etc.
	//
	//		Precondition - Identifier(id) is required for store, as id is used for differentiating row items.
	//		Known issue - The preserved selections might be inaccurate if some unloaded rows are previously selected by range(e.g.SHIFT + click)
	//
	// example:
	// |	//To turn on this - set 'keepSelection' attribute to true
	// |	<div dojoType="dojox.grid.EnhancedGrid" keepSelection = true .../>
	
	constructor: function(selection){
		var grid = this.grid;
		grid.onSelectedById = this.onSelectedById;
		this._oldClearData = grid._clearData;
		var self = this;
		grid._clearData = function(){
			self._updateMapping(!grid._noInternalMapping);
			self._trustSelection = [];
			self._oldClearData.apply(grid, arguments);
		};
		this._connects.push(
			connect.connect(selection, 'selectRange', lang.hitch(this, '_updateMapping', true, true, false)),
			connect.connect(selection, 'deselectRange', lang.hitch(this, '_updateMapping', true, false, false)),
			connect.connect(selection, 'deselectAll', lang.hitch(this, '_updateMapping', true, false, true))
		);
	},
	destroy: function(){
		this.inherited(arguments);
		this.grid._clearData = this._oldClearData;
	},
	reset: function(){
		this.inherited(arguments);
		this._idMap = [];
		this._trustSelection = [];
		this._defaultSelected = false;
	},
	_reSelectById: function(item, index){
		// summary:
		//		Overwritten
		var s = this.selection, g = this.grid;
		if(item && g._hasIdentity){
			var id = g.store.getIdentity(item);
			if(this._selectedById[id] === undefined){
				if(!this._trustSelection[index]){
					s.selected[index] = this._defaultSelected;
				}
			}else{
				s.selected[index] = this._selectedById[id];
			}
			this._idMap.push(id);
			g.onSelectedById(id, index, s.selected[index]);
		}
	},
	_selectById: function(toSelect, inItemOrIndex){
		// summary:
		//		Overwritten
		if(!this.inherited(arguments)){
			this._trustSelection[inItemOrIndex] = true;
		}
	},
	onSelectedById: function(id, rowIndex, value){},
	
	_updateMapping: function(trustSelection, isSelect, isForAll, from, to){
		// summary:
		//		This function try to keep the selection info updated when range selection is performed.
		//		1. Calculate how many unloaded rows are there;
		//		2. update _selectedById data if grid.selection._selected can be trusted, so loaded but unselected rows can
		//			be properly recorded.
		var s = this.selection, g = this.grid, flag = 0, unloaded = 0, i, id;
		for(i = g.rowCount - 1; i >= 0; --i){
			if(!g._by_idx[i]){
				++unloaded;
				flag += s.selected[i] ? 1 : -1;
			}else{
				id = g._by_idx[i].idty;
				if(id && (trustSelection || this._selectedById[id] === undefined)){
					this._selectedById[id] = !!s.selected[i];
				}
			}
		}
		if(unloaded){
			this._defaultSelected = flag > 0;
		}
		if(!isForAll && from !== undefined && to !== undefined){
			isForAll = !g.usingPagination && Math.abs(to - from + 1) === g.rowCount;
		}
		// When deselectAll, make sure every thing is deselected, even if it was selected but not loaded now.
		// This occurs only when pagination's "All" is used.
		if(isForAll && (!g.usingPagination || g.selectionMode === 'single')){
			for(i = this._idMap.length - 1; i >= 0; --i){
				this._selectedById[this._idMap[i]] = isSelect;
			}
		}
	}
});
});

},
'gridx/modules/exporter/Exporter':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"../../core/_Module",
	"dojo/_base/Deferred",
	"dojo/_base/array"
], function(declare, lang, _Module, Deferred, array){

/*=====
	__ExportArgs = function(){
		//columns: String[]?
		//		An array of column ID. Indicates which columns to be exported. 
		//		If invalid or empty array, export all grid columns.
		//start: Number?
		//		Indicates from which row to start exporting. If invalid, 
		//		default to 0.
		//count: Number?
		//		Indicates the count of rows export.
		//		If invalid, export all rows up to the end of grid.
		//selectedOnly: Boolean?
		//		Whether only export selected rows. This constraint is applied 
		//		upon the rows specified by start and count parameters.
		//		This paramter and the filter parameter can be used togetther. 
		//		Default to false.
		//filter: Function?
		//		A predicate function (returns Boolean) to judge whether to 
		//		export a row. 
		//		This constraint is applied upon the result rows of the 
		//		selectedOnly parameter, if provided.
		//useStoreData: Boolean?
		//		Indicates whether to export grid data (formatted data) or 
		//		store data. Default to false.
		//formatters: Associative array?
		//		A customized way to export data, if neither grid data nor store 
		//		data could meet the requirement. 
		//		This is an associative array from column id to formatter function. 
		//		A grid cell object will be passed into that function as an argument.
		//omitHeader: Boolean?
		//		Indicates whether to export grid header. Default to false.
		//progressStep: Number?
		//		Number of rows in each progress. Default to 0 (invalid means only one progress).
		//		After each progress, the deferred.progress() is called, so the 
		//		exporting process can be controlled by the user.
	};
=====*/

/*=====
	var __ExportContext = function(){
		//columnIds: String[]
		//		Available for header.
		//column: Grid Column
		//		Available for header cell or a body cell.
		//row: Grid Row
		//		Available for a row or a body cell.
		//cell: Grid Cell
		//		Available for a body cell
	};
=====*/

	function check(writer, method, context, args){
		return !writer[method] || false !== writer[method](context || args, args);
	}

	function prepareReqs(args, rowIndexes, size){
		var reqs = [],
			i, start = 0, end = 0,
			ps = args.progressStep;
		if(typeof args.start == 'number' && args.start >= 0){
			start = args.start;
		}
		if(typeof args.count == 'number' && args.count > 0){
			end = start + args.count;
		}
		end = end || size;
		if(rowIndexes){
			rowIndexes = array.filter(rowIndexes, function(idx){
				return idx >= start && idx < end;
			});
			if(rowIndexes.length && (!ps || rowIndexes.length <= ps)){
				reqs.push(rowIndexes);
			}else{
				for(i = 0; i < rowIndexes.length; i += ps){
					reqs.push(rowIndexes.slice(i, i + ps));
				}
			}
		}else{
			var count = end - start;
			if(!ps || count <= ps){
				reqs.push({
					start: start,
					count: count
				});
			}else{
				for(i = start; i < end; i += ps){
					reqs.push({
						start: i,
						count: i + ps < end ? ps : end - i
					});
				}
			}
		}
		reqs.p = 0;
		return reqs;
	}

	function first(req, grid){
		return lang.isArray(req) ? {
			p: 0,
			row: grid.row(req[0])
		} : {
			p: req.start,
			row: grid.row(req.start)
		};
	}

	function next(req, grid, prevRow){
		var p = prevRow.p + 1,
			isArray = lang.isArray(req);
		return p < (isArray ? req.length : req.start + req.count) ? {
			p: p,
			row: grid.row(isArray ? req[p] : p)
		} : null;
	}

	return _Module.register(
	declare(_Module, {
		name: 'exporter',

		getAPIPath: function(){
			return {
				'exporter': this
			};
		},
	
		//Package ---------------------------------------------------------------------
		_export: function(writer, /* __ExportArgs */ args){
			// summary:
			//		Go through the grid using the given args and writer implementation.
			//		Return a dojo.Deferred object. Users can cancel and see progress 
			//		of the exporting process.
			//		Pass the exported result to the callback function of the Deferred object.
			// tags:
			//		private
			var d = new Deferred(),
				t = this,
				model = t.model,
				cols = t._getColumns(args),
				s = t.grid.select,
				sr = s && s.row,
				sc = s && s.column,
				waitForRows,
				rowIds,
				context = {
					columnIds: cols
				},
				success = function(){
					check(writer, 'afterBody', context, args);
					d.callback(writer.getResult());
				},
				fail = lang.hitch(d, d.errback);
			
			try{
				check(writer, 'initialize', 0, args);
				if(!args.omitHeader && check(writer, 'beforeHeader', context, args)){
					array.forEach(cols, function(cid){
						context.column = t.grid.column(cid, 1);	//1 as true
						check(writer, 'handleHeaderCell', context, args);
					});
					check(writer, 'afterHeader', context, args);
				}
				if(check(writer, 'beforeBody', context, args)){
					if(args.selectedOnly && sr && (!sc || !sc.getSelected().length)){
						waitForRows = model.when().then(function(){
							rowIds = sr.getSelected();
						}, fail);
					}
					Deferred.when(waitForRows, function(){
						var rowIdxes,
							waitForRowIndex = rowIds && model.when({id: rowIds}, function(){
								rowIdxes = array.map(rowIds, function(id){
									return model.idToIndex(id);
								});
								rowIdxes.sort(function(a, b){
									return a - b;
								});
							}, fail);
						Deferred.when(waitForRowIndex, function(){
							var dd = new Deferred(),
								rowCount = model.size();
							t._fetchRows(d, writer, context, args, dd, prepareReqs(args, rowIdxes, rowCount));
							dd.then(success, fail);
						}, fail);
					}, fail);
				}else{
					d.callback(writer.getResult());
				}
			}catch(e){
				fail(e);
			}
			return d;
		},

		//Private---------------------------------------------------------------------------
		_fetchRows: function(defer, writer, context, args, d, reqs){
			var t = this,
				g = t.grid,
				f = args.filter,
				cols = context.columnIds,
				req = reqs[reqs.p++],
				fail = lang.hitch(d, d.errback),
				func = function(){
					defer.progress(reqs.p / reqs.length);
					t.model.when(req, function(){
						for(var r = first(req, g); r && r.row; r = next(req, g, r)){
							context.row = r.row;
							if((!f || f(r.row)) && check(writer, 'beforeRow', context, args)){
								for(var i = 0; i < cols.length; ++i){
									var col = g.column(cols[i], 1),	//1 as true
										cell = context.cell = g.cell(r.row, col);
									context.column = col;
									context.data = t._format(args, cell);
									check(writer, 'handleCell', context, args);
								}
								check(writer, 'afterRow', context, args);
							}
						}
					}).then(function(){
						t._fetchRows(defer, writer, context, args, d, reqs);
					}, fail);
				};
			if(req){
				if(reqs.length > 1){
					setTimeout(func, 10);
				}else{
					func();
				}
			}else{
				d.callback();
			}
		},

		_format: function(args, cell){
			var fs = args.formatters,
				cid = cell.column.id;
			if(fs && lang.isFunction(fs[cid])){
				return fs[cid](cell);
			}else if(args.useStoreData){
				return cell.rawData() || '';
			}
			return cell.data() || '';
		},

		_getColumns: function(args){
			var g = this.grid,
				colsById = g._columnsById,
				s = g.select,
				sc = s && s.column,
				cols;
			if(lang.isArrayLike(args.columns) && args.columns.length){
				cols = array.filter(args.columns, function(cid){
					return colsById[cid];
				});
				cols.sort(function(a, b){
					return colsById[a].index - colsById[b].index;
				});
			}else{
				cols = array.map(g._columns, function(c){
					return c.id;
				});
			}
			return cols;
		}
	}));
});

},
'dojoe/treetable/_QuickSearchFilter':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/i18n!dojoe/treetable/nls/TreeTable","dojo/require!dijit/_Widget,dijit/_Templated,dojoe/treetable/filter/_FilterExpr,dojoe/treetable/filter/FilterLayer"], function(dijit,dojo,dojox){
// dojo.provide allows pages to use all of the types declared in this resource.
dojo.provide("dojoe.treetable._QuickSearchFilter");

//dojo.require the necessary dijit hierarchy
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojoe.treetable.filter._FilterExpr");
dojo.require("dojoe.treetable.filter.FilterLayer");

dojo.requireLocalization("dojoe.treetable", "TreeTable");
dojo.declare("dojoe.treetable._QuickSearchFilter", [dijit._Widget,dijit._Templated], {
	
		// summary:
		//		Private class to create and handle Quick Search Filter on the Tree table toolbar
		// tags: private
	
		// Path to the template
		templateString : dojo.cache("dojoe.treetable", "templates/QuickSearchFilter.html", "<div class=\"dijitReset  dijitTextBox quickFilter\" dojoAttachEvent='onmouseover:_onHover,mouseleave:_onUnHover,onblur:_onBlur,onclick: _onClick'>\r\n\t<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" width=\"100%\" role=\"presentation\">\r\n\t\t<tbody>\r\n\t\t\t<tr valign=\"center\">\r\n\t\t\t\t<td width=\"98%\" height=\"100%\">\r\n\t\t\t\t\t<div class=\"dijitReset  dijitInputContainer\">\t\t    \r\n\t\t\t\t\t\t<input dojoAttachPoint=\"searchField\" title=\"${resources_.QUICK_FILTER}\" class=\"dijitInputInner qf-textfield\" type=\"text\" \r\n\t\t\t\t\t\t\tvalue=\"${resources_.QF_DEFAULT_TEXT}\" valign=\"middle\" tabindex=\"0\"\r\n\t\t\t\t\t\t\tdojoAttachEvent='onfocus:_onFocus,onclick:_onClick,onkeyup:_onKeyUp,onpaste:_onKeyUp,oncut:_onKeyUp,onKeyPress:_onKeyUp' role=\"textbox\"/>\t\r\n\t\t\t\t\t</div>\r\n\t\t\t\t</td>\t\r\n\t\t\t\t<td width=\"2%\" height=\"100%\">\r\n\t\t\t\t\t<div class=\"dijitReset  dijitArrowButtonContainer \">\r\n\t\t\t\t\t\t<button title=\"${resources_.CLEAR_FILTER}\" class=\"dijitReset quickFilterClearInner\" type=\"button\" tabindex=\"0\" \r\n\t\t\t\t\t\t\tvalign=\"middle\"  dojoAttachPoint='clearFilterNode' dojoAttachEvent='onclick:_onCancel,onkeypress:_onCancel' role=\"button\"></button>\t\t\t\r\n\t\t\t\t\t</div>\r\n\t\t\t\t</td>\r\n\t\t\t</tr>\r\n\t\t</tbody>\t\t\r\n\t</table>\r\n</div>"), 
		
		// Set this to true if your widget contains other widgets
		widgetsInTemplate : false, 
		grid : null, 
		handle : null, 
		filterText : null, 
		filterLayer : null, 
		filterOptions : null, 
		_quickFilterButton: false, //change filter on type based on this	
		_quickFilterDelay: 700,	//delay in ms before quick filter is applied as typed		
		
		// Override this method to perform custom behavior during dijit construction.
		// Common operations for constructor:
		// 1) Initialize non-primitive types (i.e. objects and arrays)
		// 2) Add additional properties needed by succeeding lifecycle methods
		constructor : function () {
			this.resources_ = dojo.i18n.getLocalization("dojoe.treetable", "TreeTable", this.lang);
		}, 
		
		postCreate : function () {
			if(this.filterOptions){
				if(this.filterOptions.showQuickFilterButton !== undefined) this._quickFilterButton = this.filterOptions.showQuickFilterButton;
				if(this.filterOptions.quickFilterDelay) this._quickFilterDelay = this.filterOptions.quickFilterDelay;
			}
			/*LS - 2/08/2011 - 25076 - Treetable - when clicking clear filter 'x' button few times, data is not re-fetched (server-side store) */
			//disabling clear filter icon when there is no filter applied to avoid above confusion	
			if(this.filterText !== null && this.filterText !== "" )	{
				dojo.style(this.clearFilterNode, "display", "block");
			}else{
				dojo.style(this.clearFilterNode, "display", "none");
			}		
		}, 		
		
		// When this method is called, all variables inherited from superclasses are 'mixed in'.
		// Common operations for postMixInProperties
		// 1) Modify or assign values for widget property variables defined in the template HTML file
		postMixInProperties : function () {
		}, 
		
		_parseDecimal : function (v) {
			return parseInt(v, 10);
		}, 
		
		_onFocus : function () {
			//LS- 26/10/2011 - # 30377 - Treetable - incorrect results when setting filter or sorting while data is being loaded
			//return if grid._loading a variable indicating the grid loading is true
			if(this.grid._loading) return;
			dojo.addClass(this.domNode, 'dijitTextBoxFocused');
			if (this.searchField.value == this.resources_.QF_DEFAULT_TEXT) {
				//Richa- Color settings should be there in css files
				//this.searchField.style.color = "#000000";
				this.searchField.value = "";
			}
		}, 
		
		//Latha : 10Apr2011 Fix for defect 16795. Setting Focus to filter box
		_onClick : function () {
			//LS- 26/10/2011 - # 30377 - Treetable - incorrect results when setting filter or sorting while data is being loaded
			//return if grid._loading a variable indicating the grid loading is true
			if(this.grid._loading) return;
			dojo.addClass(this.domNode, 'dijitTextBoxFocused');
			this.searchField.focus();
			//Richa- Color settings should be there in css files
			//this.searchField.style.color = "#000000";
			if (this.searchField.value == this.resources_.QF_DEFAULT_TEXT) {
				this.searchField.value = "";
			}
		},
		
		_onBlur : function () {
			dojo.removeClass(this.domNode, 'dijitTextBoxFocused');
			dojo.removeClass(this.domNode, 'dijitTextBoxHover');
			if (!this.searchField.value) {
				this.searchField.style.color = "#999999";
				this.searchField.value = this.resources_.QF_DEFAULT_TEXT;
			}
		}, 
		
		_onHover : function () {
			//LS- 26/10/2011 - # 30377 - Treetable - incorrect results when setting filter or sorting while data is being loaded
			//return if grid._loading a variable indicating the grid loading is true
			if(this.grid._loading) return;
			dojo.addClass(this.domNode, 'dijitTextBoxHover');
		}, 
		
		_onUnHover : function () {
			dojo.removeClass(this.domNode, 'dijitTextBoxHover');
		}, 
		
		_onKeyUp : function (event) {	
			//LS- 26/10/2011 - # 30377 - Treetable - incorrect results when setting filter or sorting while data is being loaded
			//return if grid._loading a variable indicating the grid loading is true
			if(this.grid._loading) return;
			//Bobby: 25Feb2011, don't do filter when tabbed thru
			//tab & shift keys		
			//console.info("key -"+ event.keyCode);	
			this.filterText = this.searchField.value;
			if(event){
				if (event.keyCode == 9 || event.keyCode == 16) {
					return;
				}else if(this._quickFilterButton && event.keyCode !== 13){
					return;
				}			
				//console.log("filter ---"+this.filterText);
				if (event.keyCode == 13) {
					dojo.stopEvent(event);
					this._setFilter();
					return;
				}
			}
			this.onChangeFilterText();
		}, 
		
		_onCancel : function (event) {			
			//Bobby: 17Apr2011, don't do anything when tabbed thru
			//tab & shift keys	
			if(event.keyCode == 9 || event.keyCode == 16) {
				//Bobby:17Apr2011, tab gets stuck if stopped
				//dojo.stopEvent(event);
				return;
			}else{
				this._clearFilter();			
			}			
		}, 
		
		wrapStore : function () {
			this._store = this.grid.store;
			this._doFilter();
			//this.grid.store = this._store;
			
			this.connect(this.grid, "setStore", function (store) {
				if (store !== this._store) {
					//this.unWrapStore();
					this._doFilter();
					this._store = store;
				}
			});
		}, 
		
		_clearFilter : function () {
			this.searchField.value = "";		
			
			/*pass a string 'clear'  to identify clear filter condition*/
			//Bobby:25Mar2011, need to clear only if layer exists.
			//else error: this.grid.store.layer is not a function
			//Bobby:29Mar2011, filter should be cleared differently for client & server
			//15113: Treetable quickfilter cannot be disabled once entered
			//console.info("clearFilter ...... BEGIN");
			//console.info("filterOptions.isServerSide:"+this.filterOptions.isServerSide);
			this.grid.filterApplied = false;	
			if (this.filterOptions.isServerSide){
				//console.info("calling grid.setQuery.....");
				if (this.filterLayer) {
					this.filterLayer.filterDef(null);
					//#15380 fix to help CURI store to return individual rows when filtered
					//this.unWrapStore();
					/*#17606 - 19/07/2011 - TreeTable: quickfilter disabling results in error*/
					// retaining the old query and appending origins true
					if(this.grid.query){
						this.grid.setQuery(dojo.mixin(this.grid.query,{origins:true,"filterText" : null}));
					}else{
						// if there is no existing query, just fetch origins
						this.grid.setQuery({origins:true});
					}
					this.grid._cleanup(); 
					this.grid._refresh();
				}
			}else{
				if (this.filterLayer) {
					//console.info("clearing filterDef.....");
					this.filterLayer.filterDef('clear');
					//this.filterLayer.filterDef(expr);
					this.grid._cleanup(); 
					this.grid._refresh();
				}
			}	
			this.filterText=null;
			/*LS - 2/08/2011 - 25076 - Treetable - when clicking clear filter 'x' button few times, data is not re-fetched (server-side store) */
			//disabling clear filter icon when there is no filter applied to avoid above confusion		
			this._disableQuickFilter();
			//Connect to onFitered  through filter layers oncommandLoad or connect to fetch complete to enable the filter back with clear filter icon
			this.connect(this.filterLayer,"onFiltered", dojo.hitch(this,"_enableQuickFilter"));
			this.fetchEnableFilterHandler = this.connect(this.grid,"_onFetchComplete", dojo.hitch(this,"_enableQuickFilter"));
			//console.dir(this.grid);
			//console.info("clearFilter ...... END");			
		}, 
		//Satanik:31May2011: Added new api for setting and getting filter. Direct connect to wrapStore is no more required instead now _setFilter should be conected.
		_setFilter : function (filterText){
			if(filterText !== undefined && filterText !==''){
				this.filterText=filterText;
				this.searchField.value=filterText;
			}
			this.wrapStore();
		},
		
		_getFilter : function(){
			return this.filterText;
		},
		_doFilter : function () {
			//Bobby:18Aug2011, adding option to prevent clearing selection on Filter
			//27171: Need option to allow selection after filtering
			if(this.filterOptions && this.filterOptions.retainSelection !== true){
				//Bobby: 09Mar2011
				//13285: Allow for selection to be cleared when filter occurs				
				this.grid.selection.clear();
			}				
			
			var args = this.filterOptions;
			
			//TODO: Using the Enhanced Grid Plugin Filter , might need to copy the StoreLayer.js into treetable toavoid any dependencies.
			//var ns = dojox.grid.enhanced.plugins;
			//Uncomment the below variable to use the enhanced grid filter and comment the next fns
			//var fns = dojox.grid.enhanced.plugins.filter;			
			//Uncomment the below variable to use the customized filter for treegrid
			var fns = dojoe.treetable.filter;
			var QFinput = this.filterText;
			//Satanik:23Jun2011: adding a null check for the filtertext before filtering
			if (QFinput === null || QFinput === undefined) {
				return;
			}
			if (QFinput === "") {
				this._clearFilter();
				return;
			}
			var cells = this.grid.layout.cells;
			var expr = new fns.LogicANY(dojo.map(cells, function (cell) {
				return new fns.ContainsIgnoreCase(
					new fns.StringExpr(cell, true), 
					new fns.StringExpr(QFinput, false));
			}));
			
			if (!this.filterLayer) {
				if (args.isServerSide) {
					this.filterLayer = new fns.ServerSideFilterLayer(args);
					fns.wrap(this.grid.store, "fetch", this.filterLayer);
				} else {
					//Oliver comment : remove cache size
					this.filterLayer = new fns.ClientSideFilterLayer({
							fetchAll : true, 
							grid : this.grid, 
							getter : dojo.hitch(this.grid ? this.grid : this, function (/* data item */datarow, /* cell */cell, /* int */rowIndex) {
									// summary:
									//     Define the grid-specific way to get data from a row.
									//     Argument "cell" is provided by FilterDefDialog when defining filter expressions.
									//     Argument "rowIndex" is provided by FilterLayer when checking a row.
									//     FilterLayer also provides a forth argument: "store", which is inGrid.store,
									//     but we don't need it here.
									try {
										return cell.get(rowIndex, datarow);
									} catch (ex) {
										console.log(ex.message);
									}
								}) 
						});
					fns.wrap(this.grid.store, "fetch", this.filterLayer);
				}
			}
			if (args.isServerSide) {
				this.grid.setQuery({
					filterText : QFinput
				});
			}
			this.filterLayer.filterDef(expr);
			this.grid.filterApplied = true;
			/*LS - 2/08/2011 - 25076 - Treetable - when clicking clear filter 'x' button few times, data is not re-fetched (server-side store) */
			/*LS - 12/11/2011 - Disable QF on press of enter or filter button until the data loads in the grid, */
			this._disableQuickFilter();
			//Connect to onFitered  through filter layers oncommandLoad or connect to fetch complete to enable the filter back with clear filter icon
			this.connect(this.filterLayer,"onFiltered", dojo.hitch(this,"_enableQuickFilter"));
			this.fetchEnableFilterHandler = this.connect(this.grid,"_onFetchComplete", dojo.hitch(this,"_enableQuickFilter"));
			//Display the results
			//Satanik:2Jun2011:Fixing the QF for the ORIA changed in the driver of 1Jun2011
			this.grid._cleanup(); 
			this.grid._refresh(true);
			//this.grid.store.layer("filter").filterDef(null);			
		}, 
		
		_enableQuickFilter: function(fitleredSize, totalSize) {
			if(this.clearFilterNode && this.clearFilterNode.style.display == "none" &&
				(this.filterText && this.filterText !=="" && this.searchField.value != this.resources_.QF_DEFAULT_TEXT )){
					dojo.style(this.clearFilterNode, "display", "block");
				}
			if(this.searchField && this.searchField.disabled == true){
				this.searchField.disabled = false;
				if(this.filterText && this.filterText !=="" && this.searchField.value != this.resources_.QF_DEFAULT_TEXT ){
					this.searchField.focus();
					//Richa- Color settings should be there in css files
					//this.searchField.style.color = "#000000";
				}else if (!this.searchField.value || this.searchField.value == this.resources_.QF_DEFAULT_TEXT ){
					this.searchField.style.color = "#999999";
					this.searchField.value = this.resources_.QF_DEFAULT_TEXT;
				}
			}
			this.disconnect(this.fetchEnableFilterHandler);
			
		},
		
		_disableQuickFilter: function() {
			if(this.clearFilterNode && this.clearFilterNode.style.display == "block"){
				dojo.style(this.clearFilterNode, "display", "none");
			}
			if(this.searchField && this.searchField.disabled == false){
				this.searchField.disabled = true;
				this._onBlur();
				this.searchField.style.color = "#999999";
			}
		},
		
		unWrapStore : function () {
			//console.log("UNWRAPPING STORE");
			this._store = this.grid.store;
			try {
				this._store.unwrap("filter");
			} catch (ex) {
				console.log("no layers wrapped");
			}
		}, 
		
		onChangeFilterText : function () {
			if (this.handle) {
				clearTimeout(this.handle);
				this.handle = null;
			}
			var fnc = dojo.hitch(this, this._setFilter, this.filterText);
			this.handle = setTimeout(fnc, this._quickFilterDelay);
		}, 
		
		destroy: function(){
			// summary:  Destroys the toolbar.
			// tags: public	
			//memory leak issue: custom destroy method to take care of removing all widgets					
			try{
				delete this.resources_;
				//this is for destroying child templated widgets 
				//this should happen by itself but need to figure out why dijit._Templated is not taking care of it...
				var widgets = dijit.findWidgets(this.domNode);
				dojo.forEach(widgets, function(w) {
					if(w){
						if(w.destroyRecursive){
							w.destroyRecursive();
						} else if(w.destroy){
							w.destroy();
						}
					}
				});
				// this is for widgets added as child to this widget and their children..
				//not present in template
				var widgetArr = this.getChildren();
				dojo.forEach(widgetArr, function (child) {
					if(child){
						if(child.destroyRecursive){
							child.destroyRecursive();
						} else if(child.destroy){
							child.destroy();
						}
					}
				});
				this.inherited(arguments);
			}catch(ex){
				console.warn("Widget destroy failed: "+this.id+", ex="+ex.name+":"+ex.message);
			}
		}		
	});
 
});

},
'ibm/tivoli/tip/dijit/CMSMenuProviderV2':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!ibm/tivoli/tip/dijit/CMSMenuProvider"], function(dijit,dojo,dojox){
/**
 *  The CMSMenuProviderV2 is an advanced version of the CMSMenuProvider.
 *  This provider allows for resource context fitlers for multiple resources.
 *  It is recommended that you use this version of the CMS menu provider unless you
 *  have some very specific needs.
 *
 * {
 *  rcfs: 
 *  [
 *     [ // per resource
 *       { name : "SourceToken", value : "1.1", isNamingAttribute : false},
 *       { name : "Signature", value : "192.168.1.1", isNamingAttribute : true}
 *     ],
 *     [
 *       { name : "SourceToken", value : "1.2"},
 *       { name : "Signature", value : "192.168.1.2", isNamingAttribute : true }
 *     ]
 *  ],
 *  launchTypes: ["WEB_URL", "PORTAL_PAGE"],
 *  excludedAppIDs: [ { name: "app1", indicator: "isLaunchingProductAndExcludeMyMenus" }, { name: "app2"} ],
 *  substitutionVariables: [ {name: "foo", value: "bar"} ],
 *  options: [ {name: "option", value: "optval"} ]
 * } 
 */
dojo.require("ibm.tivoli.tip.dijit.CMSMenuProvider"); 
dojo.provide("ibm.tivoli.tip.dijit.CMSMenuProviderV2");
 
dojo.declare("ibm.tivoli.tip.dijit.CMSMenuProviderV2", 
             ibm.tivoli.tip.dijit.CMSMenuProvider, 
{
   // This menu uses the same getCMSMenus command, but on the server, we use a different
   // data provider for this version of the CMS menu provider.
   getMenusCommandName: "getCMSMenus",

   getMenuActionCommandName: "getCMSAction",

   // Note that this method should not be called from an outside program.  Here we override the base
   // implmementation of this just in case to help with debugging.
   getRCFURL: function (kw) 
   {
      console.error("Not valid for CMSMenuProviderV2 since we use different context.");
   },

   getMenuItems: function(context) 
   {
      var kw = null;

      kw = {
              sync:         true,
              noBusyDialog: true,
              content:      {
                               command:              this.getMenusCommandName,
                               arg_idPrefix:         this.getIdPrefix(),
                               arg_portletNamespace: this.global.namespace,
                               arg_parentId:         this.parentid,
                               arg_context: dojo.toJson(context)
                            },
              timeout:      this.requestTimeout,
              onResponse:   {
                               callback: dojo.hitch( this, function( response ) { this.menus_ = response; } )
                            }
            };

      if (this.menuName) 
      {
         kw.content.arg_menuName = this.menuName;
      }
      
      ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).post( kw );

      return this.menus_;
   },
   
   getMenuCommandFromServer: function (commandName, context, menuItem) 
   {
      var kw = {
                  content:   {
                                command: this.getMenuActionCommandName,
                                arg_menuCommandName: commandName,
                                arg_idPrefix: this.getIdPrefix(),
                                arg_portletNamespace: this.global.namespace,
                                arg_parentId: this.parentid,
                                arg_context: dojo.toJson(context)
                             },
                  timeout:   this.requestTimeout,
                  onResponse:{
                                callback: dojo.hitch(this, this.handleMenuAction)
                             }
               };
               
       if (this.menuName) 
       {
          kw.content.arg_menuName = this.menuName;
       }
      
       ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).post( kw );
   }

});
});

},
'ibm/tivoli/tip/navwidget/dojo/NavWidgetPluggableViewObjectContext':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!ibm/tivoli/tip/navwidget/dojo/NavWidgetUtil,dojox/collections/SortedList"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.dojo.NavWidgetPluggableViewObjectContext");

dojo.require("ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil");
dojo.require("dojox.collections.SortedList");

dojo.declare(
    "ibm.tivoli.tip.navwidget.dojo.NavWidgetPluggableViewObjectContext",
    null,
    {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
    	
    	// _items: [private] SortedList
    	//		The list of items that are selected
  		_items: null,
    	
  		// _links: [private] SortedList
    	//		The list of links that are selected
  		_links: null,
  		
  		// _relationships: [private] SortedList
    	//		The list of links that are selected
  		_relationships: null,
    	
    	// _pluggableView: [private] ibm.tivoli.tip.navwidget.dijit.NavWidgetPluggableView
    	//		The pluggable view for which we're managing selection
  		_pluggableView: null,
  		
    	// dataset: [private] ibm.tivoli.navmodel.Dataset
    	//		The root level dataset for the selection we're managing
  		_dataset: null,
  		
		/*=====
		constructor_kwArgs: function(dataset, pluggableView, itemIds, relationshipIds) {
			// dataset: ibm.tivoli.navmodel.Dataset
			//		
			// pluggableView: ibm.tivoli.tip.navwidget.dijit.NavWidgetPluggableView
			//		
			// itemIds: String[]
			//		
			// relationshipIds: String[]
			//		
		},
		=====*/
  		
  		constructor: function(/*constructor_kwArgs*/ kwArgs) {		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
    	
    		//The sorted list can suffer from performance issues if there are a lot of insertions, but in a lot of cases we want to keep the list of selected items
    		//sorted (e.g., so properties view could show last selected item). Since the selection will usually be small, I'm guessing this will be acceptable.
    		this._items = new dojox.collections.SortedList();
    		this._links = new dojox.collections.SortedList();
    		this._relationships = new dojox.collections.SortedList();
    		
    		//Process the args
    		if (kwArgs) {
    			this._dataset = kwArgs.dataset;
    			this._pluggableView = kwArgs.pluggableView;
    			this._setSelectedObjectIds(kwArgs.itemIds, kwArgs.linkIds, kwArgs.relationshipIds);
    		}
	 	},
	 	
	 	_setSelectedObjectIds: function(/*String[]*/ itemIds, /*String[]*/ linkIds, /*String[]*/ relationshipIds){
			// summary:
			//		Coming soon
			// itemsIds:
			//		An array of absolute item ids
			// linkIds:
			//		An array of absolute link ids
	 		// relationshipIds:
			//		An array of absolute relationship ids
			// description:
			//		Coming soon
			// tags:
			//		private

	 		//Clear out current selection
	 		this._items.clear();
	 		this._links.clear();
	 		this._relationships.clear();
	 		
	 		//Look up the objects in the dataset and add to hash tables
	 		//Items
	 		if (itemIds) {
		 		for (var i = 0; i < itemIds.length; i++) {
	 				var absItemId = itemIds[i];
	 				var item = ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getItemByAbsoluteId(this._dataset, absItemId);
	 				this._items.add(absItemId, item);
	 			}
	 		}
	 		
	 		//Links
	 		if (linkIds) {
		 		for (var i = 0; i < linkIds.length; i++) {
	 				var absLinkId = linkIds[i];
	 				var link = ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getLinkByAbsoluteId(this._dataset, absLinkId);
	 				this._links.add(absLinkId, link);
	 			}
	 		}
	 		
	 		//Relationships
	 		if (relationshipIds) {
		 		for (var i = 0; i < relationshipIds.length; i++) {
	 				var absRelationshipId = relationshipIds[i];
	 				var relationship = ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getRelationshipByAbsoluteId(this._dataset, absRelationshipId);
	 				this._relationships.add(absRelationshipId, relationship);
	 			}
	 		}
		},
		
		_setSelectedObjectsBase: function(/*ibm.tivoli.navmodel.Item[]*/ items, /*ibm.tivoli.navmodel.Link[]*/ links, /*ibm.tivoli.navmodel.Relationship[]*/ relationships){
			// summary:
			//		Coming soon
			// items:
			//		An array of items
			// links:
			//		An array of links
			// relationships:
			//		An array of relationships
			// description:
			//		Coming soon
			// tags:
			//		private
		
			//Clear out current selection
	 		this._items.clear();
	 		this._links.clear();
	 		this._relationships.clear();
	 		
	 		//Look up the objects in the dataset and add to hash tables
	 		//Items
	 		if (items) {
		 		for (var i = 0; i < items.length; i++) {
		 			var item = items[i];
		 			var absId = ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getItemAbsoluteId(item);
	 				this._items.add(absId, item);
	 			}
	 		}
	 		
	 		//Links
	 		if (links) {
		 		for (var i = 0; i < links.length; i++) {
		 			var link = links[i];
		 			var absId = ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getLinkAbsoluteId(link);
	 				this._links.add(absId, link);
	 			}
	 		}
	 		
	 		//Relationships
	 		if (relationships) {
		 		for (var i = 0; i < relationships.length; i++) {
		 			var relationship = relationships[i];
		 			var absId = ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getRelationshipAbsoluteId(relationship);
	 				this._relationships.add(absId, relationship);
	 			}
	 		}
		},
		
	 	_deselectObjectIds: function(/*String[]*/ itemIds, /*String[]*/ linkIds, /*String[]*/ relationshipIds){
			// summary:
			//		Coming soon
			// itemsIds:
			//		An array of absolute item ids
			// linkIds:
			//		An array of absolute link ids
	 		// relationshipIds:
			//		An array of absolute relationship ids
			// description:
			//		Coming soon
			// tags:
			//		private

	 		//Look up the objects in the dataset and remove from hash tables
	 		//Items
	 		if (itemIds) {
		 		for (var i = 0; i < itemIds.length; i++) {
	 				var absItemId = itemIds[i];	 
					if (this._items.contains(absItemId)) {
		 				this._items.remove(absItemId);
					}
	 			}
	 		}
	 		
	 		//Links
	 		if (linkIds) {
		 		for (var i = 0; i < linkIds.length; i++) {
	 				var absLinkId = linkIds[i];
					if (this._links.contains(absLinkId)) {
		 				this._links.remove(absLinkId);
					}
	 			}
	 		}
	 		
	 		//Relationships
	 		if (relationshipIds) {
		 		for (var i = 0; i < relationshipIds.length; i++) {
					var absRelationshipId = relationshipIds[i];
					if (this._relationships.contains(absRelationshipId)) {
		 				this._relationships.remove(absRelationshipId);
					}
	 			}
	 		} 
		},

		_deselectObjects: function(/*ibm.tivoli.navmodel.Item[]*/ items, /*ibm.tivoli.navmodel.Link[]*/ links, /*ibm.tivoli.navmodel.Relationship[]*/ relationships) {
			// summary:
			//		Coming soon
			// items:
			//		An array of items
			// links:
			//		An array of links
			// relationships:
			//		An array of relationships
			// description:
			//		Coming soon
			// tags:
			//		private

	 		//Look up the objects in the dataset and remove from hash tables
	 		//Items
	 		if (items) {
		 		for (var i = 0; i < items.length; i++) {
		 			var item = items[i];
		 			var absId = ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getItemAbsoluteId(item);
					if (this._items.contains(absId)) {
		 				this._items.remove(absId);
					}
	 			}
	 		}
	 		
	 		//Links
	 		if (links) {
		 		for (var i = 0; i < links.length; i++) {
	 				var link = links[i];
		 			var absId = ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getLinkAbsoluteId(link);
					if (this._links.contains(absId)) {
		 				this._links.remove(absId);
					}
	 			}
	 		}
	 		
	 		//Relationships
	 		if (relationships) {
		 		for (var i = 0; i < relationshipIds.length; i++) {
					var relationship = relationships[i];
		 			var absId = ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getRelationshipAbsoluteId(relationship);
					if (this._relationships.contains(absId)) {
		 				this._relationships.remove(absId);
					}

	 		
	 			}
	 		} 
		},

		//Dataset
    	getDataset: function() {		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
	 		return this._dataset;
	 	},
	 	
		//Items
    	getItems: function() {		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
	 		return this._items.getValueList();
	 	},
	 	
	 	getItemIds: function() {		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
	 		return this._items.getKeyList();
	 	},
	 	
	 	getItemCount: function() {		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
	 		return this._items.count;
	 	},
	 	
	 	//Links
	 	getLinks: function() {		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
	 		return this._links.getValueList();
	 	},
	 	
	 	getLinkIds: function() {		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
	 		return this._links.getKeyList();
	 	},
	 	
	 	getLinkCount: function() {		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
	 		return this._links.count;
	 	},
	 	
	 	//Relationships
	 	getRelationships: function() {		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
	 		return this._relationships.getValueList();
	 	},
	 	
	 	getRelationshipIds: function() {		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
	 		return this._relationships.getKeyList();
	 	},
	 	
	 	getRelationshipCount: function() {		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
	 		return this._relationships.count;
	 	},
	 	
	 	//Misc
	 	getTotalSelected: function() {		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
	 		return this.getItemCount() + this.getLinkCount() + this.getRelationshipCount();
	 	},
	 	
	 	isEmpty: function() {		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
	 		return this.getTotalSelected() == 0;
	 	},
	 	
	 	getPluggableView: function() {		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
	 		return this._pluggableView;
	 	}
    }
);
});

},
'gridx/core/model/cache/_Cache':function(){
define([
	'dojo/_base/declare',
	'dojo/_base/array',
	'dojo/_base/lang',
	'dojo/_base/Deferred',
	'../_Extension'
], function(declare, array, lang, Deferred, _Extension){

	var hitch = lang.hitch,
		mixin = lang.mixin,
		indexOf = array.indexOf;

	return declare(_Extension, {
		

		// summary:
		//		Abstract base cache class, providing cache data structure and some common cache functions.
		constructor: function(model, args){
			var t = this;
			t.setStore(args.store);
			t.columns = args.columns;
			t._mixinAPI('byIndex', 'byId', 'indexToId', 'idToIndex', 'size', 'treePath', 'parentId',
				'hasChildren', 'children', 'keep', 'free');
		},

		destroy: function(){
			this.inherited(arguments);
			this.clear();
		},

		setStore: function(store){
			var t = this,
				c = 'aspect',
				old = store.fetch;
			t.clear();
			t.store = store;
			if(!old && store.notify){
				//The store implements the dojo.store.Observable API
				t[c](store, 'notify', function(item, id){
					if(item === undefined){
						t._onDelete(id);
					}else if(id === undefined){
						t._onNew(item);
					}else{
						t._onSet(item);
					}
				});
			}else{
				t[c](store, old ? "onSet" : "put", "_onSet");
				t[c](store, old ? "onNew" : "add", "_onNew");
				t[c](store, old ? "onDelete" : "remove", "_onDelete");
			}
		},
		
		//Public----------------------------------------------
		clear: function(){
			var t = this;
			t._filled = 0;
			t._priority = [];
			t._struct = {};
			t._cache = {};
			t._size = {};
			//virtual root node, with id ''.
			t._struct[''] = [];
			t._size[''] = -1;
		},
		
		
		byIndex: function(index, parentId){
			this._init('byIndex', arguments);
			return this._cache[this.indexToId(index, parentId)];
		},

		
		byId: function(id){
			this._init('byId', arguments);
			return this._cache[id];
		},

		
		indexToId: function(index, parentId){
			this._init('indexToId', arguments);
			var items = this._struct[this.model.isId(parentId) ? parentId : ''];
			return typeof index == 'number' && index >= 0 ? items && items[index + 1] : undefined;
		},

		
		idToIndex: function(id){
			this._init('idToIndex', arguments);
			var s = this._struct,
				pid = s[id] && s[id][0],
				index = indexOf(s[pid] || [], id);
			return index > 0 ? index - 1 : -1;
		},

		
		treePath: function(id){
			this._init('treePath', arguments);
			var s = this._struct,
				path = [];
			while(id !== undefined){
				path.unshift(id);
				id = s[id] && s[id][0];
			}
			if(path[0] !== ''){
				path = [];
			}else{
				path.pop();
			}
			return path;
		},

		
		parentId: function(id){
			return this.treePath(id).pop();
		},

		
		hasChildren: function(id){
			var t = this,
				s = t.store,
				c;
			t._init('hasChildren', arguments);
			c = t.byId(id);
			return s.hasChildren && s.hasChildren(id, c && c.item);
		},
		
		
		children: function(parentId){
			this._init('children', arguments);
			parentId = this.model.isId(parentId) ? parentId : '';
			var size = this._size[parentId],
				children = [];
			for(var i = 0; i < size; ++i){
				children.push(this.indexToId(i, parentId));
			}
			return children;
		},

		
		size: function(parentId){
			this._init('size', arguments);
			var s = this._size[this.model.isId(parentId) ? parentId : ''];
			return s >= 0 ? s : -1;
		},

		//Events--------------------------------------------
		onBeforeFetch: function(){},
		onAfterFetch: function(){},
		onLoadRow: function(){},

		onSetColumns: function(columns){
			var t = this, id, c, colId, col;
			t.columns = columns;
			for(id in t._cache){
				c = t._cache[id];
				for(colId in columns){
					col = columns[colId];
					c.data[colId] = t._formatCell(col.id, c.rawData);
				}
			}
		},

		//Protected-----------------------------------------
		_itemToObject: function(item){
			var s = this.store;
			if(s.fetch){
				var obj = {};
				array.forEach(s.getAttributes(item), function(attr){
					obj[attr] = s.getValue(item, attr);
				});
				return obj;	
			}
			return item;
		},

		_formatCell: function(colId, rawData){
			var col = this.columns[colId];
			return col.formatter ? col.formatter(rawData) : rawData[col.field || colId];
		},

		_formatRow: function(rowData){
			var cols = this.columns, res = {}, colId;
			for(colId in cols){
				res[colId] = this._formatCell(colId, rowData);
			}
			return res;
		},

		_addRow:function(id, index, rowData, item, parentId){
			var t = this,
				st = t._struct,
				pr = t._priority,
				pid = t.model.isId(parentId) ? parentId : '',
				ids = st[pid],
				i;
			if(!ids){
				throw new Error("Fatal error of cache._addRow: parent item " + pid + " of " + id + " is not loaded");
			}
			if(!ids[index + 1]){
				ids[index + 1] = id;
			}else if(ids[index + 1] !== id){
				throw new Error("Fatal error of cache._addRow: different row id " + id + " and " + ids[index + 1] + " for same row index " + index);
			}
			st[id] = st[id] || [pid];
			if(pid === ''){
				i = indexOf(pr, id);
				if(i >= 0){
					pr.splice(i, 1);
				}
				pr.push(id);
			}
			t._cache[id] = {
				data: t._formatRow(rowData),
				rawData: rowData,
				item: item
			};
			t.onLoadRow(id);
		},

		_loadChildren: function(parentId){
			var t = this,
				d = new Deferred(),
				s = t.store,
				row = t.byId(parentId),
				items = row && s.getChildren && s.getChildren(row.item) || [];
			Deferred.when(items, function(items){
				var i = 0,
					item,
					len = t._size[parentId] = items.length;
				for(; i < len; ++i){
					item = items[i];
					t._addRow(s.getIdentity(item), i, t._itemToObject(item), item, parentId);
				}
				d.callback();
			}, hitch(d, d.errback));
			return d;
		},

		_onBegin: function(size){
			this._size[''] = parseInt(size, 10);
		},

		_onComplete: function(d, start, items){
			try{
				var t = this, i = 0, item;
				for(; item = items[i]; ++i){
					t._addRow(t.store.getIdentity(item), start + i, t._itemToObject(item), item);
				}
				d.callback();
			}catch(e){
				d.errback(e);
			}
		},
	
		_storeFetch: function(options, onFetched){
//            console.debug("\tFETCH start: ",
//                    options.start, ", count: ",
//                    options.count, ", end: ",
//                    options.count && options.start + options.count - 1, ", options:",
//                    this.options);

			var t = this,
				s = t.store,
				d = new Deferred(),
				req = mixin({}, t.options || {}, options),
				onBegin = hitch(t, t._onBegin),
				onComplete = hitch(t, t._onComplete, d, options.start),
				onError = hitch(d, d.errback);
			t._filled = 1;	//1 as true;
			t.onBeforeFetch();
			if(s.fetch){
				s.fetch(mixin(req, {
					onBegin: onBegin,
					onComplete: onComplete,
					onError: onError
				}));
			}else{
				var results = s.query(req.query, req);
				Deferred.when(results.total, onBegin);
				Deferred.when(results, onComplete, onError);
			}
			d.then(hitch(t, t.onAfterFetch));
			return d;
		},

		//--------------------------------------------------------------------------
		_onSet: function(item){
			var t = this,
				id = t.store.getIdentity(item),
				index = t.idToIndex(id),
				path = t.treePath(id),
				old = t._cache[id];
			if(path.length){
				t._addRow(id, index, t._itemToObject(item), item, path.pop());
			}
			t.onSet(id, index, t._cache[id], old);
		},
	
		_onNew: function(item, parentInfo){
			var t = this, s = t.store,
				row = t._itemToObject(item),
				parentItem = parentInfo && parentInfo[s.fetch ? 'item' : 'parent'],
				parentId = parentItem ? s.getIdentity(parentItem) : '',
				size = t._size[''];
			t.clear();
			t.onNew(s.getIdentity(item), 0, {
				data: t._formatRow(row),
				rawData: row,
				item: item
			});
			if(!parentItem && size >= 0){
				t._size[''] = size + 1;
				t.model._onSizeChange();
			}
		},
	
		_onDelete: function(item){
			var t = this, s = t.store, st = t._struct,
				id = s.fetch ? s.getIdentity(item) : item, 
				path = t.treePath(id);
			if(path.length){
				var children, i, j, ids = [id],
					parentId = path.pop(),
					sz = t._size,
					size = sz[''],
					index = indexOf(st[parentId], id);
				//This must exist, because we've already have treePath
				st[parentId].splice(index, 1);
				--sz[parentId];
	
				for(i = 0; i < ids.length; ++i){
					children = st[ids[i]];
					if(children){
						for(j = children.length - 1; j > 0; --j){
							ids.push(children[j]);
						}
					}
				}
				for(i = ids.length - 1; i >= 0; --i){
					j = ids[i];
					delete t._cache[j];
					delete st[j];
					delete sz[j];
				}
				i = indexOf(t._priority, id);
				if(i >= 0){
					t._priority.splice(i, 1);
				}
				t.onDelete(id, index - 1);
				if(!parentId && size >= 0){
					sz[''] = size - 1;
					t.model._onSizeChange();
				}
			}else{
				t.onDelete(id);
//                var onBegin = hitch(t, t._onBegin),
//                    req = mixin({}, t.options || {}, {
//                        start: 0,
//                        count: 1
//                    });
//                setTimeout(function(){
//                    if(s.fetch){
//                        s.fetch(mixin(req, {
//                            onBegin: onBegin
//                        }));
//                    }else{
//                        var results = s.query(req.query, req);
//                        Deferred.when(results.total, onBegin);
//                    }
//                }, 10);
			}
		}
	});
});

},
'dojox/mvc/Bind':function(){
define([
	"dojo/_base/lang",
	"dojo/_base/array"
], function(lang, array){
	var mvc = lang.getObject("dojox.mvc", true);
	/*=====
		mvc = dojox.mvc;
	=====*/

	return lang.mixin(mvc, {
		bind: function(/*dojo.Stateful*/ source, /*String*/ sourceProp,
					/*dojo.Stateful*/ target, /*String*/ targetProp,
					/*Function?*/ func, /*Boolean?*/ bindOnlyIfUnequal){
			// summary:
			//		Bind the specified property of the target to the specified
			//		property of the source with the supplied transformation.
			//	source:
			//		The source dojo.Stateful object for the bind.
			//	sourceProp:
			//		The name of the source's property whose change triggers the bind.
			//	target:
			//		The target dojo.Stateful object for the bind whose
			//		property will be updated with the result of the function.
			//	targetProp:
			//		The name of the target's property to be updated with the
			//		result of the function.
			//	func:
			//		The optional calculation to be performed to obtain the target
			//		property value.
			//	bindOnlyIfUnequal:
			//		Whether the bind notification should happen only if the old and
			//		new values are unequal (optional, defaults to false).
			var convertedValue;
			return source.watch(sourceProp, function(prop, oldValue, newValue){
				convertedValue = lang.isFunction(func) ? func(newValue) : newValue;
				if(!bindOnlyIfUnequal || convertedValue != target.get(targetProp)){
					target.set(targetProp, convertedValue);
				}
			});
		},

		bindInputs: function(/*dojo.Stateful[]*/ sourceBindArray, /*Function*/ func){
			// summary:
			//		Bind the values at the sources specified in the first argument
			//		array such that a composing function in the second argument is
			//		called when any of the values changes.
			//	sourceBindArray:
			//		The array of dojo.Stateful objects to watch values changes on.
			//	func:
			//		The composing function that is called when any of the source
			//		values changes.
			// tags:
			//		protected
			var watchHandles = [];
			array.forEach(sourceBindArray, function(h){
				watchHandles.push(h.watch("value", func));
			});
			return watchHandles;
		}
	});
});

},
'dijit/form/_ButtonMixin':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.setSelectable
	"dojo/_base/event", // event.stop
	"../registry"		// registry.byNode
], function(declare, dom, event, registry){

// module:
//		dijit/form/_ButtonMixin
// summary:
//		A mixin to add a thin standard API wrapper to a normal HTML button

return declare("dijit.form._ButtonMixin", null, {
	// summary:
	//		A mixin to add a thin standard API wrapper to a normal HTML button
	// description:
	//		A label should always be specified (through innerHTML) or the label attribute.
	//		Attach points:
	//			focusNode (required): this node receives focus
	//			valueNode (optional): this node's value gets submitted with FORM elements
	//			containerNode (optional): this node gets the innerHTML assignment for label
	// example:
	// |	<button data-dojo-type="dijit.form.Button" onClick="...">Hello world</button>
	//
	// example:
	// |	var button1 = new dijit.form.Button({label: "hello world", onClick: foo});
	// |	dojo.body().appendChild(button1.domNode);

	// label: HTML String
	//		Content to display in button.
	label: "",

	// type: [const] String
	//		Type of button (submit, reset, button, checkbox, radio)
	type: "button",

	_onClick: function(/*Event*/ e){
		// summary:
		//		Internal function to handle click actions
		if(this.disabled){
			event.stop(e);
			return false;
		}
		var preventDefault = this.onClick(e) === false; // user click actions
		if(!preventDefault && this.type == "submit" && !(this.valueNode||this.focusNode).form){ // see if a non-form widget needs to be signalled
			for(var node=this.domNode; node.parentNode; node=node.parentNode){
				var widget=registry.byNode(node);
				if(widget && typeof widget._onSubmit == "function"){
					widget._onSubmit(e);
					preventDefault = true;
					break;
				}
			}
		}
		if(preventDefault){
			e.preventDefault();
		}
		return !preventDefault;
	},

	postCreate: function(){
		this.inherited(arguments);
		dom.setSelectable(this.focusNode, false);
	},

	onClick: function(/*Event*/ /*===== e =====*/){
		// summary:
		//		Callback for when button is clicked.
		//		If type="submit", return true to perform submit, or false to cancel it.
		// type:
		//		callback
		return true;		// Boolean
	},

	_setLabelAttr: function(/*String*/ content){
		// summary:
		//		Hook for set('label', ...) to work.
		// description:
		//		Set the label (text) of the button; takes an HTML string.
		this._set("label", content);
		(this.containerNode||this.focusNode).innerHTML = content;
	}
});

});

},
'url:dijit/form/templates/Button.html':"<span class=\"dijit dijitReset dijitInline\" role=\"presentation\"\n\t><span class=\"dijitReset dijitInline dijitButtonNode\"\n\t\tdata-dojo-attach-event=\"ondijitclick:_onClick\" role=\"presentation\"\n\t\t><span class=\"dijitReset dijitStretch dijitButtonContents\"\n\t\t\tdata-dojo-attach-point=\"titleNode,focusNode\"\n\t\t\trole=\"button\" aria-labelledby=\"${id}_label\"\n\t\t\t><span class=\"dijitReset dijitInline dijitIcon\" data-dojo-attach-point=\"iconNode\"></span\n\t\t\t><span class=\"dijitReset dijitToggleButtonIconChar\">&#x25CF;</span\n\t\t\t><span class=\"dijitReset dijitInline dijitButtonText\"\n\t\t\t\tid=\"${id}_label\"\n\t\t\t\tdata-dojo-attach-point=\"containerNode\"\n\t\t\t></span\n\t\t></span\n\t></span\n\t><input ${!nameAttrSetting} type=\"${type}\" value=\"${value}\" class=\"dijitOffScreen\"\n\t\ttabIndex=\"-1\" role=\"presentation\" data-dojo-attach-point=\"valueNode\"\n/></span>\n",
'dijit/focus':function(){
define([
	"dojo/aspect",
	"dojo/_base/declare", // declare
	"dojo/dom", // domAttr.get dom.isDescendant
	"dojo/dom-attr", // domAttr.get dom.isDescendant
	"dojo/dom-construct", // connect to domConstruct.empty, domConstruct.destroy
	"dojo/Evented",
	"dojo/_base/lang", // lang.hitch
	"dojo/on",
	"dojo/ready",
	"dojo/_base/sniff", // has("ie")
	"dojo/Stateful",
	"dojo/_base/unload", // unload.addOnWindowUnload
	"dojo/_base/window", // win.body
	"dojo/window", // winUtils.get
	"./a11y",	// a11y.isTabNavigable
	"./registry",	// registry.byId
	"."		// to set dijit.focus
], function(aspect, declare, dom, domAttr, domConstruct, Evented, lang, on, ready, has, Stateful, unload, win, winUtils,
			a11y, registry, dijit){

	// module:
	//		dijit/focus
	// summary:
	//		Returns a singleton that tracks the currently focused node, and which widgets are currently "active".

/*=====
	dijit.focus = {
		// summary:
		//		Tracks the currently focused node, and which widgets are currently "active".
		//		Access via require(["dijit/focus"], function(focus){ ... }).
		//
		//		A widget is considered active if it or a descendant widget has focus,
		//		or if a non-focusable node of this widget or a descendant was recently clicked.
		//
		//		Call focus.watch("curNode", callback) to track the current focused DOMNode,
		//		or focus.watch("activeStack", callback) to track the currently focused stack of widgets.
		//
		//		Call focus.on("widget-blur", func) or focus.on("widget-focus", ...) to monitor when
		//		when widgets become active/inactive
		//
		//		Finally, focus(node) will focus a node, suppressing errors if the node doesn't exist.

		// curNode: DomNode
		//		Currently focused item on screen
		curNode: null,

		// activeStack: dijit._Widget[]
		//		List of currently active widgets (focused widget and it's ancestors)
		activeStack: [],

		registerIframe: function(iframe){
			// summary:
			//		Registers listeners on the specified iframe so that any click
			//		or focus event on that iframe (or anything in it) is reported
			//		as a focus/click event on the <iframe> itself.
			// description:
			//		Currently only used by editor.
			// returns:
			//		Handle with remove() method to deregister.
		},

		registerWin: function(targetWindow, effectiveNode){
			// summary:
			//		Registers listeners on the specified window (either the main
			//		window or an iframe's window) to detect when the user has clicked somewhere
			//		or focused somewhere.
			// description:
			//		Users should call registerIframe() instead of this method.
			// targetWindow: Window?
			//		If specified this is the window associated with the iframe,
			//		i.e. iframe.contentWindow.
			// effectiveNode: DOMNode?
			//		If specified, report any focus events inside targetWindow as
			//		an event on effectiveNode, rather than on evt.target.
			// returns:
			//		Handle with remove() method to deregister.
		}
	};
=====*/

	var FocusManager = declare([Stateful, Evented], {
		// curNode: DomNode
		//		Currently focused item on screen
		curNode: null,

		// activeStack: dijit._Widget[]
		//		List of currently active widgets (focused widget and it's ancestors)
		activeStack: [],

		constructor: function(){
			// Don't leave curNode/prevNode pointing to bogus elements
			var check = lang.hitch(this, function(node){
				if(dom.isDescendant(this.curNode, node)){
					this.set("curNode", null);
				}
				if(dom.isDescendant(this.prevNode, node)){
					this.set("prevNode", null);
				}
			});
			aspect.before(domConstruct, "empty", check);
			aspect.before(domConstruct, "destroy", check);
		},

		registerIframe: function(/*DomNode*/ iframe){
			// summary:
			//		Registers listeners on the specified iframe so that any click
			//		or focus event on that iframe (or anything in it) is reported
			//		as a focus/click event on the <iframe> itself.
			// description:
			//		Currently only used by editor.
			// returns:
			//		Handle with remove() method to deregister.
			return this.registerWin(iframe.contentWindow, iframe);
		},

		registerWin: function(/*Window?*/targetWindow, /*DomNode?*/ effectiveNode){
			// summary:
			//		Registers listeners on the specified window (either the main
			//		window or an iframe's window) to detect when the user has clicked somewhere
			//		or focused somewhere.
			// description:
			//		Users should call registerIframe() instead of this method.
			// targetWindow:
			//		If specified this is the window associated with the iframe,
			//		i.e. iframe.contentWindow.
			// effectiveNode:
			//		If specified, report any focus events inside targetWindow as
			//		an event on effectiveNode, rather than on evt.target.
			// returns:
			//		Handle with remove() method to deregister.

			// TODO: make this function private in 2.0; Editor/users should call registerIframe(),

			var _this = this;
			var mousedownListener = function(evt){
				_this._justMouseDowned = true;
				setTimeout(function(){ _this._justMouseDowned = false; }, 0);

				// workaround weird IE bug where the click is on an orphaned node
				// (first time clicking a Select/DropDownButton inside a TooltipDialog)
				if(has("ie") && evt && evt.srcElement && evt.srcElement.parentNode == null){
					return;
				}

				_this._onTouchNode(effectiveNode || evt.target || evt.srcElement, "mouse");
			};

			// Listen for blur and focus events on targetWindow's document.
			// IIRC, I'm using attachEvent() rather than dojo.connect() because focus/blur events don't bubble
			// through dojo.connect(), and also maybe to catch the focus events early, before onfocus handlers
			// fire.
			// Connect to <html> (rather than document) on IE to avoid memory leaks, but document on other browsers because
			// (at least for FF) the focus event doesn't fire on <html> or <body>.
			var doc = has("ie") ? targetWindow.document.documentElement : targetWindow.document;
			if(doc){
				if(has("ie")){
					targetWindow.document.body.attachEvent('onmousedown', mousedownListener);
					var activateListener = function(evt){
						// IE reports that nodes like <body> have gotten focus, even though they have tabIndex=-1,
						// ignore those events
						var tag = evt.srcElement.tagName.toLowerCase();
						if(tag == "#document" || tag == "body"){ return; }

						// Previous code called _onTouchNode() for any activate event on a non-focusable node.   Can
						// probably just ignore such an event as it will be handled by onmousedown handler above, but
						// leaving the code for now.
						if(a11y.isTabNavigable(evt.srcElement)){
							_this._onFocusNode(effectiveNode || evt.srcElement);
						}else{
							_this._onTouchNode(effectiveNode || evt.srcElement);
						}
					};
					doc.attachEvent('onactivate', activateListener);
					var deactivateListener =  function(evt){
						_this._onBlurNode(effectiveNode || evt.srcElement);
					};
					doc.attachEvent('ondeactivate', deactivateListener);

					return {
						remove: function(){
							targetWindow.document.detachEvent('onmousedown', mousedownListener);
							doc.detachEvent('onactivate', activateListener);
							doc.detachEvent('ondeactivate', deactivateListener);
							doc = null;	// prevent memory leak (apparent circular reference via closure)
						}
					};
				}else{
					doc.body.addEventListener('mousedown', mousedownListener, true);
					doc.body.addEventListener('touchstart', mousedownListener, true);
					var focusListener = function(evt){
						_this._onFocusNode(effectiveNode || evt.target);
					};
					doc.addEventListener('focus', focusListener, true);
					var blurListener = function(evt){
						_this._onBlurNode(effectiveNode || evt.target);
					};
					doc.addEventListener('blur', blurListener, true);

					return {
						remove: function(){
							doc.body.removeEventListener('mousedown', mousedownListener, true);
							doc.body.removeEventListener('touchstart', mousedownListener, true);
							doc.removeEventListener('focus', focusListener, true);
							doc.removeEventListener('blur', blurListener, true);
							doc = null;	// prevent memory leak (apparent circular reference via closure)
						}
					};
				}
			}
		},

		_onBlurNode: function(/*DomNode*/ /*===== node =====*/){
			// summary:
			// 		Called when focus leaves a node.
			//		Usually ignored, _unless_ it *isn't* followed by touching another node,
			//		which indicates that we tabbed off the last field on the page,
			//		in which case every widget is marked inactive
			this.set("prevNode", this.curNode);
			this.set("curNode", null);

			if(this._justMouseDowned){
				// the mouse down caused a new widget to be marked as active; this blur event
				// is coming late, so ignore it.
				return;
			}

			// if the blur event isn't followed by a focus event then mark all widgets as inactive.
			if(this._clearActiveWidgetsTimer){
				clearTimeout(this._clearActiveWidgetsTimer);
			}
			this._clearActiveWidgetsTimer = setTimeout(lang.hitch(this, function(){
				delete this._clearActiveWidgetsTimer;
				this._setStack([]);
				this.prevNode = null;
			}), 100);
		},

		_onTouchNode: function(/*DomNode*/ node, /*String*/ by){
			// summary:
			//		Callback when node is focused or mouse-downed
			// node:
			//		The node that was touched.
			// by:
			//		"mouse" if the focus/touch was caused by a mouse down event

			// ignore the recent blurNode event
			if(this._clearActiveWidgetsTimer){
				clearTimeout(this._clearActiveWidgetsTimer);
				delete this._clearActiveWidgetsTimer;
			}

			// compute stack of active widgets (ex: ComboButton --> Menu --> MenuItem)
			var newStack=[];
			try{
				while(node){
					var popupParent = domAttr.get(node, "dijitPopupParent");
					if(popupParent){
						node=registry.byId(popupParent).domNode;
					}else if(node.tagName && node.tagName.toLowerCase() == "body"){
						// is this the root of the document or just the root of an iframe?
						if(node === win.body()){
							// node is the root of the main document
							break;
						}
						// otherwise, find the iframe this node refers to (can't access it via parentNode,
						// need to do this trick instead). window.frameElement is supported in IE/FF/Webkit
						node=winUtils.get(node.ownerDocument).frameElement;
					}else{
						// if this node is the root node of a widget, then add widget id to stack,
						// except ignore clicks on disabled widgets (actually focusing a disabled widget still works,
						// to support MenuItem)
						var id = node.getAttribute && node.getAttribute("widgetId"),
							widget = id && registry.byId(id);
						if(widget && !(by == "mouse" && widget.get("disabled"))){
							newStack.unshift(id);
						}
						node=node.parentNode;
					}
				}
			}catch(e){ /* squelch */ }

			this._setStack(newStack, by);
		},

		_onFocusNode: function(/*DomNode*/ node){
			// summary:
			//		Callback when node is focused

			if(!node){
				return;
			}

			if(node.nodeType == 9){
				// Ignore focus events on the document itself.  This is here so that
				// (for example) clicking the up/down arrows of a spinner
				// (which don't get focus) won't cause that widget to blur. (FF issue)
				return;
			}

			this._onTouchNode(node);

			if(node == this.curNode){ return; }
			this.set("curNode", node);
		},

		_setStack: function(/*String[]*/ newStack, /*String*/ by){
			// summary:
			//		The stack of active widgets has changed.  Send out appropriate events and records new stack.
			// newStack:
			//		array of widget id's, starting from the top (outermost) widget
			// by:
			//		"mouse" if the focus/touch was caused by a mouse down event

			var oldStack = this.activeStack;
			this.set("activeStack", newStack);

			// compare old stack to new stack to see how many elements they have in common
			for(var nCommon=0; nCommon<Math.min(oldStack.length, newStack.length); nCommon++){
				if(oldStack[nCommon] != newStack[nCommon]){
					break;
				}
			}

			var widget;
			// for all elements that have gone out of focus, set focused=false
			for(var i=oldStack.length-1; i>=nCommon; i--){
				widget = registry.byId(oldStack[i]);
				if(widget){
					widget._hasBeenBlurred = true;		// TODO: used by form widgets, should be moved there
					widget.set("focused", false);
					if(widget._focusManager == this){
						widget._onBlur(by);
					}
					this.emit("widget-blur", widget, by);
				}
			}

			// for all element that have come into focus, set focused=true
			for(i=nCommon; i<newStack.length; i++){
				widget = registry.byId(newStack[i]);
				if(widget){
					widget.set("focused", true);
					if(widget._focusManager == this){
						widget._onFocus(by);
					}
					this.emit("widget-focus", widget, by);
				}
			}
		},

		focus: function(node){
			// summary:
			//		Focus the specified node, suppressing errors if they occur
			if(node){
				try{ node.focus(); }catch(e){/*quiet*/}
			}
		}
	});

	var singleton = new FocusManager();

	// register top window and all the iframes it contains
	ready(function(){
		var handle = singleton.registerWin(win.doc.parentWindow || win.doc.defaultView);
		if(has("ie")){
			unload.addOnWindowUnload(function(){
				handle.remove();
				handle = null;
			})
		}
	});

	// Setup dijit.focus as a pointer to the singleton but also (for backwards compatibility)
	// as a function to set focus.
	dijit.focus = function(node){
		singleton.focus(node);	// indirection here allows dijit/_base/focus.js to override behavior
	};
	for(var attr in singleton){
		if(!/^_/.test(attr)){
			dijit.focus[attr] = typeof singleton[attr] == "function" ? lang.hitch(singleton, attr) : singleton[attr];
		}
	}
	singleton.watch(function(attr, oldVal, newVal){
		dijit.focus[attr] = newVal;
	});

	return singleton;
});

},
'dojo/dnd/TimedMoveable':function(){
define(["../main", "./Moveable"], function(dojo) {
	// module:
	//		dojo/dnd/TimedMoveable
	// summary:
	//		TODOC

	/*=====
	dojo.declare("dojo.dnd.__TimedMoveableArgs", [dojo.dnd.__MoveableArgs], {
		// timeout: Number
		//		delay move by this number of ms,
		//		accumulating position changes during the timeout
		timeout: 0
	});
	=====*/

	// precalculate long expressions
	var oldOnMove = dojo.dnd.Moveable.prototype.onMove;

	dojo.declare("dojo.dnd.TimedMoveable", dojo.dnd.Moveable, {
		// summary:
		//		A specialized version of Moveable to support an FPS throttling.
		//		This class puts an upper restriction on FPS, which may reduce
		//		the CPU load. The additional parameter "timeout" regulates
		//		the delay before actually moving the moveable object.

		// object attributes (for markup)
		timeout: 40,	// in ms, 40ms corresponds to 25 fps

		constructor: function(node, params){
			// summary:
			//		an object that makes a node moveable with a timer
			// node: Node||String
			//		a node (or node's id) to be moved
			// params: dojo.dnd.__TimedMoveableArgs
			//		object with additional parameters.

			// sanitize parameters
			if(!params){ params = {}; }
			if(params.timeout && typeof params.timeout == "number" && params.timeout >= 0){
				this.timeout = params.timeout;
			}
		},

		onMoveStop: function(/* dojo.dnd.Mover */ mover){
			if(mover._timer){
				// stop timer
				clearTimeout(mover._timer);
				// reflect the last received position
				oldOnMove.call(this, mover, mover._leftTop)
			}
			dojo.dnd.Moveable.prototype.onMoveStop.apply(this, arguments);
		},
		onMove: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop){
			mover._leftTop = leftTop;
			if(!mover._timer){
				var _t = this;	// to avoid using dojo.hitch()
				mover._timer = setTimeout(function(){
					// we don't have any pending requests
					mover._timer = null;
					// reflect the last received position
					oldOnMove.call(_t, mover, mover._leftTop);
				}, this.timeout);
			}
		}
	});

	return dojo.dnd.TimedMoveable;
	
});

},
'ibm/usmi/widgets/TaskLink':function(){
require({cache:{
'url:ibm/usmi/widgets/templates/TaskLink.html':"<a class=\"taskLink taskLinkAnchor\" data-dojo-attach-point=\"taskAnchor\" role=\"link\">\r\n<!--\r\n<a class=\"taskLink taskLinkAnchor\" data-dojo-attach-point=\"taskAnchor\" role=\"link\" href=\"#${link}\">\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n\t<div class=\"taskLink-iehack\">\r\n\t    <div class=\"taskLinkLabelContainer\">\r\n<!-- \t\t      <div class=\"taskLinkIcon sprite\" data-dojo-attach-point=\"iconPoint\"></div> -->\r\n\t\t      <img class=\"taskLinkIcon sprite\" data-dojo-attach-point=\"iconPoint\"></img>\r\n\t\t      <div class=\"taskLinkLabel\" data-dojo-attach-point=\"labelPoint\"></div>\r\n\t\t</div>\r\n\t\t<div class=\"taskLinkStatusIcon right sprite\" data-dojo-attach-point=\"statusIconPoint\"></div>\r\n\t</div>\t\r\n</a>"}});
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/widgets/TaskLink", [
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dojo/topic",
	"dijit/_WidgetBase",
	"dijit/_TemplatedMixin",
	"dojo/text!./templates/TaskLink.html",
	"dijit/_Contained",
	"dijit/focus",
	// NextGen imports
	"ibm/usmi/util/BaseWidget",
	"ibm/usmi/widgets/util/TextAbbr",
	"ibm/usmi/widgets/util/Constants",
	"ibm/usmi/core/utils/Logger"
], function (dojo, declare, topic, _WidgetBase, _TemplatedMixin, TaskLinkTemplate, _Contained, focus, BaseWidget,
		TextAbbr, Constants, Logger) {

	return declare([
		_WidgetBase,
		_TemplatedMixin,
		_Contained,
		BaseWidget
	], {

		// Template info
		templateString : TaskLinkTemplate,

		/**
		 * NLS translated label for this link which the user will see
		 * 
		 * @param
		 */
		label : "",

		resourceLabel : "",

		criteriaString : "",

		path : "",

		pathKey : "",

		/**
		 * CSS class name of a 16x16 pixel icon to display by this link
		 * 
		 * @param
		 */
		icon : "",

		// path to where the icons are located
		iconPath : "",

		/**
		 * unique identifier for this link which will be sent out in the
		 * topic.publish when the link is pressed
		 * 
		 * @param
		 */
		link : "",

		/**
		 * unique in conjunction with the link which will be sent out in the
		 * topic.publish when the link is pressed
		 * 
		 * @param
		 */
		taskId : "",

		/**
		 * Type of task launch
		 * 
		 * @param
		 */
		type : "",

		/*
		 * Map TabIndex to the taskAnchor node so this widget can be managed by _KeyNavContainers
		 */
		attributeMap : dojo.delegate(_WidgetBase.prototype.attributeMap, {
			tabIndex : "taskAnchor"
		}),

		/**
		 * Default status of the link: normal, error, warning etc.
		 * 
		 * @public
		 */
		status : "normal",

		/**
		 * Default count displayed next to the link: (5)
		 * 
		 * @public
		 */
		count : null,

		textAbbr : null,

		/**
		 * Whether this link will attempt to pre-select itself when initialzied.
		 * Only one link can be selected at a time but multiple can try... only
		 * one will succeed. Use with care
		 * 
		 * @public
		 */
		selected : false,
		show : true, //false for dynamic task links
		
		wasClicked: false,

		// the logger
		_taskLinkLogger : new Logger("ibm.usmi.widgets", "TaskLink"),

		postCreate : function () {
			this.inherited(arguments);
			this.connect(this.taskAnchor, Constants.ON_CLICK, this._handleClick);
		},

		startup : function () {
			this.inherited(arguments);
			
			// check to see if task selection is set
			if (this.getUrlHashParm(Constants.HASH_TASK_SELECTION)) {
				// check to see what type of link was bookmarked if any
				if (this.type.toUpperCase() === Constants.TASK_TYPE_RELATED_RESOURCE) {
					// since type is related resource, taskSeelction for all related resources
					// is relatedResource, need to look at RELATIONSHIP has value
					if (this.getUrlHashParm(Constants.HASH_TASK_RELATIONSHIP) === this.taskId) {
						this.selected = true;
					}
				} else {
					// check the task selection equal to taskID
					if (this.getUrlHashParm(Constants.HASH_TASK_SELECTION) === this.taskId) {
						this.selected = true;
					}
				}
				
			}	
			
			// fire off the update
			this.update();
		},

		buildRendering : function () {
			this.inherited(arguments);
		},

		update : function (args) {
			dojo.mixin(this, args);
			dojo.toggleClass(this.taskAnchor, "hidden", !this.show);
			this.updateLabel(); // do this unconditionally to add content to the anchor else live validators complain e.g. RPT
			if (this.show) {
				this.updateIcon();
				this.updateStatusIcon();
				this.selectionCheck();
			}
		},

		select : function (select) {
			this.selected = select;
			//fix defect 24146: focus != seclect in L3 page
			//when link is selected by url anchor (dojo.hash)
			if (select && (this.getUrlHashParm(Constants.HASH_TASK_SELECTION) === this.taskId)) {
				var node = this.domNode, focusedArray = dojo.query(node.tagName + ":focus", this.getParent()
						.getParent().domNode);
				//focus on one link but not me, then change focus to me.
				if (focusedArray.length > 0 && focusedArray[0] !== node) {
					node.focus();
				}
			}
			dojo.toggleClass(this.domNode, "taskLink-selected", select);
		},

		selectionCheck : function () {
			if (this.selected) {
				this.getParent().getParent()._linkClicked("selfSelect", this);
				this._handleClick();
			}
		},

		updateIcon : function () {
			//dojo.removeClass(this.iconPoint);
			// dojo.addClass(this.iconPoint, ["taskLinkIcon", "sprite", this.icon]);
			dojo.attr(this.iconPoint, "src", this.iconPath + this.icon);

		},

		updateLabel : function () {
			var formattedLabel = this.label;

			//update the textAbbr if it exists
			if (this.textAbbr) {
				this.textAbbr.update(formattedLabel);
			} else {//create textAbbr
				this.textAbbr = new TextAbbr({
					text : formattedLabel,
					maxLength : 34,
					format : " (x)"
				}, this.labelPoint);
			}
		},

		updateStatusIcon : function () {
			var domNode = this.statusIconPoint, icons = [
				"errorIcon",
				"warningIcon",
				"unknownIcon"
			], lookup = {
				"error" : {
					iconClass : "errorIcon",
					altText : "(X)"
				},
				"warning" : {
					iconClass : "warningIcon",
					altText : "(!)"
				},
				"unknown" : {
					iconClass : "unknownIcon",
					altText : "(?)"
				},
				"normal" : {
					iconClass : "",
					altText : ""
				}
			}, statusObject = lookup[this.status] || lookup.normal;

			dojo.removeClass(domNode, dojo.filter(icons, function (icon) {
				return icon !== statusObject.iconClass;
			}));
			dojo.addClass(domNode, statusObject.iconClass);
			if (dojo.hasClass(dojo.body(), "dijit_a11y")) {
				dojo.create("span", {
					innerHTML : statusObject.altText
				}, domNode, "only");
			}
		},

		/*
		 * Simulate _FormWidget functions so this widget can be managed by _KeyNavContainers
		 */
		isFocusable : function () {
			return !this.disabled && (dojo.style(this.domNode, "display") !== "none");
		},

		focus : function () {
			focus.focus(this.taskAnchor);
		},

		getLabel : function () {
			return this.label;
		},

		getResourceLabel : function () {
			return this.resourceLabel;
		},

		getCriteriaString : function () {
			return this.criteriaString;
		},

		getPath : function () {
			return this.path;
		},

		getPathKey : function () {
			return this.pathKey;
		},
		
		getWasClicked : function () {
			return this.wasClicked;
		},
		
		resetWasClicked : function () {
			this.wasClicked = false;
		},

		_handleClick : function (evt) {
			if (this._taskLinkLogger.isFinest()) {
				this._taskLinkLogger.entering("", "_handleClick");
			}
			var obj, launchParms, targetInfo;
			
			this.wasClicked = true;
			// if task type is widget, hash the link
			if (this.type.toUpperCase() === Constants.TASK_TYPE_WIDGET) {
				if (this._taskLinkLogger.isFiner()) {
					this._taskLinkLogger.finer(this.link, "_handleClick");
				}
				obj = dojo.queryToObject(this.link);
				
				// remove the RELATIONSHIP hashkey if there
				if(obj['relationship']) {
					delete obj.relationship;
				}
				
				//if path specified, set it to be included in hash
				if (this.path && this.pathKey) {
					obj.taskPath = this.path;
					obj.taskPathKey = this.pathKey;
				}
				dojo.hash(dojo.objectToQuery(obj), true); // set
				targetInfo = {
					sourceToken : this.sourceToken,
					taskSelection : obj.taskSelection,
					taskPath : this.path,
					taskPathKey : this.pathKey
				};
				topic.publish(Constants.TOPIC_RETARGET_INFO, targetInfo); //tell WorkArea the new info

			} else if (this.type.toUpperCase() === Constants.TASK_TYPE_RELATED_RESOURCE) {
				if (this._taskLinkLogger.isFiner()) {
					this._taskLinkLogger.finer(this.link, "_handleClick");
				}
				obj = dojo.queryToObject(this.link);
				obj.relationship = this.criteriaString;
				dojo.hash(dojo.objectToQuery(obj), true); // set
				targetInfo = {
					relationship : this.criteriaString,
					sourceToken : this.sourceToken,
					taskSelection : obj.taskSelection
				};
				topic.publish(Constants.TOPIC_RETARGET_INFO, targetInfo); //tell RRTable the new info

			}
			// if is URL, publish the URL
			else if (this.type.toUpperCase() === Constants.TASK_TYPE_URL) {
				launchParms = {
					topic : Constants.TOPIC_DEFAULT_URL_LAUNCH,
					launchType : Constants.TASK_TYPE_URL,
					urlToLaunch : this.link
				};

				topic.publish(Constants.TOPIC_DEFAULT_URL_LAUNCH, launchParms);
			}

			if (this._taskLinkLogger.isFinest()) {
				this._taskLinkLogger.exiting("", "_handleClick");
			}
		},
		
		fakeClick : function () {
			if (this._taskLinkLogger.isFinest()) {
				this._taskLinkLogger.entering("", "fakeClick");
			}
			var obj, launchParms, targetInfo;
			
			// if task type is widget, hash the link
			if (this.type.toUpperCase() === Constants.TASK_TYPE_WIDGET) {
				if (this._taskLinkLogger.isFiner()) {
					this._taskLinkLogger.finer(this.link, "fakeClick");
				}
				obj = dojo.queryToObject(this.link);
				targetInfo = {
					sourceToken : this.sourceToken,
					taskSelection : obj.taskSelection,
					taskPath : this.path,
					taskPathKey : this.pathKey
				};
				topic.publish(Constants.TOPIC_RETARGET_INFO, targetInfo); //tell WorkArea the new info

			} else if (this.type.toUpperCase() === Constants.TASK_TYPE_RELATED_RESOURCE) {
				if (this._taskLinkLogger.isFiner()) {
					this._taskLinkLogger.finer(this.link, "fakeClick");
				}
				obj = dojo.queryToObject(this.link);
				targetInfo = {
					relationship : this.criteriaString,
					sourceToken : this.sourceToken,
					taskSelection : obj.taskSelection
				};
				topic.publish(Constants.TOPIC_RETARGET_INFO, targetInfo); //tell RRTable the new info

			}
			// if is URL, publish the URL
			else if (this.type.toUpperCase() === Constants.TASK_TYPE_URL) {
				launchParms = {
					topic : Constants.TOPIC_DEFAULT_URL_LAUNCH,
					launchType : Constants.TASK_TYPE_URL,
					urlToLaunch : this.link
				};

				topic.publish(Constants.TOPIC_DEFAULT_URL_LAUNCH, launchParms);
			}

			if (this._taskLinkLogger.isFinest()) {
				this._taskLinkLogger.exiting("", "_handleClick");
			}			
		},
		
		destroy : function () {
			if (this.textAbbr) {
				this.textAbbr.destroy();
			}
			this.inherited(arguments);
		}
	});
});
},
'ibm/tivoli/tip/navwidget/topology/dijit/MenuProviderHelperBase':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/i18n!ibm/tivoli/tip/navwidget/nls/resources"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.topology.dijit.MenuProviderHelperBase");

dojo.requireLocalization("ibm.tivoli.tip.navwidget", "resources");

dojo.declare("ibm.tivoli.tip.navwidget.topology.dijit.MenuProviderHelperBase",null,
{		
	// summary:
	//		Coming soon
	// description:
	//		Coming soon
	
	constructor: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		this._resources = dojo.i18n.getLocalization("ibm.tivoli.tip.navwidget", "resources");    
	},

	getMenuItems: function(/*Object*/ context) {		
		// summary:
		//		Coming soon
		// description:
		//		required, as it is used as the getMenuItemsCallback with the menu provider
		// context:
		//		Coming soon
		return [];		// by default, return empty collection of menu items
	},
	
	// required, as it is used as the getMenuCommandCallback with the menu provider
	getMenuCommand: function(commandName, context, menuItem) {
		return menuItem;	// by default, return the menuItem, i.e., no processing
	}
});



});

},
'dojox/mvc/_DataBindingMixin':function(){
define([
	"dojo/_base/lang",
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/Stateful",
	"dijit/registry"
], function(lang, array, declare, Stateful, registry){
	/*=====
	registry = dijit.registry;
	=====*/

	return declare("dojox.mvc._DataBindingMixin", null, {
		// summary:
		//		Provides the ability for dijits or custom view components to become
		//		data binding aware.
		//
		// description:
		//		Data binding awareness enables dijits or other view layer
		//		components to bind to locations within a client-side data model,
		//		which is commonly an instance of the dojox.mvc.StatefulModel class. A
		//		bind is a bi-directional update mechanism which is capable of
		//		synchronizing value changes between the bound dijit or other view
		//		component and the specified location within the data model, as well
		//		as changes to other properties such as "valid", "required",
		//		"readOnly" etc.
		//
		//		The data binding is commonly specified declaratively via the "ref"
		//		property in the "data-dojo-props" attribute value.
		//
		//		Consider the following simple example:
		//
		//		|	<script>
		//		|		var model;
		//		|		require(["dijit/StatefulModel", "dojo/parser"], function(StatefulModel, parser){
		//		|			model = new StatefulModel({ data : {
		//		|				hello : "Hello World"
		//		|			}});
		//		|			parser.parse();
		//		|		});
		//		|	</script>
		//		|
		//		|	<input id="hello1" data-dojo-type="dijit.form.TextBox"
		//		|		data-dojo-props="ref: model.hello"></input>
		//		|
		//		|	<input id="hello2" data-dojo-type="dijit.form.TextBox"
		//		|		data-dojo-props="ref: model.hello"></input>
		//
		//		In the above example, both dijit.form.TextBox instances (with IDs
		//		"hello1" and "hello2" respectively) are bound to the same reference
		//		location in the data model i.e. "hello" via the "ref" expression
		//		"model.hello". Both will have an initial value of "Hello World".
		//		Thereafter, a change in the value of either of the two textboxes
		//		will cause an update of the value in the data model at location
		//		"hello" which will in turn cause a matching update of the value in
		//		the other textbox.
	
		// ref: String||dojox.mvc.StatefulModel
		//		The value of the data binding expression passed declaratively by
		//		the developer. This usually references a location within an
		//		existing datamodel and may be a relative reference based on the
		//		parent / container data binding (dot-separated string).
		ref: null,

/*=====
		// binding: [readOnly] dojox.mvc.StatefulModel
		//		The read only value of the resolved data binding for this widget.
		//		This may be a result of resolving various relative refs along
		//		the parent axis.
		binding: null,
=====*/

		//////////////////////// PUBLIC METHODS ////////////////////////
	
		isValid: function(){
			// summary:
			//		Returns the validity of the data binding.
			// returns:
			//		Boolean
			//		The validity associated with the data binding.
			// description:
			//		This function is meant to provide an API bridge to the dijit API.
			//		Validity of data-bound dijits is a function of multiple concerns:
			//		- The validity of the value as ascertained by the data binding
			//		  and constraints specified in the data model (usually semantic).
			//		- The validity of the value as ascertained by the widget itself
			//		  based on widget constraints (usually syntactic).
			//		In order for dijits to function correctly in data-bound
			//		environments, it is imperative that their isValid() functions
			//		assess the model validity of the data binding via the
			//		this.inherited(arguments) hierarchy and declare any values
			//		failing the test as invalid.
			return this.get("binding") ? this.get("binding").get("valid") : true;
		},

		//////////////////////// LIFECYCLE METHODS ////////////////////////

		_dbstartup: function(){
			// summary:
			//		Tie data binding initialization into the widget lifecycle, at
			//		widget startup.
			// tags:
			//		private
			if(this._databound){
				return;
			}
			this._unwatchArray(this._viewWatchHandles);
			// add 2 new view watches, active only after widget has started up
			this._viewWatchHandles = [
				// 1. data binding refs
				this.watch("ref", function(name, old, current){
					if(this._databound){
						this._setupBinding();
					}
				}),
				// 2. widget values
				this.watch("value", function(name, old, current){
					if(this._databound){
						var binding = this.get("binding");
						if(binding){
							// dont set value if the valueOf current and old match.
							if(!((current && old) && (old.valueOf() === current.valueOf()))){
								binding.set("value", current);
							}
						}
					}
				})
			];
			this._beingBound = true;
			this._setupBinding();
			delete this._beingBound;
			this._databound = true;
		},

		//////////////////////// PRIVATE METHODS ////////////////////////

		_setupBinding: function(parentBinding){
			// summary:
			//		Calculate and set the dojo.Stateful data binding for the
			//		associated dijit or custom view component.
			//	parentBinding:
			//		The binding of this widget/view component's data-bound parent,
			//		if available.
			// description:
			//		The declarative data binding reference may be specified in two
			//		ways via markup:
			//		- For older style documents (non validating), controls may use
			//		  the "ref" attribute to specify the data binding reference
			//		  (String).
			//		- For validating documents using the new Dojo parser, controls
			//		  may specify the data binding reference (String) as the "ref"
			//		  property specified in the data-dojo-props attribute.
			//		Once the ref value is obtained using either of the above means,
			//		the binding is set up for this control and its required, readOnly
			//		etc. properties are refreshed.
			//		The data binding may be specified as a direct reference to the
			//		dojo.Stateful model node or as a string relative to its DOM
			//		parent or another widget.
			//		There are three ways in which the data binding node reference is
			//		calculated when specified as a string:
			//		- If an explicit parent widget is specified, the binding is
			//		  calculated relative to the parent widget's data binding.
			//		- For any dijits that specify a data binding reference,
			//		  we walk up their DOM hierarchy to obtain the first container
			//		  dijit that has a data binding set up and use the reference String
			//		  as a property name relative to the parent's data binding context.
			//		- If no such parent is found i.e. for the outermost container
			//		  dijits that specify a data binding reference, the binding is
			//		  calculated by treating the reference String as an expression and
			//		  evaluating it to obtain the dojo.Stateful node in the datamodel.
			//		This method throws an Error in these two conditions:
			//		- The ref is an expression i.e. outermost bound dijit, but the
			//		  expression evaluation fails.
			//		- The calculated binding turns out to not be an instance of a
			//		  dojo.Stateful node.
			// tags:
			//		private
			if(!this.ref){
				return; // nothing to do here
			}
			var ref = this.ref, pw, pb, binding;
			// Now compute the model node to bind to
			if(ref && lang.isFunction(ref.toPlainObject)){ // programmatic instantiation or direct ref
				binding = ref;
			}else if(/^\s*expr\s*:\s*/.test(ref)){ // declarative: refs as dot-separated expressions
				ref = ref.replace(/^\s*expr\s*:\s*/, "");
				binding = lang.getObject(ref);
			}else if(/^\s*rel\s*:\s*/.test(ref)){ // declarative: refs relative to parent binding, dot-separated 
				ref = ref.replace(/^\s*rel\s*:\s*/, "");
				parentBinding = parentBinding || this._getParentBindingFromDOM();
				if(parentBinding){
					binding = lang.getObject("" + ref, false, parentBinding);
				}
			}else if(/^\s*widget\s*:\s*/.test(ref)){ // declarative: refs relative to another dijits binding, dot-separated
				ref = ref.replace(/^\s*widget\s*:\s*/, "");
				var tokens = ref.split(".");
				if(tokens.length == 1){
					binding = registry.byId(ref).get("binding");
				}else{
					pb = registry.byId(tokens.shift()).get("binding");
					binding = lang.getObject(tokens.join("."), false, pb);
				}
			}else{ // defaults: outermost refs are expressions, nested are relative to parents
				parentBinding = parentBinding || this._getParentBindingFromDOM();
				if(parentBinding){
					binding = lang.getObject("" + ref, false, parentBinding);
				}else{
					try{
						if(lang.getObject(ref) instanceof Stateful){
							binding = lang.getObject(ref);
						}						
					}catch(err){
						if(ref.indexOf("${") == -1){ // Ignore templated refs such as in repeat body
							throw new Error("dojox.mvc._DataBindingMixin: '" + this.domNode +
								"' widget with illegal ref expression: '" + ref + "'");
						}
					}
				}
			}
			if(binding){
				if(lang.isFunction(binding.toPlainObject)){
					this.binding = binding;
					this._updateBinding("binding", null, binding);
				}else{
					throw new Error("dojox.mvc._DataBindingMixin: '" + this.domNode +
						"' widget with illegal ref not evaluating to a dojo.Stateful node: '" + ref + "'");
				}
			}
		},

		_isEqual: function(one, other){
        	// test for equality
			return one === other ||
				// test for NaN === NaN
				isNaN(one) && typeof one === 'number' &&
				isNaN(other) && typeof other === 'number';
		},

		_updateBinding: function(name, old, current){
			// summary:
			//		Set the data binding to the supplied value, which must be a
			//		dojo.Stateful node of a data model.
			//	name:
			//		The name of the binding property (always "binding").
			//	old:
			//		The old dojo.Stateful binding node of the data model.
			//	current:
			//		The new dojo.Stateful binding node of the data model.
			// description:
			//		Applies the specified data binding to the attached widget.
			//		Loses any prior watch registrations on the previously active
			//		bind, registers the new one, updates data binds of any contained
			//		widgets and also refreshes all associated properties (valid,
			//		required etc.)
			// tags:
			//		private
	
			// remove all existing watches (if there are any, there will be 5)
			this._unwatchArray(this._modelWatchHandles);
			// add 5 new model watches
			var binding = this.get("binding");
			if(binding && lang.isFunction(binding.watch)){
				var pThis = this;
				this._modelWatchHandles = [
					// 1. value - no default
					binding.watch("value", function (name, old, current){
						if(pThis._isEqual(old, current)){return;}
						if(pThis._isEqual(pThis.get('value'), current)){return;}
						pThis.set("value", current);
					}),
					// 2. valid - default "true"
					binding.watch("valid", function (name, old, current){
						pThis._updateProperty(name, old, current, true);
						if(current !== pThis.get(name)){
							if(pThis.validate && lang.isFunction(pThis.validate)){
								pThis.validate();
							}
						}
					}),
					// 3. required - default "false"
					binding.watch("required", function (name, old, current){
						pThis._updateProperty(name, old, current, false, name, current);
					}),
					// 4. readOnly - default "false"
					binding.watch("readOnly", function (name, old, current){
						pThis._updateProperty(name, old, current, false, name, current);
					}),
					// 5. relevant - default "true"
					binding.watch("relevant", function (name, old, current){
						pThis._updateProperty(name, old, current, false, "disabled", !current);
					})
				];
				var val = binding.get("value");
				if(val != null){
					this.set("value", val);
				}
			}
			this._updateChildBindings();
		},
	
		_updateProperty: function(name, old, current, defaultValue, setPropName, setPropValue){
			// summary:
			//		Update a binding property of the bound widget.
			//	name:
			//		The binding property name.
			//	old:
			//		The old value of the binding property.
			//	current:
			//		The new or current value of the binding property.
			//	defaultValue:
			//		The optional value to be applied as the current value of the
			//		binding property if the current value is null.
			//	setPropName:
			//		The optional name of a stateful property to set on the bound
			//		widget.
			//	setPropValue:
			//		The value, if an optional name is provided, for the stateful
			//		property of the bound widget.
			// tags:
			//		private
			if(old === current){
				return;
			}
			if(current === null && defaultValue !== undefined){
				current = defaultValue;
			}
			if(current !== this.get("binding").get(name)){
				this.get("binding").set(name, current);
			}
			if(setPropName){
				this.set(setPropName, setPropValue);
			}
		},
		_updateChildBindings: function(parentBind){
			// summary:
			//		Update this widget's value based on the current binding and
			//		set up the bindings of all contained widgets so as to refresh
			//		any relative binding references. 
			// 		findWidgets does not return children of widgets so need to also
			//		update children of widgets which are not bound but may hold widgets which are.
			//	parentBind:
			//		The binding on the parent of a widget whose children may have bindings 
			//		which need to be updated.
			// tags:
			//		private
			var binding = this.get("binding") || parentBind;
			if(binding && !this._beingBound){
				array.forEach(registry.findWidgets(this.domNode), function(widget){
					if(widget.ref && widget._setupBinding){
						widget._setupBinding(binding);
					}else{	
						widget._updateChildBindings(binding);
					}
				});
			}
		},

		_getParentBindingFromDOM: function(){
			// summary:
			//		Get the parent binding by traversing the DOM ancestors to find
			//		the first enclosing data-bound widget.
			// returns:
			//		The parent binding, if one exists along the DOM parent axis.
			// tags:
			//		private
			var pn = this.domNode.parentNode, pw, pb;
			while(pn){
				pw = registry.getEnclosingWidget(pn);
				if(pw){
					pb = pw.get("binding");
					if(pb && lang.isFunction(pb.toPlainObject)){
						break;
					}
				}
				pn = pw ? pw.domNode.parentNode : null;
			}
			return pb;
		},

		_unwatchArray: function(watchHandles){
			// summary:
			//		Given an array of watch handles, unwatch all.
			//	watchHandles:
			//		The array of watch handles.
			// tags:
			//		private
			array.forEach(watchHandles, function(h){ h.unwatch(); });
		}
	});
});

},
'idx/oneui/Dialog':function(){
/*
 * Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

define([
	"require",
	"dojo/_base/array", // array.forEach array.indexOf array.map
	"dojo/_base/connect", // connect._keypress
	"dojo/_base/declare", // declare
	"dojo/_base/Deferred", // Deferred
	"dojo/dom", // dom.isDescendant
	"dojo/dom-class", // domClass.add domClass.contains
	"dojo/dom-construct",
	"dojo/dom-geometry", // domGeometry.position
	"dojo/dom-style", // domStyle.set
	"dojo/dom-attr", // attr.has
	"dojo/_base/event", // event.stop
	"dojo/_base/fx", // fx.fadeIn fx.fadeOut
	"dojo/i18n", // i18n.getLocalization
	"dojo/_base/kernel", // kernel.isAsync
	"dojo/keys",
	"dojo/_base/lang", // lang.mixin lang.hitch
	"dojo/on",
	"dojo/ready",
	"dojo/_base/sniff", // has("ie") has("opera")
	"dojo/_base/window", // win.body
	"dojo/window", // winUtils.getBox
	"dijit/focus",
	"dijit/a11y",
	"dijit/_base/manager",	// manager.defaultDuration
	"dijit/Dialog",
	"dijit/form/Button",
	"dojo/text!./templates/Dialog.html",
	"dijit",			// for back-compat, exporting dijit._underlay (remove in 2.0)
	"dojo/i18n!./nls/Dialog"
], function(require, array, connect, declare, Deferred,
			dom, domClass, domConstruct, domGeometry, domStyle, domAttr, event, fx, i18n, kernel, keys, lang, on, ready, has, win, winUtils,
			focus, a11y, manager, Dialog, Button, template, dijit){
	/**
	 * Creates a new idx.oneui.Dialog
	 * @name idx.oneui.Dialog
	 * @class idx.oneui.Dialog enhanced dijit.Dialog with specified structure following IBM One UI standard
	 * <b><a href="http://mds.torolab.ibm.com/IBM_OneUI/UI_DesignSignature_Workstream/UI_design_signature/9%2E%20%20Forms_Dialog%20Box/VS_DialogBox_BP_Mar-21-12%2Epng">Dialog Box</a></b> 
	 * @augments dijit.Dialog
	 * @example
	 * var dialog = new idx.oneui.Dialog({
			id: "dialog",
			title: "Dialog Title",
			instruction: "Instructional information goes here.",
			content: "&lt;div style='height:80px'&gt;Lorem ipsum dolor sit amet, consectetuer adipiscing elit.&lt;/div&gt;",
			reference: {
				name: "Link goes here",
				link: "http://dojotoolkit.org/"
			},
			closeButtonLabel: "Cancel"
		}, "div");
	 */
	var Dialog = declare("idx.oneui.Dialog", Dialog, {
	/**@lends idx.oneui.Dialog.prototype*/
		templateString: template,
		baseClass: "idxDialog",
		_setTitleAttr: [],
		
		draggable: false,
		/**
		 * Dialog title
		 * @type String
		 */
		title: "",
		/**
		 * Dialog instruction, just below the title
		 * @type String
		 */
		instruction: "",
		/**
		 * Dialog content
		 * @type String | dijit.layout.TabContainer
		 */
		content: "",
		/**
		 * Referance link of Dialog, reference.name for link name, and reference.link for link url
		 * @type Object
		 */
		reference: {
			name: "",
			link: ""
		},
		/**
		 * Action buttons for Dialog in the action bar
		 * @type Array [dijit.form.Button]
		 */
		buttons: null,
		/**
		 * Label on Dialog close button
		 * @type String
		 */
		closeButtonLabel: "",
		
		referenceName: "",
		referenceLink: "",
		
		postMixInProperties: function(){
			this.inherited(arguments);
			this.referenceName = this.referenceName || this.reference.name;
			this.referenceLink = this.referenceLink || this.reference.link;
			this._nlsResources = i18n.getLocalization("idx.oneui", "Dialog", this.lang);
		},
		postCreate: function(){
			this.inherited(arguments);
			this.closeButton = new Button({
				label: this.closeButtonLabel || this._nlsResources.closeButtonLabel,
				onClick: lang.hitch(this, function(evt){
					this.onCancel();
					event.stop(evt);
				})
			}, this.closeButtonNode);
			array.forEach(this.buttons, function(button){
				if(button.declaredClass == "dijit.form.Button"){
					if(!domClass.contains(this.closeButton.domNode, "idxSecondaryButton")){
						domClass.add(this.closeButton.domNode, "idxSecondaryButton");
					}
					domConstruct.place(button.domNode, this.closeButton.domNode, "before");
				}
			}, this);
		},
		startup: function(){
			this.inherited(arguments);
			if(this.containerNode.children[0] && 
			domClass.contains(this.containerNode.children[0], "dijitTabContainer")){
				domStyle.set(this.contentWrapper, {
					borderTop: "0 none",
					paddingTop: "0"
				});
			}
		},
		_size: function(){
			this.inherited(arguments);
			//resize the Dialog to wrap the content
			var children = this.containerNode.children,
				innerWidth = 0;
			array.forEach(children, function(child){
				innerWidth = Math.max(domStyle.get(child, "width"), innerWidth);
			});
			if(innerWidth > domStyle.get(this.containerNode, "width")){
				domStyle.set(this.domNode, {
					width:"auto"
				});
				domStyle.set(this.containerNode, {
					width:"auto",
					height:"auto"
				})
			}
		},
		_onKey: function(evt){
			this.inherited(arguments);
			var node = evt.target;
			if(domAttr.has(node, "href")){return;}
			if(node == this.closeButton.domNode){return;}
			while(node){
				if(node == this.domNode || domClass.contains(node, "dijitPopup")){
					if(evt.keyCode == keys.ENTER){
						this.onExecute();
					}else{
						return; // just let it go
					}
				}
				node = node.parentNode;
			}
			event.stop(evt);
		},
		_getFocusItems: function(){
			//	summary:
			//		override _DialogMixin._getFocusItems.
			if(this._firstFocusItem){
				this._firstFocusItem = this._getFirstItem();
				if(!this._firstFocusItem){
					var elems = a11y._getTabNavigable(this.messageWrapper);
					this._firstFocusItem = elems.lowest || elems.first || this.closeButton.focusNode || this.domNode;
				}
				return;
			}
			this._firstFocusItem = this.closeButton.focusNode;
			this._lastFocusItem = this.closeButton.focusNode;
		},
		hide: function(){
			this.inherited(arguments);
			this._firstFocusItem = null;
		},
		_getFirstItem: function(){
			if(this.title){return this.titleNode;}
			if(this.instruction){return this.instructionNode;}
			return null;
		}
	});

	// Back compat w/1.6, remove for 2.0
	if(!kernel.isAsync){
		ready(0, function(){
			var requires = ["dijit/TooltipDialog"];
			require(requires);	// use indirection so modules not rolled into a build
		});
	}

	return Dialog;
});

},
'dojoe/tablex/plugins/GridFilter':function(){
define("dojoe/tablex/plugins/GridFilter", ["dojo", "dijit"], function(dojo, dijit) {

	// summary:
	//		plugin class to enable Advanced grid Filter
	// description:
	//		Please refer to dojox.grid.enhanced.plugins.Filter
	
	
});
},
'dojo/string':function(){
define(["./_base/kernel", "./_base/lang"], function(dojo, lang) {
	// module:
	//		dojo/string
	// summary:
	//		TODOC

lang.getObject("string", true, dojo);

/*=====
dojo.string = {
	// summary: String utilities for Dojo
};
=====*/

dojo.string.rep = function(/*String*/str, /*Integer*/num){
	// summary:
	//		Efficiently replicate a string `n` times.
	// str:
	//		the string to replicate
	// num:
	//		number of times to replicate the string

	if(num <= 0 || !str){ return ""; }

	var buf = [];
	for(;;){
		if(num & 1){
			buf.push(str);
		}
		if(!(num >>= 1)){ break; }
		str += str;
	}
	return buf.join("");	// String
};

dojo.string.pad = function(/*String*/text, /*Integer*/size, /*String?*/ch, /*Boolean?*/end){
	// summary:
	//		Pad a string to guarantee that it is at least `size` length by
	//		filling with the character `ch` at either the start or end of the
	//		string. Pads at the start, by default.
	// text:
	//		the string to pad
	// size:
	//		length to provide padding
	// ch:
	//		character to pad, defaults to '0'
	// end:
	//		adds padding at the end if true, otherwise pads at start
	// example:
	//	|	// Fill the string to length 10 with "+" characters on the right.  Yields "Dojo++++++".
	//	|	dojo.string.pad("Dojo", 10, "+", true);

	if(!ch){
		ch = '0';
	}
	var out = String(text),
		pad = dojo.string.rep(ch, Math.ceil((size - out.length) / ch.length));
	return end ? out + pad : pad + out;	// String
};

dojo.string.substitute = function(	/*String*/		template,
									/*Object|Array*/map,
									/*Function?*/	transform,
									/*Object?*/		thisObject){
	// summary:
	//		Performs parameterized substitutions on a string. Throws an
	//		exception if any parameter is unmatched.
	// template:
	//		a string with expressions in the form `${key}` to be replaced or
	//		`${key:format}` which specifies a format function. keys are case-sensitive.
	// map:
	//		hash to search for substitutions
	// transform:
	//		a function to process all parameters before substitution takes
	//		place, e.g. mylib.encodeXML
	// thisObject:
	//		where to look for optional format function; default to the global
	//		namespace
	// example:
	//		Substitutes two expressions in a string from an Array or Object
	//	|	// returns "File 'foo.html' is not found in directory '/temp'."
	//	|	// by providing substitution data in an Array
	//	|	dojo.string.substitute(
	//	|		"File '${0}' is not found in directory '${1}'.",
	//	|		["foo.html","/temp"]
	//	|	);
	//	|
	//	|	// also returns "File 'foo.html' is not found in directory '/temp'."
	//	|	// but provides substitution data in an Object structure.  Dotted
	//	|	// notation may be used to traverse the structure.
	//	|	dojo.string.substitute(
	//	|		"File '${name}' is not found in directory '${info.dir}'.",
	//	|		{ name: "foo.html", info: { dir: "/temp" } }
	//	|	);
	// example:
	//		Use a transform function to modify the values:
	//	|	// returns "file 'foo.html' is not found in directory '/temp'."
	//	|	dojo.string.substitute(
	//	|		"${0} is not found in ${1}.",
	//	|		["foo.html","/temp"],
	//	|		function(str){
	//	|			// try to figure out the type
	//	|			var prefix = (str.charAt(0) == "/") ? "directory": "file";
	//	|			return prefix + " '" + str + "'";
	//	|		}
	//	|	);
	// example:
	//		Use a formatter
	//	|	// returns "thinger -- howdy"
	//	|	dojo.string.substitute(
	//	|		"${0:postfix}", ["thinger"], null, {
	//	|			postfix: function(value, key){
	//	|				return value + " -- howdy";
	//	|			}
	//	|		}
	//	|	);

	thisObject = thisObject || dojo.global;
	transform = transform ?
		lang.hitch(thisObject, transform) : function(v){ return v; };

	return template.replace(/\$\{([^\s\:\}]+)(?:\:([^\s\:\}]+))?\}/g,
		function(match, key, format){
			var value = lang.getObject(key, false, map);
			if(format){
				value = lang.getObject(format, false, thisObject).call(thisObject, value, key);
			}
			return transform(value, key).toString();
		}); // String
};

/*=====
dojo.string.trim = function(str){
	// summary:
	//		Trims whitespace from both sides of the string
	// str: String
	//		String to be trimmed
	// returns: String
	//		Returns the trimmed string
	// description:
	//		This version of trim() was taken from [Steven Levithan's blog](http://blog.stevenlevithan.com/archives/faster-trim-javascript).
	//		The short yet performant version of this function is dojo.trim(),
	//		which is part of Dojo base.  Uses String.prototype.trim instead, if available.
	return "";	// String
}
=====*/

dojo.string.trim = String.prototype.trim ?
	lang.trim : // aliasing to the native function
	function(str){
		str = str.replace(/^\s+/, '');
		for(var i = str.length - 1; i >= 0; i--){
			if(/\S/.test(str.charAt(i))){
				str = str.substring(0, i + 1);
				break;
			}
		}
		return str;
	};

return dojo.string;
});

},
'dojo/dnd/Manager':function(){
define(["../main", "../Evented", "./common", "./autoscroll", "./Avatar"], function(dojo, Evented) {
	// module:
	//		dojo/dnd/Manager
	// summary:
	//		TODOC


var Manager = dojo.declare("dojo.dnd.Manager", [Evented], {
	// summary:
	//		the manager of DnD operations (usually a singleton)
	constructor: function(){
		this.avatar  = null;
		this.source = null;
		this.nodes = [];
		this.copy  = true;
		this.target = null;
		this.canDropFlag = false;
		this.events = [];
	},

	// avatar's offset from the mouse
	OFFSET_X: 16,
	OFFSET_Y: 16,

	// methods
	overSource: function(source){
		// summary:
		//		called when a source detected a mouse-over condition
		// source: Object
		//		the reporter
		if(this.avatar){
			this.target = (source && source.targetState != "Disabled") ? source : null;
			this.canDropFlag = Boolean(this.target);
			this.avatar.update();
		}
		dojo.publish("/dnd/source/over", [source]);
	},
	outSource: function(source){
		// summary:
		//		called when a source detected a mouse-out condition
		// source: Object
		//		the reporter
		if(this.avatar){
			if(this.target == source){
				this.target = null;
				this.canDropFlag = false;
				this.avatar.update();
				dojo.publish("/dnd/source/over", [null]);
			}
		}else{
			dojo.publish("/dnd/source/over", [null]);
		}
	},
	startDrag: function(source, nodes, copy){
		// summary:
		//		called to initiate the DnD operation
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise
		this.source = source;
		this.nodes  = nodes;
		this.copy   = Boolean(copy); // normalizing to true boolean
		this.avatar = this.makeAvatar();
		dojo.body().appendChild(this.avatar.node);
		dojo.publish("/dnd/start", [source, nodes, this.copy]);
		this.events = [
			dojo.connect(dojo.doc, "onmousemove", this, "onMouseMove"),
			dojo.connect(dojo.doc, "onmouseup",   this, "onMouseUp"),
			dojo.connect(dojo.doc, "onkeydown",   this, "onKeyDown"),
			dojo.connect(dojo.doc, "onkeyup",     this, "onKeyUp"),
			// cancel text selection and text dragging
			dojo.connect(dojo.doc, "ondragstart",   dojo.stopEvent),
			dojo.connect(dojo.body(), "onselectstart", dojo.stopEvent)
		];
		var c = "dojoDnd" + (copy ? "Copy" : "Move");
		dojo.addClass(dojo.body(), c);
	},
	canDrop: function(flag){
		// summary:
		//		called to notify if the current target can accept items
		var canDropFlag = Boolean(this.target && flag);
		if(this.canDropFlag != canDropFlag){
			this.canDropFlag = canDropFlag;
			this.avatar.update();
		}
	},
	stopDrag: function(){
		// summary:
		//		stop the DnD in progress
		dojo.removeClass(dojo.body(), ["dojoDndCopy", "dojoDndMove"]);
		dojo.forEach(this.events, dojo.disconnect);
		this.events = [];
		this.avatar.destroy();
		this.avatar = null;
		this.source = this.target = null;
		this.nodes = [];
	},
	makeAvatar: function(){
		// summary:
		//		makes the avatar; it is separate to be overwritten dynamically, if needed
		return new dojo.dnd.Avatar(this);
	},
	updateAvatar: function(){
		// summary:
		//		updates the avatar; it is separate to be overwritten dynamically, if needed
		this.avatar.update();
	},

	// mouse event processors
	onMouseMove: function(e){
		// summary:
		//		event processor for onmousemove
		// e: Event
		//		mouse event
		var a = this.avatar;
		if(a){
			dojo.dnd.autoScrollNodes(e);
			//dojo.dnd.autoScroll(e);
			var s = a.node.style;
			s.left = (e.pageX + this.OFFSET_X) + "px";
			s.top  = (e.pageY + this.OFFSET_Y) + "px";
			var copy = Boolean(this.source.copyState(dojo.isCopyKey(e)));
			if(this.copy != copy){
				this._setCopyStatus(copy);
			}
		}
	},
	onMouseUp: function(e){
		// summary:
		//		event processor for onmouseup
		// e: Event
		//		mouse event
		if(this.avatar){
			if(this.target && this.canDropFlag){
				var copy = Boolean(this.source.copyState(dojo.isCopyKey(e))),
				params = [this.source, this.nodes, copy, this.target, e];
				dojo.publish("/dnd/drop/before", params);
				dojo.publish("/dnd/drop", params);
			}else{
				dojo.publish("/dnd/cancel");
			}
			this.stopDrag();
		}
	},

	// keyboard event processors
	onKeyDown: function(e){
		// summary:
		//		event processor for onkeydown:
		//		watching for CTRL for copy/move status, watching for ESCAPE to cancel the drag
		// e: Event
		//		keyboard event
		if(this.avatar){
			switch(e.keyCode){
				case dojo.keys.CTRL:
					var copy = Boolean(this.source.copyState(true));
					if(this.copy != copy){
						this._setCopyStatus(copy);
					}
					break;
				case dojo.keys.ESCAPE:
					dojo.publish("/dnd/cancel");
					this.stopDrag();
					break;
			}
		}
	},
	onKeyUp: function(e){
		// summary:
		//		event processor for onkeyup, watching for CTRL for copy/move status
		// e: Event
		//		keyboard event
		if(this.avatar && e.keyCode == dojo.keys.CTRL){
			var copy = Boolean(this.source.copyState(false));
			if(this.copy != copy){
				this._setCopyStatus(copy);
			}
		}
	},

	// utilities
	_setCopyStatus: function(copy){
		// summary:
		//		changes the copy status
		// copy: Boolean
		//		the copy status
		this.copy = copy;
		this.source._markDndStatus(this.copy);
		this.updateAvatar();
		dojo.replaceClass(dojo.body(),
			"dojoDnd" + (this.copy ? "Copy" : "Move"),
			"dojoDnd" + (this.copy ? "Move" : "Copy"));
	}
});

// dojo.dnd._manager:
//		The manager singleton variable. Can be overwritten if needed.
dojo.dnd._manager = null;

Manager.manager = dojo.dnd.manager = function(){
	// summary:
	//		Returns the current DnD manager.  Creates one if it is not created yet.
	if(!dojo.dnd._manager){
		dojo.dnd._manager = new dojo.dnd.Manager();
	}
	return dojo.dnd._manager;	// Object
};

return Manager;
});

},
'gridx/core/Core':function(){
define([
	"require",
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/_base/Deferred",
	"dojo/DeferredList",
	"./model/Model",
	"./Row",
	"./Column",
	"./Cell",
	"./_Module"
], function(require, declare, array, lang, Deferred, DeferredList, Model, Row, Column, Cell, _Module){	

	var delegate = lang.delegate,
		isFunc = lang.isFunction,
		isString = lang.isString,
		hitch = lang.hitch,
		forEach = array.forEach;

	function shallowCopy(obj){
		var ret = {}, i;
		for(i in obj){
			ret[i] = obj[i];
		}
		return ret;
	}

	function getDepends(mod){
		var p = mod.moduleClass.prototype;
		return (p.forced || []).concat(p.optional || []);
	}

	function configColumns(columns){
		var cs = {}, c, i, len;
		if(lang.isArray(columns)){
			for(i = 0, len = columns.length; i < len; ++i){
				c = columns[i];
				c.index = i;
				c.id = c.id || String(i + 1);
				cs[c.id] = c;
			}
		}
		return cs;
	}
	
	function mixinArrayUtils(arr){
		for(var f in array){
			if(isFunc(array[f])){
				arr[f] = lang.partial(array[f], arr);
			}
		}
		return arr;
	}

	function mixinAPI(base, apiPath){
		if(apiPath){
			for(var path in apiPath){
				var bp = base[path],
					ap = apiPath[path];
				if(bp && lang.isObject(bp) && !isFunc(bp)){
					mixinAPI(bp, ap);
				}else{
					base[path] = ap;
				}
			}
		}
	}

	function removeAPI(base, apiPath){
		if(apiPath){
			for(var path in apiPath){
				delete base[path];
			}
		}
	}

	function normalizeModules(args, coreMods){
		var mods = [],
			coreModCount = coreMods && coreMods.length || 0;
		forEach(args.modules, function(m, i){
			if(isFunc(m) || isString(m)){
				m = {
					moduleClass: m
				};
			}
			if(m){
				var mc = m.moduleClass;
				if(isString(mc)){
					try{
						mc = m.moduleClass = require(mc);
					}catch(e){
						console.error(e);
					}
				}
				if(isFunc(mc)){
					mods.push(m);
					return;
				}
			}
			console.error(["The ", (i + 1 - coreModCount), 
				"-th declared module can NOT be found, please require it before using it"].join(''));
		});
		args.modules = mods;
		return args;
	}
	
	function checkForced(args){
		var registeredMods = _Module._modules,
			modules = args.modules, i, j, k, p, deps, depName, err;
		for(i = 0; i < modules.length; ++i){
			p = modules[i].moduleClass.prototype;
			deps = (p.forced || []).concat(p.required || []);
			for(j = 0; j < deps.length; ++j){
				depName = deps[j];
				for(k = modules.length - 1; k >= 0; --k){
					if(modules[k].moduleClass.prototype.name === depName){
						break;
					}
				}
				if(k < 0){
					if(registeredMods[depName]){
						modules.push({
							moduleClass: registeredMods[depName]
						});
					}else{
						err = 1;	//1 as true
						console.error(["Forced/Required dependent module '", depName, 
							"' is NOT found for '", p.name, "' module."].join(''));
					}
				}
			}
		}
		if(err){
			throw new Error("Some forced/required dependent modules are NOT found.");
		}
		return args;
	}

	function removeDuplicate(args){
		var i, mods = {}, modules = [];
		forEach(args.modules, function(m){
			mods[m.moduleClass.prototype.name] = m;
		});
		for(i in mods){
			modules.push(mods[i]);
		}
		args.modules = modules;
		return args;
	}

	function checkCircle(args){
		var modules = args.modules, i, m, modName, q, key,
			getModule = function(modName){
				for(var j = modules.length - 1; j >= 0; --j){
					if(modules[j].moduleClass.prototype.name == modName){
						return modules[j];
					}
				}
				return null;
			};
		for(i = modules.length - 1; m = modules[i]; --i){
			modName = m.moduleClass.prototype.name;
			q = getDepends(m);
			while(q.length){
				key = q.shift();
				if(key == modName){
					throw new Error("Module '" + key + "' is in a dependancy circle!");
				}
				m = getModule(key);
				if(m){
					q = q.concat(getDepends(m));
				}
			}
		}
		return args;
	}

	function checkModelExtensions(args){
		var modules = args.modules,
			i, modExts;
		for(i = modules.length - 1; i >= 0; --i){
			modExts = modules[i].moduleClass.prototype.modelExtensions;
			if(modExts){
				[].push.apply(args.modelExtensions, modExts);
			}
		}
		return args;
	}

	return declare(/*===== "gridx.core.Core", =====*/[], {
		// summary:
		//		This is the logical grid (also the base class of the grid widget), 
		//		providing grid data model and defines a module/plugin framework
		//		so that the whole grid can be as flexible as possible while still convenient enough for
		//		web page developers.

		

		_reset: function(args){
			//Reset the grid data model completely. Also used in initialization.
			var t = this,
				d = t._deferStartup = new Deferred();
			args = shallowCopy(args);
			t.store = args.store;
			args.modules = args.modules || [];
			args.modelExtensions = args.modelExtensions || [];
			t.setColumns(args.structure);
			args.columns = t._columnsById;
			args = checkModelExtensions(
					checkCircle(
						removeDuplicate(
							checkForced(
								normalizeModules(args, t.coreModules)))));
			//Create model before module creation, so that all modules can use the logic grid from very beginning.
			t.model = new Model(args);
			t.when = lang.hitch(t.model, t.model.when);
			t._create(args);
			t._preload();
			t._load(d).then(hitch(t, 'onModulesLoaded'));
		},

		onModulesLoaded: function(){
			// summary:
			//		Fired when all grid modules are loaded. Can be used as a signal of grid creation complete.
			// tags:
			//		callback
		},

		
		setStore: function(store){
			// summary:
			//		Change the store for grid.
			// store: dojo.data.*|dojox.data.*|dojo.store.*
			//		The new data store
			if(this.store != store){
				this.store = store;
				this.model.setStore(store);
			}
		},

		
		setColumns: function(columns){
			// summary:
			//		Change all the column definitions for grid.
			// columns: Array
			//		The new column structure
			var t = this;
			t.structure = columns;
			t._columns = lang.clone(columns);
			t._columnsById = configColumns(t._columns);
			if(t.model){
				t.model._cache.onSetColumns(t._columnsById);
			}
		},

		
		row: function(row, isId, parentId){
			// summary:
			//		Get a row object by ID or index.
			//		For asyc store, if the data of this row is not in cache, then null will be returned.
			// row: Integer|String
			//		Row index of row ID
			// isId: Boolean?
			//		If the row parameter is a numeric ID, set this to true
			// returns:
			//		If the params are valid and row data is in cache, return a row object, else return null.
			var t = this;
			if(typeof row == "number" && !isId){
				row = t.model.indexToId(row, parentId);
			}
			if(t.model.byId(row)){
				t._rowObj = t._rowObj || t._mixin(new Row(t), "row");
				return delegate(t._rowObj, {	//gridx.core.Row
					id: row
				});
			}
			return null;	//null
		},

		
		column: function(column, isId){
			// summary:
			//		Get a column object by ID or index
			// column: Integer|String
			//		Column index or column ID
			// isId: Boolean
			//		If the column parameter is a numeric ID, set this to true
			// returns:
			//		If the params are valid return a column object, else return NULL
			var t = this, c, a, obj = {};
			if(typeof column == "number" && !isId){
				c = t._columns[column];
				column = c && c.id;
			}
			c = t._columnsById[column];
			if(c){
				t._colObj = t._colObj || t._mixin(new Column(t), "column");
				for(a in c){
					if(t._colObj[a] === undefined){
						obj[a] = c[a];
					}
				}
				return delegate(t._colObj, obj);	//gridx.core.Column
			}
			return null;	//null
		},

		
		cell: function(row, column, isId, parentId){
			// summary:
			//		Get a cell object
			// row: gridx.core.Row|Integer|String
			//		Row index or row ID or a row object
			// column: gridx.core.Column|Integer|String
			//		Column index or column ID or a column object
			// isId: Boolean?
			//		If the row and coumn params are numeric IDs, set this to true
			// returns:
			//		If the params are valid and the row is in cache return a cell object, else return null.
			var t = this, r = row instanceof Row ? row : t.row(row, isId, parentId);
			if(r){
				var c = column instanceof Column ? column : t.column(column, isId);
				if(c){
					t._cellObj = t._cellObj || t._mixin(new Cell(t), "cell");
					return delegate(t._cellObj, {	//gridx.core.Cell
						row: r,
						column: c
					});
				}
			}
			return null;	//null
		},

		
		columnCount: function(){
			// summary:
			//		Get the number of columns
			// returns:
			//		The count of columns
			return this._columns.length;	//Integer
		},

		
		rowCount: function(parentId){
			// summary:
			//		Get the number of rows.
			// description:
			//		For async store, the return value is valid only when the grid has fetched something from the store.
			// parentId: String?
			//		If provided, return the child count of the given parent row.
			// returns:
			//		The count of rows. -1 if the size info is not available (using server side store and never fetched any data)
			return this.model.size(parentId);	//Integer
		},

		
		columns: function(start, count){
			// summary:
			//		Get a range of columns, from index 'start' to index 'start + count'.
			// start: Integer?
			//		The index of the first column in the returned array.
			//		If omitted, defaults to 0, so grid.columns() gets all the columns.
			// count: Integer?
			//		The number of columns to return.
			//		If omitted, all the columns starting from 'start' will be returned.
			// returns:
			//		An array of column objects
			return this._arr(this._columns.length, 'column', start, count);	//gridx.core.Column[]
		},

		
		rows: function(start, count, parentId){
			// summary:
			//		Get a range of rows, from index 'start' to index 'start + count'.
			// description:
			//		For async store, if some rows are not in cache, then there will be NULLs in the returned array.
			// start: Integer?
			//		The index of the first row in the returned array.
			//		If omitted, defaults to 0, so grid.rows() gets all the rows.
			// count: Integer?
			//		The number of rows to return.
			//		If omitted, all the rows starting from 'start' will be returned.
			// returns:
			//		An array of row objects
			return this._arr(this.rowCount(parentId), 'row', start, count, parentId);	//gridx.core.Row[]
		},
		
		//Private-------------------------------------------------------------------------------------
		_uninit: function(){
			var t = this, mods = t._modules, m;
			for(m in mods){
				m = mods[m].mod;
				if(m.getAPIPath){
					removeAPI(t, m.getAPIPath());
				}
				m.destroy();
			}
			if(t.model){
				t.model.destroy();
			}
		},

		_arr: function(total, type, start, count, pid){
			var i = start || 0, end = count >= 0 ? start + count : total, r = [];
			for(; i < end && i < total; ++i){
				r.push(this[type](i, 0, pid));
			}
			return mixinArrayUtils(r);
		},
		
		_preload: function(){
			var m, mods = this._modules;
			for(m in mods){
				m = mods[m];
				if(m.mod.preload){
					m.mod.preload(m.args);
				}
			}
		},

		_load: function(deferredStartup){
			var dl = [], m;
			for(m in this._modules){
				dl.push(this._initMod(deferredStartup, m));
			}
			return new DeferredList(dl, 0, 1);
		},
		
		_mixin: function(component, name){
			var m, a, mods = this._modules;
			for(m in mods){
				m = mods[m].mod;
				a = m[name + 'Mixin'];
				if(isFunc(a)){
					a = a.apply(m);
				}
				lang.mixin(component, a || {});
			}
			return component;
		},
	
		_create: function(args){
			var t = this,
				mods = t._modules = {};
			forEach(args.modules, function(mod){
				var m, key = mod.moduleClass.prototype.name;
				if(!mods[key]){
					mods[key] = {
						args: mod,
						mod: m = new mod.moduleClass(t, mod),
						deps: getDepends(mod)
					};
					if(m.getAPIPath){
						mixinAPI(t, m.getAPIPath());
					}
				}
			});
		},

		_initMod: function(deferredStartup, key){
			var t = this,
				mods = t._modules,
				m = mods[key],
				mod = m.mod,
				d = mod.loaded;
			if(!m.done){
				m.done = 1;
				new DeferredList(array.map(array.filter(m.deps, function(depModName){
					return mods[depModName];
				}), hitch(t, t._initMod, deferredStartup)), 0, 1).then(function(){
					if(mod.load){
						mod.load(m.args, deferredStartup);
					}else if(d.fired < 0){
						d.callback();
					}
				});
			}
			return d;
		}
	});
});

},
'ibm/tivoli/tip/navwidget/dojo/NavWidgetUtil':function(){
// wrapped by build app
define("ibm/tivoli/tip/navwidget/dojo/NavWidgetUtil", ["dijit","dojo","dojox"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * "Restricted Materials of IBM"
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil");

/*=====
ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil = {
	// summary: Miscellaneous utilities
};
=====*/

//initializing variable for tab index values to specify a starting index
var _tabIndex=100;
// a multiplication factor to have different starting index for each of the views, 100,200,300 etc 
var _multIndex=1;

ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil");

ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.constants = {
	THEMES_NAME: "themes/",
	THEMES_FULL_PATH: dojo.config.cacheBust ? dojo.moduleUrl("ibm.tivoli.tip.navwidget", "themes/").slice(0,dojo.moduleUrl("ibm.tivoli.tip.navwidget", "themes/").indexOf("?")) :
		                                      dojo.moduleUrl("ibm.tivoli.tip.navwidget", "themes/"),   // 256005
	WIDGET_TAG : "[widget]/"
};

ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.cleanUpImagePath = function(/*String*/ imagePath,/*String?*/ basePath){
	// summary:
	//		Coming soon
	// description:
	//		Coming soon
	// imagePath:
	//		Coming soon
	// tags:
	//		public
	
	var returnVal = imagePath;
	
	//If we find "[widget]/", we want to assume the provider wants the image retrieved from the widget's default relative location, so let's replace
	//the "[widget]/" string with the right base path
	if (imagePath) {
		var widgetTagStartIndex = imagePath.indexOf(ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.constants.WIDGET_TAG);
		if (widgetTagStartIndex == 0) {
			var relativePath = imagePath.substring(ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.constants.WIDGET_TAG.length);
			if (basePath) {
				returnVal = basePath + ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.constants.THEMES_NAME + relativePath;
			} else {
				returnVal = ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.constants.THEMES_FULL_PATH + relativePath;
			}
		}
        // remove 7 lines for defect 44689 (no need to check for leading "/" on the imagePath				
		if (dojo.config.cacheBust) {
            returnVal = returnVal + "?" + dojo.config.cacheBust;
        }
	}
	return returnVal;
};

ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getObjectInstance = function(/*String*/ fullClassName, /*Object*/ args){
	// summary:
	//		Coming soon
	// description:
	//		Coming soon
	// fullClassName:
	//		Coming soon
	// args:
	//		Coming soon
	// tags:
	//		public

	var returnVal = null;
	
	dojo["require"](fullClassName);
	var objectClass = dojo.getObject(fullClassName);
	if (objectClass) {
		returnVal = new objectClass(args);
	} else {
		if (ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil._trace.isError()) {
			var methodName = "ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getObjectInstance";
			ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil._trace.error("Could not load " + fullClassName, methodName);
		}
	}

	return returnVal;
};

ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.createWidgetDefinition = function(/*String*/ widgetDefClassName, /*String*/ widgetDefId, /*Object*/ widgetDefInfo, /*ibm.tivoli.navmodel.Dataset*/ dataset, /*ibm.tivoli.navmodel.WidgetDef?*/ parentWidgetDef){
	// summary:
	//		Coming soon
	// description:
	//		Coming soon
	// widgetDefClassName:
	//		Coming soon
	// widgetDefId:
	//		Coming soon
	// widgetDefInfo:
	//		Coming soon
	// dataset:
	//		Coming soon	
	// parentWidgetDef:
	//		Coming soon	
	// tags:
	//		public
	
	var methodName = "createWidgetDefinition";
	
	var widgetDefinition = null;
	var baseUrl = null;
	var provider = null;
	var datasource = null;
	var profileID = null;

	if (dataset) {
		baseUrl = dataset.getArgs().baseUrl;
		provider = dataset.getProviderId();
		datasource = dataset._args.datasource;
	} else {
		baseUrl = widgetDefInfo.baseUrl;
		provider = widgetDefInfo.provider;
		datasource = widgetDefInfo.datasource;
		profileID = widgetDefInfo.profileID;
	}
		
	var args = {
		baseUrl: baseUrl,
        	provider: provider,
        	datasource: datasource,
        	widget: widgetDefInfo.widget,
        	definition: widgetDefId,
		profileID: profileID,	 
		parentWidgetDef: parentWidgetDef
	};
	try {
		widgetDefinition = ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getObjectInstance(widgetDefClassName, args);		
	} catch (exception){
		// Check specifically for case where cannot connect to provider
		if (exception && exception.getMsgId() == "ATKRST102E") {
			if (ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil._trace.isWarn()) {
				ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil._trace.warn(exception.getMsgText(), methodName);
			}
			throw exception;
		}
		if (ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil._trace.isWarn()) {
			ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil._trace.warn("Specified widget definition cannot be found. Using default one. args = " + dojo.toJson(args), methodName);
		}
		
		var args = {
			baseUrl: widgetDefInfo.baseUrl,
            	provider: widgetDefInfo.provider,
            	datasource: widgetDefInfo.datasource,
            	widget: widgetDefInfo.widget,
            	definition: widgetDefInfo.definition,
			profileID: widgetDefInfo.profileID,
			parentWidgetDef: parentWidgetDef
		};
		try {
			widgetDefinition = ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getObjectInstance(widgetDefClassName, args);				
		} catch (exception){
			if (ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil._trace.isWarn()) {
				ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil._trace.warn("Default widget definition cannot be found. Using default settings. Attempted args = " + dojo.toJson(args), methodName);
			}
		};
	};
	
	return widgetDefinition;
};

ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getItemByAbsoluteId = function(/*ibm.tivoli.navmodel.Dataset*/ rootDataset, /*String*/ absItemId){
	// summary:
	//		Finds an Item by absolute id.
	// description:
	//		Finds an Item by absolute id.
	// rootDataset:
	//		The top-level dataset to start the search from.
	// absItemId:
	//		The absolute item id is a ":" delimited list of ids of the form "CHILD_DATASET_A:CHILD_DATASET1:...:ITEM_1". The last
	// 		entry is the item id. Entries leading up to the last entry allow us to walk to the child dataset tree from the root dataset
	//      to find the child dataset containing the item.
	// tags:
	//		public

	//Get the item out of whatever dataset we determined contains the item based on the absolute id
	var item = rootDataset.getItem({itemId: absItemId});

	return item;
};

ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getLinkByAbsoluteId = function(/*ibm.tivoli.navmodel.Dataset*/ rootDataset, /*String*/ absLinkId){
	//	summary:
	//		Finds an Link by absolute id.
	//	description:
	//		Finds an Link by absolute id.
	//	rootDataset:
	//		The top-level dataset to start the search from.
	//  absLinkId:
	//		The absolute link id is a ":" delimited list of ids of the form "CHILD_DATASET_A:CHILD_DATASET1:...:LINK_1". The last
	// 		entry is the link id. Entries leading up to the last entry allow us to walk to the child dataset tree from the root dataset
	//      to find the child dataset containing the link.
	//  tags:
	//		public

	//Get the link out of whatever dataset we determined contains the link based on the absolute id
	var link = rootDataset.getLink({linkId: absLinkId});

	return link;
};

ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getRelationshipByAbsoluteId = function(/*ibm.tivoli.navmodel.Dataset*/ rootDataset, /*String*/ absRelationshipId){
	// summary:
	//		Finds a Relationship by absolute id.
	// description:
	//		Finds a Relationship by absolute id.
	// rootDataset:
	//		The top-level dataset to start the search from.
	// absRelationshipId:
	//		The absolute relationship id is a ":" delimited list of ids of the form "CHILD_DATASET_A:CHILD_DATASET1:...:RELATIONSHIP_1". The last
	// 		entry is the relationship id. Entries leading up to the last entry allow us to walk to the child dataset tree from the root dataset
	//      to find the child dataset containing the relationship.
	// tags:
	//		public

    //Get the relationship out of whatever dataset we determined contains the relationship based on the absolute id
	var relationship = rootDataset.getRelationship({relationshipId: absRelationshipId});

	return relationship;
};

ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getItemAbsoluteId = function(/*ibm.tivoli.navmodel.Item*/ item){
	// summary:
	//		calculates absolute id of an item
	// item:
	//		An item
	// description:
	//		calculates absolute id of an item
	// tags:
	//		public

	//Return id
	return ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil._getItemOrLinkOrRelationshipAbsoluteId(item);
};

ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getLinkAbsoluteId = function(/*ibm.tivoli.navmodel.Link*/ link){
	// summary:
	//		calculates absolute id of a link
	// link:
	//		A link
	// description:
	//		calculates absolute id of a link
	// tags:
	//		public

	//Return id
	return ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil._getItemOrLinkOrRelationshipAbsoluteId(link);
};

ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getRelationshipAbsoluteId = function(/*ibm.tivoli.navmodel.Relationship*/ relationship){
	// summary:
	//		calculates absolute id of a relationship
	// link:
	//		A relationship
	// description:
	//		calculates absolute id of a relationship
	// tags:
	//		public

	//Return id
	return ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil._getItemOrLinkOrRelationshipAbsoluteId(relationship);
};

ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil._getItemOrLinkOrRelationshipAbsoluteId = function(/*Object*/ object){
	// summary:
	//		calculates absolute id of an item or link
	// object:
	//		An item or a link
	// description:
	//		calculates absolute id of an item or link
	// tags:
	//		private

	//Initialize the abs id
	var absId = object.getId();
	
	//Return id
	return absId;
};

ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getStringWithEllipsis = function(/*String*/ str, /*Boolean*/isAltTitle){
	// summary:
	//		For whatever reason, it was only discovered after PII freeze that we had no ellipses
	//		next to strings in our resource bundle. This is a temporary approach to add ellipses
	//		while avoiding string concatenation. Long-term solution is to fix the resource bundle.
	// description:
	//		For whatever reason, it was only discovered after PII freeze that we had no ellipses
	//		next to strings in our resource bundle. This is a temporary approach to add ellipses
	//		while avoiding string concatenation. Long-term solution is to fix the resource bundle.
	// upperBound:
	//		Coming soon
	// tags:
	//		public

	var templateString = "${label}";
	if (isAltTitle) {
		templateString += "..."; //Button flyovers don't seem to deal very well with HTML char codes like &hellip;
	} else {
		templateString += "&hellip;";
	}
	return dojo.string.substitute(templateString, {label: str});
};

ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getNextRandomInt = function(/*int*/ upperBound){
	// summary:
	//		Generates random int value between 0 (inclusive) and upperBound (exclusive)
	// description:
	//		Generates random int value between 0 (inclusive) and upperBound (exclusive)
	// upperBound:
	//		Coming soon
	// tags:
	//		public

	var rand = Math.floor(upperBound*Math.random());
	return rand;
};

ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getNextRandomBoolean = function(){
	// summary:
	//		Generates true or false (like a coin flip)
	// description:
	//		Generates true or false (like a coin flip)
	// tags:
	//		public

	var rand = Math.random();
	return rand >= 0.5;
};


//for tab index values
ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.tabIndexFunc = function(){
	// Summary:
	//		Assigning different tab index values to start with for each view
	// Description:
	// 		Assigning different tab index values to start with for each view
	// tags:
	//  	public
		
	return ((_multIndex++) * (_tabIndex));
};

// Replace special characters with a given replaceChar
ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.replaceSpecialChars = function(/*Char*/ replaceChar, /*String*/ inputString) {  
	if (inputString && replaceChar) {
		inputString =  inputString.replace(/[^a-zA-Z 0-9]+/g,replaceChar);
	}
	return inputString;
};


});

},
'url:dijit/templates/Tooltip.html':"<div class=\"dijitTooltip dijitTooltipLeft\" id=\"dojoTooltip\"\n\t><div class=\"dijitTooltipContainer dijitTooltipContents\" data-dojo-attach-point=\"containerNode\" role='alert'></div\n\t><div class=\"dijitTooltipConnector\" data-dojo-attach-point=\"connectorNode\"></div\n></div>\n",
'ibm/tivoli/tip/navwidget/dijit/AsynchContextMenu':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/i18n!ibm/tivoli/tip/dijit/nls/resources","dojo/require!dijit/Menu,dijit/MenuItem,ibm/tivoli/tip/dijit/TIPContextMenu,ibm/tivoli/tip/dijit/TIPContextMenuProvider"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * "Restricted Materials of IBM"
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.dijit.AsynchContextMenu");

dojo.require("dijit.Menu");
dojo.require("dijit.MenuItem");
dojo.require("ibm.tivoli.tip.dijit.TIPContextMenu"); 
dojo.require("ibm.tivoli.tip.dijit.TIPContextMenuProvider");

dojo.requireLocalization("ibm.tivoli.tip.dijit", "resources");

dojo.declare("ibm.tivoli.tip.navwidget.dijit.AsynchContextMenu",
        ibm.tivoli.tip.dijit.TIPContextMenu,
{		
	// summary:
	//		Coming soon
	// description:
	//		Coming soon
	
	loadingMenuItemID: "LOADING_ID",
	processingLabel: null,
	
	constructor: function(){
        this.processingLabel = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources").PROCESSING;
    },
    
    _destroyMenu: function() {  // 37703
        this.inherited(arguments);
        this.cmProvider.cancelPendingRequest();
    },    
    
 	_loadMenuItems: function() {
		// summary:
		//		If the menu is currently empty (i.e., just created), return the placeholder
		//		menu item and tell the asynch provider to get the menu items asynchronously,
		//		otherwise, remove the placeholder and ask the provider for the menu items
		//		now that the asynch request has returned the menu items.
		var items = null;
		var contextMenu = this.getMenu();
		if (contextMenu != null) {
			var childItems = contextMenu.getChildren();
			if (childItems.length == 0) {
				items = [{ 
        				"label": this.processingLabel,
        				"iconClass": "tipMenuIcon loadingIcon", 
        				"actionID": this.loadingMenuItemID,
        				"launchType": "NOOP"
        				}];
				this.cmProvider.getMenuItemsAsynch(this._context, dojo.hitch(this, this._menuItemsLoaded));
			}
			else {
				this._removeLoadingItem();
				items = this.cmProvider.getMenuItems(this._context);
			}
		}
		return items;
 	},
 	
 	_menuItemsLoaded: function(/*Boolean*/ success) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
 		// success:
 		//		
 		
		var menu = this.getMenu();
 		if (success) {
 			// repopulate menu
 			this.populateMenu(menu);
 			// the menu is already marked as started, so populateMenu skips these
 			// steps, so do them again here
 			dojo.forEach(menu.getChildren(), function(child){ child.startup(); });
 			menu.startupKeyNavChildren();
 			// The dijit.Menu still remembers the "Loading..." item as focused so reset
 	        menu.focusedChild = null;
 	        menu.focus();
 		}
 	    else {
 	    	// There were either no tasks returned or an error occurred, so dismiss the context menu
 	    	dijit.popup.close(menu);
 	    }
 	},
 	
 	_removeLoadingItem: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
 		var contextMenu = this.getMenu();
	    var childItems = contextMenu.getChildren();
 	    var loadingItem = childItems[0];
 	 	contextMenu.removeChild(loadingItem);
 	 	loadingItem.destroy();
 	}
	
	
});       // 256005  Split AsynchContextMenuProvider and RESTContextMenuProvider to separate files for Dojo 1.7 rules, one module per file


});

},
'dojoe/treetablex/TreeTable':function(){
define([
	"dojo/_base/declare",
	"dijit/_Widget",
	"dijit/_TemplatedMixin",
	"gridx/Grid",
	"gridx/core/model/cache/Async",
	"gridx/core/model/extensions/Query",
	"gridx/modules/Persist",
	"gridx/modules/Tree",
	"gridx/modules/VirtualVScroller",
	"gridx/modules/ColumnResizer",
	"gridx/modules/SingleSort",
	"gridx/modules/select/Row",
	"gridx/modules/select/Column",
	"gridx/modules/extendedSelect/Row",
	"gridx/modules/extendedSelect/Column",
	"gridx/modules/IndirectSelect",
	"gridx/modules/RowHeader",
	"gridx/modules/filter/Filter",
	"gridx/modules/filter/FilterBar",
	"dojo/dom-geometry",
	"dojoe/treetablex/_Toolbar",
	"dojoe/slidingpane/SlidingPane",
	"dojoe/treetablex/modules/StatefulRefresh",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/window",
	"dojo/dom-style",
	"dojo/i18n!dojoe/treetablex/nls/TreeTable",
	"dojo/text!./templates/TreeTable.html"
], function (declare, _Widget, _TemplatedMixin, Gridx, Cache, Query, Persist, Tree, VirtualVScroller, ColumnResizer, SingleSort, SelectRow, SelectColumn, ExtSelectRow, ExtSelectColumn, IndirectSelect, RowHeader, Filter, FilterBar, domGeom , _Toolbar, SlidingPane, StatefulRefresh, array, Lang, Window, domStyle, _resources, template) {

	//summary: 
	//		Tree table is a hybrid widget of a tree and table 
	//description: 
	//		TreeTable widget implementation is based gridx
	//		Provides additional features upon gridx like toolbar with state control buttons (refresh) and custom actions dropdown etc

	TreeTable = declare('dojoe.treetablex.TreeTable', [_Widget, _TemplatedMixin], {
	
		//summary: Set this to true if the widget contains other widgets
		//tags: private
		widgetsInTemplate : true,
		
		//summary: Path to the template
		//tags: private
		templateString: template,
		
		// _grid: Object
		//		Reference to the gridx used
		_grid: null,
		
		// gridParams_: object
		//		holds all the parameters passed to the gridx by the user.
		gridParams_: null,
		
		// filterOptions:  Object
		//		Holds all the filter information passed by the user.
		filterOptions : {},
		
		// mods: object
		//		holds the modules passed to the gridx.
		_modules: [ Tree, VirtualVScroller, SelectRow, SelectColumn, SingleSort, ColumnResizer],
		
		// mods: object
		//		holds the filter modules passed to the gridx.
		_filterModules: null,
		
		// mods: object
		//		holds the indirect selection modules and its dependencies passed to the gridx.
		_indirectSelectModules: null,
		
		// toolbarVisible: boolean
		//		control the _toolbar show/hide
		toolbarVisible:true,
		
		// enableCollapsibleToolbar: boolean
		//		To add collapsible toolbar  to the table
		enableCollapsibleToolbar : false,
		
		// toolbarExpand: boolean
		//		To indicate(preset) the state of the toolbar on load
		toolbarExpand : false,
		
		//enableResize: boolean
		//		To disable table resize on window resize event.Default true, set false to disable table resize on window resize event	
		enableResize:true,		
		
		// width: number in px or % or em
		//		holds the width of the grid
		width:'',
		
		// height: number in px or % or em
		//		holds the height of the grid
		height:'',
		
		// _defaultHeight: number in %
		//		holds the default height of the grid
		_defaultHeight: '100%',
		
		// _defaultWidth: number in % 
		//		holds the default width of the grid
		_defaultWidth: '100%',
		
		//summary: Reference to the toolbar
		//tags: private
		_toolbar:null,
		
		// toolbarOptions: object
		//	holds the list of values for enabling/disabling toolbar options
		toolbarOptions:null,
	
		// state: object
		//		Tree Grid state object
		state: null,
		
		// enableColumnReorder: boolean
		//		To enable column reorder, by default its true
		enableColumnReorder:true,
		
		// indirectSelection: boolean
		//		Set tree table with basic Selection, if set to true enables Indirect selection Single and multiple grid selection modes
		indirectSelection: false,
		
		//menu: object
		// 		To handle custom menus
		menu: null,
		
		//summary: keeps track of the menu event object which provides the context value when right clicked
		onContextMenuEvtObj: null,
		
		// _originalStructure: original Structure set to the table
		// tags: private
		_origLayout : null,
		
		// _currentLayout: layout object after filtering columns with hidden parameters
		// tags: private
		_currentLayout : null,
		
		//_selectionMode: string
		// 		To handle selection mode. by default it is single 
		_selectionMode : "single",
		
		constructor: function(args){
			// summary: Constructor
			this.gridParams_ = {};
			this.actionList = {};
			//this._modules = [ Tree, VirtualVScroller, SelectRow, SelectColumn, SingleSort, ColumnResizer, Persist];
			this._filterModules = [Filter, FilterBar];
			this._indirectSelectModules = [IndirectSelect, RowHeader];
		},
		
		postCreate: function(){
			//summary: 
			//		creates and initialize the tree table 
		
			this._mapGridParams();
			this._defineModules();
			// set statefulRefresh in gridparams if the expansion state needs to be persisted during dynamic updates..
			if(this.gridParams_.statefulRefresh){
				this._modules.push(Persist);
				this._modules.push(StatefulRefresh);
			
			}
			if( !this.gridParams_.autoWidth){
				domStyle.set(this.domNode, "width", this.width);
			}
			if( !this.gridParams_.autoHeight){
				domStyle.set(this.domNode, "height", this.height);
			}
			this._makeGrid();
			// set statefulRefresh in gridparams if the expansion state needs to be persisted during dynamic updates..
			// statefulRefresh is a new module on Gridx which when loaded enables persisting the expansion state
			
			if(this.gridParams_.statefulRefresh){
				this.connect(this, "onRefresh",dojo.hitch(this._grid.statefulRefresh,"_applyTreeState")) ;
				
			}
			if(this.menu !== null){
				this.connect(this._grid, "onRowContextMenu", dojo.hitch(this, "showRowContextMenu"));
			}	
			if(this.toolbarVisible ){
				this._toolbar = new _Toolbar({treeTable:this}, this.toolbarNode);
				
				if (this.enableCollapsibleToolbar) {
					this.slidingPane = new SlidingPane({closeNodeClass: "slidingPaneClose", open: this.toolbarExpand}, this.wipeNode);
					this.connect(this.slidingPane, "onClick", dojo.hitch(this, "slidingPaneClicked"));	//Connecting event onClick to sliding pane.
				}
				
			} else {
				if(this.filterOptions.hideFilterBar == true && this._grid.filterBar !== undefined){
					this._grid.filterBar.hide();
				} 
			}
			this.resize();
			if(this.enableResize === true) {
				this.connect(window, "onresize", dojo.hitch(this, "resize"));			
			}
			
			if(this._toolbar && this._toolbar.actionListWidget){	
				this.connect(this._toolbar.actionListWidget, 'openDropDown', dojo.hitch(this,"onActionListOpen")); 			
				this.connect(this._toolbar.actionListWidget, 'closeDropDown', dojo.hitch(this,"onActionListClose")); 
			}
			
			if(this.gridParams_.preFilter){
				dojo.connect(this, 'onTreeTableLoad', dojo.hitch(this,"_preFilter"));
			}
			setTimeout(dojo.hitch(this, "onTreeTableLoad", this), 1000);
		},
		onTreeTableLoad: function(treetable){
			// summary: 
			//		Called after the tree Table loads
			// tags:
			//      callback
			console.info("TreeTable load complete....."+treetable.id);
		},
		showRowContextMenu: function(e){
			//summary: 
			//		Associates row context menu with tree table and also returns the contextEvent Object
			//tags: public
			e.preventDefault();
			if(this.menu){
				this.menu._openMyself({
					target: e.target,
					coords: "pageX" in e ? {
						x: e.pageX,
						y: e.pageY
					} : null
				});
			}
			this.onContextMenuEvtObj = e;
			return e;
		},	
		
		_defineModules: function() {
			this._modules = this._modules.concat(this._filterModules);
			if(this._selectionMode == 'single') {
				this._modules = this._modules.concat({moduleClass: SelectRow, multiple : false, treeMode: false, triggerOnCell : true});
			} else if(this._selectionMode == 'multiple') {
				this._modules = this._modules.concat([ExtSelectRow, ExtSelectColumn]);	
				this._modules = this._modules.concat({moduleClass: ExtSelectRow, triggerOnCell : true});
			}
			if(this.gridParams_.modules) {
				this._modules = this._modules.concat(this.gridParams_.modules);			
			}
			if(this.indirectSelection) {
				this._modules = this._modules.concat(this._indirectSelectModules);			
			}
		},
		
		_mapGridParams: function() {
			//if no width is given default is set
			if(this.width ===''&& !this.gridParams_.autoWidth){
				this.width = this._defaultWidth;
			}
			//if no height is given default is set
			if(this.height ==='' && !this.gridParams_.autoHeight){
				this.height = this._defaultHeight;
			}
			if(this.gridParams_.indirectSelection){
				this.indirectSelection=this.gridParams_.indirectSelection;
			}
			if(this.gridParams_.selectionMode){
				this._selectionMode=this.gridParams_.selectionMode;
			}
			
		},
		
		slidingPaneClicked : function () {
			// summary:
			//		callback when collapsible tool bar sliding pane is clicked
			// tags: callback
			this.resize();
		},
		
		_makeGrid: function(){
			
			// summary: 
			//		creates and initializes gridx
			// tags: private
			
			var store =  this.gridParams_.store;
			if(!this.gridParams_.cacheClass){
				this.gridParams_.cacheClass = Cache;
			}
			this.gridParams_.modules = this._modules;
			if(!this.gridParams_.modelExtensions){
				this.gridParams_.modelExtensions = [Query];
			}
	
			this._grid=new Gridx(this.gridParams_,this.treeGridNode);
			if (!("hasChildren" in store)) {	
					store.hasChildren = function(id, item){
						return item && store.getValues(item, 'children').length;
					};
			}
			if (!("getChildren" in store)) {	
					store.getChildren = function(item){
						return store.getValues(item, 'children');
					};
			}
			
		/*	if(!this.gridParams_.noDataMessage){
				this.grid.noDataMessage=this._resources.NO_DATA;
			}
			else{
				this.grid.noDataMessage=this.gridParams_.noDataMessage;
			} */
		
			this._grid.startup();	
			//set original column structure to keep a reference of overall structure including hidden columns
			if(this.gridParams_.structure && !this._origLayout){
				this._origLayout = this.gridParams_.structure;
						
			}
			// set filtered structure in case hidden parameters are present
			this._filterHiddenColumns(this._origLayout);
				
			
		},
		_filterHiddenColumns: function (structure){
			// summary: Returns the layout after filtering hidden column definition
			// tags: private
			// returns: layout object
			var filteredColumns = [];
			if(Lang.isArray(structure)){
				for(var i in structure) {
					var column = structure[i];
					
					if(!(column.hidden && column.hidden == true) ){
						filteredColumns.push(column);
					}else if(i==0 || column.expandLevel == 1){
						//root node do not hide it...
						filteredColumns.push(column);
					}
				}
			}
			if(filteredColumns.length >0 )this._currentLayout = filteredColumns;
			this._grid.setColumns(this._currentLayout);
			
		},
		
		resize: function (event) {
			// summary:  
			//		Resizes the tree table. Also takes care of window resize events
			// tags:  public
			// event: object
			//		window resize event
			
			if (!this._grid || !this.domNode || !this.domNode.parentNode || this.domNode.parentNode.nodeType != 1) {  //Javascript error in resize() sometimes on IE8
				return;
			}
			
			if (!this._grid.autoWidth) {
			domStyle.set(this.domNode, "width", this.width);
			} else {
				domStyle.set(this.domNode, "width", domGeom.getMarginBox(domNode).w + 'px');
			}
			if (!this._grid.autoHeight) {
			domStyle.set(this.domNode, "height", this.height);
			} else {
				domStyle.set(this.domNode, "height", domGeom.getMarginBox(domNode).h + 'px');
			}
			
			var resizeEvent = false;
			if(event && event.currentTarget){
				resizeEvent = true;
			}
			
			var heightPx = this.height;
			var size = Window.getBox();
				
			if (!(this.domNode.parentNode.style && this.domNode.parentNode.style.height)) {
				var sizeType = this.height.substring(this.height.length - 1, this.height.length);
				//if the height is given in % then translate it to pixel
				if (sizeType == "%") {
					//convert to px
					heightPx = size.h * ((parseInt(this.height.slice(0, -1))) / 100);
					heightPx = heightPx + "px";
				}
			}
		
			domStyle.set(this.domNode, "height", heightPx);
			setTimeout(dojo.hitch(this, function(){this._delayResize(event);}), 100);
		},
		
		_delayResize: function (event) {
			// summary:  
			//		invoked with delay for grid resize.
			// tags: private
			
					
			/*using marginBox to include margins/borders of the domNode*/
			var tableSize =  domGeom.getMarginBox(this.domNode);
					
			if (this._toolbar) {
				/*calculating toolbarsize including margins and borders as they contibute to some height*/
				var toolbarSize = domGeom.getMarginBox(this._toolbar.domNode);
				//removing grid padding height which will be later added when pagination bar is placed and grid view node is resized.
				tableSize.h = tableSize.h - toolbarSize.h;				
			}
			if((tableSize.h <= 0 || tableSize.w <= 0 )){
				return;
			}
			if(event && event.currentTarget){
				this._grid.resize({
					h : tableSize.h,
					w : tableSize.w
				});
			}else{
				this._grid.resize({
					h : tableSize.h,
					w : tableSize.w
				});
			}
			
			if(this.enableCollapsibleToolbar) {
						if(this.slidingPane!=null && !this.slidingPane.open) {
							tableSize.h = tableSize.h + toolbarSize.h ;
							this._grid.resize({
								h : tableSize.h,
								w : tableSize.w
							});
						}
			} 
		},
		
		onActionListOpen : function(){
			// summary:
			//		Callback when toolbar Action list is opened
			// tags: public
		
			//console.info("actionListOpen.....");
			
		},
		
		onActionListClose : function(){
			// summary:
			//		Callback when toolbar Action list is closed
			// tags: public
			
			//console.info("actionListClosed.....");
			
		},
		
		_preFilter : function(){
			// summary: 
			//		Private function to set "gridparams.preFilter" value when tree table loads
			// tags: private
			
			if (this.gridParams_.preFilter) {
					if(this.gridParams_.preFilter[0].column==='any'){
						//is.grid._cleanup(); 
						//this.grid._clearData();
						if(this.gridParams_.preFilter[0].value){
							this.setFilter(this.gridParams_.preFilter[0].value);
						}
					}
			}
		},
		
		setFilter : function (filterText){
			//summary:
			//		set filter value programmatically
			// tags: public
			if(this._toolbar){
				if(this._toolbar.quickFilter){
					this._toolbar.quickFilter._setFilter(filterText);
				}
			}
		},
		
		getFilter : function(){
			// summary:
			//		get filter value
			// tags: public
			
			if(this._toolbar){
				if(this._toolbar.quickFilter){
					return this._toolbar.quickFilter._getFilter();
				}
			}
		},
		
		addToActionList: function(item) {
			// summary: 
			//		To add a new item to the toolbar action menu
			// tags: public
			
			if(this._toolbar) {
				if(this._toolbar.actionMenu) {
					this._toolbar.actionListWidget.dropDown.addChild(item,0);
				}
			}
		},
		
		addToToolbar : function (item, position, row, noResize) {
			// summary:  
			//		Add an item to the toolbar
			// tags: public
			// item:  object   
			//		The item that needs to be added
			// position:  number 
			//		The position of the item
			// row:  number 
			//		The row number to which the item needs to be added
			// noResize: boolean 
			//		To control whether a table resize should be done or not
		
			if (this._toolbar) {
				if (item) {
					this._toolbar._addCustomControl(item, position, row);
				}
				if (noResize == false){
				} else {
				this.resize();	
				}
			}
		}, 
		
		addItemsToToolbar : function (items, position, row, noResize) {
			// summary:  
			//		Add a list of items to the toolbar
			// tags: public
			// items:  array   
			//		The items that needs to be added
			// position:  number 
			//		The position of the starting item 
			// row:  number 
			//		The row number to which the item needs to be added
			// noResize: boolean 
			//		To control whether a table resize should be done or not
		
			if (this._toolbar) {
				for (var i = 0; i < items.length; i++) {		
					if (items[i]) {
						this._toolbar._addCustomControl(items[i], position + i, row);
					}
				}
				if (noResize == false){
				} else {
				this.resize();	
				}			
			}
		}, 		
		
		addContextMenuToTreeTable: function(position, customMenu){
			// summary: 
			//		Allows adding a context menu to the TreeTable
			//tags: public
			
			if(position == "rowMenu") {
				customMenu.bindDomNode(this.grid.domNode);
				this.grid.onRowContextMenu = function(e){				
					if(!this.menu){
						this.menu = customMenu;
					}
				};
			}
		},
		
		removeItemFromToolbar: function(itemId){
			// summary: 
			//		to remove an item from the toolbar
			// tags: public
			// itemId: String
			if(this._toolbar) {
				this._toolbar._removeItem(itemId);
			}			
		},
	
		removeFromActionList: function (menuItemId){
			// summary: 
			//		Removes a menu Item from ActionList
			// tags: public
			// menuItemId : String
			//		Id of the menuItem to be removed.
			if(this._toolbar) {
				this._toolbar._removeFromActionList(menuItemId);
			}
		},
		
		getRowCount : function(){
			// summary: 
			//		rowCount of the table
			// tags: public
			// return: number
			//		the rowCount	
			return this._grid.rowCount();
		},
		
		getSelectedRowCount : function(){
			// summary: 
			//		Gets a selected rowCount of the table
			// tags: public
			// return: number
			//		the selected rowCount
			return this._grid.select.row.getSelected().length;
		},
		
		getSelectedRows : function(){
			// summary:  
			//		Returns the selected rows from the grid.
			// tags: public
			// returns: array
			//		selected rows.
			return this._grid.select.row.getSelected();
		},	
		
		isRowSelected: function(rowId){
			// summary: 
			//		Returns true if given row is selected
			// tags: public
			
			return this._grid.select.row.isSelected(rowId);
		},
		
		disableMenuItem : function(menuid){
			// summary: 
			//		Disable a menu item of the context menu attached to the treetable.
			// tags: public
			
			var menuitem= dijit.byId(menuid);
			if(menuitem) {
				menuitem.setDisabled(true);
			}
		},
		
		enableMenuItem : function(menuid){
			// summary:
			//		Enable a menu item of the context menu attached to the treetable.
			// tags: public
			var menuitem= dijit.byId(menuid);
			if(menuitem) {
				menuitem.setDisabled(false);
			}
		},
		
		removeSelectedRows : function(){
			// summary: 
			//		deletes selected rows from the treetable
			// tags: public
			
			var grid = this._grid;
			var selectedArray = grid.select.row.getSelected();
			array.forEach(selectedArray, function(row){
				var item = grid.row(row, true).item();
				console.log("item..");
				console.log(item);
				grid.store.deleteItem(item);
			});
		},
		
		deleteRow: function(item){
			// summary: 
			//		deletes the given item from the store
			// tags: public
			// item: object
			//		Row element of the table
			this._grid.store.deleteItem(item);
		},
	
		removeItemFromToolbar: function(itemId){
			// summary: 
			//		to remove an item from the toolbar
			// tags: public
			// itemId: String
			if(this._toolbar) {
				this._toolbar._removeItem(itemId);
			}			
		},
		
		addRow: function(item,parent) {
			// summary: 
			//		Adds a new row to the treetable 
			// tags: public
			// description: 
			//		If no parent id is given, then the new row will be added as the last row.
			//      If parent id is given, then the row will be added as the last child of the parent.
			// item: object
			//		The item to be added
			// parent: string
			//		The parent id to which the new row should be added as a child
			
			//If the row is selected to be as a parent then the new row will be added as a child to that row.
			if(this._toolbar){
				if(this._toolbar.quickFilter && this._toolbar.quickFilter.filterText !== null){
					this._toolbar.quickFilter._clearFilter();
				}
				this._toolbar._clearSort();
			}
			if(!parent){
				if(this._grid){
					var selectedItem = this._grid.store.newItem(item);
				}
			}else{
				var s = this._grid.store;
				s.fetchItemByIdentity({identity: parent, onItem: function(item){
					parent = item;
				}});
				if(parent){
					s.newItem(item, {parent: parent, attribute: "children"});
				}
				this._grid.body.refresh();
			}
		},
		
		removeMenuItem : function (menuid){
			// summary: 
			//		Removes menu item from the context menu
			// tags: public
			var menuitem= dijit.byId(menuid);
			if(menuitem) {
				menuitem.destroyRecursive();
			}
		},
			
		refresh: function(updateBody) {
			// summary: 
			//		Refreshes the grid with updated values. If updateBody is true it refreshes the grid body also. 
			// tags: public
			var deferred = null;
			if (this._grid) {
				if (this._grid.model.store.CURIRefresh) {
					// Refresh is being driven by change to CURI dataset
					// on the server
					this._grid.store.CURIRefresh();
					// Since CURI dataset could have significant changes
					// can't count on selected item(s) to still be there
					// and can't count on tree expanded branch states, so
					// have to clear all of them (unless we can find out
					// from gridx team how to gracefully handle cases where
					// old child items no longer exist or have different IDs now.
	                this._grid.select.row.clear();
	                this._grid.model.clearCache();
	                this._grid.tree._clear();
	                deferred =  this._grid.body.refresh();  // Got to redraw all rows
				}
				else {
					this._grid.model.clearCache();
					if (updateBody) {
						deferred = this._grid.body.refresh();
					}					
				}
				var t = this;
				deferred.then(function(value){
					t.onRefresh();
				});
			}

			return deferred ;
		},
		
		onRefresh: function() {
			// summary:  
			//		Callback on tree table refresh. Connect to this to perform some actions after gridx refresh is complete
			// tags: public	
		},
		
		destroy: function(){
			// summary:  
			//		Destroys the tree table.
			// tags: public	
			
			// memory leak issue: custom destroy method to take care of removing all widgets					
			try{
				if(this.menu){
					this.menu.destroyRecursive();
				}
				delete this.resources_;			
				//this is for destroying child templated widgets 
				//this should happen by itself but need to figure out why dijit._Templated is not taking care of it...
				var widgets = dijit.findWidgets(this.domNode);
				dojo.forEach(widgets, function(w) {
					if(w){
						if(w.destroyRecursive){
							w.destroyRecursive();
						} else if(w.destroy){
							w.destroy();
						}
					}
				});
				// this is for widgets added as child to this widget and their children..
				//not present in template
				var widgetArr = this.getChildren();
				dojo.forEach(widgetArr, function (child) {
					if(child){
						if(child.destroyRecursive){
							child.destroyRecursive();
						} else if(child.destroy){
							child.destroy();
						}
					}
				});
				this.inherited(arguments);
			}catch(ex){
				console.warn("Widget destroy failed: "+this.id+", ex="+ex.name+":"+ex.message);
			}
		}	
		
	});
	
	return TreeTable;
});
},
'url:dijit/templates/TooltipDialog.html':"<div role=\"presentation\" tabIndex=\"-1\">\n\t<div class=\"dijitTooltipContainer\" role=\"presentation\">\n\t\t<div class =\"dijitTooltipContents dijitTooltipFocusNode\" data-dojo-attach-point=\"containerNode\" role=\"dialog\"></div>\n\t</div>\n\t<div class=\"dijitTooltipConnector\" role=\"presentation\"></div>\n</div>\n",
'url:dijit/templates/Calendar.html':"<table cellspacing=\"0\" cellpadding=\"0\" class=\"dijitCalendarContainer\" role=\"grid\" aria-labelledby=\"${id}_mddb ${id}_year\">\n\t<thead>\n\t\t<tr class=\"dijitReset dijitCalendarMonthContainer\" valign=\"top\">\n\t\t\t<th class='dijitReset dijitCalendarArrow' data-dojo-attach-point=\"decrementMonth\">\n\t\t\t\t<img src=\"${_blankGif}\" alt=\"\" class=\"dijitCalendarIncrementControl dijitCalendarDecrease\" role=\"presentation\"/>\n\t\t\t\t<span data-dojo-attach-point=\"decreaseArrowNode\" class=\"dijitA11ySideArrow\">-</span>\n\t\t\t</th>\n\t\t\t<th class='dijitReset' colspan=\"5\">\n\t\t\t\t<div data-dojo-attach-point=\"monthNode\">\n\t\t\t\t</div>\n\t\t\t</th>\n\t\t\t<th class='dijitReset dijitCalendarArrow' data-dojo-attach-point=\"incrementMonth\">\n\t\t\t\t<img src=\"${_blankGif}\" alt=\"\" class=\"dijitCalendarIncrementControl dijitCalendarIncrease\" role=\"presentation\"/>\n\t\t\t\t<span data-dojo-attach-point=\"increaseArrowNode\" class=\"dijitA11ySideArrow\">+</span>\n\t\t\t</th>\n\t\t</tr>\n\t\t<tr>\n\t\t\t${!dayCellsHtml}\n\t\t</tr>\n\t</thead>\n\t<tbody data-dojo-attach-point=\"dateRowsNode\" data-dojo-attach-event=\"onclick: _onDayClick\" class=\"dijitReset dijitCalendarBodyContainer\">\n\t\t\t${!dateRowsHtml}\n\t</tbody>\n\t<tfoot class=\"dijitReset dijitCalendarYearContainer\">\n\t\t<tr>\n\t\t\t<td class='dijitReset' valign=\"top\" colspan=\"7\" role=\"presentation\">\n\t\t\t\t<div class=\"dijitCalendarYearLabel\">\n\t\t\t\t\t<span data-dojo-attach-point=\"previousYearLabelNode\" class=\"dijitInline dijitCalendarPreviousYear\" role=\"button\"></span>\n\t\t\t\t\t<span data-dojo-attach-point=\"currentYearLabelNode\" class=\"dijitInline dijitCalendarSelectedYear\" role=\"button\" id=\"${id}_year\"></span>\n\t\t\t\t\t<span data-dojo-attach-point=\"nextYearLabelNode\" class=\"dijitInline dijitCalendarNextYear\" role=\"button\"></span>\n\t\t\t\t</div>\n\t\t\t</td>\n\t\t</tr>\n\t</tfoot>\n</table>\n",
'gridx/modules/pagination/LinkPager':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/_base/sniff",
	"dojo/_base/query",
	"dojo/_base/event",
	"dojo/dom-class",
	"dojo/string",
	"dojo/keys",
	"../../core/util",
	"./_PagerBase",
	"dojo/text!../../templates/PaginationBar.html"
], function(declare, array, lang, sniff, query, event, domClass, string, keys, util, _PagerBase, barTemplate){

	var hasClass = domClass.contains,
		toggle = domClass.toggle,
		substitute = string.substitute;

	function findNodeByEvent(evt, targetClass, containerClass){
		var n = evt.target;
		while(!hasClass(n, targetClass)){
			if(hasClass(n, containerClass)){
				return null;
			}
			n = n.parentNode;
		}
		return n;
	}

	function toggleHover(evt, targetCls, containerCls, hoverCls){
		var n = findNodeByEvent(evt, targetCls, containerCls);
		if(n){
			toggle(n, hoverCls, evt.type == 'mouseover');
		}
	}

	function focus(nodes, node, isMove, isLeft, isFocusable){
		//Try to focus on node, but if node is not focsable, find the next focusable node in nodes 
		//along the given direction. If not found, try the other direction.
		//Return the node if successfully focused, null if not.
		var dir = isLeft ? -1 : 1,
			i = node ? array.indexOf(nodes, node) + (isMove ? dir : 0) : (isLeft ? nodes.length - 1 : 0),
			findNode = function(i, dir){
				while(nodes[i] && !isFocusable(nodes[i])){
					i += dir;
				}
				return nodes[i];
			};
		node = findNode(i, dir) || findNode(i - dir, -dir);
		if(node){
			node.focus();
		}
		return node;
	}

	return declare(_PagerBase, {
		templateString: barTemplate,
	
		_tabIndex: -1,
	
		postMixInProperties: function(){
			if(sniff('ie')){
				//IE does not support inline-block, so have to set tabIndex
				var gridTabIndex = this.module.grid.domNode.getAttribute('tabindex');
				this._tabIndex = gridTabIndex > 0 ? gridTabIndex : 0;
			}
		},
	
		refresh: function(){
			var t = this;
			t._createDescription();
			t._createPageStepper();
			t._createPageSizeSwitch();
			t._createGotoButton();
		},
	
		_onSwitchPage: function(){
			this._createPageStepper();
			this.module.grid.vLayout.reLayout();
		},

		_onSizeChange: function(){
			this._createDescription();
			this._onSwitchPage();
		},

		_focusArea: function(){
			var focus = this.module.grid.focus;
			return focus && focus.currentArea();
		},
	
		_onChangePageSize: function(size, oldSize){
			var t = this,
				n = query('[pagesize="' + size + '"]', t._sizeSwitchContainer)[0];
			if(n){
				domClass.add(n, 'gridxPagerSizeSwitchBtnActive');
			}
			n = query('[pagesize="' + oldSize + '"]', t._sizeSwitchContainer)[0];
			if(n){
				domClass.remove(n, 'gridxPagerSizeSwitchBtnActive');
			}
			if(t._focusArea() == t.position + 'PageSizeSwitch'){
				t._findNextPageSizeSwitch();
			}
			t._createPageStepper();
			t.module.grid.vLayout.reLayout();
		},
		
		_createPageStepper: function(){
			var t = this, mod = t.module;
			if(t._toggleNode('gridxPagerStepper', mod._exist(t.position, 'stepper'))){
				var p = t.pagination,
					pageCount = p.pageCount(),
					currentPage = p.currentPage(),
					count = mod.arg('visibleSteppers'),
					sb = [], tabIndex = t._tabIndex,
					disableNext = false,
					disablePrev = false,
					nlsArr = [
						mod.arg('pageIndexTitleTemplate', t.pageIndexTitle),
						mod.arg('pageIndexWaiTemplate', t.pageIndexTitle),
						mod.arg('pageIndexTemplate', t.pageIndex)
					],
					ellipsis = '<span class="gridxPagerStepperEllipsis">&hellip;</span>',
					stepper = function(page){
						return ['<span class="gridxPagerStepperBtn gridxPagerPage ',
							currentPage == page ? 'gridxPagerStepperBtnActive' : '',
							'" aria-pressed="', currentPage == page ? 'true' : 'false',
							'" pageindex="', page,
							'" title="', substitute(nlsArr[0], [page + 1]),
							'" aria-label="', substitute(nlsArr[1], [page + 1]),
							'" role="button" tabindex="', tabIndex, '">', substitute(nlsArr[2], [page + 1]),
						'</span>'].join('');
					};
				if(pageCount){
					var firstPage = currentPage - Math.floor((count - 1) / 2),
						lastPage = firstPage + count - 1;
					if(firstPage < 1){
						firstPage = 1;
						lastPage = count - 1;
					}else if(pageCount > count && firstPage >= pageCount - count){
						firstPage = pageCount - count;
					}
					if(lastPage >= pageCount - 1){
						lastPage = pageCount - 2;
					}
					sb.push(stepper(0));
					if(pageCount > 2){
						if(firstPage > 1){
							sb.push(ellipsis);
						}
						for(var i = firstPage; i <= lastPage; ++i){
							sb.push(stepper(i));
						}
						if(lastPage < pageCount - 2){
							sb.push(ellipsis);
						}
					}
					if(pageCount > 1){
						sb.push(stepper(pageCount - 1));
					}
				}
				t._pageBtnContainer.innerHTML = sb.join('');
				
				if(!currentPage || currentPage === pageCount - 1){
					disablePrev = !currentPage || pageCount <= 1;
					disableNext = currentPage || pageCount <= 1;
				}
				toggle(t._nextPageBtn, 'gridxPagerStepperBtnDisable gridxPagerNextPageDisable', disableNext);
				toggle(t._prevPageBtn, 'gridxPagerStepperBtnDisable gridxPagerPrevPageDisable', disablePrev);
				t._nextPageBtn.setAttribute('aria-disabled', disableNext);
				t._prevPageBtn.setAttribute('aria-disabled', disablePrev);
		
				if(t._focusArea() == t.position + 'PageStepper'){
					t._findNextPageStepperBtn();
				}
			}	
		},	
	
		_gotoPrevPage: function(){
			this._focusPageIndex = 'Prev';
			var p = this.pagination;
			p.gotoPage(p.currentPage() - 1);
		},
	
		_gotoNextPage: function(){
			this._focusPageIndex = 'Next';
			var p = this.pagination;
			p.gotoPage(p.currentPage() + 1);
		},

		_gotoPage: function(evt){
			var n = findNodeByEvent(evt, 'gridxPagerStepperBtn', 'gridxPagerPages');
			if(n){
				var page = this._focusPageIndex = n.getAttribute('pageindex');
				this.pagination.gotoPage(parseInt(page, 10));
			}
		},
	
		_onHoverPageBtn: function(evt){
			toggleHover(evt, 'gridxPagerStepperBtn', 'gridxPagerPages', 'gridxPagerStepperBtnHover');
		},

		_onHoverSizeBtn: function(evt){
			toggleHover(evt, 'gridxPagerSizeSwitchBtn', 'gridxPagerSizeSwitch', 'gridxPagerSizeSwitchBtnHover');
		},
	
		_createPageSizeSwitch: function(){
			var t = this, mod = t.module;
			if(t._toggleNode('gridxPagerSizeSwitch', mod._exist(t.position, 'sizeSwitch'))){
				var sb = [], tabIndex = t._tabIndex,
					separator = mod.arg('sizeSeparator'),
					currentSize = t.pagination.pageSize(),
					nlsArr = [
						mod.arg('pageSizeTitleTemplate', t.pageSizeTitle),
						mod.arg('pageSizeWaiTemplate', t.pageSizeTitle),
						mod.arg('pageSizeTemplate', t.pageSize),
						mod.arg('pageSizeAllTitleText', t.pageSizeAllTitle),
						mod.arg('pageSizeAllWaiText', t.pageSizeAllTitle),
						mod.arg('pageSizeAllText', t.pageSizeAll)
					];
		
				array.forEach(mod.arg('sizes'), function(pageSize){
					var isAll = false;
					//pageSize might be invalid inputs, so be strict here.
					if(!(pageSize > 0)){
						pageSize = 0;
						isAll = true;
					}
					sb.push('<span class="gridxPagerSizeSwitchBtn ',
						currentSize === pageSize ? 'gridxPagerSizeSwitchBtnActive' : '',
						'" aria-pressed="', currentSize === pageSize ? 'true' : 'false',
						'" pagesize="', pageSize,
						'" title="', isAll ? nlsArr[3] : substitute(nlsArr[0], [pageSize]),
						'" aria-label="', isAll ? nlsArr[4] : substitute(nlsArr[1], [pageSize]),
						'" role="button" tabindex="', tabIndex, '">', isAll ? nlsArr[5] : substitute(nlsArr[2], [pageSize]),
						'</span>',
						//Separate the "separator, so we can pop the last one.
						'<span class="gridxPagerSizeSwitchSeparator">' + separator + '</span>');
				});
				sb.pop();
				t._sizeSwitchContainer.innerHTML = sb.join('');
			}
		},
	
		_switchPageSize: function(evt){
			var n = findNodeByEvent(evt, 'gridxPagerSizeSwitchBtn', 'gridxPagerSizeSwitch');
			if(n){
				var pageSize = this._focusPageSize = n.getAttribute('pagesize');
				this.pagination.setPageSize(parseInt(pageSize, 10));
			}
		},
		
		_createGotoButton: function(){
			this._toggleNode('gridxPagerGoto', this.module._exist(this.position, 'gotoButton'));
		},

		_showGotoDialog: function(){
			var t = this, mod = t.module;
			if(!t._gotoDialog){
				var cls = mod.arg('dialogClass'),
					gppane = mod.arg('gotoPagePane'),
					props = lang.mixin({
						'class': 'gridxGotoPageDialog',
						title: t.gotoDialogTitle,
						content: new gppane({
							pager: t
						})
					}, mod.arg('dialogProps') || {});
				t._gotoDialog = new cls(props);
			}
			var pageCount = t.pagination.pageCount(),
				pane = t._gotoDialog.content;
			pane.pageCountMsgNode.innerHTML = substitute(t.gotoDialogPageCount, [pageCount]);
			pane.pageInputBox.set('constraints', {
				fractional: false, 
				min: 1, 
				max: pageCount
			});
			t._gotoDialog.show();
		},

		//Focus--------------------------------------------------------------------------------
		_initFocus: function(){
			var t = this, g = t.module.grid, focus = g.focus;
			if(focus){
				var p = g.pagination, pos = t.position, fp = t.focusPriority,
					leftKey = g.isLeftToRight() ? keys.LEFT_ARROW : keys.RIGHT_ARROW;

				focus.registerArea({
					name: pos + 'PageStepper',
					priority: fp,
					focusNode: t._pageStepperContainer,
					doFocus: lang.hitch(t, t._findNextPageStepperBtn, false, false)
				});
				t.connect(t._pageStepperContainer, 'onkeydown', function(evt){
					if(evt.keyCode == keys.LEFT_ARROW || evt.keyCode == keys.RIGHT_ARROW){
						t._findNextPageStepperBtn(true, evt.keyCode == leftKey);
					}else if(evt.keyCode == keys.ENTER && 
						hasClass(evt.target, 'gridxPagerStepperBtn') && 
						!hasClass(evt.target, 'gridxPagerStepperBtnActive') &&
						!hasClass(evt.target, 'gridxPagerStepperBtnDisable')){
						if(isNaN(parseInt(t._focusPageIndex, 10))){
							t['_goto' + t._focusPageIndex + 'Page']();
						}else{
							p.gotoPage(parseInt(t._focusPageIndex, 10));
						}
					}else{
						return;
					}
					event.stop(evt);
				});

				focus.registerArea({
					name: pos + 'PageSizeSwitch',
					priority: fp + 0.001,
					focusNode: t._sizeSwitchContainer,
					doFocus: lang.hitch(t, t._findNextPageSizeSwitch, false, false)
				});
				t.connect(t._sizeSwitchContainer, 'onkeydown', function(evt){
					if(evt.keyCode == keys.LEFT_ARROW || evt.keyCode == keys.RIGHT_ARROW){
						t._findNextPageSizeSwitch(true, evt.keyCode == leftKey);
					}else if(evt.keyCode == keys.ENTER &&
						hasClass(evt.target, 'gridxPagerSizeSwitchBtn') &&
						!hasClass(evt.target, 'gridxPagerSizeSwitchBtnActive')){
						p.setPageSize(parseInt(t._focusPageSize, 10));
					}else{
						return;
					}
					event.stop(evt);
				});

				focus.registerArea({
					name: pos + 'GotoPage',
					priority: fp + 0.002,
					focusNode: t._gotoBtn,
					doFocus: function(evt){
						util.stopEvent(evt);
						t._gotoBtn.focus();
						return true;
					}
				});
				t.connect(t._gotoBtn, 'onkeydown', function(evt){
					if(evt.keyCode == keys.ENTER){
						t._showGotoDialog();
						event.stop(evt);
					}
				});
			}
		},

		_findNextPageSizeSwitch: function(isMove, isLeft, evt){
			util.stopEvent(evt);
			var t = this,
				c = t._sizeSwitchContainer,
				n = query('[pagesize="' + t._focusPageSize + '"]', c)[0];
			n = focus(query('.gridxPagerSizeSwitchBtn', c), n, isMove, isLeft, function(node){
				return !hasClass(node, 'gridxPagerSizeSwitchBtnActive');
			});
			if(n){
				t._focusPageSize = n.getAttribute('pagesize');
			}
			return n;
		},

		_findNextPageStepperBtn: function(isMove, isLeft, evt){
			util.stopEvent(evt);
			var t = this,
				c = t._pageStepperContainer,
				n = query('[pageindex="' + t._focusPageIndex + '"]', c)[0];
			n = focus(query('.gridxPagerStepperBtn', c), n, isMove, isLeft, function(node){
				return !hasClass(node, 'gridxPagerStepperBtnActive') &&
					!hasClass(node, 'gridxPagerStepperBtnDisable');
			});
			if(n){
				t._focusPageIndex = n.getAttribute('pageindex');
			}
			return n;
		}
	});
});

},
'dijit/tree/ForestStoreModel':function(){
define([
	"dojo/_base/array", // array.indexOf array.some
	"dojo/_base/declare", // declare
	"dojo/_base/lang", // lang.hitch
	"dojo/_base/window", // win.global
	"./TreeStoreModel"
], function(array, declare, lang, win, TreeStoreModel){

/*=====
var TreeStoreModel = dijit.tree.TreeStoreModel;
=====*/

// module:
//		dijit/tree/ForestStoreModel
// summary:
//		Interface between a dijit.Tree and a dojo.data store that doesn't have a root item,
//		a.k.a. a store that has multiple "top level" items.

return declare("dijit.tree.ForestStoreModel", TreeStoreModel, {
	// summary:
	//		Interface between a dijit.Tree and a dojo.data store that doesn't have a root item,
	//		a.k.a. a store that has multiple "top level" items.
	//
	// description
	//		Use this class to wrap a dojo.data store, making all the items matching the specified query
	//		appear as children of a fabricated "root item".  If no query is specified then all the
	//		items returned by fetch() on the underlying store become children of the root item.
	//		This class allows dijit.Tree to assume a single root item, even if the store doesn't have one.
	//
	//		When using this class the developer must override a number of methods according to their app and
	//		data, including:
	//			- onNewRootItem
	//			- onAddToRoot
	//			- onLeaveRoot
	//			- onNewItem
	//			- onSetItem

	// Parameters to constructor

	// rootId: String
	//		ID of fabricated root item
	rootId: "$root$",

	// rootLabel: String
	//		Label of fabricated root item
	rootLabel: "ROOT",

	// query: String
	//		Specifies the set of children of the root item.
	// example:
	//	|	{type:'continent'}
	query: null,

	// End of parameters to constructor

	constructor: function(params){
		// summary:
		//		Sets up variables, etc.
		// tags:
		//		private

		// Make dummy root item
		this.root = {
			store: this,
			root: true,
			id: params.rootId,
			label: params.rootLabel,
			children: params.rootChildren	// optional param
		};
	},

	// =======================================================================
	// Methods for traversing hierarchy

	mayHaveChildren: function(/*dojo.data.Item*/ item){
		// summary:
		//		Tells if an item has or may have children.  Implementing logic here
		//		avoids showing +/- expando icon for nodes that we know don't have children.
		//		(For efficiency reasons we may not want to check if an element actually
		//		has children until user clicks the expando node)
		// tags:
		//		extension
		return item === this.root || this.inherited(arguments);
	},

	getChildren: function(/*dojo.data.Item*/ parentItem, /*function(items)*/ callback, /*function*/ onError){
		// summary:
		// 		Calls onComplete() with array of child items of given parent item, all loaded.
		if(parentItem === this.root){
			if(this.root.children){
				// already loaded, just return
				callback(this.root.children);
			}else{
				this.store.fetch({
					query: this.query,
					onComplete: lang.hitch(this, function(items){
						this.root.children = items;
						callback(items);
					}),
					onError: onError
				});
			}
		}else{
			this.inherited(arguments);
		}
	},

	// =======================================================================
	// Inspecting items

	isItem: function(/* anything */ something){
		return (something === this.root) ? true : this.inherited(arguments);
	},

	fetchItemByIdentity: function(/* object */ keywordArgs){
		if(keywordArgs.identity == this.root.id){
			var scope = keywordArgs.scope?keywordArgs.scope:win.global;
			if(keywordArgs.onItem){
				keywordArgs.onItem.call(scope, this.root);
			}
		}else{
			this.inherited(arguments);
		}
	},

	getIdentity: function(/* item */ item){
		return (item === this.root) ? this.root.id : this.inherited(arguments);
	},

	getLabel: function(/* item */ item){
		return	(item === this.root) ? this.root.label : this.inherited(arguments);
	},

	// =======================================================================
	// Write interface

	newItem: function(/* dojo.dnd.Item */ args, /*Item*/ parent, /*int?*/ insertIndex){
		// summary:
		//		Creates a new item.   See dojo.data.api.Write for details on args.
		//		Used in drag & drop when item from external source dropped onto tree.
		if(parent === this.root){
			this.onNewRootItem(args);
			return this.store.newItem(args);
		}else{
			return this.inherited(arguments);
		}
	},

	onNewRootItem: function(/* dojo.dnd.Item */ /*===== args =====*/){
		// summary:
		//		User can override this method to modify a new element that's being
		//		added to the root of the tree, for example to add a flag like root=true
	},

	pasteItem: function(/*Item*/ childItem, /*Item*/ oldParentItem, /*Item*/ newParentItem, /*Boolean*/ bCopy, /*int?*/ insertIndex){
		// summary:
		//		Move or copy an item from one parent item to another.
		//		Used in drag & drop
		if(oldParentItem === this.root){
			if(!bCopy){
				// It's onLeaveRoot()'s responsibility to modify the item so it no longer matches
				// this.query... thus triggering an onChildrenChange() event to notify the Tree
				// that this element is no longer a child of the root node
				this.onLeaveRoot(childItem);
			}
		}
		this.inherited(arguments, [childItem,
			oldParentItem === this.root ? null : oldParentItem,
			newParentItem === this.root ? null : newParentItem,
			bCopy,
			insertIndex
		]);
		if(newParentItem === this.root){
			// It's onAddToRoot()'s responsibility to modify the item so it matches
			// this.query... thus triggering an onChildrenChange() event to notify the Tree
			// that this element is now a child of the root node
			this.onAddToRoot(childItem);
		}
	},

	// =======================================================================
	// Handling for top level children

	onAddToRoot: function(/* item */ item){
		// summary:
		//		Called when item added to root of tree; user must override this method
		//		to modify the item so that it matches the query for top level items
		// example:
		//	|	store.setValue(item, "root", true);
		// tags:
		//		extension
		console.log(this, ": item ", item, " added to root");
	},

	onLeaveRoot: function(/* item */ item){
		// summary:
		//		Called when item removed from root of tree; user must override this method
		//		to modify the item so it doesn't match the query for top level items
		// example:
		// 	|	store.unsetAttribute(item, "root");
		// tags:
		//		extension
		console.log(this, ": item ", item, " removed from root");
	},

	// =======================================================================
	// Events from data store

	_requeryTop: function(){
		// reruns the query for the children of the root node,
		// sending out an onSet notification if those children have changed
		var oldChildren = this.root.children || [];
		this.store.fetch({
			query: this.query,
			onComplete: lang.hitch(this, function(newChildren){
				this.root.children = newChildren;

				// If the list of children or the order of children has changed...
				if(oldChildren.length != newChildren.length ||
					array.some(oldChildren, function(item, idx){ return newChildren[idx] != item;})){
					this.onChildrenChange(this.root, newChildren);
				}
			})
		});
	},

	onNewItem: function(/* dojo.data.Item */ item, /* Object */ parentInfo){
		// summary:
		//		Handler for when new items appear in the store.  Developers should override this
		//		method to be more efficient based on their app/data.
		// description:
		//		Note that the default implementation requeries the top level items every time
		//		a new item is created, since any new item could be a top level item (even in
		//		addition to being a child of another item, since items can have multiple parents).
		//
		//		If developers can detect which items are possible top level items (based on the item and the
		//		parentInfo parameters), they should override this method to only call _requeryTop() for top
		//		level items.  Often all top level items have parentInfo==null, but
		//		that will depend on which store you use and what your data is like.
		// tags:
		//		extension
		this._requeryTop();

		this.inherited(arguments);
	},

	onDeleteItem: function(/*Object*/ item){
		// summary:
		//		Handler for delete notifications from underlying store

		// check if this was a child of root, and if so send notification that root's children
		// have changed
		if(array.indexOf(this.root.children, item) != -1){
			this._requeryTop();
		}

		this.inherited(arguments);
	},

	onSetItem: function(/* item */ item,
					/* attribute-name-string */ attribute,
					/* object | array */ oldValue,
					/* object | array */ newValue){
		// summary:
		//		Updates the tree view according to changes to an item in the data store.
		//		Developers should override this method to be more efficient based on their app/data.
		// description:
		//		Handles updates to an item's children by calling onChildrenChange(), and
		//		other updates to an item by calling onChange().
		//
		//		Also, any change to any item re-executes the query for the tree's top-level items,
		//		since this modified item may have started/stopped matching the query for top level items.
		//
		//		If possible, developers should override this function to only call _requeryTop() when
		//		the change to the item has caused it to stop/start being a top level item in the tree.
		// tags:
		//		extension

		this._requeryTop();
		this.inherited(arguments);
	}

});

});

},
'dojox/mvc/_base':function(){
define([
	"dojo/_base/kernel",
	"dojo/_base/lang",
	"./StatefulModel",
	"./Bind",
	"./_DataBindingMixin",
	"./_patches"
], function(kernel, lang, StatefulModel){
	// module:
	//		dojox/mvc/_base
	// summary:
	//		Pulls in essential MVC dependencies such as basic support for
	//		data binds, a data model and data binding mixin for dijits.
	kernel.experimental("dojox.mvc");

	var mvc = lang.getObject("dojox.mvc", true);
	/*=====
		mvc = dojox.mvc;
	=====*/

	// Factory method for dojox.mvc.StatefulModel instances
	mvc.newStatefulModel = function(/*Object*/args){
		// summary:
		//		Factory method that instantiates a new data model that view
		//		components may bind to.
		//	args:
		//		The mixin properties.
		// description:
		//		Factory method that returns a client-side data model, which is a
		//		tree of dojo.Stateful objects matching the initial data structure
		//		passed as input:
		//		- The mixin property "data" is used to provide a plain JavaScript
		//		  object directly representing the data structure.
		//		- The mixin property "store", along with an optional mixin property
		//		  "query", is used to provide a data store to query to obtain the
		//		  initial data.
		//		This function returns an immediate dojox.mvc.StatefulModel instance or
		//		a Promise for such an instance as follows:
		//		- if args.data: returns immediate
		//		- if args.store:
		//			- if store returns immediate: this function returns immediate
		//			- if store returns a Promise: this function returns a model
		//			  Promise
		if(args.data){
			return new StatefulModel({ data : args.data });
		}else if(args.store && lang.isFunction(args.store.query)){
			var model;
			var result = args.store.query(args.query);
			if(result.then){
				return (result.then(function(data){
					model = new StatefulModel({ data : data });
					model.store = args.store;
					return model;
				}));
			}else{
				model = new StatefulModel({ data : result });
				model.store = args.store;
				return model;
			}
		}
	};

	return mvc;
});

},
'ibm/tivoli/tip/dojo/util':function(){
// wrapped by build app
define("ibm/tivoli/tip/dojo/util", ["dijit","dojo","dojox","dojo/require!dojo/date/locale"], function(dijit,dojo,dojox){
/****************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* 5724-Q87 
*
* (C) Copyright IBM Corp. 2007
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
********************************************************** {COPYRIGHT-END} **/
dojo.require("dojo.date.locale");

dojo.provide("ibm.tivoli.tip.dojo.util");

ibm.tivoli.tip.dojo.util._stdDateFormat = { // selector:     'dateOnly', 
                                      formatLength: 'short' 
                                    };

/************************************************************************
* "apply" any GLOBAL-based data of interest.
************************************************************************/	
ibm.tivoli.tip.dojo.util.applyGlobal = function( global ) 
{
  ibm.tivoli.tip.dojo.util._stdDateFormat = global.l10n.date.stdTS;

  console.debug( "** util.applyGlobal **", dojo.toJson(ibm.tivoli.tip.dojo.util._stdDateFormat ) ); 
};

/************************************************************************
* Format a DATE with NLS rules
************************************************************************/	
ibm.tivoli.tip.dojo.util.formatDate = function( date ) 
{
	var fDate = ( date === null ) ? "" : dojo.date.locale.format( date, ibm.tivoli.tip.dojo.util._stdDateFormat );
 
	return( fDate );
};

/************************************************************************
* Standard formatter for a table column containing Date objects.
************************************************************************/	
ibm.tivoli.tip.dojo.util.dateColumnFormater = function( /* Date */ cellValue )
 {
   // console.debug( "cell: ", cellValue );

   return( ibm.tivoli.tip.dojo.util.formatDate( cellValue ) );
 };

/************************************************************************
* Refresh the page
************************************************************************/	
ibm.tivoli.tip.dojo.util.refreshPage = function()
{
	window.location.reload( false );
};

/*****************************************************************************
* Return a string of all the fields contained in an object.
*****************************************************************************/
ibm.tivoli.tip.dojo.util.fieldsOf = function( object )
{ 
  var s = "";

  if ( object && ( object !== null ) ) 
  {
    for ( var field in object )
    {
    	if (field) {
	      s += field + "; ";
	    }
    }
  }

  return( s );
};

/************************************************************************
* Clear a datastore (must implement dojo.data.api.Write)
* 
* Returns dojo.Deferred() -- must add Callback
*
* example usage:
* ibm.tivoli.tip.dojo.util.clearDataStore( store ).addCallback( function(){} );
************************************************************************/ 
ibm.tivoli.tip.dojo.util.clearDataStore = function( store )
{
  var deferred = new dojo.Deferred();
    
  var callback = 
  {  
        deferred: deferred,
           store: store,        
      onComplete: function( items, request ) 
                  {                    
                    for ( var i in items )
                    {
                    	if (i) {
	                      this.store.deleteItem( items[i] );
	                    }
                    }

                    this.deferred.callback();
                  }
  };
  
  store.fetch( { onComplete: dojo.hitch( callback, "onComplete" ) } );
  
  return deferred;
};

/************************************************************************
* Clear a datastore (must implement dojo.data.api.Write)
************************************************************************/ 
ibm.tivoli.tip.dojo.util.addItemsToDataStore = function( store, items )
{
  for ( var i in items )
  {
  	if (i) {
	    store.newItem( items[i] );
	}
  }

  store.save();
};

/************************************************************************
* Find the default name for a specific value using a name/value array.
*
* [
*  { value: "-1", name: this.global.nls( "CBOX_CONFIG_DEFAULT", { config : config.maxSnapshotsDefault } ) }, 
*  { value:  "0", name: bundle.CBOX_DISABLED }, 
*  { value:  "1", name:  "1" }
* ]
************************************************************************/ 
ibm.tivoli.tip.dojo.util.findDefaultName = function( value, nameValueArray )
{
  var name = nameValueArray[ 0 ].name;     // default to first element

  for ( var i in nameValueArray )
  {
  	if (i) {
    	var nv = nameValueArray[ i ];

    	if ( nv.value == value ) 
    	{
      		name = nv.name;
      		break;
    	}
    }
  }

  // console.debug( "value: " + value + ", name: " + name );

  return( name );
};


});

},
'gridx/modules/select/_RowCellBase':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"./_Base",
	"../../core/model/extensions/Mark"
], function(declare, lang, _Base, Mark){

	return declare(_Base, {
		modelExtensions: [Mark],

		selectById: function(rowId, columnId){
			var t = this, m = t.model;
			if(t.arg('enabled')){
				m.markById(rowId, 1, t._getMarkType(columnId));
				m.when();
			}
		},
		
		deselectById: function(rowId, columnId){
			var t = this, m = t.model;
			if(t.arg('enabled')){
				m.markById(rowId, 0, t._getMarkType(columnId));
				m.when();
			}
		},
		
		isSelected: function(rowId, columnId){
			return this.model.getMark(rowId, this._getMarkType(columnId)) === true;	//Mixed status is not selected
		},

		//Private-----------------------------------------------------------------
		_init: function(){
			var t = this, m = t.model;
			t.connect(m, 'onMarkChange', '_onMark');
		}
	});
});

},
'url:idx/oneui/messaging/templates/SingleMessage.html':"<div class=\"dijit dijitReset\"\r\n\t><div class=\"idxMessageHead\"\r\n\t\t><div class=\"dijitInline idxMessageIcon\" src=\"${_blankGif}\" dojoAttachPoint=\"iconNode\" dojoAttachEvent=\"onmouseenter: onIconEnter, onmouseleave: onIconLeave\"\r\n\t\t\t><span id=\"${id}_type\" dojoAttachPoint=\"typeNode\" class=\"idxMessageIconText idxMessageContent idxMessageAltText\"></span></div\r\n\t\t><div class=\"dijitInline idxMessageContent\" dojoAttachPoint=\"focusNode\" tabIndex=\"${tabIndex}\" dojoAttachEvent=\"ondijitclick: _onClick\"\r\n\t\t\t><span class=\"dijitInline idxMessageTitle\" dojoAttachPoint=\"titleNode\"></span\r\n\t\t\t><span class=\"dijitInline idxMessageId messagesContrast\" dojoAttachPoint=\"idNode\"></span\r\n\t\t></div\r\n\t\t><div class=\"idxMessageFakeContent\" dojoAttachPoint=\"fakeFocusNode\" tabIndex=\"${tabIndex}\" dojoAttachEvent=\"ondijitclick: _onClick\"\r\n\t\t\t><span class=\"idxMessageTitle\" dojoAttachPoint=\"fakeTitleNode\"></span\r\n\t\t\t><span class=\"idxMessageId messagesContrast\" dojoAttachPoint=\"fakeIdNode\"></span\r\n\t\t></div\r\n\t\t><div class=\"dijitInline idxMessageRightAligned\" dojoAttachPoint=\"infoNode\"\r\n\t\t\t><div class=\"dijitInline idxMessageTimestamp messagesContrast\" dojoAttachPoint=\"timestampNode\">03 March 2011  12:00 PM</div\r\n\t\t\t><div class=\"dijitInline idxMessageSeparator\" dojoAttachPoint=\"separatorNode\">|</div\r\n\t\t\t><a class=\"dijitInline idxMessageAction\" tabIndex=\"${tabIndex}\" dojoAttachPoint=\"actionNode\" dojoAttachEvent=\"ondijitclick: onAction\" href=\"javascript: void(0);\"></a\r\n\t\t\t><div class=\"dijitInline idxMessageCloseIcon\" tabIndex=\"${tabIndex}\" role=\"button\" dojoAttachEvent=\"ondijitclick: _onClose\" dojoAttachPoint=\"closeNode\"\r\n\t\t\t\t><span class=\"idxMessageContent idxMessageAltText\">X</span\r\n\t\t\t></div\r\n\t\t></div\r\n\t></div\r\n\t><div class=\"idxMessageSplitter\" src=\"${_blankGif}\"/></div\r\n\t><div class=\"idxMessageBody\" dojoAttachPoint=\"bodyNode\" role=\"document\"\r\n\t\t><span class=\"idxMessageDetail\" dojoAttachPoint=\"descriptionNode\" tabIndex=\"${tabIndex}\"></span\r\n\t\t><a class=\"idxMessageViewDetails\" tabIndex=\"${tabIndex}\" dojoAttachPoint=\"viewDetailsNode\" href=\"javascript: void(0);\" dojoAttachEvent=\"ondijitclick: onMoreDetails\"></a\r\n\t></div\r\n\t><div class=\"idxMessageRefresh\" \r\n\t><span dojoAttachPoint=\"refreshNode\" tabIndex=\"${tabIndex}\" dojoAttachEvent=\"ondijitclick: onRefresh\"></span></div\r\n></div>",
'gridx/modules/Menu':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/connect",
	"../core/_Module",
	"dojo/dom-class",
	"dojo/keys",
	"dojo/_base/event",
	"dojo/_base/lang",
	"dijit/registry",
	"dijit/Menu"
], function(declare, connect, _Module, domClass, keys, event, lang, registry, Menu){

/*=====
	var _MenuArgs = function(){
		//hookPoint: String?
		//		Indicates from where the menu should occur.
		//		One of "cell", "headercell", "row", "header", "body", "grid". If invalid, default to "grid".
		//selected: Boolean?
		//		Indicates whether to bind this menu only to the selected items. Default is false.
	};

	var _MenuContext = function(){
		//grid: Grid
		//		This grid that triggers this menu.
		//column: Column
		//		The column that triggers this menu. Only valid for "headercell" hookpoint.
		//row: Row
		//		The row that triggers this menu. Only valid for "row" hookpoint.
		//cell: Cell
		//		The cell that triggers this menu. Only valid for "cell" hookpoint.
	};
=====*/

	return declare(/*===== "gridx.modules.Menu", =====*/_Module, {
		name: 'menu',

		getAPIPath: function(){
			return {
				menu: this 
			};
		},
	
		constructor: function(){
			this._menus = {};
		},

		//Public---------------------------------------------

		//context: __MenuContext
		//		An object representing the current context when user triggers a context menu.
		//		This property is updated everytime a menu of grid is popped up.
		//		Users can refer to this in their menu action handlers by grid.menu.context.
		context: null,

		bind: function(/* dijit.Menu|ID */ menu, /* __MenuArgs? */ args){
			// summary:
			//		Bind a memu to grid, according to the provided args
			//menu: dijit.Menu | ID
			//		The menu to be binded.
			//args: __MenuArgs
			//		Indicates how to bind the menu
			args = args || {};
			var t = this,
				g = t.grid,
				hookPoint = args.hookPoint && args.hookPoint.toLowerCase() || 'grid',
				type = args.selected ? hookPoint + '-selected' : hookPoint,
				evtName = t._evtMap[hookPoint],
				m = t._menus[type] = t._menus[type] || {},
				showMenu = lang.partial(t._showMenu, type);
			connect.disconnect(m.open);
			connect.disconnect(m.close);
			m.menu = registry.byId(menu);
			if(evtName){
				m.open = t.connect(g, evtName, showMenu);
			}else if(hookPoint == 'body'){
				m.open = t.connect(g.bodyNode, 'oncontextmenu', showMenu);
			}else{
				m.open = t.connect(g.domNode, 'oncontextmenu', showMenu);
			}
			m.close = t.connect(m.menu, 'onClose', function(){
				t._mutex = 0;
			});
		},

		unbind: function(menu){
			// summary:
			//		Unbind a menu from grid.
			//menu: dijit.Menu | ID
			//		The menu to be unbinded.
			var type, menus = this._menus, m;
			menu = registry.byId(menu);
			for(type in menus){
				m = menus[type];
				if(m.menu == menu){
					connect.disconnect(m.open);
					connect.disconnect(m.close);
					delete menus[type];
					return;
				}
			}
		},
		
		//[private]==
		_evtMap: {
			header: 'onHeaderContextMenu',
			headercell: 'onHeaderCellContextMenu',
			cell: 'onCellContextMenu',
			row: 'onRowContextMenu'
		},

		_showMenu: function(type, e){
			var t = this, menus = t._menus;
			if(!t._mutex && menus[type].menu){
				var g = t.grid,
					rid = e.rowId,
					cid = e.columnId,
					isRow = !type.indexOf('row'),
					isCell = !type.indexOf('cell'),
					isHeaderCell = !type.indexOf('headercell'),
					isSelectedType = type.indexOf('-') > 0,
					selected = !!((isCell && domClass.contains(e.cellNode, "gridxCellSelected")) ||
						(isHeaderCell && domClass.contains(g.header.getHeaderNode(cid), "gridxColumnSelected")) ||
						(isRow && domClass.contains(g.body.getRowNode({rowId: rid}), "gridxRowSelected")));
				if(isSelectedType == selected || (!isSelectedType && selected && !menus[type + '-selected'])){
					t.context = {
						grid: g,
						column: isHeaderCell && g.column(cid, 1),
						row: isRow && g.row(rid, 1),
						cell: isCell && g.cell(rid, cid, 1)
					};
					event.stop(e);
					t._mutex = 1;
					menus[type].menu._openMyself({
						target: e.target, 
						coords: e.keyCode != keys.F10 && "pageX" in e ? {x: e.pageX, y: e.pageY} : null
					});
				}
			}
		}
	});
});

},
'gridx/modules/VirtualVScroller':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/array",
	"dojo/_base/sniff",
	"dojo/_base/event",
	"dojo/_base/Deferred",
	"dojo/query",
	"dojo/keys",
	"./VScroller",
	"../core/_Module"
], function(declare, lang, array, sniff, event, Deferred, query, keys, VScroller, _Module){
	
	return declare(/*===== "gridx.modules.VirtualVScroller", =====*/VScroller, {
		// summary:
		//		This module implements lazy-rendering when virtically scrolling grid.
		// description:
		//		This module takes a DOMNode-based way to implement lazy-rendering.
		//		It tries to remove all the DOMNodes that are out of the grid body viewport,
		//		so that the DOMNodes in grid are always limited to a very small number.

		constructor: function(grid, args){
			if(grid.autoHeight){
				lang.mixin(this, new VScroller(grid, args));
			}else{
				this._scrolls = [];
			}
		},

		//Public ----------------------------------------------------

		// buffSize: Integer
		//		The count row nodes that should be maintained above/below the grid body viewport.
		//		The total count row nodes consists of the count of rows that are visible, and buffSize * 2.
		buffSize: 5,
		
		// lazy: Boolean
		//		If this argument is set to true, the grid will not fetch data during scrolling.
		//		Instead, it'll fetch data after the scrolling process is completed (plus a timeout).
		//		This is useful when a large slow server side data store is used, because frequent
		//		data fetch requests are avoided.
		lazy: false,
		
		// lazyTimeout: Number
		//		This is the timeout for the "lazy" argument.
		lazyTimeout: 50,
	
		scrollToRow: function(rowVisualIndex, toTop){
			// summary:
			//		Override VScroller.scrollToRow
			// tags:
			//		extension
			var d = new Deferred(), t = this, s = t._scrolls,
				f = function(){
					t._subScrollToRow(rowVisualIndex, d, toTop);
				};
			s.push(d);
			t._lazy = t.arg('lazy');
			t.lazy = false;
			if(s.length > 1){
				s[s.length - 2].then(f);
			}else{
				f();
			}
			return d;
		},

		//Private -------------------------------------------------
		_subScrollToRow: function(rowVisualIndex, defer, toTop){
			var t = this,
				dif = 0,
				rowHeight = t._avgRowHeight,
				bn = t.grid.bodyNode,
				dn = t.domNode,
				bst = bn.scrollTop,
				dst = dn.scrollTop,
				node = query('[visualindex="' + rowVisualIndex + '"]', bn)[0],
				focus = t.grid.focus,
				finish = function(success){
					t._scrolls.splice(array.indexOf(t._scrolls, defer), 1);
					//since we've changed the focus row index, re-focus it.
					if(focus && focus.currentArea() == 'body'){
						focus.focusArea('body', 1);	//1 as true
					}
					t.lazy = t._lazy;
					//wait for the dom nodes to settle down.
					setTimeout(function(){
						defer.callback(success);
					}, 5);
				};
			if(node){
				var offsetTop = node.offsetTop;
				if(offsetTop + node.offsetHeight > bst + bn.clientHeight){
					dif = offsetTop - bst;
					if(!toTop){
						dif += node.offsetHeight - bn.clientHeight;
					}
				}else if(offsetTop < bst || (toTop && offsetTop > bst)){
					dif = offsetTop - bst;
				}else{
					finish(true);
					return;
				}
			}else if(bn.childNodes.length){
				//Find a visible node.
				var n = bn.firstChild;
				while(n && n.offsetTop < bst){
					n = n.nextSibling;
				}
				var idx = n && n.getAttribute('visualindex');
				if(n && rowVisualIndex < idx){
					dif = (rowVisualIndex - idx) * rowHeight;
				}else{
					n = bn.lastChild;
					while(n && n.offsetTop + n.offsetHeight > bst + bn.clientHeight){
						n = n.previousSibling;
					}
					idx = n && n.getAttribute('visualindex');
					if(n && rowVisualIndex > idx){
						dif = (rowVisualIndex - idx) * rowHeight;
					}else{
						finish(false);
						return;
					}
				}
			}else{
				finish(false);
				return;
			}
			var istop = dst === 0 && dif < 0,
				isbottom = dst >= dn.scrollHeight - dn.offsetHeight && dif > 0;
			if(istop || isbottom){
				t._doVirtualScroll(1);
			}else{
				dn.scrollTop += dif / t._ratio;
			}
			if((istop && bn.firstChild.getAttribute('visualindex') == 0) || 
					(isbottom && bn.lastChild.getAttribute('visualindex') == t.grid.body.visualCount - 1)){
				finish(false);
				return;
			}
			setTimeout(function(){
				t._subScrollToRow(rowVisualIndex, defer, toTop);
			}, 5);
		},
	
		_init: function(args){
			var t = this;
			t._rowHeight = {};
			t._syncHeight();
			t.connect(t.grid, '_onResizeEnd', function(){
				t._doScroll(0, 1);
			});
			t._doScroll(0, 1);
		},
	
		_doVirtualScroll: function(forced){
			var t = this,
				dn = t.domNode,
				a = dn.scrollTop,
				deltaT = t._ratio * (a - (t._lastScrollTop || 0));
	
			if(forced || deltaT){
				t._lastScrollTop = a;
	
				var buffSize = t.arg('buffSize'),
					scrollRange = dn.scrollHeight - dn.offsetHeight,
					body = t.grid.body,
					visualStart = body.visualStart,
					visualEnd = visualStart + body.visualCount,
					bn = t.grid.bodyNode,
					firstRow = bn.firstChild,
					firstRowTop = firstRow && firstRow.offsetTop - deltaT,
					lastRow = bn.lastChild,
					lastRowBtm = lastRow && lastRow.offsetTop - deltaT + lastRow.offsetHeight,
					bnTop = bn.scrollTop,
					bnBtm = bnTop + bn.clientHeight,
					h = t._avgRowHeight,
					pageRowCount = Math.ceil(dn.offsetHeight / h) + 2 * buffSize,
					start, end, pos, d;
				if(bnTop == bnBtm && !bnBtm){
					//The grid is not correctly shown, so we just ignore.
					return;
				}
				if(firstRow && firstRowTop > bnTop && firstRowTop < bnBtm){
					//Add some rows to the front
					end = body.renderStart;
					d = Math.ceil((firstRowTop - bnTop) / h) + buffSize;
					start = a === 0 ? visualStart : Math.max(end - d, visualStart);
					pos = "top";
//                    console.log('top: ', start, end);
				}else if(lastRow && lastRowBtm > bnTop && lastRowBtm < bnBtm){
					//Add some rows to the end
					start = body.renderStart + body.renderCount;
					d = Math.ceil((bnBtm - lastRowBtm) / h) + buffSize;
					end = a === scrollRange && a ? visualEnd : Math.min(start + d, visualEnd);
					pos = "bottom";
//                    console.log('bottom: ', start, end);
				}else if(!firstRow || firstRowTop > bnBtm || !lastRow || lastRowBtm < bnTop){
					//Replace all
					if(a <= scrollRange / 2){
						start = a === 0 ? visualStart : visualStart + Math.max(Math.floor(a / h) - buffSize, 0);
						end = Math.min(start + pageRowCount, visualEnd);
					}else{
						end = a === scrollRange ? visualEnd : visualEnd + Math.min(pageRowCount - Math.floor((scrollRange - a) / h), 0);
						start = Math.max(end - pageRowCount, visualStart);
					}
					pos = "clear";
				}else if(firstRow){
					//The body and the scroller bar may be mis-matched, so force to sync here.
					if(a === 0){
						var firstRowIndex = body.renderStart;
						if(firstRowIndex > visualStart){
							start = visualStart;
							end = firstRowIndex;
							pos = "top";
//                            console.debug("Recover top", end - start);
						}	
					}else if(a === scrollRange){
						var lastRowIndex = body.renderStart + body.renderCount - 1;
						if(lastRowIndex < visualEnd - 1){
							start = lastRowIndex + 1;
							end = visualEnd;
							pos = "bottom";
//                            console.debug("Recover bottom", end - start);
						}
					}
				}
				
				if(typeof start == 'number' && typeof end == 'number'){
//                    console.debug("render: ", start, end, pos, a, scrollRange);
					//Only need to render when the range is valid
					body.renderRows(start, end - start, pos);
					if(a && start < end){
						//Scroll the body to hide the newly added top rows.
						var n = query('[visualindex="' + end + '"]', bn)[0];
						if(n){
							deltaT += n.offsetTop;
						}
					}
				}
				//Ensure the position when user scrolls to end points
				if(a === 0){
					bn.scrollTop = 0;
				}else if(a >= scrollRange){//Have to use >=, because with huge store, a will sometimes be > scrollRange
					bn.scrollTop = bn.scrollHeight;
				}else if(pos != "clear"){
					bn.scrollTop += deltaT;
				}
			}
		},
		
		_doScroll: function(e, forced){
			var t = this;
			if(t.arg('lazy')){
				if(t._lazyScrollHandle){
					clearTimeout(t._lazyScrollHandle);
				}
				t._lazyScrollHandle = setTimeout(lang.hitch(t, t._doVirtualScroll, forced), t.arg('lazyTimeout'));
			}else{
				t._doVirtualScroll(forced);
			}
		},
	
		_onMouseWheel: function(e){
			if(this.grid.vScrollerNode.style.display != 'none'){
				var rolled = typeof e.wheelDelta === "number" ? e.wheelDelta / 3 : (-40 * e.detail); 
				this.domNode.scrollTop -= rolled / this._ratio;
				event.stop(e);
			}
		},
	
		_onBodyChange: function(){
			this._update();
			this._doScroll(0, 1);
			this._doVirtual();
		},
	
		_onForcedScroll: function(){
			this._rowHeight = {};
			this._doScroll(0, 1);
			this._doVirtual();
		},

		//Private ---------------------------------------------------
		_avgRowHeight: 24,
		_rowHeight: null,
		_ratio: 1,
	
		_syncHeight: function(){
			var t = this,
				h = t._avgRowHeight * t.grid.body.visualCount,
				maxHeight = 1342177;
			if(sniff('ff')){
				maxHeight = 17895697;
			}else if(sniff('webkit')){
				maxHeight = 134217726;
			}
			if(h > maxHeight){
				t._ratio = h / maxHeight;
				h = maxHeight;
			}
			t.stubNode.style.height = h + 'px';
		},
	
		_doVirtual: function(){
			var t = this;
			clearTimeout(t._pVirtual);
			t._pVirtual = setTimeout(function(){
				t._updateRowHeight();
			}, 100);
		},
	
		_updateRowHeight: function(){
			//Update average row height and unrender rows
			var t = this,
				preCount = 0,
				postCount = 0,
				g = t.grid,
				bd = g.body,
				bn = g.bodyNode,
				buff = t.buffSize * t._avgRowHeight,
				st = bn.scrollTop,
				top = st - buff,
				bottom = st + bn.clientHeight + buff,
				rh = t._rowHeight;
	
			array.forEach(bn.childNodes, function(n){
				rh[n.getAttribute('rowid')] = n.offsetHeight;
				if(n.offsetTop > bottom){
					++postCount;
				}else if(n.offsetTop + n.offsetHeight < top){
					++preCount;
				}
			});
			bd.unrenderRows(preCount);
			bd.unrenderRows(postCount, 'post');
	
			var p, h = 0, c = 0;
			for(p in rh){
				h += rh[p];
				++c;
			}
			if(c){
				t._avgRowHeight = h / c;
				t._syncHeight();
			}
		},

		_onKeyScroll: function(evt){
			var t = this,
				bd = t.grid.body,
				focus = t.grid.focus,
				sn = t.domNode,
				st = 'scrollTop',
				r,
				fc = '_focusCellRow';
			if(!focus || focus.currentArea() == 'body'){
				if(evt.keyCode == keys.HOME){
					bd[fc] = 0;
					sn[st] = 0;
				}else if(evt.keyCode == keys.END){
					bd[fc] = bd.visualCount - 1;
					sn[st] = t.stubNode.clientHeight - bd.domNode.offsetHeight;
				}else if(evt.keyCode == keys.PAGE_UP){
					r = bd[fc] = Math.max(bd.renderStart - bd.renderCount, 0);
					t.scrollToRow(r);
				}else if(evt.keyCode == keys.PAGE_DOWN){
					r = bd[fc] = Math.min(bd.visualCount - 1, bd.renderStart + bd.renderCount);
					t.scrollToRow(r, 1);	//1 as true
				}else{
					return;
				}
				event.stop(evt);
			}
		}
	});
});

},
'gridx/modules/extendedSelect/Column':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/query",
	"dojo/_base/lang",
	"dojo/_base/sniff",
	"dojo/dom-class",
	"dojo/mouse",
	"dojo/keys",
	"../../core/_Module",
	"./_Base"
], function(declare, array, query, lang, sniff, domClass, mouse, keys, _Module, _Base){

	return declare(/*===== "gridx.modules.extendedSelect.Column", =====*/_Base, {
		// summary:
		//		Provides advanced column selections.
		// description:
		//		This module provides an advanced way for selecting columns by clicking, swiping, SPACE key, or CTRL/SHIFT CLICK to select multiple columns.
		//
		// example:
		//		1. Use select api on column object obtained from grid.column(i)
		//		|	grid.column(1).select();
		//		|	grid.column(1).deselect();
		//		|	grid.column(1).isSelected();
		//
		//		2. Use select api on select.row module
		//		|	grid.select.column.selectById(columnId);
		//		|	grid.select.column.deSelectById(columnId);
		//		|	grid.select.column.isSelected(columnId);
		//		|	grid.select.column.getSelected();//[]
		//		|	grid.select.column.clear();

		// name: [readonly] String
		//		module name
		name: 'selectColumn',

//        optional: ['columnResizer'],

		// columnMixin: Object
		//		A map of functions to be mixed into grid column object, so that we can use select api on column object directly
		//		- grid.column(1).select() | deselect() | isSelected();
		columnMixin: {
			select: function(){
				this.grid.select.column.selectById(this.id);
				return this;
			},
			deselect: function(){
				this.grid.select.column.deselectById(this.id);
				return this;
			},
			isSelected: function(){
				return !!this.grid._columnsById[this.id]._selected;
			}
		},

		//Public-----------------------------------------------------------------
/*=====
		selectById: function(columnId){
			// summary:
			//		Select a column by id.
		},
		
		deselectById: function(columnId){
			// summary:
			//		Deselect a column by id.
		},
		
		selectByIndex: function(columnIndex){
			// summary:
			//		Select a column by index
		},
		
		deSelectByIndex: function(columnIndex){
			// summary:
			//		Deselect a column by index.
		},		
=====*/
		
		getSelected: function(){
			// summary:
			//		Get id array of all selected columns
			return array.map(array.filter(this.grid._columns, function(col){
				return col._selected;
			}), function(col){
				return col.id;
			});
		},

		clear: function(silent){
			// summary:
			//		Deselected all selected columns;			
			query(".gridxColumnSelected", this.grid.domNode).forEach(function(node){
				domClass.remove(node, 'gridxColumnSelected');
				node.removeAttribute('aria-selected');
			});
			array.forEach(this.grid._columns, function(col){
				col._selected = 0;	//0 as false
			});
			this._clear();
			if(!silent){
				this._onSelectionChange();
			}
		},

		isSelected: function(){
			// summary:
			//		Check if the given column(s) are all selected.			
			var cols = this.grid._columnsById;
			return array.every(arguments, function(id){
				var col = cols[id];
				return col && col._selected;
			});
		},
		
		//Private---------------------------------------------------------------
		_type: 'column',

		_markById: function(args, toSelect){
			array.forEach(args, function(colId){
				var col = this.grid._columnsById[colId];
				if(col){
					col._selected = toSelect;
					this._doHighlight({column: col.index}, toSelect);
				}
			}, this);
		},

		_markByIndex: function(args, toSelect){
			var i, col, columns = this.grid._columns;
			for(i = 0; i < args.length; ++i){
				var arg = args[i];
				if(lang.isArrayLike(arg)){
					var start = arg[0],
						end = arg[1],
						count;
					if(start >= 0 && start < Infinity){
						if(!(end >= start && end < Infinity)){
							end = columns.length - 1;
						}
						for(; start < end + 1; ++start){
							col = columns[start];
							if(col){
								col._selected = toSelect;
								this._doHighlight({column: col.index}, toSelect);
							}
						}
					}
				}else if(arg >= 0 && arg < Infinity){
					col = columns[arg];
					if(col){
						col._selected = toSelect;
						this._doHighlight({column: arg}, toSelect);
					}
				}
			}
		},
		
		_init: function(){
			var t = this, g = t.grid;
			t.batchConnect(
				[g, 'onHeaderCellMouseDown', function(e){
					if(mouse.isLeft(e) && !domClass.contains(e.target, 'gridxArrowButtonNode')){
						t._start({column: e.columnIndex}, g._isCopyEvent(e), e.shiftKey);
					}
				}],
				[g, 'onHeaderCellMouseOver', function(e){
					t._highlight({column: e.columnIndex});
				}],
				[g, 'onCellMouseOver', function(e){
					t._highlight({column: e.columnIndex});
				}],
				[g, sniff('ff') < 4 ? 'onHeaderCellKeyUp' : 'onHeaderCellKeyDown', function(e){
					if((e.keyCode == keys.SPACE || e.keyCode == keys.ENTER) && !domClass.contains(e.target, 'gridxArrowButtonNode')){
						t._start({column: e.columnIndex}, g._isCopyEvent(e), e.shiftKey);
						t._end();
					}
				}],
				[g.header, 'onMoveToHeaderCell', '_onMoveToHeaderCell']
			);
		},

		_onRender: function(start, count){
			var i, j, end = start + count, g = this.grid, bn = g.bodyNode, node,
				cols = array.filter(g._columns, function(col){
					return col._selected;
				});
			for(i = cols.length - 1; i >= 0; --i){
				for(j = start; j < end; ++j){
					node = query(['[visualindex="', j, '"] [colid="', cols[i].id, '"]'].join(''), bn)[0];
					domClass.add(node, 'gridxColumnSelected');
					node.setAttribute('aria-selected', true);
				}
			}
		},

		_onMoveToHeaderCell: function(columnId, e){
			if(e.shiftKey && (e.keyCode == keys.LEFT_ARROW || e.keyCode == keys.RIGHT_ARROW)){
				var t = this, col = t.grid._columnsById[columnId];
				t._start({column: col.index}, t.grid._isCopyEvent(e), 1);	//1 as true
				t._end();
			}
		},

		_isSelected: function(target){
			var t = this, col = t.grid._columns[target.column], id = col.id;
			return t._isRange ? array.indexOf(t._refSelectedIds, id) >= 0 : col._selected;
		},

		_beginAutoScroll: function(){
			var autoScroll = this.grid.autoScroll;
			this._autoScrollV = autoScroll.vertical;
			autoScroll.vertical = false;
		},

		_endAutoScroll: function(){
			this.grid.autoScroll.vertical = this._autoScrollV;
		},

		_doHighlight: function(target, toHighlight){
			query('[colid="' + this.grid._columns[target.column].id + '"].gridxCell', this.grid.domNode).forEach(function(node){
				domClass.toggle(node, 'gridxColumnSelected', toHighlight);
			});
		},

		_focus: function(target){
			var g = this.grid;
			if(g.focus){
				//Seems breaking encapsulation...
				g.header._focusNode(query('[colid="' + g._columns[target.column].id + '"].gridxCell', g.header.domNode)[0]);
			}
		},

		_addToSelected: function(start, end, toSelect){
			var t = this, g = t.grid, a, i;
			if(!t._isRange){
				t._refSelectedIds = t.getSelected();
			}
			if(t._isRange && t._inRange(end.column, start.column, t._lastEndItem.column)){
				start = Math.min(end.column, t._lastEndItem.column);
				end = Math.max(end.column, t._lastEndItem.column);
				for(i = start; i <= end; ++i){
					g._columns[i]._selected = array.indexOf(t._refSelectedIds, g._columns[i].id) >= 0;
				}
			}else{
				a = Math.min(start.column, end.column);
				end = Math.max(start.column, end.column);
				start = a;
				for(i = start; i <= end; ++i){
					g._columns[i]._selected = toSelect;
				}
			}
		}
	});
});

},
'dojoe/wizard/Page':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/i18n!dojoe/wizard/nls/Wizard","dojo/require!dijit/_Widget,dijit/_Templated,dijit/layout/BorderContainer,dijit/layout/StackContainer,dijit/layout/ContentPane,dijit/form/Button,dijit/form/_FormMixin,dojo/i18n"], function(dijit,dojo,dojox){
dojo.provide("dojoe.wizard.Page");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.layout.BorderContainer");
dojo.require("dijit.layout.StackContainer");
dojo.require("dijit.layout.ContentPane");
dojo.require("dijit.form.Button");
dojo.require("dijit.form._FormMixin");
dojo.require("dojo.i18n");
dojo.requireLocalization("dojoe.wizard", "Wizard");

/**
 * 
 * A page that may be used in both the notebook and wizard widgets.
 * 
 * For the wizard, it handles substeps.
 * 
 */
dojo.declare("dojoe.wizard.Page", [dijit._Widget, dijit.layout._ContentPaneResizeMixin ,dijit._Templated, dijit.form._FormMixin], 
	{
		widgetsInTemplate: true,
		title: "",			// title is show in the ToC
		visited: false,			// has this page been visited?
		validated: false,		// has this page been validated?
		substeps: null,			// collection of substeps for this main step
		substepsList: null,		// ordered list of the substep list items for this main step
		listItem: null,			// list entry
		connectionLink: null,	// its "connect" link
		disabled: false, 
		required: false,
		loops: false,			// if true, this page and substeps form a loop
		titleOnly: false,		// if true and this main step has substeps, main page is never shown; only its
                         

	/**
	* Render the page itself
	*/
		buildRendering: function () {
			this.inherited(arguments);
			// Page subclasses ContentPane, but adds in _Templated, which ContentPane didn't expect
			// we have to fix some stuff.
			dojo.attr(this.domNode, "region", "center");
			this.containerNode = this.domNode;
			dojo.addClass(this.domNode, "tiv-page");
		},

	/**
	* Render this page's entry in the ToC.  The list item is created once. 
	* For wizard, it is used to jump back to the associated page.
	*/
		buildTOCRendring: function () {
			// if the list entry hasn't been created yet
			if (this.listItem == null) {
				// create it
				if (this.wizard == null) {
				// vertical notebook
					this.listItem = dojo.create("li", {
						innerHTML: "<div class='tiv-page-toc-icon'></div><div class='tiv-page-toc-title'>" + this.title + "</div>",
						pageID: this.id,
						className: this.visited ? "tiv-page-visited" : ""
					});        
				} else {
					// wizard
					if ('v' === this.wizard.layout) {
						// vertical toc
						this.listItem = dojo.create("li", {
							innerHTML: "<span class='tiv-page-toc-icon'></span><span class='tiv-page-required-flag'></span>" + "<div class='tiv-page-toc-title'>" + this.title + "</div>",
							pageID: this.id,
							className: this.visited ? "tiv-page-visited" : ""
						});        
					} else {
						// horizontal toc
						if (this.isMainStep()) {
							if (!this.wizard.renderNoArrows) {
								// onlick function is needed so link is not followed
								// link wraps entire arrow for tabbing
								this.listItem = dojo.create("li", {
									innerHTML: "<a href='#' onclick='return false;'><i></i><span><em></em>" + this.title + "</span><b></b></a>",
									pageID: this.id,
									className: this.visited ? "tiv-page-visited" : ""
								});
							} else {
								// render without arrow graphics like substeps 
								// required flag is applicable
								this.listItem = dojo.create("li", {
									innerHTML: "<a href='#' onclick='return false;'><span class='tiv-page-toc-title'><em></em>" + this.title + "</span></a>",
									pageID: this.id,
									className: this.visited ? "tiv-page-visited" : ""
								});                      
							}              
						} else {
							// substep
							this.listItem = dojo.create("li", {
								innerHTML: "<div class='tiv-page-toc-title'>" + this.title + "</div>",
								pageID: this.id,
								className: this.visited ? "tiv-page-visited" : ""
							});        
						}
					}
					// if this is a disabled page, mark it
					if (this.disabled) {
						this.setDisabled(true);
					} else {
						// connect it to this page
						this.connectionLink = dojo.connect(this.listItem, "onclick", this, this.jumpBack);
					}
					// if this is a required page, mark it
					if (this.required) {
						this.setRequired(true);      
					}
				}        
			}
			return this.listItem;
		},

		/**
		* This is always true; substeps return false.
		*/
		isMainStep: function () {
			return (true);
		},
  
		/**
		* Return true if page has substeps
		*/
		hasSubsteps: function () {
			if (this.substeps != null && this.substeps.length > 0) {
				return (true);
			} else {
				return (false);
			}
		},
  
		/**
		* Return ordered list of substep item entries
		*/
		getSubstepsList: function () {
			return this.substepsList;
		},

		/** 
		* Return true if the page is disabled
		*/
		isDisabled: function () {
			return (this.disabled);
		},

		/** 
		* Enable or disable a page
		*/
		setDisabled: function (aBoolean) {
			this.disabled = aBoolean;
			if (this.disabled) {
				dojo.addClass(this.listItem, "tiv-page-disabled");
			} else {
				dojo.removeClass(this.listItem, "tiv-page-disabled"); 
			}
		},

		/**
		* Turn on or off the "required" flag, which adds or removes a class
		* that controls the display of an icon.
		*/
		setRequired: function (aBoolean) {
			this.required = aBoolean;
			if (this.required) {
				dojo.addClass(this.listItem, "tiv-page-required");
			} else {
				dojo.removeClass(this.listItem, "tiv-page-required");
			}
			// need to update UI to allow for asterisk image
			this.wizard.updateUI();
		},
  
		/**
		* Returns true if step is part of a loop.
		*/
		isPartOfLoop: function () {
			return this.loops;
		},
  
		/**
		* Returns true if this is a main step with substeps and only the
		* title should show in the ToC, i.e. the main page itself is
		* never shown during navigation.
		*/
		showTitleOnly: function () {
			if (this.titleOnly && this.hasSubsteps()) {
				return true;
			} else {
				return false;
			}
		},
  
		/**
		* Indicate that main page should show or not when it contains substeps.
		*/
		setTitleOnly: function (titleOnly) {
			this.titleOnly = titleOnly;
		},

		/**
		* The page may want to validate the data before allowing the user to
		* move back a page.
		* Returning false will block the page container from moving back a page.
		* Putting up a message and moving the focus to the appropriate field
		* is the responsibility of the page.
		*/
		allowBack: function () {
			return (true);
		},

		/**
		* The page may want to validate the data before allowing the user to
		* move to the next page.
		* Returning false will block the page container from moving to the next page.
		* Putting up a message and moving the focus to the appropriate field
		* is the responsibility of the page.
		*/
		allowNext: function () {
			return (true);
		},

		/**
		* The page may want to validate the data before allowing the user to
		* use the "Finish" button.
		* Returning false will block the page container from allowing "Finish"
		* Putting up a message and moving the focus to the appropriate field
		* is the responsibility of the page.
		*/
		allowFinish: function () {
			return (true);
		},

		/**
		* When jumping back to a page, the current page is marked as 
		* visited (or complete).  Override this function and return
		* false to keep the current page as unvisited.
		*/
		allowMarkAsComplete: function () {
			return (true);    
		},
  
		/**
		* Disable jumping back to this page
		*/
		disableJumpBack: function () {
			// if there is a connection link
			if (this.connectionLink != null) {
				// disconnect the handler
				dojo.disconnect(this.connectionLink);
				// set it to null so it won't be used again
				this.connectionLink = null;
			}
			// add the "disabled" class
			dojo.addClass(this.listItem, "tiv-page-disabled");
		},

		/**
		* Enable jump back for the item
		* But not if it is disabled
		*/
		enableJumpBack: function () {
			if (this.disabled == null || this.disabled == false) {
				// if there is no handler set up
				if (this.connectionLink == null) {
					// create the handler
					this.connectionLink = dojo.connect(this.listItem, "onclick", this, this.jumpBack);
					// remove the "disabled" class
					dojo.removeClass(this.listItem, "tiv-page-disabled");
				}
			}
		},

		/**
		* The user wants to jump back to a previously visited page.
		* However, the "onclick" handler is attached when the table of
		* contents list item is created. The cursor will not be the "hand"
		* but the handler will still be called. So a check to see if this
		* page has been visited is made before letting the parent container
		* jump back to this page.
		*/
		jumpBack: function (event) {
			// if this page has been visited
			if (this.visited) {
				// then jump back to it
				this.wizard.jumpBackToPage(this);
			}
		},
  
		/**
		* Add another substep to the collection
		*/
		addSubstep: function (anotherSubstep) {
			if (this.substeps == null) {
				this.substeps = [];
			}
			this.substeps[this.substeps.length] = anotherSubstep;
		},

		/**
		* Returns the substeps for this page
		*/
		getSubsteps: function () {
			return this.substeps;
		},

		/**
		* Hide any substeps that this main step might have
		*/
		hideSubsteps: function () {
			// if there are any substeps
			if (this.substeps != null) {
				// if vertical toc
				if ('v' === this.wizard.layout) {
					// do for all substeps
					var i;
					for (i = 0; i < this.substeps.length; i++) {
						// get the table of contents entry
						var tocEntry = this.substeps[i].buildTOCRendring();
						// and hide it
						dojo.style(tocEntry, "display", "none");
					}
				} else {
					// horizontal mode
					if (this.wizard.tocSubstepsArea.childNodes.length > 0) {
						// remove substeps list
						this.wizard.tocSubstepsArea.removeChild(this.substepsList);
					}
				}
			}
		},
  
		/**
		* Show any substeps that this main step might have
		*/
		showSubsteps: function () {
			// if there are any substeps
			if (this.substeps != null) {
				// if vertical toc
				if (this.wizard == null || 'v' === this.wizard.layout) {
					// do for all substeps
					var i;
					for (i = 0; i < this.substeps.length; i++) {
						// get the table of contents entry
						var tocEntry = this.substeps[i].buildTOCRendring();
						// and make it visible
						dojo.style(tocEntry, "display", "block");
					}
				} else {
					// horizontal mode - add substeps ordered list to substeps area
					this.wizard.tocSubstepsArea.appendChild(this.substepsList);
				}
			}    
		},

		/**
		* Mark everything up to this page as having been visited
		*/
		markPreviousPagesVisited: function () {
			this.wizard.markPreviousPagesVisited(this);
		},

		// dijit.form._FormMixin will walk through all the child widgets checking their validity,
		// if you need to do something else, override
		isValid: function () {
			this.validated = this.inherited(arguments); 
			return this.validated;
		},
		
		// same as isValid, but moves focus to the first invalid child widget
		validate: function () {
			this.validated = this.inherited(arguments);
			return this.validated;
		},

		/**
		* Slightly different because substeps might be involved.
		* When a main step becomes the selected page all of its substeps
		* should be made visible.
		*/
		OnPageSelected: function (wiz) {
			// show substeps if there are any...
			if (this.wizard) {
			    this.showSubsteps();
			}
		},
  
		/**
		* catch OnChange events from my child widgets,
		* then let listeners know something's changed
		*/
		_onPageContentChange: function (evt) {
			this.OnPageContentChange(this);
		},
  
		OnPageContentChange: function (page) {
			// override or connect to me
		}
	});


});

},
'gridx/modules/pagination/_PaginationBarBase':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/array",
	"../../core/_Module"
], function(declare, array, _Module){

	return declare(/*===== "gridx.modules.pagination._PaginationBarBase", =====*/_Module, {
		name: 'paginationBar',	
	
		required: ['vLayout'],

		forced: ['pagination'],
		
		getAPIPath: function(){
			return {
				paginationBar: this
			};
		},
		
		preload: function(){
			var t = this,
				g = t.grid,
				vLayout = g.vLayout,
				p = g.pagination;
			t._pagers = [];
			//Register UI before startup
			if(t._exist('top')){
				vLayout.register(t, '_topPagerNode', 'headerNode', -5);
			}
			if(t._exist('bottom')){
				vLayout.register(t, '_bottomPagerNode', 'footerNode', 5);
			}
			if(!p.arg('initialPageSize')){
				p.initialPageSize = t.arg('sizes')[0];
			}
		},

		load: function(args, startup){
			var t = this;
			startup.then(function(){
				t._init();
				t.loaded.callback();
			});
		},
		
		destroy: function(){
			this.inherited(arguments);
			array.forEach(this._pagers, function(pager){
				pager.destroyRecursive();
			});
		},
		
		//Public-------------------------------------------------------

		// sizeSwitch: Boolean|String
		//		Whether (and where) to show "size switch" part of the pagination bar UI.
		//		Can be true/false, or "bottom", or "top"
		sizes: [10, 25, 50, 100, 0],

		// position: String
		//		The position of the pagination bar, can be "bottom" (default), "top" or "both" (any other value means "both")
		position: 'bottom',

		sizeSwitch: true,

		// stepper: Boolean|String
		//		Whether (and where) to show "page stepper" part of the pagination bar UI.
		//		Can be true/false, or "bottom", or "top"
		stepper: true,

		// description: Boolean|String
		//		Whether (and where) to show "description" part of the pagination bar UI.
		//		Can be true/false, or "bottom", or "top"
		description: true,
	
		refresh: function(){
			// summary:
			//		Refresh the pagination bar UI
			array.forEach(this._pagers, function(pager){
				pager.refresh();
			});
			this.grid.vLayout.reLayout();
		},
	
		//Private---------------------------------------------------------------------------------
		//pagerClass: null,

		_exist: function(pos, argName){
			var v = this.arg(argName || 'position');
			v = v && String(v).toLowerCase();
			return v && ((v != 'top' && v != 'bottom') || v == pos);
		},

		_init: function(){
			var t = this;
			array.forEach(['top', 'bottom'], function(pos){
				if(t._exist(pos)){
					var cls = t.arg('pagerClass'),
						pager = new cls({
							pagination: t.grid.pagination,
							module: t,
							position: pos,
							focusPriority: {
								top: -5,
								bottom: 5
							}[pos]
						});
					t._pagers.push(pager);
					t['_' + pos + 'PagerNode'] = pager.domNode;
				}
			});
		}
	});	
});

},
'dijit/_editor/selection':function(){
define([
	"dojo/dom", // dom.byId
	"dojo/_base/lang",
	"dojo/_base/sniff", // has("ie") has("opera")
	"dojo/_base/window", // win.body win.doc win.doc.createElement win.doc.selection win.doc.selection.createRange win.doc.selection.type.toLowerCase win.global win.global.getSelection
	".."		// for exporting symbols to dijit._editor.selection (TODO: remove in 2.0)
], function(dom, lang, has, win, dijit){

// module:
//		dijit/_editor/selection
// summary:
//		Text selection API


lang.getObject("_editor.selection", true, dijit);

// FIXME:
//		all of these methods branch internally for IE. This is probably
//		sub-optimal in terms of runtime performance. We should investigate the
//		size difference for differentiating at definition time.

lang.mixin(dijit._editor.selection, {
	getType: function(){
		// summary:
		//		Get the selection type (like win.doc.select.type in IE).
		if(has("ie") < 9){
			return win.doc.selection.type.toLowerCase();
		}else{
			var stype = "text";

			// Check if the actual selection is a CONTROL (IMG, TABLE, HR, etc...).
			var oSel;
			try{
				oSel = win.global.getSelection();
			}catch(e){ /*squelch*/ }

			if(oSel && oSel.rangeCount == 1){
				var oRange = oSel.getRangeAt(0);
				if(	(oRange.startContainer == oRange.endContainer) &&
					((oRange.endOffset - oRange.startOffset) == 1) &&
					(oRange.startContainer.nodeType != 3 /* text node*/)
				){
					stype = "control";
				}
			}
			return stype; //String
		}
	},

	getSelectedText: function(){
		// summary:
		//		Return the text (no html tags) included in the current selection or null if no text is selected
		if(has("ie") < 9){
			if(dijit._editor.selection.getType() == 'control'){
				return null;
			}
			return win.doc.selection.createRange().text;
		}else{
			var selection = win.global.getSelection();
			if(selection){
				return selection.toString(); //String
			}
		}
		return '';
	},

	getSelectedHtml: function(){
		// summary:
		//		Return the html text of the current selection or null if unavailable
		if(has("ie") < 9){
			if(dijit._editor.selection.getType() == 'control'){
				return null;
			}
			return win.doc.selection.createRange().htmlText;
		}else{
			var selection = win.global.getSelection();
			if(selection && selection.rangeCount){
				var i;
				var html = "";
				for(i = 0; i < selection.rangeCount; i++){
					//Handle selections spanning ranges, such as Opera
					var frag = selection.getRangeAt(i).cloneContents();
					var div = win.doc.createElement("div");
					div.appendChild(frag);
					html += div.innerHTML;
				}
				return html; //String
			}
			return null;
		}
	},

	getSelectedElement: function(){
		// summary:
		//		Retrieves the selected element (if any), just in the case that
		//		a single element (object like and image or a table) is
		//		selected.
		if(dijit._editor.selection.getType() == "control"){
			if(has("ie") < 9){
				var range = win.doc.selection.createRange();
				if(range && range.item){
					return win.doc.selection.createRange().item(0);
				}
			}else{
				var selection = win.global.getSelection();
				return selection.anchorNode.childNodes[ selection.anchorOffset ];
			}
		}
		return null;
	},

	getParentElement: function(){
		// summary:
		//		Get the parent element of the current selection
		if(dijit._editor.selection.getType() == "control"){
			var p = this.getSelectedElement();
			if(p){ return p.parentNode; }
		}else{
			if(has("ie") < 9){
				var r = win.doc.selection.createRange();
				r.collapse(true);
				return r.parentElement();
			}else{
				var selection = win.global.getSelection();
				if(selection){
					var node = selection.anchorNode;
					while(node && (node.nodeType != 1)){ // not an element
						node = node.parentNode;
					}
					return node;
				}
			}
		}
		return null;
	},

	hasAncestorElement: function(/*String*/tagName /* ... */){
		// summary:
		// 		Check whether current selection has a  parent element which is
		// 		of type tagName (or one of the other specified tagName)
		// tagName: String
		//		The tag name to determine if it has an ancestor of.
		return this.getAncestorElement.apply(this, arguments) != null; //Boolean
	},

	getAncestorElement: function(/*String*/tagName /* ... */){
		// summary:
		//		Return the parent element of the current selection which is of
		//		type tagName (or one of the other specified tagName)
		// tagName: String
		//		The tag name to determine if it has an ancestor of.
		var node = this.getSelectedElement() || this.getParentElement();
		return this.getParentOfType(node, arguments); //DOMNode
	},

	isTag: function(/*DomNode*/ node, /*String[]*/ tags){
		// summary:
		//		Function to determine if a node is one of an array of tags.
		// node:
		//		The node to inspect.
		// tags:
		//		An array of tag name strings to check to see if the node matches.
		if(node && node.tagName){
			var _nlc = node.tagName.toLowerCase();
			for(var i=0; i<tags.length; i++){
				var _tlc = String(tags[i]).toLowerCase();
				if(_nlc == _tlc){
					return _tlc; // String
				}
			}
		}
		return "";
	},

	getParentOfType: function(/*DomNode*/ node, /*String[]*/ tags){
		// summary:
		//		Function to locate a parent node that matches one of a set of tags
		// node:
		//		The node to inspect.
		// tags:
		//		An array of tag name strings to check to see if the node matches.
		while(node){
			if(this.isTag(node, tags).length){
				return node; // DOMNode
			}
			node = node.parentNode;
		}
		return null;
	},

	collapse: function(/*Boolean*/beginning){
		// summary:
		//		Function to collapse (clear), the current selection
		// beginning: Boolean
		//		Boolean to indicate whether to collapse the cursor to the beginning of the selection or end.
		if(window.getSelection){
			var selection = win.global.getSelection();
			if(selection.removeAllRanges){ // Mozilla
				if(beginning){
					selection.collapseToStart();
				}else{
					selection.collapseToEnd();
				}
			}else{ // Safari
				// pulled from WebCore/ecma/kjs_window.cpp, line 2536
				selection.collapse(beginning);
			}
		}else if(has("ie")){ // IE
			var range = win.doc.selection.createRange();
			range.collapse(beginning);
			range.select();
		}
	},

	remove: function(){
		// summary:
		//		Function to delete the currently selected content from the document.
		var sel = win.doc.selection;
		if(has("ie") < 9){
			if(sel.type.toLowerCase() != "none"){
				sel.clear();
			}
			return sel; //Selection
		}else{
			sel = win.global.getSelection();
			sel.deleteFromDocument();
			return sel; //Selection
		}
	},

	selectElementChildren: function(/*DomNode*/element,/*Boolean?*/nochangefocus){
		// summary:
		//		clear previous selection and select the content of the node
		//		(excluding the node itself)
		// element: DOMNode
		//		The element you wish to select the children content of.
		// nochangefocus: Boolean
		//		Boolean to indicate if the foxus should change or not.
		var global = win.global;
		var doc = win.doc;
		var range;
		element = dom.byId(element);
		if(doc.selection && has("ie") < 9 && win.body().createTextRange){ // IE
			range = element.ownerDocument.body.createTextRange();
			range.moveToElementText(element);
			if(!nochangefocus){
				try{
					range.select(); // IE throws an exception here if the widget is hidden.  See #5439
				}catch(e){ /* squelch */}
			}
		}else if(global.getSelection){
			var selection = win.global.getSelection();
			if(has("opera")){
				//Opera's selectAllChildren doesn't seem to work right
				//against <body> nodes and possibly others ... so
				//we use the W3C range API
				if(selection.rangeCount){
					range = selection.getRangeAt(0);
				}else{
					range = doc.createRange();
				}
				range.setStart(element, 0);
				range.setEnd(element,(element.nodeType == 3)?element.length:element.childNodes.length);
				selection.addRange(range);
			}else{
				selection.selectAllChildren(element);
			}
		}
	},

	selectElement: function(/*DomNode*/element,/*Boolean?*/nochangefocus){
		// summary:
		//		clear previous selection and select element (including all its children)
		// element:  DOMNode
		//		The element to select.
		// nochangefocus: Boolean
		//		Boolean indicating if the focus should be changed.  IE only.
		var range;
		var doc = win.doc;
		var global = win.global;
		element = dom.byId(element);
		if(has("ie") < 9 && win.body().createTextRange){
			try{
				var tg = element.tagName ? element.tagName.toLowerCase() : "";
				if(tg === "img" || tg === "table"){
					range = win.body().createControlRange();
				}else{
					range = win.body().createRange();
				}
				range.addElement(element);
				if(!nochangefocus){
					range.select();
				}
			}catch(e){
				this.selectElementChildren(element,nochangefocus);
			}
		}else if(global.getSelection){
			var selection = global.getSelection();
			range = doc.createRange();
			if(selection.removeAllRanges){ // Mozilla
				// FIXME: does this work on Safari?
				if(has("opera")){
					//Opera works if you use the current range on
					//the selection if present.
					if(selection.getRangeAt(0)){
						range = selection.getRangeAt(0);
					}
				}
				range.selectNode(element);
				selection.removeAllRanges();
				selection.addRange(range);
			}
		}
	},

	inSelection: function(node){
		// summary:
		//		This function determines if 'node' is
		//		in the current selection.
		// tags:
		//		public
		if(node){
			var newRange;
			var doc = win.doc;
			var range;

			if(win.global.getSelection){
				//WC3
				var sel = win.global.getSelection();
				if(sel && sel.rangeCount > 0){
					range = sel.getRangeAt(0);
				}
				if(range && range.compareBoundaryPoints && doc.createRange){
					try{
						newRange = doc.createRange();
						newRange.setStart(node, 0);
						if(range.compareBoundaryPoints(range.START_TO_END, newRange) === 1){
							return true;
						}
					}catch(e){ /* squelch */}
				}
			}else if(doc.selection){
				// Probably IE, so we can't use the range object as the pseudo
				// range doesn't implement the boundry checking, we have to
				// use IE specific crud.
				range = doc.selection.createRange();
				try{
					newRange = node.ownerDocument.body.createControlRange();
					if(newRange){
						newRange.addElement(node);
					}
				}catch(e1){
					try{
						newRange = node.ownerDocument.body.createTextRange();
						newRange.moveToElementText(node);
					}catch(e2){/* squelch */}
				}
				if(range && newRange){
					// We can finally compare similar to W3C
					if(range.compareEndPoints("EndToStart", newRange) === 1){
						return true;
					}
				}
			}
		}
		return false; // boolean
	}

});

return dijit._editor.selection;
});

},
'dojox/grid/_Builder':function(){
define([
	"../main",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/_base/window",
	"dojo/_base/event",
	"dojo/_base/sniff",
	"dojo/_base/connect",
	"dojo/dnd/Moveable",
	"dojox/html/metrics",
	"./util",
	"dojo/_base/html"
], function(dojox, array, lang, win, event, has, connect, Moveable, metrics, util, html){

	var dg = dojox.grid;

	var getTdIndex = function(td){
		return td.cellIndex >=0 ? td.cellIndex : array.indexOf(td.parentNode.cells, td);
	};
	
	var getTrIndex = function(tr){
		return tr.rowIndex >=0 ? tr.rowIndex : array.indexOf(tr.parentNode.childNodes, tr);
	};
	
	var getTr = function(rowOwner, index){
		return rowOwner && ((rowOwner.rows||0)[index] || rowOwner.childNodes[index]);
	};

	var findTable = function(node){
		for(var n=node; n && n.tagName!='TABLE'; n=n.parentNode){}
		return n;
	};
	
	var ascendDom = function(inNode, inWhile){
		for(var n=inNode; n && inWhile(n); n=n.parentNode){}
		return n;
	};
	
	var makeNotTagName = function(inTagName){
		var name = inTagName.toUpperCase();
		return function(node){ return node.tagName != name; };
	};

	var rowIndexTag = util.rowIndexTag;
	var gridViewTag = util.gridViewTag;

	// base class for generating markup for the views
	var _Builder = dg._Builder = lang.extend(function(view){
		if(view){
			this.view = view;
			this.grid = view.grid;
		}
	},{
		view: null,
		// boilerplate HTML
		_table: '<table class="dojoxGridRowTable" border="0" cellspacing="0" cellpadding="0" role="presentation"',

		// Returns the table variable as an array - and with the view width, if specified
		getTableArray: function(){
			var html = [this._table];
			if(this.view.viewWidth){
				html.push([' style="width:', this.view.viewWidth, ';"'].join(''));
			}
			html.push('>');
			return html;
		},
		
		// generate starting tags for a cell
		generateCellMarkup: function(inCell, inMoreStyles, inMoreClasses, isHeader){
			var result = [], html;
			if(isHeader){
				var sortInfo = inCell.index != inCell.grid.getSortIndex() ? "" : inCell.grid.sortInfo > 0 ? 'aria-sort="ascending"' : 'aria-sort="descending"';
				if (!inCell.id){
					inCell.id = this.grid.id + "Hdr" + inCell.index;
				}
				// column headers are not editable, mark as aria-readonly=true
				html = ['<th tabIndex="-1" aria-readonly="true" role="columnheader"', sortInfo, ' id="', inCell.id, '"'];
			}else{
				// cells inherit grid aria-readonly property; default value for aria-readonly is false(grid is editable)
				// if grid is editable (had any editable cells), mark non editable cells as aria-readonly=true
				// if no editable cells, grid's aria-readonly value will have been set to true and cells will inherit
				var editInfo = this.grid.editable && !inCell.editable ? 'aria-readonly="true"' : "";
				// When multiview, JAWS will read corresponding headers on every view, so can't tell which one is the header of this cell
				// So adding aria-describedby. Don't know better solution. Maybe this is a WAI-ARIA issue.
				html = ['<td tabIndex="-1" role="gridcell" ', editInfo, ' aria-describedby="', this.grid.id, "Hdr", inCell.index, '"'];
			}
			if(inCell.colSpan){
				html.push(' colspan="', inCell.colSpan, '"');
			}
			if(inCell.rowSpan){
				html.push(' rowspan="', inCell.rowSpan, '"');
			}
			html.push(' class="dojoxGridCell ');
			if(inCell.classes){
				html.push(inCell.classes, ' ');
			}
			if(inMoreClasses){
				html.push(inMoreClasses, ' ');
			}
			// result[0] => td opener, style
			result.push(html.join(''));
			// SLOT: result[1] => td classes
			result.push('');
			html = ['" idx="', inCell.index, '" style="'];
			if(inMoreStyles && inMoreStyles[inMoreStyles.length-1] != ';'){
				inMoreStyles += ';';
			}
			html.push(inCell.styles, inMoreStyles||'', inCell.hidden?'display:none;':'');
			if(inCell.unitWidth){
				html.push('width:', inCell.unitWidth, ';');
			}
			// result[2] => markup
			result.push(html.join(''));
			// SLOT: result[3] => td style
			result.push('');
			html = [ '"' ];
			if(inCell.attrs){
				html.push(" ", inCell.attrs);
			}
			html.push('>');
			// result[4] => td postfix
			result.push(html.join(''));
			// SLOT: result[5] => content
			result.push('');
			// result[6] => td closes
			result.push(isHeader?'</th>':'</td>');
			return result; // Array
		},

		// cell finding
		isCellNode: function(inNode){
			return Boolean(inNode && inNode!=win.doc && html.attr(inNode, "idx"));
		},
		
		getCellNodeIndex: function(inCellNode){
			return inCellNode ? Number(html.attr(inCellNode, "idx")) : -1;
		},
		
		getCellNode: function(inRowNode, inCellIndex){
			for(var i=0, row; ((row = getTr(inRowNode.firstChild, i)) && row.cells); i++){
				for(var j=0, cell; (cell = row.cells[j]); j++){
					if(this.getCellNodeIndex(cell) == inCellIndex){
						return cell;
					}
				}
			}
			return null;
		},
		
		findCellTarget: function(inSourceNode, inTopNode){
			var n = inSourceNode;
			while(n && (!this.isCellNode(n) || (n.offsetParent && gridViewTag in n.offsetParent.parentNode && n.offsetParent.parentNode[gridViewTag] != this.view.id)) && (n!=inTopNode)){
				n = n.parentNode;
			}
			return n!=inTopNode ? n : null;
		},
		
		// event decoration
		baseDecorateEvent: function(e){
			e.dispatch = 'do' + e.type;
			e.grid = this.grid;
			e.sourceView = this.view;
			e.cellNode = this.findCellTarget(e.target, e.rowNode);
			e.cellIndex = this.getCellNodeIndex(e.cellNode);
			e.cell = (e.cellIndex >= 0 ? this.grid.getCell(e.cellIndex) : null);
		},
		
		// event dispatch
		findTarget: function(inSource, inTag){
			var n = inSource;
			while(n && (n!=this.domNode) && (!(inTag in n) || (gridViewTag in n && n[gridViewTag] != this.view.id))){
				n = n.parentNode;
			}
			return (n != this.domNode) ? n : null;
		},

		findRowTarget: function(inSource){
			return this.findTarget(inSource, rowIndexTag);
		},

		isIntraNodeEvent: function(e){
			try{
				return (e.cellNode && e.relatedTarget && html.isDescendant(e.relatedTarget, e.cellNode));
			}catch(x){
				// e.relatedTarget has permission problem in FF if it's an input: https://bugzilla.mozilla.org/show_bug.cgi?id=208427
				return false;
			}
		},

		isIntraRowEvent: function(e){
			try{
				var row = e.relatedTarget && this.findRowTarget(e.relatedTarget);
				return !row && (e.rowIndex==-1) || row && (e.rowIndex==row.gridRowIndex);
			}catch(x){
				// e.relatedTarget on INPUT has permission problem in FF: https://bugzilla.mozilla.org/show_bug.cgi?id=208427
				return false;
			}
		},

		dispatchEvent: function(e){
			if(e.dispatch in this){
				return this[e.dispatch](e);
			}
			return false;
		},

		// dispatched event handlers
		domouseover: function(e){
			if(e.cellNode && (e.cellNode!=this.lastOverCellNode)){
				this.lastOverCellNode = e.cellNode;
				this.grid.onMouseOver(e);
			}
			this.grid.onMouseOverRow(e);
		},

		domouseout: function(e){
			if(e.cellNode && (e.cellNode==this.lastOverCellNode) && !this.isIntraNodeEvent(e, this.lastOverCellNode)){
				this.lastOverCellNode = null;
				this.grid.onMouseOut(e);
				if(!this.isIntraRowEvent(e)){
					this.grid.onMouseOutRow(e);
				}
			}
		},
		
		domousedown: function(e){
			if (e.cellNode)
				this.grid.onMouseDown(e);
			this.grid.onMouseDownRow(e);
		}
	});

	// Produces html for grid data content. Owned by grid and used internally
	// for rendering data. Override to implement custom rendering.
	var _ContentBuilder = dg._ContentBuilder = lang.extend(function(view){
		_Builder.call(this, view);
	},_Builder.prototype,{
		update: function(){
			this.prepareHtml();
		},

		// cache html for rendering data rows
		prepareHtml: function(){
			var defaultGet=this.grid.get, cells=this.view.structure.cells;
			for(var j=0, row; (row=cells[j]); j++){
				for(var i=0, cell; (cell=row[i]); i++){
					cell.get = cell.get || (cell.value == undefined) && defaultGet;
					cell.markup = this.generateCellMarkup(cell, cell.cellStyles, cell.cellClasses, false);
					if (!this.grid.editable && cell.editable){
						this.grid.editable = true;
					}
				}
			}
		},

		// time critical: generate html using cache and data source
		generateHtml: function(inDataIndex, inRowIndex){
			var
				html = this.getTableArray(),
				v = this.view,
				cells = v.structure.cells,
				item = this.grid.getItem(inRowIndex);

			util.fire(this.view, "onBeforeRow", [inRowIndex, cells]);
			for(var j=0, row; (row=cells[j]); j++){
				if(row.hidden || row.header){
					continue;
				}
				html.push(!row.invisible ? '<tr>' : '<tr class="dojoxGridInvisible">');
				for(var i=0, cell, m, cc, cs; (cell=row[i]); i++){
					m = cell.markup; cc = cell.customClasses = []; cs = cell.customStyles = [];
					// content (format can fill in cc and cs as side-effects)
					m[5] = cell.format(inRowIndex, item);
					m[5] = util.fixIEEmptyCell(m[5]);
					// classes
					m[1] = cc.join(' ');
					// styles
					m[3] = cs.join(';');
					// in-place concat
					html.push.apply(html, m);
				}
				html.push('</tr>');
			}
			html.push('</table>');
			return html.join(''); // String
		},

		decorateEvent: function(e){
			e.rowNode = this.findRowTarget(e.target);
			if(!e.rowNode){return false;}
			e.rowIndex = e.rowNode[rowIndexTag];
			this.baseDecorateEvent(e);
			e.cell = this.grid.getCell(e.cellIndex);
			return true; // Boolean
		}
	});

	// Produces html for grid header content. Owned by grid and used internally
	// for rendering data. Override to implement custom rendering.
	var _HeaderBuilder = dg._HeaderBuilder = lang.extend(function(view){
		this.moveable = null;
		_Builder.call(this, view);
	},_Builder.prototype,{
		_skipBogusClicks: false,
		overResizeWidth: 4,
		minColWidth: 1,
		
		update: function(){
			if(this.tableMap){
				this.tableMap.mapRows(this.view.structure.cells);
			}else{
				this.tableMap = new dg._TableMap(this.view.structure.cells);
			}
		},

		generateHtml: function(inGetValue, inValue){
			var html = this.getTableArray(), cells = this.view.structure.cells;
			
			util.fire(this.view, "onBeforeRow", [-1, cells]);
			for(var j=0, row; (row=cells[j]); j++){
				if(row.hidden){
					continue;
				}
				html.push(!row.invisible ? '<tr>' : '<tr class="dojoxGridInvisible">');
				for(var i=0, cell, markup; (cell=row[i]); i++){
					cell.customClasses = [];
					cell.customStyles = [];
					if(this.view.simpleStructure){
						if(cell.draggable){
							if(cell.headerClasses){
								if(cell.headerClasses.indexOf('dojoDndItem') == -1){
									cell.headerClasses += ' dojoDndItem';
								}
							}else{
								cell.headerClasses = 'dojoDndItem';
							}
						}
						if(cell.attrs){
							if(cell.attrs.indexOf("dndType='gridColumn_") == -1){
								cell.attrs += " dndType='gridColumn_" + this.grid.id + "' ";
							}
						}else{
							cell.attrs = "dndType='gridColumn_" + this.grid.id + "' ";
						}
					}
					markup = this.generateCellMarkup(cell, cell.headerStyles, cell.headerClasses, true);
					// content
					markup[5] = (inValue != undefined ? inValue : inGetValue(cell));
					// styles
					markup[3] = cell.customStyles.join(';');
					// classes
					markup[1] = cell.customClasses.join(' '); //(cell.customClasses ? ' ' + cell.customClasses : '');
					html.push(markup.join(''));
				}
				html.push('</tr>');
			}
			html.push('</table>');
			return html.join('');
		},

		// event helpers
		getCellX: function(e){
			var n, x = e.layerX;
			if(has("mozilla") || has("ie") >= 9){
				n = ascendDom(e.target, makeNotTagName("th"));
				x -= (n && n.offsetLeft) || 0;
				var t = e.sourceView.getScrollbarWidth();
				if(!this.grid.isLeftToRight()/*&& e.sourceView.headerNode.scrollLeft < t*/){
					//fix #11253
					table = ascendDom(n,makeNotTagName("table"));
					x -= (table && table.offsetLeft) || 0;
				}
				//x -= getProp(ascendDom(e.target, mkNotTagName("td")), "offsetLeft") || 0;
			}
			n = ascendDom(e.target, function(){
				if(!n || n == e.cellNode){
					return false;
				}
				// Mozilla 1.8 (FF 1.5) has a bug that makes offsetLeft = -parent border width
				// when parent has border, overflow: hidden, and is positioned
				// handle this problem here ... not a general solution!
				x += (n.offsetLeft < 0 ? 0 : n.offsetLeft);
				return true;
			});
			return x;
		},

		// event decoration
		decorateEvent: function(e){
			this.baseDecorateEvent(e);
			e.rowIndex = -1;
			e.cellX = this.getCellX(e);
			return true;
		},

		// event handlers
		// resizing
		prepareResize: function(e, mod){
			do{
				var i = e.cellIndex;
				e.cellNode = (i ? e.cellNode.parentNode.cells[i+mod] : null);
				e.cellIndex = (e.cellNode ? this.getCellNodeIndex(e.cellNode) : -1);
			}while(e.cellNode && e.cellNode.style.display == "none");
			return Boolean(e.cellNode);
		},

		canResize: function(e){
			if(!e.cellNode || e.cellNode.colSpan > 1){
				return false;
			}
			var cell = this.grid.getCell(e.cellIndex);
			return !cell.noresize && cell.canResize();
		},

		overLeftResizeArea: function(e){
			// We are never over a resize area if we are in the process of moving
			if(html.hasClass(win.body(), "dojoDndMove")){
				return false;
			}
			//Bugfix for crazy IE problem (#8807).  IE returns position information for the icon and text arrow divs
			//as if they were still on the left instead of returning the position they were 'float: right' to.
			//So, the resize check ends up checking the wrong adjacent cell.  This checks to see if the hover was over
			//the image or text nodes, then just ignored them/treat them not in scale range.
			if(has("ie")){
				var tN = e.target;
				if(html.hasClass(tN, "dojoxGridArrowButtonNode") ||
					html.hasClass(tN, "dojoxGridArrowButtonChar")){
					return false;
				}
			}

			if(this.grid.isLeftToRight()){
				return (e.cellIndex>0) && (e.cellX > 0 && e.cellX < this.overResizeWidth) && this.prepareResize(e, -1);
			}
			var t = e.cellNode && (e.cellX > 0 && e.cellX < this.overResizeWidth);
			return t;
		},

		overRightResizeArea: function(e){
			// We are never over a resize area if we are in the process of moving
			if(html.hasClass(win.body(), "dojoDndMove")){
				return false;
			}
			//Bugfix for crazy IE problem (#8807).  IE returns position information for the icon and text arrow divs
			//as if they were still on the left instead of returning the position they were 'float: right' to.
			//So, the resize check ends up checking the wrong adjacent cell.  This checks to see if the hover was over
			//the image or text nodes, then just ignored them/treat them not in scale range.
			if(has("ie")){
				var tN = e.target;
				if(html.hasClass(tN, "dojoxGridArrowButtonNode") ||
					html.hasClass(tN, "dojoxGridArrowButtonChar")){
					return false;
				}
			}

			if(this.grid.isLeftToRight()){
				return e.cellNode && (e.cellX >= e.cellNode.offsetWidth - this.overResizeWidth);
			}
			return (e.cellIndex>0) && (e.cellX >= e.cellNode.offsetWidth - this.overResizeWidth) && this.prepareResize(e, -1);
		},

		domousemove: function(e){
			//console.log(e.cellIndex, e.cellX, e.cellNode.offsetWidth);
			if(!this.moveable){
				var c = (this.overRightResizeArea(e) ? 'dojoxGridColResize' : (this.overLeftResizeArea(e) ? 'dojoxGridColResize' : ''));
				if(c && !this.canResize(e)){
					c = 'dojoxGridColNoResize';
				}
				html.toggleClass(e.sourceView.headerNode, "dojoxGridColNoResize", (c == "dojoxGridColNoResize"));
				html.toggleClass(e.sourceView.headerNode, "dojoxGridColResize", (c == "dojoxGridColResize"));
				if(c){
					event.stop(e);
				}
			}
		},

		domousedown: function(e){
			if(!this.moveable){
				if((this.overRightResizeArea(e) || this.overLeftResizeArea(e)) && this.canResize(e)){
					this.beginColumnResize(e);
				}else{
					this.grid.onMouseDown(e);
					this.grid.onMouseOverRow(e);
				}
				//else{
				//	this.beginMoveColumn(e);
				//}
			}
		},

		doclick: function(e) {
			if(this._skipBogusClicks){
				event.stop(e);
				return true;
			}
			return false;
		},

		// column resizing
		colResizeSetup: function(/*Event Object*/e, /*boolean*/ isMouse ){
			//Set up the drag object for column resizing
			// Called with mouse event in case of drag and drop,
			// Also called from keyboard shift-arrow event when focus is on a header
			var headContentBox = html.contentBox(e.sourceView.headerNode);
			
			if(isMouse){  //IE draws line even with no mouse down so separate from keyboard
				this.lineDiv = document.createElement('div');

				var vw = html.position(e.sourceView.headerNode, true);
				var bodyContentBox = html.contentBox(e.sourceView.domNode);
				//fix #11340
				var l = e.pageX;
				if(!this.grid.isLeftToRight() && has("ie") < 8){
					l -= metrics.getScrollbar().w;
				}
				html.style(this.lineDiv, {
					top: vw.y + "px",
					left: l + "px",
					height: (bodyContentBox.h + headContentBox.h) + "px"
				});
				html.addClass(this.lineDiv, "dojoxGridResizeColLine");
				this.lineDiv._origLeft = l;
				win.body().appendChild(this.lineDiv);
			}
			var spanners = [], nodes = this.tableMap.findOverlappingNodes(e.cellNode);
			for(var i=0, cell; (cell=nodes[i]); i++){
				spanners.push({ node: cell, index: this.getCellNodeIndex(cell), width: cell.offsetWidth });
				//console.log("spanner: " + this.getCellNodeIndex(cell));
			}

			var view = e.sourceView;
			var adj = this.grid.isLeftToRight() ? 1 : -1;
			var views = e.grid.views.views;
			var followers = [];
			for(var j=view.idx+adj, cView; (cView=views[j]); j=j+adj){
				followers.push({ node: cView.headerNode, left: window.parseInt(cView.headerNode.style.left) });
			}
			var table = view.headerContentNode.firstChild;
			var drag = {
				scrollLeft: e.sourceView.headerNode.scrollLeft,
				view: view,
				node: e.cellNode,
				index: e.cellIndex,
				w: html.contentBox(e.cellNode).w,
				vw: headContentBox.w,
				table: table,
				tw: html.contentBox(table).w,
				spanners: spanners,
				followers: followers
			};
			return drag;
		},
		beginColumnResize: function(e){
			this.moverDiv = document.createElement("div");
			html.style(this.moverDiv,{position: "absolute", left:0}); // to make DnD work with dir=rtl
			win.body().appendChild(this.moverDiv);
			html.addClass(this.grid.domNode, "dojoxGridColumnResizing");
			var m = (this.moveable = new Moveable(this.moverDiv));

			var drag = this.colResizeSetup(e,true);

			m.onMove = lang.hitch(this, "doResizeColumn", drag);

			connect.connect(m, "onMoveStop", lang.hitch(this, function(){
				this.endResizeColumn(drag);
				if(drag.node.releaseCapture){
					drag.node.releaseCapture();
				}
				this.moveable.destroy();
				delete this.moveable;
				this.moveable = null;
				html.removeClass(this.grid.domNode, "dojoxGridColumnResizing");
			}));

			if(e.cellNode.setCapture){
				e.cellNode.setCapture();
			}
			m.onMouseDown(e);
		},

		doResizeColumn: function(inDrag, mover, leftTop){
			var changeX = leftTop.l;
			var data = {
				deltaX: changeX,
				w: inDrag.w + (this.grid.isLeftToRight() ? changeX : -changeX),//fix #11341
				vw: inDrag.vw + changeX,
				tw: inDrag.tw + changeX
			};
			
			this.dragRecord = {inDrag: inDrag, mover: mover, leftTop:leftTop};
			
			if(data.w >= this.minColWidth){
				if (!mover) { // we are using keyboard do immediate resize
					this.doResizeNow(inDrag, data);
				}
				else{
					html.style(this.lineDiv, "left", (this.lineDiv._origLeft + data.deltaX) + "px");
				}
			}
		},

		endResizeColumn: function(inDrag){
			if(this.dragRecord){
				var leftTop = this.dragRecord.leftTop;
				var changeX = this.grid.isLeftToRight() ? leftTop.l : -leftTop.l;
				// Make sure we are not under our minimum
				// http://bugs.dojotoolkit.org/ticket/9390
				changeX += Math.max(inDrag.w + changeX, this.minColWidth) - (inDrag.w + changeX);
				if(has("webkit") && inDrag.spanners.length){
					// Webkit needs the pad border extents back in
					changeX += html._getPadBorderExtents(inDrag.spanners[0].node).w;
				}
				var data = {
					deltaX: changeX,
					w: inDrag.w + changeX,
					vw: inDrag.vw + changeX,
					tw: inDrag.tw + changeX
				};
				// Only resize the columns when the drag has finished
				this.doResizeNow(inDrag, data);
				delete this.dragRecord;
			}
			
			html.destroy(this.lineDiv);
 			html.destroy(this.moverDiv);
			html.destroy(this.moverDiv);
			delete this.moverDiv;
			this._skipBogusClicks = true;
			inDrag.view.update();
			this._skipBogusClicks = false;
			this.grid.onResizeColumn(inDrag.index);
		},
		doResizeNow: function(inDrag, data){
			inDrag.view.convertColPctToFixed();
			if(inDrag.view.flexCells && !inDrag.view.testFlexCells()){
				var t = findTable(inDrag.node);
				if(t){
					(t.style.width = '');
				}
			}
			var i, s, sw, f, fl;
			for(i=0; (s=inDrag.spanners[i]); i++){
				sw = s.width + data.deltaX;
				if(sw > 0){
					s.node.style.width = sw + 'px';
					inDrag.view.setColWidth(s.index, sw);
				}
			}
			if(this.grid.isLeftToRight() || !has("ie")){//fix #11339
				for(i=0; (f=inDrag.followers[i]); i++){
					fl = f.left + data.deltaX;
					f.node.style.left = fl + 'px';
				}
			}
			inDrag.node.style.width = data.w + 'px';
			inDrag.view.setColWidth(inDrag.index, data.w);
			inDrag.view.headerNode.style.width = data.vw + 'px';
			inDrag.view.setColumnsWidth(data.tw);
			if(!this.grid.isLeftToRight()){
				inDrag.view.headerNode.scrollLeft = inDrag.scrollLeft + data.deltaX;
			}
		}
	});

	// Maps an html table into a structure parsable for information about cell row and col spanning.
	// Used by HeaderBuilder.
	dg._TableMap = lang.extend(function(rows){
		this.mapRows(rows);
	},{
		map: null,

		mapRows: function(inRows){
			// summary: Map table topography

			//console.log('mapRows');
			// # of rows
			var rowCount = inRows.length;
			if(!rowCount){
				return;
			}
			// map which columns and rows fill which cells
			this.map = [];
			var row;
			for(var k=0; (row=inRows[k]); k++){
				this.map[k] = [];
			}
			for(var j=0; (row=inRows[j]); j++){
				for(var i=0, x=0, cell, colSpan, rowSpan; (cell=row[i]); i++){
					while(this.map[j][x]){x++;}
					this.map[j][x] = { c: i, r: j };
					rowSpan = cell.rowSpan || 1;
					colSpan = cell.colSpan || 1;
					for(var y=0; y<rowSpan; y++){
						for(var s=0; s<colSpan; s++){
							this.map[j+y][x+s] = this.map[j][x];
						}
					}
					x += colSpan;
				}
			}
			//this.dumMap();
		},

		dumpMap: function(){
			for(var j=0, row, h=''; (row=this.map[j]); j++,h=''){
				for(var i=0, cell; (cell=row[i]); i++){
					h += cell.r + ',' + cell.c + '   ';
				}
			}
		},

		getMapCoords: function(inRow, inCol){
			// summary: Find node's map coords by it's structure coords
			for(var j=0, row; (row=this.map[j]); j++){
				for(var i=0, cell; (cell=row[i]); i++){
					if(cell.c==inCol && cell.r == inRow){
						return { j: j, i: i };
					}
					//else{console.log(inRow, inCol, ' : ', i, j, " : ", cell.r, cell.c); };
				}
			}
			return { j: -1, i: -1 };
		},
		
		getNode: function(inTable, inRow, inCol){
			// summary: Find a node in inNode's table with the given structure coords
			var row = inTable && inTable.rows[inRow];
			return row && row.cells[inCol];
		},
		
		_findOverlappingNodes: function(inTable, inRow, inCol){
			var nodes = [];
			var m = this.getMapCoords(inRow, inCol);
			//console.log("node j: %d, i: %d", m.j, m.i);
			for(var j=0, row; (row=this.map[j]); j++){
				if(j == m.j){ continue; }
				var rw = row[m.i];
				//console.log("overlaps: r: %d, c: %d", rw.r, rw.c);
				var n = (rw?this.getNode(inTable, rw.r, rw.c):null);
				if(n){ nodes.push(n); }
			}
			//console.log(nodes);
			return nodes;
		},
		
		findOverlappingNodes: function(inNode){
			return this._findOverlappingNodes(findTable(inNode), getTrIndex(inNode.parentNode), getTdIndex(inNode));
		}
	});

	return {
		_Builder: _Builder,
		_HeaderBuilder: _HeaderBuilder,
		_ContentBuilder: _ContentBuilder
	};
});

},
'gridx/modules/filter/FilterBar':function(){
define([
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dijit/registry",
	"dojo/_base/lang",
	"dojo/_base/array",
	"dojo/_base/event",
	"dojo/dom-construct",
	"dojo/dom-attr",
	"dojo/dom-class",
	"dojo/string",
	"dojo/parser",
	"dojo/query",
	"../../core/_Module",
	"dojo/text!../../templates/FilterBar.html",
	"dojo/i18n!../../nls/FilterBar",
	"./Filter",
	"./FilterDialog",
	"./FilterConfirmDialog",
	"./FilterTooltip",
	"dijit/TooltipDialog",
	"dijit/popup",
	"dijit/Tooltip",
	"dijit/form/Button"
], function(kernel, declare, registry, lang, array, event, dom, domAttr, css, string, parser, query, _Module, template, nls, Filter, FilterDialog, FilterConfirmDialog, FilterTooltip){

	/*=====
	var columnDefinitionFilterMixin = {
		// filterable: Boolean
		//		If FALSE, then this column should not occur in the Filter Definition Dialog for future rules.
		//		But this does not influence existing filter rules. Default to be TRUE.
		filterable: true,
	
		// disabledConditions: String[]
		//		If provided, all the listed conditions will not occur in the Filter Definition Dialog for future rules.
		//		But this does not influence existing filter rules. Default to be an empty array.
		disabledConditions: [],
	
		// dataType: String
		//		Specify the data type of this column. Should be one of "string", "number", "date", "time", and "boolean".
		//		Case insensitive. Data type decides which conditions to use in the Filter Definition Dialog.
		dataType: 'date'
		storeDatePattern: ''
		formatter: ''
		dateParsePatter: 'yyyy/MM/dd HH:mm:ss'
		filterArgs: {
			trueLabel: '',
			falseLabel: '',
			valueDijitArgs: {}
		}
		// dataTypeArgs: Object
		//		Passing any other special config options for this column. For example, if the column is of type 'date', but the data
		//		in store is of string type, then a 'converter' function is needed here:
		//		dataTypeArgs: {
		//			useRawData: true,
		//			converter: function(v){
		//				return dojo.date.locale.parse(v, {...});
		//			}
		//		}
		dataTypeArgs: {}
	};
	=====*/
	
	return declare(/*===== "gridx.modules.filter.FilterBar", =====*/_Module, {
		name: 'filterBar',
		forced: ['filter'],
		getAPIPath: function(){
			return {
				filterBar: this
			};
		},
		//Public-----------------------------------------------------------
		
		// closeButton: Boolean
		//		TRUE to show a small button on the filter bar for the user to close/hide the filter bar.
		closeButton: true,
	
		// defineFilterButton: Boolean
		//		FALSE to hide the define filter button on the left side (right side for RTL) of the filter bar.
		defineFilterButton: true,
		
		// tooltipDelay: Number
		//		Time in mili-seconds of the delay to show the Filter Status Tooltip when mouse is hovering on the filter bar.
		tooltipDelay: 300,
	
		// maxRuleCount: Integer
		//		Maximum rule count that can be applied in the Filter Definition Dialog.
		//		If <= 0 or not number, then infinite rules are supported.
		maxRuleCount: 0,
		
		// ruleCountToConfirmClearFilter: Integer | Infinity | null
		//		If the filter rule count is larger than or equal to this value, then a confirm dialog will show when clearing filter.
		//		If set to less than 1 or null, then always show the confirm dialog.
		//		If set to Infinity, then never show the confirm dialog.
		//		Default value is 2.
		ruleCountToConfirmClearFilter: 2,
		
		// autoComplete: Boolean
		//		If enabled the Value field of the filter rule box will be editable combo with values from grid store in the dropdown
		//		autocomplete feature enabled by default for String or Text data types
		//		TWL #55300: this is a temporary fix until dojo team provides a flag
		autoComplete: true,
	
	/*=====
		// itemsName: String
		//		The general name of the items listed in the grid.
		//		If not provided, then search the language bundle.
		itemsName: '',=====*/
	
		// condition:
		//		Name of all supported conditions.
		//		Hard coded here or dynamicly generated is up to the implementer. Anyway, users should be able to get this info.
		conditions: {
			string: ['equal', 'contain','startWith', 'endWith', 'notEqual','notContain', 'notStartWith', 'notEndWith',	'isEmpty'],
			number: ['equal','greater','less','greaterEqual','lessEqual','notEqual','isEmpty'],
			date: ['equal','before','after','range','isEmpty'],
			time: ['equal','before','after','range','isEmpty'],
			'boolean': ['equal','isEmpty']
		},
		
		load: function(args, startup){
			// summary:
			//	Init filter bar UI
			//Add before and after expression for filter.
			var F = Filter;
			F.before = F.lessEqual;
			F.after = F.greaterEqual;
			//For backward compatibility
			kernel.deprecated('FilterBar module property closeFilterBarButton is deprecated.', 'Use closeButton instead', '1.1');
			this.closeFilterBarButton = this.arg('closeButton', this.arg('closeFilterBarButton'));
			this._nls = nls;
			this.domNode = dom.create('div', {
				innerHTML: string.substitute(template, nls),
				'class': 'gridxFilterBar'
			});
			parser.parse(this.domNode);
			css.toggle(this.domNode, 'gridxFilterBarHideCloseBtn', !this.arg('closeButton'));
			this.grid.vLayout.register(this, 'domNode', 'headerNode', -1);
			this._initWidgets();
			this._initFocus();
			this.refresh();
			this.connect(this.domNode, 'onclick', 'onDomClick');
			this.connect(this.domNode, 'onmouseover', 'onDomMouseOver');
			this.connect(this.domNode, 'onmousemove', 'onDomMouseMove');
			this.connect(this.domNode, 'onmouseout', 'onDomMouseOut');
			this.loaded.callback();
		},
		onDomClick: function(e){
			if(!e.target || !e.target.tagName){return;}
			if(domAttr.get(e.target, 'action') === 'clear'){
				this.clearFilter();
			}else if(css.contains(e.target, 'gridxFilterBarCloseBtn') || css.contains(e.target,'gridxFilterBarCloseBtnText')){
				this.hide();
			}else{
				this.showFilterDialog();
			}
		},
		onDomMouseMove: function(e){
			if(e && e.target && (domAttr.get(e.target, 'action') === 'clear'
				|| this.btnFilter === dijit.getEnclosingWidget(e.target))){return;}
			this._showTooltip(e);
		},
		onDomMouseOver: function(e){},
		onDomMouseOut: function(e){
			//Make sure to not hide tooltip when mouse moves to tooltip itself.
			window.setTimeout(lang.hitch(this, '_hideTooltip'), 10);
		},
		
		applyFilter: function(filterData){
			// summary:
			//		Apply the filter data.
			var F = Filter, exps = [];
			this.filterData = filterData;
			
			array.forEach(filterData.conditions, function(data){
				var type = 'string';
				if(data.colId){
					type = this.grid.column(data.colId).dataType();
					exps.push(this._getFilterExpression(data.condition, data, type, data.colId));
				}else{
					//any column
					var arr = [];
					array.forEach(this.grid.columns(), function(col){
						if(!col.isFilterable()){return;}
						arr.push(this._getFilterExpression(data.condition, data, type, col.id));
					}, this);
					exps.push(F.or.apply(F, arr));
				}
			}, this);
			var filter = (filterData.type === 'all' ? F.and : F.or).apply(F, exps);
			this.grid.filter.setFilter(filter);
			var _this = this;
			this.model.when({}).then(function(){
				_this._currentSize = _this.model.size();
				_this._totalSize = _this.model._cache.size();
				_this._buildFilterState();
			});
		},
		
		confirmToExecute: function(callback, scope){
			var max = this.arg('ruleCountToConfirmClearFilter');
			if(this.filterData && (this.filterData.conditions.length >= max || max <= 0)){
				if(!this._cfmDlg){
					this._cfmDlg = new FilterConfirmDialog();
				}
				this._cfmDlg.execute = lang.hitch(scope, callback);
				this._cfmDlg.show();
			}else{
				callback.apply(scope);
			}
		},
		
		clearFilter: function(noConfirm){
			if(!noConfirm){
				this.confirmToExecute(lang.hitch(this, 'clearFilter', true), this);
			}else{
				this.filterData = null;
				this.grid.filter.setFilter();
				this._buildFilterState();
			}
		},
	
		columnMixin: {
			isFilterable: function(){
				// summary:
				//		Check if this column is filterable.
				// return: Boolean
				return this.grid._columnsById[this.id].filterable !== false;
			},
	
			setFilterable: function(filterable){
				// summary:
				//		Set filterable for this column.
				// filterable: Boolean
				//		TRUE for filterable, FALSE for not.
				// return:
				//		column object itself
				this.grid.filterBar._setFilterable(this.id, filterable);
				return this;
			},
	
			dataType: function(){
				// summary:
				//		Get the data type of this column. Always lowercase.
				// return: String
				return (this.grid._columnsById[this.id].dataType || 'string').toLowerCase();
			},
	
			filterConditions: function(){
				// summary:
				//		Get the available conditions for this column.	
				return this.grid.filterBar._getColumnConditions(this.id);
			}
		},
	
		refresh: function(){
			// summary:
			//		Re-draw the filter bar if necessary with the current attributes.
			// example:
			//		grid.filterBar.closeButton = true;
			//		grid.filterBar.refresh();
			this.btnClose.style.display = this.closeButton ? '': 'none';
			this.btnFilter.domNode.style.display = this.arg('defineFilterButton') ? '': 'none';
		},
		isVisible: function(){
			return this.domNode.style.display != 'none';
		},
		show: function(){
			// summary:
			//		Show the filter bar. (May add animation later)
			this.domNode.style.display = 'block';
			this.grid.vLayout.reLayout();
			this.onShow();
		},
	
		hide: function(){
			// summary:
			//		Hide the filter bar. (May add animation later)
			this.domNode.style.display = 'none';
			this.grid.vLayout.reLayout();
			this._hideTooltip();
			this.onHide();
		},
		onShow: function(){
			
		},
		onHide: function(){
			
		},
		showFilterDialog: function(){
			// summary:
			//		Show the filter define dialog.
			var dlg = this._filterDialog;
			if(!dlg){
				this._filterDialog = dlg = new FilterDialog({
					grid: this.grid,
					// TWL #55300: this is a temporary fix until dojo team provides a flag
					autoComplete: this.autoComplete
				});
			}
			if(dlg.open){return;}
			//Fix #7345: If there exists filterData, it should be set after dlg is shown;
			//If there is no filterData, dlg.setData have to be called before dlg.show(),
			//otherwise, the dlg will not show any condition boxes.
			//TODO: Need more investigation on this to make the logic more reasonable!
			if(!this.filterData){
				dlg.setData(this.filterData);
			}
			dlg.show();
			if(this.filterData){
				dlg.setData(this.filterData);
			}
		},
		
		uninitialize: function(){
			this._filterDialog && this._filterDialog.destroyRecursive();
			this.inherited(arguments);
			dom.destroy(this.domNode);
		},
	
		//Private---------------------------------------------------------------
		_getColumnConditions: function(colId){
			// summary:
			//		Get the available conditions for a specific column. 
			// 		Excluded condtions is defined by col.disabledConditions
			// tag:
			//		private
			// colId: String|Number
			//		The ID of a column.
			// return: String[]
			//		An array of condition names.
			
			var disabled, type;
			if(!colId){
				//any column
				disabled = [];
				type = 'string';
			}else{
				disabled = this.grid._columnsById[colId].disabledConditions || [];
				type = (this.grid._columnsById[colId].dataType || 'string').toLowerCase();
			}
			
			var ret = this.conditions[type], hash = {};
			if(!ret){ret = this.conditions['string'];}
			array.forEach(disabled, function(name){hash[name] = true;});
			ret = array.filter(ret, function(name){return !hash[name];});
			return ret;
		},
		
		_setFilterable: function(colId, filterable){
			var col = this.grid._columnsById[colId];
			if(!col){return;}
			if(col.filterable == !!filterable){return;}
			col.filterable = !!filterable;
			if(this.filterData){
				var d = this.filterData, len = d.conditions.length;
				d.conditions = array.filter(d.conditions, function(c){
					return c.colId != colId;
				});
				if(len != d.conditions.length){
					this.applyFilter(d);
				}
				if(this._filterDialog.open){
					this._filterDialog.setData(d);
				}
			}
		},
		_initWidgets: function(){
			this.btnFilter = registry.byNode(query('.dijitButton', this.domNode)[0]);
			this.btnClose = query('.gridxFilterBarCloseBtn', this.domNode)[0];
			this.statusNode = query('.gridxFilterBarStatus', this.domNode)[0].firstChild;
			domAttr.remove(this.btnFilter.focusNode, 'aria-labelledby');
		},
		
		_buildFilterState: function(){
			// summary:
			//		Build the tooltip dialog to show all applied filters.
			if(!this.filterData || !this.filterData.conditions.length){
				this.statusNode.innerHTML = this.arg('noFilterMessage', nls.filterBarMsgNoFilterTemplate);
				return;
			}
			this.statusNode.innerHTML = string.substitute(this.arg('hasFilterMessage', nls.filterBarMsgHasFilterTemplate),
				[this._currentSize, this._totalSize, nls.defaultItemsName]) + 
				'&nbsp; &nbsp; <a href="javascript:void(0);" action="clear" title="' + nls.filterBarClearButton + '">'
					 + nls.filterBarClearButton + '</a>';
			this._buildTooltip();
		},
		_buildTooltip: function(){
			if(!this._tooltip){
				this._tooltip = new FilterTooltip({grid: this.grid});
			}
			this._tooltip.buildContent();
		},
		_showTooltip: function(evt, delayed){
			this._hideTooltip();
			if(!this.filterData || 
				!this.filterData.conditions || 
				!this.filterData.conditions.length){return;}
			if(!delayed){
				this._pointTooltipDelay = window.setTimeout(lang.hitch(this, '_showTooltip', 
					evt, true),this.arg('tooltipDelay'));
				return;
			}
			this._tooltip.show(evt);
		},
		_hideTooltip: function(){
			var dlg = this._tooltip;
			if(!dlg){return;}
			if(dlg.isMouseOn){return;}
			if(this._pointTooltipDelay){
				window.clearTimeout(this._pointTooltipDelay);
				this._pointTooltipDelay = null;
			}
			dlg.hide();
		},
		_getRuleString: function(condition, value, type){
			var valueString, type;
			if(condition == 'isEmpty'){
				valueString = '';
			}else if(/^date|^time/i.test(type)){
				var f = this._formatDate;
				if(/^time/i.test(type)){f = this._formatTime;}
				
				if(condition === 'range'){
					var tpl = this.arg('rangeTemplate', nls.rangeTemplate);
					valueString = string.substitute(tpl, [f(value.start), f(value.end)]);
				}else{
					valueString = f(value);
				}
			}else{
				valueString = value;
			}
			return '<span style="font-style:italic">' + this._getConditionDisplayName(condition) + '</span> ' + valueString;
		},
		_getConditionDisplayName: function(c){
			var k = c.charAt(0).toUpperCase() + c.substring(1);
			return this.arg('condition' + k, nls['condition' + k]);
		},
		_getConditionOptions: function(colId){
			var cache = this._conditionOptions = this._conditionOptions || {};
			if(!cache[colId]){
				var arr = [];
				array.forEach(this._getColumnConditions(colId), function(s){
					var k = s.charAt(0).toUpperCase() + s.substring(1);
					arr.push({
						label: this.arg('condition' + k, nls['condition' + k]),
						value: s
					});
				}, this);
				cache[colId] = arr;
			}
			return cache[colId];
		},
		
		_getFilterExpression: function(condition, data, type, colId){
			//get filter expression by condition,data, column and type
			var F = Filter;
			var dc = this.grid._columnsById[colId].dateParser || this._stringToDate;
			var tc = this.grid._columnsById[colId].timeParser || this._stringToTime;
			var converter = {date: dc, time: tc};
			var c = data.condition, exp, isNot = false, type = c == 'isEmpty' ? 'string' : type; //isEmpty always treat type as string
			if(c === 'range'){
				var startValue = F.value(data.value.start, type),
					endValue = F.value(data.value.end, type), 
					columnValue = F.column(colId, type, converter[type]);
				exp = F.and(F.greaterEqual(columnValue, startValue), F.lessEqual(columnValue, endValue));
			}else{
				if(/^not/.test(c)){
					isNot = true;
					c = c.replace(/^not/g, '');
					c = c.charAt(0).toLowerCase() + c.substring(1);
				}
				exp = F[c](F.column(colId, type, converter[type]), c == 'isEmpty' ? null : F.value(data.value, type));
				if(isNot){exp = F.not(exp);}
			}
			return exp;
		},
		_stringToDate: function(s, pattern){
			pattern = pattern || /(\d{4})\/(\d\d?)\/(\d\d?)/;
			pattern.test(s);
			var d = new Date();
			d.setFullYear(parseInt(RegExp.$1));
			d.setMonth(parseInt(RegExp.$2)-1);
			return d;
		},
		_stringToTime: function(s, pattern){
			pattern = pattern || /(\d\d?):(\d\d?):(\d\d?)/;
			pattern.test(s);
			var d = new Date();
			d.setHours(parseInt(RegExp.$1));
			d.setMinutes(parseInt(RegExp.$2));
			d.setSeconds(parseInt(RegExp.$3));
			return d;
		},
		_formatDate: function(date){
			//this may be customized by grid layout definition
			var m = date.getMonth() + 1, d = date.getDate();
			return m + '/' + d + '/' + date.getFullYear();
		},
		_formatTime: function(time){
			//this may be customized by grid layout definition
			var h = time.getHours(), m = time.getMinutes();
			if(h < 10){h = '0' + h;}
			if(m < 10){m = '0' + m;}
			return h + ':' + m + ':00';
		},
		
		_initFocus: function(){
			var focus = this.grid.focus;
			if(focus){
				focus.registerArea({
					name: 'filterbar_btn',
					priority: -1,
					focusNode: this.btnFilter.domNode,
					doFocus: this._doFocusBtnFilter,
					scope: this
				});
				
				focus.registerArea({
					name: 'filterbar_clear',
					priority: -0.9,
					focusNode: this.domNode,
					doFocus: this._doFocusClearLink,
					scope: this
				});
				
				focus.registerArea({
					name: 'filterbar_close',
					priority: -0.8,
					focusNode: this.btnClose,
					doFocus: this._doFocusBtnClose,
					scope: this
				});
			}
		},
		_doFocusBtnFilter: function(evt){
			this.btnFilter.focus();
			if(evt){event.stop(evt);}
			return true;
		},
		_doFocusClearLink: function(evt){
			this.btnFilter.focus();
			var link = query('a[action="clear"]')[0];
			if(link){
				link.focus();
				if(evt){event.stop(evt);}
				return true;
			}
			return false;
		},
		_doFocusBtnClose: function(evt){
			this.btnClose.focus();
			if(evt){event.stop(evt);}
			return true;
		},
		
		_doBlur: function(){
			return true;
		},
		destroy: function(){
			this._filterDialog && this._filterDialog.destroy();
			dom.destroy(this.domNode);
			this.inherited(arguments);
		}
		
	});
});

},
'dojo/date':function(){
define(["./_base/kernel", "./_base/lang"], function(dojo, lang) {
	// module:
	//		dojo/date
	// summary:
	//		TODOC

lang.getObject("date", true, dojo);

/*=====
dojo.date = {
	// summary: Date manipulation utilities
}
=====*/

dojo.date.getDaysInMonth = function(/*Date*/dateObject){
	//	summary:
	//		Returns the number of days in the month used by dateObject
	var month = dateObject.getMonth();
	var days = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
	if(month == 1 && dojo.date.isLeapYear(dateObject)){ return 29; } // Number
	return days[month]; // Number
};

dojo.date.isLeapYear = function(/*Date*/dateObject){
	//	summary:
	//		Determines if the year of the dateObject is a leap year
	//	description:
	//		Leap years are years with an additional day YYYY-02-29, where the
	//		year number is a multiple of four with the following exception: If
	//		a year is a multiple of 100, then it is only a leap year if it is
	//		also a multiple of 400. For example, 1900 was not a leap year, but
	//		2000 is one.

	var year = dateObject.getFullYear();
	return !(year%400) || (!(year%4) && !!(year%100)); // Boolean
};

// FIXME: This is not localized
dojo.date.getTimezoneName = function(/*Date*/dateObject){
	//	summary:
	//		Get the user's time zone as provided by the browser
	// dateObject:
	//		Needed because the timezone may vary with time (daylight savings)
	//	description:
	//		Try to get time zone info from toString or toLocaleString method of
	//		the Date object -- UTC offset is not a time zone.  See
	//		http://www.twinsun.com/tz/tz-link.htm Note: results may be
	//		inconsistent across browsers.

	var str = dateObject.toString(); // Start looking in toString
	var tz = ''; // The result -- return empty string if nothing found
	var match;

	// First look for something in parentheses -- fast lookup, no regex
	var pos = str.indexOf('(');
	if(pos > -1){
		tz = str.substring(++pos, str.indexOf(')'));
	}else{
		// If at first you don't succeed ...
		// If IE knows about the TZ, it appears before the year
		// Capital letters or slash before a 4-digit year
		// at the end of string
		var pat = /([A-Z\/]+) \d{4}$/;
		if((match = str.match(pat))){
			tz = match[1];
		}else{
		// Some browsers (e.g. Safari) glue the TZ on the end
		// of toLocaleString instead of putting it in toString
			str = dateObject.toLocaleString();
			// Capital letters or slash -- end of string,
			// after space
			pat = / ([A-Z\/]+)$/;
			if((match = str.match(pat))){
				tz = match[1];
			}
		}
	}

	// Make sure it doesn't somehow end up return AM or PM
	return (tz == 'AM' || tz == 'PM') ? '' : tz; // String
};

// Utility methods to do arithmetic calculations with Dates

dojo.date.compare = function(/*Date*/date1, /*Date?*/date2, /*String?*/portion){
	//	summary:
	//		Compare two date objects by date, time, or both.
	//	description:
	//  	Returns 0 if equal, positive if a > b, else negative.
	//	date1:
	//		Date object
	//	date2:
	//		Date object.  If not specified, the current Date is used.
	//	portion:
	//		A string indicating the "date" or "time" portion of a Date object.
	//		Compares both "date" and "time" by default.  One of the following:
	//		"date", "time", "datetime"

	// Extra step required in copy for IE - see #3112
	date1 = new Date(+date1);
	date2 = new Date(+(date2 || new Date()));

	if(portion == "date"){
		// Ignore times and compare dates.
		date1.setHours(0, 0, 0, 0);
		date2.setHours(0, 0, 0, 0);
	}else if(portion == "time"){
		// Ignore dates and compare times.
		date1.setFullYear(0, 0, 0);
		date2.setFullYear(0, 0, 0);
	}

	if(date1 > date2){ return 1; } // int
	if(date1 < date2){ return -1; } // int
	return 0; // int
};

dojo.date.add = function(/*Date*/date, /*String*/interval, /*int*/amount){
	//	summary:
	//		Add to a Date in intervals of different size, from milliseconds to years
	//	date: Date
	//		Date object to start with
	//	interval:
	//		A string representing the interval.  One of the following:
	//			"year", "month", "day", "hour", "minute", "second",
	//			"millisecond", "quarter", "week", "weekday"
	//	amount:
	//		How much to add to the date.

	var sum = new Date(+date); // convert to Number before copying to accomodate IE (#3112)
	var fixOvershoot = false;
	var property = "Date";

	switch(interval){
		case "day":
			break;
		case "weekday":
			//i18n FIXME: assumes Saturday/Sunday weekend, but this is not always true.  see dojo.cldr.supplemental

			// Divide the increment time span into weekspans plus leftover days
			// e.g., 8 days is one 5-day weekspan / and two leftover days
			// Can't have zero leftover days, so numbers divisible by 5 get
			// a days value of 5, and the remaining days make up the number of weeks
			var days, weeks;
			var mod = amount % 5;
			if(!mod){
				days = (amount > 0) ? 5 : -5;
				weeks = (amount > 0) ? ((amount-5)/5) : ((amount+5)/5);
			}else{
				days = mod;
				weeks = parseInt(amount/5);
			}
			// Get weekday value for orig date param
			var strt = date.getDay();
			// Orig date is Sat / positive incrementer
			// Jump over Sun
			var adj = 0;
			if(strt == 6 && amount > 0){
				adj = 1;
			}else if(strt == 0 && amount < 0){
			// Orig date is Sun / negative incrementer
			// Jump back over Sat
				adj = -1;
			}
			// Get weekday val for the new date
			var trgt = strt + days;
			// New date is on Sat or Sun
			if(trgt == 0 || trgt == 6){
				adj = (amount > 0) ? 2 : -2;
			}
			// Increment by number of weeks plus leftover days plus
			// weekend adjustments
			amount = (7 * weeks) + days + adj;
			break;
		case "year":
			property = "FullYear";
			// Keep increment/decrement from 2/29 out of March
			fixOvershoot = true;
			break;
		case "week":
			amount *= 7;
			break;
		case "quarter":
			// Naive quarter is just three months
			amount *= 3;
			// fallthrough...
		case "month":
			// Reset to last day of month if you overshoot
			fixOvershoot = true;
			property = "Month";
			break;
//		case "hour":
//		case "minute":
//		case "second":
//		case "millisecond":
		default:
			property = "UTC"+interval.charAt(0).toUpperCase() + interval.substring(1) + "s";
	}

	if(property){
		sum["set"+property](sum["get"+property]()+amount);
	}

	if(fixOvershoot && (sum.getDate() < date.getDate())){
		sum.setDate(0);
	}

	return sum; // Date
};

dojo.date.difference = function(/*Date*/date1, /*Date?*/date2, /*String?*/interval){
	//	summary:
	//		Get the difference in a specific unit of time (e.g., number of
	//		months, weeks, days, etc.) between two dates, rounded to the
	//		nearest integer.
	//	date1:
	//		Date object
	//	date2:
	//		Date object.  If not specified, the current Date is used.
	//	interval:
	//		A string representing the interval.  One of the following:
	//			"year", "month", "day", "hour", "minute", "second",
	//			"millisecond", "quarter", "week", "weekday"
	//		Defaults to "day".

	date2 = date2 || new Date();
	interval = interval || "day";
	var yearDiff = date2.getFullYear() - date1.getFullYear();
	var delta = 1; // Integer return value

	switch(interval){
		case "quarter":
			var m1 = date1.getMonth();
			var m2 = date2.getMonth();
			// Figure out which quarter the months are in
			var q1 = Math.floor(m1/3) + 1;
			var q2 = Math.floor(m2/3) + 1;
			// Add quarters for any year difference between the dates
			q2 += (yearDiff * 4);
			delta = q2 - q1;
			break;
		case "weekday":
			var days = Math.round(dojo.date.difference(date1, date2, "day"));
			var weeks = parseInt(dojo.date.difference(date1, date2, "week"));
			var mod = days % 7;

			// Even number of weeks
			if(mod == 0){
				days = weeks*5;
			}else{
				// Weeks plus spare change (< 7 days)
				var adj = 0;
				var aDay = date1.getDay();
				var bDay = date2.getDay();

				weeks = parseInt(days/7);
				mod = days % 7;
				// Mark the date advanced by the number of
				// round weeks (may be zero)
				var dtMark = new Date(date1);
				dtMark.setDate(dtMark.getDate()+(weeks*7));
				var dayMark = dtMark.getDay();

				// Spare change days -- 6 or less
				if(days > 0){
					switch(true){
						// Range starts on Sat
						case aDay == 6:
							adj = -1;
							break;
						// Range starts on Sun
						case aDay == 0:
							adj = 0;
							break;
						// Range ends on Sat
						case bDay == 6:
							adj = -1;
							break;
						// Range ends on Sun
						case bDay == 0:
							adj = -2;
							break;
						// Range contains weekend
						case (dayMark + mod) > 5:
							adj = -2;
					}
				}else if(days < 0){
					switch(true){
						// Range starts on Sat
						case aDay == 6:
							adj = 0;
							break;
						// Range starts on Sun
						case aDay == 0:
							adj = 1;
							break;
						// Range ends on Sat
						case bDay == 6:
							adj = 2;
							break;
						// Range ends on Sun
						case bDay == 0:
							adj = 1;
							break;
						// Range contains weekend
						case (dayMark + mod) < 0:
							adj = 2;
					}
				}
				days += adj;
				days -= (weeks*2);
			}
			delta = days;
			break;
		case "year":
			delta = yearDiff;
			break;
		case "month":
			delta = (date2.getMonth() - date1.getMonth()) + (yearDiff * 12);
			break;
		case "week":
			// Truncate instead of rounding
			// Don't use Math.floor -- value may be negative
			delta = parseInt(dojo.date.difference(date1, date2, "day")/7);
			break;
		case "day":
			delta /= 24;
			// fallthrough
		case "hour":
			delta /= 60;
			// fallthrough
		case "minute":
			delta /= 60;
			// fallthrough
		case "second":
			delta /= 1000;
			// fallthrough
		case "millisecond":
			delta *= date2.getTime() - date1.getTime();
	}

	// Round for fractional values and DST leaps
	return Math.round(delta); // Number (integer)
};

return dojo.date;
});

},
'dojox/grid/cells':function(){
define(["../main", "./cells/_base"], function(dojox){
	return dojox.grid.cells;
});
},
'ibm/tivoli/navmodel/DatasetParameter':function(){
// wrapped by build app
define("ibm/tivoli/navmodel/DatasetParameter", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/Utils"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.DatasetParameter");

// includes
dojo.require("ibm.tivoli.navmodel.Utils");

////////////////////////////////////
// DatasetParameter
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.DatasetParameter", null, {
	// private members
	_param: null,
	_baseUrl: null,
	_version: null,

	/*=====
	constructor_kwArgs: function(parameter, baseUrl, version) {
		// parameter: Object
		//		json parameter object
	},
	=====*/

	 constructor: function(/*constructor_kwArgs*/ kwArgs) {		
		// summary:
		//		Constructs Property object.
		this._param = kwArgs.parameter;
		this._baseUrl = kwArgs.baseUrl;
		this._version = (kwArgs.version ? kwArgs.version : ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion());
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	getAttribute: function(key) {		
		// summary:
		//		Returns general attribute for requested key
		return this._param[key];
	},

	setAttribute: function(key, value) {		
		// summary:
		//		Set general attribute for requested key
		this._param[key] = value;
	},
	
	getAttributes: function() {		
		// summary:
		//		Get all general attributes for the property
		return dojo.clone(this._param);		
	},
	
	removeAttribute: function(key) {
		// summary:
		//		Remove general attribute for requested key
		delete this._param[key];
	},
	
	getId: function() {		
		// summary:
		//		Returns the parameter id
		return this._param.id;
	},

	setId: function(id) {		
		// summary:
		//		Set the parameter id
		this._param.id = id;
	},

	getType: function() {		
		// summary:
		//		Returns the parameter type
		//		|	values:	
		//		|		"string", "string[]",
		//		|		"byte", "short", "int", "long", "byte[]", "short[]", "int[]", "long[]",
		//		|		"ubyte, "ushort", "uint", "ulong", "ubyte[]", "ushort[]", "uint[]", "ulong[]",	
		//		|		"boolean", "boolean[]"	
		//		|		"enum", "map"
		//		|		"date", "datetime", "time", "date[]", "datetime[]", "time[]"
		//		|		"isodate", isodatetime", "isotime", "isodate[]", "isodatetime[], "isotime[]", "isointerval", "isoduration"
		//		|		"float", "double", "float[]", "double[]"
		return this._param.type;
	},

	setType: function(type) {		
		// summary:
		//		Set the parameter type
		//		|	values:	
		//		|		"string", "string[]",
		//		|		"byte", "short", "int", "long", "byte[]", "short[]", "int[]", "long[]",
		//		|		"ubyte, "ushort", "uint", "ulong", "ubyte[]", "ushort[]", "uint[]", "ulong[]",	
		//		|		"boolean", "boolean[]"	
		//		|		"enum", "map"
		//		|		"date", "datetime", "time", "date[]", "datetime[]", "time[]"
		//		|		"isodate", isodatetime", "isotime", "isodate[]", "isodatetime[], "isotime[]", "isointerval", "isoduration"
		//		|		"float", "double", "float[]", "double[]"
		this._param.type = type;
	},
	
	getLabel: function() {		
		// summary:
		//		Returns the parameter label
		return this._param.label;
	},

	setLabel: function(label) {		
		// summary:
		//		Set the parameter label
		this._param.label = label;
	},
	
	getDescription: function() {		
		// summary:
		//		Returns the parameter description
		return this._param.description;
	},

	setDescription: function(description) {		
		// summary:
		//		Set the parameter description
		this._param.description = description;
	},

	isHidden: function() {
		// summary:
		//		Indicates if parameter should be hidden from user
		return this._param.hidden;
	},
	
	setHidden: function(hidden) {
		// summary:
		//		Set value indicating if parameter should be hidden from user
		this._param.hidden = hidden;
	},
	
	isRequired: function() {
		// summary:
		//		Indicates if parameter is required when creating dataset
		return this._param.required;
	},
	
	setRequired: function(required) {
		// summary:
		//		Set value indicating if parameter is required when creating dataset
		this._param.required = required;
	},
	
	hasPossibleValues: function() {
		// summary:
		//		Returns true if the parameter supports a list of possible values		
		return (this._param.values || this._param.valuesUri ? true : false);
	},
	
	hasInlinedPossibleValues: function() {
		// summary:
		//		Returns true if the paramter supports possible values and
		//		all of those values have been inlined in the json
		return (this._param.values ? true : false);
	},
	
	hasUrlAddressablePossibleValues: function() {
		// summary:
		//		Returns true if the parameter supports possible values and
		//		the values are addressable via a url
		return (this._param.valuesUri ? true : false);
	},
	
	getPossibleValues: function(kwArgs) {
		// summary:
		//		Retrieves a list of possible values for parameter
		var args = {
			onComplete: function(values, totalCount, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			start: 0,
			count: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
	
		// possible values were inlined; just return them
		if (this._param.values) {
			if (args.start === 0 && args.count === null) {
				args.onComplete(this._param.values, this._param.values.length, args);
			}
			else {
				var values = [];
				var length = this._param.values.length;
				var count = args.count ? args.count : length;
				for (var index = args.start;  index < length && count > 0;  ++index, --count) {
					values.push(this._param.values[index]);
				}
				args.onComplete(values, length, args);
			}
		}
		// possible values are available via uri
		else if (this._param.valuesUri) {
			var content = ibm.tivoli.navmodel.Utils.getContent();
			if (args.start) {
				content.start = args.start;
			}
			if (args.count) {
				content.count = args.count;
			}
			
			dojo.xhrGet({
				url: this._baseUrl + this._param.valuesUri,
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version, "datasetParmValueList")
				},
				handleAs: "json",
				sync: args.sync,
				content: content,		
				load: dojo.hitch(this, function(result, ioArgs){
					args.onComplete(result.items, result.totalRows, args);					
				}),
				error: dojo.hitch(this, function(error, ioArgs){
					args.onError(error, ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "getPossibleValues", this._trace, this._version), ioArgs);
				})
			});	
		}
		// doesn't have any
		else {
			args.onComplete(null, 0, args);
		}		
	},
	
	getDefaultValue: function() {
		// summary:
		//		Returns default value for parameter
		return this._param["default"];
	},
	
	setDefaultValue: function(defaultVal) {
		// summary:
		//		Set the default value for parameter
		this._param["default"] = defaultVal;
	},
	
	getMinValue: function() {		
		// summary:
		//		Returns minimum value;  for string parms this is minimum length
		return this._prop.min;
	},

	setMinValue: function(min) {		
		// summary:
		//		Set the minimum value
		this._prop.min = min;
	},
	
	getMaxValue: function() {		
		// summary:
		//		Returns maximum value;  for string parms this is maximum length
		return this._prop.max;
	},

	setMaxValue: function(max) {		
		// summary:
		//		Set the maximum value
		this._prop.max = max;
	}
});
});

},
'ibm/usmi/widgets/table/DefaultItemsTableView':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/widgets/table/DefaultItemsTableView", [
	"dojo/_base/declare",
	"dojo/_base/kernel",
	"dojo/_base/lang",
	"dojo/_base/array",
	"dojo/aspect",
	"dijit/registry",
	"dojo/i18n!./nls/ColumnChooser",
	"ibm/tivoli/tip/navwidget/table/dijit/ItemsTableView",
	"ibm/usmi/widgets/table/ConfigureColumnsDialog",
	"ibm/usmi/core/utils/Logger",
	"dijit/place",
	"dojo/_base/connect",
	"ibm/usmi/widgets/table/DefaultTablePersistenceMixin"
], function (declare, dojo, lang, array, aspect, registry, ConfigureColumnsMRI, ItemsTableView, ConfigureColumnsDialog,
		Logger, place, connect, DefaultTablePersistenceMixin) {

	return declare([ ItemsTableView, DefaultTablePersistenceMixin
	], {

		// the logger
		_defaultItemsTableViewLogger : new Logger("ibm.usmi.widgets.table", "DefaultItemsTableView"),

		constructor : function () {
			this._pDEFAULT_WIDGET_DEF_INFO = {
				className : "ibm.tivoli.navmodel.TableWidgetDef",
				baseUrl : "/ibm/tivoli/rest",
				provider : "any",
				datasource : "any",
				widget : "table",
				definition : "UsmiItemsTable"
			};
		},

		augmentGridParams : function (gridParams) {
			// For info on gridParams, look at ibm/tivoli/tip/navwidget/table/dijit/AbstractTableView
			// and also at dojoe/table/Table.
			// Look at the source; do not trust the comments.
			this.inherited(arguments);

			gridParams.plugins.nestedSorting = false;

			// Make it scroll by default, instead of paging
			gridParams.plugins.pagination = false;

			// Get rid of the checkbox column for selection
			gridParams.plugins.indirectSelection = false;

			// Get rid of the refresh menu option
			gridParams.toolbarOptions.refreshMenu = false;

			// Get rid of the printer menu option
			gridParams.toolbarOptions.printerMenu = false;

			// Get rid of the export menu option
			gridParams.toolbarOptions.exporterMenu = false;

			// Disable drag-and-drop of rows and columns
			gridParams.plugins.dnd = false;

			// Row selection only, not columns or cells
			gridParams.plugins.selector = {
				"cell" : "disabled",
				"row" : "multiple",
				"col" : "disabled"
			};

			gridParams = this.persistenceGridParams(this._dataStore, gridParams);

			return gridParams;
		},

		// AbstractTableView._initTableBilder connects to onTableLoad, and inadvertently shows the filter bar.
		// This aspect is called afterward, and hides the filter bar again.
		_initTableBuilder : function () {
			//TODO, hack, memory leak on NavModelTable.js, luckily we can clean up here

			var toolbarId, configureTableButtonId, configureTableButton, actionMenu, menuItems, lastMenuItem;
			if (this._table && this._table._tooltip) {
				this._table._tooltip.destroy();
			}
			this.inherited(arguments);
			aspect.after(this._table, "onTableLoad", lang.hitch(this, function () {
				if (this._table) {
					this._table.setFilter(null, true);

					//@188636 START
					this.initPersistence(this, this._grid, this._table._toolbar._quickFilter.searchField);
					//@188636 END
				}
			}));

			//Hack in our column chooser
			if (this._table && this._table._toolbar) {
				if (this._defaultItemsTableViewLogger.isFinest()) {
					this._defaultItemsTableViewLogger.finest("setting the custom Column Chooser", "_initTableBuilder");
				}
				this._table._toolbar._configureTable = function () {
					this.configureTableDialog = new ConfigureColumnsDialog({
						grid : this.inGrid,
						_structure : this._origStructure
					});
					this.configureTableDialog.show();
				};
				toolbarId = this._table._toolbar.id;
				if (toolbarId) {
					configureTableButtonId = "configureColIcon" + toolbarId;
					configureTableButton = registry.byId(configureTableButtonId);
					if (configureTableButton) {
						configureTableButton.set("title", ConfigureColumnsMRI.CONFIG_COLUMNS);
					}
				}
				if (this._table._toolbar.actionListWidget) {
					actionMenu = this._table._toolbar.actionListWidget.dropDown;
					if (actionMenu) {
						menuItems = actionMenu.getChildren();
						if (menuItems && menuItems.length > 0) {
							lastMenuItem = menuItems[menuItems.length - 1];
							if (lastMenuItem && lastMenuItem.iconClass === "configureColIcon") {
								lastMenuItem.set("title", ConfigureColumnsMRI.CONFIG_COLUMNS);
								lastMenuItem.set("label", ConfigureColumnsMRI.CONFIG_COLUMNS);
							}
						}
					}
				}

			}
			
			
			//need to change the total items in filter bar message to the dataset items count
			if(this._table && this._table._grid && this._table._grid.filterBar){
				aspect.before(this._table._grid.filterBar, "_buildFilterState", lang.hitch(this, function () {
					this._table._grid.filterBar._totalSize=this.dataset.getItemCount();
					//need to show the filter bar if it's not showed
					if(!this._table._grid.filterBar.isVisible()){
						this._table._grid.filterBar.show();
					}
				}));
			}
			
			
			if(this._table && this._table._grid && this._table._grid && this._table._grid.model){
				connect.connect(this._table._grid.model, "onNew", lang.hitch(this, this.refreshFilterBar));
				connect.connect(this._table._grid.model, "onDelete", lang.hitch(this, this.refreshFilterBar));
				connect.connect(this._table._grid.model, "onSet", lang.hitch(this, this.refreshFilterBar));
			}
			
		},	
		
		refreshFilterBar: function(){
			if(this._table && this._table._grid && this._table._grid && this._table._grid.model && this._table._grid.filterBar){
				if(!this._table._grid.filterBar.isVisible()){
					//if filter bar is not visible, we do not want to refresh it
					return;
				}
				this._table._grid.filterBar._currentSize = this._table._grid.model.size();
				this._table._grid.filterBar._buildFilterState();
			}
		},

		_cleanUpImagePath : function (imagePath) {
			return this.inherited(arguments);
		},		

		handleLinkClickAction : function (viewId, objectId, propId, objectType) {
			// summary:
			//        Method to override base implementation so we get notified and can
			//      handle when a link is clicked in a table cell
			if (this._defaultItemsTableViewLogger.isFinest()) {
				this._defaultItemsTableViewLogger.entering("", "handleLinkClickAction");
				this._defaultItemsTableViewLogger.finest("handling link click for table view - prop: " + propId,
						"handleLinkClickAction");
			}
			var object, property;

			//ask the dataset for the correct object based on the objects id.
			//since we have already rendered these items, the value should be
			//cached and should be quick.
			switch (objectType) {
			case "ibm.tivoli.navmodel.Item":
				object = this.dataset.getItem({
					itemId : objectId
				});
				break;
			case "ibm.tivoli.navmodel.Link":
				object = this.dataset.getLink({
					itemId : objectId
				});
				break;
			case "ibm.tivoli.navmodel.Relationship":
				object = this.dataset.getRelationship({
					itemId : objectId
				});
				break;
			default:
				if (this._defaultItemsTableViewLogger.isFinest()) {
					this._defaultItemsTableViewLogger.finest("unknown objectType: " + objectType,
							"handleLinkClickAction");
				}
			}

			if (object) {
				property = object.getProperty(propId);
			}

			if (object && property) {
				this.daHelper.executeDefaultAction(this, property);
			}

			if (this._defaultItemsTableViewLogger.isFinest()) {
				this._defaultItemsTableViewLogger.exiting("", "handleLinkClickAction");
			}
		},

		// Override getContextMenuAdapter, so we can reposition the context menu.
		// Necessary since the menu will change size after it is first displayed,
		// as more items come in over CURI.		
		getContextMenuAdapter : function () {
			// get the ContextMenuAdapter, which I will later return.
			var cma = this.inherited(arguments);
			var coords;

			// Before the ContextMenuAdapter.contextMenuOpen method
			aspect.before(cma, "contextMenuOpen", function (event) {
				// Save the original mouse-click coordinates for later
				coords = {
					x : event.pageX,
					y : event.pageY
				};
			});

			// After the ContextMenuAdapter._menuItemsLoaded method
			aspect.after(cma, "_menuItemsLoaded", function (success) {
				// Put the menu in the correct place, so it does not go off the page
				if (success) {
					var menu = this.getMenu();
					var orient = menu.isLeftToRight() ? 'L' : 'R';
					var wrapper = menu.domNode.parentNode;
					if (wrapper !== null && coords) {
						place.at(wrapper, coords, orient == 'R' ? [
							'TR',
							'BR',
							'TL',
							'BL'
						] : [
							'TL',
							'BL',
							'TR',
							'BR'
						]);
					}
				}
			}, true);

			return cma;
		},

		destroy : function () {
			//TODO, hack, memory leak on NavModelTable.js, luckily we can clean up here
			if (this._table && this._table._tooltip) {
				this._table._tooltip.destroy();
				this._table._tooltip = null;
			}
			if (this._table && this._table._tooltipProperties) {
				this._table._tooltipProperties.destroyRecursive();
				this._table._tooltipProperties=null;
			}
			this.inherited(arguments);
		}

	});
});

},
'dijit/_KeyNavContainer':function(){
define([
	"dojo/_base/kernel", // kernel.deprecated
	"./_Container",
	"./_FocusMixin",
	"dojo/_base/array", // array.forEach
	"dojo/keys", // keys.END keys.HOME
	"dojo/_base/declare", // declare
	"dojo/_base/event", // event.stop
	"dojo/dom-attr", // domAttr.set
	"dojo/_base/lang" // lang.hitch
], function(kernel, _Container, _FocusMixin, array, keys, declare, event, domAttr, lang){

/*=====
	var _FocusMixin = dijit._FocusMixin;
	var _Container = dijit._Container;
=====*/

	// module:
	//		dijit/_KeyNavContainer
	// summary:
	//		A _Container with keyboard navigation of its children.

	return declare("dijit._KeyNavContainer", [_FocusMixin, _Container], {

		// summary:
		//		A _Container with keyboard navigation of its children.
		// description:
		//		To use this mixin, call connectKeyNavHandlers() in
		//		postCreate().
		//		It provides normalized keyboard and focusing code for Container
		//		widgets.

/*=====
		// focusedChild: [protected] Widget
		//		The currently focused child widget, or null if there isn't one
		focusedChild: null,
=====*/

		// tabIndex: Integer
		//		Tab index of the container; same as HTML tabIndex attribute.
		//		Note then when user tabs into the container, focus is immediately
		//		moved to the first item in the container.
		tabIndex: "0",

		connectKeyNavHandlers: function(/*keys[]*/ prevKeyCodes, /*keys[]*/ nextKeyCodes){
			// summary:
			//		Call in postCreate() to attach the keyboard handlers
			//		to the container.
			// preKeyCodes: keys[]
			//		Key codes for navigating to the previous child.
			// nextKeyCodes: keys[]
			//		Key codes for navigating to the next child.
			// tags:
			//		protected

			// TODO: call this automatically from my own postCreate()

			var keyCodes = (this._keyNavCodes = {});
			var prev = lang.hitch(this, "focusPrev");
			var next = lang.hitch(this, "focusNext");
			array.forEach(prevKeyCodes, function(code){ keyCodes[code] = prev; });
			array.forEach(nextKeyCodes, function(code){ keyCodes[code] = next; });
			keyCodes[keys.HOME] = lang.hitch(this, "focusFirstChild");
			keyCodes[keys.END] = lang.hitch(this, "focusLastChild");
			this.connect(this.domNode, "onkeypress", "_onContainerKeypress");
			this.connect(this.domNode, "onfocus", "_onContainerFocus");
		},

		startupKeyNavChildren: function(){
			kernel.deprecated("startupKeyNavChildren() call no longer needed", "", "2.0");
		},

		startup: function(){
			this.inherited(arguments);
			array.forEach(this.getChildren(), lang.hitch(this, "_startupChild"));
		},

		addChild: function(/*dijit._Widget*/ widget, /*int?*/ insertIndex){
			this.inherited(arguments);
			this._startupChild(widget);
		},

		focus: function(){
			// summary:
			//		Default focus() implementation: focus the first child.
			this.focusFirstChild();
		},

		focusFirstChild: function(){
			// summary:
			//		Focus the first focusable child in the container.
			// tags:
			//		protected
			this.focusChild(this._getFirstFocusableChild());
		},

		focusLastChild: function(){
			// summary:
			//		Focus the last focusable child in the container.
			// tags:
			//		protected
			this.focusChild(this._getLastFocusableChild());
		},

		focusNext: function(){
			// summary:
			//		Focus the next widget
			// tags:
			//		protected
			this.focusChild(this._getNextFocusableChild(this.focusedChild, 1));
		},

		focusPrev: function(){
			// summary:
			//		Focus the last focusable node in the previous widget
			//		(ex: go to the ComboButton icon section rather than button section)
			// tags:
			//		protected
			this.focusChild(this._getNextFocusableChild(this.focusedChild, -1), true);
		},

		focusChild: function(/*dijit._Widget*/ widget, /*Boolean*/ last){
			// summary:
			//		Focus specified child widget.
			// widget:
			//		Reference to container's child widget
			// last:
			//		If true and if widget has multiple focusable nodes, focus the
			//		last one instead of the first one
			// tags:
			//		protected

			if(!widget){ return; }

			if(this.focusedChild && widget !== this.focusedChild){
				this._onChildBlur(this.focusedChild);	// used by _MenuBase
			}
			widget.set("tabIndex", this.tabIndex);	// for IE focus outline to appear, must set tabIndex before focs
			widget.focus(last ? "end" : "start");
			this._set("focusedChild", widget);
		},

		_startupChild: function(/*dijit._Widget*/ widget){
			// summary:
			//		Setup for each child widget
			// description:
			//		Sets tabIndex=-1 on each child, so that the tab key will
			//		leave the container rather than visiting each child.
			// tags:
			//		private

			widget.set("tabIndex", "-1");

			this.connect(widget, "_onFocus", function(){
				// Set valid tabIndex so tabbing away from widget goes to right place, see #10272
				widget.set("tabIndex", this.tabIndex);
			});
			this.connect(widget, "_onBlur", function(){
				widget.set("tabIndex", "-1");
			});
		},

		_onContainerFocus: function(evt){
			// summary:
			//		Handler for when the container gets focus
			// description:
			//		Initially the container itself has a tabIndex, but when it gets
			//		focus, switch focus to first child...
			// tags:
			//		private

			// Note that we can't use _onFocus() because switching focus from the
			// _onFocus() handler confuses the focus.js code
			// (because it causes _onFocusNode() to be called recursively)
			// Also, _onFocus() would fire when focus went directly to a child widget due to mouse click.

			// Ignore spurious focus events:
			//	1. focus on a child widget bubbles on FF
			//	2. on IE, clicking the scrollbar of a select dropdown moves focus from the focused child item to me
			if(evt.target !== this.domNode || this.focusedChild){ return; }

			this.focusFirstChild();

			// and then set the container's tabIndex to -1,
			// (don't remove as that breaks Safari 4)
			// so that tab or shift-tab will go to the fields after/before
			// the container, rather than the container itself
			domAttr.set(this.domNode, "tabIndex", "-1");
		},

		_onBlur: function(evt){
			// When focus is moved away the container, and its descendant (popup) widgets,
			// then restore the container's tabIndex so that user can tab to it again.
			// Note that using _onBlur() so that this doesn't happen when focus is shifted
			// to one of my child widgets (typically a popup)
			if(this.tabIndex){
				domAttr.set(this.domNode, "tabIndex", this.tabIndex);
			}
			this.focusedChild = null;
			this.inherited(arguments);
		},

		_onContainerKeypress: function(evt){
			// summary:
			//		When a key is pressed, if it's an arrow key etc. then
			//		it's handled here.
			// tags:
			//		private
			if(evt.ctrlKey || evt.altKey){ return; }
			var func = this._keyNavCodes[evt.charOrCode];
			if(func){
				func();
				event.stop(evt);
			}
		},

		_onChildBlur: function(/*dijit._Widget*/ /*===== widget =====*/){
			// summary:
			//		Called when focus leaves a child widget to go
			//		to a sibling widget.
			//		Used by MenuBase.js (TODO: move code there)
			// tags:
			//		protected
		},

		_getFirstFocusableChild: function(){
			// summary:
			//		Returns first child that can be focused
			return this._getNextFocusableChild(null, 1);	// dijit._Widget
		},

		_getLastFocusableChild: function(){
			// summary:
			//		Returns last child that can be focused
			return this._getNextFocusableChild(null, -1);	// dijit._Widget
		},

		_getNextFocusableChild: function(child, dir){
			// summary:
			//		Returns the next or previous focusable child, compared
			//		to "child"
			// child: Widget
			//		The current widget
			// dir: Integer
			//		* 1 = after
			//		* -1 = before
			if(child){
				child = this._getSiblingOfChild(child, dir);
			}
			var children = this.getChildren();
			for(var i=0; i < children.length; i++){
				if(!child){
					child = children[(dir>0) ? 0 : (children.length-1)];
				}
				if(child.isFocusable()){
					return child;	// dijit._Widget
				}
				child = this._getSiblingOfChild(child, dir);
			}
			// no focusable child found
			return null;	// dijit._Widget
		}
	});
});

},
'url:ibm/usmi/statuspod/templates/alertStatusTip.html':"<div data-dojo-attach-point=\"tipContent\" class=\"statusPodTip\">\r\n<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n</div>",
'dijit/form/MultiSelect':function(){
define([
	"dojo/_base/array", // array.indexOf, array.map
	"dojo/_base/declare", // declare
	"dojo/dom-geometry", // domGeometry.setMarginBox
	"dojo/query", // query
	"./_FormValueWidget"
], function(array, declare, domGeometry, query, _FormValueWidget){

/*=====
	var _FormValueWidget = dijit.form._FormValueWidget;
=====*/

// module:
//		dijit/form/MultiSelect
// summary:
//		Widget version of a <select multiple=true> element,
//		for selecting multiple options.

return declare("dijit.form.MultiSelect", _FormValueWidget, {
	// summary:
	//		Widget version of a <select multiple=true> element,
	//		for selecting multiple options.

	// size: Number
	//		Number of elements to display on a page
	//		NOTE: may be removed in version 2.0, since elements may have variable height;
	//		set the size via style="..." or CSS class names instead.
	size: 7,

	templateString: "<select multiple='true' ${!nameAttrSetting} data-dojo-attach-point='containerNode,focusNode' data-dojo-attach-event='onchange: _onChange'></select>",

	addSelected: function(/*dijit.form.MultiSelect*/ select){
		// summary:
		//		Move the selected nodes of a passed Select widget
		//		instance to this Select widget.
		//
		// example:
		// |	// move all the selected values from "bar" to "foo"
		// | 	dijit.byId("foo").addSelected(dijit.byId("bar"));

		select.getSelected().forEach(function(n){
			this.containerNode.appendChild(n);
			// scroll to bottom to see item
			// cannot use scrollIntoView since <option> tags don't support all attributes
			// does not work on IE due to a bug where <select> always shows scrollTop = 0
			this.domNode.scrollTop = this.domNode.offsetHeight; // overshoot will be ignored
			// scrolling the source select is trickier esp. on safari who forgets to change the scrollbar size
			var oldscroll = select.domNode.scrollTop;
			select.domNode.scrollTop = 0;
			select.domNode.scrollTop = oldscroll;
		},this);
		this._set('value', this.get('value'));
	},

	getSelected: function(){
		// summary:
		//		Access the NodeList of the selected options directly
		return query("option",this.containerNode).filter(function(n){
			return n.selected; // Boolean
		}); // dojo.NodeList
	},

	_getValueAttr: function(){
		// summary:
		//		Hook so get('value') works.
		// description:
		//		Returns an array of the selected options' values.

		// Don't call getSelect.map() because it doesn't return a real array,
		// and that messes up dojo.toJson() calls like in the Form.html test
		return array.map(this.getSelected(), function(n){
			return n.value;
		});
	},

	multiple: true, // for Form

	_setValueAttr: function(/*Array*/ values, /*Boolean?*/ priorityChange){
		// summary:
		//		Hook so set('value', values) works.
		// description:
		//		Set the value(s) of this Select based on passed values
		query("option",this.containerNode).forEach(function(n){
			n.selected = (array.indexOf(values,n.value) != -1);
		});
		this.inherited(arguments);
	},

	invertSelection: function(/*Boolean?*/ onChange){
		// summary:
		//		Invert the selection
		// onChange: Boolean
		//		If false, onChange is not fired.
		var val = [];
		query("option",this.containerNode).forEach(function(n){
			if(!n.selected){ val.push(n.value); }
		});
		this._setValueAttr(val, !(onChange === false || onChange == null));
	},

	_onChange: function(/*Event*/){
		this._handleOnChange(this.get('value'), true);
	},

	// for layout widgets:
	resize: function(/*Object*/ size){
		if(size){
			domGeometry.setMarginBox(this.domNode, size);
		}
	},

	postCreate: function(){
		this._set('value', this.get('value'));
		this.inherited(arguments);
	}
});

});

},
'ibm/usmi/core/utils/Logger':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/core/utils/Logger", [
	"dojo/_base/declare",
	"dojo/_base/config",
	"dojo/json",
	"dojo/topic",
	"ibm/usmi/widgets/util/Constants"
], function (declare, config, json, topic, Constants) {
	"use strict";
	return declare(null, {
		className : null,
		traceName : "",
		level : 0, // error

		constructor : function (newLoggerName, classOrigin) {
			var traceConfigElement = null, loggerSpecificity = 0;
			this.traceName = newLoggerName;
			this.className = classOrigin || "(unknown class)";

			if (config.trace) {
				// "all" overrides individual class entries
				if (config.trace.all) {
					this.level = this._translateLevel("all", config.trace.all);
				}
				for (traceConfigElement in config.trace) {
					if (config.trace.hasOwnProperty(traceConfigElement)) {
						if (newLoggerName.indexOf(traceConfigElement) === 0) {
							// have a match.  want to make sure that a more specific rule
							// hasn't been defined for the logger.
							if (loggerSpecificity < traceConfigElement.split('.').length) {
								// more dots means more specific
								loggerSpecificity = traceConfigElement.split('.').length;
								this.level = this._translateLevel(name, config.trace[traceConfigElement]);
								if (newLoggerName === traceConfigElement) {
									// found an exact match for the logger.  no need to search further.
									break;
								}
							}
						}
					}
				}
			}
		},

		/////////////////////////////
		// public methods
		/////////////////////////////

		isError : function () {
			return (this.level >= 0);
		},

		isWarn : function () {
			return (this.level >= 1);
		},

		isInfo : function () {
			return (this.level >= 2);
		},

		isFine : function () {
			return (this.level >= 3);
		},

		isFiner : function () {
			return (this.level >= 4);
		},

		isFinest : function () {
			return (this.level >= 5);
		},

		error : function (dbgString, methodName) {
			if (this.isError()) {
				if (dbgString && dbgString instanceof Error) {
					console.error(this._output("EXCEPTION", methodName, dbgString.toString()));
					this._pushToServer("SEVERE", methodName, dbgString.toString());
					//} else if (dbgString && dbgString.isInstanceOf && dbgString.isInstanceOf(ibm.tivoli.navmodel.Exception)) {
					//    console.error(this._output("EXCEPTION", methodName, dbgString.getMsgText()));
				} else {
					console.error(this._output("ERROR", methodName, dbgString));
					this._pushToServer("SEVERE", methodName, dbgString);
				}
			}
		},

		warn : function (dbgString, methodName) {
			if (this.isWarn()) {
				console.warn(this._output("WARN", methodName, dbgString));
				//this._pushToServer("WARNING", methodName, dbgString); //toggle if we want it
			}
		},

		info : function (dbgString, methodName) {
			if (this.isInfo()) {
				console.info(this._output("INFO", methodName, dbgString));
			}
		},

		fine : function (dbgString, methodName) {
			if (this.isFine()) {
				console.info(this._output("FINE", methodName, dbgString));
			}
		},

		finer : function (dbgString, methodName) {
			if (this.isFiner()) {
				console.info(this._output("FINER", methodName, dbgString));
			}
		},

		finest : function (dbgString, methodName) {
			if (this.isFinest()) {
				console.info(this._output("FINEST", methodName, dbgString));
			}
		},

		entering : function (dbgString, methodName) {
			if (this.isFinest()) {
				console.info(this._output("ENTERING", methodName, dbgString));
			}
		},

		exiting : function (dbgString, methodName) {
			if (this.isFinest()) {
				console.info(this._output("EXITING", methodName, dbgString));
			}
		},

		object : function (dbgObject, methodName) {
			if (this.isFine()) {
				console.log(this._output("OBJECT", methodName, json.stringify(dbgObject)));
			}
		},

		assert : function (predicate, message, methodName) {
			if (this.isFine()) {
				if (predicate === false) {
					console.log(this._output("ASSERT", methodName, "assertion failed: " + message));
				}
			}
		},

		/////////////////////////////
		// private methods
		/////////////////////////////

		_translateLevel : function (className, value) {
			var level = 0;

			switch (value) {
			case "off":
				level = -1;
				break;
			case "error":
				level = 0;
				break;
			case "warn":
				level = 1;
				break;
			case "info":
				level = 2;
				break;
			case "fine":
				level = 3;
				break;
			case "finer":
				level = 4;
				break;
			case "finest":
				level = 5;
				break;
			default:
				console.log("className=" + className + " has invalid level=" + value);
			}

			return level;
		},

		//***** DEBUG::trace=ibm.tivoli.navmodel.Dataset, method=EXITING; event received, type = itemStatusChanged
		//::class.method=com.ibm.usmi.services.updates.tasks.UpdateInstallTask.jobActivationUpdated()
		_output : function (level, method, message) {
			var methodName = method || "(unknown method)", outputMessage = "***** " + level;
			if (message) {
				outputMessage = outputMessage + ": " + message;
			}

			outputMessage = outputMessage + " ::class.method=" + this.className + "." + methodName + "() ::loggername="
					+ this.traceName;
			return outputMessage;
		},

		_pushToServer : function (level, method, message) {
			//nested object here. "logData" is key value used on server
			var parm = {
				logData : {
					severity : level,
					className : this.className,
					method : method,
					message : message
				}
			};
			topic.publish(Constants.TOPIC_LOG_JS_TO_SERVER, parm);
		}
	});
});

},
'url:ibm/usmi/scheduler/wizard_pages/templates/SchedulePage.html':"<div>\r\n<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n<!-- \r\n\t<div class=\"tiv-page-header\">\r\n\t\t<h2>Schedule</h2>\r\n\t\t<p>Job name and schedule</p>\r\n\t</div>\r\n\t-->\r\n\t<div class=\"tiv-page-content\" \r\n\t\tstyle=\"margin-left: 5px; margin-bottom: 5px; margin-right: 5px; margin-top: 5px; padding-left: 5px; padding-bottom: 5px; padding-right: 5px; padding-top: 5px;\">\r\n\t\t<label for=\"jobName\" class=\"required\">${persistResources.JOB_NAME_LABEL}</label> \r\n\t\t<input data-dojo-attach-point=\"jobName\" type=\"text\" name=\"widget_name\" data-dojo-type=\"dijit.form.ValidationTextBox\" style=\"width: 100%;\"\r\n\t\t\trequired=\"true\" />\r\n\t</div>\r\n\t<label for=\"WhenToRun\">${persistResources.WHEN_TO_RUN_LABEL}</label>\r\n\t<div id=\"WhenToRun\"\t>\r\n\t\t<input type=\"radio\" name=\"whenToRun\" data-dojo-type=\"dijit/form/RadioButton\" data-dojo-attach-point=\"runNowRadio\" checked value=\"now\"\r\n\t\t\tonclick=\"document.getElementById('repeatStuff').style.display='none'; document.getElementById('scheduleData').style.display='none';\" />${persistResources.RUN_NOW_VALUE} <br /> \r\n\t\t<input type=\"radio\" name=\"whenToRun\" data-dojo-type=\"dijit.form.RadioButton\" id=\"scheduleRadio\" style=\"margin-bottom: 5px; padding-bottom: 5px;\" value=\"schedule\"\r\n\t\t\tonclick=\"document.getElementById('repeatStuff').style.display='block'; document.getElementById('scheduleData').style.display='block';\" />${persistResources.RUN_LATER_VALUE} <br />\r\n\t\t<div id=\"scheduleData\" style=\"display: none;\">\r\n\t\t\t<label id=\"scheduleDateLabel\" for=\"scheduleDate\" style=\"display: inline;\">${persistResources.SCHEDULE_DATE_LABEL}</label> <label for=\"scheduleTime\"\r\n\t\t\t\tid=\"scheduleTimeLabel\" style=\"display: inline; margin-left: 185px;\">${persistResources.SCHEDULE_TIME_LABEL}<br />\r\n\t\t\t</label>\r\n\t\t\t<input data-dojo-attach-point=\"scheduleDate\" required=\"true\" type=\"text\" data-dojo-type=\"dijit.form.DateTextBox\" value=\"now\" /> \r\n\t\t\t<input data-dojo-attach-point=\"scheduleTime\" style=\"margin-left: 50px;\" type=\"text\" data-dojo-type=\"dojox.form.TimeSpinner\" \r\n\t\t\t\thours=\"12\" trim=\"false\" uppercase=\"false\" lowercase=\"false\" propercase=\"false\" required=\"false\" scrollonfocus=\"true\" />\r\n\t\t</div>\r\n\t</div>\r\n\t<div id=\"repeatStuff\" style=\"display: none;\">\r\n\t\t<label for=\"RepeatOptions\">${persistResources.REPEAT_OPTIONS_LABEL}</label>\r\n\t\t<div id=\"RepeatOptions\">\r\n\t\t\t<label for=\"repeatSelection\">${persistResources.FREQUENCY_LABEL}</label> \r\n\t\t\t<select data-dojo-attach-point=\"repeatSelection\" data-dojo-type=\"dijit/form/Select\" type=\"text\" iconclass=\"dijitNoIcon\"\r\n\t\t\t\temptylabel=\"&amp;nbsp;\" required=\"false\" multiple=\"false\" scrollonfocus=\"true\" autowidth=\"true\">\r\n\t\t\t\t<option  value=\"ONCE\" selected=\"selected\">${persistResources.REPEAT_SELECTION_ONCE_VALUE}</option>\r\n\t\t\t\t<option  value=\"HOURLY\">${persistResources.REPEAT_SELECTION_HOURLY_VALUE}</option>\r\n\t\t\t\t<option  value=\"DAILY\" >${persistResources.REPEAT_SELECTION_DAILY_VALUE}</option>\r\n\t\t\t\t<option  value=\"WEEKLY\" >${persistResources.REPEAT_SELECTION_WEEKLY_VALUE}</option>\r\n\t\t\t\t<option  value=\"MONTHLY_DATES\" >${persistResources.REPEAT_SELECTION_MONTHLY_VALUE}</option>\r\n\t\t\t\t<option  value=\"YEARLY\" >${persistResources.REPEAT_SELECTION_YEARLY_VALUE}</option>\r\n\t\t\t\t<option  value=\"CUSTOM\" >${persistResources.REPEAT_SELECTION_CUSTOM_VALUE}</option>\r\n\t\t\t</select><br />\r\n\t\t\t<div data-dojo-attach-point=\"allRepeatDetails\" style=\"display: none;\">\r\n\t\t\t\t<div data-dojo-attach-point=\"weekendStuff\">\r\n\t\t\t\t\t<label for=\"weekendValue\">${persistResources.WEEKEND_LABEL}</label>\r\n\t\t\t\t\t<div id=\"weekendValue\">\r\n\t\t\t\t\t\t<select data-dojo-attach-point=\"weekendOption\" data-dojo-type=\"dijit.form.Select\" type=\"text\"\r\n\t\t\t\t\t\t\ticonclass=\"dijitNoIcon\" emptylabel=\"&amp;nbsp;\" required=\"false\" multiple=\"false\" scrollonfocus=\"true\" autowidth=\"true\">\r\n\t\t\t\t\t\t\t<option value=\"DONTMOVE\" selected=\"selected\">${persistResources.WEEKEND_DONT_MOVE_VALUE}</option>\r\n\t\t\t\t\t\t\t<option value=\"MOVETOFRIDAY\" >${persistResources.WEEKEND_MOVE_TO_FRIDAY_VALUE}</option>\r\n\t\t\t\t\t\t\t<option value=\"MOVETOMONDAY\" >${persistResources.WEEKEND_MOVE_TO_MONDAY_VALUE}</option>\r\n\t\t\t\t\t\t\t<option value=\"MOVETONEARESTWEEKDAY\" >${persistResources.WEEKEND_MOVE_TO_WEEKDAY_VALUE}</option>\r\n\t\t\t\t\t\t\t<option value=\"DELETE\" >${persistResources.WEEKEND_DELETE_VALUE}</option>\r\n\t\t\t\t\t\t</select>\r\n\t\t\t\t\t</div>\r\n\t\t\t\t</div>\r\n\t\t\t\t<div data-dojo-attach-point=\"weeklyStuff\" style=\"display: none;\">\r\n\t\t\t\t\t<label for=\"weeklyValue\">${persistResources.WEEKLY_VALUES_LABEL}</label>\r\n\t\t\t\t\t<div id=\"daysOfWeek\">\r\n\t\t\t\t\t\t<input type=\"checkbox\" data-dojo-attach-point=\"sundayDay\"\r\n\t\t\t\t\t\t\tdata-dojo-type=\"dijit.form.CheckBox\" iconclass=\"dijitNoIcon\" />${persistResources.WEEKLY_SUNDAY_VALUE}\r\n\t\t\t\t\t\t<input type=\"checkbox\" data-dojo-attach-point=\"mondayDay\"\r\n\t\t\t\t\t\t\tdata-dojo-type=\"dijit.form.CheckBox\" iconclass=\"dijitNoIcon\" />${persistResources.WEEKLY_MONDAY_VALUE}\r\n\t\t\t\t\t\t<input type=\"checkbox\" data-dojo-attach-point=\"tuesdayDay\"\r\n\t\t\t\t\t\t\tdata-dojo-type=\"dijit.form.CheckBox\" iconclass=\"dijitNoIcon\" />${persistResources.WEEKLY_TUESDAY_VALUE}\r\n\t\t\t\t\t\t<input type=\"checkbox\" data-dojo-attach-point=\"wednesdayDay\"\r\n\t\t\t\t\t\t\tdata-dojo-type=\"dijit.form.CheckBox\" iconclass=\"dijitNoIcon\" />${persistResources.WEEKLY_WEDNESDAY_VALUE}\r\n\t\t\t\t\t\t<input type=\"checkbox\" data-dojo-attach-point=\"thursdayDay\"\r\n\t\t\t\t\t\t\tdata-dojo-type=\"dijit.form.CheckBox\" iconclass=\"dijitNoIcon\" />${persistResources.WEEKLY_THURSDAY_VALUE}\r\n\t\t\t\t\t\t<input type=\"checkbox\" data-dojo-attach-point=\"fridayDay\"\r\n\t\t\t\t\t\t\tdata-dojo-type=\"dijit.form.CheckBox\" iconclass=\"dijitNoIcon\" />${persistResources.WEEKLY_FRIDAY_VALUE}\r\n\t\t\t\t\t\t<input type=\"checkbox\" data-dojo-attach-point=\"saturdayDay\"\r\n\t\t\t\t\t\t\tdata-dojo-type=\"dijit.form.CheckBox\" iconclass=\"dijitNoIcon\" />${persistResources.WEEKLY_SATURDAY_VALUE}\r\n\t\t\t\t\t</div>\r\n\t\t\t\t</div>\r\n\t\t\t\t<div data-dojo-attach-point=\"monthlyStuff\" style=\"display: none;\">\r\n\t\t\t\t\t<label for=\"selectedDayOfMonth\">${persistResources.MONTHLY_SELECT_LABEL}</label>\r\n\t\t\t\t\t<!--\tTODO:  NOT SURE HOW TO FORMAT THIS - WHEN I TRIED TO USE DIV'S, FORMATTING WAS MESSED UP...I ENDED UP USING A TABLE INSTEAD. -->\t\t\t\t\t\t\r\n\t\t\t\t\t<table border=\"0\">\r\n\t\t\t\t\t\t<tr>\r\n\t\t\t\t\t\t\t<td valign=\"top\">\r\n\t\t\t\t\t\t\t\t<select data-dojo-attach-point=\"dayOfMonthSelection\"\r\n\t\t\t\t\t\t\t\t\tdata-dojo-type=\"dijit.form.Select\" type=\"text\" iconclass=\"dijitNoIcon\"\r\n\t\t\t\t\t\t\t\t\temptylabel=\"&amp;nbsp;\" required=\"false\" multiple=\"false\"\r\n\t\t\t\t\t\t\t\t\tscrollonfocus=\"true\"  style=\"width: 50px;\">\r\n\t\t\t\t\t\t\t\t\t<option value=1 selected=\"selected\">${persistResources.MONTHLY_DATE_1ST_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=2 >${persistResources.MONTHLY_DATE_2ND_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=3 >${persistResources.MONTHLY_DATE_3RD_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=4 >${persistResources.MONTHLY_DATE_4TH_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=5 >${persistResources.MONTHLY_DATE_5TH_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=6 >${persistResources.MONTHLY_DATE_6TH_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=7 >${persistResources.MONTHLY_DATE_7TH_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=8 >${persistResources.MONTHLY_DATE_8TH_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=9 >${persistResources.MONTHLY_DATE_9TH_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=10 >${persistResources.MONTHLY_DATE_10TH_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=11 >${persistResources.MONTHLY_DATE_11TH_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=12 >${persistResources.MONTHLY_DATE_12TH_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=13 >${persistResources.MONTHLY_DATE_13TH_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=14 >${persistResources.MONTHLY_DATE_14TH_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=15 >${persistResources.MONTHLY_DATE_15TH_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=16 >${persistResources.MONTHLY_DATE_16TH_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=17 >${persistResources.MONTHLY_DATE_17TH_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=18 >${persistResources.MONTHLY_DATE_18TH_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=19 >${persistResources.MONTHLY_DATE_19TH_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=20 >${persistResources.MONTHLY_DATE_20TH_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=21 >${persistResources.MONTHLY_DATE_21ST_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=22 >${persistResources.MONTHLY_DATE_22ND_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=23 >${persistResources.MONTHLY_DATE_23RD_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=24 >${persistResources.MONTHLY_DATE_24TH_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=25 >${persistResources.MONTHLY_DATE_25TH_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=26 >${persistResources.MONTHLY_DATE_26TH_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=27 >${persistResources.MONTHLY_DATE_27TH_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=28 >${persistResources.MONTHLY_DATE_28TH_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=29 >${persistResources.MONTHLY_DATE_29TH_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=30 >${persistResources.MONTHLY_DATE_30TH_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=31 >${persistResources.MONTHLY_DATE_31ST_VALUE}</option>\r\n\t\t\r\n\t\t\t\t\t\t\t\t</select>\r\n\t\t\t\t\t\t\t</td>\r\n\t\t\t\t\t\t\t<td valign=\"top\">\r\n\t\t\t\t\t\t\t\t<div id=\"monthButtons\"> \r\n\t\t\t\t\t\t\t\t\t<input data-dojo-attach-point=\"addDayOfMonth\" type=\"button\"\r\n\t\t\t\t\t\t\t\t\t\tdata-dojo-type=\"dijit.form.Button\"\r\n\t\t\t\t\t\t\t\t\t\tintermediatechanges=\"false\" label=${persistResources.ADD_BUTTON_LABEL} iconclass=\"dijitNoIcon\" /><br />\r\n\t\t\t\t\t\t\t\t\t<input data-dojo-attach-point=\"removeDaysOfMonth\" type=\"button\"\r\n\t\t\t\t\t\t\t\t\t\tdata-dojo-type=\"dijit.form.Button\"\r\n\t\t\t\t\t\t\t\t\t\tintermediatechanges=\"false\" label=${persistResources.REMOVE_BUTTON_LABEL}\r\n\t\t\t\t\t\t\t\t\t\ticonclass=\"dijitNoIcon\" />\r\n\t\t\t\t\t\t\t\t</div>\r\n\t\t\t\t\t\t\t</td>\r\n\t\t\t\t\t\t\t<td valign=\"top\">\r\n\t\t\t\t\t\t\t\t<select data-dojo-attach-point=\"selectedMonthlyDates\" multiple=\"multiple\"\r\n\t\t\t\t\t\t\t\t\tdata-dojo-type=\"dijit.form.MultiSelect\"\r\n\t\t\t\t\t\t\t\t\treadonly=\"false\" intermediatechanges=\"false\"\r\n\t\t\t\t\t\t\t\t\tstyle=\"width: 200px;\">\r\n\t\t\t\t\t\t\t\t</select>\r\n\t\t\t\t\t\t\t</td>\r\n\t\t\t\t\t\t</tr>\r\n\t\t\t\t\t</table> \r\n\t\t\t\t\t\r\n\t\t\t\t</div>\r\n\t\t\t\t<div data-dojo-attach-point=\"customStuff\" style=\"display: none;\">\r\n\t\t\t\t\t<label for=\"customValues\">${persistResources.CUSTOM_DATES_ADD_VALUES_PROMPT}</label>\r\n\t\t\t\t\t<div id=\"customDatesSelection\">\r\n\t\t\t\t\t\t<div id=\"customDatesButtons\">\r\n\t\t\t\t\t\t\t<input data-dojo-attach-point=\"addCustomDate\" type=\"button\"\r\n\t\t\t\t\t\t\t\tdata-dojo-type=\"dijit.form.Button\"\r\n\t\t\t\t\t\t\t\tintermediatechanges=\"false\" label=${persistResources.ADD_BUTTON_LABEL} iconclass=\"dijitNoIcon\" />\r\n\t\t\t\t\t\t\t<input data-dojo-attach-point=\"removeCustomDate\" type=\"button\"\r\n\t\t\t\t\t\t\t\tdata-dojo-type=\"dijit.form.Button\"\r\n\t\t\t\t\t\t\t\tintermediatechanges=\"false\" label=${persistResources.REMOVE_BUTTON_LABEL}\r\n\t\t\t\t\t\t\t\ticonclass=\"dijitNoIcon\" />\r\n\t\t\t\t\t\t</div>\r\n\t\t\t\t\t\t<div data-dojo-attach-point=\"selectedCustomDatesDiv\" style=\"display: inline;\">\r\n\t\t\t\t\t\t\t<select data-dojo-attach-point=\"selectedCustomDates\"  multiple=\"multiple\"\r\n\t\t\t\t\t\t\t\tdata-dojo-type=\"dijit.form.MultiSelect\"\r\n\t\t\t\t\t\t\t\treadonly=\"false\" intermediatechanges=\"false\"\r\n\t\t\t\t\t\t\t\tstyle=\"width: 400px;\">\r\n\t\t\t\t\t\t\t</select>\r\n\t\t\t\t\t\t</div>\r\n\t\t\t\t\t</div>\r\n\t\t\t\t</div>\r\n\t\t\t\t<div data-dojo-attach-point=\"durationStuff\">\r\n\t\t\t\t\t<label for=\"durationValue\">${persistResources.DURATION_LABEL}</label>\r\n\t\t\t\t\t<div id=\"durationValue\">\r\n\t\t\t\t\t\t<input type=\"radio\" name=\"duration\"\r\n\t\t\t\t\t\t\tdata-dojo-type=\"dijit.form.RadioButton\"\r\n\t\t\t\t\t\t\tdata-dojo-attach-point=\"forDurationRadio\" value=\"for\" checked\r\n\t\t\t\t\t\t\tonclick=\"document.getElementById('repeatForDiv').style.display='inline';  \r\n\t\t\t\t\t\t\t\t\tdocument.getElementById('repeatUntilDiv').style.display='none';\r\n\t\t\t\t\t\t\t\t\tdocument.getElementById('repeatForeverDiv').style.display='none';\" />${persistResources.REPEAT_FOR_PROMPT}<br /> \r\n\t\t\t\t\t\t<input type=\"radio\" name=\"duration\"\r\n\t\t\t\t\t\t\tdata-dojo-type=\"dijit.form.RadioButton\" id=\"untilDurationRadio\"\r\n\t\t\t\t\t\t\tdata-dojo-attach-point= \"untilDurationRadio\" value=\"until\"\r\n\t\t\t\t\t\t\tonclick=\"document.getElementById('repeatForDiv').style.display='none'; \r\n\t\t\t\t\t\t\t\t\tdocument.getElementById('repeatUntilDiv').style.display='inline';\r\n\t\t\t\t\t\t\t\t\tdocument.getElementById('repeatForeverDiv').style.display='none';\" />${persistResources.REPEAT_UNTIL_PROMPT}\r\n\t\t\t\t\t\t\t<div id=\"repeatUntilDiv\" style=\"display: none;\">\r\n\t\t\t\t\t\t\t\t<label for=\"repeateUntil\" class=\"required\" style=\"display: inline; margin-left: 75px;\">${persistResources.REPEAT_UNTIL_VALUE_LABEL}</label> \r\n\t\t\t\t\t\t\t\t<input data-dojo-attach-point=\"repeatUntil\" required=\"true\" type=\"text\" data-dojo-type=\"dijit.form.DateTextBox\" value=\"now\"><br />\r\n\t\t\t\t\t\t\t</div>\r\n\t\t\t\t\t\t\t<div id=\"repeatForDiv\" style=\"display: inline;\">\r\n\t\t\t\t\t\t\t\t<label for=\"repeatFor\" class=\"required\"\tstyle=\"display: inline; margin-left: 75px;\">${persistResources.REPEAT_FOR_VALUE_LABEL}</label>\r\n\t\t\t\t\t\t\t\t<input type=\"text\" data-dojo-attach-point=\"repeatFor\" data-dojo-type=\"dijit.form.NumberSpinner\"\r\n\t\t\t\t\t\t\t\t\tdata-dojo-props=\"constraints:{min:1,max:5000,places:0}\" value=\"1\" /> \r\n\t\t\t\t\t\t\t\t<select data-dojo-type=\"dijit.form.Select\" type=\"text\"\r\n\t\t\t\t\t\t\t\t\tdata-dojo-attach-point=\"repeatForUnits\" iconclass=\"dijitNoIcon\"\r\n\t\t\t\t\t\t\t\t\temptylabel=\"&amp;nbsp;\" required=\"false\" multiple=\"false\"\r\n\t\t\t\t\t\t\t\t\tscrollonfocus=\"true\" autowidth=\"true\">\r\n\t\t\t\t\t\t\t\t\t<option value=\"FOR_HOURS\" selected=\"selected\">\r\n\t\t\t\t\t\t\t\t\t\t${persistResources.REPEAT_FOR_HOURS_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=\"FOR_DAYS\" >\r\n\t\t\t\t\t\t\t\t\t\t${persistResources.REPEAT_FOR_DAYS_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=\"FOR_WEEKS\" >\r\n\t\t\t\t\t\t\t\t\t\t${persistResources.REPEAT_FOR_WEEKS_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=\"FOR_MONTHS\" >\r\n\t\t\t\t\t\t\t\t\t\t${persistResources.REPEAT_FOR_MONTHS_VALUE}</option>\r\n\t\t\t\t\t\t\t\t\t<option value=\"FOR_YEARS\" >\r\n\t\t\t\t\t\t\t\t\t\t${persistResources.REPEAT_FOR_YEARS_VALUE}</option>\r\n\t\t\t\t\t\t\t\t</select> <br />\r\n\t\t\t\t\t\t\t</div>\r\n\t\t\t\t\t\t<div id=\"repeatForeverDiv\" style=\"display: none;\">\r\n\t\t\t\t\t\t\t<label style=\"display: inline; margin-left: 75px;\">${persistResources.REPEAT_FOREVER_VALUE}</label> <br />\r\n\t\t\t\t\t\t</div>\r\n\t\t\t\t\t\t<input type=\"radio\" name=\"duration\" data-dojo-type=\"dijit.form.RadioButton\"\r\n\t\t\t\t\t\t\tid=\"unlimitedDurationRadio\" data-dojo-attach-point=\"unlimitedDurationRadio\" value=\"unlimited\"\r\n\t\t\t\t\t\t\tonclick=\"document.getElementById('repeatForeverDiv').style.display='inline'; \r\n\t\t\t\t\t\t\t\t\tdocument.getElementById('repeatForDiv').style.display='none';\r\n\t\t\t\t\t\t\t\t\tdocument.getElementById('repeatUntilDiv').style.display='none';\" />${persistResources.REPEAT_FOREVER_PROMPT}\r\n\t\t\t\t\t\t<br />\r\n\t\t\t\t\t</div>\r\n\t\t\t\t</div>\r\n\t\t\t</div>\r\n\t\t</div>\r\n\t</div>\r\n</div>",
'idx/oneui/messaging/SingleMessage':function(){
/*
 * Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

define([
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/event",
	"dojo/_base/lang",
	"dojo/_base/sniff",
	"dojo/date/locale",
	"dojo/dom-class",
	"dojo/dom-attr",
	"dojo/dom-style",
	"dojo/dom-geometry",
	"dojo/i18n",
	"dojo/keys",
	"dijit/focus",
	"dijit/_Widget",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	"dojo/text!./templates/SingleMessage.html",
	"dojo/i18n!./nls/_MessageItem",
	"dojox/html/ellipsis"
], function(declare, array, event, lang, has, locale, domClass, domAttr, domStyle, domGeometry, i18n, keys, focus, _Widget, _TemplatedMixin,
			_WidgetsInTemplateMixin, template){
	/**
	 * @name idx.oneui.messaging.SingleMessage
	 * @class SingleMessage is implemented according to IBM One UI(tm) 
	 * <b><a href="http://dleadp.torolab.ibm.com/uxd/uxd_oneui.jsp?site=ibmoneui&top=x1&left=y11&vsub=*&hsub=*&options=M&openpanes=0000011000">Messaging Standard</a></b>.
	 * It provides a convenient way to create One UI compliant messages in a designated area. SingleMessage is highly configurable with following options:
	 * <ul>
	 *    <li><b>Message type</b>: SingleMessage supports 7 different message types. It can be configured via setting the 'type' property.</li>
	 *    <li><b>Message ID</b>: The ID of a SingleMessage can be configured via  setting the 'messageId' property. Setting the 'showId' property can trun on/off the Message ID.</li>
	 *    <li><b>Timestamp</b>: The timestamp of a SingleMessage can be configured via  setting the 'date' property. In addtion, the date format can be set with the
	 *    'dateFormat' property</li>
	 *    <li><b>Message text</b>: The message text can be configured via setting the 'title' property. The message text gets truncated with an ellipsis automatically when a SingleMessage resizes.</li>
	 *    <li><b>Explanation</b>: The message explanation can be confgiured via setting the 'description' property.</li>
	 *    <li><b>Refresh Button</b>: A SingleMessage contains a refresh link after the message explanation by default. It can be turned off by setting 'showRefresh' property to false.</li>
	 *    <li><b>View All</b> link: A SingleMessage contains a View All(n) link by default. The message number 'n' can be configured with 'messageNumber' property. It can be turned off by setting
	 *     'showAction' property to false.
	 *    </li>
	 * <ul>
	 * @augments dijit._Widget
	 * @augments dijit._TemplatedMixin
	 * @augments dijit._WidgetsInTemplateMixin
	 * @example
	 * Programmatic Example:
	 * <pre>
	 * new idx.oneui.SingleMessage({
	 *  type: 'error',
	 *  title: 'Error message with long timestamp',
	 *  dateFormat: {datePattern: 'dd MMMM y &nbsp', timePattern: 'hh:mm a'},
	 *  messageId: 'CAT123456',
	 *  messageNumber: 7,
	 *  style: 'width: 970px;',
	 *  description: 'Here is the detail message description, it can be configured via setting the \'description\' parameter of a SingleMessage widget. By default the message description is the same as the message title.'
	 * }, domNode);
	 * </pre>
	 * Declarative Example:
	 * <pre>
	 *    &lt;div data-dojo-type="idx.oneui.messaging.SingleMessage" data-dojo-props="type: 'error', title: 'Error message with long timestamp',
	 *    dateFormat:{datePattern: 'dd MMMM y &nbsp', timePattern: 'hh:mm a'}, messageId: 'CAT123456', messageNumber: 7, style: 'width: 970px;', description: 'Here is the detail message description, it can be configured via setting the \'description\' parameter of a SingleMessage widget. By default the message description is the same as the message title.'"></div>
	 * </pre>
	 */
	return declare("idx.oneui.messaging.SingleMessage", [_Widget, _TemplatedMixin, _WidgetsInTemplateMixin],
	/**@lends idx.oneui.messaging.SingleMessage.prototype*/
	{
		templateString: template,
		
		baseClass: "idxSingleMessage",
		
		tabIndex: "0",
		
		/**
		 * The ID of a SingleMessage. Can be turned off by setting 'showId' to false.
		 * For message type 'error', 'warning', 'critical', message id is required.
		 * @type String
		 */
		messageId: "",
		
		/**
		 * The type of a Single Message. Can be one of 'error', 'warning', 'success',
		 * 'information', 'critical', 'attention', 'compliance'.
		 * @type String
		 * @default "error"
		 */
		type: "error",
		
		/**
		 * The timestamp of a Single Message. The timestamp format can be configured with 'dateFormat'.
		 * Example:
		 * date: new Date()
		 * @type Date
		 */
		date: new Date(),
		
		/**
		 * The options being used for format the timestamp.
		 * Example:
		 * <pre>
		 * dateFormat: {
		 * 	formatLength: "medium",
		 * 	locale: this.lang
		 * }
		 * </pre>
		 * @type dojo.date.locale.__FormatOptions
		 */
		dateFormat: {
			formatLength: "medium",
			locale: this.lang
		},
		
		/**
		 * The message text of a SingleMessage. It will be trancated with an ellipsis if needed when
		 * SingleMessage resizes.
		 * @type String
		 */
		title: "",
		
		/**
		 * The text of the action link. By default, the action link is used as the View All link.
		 * The value is loaded from the nls bundle. In most cases, you should not change it. But in special cases,
		 * you still can change it with a string.
		 * @type String
		 */
		actionText: "",
		
		/**
		 * 'showId' decides whether to show the message ID. By default, for success message and information message, it is
		 * turned off.
		 * Note: For error, critical, warning messages, message ID is required and cannot be turned off.
		 * @type Boolean
		 * @default true
		 */
		showId: true,
		
		/**
		 * 'showAction' decides whether to show the action link (ususally, it's View All link).
		 * @type Boolean
		 * @default true
		 */
		showAction: true,
		
		/**
		 * 'showRefresh' decides whether to show the refresh link in the message description part.
		 * @type Boolean
		 * @default true
		 */
		showRefresh: true,
		
		/**
		 * 'showRefresh' decides whether to show the More Details link in the message description part.
		 * @type Boolean
		 * @default true
		 */
		showDetailsLink: true,
		
		/**
		 * The message number to be put in the View All link 'View All(n)'.
		 * @type Integer | String
		 * @default "n"
		 */
		messageNumber: "n",
		
		/**
		 * The explanation of a SingleMessage.
		 * @type String
		 */
		description: "",
		
		/**
		 * 'collapsed' decides whether the explanation part is hidden.
		 * @type Boolean
		 */
		collapsed: true,
		
		/**
		 * 'closable' decides whether the SingleMessage can be closed.
		 * @type Boolean
		 */
		closable: true,
		
		/**
		 * Alt text for message icons
		 * @private
		 * @type Object
		 */
		_iconTextMap: {
			"error": "X",
			"warning": "!",
			"information": "i",
			"success": "&#8730;",
			"critical": "X",
			"attention": "&#9670",
			"compliance": "&#9671"
		},
		
		
		/**
		 * Possible message types
		 * @private
		 * @type Array
		 * @default ["error", "warning", "information", "success", "critical", "attention", "compliance"]
		 */
		_allowedTypes: ["error", "warning", "information", "success", "critical", "attention", "compliance"],
		
		postMixInProperties: function(){
			this.inherited(arguments);
			this._nlsResources = i18n.getLocalization("idx.oneui.messaging", "SingleMessage", this.lang);
			this.type = array.indexOf(this.type.toLowerCase()) ? this.type.toLowerCase() : "error";
			if(!this.description){
				this.description = this.title;
			}
		},
		
		postCreate: function(){
			// summary:
			//		Set tab index and time stamp for the message
			this.inherited(arguments);
			this._created = true;
			if(!this.actionText){
				this.set("actionText", this._nlsResources.viewAll);
			}
			domAttr.set(this.viewDetailsNode, "innerHTML", this._nlsResources.viewDetails);
			domAttr.set(this.refreshNode, "innerHTML", this._nlsResources.refresh);
			this.set("title", this.title);
		},
		
		_setActionTextAttr: function(value){
			this._set("actionText", value);
			domAttr.set(this.actionNode, "innerHTML", lang.replace(value, {num: this.messageNumber}));
		},
		
		_setTitleAttr: function(value){
			this._set("title", value);
			this._resizeTitle();
			domAttr.set(this.fakeTitleNode, "innerHTML", value + "&nbsp;&nbsp;");
		},
		
		
		_setDescriptionAttr: function(value){
			this._set("description", value);
			domAttr.set(this.descriptionNode, "innerHTML", value);
		},
		
		_setMaxLengthAttr: function(value){
			this._set("maxLength", value);
			this._resizeTitle();
		},
		
		_setMessageIdAttr: function(/*String*/ value){
			domAttr.set(this.idNode, "innerHTML", value);
			this._set("messageId", value);
			this._resizeTitle();
			domAttr.set(this.fakeIdNode, "innerHTML", value);
		},
		
		_setTypeAttr: function(/*String*/ value){
			domAttr.set(this.typeNode, "innerHTML", this._iconTextMap[value]);
			domClass.toggle(this.domNode, this.value + "Message", false);
			domClass.toggle(this.domNode, value + "Message", true);
			this._set("type", value);
			this._toggleId();
		},
		
		_setDateAttr: function(/*Date*/ value){
			this._set("date", value);
			domAttr.set(this.timestampNode, "innerHTML", locale.format(this.date, this.dateFormat));
			this._resizeTitle();
		},
		
		_setDateFormatAttr: function(/*dojo.date.locale.__FormatOptions?*/ value){
			this._set("dateFormat", value);
			domAttr.set(this.timestampNode, "innerHTML", locale.format(this.date, this.dateFormat));
			this._resizeTitle();
		},
		
		_setMessageNumberAttr: function(/*Integer|String*/ value){
			this._set("messageNumber", value);
			this.set("actionText", this.actionText);
			this._resizeTitle();
		},
		
		_setShowIdAttr: function(/*Boolean*/ value){
			this._set("showId", value);
			this._toggleId();
			this._resizeTitle();
		},
		
		_setShowActionAttr: function(/*Boolean*/ value){
			this._set("showAction", value);
			domClass.toggle(this.actionNode, "dijitHidden", !this.showAction);
			domClass.toggle(this.separatorNode, "dijitHidden", !this.showAction);
			if(has("ie") == 6 || has("ie") == 7){
				domClass.toggle(this.timestampNode, "idxMessageTimeStampMargin", !value);
			}
			this._resizeTitle();
		},
		
		_setShowRefreshAttr: function(/*Boolean*/ value){
			this._set("showRefresh", value);
			domClass.toggle(this.refreshNode, "dijitHidden", !this.showRefresh);
		},
		
		_setShowDetailsLinkAttr: function(/*Boolean*/ value){
			this._set("showDetailsLink", value);
			domClass.toggle(this.viewDetailsNode, "dijitHidden", !this.showDetailsLink);
		},
		
		_setClosableAttr: function(/*Boolean*/ value){
			this._set("closable", value);
			domStyle.set(this.closeNode, {
				"visibility": this.closable ? "visible" : "hidden"
			});
		},
		
		_toggleId: function(){
			if(this.type == "information" || this.type == "success" || this.type == "attention" || this.type == "compliance"){
				domClass.toggle(this.idNode, "dijitHidden", !this.showId);
				domClass.toggle(this.fakeIdNode, "dijitHidden", !this.showId);
			}
		},
		
		_resizeTitle: function(){
			if(!this._created){
				return;
			}
			domStyle.set(this.titleNode, {"width": "auto"});
			domAttr.set(this.titleNode, {"innerHTML": ''});
			if(this.collapsed){
				var idWidth = domGeometry.getMarginBox(this.idNode).w;
				var width = domGeometry.getContentBox(this.domNode).w - domGeometry.getMarginBox(this.iconNode).w
							- domGeometry.getMarginBox(this.infoNode).w - idWidth;
				domAttr.set(this.titleNode, {"innerHTML": '<div class="messageTitles">' + this.title + '&nbsp&nbsp</div>'});
				var currentWidth = domStyle.get(this.titleNode, "width");
				if(width > 20){
					width = width - 10;
				}

				if(currentWidth > width){
					if(width < 0){
						width = 0;
					}
					domStyle.set(this.titleNode, {"width": width + "px"});
					domAttr.set(this.titleNode, {"innerHTML": '<div class="messageTitles dojoxEllipsis">' + this.title + '&nbsp&nbsp</div>'});;
					//console.log("idWidth:" + idWidth + ",width:" + width + ", currentWidth:" + currentWidth);
					domStyle.set(this.fakeFocusNode, {"width": width + idWidth + "px"});
				}else{
					domAttr.set(this.titleNode, {"innerHTML": '<div class="messageTitles">' + this.title + '&nbsp&nbsp</div>'});;
				}
			}else{
				var idWidth = domGeometry.getMarginBox(this.fakeIdNode).w;
				var width = domGeometry.getContentBox(this.domNode).w - domGeometry.getMarginBox(this.iconNode).w
							- domGeometry.getMarginBox(this.infoNode).w - idWidth;
				var currentWidth = domStyle.get(this.fakeTitleNode, "width");
				if(width > 20){
					width = width - 10;
				}
				domStyle.set(this.fakeFocusNode, {"width": width + idWidth + "px"});
			}
		},
		
		_setCollapsedAttr: function(value){
			if(value){
				domAttr.set(this.focusNode, "title", this._nlsResources.showDetails);
			}else{
				domAttr.set(this.fakeFocusNode, "title", this._nlsResources.hideDetails);
			}
			
			domClass.toggle(this.domNode, "idxMessageCollapsed", value);
			if(has("ie") == 6 || has("ie") == 7){
				if(value){
					domStyle.set(this.domNode, {"height": "33px"});
				}else{
					domStyle.set(this.domNode, {"height": "auto"});
				}
			}
			domClass.toggle(this.focusNode, "dijitHidden", !value);
			domClass.toggle(this.fakeFocusNode, "dijitHidden", value);
			this._set("collapsed", value);
			this._resizeTitle();
			
			if(has("ie") == 6){
				this.resize();
			}
		},
		
		_onClick: function(e){
			this.set("collapsed", !this.collapsed);
			focus.focus(this.collapsed ? this.focusNode : this.fakeFocusNode);
			this.onClick(e);
		},
		
		_onClose: function(e){
			this.onClose(e);
			this.destroy();
		},
		/**
		 * Ajust the content of the SingleMessage.
		 */
		resize: function(){
			this._resizeTitle();
		},
		
		/**
		 * Event triggered  when the close button is clicked. Before 'destroy' method is called. 
		 */
		onClose: function(e){
		},
		
		/**
		 * Event triggered when mouse enter the icon image.
		 */
		onIconEnter: function(e){
		},
		
		/**
		 * Event triggered when mouse leave the icon image.
		 */
		onIconLeave: function(e){
		},
		
		/**
		 * Event triggered when message title is clicked.
		 */
		onClick: function(e){
		},
		
		/**
		 * Event triggered when action link 'View All' is clicked.
		 */
		onAction: function(e){
		},
		
		/**
		 * Event triggered when action link 'Refresh' is clicked.
		 */
		onRefresh: function(e){
		},
		
		/**
		 * Event triggered when action link 'More Details' is clicked.
		 */
		onMoreDetails: function(e){
		}
	});
});




},
'gridx/core/model/cache/Sync':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/Deferred",
	"./_Cache"
], function(declare, lang, Deferred, _Cache){

	return declare(/*===== "gridx.core.model.cache.Async", =====*/_Cache, {
		keep: function(){},
		free: function(){},

		when: function(args, callback){
			var d = new Deferred();
			try{
				if(callback){
					callback();
				}
				d.callback();
			}catch(e){
				d.errback(e);
			}
			return d;
		},

		//Private---------------------------------------------
		_init: function(/*method, args*/){
			var t = this;
			if(!t._filled){
				t._storeFetch({ start: 0 });
				if(t.store.getChildren){
					t._fetchChildren();
				}
				t.model._onSizeChange();
			}
		},

		_fetchChildren: function(){
			var s = this._struct,
				pids = s[''].slice(1),
				pid,
				appendChildren = function(pid){
					[].push.apply(pids, s[pid].slice(1));
				};
			while(pids.length){
				pid = pids.shift();
				Deferred.when(this._loadChildren(pid), lang.partial(appendChildren, pid));
			}
		}
	});
});

},
'dijit/form/ComboButton':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/_base/event", // event.stop
	"dojo/_base/lang", // lang.hitch lang.isFunction
	"../registry",	// registry.byNode
	"dojo/keys", // keys
	"../focus",		// focus.focus()
	"./DropDownButton",
	"dojo/text!./templates/ComboButton.html"
], function(declare, event, lang, registry, keys, focus, DropDownButton, template){

/*=====
	var DropDownButton = dijit.form.DropDownButton;
=====*/

// module:
//		dijit/form/ComboButton
// summary:
//		A combination button and drop-down button.

return declare("dijit.form.ComboButton", DropDownButton, {
	// summary:
	//		A combination button and drop-down button.
	//		Users can click one side to "press" the button, or click an arrow
	//		icon to display the drop down.
	//
	// example:
	// |	<button data-dojo-type="dijit.form.ComboButton" onClick="...">
	// |		<span>Hello world</span>
	// |		<div data-dojo-type="dijit.Menu">...</div>
	// |	</button>
	//
	// example:
	// |	var button1 = new dijit.form.ComboButton({label: "hello world", onClick: foo, dropDown: "myMenu"});
	// |	dojo.body().appendChild(button1.domNode);
	//

	templateString: template,

	// Map widget attributes to DOMNode attributes.
	_setIdAttr: "",	// override _FormWidgetMixin which puts id on the focusNode
	_setTabIndexAttr: ["focusNode", "titleNode"],
	_setTitleAttr: "titleNode",

	// optionsTitle: String
	//		Text that describes the options menu (accessibility)
	optionsTitle: "",

	baseClass: "dijitComboButton",

	// Set classes like dijitButtonContentsHover or dijitArrowButtonActive depending on
	// mouse action over specified node
	cssStateNodes: {
		"buttonNode": "dijitButtonNode",
		"titleNode": "dijitButtonContents",
		"_popupStateNode": "dijitDownArrowButton"
	},

	_focusedNode: null,
	
	//dynamic:Boolean
	// If enabled the ComboButton default action is set with menu item selected
	dynamic: false,

	_onButtonKeyPress: function(/*Event*/ evt){
		// summary:
		//		Handler for right arrow key when focus is on left part of button
		if(evt.charOrCode == keys[this.isLeftToRight() ? "RIGHT_ARROW" : "LEFT_ARROW"]){
			focus.focus(this._popupStateNode);
			event.stop(evt);
		}
	},

	_onArrowKeyPress: function(/*Event*/ evt){
		// summary:
		//		Handler for left arrow key when focus is on right part of button
		if(evt.charOrCode == keys[this.isLeftToRight() ? "LEFT_ARROW" : "RIGHT_ARROW"]){
			focus.focus(this.titleNode);
			event.stop(evt);
		}
	},
	_onDropDownMouseUp: function(/*Event?*/ e){
		//summary:
		// overrides _HasDropDown._onDropDownMouseUp to disconnect custom event handlers
		// if dynamic then check for attached event handlers for drop down and disconnect
		if(this.dynamic && this._onItemClickHandler){
			this.disconnect(this._onItemClickHandler);
			this.disconnect(this._buttonClickHandler);
			this.disconnect(this._onMouseOverHandler);
		}
	
		this.inherited(arguments);
	},
	
	_onDropDownClick: function(/*Event*/ e){
		//summary:
		//overrides _HasDropDown._onDropDownClick
		//Connect to onItemClick and onMouseOver handlers to dynamically update the button with selected item action.
			if(this.dynamic){
				this._onItemClickHandler = this.connect(this.dropDown,"onItemClick",lang.hitch(this, "_onDropDownMenuChange"));
				this._onMouseOverHandler = this.connect(this.dropDown,"onMouseOver",lang.hitch(this, "_onDropDownMouseOver"));
			}
			
			this.inherited(arguments);
	},
	
	_onDropDownMouseOver: function(evt){
		//summary:
		//to handle the scenario where there are submenus
	 	//if dropdown consists of sub menu, then attach onItemClick handler to submenu on mouseOver	
		var t = evt.target;
		var menuItem;
		while(t && !(menuItem =  registry.byNode(t))){
			t = t.parentNode;
		}
		if(menuItem.popup){
			this._onItemClickHandler = this.connect(menuItem.popup,"onItemClick",lang.hitch(this, "_onDropDownMenuChange"));
				
		}
	},

	_onDropDownMenuChange: function(item, evt){
		//summary:
		// handles onItemClick event of the drop down to make combo button dynamic
		// update the button to take up new selected menu item action only if it doesn't have sub menu and is not disabled
		if(!item.popup && !item.disabled && item.dynamic){
			if(this.showLabel){
				this.set('label',item.label)  ;
			}
			this.set('iconClass',item.iconClass)  ;
			//connect ComboButton onClick to selected item onClick handler.
			this._buttonClickHandler = this.connect(this, "onClick",lang.hitch(item, "onClick"));
		}
	},

	focus: function(/*String*/ position){
		// summary:
		//		Focuses this widget to according to position, if specified,
		//		otherwise on arrow node
		// position:
		//		"start" or "end"
		if(!this.disabled){
			focus.focus(position == "start" ? this.titleNode : this._popupStateNode);
		}
	}
});

});

},
'dojox/grid/cells/_base':function(){
define([
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/event",
	"dojo/_base/connect",
	"dojo/_base/array",
	"dojo/_base/sniff",
	"dojo/dom",
	"dojo/dom-attr",
	"dojo/dom-construct",
	"dijit/_Widget",
	"../util"
], function(dojo, declare, lang, event, connect, array, has, dom, domAttr, domConstruct, _Widget, util){

	var _DeferredTextWidget = declare("dojox.grid._DeferredTextWidget", _Widget, {
		deferred: null,
		_destroyOnRemove: true,
		postCreate: function(){
			if(this.deferred){
				this.deferred.addBoth(lang.hitch(this, function(text){
					if(this.domNode){
						this.domNode.innerHTML = text;
					}
				}));
			}
		}
	});

	var focusSelectNode = function(inNode){
		try{
			util.fire(inNode, "focus");
			util.fire(inNode, "select");
		}catch(e){// IE sux bad
		}
	};
	
	var whenIdle = function(/*inContext, inMethod, args ...*/){
		setTimeout(lang.hitch.apply(dojo, arguments), 0);
	};

	var BaseCell = declare("dojox.grid.cells._Base", null, {
		// summary:
		//	Respresents a grid cell and contains information about column options and methods
		//	for retrieving cell related information.
		//	Each column in a grid layout has a cell object and most events and many methods
		//	provide access to these objects.
		styles: '',
		classes: '',
		editable: false,
		alwaysEditing: false,
		formatter: null,
		defaultValue: '...',
		value: null,
		hidden: false,
		noresize: false,
		draggable: true,
		//private
		_valueProp: "value",
		_formatPending: false,

		constructor: function(inProps){
			this._props = inProps || {};
			lang.mixin(this, inProps);
			if(this.draggable === undefined){
				this.draggable = true;
			}
		},

		_defaultFormat: function(inValue, callArgs){
			var s = this.grid.formatterScope || this;
			var f = this.formatter;
			if(f && s && typeof f == "string"){
				f = this.formatter = s[f];
			}
			var v = (inValue != this.defaultValue && f) ? f.apply(s, callArgs) : inValue;
			if(typeof v == "undefined"){
				return this.defaultValue;
			}
			if(v && v.addBoth){
				// Check if it's a deferred
				v = new _DeferredTextWidget({deferred: v},
									domConstruct.create("span", {innerHTML: this.defaultValue}));
			}
			if(v && v.declaredClass && v.startup){
				return "<div class='dojoxGridStubNode' linkWidget='" +
						v.id +
						"' cellIdx='" +
						this.index +
						"'>" +
						this.defaultValue +
						"</div>";
			}
			return v;
		},
		
		// data source
		format: function(inRowIndex, inItem){
			// summary:
			//	provides the html for a given grid cell.
			// inRowIndex: int
			// grid row index
			// returns: html for a given grid cell
			var f, i=this.grid.edit.info, d=this.get ? this.get(inRowIndex, inItem) : (this.value || this.defaultValue);
			d = (d && d.replace && this.grid.escapeHTMLInData) ? d.replace(/&/g, '&amp;').replace(/</g, '&lt;') : d;
			if(this.editable && (this.alwaysEditing || (i.rowIndex==inRowIndex && i.cell==this))){
				return this.formatEditing(d, inRowIndex);
			}else{
				return this._defaultFormat(d, [d, inRowIndex, this]);
			}
		},
		formatEditing: function(inDatum, inRowIndex){
			// summary:
			//	formats the cell for editing
			// inDatum: anything
			//	cell data to edit
			// inRowIndex: int
			//	grid row index
			// returns: string of html to place in grid cell
		},
		// utility
		getNode: function(inRowIndex){
			// summary:
			//	gets the dom node for a given grid cell.
			// inRowIndex: int
			// grid row index
			// returns: dom node for a given grid cell
			return this.view.getCellNode(inRowIndex, this.index);
		},
		getHeaderNode: function(){
			return this.view.getHeaderCellNode(this.index);
		},
		getEditNode: function(inRowIndex){
			return (this.getNode(inRowIndex) || 0).firstChild || 0;
		},
		canResize: function(){
			var uw = this.unitWidth;
			return uw && (uw!=='auto');
		},
		isFlex: function(){
			var uw = this.unitWidth;
			return uw && lang.isString(uw) && (uw=='auto' || uw.slice(-1)=='%');
		},
		// edit support
		applyEdit: function(inValue, inRowIndex){
			this.grid.edit.applyCellEdit(inValue, this, inRowIndex);
		},
		cancelEdit: function(inRowIndex){
			this.grid.doCancelEdit(inRowIndex);
		},
		_onEditBlur: function(inRowIndex){
			if(this.grid.edit.isEditCell(inRowIndex, this.index)){
				//console.log('editor onblur', e);
				this.grid.edit.apply();
			}
		},
		registerOnBlur: function(inNode, inRowIndex){
			if(this.commitOnBlur){
				connect.connect(inNode, "onblur", function(e){
					// hack: if editor still thinks this editor is current some ms after it blurs, assume we've focused away from grid
					setTimeout(lang.hitch(this, "_onEditBlur", inRowIndex), 250);
				});
			}
		},
		//protected
		needFormatNode: function(inDatum, inRowIndex){
			this._formatPending = true;
			whenIdle(this, "_formatNode", inDatum, inRowIndex);
		},
		cancelFormatNode: function(){
			this._formatPending = false;
		},
		//private
		_formatNode: function(inDatum, inRowIndex){
			if(this._formatPending){
				this._formatPending = false;
				// make cell selectable
				if(!has("ie")){
					dom.setSelectable(this.grid.domNode, true);
				}
				this.formatNode(this.getEditNode(inRowIndex), inDatum, inRowIndex);
			}
		},
		//protected
		formatNode: function(inNode, inDatum, inRowIndex){
			// summary:
			//	format the editing dom node. Use when editor is a widget.
			// inNode: dom node
			// dom node for the editor
			// inDatum: anything
			//	cell data to edit
			// inRowIndex: int
			//	grid row index
			if(has("ie")){
				// IE sux bad
				whenIdle(this, "focus", inRowIndex, inNode);
			}else{
				this.focus(inRowIndex, inNode);
			}
		},
		dispatchEvent: function(m, e){
			if(m in this){
				return this[m](e);
			}
		},
		//public
		getValue: function(inRowIndex){
			// summary:
			//	returns value entered into editor
			// inRowIndex: int
			// grid row index
			// returns:
			//	value of editor
			return this.getEditNode(inRowIndex)[this._valueProp];
		},
		setValue: function(inRowIndex, inValue){
			// summary:
			//	set the value of the grid editor
			// inRowIndex: int
			// grid row index
			// inValue: anything
			//	value of editor
			var n = this.getEditNode(inRowIndex);
			if(n){
				n[this._valueProp] = inValue;
			}
		},
		focus: function(inRowIndex, inNode){
			// summary:
			//	focus the grid editor
			// inRowIndex: int
			// grid row index
			// inNode: dom node
			//	editor node
			focusSelectNode(inNode || this.getEditNode(inRowIndex));
		},
		save: function(inRowIndex){
			// summary:
			//	save editor state
			// inRowIndex: int
			// grid row index
			this.value = this.value || this.getValue(inRowIndex);
			//console.log("save", this.value, inCell.index, inRowIndex);
		},
		restore: function(inRowIndex){
			// summary:
			//	restore editor state
			// inRowIndex: int
			// grid row index
			this.setValue(inRowIndex, this.value);
			//console.log("restore", this.value, inCell.index, inRowIndex);
		},
		//protected
		_finish: function(inRowIndex){
			// summary:
			//	called when editing is completed to clean up editor
			// inRowIndex: int
			// grid row index
			dom.setSelectable(this.grid.domNode, false);
			this.cancelFormatNode();
		},
		//public
		apply: function(inRowIndex){
			// summary:
			//	apply edit from cell editor
			// inRowIndex: int
			// grid row index
			this.applyEdit(this.getValue(inRowIndex), inRowIndex);
			this._finish(inRowIndex);
		},
		cancel: function(inRowIndex){
			// summary:
			//	cancel cell edit
			// inRowIndex: int
			// grid row index
			this.cancelEdit(inRowIndex);
			this._finish(inRowIndex);
		}
	});
	BaseCell.markupFactory = function(node, cellDef){
		var formatter = lang.trim(domAttr.get(node, "formatter")||"");
		if(formatter){
			cellDef.formatter = lang.getObject(formatter)||formatter;
		}
		var get = lang.trim(domAttr.get(node, "get")||"");
		if(get){
			cellDef.get = lang.getObject(get);
		}
		var getBoolAttr = function(attr, cell, cellAttr){
			var value = lang.trim(domAttr.get(node, attr)||"");
			if(value){ cell[cellAttr||attr] = !(value.toLowerCase()=="false"); }
		};
		getBoolAttr("sortDesc", cellDef);
		getBoolAttr("editable", cellDef);
		getBoolAttr("alwaysEditing", cellDef);
		getBoolAttr("noresize", cellDef);
		getBoolAttr("draggable", cellDef);

		var value = lang.trim(domAttr.get(node, "loadingText")||domAttr.get(node, "defaultValue")||"");
		if(value){
			cellDef.defaultValue = value;
		}

		var getStrAttr = function(attr, cell, cellAttr){
			var value = lang.trim(domAttr.get(node, attr)||"")||undefined;
			if(value){ cell[cellAttr||attr] = value; }
		};
		getStrAttr("styles", cellDef);
		getStrAttr("headerStyles", cellDef);
		getStrAttr("cellStyles", cellDef);
		getStrAttr("classes", cellDef);
		getStrAttr("headerClasses", cellDef);
		getStrAttr("cellClasses", cellDef);
	};

	var Cell = declare("dojox.grid.cells.Cell", BaseCell, {
		// summary
		// grid cell that provides a standard text input box upon editing
		constructor: function(){
			this.keyFilter = this.keyFilter;
		},
		// keyFilter: RegExp
		//		optional regex for disallowing keypresses
		keyFilter: null,
		formatEditing: function(inDatum, inRowIndex){
			this.needFormatNode(inDatum, inRowIndex);
			return '<input class="dojoxGridInput" type="text" value="' + inDatum + '">';
		},
		formatNode: function(inNode, inDatum, inRowIndex){
			this.inherited(arguments);
			// FIXME: feels too specific for this interface
			this.registerOnBlur(inNode, inRowIndex);
		},
		doKey: function(e){
			if(this.keyFilter){
				var key = String.fromCharCode(e.charCode);
				if(key.search(this.keyFilter) == -1){
					event.stop(e);
				}
			}
		},
		_finish: function(inRowIndex){
			this.inherited(arguments);
			var n = this.getEditNode(inRowIndex);
			try{
				util.fire(n, "blur");
			}catch(e){}
		}
	});
	Cell.markupFactory = function(node, cellDef){
		BaseCell.markupFactory(node, cellDef);
		var keyFilter = lang.trim(domAttr.get(node, "keyFilter")||"");
		if(keyFilter){
			cellDef.keyFilter = new RegExp(keyFilter);
		}
	};

	var RowIndex = declare("dojox.grid.cells.RowIndex", Cell, {
		name: 'Row',

		postscript: function(){
			this.editable = false;
		},
		get: function(inRowIndex){
			return inRowIndex + 1;
		}
	});
	RowIndex.markupFactory = function(node, cellDef){
		Cell.markupFactory(node, cellDef);
	};

	var Select = declare("dojox.grid.cells.Select", Cell, {
		// summary:
		// grid cell that provides a standard select for editing

		// options: Array
		// 		text of each item
		options: null,

		// values: Array
		//		value for each item
		values: null,

		// returnIndex: Integer
		// 		editor returns only the index of the selected option and not the value
		returnIndex: -1,

		constructor: function(inCell){
			this.values = this.values || this.options;
		},
		formatEditing: function(inDatum, inRowIndex){
			this.needFormatNode(inDatum, inRowIndex);
			var h = [ '<select class="dojoxGridSelect">' ];
			for (var i=0, o, v; ((o=this.options[i]) !== undefined)&&((v=this.values[i]) !== undefined); i++){
				v = v.replace ? v.replace(/&/g, '&amp;').replace(/</g, '&lt;') : v;
				o = o.replace ? o.replace(/&/g, '&amp;').replace(/</g, '&lt;') : o;
				h.push("<option", (inDatum==v ? ' selected' : ''), ' value="' + v + '"', ">", o, "</option>");
			}
			h.push('</select>');
			return h.join('');
		},
		_defaultFormat: function(inValue, callArgs){
			var v = this.inherited(arguments);
			// when 'values' and 'options' both provided and there is no cutomized formatter,
			// then we use 'options' as label in order to be consistent
			if(!this.formatter && this.values && this.options){
				var i = array.indexOf(this.values, v);
				if(i >= 0){
					v = this.options[i];
				}
			}
			return v;
		},
		getValue: function(inRowIndex){
			var n = this.getEditNode(inRowIndex);
			if(n){
				var i = n.selectedIndex, o = n.options[i];
				return this.returnIndex > -1 ? i : o.value || o.innerHTML;
			}
		}
	});
	Select.markupFactory = function(node, cell){
		Cell.markupFactory(node, cell);
		var options = lang.trim(domAttr.get(node, "options")||"");
		if(options){
			var o = options.split(',');
			if(o[0] != options){
				cell.options = o;
			}
		}
		var values = lang.trim(domAttr.get(node, "values")||"");
		if(values){
			var v = values.split(',');
			if(v[0] != values){
				cell.values = v;
			}
		}
	};

	var AlwaysEdit = declare("dojox.grid.cells.AlwaysEdit", Cell, {
		// summary:
		// grid cell that is always in an editable state, regardless of grid editing state
		alwaysEditing: true,
		_formatNode: function(inDatum, inRowIndex){
			this.formatNode(this.getEditNode(inRowIndex), inDatum, inRowIndex);
		},
		applyStaticValue: function(inRowIndex){
			var e = this.grid.edit;
			e.applyCellEdit(this.getValue(inRowIndex), this, inRowIndex);
			e.start(this, inRowIndex, true);
		}
	});
	AlwaysEdit.markupFactory = function(node, cell){
		Cell.markupFactory(node, cell);
	};

	var Bool = declare("dojox.grid.cells.Bool", AlwaysEdit, {
		// summary:
		// grid cell that provides a standard checkbox that is always on for editing
		_valueProp: "checked",
		formatEditing: function(inDatum, inRowIndex){
			return '<input class="dojoxGridInput" type="checkbox"' + (inDatum ? ' checked="checked"' : '') + ' style="width: auto" />';
		},
		doclick: function(e){
			if(e.target.tagName == 'INPUT'){
				this.applyStaticValue(e.rowIndex);
			}
		}
	});
	Bool.markupFactory = function(node, cell){
		AlwaysEdit.markupFactory(node, cell);
	};

	return BaseCell;

});
},
'ibm/tivoli/navmodel/Utils':function(){
// wrapped by build app
define(["dijit","dojo","dojox"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Utils"); 

////////////////////////////////////
// Utils
////////////////////////////////////

ibm.tivoli.navmodel.Utils.isTIP = function() {
	//	summary:
	//		Returns true if environment is running in Tivoli Integrated Portal  
	
	// Test first for TIP 3.1
	var runningInTIP =  (typeof tip !== "undefined") && (tip != null) && (tip.getVersion != null); // 46154, 49895
	
	if (!runningInTIP) {           // 46154
		// test for TIP 2.2.0.x
	  	var themes = ["TIPLight", "TIPDark", "ISCTheme"];
	  	var themeCount = themes.length;
	  	var body = dojo.body();
	  	for (var index = 0;  index < themeCount;  ++index) {
	  		if (dojo.hasClass(body, themes[index])) {
	  			runningInTIP = true;         // 46154
	  		}
	  	}		
	} 
  	
  	return runningInTIP;     // 46154
};

ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion = function() {
	return 1.0;
};

ibm.tivoli.navmodel.Utils.getContentType = function(version, format) {
	// summary:
	//		Return the HttpHeader "Content-Type" that REST service is expecting
	//		;format=xxxx   can be appended to indicate the json format object
	var contentType = "application/vnd.ibm.com.tivolidis.json;version=" + version;	
	if (format) {
		contentType += ";format=" + format;
	}
	
	return contentType;
};

ibm.tivoli.navmodel.Utils.getContent = function() {		
	// summary:
	//		Construct a "content" object for sending an xdr request;
	//		IE will sometimes return http requests from cache without unique parms
	var content = {};
		
	// add date and random
	var now = new Date();		
	content.d = now.getTime();
	content.r = Math.random();
		
	return content;
};

ibm.tivoli.navmodel.Utils.mixinParams = function(content, dataset) {		
	// summary:
	//		Mixin the "param_" qualified arguments;  all parameters are 
	//		prefixed with "param_" so the rest service can use them to identify
	//		dataset cache entry
	var parameters = dataset.getArg("parameters");
	for (var parm in parameters) {
		if (parameters.hasOwnProperty(parm)) {
			content["param_" + parm] = parameters[parm];
		}
	}
};

ibm.tivoli.navmodel.Utils.mixinProperties = function(content, key, properties) {		
	// summary:
	//		Mixin the "properties" into the content as a comma delimited list
	if (properties && properties.length > 0) {
		content[key] = properties.join(",");
	}
};

ibm.tivoli.navmodel.Utils.getParamUrl = function(url, dataset) {		
	// summary:
	//		Tack on dataset version and param_* values to the url
	var questionMark = false;
		
	// add "param_" parms
	var parameters = dataset.getArg("parameters");
	for (var parm in parameters) {
		if (parameters.hasOwnProperty(parm)) {
			if (!questionMark) {
				questionMark = true;
				url += "?";
			}
			else {
				url += "&";
			}
			url += ("param_" + parm + "=" + parameters[parm]);				
		}
	}
		
	// IE will sometimes return http requests from cache without unique parms
	var now = new Date();
	if (!questionMark) {
		questionMark = true;
		url += "?d=" + now.getTime() + "&r=" + Math.random();
	}
	else {
		url += "&d=" + now.getTime() + "&r=" + Math.random();
	}
	
	return url;	
};

ibm.tivoli.navmodel.Utils.getException = function(error, ioArgs, funcName, trace, version) {		
	// summary:
	//		Returns Exception object contained in the dojo.xhr ioArgs
	var exception = null;
	var responseText = ioArgs.xhr.responseText;
	if (responseText) {
		try {
			var jsonEx = dojo.fromJson(responseText);
			if (jsonEx.msgId) {
				trace.error(funcName + " generated exception=" + responseText, "getException");				
				exception = new ibm.tivoli.navmodel.Exception({exception: jsonEx, version: version});
			}
			else {
				trace.error(funcName + " generated exception (non rest) = " + error, "getException");				
			}
		}
		catch(e) {
			trace.error(funcName + " generated exception (non rest) = " + error, "getException");
			exception = new ibm.tivoli.navmodel.Exception({exception: {msgText: error}, version: version});
		}
			
		trace.error("exception url=" + ioArgs.url, "getException");							
	}
	
	return exception;
};

ibm.tivoli.navmodel.Utils.NorthernHemisphere = {
	"GMT-10:00": "US/Aleutian",
  	"GMT-9:00":  "US/Alaska",
  	"GMT-8:00":  "US/Pacific",
  	"GMT-7:00":  "US/Mountain",
	"GMT-6:00":  "US/Central",
	"GMT-5:00":  "US/Eastern",
	"GMT-4:00":  "Canada/Atlantic",
	"GMT-3:30":  "Canada/Newfoundland",
	"GMT-3:00":  "America/Godthab",
    "GMT-2:00":  "America/Noronha",
    "GMT-1:00":  "Atlantic/Azores",
    "GMT":       "Europe/London",
    "GMT+1:00":  "Europe/Amsterdam",
    "GMT+2:00":  "Europe/Athens",
    "GMT+3:00":  "Europe/Moscow",
    "GMT+3:30":  "Asia/Tehran",
    "GMT+4:00":  "Asia/Baku",
    "GMT+5:00":  "Asia/Yekaterinburg",
    "GMT+6:00":  "Asia/Novosibirsk",
    "GMT+7:00":  "Asia/Krasnoyarsk",
    "GMT+8:00":  "Asia/Irkutsk",
    "GMT+9:00":  "Asia/Yakutsk",
    "GMT+10:00": "Asia/Vladivostok"
};

ibm.tivoli.navmodel.Utils.SouthernHemisphere = {
	"GMT-4:00":  "America/Asuncion",
	"GMT-3:00":  "America/Montevideo",
	"GMT+2:00":  "Africa/Windhoek",
	"GMT+9:30":  "Australia/Adelaide",
	"GMT+10:00": "Australia/Sydney",
	"GMT+12:00": "Pacific/Fiji"
};

ibm.tivoli.navmodel.Utils.NoDST = {  
	"GMT-11:00": "US/Samoa",
	"GMT-10:00": "US/Hawaii",
	"GMT-7:00":	 "US/Arizona",
	"GMT-6:00":  "Canada/Saskatchewan",
	"GMT-5:00":  "US/East-Indiana",
	"GMT-4:30":  "America/Caracas",
	"GMT-4:00":  "America/Manaus",
	"GMT-3:00":  "America/Buenos_Aires",
	"GMT-2:00":  "Atlantic/South_Georgia",
	"GMT-1:00":  "Atlantic/Cape_Verde",
	"GMT":       "Africa/Monrovia",
	"GMT+1:00":  "Africa/Algiers",
	"GMT+2:00":  "Africa/Harare",
	"GMT+3:00":  "Asia/Baghdad",
	"GMT+4:00":  "Asia/Dubai",
	"GMT+4:30":  "Asia/Kabul",
	"GMT+5:00":  "Asia/Karachi",
	"GMT+5:30":  "Asia/Kolkata",
	"GMT+5:45":  "Asia/Katmandu",
	"GMT+6:00":  "Asia/Dhaka",
	"GMT+6:30":  "Asia/Rangoon",
	"GMT+7:00":  "Asia/Bangkok",
	"GMT+8:00":  "Asia/Hong_Kong",
	"GMT+9:00":  "Asia/Tokyo",
	"GMT+9:30":  "Australia/Darwin",
	"GMT+10:00": "Australia/Brisbane",
	"GMT+11:00": "Asia/Magadan",
	"GMT+12:00": "Pacific/Wake",
	"GMT+13:00": "Pacific/Tongatapu"
};

// OlsonTimeZone name
ibm.tivoli.navmodel.Utils.timeZoneName = null;

ibm.tivoli.navmodel.Utils.setTimeZone = function(timeZoneName) {
	// summary:
	//		Set the timeZoneName that will be returned by getTimeZone()	
	ibm.tivoli.navmodel.Utils.timeZoneName = timeZoneName;
};

ibm.tivoli.navmodel.Utils.getTimeZone = function() {
	// summary:
	//		Calculate the users Olson timezone name
	
	// if one has been set; just return it
	if (ibm.tivoli.navmodel.Utils.timeZoneName) {
		return ibm.tivoli.navmodel.Utils.timeZoneName;
	}
	
	var janUTC = ibm.tivoli.navmodel.Utils.getUTCOffset(1, 1);
	var janGMT = ibm.tivoli.navmodel.Utils.getGMTOffset(1, 1);
	var julyUTC = ibm.tivoli.navmodel.Utils.getUTCOffset(7, 1);
	var julyGMT = ibm.tivoli.navmodel.Utils.getGMTOffset(7, 1);	
	var dst = (janUTC !== julyUTC);	
	var tzName = null;
	
	// DST
	if (dst) {
		if (janUTC < julyUTC) {
			tzName = ibm.tivoli.navmodel.Utils.NorthernHemisphere[janGMT];
		}
		else {
			tzName = ibm.tivoli.navmodel.Utils.SouthernHemisphere[julyGMT];
		}
	}
	// No DST
	else {
		tzName = ibm.tivoli.navmodel.Utils.NoDST[janGMT];
	}
	
	// can't find it in tables; so return GMT offset
	if (!tzName) {
		tzName = ibm.tivoli.navmodel.Utils.getGMTOffset();
	}
	
	//alert('jan=' + janGMT + '  july=' + julyGMT + ' dst=' + dst + ' tz=' + tzName);
		
	return tzName;
};

ibm.tivoli.navmodel.Utils.getUTCOffset = function(month, day) {
	// summary:
	//		Calculate the users current GMT offset
	var date = new Date();
	if (month) {
		date.setMonth(month);
	}
	if (day) {
		date.setDate(day);
	}
	
	return -(date.getTimezoneOffset());	
};

ibm.tivoli.navmodel.Utils.getGMTOffset = function(month, day) {
	// summary:
	//		Calculate the users current GMT offset
	var date = new Date();
	if (month) {
		date.setMonth(month);
	}
	if (day) {
		date.setDate(day);
	}
	
	var timezone = "";
	var minutesFromGMT = date.getTimezoneOffset();
	var extraMinutes = minutesFromGMT%60;
	var gmtHours = null;
	var gmtMinutes = "00";
		
	if (extraMinutes === 0) {
		gmtHours = -(minutesFromGMT/60);
	}
	else {
		gmtHours = -((minutesFromGMT-extraMinutes)/60);
		gmtMinutes = (Math.abs(extraMinutes)).toString();		
	}	
		
	timezone = "GMT";
	if (gmtHours !== 0) {
		if (gmtHours > 0) {
			timezone += "+";				
		}
		timezone += gmtHours;
		timezone += ":";
		timezone += gmtMinutes;
	}
		
	return timezone;
};

});

},
'ibm/usmi/menus/NextGenUIHelpMenu':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

/**
 * NextGenUIHelpMenu is a specialized menu for help links and extends from the
 * oneui menu
 */
define("ibm/usmi/menus/NextGenUIHelpMenu", [
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dijit/_WidgetBase",
	"dojo/dom-construct",
	"dijit/registry",
	"dojo/query",
	"dijit/MenuItem",
	"idx/oneui/Menu",
	"ibm/usmi/util/DataStore",
	"ibm/usmi/core/utils/_UsmiBrowserPersistence",
	"dojo/i18n!ibm/usmi/nls/frameworkResources",
	"ibm/usmi/menus/NextGenUIMenuUtils",
	"ibm/usmi/core/utils/Logger",
	"dojo/topic"
], function (dojo, declare, WidgetBase, domConstruct, registry, query, MenuItem, Menu, DataStore,
		_UsmiBrowserPersistence, MRI, NextGenUIMenuUtils, Logger, topic) {
	return declare([
		WidgetBase,
		Menu,
		_UsmiBrowserPersistence
	], {

		_helpMenu : null,
		_dataStore : null,
		_nextGenUIMenuUtils : new NextGenUIMenuUtils({}),
		_nextGenUIHelpMenuLogger : new Logger("ibm.usmi.menus", "NextGenUIHelpMenu"),
		_PERSISTENCE_HELP_MENU_ID : "ibm.usmi.menus.NextGenUIHelpMenu.contents",

		constructor : function (args) {
			dojo.safeMixin(this, args);
		},

		postCreate : function () {
			if (this._nextGenUIHelpMenuLogger.isFinest()) {
				this._nextGenUIHelpMenuLogger.entering("", "postCreate");
			}
			this.inherited(arguments);

			this._helpMenu = this.getData(this._PERSISTENCE_HELP_MENU_ID);
			if (this._nextGenUIHelpMenuLogger.isInfo()) {
				this._nextGenUIHelpMenuLogger.info(
						"Checked session storage for help menu, value is: " + this._helpMenu, "postCreate");
			}
			if (this._helpMenu) {
				if (this._nextGenUIHelpMenuLogger.isInfo()) {
					this._nextGenUIHelpMenuLogger.info("Posting topic that help menu is loaded", "postCreate");
				}
				topic.publish("nextgen-ui-helpmenu-loaded", null);
			} else {
				if (this._nextGenUIHelpMenuLogger.isInfo()) {
					this._nextGenUIHelpMenuLogger.info(
							"Help menu hasn't been stored yet, need to pull contents from server.", "postCreate");
				}
				this._dataStore = new DataStore({
					datasetId : 'HeaderMegaMenuCollection',
					homeParms : null,
					getItems : true,
					retrieveItemProperties : true,
					getTasks : false,
					initReadyCallback : dojo.hitch(this, this.initializeDataStores)
				});
			}
			if (this._nextGenUIHelpMenuLogger.isFinest()) {
				this._nextGenUIHelpMenuLogger.exiting("", "postCreate");
			}
		},

		initializeDataStores : function () {
			if (this._nextGenUIHelpMenuLogger.isInfo()) {
				this._nextGenUIHelpMenuLogger.info(
						"Help menu data retrieved from server. Converting CURI data into the help menu object",
						"initializeDataStores");
			}
			this._convertMenuCollectionToHelpMenuObject();
			if (this._nextGenUIHelpMenuLogger.isInfo()) {
				this._nextGenUIHelpMenuLogger.info("Saving help menu data to the session", "initializeDataStores");
			}
			this.setData(this._PERSISTENCE_HELP_MENU_ID, this._helpMenu);

			// after we process datastore items, and persist in storage clean up the datastore as it is no
			// no longer needed.
			this.destroyDataStore();
		},
		
		destroyDataStore : function () {
			// clean up the datastore
			if (this._dataStore) {
				this._dataStore.getDataset().removeEventListener(this);
				this._dataStore.destroy();
				this._dataStore = null;
			}
			
			this.inherited(arguments);
		},

		/**
		 * converts a collection into a help menu object for easy storage in
		 * session persistence
		 */
		_convertMenuCollectionToHelpMenuObject : function () {
			if (this._nextGenUIHelpMenuLogger.isFinest()) {
				this._nextGenUIHelpMenuLogger.entering("", "_convertMenuCollectionToHelpMenuObject");
			}
			var items, sortedItems, itemType, menuId, i;
			sortedItems = [];
			items = this._dataStore.getItems();
			this._helpMenu = [];
			for (i = 0; i < items.length; i++) {
				sortedItems[items[i].getProperty('ORDER_COLUMN').getValue()] = items[i];
			}

			dojo.forEach(sortedItems, dojo.hitch(this, function (entry, i) {
				itemType = entry.getProperty('TYPE_COLUMN').getValue();
				menuId = entry.getProperty('MENU_ID_COLUMN').getValue();
				if (itemType === 'menuItem') {
					if (menuId === 'HelpMenu') {
						this._helpMenu.push({
							"type" : itemType,
							"column" : entry.getProperty('COLUMN_COLUMN').getValue(),
							"title" : entry.getProperty('TITLE_COLUMN').getValue(),
							"task" : entry.getProperty('TASK_COLUMN').getValue(),
							"id" : menuId
						});
					}
				} else if (itemType === 'menuHeading') {
					if (menuId === 'HelpMenu') {
						this._helpMenu.push({
							"type" : itemType,
							"column" : entry.getProperty('COLUMN_COLUMN').getValue(),
							"title" : entry.getProperty('TITLE_COLUMN').getValue()
						});
					}
				}

			}));

			if (this._nextGenUIHelpMenuLogger.isInfo()) {
				this._nextGenUIHelpMenuLogger.info("Posting topic that help menu is loaded",
						"_convertMenuCollectionToHelpMenuObject");
			}
			topic.publish("nextgen-ui-helpmenu-loaded", null);
			if (this._nextGenUIHelpMenuLogger.isFinest()) {
				this._nextGenUIHelpMenuLogger.exiting("", "_convertMenuCollectionToHelpMenuObject");
			}
		},

		/**
		 * creates the help menu from the help menu object
		 */
		createHelpMenu : function () {
			if (this._nextGenUIHelpMenuLogger.isFinest()) {
				this._nextGenUIHelpMenuLogger.entering("", "_createHelpMenu");
			}
			var i;
			if (this._helpMenu) {
				for (i = 0; i < this._helpMenu.length; i++) {
					if (this._helpMenu[i].type === 'menuItem') {
						this.addChild(this._nextGenUIMenuUtils.createMenuItem(this._helpMenu[i].title,
								this._helpMenu[i].column, this._helpMenu[i].task, MRI));
					} else if (this._helpMenu[i].type === 'menuHeading') {
						this.addChild(this._nextGenUIMenuUtils.createMenuHeading(this._helpMenu[i].title,
								this._helpMenu[i].column, MRI));
					}
				}
			} else {
				if (this._nextGenUIHelpMenuLogger.isWarn()) {
					this._nextGenUIHelpMenuLogger.warn("Help menu object is null", "_createHelpMenu");
				}
			}
			if (this._nextGenUIHelpMenuLogger.isFinest()) {
				this._nextGenUIHelpMenuLogger.exiting("", "_createHelpMenu");
			}
		}
	});

});
},
'gridx/modules/Printer':function(){
define([
	"../core/_Module",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/Deferred",
	"dojo/_base/xhr",
	"dojo/_base/array",
	"dojo/_base/sniff",
	"dojo/_base/window"
], function(_Module, declare, lang, Deferred, xhr, array, sniff, win){

	/*=====
	declare('__PrinterArgs', __ExporterArgs, {
		// style: String
		//		The CSS string for the printed document
		style: '',

		// styleSrc: URL String
		//		The CSS file url used for the printed document
		styleSrc: '',

		// title: String
		//		The content of the <title> element in the <head> of the printed document
		title: '',

		// description: String
		//		Any HTML content that will be put before the grid in the printed document.
		description: ''

		// customHead: String
		//		Any HTML <head> content that will be put in the <head> of the printed document.
		customHead: ''
	});
	=====*/

	var printFrame,
		hitch = lang.hitch;

	function loadStyleFiles(src){
		var d = new Deferred(),
			loaded = hitch(d, d.callback);
		if(src){
			xhr.get({
				url: src
			}).then(loaded, function(){
				console.warn('Failed to load resource: ', src);
				loaded('');
			});
		}else{
			loaded('');
		}
		return d;
	}

	function print(str){
		var w, fillDoc = function(w){
			var doc = w.document;
			doc.open();
			doc.write(str);
			doc.close();
		};
		if(!window.print){
			console.warn('Print function is not available');
			return;
		}else if(sniff('chrome') || sniff('opera')){
			//referred from dijit._editor.plugins.Print._print()
			//In opera and chrome the iframe.contentWindow.print
			//will also print the outside window. So we must create a
			//stand-alone new window.
			w = window.open("javascript:''", "",
				"status=0,menubar=0,location=0,toolbar=0,width=1,height=1,resizable=0,scrollbars=0");
			fillDoc(w);
			w.print();
			//Opera will stop at this point, showing the popping-out window.
			//If the user closes the window, the following codes will not execute.
			//If the user returns focus to the main window, the print function
			// is executed, but still a no-op.
			w.close();
		}else{
			if(!printFrame){
				//create an iframe to store the grid data.
				printFrame = win.doc.createElement('iframe');
				printFrame.frameBorder = 0;
				var s = printFrame.style;
				s.position = 'absolute';
				s.width = s.height = '1px';
				s.right = s.bottom = 0;
				s.border = 'none';
				s.overflow = 'hidden';
				if(!sniff('ie')){
					s.visibility = 'hidden';
				}
				win.body().appendChild(printFrame);
			}
			w = printFrame.contentWindow;
			fillDoc(w);
			//IE requires the frame to be focused for print to work, and it's harmless for FF.
			w.focus();
			w.print();
		}
	}
	
	return declare(/*===== "gridx.modules.Printer", =====*/_Module, {
		// summary:
		//		This module provides the API to print grid contents or provide print preview
		// description:
		//		This module relies on the ExportTable module to produce HTML table from grid.
		name: 'printer',

		forced: ['exportTable'],
		
		getAPIPath: function(){
			// tags:
			//		protected extension
			return {
				printer: this
			};
		},
	
		//Public-----------------------------------------------------------------
		print: function(args){
			// summary:
			//		Print grid contents.
			// args: Object
			//		Please refer to `grid.printer.__PrinterArgs`
			// returns:
			//		A deferred object indicating when the export process is completed.
			return this.toHTML(args).then(print);	//dojo.Deferred
		},

		toHTML: function(args){
			// summary
			//		Export to printable html, used for preview
			// args: Object
			//		Please refer to `grid.printer.__PrinterArgs`
			// returns:
			//		A deferred object indicating when the export process is completed.
			var t = this, d = new Deferred();
			loadStyleFiles(args.styleSrc).then(function(styleSrc){
				t.grid.exporter.toTable(args).then(function(str){
					d.callback(t._wrap(args, styleSrc, str));
				}, hitch(d, d.errback), hitch(d, d.progress));
			});
			return d;	//dojo.Deferred
		},
		
		//private----------------------------------------------------------------------------
		_wrap: function(args, styleSrc, bodyContent){
			return [
				'<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd"><html ',
					this.grid.isLeftToRight() ? '' : 'dir="rtl"',
				'><head><title>',
					args.title || '',
				'</title><meta http-equiv="Content-Type" content="text/html; charset=utf-8"></meta><style type="text/css">',
					styleSrc,
				'</style><style type="text/css">',
					args.style || '',
				'</style>',
					args.customHead || '',
				'</head><body>', 
					args.description || '', 
					bodyContent,
				'</body></html>'
			].join('');
		}
	});
});

},
'ibm/usmi/menus/Menu':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/menus/Menu", [
	"require",
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dijit/_WidgetBase",
	"dojo/_base/lang",
	"idx/oneui/MenuBar",
	"ibm/usmi/statuspod/AlertStatusPod",
	"ibm/usmi/statuspod/JobStatusPod"
], function (require, dojo, declare, WidgetBase, lang, MenuBar, AlertStatusPod,
		JobStatusPod) {
	return declare([
		WidgetBase,
		MenuBar
	], {

		showProblemStatusPod : false,

		showJobStatusPod : false,

		baseClass : "menus.Menu",

		constructor : function (args) {
			dojo.safeMixin(this, args);

		},

		postCreate : function () {
			this.inherited(arguments);
			if (this.showProblemStatusPod) {
				var problemMenuBarItem = new AlertStatusPod({
					id : "alertStatusPod"
				});
				this.addChild(problemMenuBarItem);
			}
			if (this.showJobStatusPod) {
				var jobMenuBarItem = new JobStatusPod({
					id : "jobStatusPod"
				});
				this.addChild(jobMenuBarItem);
			}
		}
	});
});

},
'idx/oneui/MenuHeading':function(){
/*
 * Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

define(["dojo/_base/declare", 
		"dijit/MenuSeparator",
		"dojo/text!../oneui/templates/MenuHeading.html"],
		function(declare,
				 MenuSeparator,
				 template){
	/**
	 * Creates a new idx.oneui.MenuHeading
	 * @name idx.oneui.MenuHeading
	 * @class The MenuHeading widget provides a non-selectable, full-width menu entry
	 * suitable for labelling groups of menu items.
	 * @augments dijit.MenuSeparator
	 * @example &lt;div data-dojo-type="idx.oneui.Menu"&gt;
  &lt;div data-dojo-type="idx.oneui.MenuHeading"
       data-dojo-props="column:'0',<span class="highlitCode">label:'Column #0'</span>"&gt;
  &lt;/div&gt;
    ...
&lt;/div&gt;
	 */
	return declare("idx.oneui.MenuHeading", [MenuSeparator], 
	/** @lends idx.oneui.MenuHeading.prototype */
	{
		/**
		 * The text and markup to display in the item.
		 * @type string
		 */		
		label: '',
		
		/**
		 * Standard Dojo setter config for handling the 'label' property via calls to 
		 * set().
		 * @constant
		 * @type Object
		 */
		_setLabelAttr: { node: "containerNode", type: "innerHTML" },

		/**
	 	 * The template HTML for the widget.
		 * @constant
		 * @type string
		 * @private
		 * @default Loaded from idx/oneui/templates/MenuHeading.html.
		 */
		templateString: template

	});
});

},
'url:gridx/templates/GotoPagePane.html':"<div class='gridxGotoPage'\n\t><table><tbody\n\t\t><tr><td id='${id}-pageInputLabel' class='gridxGotoPageMainMsg'>\n\t\t${gotoDialogMainMsg}\n\t\t</td></tr\n\t\t><tr><td class='gridxGotoPageInput'\n\t\t\t><input data-dojo-type='${numberTextBoxClass}'\n\t\t\t\tdata-dojo-props='\"aria-labelledby\": \"${id}-pageInputLabel\"'\n\t\t\t\tclass='gridxGotoPageInputBox'\n\t\t\t\tdata-dojo-attach-point='pageInputBox'\n\t\t\t\tdata-dojo-attach-event='onKeyUp: _updateStatus, onKeyDown: _onKeyDown'></input\n\t\t\t><span\n\t\t\t\tclass='gridxPageCountMsg'\n\t\t\t\tdata-dojo-attach-point='pageCountMsgNode'></span\n\t\t></td></tr\n\t\t><tr><td class='gridxGotoPageBtns'\n\t\t\t><button data-dojo-type='${buttonClass}' \n\t\t\t\tdata-dojo-attach-point='okBtn'\n\t\t\t\tdata-dojo-attach-event='onClick: _onOK'>\n\t\t\t\t${gotoDialogOKBtn}\n\t\t\t</button\n\t\t\t><button data-dojo-type='${buttonClass}'\n\t\t\t\tdata-dojo-attach-event='onClick: _onCancel'>\n\t\t\t\t${gotoDialogCancelBtn}\n\t\t\t</button\n\t\t></td></tr\n\t></tbody></table\n></div>\n",
'gridx/modules/filter/FilterPane':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/array",
	"dojo/dom-construct",
	"dojo/dom-class",
	"dojo/string",
	"dojo/query",
	"dijit/registry",
	"dojox/html/ellipsis",
	"dojox/html/metrics",
	"./DistinctComboBoxMenu",
	"./Filter",
	"dojo/text!../../templates/FilterPane.html",
	"dojo/i18n!../../nls/FilterBar",
	"dijit/layout/ContentPane",
	"dijit/form/Select",
	"dijit/form/TextBox",
	"dijit/form/DateTextBox",
	"dijit/form/TimeTextBox",
	"dijit/form/RadioButton",
	"dijit/form/NumberTextBox",
	"dijit/form/ComboBox"
], function(declare, lang, array, dom, css, string, query, registry, ellipsis, metrics, DistinctComboBoxMenu, Filter, template, i18n){
	var ANY_COLUMN_VALUE = '_gridx_any_column_value_';
	
	function isAnyColumn(colid){
		return colid == ANY_COLUMN_VALUE;
	}
	return declare([dijit.layout.ContentPane], {
		//content: template,
		sltColumn: null,
		grid: null,
		title: i18n.defaultRuleTitle,
		//TWL #55300: this is a temporary fix until dojo team provides a flag
		autoComplete: true,
		postCreate: function(){
			this.inherited(arguments);
			this.i18n = i18n;
			this.set('content', string.substitute(template, this));
			this._initFields();
			this._initSltCol();
			this.connect(this.sltColumn, 'onChange', '_onColumnChange');
			this.connect(this.sltCondition, 'onChange', '_onConditionChange');
			this.comboText.dropDownClass = DistinctComboBoxMenu;
			this._onConditionChange();//In the latest dijit, onChange event is no longer fired after creation
		},
	
		getData: function(){
			// summary:
			//	Get the filter defined by this filter pane.
			var value = this._getValue(), 
				colId = this.sltColumn.get('value'),
				condition = this.sltCondition.get('value');
			
			if(condition === 'isEmpty' || (value !== null && (condition !== 'range' || (value.start && value.end)))){
				return {
					colId: isAnyColumn(colId) ? '' : colId,
					condition: condition,
					value: condition === 'isEmpty' ? '' : value,
					type: this._getType()
				};
			}else{
				return null;
			}
		},
		setData: function(data){
			// summary:
			//	Set the data of the pane to restore UI.
			if(data === null){return;}
			this.sltColumn.set('value', data.colId, null);
			this._onColumnChange();
			var _this = this;
			
			window.setTimeout(function(){
				_this.sltCondition.set('value', data.condition, null);
				_this._onConditionChange();
				_this._setValue(data.value);
			}, 10);
		},
		close: function(){
			var ac = this._getContainer();
			if(ac.getChildren().length === 4){
				//while there's less than 4 rules, no scroll bar
				ac._contentBox.w += metrics.getScrollbar().w;
			}
			
			if(this === ac.selectedChildWidget){
				//select previous pane if this is current, consistent with EDG filter.
				var i = array.indexOf(ac.getChildren(), this);
				if(i > 0){ac.selectChild(ac.getChildren()[i-1]);}
			}
			
			ac.removeChild(this);
			css.toggle(ac.domNode, 'gridxFilterSingleRule', ac.getChildren().length === 1);
			this.grid.filterBar._filterDialog._updateAccordionContainerHeight();
		},
		onChange: function(){
			// summary:
			//	event: fired when column, condition or value is changed
		},
		_getContainer: function(){
			return registry.byNode(this.domNode.parentNode.parentNode.parentNode);
		},
		_initFields: function(){
			this.sltColumn = registry.byNode(query('li>table', this.domNode)[0]);
			this.sltCondition = registry.byNode(query('li>table', this.domNode)[1]);
			var fields = this._fields = [
				this.tbSingle = registry.byNode(query('.gridxFilterPaneTextWrapper > .dijitTextBox', this.domNode)[0]),
				this.tbNumber = registry.byNode(query('.gridxFilterPaneNumberWrapper > .dijitTextBox', this.domNode)[0]),
				this.comboText = registry.byNode(query('.gridxFilterPaneComboWrapper > .dijitComboBox', this.domNode)[0]),
				this.sltSingle = registry.byNode(query('.gridxFilterPaneSelectWrapper > .dijitSelect', this.domNode)[0]),
				this.dtbSingle = registry.byNode(query('.gridxFilterPaneDateWrapper > .dijitDateTextBox', this.domNode)[0]),
				this.dtbStart = registry.byNode(query('.gridxFilterPaneDateRangeWrapper > .dijitDateTextBox', this.domNode)[0]),
				this.dtbEnd = registry.byNode(query('.gridxFilterPaneDateRangeWrapper > .dijitDateTextBox', this.domNode)[1]),
				this.ttbSingle = registry.byNode(query('.gridxFilterPaneTimeWrapper > .dijitTimeTextBox', this.domNode)[0]),
				this.ttbStart = registry.byNode(query('.gridxFilterPaneTimeRangeWrapper > .dijitTimeTextBox', this.domNode)[0]),
				this.ttbEnd = registry.byNode(query('.gridxFilterPaneTimeRangeWrapper > .dijitTimeTextBox', this.domNode)[1]),
				this.rbTrue = registry.byNode(query('.gridxFilterPaneRadioWrapper .dijitRadio', this.domNode)[0]),
				this.rbFalse = registry.byNode(query('.gridxFilterPaneRadioWrapper .dijitRadio', this.domNode)[1])
			];
			
			this.rbTrue.domNode.nextSibling.htmlFor = this.rbTrue.id;
			this.rbFalse.domNode.nextSibling.htmlFor = this.rbFalse.id;
			var name = 'rb_name_' + Math.random();
			this.rbTrue.set('name', name);
			this.rbFalse.set('name', name);
			
			array.forEach(fields, function(field){
				this.connect(field, 'onChange', '_onValueChange');
			}, this);
		},
		_initSltCol: function(){
			var colOpts = [{label: i18n.anyColumnOption, value: ANY_COLUMN_VALUE}],
				fb = this.grid.filterBar, 
				sltCol = this.sltColumn;
			array.forEach(this.grid.columns(), function(col){
				if(!col.isFilterable())return;
				colOpts.push({value: col.id, label: col.name()});
			}, this);
			sltCol.addOption(colOpts);
		},
		_initCloseButton: function(){
			// summary:
			//	Add a close button to the accordion pane.
			//  Must be called after adding to an accordion container.
			var btnWidget = this._buttonWidget;
			var closeButton = dom.create('span', {
				className: 'gridxFilterPaneCloseButton',
				innerHTML: '<img src="' + this._blankGif + '"/>',
				tabIndex: 0,
				title: 'Close'
			}, btnWidget.domNode, 'last');
			this.connect(closeButton, 'onclick', 'close');
			css.add(btnWidget.titleTextNode, 'dojoxEllipsis');
		},
		
		_onColumnChange: function(){
			var colId = this.sltColumn.get('value');
			var opt = this.grid.filterBar._getConditionOptions(isAnyColumn(colId) ? '' : colId);
			var slt = this.sltCondition;
			if(slt.options && slt.options.length){slt.removeOption(slt.options);}
			slt.addOption(lang.clone(opt));
			this._updateTitle();
			this._updateValueField();
			this.onChange();
		},
		_onConditionChange: function(){
			this._updateValueField();
			this._updateTitle();
			this.onChange();
		},
		_onValueChange: function(){
			this._updateTitle();
			this.onChange();
		},
		_getDataType: function(){
			// summary:
			//		Get current column data type
			var colid = this.sltColumn.get('value');
			var dataType = 'string';
			if(!isAnyColumn(colid)){
				dataType = this.grid.column(colid).dataType();
			}
			return dataType;
		},
		_getType: function(){
			// summary:
			//	Get current filter type, determined by data type and condition.
			var mapping = {'string': 'Text', number: 'Number', date: 'Date', time: 'Time', 'boolean': 'Radio'};
			var type = mapping[this._getDataType()];
			if('range' === this.sltCondition.get('value')){type += 'Range';} ;
			return type;
		},
		_updateTitle: function(){
			if(!this._buttonWidget){return;}
			var title, value = this._getValue(), 
				type = this._getType(), condition = this.sltCondition.get('value'),
				txtNode = this._buttonWidget.titleTextNode;
			
			if(value && (condition !== 'range' || (value.start && value.end))){
				title = this.sltColumn.get('displayedValue') + ' ' + this.grid.filterBar._getRuleString(condition, value, type);
			}else{
				var ruleNumber = array.indexOf(this._getContainer().getChildren(), this) + 1;
				title = string.substitute(this.i18n.ruleTitleTemplate, {ruleNumber: ruleNumber});
			}
			txtNode.innerHTML = title;
			txtNode.title = title.replace(/<\/?span[^>]*>/g, '').replace('&nbsp;', ' ');
		},
		_needComboBox: function(){
			// summary:
			//	Whether current state needs a combo box for string input, may rewrite to support virtual column
			var colId = this.sltColumn.get('value');
			//TWL #55300: this is a temporary fix until dojo team provides a flag
			return this.autoComplete && this._getType() === 'Text' && !isAnyColumn(colId) && this.grid._columnsById[colId].field;
		},
		_updateValueField: function(){
			// summary:
			//	Update the UI for field to show/hide fields.
			var type = this._getType(), colId = this.sltColumn.get('value');
			var combo = this._needComboBox();
			
			array.forEach(['Text','Combo', 'Date', 'Number', 'DateRange', 'Time', 'TimeRange', 'Select', 'Radio'], function(k){
				css.remove(this.domNode, 'gridxFilterPane' + k);
			}, this);
			
			css.add(this.domNode, 'gridxFilterPane' + (combo ? 'Combo' : type));
			var disabled = this.sltCondition.get('value') === 'isEmpty';
			array.forEach(this._fields, function(f){f.set('disabled', disabled)});
			
			if(combo){
				if(!this._dummyCombo){
					//HACK: mixin query, get, etc methods to store, remove from 2.0.
					this._dummyCombo = new dijit.form.ComboBox({store: this.grid.store});
				}
				//init combobox
				var col = this.grid._columnsById[colId];
				lang.mixin(this.comboText, {
					store: this.grid.store,
					searchAttr: col.field,
					fetchProperties: {sort:[{attribute: col.field, descending: false}]}
				});
			}
		},
		_getValue: function(){
			// summary:
			//		Get current filter value
			var type = this._getType(), combo = this._needComboBox();
			switch(type){
				case 'Text':
					return (combo ? this.comboText : this.tbSingle).get('value') || null;
				case 'Number':
					return isNaN(this.tbNumber.get('value')) ? null : this.tbNumber.get('value');
				case 'Select':
					return this.sltSingle.get('value') || null;
				case 'Date':
					return this.dtbSingle.get('value') || null;
				case 'DateRange':
					return {start: this.dtbStart.get('value'), end: this.dtbEnd.get('value')};
				case 'Time':
					return this.ttbSingle.get('value') || null;
				case 'TimeRange':
					return {start: this.ttbStart.get('value'), end: this.ttbEnd.get('value')};
				case 'Radio':
					return !!this.rbTrue.get('checked');
				default:
					return null;
			}
		},
		_setValue: function(value){
			if(!value){return;}
			var type = this._getType(), combo = this._needComboBox();
			switch(type){
				case 'Text':
					(combo ? this.comboText : this.tbSingle).set('value', value);
					break;
				case 'Number':
					this.tbNumber.set('value', value);
					break;
				case 'Select':
					this.sltSingle.set('value', value);
					break;
				case 'Date':
					this.dtbSingle.set('value', value);
					break;
				case 'DateRange':
					this.dtbStart.set('value', value.start);
					this.dtbEnd.set('value', value.end);
					break;
				case 'Time':
					this.ttbSingle.set('value', value);
					break;
				case 'TimeRange':
					this.ttbStart.set('value', value.start);
					this.ttbEnd.set('value', value.end);
					break;
				case 'Radio':
					this.rbTrue.set('checked', true);
					break;
			}
		},
		
		uninitialize: function(){
			this.inherited(arguments);
			if(this._dummyCombo){this._dummyCombo.destroyRecursive();}
		}
	});
});

},
'ibm/usmi/util/DefaultUserInterfaceTask':function(){
// wrapped by build app
define("ibm/usmi/util/DefaultUserInterfaceTask", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/Trace,dojox/data/dom,ibm/usmi/util/DefaultMessageHandler"], function(dijit,dojo,dojox){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

dojo.provide("ibm.usmi.util.DefaultUserInterfaceTask");

dojo.require("ibm.tivoli.navmodel.Trace");
dojo.require("dojox.data.dom");
dojo.require("ibm.usmi.util.DefaultMessageHandler");

ibm.usmi.util.DefaultUserInterfaceTask._trace = new ibm.tivoli.navmodel.Trace("ibm.usmi.util.DefaultUserInterfaceTask");

dojo.declare("ibm.usmi.util.DefaultUserInterfaceTask", null, {

    // private members
    _launchParms: null,
    _menuItem: null,
    _context: null,
    _trace : ibm.usmi.util.DefaultUserInterfaceTask._trace,

    constructor : function() {
        // summary:
        //		Coming soon
        // description:
        //		Coming soon

        // Register for a URL launch task event
        dojo.subscribe("DefaultUserInterfaceTask", this, "_handleUserInterfaceTask");

    },


    _handleUserInterfaceTask : function(launchParms, menuItem, context) {
        // summary:
        //		Coming soon
        // description:
        //		Coming soon
        
        this._launchParms = launchParms;
        this._menuItem = menuItem;
        this._context = context;

        var runServerTask = function(tasks){
            var task = null;
        
            task = this.fetchTask(tasks);
            
            // Call execute on the server task
            task.execute({onError: dojo.hitch(this, this._handleExecuteOnServerError),
                          selection: this.getSelection(context)});
        };
        

        // Process the user interface task
        // Check launchParms to decide how to handle
        if (launchParms) {
            if (launchParms.launchType === "UserInterfaceTask") {
                // Get the task and execute it
                context.dataset.getTasks({
                onComplete: dojo.hitch(this, runServerTask),
                onError: dojo.hitch(this, this.getTasksError),
                taskCategory: "menu",
                selection: this.getSelection(context)});
            }
            else {
                alert("launchParms are not valid");
                // TODO log error and put up message?
            }
        }
        else {
            alert("undefined launch parms from server task");
            // TODO log error and put up message?
        }
    },


    fetchTask: function(tasks) {
    
        //Want to loop through tasks looking for match to menuItem actionID
        if (tasks) {
            for (var i = 0; i < tasks.length; i++) {
                if (tasks[i].getId() === this._menuItem.actionID) {
                    return tasks[i];
                }
                else if (tasks[i].getChildTasks()) {
                    var subTasks = tasks[i].getChildTasks();
                    if (subTasks.length > 0) {
                        var task = this.fetchTask(subTasks);
                        if (task && (task.getId() === this._menuItem.actionID)) {
                            return task;
                        }
                    }
                }
            }
        }
    },


    getSelection: function(context) {
	    var selectedThings = context.selection; //this._args.context.selection;
	    var currentSelection = null;
	    if (selectedThings) {
                currentSelection = new ibm.tivoli.navmodel.Selection({version: context.dataset.getVersion()}); //this.getDataSet().getVersion()});
	        currentSelection.addItems(selectedThings.getItemIds());
	        currentSelection.addLinks(selectedThings.getLinkIds());
	        currentSelection.addRelationships(selectedThings.getRelationshipIds());
	    }
	    return currentSelection;
    },


    getTasksError: function(error, request){
        //There was a problem getting the tasks from the dataset
         var msgHandler = new ibm.usmi.util.DefaultMessageHandler({});
         msgHandler.displayException(exception);
s    },
	
	
    _handleExecuteOnServerError: function(error, exception, ioArgs) {
        // summary:
        //		Coming soon
        // description:
        //		Coming soon
        // error:
        //		
        // exception:
        //		
        // ioArgs:
        //		

        if (this._trace.isError()) {
            this._trace.error(">>> error executing server task = " + error, "_handleExecuteOnServerError");
        }
		
        //There was a problem executing the task on the server
        var msgHandler = new ibm.usmi.util.DefaultMessageHandler({});
        msgHandler.displayException(exception);
    },
});

});

},
'gridx/modules/pagination/Pagination':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/array",
	"../../core/_Module"
], function(declare, array, _Module){

	return declare(/*===== "gridx.modules.pagination.Pagination", =====*/_Module, {
		// summary:
		//		This module provides (logical) pagination functionality for grid.
		// description:
		//		This module does not include any UI buttons for pagination, so that various
		//		kinds of pagination UI implementations can benifit from this module.

		// [Module Dependency Management] --------------------------------------------
		name: 'pagination',	

		forced: ['body'],

		// [Module API Management] ---------------------------------------------------
		getAPIPath: function(){
			return {
				pagination: this
			};
		},

		rowMixin: {
			getPage: function(){
				// summary:
				//		Get the page index this row belongs to.
				return this.grid.pagination.pageOfIndex(this.index());
			},

			indexInPage: function(){
				// summary:
				//		Get the index of this row in its page.
				return this.grid.pagination.indexInPage(this.index());
			}
		},

		// [Module Lifetime Management] -----------------------------------------------
		preload: function(){
			// tags:
			//		protected extension
			this.grid.body.autoChangeSize = false;
		},

		load: function(){
			// tags:
			//		protected extension
			var t = this;
			t._pageSize = t.arg('initialPageSize') || t._pageSize;
			t._page = t.arg('initialPage', t._page, function(arg){
				return arg >= 0;
			});
			t.model.when({}, function(){
				t._updateBody(1);
				t.connect(t.model, 'onSizeChange', '_onSizeChange');
				t.loaded.callback();	
			});
		},

		// [Public API] --------------------------------------------------------

		// initialPageSize: Integer
		//		Specify the page size (row count per page) when the grid is created.
		//initialPageSize: 10,

		// initialPage: Integer
		//		Specify which page the grid should show when it is created.
		//initialPage: 0,

		// GET functions
		pageSize: function(){
			// summary:
			//		Get current page size
			// returns:
			//		The current page size
			var s = this._pageSize;
			return s > 0 ? s : this.model.size();	//Integer
		},

		isAll: function(){
			// summary:
			//		Check if the grid is currently showing all rows (page size set to 0).
			// returns:
			//		Whether the grid is showing all rows.
			return this._pageSize === 0;	//Boolean
		},

		pageCount: function(){
			// summary:
			//		Get the current count of pages.
			// returns:
			//		The current count of pages.
			return this.isAll() ? 1 : Math.max(Math.ceil(this.model.size() / this.pageSize()), 1);	//Integer
		},

		currentPage: function(){
			// summary:
			//		Get the index of current page.
			// returns:
			//		The index of current page.
			return this._page;	//Integer
		},

		firstIndexInPage: function(page){
			// summary:
			//		Get the index of the first row in the given page.
			// page: Integer
			//		The index of a page.
			// returns:
			//		The index of the first row in the page. If page is not valid, return -1.
			if(!page && page !== 0){
				page = this._page;
			}else if(!(page >= 0)){
				return -1;	//Integer
			}
			var index = page * this.pageSize();
			return index < this.model.size() ? index : -1;	//Integer
		},

		lastIndexInPage: function(page){
			// summary:
			//		Get the index of the last row in the given page.
			// page: Integer
			//		The index of a page
			// returns:
			//		The index of the last row in the given page.
			var t = this,
				firstIndex = t.firstIndexInPage(page);
			if(firstIndex >= 0){
				var lastIndex = firstIndex + t.pageSize() - 1,
					size = t.model.size();
				return lastIndex < size ? lastIndex : size - 1;	//Integer
			}
			return -1;	//Integer
		},

		pageOfIndex: function(index){
			// summary:
			//		Get the index of the page that the given row is in.
			// index: Integer
			//		The row index
			// returns:
			//		The page index
			return this.isAll() ? 0 : Math.floor(index / this.pageSize());	//Integer
		},

		indexInPage: function(index){
			// summary:
			//		Get the row index in page by overall row index
			// index: Integer
			//		The row index
			// returns:
			//		The row index in page
			return this.isAll() ? index : index % this.pageSize();	//Integer
		},

		filterIndexesInPage: function(indexes, page){
			// summary:
			//		Filter out the indexes that are in the given page.
			// indexes: Integer[]
			//		An array of row indexes.
			// page: Integer
			//		A page index
			// returns:
			//		A subset of indexes that appear in the given page.
			var first = this.firstIndexInPage(page),
				end = this.lastIndexInPage(page);
			return first < 0 ? [] : array.filter(indexes, function(index){	//Integer[]
				return index >= first && index <= end;
			});
		},

		//SET functions
		gotoPage: function(page){
			// summary:
			//		Set the current page
			// page: Integer
			//		A page index
			var t = this, oldPage = t._page;
			if(page != oldPage && t.firstIndexInPage(page) >= 0){
				t._page = page;
				t._updateBody();
				t.onSwitchPage(page, oldPage);
			}
		},

		setPageSize: function(size){
			// summary:
			//		Set page size (count of rows in one page)
			// size: Integer
			//		The new page size 
			var t = this, oldSize = t._pageSize;
			if(size != oldSize && size >= 0){
				var index = t.firstIndexInPage(),
					oldPage = -1;
				t._pageSize = size;
				if(t._page >= t.pageCount()){
					oldPage = t._page;
					t._page = t.pageOfIndex(index);
				}
				t._updateBody();
				t.onChangePageSize(size, oldSize);
				if(oldPage >= 0){
					t.onSwitchPage(t._page, oldPage);
				}
			}
		},

		// [Events] ----------------------------------------------------------------
		onSwitchPage: function(/*currentPage, originalPage*/){
			// summary:
			//		Fired when switched to another page.
			// tags:
			//		callback
		},

		onChangePageSize: function(/*currentSize, originalSize*/){
			// summary:
			//		Fired when the page size is changed
			// tags:
			//		callback
		},
		
		// [Private] -------------------------------------------------------
		_page: 0,

		_pageSize: 10,

		_updateBody: function(noRefresh){
			var t = this,
				bd = t.grid.body,
				size = t.model.size(),
				count = t.pageSize(),
				start = t.firstIndexInPage();
			if(size === 0 || start < 0){
				start = 0;
				count = 0;
			}else if(size - start < count){
				count = size - start;
			}
			bd.updateRootRange(start, count);
			if(!noRefresh){
				bd.refresh();
			}
		},

		_onSizeChange: function(size){
			var t = this;
			if(size === 0){
				t._page = 0;
				t.grid.body.updateRootRange(0, 0);
			}else{
				var first = t.firstIndexInPage();
				if(first < 0){
					if(t._page !== 0){
						var oldPage = t._page;
						t._page = 0;
						t.onSwitchPage(0, oldPage);
					}
				}			
				t._updateBody();
			}
		}
	});	
});

},
'ibm/usmi/statuspod/AlertStatusTip':function(){
require({cache:{
'url:ibm/usmi/statuspod/templates/alertStatusTip.html':"<div data-dojo-attach-point=\"tipContent\" class=\"statusPodTip\">\r\n<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n</div>",
'url:ibm/usmi/statuspod/templates/alertStatusTipTableRow.html':"<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n\t\t<tr>\r\n\t\t\t<td style=\"${padding}\" colspan=\"2\">\r\n\t\t\t\t<img src=\"${alertSev}\"/> \r\n\t\t\t\t<span class=\"statusPodTipAlertText\" >${alertText}\r\n\t\t\t\t</span>\r\n\t\t\t</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<td>\r\n\t\t\t\t<span class=\"statusPodTipAlert\">${StatusTipCategory} ${alertCat}</span>\r\n\t\t\t</td>\r\n\t\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<td>\r\n\t\t\t\t<span class=\"statusPodTipAlert\">${StatusTipDate} ${alertDateTime}</span>\r\n\t\t\t</td>\r\n\t\t</tr>"}});
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/statuspod/AlertStatusTip", [
	"ibm/tivoli/tip/TCTTable",
	"ibm/tivoli/tip/TCTNavModel",
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dijit/_WidgetBase",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	"dojo/text!./templates/alertStatusTip.html",
	"dojo/text!./templates/alertStatusTipTableRow.html",
	"dojo/string",
	"dojo/i18n!./nls/statuspodResources",
	"ibm/usmi/util/DataStore",
	"dojo/_base/Deferred",
	'dojo/_base/unload',
	"ibm/usmi/core/utils/Logger",
	"dojo/date/locale",
	"idx/oneui/HoverHelpTooltip"
], function (TCTTable, TCTNavModel, dojo, declare, lang, WidgetBase, TemplatedMixin, WidgetsInTemplateMixin,
		template, rowTemplate, string, MRI, DataStore, Deferred, baseUnload, Logger, locale) {
	return declare([
		WidgetBase,
		TemplatedMixin,
		WidgetsInTemplateMixin
	], {
		templateString : template,

		entityNode : "",
		objectId : null,
		_dataStore : null,
		isProblems : false,

		isClosed : false,

		activeAlertProperties : [
			"DateTime",
			"SetName",
			"CatName",
			"Severity"
		],
		activeAlertsSort : [ {
			attribute : "DateTime",
			descending : true
		}
		],

		_deferred : null,
		
		_logger : new Logger("ibm.usmi.statuspod", "AlertStatusTip"),

		postCreate : function () {
			this.inherited(arguments);
		},

		startup : function () {
			this.inherited(arguments);
		},

		show : function () {
			this.isClosed = false;
			if (!this._dataStore) {
				this.makeDataSet();
			} else {
				this._showStatusDialog();
			}
		},
		_showStatusDialog : function () {
			if (this.isClosed) {
				this._logger.finest("tooltip is closed, not show it", "_showStatusDialog");
				return;
			}
			idx.oneui.showHoverHelpTooltip(this.tipContent.innerHTML, this.entityNode, /*this.defaultPosition*/null, null, null, null, null, false);
		},

		hide : function () {
			idx.oneui.hideHoverHelpTooltip(this.entityNode);
			this.isClosed = true;
		},

		makeDataSet : function () {
			this._dataStore = new DataStore({
				datasetId : "ACTIVE_ALERTS",
				getTasks : false,
				parameters : {
					sourceToken : this.objectId
				},
				initReadyCallback : lang.hitch(this, this.updateAlertsTable)
			}, {
				sort : this.activeAlertsSort,
				defaultProperties : this.activeAlertProperties,
				createEventRouter : false //do not create event router
			});
			baseUnload.addOnUnload(this._dataStore, "destroy");

		},

		updateAlertsTable : function (items) {
			items = items || [];
			var itemIndex, itemsLength = items.length, html = [], first = true;
			var tipSummary = MRI.StatusShowingTipPartial;
			var maxAlertsInTip = 5, candicateItems = [], item, totalItemsCount = 0, candicateItemsCount = 0;

			items.sort(function (s, t) {
				var a = s.getProperty("Severity").getValue().getId();
				var b = t.getProperty("Severity").getValue().getId();
				a = Number(a);
				b = Number(b);
				if (a < b){
					return -1;
				}
				if (a > b){
					return 1;
				}
				return 0;
			});

			html.push("<table>");

			for (itemIndex = 0; itemIndex < itemsLength; ++itemIndex) {
				item = items[itemIndex];
				var severityId = item.getProperty("Severity").getValue().getId();
				if (severityId < 0 || severityId > 3) {
					if (this.isProblems || (severityId !== 5 && severityId !== "5")) {
						continue;
					}
				}
				var complianceEntry = item.getId().indexOf('ComplianceStatus') !== -1;
				if (this.isProblems && complianceEntry) {
					continue;
				} else if (!this.isProblems && !complianceEntry) {
					continue;
				}
				totalItemsCount++;
				if (candicateItemsCount >= maxAlertsInTip) {
					continue;
				} else {
					candicateItems.push(item);
					candicateItemsCount++;
				}
			}

			for (itemIndex = 0; itemIndex < candicateItems.length; ++itemIndex) {
				item = candicateItems[itemIndex];
				var alertSev = item.getProperty("Severity").getValue().getImage();
				var alertText = item.getProperty("SetName").getValue();
				var alertCat = item.getProperty("CatName").getValue();
				var alertDateTime = this._formatDateStr(item.getProperty("DateTime").getValue());
				var padding = "padding-top:10px;";
				if (first) {
					first = false;
					padding = "";
				}
				var row = string.substitute(rowTemplate, {
					alertText : alertText,
					alertCat : alertCat,
					alertDateTime : alertDateTime,
					padding : padding,
					alertSev : alertSev,
					StatusTipCategory : MRI.StatusTipCategory,
					StatusTipDate : MRI.StatusTipDate

				});
				html.push(row);
			}
			html.push("</table>");

			var text = lang.replace(tipSummary, [
				candicateItemsCount,
				totalItemsCount
			]);

			html.push("<hr><span class='statusPodSummary'>" + text + "</span>");
			var result = html.join("");
			this.tipContent.innerHTML = result;
			this._showStatusDialog();
		},

		_formatDateStr : function (dateTime) {
			var formatedStr;
			if (dateTime) {
				formatedStr=locale.format(dateTime, {formatLength: 'short', timePattern: 'HH:mm:ss ZZZZ'});
			}
			//console.debug("shorten date is:" + formatedStr);
			return formatedStr;
		},
		destroy : function () {
			var dataset;
			this.inherited(arguments);
			this.hide();
			if (this._dataStore) {
				dataset=this._dataStore.getDataset();
				if(dataset){
					dataset.destroy();
				}
				this._dataStore = null;
			}
		}

	});
});
},
'ibm/usmi/dashboard/ResourceGroupWidget':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define([
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dijit/_WidgetBase",
	"dijit/_TemplatedMixin",
	"dojo/text!./templates/ResourceGroupWidget.html",
	"dijit/_WidgetsInTemplateMixin",
	"./Constants",
	"dojo/_base/connect",
	"dojo/_base/lang",
	"dojo/hash",
	"dojo/io-query",
	"dojo/DeferredList",
	"ibm/usmi/core/utils/Logger"
], function (dojo, declare, WidgetBase, TemplatedMixin, template, WidgetsInTemplateMixin, Constants,
		connect, lang, hash, ioQuery, DeferredList, Logger) {
	return declare([
		WidgetBase,
		TemplatedMixin,
		WidgetsInTemplateMixin
	], {
		templateString : template,

		resourceGroup : null,

		//Subscriptions to clean up
		hashSubscription : null,
		hoverSubscription : null,
		
		// status class array
		_statusIconClass : {
			0 : "sprite dashboardErrorIcon",
			1 : "sprite dashboardWarningIcon",
			6 : "sprite normalIcon",
			5 : "sprite unknownIcon"
		},

		resourceImageFocus : true,
		
        // the logger
        _resourceGroupWidgetLogger : new Logger("ibm.usmi.dashboard", "ResourceGroupWidget"),

		/**
		 * Set up the component. Listen for mouse and keyboard events. Set the
		 * label to display an unknown amount of this component.
		 */
		postCreate : function () {
			this.inherited(arguments);
			this.resourceGroup.normalImage = this.resourceGroup.name + "Icon";
			this.connect(this.groupPod, "onmouseenter", "_onFocus");
			this.connect(this.groupPod, "onmouseleave", "_onBlur");
			this.connect(this.groupPod, "onblur", "_onBlur");
			this.connect(this.groupPod, "onfocus", "_onFocus");
			this.connect(this.groupPod, "onclick", "_onSelect");
			this.set("footerContentCount", this.resourceGroup.loadingLabel);
			this.hashSubscription = connect.subscribe("/dojo/hashchange", this, this.updateSelection);
			if (this.resourceGroup.selected) {
				this.set("imageClass", this.resourceGroup.normalImage + "Selected");
			} else {
				this.set("imageClass", this.resourceGroup.normalImage);
			}

		},

		/**
		 * Update the icon as not selected if other group's was selected.
		 */
		updateSelection : function (hashStr) {
			var hashObj = ioQuery.queryToObject(hashStr), selectedGroup = hashObj.selectedGroup;
			if (!selectedGroup) {
				if (this._resourceGroupWidgetLogger.isFinest()) {
					this._resourceGroupWidgetLogger.entering("selectedGroup not specified", "updateSelection");
				}
				return;
			}
			if (selectedGroup === this.resourceGroup.name && this.resourceGroup.selected) {
				//already selected for this component
				return;
			}

			if (selectedGroup === this.resourceGroup.name) {
				//this group is not selected yet
				this.resourceGroup.selected = true;
				this.set("imageClass", this.resourceGroup.normalImage + this.getStatusString() + "Selected");
			} else {
				//the selection has been changed
				this.resourceGroup.selected = false;
				try {
					this.set("imageClass", this.resourceGroup.normalImage + this.getStatusString());
				} catch (error) {
					if (this._resourceGroupWidgetLogger.isError()) {
						this._resourceGroupWidgetLogger.error("Setting imageClass failed:  "
								+ error, "updateSelection");
					} 
				}
			}
		},

		/**
		 * Set the height of this component to be the height of the image plus
		 * the height of the label. Needed to stop this component from taking up
		 * too much room on the page.
		 */
		resize : function () {
			if (this._resourceGroupWidgetLogger.isFinest()) {
				this._resourceGroupWidgetLogger.finest("resize group widget " + this.resourceGroup.name, "resize");
			}

			this.inherited(arguments);
			//dojo.style(this.containerNode, "height", (dojo.marginBox(this.image).h + dojo.marginBox(this.footer).h) + "px");
			//dojo.style(this.containerNode, "width", dojo.marginBox(this.image).w + "px");
		},

		/**
		 * Update the Dashbar icon with the current status.
		 */
		loadInitialStatus : function (totalNumber) {
			//initially load it as unknown
			if (this.resourceGroup.selected) {
				this.set("imageClass", this.resourceGroup.normalImage + "Unknown" + "Selected");			
			} else {
				this.set("imageClass", this.resourceGroup.normalImage + "Unknown");
			}
			this.set("footerContentCount", totalNumber);
			
		},
		
		getStatusString : function () {
			var statusString = "";
			if (this.resourceGroup.getGroupStatusValue() === 0) {
				statusString = "Critical";
			}
			else if (this.resourceGroup.getGroupStatusValue() === 1) {
				statusString = "Warning";	
			}
			else if (this.resourceGroup.getGroupStatusValue() === 5) {
				statusString = "Unknown";	
			}
			return statusString; 
		},
		
		/**
		 * Update the footer's status icon.
		 */
		updateStatus : function (status) {
			this.resourceGroup.setGroupStatus(status);
			//this.set("footerContent", status.getValue(), this.resourceGroup.getGroupCount());
			this.set("footerContentStatus", status.getValue());
		},
		
		/**
		 * Update the footer's total group item count number.
		 */
		updateCount : function (totalNumber) {
			this.set("footerContentCount", totalNumber);
			//this.set("footerContent", this.resourceGroup.getGroupStatusValue(), totalNumber);
		},

		/**
		 * Getter for the class used by this component's image. This method is
		 * called when this component gets and loses focus.
		 */
		_getImageClassAttr : function () {
			return this.image.className;
		},

		/**
		 * Setter for the class used by this component's image. This method is
		 * called when this component gets and loses focus.
		 */
		_setImageClassAttr : function (className) {
			this.image.className = className;
		},
		
		/**
		 * Setter to change this components status icon and text in the label.
		 * This is used when this component gets a result update from the back
		 * end. The status us set to the worst status in the results.
		 * 
		 * @param status
		 *            {String} key for the _statusIconClass map
		 */
		_setFooterContentStatusAttr : function (status) {
			if (!status && status !== 0) {
				// 5 is unknown
				status = 5;
			}
			
			var statusStr = "";
			if (status === 0) {
				statusStr = "Critical";
			}
			else if (status === 1) {
				statusStr = "Warning";
			}
			else if (status === 5) {
				statusStr = "Unknown";
			}
			
			if (this.resourceGroup.selected) {
				this.set("imageClass", this.resourceGroup.normalImage + statusStr + "Selected");			
			} else {
				this.set("imageClass", this.resourceGroup.normalImage + statusStr);
			}				
		},
		
		/**
		 * Setter to change this components status icon and text in the label.
		 * This is used when this component gets a result update from the back
		 * end. The status us set to the worst status in the results.
		 * 
		 * @param total
		 *            {String} the number or "?" to append indicating the number
		 *            of entities represented by the component
		 */
		_setFooterContentCountAttr : function (total) {

			total = total || 0;

			// abbreviate the label if necessary to make sure it doesn't go over the width of the footer
			var countText = " (" + total + ")";
			this.label.innerHTML = this.resourceGroup.label + countText;
		},

		/**
		 * Listener method for when this component receives focus. Set the class
		 * name of the main image to a hover class.
		 */
		_onFocus : function () {
			// tab events come in multiple times, so check if we already have focus
			// before setting the image
			connect.publish(Constants.resourceGroupHoved, this.resourceGroup.name);
			this.resourceImageFocus = true;
			if (this.get("imageClass").indexOf("Hover") === -1) {
				this.set("imageClass", this.resourceGroup.normalImage + this.getStatusString() + "Hover");
			}
		},

		/**
		 * Listener method for when this component loses focus. Set the class
		 * name of the main image to a non-hover class.
		 */
		_onBlur : function () {
			if (this.resourceGroup.selected) {
				return;
			}
			this.set("imageClass", this.resourceGroup.normalImage + this.getStatusString());
		},

		/**
		 * Listener method for when this component has been clicked on. Launches
		 * the table specified by the "tableName"
		 */
		_onSelect : function () {
			var refreshFlag = false;
			if (this.resourceGroup.selected === true) {
				refreshFlag = true;
			}
			this.resourceGroup.selected = true;
			if (this.get("imageClass").indexOf("Selected") === -1) {
				this.set("imageClass", this.resourceGroup.normalImage + this.getStatusString() + "Selected");
			}
			//connect.publish(Constants.resourceGroupSelected, this.name);
			var hashObj = ioQuery.queryToObject(hash());
			hashObj.selectedGroup = this.resourceGroup.name;
			
			// check to see if treeTopperId is set in hash, if so remove
			if (hashObj['treeTopperId']) {
				delete hashObj.treeTopperId;
			}

			var hashStr = ioQuery.objectToQuery(hashObj);
			hash(hashStr, true);

			if (refreshFlag) {
				//if it's clicked again, publish refresh message
				connect.publish(Constants.resourceGroupSelected, this.resourceGroup.name);
			}

		},

		getResourceGroupGroupOID : function () {
			return this.resourceGroup.getGroupOID();
		},
		
		/*
		 * load data from the backend
		 */
		loadSystemStatus : function () {
			if (this._resourceGroupWidgetLogger.isFinest()) {
				this._resourceGroupWidgetLogger.finest("loading system status for group " + this.resourceGroup.name, "loadSystemStatus");
			}

			var count = this.resourceGroup.getGroupCount();
			this.loadInitialStatus(count);
		},
		


		destroy : function () {
			if (this.hashSubscription) {
				this.hashSubscription.remove();
				this.hashSubscription = null;
			}
			if (this.hoverSubscription) {
				this.hoverSubscription.remove();
				this.hoverSubscription = null;
			}
			this.inherited(arguments);
		},
		destroyRecursive : function () {
			this.inherited(arguments);
		}

	});

});

},
'dijit/layout/StackContainer':function(){
define([
	"dojo/_base/array", // array.forEach array.indexOf array.some
	"dojo/cookie", // cookie
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.add domClass.replace
	"dojo/_base/kernel",	// kernel.isAsync
	"dojo/_base/lang",	// lang.extend
	"dojo/ready",
	"dojo/topic", // publish
	"../registry",	// registry.byId
	"../_WidgetBase",
	"./_LayoutWidget",
	"dojo/i18n!../nls/common"
], function(array, cookie, declare, domClass, kernel, lang, ready, topic,
			registry, _WidgetBase, _LayoutWidget){

/*=====
var _WidgetBase = dijit._WidgetBase;
var _LayoutWidget = dijit.layout._LayoutWidget;
var StackController = dijit.layout.StackController;
=====*/

// module:
//		dijit/layout/StackContainer
// summary:
//		A container that has multiple children, but shows only one child at a time.

// Back compat w/1.6, remove for 2.0
if(!kernel.isAsync){
	ready(0, function(){
		var requires = ["dijit/layout/StackController"];
		require(requires);	// use indirection so modules not rolled into a build
	});
}

// These arguments can be specified for the children of a StackContainer.
// Since any widget can be specified as a StackContainer child, mix them
// into the base widget class.  (This is a hack, but it's effective.)
lang.extend(_WidgetBase, {
	// selected: Boolean
	//		Parameter for children of `dijit.layout.StackContainer` or subclasses.
	//		Specifies that this widget should be the initially displayed pane.
	//		Note: to change the selected child use `dijit.layout.StackContainer.selectChild`
	selected: false,

	// closable: Boolean
	//		Parameter for children of `dijit.layout.StackContainer` or subclasses.
	//		True if user can close (destroy) this child, such as (for example) clicking the X on the tab.
	closable: false,

	// iconClass: String
	//		Parameter for children of `dijit.layout.StackContainer` or subclasses.
	//		CSS Class specifying icon to use in label associated with this pane.
	iconClass: "dijitNoIcon",

	// showTitle: Boolean
	//		Parameter for children of `dijit.layout.StackContainer` or subclasses.
	//		When true, display title of this widget as tab label etc., rather than just using
	//		icon specified in iconClass
	showTitle: true
});

return declare("dijit.layout.StackContainer", _LayoutWidget, {
	// summary:
	//		A container that has multiple children, but shows only
	//		one child at a time
	//
	// description:
	//		A container for widgets (ContentPanes, for example) That displays
	//		only one Widget at a time.
	//
	//		Publishes topics [widgetId]-addChild, [widgetId]-removeChild, and [widgetId]-selectChild
	//
	//		Can be base class for container, Wizard, Show, etc.

	// doLayout: Boolean
	//		If true, change the size of my currently displayed child to match my size
	doLayout: true,

	// persist: Boolean
	//		Remembers the selected child across sessions
	persist: false,

	baseClass: "dijitStackContainer",

/*=====
	// selectedChildWidget: [readonly] dijit._Widget
	//		References the currently selected child widget, if any.
	//		Adjust selected child with selectChild() method.
	selectedChildWidget: null,
=====*/

	buildRendering: function(){
		this.inherited(arguments);
		domClass.add(this.domNode, "dijitLayoutContainer");
		this.containerNode.setAttribute("role", "tabpanel");
	},

	postCreate: function(){
		this.inherited(arguments);
		this.connect(this.domNode, "onkeypress", this._onKeyPress);
	},

	startup: function(){
		if(this._started){ return; }

		var children = this.getChildren();

		// Setup each page panel to be initially hidden
		array.forEach(children, this._setupChild, this);

		// Figure out which child to initially display, defaulting to first one
		if(this.persist){
			this.selectedChildWidget = registry.byId(cookie(this.id + "_selectedChild"));
		}else{
			array.some(children, function(child){
				if(child.selected){
					this.selectedChildWidget = child;
				}
				return child.selected;
			}, this);
		}
		var selected = this.selectedChildWidget;
		if(!selected && children[0]){
			selected = this.selectedChildWidget = children[0];
			selected.selected = true;
		}

		// Publish information about myself so any StackControllers can initialize.
		// This needs to happen before this.inherited(arguments) so that for
		// TabContainer, this._contentBox doesn't include the space for the tab labels.
		topic.publish(this.id+"-startup", {children: children, selected: selected});

		// Startup each child widget, and do initial layout like setting this._contentBox,
		// then calls this.resize() which does the initial sizing on the selected child.
		this.inherited(arguments);
	},

	resize: function(){
		// Resize is called when we are first made visible (it's called from startup()
		// if we are initially visible). If this is the first time we've been made
		// visible then show our first child.
		if(!this._hasBeenShown){
			this._hasBeenShown = true;
			var selected = this.selectedChildWidget;
			if(selected){
				this._showChild(selected);
			}
		}
		this.inherited(arguments);
	},

	_setupChild: function(/*dijit._Widget*/ child){
		// Overrides _LayoutWidget._setupChild()

		this.inherited(arguments);

		domClass.replace(child.domNode, "dijitHidden", "dijitVisible");

		// remove the title attribute so it doesn't show up when i hover
		// over a node
		child.domNode.title = "";
	},

	addChild: function(/*dijit._Widget*/ child, /*Integer?*/ insertIndex){
		// Overrides _Container.addChild() to do layout and publish events

		this.inherited(arguments);

		if(this._started){
			topic.publish(this.id+"-addChild", child, insertIndex);	// publish

			// in case the tab titles have overflowed from one line to two lines
			// (or, if this if first child, from zero lines to one line)
			// TODO: w/ScrollingTabController this is no longer necessary, although
			// ScrollTabController.resize() does need to get called to show/hide
			// the navigation buttons as appropriate, but that's handled in ScrollingTabController.onAddChild().
			// If this is updated to not layout [except for initial child added / last child removed], update
			// "childless startup" test in StackContainer.html to check for no resize event after second addChild()
			this.layout();

			// if this is the first child, then select it
			if(!this.selectedChildWidget){
				this.selectChild(child);
			}
		}
	},

	removeChild: function(/*dijit._Widget*/ page){
		// Overrides _Container.removeChild() to do layout and publish events

		this.inherited(arguments);

		if(this._started){
			// this will notify any tablists to remove a button; do this first because it may affect sizing
			topic.publish(this.id + "-removeChild", page);	// publish
		}

		// If all our children are being destroyed than don't run the code below (to select another page),
		//  because we are deleting every page one by one
		if(this._descendantsBeingDestroyed){ return; }

		// Select new page to display, also updating TabController to show the respective tab.
		// Do this before layout call because it can affect the height of the TabController.
		if(this.selectedChildWidget === page){
			this.selectedChildWidget = undefined;
			if(this._started){
				var children = this.getChildren();
				if(children.length){
					this.selectChild(children[0]);
				}
			}
		}

		if(this._started){
			// In case the tab titles now take up one line instead of two lines
			// (note though that ScrollingTabController never overflows to multiple lines),
			// or the height has changed slightly because of addition/removal of tab which close icon
			this.layout();
		}
	},

	selectChild: function(/*dijit._Widget|String*/ page, /*Boolean*/ animate){
		// summary:
		//		Show the given widget (which must be one of my children)
		// page:
		//		Reference to child widget or id of child widget

		page = registry.byId(page);

		if(this.selectedChildWidget != page){
			// Deselect old page and select new one
			var d = this._transition(page, this.selectedChildWidget, animate);
			this._set("selectedChildWidget", page);
			topic.publish(this.id+"-selectChild", page);	// publish

			if(this.persist){
				cookie(this.id + "_selectedChild", this.selectedChildWidget.id);
			}
		}

		return d;		// If child has an href, promise that fires when the child's href finishes loading
	},

	_transition: function(newWidget, oldWidget /*===== ,  animate =====*/){
		// summary:
		//		Hide the old widget and display the new widget.
		//		Subclasses should override this.
		// newWidget: dijit._Widget
		//		The newly selected widget.
		// oldWidget: dijit._Widget
		//		The previously selected widget.
		// animate: Boolean
		//		Used by AccordionContainer to turn on/off slide effect.
		// tags:
		//		protected extension
		if(oldWidget){
			this._hideChild(oldWidget);
		}
		var d = this._showChild(newWidget);

		// Size the new widget, in case this is the first time it's being shown,
		// or I have been resized since the last time it was shown.
		// Note that page must be visible for resizing to work.
		if(newWidget.resize){
			if(this.doLayout){
				newWidget.resize(this._containerContentBox || this._contentBox);
			}else{
				// the child should pick it's own size but we still need to call resize()
				// (with no arguments) to let the widget lay itself out
				newWidget.resize();
			}
		}

		return d;	// If child has an href, promise that fires when the child's href finishes loading
	},

	_adjacent: function(/*Boolean*/ forward){
		// summary:
		//		Gets the next/previous child widget in this container from the current selection.
		var children = this.getChildren();
		var index = array.indexOf(children, this.selectedChildWidget);
		index += forward ? 1 : children.length - 1;
		return children[ index % children.length ]; // dijit._Widget
	},

	forward: function(){
		// summary:
		//		Advance to next page.
		return this.selectChild(this._adjacent(true), true);
	},

	back: function(){
		// summary:
		//		Go back to previous page.
		return this.selectChild(this._adjacent(false), true);
	},

	_onKeyPress: function(e){
		topic.publish(this.id+"-containerKeyPress", { e: e, page: this});	// publish
	},

	layout: function(){
		// Implement _LayoutWidget.layout() virtual method.
		var child = this.selectedChildWidget;
		if(child && child.resize){
			if(this.doLayout){
				child.resize(this._containerContentBox || this._contentBox);
			}else{
				child.resize();
			}
		}
	},

	_showChild: function(/*dijit._Widget*/ page){
		// summary:
		//		Show the specified child by changing it's CSS, and call _onShow()/onShow() so
		//		it can do any updates it needs regarding loading href's etc.
		// returns:
		//		Promise that fires when page has finished showing, or true if there's no href
		var children = this.getChildren();
		page.isFirstChild = (page == children[0]);
		page.isLastChild = (page == children[children.length-1]);
		page._set("selected", true);

		domClass.replace(page.domNode, "dijitVisible", "dijitHidden");

		return (page._onShow && page._onShow()) || true;
	},

	_hideChild: function(/*dijit._Widget*/ page){
		// summary:
		//		Hide the specified child by changing it's CSS, and call _onHide() so
		//		it's notified.
		page._set("selected", false);
		domClass.replace(page.domNode, "dijitHidden", "dijitVisible");

		page.onHide && page.onHide();
	},

	closeChild: function(/*dijit._Widget*/ page){
		// summary:
		//		Callback when user clicks the [X] to remove a page.
		//		If onClose() returns true then remove and destroy the child.
		// tags:
		//		private
		var remove = page.onClose(this, page);
		if(remove){
			this.removeChild(page);
			// makes sure we can clean up executeScripts in ContentPane onUnLoad
			page.destroyRecursive();
		}
	},

	destroyDescendants: function(/*Boolean*/ preserveDom){
		this._descendantsBeingDestroyed = true;
		this.selectedChildWidget = undefined;
		array.forEach(this.getChildren(), function(child){
			if(!preserveDom){
				this.removeChild(child);
			}
			child.destroyRecursive(preserveDom);
		}, this);
		this._descendantsBeingDestroyed = false;
	}
});

});

},
'ibm/usmi/statuspod/StatusPodMenuItem':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/statuspod/StatusPodMenuItem", [
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dijit/MenuItem",
	"ibm/usmi/core/utils/Logger"
], function (dojo, declare, MenuItem, Logger) {
	return declare([ MenuItem
	], {
		hoverTip : null,

		//object id of resource for this menu item
		objectId : null,

		_logger : new Logger("ibm.usmi.statuspod", "StatusPodMenuItem"),
		
		_onBlur : function () {
			this.inherited(arguments);
			this._setSelected(false);
		},
		
		_onHover : function () {
			//TODO: why does this crash on refresh? comment out for now
			//this.inherited(arguments);
			if (this.hoverTip) {
				this.hoverTip.show();
			}
		},

		_onUnhover : function () {
			this.inherited(arguments);
			if (this.hoverTip) {
				this.hoverTip.hide();
			}
		},

		setHoverTip : function (myHoverTip) {
			this.hoverTip = myHoverTip;
		},

		close : function () {
			this._onUnhover();
		},
		destroy : function () {
			this.inherited(arguments);
			if (this.hoverTip) {
				this.hoverTip.destroy();
				this.hoverTip = null;
			}
		}
	});
});

},
'dojox/grid/enhanced/_Plugin':function(){
define([
	"dojo/_base/kernel",
	"dojo/_base/lang",
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/connect",
    "../EnhancedGrid"
], function(dojo, lang, declare, array, connect){
	
return declare("dojox.grid.enhanced._Plugin", null, {
	// summary:
	//		Base class for all plugins.
	//
	// description:
	//		Provides common plugin functionality and basic life cycle management.
	//
	//		Each concrete plugin must have a name field and is responsible for registering itself to the global plugin registry
	//		e.g. for dnd plugin:
	// |		dojox.grid.EnhancedGrid.registerPlugin("dnd" /*plugin name*/,
	// |												dojox.grid.enhanced.plugins.DnD /*full class name of a plugin*/
	// |												{"preInit": false, "dependency": ["nestedSorting"]} /*properties*/);
	//
	//		[Keywords] of plugin properties(case sensitive)
	//		- "preInit": boolean, whether a plugin should be created before EnhancedGrid.postCreate(),
	//		   false by default(plugins are created after EnhancedGrid.postCreate()).
	//		- "dependency": array or string, plugin(s) indicated by "dependency" will be created before the current one.
	//		   Note: recursive cycle dependencies are not supported e.g. following dependency is invalid:
	//		   pluginA -> pluginB -> pluginA
	//
	// example:
	//		1. Customize default DnD plugin
	// |	dojo.declare("mygrid.MyDnD", dojox.grid.enhanced.plugins.DnD, {
	// |		name:"dnd" //still reuse the plugin name
	// |		constructor: function(inGrid, option){ ... }
	// |	});
	// |	dojox.grid.EnhancedGrid.registerPlugin("dnd", mygrid.MyDnD);
	//
	//		2. Add new plugin - PluginA
	// |	dojo.declare("mygrid.PluginA", dojox.grid.enhanced._Plugin, {
	// |		name: "pA",
	// |		constructor: function(inGrid, option){ ... }
	// |	});
	// |	dojox.grid.EnhancedGrid.registerPlugin("pA",mygrid.PluginA);
	//
	//		3. Use plugins
	// |	dojo.require("mygrid.MyDnD");
	// |	dojo.require("mygrid.PluginA");
	
	// |	<script type="text/javascript">
	// |		var grid = new dojox.grid.EnhancedGrid(
	// |		{plugins: {dnd:true, pA:true}, ... }, dojo.byId("gridDiv"));
	// |		grid.startup();
	// |	</script>

	//name: String
	//		Plugin name, e.g. 'nestedSorting', 'dnd'...
	name: 'plugin',
	
	//grid: Object
	//		Grid that the plugin belongs to
	grid: null,

	//option: Object
	//		Plugin properties - leveraged with default and user specified properties.
	//		e.g. for dnd plugin, it may look like {"class": dojox.grid.enhanced.plugins.DnD, "dependency": ["nestedSorting"], ...}
	option: {},

	//_connects: Array
	//		List of all connections.
	_connects: [],
	
	//_subscribes: Array
	//		List of all subscribes.
	_subscribes: [],

	//privates: Object
	//		Private properties/methods shouldn't be mixin-ed anytime.
	privates: {},
	
	constructor: function(inGrid, option){
		this.grid = inGrid;
		this.option = option;
		this._connects = [];
		this._subscribes = [];
		this.privates = lang.mixin({},dojox.grid.enhanced._Plugin.prototype);
		this.init();
	},
	
	init: function(){},
	
	onPreInit: function(){},
	
	onPostInit: function(){},
	
	onStartUp: function(){},
	
	connect: function(obj, event, method){
		// summary:
		//		Connects specified obj/event to specified method of this object.
		// example:
		//	|	var plugin = new dojox.grid.enhanced._Plugin(grid,"myPlugin",{...});
		//	|	// when foo.bar() is called, call the listener in the scope of plugin
		//	|	plugin.connect(foo, "bar", function(){
		//	|		console.debug(this.xxx());//"this" - plugin scope
		//	|	});
		var conn = connect.connect(obj, event, this, method);
		this._connects.push(conn);
		return conn;
	},
	disconnect: function(handle){
		// summary:
		//		Disconnects handle and removes it from connection list.
		array.some(this._connects, function(conn, i, conns){
			if(conn == handle){
				connect.disconnect(handle);
				conns.splice(i, 1);
				return true;
			}
			return false;
		});
	},
	subscribe: function(topic, method){
		// summary:
		//		Subscribes to the specified topic and calls the specified method
		//		of this object.
		// example:
		//	|	var plugin = new dojox.grid.enhanced._Plugin(grid,"myPlugin",{...});
		//	|	// when /my/topic is published, call the subscriber in the scope of plugin
		//	|	// with passed parameter - "v"
		//	|	plugin.subscribe("/my/topic", function(v){
		//	|		console.debug(this.xxx(v));//"this" - plugin scope
		//	|	});
		var subscribe = connect.subscribe(topic, this, method);
		this._subscribes.push(subscribe);
		return subscribe;
	},
	unsubscribe: function(handle){
		// summary:
		//		Un-subscribes handle and removes it from subscriptions list.
		array.some(this._subscribes, function(subscribe, i, subscribes){
			if(subscribe == handle){
				connect.unsubscribe(handle);
				subscribes.splice(i, 1);
				return true;
			}
			return false;
		});
	},
	onSetStore: function(store){
		// summary:
		//		Called when store is changed.
	},
	destroy: function(){
		// summary:
		//		Destroy all resources.
		array.forEach(this._connects, connect.disconnect);
		array.forEach(this._subscribes, connect.unsubscribe);
		delete this._connects;
		delete this._subscribes;
		delete this.option;
		delete this.privates;
		//console.log('Plugin [', this.name, '].destroy() executed!');
	}
});

//Each plugin is responsible for registering itself
// e.g. for DnD plugin(name:'dnd'):
// |	dojox.grid.EnhancedGrid.registerPlugin(dojox.grid.enhanced.plugins.DnD/*class*/,
// |		{"dependency": ["nestedSorting"]}/*Optional - properties*/);

});
},
'ibm/usmi/statuspod/_StatusPodPopupMixin':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/statuspod/_StatusPodPopupMixin", [
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/array",
	"ibm/usmi/core/utils/Logger"
], function (dojo, declare, lang, array, Logger) {
	return declare(null, {
		menuItems : [],

		_logger : new Logger("ibm.usmi.statuspod", "_StatusPopupMixin"),

		closeDescendants : function () {
			this._logger.finest("close all descendants menu items", "closeDescendants");
			if (!this.menuItems) {
				return;
			}
			array.forEach(this.menuItems, lang.hitch(this, function (menuItem) {
				if (menuItem) {
					this._logger.finest("close menu item", "closeDescendants");
					menuItem.close();
				}
			}));
		},

		addDescendant : function (/*StatusPodMenuItem*/menuItem) {
			this.menuItems.push(menuItem);
		}

	});
});

},
'idx/oneui/Menu':function(){
/*
 * Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

define(["dojo/_base/array",
		"dojo/_base/declare",
		"dojo/_base/event",
		"dojo/dom-geometry",
		"dijit/_TemplatedMixin",
		"dijit/_WidgetBase",
		"dijit/Menu",
		"dijit/MenuItem",
		"dijit/registry",
		"idx/oneui/_MenuOpenOnHoverMixin",
		"dojo/text!../oneui/templates/Menu.html",
		"dojo/text!../oneui/templates/_MenuColumn.html"],
		function(array,
				 declare,
				 event,
				 domgeometry,
				 _TemplatedMixin,
				 _WidgetBase,
				 Menu,
				 MenuItem,
				 registry,
				 _MenuOpenOnHoverMixin,
				 template,
				 columntemplate){
				 	
	var icons = {
		"error":        "oneuiErrorMenuItemIcon",
		"warning":      "oneuiWarningMenuItemIcon",
		"confirmation": "oneuiConfirmationMenuItemIcon",
		"information":  "oneuiInformationMenuItemIcon",
		"success":      "oneuiSuccessMenuItemIcon",
		"critical":     "oneuiCriticalMenuItemIcon",
		"attention":    "oneuiAttentionMenuItemIcon",
		"compliance":   "oneuiComplianceMenuItemIcon"
	}

	/**
	 * Creates a new idx.oneui.Menu
	 * @name idx.oneui.Menu
	 * @class The Menu widget provides a menu that facilitates event routing and
	 * functional linkage when used inside an idx.oneui.MenuDialog, and
	 * provides cascade-on-hover behaviour when the menu is used "flat". It
	 * also provides multi-column menu support including keyboard navigation.
	 * <p>
	 * Instances can be supplied as the "popup" parameter for dijit.PopupMenuItem
	 * and dijit.PopupMenuBarItem, and will operate as dijit.Menu. Instances can
	 * operate as popup menus on arbitrary DOM nodes, or for the whole window,
	 * and will operate as dijit.Menu. Instances can be placed "flat" within a
	 * layout in the UI, and will operate as dijit.Menu except that
	 * cascade-on-hover behaviour is available (controlled by the openOnHover
	 * property). Instances can be placed "flat" within an idx.oneui.MenuDialog,
	 * and can operate as a drop-down menu in combination with the containing
	 * dialog (controlled by the menuForDialog property).
	 * </p>
	 * <p>
	 * When menu items are added to this widget, the "column" property on each
	 * item can be used to specify the column in which the item should be placed.
	 * Columns will be created as needed to accommodate each new item, and can be
	 * styled by CSS.
	 * </p>
	 * @augments dijit.Menu
	 * @augments idx.oneui._MenuOpenOnHoverMixin
	 * @borrows idx.oneui._MenuOpenOnHoverMixin#openOnHover as this.openOnHover
	 * @example
	 * &lt;div data-dojo-type="idx.oneui.MenuDialog"&gt;
  &lt;div data-dojo-type="idx.oneui.Menu"&gt;
    &lt;div data-dojo-type="dijit.MenuItem" onclick="..."&gt;
      ...
    &lt;/div&gt;
      ...
  &lt;/div&gt;</span>
    ...
&lt;/div&gt;
	 */
	var Menu = declare("idx.oneui.Menu", [Menu, _MenuOpenOnHoverMixin], 
	/** @lends idx.oneui.Menu.prototype */
	{
		/**
		 * An array of the DOM nodes (tbody's), one per column, that
		 * contain the items that are in each column.
		 * @type DOMNode[]
		 * @private
		 */		
		_containerNodes: null,
		
		/**
		 * An array of the DOM nodes that can be used for styling individual
		 * columns in a multi-column menu. The nodes are ('td's), one per column,
		 * which are in a table row that implements the multiple columns, 
		 * and each of which contains
		 * a single column table containing the menu items. The column nodes
		 * are higher up in the DOM structure than the container nodes, but
		 * have a 1 to 1 relationship. They are externalised as they are more 
		 * suitable for styling the columns than the container nodes.
		 * @type DOMNode[]
		 */ 
		columnNodes: null,
		
		/**
		 * Indicates if this menu should be used by an idx.oneui.MenuDialog it
		 * is in to provide the primary menu functionality.
		 * @type boolean 
		 */
		menuForDialog: true,
		
		/**
	 	 * The template HTML for the widget.
		 * @constant
		 * @type string
		 * @private
		 * @default Loaded from idx/oneui/templates/Menu.html.
		 */
		templateString: template,
		
		/**
		 * Constructor.
		 * @private
		 */
		constructor: function(){
			this._containerNodes = [];
			this.columnNodes = [];
		},
		
		/**
		 * Returns the next or previous focusable child, compared to "child".
		 * Overridden because original does not work if the child widgets are
		 * not immediate peers in the DOM.
		 * <p>THIS IMPLEMENTATION ASSUMES THAT getChildren() RETURNS THE
		 * CHILDREN IN THE CORRECT ORDER FOR NAVIGATION. THIS IS TRUE
		 * AS OF DOJO V1.7</p>
		 * @param {dijit._Widget} child The current widget.
		 * @param {number} dir Integer: 1 = after, -1 = before.
		 */
		_getNextFocusableChild: function(child, dir){
			var nextFocusableChild = null;
			var children = this.getChildren();
			var startIndex;
			if(child != null){
				startIndex = array.indexOf(children, child);
				if (startIndex != -1) {
					startIndex += dir;
					if(startIndex < 0) 
						startIndex = children.length - 1;
					if(startIndex >= children.length) 
						startIndex = 0;
				}
			}
			else if(children.length == 0) 
				startIndex = -1;
			else 
				startIndex = (dir == 1) ? 0 : children.length - 1;
			
			if(startIndex != -1){
				// If there are children and the specifed start child if any
				// is one of them. Then iterate through the array of children
				// starting at the start position plus/minus. If either end of 
				// the array is hit then loop to the other end and continue
				// until a focusable child is found or the start position is
				// reached.
				var i = startIndex;
				do{
					if (children[i].isFocusable()) {
						nextFocusableChild = children[i];
						break;
					}
					i += dir;
					if(i < 0)
						i = children.length - 1;
					if(i >= children.length)
						i = 0;
				}while(i != startIndex);
			}
			
			return nextFocusableChild;
		},
		
		/**
		 * Finds the closest column in the specified direction with one or
		 * more focusable menu items in it and moves focus to the one level
		 * with the top of the currently focused menu item, or next higher or
		 * lower if that is not possible.
		 * @param {number} dir Direction - indicates whether to move focus to
		 * the next (+1) or previous (-1) column.
		 * @returns true if focus sucessfully moved, else false.
		 */
		_moveToColumn: function(dir){
			
			// Identify the column containing the currently focused menu
			// item and determine its y-position.
			if(this.focusedChild){
				for(var i = 0; i < this._containerNodes.length; i++){
					if(this.focusedChild.domNode.parentNode == this._containerNodes[i]){
						var focusedCol = i, yPos = domgeometry.getMarginBox(this.focusedChild.domNode).t;
						break;
					}
				}
			}
			if(focusedCol != undefined){
				// Try to locate the closest column in the appropriate direction
				// with at least one focusable item in it.
				for (i = focusedCol + dir; i >= 0 && i < this._containerNodes.length; i += dir) {
					var children = registry.findWidgets(this._containerNodes[i]);
					var focusableChildren = dojo.filter(children, function(child){ return child.isFocusable() })
					if(focusableChildren.length > 0){
						var targetColumn = i;
						break;
					}
				}
				if(targetColumn != undefined){
					// Iterate through the focusable children and transfer 
					// focus to the item level with the currently focused
					// one, or the next higher, or the next lower, in that
					// order.
					for (i = 0; i < focusableChildren.length; i++){
						var child = focusableChildren[i];
						var childBox = domgeometry.getMarginBox(child.domNode);
						if(yPos >= childBox.t && yPos <= childBox.t + childBox.h - 1){
							this.focusChild(child);
							return true;
						} 
						else if(yPos < childBox.t){
							if(i > 0){
								this.focusChild(focusableChildren[i - 1]);
								return true;
							}else{
								this.focusChild(child);
								return true;
							}
						}else if(i == focusableChildren.length - 1){
							this.focusChild(child);
							return true;
						} 
					}
				}
			}
			
			// If the method didn't get to the point of focusing another menu
			// item then it failed.
			return false;
		},

		/**
		 * We replace _onKeyPress because our base class does NOT stop a
		 * close-sub-menu-key event even when it has handled the event
		 * (by cancelling itself or passing the navigation request to a
		 * parent menu bar). Curiously, it DOES stop the keypress event
		 * when it DOESN'T handle it, and this seems exactly the wrong way
		 * around. If the base class key handling changes so that handled
		 * keypress are also stopped, this override version can be deleted.
		 * @param {Event} evt
		 */		
		_onKeyPress: function(evt){

			if(evt.ctrlKey || evt.altKey){ return; }

			switch(evt.charOrCode){
				case this._openSubMenuKey:
					if (!this._moveToColumn(+1)) {
						this._moveToPopup(evt);
					}
					event.stop(evt);
					break;
				case this._closeSubMenuKey:
					if(!this._moveToColumn(-1)){
						if(this.parentMenu){
							if(this.parentMenu._isMenuBar){
								this.parentMenu.focusPrev();
							}else{
								this.onCancel(false);
							}
						}
					}				
					event.stop(evt);
					break;
			}
		},
		
		/**
		 * Refresh the layout of the menu items in the columns.
		 */
		refresh: function(){

			// Make sure that all child widgets are in the correct columns.
			var children = this.getChildren();
			for(var i = 0; i < children.length; i++){
				this.addChild(children[i]);
			}
		},		

		/**
		 * Standard widget lifecycle startup() method.
		 * @private
		 */		
		startup: function(){

			if(this._started){ return; }
			this._started = true;
			this.inherited(arguments);
			
			// When the widget is created the container node is associated
			// with column 0 (by the template), so that menu items are 
			// inserted into column 0 by the default template logic. The
			// container node now needs to be moved up the DOM tree so that 
			// it covers all the columns of menu items, and the menu items that
			// were initially loaded into column 0 moved to other columns, if 
			// necessary.
			this.containerNode = this._columnContainerNode;
			this.refresh();
		},
		
		/**
		 * Override of standard container addChild() method, to allocate
		 * menuitems and separators to specified columns. 
		 * @param {dijit._Widget} widget Menu item, separator or heading to be
		 * added to the menu.
		 * @param {number} insertIndex Optional position to insert the item
		 * into the column in.
		 */
		addChild: function(widget, insertIndex){

			// Create new columns for the menu as necessary, if the item is
			// to be placed in a column that does not exist yet.
			while(this._containerNodes.length <= (widget.column || 0)){
				var node = _TemplatedMixin.getCachedTemplate(columntemplate).cloneNode(true);
				this._attachTemplateNodes(node, function(n,p){ return n.getAttribute(p); });
				this._columnContainerNode.appendChild(node);			
			}
			
			// Add the item to the column by temporarily making the column 
			// node the container node for the widget and then invoking the
			// standard, inherited dijit._Container functionality.
			this.containerNode = this._containerNodes[widget.column || 0];		
			this.inherited(arguments);
			this.containerNode = this._columnContainerNode;
		}
	});
	
	/**
	 * Create a menu item that represents a single message and which
	 * can be inserted into One UI message menus (menus which have the
	 * additional CSS class "oneuiMessageMenu").
	 * @name createMessageMenuItem
	 * @function
	 * @memberOf idx.oneui.Menu
	 * @param {Object} args An object containing some the following fields,
	 * which are all optional:
	 * <ul>
	 * 	<li>
	 * 	  type: {string}
	 * 		The type of message. This can be "error", "warning",
	 *          "information", or "success".
	 *  </li>
	 * 	<li>
	 * 	  content: {string}
	 * 		The message content.
	 *  </li>
	 * 	<li>
	 * 	  messageId: {string}
	 * 	    An identifier for the message, displayed alongside the
	 *          content.
	 *  </li>
	 * 	<li>
	 * 	  timestamp: {string}
	 * 	    The date/time that the message was originated, displayed
	 *          alongside the content.
	 *  </li>
	 * </ul>
	 * @example var menuItem = Menu.createMessageMenuItem({
     *	type: "error",    // or "warning", "information" or "success"
     *	content: "Hello, world!",
     *	timestamp: locale.format(new Date(), { formatLength: "medium", 
     *	                                       locale: this.lang }),
     *	messageId: "CAT123456"
     *});
	 */
	Menu.createMessageMenuItem = function(args){

		var label = "";
		
		if(args){
			if(args.timestamp){
				label += '\u200f<span class="messageMenuTimestamp messagesContrast">\u200e' + args.timestamp + '\u200f</span>\u200e';
			}
			
			if(args.content){
				label += '\u200f <span class="messageTitles">\u200e' + args.content + '\u200f</span>\u200e';
			}
			
			if(args.messageId){
				label += '\u200f <span class="messagesContrast">(\u200e' + args.messageId + '\u200f)</span>\u200e';
			}
		}

		return new MenuItem({ label: label, iconClass: args && args.type && icons[args.type] });
	}
	
	return Menu;
});
},
'dojox/grid/_TreeView':function(){
define([
	"dijit/registry",
	"../main",
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/_base/event",
	"dojo/dom-attr",
	"dojo/dom-class",
	"dojo/dom-style",
	"dojo/dom-construct",
	"dojo/query",
	"dojo/parser",
	"dojo/text!./resources/Expando.html",
	"dijit/_Widget",
	"dijit/_TemplatedMixin",
	"./_View",
	"./_Builder",
	"./util"
], function(dijit, dojox, declare, array, lang, event, domAttr, domClass, 
	domStyle, domCtr, query, parser, template, _Widget, _TemplatedMixin, _View, _Builder, util){

declare("dojox.grid._Expando", [ _Widget, _TemplatedMixin ], {
	open: false,
	toggleClass: "",
	itemId: "",
	cellIdx: -1,
	view: null,
	rowNode: null,
	rowIdx: -1,
	expandoCell: null,
	level: 0,
	templateString: template,
	_toggleRows: function(toggleClass, open){
		if(!toggleClass || !this.rowNode){ return; }
		if(query("table.dojoxGridRowTableNeedsRowUpdate").length){
			if(this._initialized){
				this.view.grid.updateRow(this.rowIdx);
			}
			return;
		}
		var self = this;
		var g = this.view.grid;
		if(g.treeModel){
			var p = this._tableRow ? domAttr.get(this._tableRow, "dojoxTreeGridPath") : "";
			if(p){
				query("tr[dojoxTreeGridPath^=\"" + p + "/\"]", this.rowNode).forEach(function(n){
					var en = query(".dojoxGridExpando", n)[0];
					if(en && en.parentNode && en.parentNode.parentNode &&
								!domClass.contains(en.parentNode.parentNode, "dojoxGridNoChildren")){
						var ew = dijit.byNode(en);
						if(ew){
							ew._toggleRows(toggleClass, ew.open&&open);
						}
					}
					n.style.display = open ? "" : "none";
				});
			}
		}else{
			query("tr." + toggleClass, this.rowNode).forEach(function(n){
				if(domClass.contains(n, "dojoxGridExpandoRow")){
					var en = query(".dojoxGridExpando", n)[0];
					if(en){
						var ew = dijit.byNode(en);
						var toggleClass = ew ? ew.toggleClass : en.getAttribute("toggleClass");
						var wOpen = ew ? ew.open : self.expandoCell.getOpenState(en.getAttribute("itemId"));
						self._toggleRows(toggleClass, wOpen&&open);
					}
				}
				n.style.display = open ? "" : "none";
			});
		}
	},
	setOpen: function(open){
		if(open && domClass.contains(this.domNode, "dojoxGridExpandoLoading")){
			open = false;
		}
		var view = this.view;
		var grid = view.grid;
		var store = grid.store;
		var treeModel = grid.treeModel;
		var d = this;
		var idx = this.rowIdx;
		var me = grid._by_idx[idx];
		if(!me){ return; }
		if(treeModel && !this._loadedChildren){
			if(open){
				// Do this to make sure our children are fully-loaded
				var itm = grid.getItem(domAttr.get(this._tableRow, "dojoxTreeGridPath"));
				if(itm){
					this.expandoInner.innerHTML = "o";
					domClass.add(this.domNode, "dojoxGridExpandoLoading");
					treeModel.getChildren(itm, function(items){
						d._loadedChildren = true;
						d._setOpen(open);
					});
				}else{
					this._setOpen(open);
				}
			}else{
				this._setOpen(open);
			}
		}else if(!treeModel && store){
			if(open){
				var data = grid._by_idx[this.rowIdx];
				if(data&&!store.isItemLoaded(data.item)){
					this.expandoInner.innerHTML = "o";
					domClass.add(this.domNode, "dojoxGridExpandoLoading");
					store.loadItem({
						item: data.item,
						onItem: lang.hitch(this, function(i){
							var idty = store.getIdentity(i);
							grid._by_idty[idty] = grid._by_idx[this.rowIdx] = { idty: idty, item: i };
							this._setOpen(open);
						})
					});
				}else{
					this._setOpen(open);
				}
			}else{
				this._setOpen(open);
			}
		}else{
			this._setOpen(open);
		}
	},
	_setOpen: function(open){
		if(open && this._tableRow && domClass.contains(this._tableRow, "dojoxGridNoChildren")){
			this._setOpen(false);
			return;
		}
		this.expandoInner.innerHTML = open ? "-" : "+";
		domClass.remove(this.domNode, "dojoxGridExpandoLoading");
		domClass.toggle(this.domNode, "dojoxGridExpandoOpened", open);
		if(this._tableRow){
			domClass.toggle(this._tableRow, "dojoxGridRowCollapsed", !open);
			var base = domAttr.get(this._tableRow, "dojoxTreeGridBaseClasses");
			var new_base = "";
			if(open){
				new_base = lang.trim((" " + base + " ").replace(" dojoxGridRowCollapsed ", " "));
			}else{
				if((" " + base + " ").indexOf(' dojoxGridRowCollapsed ') < 0){
					new_base = base + (base ? ' ' : '' ) + 'dojoxGridRowCollapsed';
				}else{
					new_base = base;
				}
			}
			domAttr.set(this._tableRow, 'dojoxTreeGridBaseClasses', new_base);
		}
		var changed = (this.open !== open);
		this.open = open;
		if(this.expandoCell && this.itemId){
			this.expandoCell.openStates[this.itemId] = open;
		}
		var v = this.view;
		var g = v.grid;
		if(this.toggleClass && changed){
			if(!this._tableRow || !this._tableRow.style.display){
				this._toggleRows(this.toggleClass, open);
			}
		}
		if(v && this._initialized && this.rowIdx >= 0){
			g.rowHeightChanged(this.rowIdx);
			g.postresize();
			v.hasVScrollbar(true);
		}
		this._initialized = true;
	},
	onToggle: function(e){
		this.setOpen(!this.open);
		event.stop(e);
	},
	setRowNode: function(rowIdx, rowNode, view){
		if(this.cellIdx < 0 || !this.itemId){ return false; }
		this._initialized = false;
		this.view = view;
		this.rowNode = rowNode;
		this.rowIdx = rowIdx;
		this.expandoCell = view.structure.cells[0][this.cellIdx];
		var d = this.domNode;
		if(d && d.parentNode && d.parentNode.parentNode){
			this._tableRow = d.parentNode.parentNode;
		}
		this.open = this.expandoCell.getOpenState(this.itemId);
		if(view.grid.treeModel){
			// TODO: Rather than hard-code the 18px and 3px, we should probably
			// calculate them based off css or something...  However, all the
			// themes that we support use these values.
			domStyle.set(this.domNode , "marginLeft" , (this.level * 18) + "px");
			if(this.domNode.parentNode){
				domStyle.set(this.domNode.parentNode, "backgroundPosition", ((this.level * 18) + (3)) + "px");
			}
		}
		this.setOpen(this.open);
		return true;
	}
});

var _TreeContentBuilder = declare("dojox.grid._TreeContentBuilder", _Builder._ContentBuilder, {
	generateHtml: function(inDataIndex, inRowIndex){
		var
			html = this.getTableArray(),
			v = this.view,
			row = v.structure.cells[0],
			item = this.grid.getItem(inRowIndex),
			grid = this.grid,
			store = this.grid.store;

		util.fire(this.view, "onBeforeRow", [inRowIndex, [row]]);
		
		var createRow = function(level, rowItem, summaryRow, toggleClasses, rowStack, shown){
			if(!shown){
				if(html[0].indexOf('dojoxGridRowTableNeedsRowUpdate') == -1){
					html[0] = html[0].replace("dojoxGridRowTable", "dojoxGridRowTable dojoxGridRowTableNeedsRowUpdate");
				}
				return;
			}
			var rowNodeIdx = html.length;
			toggleClasses = toggleClasses || [];
			var tcJoin = toggleClasses.join('|');
			var tcString = toggleClasses[toggleClasses.length - 1];
			var clString = tcString + (summaryRow ? " dojoxGridSummaryRow" : "");
			var sString = "";
			if(grid.treeModel && rowItem && !grid.treeModel.mayHaveChildren(rowItem)){
				clString += " dojoxGridNoChildren";
			}
			html.push('<tr style="' + sString + '" class="' + clString + '" dojoxTreeGridPath="' + rowStack.join('/') +  '" dojoxTreeGridBaseClasses="' + clString + '">');
			var nextLevel = level + 1;
			var parentCell = null;
			for(var i=0, cell; (cell=row[i]); i++){
				var m = cell.markup, cc = cell.customClasses = [], cs = cell.customStyles = [];
				// content (format can fill in cc and cs as side-effects)
				m[5] = cell.formatAtLevel(rowStack, rowItem, level, summaryRow, tcString, cc);
				// classes
				m[1] = cc.join(' ');
				// styles
				m[3] = cs.join(';');
				// in-place concat
				html.push.apply(html, m);
				if(!parentCell && cell.level === nextLevel && cell.parentCell){
					parentCell = cell.parentCell;
				}
			}
			html.push('</tr>');
			if(rowItem && store && store.isItem(rowItem)){
				var idty = store.getIdentity(rowItem);
				if(typeof grid._by_idty_paths[idty] == "undefined"){
					grid._by_idty_paths[idty] = rowStack.join('/');
				}
			}
			var expandoCell;
			var parentOpen;
			var path;
			var values;
			var iStack = rowStack.concat([]);
			if(grid.treeModel && rowItem){
				if(grid.treeModel.mayHaveChildren(rowItem)){
					expandoCell = v.structure.cells[0][grid.expandoCell||0];
					parentOpen = expandoCell.getOpenState(rowItem) && shown;
					path = new dojox.grid.TreePath(rowStack.join('/'), grid);
					values = path.children(true)||[];
					array.forEach(values, function(cItm, idx){
						var nToggle = tcJoin.split('|');
						nToggle.push(nToggle[nToggle.length - 1] + "-" + idx);
						iStack.push(idx);
						createRow(nextLevel, cItm, false, nToggle, iStack, parentOpen);
						iStack.pop();
					});
				}
			}else if(rowItem && parentCell && !summaryRow){
				expandoCell = v.structure.cells[0][parentCell.level];
				parentOpen = expandoCell.getOpenState(rowItem) && shown;
				if(store.hasAttribute(rowItem, parentCell.field)){
					var tToggle = tcJoin.split('|');
					tToggle.pop();
					path = new dojox.grid.TreePath(rowStack.join('/'), grid);
					values = path.children(true)||[];
					if(values.length){
						html[rowNodeIdx] = '<tr class="' + tToggle.join(' ') +' dojoxGridExpandoRow" dojoxTreeGridPath="' + rowStack.join('/') + '">';
						array.forEach(values, function(cItm, idx){
							var nToggle = tcJoin.split('|');
							nToggle.push(nToggle[nToggle.length - 1] + "-" + idx);
							iStack.push(idx);
							createRow(nextLevel, cItm, false, nToggle, iStack, parentOpen);
							iStack.pop();
						});
						iStack.push(values.length);
						createRow(level, rowItem, true, toggleClasses, iStack, parentOpen);
					}else{
						html[rowNodeIdx] = '<tr class="' + tcString + ' dojoxGridNoChildren" dojoxTreeGridPath="' + rowStack.join('/') + '">';
					}
				}else{
					if(!store.isItemLoaded(rowItem)){
						html[0] = html[0].replace("dojoxGridRowTable", "dojoxGridRowTable dojoxGridRowTableNeedsRowUpdate");
					}else{
						html[rowNodeIdx] = '<tr class="' + tcString + ' dojoxGridNoChildren" dojoxTreeGridPath="' + rowStack.join('/') + '">';
					}
				}
			}else if(rowItem && !summaryRow && toggleClasses.length > 1){
				html[rowNodeIdx] = '<tr class="' + toggleClasses[toggleClasses.length - 2] + '" dojoxTreeGridPath="' + rowStack.join('/') + '">';
			}
		};
		createRow(0, item, false, ["dojoxGridRowToggle-" + inRowIndex], [inRowIndex], true);
		html.push('</table>');
		return html.join(''); // String
	},
	findTarget: function(inSource, inTag){
		var n = inSource;
		while(n && (n!=this.domNode)){
			if(n.tagName && n.tagName.toLowerCase() == 'tr'){
				break;
			}
			n = n.parentNode;
		}
		return (n != this.domNode) ? n : null;
	},
	getCellNode: function(inRowNode, inCellIndex){
		var node = query("td[idx='" + inCellIndex + "']", inRowNode)[0];
		if(node&&node.parentNode&&!domClass.contains(node.parentNode, "dojoxGridSummaryRow")){
			return node;
		}
	},
	decorateEvent: function(e){
		e.rowNode = this.findRowTarget(e.target);
		if(!e.rowNode){return false;}
		e.rowIndex = domAttr.get(e.rowNode, 'dojoxTreeGridPath');
		this.baseDecorateEvent(e);
		e.cell = this.grid.getCell(e.cellIndex);
		return true; // Boolean
	}
});

return declare("dojox.grid._TreeView", _View, {
	_contentBuilderClass: _TreeContentBuilder,
	_onDndDrop: function(source, nodes, copy){
		if(this.grid && this.grid.aggregator){
			this.grid.aggregator.clearSubtotalCache();
		}
		this.inherited(arguments);
	},
	postCreate: function(){
		this.inherited(arguments);
		this.connect(this.grid, '_cleanupExpandoCache', '_cleanupExpandoCache');
	},
	_cleanupExpandoCache: function(index, identity, item){
		if(index == -1){
			return;
		}
		array.forEach(this.grid.layout.cells, function(cell){
			if(typeof cell['openStates'] != 'undefined'){
				if(identity in cell.openStates){
					delete cell.openStates[identity];
				}
			}
		});
		if(typeof index == "string" && index.indexOf('/') > -1){
			var path = new dojox.grid.TreePath(index, this.grid);
			var ppath = path.parent();
			while(ppath){
				path = ppath;
				ppath = path.parent();
			}
			var pitem = path.item();
			if(!pitem){
				return;
			}
			var idty = this.grid.store.getIdentity(pitem);
			if(typeof this._expandos[idty] != 'undefined'){
				for(var i in this._expandos[idty]){
					var exp = this._expandos[idty][i];
					if(exp){
						exp.destroy();
					}
					delete this._expandos[idty][i];
				}
				delete this._expandos[idty];
			}
		}else{
			for(var i in this._expandos){
				if(typeof this._expandos[i] != 'undefined'){
					for(var j in this._expandos[i]){
						var exp = this._expandos[i][j];
						if(exp){
							exp.destroy();
						}
					}
				}
			}
			this._expandos = {};
		}
	},
	postMixInProperties: function(){
		this.inherited(arguments);
		this._expandos = {};
	},
	onBeforeRow: function(inRowIndex, cells){
		// Save off our expando if we have one so we don't have to create it
		// again
		var g = this.grid;
		if(g._by_idx && g._by_idx[inRowIndex] && g._by_idx[inRowIndex].idty){
			var idty = g._by_idx[inRowIndex].idty;
			this._expandos[idty] = this._expandos[idty] || {};
		}
		this.inherited(arguments);
	},
	onAfterRow: function(inRowIndex, cells, inRowNode){
		array.forEach(query("span.dojoxGridExpando", inRowNode), function(n){
			if(n && n.parentNode){
				// Either create our expando or put the existing expando back
				// into place
				var tc = n.getAttribute("toggleClass");
				var idty;
				var expando;
				var g = this.grid;
				if(g._by_idx && g._by_idx[inRowIndex] && g._by_idx[inRowIndex].idty){
					idty = g._by_idx[inRowIndex].idty;
					expando = this._expandos[idty][tc];
				}
				if(expando){
					domCtr.place(expando.domNode, n, "replace");
					expando.itemId = n.getAttribute("itemId");
					expando.cellIdx = parseInt(n.getAttribute("cellIdx"), 10);
					if(isNaN(expando.cellIdx)){
						expando.cellIdx = -1;
					}
				}else{
					if(idty){
						expando = parser.parse(n.parentNode)[0];
						this._expandos[idty][tc] = expando;
					}
				}
				if(expando && !expando.setRowNode(inRowIndex, inRowNode, this)){
					expando.domNode.parentNode.removeChild(expando.domNode);
				}
			}
		}, this);
		var alt = false;
		var self = this;
		query("tr[dojoxTreeGridPath]", inRowNode).forEach(function(n){
			domClass.toggle(n, "dojoxGridSubRowAlt", alt);
			domAttr.set(n, "dojoxTreeGridBaseClasses", n.className);
			alt = !alt;
			self.grid.rows.styleRowNode(domAttr.get(n, 'dojoxTreeGridPath'), n);
		});
		this.inherited(arguments);
	},
	updateRowStyles: function(inRowIndex){
		var rowNodes = query("tr[dojoxTreeGridPath='" + inRowIndex + "']", this.domNode);
		if(rowNodes.length){
			this.styleRowNode(inRowIndex, rowNodes[0]);
		}
	},
	getCellNode: function(inRowIndex, inCellIndex){
		var row = query("tr[dojoxTreeGridPath='" + inRowIndex + "']", this.domNode)[0];
		if(row){
			return this.content.getCellNode(row, inCellIndex);
		}
	},
	destroy: function(){
		this._cleanupExpandoCache();
		this.inherited(arguments);
	}
});
});
},
'ibm/tivoli/tip/navwidget/properties/dijit/PropertiesNotebook':function(){
// wrapped by build app
define("ibm/tivoli/tip/navwidget/properties/dijit/PropertiesNotebook", ["dijit","dojo","dojox","dojo/i18n!ibm/tivoli/tip/navwidget/nls/resources","dojo/require!dijit/dijit,dijit/layout/TabContainer,dijit/layout/ContentPane,ibm/tivoli/tip/dijit/BusyPane,ibm/tivoli/navmodel/Trace,ibm/tivoli/tip/navwidget/dijit/NavWidgetObjectPluggableView,ibm/tivoli/tip/navwidget/properties/PropertyRenderer,ibm/tivoli/tip/navwidget/dijit/NavWidgetPluggableView,ibm/tivoli/tip/navwidget/dijit/NavWidgetTabContainerObjectPluggableView"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebook");

dojo.require("dijit.dijit");
dojo.require("dijit.layout.TabContainer");
dojo.require("dijit.layout.ContentPane");
dojo.require("ibm.tivoli.tip.dijit.BusyPane");
dojo.require("ibm.tivoli.navmodel.Trace");
// 41517 remove dojo.require("dojox.data.dom");
dojo.require("ibm.tivoli.tip.navwidget.dijit.NavWidgetObjectPluggableView");
dojo.require("ibm.tivoli.tip.navwidget.properties.PropertyRenderer");
dojo.require("ibm.tivoli.tip.navwidget.dijit.NavWidgetPluggableView");
dojo.require("ibm.tivoli.tip.navwidget.dijit.NavWidgetTabContainerObjectPluggableView");
dojo.requireLocalization("ibm.tivoli.tip.navwidget", "resources");


ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebook._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebook");




dojo.declare("ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebook",
		[ibm.tivoli.tip.navwidget.dijit.NavWidgetTabContainerObjectPluggableView],
    
	{		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
	
		//*************************** 
		// "Private"
		//***************************	
	
		// _trace: [private] ibm.tivoli.navmodel.Trace
		//		Utility for trace and logging
	    _trace: ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebook._trace,
		
	    //_DEFAULT_WIDGET_DEF_INFO: [private const] Object
	    //		Coming soon
	    _DEFAULT_WIDGET_DEF_INFO: {
			className: "ibm.tivoli.navmodel.NotebookWidgetDef",
			baseUrl: "/ibm/tivoli/rest",
	    	provider: "any",
	    	datasource: "any",
	    	widget: "notebook",
	    	definition: "DefaultNotebook"
		},
		
		constructor: function () {		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon

		},

		postCreate: function(){		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			
			//Call superclass
			this.inherited(arguments);
		},
	
		//***********************************************************
		// override createWigetDefinition
		//***********************************************************
		_createWidgetDefinition: function(){		
			//Call super class which will load widget def
			this.inherited(arguments);
			
			//Look at widget def for attributes we need
			if (this.widgetDefinition) {
				//now set tab orientation 
				this.tabPosition = this.widgetDefinition.getTabOrientation();
			}
		},
		
		_getDefaultWidgetDefInfo: function(){
			return this._DEFAULT_WIDGET_DEF_INFO;
		},
		
		startup: function(){		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			this.inherited(arguments);	
			
		},
			
	
		destroy: function(){		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
            this.inherited(arguments);
	
            
		}		
		
	
});
});

},
'ibm/usmi/widgets/table/DefaultTablePersistenceMixin':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/widgets/table/DefaultTablePersistenceMixin", [
	"dojo/_base/declare",
	"dojo/_base/kernel",
	"dojo/_base/lang",
	"dojo/_base/array",
	"dojo/aspect",
	"dijit/registry",
	"ibm/usmi/widgets/table/DefaultItemsTableViewPersist"
], function (declare, dojo, lang, array, aspect, registry, Persist) {

	return declare([], {

		// add gridparams necessary for persistence
		persistenceGridParams : function (datastore, gridParams) {
			var persistId;
			if (!gridParams.id) {
				if (datastore && datastore.dataset && datastore.dataset._dataset) {
					persistId = "persistgrid:" + datastore.dataset._dataset.id;
				} else {
					persistId = "emptygrid";
				}
			} else {
				persistId = gridParams.id;
			}
			
			var module = {
				moduleClass : Persist,
				key : this._buildPersistenceKey(persistId)
			};
			if (gridParams.modules) {
				gridParams.modules.push(module);
			} else {
				gridParams.modules = [ module ];
			}
			return gridParams;
		},

		// hook up persistence
		initPersistence : function (t, g, searchField) {
			var selection;
			var filter;

			// check parameters for whether row selection should be persisted...
			if (g.persist) {
				filter = g.persist.registerAndLoad('filter', function () {
					if (g.filterBar) {
						return g.filterBar.filterData;
					} else {
						return null;
					}
				});

				selection = g.persist.registerAndLoad('selection', function () {
					if (g.select) {
						return g.select.row.getSelected();
					} else {
						return null;
					}
				});

				if (filter && g.filterBar) {
					// if it is a single basic filter, insert text into quick filter field
					if (searchField && filter.type === "any" && filter.conditions && filter.conditions.length === 1
							&& filter.conditions[0].condition === "contain") {
						searchField.value = filter.conditions[0].value;
					}
					g.filterBar.applyFilter(filter);
					g.filterBar.show(); // ensure filter bar displays
				}

				if (selection) {
					var model = g.model;
					array.forEach(selection, function (id) {
						model.markById(id, 1);
					});
					model.when();
				
// David said following was needed for toolbar revalidation - internally crashes for tree tables					
//					if(g.select.row) {
//						g.select.row.onSelected();
//					}

					// scroll to first selection
					// index/scroll needs to be delayed until the model data shows up
					// TODO: find a better method than a timeout delay
					setTimeout(function () {
						if (g.vScroller) {
							var selectIndex = -1;
							array.forEach(selection, function (id) {
								var index = model.idToIndex(id);
								if (selectIndex === -1 || selectIndex > index) {
									selectIndex = index;
								}
							});
							g.vScroller.scrollToRow(selectIndex);
						}
					}, 1000);
				}
			}
		},

		// build a new persistence key based on the url and sorted hash parameters
		// strip out sourceToken and other variables that artificially make the key unique for table
		_buildPersistenceKey : function (id) {
			var hash = dojo.hash();

			// build up an array so we can sort it
			var storedParams = [];
			var params = hash.split('&');
			for ( var i = 0; i < params.length; i++) {
				if (params[i].split('=')[0] !== "sourceToken") {
					storedParams.push(params[i]);
				}
			}

			// sort it...
			var newHash = "";
			storedParams.sort();
			for ( var j = 0; j < storedParams.length; j++) {
				if (j > 0) {
					newHash += "&";
				}
				newHash += storedParams[j];
			}

			var loc = window.location;
			return loc.protocol + "//" + loc.host + loc.pathname + "?" + newHash + '/' + id;
		}

	});
});
},
'ibm/tivoli/tip/dijit/BreadcrumbSeparator':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!dijit/_Widget,dijit/_Templated"], function(dijit,dojo,dojox){
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/

dojo.provide("ibm.tivoli.tip.dijit.BreadcrumbSeparator");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.declare(
		"ibm.tivoli.tip.dijit.BreadcrumbSeparator",
		[ dijit._Widget, dijit._Templated ],
	{
		// summary
		//	A line between two menu items
		templateString: '<div class="breadcrumbDivider dijitInline">&gt;</div>',
		postCreate: function(){ dojo.setSelectable(this.domNode, false); },
		isFocusable: function(){ return false; }
	});


});

},
'idx/oneui/_MenuOpenOnHoverMixin':function(){
/*
 * Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

define(["dojo/_base/declare"],
        function(declare){
    
	// Ensure we're not relying on the old globals, ready for 2.0.
	var dojo = {}, dijit = {};

  
	/**
	 * Creates a new idx.oneui._MenuOpenOnHoverMixin
	 * @name idx.oneui._MenuOpenOnHoverMixin
	 * @class This mix-in can be mixed into menus and menu bars to make them
	 * permanently active so that their drop down and cascade menus are
	 * activated by mouse hover without the need for the menu or menu bar to
	 * be clicked on first.
	 */
	return declare("idx.oneui._MenuOpenOnHoverMixin", null,
	/** @lends idx.oneui._MenuOpenOnHoverMixin.prototype */
	{
		/**
		 * If true, this menu / menu bar will open popup menu items when they
		 * are hovered over at any time. This is in addition to the usual
		 * mouse-click and key activations, which continue to work as
		 * usual. If false, the menu / menu bar will still open popup menu
		 * items on hover when it is "active" (ie, when the user is in the
		 * process of interacting with it), but will NOT open popup menu
		 * items on hover when the menu is inactive: only mouse-click and
		 * key activations work in this case. The default value is true,
		 * enabling menus to be activated by hover.
		 * @type boolean
		 */
		openOnHover: true,

		/**
		 * Used internally to track our true activation state, because when
		 * openOnHover is on we never allow ourselves to become inactive even
		 * when we normally would.
		 * @type boolean
		 */
		_isActuallyActive: false,

		/**
		 * Standard Dojo setter for handling the 'opnOnHover' property via calls to 
		 * set().
		 * @param {Object} newvalue
		 */
		_setOpenOnHoverAttr: function(newvalue){
			this.openOnHover = newvalue;
			if(newvalue){
				this._forceActive();
			}else{
				this._restoreActive();
			}
		},
			
		/**
		 * Keep track of our 'actual' active state.
		 */
		_markActive: function(){
			this.inherited(arguments);
			this._isActuallyActive = true;
		},

		/**
		 * Keep track of our 'actual' active state, but when open-on-hover
		 * is enabled prevent us becoming inactive even when we normally would.
		 */		
		_markInactive: function(){
			if(!this.openOnHover){
				this.inherited(arguments);
			}
			this._isActuallyActive = false;
		},
		
		/**
		 * Mark the menubar as active regardless of whether it 'actually' is,
		 * but preserve our memory of our 'actual' active state.
		 */
		_forceActive: function(){
			var actual = this._isActuallyActive;
			this._markActive();
			this._isActuallyActive = actual;
		},
		
		/**
		 * Return to our 'actual' active state after a force.
		 */
		_restoreActive: function(){
			if(this._isActuallyActive){
				this._markActive();
			}else{
				this._markInactive();
			}
		}

	});
});
},
'ibm/usmi/util/DataStore':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/array",
	"ibm/tivoli/tip/navwidget/dijit/DefaultMessageHandler",
	"ibm/tivoli/navmodel/DatasetItemsStore",
	"ibm/tivoli/navmodel/Dataset" ], function (declare, djLang, djArray, DefaultMessageHandler, DatasetItemsStore,
	Dataset) {
	return declare([], {
		_dataset : null,
		_items : null,
		_tasks : null,
		_datasetJson : null,
		_eventCountToNotify : 1, //default is 1, since we always get the dataset event
		_currentEventCount : 0,
		constructorParms : null,
		datasetArgs : null,
		/* constructorParms:
		 * You can construct the dataset in 2 ways. 
		 * 1. Use it to retrieve a regular collection, passing in getItems, getTasks, and retrieveItemProperties. 
		 * 2. Use it to retrieve a NGen task using a JSON object, passing in useJson
		 * getItems	- Boolean - retrieve collection items true/false - Default: true if useJson == false
		 * getTasks - Boolean - retrieve collection tasks true/false - Default: true if useJson == false
		 * retrieveItemProperties - Boolean - Retrieve collection items properties true/false - Default: true if useJson == false
		 * useJson - don't retrieve items, instead, get json string from dataset
		 * parameters - parameters to pass in to the collection creations, such as homeParms or source token 
		 * for ex:
		 * parameters: {homeParms : 'filterThis', sourceToken : 3445}
		 * initReadyCallback(Required) - method reference to call back when data retrieval is complete    
		*/

		constructor : function (args, additionalDatasetArgs) {
			this.constructorParms = {};
			this.datasetArgs = {};
			djLang.mixin(this.constructorParms, args);
			if (!this.constructorParms.parameters) {
				this.constructorParms.parameters = {};
			}

			if (additionalDatasetArgs) {
				djLang.mixin(this.datasetArgs, additionalDatasetArgs);
			}
			if (this.constructorParms.useJson === undefined) {
				this.constructorParms.useJson = false;
			}
			if (this.constructorParms.useJson === false) {
				if (this.constructorParms.getItems === undefined) {
					this.constructorParms.getItems = true;
				}
				if (this.constructorParms.getTasks === undefined) {
					this.constructorParms.getTasks = true;
				}
				if (this.constructorParms.retrieveItemProperties === undefined) {
					this.constructorParms.retrieveItemProperties = true;
				}
			} else {
				this.constructorParms.getTasks = true;
			}

			this._setupEventListening();
			if (this.constructorParms.getItems === true) {
				if (this.constructorParms.retrieveItemProperties === true) {
					this.sendCuriDataRequest(this.constructorParms.datasetId, djLang
						.hitch(this, this._datasetRetrieved), djLang.hitch(this, this._errorOnDatasetRetrieved),
						this.constructorParms.parameters, djLang.hitch(this, this._datasetItemsRetrieved), djLang
							.hitch(this, this._errorOnDatasetItemsRetrieved), true);
				} else {
					this.sendCuriDataRequest(this.constructorParms.datasetId, djLang
						.hitch(this, this._datasetRetrieved), djLang.hitch(this, this._errorOnDatasetRetrieved),
						this.constructorParms.parameters, djLang.hitch(this, this._datasetItemsRetrieved), djLang
							.hitch(this, this._errorOnDatasetItemsRetrieved));
				}
			} else {
				this.sendCuriDataRequest(this.constructorParms.datasetId, djLang.hitch(this, this._datasetRetrieved),
					djLang.hitch(this, this._errorOnDatasetRetrieved), this.constructorParms.parameters);
			}

		},
		/**
		 * return dataset object
		 */
		getDataset : function () {
			return this._dataset;
		},
		/**
		 * return tasks that belong to the dataset
		 */
		getTasks : function () {
			return this._tasks;
		},
		/**
		 * return all items that belong to the dataset
		 */
		getItems : function () {
			return this._items;
		},
		/**
		 * return a dataset(collection) attribute.
		 */
		getDatasetAttribute : function (key) {
			return this._dataset.getAttribute(key);
		},
		/**
		 * return an item by index (numeric)
		 */
		getItem : function (/*Integer*/index) {
			return this._items[index];
		},
		/**
		 * returns the value of a item property.
		 */
		getItemProperty : function (/*Dataset item*/item, /*Property id string*/property) {
			return item.getProperty(property).getValue();
		},
		/**
		 * goes through all items in data set and returns item that matches the
		 * value of a property. Use only when there is no duplicate values for
		 * the given property
		 */
		getItemFromProperty : function (propertyName, propertyValue) {
			var item;
			djArray.forEach(this.getItems(), function (entry, i) {
				var propValue = entry.getProperty(propertyName).getValue();
				if (propValue === propertyValue) {
					item = entry;
					return entry;
				}
			});
			return item;
		},
		/**
		 * Returns all properties for an item in JSON format
		 */
		getItemProperties : function (item) {
			var properties = {};
			var itemProps = item.getCachedPropertyIds(true);
			var i = 0;
			for (i = 0; i < itemProps.length; i++) {
				properties[itemProps[i]] = item.getProperty(itemProps[i]).getValue();
			}
			return properties;
		},
		/**
		 * return the dataset json data in JSON object format
		 */
		getDatasetJson : function () {
			return this._datasetJson;
		},
		updateServerJsonData : function (jsonObject) {
			var jsonString = JSON.stringify(jsonObject);
			this.executeTask("executeJsonTask", {
				selectionParms : {
					jsonData : jsonString
				}
			});
		},

		/**
		 * Execute a dataset task. TODO: implement passing context Usage :
		 * executeTask(task_id OR task_object, { selectionParms : { YOUR_PROP_ID :
		 * YOUR_PROP_VALUE, YOUR_PROP_ID2 : YOUR_PROP_VALUE } });
		 */
		executeTask : function (task, args) {
			var taskArgs = {
				onComplete : function (result, kwArgs) {
				// Utils.Trace('task', 'success');
				},
				onError : function (error, exception, ioArgs) {
				//  Utils.Trace('task', 'error');
				},
				selectionParms : {}
			};
			djLang.mixin(taskArgs, args);
			if (typeof task === "string") {
				var i;
				for (i = 0; i < this._tasks.length; i++) {
					if (this._tasks[i].getId() == task) {
						task = this._tasks[i];
					}
				}
			}
			taskArgs.selection = new ibm.tivoli.navmodel.Selection({
				version : '1'
			});

			task.execute(taskArgs);
		},
		_setupEventListening : function () {
			this._eventCountToNotify = 1;
			this._currentEventCount = 0;
			if (this.constructorParms.getItems === true) {
				this._eventCountToNotify++;
			}
			if (this.constructorParms.getTasks === true) {
				this._eventCountToNotify++;
			}

		},
		_datasetRetrieved : function (dataset) {
			console.info('_datasetRetrieved');
			//  Utils.Trace(dataset);
			this._dataset = dataset;
			if (this.constructorParms.useJson === true) {
				this._datasetJson = JSON.parse(dataset.getAttribute("jsonData"));
			}
			if (this.constructorParms.getTasks) {
				var functions = {
					onComplete : djLang.hitch(this, this._datasetTasksRetrieved),
					onError : djLang.hitch(this, this._errorOnDatasetTasksRetrieved)
				};
				this._dataset.getTasks(functions);
			}
			if (this.constructorParms.getItems === true) {
				if (dataset._args.sort) {
					var store = new DatasetItemsStore({
						dataset : dataset,
						defaultProperties : dataset._args.defaultProperties
					});
					store.fetch({
						onComplete : djLang.hitch(this, this._datasetItemsRetrieved),
						onError : djLang.hitch(this, this._errorOnDatasetItemsRetrieved),
						sort : dataset._args.sort
					});
				}
			}
			this._eventReceived();
		},
		_errorOnDatasetRetrieved : function (error) {
			//TODO: allow callback on exploiters to get notified of specific errors
			this._retrievalError(error);
		},
		_datasetItemsRetrieved : function (items) {
			console.info('_datasetItemsRetrieved' + this.constructorParms.datasetId);
			this._items = items;
			this._eventReceived();
		},
		_errorOnDatasetItemsRetrieved : function (error) {
			//TODO: allow callback on exploiters to get notified of specific errors
			this._retrievalError(error);
		},
		_datasetTasksRetrieved : function (tasks) {
			console.info('_datasetTasksRetrieved');
			// Utils.Trace(tasks);

			this._tasks = tasks;
			this._eventReceived();
		},
		_errorOnDatasetTasksRetrieved : function (error, exception, ioArgs) {
			//TODO: allow callback on exploiters to get notified of specific errors
			this._retrievalError(error, exception, ioArgs);
		},
		_retrievalError : function (error, exception, ioArgs) {
			console.error('Dataset error for ' + this.datasetId);
			var msgHandler = new DefaultMessageHandler({});
			msgHandler.displayException(exception);
		},
		_eventReceived : function () {
			if (this.constructorParms.initReadyCallback == null) {
				return;
			}
			this._currentEventCount++;
			if (this._currentEventCount === this._eventCountToNotify) {
				if (this.constructorParms.useJson === true) {
					this.constructorParms.initReadyCallback(this._datasetJson);
				} else {
					this.constructorParms.initReadyCallback(this._items);
				}
			}
		},
		sendCuriDataRequest : function (navModelId, requestCallBack, requestErrorCallBack, parameters,
			nodesRetrievedCallBack, errorCallBack, retrieveItemProperties) {
			this._sendDataRequest(parameters, "/ibm/tivoli/rest", navModelId, requestCallBack, requestErrorCallBack,
				nodesRetrievedCallBack, errorCallBack, retrieveItemProperties);

		},
		_sendDataRequest : function (myParameters, myBaseUrl, myDataset, requestCallBack, requestErrorCallBack,
			nodesRetrievedCallBack, errorCallBack, retrieveItemProperties) {

			args = {
				baseUrl : myBaseUrl,
				provider : "USMI",
				datasource : "",
				dataset : myDataset,
				parameters : myParameters,
				useDatastore : false,
				onComplete : requestCallBack,
				onError : requestErrorCallBack,
				defaultProperties : [ "all" ]
			};
			djLang.mixin(args, this.datasetArgs);
			var dataset = new Dataset(args);
			if (requestCallBack) {
				dataset.onComplete = requestCallBack;
			}
			if (requestErrorCallBack) {
				dataset.onError = requestErrorCallBack;
			}
			if (nodesRetrievedCallBack) {
				var functions = {
					onComplete : nodesRetrievedCallBack
				};
				if (errorCallBack) {
					functions.onError = errorCallBack;
				}
				if (args.sort) {

				} else {
					dataset.getItemsAndLinks(functions);
				}
			}

		},
		destroy : function () {
			if (this._dataset) {
				this._dataset.destroy();
			}
			this._dataset = null;
			this._items = null;
			this._tasks = null;
		}

	});
});
},
'dojox/grid/_RowManager':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/dom-class"
], function(declare, lang, domClass){

	var setStyleText = function(inNode, inStyleText){
		if(inNode.style.cssText == undefined){
			inNode.setAttribute("style", inStyleText);
		}else{
			inNode.style.cssText = inStyleText;
		}
	};

	return declare("dojox.grid._RowManager", null, {
		//	Stores information about grid rows. Owned by grid and used internally.
		constructor: function(inGrid){
			this.grid = inGrid;
		},
		linesToEms: 2,
		overRow: -2,
		// styles
		prepareStylingRow: function(inRowIndex, inRowNode){
			return {
				index: inRowIndex,
				node: inRowNode,
				odd: Boolean(inRowIndex&1),
				selected: !!this.grid.selection.isSelected(inRowIndex),
				over: this.isOver(inRowIndex),
				customStyles: "",
				customClasses: "dojoxGridRow"
			};
		},
		styleRowNode: function(inRowIndex, inRowNode){
			var row = this.prepareStylingRow(inRowIndex, inRowNode);
			this.grid.onStyleRow(row);
			this.applyStyles(row);
		},
		applyStyles: function(inRow){
			var i = inRow;

			i.node.className = i.customClasses;
			var h = i.node.style.height;
			setStyleText(i.node, i.customStyles + ';' + (i.node._style||''));
			i.node.style.height = h;
		},
		updateStyles: function(inRowIndex){
			this.grid.updateRowStyles(inRowIndex);
		},
		// states and events
		setOverRow: function(inRowIndex){
			var last = this.overRow;
			this.overRow = inRowIndex;
			if((last!=this.overRow)&&(lang.isString(last) || last >= 0)){
				this.updateStyles(last);
			}
			this.updateStyles(this.overRow);
		},
		isOver: function(inRowIndex){
			return (this.overRow == inRowIndex && !domClass.contains(this.grid.domNode, "dojoxGridColumnResizing"));
		}
	});
});
},
'ibm/tivoli/tip/navwidget/topology/dijit/TopologyLabel':function(){
// wrapped by build app
define("ibm/tivoli/tip/navwidget/topology/dijit/TopologyLabel", ["dijit","dojo","dojox","dojo/require!dijit/_Widget,dijit/_Templated"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.provide("ibm.tivoli.tip.navwidget.topology.dijit.TopologyLabel");

// author: Karol Romanowski
dojo.declare("ibm.tivoli.tip.navwidget.topology.dijit.TopologyLabel",
            [ dijit._Widget, dijit._Templated ],
{		
	// summary:
	//		A Label for adding labels to the toolbar.
	// description:
	//		Coming soon
	
	// labelFor: [public] String
	labelFor: "",
	
	// labelText: [public] String
	labelText: "",
	
	// templateString: [public] String
	templateString:"<label for=\"${labelFor}\" dojoAttachPoint=\"containerNode\">${labelText}</label>\n",		

   postCreate: function() {          // 255810
      // If labelFor was not specified on the constructor, then this label is not labeling some other
      // form control, so remove the "for" attribute to not leave a dangling reference.  For WCAG rules
      if (this.labelFor.length == 0) {
         dojo.removeAttr(this.domNode, "for");
      }
   },

	setLabelText: function(/*String*/ content){
        		// summary:
        		//		Deprecated.  Use attr('labelText', ...) instead.
        		this.attr("labelText", content);
       },

     _setLabelTextAttr: function(/*String*/ content){
        		// summary:
        		//		Hook for attr('labelText', ...) to work.
        		// description:
        		//		Set the label (text) of the label; takes an HTML string.
        		this.containerNode.innerHTML = this.labelText = content;
       }

});

});

},
'ibm/usmi/scheduler/wizard_pages/SchedulePage':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
/*global define */
define([ "dojo/_base/declare",
    "dojo/_base/lang",
	"dojo/date",
	"dojox/date/php",
	"dojo/dom-construct",
	"dojo/on",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	"dijit/form/CheckBox",
	"dijit/form/RadioButton",
	"dijit/form/NumberSpinner",
	"dijit/form/ValidationTextBox",
	"dijit/form/DateTextBox",
	"dijit/form/Select",
	"dijit/form/MultiSelect",
	"dojox/form/TimeSpinner",
	"dojox/validate",
	"dojoe/wizard/Page",
	"ibm/tivoli/tip/navwidget/dijit/DefaultMessageHandler",
	"dojo/text!./templates/SchedulePage.html",
	// The resource bundle definition
	"dojo/i18n!./nls/JobLaunchWizardResources" ], function (
	declare, lang, date, php, construct, on, _TemplatedMixin, _WidgetsInTemplateMixin, CheckBox, RadioButton, NumberSpinner,
	ValidationTextBox, DateTextBox, Select, MultiSelect, TimeSpinner, validate, Page, DefaultMessageHandler,
	SchedulePage, mri) {
	return declare([ Page,
		_TemplatedMixin,
		_WidgetsInTemplateMixin ], {

		templateString : SchedulePage,
		persistResources : mri,

		title : "",
		required : true,

		allowNext : function () {
			return this.isValid(); //validateChoices();
		},

		allowFinish : function () {
			return this.isValid();
		},

		isValid : function () {
			return this.validate();
		},

		validate : function () {
			var msgHandler = new DefaultMessageHandler();
			var msgId = ""; // not specifying a message ID
			var msgSeverity = "error";
			var theMessageTitle = "";

			// Start at the top and validate the user input.

			// Validate the jobName
			// retrieve the job name (required by every job)
			var jobNameParm = this.jobName.get('value');

			// not needed?  We should get this for free since job name is marked as required.....
			if ((jobNameParm === null) || (jobNameParm.length <= 0)) {
			//	theMessageTitle = this.persistResources.NO_VALID_JOB_NAME_ERROR;
			//	msgHandler.displayMessage(msgId, msgSeverity, theMessageTitle);
				return false;
			}  

			// retrieve the "runnow" radio button....if it's checked, all other scheduling information is moot:
			var runNow = this.runNowRadio.get('value');
			if (runNow === "now") {

				//this.wizard.enableNextButton(true);
				//this.wizard.enableFinishButton(true);

				return true;
			}

			// we must have some sort of schedule associated with us....determine our frequency
			// retrieve the repeat option selected
			var frequency_option = this.repeatSelection.get('value');
			if(lang.isArray(frequency_option)){
				frequency_option=frequency_option[0];
			}

			if (frequency_option === "CUSTOM") {
				// there must be at least one custom date
				var customDatesContainer = this.selectedCustomDates.containerNode;
				if (customDatesContainer.options.length > 0) {

					//this.wizard.enableNextButton(true);
					//this.wizard.enableFinishButton(true);

					return true;
				} else {
					// There are no custom dates -- but the user selected custom dates option...That's not valid.
					//this.wizard.enableNextButton(false);
					//this.wizard.enableFinishButton(false);

					theMessageTitle = this.persistResources.MISSING_CUSTOM_DATES;
					msgHandler.displayMessage(msgId, msgSeverity, theMessageTitle);

					return false;
				}

			} else {

				// Retrieve the selected dates and times....make sure that the selected date/time is not in the past....
				var theDate = this.scheduleDate.get('value');
				var theTime = this.scheduleTime.get('value');

				// Construct a new Date object using the date from "theDate" and the time from "theTime" -- so we can compare it to the current date/time.
				var customDate = new Date(theDate); // this is only taking the date into account....

				// retrieve the hours and minutes from the time field
				var hours = date.locale.format(theTime, {
					timePattern : "H",
					selector : "time"
				});
				var mins = date.locale.format(theTime, {
					timePattern : "mm",
					selector : "time"
				});

				// set the time field on our start date
				customDate.setHours(hours, mins);// make sure we've got a start date/time

				var currentDate = new Date();
				if (date.compare(currentDate, customDate) >= 0) {

					if (frequency_option === "ONCE") {
						// the selected date and time combination is either in the past or is right now....That's not valid.
						//this.wizard.enableNextButton(false);
						//this.wizard.enableFinishButton(false);

						theMessageTitle = this.persistResources.SELECTED_DATE_IN_PAST;
						msgHandler.displayMessage(msgId, msgSeverity, theMessageTitle);

						return false;
					} // else we're not once -- but we don't really care if the current date/time is in the past if we're going to run every Tuesday, for example...or monthly on the 3rd..
				} else {
					// the date is correct....and if our frequency_option is "ONCE" -- we're valid; otherwise, continue validation
					if (frequency_option === "ONCE") {
						return true;
					}
				}

				// get the duration -- and make sure that if it's not "forever", that there's a valid duration 
				// that is, "0" is not a valid duration...(run it every hour for 0 hours)
				var theDuration = this.forDurationRadio.get('value');
				var theDurationValue;
				if (theDuration === "for") {

					theDurationValue = this.repeatFor.get('value'); // the value (like "6")
					if ((!theDurationValue) || (theDurationValue <= 0) || (theDurationValue > 5000)) {
						// the duration for value is less than or equal to 0...That's not valid.
						//this.wizard.enableNextButton(false);
						//this.wizard.enableFinishButton(false);

						theMessageTitle = this.persistResources.REPEAT_FOR_DURATION_NOT_VALID;
						msgHandler.displayMessage(msgId, msgSeverity, theMessageTitle);

						return false;
					}

				} else {
					theDuration = this.untilDurationRadio.get('value');
					if (theDuration === "until") { // we're "until"
						theDurationValue = ((new Date(this.repeatUntil.get('value'))).getTime());

						if (date.compare(customDate, theDurationValue) >= 0) {
							// the duration until value is before the scheduled date...That's not valid.			
							//this.wizard.enableNextButton(false);
							//this.wizard.enableFinishButton(false);

							theMessageTitle = this.persistResources.REPEAT_UNTIL_DURATION_NOT_VALID;
							msgHandler.displayMessage(msgId, msgSeverity, theMessageTitle);

							return false;
						}
					}

				}

				if (frequency_option === "WEEKLY") {
					// We MUST have at least one day of the week selected							
					if (!((this.sundayDay.get('value')) || (this.mondayDay.get('value')) ||
						(this.tuesdayDay.get('value')) || (this.wednesdayDay.get('value')) ||
						(this.thursdayDay.get('value')) || (this.fridayDay.get('value')) || (this.saturdayDay
						.get('value')))) {
						// No days of the week have been selected...That's not valid.						
						//this.wizard.enableNextButton(false);
						//this.wizard.enableFinishButton(false);

						theMessageTitle = this.persistResources.MISSING_SELECTED_DAY_OF_WEEK;
						msgHandler.displayMessage(msgId, msgSeverity, theMessageTitle);

						return false;
					}

				}

				if (frequency_option === "MONTHLY_DATES") {
					// there must be at least one monthly date
					var monthlyDatesContainer = this.selectedMonthlyDates.containerNode;
					if (monthlyDatesContainer.options.length > 0) {

						//this.wizard.enableNextButton(true);
						//this.wizard.enableFinishButton(true);

						return true;
					} else {
						// There are no monthly dates -- but the user selected monthly dates option...That's not valid.
						//this.wizard.enableNextButton(false);
						//this.wizard.enableFinishButton(false);

						theMessageTitle = this.persistResources.MISSING_MONTHLY_DATES;
						msgHandler.displayMessage(msgId, msgSeverity, theMessageTitle);

						return false;
					}

				}
			}

			//this.wizard.enableNextButton(true);
			//this.wizard.enableFinishButton(true);

			return true;
		},

		postCreate : function () {
			// Build the job name...
			// first get the current date/time
			var currentDate = new Date();

			// TODO:  5/23 This formatting of the date does not seem to be working any longer.  I expected it to be 
			//        "Friday, Mar 30 2012"....instead it's coming out like:  "2 2012 3 30"....I thought 
			//        the formatLength: "long" with selector : "date" would get me closer to what I need --
			//        In theory, it should deliver:  EEEE, Y MMMM DD (or Friday, 2012 March 30) -- instead, it's giving me
			//        6, 2012 3 30 -- so, numbers for what I'd expected to be words....
			//        I've tried using locale.format as well as dojo.date.locale.format with no luck....
			//        Similarly, the scheduleDate control is supposed to be a dateTextBox, but when you utilize the control to 
			//        specify a new date, the displayed value for the month is a number:  6.
			var theDate = date.locale.format(currentDate, {
				selector : 'date',
				formatLength : 'full'
			});
			
			var theTime = date.locale.format(currentDate, {
				timePattern : "h:mm:ss a v",
				selector : "time"
			});

			// TODO:  8/24 Based on the above, I need to clean up the format of the job name -- it doesn't match ISD's job names ("Collect Inventory - March 30, 2012 1:29:03 PM EDT" 
			//        -- instead it's coming out as: "Collect Inventory - 2, 2012 6 04 1:23:52 PM Central Daylight Time)"   
			// 		  I should be able to do the commented out line below, but instead, I'm just using "currentDate" since it's most pretty...
    		this.jobName.set('value', this.jobNameBase + " - " + theDate + " - " + theTime);
			//this.jobName.set('value', this.jobNameBase + " - " + currentDate);

			// Ensure the user cannot select a date in the past.
			this.scheduleDate.constraints.min = currentDate;
			this.repeatUntil.constraints.min = currentDate;

			// Ensure the schedule time is....right now....
			var defaultScheduleDate = date.locale.format(currentDate, {
				timePattern : "h:mm a",
				selector : "time"
			});
			this.scheduleTime.set('value', currentDate);

			// Link up our functions...
			// Link repeat options
			on(this.repeatSelection, "change", lang.hitch(this, this.doSchedulePageRendering));

			// Link monthly add/remove buttons
			on(this.addDayOfMonth, "click", lang.hitch(this, this.doMonthlyAdd));
			on(this.removeDaysOfMonth, "click", lang.hitch(this, this.doMonthlyRemove));

			// Link custom schedule add/remove buttons
			on(this.addCustomDate, "click", lang.hitch(this, this.doCustomAdd));
			on(this.removeCustomDate, "click", lang.hitch(this, this.doCustomRemove));
			on(this.selectedCustomDatesDiv, "click", lang.hitch(this, this.doCustomButtonEnablement));

			// set the title of the page
			this.title = this.persistResources.SCHEDULE_PAGE_TITLE;
		},

		reset : function () {},

		doSchedulePageRendering : function () {
			// Get the selection the user made
			var theSelection=this.getRepeatSelectionValue();

			if (theSelection === "ONCE") { // "once"
				// hide everything....
				this.allRepeatDetails.style.display = 'none';

			} else if (theSelection === "WEEKLY") { // "weekly"
				this.allRepeatDetails.style.display = 'block';
				this.weeklyStuff.style.display = 'block';
				this.durationStuff.style.display = 'block';
				this.weekendStuff.style.display = 'none';
				this.monthlyStuff.style.display = 'none';
				this.customStuff.style.display = 'none';

			} else if (theSelection === "MONTHLY_DATES") { // "monthly"
				this.allRepeatDetails.style.display = 'block';
				this.monthlyStuff.style.display = 'block';
				this.durationStuff.style.display = 'block';
				this.weekendStuff.style.display = 'block';
				this.weeklyStuff.style.display = 'none';
				this.customStuff.style.display = 'none';

			} else if (theSelection === "CUSTOM") { // "custom"
				this.allRepeatDetails.style.display = 'block';
				this.customStuff.style.display = 'block';
				this.weekendStuff.style.display = 'none';
				this.weeklyStuff.style.display = 'none';
				this.monthlyStuff.style.display = 'none';
				this.durationStuff.style.display = 'none';

			} else { // valid for hourly, daily, and yearly
				this.allRepeatDetails.style.display = 'block';
				this.weekendStuff.style.display = 'block';
				this.durationStuff.style.display = 'block';
				this.weeklyStuff.style.display = 'none';
				this.monthlyStuff.style.display = 'none';
				this.customStuff.style.display = 'none';
			}
		},

		doCustomAdd : function () {
			var msgHandler = new DefaultMessageHandler();
			var msgId = ""; // not specifying a message ID
			var msgSeverity = "error";
			var theMessageTitle = "";

			// Get the control where you want the dates to be listed.
			var theControl = this.selectedCustomDates.containerNode;

			// Retrieve the selected dates and times....
			var theDate = this.scheduleDate.get('value');
			var theTime = this.scheduleTime.get('value');

			// Construct a new Date object using the date from "theDate" and the time from "theTime" -- so that it can be added to the list....
			var customDate = new Date(theDate); // this is only taking the date into account....

			// retrieve the hours and minutes from the time field
			var hours = date.locale.format(theTime, {
				timePattern : "H",
				selector : "time"
			});
			var mins = date.locale.format(theTime, {
				timePattern : "mm",
				selector : "time"
			});
			// set the time field on our start date
			customDate.setHours(hours, mins);
			
			// Assume we can add this custom date to our list...
			var canAdd = true;

			// We can only add this date IF it does not already exist in the list.....
			for ( var i = 0; i < theControl.options.length; i++) {
				canAdd = canAdd && (date.compare((new Date(theControl.options.item(i).value)), customDate) !== 0);
			}

			if (canAdd) {
				// well, the date doesn't already exist in the list...now, we need to verify that it isn't set in the past.
				var currentDate = new Date();
				if (date.compare(currentDate, customDate) >= 0) {
					// the custom date is either in the past or is the current time....don't add it
					canAdd = false;
				}
			} else {// the date existed in the list already..... error!
				theMessageTitle = this.persistResources.CUSTOM_DATE_ALREADY_SELECTED;
				msgHandler.displayMessage(msgId, msgSeverity, theMessageTitle);
				return;
			}

			// If the date is good (it's not already in the list)...then add it to the list.
			if (canAdd) {
				construct.create("option", {
					value : customDate,
					innerHTML : customDate
				}, theControl);
			} else {
				theMessageTitle = this.persistResources.CUSTOM_DATE_IN_PAST;
				msgHandler.displayMessage(msgId, msgSeverity, theMessageTitle);
				return;
			}

		},

		doCustomRemove : function () {
			// Get the control where you want the custom dates to be removed.
			var theControl = this.selectedCustomDates.containerNode;
			
			// find out which one is selected
			var theIndex = theControl.selectedIndex;

			// While we have an index to a selected object,
			while (theIndex != -1) {
				// try to remove the object
				theControl.remove(theIndex);				
				// find out which one is now selected (if any)
				theIndex = theControl.selectedIndex;
			}// end loop through the list.	

		},

		doMonthlyAdd : function () {
			var msgHandler = new DefaultMessageHandler();
			var msgId = ""; // not specifying a message ID
			var msgSeverity = "error";
			var theMessageTitle = "";

			// Access the list of custom dates
			var theControl = this.selectedMonthlyDates.containerNode;
			// Retrieve the selected day of the month
			var theSelection = this.dayOfMonthSelection;

			var theDayOfMonthSelectonVal=this.getDayOfMonthSelectionValue();

			// Assume we can add this monthly date to our list...
			var canAdd = true;

			// We can only add this date IF it does not already exist in the list.....
			for ( var i = 0; i < theControl.options.length; i++) {
				canAdd = canAdd && ((theControl.options.item(i).value) !== theDayOfMonthSelectonVal);
			}

			// If the date of the month is good (it's not already in the list)...then add it to the list.
			if (canAdd) {
				construct.create("option", {
					value : theDayOfMonthSelectonVal,
					innerHTML : theSelection.get('displayedValue')
				}, theControl);
			} else {// the date existed in the list already..... error!
				theMessageTitle = this.persistResources.MONTHLY_DATE_ALREADY_SELECTED;
				msgHandler.displayMessage(msgId, msgSeverity, theMessageTitle);
				return;

			}
		},

		getDayOfMonthSelectionValue: function (){
			var theSelection = this.dayOfMonthSelection.get('value');
			if(lang.isArray(theSelection)){
				theSelection=theSelection[0];
			}
			return theSelection;
		},
		
		getRepeatSelectionValue: function (){
			var theSelection = this.repeatSelection.get('value');
			if(lang.isArray(theSelection)){
				theSelection=theSelection[0];
			}
			return theSelection;
		},
		
		getWeekEndOptionValue: function (){
			var theSelection = this.weekendOption.get('value');
			if(lang.isArray(theSelection)){
				theSelection=theSelection[0];
			}
			return theSelection;
		},
		
		getDurationRepeatForUnitsValue: function (){
			var theSelection = this.repeatForUnits.get('value');
			if(lang.isArray(theSelection)){
				theSelection=theSelection[0];
			}
			return theSelection;
		},
		
		getDurationRepeatForValue: function (){
			var theSelection = this.repeatFor.get('value');
			if(lang.isArray(theSelection)){
				theSelection=theSelection[0];
			}
			return theSelection;
		},


		doMonthlyRemove : function () {
			// Get the control where you want the monthly dates to be removed.
			var theControl = this.selectedMonthlyDates.containerNode;

			// find out which one is selected
			var theIndex = theControl.selectedIndex;

			// While we have an index to a selected object,
			while (theIndex != -1) {
				// try to remove the object
				theControl.remove(theIndex);				
				// find out which one is now selected (if any)
				theIndex = theControl.selectedIndex;
			}// end loop through the list.			
		}
	});
});

},
'ibm/usmi/widgets/finder/TaskCategory':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
// Finder TaskCategory
define([ "dojo/_base/declare"
], function (declare) {

	return declare("itme.finder.TaskCategory", null, {

		// private members       
		_category : null,
		_ids : null,
		_names : null,
		_icons : null,
		_selectedIndexes : null,
		_currentArrayIndex : 0,
		_currentSelectedIndex : 0,

		constructor : function (category) {
			this._category = category;
			this._ids = [];
			this._names = [];
			this._selectedIndexes = [];
			this._icons = [];
		},
		add : function (id, name, icon) {
			this._ids[this._currentArrayIndex] = id;
			this._names[this._currentArrayIndex] = name;
			this._icons[this._currentArrayIndex] = icon;
			this._currentArrayIndex = this._currentArrayIndex + 1;
			return this._currentArrayIndex - 1;
		},
		getCategory : function () {
			return this._category;
		},
		clearSelection : function () {
			this._selectedIndexes = null;
			this._currentSelectedIndex = 0;
			this._selectedIndexes = [];
		},
		addSelected : function (index) {
			this._selectedIndexes[this._currentSelectedIndex] = index;
			this._currentSelectedIndex = this._currentSelectedIndex + 1;
		},
		getSelectedCount : function () {
			return this._currentSelectedIndex;
		},
		getSelectedName : function (index) {
			return this._names[this._selectedIndexes[index]];
		},
		getSelectedIcon : function (index) {
			return this._icons[this._selectedIndexes[index]];
		},
		getSelectedId : function (index) {
			return this._ids[this._selectedIndexes[index]];
		},
		getIds : function () {
			return this._ids;
		},
		destroy : function () {
			this._ids.length = 0;
			this._names.length = 0;
			this._selectedIndexes.length = 0;
			this._icons.length = 0;
		}
	});
});
},
'ibm/usmi/widgets/wizard/DirectorBorderContainer':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
/*global define */
define("ibm/usmi/widgets/wizard/DirectorBorderContainer", [ "dojo/_base/declare",
	"dojo/dom-style",
	"dijit/layout/BorderContainer" ], function (
	declare, domstyle, BorderContainer) {
	// I declare that I'm support.wizard.DirectorBorderContainer, and I am extending dijit.layout.BorderContainer
	declare([ BorderContainer ], {

		// TODO:  need to determine why I need to update the default BorderContainer resize -- to pad to 325px
		resize : function (
			newSize, currentSize) {
			/*
			 * var node = this.domNode; node.setAttribute('style', "width: auto;
			 * height: auto; padding 325px"); this.inherited(arguments);
			 * 
			 * if(!this.inherited.cs || !this.inherited.pe){ var node =
			 * this.domNode;
			 * 
			 * var myStyle = domStyle; // this.inherited.cs =
			 * domStyle.getComputedStyle(node); // this.inherited.pe =
			 * domGeometry.getPadExtents(node, this.inherited.cs); //
			 * this.inherited.pe.r = domStyle.toPixelValue(node,
			 * this.inherited.cs.paddingRight); // this.inherited.pe.b =
			 * domStyle.toPixelValue(node, this.inherited.cs.paddingBottom); //
			 * domStyle.set(node, "padding", "325px");
			 * node.setAttribute('style', "width: auto; height: auto; padding
			 * 325px"); }
			 */
			// call my parent....
			this.inherited(arguments);
		}

	});
});

},
'ibm/tivoli/navmodel/WidgetGroupInfo':function(){
// wrapped by build app
define("ibm/tivoli/navmodel/WidgetGroupInfo", ["dijit","dojo","dojox"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */
dojo.provide("ibm.tivoli.navmodel.WidgetGroupInfo"); 

////////////////////////////////////
// WidgetGroupInfo
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.WidgetGroupInfo", null, {
	// private members       
	_widgetGroup: null,
	_version: null,
	
	/*=====
	constructor_kwArgs: function(widgetGroup, version) {
		// widgetGroup: Object
		//		json widget group information object
		// version: Float
		//		optional: version used to communicate with rest service (i.e. json version)
	},
	=====*/
	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs Provider object.
		this._version = kwArgs.version;
		this._widgetGroup = kwArgs.widgetGroup;
		if (!this._widgetGroup) {
			this._widgetGroup = {};
		}
	},

	getAttribute: function(key) {
		// summary:
		//		Returns general attribute for requested key
		return this._widgetGroup[key];
	},	
	
	getId: function() {
		// summary:
		//		Returns the id of this definition
		return this._widgetGroup.id;
	},
	
	getLabel: function() {
		// summary:
		//		Returns the widget group label
		return this._widgetGroup.label;
	},
	
	getDescription: function() {
		// summary:
		//		Returns the widget group description
		return this._widgetGroup.description;
	},
	
	getProviderType: function() {
		// summary:
		//		Returns the widget group provider type
		return this._widgetGroup.providerType;
	},
	
	getDatasourceType: function() {
		// summary:
		//		Returns the widget group datasource type
		return this._widgetGroup.datasourceType;
	}
});

});

},
'ibm/tivoli/tip/dijit/CMSMenuProvider':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!ibm/tivoli/tip/dijit/TIPContextMenuProvider"], function(dijit,dojo,dojox){


/**
 *  The CMSMenuProvider is a specialized menu provider for use with the 
 *  context menu service.  Certainly you may use this provider if not using CMS,
 *  but it returns extra data that is used for CMS.  The CMS provider will
 *  make requests to the server to obtain both the menu items as well as the
 *  menu commands.
 *
 *  The CMSMenuProvider expects the following to be included in the context 
 *  passed to contextMenuOpen
 *  {
 *     selectedData: <the selection data>
 *  }
 */

dojo.provide("ibm.tivoli.tip.dijit.CMSMenuProvider");
dojo.require("ibm.tivoli.tip.dijit.TIPContextMenuProvider");
 
dojo.declare("ibm.tivoli.tip.dijit.CMSMenuProvider", 
             ibm.tivoli.tip.dijit.TIPContextMenuProvider, 
{
   global: null,
   rcfItems: null, 
   menuName: null, 
   parentid: null,
   // This menu uses the same getCMSMenus command, but on the server, we use a different
   // data provider for this version of the CMS menu provider.
   getMenusCommandName: "getCMSMenus",
   getMenuActionCommandName: "getMenuCommand",
   
   
   // When the following is true, then this provider will first look 
   // at the menu items for a launchType, and if one exists, it will
   // execute that action here instead of going back to the host.
   hasLocalActions: false, 

	/* Default timeout for requests made to the server */
	requestTimeout: 30000,

   getIdPrefix: function() 
   {
       return this.id_;
   },
   
   setParentId: function (parentid) 
   {
      this.parentid = parentid;
   },
   
   getParentId: function ()
   {
     return this.parentid;
   },
   
   setResourceContextFilters: function (rcfItems) 
   {
      this.rcfItems = rcfItems;
   },
   
   getRCFURL: function (kw) 
   {
      if (this.rcfItems) 
      {
         for (var i in this.rcfItems) 
         {
            if (i) 
            {
               for (var name in this.rcfItems[i]) 
               {
                  if (name) 
                  {
                       kw.content["rcf_" + name]=this.rcfItems[i][name];
                  }
               }
            }
         }
      }   
   },

   getMenuItems: function(context) 
   {
   	console.log ("enter getMenuItems with context: ");
   	console.log (context);
   	
      var kw = null;

      kw = {
              sync:         true,
              noBusyDialog: true,
              content:      {
                               command:              this.getMenusCommandName,
                               arg_idPrefix:         this.getIdPrefix(),
                               arg_portletNamespace: this.global.namespace,
                               arg_parentId:         this.parentid
                            },
              timeout:      this.requestTimeout,
              onResponse:   {
                               callback: dojo.hitch( this, function( response ) { this.menus_ = response; } )
                            }
            };

      this.dataToURL (context.selectedData,kw);
      
      this.getRCFURL(kw);
         
      if (this.menuName) 
      {
         kw.content.arg_menuName = this.menuName;
      }
      
      ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );

      return this.menus_;
   },
   
   getMenuCommand: function (commandName, context, menuItem) 
   {
      var result = null;
      
      if (this.getMenuCommandCallback !== null) 
      {
         result = this.getMenuCommandCallback(commandName, context, menuItem);
         this.handleMenuAction(result);
      }
      else
      {
         this.getMenuCommandNoCallback(commandName, context, menuItem);
      }
   },

   getMenuCommandNoCallback: function (commandName, context, menuItem) 
   {
      var actionHandled = false;
      if (this.hasLocalActions === true)
      {
         // Check if the menu item contains a launchType.  If so,
         // handle it locally.
         var result = this.getItem(commandName);
         if (result && result.launchType)  // FIXME: look for something other than launch type
         {
             actionHandled = true;
             this.handleMenuAction(result);
         }
      }
      
      if (actionHandled === false)
      {
         this.getMenuCommandFromServer(commandName, context, menuItem);
      }
   },

   getMenuCommandFromServer: function (commandName, context, menuItem) 
   {
      var kw = {
                  content:   {
                                command: this.getMenuActionCommandName,
                                arg_menuCommandName: commandName,
                                arg_idPrefix: this.getIdPrefix(),
                                arg_portletNamespace: this.global.namespace,
                                arg_parentId: this.parentid
                             },
                  timeout:   this.requestTimeout,
                  onResponse:{
                                callback: dojo.hitch(this, this.handleMenuAction)
                             }
               };
               
       this.dataToURL (context.selectedData,kw);
       this.getRCFURL(kw);
       
       if (this.menuName) 
       {
          kw.content.arg_menuName = this.menuName;
       }
      
       ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );
   },   
   
   dataToURL: function (selectedData, kw) 
   {
   	  console.log ("enter dataToURL: " + selectedData);
      // if you get an exception here, it means you did not provide
      // selection data in your context.  I'll put a catch of the 
      // exception here because I bet this happens a lot.
      var columns = null; 
      try 
      {
         columns = selectedData.columns;
      } catch (ex) {
         console.error ("Caught javascript exception in cmsContextMenuProvider.dataToURL: " + ex);
         console.error ("You must provide 'selectedData' on your context.");
         alert (ex);
      }
      var rows = selectedData.rows;
      var row = null;
      if (this.isArray (rows)) {
         row = rows[0];
      } else {
         row = rows;
      }
      
      for (var i=0; i<columns.length; i++) 
      {
         var rowField = columns[i].field;
         var value = row[columns[i].field];

         //if (!value) //value that of numberic type and takes 0
         if (value==undefined || value==null)
         {
            value = "";
         }

         if (columns[i].isNamingAttribute === "true") 
         {
            kw.content["rn_"+columns[i].field]=value;
         } else {
            kw.content["sv_"+columns[i].field]=value;
         }
      }
         
      if (row.id) 
      {
        kw.content.arg_rowId = row.id;
      }
   }
   
});
});

},
'dojo/store/util/QueryResults':function(){
define(["../../_base/array", "../../_base/lang", "../../_base/Deferred"
], function(array, lang, Deferred) {
  //  module:
  //    dojo/store/util/QueryResults
  //  summary:
  //    The module defines a query results wrapper

var util = lang.getObject("dojo.store.util", true);

util.QueryResults = function(results){
	// summary:
	//		A function that wraps the results of a store query with additional
	//		methods.
	//
	// description:
	//		QueryResults is a basic wrapper that allows for array-like iteration
	//		over any kind of returned data from a query.  While the simplest store
	//		will return a plain array of data, other stores may return deferreds or
	//		promises; this wrapper makes sure that *all* results can be treated
	//		the same.
	//
	//		Additional methods include `forEach`, `filter` and `map`.
	//
	// returns: Object
	//		An array-like object that can be used for iterating over.
	//
	// example:
	//		Query a store and iterate over the results.
	//
	//	|	store.query({ prime: true }).forEach(function(item){
	//	|		//	do something
	//	|	});

	if(!results){
		return results;
	}
	// if it is a promise it may be frozen
	if(results.then){
		results = lang.delegate(results);
	}
	function addIterativeMethod(method){
		if(!results[method]){
			results[method] = function(){
				var args = arguments;
				return Deferred.when(results, function(results){
					Array.prototype.unshift.call(args, results);
					return util.QueryResults(array[method].apply(array, args));
				});
			};
		}
	}
	addIterativeMethod("forEach");
	addIterativeMethod("filter");
	addIterativeMethod("map");
	if(!results.total){
		results.total = Deferred.when(results, function(results){
			return results.length;
		});
	}
	return results;
};

return util.QueryResults;
});

},
'gridx/modules/dnd/Row':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/Deferred",
	"dojo/_base/lang",
	"dojo/dom-class",
	"dojo/dom-geometry",
	"dojo/_base/sniff",
	"./_Base",
	"../../core/_Module"
], function(declare, array, Deferred, lang, domClass, domGeometry, sniff, _Base, _Module){

	var hitch = lang.hitch,
		forEach = array.forEach;

	function getSourceData(source, nodes){
		if(source.grid){
			var d = new Deferred(),
				success = hitch(d, d.callback),
				fail = hitch(d, d.errback),
				dataArr = [],
				sg = source.grid,
				rowIds = sg.dnd.row._selectedRowIds;
			sg.model.when({id: rowIds}, function(){
				forEach(rowIds, function(id){
					var idx = sg.model.idToIndex(id),
						row = sg.model.byId(id);
					if(row){
						dataArr.push(lang.clone(row.rawData));
					}
				});
			}).then(function(){
				success(dataArr);
			}, fail);
			return d;
		}else{
			return source.getGridDndRowData && source.getGridDndRowData(nodes) || [];
		}
	}

	return declare(/*===== "gridx.modules.dnd.Row", =====*/_Base, {
		// summary:
		//		This module provides an implementation of row drag & drop.
		//		It supports row reordering within grid, dragging out of grid, and dragging into grid.

		name: 'dndRow',
		
		required: ['_dnd', 'moveRow'],

		getAPIPath: function(){
			return {
				dnd: {
					row: this
				}
			};
		},

		//Public---------------------------------------------------------------------------
		// accept: String[]
		//		Can drag in what kind of stuff
		accept: ['grid/rows'],

		// provide: String[]
		//		Can drag out what kind of stuff
		provide: ['grid/rows'],

		onDraggedOut: function(targetSource){
			var t = this,
				targetAccept = [];
			if(targetSource.grid){
				targetAccept = targetSource.grid.dnd._dnd.profile.arg('accept');
			}else{
				for(var n in targetSource.accept){
					targetAccept.push(n);
				}
			}
			if(!t.checkArg('copyWhenDragOut', targetAccept)){
				var g = t.grid,
					m = g.model,
					s = g.store,
					rowIds = t._selectedRowIds;
				if(s.fetch){
					var items = [];
					g.model.when({id: rowIds}, function(){
						forEach(rowIds, function(id){
							var row = m.byId(id);
							if(row){
								items.push(row.item);
							}
						});
					}).then(function(){
						forEach(items, s.deleteItem, s);
						s.save();
					});
				}else{
					forEach(rowIds, s.remove, s);
				}
			}
		},
	
		//Package-----------------------------------------------------------------------------------
		_checkDndReady: function(evt){
			var t = this, m = t.model;
			if(!m.getMark || m.getMark(evt.rowId)){
				t.grid.dnd._dnd.profile = t;
				t._selectedRowIds = m.getMarkedIds ? m.getMarkedIds() : [evt.rowId];
				return true;
			}
			return false;
		},

		//Private-----------------------------------------------------------------------------
		_cssName: 'Row',

		_onBeginDnd: function(source){
			source.delay = this.arg('delay');
		},

		_getDndCount: function(){
			return this._selectedRowIds.length;
		},

		_onEndDnd: function(){},

		_buildDndNodes: function(){
			var gid = this.grid.id;
			return array.map(this._selectedRowIds, function(rowId){
				return ["<div id='", gid, '_dndrow_', rowId, "' gridid='", gid, "' rowid='", rowId, "'></div>"].join('');
			}).join('');
		},

		_onBeginAutoScroll: function(){
			var autoScroll = this.grid.autoScroll;
			this._autoScrollH = autoScroll.horizontal;
			autoScroll.horizontal = false;
		},

		_onEndAutoScroll: function(){
			this.grid.autoScroll.horizontal = this._autoScrollH;
		},

		_getItemData: function(id){
			return id.substring((this.grid.id + '_dndrow_').length);
		},
		
		//----------------------------------------------------------------------------
		_calcTargetAnchorPos: function(evt, containerPos){
			var t = this,
				node = evt.target,
				body = t.grid.body,
				ret = {
					width: containerPos.w + "px",
					height: '',
					left: ''
				},
				isSelected = function(n){
					return t.model.getMark && t.model.getMark(n.getAttribute('rowid'));
				},
				getVIdx = function(n){
					return parseInt(n.getAttribute('visualindex'), 10);
				},
				calcPos = function(node){
					var n = node, first = n, last = n;
					if(isSelected(n)){
						var prenode = n.previousSibling;
						while(prenode && isSelected(prenode)){
							n = prenode;
							prenode = prenode.previousSibling;
						}
						first = n;
						n = node;
						var nextnode = n.nextSibling;
						while(nextnode && isSelected(nextnode)){
							n = nextnode;
							nextnode = nextnode.nextSibling;
						}
						last = n;
					}
					if(first && last){
						var firstPos = domGeometry.position(first),
							lastPos = domGeometry.position(last),
							middle = (firstPos.y + lastPos.y + lastPos.h) / 2;
						if(evt.clientY < middle){
							t._target = getVIdx(first);
							ret.top = (firstPos.y - containerPos.y) + "px";
						}else{
							t._target = getVIdx(last) + 1;
							ret.top = (lastPos.y + lastPos.h - containerPos.y) + "px";
						}
					}else{
						delete t._target;
					}
					return ret;
				};
			if(!sniff('ff')){
				//In FF, this conflicts with the overflow:hidden css rule for grid row DIV, which is required by ColumnLock.
				while(node){
					if(domClass.contains(node, 'gridxRow')){
						return calcPos(node);
					}
					node = node.parentNode;
				}
			}
			var bn = body.domNode,
				nodes = bn.childNodes;
			if(!nodes.length){
				ret.top = '0px';
				t._target = 0;
			}else{
				node = bn.firstChild;
				var idx = getVIdx(node),
					pos = domGeometry.position(node);
				if(idx === 0 && evt.clientY <= pos.y + pos.h){
					ret.top = (pos.y - containerPos.y) + 'px';
					t._target = 0;
				}else{
					node = bn.lastChild;
					idx = getVIdx(node);
					pos = domGeometry.position(node);
					if(idx === body.visualCount - 1 && evt.clientY > pos.y + pos.h){
						ret.top = (pos.y + pos.h - containerPos.y) + 'px';
						t._target = body.visualCount;
					}else{
						var rowFound = array.some(nodes, function(rowNode){
							pos = domGeometry.position(rowNode);
							if(pos.y <= evt.clientY && pos.y + pos.h >= evt.clientY){
								node = rowNode;
								return true;
							}
						});
						return rowFound ? calcPos(node) : null;
					}
				}
			}
			return ret;
		},

		_onDropInternal: function(nodes, copy){
			var t = this, g = t.grid;
			if(t._target >= 0){
				t.model.when({id: t._selectedRowIds}, function(){
					var indexes = array.map(t._selectedRowIds, function(rowId){
						return t.model.idToIndex(rowId);
					});
					g.move.row.move(indexes, g.body.getRowInfo({
						visualIndex: t._target
					}).rowIndex);
				});
			}
		},

		_onDropExternal: function(source, nodes, copy){
			var d = new Deferred(),
				success = hitch(d, d.callback),
				fail = hitch(d, d.errback),
				g = this.grid,
				target = this._target,
				targetRow, preRow,
				sourceData = getSourceData(source, nodes);
			g.model.when([target - 1, target], function(){
				targetRow = g.model.byIndex(target);
				preRow = g.model.byIndex(target - 1);
			}).then(function(){
				//Inserting and deleting (and other operations that changes store) are better to happen outside 
				//"model.when", because during "when", it is not allowed to clear cache.
				Deferred.when(sourceData, function(dataArr){
					if(dataArr && dataArr.length){
						var inserted = g.model.insert(dataArr, preRow && preRow.item, targetRow && targetRow.item);
						Deferred.when(inserted, success, fail);
					}
				}, fail);
			}, fail);
			return d;
		}
	});
});

},
'dojo/data/util/simpleFetch':function(){
define(["dojo/_base/lang", "dojo/_base/window", "./sorter"], 
  function(lang, winUtil, sorter) {
	// module:
	//		dojo/data/util/simpleFetch
	// summary:
	//		TODOC

var simpleFetch = lang.getObject("dojo.data.util.simpleFetch", true);

simpleFetch.fetch = function(/* Object? */ request){
	//	summary:
	//		The simpleFetch mixin is designed to serve as a set of function(s) that can
	//		be mixed into other datastore implementations to accelerate their development.
	//		The simpleFetch mixin should work well for any datastore that can respond to a _fetchItems()
	//		call by returning an array of all the found items that matched the query.  The simpleFetch mixin
	//		is not designed to work for datastores that respond to a fetch() call by incrementally
	//		loading items, or sequentially loading partial batches of the result
	//		set.  For datastores that mixin simpleFetch, simpleFetch
	//		implements a fetch method that automatically handles eight of the fetch()
	//		arguments -- onBegin, onItem, onComplete, onError, start, count, sort and scope
	//		The class mixing in simpleFetch should not implement fetch(),
	//		but should instead implement a _fetchItems() method.  The _fetchItems()
	//		method takes three arguments, the keywordArgs object that was passed
	//		to fetch(), a callback function to be called when the result array is
	//		available, and an error callback to be called if something goes wrong.
	//		The _fetchItems() method should ignore any keywordArgs parameters for
	//		start, count, onBegin, onItem, onComplete, onError, sort, and scope.
	//		The _fetchItems() method needs to correctly handle any other keywordArgs
	//		parameters, including the query parameter and any optional parameters
	//		(such as includeChildren).  The _fetchItems() method should create an array of
	//		result items and pass it to the fetchHandler along with the original request object
	//		-- or, the _fetchItems() method may, if it wants to, create an new request object
	//		with other specifics about the request that are specific to the datastore and pass
	//		that as the request object to the handler.
	//
	//		For more information on this specific function, see dojo.data.api.Read.fetch()
	request = request || {};
	if(!request.store){
		request.store = this;
	}
	var self = this;

	var _errorHandler = function(errorData, requestObject){
		if(requestObject.onError){
			var scope = requestObject.scope || winUtil.global;
			requestObject.onError.call(scope, errorData, requestObject);
		}
	};

	var _fetchHandler = function(items, requestObject){
		var oldAbortFunction = requestObject.abort || null;
		var aborted = false;

		var startIndex = requestObject.start?requestObject.start:0;
		var endIndex = (requestObject.count && (requestObject.count !== Infinity))?(startIndex + requestObject.count):items.length;

		requestObject.abort = function(){
			aborted = true;
			if(oldAbortFunction){
				oldAbortFunction.call(requestObject);
			}
		};

		var scope = requestObject.scope || winUtil.global;
		if(!requestObject.store){
			requestObject.store = self;
		}
		if(requestObject.onBegin){
			requestObject.onBegin.call(scope, items.length, requestObject);
		}
		if(requestObject.sort){
			items.sort(sorter.createSortFunction(requestObject.sort, self));
		}
		if(requestObject.onItem){
			for(var i = startIndex; (i < items.length) && (i < endIndex); ++i){
				var item = items[i];
				if(!aborted){
					requestObject.onItem.call(scope, item, requestObject);
				}
			}
		}
		if(requestObject.onComplete && !aborted){
			var subset = null;
			if(!requestObject.onItem){
				subset = items.slice(startIndex, endIndex);
			}
			requestObject.onComplete.call(scope, subset, requestObject);
		}
	};
	this._fetchItems(request, _fetchHandler, _errorHandler);
	return request;	// Object
};

return simpleFetch;
});

},
'gridx/modules/extendedSelect/_Base':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/query",
	"dojo/_base/connect",
	"dojo/_base/Deferred",
	"dojo/_base/sniff",
	"dojo/_base/window",
	"dojo/dom",
	"dojo/keys",
	"../../core/_Module",
	"../AutoScroll"
], function(declare, query, connect, Deferred, sniff, win, dom, keys, _Module){

	return declare(_Module, {
		required: ['autoScroll'],

		getAPIPath: function(){
			var path = {
				select: {}
			};
			path.select[this._type] = this;
			return path;
		},
		
		load: function(){
			var t = this, g = t.grid, doc = win.doc;
			g.domNode.setAttribute('aria-multiselectable', true);
			t._refSelectedIds = [];
			t.subscribe('gridClearSelection_' + g.id, function(type){
				if(type != t._type){
					t.clear();
				}
			});
			t.batchConnect(
				[g.body, 'onRender', '_onRender'],
				[doc, 'onmouseup', '_end'],
				[doc, 'onkeydown', function(e){
					if(e.keyCode == keys.SHIFT){
						dom.setSelectable(g.domNode, false);
					}
				}],
				[doc, 'onkeyup', function(e){
					if(e.keyCode == keys.SHIFT){
						dom.setSelectable(g.domNode, true);
					}
				}]
			);
			t._init();
			t.loaded.callback();
		},

		//Public ------------------------------------------------------------------
		enabled: true,

		holdingCtrl: false,

		holdingShift: false,

		selectById: function(/* id */){
			return this._subMark('_markById', arguments, true);
		},

		deselectById: function(/* id */){
			return this._subMark('_markById', arguments, false);
		},

		selectByIndex: function(/* start, end */){
			return this._subMark('_markByIndex', arguments, true);
		},

		deselectByIndex: function(/* start, end */){
			return this._subMark('_markByIndex', arguments, false);
		},

		onSelectionChange: function(/*newSelectedIds, oldSelectedIds*/){
			// summary:
			//		Event: fired when the selection is changed.
		},

		//Private -----------------------------------------------------------------
		_clear: function(){
			var t = this;
			delete t._lastToSelect;
			delete t._lastStartItem;
			delete t._lastEndItem;
		},

		_subMark: function(func, args, toSelect){
			var t = this;
			if(t.arg('enabled')){
				if(toSelect){
					connect.publish('gridClearSelection_' + t.grid.id, [t._type]);
				}
				t._lastSelectedIds = t.getSelected();
				t._refSelectedIds = [];
				return Deferred.when(t[func](args, toSelect), function(){
					t._onSelectionChange();
				});
			}
		},

		_start: function(item, extending, isRange){
			var t = this;
			if(!t._selecting && !t._marking && t.arg('enabled')){
				dom.setSelectable(t.grid.domNode, false);
				t._fixFF(1);
				var isSelected = t._isSelected(item);
				isRange = isRange || t.arg('holdingShift');
				if(isRange && t._lastStartItem){
					t._isRange = 1;	//1 as true
					t._toSelect = t._lastToSelect;
					t._startItem = t._lastStartItem;
					t._currentItem = t._lastEndItem;
				}else{
					t._startItem = item;
					t._currentItem = null;
					if(extending || t.arg('holdingCtrl')){
						t._toSelect = !isSelected;
					}else{
						t._toSelect = 1;	//1 as true
						t.clear(1);
					}
				}
				connect.publish('gridClearSelection_' + t.grid.id, [t._type]);
				t._beginAutoScroll();
				t.grid.autoScroll.enabled = true;
				t._lastSelectedIds = t.getSelected();
				t._selecting = 1;	//1 as true
				t._highlight(item);
			}
		},

		_highlight: function(target){
			var t = this;
			if(t._selecting){
				var type = t._type,
					start = t._startItem,
					current = t._currentItem,
					highlight = function(from, to, toHL){
						from = from[type];
						to = to[type];
						var dir = from < to ? 1 : -1;
						for(; from != to; from += dir){
							var item = {};
							item[type] = from;
							t._highlightSingle(item, toHL);
						}
					};
				if(current === null){
					//First time select.
					t._highlightSingle(target, 1);	//1 as true
				}else{
					if(t._inRange(target[type], start[type], current[type])){
						//target is between start and current, some selected should be deselected.
						highlight(current, target, 0);	//0 as false
					}else{
						if(t._inRange(start[type], target[type], current[type])){
							//selection has jumped to different direction, all should be deselected.
							highlight(current, start, 0);	//0 as false
							current = start;
						}
						highlight(target, current, 1);	//1 as true
					}
				}
				t._currentItem = target;
				t._focus(target);
			}
		},

		_end: function(){
			var t = this, g = t.grid;
			if(t._selecting){
				t._fixFF();
				t._endAutoScroll();
				t._selecting = 0;	//0 as false
				t._marking = 1;	//1 as true
				g.autoScroll.enabled = false;
				var d = t._addToSelected(t._startItem, t._currentItem, t._toSelect);
				t._lastToSelect = t._toSelect;
				t._lastStartItem = t._startItem;
				t._lastEndItem = t._currentItem;
				t._startItem = t._currentItem = t._isRange = null;
				Deferred.when(d, function(){
					dom.setSelectable(g.domNode, true);
					t._marking = 0;	//0 as false
					t._onSelectionChange();
				});
			}
		},

		_highlightSingle: function(target, toHighlight){
			toHighlight = toHighlight ? this._toSelect : this._isSelected(target);
			this._doHighlight(target, toHighlight);
		},

		_onSelectionChange: function(){
			var t = this, selectedIds = t.getSelected();
			t.onSelectionChange(selectedIds, t._lastSelectedIds);
			t._lastSelectedIds = selectedIds;
		},

		_inRange: function(value, start, end, isClose){
			return ((value >= start && value <= end) || (value >= end && value <= start)) && (isClose || value != end);
		},

		_fixFF: function(isStart){
			if(sniff('ff')){
				query('.gridxSortNode', this.grid.headerNode).style('overflow', isStart ? 'visible' : '');
			}
		}
	});
});

},
'gridx/modules/select/Row':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/sniff",
	"dojo/_base/query",
	"dojo/dom-class",
	"dojo/keys",
	"./_RowCellBase",
	"../../core/_Module"
], function(declare, array, sniff, query, domClass, keys, _RowCellBase, _Module){

	return declare(/*===== "gridx.modules.select.Row", =====*/_RowCellBase, {
		// summary:
		//		Provides simple row selection.
		// description:
		//		This module provides a simple way for selecting rows by clicking or SPACE key, or CTRL + Click to select multiple rows.
		//
		// example:
		//		1. Use select api on grid row object obtained from grid.row(i)
		//		|	grid.row(1).select();
		//		|	grid.row(1).deselect();
		//		|	grid.row(1).isSelected();
		//
		//		2. Use select api on select.row module
		//		|	grid.select.row.selectById(rowId);
		//		|	grid.select.row.deSelectById(rowId);
		//		|	grid.select.row.isSelected(rowId);
		//		|	grid.select.row.getSelected();//[]
		//		|	grid.select.row.clear();

		// name: [readonly] String
		//		module name
		name: "selectRow",
		
		// rowMixin: Object
		//		A map of functions to be mixed into grid row object, so that we can use select api on row object directly
		//		- grid.row(1).select() | deselect() | isSelected();
		rowMixin: {
			select: function(){
				this.grid.select.row.selectById(this.id);
				return this;
			},

			deselect: function(){
				this.grid.select.row.deselectById(this.id);
				return this;
			},

			isSelected: function(){
				return this.grid.select.row.isSelected(this.id);
			}
		},
		
		//Public API--------------------------------------------------------------------------------
		
		// triggerOnCell: [readonly] Boolean
		//		Whether row will be selected by clicking on cell, false by default
		triggerOnCell: false,

		// treeMode: Boolean
		//		Whether to apply tri-state selection for child rows.
		treeMode: true,

/*=====
		selectById: function(rowId){
			// summary:
			//		Select a row by id.
		},
		
		deselectById: function(rowId){
			// summary:
			//		Deselect a row by id.
		},
		
		isSelected: function(rowId){
			// summary:
			//		Check if a row is already selected.
		},
=====*/

		getSelected: function(){
			// summary:
			//		Get id array of all selected rows
			return this.model.getMarkedIds();
		},
		
		clear: function(){
			// summary:
			//		Deselected all selected rows;
			if(this.arg('enabled')){
				var model = this.model;
				array.forEach(model.getMarkedIds(), function(id){
					model.markById(id, 0);
				});
				model.when();
			}
		},

/*=====
		onSelected: function(row){
			// summary:
			//		Fired when a row is selected.
			// row: gridx.core.Row
			//		The row object
		},

		onDeselected: function(row){
			// summary:
			//		Fired when a row is deselected.
			// row: gridx.core.Row
			//		The row object
		},

		onHighlightChange: function(){
			// summary:
			//		Fired when a row's highlight is changed.
			// tags:
			//		private package
		},
=====*/

		//Private--------------------------------------------------------------------------------
		_type: 'row',

		_init: function(){
			var t = this,
				g = t.grid;
			t.model.treeMarkMode('', t.arg('treeMode'));
			t.inherited(arguments);
			t.model._spTypes.select = 1;
			t.batchConnect(
				[g, 'onRowMouseDown', function(e){
					//Have to check whether we are on the 
					if((t.arg('triggerOnCell') &&
						!domClass.contains(e.target, 'gridxTreeExpandoIcon') &&
						!domClass.contains(e.target, 'gridxTreeExpandoInner')) ||
						!e.columnId){
						t._select(e.rowId, g._isCopyEvent(e));
					}
				}],
				[g, sniff('ff') < 4 ? 'onRowKeyUp' : 'onRowKeyDown', function(e){
					if((t.arg('triggerOnCell') || !e.columnId) && e.keyCode == keys.SPACE){
						t._select(e.rowId, g._isCopyEvent(e));
					}
				}]);
		},

		_onMark: function(id, toMark, oldState, type){
			if(type == 'select'){
				var t = this;
				t._highlight(id, toMark);
				//Fire event when the row is loaded, so users can use the row directly.
				t.model.when({id: id}, function(){
					t[toMark ? 'onSelected' : 'onDeselected'](t.grid.row(id, 1));	//1 as true
				});
			}
		},
		
		_highlight: function(rowId, toHighlight){
			var nodes = query('[rowid="' + rowId + '"]', this.grid.mainNode),
				selected = toHighlight && toHighlight != 'mixed';
			if(nodes.length){
				nodes.forEach(function(node){
					domClass.toggle(node, "gridxRowSelected", selected);
					domClass.toggle(node, "gridxRowPartialSelected", toHighlight == 'mixed');
					node.setAttribute('aria-selected', !!selected);
				});
				this.onHighlightChange({row: parseInt(nodes[0].getAttribute('visualindex'), 10)}, toHighlight);
			}
		},

		_markById: function(id, toMark){
			var m = this.model;
			if(m.treeMarkMode() && !m.getMark(id) && toMark){
				toMark = 'mixed';
			}
			m.markById(id, toMark);
			m.when();
		},

		_onRender: function(start, count){
			var t = this,
				model = t.model,
				end = start + count,
				i, id, rowNode;
			for(i = start; i < end; ++i){
				rowNode = t.grid.body.getRowNode({visualIndex: i});
				if(rowNode){
					id = rowNode.getAttribute('rowid');
					t._highlight(id, model.getMark(id));
				}
			}
		}
	});
});

},
'url:idx/oneui/templates/Menu.html':"<table class=\"dijit dijitMenu dijitMenuPassive dijitReset dijitMenuTable\" role=\"menu\" tabIndex=\"${tabIndex}\" data-dojo-attach-event=\"onkeypress:_onKeyPress\" cellspacing=\"0\">\r\n\t<tbody class=\"dijitReset\">\r\n\t\t<tr data-dojo-attach-point=\"_columnContainerNode\">\r\n\t\t\t<td class=\"dijitReset oneuiMenuColumn\" data-dojo-attach-point=\"columnNodes\">\r\n\t\t\t\t<table class=\"dijitReset\" cellspacing=\"0\" width=\"100%\" role=\"presentation\">\r\n\t\t\t\t\t<tbody class=\"dijitReset\" data-dojo-attach-point=\"_containerNodes,containerNode\">\r\n<!-- this is column 0, which also starts out as the container node so menu items are initially loaded here.\r\n     containerNode changes to point to _columnContainerNode once the widget has initialised, so the whole set of columns is the container.\r\n\t this must be kept in synch with _MenuColumn.html -->\r\n\t\t\t\t\t</tbody>\r\n\t\t\t\t</table>\r\n\t\t\t</td>\r\n\t\t</tr>\r\n\t</tbody>\r\n</table>\r\n",
'ibm/usmi/core/framework/ContentTitle':function(){
require({cache:{
'url:ibm/usmi/core/framework/templates/ContentTitle.html':"<div data-dojo-attach-point=\"titleContainer\" class=\"contentTitleContainer\">\r\n<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n\t<span class=\"contentTitle\"></span>\r\n\t<span class=\"contentStatus\"></span>\r\n</div>\r\n"}});
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/core/framework/ContentTitle", [
	"require",
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dijit/_WidgetBase",
	"dijit/_TemplatedMixin",
	"dojo/text!./templates/ContentTitle.html",
	"dojo/_base/lang",
	"dojo/topic",
	"dojo/query"
], function (require, dojo, declare, WidgetBase, TemplatedMixin, template, lang, topic, query) {
	return declare([
		WidgetBase,
		TemplatedMixin
	], {

		// Path to the template
		templateString : template,

		baseClass : "contentTitle",

		productName : "product name",

		constructor : function (args) {
			dojo.safeMixin(this, args);
		},

		postCreate : function () {
			this.inherited(arguments);

			topic.subscribe("updateTitle", lang.hitch(this, this.updateTitle));
			topic.subscribe("reTarget", lang.hitch(this, this.reTarget));
		},

		updateTitle : function (subtitles) {
			var title;
			if (subtitles.resource) {
				title = subtitles.resource;
			}
			if (subtitles.view) {
				if (title) {
					title += ": ";
				} else {
					title = "";
				}
				title += subtitles.view;
			}

			var titleContainer = this.get("titleContainer");
			var titleElement = query(".contentTitle", titleContainer);
			titleElement[0].innerHTML = title;
			document.title = title + " - " + this.productName;

			var status = "";
			if (subtitles.loading) {
				status = "(loading)";
			}
			titleElement = query(".contentStatus", titleContainer);
			titleElement[0].innerHTML = status;
		},

		reTarget : function (dataset) {
			var title = dataset._dataset.label; // need way to get title w/o using private - Aaron?
			this.updateTitle({
				resource : title
			});
		}
	});
});

},
'dijit/form/ComboBoxMixin':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/_base/Deferred",
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/lang", // lang.mixin
	"dojo/store/util/QueryResults",	// dojo.store.util.QueryResults
	"./_AutoCompleterMixin",
	"./_ComboBoxMenu",
	"../_HasDropDown",
	"dojo/text!./templates/DropDownBox.html"
], function(declare, Deferred, kernel, lang, QueryResults, _AutoCompleterMixin, _ComboBoxMenu, _HasDropDown, template){

/*=====
	var _AutoCompleterMixin = dijit.form._AutoCompleterMixin;
	var _ComboBoxMenu = dijit.form._ComboBoxMenu;
	var _HasDropDown = dijit._HasDropDown;
=====*/

	// module:
	//		dijit/form/ComboBoxMixin
	// summary:
	//		Provides main functionality of ComboBox widget

	return declare("dijit.form.ComboBoxMixin", [_HasDropDown, _AutoCompleterMixin], {
		// summary:
		//		Provides main functionality of ComboBox widget

		// dropDownClass: [protected extension] Function String
		//		Dropdown widget class used to select a date/time.
		//		Subclasses should specify this.
		dropDownClass: _ComboBoxMenu,

		// hasDownArrow: Boolean
		//		Set this textbox to have a down arrow button, to display the drop down list.
		//		Defaults to true.
		hasDownArrow: true,

		templateString: template,

		baseClass: "dijitTextBox dijitComboBox",

		/*=====
		// store: [const] dojo.store.api.Store || dojo.data.api.Read
		//		Reference to data provider object used by this ComboBox.
		//
		//		Should be dojo.store.api.Store, but dojo.data.api.Read supported
		//		for backwards compatibility.
		store: null,
		=====*/

		// Set classes like dijitDownArrowButtonHover depending on
		// mouse action over button node
		cssStateNodes: {
			"_buttonNode": "dijitDownArrowButton"
		},

		_setHasDownArrowAttr: function(/*Boolean*/ val){
			this._set("hasDownArrow", val);
			this._buttonNode.style.display = val ? "" : "none";
		},

		_showResultList: function(){
			// hide the tooltip
			this.displayMessage("");
			this.inherited(arguments);
		},

		_setStoreAttr: function(store){
			// For backwards-compatibility, accept dojo.data store in addition to dojo.store.store.  Remove in 2.0.
			if(!store.get){
				lang.mixin(store, {
					_oldAPI: true,
					get: function(id){
						// summary:
						//		Retrieves an object by it's identity. This will trigger a fetchItemByIdentity.
						//		Like dojo.store.DataStore.get() except returns native item.
						var deferred = new Deferred();
						this.fetchItemByIdentity({
							identity: id,
							onItem: function(object){
								deferred.resolve(object);
							},
							onError: function(error){
								deferred.reject(error);
							}
						});
						return deferred.promise;
					},
					query: function(query, options){
						// summary:
						//		Queries the store for objects.   Like dojo.store.DataStore.query()
						//		except returned Deferred contains array of native items.
						var deferred = new Deferred(function(){ fetchHandle.abort && fetchHandle.abort(); });
						var fetchHandle = this.fetch(lang.mixin({
							query: query,
							onBegin: function(count){
								deferred.total = count;
							},
							onComplete: function(results){
								deferred.resolve(results);
							},
							onError: function(error){
								deferred.reject(error);
							}
						}, options));
						return QueryResults(deferred);
					}
				});
			}
			this._set("store", store);
		},

		postMixInProperties: function(){
			// Since _setValueAttr() depends on this.store, _setStoreAttr() needs to execute first.
			// Unfortunately, without special code, it ends up executing second.
			if(this.params.store){
				this._setStoreAttr(this.params.store);
			}

			this.inherited(arguments);

			// User may try to access this.store.getValue() etc.  in a custom labelFunc() function.
			// It's not available with the new data store for handling inline <option> tags, so add it.
			if(!this.params.store){
				var clazz = this.declaredClass;
				lang.mixin(this.store, {
					getValue: function(item, attr){
						kernel.deprecated(clazz + ".store.getValue(item, attr) is deprecated for builtin store.  Use item.attr directly", "", "2.0");
						return item[attr];
					},
					getLabel: function(item){
						kernel.deprecated(clazz + ".store.getLabel(item) is deprecated for builtin store.  Use item.label directly", "", "2.0");
						return item.name;
					},
					fetch: function(args){
						kernel.deprecated(clazz + ".store.fetch() is deprecated for builtin store.", "Use store.query()", "2.0");
						var shim = ["dojo/data/ObjectStore"];	// indirection so it doesn't get rolled into a build
						require(shim, lang.hitch(this, function(ObjectStore){
							new ObjectStore({objectStore: this}).fetch(args);
						}));
					}
				});
			}
		}
	});
});

},
'gridx/modules/dnd/_Dnd':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/Deferred",
	"dojo/dom-construct",
	"dojo/dom-geometry",
	"dojo/dom-class",
	"dojo/dom-style",
	"dojo/dom",
	"dojo/_base/window",
	"dojo/_base/sniff",
	"dojo/dnd/Source",
	"dojo/dnd/Manager",
	"../../core/_Module",
	"../AutoScroll"
], function(declare, lang, Deferred, domConstruct, domGeometry, domClass, domStyle, dom, win, sniff,
	Source, DndManager, _Module){

	var hitch = lang.hitch;

	return _Module.register(
	declare(_Module, {
		name: '_dnd',

		constructor: function(){
			var t = this,
				g = t.grid,
				doc = win.doc;
			t.accept = [];
			t._profiles = {};
			t._selectStatus = {};
			t._node = domConstruct.create('div');
			t.batchConnect(
				[g, 'onCellMouseOver', '_checkDndReady'],
				[g, 'onCellMouseOut', '_dismissDndReady'],
				[g, 'onCellMouseDown', '_beginDnd'],
				[doc, 'onmouseup', '_endDnd'],
				[doc, 'onmousemove', '_onMouseMove']
			);
			t.subscribe("/dnd/cancel", '_endDnd');
		},

		load: function(args){
			var t = this,
				n = t.grid.mainNode;
			t._source = new Source(n, {
				isSource: false,
				accept: t.accept,
				getSelectedNodes: function(){return [0];},
				getItem: hitch(t, '_getItem'),
				checkAcceptance: hitch(t, '_checkAcceptance'),
				onDraggingOver: hitch(t, '_onDraggingOver'),
				onDraggingOut: hitch(t, '_onDraggingOut'),
				onDropExternal: hitch(t, '_onDropExternal'),
				onDropInternal: hitch(t, '_onDropInternal')
			});
			if(sniff('ff')){
				t._fixFF(t._source, n);
			}
			t._source.grid = t.grid;
			t._saveSelectStatus();
			t.loaded.callback();
		},
		
		destroy: function(){
			this.inherited(arguments);
			this._source.destroy();
			domConstruct.destroy(this._node);
		},
	
		getAPIPath: function(){
			return {
				dnd: {
					_dnd: this
				}
			};
		},

		//Public----------------------------------------------------------------------
		profile: null,

		register: function(name, profile){
			this._profiles[name] = profile;
			[].push.apply(this.accept, profile.arg('accept'));
		},
		
		//Private-----------------------------------------------------------------
		_fixFF: function(source){
			return this.connect(win.doc, 'onmousemove', function(evt){
				var pos = domGeometry.position(source.node),
					x = evt.clientX,
					y = evt.clientY,
					alreadyIn = source._alreadyIn,
					isIn = y >= pos.y && y <= pos.y + pos.h && x >= pos.x && x <= pos.x + pos.w;
				if(!alreadyIn && isIn){
					source._alreadyIn = 1;	//1 as true
					source.onOverEvent();
				}else if(alreadyIn && !isIn){
					source._alreadyIn = 0;	//0 as false
					source.onOutEvent();
				}
			});
		},

		_onMouseMove: function(evt){
			var t = this;
			if(t._alreadyIn && (t._dnding || t._extDnding)){
				t._markTargetAnchor(evt);
			}
		},

		_saveSelectStatus: function(enabled){
			var name, selector, selectors = this.grid.select;
			if(selectors){
				for(name in selectors){
					selector = selectors[name];
					if(selector && lang.isObject(selector)){
						this._selectStatus[name] = selector.arg('enabled');
						if(enabled !== undefined){
							selector.enabled = enabled;
						}
					}
				}
			}
		},

		_loadSelectStatus: function(){
			var name, selector, selectors = this.grid.select;
			if(selectors){
				for(name in selectors){
					selector = selectors[name];
					if(selector && lang.isObject(selector)){
						selector.enabled = this._selectStatus[name];
					}
				}
			}
		},

		_checkDndReady: function(evt){
			var t = this, name, p;
			if(!t._dndReady && !t._dnding && !t._extDnding){
				for(name in t._profiles){
					p = t._profiles[name];
					if(p.arg('enabled') && p._checkDndReady(evt)){
						t.profile = p;
						t._saveSelectStatus(false);
						domClass.add(win.body(), 'gridxDnDReadyCursor');
						t._dndReady = 1;
						return;
					}
				}
			}
		},
		
		_dismissDndReady: function(){
			if(this._dndReady){
				this._loadSelectStatus();
				this._dndReady = 0;	//0 as false
				domClass.remove(win.body(), 'gridxDnDReadyCursor');
			}
		},

		_beginDnd: function(evt){
			var t = this;
			t._checkDndReady(evt);
			if(t._dndReady){
				var p = t.profile,
					m = DndManager.manager();
				t._source.isSource = true;
				t._source.canNotDragOut = !p.arg('provide').length;
				t._node.innerHTML = p._buildDndNodes();
				t._oldStartDrag = m.startDrag;
				m.startDrag = hitch(t, '_startDrag', evt);
				
				if(t.avatar){
					t._oldMakeAvatar = m.makeAvatar;
					m.makeAvatar = function(){
						return new t.avatar(m);
					};
				}
				m._dndInfo = {
					cssName: p._cssName,
					count: p._getDndCount()
				};
				p._onBeginDnd(t._source);
				dom.setSelectable(t.grid.domNode, false);	
			}
		},

		_endDnd: function(){
			var t = this,
				m = DndManager.manager();
			t._source.isSource = false;
			t._alreadyIn = 0;	//0 as false
			delete m._dndInfo;
			if(t._oldStartDrag){
				m.startDrag = t._oldStartDrag;
				delete t._oldStartDrag;
			}
			if(t._oldMakeAvatar){
				m.makeAvatar = t._oldMakeAvatar;
				delete t._oldMakeAvatar;
			}
			if(t._dndReady || t._dnding || t._extDnding){
				t._dnding = t._extDnding = 0;	//0 as false
				t._destroyUI();
				dom.setSelectable(t.grid.domNode, true);
				domClass.remove(win.body(), 'gridxDnDReadyCursor');
				t.profile._onEndDnd();
				t._loadSelectStatus();
			}
		},
		
		_createUI: function(){
			domClass.add(win.body(), 'gridxDnDCursor');
			if(this.grid.autoScroll){
				this.profile._onBeginAutoScroll();
				this.grid.autoScroll.enabled = true;
			}
		},

		_destroyUI: function(){
			var t = this;
			t._unmarkTargetAnchor();
			domClass.remove(win.body(), 'gridxDnDCursor');
			if(t.grid.autoScroll){
				t.profile._onEndAutoScroll();
				t.grid.autoScroll.enabled = false;
			}
		},

		_createTargetAnchor: function(){
			return domConstruct.create("div", {
				"class": "gridxDnDAnchor"
			});
		},

		_markTargetAnchor: function(evt){
			var t = this;
			if(t._extDnding || t.profile.arg('canRearrange')){
				var targetAnchor = t._targetAnchor,
					containerPos = domGeometry.position(t.grid.mainNode);
				if(!targetAnchor){
					targetAnchor = t._targetAnchor = t._createTargetAnchor();
					targetAnchor.style.display = "none";
					t.grid.mainNode.appendChild(targetAnchor);
				}
				domClass.add(targetAnchor, 'gridxDnDAnchor' + t.profile._cssName);
				var pos = t.profile._calcTargetAnchorPos(evt, containerPos);
				if(pos){
					domStyle.set(targetAnchor, pos);
					targetAnchor.style.display = "block";
				}else{
					targetAnchor.style.display = "none";
				}
			}
		},

		_unmarkTargetAnchor: function(){
			var targetAnchor = this._targetAnchor;
			if(targetAnchor){
				targetAnchor.style.display = "none";
				domClass.remove(targetAnchor, 'gridxDnDAnchor' + this.profile._cssName);
			}
		},

		//---------------------------------------------------------------------------------
		_startDrag: function(evt, source, nodes, copy){
			var t = this;
			if(t._dndReady && source === t._source){
				t._oldStartDrag.call(DndManager.manager(), source, t._node.childNodes, copy);
				t._dndReady = 0;	//0 as false
				t._dnding = t._alreadyIn = 1;	//1 as true
				t._createUI();
				t._markTargetAnchor(evt);
			}
		},

		_getItem: function(id){
			return {
				type: this.profile.arg('provide'),
				data: this.profile._getItemData(id)
			};
		},

		_checkAcceptance: function(source, nodes){
			var t = this,
				getHash = function(arr){
					var res = {};
					for(var i = arr.length - 1; i >= 0; --i){
						res[arr[i]] = 1;
					}
					return res;
				},
				checkAcceptance = Source.prototype.checkAcceptance,
				res = checkAcceptance.apply(t._source, arguments);
			if(res){
				if(source.grid === t.grid){
					return t.profile.arg('canRearrange');
				}
				if(!source.canNotDragOut){
					for(var name in t._profiles){
						var p = t._profiles[name];
						var accepted = checkAcceptance.apply({
							accept: getHash(p.arg('accept'))
						}, arguments);
						if(p.arg('enabled') && accepted &&
							(!p.checkAcceptance || p.checkAcceptance.apply(p, arguments))){
							t.profile = p;
							t._extDnding = 1;	//1 as true
							return true;
						}
					}
				}
			}
			return false;
		},

		_onDraggingOver: function(){
			var t = this;
			if(t._dnding || t._extDnding){
				t._alreadyIn = 1;	//1 as true
				t._createUI();
			}
		},

		_onDraggingOut: function(){
			var t = this;
			if(t._dnding || t._extDnding){
				t._alreadyIn = 0;	//0 as false
				t._destroyUI();
			}
		},

		_onDropInternal: function(nodes, copy){
			this.profile._onDropInternal(nodes, copy);
		},
		
		_onDropExternal: function(source, nodes, copy){
			var t = this, dropped = t.profile._onDropExternal(source, nodes, copy);
			Deferred.when(dropped, function(){
				if(!copy){
					if(source.grid){
						source.grid.dnd._dnd.profile.onDraggedOut(t._source);
					}else{
						source.deleteSelectedNodes();
					}
				}
			});
		}
	}));
});

},
'dojoe/treetable/TreeTable':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/i18n!dojoe/treetable/nls/TreeTable","dojo/require!dijit/_Widget,dijit/_Templated,dojox/grid/LazyTreeGrid,dojo/i18n,dojo/string,dojox/grid/cells/dijit,dojoe/treetable/_Toolbar,dojoe/treetable/_IndirectSelection,dojoe/treetable/_QuickSearchFilter,dojoe/slidingpane/SlidingPane"], function(dijit,dojo,dojox){
dojo.provide("dojoe.treetable.TreeTable");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.grid.LazyTreeGrid");
dojo.require("dojo.i18n");
dojo.require('dojo.string');
dojo.require("dojox.grid.cells.dijit");
dojo.require("dojoe.treetable._Toolbar");
dojo.require("dojoe.treetable._IndirectSelection");
dojo.require("dojoe.treetable._QuickSearchFilter");
dojo.require("dojoe.slidingpane.SlidingPane");


//NLS
dojo.requireLocalization("dojoe.treetable", "TreeTable");

dojo.declare("dojoe.treetable.TreeTable",[ dijit._Widget, dijit._Templated ], {
	//summary: 
	//		Tree table is a hybrid widget of a tree and table 
	//description: 
	//		TreeTable widget implementation is based one dojox.grid.LazyTreeGrid 
	//		Provides additional features upon LazyTreeGrid like toolbar with state control buttons (refresh)
	//		custom actions drop down and Filter(search) controls.
	//	Usage
	//
	// example:
	//		Usage example 1:
	//
	// |		var treeTable = new dojoe.treetable.TreeTable({
	// |											  gridParams : {
	// |															indirectSelection : true,
	// | 															queryOptions : {deep : true},
	// |															treeModel:model_TreeTable,
	// |															structure:layout_TreeTable,
	// |															filter: {"isServerSide" : false}
	// |															},
	// |															enableResize : true
	// |											  }, domNode);	
	//		
	
	//summary: Path to the template
	//tags: private
	templateString : dojo.cache("dojoe.treetable", "templates/TreeTable.html", "\r\n<div class=\"tivTreeTable\" tabIndex=0 aria-label=\"tree table\" role=\"application\">\r\n\t<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\"  width=\"100%\" height=\"100%\" style=\"width:100%;height:100%;\"  role=\"presentation\">\t\t\r\n\t\t<tr valign=\"center\">\r\n\t\t\t<td width=\"100%\" role=\"toolbar\">\r\n\t\t\t\t<div dojoAttachPoint=\"wipeNode\">\r\n\t\t\t\t\t<div dojoAttachPoint=\"toolbarNode\"></div>\r\n\t\t\t\t</div>\r\n\t\t\t</td>\r\n\t\t</tr>\r\n\t\t<tr valign=\"center\">\r\n\t\t\t<td width=\"100%\" role=\"treegrid\">\r\n\t\t\t\t<div dojoAttachPoint=\"treeGridNode\"></div>\t\r\n\t\t\t</td>\r\n\t\t</tr>\t\t\r\n\t</table>\r\n</div>\t"),

	//summary: Set this to true if your widget contains other widgets
	//tags: private
	widgetsInTemplate : true,
	
	// gridparams: object
	//		holds all the parameters passed to the grid(LazyTreeGrid) by the user.
	gridParams:null,
	
	// grid: Object
	//		Reference to the lazy tree grid used
	grid:null,
	
	// toolbarVisible: boolean
	//		control the _toolbar show/hide
	toolbarVisible:true,
	
	// width: number in px or % or em
	//		holds the width of the grid
	width:'',
	
	// height: number in px or % or em
	//		holds the height of the grid
	height:'',
	
	// toolbarOptions: object
	//	holds the list of values for enabling/disabling toolbar options
	toolbarOptions:null,
	
	//summary: keeps track of the menu event object which provides the context value when right clicked
	onContextMenuEvtObj: null,
	
	//summary: 
	//		To resize the table on a particular event, connect the event to table's resize() function
	//description: 
	//		enableResize: true, to enable the table resize when the window resizes
	//enableResize: boolean
	//		to disable table resize on window resize event.
	//default true, set false to disable table resize on window resize event	
	enableResize:true,			
	
	//summary: Reference to the toolbar
	//tags: private
	_toolbar:null,
	
	//summary: Set the table with the default height
	_defaultHeight: '80%',
	
	//sum	mary: Set the table with the default width
	_defaultWidth: '80%',
	
	// indirectSelection: boolean
	//		Set table with basic Selection, if set to true enables Indirect selection Single and multiple grid selection modes
	indirectSelection: false,
	
	// enableColumnReorder: boolean
	//		to enable column reorder, by default its true
	enableColumnReorder:true,
	
	// state: object
	//		Tree Grid state object
	state: null,
	
	// _loadIndSelection: object
	//		just a reference to Indirect Selection plugin
	_loadIndSelection: null,
	
	// enableCollapsibleToolbar: boolean
	//		To add collapsible toolbar  to the table
	enableCollapsibleToolbar : false,
		
	// toolbarExpand: boolean
	//		To indicate(preset) the state of the toolbar on load
	toolbarExpand : true,
	
	constructor: function(args){
		// summary: Constructor
		//this.inherited( args );
		//console.info("constructor called");
		this.resources_ = dojo.i18n.getLocalization("dojoe.treetable", "TreeTable",this.lang);
		this.gridParams = {};
		this.actionList = {};
		//dojo.mixin( this, { style: "height: 100%; width: 100%;" } );
	},
	
	postCreate: function(){
		//summary: 
		//		creates and initialize the tree table 
		//tags: extension
		
	    //if no width is given default is set
        if(this.width ===''&& !this.gridParams.autoWidth){
			this.width = this._defaultWidth;
		}
		//if no height is given default is set
		if(this.height ===''){
			this.height = this._defaultHeight;
		}
		
        var size = dijit.getViewport();
		var heightPx = this.height;
		var sizeType = this.height.substring(this.height.length-1, this.height.length);
		//if the height is given in % then translate it to pixel
		if(sizeType == "%") {
			//convert to px
			heightPx = size.h * ((parseInt(this.height.slice(0, -1))) / 100);
			heightPx = heightPx + "px";
		}
		if( !this.gridParams.autoWidth){
			dojo.style(this.domNode, "width", this.width);
		}
		dojo.style(this.domNode, "height", heightPx);
		//Bobby:17Mar2011
		//Disable column reordering and provide this via Configure Options
		this.gridParams.columnReordering = false;

		//Satanik:30Mar2011: selection is remembered by rowId instead of index if keepSelection = true		
		//Satanik:10Jun2011: as per ILMT request this is configurable now and i moved it under makeGrid method		
		//23907: Treetable - unexpected selection change on collapse/expand
		//Bobby:30Jun11, this should be set before creating grid, so adding the check here itself!
		if(this.gridParams.keepSelection == undefined){
			this.gridParams.keepSelection = true;
		}
		
		//According to BJ Tivoli doesn't need rowSelector. rowSelector is useless with indirectSelection
		if (!this.gridParams.rowSelector || this.gridParams.indirectSelection) this.gridParams.rowSelector = false;
		//Satanik:13Jun2011: added check for the structure is null or not.
		if(this.gridParams.structure === null){
			this.gridParams.structure=[{}];
		}
		this._makeGrid();
		
		/*Satanik:14Mar11 Not able to disable sorting in TreeGrid and is causing more problems
		this.grid.canSort = function(col){
			if(this.indirectSelection){
				if(Math.abs(col) == 2) { 
					return true; 
				} else { 
					return false; 
				}
			}else{
				if(Math.abs(col) == 1) { 
					return true; 
				} else { 
					return false; 
				}			
			}				
		}
		*/
		//17606: TreeTable: quickfilter actions showing only 1st tree level, quicfilter disabling results in error
		//Bobby:22Apr2011 Adding option to show hierarchy if someone wants to implement on serverside
		if(this.gridParams.filter && this.gridParams.filter.showHierarchy){		
		}else{		
			this.grid.treeModel.mayHaveChildren = dojo.hitch(this.grid,function(/*dojo.data.Item*/ item){
				var children = null;
				//console.dir(this.cache);
				//console.debug('filter??????'+this.filterApplied);
				if(this.filterApplied) {
					return false;			
				}
				/*Adding one more condition 'dojo.isArray(children)' to support CustomQueryReadStore*/
				/*this should be also present in customTreeGridModel so that when this part of code is removed after filter hierarchy impl
				CustomQueryStore support is not broken*/
				return dojo.some(this.treeModel.childrenAttrs, function(attr){
					children = this.treeModel.store.getValue(item, attr);
					if(dojo.isString(children)){
						return parseInt(children, 10) > 0 || children.toLowerCase() === "true" ? true : false;
					}else if(typeof children == "number"){
						return children > 0;
					}else if(typeof children == "boolean"){
						return children;
					}else if(this.store.isItem(children)){
						children = this.treeModel.store.getValues(item, attr);
						return dojo.isArray(children) ? children.length > 0 : false;
					}else if(dojo.isArray(children) ){
						return children.length > 0;
					}else{
							return false;
						}
				}, this);
			});
		}
		
		if(this.menu !== null){
			dojo.connect(this.grid, "onRowContextMenu", dojo.hitch(this, "showRowContextMenu"));
		}
		if(this.toolbarVisible ){
			this._toolbar = new dojoe.treetable._Toolbar({treeTable:this,filterOptions:this.gridParams.filter}, this.toolbarNode);
			
			if (this.enableCollapsibleToolbar) {
				this.slidingPane = new dojoe.slidingpane.SlidingPane({closeNodeClass: "slidingPaneClose", open: this.toolbarExpand}, this.wipeNode);
				dojo.connect(this.slidingPane, "onClick", dojo.hitch(this, "slidingPaneClicked"));	//Connecting event onClick to sliding pane.
			}
		}
		
		/* #8414 - height issue - call resize regardless of 'enableResize' is true/false 
			if 'enableResize' is true then connect resize to window resize */
		this.resize();
		if(this.enableResize === true) {
	     	this.connect(window, "onresize", dojo.hitch(this, "resize"));			
		}
		
		//Satanik: 13Apr2011: adding listener function for the notification of action list open event.
		//Bobby,14Apr2011: Adding check for toolbar and actionListWidget
		if(this._toolbar && this._toolbar.actionListWidget){	
			//dojo.connect(this._toolbar.actionListWidget, 'onClick', dojo.hitch(this,"onActionListOpen"));
			//dojo.connect(this._toolbar.actionListWidget, 'onKeyDown', dojo.hitch(this,"onActionListOpen")); 
			//Bobby,13Apr2011: adding listener functions for the notification of action list open/close.	
			this.connect(this._toolbar.actionListWidget, 'openDropDown', dojo.hitch(this,"onActionListOpen")); 			
			this.connect(this._toolbar.actionListWidget, 'closeDropDown', dojo.hitch(this,"onActionListClose")); 
		}
		
		if(this.grid.defaultState && this.grid.defaultState.filter){
			// TODO: restore custom state
			//console.info("Restoring filter:"+this.grid.defaultState.filter);
			this.setFilter(this.grid.defaultState.filter);
		}		
		//Satanik:08Jun2011: adding the _preFilter for enable filter on rendering
		if(this.gridParams.preFilter){
			dojo.connect(this, 'onTreeTableLoad', dojo.hitch(this,"_preFilter"));
		}
		
		//Bobby: 04Mar11
		//Event generated on completion of TreeTable load
		setTimeout(dojo.hitch(this, "onTreeTableLoad", this), 1000);
		//this.onTreeTableLoad(this);		
	},
	
	slidingPaneClicked : function () {
		// summary:
		//		callback when collapsible tool bar sliding pane is clicked
		// tags: callback
		this.resize();
	},
	
	//Connect to this to monitor TreeTable load complete.
	onTreeTableLoad: function(treetable){
		// summary: 
		//		Called after the tree Table loads
		// tags:
	    //      callback
		console.info("TreeTable load complete....."+treetable.id);
	},
	
	//Satanik:08Jun2011: adding the _preFilter for enable filter on rendering
	_preFilter : function(){
		// summary: 
		//		Private function to set "gridparams.preFilter" value when tree table loads
		// tags: private
		
		if (this.gridParams.preFilter) {
				if(this.gridParams.preFilter[0].column==='any'){
					//is.grid._cleanup(); 
					//this.grid._clearData();
					if(this.gridParams.preFilter[0].value){
						this.setFilter(this.gridParams.preFilter[0].value);
					}
				}
		}
	},
	
	//Connect to this to monitor Opening of actionList
	onActionListOpen : function(){
		// summary:
		//		Callback when toolbar Action list is opened
		// tags: public
		
		//console.info("actionListOpen.....");
	},
	
	//Connect to this to monitor Closing of actionList
	onActionListClose : function(){
		// summary:
		//		Callback when toolbar Action list is closed
		// tags: public
		
		//console.info("actionListClosed.....");
	},
	
	_disableColumnMove: function(event){
		// summary: 
		//		private function to disable column reorder
		// tags: private
		
		console.info("_disableColumnMove.....");
		//console.dir(event);
		if(event){
			try{
				event.cancelBubble = true;
				dojo.stopEvent(event);
			}catch(e){
				console.error(e);
			}
		}
	},	
	
	setFilter : function (filterText){
		//summary:
		//		set filter value programmatically
		// tags: public
		if(this._toolbar){
			if(this._toolbar.quickFilter){
				this._toolbar.quickFilter._setFilter(filterText);
			}
		}
	},
		
	getFilter : function(){
		// summary:
		//		get filter value
		// tags: public
		
		if(this._toolbar){
			if(this._toolbar.quickFilter){
				return this._toolbar.quickFilter._getFilter();
			}
		}
	},	
	
	//Bobby:01Jun11. New API to retrieve and restore TreeTable state
	//Bobby:07Jun11. setState is not fully supported by LazyTreeGrid, use defaultState param while instantiating instead
	_setState : function (stateObj){
		// summary:
		//		Private function to retrieve ad restore TreeTable state
		// tags: private
		
		//console.info("setState -> state:");
		//console.log(stateObj);
		if(stateObj){
			//this.gridParams.cachedItems = stateObj.grid;
			this.gridParams.defaultState = stateObj;
		}
	},
	
	getState : function(){
		// summary: 
		//		returns the grid state information
		// returns: object
		//		the state information including filter state of the tree table
		
		/*state = new Object();
		state.grid = this.grid.cache.getCurrentCachedItems();*/
		state = this.grid.getState();
		console.info("getState -> state:");
		console.log(state);
		//return state;
		return dojo.mixin({filter: this.getFilter()}, state);
	},	
	
	//method to handle the header row click for select/deselect all
	selectHeaderRow : function(event){	
		// summary: 
		//		Callback on click of Select column header for selecting/de-selecting all the rows 
		// tags: public
		
		if(event.cell.field != 'selector'){
			return;
		}
		//event.cancelBubble = true;
		var headerdiv=dojo.byId("selectDiv");
		//Bobby:15Mar2011, why is this reqd?
		//this.deSelectAllRows();
		if(dojo.hasClass(headerdiv,"treeTableToggleButtonChecked")){
			dojo.removeClass(headerdiv, ["treeTableToggleButtonChecked"]);
			dojo.addClass(headerdiv, ["treeTableCheckBox"]);
			this.deSelectAllRows();
		} else{
			dojo.removeClass(headerdiv, ["treeTableCheckBox"]);
			//dojo.addClass(headerdiv, ["treeTableToggleButtonChecked"]);
			this.selectAllRows();
			dojo.addClass(headerdiv, ["treeTableToggleButtonChecked"]);
		}
		if(event){
			try{
				event.cancelBubble = true;
				dojo.stopEvent(event);
			}catch(e){
				console.error(e);
			}
		}
	},

	_makeGrid: function(){
		// summary: 
		//		creates and initializes dojox.grid.LazyTreeGrid widget
		// tags: private
		
		//creates the lazy tree grid instance and initialize it
		//tags: private
		if(this.gridParams.indirectSelection){
			//Latha : 5Sep2012 : commenting this as indirect selection is not fully implemented for TreeTable based on LazyTreeGrid.
			//this.indirectSelection=this.gridParams.indirectSelection;
			console.warn("Indirect selection is deprecated for dojoe.treetable.TreeTable, using direct selection...");
		}	
		//Satanik:25May2011: changes made to address ILMT requiremnt 19890
		if(this.gridParams.enableColumnReorder === false){
			this.enableColumnReorder=this.gridParams.enableColumnReorder;
		}
		
		//Bobby:01Jun11, Currently state can be set only before initializing grid!
		if(this.state) this._setState(this.state);	
		
		this.grid=new dojox.grid.LazyTreeGrid(this.gridParams,this.treeGridNode);
		//to display the default when grid is empty or filter result is none
		if(!this.gridParams.noDataMessage){
			this.grid.noDataMessage=this.resources_.NO_DATA;
		}
		else{
			this.grid.noDataMessage=this.gridParams.noDataMessage;
		}
		
		//Satanik:09Jun2011: enabling the selection as default in treetable
		if(this.gridParams.selectable !== undefined){
			this.grid.selectable = this.gridParams.selectable;
		} else{
			this.grid.selectable = true;
		}
		
		//23907: Treetable - unexpected selection change on collapse/expand
		//Bobby:30Jun11, this should be set before creating grid		
		//Satanik:10Jun2011: keepSelection can be configured by setting the grid param.
		/*if(this.gridParams.keepSelection !== undefined){
			this.grid.keepSelection = this.gridParams.keepSelection;
		} else{
			//Bobby:30Jun11, this should be grid and not gridParams	
			this.gridParams.keepSelection = true;
		}*/		
			
		this.grid.startup();	

		//Indirect selection is a new class now, create if indirect selection is enabled for treetable
		//this feature is supported on top of LazyTreeGrid. Once dojo incorporates indirect selection plugin, this can be removed 
		if(this.indirectSelection){
			//enabled
			this._loadIndSelection = new dojoe.treetable._IndirectSelection(this.grid);
		}	
	},
		
	showRowContextMenu: function(e){
		//summary: 
		//		Associates row context menu with tree table and also returns the contextEvent Object
		//tags: public
		
		if(this.menu){
			this.menu._openMyself({
				target: e.target,
				coords: "pageX" in e ? {
					x: e.pageX,
					y: e.pageY
				} : null
			});
		}
		return e;
	},	
	
	addToActionList: function(item) {
		// summary: 
		//		To add a new item to the toolbar action menu
		// tags: public
		
		if(this._toolbar) {
			if(this._toolbar.actionMenu) {
				this._toolbar.actionListWidget.dropDown.addChild(item,0);
			}
		}
	},
	
	addContextMenuToTreeTable: function(position, customMenu){
		// summary: 
		//		Allows adding a context menu to the TreeTable
		//tags: public
		
		if(position == "rowMenu") {
			customMenu.bindDomNode(this.grid.domNode);
			this.grid.onRowContextMenu = function(e){				
				if(!this.menu){
					this.menu = customMenu;
				}
			};
		}
	},
		
	//20470: Tivtable: Same Issue with  TreeTable load takes significant amount of time to load
	//Satanik:25May11, adding noResize to control Table resize
	//resize is very costly in iWidgets
	addToToolbar : function (item, position, row, noResize) {
			// summary:  
			//		Add an item to the toolbar
			// tags: public
			// item:  object   
			//		The item that needs to be added
			// position:  number 
			//		The position of the item
			// row:  number 
			//		The row number to which the item needs to be added
			// noResize: boolean 
			//		To control whether a table resize should be done or not
		
			if (this._toolbar) {
				if (item) {
					this._toolbar._addCustomControl(item, position, row);
				}
				/* #8414: height issue - call resize on this widget whenever toolbar layout is modified, 
				this way new toolbar height can be calculated and grid can be resized accordingly*/
				//20May2011: Bobby
				//performance issue when multiple items are added to toolbar (Sunset/Analytics)
				if(noResize == false){}
				else this.resize();						
			}
	}, 
		
	//20470: Tivtable: Same Issue with TreeTable load takes significant amount of time to load
	//Satanik:25May11, new API to add multiple toolbar items together		
	addItemsToToolbar : function (items, position, row, noResize) {
			// summary:  
			//		Add a list of items to the toolbar
			// tags: public
			// items:  array   
			//		The items that needs to be added
			// position:  number 
			//		The position of the starting item 
			// row:  number 
			//		The row number to which the item needs to be added
			// noResize: boolean 
			//		To control whether a table resize should be done or not
		
			if (this._toolbar) {
				for (var i = 0; i < items.length; i++) {		
					if (items[i]) {
						this._toolbar._addCustomControl(items[i], position + i, row);
					}
				}
				/* #8414: height issue - call resize on this widget whenever toolbar layout is modified, 
				this way new toolbar height can be calculated and grid can be resized accordingly*/
				//20May2011: Bobby
				//performance issue when multiple items are added to toolbar (Sunset/Analytics)
				if(noResize == false){}
				else this.resize();				
			}
	}, 		
	
	
	getItem: function(/*Integer */rowId){
		// summary: 
		//		Returns the rowItem when rowId is supplied
		// tags: public
		// return: object
		//		returns Item given the row id
		
		return this.grid.getItem(rowId);
	},
	
	getRowCount : function(){
		// summary: 
		//		rowCount of the table
		// tags: public
		// return: number
		//		the rowCount		
		return this.grid.rowCount;
	},
	
	getSelectedRowCount : function(){
		// summary: 
		//		Gets a selected rowCount of the table
		// tags: public
		// return: number
		//		the selected rowCount
		return this.grid.selection.getSelectedCount();
	},
	
	getSelectedRows : function(){
		// summary:  
		//		Returns the selected rows from the grid.
		// tags: public
		// returns: array
		//		selected rows.
		return this.grid.selection.getSelected();
	},	
	
	removeSelectedRows : function(){
		// summary: 
		//		deletes selected rows from the treetable
		// tags: public
		
		this.grid.removeSelectedRows();
	},
	
	isRowSelected: function(rowId){
		// summary: 
		//		Returns true if given row is selected
		// tags: public
		
		return this.grid.selection.isSelected(rowId);
	},
	
	disableMenuItem : function(menuid){
		// summary: 
		//		Disable a menu item of the context menu attached to the treetable.
		// tags: public
		
		var menuitem= dijit.byId(menuid);
		if(menuitem) {
			menuitem.setDisabled(true);
		}
	},
	
	enableMenuItem : function(menuid){
		// summary:
		//		Enable a menu item of the context menu attached to the treetable.
		// tags: public
		var menuitem= dijit.byId(menuid);
		if(menuitem) {
			menuitem.setDisabled(false);
		}
	},
	
     refresh: function(forceRefresh) {
    	// summary: 
		//		Refreshes the grid with updated values(force refresh)
    	// tags: public
		
    	//Satanik: 5Apr2011: added for CURI store notification of refresh in progress
		if (this.grid.store.CURIRefresh) {
			this.grid.store.CURIRefresh();
			/*LA - 09/01/2012 - TWL - 16212- TreeTable CURI refresh not clearing cache*/
			//cleanup the grid cache to reload the data from the CURI store..
			this.grid._cleanup();
		}
		//setting forceRefresh true clears the LTG cache so that data from stores is reloaded...
		if(forceRefresh){
			this.grid._cleanup();
		}
		var scrollpos = this.grid.scrollTop;
		this.grid._refresh(true);
		this.grid.setScrollTop(scrollpos);
    },
    
	removeMenuItem : function (menuid){
		// summary: 
    	//		Removes menu item from the context menu
		// tags: public
		var menuitem= dijit.byId(menuid);
		if(menuitem) {
			menuitem.destroyRecursive();
		}
	},
	
	deleteRow: function(item){
		// summary: 
		//		deletes the given item from the store
		// tags: public
		// item: object
		//		Row element of the table
		this.grid.store.deleteItem(item);
	},
	
	removeItemFromToolbar: function(itemId){
		// summary: 
		//		to remove an item from the toolbar
		// tags: public
		// itemId: String
		if(this._toolbar) {
			this._toolbar._removeItem(itemId);
		}			
	},
	
	removeFromActionList: function (menuItemId){
		// summary: 
		//		Removes a menu Item from ActionList
		// tags: public
		// menuItemId : String
		//		Id of the menuItem to be removed.
		if(this._toolbar) {
			this._toolbar._removeFromActionList(menuItemId);
		}
	},
	
	addRow: function(item,parent) {
		// summary: 
		//		Adds a new row to the treetable 
		// tags: public
		// description: 
		//		If no parent id is given, then the new row will be added as the last row.
		//      If parent id is given, then the row will be added as the last child of the parent.
		// item: object
		//		The item to be added
		// parent: string
		//		The parent id to which the new row should be added as a child
		
		//If the row is selected to be as a parent then the new row will be added as a child to that row.
		if(this._toolbar){
			if(this._toolbar.quickFilter && this._toolbar.quickFilter.filterText !== null){
				this._toolbar.quickFilter._clearFilter();
			}
			this._toolbar._clearSort();
		}
		//summary: add a row to the table and scroll to it
		//item: Row element of the table
		if(!parent){
			if(this.grid){
				var selectedItem = this.grid.store.newItem(item);
				var rowIndex = this.grid.getItemIndex(selectedItem);
				this.grid.scrollToRow(rowIndex);
			}
		}else{
			var s = this.grid.store;
			s.fetchItemByIdentity({identity: parent, onItem: function(item){
				parent = item;
			}});
			if(parent){
				s.newItem(item, {parent: parent, attribute: "children"});
			}				
		}
	},
	//Satanik:25May2011:syncing the resize with table resize	
	resize : function (event) {
		// summary:  
		//		Resizes the tree table. Also takes care of window resize events
		// tags:  public
		// event: object
		//		window resize event
		
		//17074: Javascript error in resize( ) sometimes on IE8
		//if we have set up everything except the DOM, we cannot resize
		//if (!this.domNode.parentNode || this.domNode.parentNode.nodeType != 1) {
		if (!this.grid || !this.domNode || !this.domNode.parentNode || this.domNode.parentNode.nodeType != 1) {
			return;
		}
		/*supporting autoWidth feature of grid*/
		if (!this.grid.autoWidth) {
			dojo.style(this.domNode, "width", this.width);
			
		} else {
			dojo.style(this.domNode, "width", dojo.marginBox(this.grid.domNode).w + 'px');
		}
		var resizeEvent = false;
		if(event && event.currentTarget){
			resizeEvent = true;
		}
		var size = dijit.getViewport();
		/*LS - 6 Jan 2012 - added this to keep in sync with table*/
		if(this._previousSize && this._previousSize.h == size.h && this._previousSize.w == size.w && resizeEvent == true){
				//console.warn(this.id+":Not resizing since there is no change in viewport size, wrong resize event!!!");
				return;				
			}else{
				//console.info(this.id+":resizing...");
				this._previousSize =  size;
			}		
		
		var heightPx = this.height;
		
		if (!(this.domNode.parentNode.style && this.domNode.parentNode.style.height)) {
			var sizeType = this.height.substring(this.height.length - 1, this.height.length);
			//if the height is given in % then translate it to pixel
			if (sizeType == "%") {
				//convert to px
				heightPx = size.h * ((parseInt(this.height.slice(0, -1))) / 100);
				heightPx = heightPx + "px";
			}
		}
		
		dojo.style(this.domNode, "height", heightPx);
		/*LS-29/09/2011 - #22273,#22274,#27221 - resize issues*/
		setTimeout(dojo.hitch(this, function(){this._delayResize(event);}), 100);
	},
	
	_delayResize: function (event) {
		// summary:  
		//		invoked with delay for grid resize.
		// tags: private
		
		///*LS-29/09/2011 - #22273,#22274,#27221 - resize issues*/
		
		/*using marginBox to include margins/borders of the domNode*/
		var tableSize = dojo.marginBox(this.domNode);
		/*grid height is total grid height - margins/padding calculating actual grid height for resize*/
		var gridPadBorder = this.grid._getPadBorder().h;
		
		/* #12836 - commenting the below fix as there were few resizing issues....*/			
		/* # 8414: height issue - check for height type and if in '%' just call resize on grid, grid will automatically fit into the layout */
		//var sizeType = this.height.substring(this.height.length-1, this.height.length);
		//if(sizeType !== "%") {
		if (this._toolbar) {
			/*calculating toolbarsize including margins and borders as they contibute to some height*/
			var toolbarSize = dojo.marginBox(this._toolbar.domNode);
			//removing grid padding height which will be later added when pagination bar is placed and grid view node is resized.
			tableSize.h = tableSize.h - toolbarSize.h - (2 * gridPadBorder);				
		}
		/*LS - 06 Jan 2012 - 14602 - Resize issue with tables in multiple tabs, 
				15999 - TED table should not attempt to resize when hidden 
				also applicable for tree table..*/
		if((tableSize.h <= 0 || tableSize.w <= 0 )){
			return;
		}
		/*LS - 18 Aug 2011 - 22072: Table resize issue*/
		//Applying this change as it exists for Tree table also
		//IWidget resize issue- grid was not getting proper width on window resize so on resizEvent(window resize) setting table width explicitly to grid
		if(event && event.currentTarget){
			this.grid.resize({
				h : tableSize.h,
				w : tableSize.w
			});
		}else{
			this.grid.resize({
				h : tableSize.h
			});
		}
		
		if(this.enableCollapsibleToolbar) {
				if(this.slidingPane!=null && !this.slidingPane.open) {
					tableSize.h = tableSize.h + toolbarSize.h ;
					this.grid.resize({
						h : tableSize.h
					});
				}
		} 
	},
		
	destroy: function(){
		// summary:  
		//		Destroys the tree table.
		// tags: public	
		
		// memory leak issue: custom destroy method to take care of removing all widgets					
		try{
			//21064: In Treetable, Clear filter does not work after "Destroy and Restore Treetable" action.
			//Bobby:04Jul11, the filter layer needs to be destroyed too
			if(this._toolbar && this._toolbar.quickFilter){
				this._toolbar.quickFilter.unWrapStore();
			}		
			
			if(this.menu){
				this.menu.destroyRecursive();
			}
			delete this.resources_;			
			if(this._loadIndSelection) this._loadIndSelection.destroy(); 
			//this is for destroying child templated widgets 
			//this should happen by itself but need to figure out why dijit._Templated is not taking care of it...
			var widgets = dijit.findWidgets(this.domNode);
			dojo.forEach(widgets, function(w) {
				if(w){
					if(w.destroyRecursive){
						w.destroyRecursive();
					} else if(w.destroy){
						w.destroy();
					}
				}
			});
			// this is for widgets added as child to this widget and their children..
			//not present in template
			var widgetArr = this.getChildren();
			dojo.forEach(widgetArr, function (child) {
				if(child){
					if(child.destroyRecursive){
						child.destroyRecursive();
					} else if(child.destroy){
						child.destroy();
					}
				}
			});
			this.inherited(arguments);
		}catch(ex){
			console.warn("Widget destroy failed: "+this.id+", ex="+ex.name+":"+ex.message);
		}
	}	
});


String.prototype.startWith = function(str){
	if(str==null||str==""||this.length==0||str.length>this.length){return false;}
	if(this.substring(0,str.length)==str){
		return true;
	}else{
		return false;
	}		
}
});

},
'dojoe/messagedialog/MessageDialog':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/i18n!dojoe/messagedialog/nls/MessageDialog","dojo/require!dijit/_Widget,dijit/Dialog,dijit/form/CheckBox,dijit/form/Button,dojo/i18n,dojox/layout/ResizeHandle,dojoe/Util"], function(dijit,dojo,dojox){
dojo.provide("dojoe.messagedialog.MessageDialog");

dojo.require("dijit._Widget");
dojo.require("dijit.Dialog"); 
dojo.require("dijit.form.CheckBox");
dojo.require("dijit.form.Button");
dojo.require("dojo.i18n");
dojo.require("dojox.layout.ResizeHandle");
dojo.require("dojoe.Util");

//Localization of resource bundles
dojo.requireLocalization("dojoe.messagedialog", "MessageDialog");


dojo.declare("dojoe.messagedialog.MessageDialog",[ dijit._Widget ],
{
	// summary:  MessageDialog is a widget to show message dialogs of type error,warning,info,confirm. 
	// description: MessageDialog widget uses the dijit.Dialog for its implementation.
	
	//theme: String
	// defined the theme for Message Dialog
	theme:null,	
	
	//resources:object
	//Reference to the resources
	resources_:null,
	 
    // resizeAxis: String
    //  x | xy | y to limit pane's sizing direction.
    resizeAxis: "xy",

	// type: String
    // defines the type of message dialog ("Error","Critical", "Warning", "Info", "Confirm").
	type: "Error",
	
	// portlet title associated w/ this message
	portletTitle: null,

	// message: String
	//the displayable translated message
	message: null,

	// messageId: String
	//the displayable translated message's unique identifier
	messageId: null,
	
	// width: String
	//gives the pixel width of the dialog (Default: 500px)
	width: "380px",
	
	// height: String
	//gives the pixel height of the dialog (Default: 200px)
	height: "250px",
	
	// buttons: String Array
	//a list of buttons to display (Default: OK)
	buttons: [],
	
	// callback: Function
	//a callback function when the dialog is closed
	callback: null,
	
	// parameters to pass on callback when dialog closes
	callbackParms: null,
	
	// url: String
	//a URL that links the message to an InfoCenter page
	url: null,
	
	// windowName: String
	//window.open(URL,name,specs,replace)
	windowName: '',
	// windowSpecs: String
	windowSpecs: '',
	// windowReplace: String
	windowReplace: '',
	
	// show "Never show again checkbox"
	showCheckbox: false,
	
	// contentIsolation: boolean parameter to open PopupUtil in content isolation mode in portlets
	contentIsolation: false,
	
	// Destroy the dialog on close
	destroyOnClose: true,

	// enableResizeHandle: boolean
	//a flag to enable/disable the resize handle, default is true
	enableResizeHandle: true,
	
	// namespace: user defined namespace for widget
	namespace :"",
	
	dir : "",

  // The dialog footer that contains the buttons
  footer: null,
	
	constructor: function() {
		// summary: An extension to the Dojo Dialog widget that supports 
		//      advanced features.
		// resourceMessages_: JSON
		//		Resource bundle.
		// dialog_: dojo.widget.FloatingPane
		//		An instance of the Dialog widget.
		// title_: String
		//		Title bar string.
		// icon_: String
		//		The dialog icon's uri.
		// buttonPressedId_: String
		//		The ID of the button pressed; this value is passed to the callback function.

	    this.resources_ = dojo.i18n.getLocalization("dojoe.messagedialog", "MessageDialog",this.lang);
		
		if (this.contentIsolation == true) {
			var statusAr = startsWith(dojoConfig.locale,"ar");
			var statusHe = startsWith(dojoConfig.locale,"he");
			if (statusAr == true || statusHe == true) {
				this.dir = "rtl";
			}
		}
	    
	    this.dialog_ = null;
	    this.title_ = null;
	    this.icon_ = null;
	    this.buttonPressedId_ = -1;
	},


	postCreate: function() { 
		// summary:  Creates and initialises the message Dialog as per its type.
		// tags: public
	    if (this.buttons.length === 0) {
	        this.buttons.push(this.resources_.DIALOG_BUTTON_OK);
	    }

		var className = null;
		//Bobby:20Dec11 - changing all images to use latest in guidelines - BEGIN		
	    if (this.type.toLowerCase() == "error" ||this.type.toLowerCase() == "critical") {
	    	if (this.portletTitle) {
		    	this.title_ = dojo.string.substitute(this.resources_.MESSAGE_TITLE, { 0: this.resources_.DIALOG_ERROR, 1: this.portletTitle });
	    	} else {
		        this.title_ = this.resources_.DIALOG_ERROR;
		    }
	        this.icon_ = dojo.moduleUrl("dojoe.common", "themes/images/st34critical_shad.png");
	        className = "critical";
	    }
	    else if (this.type.toLowerCase() == "warning") {
	    	if (this.portletTitle) {
		    	this.title_ = dojo.string.substitute(this.resources_.MESSAGE_TITLE, { 0: this.resources_.DIALOG_WARNING, 1: this.portletTitle });
	    	} else {
		        this.title_ = this.resources_.DIALOG_WARNING;
		    }
	        this.icon_ = dojo.moduleUrl("dojoe.common", "themes/images/st34minorwarning_shad.png");
	        className = "warning";
	    }
	    else if (this.type.toLowerCase() == "confirm") {
	    	if (this.portletTitle) {
		    	this.title_ = dojo.string.substitute(this.resources_.MESSAGE_TITLE, { 0: this.resources_.DIALOG_CONFIRM, 1: this.portletTitle });
	    	} else {
		        this.title_ = this.resources_.DIALOG_CONFIRM;
		    }
	        //Richa-14thAug12 - Changed the Confirm icon from unknown to warning.
	        this.icon_ = dojo.moduleUrl("dojoe.common", "themes/images/st34minorwarning_shad.png");
	        className = "confirm";
	    }
	    else if (this.type.toLowerCase() == "validation") {
	    	if (this.portletTitle) {
		    	this.title_ = dojo.string.substitute(this.resources_.MESSAGE_TITLE, { 0: this.resources_.DIALOG_VALIDATION, 1: this.portletTitle });
	    	} else {
		        this.title_ = this.resources_.DIALOG_VALIDATION;
		    }
	        this.icon_ = dojo.moduleUrl("dojoe.common", "themes/images/st34success_shad.png");
	        className = "validation";
	    }
	    else { // if (this.type.toLowerCase() == "info") {
	    	if (this.portletTitle) {
		    	this.title_ = dojo.string.substitute(this.resources_.MESSAGE_TITLE, { 0: this.resources_.DIALOG_INFO, 1: this.portletTitle });
	    	} else {
		        this.title_ = this.resources_.DIALOG_INFO;
		    }
	        this.icon_ = dojo.moduleUrl("dojoe.common", "themes/images/st34informational_shad.png");
	        className = "info";
	    }
	    //Bobby:20Dec11 - changing all images to use latest in guidelines - END
		
		var div = document.createElement('div'); 
		document.body.appendChild(div);
		var theme=this.theme;
		var dir = this.dir;
        this.dialog_ = new dijit.Dialog (
			                             { title: this.title_,
											 							 dir: dir,
		                                 style: "width: " + this.width },
		                                 div); 
		/* HACK: Adjust the base class of the dialog to includ dojoeMessageDialog so we can style the dialog 
		         independent of rest of dijits. */ 
     this.dialog_.baseClass = this.dialog_.baseClass + " dojoeMessageDialog";

		var dojoAttachpoint = this.dialog_.containerNode;
		this.dialog_.closeButtonNode.setAttribute("title", this.resources_.DIALOG_BUTTON_CLOSE);
		this.dialog_.closeButtonNode.setAttribute("tabIndex", 0);
		dojo.connect(this.dialog_.closeButtonNode,'onkeypress',this,'onKeyPress');
		dojo.style(div,'class',theme);
		dojoAttachpoint.innerHTML = this.getContent(className,theme);
		// dojo.connect(dijit.Dialog.prototype, "postMixInProperties", function(){ this.buttonCancel="Close" });
    
		// Create the dialog footer and buttons
		this.footer = dojo.create( "div", { className: "dijitDialogPaneActionBar " + className, id: this.id + "_footer" } );
		
		var i, button;
		for ( i = 0; i < this.buttons.length; i++) {
			button = new dijit.form.Button( { label: this.buttons[i],
																 id: this.id + "mmd_button_" + i,
																 onClick: dojo.hitch( this, this.onClick ) } );
			//console.log( "[PostCreate] New Button ", button );
			this.footer.appendChild( button.domNode );   
		}
		// Add the buttons to the bottom of the dialog's DOM
		dojo.place( this.footer, this.dialog_.domNode, "last" );
		
		if(this.enableResizeHandle == true){
			//Build and place resize handle       
			var resizeHandleDOM = dojo.doc.createElement( 'span' );
			resizeHandleDOM.className = "dojoxFloatingResizeHandle";    
			dojo.place( resizeHandleDOM, this.dialog_.domNode, "last" );		
			this.resizeHandle = new dojox.layout.ResizeHandle(
			{ 
				targetId: this.dialog_.id, 
				resizeAxis: this.resizeAxis 
			}, resizeHandleDOM );  
			this.onResizeListener = dojo.connect( this.dialog_, "resize", this, "customLayout" );			
		}
	 
     this.onCloseListener  = dojo.connect( this.dialog_, "hide", this, "_onHide" );
	    
	},
	
    /**************************************************************************
    * The dialog has been hidden.  Wait 500 miliseconds to destroy the dialog.
    *
    * HACK -- Can't figure out a better way to destroy the dialog on close.
    **************************************************************************/
    _onHide:function()
    {// summary: Closes the MessageDialog .
		// tags: private
      setTimeout( dojo.hitch( this, "destroy" ), 500 );
    },  
	
	onKeyPress: function (e) {
		if (e.keyCode == 13 || e.keyCode == 32 || e.keyCode == 0) { //  Space keycode is 32 on IE8 and 0 on mozilla.  Enter keycode is 13 for both. 
			this.dialog_.hide();
		}
	},
	
	onClick: function(/* HTMLEvent */e) {
    	// summary:  Calls the callback on click of Ok else destroys.
		// tags: public
		var index = this.id.length + "mmd_button_".length, targetId;
		targetId=e.target.id;
		if(targetId===""){
			if(e.target.firstChild){
				targetId=e.target.firstChild.id;
			}
		}
		this.buttonPressedId_ = parseInt(targetId.substring(index), 0);
	    //console.log ("MessageDialog.onClick: buttonPressedId_: " + this.buttonPressedId_);
	    var checkbox = dojo.byId (this.id + "neverShowAgain");
	    var checked = checkbox ? checkbox.checked : false;

	    this.dialog_.hide();
	    
	    if (this.callback) {
	    	this.callback(this.buttonPressedId_, this.messageId, checked, this.callbackParms);
	    }
    },

	show: function() {
    	// summary:  Displays the messageDialog.
		// tags: public
	    if( this.contentIsolation == true ){
			// Get the namespace identifying the content that launched the context menu
			//console.info("Inside Show if PopupUtil Undefined");
			var namespace="";
			if(this.namespace != null)
			 namespace = this.namespace;
			
			//Close (just in case) and then open the popup covering the whole page
			//console.info("Opening Popup namespace "+namespace);
			var xcord=100;
			var ycord=100;
			var dialogNamespace=namespace+'dojoe.messagedialog.MessageDialog';
			var old_popup_iframe = PopupUtil.ClosePopup( namespace+'dojoe.messagedialog.MessageDialog' );
			//Changes for fixing Content Isolation issue with Claro theme
			var theme = dojo.body().className;
			//var theme=this.theme; 
			
			//Changes to fix CSBUG #158121
			/*var popupUrl =dojo.baseUrl+'../html/MessageDialogPopUp.html?PopupID='+dialogNamespace+'&eLeft='+xcord+'&eTop='+ycord+'&locale='+dojoConfig.locale+"&theme="+theme;*/
			
			var popupUrl = require.toUrl('dojoe/messagedialog/MessageDialogPopUp.html?PopupID='+dialogNamespace+'&eLeft='+xcord+'&eTop='+ycord+'&locale='+dojo.locale+"&theme="+theme);
			/*var popupUrl =this.baseUrl+'sample/jsp/html/MessageDialogPopUp.html?PopupID='+dialogNamespace+'&eLeft='+xcord+'&eTop='+ycord+'&locale='+dojoConfig.locale+"&theme="+theme;*/
			//console.info("url "+popupUrl);
			var popup_iframe = PopupUtil.OpenPopup( 
				namespace, 
				namespace+'dojoe.messagedialog.MessageDialog', 
				true, 
				null, 
				null, 
				null, 
				null, 
				popupUrl, 
				null
			);
			//console.info("after popup opening code");
			//Initialize the contextMenu when the iframe popup is finished loading
			var tThis = this;
			if( popup_iframe.addEventListener ){
				popup_iframe.addEventListener(
					"load", 
					function(){
						popup_iframe.contentWindow.makeMessageDialog( tThis );
					}, 
					false
				);
			}
			else if (popup_iframe.attachEvent){
				popup_iframe.attachEvent(
					"onload", 
					function(){ 
						popup_iframe.contentWindow.makeMessageDialog( tThis);
					}
				);
			}
		}
		else{
			this.dialog_.show();
			this.customLayout();
			this.correctPNG();
        }
	},

      customLayout: function() 
      {
		/* summary: Resizes the Message Dialog.  We only adjust the height of the dialog and allow the containing table 
		   to keep the width at 100%. This appears to work for IE 8, FireFox, and Chrome. */
		// tags: public
		
		var messageDiv = dojo.byId( this.id + '_messagediv' );
		var dialogSize = dojo.contentBox( this.dialog_.domNode );

    // IE 8 was causing some bogus numbers to appear when the dialog first open, if that happens ignore the caller.
    if( dialogSize.w < 0 || dialogSize.h < 0 ){
			console.log( "[customLayout] Ignoring resize request since dialogSize is negative. " );
			return;
		}
		
		//44160: TED - dojoe.messagedialog.MessageDialog: Wrong Size Calculated
		//Bobby Joseph: 27Jun2012, using marginBox instead, since OneUI has margins in title bar
		var titleBarSize = dojo.marginBox( this.dialog_.titleBar );
		
		// Start collect information about the various elements of the dialog box
		var cBox = dojo.marginBox( this.dialog_.containerNode );
    var msgBox = dojo.marginBox( messageDiv );
		var fBox = dojo.marginBox( this.footer );
		var newSize = { h:0 };
		
		/* Debug information to validate calculations or if funny things happen 
    console.log( "[customLayout] =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=" );
    console.log( "[customLayout] dialogSize [ w=" + dialogSize.w + ", h="+ dialogSize.h + "]" );
    console.log( "[customLayout] fBox [ w=" + fBox.w + ", h=" + fBox.h + "]" );
		console.log( "[customLayout] cBox [ w=" + cBox.w + ", h=" + cBox.h + "]" );
		console.log( "[customLayout] msgBox [ w=" + msgBox.w + ", h=" + msgBox.h + "]" );
	  console.log( "[customLayout] titleBarSize [ w=" + titleBarSize.w + ", h=" + titleBarSize.h + "]" ); */
		
		/* We subtract the current message box size from the content box size to figure out how much margins, padding, and
		   surrounding content are taking up space so we can correctly adjust the width and height of the message description
			 based on the detla we're given by the resize event. */
		newSize.h = dialogSize.h - titleBarSize.h - fBox.h - (cBox.h - msgBox.h);
		
		// Use the height of the message-id to be the minimum height of the message area.  That should give
		// us at least one line of text before scrolling. 
		// Fall back on the dialog title as the min height.  Might be bigger than we need but it is at least something.
		var msgId = dojo.query( ".message-id", this.dialog_.domNode )[0] || 
		            dojo.query( ".dijitDialogTitle", this.dialog_.domNode )[0] || null;
		
		
		if( msgId !== null ){	
		  var minHeight = dojo.marginBox( msgId );

      // Make sure we use a good min size since the user could resize the box too small to fit the content. 
		  newSize.h = Math.max( minHeight.h, newSize.h );	
		} else {
			console.log( "[customLayout] Unable to determine the min height of a text string all plan exhasted.", msgId );
		}
	
	  //console.log( "[customLayout] final size [ h=" + newSize.h + "]" );
		
    dojo.contentBox( messageDiv, newSize );
	},
	
	getContent: function(className,theme) {
		// summary:  gets the content to show inside messageDialog.
		// tags: public
		var innerHTML = "";
		
		innerHTML += "<div class='messagedialog'>";
		if (this.showCheckbox === true) {
			innerHTML += "<table class=\"" + className + "checkbox\" >";
		}else innerHTML += "<table class=\"" + className + "\" >";
		innerHTML += "<tr valign=\"top\">";
		innerHTML += "<td>";
		innerHTML += "<img role='img'  align=\"top\" src=\"" + this.icon_ + "\" tabIndex=0 title=\"" + this.title_ + "\" width='34' height='34' >";
		innerHTML += "</img>"		
		innerHTML += "</td>";
		innerHTML += "<td>";		

		innerHTML += "<table style='width: 100%;'>";
		innerHTML += "<tr >";
		
		
		//Bobby:31Oct11, making messageId optional as per request from TPM
		if (this.messageId) {		
			innerHTML += "<td class=\"message-id\" tabIndex=0 aria-label=\"" + this.messageId + "\">";		
			// add URL to infocenter
			if (this.url) {
				innerHTML += '<a href="javascript:void(0)" onclick="window.open(\'' + this.url+"','"
					+ this.windowName+"','"
					+ this.windowSpecs+"','"
					+ this.windowReplace
					+ "\')\">";
				innerHTML += this.messageId;
				innerHTML += "</a>";
			}
			else {
				innerHTML += this.messageId;
			}			
			innerHTML += "</td>";
		}
		innerHTML += "</tr>";
		innerHTML += "<tr>";
		innerHTML += "<td class=\"message-description\" >";
		innerHTML += "<div id='" + this.id + "_messagediv" + "' style='overflow:auto; width:100%;' tabIndex=0 aria-label=\"" + this.message + "\">";
		innerHTML += this.message;
		innerHTML += "</div>";
		innerHTML += "</td>";
		innerHTML += "</tr>";
		if (!this.messageId) {	
			innerHTML += "<tr>";
			innerHTML += "<td>";
			innerHTML += "&nbsp;";		
			innerHTML += "</td>";
			innerHTML += "</tr>";			
		}	
		innerHTML += "</table>";

		innerHTML += "</td>";
		innerHTML += "</tr>";

		if (this.showCheckbox === true) {
			innerHTML += "<tr>";
			innerHTML += "<td>";	
			innerHTML += "</td>";			
			innerHTML += "<td class=\"checkbox\" valign=\"bottom\">";
			
			innerHTML += "<table>";
			innerHTML += "<tr valign='bottom'>";
			innerHTML += "<td>";
			innerHTML += "<input type=\"checkbox\" dojoType=\"dijit.form.CheckBox\" id=\"" + this.id + "neverShowAgain\"/>&nbsp;";
			innerHTML += "</td>";
			innerHTML += "<td valign='middle'>";
			innerHTML += "<label for=\"" + this.id + "neverShowAgain\">" + this.resources_.NEVER_SHOW + "</label>";
			innerHTML += "</td>";
			innerHTML += "</tr>";	
			innerHTML += "</table>";		
			
			innerHTML += "</td>";
			innerHTML += "</tr>";
		}

    innerHTML += "</table>";
		
		innerHTML += "</div>";
		
		return innerHTML;
	},
	
	// correctly handle PNG transparency in Win IE 5.5 & 6.
	correctPNG: function() {
		var arVersion = navigator.appVersion.split("MSIE");
		var version = parseFloat(arVersion[1]);
		if ((version >= 5.5) && document.body.filters) {
			for (var i = 0; i < document.images.length; i++) {
				var img = document.images[i];
				var imgName = img.src.toUpperCase();
				if (imgName.substring(imgName.length-3, imgName.length) == "PNG") {
					var imgID = img.id ? "id='" + img.id + "' " : "";
					var imgClass = img.className ? "class='" + img.className + "' " : "";
					var imgTitle = img.title ? "title='" + img.title + "' " : "title='" + img.alt + "' ";
					var imgStyle = "display:inline-block;" + img.style.cssText;
					if (img.align == "left") {
						imgStyle = "float:left;" + imgStyle;
					}
					if (img.align == "right") {
						imgStyle = "float:right;" + imgStyle;
					}
					if (img.parentElement.href) {
						imgStyle = "cursor:hand;" + imgStyle;
					}
					var strNewHTML = "<span " + imgID + imgClass + imgTitle +
									 " style=\"" + "width:" + img.width + "px; height:" + img.height + "px;" + imgStyle + ";" +
									 "filter:progid:DXImageTransform.Microsoft.AlphaImageLoader" +
									 "(src=\'" + img.src + "\', sizingMethod='scale');\"></span>";
					img.outerHTML = strNewHTML;
					i = i - 1;
				}
			}
		}    
	},
	
	destroy: function () {
		// summary:  destroys the message Dialog.
		// tags: public
        dojo.disconnect( this.onCloseListener );
		if(this.onResizeListener){
			dojo.disconnect( this.onResizeListener );
		}
        
        //TODO:  The fact this can NOT be called means we have not created the widget correctly
        //       This widget should directly extend dialog to be created correctly
        //this.inherited( "destroy", arguments );
        
        if ( this.dialog_ ) { this.dialog_.destroy(); dijit.registry.remove(this.dialog_.id); delete this.dialog_; }
        
		if (dijit.byId (this.id + "neverShowAgain")) {
			dijit.registry.remove (this.id + "neverShowAgain");
		}

		for (var i = 0; i < this.buttons.length; i++) {
			dijit.registry.remove (this.id + "mmd_button_" + i);
		}

		delete this.type;
		delete this.message;
		delete this.messageId;
		delete this.width;
		delete this.height;
		delete this.buttons;
		delete this.callbackParms;
		delete this.url;
		delete this.showCheckbox;
		delete this.destroyOnClose;
		delete this.title_;
		delete this.icon_;
		delete this.resources_;

		dijit.registry.remove (this.id);
	}

});



});

},
'dijit/BackgroundIframe':function(){
define([
	"require",			// require.toUrl
	".",	// to export dijit.BackgroundIframe
	"dojo/_base/config",
	"dojo/dom-construct", // domConstruct.create
	"dojo/dom-style", // domStyle.set
	"dojo/_base/lang", // lang.extend lang.hitch
	"dojo/on",
	"dojo/_base/sniff", // has("ie"), has("mozilla"), has("quirks")
	"dojo/_base/window" // win.doc.createElement
], function(require, dijit, config, domConstruct, domStyle, lang, on, has, win){

	// module:
	//		dijit/BackgroundIFrame
	// summary:
	//		new dijit.BackgroundIframe(node)
	//		Makes a background iframe as a child of node, that fills
	//		area (and position) of node

	// TODO: remove _frames, it isn't being used much, since popups never release their
	// iframes (see [22236])
	var _frames = new function(){
		// summary:
		//		cache of iframes

		var queue = [];

		this.pop = function(){
			var iframe;
			if(queue.length){
				iframe = queue.pop();
				iframe.style.display="";
			}else{
				if(has("ie") < 9){
					var burl = config["dojoBlankHtmlUrl"] || require.toUrl("dojo/resources/blank.html") || "javascript:\"\"";
					var html="<iframe src='" + burl + "' role='presentation'"
						+ " style='position: absolute; left: 0px; top: 0px;"
						+ "z-index: -1; filter:Alpha(Opacity=\"0\");'>";
					iframe = win.doc.createElement(html);
				}else{
					iframe = domConstruct.create("iframe");
					iframe.src = 'javascript:""';
					iframe.className = "dijitBackgroundIframe";
					iframe.setAttribute("role", "presentation");
					domStyle.set(iframe, "opacity", 0.1);
				}
				iframe.tabIndex = -1; // Magic to prevent iframe from getting focus on tab keypress - as style didn't work.
			}
			return iframe;
		};

		this.push = function(iframe){
			iframe.style.display="none";
			queue.push(iframe);
		}
	}();


	dijit.BackgroundIframe = function(/*DomNode*/ node){
		// summary:
		//		For IE/FF z-index schenanigans. id attribute is required.
		//
		// description:
		//		new dijit.BackgroundIframe(node)
		//			Makes a background iframe as a child of node, that fills
		//			area (and position) of node

		if(!node.id){ throw new Error("no id"); }
		if(has("ie") || has("mozilla")){
			var iframe = (this.iframe = _frames.pop());
			node.appendChild(iframe);
			if(has("ie")<7 || has("quirks")){
				this.resize(node);
				this._conn = on(node, 'resize', lang.hitch(this, function(){
					this.resize(node);
				}));
			}else{
				domStyle.set(iframe, {
					width: '100%',
					height: '100%'
				});
			}
		}
	};

	lang.extend(dijit.BackgroundIframe, {
		resize: function(node){
			// summary:
			// 		Resize the iframe so it's the same size as node.
			//		Needed on IE6 and IE/quirks because height:100% doesn't work right.
			if(this.iframe){
				domStyle.set(this.iframe, {
					width: node.offsetWidth + 'px',
					height: node.offsetHeight + 'px'
				});
			}
		},
		destroy: function(){
			// summary:
			//		destroy the iframe
			if(this._conn){
				this._conn.remove();
				this._conn = null;
			}
			if(this.iframe){
				_frames.push(this.iframe);
				delete this.iframe;
			}
		}
	});

	return dijit.BackgroundIframe;
});

},
'gridx/modules/exporter/Table':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"../../core/_Module",
	"./Exporter"
], function(declare, lang, _Module){

/*=====
	dojo.declare('__TableExportArgs', __ExportArgs, {
		natualWidth: false
		columnWidth: Associative array
	});
=====*/

	function cellattrs(args, col){
		var cw = args.columnWidth,
			w = (cw && cw[col.id]) || (args.natualWidth ? '' : col.getWidth()) || 'auto';
		return [' colid="', col.id, '" style="width:', w, '"'].join('');
	}

	return declare(/*===== "gridx.modules.exporter.Table", =====*/_Module, {
		// summary:
		//		This module provides the API to export grid contents to an HTML table, which is mainly used in print.

		name: 'exportTable',

		forced: ['exporter'],

		getAPIPath: function(){
			return {
				exporter: {
					toTable: lang.hitch(this, this.toTable)
				}
			};
		},
	
		//Public ---------------------------------------------------------------------
		toTable: function(/* __TableExportArgs */ args){
			// summary:
			//		Export the grid contents to HTML table according to the given args.
			//		This method should be called through grid.exporter.toCSV();
			// args: __TableExportArgs?
			//		The args to configure the export result and the export process.
			// returns:
			//		A deferred object indicating when the export process is completed,
			//		and then pass the exported HTML table (as string) to callbacks.
			return this.grid.exporter._export(this, args || {});	//dojo.Deferred
		},

		//Package --------------------------------------------------------------------
		initialize: function(/* __TableExportArgs */ args){
			// tags:
			//		private
			this._rst = ['<table class="grid"',
				args.natualWidth ? '' : ' style="table-layout:fixed;"',
				' border="0" cellpadding="0" cellspacing="0">'
			];
		},

		beforeHeader: function(){
			// tags:
			//		private
			this._rst.push('<thead><tr class="grid_header">');
		},

		handleHeaderCell: function(/* __ExportContext */ context, /* __TableExportArgs */ args){
			// tags:
			//		private
			var col = context.column;
			this._rst.push('<th class="grid_header_cell"', cellattrs(args, col), '>', col.name(), '</th>');
		},

		afterHeader: function(){
			// tags:
			//		private
			this._rst.push('</tr><thead>');
		},

		beforeBody: function(){
			// tags:
			//		private
			this._rst.push('<tbody>');
		},

		beforeRow: function(/* __ExportContext */ context){
			// tags:
			//		private
			var r = context.row, idx = r.index();
			this._rst.push('<tr class="grid_row grid_row_', idx % 2 ? 'even' : 'odd',
				'" rowid="', r.id, '" rowindex="', idx, '">');
		},

		handleCell: function(/* __ExportContext */ context, /* __TableExportArgs */ args){
			// tags:
			//		private
			this._rst.push('<td class="grid_cell"', cellattrs(args, context.column), '>', context.data, '</td>');
		},

		afterRow: function(){
			// tags:
			//		private
			this._rst.push('</tr>');
		},

		afterBody: function(){
			// tags:
			//		private
			this._rst.push('</tbody></table>');
		},

		getResult: function(){
			// tags:
			//		private
			return this._rst.join('');
		}
	});
});

},
'ibm/tivoli/navmodel/Event':function(){
// wrapped by build app
define("ibm/tivoli/navmodel/Event", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/Exception"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Event"); 

// includes
dojo.require("ibm.tivoli.navmodel.Exception");

////////////////////////////////////
// Event
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Event", null, {
	// private members
	_dataset: null,
	_event: null,
	_items: null,
	_links: null,
	_relationships: null,
	_props: null,
	_eventRouterStatus: null,
	
	/*=====
	constructor_kwArgs: function(dataset, event) {
		// dataset: Object
		//		the dataset that the event is for
		// event: Object
		//		raw event object
	},
	=====*/

	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs Event object.
		this._dataset = kwArgs.dataset;		
		this._event = kwArgs.event;
		this._items = [];
		this._links = [];
		this._relationships = [];
		this._props = [];
		
		// "eventRouterStatus" events originate from the EventRouter;
		// fill in this info to be consistent with other events
		if (this._event.id == "eventRouter") {
			this._event.providerId = this._dataset.getProviderId();
			this._event.datasourceId = this._dataset.getDatasourceId();
			this._event.datasetId = this._dataset.getId();
			this._event.version = this._dataset.getVersion();
		}
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	
	getDataset: function() {
		return this._dataset;
	},
	
	getVersion: function() {
		return this._event.version;
	},
	
	getProviderId: function() {
		return this._event.providerId;
	},
	
	getDatasourceId: function() {
		return this._event.datasourceId;
	},
	
	getDatasetId: function() {
		return this._event.datasetId;
	},
	
	getId: function() {
		return this._event.id;
	},
	
	getType: function() {
		return this._event.type;
	},
	
	getItems: function() {
		return this._items;
	},
	
	getLinks: function() {
		return this._links;
	},
	
	getRelationships: function() {
		return this._relationships;
	},
	
	getProperties: function() {
		return this._props;
	},
	
	getLinkChanged: function() {
		return this._links[0];
	},
	
	getLinkChangedReason: function() {
		return this._event.reason;
	},
	
	getLinkChangedRelationships: function() {
		return this._relationships;
	},
	
	getNewLabel: function() {
		return this._event.label;
	},
	
	getException: function() {
		return new ibm.tivoli.navmodel.Exception({exception: this._event.exception, version: this._dataset.getVersion()});
	},
	
	getEventRouterStatus: function() {
		return this._event.eventRouterStatus;
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////
	
	_addItem: function(item, props) {
		this._items.push(item);
		this._props.push(props);
	},
	
	_addLink: function(link, props) {
		this._links.push(link);
		this._props.push(props);
	},
	
	_addRelationship: function(relationship, props) {
		this._relationships.push(relationship);
		this._props.push(props);
	},
	
	_getEvent: function() {
		return this._event;
	},
	
	_getEventItems: function() {
		return this._event.items;
	},
	
	_getEventLinks: function() {
		return this._event.links;
	},
	
	_getEventLink: function() {
		return this._event.link;
	},
	
	_getEventRelationships: function() {
		return this._event.relationships;
	},
	
	_getEventNewLabel: function() {
		return this._event.label;
	},
	
	_getEventStatuses: function() {
		return this._event.status;
	},
	
	_getEventDatasetItemCount: function() {
		return this._event.datasetItemCount;
	},
	
	_getEventDatasetLinkCount: function() {
		return this._event.datasetLinkCount;
	},
	
	_getEventDatasetRelationshipCount: function() {
		return this._event.datasetRelationshipCount;
	}
});


////////////////////////////////////////////////////////////////////////
// ibm.tivoli.navmodel.ClientEvent
//
// For creating client initiated ibm.tivoli.navmodel.Event objects
////////////////////////////////////////////////////////////////////////

ibm.tivoli.navmodel.ClientEvent = {
	getChangedEvent: function(dataset) {
		// summary:
		//		Creates client-side event for type "changed"
		var eventObj = this._eventObj(dataset, "changed");		
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getItemsAddedEvent: function(dataset, items) {
		var eventObj = this._eventObj(dataset, "itemsAdded");
		eventObj.datasetItemCount = dataset.getItemCount() + items.length;		
		this._mixinItems(eventObj, items);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getItemsRemovedEvent: function(dataset, items) {
		var eventObj = this._eventObj(dataset, "itemsRemoved");
		eventObj.datasetItemCount = dataset.getItemCount() - items.length;		
		this._mixinItems(eventObj, items);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getItemStatusChangedEvent: function(dataset, items) {
		var eventObj = this._eventObj(dataset, "itemStatusChanged");
		this._mixinItems(eventObj, items);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getItemPropertiesChangedEvent: function(dataset, items) {
		var eventObj = this._eventObj(dataset, "itemPropertiesChanged");
		this._mixinItems(eventObj, items);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getLinksAddedEvent: function(dataset, links) {
		var eventObj = this._eventObj(dataset, "linksAdded");
		eventObj.datasetLinkCount = dataset.getLinkCount() + links.length;		
		this._mixinLinks(eventObj, links);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getLinksRemovedEvent: function(dataset, links) {
		var eventObj = this._eventObj(dataset, "linksRemoved");
		eventObj.datasetLinkCount = dataset.getLinkCount() - links.length;		
		this._mixinLinks(eventObj, links);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getLinkStatusChangedEvent: function(dataset, links) {
		var eventObj = this._eventObj(dataset, "linkStatusChanged");
		this._mixinLinks(eventObj, links);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getLinkPropertiesChangedEvent: function(dataset, links) {
		var eventObj = this._eventObj(dataset, "linkPropertiesChanged");
		this._mixinLinks(eventObj, links);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getLinkChangedEvent: function(dataset, link, relationships, relationshipAdded) {
		var eventObj = this._eventObj(dataset, "linkPropertiesChanged");
		eventObj.reason = (relationshipAdded ? "relationshipAdded" : "relationshipRemoved");
		eventObj.link = link.getAttributes();
		this._mixinRelationships(eventObj, links);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getLabelChangedEvent: function(dataset, newLabel) {
		var eventObj = this._eventObj(dataset, "labelChanged");
		eventObj.label = label;
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	_eventObj: function(dataset, type) {
		return {
			"id": dataset.getEventId(),
			"type": type,
			"providerId": dataset.getProviderId(),
			"datasourceId": dataset.getDatasourceId(),
			"datasetId": dataset.getId(),
			"version": dataset.getVersion()
		};		
	},
	
	_mixinItems: function(eventObj, items) {
		var itemObjs = [];
		var itemLength = items.length;
		for (var itemIndex = 0;  itemIndex < itemLength;  ++itemIndex) {
			itemObjs[itemIndex] = items[itemIndex].getAttributes();
		}
		eventObj.items = itemObjs;
	},
	
	_mixinLinks: function(eventObj, links) {
		var linkObjs = [];
		var linkLength = links.length;
		for (var linkIndex = 0;  linkIndex < linkLength;  ++linkIndex) {
			linkObjs[linkIndex] = links[linkIndex].getAttributes();
		}
		eventObj.links = linkObjs;
	},
	
	_mixinRelationships: function(eventObj, relationships) {
		var relationshipObjs = [];
		var relationshipLength = relationships.length;
		for (var relationshipIndex = 0;  relationshipIndex < relationshipLength;  ++relationshipIndex) {
			relationshipObjs[relationshipIndex] = relationships[relationshipIndex].getAttributes();
		}
		eventObj.relationships = relationshipObjs;
	}
};
});

},
'dojox/mvc/StatefulModel':function(){
define([
	"dojo/_base/lang",
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/Stateful"
], function(lang, array, declare, Stateful){
	/*=====
		declare = dojo.declare;
		Stateful = dojo.Stateful;
	=====*/

	var StatefulModel = declare("dojox.mvc.StatefulModel", [Stateful], {
		// summary:
		//		The first-class native JavaScript data model based on dojo.Stateful
		//		that wraps any data structure(s) that may be relevant for a view,
		//		a view portion, a dijit or any custom view layer component.
		//
		//  description:
		//		A data model is effectively instantiated with a plain JavaScript
		//		object which specifies the initial data structure for the model.
		//
		//		|	var struct = {
		//		|		order	: "abc123",
		//		|		shipto	: {
		//		|			address	: "123 Example St, New York, NY",
		//		|			phone	: "212-000-0000"
		//		|		},
		//		|		items : [
		//		|			{ part : "x12345", num : 1 },
		//		|			{ part : "n09876", num : 3 }
		//		|		]
		//		|	};
		//		|
		//		|	var model = dojox.mvc.newStatefulModel({ data : struct });
		//
		//		The simple example above shows an inline plain JavaScript object
		//		illustrating the data structure to prime the model with, however
		//		the underlying data may be made available by other means, such as
		//		from the results of a dojo.store or dojo.data query.
		//
		//		To deal with stores providing immediate values or Promises, a
		//		factory method for model instantiation is provided. This method
		//		will either return an immediate model or a model Promise depending
		//		on the nature of the store.
		//
		//		|	var model = dojox.mvc.newStatefulModel({ store: someStore });
		//
		//		The created data model has the following properties:
		//
		//		- It enables dijits or custom components in the view to "bind" to
		//		  data within the model. A bind creates a bi-directional update
		//		  mechanism between the bound view and the underlying data:
		//			- The data model is "live" data i.e. it maintains any updates
		//			  driven by the view on the underlying data.
		//			- The data model issues updates to portions of the view if the
		//			  data they bind to is updated in the model. For example, if two
		//			  dijits are bound to the same part of a data model, updating the
		//			  value of one in the view will cause the data model to issue an
		//			  update to the other containing the new value.
		//
		//		- The data model internally creates a tree of dojo.Stateful
		//		  objects that matches the input, which is effectively a plain
		//		  JavaScript object i.e. "pure data". This tree allows dijits or
		//		  other view components to bind to any node within the data model.
		//		  Typically, dijits with simple values bind to leaf nodes of the
		//		  datamodel, whereas containers bind to internal nodes of the
		//		  datamodel. For example, a datamodel created using the object below
		//		  will generate the dojo.Stateful tree as shown:
		//
		//		|	var model = dojox.mvc.newStatefulModel({ data : {
		//		|		prop1	: "foo",
		//		|		prop2	: {
		//		|			leaf1	: "bar",
		//		|			leaf2	: "baz"
		//		|		}
		//		|	}});
		//		|
		//		|	// The created dojo.Stateful tree is illustrated below (all nodes are dojo.Stateful objects)
		//		|	//
		//		|	//	                o  (root node)
		//		|	//	               / \
		//		|	//	 (prop1 node) o   o (prop2 node)
		//		|	//	                 / \
		//		|	//	   (leaf1 node)	o   o (leaf2 node)
		//		|	//
		//		|	// The root node is accessed using the expression "model" (the var name above). The prop1
		//		|	// node is accessed using the expression "model.prop1", the leaf2 node is accessed using
		//		|	// the expression "model.prop2.leaf2" and so on.
		//
		//		- Each of the dojo.Stateful nodes in the model may store data as well
		//		  as associated "meta-data", which includes things such as whether
		//		  the data is required or readOnly etc. This meta-data differs from
		//		  that maintained by, for example, an individual dijit in that this
		//		  is maintained by the datamodel and may therefore be affected by
		//		  datamodel-level constraints that span multiple dijits or even
		//		  additional criteria such as server-side computations.
		//
		//		- When the model is backed by a dojo.store or dojo.data query, the
		//		  client-side updates can be persisted once the client is ready to
		//		  "submit" the changes (which may include both value changes or
		//		  structural changes - adds/deletes). The datamodel allows control
		//		  over when the underlying data is persisted i.e. this can be more
		//		  incremental or batched per application needs.
		//
		//		There need not be a one-to-one association between a datamodel and
		//		a view or portion thereof. For example, multiple datamodels may
		//		back the dijits in a view. Indeed, this may be useful where the
		//		binding data comes from a number of data sources or queries, for
		//		example. Just as well, dijits from multiple portions of the view
		//		may be bound to a single datamodel.
		//
		//		Finally, requiring this class also enables all dijits to become data
		//		binding aware. The data binding is commonly specified declaratively
		//		via the "ref" property in the "data-dojo-props" attribute value.
		//
		//		To illustrate, the following is the "Hello World" of such data-bound
		//		widget examples:
		//
		//		|	<script>
		//		|		dojo.require("dojox.mvc");
		//		|		dojo.require("dojo.parser");
		//		|		var model;
		//		|		dojo.addOnLoad(function(){
		//		|			model = dojox.mvc.newStatefulModel({ data : {
		//		|				hello : "Hello World"
		//		|			}});
		//		|			dojo.parser.parse();
		//		|		}
		//		|	</script>
		//		|
		//		|	<input id="helloInput" dojoType="dijit.form.TextBox"
		//		|		ref="model.hello">
		//
		//		or
		//
		//		|	<script>
		//		|		var model;
		//		|		require(["dojox/mvc", "dojo/parser"], function(dxmvc, parser){
		//		|			model = dojox.mvc.newStatefulModel({ data : {
		//		|				hello : "Hello World"
		//		|			}});
		//		|			parser.parse();
		//		|		});
		//		|	</script>
		//		|
		//		|	<input id="helloInput" data-dojo-type="dijit.form.TextBox"
		//		|		data-dojo-props="ref: 'model.hello'">
		//
		//		Such data binding awareness for dijits is added by extending the
		//		dijit._WidgetBase class to include data binding capabilities
		//		provided by dojox.mvc._DataBindingMixin, and this class declares a
		//		dependency on dojox.mvc._DataBindingMixin.
		//
		//		The presence of a data model and the data-binding capabilities
		//		outlined above support the flexible development of a number of MVC
		//		patterns on the client. As an example, CRUD operations can be
		//		supported with minimal application code.
	
		// data: Object
		//		The plain JavaScript object / data structure used to initialize
		//		this model. At any point in time, it holds the lasted saved model
		//		state.
		//		Either data or store property must be provided.
		data: null,

		// store: dojo.store.DataStore
		//		The data store from where to retrieve initial data for this model.
		//		An optional query may also be provided along with this store.
		//		Either data or store property must be provided.
		store: null,
	
		// valid: boolean
		//		Whether this model deems the associated data to be valid.
		valid: true,

		// value: Object
		//		The associated value (if this is a leaf node). The value of
		//		intermediate nodes in the model is not defined.
		value: "",

		//////////////////////// PUBLIC METHODS / API ////////////////////////

		reset: function(){
			// summary:
			//		Resets this data model values to its original state.
			//		Structural changes to the data model (such as adds or removes)
			//		are not restored.
			if(lang.isObject(this.data) && !(this.data instanceof Date) && !(this.data instanceof RegExp)){	
				for(var x in this){
					if(this[x] && lang.isFunction(this[x].reset)){
						this[x].reset();
					}
				}
			}else{
				this.set("value", this.data);
			}
		},

		commit: function(/*"dojo.store.DataStore?"*/ store){
			// summary:
			//		Commits this data model:
			//		- Saves the current state such that a subsequent reset will not
			//		  undo any prior changes.
			//		- Persists client-side changes to the data store, if a store
			//		  has been supplied as a parameter or at instantiation.
			//	store:
			//		dojo.store.DataStore
			//		Optional dojo.store.DataStore to use for this commit, if none
			//		provided but one was provided at instantiation time, that store
			//		will be used instead.
			this._commit();
			var ds = store || this.store;
			if(ds){
				this._saveToStore(ds);
			}
		},

		toPlainObject: function(){
			// summary:
			//		Produces a plain JavaScript object representation of the data
			//		currently within this data model.
			// returns:
			//		Object
			//		The plain JavaScript object representation of the data in this
			//		model.
			var ret = {};
			var nested = false;
			for(var p in this){
				if(this[p] && lang.isFunction(this[p].toPlainObject)){
					if(!nested && typeof this.get("length") === "number"){
						ret = [];
					}
					nested = true;
					ret[p] = this[p].toPlainObject();
				}
			}
			if(!nested){
				if(this.get("length") === 0){
					ret = [];
				}else{				
					ret = this.value;
				}
			}
			return ret;
		},

		add: function(/*String*/ name, /*dojo.Stateful*/ stateful){
			// summary:
			//		Adds a dojo.Stateful tree represented by the given
			//		dojox.mvc.StatefulModel at the given property name.
			//	name:
			//		The property name to use whose value will become the given
			//		dijit.Stateful tree.
			//	stateful:
			//		The dojox.mvc.StatefulModel to insert.
			// description:
			//		In case of arrays, the property names are indices passed
			//		as Strings. An addition of such a dojo.Stateful node
			//		results in right-shifting any trailing sibling nodes.
			var n, n1, elem, elem1, save = new StatefulModel({ data : "" });
			if(typeof this.get("length") === "number" && /^[0-9]+$/.test(name.toString())){
				n = name;
				if(!this.get(n)){
					if(this.get("length") == 0 && n == 0){ // handle the empty array case
						this.set(n, stateful);
					} else {
						n1 = n-1;
						if(!this.get(n1)){
							throw new Error("Out of bounds insert attempted, must be contiguous.");
						}
						this.set(n, stateful);
					}
				}else{
					n1 = n-0+1;
					elem = stateful;
					elem1 = this.get(n1);
					if(!elem1){
						this.set(n1, elem);
					}else{
						do{
							this._copyStatefulProperties(elem1, save);
							this._copyStatefulProperties(elem, elem1);
							this._copyStatefulProperties(save, elem);
							this.set(n1, elem1); // for watchers
							elem1 = this.get(++n1);
						}while(elem1);
						this.set(n1, elem);
					}
				}
				this.set("length", this.get("length") + 1);
			}else{
				this.set(name, stateful);
			}
		},

		remove: function(/*String*/ name){
			// summary:
			//		Removes the dojo.Stateful tree at the given property name.
			//	name:
			//		The property name from where the tree will be removed.
			// description:
			//		In case of arrays, the property names are indices passed
			//		as Strings. A removal of such a dojo.Stateful node
			//		results in left-shifting any trailing sibling nodes.
			var n, elem, elem1;
			if(typeof this.get("length") === "number" && /^[0-9]+$/.test(name.toString())){
				n = name;
				elem = this.get(n);
				if(!elem){
					throw new Error("Out of bounds delete attempted - no such index: " + n);
				}else{
					this._removals = this._removals || [];
					this._removals.push(elem.toPlainObject());
					n1 = n-0+1;
					elem1 = this.get(n1);
					if(!elem1){
						this.set(n, undefined);
						delete this[n];
					}else{
						while(elem1){
							this._copyStatefulProperties(elem1, elem);
							elem = this.get(n1++);
							elem1 = this.get(n1);
						}
						this.set(n1-1, undefined);
						delete this[n1-1];
					}
					this.set("length", this.get("length") - 1);
				}
			}else{
				elem = this.get(name);
				if(!elem){
					throw new Error("Illegal delete attempted - no such property: " + name);
				}else{
					this._removals = this._removals || [];
					this._removals.push(elem.toPlainObject());
					this.set(name, undefined);
					delete this[name];
				}
			}
		},

		valueOf: function(){
			// summary:
			//		Returns the value representation of the data currently within this data model.
			// returns:
			//		Object
			//		The object representation of the data in this model.
			return this.toPlainObject();
		},

		toString: function(){
			// summary:
			//		Returns the string representation of the data currently within this data model.
			// returns:
			//		String
			//		The object representation of the data in this model.
			return this.value === "" && this.data ? this.data.toString() : this.value.toString();
		},

		//////////////////////// PRIVATE INITIALIZATION METHOD ////////////////////////

		constructor: function(/*Object*/ args){
			// summary:
			//		Instantiates a new data model that view components may bind to.
			//		This is a private constructor, use the factory method
			//		instead: dojox.mvc.newStatefulModel(args)
			//	args:
			//		The mixin properties.
			// description:
			//		Creates a tree of dojo.Stateful objects matching the initial
			//		data structure passed as input. The mixin property "data" is
			//		used to provide a plain JavaScript object directly representing
			//		the data structure.
			// tags:
			//		private
			var data = (args && "data" in args) ? args.data : this.data; 
			this._createModel(data);
		},

		//////////////////////// PRIVATE METHODS ////////////////////////

		_createModel: function(/*Object*/ obj){
			// summary:
			//		Create this data model from provided input data.
			//	obj:
			//		The input for the model, as a plain JavaScript object.
			// tags:
			//		private
			if(lang.isObject(obj) && !(obj instanceof Date) && !(obj instanceof RegExp) && obj !== null){
				for(var x in obj){
					var newProp = new StatefulModel({ data : obj[x] });
					this.set(x, newProp);
				}
				if(lang.isArray(obj)){
					this.set("length", obj.length);
				}
			}else{
				this.set("value", obj);
			}
		},

		_commit: function(){
			// summary:
			//		Commits this data model, saves the current state into data to become the saved state, 
			//		so a reset will not undo any prior changes.  
			// tags:
			//		private
			for(var x in this){
				if(this[x] && lang.isFunction(this[x]._commit)){
					this[x]._commit();
				}
			}
			this.data = this.toPlainObject();
		},

		_saveToStore: function(/*"dojo.store.DataStore"*/ store){
			// summary:
			//		Commit the current values to the data store:
			//		- remove() any deleted entries
			//		- put() any new or updated entries
			//	store:
			//		dojo.store.DataStore to use for this commit.
			// tags:
			//		private
			if(this._removals){
				array.forEach(this._removals, function(d){
					store.remove(store.getIdentity(d));
				}, this);
				delete this._removals;
			}
			var dataToCommit = this.toPlainObject();
			if(lang.isArray(dataToCommit)){
				array.forEach(dataToCommit, function(d){
					store.put(d);
				}, this);
			}else{
				store.put(dataToCommit);
			}
		},

		_copyStatefulProperties: function(/*dojo.Stateful*/ src, /*dojo.Stateful*/ dest){
			// summary:
			//		Copy only the dojo.Stateful properties from src to dest (uses
			//		duck typing).
			//	src:
			//		The source object for the copy.
			//	dest:
			//		The target object of the copy.
			// tags:
			//		private
			for(var x in src){
				var o = src.get(x);
				if(o && lang.isObject(o) && lang.isFunction(o.get)){
					dest.set(x, o);
				}
			}
		}
	});

	return StatefulModel;
});

},
'ibm/tivoli/navmodel/DatasetLazyTreeStore':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/DatasetStore,ibm/tivoli/navmodel/Dataset"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */
 
dojo.provide("ibm.tivoli.navmodel.DatasetLazyTreeStore");

// includes
dojo.require("ibm.tivoli.navmodel.DatasetStore");
dojo.require("ibm.tivoli.navmodel.Dataset");
	
///////////////////////////////////////////
// This dojo datastore is compatible with:
// 		dojox.grid.LazyTreeGrid	
//
// Do NOT use for:
//		dijit.Tree
//		dojox.grid.TreeGrid
// instead use DatasetTreeStore
///////////////////////////////////////////	
ibm.tivoli.navmodel.DatasetLazyTreeStore._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.DatasetLazyTreeStore");
	
dojo.declare("ibm.tivoli.navmodel.DatasetLazyTreeStore", ibm.tivoli.navmodel.DatasetStore, {
	// private members 
	_clearCache: false,
	_origins: null,
	_children: null,
	_sort: null,
	_condition: null,
	_sync: false,
	_trace: ibm.tivoli.navmodel.DatasetLazyTreeStore._trace,
	
	/*=====
	constructor_kwArgs: function(dataset, defaultProperties, relationships) {
		// summary:
		//		An anonymous object used to configure the datastore
		// dataset: Object
		//		A ibm.tivoli.navmodel.Dataset instance
		// defaultProperties: String[]
		//		An array of property id values that are pre-fetched with the items
		// relationships: String  (optional)
		//		Comma delimited list of relationships to determine related items; 
		//		can also be keywords: child, parent, all.  If not specified the value is
		//		retrieved from the individual item that is being expanded			
	},
	=====*/
	
	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructor for dojo datastore;
		//		that receives item data (in tree format) from an ibm.tivoli.navmodel.Dataset
		//		Base class will mixin kwArgs
		
		// LazyTreeGrid seems to expects us to behave in a synchronous manner
		// when dynamic refresh is being processed
		this._sync = this.dataset.isEventsEnabled();
		
		this._clearOrigins();
		this._clearChildren();		
	},		
	
	//////////////////////////////////////////////////////////////
	// dojo.data.api.Identity
	//////////////////////////////////////////////////////////////
	
    getIdentity: function(/*Object*/ item){
		// summary:
    	//		Returns a unique identifier for an item. The return value will be
    	//		either a string or something that has a toString() method.
    	// item:
    	//		The item from the store from which to obtain its identifier.
		// exceptions:
    	//		Conforming implementations may throw an exception or return null if
    	//		item is not an item.
		if (!this.isItem(item)){
  			throw new Error("Specified item argument is not a valid item.");
  		}
  		
		return item.getId();		
    },
    
	/*=====
	_onItem: function(item) {},
	=====*/
	
	/*=====
	_onError: function(error) {},
	=====*/
	
	/*=====
	fetchItemByIdentity_kwArgs: function(identity, onItem, onError, scope) {
		// identity: String|Object
		//		The identity parameter is the identity of the item you wish to locate and load
		//		This attribute is required. It should be a string or an object that toString()
		//		can be called on.
		// onItem: _onItem
		//		The onItem parameter is the callback to invoke when the item has been loaded. It takes only one
		//		parameter, the item located, or null if none found.
		// onError: _onError
		//		The onError parameter is the callback to invoke when the item load encountered an error. It takes 
		//		only one parameter, the error object
		// scope: Object
		//		If a scope object is provided, all of the callback functions (onItem,
		//		onError, etc) will be invoked in the context of the scope object.
	},
	=====*/
	
 	fetchItemByIdentity: function(/*fetchItemByIdentity_kwArgs*/ kwArgs) {
		// summary:
		//		Given the identity of an item, this method returns the item that has
		//		that identity through the onItem callback. Conforming implementations
		//		should return null if there is no item with the given identity.
		//		Implementations of fetchItemByIdentity() may sometimes return an item
		//		from a local cache and may sometimes fetch an item from a remote server
	 	var args = {
 			identity: null,
 			onItem: function(item) {},
 			onError: function(error) {},
 			scope: null
 		};
 		dojo.mixin(args, kwArgs);
 		
 		var item = null;
 		var error = null;
 		if (args.identity) {
 			try { 			
  				item = this.dataset.getItem({itemId: args.identity, properties: this.defaultProperties});
  			}
  			catch(err) {
  				error = err;
  			}  			
  		}  		
  		
 		if (item) {
 			args.onItem.call(args.scope, item);
 		}
 		else {
 			args.onError.call(args.scope, error ? error : "Could not fetch item for identity=" + args.identity);
 		}		
	},
	
	
	//////////////////////////////////////////////////////////////
	// dojo.data.api.Read
	//////////////////////////////////////////////////////////////
	
	/*=====
	fetch_onBegin: function(size, request){},
	=====*/
	
	/*=====
	fetch_onItem: function(item, request){},
	=====*/
	
	/*=====
	fetch_onComplete: function(items, request){},
	=====*/
	
	/*=====
	fetch_onError: function(errorData, request){},
	=====*/
	
	/*=====
	fetch_kwArgs: function(query, queryOptions, onBegin, onItem, onComplete, onError, scope, start, count, sort) {
		// summary:
		//		The kwArgs parameter may either be an instance of conforming to 
		//		dojo.data.api.Request or may be a simple anonymous object
		//		containing any of the above parameters.
		// query: String|Object
		//		The query parameter is used to specify search and filter condition information for the fetch.
		//		Only those items that match the search and/or filter will be returned.
		//		|	{	ids: {
		//		|			propId: boolean		// indicates propIds involved in the expression
		//		|		},
		//		|		expression: string,		// expression involiving propId's and values  (and/or together)
		//		|								// example:  ((Name contains 'abc' || Type contains 'abc)) and (Size > 100)
		//		|
		//		|		// the following is for compatability with legacy datastore
		//		| 		navModel: {
		//		|			search: {
		//		|				term: string,		the search text
		//		|				attributes: array	["property id1", "property id2", ...]
		//		|			}
		//		|		}
		//		|	}
		// queryOptions: Object?
		//		The queryOptions parameter is an optional parameter that can be used to modify the query.  The
		//		ignoreCase case can be specified to globally make all of the filter conditions case sensitive/insensitive.
		//		|	{ ignoreCase: boolean
		//		|	}
		// onBegin: fetch_onBegin
		//		If an onBegin callback function is provided, the callback function will be called just once, 
		//		before the first onItem callback is called. The onBegin callback function will be passed two arguments, 
		//		the the total number of items identified and the Request object.  If the total number is
		//		unknown, then size will be -1.  Note that size is not necessarily the size of the collection of items 
		//		returned from the query, as the request may have specified to return only a subset of the total set 
		//		of items through the use of the start and count parameters.
		// onItem: fetch_onItem
		//		If an onItem callback function is provided, the callback function will be called as each item 
		//		in the result is received. The callback function will be passed two arguments: the item itself, 
		//		and the Request object.
		// onComplete: fetch_onComplete
		//		If an onComplete callback function is provided, the callback function will be called just once,
		//		after the last onItem callback is called. Note that if the onItem callback is not present,
		//		then onComplete will be passed an array containing all items which matched the query and the 
		//		request object. If the onItem callback is present, then onComplete is called as: onComplete(null, request).
		// onError: fetch_onError
		//		If an onError callback function is provided, the callback function will be called if there is any 
		//		sort of error while attempting to execute the query. The onError callback function will be passed two arguments:
		//		an Error object and the Request object.
		// scope: Object
		//		If a scope object is provided, all of the callback functions (onItem, onComplete, onError, etc) 
		//		will be invoked in the context of the scope object.
		// start: int
		//		If a start parameter is specified, this is a indication to the datastore to
		//		only start returning items once the start number of items have been located and
		//		skipped.  When this parameter is paired with 'count', the store should be able
		//		to page across queries with millions of hits by only returning subsets of the
		//		hits for each query
		// count: int
		//		 If a count parameter is specified, this is a indication to the datastore to
		//		 only return up to that many items.  This allows a fetch call that may have
		//		millions of item matches to be paired down to something reasonable.
		// sort: Object[]
		//		If a sort parameter is specified, this is a indication to the datastore to
		//		sort the items in some manner before returning the items.  The array is an array of
		//		javascript objects that must conform to the following format to be applied to the
		//		fetching of items:
		//		|	{
		//		|		attribute: attribute || attribute-name-string,
		//		|		descending: true|false;   // Optional.  Default is false.
		//		|	}
	},
	=====*/
		
	fetch: function(/*fetch_kwArgs*/ kwArgs) {
		// summary:
		//		This datastore exposes the tree origin nodes of the Dataset and allows for
		//		lazy loading of children afterwards.
		//		The format and expectations of stores is that they operate in a generally asynchronous
		//		manner, therefore callbacks are always used to return items located by the fetch parameters.
		// description:
		//		A Request object will always be returned and is returned immediately.
		//		The basic request is nothing more than the keyword args passed to fetch and
		//		an additional function attached, abort().  The returned request object may then be used
		//		to cancel a fetch.  All data items returns are passed through the callbacks defined in the
		//		fetch parameters and are not present on the 'request' object.
		// returns:
		//		The fetch() method will return a javascript object conforming to the API
		//		defined in dojo.data.api.Request.  In general, it will be the keywordArgs
		//		object returned with the required functions in Request.js attached.
		//		Its general purpose is to provide a convenient way for a caller to abort an
		//		ongoing fetch.
		// exceptions:
		//		Throws an exception if the query is not valid, or if the query is required but was not supplied.
		var args = {
			onBegin: function(size, request) {},
			onItem: function(item, request) {},
			onComplete: function(items, request){},
			onError: function(errorData, request){},
			scope: null,
			abort: function() {}
		};
		dojo.mixin(args, kwArgs);
		
		// make sure event handling is set up
		this._validateEventHandler();
		
		// TED refresh is occurring
		this._clearCache = false;
		if (this.isRefresh()) {
			this._clearOrigins();
			this._clearChildren();
			this._clearCache = this.dataset.isClearCacheOnRefresh();						
		}
		
		// extract sort (see if it has changed from last time)
		var sort = this._getSort(kwArgs.sort);
		if ((sort && !this._sort) || (!sort && this._sort) || (sort !== this._sort)) {
			this._clearOrigins();
			this._clearChildren();			
		}
		this._sort = sort;			
						
		// extract condition (see if it has changed from last time)
		var condition = this._getCondition(args.query);	
		if ((condition && !this._condition) || (!condition && this._condition) || (condition !== this._condition)) {
			this._clearOrigins();
			this._clearChildren();			
		}
		this._condition = condition;
		
		// query was supplied
		if (args.query) {
			// fetch flat filtered list with no hierarchy
			if (this._condition) {
				this._fetchItems(args);
			}
			// fetch children for "parentId"
			else if (args.query.parentId) {
				if (this._trace.isFinest()) {
					this._trace.finest("fetching children for parent=" + args.query.parentId + " start=" + kwArgs.start + " count=" + kwArgs.count, "fetch");
				}
				var parentItem = this._getItem(args.query.parentId);
				this._fetchChildren(args, parentItem);
			}
			// fetch origin items
			else if (args.query.origins) {
				if (this._trace.isFinest()) {
					this._trace.finest("fetching origins start=" + kwArgs.start + " count=" + kwArgs.count, "fetch");
				}				
				this._fetchOrigins(args);
			}
			// fetch all items
			else {
				this._fetchItems(args);
			}
		}		
		// query was not supplied; fetch all items
		else {
			this._fetchItems(args);
		}
		       
        return args;
    },
    
    getAttributes: function(/*Object*/ item) {
		// summary:
		//		Returns an array with all the attributes that this item has.  This
		//		method will always return an array; if the item has no attributes
		//		at all, getAttributes() will return an empty array: [].
		// item:
		//		The item to access attributes on.
		//		ibm.tivoli.navmodel.Item object
		// exceptions:
		//		Throws an exception if *item* is not an item
		var attributes = this.inherited(arguments);
  		
		// add "hasChildren" attribute
		if (this._hasChildren(item)) {
			attributes.push("hasChildren");			
		}  		
		
		return attributes;		
    },
    
    getValue: function(/*Object*/ item, /*String*/ attribute, /*Object?*/ defaultValue) {
		// summary:
		//		Returns a single attribute value. Returns defaultValue if and only if item does not have a 
		//		value for attribute. Returns null if and only if null was explicitly set as the attribute value. 
		//		Returns undefined if and only if the item does not have a value for the given attribute 
		//		(which is the same as saying the item does not have the attribute).
		// item:
		//		The item to access value on.
		//		ibm.tivoli.navmodel.Item object
		// attribute:
		//		The attribute to access represented as a string.
		// defaultValue:
		//		Optional. A default value to use for the getValue return if the attribute does not exist or 
		//		has no value.
		// exceptions:
		//		Throws an exception if *item* is not an item, or *attribute* is not a string
    	if (arguments.length < 2) {
            throw new Error("Incorrect number of arguments specified: expecting 2 (item, attribute)");
        }
        
        if (!this.isItem(item)) {
            throw new Error("Specified item argument is not a valid item.");
        }
        
        if (!dojo.isString(attribute)) {
            throw new Error("Specified attribute argument is not a string.");
        }  
               
        var value = defaultValue;
        
        // asking if item has children
        if (attribute === "hasChildren") {
        	value = this._hasChildren(item);
        }
        // asking for entire item
        else if (this.returnItem) {
        	value = item;
        }
		// asking for item property
        else {
        	var prop = item.getProperty(attribute);
        	if (prop) {
        		value = prop.getDisplayValue();            
        	}
        }            
                           	
        return value;
    },

    // GRIDX
    getValues: function(/*Object*/ item, /*String*/ attribute) {
		// summary:
		//		This getValues() method works just like the getValue() method, but getValues()
		//		always returns an array rather than a single attribute value.  The array
		//		may be empty, may contain a single attribute value, or may contain many
		//		attribute values.
		//		
		//		If the item does not have a value for the given attribute, then getValues()
		//		will return an empty array: [].  (So, if store.hasAttribute(item, attribute)
		//		returns false, then store.getValues(item, attribute) will return [].)
		// item:
		//		The item to access values on.
		//		ibm.tivoli.navmodel.Item object
		// attribute:
		//		The attribute to access represented as a string.
		// exceptions:
		//		Throws an exception if *item* is not an item, or *attribute* is not a string
    	if (arguments.length < 2) {
            throw new Error("Incorrect number of arguments specified: expecting 2 (item, attribute)");
        }
        
        if (!this.isItem(item)) {
            throw new Error("Specified item argument is not a valid item.");
        }
        
        if (!dojo.isString(attribute)) {
            throw new Error("Specified attribute argument is not a string.");
        }
        
        var values = [];
        
        // asking for the "children" of this item
        if (attribute === "children") {
            var args = {
            	onBegin: function(size, request) {},
            	onItem: function(item, request) {},
            	onComplete: function(items, request){
            		values = items;
            	},
            	onError: function(errorData, request){},
            	scope: null,
            	start: 0,
            	query: {parentId: item._item.id},
            	sync: true,
            	abort: function() {}
			};
            // With query set with parentId specified, run it back through fetch()
        	this.fetch(args);
        }
        // general attribute value
    	else {
    		values = [this.getValue(item, attribute)];
    	}
    	
    	return values;
    },
    
    /*=====
	loadItem_kwArgs: function(item, onItem, onError, scope) {
		// summary:
		//		An anonymous object that defines the item to load and callbacks to invoke when the
		//		load has completed.
		// item: Object
		//		The item parameter is an object that represents the item in question that should be
		//		contained by the store.  This attribute is required.
		//		ibm.tivoli.navmodel.Item object
		// onItem: _onItem
		//		The onItem parameter is the callback to invoke when the item has been loaded.	
		//		It takes only one parameter, the fully loaded item.
		// onError: _onError
		//		The onError parameter is the callback to invoke when the item load encountered an error.
		//		It takes only one parameter, the error object.
		// scope: Object
		//		If a scope object is provided, all of the callback functions (onItem, onError, etc)
		//		will be invoked in the context of the scope object.
		//		onItem.call(dojo.global(), item, request)
	},
	=====*/
	
	loadItem: function(/*loadItem_kwArgs*/ kwArgs) {
		var args = {
			item : null,
			onItem: function(item) {},
			onError: function(error){},
			scope: null
		};		
		dojo.advancedMixin(args, kwArgs);
	
		if (!this.isItem(args.item)){
  			throw new Error("Specified item argument is not a valid item.");
  		}
  		
		if (this.isItemLoaded(args.item)){
			return args.item;
		}
		
		try {
			var dataset = args.item.getDataset();
			var item = dataset.getItem({itemId: args.item.getId(), properties: this.defaultProperties});			
			args.onItem.call(args.scope, item);
		}
		catch(err) {
			args.onError.call(args.scope, err);
		}
	},
	
	//////////////////////////////////////////////////////////////
	// private methods
	///////////////////////////////////////////////////////////////
	
	_getObjectTypes: function() {
		// summary:
		//		Returns String[] of object types that can in the datastore
    	return {
    		"item": true
		};
    },
    
    _fetchOrigins: function(/*fetch_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch origin items for the tree
		
		// origin items cached
		if (this._origins) {
			var length = this._origins.length;
			var start = kwArgs.start ? kwArgs.start : 0;
			var count = kwArgs.count ? kwArgs.count : length;
			if (start + count > length) {
				count = length - start;
			}
			
			if (this._trace.isFinest()) {
				this._trace.finest("datasetId=" + this.dataset.getId() + " returning cached origins; start=" + start + " count=" + count, "_fetchOrigins");
			}
				
			kwArgs.onBegin.call(kwArgs.scope, length, kwArgs);				
			for(var index = 0;  index < count;  ++index) {
				if (this._trace.isFinest()) {
					this._trace.finest("cached origins[" + (start+index) + "]=" + this._origins[start+index].getId(), "_fetchOrigins");
				}                 	                	
                kwArgs.onItem.call(kwArgs.scope, this._origins[start+index], kwArgs);                	
			}					
			kwArgs.onComplete.call(kwArgs.scope, this._origins.slice(start,start+count), kwArgs);			
		}
		// fetch origin items
		else {
			var originArgs = {
				clearCache: this._clearCache,
				start: kwArgs.start,
				count: kwArgs.count,
				sort: this._sort,
				properties: this.defaultProperties,
				sync: this._sync,
				onComplete: dojo.hitch(this, function(originItems, originArgs) {				
					// we have origin items		
					if (originItems && originItems.length > 0) {
						var originsLength = originItems.length;
						
						if (this._trace.isFinest()) {
							this._trace.finest("datasetId=" + this.dataset.getId() + " returning origins; start=" + kwArgs.start + " count=" + kwArgs.count, "_fetchOrigins");
						}
			
						kwArgs.onBegin.call(kwArgs.scope, originArgs.filteredRows, kwArgs);				
						for(var originIndex = 0;  originIndex < originsLength;  ++originIndex) {
							if (this._trace.isFinest()) {
								this._trace.finest("origins[" + (kwArgs.start+originIndex) + "]=" + originItems[originIndex].getId(), "_fetchOrigins");
							}                	                	
              				kwArgs.onItem.call(kwArgs.scope, originItems[originIndex], kwArgs);                	
						}					
						kwArgs.onComplete.call(kwArgs.scope, originItems, kwArgs);
						
						// we received all of the origins so cache them
						if (originsLength === originArgs.filteredRows) {
							this._origins = originItems;
						}				
					}
					// no origins declared
					else {						
						this._fetchItems(kwArgs);					
					}
				}),			
				onError: dojo.hitch(this, function(error, exception, ioArgs) {
				console.log("ERROR: ");
					kwArgs.onError.call(kwArgs.scope, error, kwArgs);
				})						
			}; 		
			this.dataset.getOriginItems(originArgs);
		}	
    },    
    
    // GRIDX
    hasChildren: function(id, item) {
       // summary:
       //        Mapping to _hasChildren(item) that is needed for use with GridX based tree table

       return this._hasChildren(item);
    },
    
    _hasChildren: function(/*Object*/ item) {
		// summary:
		//		Indicates if the item has children items
		
		// if we have a filter; then we are showing a flat list
		// so we would never want to indicate children present
		if (this._condition) {
			return false;
		}
		
		// lazyLoad indicates that there might be children;
		// lazyLoadRelationships indicates that there is children and this is the relationship to fetch them
    	return (item && (item.isLazyLoad() || item.getLazyLoadRelationships())) ? true : false;  // GRIDX
    },
    
    _fetchChildren: function(kwArgs, item) {
    	// children cached
    	var children = this._getChildren(item.getId());
    	if (children) {
    		var length = children.length;
			var start = kwArgs.start ? kwArgs.start : 0;
			var count = kwArgs.count ? kwArgs.count : length;
			if (start + count > length) {
				count = length - start;
			}
					
			if (this._trace.isFinest()) {
				this._trace.finest("datasetId=" + this.dataset.getId() + " returning cached children for item=" + item.getId() + " start=" + start + " count=" + count, "_fetchChildren");
			}
				    			
			kwArgs.onBegin.call(kwArgs.scope, length, kwArgs);				
			for(var index = 0;  index < count;  ++index) {
				if (this._trace.isFinest()) {
					this._trace.finest("cached children[" + (start+index) + "]=" + children[start+index].getId(), "_fetchChildren");
				}                 	                	
                kwArgs.onItem.call(kwArgs.scope, children[start+index], kwArgs);                	
			}					
			kwArgs.onComplete.call(kwArgs.scope, children.slice(start,start+count), kwArgs);	
    	}
    	// fetch children
    	else {
    		
    		var relatedArgs = {
    			clearCache: this._clearCache,
    			start: kwArgs.start,
    			count: kwArgs.count,
    			relationships: this.relationships,
    			properties: this.defaultProperties,
    			sort: this._sort,
    			sync: kwArgs.sync ? kwArgs.sync : this._sync,  // GRIDX  		
    			onComplete: dojo.hitch(this, function(items, links, relatedArgs) {
    				if (this._trace.isFinest()) {
						this._trace.finest("datasetId=" + this.dataset.getId() + " returning children for item=" + item.getId() + " start=" + kwArgs.start + " count=" + kwArgs.count, "_fetchChildren");
					}			
					kwArgs.onBegin.call(kwArgs.scope, relatedArgs.filteredRows, kwArgs);
								
					if (items) {
						var itemsLength = items.length;
						for(var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {                 	                	
                			if (this._trace.isFinest()) {
								this._trace.finest("children[" + (kwArgs.start+itemIndex) + "]=" + items[itemIndex].getId(), "_fetchChildren");
							} 
							kwArgs.onItem.call(kwArgs.scope, items[itemIndex], kwArgs);                	
						}
						
						// we received all of the children so cache them
						if (itemsLength === relatedArgs.filteredRows) {
							this._setChildren(item.getId(), items);
						}
					}				
					
					kwArgs.onComplete.call(kwArgs.scope, items, kwArgs);
				}),			
				onError: dojo.hitch(this, function(error, exception, ioArgs) {
					kwArgs.onError.call(kwArgs.scope, error, kwArgs);				
				})
    		};
    		item.getRelatedItems(relatedArgs);
    	}
    	
    },
    
    _getItem: function(/*String*/ identity) {
    	return this.dataset.getItem({itemId: identity, properties: this.defaultProperties}); 
    },
        
    _fetchItems: function(/*fetch_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch all items for the dataset if the query did not indicate that origins should be returned
		
    	var itemArgs = {
			clearCache: this._clearCache,		
			properties: this.defaultProperties,
			sort: this._sort,
			condition: this._condition,
			sync: this._sync,
			onComplete: dojo.hitch(this, function(items, itemArgs) {
				kwArgs.onBegin.call(kwArgs.scope, itemArgs.filteredRows, kwArgs);
								
				if (items) {
					var itemsLength = items.length;
					for(var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {                 	                	
                		kwArgs.onItem.call(kwArgs.scope, items[itemIndex], kwArgs);                	
					}				
				} else {
					// 36285/KDM: Set items to empty array as causes exception in tree table
					items = {};
				}
				kwArgs.onComplete.call(kwArgs.scope, items, kwArgs);
			}),			
			onError: dojo.hitch(this, function(error, exception, ioArgs) {
				kwArgs.onError.call(kwArgs.scope, error, kwArgs);
			})
		}; 		
		this.dataset.getItems(itemArgs);
    },
    
    _clearOrigins: function() {
    	// summary:
		//		Clear cached origins
    	this._origins = null;
    },
    
    _clearChildren: function() {
		// summary:
		//		Clear cached children
    	this._children = {};    	
    },
    
    _getChildren: function(/*String*/ id) {
		// summary:
		//		Get cached children for parent id
    	return this._children[id];
    },
    
	_setChildren: function(/*String*/ id, /*String*/ children) {
		// summary:
		//		Set cached children for parent id
    	this._children[id] = children;
    },
    
    _getCondition: function(/*Object*/ query) {
		// summary: 
		//		Returns array of condition objects involved in the current filter
		//			ibm.tivoli.navmodel.[String|Number|Date|Boolean|Enum|Map]Condition
		this._conditionIds = null;
		var condition = null;
 		 		
		if (query && query.filterText && query.filterText.length > 0) {
			var filterText = query.filterText.replace(/'/g, "''");
			var propIds = this.defaultProperties;
			if (propIds) {
				this._conditionIds = {};				
				condition = "(";
				var length = propIds.length;
				for (var index = 0;  index < length;  ++index) {
					if (index > 0) {
						condition += " or ";
					}					
					var id = propIds[index];
					condition += "(" + id + " contains '" + filterText + "')";
					this._conditionIds[id] = true;
				}
				condition += ")";
			}
		}
		
		// 38638
		// include custom filter condition, if set
		condition = this._appendCustomFilterCondition(condition); 
		
		return condition;
 	},
 	 	
    //////////////////////////////////////////////////////////////
	// async event handlers
	///////////////////////////////////////////////////////////////
	
	_handleChangedEvent: function(/*Object*/ event) {
    	this._clearOrigins();
    	this._clearChildren();
    	this.onRefresh();
	},
	
	_handleItemPropertiesChangedEvent: function(/*Object*/ event) {
		var items = event.getItems();    	
    	var propsArray = event.getProperties();			
		if (items && propsArray) {
			var refresh = false;
    		var length = items.length;			
			for (var index = 0;  index < length;  ++index) {
				var properties = propsArray[index];				
				if (properties) {
					var propsLength = properties.length;
					for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
						var item = items[index];
						var property = properties[propIndex];	
						var propId = property.getId();						
						if (this._trace.isFinest()) {
							this._trace.finest("datasetId=" + event.getDataset().getId() + " itemId=" + item.getId() + " prop=" + propId + " value=" + property.getDisplayValue(), "_handleItemPropertiesChangedEvent");
						}
						
						this.onSet(item, propId, null, property.getDisplayValue());
						
						if (!refresh && (this._affectsSort(propId) || this._affectsCondition(propId))) {
							refresh = true;							
						}
					}										
				}				
			}
			
			if (refresh) {
				this.onRefresh();
			}		
		}	
	},
	
    _handleItemsAddedEvent: function(/*Object*/ event) {
    	var items = event.getItems();
		if (items) {
			var refresh = false;
    		var length = items.length;			
			for (var index = 0;  index < length;  ++index) {
				var item = items[index];				
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + event.getDataset().getId() + " itemId=" + item.getId() + " origin=" + item.isOrigin(), "_handleItemsAddedEvent");
				}
				
				this._clearChildren();
				
				if (this._hasCondition()) {
					refresh = true;
					this.onNew(item);					
				}
				else if (item.isOrigin()) {
					this._clearOrigins();
					this.onNew(item);					
				}
			}
			
			if (refresh) {
				this.onRefresh();
			}
		}    		
	},
	
	_handleItemsRemovedEvent: function(/*Object*/ event) {
		var items = event.getItems();
		if (items) {
			var length = items.length;			
			for (var index = 0;  index < length;  ++index) {
				var item = items[index];				
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + event.getDataset().getId() + " itemId=" + item.getId(), "_handleItemsRemovedEvent");
				}
				
				this._clearChildren();								
				this._clearOrigins();				
				this.onDelete(item);
			}
		}
	},
	
	_handleLinksAddedEvent:function(/*Object*/ event) {
		var links = event.getLinks();
		
		// if condition is set we have a flat list and don't need
		// to worry about any parent/child hierarchy from relationships
		if (links && !this._hasCondition()) {
			var linksLength = links.length;
			for (var linkIndex = 0;  linkIndex < linksLength;  ++linkIndex) {
				var link = links[linkIndex];
				var item1 = link.getItem1();
				var item2 = link.getItem2();
				var parent = null;
				var child = null;
								
				switch(link.getDirection()) {				
					case "1_2":
					case "both":
						parent = {item: item1, attribute: 'hasChildren'};
						if (!item1.getLazyLoadRelationships()) {
							item1.setLazyLoadRelationships(this.relationships ? this.relationships : "child");
						}
						child = item2;
						break;
					case "2_1":
						parent = {item: item2, attribute: 'hasChildren'};
						if (!item2.getLazyLoadRelationships()) {
							item2.setLazyLoadRelationships(this.relationships ? this.relationships : "parent");
						}
						child = item1;
				}					
				
				if (parent && child) {
					if (this._trace.isFinest()) {
						this._trace.finest("datasetId=" + event.getDataset().getId() + " parent=" + parent.item.getId() + " child=" + child.getId(), "_handleLinksAddedEvent");
					}
					this._clearChildren();
					this.onNew(child, parent);
				}
				else if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + event.getDataset().getId() + " ignoring linkId=" + link.getId(), "_handleLinksAddedEvent");
				}	
			}			
		}
	}	
});
});

},
'gridx/core/_Module':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/array",
	"dojo/_base/Deferred",
	"dojo/_base/connect",
	"dojo/aspect"
], function(declare, lang, array, Deferred, connect, aspect){

var isFunc = lang.isFunction,
	c = 'connect',	//To reduce code size

	
	moduleBase = declare(/*===== "gridx.core._Module", =====*/[], {
	/*=====
		// name: String
		//		The API set name of this module. This name represents the API set that this module implements, 
		//		instead of this module itself. Two different modules can have the same name, so that they provide
		//		two different implementations of this API set.
		//		For example, simple row selection and extended row selection are two modules implementing a same set of APIs.
		//		They can be used in two different grids in one page (maybe due to different requirements), 
		//		without worrying about conflicting with eachother. And any module of grid can be replaced by a new implementation
		//		without re-writing any other modules.
		//		This property is mandatary.
		name: "SomeModule",
		
		// forced: String[] 
		//		An array of module names. All these modules must exist, and have finished loading before this module loads.
		//		This property can be omitted.
		forced: [],
		
		// optional: String[] 
		//		An array of module names. These modules can be absent, but if they do exist, 
		//		they must be loaded before this module loads.
		//		This property can be omitted.
		optional: [],
		
		// required: []
		//		An array of module names. These modules must exist, but they can be loaded at any time.
		//		This property can be omitted.
		required: [],

		getAPIPath: function(){
			// summary: 
			//		This function defines how to access this module's methods from the grid object.
			// description:
			//		The returned object of this function will be "recursively" mixed into the grid object.
			//		That is, any property of object type in grid will be preserved. For example, if this function
			//		returns { abc: { def: 'ghi'} }, and the grid already has a property called "abc", and 
			//		grid.abc is { jkl: 'mno'}. Then after mixin, grid.abc will still have this jkl property:
			// |	{
			// |		abc: {
			// |			jkl: 'mno',
			// |			def: 'ghi'
			// |		}
			// |	}
			//		This mechanism makes it possible for different modules to provide APIs to a same sub-API object.
			//		Sub-API object is used to provide structures for grid APIs, so as to avoid API conflicts as much as possible.
			//		This function can be omitted.
			return {}
		},

		preload: function(args){
			// summary:
			//		Preload this module.
			// description:
			//		If this function exists, it is called after all modules are created ("new"-ed), but not yet loaded.
			//		At this time point, all the module APIs are already accessable, so all the mothods of those modules that
			//		do not need to load can be used here.
			//		Note that this function is not the "load" process, so the module dependancy is not honored. For example,
			//		if module A forcedly depends on module B, it is still possible that module A.preload is called before 
			//		module B.preload.
			//		This function can be omitted.
		},

		load: function(args, deferStartup){
			// summary: 
			//		Completely load this module.
			// description:
			//		This is the formal loading process of this module. This function will not be called until all the "forced"
			//		and existing "optional" modules are loaded. When the loading process of this module is finished (Note that
			//		this might be an async process), this.loaded.callback() must be called to tell any other modules that
			//		depend on this module.
			this.loaded.callback();
		},

		// grid: gridx.Grid
		//		Reference to the grid
		grid: null,
		
		// model: gridx.core.model.Model
		//		Reference to the grid model
		model: null,

		// loaded: dojo.Deferred
		//		Indicate when this module is completely loaded.
		loaded: null,
	=====*/
	
		
		constructor: function(grid, args){
			var t = this;
			t.grid = grid;
			t.model = grid.model;
			t.loaded = new Deferred;
			t._cnnts = [];
			t._sbscs = [];
			lang.mixin(t, args);
		},

		destroy: function(){
			var f = array.forEach;
			f(this._cnnts, connect.disconnect);
			f(this._sbscs, connect.unsubscribe);
		},

		arg: function(argName, defaultValue, validate){
			// summary:
			//		This method provides a normalized way to access module arguments.
			// description:
			//		There are two ways to provide module arguments when creating grid.
			//		One is to write them in the module declaration object:
			// |	var grid = new Grid({
			// |		......
			// |		modules: [
			// |			{
			// |				moduleClass: gridx.modules.Pagination,
			// |				initialPage: 1		//Put module arguments in module declaration object
			// |			}
			// |		],
			// |		......
			// |	});
			//		This way is straightforward, but quite verbose. And if user would like to set arguments 
			//		for pre-included core modules (e.g. Header, Body), he'd have to explictly declare the
			//		module. This would be too demanding for a grid user, so we need another approach.
			//		The other way is to treat them as grid arguments:
			// |	var grid = new Grid({
			// |		......
			// |		modules: [
			// |			gridx.modules.Pagination
			// |		],
			// |		paginationInitialPage: 1,	//Treat module arguments as grid arguments
			// |		......
			// |	});
			//		In this way, there's no need to provide a module declaration object, but one has to tell
			//		grid for which module the arguments is applied. One can simply put the module name at the
			//		front of every module argument:
			//			"pagination" -- module name
			//			"initialPage" -- module argument
			//			---------------------------------
			//			paginationInitialPage -- module argument treated as grid argument
			//		Note the first letter of the module arugment must be capitalized in the combined argument.
			//
			//		This "arg" method makes it possible to access module arguments without worring about where
			//		they are declared. The priority of every kinds of declarations are:
			//			Module argument > Grid argument > default value > Base class argument (inherited)
			//		After this method, the argument will automatically become module argument. But it is still
			//		recommended to alway access arguments by this.arg(...);
			// argName: String
			//		The name of this argument. This is the "short" name, not the name prefixed with module name.
			// defaultValue: anything?
			//		This value will by asigned to the argument if there's no user provided values.
			// validate: Function?
			//		This is a validation function and it must return a boolean value. If the user provided value
			//		can not pass validation, the default value will be used.
			//		Note if this function is provided, defaultValue must also be provided.
			// returns:
			//		The value of this argument.
			if(arguments.length == 2 && isFunc(defaultValue)){
				validate = defaultValue;
				defaultValue = undefined;
			}
			var t = this, g = t.grid, r = t[argName];
			if(!t.hasOwnProperty(argName)){
				var gridArgName = t.name + argName.substring(0, 1).toUpperCase() + argName.substring(1);
				if(g[gridArgName] === undefined){
					if(defaultValue !== undefined){
						r = defaultValue;
					}
				}else{
					r = g[gridArgName];
				}
			}
			t[argName] = (validate && !validate(r)) ? defaultValue : r;
			return r;	//anything
		},

		aspect: function(obj, e, method, scope, pos){
			var cnnt = aspect[pos || 'after'](obj, e, lang.hitch(scope || this, method), 1);
			this._cnnts.push(cnnt);
			return cnnt;
		},

		connect: function(obj, e, method, scope, flag){
			// summary:
			//		Connect an event handler to an event or function.
			// description:
			//		Similar to widget.connect, the scope of the listener will be default to this module.
			//		But in this API, the scope argument is placed behind the listener function, so as to
			//		avoid arguemnt checking logic.
			//		This method also allows conditional event firing using the flag argument.
			// obj: Object
			// e: String
			// method: String|Function
			// scope: Object?
			// flag: Anything
			//		If provided, the listener will only be triggered when grid._eventFlags[e] is set to flag.
			// returns:
			//		The connect handle
			var t = this,
				cnnt,
				g = t.grid,
				s = scope || t;
			if(obj === g && typeof e == 'string'){
				cnnt = connect[c](obj, e, function(){
					var a = arguments;
					if(g._eventFlags[e] === flag){
						if(isFunc(method)){
							method.apply(s, a);
						}else if(isFunc(s[method])){
							s[method].apply(s, a);
						}
					}
				});
			}else{
				cnnt = connect[c](obj, e, s, method);
			}
			t._cnnts.push(cnnt);
			return cnnt;	//Object
		},

		batchConnect: function(){
			// summary:
			//		Do a lot of connects in a batch.
			// description:
			//		This method is used to optimize code size.
			for(var i = 0, args = arguments, len = args.length; i < len; ++i){
				if(lang.isArrayLike(args[i])){
					this[c].apply(this, args[i]);
				}
			}
		},

		subscribe: function(topic, method, scope){
			// summary:
			//		Subscribe to a topic.
			// description:
			//		This is similar to widget.subscribe, except that the "scope" argument in this method is behind the listener function.
			// returns:
			//		The subscription handle
			var s = connect.subscribe(topic, scope || this, method);
			this._sbscs.push(s);
			return s;	//Object
		}
	}),
	mods = moduleBase._modules = {};
	
	moduleBase.register = function(modClass){
		var p = modClass.prototype;
		return mods[p.name || p.declaredClass] = modClass;
	};
	//! means not string, should be 'eval'ed.
	moduleBase._markupAttrs = ['id', 'name', 'field', 'width', 'dataType', '!formatter', '!decorator', '!sortable'];

	return moduleBase;
});

},
'ibm/tivoli/tip/navwidget/dojo/NavWidgetPluggableViewSelection':function(){
// wrapped by build app
define("ibm/tivoli/tip/navwidget/dojo/NavWidgetPluggableViewSelection", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/tip/navwidget/dojo/NavWidgetUtil,ibm/tivoli/tip/navwidget/dojo/NavWidgetPluggableViewObjectContext,dojox/collections/SortedList"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.dojo.NavWidgetPluggableViewSelection");

dojo.require("ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil");
dojo.require("ibm.tivoli.tip.navwidget.dojo.NavWidgetPluggableViewObjectContext");
dojo.require("dojox.collections.SortedList");

ibm.tivoli.tip.navwidget.dojo.NavWidgetPluggableViewSelection._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.tip.navwidget.dojo.NavWidgetPluggableViewSelection");

dojo.declare(
    "ibm.tivoli.tip.navwidget.dojo.NavWidgetPluggableViewSelection",
    ibm.tivoli.tip.navwidget.dojo.NavWidgetPluggableViewObjectContext,
    {
    	// summary:
    	//		Manages selection of items, links, relationships, and child datasets in a pluggable view. It is used internally
    	//		by the view. Override to implement custom selection.
    	// description:
    	//		Coming soon
    	
    	
  		// _changing: [private] ibm.tivoli.navmodel.Dataset
    	//		True is selection state is currently being updated
  		_changing: false,
  		
  		// _changing: [private] ibm.tivoli.navmodel.Dataset
    	//		True is selection state is currently being updated
  		_mode: "extended",
  		
  		// _trace: [private] Trace
  		//		Manages tracing for this class
  		_trace: ibm.tivoli.tip.navwidget.dojo.NavWidgetPluggableViewSelection._trace,
    
  		// operation:  this could be either a select or a deselect
		operation: "select",
  		
	 	isChanging: function(){
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		public
		
	 		return this._changing;
		},
		
		setMode: function(mode){
			//Let's clear selection on any mode changes
			if(this.getTotalSelected() > 0){
				this.deselectAll();
			}
			
			//Check that we have a valid mode passed in
			if(mode != "extended" && mode != "multiple" && mode != "single" && mode != "none"){
				this._mode = "extended";
				
				if (this._trace.isWarn()) {
					this._trace.warn("Invalid selection mode specified; mode = " + mode, "setMode");
				}
			}else{
				this._mode = mode;
			}
		},
		
		getMode: function(m){		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			// m:
			//		
			return this._mode;
		},
		
	 	setDataset: function(/*ibm.tivoli.navmodel.Dataset*/ dataset){
			// summary:
			//		TODO
			// dataset:
			//		The dataset we're managing selection fot
			// description:
			//		Coming soon
			// tags:
			//		public
		
			//Mark we're changing
			this._changing = true;
			
	 		//Set dataset
			this._dataset = dataset;
			
			//Clear out current selection
			if (!this.isEmpty()) {
				this._setSelectedObjectsBase(null, null, null, null);
			}
			
			//pass along the new dataset to the selection objects of the pluggable views
			var iterator = this._pluggableView.getPluggableViewIterator();
			while (!iterator.atEnd()) {
				var view = iterator.get().value;
				view.selection.setDataset(dataset);
			}
			
			//Mark we're done changing
			this._changing = false;
		},
	 	
	 	setSelectedObjectIds: function(/*String[]*/ itemIds, /*String[]*/ linkIds, /*String[]*/ relationshipIds){
			// summary:
			//		Coming soon
			// itemIds:
			//		An array of absolute item ids
			// linkIds:
			//		An array of absolute link ids
			// relationshipIds:
			//		An array of absolute relationship ids
			// description:
			//		Coming soon
			// tags:
			//		public
	 		
	 		//Mark we're changing
			this._changing = true; 
			
	 		//Clear out current selection
	 		this._items.clear();
	 		this._links.clear();
	 		this._relationships.clear();
	 		
	 		//Look up the objects in the dataset and add to hash tables
	 		this._setSelectedObjectIds(itemIds, linkIds, relationshipIds);

	 		//Pass along to selection objects on the sub-views
			var iterator = this._pluggableView.getPluggableViewIterator();
			while (!iterator.atEnd()) {
				var view = iterator.get().value;
				view.selection.setSelectedObjectIds(itemIds, linkIds, relationshipIds);
			}
			
			//Mark we're done changing
			this._changing = false;
	 		
	 		//Fire selection changed
			this.onSelectionChanged(this);
		},
		
		setSelectedObjects: function(/*ibm.tivoli.navmodel.Item[]*/ items, /*ibm.tivoli.navmodel.Link[]*/ links, /*ibm.tivoli.navmodel.Relationship[]*/ relationships){
			// summary:
			//		Coming soon
			// items:
			//		An array of items
			// links:
			//		An array of links
			// relationships:
			//		An array of relationships
			// description:
			//		Coming soon
			// tags:
			//		public
		
			//Mark we're changing
			this._changing = true;

			//Copy items/links/relationships into ourselves
	 		this._setSelectedObjectsBase(items, links, relationships);

	 		//Pass along to selection objects on the sub-views
			var iterator = this._pluggableView.getPluggableViewIterator();
			while (!iterator.atEnd()) {
				var view = iterator.get().value;
				view.selection.setSelectedObjects(items, links, relationships);
			}
			
			//Mark we're done changing
			this._changing = false;
	 		
	 		//Fire selection changed
			this.onSelectionChanged(this);
		},

		deselectAll: function() {
			this.setSelectedObjects(null, null, null, null);
		},
		
		setSelectedItems: function(/*ibm.tivoli.navmodel.Item[]*/ items){
			// summary:
			//		Coming soon
			// items:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		public
		
			this.setSelectedObjects(items, null, null, null);
		},
		
		setSelectedItem: function(/*ibm.tivoli.navmodel.Item*/ item){
			// summary:
			//		Coming soon
			// item:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		public
		
			this.setSelectedItems([item]);
		},
		
		setSelectedLinks: function(/*ibm.tivoli.navmodel.Link[]*/ links){
			// summary:
			//		Coming soon
			// links:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		public
		
			this.setSelectedObjects(null, links, null, null);
		},
		
		setSelectedLink: function(/*ibm.tivoli.navmodel.Link*/ link){
			// summary:
			//		Coming soon
			// link:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		public
		
			this.setSelectedLinks([link]);
		},
		
		setSelectedRelationships: function(/*ibm.tivoli.navmodel.Relationship[]*/ relationships){
			// summary:
			//		Coming soon
			// relationships:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		public
		
			this.setSelectedObjects(null, null, relationships, null);
		},
		
		setSelectedLink: function(/*ibm.tivoli.navmodel.Relationship*/ relationship){
			// summary:
			//		Coming soon
			// relationship:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		public
		
			this.setSelectedRelationships([relationship]);
		},

		chainSelectionChanged: function(objectContext){
			// summary:
			//		Coming soon
			// objectContext:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		protected
		
			//Mark we're changing
			this._changing = true;
			
			//Copy items/links into ourselves
	 		this._setSelectedObjectsBase(objectContext.getItems(), objectContext.getLinks(), objectContext.getRelationships());
	 		
	 		//NOTE: Let's not pass along to subviews at in this case
	 		//Pass along to pluggable views that are NOT an anscestor of the view managed by this selection object
			var iterator = this._pluggableView.getPluggableViewIterator();
			while (!iterator.atEnd()) {
				var view = iterator.get().value;
				if (!view.isAncestorOfPluggableView(objectContext._pluggableView)) {
					view.selection.setSelectedObjects(objectContext.getItems(), objectContext.getLinks(), objectContext.getRelationships());
				}
			}
	 		
	 		//Mark we're done changing
			this._changing = false;
			
	 		//Fire selection changed
			this.onSelectionChanged(this);
		},
		
	 	deselectObjectIds: function(/*String[]*/ itemIds, /*String[]*/ linkIds, /*String[]*/ relationshipIds){
			// summary:
			//		Coming soon
			// itemIds:
			//		An array of absolute item ids
			// linkIds:
			//		An array of absolute link ids
			// relationshipIds:
			//		An array of absolute relationship ids
			// description:
			//		Coming soon
			// tags:
			//		public
	 		
	 		//Mark we're changing
			this._changing = true; 
	 		
	 		//Look up the objects in the dataset and remove from hash tables
	 		this._deselectObjectIds(itemIds, linkIds, relationshipIds);

			// Now select what remains					
	 		//Look up the objects in the dataset and add to hash tables
	 		this._setSelectedObjectIds(this._items.getKeyList(), this._links.getKeyList(), this._relationships.getKeyList());

	 		//Pass along to selection objects on the sub-views
			var iterator = this._pluggableView.getPluggableViewIterator();
			while (!iterator.atEnd()) {
				var view = iterator.get().value;
				view.selection.setSelectedObjectIds(this._items.getKeyList(), this._links.getKeyList(), this._relationships.getKeyList());
			}
			
			//Mark we're done changing
			this._changing = false;

	 		//Fire selection changed
			this.operation = "deselect";
			this.onSelectionChanged(this);

			//Reset to default "select" operator
			this.operation = "select";
		},
		
		deselectObjects: function(/*ibm.tivoli.navmodel.Item[]*/ items, /*ibm.tivoli.navmodel.Link[]*/ links, /*ibm.tivoli.navmodel.Relationship[]*/ relationships){
			// summary:
			//		Coming soon
			// items:
			//		An array of items
			// links:
			//		An array of links
			// relationships:
			//		An array of relationships
			// description:
			//		Coming soon
			// tags:
			//		public
		
			//Mark we're changing
			this._changing = true;

			//Copy items/links/relationships into ourselves
	 		this._deselectObjects(items, links, relationships);
			
			// Now select what remains			
	 		// Look up the objects in the dataset and add to hash tables
	 		this._setSelectedObjectIds(this._items.getKeyList(), this._links.getKeyList(), this._relationships.getKeyList());

	 		//Pass along to selection objects on the sub-views
			var iterator = this._pluggableView.getPluggableViewIterator();
			while (!iterator.atEnd()) {
				var view = iterator.get().value;
				view.selection.setSelectedObjectIds(this._items.getKeyList(), this._links.getKeyList(), this._relationships.getKeyList());
			}

			//Mark we're done changing
			this._changing = false;
	 		
	 		//Fire selection changed
			this.operation = "deselect";
			this.onSelectionChanged(this);

			//Reset to default "select" operator
			this.operation = "select";
		},

		deselectItems: function(/*ibm.tivoli.navmodel.Item[]*/ items){
			// summary:
			//		Coming soon
			// items:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		public
		
			this.deselectObjects(items, null, null);
		},
		
		deselectItem: function(/*ibm.tivoli.navmodel.Item*/ item){
			// summary:
			//		Coming soon
			// item:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		public
		
			this.deselectItems([item]);
		},
		
		deselectLinks: function(/*ibm.tivoli.navmodel.Link[]*/ links){
			// summary:
			//		Coming soon
			// links:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		public
		
			this.deselectObjects(null, links, null);
		},
		
		deselectLink: function(/*ibm.tivoli.navmodel.Link*/ link){
			// summary:
			//		Coming soon
			// link:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		public
		
			this.deselectLinks([link]);
		},
		
		deselectRelationships: function(/*ibm.tivoli.navmodel.Relationship[]*/ relationships){
			// summary:
			//		Coming soon
			// relationships:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		public
		
			this.deselectObjects(null, null, relationships);
		},
		
		deselectRelationship: function(/*ibm.tivoli.navmodel.Relationship*/ relationship){
			// summary:
			//		Coming soon
			// relationship:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		public
		
			this.deselectRelationships([relationship]);
		},


		//*****************************************************************************************************
		// Events we fire
		//*****************************************************************************************************
		onSelectionChanged: function(objectContext){
			// summary:
			//		Event fired when an item, link, or relationship is selected
			// description:
			//		Coming soon
			// objectContext: ibm.tivoli.tip.navwidget.dojo.NavWidgetPluggableViewObjectContext
			//		Representation of all of the items, links, and relationships in the selection
			
			//Do nothing
		}
    }
);
});

},
'dojox/grid/enhanced/_FocusManager':function(){
define([
	"dojo/_base/kernel",
	"dojo/_base/lang",
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/connect",
	"dojo/_base/event",
	"dojo/_base/sniff",
	"dojo/_base/html",
	"dojo/keys",
	"dijit/a11y",
	"dijit/focus",
	"../_FocusManager"
], function(dojo, lang, declare, array, connect, event, has, html, keys, dijitA11y, dijitFocus, _FocusManager){

var _FocusArea = declare("dojox.grid.enhanced._FocusArea", null, {
	// summary:
	//		This is a friend class of _FocusManager
/*=====
		// name: string
		//		Name of this area.
		name: "",
		
		// onFocus: function(event, step)
		//		Called when this area logically gets focus.
		//		event: Event object
		//				May be unavailable, should check before use.
		//		step: Integer
		//				The distance in the tab sequence from last focused area to this area.
		//		returns:
		//				whether this area is successfully focused. If not, the next area will get focus.
		onFocus: function(event, step){return true;},
		
		// onBlur: function(event, step)
		//		Called when this area logically loses focus.
		//		event: Event object
		//				May be unavailable, should check before use.
		//		step: Integer
		//				The distance in the tab sequence from this area to the area to focus.
		//		returns:
		//				If Boolean, means whether this area has successfully blurred. If not, the next area to focus is still this one.
		//				If String, means the next area to focus is given by this returned name.
		onBlur: function(event, step){return true;},
		
		// onMove: function(rowStep, colStep, event)
		//		Called when focus is moving around within this area.
		//		rowStep: Integer
		//		colStep: Integer
		//		event: Event object
		//				May be unavailable, should check before use.
		onMove: function(rowStep, colStep, event){},
		
		// onKey: function(event, isBubble)
		//		Called when some key is pressed when focus is logically in this area.
		//		event: Event object
		//		isBubble: Boolean
		//				Whether is in bubble stage (true) or catch stage (false).
		//		returns:
		//				If you do NOT want the event to propagate any further along the area stack, return exactly false.
		//				So if you return nothing (undefined), this event is still propagating.
		onKey: function(event, isBubble){return true},
		
		// getRegions: function()
		//		Define the small regions (dom nodes) in this area.
		//		returns: Array of dom nodes.
		getRegions: function(){},
		
		// onRegionFocus: function(event)
		//		Connected to the onfocus event of the defined regions (if any)
		onRegionFocus: function(event){},
		
		// onRegionBlur: function(event)
		//		Connected to the onblur event of the defined regions (if any)
		onRegionBlur: function(event){},
=====*/
	constructor: function(area, focusManager){
		this._fm = focusManager;
		this._evtStack = [area.name];
		var dummy = function(){return true;};
		area.onFocus = area.onFocus || dummy;
		area.onBlur = area.onBlur || dummy;
		area.onMove = area.onMove || dummy;
		area.onKeyUp = area.onKeyUp || dummy;
		area.onKeyDown = area.onKeyDown || dummy;
		lang.mixin(this, area);
	},
	move: function(rowStep, colStep, evt){
		if(this.name){
			var i, len = this._evtStack.length;
			for(i = len - 1; i >= 0; --i){
				if(this._fm._areas[this._evtStack[i]].onMove(rowStep, colStep, evt) === false){
					return false;
				}
			}
		}
		return true;
	},
	_onKeyEvent: function(evt, funcName){
		if(this.name){
			var i, len = this._evtStack.length;
			for(i = len - 1; i >= 0; --i){
				if(this._fm._areas[this._evtStack[i]][funcName](evt, false) === false){
					return false;
				}
			}
			for(i = 0; i < len; ++i){
				if(this._fm._areas[this._evtStack[i]][funcName](evt, true) === false){
					return false;
				}
			}
		}
		return true;
	},
	keydown: function(evt){
		return this._onKeyEvent(evt, "onKeyDown");
	},
	keyup: function(evt){
		return this._onKeyEvent(evt, "onKeyUp");
	},
	contentMouseEventPlanner: function(){
		return 0;
	},
	headerMouseEventPlanner: function(){
		return 0;
	}
});

return declare("dojox.grid.enhanced._FocusManager", _FocusManager, {
	_stopEvent: function(evt){
		try{
			if(evt && evt.preventDefault){
				event.stop(evt);
			}
		}catch(e){}
	},
	
	constructor: function(grid){
		this.grid = grid;
		this._areas = {};
		this._areaQueue = [];
		this._contentMouseEventHandlers = [];
		this._headerMouseEventHandlers = [];
		this._currentAreaIdx = -1;
		this._gridBlured = true;
		this._connects.push(connect.connect(grid, "onBlur", this, "_doBlur"));
		this._connects.push(connect.connect(grid.scroller, "renderPage", this, "_delayedCellFocus"));
		
		this.addArea({
			name: "header",
			onFocus: lang.hitch(this, this.focusHeader),
			onBlur: lang.hitch(this, this._blurHeader),
			onMove: lang.hitch(this, this._navHeader),
			getRegions: lang.hitch(this, this._findHeaderCells),
			onRegionFocus: lang.hitch(this, this.doColHeaderFocus),
			onRegionBlur: lang.hitch(this, this.doColHeaderBlur),
			onKeyDown: lang.hitch(this, this._onHeaderKeyDown)
		});
		this.addArea({
			name: "content",
			onFocus: lang.hitch(this, this._focusContent),
			onBlur: lang.hitch(this, this._blurContent),
			onMove: lang.hitch(this, this._navContent),
			onKeyDown: lang.hitch(this, this._onContentKeyDown)
		});
		this.addArea({
			name: "editableCell",
			onFocus: lang.hitch(this, this._focusEditableCell),
			onBlur: lang.hitch(this, this._blurEditableCell),
			onKeyDown: lang.hitch(this, this._onEditableCellKeyDown),
			onContentMouseEvent: lang.hitch(this, this._onEditableCellMouseEvent),
			contentMouseEventPlanner: function(evt, areas){ return -1; }
		});
		this.placeArea("header");
		this.placeArea("content");
		this.placeArea("editableCell");
		this.placeArea("editableCell","above","content");
	},
	destroy: function(){
		for(var name in this._areas){
			var area = this._areas[name];
			array.forEach(area._connects, connect.disconnect);
			area._connects = null;
			if(area.uninitialize){
				area.uninitialize();
			}
		}
		this.inherited(arguments);
	},
	addArea: function(area){
		if(area.name && lang.isString(area.name)){
			if(this._areas[area.name]){
				//Just replace the original area, instead of remove it, so the position does not change.
				array.forEach(area._connects, connect.disconnect);
			}
			this._areas[area.name] = new _FocusArea(area, this);
			if(area.onHeaderMouseEvent){
				this._headerMouseEventHandlers.push(area.name);
			}
			if(area.onContentMouseEvent){
				this._contentMouseEventHandlers.push(area.name);
			}
		}
	},
	getArea: function(areaName){
		return this._areas[areaName];
	},
	_bindAreaEvents: function(){
		var area, hdl, areas = this._areas;
		array.forEach(this._areaQueue, function(name){
			area = areas[name];
			if(!area._initialized && lang.isFunction(area.initialize)){
				area.initialize();
				area._initialized = true;
			}
			if(area.getRegions){
				area._regions = area.getRegions() || [];
				array.forEach(area._connects || [], connect.disconnect);
				area._connects = [];
				array.forEach(area._regions, function(r){
					if(area.onRegionFocus){
						hdl = connect.connect(r, "onfocus", area.onRegionFocus);
						area._connects.push(hdl);
					}
					if(area.onRegionBlur){
						hdl = connect.connect(r, "onblur", area.onRegionBlur);
						area._connects.push(hdl);
					}
				});
			}
		});
	},
	removeArea: function(areaName){
		var area = this._areas[areaName];
		if(area){
			this.ignoreArea(areaName);
			var i = array.indexOf(this._contentMouseEventHandlers, areaName);
			if(i >= 0){
				this._contentMouseEventHandlers.splice(i, 1);
			}
			i = array.indexOf(this._headerMouseEventHandlers, areaName);
			if(i >= 0){
				this._headerMouseEventHandlers.splice(i, 1);
			}
			array.forEach(area._connects, connect.disconnect);
			if(area.uninitialize){
				area.uninitialize();
			}
			delete this._areas[areaName];
		}
	},
	currentArea: function(areaName, toBlurOld){
		// summary:
		//		Set current area to the one areaName refers.
		// areaName: String
		var idx, cai = this._currentAreaIdx;
		if(lang.isString(areaName) && (idx = array.indexOf(this._areaQueue, areaName)) >= 0){
			if(cai != idx){
				this.tabbingOut = false;
				if(toBlurOld && cai >= 0 && cai < this._areaQueue.length){
					this._areas[this._areaQueue[cai]].onBlur();
				}
				this._currentAreaIdx = idx;
			}
		}else{
			return (cai < 0 || cai >= this._areaQueue.length) ?
				new _FocusArea({}, this) :
				this._areas[this._areaQueue[this._currentAreaIdx]];
		}
		return null;
	},
	placeArea: function(name, pos, otherAreaName){
		// summary:
		//		Place the area refered by *name* at some logical position relative to an existing area.
		// example:
		//		placeArea("myarea","before"|"after",...)
		//		placeArea("myarea","below"|"above",...)
		if(!this._areas[name]){ return; }
		var idx = array.indexOf(this._areaQueue,otherAreaName);
		switch(pos){
			case "after":
				if(idx >= 0){ ++idx; }
				//intentional drop through
			case "before":
				if(idx >= 0){
					this._areaQueue.splice(idx,0,name);
					break;
				}
				//intentional drop through
			default:
				this._areaQueue.push(name);
				break;
			case "above":
				var isAbove = true;
				//intentional drop through
			case "below":
				var otherArea = this._areas[otherAreaName];
				if(otherArea){
					if(isAbove){
						otherArea._evtStack.push(name);
					}else{
						otherArea._evtStack.splice(0,0,name);
					}
				}
		}
	},
	ignoreArea: function(name){
		this._areaQueue = array.filter(this._areaQueue,function(areaName){
			return areaName != name;
		});
	},
	focusArea: function(/* int|string|areaObj */areaId,evt){
		var idx;
		if(typeof areaId == "number"){
			idx = areaId < 0 ? this._areaQueue.length + areaId : areaId;
		}else{
			idx = array.indexOf(this._areaQueue,
				lang.isString(areaId) ? areaId : (areaId && areaId.name));
		}
		if(idx < 0){ idx = 0; }
		var step = idx - this._currentAreaIdx;
		this._gridBlured = false;
		if(step){
			this.tab(step, evt);
		}else{
			this.currentArea().onFocus(evt, step);
		}
	},
	tab: function(step,evt){
		//console.log("===========tab",step,"curArea",this._currentAreaIdx,"areaCnt",this._areaQueue.length);
		this._gridBlured = false;
		this.tabbingOut = false;
		if(step === 0){
			return;
		}
		var cai = this._currentAreaIdx;
		var dir = step > 0 ? 1:-1;
		if(cai < 0 || cai >= this._areaQueue.length){
			cai = (this._currentAreaIdx += step);
		}else{
			var nextArea = this._areas[this._areaQueue[cai]].onBlur(evt,step);
			if(nextArea === true){
				cai = (this._currentAreaIdx += step);
			}else if(lang.isString(nextArea) && this._areas[nextArea]){
				cai = this._currentAreaIdx = array.indexOf(this._areaQueue,nextArea);
			}
		}
		//console.log("target area:",cai);
		for(; cai >= 0 && cai < this._areaQueue.length; cai += dir){
			this._currentAreaIdx = cai;
			if(this._areaQueue[cai] && this._areas[this._areaQueue[cai]].onFocus(evt,step)){
				//console.log("final target area:",this._currentAreaIdx);
				return;
			}
		}
		//console.log("tab out");
		this.tabbingOut = true;
		if(step < 0){
			this._currentAreaIdx = -1;
			dijitFocus.focus(this.grid.domNode);
		}else{
			this._currentAreaIdx = this._areaQueue.length;
			dijitFocus.focus(this.grid.lastFocusNode);
		}
	},
	_onMouseEvent: function(type, evt){
		var lowercase = type.toLowerCase(),
			handlers = this["_" + lowercase + "MouseEventHandlers"],
			res = array.map(handlers, function(areaName){
				return {
					"area": areaName,
					"idx": this._areas[areaName][lowercase + "MouseEventPlanner"](evt, handlers)
				};
			}, this).sort(function(a, b){
				return b.idx - a.idx;
			}),
			resHandlers = array.map(res, function(handler){
				return res.area;
			}),
			i = res.length;
		while(--i >= 0){
			if(this._areas[res[i].area]["on" + type + "MouseEvent"](evt, resHandlers) === false){
				return;
			}
		}
	},
	contentMouseEvent: function(evt){
		this._onMouseEvent("Content", evt);
	},
	headerMouseEvent: function(evt){
		this._onMouseEvent("Header", evt);
	},
	initFocusView: function(){
		// summary:
		//		Overwritten
		this.focusView = this.grid.views.getFirstScrollingView() || this.focusView || this.grid.views.views[0];
		this._bindAreaEvents();
	},
	isNavHeader: function(){
		// summary:
		//		Overwritten
		//		Check whether currently navigating among column headers.
		// return:
		//		true - focus is on a certain column header | false otherwise
		return this._areaQueue[this._currentAreaIdx] == "header";
	},
	previousKey: function(e){
		// summary:
		//		Overwritten
		this.tab(-1,e);
	},
	nextKey: function(e){
		// summary:
		//		Overwritten
		this.tab(1,e);
	},
	setFocusCell: function(/* Object */inCell, /* Integer */inRowIndex){
		// summary:
		//		Overwritten - focuses the given grid cell
		if(inCell){
			this.currentArea(this.grid.edit.isEditing() ? "editableCell" : "content", true);
			//This is very slow when selecting cells!
			//this.focusGridView();
			this._focusifyCellNode(false);
			this.cell = inCell;
			this.rowIndex = inRowIndex;
			this._focusifyCellNode(true);
		}
		this.grid.onCellFocus(this.cell, this.rowIndex);
	},
	doFocus: function(e){
		// summary:
		//		Overwritten
		//		trap focus only for grid dom node
		//		do not focus for scrolling if grid is about to blur
		if(e && e.target == e.currentTarget && !this.tabbingOut){
			if(this._gridBlured){
				this._gridBlured = false;
				if(this._currentAreaIdx < 0 || this._currentAreaIdx >= this._areaQueue.length){
					this.focusArea(0, e);
				}else{
					this.focusArea(this._currentAreaIdx, e);
				}
			}
		}else{
			this.tabbingOut = false;
		}
		event.stop(e);
	},
	_doBlur: function(){
		this._gridBlured = true;
	},
	doLastNodeFocus: function(e){
		// summary:
		//		Overwritten
		if(this.tabbingOut){
			this.tabbingOut = false;
		}else{
			this.focusArea(-1, e);
		}
	},
	_delayedHeaderFocus: function(){
		// summary:
		//		Overwritten
		if(this.isNavHeader()){
			this.focusHeader();
		}
	},
//    _delayedCellFocus: function(){
		// summary:
		//		Overwritten
		//If focus header here, the page will scroll to grid when the grid is created.
		//this.focusArea("header");
//    },
	_changeMenuBindNode: function(oldBindNode, newBindNode){
		var hm = this.grid.headerMenu;
		if(hm && this._contextMenuBindNode == oldBindNode){
			hm.unBindDomNode(oldBindNode);
			hm.bindDomNode(newBindNode);
			this._contextMenuBindNode = newBindNode;
		}
	},
	//---------------Header Area------------------------------------------
	focusHeader: function(evt, step){ //need a further look why these changes to parent's
		// summary:
		//		Overwritten
		var didFocus = false;
		this.inherited(arguments);
		if(this._colHeadNode && html.style(this._colHeadNode, 'display') != "none"){
			dijitFocus.focus(this._colHeadNode);
			this._stopEvent(evt);
			didFocus = true;
		}
		return didFocus;
	},
	_blurHeader: function(evt,step){
		// summary:
		//		Overwritten
		if(this._colHeadNode){
			html.removeClass(this._colHeadNode, this.focusClass);
		}
		html.removeAttr(this.grid.domNode,"aria-activedescendant");
		// reset contextMenu onto viewsHeaderNode so right mouse on header will invoke (see focusHeader)
		this._changeMenuBindNode(this.grid.domNode,this.grid.viewsHeaderNode);
		//moved here from nextKey
		this._colHeadNode = this._colHeadFocusIdx = null;
		return true;
	},
	_navHeader: function(rowStep, colStep, evt){
		var colDir = colStep < 0 ? -1 : 1,
			savedIdx = array.indexOf(this._findHeaderCells(), this._colHeadNode);
		if(savedIdx >= 0 && (evt.shiftKey && evt.ctrlKey)){
			this.colSizeAdjust(evt, savedIdx, colDir * 5);
			return;
		}
		this.move(rowStep, colStep);
	},
	_onHeaderKeyDown: function(e, isBubble){
		if(isBubble){
			var dk = keys;
			switch(e.keyCode){
				case dk.ENTER:
				case dk.SPACE:
					var colIdx = this.getHeaderIndex();
					if(colIdx >= 0 && !this.grid.pluginMgr.isFixedCell(e.cell)/*TODO*/){
						this.grid.setSortIndex(colIdx, null, e);
						event.stop(e);
					}
					break;
			}
		}
		return true;
	},
	_setActiveColHeader: function(){
		// summary:
		//		Overwritten
		this.inherited(arguments);
		//EDG now will decorate event on header key events, if no focus, the cell will be wrong
		dijitFocus.focus(this._colHeadNode);
	},
	//---------------Content Area------------------------------------------
	findAndFocusGridCell: function(){
		// summary:
		//		Overwritten
		this._focusContent();
	},
	_focusContent: function(evt,step){
		var didFocus = true;
		var isEmpty = (this.grid.rowCount === 0); // If grid is empty this.grid.rowCount == 0
		if(this.isNoFocusCell() && !isEmpty){
			//skip all the hidden cells
			for(var i = 0, cell = this.grid.getCell(0); cell && cell.hidden; cell = this.grid.getCell(++i)){}
			this.setFocusIndex(0, cell ? i : 0);
		}else if(this.cell && !isEmpty){
			if(this.focusView && !this.focusView.rowNodes[this.rowIndex]){
				// if rowNode for current index is undefined (likely as a result of a sort and because of #7304)
				// scroll to that row
				this.grid.scrollToRow(this.rowIndex);
				this.focusGrid();
			}else{
				this.setFocusIndex(this.rowIndex, this.cell.index);
			}
		}else if(isEmpty){
			this.grid.messagesNode.focus();
		}else{
			didFocus = false;
		}
		if(didFocus){ this._stopEvent(evt); }
		return didFocus;
	},
	_blurContent: function(evt,step){
		this._focusifyCellNode(false);
		return true;
	},
	_navContent: function(rowStep, colStep, evt){
		if((this.rowIndex === 0 && rowStep < 0) || (this.rowIndex === this.grid.rowCount - 1 && rowStep > 0)){
			return;
		}
		this._colHeadNode = null;
		this.move(rowStep, colStep, evt);
		if(evt){
			event.stop(evt);
		}
	},
	_onContentKeyDown: function(e, isBubble){
		if(isBubble){
			var dk = keys, s = this.grid.scroller;
			switch(e.keyCode){
				case dk.ENTER:
				case dk.SPACE:
					var g = this.grid;
					if(g.indirectSelection){ break; }
					g.selection.clickSelect(this.rowIndex, connect.isCopyKey(e), e.shiftKey);
					g.onRowClick(e);
					event.stop(e);
					break;
				case dk.PAGE_UP:
					if(this.rowIndex !== 0){
						if(this.rowIndex != s.firstVisibleRow + 1){
							this._navContent(s.firstVisibleRow - this.rowIndex, 0);
						}else{
							this.grid.setScrollTop(s.findScrollTop(this.rowIndex - 1));
							this._navContent(s.firstVisibleRow - s.lastVisibleRow + 1, 0);
						}
						event.stop(e);
					}
					break;
				case dk.PAGE_DOWN:
					if(this.rowIndex + 1 != this.grid.rowCount){
						event.stop(e);
						if(this.rowIndex != s.lastVisibleRow - 1){
							this._navContent(s.lastVisibleRow - this.rowIndex - 1, 0);
						}else{
							this.grid.setScrollTop(s.findScrollTop(this.rowIndex + 1));
							this._navContent(s.lastVisibleRow - s.firstVisibleRow - 1, 0);
						}
						event.stop(e);
					}
					break;
			}
		}
		return true;
	},
	//------------------editable content area-------------------------
	_blurFromEditableCell: false,
	_isNavigating: false,
	_navElems: null,
	_focusEditableCell: function(evt,step){
		var didFocus = false;
		if(this._isNavigating){
			didFocus = true;
		}else if(this.grid.edit.isEditing() && this.cell){
			if(this._blurFromEditableCell || !this._blurEditableCell(evt, step)){
				this.setFocusIndex(this.rowIndex,this.cell.index);
				didFocus = true;
			}
			this._stopEvent(evt);
		}
		return didFocus;
	},
	_applyEditableCell: function(){
		try{
			this.grid.edit.apply();
		}catch(e){
			console.warn("_FocusManager._applyEditableCell() error:", e);
		}
	},
	_blurEditableCell: function(evt,step){
		this._blurFromEditableCell = false;
		if(this._isNavigating){
			var toBlur = true;
			if(evt){
				var elems = this._navElems;
				var firstElem = elems.lowest || elems.first;
				var lastElem = elems.last || elems.highest || firstElem;
				var target = has("ie") ? evt.srcElement : evt.target;
				toBlur = target == (step > 0 ? lastElem : firstElem);
			}
			if(toBlur){
				this._isNavigating = false;
				html.setSelectable(this.cell.getNode(this.rowIndex), false);
				return "content";
			}
			return false;
		}else if(this.grid.edit.isEditing() && this.cell){
			if(!step || typeof step != "number"){ return false; }
			var dir = step > 0 ? 1 : -1;
			var cc = this.grid.layout.cellCount;
			for(var cell, col = this.cell.index + dir; col >= 0 && col < cc; col += dir){
				cell = this.grid.getCell(col);
				if(cell.editable){
					this.cell = cell;
					this._blurFromEditableCell = true;
					return false;
				}
			}
			if((this.rowIndex > 0 || dir == 1) && (this.rowIndex < this.grid.rowCount || dir == -1)){
				this.rowIndex += dir;
				//this.cell = this.grid.getCell(0); //There must be an editable cell, so this is not necessary.
				for(col = dir > 0 ? 0 : cc - 1; col >= 0 && col < cc; col += dir){
					cell = this.grid.getCell(col);
					if(cell.editable){
						this.cell = cell;
						break;
					}
				}
				this._applyEditableCell();
				return "content";
			}
		}
		return true;
	},
	_initNavigatableElems: function(){
		this._navElems = dijitA11y._getTabNavigable(this.cell.getNode(this.rowIndex));
	},
	_onEditableCellKeyDown: function(e, isBubble){
		var dk = keys,
			g = this.grid,
			edit = g.edit,
			editApplied = false,
			toPropagate = true;
		switch(e.keyCode){
			case dk.ENTER:
				if(isBubble && edit.isEditing()){
					this._applyEditableCell();
					editApplied = true;
					event.stop(e);
				}
				//intentional drop through
			case dk.SPACE:
				if(!isBubble && this._isNavigating){
					toPropagate = false;
					break;
				}
				if(isBubble){
					if(!this.cell.editable && this.cell.navigatable){
						this._initNavigatableElems();
						var toFocus = this._navElems.lowest || this._navElems.first;
						if(toFocus){
							this._isNavigating = true;
							html.setSelectable(this.cell.getNode(this.rowIndex), true);
							dijitFocus.focus(toFocus);
							event.stop(e);
							this.currentArea("editableCell", true);
							break;
						}
					}
					if(!editApplied && !edit.isEditing() && !g.pluginMgr.isFixedCell(this.cell)){
						edit.setEditCell(this.cell, this.rowIndex);
					}
					if(editApplied){
						this.currentArea("content", true);
					}else if(this.cell.editable && g.canEdit()){
						this.currentArea("editableCell", true);
					}
				}
				break;
			case dk.PAGE_UP:
			case dk.PAGE_DOWN:
				if(!isBubble && edit.isEditing()){
					//prevent propagating to content area
					toPropagate = false;
				}
				break;
			case dk.ESCAPE:
				if(!isBubble){
					edit.cancel();
					this.currentArea("content", true);
				}
		}
		return toPropagate;
	},
	_onEditableCellMouseEvent: function(evt){
		if(evt.type == "click"){
			var cell = this.cell || evt.cell;
			if(cell && !cell.editable && cell.navigatable){
				this._initNavigatableElems();
				if(this._navElems.lowest || this._navElems.first){
					var target = has("ie") ? evt.srcElement : evt.target;
					if(target != cell.getNode(evt.rowIndex)){
						this._isNavigating = true;
						this.focusArea("editableCell", evt);
						html.setSelectable(cell.getNode(evt.rowIndex), true);
						dijitFocus.focus(target);
						return false;
					}
				}
			}else if(this.grid.singleClickEdit){
				this.currentArea("editableCell");
				return false;
			}
		}
		return true;
	}
});
});

},
'dijit/form/SimpleTextarea':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.add
	"dojo/_base/sniff", // has("ie") has("opera")
	"dojo/_base/window", // win.doc.selection win.doc.selection.createRange
	"./TextBox"
], function(declare, domClass, has, win, TextBox){

/*=====
	var TextBox = dijit.form.TextBox;
=====*/

// module:
//		dijit/form/SimpleTextarea
// summary:
//		A simple textarea that degrades, and responds to
// 		minimal LayoutContainer usage, and works with dijit.form.Form.
//		Doesn't automatically size according to input, like Textarea.

return declare("dijit.form.SimpleTextarea", TextBox, {
	// summary:
	//		A simple textarea that degrades, and responds to
	// 		minimal LayoutContainer usage, and works with dijit.form.Form.
	//		Doesn't automatically size according to input, like Textarea.
	//
	// example:
	//	|	<textarea data-dojo-type="dijit.form.SimpleTextarea" name="foo" value="bar" rows=30 cols=40></textarea>
	//
	// example:
	//	|	new dijit.form.SimpleTextarea({ rows:20, cols:30 }, "foo");

	baseClass: "dijitTextBox dijitTextArea",

	// rows: Number
	//		The number of rows of text.
	rows: "3",

	// rows: Number
	//		The number of characters per line.
	cols: "20",

	templateString: "<textarea ${!nameAttrSetting} data-dojo-attach-point='focusNode,containerNode,textbox' autocomplete='off'></textarea>",

	postMixInProperties: function(){
		// Copy value from srcNodeRef, unless user specified a value explicitly (or there is no srcNodeRef)
		// TODO: parser will handle this in 2.0
		if(!this.value && this.srcNodeRef){
			this.value = this.srcNodeRef.value;
		}
		this.inherited(arguments);
	},

	buildRendering: function(){
		this.inherited(arguments);
		if(has("ie") && this.cols){ // attribute selectors is not supported in IE6
			domClass.add(this.textbox, "dijitTextAreaCols");
		}
	},

	filter: function(/*String*/ value){
		// Override TextBox.filter to deal with newlines... specifically (IIRC) this is for IE which writes newlines
		// as \r\n instead of just \n
		if(value){
			value = value.replace(/\r/g,"");
		}
		return this.inherited(arguments);
	},

	_onInput: function(/*Event?*/ e){
		// Override TextBox._onInput() to enforce maxLength restriction
		if(this.maxLength){
			var maxLength = parseInt(this.maxLength);
			var value = this.textbox.value.replace(/\r/g,'');
			var overflow = value.length - maxLength;
			if(overflow > 0){
				var textarea = this.textbox;
				if(textarea.selectionStart){
					var pos = textarea.selectionStart;
					var cr = 0;
					if(has("opera")){
						cr = (this.textbox.value.substring(0,pos).match(/\r/g) || []).length;
					}
					this.textbox.value = value.substring(0,pos-overflow-cr)+value.substring(pos-cr);
					textarea.setSelectionRange(pos-overflow, pos-overflow);
				}else if(win.doc.selection){ //IE
					textarea.focus();
					var range = win.doc.selection.createRange();
					// delete overflow characters
					range.moveStart("character", -overflow);
					range.text = '';
					// show cursor
					range.select();
				}
			}
		}
		this.inherited(arguments);
	}
});

});

},
'ibm/tivoli/tip/navwidget/tree/dijit/TreeTableView':function(){
// wrapped by build app
define("ibm/tivoli/tip/navwidget/tree/dijit/TreeTableView", ["dijit","dojo","dojox","dojo/i18n!ibm/tivoli/tip/navwidget/nls/resources","dojo/require!dojo/data/ItemFileReadStore,dojox/grid/LazyTreeGridStoreModel,ibm/tivoli/navmodel/DatasetLazyTreeStore,ibm/tivoli/tip/navwidget/table/dijit/AbstractTableView,ibm/tivoli/tip/navwidget/tree/dijit/NavModelTreeTable,ibm/tivoli/tip/navwidget/tree/dijit/TreeTableSelection,dojoe/treetable/TreeTableAll"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide ("ibm.tivoli.tip.navwidget.tree.dijit.TreeTableView");

dojo.require("dojo.data.ItemFileReadStore");
dojo.require("dojox.grid.LazyTreeGridStoreModel");
dojo.require("ibm.tivoli.navmodel.DatasetLazyTreeStore");

dojo.require ("ibm.tivoli.tip.navwidget.table.dijit.AbstractTableView");
dojo.require("ibm.tivoli.tip.navwidget.tree.dijit.NavModelTreeTable");
dojo.require("ibm.tivoli.tip.navwidget.tree.dijit.TreeTableSelection");

dojo.require("dojoe.treetable.TreeTableAll");

dojo.requireLocalization("ibm.tivoli.tip.navwidget", "resources");

ibm.tivoli.tip.navwidget.tree.dijit.TreeTableView._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.tip.navwidget.table.dijit.TreeTableView");

dojo.declare ("ibm.tivoli.tip.navwidget.tree.dijit.TreeTableView",
	ibm.tivoli.tip.navwidget.table.dijit.AbstractTableView,
{
	// summary:
	//		Coming soon
	// description:
	//		Coming soon
	
	//***************************
	// "Private"
	//***************************
	
	// _trace: [private] Trace
	//		Utility for trace and logging
	_trace: ibm.tivoli.tip.navwidget.tree.dijit.TreeTableView._trace,
	
	constructor: function () {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		//this._global = ibm.tivoli.tip.dojo.Global.setSingleton( "123", " ");
		//this.title = this.commonResources.RESOURCE_VIEW;
	    this._pDEFAULT_WIDGET_DEF_INFO = {
			className: "ibm.tivoli.navmodel.TreeTableWidgetDef",
			baseUrl: "/ibm/tivoli/rest",
	    	provider: "any",
	    	datasource: "any",
	    	widget: "treetable-0.1.0",
	    	definition: "DefaultTreeTable-0.1.0"
		},
		
		this._pTableType = "item";
		this._pTableClassName = "ibm.tivoli.tip.navwidget.tree.dijit.NavModelTreeTable";
		this._pSelectionClassName = "ibm.tivoli.tip.navwidget.tree.dijit.TreeTableSelection";
		
		// make sure dataset gets destroyed  (TODO: Is this needed? Parent class has _destroyTableStore()...ask Steve)
		// this.destroyDatasetWhenDone = true;
	},
	
	// overrides AbstractTableView
	augmentGridParams: function(gridParams) {
		// summary:
		//		Add a tree model to the gridParams
		// description:
		//		Create a LazyTreeGridStoreModel and add it to the gridParams.
		
		/* Not needed for gridx
		var treeGridModel = new dojox.grid.LazyTreeGridStoreModel({
			rootId: 'root',
			deferItemLoadingUntilExpand: true,
			serverStore: true,
			store: gridParams.store,
			query: {origins: true},
			queryOptions: null,
			childrenAttrs: ['hasChildren']		
		});
		gridParams.treeModel = treeGridModel;
		*/
		// Set query here now so it can be passed in to gridx constructor
		gridParams.query = {origins: true};
		
		gridParams.lazyModel = true;   // TODO: Is this still needed? Can't find in TED code
		gridParams.storeType = false;  // TODO: Is this still needed? Can't find in TED code
		gridParams.filter = {isServerSide : true};
		gridParams.keepSelection = true;
		
		// Reset the structure value to only the array of column descriptors and not the defaultCell info
		// TED does not support multiple or complicated nested layouts or locked columns
 		if ((gridParams.structure.length == 1) && (gridParams.store instanceof dojo.data.ItemFileReadStore)) {						
			gridParams.structure = [{}];
		} else if ((gridParams.structure.length == 1) && ("rows" in gridParams.structure[0]) && (gridParams.structure[0].rows instanceof Array)) {
	     		gridParams.structure = gridParams.structure[0].rows;	    		
	    	} else if((gridParams.structure.length == 2) && ("rows" in gridParams.structure[0]) && (gridParams.structure[0].rows instanceof Array) && ("rows" in 
gridParams.structure[1]) && (gridParams.structure[1].rows instanceof Array)  ) {
	    		//if there are some defined locked columns, merge them into one array since
	    		//tree table does not support true column locking, they will however appear
	    		//at the beginning of the table
	    		var layout1 = gridParams.structure[0].rows;
	    		var layout2 = gridParams.structure[1].rows;
	    		var layoutMerged = [];
    	
	    	layoutMerged = layout1.concat(layout2);

	    	gridParams.structure = layoutMerged;
	    	
	    	
	    }
 		
        // Set first column as expando column
        if (gridParams.structure.length > 1) {
           gridParams.structure[0].expandLevel = "all";
        }
	    
		return gridParams;
	},	
	
	getWidgetState: function() {
		// summary:
		//	Convenience method to manage the underlying table widget state
		//	Not to be confused with state object, which is used to persist state in a widget def
		return this._table.getState();
	},
	
	_createTableStore: function(columnIds){		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// columnIds:
		//		
		
		// create or update store
		if(!this._dataStore) {
			if (this.dataset) {
				this._dataStore = new ibm.tivoli.navmodel.DatasetLazyTreeStore({dataset:this.dataset, defaultProperties:columnIds, returnItem:true});
			} else {
				//No dataset, so just create an empty store
				var emptyStore = 	new dojo.data.ItemFileReadStore({
            						data: { items: []}
	                				});
				this._dataStore = emptyStore;
			}
			this.inherited(arguments);
		}
	}	
	
});

});

},
'dojox/grid/EnhancedGrid':function(){
define([
	"dojo/_base/kernel",
	"../main",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/array",
	"dojo/_base/sniff",
	"dojo/dom",
	"dojo/dom-geometry",
	"dojo/i18n",
	"./DataGrid",
	"./DataSelection",
	"./enhanced/_PluginManager",
	"./enhanced/plugins/_SelectionPreserver",//default loaded plugin
	"dojo/i18n!./enhanced/nls/EnhancedGrid"
], function(dojo, dojox, declare, lang, array, has, dom, domGeometry, i18n,
	DataGrid, DataSelection, _PluginManager, _SelectionPreserver){

dojo.experimental("dojox.grid.EnhancedGrid");

var EnhancedGrid = declare("dojox.grid.EnhancedGrid", DataGrid, {
	// summary:
	//		Provides enhanced features based on DataGrid
	//
	// description:
	//		EnhancedGrid features are implemented as plugins that could be loaded on demand.
	//		Explicit dojo.require() is needed to use these feature plugins.
	//
	// example:
	//		A quick sample to use EnhancedGrid features:
	//
	//	   Step 1. Load EnhancedGrid and required features
	// |   <script type="text/javascript">
	// |		dojo.require("dojox.grid.EnhancedGrid");
	// |		dojo.require("dojox.grid.enhanced.plugins.DnD");
	// |		dojo.require("dojox.grid.enhanced.plugins.Menu");
	// |		dojo.require("dojox.grid.enhanced.plugins.NestedSorting");
	// |		dojo.require("dojox.grid.enhanced.plugins.IndirectSelection");
	// |	</script>
	//
	//		Step 2. Use EnhancedGrid
	//		- Via HTML markup
	// |	<div dojoType="dojox.grid.EnhancedGrid" ...
	// |		plugins="{nestedSorting: true, dnd: true, indirectSelection: true,
	// |		menus:{headerMenu:"headerMenuId", rowMenu:"rowMenuId", cellMenu:"cellMenuId",
	// |		selectedRegionMenu:"selectedRegionMenuId"}}">
	// |			...
	// |	</div>
	//
	//		- Or via JavaScript
	// |	<script type="text/javascript">
	// |		var grid = new dojox.grid.EnhancedGrid({plugins : {nestedSorting: true, dnd: true, indirectSelection: true,
	// |	               menus:{headerMenu:"headerMenuId", rowMenu:"rowMenuId", cellMenu:"cellMenuId",selectedRegionMenu:"selectedRegionMenuId"}},
	// |			       ... }, dojo.byId('gridDiv'));
	// |		grid.startup();
	// |	</script>
	//
	//
	//		Plugin Support
	//		[Note: Plugin support is still experimental]
	//
	//		You can either customize the default plugins or add new ones, more details please see
	//		- dojox.grid.enhanced._PluginManager
	//		- dojox.grid.enhanced._Plugin
	//		- dojox.grid.enhanced.plugins.*

	//plugins: Object
	//		Plugin properties, e.g. {nestedSorting: true, dnd: true, ...}
	plugins: null,

	//pluginMgr: Object
	//		Singleton plugin manager
	pluginMgr: null,

	//_pluginMgrClass: Object
	//		Default plugin manager class
	_pluginMgrClass: _PluginManager,

	postMixInProperties: function(){
		//load nls bundle
		this._nls = i18n.getLocalization("dojox.grid.enhanced", "EnhancedGrid", this.lang);
		this.inherited(arguments);
	},
	postCreate: function(){
		//create plugin manager
		this.pluginMgr = new this._pluginMgrClass(this);
		this.pluginMgr.preInit();
		this.inherited(arguments);
		this.pluginMgr.postInit();
	},
	plugin: function(/*String*/name){
		// summary:
		//		An easier way for getting a plugin, e.g. grid.plugin('dnd')
		return this.pluginMgr.getPlugin(name);
	},
	startup: function(){
		this.inherited(arguments);
		this.pluginMgr.startup();
	},
	createSelection: function(){
		this.selection = new dojox.grid.enhanced.DataSelection(this);
	},
	canSort: function(colIndex, field){
		// summary:
		//		Overwritten
		return true;
	},
	doKeyEvent: function(e){
		// summary:
		//		Overwritten, see _Grid.doKeyEvent()
		try{
			var view = this.focus.focusView;
			view.content.decorateEvent(e);
			if(!e.cell){ view.header.decorateEvent(e); }
		}catch(e){}
		this.inherited(arguments);
	},
	doApplyCellEdit: function(inValue, inRowIndex, inAttrName){
		// summary:
		//		Overwritten, see DataGrid.doApplyCellEdit()
		if(!inAttrName){
			this.invalidated[inRowIndex] = true;
			return;
		}
		this.inherited(arguments);
	},
	mixin: function(target, source){
		var props = {};
		for(var p in source){
			if(p == '_inherited' || p == 'declaredClass' || p == 'constructor' ||
				source['privates'] && source['privates'][p]){
				continue;
			}
			props[p] = source[p];
		}
		lang.mixin(target, props);
	},
	_copyAttr: function(idx, attr){
		// summary:
		//		Overwritten, see DataGrid._copyAttr()
		//		Fix cell TAB navigation for single click editing
		if(!attr){ return; }
		return this.inherited(arguments);
	},
	_getHeaderHeight: function(){
		// summary:
		//		Overwritten, see _Grid._getHeaderHeight()
		//		Should include borders/margins of this.viewsHeaderNode
		this.inherited(arguments);
		return domGeometry.getMarginBox(this.viewsHeaderNode).h;
	},
	_fetch: function(start, isRender){
		// summary:
		//		Overwritten, see DataGrid._fetch()
		if(this.items){
			return this.inherited(arguments);
		}
		start = start || 0;
		if(this.store && !this._pending_requests[start]){
			if(!this._isLoaded && !this._isLoading){
				this._isLoading = true;
				this.showMessage(this.loadingMessage);
			}
			this._pending_requests[start] = true;
			try{
				var req = {
					start: start,
					count: this.rowsPerPage,
					query: this.query,
					sort: this.getSortProps(),
					queryOptions: this.queryOptions,
					isRender: isRender,
					onBegin: lang.hitch(this, "_onFetchBegin"),
					onComplete: lang.hitch(this, "_onFetchComplete"),
					onError: lang.hitch(this, "_onFetchError")
				};
				this._storeLayerFetch(req);
			}catch(e){
				this._onFetchError(e, {start: start, count: this.rowsPerPage});
			}
		}
		return 0;
	},
	_storeLayerFetch: function(req){
		// summary:
		//		Extracted fetch specifically for store layer use
		this.store.fetch(req);
	},
	getCellByField: function(field){
		return array.filter(this.layout.cells, function(cell){
			return cell.field == field;
		})[0];
	},
	onMouseUp: function(e){	},
	createView: function(){
		// summary
		//		Overwrite: rewrite getCellX of view.header
		var view = this.inherited(arguments);
		if(has("mozilla")){
			var ascendDom = function(inNode, inWhile){
				for(var n = inNode; n && inWhile(n); n = n.parentNode){}
				return n;
			};//copied from dojox.grid._Builder
			var makeNotTagName = function(inTagName){
				var name = inTagName.toUpperCase();
				return function(node){ return node.tagName != name; };
			};//copied from dojox.grid._Builder

			var func = view.header.getCellX;
			view.header.getCellX = function(e){
				var x = func.call(view.header, e);
				var n = ascendDom(e.target, makeNotTagName("th"));
				if(n && n !== e.target && dom.isDescendant(e.target, n)){ x += n.firstChild.offsetLeft; }
				return x;
			};
		}
		return view;
	},
	destroy: function(){
		// summary:
		//		Destroy all resources
		delete this._nls;
		this.pluginMgr.destroy();
		this.inherited(arguments);
	}
});

declare("dojox.grid.enhanced.DataSelection", DataSelection, {
	constructor: function(grid){
		if(grid.keepSelection){
			if(this.preserver){
				this.preserver.destroy();
			}
			this.preserver = new _SelectionPreserver(this);
		}
	},
	_range: function(inFrom, inTo){
		this.grid._selectingRange = true;
		this.inherited(arguments);
		this.grid._selectingRange = false;
		this.onChanged();
	},
	deselectAll: function(inItemOrIndex){
		this.grid._selectingRange = true;
		this.inherited(arguments);
		this.grid._selectingRange = false;
		this.onChanged();
	}
});

EnhancedGrid.markupFactory = function(props, node, ctor, cellFunc){
	return dojox.grid._Grid.markupFactory(props, node, ctor,
					lang.partial(DataGrid.cell_markupFactory, cellFunc));
};

EnhancedGrid.registerPlugin = function(clazz, props){
	_PluginManager.registerPlugin(clazz, props);
};

return EnhancedGrid;

});
},
'idx/oneui/MenuDialog':function(){
/*
 * Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

define(["dojo/_base/declare",
		"dojo/_base/array",
		"dojo/_base/connect",
		"dojo/_base/event",
		"dojo/_base/lang",
		"dojo/_base/sniff",
		"dojo/_base/window",
		"dojo/aspect",
		"dojo/dom",
		"dojo/dom-class",
		"dojo/dom-geometry",
		"dojo/dom-style",
		"dojo/io/iframe",
		"dojo/keys",
		"dojo/window",
		"dijit/popup",
		"dijit/TooltipDialog",
		"idx/oneui/_EventTriggerMixin",
		"dojo/text!../oneui/templates/MenuDialog.html"],
		function(declare, array, connect, event, lang, has, win, aspect, dom, domClass, domGeometry, domStyle, iframe, keys, window, popup, TooltipDialog, _EventTriggerMixin, template){
		
	function nodeUsesArrowKeys(/*DOMNode*/ node){
		// return true if the DOM node is one that uses arrow keys (like
		// a text entry field) but doesn't stop them propagating -- we
		// will ignore arrow keypresses that were originally targetted
		// at such nodes so as not to double-act when the arrow keys are
		// pressed.
		
		return (node.nodeName === "TEXTAREA")
		    || ((node.nodeName === "INPUT") && (node.type === "text"));
	}

	/**
	 * Creates a new idx.oneui.MenuDialog
	 * @name idx.oneui.MenuDialog
	 * @class The MenuDialog widget is the main container for rich mega-menu
	 * content.
	 * <p>Instances can be supplied as the "popup" parameter for 
	 * dijit.PopupMenuItem and dijit.PopupMenuBarItem, and will work
	 * "as expected". Instances can also operate as popup menus on arbitrary
	 * DOM nodes, or for the whole window.
	 * </p>
	 * <p>Instances can display with a "shark fin" connector to identify
	 * the initiating element or with a thinner border, no shark
	 * fin and abutting the initiating element.
	 * </p>
	 * <p>Instances behave as content panes, and HTML and dijits may be
	 * laid out within them. If a menu with "menuForDialog" set to true is
	 * included within the layout, the menu dialog will operate as a
	 * drop-down menu in combination with the contained	menu.
	 * </p>
	 * @augments dijit.TooltipDialog
	 * @augments idx.oneui._EventTriggerMixin
	 * @example
	 * &lt;div data-dojo-type="idx.oneui.MenuBar"&gt;
  &lt;div data-dojo-type="dijit.PopupMenuBarItem"&gt;
    &lt;span&gt;Menu #1&lt;/span&gt;
    &lt;div data-dojo-type="idx.oneui.MenuDialog"&gt;</span>
      &lt;!-- Arbitrary mega menu content --&gt;
    &lt;/div&gt;
  &lt;/div&gt;
&lt;/div&gt;
	 */
	return declare("idx.oneui.MenuDialog", [TooltipDialog, _EventTriggerMixin],
	/** @lends idx.oneui.MenuDialog.prototype */
	{
		/**
		 * CSS class applied to DOM node. Part of standard templated widget 
		 * behavior.
		 * @type string
		 * @constant
		 * @private
		 */
		baseClass: "oneuiMenuDialog",
		
		/**
		 * Flag to indicate whether we need to close ourselves in onBlur. This
		 * flag is only true while we are popped up from a manual open or
		 * bindDomNode trigger.
		 * @type boolean
		 * @private
		 */
		_closeOnBlur: false,
		
		/**
		 * If true, hover will trigger popup whenever the menu dialog is
		 * bound to a DOM node (via bindToDomNode). Otherwise, a mouse
		 * click (or tap on a touch interface) will be required. This setting
		 * does not affect keyboard alternative triggers, which will be
		 * active in the same way in all cases. This value can be overridden
		 * for individual calls to bindToDomNode by including a "hoverToOpen"
		 * property in the (optional) parameters object.
		 * @type boolean
		 */
		hoverToOpen: true,
		
		/**
		 * True if the menu dialog is currently showing, false otherwise. Part
		 * of the standard dijit menu logic.
		 * @type boolean
		 * @private
		 */
		isShowingNow: false,
		
		/**
		 * If true, a left mouse click (or tap on a touch interface) will
		 * trigger popup whenever the menu dialog is bound to a DOM node
		 * (via bindDomNode). Otherwise, a right mouse click will be
		 * required. Furthermore, when leftClickToOpen is true the keyboard
		 * alternative triggers become space-bar and enter key and the popup
		 * is automatically positioned relative to the triggering DOM node,
		 * whilst when leftClickToOpen is false the keyboard alternative
		 * trigger is Shift+F10 and the popup is positioned relative to the
		 * mouse position (when triggered by a mouse event, and relative to
		 * the triggering DOM node otherwise). This value can be overridden
		 * for individual calls to bindDomNode by including a
		 * "leftClickToOpen" property in the (optional) parameters object.
		 * @type boolean
		 */
		leftClickToOpen: false,
		
		/**
		 * Pointer to menu that displayed the MenuDialog. Part
		 * of the standard dijit menu logic.
		 * @type dijit.Widget
		 * @private
		 */
		parentMenu: null,

		/**
		 * Ordered list of positions to try placing the popup at whenever the
		 * menu dialog is bound to a DOM node (via bindToDomNode). The first
		 * position which allows the popup to be fully visible within the
		 * viewport will be used. Possible values are:
		 * <ul>
		 * <li>
		 * before: places drop down to the left of the anchor node/widget, or
		 * to the right in RTL mode
		 * </li>
		 * <li>
		 * after: places drop down to the right of the anchor node/widget, or
		 * to the right in RTL mode
		 * </li>
		 * <li>
		 * above: drop down goes above anchor node/widget
		 * </li>
		 * <li>
		 * above-alt: same as above except right sides aligned instead of left
		 * </li>
		 * <li>
		 * below: drop down goes below anchor node/widget
		 * </li>
		 * <li>
		 * below-alt: same as below except right sides aligned instead of left
		 * </li>
		 * </ul>
		 * This value can be overridden for individual calls to bindToDomNode
		 * by including a "popupPosition" property in the (optional)
		 * parameters object. If no value is provided (or null) a default
		 * sequence of positions (below, below-alt, above, above-alt) is used.
		 * @name popupPosition
		 * @memberOf idx.oneui.MenuDialog.prototype
		 * @type string[]
		 */
		
		/**
		 * If true, when this menu dialog closes re-focus the element
		 * which had focus before it was opened.
		 * @type boolean
		 * @private
		 */
		refocus: true,
	
		/**
		 * An array of DOM node ids to which the menu dialog should be
		 * bound during widget initialisation. Note that this array is
		 * processed only once, during creation, and later changes that
		 * may be made will be ignored; also, the array is not updated
		 * if/when further explicit calls to bindDomNode/unbindDomNode are made.
		 * @name targetNodeIds
		 * @memberOf idx.oneui.MenuDialog.prototype
		 * @type DOMNode[]
		 */
		
		/**
	 	 * The template HTML for the widget.
		 * @constant
		 * @type string
		 * @private
		 * @default Loaded from idx/oneui/templates/MenuDialog.html.
		 */
		templateString: template,
		
		/**
		 * If true, the menu will show a "shark-fin" connector linking the
		 * opened popup dialog to the initiating DOM element or menu item.
		 * This value can be overridden for individual calls to bindDomNode
		 * by including a "useConnector" property in the (optional)
		 * parameters object.
		 * @type boolean
		 */
		useConnector: false,

		/**
		 * Standard Widget lifecycle method.
		 * @private
		 */		
		postCreate: function(){
			this.inherited(arguments);
			
			// intercept and process certain navigational keystrokes
			var l = this.isLeftToRight();
			this._nextMenuKey = l ? keys.RIGHT_ARROW : keys.LEFT_ARROW;
			this._prevMenuKey = l ? keys.LEFT_ARROW : keys.RIGHT_ARROW;
			this.connect(this.domNode, "onkeypress", "_onDomNodeKeypress");
			
			// do any initial bindings to DOM nodes requested
			if(this.contextMenuForWindow){
				this.bindDomNode(win.body());
			}else{
				array.forEach(this.targetNodeIds, function(nodeid){
					this.bindDomNode(nodeid);
				}, this);
			}
		},
		
		/**
		 * Handle keypress events at the outer containing DOM node to process
		 * arrow-key navigation.
		 * @param {Event} evt
		 */
		_onDomNodeKeypress: function(evt){
			var target = evt.target || evt.srcElement,
				handled = false;

			if(this.parentMenu && !evt.ctrlKey && !evt.altKey && (!target || !nodeUsesArrowKeys(target))){
				switch(evt.charOrCode){
					case this._nextMenuKey:
						this.parentMenu._getTopMenu().focusNext();
						handled = true;
						break;
						
					case this._prevMenuKey:
						if(this.parentMenu._isMenuBar){
							this.parentMenu.focusPrev();
						}else{
							this.onCancel(false);
						}
						handled = true;
						break;
				}
			}
			
			if(handled){
				event.stop(evt);
			}else{
				this.inherited(arguments);
			}
		},		
		
		/**
		 * Return the menu for the dialog, or undefined if no menu for the
		 * dialog can be found.
		 */
		_getMenuForDialog: function(){
			var children = this.getChildren(),
				result;
				
			for(var i = 0; !result && (i < children.length); i++){
				if(children[i] && children[i].menuForDialog){
					result = children[i];
				}
			}

			return result;
		},
		
		/**
		 * Finds focusable items in dialog,	and sets this._firstFocusItem and
		 * this._lastFocusItem.
		 */		
		_getFocusItems: function(){
			this.inherited(arguments);
			
			// if _firstFocusItem or _lastFocusItem have been set to our DOM node,
			// set them to the containerNode instead, which is a better focus target
			if(this._firstFocusItem == this.domNode){
				this._firstFocusItem = this.containerNode;
			}
			if(this._lastFocusItem == this.domNode){
				this._lastFocusItem = this.containerNode;
			}
		},
		
		/**
		 * Called when an event occurs that should result in the MenuDialog
		 * being shown.
		 * <p>
		 * Set timer to display myself. Using a timer rather than displaying
		 * immediately solves two problems:
		 * <ol>
		 * <li>
		 * IE: without the delay, focus work in "open" causes the system
		 * context menu to appear in spite of stopEvent.
		 * </li>
		 * <li>
		 * Avoid double-shows on linux, where shift-F10 generates an 
		 * oncontextmenu event even after a event.stop(e). (Shift-F10 on
		 * windows doesn't generate the oncontextmenu event.)
		 * </p>
		 * @private
		 * @param {Object} params An object containing data about the event. See
		 * {@link idx.oneui._EventTriggerMixin#_onTrigger}
		 */
		_onTrigger: function(params){
			var coords = null;
			if(!params.additionalData.leftClickToOpen && ("pageX" in params.event)){
				coords = { x: params.event.pageX, y: params.event.pageY }; 			
				if (params.triggerNode.tagName === "IFRAME") {
					// Specified coordinates are on <body> node of an <iframe>, convert to match main document
					var ifc = domGeometry.position(params.triggerNode, true), scroll = win.withGlobal(window.get(iframe.doc(params.triggerNode)), "docScroll", domGeometry);
					
					var cs = domStyle.getComputedStyle(params.triggerNode), tp = domStyle.toPixelValue, left = (has("ie") && has("quirks") ? 0 : tp(params.triggerNode, cs.paddingLeft)) + (has("ie") && has("quirks") ? tp(params.triggerNode, cs.borderLeftWidth) : 0), top = (has("ie") && has("quirks") ? 0 : tp(params.triggerNode, cs.paddingTop)) + (has("ie") && has("quirks") ? tp(params.triggerNode, cs.borderTopWidth) : 0);
					
					coords.x += ifc.x + left - scroll.x;
					coords.y += ifc.y + top - scroll.y;
				}
			}
	
			if(!this._openTimer){
				this._openTimer = setTimeout(lang.hitch(this, function(){
					delete this._openTimer;

					this.open({
						around: params.triggerNode,
						coords: coords,
						position: params.additionalData.popupPosition,
						useConnector: params.additionalData.useConnector
					});
				}), 1);
			}
			
			if(params.event.type != "hover")
				event.stop(params.event);
		},
	
		/**
		 * An attach point that is called when the MenuDialog loses focus. 
		 */
		onBlur: function(){
			this.inherited(arguments);

			if(this._closeOnBlur){
				this.close();
			}
		},

		/**
		 * Opens the MenuDialog at the specified position.
		 * @param {Object} args An object containing some the following fields,
		 * which are all optional but at least one of 'around' and 'coords'
		 * should be supplied:
		 * <ul>
		 * <li>
		 * 	around: A DOM node that the popup should be placed around.
		 * </li>
		 * <li>
		 * 	coords: A position (with fields x and y) containing coordinates
		 * 	that the popup should be placed at.
		 * </li>
		 * <li>
		 * 	position: Ordered list of positions to try placing the popup at.
		 * 	The first position which allows the popup to be fully visible
		 * 	within the viewport will be used, or the best available option.
		 * 	Possible values are:
		 * 	<ul>
		 * 		<li>
		 * 			before: places drop down to the left of the around
		 * 			node/coords, or to the right in RTL mode
		 * 		</li>
		 * 		<li>
		 *			after: places drop down to the right of the around
		 *			node/coords, or to the right in RTL mode
		 * 		</li>
		 * 		<li>
		 *			above: drop down goes above around node/coords
		 * 		</li>
		 * 		<li>
		 *			above-alt: same as above except right sides aligned
		 *			instead of left
		 * 		</li>
		 * 		<li>
		 *			below: drop down goes below around node/coords
		 * 		</li>
		 * 		<li>
		 *			below-alt: same as below except right sides aligned instead
		 *			of left
		 * 		</li>
		 * 		</ul>
		 *		If this field is omitted the current value of the widget
		 *		popupPosition property is used.
		 * </li>
		 * <li>
		 *		useConnector: If true, a shark-fin connector is shown linking
		 *		the popup to the around node/coords. If this field is omitted
		 *		the current value of the widget useConnector property is used.
		 * </li>
		 * </ul>
		 */		
		open: function(args){
			// store the node which focus should return to, if necessary, and if possible
			var refocusNode = null;
			if(this.refocus){
				refocusNode = this._focusManager.get("curNode");
				if(!refocusNode || dom.isDescendant(refocusNode, this.domNode)){
					refocusNode = this._focusManager.get("prevNode");
				}
				if(dom.isDescendant(refocusNode, this.domNode)){
					refocusNode = null;
				}
			}
			
			// use coords if supplied, otherwise use the around node if supplied,
			// or use current focus node, or (0,0) as a last resort
			var around = (args && (args.coords ? { x: args.coords.x, y: args.coords.y, w: 0, h: 0 } : args.around)) || refocusNode || this._focusManager.get("curNode") || { x: 0, y: 0, w: 0, h: 0 };
				
			var closeFunction = lang.hitch(this, function(){
				if(refocusNode){
					refocusNode.focus();
				}
				
				this.close();
			});
						
			this._useConnectorForPopup = (args && ("useConnector" in args)) ? args.useConnector : this.useConnector;
				
			popup.open({
				popup: this,
				around: around,
				onExecute: closeFunction,
				onCancel: closeFunction,
				orient: (args && ("position" in args)) ? args.position : this.popupPosition
			});
				
			delete this._useConnectorForPopup;
						
			this.focus();
			this._closeOnBlur = true;
		},
		
		/**
		 * Closes the MenuDialog. 
		 */
		close: function(){
			popup.close(this);
		},
		
		/**
		 * Attach the MenuDialog to the specified trigger node.
		 * @param {string|DOMNode} node The node to be used to trigger the
		 * MenuDialog to pop-up. 
		 * @param {Object} params An object that can contain properties that 
		 * override the popupPosition, useConnector, leftClickToOpen,
		 * and hoverToOpen properties of the 
		 * MenuDialog on a per node attachment basis.
		 */
		bindDomNode: function(/*String|DomNode*/ node, /*Object*/ params){
			var settings = lang.delegate(this);
			for(var name in params){
				settings[name] = params[name];
			}
			
			this._addEventTrigger(node, "click", function(fparams){
				return settings.leftClickToOpen;
			}, settings);
			
			this._addEventTrigger(node, "contextmenu", function(fparams){
				return !settings.leftClickToOpen;
			}, settings);
			
			this._addEventTrigger(node, "keydown", function(fparams){
				// if right-click is the trigger, accept Shift+F10 as an equivalent
				// in case the browser doesn't send us the contextmenu event
				return !settings.leftClickToOpen && fparams.event.shiftKey && (fparams.event.keyCode == keys.F10);
			}, settings);
			
			this._addEventTrigger(node, "hover", function(fparams){
				return settings.hoverToOpen;
			}, settings);
		},
	
		/**
		 * Detach menu dialog from given DOM node. If no node is specified,
		 * detach menu dialog from all bound DOM nodes.
		 * @param {string|DOMNode} nodeName The DOM node to dissociate the 
		 * MenuDialog from.
		 */
		unBindDomNode: function(/*String|DomNode*/ nodeName){
			this._removeEventTriggers(nodeName);
		},
		
		/**
		 * Configure the widget to display in a given position. This may be
		 * called several times during popup in order to "try out" different
		 * configurations and then select the best. It will always be called
		 * one final time with the actual configuration to be used.
		 * @param {Object} aroundNodeSize
		 * @param {string} aroundCorner
		 * @param {string} corner
		 * @param {boolean} useConnector
		 */
		_layoutNodes: function(/*Object*/ aroundNodeSize, /*String*/ aroundCorner, /*String*/ corner, /*Boolean*/ useConnector){
			var newcss = useConnector ? "oneuiMenuDialogConnected" : "",
				newparentcss = "",
				cornerv = corner && (corner.length >= 1) && corner.charAt(0),
				cornerh = corner && (corner.length >= 2) && corner.charAt(1),
				mybox = domGeometry.getContentBox(this.domNode),
				displacementProperty, displacementValue,
				getContentBox = function(node){
					var style = node.style,
						oldDisplay = style.display,
						oldVis = style.visibility;
					if(style.display == "none"){
						style.visibility = "hidden";
						style.display = "";
					}
					var result = domGeometry.getContentBox(node);
					style.display = oldDisplay;
					style.visibility = oldVis;
					return result;
				}
			
			// we use a combination of two classes to enable the CSS markup to be applied
			// to position our connector correctly.
			//
			// dijitTooltipAbove, dijitTooltipBelow, dijitTooltipLeft, dijitTooltipRight,
			// indicating which way the popup goes from the position or around node
			//
			// When dijitTooltipAbove or dijitTooltipBelow are used, one of the classes
			// dijitTooltipABLeft, dijitTooltipABRight, dijitTooltipABMiddle is also
			// applied to indicate where on the top/bottom edge the connector should appear.    
			//
			// When dijitTooltipLeft or dijitTooltipRight are used, one of the classes
			// dijitTooltipLRTop, dijitTooltipLRBottom, dijitTooltipLRMiddle is also
			// applied to indicate where on the left/right edge the connector should appear.			
			 
			if( (cornerv === 'M')  // 'ML' or 'MR'
			 || ((cornerh !== 'M') && (cornerh !== aroundCorner.charAt(1)))){  // '?L' against '?R' means a left/right abuttal
				// the popup is going left or right
				newcss += " dijitTooltip" + (cornerh === 'L' ? "Right" : "Left");
				
				switch(cornerv){
					case 'M':
						newcss += " dijitTooltipLRMiddle";
						break;
					case 'T':
						newcss += " dijitTooltipLRTop";
						useConnector && (newparentcss = "connectorNearTopEdge");
						if(aroundNodeSize.h > 0){
							displacementProperty = "top";
							displacementValue = Math.max(4, 4 + Math.min(getContentBox(this.domNode.parentNode).h - 24, aroundNodeSize.h / 2)) + "px"; 
						}
						break;
					case 'B':
						newcss += " dijitTooltipLRBottom";
						useConnector && (newparentcss = "connectorNearBottomEdge");
						if(aroundNodeSize.h > 0){
							displacementProperty = "bottom";
							displacementValue = (4 + Math.min(getContentBox(this.domNode.parentNode).h - 24, aroundNodeSize.h / 2)) + "px"; 
						}
						break;
				}
			}else{  // 'TM' or 'BM', or '?L' against '?L' or '?R' against '?R' which both mean an above/below abuttal
				// the popup is going above or below
				newcss += " dijitTooltip" + (cornerv === 'T' ? "Below" : "Above");
				
				switch(cornerh){
					case 'M':
						newcss += " dijitTooltipABMiddle";
						break;
					case 'L':
						newcss += " dijitTooltipABLeft";
						useConnector && (newparentcss = "connectorNearLeftEdge");
						if(aroundNodeSize.w > 0){
							displacementProperty = "left";
							displacementValue = Math.max(4, 4 + Math.min(getContentBox(this.domNode.parentNode).w - 16, aroundNodeSize.w / 2)) + "px"; 
						}
						break;
					case 'R':
						newcss += " dijitTooltipABRight";
						useConnector && (newparentcss = "connectorNearRightEdge");
						if(aroundNodeSize.h > 0){
							displacementProperty = "right";
							displacementValue = (4 + Math.min(getContentBox(this.domNode.parentNode).w - 24, aroundNodeSize.w / 2)) + "px"; 
						}
						break;
				}
			}
			
			domClass.replace(this.domNode, newcss, this._currentOrientClass || "");
			this._currentOrientClass = newcss;
			
			domClass.replace(this.domNode.parentNode, newparentcss, this._currentConnectorClass || "");
			this._currentConnectorClass = newparentcss;
			
			// remove any previous displacement values that might have been applied
			this.connectorNode.style.top = "";
			this.connectorNode.style.bottom = "";
			this.connectorNode.style.left = "";
			this.connectorNode.style.right = "";
			
			if(displacementProperty){
				this.connectorNode.style[displacementProperty] = displacementValue;
			}
		},
		
		/**
		 * Configure widget to be displayed in given position relative to the
		 * trigger node. This is called from the dijit.popup code, and should
		 * not be called directly.
		 * @private
		 * @param {DOMNode} node
		 * @param {string} aroundCorner
		 * @param {string} corner
		 * @param {Object} spaceAvailable
		 * @param {Object} aroundNodeCoords
		 */
		orient: function(/*DomNode*/ node, /*String*/ aroundCorner, /*String*/ corner, spaceAvailable, aroundNodeCoords){
			this._layoutNodes(aroundNodeCoords, aroundCorner, corner, ("_useConnectorForPopup" in this) ? this._useConnectorForPopup : this.useConnector);
		},

		/**
		 * An attach point that is called when the MenuDialog is opened. 
		 */
		onOpen: function(pos){
			this.isShowingNow = true;
			
			// set up our initial display
			this._layoutNodes(pos.aroundNodePos, pos.aroundCorner, pos.corner, ("_useConnectorForPopup" in this) ? this._useConnectorForPopup : this.useConnector);
			this.reset();
			
			// if there is a menu we are to work with, connect to it now
			var menu = this._getMenuForDialog();
			if(menu){
				// ensure that our menu has parentMenu set to our parent
				// so that focus chains connect up correctly
				if(this._menuparented){
					// this shouldn't happen, but handle it cleanly if it does
					this._menuparented.parentMenu = null;
				}
				menu.parentMenu = this.parentMenu;
				this._menuparented = menu;
				
				// ensure that "execute" events on our menu trigger "execute"
				// events on us, so that the notification passes up the menu
				// cascade chain correctly
				if(this._handleexecute){
					// this shouldn't happen, but handle it cleanly if it does
					this._handleexecute.remove();
				}
				var fireexecute = lang.hitch(this, this.onExecute);
				this._handleexecute = menu.on("execute", fireexecute);
				
				// ensure that "execute" events returning from popup/cascade
				// menus opened in turn by our menu also trigger "execute"
				// events on us. The "execute" events will otherwise terminate
				// there, because our menu will not be in the popup stack; all
				// the child menus cascading from it will be correctly folded
				// away, but we need to pass the notification upwards so that
				// the next segment of the popup stack can also be folded away.
				if(this._handleopen){
					// this shouldn't happen, but handle it cleanly if it does
					this._handleopen.remove();
				}
				this._handleopen = aspect.after(menu, "_openPopup", function(){
					// after _openPopup is called, retrieve the popup stack item just
					// added and hook into its onExecute() method, which will be called
					// when an execute is triggered on a descendant menu item
					var stackitem = popup._stack[popup._stack.length - 1];
					if(!stackitem._menuregistered){
						stackitem._menuregistered = true;
						stackitem.handlers.push(aspect.around(stackitem, "onExecute", function(originalOnExecute){
							return function(){
								// during the default onExecute handling, all the handlers
								// will be removed, so we can't use an 'after'. Instead,
								// we use 'around', call the original, then fire our own
								// 'execute' event. 
								originalOnExecute.apply(this, arguments);
								fireexecute();							
							}
						}));
					}
				}, true);
			}
			
			this._onShow(); // lazy load trigger
		},

		/**
		 * An attach point that is called when the MenuDialog is closed. 
		 */
		onClose: function(){
			this.isShowingNow = false;
			this._closeOnBlur = false;
			
			// if we connected to a menu, disconnect it now
			if(this._handleexecute){
				this._handleexecute.remove();
				this._handleexecute = null;
			}
			if(this._handleopen){
				this._handleopen.remove();
				this._handleopen= null;
			}
			
			// if we set the parent of a menu, reset it now
			if(this._menuparented){
				this._menuparented.parentMenu = null;
				this._menuparented = null;
			}
			
			this.onHide();
		},

		/**
		 * An attach point for the parent menu to listen for the MenuDialog being
		 * cancelled. Calling it will cause this MenuDialog to be closed, 
		 * but not any menus it may have cascaded from.
		 * @name idx.oneui.MenuDialog.prototype.onCancel
		 * @function
		 * @public
		 */		

		/**
		 * An attach point for the parent menu to listen for menu items being
		 * executed. Calling it will cause this MenuDialog and any menus it
		 * may have cascaded from to be closed.
		 */		
		onExecute: function(){
		}

	});
});

},
'ibm/usmi/util/DefaultURLLaunchTask':function(){
// wrapped by build app
define("ibm/usmi/util/DefaultURLLaunchTask", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/Trace,dojox/data/dom,ibm/usmi/util/DefaultMessageHandler,ibm/usmi/core/utils/Logger"], function(dijit,dojo,dojox){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

dojo.provide("ibm.usmi.util.DefaultURLLaunchTask");

dojo.require("ibm.tivoli.navmodel.Trace");
dojo.require("dojox.data.dom");
dojo.require("ibm.usmi.util.DefaultMessageHandler");
dojo.require("ibm.usmi.core.utils.Logger");

ibm.usmi.util.DefaultURLLaunchTask._trace = new ibm.tivoli.navmodel.Trace("ibm.usmi.util.DefaultURLLaunchTask");
ibm.usmi.util.DefaultURLLaunchTask._defaultURLLaunchTaskLogger = new ibm.usmi.core.utils.Logger("ibm.usmi.util", "DefaultURLLaunchTask");

dojo.declare("ibm.usmi.util.DefaultURLLaunchTask", null, {

    // private members
    _launchParms : null,
    _menuItem : null,
    _context : null,
    _trace : ibm.usmi.util.DefaultURLLaunchTask._trace,
    
    // the logger
    _defaultURLLaunchTaskLogger : ibm.usmi.util.DefaultURLLaunchTask._defaultURLLaunchTaskLogger,

    constructor : function() {
        // summary:
        //        Coming soon
        // description:
        //        Coming soon

        // Register for a URL launch task event
        dojo.subscribe("DefaultURLLaunch", this, "_handleURLLaunch");

    },

    _handleURLLaunch : function(launchParms, menuItem, context) {
        // summary:
        //        Coming soon
        // description:
        //        Coming soon

        this._launchParms = launchParms;
        this._menuItem = menuItem;
        this._context = context;

        var runServerTaskAndLaunchURL = function(tasks) {
            var task = null;

            var launchUrl = function() {
                this.launchURL(this._launchParms);
            };

            // Get the selected task from all available tasks
            task = this.fetchTask(tasks);

            // Call execute on the server task
            task.execute({
                onComplete : dojo.hitch(this, launchUrl),
                onError : dojo.hitch(this, this._handleExecuteOnServerError),
                selection : this.getSelection(this._context)
            });
        };

        // Special case for group export - need to call execute on task to populate file to be exported first
        if (menuItem && menuItem.actionID == "GroupExport") {

            // set the task
            context.dataset.getTasks({
                onComplete : dojo.hitch(this, runServerTaskAndLaunchURL),
                onError : dojo.hitch(this, this.getTasksError),
                taskCategory : "menu", //TODO: Could I specifically request nav hints instead of all tasks?
                selection : this.getSelection(context)
            });
        } else {
            this.launchURL(launchParms);
        }
    },

    fetchTask : function(tasks) {

        //Want to loop through tasks looking for match to menuItem actionID
        if (tasks) {
            for ( var i = 0; i < tasks.length; i++) {
                if (tasks[i].getId() === this._menuItem.actionID) {
                    return tasks[i];
                } else if (tasks[i].getChildTasks()) {
                    var subTasks = tasks[i].getChildTasks();
                    if (subTasks.length > 0) {
                        var task = this.fetchTask(subTasks);
                        if (task && (task.getId() === this._menuItem.actionID)) {
                            return task;
                        }
                    }
                }
            }
        }
    },

    launchURL : function(launchParms) {

        // Check launchParms to decide how to handle
        if (launchParms) {
            // ISCPage launch
            if (launchParms.launchType === "ISCPage") {      
            	try {
	                if (this._context.selection && !(this._context.selection.isEmpty())) {
	                    var targets = this._context.selection.getItemIds();
	                	launchInContext(launchParms.pageID, targets, false);
	                } else {
	                	launchInContext(launchParms.pageID);
	                } 
            	} catch (ex) {
                    //There was a problem launching the window
                    var msgHandler = new ibm.usmi.util.DefaultMessageHandler({});
                    msgHandler.displayException(ex);
                }
            }

            // URL launch
            else if (launchParms.launchType === "URL") {
                if (launchParms.urlToLaunch) {
                    if (launchParms.windowId) {
                        try {
                            var openWindow = window.open(launchParms.urlToLaunch, launchParms.windowId);
                            openWindow.focus();
                        } catch (ex) {
                            //There was a problem launching the window
                            var msgHandler = new ibm.usmi.util.DefaultMessageHandler({});
                            msgHandler.displayException(ex);
                        }
                    } else {
                        try {
                            var openWindow = window.open(launchParms.urlToLaunch);
                            openWindow.focus();
                        } catch (ex) {
                            //There was a problem launching the window
                            var msgHandler = new ibm.usmi.util.DefaultMessageHandler({});
                            msgHandler.displayException(ex);
                        }
                    }
                } else {
					if (this._defaultURLLaunchTaskLogger.isWarn()) {
						this._defaultURLLaunchTaskLogger.warn("launchParms are not valid - urlToLaunch not defined", "launchURL");
					}
                }
            }

            // Widget launch, used by DefaultDojoUrlTask, were we are loading new widgets
            else if (launchParms.launchType === "Widget") {

                if(launchParms.urlParameters) {
                    // Launch the widget within the framework
                    try {
                        launchNextGenPage(launchParms.urlParameters);
                    } catch (ex) {
                        //There was a problem launching the window
                        var msgHandler = new ibm.usmi.util.DefaultMessageHandler({});
                        msgHandler.displayException(ex);
                    }
                }
                else {
					if (this._defaultURLLaunchTaskLogger.isWarn()) {
						this._defaultURLLaunchTaskLogger.warn("launchParms are not valid - urlParameters not defined", "launchURL");
					}
                }
            } else {
				if (this._defaultURLLaunchTaskLogger.isWarn()) {
					this._defaultURLLaunchTaskLogger.warn("launchParms are not valid - launchType not define or not valid", "launchURL");
				}
            }

        } else {
			if (this._defaultURLLaunchTaskLogger.isWarn()) {
				this._defaultURLLaunchTaskLogger.warn("launchParms are not defined", "launchURL");
			}
        }
    },

    getSelection : function(context) {
        var selectedThings = context.selection; //this._args.context.selection;
        var currentSelection = null;
        if (selectedThings) {
            currentSelection = new ibm.tivoli.navmodel.Selection({
                version : context.dataset.getVersion()
            }); //this.getDataSet().getVersion()});
            currentSelection.addItems(selectedThings.getItemIds());
            currentSelection.addLinks(selectedThings.getLinkIds());
            currentSelection.addRelationships(selectedThings.getRelationshipIds());
        }
        return currentSelection;
    },

    getTasksError : function(error, request) {
        //There was a problem getting the tasks from the dataset
        var msgHandler = new ibm.usmi.util.DefaultMessageHandler({});
        msgHandler.displayException(error);
    },

    _handleExecuteOnServerError : function(error, exception, ioArgs) {
        // summary:
        //        Coming soon
        // description:
        //        Coming soon
        // error:
        //
        // exception:
        //
        // ioArgs:
        //

        if (this._trace.isError()) {
            this._trace.error(">>> error executing server task = " + error, "_handleExecuteOnServerError");
        }

        //There was a problem executing the task on the server
        var msgHandler = new ibm.usmi.util.DefaultMessageHandler({});
        msgHandler.displayException(exception);
    },
});

});

},
'ibm/tivoli/navmodel/Provider':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/ProviderUtils"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */
dojo.provide("ibm.tivoli.navmodel.Provider"); 

// includes
dojo.require("ibm.tivoli.navmodel.ProviderUtils");

////////////////////////////////////
// Provider
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Provider", null, {
	// private members       
	_provider: null,
	_baseUrl: null,
	_version: null,
	
	/*=====
	constructor_kwArgs: function(provider, baseUrl, version) {
		// provider: Object
		//		json provider object
		// baseUrl: String
		//		base url for identifying rest service
		// version: Float
		//		optional: version used to communicate with rest service (i.e. json version)
	},
	=====*/
	
	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs Provider object.
		this._baseUrl = kwArgs.baseUrl;
		this._version = kwArgs.version;
		this._provider = kwArgs.provider;
		if (!this._provider) {
			this._provider = {};
		}
	},

	getAttributes: function() {
		// summary:
		//		Get all general attributes for the provider
		return dojo.clone(this._provider);
	},
	
	getAttribute: function(key) {
		// summary:
		//		Returns general attribute for requested key
		return this._provider[key];
	},

	setAttribute: function(key, value) {		
		// summary:
		//		Set general attribute for requested key
		this._provider[key] = value;
	},
	
	removeAttribute: function(key) {
		// summary:
		//		Remove general attribute for requested key
		delete this._provider[key];
	},
	
	isRemote: function() {
		// summary:
		//		Returns true if the provider is remote; false
		//		if the provider is local to the server
		return this._provider.remote;
	},	
	
	getMSSName: function() {
		// summary:
		//		Returns the MSSName of this provider.
		return this._provider.MSSName;
	},	
		
	getId: function() {
		// summary:
		//		Returns the id of this provider
		return this._provider.id;
	},
	
	setId: function(id) {
		// summary:
		//		Sets the provider id
		this._provider.id = id;
	},
	
	getExternalProviderId: function() {
		// summary:
		//		Returns the externalProviderId (i.e. the id that the provider
		//		is known by on the remote server)
		return this._provider.externalProviderId;
	},
	
	setExternalProviderId: function(externalProviderId) {
		// summary:
		//		Sets the externalProviderId (i.e. the id that the provider
		//		is known by on the remote server)
		this._provider.externalProviderId = externalProviderId;
	},
	
	getLabel: function() {
		// summary:
		//		Returns the label of this provider
		return this._provider.label;
	},
	
	setLabel: function(label) {
		// summary:
		//		Sets the provider label
		this._provider.label = label;
	},
	
	getDescription: function() {
		// summary:
		//		Returns the description of this provider
		return this._provider.description;
	},
	
	setDescription: function(description) {
		// summary:
		//		Sets the provider description
		this._provider.description = description;
	},
	
	getType: function() {
		// summary:
		//		Returns the type of this provider.
		return this._provider.type;
	},
	
	setType: function(type) {
		// summary:
		//		Sets the provider type
		this._provider.type = type;
	},
	
	getAuthUser: function() {
		// summary:
		//		Returns optional authUser attribute
		return this._provider.authUser;
	},
	
	setAuthUser: function(authUser) {
		// summary:
		//		Sets optional authUser attribute
		this._provider.authUser = authUser;
	},
	
	setAuthPassword: function(authPassword) {
		// summary:
		//		Sets optional authPassword attribute
		this._provider.authPassword = authPassword;
	},
	
	getBaseUrl: function() {
		// summary:
		//		Returns baseUrl attribute
		return this._provider.baseUrl;
	},
	
	setBaseUrl: function(baseUrl) {
		// summary:
		//		Sets baseUrl attribute
		this._provider.baseUrl = baseUrl;
	},
	
	getAuthUrl: function() {
		// summary:
		//		Returns authUrl attribute
		return this._provider.authUrl;
	},
	
	setAuthUrl: function(authUrl) {
		// summary:
		//		Sets baseUrl attribute
		this._provider.authUrl = authUrl;
	},
	
	getProxyHost: function() {
		// summary:
		//		Returns proxyHost attribute
		return this._provider.proxyHost;
	},
	
	getProxyPort: function() {
		// summary:
		//		Returns proxyPort attribute
		return this._provider.proxyPort;
	},
	
	setProxyHostAndPort: function(proxyHost, proxyPort) {
		// summary:
		//		Sets proxyHost and proxyPort attributes
		this._provider.proxyHost = proxyHost;
		this._provider.proxyPort = proxyPort;
	},

	getClientFacingHost: function() {
		// summary:
		//		Returns clientFacingHost attribute
		return this._provider.clientFacingHost;
	},
	
	getClientFacingPort: function() {
		// summary:
		//		Returns clientFacingPort attribute
		return this._provider.clientFacingPort;
	},

	setClientFacingHostAndPort: function(clientFacingHost, clientFacingPort) {
		// summary:
		//		Sets clientFacingHost and clientFacingPort attributes
		this._provider.clientFacingHost = clientFacingHost;
		this._provider.clientFacingPort = clientFacingPort;
	},
	 
	isUseFIPS: function() {
		// summary:
		//		Returns useFIPS attribute
		return (this._provider.useFIPS !== undefined ? this._provider.useFIPS : false);
	},
	
	setUseFIPS: function(useFIPS) {
		// summary:
		//		Sets useFIPS attribute
		this._provider.useFIPS = useFIPS;
	},
	
	isSSO: function() {
		// summary:
		//		Returns sso attribute
		return (this._provider.sso !== undefined ? this._provider.sso : false);
	},
	
	setSSO: function(sso) {
		// summary:
		//		Sets sso attribute
		this._provider.sso = sso;
	},
    
	/*=====
	getDatasources_kwArgs: function(onComplete, onError, start, count, condition, sync) {
		// onComplete: function(datasources, datasourceCount, kwArgs)
		//		callback for successful fetch
		// onError: function(error, exception, ioArgs)
		//		callback for error conditions
		// start: int
		//		start index to fetch
		// count: int
		//		number of datasources to fetch; null means all
		// sort: String
		//		comma deliminated attribute string (preceded with optional +/- indicating ascending/descending)
		//		example:  label,type
		// condition: String
		//		condition expression string 
		//		example: description contains 'abcd'
		// sync: Boolean
		//		indicates if fetch is synchronous		
	},
	=====*/
	
	getDatasources: function(/*getDatasources_kwArgs*/ kwArgs) {
		var args = {
			onComplete: function(datasources, datasourceCount, kwArgs) {
			},
			onError: function(error, exception, ioArgs) {
			},
			baseUrl: this._baseUrl,
			provider: this._provider.id,			
			start: 0,
			count: null,
			sourt: null,
			condition: null,
			sync: false,
			version: this._version		
		};
		dojo.mixin(args, kwArgs);
		
		return ibm.tivoli.navmodel.ProviderUtils.getDatasources(args);
	},
	
	/*=====
	getDatasets_kwArgs: function(onComplete, onError, datasource, start, count, condition, sync) {
		// onComplete: function(datasets, datasetCount, kwArgs)
		//		callback for successful fetch
		// onError: function(error, exception, ioArgs)
		//		callback for error conditions
		// datasource: String
		// 		datasource id
		// start: int
		//		start index to fetch
		// count: int
		//		number of datasources to fetch; null means all
		// sort: String
		//		comma deliminated attribute string (preceded with optional +/- indicating ascending/descending)
		//		example:  label,type
		// condition: String
		//		condition expression string 
		//		example: description contains 'abcd'
		// sync: Boolean
		//		indicates if fetch is synchronous		
	},
	=====*/
	
	getDatasets: function(/*getDatasets_kwArgs*/ kwArgs) {
		var args = {
			onComplete: function(datasets, datasetCount, kwArgs) {
			},
			onError: function(error, exception, ioArgs) {
			},
			baseUrl: this._baseUrl,
			provider: this._provider.id,
			datasource: null,		
			start: 0,
			count: null,
			sourt: null,
			condition: null,
			sync: false,
			version: this._version	
		};
		dojo.mixin(args, kwArgs);
		
		return ibm.tivoli.navmodel.ProviderUtils.getDatasets(args);
	}
});

});

},
'ibm/tivoli/tip/dojo/TIPHelper':function(){
// wrapped by build app
define("ibm/tivoli/tip/dojo/TIPHelper", ["dijit","dojo","dojox"], function(dijit,dojo,dojox){
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
dojo.provide("ibm.tivoli.tip.dojo.TIPHelper");

dojo.declare("ibm.tivoli.tip.dojo.TIPHelper",
	          null,
{
	constructor: function () {
	},
	// childNode must be a node within your portlet
	// padding is the amount of padding you want around your portlet
	//   e.g., "0px"
	setPortletPadding: function (childNode, padding) {
		var node = this.getAncestorByClass (childNode, "isc-portlet-bkg");
		if (node) {
			dojo.style (node, "padding", padding);
	    var nextnode = dojo.query( 'table[id$="_portlet_content"]', node )[0];
        if (nextnode) {
        	dojo.style( nextnode, "padding", padding );
        }
		}
		
	},
	getAncestorByClass: function (element, className) {
		var node = null;
		for(node = element.parentNode; node!= null; node = node.parentNode) {
			if(dojo.hasClass(node, className)) {
				break;
			}
		}
		return node;
	},	
	getAncestorByName: function (element, name) {
		var result=null;
		try {
			while(result===null && element!==null) {
				element=element.parentNode;
				var elementname = element.getAttribute ("name");
				if (elementname == name) {
					result=element;
				}
			}
		} catch(e) {
			//no node
		}
		//console.log ("Nodename = " + result);
		return(result);
	},
	getAncestorByID: function (element, id) {
		var result=null;
		try {
			while(result===null && element!==null) {
				element=element.parentNode;
				var elementid = element.getAttribute ("id");
				if (elementid == id) {
					result=element;
				}
			}
		} catch(e) {
			//no node
		}
		//console.log ("Nodename = " + result);
		return(result);
		
	},
	getPortletModel: function (element) {
		console.log ("looking for IscPortlet, starting with element: ");
		console.log (element);
		var node = this.getAncestorByName (element, "IscPortlet");
		console.log ("found IscPortlet node: ");
		console.log (node);
		var dataObject = null;
		
		if (node) {
			var data = node.getAttribute ("portletdataobject");
			console.log ("data: ");
			console.log (data);
			dataObject = dojo.fromJson (data);
			console.log ("get portlet data: ");
			console.log (dataObject);
			return (dataObject);
		} else {
			return (undefined);
		}
	},
	getFreeformPageModel: function (element) {
		node = this.getAncestorByClass (element, "tip-freeform-page");
		if (node === null) {
			console.error ("pageModel cannot be found. node is null");
			return (undefined);
		} 

		console.log ("node with tip-freeform-page class: ");
		console.log (node);
			
		var dataObject = node.modelObject.pageData;
		if ((dataObject === null) || (dataObject === undefined)) {
			var data = node.getAttribute ("pagedataobject");
			dataObject = dojo.fromJson (data);
		}
		
		console.log ("get portlet data: ");
		console.log (dataObject);
		return (dataObject);
	},
	getPortletOID: function (element) {
		var dataObject = this.getPortletModel (element);
		if (dataObject) {
			var result = dataObject.oid;
			return (result);
		} else { 
			return (undefined);
		}
	},
	getPortletWindowID: function (element) {
		var dataObject = this.getPortletModel (element);
		if (dataObject) {
			var result = dataObject.portletName;
			return (result);
		} else {
			return (undefined);
		}
	},
	getPortletPageID: function (element) {
		console.log("Enter getPortletPageID");
		var parent = this.getAncestorByName (element, "IscPage");
		var data = parent.getAttribute ("pagedataobject");
		var dataObject = dojo.fromJson (data);
		var result = dataObject.uniquePageName;
		return result;
		/*} else {
			dataObject = this.getFreeformPageModel (element);
			var result = dataObject.uniquePageName;
			return (result);
		}*/
	},
	getPortletPageName: function (element) {
		var parent = this.getAncestorByName (element, "IscPage");
		if (parent) {
		    var pageName = parent.getAttribute("aria-label");
			if(pageName)
				return pageName;
			var node = dojo.query ("td.isc-layout-titletext", parent);
			if (node === undefined) {
				node = dojo.query ("div.tip-portlet-titletext", parent);
			}
			
	//		var node = dojo.query ("td.isc-tab-sel-l", frames["taskbar"].document);
			if (node) {
				if (node.length == 1) {
//					var result = node[0].innerHTML.alt;
					var result = node[0].childNodes[0].nodeValue;
					if (result) {
						// strip leading white space
						result = result.replace(/^\s+/,"");
						// strip trailing white space
						result = result.replace(/\s+$/,"");
					}
					console.log ("portlet page name: ");
					console.log (result);
					return (result);
				} else if (node.length === 0) {
					console.log ("could not find node for portlet page name. array length is 0.");
				} else {
					console.log ("found multiple nodes for portlet page names");
					console.log (node);
				}
			} else {
				console.log ("could not find node for portlet page name");
			}
		} else {
			console.log ("no parent node to get portlet page name");
		}
	},
	getTopPortletNodeFreeform: function (element) {
		var node = this.getAncestorByClass (element, "tip-portlet-frame");
		return (node);
	}, 
	getTopPortletNode: function (element) {
		var node = this.getAncestorByName (element, "IscPortlet");
		if (node) {
		} else {
			console.log ("cannot find TopPortletNode with name IscPortlet... looking for tip-portlet-frame class instead... must be freeform page");
			node = this.getTopPortletNodeFreeform (element);
		}
		
		return (node);		
	},
	getPortletTitle: function (element) {
		console.log ("enter getPortletTitle");
		var oid = this.getPortletOID (element);
		if (oid) {
			var node = dojo.byId ("titleText." + oid);
			if (node) {
				return node.innerHTML;
			} else {
				console.log ("cannot find node for title text: titleText." + oid);
			}
		} else {
			console.log ("freeform title search");
			var z = this.getAncestorByClass (element, "tip-portlet-frame");
			if (z) {
				// OMG... what a hack!
				/*
				z = z.childNodes[1];  // tip-portlet-inner
				z = z.childNodes[1];  // tip-portlet-titlebar
				z = z.childNodes[1];  // tip-portlet-titletext
				z = z.childNodes[1];  // h1
				*/
				for(var i=0; i<4 && z.childNodes[1]; i++){
					z = z.childNodes[1];
				}
				if (z.childNodes[0].getAttribute) {  // getAttribute function must exist for it to be an element
					z = z.childNodes[0];  // span
				}

				return z.innerHTML;
			}
		}
		
		return (undefined);
	},
	_setPortletTitleFreeform: function (element, title) {
		console.log ("enter _setPortletTitleFreeform");
		var z = this.getAncestorByClass (element, "tip-portlet-frame");
		if (z) {
			// OMG... what a hack!
			z = z.childNodes[1];  // tip-portlet-inner
			z = z.childNodes[1];  // tip-portlet-titlebar
			z = z.childNodes[1];  // tip-portlet-titletext
			z = z.childNodes[1];  // h1
			if (z.childNodes[0].getAttribute) {  // getAttribute function must exist for it to be an element
				z = z.childNodes[0];  // span
			}
			
			console.log ("found element for title: ");
			console.log (z);
			z.innerHTML = title;
		}
	},
	setPortletTitle: function (element, title) {
		console.log ("enter setPortletTitle: " + title);

		try{
		
		var oid = this.getPortletOID (element);

		if (oid) {
			var node = dojo.byId ("titleText." + oid);
			if (node) {
				node.innerHTML = title;
			} else {
				console.log ("cannot find node for title text: titleText." + oid);
				this._setPortletTitleFreeform (element, title);
			}
		} else {
				var y = this.getAncestorByClass(element, "tip-portlet-inner");
				y.setAttribute ("id", title);
				dojo.query('h1', y).forEach(function(node, index, arr){						
					node.innerHTML = title;					
				});
				/*
				var z = y.childNodes[1];
				z = z.childNodes[1];
				z = z.childNodes[1];
				//z = z.childNodes[1];
				z.innerHTML = title;
				*/
		}

		}
		catch(e){
			console.info("failure="+e);
		}
	}
});

});

},
'dijit/MenuBar':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/_base/event", // event.stop
	"dojo/keys", // keys.DOWN_ARROW
	"./_MenuBase",
	"dojo/text!./templates/MenuBar.html"
], function(declare, event, keys, _MenuBase, template){

/*=====
	var _MenuBase = dijit._MenuBase;
=====*/

// module:
//		dijit/MenuBar
// summary:
//		A menu bar, listing menu choices horizontally, like the "File" menu in most desktop applications

return declare("dijit.MenuBar", _MenuBase, {
	// summary:
	//		A menu bar, listing menu choices horizontally, like the "File" menu in most desktop applications

	templateString: template,

	baseClass: "dijitMenuBar",

	// _isMenuBar: [protected] Boolean
	//		This is a MenuBar widget, not a (vertical) Menu widget.
	_isMenuBar: true,

	postCreate: function(){
		var l = this.isLeftToRight();
		this.connectKeyNavHandlers(
			l ? [keys.LEFT_ARROW] : [keys.RIGHT_ARROW],
			l ? [keys.RIGHT_ARROW] : [keys.LEFT_ARROW]
		);

		// parameter to dijit.popup.open() about where to put popup (relative to this.domNode)
		this._orient = ["below"];
	},

	focusChild: function(item){
		// overload focusChild so that whenever the focus is moved to a new item,
		// check the previous focused whether it has its popup open, if so, after
		// focusing the new item, open its submenu immediately
		var prev_item = this.focusedChild,
			showpopup = prev_item && prev_item.popup && prev_item.popup.isShowingNow;
		this.inherited(arguments);
		if(showpopup && item.popup && !item.disabled){
			this._openPopup();		// TODO: on down arrow, _openPopup() is called here and in onItemClick()
		}
	},

	_onKeyPress: function(/*Event*/ evt){
		// summary:
		//		Handle keyboard based menu navigation.
		// tags:
		//		protected

		if(evt.ctrlKey || evt.altKey){ return; }

		switch(evt.charOrCode){
			case keys.DOWN_ARROW:
				this._moveToPopup(evt);
				event.stop(evt);
		}
	},

	onItemClick: function(/*dijit._Widget*/ item, /*Event*/ evt){
		// summary:
		//		Handle clicks on an item. Cancels a dropdown if already open.
		// tags:
		//		private
		if(item.popup && item.popup.isShowingNow){
			item.popup.onCancel();
		}else{
			this.inherited(arguments);
		}
	}
});

});

},
'ibm/tivoli/navmodel/DatasetTreeStore':function(){
// wrapped by build app
define("ibm/tivoli/navmodel/DatasetTreeStore", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/DatasetStore,ibm/tivoli/navmodel/Dataset"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */
 
dojo.provide("ibm.tivoli.navmodel.DatasetTreeStore");

// includes
dojo.require("ibm.tivoli.navmodel.DatasetStore");
dojo.require("ibm.tivoli.navmodel.Dataset");
	
ibm.tivoli.navmodel.DatasetTreeStore._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.DatasetTreeStore");
	
///////////////////////////////////////////
// This dojo datastore is compatible with:
// 		dijit.Tree
//		dojox.grid.TreeGrid
//
// Do NOT use for:
//		dojox.grid.LazyTreeGrid
// instead use DatasetLazyTreeStore
///////////////////////////////////////////

dojo.declare("ibm.tivoli.navmodel.DatasetTreeStore", ibm.tivoli.navmodel.DatasetStore, {
	// private members 
	_clearCache: false,
	_parent: null,
	_children: null,
	_isTreeGrid: false,
	_sort: null,
	_trace: ibm.tivoli.navmodel.DatasetTreeStore._trace,
	
	/*=====
	constructor_kwArgs: function(dataset, defaultProperties) {
		// summary:
		//		An anonymous object used to configure the datastore
		// dataset: Object
		//		A ibm.tivoli.navmodel.Dataset instance
		// defaultProperties: String[]
		//		An array of property id values that are pre-fetched with the items
		// relationships: String  (optional)
		//		Comma delimited list of relationships to determine related items; 
		//		can also be keywords: child, parent, all.  If not specified the value is
		//		retrieved from the individual item that is being expanded
		// isTreeGrid: boolean  (optional)
		//		Indicates that the store is being used by a TreeGrid component; and a 
		//		onRefresh should be used instead of onDelete (there are bugs in the TreeGrid) 
	},
	=====*/
	
	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructor for dojo datastore;
		//		that receives item data (in tree format) from an ibm.tivoli.navmodel.Dataset
		//		Base class will mixin kwArgs
		
		this._isTreeGrid = kwArgs.isTreeGrid;
		this._clearParents();
		this._clearChildren();		
	},
	
	
	//////////////////////////////////////////////////////////////
	// dojo.data.api.Identity
	//////////////////////////////////////////////////////////////
	
	getIdentity: function(/*Object*/ item){
		// summary:
		//		Returns a unique identifier for an item. The return value will be
		//		either a string or something that has a toString() method.
		// item:
		//		The item from the store from which to obtain its identifier.
		//		ibm.tivoli.navmodel.Item object
		// exceptions:
		//		Conforming implementations may throw an exception or return null if
		//		item is not an item.
		if (!this.isItem(item)){
  			throw new Error("Specified item argument is not a valid item.");
  		}
  		
  		var identity;
  		switch(item.getObjectType()) {
  			case "item":
  				identity = item.getId();
  				break;
  		}
  		  		
  		return identity;
    },
    
	/*=====
	_onItem: function(item) {},
	=====*/
	
	/*=====
	_onError: function(error) {},
	=====*/
    
    /*=====
	fetchItemByIdentity_kwArgs: function(identity, onItem, onError, scope) {
		// summary:
		//		An anonymous object that defines the item to locate and callbacks to invoke when the
		//		item has been located and load has completed.
		// identity: String|Object
		//		The identity parameter is the identity of the item you wish to locate and load
		//		This attribute is required. It should be a string or an object that toString()
		//		can be called on.
		// onItem: _onItem
		//		The onItem parameter is the callback to invoke when the item has been loaded.
		//		It takes only one parameter, the item located, or null if none found.
		// onError: _onError
		//		The onError parameter is the callback to invoke when the item load encountered an error.
		//		It takes only one parameter, the error object.
		// scope: Object
		//		If a scope object is provided, all of the callback functions (onItem, onError, etc)
		//		will be invoked in the context of the scope object.
	},
	=====*/
	
 	fetchItemByIdentity: function(/*fetchItemByIdentity_kwArgs*/ kwArgs) {
		// summary:
		//		Given the identity of an item, this method returns the item that has
		//		that identity through the onItem callback. Conforming implementations
		//		should return null if there is no item with the given identity.
		//		Implementations of fetchItemByIdentity() may sometimes return an item
		//		from a local cache and may sometimes fetch an item from a remote server.
 		var args = {
 			identity: null,
 			onItem: function(item) {},
 			onError: function(error) {},
 			scope: null
 		};
 		dojo.mixin(args, kwArgs);
 		
 		var item = null;
 		var error = null;
 		if (args.identity) {
 			// <itemId>
 			try {
				item = this.dataset.getItem({itemId: args.identity, properties: this.defaultProperties});
			}
			catch(error1) {
				error = error1;
			}
  		}
  		
  		if (item) {
 			args.onItem.call(args.scope, item);
 		}
 		else {
 			args.onError.call(args.scope, error ? error : "Could not fetch item for identity=" + args.identity);
 		}
	},
	
	
	//////////////////////////////////////////////////////////////
	// dojo.data.api.Read
	//////////////////////////////////////////////////////////////
	
	/*=====
	fetch_onBegin: function(size, request){},
	=====*/
	
	/*=====
	fetch_onItem: function(item, request){},
	=====*/
	
	/*=====
	fetch_onComplete: function(items, request){},
	=====*/
	
	/*=====
	fetch_onError: function(errorData, request){},
	=====*/
	
	/*=====
	fetch_kwArgs: function(query, queryOptions, onBegin, onItem, onComplete, onError, scope, start, count, sort) {
		// summary:
		//		The kwArgs parameter may either be an instance of conforming to 
		//		dojo.data.api.Request or may be a simple anonymous object
		//		containing any of the above parameters.
		// query: String|Object
		//		The query parameter is used to specify search and filter condition information for the fetch.
		//		Only those items that match the search and/or filter will be returned.
		//		|	{	ids: {
		//		|			propId: boolean		// indicates propIds involved in the expression
		//		|		},
		//		|		expression: string,		// expression involiving propId's and values  (and/or together)
		//		|								// example:  ((Name contains 'abc' || Type contains 'abc)) and (Size > 100)
		//		|
		//		|		// the following is for compatability with legacy datastore
		//		| 		navModel: {
		//		|			search: {
		//		|				term: string,		the search text
		//		|				attributes: array	["property id1", "property id2", ...]
		//		|			}
		//		|		}
		//		|	}
		// queryOptions: Object?
		//		The queryOptions parameter is an optional parameter that can be used to modify the query.  The
		//		ignoreCase case can be specified to globally make all of the filter conditions case sensitive/insensitive.
		//		|	{ ignoreCase: boolean
		//		|	}
		// onBegin: fetch_onBegin
		//		If an onBegin callback function is provided, the callback function will be called just once, 
		//		before the first onItem callback is called. The onBegin callback function will be passed two arguments, 
		//		the the total number of items identified and the Request object.  If the total number is
		//		unknown, then size will be -1.  Note that size is not necessarily the size of the collection of items 
		//		returned from the query, as the request may have specified to return only a subset of the total set 
		//		of items through the use of the start and count parameters.
		// onItem: fetch_onItem
		//		If an onItem callback function is provided, the callback function will be called as each item 
		//		in the result is received. The callback function will be passed two arguments: the item itself, 
		//		and the Request object.
		// onComplete: fetch_onComplete
		//		If an onComplete callback function is provided, the callback function will be called just once,
		//		after the last onItem callback is called. Note that if the onItem callback is not present,
		//		then onComplete will be passed an array containing all items which matched the query and the 
		//		request object. If the onItem callback is present, then onComplete is called as: onComplete(null, request).
		// onError: fetch_onError
		//		If an onError callback function is provided, the callback function will be called if there is any 
		//		sort of error while attempting to execute the query. The onError callback function will be passed two arguments:
		//		an Error object and the Request object.
		// scope: Object
		//		If a scope object is provided, all of the callback functions (onItem, onComplete, onError, etc) 
		//		will be invoked in the context of the scope object.
		// start: int
		//		If a start parameter is specified, this is a indication to the datastore to
		//		only start returning items once the start number of items have been located and
		//		skipped.  When this parameter is paired with 'count', the store should be able
		//		to page across queries with millions of hits by only returning subsets of the
		//		hits for each query
		// count: int
		//		 If a count parameter is specified, this is a indication to the datastore to
		//		 only return up to that many items.  This allows a fetch call that may have
		//		millions of item matches to be paired down to something reasonable.
		// sort: Object[]
		//		If a sort parameter is specified, this is a indication to the datastore to
		//		sort the items in some manner before returning the items.  The array is an array of
		//		javascript objects that must conform to the following format to be applied to the
		//		fetching of items:
		//		|	{
		//		|		attribute: attribute || attribute-name-string,
		//		|		descending: true|false;   // Optional.  Default is false.
		//		|	}
	},
	=====*/
		
	fetch: function(/*fetch_kwArgs*/ kwArgs) {
		// summary:
		//		This datastore exposes the tree origin nodes of the Dataset and allows for
		//		lazy loading of children afterwards.
		//		The format and expectations of stores is that they operate in a generally asynchronous
		//		manner, therefore callbacks are always used to return items located by the fetch parameters.
		// description:
		//		A Request object will always be returned and is returned immediately.
		//		The basic request is nothing more than the keyword args passed to fetch and
		//		an additional function attached, abort().  The returned request object may then be used
		//		to cancel a fetch.  All data items returns are passed through the callbacks defined in the
		//		fetch parameters and are not present on the 'request' object.
		// returns:
		//		The fetch() method will return a javascript object conforming to the API
		//		defined in dojo.data.api.Request.  In general, it will be the keywordArgs
		//		object returned with the required functions in Request.js attached.
		//		Its general purpose is to provide a convenient way for a caller to abort an
		//		ongoing fetch.
		// exceptions:
		//		Throws an exception if the query is not valid, or if the query is required but was not supplied.
		var args = {
			onBegin: function(size, request) {},
			onItem: function(item, request) {},
			onComplete: function(items, request){},
			onError: function(errorData, request){},
			scope: null,
			abort: function() {}
		};
		dojo.mixin(args, kwArgs);
		
		if (this._trace.isFinest()) {
			this._trace.finest("dataset=" + this.dataset.getId() + " fetching origin node(s)", "fetch");
		}
		
		// make sure event handling is set up
		this._validateEventHandler();
		
		// TED refresh is occurring
		this._clearCache = false;
		if (this.isRefresh()) {
			this._clearParents();
			this._clearChildren();
			this._clearCache = this.dataset.isClearCacheOnRefresh();						
		}
		
		// fetching "origin" items
		if (args.query && args.query.origins) {
			this._clearChildren();			
			this._fetchOrigins(args);
		}		
		// fetch all items
		else {
			this._fetchItems(args);
		}
		       
        return args;
    },
    
    getAttributes: function(/*Object*/ item) {
		// summary:
		//		Returns an array with all the attributes that this item has.  This
		//		method will always return an array; if the item has no attributes
		//		at all, getAttributes() will return an empty array: [].
		// item:
		//		The item to access attributes on.
		//		ibm.tivoli.navmodel.Item object
		// exceptions:
		//		Throws an exception if *item* is not an item
		var attributes = this.inherited(arguments);
  		
		// add in "children" attribute if appropriate
		if (this._hasChildren(item)) {
			attributes.push("children");
		}
		
		return attributes;		
    },
    
    getValue: function(/*Object*/ item, /*String*/ attribute, /*Object?*/ defaultValue) {
		// summary:
		//		Returns a single attribute value. Returns defaultValue if and only if item does not have a 
		//		value for attribute. Returns null if and only if null was explicitly set as the attribute value. 
		//		Returns undefined if and only if the item does not have a value for the given attribute 
		//		(which is the same as saying the item does not have the attribute).
		// item:
		//		The item to access value on.
		//		ibm.tivoli.navmodel.Item object
		// attribute:
		//		The attribute to access represented as a string.
		// defaultValue:
		//		Optional. A default value to use for the getValue return if the attribute does not exist or 
		//		has no value.
		// exceptions:
		//		Throws an exception if *item* is not an item, or *attribute* is not a string
    	if (arguments.length < 2) {
            throw new Error("Incorrect number of arguments specified: expecting 2 (item, attribute)");
        }
        
        if (!this.isItem(item)) {
            throw new Error("Specified item argument is not a valid item.");
        }
        
        if (!dojo.isString(attribute)) {
            throw new Error("Specified attribute argument is not a string.");
        }  
               
        var value = defaultValue;
        
        // general attribute value
    	if (this.returnItem) {
    		return item;
    	}
    
    	var prop = item.getProperty(attribute);
    	if (prop) {
    		value = prop.getDisplayValue();            
    	}	         
                           	
        return value;
    },
    
    getValues: function(/*Object*/ item, /*String*/ attribute) {
		// summary:
		//		This getValues() method works just like the getValue() method, but getValues()
		//		always returns an array rather than a single attribute value.  The array
		//		may be empty, may contain a single attribute value, or may contain many
		//		attribute values.
		//		
		//		If the item does not have a value for the given attribute, then getValues()
		//		will return an empty array: [].  (So, if store.hasAttribute(item, attribute)
		//		returns false, then store.getValues(item, attribute) will return [].)
		// item:
		//		The item to access values on.
		//		ibm.tivoli.navmodel.Item object
		// attribute:
		//		The attribute to access represented as a string.
		// exceptions:
		//		Throws an exception if *item* is not an item, or *attribute* is not a string
    	if (arguments.length < 2) {
            throw new Error("Incorrect number of arguments specified: expecting 2 (item, attribute)");
        }
        
        if (!this.isItem(item)) {
            throw new Error("Specified item argument is not a valid item.");
        }
        
        if (!dojo.isString(attribute)) {
            throw new Error("Specified attribute argument is not a string.");
        }
        
        var values = null;
        
        // asking for the "children" of this item
        if (attribute === "children") {
        	values = this._fetchChildren(item);
        }
        // general attribute value
    	else {
    		values = [this.getValue(item, attribute)];
    	}
    	
    	return values;
    },
    
    
    /*=====
	loadItem_kwArgs: function(item, onItem, onError, scope) {
		// summary:
		//		An anonymous object that defines the item to load and callbacks to invoke when the
		//		load has completed.
		// item: Object
		//		The item parameter is an object that represents the item in question that should be
		//		contained by the store.  This attribute is required.
		//		ibm.tivoli.navmodel.Item object
		// onItem: _onItem
		//		The onItem parameter is the callback to invoke when the item has been loaded.	
		//		It takes only one parameter, the fully loaded item.
		// onError: _onError
		//		The onError parameter is the callback to invoke when the item load encountered an error.
		//		It takes only one parameter, the error object.
		// scope: Object
		//		If a scope object is provided, all of the callback functions (onItem, onError, etc)
		//		will be invoked in the context of the scope object.
		//		onItem.call(dojo.global(), item, request)
	},
	=====*/
	
	loadItem: function(/*loadItem_kwArgs*/ kwArgs) {
		var args = {
			item : null,
			onItem: function(item) {},
			onError: function(error){},
			scope: null
		};		
		dojo.advancedMixin(args, kwArgs);
	
		if (!this.isItem(args.item)){
  			throw new Error("Specified item argument is not a valid item.");
  		}
  		
		if (this.isItemLoaded(args.item)){
			return args.item;
		}
		
		try {
			var item = null;
			var dataset = args.item.getDataset();
			switch(item.getObjectType()) {
				case "item":					
					item = dataset.getItem({itemId: args.item.getId(), properties: this.defaultProperties});
					break;
			}	
			args.onItem.call(args.scope, item);
		}
		catch(err) {
			args.onError.call(args.scope, err);
		}
	},
	
	//////////////////////////////////////////////////////////////
	// private methods
	///////////////////////////////////////////////////////////////
	
	_getObjectTypes: function() {
		// summary:
		//		Returns String[] of object types that can in the datastore
    	return {
    		"item": true
		};
    },
    
    _fetchOrigins: function(/*fetch_kwArgs*/ args) {
		// summary:
		//		Fetch origin items for the tree
		
		// we do have some possible origins
    	if (this.dataset.getItemCount() > 0) {
    		this._sort = this._getSort(args.sort);
				
			var originArgs = {
				clearCache: this._clearCache,
				properties: this.defaultProperties,
				sort: this._sort,
				onComplete: dojo.hitch(this, function(originItems, originArgs) {
					// we have origin items		
					var origins = [];
					if (originItems && originItems.length > 0) {
						origins = this._concat(origins, originItems, "root");
					
						var originsLength = origins.length;
						args.onBegin.call(args.scope, originsLength, args);				
						for(var originIndex = 0;  originIndex < originsLength;  ++originIndex) {                 	                	
                			args.onItem.call(args.scope, origins[originIndex], args);                	
						}					
						args.onComplete.call(args.scope, origins, args);					
					}
					// no origins declared
					else {
						this._finishFetchWhenNoOrigins(args);
					}
				}),			
				onError: dojo.hitch(this, function(error, exception, ioArgs) {
					args.onError.call(args.scope, error, args);
				})
			}; 		
			this.dataset.getOriginItems(originArgs);
		}
		// no items; so no origins
		else {
			this._finishFetchWhenNoOrigins(args);
		}
    },    
    
    _finishFetchWhenNoOrigins: function(/*fetch_kwArgs*/ args) {
		// summary:
		//		Fetch origin items for the tree when there are no items marked as origins		
    	var origins = [];    			
    	
    	// has items and possible child datasets
    	if (this.dataset.getItemCount() > 0) {
    		var itemsArgs = {
    			properties: this.defaultProperties,
    			onComplete: dojo.hitch(this, function(items, itemsArgs) {
    				if (items && items.length > 0) {
    					origins = this._concat(origins, items, "root");
    				}
    		
					var originsLength = origins.length;
					args.onBegin.call(args.scope, originsLength, args);				
					for(var originIndex = 0;  originIndex < originsLength;  ++originIndex) {                 	                	
                		args.onItem.call(args.scope, origins[originIndex], args);                	
					}					
					args.onComplete.call(args.scope, origins, args);
				}),			
				onError: dojo.hitch(this, function(error, exception, ioArgs) {
					args.onError.call(args.scope, error, args);
				})
    		};
    		this.dataset.getItems(itemsArgs);
    	}
    	// does NOT have items; check child datasets
    	else {
			var originsLength = origins.length;
			args.onBegin.call(args.scope, originsLength, args);				
			for(var originIndex = 0;  originIndex < originsLength;  ++originIndex) {                 	                	
                args.onItem.call(args.scope, origins[originIndex], args);                	
			}					
			args.onComplete.call(args.scope, origins, args);
    	}
    },
    
    _fetchItems: function(/*fetch_kwArgs*/ args) {
		// summary:
		//		Fetch all items for the dataset if the query did not indicate that origins should be returned
		
    	// we do have some possible origins
    	this._sort = this._getSort(args.sort);
				
		var itemArgs = {
			clearCache: this._clearCache,
			properties: this.defaultProperties,
			sort: this._sort,
			onComplete: dojo.hitch(this, function(items, itemArgs) {
				args.onBegin.call(args.scope, itemArgs.filteredRows, args);
								
				if (items) {
					var itemsLength = items.length;
					for(var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {                 	                	
                		args.onItem.call(args.scope, items[itemIndex], args);                	
					}
				}				
					
				args.onComplete.call(args.scope, origins, args);
			}),			
			onError: dojo.hitch(this, function(error, exception, ioArgs) {
				args.onError.call(args.scope, error, args);
			})	
		}; 		
		this.dataset.getItems(itemArgs);
    },
    
	_hasChildren: function(/*Object*/ item) {
		// summary:
		//		Indicates if the item has children items
    	var hasChildren = false;    	
    	switch(item.getObjectType()) {
    		case "item":
    			hasChildren = (item.isLazyLoad() || item.getLazyLoadRelationships());
    			break;
    	}
    	
    	return hasChildren;
    },
    
	_fetchChildren: function(/*Object*/ item) {
		// summary:
		//		Fetch children for item from the dataset
    	var children = [];
    	
		if (this._trace.isFinest()) {
			this._trace.finest("datasetId=" + this.dataset.getId() + " getting children for item=" + item.getId(), "_fetchChildren");
		}
						
    	if (this._hasChildren(item)) {
    		var cachedChildren = this._getChildren(item.getId());
    		if (cachedChildren) {
    			children = cachedChildren;
    			if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + this.dataset.getId() + " returning cached children for item=" + item.getId(), "_fetchChildren");
				}
    		}
    		else {
        		switch(item.getObjectType()) {
        			case "item":
						children = this._fetchChildrenForItem(item);
        				break;
				}
				
				// cache the children for this item
				this._setChildren(item.getId(), children);
			}
		}
					
		return children;		
    },
    		
    _fetchChildrenForItem: function(/*Object*/ item) {
		// summary:
		//		Fetch children for an "item" object
    	var children = [];  
    	var relatedArgs = {
        	onComplete: dojo.hitch(this, function(items, links, kwArgs) {
				if (items && items.length > 0) {
					children = this._concatSiblings(children, items, item);									
				}
			}),			
			onError: dojo.hitch(this, function(error, exception, ioArgs) {				
			}),
			relationships: this.relationships,
			properties: this.defaultProperties,
			sort: this._sort,
			sync: true
        };
        item.getRelatedItems(relatedArgs);
        			
    	return children;    	
    },
    
    _getParentId: function(/*String*/ id) {
		// summary:
		//		Get parentId for the requested child id
    	return this._parent[id];
    },
    
	_setParentId: function(/*String*/ id, /*String*/ parentId) {
		// summary:
		//		Set parentId for the requested child id
    	this._parent[id] = parentId;
    },
    
	_clearParents: function(/*String*/ id) {
		// summary:
		//		Clear parentId for the requested child id or all if not specified
    	if (id) {
    		delete this._parent[id];
    	}
    	else {
    		this._parent = {};
    	}
    },
    
	_getChildren: function(/*String*/ id) {
		// summary:
		//		Get cached children for parent id
    	return this._children[id];
    },
    
	_setChildren: function(/*String*/ id, /*String*/ children) {
		// summary:
		//		Set cached children for parent id
    	this._children[id] = children;
    },
    
	_clearChildren: function(/*String*/ id) {
		// summary:
		//		Clear cached children for parent id or all if not specified
    	if (id) {
    		delete this._children[id];
    	}
    	else {
    		this._children = {};
    	}
    },
      
    _concat: function(/*Object[]*/ list, /*Object[]*/ items, /*String*/ parentId) {
		var itemsLength = items.length;
    	for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
    		var item = items[itemIndex];
    		var prevParentId = this._getParentId(item.getId());    			
    		if (!prevParentId || prevParentId === parentId) {
				this._setParentId(item.getId(), parentId);
				list.push(item);
					
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + this.dataset.getId() + " parent=" + parentId + " child=" + item.getId(), "_concat");
				}
			}				
    	}
    	
    	return list;
    },
    
    _concatItem: function(/*Object[]*/ list, /*Object*/ item, /*String*/ parentId) {
    	var prevParentId = this._getParentId(item.getId());
    	if (!prevParentId || prevParentId === parentId) {
			this._setParentId(item.getId(), parentId);
			list.push(item);
				
			if (this._trace.isFinest()) {
				this._trace.finest("datasetId=" + this.dataset.getId() + " parent=" + parentId + " child=" + item.getId(), "_concatItem");
			}
		}				    	
    	
    	return list;    	
    },
    
    _concatSiblings: function(/*Object[]*/ list, /*Object[]*/ items, /*Object*/ siblingItem) {
    	var parentId = siblingItem.getId();
    	var itemsLength = items.length;
    	for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
    		var item = items[itemIndex];
    		if (item.getDataset() === siblingItem.getDataset()) {    			
    			var prevParentId = this._getParentId(item.getId());    		
    			if (!prevParentId || prevParentId === parentId) {
					this._setParentId(item.getId(), parentId);
					list.push(item);
					
					if (this._trace.isFinest()) {
						this._trace.finest("datasetId=" + this.dataset.getId() + " parent=" + siblingItem.getId() + " child=" + item.getId(), "_concatSiblings");
					}
				}					
    		}    			    		
    	}
    	
    	return list;
    },
    
    _handleChangedEvent: function(/*Object*/ event) {
    	this._clearParents();
    	this._clearChildren();
    	this.onRefresh();
	},
	
    _handleLabelChangedEvent: function(/*Object*/ event) {
    	if (this._trace.isFinest()) {
			this._trace.finest("datasetId=" + event.getDataset().getId() + " label=" + event.getNewLabel(), "_handleLabelChangedEvent");
		}		
    },
	
	_handleItemPropertiesChangedEvent: function(/*Object*/ event) {
		var items = event.getItems();    	
    	var propsArray = event.getProperties();			
		if (items && propsArray) {
    		var length = items.length;			
			for (var index = 0;  index < length;  ++index) {
				var properties = propsArray[index];				
				if (properties) {
					var propsLength = properties.length;
					for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
						var item = items[index];
						var property = properties[propIndex];						
						if (this._trace.isFinest()) {
							this._trace.finest("datasetId=" + event.getDataset().getId() + " itemId=" + item.getId() + " prop=" + property.getId() + " value=" + property.getDisplayValue(), "_handleItemPropertiesChangedEvent");
						}
						
						this.onSet(item, property.getId(), null, property.getDisplayValue());
					}										
				}				
			}	
		}	
	},
	
    _handleItemsAddedEvent: function(/*Object*/ event) {
    	var dataset = event.getDataset();
    	var isDescendent = dataset.getId() != this.dataset.getId();
		var items = event.getItems();
		if (items) {
    		var length = items.length;			
			for (var index = 0;  index < length;  ++index) {
				var item = items[index];				
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + event.getDataset().getId() + " itemId=" + item.getId() + " origin=" + item.isOrigin(), "_handleItemsAddedEvent");
				}
				
				if (item.isOrigin()) {
					// new origin for main dataset
					this.onNew(item);
				}
			}
		}    		
	},
	
	_handleItemsRemovedEvent: function(/*Object*/ event) {
		if (this._isTreeGrid) {
			this._trace.finest("datasetId=" + event.getDataset().getId() + " issuing onRefresh because isTreeGrid=true", "_handleItemsRemovedEvent");			
			this.onRefresh();
			return;
		}
			
		var items = event.getItems();
		if (items) {
			var length = items.length;			
			for (var index = 0;  index < length;  ++index) {
				var item = items[index];				
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + event.getDataset().getId() + " itemId=" + item.getId(), "_handleItemsRemovedEvent");
				}
				
				this._clearChildren(this._getParentId(item.getId()));
				this._clearParents(item.getId());
				this.onDelete(item);
			}			
		}
	},
	
	_handleLinksAddedEvent:function(/*Object*/ event) {
		var links = event.getLinks();
		if (links) {
			var linksLength = links.length;
			for (var linkIndex = 0;  linkIndex < linksLength;  ++linkIndex) {
				var link = links[linkIndex];
				var item1 = link.getItem1();
				var item2 = link.getItem2();
				var parent = null;
				var child = null;
					
				// we have a tree			
				if (this.dataset.getViewType() === "tree") {
					switch(link.getDirection()) {				
						case "1_2":
							parent = {item: item1};
							if (!item1.getLazyLoadRelationships()) {
								item1.setLazyLoadRelationships(this.relationships ? this.relationships : "child");
							}
							child = item2;
							break;
						case "2_1":
							parent = {item: item2};
							if (!item2.getLazyLoadRelationships()) {
								item2.setLazyLoadRelationships(this.relationships ? this.relationships : "parent");
							}
							child = item1;
					}					
				}
				// not necessarily a tree; see if item1 is in the tree and item2 is not
				else if (this._getParentId(item1.getId()) && !this._getParentId(item2.getId())) {
					parent = {item: item1};
					if (!item1.getLazyLoadRelationships()) {
						item1.setLazyLoadRelationships(this.relationships ? this.relationships : "all");
					}						
					child = item2;
				}
				// not necessarily a tree; see if item2 is in the tree and item1 is not
				else if (this._getParentId(item2.getId()) && !this._getParentId(item1.getId())) {
					parent = {item: item2};
					if (!item2.getLazyLoadRelationships) {
						item2.setLazyLoadRelationships(this.relationships ? this.relationships : "all");
					}							
					child = item1;
				}
				
				if (parent && child) {
					if (this._trace.isFinest()) {
						this._trace.finest("datasetId=" + event.getDataset().getId() + " parent=" + parent.item.getId() + " child=" + child.getId(), "_handleLinksAddedEvent");
					}	
					this._clearChildren(parent.item.getId());
					this.onNew(child, parent);
				}
				else if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + event.getDataset().getId() + " ignoring linkId=" + link.getId(), "_handleLinksAddedEvent");
				}	
			}			
		}
	}
});
});

},
'ibm/tivoli/tip/dijit/TIPMessageDialog':function(){
define("ibm/tivoli/tip/dijit/TIPMessageDialog", ["dojo", "dijit", "dojoe/messagedialog/MessageDialog"], function(dojo, dijit) {

return dojoe.messagedialog.MessageDialog;

});

},
'dojox/data/AndOrReadStore':function(){
define(["dojo/_base/kernel", "dojo/_base/declare", "dojo/_base/lang", "dojo/data/util/filter", "dojo/data/util/simpleFetch",
		"dojo/_base/array", "dojo/date/stamp", "dojo/_base/json", "dojo/_base/window", "dojo/_base/xhr"], 
  function(kernel, declare, lang, filterUtil, simpleFetch, array, dateStamp, json, winUtil, xhr) {

var AndOrReadStore = declare("dojox.data.AndOrReadStore", null, {
	//	summary:
	//		AndOrReadStore uses ItemFileReadStore as a base, modifying only the query (_fetchItems) section.
	//		Supports queries of the form: query:"id:1* OR dept:'Sales Department' || (id:2* && NOT dept:S*)"
	//		Includes legacy/widget support via:
	//			query:{complexQuery:"id:1* OR dept:'Sales Department' || (id:2* && NOT dept:S*)"}
	//		The ItemFileReadStore implements the dojo.data.api.Read API and reads
	//		data from JSON files that have contents in this format --
	//		{ items: [
	//			{ name:'Kermit', color:'green', age:12, friends:['Gonzo', {_reference:{name:'Fozzie Bear'}}]},
	//			{ name:'Fozzie Bear', wears:['hat', 'tie']},
	//			{ name:'Miss Piggy', pets:'Foo-Foo'}
	//		]}
	//		Note that it can also contain an 'identifer' property that specified which attribute on the items
	//		in the array of items that acts as the unique identifier for that item.
	//
	constructor: function(/* Object */ keywordParameters){
		//	summary: constructor
		//	keywordParameters: {url: String}
		//	keywordParameters: {data: jsonObject}
		//	keywordParameters: {typeMap: object)
		//		The structure of the typeMap object is as follows:
		//		{
		//			type0: function || object,
		//			type1: function || object,
		//			...
		//			typeN: function || object
		//		}
		//		Where if it is a function, it is assumed to be an object constructor that takes the
		//		value of _value as the initialization parameters.  If it is an object, then it is assumed
		//		to be an object of general form:
		//		{
		//			type: function, //constructor.
		//			deserialize:	function(value) //The function that parses the value and constructs the object defined by type appropriately.
		//		}
	
		this._arrayOfAllItems = [];
		this._arrayOfTopLevelItems = [];
		this._loadFinished = false;
		this._jsonFileUrl = keywordParameters.url;
		this._ccUrl = keywordParameters.url;
		this.url = keywordParameters.url;
		this._jsonData = keywordParameters.data;
		this.data = null;
		this._datatypeMap = keywordParameters.typeMap || {};
		if(!this._datatypeMap['Date']){
			//If no default mapping for dates, then set this as default.
			//We use the dojo.date.stamp here because the ISO format is the 'dojo way'
			//of generically representing dates.
			this._datatypeMap['Date'] = {
											type: Date,
											deserialize: function(value){
												return dateStamp.fromISOString(value);
											}
										};
		}
		this._features = {'dojo.data.api.Read':true, 'dojo.data.api.Identity':true};
		this._itemsByIdentity = null;
		this._storeRefPropName = "_S"; // Default name for the store reference to attach to every item.
		this._itemNumPropName = "_0"; // Default Item Id for isItem to attach to every item.
		this._rootItemPropName = "_RI"; // Default Item Id for isItem to attach to every item.
		this._reverseRefMap = "_RRM"; // Default attribute for constructing a reverse reference map for use with reference integrity
		this._loadInProgress = false; //Got to track the initial load to prevent duelling loads of the dataset.
		this._queuedFetches = [];

		if(keywordParameters.urlPreventCache !== undefined){
			this.urlPreventCache = keywordParameters.urlPreventCache?true:false;
		}
		if(keywordParameters.hierarchical !== undefined){
			this.hierarchical = keywordParameters.hierarchical?true:false;
		}
		if(keywordParameters.clearOnClose){
			this.clearOnClose = true;
		}
	},
	
	url: "", // use "" rather than undefined for the benefit of the parser (#3539)

	//Internal var, crossCheckUrl.  Used so that setting either url or _jsonFileUrl, can still trigger a reload
	//when clearOnClose and close is used.
	_ccUrl: "",

	data: null, //Make this parser settable.

	typeMap: null, //Make this parser settable.

	//Parameter to allow users to specify if a close call should force a reload or not.
	//By default, it retains the old behavior of not clearing if close is called.  But
	//if set true, the store will be reset to default state.  Note that by doing this,
	//all item handles will become invalid and a new fetch must be issued.
	clearOnClose: false,

	//Parameter to allow specifying if preventCache should be passed to the xhrGet call or not when loading data from a url.
	//Note this does not mean the store calls the server on each fetch, only that the data load has preventCache set as an option.
	//Added for tracker: #6072
	urlPreventCache: false,

	//Parameter to indicate to process data from the url as hierarchical
	//(data items can contain other data items in js form).  Default is true
	//for backwards compatibility.  False means only root items are processed
	//as items, all child objects outside of type-mapped objects and those in
	//specific reference format, are left straight JS data objects.
	hierarchical: true,

	_assertIsItem: function(/* item */ item){
		//	summary:
		//		This function tests whether the item passed in is indeed an item in the store.
		//	item:
		//		The item to test for being contained by the store.
		if(!this.isItem(item)){
			throw new Error("dojox.data.AndOrReadStore: Invalid item argument.");
		}
	},

	_assertIsAttribute: function(/* attribute-name-string */ attribute){
		//	summary:
		//		This function tests whether the item passed in is indeed a valid 'attribute' like type for the store.
		//	attribute:
		//		The attribute to test for being contained by the store.
		if(typeof attribute !== "string"){
			throw new Error("dojox.data.AndOrReadStore: Invalid attribute argument.");
		}
	},

	getValue: function(	/* item */ item,
						/* attribute-name-string */ attribute,
						/* value? */ defaultValue){
		//	summary:
		//		See dojo.data.api.Read.getValue()
		var values = this.getValues(item, attribute);
		return (values.length > 0)?values[0]:defaultValue; // mixed
	},

	getValues: function(/* item */ item,
						/* attribute-name-string */ attribute){
		//	summary:
		//		See dojo.data.api.Read.getValues()

		this._assertIsItem(item);
		this._assertIsAttribute(attribute);
		var arr = item[attribute] || [];
		// Clone it before returning.  refs: #10474
		return arr.slice(0, arr.length); // Array
	},

	getAttributes: function(/* item */ item){
		//	summary:
		//		See dojo.data.api.Read.getAttributes()
		this._assertIsItem(item);
		var attributes = [];
		for(var key in item){
			// Save off only the real item attributes, not the special id marks for O(1) isItem.
			if((key !== this._storeRefPropName) && (key !== this._itemNumPropName) && (key !== this._rootItemPropName) && (key !== this._reverseRefMap)){
				attributes.push(key);
			}
		}
		return attributes; // Array
	},

	hasAttribute: function(	/* item */ item,
							/* attribute-name-string */ attribute){
		//	summary:
		//		See dojo.data.api.Read.hasAttribute()
		this._assertIsItem(item);
		this._assertIsAttribute(attribute);
		return (attribute in item);
	},

	containsValue: function(/* item */ item,
							/* attribute-name-string */ attribute,
							/* anything */ value){
		//	summary:
		//		See dojo.data.api.Read.containsValue()
		var regexp = undefined;
		if(typeof value === "string"){
			regexp = filterUtil.patternToRegExp(value, false);
		}
		return this._containsValue(item, attribute, value, regexp); //boolean.
	},

	_containsValue: function(	/* item */ item,
								/* attribute-name-string */ attribute,
								/* anything */ value,
								/* RegExp?*/ regexp){
		//	summary:
		//		Internal function for looking at the values contained by the item.
		//	description:
		//		Internal function for looking at the values contained by the item.  This
		//		function allows for denoting if the comparison should be case sensitive for
		//		strings or not (for handling filtering cases where string case should not matter)
		//
		//	item:
		//		The data item to examine for attribute values.
		//	attribute:
		//		The attribute to inspect.
		//	value:
		//		The value to match.
		//	regexp:
		//		Optional regular expression generated off value if value was of string type to handle wildcarding.
		//		If present and attribute values are string, then it can be used for comparison instead of 'value'
		return array.some(this.getValues(item, attribute), function(possibleValue){
			if(possibleValue !== null && !lang.isObject(possibleValue) && regexp){
				if(possibleValue.toString().match(regexp)){
					return true; // Boolean
				}
			} else if(value === possibleValue){
				return true; // Boolean
			} else {
				return false;
			}
		});
	},

	isItem: function(/* anything */ something){
		//	summary:
		//		See dojo.data.api.Read.isItem()
		if(something && something[this._storeRefPropName] === this){
			if(this._arrayOfAllItems[something[this._itemNumPropName]] === something){
				return true;
			}
		}
		return false; // Boolean
	},

	isItemLoaded: function(/* anything */ something){
		//	summary:
		//		See dojo.data.api.Read.isItemLoaded()
		return this.isItem(something); //boolean
	},

	loadItem: function(/* object */ keywordArgs){
		//	summary:
		//		See dojo.data.api.Read.loadItem()
		this._assertIsItem(keywordArgs.item);
	},

	getFeatures: function(){
		//	summary:
		//		See dojo.data.api.Read.getFeatures()
		return this._features; //Object
	},

	getLabel: function(/* item */ item){
		//	summary:
		//		See dojo.data.api.Read.getLabel()
		if(this._labelAttr && this.isItem(item)){
			return this.getValue(item,this._labelAttr); //String
		}
		return undefined; //undefined
	},

	getLabelAttributes: function(/* item */ item){
		//	summary:
		//		See dojo.data.api.Read.getLabelAttributes()
		if(this._labelAttr){
			return [this._labelAttr]; //array
		}
		return null; //null
	},

	_fetchItems: function(	/* Object */ keywordArgs,
							/* Function */ findCallback,
							/* Function */ errorCallback){
		//	summary:
		//		See dojo.data.util.simpleFetch.fetch()
		//		filter modified to permit complex queries where
		//			logical operators are case insensitive:
		//			, NOT AND OR ( ) ! && ||
		//			Note:  "," included for quoted/string legacy queries.
		var self = this;
		var filter = function(requestArgs, arrayOfItems){
			var items = [];
			if(requestArgs.query){
				//Complete copy, we may have to mess with it.
				//Safer than clone, which does a shallow copy, I believe.
				var query = json.fromJson(json.toJson(requestArgs.query));
				//Okay, object form query, we have to check to see if someone mixed query methods (such as using FilteringSelect
				//with a complexQuery).  In that case, the params need to be anded to the complex query statement.
				//See defect #7980
				if(typeof query == "object" ){
					var count = 0;
					var p;
					for(p in query){
						count++;
					}
					if(count > 1 && query.complexQuery){
						var cq = query.complexQuery;
						var wrapped = false;
						for(p in query){
							if(p !== "complexQuery"){
								//We should wrap this in () as it should and with the entire complex query
								//Not just part of it.
								if(!wrapped){
									cq = "( " + cq + " )";
									wrapped = true;
								}
								//Make sure strings are quoted when going into complexQuery merge.
								var v = requestArgs.query[p];
								if(lang.isString(v)){
									v = "'" + v + "'";
								}
								cq += " AND " + p + ":" + v;
								delete query[p];
								
							}
						}
						query.complexQuery = cq;
					}
				}

				var ignoreCase = requestArgs.queryOptions ? requestArgs.queryOptions.ignoreCase : false;
				//for complex queries only:  pattern = query[:|=]"NOT id:23* AND (type:'test*' OR dept:'bob') && !filed:true"
				//logical operators are case insensitive:  , NOT AND OR ( ) ! && ||  // "," included for quoted/string legacy queries.
				if(typeof query != "string"){
					query = json.toJson(query);
					query = query.replace(/\\\\/g,"\\"); //counter toJson expansion of backslashes, e.g., foo\\*bar test.
				}
				query = query.replace(/\\"/g,"\"");   //ditto, for embedded \" in lieu of " availability.
				var complexQuery = lang.trim(query.replace(/{|}/g,"")); //we can handle these, too.
				var pos2, i;
				if(complexQuery.match(/"? *complexQuery *"?:/)){ //case where widget required a json object, so use complexQuery:'the real query'
					complexQuery = lang.trim(complexQuery.replace(/"?\s*complexQuery\s*"?:/,""));
					var quotes = ["'",'"'];
					var pos1,colon;
					var flag = false;
					for(i = 0; i<quotes.length; i++){
						pos1 = complexQuery.indexOf(quotes[i]);
						pos2 = complexQuery.indexOf(quotes[i],1);
						colon = complexQuery.indexOf(":",1);
						if(pos1 === 0 && pos2 != -1 && colon < pos2){
							flag = true;
							break;
						} //first two sets of quotes don't occur before the first colon.
					}
					if(flag){	//dojo.toJson, and maybe user, adds surrounding quotes, which we need to remove.
						complexQuery = complexQuery.replace(/^\"|^\'|\"$|\'$/g,"");
					}
				} //end query="{complexQuery:'id:1* || dept:Sales'}" parsing (for when widget required json object query).
				var complexQuerySave = complexQuery;
				//valid logical operators.
				var begRegExp = /^,|^NOT |^AND |^OR |^\(|^\)|^!|^&&|^\|\|/i; //trailing space on some tokens on purpose.
				var sQuery = ""; //will be eval'ed for each i-th candidateItem, based on query components.
				var op = "";
				var val = "";
				var pos = -1;
				var err = false;
				var key = "";
				var value = "";
				var tok = "";
				pos2 = -1;
				for(i = 0; i < arrayOfItems.length; ++i){
					var match = true;
					var candidateItem = arrayOfItems[i];
					if(candidateItem === null){
						match = false;
					}else{
						//process entire string for this i-th candidateItem.
						complexQuery = complexQuerySave; //restore query for next candidateItem.
						sQuery = "";
						//work left to right, finding either key:value pair or logical operator at the beginning of the complexQuery string.
						//when found, concatenate to sQuery and remove from complexQuery and loop back.
						while(complexQuery.length > 0 && !err){
							op = complexQuery.match(begRegExp);
							
							//get/process/append one or two leading logical operators.
							while(op && !err){ //look for leading logical operators.
								complexQuery = lang.trim(complexQuery.replace(op[0],""));
								op = lang.trim(op[0]).toUpperCase();
								//convert some logical operators to their javascript equivalents for later eval.
								op = op == "NOT" ? "!" : op == "AND" || op == "," ? "&&" : op == "OR" ? "||" : op;
								op = " " + op + " ";
								sQuery += op;
								op = complexQuery.match(begRegExp);
							}//end op && !err
							
							//now get/process/append one key:value pair.
							if(complexQuery.length > 0){
								pos = complexQuery.indexOf(":");
								if(pos == -1){
									err = true;
									break;
								}else{
									key = lang.trim(complexQuery.substring(0,pos).replace(/\"|\'/g,""));
									complexQuery = lang.trim(complexQuery.substring(pos + 1));
									tok = complexQuery.match(/^\'|^\"/);	//quoted?
									if(tok){
										tok = tok[0];
										pos = complexQuery.indexOf(tok);
										pos2 = complexQuery.indexOf(tok,pos + 1);
										if(pos2 == -1){
											err = true;
											break;
										}
										value = complexQuery.substring(pos + 1,pos2);
										if(pos2 == complexQuery.length - 1){ //quote is last character
											complexQuery = "";
										}else{
											complexQuery = lang.trim(complexQuery.substring(pos2 + 1));
										}
										sQuery += self._containsValue(candidateItem, key, value, filterUtil.patternToRegExp(value, ignoreCase));
									}
									else{ //not quoted, so a space, comma, or closing parens (or the end) will be the break.
										tok = complexQuery.match(/\s|\)|,/);
										if(tok){
											var pos3 = new Array(tok.length);
											for(var j = 0;j<tok.length;j++){
												pos3[j] = complexQuery.indexOf(tok[j]);
											}
											pos = pos3[0];
											if(pos3.length > 1){
												for(var j=1;j<pos3.length;j++){
													pos = Math.min(pos,pos3[j]);
												}
											}
											value = lang.trim(complexQuery.substring(0,pos));
											complexQuery = lang.trim(complexQuery.substring(pos));
										}else{ //not a space, so must be at the end of the complexQuery.
											value = lang.trim(complexQuery);
											complexQuery = "";
										} //end  inner if(tok) else
										sQuery += self._containsValue(candidateItem, key, value, filterUtil.patternToRegExp(value, ignoreCase));
									} //end outer if(tok) else
								} //end found ":"
							} //end if(complexQuery.length > 0)
						} //end while complexQuery.length > 0 && !err, so finished the i-th item.
						match = eval(sQuery);
					} //end else is non-null candidateItem.
					if(match){
						items.push(candidateItem);
					}
				} //end for/next of all items.
				if(err){
					//soft fail.
					items = [];
					console.log("The store's _fetchItems failed, probably due to a syntax error in query.");
				}
				findCallback(items, requestArgs);
			}else{
				// No query...
				// We want a copy to pass back in case the parent wishes to sort the array.
				// We shouldn't allow resort of the internal list, so that multiple callers
				// can get lists and sort without affecting each other.  We also need to
				// filter out any null values that have been left as a result of deleteItem()
				// calls in ItemFileWriteStore.
				for(var i = 0; i < arrayOfItems.length; ++i){
					var item = arrayOfItems[i];
					if(item !== null){
						items.push(item);
					}
				}
				findCallback(items, requestArgs);
			} //end if there is a query.
		}; //end filter function

		if(this._loadFinished){
			filter(keywordArgs, this._getItemsArray(keywordArgs.queryOptions));
		}else{
			if(this._jsonFileUrl !== this._ccUrl){
				kernel.deprecated("dojox.data.AndOrReadStore: ",
								"To change the url, set the url property of the store," +
								" not _jsonFileUrl.  _jsonFileUrl support will be removed in 2.0");
				this._ccUrl = this._jsonFileUrl;
				this.url = this._jsonFileUrl;
			}else if(this.url !== this._ccUrl){
				this._jsonFileUrl = this.url;
				this._ccUrl = this.url;
			}
			//See if there was any forced reset of data.
			if(this.data != null && this._jsonData == null){
				this._jsonData = this.data;
				this.data = null;
			}
			if(this._jsonFileUrl){
				//If fetches come in before the loading has finished, but while
				//a load is in progress, we have to defer the fetching to be
				//invoked in the callback.
				if(this._loadInProgress){
					this._queuedFetches.push({args: keywordArgs, filter: filter});
				}else{
					this._loadInProgress = true;
					var getArgs = {
							url: self._jsonFileUrl,
							handleAs: "json-comment-optional",
							preventCache: this.urlPreventCache
						};
					var getHandler = xhr.get(getArgs);
					getHandler.addCallback(function(data){
						try{
							self._getItemsFromLoadedData(data);
							self._loadFinished = true;
							self._loadInProgress = false;
							
							filter(keywordArgs, self._getItemsArray(keywordArgs.queryOptions));
							self._handleQueuedFetches();
						}catch(e){
							self._loadFinished = true;
							self._loadInProgress = false;
							errorCallback(e, keywordArgs);
						}
					});
					getHandler.addErrback(function(error){
						self._loadInProgress = false;
						errorCallback(error, keywordArgs);
					});
					
					//Wire up the cancel to abort of the request
					//This call cancel on the deferred if it hasn't been called
					//yet and then will chain to the simple abort of the
					//simpleFetch keywordArgs
					var oldAbort = null;
					if(keywordArgs.abort){
						oldAbort = keywordArgs.abort;
					}
					keywordArgs.abort = function(){
						var df = getHandler;
						if(df && df.fired === -1){
							df.cancel();
							df = null;
						}
						if(oldAbort){
							oldAbort.call(keywordArgs);
						}
					};
				}
			}else if(this._jsonData){
				try{
					this._loadFinished = true;
					this._getItemsFromLoadedData(this._jsonData);
					this._jsonData = null;
					filter(keywordArgs, this._getItemsArray(keywordArgs.queryOptions));
				}catch(e){
					errorCallback(e, keywordArgs);
				}
			}else{
				errorCallback(new Error("dojox.data.AndOrReadStore: No JSON source data was provided as either URL or a nested Javascript object."), keywordArgs);
			}
		} //end deferred fetching.
	}, //end _fetchItems

	_handleQueuedFetches: function(){
		//	summary:
		//		Internal function to execute delayed request in the store.
		//Execute any deferred fetches now.
		if(this._queuedFetches.length > 0){
			for(var i = 0; i < this._queuedFetches.length; i++){
				var fData = this._queuedFetches[i];
				var delayedQuery = fData.args;
				var delayedFilter = fData.filter;
				if(delayedFilter){
					delayedFilter(delayedQuery, this._getItemsArray(delayedQuery.queryOptions));
				}else{
					this.fetchItemByIdentity(delayedQuery);
				}
			}
			this._queuedFetches = [];
		}
	},

	_getItemsArray: function(/*object?*/queryOptions){
		//	summary:
		//		Internal function to determine which list of items to search over.
		//	queryOptions: The query options parameter, if any.
		if(queryOptions && queryOptions.deep){
			return this._arrayOfAllItems;
		}
		return this._arrayOfTopLevelItems;
	},

	close: function(/*dojo.data.api.Request || keywordArgs || null */ request){
		//	summary:
		//		See dojo.data.api.Read.close()
		if(this.clearOnClose &&
			this._loadFinished &&
			!this._loadInProgress){
			 //Reset all internalsback to default state.  This will force a reload
			 //on next fetch.  This also checks that the data or url param was set
			 //so that the store knows it can get data.  Without one of those being set,
			 //the next fetch will trigger an error.

			 if(((this._jsonFileUrl == "" || this._jsonFileUrl == null) &&
				 (this.url == "" || this.url == null)
				) && this.data == null){
				 console.debug("dojox.data.AndOrReadStore: WARNING!  Data reload " +
					" information has not been provided." +
					"  Please set 'url' or 'data' to the appropriate value before" +
					" the next fetch");
			 }
			 this._arrayOfAllItems = [];
			 this._arrayOfTopLevelItems = [];
			 this._loadFinished = false;
			 this._itemsByIdentity = null;
			 this._loadInProgress = false;
			 this._queuedFetches = [];
		 }
	},

	_getItemsFromLoadedData: function(/* Object */ dataObject){
		//	summary:
		//		Function to parse the loaded data into item format and build the internal items array.
		//	description:
		//		Function to parse the loaded data into item format and build the internal items array.
		//
		//	dataObject:
		//		The JS data object containing the raw data to convery into item format.
		//
		// 	returns: array
		//		Array of items in store item format.
		
		// First, we define a couple little utility functions...
		
		var self = this;
		function valueIsAnItem(/* anything */ aValue){
			// summary:
			//		Given any sort of value that could be in the raw json data,
			//		return true if we should interpret the value as being an
			//		item itself, rather than a literal value or a reference.
			// example:
			// 	|	false == valueIsAnItem("Kermit");
			// 	|	false == valueIsAnItem(42);
			// 	|	false == valueIsAnItem(new Date());
			// 	|	false == valueIsAnItem({_type:'Date', _value:'May 14, 1802'});
			// 	|	false == valueIsAnItem({_reference:'Kermit'});
			// 	|	true == valueIsAnItem({name:'Kermit', color:'green'});
			// 	|	true == valueIsAnItem({iggy:'pop'});
			// 	|	true == valueIsAnItem({foo:42});
			var isItem = (
				(aValue !== null) &&
				(typeof aValue === "object") &&
				(!lang.isArray(aValue)) &&
				(!lang.isFunction(aValue)) &&
				(aValue.constructor == Object) &&
				(typeof aValue._reference === "undefined") &&
				(typeof aValue._type === "undefined") &&
				(typeof aValue._value === "undefined") &&
				self.hierarchical
			);
			return isItem;
		}
		
		function addItemAndSubItemsToArrayOfAllItems(/* Item */ anItem){
			self._arrayOfAllItems.push(anItem);
			for(var attribute in anItem){
				var valueForAttribute = anItem[attribute];
				if(valueForAttribute){
					if(lang.isArray(valueForAttribute)){
						var valueArray = valueForAttribute;
						for(var k = 0; k < valueArray.length; ++k){
							var singleValue = valueArray[k];
							if(valueIsAnItem(singleValue)){
								addItemAndSubItemsToArrayOfAllItems(singleValue);
							}
						}
					}else{
						if(valueIsAnItem(valueForAttribute)){
							addItemAndSubItemsToArrayOfAllItems(valueForAttribute);
						}
					}
				}
			}
		}

		this._labelAttr = dataObject.label;

		// We need to do some transformations to convert the data structure
		// that we read from the file into a format that will be convenient
		// to work with in memory.

		// Step 1: Walk through the object hierarchy and build a list of all items
		var i;
		var item;
		this._arrayOfAllItems = [];
		this._arrayOfTopLevelItems = dataObject.items;

		for(i = 0; i < this._arrayOfTopLevelItems.length; ++i){
			item = this._arrayOfTopLevelItems[i];
			addItemAndSubItemsToArrayOfAllItems(item);
			item[this._rootItemPropName]=true;
		}

		// Step 2: Walk through all the attribute values of all the items,
		// and replace single values with arrays.  For example, we change this:
		//		{ name:'Miss Piggy', pets:'Foo-Foo'}
		// into this:
		//		{ name:['Miss Piggy'], pets:['Foo-Foo']}
		//
		// We also store the attribute names so we can validate our store
		// reference and item id special properties for the O(1) isItem
		var allAttributeNames = {};
		var key;

		for(i = 0; i < this._arrayOfAllItems.length; ++i){
			item = this._arrayOfAllItems[i];
			for(key in item){
				if(key !== this._rootItemPropName){
					var value = item[key];
					if(value !== null){
						if(!lang.isArray(value)){
							item[key] = [value];
						}
					}else{
						item[key] = [null];
					}
				}
				allAttributeNames[key]=key;
			}
		}

		// Step 3: Build unique property names to use for the _storeRefPropName and _itemNumPropName
		// This should go really fast, it will generally never even run the loop.
		while(allAttributeNames[this._storeRefPropName]){
			this._storeRefPropName += "_";
		}
		while(allAttributeNames[this._itemNumPropName]){
			this._itemNumPropName += "_";
		}
		while(allAttributeNames[this._reverseRefMap]){
			this._reverseRefMap += "_";
		}

		// Step 4: Some data files specify an optional 'identifier', which is
		// the name of an attribute that holds the identity of each item.
		// If this data file specified an identifier attribute, then build a
		// hash table of items keyed by the identity of the items.
		var arrayOfValues;

		var identifier = dataObject.identifier;
		if(identifier){
			this._itemsByIdentity = {};
			this._features['dojo.data.api.Identity'] = identifier;
			for(i = 0; i < this._arrayOfAllItems.length; ++i){
				item = this._arrayOfAllItems[i];
				arrayOfValues = item[identifier];
				var identity = arrayOfValues[0];
				if(!this._itemsByIdentity[identity]){
					this._itemsByIdentity[identity] = item;
				}else{
					if(this._jsonFileUrl){
						throw new Error("dojox.data.AndOrReadStore:  The json data as specified by: [" + this._jsonFileUrl + "] is malformed.  Items within the list have identifier: [" + identifier + "].  Value collided: [" + identity + "]");
					}else if(this._jsonData){
						throw new Error("dojox.data.AndOrReadStore:  The json data provided by the creation arguments is malformed.  Items within the list have identifier: [" + identifier + "].  Value collided: [" + identity + "]");
					}
				}
			}
		}else{
			this._features['dojo.data.api.Identity'] = Number;
		}

		// Step 5: Walk through all the items, and set each item's properties
		// for _storeRefPropName and _itemNumPropName, so that store.isItem() will return true.
		for(i = 0; i < this._arrayOfAllItems.length; ++i){
			item = this._arrayOfAllItems[i];
			item[this._storeRefPropName] = this;
			item[this._itemNumPropName] = i;
		}

		// Step 6: We walk through all the attribute values of all the items,
		// looking for type/value literals and item-references.
		//
		// We replace item-references with pointers to items.  For example, we change:
		//		{ name:['Kermit'], friends:[{_reference:{name:'Miss Piggy'}}] }
		// into this:
		//		{ name:['Kermit'], friends:[miss_piggy] }
		// (where miss_piggy is the object representing the 'Miss Piggy' item).
		//
		// We replace type/value pairs with typed-literals.  For example, we change:
		//		{ name:['Nelson Mandela'], born:[{_type:'Date', _value:'July 18, 1918'}] }
		// into this:
		//		{ name:['Kermit'], born:(new Date('July 18, 1918')) }
		//
		// We also generate the associate map for all items for the O(1) isItem function.
		for(i = 0; i < this._arrayOfAllItems.length; ++i){
			item = this._arrayOfAllItems[i]; // example: { name:['Kermit'], friends:[{_reference:{name:'Miss Piggy'}}] }
			for(key in item){
				arrayOfValues = item[key]; // example: [{_reference:{name:'Miss Piggy'}}]
				for(var j = 0; j < arrayOfValues.length; ++j){
					value = arrayOfValues[j]; // example: {_reference:{name:'Miss Piggy'}}
					if(value !== null && typeof value == "object"){
						if(("_type" in value) && ("_value" in value)){
							var type = value._type; // examples: 'Date', 'Color', or 'ComplexNumber'
							var mappingObj = this._datatypeMap[type]; // examples: Date, dojo.Color, foo.math.ComplexNumber, {type: dojo.Color, deserialize(value){ return new dojo.Color(value)}}
							if(!mappingObj){
								throw new Error("dojox.data.AndOrReadStore: in the typeMap constructor arg, no object class was specified for the datatype '" + type + "'");
							}else if(lang.isFunction(mappingObj)){
								arrayOfValues[j] = new mappingObj(value._value);
							}else if(lang.isFunction(mappingObj.deserialize)){
								arrayOfValues[j] = mappingObj.deserialize(value._value);
							}else{
								throw new Error("dojox.data.AndOrReadStore: Value provided in typeMap was neither a constructor, nor a an object with a deserialize function");
							}
						}
						if(value._reference){
							var referenceDescription = value._reference; // example: {name:'Miss Piggy'}
							if(!lang.isObject(referenceDescription)){
								// example: 'Miss Piggy'
								// from an item like: { name:['Kermit'], friends:[{_reference:'Miss Piggy'}]}
								arrayOfValues[j] = this._getItemByIdentity(referenceDescription);
							}else{
								// example: {name:'Miss Piggy'}
								// from an item like: { name:['Kermit'], friends:[{_reference:{name:'Miss Piggy'}}] }
								for(var k = 0; k < this._arrayOfAllItems.length; ++k){
									var candidateItem = this._arrayOfAllItems[k];
									var found = true;
									for(var refKey in referenceDescription){
										if(candidateItem[refKey] != referenceDescription[refKey]){
											found = false;
										}
									}
									if(found){
										arrayOfValues[j] = candidateItem;
									}
								}
							}
							if(this.referenceIntegrity){
								var refItem = arrayOfValues[j];
								if(this.isItem(refItem)){
									this._addReferenceToMap(refItem, item, key);
								}
							}
						}else if(this.isItem(value)){
							//It's a child item (not one referenced through _reference).
							//We need to treat this as a referenced item, so it can be cleaned up
							//in a write store easily.
							if(this.referenceIntegrity){
								this._addReferenceToMap(value, item, key);
							}
						}
					}
				}
			}
		}
	},

	_addReferenceToMap: function(/*item*/ refItem, /*item*/ parentItem, /*string*/ attribute){
		 //	summary:
		 //		Method to add an reference map entry for an item and attribute.
		 //	description:
		 //		Method to add an reference map entry for an item and attribute. 		 //
		 //	refItem:
		 //		The item that is referenced.
		 //	parentItem:
		 //		The item that holds the new reference to refItem.
		 //	attribute:
		 //		The attribute on parentItem that contains the new reference.
		 
		 //Stub function, does nothing.  Real processing is in ItemFileWriteStore.
	},

	getIdentity: function(/* item */ item){
		//	summary:
		//		See dojo.data.api.Identity.getIdentity()
		var identifier = this._features['dojo.data.api.Identity'];
		if(identifier === Number){
			return item[this._itemNumPropName]; // Number
		}else{
			var arrayOfValues = item[identifier];
			if(arrayOfValues){
				return arrayOfValues[0]; // Object || String
			}
		}
		return null; // null
	},

	fetchItemByIdentity: function(/* Object */ keywordArgs){
		//	summary:
		//		See dojo.data.api.Identity.fetchItemByIdentity()

		// Hasn't loaded yet, we have to trigger the load.
		if(!this._loadFinished){
			var self = this;
			if(this._jsonFileUrl !== this._ccUrl){
				kernel.deprecated("dojox.data.AndOrReadStore: ",
								"To change the url, set the url property of the store," +
								" not _jsonFileUrl.  _jsonFileUrl support will be removed in 2.0");
				this._ccUrl = this._jsonFileUrl;
				this.url = this._jsonFileUrl;
			}else if(this.url !== this._ccUrl){
				this._jsonFileUrl = this.url;
				this._ccUrl = this.url;
			}
			//See if there was any forced reset of data.
			if(this.data != null && this._jsonData == null){
				this._jsonData = this.data;
				this.data = null;
			}
			if(this._jsonFileUrl){

				if(this._loadInProgress){
					this._queuedFetches.push({args: keywordArgs});
				}else{
					this._loadInProgress = true;
					var getArgs = {
							url: self._jsonFileUrl,
							handleAs: "json-comment-optional",
							preventCache: this.urlPreventCache
					};
					var getHandler = xhr.get(getArgs);
					getHandler.addCallback(function(data){
						var scope = keywordArgs.scope?keywordArgs.scope:winUtil.global;
						try{
							self._getItemsFromLoadedData(data);
							self._loadFinished = true;
							self._loadInProgress = false;
							var item = self._getItemByIdentity(keywordArgs.identity);
							if(keywordArgs.onItem){
								keywordArgs.onItem.call(scope, item);
							}
							self._handleQueuedFetches();
						}catch(error){
							self._loadInProgress = false;
							if(keywordArgs.onError){
								keywordArgs.onError.call(scope, error);
							}
						}
					});
					getHandler.addErrback(function(error){
						self._loadInProgress = false;
						if(keywordArgs.onError){
							var scope = keywordArgs.scope?keywordArgs.scope:winUtil.global;
							keywordArgs.onError.call(scope, error);
						}
					});
				}

			}else if(this._jsonData){
				// Passed in data, no need to xhr.
				self._getItemsFromLoadedData(self._jsonData);
				self._jsonData = null;
				self._loadFinished = true;
				var item = self._getItemByIdentity(keywordArgs.identity);
				if(keywordArgs.onItem){
					var scope = keywordArgs.scope?keywordArgs.scope:winUtil.global;
					keywordArgs.onItem.call(scope, item);
				}
			}
		}else{
			// Already loaded.  We can just look it up and call back.
			var item = this._getItemByIdentity(keywordArgs.identity);
			if(keywordArgs.onItem){
				var scope = keywordArgs.scope?keywordArgs.scope:winUtil.global;
				keywordArgs.onItem.call(scope, item);
			}
		}
	},

	_getItemByIdentity: function(/* Object */ identity){
		//	summary:
		//		Internal function to look an item up by its identity map.
		var item = null;
		if(this._itemsByIdentity){
			item = this._itemsByIdentity[identity];
		}else{
			item = this._arrayOfAllItems[identity];
		}
		if(item === undefined){
			item = null;
		}
		return item; // Object
	},

	getIdentityAttributes: function(/* item */ item){
		//	summary:
		//		See dojo.data.api.Identity.getIdentifierAttributes()
		 
		var identifier = this._features['dojo.data.api.Identity'];
		if(identifier === Number){
			// If (identifier === Number) it means getIdentity() just returns
			// an integer item-number for each item.  The dojo.data.api.Identity
			// spec says we need to return null if the identity is not composed
			// of attributes
			return null; // null
		}else{
			return [identifier]; // Array
		}
	},
	
	_forceLoad: function(){
		//	summary:
		//		Internal function to force a load of the store if it hasn't occurred yet.  This is required
		//		for specific functions to work properly.
		var self = this;
		if(this._jsonFileUrl !== this._ccUrl){
			kernel.deprecated("dojox.data.AndOrReadStore: ",
							"To change the url, set the url property of the store," +
							" not _jsonFileUrl.  _jsonFileUrl support will be removed in 2.0");
			this._ccUrl = this._jsonFileUrl;
			this.url = this._jsonFileUrl;
		}else if(this.url !== this._ccUrl){
			this._jsonFileUrl = this.url;
			this._ccUrl = this.url;
		}
		//See if there was any forced reset of data.
		if(this.data != null && this._jsonData == null){
			this._jsonData = this.data;
			this.data = null;
		}
		if(this._jsonFileUrl){
				var getArgs = {
					url: self._jsonFileUrl,
					handleAs: "json-comment-optional",
					preventCache: this.urlPreventCache,
					sync: true
				};
			var getHandler = xhr.get(getArgs);
			getHandler.addCallback(function(data){
				try{
					//Check to be sure there wasn't another load going on concurrently
					//So we don't clobber data that comes in on it.  If there is a load going on
					//then do not save this data.  It will potentially clobber current data.
					//We mainly wanted to sync/wait here.
					//TODO:  Revisit the loading scheme of this store to improve multi-initial
					//request handling.
					if(self._loadInProgress !== true && !self._loadFinished){
						self._getItemsFromLoadedData(data);
						self._loadFinished = true;
					}else if(self._loadInProgress){
						//Okay, we hit an error state we can't recover from.  A forced load occurred
						//while an async load was occurring.  Since we cannot block at this point, the best
						//that can be managed is to throw an error.
						throw new Error("dojox.data.AndOrReadStore:  Unable to perform a synchronous load, an async load is in progress.");
					}
				}catch(e){
					console.log(e);
					throw e;
				}
			});
			getHandler.addErrback(function(error){
				throw error;
			});
		}else if(this._jsonData){
			self._getItemsFromLoadedData(self._jsonData);
			self._jsonData = null;
			self._loadFinished = true;
		}
	}
});
//Mix in the simple fetch implementation to this class.
lang.extend(AndOrReadStore, simpleFetch);

return AndOrReadStore;
});



},
'dojox/collections/SortedList':function(){
define(["dojo/_base/kernel", "dojo/_base/array", "./_base"], function(dojo, darray, dxc){
/*=====
var dxc = dojox.collections;
=====*/
	dxc.SortedList=function(/* object? */ dictionary){
		//	summary
		//	creates a collection that acts like a dictionary but is also internally sorted.
		//	Note that the act of adding any elements forces an internal resort, making this object potentially slow.
		var _this=this;
		var items={};
		var q=[];
		var sorter=function(a,b){
			if (a.key > b.key) return 1;
			if (a.key < b.key) return -1;
			return 0;
		};
		var build=function(){
			q=[];
			var e=_this.getIterator();
			while (!e.atEnd()){
				q.push(e.get());
			}
			q.sort(sorter);
		};
		var testObject={};

		this.count=q.length;
		this.add=function(/* string */ k,/* object */v){
			//	summary
			//	add the passed value to the dictionary at location k
			if (!items[k]) {
				items[k]=new dxc.DictionaryEntry(k,v);
				this.count=q.push(items[k]);
				q.sort(sorter);
			}
		};
		this.clear=function(){
			//	summary
			//	clear the internal collections
			items={};
			q=[];
			this.count=q.length;
		};
		this.clone=function(){
			//	summary
			//	create a clone of this sorted list
			return new dxc.SortedList(this);	//	dojox.collections.SortedList
		};
		this.contains=this.containsKey=function(/* string */ k){
			//	summary
			//	Check to see if the list has a location k
			if(testObject[k]){
				return false;			//	bool
			}
			return (items[k]!=null);	//	bool
		};
		this.containsValue=function(/* object */ o){
			//	summary
			//	Check to see if this list contains the passed object
			var e=this.getIterator();
			while (!e.atEnd()){
				var item=e.get();
				if(item.value==o){
					return true;	//	bool
				}
			}
			return false;	//	bool
		};
		this.copyTo=function(/* array */ arr, /* int */ i){
			//	summary
			//	copy the contents of the list into array arr at index i
			var e=this.getIterator();
			var idx=i;
			while(!e.atEnd()){
				arr.splice(idx,0,e.get());
				idx++;
			}
		};
		this.entry=function(/* string */ k){
			//	summary
			//	return the object at location k
			return items[k];	//	dojox.collections.DictionaryEntry
		};
		this.forEach=function(/* function */ fn, /* object? */ scope){
			//	summary
			//	functional iterator, following the mozilla spec.
			dojo.forEach(q, fn, scope);
		};
		this.getByIndex=function(/* int */ i){
			//	summary
			//	return the item at index i
			return q[i].valueOf();	//	object
		};
		this.getIterator=function(){
			//	summary
			//	get an iterator for this object
			return new dxc.DictionaryIterator(items);	//	dojox.collections.DictionaryIterator
		};
		this.getKey=function(/* int */ i){
			//	summary
			//	return the key of the item at index i
			return q[i].key;
		};
		this.getKeyList=function(){
			//	summary
			//	return an array of the keys set in this list
			var arr=[];
			var e=this.getIterator();
			while (!e.atEnd()){
				arr.push(e.get().key);
			}
			return arr;	//	array
		};
		this.getValueList=function(){
			//	summary
			//	return an array of values in this list
			var arr=[];
			var e=this.getIterator();
			while (!e.atEnd()){
				arr.push(e.get().value);
			}
			return arr;	//	array
		};
		this.indexOfKey=function(/* string */ k){
			//	summary
			//	return the index of the passed key.
			for (var i=0; i<q.length; i++){
				if (q[i].key==k){
					return i;	//	int
				}
			}
			return -1;	//	int
		};
		this.indexOfValue=function(/* object */ o){
			//	summary
			//	return the first index of object o
			for (var i=0; i<q.length; i++){
				if (q[i].value==o){
					return i;	//	int
				}
			}
			return -1;	//	int
		};
		this.item=function(/* string */ k){
			// 	summary
			//	return the value of the object at location k.
			if(k in items && !testObject[k]){
				return items[k].valueOf();	//	object
			}
			return undefined;	//	object
		};
		this.remove=function(/* string */k){
			// 	summary
			//	remove the item at location k and rebuild the internal collections.
			delete items[k];
			build();
			this.count=q.length;
		};
		this.removeAt=function(/* int */ i){
			//	summary
			//	remove the item at index i, and rebuild the internal collections.
			delete items[q[i].key];
			build();
			this.count=q.length;
		};
		this.replace=function(/* string */ k, /* object */ v){
			//	summary
			//	Replace an existing item if it's there, and add a new one if not.
			if (!items[k]){
				//	we're adding a new object, return false
				this.add(k,v);
				return false; // bool
			}else{
				//	we're replacing an object, return true
				items[k]=new dxc.DictionaryEntry(k,v);
				build();
				return true; // bool
			}
		};
		this.setByIndex=function(/* int */ i, /* object */ o){
			//	summary
			//	set an item by index
			items[q[i].key].value=o;
			build();
			this.count=q.length;
		};
		if (dictionary){
			var e=dictionary.getIterator();
			while (!e.atEnd()){
				var item=e.get();
				q[q.length]=items[item.key]=new dxc.DictionaryEntry(item.key,item.value);
			}
			q.sort(sorter);
		}
	};
	return dxc.SortedList;
});

},
'ibm/usmi/statuspod/Constants':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/statuspod/Constants", {
	updateJobsLabel : "/ibm/usmi/statuspod/updateJobsLabel",
	STATUS_CLASS_ERROR: "status-msgs-icon-error",
	STATUS_CLASS_WARNING: "status-msgs-icon-warning",
	STATUS_CLASS_NORMAL: "status-msgs-icon-confirmation",
	STATUS_CLASS_UNKNOWN: "status-msgs-icon-unknown"
});

},
'dojo/text':function(){
define(["./_base/kernel", "require", "./has", "./_base/xhr"], function(dojo, require, has, xhr){
	// module:
	//		dojo/text
	// summary:
	//		This module implements the !dojo/text plugin and the dojo.cache API.
	// description:
	//		We choose to include our own plugin to leverage functionality already contained in dojo
	//		and thereby reduce the size of the plugin compared to various foreign loader implementations.
	//		Also, this allows foreign AMD loaders to be used without their plugins.
	//
	//		CAUTION: this module is designed to optionally function synchronously to support the dojo v1.x synchronous
	//		loader. This feature is outside the scope of the CommonJS plugins specification.

	var getText;
	if(1){
		getText= function(url, sync, load){
			xhr("GET", {url:url, sync:!!sync, load:load});
		};
	}else{
		// TODOC: only works for dojo AMD loader
		if(require.getText){
			getText= require.getText;
		}else{
			console.error("dojo/text plugin failed to load because loader does not support getText");
		}
	}

	var
		theCache= {},

		strip= function(text){
			//Strips <?xml ...?> declarations so that external SVG and XML
			//documents can be added to a document without worry. Also, if the string
			//is an HTML document, only the part inside the body tag is returned.
			if(text){
				text= text.replace(/^\s*<\?xml(\s)+version=[\'\"](\d)*.(\d)*[\'\"](\s)*\?>/im, "");
				var matches= text.match(/<body[^>]*>\s*([\s\S]+)\s*<\/body>/im);
				if(matches){
					text= matches[1];
				}
			}else{
				text = "";
			}
			return text;
		},

		notFound = {},

		pending = {},

		result= {
			dynamic:
				// the dojo/text caches it's own resources because of dojo.cache
				true,

			normalize:function(id, toAbsMid){
				// id is something like (path may be relative):
				//
				//	 "path/to/text.html"
				//	 "path/to/text.html!strip"
				var parts= id.split("!"),
					url= parts[0];
				return (/^\./.test(url) ? toAbsMid(url) : url) + (parts[1] ? "!" + parts[1] : "");
			},

			load:function(id, require, load){
				// id is something like (path is always absolute):
				//
				//	 "path/to/text.html"
				//	 "path/to/text.html!strip"
				var
					parts= id.split("!"),
					stripFlag= parts.length>1,
					absMid= parts[0],
					url = require.toUrl(parts[0]),
					text = notFound,
					finish = function(text){
						load(stripFlag ? strip(text) : text);
					};
				if(absMid in theCache){
					text = theCache[absMid];
				}else if(url in require.cache){
					text = require.cache[url];
				}else if(url in theCache){
					text = theCache[url];
				}
				if(text===notFound){
					if(pending[url]){
						pending[url].push(finish);
					}else{
						var pendingList = pending[url] = [finish];
						getText(url, !require.async, function(text){
							theCache[absMid]= theCache[url]= text;
							for(var i = 0; i<pendingList.length;){
								pendingList[i++](text);
							}
							delete pending[url];
						});
					}
				}else{
					finish(text);
				}
			}
		};

	dojo.cache= function(/*String||Object*/module, /*String*/url, /*String||Object?*/value){
		//	 * (string string [value]) => (module, url, value)
		//	 * (object [value])        => (module, value), url defaults to ""
		//
		//	 * if module is an object, then it must be convertable to a string
		//	 * (module, url) module + (url ? ("/" + url) : "") must be a legal argument to require.toUrl
		//	 * value may be a string or an object; if an object then may have the properties "value" and/or "sanitize"
		var key;
		if(typeof module=="string"){
			if(/\//.test(module)){
				// module is a version 1.7+ resolved path
				key = module;
				value = url;
			}else{
				// module is a version 1.6- argument to dojo.moduleUrl
				key = require.toUrl(module.replace(/\./g, "/") + (url ? ("/" + url) : ""));
			}
		}else{
			key = module + "";
			value = url;
		}
		var
			val = (value != undefined && typeof value != "string") ? value.value : value,
			sanitize = value && value.sanitize;

		if(typeof val == "string"){
			//We have a string, set cache value
			theCache[key] = val;
			return sanitize ? strip(val) : val;
		}else if(val === null){
			//Remove cached value
			delete theCache[key];
			return null;
		}else{
			//Allow cache values to be empty strings. If key property does
			//not exist, fetch it.
			if(!(key in theCache)){
				getText(key, true, function(text){
					theCache[key]= text;
				});
			}
			return sanitize ? strip(theCache[key]) : theCache[key];
		}
	};

	return result;

/*=====
dojo.cache = function(module, url, value){
	// summary:
	//		A getter and setter for storing the string content associated with the
	//		module and url arguments.
	// description:
	//		If module is a string that contains slashes, then it is interpretted as a fully
	//		resolved path (typically a result returned by require.toUrl), and url should not be
	//		provided. This is the preferred signature. If module is a string that does not
	//		contain slashes, then url must also be provided and module and url are used to
	//		call `dojo.moduleUrl()` to generate a module URL. This signature is deprecated.
	//		If value is specified, the cache value for the moduleUrl will be set to
	//		that value. Otherwise, dojo.cache will fetch the moduleUrl and store it
	//		in its internal cache and return that cached value for the URL. To clear
	//		a cache value pass null for value. Since XMLHttpRequest (XHR) is used to fetch the
	//		the URL contents, only modules on the same domain of the page can use this capability.
	//		The build system can inline the cache values though, to allow for xdomain hosting.
	// module: String||Object
	//		If a String with slashes, a fully resolved path; if a String without slashes, the
	//		module name to use for the base part of the URL, similar to module argument
	//		to `dojo.moduleUrl`. If an Object, something that has a .toString() method that
	//		generates a valid path for the cache item. For example, a dojo._Url object.
	// url: String
	//		The rest of the path to append to the path derived from the module argument. If
	//		module is an object, then this second argument should be the "value" argument instead.
	// value: String||Object?
	//		If a String, the value to use in the cache for the module/url combination.
	//		If an Object, it can have two properties: value and sanitize. The value property
	//		should be the value to use in the cache, and sanitize can be set to true or false,
	//		to indicate if XML declarations should be removed from the value and if the HTML
	//		inside a body tag in the value should be extracted as the real value. The value argument
	//		or the value property on the value argument are usually only used by the build system
	//		as it inlines cache content.
	//	example:
	//		To ask dojo.cache to fetch content and store it in the cache (the dojo["cache"] style
	//		of call is used to avoid an issue with the build system erroneously trying to intern
	//		this example. To get the build system to intern your dojo.cache calls, use the
	//		"dojo.cache" style of call):
	//		| //If template.html contains "<h1>Hello</h1>" that will be
	//		| //the value for the text variable.
	//		| var text = dojo["cache"]("my.module", "template.html");
	//	example:
	//		To ask dojo.cache to fetch content and store it in the cache, and sanitize the input
	//		 (the dojo["cache"] style of call is used to avoid an issue with the build system
	//		erroneously trying to intern this example. To get the build system to intern your
	//		dojo.cache calls, use the "dojo.cache" style of call):
	//		| //If template.html contains "<html><body><h1>Hello</h1></body></html>", the
	//		| //text variable will contain just "<h1>Hello</h1>".
	//		| var text = dojo["cache"]("my.module", "template.html", {sanitize: true});
	//	example:
	//		Same example as previous, but demostrates how an object can be passed in as
	//		the first argument, then the value argument can then be the second argument.
	//		| //If template.html contains "<html><body><h1>Hello</h1></body></html>", the
	//		| //text variable will contain just "<h1>Hello</h1>".
	//		| var text = dojo["cache"](new dojo._Url("my/module/template.html"), {sanitize: true});
	return val; //String
};
=====*/
});


},
'ibm/usmi/widgets/util/Utils':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!ibm/usmi/widgets/util/Constants"], function(dijit,dojo,dojox){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
dojo.provide("ibm.usmi.widgets.util.Utils");
//dojo.require("util.Format");
dojo.require("ibm.usmi.widgets.util.Constants");

/** copied from TPC Util.js **/
ibm.usmi.widgets.util.Utils = {
    /* Transforms an object with multiple attributes (arrays), into an array with
     * attributes with those names.
     *
     * Input: { id: [0,1,2], dontcare: "value", names: ['a','b','c'] }
     * Output: [{ id: 0, names: 'a'}, { id: 1, names: 'b' }, { id: 2, names: 'c' }]
     */
    Object2Array: function _Object2Array(obj) {
        var fields = [], out = [], sz, field, attr, i, j;

        // Gather names of fields that are arrays
        for (attr in obj) {
            if (dojo.isArray(obj[attr])) {
                fields.push(attr);
            }
        }
        if (fields.length) {
            // get final length of output array by measuring first field's size
            sz = obj[fields[0]].length;
            for (i = 0; i < sz; i += 1) {
                out[i] = {};
                for (j in fields) {
                    field = fields[j];
                    out[i][field] = obj[field][i];
                }
            }
        }
        return out;
    },

    /*
     * Current time since epoch in milliseconds
     */
     currentTimeMillis: function() {
        return (new Date()).getTime();
     },

    /* Convert an error object returned by the xhr errback to a TPC Message object
     * Returned message object format
     * {result : {
     *      type:
     *      msgId:
     *      time:
     *      text:
     * }}
     */
    toBSMMessage: function(error /*object*/) {
        if (!error || !error.message) {
            return error;
        } else {
            var bsmMsg = {
                result : {
                    type: "E",
                       msgId: "000001E",  //TODO: Get the correct ID for this message
                    time: "timestampe",//util.Format.formatTimestamp(new Date()),
                    text: error.message
                }
            };
            return bsmMsg;
        }
    },



    /**
     * The following three functions are testing the status strings based off the following convention:
     *     S:Success
     *   I:Information
     *   W:Warning
     *   E:Error
     *
     * These functions are useful when checking the result of a task.
     * General format for the result:
     *    result{ status: statusString, msgId: msgIdString, text: msgTextString }
     */
    /* Checks the string to see if its a success string*/
    isSuccess: function _isSuccess(/*String*/status){
        return status === "S";
    },

    isInformation: function _isInformation(/*String*/status){
        return status === "I";
    },

    /* Checks the string to see if its a warning string*/
    isWarning: function _isWarning(/*String*/status){
        return status === "W";
    },

    /* Checks the string to see if its an error string*/
    isError: function _isError(/*String*/status){
        return status === "E";
    },

    getLaunchURL: function _getLaunchURL(/*String*/ page) {
        return ibm.usmi.widgets.util.Constants.LAYOUT_PAGE + page;
    },

    /* Launch a page */
    launchPage: function _launchPage(/*String*/ page){
        window.location = ibm.usmi.widgets.util.Utils.getLaunchURL(page);
    },

    /* Launch a page in a seaparate browser window/tab */
    launchNewPage: function _launchNewPage(/*String*/ page){
        window.open(ibm.usmi.widgets.util.Utils.getLaunchURL(page));
    },

    /**
     * Abbreviate the string.
     * Usage: tpc.util.Utils.abbreviate("HelloThere", {min: 5, max: 6, center: false});
     * @param st - A string
     * @param optional - {min: aNumber, max: aNumber, center: boolean}
     */
    abbreviate: function _abbreviate(/*String*/ st, /*Object*/ optional) {
        if (st) {
            optional = dojo.mixin({min: 5, max: 23, center: true}, optional);
            var len = st.length,
            max = Math.max(optional.min, optional.max),
            sansEllipsis, n1, n2,suffix;
            if (len > max) {
                sansEllipsis = (max - 3);

                if (optional.format && optional.format === (" (x)") ) {
                    n2 =  st.lastIndexOf("(");
                    if (n2 > 0) {
                        suffix = "... " + st.substring(n2);
                        n1 = max - suffix.length;
                        return st.substring(0, n1) + suffix;
                    } else {
                        return st.substring(0, sansEllipsis) + "...";
                    }
                } else if (optional.center === true) {
                    n1 = Math.ceil(sansEllipsis/2);
                    n2 = len - (sansEllipsis - n1);
                    return st.substring(0, n1) + "..." + st.substring(n2);
                } else {
                    return st.substring(0, sansEllipsis) + "...";
                }
            }
        }
        return st;
    },

    /*
     * Compare val1 and val2 and, if they are equal or if both are null, undefined, empty or NaN,
     * then returns true, else returns false.
     */
    areValuesEqual: function (/*String|Number*/ val1, /*String|Number*/ val2, /*Boolean*/ trim) {
        if (trim) {
            val1 = this.trim(val1);
            val2 = this.trim(val2);
        }
        if (val1 === val2) {
            return true;    //Values are equal
        }
        //Values are not equal, but we must verify whether both are null, undefined, empty or NaN
        if ((val1 === null) || (typeof val1 === "undefined") || (val1 === "") ||
                (typeof val1 === "number" && isNaN(val1))) {
            //val1 is null, undefined, empty or NaN. Verify whether val2 is too.
            return (val2 === null) || (typeof val2 === "undefined") || (val2 === "") ||
                (typeof val2 === "number" && isNaN(val2));
        }
        return false;
    },

    /**
     * Returns the trimmed string, if the argument is a string, or the original argument otherwise
     */
    trim: function _trim(src) {
        return (typeof src === "string") ? dojo.trim(src) : src;
    },

    /**
     * Convert a plain text into HTML by replacing the characters having special meaning in HTML
     * by the corresponding escape sequence.
     */
    convertTextToHTML: function _convertTextToHTML(/*String*/ src) {
        return (typeof src === "string") ?
            src.replace(/&/gm, "&amp;").replace(/</gm, "&lt;").replace(/>/gm, "&gt;").replace(/"/gm, "&quot;").replace(/\n/g, "<br>")
            : src;
    },

    /**
     * Return the status code inside the given request object if it exists.  Return -1 if no status is found.
     */
    getRequestStatus: function(request) {
        return (request.xhr && request.xhr.status) ? request.xhr.status : -1;
    },

    /*
     * utility function to add and remove a class for a node
     */
    addAndRemoveClass : function(/* DOMnode*/ node, /*String */ classToAdd, /*String */ classToRemove) {

         dojo.removeClass(node, classToRemove);
         dojo.addClass(node, classToAdd);
    },


    contains: function(str1, str2, caseSensitive) {
        if(!caseSensitive) {
            str1 = str1.toLowerCase();
            str2 = str2.toLowerCase();
        }
        return str2.indexOf(str1) !== -1;
    }
};

});

},
'dojox/xml/parser':function(){
define(['dojo/_base/kernel', 'dojo/_base/lang', 'dojo/_base/array', 'dojo/_base/window', 'dojo/_base/sniff'], function(dojo){

dojo.getObject("xml.parser", true, dojox);

//DOM type to int value for reference.
//Ints make for more compact code than full constant names.
//ELEMENT_NODE                  = 1;
//ATTRIBUTE_NODE                = 2;
//TEXT_NODE                     = 3;
//CDATA_SECTION_NODE            = 4;
//ENTITY_REFERENCE_NODE         = 5;
//ENTITY_NODE                   = 6;
//PROCESSING_INSTRUCTION_NODE   = 7;
//COMMENT_NODE                  = 8;
//DOCUMENT_NODE                 = 9;
//DOCUMENT_TYPE_NODE            = 10;
//DOCUMENT_FRAGMENT_NODE        = 11;
//NOTATION_NODE                 = 12;

dojox.xml.parser.parse = function(/*String?*/ str, /*String?*/ mimetype){
	//	summary:
	//		cross-browser implementation of creating an XML document object from null, empty string, and XML text..
	//
	//	str:
	//		Optional text to create the document from.  If not provided, an empty XML document will be created.
	//		If str is empty string "", then a new empty document will be created.
	//	mimetype:
	//		Optional mimetype of the text.  Typically, this is text/xml.  Will be defaulted to text/xml if not provided.
	var _document = dojo.doc;
	var doc;

	mimetype = mimetype || "text/xml";
	if(str && dojo.trim(str) && "DOMParser" in dojo.global){
		//Handle parsing the text on Mozilla based browsers etc..
		var parser = new DOMParser();
		doc = parser.parseFromString(str, mimetype);
		var de = doc.documentElement;
		var errorNS = "http://www.mozilla.org/newlayout/xml/parsererror.xml";
		if(de.nodeName == "parsererror" && de.namespaceURI == errorNS){
			var sourceText = de.getElementsByTagNameNS(errorNS, 'sourcetext')[0];
			if(sourceText){
				sourceText = sourceText.firstChild.data;
			}
        	throw new Error("Error parsing text " + de.firstChild.data + " \n" + sourceText);
		}
		return doc;

	}else if("ActiveXObject" in dojo.global){
		//Handle IE.
		var ms = function(n){ return "MSXML" + n + ".DOMDocument"; };
		var dp = ["Microsoft.XMLDOM", ms(6), ms(4), ms(3), ms(2)];
		dojo.some(dp, function(p){
			try{
				doc = new ActiveXObject(p);
			}catch(e){ return false; }
			return true;
		});
		if(str && doc){
			doc.async = false;
			doc.loadXML(str);
			var pe = doc.parseError;
			if(pe.errorCode !== 0){
				throw new Error("Line: " + pe.line + "\n" +
					"Col: " + pe.linepos + "\n" +
					"Reason: " + pe.reason + "\n" +
					"Error Code: " + pe.errorCode + "\n" +
					"Source: " + pe.srcText);
			}
		}
		if(doc){
			return doc; //DOMDocument
		}
	}else if(_document.implementation && _document.implementation.createDocument){
		if(str && dojo.trim(str) && _document.createElement){
			//Everyone else that we couldn't get to work.  Fallback case.
			// FIXME: this may change all tags to uppercase!
			var tmp = _document.createElement("xml");
			tmp.innerHTML = str;
			var xmlDoc = _document.implementation.createDocument("foo", "", null);
			dojo.forEach(tmp.childNodes, function(child){
				xmlDoc.importNode(child, true);
			});
			return xmlDoc;	//	DOMDocument
		}else{
			return _document.implementation.createDocument("", "", null); // DOMDocument
		}
	}
	return null;	//	null
};

dojox.xml.parser.textContent = function(/*Node*/node, /*String?*/text){
	//	summary:
	//		Implementation of the DOM Level 3 attribute; scan node for text
	//	description:
	//		Implementation of the DOM Level 3 attribute; scan node for text
	//		This function can also update the text of a node by replacing all child
	//		content of the node.
	//	node:
	//		The node to get the text off of or set the text on.
	//	text:
	//		Optional argument of the text to apply to the node.
	if(arguments.length>1){
		var _document = node.ownerDocument || dojo.doc;  //Preference is to get the node owning doc first or it may fail
		dojox.xml.parser.replaceChildren(node, _document.createTextNode(text));
		return text;	//	String
	}else{
		if(node.textContent !== undefined){ //FF 1.5 -- remove?
			return node.textContent;	//	String
		}
		var _result = "";
		if(node){
			dojo.forEach(node.childNodes, function(child){
				switch(child.nodeType){
					case 1: // ELEMENT_NODE
					case 5: // ENTITY_REFERENCE_NODE
						_result += dojox.xml.parser.textContent(child);
						break;
					case 3: // TEXT_NODE
					case 2: // ATTRIBUTE_NODE
					case 4: // CDATA_SECTION_NODE
						_result += child.nodeValue;
				}
			});
		}
		return _result;	//	String
	}
};

dojox.xml.parser.replaceChildren = function(/*Element*/node, /*Node || Array*/ newChildren){
	//	summary:
	//		Removes all children of node and appends newChild. All the existing
	//		children will be destroyed.
	//	description:
	//		Removes all children of node and appends newChild. All the existing
	//		children will be destroyed.
	// 	node:
	//		The node to modify the children on
	//	newChildren:
	//		The children to add to the node.  It can either be a single Node or an
	//		array of Nodes.
	var nodes = [];

	if(dojo.isIE){
		dojo.forEach(node.childNodes, function(child){
			nodes.push(child);
		});
	}

	dojox.xml.parser.removeChildren(node);
	dojo.forEach(nodes, dojo.destroy);

	if(!dojo.isArray(newChildren)){
		node.appendChild(newChildren);
	}else{
		dojo.forEach(newChildren, function(child){
			node.appendChild(child);
		});
	}
};

dojox.xml.parser.removeChildren = function(/*Element*/node){
	//	summary:
	//		removes all children from node and returns the count of children removed.
	//		The children nodes are not destroyed. Be sure to call dojo.destroy on them
	//		after they are not used anymore.
	//	node:
	//		The node to remove all the children from.
	var count = node.childNodes.length;
	while(node.hasChildNodes()){
		node.removeChild(node.firstChild);
	}
	return count; // int
};


dojox.xml.parser.innerXML = function(/*Node*/node){
	//	summary:
	//		Implementation of MS's innerXML function.
	//	node:
	//		The node from which to generate the XML text representation.
	if(node.innerXML){
		return node.innerXML;	//	String
	}else if(node.xml){
		return node.xml;		//	String
	}else if(typeof XMLSerializer != "undefined"){
		return (new XMLSerializer()).serializeToString(node);	//	String
	}
	return null;
};

return dojox.xml.parser;

});

},
'gridx/modules/HLayout':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/Deferred",
	"dojo/_base/array",
	"dojo/dom-style",
	"dojo/DeferredList",
	"../core/_Module"
], function(declare, Deferred, array, domStyle, DeferredList, _Module){

	return declare(/*===== "gridx.modules.HLayout", =====*/_Module, {
		// summary:
		//		This module manages the horizontal layout of all grid UI parts.
		// description:
		//		When a user creates a grid with a given width, it means the width of the whole grid,
		//		which includes grid body, row header, and virtical scrollerbar (and maybe more in the future).
		//		So the width of the grid body must be calculated out so as to layout the grid properly.
		//		This module calculates grid body width by collecting width from all the registered
		//		grid UI parts. It is assumed that the width of these UI parts will not change when grid is resized.

		name: 'hLayout',

		getAPIPath: function(){
			// tags:
			//		protected extension
			return {
				hLayout: this
			};
		},
	
		load: function(args, startup){
			// tags:
			//		protected extension
			var t = this;
			t.connect(t.grid, '_onResizeEnd', function(changeSize, ds){
				var d, dl = [];
				for(d in ds){
					dl.push(ds[d]);
				}
				new DeferredList(dl).then(function(){
					t.reLayout();
				});
			});
			startup.then(function(){
				t._layout();
			});
		},

		//Package--------------------------------------------------------

		// lead: [package readonly] Number
		//		The pixel size of the total width of all the UI parts that are before(LTR: left, RTL: right) the grid body.
		lead: 0,

		// tail: [package readonly] Number
		//		The pixel size of the total width of all the UI parts that are after(LTR: right, RTL: left) the grid body.
		tail: 0,
	
		register: function(ready, refNode, isTail){
			// summary:
			//		Register a 'refNode' so this module can calculate its width when it is 'ready'
			// tags:
			//		package
			// ready: dojo.Deferred|null
			//		A deferred object indicating when the DOM node is ready for width calculation.
			//		If omitted, it means the refNode can be calculated at any time.
			// refNode: DOMNode
			//		The DOM node that represents a UI part in grid.
			// isTail: Boolean?
			//		If the 'refNode' appears after(LTR: right, RTL: left) the grid body, set this to true.
			var r = this._regs = this._regs || [];
			if(!ready){
				ready = new Deferred();
				ready.callback();
			}
			r.push([ready, refNode, isTail]);
		},

		reLayout: function(){
			// summary:
			//		Re-layout the grid horizontally. This means calculated the width of all registered
			//		grid UI components except the grid body. Then update the grid body width.
			//		Usually there's no need for users to call this method. It'll be automatically called
			//		when calling grid.resize().
			var t = this,
				r = t._regs,
				lead = 0,
				tail = 0;
			if(r){
				array.forEach(r, function(reg){
					var w = reg[1].offsetWidth || domStyle.get(reg[1], 'width');
					if(reg[2]){
						tail += w;
					}else{
						lead += w;
					}
				});
				t.lead = lead;
				t.tail = tail;
				t.onUpdateWidth(lead, tail);
			}
		},

		//Event---------------------------------------------------------
		onUpdateWidth: function(){
			// summary:
			//		Fired when the body width is updated.
			// tags:
			//		package
		},

		//Private-------------------------------------------------------
		_layout: function(){
			var t = this, r = t._regs;
			if(r){
				var lead = 0, tail = 0,
					dl = array.map(r, function(reg){
						return reg[0];
					});
				new DeferredList(dl).then(function(){
					array.forEach(r, function(reg){
						var w = reg[1].offsetWidth || domStyle.get(reg[1], 'width');
						if(reg[2]){
							tail += w;
						}else{
							lead += w;
						}
					});
					t.lead = lead;
					t.tail = tail;
					t.loaded.callback();
				});
			}else{
				t.loaded.callback();
			}
		}
	});
});

},
'dijit/form/_RadioButtonMixin':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.set
	"dojo/_base/event", // event.stop
	"dojo/_base/lang", // lang.hitch
	"dojo/query", // query
	"dojo/_base/window", // win.doc
	"../registry"	// registry.getEnclosingWidget
], function(array, declare, domAttr, event, lang, query, win, registry){

	// module:
	//		dijit/form/_RadioButtonMixin
	// summary:
	// 		Mixin to provide widget functionality for an HTML radio button

	return declare("dijit.form._RadioButtonMixin", null, {
		// summary:
		// 		Mixin to provide widget functionality for an HTML radio button

		// type: [private] String
		//		type attribute on <input> node.
		//		Users should not change this value.
		type: "radio",

		_getRelatedWidgets: function(){
			// Private function needed to help iterate over all radio buttons in a group.
			var ary = [];
			query("input[type=radio]", this.focusNode.form || win.doc).forEach( // can't use name= since query doesn't support [] in the name
				lang.hitch(this, function(inputNode){
					if(inputNode.name == this.name && inputNode.form == this.focusNode.form){
						var widget = registry.getEnclosingWidget(inputNode);
						if(widget){
							ary.push(widget);
						}
					}
				})
			);
			return ary;
		},

		_setCheckedAttr: function(/*Boolean*/ value){
			// If I am being checked then have to deselect currently checked radio button
			this.inherited(arguments);
			if(!this._created){ return; }
			if(value){
				array.forEach(this._getRelatedWidgets(), lang.hitch(this, function(widget){
					if(widget != this && widget.checked){
						widget.set('checked', false);
					}
				}));
			}
		},

		_onClick: function(/*Event*/ e){
			if(this.checked || this.disabled){ // nothing to do
				event.stop(e);
				return false;
			}
			if(this.readOnly){ // ignored by some browsers so we have to resync the DOM elements with widget values
				event.stop(e);
				array.forEach(this._getRelatedWidgets(), lang.hitch(this, function(widget){
					domAttr.set(this.focusNode || this.domNode, 'checked', widget.checked);
				}));
				return false;
			}
			return this.inherited(arguments);
		}
	});
});

},
'ibm/usmi/dashboard/ResourceGroup':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define(["dojo/_base/declare",
		"dojo/_base/Deferred",
		"dojo/_base/xhr",
		"dojo/_base/lang",
		"./DatasetItemStoreAdapter",
		"./Constants",
    	"dojo/i18n!./nls/DashbarResources"		
	], function (declare, Deferred, xhr, lang, DatasetItemStoreAdapter, Constants, mri) {
	return declare(null, {
		//component name for this group
		name: "",
		
		label: "",
		
		dataSetID: null,
		
		groupSourceToken: 0,
		
		groupID: "",
		
		topperLinks: null,
		
		selected: false,
		
		loadingLabel : "",
		
		overallStatus : "",
		
		normalImage: "",
		
		selectorHome: null,
		
		hasSelector: false,
		
		groupStatus : 0,
		
		groupCount : 0,
		
		groupOID : 0,
		
        //resource bundle
        _resources : mri,
		
		/**
		 * Creates a resource group.
		 *
		 * @constructor
		 * @param args
		 */
		constructor : function (args) {
			declare.safeMixin(this, args);
			if(this.hasSelector){
				this.selectorHome = this.dataSetID;
			} 
		},
		
		postMixInProperties: function () {
			// summary:  set nls strings
			// tags: private
			this.loadingLabel = this._resources.Loading;
			this.inherited(arguments);
		},
		
		getLabel: function () {
			return this.label;
		},
		
		getTopperLinks: function () {
			return this.topperLinks;
		},
		
		/**
		 *fetch the group memebers count
		 */
		fetchGroupMembersCountForUpdate : function () {

			var xhrArgs = {
				url : '/ibm/tivoli/rest/providers/USMI/datasets/GROUP:' + this.groupID,
				//url : '/ibm/tivoli/rest/providers/USMI/datasets/' + datasetId,
				handleAs : "json",
				preventCache : true
			};
			
			var handler = xhr.get(xhrArgs);
			return handler;
		},
		
		getGroupCount : function () {
			return this.groupCount;
		},
		
		setGroupCount : function (count) {
			this.groupCount = count;
		},
		
		getGroupStatusValue : function () {
			if (this.groupStatus){
				return this.groupStatus.getValue();
			}
			else{
				return 5; //Unknown
			}
		},
		
		getGroupStatus : function () {
			return this.groupStatus;
		},
		
		setGroupStatus : function (status) {
			this.groupStatus = status;
		},
		
		getGroupOID : function () {
			return this.groupOID;
		}
		
		
    });
});

},
'ibm/tivoli/navmodel/WidgetDefInfo':function(){
// wrapped by build app
define(["dijit","dojo","dojox"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */
dojo.provide("ibm.tivoli.navmodel.WidgetDefInfo"); 

////////////////////////////////////
// WidgetDefInfo
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.WidgetDefInfo", null, {
	// private members       
	_widgetDef: null,
	_version: null,
	
	/*=====
	constructor_kwArgs: function(widgetDef, version) {
		// widgetDef: Object
		//		json widget definition information object
		// version: Float
		//		optional: version used to communicate with rest service (i.e. json version)
	},
	=====*/
	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs Provider object.
		this._version = kwArgs.version;
		this._widgetDef = kwArgs.widgetDef;
		if (!this._widgetDef) {
			this._widgetDef = {};
		}
	},

	getAttribute: function(key) {
		// summary:
		//		Returns general attribute for requested key
		return this._widgetDef[key];
	},	
	
	getId: function() {
		// summary:
		//		Returns the id of this definition
		return this._widgetDef.id;
	},
	
	getVersion: function() {
		// summary:
		//		Returns the definition schema version
		return this._widgetDef.version;
	},
	
	getType: function() {
		// summary:
		//		Returns the widget type
		return this._widgetDef.type;
	},
	
	getProviderType: function() {
		// summary:
		//		Returns the provider type (can also be "any")
		return this._widgetDef.providerType;
	},
	
	getDatasourceType: function() {
		// summary:
		//		Returns the datasource type (can also be "any")
		return this._widgetDef.datasourceType;
	}
});

});

},
'gridx/modules/dnd/Avatar':function(){
define([
	"dojo/_base/declare",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dojo/_base/window",
	"dojo/dnd/Avatar"
], function(declare, domClass, domConstruct, win, Avatar){

	return declare(Avatar, {
		construct: function(manager){
			// summary:
			//		constructor function;
			//		it is separate so it can be (dynamically) overwritten in case of need
			var t = this;
			t.isA11y = domClass.contains(win.body(), "dijit_a11y");
			
			t.node = domConstruct.toDom(["<table border='0' cellspacing='0' class='gridxDndAvatar' ",
				"style='position: absolute; z-index: 1999; margin: 0'>",
				"<tbody><tr style='opacity: 0.9;'>",
					"<td class='gridxDnDIcon'>",
						t.isA11y ? "<span id='a11yIcon'>" + (t.manager.copy ? '+' : '<') + "</span>" : '',
					"</td>",
					"<td class='gridxDnDItemIcon ", t._getIconClass(), "'></td>",
					"<td><span class='gridxDnDItemCount'>", t._generateText(), "</span></td>",
				"</tr></tbody></table>"
			].join(''));
		},

		_getIconClass: function(){
			var info = this.manager._dndInfo;
			return ['gridxDnDIcon', info.cssName, info.count === 1 ? 'Single' : 'Multi'].join('');
		},

		_generateText: function(){
			// summary:
			//		generates a proper text to reflect copying or moving of items
			return "(" + this.manager._dndInfo.count + ")";
		}
	});
});

},
'dijit/form/_FormWidget':function(){
define([
	"dojo/_base/declare",	// declare
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/ready",
	"../_Widget",
	"../_CssStateMixin",
	"../_TemplatedMixin",
	"./_FormWidgetMixin"
], function(declare, kernel, ready, _Widget, _CssStateMixin, _TemplatedMixin, _FormWidgetMixin){

/*=====
var _Widget = dijit._Widget;
var _TemplatedMixin = dijit._TemplatedMixin;
var _CssStateMixin = dijit._CssStateMixin;
var _FormWidgetMixin = dijit.form._FormWidgetMixin;
=====*/

// module:
//		dijit/form/_FormWidget
// summary:
//		FormWidget


// Back compat w/1.6, remove for 2.0
if(!kernel.isAsync){
	ready(0, function(){
		var requires = ["dijit/form/_FormValueWidget"];
		require(requires);	// use indirection so modules not rolled into a build
	});
}

return declare("dijit.form._FormWidget", [_Widget, _TemplatedMixin, _CssStateMixin, _FormWidgetMixin], {
	// summary:
	//		Base class for widgets corresponding to native HTML elements such as <checkbox> or <button>,
	//		which can be children of a <form> node or a `dijit.form.Form` widget.
	//
	// description:
	//		Represents a single HTML element.
	//		All these widgets should have these attributes just like native HTML input elements.
	//		You can set them during widget construction or afterwards, via `dijit._Widget.attr`.
	//
	//		They also share some common methods.

	setDisabled: function(/*Boolean*/ disabled){
		// summary:
		//		Deprecated.  Use set('disabled', ...) instead.
		kernel.deprecated("setDisabled("+disabled+") is deprecated. Use set('disabled',"+disabled+") instead.", "", "2.0");
		this.set('disabled', disabled);
	},

	setValue: function(/*String*/ value){
		// summary:
		//		Deprecated.  Use set('value', ...) instead.
		kernel.deprecated("dijit.form._FormWidget:setValue("+value+") is deprecated.  Use set('value',"+value+") instead.", "", "2.0");
		this.set('value', value);
	},

	getValue: function(){
		// summary:
		//		Deprecated.  Use get('value') instead.
		kernel.deprecated(this.declaredClass+"::getValue() is deprecated. Use get('value') instead.", "", "2.0");
		return this.get('value');
	},

	postMixInProperties: function(){
		// Setup name=foo string to be referenced from the template (but only if a name has been specified)
		// Unfortunately we can't use _setNameAttr to set the name due to IE limitations, see #8484, #8660.
		// Regarding escaping, see heading "Attribute values" in
		// http://www.w3.org/TR/REC-html40/appendix/notes.html#h-B.3.2
		this.nameAttrSetting = this.name ? ('name="' + this.name.replace(/'/g, "&quot;") + '"') : '';
		this.inherited(arguments);
	},

	// Override automatic assigning type --> focusNode, it causes exception on IE.
	// Instead, type must be specified as ${type} in the template, as part of the original DOM
	_setTypeAttr: null
});

});

},
'ibm/usmi/widgets/util/ResourceNavigatorDAHelper':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define([
	"dojo/_base/kernel",
	"dojo/_base/declare",
    "dojo/topic",
	// NextGen imports
	"ibm/tivoli/tip/navwidget/topology/dijit/DefaultActionHelper",
	"ibm/tivoli/navmodel/Selection",
	"ibm/usmi/core/utils/Logger"
], function (dojo, declare, topic, DefaultActionHelper, Selection, Logger) {

	return declare([ DefaultActionHelper
	], {

		runCustomAction : true,
		_args : null,
		_task : null,
		urlProperty : "URL",
		
		// the logger
		_daLogger : new Logger("ibm.usmi.widgets.util", "ResourceNavigatorDAHelper"),
		
		noOp: false,

		/*
		 * Method which check context for default server task and executes if needed
		 * else executes the callback method
		 */
		executeDefaultAction : function (/*Object*/ objectContext, /*Property*/ property) {

			if(this.noOp){
				return;
			}
			this._args = {
				launchParms : "",
				menuItem : "",
				context : ""
			};

			dojo.mixin(this._args, objectContext);

			if(property){				
				this.executePropertyAction(objectContext, property);				
			} else {
				
				var selectedObjects = objectContext.selection, object, oid, URL, currentLocation, taskSelection;
				if ((selectedObjects.getLinkCount() + selectedObjects.getItemCount()) !== 1) {
					//if there is more than one item then just return and do nothing for now
					return;
				}
	
				object = null;
	
				//set the selected object
				if (selectedObjects.getItemCount() === 1) {
					object = selectedObjects.getItems()[0];
				}
	
				//var URL = (object.getProperty(this.urlProperty)).getValue();
				//TODO 2928
				oid = object.getId();
				//currentLocation = dojo.doc.location.href.substr(0, dojo.doc.location.href.indexOf("#"));

                if (object.getType() === "Group"){
                    taskSelection="&taskSelection=ibm/usmi/widgets/ResourceNavigator";
				}
                else {
                    taskSelection="";
                }
                
                //close the tooltip before launch page
        		if(objectContext._table){
        			if(objectContext._table.closeTooltip){
        				//close tooltip while double click;
        				objectContext._table.closeTooltip();
        			}
        		}

                //TOGGLE PAGE_ORIENTED DESIGN
                if (!pageLoadAssembly) {
                    launchNextGenPage("workAreaDijit=ibm/usmi/widgets/WorkArea&sourceToken=" + oid + "&selector=details"+taskSelection);
                } else {
                    launchNextGenPage("workAreaDijit=ibm/usmi/widgets/WorkArea&selector=details#sourceToken=" + oid+taskSelection);
                }
		}

		},
		
		executePropertyAction: function(/*Object*/ objectContext, /*Property*/ property){
			// summary:
			//		Method which checks context for default server task and executes if needed
			//		else executes the callback method
			// description:
			//		Coming soon
			// objectContext:
			//		Coming soon
			if (this._daLogger.isFinest()) {
				this._daLogger.entering("", "executePropertyAction");
			}
			var defaultTask = property.getTask();
			if (defaultTask) {
				
				if (defaultTask.getType() === "javascript") {
					if (this._daLogger.isFinest()) {
						this._daLogger.finest("default property task " + defaultTask.getId(),
								"executePropertyAction");
					}
					
					var jsonLaunchParms = defaultTask.getLaunchJavascript();
					var launchParms = dojo.fromJson(jsonLaunchParms);
					launchParms.selection = objectContext.selection;
						              
	                if (launchParms.topic) {
	                    topic.publish(launchParms.topic, launchParms, defaultTask, objectContext);
	                }
	                else {
						if (this._daLogger.isWarn()) {
							this._daLogger.warn("default property task " + defaultTask.getId()+ 
									"did not specify topic in launch parms", "executePropertyAction");
						}
	                }
	                return;					
				}						
			}
			if (this._daLogger.isFinest()) {
				this._daLogger.exiting("", "executePropertyAction");
			}
		},

		getDataSet : function () {
			return this._args._dataStore.dataset;
		},

		getSelection : function () {
			var selectedThings = this._args.selection, //        this._args.context.selection;
			currentSelection = null;
			if (selectedThings) {
				currentSelection = new Selection({
					version : this.getDataSet().getVersion()
				});
				currentSelection.addItems(selectedThings.getItemIds());
				currentSelection.addLinks(selectedThings.getLinkIds());
				currentSelection.addRelationships(selectedThings.getRelationshipIds());
			}
			return currentSelection;
		}
	});
});
},
'dojox/grid/_EditManager':function(){
define([
	"dojo/_base/lang",
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/_base/connect",
	"dojo/_base/sniff",
	"./util"
], function(lang, array, declare, connect, has, util){

return declare("dojox.grid._EditManager", null, {
	// summary:
	//		Controls grid cell editing process. Owned by grid and used internally for editing.
	constructor: function(inGrid){
		// inGrid: dojox.Grid
		//		The dojox.Grid this editor should be attached to
		this.grid = inGrid;
		if(has("ie")){
			this.connections = [connect.connect(document.body, "onfocus", lang.hitch(this, "_boomerangFocus"))];
		}else{
			this.connections = [connect.connect(this.grid, 'onBlur', this, 'apply')];
		}
	},
	
	info: {},

	destroy: function(){
		array.forEach(this.connections, connect.disconnect);
	},

	cellFocus: function(inCell, inRowIndex){
		// summary:
		//		Invoke editing when cell is focused
		// inCell: cell object
		//		Grid cell object
		// inRowIndex: Integer
		//		Grid row index
		if(this.grid.singleClickEdit || this.isEditRow(inRowIndex)){
			// if same row or quick editing, edit
			this.setEditCell(inCell, inRowIndex);
		}else{
			// otherwise, apply any pending row edits
			this.apply();
		}
		// if dynamic or static editing...
		if(this.isEditing() || (inCell && inCell.editable && inCell.alwaysEditing)){
			// let the editor focus itself as needed
			this._focusEditor(inCell, inRowIndex);
		}
	},

	rowClick: function(e){
		if(this.isEditing() && !this.isEditRow(e.rowIndex)){
			this.apply();
		}
	},

	styleRow: function(inRow){
		if(inRow.index == this.info.rowIndex){
			inRow.customClasses += ' dojoxGridRowEditing';
		}
	},

	dispatchEvent: function(e){
		var c = e.cell, ed = (c && c["editable"]) ? c : 0;
		return ed && ed.dispatchEvent(e.dispatch, e);
	},

	// Editing
	isEditing: function(){
		// summary:
		//		Indicates editing state of the grid.
		// returns: Boolean
		//	 	True if grid is actively editing
		return this.info.rowIndex !== undefined;
	},

	isEditCell: function(inRowIndex, inCellIndex){
		// summary:
		//		Indicates if the given cell is being edited.
		// inRowIndex: Integer
		//		Grid row index
		// inCellIndex: Integer
		//		Grid cell index
		// returns: Boolean
		//	 	True if given cell is being edited
		return (this.info.rowIndex === inRowIndex) && (this.info.cell.index == inCellIndex);
	},

	isEditRow: function(inRowIndex){
		// summary:
		//		Indicates if the given row is being edited.
		// inRowIndex: Integer
		//		Grid row index
		// returns: Boolean
		//	 	True if given row is being edited
		return this.info.rowIndex === inRowIndex;
	},

	setEditCell: function(inCell, inRowIndex){
		// summary:
		//		Set the given cell to be edited
		// inRowIndex: Integer
		//		Grid row index
		// inCell: Object
		//		Grid cell object
		if(!this.isEditCell(inRowIndex, inCell.index) && this.grid.canEdit && this.grid.canEdit(inCell, inRowIndex)){
			this.start(inCell, inRowIndex, this.isEditRow(inRowIndex) || inCell.editable);
		}
	},

	_focusEditor: function(inCell, inRowIndex){
		util.fire(inCell, "focus", [inRowIndex]);
	},

	focusEditor: function(){
		if(this.isEditing()){
			this._focusEditor(this.info.cell, this.info.rowIndex);
		}
	},

	// implement fix for focus boomerang effect on IE
	_boomerangWindow: 500,
	_shouldCatchBoomerang: function(){
		return this._catchBoomerang > new Date().getTime();
	},
	_boomerangFocus: function(){
		//console.log("_boomerangFocus");
		if(this._shouldCatchBoomerang()){
			// make sure we don't utterly lose focus
			this.grid.focus.focusGrid();
			// let the editor focus itself as needed
			this.focusEditor();
			// only catch once
			this._catchBoomerang = 0;
		}
	},
	_doCatchBoomerang: function(){
		// give ourselves a few ms to boomerang IE focus effects
		if(has("ie")){this._catchBoomerang = new Date().getTime() + this._boomerangWindow;}
	},
	// end boomerang fix API

	start: function(inCell, inRowIndex, inEditing){
		if(!this._isValidInput()){
			return;
		}
		this.grid.beginUpdate();
		this.editorApply();
		if(this.isEditing() && !this.isEditRow(inRowIndex)){
			this.applyRowEdit();
			this.grid.updateRow(inRowIndex);
		}
		if(inEditing){
			this.info = { cell: inCell, rowIndex: inRowIndex };
			this.grid.doStartEdit(inCell, inRowIndex);
			this.grid.updateRow(inRowIndex);
		}else{
			this.info = {};
		}
		this.grid.endUpdate();
		// make sure we don't utterly lose focus
		this.grid.focus.focusGrid();
		// let the editor focus itself as needed
		this._focusEditor(inCell, inRowIndex);
		// give ourselves a few ms to boomerang IE focus effects
		this._doCatchBoomerang();
	},

	_editorDo: function(inMethod){
		var c = this.info.cell;
		//c && c.editor && c.editor[inMethod](c, this.info.rowIndex);
		if(c && c.editable){
			c[inMethod](this.info.rowIndex);
		}
	},

	editorApply: function(){
		this._editorDo("apply");
	},

	editorCancel: function(){
		this._editorDo("cancel");
	},

	applyCellEdit: function(inValue, inCell, inRowIndex){
		if(this.grid.canEdit(inCell, inRowIndex)){
			this.grid.doApplyCellEdit(inValue, inRowIndex, inCell.field);
		}
	},

	applyRowEdit: function(){
		this.grid.doApplyEdit(this.info.rowIndex, this.info.cell.field);
	},

	apply: function(){
		// summary:
		//		Apply a grid edit
		if(this.isEditing() && this._isValidInput()){
			this.grid.beginUpdate();
			this.editorApply();
			this.applyRowEdit();
			this.info = {};
			this.grid.endUpdate();
			this.grid.focus.focusGrid();
			this._doCatchBoomerang();
		}
	},

	cancel: function(){
		// summary:
		//		Cancel a grid edit
		if(this.isEditing()){
			this.grid.beginUpdate();
			this.editorCancel();
			this.info = {};
			this.grid.endUpdate();
			this.grid.focus.focusGrid();
			this._doCatchBoomerang();
		}
	},

	save: function(inRowIndex, inView){
		// summary:
		//		Save the grid editing state
		// inRowIndex: Integer
		//		Grid row index
		// inView: Object
		//		Grid view
		var c = this.info.cell;
		if(this.isEditRow(inRowIndex) && (!inView || c.view==inView) && c.editable){
			c.save(c, this.info.rowIndex);
		}
	},

	restore: function(inView, inRowIndex){
		// summary:
		//		Restores the grid editing state
		// inRowIndex: Integer
		//		Grid row index
		// inView: Object
		//		Grid view
		var c = this.info.cell;
		if(this.isEditRow(inRowIndex) && c.view == inView && c.editable){
			c.restore(this.info.rowIndex);
		}
	},
	
	_isValidInput: function(){
		var w = (this.info.cell || {}).widget;		
		if(!w || !w.isValid){
			//no validation needed
			return true;
		}		
		w.focused = true;
		return w.isValid(true);
	}
});
});
},
'ibm/usmi/dashboard/ResourceDashBoard':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define([
    "ibm/tivoli/tip/TCTTable",
	"ibm/tivoli/navmodel/Selection",
    "dojo/_base/kernel",
    "dojo/_base/declare",
    "dijit/_WidgetBase",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "./ResourceGroup",
    "./ResourceGroupWidget",
    "./DashBoardResourceNavigator",
    "dojo/text!./templates/ResourceDashBoard.html",
    "./Constants",
    "ibm/usmi/widgets/util/Constants",
    "dojo/topic",
    "ibm/usmi/util/BaseWidget",
    "ibm/usmi/core/utils/Logger",
    "ibm/usmi/core/utils/_UsmiBrowserPersistence",
    "dojo/hash",
    "dojo/io-query",
    "dojo/_base/connect",
    "dojo/_base/array",
    "dojo/dom-construct",
    "ibm/usmi/widgets/TreeSelector",
    "ibm/usmi/widgets/TreeTopperLinks",
    "ibm/usmi/widgets/TreeTopper",
    "dijit/registry",
    "dojo/dom-class",
    "dijit/layout/ContentPane",
    "dijit/layout/BorderContainer",
    "ibm/usmi/util/DataStore",
    "dojo/i18n!./nls/DashbarResources",
    "ibm/usmi/util/DefaultMessageHandler",
    "dojo/_base/lang",
    "dojo/dom-style",
    "dojo/dom"
], function (TCTTable, Selection, dojo, declare, WidgetBase, TemplatedMixin, WidgetsInTemplateMixin, ResourceGroup,
        ResourceGroupWidget, DashBoardResourceNavigator, template, Constants, WidgetConstants, topic, BaseWidget,
        Logger, _UsmiBrowserPersistence, hash, ioQuery, connect, array, domConstruct, SelectorTree, TreeTopperLinks,
        TreeTopper, registry, domClass, ContentPane, BorderContainer, DataStore, mri, DefaultMessageHandler, lang, domStyle, dom) {
    return declare("ibm.usmi.dashboard.ResourceDashBoard", [
        WidgetBase,
        TemplatedMixin,
        WidgetsInTemplateMixin,
        BaseWidget,
        _UsmiBrowserPersistence
    ], {

        // Template info:  Path
        templateString : template,

        // array of dash board resource groups objects
        resourceGroups : {},

        // array of dash board resource group widgets
        resourceGroupWidgets : null,

        // Tree selector widget
        selector : null,

        // Tree topper widget
        treeTopper : null,

        // Id of resource group currently selected
        _prevSelectionGroup : "",

        // Id of curently selector group
        _selectedGroup : "",

        //Subscriptions to clean up
        _subscriptions : null,

        // convenience wrapper for dataset
        _dataStore : null,

        _dashbar : null,
				_registerStatusListenerTask : null,

        //resource bundle
        _resources : mri,

        // the logger
        _resourceDashBoardLogger : new Logger("ibm.usmi.dashboard", "ResourceDashBoard"),

            // summary:
            //         Called after a widget's dom has been setup
            // tags:
            //        protected

		postCreate : function () {
            this._subscriptions = [];
            this.resourceGroupWidgets = [];
            this.inherited(arguments);

            // subscribe to workarea isReady event
            this._subscriptions.push(connect.subscribe(WidgetConstants.TOPIC_READY_WORKAREA, this,
                    this.workAreaIsActive));

            this._dataStore = new DataStore({
                datasetId : 'DashbarCollection',
                homeParms : null,
                getItems : true,
                retrieveItemProperties : true,
                getTasks : false,
                initReadyCallback : dojo.hitch(this, this.initializeDataStores)
            });
        },

        initializeDataStores : function () {
            if (this._resourceDashBoardLogger.isInfo()) {
                this._resourceDashBoardLogger.info(
                        "Dashbar data retrieved from server. Converting CURI data into the dashbar object",
                        "initializeDataStores");
            }

            this._dataStore.getDataset().addEventListener(this, lang.hitch(this, this._eventHandler));

 //           this._convertDashbarCollectiontoDashbarObject();

            this._createDashBar();
        },

        /*
        * Called after getting tasks on the dataset. 
        * Will be used for registering a status listener 
        * after page is painted 
        */
				_getTasksComplete : function (itemId, tasks) {
            if (this._resourceDashBoardLogger.isFinest()) {
                this._resourceDashBoardLogger.entering("", "_getTasksComplete");
            }

            var taskExecuteComplete = dojo.hitch(this, function () {
                if (this._resourceDashBoardLogger.isFinest()) {
                    this._resourceDashBoardLogger.finest("ran server side task to register listeners",
                            "taskExecuteComplete");
                }

            });

            //if we got the task back, run it
            if (tasks[0]) {
                var selection = new Selection({
                    version : ""
                });
						selection.addItem(itemId);
                tasks[0].execute({
                    onComplete : dojo.hitch(this, taskExecuteComplete),
                    selection : selection
                });
            }
            if (this._resourceDashBoardLogger.isFinest()) {
                this._resourceDashBoardLogger.exiting("", "_getTasksComplete");
            }

        },

        /**
         * converts a collection into a dashbar objects for easy storage in
         * session persistence
         */
        _convertDashbarCollectiontoDashbarObject : function () {
            if (this._resourceDashBoardLogger.isFinest()) {
                this._resourceDashBoardLogger.entering("", "_convertDashbarCollectiontoDashbarObject");
            }
            var items, id, title, image, collection, groupID, widget, hasSelector, topperLinks,
            count, status, groupOID;

            items = this._dataStore.getItems();

            this._dashbar = {
                "dashbarItems" : []
            };

            /*
                First create the menus themselves reading from the collection.
                Once the menus are created, then read through the
                collection to populate them.
            */
            dojo.forEach(items, dojo.hitch(this, function (entry, i) {
                id = entry.getProperty('ID_PROP').getValue();
                title = entry.getProperty('TITLE_PROP').getValue();
                image = entry.getProperty('IMAGE_PROP').getValue();
                collection = entry.getProperty('COLLECTION_PROP').getValue();
                groupID = entry.getProperty('GROUP_PROP').getValue();
                widget = entry.getProperty('WIDGET_PROP').getValue();
                hasSelector = entry.getProperty('HAS_SELECTOR_PROP').getValue();
                topperLinks = entry.getProperty('TOPPER_STRING_PROP').getValue();
                count = entry.getProperty('GROUP_COUNT_PROP').getValue();
                status = entry.getProperty('GROUP_STATUS_PROP').getValue();
                groupOID = entry.getProperty('GROUP_OID_PROP').getValue();

                this._dashbar.dashbarItems.push({
                    "id" : id,
                    "title" : title,
                    "image" : image,
                    "collection" : collection,
                    "groupID" : groupID,
                    "widget" : widget,
                    "hasSelector" : hasSelector,
                    "topperLinks" : topperLinks,
                    "groupCount" : count,
                    "groupStatus" : status,
                    "groupOID" : groupOID
                });
            }));

            if (this._resourceDashBoardLogger.isFinest()) {
                this._resourceDashBoardLogger.exiting("", "_convertDashbarCollectiontoDashbarObject");
            }
        },

        /**
         * creates the menu headings from a menu bar object that's created from
         * a collection, creates a new popup menu bar item, menu dialog, and
         * submenu for each one.
         */
        _createDashBar : function () {
            if (this._resourceDashBoardLogger.isFinest()) {
            	this._resourceDashBoardLogger.entering("Creating a menu bar with " + this._dataStore.getItems().length
            			+ " menus", "_createDashBar");
            }

            var dashItemTitleKey, hashStr = hash(), hashObj, persistedSelectedGroup, hashStr2, items, groupOnUrl = false;

            this._selectedGroup = "systems";

            // If chassis exists make it the default view
            items = this._dataStore.getItems();
					for ( var i = 0; i < items.length; i += 1) {
                var id = items[i].getProperty('ID_PROP').getValue();
                if (id === "chassis") {
                  this._selectedGroup = "chassis";
                  break;
                }
            }

            // check the hash value
            if (hashStr) {
                // if hash is set, get the selected group value
                hashObj = ioQuery.queryToObject(hashStr);

                // if hash has selectedGroup specified, that is what we will use
                if (hashObj.selectedGroup) {
                    this._selectedGroup = hashObj.selectedGroup;
                    
                    // set the previous, so we dont process a hashupdate for this
                    this._prevSelectionGroup = this._selectedGroup;
                    groupOnUrl = true;
                } else {
                    // lets check to see if a selectedGroup was stored off into persistence, if so use it
                    persistedSelectedGroup = this.getData("selectedGrpkey", true);
                    if (persistedSelectedGroup) {
                        this._selectedGroup = persistedSelectedGroup.selectedGroup;
                    }

                    hashObj.selectedGroup = this._selectedGroup;
                    hashStr2 = ioQuery.objectToQuery(hashObj);
                    hash(hashStr2, true);
                }
            } else {
                // initialize to "systems" dash board group to be selected.
                hashObj = {
                    "selectedGroup" : this._selectedGroup,
                    "workAreaDijit" : "ibm/usmi/widgets/WorkArea",
                    "selector" : "ibm/usmi/dashboard/ResourceDashBoard"
                };
                // base widget utility method to add object to hash
                this.setUrlHashObj(hashObj);
            }

            // only parse the menu headings if the menu bar data has been set
//            if (this._dashbar) {
            items = this._dataStore.getItems();
//                for (i = 0; i < this._dashbar.dashbarItems.length; i+=1) {
            dojo.forEach(items, dojo.hitch(this, function (dashbarItem, i) {
                    var selected = false, resourceGroup, resouceGroupWidget, dashItemID;

                    dashItemTitleKey = dashbarItem.getProperty('TITLE_PROP').getValue();
                    dashItemID = dashbarItem.getProperty('ID_PROP').getValue();
                    	
                    if (this._resourceDashBoardLogger.isFinest()) {
                        this._resourceDashBoardLogger.finest("Creating dashbar item: " + dashItemTitleKey, "_createDashBar");
                    }

                    // set selected on group that was set on hash
                    if (this._selectedGroup === dashItemID) {
                        selected = true;
                    }

                    // create a resource Group Object
                    resourceGroup = new ResourceGroup({
                        name : dashItemID,
                        label : this._resources[dashItemTitleKey],
                        dataSetID : dashbarItem.getProperty('COLLECTION_PROP').getValue(),
                        groupID : dashbarItem.getProperty('GROUP_PROP').getValue(),
                        selectorHome : dashbarItem.getProperty('COLLECTION_PROP').getValue(),
                        hasSelector : dashbarItem.getProperty('HAS_SELECTOR_PROP').getValue(),
                        topperLinks : dashbarItem.getProperty('TOPPER_STRING_PROP').getValue(),
                        selected : selected,
                        groupCount : dashbarItem.getProperty('GROUP_COUNT_PROP').getValue(),
							//groupStatus : dashbarItem.getProperty('GROUP_STATUS_PROP').getValue(),
                        groupOID : dashbarItem.getProperty('GROUP_OID_PROP').getValue()
                    });

                    // add the resource Group to local array
                    this.resourceGroups[dashItemID] = resourceGroup;

                    // create a resource Group Widget
                    resouceGroupWidget = new ResourceGroupWidget({
                        resourceGroup : resourceGroup
                    }, dashItemID);

                    // load up the widget with initial data
                    resouceGroupWidget.loadSystemStatus();

                    // add this widget to widget array
                    this.resourceGroupWidgets.push(resouceGroupWidget);
            }));
//            }

            // update the title over the work area to selected resources group label
            topic.publish(WidgetConstants.TOPIC_TASK_TITLE_CHANGE, {
                view : "Resource Dashboard: " + this.resourceGroups[this._selectedGroup].label
            });

            // Listen for hash updates
            this._subscriptions.push(connect.subscribe(WidgetConstants.TOPIC_HASH_CHANGE, this, this.hashChange));
            // Listen if resourceGroup has been re-selected
            this._subscriptions.push(connect.subscribe(Constants.resourceGroupSelected, this, function (message) {
                this._update(message);
            }));

            //kick off an update if group was set on URl, if not we set the hash up above, which will
            // kick off hash update which will call _update
            if(groupOnUrl == true) {
            	this._update(this._selectedGroup);
            }

					//everything's setup at this point. Lets get status and update it if necessary
					var ds = this._dataStore.getDataset();
					//get these individually so we can set up listeners individually.
					ds.getTasks({
						sync : true,
						onComplete : dojo.hitch(this, function(tasks){
							this._registerStatusListenerTask=tasks[0];
						})							
					});
					
					for(var itemNum=0; itemNum<this._dataStore.getItems().length; itemNum++){
						this._getTasksComplete(items[itemNum].getId(), [this._registerStatusListenerTask]);
					}

            if (this._resourceDashBoardLogger.isFinest()) {
                this._resourceDashBoardLogger.exiting("", "_createDashBar");
            }
        },

        startup : function () {
            // summary:
            //         Called after any DOM fragments have been actually added to the document,
            //           not fired until
            //         after any potential children widgets have been created
            // tags:
            //        protected

            if (this._resourceDashBoardLogger.isFinest()) {
                this._resourceDashBoardLogger.entering("", "startup");
            }

            this.inherited(arguments);

            if (this._resourceDashBoardLogger.isFinest()) {
                this._resourceDashBoardLogger.exiting("", "startup");
            }
        },

        workAreaIsActive : function () {
    //        this._update(this._selectedGroup);
        },

        hashChange : function (hash) {
            var hashObj = ioQuery.queryToObject(hash), selectedGroup = hashObj.selectedGroup;
            if (selectedGroup && (this._prevSelectionGroup !== selectedGroup)) {
                topic.publish(WidgetConstants.TOPIC_TASK_TITLE_CHANGE, {
                    view : "Resource Dashboard: " + this.resourceGroups[selectedGroup].label
                });

                // store the selectedGroup
                this._prevSelectionGroup = selectedGroup;
                // kick off the update
                this._update(selectedGroup);

                // store the entire hash off for now.
                if (this._resourceDashBoardLogger.isFiner()) {
                    this._resourceDashBoardLogger.finer("save off dashboard selection", "hashChange");
                }
                this.setData("selectedGrpkey", hashObj, true);
            }
        },

        _createSelectorTopper : function (groupName) {
            // summary:
            //     If resource group widget needs a tree selector, check to see if it has base nodes to go
            //     above the tree, for example "All Virtual Servers"
            // tags:
            //     private

            if (this._resourceDashBoardLogger.isFinest()) {
                this._resourceDashBoardLogger.entering("", "_createSelectorTopper");
            }

            var topperLabel, topperLinksString, topperLinksArray, topperLinkObj, topperLinkTitleKey, treeTopperNode, i, resourceGroup;

            resourceGroup = this.resourceGroups[groupName];
            topperLabel = resourceGroup.getLabel();

            this.treeTopper = new TreeTopper({
                label : topperLabel,
                style : "width:100%; height:100%;"
            }, domConstruct.create("DIV"));

            topperLinksString = resourceGroup.getTopperLinks();//Constants.treeToppers[groupName];
            if(topperLinksString) {
                topperLinksArray = JSON.parse(topperLinksString);

                if (topperLinksArray) {
                    for (i = 0; i < topperLinksArray.length; i += 1) {
                        topperLinkObj = topperLinksArray[i];
                        topperLinkTitleKey = topperLinkObj.title;
                        // construct the tree selector
                        treeTopperNode = new TreeTopperLinks({
                        	topperId : topperLinkObj.id,
                            label : this._resources[topperLinkTitleKey],
                            collectionID : topperLinkObj.collectionID,
                            taskWidgetId : topperLinkObj.widgetID,
                            icon : topperLinkObj.image
                        });

                        this.treeTopper.addTreeTopperLink(treeTopperNode);
                    }

                    this.treeTopper.placeAt("dashboard_selectorTreeTopper");
                    this.treeTopper.startup();
                }
            }

            if (this._resourceDashBoardLogger.isFinest()) {
                this._resourceDashBoardLogger.exiting("", "_createSelectorTopper");
            }
        },

        _createSelector : function (groupName, dashboardContainer, selectorCPane, outermostBorderContainer) {
            // summary:
            //     If resource group widget needs a tree selector, this method is called to
            //       create selector and add it to the page.
            // tags:
            //     private

            if (this._resourceDashBoardLogger.isFinest()) {
                this._resourceDashBoardLogger.entering("", "_createSelector");
            }

            // set the size of the selector pane, tree selector plus dashboard widget
            selectorCPane.setAttribute('style', "width: 460px;");

            // construct the tree selector
            this.selector = new SelectorTree({
                id : "resourceViewTreeSelector",
                selectorHome : this.resourceGroups[groupName].selectorHome,
		style : "width:305px; height:100%;padding:0px;margin-left: 5px;"
            }, domConstruct.create("DIV"));
            this.selector.placeAt("dashboard_selectorTree");
            this.selector.startup();

            // call resize on the main div to resize to content
            outermostBorderContainer.resize();

            if (this._resourceDashBoardLogger.isFinest()) {
                this._resourceDashBoardLogger.exiting("", "_createSelector");
            }
        },
        
        _getTreeCalculatedHeight: function(){
            var topperNode, containerNode, topperHeight, selectorHeight, calculated="100%";
            topperNode=dojo.byId("dashboard_selectorTreeTopper");
            containerNode=dojo.byId("dashbaordFullContainer");
            if (topperNode && containerNode){
                topperHeight=dojo.getContentBox(topperNode).h;
                selectorHeight = dojo.getContentBox(containerNode).h;
                calculated=selectorHeight-topperHeight-30+"px";
    
                if (this._resourceDashBoardLogger.isFinest()) {
                    this._resourceDashBoardLogger.finest("getTreeCalculatedHeight, topper height: "+topperHeight);
                    this._resourceDashBoardLogger.finest("getTreeCalculatedHeight, selector height: "+selectorHeight);
                    this._resourceDashBoardLogger.finest("getTreeCalculatedHeight, calced height: "+calculated);
                }
            }
            else if (this._resourceDashBoardLogger.isFinest()) {
                this._resourceDashBoardLogger.finest("getTreeCalculatedHeight, nodes not found");
            }
            return calculated;
        },
        
        resize : function () {
            if (this._resourceDashBoardLogger.isFinest()) {
                this._resourceDashBoardLogger.finest("resize resourcedashboard");
            }
            this.inherited(arguments);

            var resTreeDomNode=dom.byId("dashboard_selectorTree");
            if (resTreeDomNode){
                var calculatedHeight=this._getTreeCalculatedHeight();
                domStyle.set(resTreeDomNode, "height", calculatedHeight);
                if (this._resourceDashBoardLogger.isFinest()) {
                    this._resourceDashBoardLogger.finest("resize resourcedashboard, calced height: "+calculatedHeight);
                    this._resourceDashBoardLogger.finest("resize resourcedashboard, tree selector: "+registry.byId("resourceViewTreeSelector"));
                }
            } //end tree resize
        },

        _update : function (groupName) {
            // summary:
            //     Called when a dash board widget is selected.   Creates/destroys selector continer
            //        based on need.
            // tags:
            //     private

            if (this._resourceDashBoardLogger.isFinest()) {
                this._resourceDashBoardLogger.entering("", "_update");
            }

            var showSelector, dashboarLayout, dashboardSelectorContainer, selectorDomNode;
            var selectorTopperDomNode, selectorContentPane, outermostBorderContainer;

            // if selector was previously shown, destroy
            if (this.selector) {
                this.treeTopper.destroyRecursive();
                this.selector.destroyRecursive();
            }

            // get the selector home dataset id
            showSelector = this.resourceGroups[groupName].hasSelector;

            // get the dash board's border container
            dashboarLayout = registry.byId("dashbaordFullContainer");
            // get the dash board's selector content pane
            dashboardSelectorContainer = registry.byId("dashboardSelectorContainer");
            // get the frameworks selector content pane
            selectorContentPane = registry.byId("selectorContainer");
            // get the frameworks border container, used to resize the slider
            outermostBorderContainer = registry.byId("contentAreaLayout");

            // if there is no selectorHome for selected dash board widget, but the dash board content
            //pane exists need to remove and destroy it
            if (!showSelector && dashboardSelectorContainer) {
                try {
                    if (this._resourceDashBoardLogger.isFinest()) {
                        this._resourceDashBoardLogger.finest("removing child");
                    }
                    dashboarLayout.removeChild(dashboardSelectorContainer);

                    if (this._resourceDashBoardLogger.isFinest()) {
                        this._resourceDashBoardLogger.finest("destroying selector contentpane");
                    }
                    dashboardSelectorContainer.destroyRecursive();
                    selectorContentPane.setAttribute('style', "width: 165px;");

                    if (this._resourceDashBoardLogger.isFinest()) {
                        this._resourceDashBoardLogger.finest("successfully destoryed selector container");
                    }

                    // resize the border container, so fits contents of panes
                    outermostBorderContainer.resize();
                } catch (ex) {
                    if (this._resourceDashBoardLogger.isError()) {
                        this._resourceDashBoardLogger.error("udpate failed: " + ex, "_update");
                    }
                }
            }

            // if there is a selectorHome for selected dash board widget, but the dash board content pane
            //exists need to create it and add
            if (showSelector && !dashboardSelectorContainer) {
                dashboardSelectorContainer = new ContentPane({
                    id : "dashboardSelectorContainer",
                    region : 'left',
                    splitter : false,
                    style : 'width:310px;'
                }, domConstruct.create("DIV", {
                    id : "dashboardSelectorContainer"
                }));
                domClass.add(dashboardSelectorContainer.domNode, "dashboardSelectorContainer");

                // create the selectorTreeTopper dom node
                selectorTopperDomNode = domConstruct.create("DIV", {
                    id : "dashboard_selectorTreeTopper",
                    style : 'width:100%;'
                }, dashboardSelectorContainer.domNode);
                //      domClass.add(selectorTopperDomNode, "interiorSpacious");

                // create the selectorTree dom node
                selectorDomNode = domConstruct.create("DIV", {
                    id : "dashboard_selectorTree",
                    style : 'width:100%;height:100%'
                }, dashboardSelectorContainer.domNode);
                domClass.add(selectorDomNode, "interiorSpacious");
                dashboarLayout.addChild(dashboardSelectorContainer);
            }

            //

            // if tree selector is needed create one
            if (showSelector) {
                // make sure the width is set to hold selector, when coming from a bookmark, width is not set
                // appropriately, so get a half tree
                dashboardSelectorContainer.setAttribute('style', 'width:310px;height:100%;');
                // create nodes that go above the tree
                this._createSelectorTopper(groupName);
                this._createSelector(groupName, dashboarLayout, selectorContentPane, outermostBorderContainer);
            } else {
                // no tree selector, so we need to tell the workarea what to show
                this._setTableDetails(groupName);
            }

            if (this._resourceDashBoardLogger.isFinest()) {
                this._resourceDashBoardLogger.exiting("", "_update");
            }
            dijit.byId("pageLayout").resize();
        },

        _setTableDetails : function (groupName) {
            // summary:
            //     If resource group widget does not needa tree selector, this method is called to
            //       setup hash parms so work area widet can load correclty.
            // tags:
            //     private

            if (this._resourceDashBoardLogger.isFinest()) {
                this._resourceDashBoardLogger.entering("", "_setTableDetails");
            }

            var datasetCollectionName, hashObj, taskInfoObj = {};
            if (this.resourceGroups[groupName]) {
                datasetCollectionName = this.resourceGroups[groupName].dataSetID;
            } else {
                datasetCollectionName = "GROUP:ALL_ENDPOINTS";
            }

            //    this.setUrlHash(this.updateUrlHash(Constants.SOURCE_TOKEN, sourceToken), true);
            taskInfoObj[WidgetConstants.COLLECTION] = datasetCollectionName;
            taskInfoObj[WidgetConstants.HASH_TASK_SELECTION] = "ibm/usmi/widgets/ResourceNavigator";
            taskInfoObj[WidgetConstants.SOURCE_TOKEN] = this.sourceToken;
            topic.publish(WidgetConstants.TOPIC_RETARGET_INFO, taskInfoObj);

            if (this._resourceDashBoardLogger.isFinest()) {
                this._resourceDashBoardLogger.exiting("", "_setTableDetails");
            }
        },

        _eventHandler : function (listenerObject, event) {
            // summary:
            //         Handle dataset events
            // tags:
            //     callback

            if (this._resourceDashBoardLogger.isFinest()) {
                this._resourceDashBoardLogger.entering("", "_eventHandler");
                this._resourceDashBoardLogger.finest("#***ResourceDashBoard- event datasetId="
                        + event.getDataset().getId() + " event=" + event.getType() + " eventId=" + event.getId());
            }
            
            var itemID, property, newCount, newStatus;
            switch (event.getType()) {
            
                case "itemPropertiesChanged":
                    // This is the event that is fired when a member is added / removed
                    for (var nodeNum=0; nodeNum<event.getItems().length; nodeNum++){
                        itemID = event.getItems()[nodeNum].getId();
                        array.forEach(listenerObject.resourceGroupWidgets, function (resourceGroupWidget) {
                            if (this._resourceDashBoardLogger.isFinest()) {
                                this._resourceDashBoardLogger.finest(resourceGroupWidget.resourceGroup.name, "_eventHandler");
                            }
            
                            if(itemID == resourceGroupWidget.getResourceGroupGroupOID()) {
                                // get count of property, set new value                         
                                property = event.getProperties()[nodeNum][0];
                                if(property.getId() === "GROUP_COUNT_PROP") {
                                    newCount = property.getValue();
                                    resourceGroupWidget.updateCount(newCount);
                                } else if(property.getId() === "GROUP_STATUS_PROP") {
                                    newStatus = property.getValue();
                                    resourceGroupWidget.updateStatus(newStatus);
                                }
            
                                if (this._resourceDashBoardLogger.isFinest()) {
                                    this._resourceDashBoardLogger.finest("refreshed resource " + resourceGroupWidget.resourceGroup.name, "_eventHandler");
                                }
                            }       
                        }, listenerObject);
                    }
                    break;
                default:            
                    break;
            }

            if (this._resourceDashBoardLogger.isFinest()) {
                this._resourceDashBoardLogger.exiting("", "eventHandler");
            }
        },
        
        destroy : function () {
        	// clean up subscriptions
            var idx = 0;
            for (; idx < this._subscriptions.length; idx++) {
                this._subscriptions[idx].remove();
            }
            this._subscriptions.length = 0;
            
            // cleanup widgets array
            var idy = 0;
            for (; idy < this.resourceGroupWidgets.length; idy++) {
                this.resourceGroupWidgets[idy].destroy();
            }
            
 			//Clear out the array
			this.resourceGroupWidgets.splice(0); 
            
            
            // clean up the datastore
			if (this._dataStore) {
				this._dataStore.getDataset().removeEventListener(this);
				this._dataStore.destroy();
				this._dataStore = null;
			}
			
            this.inherited(arguments);
        },
        
        destroyRecursive : function () {
            this.inherited(arguments);
        }

    });
});

},
'dijit/DropDownMenu':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/_base/event", // event.stop
	"dojo/keys", // keys
	"dojo/text!./templates/Menu.html",
	"./_OnDijitClickMixin",
	"./_MenuBase"
], function(declare, event, keys, template, _OnDijitClickMixin, _MenuBase){

/*=====
	var _MenuBase = dijit._MenuBase;
	var _OnDijitClickMixin = dijit._OnDijitClickMixin;
=====*/

	// module:
	//		dijit/DropDownMenu
	// summary:
	//		dijit.DropDownMenu widget

	return declare("dijit.DropDownMenu", [_MenuBase, _OnDijitClickMixin], {
		// summary:
		//		A menu, without features for context menu (Meaning, drop down menu)

		templateString: template,

		baseClass: "dijitMenu",

		postCreate: function(){
			var l = this.isLeftToRight();
			this._openSubMenuKey = l ? keys.RIGHT_ARROW : keys.LEFT_ARROW;
			this._closeSubMenuKey = l ? keys.LEFT_ARROW : keys.RIGHT_ARROW;
			this.connectKeyNavHandlers([keys.UP_ARROW], [keys.DOWN_ARROW]);
		},

		_onKeyPress: function(/*Event*/ evt){
			// summary:
			//		Handle keyboard based menu navigation.
			// tags:
			//		protected

			if(evt.ctrlKey || evt.altKey){ return; }

			switch(evt.charOrCode){
				case this._openSubMenuKey:
					this._moveToPopup(evt);
					event.stop(evt);
					break;
				case this._closeSubMenuKey:
					if(this.parentMenu){
						if(this.parentMenu._isMenuBar){
							this.parentMenu.focusPrev();
						}else{
							this.onCancel(false);
						}
					}else{
						event.stop(evt);
					}
					break;
			}
		}
	});
});

},
'dijit/a11y':function(){
define([
	"dojo/_base/array", // array.forEach array.map
	"dojo/_base/config", // defaultDuration
	"dojo/_base/declare", // declare
	"dojo/dom",			// dom.byId
	"dojo/dom-attr", // domAttr.attr domAttr.has
	"dojo/dom-style", // style.style
	"dojo/_base/sniff", // has("ie")
	"./_base/manager",	// manager._isElementShown
	"."	// for exporting methods to dijit namespace
], function(array, config, declare, dom, domAttr, domStyle, has, manager, dijit){

	// module:
	//		dijit/a11y
	// summary:
	//		Accessibility utility functions (keyboard, tab stops, etc.)

	var shown = (dijit._isElementShown = function(/*Element*/ elem){
		var s = domStyle.get(elem);
		return (s.visibility != "hidden")
			&& (s.visibility != "collapsed")
			&& (s.display != "none")
			&& (domAttr.get(elem, "type") != "hidden");
	});

	dijit.hasDefaultTabStop = function(/*Element*/ elem){
		// summary:
		//		Tests if element is tab-navigable even without an explicit tabIndex setting

		// No explicit tabIndex setting, need to investigate node type
		switch(elem.nodeName.toLowerCase()){
			case "a":
				// An <a> w/out a tabindex is only navigable if it has an href
				return domAttr.has(elem, "href");
			case "area":
			case "button":
			case "input":
			case "object":
			case "select":
			case "textarea":
				// These are navigable by default
				return true;
			case "iframe":
				// If it's an editor <iframe> then it's tab navigable.
				var body;
				try{
					// non-IE
					var contentDocument = elem.contentDocument;
					if("designMode" in contentDocument && contentDocument.designMode == "on"){
						return true;
					}
					body = contentDocument.body;
				}catch(e1){
					// contentWindow.document isn't accessible within IE7/8
					// if the iframe.src points to a foreign url and this
					// page contains an element, that could get focus
					try{
						body = elem.contentWindow.document.body;
					}catch(e2){
						return false;
					}
				}
				return body && (body.contentEditable == 'true' ||
					(body.firstChild && body.firstChild.contentEditable == 'true'));
			default:
				return elem.contentEditable == 'true';
		}
	};

	var isTabNavigable = (dijit.isTabNavigable = function(/*Element*/ elem){
		// summary:
		//		Tests if an element is tab-navigable

		// TODO: convert (and rename method) to return effective tabIndex; will save time in _getTabNavigable()
		if(domAttr.get(elem, "disabled")){
			return false;
		}else if(domAttr.has(elem, "tabIndex")){
			// Explicit tab index setting
			return domAttr.get(elem, "tabIndex") >= 0; // boolean
		}else{
			// No explicit tabIndex setting, so depends on node type
			return dijit.hasDefaultTabStop(elem);
		}
	});

	dijit._getTabNavigable = function(/*DOMNode*/ root){
		// summary:
		//		Finds descendants of the specified root node.
		//
		// description:
		//		Finds the following descendants of the specified root node:
		//		* the first tab-navigable element in document order
		//		  without a tabIndex or with tabIndex="0"
		//		* the last tab-navigable element in document order
		//		  without a tabIndex or with tabIndex="0"
		//		* the first element in document order with the lowest
		//		  positive tabIndex value
		//		* the last element in document order with the highest
		//		  positive tabIndex value
		var first, last, lowest, lowestTabindex, highest, highestTabindex, radioSelected = {};

		function radioName(node){
			// If this element is part of a radio button group, return the name for that group.
			return node && node.tagName.toLowerCase() == "input" &&
				node.type && node.type.toLowerCase() == "radio" &&
				node.name && node.name.toLowerCase();
		}

		var walkTree = function(/*DOMNode*/parent){
			for(var child = parent.firstChild; child; child = child.nextSibling){
				// Skip text elements, hidden elements, and also non-HTML elements (those in custom namespaces) in IE,
				// since show() invokes getAttribute("type"), which crash on VML nodes in IE.
				if(child.nodeType != 1 || (has("ie") && child.scopeName !== "HTML") || !shown(child)){
					continue;
				}

				if(isTabNavigable(child)){
					var tabindex = domAttr.get(child, "tabIndex");
					if(!domAttr.has(child, "tabIndex") || tabindex == 0){
						if(!first){
							first = child;
						}
						last = child;
					}else if(tabindex > 0){
						if(!lowest || tabindex < lowestTabindex){
							lowestTabindex = tabindex;
							lowest = child;
						}
						if(!highest || tabindex >= highestTabindex){
							highestTabindex = tabindex;
							highest = child;
						}
					}
					var rn = radioName(child);
					if(domAttr.get(child, "checked") && rn){
						radioSelected[rn] = child;
					}
				}
				if(child.nodeName.toUpperCase() != 'SELECT'){
					walkTree(child);
				}
			}
		};
		if(shown(root)){
			walkTree(root);
		}
		function rs(node){
			// substitute checked radio button for unchecked one, if there is a checked one with the same name.
			return radioSelected[radioName(node)] || node;
		}

		return { first: rs(first), last: rs(last), lowest: rs(lowest), highest: rs(highest) };
	};
	dijit.getFirstInTabbingOrder = function(/*String|DOMNode*/ root){
		// summary:
		//		Finds the descendant of the specified root node
		//		that is first in the tabbing order
		var elems = dijit._getTabNavigable(dom.byId(root));
		return elems.lowest ? elems.lowest : elems.first; // DomNode
	};

	dijit.getLastInTabbingOrder = function(/*String|DOMNode*/ root){
		// summary:
		//		Finds the descendant of the specified root node
		//		that is last in the tabbing order
		var elems = dijit._getTabNavigable(dom.byId(root));
		return elems.last ? elems.last : elems.highest; // DomNode
	};

	return {
		hasDefaultTabStop: dijit.hasDefaultTabStop,
		isTabNavigable: dijit.isTabNavigable,
		_getTabNavigable: dijit._getTabNavigable,
		getFirstInTabbingOrder: dijit.getFirstInTabbingOrder,
		getLastInTabbingOrder: dijit.getLastInTabbingOrder
	};
});

},
'ibm/tivoli/navmodel/ClientDataset':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/Dataset"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.ClientDataset"); 
 
// includes
dojo.require("ibm.tivoli.navmodel.Dataset");

ibm.tivoli.navmodel.ClientDataset._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.ClientDataset");

////////////////////////////////////
// ClientDataset
//
// NOTE: do not call any abstract methods directly;
// call the public version of the methods defined in
// AbstractDataset instead and it will in turn call
// the abstract overrides.
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.ClientDataset", ibm.tivoli.navmodel.Dataset, {
	// private members
	_trace: ibm.tivoli.navmodel.ClientDataset._trace,
	_json: null,
	_items: null,
	_links: null,
	
	/*=====
	constructor_kwArgs: function(onComplete, onError, provider, datasource, dataset, dataUrl, data) {
		// onComplete: constructor_onComplete
	 	//		callback to be invoked when the Dataset is initialized and ready for methods
	 	//		to be invoked  (i.e. asynchronous initialization must be performed). 
	 	//		Note: A null value will result in synchronous initialization.
	 	// onError: constructor_onError
	 	//		callback to be invoked if there is a problem constructing the Dataset
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// dataset: String
		//		dataset id
		// dataUrl: String
		//		url to file containing dataset json
		// data: String
		//		alternative to the fileUrl (i.e. provide the json directly)
	},
	=====*/

	constructor: function(/*constructor_kwArgs*/ kwArgs) {	
		// summary:
		//		Constructs ClientDataset object.
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
	},
	
	/////////////////////////////
	// abstract method overrides
	//
	// NOTE: do not call any abstract methods directly;
	// call the public version of the methods defined in
	// AbstractDataset instead and it will in turn call
	// the abstract overrides.
	/////////////////////////////
	
	/*=====
	initDataset_kwArgs: function(createEventRouter) {
		// createEventRouter: Boolean
		//		boolean indicating whether new event router should be created
	},
	=====*/
	
	_initDataset: function(/*initDataset_kwArgs*/ kwArgs) {
		// summary:
		//		Initialize internal dataset object
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		
		// reset internal dataset info
		this._initEmptyDataset();				
		
		// see if json was passed directly or a file url was provided
		this._json = this.getArg("data");
		var dataUrl = this.getArg("dataUrl");
		
		// make synchronous call if no onComplete callback
		var synchronous = (kwArgs.onComplete ? false : true);
		var exception = null;
		
		// url to fetch json
		if (!this._json && dataUrl) {
			var now = new Date();		
			var content = {d: now.getTime(), r: Math.random()};
						
			dojo.xhrGet({
				url: dataUrl,
				content: content,
				handleAs: "json",
				sync: synchronous,
				load: dojo.hitch(this, function(result, ioArgs) {
					this._json = result;
					
					if (this._json.items) {
						this._items = this._extractItems();		
						this._setItemCount(this._items.length);
						this._cacheItems(this._items);
					}
					
					if (this._json.links) {
						this._links = this._extractLinks();
						this._setLinkCount(this._links.length);
						this._cacheLinks(this._links);
					}
					
					// misc attributes of the dataset
					if (this._json.dataset) {
						this.mixinAttributes(this._json.dataset);		
					}
					
					// call onComplete callback
					if (kwArgs.onComplete) {
						kwArgs.onComplete(this, kwArgs);
					}					
				}),
				error: dojo.hitch(this, function(error, ioArgs){
					exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_initDataset", this._trace, this.getVersion());
					if (kwArgs.onError) {
						kwArgs.onError(error, exception, ioArgs);
					}	
				})
			});
		}
		
		if (synchronous && exception) {
			throw exception;
		}		
	},

	_deleteDataset: function() {
		// summary:
		//		Dataset is no longer in use
		
		// nothing to do
	},

    _clearAllCaches: function() {
		// summary:
		//		Override clear caches since all data is client side
        //          and we are not fetching the data again from a server
		
		// nothing to do
    },
      
	/*=====
	fetchItemsAndLinks_onComplete: function(items, links, datasetItems, toolbar, kwArgs){},
	=====*/
	
	/*=====
	_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	fetchItemsAndLink_kwArgs: function(onComplete, onError, sync) {
		// onComplete: fetchItemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchItemsAndLinks: function(/*fetchItemsAndLink_kwArgs*/ kwArgs) {
		// summary:
		//		Returns both the items and links associated with the dataset
		
		// build topology
		var items = this._extractItems();
		var links = this._extractLinks();
		var toolbar = this._extractToolbar();
		
		// execute the callback
		if (kwArgs.sync) {
			kwArgs.onComplete(items, links, toolbar, kwArgs);
		}
		else {
			setTimeout(dojo.hitch(this, kwArgs.onComplete, items, links, toolbar, kwArgs), 0);
		}		
	},
	
	/*=====
	items_onComplete: function(items, kwArgs){},
	=====*/

	/*=====
	fetchItems_kwArgs: function(onComplete, onError, start, count, properties, metadata, condition, sync) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error conditions 
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	_fetchItems: function(/*fetchItems_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch items from rest service
		var items = this._items;			
		var itemLength = items.length;
		var start = kwArgs.start;
		var count = kwArgs.count;
		if (!count) {
			count = itemLength;
		}
		
		// need to pull out just the items requested
		if (start !== 0 || count !== itemLength) {
			items = items.slice(start, start + count);
		}
		
		// place counts in the arguments map
		kwArgs.numRows = items.length;
		kwArgs.filteredRows = itemLength;
		kwArgs.totalRows = itemLength;
							
		// execute the callback
		if (kwArgs.sync) {
			kwArgs.onComplete(items, kwArgs);
		}
		else {
			setTimeout(dojo.hitch(this, kwArgs.onComplete, items, kwArgs), 0);
		}
	},

	/*=====
	fetchItem_kwArgs: function(itemId, properties, metadata) {
		// itemId: String
		//		item id to get
		// properties: String[]
		//		array of property ids to pre-fetch
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
	},
	=====*/
	
	_fetchItem: function(/*fetchItem_kwArgs*/ kwArgs) {
		// summary:
		//		Synchronously fetch an individual item from the rest service
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		return this._getCachedItem(kwArgs.itemId);		
	},

	/*=====
	fetchOriginItems_kwArgs: function(onComplete, onError, start, count, properties, metadata, sync) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchOriginItems: function(/*fetchOriginItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns orign items of the dataset
		var items = this._items;		
		var origins = [];
		var itemsLength = items.length;
		for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
			if (items[itemIndex].isOrigin()) {
				origins.push(items[itemIndex]);
			}
		}
		
		// Need to set as all item fetch calls are supposed to set these
        kwArgs.numRows = origins.length;
        kwArgs.filteredRows = origins.length;
        kwArgs.totalRows = origins.length;		
		
		// execute the callback
		if (kwArgs.sync) {
			kwArgs.onComplete(origins, kwArgs);
		}
		else {
			setTimeout(dojo.hitch(this, kwArgs.onComplete, origins, kwArgs), 0);
		}
	},
	
	/*=====
	fetchRelatedItems_kwArgs: function(onComplete, onError, itemId, relationships, start, count, properties, metadata, sync) {
		// onComplete: itemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// itemId: String
		//		id of the base item
		// relationships: String
		//		comma delimited list of relationships to determine related items; 
		//		can also be keywords: child, parent, all
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchRelatedItems: function(/*fetchRelatedItems_kwArgs*/ kwArgs) {
		var items = [];
		var links = [];
		var datasetItems = [];
		
		//TODO: find related items (simple first cut)
        if (kwArgs.itemId) {
            dojo.forEach(this._links, function(link) {
                 if (link.getItem1Id() == kwArgs.itemId) {
                     items.push(link.getItem2());
                 }                     
            });
         }

         kwArgs.numRows = items.length;
         kwArgs.filteredRows = items.length;
         kwArgs.totalRows = items.length;
         // END TODO (simple first cut)
		
		// execute the callback
		if (kwArgs.sync) {
			kwArgs.onComplete(items, links, datasetItems, kwArgs);
		}
		else {
			setTimeout(dojo.hitch(this, kwArgs.onComplete, items, links, datasetItems, kwArgs), 0);
		}
	},

	/*=====
	queryItems_kwArgs: function(onComplete, onError, query, start, count, properties, metadata, sync) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// query: Object
		//		custom query object to be sent as request body
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	 _queryItems: function(/*queryItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items associated with the dataset that match
		//		the provided custom query
	 	var items = [];
		
		//TODO: find items
		
		// execute the callback
		if (kwArgs.sync) {
			kwArgs.onComplete(items, kwArgs);
		}
	 	else {
	 		setTimeout(dojo.hitch(this, kwArgs.onComplete, items, kwArgs), 0);
	 	}
	 },

	/*=====
	links_onComplete: function(items, kwArgs){},
	=====*/
	 
	/*=====
	fetchLinks_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: links_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error conditions
		// start: int
		//		start index to fetch
		// count: int
		//		number of links to fetch; null means all
		// properties: String[]
		//		link property ids to fetch by default (can specify \"all\")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	 
	_fetchLinks: function(/*fetchLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch links from rest service
		var links = this._links;		
		var linkLength = links.length;
		var start = kwArgs.start;
		var count = kwArgs.count;
		if (!count) {
			count = linkLength;
		}
		
		// need to pull out just the links requested
		if (start !== 0 || count !== linkLength) {
			links = links.slice(start, start + count);
		}
		
		// place counts in the arguments map
		kwArgs.numRows = links.length;
		kwArgs.filteredRows = linkLength;
		kwArgs.totalRows = linkLength;
		
		// execute the callback
		if (kwArgs.sync) {
			kwArgs.onComplete(links, kwArgs);
		}
		else {
			setTimeout(dojo.hitch(this, kwArgs.onComplete, links, kwArgs), 0);
		}
	},
	
	/*=====
	fetchLink_kwArgs: function(itemId, properties, metadata) {
		// linkId: String
		//		link id to get
		// properties: String[]
		//		array of property ids to pre-fetch
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
	},
	=====*/
	
	_fetchLink: function(/*fetchLink_kwArgs*/ kwArgs) {
		// summary:
		//		Synchronously fetch an individual link from the rest service
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		return this._getCachedLink(kwArgs.linkId);		
	},
	
	/*=====
	relationships_onComplete: function(relationships, kwArgs){},
	=====*/
	
	/*=====
	fetchRelationships_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: relationships_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error conditions
		// start: int
		//		start index to fetch
		// count: int
		//		number of relationships to fetch; null means all
		// properties: String[]
		//		relationship property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchRelationships: function(/*fetchRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch relationships from rest service
		var relationships = [];		
		var links = this._links;
		var linkLength = links.length;
		for (var linkIndex = 0;  linkIndex < linkLength;  ++linkIndex) {
			var rels = links[linkIndex].getRelationships();
			if (rels) {
				relationships = relationships.concat(rels);
			}
		}
		
		var start = kwArgs.start;
		var count = kwArgs.count;
		if (!count) {
			count = relationships.length;
		}
		
		// need to pull out just the relationships requested
		if (start !== 0 || count !== relationships.length) {
			relationships = relationships.slice(start, start + count);
		}
		
		// place counts in the arguments map
		kwArgs.numRows = relationships.length;
		kwArgs.filteredRows = relationshipLength;
		kwArgs.totalRows = relationshipLength;
		
		// execute the callback
		if (kwArgs.sync) {
			kwArgs.onComplete(relationships, kwArgs);
		}
		else {
			setTimeout(dojo.hitch(this, kwArgs.onComplete, relationships, kwArgs), 0);
		}
	},
	
	/*=====
	fetchRelationship_kwArgs: function(relationshipId, properties, metadata) {
		// relationshipId: String
		//		relationship id to fetch
		// properties: String[]
		//		array of property ids to pre-fetch
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
	},
	=====*/
	
	_fetchRelationship: function(/*fetchRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Synchronously fetch an individual relationship from the rest service
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var relationship = this._getCachedRelationship(kwArgs.relationshipId);
		if (relationship) {
			return relationship;
		}
		
		var links = this._links;
		var linkLength = links.length;
		for (var linkIndex = 0;  linkIndex < linkLength;  ++linkIndex) {
			var relationships = links[linkIndex].getRelationships();
			if (relationships) {
				var relLength = relationships.length;
				for (var relIndex = 0;  relIndex < relLength;  ++relIndex) {
					if (relationships[relIndex].getId() === kwArgs.relationshipId) {
						this._cacheRelationship(relationships[relIndex]);
						return relationships[relIndex];
					}
				}										
			}
		}
		
		return null;
	},
	
	/*=====
	itemStyles_onComplete: function(itemStyles, kwArgs){},
	=====*/
	
	/*=====
	fetchItemStyles_kwArgs: function(onComplete, onError, start, count, sync) {
		// onComplete: itemStyle_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of item styles to fetch;  null means all
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	_fetchItemStyles: function(/*fetchItemStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current item styles for the Dataset
		var styles = [];
		var styleObjs = this._json.itemStyles;
		if (styleObjs) {
			var styleLength = styleObjs.length;			
			var start = kwArgs.start;
			var count = kwArgs.count;
			if (!count) {
				count = styleLength;
			}			
			for (var styleIndex = start;  styleIndex < styleLength && count > 0;  ++styleIndex, --count) {
				styles[styleIndex] = this._newItemStyle(styleObjs[styleIndex]);
			}			
		}
		
		// execute the callback
		if (kwArgs.sync) {
			kwArgs.onComplete(styles, kwArgs);
		}
		else {
			setTimeout(dojo.hitch(this, kwArgs.onComplete, styles, kwArgs), 0);
		}
	},
	
	/*=====
	linkStyles_onComplete: function(linkStyles, kwArgs){},
	=====*/
	
	/*=====
	fetchLinkStyles_kwArgs: function(onComplete, onError, start, count, sync) {
		// onComplete: linkStyles_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of link styles to fetch; null means all
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	_fetchLinkStyles: function(/*fetchLinkStyles_kwArgs*/ kwArgs) {
		var styles = [];
		var styleObjs = this._json.linkStyles;
		if (styleObjs) {
			var styleLength = styleObjs.length;
			var start = kwArgs.start;
			var count = kwArgs.count;
			if (!kwArgs.count) {
				count = styleLength;
			}
			for (var styleIndex = start;  styleIndex < styleLength && count > 0;  ++styleIndex, --count) {
				styles[styleIndex] = this._newLinkStyle(styleObjs[styleIndex]);
			}			
		}
		
		// execute the callback
		if (kwArgs.sync) {
			kwArgs.onComplete(styles, kwArgs);
		}
		else {
			setTimeout(dojo.hitch(this, kwArgs.onComplete, styles, kwArgs), 0);
		}
	},
	
	/*=====
	relStyles_onComplete: function(relStyles, kwArgs){},
	=====*/
	
	/*=====
	fetchRelationshipStyles_kwArgs: function(onComplete, onError, start, count, sync) {
		// onComplete: relStyles_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of relationship styles to fetch; null means all
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	_fetchRelationshipStyles: function(/*fetchRelationshipStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current relationship styles for the Dataset
		var styles = [];
		var styleObjs = this._json.relationshipStyles;
		if (styleObjs) {
			var styleLength = styleObjs.length;
			var start = kwArgs.start;
			var count = kwArgs.count;
			if (!kwArgs.count) {
				count = styleLength;
			}
			for (var styleIndex = start;  styleIndex < styleLength && count > 0;  ++styleIndex, --count) {
				styles[styleIndex] = this._newLinkStyle(styleObjs[styleIndex]);
			}			
		}
		
		// execute the callback
		if (kwArgs.sync) {
			kwArgs.onComplete(styles, kwArgs);
		}
		else {
			setTimeout(dojo.hitch(this, kwArgs.onComplete, styles, kwArgs), 0);
		}
	},
	
	/*=====
	itemStatuses_onComplete: function(itemStatuses, kwArgs){},
	=====*/
	
	/*=====
	fetchStatuses_kwArgs: function(onComplete, onError, type, start, count, sync) {
		// onComplete: itemStatuses_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// type: String
		//		object type for columns: "item", "link", "relationship"
		// start: int
		//		start index to fetch
		// count: int
		//		number of status values to fetch; null means all
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchStatuses: function(/*fetchStatuses_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of possible item status values for the Dataset
		var status = [];
		var statusObjs = null;
		
		switch(kwArgs.type) {
			case "item":
				statusObjs = this._json.itemStatus;
				break;
			case "link":
				statusObjs = this._json.linkStatus;
				break;
			case "relationship":
				statusObjs = this._json.relationshipStatus;
		}
		
		if (statusObjs) {
			var statusLength = statusObjs.length;
			var start = kwArgs.start;
			var count = kwArgs.count;
			if (!count) {
				count = statusLength;
			}
			for (var statusIndex = start;  statusIndex < statusLength && count > 0;  ++statusIndex, --count) {
				status[statusIndex] = this._newStatus(statusObjs[statusIndex]);
			}			
		}
		
		// execute the callback
		if (kwArgs.sync) {
			kwArgs.onComplete(status, kwArgs);
		}
		else {
			setTimeout(dojo.hitch(this, kwArgs.onComplete, status, kwArgs), 0);
		}		
	},

	/*=====
	tasks_onComplete: function(tasks, kwArgs){},
	=====*/
	
	/*=====
	fetchTasks_kwArgs: function(onComplete, onError, taskCategory, selection, itemIds, linkIds, itemIndexes, sync) {
		// onComplete: tasks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// taskCategory: String
		//		comma delimited list indicating type of tasks to fetch
		//		legal values are: menu, toolbar, and breadcrumb
		// selection: Object
		//		ibm.tivoli.navmodel.Selection object
		//		if selection is specified: itemIds, linkIds, itemIndexes is ignored
		// itemIds: String
		//		comma delimited list of item ids that are selected
		// linkIds: String
		//		comma delimited list of link ids that are selected
		// itemIndexes: String
		//		comma delimited list of item ids or ranges that are selected
		//		ex: "0,1,10,20" or "0-10,30,40-50"
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	_fetchTasks: function(/*fetchTasks_kwArgs*/ kwArgs) {
		// summary:
		//		Returns tasks associated with the dataset
		var tasks = [];
		var categories = kwArgs.taskCategory.split(",");
		var length = categories.length;
		
		if (this._json.tasks) {
			for (var index = 0;  index < length;  ++index) {
				var taskObjs = this._json.tasks[categories[index]];
				if (taskObjs) {
					var taskLength = taskObjs.length;
					for (var taskIndex = 0;  taskIndex < taskLength;  ++taskIndex) {
						tasks.push(this._newTask(taskObjs[taskIndex]));
					}
				}
			}
		}
		
		// execute the callback
		if (kwArgs.sync) {
			kwArgs.onComplete(tasks, kwArgs);
		}
		else {
			setTimeout(dojo.hitch(this, kwArgs.onComplete, tasks, kwArgs), 0);
		}
	},

	/*=====
	dataset_onComplete: function(dataset, kwArgs){},
	=====*/
	
	/*=====
	navigate_kwArgs: function(onComplete, onError, taskId, taskCategory, selectionsync) {
		// onComplete: dataset_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// taskId: String
		//		navigation task id
		// taskCategory: String
		//		navigation task category
		// selection: Object
		//		ibm.tivoli.navmodel.Selection object
		//		if selection is specified: itemIds, linkIds, itemIndexes is ignored
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	_navigate: function(/*navigate_kwArgs*/ kwArgs) {
		// summary:
		//		Navigate to another Dataset from a navigation task
	},
	
	/*=====
	cols_onComplete: function(cols, kwArgs){},
	=====*/
	
	/*=====
	fetchColumns_kwArgs: function(onComplete, onError, type, properties, start, count, sync) {
		// onComplete: cols_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// type: String
		//		object type: "item", "link", "relationship" 
		// properties: String[]
		//		array of column ids; or keyword "current", "default", or "all"
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	_fetchColumns: function(/*fetchColumns_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch columns from rest service
		var cols = [];
		var colObjs = this._extractColumnObjs(kwArgs.type, kwArgs.properties);
		if (colObjs) {
			var colLength = colObjs.length;
			var start = kwArgs.start;
			var count = kwArgs.count;
			if (!count) {
				count = colLength;
			}
			for (var colIndex = start;  colIndex < colLength && count > 0;  ++colIndex, --count) {
				cols[colIndex] = this._newColumn(colObjs[colIndex], kwArgs.type);
			}
		}
		
		// execute the callback
		if (kwArgs.sync) {
			kwArgs.onComplete(cols, kwArgs);
		}
		else {
			setTimeout(dojo.hitch(this, kwArgs.onComplete, cols, kwArgs), 0);
		}
	},
	
	/*=====
	setColumns_kwArgs: function(onComplete, onError, columns, type, sync) {
		// onComplete: cols_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// columns: ibm.tivoli.navmodel.Column[]
		//		array of columns to be saved
		// type: String
		//		object type: "item", "link", "relationship" 
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_setColumns: function(/*setColumns_kwArgs*/ kwArgs) {
		// summary:
		//		Save columns
	},
	
	/*=====
	item_onComplete: function(item, kwArgs){},
	=====*/
	
	/*=====
	createItem_kwArgs: function(onComplete, onError, item) {
		// onComplete: item_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// item: Object
		//		ibm.tivoli.navmodel.Item to be created on server
	},
	=====*/
	_createItem: function(/*createItem_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new item in the dataset
	},
	
	/*=====
	createItems_kwArgs: function(onComplete, onError, item) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// item: Object[]
		//		array of ibm.tivoli.navmodel.Item to be created on server
	},
	=====*/
	_createItems: function(/*createItems_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new items in the dataset
	},
	
	/*=====
	_onComplete: function(kwArgs){},
	=====*/
	
	/*=====
	deleteItem_kwArgs: function(onComplete, onError, itemId, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// itemId: String
		//		id of item to be deleted
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of item
	},
	=====*/
	
	_deleteItem: function(/*deleteItem_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a item from the dataset
	},

	/*=====
	deleteItems_kwArgs: function(onComplete, onError, itemIds, conditions, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// itemIds: String[]
		//		array of item ids to be deleted
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of items
	},
	=====*/
	
	_deleteItems: function(/*deleteItems_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete items from the dataset
	},

	/*=====
	link_onComplete: function(link, kwArgs){},
	=====*/
	
	/*=====
	createLink_kwArgs: function(onComplete, onError, link) {
		// onComplete: link_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// link: Object
		//		ibm.tivoli.navmodel.Link to be created on server
	},
	=====*/
	
	_createLink: function(/*createLink_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new link in the dataset
	},

	/*=====
	createLinks_kwArgs: function(onComplete, onError, links) {
		// onComplete: links_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// links: Object[]
		//		array of ibm.tivoli.navmodel.Link to be created on server
	},
	=====*/
	
	_createLinks: function(/*createLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new links in the dataset
	},
	
	/*=====
	deleteLink_kwArgs: function(onComplete, onError, linkId, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// linkId: String
		//		id of link to be deleted
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of link
	},
	=====*/
	
	_deleteLink: function(/*deleteLink_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a link from the dataset
	},

	/*=====
	deleteLinks_kwArgs: function(onComplete, onError, linkIds, conditions, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// linkIds: String[]
		//		array of link ids to be deleted
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of links
	},
	=====*/
	
	_deleteLinks: function(/*deleteLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete links from the dataset
	},

	/*=====
	relationship_onComplete: function(relationship, kwArgs){},
	=====*/
	
	/*=====
	createRelationship_kwArgs: function(onComplete, onError, relationship) {
		// onComplete: relationship_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationship: Object
		//		ibm.tivoli.navmodel.Relationship to be created on server
	},
	=====*/
	
	_createRelationship: function(/*createRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new relationship in the dataset
	},

	/*=====
	createRelationships_kwArgs: function(onComplete, onError, relationships) {
		// onComplete: relationships_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationship: Object[]
		//		array of ibm.tivoli.navmodel.Relationship to be created on server
	},
	=====*/
	
	_createRelationships: function(/*reateRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new relationships in the dataset
	},
	
	/*=====
	deleteRelationship_kwArgs: function(onComplete, onError, relationshipId, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationshipId: String
		//		id of relationship to be deleted
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of relationship
	},
	=====*/
	
	_deleteRelationship: function(/*deleteRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a relationship from the dataset
	},

	/*=====
	deleteRelationships_kwArgs: function(onComplete, onError, relationshipIds, conditions, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationshipIds: String[]
		//		array of relationship ids to be deleted
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of relationship
	},
	=====*/
	
	_deleteRelationships: function(/*deleteRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete relationships from the dataset
	},
	
	/////////////////////////////
	// protected methods
	/////////////////////////////

	_newItem: function(itemObj) {
		// summary:
		//		Create a new item
		return new ibm.tivoli.navmodel.Item({item: itemObj, dataset: this, allProperties: true, allMetadata: true});		
	},

	_newLink: function(linkObj) {
		// summary:
		//		Create a new link
		return new ibm.tivoli.navmodel.Link({link: linkObj, dataset: this, allProperties: true, allMetadata: true});		
	},

	_newItemStyle: function(styleObj) {
		// summary:
		//		Create a new item style	
		return new ibm.tivoli.navmodel.ItemStyle({style: styleObj, version: this.getVersion()});
	},
	
	_newLinkStyle: function(styleObj) {
		// summary:
		//		Create a new link style
		return new ibm.tivoli.navmodel.LinkStyle({style: styleObj, version: this.getVersion()});
	},

	_newStatus: function(statusObj) {
		// summary:
		//		Create a new status
		return new ibm.tivoli.navmodel.Status({status: statusObj, version: this.getVersion()});
	},

	_newTask: function(taskObj) {
		// summary:
		//		Create a new task
		return new ibm.tivoli.navmodel.Task({task: taskObj, version: this.getVersion()});
	},

	_newColumn: function(colObj, objectType) {
		// summary:
		//		Create a new column
		return new ibm.tivoli.navmodel.Column({column: colObj, objectType: objectType, dataset: this});
	},
	
	_validateItems: function() {
		// summary:
		//		Fetch items if we haven't already done so
		
		// get cached items
		if (!this._items) {
			this._this._extractItems();						
		}
	},

	_extractItems: function() {
		// summary:
		//		Extract items from json
		var items = [];
		
		// create items
		var itemObjs = this._json.items;
		if (itemObjs) {
			var itemLength = itemObjs.length;
			for (var itemIndex = 0;  itemIndex < itemLength;  ++itemIndex) {
				items[itemIndex] = this._newItem(itemObjs[itemIndex]);
			}
		}
		
		return items;
	},

	_extractLinks: function() {
		// summary:
		//		Extract links from json
		var links = [];
		
		// create links
		var linkObjs = this._json.links;
		if (linkObjs) {
			var linkLength = linkObjs.length;
			for (var linkIndex = 0;  linkIndex < linkLength;  ++linkIndex) {
				links[linkIndex] = this._newLink(linkObjs[linkIndex]);
			}
		}
		
		return links;	
	},

	_extractToolbar: function() {
		// summary:
		//		Extract toolbar from json
		var toolbar = [];
		return toolbar;
	},

	_extractColumnObjs: function(type, properties) {
		// summary:
		//		Extract json columns 
		var cols = [];
		var columnType = null;
		switch(type) {
			case "item":
				columnType = this._json.itemColumns;
				break;
			case "link":
				columnType = this._json.linkColumns;
				break;
			case "relationship":
				columnType = this._json.relationshipColumns;
		}
		
		if (columnType) {	
			var keyWord = properties.join(",");
			switch(keyWord) {
				case "current":
					cols = columnType.current;
					if (!cols || cols.length === 0) {
						cols = columnType.all;
					}
					break;
					
				case "default":
					cols = columnType["default"];
					if (!cols || cols.length === 0) {
						cols = columnType.all;
					}
					break;
					
				case "all":
					cols = columnType.all;
					break;
					
				default:
					var all = columnType.all;
					if (all) {
						var ids = properties;
						var idLength = ids.length;
						var length = all.length;
				
						for (var index = 0;  index < length;  ++index) {
							var colId = all[index].id;					
							for (var idIndex = 0;  idIndex < idLength;  ++idIndex) {
								if (colId === ids[idIndex]) {
									cols.push(all[index]);
									break;							
								}
							}
						}
					}
			}
		}
			
		return cols;	
	}
});
});

},
'dojox/validate/regexp':function(){
define(["dojo/_base/lang", "dojo/regexp", "dojox/main"], 
  function(lang, regexp, dojox){

var dxregexp = lang.getObject("validate.regexp", true, dojox);
dxregexp = dojox.validate.regexp = {
	
	ipAddress: function(/*Object?*/flags){
		// summary: Builds a RE that matches an IP Address
		//
		// description:
		//  Supports 5 formats for IPv4: dotted decimal, dotted hex, dotted octal, decimal and hexadecimal.
		//  Supports 2 formats for Ipv6.
		//
		// flags  An object.  All flags are boolean with default = true.
		//    flags.allowDottedDecimal  Example, 207.142.131.235.  No zero padding.
		//    flags.allowDottedHex  Example, 0x18.0x11.0x9b.0x28.  Case insensitive.  Zero padding allowed.
		//    flags.allowDottedOctal  Example, 0030.0021.0233.0050.  Zero padding allowed.
		//    flags.allowDecimal  Example, 3482223595.  A decimal number between 0-4294967295.
		//    flags.allowHex  Example, 0xCF8E83EB.  Hexadecimal number between 0x0-0xFFFFFFFF.
		//      Case insensitive.  Zero padding allowed.
		//    flags.allowIPv6   IPv6 address written as eight groups of four hexadecimal digits.
		//	FIXME: ipv6 can be written multiple ways IIRC
		//    flags.allowHybrid   IPv6 address written as six groups of four hexadecimal digits
		//      followed by the usual 4 dotted decimal digit notation of IPv4. x:x:x:x:x:x:d.d.d.d

		// assign default values to missing paramters
		flags = (typeof flags == "object") ? flags : {};
		if(typeof flags.allowDottedDecimal != "boolean"){ flags.allowDottedDecimal = true; }
		if(typeof flags.allowDottedHex != "boolean"){ flags.allowDottedHex = true; }
		if(typeof flags.allowDottedOctal != "boolean"){ flags.allowDottedOctal = true; }
		if(typeof flags.allowDecimal != "boolean"){ flags.allowDecimal = true; }
		if(typeof flags.allowHex != "boolean"){ flags.allowHex = true; }
		if(typeof flags.allowIPv6 != "boolean"){ flags.allowIPv6 = true; }
		if(typeof flags.allowHybrid != "boolean"){ flags.allowHybrid = true; }

		// decimal-dotted IP address RE.
		var dottedDecimalRE =
			// Each number is between 0-255.  Zero padding is not allowed.
			"((\\d|[1-9]\\d|1\\d\\d|2[0-4]\\d|25[0-5])\\.){3}(\\d|[1-9]\\d|1\\d\\d|2[0-4]\\d|25[0-5])";

		// dotted hex IP address RE.  Each number is between 0x0-0xff.  Zero padding is allowed, e.g. 0x00.
		var dottedHexRE = "(0[xX]0*[\\da-fA-F]?[\\da-fA-F]\\.){3}0[xX]0*[\\da-fA-F]?[\\da-fA-F]";

		// dotted octal IP address RE.  Each number is between 0000-0377.
		// Zero padding is allowed, but each number must have at least 4 characters.
		var dottedOctalRE = "(0+[0-3][0-7][0-7]\\.){3}0+[0-3][0-7][0-7]";

		// decimal IP address RE.  A decimal number between 0-4294967295.
		var decimalRE =  "(0|[1-9]\\d{0,8}|[1-3]\\d{9}|4[01]\\d{8}|42[0-8]\\d{7}|429[0-3]\\d{6}|" +
			"4294[0-8]\\d{5}|42949[0-5]\\d{4}|429496[0-6]\\d{3}|4294967[01]\\d{2}|42949672[0-8]\\d|429496729[0-5])";

		// hexadecimal IP address RE.
		// A hexadecimal number between 0x0-0xFFFFFFFF. Case insensitive.  Zero padding is allowed.
		var hexRE = "0[xX]0*[\\da-fA-F]{1,8}";

		// IPv6 address RE.
		// The format is written as eight groups of four hexadecimal digits, x:x:x:x:x:x:x:x,
		// where x is between 0000-ffff. Zero padding is optional. Case insensitive.
		var ipv6RE = "([\\da-fA-F]{1,4}\\:){7}[\\da-fA-F]{1,4}";

		// IPv6/IPv4 Hybrid address RE.
		// The format is written as six groups of four hexadecimal digits,
		// followed by the 4 dotted decimal IPv4 format. x:x:x:x:x:x:d.d.d.d
		var hybridRE = "([\\da-fA-F]{1,4}\\:){6}" +
			"((\\d|[1-9]\\d|1\\d\\d|2[0-4]\\d|25[0-5])\\.){3}(\\d|[1-9]\\d|1\\d\\d|2[0-4]\\d|25[0-5])";

		// Build IP Address RE
		var a = [];
		if(flags.allowDottedDecimal){ a.push(dottedDecimalRE); }
		if(flags.allowDottedHex){ a.push(dottedHexRE); }
		if(flags.allowDottedOctal){ a.push(dottedOctalRE); }
		if(flags.allowDecimal){ a.push(decimalRE); }
		if(flags.allowHex){ a.push(hexRE); }
		if(flags.allowIPv6){ a.push(ipv6RE); }
		if(flags.allowHybrid){ a.push(hybridRE); }

		var ipAddressRE = "";
		if(a.length > 0){
			ipAddressRE = "(" + a.join("|") + ")";
		}
		return ipAddressRE; // String
	},

	host: function(/*Object?*/flags){
		// summary: Builds a RE that matches a host
		// description: A host is a named host (A-z0-9_- but not starting with -), a domain name or an IP address, possibly followed by a port number.
		// flags: An object.
		//	  flags.allowNamed Allow a named host for local networks. Default is false.
		//    flags.allowIP  Allow an IP address for hostname.  Default is true.
		//    flags.allowLocal  Allow the host to be "localhost".  Default is false.
		//    flags.allowPort  Allow a port number to be present.  Default is true.
		//    flags in regexp.ipAddress can be applied.

		// assign default values to missing paramters
		flags = (typeof flags == "object") ? flags : {};

		if(typeof flags.allowIP != "boolean"){ flags.allowIP = true; }
		if(typeof flags.allowLocal != "boolean"){ flags.allowLocal = false; }
		if(typeof flags.allowPort != "boolean"){ flags.allowPort = true; }
		if(typeof flags.allowNamed != "boolean"){ flags.allowNamed = false; }

		//TODO: support unicode hostnames?
		// Domain name labels can not end with a dash.
		var domainLabelRE = "(?:[\\da-zA-Z](?:[-\\da-zA-Z]{0,61}[\\da-zA-Z])?)";
		var domainNameRE = "(?:[a-zA-Z](?:[-\\da-zA-Z]{0,6}[\\da-zA-Z])?)"; // restricted version to allow backwards compatibility with allowLocal, allowIP

		// port number RE
		var portRE = flags.allowPort ? "(\\:\\d+)?" : "";

		// build host RE
		var hostNameRE = "((?:" + domainLabelRE + "\\.)+" + domainNameRE + "\\.?)";
		if(flags.allowIP){ hostNameRE += "|" +  dxregexp.ipAddress(flags); }
		if(flags.allowLocal){ hostNameRE += "|localhost"; }
		if(flags.allowNamed){ hostNameRE += "|^[^-][a-zA-Z0-9_-]*"; }
		return "(" + hostNameRE + ")" + portRE; // String

	},

	url: function(/*Object?*/flags){
		// summary: Builds a regular expression that matches a URL
		//
		// flags: An object
		//    flags.scheme  Can be true, false, or [true, false].
		//      This means: required, not allowed, or match either one.
		//    flags in regexp.host can be applied.
		//    flags in regexp.ipAddress can be applied.

		// assign default values to missing paramters
		flags = (typeof flags == "object") ? flags : {};
		if(!("scheme" in flags)){ flags.scheme = [true, false]; }

		// Scheme RE
		var protocolRE = regexp.buildGroupRE(flags.scheme,
			function(q){ if(q){ return "(https?|ftps?)\\://"; } return ""; }
		);

		// Path and query and anchor RE
		var pathRE = "(/(?:[^?#\\s/]+/)*(?:[^?#\\s/]+(?:\\?[^?#\\s/]*)?(?:#[A-Za-z][\\w.:-]*)?)?)?";

		return protocolRE + dxregexp.host(flags) + pathRE;
	},

	emailAddress: function(/*Object?*/flags){

		// summary: Builds a regular expression that matches an email address
		//
		//flags: An object
		//    flags.allowCruft  Allow address like <mailto:foo@yahoo.com>.  Default is false.
		//    flags in regexp.host can be applied.
		//    flags in regexp.ipAddress can be applied.

		// assign default values to missing paramters
		flags = (typeof flags == "object") ? flags : {};
		if (typeof flags.allowCruft != "boolean") { flags.allowCruft = false; }
		flags.allowPort = false; // invalid in email addresses

		// user name RE per rfc5322
		var usernameRE = "([!#-'*+\\-\\/-9=?A-Z^-~]+[.])*[!#-'*+\\-\\/-9=?A-Z^-~]+";

		// build emailAddress RE
		var emailAddressRE = usernameRE + "@" + dxregexp.host(flags);

		// Allow email addresses with cruft
		if ( flags.allowCruft ) {
			emailAddressRE = "<?(mailto\\:)?" + emailAddressRE + ">?";
		}

		return emailAddressRE; // String
	},

	emailAddressList: function(/*Object?*/flags){
		// summary: Builds a regular expression that matches a list of email addresses.
		//
		// flags: An object.
		//    flags.listSeparator  The character used to separate email addresses.  Default is ";", ",", "\n" or " ".
		//    flags in regexp.emailAddress can be applied.
		//    flags in regexp.host can be applied.
		//    flags in regexp.ipAddress can be applied.

		// assign default values to missing paramters
		flags = (typeof flags == "object") ? flags : {};
		if(typeof flags.listSeparator != "string"){ flags.listSeparator = "\\s;,"; }

		// build a RE for an Email Address List
		var emailAddressRE = dxregexp.emailAddress(flags);
		var emailAddressListRE = "(" + emailAddressRE + "\\s*[" + flags.listSeparator + "]\\s*)*" +
			emailAddressRE + "\\s*[" + flags.listSeparator + "]?\\s*";

		return emailAddressListRE; // String
	},
	
	numberFormat: function(/*Object?*/flags){
		// summary: Builds a regular expression to match any sort of number based format
		// description:
		//  Use this method for phone numbers, social security numbers, zip-codes, etc.
		//  The RE can match one format or one of multiple formats.
		//
		//  Format
		//    #        Stands for a digit, 0-9.
		//    ?        Stands for an optional digit, 0-9 or nothing.
		//    All other characters must appear literally in the expression.
		//
		//  Example
		//    "(###) ###-####"       ->   (510) 542-9742
		//    "(###) ###-#### x#???" ->   (510) 542-9742 x153
		//    "###-##-####"          ->   506-82-1089       i.e. social security number
		//    "#####-####"           ->   98225-1649        i.e. zip code
		//
		// flags:  An object
		//    flags.format  A string or an Array of strings for multiple formats.

		// assign default values to missing paramters
		flags = (typeof flags == "object") ? flags : {};
		if(typeof flags.format == "undefined"){ flags.format = "###-###-####"; }

		// Converts a number format to RE.
		var digitRE = function(format){
			// escape all special characters, except '?'
			return regexp.escapeString(format, "?")
				// Now replace '?' with Regular Expression
				.replace(/\?/g, "\\d?")
				// replace # with Regular Expression
				.replace(/#/g, "\\d")
			;
		};

		// build RE for multiple number formats
		return regexp.buildGroupRE(flags.format, digitRE); //String
	},
	
	ca: {

		postalCode: function(){
			// summary: String regular Express to match Canadain Postal Codes
			return "([A-Z][0-9][A-Z] [0-9][A-Z][0-9])";
		},

		province: function(){
			// summary: a regular expression to match Canadian Province Abbreviations
			return "(AB|BC|MB|NB|NL|NS|NT|NU|ON|PE|QC|SK|YT)";
		}

	},
	
	us:{

		state: function(/*Object?*/flags){
			// summary: A regular expression to match US state and territory abbreviations
			//
			// flags  An object.
			//    flags.allowTerritories  Allow Guam, Puerto Rico, etc.  Default is true.
			//    flags.allowMilitary  Allow military 'states', e.g. Armed Forces Europe (AE).  Default is true.

			// assign default values to missing paramters
			flags = (typeof flags == "object") ? flags : {};
			if(typeof flags.allowTerritories != "boolean"){ flags.allowTerritories = true; }
			if(typeof flags.allowMilitary != "boolean"){ flags.allowMilitary = true; }

			// state RE
			var statesRE =
				"AL|AK|AZ|AR|CA|CO|CT|DE|DC|FL|GA|HI|ID|IL|IN|IA|KS|KY|LA|ME|MD|MA|MI|MN|MS|MO|MT|" +
				"NE|NV|NH|NJ|NM|NY|NC|ND|OH|OK|OR|PA|RI|SC|SD|TN|TX|UT|VT|VA|WA|WV|WI|WY";

			// territories RE
			var territoriesRE = "AS|FM|GU|MH|MP|PW|PR|VI";

			// military states RE
			var militaryRE = "AA|AE|AP";

			// Build states and territories RE
			if(flags.allowTerritories){ statesRE += "|" + territoriesRE; }
			if(flags.allowMilitary){ statesRE += "|" + militaryRE; }

			return "(" + statesRE + ")"; // String
		}

	}
	
};

return dxregexp;

});

},
'dojoe/tablex/TableAll':function(){
define("dojoe/tablex/TableAll",[
	"dojoe/tablex/Table",
	"dojoe/tablex/_Toolbar",
	"dojoe/tablex/_TableOptions",
	"dojoe/tablex/_QuickFilter",
	"dojoe/tablex/_ConfigureColumnsDialog",
	"dojoe/tablex/_ColumnVisible",
	"dojoe/tablex/plugins/GridFilter",
	"dojoe/tablex/plugins/Menu",
	"dojoe/tablex/plugins/Pagination",
	"dojoe/tablex/plugins/_MapToModules"
	
], function(){

	// module:
	//		table/table-all
	// summary:
	//		A rollup that includes every dijit. You probably don't need this.

	//console.warn("table-all may include much more code than your application actually requires. We strongly recommend that you investigate a custom build or the web build tool");

	return dojoe.tablex.TableAll;
});

},
'dijit/_editor/range':function(){
define([
	"dojo/_base/array", // array.every
	"dojo/_base/declare", // declare
	"dojo/_base/lang", // lang.isArray
	"dojo/_base/window", // win.global
	".."	// for exporting symbols to dijit, TODO: remove in 2.0
], function(array, declare, lang, win, dijit){

// module:
//		dijit/_editor/range
// summary:
//		W3C range API


dijit.range={};

dijit.range.getIndex = function(/*DomNode*/node, /*DomNode*/parent){
//	dojo.profile.start("dijit.range.getIndex");
	var ret = [], retR = [];
	var onode = node;

	var pnode, n;
	while(node != parent){
		var i = 0;
		pnode = node.parentNode;
		while((n = pnode.childNodes[i++])){
			if(n === node){
				--i;
				break;
			}
		}
		//if(i>=pnode.childNodes.length){
			//dojo.debug("Error finding index of a node in dijit.range.getIndex");
		//}
		ret.unshift(i);
		retR.unshift(i - pnode.childNodes.length);
		node = pnode;
	}

	//normalized() can not be called so often to prevent
	//invalidating selection/range, so we have to detect
	//here that any text nodes in a row
	if(ret.length > 0 && onode.nodeType == 3){
		n = onode.previousSibling;
		while(n && n.nodeType == 3){
			ret[ret.length - 1]--;
			n = n.previousSibling;
		}
		n = onode.nextSibling;
		while(n && n.nodeType == 3){
			retR[retR.length - 1]++;
			n = n.nextSibling;
		}
	}
//	dojo.profile.end("dijit.range.getIndex");
	return {o: ret, r:retR};
};

dijit.range.getNode = function(/*Array*/index, /*DomNode*/parent){
	if(!lang.isArray(index) || index.length == 0){
		return parent;
	}
	var node = parent;
//	if(!node)debugger
	array.every(index, function(i){
		if(i >= 0 && i < node.childNodes.length){
			node = node.childNodes[i];
		}else{
			node = null;
			//console.debug('Error: can not find node with index',index,'under parent node',parent );
			return false; //terminate array.every
		}
		return true; //carry on the every loop
	});

	return node;
};

dijit.range.getCommonAncestor = function(n1, n2, root){
	root = root || n1.ownerDocument.body;
	var getAncestors = function(n){
		var as = [];
		while(n){
			as.unshift(n);
			if(n !== root){
				n = n.parentNode;
			}else{
				break;
			}
		}
		return as;
	};
	var n1as = getAncestors(n1);
	var n2as = getAncestors(n2);

	var m = Math.min(n1as.length, n2as.length);
	var com = n1as[0]; //at least, one element should be in the array: the root (BODY by default)
	for(var i = 1; i < m; i++){
		if(n1as[i] === n2as[i]){
			com = n1as[i]
		}else{
			break;
		}
	}
	return com;
};

dijit.range.getAncestor = function(/*DomNode*/node, /*RegEx?*/regex, /*DomNode?*/root){
	root = root || node.ownerDocument.body;
	while(node && node !== root){
		var name = node.nodeName.toUpperCase();
		if(regex.test(name)){
			return node;
		}

		node = node.parentNode;
	}
	return null;
};

dijit.range.BlockTagNames = /^(?:P|DIV|H1|H2|H3|H4|H5|H6|ADDRESS|PRE|OL|UL|LI|DT|DE)$/;
dijit.range.getBlockAncestor = function(/*DomNode*/node, /*RegEx?*/regex, /*DomNode?*/root){
	root = root || node.ownerDocument.body;
	regex = regex || dijit.range.BlockTagNames;
	var block = null, blockContainer;
	while(node && node !== root){
		var name = node.nodeName.toUpperCase();
		if(!block && regex.test(name)){
			block = node;
		}
		if(!blockContainer && (/^(?:BODY|TD|TH|CAPTION)$/).test(name)){
			blockContainer = node;
		}

		node = node.parentNode;
	}
	return {blockNode:block, blockContainer:blockContainer || node.ownerDocument.body};
};

dijit.range.atBeginningOfContainer = function(/*DomNode*/container, /*DomNode*/node, /*Int*/offset){
	var atBeginning = false;
	var offsetAtBeginning = (offset == 0);
	if(!offsetAtBeginning && node.nodeType == 3){ //if this is a text node, check whether the left part is all space
		if(/^[\s\xA0]+$/.test(node.nodeValue.substr(0, offset))){
			offsetAtBeginning = true;
		}
	}
	if(offsetAtBeginning){
		var cnode = node;
		atBeginning = true;
		while(cnode && cnode !== container){
			if(cnode.previousSibling){
				atBeginning = false;
				break;
			}
			cnode = cnode.parentNode;
		}
	}
	return atBeginning;
};

dijit.range.atEndOfContainer = function(/*DomNode*/container, /*DomNode*/node, /*Int*/offset){
	var atEnd = false;
	var offsetAtEnd = (offset == (node.length || node.childNodes.length));
	if(!offsetAtEnd && node.nodeType == 3){ //if this is a text node, check whether the right part is all space
		if(/^[\s\xA0]+$/.test(node.nodeValue.substr(offset))){
			offsetAtEnd = true;
		}
	}
	if(offsetAtEnd){
		var cnode = node;
		atEnd = true;
		while(cnode && cnode !== container){
			if(cnode.nextSibling){
				atEnd = false;
				break;
			}
			cnode = cnode.parentNode;
		}
	}
	return atEnd;
};

dijit.range.adjacentNoneTextNode = function(startnode, next){
	var node = startnode;
	var len = (0 - startnode.length) || 0;
	var prop = next ? 'nextSibling' : 'previousSibling';
	while(node){
		if(node.nodeType != 3){
			break;
		}
		len += node.length;
		node = node[prop];
	}
	return [node,len];
};

dijit.range._w3c = Boolean(window['getSelection']);
dijit.range.create = function(/*Window?*/window){
	if(dijit.range._w3c){
		return (window || win.global).document.createRange();
	}else{//IE
		return new dijit.range.W3CRange;
	}
};

dijit.range.getSelection = function(/*Window*/win, /*Boolean?*/ignoreUpdate){
	if(dijit.range._w3c){
		return win.getSelection();
	}else{//IE
		var s = new dijit.range.ie.selection(win);
		if(!ignoreUpdate){
			s._getCurrentSelection();
		}
		return s;
	}
};

if(!dijit.range._w3c){
	dijit.range.ie = {
		cachedSelection: {},
		selection: function(win){
			this._ranges = [];
			this.addRange = function(r, /*boolean*/internal){
				this._ranges.push(r);
				if(!internal){
					r._select();
				}
				this.rangeCount = this._ranges.length;
			};
			this.removeAllRanges = function(){
				//don't detach, the range may be used later
//				for(var i=0;i<this._ranges.length;i++){
//					this._ranges[i].detach();
//				}
				this._ranges = [];
				this.rangeCount = 0;
			};
			var _initCurrentRange = function(){
				var r = win.document.selection.createRange();
				var type = win.document.selection.type.toUpperCase();
				if(type == "CONTROL"){
					//TODO: multiple range selection(?)
					return new dijit.range.W3CRange(dijit.range.ie.decomposeControlRange(r));
				}else{
					return new dijit.range.W3CRange(dijit.range.ie.decomposeTextRange(r));
				}
			};
			this.getRangeAt = function(i){
				return this._ranges[i];
			};
			this._getCurrentSelection = function(){
				this.removeAllRanges();
				var r = _initCurrentRange();
				if(r){
					this.addRange(r, true);
					this.isCollapsed = r.collapsed;
				}else{
					this.isCollapsed = true;
				}
			};
		},
		decomposeControlRange: function(range){
			var firstnode = range.item(0), lastnode = range.item(range.length - 1);
			var startContainer = firstnode.parentNode, endContainer = lastnode.parentNode;
			var startOffset = dijit.range.getIndex(firstnode, startContainer).o[0];
			var endOffset = dijit.range.getIndex(lastnode, endContainer).o[0] + 1;
			return [startContainer, startOffset,endContainer, endOffset];
		},
		getEndPoint: function(range, end){
			var atmrange = range.duplicate();
			atmrange.collapse(!end);
			var cmpstr = 'EndTo' + (end ? 'End' : 'Start');
			var parentNode = atmrange.parentElement();

			var startnode, startOffset, lastNode;
			if(parentNode.childNodes.length > 0){
				array.every(parentNode.childNodes, function(node, i){
					var calOffset;
					if(node.nodeType != 3){
						atmrange.moveToElementText(node);

						if(atmrange.compareEndPoints(cmpstr, range) > 0){
							//startnode = node.previousSibling;
							if(lastNode && lastNode.nodeType == 3){
								//where shall we put the start? in the text node or after?
								startnode = lastNode;
								calOffset = true;
							}else{
								startnode = parentNode;
								startOffset = i;
								return false;
							}
						}else{
							if(i == parentNode.childNodes.length - 1){
								startnode = parentNode;
								startOffset = parentNode.childNodes.length;
								return false;
							}
						}
					}else{
						if(i == parentNode.childNodes.length - 1){//at the end of this node
							startnode = node;
							calOffset = true;
						}
					}
					//			try{
					if(calOffset && startnode){
						var prevnode = dijit.range.adjacentNoneTextNode(startnode)[0];
						if(prevnode){
							startnode = prevnode.nextSibling;
						}else{
							startnode = parentNode.firstChild; //firstChild must be a text node
						}
						var prevnodeobj = dijit.range.adjacentNoneTextNode(startnode);
						prevnode = prevnodeobj[0];
						var lenoffset = prevnodeobj[1];
						if(prevnode){
							atmrange.moveToElementText(prevnode);
							atmrange.collapse(false);
						}else{
							atmrange.moveToElementText(parentNode);
						}
						atmrange.setEndPoint(cmpstr, range);
						startOffset = atmrange.text.length - lenoffset;

						return false;
					}
					//			}catch(e){ debugger }
					lastNode = node;
					return true;
				});
			}else{
				startnode = parentNode;
				startOffset = 0;
			}

			//if at the end of startnode and we are dealing with start container, then
			//move the startnode to nextSibling if it is a text node
			//TODO: do this for end container?
			if(!end && startnode.nodeType == 1 && startOffset == startnode.childNodes.length){
				var nextnode = startnode.nextSibling;
				if(nextnode && nextnode.nodeType == 3){
					startnode = nextnode;
					startOffset = 0;
				}
			}
			return [startnode, startOffset];
		},
		setEndPoint: function(range, container, offset){
			//text node
			var atmrange = range.duplicate(), node, len;
			if(container.nodeType != 3){ //normal node
				if(offset > 0){
					node = container.childNodes[offset - 1];
					if(node){
						if(node.nodeType == 3){
							container = node;
							offset = node.length;
							//pass through
						}else{
							if(node.nextSibling && node.nextSibling.nodeType == 3){
								container = node.nextSibling;
								offset = 0;
								//pass through
							}else{
								atmrange.moveToElementText(node.nextSibling ? node : container);
								var parent = node.parentNode;
								var tempNode = parent.insertBefore(node.ownerDocument.createTextNode(' '), node.nextSibling);
								atmrange.collapse(false);
								parent.removeChild(tempNode);
							}
						}
					}
				}else{
					atmrange.moveToElementText(container);
					atmrange.collapse(true);
				}
			}
			if(container.nodeType == 3){
				var prevnodeobj = dijit.range.adjacentNoneTextNode(container);
				var prevnode = prevnodeobj[0];
				len = prevnodeobj[1];
				if(prevnode){
					atmrange.moveToElementText(prevnode);
					atmrange.collapse(false);
					//if contentEditable is not inherit, the above collapse won't make the end point
					//in the correctly position: it always has a -1 offset, so compensate it
					if(prevnode.contentEditable != 'inherit'){
						len++;
					}
				}else{
					atmrange.moveToElementText(container.parentNode);
					atmrange.collapse(true);
				}

				offset += len;
				if(offset > 0){
					if(atmrange.move('character', offset) != offset){
						console.error('Error when moving!');
					}
				}
			}

			return atmrange;
		},
		decomposeTextRange: function(range){
			var tmpary = dijit.range.ie.getEndPoint(range);
			var startContainer = tmpary[0], startOffset = tmpary[1];
			var endContainer = tmpary[0], endOffset = tmpary[1];

			if(range.htmlText.length){
				if(range.htmlText == range.text){ //in the same text node
					endOffset = startOffset + range.text.length;
				}else{
					tmpary = dijit.range.ie.getEndPoint(range, true);
					endContainer = tmpary[0],endOffset = tmpary[1];
//					if(startContainer.tagName == "BODY"){
//						startContainer = startContainer.firstChild;
//					}
				}
			}
			return [startContainer, startOffset, endContainer, endOffset];
		},
		setRange: function(range, startContainer, startOffset, endContainer, endOffset, collapsed){
			var start = dijit.range.ie.setEndPoint(range, startContainer, startOffset);

			range.setEndPoint('StartToStart', start);
			if(!collapsed){
				var end = dijit.range.ie.setEndPoint(range, endContainer, endOffset);
			}
			range.setEndPoint('EndToEnd', end || start);

			return range;
		}
	};

declare("dijit.range.W3CRange",null, {
	constructor: function(){
		if(arguments.length>0){
			this.setStart(arguments[0][0],arguments[0][1]);
			this.setEnd(arguments[0][2],arguments[0][3]);
		}else{
			this.commonAncestorContainer = null;
			this.startContainer = null;
			this.startOffset = 0;
			this.endContainer = null;
			this.endOffset = 0;
			this.collapsed = true;
		}
	},
	_updateInternal: function(){
		if(this.startContainer !== this.endContainer){
			this.commonAncestorContainer = dijit.range.getCommonAncestor(this.startContainer, this.endContainer);
		}else{
			this.commonAncestorContainer = this.startContainer;
		}
		this.collapsed = (this.startContainer === this.endContainer) && (this.startOffset == this.endOffset);
	},
	setStart: function(node, offset){
		offset=parseInt(offset);
		if(this.startContainer === node && this.startOffset == offset){
			return;
		}
		delete this._cachedBookmark;

		this.startContainer = node;
		this.startOffset = offset;
		if(!this.endContainer){
			this.setEnd(node, offset);
		}else{
			this._updateInternal();
		}
	},
	setEnd: function(node, offset){
		offset=parseInt(offset);
		if(this.endContainer === node && this.endOffset == offset){
			return;
		}
		delete this._cachedBookmark;

		this.endContainer = node;
		this.endOffset = offset;
		if(!this.startContainer){
			this.setStart(node, offset);
		}else{
			this._updateInternal();
		}
	},
	setStartAfter: function(node, offset){
		this._setPoint('setStart', node, offset, 1);
	},
	setStartBefore: function(node, offset){
		this._setPoint('setStart', node, offset, 0);
	},
	setEndAfter: function(node, offset){
		this._setPoint('setEnd', node, offset, 1);
	},
	setEndBefore: function(node, offset){
		this._setPoint('setEnd', node, offset, 0);
	},
	_setPoint: function(what, node, offset, ext){
		var index = dijit.range.getIndex(node, node.parentNode).o;
		this[what](node.parentNode, index.pop()+ext);
	},
	_getIERange: function(){
		var r = (this._body || this.endContainer.ownerDocument.body).createTextRange();
		dijit.range.ie.setRange(r, this.startContainer, this.startOffset, this.endContainer, this.endOffset, this.collapsed);
		return r;
	},
	getBookmark: function(){
		this._getIERange();
		return this._cachedBookmark;
	},
	_select: function(){
		var r = this._getIERange();
		r.select();
	},
	deleteContents: function(){
		var s = this.startContainer, r = this._getIERange();
		if(s.nodeType === 3 && !this.startOffset){
			//if the range starts at the beginning of a
			//text node, move it to before the textnode
			//to make sure the range is still valid
			//after deleteContents() finishes
			this.setStartBefore(s);
		}
		r.pasteHTML('');
		this.endContainer = this.startContainer;
		this.endOffset = this.startOffset;
		this.collapsed = true;
	},
	cloneRange: function(){
		var r = new dijit.range.W3CRange([this.startContainer,this.startOffset,
			this.endContainer,this.endOffset]);
		r._body = this._body;
		return r;
	},
	detach: function(){
		this._body = null;
		this.commonAncestorContainer = null;
		this.startContainer = null;
		this.startOffset = 0;
		this.endContainer = null;
		this.endOffset = 0;
		this.collapsed = true;
}
});
} //if(!dijit.range._w3c)


return dijit.range;
});

},
'ibm/tivoli/tip/navwidget/topology/dijit/MenuProviderHelper':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/i18n!ibm/tivoli/tip/navwidget/nls/resources","dojo/require!ibm/tivoli/tip/navwidget/topology/dijit/MenuProviderHelperBase,ibm/tivoli/tip/navwidget/dojo/NavWidgetUtil,ibm/tivoli/tip/navwidget/properties/dijit/PropertiesNotebookDialog,ibm/tivoli/navmodel/NavWidgetDef,ibm/tivoli/navmodel/Trace,dojox/layout/ResizeHandle"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.topology.dijit.MenuProviderHelper");

dojo.require("ibm.tivoli.tip.navwidget.topology.dijit.MenuProviderHelperBase");    // 256005  Moved out to its own file for Dojo 1.7
dojo.require("ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil");
dojo.require("ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebookDialog");
dojo.require("ibm.tivoli.navmodel.NavWidgetDef");
dojo.require("ibm.tivoli.navmodel.Trace");
dojo.requireLocalization("ibm.tivoli.tip.navwidget", "resources");
dojo.require("dojox.layout.ResizeHandle");

dojo.declare("ibm.tivoli.tip.navwidget.topology.dijit.MenuProviderHelper",
	      ibm.tivoli.tip.navwidget.topology.dijit.MenuProviderHelperBase,
{		
	// summary:
	//		Coming soon
	// description:
	//		Coming soon
	
	//_propDialog: [private] Object
	//		Coming soon
	_propDialog: null,

	// resizeAxis: String
	//  x | xy | y to limit pane's sizing direction.
	resizeAxis: "xy",
			
	// enableResizeHandle: boolean
	//a flag to enable/disable the resize handle, default is true
	enableResizeHandle: true,
	
	/* Richa Defect#54789 Adding the resize handle to properties dialog box. */
	setSize: function(){			
			if(this.enableResizeHandle == true){
			//Build and place resize handle       
			var resizeHandleDOM = dojo.doc.createElement( 'span' );
			resizeHandleDOM.className = "dojoxFloatingResizeHandle";    
			dojo.place( resizeHandleDOM, this._propDialog.domNode, "last" );
			this.resizeHandle = new dojox.layout.ResizeHandle(
			{ 
				targetId: this._propDialog.id, 
				resizeAxis: this.resizeAxis 
			}, resizeHandleDOM );  
			this.onResizeListener = dojo.connect( this._propDialog, "resize", this, "customLayout" );		
		}
		},

	customLayout: function() 
      {
		/* summary: Resizes the Properties Dialog. */

		var generalDiv1 = dojo.byId( this._propDialog.id + '_myProperties_GENERAL_PROPERTIES_VIEW' );
		var generalDiv = dojo.byId( this._propDialog.id + '_myProperties' );
	//	var x = this._propDialog.id + '_myProperties_tablist_'+ this._propDialog.id + '_myProperties_GENERAL_PROPERTIES_VIEW'
		var dialogSize = dojo.contentBox( this._propDialog.domNode );


    // IE 8 was causing some bogus numbers to appear when the dialog first open, if that happens ignore the caller.
    if( dialogSize.w < 0 || dialogSize.h < 0 ){
			console.log( "[customLayout] Ignoring resize request since dialogSize is negative. " );
			return;
		}
		
		var titleBarSize = dojo.marginBox( this._propDialog.titleBar );
		
		// Start collect information about the various elements of the dialog box
		var cBox = dojo.marginBox( this._propDialog.containerNode );
	    var msgBox = dojo.marginBox( generalDiv );
		var newSize = { h:0 };

		//Debug information to validate calculations or if funny things happen 
	/*    console.info( "[customLayout] =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=" );
	    console.info( "[customLayout] dialogSize [ w=" + dialogSize.w + ", h="+ dialogSize.h + "]" );
		console.info( "[customLayout] cBox [ w=" + cBox.w + ", h=" + cBox.h + "]" );
		console.info( "[customLayout] msgBox [ w=" + msgBox.w + ", h=" + msgBox.h + "]" );
		console.info( "[customLayout] titleBarSize [ w=" + titleBarSize.w + ", h=" + titleBarSize.h + "]" ); */
		
		/* We subtract the current message box size from the content box size to figure out how much margins, padding, and
		   surrounding content are taking up space so we can correctly adjust the width and height of the message description
			 based on the detla we're given by the resize event. */

		newSize.h = dialogSize.h - titleBarSize.h - (cBox.h - msgBox.h);
		
		//Set the minimum height.. calculations not working now..fixed to 100px.
		minDataDiv=('.' + this._propDialog.id + '_myProperties_GENERAL_PROPERTIES_VIEWgeneralTab' );
		minDataDiv1=('.' + this._propDialog.id + '_myProperties_GENERAL_PROPERTIES_VIEW_NAME_Name.value_id' );

		var msgId = dojo.query( minDataDiv, this._propDialog.domNode )[0]|| 
					dojo.query( minDataDiv1, this._propDialog.domNode )[0] ||null;

		if( msgId !== null ){	
		  var minHeight = dojo.marginBox( msgId );	  
		} else {
			//console.log( "[customLayout] Unable to determine the min height of a text string all plan exhasted.", msgId );
			var minHeight = { h:100 };
		}
		newSize.h = Math.max( minHeight.h, newSize.h );
		console.info("newSize.h is "+newSize.h);
		
	    dojo.contentBox( generalDiv, newSize );
		dojo.contentBox( generalDiv1, newSize );
	},

	getMenuItems: function(/*Object*/ context) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// context:
		//		Coming soon
		var items = this.inherited(arguments);
		
		// Note: Add code here to add JSON menu item objects (for our built-in actions) to items
		// Example: 
		//    items.push( { "actionID":"someUniqueID",
		//                  "label":"someLabelForTheItem",
		//                  "launchType":"NOOP" } );
		// Note: For globalization, the label will need to be loaded from a resource bundle.
		//       Also, setting launchType to NOOP is optional, but may simplify code in
		//       getMenuCommand (see below)
		
		
		//The PropertiesNotebook should not be displayed unless only a single item is selected
		//first check the context to make sure multiple items are not selected
		//checks if only one item or link
		var selectedObjects = context.selection;
		if(( selectedObjects.getItemCount() + selectedObjects.getLinkCount() + selectedObjects.getRelationshipCount()) == 1){
				
			var object = null;
			
			if(selectedObjects.getItemCount() == 1){
				object = selectedObjects.getItems()[0];            
			}else if(selectedObjects.getRelationshipCount() == 1){
				object = selectedObjects.getRelationships()[0];
			}else if(selectedObjects.getLinkCount() == 1){
				object = selectedObjects.getLinks()[0];
			}else{
				console.log("Cannot handle non link, rel, or item objects. No action preformed.");
			}
			
			//if the object doesnt support properties, dont add prop menu item.
			if(object && object.supportsProperties() ){
				//Add the PropertiesNotebook to the Menu list of items
				items.push(
					{ 
					  "actionID": "_separator_" 
					},
					{ "actionID": "nwmi_PropertiesNotebookDialog",
					  "label": ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getStringWithEllipsis(this._resources.PROPERTIES),
					  "launchType":"NOOP"
					}			
				  );	
			}
			
		} else {

			//Get Default Actions for this specific view
			var tasks = context.getViewSpecificTasks();
			if (tasks) {
				items.push(
						{ 
						  "actionID": "_separator_",
						  "launchType":"VIEW_COMMAND"
						});		
				for (var i=0;i<tasks.length;i++) {			
					items.push(
							{ "actionID": tasks[i][0],
							  "label": tasks[i][1],
						 	 "launchType":tasks[i][2]
							}			
				  		);
				}
			}		
		}	

		return items;
	},
	
	getMenuCommand: function(/*Object*/ commandName, /*Object*/ context, /*Object*/ menuItem) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// commandName:
		//		Coming soon
		// context:
		//		Coming soon
		// menuItem:
		//		Coming soon
		var result = null;
		
		// Note: add code here to check the commandName (which is also the actionID from 
		// menuItem) and if it is one of our actions, do or call whatever is needed to do 
		// the work and then set the result to a no-op command so the TIPContextMenu will 
		// take no action on the returned result
		// Ex:
		//     result = {"launchType":"NOOP"};
		//  or
		//     result = menuItem;  (if the launchType="NOOP" was already an attribute in the menu item)
		//
		
		//command name is = to actionID...so if one of the special menu items added above, do special processing
		
		// make sure we have a menu item (default action/task processing can call with just a 
		// command and no menuItem)
		if (menuItem && (menuItem.launchType == "VIEW_COMMAND")) {
			context.executeViewSpecificTask(commandName);
			result = {"launchType":"NOOP"}; 
			return result;
		}

		switch (commandName) {
			case "nwmi_PropertiesNotebookDialog":			
				//The PropertiesNotebook should not be displayed unless only a single item is selected
				//first check the context to make sure multiple items are not selected
				//checks if only one item or link
				var selectedObjects = context.selection;
				if(( selectedObjects.getRelationshipCount() + selectedObjects.getLinkCount() + selectedObjects.getItemCount() ) != 1){
					//if there is more than one item then just return and do nothing
					result = menuItem;
					return result;
				}
				
				//Try to look up nav widget def and get widget definition for props notebook
				var dialogWidgetDefinitionId = null;
				var topLevelPluggableViewWidgetDefinition = context.getTopLevelPluggableViewWidgetDefinition();
				if (topLevelPluggableViewWidgetDefinition != null && topLevelPluggableViewWidgetDefinition.isInstanceOf(ibm.tivoli.navmodel.NavWidgetDef)) {
					var notebook = topLevelPluggableViewWidgetDefinition.getNotebook();
					if (notebook) {
						dialogWidgetDefinitionId = notebook.definition;
					}
				}
			
				// if there is one item then create the PropertiesNotebookDialog
                this._propDialog = new ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebookDialog({
                    id: context.id + "_PropertiesDialog",
                     //Richa defect #54789: Removing the tundra class setting
                    //className: "tundra",
                  //Richa defect #54789: adding class to override the styles
                    className: "propertiesNotebook",
                  //Richa : Min width of 400px was not being set because of this and hence the scrollbar was hidden
                   // style: "width:40%",
                    managingPluggableView: context,
                    widgetDefinitionId: dialogWidgetDefinitionId
                });
              
                this._propDialog.setDataset(context.dataset);
                
                if(selectedObjects.getItemCount() == 1){
                	var item = selectedObjects.getItems()[0];
                	//set the object on the dialog (hardcoded to 0 since we checked there is only one item
                    this._propDialog.setObjects([item]);
                    
                }else if(selectedObjects.getRelationshipCount() == 1){
                	var rel = selectedObjects.getRelationships()[0];
                	//set the object on the dialog (hardcoded to 0 since we checked there is only one item
                    this._propDialog.setObjects([],[],[rel]);
                }else if(selectedObjects.getLinkCount() == 1) {
                	var link = selectedObjects.getLinks()[0];
                	//set the object on the dialog (hardcoded to 0 since we checked there is only one item
                    this._propDialog.setObjects([],[link]);
                }else{
                	console.log("Properties dialog does not render anything other than links, items, or relationships.");
                }
                this.setSize();
                //display the dialog
                this._propDialog.startup();		
                this._propDialog.show();	
               // dojo.connect(this._propDialog,"onCancel" ,dojo.hitch(this, this._cleanupDialog));
				
                //done return
				result = menuItem;
				return result;
			
			default:
				// Note: If this is NOT one of our client-side actions, DO NOT set result and let
				// the parent class command be returned
				return (result) ? result : this.inherited(arguments);
			
		}
	}
	
});
});

},
'dojox/grid/cells/tree':function(){
define([
	"dojo/_base/kernel",
	"../../main",
	"dojo/_base/lang",
	"../cells"
], function(dojo, dojox, lang){

dojox.grid.cells.TreeCell = {
	formatAggregate: function(inItem, level, inRowIndexes){
		var f, g=this.grid, i=g.edit.info,
			d=g.aggregator ? g.aggregator.getForCell(this, level, inItem, level === this.level ? "cnt" : this.parentCell.aggregate) : (this.value || this.defaultValue);
		return this._defaultFormat(d, [d, level - this.level, inRowIndexes, this]);
	},
	formatIndexes: function(inRowIndexes, inItem){
		var f, g=this.grid, i=g.edit.info,
			d=this.get ? this.get(inRowIndexes[0], inItem, inRowIndexes) : (this.value || this.defaultValue);
		if(this.editable && (this.alwaysEditing || (i.rowIndex==inRowIndexes[0] && i.cell==this))){
			return this.formatEditing(d, inRowIndexes[0], inRowIndexes);
		}else{
			return this._defaultFormat(d, [d, inRowIndexes[0], inRowIndexes, this]);
		}
	},
	getOpenState: function(itemId){
		var grid = this.grid, store = grid.store, itm = null;
		if(store.isItem(itemId)){
			itm = itemId;
			itemId = store.getIdentity(itemId);
		}
		if(!this.openStates){ this.openStates = {}; }
		if(typeof itemId != "string" || !(itemId in this.openStates)){
			this.openStates[itemId] = grid.getDefaultOpenState(this, itm);
		}
		return this.openStates[itemId];
	},
	formatAtLevel: function(inRowIndexes, inItem, level, summaryRow, toggleClass, cellClasses){
		if(!lang.isArray(inRowIndexes)){
			inRowIndexes = [inRowIndexes];
		}
		var result = "";
		if(level > this.level || (level === this.level && summaryRow)){
			cellClasses.push("dojoxGridSpacerCell");
			if(level === this.level){
				cellClasses.push("dojoxGridTotalCell");
			}
			result = '<span></span>';
		}else if(level < this.level){
			cellClasses.push("dojoxGridSummaryCell");
			result = '<span class="dojoxGridSummarySpan">' + this.formatAggregate(inItem, level, inRowIndexes) + '</span>';
		}else{
			var ret = "";
			if(this.isCollapsable){
				var store = this.grid.store, id = "";
				if(store.isItem(inItem)){
					id = store.getIdentity(inItem);
				}
				cellClasses.push("dojoxGridExpandoCell");
				ret = '<span ' + dojo._scopeName + 'Type="dojox.grid._Expando" level="' + level + '" class="dojoxGridExpando"' +
						'" toggleClass="' + toggleClass + '" itemId="' + id + '" cellIdx="' + this.index + '"></span>';
			}
			result = ret + this.formatIndexes(inRowIndexes, inItem);
		}

		if(this.grid.focus.cell && this.index == this.grid.focus.cell.index &&
			inRowIndexes.join('/') == this.grid.focus.rowIndex){
			cellClasses.push(this.grid.focus.focusClass);
		}

		return result;
	}
};

return dojox.grid.cells.TreeCell;

});
},
'ibm/usmi/widgets/util/DefaultContextMenuProvider':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/widgets/util/DefaultContextMenuProvider", ["dojo/_base/kernel",
        "dojo/_base/declare",
        // TWL imports
        "ibm/tivoli/tip/navwidget/dijit/RESTContextMenuProvider",
        "ibm/tivoli/navmodel/Selection",
        "ibm/usmi/widgets/util/Constants"],
    function (dojo, declare, RESTContextMenuProvider, Selection, Constants) {

    return declare([RESTContextMenuProvider], {

        // summary:
        //        Coming soon
        // description:
        //        Coming soon

        // _handle:
        //
        _handle: null,

        filterNavigationTasks: function(/*ibm.tivoli.navmodel.Task[]*/ tasks) {
            // summary:
            //    Return a filtered collection of tasks
            //    Filter out related resources (parent has id RELATED_RESOURCES,
            //       children have id RELATIONSHIP:*
            //    Filter out related resources (parent has id PERSPECTIVES,
            //       children have id PERSPECTIVE:*
            //    Note:  Separators have no task id.
            //
            var filteredTasks = [];
            for (var i = 0; i < tasks.length; i++) {
                if ((tasks[i].getId() === undefined) ||
                    ((tasks[i].getId() !== "RELATED_RESOURCES") &&
                    (tasks[i].getId().indexOf("RELATIONSHIP:") !== 0) &&
                    (tasks[i].getId() !== "PERSPECTIVES") &&
                    (tasks[i].getId().indexOf("PERSPECTIVE:") !== 0) &&
                    (tasks[i].getId() !== "VIEW_MEMBERS"))) {
                    filteredTasks.push(tasks[i]);
                }
            } 
            return filteredTasks;
        },

        handleNavigation: function(data, ioArgs) {
            // summary:
            //        Coming soon
            // description:
            //        Coming soon
            // data:
            //
            // ioArgs:
            //

            // listen for event indicating new dataset is loaded and dismiss the busy pane
            this._handle = dojo.connect(this.view, Constants.SET_DATASET, this, function(dataset) {
                                                                                //this.view.hideBusyPane();
                                                                                dojo.disconnect(this._handle);});
        //    this.view.showBusyPane();
            this.view.getCurrentDataset().navigate(
                    {onComplete: dojo.hitch(this, this._processNavNewDataSet),
                    onError: dojo.hitch(this, this._handleNavNewDataSetError),
                    taskId: data.actionID,
                    taskCategory: data.category,
                    selection: this.buildClientNavModelSelection(data, this.view.dataset.getVersion())});
        },

        _processNavNewDataSet: function(dataset, kwArgs) {
            // summary:
            //        Coming soon
            // description:
            //        Coming soon
            // dataset:
            //
            // kwArgs:
            //
            this.view.addDataset(dataset);
        //    this.view.hideBusyPane();
        },

        _handleNavNewDataSetError: function(error, exception, ioArgs) {
            // summary:
            //        Coming soon
            // description:
            //        Coming soon
            // error:
            //
            // exception:
            //
            // ioArgs:
            //

            // in case of an error, dismiss the busy pane
    //        this.view.hideBusyPane();
            dojo.disconnect(this._handle);

            //Give the user some feedback
            this.view.displayException(error);
        }

    });
});
},
'url:dijit/layout/templates/_TabButton.html':"<div role=\"presentation\" data-dojo-attach-point=\"titleNode\" data-dojo-attach-event='onclick:onClick'>\n    <div role=\"presentation\" class='dijitTabInnerDiv' data-dojo-attach-point='innerDiv'>\n        <div role=\"presentation\" class='dijitTabContent' data-dojo-attach-point='tabContent'>\n        \t<div role=\"presentation\" data-dojo-attach-point='focusNode'>\n\t\t        <img src=\"${_blankGif}\" alt=\"\" class=\"dijitIcon dijitTabButtonIcon\" data-dojo-attach-point='iconNode' />\n\t\t        <span data-dojo-attach-point='containerNode' class='tabLabel'></span>\n\t\t        <span class=\"dijitInline dijitTabCloseButton dijitTabCloseIcon\" data-dojo-attach-point='closeNode'\n\t\t        \t\tdata-dojo-attach-event='onclick: onClickCloseButton' role=\"presentation\">\n\t\t            <span data-dojo-attach-point='closeText' class='dijitTabCloseText'>[x]</span\n\t\t        ></span>\n\t\t\t</div>\n        </div>\n    </div>\n</div>\n",
'dojoe/progressbar/EnhancedProgressBar':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/i18n!dojoe/progressbar/nls/EnhancedProgressBar","dojo/require!dijit/_Widget,dijit/_Templated,dijit/ProgressBar,dijit/form/Button,dijit/Dialog,dojo/parser,dojoe/messagedialog/MessageDialog,dojo/i18n"], function(dijit,dojo,dojox){
dojo.provide("dojoe.progressbar.EnhancedProgressBar" );

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.ProgressBar"); 
dojo.require("dijit.form.Button");
dojo.require("dijit.Dialog");
dojo.require("dojo.parser");
dojo.require("dojoe.messagedialog.MessageDialog");
dojo.require("dojo.i18n");

/******************************************************************************
 * Busy Pane that blocks all user input and shows progress message.
 ******************************************************************************/
//Localization of resource bundles
dojo.requireLocalization("dojoe.progressbar", "EnhancedProgressBar");

dojo.declare("dojoe.progressbar.EnhancedProgressBar", [dijit._Widget, dijit._Templated],
		{
	// summary:  EnhancedProgressBar is a widget to show progress in determinate(indicates progress) or indeterminate way .
	// It can also be used as a standby or busy pane
	// description: EnhancedProgressBar widget uses the dijit.ProgressBar for its implementation.
	//		  EnhancedProgressBar has also been customized with multiple default actions .

	//	processingImageSrc:String
	//  Path to the processing image
	processingImageSrc: dojo.moduleUrl("dojoe.common","themes/images/progress32.gif"), 

	// _resources:  Object
	//	Reference to the resources
	resources_:null,
	//  progressBar:Object
	//  refrence to the progressbar  
	progressBar: null,

	// cancellable: Boolean
	//Set this to true if cancel button is needed in progressbar	
	cancellable:false,	

	//pauseable: Boolean
	//Set this to true if pause/resume toggle button is needed in progressbar	
	pauseable:false,

	//closeable: Boolean
	//Set this to true if close button is needed in progressbar	
	closeable:false,  

	//  progressIndicatorType:number
	//  indicates the type of progressbar.It can be set to  0:indeterminate or 1:determinant  
	progressIndicatorType : 0,

	//showmessagetext:Boolean
	//Set this to true to enable message from the user	
	showmessagetext:true,

	//processingLabel:String
	//Label to be shown when progress starts
	processingLabel:'Processing',

	//hidden:boolean
	//Set to true if the progressbar needs to be hidden once progress completes
	hidden:false,

	//maximumValue:number
	// maximum progress value
	maximumValue : 100,

	//progress:number
	//Holds the progress set by the user operation
	progress  : 0,

	//width: number
	//Holds the width of the progressbar set by user. It can be set in px,percentage or em like '150px','150%' or '150em'.
	width : "150px",

	//height: number
	//Holds the height of the progressbar set by user. It can be set in px,percentage or em like '150px','150%' or '150em'.
	height : "50px",

	//size:String
	//Holds the size of the processing image,it can be set to small,medium,large,huge
	size:"large",

	/*#36265 TED: EnhancedProgressBar: unable to create instance via declarative declaration */
	//Commenting this as id attribute comes with dijit progressbar by default and giving problem while parsing the declarative markup. 
	//id:  String
	// id of the widget
	//id:null, 

	//contentIsolation:  Boolean
	// Flag to indicate if sample is isolated
	contentIsolation:false,


	//namespace:  String
	// namespace for messageDialog in case of content isolation
	namespace:null,

	// target: DOMNode||DOMID(String)||WidgetID(String)
	//		The target to overlay when active.  Can be a widget id, a
	//		dom id, or a direct node reference.
	target: "",

	// _resizeCheck: [private] Object
	//		Handle to interval function that checks the target for changes.
	_resizeCheck: null,

	// zIndex: String
	//		Control that lets you specify if the zIndex for the overlay
	//		should be auto-computed based off parent zIndex, or should be set
	//		to a particular value.  This is useful when you want to overlay
	//		things in digit.Dialogs, you can specify a base zIndex to append from.
	zIndex: "auto",
	
	// _underlayNode: [private] DOMNode
	//		The node that is the translucent underlay for the
	//		image that blocks access to the target.
	_underlayNode: null,

	// color:	String
	//		The color to set the overlay.  Should be in #XXXXXX form.
	//		Default color for the translucent overlay is light gray.
	color: "#C0C0C0",
	
	// duration: integer
	//		Integer defining how long the show and hide effects should take.
	duration: 500,
	
	// _displayed: [private] Boolean
	//		Indicates whether EPB is displayed or not
	_displayed : false,

	// _imageSize: String
	//		Indicates the sixe of the progress image to be used
	_imageSize : null,
	
	//templateString:  String
	// Path to the template
	templateString:dojo.cache("dojoe.progressbar", "templates/EnhancedProgressBar.html", "\r\n<div class=\"enhanced t\" style=\"overflow:auto;\">\r\n<div style=\"display: none; opacity: 0; z-index: 9999; position: absolute;\" dojoAttachPoint=\"_underlayNode\"></div>\r\n<div   dojoAttachPoint='rootDiv' style=\"overflow:auto;;position:absolute;display:none;\" >\r\n\t\r\n\t<table  dojoAttachPoint='progressBarTable' cellspacing='0' cellpadding='0' frame='border' style=\"width:100%;overflow:auto\" role=\"region\">\r\n\t         \r\n\t\t\t\t  <tr class='EnhancedProgressBarProcessing' style=\"overflow:auto;\" >\r\n\t\t\t\t    \r\n\t\t\t\t\t   <td class='process-anim'  width='10%'> \r\n\t\t\t\t\t   <!-- 23Aug12 : Updated to read the image from css -->\r\n\t\t\t\t<!--\t   <img src='${processingImageSrc}'role=\"img\" alt='${processingLabel}' title='${processingLabel}'/>  -->\r\n\t\t\t\t\t\t\t<div dojoAttachPoint=\"progressImage\" alt='${processingLabel}' title='${processingLabel}' ></td>\r\n\t\t\t\t\t   <td width=\"x-small;overflow:auto;\"> \r\n\t\t\t\t\t      \r\n\t\t\t\t\t\t\t <table width=\"100%\" height=\"70%\" role=\"presentation\">\r\n\t\t\t\t\t\t\t\t<tr>\r\n\t\t\t\t\t\t\t\t\t<td width=\"40%\" class='process-message' dojoAttachPoint='messageArea'>\r\n\t\t\t\t\t\t\t\t       <div dojoAttachPoint='msgareaDiv' role=\"log\">${processingLabel}</div>\r\n\t\t\t\t\t\t\t\t    </td> \r\n\t\t\t\t\t\t\t\t</tr>\r\n\t\t\t\t\t\t\t\t<tr>\r\n\t\t\t\t\t\t\t\t\t<td width=\"x-small\"  >\r\n\t\t\t\t\t\t\t    \t<div dojoAttachPoint='progressBarNode' role=\"progressbar\"></div>\r\n\t\t\t\t\t\t\t\t\t</td> \r\n\t\t\t\t\t\t\t\t\r\n\t\t\t\t\t\t\t\t\t<td width=\"10%\"> \r\n\t\t\t\t\t\t\t\t\t</td>\r\n\t\t\t\t\t\t\t\t</tr>\r\n\t\t\t\t\t\t\t\t<tr> \r\n\t\t\t\t\t\t\t\t\t<td width=\"100%\" colspan=\"2\"> \r\n\t\t\t\t\t\t\t\t\t<table>\r\n\t\t\t\t\t\t\t\t\t\t<tr>\r\n\t\t\t\t\t\t\t\t\t\t\t<td>\r\n\t\t\t\t\t\t\t\t\t\t\t\t<div dojoAttachPoint='progressBarPause' style=\"overflow:auto;\" role=\"button\"> </div>\r\n\t\t\t\t\t\t\t\t\t\t\t</td>\r\n\t\t\t\t\t\t\t\t\t\t\t<td>\r\n\t\t\t\t\t\t\t\t\t\t\t\t<div dojoAttachPoint='progressBarCancel' style=\"overflow:auto;\" role=\"button\"> </div>\t\t\t\t\t\t\t\t\t\t\t\t\r\n\t\t\t\t\t\t\t\t\t\t\t</td>\r\n\t\t\t\t\t\t\t\t\t\t\t<td> \r\n\t\t\t\t\t\t\t\t\t\t\t<div dojoAttachPoint='progressBarClose' style=\"overflow:auto;\" role=\"button\"> </div>\r\n\t\t\t\t\t\t\t\t\t\t\t</td>\r\n\t\t\t\t\t\t\t\t\t\t\t\r\n\t\t\t\t\t\t\t\t\t\t</tr>\r\n\t\t\t\t\t\t\t\t\t</table>\r\n\t\t\t\t\t\t\t\t\t</td>\r\n\t\t\t\t\t\t\t\t</tr>\t \r\n\t\t\t\t\t\t\t </table>\r\n\t\t\t\t\t\t\t\r\n\t\t\t\t\t\t</td> \r\n\t\t\t\t\t\t\r\n\t\t\t\t\t</tr>\r\n\t\t\t  \r\n\t\t</table>\r\n\t\t\r\n\t\t\r\n\t</div>\r\n</div>"),

	preamble: function(){  //Called before Constructor
		//23Aug12:Richa ,Commented to move the images out of js file
		//Only 2 image sizes will be supported, medium and large.
	/*	if(arguments[0].size=="small")
			this.processingImageSrc=dojo.moduleUrl("dojoe.common","themes/images/progress_22_DARK.gif");
		else if(arguments[0].size=="medium")
			this.processingImageSrc= dojo.moduleUrl("dojoe.common","themes/images/progress_22_DARK.gif");
		else if(arguments[0].size=="large")
			this.processingImageSrc= dojo.moduleUrl("dojoe.common","themes/images/progress_32_DARK.gif");
		else if(arguments[0].size=="huge")
			this.processingImageSrc= dojo.moduleUrl("dojoe.common","themes/images/progress_32_DARK.gif");*/
		
		//24Aug12 :Richa Set the image size based on the argument passed
		if(arguments[0].size=="small" || arguments[0].size=="medium"){
			this._imageSize="medium";
			}
		else if(arguments[0].size=="large" ||arguments[0].size=="huge" ){
			this._imageSize="large";
		}
		
		if(arguments[0].cancellable !==null && arguments[0].cancellable== true){
			this.cancelButton=true;	
		}				
		else
			this.cancelButton=false;	
	
		if(arguments[0].closeable !==null && arguments[0].closeable== true)
		{
	
			this.closeButton=true;
		}
		else
			this.closeButton=false;	
	
		if(arguments[0].pauseable !==null && arguments[0].pauseable== true)
		{
	
			this.pauseButton=true;
		}
		else
			this.pauseButton=false;	

	},

	constructor: function () {
		// resources_:  Object
		//Reference to the resources
		this.resources_ = dojo.i18n.getLocalization("dojoe.progressbar", "EnhancedProgressBar",this.lang); 
	},

	postCreate: function()
	{
		// summary:  Creates and initialises the progressbar with actions like close,cancel,pause/resume.
		// tags: public
	
		//24Aug12 -Richa:Set the  class based on the imagesize
		if(this._imageSize=="medium"){
			dojo.addClass( this.progressImage,"medium");
			}
		else if(this._imageSize=="large"){
			dojo.addClass( this.progressImage,"large");
			}

		
		this.inherited( arguments );
		var size = dijit.getViewport();
		this._allProgressBarButtons = [];
		var height = this.height;
		var sizeType = this.height.substring(this.height.length-1, this.height.length);
		if(sizeType == "%") {
			//convert to px
			var height = size.h * ((parseInt(this.height.slice(0, -1))) / 100);
			height = height + "px";
		}
		if(this.progressIndicatorType==1) {	
	
			this.progressBar = new dijit.ProgressBar({id:this.id+"progressBar",
				width:this.width ,
				height:this.height, 	
				report:function(percent){	
				return '';
			}},this.progressBarNode);
		}
	
		if(this.cancelButton ==true) {	
	
			this.cancelButton = new dijit.form.Button({
				id: this.id+"Cancel",
				showLabel: true,						
				label: this.resources_.CANCEL
			},this.progressBarCancel);
	
			this.cancelButton.onClick = dojo.hitch (this, "_onCancelClick");
			this._allProgressBarButtons.push( this.cancelButton );
	
	
		}
		if(this.closeButton ==true) {	 
			this.closeButton = new dijit.form.Button({
				id: this.id+"Close",
				showLabel: true,						
				label: this.resources_.CLOSE
			},this.progressBarClose);
	
			this.closeButton.onClick = dojo.hitch (this, "_onCloseClick");		
			this._allProgressBarButtons.push( this.closeButton );
		}
		if(this.pauseButton ==true) {
			if(this.progress !== this.maximumValue){
				this.pauseButton=new dijit.form.ToggleButton({
					id: this.id+"Toggle",
					ShowLabel: true,
					checked: false,
					label: this.resources_.PAUSE
				},this.progressBarPause);
				this.pauseButton.onChange = dojo.hitch (this, "_onToggleClick");		
				this._allProgressBarButtons.push( this.pauseButton );
			}
		}
	
		this.calculateMsgWidth();
	
		/*# 40363: Enhanced Progressbar need "modal" option to be compatible with previous LDTW's BusyPane 
		 * set the target to default dom node of this widget
		 * This widget should function just as a progress bar attached to the given node and 
		 * also as a busy pane which centers itself for the target node...*/
		if(this.target == "" || this.target == null){
			this.target = this.rootDiv;
		}else{
		
			dojo.style(this._underlayNode, {
						display: "none",
						backgroundColor: this.color
					});
					
			this.connect(this._underlayNode, "onclick", "_ignore");
		}
		/*#36265 TED: EnhancedProgressBar: unable to create instance via declarative declaration */
		//the line below is causing declarative sample not to work
		//this.hide();
	},
	_ignore: function(e){
		// summary:
		//		Function to ignore events that occur on the overlay.
		// event: Event
		//		The event to halt
		// tags:
		//		private
		if(e){
			console.log('here in onclick of underlay');
			dojo.stopEvent(e);
		}
	},
	_fadeIn: function(){
		// summary:
		//		Internal function that does the opacity style fade in animation.
		// tags:
		//		private
		var self = this;
		var underlayNodeAnim = dojo.animateProperty({
			duration: self.duration,
			node: self._underlayNode,
			properties: {opacity: {start: 0, end: 0.75}}
		});
		
		this._anim = underlayNodeAnim;
		this._anim.play();
	},

	_fadeOut: function(){
		// summary:
		//		Internal function that does the opacity style fade out animation.
		// tags:
		//		private
		var self = this;
		var underlayNodeAnim = dojo.animateProperty({
			duration: self.duration,
			node: self._underlayNode,
			properties: {opacity: {start: 0.75, end: 0}},
			onEnd: function(){
				dojo.style(this.node,{"display":"none", "zIndex": "-1000"});
			}
		});
		
		this._anim = underlayNodeAnim;
		this._anim.play();
	},
	_onToggleClick: function(/* Event */e){  
	
		// summary:  indicates pause/resume state of the progressbar and calls appropriate functions.
		//Also published events onPauseClick and onResumeClick.
		// tags: private			
		if(e==true)
		{
			this.pauseButton.set('label', this.resources_.RESUME);	
	
			dojo.publish(this.id +"#onPauseClick", ["Paused"]);
			this.progressPaused=true;
		}
		else
		{
			this.pauseButton.set('label', this.resources_.PAUSE);	
	
			dojo.publish(this.id +"#onResumeClick", ["Resumed"]);
			this.progressPaused=false;
		}
	
	},
	_onCloseClick: function(/* Event */e){  
		// summary:  indicates closed state of the progressbar and publish event onCloseClick.
		// tags: private	
		this.closed=true;
		dojo.publish(this.id +"#onCloseClick", ["Closed"]);
		this.hide();
		setTimeout( dojo.hitch( this, "destroy" ), 500 );	
	},

	calculateMsgWidth :function() {
		// summary:  calculate the width of the message set by the user.
		// tags: public
		var strWidth = this.width.substring(0,this.width.length-2); // get numeric width 
		var totalWidth = parseInt(strWidth);
		var animwidth = .1 * totalWidth;
		var animwidth =animwidth+5;
		var	msgWidthtemp = totalWidth-(animwidth+20);
		this.msgWidth=msgWidthtemp+"px";
	
	},

	doProgress : function(progress,message) {
		//summary: Trigger for start displaying the progress
		//tags :public
		if( this.closed!=true){
			this.show();	
			this.getProgress(progress,message);
		}
	},	

	getProgress :function (progress,message) {
		//summary: Trigger for start displaying the progress.
		//tags :public
		this.progressCallBack(progress,message);
	
	},

	progressCallBack : function  (progress,message){
		//summary: Method which shows progress
		//tags :public
	
		if( progress < this.maximumValue ) {  //  Progress% is less than 100 and progress is not cancelled
			var y = parseInt(progress);
			if (!isNaN(y)) {
				this.progress = y;
				if(this.progressIndicatorType==1 && this.progressCancelled!=true && this.progressPaused!=true ) {			      
					this.progressBar.update({ maximum:this.maximumValue, progress: this.progress });  //Update the progressbar
					if(this.showmessagetext==true)
						this.setMessage(message);
					else
						this.setMessage("");
	
				}else if(this.progressCancelled==true){
	
					this.hide();
					this.cancelButton.disabled=true;
					this.pauseButton.disabled=true;
				}
			}
	
		}
		else if(progress=this.maximumValue) { 
			// Progress Process is completed
			if(	this.progressIndicatorType==1){	 
				this.progressBar.update({ maximum:this.maximumValue, progress: this.progress });		
	
				if(this.showmessagetext==true)
				{
					this.setMessage(message);
				}
				else
				{
					this.setMessage("");
				}}
			var callback = dojo.hitch(this, this.afterCompletion);
	
			setTimeout(callback,1000);
		}
	},

	afterCompletion : function () {
		//summary: Called after the progress completes,hides the progressbar,if hidden enabled,disables the cancel and pause/resume button.
		//tags :public
		if(this.hidden==true){
			this.hide();
		}
		//Reset Progress Data	 
	
		this.cancelButton.disabled=true;
		this.pauseButton.disabled=true;
	
		setTimeout( dojo.hitch( this, "destroyRecursive" ), 5000 );
	
	},

	updateProgress : function(progressParam)
	{
		//summary: updates the progress of determinate progressbar
		//tags :public
	
		if(this.progressIndicatorType==1) { //Updates Progress Only If type is Determinate otherwise ignore	
			this.progress=progressParam;		
			this.progressBar.update({ maximum:this.maximumValue, progress: this.progress });
	
		}
	},


	onCancelClick: function(/* Event */e){
		//summary: Cancels the operation once Ok is clicked in the confirmation dialog ,also publishes onCancelClick.
		//tags :public
		if(e==0){
			if(this.cancellable==true ) {	
	
				var callback = dojo.hitch(this, this.afterCompletion);
				if(this.contentIsolation==true){
					this.successfulDialog = new dojoe.messagedialog.MessageDialog(
							{message: this.resources_.SUCCESSFULCANCEL_MESSAGE,
								messageId: "Success",
								type: "Info",	
								namespace	:  this.namespace,
								showCheckbox: false,
								contentIsolation:true});
				}else{
					this.successfulDialog = new dojoe.messagedialog.MessageDialog(
							{message: this.resources_.SUCCESSFULCANCEL_MESSAGE,
								messageId: "Success",
								type: "Info",					 
								showCheckbox: false});
				}
				this.successfulDialog.show();	
	
				dojo.publish(this.id +"#onCancelClick", ["Cancelled"]);
				this.progressCancelled=true;			
				setTimeout(callback,1000);
	
			}   
		}
	},

	_onCancelClick: function(/* Event */e){
		//summary: Pops up the confirmation dialog once cancel button is clicked,also publishes onCancelClick.
		//tags :private 
	
		if(this.contentIsolation==true){
			this.msgDlg = new dojoe.messagedialog.MessageDialog(
					{
						message: this.resources_.CANCEL_MESSAGE,
						messageId: "Confirmation",
						type: "Confirm",
						namespace	:  this.namespace,
						callback:  dojo.hitch(this, this.onCancelClick) ,
						buttons: [this.resources_.CONFIRM_OK, this.resources_.CONFIRM_CANCEL],
						showCheckbox: false,
						contentIsolation:true
					});
	
		}else{
			this.msgDlg = new dojoe.messagedialog.MessageDialog(
					{
						message: this.resources_.CANCEL_MESSAGE,
						messageId: "Confirmation",
						type: "Confirm",			 
						callback:  dojo.hitch(this, this.onCancelClick) ,
						buttons: [this.resources_.CONFIRM_OK, this.resources_.CONFIRM_CANCEL],
						showCheckbox: false
	
					});
	
	
		}
	
		this.msgDlg.show();
	},

	setMessage:function(msg){
		//summary: Sets the message generated by user operation.
		//tags :public
		this.messageArea.innerHTML=msg;
	},
	
	getProgressIndicatorType : function()
	{
		//summary:Retuns the type of progressIndicator 0 if indeterminate 1 if determinant
		//tags:public
		return this.progressIndicatorType;
	},
	/*  setProgressIndicatorType : function(type)
	  {
	    this.progressIndicatorType=type;
	  },*/
	getWidth : function()
	{
		//summary:returns the width of the progressbar.
		//tags:public
		return this.width;
	},
	
	setWidth : function(widthParam)
	{
		//summary:sets the width of the progressbar
		//tags:public
		this.width=widthParam;
		this.stylePage();
	},
	
	getHeight  : function()
	{
		//summary:returns the height of the progressbar
		//tags:public
		return this.height;
	},
	
	setHeight : function(heightParam)
	{
		//summary:sets the height of the progressbar
		//tags:public
		this.height=heightParam;
		this.stylePage();
	},
	
	stylePage: function () {
		//summary:styles the progressbar by setting height,width,message width
		//tags:public
		if(this.rootDiv){
			dojo.style (this.rootDiv, "align", "center");
			//dojo.style (this.progressBarTable, "width", this.width);
			dojo.style (this.progressBarTable, "height", this.height);  
			dojo.style (this.rootDiv, "width", this.width);
			var msgWidth = this.calculateMsgWidth();
			//dojo.style (this.msgareaDiv, "width", this.msgWidth); 
		}
	},
	
	show: function(  )
	{
		//summary:display the progressbar
		//tags:public
		this.stylePage();
		if(this.target == this.rootDiv){
			this.rootDiv.style.display="block";
		}
		else if(!this._displayed){
			this.domNode.style.display="block";
			this._displayed = true;
			if(this.target != this.rooDiv){
				this._size();
				this._fadeIn();
			}
		}
		//this.rootDiv.style.display="";
	},

	hide: function() {
		//summary:hides the progressbar
		//tags:public
		if(this.domNode){
			this._displayed = false;
			if(this.target != this.rooDiv){
				this._fadeOut();
			}
			this.domNode.style.display="none";
			
		}
		//this.rootDiv.style.display="none";
	},

	_size: function(){
		// summary:
		//		Internal function that handles centering of the progressbar on window resizing
		//		and when the target node is specified
		//		It also sizes and styles the underlay node to block/overlay the target node...
		// tags:
		//		private
		
		/*# 40363: Enhanced Progressbar need "modal" option to be compatible with previous LDTW's BusyPane 
		 * A new function to handle the centering of the progress bar when target is specified..*/
		if(this._displayed) {
			var dir = dojo.body().dir;
			if(dir){dir = dir.toLowerCase();}
			var _ie7zoom;
			var scrollers = this._scrollerWidths();
			var target = this.target;
	
			var box = (target == this.domNode)?null:dojo.position(target, true);
	
			if(target === dojo.body() || target === dojo.doc){
				// Target is the whole doc, so scale to viewport.
				box = dojo.window.getBox();
				box.x = box.l;
				box.y = box.t;
			}
			var cntrIndicator = dojo.getMarginBox(this.rootDiv);
	
			//IE has a horrible zoom bug.  So, we have to try and account for
			//it and fix up the scaling.
			if(this._ieFixNode){
				_ie7zoom = -this._ieFixNode.offsetTop / 1000;
				box.x = Math.floor((box.x + 0.9) / _ie7zoom);
				box.y = Math.floor((box.y + 0.9) / _ie7zoom);
				box.w = Math.floor((box.w + 0.9) / _ie7zoom);
				box.h = Math.floor((box.h + 0.9) / _ie7zoom);
			}
	
			//Figure out how to zIndex this thing over the target.
			var zi = dojo.getAttr(target, "zIndex");
			var ziIn = zi;
			var ziUl = zi;
	
			if(this.zIndex === "auto"){
				if(zi != "auto" && zi != null){
					ziUl = parseInt(ziUl, 10) + 1;
					ziIn = parseInt(ziIn, 10) + 2;
				}else{
					//We need to search up the chain to see if there
					//are any parent zIndexs to overlay.
					var cNode = target.parentNode;
					var oldZi = -100000;
					while(cNode && cNode !== dojo.body()){
						zi = dojo.getAttr(cNode, "zIndex");
						if(!zi || zi === "auto"){
							cNode = cNode.parentNode;
						}else{
							var newZi = parseInt(zi, 10);
							if(oldZi < newZi){
								oldZi = newZi;
								ziUl = newZi + 1;
								ziIn = newZi + 2;
							}
							// Keep looking until we run out, we want the highest zIndex.
							cNode = cNode.parentNode;
						}
					}
					if(ziUl == null){
						ziUl = 9999;
						ziIn = ziUl + 2;
					}
				}
			}else{
				ziUl = parseInt(this.zIndex, 10) + 1;
				ziIn = parseInt(this.zIndex, 10) + 2;
			}
	
			dojo.style(this._underlayNode, "zIndex", ziUl);
			dojo.style(this.rootDiv, "zIndex", ziIn);
	
	
			var pn = target.parentNode;
			if(pn && pn !== dojo.body() &&
					target !== dojo.body() &&
					target !== dojo.doc && target !== this.domNode){
	
				// If the parent is the body tag itself,
				// we can avoid all this, the body takes
				// care of overflow.  Besides, browser
				// weirdness with height and width on body causes
				// problems with this sort of intersect testing
				// anyway.
				var obh = box.h;
				var obw = box.w;
				var pnBox = dojo.position(pn, true);
	
				//More IE zoom corrections.
				if(this._ieFixNode){
					_ie7zoom = -this._ieFixNode.offsetTop / 1000;
					pnBox.x = Math.floor((pnBox.x + 0.9) / _ie7zoom);
					pnBox.y = Math.floor((pnBox.y + 0.9) / _ie7zoom);
					pnBox.w = Math.floor((pnBox.w + 0.9) / _ie7zoom);
					pnBox.h = Math.floor((pnBox.h + 0.9) / _ie7zoom);
				}
	
				//Shift the parent width/height a bit if scollers are present.
				pnBox.w -= pn.scrollHeight > pn.clientHeight &&
				pn.clientHeight > 0 ? scrollers.v: 0;
				pnBox.h -= pn.scrollWidth > pn.clientWidth &&
				pn.clientWidth > 0 ? scrollers.h: 0;
	
				//RTL requires a bit of massaging in some cases
				//(and differently depending on browser, ugh!)
				//WebKit and others still need work.
				if(dir === "rtl"){
					if(dojo.isOpera){
						box.x += pn.scrollHeight > pn.clientHeight &&
						pn.clientHeight > 0 ? scrollers.v: 0;
						pnBox.x += pn.scrollHeight > pn.clientHeight &&
						pn.clientHeight > 0 ? scrollers.v: 0;
					}else if(dojo.isIE){
						pnBox.x += pn.scrollHeight > pn.clientHeight &&
						pn.clientHeight > 0 ? scrollers.v: 0;
					}else if(dojo.isWebKit){
						//TODO:  FIX THIS!
					}
				}
	
				//Figure out if we need to adjust the overlay to fit a viewable
				//area, then resize it, we saved the original height/width above.
				//This is causing issues on IE.  Argh!
				if(pnBox.w < box.w){
					//Scale down the width if necessary.
					box.w = box.w - pnBox.w;
				}
				if(pnBox.h < box.h){
					//Scale down the width if necessary.
					box.h = box.h - pnBox.h;
				}
				//Look at the y positions and see if we intersect with the
				//viewport borders.  Will have to do computations off it.
				var vpTop = pnBox.y;
				var vpBottom = pnBox.y + pnBox.h;
				var bTop = box.y;
				var bBottom = box.y + obh;
				var vpLeft = pnBox.x;
				var vpRight = pnBox.x + pnBox.w;
				var bLeft = box.x;
				var bRight = box.x + obw;
				var delta;
				//Adjust the height now
				if(bBottom > vpTop &&
						bTop < vpTop){
					box.y = pnBox.y;
					//intersecting top, need to do some shifting.
					delta = vpTop - bTop;
					var visHeight = obh - delta;
					//If the visible height < viewport height,
					//We need to shift it.
					if(visHeight < pnBox.h){
						box.h = visHeight;
					}else{
						//Deal with horizontal scrollbars if necessary.
						box.h -= 2*(pn.scrollWidth > pn.clientWidth &&
								pn.clientWidth > 0? scrollers.h: 0);
					}
				}else if(bTop < vpBottom && bBottom > vpBottom){
					//Intersecting bottom, just figure out how much
					//overlay to show.
					box.h = vpBottom - bTop;
				}else if(bBottom <= vpTop || bTop >= vpBottom){
					//Outside view, hide it.
					box.h = 0;
				}
	
				//adjust width
				if(bRight > vpLeft && bLeft < vpLeft){
					box.x = pnBox.x;
					//intersecting left, need to do some shifting.
					delta = vpLeft - bLeft;
					var visWidth = obw - delta;
					//If the visible width < viewport width,
					//We need to shift it.
					if(visWidth < pnBox.w){
						box.w = visWidth;
					}else{
						//Deal with horizontal scrollbars if necessary.
						box.w -= 2*(pn.scrollHeight > pn.clientHeight &&
								pn.clientHeight > 0? scrollers.w:0);
					}
				}else if(bLeft < vpRight && bRight > vpRight){
					//Intersecting right, just figure out how much
					//overlay to show.
					box.w = vpRight - bLeft;
				}else if(bRight <= vpLeft || bLeft >= vpRight){
					//Outside view, hide it.
					box.w = 0;
				}
	
			}
	
	
			if(box && box.h > 0 && box.w > 0){
				//Set position and size of the blocking div overlay.
						dojo.style(this._underlayNode, {
							display: "block",
							width: box.w + "px",
							height: box.h + "px",
							top: box.y + "px",
							left: box.x + "px",
							backgroundColor: this.color
						});
						
				var styles = ["borderRadius", "borderTopLeftRadius",
							"borderTopRightRadius","borderBottomLeftRadius",
							"borderBottomRightRadius"];
						this._cloneStyles(styles);
				if(!dojo.isIE){
					//Browser specific styles to try and clone if non-IE.
					styles = ["MozBorderRadius", "MozBorderRadiusTopleft",
						"MozBorderRadiusTopright","MozBorderRadiusBottomleft",
						"MozBorderRadiusBottomright","WebkitBorderRadius",
						"WebkitBorderTopLeftRadius", "WebkitBorderTopRightRadius",
						"WebkitBorderBottomLeftRadius","WebkitBorderBottomRightRadius"
					];
					this._cloneStyles(styles, this);
				}
				var cntrIndicatorTop = (box.h/2) - (cntrIndicator.h/2);
				var cntrIndicatorLeft = (box.w/2) - (cntrIndicator.w/2);
				//Only show the image if there is height and width room.
				var top = cntrIndicatorTop + box.y;
				var left = cntrIndicatorLeft + box.x;
				if(box.h >= cntrIndicator.h && box.w >= cntrIndicator.w){
					dojo.style(this.rootDiv, {
						top: top + "px",
						left: left + "px",
						display: "block"
					});
				}else{
					console.info('here in else...should not come here...');
					dojo.style(this._underlayNode, "display", "none");
					dojo.style(this.rootDiv, "display", "none");
				}
			}else{
				//Target has no size, do not position
				//a case when the current domnode itself is a target (defautl case)
				//do nothing on positioning the progress bar
			}
			if(this._resizeCheck === null){
				//Set an interval timer that checks the target size and scales as needed.
				//Checking every 10th of a second seems to generate a fairly smooth update.
				var self = this;
				//hope this is not needed as window resize is getting handled....
				//keeping it commented in case it is required...
				//note: it is a overhead if this keeps running every fraction of a second...
				//should be used only if required...
				this._resizeCheck = setInterval(function(){self._size();}, 100);
			}
		}
		//}
	},
	_cloneStyles: function(list){
			// summary:
			//		Internal function to clone a set of styles from the target to
			//		the underlay.
			// list: Array
			//		An array of style names to clone.
			//
			// tags:
			//		private
			dojo.forEach(list, function(s){
				dojo.style(this._underlayNode, s, dojo.getStyle(this.target, s));
			}, this);
	},
	
	_scrollerWidths: function(){
		// summary:
		//		This function will calculate the size of the vertical and
		//		horizontal scrollbars.
		// returns:
		//		Object of form: {v: Number, h: Number} where v is vertical scrollbar width
		//		and h is horizontal scrollbar width.
		// tags:
		//		private
		/*# 40363: Enhanced Progressbar need "modal" option to be compatible with previous LDTW's BusyPane */
		 
		var div = dojo.create("div");
		dojo.style(div, {
			position: "absolute",
			opacity: 0,
			overflow: "hidden",
			width: "50px",
			height: "50px",
			zIndex: "-100",
			top: "-200px",
			padding: "0px",
			margin: "0px"
		});
		var iDiv = dojo.create("div");
		dojo.style(iDiv, {
			width: "200px",
			height: "10px"
		});
		div.appendChild(iDiv);
		dojo.body().appendChild(div);
	
		//Figure out content size before and after
		//scrollbars are there, then just subtract to
		//get width.
		var b = dojo.getContentBox(div);
		dojo.style(div, "overflow", "scroll");
		var a = dojo.getContentBox(div);
		dojo.body().removeChild(div);
		return { v: b.w - a.w, h: b.h - a.h };
	},

	//Satanik:03May2011: updating destroy for complete cleanup
	destroy: function(){
		console.info("EPB destroy called.....");
		// summary:  Destroys the toolbar.
		// tags: public	
		//memory leak issue: custom destroy method to take care of removing all widgets					
		try{
			//this.domNode was not getting cleanedup completely by the destroy so hiding the dom node.
			//this.domNode.style.display = "none";
			delete this.resources_;
			//this is for destroying child templated widgets 
			//this should happen by itself but need to figure out why dijit._Templated is not taking care of it...
			if(this.domNode){
				this._displayed = false;
				var widgets = dijit.findWidgets(this.domNode);
				dojo.forEach(widgets, function(w) {
					if(w){
						if(w.destroyRecursive){
							w.destroyRecursive();
						} else if(w.destroy){
							w.destroy();
						}
					}
				});
				// this is for widgets added as child to this widget and their children..
				//not present in template
				var widgetArr = this.getChildren();
				dojo.forEach(widgetArr, function (child) {
					if(child){
						if(child.destroyRecursive){
							child.destroyRecursive();
						} else if(child.destroy){
							child.destroy();
						}
					}
				});
				this.inherited(arguments);
				//console.log(this.domNode);
				if(this.domNode) this.domNode.destroy();
	
			}
		}catch(ex){
			console.warn("Widget destroy failed: "+this.id+", ex="+ex.name+":"+ex.message);
		}
	},
	
	onview:function(){                                   
		//summary:parses the template
		//tags:public				
		dojo.parser.parse(this.progressBarTable);			
	}
}); 

});

},
'dijit/form/_FormMixin':function(){
define([
	"dojo/_base/array", // array.every array.filter array.forEach array.indexOf array.map
	"dojo/_base/declare", // declare
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/lang", // lang.hitch lang.isArray
	"dojo/window" // winUtils.scrollIntoView
], function(array, declare, kernel, lang, winUtils){

	// module:
	//		dijit/form/_FormMixin
	// summary:
	//		Mixin for containers of form widgets (i.e. widgets that represent a single value
	//		and can be children of a <form> node or dijit.form.Form widget)

	return declare("dijit.form._FormMixin", null, {
		// summary:
		//		Mixin for containers of form widgets (i.e. widgets that represent a single value
		//		and can be children of a <form> node or dijit.form.Form widget)
		// description:
		//		Can extract all the form widgets
		//		values and combine them into a single javascript object, or alternately
		//		take such an object and set the values for all the contained
		//		form widgets

	/*=====
		// value: Object
		//		Name/value hash for each child widget with a name and value.
		//		Child widgets without names are not part of the hash.
		//
		//		If there are multiple child widgets w/the same name, value is an array,
		//		unless they are radio buttons in which case value is a scalar (since only
		//		one radio button can be checked at a time).
		//
		//		If a child widget's name is a dot separated list (like a.b.c.d), it's a nested structure.
		//
		//		Example:
		//	|	{ name: "John Smith", interests: ["sports", "movies"] }
	=====*/

		// state: [readonly] String
		//		Will be "Error" if one or more of the child widgets has an invalid value,
		//		"Incomplete" if not all of the required child widgets are filled in.  Otherwise, "",
		//		which indicates that the form is ready to be submitted.
		state: "",

		//	TODO:
		//	* Repeater
		//	* better handling for arrays.  Often form elements have names with [] like
		//	* people[3].sex (for a list of people [{name: Bill, sex: M}, ...])
		//
		//

		_getDescendantFormWidgets: function(/*dijit._WidgetBase[]?*/ children){
			// summary:
			//		Returns all form widget descendants, searching through non-form child widgets like BorderContainer
			var res = [];
			array.forEach(children || this.getChildren(), function(child){
				if("value" in child){
					res.push(child);
				}else{
					res = res.concat(this._getDescendantFormWidgets(child.getChildren()));
				}
			}, this);
			return res;
		},

		reset: function(){
			array.forEach(this._getDescendantFormWidgets(), function(widget){
				if(widget.reset){
					widget.reset();
				}
			});
		},

		validate: function(){
			// summary:
			//		returns if the form is valid - same as isValid - but
			//		provides a few additional (ui-specific) features.
			//		1 - it will highlight any sub-widgets that are not
			//			valid
			//		2 - it will call focus() on the first invalid
			//			sub-widget
			var didFocus = false;
			return array.every(array.map(this._getDescendantFormWidgets(), function(widget){
				// Need to set this so that "required" widgets get their
				// state set.
				widget._hasBeenBlurred = true;
				var valid = widget.disabled || !widget.validate || widget.validate();
				if(!valid && !didFocus){
					// Set focus of the first non-valid widget
					winUtils.scrollIntoView(widget.containerNode || widget.domNode);
					widget.focus();
					didFocus = true;
				}
	 			return valid;
	 		}), function(item){ return item; });
		},

		setValues: function(val){
			kernel.deprecated(this.declaredClass+"::setValues() is deprecated. Use set('value', val) instead.", "", "2.0");
			return this.set('value', val);
		},
		_setValueAttr: function(/*Object*/ obj){
			// summary:
			//		Fill in form values from according to an Object (in the format returned by get('value'))

			// generate map from name --> [list of widgets with that name]
			var map = { };
			array.forEach(this._getDescendantFormWidgets(), function(widget){
				if(!widget.name){ return; }
				var entry = map[widget.name] || (map[widget.name] = [] );
				entry.push(widget);
			});

			for(var name in map){
				if(!map.hasOwnProperty(name)){
					continue;
				}
				var widgets = map[name],						// array of widgets w/this name
					values = lang.getObject(name, false, obj);	// list of values for those widgets

				if(values === undefined){
					continue;
				}
				if(!lang.isArray(values)){
					values = [ values ];
				}
				if(typeof widgets[0].checked == 'boolean'){
					// for checkbox/radio, values is a list of which widgets should be checked
					array.forEach(widgets, function(w){
						w.set('value', array.indexOf(values, w.value) != -1);
					});
				}else if(widgets[0].multiple){
					// it takes an array (e.g. multi-select)
					widgets[0].set('value', values);
				}else{
					// otherwise, values is a list of values to be assigned sequentially to each widget
					array.forEach(widgets, function(w, i){
						w.set('value', values[i]);
					});
				}
			}

			/***
			 * 	TODO: code for plain input boxes (this shouldn't run for inputs that are part of widgets)

			array.forEach(this.containerNode.elements, function(element){
				if(element.name == ''){return};	// like "continue"
				var namePath = element.name.split(".");
				var myObj=obj;
				var name=namePath[namePath.length-1];
				for(var j=1,len2=namePath.length;j<len2;++j){
					var p=namePath[j - 1];
					// repeater support block
					var nameA=p.split("[");
					if(nameA.length > 1){
						if(typeof(myObj[nameA[0]]) == "undefined"){
							myObj[nameA[0]]=[ ];
						} // if

						nameIndex=parseInt(nameA[1]);
						if(typeof(myObj[nameA[0]][nameIndex]) == "undefined"){
							myObj[nameA[0]][nameIndex] = { };
						}
						myObj=myObj[nameA[0]][nameIndex];
						continue;
					} // repeater support ends

					if(typeof(myObj[p]) == "undefined"){
						myObj=undefined;
						break;
					};
					myObj=myObj[p];
				}

				if(typeof(myObj) == "undefined"){
					return;		// like "continue"
				}
				if(typeof(myObj[name]) == "undefined" && this.ignoreNullValues){
					return;		// like "continue"
				}

				// TODO: widget values (just call set('value', ...) on the widget)

				// TODO: maybe should call dojo.getNodeProp() instead
				switch(element.type){
					case "checkbox":
						element.checked = (name in myObj) &&
							array.some(myObj[name], function(val){ return val == element.value; });
						break;
					case "radio":
						element.checked = (name in myObj) && myObj[name] == element.value;
						break;
					case "select-multiple":
						element.selectedIndex=-1;
						array.forEach(element.options, function(option){
							option.selected = array.some(myObj[name], function(val){ return option.value == val; });
						});
						break;
					case "select-one":
						element.selectedIndex="0";
						array.forEach(element.options, function(option){
							option.selected = option.value == myObj[name];
						});
						break;
					case "hidden":
					case "text":
					case "textarea":
					case "password":
						element.value = myObj[name] || "";
						break;
				}
	  		});
	  		*/

			// Note: no need to call this._set("value", ...) as the child updates will trigger onChange events
			// which I am monitoring.
		},

		getValues: function(){
			kernel.deprecated(this.declaredClass+"::getValues() is deprecated. Use get('value') instead.", "", "2.0");
			return this.get('value');
		},
		_getValueAttr: function(){
			// summary:
			// 		Returns Object representing form values.   See description of `value` for details.
			// description:

			// The value is updated into this.value every time a child has an onChange event,
			// so in the common case this function could just return this.value.   However,
			// that wouldn't work when:
			//
			// 1. User presses return key to submit a form.  That doesn't fire an onchange event,
			// and even if it did it would come too late due to the setTimeout(..., 0) in _handleOnChange()
			//
			// 2. app for some reason calls this.get("value") while the user is typing into a
			// form field.   Not sure if that case needs to be supported or not.

			// get widget values
			var obj = { };
			array.forEach(this._getDescendantFormWidgets(), function(widget){
				var name = widget.name;
				if(!name || widget.disabled){ return; }

				// Single value widget (checkbox, radio, or plain <input> type widget)
				var value = widget.get('value');

				// Store widget's value(s) as a scalar, except for checkboxes which are automatically arrays
				if(typeof widget.checked == 'boolean'){
					if(/Radio/.test(widget.declaredClass)){
						// radio button
						if(value !== false){
							lang.setObject(name, value, obj);
						}else{
							// give radio widgets a default of null
							value = lang.getObject(name, false, obj);
							if(value === undefined){
								lang.setObject(name, null, obj);
							}
						}
					}else{
						// checkbox/toggle button
						var ary=lang.getObject(name, false, obj);
						if(!ary){
							ary=[];
							lang.setObject(name, ary, obj);
						}
						if(value !== false){
							ary.push(value);
						}
					}
				}else{
					var prev=lang.getObject(name, false, obj);
					if(typeof prev != "undefined"){
						if(lang.isArray(prev)){
							prev.push(value);
						}else{
							lang.setObject(name, [prev, value], obj);
						}
					}else{
						// unique name
						lang.setObject(name, value, obj);
					}
				}
			});

			/***
			 * code for plain input boxes (see also domForm.formToObject, can we use that instead of this code?
			 * but it doesn't understand [] notation, presumably)
			var obj = { };
			array.forEach(this.containerNode.elements, function(elm){
				if(!elm.name)	{
					return;		// like "continue"
				}
				var namePath = elm.name.split(".");
				var myObj=obj;
				var name=namePath[namePath.length-1];
				for(var j=1,len2=namePath.length;j<len2;++j){
					var nameIndex = null;
					var p=namePath[j - 1];
					var nameA=p.split("[");
					if(nameA.length > 1){
						if(typeof(myObj[nameA[0]]) == "undefined"){
							myObj[nameA[0]]=[ ];
						} // if
						nameIndex=parseInt(nameA[1]);
						if(typeof(myObj[nameA[0]][nameIndex]) == "undefined"){
							myObj[nameA[0]][nameIndex] = { };
						}
					}else if(typeof(myObj[nameA[0]]) == "undefined"){
						myObj[nameA[0]] = { }
					} // if

					if(nameA.length == 1){
						myObj=myObj[nameA[0]];
					}else{
						myObj=myObj[nameA[0]][nameIndex];
					} // if
				} // for

				if((elm.type != "select-multiple" && elm.type != "checkbox" && elm.type != "radio") || (elm.type == "radio" && elm.checked)){
					if(name == name.split("[")[0]){
						myObj[name]=elm.value;
					}else{
						// can not set value when there is no name
					}
				}else if(elm.type == "checkbox" && elm.checked){
					if(typeof(myObj[name]) == 'undefined'){
						myObj[name]=[ ];
					}
					myObj[name].push(elm.value);
				}else if(elm.type == "select-multiple"){
					if(typeof(myObj[name]) == 'undefined'){
						myObj[name]=[ ];
					}
					for(var jdx=0,len3=elm.options.length; jdx<len3; ++jdx){
						if(elm.options[jdx].selected){
							myObj[name].push(elm.options[jdx].value);
						}
					}
				} // if
				name=undefined;
			}); // forEach
			***/
			return obj;
		},

	 	isValid: function(){
	 		// summary:
	 		//		Returns true if all of the widgets are valid.
			//		Deprecated, will be removed in 2.0.  Use get("state") instead.

			return this.state == "";
		},

		onValidStateChange: function(/*Boolean*/ /*===== isValid =====*/){
			// summary:
			//		Stub function to connect to if you want to do something
			//		(like disable/enable a submit button) when the valid
			//		state changes on the form as a whole.
			//
			//		Deprecated.  Will be removed in 2.0.  Use watch("state", ...) instead.
		},

		_getState: function(){
			// summary:
			//		Compute what this.state should be based on state of children
			var states = array.map(this._descendants, function(w){
				return w.get("state") || "";
			});

			return array.indexOf(states, "Error") >= 0 ? "Error" :
				array.indexOf(states, "Incomplete") >= 0 ? "Incomplete" : "";
		},

		disconnectChildren: function(){
			// summary:
			//		Remove connections to monitor changes to children's value, error state, and disabled state,
			//		in order to update Form.value and Form.state.
			array.forEach(this._childConnections || [], lang.hitch(this, "disconnect"));
			array.forEach(this._childWatches || [], function(w){ w.unwatch(); });
		},

		connectChildren: function(/*Boolean*/ inStartup){
			// summary:
			//		Setup connections to monitor changes to children's value, error state, and disabled state,
			//		in order to update Form.value and Form.state.
			//
			//		You can call this function directly, ex. in the event that you
			//		programmatically add a widget to the form *after* the form has been
			//		initialized.

			var _this = this;

			// Remove old connections, if any
			this.disconnectChildren();

			this._descendants = this._getDescendantFormWidgets();

			// (Re)set this.value and this.state.   Send watch() notifications but not on startup.
			var set = inStartup ? function(name, val){ _this[name] = val; } : lang.hitch(this, "_set");
			set("value", this.get("value"));
			set("state", this._getState());

			// Monitor changes to error state and disabled state in order to update
			// Form.state
			var conns = (this._childConnections = []),
				watches = (this._childWatches = []);
			array.forEach(array.filter(this._descendants,
				function(item){ return item.validate; }
			),
			function(widget){
				// We are interested in whenever the widget changes validity state - or
				// whenever the disabled attribute on that widget is changed.
				array.forEach(["state", "disabled"], function(attr){
					watches.push(widget.watch(attr, function(){
						_this.set("state", _this._getState());
					}));
				});
			});

			// And monitor calls to child.onChange so we can update this.value
			var onChange = function(){
				// summary:
				//		Called when child's value or disabled state changes

				// Use setTimeout() to collapse value changes in multiple children into a single
				// update to my value.   Multiple updates will occur on:
				//	1. Form.set()
				//	2. Form.reset()
				//	3. user selecting a radio button (which will de-select another radio button,
				//		 causing two onChange events)
				if(_this._onChangeDelayTimer){
					clearTimeout(_this._onChangeDelayTimer);
				}
				_this._onChangeDelayTimer = setTimeout(function(){
					delete _this._onChangeDelayTimer;
					_this._set("value", _this.get("value"));
				}, 10);
			};
			array.forEach(
				array.filter(this._descendants, function(item){ return item.onChange; } ),
				function(widget){
					// When a child widget's value changes,
					// the efficient thing to do is to just update that one attribute in this.value,
					// but that gets a little complicated when a checkbox is checked/unchecked
					// since this.value["checkboxName"] contains an array of all the checkboxes w/the same name.
					// Doing simple thing for now.
					conns.push(_this.connect(widget, "onChange", onChange));

					// Disabling/enabling a child widget should remove it's value from this.value.
					// Again, this code could be more efficient, doing simple thing for now.
					watches.push(widget.watch("disabled", onChange));
				}
			);
		},

		startup: function(){
			this.inherited(arguments);

			// Initialize value and valid/invalid state tracking.  Needs to be done in startup()
			// so that children are initialized.
			this.connectChildren(true);

			// Make state change call onValidStateChange(), will be removed in 2.0
			this.watch("state", function(attr, oldVal, newVal){ this.onValidStateChange(newVal == ""); });
		},

		destroy: function(){
			this.disconnectChildren();
			this.inherited(arguments);
		}

	});
});

},
'dojoe/treetablex/modules/StatefulRefresh':function(){
define("dojoe/treetablex/modules/StatefulRefresh", [
                                                    "dojo",
                                                    "dijit",
                                                    'dojo/_base/array', 
                                                    "gridx/modules/Tree",
                                                    "gridx/core/_Module",
                                                    "dojo/_base/Deferred",
                    								"dojo/DeferredList"
                                                    ], 
function(dojo, dijit, Array, Tree, _Module, Deferred, DeferredList) {
	
    _Module.register(Tree);
    
	return  dojo.declare('dojoe.treetablex.modules.StatefulRefresh', _Module, {
		
		// [Module Dependency Management] --------------------------------------------
		
		//Every module needs a name. 
		//It could be the same as another module's, which means that module's public API is the same as this one's.
		//If this name is omitted, the long class name will be used ('dojoe.treetablex.modules.EnhancedRefresh' in this case),
		name: 'statefulRefresh',

		constructor: function(){
			this._clear();
		},
			
		//Required dependency modules must also exist if this module exists.
		//But they don't need to finish loading before this one.
		required: ['tree'],
		
		_expStatus: null,
		
		
		// [Module API Management] ---------------------------------------------------
		
		getAPIPath: function(){
			//This method defines how to access this module's API from the grid widget.
			//Anything returned from this function is 'deeply' mixed into grid.
			//For this example, we'll be able to call:
			//		grid.enhancedRefresh
			
			//If some other module needs to extend the APIs in namespace grid.testMode,
			//it can also provide this method in exact the same way, because the mixing in
			//process is performed layer by layer (deeply).
			
			//You can even directly add methods to grid: 
			//		return {
			//			newMethodForGrid: function(){...}
			// 		};
			
			return {
				statefulRefresh: this
			};
		},
		
	
	
		// [Module Lifetime Management] -----------------------------------------------
		preload: function(args){
			//This is called after all modules are created, but not loaded yet. Dependancy sequence is not hornored here.
			var t = this,
			g = t.grid;
			t.batchConnect(
				[g.tree,'onExpand','_recordExpand'],
				[g.tree,'onCollapse','_recordCollapse'],
				[g, 'setStore', '_clear']
				//[g.body, 'refresh', '_persist']
				);
		},
	
		load: function(args, deferStartup){
			var t = this;
			
		},
		
		_applyTreeState: function(){
			var t = this;
			/*53045: LS - 9/27/2012 - clearing the cache again after refresh causes the store lose the cache...*/
			//t.grid.model.clearCache();
			t.grid.tree._clear();
			if(t.grid.persist){
				var content = t.grid.persist.get(t.grid.id);
				if(content && content["expStatus"]){
					var expandoStatusArr = content["expStatus"];
					var len = expandoStatusArr[''].opened.length;
					var i, deferredList = [];
					for(i=0; i<len; i++){
						var deferred = null;
						if(t._canExpand(expandoStatusArr[''].opened[i])){
							deferred = t.grid.tree.expand(expandoStatusArr[''].opened[i],true);
							deferredList.push(deferred );
						}
					}
					new DeferredList(deferredList).then(function(){
								t.grid.body.updateRootRange(t.grid.body.rootStart,t.grid.body.rootCount);
								t.grid.body.refresh();
							});
								
				}
			}

		},
		
		/*53045: LS - 9/27/2012 - TreeTableView stops working after refreshing when previous nodes were in expanded state...*/
		/*check for haschildren before expanding the node in case the nodes and the structure has changed...*/
		_canExpand: function(id){
			// summary:
			//		Check whether a row can be expanded.
			// id: String
			//		The row ID
			// returns:
			//		Whether the row can be expanded.
			var t = this,
			m = t.grid.model;
			return  m.hasChildren(id);
		},
		
		_recordExpand: function(id){
			if(!this._expStatus[id]){
				this._expStatus[id] = { id: id,
										status:'open'
									   };
				this._expStatus[''].opened.push(id);						
				this._persist();	
			}			
			
		},
		_recordCollapse: function(id){
			delete(this._expStatus[id]); 
			var arr = this._expStatus[''].opened;
			Array.forEach(arr, function(openId,i){
				if(openId === id){
					arr.splice(i);
				}
			});
			this._persist();
		},
		
		_persist: function(){
			// persist the visible column information
			if(this.grid.persist){
				var expStatus = this._expStatus;
				var status = {'expStatus': expStatus};
				this.grid.persist.key = this.grid.id;
				this.grid.persist.put(this.grid.persist.key, status);
				this.grid.persist.registerAndLoad('expStatus', function(){
							return expStatus;
						});
				this.grid.persist.save();
			}
		},
		_clear: function(){
			this._expStatus = {'': {id: '', status: '', opened:[]} };
			
		},
		
		destroy: function(){
			this.inherited(arguments);
			this._clear();
			//do something to tear me down here.
			//e.g.: dojo.destroy(this.domNode);
		}
	
	
	});
});
},
'ibm/tivoli/tip/dijit/Breadcrumb':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!dijit/_Widget,dijit/_Container,dijit/_Contained,dijit/_Templated,dijit/form/Button"], function(dijit,dojo,dojox){
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
dojo.provide("ibm.tivoli.tip.dijit.Breadcrumb");
dojo.require("dijit._Widget");
dojo.require("dijit._Container");
dojo.require("dijit._Contained");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
 
dojo.declare(
		"ibm.tivoli.tip.dijit.Breadcrumb",
		[ dijit.form.Button, dijit._Contained ],
	{
		baseClass: "dijitBreadcrumb",
/*		templateString:"<div class=\"dijit dijitLeft dijitInline \"\n\t><div class='dijitRight'\n\t\t><button class=\"dijitStretch dijitBreadcrumb dijitButtonContents\" dojoAttachPoint=\"focusNode,titleNode\"\n\t\t\ttype=\"${type}\" waiRole=\"button\" waiState=\"labelledby-${id}_label\"\n\t\t\t><span class=\"dijitInline ${iconClass}\" dojoAttachPoint=\"iconNode\" \n \t\t\t\t><span class=\"dijitToggleButtonIconChar\">&#10003</span \n\t\t\t></span\n\t\t\t><span class=\"dijitBreadcrumb\" id=\"${id}_label\" dojoAttachPoint=\"containerNode\">${label}</span\n\t\t></button\n\t></div\n></div>\n",*/
		
		templateString:"<div class=\"dijit dijitLeft dijitInline \"\n\tdojoAttachEvent=\"onclick:_onClick,onmouseenter:_onMouse,onmouseleave:_onMouse,onmousedown:_onMouse\"\n\t><div class='dijitRight'\n\t\t><button class=\"dijitStretch dijitBreadcrumb dijitButtonContents\" dojoAttachPoint=\"focusNode,titleNode\"\n\t\t\ttype=\"${type}\" waiRole=\"button\" waiState=\"labelledby-${id}_label\"\n\t\t\t><span class=\"dijitInline ${iconClass}\" dojoAttachPoint=\"iconNode\" \n \t\t\t\t><span class=\"dijitToggleButtonIconChar\">&#10003</span \n\t\t\t></span\n\t\t\t><span class=\"dijitBreadcrumb\" id=\"${id}_label\" dojoAttachEvent=\"onclick:_onClick,onmouseenter:_onMouse,onmouseleave:_onMouse,onmousedown:_onMouse\"dojoAttachPoint=\"containerNode\">${label}</span\n\t\t></button\n\t></div\n></div>\n",
/*
dojoAttachEvent=\"onclick:_onButtonClick,onmouseenter:_onMouse,onmouseleave:_onMouse,onmousedown:_onMouse\"

*/
		isCurrent: function() {
			return (this.getNextSibling()==null);
		},
		
		constructor: function () {
		},
		postCreate: function() {
			this.inherited(arguments);
		},
		destroy: function () {
			this.destroyDescendants();
			this.inherited (arguments);
		},
		
		_setStateClass:function(){
			if(this.isCurrent()){
				this.baseClass="dijitBreadcrumbCurrent";	//setting special class for the current breadcrumb
			} else {
				this.baseClass="dijitBreadcrumb";
			}; 
			this.inherited(arguments);
		},
		//Added fix for IBM Dojo 1.5
		_onMouse: function(){
		}
	});


});

},
'dojoe/common/toolbar/_Toolbar':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!dijit/_Widget,dijit/_TemplatedMixin"], function(dijit,dojo,dojox){
dojo.provide("dojoe.common.toolbar._Toolbar");

dojo.require("dijit._Widget");
dojo.require("dijit._TemplatedMixin");

dojo.declare("dojoe.common.toolbar._Toolbar", [ dijit._Widget, dijit._TemplatedMixin], {	

		templateString:dojo.cache("dojoe.common.toolbar", "templates/Toolbar.html", "<div  dojoAttachEvent='oncontextmenu:_stopEvent' class=\"tivTable dijitToolbar\" width=\"100%\">\r\n<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" width=\"100%\" role=\"presentation\">\r\n\t<tbody>\r\n\t\t<tr valign=\"center\">\r\n\t\t\t<td width=\"80%\" height=\"100%\">\r\n\t\t\t\t<span dojoAttachPoint=\"toolbarFirstRow\">\r\n\t\t\t\t\t<span dojoAttachPoint=\"stateIconNode\"></span>\r\n\t\t\t\t\t<span dojoAttachPoint=\"userDefinedNode\"></span>\r\n\t\t\t\t\t<span dojoAttachPoint=\"customIconNode\"></span>\r\n\t\t\t\t</span>\r\n\t\t\t</td>\r\n\t\t\t<td width=\"20%\" height=\"100%\" align=\"right\">\r\n\t\t\t\t<div dojoAttachPoint=\"quickFilterNode\" role=\"search\">\r\n\t\t\t\t</div>\r\n\t\t\t</td>\r\n\t\t\t<td>\r\n\t\t\t\t<div dojoAttachPoint=\"filterButton\" role=\"button\">\r\n\t\t\t\t</div>\r\n\t\t\t</td>\t\t\t\r\n\t\t</tr>\r\n\t\t<tr valign=\"center\">\r\n\t\t\t<td width=\"100%\">\r\n\t\t\t\t<span dojoAttachPoint=\"toolbarSecondRow\">\r\n\t\t\t\t</span>\t\r\n\t\t\t</td>\r\n\t\t</tr>\r\n\t</tbody>\t\t\r\n</table>\r\n</div>"),
		widgetsInTemplate:true,
				
		// To add the item/icon to the toolbar.
		
		_addStateControl: function(item, position) {
			if (item != null) {
				var spanNode = dojo.doc.createElement("span", {role : "button"}); 
				dojo.style(spanNode, "width", "1px"); 
				if(position !== undefined){
					dojo.place(spanNode, this.stateIconNode, parseInt(position));
				}else {
					dojo.place(spanNode, this.stateIconNode, 'last');
				}
				spanNode.appendChild(item.domNode);
			}
		},
		
		_addFastPathControl: function(item, position) {
			if (item != null) {
				var spanNode = dojo.doc.createElement("span", {role : "button"}); 
				dojo.style(spanNode, "width", "1px"); 
				if(position !== undefined){
					dojo.place(spanNode, this.customIconNode, parseInt(position));
				}else {
					dojo.place(spanNode, this.customIconNode, 'last');
				}
				spanNode.appendChild(item.domNode);
			}
		},
		
		//do not call this method directly, use table._addCustomControl(item, position, row) method to add a custom item to the toolbar before default buttons
		_addCustomControl: function(item, position, row){
			if (item != null) {
				var spanNode = dojo.doc.createElement("span", {role : "button"}); 
				dojo.style(spanNode, "width", "1px"); 
				if(row == 2) {
					if(position !== undefined){
						dojo.place(spanNode, this.toolbarSecondRow, parseInt(position));
					}else {
						dojo.place(spanNode, this.toolbarSecondRow, 'last');
					}
					spanNode.appendChild(item.domNode);
				} else {
					if(position !== undefined){
						dojo.place(spanNode, this.userDefinedNode, parseInt(position));
					}else {
						dojo.place(spanNode, this.userDefinedNode, 'last');
					}
					spanNode.appendChild(item.domNode);
				} 
			}
		}		
		
	});
});

},
'url:ibm/usmi/widgets/wizard/templates/DirectorWizardNested.html':"<div class=\"tiv-wiz\" role=\"application\">\r\n<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n<div class=\"tiv-wiz\" role=\"application\">\r\n  <div dojoType=\"dijit.layout.BorderContainer\" design=\"headline\" gutters=\"false\" class=\"tiv-wiz-outer\" dojoAttachPoint=\"_outerContainer\">\r\n        <div dojoType=\"dijit.layout.BorderContainer\" gutters=\"false\" liveSplitters=\"false\" design=\"sidebar\" region=\"center\" style=\"width:100%;height:100%;\" dojoAttachPoint=\"_borderArea\">\r\n             <div dojoType=\"dijit.layout.ContentPane\" region=\"leading\" splitter=\"false\" class=\"tiv-wiz-toc\" dojoAttachPoint=\"_tocArea\">\r\n                  <ol dojoAttachPoint=\"_toc\"></ol>\r\n             </div>\r\n             <div dojoType=\"dijit.layout.StackContainer\" region=\"center\" splitter=\"false\" dojoAttachPoint=\"_pageContainer\" class=\"tiv-wiz-work-area\"> \r\n                  <div dojoAttachPoint=\"containerNode\"></div>\r\n             </div>\r\n        </div>\r\n\t\t<div dojoType=\"dijit.layout.ContentPane\" region=\"bottom\" splitter=\"false\" class=\"tiv-wiz-footer\" dojoAttachPoint=\"_buttonArea\">\r\n\t\t\t<!-- We want the director wizard step buttons to be left-justified -->\t\t\r\n            <div class=\"director-wiz-step-buttons\" style=\"display: inline;\" data-dojo-attach-point=\"_buttons\">\r\n\t\t\t\t<button role=\"button\" type=\"button\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickBack\" data-dojo-attach-point=\"_backButton\">${backLabel}</button>\r\n\t\t\t\t<button role=\"button\" type=\"button\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickNext\" data-dojo-attach-point=\"_nextButton\">${nextLabel}</button>\r\n\t\t\t</div>\t\t\t\r\n\t\t\t<!--  we want the director wizard commit buttons to be right-justified -->\r\n\t\t\t<div class=\"director-wiz-commit-buttons\" style=\"display:inline;\" data-dojo-attach-point=\"_buttons\">\r\n\t\t\t\t<button role=\"button\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickFinish\" data-dojo-props='type:\"button\"' data-dojo-attach-point=\"_finishButton\">${finishLabel}</button>\r\n\t\t\t\t<button role=\"button\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickCancel\"  data-dojo-props='type:\"button\"' data-dojo-attach-point=\"_cancelButton\">${cancelLabel}</button>\r\n\t\t\t</div> \r\n         </div>\r\n   </div>\r\n</div>\r\n",
'ibm/tivoli/navmodel/Task':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/Dataset,ibm/tivoli/navmodel/Selection,ibm/tivoli/navmodel/Utils,ibm/tivoli/navmodel/Trace"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Task"); 
 
// includes
dojo.require("ibm.tivoli.navmodel.Dataset");
dojo.require("ibm.tivoli.navmodel.Selection");
dojo.require("ibm.tivoli.navmodel.Utils");
dojo.require("ibm.tivoli.navmodel.Trace");
ibm.tivoli.navmodel.Task._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.Task");

////////////////////////////////////
// Task
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Task", null, {
	// private members
	_task: null,
	_dataset: null,
	_childTasks: null,
	_trace: ibm.tivoli.navmodel.Task._trace,
	
	/*=====
	constructor_kwArgs: function(task, dataset) {
		// task: Object
		//		json task object
		// dataset: Object
		//		ibm.tivoli.navmodel.Dataset that created this task
	},
	=====*/
	
	 constructor: function(/*constructor_kwArgs*/ kwArgs) {				
		// summary:
		//		Constructs Task object.
		this._dataset = kwArgs.dataset;
		this._task = kwArgs.task;
		if (!this._task) {
			this._task = {};
		}
	},

	/////////////////////////////
	// public methods
	/////////////////////////////

	getAttribute: function(key) {		
		// summary:
		//		Returns general attribute for requested key
		return this._task[key];
	},

	setAttribute: function(key, value) {		
		// summary:
		//		Set general attribute for requested key
		this._task[key] = value;
	},

	getVersion: function() {		
		// summary:
		//		Returns rest version
		return this._dataset.getVersion();
	},
		
	getUri: function() {
		if (!this._task.uri) {
			this._task.uri = this._dataset.getAttribute("uri") + "/tasks/" + encodeURIComponent(this.getId());
		}
	
		return this._dataset.getArg("baseUrl") + this._task.uri;
	},
	
	getId: function() {
		return this._task.id;
	},
	
	setId: function(id) {
		this._task.id = id;
	},
	
	getLabel: function() {
		return this._task.label;
	},
	
	setLabel: function(label) {
		this._task.label = label;
	},
	
	getDescription: function() {
		return this._task.description;
	},
	
	setDescription: function(description) {
		this._task.description = description;
	},
	
	getImage: function() {
		return this._task.image;
	},
	
	setImage: function(image) {
		this._task.image = image;
	},

	getCategory: function() {		
		// summary:
		//		Returns the task category (i.e. where the task is visualized)
		//		
		//		values are: menu, menu:navigation, toolbar, toolbar:button, toolbar:menu,
		//					breadcrumb, breadcrumb:navigation, and property
		return this._task.category;
	},
	
	setCategory: function(category) {
		this._task.category = category;
	},

	isServer: function() {		
		// summary:
		//		Indicates that task should be sent back to the server for execution
		return (this._task.server !== undefined ? this._task.server : false);
	},
	
	setServer: function(isServer) {
		this._task.server = isServer;
	},
	 
	getType: function() {		
		// summary:
		//		Returns the task type
		//		
		//		values are: none, url, page, portlet, clientevent, javascript, native,
		//					dialog, noninteractive, cascade, and separator
		return this._task.type;
	},

	setType: function(type) {		
		// summary:
		//		Set the task type
		//		
		//		values are: none, url, page, portlet, clientevent, javascript, native,
		//					dialog, noninteractive, cascade, and separator
		this._task.type = type;
	},
	
	isNormal: function() {
		return (!this.isNavigation() && !this.isCascade() && !this.isSeparator());
	},

	isNavigation: function() {
		return (this._task.type === "navigation");
	},
	
	isPageLaunch: function() {
		return (this._task.type === "page");
	},
	
	isPortletLaunch: function() {
		return (this._task.type === "portlet");
	},
	
	isUrlLaunch: function() {
		return (this._task.type === "url");
	},
	
	isJavascript: function() {
		return (this._task.type === "javascript");
	},
	
	isClientEvent: function() {
		return (this._task.type === "clientevent");
	},
	
	isCascade: function() {
		return (this._task.type === "cascade" && this._task.subTasks);
	},
	
	isSeparator: function() {
		return (this._task.type === "separator");
	},
	
	isResolved: function() {
		return (this._task.resolved !== undefined ? this._task.resolved : true);
	},
	
	setResolved: function(isResolved) {
		this._task.resolved = isResolved;
	},
	
	isDefault: function() {
		return (this._task["default"] !== undefined ? this._task["default"] : false);
	},
	
	setDefault: function(isDefault) {
		this._task["default"] = isDefault;
	},
	
	isEnabled: function() {
		return (this._task.enabled !== undefined ? this._task.enabled : true);
	},
	
	setEnabled: function(isEnabled) {
		this._task.enabled = isEnabled;
	},
	
	isDisabled: function() {
		return (this._task.enabled !== undefined ? !this._task.enabled : false);
	},
	
	setDisabled: function(isDisabled) {
		this._task.enabled = !isDisabled;
	},
	
	/*=====
	resolve_kwArgs: function(selection, itemIds, linkIds, itemIndexes) {
		// selection: Object
		//		ibm.tivoli.navmodel.Selection object
		//		if selection is specified: itemIds, linkIds, itemIndexes is ignored
		// itemIds: String
		//		comma delimited list of item ids that are selected
		// linkIds: String
		//		comma delimited list of link ids that are selected
		// itemIndexes: String
		//		comma delimited list of item ids or ranges that are selected
		//		ex: "0,1,10,20" or "0-10,30,40-50"
	},
	=====*/

	resolve: function(/*resolve_kwArgs*/ kwArgs) {		
		// summary:
		//		Re-fetch the task to resolve all client side launch parameters
		var args = {
			selection: null,
			itemIds: null,
			linkIds: null,
			itemIndexes: null
		};
		dojo.mixin(args, kwArgs);
		
		// selection was provided
		var json = "";
		if (args.selection) {
			json = dojo.toJson(args.selection.getAttributes(), false);
		}
			
		return dojo.xhrPost ({
			url: this._getParamURL(args),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "task"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "taskContext"),
				'Content-Length': json.length,
				'X-Method-Override': 'GET'
			},
			handleAs: "json",
			postData: json,
			sync: true,			
			load: dojo.hitch(this, function(result, ioArgs) {
				this._task = result;
			}),
			error: dojo.hitch(this, function(error, ioArgs) {
				ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "resolveLaunchParms", this._trace, this._version());					              
			})
		});
	},
	
	getLaunchUrl: function() {
		return this._task.url;
	},
	
	setLaunchUrl: function(url) {
		this._task.url = url;
	},
	
	getLaunchBrowserWindowId: function() {
		return this._task.browserWindowId;
	},
	
	setLaunchBrowserWindowId: function(windowId) {
		this._task.browserWindowId = windowId;
	},
	
	getLaunchPageId: function() {
		return this._task.pageId;
	},
	
	setLaunchPageId: function(pageId) {
		this._task.pageId = pageId;
	},
	
	getLaunchPortletName: function() {
		return this._task.portletName;
	},
	
	setLaunchPortletName: function(portletName) {
		this._task.portletName = portletName;
	},
	
	getLaunchPortletReuse: function() {
		return this._task.portletReuse;
	},
	
	setLaunchPortletReuse: function(reuse) {
		this._task.portletReuse = reuse;
	},
	
	getLaunchPortletAppUID: function() {
		return this._task.portletAppUID;
	},
	
	setLaunchPortletAppUID: function(appUID) {
		this._task.portletAppUID = appUID;
	},
	
	getLaunchEvent: function() {
		return this._task.event;
	},
	
	setLaunchEvent: function(event) {
		this._task.event = event;
	},
	
	getLaunchJavascript: function() {
		return this._task.javascript;
	},
	
	setLaunchJavascript: function(javascript) {
		this._task.javascript = javascript;
	},
	
	getLaunchApplicationParms: function() {
		return this._task.applicationParms;
	},
	
	setLaunchApplicationParms: function(applicationParms) {
		this._task.applicationParms = applicationParms;
	},
	
	/*
	 * Returns any child tasks if this task is a cascade
	 */
	getChildTasks: function() {
		if (this.isCascade()) {
			if (!this._childTasks) {
				this._childTasks = [];
				var subTaskLength = this._task.subTasks.length;
				for (var index = 0;  index < subTaskLength;  ++index) {
					this._childTasks[index] = new ibm.tivoli.navmodel.Task({task: this._task.subTasks[index], dataset: this._dataset});
				}
			}
			
			return this._childTasks;
		}
		
		return null;
	},
	
	setChildTasks: function(childTasks) {
		if (this.isCascade()) {
			this._childTasks = childTasks;
		}
	},
	
	/*=====
	result_onComplete: function(result, kwArgs){},
	=====*/
	
	/*=====
	_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	execute_kwArgs: function(onComplete, onError, selection, sync) {
		// onComplete: result_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for successful fetch
		// selection: Object:
		//		ibm.tivoli.navmodel.Selection object
		//		if selection is specified: itemIds, linkIds, itemIndexes is ignored
		// selectionParms: Object:
		//		extra parameters that can be included in the selection when PUT
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	execute: function(/*execute_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously execute remote task
		
		// this is only for executing regular tasks
		if (this.isNavigation()) {
			alert('execute is only for non-navigation tasks');
			return;
		}
		
		var args = {                
			onComplete: function(result, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},         
			selection: null,
			selectionParms: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		// get selection and add additional parameters	
		var selection = args.selection.getAttributes();
		if (args.selectionParms) {
			selection.parameters = args.selectionParms;
		}
			
		var json = dojo.toJson(selection, false);
			
		return dojo.xhrPost ({
			url: this._getParamURL({}),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version()),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "taskContext"),
				'Content-Length': json.length				
			},
			handleAs: "json",
			postData: json,
			sync: args.sync,		
			load: dojo.hitch(this, function(result, ioArgs) {
				if (this._trace.isFine()) {
					this._trace.fine("Execute for taskId=" + this.getId() + " returned successfully", "execute");
				}
							
				args.onComplete(result, args);				
			}),
			error: dojo.hitch(this, function(error, ioArgs) {
				args.onError(error, ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "execute", this._trace, this._version()), ioArgs);
			})
		});		
	},
	
	/*=====
	dataset_onComplete: function(dataset, kwArgs){},
	=====*/
	
	/*=====
	navigate_kwArgs: function(onComplete, onError, selection, sync) {
		// onComplete: dataset_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for successful fetch
		// selection: Object:
		//		ibm.tivoli.navmodel.Selection object
		//		if selection is specified: itemIds, linkIds, itemIndexes is ignored
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	navigate: function(/*navigate_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously execute navigation task (i.e. returns new Dataset)
		
		// this is only for executing regular tasks
		if (!this.isNavigation()) {
			alert('navigate is only for navigation tasks');
			return;
		}
		
		var args = {                
			onComplete: function(dataset, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			taskId: this.getId(),
			taskCategory: this.getCategory(),                
			selection: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		// call the dataset to do the navigation
		this._dataset.navigate(args);
	},	
	
	/////////////////////////////
	// private methods
	/////////////////////////////
	
	_version: function() {
		return this._dataset.getVersion();
	},

	_getParamURL: function(kwArgs) {		
		// summary:
		//		Build url necessary to do reference task and also includes
		//		all of the "param_" and "version" arguments
		var url = this.getUri();
		var questionMark = false;
		
		// add "param_" parms
		var parameters = this._dataset.getArg("parameters");
		for (var parm in parameters) {
			if (parameters.hasOwnProperty(parm)) {
				if (!questionMark) {
					questionMark = true;
					url += "?";
				}
				else {
					url += "&";
				}
				url += ("param_" + parm + "=" + parameters[parm]);				
			}
		}
		
		// IE will sometimes return http requests from cache without unique parms
		var now = new Date();
		if (!questionMark) {
			questionMark = true;
			url += "?d=" + now.getTime() + "&r=" + Math.random();
		}
		else {
			url += "&d=" + now.getTime() + "&r=" + Math.random();
		}
		
		// add "category"
		if(!questionMark) {
			questionMark = true;
			url += "?category=" + this.getCategory();
		}
		else {
			url += "&category=" + this.getCategory();
		}
		
		// indicate selected items
		if (kwArgs.itemIds) {
			if (!questionMark) {
				questionMark = true;
				url += "?itemIds=" + kwArgs.itemIds;
			}
			else {
				url += "&itemIds=" + kwArgs.itemIds;
			}			
		}
		
		// indicate selected links
		if (kwArgs.linkIds) {
			if (!questionMark) {
				questionMark = true;
				url += "?linkIds=" + kwArgs.linkInds;
			}
			else {
				url += "&linkIds=" + kwArgs.linkIds;
			}			
		}
		
		// indicate seleted item ranges
		if (kwArgs.itemIndexes) {
			if (!questionMark) {
				questionMark = true;
				url += "?itemIndexes=" + kwArgs.itemIndexes;
			}
			else {
				url += "&itemIndexes=" + kwArgs.itemIndexes;
			}			
		}
		
		return url;	
	}
});

});

},
'url:ibm/usmi/core/framework/templates/ContentTitle.html':"<div data-dojo-attach-point=\"titleContainer\" class=\"contentTitleContainer\">\r\n<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n\t<span class=\"contentTitle\"></span>\r\n\t<span class=\"contentStatus\"></span>\r\n</div>\r\n",
'ibm/tivoli/tip/TCTNavModel':function(){
// wrapped by build app
define("ibm/tivoli/tip/TCTNavModel", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/DatasetItemsStore,ibm/tivoli/navmodel/DatasetLinksStore,ibm/tivoli/navmodel/DatasetRelationshipsStore,ibm/tivoli/navmodel/DatasetLazyTreeStore,ibm/tivoli/navmodel/DatasetTreeStore,ibm/tivoli/navmodel/Dataset,ibm/tivoli/navmodel/ClientDataset,ibm/tivoli/navmodel/Item,ibm/tivoli/navmodel/ItemStyle,ibm/tivoli/navmodel/Link,ibm/tivoli/navmodel/LinkStyle,ibm/tivoli/navmodel/Relationship,ibm/tivoli/navmodel/Property,ibm/tivoli/navmodel/Task,ibm/tivoli/navmodel/Status,ibm/tivoli/navmodel/Column,ibm/tivoli/navmodel/EventRouter,ibm/tivoli/navmodel/Event,ibm/tivoli/navmodel/Selection,ibm/tivoli/navmodel/NavWidgetDef,ibm/tivoli/navmodel/TopoWidgetDef,ibm/tivoli/navmodel/TableWidgetDef,ibm/tivoli/navmodel/TreeTableWidgetDef,ibm/tivoli/navmodel/NotebookWidgetDef,ibm/tivoli/navmodel/Exception,ibm/tivoli/navmodel/Utils,ibm/tivoli/navmodel/ProviderUtils,ibm/tivoli/navmodel/WidgetDefUtils,ibm/tivoli/navmodel/DatasetUtils,ibm/tivoli/navmodel/Trace"], function(dijit,dojo,dojox){
dojo.provide("ibm.tivoli.tip.TCTNavModel");
dojo.require("ibm.tivoli.navmodel.DatasetItemsStore");
dojo.require("ibm.tivoli.navmodel.DatasetLinksStore");
dojo.require("ibm.tivoli.navmodel.DatasetRelationshipsStore");
dojo.require("ibm.tivoli.navmodel.DatasetLazyTreeStore");
dojo.require("ibm.tivoli.navmodel.DatasetTreeStore");
dojo.require("ibm.tivoli.navmodel.Dataset");
dojo.require("ibm.tivoli.navmodel.ClientDataset");
dojo.require("ibm.tivoli.navmodel.Item");
dojo.require("ibm.tivoli.navmodel.ItemStyle");
dojo.require("ibm.tivoli.navmodel.Link");
dojo.require("ibm.tivoli.navmodel.LinkStyle");
dojo.require("ibm.tivoli.navmodel.Relationship");
dojo.require("ibm.tivoli.navmodel.Property");
dojo.require("ibm.tivoli.navmodel.Task");
dojo.require("ibm.tivoli.navmodel.Status");
dojo.require("ibm.tivoli.navmodel.Column");
dojo.require("ibm.tivoli.navmodel.EventRouter");
dojo.require("ibm.tivoli.navmodel.Event");
dojo.require("ibm.tivoli.navmodel.Selection");
dojo.require("ibm.tivoli.navmodel.NavWidgetDef");
dojo.require("ibm.tivoli.navmodel.TopoWidgetDef");
dojo.require("ibm.tivoli.navmodel.TableWidgetDef");
dojo.require("ibm.tivoli.navmodel.TreeTableWidgetDef");
dojo.require("ibm.tivoli.navmodel.NotebookWidgetDef");
dojo.require("ibm.tivoli.navmodel.Exception");
dojo.require("ibm.tivoli.navmodel.Utils");
dojo.require("ibm.tivoli.navmodel.ProviderUtils");
dojo.require("ibm.tivoli.navmodel.WidgetDefUtils");
dojo.require("ibm.tivoli.navmodel.DatasetUtils");
dojo.require("ibm.tivoli.navmodel.Trace");
});

},
'ibm/usmi/dashboard/HealthTip':function(){
require({cache:{
'url:ibm/usmi/dashboard/templates/GroupHealthTip.html':"<div class=\"healthTip\">\r\n<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n    <div data-dojo-attach-point=\"containerNode\" style=\"display: none;\">\r\n        <div class=\"healthTip-all\">\r\n\t       <!-- <div class=\"entityTypeName\">${entityType}</div> -->\r\n\t        <div data-dojo-attach-point=\"healthTipBody\" class=\"healthTipBody\">\r\n\t\t\t\t<div id=\"recentEventLogs\" data-dojo-attach-point=\"recentEventLogs\">\r\n\t\t\t\t    <span class=\"healthTip-title\">${_resources.RecentEvents} </span>\r\n\t\t\t\t\t<span data-dojo-attach-point=\"noEvents\" class=\"healthTip-title-no-contents\">${_resources.NoRecentEvents}</span>\r\n\t\t\t\t\t<div id=\"eventLogTable\" data-dojo-attach-point=\"eventLogTable\">\r\n\t\t\t\t\t</div>\r\n\t\t\t\t</div>\r\n\t\t\t\t<div id=\"recentActiveAlerts\" data-dojo-attach-point=\"recentActiveAlerts\">\r\n\t\t\t\t\t<span class=\"healthTip-title\">${_resources.RecentAlerts} </span>\r\n\t\t\t\t\t<span data-dojo-attach-point=\"noAlerts\" class=\"healthTip-title-no-contents\">${_resources.NoRecentAlerts}</span>\r\n\t\t\t\t\t<div id=\"activeAlertsTable\" data-dojo-attach-point=\"activeAlertsTable\">\r\n\t\t\t\t</div>\r\n            </div>\r\n        </div>\r\n    </div>\r\n</div>\r\n"}});
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/dashboard/HealthTip", ["dojo/_base/declare",
		"dijit/_WidgetBase",
		"dijit/_TemplatedMixin",
		"dojo/text!./templates/GroupHealthTip.html",
		"dijit/_WidgetsInTemplateMixin",
		//"dijit/Tooltip",
		"dojo/_base/lang",
		"./DatasetItemStoreAdapter",
		"idx/oneui/HoverHelpTooltip",
    	"dojo/i18n!./nls/DashbarResources"
	], function (declare, WidgetBase, TemplatedMixin, template, WidgetsInTemplateMixin,
	lang, DatasetItemStoreAdapter, HovrHelpTooltip, mri) {
	return declare([WidgetBase, TemplatedMixin, WidgetsInTemplateMixin], {

		templateString : template,
		entityNode: "",
		entityType: "",
		healthTipBody:"",
		eventLogs: [],
		activeAlerts: [],
		defaultPosition: ["below", "after"],
		sourceToken: null,
		showDelay: 500,
		hideDelay: 500,
		
        //resource bundle
        _resources : mri,
    
		postCreate: function() {
			this.inherited(arguments);
			this.update();
		},
    
		show: function() {
			if(!this._showTimer){
				this._showTimer = setTimeout(lang.hitch(this, function(){
					if(this._showTimer){
						clearTimeout(this._showTimer);
						delete this._showTimer;
					}
					idx.oneui.showHoverHelpTooltip(this.containerNode.innerHTML, this.entityNode, this.defaultPosition); //jlc
					//we do not want to use the close button from oneui HoverHelp
					if (idx.oneui._masterTT.closeButtonNode.style.display !== "none") { //jlc
						idx.oneui._masterTT.closeButtonNode.style.display="none"; //jlc
					}
					this._connectNode = this.entityNode;
				}), this.showDelay);
			}
			
			if(this._hideTimer){
				clearTimeout(this._hideTimer);
				delete this._hideTimer;
			}

		},
    
		hide: function(immediate) {
			immediate = immediate || false;
			if(this._showTimer){
				clearTimeout(this._showTimer);
				delete this._showTimer;
			}
			if(!this._hideTimer){
				if(immediate){
					//immediate to hide
					if(this._connectNode){
						idx.oneui.hideHoverHelpTooltip(this.entityNode); //jlc
						//oneui.hideHoverHelp(this.entityNode);
						delete this._connectNode;
					}
					return;
				}
				this._hideTimer = setTimeout(lang.hitch(this, function(){
					if(this._connectNode){
						idx.oneui.hideHoverHelpTooltip(this.entityNode); //jlc
						//oneui.hideHoverHelp(this.entityNode);
						delete this._connectNode;
					}
					if(this._showTimer){
					// if HoverHelp is scheduled to be shown (after a brief delay)
						clearTimeout(this._showTimer);
						delete this._showTimer;
					}
				}), this.hideDelay);
			}
			
		},
    
		update: function() {
			var datasetstore = new DatasetItemStoreAdapter({dataset: "EVENT_LOG", sourceToken: this.sourceToken});
			var deffered=datasetstore.getStore();
			deffered.then(lang.hitch(this, this._updateEventLogTable));
			
			var datasetstore2 = new DatasetItemStoreAdapter({
				dataset: "ACTIVE_ALERTS",
				_defaultProperties: ["Membername", "Severity", "DateTime", "SetName"],
				sourceToken: this.sourceToken
			});
			var deffered2=datasetstore2.getStore();
			deffered2.then(lang.hitch(this, this._updateActiveAlertsTable));
			
		},
		
		_updateEventLogTable: function(items){
			items=items || [];
			var itemIndex;
			var itemsLength = items.length;
			if(itemsLength === 0){
				this.noEvents.style.visibility="visible";
				return;
			} 
						
			var html=["<table class='healthTipTable'><tr class='healthTipTableHead'>",
			"<th class='healthTipTableColumn_Severity'></th>",
			"<th class='healthTipTableColumn_Source'>"+this._resources.Source+"</th>",
			"<th class='healthTipTableColumn_DateTime'>"+this._resources.DateAndTime+"</th>",
			"<th>"+this._resources.DateAndTime+"</th></tr>"];
				
			var setBgcolor=false;
			for(itemIndex=0;  itemIndex < itemsLength;  ++itemIndex) {
				var item=items[itemIndex];
				if(setBgcolor){
					html.push("<tr class='healthTipTableRow'>");
					setBgcolor=false;
				} else {
					html.push("<tr>");
					setBgcolor=true;
				}
				html.push("<td class='healthTipTableColumn_Severity'><img src='");
				html.push(item.getProperty("Severity").getValue().getImage());
				html.push("'>");
				html.push("</td><td>");
				html.push(item.getProperty("Source").getValue());
				html.push("</td><td>");
				var dateTime=item.getProperty("DateTime").getValue();
				dateTime=this._formatDateStr(dateTime);
				html.push(dateTime);
				html.push("</td><td>");
				html.push(item.getProperty("EventText").getValue());
				html.push("</td></tr>");
			}
			html.push("</table>");
			var result=html.join("");
			//console.debug(result);
			this.eventLogTable.innerHTML=result;
		},
		
		_updateActiveAlertsTable: function(items){
			items=items || [];
			var itemIndex = 0;
			var itemsLength = items.length;
			if(itemsLength === 0){
				this.noAlerts.style.visibility="visible";
				return;
			} 
			var html=["<table class='healthTipTable'><tr class='healthTipTableHead'>",
				"<th class='healthTipTableColumn_Severity'></th>",
				"<th class='healthTipTableColumn_Source'>"+this._resources.Source+"</th>",
				"<th class='healthTipTableColumn_DateTime'>"+this._resources.DateAndTime+"</th>",
				"<th>"+this._resources.DateAndTime+"</th></tr>"];
				
			var setBgcolor=false;
			for(itemIndex=0;  itemIndex < itemsLength;  ++itemIndex) {
				var item=items[itemIndex];
				if(setBgcolor){
					html.push("<tr class='healthTipTableRow'>");
					setBgcolor=false;
				} else {
					html.push("<tr>");
					setBgcolor=true;
				}
				html.push("<td class='healthTipTableColumn_Severity'><img src='");
				html.push(item.getProperty("Severity").getValue().getImage());
				html.push("'>");
				html.push("</td><td>");
				html.push(item.getProperty("Membername").getValue());
				html.push("</td><td>");
				var dateTime=item.getProperty("DateTime").getValue();
				dateTime=this._formatDateStr(dateTime);
				html.push(dateTime);
				html.push("</td><td>");
				html.push(item.getProperty("SetName").getValue());
				html.push("</td></tr>");
			}
			html.push("</table>");
			var result2=html.join("");
			//console.debug(result2);
			this.activeAlertsTable.innerHTML=result2;
		},
		
		_formatDateStr: function(dateTime){
			var formatedStr;
			var dateStrArr;
			if(dateTime){
				dateStrArr=dateTime.toString().split(" ");
				formatedStr=dateStrArr.slice(1, 5).join(" ");
			}
			//console.debug("shorten date is:" + formatedStr);
			return formatedStr;
		}
	});
});
},
'ibm/tivoli/navmodel/DatasetStore':function(){
// wrapped by build app
define("ibm/tivoli/navmodel/DatasetStore", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/Dataset"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */
 
dojo.provide("ibm.tivoli.navmodel.DatasetStore");

// includes
dojo.require("ibm.tivoli.navmodel.Dataset");
	
ibm.tivoli.navmodel.DatasetStore._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.DatasetStore");
	
dojo.declare("ibm.tivoli.navmodel.DatasetStore", null, {
	// private members 
	_listening: false,
	_refresh: false,
	_sortIds: null,
	_conditionIds: null,	
	_trace: ibm.tivoli.navmodel.DatasetStore._trace,
	
	/*=====
	constructor_kwArgs: function(dataset, defaultProperties, returnItem) {
		// summary:
		//		An anonymous object used to configure the datastore
		// dataset: Object
		//		A ibm.tivoli.navmodel.Dataset instance
		// defaultProperties: String[]
		//		An array of property id values that are pre-fetched with the items
		// returnItem: Boolean
		//		A boolean indicating if the getValue() method should just return the actual
		//		item so that it can be used for custom formatting of the value
	},
	=====*/
	
	constructor: function(/*constructor_kwArgs*/ kwArgs){
		// summary:
		//		Constructor for abstract base class dojo datastore;
		//		that receives ids data from an ibm.tivoli.navmodel.Dataset
		var args = {
			dataset: null,
			defaultProperties: ["all"],
			returnItem: false
		};
		dojo.mixin(args, kwArgs);
		dojo.mixin(this, args);
		
		// set up Dataset event handler
		this._setupEventHandler();			
    },
    
    CURIRefresh: function() {
		// summary:
		//		invoked by TED to indicate "Refresh" is occurring now
		this._refresh = true;
		
		if (this.dataset.isClearCacheOnRefresh()) {
			this.dataset._clearAllCaches();
		}
	},
	
	isRefresh: function() {
		// summary:
		//		returns true if TED refresh is occurring
		var refresh = this._refresh;
		this._refresh = false;
		return refresh;
	},
	

	//////////////////////////////////////////////////////////////
	// dojo.data.api.Identity
	//////////////////////////////////////////////////////////////
	
	getIdentityAttributes: function(/*Object*/ item){
		// summary:
		//		Returns an array of attribute names that are used to generate the identity.
		//		For most stores, this is a single attribute, but for some complex stores
		//		such as RDB backed stores that use compound (multi-attribute) identifiers
		//		it can be more than one. If the identity is not composed of attributes
		//		on the item, it will return null. This function is intended to identify
		//		the attributes that comprise the identity so that so that during a render
		//		of all attributes, the UI can hide the the identity information if it
		//		chooses.
		// item:
		//		The item from the store from which to obtain the array of public attributes that
		//		compose the identifier, if any.
		if (!this.isItem(item)){
  			throw new Error("Specified item argument is not a valid item.");
  		}
  		
		return null;
	},
    
    
	//////////////////////////////////////////////////////////////
	// dojo.data.api.Read
	//////////////////////////////////////////////////////////////
	
	getFeatures: function(){    	
		// summary:
		//		The getFeatures() method returns an simple keyword values object
		//		that specifies what interface features the datastore implements.
		return {
			'dojo.data.api.Read': true,
			'dojo.data.api.Identity': true,
			'dojo.data.api.Notification':true
		};
	},
		
	close: function(/*Object*/ request) {
		// summary:
		//		The close() method is intended for instructing the store to 'close' out
		//		any information associated with a particular request.
		// description:
		//		The close() method is intended for instructing the store to 'close' out
		//		any information associated with a particular request.  In general, this API
		//		expects to receive as a parameter a request object returned from a fetch.
		//		It will then close out anything associated with that request, such as
		//		clearing any internal datastore caches and closing any 'open' connections.
		//		For some store implementations, this call may be a no-op.
		// request:
		//		An instance of a request for the store to use to identify what to close out.
		//		If no request is passed, then the store should clear all internal caches (if any)
		//		and close out all 'open' connections.  It does not render the store unusable from
		//		there on, it merely cleans out any current data and resets the store to initial
		//		state.	
		if (!request) {
			this._cleanupEventHandler();
		}		
	},
  
	isItem: function(/*Object*/ something){
		// summary:
		//		Returns true if *something* is an item and came from the store instance.
		//		Returns false if *something* is a literal, an item from another store instance,
		//		or is any object other than an item.
		// something:
		//		Can be anything.
		if (!something || (typeof something !== "object")) {
			return false;
		}
                
		try {
			return this._getObjectTypes()[something.getObjectType()];			
		}
		catch(err){
			return false;    		
		}	
	},
	
	isItemLoaded: function(/*Object*/ item) {
		// summary:
		//		Returns false if isItem(something) is false.  Returns false if
		//		if isItem(something) is true but the the item is not yet loaded
		//		in local memory (for example, if the item has not yet been read
		//		from the server).
		// item:
		//		Item previously handed out by datastore.
		//		ibm.tivoli.navmodel.Item object
		if (!item || !this.isItem(item)){
  			return false;
  		}
  		
  		return !item.isExpired();
	},	
	
	getLabel: function(/*Object*/ item) {
		// summary:
		//		Method to inspect the item and return a user-readable 'label' for the item
		//		that provides a general/adequate description of what the item is.
		// item:
		//		The item to return the label for.
		//		ibm.tivoli.navmodel.Item object
		// returns:
		// 		A user-readable string representing the item or undefined if no user-readable label can
		//		be generated.	
		if (!this.isItem(item)){
  			throw new Error("Specified item argument is not a valid item.");
  		}
  		
		return item.getLabel();
	},
  
	getLabelAttributes: function(/*Object*/ item) {
		// summary:
		//		Method to inspect the item and return an array of what attributes of the item were used
		//		to generate its label, if any.
		// item:
		//		The item to return the list of label attributes for.
		//		ibm.tivoli.navmodel.Item object
		// returns:
		//		An array of attribute names that were used to generate the label, or null if public attributes
		//		were not used to generate the label.  
		if (!this.isItem(item)){
  			throw new Error("Specified item argument is not a valid item.");
  		}
  		
		return ["label"];
	},
  
 	getAttributes: function(/*Object*/ item) {
		// summary:
		//		Returns an array with all the attributes that this item has.  This
		//		method will always return an array; if the item has no attributes
		//		at all, getAttributes() will return an empty array: [].
		// item:
		//		The item to access attributes on.
		//		ibm.tivoli.navmodel.Item object
		// exceptions:
		//		Throws an exception if *item* is not an item
  		if (!this.isItem(item)){
  			throw new Error("Specified item argument is not a valid item.");
  		}
  		
  		return item.getCachedPropertyIds(true);  		
    },
    
	hasAttribute: function(/*Object*/ item, /*String*/ attribute) {
		// summary:
		//		Returns true if the given *item* has a value for the given *attribute*.
		// item:
		//		The item to access attributes on.
		//		ibm.tivoli.navmodel.Item object
		// attribute:
		//		The attribute to access represented as a string.
		// exceptions:
		//		Throws an exception if *item* is not an item, or *attribute* is not a string		
		if (!item) {
			return false;
		}
		
		if (!this.isItem(item)){
			throw new Error("Specified item argument is not a valid item.");
		}
    	
		if (!dojo.isString(attribute)) {
			throw new Error("Specified attribute argument is not a string.");
		}
        
		var attributes = this.getAttributes(item);
		if (attributes) {
			var attrsLength = attributes.length;
			for (var attrIndex = 0;  attrIndex < attrsLength;  ++ attrIndex) {
				if (attribute === attributes[attrIndex]) {
					return true;
				}
			}
		}
    	
		return false;
	},
    
  getValues: function(/*Object*/ item, /*String*/ attribute, /*Object?*/ defaultValue, /*Object*/ useValue) {
		// summary:
		//		This getValues() method works just like the getValue() method, but getValues()
		//		always returns an array rather than a single attribute value.  The array
		//		may be empty, may contain a single attribute value, or may contain many
		//		attribute values.
		//		
		//		If the item does not have a value for the given attribute, then getValues()
		//		will return an empty array: [].  (So, if store.hasAttribute(item, attribute)
		//		returns false, then store.getValues(item, attribute) will return [].)
		// item:
		//		The item to access values on.
		//		ibm.tivoli.navmodel.Item object
		// attribute:
		//		The attribute to access represented as a string.
		// exceptions:
		//		Throws an exception if *item* is not an item, or *attribute* is not a string 	
    	return [this.getValue(item, attribute, defaultValue, useValue)];
    },
    
	containsValue: function(/*Object*/ item, /*String*/ attribute, /*Object*/ value) {
		// summary:
		//		Returns true if the given *value* is one of the values that getValues()
		//		would return.
		// item:
		//		The item to access values on.
		//		ibm.tivoli.navmodel.Item object
		// attribute:
		//		The attribute to access represented as a string.
		// value:
		//		The value to match as a value for the attribute.
		// exceptions:
		//		Throws an exception if *item* is not an item, or *attribute* is not a string
		var values = this.getValues(item, attribute);
		if (values) {
			var valuesLength = values.length;
				for (var valueIndex = 0;  valueIndex < valuesLength;  ++valueIndex) {
				if (value === values[valueIndex]) {
					return true;
				}
			}
		}
    	
		return false;
    },
	
	
    //////////////////////////////////////////////////////////////
	// dojo.data.api.Notification
	//////////////////////////////////////////////////////////////
	
	onSet: function(/*Object*/ item, 
					/*String*/ attribute, 
					/*Object[]*/ oldValue,
					/*Object[]*/ newValue) {
		// summary: 
		//		See dojo.data.api.Notification.onSet()
		
		// No need to do anything. This method is here just so that the 
		// client code can connect observers to it.
	},

    onNew: function(/*Object*/ newItem, /*Object?*/ parentInfo) {
		// summary: 
		//		See dojo.data.api.Notification.onNew()
		
		// No need to do anything. This method is here just so that the 
		// client code can connect observers to it. 
	},

    onDelete: function(/*Object*/ deletedItem) {
		// summary: 
		//		See See dojo.data.api.Notification.onDelete()
		
		// No need to do anything. This method is here just so that the 
		// client code can connect observers to it. 
	},
	
	onRefresh: function() {
		// summary: 
		//		This is NOT part of the dojo.data.api.Notification but can be attached to
		//		to indicate that the datastore items have changed dramatically and should
		//		be re-fetched.
		//
		// 		Refresh is coming as a result of async notification instead
		// 		of the actual Refresh button being clicked on
		this._refresh = true;		
	},
		
		
	//////////////////////////////////////////////////////////////
	// private methods
	///////////////////////////////////////////////////////////////
   
	_hasSort: function() {
		// summary: 
		//		Returns true if a sort is active
		return (this._sortIds ? true : false);
	},
	
	_affectsSort: function(/*String*/ propId) {
		// summary: 
		//		Returns true if propId is involved in sort
		return (this._sortIds && this._sortIds[propId]);
	},
	
	_hasCondition: function() {
		// summary: 
		//		Returns true if the filter condition(s) are active
		return (this._conditionIds ? true : false);
	},
	
	_affectsCondition: function(/*String*/ propId) {
		// summary: 
		//		Returns true if propId is involved in filter condition(s)
		return (this._conditionIds && this._conditionIds[propId]);
	},
	
	_getSort: function(/*Object[]*/ sort) {
		// summary: 
		//		Returns comma delimited sort string; indicating descending sort with a preceding "-"
 		this._sortIds = null;		
		var sortOrder = null;	 	
		if (sort && sort.length > 0) {			
			this._sortIds = {};						
			sortOrder = "";
			var sortLength = sort.length;
			for (var sortIndex = 0;  sortIndex < sortLength;  ++sortIndex){
				if (sortIndex > 0) {
					sortOrder += ",";
				}
				
				if (sort[sortIndex].descending) {
					sortOrder += "-";
				}
				
				var propId = sort[sortIndex].attribute;
				sortOrder += propId;
				
        		// remember sort ids
        		this._sortIds[propId] = true;				
        	}        	        	
        }        
        return sortOrder;
 	},
 		
 	_getCondition: function(/*Object*/ query) {
		// summary: 
		//		Returns array of condition objects involved in the current filter
		//			ibm.tivoli.navmodel.[String|Number|Date|Boolean|Enum|Map]Condition
		this._conditionIds = null;
		var condition = null;
 		 		
		if (query) {
			// new query format
			if (query.expression && query.ids) {
				this._conditionIds = query.ids;
				condition = query.expression;
			}
			// old "search" format compatible with original SAPM datastore
			else if (query.navModel && query.navModel.search && query.navModel.search.term) {
				this._conditionIds = {};
				condition = "(";
				
				var searchLength = query.navModel.search.attributes.length;
 				for (var searchIndex = 0;  searchIndex < searchLength;  ++searchIndex) {
 					if (searchIndex > 0) {
 						condition += " or ";
 					}
 					
 					var id = query.navModel.search.attributes[searchIndex];
 					condition += (id + " contains '" + query.navModel.search.term + "'");
 					this._conditionIds[id] = true;
 				}
 				
 				condition += ")";
			}
		}
		
		// 22350  (moved to _appendCustomFilterCondition(condition) )
		// 38638
		condition = this._appendCustomFilterCondition(condition);
 		
		return condition;
 	},
 	
 	// 38638
 	// Add the custom filter condition (if it exists) to condition and
 	// return the modified string
 	_appendCustomFilterCondition: function(condition) {
		// Check for custom condition on the dataset and add it in if present
		if (this.dataset && this.dataset.getCustomFilterCondition()) {
			var customCondition = this.dataset.getCustomFilterCondition();
			if (condition) {
				// Is condition already wrapped in parens?
				if (condition.indexOf("(") == 0) {
					// Add opening paren for combined expression
					condition = "(" + condition;
				}
				else {
					// Add opening paren for combined expression
					condition = "((" + condition;
				}
				// Does original condition have closing paren?
				if (condition.lastIndexOf(")") == condition.length - 1) {
					// Just add the "and" condition connector
					condition = condition += " and ";
				}
				else {
					// Add the closing paren and the "and" condition connector
					condition = condition += ") and ";
				}
				// Is custom condition already parenthesized?
				if (customCondition.indexOf("(") == 0) {
					// Just add it on
					condition += customCondition;
				}
				else {
					// Parenthesize it and add it on
					condition = condition + "(" + customCondition + ")";
				}
				// Add closing paren for combined expression
				condition += ")";
			}
			else {
				// the custom filter condition is the only expression
				condition = customCondition;
				if (condition.indexOf("(") != 0) {
                    condition = "(" + condition + ")";
				}
			}
		}	
		return condition;
 	},
 	
 	_validateEventHandler: function() {
 		// summary: 
		//		Ensure that we are listening for events if we are supposed to be;
		//		a close() invocation could have caused us to stop listening
		if (!this._listening) {
			this._setupEventHandler();
		}
	},
    
 	_setupEventHandler: function() {
		// summary: 
		//		Add event listener to the dataset
 		if (!this._listening && this.dataset && this.dataset.isEventsEnabled()) {
			this.dataset.addEventListener(this, dojo.hitch(this, this._eventHandler));
			this._listening = true;
		}
 	},
 	
 	_cleanupEventHandler: function() {
		// summary: 
		//		Remove event listener from the dataset
 		if (this._listening && this.dataset && this.dataset.isEventsEnabled()) {
			this.dataset.removeEventListener(this);
			this._listening = false;			
		}
 	},
 	
	_eventHandler: function(/*Object*/ listenerObj, /*Object*/ event) {
		// summary: 
		//		Handle asynchronous events from the dataset
 		if (this._trace.isFine()) {
				this._trace.fine("datasetId=" + event.getDataset().getId() + " event=" + event.getType() +  " eventId=" + event.getId(), "_eventHandler");
		}
			
		switch(event.getType()) {
			case "changed":
				this._handleChangedEvent(event);
				break;
			case "labelChanged":
				this._handleLabelChangedEvent(event);
				break;
			case "statusChanged":
				this._handleStatusChangedEvent(event);
				break;					
			case "itemsAdded":
				this._handleItemsAddedEvent(event);
				break;
			case "itemsRemoved":
				this._handleItemsRemovedEvent(event);
				break;
			case "itemStatusChanged":
				this._handleItemStatusChangedEvent(event);
				break;
			case "itemPropertiesChanged":
				this._handleItemPropertiesChangedEvent(event);
				break;
			case "linksAdded":
				this._handleLinksAddedEvent(event);
				break;
			case "linksRemoved":
				this._handleLinksRemovedEvent(event);
				break;
			case "linkStatusChanged":
				this._handleLinkStatusChangedEvent(event);
				break;
			case "linkPropertiesChanged":
				this._handleLinkPropertiesChangedEvent(event);
				break;
			case "linkChanged":
				this._handleLinkChangedEvent(event);
				break;
			case "relationshipStatusChanged":
				this._handleRelationshipStatusChangedEvent(event);
				break;
			case "relationshipPropertiesChanged":
				this._handleRelationshipPropertiesChangedEvent(event);
				break;
			case "exception":
				this._handleExceptionEvent(event);
				break;
			case "eventRouter":
				this._handleEventRouterEvent(event);
				break;
			default:
				if (this._trace.isFine()) {
					this._trace.fine("datasetId=" + event.getDataset().getId() + " unrecognized event=" + event.getType(), "_eventHandler");
				}									
		}
	},
	
	_handleChangedEvent: function(/*Object*/ event) {
		// signal that data from store should be re-requested by consumer
		this.onRefresh();	
	},
	
	_handleLabelChangedEvent: function(/*Object*/ event) {		
	},
	
	_handleStatusChangedEvent: function(/*Object*/ event) {		
	},
	
	_handleItemsAddedEvent: function(/*Object*/ event) {		
	},
	
	_handleItemsRemovedEvent: function(/*Object*/ event) {		
	},
	
	_handleItemStatusChangedEvent: function(/*Object*/ event) {		
	},
	
	_handleItemPropertiesChangedEvent: function(/*Object*/ event) {		
	},
	
	_handleLinksAddedEvent: function(/*Object*/ event) {	
	},
	
	_handleLinksRemovedEvent: function(/*Object*/ event) {		
	},
	
	_handleLinkStatusChangedEvent: function(/*Object*/ event) {		
	},
	
	_handleLinkPropertiesChangedEvent: function(/*Object*/ event) {				
	},
	
	_handleLinkChangedEvent: function(/*Object*/ event) {		
	},
	
	_handleRelationshipStatusChangedEvent: function(/*Object*/ event) {			
	},
	
	_handleRelationshipPropertiesChangedEvent: function(/*Object*/ event) {		
	},
	
	_handleExceptionEvent: function(/*Object*/ event) {		
	},
	
	_handleEventRouterEvent: function(/*Object*/ event) {		
	}
}); 

});

},
'url:dijit/templates/MenuBar.html':"<div class=\"dijitMenuBar dijitMenuPassive\" data-dojo-attach-point=\"containerNode\"  role=\"menubar\" tabIndex=\"${tabIndex}\" data-dojo-attach-event=\"onkeypress: _onKeyPress\"></div>\n",
'dojo/data/util/filter':function(){
define(["dojo/_base/lang"], function(lang) {
	// module:
	//		dojo/data/util/filter
	// summary:
	//		TODOC

var filter = lang.getObject("dojo.data.util.filter", true);

filter.patternToRegExp = function(/*String*/pattern, /*boolean?*/ ignoreCase){
	//	summary:
	//		Helper function to convert a simple pattern to a regular expression for matching.
	//	description:
	//		Returns a regular expression object that conforms to the defined conversion rules.
	//		For example:
	//			ca*   -> /^ca.*$/
	//			*ca*  -> /^.*ca.*$/
	//			*c\*a*  -> /^.*c\*a.*$/
	//			*c\*a?*  -> /^.*c\*a..*$/
	//			and so on.
	//
	//	pattern: string
	//		A simple matching pattern to convert that follows basic rules:
	//			* Means match anything, so ca* means match anything starting with ca
	//			? Means match single character.  So, b?b will match to bob and bab, and so on.
	//      	\ is an escape character.  So for example, \* means do not treat * as a match, but literal character *.
	//				To use a \ as a character in the string, it must be escaped.  So in the pattern it should be
	//				represented by \\ to be treated as an ordinary \ character instead of an escape.
	//
	//	ignoreCase:
	//		An optional flag to indicate if the pattern matching should be treated as case-sensitive or not when comparing
	//		By default, it is assumed case sensitive.

	var rxp = "^";
	var c = null;
	for(var i = 0; i < pattern.length; i++){
		c = pattern.charAt(i);
		switch(c){
			case '\\':
				rxp += c;
				i++;
				rxp += pattern.charAt(i);
				break;
			case '*':
				rxp += ".*"; break;
			case '?':
				rxp += "."; break;
			case '$':
			case '^':
			case '/':
			case '+':
			case '.':
			case '|':
			case '(':
			case ')':
			case '{':
			case '}':
			case '[':
			case ']':
				rxp += "\\"; //fallthrough
			default:
				rxp += c;
		}
	}
	rxp += "$";
	if(ignoreCase){
		return new RegExp(rxp,"mi"); //RegExp
	}else{
		return new RegExp(rxp,"m"); //RegExp
	}

};

return filter;
});

},
'ibm/tivoli/navmodel/ItemStyle':function(){
// wrapped by build app
define("ibm/tivoli/navmodel/ItemStyle", ["dijit","dojo","dojox"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.ItemStyle"); 
 
////////////////////////////////////
// ItemStyle
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.ItemStyle", null, {
	// private members       
	_version: null,
	_style: null,
	
	/*=====
	constructor_kwArgs: function(version, style) {
		// version: Float
		//		version used to communicate with rest service
		// style: Object
		//		json item style object
	},
	=====*/

	 constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary
		//		Constructs ItemStyle object.
		this._version = kwArgs.version;
		this._style = kwArgs.style;		
		if (!this._style) {
			this._style = {};
		}
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	
	getAttribute: function(key) {
		return this._style[key];
	},
	
	setAttribute: function(key, value) {
		this._style[key] = value;
	},
	
	getId: function() {
		return this._style.id;
	},
	
	setId: function(id) {
		this._style = id;
	},
	
	getType: function() {
		return this._style.type;
	},
	
	setType: function(type) {
		this._style.type = type;
	},
	
	getLabel: function() {
		if (this._style.label) {
			// string
			if (typeof this._style.label === "string") {
				return this._style.label;
			}
			// object with key/value info
			else {
				return this._style.label.value;
			}
		}
		
		return null;
	},
	
	setLabel: function(label) {
		this._style.label = label;		
	},
	
	getDescription: function() {
		if (this._style.description) {
			// string
			if (typeof this._style.description === "string") {
				return this._style.description;
			}
			// object with key/value info
			else {
				return this._style.description.value;
			}
		}
		
		return null;		
	},
	
	setDescription: function(description) {
		this._style.description = description;			
	},
	
	getSmallImage: function() {
		return this._style.smallImage;
	},
	
	setSmallImage: function(image) {
		this._style.smallImage = image;
	},
	
	getSmallImageWidth: function() {
		return this._style.smallImageWidth;
	},
	
	setSmallImageWidth: function(width) {
		this._style.smallImageWidth = width;
	},
	
	getSmallImageHeight: function() {
		return this._style.smallImageHeight;
	},
	
	setSmallImageHeight: function(height) {
		this._style.smallImageHeight = height;
	},
	
	getLargeImage: function() {
		return this._style.largeImage;
	},
	
	setLargeImage: function(image) {
		this._style.largeImage = image;
	},
	
	getLargeImageWidth: function() {
		return this._style.largeImageWidth;
	},
	
	setLargeImageWidth: function(width) {
		this._style.largeImageWidth = width;
	},
	
	getLargeImageHeight: function() {
		return this._style.largeImageHeight;
	},
	
	setLargeImageHeight: function(height) {
		this._style.largeImageHeight = height;
	},
	
	getScalarImage: function() {
		return this._style.scalarImage;
	},
	
	setScalarImage: function(image) {
		this._style.scalarImage = image;
	},
	
	getScalarImageWidth: function() {
		return this._style.scalarImageWidth;
	},
	
	setScalarImageWidth: function(width) {
		this._style.scalarImageWidth = width;
	},
	
	getScalarImageHeight: function() {
		return this._style.scalarImageHeight;
	},
	
	setScalarImageHeight: function(height) {
		this._style.scalarImageHeight = height;
	},
	
	isScalarImageVisible: function() {
		return (this._style.scalarImageVisible !== undefined ? this._style.scalarImageVisible : true);
	},
	
	setScalarImageVisible: function(visible) {
		this._style.scalarImageVisible = visible;
	},
	
	clone: function(widgetDef) {
		var args = {
			style: {},
			version: this._version
		};		
		
		if (widgetDef) {
			// start with widgetDef default style
			var widgetDefaultStyle = widgetDef.getDefaultItemStyle();
			if (widgetDefaultStyle) {
				dojo.mixin(args.style, widgetDefaultStyle);
			}
		
			// mixin our item style
			dojo.mixin(args.style, this._style);
			
			// mixin widgetDef style
			var widgetStyle = widgetDef.getItemStyle(this.getType());			
			if (widgetStyle) {
				dojo.mixin(args.style, widgetStyle);
			}
		}
		// just mixin our item style
		else {
			dojo.mixin(args.style, this._style);
		}
			
		return new ibm.tivoli.navmodel.ItemStyle(args);
	}
});

});

},
'ibm/usmi/widgets/finder/GroupDataStore':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
//Finder GroupDataStore
define("ibm/usmi/widgets/finder/GroupDataStore", [
	"dojo/_base/lang",
	"dojo/_base/declare",
	"ibm/usmi/util/Utils",
	"./FinderBaseDataStore"
], function (djLang, declare, Utils, FinderBaseDataStore) {

	return declare("itme.finder.GroupDataStore", [ FinderBaseDataStore
	], {

		constructor : function (type, cache) {
			this.populateItems(cache);
		},
		populateItems : function () {
			this.inherited(arguments);
		},

		initiateTaskSearch : function (searchString, finderObj) {
			var query = "name:'*" + searchString + "*' OR category:'*" + searchString + "*'";
			Utils.CMTraceTimeDelta("finderTimer", "before initiating search", true);
			if (this._dataStore) {
				this._dataStore.fetch({
					query : {
						complexQuery : query
					},
					queryOptions : {
						ignoreCase : true
					},
					onError : djLang.hitch(finderObj, finderObj.fetchFailed),
					onComplete : djLang.hitch(finderObj, finderObj.doGroupSearch)
				});
			} else {
				finderObj.doGroupSearch(new Array(0), null);
			}
		},
		destroy : function () {
			this.inherited(arguments);
		}
	});
});

},
'dijit/tree/TreeStoreModel':function(){
define([
	"dojo/_base/array", // array.filter array.forEach array.indexOf array.some
	"dojo/aspect", // aspect.after
	"dojo/_base/declare", // declare
	"dojo/_base/json", // json.stringify
	"dojo/_base/lang" // lang.hitch
], function(array, aspect, declare, json, lang){

	// module:
	//		dijit/tree/TreeStoreModel
	// summary:
	//		Implements dijit.Tree.model connecting to a dojo.data store with a single
	//		root item.

	return declare("dijit.tree.TreeStoreModel", null, {
		// summary:
		//		Implements dijit.Tree.model connecting to a dojo.data store with a single
		//		root item.  Any methods passed into the constructor will override
		//		the ones defined here.

		// store: dojo.data.Store
		//		Underlying store
		store: null,

		// childrenAttrs: String[]
		//		One or more attribute names (attributes in the dojo.data item) that specify that item's children
		childrenAttrs: ["children"],

		// newItemIdAttr: String
		//		Name of attribute in the Object passed to newItem() that specifies the id.
		//
		//		If newItemIdAttr is set then it's used when newItem() is called to see if an
		//		item with the same id already exists, and if so just links to the old item
		//		(so that the old item ends up with two parents).
		//
		//		Setting this to null or "" will make every drop create a new item.
		newItemIdAttr: "id",

		// labelAttr: String
		//		If specified, get label for tree node from this attribute, rather
		//		than by calling store.getLabel()
		labelAttr: "",

	 	// root: [readonly] dojo.data.Item
		//		Pointer to the root item (read only, not a parameter)
		root: null,

		// query: anything
		//		Specifies datastore query to return the root item for the tree.
		//		Must only return a single item.   Alternately can just pass in pointer
		//		to root item.
		// example:
		//	|	{id:'ROOT'}
		query: null,

		// deferItemLoadingUntilExpand: Boolean
		//		Setting this to true will cause the TreeStoreModel to defer calling loadItem on nodes
		// 		until they are expanded. This allows for lazying loading where only one
		//		loadItem (and generally one network call, consequently) per expansion
		// 		(rather than one for each child).
		// 		This relies on partial loading of the children items; each children item of a
		// 		fully loaded item should contain the label and info about having children.
		deferItemLoadingUntilExpand: false,

		constructor: function(/* Object */ args){
			// summary:
			//		Passed the arguments listed above (store, etc)
			// tags:
			//		private

			lang.mixin(this, args);

			this.connects = [];

			var store = this.store;
			if(!store.getFeatures()['dojo.data.api.Identity']){
				throw new Error("dijit.Tree: store must support dojo.data.Identity");
			}

			// if the store supports Notification, subscribe to the notification events
			if(store.getFeatures()['dojo.data.api.Notification']){
				this.connects = this.connects.concat([
					aspect.after(store, "onNew", lang.hitch(this, "onNewItem"), true),
					aspect.after(store, "onDelete", lang.hitch(this, "onDeleteItem"), true),
					aspect.after(store, "onSet", lang.hitch(this, "onSetItem"), true)
				]);
			}
		},

		destroy: function(){
			var h;
			while(h = this.connects.pop()){ h.remove(); }
			// TODO: should cancel any in-progress processing of getRoot(), getChildren()
		},

		// =======================================================================
		// Methods for traversing hierarchy

		getRoot: function(onItem, onError){
			// summary:
			//		Calls onItem with the root item for the tree, possibly a fabricated item.
			//		Calls onError on error.
			if(this.root){
				onItem(this.root);
			}else{
				this.store.fetch({
					query: this.query,
					onComplete: lang.hitch(this, function(items){
						if(items.length != 1){
							throw new Error(this.declaredClass + ": query " + json.stringify(this.query) + " returned " + items.length +
							 	" items, but must return exactly one item");
						}
						this.root = items[0];
						onItem(this.root);
					}),
					onError: onError
				});
			}
		},

		mayHaveChildren: function(/*dojo.data.Item*/ item){
			// summary:
			//		Tells if an item has or may have children.  Implementing logic here
			//		avoids showing +/- expando icon for nodes that we know don't have children.
			//		(For efficiency reasons we may not want to check if an element actually
			//		has children until user clicks the expando node)
			return array.some(this.childrenAttrs, function(attr){
				return this.store.hasAttribute(item, attr);
			}, this);
		},

		getChildren: function(/*dojo.data.Item*/ parentItem, /*function(items)*/ onComplete, /*function*/ onError){
			// summary:
			// 		Calls onComplete() with array of child items of given parent item, all loaded.

			var store = this.store;
			if(!store.isItemLoaded(parentItem)){
				// The parent is not loaded yet, we must be in deferItemLoadingUntilExpand
				// mode, so we will load it and just return the children (without loading each
				// child item)
				var getChildren = lang.hitch(this, arguments.callee);
				store.loadItem({
					item: parentItem,
					onItem: function(parentItem){
						getChildren(parentItem, onComplete, onError);
					},
					onError: onError
				});
				return;
			}
			// get children of specified item
			var childItems = [];
			for(var i=0; i<this.childrenAttrs.length; i++){
				var vals = store.getValues(parentItem, this.childrenAttrs[i]);
				childItems = childItems.concat(vals);
			}

			// count how many items need to be loaded
			var _waitCount = 0;
			if(!this.deferItemLoadingUntilExpand){
				array.forEach(childItems, function(item){ if(!store.isItemLoaded(item)){ _waitCount++; } });
			}

			if(_waitCount == 0){
				// all items are already loaded (or we aren't loading them).  proceed...
				onComplete(childItems);
			}else{
				// still waiting for some or all of the items to load
				array.forEach(childItems, function(item, idx){
					if(!store.isItemLoaded(item)){
						store.loadItem({
							item: item,
							onItem: function(item){
								childItems[idx] = item;
								if(--_waitCount == 0){
									// all nodes have been loaded, send them to the tree
									onComplete(childItems);
								}
							},
							onError: onError
						});
					}
				});
			}
		},

		// =======================================================================
		// Inspecting items

		isItem: function(/* anything */ something){
			return this.store.isItem(something);	// Boolean
		},

		fetchItemByIdentity: function(/* object */ keywordArgs){
			this.store.fetchItemByIdentity(keywordArgs);
		},

		getIdentity: function(/* item */ item){
			return this.store.getIdentity(item);	// Object
		},

		getLabel: function(/*dojo.data.Item*/ item){
			// summary:
			//		Get the label for an item
			if(this.labelAttr){
				return this.store.getValue(item,this.labelAttr);	// String
			}else{
				return this.store.getLabel(item);	// String
			}
		},

		// =======================================================================
		// Write interface

		newItem: function(/* dojo.dnd.Item */ args, /*Item*/ parent, /*int?*/ insertIndex){
			// summary:
			//		Creates a new item.   See `dojo.data.api.Write` for details on args.
			//		Used in drag & drop when item from external source dropped onto tree.
			// description:
			//		Developers will need to override this method if new items get added
			//		to parents with multiple children attributes, in order to define which
			//		children attribute points to the new item.

			var pInfo = {parent: parent, attribute: this.childrenAttrs[0]}, LnewItem;

			if(this.newItemIdAttr && args[this.newItemIdAttr]){
				// Maybe there's already a corresponding item in the store; if so, reuse it.
				this.fetchItemByIdentity({identity: args[this.newItemIdAttr], scope: this, onItem: function(item){
					if(item){
						// There's already a matching item in store, use it
						this.pasteItem(item, null, parent, true, insertIndex);
					}else{
						// Create new item in the tree, based on the drag source.
						LnewItem=this.store.newItem(args, pInfo);
						if(LnewItem && (insertIndex!=undefined)){
							// Move new item to desired position
							this.pasteItem(LnewItem, parent, parent, false, insertIndex);
						}
					}
				}});
			}else{
				// [as far as we know] there is no id so we must assume this is a new item
				LnewItem=this.store.newItem(args, pInfo);
				if(LnewItem && (insertIndex!=undefined)){
					// Move new item to desired position
					this.pasteItem(LnewItem, parent, parent, false, insertIndex);
				}
			}
		},

		pasteItem: function(/*Item*/ childItem, /*Item*/ oldParentItem, /*Item*/ newParentItem, /*Boolean*/ bCopy, /*int?*/ insertIndex){
			// summary:
			//		Move or copy an item from one parent item to another.
			//		Used in drag & drop
			var store = this.store,
				parentAttr = this.childrenAttrs[0];	// name of "children" attr in parent item

			// remove child from source item, and record the attribute that child occurred in
			if(oldParentItem){
				array.forEach(this.childrenAttrs, function(attr){
					if(store.containsValue(oldParentItem, attr, childItem)){
						if(!bCopy){
							var values = array.filter(store.getValues(oldParentItem, attr), function(x){
								return x != childItem;
							});
							store.setValues(oldParentItem, attr, values);
						}
						parentAttr = attr;
					}
				});
			}

			// modify target item's children attribute to include this item
			if(newParentItem){
				if(typeof insertIndex == "number"){
					// call slice() to avoid modifying the original array, confusing the data store
					var childItems = store.getValues(newParentItem, parentAttr).slice();
					childItems.splice(insertIndex, 0, childItem);
					store.setValues(newParentItem, parentAttr, childItems);
				}else{
					store.setValues(newParentItem, parentAttr,
						store.getValues(newParentItem, parentAttr).concat(childItem));
				}
			}
		},

		// =======================================================================
		// Callbacks

		onChange: function(/*dojo.data.Item*/ /*===== item =====*/){
			// summary:
			//		Callback whenever an item has changed, so that Tree
			//		can update the label, icon, etc.   Note that changes
			//		to an item's children or parent(s) will trigger an
			//		onChildrenChange() so you can ignore those changes here.
			// tags:
			//		callback
		},

		onChildrenChange: function(/*===== parent, newChildrenList =====*/){
			// summary:
			//		Callback to do notifications about new, updated, or deleted items.
			// parent: dojo.data.Item
			// newChildrenList: dojo.data.Item[]
			// tags:
			//		callback
		},

		onDelete: function(/*dojo.data.Item*/ /*===== item =====*/){
			// summary:
			//		Callback when an item has been deleted.
			// description:
			//		Note that there will also be an onChildrenChange() callback for the parent
			//		of this item.
			// tags:
			//		callback
		},

		// =======================================================================
		// Events from data store

		onNewItem: function(/* dojo.data.Item */ item, /* Object */ parentInfo){
			// summary:
			//		Handler for when new items appear in the store, either from a drop operation
			//		or some other way.   Updates the tree view (if necessary).
			// description:
			//		If the new item is a child of an existing item,
			//		calls onChildrenChange() with the new list of children
			//		for that existing item.
			//
			// tags:
			//		extension

			// We only care about the new item if it has a parent that corresponds to a TreeNode
			// we are currently displaying
			if(!parentInfo){
				return;
			}

			// Call onChildrenChange() on parent (ie, existing) item with new list of children
			// In the common case, the new list of children is simply parentInfo.newValue or
			// [ parentInfo.newValue ], although if items in the store has multiple
			// child attributes (see `childrenAttr`), then it's a superset of parentInfo.newValue,
			// so call getChildren() to be sure to get right answer.
			this.getChildren(parentInfo.item, lang.hitch(this, function(children){
				this.onChildrenChange(parentInfo.item, children);
			}));
		},

		onDeleteItem: function(/*Object*/ item){
			// summary:
			//		Handler for delete notifications from underlying store
			this.onDelete(item);
		},

		onSetItem: function(item, attribute /*===== , oldValue, newValue =====*/){
			// summary:
			//		Updates the tree view according to changes in the data store.
			// description:
			//		Handles updates to an item's children by calling onChildrenChange(), and
			//		other updates to an item by calling onChange().
			//
			//		See `onNewItem` for more details on handling updates to an item's children.
			// item: Item
			// attribute: attribute-name-string
			// oldValue: object | array
			// newValue: object | array
			// tags:
			//		extension

			if(array.indexOf(this.childrenAttrs, attribute) != -1){
				// item's children list changed
				this.getChildren(item, lang.hitch(this, function(children){
					// See comments in onNewItem() about calling getChildren()
					this.onChildrenChange(item, children);
				}));
			}else{
				// item's label/icon/etc. changed.
				this.onChange(item);
			}
		}
	});
});

},
'dijit/_TemplatedMixin':function(){
define([
	"dojo/_base/lang", // lang.getObject
	"dojo/touch",
	"./_WidgetBase",
	"dojo/string", // string.substitute string.trim
	"dojo/cache",	// dojo.cache
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom-construct", // domConstruct.destroy, domConstruct.toDom
	"dojo/_base/sniff", // has("ie")
	"dojo/_base/unload", // unload.addOnWindowUnload
	"dojo/_base/window" // win.doc
], function(lang, touch, _WidgetBase, string, cache, array, declare, domConstruct, has, unload, win) {

/*=====
	var _WidgetBase = dijit._WidgetBase;
=====*/

	// module:
	//		dijit/_TemplatedMixin
	// summary:
	//		Mixin for widgets that are instantiated from a template

	var _TemplatedMixin = declare("dijit._TemplatedMixin", null, {
		// summary:
		//		Mixin for widgets that are instantiated from a template

		// templateString: [protected] String
		//		A string that represents the widget template.
		//		Use in conjunction with dojo.cache() to load from a file.
		templateString: null,

		// templatePath: [protected deprecated] String
		//		Path to template (HTML file) for this widget relative to dojo.baseUrl.
		//		Deprecated: use templateString with require([... "dojo/text!..."], ...) instead
		templatePath: null,

		// skipNodeCache: [protected] Boolean
		//		If using a cached widget template nodes poses issues for a
		//		particular widget class, it can set this property to ensure
		//		that its template is always re-built from a string
		_skipNodeCache: false,

		// _earlyTemplatedStartup: Boolean
		//		A fallback to preserve the 1.0 - 1.3 behavior of children in
		//		templates having their startup called before the parent widget
		//		fires postCreate. Defaults to 'false', causing child widgets to
		//		have their .startup() called immediately before a parent widget
		//		.startup(), but always after the parent .postCreate(). Set to
		//		'true' to re-enable to previous, arguably broken, behavior.
		_earlyTemplatedStartup: false,

/*=====
		// _attachPoints: [private] String[]
		//		List of widget attribute names associated with data-dojo-attach-point=... in the
		//		template, ex: ["containerNode", "labelNode"]
 		_attachPoints: [],
 =====*/

/*=====
		// _attachEvents: [private] Handle[]
		//		List of connections associated with data-dojo-attach-event=... in the
		//		template
 		_attachEvents: [],
 =====*/

		constructor: function(){
			this._attachPoints = [];
			this._attachEvents = [];
		},

		_stringRepl: function(tmpl){
			// summary:
			//		Does substitution of ${foo} type properties in template string
			// tags:
			//		private
			var className = this.declaredClass, _this = this;
			// Cache contains a string because we need to do property replacement
			// do the property replacement
			return string.substitute(tmpl, this, function(value, key){
				if(key.charAt(0) == '!'){ value = lang.getObject(key.substr(1), false, _this); }
				if(typeof value == "undefined"){ throw new Error(className+" template:"+key); } // a debugging aide
				if(value == null){ return ""; }

				// Substitution keys beginning with ! will skip the transform step,
				// in case a user wishes to insert unescaped markup, e.g. ${!foo}
				return key.charAt(0) == "!" ? value :
					// Safer substitution, see heading "Attribute values" in
					// http://www.w3.org/TR/REC-html40/appendix/notes.html#h-B.3.2
					value.toString().replace(/"/g,"&quot;"); //TODO: add &amp? use encodeXML method?
			}, this);
		},

		buildRendering: function(){
			// summary:
			//		Construct the UI for this widget from a template, setting this.domNode.
			// tags:
			//		protected

			if(!this.templateString){
				this.templateString = cache(this.templatePath, {sanitize: true});
			}

			// Lookup cached version of template, and download to cache if it
			// isn't there already.  Returns either a DomNode or a string, depending on
			// whether or not the template contains ${foo} replacement parameters.
			var cached = _TemplatedMixin.getCachedTemplate(this.templateString, this._skipNodeCache);

			var node;
			if(lang.isString(cached)){
				node = domConstruct.toDom(this._stringRepl(cached));
				if(node.nodeType != 1){
					// Flag common problems such as templates with multiple top level nodes (nodeType == 11)
					throw new Error("Invalid template: " + cached);
				}
			}else{
				// if it's a node, all we have to do is clone it
				node = cached.cloneNode(true);
			}

			this.domNode = node;

			// Call down to _Widget.buildRendering() to get base classes assigned
			// TODO: change the baseClass assignment to _setBaseClassAttr
			this.inherited(arguments);

			// recurse through the node, looking for, and attaching to, our
			// attachment points and events, which should be defined on the template node.
			this._attachTemplateNodes(node, function(n,p){ return n.getAttribute(p); });

			this._beforeFillContent();		// hook for _WidgetsInTemplateMixin

			this._fillContent(this.srcNodeRef);
		},

		_beforeFillContent: function(){
		},

		_fillContent: function(/*DomNode*/ source){
			// summary:
			//		Relocate source contents to templated container node.
			//		this.containerNode must be able to receive children, or exceptions will be thrown.
			// tags:
			//		protected
			var dest = this.containerNode;
			if(source && dest){
				while(source.hasChildNodes()){
					dest.appendChild(source.firstChild);
				}
			}
		},

		_attachTemplateNodes: function(rootNode, getAttrFunc){
			// summary:
			//		Iterate through the template and attach functions and nodes accordingly.
			//		Alternately, if rootNode is an array of widgets, then will process data-dojo-attach-point
			//		etc. for those widgets.
			// description:
			//		Map widget properties and functions to the handlers specified in
			//		the dom node and it's descendants. This function iterates over all
			//		nodes and looks for these properties:
			//			* dojoAttachPoint/data-dojo-attach-point
			//			* dojoAttachEvent/data-dojo-attach-event
			// rootNode: DomNode|Widget[]
			//		the node to search for properties. All children will be searched.
			// getAttrFunc: Function
			//		a function which will be used to obtain property for a given
			//		DomNode/Widget
			// tags:
			//		private

			var nodes = lang.isArray(rootNode) ? rootNode : (rootNode.all || rootNode.getElementsByTagName("*"));
			var x = lang.isArray(rootNode) ? 0 : -1;
			for(; x<nodes.length; x++){
				var baseNode = (x == -1) ? rootNode : nodes[x];
				if(this.widgetsInTemplate && (getAttrFunc(baseNode, "dojoType") || getAttrFunc(baseNode, "data-dojo-type"))){
					continue;
				}
				// Process data-dojo-attach-point
				var attachPoint = getAttrFunc(baseNode, "dojoAttachPoint") || getAttrFunc(baseNode, "data-dojo-attach-point");
				if(attachPoint){
					var point, points = attachPoint.split(/\s*,\s*/);
					while((point = points.shift())){
						if(lang.isArray(this[point])){
							this[point].push(baseNode);
						}else{
							this[point]=baseNode;
						}
						this._attachPoints.push(point);
					}
				}

				// Process data-dojo-attach-event
				var attachEvent = getAttrFunc(baseNode, "dojoAttachEvent") || getAttrFunc(baseNode, "data-dojo-attach-event");
				if(attachEvent){
					// NOTE: we want to support attributes that have the form
					// "domEvent: nativeEvent; ..."
					var event, events = attachEvent.split(/\s*,\s*/);
					var trim = lang.trim;
					while((event = events.shift())){
						if(event){
							var thisFunc = null;
							if(event.indexOf(":") != -1){
								// oh, if only JS had tuple assignment
								var funcNameArr = event.split(":");
								event = trim(funcNameArr[0]);
								thisFunc = trim(funcNameArr[1]);
							}else{
								event = trim(event);
							}
							if(!thisFunc){
								thisFunc = event;
							}
							// Map "press", "move" and "release" to keys.touch, keys.move, keys.release
							this._attachEvents.push(this.connect(baseNode, touch[event] || event, thisFunc));
						}
					}
				}
			}
		},

		destroyRendering: function(){
			// Delete all attach points to prevent IE6 memory leaks.
			array.forEach(this._attachPoints, function(point){
				delete this[point];
			}, this);
			this._attachPoints = [];

			// And same for event handlers
			array.forEach(this._attachEvents, this.disconnect, this);
			this._attachEvents = [];

			this.inherited(arguments);
		}
	});

	// key is templateString; object is either string or DOM tree
	_TemplatedMixin._templateCache = {};

	_TemplatedMixin.getCachedTemplate = function(templateString, alwaysUseString){
		// summary:
		//		Static method to get a template based on the templatePath or
		//		templateString key
		// templateString: String
		//		The template
		// alwaysUseString: Boolean
		//		Don't cache the DOM tree for this template, even if it doesn't have any variables
		// returns: Mixed
		//		Either string (if there are ${} variables that need to be replaced) or just
		//		a DOM tree (if the node can be cloned directly)

		// is it already cached?
		var tmplts = _TemplatedMixin._templateCache;
		var key = templateString;
		var cached = tmplts[key];
		if(cached){
			try{
				// if the cached value is an innerHTML string (no ownerDocument) or a DOM tree created within the current document, then use the current cached value
				if(!cached.ownerDocument || cached.ownerDocument == win.doc){
					// string or node of the same document
					return cached;
				}
			}catch(e){ /* squelch */ } // IE can throw an exception if cached.ownerDocument was reloaded
			domConstruct.destroy(cached);
		}

		templateString = string.trim(templateString);

		if(alwaysUseString || templateString.match(/\$\{([^\}]+)\}/g)){
			// there are variables in the template so all we can do is cache the string
			return (tmplts[key] = templateString); //String
		}else{
			// there are no variables in the template so we can cache the DOM tree
			var node = domConstruct.toDom(templateString);
			if(node.nodeType != 1){
				throw new Error("Invalid template: " + templateString);
			}
			return (tmplts[key] = node); //Node
		}
	};

	if(has("ie")){
		unload.addOnWindowUnload(function(){
			var cache = _TemplatedMixin._templateCache;
			for(var key in cache){
				var value = cache[key];
				if(typeof value == "object"){ // value is either a string or a DOM node template
					domConstruct.destroy(value);
				}
				delete cache[key];
			}
		});
	}

	// These arguments can be specified for widgets which are used in templates.
	// Since any widget can be specified as sub widgets in template, mix it
	// into the base widget class.  (This is a hack, but it's effective.)
	lang.extend(_WidgetBase,{
		dojoAttachEvent: "",
		dojoAttachPoint: ""
	});

	return _TemplatedMixin;
});

},
'dojox/grid/_Layout':function(){
define([
	"dojo/_base/kernel",
	"../main",
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/dom-geometry",
	"./cells",
	"./_RowSelector"
], function(dojo, dojox, declare, array, lang, domGeometry){

return declare("dojox.grid._Layout", null, {
	// summary:
	//	Controls grid cell layout. Owned by grid and used internally.
	constructor: function(inGrid){
		this.grid = inGrid;
	},
	// flat array of grid cells
	cells: [],
	// structured array of grid cells
	structure: null,
	// default cell width
	defaultWidth: '6em',

	// methods
	moveColumn: function(sourceViewIndex, destViewIndex, cellIndex, targetIndex, before){
		var source_cells = this.structure[sourceViewIndex].cells[0];
		var dest_cells = this.structure[destViewIndex].cells[0];

		var cell = null;
		var cell_ri = 0;
		var target_ri = 0;

		for(var i=0, c; c=source_cells[i]; i++){
			if(c.index == cellIndex){
				cell_ri = i;
				break;
			}
		}
		cell = source_cells.splice(cell_ri, 1)[0];
		cell.view = this.grid.views.views[destViewIndex];

		for(i=0, c=null; c=dest_cells[i]; i++){
			if(c.index == targetIndex){
				target_ri = i;
				break;
			}
		}
		if(!before){
			target_ri += 1;
		}
		dest_cells.splice(target_ri, 0, cell);

		var sortedCell = this.grid.getCell(this.grid.getSortIndex());
		if(sortedCell){
			sortedCell._currentlySorted = this.grid.getSortAsc();
		}

		this.cells = [];
		cellIndex = 0;
		var v;
		for(i=0; v=this.structure[i]; i++){
			for(var j=0, cs; cs=v.cells[j]; j++){
				for(var k=0; c=cs[k]; k++){
					c.index = cellIndex;
					this.cells.push(c);
					if("_currentlySorted" in c){
						var si = cellIndex + 1;
						si *= c._currentlySorted ? 1 : -1;
						this.grid.sortInfo = si;
						delete c._currentlySorted;
					}
					cellIndex++;
				}
			}
		}
		
		//Fix #9481 - reset idx in cell markup
		array.forEach(this.cells, function(c){
			var marks = c.markup[2].split(" ");
			var oldIdx = parseInt(marks[1].substring(5));//get old "idx"
			if(oldIdx != c.index){
				marks[1] = "idx=\"" + c.index + "\"";
				c.markup[2] = marks.join(" ");
			}
		});
		
		this.grid.setupHeaderMenu();
		//this.grid.renderOnIdle();
	},

	setColumnVisibility: function(columnIndex, visible){
		var cell = this.cells[columnIndex];
		if(cell.hidden == visible){
			cell.hidden = !visible;
			var v = cell.view, w = v.viewWidth;
			if(w && w != "auto"){
				v._togglingColumn = domGeometry.getMarginBox(cell.getHeaderNode()).w || 0;
			}
			v.update();
			return true;
		}else{
			return false;
		}
	},
	
	addCellDef: function(inRowIndex, inCellIndex, inDef){
		var self = this;
		var getCellWidth = function(inDef){
			var w = 0;
			if(inDef.colSpan > 1){
				w = 0;
			}else{
				w = inDef.width || self._defaultCellProps.width || self.defaultWidth;

				if(!isNaN(w)){
					w = w + "em";
				}
			}
			return w;
		};

		var props = {
			grid: this.grid,
			subrow: inRowIndex,
			layoutIndex: inCellIndex,
			index: this.cells.length
		};

		if(inDef && inDef instanceof dojox.grid.cells._Base){
			var new_cell = lang.clone(inDef);
			props.unitWidth = getCellWidth(new_cell._props);
			new_cell = lang.mixin(new_cell, this._defaultCellProps, inDef._props, props);
			return new_cell;
		}

		var cell_type = inDef.type || inDef.cellType || this._defaultCellProps.type || this._defaultCellProps.cellType || dojox.grid.cells.Cell;
		if(lang.isString(cell_type)){
			cell_type = lang.getObject(cell_type);
		}

		props.unitWidth = getCellWidth(inDef);
		return new cell_type(lang.mixin({}, this._defaultCellProps, inDef, props));
	},
	
	addRowDef: function(inRowIndex, inDef){
		var result = [];
		var relSum = 0, pctSum = 0, doRel = true;
		for(var i=0, def, cell; (def=inDef[i]); i++){
			cell = this.addCellDef(inRowIndex, i, def);
			result.push(cell);
			this.cells.push(cell);
			// Check and calculate the sum of all relative widths
			if(doRel && cell.relWidth){
				relSum += cell.relWidth;
			}else if(cell.width){
				var w = cell.width;
				if(typeof w == "string" && w.slice(-1) == "%"){
					pctSum += window.parseInt(w, 10);
				}else if(w == "auto"){
					// relative widths doesn't play nice with auto - since we
					// don't have a way of knowing how much space the auto is
					// supposed to take up.
					doRel = false;
				}
			}
		}
		if(relSum && doRel){
			// We have some kind of relWidths specified - so change them to %
			array.forEach(result, function(cell){
				if(cell.relWidth){
					cell.width = cell.unitWidth = ((cell.relWidth / relSum) * (100 - pctSum)) + "%";
				}
			});
		}
		return result;
	
	},

	addRowsDef: function(inDef){
		var result = [];
		if(lang.isArray(inDef)){
			if(lang.isArray(inDef[0])){
				for(var i=0, row; inDef && (row=inDef[i]); i++){
					result.push(this.addRowDef(i, row));
				}
			}else{
				result.push(this.addRowDef(0, inDef));
			}
		}
		return result;
	},
	
	addViewDef: function(inDef){
		this._defaultCellProps = inDef.defaultCell || {};
		if(inDef.width && inDef.width == "auto"){
			delete inDef.width;
		}
		return lang.mixin({}, inDef, {cells: this.addRowsDef(inDef.rows || inDef.cells)});
	},
	
	setStructure: function(inStructure){
		this.fieldIndex = 0;
		this.cells = [];
		var s = this.structure = [];

		if(this.grid.rowSelector){
			var sel = { type: dojox._scopeName + ".grid._RowSelector" };

			if(lang.isString(this.grid.rowSelector)){
				var width = this.grid.rowSelector;

				if(width == "false"){
					sel = null;
				}else if(width != "true"){
					sel['width'] = width;
				}
			}else{
				if(!this.grid.rowSelector){
					sel = null;
				}
			}

			if(sel){
				s.push(this.addViewDef(sel));
			}
		}

		var isCell = function(def){
			return ("name" in def || "field" in def || "get" in def);
		};

		var isRowDef = function(def){
			if(lang.isArray(def)){
				if(lang.isArray(def[0]) || isCell(def[0])){
					return true;
				}
			}
			return false;
		};

		var isView = function(def){
			return (def !== null && lang.isObject(def) &&
					("cells" in def || "rows" in def || ("type" in def && !isCell(def))));
		};

		if(lang.isArray(inStructure)){
			var hasViews = false;
			for(var i=0, st; (st=inStructure[i]); i++){
				if(isView(st)){
					hasViews = true;
					break;
				}
			}
			if(!hasViews){
				s.push(this.addViewDef({ cells: inStructure }));
			}else{
				for(i=0; (st=inStructure[i]); i++){
					if(isRowDef(st)){
						s.push(this.addViewDef({ cells: st }));
					}else if(isView(st)){
						s.push(this.addViewDef(st));
					}
				}
			}
		}else if(isView(inStructure)){
			// it's a view object
			s.push(this.addViewDef(inStructure));
		}

		this.cellCount = this.cells.length;
		this.grid.setupHeaderMenu();
	}
});
});
},
'ibm/usmi/widgets/ResourceNavigator':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/widgets/ResourceNavigator", [
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dojo/topic",
	"dijit/_WidgetBase",
	"dijit/layout/ContentPane",
	"dijit/layout/BorderContainer",
	"dijit/registry",
	// TWL imports
	"ibm/tivoli/tip/TCTTable",
	"ibm/tivoli/navmodel/Dataset",
	"ibm/usmi/util/DefaultMessageHandler",
	"ibm/tivoli/tip/navwidget/dijit/NavWidget",
	// NextGen imports
	"ibm/usmi/widgets/table/DefaultItemsTableView",
	"ibm/usmi/util/BaseWidget",
	"ibm/usmi/widgets/util/ResourceNavigatorDAHelper",
	"ibm/usmi/widgets/util/DefaultContextMenuProvider",
	"ibm/usmi/widgets/util/ResourceMenuProviderHelper",
	"ibm/usmi/widgets/util/Constants",
	"ibm/usmi/core/utils/Logger",
	"dojo/i18n!./nls/CommonWidgetsResources"
], function (dojo, declare, topic, _WidgetBase, ContentPane, BorderContainer, registry, TCTTable, Dataset,
		DefaultMessageHandler, NavWidget, DefaultItemsTableView, BaseWidget, ResourceNavigatorDAHelper, 
		DefaultContextMenuProvider, ResourceMenuProviderHelper, Constants, Logger, mri) {

	return declare([
		_WidgetBase,
		BaseWidget,
		BorderContainer
	], {

		// attr that can be set html tag
		navigatorWidgetDefinitionId : "",

		// boolean: navigator stand alone, used to deterimine if should load dataset or wait and it will be set
		// by another means, ie. TreeSelector
		standAlone : true,

		// cmProvider: Custom context menu provider (must be provided on construction to be used)
		cmProvider : null,

		// Defaults
		DEFAULT_COLLECTION : "RESOURCE_NAVIGATOR",

        //attribute to disable retarget listening
        noRetarget : null,

		///////////////////////////////////////////////////////////////////
		// private members
		///////////////////////////////////////////////////////////////////

		//resource bundle
		_resources : mri,

		// the logger
		_resourceNavigatorLogger : new Logger("ibm.usmi.widgets", "ResourceNavigator"),

		// load up the task utils for the different types of task launch
		_taskUtil : null,

		// handle to TWL navigator
		_navWidget : null,

		// TODO -  This is tied to launching the navigator from action.
		_dataSetExternally : false,

		// dataset object
		_dataset : null,

		defaultTaskUtil : null,
		defaultActionHelper : null,
		_subscriptions : null,

		_createCMProvider : function () {
			// summary:
			//        Coming soon
			// description:
			//        Coming soon

			//Create the menu provider that we'll pass off to the pluggable views
			if (!this.cmProvider) {
				var helper = new ResourceMenuProviderHelper();
				this.cmProvider = new DefaultContextMenuProvider({
					view : this,
					getMenuItemsCallback : dojo.hitch(helper, helper.getMenuItems),
					getMenuCommandCallback : dojo.hitch(helper, helper.getMenuCommand)
				});
			}

		},

		constructor : function (/*constructor_kwArgs*/kwArgs) {
			// summary:
			//        Constructs PropertyNotebook object.

			if (this._resourceNavigatorLogger.isFinest()) {
				this._resourceNavigatorLogger.entering("", "constructor");
			}

			this.title = this._resources.ResourceNavigatorTitle;
			this._subscriptions = [];
			if (!this.collectionID) {
				var targetInfo = this.getInitArg(Constants.TARGET_INFO);
				if (targetInfo) {
					this.collectionID = targetInfo[Constants.COLLECTION];
				}
			}

			if (this._resourceNavigatorLogger.isFinest()) {
				this._resourceNavigatorLogger.exiting("", "constructor");
			}
		},

		//look at these two functions and do things in a standard way
		startup : function () {
			this.ready();
		},

		ready : function () {
			console.log("Resource Navigator Ready!");
			this.makeNavWidget();
			if (this._dataSetExternally) {
				this._updateTargets(this._dataset, null);
				this._dataSetExternally = false;
			}
			
            //provide noRetarget option so users can turn off listening for retarget events
			if (!this.noRetarget){
                var sub = topic.subscribe(Constants.TOPIC_RETARGET_INFO, dojo.hitch(this, this.retargeted));
                this._subscriptions.push(sub);
            }
		},

		postCreate : function () {
			var datasetArg = this.getInitArg("dataset");
			if (datasetArg) {
				this.collectionID = datasetArg;
			}
			if (!this.collectionID) {
				this.standAlone = false;
			}

			this._subscriptions.push(dojo.subscribe(Constants.TOPIC_HASH_CHANGE, this, this.hashChange));
		},

		//Create the items table view
		makeNavWidget : function () {
			if (this._resourceNavigatorLogger.isFinest()) {
				this._resourceNavigatorLogger.entering("", "makeNavWidget");
			}

			// create a default context menu provider
			this.defaultActionHelper = new ResourceNavigatorDAHelper();
			// widget def on URl Overrides all
			defOnUrl = this.getUrlParmsObj().navigatorWidgetDefinitionId;

			if (defOnUrl) {
				this.navigatorWidgetDefinitionId = defOnUrl;
			} // check to see if it was specified as attribute
			else if (!this.navigatorWidgetDefinitionId) {
				this.navigatorWidgetDefinitionId = "UsmiNavigatorView-0.1.0";
			}

			// make sure cmProvider has been constructed
			this._createCMProvider();

			// id is passed in via data-dojo-props
			this._navWidget = new DefaultItemsTableView({
				id : "_navWidget" + this.id,
				'class' : 'interiorSpacious',
				height : "98%",
				//homeDataset: dataset,
				daHelper : this.defaultActionHelper,
				//            widgetDefinitionId: this.navigatorWidgetDefinitionId,
				cmProvider : this.cmProvider
			});

			this.addChild(this._navWidget);

			//see if we were passed a dataset at construction
			if (this.getInitArg(Constants.TARGET_INFO) && this.getInitArg(Constants.TARGET_INFO).dataset) {
				this._dataset = this.getInitArg(Constants.TARGET_INFO).dataset;
				this._navWidget.setDataset(this._dataset, null, null);
				this.setTitle(this._dataset);
			} else {
				// We need to go ahead and return now to keep the main UI thread active
				// We'll schedule this for later
				//this.loadDataset();
				setTimeout(dojo.hitch(this, this.loadDataset), 0);
			}

			this.inherited(arguments);

			if (this._resourceNavigatorLogger.isFinest()) {
				this._resourceNavigatorLogger.exiting("", "makeNavWidget");
			}
		},

		loadDataset : function () {
			// summary:
			//            Create the dataset for the table, only called if collectionID is provided on URL or 
			//			  as attr, otherwise assumed will be set later by selector.

			if (this._resourceNavigatorLogger.isFinest()) {
				this._resourceNavigatorLogger.entering("", "loadDataset");
			}
			this._resourceNavigatorLogger.finer("this.collectionID: " + this.collectionID);
			var _processDataSet;
			// Check to make sure we were given provider, collection information
			if (!this.providerID) {
				// load default
				this.providerID = Constants.DEFAULT_PROVIDER;
			}

			if (!this.collectionID) {
				// load default
				this.collectionID = this.DEFAULT_COLLECTION;
			}

			//var myNavWidget = registry.byId("myNavWidget");
			_processDataSet = function (dataset) {
				this._navWidget.setDataset(dataset, null, null);
				this.setTitle(dataset);

				//Listen for when the rendered dataset changes
				dojo
						.connect(this._navWidget, Constants.SET_DATASET, dojo.hitch(this,
								this._navWidgetSetDatasetHandler));

				//Listen for selection changes on nav widget
				dojo.connect(this._navWidget.selection, Constants.SELECTION_CHANGE, dojo.hitch(this,
						this._navWidgetSelectionChangedHandler));
			};

			// can't blindly passdown sourceToken, if the sourceToken is null the create doesn't
			// include it on the rest call, but subsequent getTasks call do, which causes multiple
			// navcollections on the server side to be created.
			if(this.sourceToken != null) {
				this._dataset = new Dataset({
					onComplete : dojo.hitch(this, _processDataSet),
					onError : function (error, exception, ioArgs) {
						var msgHandler = new DefaultMessageHandler({});
						msgHandler.displayException(exception);
					},
					baseUrl : Constants.BASE_REST_URL,
					provider : this.providerID,
					datasource : "",
					dataset : this.collectionID,
					parameters : {
						sourceToken : this.sourceToken
					},
					defaultProperties : []
				});
			} else {
				this._dataset = new Dataset({
					onComplete : dojo.hitch(this, _processDataSet),
					onError : function (error, exception, ioArgs) {
						var msgHandler = new DefaultMessageHandler({});
						msgHandler.displayException(exception);
					},
					baseUrl : Constants.BASE_REST_URL,
					provider : this.providerID,
					datasource : "",
					dataset : this.collectionID,
					parameters : {},
					defaultProperties : []
				});				
			}

			if (this._resourceNavigatorLogger.isFinest()) {
				this._resourceNavigatorLogger.exiting("", "loadDataset");
			}
		},

		//Destroy the topology canvas (largely so server will get notified to clean itself up)
		destroy : function () {
			// summary:
			//        Clean up any objects the Resource Navigator is hanging onto
			if (this._resourceNavigatorLogger.isFinest()) {
				this._resourceNavigatorLogger.entering("", "destroy");
			}

			if (this._dataset) {
				this._dataset.destroy();
			}
			if (this.cmProvider) {
				this.cmProvider.destroy();
			}
			if (this._navWidget) {
				this._navWidget.destroyRecursive();
			}
			if (this.defaultTaskUtil) {
				this.defaultTaskUtil.destroy();
			}
			if (this._subscriptions) {
				var i = 0;
				for (; i < this._subscriptions.length; i++) {
					this._subscriptions[i].remove();
				}
				this._subscriptions = null;
			}

			this.inherited(arguments);

			if (this._resourceNavigatorLogger.isFinest()) {
				this._resourceNavigatorLogger.exiting("", "destroy");
			}
		},

		hashChange : function (newHash) {
			if (this._resourceNavigatorLogger.isFiner()) {
				this._resourceNavigatorLogger.finer("hashchange called!" + dojo.hash(), "hashChange");

			}
			var urlParms = newHash.split('&');
			var idx = 0;
			var retargetedParms = {};
			for (; idx < urlParms.length; idx++) {
				var paramValues = urlParms[idx].split("=");
				if (paramValues[0] === 'dataset') {
					retargetedParms.collectionID = paramValues[1];
				} else if (paramValues[0] === 'workAreaDijit') {
					if (paramValues[1] === "ibm/usmi/widgets/ResourceNavigator") {
						retargetedParms.taskSelection = paramValues[1];
					}
				}

			}
			if (retargetedParms.collectionID && retargetedParms.taskSelection) {
				this.retargeted(retargetedParms);
			}

		},

		setDataset : function (/*ibm.tivoli.navmodel.Dataset*/dataset) {
			this._dataset = dataset;
			this._dataSetExternally = true;

		},

		//handler for dataset is set
		_navWidgetSetDatasetHandler : function (/*ibm.tivoli.navmodel.Dataset*/dataset) {
			// summary:
			//        Coming soon
			// description:
			//        Coming soon
			// dataset:
			//
			if (dataset) {
				if (this._resourceNavigatorLogger.isFiner()) {
					this._resourceNavigatorLogger.finer("DatasetHandler called: dataset = " + dataset.id,
							"_navWidgetSetDatasetHandler");
				}
			} else {
				if (this._resourceNavigatorLogger.isFiner()) {
					this._resourceNavigatorLogger.finer("DatasetHandler called dataset is null",
							"_navWidgetSetDatasetHandler"); // Randy added message for this condition
				}
			}
		},

		//handler for selection changes
		_navWidgetSelectionChangedHandler : function (objectContext) {
			// summary:
			//        Coming soon
			// description:
			//        Coming soon
			// objectContext:
			//

			var items = objectContext.getItems();
			if (items) {
				if (items.length > 0) {
					if (this._resourceNavigatorLogger.isFiner()) {
						this._resourceNavigatorLogger.finer("NavWidget first selected item's id is = "
								+ items[0].getId(), "_navWidgetSelectionChangedHandler");
					}
				}
			}
			//            navWidget.hideBusyPane();

		},

		_updateTargets : function (/*ibm.tivoli.navmodel.Dataset*/dataset, /*ibm.tivoli.navmodel.Item[]*/
		items) {
			// summary:
			//       update based on retarget event being fired
			// description:
			//        Coming soon
			// dataset:

			//TODO: consider if we need to accomodate the _navWidget not really being there yet
			//save off the dataset and check for it when we're ready if we do
			this._navWidget.setDataset(dataset, null, null);
		},
		retargeted : function (retargetInfo) {
			if (retargetInfo.taskSelection === "ibm/usmi/widgets/ResourceNavigator") {
				if (retargetInfo.dataset) {
					if (this._dataset) {
						this._dataset.destroy(); //destroy old dataset
					}
					
					// set to new one
					this._dataset = retargetInfo.dataset;
					this._navWidget.setDataset(this._dataset, null, null);
					this.setTitle(this._dataset);
				}
				//no dataset included. Handle collectionId or sourcetoken setup
				else if (retargetInfo.sourceToken || retargetInfo.collectionID) {
					this.sourceToken = retargetInfo.sourceToken;
					this.collectionID = retargetInfo.collectionID;
					if (this._dataset) {
						this._dataset.destroy(); //destroy old dataset
					}
					this.loadDataset();
				}
			}
		},
		setTitle : function (dataset) {
			if (!this.notUpdateTitle) {
				dojo.publish(Constants.TOPIC_TASK_TITLE_CHANGE, {
					resource : dataset._dataset.label
				});
			}
		}
	});
});
},
'dojoe/slidingpane/SlidingPane':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/i18n!dojoe/slidingpane/nls/SlidingPane","dojo/require!dojo/fx,dijit/_TemplatedMixin,dijit/layout/ContentPane,dojo/i18n"], function(dijit,dojo,dojox){
dojo.provide( "dojoe.slidingpane.SlidingPane" );

dojo.require( "dojo.fx" );
dojo.require( "dijit._TemplatedMixin" );
dojo.require( "dijit.layout.ContentPane" );
dojo.require("dojo.i18n");
dojo.requireLocalization("dojoe.slidingpane", "SlidingPane");

dojo.declare("dojoe.slidingpane.SlidingPane", [dijit._Widget, dijit._TemplatedMixin],
	{
		// summary:  The SlidingPane provides collapsible interface to the user. It is possible to add any widget or content within the sliding pane. 
		// description: SlidingPane widget uses the dojo animation features for its implementation. Use InnerDiv attachpoint to add graphics to the slidingpane. can be opened and closed.
	
		// widgetsInTemplate: [public]Boolean
		//		Set this to true if the widget contains other widgets
		widgetsInTemplate: true,
		
 		// templatePath:  [private]String
		//		Path to the template
		templateString : dojo.cache("dojoe.slidingpane", "templates/SlidingPane.html", "<div class='slidingPane open' tabIndex=0 aria-label=\"slidingpane\" >\r\n\t<div dojoAttachPoint='innerDIV' aria-label=\"content inside slidingpane\">\r\n\t\t<div dojoAttachPoint='hideNode'>\r\n\t\t\t<div class='dijitReset' dojoAttachPoint='wipeNode'>\r\n\t\t\t\t<div dojoAttachPoint='containerNode'></div>\r\n\t\t\t</div>\r\n\t\t</div>\r\n\t</div>\r\n\t<div class='slidingPaneClose' dojoAttachPoint='closeNode' dojoAttachEvent=\"onclick:_toggle, onkeypress:_keyEvent\" tabIndex=0>\r\n\t</div>\r\n</div>\r\n\t"),
		
		// backgroundClass:  [private]String
		//		Class set to the inner div.
		backgroundClass: null,
		
		closeNodeClass: null,
		
		// open: Boolean
		//		Whether description is opened or closed on first render.
		open: true,
		
		// duration: Integer
		//		Time in milliseconds to wipe in/wipe out.
		duration: 500,
		
		constructor: function () {
			this._resources = dojo.i18n.getLocalization("dojoe.slidingpane", "SlidingPane",this.lang);
		},
  
		postCreate: function () {
			// summary:  Creates  and initialises the SlidingPane and  sets open/close animations.
			// tags: private
			
			if (!this.open ) {
				this.hideNode.style.display = this.wipeNode.style.display = "none";
			}
			this._updateClass(); 
			this.inherited("postCreate",arguments);
			this.wipeNode.containerNode = this.containerNode;
			if (this.backgroundClass !== null ) {
				this.innerDIV.className = this.backgroundClass;
			}
			if (this.closeNodeClass !== null ) {
				this.closeNode.className = this.closeNodeClass;
			}
			// setup open/close animations
			var hideNode = this.hideNode, 
			wipeNode = this.wipeNode;
			this._wipeIn = dojo.fx.wipeIn({
				node: this.wipeNode,
				duration: this.duration,
				beforeBegin: function () {
					hideNode.style.display="";
				}
			});
    		this._wipeOut = dojo.fx.wipeOut({
				node: this.wipeNode,
				duration: this.duration,
				onEnd: function () {
					hideNode.style.display="none";
				}
			});
    
		},
		
		_keyEvent: function (evt) {
			if(dojo.isIE > 8 && evt.keyCode == 13) {
				evt.preventDefault();
			}
			if (evt!=null) {
				var charCode = evt.keyCode;
				if (charCode == 13) {
					this._toggle();
				}
			}
		},
  
		_toggle: function () {
			// summary:  Changes the animation effect for open/close state of slidingpane choosen by the user.
			// tags: private
			
			dojo.forEach([this._wipeIn, this._wipeOut], function (animation){
				if (animation.status() == "playing") {
					animation.stop();
				}
			});
			this[this.open ? "_wipeOut" : "_wipeIn"].play();
			this.open =! this.open;    
			this._updateClass(); 
			this.onClick();	
		},
  
		_updateClass: function () {
			// summary:  Updates the necessary class when  open/close state of the slidingpane choosen by the user.
			// tags: private
			
			if (this.open) {
				dojo.removeClass( this.domNode, "close"  );
				dojo.addClass( this.domNode, "open" );   
				this.closeNode.setAttribute("title", this._resources.CLOSE);
			} else {
				dojo.removeClass( this.domNode, "open" );
				dojo.addClass( this.domNode, "close"  );
				this.closeNode.setAttribute("title", this._resources.EXPAND);
			}  
		},

		onClick: function () {
			// summary:  Event gets fired when slidingpane is clicked
			// tags: public
			
		},
		
		//Latha:06May2011: updating destroy for complete cleanup
		destroy : function () {
			// summary:  Destroys the sliding pane.
			// tags: public
			try {
				delete this._resources;
				var widgetArr = this.getChildren();
				dojo.forEach(widgetArr, function (child) {
					if(child){
						if(child.destroyRecursive){
							child.destroyRecursive();
						} else if(child.destroy){
							child.destroy();
						}
					}
				});
				this.inherited(arguments);
			} catch (ex) {
				console.warn("Widget destroy failed: "+this.id+", ex="+ex.name+":"+ex.message);
			}
		}
	});

});

},
'ibm/usmi/statuspod/StatusPodMenuDialog':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/statuspod/StatusPodMenuDialog", [
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"idx/oneui/MenuDialog",
	"ibm/usmi/core/utils/Logger"
], function (dojo, declare, MenuDialog, Logger) {
	return declare([ MenuDialog
	], {
		_logger : new Logger("ibm.usmi.statuspod", "StatusPodMenuDialog"),

		onClose : function () {
			this.inherited(arguments);
			this._logger.finest("menu dialog is on closing!", "onClose");
			if (this.content.closeDescendants) {
				this.content.closeDescendants();
			}
		}

	});
});

},
'dijit/form/RadioButton':function(){
define([
	"dojo/_base/declare", // declare
	"./CheckBox",
	"./_RadioButtonMixin"
], function(declare, CheckBox, _RadioButtonMixin){

/*=====
	var CheckBox = dijit.form.CheckBox;
	var _RadioButtonMixin = dijit.form._RadioButtonMixin;
=====*/

	// module:
	//		dijit/form/RadioButton
	// summary:
	//		Radio button widget

	return declare("dijit.form.RadioButton", [CheckBox, _RadioButtonMixin], {
		// summary:
		// 		Same as an HTML radio, but with fancy styling.

		baseClass: "dijitRadio"
	});
});

},
'dojoe/tablex/plugins/Pagination':function(){
define("dojoe/tablex/plugins/Pagination", ["dojo", "dijit", "gridx/modules/pagination/Pagination"], function(dojo, dijit) {

	// summary:
	//		plugin to enable Pagination
	// description:
	//		Please refer to dojox.grid.enhanced.plugins.Pagination. By default this is enabled on Table
	
	//return gridx.modules.pagination.Pagination;
});
},
'dojoe/tablex/plugins/_MapToModules':function(){
define(["dojo/_base/declare",
		"dojo/_base/array",
        "dojo/_base/lang",
    	"dojo/dom-class",
    	"dojo/dom-geometry",
    	"dojo/_base/query",
    	"dojo/window",
    	"dojo/dom-style",
		"gridx/modules/Body",
		"gridx/modules/Header",
		"gridx/modules/VirtualVScroller",
		"gridx/modules/Focus",
		"gridx/modules/ColumnResizer",
		"gridx/modules/SingleSort",
		"gridx/modules/select/Row",
		"gridx/modules/select/Cell",
		"gridx/modules/select/Column",
		"gridx/modules/extendedSelect/Cell",
		"gridx/modules/extendedSelect/Row",
		"gridx/modules/extendedSelect/Column",
		"gridx/modules/RowHeader",
		"gridx/modules/IndirectSelect",
		"gridx/modules/SummaryBar",
		"gridx/modules/pagination/Pagination",
		"gridx/modules/pagination/PaginationBar",
		"gridx/modules/pagination/PaginationBarDD",
		"gridx/modules/NestedSort",
		"gridx/modules/dnd/_Dnd",
		//"gridx/modules/move/Cell",
		//"gridx/modules/dnd/Cell",
		"gridx/modules/move/Column",
		"gridx/modules/move/Row",
		"gridx/modules/dnd/Row",
		"gridx/modules/dnd/Column",
		"gridx/modules/filter/Filter",
		"gridx/modules/filter/FilterBar",
		"gridx/modules/Menu",
		"gridx/modules/exporter/Exporter",
		"gridx/modules/exporter/Table",
		"gridx/modules/Printer",
		"gridx/modules/exporter/CSV",
		"gridx/modules/HLayout",
		"gridx/modules/VLayout",
		"gridx/modules/ColumnWidth"
	//	"dojoe/tablex/plugins/IndirectSelection"	
	], function (declare, array, lang, domClass, domGeometry, query, window, 
	domStyle, body, header, virtualVScroller, focus, columnResizer, singleSort, selectRow, selectCell, selectColumn, extSelCell, extSelRow, extSelColumn,
	rowHeader, indirectSelect, SummaryBar, pagination, paginationBar, paginationBarDD, nestedSort,_Dnd, /*MoveCell, DndCell,*/  
	 moveColumn, moveRow, dndRow, dndColumn, filter, filterBar, menu, exporter, exportTable, printer, exportCSV, HLayout, VLayout, ColumnWidth /* ,IndirectSelection*/) {
	
		var forEach = array.forEach;
		
		var mapToModules = declare("dojoe.tablex.plugins._MapToModules",[], {
			_modules: [],
			_coreModules: [
							header,
							body,
							virtualVScroller,
							focus,
							columnResizer,
							singleSort,
							//Latha: 05 Oct 2012 : Desired Selection Behavior broken in TWL 1.0.2 : removing ExtSelectCell as this module disturbs row selection.
					//		extSelCell,
							extSelRow,
					//		extSelColumn,
					//		rowHeader,
							//LS - 10/6/12 : modules to enable setting column width in the layout
							// Please do not set last column width in the layout so that it defaults to "auto" occupying the remaining space in the grid
							HLayout,
							VLayout,
							ColumnWidth
							
						 ],
			_plugins: null,

			_convertToModules: function (/*gridparams.plugins object*/plugins) {
				this._modules = [];
				this._modules = this._modules.concat(this._coreModules);
				for(var key in plugins) {
					if (plugins.hasOwnProperty(key)) {						
						switch(key){
							case "indirectSelection" :
								if( plugins[key]!= false){
									this._mapIndSelectAttrs(plugins[key]);
								}
								break;
							case "pagination" :
								if( plugins[key]!= false){
									this._mapPaginationAttrs(plugins[key]);
								}
								break;
							case "dnd" :
								if( plugins[key]!= false){
									this._mapDnDAttrs(plugins[key]);
								}
								break;
							case "sortFields" :
								//@189518 handled by nestedSorting plugin, TWL could probably combine
								break;
							case "nestedSorting" :
								if( plugins[key]!= false){
									this._mapNestedSortingAttrs(plugins[key], plugins["sortFields"]);	//@189518
								} else {
									this._mapSingleSortAttrs(plugins["sortFields"]);					//@189518					
								}
								break;
							case "filter" :
								if( plugins[key]!= false){
									this._mapFilterAttrs(plugins[key]);
								}
								break;
							case "menus" :
								if( plugins[key]!= false){
									this._mapMenuAttrs(plugins[key]);
								}
								break;
							case "printer" :
								if( plugins[key]!= false){
									this._mapPrinterAttrs(plugins[key]);
								}
								break;
							case "exporter" :
								if( plugins[key]!= false){
									this._mapExporterAttrs(plugins[key]);
								}
								break;
							case "selector" :
								if( plugins[key]!= false){
									this._mapSelectorAttrs(plugins[key]);
								}
								break;
							default: console.error('Plugin not supported: '+ key );
						}						
					}
				}
				return this._modules;				
			},
			
			_getCoreModules: function (){				
				return this._coreModules;
			},
			
			_mapIndSelectAttrs: function (pluginObj){
				var mdIndex = array.indexOf(this._coreModules, extSelRow);
				var rowSelectMod = 
					(mdIndex >=0)?{moduleClass:this._coreModules[mdIndex], triggerOnCell: true}
						:{moduleClass: extSelRow, triggerOnCell: true};
				var rowHeaderMod = {moduleClass: rowHeader};
				var indSelMod = {moduleClass: indirectSelect};
				//var indSelPlugin = {moduleClass: IndirectSelection};
				if(lang.isObject(pluginObj)){
					if(pluginObj.headerSelector == true){
						indSelMod.all = true;
					}
					if(pluginObj.multiple == false){
						rowSelectMod = {moduleClass: selectRow, triggerOnCell: true};
						rowSelectMod.multiple = false;
					}
				}
				if(mdIndex >= 0){
					this._coreModules[mdIndex] = rowSelectMod;
					this._modules.push(rowHeaderMod, indSelMod);	
				}else{
					this._modules.push(rowSelectMod,rowHeaderMod, indSelMod);	
				}
			},
			
			_mapSelectorAttrs: function(pluginObj) {
				var mdIndexSelRow = array.indexOf(this._coreModules, extSelRow);
				var selectRowMod = 
					(mdIndexSelRow >=0)? {moduleClass:this._coreModules[mdIndexSelRow], triggerOnCell: true}
						:{moduleClass: extSelRow, triggerOnCell: true};
				var selectCellMod = {moduleClass: extSelCell};
				var selectColumnMod = {moduleClass: extSelColumn};
				
				if(lang.isObject(pluginObj)){
					if(pluginObj.row == "single"){
						selectRowMod = {moduleClass: selectRow, triggerOnCell: true};
						selectRowMod.multiple = false;
					}
					if(pluginObj.cell == "single"){
						selectCellMod = {moduleClass: selectCell};
					}
					if( pluginObj.col == "single"){
						selectColumnMod = {moduleClass: selectColumn};
					}					
				}
				if(pluginObj.col != "disabled"){
					this._modules.push(selectColumnMod);
					
				}
				if(pluginObj.cell != "disabled"){
					this._modules.push(selectCellMod);
				}
				if(pluginObj.row != "disabled"){
					if(mdIndexSelRow >= 0){
						this._coreModules[mdIndexSelRow] = selectRowMod;
					}else{
						this._modules.push(selectRowMod);
					}
				}				
			},
			
			_mapDnDAttrs: function (){
				this._modules.push(
						{moduleClass: _Dnd}, 
						{moduleClass: extSelColumn},
					//	{moduleClass: MoveCell}, 
						{moduleClass: moveColumn},
						{moduleClass: moveRow}, 
					//	{moduleClass: DndCell},
						{moduleClass: dndRow},
						{moduleClass: dndColumn}
				);
			},
			
			_mapPaginationAttrs: function (pluginObj){
				var baseMod = {moduleClass: pagination};
				var paginationMod = {moduleClass: paginationBar};
				if( lang.isObject(pluginObj) && !pluginObj.onlyDesc ){
					for(var key in pluginObj) {
						if (pluginObj.hasOwnProperty(key)) {
							switch(key){
								case "defaultPageSize" :
									baseMod.initialPageSize = pluginObj[key];
									break;
								case "pageSizes" :
									paginationMod.sizes = pluginObj[key];
									break;
								case "description" :
									paginationMod.description = pluginObj[key];
									break;
								case "sizeSwitch" :
									paginationMod.sizeSwitch = pluginObj[key];
									break;
								case "pageStepper" :
									paginationMod.stepper = pluginObj[key];
									break;
								case "gotoButton" :
									paginationMod.gotoButton = pluginObj[key];
									break;
								case "maxPageStep" :
									paginationMod.visibleSteppers = pluginObj[key];
									break;
								case "position" :
									paginationMod.position = pluginObj[key];
									break;
								case "descTemplate" :
									paginationMod.descriptionTemplate = pluginObj[key];
									break;
								default: console.warn('Parameter not supported: '+ key 
										+ ' please use gridx modules instead of plugin to take advantage of extra configurations! ');
							}
						}
					}
					this._modules.push(baseMod,paginationMod);
				}
				
				if(lang.isObject(pluginObj) && pluginObj.onlyDesc == true){
					//description only so use SummaryBar 
					var summaryBarMod = {moduleClass: SummaryBar};
					this._modules.push(summaryBarMod);
				}				
			},
			
			//@189518 START
			_mapSingleSortAttrs: function(pluginObj) {
				var sort = {moduleClass: singleSort};
				sort.initialOrder = pluginObj;
				this._modules.push(sort);					
			},
			
			_mapNestedSortingAttrs: function (pluginObj, initialOrder){
				var nest = {moduleClass: nestedSort};
				nest.initialOrder = initialOrder;
				this._modules.push(
						//{moduleClass: body},
						//{moduleClass: header},
						nest
				);	
			},
			//@189518 END	
			
			_mapMenuAttrs: function(pluginObj) {
				this._modules.push({moduleClass: menu});
			},
			
			_mapFilterAttrs: function(pluginObj) {
				var baseMod = {moduleClass: filter};
				var filterBarMod = {moduleClass: filterBar};
				if(lang.isObject(pluginObj)){
					for(var key in pluginObj) {
						if (pluginObj.hasOwnProperty(key)) {
							switch(key){
								case "isServerSide" :
									baseMod.serverMode = pluginObj[key];
									break;	
								case "closeFilterbarButton" :
									filterBarMod.closeFilterBarButton = pluginObj[key];
									break;	
								case "ruleCountToConfirmClearFilter" :
									filterBarMod.ruleCountToConfirmClearFilter = pluginObj[key];
									break;		
								case "setupFilterQuery" :
									baseMod.setupFilterQuery = pluginObj[key];
									break;
								default: console.warn('Parameter not supported: '+ key 
										+ ' please use gridx modules instead of plugin to take advantage of extra configurations! ');
							}
						}
					}
				}
				this._modules.push(baseMod, filterBarMod);
			},
			
			_mapPrinterAttrs: function(pluginObj) {				
				this._modules.push({moduleClass:exporter}, {moduleClass:exportTable}, {moduleClass: printer});
			},
			
			_mapExporterAttrs: function(pluginObj){
				this._modules.push({moduleClass:exporter}, {moduleClass:exportCSV}, {moduleClass:exportTable});
			}			
		});
		
		mapToModules.convertToModules = function(plugins){
			return mapToModules.prototype._convertToModules(plugins);
		};
		mapToModules._getCoreModules = function(){
			return mapToModules.prototype._getCoreModules();
		};
	
		return mapToModules;
	}
);
},
'dijit/_CssStateMixin':function(){
define([
	"dojo/touch",
	"dojo/_base/array", // array.forEach array.map
	"dojo/_base/declare",	// declare
	"dojo/dom-class", // domClass.toggle
	"dojo/_base/lang", // lang.hitch
	"dojo/_base/window" // win.body
], function(touch, array, declare, domClass, lang, win){

// module:
//		dijit/_CssStateMixin
// summary:
//		Mixin for widgets to set CSS classes on the widget DOM nodes depending on hover/mouse press/focus
//		state changes, and also higher-level state changes such becoming disabled or selected.

return declare("dijit._CssStateMixin", [], {
	// summary:
	//		Mixin for widgets to set CSS classes on the widget DOM nodes depending on hover/mouse press/focus
	//		state changes, and also higher-level state changes such becoming disabled or selected.
	//
	// description:
	//		By mixing this class into your widget, and setting the this.baseClass attribute, it will automatically
	//		maintain CSS classes on the widget root node (this.domNode) depending on hover,
	//		active, focus, etc. state.   Ex: with a baseClass of dijitButton, it will apply the classes
	//		dijitButtonHovered and dijitButtonActive, as the user moves the mouse over the widget and clicks it.
	//
	//		It also sets CSS like dijitButtonDisabled based on widget semantic state.
	//
	//		By setting the cssStateNodes attribute, a widget can also track events on subnodes (like buttons
	//		within the widget).

	// cssStateNodes: [protected] Object
	//		List of sub-nodes within the widget that need CSS classes applied on mouse hover/press and focus
	//.
	//		Each entry in the hash is a an attachpoint names (like "upArrowButton") mapped to a CSS class names
	//		(like "dijitUpArrowButton"). Example:
	//	|		{
	//	|			"upArrowButton": "dijitUpArrowButton",
	//	|			"downArrowButton": "dijitDownArrowButton"
	//	|		}
	//		The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it
	//		is hovered, etc.
	cssStateNodes: {},

	// hovering: [readonly] Boolean
	//		True if cursor is over this widget
	hovering: false,

	// active: [readonly] Boolean
	//		True if mouse was pressed while over this widget, and hasn't been released yet
	active: false,

	_applyAttributes: function(){
		// This code would typically be in postCreate(), but putting in _applyAttributes() for
		// performance: so the class changes happen before DOM is inserted into the document.
		// Change back to postCreate() in 2.0.  See #11635.

		this.inherited(arguments);

		// Automatically monitor mouse events (essentially :hover and :active) on this.domNode
		array.forEach(["onmouseenter", "onmouseleave", touch.press], function(e){
			this.connect(this.domNode, e, "_cssMouseEvent");
		}, this);

		// Monitoring changes to disabled, readonly, etc. state, and update CSS class of root node
		array.forEach(["disabled", "readOnly", "checked", "selected", "focused", "state", "hovering", "active"], function(attr){
			this.watch(attr, lang.hitch(this, "_setStateClass"));
		}, this);

		// Events on sub nodes within the widget
		for(var ap in this.cssStateNodes){
			this._trackMouseState(this[ap], this.cssStateNodes[ap]);
		}
		// Set state initially; there's probably no hover/active/focus state but widget might be
		// disabled/readonly/checked/selected so we want to set CSS classes for those conditions.
		this._setStateClass();
	},

	_cssMouseEvent: function(/*Event*/ event){
		// summary:
		//	Sets hovering and active properties depending on mouse state,
		//	which triggers _setStateClass() to set appropriate CSS classes for this.domNode.

		if(!this.disabled){
			switch(event.type){
				case "mouseenter":
				case "mouseover":	// generated on non-IE browsers even though we connected to mouseenter
					this._set("hovering", true);
					this._set("active", this._mouseDown);
					break;

				case "mouseleave":
				case "mouseout":	// generated on non-IE browsers even though we connected to mouseleave
					this._set("hovering", false);
					this._set("active", false);
					break;

				case "mousedown":
				case "touchpress":
					this._set("active", true);
					this._mouseDown = true;
					// Set a global event to handle mouseup, so it fires properly
					// even if the cursor leaves this.domNode before the mouse up event.
					// Alternately could set active=false on mouseout.
					var mouseUpConnector = this.connect(win.body(), touch.release, function(){
						this._mouseDown = false;
						this._set("active", false);
						this.disconnect(mouseUpConnector);
					});
					break;
			}
		}
	},

	_setStateClass: function(){
		// summary:
		//		Update the visual state of the widget by setting the css classes on this.domNode
		//		(or this.stateNode if defined) by combining this.baseClass with
		//		various suffixes that represent the current widget state(s).
		//
		// description:
		//		In the case where a widget has multiple
		//		states, it sets the class based on all possible
		//	 	combinations.  For example, an invalid form widget that is being hovered
		//		will be "dijitInput dijitInputInvalid dijitInputHover dijitInputInvalidHover".
		//
		//		The widget may have one or more of the following states, determined
		//		by this.state, this.checked, this.valid, and this.selected:
		//			- Error - ValidationTextBox sets this.state to "Error" if the current input value is invalid
		//			- Incomplete - ValidationTextBox sets this.state to "Incomplete" if the current input value is not finished yet
		//			- Checked - ex: a checkmark or a ToggleButton in a checked state, will have this.checked==true
		//			- Selected - ex: currently selected tab will have this.selected==true
		//
		//		In addition, it may have one or more of the following states,
		//		based on this.disabled and flags set in _onMouse (this.active, this.hovering) and from focus manager (this.focused):
		//			- Disabled	- if the widget is disabled
		//			- Active		- if the mouse (or space/enter key?) is being pressed down
		//			- Focused		- if the widget has focus
		//			- Hover		- if the mouse is over the widget

		// Compute new set of classes
		var newStateClasses = this.baseClass.split(" ");

		function multiply(modifier){
			newStateClasses = newStateClasses.concat(array.map(newStateClasses, function(c){ return c+modifier; }), "dijit"+modifier);
		}

		if(!this.isLeftToRight()){
			// For RTL mode we need to set an addition class like dijitTextBoxRtl.
			multiply("Rtl");
		}

		var checkedState = this.checked == "mixed" ? "Mixed" : (this.checked ? "Checked" : "");
		if(this.checked){
			multiply(checkedState);
		}
		if(this.state){
			multiply(this.state);
		}
		if(this.selected){
			multiply("Selected");
		}

		if(this.disabled){
			multiply("Disabled");
		}else if(this.readOnly){
			multiply("ReadOnly");
		}else{
			if(this.active){
				multiply("Active");
			}else if(this.hovering){
				multiply("Hover");
			}
		}

		if(this.focused){
			multiply("Focused");
		}

		// Remove old state classes and add new ones.
		// For performance concerns we only write into domNode.className once.
		var tn = this.stateNode || this.domNode,
			classHash = {};	// set of all classes (state and otherwise) for node

		array.forEach(tn.className.split(" "), function(c){ classHash[c] = true; });

		if("_stateClasses" in this){
			array.forEach(this._stateClasses, function(c){ delete classHash[c]; });
		}

		array.forEach(newStateClasses, function(c){ classHash[c] = true; });

		var newClasses = [];
		for(var c in classHash){
			newClasses.push(c);
		}
		tn.className = newClasses.join(" ");

		this._stateClasses = newStateClasses;
	},

	_trackMouseState: function(/*DomNode*/ node, /*String*/ clazz){
		// summary:
		//		Track mouse/focus events on specified node and set CSS class on that node to indicate
		//		current state.   Usually not called directly, but via cssStateNodes attribute.
		// description:
		//		Given class=foo, will set the following CSS class on the node
		//			- fooActive: if the user is currently pressing down the mouse button while over the node
		//			- fooHover: if the user is hovering the mouse over the node, but not pressing down a button
		//			- fooFocus: if the node is focused
		//
		//		Note that it won't set any classes if the widget is disabled.
		// node: DomNode
		//		Should be a sub-node of the widget, not the top node (this.domNode), since the top node
		//		is handled specially and automatically just by mixing in this class.
		// clazz: String
		//		CSS class name (ex: dijitSliderUpArrow).

		// Current state of node (initially false)
		// NB: setting specifically to false because domClass.toggle() needs true boolean as third arg
		var hovering=false, active=false, focused=false;

		var self = this,
			cn = lang.hitch(this, "connect", node);

		function setClass(){
			var disabled = ("disabled" in self && self.disabled) || ("readonly" in self && self.readonly);
			domClass.toggle(node, clazz+"Hover", hovering && !active && !disabled);
			domClass.toggle(node, clazz+"Active", active && !disabled);
			domClass.toggle(node, clazz+"Focused", focused && !disabled);
		}

		// Mouse
		cn("onmouseenter", function(){
			hovering = true;
			setClass();
		});
		cn("onmouseleave", function(){
			hovering = false;
			active = false;
			setClass();
		});
		cn(touch.press, function(){
			active = true;
			setClass();
		});
		cn(touch.release, function(){
			active = false;
			setClass();
		});

		// Focus
		cn("onfocus", function(){
			focused = true;
			setClass();
		});
		cn("onblur", function(){
			focused = false;
			setClass();
		});

		// Just in case widget is enabled/disabled while it has focus/hover/active state.
		// Maybe this is overkill.
		this.watch("disabled", setClass);
		this.watch("readOnly", setClass);
	}
});
});

},
'url:dijit/layout/templates/AccordionButton.html':"<div data-dojo-attach-event='onclick:_onTitleClick' class='dijitAccordionTitle' role=\"presentation\">\n\t<div data-dojo-attach-point='titleNode,focusNode' data-dojo-attach-event='onkeypress:_onTitleKeyPress'\n\t\t\tclass='dijitAccordionTitleFocus' role=\"tab\" aria-expanded=\"false\"\n\t\t><span class='dijitInline dijitAccordionArrow' role=\"presentation\"></span\n\t\t><span class='arrowTextUp' role=\"presentation\">+</span\n\t\t><span class='arrowTextDown' role=\"presentation\">-</span\n\t\t><img src=\"${_blankGif}\" alt=\"\" class=\"dijitIcon\" data-dojo-attach-point='iconNode' style=\"vertical-align: middle\" role=\"presentation\"/>\n\t\t<span role=\"presentation\" data-dojo-attach-point='titleTextNode' class='dijitAccordionText'></span>\n\t</div>\n</div>\n",
'gridx/modules/Header':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/array",
	"dojo/dom-construct",
	"dojo/dom-class",
	"dojo/dom-geometry",
	"dojo/_base/query",
	"dojo/_base/sniff",
	"dojo/keys",
	"../core/util",
	"../core/_Module"
], function(declare, lang, array, domConstruct, domClass, domGeometry, query, sniff, keys, util, _Module){

	
	return declare(/*===== "gridx.modules.Header", =====*/_Module, {
		// summary:
		//		The header UI of grid
		// description:
		//		This module is in charge of the rendering of the grid header. But it should not manage column width,
		//		which is the responsibility of ColumnWidth module.

		name: 'header',

		getAPIPath: function(){
			// tags:
			//		protected extension
			return {
				

				
				header: this
			};
		},

		constructor: function(){
			var t = this,
				dn = t.domNode = domConstruct.create('div', {
					'class': 'gridxHeaderRow',
					role: 'presentation'
				}),
				inner = t.innerNode = domConstruct.create('div', {
					'class': 'gridxHeaderRowInner',
					role: 'row'
				});
			t.grid._connectEvents(dn, '_onMouseEvent', t);
		},

		preload: function(args){
			// tags:
			//		protected extension
			var t = this,
				g = t.grid;
			t.domNode.appendChild(t.innerNode);
			t._build();
			g.headerNode.appendChild(t.domNode);
			//Add this.domNode to be a part of the grid header
			g.vLayout.register(t, 'domNode', 'headerNode');
			t.aspect(g, 'onHScroll', '_onHScroll');
			t.aspect(g, 'onHeaderCellMouseOver', '_onHeaderCellMouseOver');
			t.aspect(g, 'onHeaderCellMouseOut', '_onHeaderCellMouseOver');
			if(g.columnResizer){
				t.aspect(g.columnResizer, 'onResize', function(){
					if(g.hScrollerNode.style.display == 'none'){
						t._onHScroll(0);
					}
				});
			}
			t._initFocus();
		},

		destroy: function(){
			// tags:
			//		protected extension
			this.inherited(arguments);
			domConstruct.destroy(this.domNode);
		},

		columnMixin: {
			

			
			headerNode: function(){
				return this.grid.header.getHeaderNode(this.id);
			}
		},
	
		//Public-----------------------------------------------------------------------------
		

		// hidden: Boolean
		//		Whether the header UI should be hidden.
		hidden: false,

		
		getHeaderNode: function(id){
			// summary:
			//		Get the header DOM node by column ID.
			// id: String
			//		The column ID
			// returns:
			//		The header DOM node
			return query("[colid='" + id + "']", this.domNode)[0];	//DOMNode
		},
		
		
		refresh: function(){
			// summary:
			//		Re-build the header UI.
			this._build();
			this._onHScroll(this._scrollLeft);
			this.onRender();
		},

		onRender: function(){
			// summary:
			//		Fired when the header is rendered.
			// tags:
			//		callback
		},

		onMoveToHeaderCell: function(/* columnId, e */){
			// summary:
			//		Fired when the focus is moved to a header cell by keyboard.
			// tags:
			//		callback
		},
		
		//Private-----------------------------------------------------------------------------
		_scrollLeft: 0,

		_build: function(){
			var t = this,
				g = t.grid,
				f = g.focus,
				sb = ['<table role="presentation" border="0" cellpadding="0" cellspacing="0"><tr>'];
			array.forEach(g._columns, function(col){
				sb.push('<th id="', g.id, '-', col.id,
					'" role="columnheader" aria-readonly="true" tabindex="-1" colid="', col.id,
					'" class="gridxCell ',
					f && f.currentArea() == 'header' && col.id == t._focusHeaderId ? t._focusClass : '',
					(lang.isFunction(col.headerClass) ? col.headerClass(col) : col.headerClass) || '',
					'" style="width: ', col.width, ';',
					(lang.isFunction(col.headerStyle) ? col.headerStyle(col) : col.headerStyle) || '',
					'"><div class="gridxSortNode">',
					col.name || '',
					'</div></th>');
			});
			sb.push('</tr></table>');
			t.innerNode.innerHTML = sb.join('');
			domClass.toggle(t.domNode, 'gridxHeaderRowHidden', t.arg('hidden'));
		},

		_onHScroll: function(left){
			var ltr = this.grid.isLeftToRight();
			this.innerNode.firstChild.style[ltr ? 'marginLeft' : 'marginRight'] = (!ltr && sniff('ff') ? left : -left) + 'px';
			this._scrollLeft = left;
		},
	
		_onMouseEvent: function(eventName, e){
			var g = this.grid,
				evtCell = 'onHeaderCell' + eventName,
				evtRow = 'onHeader' + eventName;
			if(g._isConnected(evtCell) || g._isConnected(evtRow)){
				this._decorateEvent(e);
				if(e.columnIndex >= 0){
					g[evtCell](e);
				}
				g[evtRow](e);
			}
		},
	
		_decorateEvent: function(e){
			for(var n = e.target, c; n && n !== this.domNode; n = n.parentNode){
				if(n.tagName.toLowerCase() == 'th'){
					c = this.grid._columnsById[n.getAttribute('colid')];
					if(c){
						e.headerCellNode = n;
						e.columnId = c.id;
						e.columnIndex = c.index;
					}
					return;
				}
			}
		},
		
		_onHeaderCellMouseOver: function(e){
			domClass.toggle(this.getHeaderNode(e.columnId), 'gridxHeaderCellOver', e.type == 'mouseover');
		},
		
		// Focus
		_focusHeaderId: null,

		_focusClass: "gridxHeaderCellFocus",

		_initFocus: function(){
			var t = this, g = t.grid;
			if(g.focus){
				g.focus.registerArea({
					name: 'header',
					priority: 0,
					focusNode: t.domNode,
					scope: t,
					doFocus: t._doFocus,
					doBlur: t._blurNode,
					onBlur: t._blurNode,
					connects: [
						t.connect(g, 'onHeaderCellKeyDown', '_onKeyDown'),
						t.connect(g, 'onHeaderCellMouseDown', function(evt){
							t._focusNode(t.getHeaderNode(evt.columnId));
						})
					]
				});
			}
		},

		_doFocus: function(evt, step){
			var t = this, 
				n = t._focusHeaderId && t.getHeaderNode(t._focusHeaderId),
				r = t._focusNode(n || query('th.gridxCell', t.domNode)[0]);
			util.stopEvent(r && evt);
			return r;
		},
		
		_focusNode: function(node){
			if(node){
				var t = this, g = t.grid,
					fid = t._focusHeaderId = node.getAttribute('colid');
				if(fid){
					t._blurNode();
					if(g.hScroller){
						g.hScroller.scrollToColumn(fid);
					}
					g.body._focusCellCol = g._columnsById[fid].index;

					domClass.add(node, t._focusClass);
					//If no timeout, the header and body may be mismatch.
					setTimeout(function(){
						//For webkit browsers, when moving column using keyboard, the header cell will lose this focus class,
						//although it was set correctly before this setTimeout. So re-add it here.
						if(sniff('webkit')){
							domClass.add(node, t._focusClass);
						}
						node.focus();
					}, 0);
					return true;
				}
			}
			return false;
		},

		_blurNode: function(){
			var t = this, n = query('th.' + t._focusClass, t.domNode)[0];
			if(n){
				domClass.remove(n, t._focusClass);
			}
			return true;
		},

		_onKeyDown: function(evt){
			var t = this, g = t.grid, col,
				dir = g.isLeftToRight() ? 1 : -1,
				delta = evt.keyCode == keys.LEFT_ARROW ? -dir : dir;
			if(t._focusHeaderId && !evt.ctrlKey && !evt.altKey &&
				(evt.keyCode == keys.LEFT_ARROW || evt.keyCode == keys.RIGHT_ARROW)){
				//Prevent scrolling the whole page.
				util.stopEvent(evt);
				col = g._columnsById[t._focusHeaderId];
				col = g._columns[col.index + delta];
				if(col){
					t._focusNode(t.getHeaderNode(col.id));
					t.onMoveToHeaderCell(col.id, evt);
				}
			}
		}
	});
});

},
'ibm/tivoli/tip/navwidget/dijit/NavWidgetTabContainerObjectPluggableView':function(){
// wrapped by build app
define("ibm/tivoli/tip/navwidget/dijit/NavWidgetTabContainerObjectPluggableView", ["dijit","dojo","dojox","dojo/require!dijit/layout/BorderContainer,dijit/_Widget,ibm/tivoli/navmodel/Trace,ibm/tivoli/tip/navwidget/dijit/NavWidgetObjectPluggableView,dijit/layout/TabContainer,dijit/layout/ContentPane"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide ("ibm.tivoli.tip.navwidget.dijit.NavWidgetTabContainerObjectPluggableView");

dojo.require ("dijit.layout.BorderContainer");
dojo.require("dijit._Widget");
dojo.require("ibm.tivoli.navmodel.Trace");
dojo.require("ibm.tivoli.tip.navwidget.dijit.NavWidgetObjectPluggableView");
dojo.require("dijit.layout.TabContainer");
dojo.require("dijit.layout.ContentPane");

ibm.tivoli.tip.navwidget.dijit.NavWidgetTabContainerObjectPluggableView._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.tip.navwidget.dijit.NavWidgetTabContainerObjectPluggableView");

dojo.declare ("ibm.tivoli.tip.navwidget.dijit.NavWidgetTabContainerObjectPluggableView",
	[dijit.layout.TabContainer, ibm.tivoli.tip.navwidget.dijit.NavWidgetObjectPluggableView],
	{
		// summary:
		//		Base class for a view that wants to be in a tab container
		// description:
		//		Coming soon
	
		// _trace: [private] Trace
		//		Utility for trace and logging
	    _trace: ibm.tivoli.tip.navwidget.dijit.NavWidgetTabContainerObjectPluggableView._trace,

		postCreate: function(){		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
	
			//Call superclass
			this.inherited(arguments);
		},
		
		startup: function(){		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			this.inherited(arguments);
		},
		
		
		destroy: function(){		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
            this.inherited(arguments);
		}
});

});

},
'dijit/form/ValidationTextBox':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/i18n", // i18n.getLocalization
	"./TextBox",
	"../Tooltip",
	"dojo/text!./templates/ValidationTextBox.html",
	"dojo/i18n!./nls/validate"
], function(declare, i18n, TextBox, Tooltip, template){

/*=====
	var Tooltip = dijit.Tooltip;
	var TextBox = dijit.form.TextBox;
=====*/

	// module:
	//		dijit/form/ValidationTextBox
	// summary:
	//		Base class for textbox widgets with the ability to validate content of various types and provide user feedback.


	/*=====
		dijit.form.ValidationTextBox.__Constraints = function(){
			// locale: String
			//		locale used for validation, picks up value from this widget's lang attribute
			// _flags_: anything
			//		various flags passed to regExpGen function
			this.locale = "";
			this._flags_ = "";
		}
	=====*/

	return declare("dijit.form.ValidationTextBox", TextBox, {
		// summary:
		//		Base class for textbox widgets with the ability to validate content of various types and provide user feedback.
		// tags:
		//		protected

		templateString: template,
		baseClass: "dijitTextBox dijitValidationTextBox",

		// required: Boolean
		//		User is required to enter data into this field.
		required: false,

		// promptMessage: String
		//		If defined, display this hint string immediately on focus to the textbox, if empty.
		//		Also displays if the textbox value is Incomplete (not yet valid but will be with additional input).
		//		Think of this like a tooltip that tells the user what to do, not an error message
		//		that tells the user what they've done wrong.
		//
		//		Message disappears when user starts typing.
		promptMessage: "",

		// invalidMessage: String
		// 		The message to display if value is invalid.
		//		The translated string value is read from the message file by default.
		// 		Set to "" to use the promptMessage instead.
		invalidMessage: "$_unset_$",

		// missingMessage: String
		// 		The message to display if value is empty and the field is required.
		//		The translated string value is read from the message file by default.
		// 		Set to "" to use the invalidMessage instead.
		missingMessage: "$_unset_$",

		// message: String
		//		Currently error/prompt message.
		//		When using the default tooltip implementation, this will only be
		//		displayed when the field is focused.
		message: "",

		// constraints: dijit.form.ValidationTextBox.__Constraints
		//		user-defined object needed to pass parameters to the validator functions
		constraints: {},

		// regExp: [extension protected] String
		//		regular expression string used to validate the input
		//		Do not specify both regExp and regExpGen
		regExp: ".*",

		regExpGen: function(/*dijit.form.ValidationTextBox.__Constraints*/ /*===== constraints =====*/){
			// summary:
			//		Overridable function used to generate regExp when dependent on constraints.
			//		Do not specify both regExp and regExpGen.
			// tags:
			//		extension protected
			return this.regExp; // String
		},

		// state: [readonly] String
		//		Shows current state (ie, validation result) of input (""=Normal, Incomplete, or Error)
		state: "",

		// tooltipPosition: String[]
		//		See description of `dijit.Tooltip.defaultPosition` for details on this parameter.
		tooltipPosition: [],

		_setValueAttr: function(){
			// summary:
			//		Hook so set('value', ...) works.
			this.inherited(arguments);
			this.validate(this.focused);
		},

		validator: function(/*anything*/ value, /*dijit.form.ValidationTextBox.__Constraints*/ constraints){
			// summary:
			//		Overridable function used to validate the text input against the regular expression.
			// tags:
			//		protected
			return (new RegExp("^(?:" + this.regExpGen(constraints) + ")"+(this.required?"":"?")+"$")).test(value) &&
				(!this.required || !this._isEmpty(value)) &&
				(this._isEmpty(value) || this.parse(value, constraints) !== undefined); // Boolean
		},

		_isValidSubset: function(){
			// summary:
			//		Returns true if the value is either already valid or could be made valid by appending characters.
			//		This is used for validation while the user [may be] still typing.
			return this.textbox.value.search(this._partialre) == 0;
		},

		isValid: function(/*Boolean*/ /*===== isFocused =====*/){
			// summary:
			//		Tests if value is valid.
			//		Can override with your own routine in a subclass.
			// tags:
			//		protected
			return this.validator(this.textbox.value, this.constraints);
		},

		_isEmpty: function(value){
			// summary:
			//		Checks for whitespace
			return (this.trim ? /^\s*$/ : /^$/).test(value); // Boolean
		},

		getErrorMessage: function(/*Boolean*/ /*===== isFocused =====*/){
			// summary:
			//		Return an error message to show if appropriate
			// tags:
			//		protected
			return (this.required && this._isEmpty(this.textbox.value)) ? this.missingMessage : this.invalidMessage; // String
		},

		getPromptMessage: function(/*Boolean*/ /*===== isFocused =====*/){
			// summary:
			//		Return a hint message to show when widget is first focused
			// tags:
			//		protected
			return this.promptMessage; // String
		},

		_maskValidSubsetError: true,
		validate: function(/*Boolean*/ isFocused){
			// summary:
			//		Called by oninit, onblur, and onkeypress.
			// description:
			//		Show missing or invalid messages if appropriate, and highlight textbox field.
			// tags:
			//		protected
			var message = "";
			var isValid = this.disabled || this.isValid(isFocused);
			if(isValid){ this._maskValidSubsetError = true; }
			var isEmpty = this._isEmpty(this.textbox.value);
			var isValidSubset = !isValid && isFocused && this._isValidSubset();
			this._set("state", isValid ? "" : (((((!this._hasBeenBlurred || isFocused) && isEmpty) || isValidSubset) && this._maskValidSubsetError) ? "Incomplete" : "Error"));
			this.focusNode.setAttribute("aria-invalid", isValid ? "false" : "true");

			if(this.state == "Error"){
				this._maskValidSubsetError = isFocused && isValidSubset; // we want the error to show up after a blur and refocus
				message = this.getErrorMessage(isFocused);
			}else if(this.state == "Incomplete"){
				message = this.getPromptMessage(isFocused); // show the prompt whenever the value is not yet complete
				this._maskValidSubsetError = !this._hasBeenBlurred || isFocused; // no Incomplete warnings while focused
			}else if(isEmpty){
				message = this.getPromptMessage(isFocused); // show the prompt whenever there's no error and no text
			}
			this.set("message", message);

			return isValid;
		},

		displayMessage: function(/*String*/ message){
			// summary:
			//		Overridable method to display validation errors/hints.
			//		By default uses a tooltip.
			// tags:
			//		extension
			if(message && this.focused){
				Tooltip.show(message, this.domNode, this.tooltipPosition, !this.isLeftToRight());
			}else{
				Tooltip.hide(this.domNode);
			}
		},

		_refreshState: function(){
			// Overrides TextBox._refreshState()
			this.validate(this.focused);
			this.inherited(arguments);
		},

		//////////// INITIALIZATION METHODS ///////////////////////////////////////

		constructor: function(){
			this.constraints = {};
		},

		_setConstraintsAttr: function(/*Object*/ constraints){
			if(!constraints.locale && this.lang){
				constraints.locale = this.lang;
			}
			this._set("constraints", constraints);
			this._computePartialRE();
		},

		_computePartialRE: function(){
			var p = this.regExpGen(this.constraints);
			this.regExp = p;
			var partialre = "";
			// parse the regexp and produce a new regexp that matches valid subsets
			// if the regexp is .* then there's no use in matching subsets since everything is valid
			if(p != ".*"){ this.regExp.replace(/\\.|\[\]|\[.*?[^\\]{1}\]|\{.*?\}|\(\?[=:!]|./g,
				function(re){
					switch(re.charAt(0)){
						case '{':
						case '+':
						case '?':
						case '*':
						case '^':
						case '$':
						case '|':
						case '(':
							partialre += re;
							break;
						case ")":
							partialre += "|$)";
							break;
						 default:
							partialre += "(?:"+re+"|$)";
							break;
					}
				}
			);}
			try{ // this is needed for now since the above regexp parsing needs more test verification
				"".search(partialre);
			}catch(e){ // should never be here unless the original RE is bad or the parsing is bad
				partialre = this.regExp;
				console.warn('RegExp error in ' + this.declaredClass + ': ' + this.regExp);
			} // should never be here unless the original RE is bad or the parsing is bad
			this._partialre = "^(?:" + partialre + ")$";
		},

		postMixInProperties: function(){
			this.inherited(arguments);
			this.messages = i18n.getLocalization("dijit.form", "validate", this.lang);
			if(this.invalidMessage == "$_unset_$"){ this.invalidMessage = this.messages.invalidMessage; }
			if(!this.invalidMessage){ this.invalidMessage = this.promptMessage; }
			if(this.missingMessage == "$_unset_$"){ this.missingMessage = this.messages.missingMessage; }
			if(!this.missingMessage){ this.missingMessage = this.invalidMessage; }
			this._setConstraintsAttr(this.constraints); // this needs to happen now (and later) due to codependency on _set*Attr calls attachPoints
		},

		_setDisabledAttr: function(/*Boolean*/ value){
			this.inherited(arguments);	// call FormValueWidget._setDisabledAttr()
			this._refreshState();
		},

		_setRequiredAttr: function(/*Boolean*/ value){
			this._set("required", value);
			this.focusNode.setAttribute("aria-required", value);
			this._refreshState();
		},

		_setMessageAttr: function(/*String*/ message){
			this._set("message", message);
			this.displayMessage(message);
		},

		reset:function(){
			// Overrides dijit.form.TextBox.reset() by also
			// hiding errors about partial matches
			this._maskValidSubsetError = true;
			this.inherited(arguments);
		},

		_onBlur: function(){
			// the message still exists but for back-compat, and to erase the tooltip
			// (if the message is being displayed as a tooltip), call displayMessage('')
			this.displayMessage('');

			this.inherited(arguments);
		}
	});
});

},
'gridx/core/Column':function(){
define([
	"dojo/_base/declare"
], function(declare){

	
	return declare(/*===== "gridx.core.Column", =====*/[], {
		// summary:
		//		Represents a column of a grid
		// description:
		//		An instance of this class represents a grid column.
		//		This class should not be directly instantiated by users. It should be returned by grid APIs.

		/*=====
		// id: [readonly] String
		//		The ID of this column
		id: null,

		// grid: [readonly] gridx.Grid
		//		Reference to the grid
		grid: null,

		// model: [readonly] grid.core.model.Model
		//		Reference to this grid model
		model: null,
		=====*/

		

		constructor: function(grid, id){
			this.grid = grid;
			this.model = grid.model;
			this.id = id;
		},

		
		index: function(){
			// summary:
			//		Get the index of this column
			// returns:
			//		The index of this column
			var c = this.def();
			return c ? c.index : -1;	//Integer
		},

		
		def: function(){
			// summary:
			//		Get the definition of this column
			// returns:
			//		The definition of this column
			return this.grid._columnsById[this.id];	//Object
		},

		
		cell: function(row, isId, parentId){
			// summary:
			//		Get a cell object in this column
			// row: gridx.core.Row|Integer|String
			//		Row index or row ID or a row object
			// returns:
			//		If the params are valid and the row is in cache return a cell object, else return null
			return this.grid.cell(row, this, isId, parentId);	//gridx.core.Cell|null 
		},

		
		cells: function(start, count, parentId){
			// summary:
			//		Get cells in this column.
			//		If some rows are not in cache, there will be NULLs in the returned array.
			// start: Integer?
			//		The row index of the first cell in the returned array.
			//		If omitted, defaults to 0, so column.cells() gets all the cells.
			// count: Integer?
			//		The number of cells to return.
			//		If omitted, all the cells starting from row 'start' will be returned.
			// returns:
			//		An array of cells in this column
			var t = this,
				g = t.grid,
				cells = [],
				total = g.rowCount(parentId),
				i = start || 0,
				end = count >= 0 ? start + count : total;
			for(; i < end && i < total; ++i){
				cells.push(g.cell(i, t, 0, parentId));	//1 as true
			}
			return cells;	//gridx.core.Cell[]
		},

		
		name: function(){
			// summary:
			//		Get the name of this column.
			// description:
			//		Column name is the string displayed in the grid header cell.
			//		Column names can be anything. Two columns can share one name. But they must have different IDs.
			// returns:
			//		The name of this column
			return this.def().name || '';	//String
		},

		
		setName: function(name){
			// summary:
			//		Set the name of this column
			// name: String
			//		The new name
			// returns:
			//		Return self reference, so as to cascade methods
			this.def().name = name;
			return this;	//gridx.core.Column
		},

		
		field: function(){
			// summary:
			//		Get the store field of this column
			// description:
			//		If a column corresponds to a field in store, this method returns the field.
			//		It's possible for a column to have no store field related.
			// returns:
			//		The store field of this column
			return this.def().field || null;	//String
		},

		
		getWidth: function(){
			// summary:
			//		Get the width of this column
			// returns:
			//		The CSS value of column width
			return this.def().width;	//String
		}
	});
});

},
'dijit/_base/sniff':function(){
define([ "dojo/uacss" ], function(){
	// module:
	//		dijit/_base/sniff
	// summary:
	//		Back compatibility module, new code should require dojo/uacss directly instead of this module.
});

},
'ibm/tivoli/tip/dojo/_Global':function(){
// wrapped by build app
define("ibm/tivoli/tip/dojo/_Global", ["dijit","dojo","dojox","dojo/i18n!ibm/tivoli/tip/dojo/nls/resources"], function(dijit,dojo,dojox){
/****************************************************************************
* _Global -- the actual Global object
*
* Notice that there is a '_' underscore prefixing this object.  This is
* important because removing it will delete the Global module functions
* such as getSingleton.  This is OK because we will never construct one
* of these things without calling getSingleton.
****************************************************************************/
dojo.requireLocalization("ibm.tivoli.tip.dojo", "resources");
dojo.provide("ibm.tivoli.tip.dojo._Global");

dojo.declare("ibm.tivoli.tip.dojo._Global",null,
{	 
  _actionBroker:  null,
  _portletTitle: null,
  contentIsolation: false,
  namespace: "",
	
	constructor: function( context, contentIsolation )
	{
      //add all functions, fields, objects inside context to this object
      dojo.mixin( this, context );
      
      //create path to JSON controller servlet
      this.controller = context.controllerURL;
         
      // Create resource bundle for TIP based resource strings
      this.nlsTIPResources = dojo.i18n.getLocalization( "ibm.tivoli.tip.dojo", "resources" );
      
	  //Setting Namespace variable
	  this.namespace = context.namespace;
	  
	  //Setting Boolean variable to show MessageDialog in Content Isolation mode
      this.contentIsolation = contentIsolation;
	},

  /************************************************************************
  * Get the action broker.
  ************************************************************************/
  /* ibm.tivoli.tip.dojo.ActionBroker */ 
  getActionBroker : function() 
  {
    return( this._actionBroker );
  },

  /************************************************************************
  * Set the action broker.
  ************************************************************************/
  /* ibm.tivoli.tip.dojo.ActionBroker */ 
  setActionBroker : function( /* ibm.tivoli.tip.dojo.ActionBroker */ broker ) 
  {
    return( this._actionBroker = broker );
  },
  
  getPortletTitle : function () {
  	return this._portletTitle;
  },
  
  setPortletTitle: function (/*String*/ title) {
  	this._portletTitle = title;
  },
	
	/************************************************************************
	* Generate Conextual URL
	************************************************************************/
	contextualURL : function( baseURL ) 
	{
  	return( this.contextPath + baseURL );
	},

	/************************************************************************
	* Generate namespaced ID from a BASE ID.
	************************************************************************/
	namespaceID: function( baseID ) 
	{
  	return( this.namespace + baseID );
	},	

  /************************************************************************
  * show error message dialog
  ************************************************************************/
  showErrorMessage: function( msgMixin )
  {
    var msgKW = {
              type:         "Error",
              showCheckbox: false,
			  namespace: this.namespace,
			  contentIsolation: this.contentIsolation,
              portletTitle: this._portletTitle
            };

    dojo.mixin( msgKW, msgMixin );
    // var x = new ibm.tivoli.tip.dijit.TIPMessageDialog( msgKW ); 
    // Refer 40366 - TWL RTC
	var x = new dojoe.messagedialog.MessageDialog( msgKW );
    x.show();
  },

  /************************************************************************
  * Retrieve a specific NLS string from the TIP resource bundle.  Deployers
  * could create a method like this for their own product that calls nls
  * with their resource bundle for convenience.  This is meant to only
  * be called by TIP code.
  ************************************************************************/
  tipnls: function( nlsKey, inserts )
  {
     return this.nls( nlsKey, inserts, this.nlsTIPResources );
  },

  /************************************************************************
  * Retrieve a specific NLS string optionally performing substitution(s).
  * This is a fairly generic routine meant to be used by exploiters of 
  * the Global object.  If an exploiter specifies a resource bundle in 
  * the Global context via the name 'nlsResources' when creating the Global object
  * then a bundle need not be specified on the invocation of this method
  * because it will use nlsResources instead.
  ************************************************************************/
  nls: function( nlsKey, inserts, myBundle )
  {
    // attempt to retrieve the specified string via its key
    var s = null;
    
    // If a bundle has been specified, then use it, else
    // use the bundle specified on nlsResources.
    if (myBundle !== null)
    {
       s = myBundle[ nlsKey ];
    }
    else
    {
       s = this.nlsResources[ nlsKey ];
    }

    // substitute "key" if not able to resolve
    if ( ! s )
    {
      s = nlsKey;
    }

    // string insert(s) specified?
    if ( inserts ) 
    {
      // if not already a KW object -or- array 
      if ( ! ( typeof inserts == 'object' ) )
      {
        // convert single number/string/... to array
        inserts = [ inserts ];
      }

      // perform the substitutions
      s = dojo.string.substitute( s, inserts );
    }

    return( s );
  },
  
  showMessageDialog: function (type, msgID, msg, showCheckbox) {
	if (!showCheckbox) {
		showCheckbox = false;
	}
    var modalWindow = new dojoe.messagedialog.MessageDialog (
		                                            { 
		                                              type: type,
	    	                                          message: msg,
	        	                                      messageId: msgID,
													  namespace: this.namespace,
													  contentIsolation: this.contentIsolation,
	        	                                      showCheckbox: showCheckbox
	                    	                        }); 
	modalWindow.show();
  },

  showMessage: function (msgMixin) {
	var messageid = msgMixin.messageId;
	var messagetype = messageid.substring(messageid.length-1, messageid.length);
	var type = "";
	switch(messagetype)
	{
		case "E":
			type = "error";
			break;
		case "W":
			type = "warning";
			break;
		case "C":
			type = "confirm";
			break;
		case "V":
			type = "validation";
			break;
		default:
			type = "";
	}
	
    var msgKW = {
              type: type,
              showCheckbox: false,
			  namespace: this.namespace,
			  contentIsolation: this.contentIsolation,
              portletTitle: this._portletTitle
            };

    dojo.mixin( msgKW, msgMixin );
    var x = new dojoe.messagedialog.MessageDialog( msgKW );
	x.show();
  },
  
  handleServerException: function (exception)  {
 	console.error ("Received server exception: " + exception.messageID + ": " + exception.messageString);
	console.error ("with server stack trace: ");
	console.error (exception.stackTrace);
	this.showMessageDialog (exception.type, exception.messageID, exception.messageString, false);
  },
  
	isBidiChar: function (c){
		if (c >= 0x05d0 && c <= 0x065f ||
			c >= 0x066a && c <= 0x06ef ||
			c >= 0x06fa && c <= 0x07ff ||
			c >= 0xfb1d && c <= 0xfdff ||
			c >= 0xfe70 && c <= 0xfefc) {
			console.log ("isBidiChar: true: " + c);
			return true;
		} else {
			console.log ("isBidiChar: false: " + c);
			return false;
		}
	},

	isLatinChar: function(c){
    	if((c > 64 && c < 91)||(c > 96 && c < 123)) {
	        return true;
		} else {            
    		return false;
    	}    
	},

	checkContextual: function (text){
//		var text = obj.value;
		var len = text.length;
		for(var i = 0;i < len;i++)  {
			symbol = text.charCodeAt(i);
			if(this.isBidiChar(symbol)) {
				return "RTL";
		//	} else if(this.isLatinChar(symbol)) {
		//		return "LTR";
			}
		}
		return "LTR";
	},
  
  bidiEnableLabel: function (text) {
		var result = text;
		
		if (this.bidiTxtDir == "RTL") {
			result = "&#8235;" + text + "&#8236;"; 
		} else if (this.bidiTxtDir == "CONTEXTUAL") {
			var contextDir = this.checkContextual (text)
			if (contextDir == "RTL") {
				result = "&#8235;" + text + "&#8236;";
			} else {
				result = text;
			}
		} else {  // ltf
			result = text;
		}

		return (result);
  }
  
    
} );
});

},
'dijit/form/HorizontalSlider':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dnd/move",
	"dojo/_base/event", // event.stop
	"dojo/_base/fx", // fx.animateProperty
	"dojo/dom-geometry", // domGeometry.position
	"dojo/dom-style", // domStyle.getComputedStyle
	"dojo/keys", // keys.DOWN_ARROW keys.END keys.HOME keys.LEFT_ARROW keys.PAGE_DOWN keys.PAGE_UP keys.RIGHT_ARROW keys.UP_ARROW
	"dojo/_base/lang", // lang.hitch
	"dojo/_base/sniff", // has("ie") has("mozilla")
	"dojo/dnd/Moveable", // Moveable
	"dojo/dnd/Mover", // Mover Mover.prototype.destroy.apply
	"dojo/query", // query
	"../registry", // registry.findWidgets
	"../focus",		// focus.focus()
	"../typematic",
	"./Button",
	"./_FormValueWidget",
	"../_Container",
	"dojo/text!./templates/HorizontalSlider.html"
], function(array, declare, move, event, fx, domGeometry, domStyle, keys, lang, has, Moveable, Mover, query,
			registry, focus, typematic, Button, _FormValueWidget, _Container, template){

/*=====
	var Button = dijit.form.Button;
	var _FormValueWidget = dijit.form._FormValueWidget;
	var _Container = dijit._Container;
=====*/

// module:
//		dijit/form/HorizontalSlider
// summary:
//		A form widget that allows one to select a value with a horizontally draggable handle


var _SliderMover = declare("dijit.form._SliderMover", Mover, {
	onMouseMove: function(e){
		var widget = this.widget;
		var abspos = widget._abspos;
		if(!abspos){
			abspos = widget._abspos = domGeometry.position(widget.sliderBarContainer, true);
			widget._setPixelValue_ = lang.hitch(widget, "_setPixelValue");
			widget._isReversed_ = widget._isReversed();
		}
		var pixelValue = e[widget._mousePixelCoord] - abspos[widget._startingPixelCoord];
		widget._setPixelValue_(widget._isReversed_ ? (abspos[widget._pixelCount]-pixelValue) : pixelValue, abspos[widget._pixelCount], false);
	},

	destroy: function(e){
		Mover.prototype.destroy.apply(this, arguments);
		var widget = this.widget;
		widget._abspos = null;
		widget._setValueAttr(widget.value, true);
	}
});

var HorizontalSlider = declare("dijit.form.HorizontalSlider", [_FormValueWidget, _Container], {
	// summary:
	//		A form widget that allows one to select a value with a horizontally draggable handle

	templateString: template,

	// Overrides FormValueWidget.value to indicate numeric value
	value: 0,

	// showButtons: [const] Boolean
	//		Show increment/decrement buttons at the ends of the slider?
	showButtons: true,

	// minimum:: [const] Integer
	//		The minimum value the slider can be set to.
	minimum: 0,

	// maximum: [const] Integer
	//		The maximum value the slider can be set to.
	maximum: 100,

	// discreteValues: Integer
	//		If specified, indicates that the slider handle has only 'discreteValues' possible positions,
	//		and that after dragging the handle, it will snap to the nearest possible position.
	//		Thus, the slider has only 'discreteValues' possible values.
	//
	//		For example, if minimum=10, maxiumum=30, and discreteValues=3, then the slider handle has
	//		three possible positions, representing values 10, 20, or 30.
	//
	//		If discreteValues is not specified or if it's value is higher than the number of pixels
	//		in the slider bar, then the slider handle can be moved freely, and the slider's value will be
	//		computed/reported based on pixel position (in this case it will likely be fractional,
	//		such as 123.456789).
	discreteValues: Infinity,

	// pageIncrement: Integer
	//		If discreteValues is also specified, this indicates the amount of clicks (ie, snap positions)
	//		that the slider handle is moved via pageup/pagedown keys.
	//		If discreteValues is not specified, it indicates the number of pixels.
	pageIncrement: 2,

	// clickSelect: Boolean
	//		If clicking the slider bar changes the value or not
	clickSelect: true,

	// slideDuration: Number
	//		The time in ms to take to animate the slider handle from 0% to 100%,
	//		when clicking the slider bar to make the handle move.
	slideDuration: registry.defaultDuration,

	// Map widget attributes to DOMNode attributes.
	_setIdAttr: "",		// Override _FormWidget which sends id to focusNode

	baseClass: "dijitSlider",

	// Apply CSS classes to up/down arrows and handle per mouse state
	cssStateNodes: {
		incrementButton: "dijitSliderIncrementButton",
		decrementButton: "dijitSliderDecrementButton",
		focusNode: "dijitSliderThumb"
	},

	_mousePixelCoord: "pageX",
	_pixelCount: "w",
	_startingPixelCoord: "x",
	_handleOffsetCoord: "left",
	_progressPixelSize: "width",

	_onKeyUp: function(/*Event*/ e){
		if(this.disabled || this.readOnly || e.altKey || e.ctrlKey || e.metaKey){ return; }
		this._setValueAttr(this.value, true);
	},

	_onKeyPress: function(/*Event*/ e){
		if(this.disabled || this.readOnly || e.altKey || e.ctrlKey || e.metaKey){ return; }
		switch(e.charOrCode){
			case keys.HOME:
				this._setValueAttr(this.minimum, false);
				break;
			case keys.END:
				this._setValueAttr(this.maximum, false);
				break;
			// this._descending === false: if ascending vertical (min on top)
			// (this._descending || this.isLeftToRight()): if left-to-right horizontal or descending vertical
			case ((this._descending || this.isLeftToRight()) ? keys.RIGHT_ARROW : keys.LEFT_ARROW):
			case (this._descending === false ? keys.DOWN_ARROW : keys.UP_ARROW):
			case (this._descending === false ? keys.PAGE_DOWN : keys.PAGE_UP):
				this.increment(e);
				break;
			case ((this._descending || this.isLeftToRight()) ? keys.LEFT_ARROW : keys.RIGHT_ARROW):
			case (this._descending === false ? keys.UP_ARROW : keys.DOWN_ARROW):
			case (this._descending === false ? keys.PAGE_UP : keys.PAGE_DOWN):
				this.decrement(e);
				break;
			default:
				return;
		}
		event.stop(e);
	},

	_onHandleClick: function(e){
		if(this.disabled || this.readOnly){ return; }
		if(!has("ie")){
			// make sure you get focus when dragging the handle
			// (but don't do on IE because it causes a flicker on mouse up (due to blur then focus)
			focus.focus(this.sliderHandle);
		}
		event.stop(e);
	},

	_isReversed: function(){
		// summary:
		//		Returns true if direction is from right to left
		// tags:
		//		protected extension
		return !this.isLeftToRight();
	},

	_onBarClick: function(e){
		if(this.disabled || this.readOnly || !this.clickSelect){ return; }
		focus.focus(this.sliderHandle);
		event.stop(e);
		var abspos = domGeometry.position(this.sliderBarContainer, true);
		var pixelValue = e[this._mousePixelCoord] - abspos[this._startingPixelCoord];
		this._setPixelValue(this._isReversed() ? (abspos[this._pixelCount] - pixelValue) : pixelValue, abspos[this._pixelCount], true);
		this._movable.onMouseDown(e);
	},

	_setPixelValue: function(/*Number*/ pixelValue, /*Number*/ maxPixels, /*Boolean?*/ priorityChange){
		if(this.disabled || this.readOnly){ return; }
		var count = this.discreteValues;
		if(count <= 1 || count == Infinity){ count = maxPixels; }
		count--;
		var pixelsPerValue = maxPixels / count;
		var wholeIncrements = Math.round(pixelValue / pixelsPerValue);
		this._setValueAttr(Math.max(Math.min((this.maximum-this.minimum)*wholeIncrements/count + this.minimum, this.maximum), this.minimum), priorityChange);
	},

	_setValueAttr: function(/*Number*/ value, /*Boolean?*/ priorityChange){
		// summary:
		//		Hook so set('value', value) works.
		this._set("value", value);
		this.valueNode.value = value;
		this.focusNode.setAttribute("aria-valuenow", value);
		this.inherited(arguments);
		var percent = (value - this.minimum) / (this.maximum - this.minimum);
		var progressBar = (this._descending === false) ? this.remainingBar : this.progressBar;
		var remainingBar = (this._descending === false) ? this.progressBar : this.remainingBar;
		if(this._inProgressAnim && this._inProgressAnim.status != "stopped"){
			this._inProgressAnim.stop(true);
		}
		if(priorityChange && this.slideDuration > 0 && progressBar.style[this._progressPixelSize]){
			// animate the slider
			var _this = this;
			var props = {};
			var start = parseFloat(progressBar.style[this._progressPixelSize]);
			var duration = this.slideDuration * (percent-start/100);
			if(duration == 0){ return; }
			if(duration < 0){ duration = 0 - duration; }
			props[this._progressPixelSize] = { start: start, end: percent*100, units:"%" };
			this._inProgressAnim = fx.animateProperty({ node: progressBar, duration: duration,
				onAnimate: function(v){
					remainingBar.style[_this._progressPixelSize] = (100 - parseFloat(v[_this._progressPixelSize])) + "%";
				},
				onEnd: function(){
					delete _this._inProgressAnim;
				},
				properties: props
			});
			this._inProgressAnim.play();
		}else{
			progressBar.style[this._progressPixelSize] = (percent*100) + "%";
			remainingBar.style[this._progressPixelSize] = ((1-percent)*100) + "%";
		}
	},

	_bumpValue: function(signedChange, /*Boolean?*/ priorityChange){
		if(this.disabled || this.readOnly){ return; }
		var s = domStyle.getComputedStyle(this.sliderBarContainer);
		var c = domGeometry.getContentBox(this.sliderBarContainer, s);
		var count = this.discreteValues;
		if(count <= 1 || count == Infinity){ count = c[this._pixelCount]; }
		count--;
		var value = (this.value - this.minimum) * count / (this.maximum - this.minimum) + signedChange;
		if(value < 0){ value = 0; }
		if(value > count){ value = count; }
		value = value * (this.maximum - this.minimum) / count + this.minimum;
		this._setValueAttr(value, priorityChange);
	},

	_onClkBumper: function(val){
		if(this.disabled || this.readOnly || !this.clickSelect){ return; }
		this._setValueAttr(val, true);
	},

	_onClkIncBumper: function(){
		this._onClkBumper(this._descending === false ? this.minimum : this.maximum);
	},

	_onClkDecBumper: function(){
		this._onClkBumper(this._descending === false ? this.maximum : this.minimum);
	},

	decrement: function(/*Event*/ e){
		// summary:
		//		Decrement slider
		// tags:
		//		private
		this._bumpValue(e.charOrCode == keys.PAGE_DOWN ? -this.pageIncrement : -1);
	},

	increment: function(/*Event*/ e){
		// summary:
		//		Increment slider
		// tags:
		//		private
		this._bumpValue(e.charOrCode == keys.PAGE_UP ? this.pageIncrement : 1);
	},

	_mouseWheeled: function(/*Event*/ evt){
		// summary:
		//		Event handler for mousewheel where supported
		event.stop(evt);
		var janky = !has("mozilla");
		var scroll = evt[(janky ? "wheelDelta" : "detail")] * (janky ? 1 : -1);
		this._bumpValue(scroll < 0 ? -1 : 1, true); // negative scroll acts like a decrement
	},

	startup: function(){
		if(this._started){ return; }

		array.forEach(this.getChildren(), function(child){
			if(this[child.container] != this.containerNode){
				this[child.container].appendChild(child.domNode);
			}
		}, this);

		this.inherited(arguments);
	},

	_typematicCallback: function(/*Number*/ count, /*Object*/ button, /*Event*/ e){
		if(count == -1){
			this._setValueAttr(this.value, true);
		}else{
			this[(button == (this._descending? this.incrementButton : this.decrementButton)) ? "decrement" : "increment"](e);
		}
	},

	buildRendering: function(){
		this.inherited(arguments);
		if(this.showButtons){
			this.incrementButton.style.display="";
			this.decrementButton.style.display="";
		}

		// find any associated label element and add to slider focusnode.
		var label = query('label[for="'+this.id+'"]');
		if(label.length){
			label[0].id = (this.id+"_label");
			this.focusNode.setAttribute("aria-labelledby", label[0].id);
		}

		this.focusNode.setAttribute("aria-valuemin", this.minimum);
		this.focusNode.setAttribute("aria-valuemax", this.maximum);
	},

	postCreate: function(){
		this.inherited(arguments);

		if(this.showButtons){
			this._connects.push(typematic.addMouseListener(
				this.decrementButton, this, "_typematicCallback", 25, 500));
			this._connects.push(typematic.addMouseListener(
				this.incrementButton, this, "_typematicCallback", 25, 500));
		}
		this.connect(this.domNode, !has("mozilla") ? "onmousewheel" : "DOMMouseScroll", "_mouseWheeled");

		// define a custom constructor for a SliderMover that points back to me
		var mover = declare(_SliderMover, {
			widget: this
		});
		this._movable = new Moveable(this.sliderHandle, {mover: mover});

		this._layoutHackIE7();
	},

	destroy: function(){
		this._movable.destroy();
		if(this._inProgressAnim && this._inProgressAnim.status != "stopped"){
			this._inProgressAnim.stop(true);
		}
		this._supportingWidgets = registry.findWidgets(this.domNode); // tells destroy about pseudo-child widgets (ruler/labels)
		this.inherited(arguments);
	}
});

HorizontalSlider._Mover = _SliderMover;	// for monkey patching

return HorizontalSlider;
});

},
'dojox/grid/_FocusManager':function(){
define([
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/_base/declare",
	"dojo/_base/connect",
	"dojo/_base/event",
	"dojo/_base/sniff",
	"dojo/query",
	"./util",
	"dojo/_base/html"
], function(array, lang, declare, connect, event, has, query, util, html){

// focus management
return declare("dojox.grid._FocusManager", null, {
	// summary:
	//	Controls grid cell focus. Owned by grid and used internally for focusing.
	//	Note: grid cell actually receives keyboard input only when cell is being edited.
	constructor: function(inGrid){
		this.grid = inGrid;
		this.cell = null;
		this.rowIndex = -1;
		this._connects = [];
		this._headerConnects = [];
		this.headerMenu = this.grid.headerMenu;
		this._connects.push(connect.connect(this.grid.domNode, "onfocus", this, "doFocus"));
		this._connects.push(connect.connect(this.grid.domNode, "onblur", this, "doBlur"));
		this._connects.push(connect.connect(this.grid.domNode, "mousedown", this, "_mouseDown"));
		this._connects.push(connect.connect(this.grid.domNode, "mouseup", this, "_mouseUp"));
		this._connects.push(connect.connect(this.grid.domNode, "oncontextmenu", this, "doContextMenu"));
		this._connects.push(connect.connect(this.grid.lastFocusNode, "onfocus", this, "doLastNodeFocus"));
		this._connects.push(connect.connect(this.grid.lastFocusNode, "onblur", this, "doLastNodeBlur"));
		this._connects.push(connect.connect(this.grid,"_onFetchComplete", this, "_delayedCellFocus"));
		this._connects.push(connect.connect(this.grid,"postrender", this, "_delayedHeaderFocus"));
	},
	destroy: function(){
		array.forEach(this._connects, connect.disconnect);
		array.forEach(this._headerConnects, connect.disconnect);
		delete this.grid;
		delete this.cell;
	},
	_colHeadNode: null,
	_colHeadFocusIdx: null,
	_contextMenuBindNode: null,
	tabbingOut: false,
	focusClass: "dojoxGridCellFocus",
	focusView: null,
	initFocusView: function(){
		this.focusView = this.grid.views.getFirstScrollingView() || this.focusView || this.grid.views.views[0];
		this._initColumnHeaders();
	},
	isFocusCell: function(inCell, inRowIndex){
		// summary:
		//	states if the given cell is focused
		// inCell: object
		//	grid cell object
		// inRowIndex: int
		//	grid row index
		// returns:
		//	true of the given grid cell is focused
		return (this.cell == inCell) && (this.rowIndex == inRowIndex);
	},
	isLastFocusCell: function(){
		if(this.cell){
			return (this.rowIndex == this.grid.rowCount-1) && (this.cell.index == this.grid.layout.cellCount-1);
		}
		return false;
	},
	isFirstFocusCell: function(){
		if(this.cell){
			return (this.rowIndex === 0) && (this.cell.index === 0);
		}
		return false;
	},
	isNoFocusCell: function(){
		return (this.rowIndex < 0) || !this.cell;
	},
	isNavHeader: function(){
		// summary:
		//	states whether currently navigating among column headers.
		// returns:
		//	true if focus is on a column header; false otherwise.
		return (!!this._colHeadNode);
	},
	getHeaderIndex: function(){
		// summary:
		//	if one of the column headers currently has focus, return its index.
		// returns:
		//	index of the focused column header, or -1 if none have focus.
		if(this._colHeadNode){
			return array.indexOf(this._findHeaderCells(), this._colHeadNode);
		}else{
			return -1;
		}
	},
	_focusifyCellNode: function(inBork){
		var n = this.cell && this.cell.getNode(this.rowIndex);
		if(n){
			html.toggleClass(n, this.focusClass, inBork);
			if(inBork){
				var sl = this.scrollIntoView();
				try{
					if(!this.grid.edit.isEditing()){
						util.fire(n, "focus");
						if(sl){ this.cell.view.scrollboxNode.scrollLeft = sl; }
					}
				}catch(e){}
			}
		}
	},
	_delayedCellFocus: function(){
		if(this.isNavHeader()||!this.grid.focused){
				return;
		}
		var n = this.cell && this.cell.getNode(this.rowIndex);
		if(n){
			try{
				if(!this.grid.edit.isEditing()){
					html.toggleClass(n, this.focusClass, true);
					if(this._colHeadNode){
						this.blurHeader();
					}
					util.fire(n, "focus");
				}
			}
			catch(e){}
		}
	},
	_delayedHeaderFocus: function(){
		if(this.isNavHeader()){
			this.focusHeader();
			dijit.focus(this.grid.domNode);
		}
	},
	_initColumnHeaders: function(){
		array.forEach(this._headerConnects, connect.disconnect);
		this._headerConnects = [];
		var headers = this._findHeaderCells();
		for(var i = 0; i < headers.length; i++){
			this._headerConnects.push(connect.connect(headers[i], "onfocus", this, "doColHeaderFocus"));
			this._headerConnects.push(connect.connect(headers[i], "onblur", this, "doColHeaderBlur"));
		}
	},
	_findHeaderCells: function(){
		// This should be a one liner:
		//	query("th[tabindex=-1]", this.grid.viewsHeaderNode);
		// But there is a bug in query() for IE -- see trac #7037.
		var allHeads = query("th", this.grid.viewsHeaderNode);
		var headers = [];
		for (var i = 0; i < allHeads.length; i++){
			var aHead = allHeads[i];
			var hasTabIdx = html.hasAttr(aHead, "tabIndex");
			var tabindex = html.attr(aHead, "tabIndex");
			if (hasTabIdx && tabindex < 0) {
				headers.push(aHead);
			}
		}
		return headers;
	},
	_setActiveColHeader: function(/*Node*/colHeaderNode, /*Integer*/colFocusIdx, /*Integer*/ prevColFocusIdx){
		//console.log("setActiveColHeader() - colHeaderNode:colFocusIdx:prevColFocusIdx = " + colHeaderNode + ":" + colFocusIdx + ":" + prevColFocusIdx);
		this.grid.domNode.setAttribute("aria-activedescendant",colHeaderNode.id);
		if (prevColFocusIdx != null && prevColFocusIdx >= 0 && prevColFocusIdx != colFocusIdx){
			html.toggleClass(this._findHeaderCells()[prevColFocusIdx],this.focusClass,false);
		}
		html.toggleClass(colHeaderNode,this.focusClass, true);
		this._colHeadNode = colHeaderNode;
		this._colHeadFocusIdx = colFocusIdx;
		this._scrollHeader(this._colHeadFocusIdx);
	},
	scrollIntoView: function(){
		var info = (this.cell ? this._scrollInfo(this.cell) : null);
		if(!info || !info.s){
			return null;
		}
		var rt = this.grid.scroller.findScrollTop(this.rowIndex);
		// place cell within horizontal view
		if(info.n && info.sr){
			if(info.n.offsetLeft + info.n.offsetWidth > info.sr.l + info.sr.w){
				info.s.scrollLeft = info.n.offsetLeft + info.n.offsetWidth - info.sr.w;
			}else if(info.n.offsetLeft < info.sr.l){
				info.s.scrollLeft = info.n.offsetLeft;
			}
		}
		// place cell within vertical view
		if(info.r && info.sr){
			if(rt + info.r.offsetHeight > info.sr.t + info.sr.h){
				this.grid.setScrollTop(rt + info.r.offsetHeight - info.sr.h);
			}else if(rt < info.sr.t){
				this.grid.setScrollTop(rt);
			}
		}

		return info.s.scrollLeft;
	},
	_scrollInfo: function(cell, domNode){
		if(cell){
			var cl = cell,
				sbn = cl.view.scrollboxNode,
				sbnr = {
					w: sbn.clientWidth,
					l: sbn.scrollLeft,
					t: sbn.scrollTop,
					h: sbn.clientHeight
				},
				rn = cl.view.getRowNode(this.rowIndex);
			return {
				c: cl,
				s: sbn,
				sr: sbnr,
				n: (domNode ? domNode : cell.getNode(this.rowIndex)),
				r: rn
			};
		}
		return null;
	},
	_scrollHeader: function(currentIdx){
		var info = null;
		if(this._colHeadNode){
			var cell = this.grid.getCell(currentIdx);
			if(!cell){ return; }
			info = this._scrollInfo(cell, cell.getNode(0));
		}
		if(info && info.s && info.sr && info.n){
			// scroll horizontally as needed.
			var scroll = info.sr.l + info.sr.w;
			if(info.n.offsetLeft + info.n.offsetWidth > scroll){
				info.s.scrollLeft = info.n.offsetLeft + info.n.offsetWidth - info.sr.w;
			}else if(info.n.offsetLeft < info.sr.l){
				info.s.scrollLeft = info.n.offsetLeft;
			}else if(has("ie") <= 7 && cell && cell.view.headerNode){
				// Trac 7158: scroll dojoxGridHeader for IE7 and lower
				cell.view.headerNode.scrollLeft = info.s.scrollLeft;
			}
		}
	},
	_isHeaderHidden: function(){
		// summary:
		//		determine if the grid headers are hidden
		//		relies on documented technique of setting .dojoxGridHeader { display:none; }
		// returns: Boolean
		//		true if headers are hidden
		//		false if headers are not hidden
		
		var curView = this.focusView;
		if (!curView){
			// find one so we can determine if headers are hidden
			// there is no focusView after adding items to empty grid (test_data_grid_empty.html)
			for (var i = 0, cView; (cView = this.grid.views.views[i]); i++) {
				if(cView.headerNode ){
					curView=cView;
					break;
				}
			}
		}
		return (curView && html.getComputedStyle(curView.headerNode).display == "none");
	},
	colSizeAdjust: function (e, colIdx, delta){ // adjust the column specified by colIdx by the specified delta px
		var headers = this._findHeaderCells();
		var view = this.focusView;
		if (!view) {
			for (var i = 0, cView; (cView = this.grid.views.views[i]); i++) {
				// find first view with a tableMap in order to work with empty grid
				if(cView.header.tableMap.map ){
					view=cView;
					break;
				}
			}
		}
		var curHeader = headers[colIdx];
		if (!view || (colIdx == headers.length-1 && colIdx === 0)){
			return; // can't adjust single col. grid
		}
		view.content.baseDecorateEvent(e);
		// need to adjust event with header cell info since focus is no longer on header cell
		e.cellNode = curHeader; //this.findCellTarget(e.target, e.rowNode);
		e.cellIndex = view.content.getCellNodeIndex(e.cellNode);
		e.cell = (e.cellIndex >= 0 ? this.grid.getCell(e.cellIndex) : null);
		if (view.header.canResize(e)){
			var deltaObj = {
				l: delta
			};
			var drag = view.header.colResizeSetup(e,false);
			view.header.doResizeColumn(drag, null, deltaObj);
			view.update();
		}
	},
	styleRow: function(inRow){
		return;
	},
	setFocusIndex: function(inRowIndex, inCellIndex){
		// summary:
		//	focuses the given grid cell
		// inRowIndex: int
		//	grid row index
		// inCellIndex: int
		//	grid cell index
		this.setFocusCell(this.grid.getCell(inCellIndex), inRowIndex);
	},
	setFocusCell: function(inCell, inRowIndex){
		// summary:
		//	focuses the given grid cell
		// inCell: object
		//	grid cell object
		// inRowIndex: int
		//	grid row index
		if(inCell && !this.isFocusCell(inCell, inRowIndex)){
			this.tabbingOut = false;
			if (this._colHeadNode){
				this.blurHeader();
			}
			this._colHeadNode = this._colHeadFocusIdx = null;
			this.focusView = inCell.view;
			this.focusGridView();
			this._focusifyCellNode(false);
			this.cell = inCell;
			this.rowIndex = inRowIndex;
			this._focusifyCellNode(true);
		}
		// even if this cell isFocusCell, the document focus may need to be rejiggered
		// call opera on delay to prevent keypress from altering focus
		if(has("opera")){
			setTimeout(lang.hitch(this.grid, 'onCellFocus', this.cell, this.rowIndex), 1);
		}else{
			this.grid.onCellFocus(this.cell, this.rowIndex);
		}
	},
	next: function(){
		// summary:
		//	focus next grid cell
		if(this.cell){
			var row=this.rowIndex, col=this.cell.index+1, cc=this.grid.layout.cellCount-1, rc=this.grid.rowCount-1;
			if(col > cc){
				col = 0;
				row++;
			}
			if(row > rc){
				col = cc;
				row = rc;
			}
			if(this.grid.edit.isEditing()){ //when editing, only navigate to editable cells
				var nextCell = this.grid.getCell(col);
				if (!this.isLastFocusCell() && (!nextCell.editable ||
					this.grid.canEdit && !this.grid.canEdit(nextCell, row))){
					this.cell=nextCell;
					this.rowIndex=row;
					this.next();
					return;
				}
			}
			this.setFocusIndex(row, col);
		}
	},
	previous: function(){
		// summary:
		//	focus previous grid cell
		if(this.cell){
			var row=(this.rowIndex || 0), col=(this.cell.index || 0) - 1;
			if(col < 0){
				col = this.grid.layout.cellCount-1;
				row--;
			}
			if(row < 0){
				row = 0;
				col = 0;
			}
			if(this.grid.edit.isEditing()){ //when editing, only navigate to editable cells
				var prevCell = this.grid.getCell(col);
				if (!this.isFirstFocusCell() && !prevCell.editable){
					this.cell=prevCell;
					this.rowIndex=row;
					this.previous();
					return;
				}
			}
			this.setFocusIndex(row, col);
		}
	},
	move: function(inRowDelta, inColDelta) {
		// summary:
		//	focus grid cell or  simulate focus to column header based on position relative to current focus
		// inRowDelta: int
		// vertical distance from current focus
		// inColDelta: int
		// horizontal distance from current focus

		var colDir = inColDelta < 0 ? -1 : 1;
		// Handle column headers.
		if(this.isNavHeader()){
			var headers = this._findHeaderCells();
			var savedIdx = currentIdx = array.indexOf(headers, this._colHeadNode);
			currentIdx += inColDelta;
			while(currentIdx >=0 && currentIdx < headers.length && headers[currentIdx].style.display == "none"){
				// skip over hidden column headers
				currentIdx += colDir;
			}
			if((currentIdx >= 0) && (currentIdx < headers.length)){
				this._setActiveColHeader(headers[currentIdx],currentIdx, savedIdx);
			}
		}else{
			if(this.cell){
				// Handle grid proper.
				var sc = this.grid.scroller,
					r = this.rowIndex,
					rc = this.grid.rowCount-1,
					row = Math.min(rc, Math.max(0, r+inRowDelta));
				if(inRowDelta){
					if(inRowDelta>0){
						if(row > sc.getLastPageRow(sc.page)){
							//need to load additional data, let scroller do that
							this.grid.setScrollTop(this.grid.scrollTop+sc.findScrollTop(row)-sc.findScrollTop(r));
						}
					}else if(inRowDelta<0){
						if(row <= sc.getPageRow(sc.page)){
							//need to load additional data, let scroller do that
							this.grid.setScrollTop(this.grid.scrollTop-sc.findScrollTop(r)-sc.findScrollTop(row));
						}
					}
				}
				var cc = this.grid.layout.cellCount-1,
				i = this.cell.index,
				col = Math.min(cc, Math.max(0, i+inColDelta));
				var cell = this.grid.getCell(col);
				while(col>=0 && col < cc && cell && cell.hidden === true){
					// skip hidden cells
					col += colDir;
					cell = this.grid.getCell(col);
				}
				if (!cell || cell.hidden === true){
					// don't change col if would move to hidden
					col = i;
				}
				//skip hidden row|cell
				var n = cell.getNode(row);
				if(!n && inRowDelta){
					if((row + inRowDelta) >= 0 && (row + inRowDelta) <= rc){
						this.move(inRowDelta > 0 ? ++inRowDelta : --inRowDelta, inColDelta);
					}
					return;
				}else if((!n || html.style(n, "display") === "none") && inColDelta){
					if((col + inRowDelta) >= 0 && (col + inRowDelta) <= cc){
						this.move(inRowDelta, inColDelta > 0 ? ++inColDelta : --inColDelta);
					}
					return;
				}
				this.setFocusIndex(row, col);
				//This will cause losing selection effect when using keyboard cell selection
//				if(inRowDelta){
//					this.grid.updateRow(r);
//				}
			}
		}
	},
	previousKey: function(e){
		if(this.grid.edit.isEditing()){
			event.stop(e);
			this.previous();
		}else if(!this.isNavHeader() && !this._isHeaderHidden()) {
			this.grid.domNode.focus(); // will call doFocus and set focus into header.
			event.stop(e);
		}else{
			this.tabOut(this.grid.domNode);
			if (this._colHeadFocusIdx != null) { // clear grid header focus
				html.toggleClass(this._findHeaderCells()[this._colHeadFocusIdx], this.focusClass, false);
				this._colHeadFocusIdx = null;
			}
			this._focusifyCellNode(false);
		}
	},
	nextKey: function(e) {
		var isEmpty = (this.grid.rowCount === 0);
		if(e.target === this.grid.domNode && this._colHeadFocusIdx == null){
			this.focusHeader();
			event.stop(e);
		}else if(this.isNavHeader()){
			// if tabbing from col header, then go to grid proper.
			this.blurHeader();
			if(!this.findAndFocusGridCell()){
				this.tabOut(this.grid.lastFocusNode);
			}
			this._colHeadNode = this._colHeadFocusIdx= null;
		}else if(this.grid.edit.isEditing()){
			event.stop(e);
			this.next();
		}else{
			this.tabOut(this.grid.lastFocusNode);
		}
	},
	tabOut: function(inFocusNode){
		this.tabbingOut = true;
		inFocusNode.focus();
	},
	focusGridView: function(){
		util.fire(this.focusView, "focus");
	},
	focusGrid: function(inSkipFocusCell){
		this.focusGridView();
		this._focusifyCellNode(true);
	},
	findAndFocusGridCell: function(){
		// summary:
		//		find the first focusable grid cell
		// returns: Boolean
		//		true if focus was set to a cell
		//		false if no cell found to set focus onto
		
		var didFocus = true;
		var isEmpty = (this.grid.rowCount === 0); // If grid is empty this.grid.rowCount == 0
		if (this.isNoFocusCell() && !isEmpty){
			var cellIdx = 0;
			var cell = this.grid.getCell(cellIdx);
			if (cell.hidden) {
				// if first cell isn't visible, use _colHeadFocusIdx
				// could also use a while loop to find first visible cell - not sure that is worth it
				cellIdx = this.isNavHeader() ? this._colHeadFocusIdx : 0;
			}
			this.setFocusIndex(0, cellIdx);
		}
		else if (this.cell && !isEmpty){
			if (this.focusView && !this.focusView.rowNodes[this.rowIndex]){
				// if rowNode for current index is undefined (likely as a result of a sort and because of #7304)
				// scroll to that row
				this.grid.scrollToRow(this.rowIndex);
			}
			this.focusGrid();
		}else {
			didFocus = false;
		}
		this._colHeadNode = this._colHeadFocusIdx= null;
		return didFocus;
	},
	focusHeader: function(){
		var headerNodes = this._findHeaderCells();
		var saveColHeadFocusIdx = this._colHeadFocusIdx;
		if (this._isHeaderHidden()){
			// grid header is hidden, focus a cell
			this.findAndFocusGridCell();
		}
		else if (!this._colHeadFocusIdx) {
			if (this.isNoFocusCell()) {
				this._colHeadFocusIdx = 0;
			}
			else {
				this._colHeadFocusIdx = this.cell.index;
			}
		}
		this._colHeadNode = headerNodes[this._colHeadFocusIdx];
		while(this._colHeadNode && this._colHeadFocusIdx >=0 && this._colHeadFocusIdx < headerNodes.length &&
				this._colHeadNode.style.display == "none"){
			// skip over hidden column headers
			this._colHeadFocusIdx++;
			this._colHeadNode = headerNodes[this._colHeadFocusIdx];
		}
		if(this._colHeadNode && this._colHeadNode.style.display != "none"){
			this.focusView = this.grid.getCell(this._colHeadFocusIdx).view;
			// Column header cells know longer receive actual focus.  So, for keyboard invocation of
			// contextMenu to work, the contextMenu must be bound to the grid.domNode rather than the viewsHeaderNode.
			// unbind the contextmenu from the viewsHeaderNode and to the grid when header cells are active.  Reset
			// the binding back to the viewsHeaderNode when header cells are no longer acive (in blurHeader) #10483
			if (this.headerMenu && this._contextMenuBindNode != this.grid.domNode){
				this.headerMenu.unBindDomNode(this.grid.viewsHeaderNode);
				this.headerMenu.bindDomNode(this.grid.domNode);
				this._contextMenuBindNode = this.grid.domNode;
			}
			this._setActiveColHeader(this._colHeadNode, this._colHeadFocusIdx, saveColHeadFocusIdx);
			this._scrollHeader(this._colHeadFocusIdx);
			this._focusifyCellNode(false);
		}else {
			// all col head nodes are hidden - focus the grid
			this.findAndFocusGridCell();
		}
	},
	blurHeader: function(){
		html.removeClass(this._colHeadNode, this.focusClass);
		html.removeAttr(this.grid.domNode,"aria-activedescendant");
		// reset contextMenu onto viewsHeaderNode so right mouse on header will invoke (see focusHeader)
		if (this.headerMenu && this._contextMenuBindNode == this.grid.domNode) {
			var viewsHeader = this.grid.viewsHeaderNode;
			this.headerMenu.unBindDomNode(this.grid.domNode);
			this.headerMenu.bindDomNode(viewsHeader);
			this._contextMenuBindNode = viewsHeader;
		}
	},
	doFocus: function(e){
		// trap focus only for grid dom node
		if(e && e.target != e.currentTarget){
			event.stop(e);
			return;
		}
		// don't change focus if clicking on scroller bar
		if(this._clickFocus){
			return;
		}
		// do not focus for scrolling if grid is about to blur
		if(!this.tabbingOut){
			this.focusHeader();
		}
		this.tabbingOut = false;
		event.stop(e);
	},
	doBlur: function(e){
		event.stop(e);	// FF2
	},
	doContextMenu: function(e){
	//stop contextMenu event if no header Menu to prevent default/browser contextMenu
		if (!this.headerMenu){
			event.stop(e);
		}
	},
	doLastNodeFocus: function(e){
		if (this.tabbingOut){
			this._focusifyCellNode(false);
		}else if(this.grid.rowCount >0){
			if (this.isNoFocusCell()){
				this.setFocusIndex(0,0);
			}
			this._focusifyCellNode(true);
		}else {
			this.focusHeader();
		}
		this.tabbingOut = false;
		event.stop(e);	 // FF2
	},
	doLastNodeBlur: function(e){
		event.stop(e);	 // FF2
	},
	doColHeaderFocus: function(e){
		this._setActiveColHeader(e.target,html.attr(e.target, "idx"),this._colHeadFocusIdx);
		this._scrollHeader(this.getHeaderIndex());
		event.stop(e);
	},
	doColHeaderBlur: function(e){
		html.toggleClass(e.target, this.focusClass, false);
	},
	_mouseDown: function(e){
		// a flag indicating grid is being focused by clicking
		this._clickFocus = dojo.some(this.grid.views.views, function(v){
			return v.scrollboxNode === e.target;
		});
	},
	_mouseUp: function(e){
		this._clickFocus = false;
	}
});
});

},
'gridx/modules/dnd/_Base':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/lang",
	"../../core/_Module",
	"./Avatar",
	"./_Dnd"
], function(declare, array, lang, _Module, Avatar){

	return declare(_Module, {

		// delay: Number
		//		The time delay before starting dnd after mouse down.
		delay: 2,
	
		// enabled: Boolean
		//		Whether this module is enabled.
		enabled: true,

		// canRearrange: Boolean
		//		Whether rearrange within grid using dnd iw allowed.
		canRearrange: true,

		// copyWhenDragOut: Boolean|Object
		//		When dragging out, whehter to delete in this grid.
		copyWhenDragOut: false,

		// avatar: Function
		//		The avatar used during dnd.
		avatar: Avatar,

		preload: function(args){
			var dnd = this.grid.dnd._dnd;
			dnd.register(this.name, this);
			dnd.avatar = this.arg('avatar');
		},

		checkArg: function(name, arr){
			var arg = this.arg(name);
			return (arg && lang.isObject(arg)) ? array.some(arr, function(v){
				return arg[v];
			}) : arg;
		}
	});
});

},
'dijit/main':function(){
define([
	"dojo/_base/kernel"
], function(dojo){
	// module:
	//		dijit
	// summary:
	//		The dijit package main module

	return dojo.dijit;
});

},
'gridx/modules/filter/DistinctComboBoxMenu':function(){
define([
"dojo/_base/declare",
'dijit/form/_ComboBoxMenu'
], function(declare, _ComboBoxMenu){
	return declare(_ComboBoxMenu, {
		createOptions: function(results, options, labelFunc){
			var hash = {};
			arguments[0] = results.filter(function(item){
				var label = labelFunc(item).label;
				if(hash[label]){return false;}
				else{return hash[label] = true;}
			});
			this.inherited(arguments);
		}
	});
});
},
'ibm/usmi/scheduler/wizard_pages/OptionsPage':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
/*global define */
define([ "dojo/_base/declare",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	"dijit/form/RadioButton",
	"dijit/form/NumberSpinner",
	"ibm/tivoli/tip/navwidget/dijit/DefaultMessageHandler",
	"dojoe/wizard/Page",
	"dojo/text!./templates/OptionsPage.html",
	// The resource bundle definition
	"dojo/i18n!./nls/JobLaunchWizardResources" ], function (
	declare, _TemplatedMixin, _WidgetsInTemplateMixin, RadioButton, NumberSpinner, DefaultMessageHandler, Page,
	OptionsPage, mri) {

	return declare([ Page,
		_TemplatedMixin,
		_WidgetsInTemplateMixin ], {

		templateString : OptionsPage,
		persistResources : mri,
		title : "",

		postCreate : function () {
			// set the title of the page
			this.title = this.persistResources.OPTIONS_PAGE_TITLE;
		},

		allowNext : function () {
			return this.isValid();
		},

		allowMarkAsComplete : function () {
			return true;
		},

		allowFinish : function () {
			return this.isValid();
		},

		isValid : function () {
			return this.validate();
		},

		visited : function () {
			// indicating that this page has already been visited allows our submit button to be enabled initially...
			return true;
		},

		doSubmitButtonCheck : function () {
			// Check if Submit button is disabled first
			if (this.wizard._finishButton.attr("disabled") === true) {
				// retrieve the maximum number of job instances
				var numberOfInstances = this.maxiumJobInstances.get('value');

				// Check if number of instances is valid
				if ((numberOfInstances >= 10) && (numberOfInstances <= 100)) {
					// Enable Next button
					this.wizard.enableFinishButton(true);
				}
			}
		},
		
		validate : function () {
			var msgHandler = new DefaultMessageHandler();
			var msgId = ""; // not specifying a message ID
			var msgSeverity = "error";
			var theMessageTitle = "";

			// retrieve the maximum number of job instances
			var numberOfInstances = this.maxiumJobInstances.get('value');

			if ((!numberOfInstances) || (numberOfInstances < 10) || (numberOfInstances > 100)) {
				// the user is choosing a value outside our range....that's not allowed....
				theMessageTitle = this.persistResources.MAXIMUM_JOB_INSTANCES_VALUE_ERROR;
				msgHandler.displayMessage(msgId, msgSeverity, theMessageTitle);
				return false;
			} else {
				return true;
			}
		}

	});
});

},
'ibm/usmi/statuspod/JobStatusPodDialog':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define([
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/array",
	"dijit/_WidgetBase",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	"dijit/Tooltip",
	"dojo/i18n!./nls/statuspodResources",
	"dojo/i18n!ibm/usmi/nls/frameworkResources",
	"dojo/text!./templates/jobStatuspodDialog.html",
	"dojo/text!./templates/jobMenuItemLabel.html",
	"dojo/string",
	"dojo/on",
	"ibm/usmi/core/utils/_UsmiBrowserPersistence",
	"./JobStatusPodMenuItem",
	"./JobStatusTip",
	"./_StatusPodPopupMixin",
	"dojo/dom-geometry",
	"dijit/MenuSeparator",
	"./Constants",
	"dojo/_base/connect",
	"dijit/MenuItem",
	"dijit/layout/TabContainer",
	"dijit/layout/ContentPane"
], function (dojo, declare, lang, array, WidgetBase, TemplatedMixin, WidgetsInTemplateMixin, Tooltip, MRI,
		frameworkMRI, template, menuItemLabel, string, on, _UsmiBrowserPersistence, JobStatusPodMenuItem, JobStatusTip,
		_StatusPodPopupMixin, domGeom, MenuSeparator, Constants, connect, MenuItem) {
	return declare([
		WidgetBase,
		TemplatedMixin,
		WidgetsInTemplateMixin,
		_StatusPodPopupMixin,
		_UsmiBrowserPersistence
	], {
		templateString : template,

		recentErrorJobsCount : 0,
		totalErrorJobsCount : 0,

		recentActiveJobsCount : 0,
		totalActiveJobsCount : 0,

		recentCompletedJobsCount : 0,
		totalCompletedJobsCount : 0,

		ViewAllJobsLabel : MRI.ViewAllJobsLabel,

		ErrorJobsTabTitle : MRI.ErrorJobsTabTitle,

		ActiveJobsTabTitle : MRI.ActiveJobsTabTitle,

		CompletedJobsTabTitle : MRI.CompletedJobsTabTitle,

		StartedLabel : MRI.StartedLabel,

		dataset : null,

		postCreate : function () {
			this.inherited(arguments);
			this.menuItems = [];
			var productName = this.getGlobalData("brandedProductKey", false);
			new Tooltip({
				connectId : [ this.errorJobsMenuLink
				],
				position : [ "below"
				],
				label : lang.replace(frameworkMRI.LiCTooltip, [ productName
				])
			});
			new Tooltip({
				connectId : [ this.activeJobsMenuLink
				],
				position : [ "below"
				],
				label : lang.replace(frameworkMRI.LiCTooltip, [ productName
				])
			});
			new Tooltip({
				connectId : [ this.completedJobsMenuLink
				],
				position : [ "below"
				],
				label : lang.replace(frameworkMRI.LiCTooltip, [ productName
				])
			});
			showFn = function () {
				this.nextSibling.nextSibling.style.display = "inline";
			};
			hideFn = function () {
				this.nextSibling.nextSibling.style.display = "none";
			};
			on(this.errorJobsMenuLink, "mouseover", showFn);
			on(this.errorJobsMenuLink, "mouseout", hideFn);

			on(this.activeJobsMenuLink, "mouseover", showFn);
			on(this.activeJobsMenuLink, "mouseout", hideFn);

			on(this.completedJobsMenuLink, "mouseover", showFn);
			on(this.completedJobsMenuLink, "mouseout", hideFn);

		},

		startup : function () {
			this._logger.finest("startup JobStatusPodDialog!", "startup");
			this.inherited(arguments);
			var jobButton = this.jobStatusPodTabContainer.tablist.pane2button;
			if (jobButton) {
				jobButton["completedJobs_pane"].tabContent.style.borderRightStyle = "none";
			}
		},

		updatesMenuTab : function (items, menuNode) {
			items = items || [];
			var itemIndex = 0;
			var itemsLength = items.length;
			if (menuNode === this.errorJobsMenu) {
				this.set("recentErrorJobsCount", itemsLength);
			} else if (menuNode === this.activeJobsMenu) {
				this.set("recentActiveJobsCount", itemsLength);
			} else if (menuNode === this.completedJobsMenu) {
				this.set("recentCompletedJobsCount", itemsLength);
			}
			for (itemIndex = 0; itemIndex < itemsLength; ++itemIndex) {
				var item = items[itemIndex];
				var name = item.getProperty("NAME").getValue();
				var lastRunStatus = item.getProperty("LAST_RUN_STATUS");
				var objectId = item.getId();
				if (lastRunStatus) {
					lastRunStatus = lastRunStatus.getValue();
				}
				lastRunStatus = lastRunStatus || "";

				var lastRun = item.getProperty("LAST_RUN");
				if (lastRun) {
					lastRun = lastRun.getValue();
				}
				lastRun = lastRun || "";

				if (lastRun && lastRun !== "" && lastRun !== "null") {
					lastRun = this._formatDateStr(lastRun);
				}
				this._logger.finest("Name: " + name + ", LAST_RUN_STATUS: " + lastRunStatus + ", lastRun: " + lastRun,
						"updatesMenuTab");
				//console.debug("menuItemLabel before substitution: "+menuItemLabel);
				var menuLabel = string.substitute(menuItemLabel, {
					Name : name,
					lastRun : lastRun,
					StartedLabel : this.StartedLabel
				//lastRunStatus : lastRunStatus
				});
				//console.debug("menuLabel: "+menuLabel);
				var menuItem = new JobStatusPodMenuItem({
					label : menuLabel,
					column : 0,
					objectId : objectId
				//iconClass: "statusPodMenuItemIcon"
				});

				this._logger.finest("item id is:" + item.getId(), "updatesMenuTab");

				var menuItemHoverTip = new JobStatusTip({
					entityNode : menuItem.domNode,
					item : item,
					itemId : objectId,
					dataset : this.dataset
				});

				menuItem.setHoverTip(menuItemHoverTip);

				menuNode.addChild(menuItem, 2 * itemIndex);
				this.addDescendant(menuItem);

				var menuSep = new MenuSeparator();
				menuNode.addChild(menuSep, 2 * itemIndex + 1);
			}
			if (itemsLength === 0) {
				if (menuNode === this.errorJobsMenu) {
					this.errorJobsSpace.style.display = "block";
				} else if (menuNode === this.activeJobsMenu) {
					this.activeJobsSpace.style.display = "block";
				} else if (menuNode === this.completedJobsMenu) {
					this.completedJobsSpace.style.display = "block";
				}
			} else {
				if (menuNode === this.errorJobsMenu) {
					this.errorJobsSpace.style.display = "none";
				} else if (menuNode === this.activeJobsMenu) {
					this.activeJobsSpace.style.display = "none";
				} else if (menuNode === this.completedJobsMenu) {
					this.completedJobsSpace.style.display = "none";
				}
			}
		},

		updateErrorJobs : function (items) {
			this.updatesMenuTab(items, this.errorJobsMenu);
		},

		updateActiveJobs : function (items) {
			this.updatesMenuTab(items, this.activeJobsMenu);
		},

		updateCompletedJobs : function (items) {
			this.updatesMenuTab(items, this.completedJobsMenu);
		},

		updateParentsCountNumber : function (items) {
			this._logger.finest("update tab's label with parents node's CATEGORY_COUNT property value",
					"updateParentsCountNumber");
			items = items || [];
			var itemIndex = 0;
			var itemsLength = items.length;
			for (itemIndex = 0; itemIndex < itemsLength; ++itemIndex) {
				var item = items[itemIndex];
				var itemId = item.getId();
				var count = item.getProperty("CATEGORY_COUNT").getValue();
				if (itemId === "Errors") {
					this._logger.finest("Error jobs count:" + count, "updateParentsCountNumber");
					this._setTotalErrorJobsCountAttr(count);

					//publish message to update top most "Jobs" label icon
					connect.publish(Constants.updateJobsLabel, count);
				}
				if (itemId === "Active") {
					this._logger.finest("Active jobs count:" + count, "updateParentsCountNumber");
					this._setTotalActiveJobsCountAttr(count);
				}
				if (itemId === "Completed") {
					this._logger.finest("Completed jobs count:" + count, "updateParentsCountNumber");
					this._setTotalCompletedJobsCountAttr(count);
				}

			}
		},

		_formatDateStr : function (dateTime) {
			var formatedStr;
			var dateStrArr;
			if (dateTime) {
				formatedStr = dojo.date.locale.format(dateTime, {
					formatLength : 'short',
					datePattern : 'MM-dd'
				});
			}
			//console.debug("shorten date is:" + formatedStr);
			return formatedStr;
		},

		_getTime : function (dateTime) {
			var timeStr;
			var dateStrArr;
			if (dateTime) {
				dateStrArr = dateTime.toString().split(" ");
				timeStr = dateStrArr[4].split(":").slice(0, 2).join(":");
			}
			return timeStr;
		},

		_setRecentErrorJobsCountAttr : function (jobsCount) {
			this._logger.finest("set recently error jobs", "_setRecentErrorJobsCountAttr");
			this.recentErrorJobsCount = jobsCount;
			var text = MRI.CompletedJobStatusSummaryTip;
			if (text) {
				this._logger.finest("innerHTML is:" + text, "_setRecentErrorJobsCountAttr");
				text = lang.replace(text, [
					this.recentErrorJobsCount,
					this.totalErrorJobsCount
				]);
				this._logger.finest("innerHTML after subsitution is:" + text, "_setRecentErrorJobsCountAttr");
				this.errorJobsSummary.innerHTML = text;
			} else {
				this._logger.finest("no innerHTML", "_setRecentErrorJobsCountAttr");
			}
		},

		_setRecentActiveJobsCountAttr : function (jobsCount) {
			this._logger.finest("set recently active jobs", "_setRecentActiveJobsCountAttr");
			this.recentActiveJobsCount = jobsCount;
			var text = MRI.CommonJobStatusSummaryTip;
			if (text) {
				this._logger.finest("innerHTML is:" + text, "_setRecentActiveJobsCountAttr");
				text = lang.replace(text, [
					this.recentActiveJobsCount,
					this.totalActiveJobsCount
				]);
				this._logger.finest("innerHTML after subsitution is:" + text, "_setRecentActiveJobsCountAttr");
				this.activeJobsSummary.innerHTML = text;
			} else {
				this._logger.finest("no innerHTML", "_setRecentActiveJobsCountAttr");
			}
		},

		_setRecentCompletedJobsCountAttr : function (jobsCount) {
			this._logger.finest("set recently completed jobs", "_setRecentCompletedJobsCountAttr");
			this.recentCompletedJobsCount = jobsCount;
			var text = MRI.CompletedJobStatusSummaryTip;
			if (text) {
				this._logger.finest("innerHTML is:" + text, "_setRecentCompletedJobsCountAttr");
				text = lang.replace(text, [
					this.recentCompletedJobsCount,
					this.totalCompletedJobsCount
				]);
				this._logger.finest("innerHTML after subsitution is:" + text, "_setRecentCompletedJobsCountAttr");
				this.completedJobsSummary.innerHTML = text;
			} else {
				this._logger.finest("no innerHTML", "_setRecentCompletedJobsCountAttr");

			}
		},

		_setTotalErrorJobsCountAttr : function (jobsCount) {
			this._logger.finest("set total error jobs", "_setTotalErrorJobsCountAttr");
			this.totalErrorJobsCount = jobsCount;
			if (this.errorJobs_pane.controlButton) {
				if (jobsCount > 0) {
					this.errorJobs_pane.controlButton.set("label", "<span class='status-msgs-icon-error'></span>"
							+ lang.replace(this.ErrorJobsTabTitle, [ this.totalErrorJobsCount
							]));
				} else {
					this.errorJobs_pane.controlButton.set("label", lang.replace(this.ErrorJobsTabTitle,
							[ this.totalErrorJobsCount
							]));
				}
			} else {
				this._logger.finest("no controlButton for the page", "_setTotalErrorJobsCountAttr");
			}
		},

		_setTotalCompletedJobsCountAttr : function (jobsCount) {
			this._logger.finest("set total completed jobs", "_setTotalCompletedJobsCountAttr");
			this.totalCompletedJobsCount = jobsCount;
			if (this.completedJobs_pane.controlButton) {
				this.completedJobs_pane.controlButton.set("label", lang.replace(this.CompletedJobsTabTitle,
						[ this.totalCompletedJobsCount
						]));
			} else {
				this._logger.finest("no controlButton for the page", "_setTotalCompletedJobsCountAttr");
			}
		},

		_setTotalActiveJobsCountAttr : function (jobsCount) {
			this._logger.finest("set total ative jobs", "_setTotalActiveJobsCountAttr");
			this.totalActiveJobsCount = jobsCount;
			if (this.activeJobs_pane.controlButton) {
				this.activeJobs_pane.controlButton.set("label", lang.replace(this.ActiveJobsTabTitle,
						[ this.totalActiveJobsCount
						]));
			} else {
				this._logger.finest("no controlButton for the page", "_setTotalActiveJobsCountAttr");
			}
		},

		clearDescendants : function () {
			this._logger.finest("clear descendants of job status pod dialog", "clearDescendants");
			if (this.menuItems) {
				array.forEach(this.menuItems, lang.hitch(this, function (menuItem) {
					if (menuItem) {
						this._logger.finest("menuItem label: " + menuItem.label, "clearDescendants");
						menuItem.destroy();
					}
				}));
				this.menuItems = [];
			}
			this.errorJobsMenu.destroyDescendants();
			this.activeJobsMenu.destroyDescendants();
			this.completedJobsMenu.destroyDescendants();

		}

	});
});

},
'ibm/usmi/widgets/TaskCategory':function(){
require({cache:{
'url:ibm/usmi/widgets/templates/TaskCategory.html':"<div tabindex=\"${tabIndex}\" class=\"taskCategory\" data-dojo-attach-point=\"mainNode\">\r\n<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n\t<div class=\"taskCategoryBar\">\r\n\t   <div class=\"taskCategoryTitle\" data-dojo-attach-point=\"titlePoint\" role=\"heading\" aria-level=2 ></div>\r\n\t</div>\r\n<!--\t<div class=\"taskCategory-shadow\"></div> -->\r\n\t<div data-dojo-attach-point=\"containerNode\"></div>\r\n</div>"}});
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/widgets/TaskCategory", ["dojo/_base/kernel",
        "dojo/_base/declare",
        "dijit/_WidgetBase",
        "dijit/_TemplatedMixin",
        "dojo/text!./templates/TaskCategory.html",
        "dijit/_KeyNavContainer",
        "dijit/_Contained",
        // NextGen imports
        "ibm/usmi/widgets/TaskLink"],
    function (dojo, declare, _WidgetBase, _TemplatedMixin, TaskCategoryTemplate, _KeyNavContainer, _Contained,
            TaskLink) {

    return declare([_WidgetBase, _TemplatedMixin, _KeyNavContainer, _Contained], {

        // Template info
        templateString: TaskCategoryTemplate,

        label: "",
        task: "",
        visible: true,

        startup: function() {
            this.inherited(arguments);

            // key nav container
            this.startupKeyNavChildren();
        },

        postCreate: function() {
            this.inherited(arguments);

            // key nav container
            this.connectKeyNavHandlers([dojo.keys.UP_ARROW],[dojo.keys.DOWN_ARROW]);
        },

        buildRendering: function() {
            this.inherited(arguments);
            this.titlePoint.innerHTML = this.label;
        },

        addLink: function(link) {
            if(link instanceof TaskLink){
                this.addChild(link);
            } else {
                this.addChild(new TaskLink(link));
            }
        },

        updateLinks: function(categoryUpdate) {
            if(!categoryUpdate) { return; }
            var _links = this.getChildren();
            dojo.forEach(_links, function(link) {
                link.update(categoryUpdate[link.link]);
            });
        },

        /*
         * Returns tpc.common.menupod.TaskLink widget.
         */
        getLink: function(/*String*/ link) {
            var subset = dojo.filter(this.getChildren(), function(/*Widget*/ child) {
                return child.isInstanceOf(TaskLink) && child.link === link;
            }, this);
            return (subset && subset[0]) || null;
        },

        //If none of the tasks links are visible, display the no task message.
        setNoTaskMessage: function() {

            if (this.getChildren().length ===
                dojo.query("a.hidden", dojo.byId(this.containerNode)).length) {
                //Each of the child tasks are hidden, display the no task message
                dojo.removeClass(this.noTasks, "hidden");
            } else {
                dojo.addClass(this.noTasks, "hidden");
            }
        },

        removeLink: function(/*String*/link) {
            var widget = this.getLink(link);
            if(widget) {
                this.removeChild(widget);
            }
        },

        _setVisibleAttr: function(visible) {
            this.visible = visible;
            this.mainNode.style.display = (visible) ? "" : "none";
        },

        _getVisibleAttr: function() {
            return this.visible;
        }

    });
});
},
'url:ibm/usmi/widgets/templates/TaskLink.html':"<a class=\"taskLink taskLinkAnchor\" data-dojo-attach-point=\"taskAnchor\" role=\"link\">\r\n<!--\r\n<a class=\"taskLink taskLinkAnchor\" data-dojo-attach-point=\"taskAnchor\" role=\"link\" href=\"#${link}\">\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n\t<div class=\"taskLink-iehack\">\r\n\t    <div class=\"taskLinkLabelContainer\">\r\n<!-- \t\t      <div class=\"taskLinkIcon sprite\" data-dojo-attach-point=\"iconPoint\"></div> -->\r\n\t\t      <img class=\"taskLinkIcon sprite\" data-dojo-attach-point=\"iconPoint\"></img>\r\n\t\t      <div class=\"taskLinkLabel\" data-dojo-attach-point=\"labelPoint\"></div>\r\n\t\t</div>\r\n\t\t<div class=\"taskLinkStatusIcon right sprite\" data-dojo-attach-point=\"statusIconPoint\"></div>\r\n\t</div>\t\r\n</a>",
'ibm/tivoli/tip/navwidget/table/dijit/NavModelTableSelection':function(){
// wrapped by build app
define("ibm/tivoli/tip/navwidget/table/dijit/NavModelTableSelection", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/tip/navwidget/dojo/NavWidgetPluggableViewSelection,ibm/tivoli/tip/navwidget/dojo/NavWidgetUtil,dojox/collections/SortedList"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.table.dijit.NavModelTableSelection");

dojo.require("ibm.tivoli.tip.navwidget.dojo.NavWidgetPluggableViewSelection");
dojo.require("ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil");
dojo.require("dojox.collections.SortedList");

dojo.declare(
    "ibm.tivoli.tip.navwidget.table.dijit.NavModelTableSelection",
    ibm.tivoli.tip.navwidget.dojo.NavWidgetPluggableViewSelection,
    {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
    	
    	setMode: function(mode){		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
    		// mode:
    		//		
    	
	    	//We know the pluggableView is a Item or Links Table View, so we can set the selection mode on the grid's selection object
    		this._pluggableView._table._grid.selection.setMode(mode);   // 49426  Gridx
	    	
			//Call superclass
    		this.inherited(arguments);
		},
    	
	 	setSelectedObjectIds: function(/*String[]*/ itemIds, /*String[]*/ linkIds, /*String[]*/ relationshipIds){		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			// itemIds:
			//		
			// linkIds:
			//		
			// relationshipIds:
			//			
			
	    	//Mark we're changing
			this._changing = true;
			
			//We know the pluggableView is a Item or Links Table View
			if (!this._pluggableView._handlingSelectionChangedCallback) { //Don't do anything if pluggable view is currently dealing with user-initiated selection
				//Clear selection
				var grid = this._pluggableView._table._grid;   // 49426  Gridx
				
				// 49426  this._pluggableView._grid.selection.clear();
				// NOTE: for gridx this would need to be grid.select.row.clear()
			    //  ...but since we have no way (yet) of looking up an item by
			    //  it's itemId in the gridx table's model to get the row...we can't support this now
				// (unless...looks like rowId may equal itemId...need to verify that and revisit this
			    console.log("ERROR: setSelectedObjectIds not supported yet for GRIDX based table");
			    grid.selection.clear();
				
				//Deal with items
				if (itemIds) {
					for (var i = 0; i < itemIds.length; i++) {
						var item = ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getItemByAbsoluteId(this._dataset, itemIds[i]);
						var idx = grid.getItemIndex(item);    // 49426  Gridx
						if (idx >= 0) {
							grid.selection.addToSelection(idx);  // 49426  Gridx
						}
					}
				}
				
				//Deal with links
				if (linkIds) {
					for (var i = 0; i < linkIds.length; i++) {
						var link = ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getLinkByAbsoluteId(this._dataset, linkIds[i]);
						var idx = grid.getItemIndex(link);  // 49426  Gridx
						if (idx >= 0) {
							grid.selection.addToSelection(idx);   // 49426  Gridx
						}
					}
				}
				
				//Deal with relationships
				if (relationshipIds) {
					for (var i = 0; i < relationshipIds.length; i++) {
						var relationship = ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getRelationshipByAbsoluteId(this._dataset, relationshipIds[i]);
						var idx = grid.getItemIndex(relationship);   // 49426  Gridx
						if (idx >= 0) {
							grid.selection.addToSelection(idx);   // 49426  Gridx
						}
					}
				}
			}
			
			//Call superclass
			this.inherited(arguments);
			
			//Mark we're done changing
			this._changing = false;
		},
		
	 	setSelectedObjects: function(/*ibm.tivoli.navmodel.Item[]*/ items, /*ibm.tivoli.navmodel.Link[]*/ links, /*ibm.tivoli.navmodel.Relationship[]*/ relationships, /*Integer[]*/rowIds){	// 49426  Gridx	
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			// items:
			// 		
			// links:
			//		
			// relationships:
			//		
			
			//Mark we're changing
			this._changing = true;
			
			//We know the pluggableView is a Item or Links Table View
			if (this._pluggableView._grid && !this._pluggableView._handlingSelectionChangedCallback) { //Don't do anything if pluggable view is currently dealing with user-initiated selection
				//Clear selection
				var grid = this._pluggableView._table._grid;  // 49426  Gridx
				// grid.select.row.clear();    remove for now...no longer needed?
				
				if (this._pluggableView.dataset) {
					//Deal with items
					// 49426  (next 5 lines)
					if (items && items.length > 0 && rowIds && rowIds.length > 0) {   
                        dojo.forEach(rowIds, function(rowId) {   
                           grid.select.row.selectById(rowId);
                        });
                    }
                    // need else case for calling in programmatically with only items
                    // but that will require lookup from item to row in Gridx
					
					//Deal with links
					// 49426  (next 5 lines)
					if (links && links.length > 0 && rowIds && rowIds.length > 0) {    
                        dojo.forEach(rowIds, function(rowId) {   
                           grid.select.row.selectById(rowId);
                        });
                    }
					
					//Deal with relationships
					// 49426  (next 5 lines)
					if (relationships && relationships.length > 0 && rowIds && rowIds.length > 0) {   
                        dojo.forEach(rowIds, function(rowId) {   
                           grid.select.row.selectById(rowId);
                        });
                    }
					
					grid.model.when();   // 49426  Gridx...Necessary?  Check with Gridx developers
				}
			}
			
    		//Call superclass
			this.inherited(arguments);
			
			//Mark we're done changing
			this._changing = false;
		},
		
		// 49426  Gridx
	    updateSelectedObjects: function(/*ibm.tivoli.navmodel.Item[]*/items, /*Integer[]*/rowIds) {  
	    	this._ignore = true;
	    	this.setSelectedObjects(items, null, null, rowIds);
	    	this._ignore = false;
	    }
		
    }
);
});

},
'url:dijit/templates/Dialog.html':"<div class=\"dijitDialog\" role=\"dialog\" aria-labelledby=\"${id}_title\">\n\t<div data-dojo-attach-point=\"titleBar\" class=\"dijitDialogTitleBar\">\n\t<span data-dojo-attach-point=\"titleNode\" class=\"dijitDialogTitle\" id=\"${id}_title\"></span>\n\t<span data-dojo-attach-point=\"closeButtonNode\" class=\"dijitDialogCloseIcon\" data-dojo-attach-event=\"ondijitclick: onCancel\" title=\"${buttonCancel}\" role=\"button\" tabIndex=\"-1\">\n\t\t<span data-dojo-attach-point=\"closeText\" class=\"closeText\" title=\"${buttonCancel}\">x</span>\n\t</span>\n\t</div>\n\t\t<div data-dojo-attach-point=\"containerNode\" class=\"dijitDialogPaneContent\"></div>\n</div>\n",
'gridx/modules/RowHeader':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/query",
	"dojo/_base/lang",
	"dojo/_base/sniff",
	"dojo/aspect",
	"dojo/dom-construct",
	"dojo/dom-class",
	"dojo/dom-style",
	"dojo/keys",
	"../core/_Module",
	"../core/util"
], function(declare, query, lang, sniff, aspect, domConstruct, domClass, domStyle, keys, _Module, util){

	return declare(/*===== "gridx.modules.RowHeader", =====*/_Module, {
		// summary:
		//		This modules provides a header before each row.
		// description:
		//		Row header can be used as a UI handler for row selection, especially when
		//		cell selection is turned on and selectRowTriggerOnCell is turned off.
		//		It can also be used as a place to hold the checkbox/radiobutton for IndirectSelect

		name: 'rowHeader',

		getAPIPath: function(){
			// tags:
			//		protected extension
			return {
				rowHeader: this
			};
		},

		constructor: function(){
			this.headerNode = domConstruct.create('div', {
				'class': 'gridxRowHeaderHeader',
				role: 'row',
				innerHTML: ['<table border="0" cellspacing="0" cellpadding="0" style="width: ', 
					this.arg('width'), 
					';"><tr><th class="gridxRowHeaderHeaderCell" role="rowheader" tabindex="-1"></th></tr></table>'
				].join('')
			});
			this.bodyNode = domConstruct.create('div', {
				'class': 'gridxRowHeaderBody'
			});
		},

		destroy: function(){
			// tags:
			//		protected extension
			this.inherited(arguments);
			this._b.remove();
			domConstruct.destroy(this.headerNode);
			domConstruct.destroy(this.bodyNode);
		},

		preload: function(){
			// tags:
			//		protected extension
			var t = this,
				rhhn = t.headerNode,
				rhbn = t.bodyNode,
				g = t.grid,
				body = g.body,
				w = t.arg('width');
			//register events
			g._initEvents(['RowHeaderHeader', 'RowHeaderCell'], g._eventNames);
			//modify header
			g.header.domNode.appendChild(rhhn);
			rhhn.style.width = w;
			t.headerCellNode = query('th', rhhn)[0];
			g._connectEvents(rhhn, '_onHeaderMouseEvent', t);
			//modify body
			g.mainNode.appendChild(rhbn);
			rhbn.style.width = w;
			g.hLayout.register(null, rhbn);
			t.batchConnect(
				[body, 'onRender', '_onRendered'],
				[body, 'onAfterRow', '_onAfterRow'],
				[body, 'onAfterCell', '_onAfterCell'],
				[body, 'onUnrender', '_onUnrender'],
				[g.bodyNode, 'onscroll', '_onScroll'],
				[g, 'onRowMouseOver', '_onRowMouseOver'],
				[g, 'onRowMouseOut', '_onRowMouseOver'],
				[g, '_onResizeEnd', '_onResize'],
				g.columnWidth && [g.columnWidth, 'onUpdate', '_onResize'],
				g.columnResizer && [g.columnResizer, 'onResize', '_onResize'],
				[g, 'onRowHeaderCellMouseOver', '_onCellMouseOver'],
				[g, 'onRowHeaderCellMouseOut', '_onCellMouseOver']);
			//TODO: need to organize this into connect/disconnect system
			t._b = aspect.before(body, 'renderRows', lang.hitch(t, t._onRenderRows), true);
			g._connectEvents(rhbn, '_onBodyMouseEvent', t);
			t._initFocus();
		},

		load: function(args, startup){
			var t = this,
				bn = t.bodyNode;
			startup.then(function(){
				var w = bn.offsetWidth || domStyle.get(bn, 'width');
				bn.style[t.grid.isLeftToRight() ? 'left' : 'right'] = -w + 'px';
				t.loaded.callback();
			});
		},

		//Public--------------------------------------------------------------------------

		// width: String
		//		The width (CSS value) of a row header.
		width: '20px',

		onMoveToRowHeaderCell: function(){
			// summary:
			//		Fired when focus is moved to a row header using keyboard.
			// tags:
			//		callback
		},

	/*=====
		// headerProvider: Function
		//		A functionn that returns an HTML string to fill the header cell of row headers.
		headerProvider: null,

		// cellProvider: Function
		//		A function that returns an HTML string to fill the body cells of row headers.
		cellProvider: null,
	=====*/

		//Private-------------------------------------------------------
		_onRenderRows: function(start, count, position){
			var nd = this.bodyNode;
			if(count > 0){
				var str = this._buildRows(start, count);
				if(position == 'top'){
					domConstruct.place(str, nd, 'first');
				}else if(position == 'bottom'){
					domConstruct.place(str, nd, 'last');
				}else{
					nd.innerHTML = str;
					nd.scrollTop = 0;
				}
			}else if(position != 'top' && position != 'bottom'){
				nd.innerHTML = '';
			}
		},

		_onAfterRow: function(row){
			var t = this,
				visualIndex = row.visualIndex(),
				n = query('[visualindex="' + visualIndex + '"].gridxRowHeaderRow', t.bodyNode)[0],
				bn = query('[visualindex="' + visualIndex + '"].gridxRow .gridxRowTable', t.grid.bodyNode)[0],
				nt = n.firstChild,
				cp = t.arg('cellProvider');
			n.setAttribute('rowid', row.id);
			n.setAttribute('rowindex', row.index());
			n.setAttribute('parentid', t.model.treePath(row.id).pop() || '');
			if(cp){
				nt.firstChild.firstChild.firstChild.innerHTML = cp.call(t, row);
			}
			t._syncRowHeight(nt, bn);
		},

		_onAfterCell: function(cell){
			//This is to ensure the rowHeader get correct height for editable cells
			var t = this,
				visualIndex = cell.row.visualIndex(),
				n = query('[visualindex="' + visualIndex + '"].gridxRowHeaderRow', t.bodyNode)[0],
				bn = query('[visualindex="' + visualIndex + '"].gridxRow .gridxRowTable', t.grid.bodyNode)[0];
			t._syncRowHeight(n.firstChild, bn);
		},

		_syncRowHeight: function(rowHeaderNode, bodyNode){
			//Check if the table is collasped.
			var t = this;
			if(t._isCollapse === undefined){
				var refNode = query('.gridxCell', t.grid.header.innerNode)[0];
				t._isCollapse = refNode && domStyle.get(refNode, 'borderCollapse') == 'collapse';
			}
			//Use setTimeout to ensure the row header height correct reflects the body row height.
			//FIXME: This is tricky and may not be working in some special cases.
			function getHeight(){
				return sniff('ie') || t._isCollapse ? bodyNode.offsetHeight + 'px' : domStyle.getComputedStyle(bodyNode).height;
			}
			rowHeaderNode.style.height = getHeight();
			setTimeout(function(){
				rowHeaderNode.style.height = getHeight();
			}, 0);
		},

		_onRendered: function(start, count){
			var t = this,
				hp = t.arg('headerProvider');
			if(hp){
				t.headerCellNode.innerHTML = hp();
			}
			t._onScroll();
		},

		_onUnrender: function(id){
			var nodes = id && query('[rowid="' + id + '"].gridxRowHeaderRow', this.bodyNode);
			if(nodes && nodes.length){
				//remove the last node instead of the first, because when refreshing, there'll be 2 nodes with same id.
				domConstruct.destroy(nodes[nodes.length - 1]);
			}
		},

		_onScroll: function(){
			this.bodyNode.scrollTop = this.grid.bodyNode.scrollTop;
		},

		_onResize: function(){
			for(var bn = this.grid.bodyNode.firstChild, n = this.bodyNode.firstChild;
				bn && n;
				bn = bn.nextSibling, n = n.nextSibling){
				n.firstChild.style.height = bn.firstChild.offsetHeight + 'px';
			}
		},

		_buildRows: function(start, count){
			var sb = [];
			for(var i = 0; i < count; ++i){
				sb.push('<div class="gridxRowHeaderRow" role="row" visualindex="', start + i,
					'"><table border="0" cellspacing="0" cellpadding="0" style="height: 24px;"><tr><td class="gridxRowHeaderCell" role="rowheader" tabindex="-1"></td></tr></table></div>');
			}
			return sb.join('');
		},

		//Events
		_onHeaderMouseEvent: function(eventName, e){
			var g = this.grid,
				evtCell = 'onRowHeaderHeader' + eventName,
				evtRow = 'onHeader' + eventName;
			if(g._isConnected(evtCell)){
				g[evtCell](e);
			}
			if(g._isConnected(evtRow)){
				g[evtRow](e);
			}
		},

		_onBodyMouseEvent: function(eventName, e){
			var g = this.grid,
				evtCell = 'onRowHeaderCell' + eventName,
				evtRow = 'onRow' + eventName,
				cellConnected = g._isConnected(evtCell),
				rowConnected = g._isConnected(evtRow);
			if(cellConnected || rowConnected){
				this._decorateBodyEvent(e);
				if(e.rowIndex >= 0){
					if(e.isRowHeader && cellConnected){
						g[evtCell](e);
					}
					if(rowConnected){
						g[evtRow](e);
					}
				}
			}
		},

		_decorateBodyEvent: function(e){
			var node = e.target || e.originalTarget;
			while(node && node != this.bodyNode){
				if(domClass.contains(node, 'gridxRowHeaderCell')){
					e.isRowHeader = true;
					e.rowHeaderCellNode = node;
				}else if(node.tagName.toLowerCase() === 'div' && domClass.contains(node, 'gridxRowHeaderRow')){
					e.rowId = node.getAttribute('rowid');
					e.parentId = node.getAttribute('parentid');
					e.rowIndex = parseInt(node.getAttribute('rowindex'), 10);
					e.visualIndex = parseInt(node.getAttribute('visualindex'), 10);
					return;
				}
				node = node.parentNode;
			}
		},

		_onRowMouseOver: function(e){
			var rowNode = query('> [rowid="' + e.rowId + '"].gridxRowHeaderRow', this.bodyNode)[0];
			if(rowNode){
				domClass.toggle(rowNode, "gridxRowOver", e.type.toLowerCase() == 'mouseover');
			}
		},

		_onCellMouseOver: function(e){
			var cellNode = query('> [rowid="' + e.rowId + '"].gridxRowHeaderRow .gridxRowHeaderCell', this.bodyNode)[0];
			if(cellNode){
				domClass.toggle(cellNode, "gridxRowHeaderCellOver", e.type.toLowerCase() == 'mouseover');
			}
		},

		//Focus--------------------------------------------------------
		_initFocus: function(){
			var t = this,
				focus = t.grid.focus;
			if(focus){
				focus.registerArea({
					name: 'rowHeader',
					priority: 0.9,
					focusNode: t.bodyNode,
					scope: t,
					doFocus: t._doFocus,
					onFocus: t._onFocus,
					doBlur: t._blur,
					onBlur: t._blur,
					connects: [
						t.connect(t.bodyNode, 'onkeydown', '_onKeyDown')
					]
				});
			}
		},

		_doFocus: function(evt){
			if(this._focusRow(this.grid.body._focusCellRow)){
				util.stopEvent(evt);
				return true;
			}
		},

		_onFocus: function(evt){
			var t = this,
				node = evt.target;
			while(node != t.bodyNode){
				if(domClass.contains(node, 'gridxRowHeaderRow')){
					var r = t.grid.body._focusCellRow = parseInt(node.getAttribute('visualindex'), 10);
					t._focusRow(r);
					return true;
				}
				node = node.parentNode;
			}
		},

		_focusRow: function(visIndex){
			var t = this,
				node = query('[visualindex="' + visIndex + '"] .gridxRowHeaderCell', t.bodyNode)[0];
			t._blur();
			node = node || t.bodyNode.firstChild;
			if(node){
				domClass.add(node, 'gridxCellFocus');
				node.focus();
			}
			return node;
		},

		_blur: function(){
			query('.gridxCellFocus', this.bodyNode).forEach(function(node){
				domClass.remove(node, 'gridxCellFocus');
			});
			return true;
		},

		_onKeyDown: function(evt){
			var t = this, g = t.grid;
			if(g.focus.currentArea() == 'rowHeader' && 
					evt.keyCode == keys.UP_ARROW || evt.keyCode == keys.DOWN_ARROW){
				util.stopEvent(evt);
				var step = evt.keyCode == keys.UP_ARROW ? -1 : 1,
					body = g.body,
					r = body._focusCellRow + step;
				body._focusCellRow = r = r < 0 ? 0 : (r >= body.visualCount ? body.visualCount - 1 : r);
				g.vScroller.scrollToRow(r).then(function(){
					t._focusRow(r);
					t.onMoveToRowHeaderCell(r, evt);
				});
			}
		}
	});
});

},
'dojox/grid/TreeSelection':function(){
define([
	"../main",
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/dom-attr",
	"dojo/query",
	"./DataSelection"
], function(dojox, declare, array, lang, domAttr, query, DataSelection){

return declare("dojox.grid.TreeSelection", DataSelection, {
	setMode: function(mode){
		this.selected = {};
		this.sorted_sel = [];
		this.sorted_ltos = {};
		this.sorted_stol = {};
		DataSelection.prototype.setMode.call(this, mode);
	},
	addToSelection: function(inItemOrIndex){
		if(this.mode == 'none'){ return; }
		var idx = null;
		if(typeof inItemOrIndex == "number" || typeof inItemOrIndex == "string"){
			idx = inItemOrIndex;
		}else{
			idx = this.grid.getItemIndex(inItemOrIndex);
		}
		if(this.selected[idx]){
			this.selectedIndex = idx;
		}else{
			if(this.onCanSelect(idx) !== false){
				this.selectedIndex = idx;
				var rowNodes = query("tr[dojoxTreeGridPath='" + idx + "']", this.grid.domNode);
				if(rowNodes.length){
					domAttr.set(rowNodes[0], "aria-selected", "true");
				}
				this._beginUpdate();
				this.selected[idx] = true;
				this._insertSortedSelection(idx);
				//this.grid.onSelected(idx);
				this.onSelected(idx);
				//this.onSetSelected(idx, true);
				this._endUpdate();
			}
		}
	},
	deselect: function(inItemOrIndex){
		if(this.mode == 'none'){ return; }
		var idx = null;
		if(typeof inItemOrIndex == "number" || typeof inItemOrIndex == "string"){
			idx = inItemOrIndex;
		}else{
			idx = this.grid.getItemIndex(inItemOrIndex);
		}
		if(this.selectedIndex == idx){
			this.selectedIndex = -1;
		}
		if(this.selected[idx]){
			if(this.onCanDeselect(idx) === false){
				return;
			}
			var rowNodes = query("tr[dojoxTreeGridPath='" + idx + "']", this.grid.domNode);
			if(rowNodes.length){
				domAttr.set(rowNodes[0], "aria-selected", "false");
			}
			this._beginUpdate();
			delete this.selected[idx];
			this._removeSortedSelection(idx);
			//this.grid.onDeselected(idx);
			this.onDeselected(idx);
			//this.onSetSelected(idx, false);
			this._endUpdate();
		}
	},
	getSelected: function(){
		var result = [];
		for(var i in this.selected){
			if(this.selected[i]){
				result.push(this.grid.getItem(i));
			}
		}
		return result;
	},
	getSelectedCount: function(){
		var c = 0;
		for(var i in this.selected){
			if(this.selected[i]){
				c++;
			}
		}
		return c;
	},
	_bsearch: function(v){
		var o = this.sorted_sel;
		var h = o.length - 1, l = 0, m;
		while(l<=h){
			var cmp = this._comparePaths(o[m = (l + h) >> 1], v);
			if(cmp < 0){ l = m + 1; continue; }
			if(cmp > 0){ h = m - 1; continue; }
			return m;
		}
		return cmp < 0 ? m - cmp : m;
	},
	_comparePaths: function(a, b){
		for(var i=0, l=(a.length < b.length ? a.length : b.length); i<l; i++){
			if(a[i]<b[i]){ return -1; }
			if(a[i]>b[i]){ return 1; }
		}
		if(a.length<b.length){ return -1; }
		if(a.length>b.length){ return 1; }
		return 0;
	},
	_insertSortedSelection: function(index){
		index = String(index);
		var s = this.sorted_sel;
		var sl = this.sorted_ltos;
		var ss = this.sorted_stol;

		var lpath = index.split('/');
		lpath = array.map(lpath, function(item){ return parseInt(item, 10); });
		sl[lpath] = index;
		ss[index] = lpath;

		if(s.length === 0){
			s.push(lpath);
			return;
		}
		if(s.length==1){
			var cmp = this._comparePaths(s[0], lpath);
			if(cmp==1){ s.unshift(lpath); }
			else{ s.push(lpath); }
			return;
		}

		var idx = this._bsearch(lpath);
		this.sorted_sel.splice(idx, 0, lpath);
	},
	_removeSortedSelection: function(index){
		index = String(index);
		var s = this.sorted_sel;
		var sl = this.sorted_ltos;
		var ss = this.sorted_stol;

		if(s.length === 0){
			return;
		}

		var lpath = ss[index];
		if(!lpath){ return; }

		var idx = this._bsearch(lpath);
		if(idx > -1){
			delete sl[lpath];
			delete ss[index];
			s.splice(idx, 1);
		}
	},
	getFirstSelected: function(){
		if(!this.sorted_sel.length||this.mode == 'none'){ return -1; }
		var fpath = this.sorted_sel[0];
		if(!fpath){
			return -1;
		}
		fpath = this.sorted_ltos[fpath];
		if(!fpath){
			return -1;
		}
		return fpath;
	},
	getNextSelected: function(inPrev){
		if(!this.sorted_sel.length||this.mode == 'none'){ return -1; }
		inPrev = String(inPrev);
		var prevPath = this.sorted_stol[inPrev];
		if(!prevPath){ return -1; }

		var idx = this._bsearch(prevPath);
		var lpath = this.sorted_sel[idx+1];
		if(!lpath){
			return -1;
		}
		return this.sorted_ltos[lpath];
	},
	_range: function(inFrom, inTo, func){
		if(!lang.isString(inFrom) && inFrom < 0){
			inFrom = inTo;
		}
		var cells = this.grid.layout.cells,
			store = this.grid.store,
			grid = this.grid;
		inFrom = new dojox.grid.TreePath(String(inFrom), grid);
		inTo = new dojox.grid.TreePath(String(inTo), grid);

		if(inFrom.compare(inTo) > 0){
			var tmp = inFrom;
			inFrom = inTo;
			inTo = tmp;
		}

		var inFromStr = inFrom._str, inToStr = inTo._str;

		// select/deselect the first
		func(inFromStr);

		var p = inFrom;
		while((p = p.next())){
			if(p._str == inToStr){
				break;
			}
			func(p._str);
		}

		// select/deselect the last
		func(inToStr);
	}
});
});
},
'gridx/modules/ColumnResizer':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/sniff",
	"dojo/_base/window",
	"dojo/_base/event",
	"dojo/dom",
	"dojo/dom-style",
	"dojo/dom-class",
	"dojo/dom-construct",
	"dojo/dom-geometry",
	"dojo/keys",
	"dojo/query",
	"../core/_Module"
], function(declare, sniff, win, event, dom, domStyle, domClass, domConstruct, domGeometry, keys, query, _Module){

	var removeClass = domClass.remove;

	function getCell(e){
		var node = e.target;
		if(node){
			if(/table/i.test(node.tagName)){
				var m = e.offsetX || e.layerX || 0,
					i = 0,
					cells = node.rows[0].cells;
				while(m > 0 && cells[i]){
					m -= cells[i].offsetWidth;
					i++;
				}
				return cells[i] || null;
			}
			while(node && node.tagName){
				if(node.tagName.toLowerCase() == 'th'){
					return node;
				}
				node = node.parentNode;
			}
		}
		return null;
	}

	return declare(/*===== "gridx.modules.ColumnResizer", =====*/_Module, {
		// summary:
		//		Column Resizer machinery.
		// description:
		//		This module provides a way to resize column width. 
		
		// name: [readonly] String
		//		module name
		name: 'columnResizer',

//		required: ['hScroller'],

		// minWidth: Integer
		//		min column width in px
		minWidth: 20,

		detectWidth: 5,

		load: function(args){
			var t = this,
				g = t.grid;
			t.batchConnect(
				[g.header.innerNode, 'mousemove', '_mousemove'],
				[g, 'onHeaderMouseOut', '_mouseout'],
				[g, 'onHeaderMouseDown', '_mousedown', t, t.name],
				[g, 'onHeaderKeyDown', '_keydown'],
				[win.doc, 'mousemove', '_docMousemove'],
				[win.doc, 'onmouseup', '_mouseup']
			);
			t.loaded.callback();
		},

		getAPIPath: function(){
			// summary:
			//		Module reference shortcut so that we can 
			//		quickly locate this module by grid.columnResizer
			return {
				columnResizer: this
			};
		},

		// columnMixin: Object
		//		A map of functions to be mixed into grid column object, so that we can use select api on column object directly
		//		- grid.column(1).setWidth(300);
		columnMixin: {
			setWidth: function(width){
				this.grid.columnResizer.setWidth(this.id, width);
			}
		},

		//Public---------------------------------------------------------------------
		setWidth: function(/*String | Integer*/colId, /*Integer*/width){
			// summary:
			//		Set width of the target column
			var t = this,
				g = t.grid, i,
				cols = g._columns,
				minWidth = t.arg('minWidth'),
				oldWidth;
			width = parseInt(width, 10);
			if(width < minWidth){
				width = minWidth;
			}
			g._columnsById[colId].width = width + 'px';
			for(i = 0; i < cols.length; ++i){
				cols[i].declaredWidth = cols[i].width;
			}
			query('[colid="' + colId + '"]', g.domNode).forEach(function(cell){
				if(!oldWidth){
					oldWidth = domStyle.get(cell, 'width');
				}
				cell.style.width = width + 'px';
			});
			g.body.onRender();
			g.vLayout.reLayout();
			
			t.onResize(colId, width, oldWidth);
		},

		//Event--------------------------------------------------------------
		onResize: function(/* colId, newWidth, oldWidth */){},

		//Private-----------------------------------------------------------
		_mousemove: function(e){
			var t = this,
				cell = getCell(e),
				flags = t.grid._eventFlags;
			if(cell){
				if(t._resizing){
					removeClass(cell, 'gridxHeaderCellOver');
				}
				if(t._resizing || !cell || t._ismousedown){
					return;
				}
				var ready = t._readyToResize = t._isInResizeRange(e);
				//Forbid anything else to happen when we are resizing a column!
				flags.onHeaderMouseDown = flags.onHeaderCellMouseDown = ready ? t.name : undefined;

				domClass.toggle(win.body(), 'gridxColumnResizing', ready);
				if(ready){
					removeClass(cell, 'gridxHeaderCellOver');
				}
			}
		},

		_docMousemove: function(e){
			if(this._resizing){
				this._updateResizerPosition(e);
			}
		},

		_mouseout: function(e){
			if(!this._resizing){
				this._readyToResize = 0;	//0 as false
				removeClass(win.body(), 'gridxColumnResizing');
			}
		},
		
		_keydown: function(evt){
			//support keyboard to resize a column
			if((evt.keyCode == keys.LEFT_ARROW || evt.keyCode == keys.RIGHT_ARROW) && evt.ctrlKey && evt.shiftKey){
				var colId = evt.columnId,
					g = this.grid,
					dir = g.isLeftToRight() ? 1 : -1,
					step = dir * 2;
				query('[colid="' + colId + '"]', g.header.domNode).forEach(function(cell){
					var width = domStyle.get(cell, 'width');
					if(evt.keyCode == keys.LEFT_ARROW){width -= step;}
					else {width += step;}
					this.setWidth(colId, width);
					event.stop(evt);
				}, this);
			}
		},
		
		_updateResizerPosition: function(e){
			var t = this,
				delta = e.pageX - t._startX,
				cell = t._targetCell,
				g = t.grid,
				hs = g.hScroller,
				h = 0,
				n,
				left = e.pageX - t._gridX,
				minWidth = t.arg('minWidth'),
				ltr = this.grid.isLeftToRight();
			if(!ltr){
				delta = -delta;
			}
			if(cell.offsetWidth + delta < minWidth){
				if(ltr){
					left = t._startX - t._gridX - cell.offsetWidth + minWidth;
				}else{
					left = t._startX - t._gridX + (cell.offsetWidth - minWidth);
				}
			}
			n = hs && hs.container.offsetHeight ? hs.container : g.bodyNode;
			h = n.parentNode.offsetTop + n.offsetHeight - g.header.domNode.offsetTop;
			domStyle.set(t._resizer, {
				top: g.header.domNode.offsetTop + 'px',
				left: left + 'px',
				height: h + 'px'
			});
		},

		_showResizer: function(e){
			var t = this;
			if(!t._resizer){
				t._resizer = domConstruct.create('div', {
					className: 'gridxColumnResizer'}, 
					t.grid.domNode, 'last');
				t.connect(t._resizer, 'mouseup', '_mouseup');
			}
			t._resizer.style.display = 'block';
			t._updateResizerPosition(e);
		},

		_hideResizer: function(){
			this._resizer.style.display = 'none';
		},

		_mousedown: function(e){
			//begin resize
			var t = this;
			if(!t._readyToResize){
				t._ismousedown = 1;	//1 as true
				return;
			}
			dom.setSelectable(t.grid.domNode, false);
			win.doc.onselectstart = function(){
				return false;
			};
			t._resizing = 1;	//1 as true
			t._startX = e.pageX;
			t._gridX = domGeometry.position(t.grid.domNode).x;
			t._showResizer(e);
		},

		_mouseup: function(e){
			//end resize
			var t = this;
			t._ismousedown = 0;	//0 as false
			if(t._resizing){
				t._resizing = 0;	//0 as false
				t._readyToResize = 0;	//0 as false
				removeClass(win.body(), 'gridxColumnResizing');
				dom.setSelectable(t.grid.domNode, true);
				win.doc.onselectstart = null;
				
				var cell = t._targetCell,
					delta = e.pageX - t._startX;
				if(!t.grid.isLeftToRight()){
					delta = -delta;
				}
				var	w = (sniff('webkit') ? cell.offsetWidth : domStyle.get(cell, 'width')) + delta,
					minWidth = t.arg('minWidth');
				if(w < minWidth){
					w = minWidth;
				}
				t.setWidth(cell.getAttribute('colid'), w);
				t._hideResizer();
				
			}
		},
		
		_isInResizeRange: function(e){
			var t = this,
				cell = getCell(e),
				x = t._getCellX(e),
				detectWidth = t.arg('detectWidth'),
				ltr = t.grid.isLeftToRight();
			if(x < detectWidth){
				//If !t._targetCell, the first cell is not able to be resize
				if(ltr){
					return !!(t._targetCell = cell.previousSibling);
				}else{
					t._targetCell = cell;
					return 1;
				}
			}else if(x > cell.offsetWidth - detectWidth && x <= cell.offsetWidth){
				if(ltr){
					t._targetCell = cell;
					return 1;	//1 as true
				}else{
					return !!(t._targetCell = cell.previousSibling);
				}
			}
			return 0;	//0 as false
		},

		_getCellX: function(e){
			var target = e.target,
				cell = getCell(e);
			if(!cell){
				return 100000;
			}
			
			if(/table/i.test(target.tagName)){
				return 0;
			}
			var lx = e.offsetX;
			if(lx == undefined){
				lx = e.layerX;
			}
			if(!/th/i.test(target.tagName)){
				lx += target.offsetLeft;
			}
			//Firefox seems have problem to get offsetX for TH
			if(sniff('ff') && /th/i.test(target.tagName)){
				var ltr = this.grid.isLeftToRight();
				var scrollLeft = -parseInt(domStyle.get(cell.parentNode.parentNode.parentNode, ltr ? 'marginLeft' : 'marginRight'));
				if(!ltr){
					scrollLeft = this.grid.header.domNode.firstChild.scrollWidth - scrollLeft - this.grid.header.innerNode.offsetWidth;
				}
				var d = lx - (cell.offsetLeft - scrollLeft);
				if(d >= 0){
					lx = d;
				}
				if(lx >= cell.offsetWidth)lx = 0;
			}
			return lx;
		}
	});
});

},
'ibm/tivoli/navmodel/Relationship':function(){
// wrapped by build app
define("ibm/tivoli/navmodel/Relationship", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/Dataset,ibm/tivoli/navmodel/Property,ibm/tivoli/navmodel/Status,ibm/tivoli/navmodel/LinkStyle,ibm/tivoli/navmodel/Exception,ibm/tivoli/navmodel/Utils,ibm/tivoli/navmodel/Trace"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Relationship"); 

// includes
dojo.require("ibm.tivoli.navmodel.Dataset");
dojo.require("ibm.tivoli.navmodel.Property");
dojo.require("ibm.tivoli.navmodel.Status");
dojo.require("ibm.tivoli.navmodel.LinkStyle");
dojo.require("ibm.tivoli.navmodel.Exception");
dojo.require("ibm.tivoli.navmodel.Utils");
dojo.require("ibm.tivoli.navmodel.Trace");

ibm.tivoli.navmodel.Relationship._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.Relationship");

////////////////////////////////////
// Relationship
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Relationship", null, {
	// private members       
	_relationship: null,
	_dataset: null,
	_props: null,
	_propsById: null,
	_nullPropIds: null,
	_allProps: false,
	_allMetadata: false,
	_status: null,
	_style: null,
	_defaultTask: null,
	_expiration: null,
	_trace: ibm.tivoli.navmodel.Relationship._trace,

	/*=====
	constructor_kwArgs: function(link, dataset, propIds, allProperties, allMetadata, expiration) {
		// link: Object
		//		json link object
		// dataset: Object
		//		ibm.tivoli.navmodel.Dataset that owns this link
		// propIds: String[]
		//		optional array indicating the property Id's that were requested with 
		//		the item; so it can be determined if any properties are null
		// allProperties: Boolean
		//		optional: indicates if the link already has "all" properties
		// allMetadata: Boolean
		//		optional: indicates if link properties already have metadata
		// expiration: Long
		//		optional: expiration time;  will use (current time + interval) if not supplied
	},
	=====*/
	
	 constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs Relationship object.
		this._relationship = kwArgs.relationship;
		if (this._relationship) {
			// optimization of json on the server can leave out the "id"
			// but we know the default format
			if (!this._relationship.id) {
				this._relationship.id = this._relationship.sourceItem + "_" + this._relationship.type + "_" + this._relationship.targetItem;
			}	
		}
		else {
			this._relationship = {};
		}
		
		this._dataset = kwArgs.dataset;		
		this._props = [];
		this._propsById = {};
		this._nullPropIds = {};
		this._allProps = kwArgs.allProperties;		
		this._allMetadata = kwArgs.allMetadata;
		
		if (this._relationship.properties) {
			var propsLength = this._relationship.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var prop = new ibm.tivoli.navmodel.Property({property: this._relationship.properties[propIndex], dataset: this._dataset, objectType: "relationship"});
				this._props[propIndex] = prop;
				this._propsById[prop.getId()] = prop;
			}
			
			// see if we should mark any properties as null because they were not returned
			if (kwArgs.propIds) {
				propsLength = kwArgs.propIds.length;
				for (propIndex = 0;  propIndex < propsLength;  ++propIndex) {
					var propId = kwArgs.propIds[propIndex];				
					if (!this._propsById[propId]) {
						this._nullPropIds[propId] = true;
					}
				}
			}
		}
		
		// expiration was supplied
		if (kwArgs.expiration) {
			this._expiration = kwArgs.expiration;
		}
		// reset to current time + interval
		else {
			this.resetExpiration();
		}
	},

	/////////////////////////////
	// public methods
	/////////////////////////////

	mixin: function(relationship) {		
		// summary:
		//		Mixin properties and status from this Relationship object
		if (relationship && relationship !== this) {
			this.removeAttribute("style");
			this._style = null;
			
			this.mixinAttributes(relationship.getAttributes());
			this.mixinProperties(relationship._props, relationship._nullPropIds);
			this._status = relationship.getStatuses({widgetDef: null});
			this._defaultTask = relationship.getDefaultTask();
			this._allProps = (this._allProps || relationship._allProps);
			this._allMetadata = (relationship._allProps && relationship._allMetadata);			
		}
	},

	getDataset: function() {		
		// summary:
		//		Returns the dataset that owns the relationship
		return this._dataset;
	},	

	getObjectType: function() {		
		// summary:
		//		Indicate that this object is a "relationship"
		return "relationship";
	},

	getAttribute: function(key) {		
		// summary:
		//		Returns general attribute for requested key
		return this._relationship[key];
	},

	setAttribute: function(key, value) {		
		// summary:
		//		Set general attribute for requested key
		this._relationship[key] = value;
	},
	
	removeAttribute: function(key) {
		// summary:
		//		Remove general attribute for requested key
		delete this._relationship[key];
	},
	
	getAttributes: function() {		
		// summary:
		//		Get all general attributes for the relationship
		var relationship = dojo.clone(this._relationship);
		
		// make sure "properties" are current as we may have received events
		if (this._props && this._props.length > 0) {
			relationship.properties = [];
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				relationship.properties[propIndex] = this._props[propIndex].getAttributes();
			}
		}
		
		return relationship;
	},

	mixinAttributes: function(attr) {		
		// summary:
		//		Mixin general attributes for the relationship
		dojo.mixin(this._relationship, attr);		
	},	

	getUri: function() {
		if (!this._relationship.uri) {
			this._relationship.uri = this._dataset.getAttribute("uri") + "/relationships/" + encodeURIComponent(this.getId());
		}
	
		return this._dataset.getArg("baseUrl") + this._relationship.uri;
	},
	
	getId: function() {		
		// summary:
		//		Returns relationship id
		return this._relationship.id;
	},

	 setId: function(id) {		
		// summary:
		//		Set the relationship id
		this._relationship.id = id;
	},

	getType: function() {		
		// summary:
		//		Returns the type of relationship
		return this._relationship.type;
	},

	setType: function(type) {		
		// summary:
		//		Set the relationship type
		this._relationship.type = type;
	},

	 getLabel: function() {		
		// summary:
		//		Returns globalized label for the relationship
		return this._relationship.label;
	},

	 setLabel: function(label) {		
		// summary:
		//		 Set the globalized label for the relationship
		this._relationship.label = label;
	},

	getTooltip: function() {		
		// summary:
		//		Returns globalized tooltip for the relationship
		return this._relationship.tooltip;
	},

	setTooltip: function(tooltip) {		
		// summary:
		//		Set the globalized tooltip for the relationship
		this._relationship.tooltip = tooltip;
	},

	getSourceItem: function() {
		// summary:
		//		Returns the sourceItem involved in the relationship	
		var item = this._dataset.getItem({itemId: this.getSourceItemId()});	
		return item;
	},	
	
	getSourceItemId: function() {		
		// summary:
		//		Returns source item id
		return this._relationship.sourceItem;
	},

	setSourceItemId: function(sourceItem) {		
		// summary:
		//		Set source item id
		this._relationship.sourceItem = sourceItem;
	},

	getTargetItem: function() {
		// summary:
		//		Returns the targetItem involved in the relationship	
		var item = this._dataset.getItem({itemId: this.getTargetItemId()});	
		return item;
	},	
	
	getTargetItemId: function() {		
		// summary:
		//		Returns target item id
		return this._relationship.targetItem;
	},

	setTargetItemId: function(targetItem) {		
		// summary:
		//		Set target item id
		this._relationship.targetItem = targetItem;
	},
	
	getLink: function() {
		return this._dataset.getLink(this.getLinkId());
	},
	
	getLinkId: function() {		
		// summary:
		//		Returns parent link id
		return (this._relationship.linkId !== undefined ? this._relationship.linkId : "");
	},

	setLinkId: function(linkId) {		
		// summary:
		//		Set parent link id
		this._relationship.linkId = linkId;
	},
	
	/*=====
	getStatuses_kwArgs: function(widgetDef) {
		// widgetDef: Object
		//		widget definition can override attributes
	},
	=====*/
	
	getStatuses: function(/*getStatuses_kwArgs*/ kwArgs) {		
		// summary:
		//		Returns ibm.tivoli.navmodel.Status[] objects indicating the relationship status values
		var args = {
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		// if expired mixin from dataset cache
		this._validateExpiration();
		
		if (!this._status && this._relationship.status) {
			this._status = [];
			var statusLength = this._relationship.status.length;
			for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
				if (this._relationship.status.label && this._relationship.status.image) {
					this._status.push(new ibm.tivoli.navmodel.Status({status: this._relationship.status[statusIndex], version: this._version()}));				
				}				
				else {
					var statusArgs = {
						type: "relationship",
						statusId: this._relationship.status[statusIndex].id,
						statusValue: this._relationship.status[statusIndex].value				
					};
				
					var statusObj = this._dataset.getStatus(statusArgs);
					if (statusObj) {				
						this._status.push(statusObj);
					}
					else {
						this._trace.error("Could not locate status with id=" + statusArgs.statusId + " value=" + statusArgs.statusValue, "getStatuses");
					}
				}
			}			
		}
		
		// merge widget definition status
		var status = this._status;
		if (status && args.widgetDef) {
			status = this._status.slice(0);
			var statusLength2 = this._status.length;
			for (var statusIndex2 = 0;  statusIndex2 < statusLength2;  ++statusIndex2) {
				status[statusIndex2] = this._status[statusIndex2].clone(args.widgetDef, "relationship");
			}					
		}
		
		return status;		
	},

	setStatuses: function(statuses) {		
		// summary:
		//		Set ibm.tivoli.navmodel.Status[] objects indicating the relationship status values
		this._status = statuses;
	},

	supportsProperties: function() {		
		// summary:
		//		Indicates if "Properties" is supported for the relationship
		return (this._relationship.supportsProperties !== undefined ? this._relationship.supportsProperties : true);
	},

	setSupportsProperties: function(supportsProperties) {		
		// summary:
		//		Set whether relationship  supports "Properties"
		this._relationship.supportsProperties = supportsProperties;
	},

	hasCachedMetadata: function() {		
		// summary:
		//		Returns true if relationship has all properties and associated metadata cached
		return this._allMetadata;
	},

	hasAllCachedProperties: function() {		
		// summary:
		//		Returns true if relationship has all properties cached
		return this._allProps;
	},

	hasCachedProperty: function(/*String*/ propId) {
		// summary:
		//		Returns true if we have the requested property in the cache
		// propId
		//		property id
		return (this._propsById[propId] ? true : false);
	},
	
	hasCachedProperties: function(/*String[]*/ properties) {		
		// summary:
		//		Returns true if we have all the requested properties in the cache
		// properties:
		//		array of property ids
		if (properties) {
			var propsLength = properties.length;
			
			// asking if we have "all" properties
			if (propsLength === 1 && properties[0] === "all") {
				return this._allProps;
			}
			
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				if (!this._propsById[properties[propIndex]]) {
					return false;
				}
			}
		}
		
		return true;
	},

	getCachedPropertyIds: function(expandAll) {		
		// summary:
		//		Returns array of property Ids currently cached in the item
		var propIds = [];
		
		// have all the properties
		if (!expandAll && this._allProps) {
			propIds.push("all");
		}
		// get individual property ids	
		else if (this._props) {
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				propIds.push(this._props[propIndex].getId());
			}
		}
		return propIds;
	},
	
	/*=====
	properties_onComplete: function(properties, kwArgs){},
	=====*/
	
	/*=====
	_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	getProperties_kwArgs: function(onComplete, onError, properties, metadata, sync) {
		// onComplete: properties_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		property ids to fetch or "all"
		// metadata: Boolean
		//		indicates whether metadata is required
		// sync: Boolean
		//		indicates if call is synchronous or asynchronous
	},
	=====*/

	getProperties: function(/*getProperties_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously returns properties for the relationship
		var args = {                
			onComplete: function(properties, args){
			},
			onError: function(error, exception, ioArgs){				
			},                
			properties: ["all"],
			metadata: false,
			sync: false
		};   
		dojo.mixin(args, kwArgs);         
		
		// if expired mixin from dataset cache
		this._validateExpiration();
		
		// fetch all properties
		if (args.properties.length === 1 && args.properties[0] === "all") {
			if (this._allProps && (!args.metadata || this._allMetadata)) {				
				if (args.sync) {
					args.onComplete(this._props, args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, this._props, args), 0);
				}		
			}
			else {
				this._fetchProperties(args);
			}
		}		
		// fetch selective properties
		else {
			// look up properties in cache
			var props = [];
			var propsLength = args.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var propId = args.properties[propIndex];
				var prop = this._propsById[propId];				
				if (prop && (!args.metadata || prop.hasMetadata())) {
					props[propIndex] = prop;					
				}
				else {
					break;
				}
			}
			
			// if we found all properties in cache then return them
			if (props.length === propsLength) {
				if (args.sync) {
					args.onComplete(props, args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, props, args), 0);
				}
			}
			// otherwise we need to fetch
			else {
				this._fetchProperties(args);
			}
		}		
	},

	setProperties: function(properties) {		
		// summary:
		//		Set the ibm.tivoli.navmodel.Property[] objects for the relationship
		//		
		//		Note: client side only operation;  no request to rest service
		this._propsById = {};
		this._nullPropIds = {};
		this._props = [];
		
		if (properties) {
			var propsLength = properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var prop = properties[propIndex];
				this._propsById[prop.getId()] = prop;
			}
		}		
	},

	mixinProperties: function(properties, nullPropIds) {		
		// summary:
		//		Mixin the ibm.tivoli.navmodel.Property[] objects for the relationship
		//		
		//		Note: client side only operation;  no request to rest service
		
		// mixin property objects
		if (properties) {
			var propsLength = properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				this.mixinProperty(properties[propIndex]);
			}
		}
		
		// mixin any propIds that were null
		if (nullPropIds) {
			for (var propId in nullPropIds) {
				if (nullPropIds.hasOwnProperty(propId)) {
					this._nullPropIds[propId] = true;
				}
			}
		}
	},

	 getProperty: function(/*String*/ propId) {		
		// summary:
		//		Synchronously returns requested property
		// propId:
		//		property id to return
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		
		// if expired mixin from dataset cache
		this._validateExpiration();
		
		// check prop cache
		var prop = this._propsById[propId];
			
		if (!prop) {
			// we have asked for this property before and it wasn't found
			if (this._nullPropIds[propId]) {
				return null;
			}
			
			// check default properties on relationship
			if (this._relationship.properties) {
				var propsLength = this._relationship.properties.length;
				for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
					var rawProp = this._relationship.properties[propIndex];
					if (rawProp.id === propId) {
						prop = new ibm.tivoli.navmodel.Property({property: rawProp, dataset: this._dataset, objectType: "relationship"});
						
						// cache property
						this._propsById[propId] = prop;
						break;
					}
				}				
			}
		}	
		
		// fetch it
		if (!prop && !this._allProps) {
			prop = this._fetchProperty(propId);
			
			// cache property
			if (prop) {
				this._propsById[propId] = prop;
			}
		}		
		
		return prop;
	},

	setProperty: function(property) {		
		// summary:
		//		Set ibm.tivoli.navmodel.Property for the relationship
		//		
		//		Note: client side only operation;  no request to rest service
		var found = this._propsById[property.getId()];		
		if (found) {
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				if (this._props[propIndex].getId() === property.getId()) {
					this._props[propIndex] = property;
					break;
				}
			}
		}
		else {
			this._props.push(property);
		}		
		
		this._propsById[property.getId()] = property;
	},
	
	mixinProperty: function(property) {		
		// summary:
		//		Mixin ibm.tivoli.navmodel.Property for the relationship
		//		
		//		Note: client side only operation;  no request to rest service
		var prop = this._propsById[property.getId()];
		if (prop) {
			prop.mixin(property);
		}
		else {
			this._props.push(property);
			this._propsById[property.getId()] = property;
		}	
	},
	
	/*=====
	_onComplete: function(kwArgs){},
	=====*/
	
	/*=====
	editProperties_kwArgs: function(onComplete, onError, properties) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		array of property ids to set
	},
	=====*/
	
	editProperties: function(/*editProperties_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously modify properties for the relationship
		var args = {                
			onComplete: function(properties, args){
			},
			onError: function(error, exception, ioArgs){				
			},                
			properties: null
		};
		dojo.mixin(args, kwArgs);
		
		if (args.properties && args.properties.length > 0) {
			var relationship = dojo.clone(this._relationship);
			relationship.properties = [];
			
			var propsLength = args.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				relationship.properties[propIndex] = args.properties[propIndex].getAttributes();
			}
			
			var json = dojo.toJson(relationship, false);
			
			dojo.xhrPost ({
				url: ibm.tivoli.navmodel.Utils.getParamUrl(this.getUri(), this._dataset),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version()),
					'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "relationshipView"),
					'Content-Length': json.length
				},
				handleAs: "json",
				postData: json,
				sync: false,			
				load: dojo.hitch(this, function(result, ioArgs) {
					args.onComplete(args);						
				}),
				error: dojo.hitch(function(error, ioArgs) {
					exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "setProperties", this._trace, this._version());				               
				})
			});	
		}
		// nothing to set
		else {
			setTimeout(dojo.hitch(this, args.onComplete, args), 0);
		}		
	},
	
	 editProperty: function(/*String*/ property) {		
		// summary:
		//		Synchronously modify property for the relationship
		// propId:
		//		property id to return
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var exception = null;
		
		if (property) {
			var relationship = dojo.clone(this._relationship);
			relationship.properties = [property.getAttributes()];
			
			var json = dojo.toJson(relationship, false);
			
			dojo.xhrPost ({
				url: ibm.tivoli.navmodel.Utils.getParamUrl(this.getUri(), this._dataset),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version()),
					'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "relationshipView"),
					'Content-Length': json.length
				},
				handleAs: "json",
				postData: json,
				sync: true,			
				load: dojo.hitch(this, function(result, ioArgs) {						
				}),
				error: dojo.hitch(this, function(error, ioArgs) {
					exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "setProperty", this._trace, this._version());									              
				})
			});				
		}
		
		if (exception) {
			throw exception;
		}
	},

	hasOwnRelationshipStyle: function() {		
		// summary:
		//		Returns true if the relationship has a LinkStyle that overrides the default
		//		relationship style for the relationship type  (i.e. different than the one
		//		retrieved from the dataset)
		return (this._relationship.style ? true: false);
	},

	/*=====
	getRelationshipStyle_kwArgs: function(widgetDef) {
		// widgetDef: Object
		//		widget definition can override attributes
	},
	=====*/
	
	getRelationshipStyle: function(/*getRelationshipStyle_kwArgs*/ kwArgs) {		
		// summary:
		//		Returns the ibm.tivoli.navmodel.LinkStyle for this relationship
		var args = {
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		if (!this._style) {
			if (this._relationship.linkStyle) {
				this._style = new ibm.tivoli.navmodel.LinkStyle({style: this._relationship.style, version: this._version()});
			}
			else {
				this._style = this._dataset.getRelationshipStyle({relationshipType: this.getType(), widgetDef: null});
			}
		}
		
		// merge widget definition styles 
		var style = this._style;
		if (style && args.widgetDef) {
			// clone and merge widget definition attributes				
			if (style) {
				style = style.clone(args.widgetDef, "relationship");
			}
			// unknown style; see if it exists in widget definition
			else {
				var widgetStyle = args.widgetDef.getRelationshipStyle(this.getType());
				if (widgetStyle) {
					style = this._dataset._createLinkStyle(args.widgetDef, widgetStyle, "relationship");
				}
			}							
		}
		
		return style;
	},

	setRelationshipStyle: function(style) {		
		// summary:
		//		Set link style for the link
		this._style = style;
	},

	getDefaultTask: function() {		
		// summary:
		//		Returns optional default task associated with the relationship
		if (!this._defaultTask && this._relationship.defaultTask) {
			this._defaultTask = new ibm.tivoli.navmodel.Task({task: this._relationship.defaultTask, dataset: this._dataset});
		}
		
		return this._defaultTask;
	},

	setDefaultTask: function(defaultTask) {		
		// summary:
		//		Set optional default task associated with the link
		this._defaultTask = defaultTask;
	},

	isExpired: function() {		
		// summary:
		//		Returns true if the item is currently expired
		return (this._expiration ? false : true);
	},

	checkExpiration: function(nowTime) {		
		// summary:
		//		Check the expiration time on the item; return true if expired
		var expired = false;
		
		// already set to null from a previous check
		if (!this._expiration) {
			expired = true;
		}
		// check agaist the time to see if expired
		else if (nowTime >= this._expiration) {
			this._expiration = null;
			expired = true;
		}
		
		return expired;		
	},

	resetExpiration: function() {		
		// summary:
		//		Rest the expiration time for this item
		if (this._dataset.cacheExpires()) {
			this._expiration = this._dataset.getNewExpirationTime();
		}
		else {
			this._expiration = -1;
		}
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////

	_version: function() {		
		// summary:
		//		Returns rest version
		return this._dataset.getVersion();
	},

	_validateExpiration: function(kwArgs) {		
		// summary:
		//		If item is expired; mixin with a current item from the dataset cache
		if (this._dataset.cacheExpires()) {
			// expired
			if (this.isExpired()) {
				var properties = null;
				var metadata = false;
				if (kwArgs) {
					properties = kwArgs.properties;
					metadata = kwArgs.metadata;
				}			
				if (!properties) {
					properties = this.getCachedPropertyIds();
				}
		
				var relationship = this._dataset.getRelationship({relationshipId: this.getId(), properties: properties, metadata: metadata});
				if (relationship) {
					this.mixin(relationship);
				}			
			}
			// NOT expired; reset expiration since touched
			else {
				this.resetExpiration();
			}
		}
	},
	
	/*=====
	fetchProperties_kwArgs: function(onComplete, onError, properties, metadata, sync) {
		// onComplete: properties_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		property ids to fetch or "all"
		// metadata: Boolean
		//		indicates whether metadata is required
		// sync: Boolean
		//		indicates if call is synchronous or asynchronous	
	},
	=====*/
	
	_fetchProperties: function(/*fetchProperties_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously returns properties for the link
		this._props = [];
		this._allProps = false;
		this._allMetadata = false;
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this._dataset);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		content.optimize = true;
		content.metadata = kwArgs.metadata;
		
		// are we fetching "all" properties
		var fetchAll = (kwArgs.properties.length === 1 && kwArgs.properties[0] === "all");
		
		return dojo.xhrGet({
			url: this.getUri(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "relationshipView")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var props = [];
				var rawProps = result.properties;
				var rawPropsLength = rawProps.length;
				for (var propIndex = 0;  propIndex < rawPropsLength;  ++propIndex) {
					var prop = new ibm.tivoli.navmodel.Property({property: rawProps[propIndex], dataset: this._dataset, objectType: "relationship"});
					props[propIndex] = prop;
					
					if (fetchAll) {
						this._props[propIndex] = prop;
					}
					
					this._propsById[prop.getId()] = prop;
				}
				
				if (fetchAll) {
					this._allProps = true;
					this._allMetadata = kwArgs.metadata;
				}
				
				kwArgs.onComplete(props, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_fetchProperties", this._trace, this._version()), ioArgs);			
			})
		});		
	},

	_fetchProperty: function(/*String*/ propId) {		
		// summary:
		//		Synchronously returns requested property
		// propId:
		//		property id to return
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var exception = null;
		var prop = null;
				
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this._dataset);
		content.optimize = true;
		content.properties = propId;
		
		dojo.xhrGet({
			url: this.getUri(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "relationshipView")
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				if (result.properties && result.properties.length > 0) {
					prop = new ibm.tivoli.navmodel.Property({property: result.properties[0], dataset: this._dataset, objectType: "relationship"});
				
					// cache property
					this._propsById[propId] = prop;
				}
				// property was not found; remember so we don't ask again
				else {
					this._nullPropIds[propId] = true;
				}					
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_fetchProperty", this._trace, this._version());				
			})
		});
		
		if (exception) {
			throw exception;
		}
		
		return prop;
	}
});

});

},
'dojoe/treetablex/_Toolbar':function(){
define([
	"dojo/_base/declare",
	"dojoe/common/toolbar/_Toolbar",
	"dijit/form/Button",
	"dijit/ToolbarSeparator",
	"dijit/Menu",
	"dijit/MenuItem",
	"dijit/form/DropDownButton",
	"dojo/i18n!dojoe/treetable/nls/TreeTable",
	"dojoe/treetablex/_ConfigureColumnsDialog",
	"dojoe/treetablex/_QuickSearchFilter"
	], function (declare, _CommonToolbar, Button, ToolbarSeparator, Menu, MenuItem, DropDownButton, _resources, _ConfigureColumnsDialog, _QuickSearchFilter) {


	_Toolbar = declare('dojoe.treetablex._Toolbar', [_CommonToolbar], {	
	
		// summary: 
		//		A private class to create tree table toolbar with icons, action lists and fitler options
		// description: 
		//		This is based on a common toolbar widget which supports both table and tree table widgets.
		//		This class is internally used to enable/disable toolbar icons, actions and search controls in the tree table
		//		Please refer dojoe.treetable.TreeTable.toolbarOptions property.
		// tags: private
		
		// widgetsInTemplate: boolean
		//		to indicate widgets in template
		widgetsInTemplate:true,
		//toolbar:null,
		
		// treeTable: object
		//		Reference to tree table instance
		treeTable:null,
		
		// actionListWidget: object
		//		Reference to Action List menu in toolbar
		actionListWidget:null,
		
				
		// refreshIcon: boolean
		//		To enable/disable refresh icon on toolbar, defaults to true
		refreshIcon:true,
		
		// refreshMenu: boolean
		//		To enable/disable refresh menu item in the action list 
		refreshMenu:true,
		
		// showSelectAllIcon: boolean
		//		to enable/disable select All icon in the toolbar
		showSelectAllIcon:true,
		
		// onlySelectAllIcon: boolean
		//		To provide a option to use only select All without de-select All option
		onlySelectAllIcon:false,
		
		// onlyDeselectAllIcon: boolean
		//		To provide a option to use only de-select All without select All option	
		onlyDeselectAllIcon:false,
		
		//09Oct2012 [54541]: Latha : Adding the attributes to provide an option to use expandAll & collapseAll individually and also retaining the property expandAllIcon for compatibility
		// expandAllIcon: boolean
		//		to indicate the state of items expand/collapse when the tree table loads
		expandAllIcon: true,
		
		// onlyExpandAllIcon: boolean
		//		To provide a option to use only expand All without collapse All option
		onlyExpandAllIcon:false,
		
		// onlyCollapseAllIcon: boolean
		//		To provide a option to use only collapse All without expand All option	
		onlyCollapseAllIcon:false,
		
		
		// clearSortIcon: boolean
		//		To enable/disable clear sort icon on the toolbar, defaults to true
		clearSortIcon: true,
		
		// configureTreeTableIcon: boolean
		//		To enable/disable Configure options in the toolbar, defaults to true
		configureTreeTableIcon:true,
		
		// filterOptions: object
		//		filter options to set to quick filter
		filterOptions: null,
		
		// actionMenu: boolean
		//		To enable/disable Action list in the toolbar, defaults to true
		actionMenu:true,
		
		// selectAllMenu: boolean
		//		To enable/disable select All menu item in the Action list, defaults to true
		selectAllMenu: true,
		
		// expandAllMenu: boolean
		//		To enable/disable expandAll menu item in Action list, defaults to true
		expandAllMenu : true,
		
		// clearSortMenu: boolean
		//		To enable/disable clear sort menu item in Action list, defaults to true
		clearSortMenu: true,
		
		// configureTreeTableMenu: boolean
		//		To enable/disable configure options menu item in Action list, defaults to true
		configureTreeTableMenu:true,
		
		// toolbarOptions: object
		//		local variable which holds the tree table toolbarOptions values 
		toolbarOptions : null,
		
		// quickFilterVisible: boolean
		//		To enable/disable quick search filter on the toolbar, defaults to true
		quickFilterVisible : true,
		
		// quickFilterVisible: boolean
		//		To enable/disable quick filter button on the toolbar, defaults to false
		showQuickFilterButton: false,	
		
		// _gridOrigStructure : object
		// Persisting the grid original structure 
		_gridOrigStructure : null,
		
		// Constructor function. Called when instance of this class is created
		constructor : function() {
			// summary:
			//		constructor
		
		},
		
		postCreate:function(){
			// summary:
			//		To handle stuff when the toolbar is rendered
			// tags: extension
			
			this.filterOptions = this.treeTable.filterOptions;
			this.toolbarOptions = this.treeTable.toolbarOptions;
			
			if(this.filterOptions && this.filterOptions.showQuickFilter !== undefined){
				this.quickFilterVisible = this.filterOptions.showQuickFilter;
			}	
			if(this.filterOptions && this.filterOptions.showQuickFilterButton !== undefined){
				this.showQuickFilterButton = this.filterOptions.showQuickFilterButton;
			}
			if(this.filterOptions.hideFilterBar == true && this.treeTable._grid.filterBar !== undefined){
				this.treeTable._grid.filterBar.hide();
			}
			if(this._gridOrigStructure == null) {
				this._gridOrigStructure = this.treeTable._grid.structure;
			}
			this._mapToolbarControls();
			this._createToolBarControls();			
		},
		
		_mapToolbarControls : function() {
			// summary: 
			//		Internal function to map toolbar options from dojoe.treetable.TreeTable to properties of this class
	
			if(this.toolbarOptions) {
				if(this.toolbarOptions.refreshIcon == false){
					this.refreshIcon = this.toolbarOptions.refreshIcon;
				}
				if(this.toolbarOptions.refreshMenu == false){
					this.refreshMenu = this.toolbarOptions.refreshMenu;
				}				
				if(this.toolbarOptions.selectAllIcon == false){
					this.showSelectAllIcon = this.toolbarOptions.selectAllIcon;
					
				}else{
					if(this.toolbarOptions.onlySelectAllIcon == true){
						this.onlySelectAllIcon=this.toolbarOptions.onlySelectAllIcon;
					}
					if(this.toolbarOptions.onlyDeselectAllIcon == true){
						this.onlyDeselectAllIcon=this.toolbarOptions.onlyDeselectAllIcon;
					}
				}
				if(this.toolbarOptions.selectAllMenu == false){
					this.selectAllMenu = this.toolbarOptions.selectAllMenu;
				}				
				if(this.toolbarOptions.expandAllIcon == false){
					this.expandAllIcon = this.toolbarOptions.expandAllIcon;
				}else{
					if(this.toolbarOptions.onlyExpandAllIcon == true){
						this.onlyExpandAllIcon=this.toolbarOptions.onlyExpandAllIcon;
					}
					if(this.toolbarOptions.onlyCollapseAllIcon == true){
						this.onlyCollapseAllIcon=this.toolbarOptions.onlyCollapseAllIcon;
					}
				}
				if(this.toolbarOptions.expandAllMenu == false){
					this.expandAllMenu = this.toolbarOptions.expandAllMenu;
				}
				
				if(this.toolbarOptions.clearSortIcon == false){
					this.clearSortIcon = this.toolbarOptions.clearSortIcon;
				}
				if(this.toolbarOptions.clearSortMenu == false){
					this.clearSortMenu = this.toolbarOptions.clearSortMenu;
				}
				
				if(this.toolbarOptions.configureTreeTableIcon == false) 
				{
					this.configureTreeTableIcon = this.toolbarOptions.configureTreeTableIcon;
				}
				if(this.toolbarOptions.configureTreeTableMenu == false){
					this.configureTreeTableMenu = this.toolbarOptions.configureTreeTableMenu;
				}
				if(this.toolbarOptions.actionMenu == false){
					this.actionMenu = this.toolbarOptions.actionMenu;
				}
				if(this.toolbarOptions.quickFilterVisible == false){
					this.quickFilterVisible = this.toolbarOptions.quickFilterVisible;
				}	
				if(this.toolbarOptions.quickFilterVisible && this.toolbarOptions.showQuickFilterButton){
					this.showQuickFilterButton = this.toolbarOptions.showQuickFilterButton;
				}
								
			}			
		},
		
		_refreshTreeTable: function(){
			// summary:
			//		Internal function to invoke tree table refresh
			
			this.treeTable.refresh();
		},
		
		_selectAll: function () 
		{
			// summary:
			//		Internal function to select all the rows in the tree table, provided Indirect Selection is enabled
			var count = this.treeTable._grid.rowCount();
				for (var i=0; i<count; i++) {
					this.treeTable._grid.row(i).select();
				}
			
		},
		
		_deselectAll: function () 
		{
			// summary:
			//		Internal function to deselect all the rows in the tree table, provided Indirect selection is enabled
			
			this.treeTable._grid.select.row.clear();
					
		},
		
		_expandAll : function(){
		
			// summary:
			//		Internal function to expand all the parent items in the tree
		
			var count = this.treeTable._grid.rowCount();
				for (var i=0; i<count; i++) {
					this.treeTable._grid.tree.expandRecursive(this.treeTable._grid.row(i).id);
				}
		},
		
		_collapseAll : function()
		{
			// summary:
			//		Internal function to collapse all the parent items in the tree
			
			var count = this.treeTable._grid.rowCount();
				for (var i=0; i<count; i++) {
					this.treeTable._grid.tree.collapseRecursive(this.treeTable._grid.row(i).id);
				}
		},
		
		_clearSort : function()
		{
			// summary:
			//		Internal function to clear sorting applied to all the columns
			
			this.treeTable._grid.sort.clear();
		},
		
		_configureColumn : function()
		{ 
			// summary:
			//		Internal function to show configure options dialog, which provides ability to show/hide columns in the tree table
			
		    //Satanik:25May2011: changes made to address ILMT requiremnt 19890
			this.tableOptionsDialog = new _ConfigureColumnsDialog ({ grid:this.treeTable._grid,indirectSelection:this.treeTable.indirectSelection,enableColumnReorder:this.treeTable.enableColumnReorder, gridStructure: this._gridOrigStructure});
			this.tableOptionsDialog.show();
		},
		
		//16716: table toolbar has browser popup menu on right click
		_stopEvent: function(e){
			// summary:
			//		to stop event propagation
			
			//Since event is oncontextmenu, just stop it!
			dojo.stopEvent(e);				
		},
		
		_createToolBarControls: function(){
			// summary:
			//		Internal function specific to tree table used to create required toolbar options/controls
			
			if(this.refreshIcon) {
				var refreshTableIcon = new Button({
					id: this.id+"refreshIcon",
					showLabel: false,
					label: _resources.REFRESH,
					title: _resources.REFRESH,
					iconClass: "refreshIcon"
				});
					
				refreshTableIcon.onClick = dojo.hitch (this, "_refreshTreeTable");
				refreshTableIcon.onKeypress = dojo.hitch (this, "_refreshTreeTable");
				var seperator = new ToolbarSeparator({id: "refSeperator"+this.id});
				
				this._addStateControl(refreshTableIcon,0);
				this._addStateControl(seperator);
			}
			if(this.showSelectAllIcon) {	
				if(this.onlySelectAllIcon){
					var selectAllIcon = new Button({
						id: this.id+"selectAllIcon",
						showLabel: false,
						label: _resources.SELECT_ALL,
						title: _resources.SELECT_ALL,
						iconClass: "selectAllIcon"
					});
						
					selectAllIcon.onClick = dojo.hitch (this, "_selectAll");
					selectAllIcon.onKeypress = dojo.hitch (this, "_selectAll");
					
					this._addFastPathControl(selectAllIcon);
				} else if(this.onlyDeselectAllIcon){
					var deselectAllIcon = new Button({
						id: this.id+"deselectAllIcon",
						showLabel: false,
						label: _resources.DESELECT_ALL,
						title: _resources.DESELECT_ALL,
						iconClass: "deselectAllIcon"
					
					});
						
					deselectAllIcon.onClick = dojo.hitch (this, "_deselectAll");
					deselectAllIcon.onKeypress = dojo.hitch (this, "_deselectAll");
					
					this._addFastPathControl(deselectAllIcon);
				} else {
					var selectAllIcon = new Button({
						id: this.id+"selectAllIcon",
						showLabel: false,
						label: _resources.SELECT_ALL,
						title: _resources.SELECT_ALL,
						iconClass: "selectAllIcon"
						
					});
						
					selectAllIcon.onClick = dojo.hitch (this, "_selectAll");
					selectAllIcon.onKeypress = dojo.hitch (this, "_selectAll");
					
					this._addFastPathControl(selectAllIcon);
					
					var deselectAllIcon = new Button({
						id: this.id+"deselectAllIcon",
						showLabel: false,
						label: _resources.DESELECT_ALL,
						title: _resources.DESELECT_ALL,
						iconClass: "deselectAllIcon"
						
					});
						
					deselectAllIcon.onClick = dojo.hitch (this, "_deselectAll");
					deselectAllIcon.onKeypress = dojo.hitch (this, "_deselectAll");
					this._addFastPathControl(deselectAllIcon);
				}
			}
			
			
			if(this.expandAllIcon) {
				if(this.onlyExpandAllIcon) {
				var expandtAllIcon = new dijit.form.Button({
						id: this.id+"expandableAllIcon",
						showLabel: false,
						label: _resources.EXPAND_ALL,
						title: _resources.EXPAND_ALL,
						iconClass: "expandtAllIcon"
					},this.expandtAllIcon);
					
				expandtAllIcon.onClick = dojo.hitch (this, "_expandAll");
				expandtAllIcon.onKeypress = dojo.hitch (this, "_expandAll");
				this._addFastPathControl(expandtAllIcon);
			} else if(this.onlyCollapseAllIcon) {
				var collapseAllIcon = new Button({
					id: this.id+"collapseAllIcon",
					showLabel: false,
					label: _resources.COLLAPSE_ALL,
					title: _resources.COLLAPSE_ALL,
					iconClass: "collapseAllIcon"
					
				});				
				collapseAllIcon.onClick = dojo.hitch (this, "_collapseAll");
				collapseAllIcon.onKeypress = dojo.hitch (this, "_collapseAll");			
				this._addFastPathControl(collapseAllIcon);
			} else {
				var expandtAllIcon = new dijit.form.Button({
						id: this.id+"expandableAllIcon",
						showLabel: false,
						label: _resources.EXPAND_ALL,
						title: _resources.EXPAND_ALL,
						iconClass: "expandtAllIcon"
					},this.expandtAllIcon);
					
				expandtAllIcon.onClick = dojo.hitch (this, "_expandAll");
				expandtAllIcon.onKeypress = dojo.hitch (this, "_expandAll");
				// 54125  Do NOT hardcode disabling expand all as it can be turned off via
				//        toolbarOptions.expandAllIcon = false
				// Temporarily disable expand all until we get performance
				// issue for gridx and our data store fixed
				// expandtAllIcon.set("disabled",true);
				this._addFastPathControl(expandtAllIcon);
				
				var collapseAllIcon = new Button({
					id: this.id+"collapseAllIcon",
					showLabel: false,
					label: _resources.COLLAPSE_ALL,
					title: _resources.COLLAPSE_ALL,
					iconClass: "collapseAllIcon"
					
				});				
				collapseAllIcon.onClick = dojo.hitch (this, "_collapseAll");
				collapseAllIcon.onKeypress = dojo.hitch (this, "_collapseAll");			
				this._addFastPathControl(collapseAllIcon);
				}
			} 
			if(this.clearSortIcon) {
				var removeSortIcon = new Button({
					id: this.id+"clearSortIcon",
					showLabel: false,
					label: _resources.CLEAR_SORT,
					title: _resources.CLEAR_SORT,
					iconClass: "removeSortIcon"
					
				});
					
				removeSortIcon.onClick = dojo.hitch (this, "_clearSort");
				removeSortIcon.onKeypress = dojo.hitch (this, "_clearSort");
				this._addFastPathControl(removeSortIcon);
			}
			
			if(this.configureTreeTableIcon) {
				var configureColIcon = new dijit.form.Button({
					id: this.id+"configureTreeTableIcon",
					showLabel: false,
					title: _resources.CONFIGURE_TREETABLE,
					label: _resources.CONFIGURE_TREETABLE,
					iconClass: "configureColIcon"
				});
					
				configureColIcon.onClick = dojo.hitch (this, "_configureColumn");
				configureColIcon.onKeypress = dojo.hitch (this, "_configureColumn");
				this._addFastPathControl(configureColIcon);
			} 
			
			if(this.quickFilterVisible && this.showQuickFilterButton) {
				this.quickFilter = new  _QuickSearchFilter({
					id: this.id+"quickFilter", 
					grid: this.treeTable._grid,
					_quickFilterButton: true,
					filterOptions: this.filterOptions },this.quickFilterNode);	
				var quickFilterIcon = new dijit.form.Button({
					showLabel: false,
					iconClass: "filterIcon",
					id: "quickFilterIcon"+this.id,
					label: _resources.QUICK_FILTER,
					title: _resources.QUICK_FILTER
				},this.filterButton);				
				quickFilterIcon.onClick = dojo.hitch (this, "_setQuickFilter");
				quickFilterIcon.onKeypress = dojo.hitch (this, "_setQuickFilter");
				quickFilterIcon.domNode.title = _resources.QUICK_FILTER;	
			} else if(this.quickFilterVisible && !this.showQuickFilterButton) {
				this.quickFilter = new _QuickSearchFilter({
					id: this.id+"quickFilter", 
					grid: this.treeTable._grid,
					_quickFilterButton: false,
					filterOptions: this.filterOptions },this.quickFilterNode);				
			}
			
			if(this.actionMenu){
				var menu = new Menu({ style: "display: none;"});
						
				if(this.refreshMenu) {
					var refreshMenuItem = new MenuItem({
						title: _resources.REFRESH,
						label: _resources.REFRESH,
						iconClass: "refreshIcon"
					});
					refreshMenuItem.onClick = dojo.hitch (this, "_refreshTreeTable");
					refreshMenuItem.onKeypress = dojo.hitch (this, "_refreshTreeTable");					 
					menu.addChild(refreshMenuItem);
				}				
				
				if(this.selectAllMenu){
					var selectAllMenuItem = new MenuItem({
						 title: _resources.SELECT_ALL,
						 label: _resources.SELECT_ALL,
						 iconClass: "selectAllIcon"			         
					 });
					 selectAllMenuItem.onClick = dojo.hitch (this, "_selectAll");
					 selectAllMenuItem.onKeypress = dojo.hitch (this, "_selectAll");	
					 menu.addChild(selectAllMenuItem);
					
					 var deselectAllMenuItem = new MenuItem({
						  title: _resources.DESELECT_ALL,
						 label: _resources.DESELECT_ALL,
						 iconClass: "deselectAllIcon"
					  });
					 deselectAllMenuItem.onClick = dojo.hitch (this, "_deselectAll");
					 deselectAllMenuItem.onKeypress = dojo.hitch (this, "_deselectAll");
					 menu.addChild(deselectAllMenuItem);
				}
				
				if(this.expandAllMenu){
					
					var expandAllMenuItem = new dijit.MenuItem({
						 title: _resources.EXPAND_ALL,
						 label: _resources.EXPAND_ALL,
						 iconClass: "expandtAllIcon"
					 });
					 expandAllMenuItem.onClick = dojo.hitch (this, "_expandAll");
					 expandAllMenuItem.onKeypress = dojo.hitch (this, "_expandAll");
					 // 54125  Do NOT hardcode disabling expand all as it can be turned off via
					 //        toolbarOptions.expandAllMenu = false
					 // Temporarily disable expand all until we get performance
					 // issue for gridx and our data store fixed
					 // expandAllMenuItem.set("disabled",true);
					 menu.addChild(expandAllMenuItem);
					 
					 var collapseAllMenuItem = new MenuItem({
						 title: _resources.COLLAPSE_ALL,
						 label: _resources.COLLAPSE_ALL,
						 iconClass: "collapseAllIcon"
					 });
					 collapseAllMenuItem.onClick = dojo.hitch (this, "_collapseAll");
					 collapseAllMenuItem.onKeypress = dojo.hitch (this, "_collapseAll");
					 menu.addChild(collapseAllMenuItem);
				}
					
				if(this.clearSortMenu){
					 var clearSortMenuItem = new MenuItem({
						label: _resources.CLEAR_SORT,
						title: _resources.CLEAR_SORT,
						iconClass: "removeSortIcon"
					 });
					 clearSortMenuItem.onClick = dojo.hitch (this, "_clearSort");
					 clearSortMenuItem.onKeypress = dojo.hitch (this, "_clearSort");
					 menu.addChild(clearSortMenuItem);
				}
				
				
				menu.startup();
				var actionList = new DropDownButton({
					optionsTitle : _resources.ACTIONS,
			        label: _resources.ACTIONS,
			        dropDown: menu,
					scrollOnFocus : true
			    });				
				this.actionListWidget = actionList;		
				this._addFastPathControl(actionList);	
			}
		},
		
		_setQuickFilter: function(){
			// summary:
			//		Internal function to set filter criteria for quick filter
			
			this.quickFilter._setFilter();
		},	
		
		_removeFromActionList: function (menuItemId){
			// summary:
			//		Internal function invoked by dojoe.treetable.TreeTable to remove items form Action list on the toolbar
			
			var menuItem = dijit.byId(menuItemId);
			if(menuItem !== null && menuItem !== undefined) {
				menuItem.destroyRecursive();
			}
		},
		
		//do not call this method directly, use table.removeItemFromToolbar(itemId) method to remove a custom item from the toolbar
		_removeItem: function(itemId){
			// summary:
			//		Internal function invoked by dojoe.treetable.TreeTable to remove controls on the toolbar
			
			var button= dijit.byId(itemId);
			if(button !== null && button !== undefined) {		
				if (button.domNode.parentNode) {
					//remove td which is the parent of the button
					buttonParent = button.domNode.parentNode;
					buttonParent.parentNode.removeChild(buttonParent);
				}
				button.destroyRecursive();
			}
		},
		
		//do not call this method directly, use treetable.addToToolbar(item,position,row) method to add a custom item to the toolbar before default buttons
		_addCustomControl: function(item, position, row){
			// summary:
			//		Internal function invoked by dojoe.treetable.TreeTable to add custom buttons, actions to the toolbar
			
			this.inherited(arguments);
		},	

		destroy: function(){
			// summary:  Destroys the toolbar.
			// tags: public	
			
			//memory leak issue: custom destroy method to take care of removing all widgets					
			try{
				delete _resources;
				//this is for destroying child templated widgets 
				//this should happen by itself but need to figure out why dijit._Templated is not taking care of it...
				var widgets = dijit.findWidgets(this.domNode);
				dojo.forEach(widgets, function(w) {
					if(w){
						if(w.destroyRecursive){
							w.destroyRecursive();
						} else if(w.destroy){
							w.destroy();
						}
					}
				});
				// this is for widgets added as child to this widget and their children..
				//not present in template
				var widgetArr = this.getChildren();
				dojo.forEach(widgetArr, function (child) {
					if(child){
						if(child.destroyRecursive){
							child.destroyRecursive();
						} else if(child.destroy){
							child.destroy();
						}
					}
				});
				this.inherited(arguments);
			}catch(ex){
				console.warn("Widget destroy failed: "+this.id+", ex="+ex.name+":"+ex.message);
			}
		}
	});
	return _Toolbar;
});
},
'dojo/cookie':function(){
define(["./_base/kernel", "./regexp"], function(dojo, regexp) {
	// module:
	//		dojo/cookie
	// summary:
	//		TODOC


/*=====
dojo.__cookieProps = function(){
	//	expires: Date|String|Number?
	//		If a number, the number of days from today at which the cookie
	//		will expire. If a date, the date past which the cookie will expire.
	//		If expires is in the past, the cookie will be deleted.
	//		If expires is omitted or is 0, the cookie will expire when the browser closes.
	//	path: String?
	//		The path to use for the cookie.
	//	domain: String?
	//		The domain to use for the cookie.
	//	secure: Boolean?
	//		Whether to only send the cookie on secure connections
	this.expires = expires;
	this.path = path;
	this.domain = domain;
	this.secure = secure;
}
=====*/


dojo.cookie = function(/*String*/name, /*String?*/value, /*dojo.__cookieProps?*/props){
	//	summary:
	//		Get or set a cookie.
	//	description:
	// 		If one argument is passed, returns the value of the cookie
	// 		For two or more arguments, acts as a setter.
	//	name:
	//		Name of the cookie
	//	value:
	//		Value for the cookie
	//	props:
	//		Properties for the cookie
	//	example:
	//		set a cookie with the JSON-serialized contents of an object which
	//		will expire 5 days from now:
	//	|	dojo.cookie("configObj", dojo.toJson(config), { expires: 5 });
	//
	//	example:
	//		de-serialize a cookie back into a JavaScript object:
	//	|	var config = dojo.fromJson(dojo.cookie("configObj"));
	//
	//	example:
	//		delete a cookie:
	//	|	dojo.cookie("configObj", null, {expires: -1});
	var c = document.cookie, ret;
	if(arguments.length == 1){
		var matches = c.match(new RegExp("(?:^|; )" + regexp.escapeString(name) + "=([^;]*)"));
		ret = matches ? decodeURIComponent(matches[1]) : undefined; 
	}else{
		props = props || {};
// FIXME: expires=0 seems to disappear right away, not on close? (FF3)  Change docs?
		var exp = props.expires;
		if(typeof exp == "number"){
			var d = new Date();
			d.setTime(d.getTime() + exp*24*60*60*1000);
			exp = props.expires = d;
		}
		if(exp && exp.toUTCString){ props.expires = exp.toUTCString(); }

		value = encodeURIComponent(value);
		var updatedCookie = name + "=" + value, propName;
		for(propName in props){
			updatedCookie += "; " + propName;
			var propValue = props[propName];
			if(propValue !== true){ updatedCookie += "=" + propValue; }
		}
		document.cookie = updatedCookie;
	}
	return ret; // String|undefined
};

dojo.cookie.isSupported = function(){
	//	summary:
	//		Use to determine if the current browser supports cookies or not.
	//
	//		Returns true if user allows cookies.
	//		Returns false if user doesn't allow cookies.

	if(!("cookieEnabled" in navigator)){
		this("__djCookieTest__", "CookiesAllowed");
		navigator.cookieEnabled = this("__djCookieTest__") == "CookiesAllowed";
		if(navigator.cookieEnabled){
			this("__djCookieTest__", "", {expires: -1});
		}
	}
	return navigator.cookieEnabled;
};

return dojo.cookie;
});

},
'dojo/Stateful':function(){
define(["./_base/declare", "./_base/lang", "./_base/array"], function(declare, lang, array) {
	// module:
	//		dojo/Stateful
	// summary:
	//		TODOC

return declare("dojo.Stateful", null, {
	// summary:
	//		Base class for objects that provide named properties with optional getter/setter
	//		control and the ability to watch for property changes
	// example:
	//	|	var obj = new dojo.Stateful();
	//	|	obj.watch("foo", function(){
	//	|		console.log("foo changed to " + this.get("foo"));
	//	|	});
	//	|	obj.set("foo","bar");
	postscript: function(mixin){
		if(mixin){
			lang.mixin(this, mixin);
		}
	},

	get: function(/*String*/name){
		// summary:
		//		Get a property on a Stateful instance.
		//	name:
		//		The property to get.
		//	returns:
		//		The property value on this Stateful instance.
		// description:
		//		Get a named property on a Stateful object. The property may
		//		potentially be retrieved via a getter method in subclasses. In the base class
		// 		this just retrieves the object's property.
		// 		For example:
		//	|	stateful = new dojo.Stateful({foo: 3});
		//	|	stateful.get("foo") // returns 3
		//	|	stateful.foo // returns 3

		return this[name]; //Any
	},
	set: function(/*String*/name, /*Object*/value){
		// summary:
		//		Set a property on a Stateful instance
		//	name:
		//		The property to set.
		//	value:
		//		The value to set in the property.
		//	returns:
		//		The function returns this dojo.Stateful instance.
		// description:
		//		Sets named properties on a stateful object and notifies any watchers of
		// 		the property. A programmatic setter may be defined in subclasses.
		// 		For example:
		//	|	stateful = new dojo.Stateful();
		//	|	stateful.watch(function(name, oldValue, value){
		//	|		// this will be called on the set below
		//	|	}
		//	|	stateful.set(foo, 5);
		//
		//	set() may also be called with a hash of name/value pairs, ex:
		//	|	myObj.set({
		//	|		foo: "Howdy",
		//	|		bar: 3
		//	|	})
		//	This is equivalent to calling set(foo, "Howdy") and set(bar, 3)
		if(typeof name === "object"){
			for(var x in name){
				if(name.hasOwnProperty(x) && x !="_watchCallbacks"){
					this.set(x, name[x]);
				}
			}
			return this;
		}
		var oldValue = this[name];
		this[name] = value;
		if(this._watchCallbacks){
			this._watchCallbacks(name, oldValue, value);
		}
		return this; //dojo.Stateful
	},
	watch: function(/*String?*/name, /*Function*/callback){
		// summary:
		//		Watches a property for changes
		//	name:
		//		Indicates the property to watch. This is optional (the callback may be the
		// 		only parameter), and if omitted, all the properties will be watched
		// returns:
		//		An object handle for the watch. The unwatch method of this object
		// 		can be used to discontinue watching this property:
		//		|	var watchHandle = obj.watch("foo", callback);
		//		|	watchHandle.unwatch(); // callback won't be called now
		//	callback:
		//		The function to execute when the property changes. This will be called after
		//		the property has been changed. The callback will be called with the |this|
		//		set to the instance, the first argument as the name of the property, the
		// 		second argument as the old value and the third argument as the new value.

		var callbacks = this._watchCallbacks;
		if(!callbacks){
			var self = this;
			callbacks = this._watchCallbacks = function(name, oldValue, value, ignoreCatchall){
				var notify = function(propertyCallbacks){
					if(propertyCallbacks){
						propertyCallbacks = propertyCallbacks.slice();
						for(var i = 0, l = propertyCallbacks.length; i < l; i++){
							propertyCallbacks[i].call(self, name, oldValue, value);
						}
					}
				};
				notify(callbacks['_' + name]);
				if(!ignoreCatchall){
					notify(callbacks["*"]); // the catch-all
				}
			}; // we use a function instead of an object so it will be ignored by JSON conversion
		}
		if(!callback && typeof name === "function"){
			callback = name;
			name = "*";
		}else{
			// prepend with dash to prevent name conflicts with function (like "name" property)
			name = '_' + name;
		}
		var propertyCallbacks = callbacks[name];
		if(typeof propertyCallbacks !== "object"){
			propertyCallbacks = callbacks[name] = [];
		}
		propertyCallbacks.push(callback);
		return {
			unwatch: function(){
				propertyCallbacks.splice(array.indexOf(propertyCallbacks, callback), 1);
			}
		}; //Object
	}

});

});

},
'ibm/tivoli/navmodel/AbstractDataset':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/Item,ibm/tivoli/navmodel/ItemStyle,ibm/tivoli/navmodel/Link,ibm/tivoli/navmodel/LinkStyle,ibm/tivoli/navmodel/Relationship,ibm/tivoli/navmodel/Property,ibm/tivoli/navmodel/Task,ibm/tivoli/navmodel/Status,ibm/tivoli/navmodel/Column,ibm/tivoli/navmodel/EventRouter,ibm/tivoli/navmodel/Event,ibm/tivoli/navmodel/Selection,ibm/tivoli/navmodel/NavWidgetDef,ibm/tivoli/navmodel/TopoWidgetDef,ibm/tivoli/navmodel/TableWidgetDef,ibm/tivoli/navmodel/NotebookWidgetDef,ibm/tivoli/navmodel/Exception,ibm/tivoli/navmodel/Utils,ibm/tivoli/navmodel/Trace"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.AbstractDataset"); 
 
// includes
dojo.require("ibm.tivoli.navmodel.Item");
dojo.require("ibm.tivoli.navmodel.ItemStyle");
dojo.require("ibm.tivoli.navmodel.Link");
dojo.require("ibm.tivoli.navmodel.LinkStyle");
dojo.require("ibm.tivoli.navmodel.Relationship");
dojo.require("ibm.tivoli.navmodel.Property");
dojo.require("ibm.tivoli.navmodel.Task");
dojo.require("ibm.tivoli.navmodel.Status");
dojo.require("ibm.tivoli.navmodel.Column");
dojo.require("ibm.tivoli.navmodel.EventRouter");
dojo.require("ibm.tivoli.navmodel.Event");
dojo.require("ibm.tivoli.navmodel.Selection");
dojo.require("ibm.tivoli.navmodel.NavWidgetDef");
dojo.require("ibm.tivoli.navmodel.TopoWidgetDef");
dojo.require("ibm.tivoli.navmodel.TableWidgetDef");
dojo.require("ibm.tivoli.navmodel.NotebookWidgetDef");
dojo.require("ibm.tivoli.navmodel.Exception");
dojo.require("ibm.tivoli.navmodel.Utils");
dojo.require("ibm.tivoli.navmodel.Trace");

ibm.tivoli.navmodel.AbstractDataset._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.AbstractDataset");

////////////////////////////////////
// AbstractDataset
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.AbstractDataset", null, {
	// private members       
	_args: null,
	
	// dataset attributes
	_dataset: null,
	_version: null,
	
	// widget definition supplied by server
	_widgetDefs: null,
	
	// item, link, relationship, caches
	_itemsById: null,
	_linksById: null,
	_relationshipsById: null,
	_cacheIntervalId: null,
	
	// styles and status caches
	_itemStyles: null,
	_itemStylesById: null,
	_itemStatus: null,
	_itemStatusById: null,	
	_linkStyles: null,
	_linkStylesById: null,
	_linkStatus: null,
	_linkStatusById: null,
	_relStyles: null,
	_relStylesById: null,
	_relStatus: null,
	_relStatusById: null,
	_status: null,
		
	// miscellaneous
	_toolbar: null,
	_eventRouter: null,
	_listeners: null,
	_destroyed: false,
    _remoteKeepAliveIntervalId: null,     // 255243
    _customFilterCondition: null,    // 22350
	_trace: ibm.tivoli.navmodel.AbstractDataset._trace, 
	
	/*=====
	constructor_onComplete: function(dataset, kwArgs){},
	=====*/
	
	/*=====
	constructor_onPage: function(start, count, kwArgs){},
	=====*/
	
	/*=====
	constructor_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	 constructor_kwArgs: function(onComplete, onError, onPage, baseUrl, provider, datasource, dataset, 
	 					parameters, defaultProperties, defaultLinkProperties, defaultRelProperties, 
	 					cacheThreshold, cacheValidationInterval, cacheExpirationTime, version, eventRouter) {
	 	// onComplete: constructor_onComplete
	 	//		callback to be invoked when the Dataset is initialized and ready for methods
	 	//		to be invoked  (i.e. asynchronous initialization must be performed). 
	 	//		Note: A null value will result in synchronous initialization.
	 	// onError: constructor_onError
	 	//		callback to be invoked if there is a problem constructing the Dataset
	 	// onPage:  constructor_onPage
	 	//		callback to be invoked when a new page is requested
		// baseUrl: String
		//		base url for identifying rest service
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// dataset: String
		//		dataset id
		// parameters: Object
		//		optional: extra parameters sent to dataset provider
		// defaultProperties: String[]
		//		optional: item property ids to fetch by default (can specify "all")
		// defaultLinkProperties: String[]
		//		optional: link property ids to fetch by default (can specify "all")
		// defaultRelProperties: String[]
		//		optional: relationship property ids to fetch by default (can specify "all")
		// cacheThreshold: int
		//		optional: cache objects if number of dataset items >= threshold
		// cacheValidationInterval: int
		//		optional: number of minutes in between cache validations
		// cacheExpirationTime: int
		//		optional: number of minutes objects can stay in cache while inactive
		// version: Float
		//		optional: version used to communicate with rest service (i.e. json version)
		// eventRouter: Object
		//		optional: object contains the following values (defaults shown)
		//		eventRouter: {
		//			timeout : 20,    		* number of seconds to wait for events  (optional)
		//			pollDelay : 0,			* number of ms to wait after an error before next poll  (optional)
		//			pollErrorDelay : 5000	* number of ms to wait after an error before next poll  (optional)
	   //		},	   	
		//		Use With Caution:  If you are the first Dataset to create an EventRouter object, the parameters 
		// 		specified here will dictate timeout, pollDelay and pollErrorDelay for ALL Datasets in the browser. 
  		//		If a Dataset has already been constructed, these parameters will have NO effect.  If the Dataset 
  		//		is destroyed, the event router will shutdown and new Dataset instances and their settings will be used.	
      // datasetImplementation: String
      //    optional: dataset data table type - 
      //                ibm.tivoli.navmodel.AbstractDataset.implementation.SIMPLE
      //                ibm.tivoli.navmodel.AbstractDataset.implementation.HYBRID
      //             or ibm.tivoli.navmodel.AbstractDataset.implementation.LARGE
      //                (default is ibm.tivoli.navmodel.AbstractDataset.implementation.SIMPLE)
      // remote: boolean
      //    optional: indicates whether the data provider is remote or local (default is false)
      // remoteKeepAliveInterval: int
      //    optional: number of minutes in between sending a request to keep a connection alive
      //              with a provider (used only if the provider is a remote provider)
	 },
	=====*/
	
	constructor: function(/*constructor_kwArgs*/ kwArgs, /*Object?*/ dataset) {
		// summary:
		//		Constructs AbstractDataset object.
		// description:
		//		The following options are normally not included in constructor (internal use only)
		//			createEventRouter: boolean - True indicates that a new event router should be created (vs. shared)
		//			dataset: object - Optional dataset object; will fetch from server if not provided
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		this._args = {
			onComplete: null,
			onError: null,
			onPage: null,
			baseUrl: "/ibm/tivoli/rest",
			provider: "",
			datasource: "",
			dataset: "",
			parameters: {},
			defaultProperties: [],
			defaultLinkProperties: [],
			defaultRelProperties: [],
			cacheThreshold:	250,
			cacheValidationInterval: 5,
			cacheExpirationTime: 15,
			version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion(),
			
			// The following options are normally not included in constructor
			createEventRouter: true,
         fetchMetaData: true,
         datasetImplementation: ibm.tivoli.navmodel.AbstractDataset.implementation.SIMPLE,
         remote: false,                   // 255243
         remoteKeepAliveInterval: 15      // 255243
		};
		dojo.mixin(this._args, kwArgs);
		
		// init empty caches
		this._itemsById = {};
		this._linksById = {};
		this._relationshipsById = {};
	
		// add param_pageId to help build unique event router queue: <user>_<sessionId>_<pageId>
		this._args.parameters.pageId = ibm.tivoli.navmodel.EventRouter.getPageId();
		
		// add param_refId to make sure server dataset is unique
		if(typeof(this._args.parameters.refId) == "undefined"){ 
			this._args.parameters.refId = Math.random();
		}
		// empty refId supplied; so don't include in parameters
		else if (this._args.parameters.refId.length === 0) {
			delete this._args.parameters.refId;
		}

		// add param_tz for the user timezone
		if (!this._args.parameters.tz) {
			this._args.parameters.tz = ibm.tivoli.navmodel.Utils.getTimeZone();
		}
		
		// subclass can initialize the dataset
		var initArgs = {
			onComplete: this._args.onComplete,
			onError: this._args.onError,
			onPage: this._args.onPage,
			optimize: true,
			dataset: dataset,
			createEventRouter: this._args.createEventRouter,
         fetchMetaData: this._args.fetchMetaData
		};
		this._initDataset(initArgs);		
	},

	/////////////////////////////
	// public methods
	/////////////////////////////

	refresh: function() {
		// summary:
		//		This method forces the dataset object to do a full refresh of its 
		//		data and caches
		
		// clear caches, refresh, and notify listeners
		this.clientChangedEvent();
	},

	destroy: function() {
		// summary:
		//		This method should be called to clean up the Dataset object
		
		if (!this._destroyed) {
			// cancel the cache validation timer
			if (this._cacheIntervalId) {
				clearInterval(this._cacheIntervalId);
			}
			
         // cancel the remote keep alive timer
         if (this._remoteKeepAliveIntervalId) {            // 255243
            clearInterval(this._remoteKeepAliveIntervalId);
         }
			
			// do any clean up like unregistering from event router, etc
			this._cleanupDataset();
			
			// destroy widget definition if we have one
			if (this._widgetDefs) {
				var length = this._widgetDefs.length;
				for (var index = 0;  index < length;  ++index) {
					if (this._widgetDefs[index]) {
						this._widgetDefs[index].destroy();
					}
				}								
			}
			
			this._destroyed = true;
		}
	},
		
	getArgs: function() {
		// summary:
		//		Returns the initial arguments passed to the dataset
		return dojo.clone(this._args);
	},

	getArg: function(key) {
		// summary:
		//		Returns a specific initial argument passed to the dataset
		return this._args[key];
	},

	getAttribute: function(key) {
		// summary:
		//		Returns general dataset attribute for requested key
		return this._dataset[key];
	},

	getAttributes: function(keyPrefix) {
		// summary:
		//		Returns attribute map for all keys that begin with keyPrefix;  if no
		//		keyPrefix is supplied; all attributes are returned
		var attr = {};		
		if (keyPrefix) {
			for (var key in this._dataset) {
				if (this._dataset.hasOwnProperty(key)) {
					if (key.indexOf(keyPrefix) === 0) {
						attr[key] = this._dataset[key];
					}			
				}
			}
		}
		else {
			dojo.mixin(attr, this._dataset);
		}
		
		return attr;
	},

	setAttribute: function(key, value) {
		// summary:
		//		Set general dataset attribute for requested key
		this._dataset[key] = value;		
	},

	mixinAttributes: function(attr) {
		// summary:
		//		Mixin general dataset attributes
		dojo.mixin(this._dataset, attr);		
	},
	
	isClearCacheOnRefresh: function() {
		// summary:
		//		Returns true if the dataset is requesting that
		//		?clearCache be placed on url when doing a Refresh;
		//		which will cause CURI cache to purge the dataset and reconstruct		
		return (!this._dataset.eventsEnabled || this._dataset.clearCacheOnRefresh);
	},
	
	isEventsEnabled: function() {
		// summary:
		//		Returns true if the dataset is capable of producing events
		return this._dataset.eventsEnabled;
	},

	getEventListenerCount: function() {
		// summary:
		//		Returns number of registered event listeners
		var listenerCount = 0;
		if (this._listeners) {
			listenerCount = this._listeners.length;
		}
		
		return listenerCount;
	},
	
	/*=====
	_listenerCallback: function(listenerObj, event) {
		// listenerObj
		// event
	},
	=====*/

	addEventListener: function(/*Object*/ listenerObj, /*_listenerCallback*/ listenerCallback) {
		// summary:
		//		Adds an event listener to the Dataset
		// listenerObj:
		//		unique object (i.e. same object must be passed to removeEventListener)
		// listenerCallback
		//		listener callback function for receiving the events
		
		// remove listener if it already exists
		this.removeEventListener(listenerObj);
			
		// add new listener
		if (!this._listeners) {
			this._listeners = [];
		}		
		this._listeners.push({listenerObj: listenerObj, listenerCallback: listenerCallback});
	},
	
	removeEventListener: function(/*Object*/ listenerObj) {
		// summary:
		//		Removes an event listener from the Dataset
		// listernObj:
		//		previously registered listener object
		if (this._listeners) {
			var listenerLength = this._listeners.length;
			for (var listenerIndex = 0;  listenerIndex < listenerLength;  ++listenerIndex) {
				if (this._listeners[listenerIndex].listenerObj === listenerObj) {
					// remove listener
					this._listeners.splice(listenerIndex, 1);
					
					// last listener so null out array					
					if (this._listeners.length === 0) {
						this._listeners = null;
					}
					
					break;
				}
			}	
		}
	},	

	getVersion: function() {
		// summary:
		//		Returns the version used to communicate with rest service
		//		(i.e. json version)
		return this._version;
	},
	
	_setVersion: function(version) {
		// summary:
		//		Sets the version used to communicate with the rest servce
		//		(i.e. json version)
		this._version = version;
	},

	getId: function() {
		// summary:
		//		Returns the dataset id
		return this._dataset.id;
	},

	setId: function(id) {
		// summary:
		//		Set the dataset id
		this._dataset.id = id;
	},

	getProviderId: function() {
		// summary:
		//		Returns the provider id
		return this._dataset.providerId;
	},

	setProviderId: function(providerId) {
		// summary:
		//		Set the provider id
		this._dataset.providerId = providerId;
	},

	getDatasourceId: function() {
		// summary:
		//		Returns the datasource id
		return this._dataset.datasourceId;
	},
	
	setDatasourceId: function(datasourceId) {
		// summary:
		//		Set the datasource id
		this._dataset.datasourceId = datasourceId;
	},

	getEventId: function() {
		// summary:
		//		Returns the event id
		return this._dataset.eventId;
	},

	setEventId: function(eventId) {
		// summary:
		//		Set the event id
		this._dataset.eventId = eventId;
	},

	getType: function() {
		// summary:
		//		Returns the dataset type
		return this._dataset.type;
	},

	setType: function(type) {
		// summary:
		//		Set the dataset type
		this._dataset.type = type;		
	},

	getViewType: function() {
		// summary:
		//		Returns the preferred view type for rendering
		//		
		//		values: "graph", "table", "tree"
		return this._dataset.viewType;
	},

	setViewType: function(viewType) {
		// summary:
		//		Set the preferred view type for rendering
		this._dataset.viewType = viewType;		
	},

   getDatasetImplementation: function() {        // 255211
       // summary:
       //		Returns the dataset implementation type
       //		
       //		values: ibm.tivoli.navmodel.AbstractDataset.implementation.SIMPLE, 
       //              ibm.tivoli.navmodel.AbstractDataset.implementation.HYBRID,
       //              ibm.tivoli.navmodel.AbstractDataset.implementation.LARGE_TABLE
       return this._dataset.datasetImplementation;
   },

   setDatasetImplementation: function(implementationType) {     // 255211
       // summary:
       //		Set the dataset implementation type
       this._dataset.datasetImplementation = implementationType;		
   },
	
	isGraph: function() {
		// summary:
		//		Returns true if the dataset represents a graph
		return this.getViewType() === "graph";
	},

	isTable: function() {
		// summary:
		//		Returns true if the dataset represents a table
		return this.getViewType() === "table";
	},

	isTree: function() {
		// summary:
		//		Returns true if the dataset represents a tree
		return this.getViewType() === "tree";
	},

	isLazyLoad: function() {
		// summary:
		//		Returns true if the dataset designates that it should be lazy loaded
		return (this._dataset.lazyLoad !== undefined ? this._dataset.lazyLoad : false);
	},
	
	setLazyLoad: function(lazyLoad) {
		// summary:
		//		Set lazy load attribute for the dataset
		this._dataset.lazyLoad = lazyLoad;
	},
	
	getLazyLoadRelationships: function() {		
		// summary:
		//		Returns the relationships that should be explored for related items
		return this._dataset.relationshipsKey;
	},

	setLazyLoadRelationships: function(relationshipsKey) {		
		// summary:
		//		Set the relationships that should be explored for related items
		this._dataset.relationshipsKey = relationshipsKey;
	},
	
	getLabel: function() {	
		// summary:
		//		Returns globalized label for the dataset
		return this._dataset.label;
	},

	setLabel: function(label) {
		// summary:
		//		Set globalized label for the dataset
		this._dataset.label = label;		
	},

	getLabelQualifier: function() {
		// summary:
		//		Returns globalized label qualifier that can be used in conjunction with the label
		//		i.e.    label (labelQualifier)
		return this._dataset.labelQualifier;
	},

	setLabelQualifier: function(labelQualifier) {
		// summary:
		//		Set globalized label qualifier that can be used in conjunction with the label
		this._dataset.labelQualifier = labelQualifier;		
	},
	
	getCustomFilterCondition: function() {   // 22350
		// summary:
		//      Returns the custom filter condition string for this dataset, or null if none is set
		
		return this._customFilterCondition;
	},
	
	setCustomFilterCondition: function(filterCondition) {   // 22350
		// summary:
		//      Set the custom filter condition string for this dataset
		//      or null to clear the custom filter condition
		
		this._customFilterCondition = filterCondition;
	},	
	
	/*=====
	depth_kwArgs1: function(widgetDef) {
		// widgetDef: Object
		//		widget definition can override the depth value
	},
	=====*/

	getDepth: function(/*depth_kwArgs*/ kwArgs) {
		// summary:
		//		Returns the default depth for the dataset if it is a graph
		//		(otherwise -1)
		var args = {
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		if (args.widgetDef) {
			return args.widgetDef.getDepth();
		}
				
		return (this._dataset.depth ? this._dataset.depth : -1);		
	},

	setDepth: function(depth) {
		// summary:
		//		Set the depth for the dataset if it is a graph
		// depth:
		//		New topology depth
		
		// update depth parameter
		var found = false;
		for (var parm in this._args.parameters) {
			if (parm === "depth" && this._args.parameters.hasOwnProperty(parm)) {
				this._args.parameters[parm] = depth;
				found = true;
				break;
			}
		}
		if (!found) {
			this._args.parameters.depth = depth;
		}
		
		// clear caches, refresh, and notify listeners
		var event = ibm.tivoli.navmodel.ClientEvent.getChangedEvent(this);
		this._eventHandler(event);		
	},

	getMaxDepth: function(/*depth_kwArgs*/ kwArgs) {
		// summary:
		//		Returns the maximum depth for the dataset if it is a graph
		var args = {
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		if (args.widgetDef) {
			return args.widgetDef.getMaxDepth();
		}
		
		return (this._dataset.maxDepth ? this._dataset.maxDepth : -1);		
	},

	getDefaultProperties: function() {
		// summary:
		//		Returns default properties that are fetched with each item
		return this._args.defaultProperties;
	},
	
	setDefaultProperties: function(itemProperties) {
		// summary:
		//		Sets default properties that are fetched with each item
		// itemProperties:
		//		Array of item property ids
		this._args.defaultProperties = itemProperties;
	},

	getDefaultLinkProperties: function() {
		// summary:
		//		Returns default properties that are fetched with each link
		return this._args.defaultLinkProperties;
	},
	
	setDefaultLinkProperties: function(linkProperties) {
		// summary:
		//		Sets default properties that are fetched with each link
		// linkProperties:
		//		Array of link properties
		this._args.defaultLinkProperties = linkProperties;
	},

	getDefaultRelProperties: function() {
		// summary:
		//		Returns default properties that are fetched with each relationship
		return this._args.defaultRelProperties;
	},
	
	setDefaultRelProperties: function(relProperties) {
		// summary:
		//		Sets default properties that are fetched with each relationship
		// relProperties:
		//		Array of relationship properties
		this._args.defaultRelProperties = relProperties;
	},
	
	cacheExpires: function() {
		// summary:
		//		Returns true if the items, links, and relationships cache expires for this dataset.  If
		//		there are only a small number of objects the dataset will not bother with expiration.
		return (this._cacheIntervalId ? true : false);
	},

	getNewExpirationTime: function() {
		// summary:
		//		Returns an expiration time for a new item, link, or relationship
		var now = new Date();
		return now.getTime() + (this._args.cacheExpirationTime*60000);
	},
	
	getItemCount: function() {
		// summary:
		//		Returns the number of items associated with the dataset
		return this._dataset.itemCount;
	},
	
	/*=====
	itemsAndLinks_onComplete: function(items, links, kwArgs){},
	=====*/
	
	/*=====
	_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	getItemsAndLinks_kwArgs: function(onComplete, onError, sync) {
		// onComplete: itemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getItemsAndLinks: function(/*getItemsAndLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Returns both the items and links associated with the dataset
		var args = {                
			onComplete: function(items, links, kwArgs) {},
			onError: function(error, exception, ioArgs) {},
			sync: false
		};   
		dojo.mixin(args, kwArgs);
		
		// fetch items and links from rest service
		var onComplete = args.onComplete;		
		args.onComplete = dojo.hitch(this, function(items, links, toolbar, args) {
			// cache items and links
			items = this._cacheItems(items);
			links = this._cacheLinks(links);
				
			// cache toolbar tasks				
			this._toolbar = toolbar;
				
			// call orignal callback
			onComplete(items, links, args);
		});			
		this._fetchItemsAndLinks(args);		
	},
	
	/*=====
	items_onComplete: function(items, kwArgs){},
	=====*/

	/*=====
	getItems_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error conditions 
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// itemIds: String
		//		comma delimited list of item ids to fetch				
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getItems: function(/*getItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items associated with the dataset
		var args = {                
			onComplete: function(items, kwArgs){
			},
			onError: function(error, exception, ioArgs){				
			},
			start: 0,
			count: null,
			itemIds: null,           // 24658
			properties: this._args.defaultProperties,
			metadata: false, 
			sort: null,
			condition: null,
			sync: false
		};   
		dojo.mixin(args, kwArgs);         
		
		///////////////////////////////////////////////////////////
		// make sure we always return asynchronously in case we are 
		// being invoked by a dojo datastore
		///////////////////////////////////////////////////////////
		
		// are NOT any items
		if (this.getItemCount() === 0) {		
			args.numRows = 0;
			args.filteredRows = 0;
			args.totalRows = 0;				
			if (args.sync) {
				args.onComplete([], args);  // 52221  Do not return null
			}
			else {
				setTimeout(dojo.hitch(this, args.onComplete, [], args), 0);  // 52221 Do not return null
			}			
		}
		// fetch items from rest service
		else {
			var onComplete = args.onComplete;		
			args.onComplete = dojo.hitch(this, function(items, args) {
				// cache items
				items = this._cacheItems(items);
				
				// call orignal callback
				onComplete(items, args);
			});
			this._fetchItems(args);
		}		
	},
	
	hasCachedItems: function() {
		// summary:
		//		indicates if dataset currently contains cached items			
		if (this._itemsById) {
			for (var id in this._itemsById) { 
    			if (this._itemsById.hasOwnProperty(id)) {
    				return true;
    			}
    		}
		}
		
		return false;
	},
	
	getCachedItems: function() {
		// summary:
		//		returns array with currently cached items	
		var items = [];
		this._traverseCachedItems(items);
		return items;
	},
	
	_traverseCachedItems: function(/*ibm.tivoli.navmodel.Item[]*/ items) {
		if (this._itemsById) {
			for (var id in this._itemsById) { 
    			if (this._itemsById.hasOwnProperty(id)) {
    				items.push(this._itemsById[id]);
    			}
    		}
		}
	},
	
	/*=====
	getItem_kwArgs: function(itemId, properties, metadata) {
		// itemId: String
		//		item id to get
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
	},
	=====*/

	getItem: function(/*getItem_kwArgs*/ kwArgs) {
		// summary:
		//		Returns the requested item
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var args = {
			itemId: null,
			properties: null,
			metadata: false
		};
		dojo.mixin(args, kwArgs);
		
		// check cache
		var item = this._getCachedItem(args.itemId);
		
		// force a fetch if we do not have all the requested properties		
		if (item && ((args.metadata && !item.hasCachedMetadata()) || (args.properties && !item.hasCachedProperties(args.properties)))) {
			item = null;			
		}
		
		// need to fetch individual item
		if (!item) {
			if (!args.properties || args.properties.length === 0) {
				args.properties = this._args.defaultProperties;
			}
			
			item = this._fetchItem(args);
			item = this._cacheItem(item);			
		}	

		return item;
	},
	
	/*=====
	getOriginItems_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: itemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metdata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getOriginItems: function(/*getOriginItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns orign items of the dataset
		var args = {                
			onComplete: function(items, links, kwArgs){
			},
			onError: function(error, exception, ioArgs){				
			},
			start: 0,
			count: null,
			properties: this._args.defaultProperties,
			metadata: false,
			sort: null,
			condition: null,
			sync: false
		};   
		dojo.mixin(args, kwArgs);
		
		var onComplete = args.onComplete;		
		args.onComplete = dojo.hitch(this, function(items, args) {
			// cache items
			items = this._cacheItems(items, true);
				
			// call orignal callback
			onComplete(items, args);
		});
		this._fetchOriginItems(args);
	},
	
	/*=====
	getRelatedItems_kwArgs: function(onComplete, onError, itemId, relationships, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: itemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// itemId: String
		//		id of the base item
		// relationships: String
		//		comma delimited list of relationships to determine related items; 
		//		can also be keywords: child, parent, all
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getRelatedItems: function(/*getRelatedItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items that are related to the requested item
		var args = {                
			onComplete: function(items, links, kwArgs){
			},
			onError: function(error, exception, ioArgs){				
			},
			itemId: null,
			relationships: null,
			start: 0,
			count: null,                
			properties: this._args.defaultProperties,
			metadata: false,
			sort: null,
			condition: null,
			sync: false
		};   
		dojo.mixin(args, kwArgs);
		
		var onComplete = args.onComplete;		
		args.onComplete = dojo.hitch(this, function(items, links, args) {
			// cache items and links
			items = this._cacheItems(items, true);
			links = this._cacheLinks(links);			
				
			// if no related items were returned; reset lazyLoad indicators
			if (!items || items.length === 0) {
				var item = this._getCachedItem(kwArgs.itemId);
				if (item) {
					item.resetLazyLoad();
				}
			}
			
			// call orignal callback
			onComplete(items, links, args);
		});		
		this._fetchRelatedItems(args);
	},

	/*=====
	queryItems_kwArgs: function(onComplete, onError, query, start, count, properties, metadata, sync) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// query: Object
		//		custom query object to be sent as request body
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	queryItems: function(/*queryItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items associated with the dataset that match
		//		the provided custom query
		var args = {                
			onComplete: function(items, kwArgs){
			},
			onError: function(error, exception, ioArgs){				
			},
			query: {},                
			start: 0,
			count: null,
			properties: this._args.defaultProperties,
			metadata: false,
			sync: false
		};   
		dojo.mixin(args, kwArgs);         
		
		var onComplete = args.onComplete;		
		args.onComplete = dojo.hitch(this, function(items, args) {
			// cache items
			items = this._cacheItems(items);
						
			// call orignal callback
			onComplete(items, args);
		});		
		this._queryItems(args);	
	},

	getLinkCount: function() {
		// summary:
		//		Returns the number of links associated with the dataset
		return this._dataset.linkCount;
	},

	/*=====
	links_onComplete: function(items, kwArgs){},
	=====*/

	/*=====
	getLinks_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: links_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error conditions
		// start: int
		//		start index to fetch
		// count: int
		//		number of links to fetch; null means all
		// linkIds: String
		//		comma delimited list of link ids to fetch						
		// properties: String[]
		//		link property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	getLinks: function(/*getLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Returns links associated with the dataset
		var args = {                
			onComplete: function(links, kwArgs){
			},
			onError: function(error, exception, ioArgs){				
			},
			start: 0,
			count: null,
			linkIds: null,   // 24658			
			properties: this._args.defaultLinkProperties,
			metadata: false,
			sort: null,
			condition: null,
			sync: false
		};   
		dojo.mixin(args, kwArgs);         

		// are NOT any links
		if (this.getLinkCount() === 0) {
			args.numRows = 0;
			args.filteredRows = 0;
			args.totalRows = 0;				
			if (args.sync) {
				args.onComplete(null, args);
			}
			else {
				setTimeout(dojo.hitch(this, args.onComplete, null, args), 0);
			}
		}
		// fetch links from rest service
		else {
			var onComplete = args.onComplete;		
			args.onComplete = dojo.hitch(this, function(links, args) {
				// cache links
				links = this._cacheLinks(links);
				
				// call orignal callback
				onComplete(links, args);
			});
			this._fetchLinks(args);
		}
	},
	
	hasCachedLinks: function() {
		// summary:
		//		indicates if dataset currently contains cached links			
		if (this._linksById) {
			for (var id in this._linksById) { 
    			if (this._linksById.hasOwnProperty(id)) {
    				return true;
    			}
    		}
		}
		
		return false;
	},
	
	getCachedLinks: function() {
		// summary:
		//		returns array with currently cached links		
		var links = [];
		this._traverseCachedLinks(links);		
		return links;
	},
	
	_traverseCachedLinks: function(/*ibm.tivoli.navmodel.Link[]*/ links) {
		if (this._linksById) {
			for (var id in this._linksById) { 
    			if (this._linksById.hasOwnProperty(id)) {
    				links.push(this._linksById[id]);
    			}
    		}
		}
	},

	/*=====
	getLink_kwArgs: function(itemId, properties, metadata) {
		// linkId: String
		//		link id to get
		// properties: String[]
		//		array of property ids to pre-fetch
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
	},
	=====*/
	
	getLink: function(/*getLink_kwArgs*/ kwArgs) {
		// summary:
		//		Returns the requested link
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var args = {
			linkId: null,
			properties: null,
			metadata: false
		};
		dojo.mixin(args, kwArgs);
		
		// check cache
		var link = this._getCachedLink(args.linkId);
		
		// force a fetch if we do not have all the requested properties		
		if (link && ((args.metadata && !link.hasCachedMetadata()) || (args.properties && !link.hasCachedProperties(args.properties)))) {
			link = null;			
		}
			
		// need to fetch individual link
		if (!link) {
			if (!args.properties || args.properties.length === 0) {
				args.properties = this._args.defaultLinkProperties;
			}
			
			link = this._fetchLink(args);
			link = this._cacheLink(link);			
		}	
		
		return link;
	},

	getRelationshipCount: function() {
		// summary:
		//		Returns the number of items associated with the dataset
		return this._dataset.relationshipCount;
	},
	
	/*=====
	relationships_onComplete: function(relationships, kwArgs){},
	=====*/
	
	/*=====
	getRelationships_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: relationships_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error conditions
		// start: int
		//		start index to fetch
		// count: int
		//		number of relationships to fetch; null means all
		// relationshipIds: String
		//		comma delimited list of relationship ids to fetch								
		// properties: String[]
		//		relationship property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getRelationships: function(/*getRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Returns relationships associated with the dataset
		var args = {                
			onComplete: function(relationships, kwArgs){
			},
			onError: function(error, exception, ioArgs){				
			},
			start: 0,
			count: null,
			relationshipIds: null,    // 24658
			properties: this._args.defaultRelProperties,
			metadata: false,
			sort: null,
			condition: null,
			sync: false
		};   
		dojo.mixin(args, kwArgs);         

		///////////////////////////////////////////////////////////
		// make sure we always return asynchronously in case we are 
		// being invoked by a dojo datastore
		///////////////////////////////////////////////////////////
		
		// are NOT any links/relationships
		if (this.getLinkCount() === 0) {
			args.numRows = 0;
			args.filteredRows = 0;
			args.totalRows = 0;				
			if (args.sync) {
				args.onComplete(null, args);
			}
			else {		
				setTimeout(dojo.hitch(this, args.onComplete, null, args), 0);
			}
		}
		// fetch relationships from rest service
		else {
			var onComplete = args.onComplete;		
			args.onComplete = dojo.hitch(this, function(relationships, args) {
				// cache relationships
				relationships = this._cacheRelationships(relationships);
				
				// call orignal callback
				onComplete(relationships, args);
			});
			this._fetchRelationships(args);
		}
	},
	
	hasCachedRelationships: function() {
		// summary:
		//		indicates if dataset currently contains cached relationships
		if (this._relationshipsById) {
			for (var id in this._relationshipsById) { 
    			if (this._relationshipsById.hasOwnProperty(id)) {
    				return true;
    			}
    		}
		}
		
		return false;
	},
	
	getCachedRelationships: function() {
		// summary:
		//		return array with currently cached relationships	
		var relationships = [];
		this._traverseCachedRelationships(relationships);
		return relationships;
	},
	
	_traverseCachedRelationships: function(/*ibm.tivoli.navmodel.Relationship[]*/ relationships) {
		if (this._relationshipsById) {
			for (var id in this._relationshipsById) { 
    			if (this._relationshipsById.hasOwnProperty(id)) {
    				relationships.push(this._relationshipsById[id]);
    			}
    		}
		}
	},
	
	/*=====
	getRelationship_kwArgs: function(relationshipId, properties, metadata) {
		// relationshipId: String
		//		relationship id to fetch
		// properties: String[]
		//		array of property ids to pre-fetch
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
	},
	=====*/
	
	getRelationship: function(/*getRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Returns the requested relationship
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var args = {
			relationshipId: null,
			properties: null,
			metadata: false
		};
		dojo.mixin(args, kwArgs);
		
		// check cache
		var relationship = this._getCachedRelationship(args.relationshipId);

		// force a fetch if we do not have all the requested properties		
		if (relationship && ((args.metadata && !relationship.hasCachedMetadata()) || (args.properties && !relationship.hasCachedProperties(args.properties)))) {
			relationship = null;			
		}
						
		// need to fetch individual relationship
		if (!relationship) {
			if (!args.properties || args.properties.length === 0) {
				args.properties = this._args.defaultRelProperties;
			}
			
			relationship = this._fetchRelationship(args);
			relationship = this._cacheRelationship(relationship);
		}	
		
		return relationship;
	},
	
	/*=====
	tasks_onComplete: function(tasks, kwArgs){},
	=====*/
	
	/*=====
	getTasks_kwArgs: function(onComplete, onError, taskCategory, selection, itemIds, linkIds, itemIndexes, sync) {
		// onComplete: tasks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// taskCategory: String
		//		comma delimited list indicating type of tasks to fetch
		//		legal values are: menu, toolbar, and breadcrumb
		// selection: Object
		//		ibm.tivoli.navmodel.Selection object
		//		if selection is specified: itemIds, linkIds, itemIndexes is ignored
		// itemIds: String
		//		comma delimited list of item ids that are selected
		// linkIds, String
		//		comma delimited list of link ids that are selected
		// itemIndexes: String
		//		comma delimited list of item ids or ranges that are selected
		//		ex: "0,1,10,20" or "0-10,30,40-50"
		// sync: Boolean
		//		indicates if fetch is synchronous
		// failOK: Boolean
		//      indicates if errors should not be logged to the console by the request object
		//      (but onError is still called, if specified).  See dojo\_base\xhr.js
	},
	=====*/

	getTasks: function(/*getTasks_kwArgs*/ kwArgs) {
		// summary:
		//		Returns tasks associated with the dataset
        var request = null;  // 37703
		var args = {
			onComplete: function(tasks, kwArgs){
			},
			onError: function(error, exception, ioArgs){				
			},
			taskCategory: "menu",
			selection: null,
			itemIds: null,
			linkIds: null,
			itemIndexes: null,
			sync: false,
            failOK: false   // 37703
		};
		dojo.mixin(args, kwArgs);
		
		// asking for "toolbar" without any selection and we previously
		// fetched the toolbar tasks with the items and links
		if (args.taskCategory === "toolbar" && this._toolbar &&
			(!args.selection || args.selection.isEmpty()) && !args.itemIds && !args.linkIds && !args.itemIndexes) {
			if (args.sync) {
				args.onComplete(this._toolbar, args);
			}
			else {
				setTimeout(dojo.hitch(this, args.onComplete, this._toolbar, args), 0);
			}		
		}		
		// fetch tasks from rest service
		else {
			request = this._fetchTasks(args);  // 37703
		}
		return request;    // 37703
	},

	/*=====
	dataset_onComplete: function(dataset, kwArgs){},
	=====*/
	
	/*=====
	navigate_kwArgs: function(onComplete, onError, taskId, taskCategory, selection, sync) {
		// onComplete: dataset_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// taskId: String
		//		navigation task id
		// taskCategory: String
		//		navigation task category
		// selection: Object
		//		ibm.tivoli.navmodel.Selection object
		//		if selection is specified: itemIds, linkIds, itemIndexes is ignored
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	navigate: function(/*navigate_kwArgs*/ kwArgs) {
		// summary:
		//		Navigate to another Dataset from a navigation task
		var args = {                
			onComplete: function(dataset, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},                
			taskId: null,
			taskCategory: "menu:navigation",
			selection: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		this._navigate(args);     
	},

	/*=====
	itemStyles_onComplete: function(itemStyles, kwArgs){},
	=====*/
	
	/*=====
	getItemStyles_kwArgs: function(onComplete, onError, start, count, widgetDef, sync) {
		// onComplete: itemStyles_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of item styles to fetch;  null means all
		// widgetDef: Object
		//		widget definition that can override attributes
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getItemStyles: function(/*getItemStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current item styles for the Dataset
		var args = {
			onComplete: function(itemStyles, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			start: 0,
			count: null,
			widgetDef: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		// already have the item styles
		if (this._itemStyles) {
			if (args.widgetDef) {
				if (args.sync) {
					args.onComplete(this._mergeItemStyles(this._itemStyles, args.widgetDef), args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, this._mergeItemStyles(this._itemStyles, args.widgetDef), args), 0);
				}
			}
			else if (args.sync) {
				args.onComplete(this._itemStyles, args);
			}
			else {
				setTimeout(dojo.hitch(this, args.onComplete, this._itemStyles, args), 0);
			}			
		}
		// need to fetch the item styles
		else {
			var onComplete = args.onComplete;		
			args.onComplete = dojo.hitch(this, function(itemStyles, args) {
				// cache styles
				if (args.start === 0 && args.count === null) {
					this._cacheItemStyles(itemStyles);					
				}
				
				// call orignal callback
				if (args.widgetDef) {
					onComplete(this._mergeItemStyles(itemStyles, args.widgetDef), args);
				}
				else {
					onComplete(itemStyles, args);
				}				
			});			
			this._fetchItemStyles(args);
		}
	},
	
	/*=====
	getItemStyle_kwArgs: function(itemType, widgetDef) {
		// itemType: String
		//		type of itemStyle
		// widgetDef: Object
		//		widget definition that can override attributes
	},
	=====*/

	getItemStyle: function(/*getItemStyle_kwArgs*/ kwArgs) {
		// summary:
		//		Returns item style for requested itemType
		var itemStyle = null;
		var args = {
			itemType: "",
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		// if we don't have item styles fetch them
		if (!this._itemStyles) {
			var fetchArgs = {
				onComplete: dojo.hitch(this, function(itemStyles, args){
					this._cacheItemStyles(itemStyles);					
				}),
				onError: function(error, exception, ioArgs){
				},
				start: 0,
				count: null,
				sync: true
			};
			this._fetchItemStyles(fetchArgs);
		}
		
		// locate requested item style in cache
		if (this._itemStylesById) {
			itemStyle = this._itemStylesById[args.itemType];
		}
		
		if (args.widgetDef) {
			// clone and merge widget definition attributes				
			if (itemStyle) {
				itemStyle = itemStyle.clone(args.widgetDef);					
			}
			// unknown style to dataset; see if it exists in widget definition
			else {
				var widgetStyle = args.widgetDef.getItemStyle(args.itemType);
				if (widgetStyle) {
					itemStyle = this._createItemStyle(args.widgetDef, widgetStyle);
				}					
			}
		}
		
		return itemStyle;
	},
	
	/*=====
	linkStyles_onComplete: function(linkStyles, kwArgs){},
	=====*/
	
	/*=====
	getLinkStyles_kwArgs: function(onComplete, onError, start, count, widgetDef, sync) {
		// onComplete: linkStyles_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of link styles to fetch; null means all
		// widgetDef: Object
		//		widget definition that can override attributes
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getLinkStyles: function(/*getLinkStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current link styles for the Dataset
		var args = {
			onComplete: function(linkStyles, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			start: 0,
			count: null,
			widgetDef: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		// already have the link styles
		if (this._linkStyles) {
			if (args.widgetDef) {
				if (args.sync) {
					args.onComplete(this._mergeLinkStyles(this._linkStyles, args.widgetDef, "link"), args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, this._mergeLinkStyles(this._linkStyles, args.widgetDef, "link"), args), 0);
				}
			}
			else if (args.sync) {
				args.onComplete(this._linkStyles, args);
			}
			else {
				setTimeout(dojo.hitch(this, args.onComplete, this._linkStyles, args), 0);
			}
		}
		// need to fetch the link styles
		else {
			var onComplete = args.onComplete;		
			args.onComplete = dojo.hitch(this, function(linkStyles, args) {
				// cache styles
				if (args.start === 0 && args.count === null) {
					this._cacheLinkStyles(linkStyles);					
				}
				
				// call orignal callback
				if (args.widgetDef) {
					onComplete(this._mergeLinkStyles(linkStyles, args.widgetDef, "link"), args);
				}
				else {
					onComplete(linkStyles, args);
				}				
			});
			this._fetchLinkStyles(args);
		}
	},
	
	/*=====
	getLinkStyle_kwArgs: function(linkType, widgetDef) {
		// linkType: String
		//		type of linkStyle
		// widgetDef: Object
		//		widget definition that can override attributes
	},
	=====*/
	
	getLinkStyle: function(/*getLinkStyle_kwArgs*/ kwArgs) {
		// summary:
		//		Returns link style for requested linkType
		var linkStyle = null;
		var args = {
			linkType: "",
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		// if we don't have link styles fetch them
		if (!this._linkStyles) {
			var fetchArgs = {
				onComplete: dojo.hitch(this, function(linkStyles, args){
					this._cacheLinkStyles(linkStyles);
				}),
				onError: function(error, exception, ioArgs){
				},
				sync: true
			};
			this._fetchLinkStyles(fetchArgs);
		}
		
		// locate requested link style in cache
		if (this._linkStylesById) {
			linkStyle = this._linkStylesById[args.linkType];
		}
		
		if (args.widgetDef) {
			// clone and merge widget definition attributes				
			if (linkStyle) {
				linkStyle = linkStyle.clone(args.widgetDef, "link");					
			}
			// unknown style to dataset; see if it exists in widget definition
			else {
				var widgetStyle = args.widgetDef.getLinkStyle(args.linkType);
				if (widgetStyle) {
					linkStyle = this._createLinkStyle(args.widgetDef, widgetStyle, "link");
				}
			}
		}
				
		return linkStyle;
	},
	
	/*=====
	relStyles_onComplete: function(relStyles, kwArgs){},
	=====*/
	
	/*=====
	getRelationshipStyles_kwArgs: function(onComplete, onError, start, count, widgetDef, sync) {
		// onComplete: relStyles_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of relationship styles to fetch; null means all
		// widgetDef: Object
		//		widget definition that can override attributes
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getRelationshipStyles: function(/*getRelationshipStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current relationship styles for the Dataset
		var args = {
			onComplete: function(relStyles, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			start: 0,
			count: null,
			widgetDef: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		// already have the relationship styles
		if (this._relStyles) {
			if (args.widgetDef) {
				if (args.sync) {
					args.onComplete(this._mergeLinkStyles(this._relStyles, args.widgetDef, "relationship"), args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, this._mergeLinkStyles(this._relStyles, args.widgetDef, "relationship"), args), 0);
				}
			}
			else if (args.sync) {
				args.onComplete(this._relStyles, args);
			}
			else {
				setTimeout(dojo.hitch(this, args.onComplete, this._relStyles, args), 0);
			}
		}
		// need to fetch the relationship styles
		else {
			var onComplete = args.onComplete;		
			args.onComplete = dojo.hitch(this, function(relStyles, args) {
				// cache styles
				if (args.start === 0 && args.count === null) {
					this._cacheRelationshipStyles(relStyles);					
				}
				
				// call orignal callback
				if (args.widgetDef) {
					onComplete(this._mergeLinkStyles(relStyles, args.widgetDef, "relationship"), args);
				}
				else {
					onComplete(relStyles, args);
				}				
			});
			this._fetchRelationshipStyles(args);
		}
	},
	
	/*=====
	getRelationshipStyle_kwArgs: function(relationshipType, widgetDef) {
		// relationshipType: String
		//		type of relationshipStyle
		// widgetDef: Object
		//		widget definition that can override attributes
	},
	=====*/

	getRelationshipStyle: function(/*getRelationshipStyle_kwArgs*/ kwArgs) {
		// summary:
		//		Returns relationship style for requested relationshipType
		var relStyle = null;
		var args = {
			relationshipType: "",
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		// if we don't have relationship styles fetch them
		if (!this._relStyles) {
			var fetchArgs = {
				onComplete: dojo.hitch(this, function(relStyles, args){
					this._cacheRelationshipStyles(relStyles);
				}),
				onError: function(error, exception, ioArgs){
				},
				sync: true
			};
			this._fetchRelationshipStyles(fetchArgs);
		}
		
		// locate requested relationship style in cache
		if (this._relStylesById) {
			relStyle = this._relStylesById[args.relationshipType];
		}
		
		if (args.widgetDef) {
			// clone and merge widget definition attributes				
			if (relStyle) {
				relStyle = relStyle.clone(args.widgetDef, "relationship");					
			}
			// unknown style to dataset; see if it exists in widget definition
			else {
				var widgetStyle = args.widgetDef.getRelationshipStyle(args.relationshipType);
				if (widgetStyle) {
					relStyle = this._createLinkStyle(args.widgetDef, widgetStyle, "relationship");
				}
			}
		}	
		
		return relStyle;
	},
		
	/*=====
	statuses_onComplete: function(statuses, kwArgs){},
	=====*/
	
	/*=====
	getStatuses_kwArgs: function(onComplete, onError, type, start, count, widgetDef, sync) {
		// onComplete: statuses_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// type: String
		//		object type: "item", "link", "relationship" 
		// start: int
		//		start index to fetch
		// count: int
		//		number of status values to fetch; null means all
		// widgetDef: Object
		//		widget definition that can override attributes
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getStatuses: function(/*getStatuses_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of possible status values for the object type
		var args = {
			onComplete: function(statuses, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			type: "item",
			start: 0,
			count: null,
			widgetDef: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		var statuses = null;
		var statusesById = null;
		switch(args.type) {
			case "item":
				statuses = this._itemStatus;
				statusesById = this._itemStatusById;
				break;
			case "link":
				statuses = this._linkStatus;
				statusesById = this._linkStatusById;
				break;
			case "relationship":
				statuses = this._relStatus;
				statusesById = this._relStatusById;			
		}
		
		// already have the status values
		if (statuses) {
			if (args.widgetDef) {
				if (args.sync) {
					args.onComplete(this._mergeStatuses(statuses, args.widgetDef, args.type), args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, this._mergeStatuses(statuses, args.widgetDef, args.type), args), 0);
				}
			}
			else if (args.sync) {
				args.onComplete(statuses, args);
			}
			else {
				setTimeout(dojo.hitch(this, args.onComplete, statuses, args), 0);
			}			
		}
		// need to fetch the status values
		else {
			var onComplete = args.onComplete;
			args.onComplete = dojo.hitch(this, function(fetchedStatuses, args) {
				// cache statuses
				if (args.start === 0 && args.count === null) {
					switch(args.type) {
						case "item":
							this._cacheItemStatuses(fetchedStatuses);
							break;
					case "link":
							this._cacheLinkStatuses(fetchedStatuses);
							break;
					case "relationship":
							this._cacheRelationshipStatuses(fetchedStatuses);
					}
				}
				
				// call orignal callback
				if (args.widgetDef) {
					onComplete(this._mergeStatuses(fetchedStatuses, args.widgetDef, args.type), args);
				}
				else {
					onComplete(fetchedStatuses, args);
				}
			});
			this._fetchStatuses(args);
		}
	},
	
	/*=====
	getStatus_kwArgs: function(type, statusId, statusValue, widgetDef) {
		// type: String
		//		object type: "item", "link", "relationship" 
		// statusId: String
		//		id of the status
		// statusValue: int
		//		value of the status
		// widgetDef: Object
		//		widget definition that can override attributes
	},
	=====*/

	getStatus: function(/*getStatus_kwArgs*/ kwArgs) {
		// summary:
		//		Returns status for requested status id and value
		var status = null;
		var args = {
			type: "item",
			statusId: "",
			statusValue: 0,
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		var statuses = null;
		switch(args.type) {
			case "item":
				statuses = this._itemStatus;
				break;
			case "link":
				statuses = this._linkStatus;
				break;
			case "relationship":
				statuses = this._relStatus;
		}
		
		// if we don't have statuses fetch them
		if (!statuses) {
			var fetchArgs = {
				onComplete: dojo.hitch(this, function(fetchedStatuses, args){
					switch(args.type) {
						case "item":
							this._cacheItemStatuses(fetchedStatuses);
							break;
						case "link":
							this._cacheLinkStatuses(fetchedStatuses);
							break;
						case "relationship":
							this._cacheRelationshipStatuses(fetchedStatuses);
					}					
				}),
				onError: function(error, exception, ioArgs){
				},
				type: args.type,
				start: 0,
				count: null,
				sync: true
			};
			this._fetchStatuses(fetchArgs);
		}
		
		var statusesById = null;
		switch(args.type) {
			case "item":
				statusesById = this._itemStatusById;
				break;
			case "link":
				statusesById = this._linkStatusById;
				break;
			case "relationship":
				statusesById = this._relStatusById;			
		}
		
		// locate requested item status in cache
		if (statusesById) {
			var id = args.statusId + "_" + args.statusValue;
			status = statusesById[id];
		}
		
		if (status && args.widgetDef) {
			status = status.clone(widgetDef, args.type);			
		}
		
		return status;
	},
	
	/*=====
	cols_onComplete: function(cols, kwArgs){},
	=====*/
	
	/*=====
	getColumns_kwArgs: function(onComplete, onError, type, properties, start, count, metadata, sync) {
		// onComplete: cols_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// type: String
		//		object type: "item", "link", "relationship" 
		// properties: String[]
		//		array of column ids; or keyword "current", "default", or "all"
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// metadata: Boolean
		//		indicates if filter metadata should be returned with columns
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	getColumns: function(/*getColumns_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current columns for the Dataset
		//		(i.e. the ones that should currently be dispayed)
		var args = {
			onComplete: function(cols, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			source: "dataset",
			type: "item",
			properties: ["current"],
			start: 0,
			count: null,
			metadata: true,
			sync: false
		};
		dojo.mixin(args, kwArgs);
				
		this._fetchColumns(args);	
	},
	
	/*=====
	setColumns_kwArgs: function(onComplete, onError, columns, type, sync) {
		// onComplete: cols_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// columns: ibm.tivoli.navmodel.Column[]
		//		array of columns to be saved
		// type: String
		//		object type: "item", "link", "relationship" 
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	setColumns: function(/*setColumns_kwArgs*/ kwArgs) {
		// summary:
		//		Set list of current columns for the Dataset
		//		(with updated attributes for width, sort, etc)
		var args = {
			onComplete: function(kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			columns: null,
			type: "item",
			sync: false
		};
		dojo.mixin(args, kwArgs);
				
		this._setColumns(args);	
	},
	
	keepAlive: function(/*keepAlive_kwArgs*/ kwArgs) {       // 255243
		// summary:
		//		Makes a request to keep a connection to a (remote) data provider alive

		var args = {
			onComplete: function(kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			sync: false
		};
		dojo.mixin(args, kwArgs);
				
		this._doKeepAlive(args);	
	},
	
	hasWidgetDefs: function() {
		// summary:
		//		Returns true if the dataset defines widget definitions	
		return (this._dataset.widgetDefinitions && dojo.isArray(this._dataset.widgetDefinitions) && this._dataset.widgetDefinitions.length > 0);
	},
	
	/*=====
	getWidgetDef_kwArgs: function() {
		// widgetType: String
		//		the type of widget def (i.e. "navwidget", "topology", "table", etc)
	},
	=====*/
	
	getWidgetDef: function(/*getWidgetDef_kwArgs*/ kwArgs) {
		// summary:
		//		Returns widget definition for the requested widget type
		var args = {
			widgetType: null
		};
		dojo.mixin(args, kwArgs);
		
		return this._getWidgetDef(this._getWidgetDefIndex(args.widgetType));		
	},
	
	getWidgetDefs: function() {
		// summary:
		//		Returns all widget definitions defined by the dataset
		var widgetDefs = null;		
		if (this.hasWidgetDefs()) {
			widgetDefs = [];			
			var length = this._dataset.widgetDefinitions.length;
			for (var index = 0;  index < length;  ++index) {
				var widgetDef = this._getWidgetDef(index);
				if (widgetDef) {
					widgetDefs.push(widgetDef);
				}
			}
		}
		
		return widgetDefs;
	},
	
	/*=====
	item_onComplete: function(item, kwArgs){},
	=====*/
	
	/*=====
	createItem_kwArgs: function(onComplete, onError, item) {
		// onComplete: item_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// item: Object
		//		ibm.tivoli.navmodel.Item to be created on server
	},
	=====*/

	createItem: function(/*createItem_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new item in the dataset
		var args = {
			onComplete: function(item, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			item: null
		};
		dojo.mixin(args, kwArgs);
		
		this._createItem(args);
	},
	
	/*=====
	createItems_kwArgs: function(onComplete, onError, item) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// item: Object[]
		//		array of ibm.tivoli.navmodel.Item to be created on server
	},
	=====*/

	createItems: function(/*createItems_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new items in the dataset
		var args = {
			onComplete: function(items, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			items: []
		};
		dojo.mixin(args, kwArgs);
		
		this._createItems(args);
	},
	
	/*=====
	_onComplete: function(kwArgs){},
	=====*/
	
	/*=====
	deleteItem_kwArgs: function(onComplete, onError, itemId, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// itemId: String
		//		id of item to be deleted
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of item
	},
	=====*/

	deleteItem: function(/*deleteItem_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a item from the dataset
		var args = {
			onComplete: function(kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			itemId: null,
			remove: false
		};
		dojo.mixin(args, kwArgs);
		
		this._deleteItem(args);
	},
	
	/*=====
	deleteItems_kwArgs: function(onComplete, onError, itemIds, condition, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// itemIds: String[]
		//		array of item ids to be deleted
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of items
	},
	=====*/

	deleteItems: function(/*deleteItems_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete items from the dataset
		var args = {
			onComplete: function(kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			itemIds: [],
			condition: null,
			remove: false
		};
		dojo.mixin(args, kwArgs);
		
		this._deleteItems(args);
	},
	
	/*=====
	link_onComplete: function(link, kwArgs){},
	=====*/
	
	/*=====
	createLink_kwArgs: function(onComplete, onError, link) {
		// onComplete: link_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// link: Object
		//		ibm.tivoli.navmodel.Link to be created on server
	},
	=====*/

	createLink: function(/*createLink_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new link in the dataset
		var args = {
			onComplete: function(link, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			link: null
		};
		dojo.mixin(args, kwArgs);
		
		this._createLink(args);	
	},
	
	/*=====
	createLinks_kwArgs: function(onComplete, onError, links) {
		// onComplete: links_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// links: Object[]
		//		array of ibm.tivoli.navmodel.Link to be created on server
	},
	=====*/

	createLinks: function(/*createLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new links in the dataset
		var args = {
			onComplete: function(links, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			links: []
		};
		dojo.mixin(args, kwArgs);
		
		this._createLinks(args);	
	},
	
	/*=====
	deleteLink_kwArgs: function(onComplete, onError, linkId, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// linkId: String
		//		id of link to be deleted
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of link
	},
	=====*/

	deleteLink: function(/*deleteLink_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a link from the dataset
		var args = {
			onComplete: function(kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			linkId: null,
			remove: false
		};
		dojo.mixin(args, kwArgs);
		
		this._deleteLink(args);
	},
	
	/*=====
	deleteLinks_kwArgs: function(onComplete, onError, linkIds, condition, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// linkIds: String[]
		//		array of link ids to be deleted
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of links
	},
	=====*/

	deleteLinks: function(/*deleteLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete links from the dataset
		var args = {
			onComplete: function(kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			linkIds: [],
			condition: null,
			remove: false
		};
		dojo.mixin(args, kwArgs);
		
		this._deleteLinks(args);
	},
	
	/*=====
	relationship_onComplete: function(relationship, kwArgs){},
	=====*/
	
	/*=====
	createRelationship_kwArgs: function(onComplete, onError, relationship) {
		// onComplete: relationship_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationship: Object
		//		ibm.tivoli.navmodel.Relationship to be created on server
	},
	=====*/
	
	createRelationship: function(/*createRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new relationship in the dataset
		var args = {
			onComplete: function(relationship, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			relationship: null
		};
		dojo.mixin(args, kwArgs);
		
		this._createRelationship(args);	
	},
	
	/*=====
	createRelationships_kwArgs: function(onComplete, onError, relationships) {
		// onComplete: relationships_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationship: Object[]
		//		array of ibm.tivoli.navmodel.Relationship to be created on server
	},
	=====*/
	
	createRelationships: function(/*createRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new relationships in the dataset
		var args = {
			onComplete: function(relationships, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			relationships: []
		};
		dojo.mixin(args, kwArgs);
		
		this._createRelationships(args);	
	},
	
	/*=====
	deleteRelationship_kwArgs: function(onComplete, onError, relationshipId, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationshipId: String
		//		id of relationship to be deleted
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of relationship
	},
	=====*/

	deleteRelationship: function(/*deleteRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a relationship from the dataset
		var args = {
			onComplete: function(kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			relatonshipId: null,
			remove: false
		};
		dojo.mixin(args, kwArgs);
		
		this._deleteRelationship(args);
	},
	
	/*=====
	deleteRelationships_kwArgs: function(onComplete, onError, relationshipIds, condition, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationshipIds: String[]
		//		array of relationship ids to be deleted
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of relationships
	},
	=====*/

	deleteRelationships: function(/*deleteRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete relationships from the dataset
		var args = {
			onComplete: function(kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			relatonshipIds: [],
			condition: null,
			remove: false
		};
		dojo.mixin(args, kwArgs);
		
		this._deleteRelationships(args);
	},

	clientChangedEvent: function() {
		// summary:
		//		Fire client side "changed" event
		var event = ibm.tivoli.navmodel.ClientEvent.getChangedEvent(this);
		this._eventHandler(event);	
	},

	clientItemsAddedEvent: function(items) {
		// summary:
		//		Fire client side "itemsAdded" event
		var event = ibm.tivoli.navmodel.ClientEvent.getItemsAddedEvent(this, items);
		this._eventHandler(event);
	},

	clientItemsRemovedEvent: function(items) {
		// summary:
		//		Fire client side "itemsRemoved" event
		var event = ibm.tivoli.navmodel.ClientEvent.getItemsRemovedEvent(this, items);
		this._eventHandler(event);
	},

	clientItemStatusChangedEvent: function(items) {
		// summary:
		//		Fire client side "itemStatusChanged" event
		var event = ibm.tivoli.navmodel.ClientEvent.getItemStatusChangedEvent(this, items);
		this._eventHandler(event);	
	},

	clientItemPropertiesChangedEvent: function(items) {
		// summary:
		//		Fire client side "itemPropertiesChanged" event
		var event = ibm.tivoli.navmodel.ClientEvent.getItemPropertiesChangedEvent(this, items);
		this._eventHandler(event);	
	},

	clientLinksAddedEvent: function(links) {
		// summary:
		//		Fire client side "linksAdded" event
		var event = ibm.tivoli.navmodel.ClientEvent.getLinksAddedEvent(this, items);
		this._eventHandler(event);
	},

	clientLinksRemovedEvent: function(links) {
		// summary:
		//		Fire client side "linksRemoved" event
		var event = ibm.tivoli.navmodel.ClientEvent.getLinksRemovedEvent(this, links);
		this._eventHandler(event);
	},

	clientLinkStatusChangedEvent: function(links) {
		// summary:
		//		Fire client side "linkStatusChanged" event
		var event = ibm.tivoli.navmodel.ClientEvent.getLinkStatusChangedEvent(this, links);
		this._eventHandler(event);
	},

	clientLinkPropertiesChangedEvent: function(links) {
		// summary:
		//		Fire client side "linkPropertiesChanged" event
		var event = ibm.tivoli.navmodel.ClientEvent.getLinkPropertiesChangedEvent(this, links);
		this._eventHandler(event);
	},

	clientLabelChangedEvent: function(newLabel) {
		// summary:
		//		Fire client side "labelChanged" event
		var event = ibm.tivoli.navmodel.ClientEvent.getLinkPropertiesChangedEvent(this, newLabel);
		this._eventHandler(event);	
	},
		
	/////////////////////////////
	// protected methods
	/////////////////////////////	
	_initEmptyDataset: function() {
		// summary:
		//		Reset internal dataset to an empty state
		this._version = this._args.version;		
		this._dataset = {};
		this._dataset.providerId = this._args.provider;
		this._dataset.datasourceId = this._args.datasource;
      this._dataset.datasetImplementation = this._args.datasetImplementation;    // 255243
		this._dataset.id = this._args.dataset;
		this._dataset.eventId = this._dataset.id;
		this._dataset.eventsEnabled = false;
		this._dataset.viewType = "graph";		
		this._dataset.itemCount = 0;
		this._dataset.linkCount = 0;
		this._dataset.relationshipCount = 0;
      this._dataset.remote = this._args.remote;   // 255243
		
		// establish our base dataset uri
		this._dataset.uri  = "/providers/" + encodeURIComponent(this._dataset.providerId);
		this._dataset.uri += (this._dataset.datasourceId !== "" ? ("/datasources/" + encodeURIComponent(this._dataset.datasourceId)) : "");
		this._dataset.uri += "/datasets/" + encodeURIComponent(this._dataset.id);		
	},

	_cleanupDataset: function() {
		// summary:
		//		Clean up by removing listener and attempting a deleteDataset if no other listeners
		
		// unregister from event router
		if (this._eventRouter) {
			var remainingListeners = this._eventRouter.removeListener(this.getEventId());
				
			// ok to delete dataset from REST service cache
			if (remainingListeners === 0) {
				this._deleteDataset();
			}
			
			this._eventRouter = null;
		}
		// no event router;  just do the delete
		else {
			this._deleteDataset();
		}
		
		// clear all item and link caches
		this._clearAllCaches();		
		this._initEmptyDataset();
	},

	_getWidgetDefIndex: function(/*String*/ widgetType) {
		// summary:
		//		Returns the index into widgetDefinitions array for the requested widgetType		
		if (widgetType && this.hasWidgetDefs()) {
			var length = this._dataset.widgetDefinitions.length;
			for (var index = 0;  index < length;  ++index) {
				var entry = this._dataset.widgetDefinitions[index];
				if (entry) {
					// entry should be in the form of  <widget type>:<widget id>:<widget class>
					var parts = entry.split(":");
					if (parts && parts.length == 3 && parts[0] === widgetType) {
						return index;
					}
				}
			}
		}
		
		return -1;
	},
	
	_getWidgetDef: function(/*int*/ index) {
		// summary:
		//		Return WidgetDef for the requested index
		if (!this._widgetDefs && this.hasWidgetDefs()) {
			this._widgetDefs = [];			
			var widgetDefsLength = this._dataset.widgetDefinitions.length;
			for (var widgetDefIndex = 0;  widgetDefIndex < widgetDefsLength;  ++widgetDefIndex) {
				this._widgetDefs[widgetDefIndex] = null;
			}
		}
		
		if (!this._widgetDefs || index < 0 || index >= this._widgetDefs.length) {
			return null;
		}
		
		if (!this._widgetDefs[index]) {
			var parts = this._dataset.widgetDefinitions[index].split(":");
			var widgetType = parts[0];
			var widgetId = parts[1];
			var widgetClass = parts[2];
				
			var objectClass = dojo.getObject(widgetClass);
			if (objectClass) {
				var args = {
					baseUrl: this._args.baseUrl,
					provider: this.getProviderId(),
					datasource: this.getDatasourceId(),
					widget: widgetType,
					definition: widgetId,
					type: "user",
					version: this.getVersion()
				};					
					
				try {
					this._widgetDefs[index] = new objectClass(args);
				}
				catch(ex) {
					this._trace.error("Exception constructing widget definition: ex=" + ex, "_getWidgetDef");					
				}
			}
			else {
				this._trace.error("Cannot instantiate widget def class=" + widgetClass, "_initWidgetDef");				
			}
		}
			
		return this._widgetDefs[index];		
	},

	_setEventRouter: function(eventRouter) {
		// summary:
		//		Associate our dataset with an existing EventRouter object
		if (eventRouter && this.isEventsEnabled()) {
			this._eventRouter = eventRouter;
			this._eventRouter.addListener(this.getEventId(), dojo.hitch(this, this._jsonEventHandler));
		}
	},

	_clearEventRouter: function() {
		// summary:
		//		Clear EventRouter for our dataset
		if (this._eventRouter) {
			this._eventRouter.removeListener(this.getEventId());
			this._eventRouter = null;
		}
	},

	_getEventRouter: function() {
		// summary:
		//		Returns current event router
		return this._eventRouter;
	},
	
	_setItemCount: function(itemCount) {
		// summary:
		//		Set internal dataset item count
		this._dataset.itemCount = itemCount;
	},

	_setLinkCount: function(linkCount) {
		// summary:
		//		Set internal dataset link count
		this._dataset.linkCount = linkCount;
	},
	
	_setRelationshipCount: function(relationshipCount) {
		// summary:
		//		Set internal dataset relationship count
		this._dataset.relationshipCount = relationshipCount;
	},

	_clearAllCaches: function() {
		// summary:
		//		Clear all item and link related caches
		this._clearItemCache();
		this._clearLinkCache();
		this._clearRelationshipCache();
		this._clearToolbarCache();
		this._clearItemStyleCache();
		this._clearItemStatusCache();
		this._clearLinkStyleCache();
		this._clearLinkStatusCache();
		this._clearRelationshipStyleCache();
		this._clearRelationshipStatusCache();
	},

	_clearItemCache: function() {
		// summary:
		//		Clear the item cache
		this._itemsById = {};		
	},
	
	_clearLinkCache: function() {
		// summary:
		//		Clear the link cache
		this._linksById = {};
	},

	_clearRelationshipCache: function() {
		// summary:
		//		Clear the relationship cache
		this._relationshipsById = {};
	},

	_clearToolbarCache: function() {
		// summary:
		//		Clear the "toolbar" cache
		this._toolbar = null;
	},

	_clearItemStyleCache: function() {
		// summary:
		//		Clear item style cache
		this._itemStyles = null;
		this._itemStylesById = null;
	},

	_clearItemStatusCache: function() {
		// summary:
		//		Clear item status cache
		this._itemStatus = null;
		this._itemStatusById = null;
	},

	_clearLinkStyleCache: function() {
		// summary:
		//		Clear link style cache
		this._linkStyles = null;
		this._linkStylesById = null;
	},

	_clearLinkStatusCache: function() {
		// summary:
		//		Clear link status cache
		
		//		protected
		this._linkStatus = null;
		this._linkStatusById = null;
	},

	_clearRelationshipStyleCache: function() {
		// summary:
		//		Clear relationship style cache
		this._relStyles = null;
		this._relStylesById = null;
	},

	_clearRelationshipStatusCache: function() {
		// summary:
		//		Clear relationship status cache
		this._relStatus = null;
		this._relStatusById = null;
	},

	_cacheItems: function(items, resetLazyLoad) {
		// summary:
		//		Cache items; merge if already cached
		if (items) {
			var itemsLength = items.length;
			for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
				items[itemIndex] = this._cacheItem(items[itemIndex], resetLazyLoad);
			}
		}
		
		return items;
	},

	_cacheItem: function(item, resetLazyLoad) {
		// summary:
		//		Cache item; merge if already cached
		var cachedItem = null;
		
		if (item) {
			// check item cache		
			cachedItem = this._getCachedItem(item.getId());
			
			// item is already in cache; merge property values
			if (cachedItem) {
				cachedItem.mixin(item, resetLazyLoad);
				
				// reset expiration to (now + interval) since item was just referenced
				if (this.cacheExpires()) {
					cachedItem.resetExpiration();
				}		
			}
			// cache new item
			else {
				this._itemsById[item.getId()] = item;
				cachedItem = item;				
			}
		}
		
		return cachedItem;
	},

	_getCachedItem: function(itemId) {
		// summary:
		//		look up item in cache
		return this._itemsById[itemId];
	},

	_cacheLinks: function(links) {
		// summary:
		//		Cache links; merge if already cached
		if (links) {
			var linksLength = links.length;
			for (var linkIndex = 0;  linkIndex < linksLength;  ++linkIndex) {
				links[linkIndex] = this._cacheLink(links[linkIndex]);
			}
		}
		
		return links;
	},

	_cacheLink: function(link) {
		// summary:
		//		Cache link; merge if already cached
		var cachedLink = null;
		
		if (link) {
			// check link cache		
			cachedLink = this._getCachedLink(link.getId());
			
			// link is already in cache; merge property values
			if (cachedLink) {
				cachedLink.mixin(link);
				
				// reset expiration to (now + interval) since link was just referenced
				if (this.cacheExpires()) {
					cachedLink.resetExpiration();
				}		
			}
			// cache new link
			else {
				this._linksById[link.getId()] = link;
				cachedLink = link;				
			}
		}
		
		return cachedLink;
	},

	_getCachedLink: function(linkId) {
		// summary:
		//		look up link in cache
		return this._linksById[linkId];
	},

	_cacheRelationships: function(relationships) {
		// summary:
		//		Cache relationships; merge if already cached
		if (relationships) {
			var relsLength = relationships.length;
			for (var relIndex = 0;  relIndex < relsLength;  ++relIndex) {
				relationships[relIndex] = this._cacheRelationship(relationships[relIndex]);				
			}
		}
		
		return relationships;
	},

	_cacheRelationship: function(relationship) {
		// summary:
		//		Cache relationship; merge if already cached
		var cachedRelationship = null;
		
		if (relationship) {
			// check relationship cache		
			cachedRelationship = this._getCachedRelationship(relationship.getId());
			
			// relationship is already in cache; merge property values
			if (cachedRelationship) {
				cachedRelationship.mixin(relationship);
				
				// reset expiration to (now + interval) since relationship was just referenced
				if (this.cacheExpires()) {
					cachedRelationship.resetExpiration();
				}			
			}
			// cache new relationship
			else {
				this._relationshipsById[relationship.getId()] = relationship;
				cachedRelationship = relationship;
			}
		}
		
		return cachedRelationship;
	},

	_getCachedRelationship: function(relationshipId) {
		// summary:
		//		look up relationship in cache
		return this._relationshipsById[relationshipId];
	},

	_cacheItemStyles: function(styles) {
		// summary:
		//		Set item style cache
		var stylesById = {};
		if (styles) {
			var stylesLength = styles.length;
			for (var styleIndex = 0;  styleIndex < stylesLength;  ++styleIndex) {
				var style = styles[styleIndex];
				stylesById[style.getType()] = style;
			}
		}
		
		this._itemStylesById = stylesById;
		this._itemStyles = styles;
	},

	_cacheItemStatuses: function(statuses) {
		// summary:
		//		Set item status cache
		var statusesById = {};
		if (statuses) {
			var statusLength = statuses.length;
			for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
				var id = statuses[statusIndex].getId() + "_" + statuses[statusIndex].getValue();
				statusesById[id] = statuses[statusIndex];
			}
		}
		
		this._itemStatusById = statusesById;
		this._itemStatus = statuses;	
	},

	_cacheLinkStyles: function(styles) {
		// summary:
		//		Set link style cache
		var stylesById = {};
		if (styles) {
			var stylesLength = styles.length;
			for (var styleIndex = 0;  styleIndex < stylesLength;  ++styleIndex) {
				var style = styles[styleIndex];
				stylesById[style.getType()] = style;
			}
		}
		
		this._linkStylesById = stylesById;
		this._linkStyles = styles;
	},

	_cacheLinkStatuses: function(statuses) {
		// summary:
		//		Set link status cache
		var statusesById = {};
		if (statuses) {
			var statusLength = statuses.length;
			for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
				var id = statuses[statusIndex].getId() + "_" + statuses[statusIndex].getValue();
				statusesById[id] = statuses[statusIndex];
			}
		}
		
		this._linkStatusById = statusesById;
		this._linkStatus = statuses;
	},

	_cacheRelationshipStyles: function(styles) {
		// summary:
		//		Set relationship style cache
		var stylesById = {};
		if (styles) {
			var stylesLength = styles.length;
			for (var styleIndex = 0;  styleIndex < stylesLength;  ++styleIndex) {
				var style = styles[styleIndex];
				stylesById[style.getType()] = style;
			}
		}
		
		this._relStylesById = stylesById;
		this._relStyles = styles;
	},
	
	_cacheRelationshipStatuses: function(statuses) {
		// summary:
		//		Set relationship status cache
		var statusesById = {};
		if (statuses) {
			var statusLength = statuses.length;
			for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
				var id = statuses[statusIndex].getId() + "_" + statuses[statusIndex].getValue();
				statusesById[id] = statuses[statusIndex];
			}
		}
		
		this._relStatusById = statusesById;
		this._relStatus = statuses;
	},

	_mergeItemStyles: function(itemStyles, widgetDef) {
		// summary:
		//		Merge item styles with any pertinent information from widget definition
		var mergedStyles = itemStyles;
				
		// widget definition was provided
		if (widgetDef) {
			mergedStyles = itemStyles.slice(0);
			
			var styleMap = {};
			var styleLength = itemStyles.length;
			for (var styleIndex = 0;  styleIndex < styleLength;  ++styleIndex) {
				styleMap[itemStyles[styleIndex].getType()] = itemStyles[styleIndex];
				mergedStyles[styleIndex] = itemStyles[styleIndex].clone(widgetDef);					
			}
			
			// see if any styles are in widget definition that are not known to dataset
			var widgetStyles = widgetDef.getItemStyles();
			if (widgetStyles) {
				styleLength = widgetStyles.length;
				for (styleIndex = 0;  styleIndex < styleLength;  ++styleIndex) {
					var widgetStyle = widgetStyles[styleIndex];
					if (!styleMap[widgetStyle.type]) {
						mergedStyles.push(this._createItemStyle(widgetDef, widgetStyle));
					}
				}
			}			
		}
		
		return mergedStyles;
	},

	_createItemStyle: function(widgetDef, widgetStyle) {
		// summary:
		//		Create an ItemStyle from information in the widget definition
		var styleAttributes = {};
		var widgetDefaultStyle = widgetDef.getDefaultItemStyle();
		if (widgetDefaultStyle) {
			dojo.mixin(styleAttributes, widgetDefaultStyle);
		}
							
		dojo.mixin(styleAttributes, widgetStyle);
					
		return new ibm.tivoli.navmodel.ItemStyle({style: styleAttributes, version: this.getVersion()});
	},

	_mergeLinkStyles: function(linkStyles, widgetDef, objType) {
		// summary:
		//		Merge link styles with any pertinent information from widget definition
		var mergedStyles = linkStyles;
		
		// widget definition was provided
		if (widgetDef) {
			mergedStyles = linkStyles.slice(0);
			
			var styleMap = {};
			var styleLength = linkStyles.length;
			for (var styleIndex = 0;  styleIndex < styleLength;  ++styleIndex) {
				styleMap[linkStyles[styleIndex].getType()] = linkStyles[styleIndex];
				mergedStyles[styleIndex] = linkStyles[styleIndex].clone(widgetDef, objType);				
			}
			
			// see if any styles are in widget definition that are not known to dataset
			var widgetStyles = widgetDef.getLinkStyles();
			if (widgetStyles) {
				styleLength = widgetStyles.length;
				for (styleIndex = 0;  styleIndex < styleLength;  ++styleIndex) {
					var widgetStyle = widgetStyles[styleIndex];
					if (!styleMap[widgetStyle.type]) {
						mergedStyles.push(this._createLinkStyle(widgetDef, widgetStyle, objType));
					}
				}
			}			
		}
		
		return mergedStyles;
	},

	_createLinkStyle: function(widgetDef, widgetStyle, objType) {
		// summary:
		//		Create a LinkStyle from information in the widget definition
		var styleAttributes = {};
		var widgetDefaultStyle = null;
		switch(objType) {
			case "link":
				widgetDefaultStyle = widgetDef.getDefaultLinkStyle();
				break;
			case "relationship":
				widgetDefaultStyle = widgetDef.getDefaultRelationshipStyle();
		}
		
		if (widgetDefaultStyle) {
			dojo.mixin(styleAttributes, widgetDefaultStyle);
		}
							
		dojo.mixin(styleAttributes, widgetStyle);
					
		return new ibm.tivoli.navmodel.LinkStyle({style: styleAttributes, version: this.getVersion()});
	},

	_mergeStatuses: function(status, widgetDef, objType) {
		// summary:
		//		Merge status with any pertinent information from widget definition
		var mergedStatus = status;
		
		// widget definition was provided
		if (widgetDef) {
			mergedStatus = status.slice(0);
			
			var statusMap = {};
			var statusLength = status.length;
			for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
				statusMap[status[statusIndex].getId()] = status[statusIndex];
				mergedStatus[statusIndex] = status[statusIndex].clone(widgetDef, objType);
			}
			
			// see if any status values are in widget definition that are not known to dataset
			var widgetStatuses = null;
			switch(objType) {
				case "item":
					widgetStatuses = widgetDef.getItemStatuses();
					break;
				case "link":
					widgetStatuses = widgetDef.getLinkStatuses();
					break;
				case "relationship":
					widgetStatuses = widgetDef.getRelationshipStatuses();
			}
			
			if (widgetStatuses) {
				statusLength = widgetStatuses.length;
				for (statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
					var widgetStatus = widgetStatuses[statusIndex];
					if (!statusMap[widgetStatus.id]) {
						mergedStatus.push(this._createStatus(widgetDef, widgetStatus, objType));
					}
				}
			}
		}
		
		return mergedStatus;
	},

	_createStatus: function(widgetDef, widgetStatus, objType) {
		// summary:
		//		Create an Status from information in the widget definition
		var statusAttributes = {};
		var widgetDefaultStatus = null;
		switch(objType) {
			case "item":
				widgetDefaultStatus = widgetDef.getDefaultItemStatus();
				break;
			case "link":
				widgetDefaultStatus = widgetDef.getDefaultLinkStatus();
				break;
			case "relationship":
				widgetDefaultStatus = widgetDef.getDefaultRelationshipStatus();
		}
		
		if (widgetDefaultStatus) {
			dojo.mixin(statusAttributes, widgetDefaultStatus);
		}
							
		dojo.mixin(statusAttributes, widgetStatus);
					
		return new ibm.tivoli.navmodel.Status({status: statusAttributes, version: this.getVersion()});
	},
		
	_notifyEventListeners: function(event) {
		// summary:
		//		Pass ibm.tivoli.navmodel.Event to all registered listeners
		if (this._listeners) {
			var listenerLength = this._listeners.length;
			for (var listenerIndex = 0;  listenerIndex < listenerLength;  ++listenerIndex) {
				try {
					var listenerObj = this._listeners[listenerIndex].listenerObj;
					this._listeners[listenerIndex].listenerCallback(listenerObj, event);
				}
				catch (ex) {
					this._trace.error("Exception forwarding eventId=" + event.getId() + " type=" + event.getType() + " ex=" + ex, "_sendEvent");
				}
			}				
		}		
	},
	
	_jsonEventHandler: function(jsonEventObj) {
		// summary:
		//		This is the routine that we register with the EventRouter to handle
		//		raw json events that come from the server
		var event = new ibm.tivoli.navmodel.Event({dataset: this, event: jsonEventObj});
		this._eventHandler(event);
	},

	_eventHandler: function(event) {
		// summary:
		//		Handles ibm.tivoli.navmodel.Event objects
		if (event.getId() === this.getEventId()) {
			if (this._trace.isFine()) {
				this._trace.fine("datasetId=" + this.getId() + " event=" + event.getType() + " eventId=" + event.getId(), "_eventHandler");
			}
			
			switch(event.getType()) {
				case "changed":
					this._handleChangedEvent(event);
					break;
				case "labelChanged":
					this._handleLabelChangedEvent(event);
					break;
				case "statusChanged":
					this._handleStatusChangedEvent(event);
					break;					
				case "itemsAdded":
					this._handleItemsAddedEvent(event);
					break;
				case "itemsRemoved":
					this._handleItemsRemovedEvent(event);
					break;
				case "itemStatusChanged":
					this._handleItemStatusChangedEvent(event);
					break;
				case "itemPropertiesChanged":
					this._handleItemPropertiesChangedEvent(event);
					break;
				case "linksAdded":
					this._handleLinksAddedEvent(event);
					break;
				case "linksRemoved":
					this._handleLinksRemovedEvent(event);
					break;
				case "linkStatusChanged":
					this._handleLinkStatusChangedEvent(event);
					break;
				case "linkPropertiesChanged":
					this._handleLinkPropertiesChangedEvent(event);
					break;
				case "linkChanged":
					this._handleLinkChangedEvent(event);
					break;
				case "relationshipStatusChanged":
					this._handleRelationshipStatusChangedEvent(event);
					break;
				case "relationshipPropertiesChanged":
					this._handleRelationshipPropertiesChangedEvent(event);
					break;
				case "exception":
					if (this._trace.isFine()) {
						this._trace.fine("	exception msgText=" + event.getException().getMsgText(), "_eventHandler");
					}
					break;
				case "eventRouter":
					if (this._trace.isFine()) {
						this._trace.fine("	eventRouterStatus=" + event.getEventRouterStatus(), "_eventHandler");
					}
			}
			
			this._notifyEventListeners(event);
		}		
	},

	_handleChangedEvent: function(event) {
		// summary:
		//		Handle "changed" event type
		this._clearAllCaches();
		
		var initArgs = {
			optimize: true,
			createEventRouter: event.createEventRouter,
			fetchMetaData: true			
		};
		this._initDataset(initArgs);
	},
	
	_handleLabelChangedEvent: function(event) {
		// summary:
		//		Handle "labelChanged" event type
		this._dataset.label = event._getEventNewLabel();
		
		if (this._trace.isFinest()) {
			this._trace.finest("datasetId=" + this.getId() + " label=" + this._dataset.label, "_handleLabelChangedEvent");
		}
	},

	_handleStatusChangedEvent: function(event) {
		// summary:
		//		Handle "statusChanged" event type
		var eventStatus = event._getEventStatuses();
		if (eventStatus) {		
			var status = [];
			var statusLength = eventStatus.length;
			for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
				status[statusIndex] = new ibm.tivoli.navmodel.Status({status: eventStatus[statusIndex], version: this.getVersion()});
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + this.getId() + " status=" + status[statusIndex].getLabel(), "_handleStatusChangedEvent");
				}
			}
		}
	},

	_handleItemsAddedEvent: function(event) {
		// summary:
		//		Handle "itemsAdded" event type
		
		// get new item count for dataset
		var itemCount = event._getEventDatasetItemCount();
		if (itemCount !== undefined) {
			this._setItemCount(itemCount);
		}
		
		// might have new list of styles
		this._clearItemStyleCache();
			
		var eventItems = event._getEventItems();
		if (eventItems) {
			var itemsLength = eventItems.length;
			for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + this.getId() + " itemId=" + eventItems[itemIndex].id, "_handleItemsAddedEvent");
				}
				
				var eventItem = eventItems[itemIndex];
				var newItem = new ibm.tivoli.navmodel.Item({item: eventItem, dataset: this});
				newItem = this._cacheItem(newItem);													
				event._addItem(newItem);				
			}					
		}
	},

	_handleItemsRemovedEvent: function(event) {
		// summary:
		//		Handle "itemsRemoved" event type
		
		// get new item count for dataset
		var itemCount = event._getEventDatasetItemCount();
		if (itemCount !== undefined) {
			this._setItemCount(itemCount);
	    	if (itemCount === 0) {
	    		this._clearItemCache();
			}
		}
		
		// might have new list of styles
		this._clearItemStyleCache();
			
		var eventItems = event._getEventItems();
		if (eventItems) {
			var itemsLength = eventItems.length;
			for (var itemIndex = itemsLength - 1;  itemIndex >= 0;  --itemIndex) {
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + this.getId() + " itemId=" + eventItems[itemIndex].id, "_handleItemsRemovedEvent");
				}
				
				var eventItem = eventItems[itemIndex];
				var deletedItem = new ibm.tivoli.navmodel.Item({item: eventItem, dataset: this});		
				var item = this._getCachedItem(eventItem.id);
				if (item) {
					delete this._itemsById[item.getId()];
					event._addItem(item);
				}
				else {	
					event._addItem(deletedItem);
				}			
			}	
		}
	},

	_handleItemStatusChangedEvent: function(event) {
		// summary:
		//		Handle "itemStatusChanged" event type
		var eventItems = event._getEventItems();
		if (eventItems) {
			var itemsLength = eventItems.length;
			for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
				var eventItem = eventItems[itemIndex];	
				var item = new ibm.tivoli.navmodel.Item({item: eventItem, dataset: this});		
				item = this._cacheItem(item);					
				event._addItem(item);
				
				if (this._trace.isFinest()) {
					var status = item.getStatuses({widgetDef: null});
					if (status) {
						var statusLength = status.length;
						for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
							this._trace.finest("datasetId=" + this.getId() + " itemId=" + eventItem.id + " status=" + status[statusIndex].getLabel(), "_handleItemStatusChangedEvent");
						}					
					}
				}			
			}
		}
	},

	_handleItemPropertiesChangedEvent: function(event) {
		// summary:
		//		Handle "itemPropertiesChanged" event type
		var eventItems = event._getEventItems();
		if (eventItems) {
			var itemsLength = eventItems.length;
			for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
				var eventItem = eventItems[itemIndex];							
				var props = [];
				var properties = eventItem.properties;					
				if (properties) {
					var propsLength = properties.length;
					for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
						var property = new ibm.tivoli.navmodel.Property({property: properties[propIndex], dataset: this, objectType: "item"});
						props[propIndex] = property;						
						if (this._trace.isFinest()) {
							this._trace.finest("datasetId=" + this.getId() + " itemId=" + eventItem.id + " prop=" + property.getId() + " value=" + property.getDisplayValue(), "_handleItemPropertiesChangedEvent");
						}																	
					}
				}
								
				var item = new ibm.tivoli.navmodel.Item({item: eventItem, dataset: this});				
				item = this._cacheItem(item);					
				event._addItem(item, props);				
			}
		}
	},

	_handleLinksAddedEvent: function(event) {
		// summary:
		//		Handle "linksAdded" event type
		
		// get new link and relationship counts for dataset
		var linkCount = event._getEventDatasetLinkCount();
		var relCount = event._getEventDatasetRelationshipCount();
		if (linkCount !== undefined) {
			this._setLinkCount(linkCount);
		}
		if (relCount !== undefined) {
			this._setRelationshipCount(relCount);
		}
		
		// might have new list of styles
		this._clearLinkStyleCache();
		this._clearRelationshipStyleCache(); 
			
		var eventLinks = event._getEventLinks();
		if (eventLinks) {
		    var eventLinksLength = eventLinks.length;
			for (var linkIndex = 0;  linkIndex < eventLinksLength;  ++linkIndex) {
				var eventLink = eventLinks[linkIndex];
				var newLink = new ibm.tivoli.navmodel.Link({link: eventLink, dataset: this});
				newLink = this._cacheLink(newLink);				
				event._addLink(newLink);
				
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + this.getId() + " linkId=" + newLink.getId(), "_handleLinksAddedEvent");
				}				
			}
		}
	},

	_handleLinksRemovedEvent: function(event) {
		// summary:
		//		Handle "linksRemoved" event type
		
		// get new link count for dataset
		var linkCount = event._getEventDatasetLinkCount();
		var relCount = event._getEventDatasetRelationshipCount();
		if (linkCount !== undefined) {
			this._setLinkCount(linkCount);
			if (linkCount === 0) {
	    		this._clearLinkCache();	    		
			}
		}
		if (relCount !== undefined) {
		 	this._setRelationshipCount(relCount);			
		}
		
		// might have new list of styles
		this._clearLinkStyleCache();
		this._clearRelationshipStyleCache();
		    
		var eventLinks = event._getEventLinks();
		if (eventLinks) {
		    var eventLinksLength = eventLinks.length;
			for (var linkIndex = eventLinksLength - 1;  linkIndex >= 0;  --linkIndex) {
				var eventLink = eventLinks[linkIndex];
				var deletedLink = new ibm.tivoli.navmodel.Link({link: eventLink, dataset: this});
				var link = this._getCachedLink(eventLink.id);
				if (link) {
					delete this._linksById[link.getId()];
					event._addLink(link);
				}
				else {	
					event._addLink(deletedLink);
				}
				
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + this.getId() + " linkId=" + deletedLink.getId(), "_handleLinksRemovedEvent");
				}	
			}
		}
	},
	
	_handleLinkStatusChangedEvent: function(event) {
		// summary:
		//		Handle "linkStatusChanged" event type
		var eventLinks = event._getEventLinks();
		if (eventLinks) {
			var eventLinksLength = eventLinks.length;
			for (var linkIndex = 0;  linkIndex < eventLinksLength;  ++linkIndex) {
				var eventLink = eventLinks[linkIndex];
				var link = new ibm.tivoli.navmodel.Link({link: eventLink, dataset: this});
				link = this._cacheLink(link);					
				event._addLink(link);
				
				if (this._trace.isFinest()) {
					var status = link.getStatuses({widgetDef: null});
					if (status) {
						var statusLength = status.length;
						for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
							this._trace.finest("datasetId=" + this.getId() + " linkId=" + link.getId() + " status=" + status[statusIndex].getLabel(), "_handleLinkStatusChangedEvent");
						}					
					}
				}		
			}
		}
	},

	_handleLinkPropertiesChangedEvent: function(event) {
		// summary:
		//		Handle "linkPropertiesChanged" event type
		var eventLinks = event._getEventLinks();
		if (eventLinks) {
			var eventLinksLength = eventLinks.length;
			for (var linkIndex = 0;  linkIndex < eventLinksLength;  ++linkIndex) {
				var eventLink = eventLinks[linkIndex];
				var link = new ibm.tivoli.navmodel.Link({link: eventLink, dataset: this});
				var props = [];
				var properties = eventLink.properties;					
				if (properties) {
					var propsLength = properties.length;
					for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
						var property = new ibm.tivoli.navmodel.Property({property: properties[propIndex], dataset: this, objectType: "link"});
						props[propIndex] = property;						
						if (this._trace.isFinest()) {
							this._trace.finest("datasetId=" + this.getId() + " linkId=" + link.getId() + " prop=" + property.getId() + " value=" + property.getDisplayValue(), "_handleLinkPropertiesChangedEvent");
						}
					}
				}
				
				link = this._cacheLink(link);					
				event._addLink(link, props);
			}
		}		
	},

	_handleLinkChangedEvent: function(event) {
		// summary:
		//		Handle "linkChanged" event type
		
		// might have new list of styles
		this._clearLinkStyleCache();
		this._clearRelationshipStyleCache();
		    
		var eventLink = event._getEventLink();
		var link = new ibm.tivoli.navmodel.Link({link: eventLink, dataset: this});
		link = this._cacheLink(link);	
		event._addLink(link);
				
		if (this._trace.isFinest()) {
			this._trace.finest("datasetId=" + this.getId() + " linkId=" + link.getId() + " reason=" + event.getLinkChangedReason(), "_handleLinkChangedEvent");
		}						
		
		// affected relationships (reason is "relationshipAdded" or "relationshipRemoved")
		var eventRelationships = event._getEventRelationships();
		if (eventRelationships) {
			var eventRelLength = eventRelationships.length;
			for (var eventRelIndex = 0;  eventRelIndex < eventRelLength;  ++eventRelIndex) {
				var relationship = new ibm.tivoli.navmodel.Relationship({relationship: eventRelationships[eventRelIndex], dataset: this});
				event._addRelationship(relationship);
			}
		}
	},

	_handleRelationshipStatusChangedEvent: function(event) {
		// summary:
		//		Handle "relationshipStatusChanged" event type
		var eventRels = event._getEventRelationships();
		if (eventRels) {
			var relLength = eventRels.length;
			for (var relIndex = 0;  relIndex < relLength;  ++relIndex) {
				var eventRel = eventRels[relIndex];
				var relationship = new ibm.tivoli.navmodel.Relationship({relationship: eventRel, dataset: this});
				relationship = this._cacheRelationship(relationship);
				event._addRelationship(relationship);
				
				if (this._trace.isFinest()) {
					var status = relationship.getStatuses({widgetDef: null});
					if (status) {
						var statusLength = status.length;
						for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
							this._trace.finest("datasetId=" + this.getId() + " relationshipId=" + eventRel.id + " status=" + status[statusIndex].getLabel(), "_handleRelationshipStatusChangedEvent");
						}					
					}
				}
			}
		}		
	},

	_handleRelationshipPropertiesChangedEvent: function(event) {
		// summary:
		//		Handle "relationshipPropertiesChanged" event type
		var eventRels = event._getEventRelationships();
		if (eventRels) {
			var relsLength = eventRels.length;
			for (var relIndex = 0;  relIndex < relsLength;  ++relIndex) {
				var eventRel = eventRels[relIndex];
				var props = [];
				var properties = eventRel.properties;					
				if (properties) {
					var propsLength = properties.length;
					for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
						var property = new ibm.tivoli.navmodel.Property({property: properties[propIndex], dataset: this, objectType: "relationship"});
						props[propIndex] = property;							
						if (this._trace.isFinest()) {
							this._trace.finest("datasetId=" + this.getId() + " relationshipId=" + eventRel.id + " prop=" + property.getId() + " value=" + property.getDisplayValue(), "_handleRelationshipPropertiesChangedEvent");
						}
					}					
				}
					
				var relationship = new ibm.tivoli.navmodel.Relationship({relationship: eventRel, dataset: this});
				relationship = this._cacheRelationship(relationship);
				event._addRelationship(relationship, props);				
			}
		}
	},
		
	_validateCache: function() {
		// summary:
		//		Check cache for items, links, and relationships that haven't been used for awhile;
		//		remove from cache
		this._trace.entering("starting validate cache", "_validateCache");
		
		this._validate(this._itemsById);
		this._validate(this._linksById);
		this._validate(this._relationshipsById);
		
		this._trace.exiting("ending validate cache", "_validateCache");				
	},
	
	_validate: function(cache) {
		// summary:
		//		Check expiration for supplied cache; items, links, or relationships
		var nowDate = new Date();
		var now = nowDate.getTime();
		
		for (var id in cache) { 
    		if (cache.hasOwnProperty(id)) {
    			var obj = cache[id];
    			if (obj && obj.checkExpiration(now)) {
    				delete cache[id];
    			}
    		}        
		} 		
	},
	
	/////////////////////////////
	// abstract methods
	/////////////////////////////
	
	/*=====
	initDataset_kwArgs: function(createEventRouter) {
		// createEventRouter: Boolean
		//		boolean indicating whether new event router should be created
	},
	=====*/
	
	_initDataset: function(/*initDataset_kwArgs*/ kwArgs) {
		// summary:
		//		Initialize internal dataset object
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		
		// determine if we need to start up the cache interval to purge
		// resources that have not been touched in a while
		if (!this._cacheIntervalId && this.getItemCount() >= this._args.cacheThreshold && this._args.cacheValidationInterval) {
			this._cacheIntervalId = setInterval(dojo.hitch(this, this._validateCache), (this._args.cacheValidationInterval*60000));
		}

      // determine if we need to start up the remote provider keep-alive interval
      // to ensure session connection through passthrough to remote provider is maintained          255243
      if (!this._remoteKeepAliveIntervalId && (this.getAttribute("remote") != null) && this.getAttribute("remote")) {
         if (this._trace.isFine()) {
             this._trace.fine("Setting up remote dataset keep-alive interval = " + this._args.remoteKeepAliveInterval + " min, for dataset = " + this.getId(), "_initDataset");
         }
         this._remoteKeepAliveIntervalId = setInterval(dojo.hitch(this, this.keepAlive), (this._args.remoteKeepAliveInterval*60000));
      }
	},

	_deleteDataset: function() {
		// summary:
		//		Dataset is no longer in use	
	},
	
	/*=====
	fetchItemsAndLinks_onComplete: function(items, links, toolbar, kwArgs){},
	=====*/
	
	/*=====
	fetchItemsAndLinks_kwArgs: function(onComplete, onError, sync) {
		// onComplete: fetchItemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	_fetchItemsAndLinks: function(/*fetchItemsAndLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Returns both the items and links associated with the dataset
	},
	
	_fetchItems: function(/*getItems_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch items from rest service
	},

	_fetchItem: function(/*getItem_kwArgs*/ kwArgs) {
		// summary:
		//		Synchronously fetch an individual item from the rest service
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		return null;
	},
	
	_fetchOriginItems: function(/*getOriginItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns orign items of the dataset
	},
	
	_fetchRelatedItems: function(/*getRelatedItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items that are related to the requested item
	},

	_queryItems: function(/*queryItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items associated with the dataset that match
		//		the provided custom query
	},

	_fetchLinks: function(/*getLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch links from rest service
	},

	_fetchLink: function(/*getLink_kwArgs*/ linkId) {
		// summary:
		//		Synchronously fetch an individual link from the rest service
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		return null;
	},
		
	_fetchRelationships: function(/*getRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch relationships from rest service
	},

	_fetchRelationship: function(/*getRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Synchronously fetch an individual relationship from the rest service
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		return null;
	},

	_fetchItemStyles: function(/*getItemStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current item styles for the Dataset
	},
	
	_fetchLinkStyles: function(/*getLinkStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current link styles for the Dataset
	},
	
	_fetchRelationshipStyles: function(/*getRelationshipStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current relationship styles for the Dataset
	},
	
	/*=====
	fetchStatuses_kwArgs: function(onComplete, onError, start, count, sync) {
		// onComplete: relStatuses_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// type: String
		//		object type for columns: "item", "link", "relationship"
		// start: int
		//		start index to fetch
		// count: int
		//		number of status values to fetch; null means all
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchStatuses: function(/*fetchStatuses_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of possible relationship status values for the Dataset
	},
	
	_fetchTasks: function(/*getTasks_kwArgs*/ kwArgs) {
		// summary:
		//		Returns tasks associated with the dataset
	},
	
	_navigate: function(/*navigate_kwArgs*/ kwArgs) {
		// summary:
		//		Navigate to another Dataset from a navigation task
	},
	
	/*=====
	fetchColumns_kwArgs: function(onComplete, onError, type, properties, start, count, metadaa, sync) {
		// onComplete: cols_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// type: String
		//		object type: "item", "link", "relationship" 
		// properties: String[]
		//		array of ids in which to fetch columns;
		//		if not specified the "current" columns will be fetched
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// metadata: Boolean
		//		indicates if filter metadata should be returned with columns
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	_fetchColumns: function(/*fetchColumns_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch columns from rest service
	},
	
	_setColumns: function(/*setColumns_kwArgs*/ kwArgs) {
		// summary:
		//		Save columns to rest service
	},

	_createItem: function(/*createItem_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new item in the dataset
	},
	
	_createItems: function(/*createItems_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new items in the dataset
	},
	
	_deleteItem: function(/*deleteItem_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a item from the dataset
	},
	
	_deleteItems: function(/*deleteItems_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete items from the dataset
	},
	
	_createLink: function(/*createLink_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new link in the dataset
	},
	
	_createLinks: function(/*createLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new links in the dataset
	},
	
	_deleteLink: function(/*deleteLink_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a link from the dataset
	},
	
	_deleteLinks: function(/*deleteLinks_kwArgs*/  kwArgs) {
		// summary:
		//		Asynchronously delete links from the dataset
	},

	_createRelationship: function(/*createRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new relationship in the dataset
	},

	_createRelationships: function(/*createRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new relationships in the dataset
	},
	
	_deleteRelationship: function(/*deleteRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a relationship from the dataset
	},
	
	_deleteRelationships: function(/*deleteRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete relationships from the dataset
	},

   _doKeepAlive: function(/*doKeepAlive_kwArgs*/ kwArgs) {       // 255243
       // summary:
       //		Query the dataset attributes again to keep the connection with the provider alive
   }
});

// 255211
// public constants for values of getDatasetImplementation() as defined by CURI INavCollection.getCollectionType()
//    SIMPLE = INavCollection.SIMPLE_COLLECTION
//    HYBRID = INavCollection.HYBRID_COLLECTION
//    LARGE_TABLE = INavCollection.LARGE_TABLE_COLLECTION
// usage:
//      if ( myDataset.getDatasetImplementation() == ibm.tivoli.navmodel.AbstractDataset.implementation.LARGE_TABLE )
//      {  /* do something */ }
ibm.tivoli.navmodel.AbstractDataset.implementation = {     // 255247
   SIMPLE: "simple",
   HYBRID: "hybrid",
   LARGE_TABLE: "large"
};

});

},
'ibm/usmi/dashboard/ResourceDashBoardPreload':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/dashboard/ResourceDashBoardPreload", [		
		"dojo/_base/kernel", "dojo/_base/declare", "dojo/topic", "dojo/i18n!./nls/DashbarResources"
	], function (dojo, declare, topic, mri) {
	return declare([], {
	
        //resource bundle
        _resources : mri,
        
		constructor : function (args) {
			dojo.safeMixin(this, args);
			
			topic.publish("updateTitle", {
				view : this._resources.ResourceDashboard,
				loading : true
			});			
		}				
	});
});

},
'dijit/_Templated':function(){
define([
	"./_WidgetBase",
	"./_TemplatedMixin",
	"./_WidgetsInTemplateMixin",
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/_base/lang", // lang.extend lang.isArray
	"dojo/_base/kernel" // kernel.deprecated
], function(_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin, array, declare, lang, kernel){

/*=====
	var _WidgetBase = dijit._WidgetBase;
	var _TemplatedMixin = dijit._TemplatedMixin;
	var _WidgetsInTemplateMixin = dijit._WidgetsInTemplateMixin;
=====*/

	// module:
	//		dijit/_Templated
	// summary:
	//		Deprecated mixin for widgets that are instantiated from a template.

	// These arguments can be specified for widgets which are used in templates.
	// Since any widget can be specified as sub widgets in template, mix it
	// into the base widget class.  (This is a hack, but it's effective.)
	lang.extend(_WidgetBase, {
		waiRole: "",
		waiState:""
	});

	return declare("dijit._Templated", [_TemplatedMixin, _WidgetsInTemplateMixin], {
		// summary:
		//		Deprecated mixin for widgets that are instantiated from a template.
		//		Widgets should use _TemplatedMixin plus if necessary _WidgetsInTemplateMixin instead.

		// widgetsInTemplate: [protected] Boolean
		//		Should we parse the template to find widgets that might be
		//		declared in markup inside it?  False by default.
		widgetsInTemplate: false,

		constructor: function(){
			kernel.deprecated(this.declaredClass + ": dijit._Templated deprecated, use dijit._TemplatedMixin and if necessary dijit._WidgetsInTemplateMixin", "", "2.0");
		},

		_attachTemplateNodes: function(rootNode, getAttrFunc){

			this.inherited(arguments);

			// Do deprecated waiRole and waiState
			var nodes = lang.isArray(rootNode) ? rootNode : (rootNode.all || rootNode.getElementsByTagName("*"));
			var x = lang.isArray(rootNode) ? 0 : -1;
			for(; x<nodes.length; x++){
				var baseNode = (x == -1) ? rootNode : nodes[x];

				// waiRole, waiState
				var role = getAttrFunc(baseNode, "waiRole");
				if(role){
					baseNode.setAttribute("role", role);
				}
				var values = getAttrFunc(baseNode, "waiState");
				if(values){
					array.forEach(values.split(/\s*,\s*/), function(stateValue){
						if(stateValue.indexOf('-') != -1){
							var pair = stateValue.split('-');
							baseNode.setAttribute("aria-"+pair[0], pair[1]);
						}
					});
				}
			}
		}
	});
});

},
'dijit/layout/ScrollingTabController':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.add domClass.contains
	"dojo/dom-geometry", // domGeometry.contentBox
	"dojo/dom-style", // domStyle.style
	"dojo/_base/fx", // Animation
	"dojo/_base/lang", // lang.hitch
	"dojo/query", // query
	"dojo/_base/sniff", // has("ie"), has("webkit"), has("quirks")
	"../registry",	// registry.byId()
	"dojo/text!./templates/ScrollingTabController.html",
	"dojo/text!./templates/_ScrollingTabControllerButton.html",
	"./TabController",
	"./utils",	// marginBox2contextBox, layoutChildren
	"../_WidgetsInTemplateMixin",
	"../Menu",
	"../MenuItem",
	"../form/Button",
	"../_HasDropDown",
	"dojo/NodeList-dom" // NodeList.style
], function(array, declare, domClass, domGeometry, domStyle, fx, lang, query, has,
	registry, tabControllerTemplate, buttonTemplate, TabController, layoutUtils, _WidgetsInTemplateMixin,
	Menu, MenuItem, Button, _HasDropDown){

/*=====
var _WidgetsInTemplateMixin = dijit._WidgetsInTemplateMixin;
var Menu = dijit.Menu;
var _HasDropDown = dijit._HasDropDown;
var TabController = dijit.layout.TabController;
=====*/


// module:
//		dijit/layout/ScrollingTabController
// summary:
//		Set of tabs with left/right arrow keys and a menu to switch between tabs not
//		all fitting on a single row.


var ScrollingTabController = declare("dijit.layout.ScrollingTabController", [TabController, _WidgetsInTemplateMixin], {
	// summary:
	//		Set of tabs with left/right arrow keys and a menu to switch between tabs not
	//		all fitting on a single row.
	//		Works only for horizontal tabs (either above or below the content, not to the left
	//		or right).
	// tags:
	//		private

	baseClass: "dijitTabController dijitScrollingTabController",

	templateString: tabControllerTemplate,

	// useMenu: [const] Boolean
	//		True if a menu should be used to select tabs when they are too
	//		wide to fit the TabContainer, false otherwise.
	useMenu: true,

	// useSlider: [const] Boolean
	//		True if a slider should be used to select tabs when they are too
	//		wide to fit the TabContainer, false otherwise.
	useSlider: true,

	// tabStripClass: [const] String
	//		The css class to apply to the tab strip, if it is visible.
	tabStripClass: "",

	widgetsInTemplate: true,

	// _minScroll: Number
	//		The distance in pixels from the edge of the tab strip which,
	//		if a scroll animation is less than, forces the scroll to
	//		go all the way to the left/right.
	_minScroll: 5,

	// Override default behavior mapping class to DOMNode
	_setClassAttr: { node: "containerNode", type: "class" },

	buildRendering: function(){
		this.inherited(arguments);
		var n = this.domNode;

		this.scrollNode = this.tablistWrapper;
		this._initButtons();

		if(!this.tabStripClass){
			this.tabStripClass = "dijitTabContainer" +
				this.tabPosition.charAt(0).toUpperCase() +
				this.tabPosition.substr(1).replace(/-.*/, "") +
				"None";
			domClass.add(n, "tabStrip-disabled")
		}

		domClass.add(this.tablistWrapper, this.tabStripClass);
	},

	onStartup: function(){
		this.inherited(arguments);

		// TabController is hidden until it finishes drawing, to give
		// a less visually jumpy instantiation.   When it's finished, set visibility to ""
		// to that the tabs are hidden/shown depending on the container's visibility setting.
		domStyle.set(this.domNode, "visibility", "");
		this._postStartup = true;
	},

	onAddChild: function(page, insertIndex){
		this.inherited(arguments);

		// changes to the tab button label or iconClass will have changed the width of the
		// buttons, so do a resize
		array.forEach(["label", "iconClass"], function(attr){
			this.pane2watches[page.id].push(
				this.pane2button[page.id].watch(attr, lang.hitch(this, function(){
					if(this._postStartup && this._dim){
						this.resize(this._dim);
					}
				}))
			);
		}, this);

		// Increment the width of the wrapper when a tab is added
		// This makes sure that the buttons never wrap.
		// The value 200 is chosen as it should be bigger than most
		// Tab button widths.
		domStyle.set(this.containerNode, "width",
			(domStyle.get(this.containerNode, "width") + 200) + "px");
	},

	onRemoveChild: function(page, insertIndex){
		// null out _selectedTab because we are about to delete that dom node
		var button = this.pane2button[page.id];
		if(this._selectedTab === button.domNode){
			this._selectedTab = null;
		}

		this.inherited(arguments);
	},

	_initButtons: function(){
		// summary:
		//		Creates the buttons used to scroll to view tabs that
		//		may not be visible if the TabContainer is too narrow.

		// Make a list of the buttons to display when the tab labels become
		// wider than the TabContainer, and hide the other buttons.
		// Also gets the total width of the displayed buttons.
		this._btnWidth = 0;
		this._buttons = query("> .tabStripButton", this.domNode).filter(function(btn){
			if((this.useMenu && btn == this._menuBtn.domNode) ||
				(this.useSlider && (btn == this._rightBtn.domNode || btn == this._leftBtn.domNode))){
				this._btnWidth += domGeometry.getMarginSize(btn).w;
				return true;
			}else{
				domStyle.set(btn, "display", "none");
				return false;
			}
		}, this);
	},

	_getTabsWidth: function(){
		var children = this.getChildren();
		if(children.length){
			var leftTab = children[this.isLeftToRight() ? 0 : children.length - 1].domNode,
				rightTab = children[this.isLeftToRight() ? children.length - 1 : 0].domNode;
			return rightTab.offsetLeft + domStyle.get(rightTab, "width") - leftTab.offsetLeft;
		}else{
			return 0;
		}
	},

	_enableBtn: function(width){
		// summary:
		//		Determines if the tabs are wider than the width of the TabContainer, and
		//		thus that we need to display left/right/menu navigation buttons.
		var tabsWidth = this._getTabsWidth();
		width = width || domStyle.get(this.scrollNode, "width");
		return tabsWidth > 0 && width < tabsWidth;
	},

	resize: function(dim){
		// summary:
		//		Hides or displays the buttons used to scroll the tab list and launch the menu
		//		that selects tabs.

		// Save the dimensions to be used when a child is renamed.
		this._dim = dim;

		// Set my height to be my natural height (tall enough for one row of tab labels),
		// and my content-box width based on margin-box width specified in dim parameter.
		// But first reset scrollNode.height in case it was set by layoutChildren() call
		// in a previous run of this method.
		this.scrollNode.style.height = "auto";
		var cb = this._contentBox = layoutUtils.marginBox2contentBox(this.domNode, {h: 0, w: dim.w});
		cb.h = this.scrollNode.offsetHeight;
		domGeometry.setContentSize(this.domNode, cb);

		// Show/hide the left/right/menu navigation buttons depending on whether or not they
		// are needed.
		var enable = this._enableBtn(this._contentBox.w);
		this._buttons.style("display", enable ? "" : "none");

		// Position and size the navigation buttons and the tablist
		this._leftBtn.layoutAlign = "left";
		this._rightBtn.layoutAlign = "right";
		this._menuBtn.layoutAlign = this.isLeftToRight() ? "right" : "left";
		layoutUtils.layoutChildren(this.domNode, this._contentBox,
			[this._menuBtn, this._leftBtn, this._rightBtn, {domNode: this.scrollNode, layoutAlign: "client"}]);

		// set proper scroll so that selected tab is visible
		if(this._selectedTab){
			if(this._anim && this._anim.status() == "playing"){
				this._anim.stop();
			}
			this.scrollNode.scrollLeft = this._convertToScrollLeft(this._getScrollForSelectedTab());
		}

		// Enable/disabled left right buttons depending on whether or not user can scroll to left or right
		this._setButtonClass(this._getScroll());

		this._postResize = true;

		// Return my size so layoutChildren() can use it.
		// Also avoids IE9 layout glitch on browser resize when scroll buttons present
		return {h: this._contentBox.h, w: dim.w};
	},

	_getScroll: function(){
		// summary:
		//		Returns the current scroll of the tabs where 0 means
		//		"scrolled all the way to the left" and some positive number, based on #
		//		of pixels of possible scroll (ex: 1000) means "scrolled all the way to the right"
		return (this.isLeftToRight() || has("ie") < 8 || (has("ie") && has("quirks")) || has("webkit")) ? this.scrollNode.scrollLeft :
				domStyle.get(this.containerNode, "width") - domStyle.get(this.scrollNode, "width")
					 + (has("ie") == 8 ? -1 : 1) * this.scrollNode.scrollLeft;
	},

	_convertToScrollLeft: function(val){
		// summary:
		//		Given a scroll value where 0 means "scrolled all the way to the left"
		//		and some positive number, based on # of pixels of possible scroll (ex: 1000)
		//		means "scrolled all the way to the right", return value to set this.scrollNode.scrollLeft
		//		to achieve that scroll.
		//
		//		This method is to adjust for RTL funniness in various browsers and versions.
		if(this.isLeftToRight() || has("ie") < 8 || (has("ie") && has("quirks")) || has("webkit")){
			return val;
		}else{
			var maxScroll = domStyle.get(this.containerNode, "width") - domStyle.get(this.scrollNode, "width");
			return (has("ie") == 8 ? -1 : 1) * (val - maxScroll);
		}
	},

	onSelectChild: function(/*dijit._Widget*/ page){
		// summary:
		//		Smoothly scrolls to a tab when it is selected.

		var tab = this.pane2button[page.id];
		if(!tab || !page){return;}

		var node = tab.domNode;

		// Save the selection
		if(node != this._selectedTab){
			this._selectedTab = node;

			// Scroll to the selected tab, except on startup, when scrolling is handled in resize()
			if(this._postResize){
				var sl = this._getScroll();

				if(sl > node.offsetLeft ||
						sl + domStyle.get(this.scrollNode, "width") <
						node.offsetLeft + domStyle.get(node, "width")){
					this.createSmoothScroll().play();
				}
			}
		}

		this.inherited(arguments);
	},

	_getScrollBounds: function(){
		// summary:
		//		Returns the minimum and maximum scroll setting to show the leftmost and rightmost
		//		tabs (respectively)
		var children = this.getChildren(),
			scrollNodeWidth = domStyle.get(this.scrollNode, "width"),		// about 500px
			containerWidth = domStyle.get(this.containerNode, "width"),	// 50,000px
			maxPossibleScroll = containerWidth - scrollNodeWidth,	// scrolling until right edge of containerNode visible
			tabsWidth = this._getTabsWidth();

		if(children.length && tabsWidth > scrollNodeWidth){
			// Scrolling should happen
			return {
				min: this.isLeftToRight() ? 0 : children[children.length-1].domNode.offsetLeft,
				max: this.isLeftToRight() ?
					(children[children.length-1].domNode.offsetLeft + domStyle.get(children[children.length-1].domNode, "width")) - scrollNodeWidth :
					maxPossibleScroll
			};
		}else{
			// No scrolling needed, all tabs visible, we stay either scrolled to far left or far right (depending on dir)
			var onlyScrollPosition = this.isLeftToRight() ? 0 : maxPossibleScroll;
			return {
				min: onlyScrollPosition,
				max: onlyScrollPosition
			};
		}
	},

	_getScrollForSelectedTab: function(){
		// summary:
		//		Returns the scroll value setting so that the selected tab
		//		will appear in the center
		var w = this.scrollNode,
			n = this._selectedTab,
			scrollNodeWidth = domStyle.get(this.scrollNode, "width"),
			scrollBounds = this._getScrollBounds();

		// TODO: scroll minimal amount (to either right or left) so that
		// selected tab is fully visible, and just return if it's already visible?
		var pos = (n.offsetLeft + domStyle.get(n, "width")/2) - scrollNodeWidth/2;
		pos = Math.min(Math.max(pos, scrollBounds.min), scrollBounds.max);

		// TODO:
		// If scrolling close to the left side or right side, scroll
		// all the way to the left or right.  See this._minScroll.
		// (But need to make sure that doesn't scroll the tab out of view...)
		return pos;
	},

	createSmoothScroll: function(x){
		// summary:
		//		Creates a dojo._Animation object that smoothly scrolls the tab list
		//		either to a fixed horizontal pixel value, or to the selected tab.
		// description:
		//		If an number argument is passed to the function, that horizontal
		//		pixel position is scrolled to.  Otherwise the currently selected
		//		tab is scrolled to.
		// x: Integer?
		//		An optional pixel value to scroll to, indicating distance from left.

		// Calculate position to scroll to
		if(arguments.length > 0){
			// position specified by caller, just make sure it's within bounds
			var scrollBounds = this._getScrollBounds();
			x = Math.min(Math.max(x, scrollBounds.min), scrollBounds.max);
		}else{
			// scroll to center the current tab
			x = this._getScrollForSelectedTab();
		}

		if(this._anim && this._anim.status() == "playing"){
			this._anim.stop();
		}

		var self = this,
			w = this.scrollNode,
			anim = new fx.Animation({
				beforeBegin: function(){
					if(this.curve){ delete this.curve; }
					var oldS = w.scrollLeft,
						newS = self._convertToScrollLeft(x);
					anim.curve = new fx._Line(oldS, newS);
				},
				onAnimate: function(val){
					w.scrollLeft = val;
				}
			});
		this._anim = anim;

		// Disable/enable left/right buttons according to new scroll position
		this._setButtonClass(x);

		return anim; // dojo._Animation
	},

	_getBtnNode: function(/*Event*/ e){
		// summary:
		//		Gets a button DOM node from a mouse click event.
		// e:
		//		The mouse click event.
		var n = e.target;
		while(n && !domClass.contains(n, "tabStripButton")){
			n = n.parentNode;
		}
		return n;
	},

	doSlideRight: function(/*Event*/ e){
		// summary:
		//		Scrolls the menu to the right.
		// e:
		//		The mouse click event.
		this.doSlide(1, this._getBtnNode(e));
	},

	doSlideLeft: function(/*Event*/ e){
		// summary:
		//		Scrolls the menu to the left.
		// e:
		//		The mouse click event.
		this.doSlide(-1,this._getBtnNode(e));
	},

	doSlide: function(/*Number*/ direction, /*DomNode*/ node){
		// summary:
		//		Scrolls the tab list to the left or right by 75% of the widget width.
		// direction:
		//		If the direction is 1, the widget scrolls to the right, if it is
		//		-1, it scrolls to the left.

		if(node && domClass.contains(node, "dijitTabDisabled")){return;}

		var sWidth = domStyle.get(this.scrollNode, "width");
		var d = (sWidth * 0.75) * direction;

		var to = this._getScroll() + d;

		this._setButtonClass(to);

		this.createSmoothScroll(to).play();
	},

	_setButtonClass: function(/*Number*/ scroll){
		// summary:
		//		Disables the left scroll button if the tabs are scrolled all the way to the left,
		//		or the right scroll button in the opposite case.
		// scroll: Integer
		//		amount of horizontal scroll

		var scrollBounds = this._getScrollBounds();
		this._leftBtn.set("disabled", scroll <= scrollBounds.min);
		this._rightBtn.set("disabled", scroll >= scrollBounds.max);
	}
});


var ScrollingTabControllerButtonMixin = declare("dijit.layout._ScrollingTabControllerButtonMixin", null, {
	baseClass: "dijitTab tabStripButton",

	templateString: buttonTemplate,

		// Override inherited tabIndex: 0 from dijit.form.Button, because user shouldn't be
		// able to tab to the left/right/menu buttons
	tabIndex: "",

	// Similarly, override FormWidget.isFocusable() because clicking a button shouldn't focus it
	// either (this override avoids focus() call in FormWidget.js)
	isFocusable: function(){ return false; }
});
/*=====
ScrollingTabControllerButtonMixin = dijit.layout._ScrollingTabControllerButtonMixin;
=====*/

// Class used in template
declare("dijit.layout._ScrollingTabControllerButton",
	[Button, ScrollingTabControllerButtonMixin]);

// Class used in template
declare(
	"dijit.layout._ScrollingTabControllerMenuButton",
	[Button, _HasDropDown, ScrollingTabControllerButtonMixin],
{
	// id of the TabContainer itself
	containerId: "",

	// -1 so user can't tab into the button, but so that button can still be focused programatically.
	// Because need to move focus to the button (or somewhere) before the menu is hidden or IE6 will crash.
	tabIndex: "-1",

	isLoaded: function(){
		// recreate menu every time, in case the TabContainer's list of children (or their icons/labels) have changed
		return false;
	},

	loadDropDown: function(callback){
		this.dropDown = new Menu({
			id: this.containerId + "_menu",
			dir: this.dir,
			lang: this.lang,
			textDir: this.textDir
		});
		var container = registry.byId(this.containerId);
		array.forEach(container.getChildren(), function(page){
			var menuItem = new MenuItem({
				id: page.id + "_stcMi",
				label: page.title,
				iconClass: page.iconClass,
				dir: page.dir,
				lang: page.lang,
				textDir: page.textDir,
				onClick: function(){
					container.selectChild(page);
				}
			});
			this.dropDown.addChild(menuItem);
		}, this);
		callback();
	},

	closeDropDown: function(/*Boolean*/ focus){
		this.inherited(arguments);
		if(this.dropDown){
			this.dropDown.destroyRecursive();
			delete this.dropDown;
		}
	}
});

return ScrollingTabController;
});

},
'dojo/dnd/Source':function(){
define(["../main", "./Selector", "./Manager"], function(dojo, Selector, Manager) {
	// module:
	//		dojo/dnd/Source
	// summary:
	//		TODOC

/*=====
Selector = dojo.dnd.Selector;
=====*/

/*
	Container property:
		"Horizontal"- if this is the horizontal container
	Source states:
		""			- normal state
		"Moved"		- this source is being moved
		"Copied"	- this source is being copied
	Target states:
		""			- normal state
		"Disabled"	- the target cannot accept an avatar
	Target anchor state:
		""			- item is not selected
		"Before"	- insert point is before the anchor
		"After"		- insert point is after the anchor
*/

/*=====
dojo.dnd.__SourceArgs = function(){
	//	summary:
	//		a dict of parameters for DnD Source configuration. Note that any
	//		property on Source elements may be configured, but this is the
	//		short-list
	//	isSource: Boolean?
	//		can be used as a DnD source. Defaults to true.
	//	accept: Array?
	//		list of accepted types (text strings) for a target; defaults to
	//		["text"]
	//	autoSync: Boolean
	//		if true refreshes the node list on every operation; false by default
	//	copyOnly: Boolean?
	//		copy items, if true, use a state of Ctrl key otherwise,
	//		see selfCopy and selfAccept for more details
	//	delay: Number
	//		the move delay in pixels before detecting a drag; 0 by default
	//	horizontal: Boolean?
	//		a horizontal container, if true, vertical otherwise or when omitted
	//	selfCopy: Boolean?
	//		copy items by default when dropping on itself,
	//		false by default, works only if copyOnly is true
	//	selfAccept: Boolean?
	//		accept its own items when copyOnly is true,
	//		true by default, works only if copyOnly is true
	//	withHandles: Boolean?
	//		allows dragging only by handles, false by default
	//  generateText: Boolean?
	//		generate text node for drag and drop, true by default
	this.isSource = isSource;
	this.accept = accept;
	this.autoSync = autoSync;
	this.copyOnly = copyOnly;
	this.delay = delay;
	this.horizontal = horizontal;
	this.selfCopy = selfCopy;
	this.selfAccept = selfAccept;
	this.withHandles = withHandles;
	this.generateText = true;
}
=====*/

// For back-compat, remove in 2.0.
if(!dojo.isAsync){
	dojo.ready(0, function(){
		var requires = ["dojo/dnd/AutoSource", "dojo/dnd/Target"];
		require(requires);	// use indirection so modules not rolled into a build
	})
}

return dojo.declare("dojo.dnd.Source", Selector, {
	// summary:
	//		a Source object, which can be used as a DnD source, or a DnD target

	// object attributes (for markup)
	isSource: true,
	horizontal: false,
	copyOnly: false,
	selfCopy: false,
	selfAccept: true,
	skipForm: false,
	withHandles: false,
	autoSync: false,
	delay: 0, // pixels
	accept: ["text"],
	generateText: true,

	constructor: function(/*DOMNode|String*/node, /*dojo.dnd.__SourceArgs?*/params){
		// summary:
		//		a constructor of the Source
		// node:
		//		node or node's id to build the source on
		// params:
		//		any property of this class may be configured via the params
		//		object which is mixed-in to the `dojo.dnd.Source` instance
		dojo.mixin(this, dojo.mixin({}, params));
		var type = this.accept;
		if(type.length){
			this.accept = {};
			for(var i = 0; i < type.length; ++i){
				this.accept[type[i]] = 1;
			}
		}
		// class-specific variables
		this.isDragging = false;
		this.mouseDown = false;
		this.targetAnchor = null;
		this.targetBox = null;
		this.before = true;
		this._lastX = 0;
		this._lastY = 0;
		// states
		this.sourceState  = "";
		if(this.isSource){
			dojo.addClass(this.node, "dojoDndSource");
		}
		this.targetState  = "";
		if(this.accept){
			dojo.addClass(this.node, "dojoDndTarget");
		}
		if(this.horizontal){
			dojo.addClass(this.node, "dojoDndHorizontal");
		}
		// set up events
		this.topics = [
			dojo.subscribe("/dnd/source/over", this, "onDndSourceOver"),
			dojo.subscribe("/dnd/start",  this, "onDndStart"),
			dojo.subscribe("/dnd/drop",   this, "onDndDrop"),
			dojo.subscribe("/dnd/cancel", this, "onDndCancel")
		];
	},

	// methods
	checkAcceptance: function(source, nodes){
		// summary:
		//		checks if the target can accept nodes from this source
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		if(this == source){
			return !this.copyOnly || this.selfAccept;
		}
		for(var i = 0; i < nodes.length; ++i){
			var type = source.getItem(nodes[i].id).type;
			// type instanceof Array
			var flag = false;
			for(var j = 0; j < type.length; ++j){
				if(type[j] in this.accept){
					flag = true;
					break;
				}
			}
			if(!flag){
				return false;	// Boolean
			}
		}
		return true;	// Boolean
	},
	copyState: function(keyPressed, self){
		// summary:
		//		Returns true if we need to copy items, false to move.
		//		It is separated to be overwritten dynamically, if needed.
		// keyPressed: Boolean
		//		the "copy" key was pressed
		// self: Boolean?
		//		optional flag that means that we are about to drop on itself

		if(keyPressed){ return true; }
		if(arguments.length < 2){
			self = this == Manager.manager().target;
		}
		if(self){
			if(this.copyOnly){
				return this.selfCopy;
			}
		}else{
			return this.copyOnly;
		}
		return false;	// Boolean
	},
	destroy: function(){
		// summary:
		//		prepares the object to be garbage-collected
		dojo.dnd.Source.superclass.destroy.call(this);
		dojo.forEach(this.topics, dojo.unsubscribe);
		this.targetAnchor = null;
	},

	// mouse event processors
	onMouseMove: function(e){
		// summary:
		//		event processor for onmousemove
		// e: Event
		//		mouse event
		if(this.isDragging && this.targetState == "Disabled"){ return; }
		dojo.dnd.Source.superclass.onMouseMove.call(this, e);
		var m = Manager.manager();
		if(!this.isDragging){
			if(this.mouseDown && this.isSource &&
					(Math.abs(e.pageX - this._lastX) > this.delay || Math.abs(e.pageY - this._lastY) > this.delay)){
				var nodes = this.getSelectedNodes();
				if(nodes.length){
					m.startDrag(this, nodes, this.copyState(dojo.isCopyKey(e), true));
				}
			}
		}
		if(this.isDragging){
			// calculate before/after
			var before = false;
			if(this.current){
				if(!this.targetBox || this.targetAnchor != this.current){
					this.targetBox = dojo.position(this.current, true);
				}
				if(this.horizontal){
					before = (e.pageX - this.targetBox.x) < (this.targetBox.w / 2);
				}else{
					before = (e.pageY - this.targetBox.y) < (this.targetBox.h / 2);
				}
			}
			if(this.current != this.targetAnchor || before != this.before){
				this._markTargetAnchor(before);
				m.canDrop(!this.current || m.source != this || !(this.current.id in this.selection));
			}
		}
	},
	onMouseDown: function(e){
		// summary:
		//		event processor for onmousedown
		// e: Event
		//		mouse event
		if(!this.mouseDown && this._legalMouseDown(e) && (!this.skipForm || !dojo.dnd.isFormElement(e))){
			this.mouseDown = true;
			this._lastX = e.pageX;
			this._lastY = e.pageY;
			dojo.dnd.Source.superclass.onMouseDown.call(this, e);
		}
	},
	onMouseUp: function(e){
		// summary:
		//		event processor for onmouseup
		// e: Event
		//		mouse event
		if(this.mouseDown){
			this.mouseDown = false;
			dojo.dnd.Source.superclass.onMouseUp.call(this, e);
		}
	},

	// topic event processors
	onDndSourceOver: function(source){
		// summary:
		//		topic event processor for /dnd/source/over, called when detected a current source
		// source: Object
		//		the source which has the mouse over it
		if(this != source){
			this.mouseDown = false;
			if(this.targetAnchor){
				this._unmarkTargetAnchor();
			}
		}else if(this.isDragging){
			var m = Manager.manager();
			m.canDrop(this.targetState != "Disabled" && (!this.current || m.source != this || !(this.current.id in this.selection)));
		}
	},
	onDndStart: function(source, nodes, copy){
		// summary:
		//		topic event processor for /dnd/start, called to initiate the DnD operation
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise
		if(this.autoSync){ this.sync(); }
		if(this.isSource){
			this._changeState("Source", this == source ? (copy ? "Copied" : "Moved") : "");
		}
		var accepted = this.accept && this.checkAcceptance(source, nodes);
		this._changeState("Target", accepted ? "" : "Disabled");
		if(this == source){
			Manager.manager().overSource(this);
		}
		this.isDragging = true;
	},
	onDndDrop: function(source, nodes, copy, target){
		// summary:
		//		topic event processor for /dnd/drop, called to finish the DnD operation
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise
		// target: Object
		//		the target which accepts items
		if(this == target){
			// this one is for us => move nodes!
			this.onDrop(source, nodes, copy);
		}
		this.onDndCancel();
	},
	onDndCancel: function(){
		// summary:
		//		topic event processor for /dnd/cancel, called to cancel the DnD operation
		if(this.targetAnchor){
			this._unmarkTargetAnchor();
			this.targetAnchor = null;
		}
		this.before = true;
		this.isDragging = false;
		this.mouseDown = false;
		this._changeState("Source", "");
		this._changeState("Target", "");
	},

	// local events
	onDrop: function(source, nodes, copy){
		// summary:
		//		called only on the current target, when drop is performed
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise

		if(this != source){
			this.onDropExternal(source, nodes, copy);
		}else{
			this.onDropInternal(nodes, copy);
		}
	},
	onDropExternal: function(source, nodes, copy){
		// summary:
		//		called only on the current target, when drop is performed
		//		from an external source
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise

		var oldCreator = this._normalizedCreator;
		// transferring nodes from the source to the target
		if(this.creator){
			// use defined creator
			this._normalizedCreator = function(node, hint){
				return oldCreator.call(this, source.getItem(node.id).data, hint);
			};
		}else{
			// we have no creator defined => move/clone nodes
			if(copy){
				// clone nodes
				this._normalizedCreator = function(node, hint){
					var t = source.getItem(node.id);
					var n = node.cloneNode(true);
					n.id = dojo.dnd.getUniqueId();
					return {node: n, data: t.data, type: t.type};
				};
			}else{
				// move nodes
				this._normalizedCreator = function(node, hint){
					var t = source.getItem(node.id);
					source.delItem(node.id);
					return {node: node, data: t.data, type: t.type};
				};
			}
		}
		this.selectNone();
		if(!copy && !this.creator){
			source.selectNone();
		}
		this.insertNodes(true, nodes, this.before, this.current);
		if(!copy && this.creator){
			source.deleteSelectedNodes();
		}
		this._normalizedCreator = oldCreator;
	},
	onDropInternal: function(nodes, copy){
		// summary:
		//		called only on the current target, when drop is performed
		//		from the same target/source
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise

		var oldCreator = this._normalizedCreator;
		// transferring nodes within the single source
		if(this.current && this.current.id in this.selection){
			// do nothing
			return;
		}
		if(copy){
			if(this.creator){
				// create new copies of data items
				this._normalizedCreator = function(node, hint){
					return oldCreator.call(this, this.getItem(node.id).data, hint);
				};
			}else{
				// clone nodes
				this._normalizedCreator = function(node, hint){
					var t = this.getItem(node.id);
					var n = node.cloneNode(true);
					n.id = dojo.dnd.getUniqueId();
					return {node: n, data: t.data, type: t.type};
				};
			}
		}else{
			// move nodes
			if(!this.current){
				// do nothing
				return;
			}
			this._normalizedCreator = function(node, hint){
				var t = this.getItem(node.id);
				return {node: node, data: t.data, type: t.type};
			};
		}
		this._removeSelection();
		this.insertNodes(true, nodes, this.before, this.current);
		this._normalizedCreator = oldCreator;
	},
	onDraggingOver: function(){
		// summary:
		//		called during the active DnD operation, when items
		//		are dragged over this target, and it is not disabled
	},
	onDraggingOut: function(){
		// summary:
		//		called during the active DnD operation, when items
		//		are dragged away from this target, and it is not disabled
	},

	// utilities
	onOverEvent: function(){
		// summary:
		//		this function is called once, when mouse is over our container
		dojo.dnd.Source.superclass.onOverEvent.call(this);
		Manager.manager().overSource(this);
		if(this.isDragging && this.targetState != "Disabled"){
			this.onDraggingOver();
		}
	},
	onOutEvent: function(){
		// summary:
		//		this function is called once, when mouse is out of our container
		dojo.dnd.Source.superclass.onOutEvent.call(this);
		Manager.manager().outSource(this);
		if(this.isDragging && this.targetState != "Disabled"){
			this.onDraggingOut();
		}
	},
	_markTargetAnchor: function(before){
		// summary:
		//		assigns a class to the current target anchor based on "before" status
		// before: Boolean
		//		insert before, if true, after otherwise
		if(this.current == this.targetAnchor && this.before == before){ return; }
		if(this.targetAnchor){
			this._removeItemClass(this.targetAnchor, this.before ? "Before" : "After");
		}
		this.targetAnchor = this.current;
		this.targetBox = null;
		this.before = before;
		if(this.targetAnchor){
			this._addItemClass(this.targetAnchor, this.before ? "Before" : "After");
		}
	},
	_unmarkTargetAnchor: function(){
		// summary:
		//		removes a class of the current target anchor based on "before" status
		if(!this.targetAnchor){ return; }
		this._removeItemClass(this.targetAnchor, this.before ? "Before" : "After");
		this.targetAnchor = null;
		this.targetBox = null;
		this.before = true;
	},
	_markDndStatus: function(copy){
		// summary:
		//		changes source's state based on "copy" status
		this._changeState("Source", copy ? "Copied" : "Moved");
	},
	_legalMouseDown: function(e){
		// summary:
		//		checks if user clicked on "approved" items
		// e: Event
		//		mouse event

		// accept only the left mouse button
		if(!dojo.mouseButtons.isLeft(e)){ return false; }

		if(!this.withHandles){ return true; }

		// check for handles
		for(var node = e.target; node && node !== this.node; node = node.parentNode){
			if(dojo.hasClass(node, "dojoDndHandle")){ return true; }
			if(dojo.hasClass(node, "dojoDndItem") || dojo.hasClass(node, "dojoDndIgnore")){ break; }
		}
		return false;	// Boolean
	}
});

});

},
'ibm/tivoli/tip/navwidget/properties/dijit/PropertiesNotebookGeneralTab':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!dijit/dijit,dijit/layout/BorderContainer,dijit/layout/ContentPane,ibm/tivoli/navmodel/Trace,dojox/xml/parser,ibm/tivoli/tip/navwidget/dijit/NavWidgetObjectPluggableView,ibm/tivoli/tip/navwidget/table/dijit/RelationshipTableViewSingleLink,ibm/tivoli/tip/navwidget/properties/PropertyRenderer"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * “Restricted Materials of IBM”
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebookGeneralTab");

dojo.require("dijit.dijit");
dojo.require("dijit.layout.BorderContainer");
dojo.require("dijit.layout.ContentPane");
dojo.require("ibm.tivoli.navmodel.Trace");
dojo.require("dojox.xml.parser");
dojo.require("ibm.tivoli.tip.navwidget.dijit.NavWidgetObjectPluggableView");
dojo.require("ibm.tivoli.tip.navwidget.table.dijit.RelationshipTableViewSingleLink");
dojo.require("ibm.tivoli.tip.navwidget.properties.PropertyRenderer");

ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebookGeneralTab._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebookGeneralTab");

dojo.declare("ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebookGeneralTab",
		[dijit.layout.BorderContainer, ibm.tivoli.tip.navwidget.dijit.NavWidgetObjectPluggableView],
	{		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
	
        //***************************
        // "Public"
        //***************************
	
		// maxProperties: [public] Object
		//		set maxProperties to null to display all properties
		maxProperties: null,
		// asahlin begin		
		daHelper: null,
		// asahlin end

        //***************************
        // "Private"
        //***************************
        
       // _propertyTabs: null,
		
		// _generalTabContentDiv: [private] Object
		//		Coming soon
        _generalTabContentDiv: null,
        
        //_outerDivTooltip: [private] Object
        // Coming soon
        _outerDivTooltip: null,
        
		// _generalTabPropGrid: [private] Object
		//		Coming soon
        _generalTabPropGrid: null,
        
		// _generalTabPropGridTable: [private] Object
		//		Coming soon
        _generalTabPropGridTable: null,     
        
        // _generalTabPropGridTooltip: [private] Object
		//		Coming soon
        _generalTabPropGridTooltip: null,
        
		// _generalTabPropGridTableTooltip: [private] Object
		//		Coming soon
        _generalTabPropGridTableTooltip: null, 
        
		// _object: [private] Object
		//		Coming soon
        _object: null,
        
		// _propertyRendererObj: [private] Object
		//		Coming soon
        _propertyRendererObj: null,
        
		// _generalTab: [private] Object
		//		Coming soon
        _generalTab: null,
        
		// _trace: [private] ibm.tivoli.navmodel.Trace
		//		Utility for trace and logging
        _trace: ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebookGeneralTab._trace, 
        
        // _propsRendered: [private] int
        //		Coming soon
        _propsRendered: 0,
        
        // _relTable: [private] Object
        //		Coming soon
        _relTable: null,
        
        // _isTooltipProperties: [protected] Boolean
        //		Coming soon
        _isTooltipProperties: false,
        
        //_tabIndexValueProperties: [private] integer
        _tabIndexValueProperties:0,

        //***************************
        // "Public functions"
        //***************************       
        constructor: function() {		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
            this._propertyTabs = new Array();           
        	//asahlin begin
            //this._propertyRendererObj = new ibm.tivoli.tip.navwidget.properties.PropertyRenderer();  
        	//asahlin end
	        //We don't have a widget def specific to this view, so let's signal to 
	        //super class not to try and load any widget defs on our behalf
			this._usesWidgetDefinition = false;
			
			//Signal to superclass we'll be firing onSetObjectsLoadComplete manually
			this._setObjectsLoadDataCompletesSynchronously = false;			
        },

        destroy: function() {
    		// summary:
    		//		Coming soon
    		// description:
    		//		Coming soon
            this.inherited(arguments);
        },

        postCreate: function() {
    		// summary:
    		//		Coming soon
    		// description:
    		//		Coming soon
        	
        	//asahlin begin
        	// cant be done in constructor, daHelper isnt set yet.
            if(this.daHelper != null) {
            	this._propertyRendererObj = new ibm.tivoli.tip.navwidget.properties.PropertyRenderer({daHelper: this.daHelper});
            } else {
            	this._propertyRendererObj = new ibm.tivoli.tip.navwidget.properties.PropertyRenderer();
            }
        	//asahlin begin
            
        	this._addGeneralTab();
        	this.title = this.commonResources.GENERAL;
        	this.addChild(this._generalTab);
           
        	this.inherited(arguments);
            
        },

        startup: function() {		
    		// summary:
    		//		Coming soon
    		// description:
    		//		Coming soon
            this.inherited(arguments);
            
            
        },
        
        getContent: function(){		
    		// summary:
    		//		Coming soon
    		// description:
    		//		Coming soon
        	
        	//return  this._generalTabPropGrid.innerHTML;
        	
        	//return the content...since tooltip will grab inner HTML...no need to do here.
        	//just return the content pane
        	return this._generalTab;
        },
        
    	handleLinkClickAction: function(viewId, objectId, propId){
    		
    		console.log("handling link click for properties view");
    		var object;
    		var property;
    		
    		
    		if(this._object){
    			property = this._object.getProperty(propId);
    		}
    		
    		if(property){
    			this._propertyRendererObj._handleLinkClick(property, this);
    		}
    	},

        _setObjectsLoadData: function(/*ibm.tivoli.navmodel.Item[]*/ items, /*ibm.tivoli.navmodel.Link[]*/ links, /*ibm.tivoli.navmodel.Relationship[]*/ relationships){		
    		// summary:
    		//		Coming soon
    		// description:
    		//		Coming soon
        	// items:
        	//		Coming soon
        	// links:
        	//		Coming soon
        	// relationships:
        	//		Coming soon
        	
        	//Clean up old dom tree
        	this._cleanUpOldDomTree();
            	
        	if (this.isMixedObjects()) {
            	//TODO: For now just log, but need to remove this and do something more useful in the end
        		console.log("Cannot currently deal with mixed content passed to GeneralTab: itemCount = "+ this.getItemCount() + ", linkCount = " + this.getLinkCount() + ", relationshipCount = " + this.getRelationshipCount());
        	
        		// fire _onSetDatasetLoadComplete
				setTimeout(dojo.hitch(this, this._onSetObjectsLoadComplete, items, links, relationships), 0);
        	} else if (this.getItemCount() > 0) {
		        	//Get properties for the new item in order to populate the new properties grid. If more than one item, for now just use the first one.
		        	this._object = items[0];
		        	this._object.getProperties({
		        		onComplete: dojo.hitch(this, this._populateGeneralTab),
		        		onError: dojo.hitch(this, this._populateGeneralTabError),
		        		properties: ["all"],
		        		metadata: true});
        	} else if (this.getLinkCount() > 0) {
        		//treat a link just like an item for now.  the populateGeneralTab will render both item and link properties
        			this._object = links[0];
        	
        		//render the relationships table if not a tooltip (and not only one rel)
        		if( !(this.maxProperties) && (this._object.getRelationshipCount()>1) ){
           	    	this._relTable = new ibm.tivoli.tip.navwidget.table.dijit.RelationshipTableViewSingleLink();
           	    	this._relTable.removeToolbar = true;
           	    	this.addPluggableView(this._relTable);//pjj
        			this._relTable.setDataset(this.dataset);
        			//this._relTable.setObjects([],links);
					
					//254228 FF4, IE9: wrong properties view for double directed links 
					//255025 IE9FF3.6TIPGeneral Properties & double linked propertiesoverlap 
					//Bobby: 11Dec11, adding a new class for relTable - BEGIN					
					//dojo.style(this._relTable.domNode, {position:'relative !important'});
					dojo.addClass(this._relTable.domNode, "relationshipTableView");
        			//Bobby: 11Dec11, adding a new class for relTable - END
					
        			//var vp = dijit.getViewport();
    				//var tablesize = { w: (vp.w*.4)-125, h: (vp.h) };
        			//this._relTable.resize(tablesize);
        		}
				
        			
        			this._object.getProperties({
        				onComplete: dojo.hitch(this, this._populateGeneralTab),
        				onError: dojo.hitch(this, this._populateGeneralTabError),
        				properties: ["all"],
        				metadata: true});
        	}else if (this.getRelationshipCount() > 0) {
	        	//Get properties for the new item in order to populate the new properties grid. If more than one item, for now just use the first one.
	        	this._object = relationships[0];
	        	this._object.getProperties({
	        		onComplete: dojo.hitch(this, this._populateGeneralTab),
	        		onError: dojo.hitch(this, this._populateGeneralTabError),
	        		properties: ["all"],
	        		metadata: true});
        	}else {
        		//For example, when setObjects gets all empty lists...
        		
        		// fire _onSetDatasetLoadComplete
				setTimeout(dojo.hitch(this, this._onSetObjectsLoadComplete, items, links, relationships), 0);
        	}
        	
        	//Call superclass
        	this.inherited(arguments);
        },
        
        clearObjects: function(){
        	this._cleanUpOldDomTree();
        	
        	//Call super class
        	this.inherited(arguments);
    	},
    	
    	_cleanUpOldDomTree: function() {
    		//Remove previously added properties grid //250874 -- changed to remove this._outerDivTooltip instead of this._generalTabPropGridTable
        	if (this._outerDivTooltip) {
        		var domNode = this._generalTab.domNode;
	        	domNode.removeChild(this._outerDivTooltip);
	        	this._outerDivTooltip = null;
        	}

       	if (this._relTable && this._relTable.domNode) {
			  var domNode = this._generalTab.domNode;
		        domNode.removeChild(this._relTable.domNode);
			  this.removePluggableView(this._relTable);
			  this._relTable.destroyRecursive();
		        this._relTable = null;
			}		
    	},

        //***************************
        // "private functions"
        //*************************** 
        _addGeneralTab: function() {
    		// summary:
    		//		Coming soon
    		// description:
    		//		Coming soon
            this._generalTabContentDiv = dojo.doc.createElement("div");
            // TODO namespace the div
            this._generalTabContentDiv.id = this.id + "_generalTab_busy_pane_container";

            this._generalTab = new dijit.layout.ContentPane({
                id: this.id + "generalTab",
                title: this.commonResources.GENERAL,
                content: this._generalTabContentDiv,
                //need this style in order to get scroll tab
                //Richa:Reducing the height and width as the scroll bar is partially hidden
                //style: "height:100%;width:100%;"
                style: "height:99%;width:99%;"
            });
            // setting the title for the general tab of properties, screen reader does not read the above title
			dojo.attr(this._generalTab.domNode,"title",this.commonResources.GENERAL);
        },

        _sortProperties: function(/*ibm.tivoli.navmodel.Property*/ a, /*ibm.tivoli.navmodel.Property*/ b) {		
    		// summary:
    		//		Function used to determine how to sort an array of properties.
        	//		This function assumes two Properties objects passed in and
        	//		it will then sort based on the Weight of the item
        	//		if weights are equal it will sort lexigraphically
    		// description:
    		//		Coming soon
        	// a:
        	//		Coming soon
        	// b:
        	//		Coming soon
        	var retval = 0;
        	retval = a.getSortWeight() - b.getSortWeight();
        	
        	if(retval == 0){
        		//sort lexigraphically
			var first = a.getLabel();
			var second = b.getLabel();
        		retval = first - second;
			if( isNaN(retval)  ){
				//compare normal lexigraphically
				if( first === second){
					//they are equal return 0
					retval = 0;
				}else{
					//else let the normal array sort figure this out (since it does lex sort)
					var compares = [first,second];
					var sorted = compares.sort();

					if( first === sorted[0] ){
						//first is bigger than second since it remained first after sort
						retval = -1;
					}else{
						//since first did not stay first, it must be second, and is thus smaller then second
						retval = 1;
					}
				}
				
			}
        		return retval;
        	}else{
        		return retval;
        	}
        },
        
		/*=====
		populateGeneralTab_kwArgs: function() {
		
		},
		=====*/
        _populateGeneralTab: function(/*ibm.tivoli.navmodel.Property[]*/ properties, /*populateGeneralTab_kwArgs*/ kwArgs) {
        	// summary:
        	//		Call back on successfully retrieving the properties of an item
        	// description:
        	//		Coming soon
        	// properties:
        	//		Coming soon
        	// kwArgs:
        	//		Coming soon

        	if (this.isDestroyed()) {
        		//Especially in the tooltip case... it's possible our request for properties is satisfied AFTER we've
        		//been destroyed. So, let's special case that
        		return;
        	}
        	
        	//outer div with the two tables
        	this._outerDivTooltip=dojo.doc.createElement("div");

        	// tooltip table
			this._generalTabPropGridTableTooltip = dojo.doc.createElement("table");
			this._generalTabPropGridTableTooltip.setAttribute('cellSpacing','3'); 
			this._generalTabPropGridTableTooltip.setAttribute('role','presentation');    // 255868
			this._generalTabPropGridTooltip = dojo.doc.createElement("tbody");
			
			
        	this._generalTabPropGridTable = dojo.doc.createElement("table");
        	this._generalTabPropGridTable.setAttribute('cellSpacing','3'); 
        	this._generalTabPropGridTable.setAttribute('role','presentation');    // 255868
        	this._generalTabPropGrid = dojo.doc.createElement("tbody");

        	
        	
        	if (this._isTooltipProperties){
        	// style for the second table
        		this._generalTabPropGridTable.className="tooltipTableGap";
        	}
            else {
                // If this is not a tooltip (i.e., actual notebook) associate the DIV containing
            	// properties content with being labeled by the tab node and if the top content
            	// node of the tabbed page does not have a role already, set it to "document" so
            	// a screen reader will read this text content (content does not have a focussable
            	// field)
                dojo.attr(this._outerDivTooltip, "aria-labelledby", this._generalTab.domNode.id);  
                if (!dojo.hasAttr(this._generalTab.domNode.parentNode, "role")) {      
                   dojo.attr(this._generalTab.domNode.parentNode, "role", "document");   
                }
             }
        	
        	
        	//first Render special properties...name, statuses, and type
        	
        	//first find name only (only one of these should exist)
        	var nameprop = null;
        		
        	nameprop = this._getPropertyType(properties, "itemName");   
        	if(!nameprop){
        		nameprop = this._getPropertyType(properties, "linkName");
        	}	
        	if(!nameprop){
        		nameprop = this._getPropertyType(properties, "relationshipName");       	    	
        	}	
        	//generate the name row and add to table if it exists!
        	if(nameprop){
        		var namerow = this._createPropTableRow(nameprop);
        		// tooltip table specific 
        		if(this._isTooltipProperties) {
        			dojo.place(namerow,this._generalTabPropGridTooltip,"last");
        		} else {
        			dojo.place(namerow, this._generalTabPropGrid, "last");
        		}
        	}
        	
        	//render to and from values for rel and links if supplied as properties
        	var item1Prop = this._getPropertyType(properties, "item1");        	
        	if(item1Prop){
        		var i1row = this._createPropTableRow(item1Prop);
        		//tooltip specific
        		if(this._isTooltipProperties)
    			dojo.place(i1row,this._generalTabPropGridTooltip,"last");
    			else
        		dojo.place(i1row, this._generalTabPropGrid, "last");
        	}
        	
        	var item2Prop = this._getPropertyType(properties, "item2");        	
        	if(item2Prop){
        		var i2row = this._createPropTableRow(item2Prop);
        		//tooltip specific
        		if(this._isTooltipProperties)
    			dojo.place(i2row,this._generalTabPropGridTooltip,"last");
    			else
        		dojo.place(i2row, this._generalTabPropGrid, "last");
        	}
        	
        	var relSourceProp = this._getPropertyType(properties, "sourceItem");        	
        	if(relSourceProp){
        		var sourceRow = this._createPropTableRow(relSourceRow);
        		//tooltip specific
        		if(this._isTooltipProperties)
    			dojo.place(sourcerow,this._generalTabPropGridTooltip,"last");
    			else
        		dojo.place(sourceRow, this._generalTabPropGrid, "last");
        	}
        	
        	var relTargetProp = this._getPropertyType(properties, "targetItem");       	
        	if(relTargetProp){
        		var targetRow = this._createPropTableRow(relTargetRow);
        		//tooltip specific
        		if(this._isTooltipProperties)
    			dojo.place(targetRow,this._generalTabPropGridTooltip,"last");
    			else
        		dojo.place(targetRow, this._generalTabPropGrid, "last");
        	}
        	
	  		//now render status if it exists
    		//find status
    		var statprop = this._getStatuses(properties);
    		for(var i=0; i<statprop.length; i++){    			
    			var statusrow = this._createPropTableRow(statprop[i]);
    			//tooltip specific
    			if(this._isTooltipProperties)
    			dojo.place(statusrow,this._generalTabPropGridTooltip,"last");
    			else
    			dojo.place(statusrow, this._generalTabPropGrid, "last");
    		}
    		
    		//find itemType only 
        	var typeprop = this._getPropertyType(properties, "itemType");	
        	//generate the type row and add to table if exists
        	if(typeprop != null){
        		var typerow = this._createPropTableRow(typeprop);
        		dojo.place(typerow, this._generalTabPropGrid,"last");
        	}
    		
    		//First sort the properties based on weight and label name
    		//
    	    properties.sort(this._sortProperties);	
    		
    		//
        	//Render all other properties except name, type, or statuses
    	    for(var i=0; i < properties.length; ++i) {
				
    	    	if(properties[i].isHidden()){
	        		//skip rendering hidden property and continue rendering the other properties
	        		continue;
    	    	}
    	    	
    	    	var propValType = properties[i].getValueType();
        		//First check to make sure this item is not the Name or Type
        		if( (propValType == "itemName") || ((propValType == "linkName") || (propValType == "relationshipName") || (propValType == "item1") || (propValType == "item2") || (propValType == "sourceItem") || (propValType == "targetItem") || propValType == "itemType") || (propValType == "status") ){
        			//skip rendering this property and continue rendering the other properties
        			continue;
        		}
        	       	
        		if(this.maxProperties != null){
        			//only render maxProperties
        			if(this.maxProperties < ++this._propsRendered){
        				break;
        			}
        		}
        		//create the property table row and add it to the table
        		var row = this._createPropTableRow(properties[i]);
        		dojo.place(row, this._generalTabPropGrid,"last");
        	}
    	    
        	dojo.place(this._generalTabPropGrid, this._generalTabPropGridTable,"last");
        	//tooltip table
        	dojo.place(this._generalTabPropGridTooltip, this._generalTabPropGridTableTooltip,"last");
        	
        	dojo.place(this._generalTabPropGridTableTooltip, this._outerDivTooltip, "last");
			dojo.place(this._generalTabPropGridTable, this._outerDivTooltip, "last");
        	
			
        	dojo.place(this._outerDivTooltip, this._generalTab.domNode, "last");

       	//If links, then render the relationship table
            var type = this._object.declaredClass;
     	    if( !(this.maxProperties) && (type == "ibm.tivoli.navmodel.Link") && (this._object.getRelationshipCount()>1) ){
     	    	//first render table header:
     	    	var tableLabelRow = dojo.doc.createElement("tr"); //row
     	        var tableLabelCell = dojo.doc.createElement("td"); //cell
     	        var tableCellLabel = dojo.doc.createTextNode(this.commonResources.RELATIONSHIP_VIEW + ":");
     	        dojo.place(tableCellLabel, tableLabelCell,"last");
     	        dojo.place(tableLabelCell, tableLabelRow,"last");         	        	
				dojo.place(tableLabelRow, this._generalTabPropGrid ,"last");	
     	        dojo.place(this._relTable.domNode, this._generalTab.domNode, "last");
     	    }
				
            //fire the _onSetObjectsLoadComplete event
            this._onSetObjectsLoadComplete(this._items, this._links, this._relationships);
        },

        _populateGeneralTabError: function(error, exception, ioArgs) {		
    		// summary:
    		//		Call back on errors on getting properties
    		// description:
    		//		Coming soon
        	// error:
        	//		Coming soon
        	// exception:
        	//		Coming soon
        	// ioArgs:
        	//		Coming soon
        	
        	//fire the _onSetObjectsLoadComplete event
            this._onSetObjectsLoadComplete(this.items, this.links, this.relationships);
            
            //Give the user some feedback
            this.displayException(error);
        },
        
        _getStatuses: function(/*ibm.tivoli.navmodel.Property[]*/ props){
        	// summary:
    		//		Simple method to get all status properties
    		// description:
    		//		Coming soon
        	// props:
        	//		Coming soon
        	// type:
        	//		Coming soon
        	var statusesArray = new Array();
        	for(var i=0; i<props.length; ++i){
    			if(props[i].getValueType() == "status"){
    				statusesArray.push(props[i]);  
    			}
    		}
    		return statusesArray;        	
        },
        
        _getPropertyType: function(/*ibm.tivoli.navmodel.Property[]*/ props, /*String*/ type){		
    		// summary:
    		//		Simple method to get a specific 1st property based on value type
    		// description:
    		//		Coming soon
        	// props:
        	//		Coming soon
        	// type:
        	//		Coming soon
        	
        	//for now just loop through array until we find it
        	for(var i=0; i<props.length; ++i){
        		//console.log(properties[i].getValueType() + ":" + properties[i].getId() +":"+ properties[i].getValue());        	          	    
    			if(props[i].getValueType() == type){
    				return props[i];
    			}
    		}    
        	//no item found, so return null
        	return null;
        },

        _createPropTableRow: function(/*ibm.tivoli.navmodel.Property*/ nameprop) { 		
    		// summary:
    		//		Function to create a single table row element for a property
    		// description:
    		//		Coming soon
        	// nameprop:
        	//		Coming soon
        	var namerow = dojo.doc.createElement("tr");

        	var nameCell = dojo.doc.createElement("td");
        	if(nameprop.getLabel()) {
        		var nameCellLabel = dojo.doc.createTextNode(nameprop.getLabel() + ":");
        		//nameCellLabel.id = nameprop.getId()+".label";
			//var labelStr =  nameprop.getId()+"_"+nameprop.getLabel()+".label";
			//dojo.attr(nameCellLabel, "id", labelStr);
        		
        		//spacing for NAME elements
        	if(this._isTooltipProperties && nameprop.getValueType()=="itemName"){	
			nameCell.className="tooltipNameSpace";
			}
        		dojo.place(nameCellLabel, nameCell,"last");
        		
        	} else {
        		var nameCellLabel = dojo.doc.createTextNode(nameprop.getId() + ":");
        		//nameCellLabel.id = nameprop.getId()+"_"+nameprop.getLabel()+".label";
        		dojo.place(nameCellLabel, nameCell, "last");
        	}
        	dojo.place(nameCell, namerow,"last");

			var nameValueCell = dojo.doc.createElement("td");
        	//call the property renderer to get the display output
        	var nameValueCellData = this._propertyRendererObj.getRenderedPropertyNode(this._object, nameprop,this);
        	//set the id of the value node...this lets us reference the value later on for dynamic update
            nameValueCellData.id = this.id + "_" + nameprop.getId()+"_"+nameprop.getLabel()+".value";
            
            // If this is not a tooltip (i.e., actual notebook view) create a labeling 
            // association between the label cell and value cell to assist screen reader software
            if (!this._isTooltipProperties) {
                nameCell.id = nameValueCellData.id + "_id";
                dojo.attr(nameValueCellData, "aria-labelledby", nameCell.id);
             }
        	
            // as per the UI guidelines, the NAME needs to be Bold
        	if(this._isTooltipProperties && nameprop.getValueType()=="itemName")
			{
			nameValueCellData.className="tooltipNameBold";
			}
			
			if(this._isTooltipProperties && nameprop.getValueType()=="status")
			{
			dojo.attr(nameValueCellData,"style","float:left;padding-bottom:2px;");
			}
			
            dojo.place(nameValueCellData, nameValueCell,"last");
            dojo.place(nameValueCell, namerow,"last");

        	return namerow;

        },
      
        _handleItemPropertiesChangedEvent: function(/*Object*/ event) {
        	//OVERWRITING handle properies changed event to update the item value if necessary
        	
    		// summary:
    		//		Handle "itemPropertiesChanged" event type.
    		// event:
    		//		An object containing information about what changed in the Dataset.
    	
             
        	if(!this._object){
        		//return if _object is not defined....maybe if event is somehow generated before rendering complete.
        		//in this case that event will be ignored.  when rendering occurs it should just grab the new value
        		return;
        	}
        	
    		//get the items that were changed from the event 
        	var itemschanged = event.getItems();
        	//get the properties that were changed from the event 
        	//two dimension array of item and the actual props changed. 
        	var propschanged = event.getProperties();
        	
        	//for each item
        	for(var i = 0; i< itemschanged.length; i++){
        		//first make sure that this item matches the correct id of the item that is open in the properties notebook
        		if(this._object.getId() == itemschanged[i].getId()){
        			//render each updated property
        			for(var j = 0; j < propschanged[i].length; j++){
        				//call property renderer to create the new value node
        				var newnode = this._propertyRendererObj.getRenderedPropertyNode(itemschanged[i],propschanged[i][j],this);
        				//actually update the value of each item that was changed
        				var orignode = dojo.byId(this.id + "_" + propschanged[i][j].getId()+"_"+propschanged[i][j].getLabel()+".value"); 
        				if (orignode) {
        					dojox.xml.parser.replaceChildren(orignode, newnode);
        				}
        			}
        		}
        	}
        	      	        	
        },
          
        _handleLinkPropertiesChangedEvent: function(/*Object*/ event) {			
    		// summary:
    		//		Coming soon
    		// description:
    		//		Coming soon	
        	// event:
        	//		Coming soon

           	if(!this._object){
        		//return if _object is not defined....maybe if event is somehow generated before rendering complete.
        		//in this case that event will be ignored.  when rendering occurs it should just grab the new value
        		return;
        	}
        	
           	
    		//get the links that were changed from the event 
        	var linkschanged = event.getLinks();
        	//get the properties that were changed from the event 
        	//two dimension array of item and the actual props changed. 
        	var propschanged = event.getProperties();
        	
        	//for each item
        	for(var i = 0; i< linkschanged.length; i++){
        		//first make sure that this item matches the correct id of the item that is open in the properties notebook        	
        		if(this._object.getId() == linkschanged[i].getId()){
        			//render each updated property
        			for(var j = 0; j < propschanged[i].length; j++){
        		
        				//call property renderer to create the new value node
        				var newnode = this._propertyRendererObj.getRenderedPropertyNode(linkschanged[i],propschanged[i][j],this);
        				//actually update the value of each item that was changed
        				var orignode = dojo.byId(this.id + "_" + propschanged[i][j].getId()+"_"+propschanged[i][j].getLabel()+".value"); 				
        				dojox.xml.parser.replaceChildren(orignode, newnode);
        			}

        		}
        	}
        },
        
        _handleRelationshipPropertiesChangedEvent: function(/*Object*/ event) {			
    		// summary:
    		//		Coming soon
    		// description:
    		//		Coming soon	
        	// event:
        	//		Coming soon

           	if(!this._object){
        		//return if _object is not defined....maybe if event is somehow generated before rendering complete.
        		//in this case that event will be ignored.  when rendering occurs it should just grab the new value
        		return;
        	}
        	
           	
    		//get the relationships that were changed from the event 
        	var relationshipsChanged = event.getRelationships();
        	//get the properties that were changed from the event 
        	//two dimension array of item and the actual props changed. 
        	var propsChanged = event.getProperties();
        	
        	//for each item
        	for(var i = 0; i< relationshipsChanged.length; i++){
        		//first make sure that this item matches the correct id of the item that is open in the properties notebook        	
        		if(this._object.getId() == relationshipsChanged[i].getId()){
        			//render each updated property
        			for(var j = 0; j < propsChanged[i].length; j++){
        		
        				//call property renderer to create the new value node
        				var newnode = this._propertyRendererObj.getRenderedPropertyNode(relationshipsChanged[i],propsChanged[i][j],this);
        				//actually update the value of each item that was changed
        				var orignode = dojo.byId(this.id + "_" + propsChanged[i][j].getId()+"_"+propsChanged[i][j].getLabel()+".value"); 				
        				dojox.xml.parser.replaceChildren(orignode, newnode);
        			}
        		}
        	}
        },
        
        _handleItemsRemovedEvent: function(/*Object*/ event) {		
    		// summary:
    		//		Coming soon
    		// description:
    		//		Coming soon
        	// event:
        	//		Coming soon
        	if(!this._object){
        		//return if _object is not defined....maybe if event is somehow generated before rendering complete.
        		//in this case that event will be ignored.  when rendering occurs it should just grab the new value
        		return;
        	}
        	
        	//check if this item is removed...if it is then we need to make a default message
        	//with like a close window button or something
    		//get the items that were changed from the event 
        	var itemschanged = event.getItems();
        	
        	for(var i = 0; i< itemschanged.length; i++){
        		if(this._object.getId() == itemschanged[i].getId()){
        			//display default message
        			var newnode = dojo.doc.createTextNode(this.commonResources.ITEM_REMOVED_MESSAGE);
        			newnode.attr("style");
        			
        			var orignode = this._generalTab.domNode;
        			
        			//replace the old content
        			dojox.xml.parser.replaceChildren(orignode, newnode);
        		}
        	}
        },
        
        _handleLinksRemovedEvent: function(/*Object*/ event) {
    		// summary:
    		//		Coming soon
    		// description:
    		//		Coming soon
        	// event:
        	//		Coming soon
        	if(!this._object){
        		//return if _object is not defined....maybe if event is somehow generated before rendering complete.
        		//in this case that event will be ignored.  when rendering occurs it should just grab the new value
        		return;
        	}
        	
        	//check if this link is removed...if it is then we need to make a default message
        	//with like a close window button or something
    		//get the links that were changed from the event 
        	var linkschanged = event.getLinks();
        	
        	for(var i = 0; i< linkschanged.length; i++){
        		if(this._object.getId() == linkschanged[i].getId()){
        			//display default message
        			var newnode = dojo.doc.createTextNode(this.commonResources.LINK_REMOVED_MESSAGE);
        			newnode.attr("style");
        			
        			var orignode = this._generalTab.domNode;
        			
        			//replace the old content
        			dojox.xml.parser.replaceChildren(orignode, newnode);
        		}
        	}
        }
        
        /*Do NOT need to handle this because this is for adding an item which will never 
         *affect the current item selected
        _handleItemsAddedEvent: function(event) {}*/
        
        //Do NOT need to handle ItemStatusChanged events either..if status is in properties it will fire
        //a property changed event in addition to a status event...the property event contains the actual
        //property object which contains the actual value that needs to be updated.
        
        /* DO NOT need to handle as this is a dataset label, not an item or properties label
        _handleLabelChangedEvent: function(event) {}*/	
        
   
});

});

},
'dijit/form/_ComboBoxMenu':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.add domClass.remove
	"dojo/dom-construct", // domConstruct.create
	"dojo/dom-style", // domStyle.get
	"dojo/keys", // keys.DOWN_ARROW keys.PAGE_DOWN keys.PAGE_UP keys.UP_ARROW
	"../_WidgetBase",
	"../_TemplatedMixin",
	"./_ComboBoxMenuMixin",
	"./_ListMouseMixin"
], function(declare, domClass, domConstruct, domStyle, keys,
			_WidgetBase, _TemplatedMixin, _ComboBoxMenuMixin, _ListMouseMixin){

/*=====
	var _WidgetBase = dijit._WidgetBase;
	var _TemplatedMixin = dijit._TemplatedMixin;
	var _ComboBoxMenuMixin = dijit.form._ComboBoxMenuMixin;
	var _ListMouseMixin = dijit.form._ListMouseMixin;
=====*/

	// module:
	//		dijit/form/_ComboBoxMenu
	// summary:
	//		Focus-less menu for internal use in `dijit.form.ComboBox`

	return declare("dijit.form._ComboBoxMenu",[_WidgetBase, _TemplatedMixin, _ListMouseMixin, _ComboBoxMenuMixin], {
		// summary:
		//		Focus-less menu for internal use in `dijit.form.ComboBox`
		//              Abstract methods that must be defined externally:
		//                      onChange: item was explicitly chosen (mousedown somewhere on the menu and mouseup somewhere on the menu)
		//                      onPage: next(1) or previous(-1) button pressed
		// tags:
		//		private

		templateString: "<div class='dijitReset dijitMenu' data-dojo-attach-point='containerNode' style='overflow: auto; overflow-x: hidden;'>"
				+"<div class='dijitMenuItem dijitMenuPreviousButton' data-dojo-attach-point='previousButton' role='option'></div>"
				+"<div class='dijitMenuItem dijitMenuNextButton' data-dojo-attach-point='nextButton' role='option'></div>"
				+"</div>",

		baseClass: "dijitComboBoxMenu",

		postCreate: function(){
			this.inherited(arguments);
			if(!this.isLeftToRight()){
				domClass.add(this.previousButton, "dijitMenuItemRtl");
				domClass.add(this.nextButton, "dijitMenuItemRtl");
			}
		},

		_createMenuItem: function(){
			return domConstruct.create("div", {
				"class": "dijitReset dijitMenuItem" +(this.isLeftToRight() ? "" : " dijitMenuItemRtl"),
				role: "option"
			});
		},

		onHover: function(/*DomNode*/ node){
			// summary:
			//		Add hover CSS
			domClass.add(node, "dijitMenuItemHover");
		},

		onUnhover: function(/*DomNode*/ node){
			// summary:
			//		Remove hover CSS
			domClass.remove(node, "dijitMenuItemHover");
		},

		onSelect: function(/*DomNode*/ node){
			// summary:
			//		Add selected CSS
			domClass.add(node, "dijitMenuItemSelected");
		},

		onDeselect: function(/*DomNode*/ node){
			// summary:
			//		Remove selected CSS
			domClass.remove(node, "dijitMenuItemSelected");
		},

		_page: function(/*Boolean*/ up){
			// summary:
			//		Handles page-up and page-down keypresses

			var scrollamount = 0;
			var oldscroll = this.domNode.scrollTop;
			var height = domStyle.get(this.domNode, "height");
			// if no item is highlighted, highlight the first option
			if(!this.getHighlightedOption()){
				this.selectNextNode();
			}
			while(scrollamount<height){
				var highlighted_option = this.getHighlightedOption();
				if(up){
					// stop at option 1
					if(!highlighted_option.previousSibling ||
						highlighted_option.previousSibling.style.display == "none"){
						break;
					}
					this.selectPreviousNode();
				}else{
					// stop at last option
					if(!highlighted_option.nextSibling ||
						highlighted_option.nextSibling.style.display == "none"){
						break;
					}
					this.selectNextNode();
				}
				// going backwards
				var newscroll = this.domNode.scrollTop;
				scrollamount += (newscroll-oldscroll)*(up ? -1:1);
				oldscroll = newscroll;
			}
		},

		handleKey: function(evt){
			// summary:
			//		Handle keystroke event forwarded from ComboBox, returning false if it's
			//		a keystroke I recognize and process, true otherwise.
			switch(evt.charOrCode){
				case keys.DOWN_ARROW:
					this.selectNextNode();
					return false;
				case keys.PAGE_DOWN:
					this._page(false);
					return false;
				case keys.UP_ARROW:
					this.selectPreviousNode();
					return false;
				case keys.PAGE_UP:
					this._page(true);
					return false;
				default:
					return true;
			}
		}
	});
});

},
'ibm/usmi/statuspod/JobStatusTip':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define([ "ibm/tivoli/tip/TCTTable",
	"ibm/tivoli/tip/TCTNavModel",
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dijit/_WidgetBase",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	"dojo/i18n!./nls/statuspodResources",
	"dojo/text!./templates/jobStatusTip.html",
	"ibm/usmi/core/utils/Logger",
	"dijit/ProgressBar",
	"idx/oneui/HoverHelpTooltip" ], function (
	TCTTable, TCTNavModel, dojo, declare, lang, WidgetBase, TemplatedMixin, WidgetsInTemplateMixin, MRI, template,
	  Logger) {
	return declare([ WidgetBase,
		TemplatedMixin,
		WidgetsInTemplateMixin ], {
		templateString : template,

		entityNode : "",

		defaultPosition : [ "after",
			"below" ],

		item : null,
		
		itemId: null,
		
		dataset: null,
		
		StartedLabel: MRI.StartedLabel,
		
		StatusLabel: MRI.StatusLabel,
		
		TaskLabel: MRI.TaskLabel,
		
		ProgressLabel: MRI.ProgressLabel,
		
		CreatedByLabel: MRI.CreatedByLabel,
		
		LastRunStatusLabel: MRI.LastRunStatusLabel,
		
		LastRunLabel: MRI.LastRunLabel,
		
		NextRunLabel: MRI.NextRunLabel,
		
		DescriptionLabel: MRI.DescriptionLabel,
		
		JobNameLabel: MRI.Name,
		
		_logger : new Logger("ibm.usmi.statuspod", "JobStatusTip"),

		postMixInProperties : function () {
			this.status = this._getPropertyValue("STATUS");
			this.task = this._getPropertyValue("TASK");
			this.progress = this._getPropertyValue("PROGRESS");
			if (this.progress) {
				try {
					this.progress = dojo.number.format(this.progress, {
						pattern : "#%"
					});
				} catch (e) {}
			}
			this.createdBy = this._getPropertyValue("CREATED_BY");
			this.lastRunStatus = this._getPropertyValue("LAST_RUN_STATUS");
			this.lastRun = this._getPropertyValue("LAST_RUN");
			this.nextRun = this._getPropertyValue("NEXT_RUN");
			this.description = this._getPropertyValue("DESCRIPTION");
			this.jobName=this._getPropertyValue("NAME");
		},

		_getPropertyValue : function (
			propName) {
			var prop, propVal, item;
			item = this.item;
			if (item) {
				prop = item.getProperty(propName);
				if (prop) {
					propVal = prop.getValue();
				}
				if (propVal instanceof Date) {
					propVal = dojo.date.locale.format(propVal, {
						formatLength : "short"
					});
				}
			}
			propVal = propVal || "";
			return propVal;
		},



		postCreate : function () {
			this.inherited(arguments);
			this.jobProgressBar.set("value", this.progress);
		},

		startup : function () {
			this.inherited(arguments);
		},

		show : function () {
			this._refresh();
			idx.oneui.showHoverHelpTooltip(this.tipContent.innerHTML, this.entityNode, /*this.defaultPosition*/null, null, null, null, null, false);
		},
		
		_refresh: function(){
			if(this.dataset){
				var item=this.dataset.getItem({itemId: this.itemId});
				if(item){
					this.item=item;
					this._logger.finest("use the item from dataset!", "_refresh");
				}
			}
			this.postMixInProperties();
			this.buildRendering();
			this.jobProgressBar.set("value", this.progress);
		},

		hide : function () {
			idx.oneui.hideHoverHelpTooltip(this.entityNode);
		}

	});
});
},
'url:dojoe/treetablex/templates/ConfigureColumnsDialog.html':"<div>\r\n\t<div dojoType=\"dijit.Dialog\" dojoAttachPoint=\"dialog\"\ttitle=\"${resources_.CONFIGURE_TREETABLE}\" tabindex=0 role=\"dialog\">\r\n\t\t<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" role=\"presentation\" style=\"width:100%height:100%\">\r\n\t\t\t<tr height=\"20%\" style=\"vertical-align:top;\"><td><label>${resources_.COLUMNS_VISIBLE}</label></td><tr>\r\n\t\t\t<tr><td>\r\n\t\t\t\t<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" role=\"presentation\">\r\n\t\t\t\t\t<tbody>\r\n\t\t\t\t\t\t<tr><td>\r\n\t\t\t\t\t\t\t\t<span dojoAttachPoint=\"selectAllButton\" role=\"button\">\r\n\t\t\t\t\t\t\t</td>\r\n\t\t\t\t\t\t\t<td>\r\n\t\t\t\t\t\t\t\t<span dojoAttachPoint=\"deselectAllButton\" role=\"button\">\r\n\t\t\t\t\t\t\t</td>\r\n\t\t\t\t\t\t</tr>\r\n\t\t\t\t\t</tbody>\r\n\t\t\t\t</table>\r\n\t\t\t</td></tr>\r\n\t\t\t<tr><td>\r\n\t\t\t\t<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" width=\"100%\"  role=\"presentation\">\r\n\t\t\t\t\t<tbody>\r\n\t\t\t\t\t\t<tr valign=\"center\">\r\n\t\t\t\t\t\t\t<td width=\"90%\" height=\"100%\">\t\t\t\t\t\r\n\t\t\t\t\t\t\t\t<div id=\"${id}tabs\" dojoType=\"dijit.layout.BorderContainer\" class=\"dialog\" style=\"height:200px;width:260px;\">\r\n\t\t\t\t\t\t\t\t\t<div dojoType=\"dijit.layout.ContentPane\" region=\"top\" >\t\t\t\t\r\n\t\t\t\t\t\t\t\t\t<div  dojoAttachPoint=\"enhancedGridNode\" style=\"height:170px;width:200px;\" role=\"grid\"></div>\r\n\t\t\t\t\t\t\t\t\t</div>\r\n\t\t\t\t\t\t\t\t</div>\r\n\t\t\t\t\t\t\t</td>\r\n\t\t\t\t\t\t\t<td width=\"10%\" height=\"100%\">\r\n\t\t\t\t\t\t\t\t<table role=\"presentation\">\r\n\t\t\t\t\t\t\t\t\t<tr><td><span dojoAttachPoint=\"upButton\" role=\"button\"></td></tr>  \r\n\t\t\t\t\t\t\t\t\t<tr><td><span dojoAttachPoint=\"downButton\" role=\"button\"></td></tr> \r\n\t\t\t\t\t\t\t\t</table>\r\n\t\t\t\t\t\t</td></tr>\t\r\n\t\t\t\t\t</tbody>\t\t\r\n\t\t\t\t</table>\t\t\t\t\t\t\t\r\n\t\t\t</td></tr>\r\n\t\t\t<tr><td>\r\n\t\t\t\t<button value=\"${resources_.OK}\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:onOK,onKeypress:onOK\" role=\"button\">${resources_.OK}</button>\r\n\t\t\t\t&nbsp;&nbsp;<button value=\"${resources_.CANCEL}\" dojoType=\"dijit.form.Button\" dojoAttachPoint=\"cancelButton\" dojoAttachEvent=\"onClick:onCancel,onKeypress:onCancel\" role=\"button\">${resources_.CANCEL}</button>\r\n\t\t\t</td></tr>\r\n\t\t</table>\r\n\t</div>\r\n</div>\r\n",
'ibm/tivoli/navmodel/DatasetInfo':function(){
// wrapped by build app
define(["dijit","dojo","dojox"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */
dojo.provide("ibm.tivoli.navmodel.DatasetInfo"); 

////////////////////////////////////
// DatasetInfo
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.DatasetInfo", null, {
	// private members       
	_dataset: null,
	_version: null,
	
	/*=====
	constructor_kwArgs: function(dataset, version) {
		// dataset: Object
		//		json dataset object
		// version: Float
		//		optional: version used to communicate with rest service (i.e. json version)
	},
	=====*/
	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs Provider object.
		this._version = kwArgs.version;
		this._dataset = kwArgs.dataset;
		if (!this._dataset) {
			this._dataset = {};
		}
	},

	getAttribute: function(key) {
		// summary:
		//		Returns general attribute for requested key
		return this._dataset[key];
	},

	setAttribute: function(key, value) {		
		// summary:
		//		Set general attribute for requested key
		this._dataset[key] = value;
	},
	
	removeAttribute: function(key) {
		// summary:
		//		Remove general attribute for requested key
		delete this._dataset[key];
	},
	
	getProviderId: function() {
		// summary:
		//		Returns the providerId of this dataset
		return this._dataset.providerId;
	},
	
	getDatasourceId: function() {
		// summary:
		//		Returns the datasourceId of this dataset
		return this._dataset.datasourceId;
	},
	
	getId: function() {
		// summary:
		//		Returns the id of this dataset
		return this._dataset.id;
	},
	
	getLabel: function() {
		// summary:
		//		Returns the label of this dataset
		return this._dataset.label;
	},
	
	getDescription: function() {
		// summary:
		//		Returns the description of this dataset
		return this._dataset.description;
	},
	
	getType: function() {
		// summary:
		//		Returns the type of this dataset.
		return this._dataset.type;
	}
});

});

},
'dojoe/Util':function(){
// wrapped by build app
define(["dijit","dojo","dojox"], function(dijit,dojo,dojox){
dojo.provide("dojoe.Util");

/*Common utility functions for all*/

isNumber = function(value){
	return (typeof value === 'number' && isFinite(value));
}

startsWith = function(all, part){
	return (all.indexOf(part) === 0);
}

endsWith = function(all, part){
	return (all.lastIndexOf(part) === all.length - part.length);
}

contains = function(all, part){
	return (all.lastIndexOf(part) >= 0);
}

trim = function(stringToTrim) {
	return stringToTrim.replace(/^\s+|\s+$/g,"");
}

ltrim = function(stringToTrim) {
	return stringToTrim.replace(/^\s+/,"");
}

rtrim = function(stringToTrim) {
	return stringToTrim.replace(/\s+$/,"");
}

});

},
'ibm/tivoli/navmodel/TopoWidgetDef':function(){
// wrapped by build app
define("ibm/tivoli/navmodel/TopoWidgetDef", ["dijit","dojo","dojox","dojo/require!ibm/tivoli/navmodel/WidgetDef"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.TopoWidgetDef"); 
 
// includes
dojo.require("ibm.tivoli.navmodel.WidgetDef");

////////////////////////////////////
// TopoWidgetDef
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.TopoWidgetDef", ibm.tivoli.navmodel.WidgetDef, {

	/*=====
	constructor_kwArgs: function(baseUrl, provider, datasource, widget, definition, type, version) {
		// baseUrl: String
		//		base url for identifying rest service
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// widget: String
		//		widget type in form <type>-<version>
		// definition: String
		//		widget definition id in form <id>-<version>
		// type: String
		//		definition type:  "user", "shared", or "base"
		// version: Float
		//		version used to communicate with rest service
	},
	=====*/ 
	
	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs TopoWidgetDef object.
	},

	/////////////////////////////
	// public methods
	/////////////////////////////

	validateDefaults: function() {
		// summary:
		//		This method is called after the widget def has been read from the server.
		//		Validate defaults that might not be present in the widget def contents.
		
		var layouts = this.getLayouts();
		if (layouts) {
			var length = layouts.length;
			for (var index = 0;  index < length;  ++index) {
				var layout = layouts[index];
				switch (layout.algorithm) {
					case "tree":
						if (!layout.layoutMode) {
							layout.layoutMode = "alternating radial";
						}
						if (!layout.flowDirection) {
							layout.flowDirection = "bottom";
						}
						if (!layout.globalLinkStyle) {
							layout.globalLinkStyle = "straight";
						}
						if (!layout.connectorStyle) {
							layout.connectorStyle = "automatic";
						}
						if (!layout.globalAlignment) {
							layout.globalAlignment = "center";
						}
						if (!layout.levelAlignment) {
							layout.levelAlignment = "center";
						}
						if (!layout.aspectRatio) {
							layout.aspectRatio = "1";
						}
						break;
					case "hierarchical":
						if (!layout.flowDirection) {
							layout.flowDirection = "bottom";
						}
						if (!layout.levelJustification) {
							layout.levelJustification = "bottom";
						}
						if (!layout.levelingStrategy) {
							layout.levelingStrategy = "optimal";
						}
						if (!layout.globalLinkStyle) {
							layout.globalLinkStyle = "straight";
						}
						if (!layout.connectorStyle) {
							layout.connectorStyle = "automatic";
						}
						break;
					case "forceDirected":
						if (!layout.layoutMode) {
							layout.layoutMode = "fast multilevel";
						}
						if (!layout.preferredLinksLength) {
							layout.preferredLinksLength = "25";
						}
						if (!layout.linkStyle) {
							layout.linkStyle = "straight";
						}
						if (!layout.maxAllowedMovePerIteration) {
							layout.maxAllowedMovePerIteration = "5";
						}
						if (!layout.allowedNumberOfIterations) {
							layout.allowedNumberOfIterations = "250";
						}
						if (!layout.convergenceThreshold) {
							layout.convergenceThreshold = "0.1";
						}						
						break;
					case "shortLink":
						if (!layout.globalLinkStyle) {
							layout.globalLinkStyle = "direct";
						}
						if (!layout.globalSelfLinkStyle) {
							layout.globalSelfLinkStyle = "two bends orthogonal";
						}
						if (!layout.globalConnectorStyle) {
							layout.globalConnectorStyle = "automatic";
						}
						if (!layout.flowDirection) {
							layout.flowDirection = "bottom";
						}
						if (!layout.levelJustification) {
							layout.levelJustification = "bottom";
						}
						if (!layout.levelingStrategy) {
							layout.levelingStrategy = "optimal";
						}
						if (!layout.connectorStyle) {
							layout.connectorStyle = "automatic";
						}
						if (!layout.linkOffset) {
							layout.linkOffset = "1";
						}
						if (!layout.minFinalSegmentLength) {
							layout.minFinalSegmentLength = "10";
						}
						break;
					case "longLink":
						if (!layout.globalLinkStyle) {
							layout.globalLinkStyle = "direct";
						}
						if (!layout.flowDirection) {
							layout.flowDirection = "bottom";
						}
						if (!layout.levelJustification) {
							layout.levelJustification = "bottom";
						}
						if (!layout.levelingStrategy) {
							layout.levelingStrategy = "optimal";
						}
						if (!layout.connectorStyle) {
							layout.connectorStyle = "automatic";
						}
						if (!layout.verticalMinOffset) {
							layout.verticalMinOffset = "3";
						}
						if (!layout.horizontalMinOffset) {
							layout.horizontalMinOffset = "3";
						}
						break;
					case "grid":
						if (!layout.layoutMode) {
							layout.layoutMode = "tile to rows";
						}
						if (!layout.globalHorizontalAlignment) {
							layout.globalHorizontalAlignment = "center";
						}
						if (!layout.globalVerticalAlignment) {
							layout.globalVerticalAlignment = "center";
						}
						if (!layout.maxNumberOfNodesPerRowOrColumn) {
							layout.maxNumberOfNodesPerRowOrColumn = "3";
						}
				}
			}
		}
	},
	
	destroy: function() {
		// summary:
		//		Do any necessary clean up 
		if (!this.isDestroyed()) {
			// call base class
			this.inherited(arguments);
		}
	},

	getDepth: function() {
		var returnValue = this._widgetDef.topology.depth;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getDepth) {
				returnValue = parent.getDepth();
			}
		}
		if(returnValue){
			return returnValue;
		} else {
			return -1;
		}
	},
	
	getInitialZoomLevel: function() {
		var returnValue = this._widgetDef.topology.initialZoomLevel;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getInitialZoomLevel) {
				returnValue = parent.getInitialZoomLevel();
			}
		}
		if(returnValue){
			return returnValue;
		} else {
			return -1; //Default to -1 which would result in just fitting the contents
		}
	},	

	zoomOnObjectAdded: function() {
		return this._widgetDef.topology.zoomOnObjectAdded;
	},

	zoomLevelOnObjectAdded: function() {
		return this._widgetDef.topology.zoomLevelOnObjectAdded;
	},

	getMaxDepth: function() {
		var returnValue = this._widgetDef.topology.maxDepth;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getMaxDepth) {
				returnValue = parent.getMaxDepth();
			}
		}
		if(returnValue){
			return returnValue;
		} else {
			return -1;
		}
	},

	useNodeLevelAndPosition: function() {
		return this._widgetDef.topology.useNodeLevelAndPosition;
	},

	getNodeOrderDirection: function() {
		return this._widgetDef.topology.nodeOrderDirection;
	},

	getNodeOrderProperty: function() {
		return this._widgetDef.topology.nodeOrderProperty;
	},

	getBackgroundColor: function() {
		var returnValue = this._widgetDef.topology.backgroundColor;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getBackgroundColor) {
				returnValue = parent.getBackgroundColor();
			}
		}
		if(returnValue){
			return returnValue;
		} else {
			return -1;
		}
	},
	
	/////////////////////////////
	// footer section data
	/////////////////////////////
	
	getIncludeFooter: function() {
		var returnValue = this._widgetDef.topology.includeFooter;
		if(returnValue===undefined) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getIncludeFooter) {
				returnValue = parent.getIncludeFooter();
			}
		}
		if(returnValue!==undefined){
			return returnValue;
		} else {
			return true;
		}
	},
	
	setIncludeFooter: function(includeFooter) {
		this._widgetDef.topology.includeFooter = includeFooter;
	},
	
	getHideFooterDateTime: function(){
		var returnValue = this._widgetDef.topology.hideFooterDateTime;
		if(returnValue===undefined) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getHideFooterDateTime) {
				returnValue = parent.getHideFooterDateTime();
			}
		}
		if(returnValue!==undefined){
			return returnValue;
		} else {
			return false;
		}
	},

	setHideFooterDateTime: function(hideFooterDateTime) {
		this._widgetDef.topology.hideFooterDateTime = hideFooterDateTime;
	},
	
	getFilters: function(){
		var returnValue = this._widgetDef.topology.filters;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getFilters) {
				returnValue = parent.getFilters();
			}
		}		
		return returnValue;			
	},

	/////////////////////////////
	// search section data
	/////////////////////////////

	isAdvSearchEnabled: function() {
		var returnValue = this._widgetDef.topology.isAdvSearchEnabled;
		if(returnValue===undefined) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.isAdvSearchEnabled) {
				returnValue = parent.isAdvSearchEnabled();
			}
		}
		if(returnValue!==undefined){
			return returnValue;
		} else {
			return false;
		}
	},
	
	getAdvSearchProperties: function() {
		var returnValue = this._widgetDef.topology.advSearchProperties;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getAdvSearchProperties) {
				returnValue = parent.getAdvSearchProperties();
			}
		}		
		return returnValue;	
	},	

	/////////////////////////////
	// layout section data
	/////////////////////////////	
	getLayouts: function() {
		var returnValue = this._widgetDef.topology.layout;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getLayouts) {
				returnValue = parent.getLayouts();
			}
		}		
		return returnValue;	
	},
	
	updateLayouts: function(layouts) {
		var delta = this.getWidgetDelta("user");
		if (delta) {
			delta.setSectionAttribute({sectionKey: "topology", attrKey: "layout", attrValue: layouts}); 
		} 
	},
	
	clearLayouts: function() {
		var delta = this.getWidgetDelta("user");
		if (delta) {
			delta.removeSectionAttribute({sectionKey: "topology", attrKey: "layout"}); 
		}
	},
	
	getLayout: function(/*String*/ id) {
		var layouts = this.getLayouts();
		for(var i=0;i<layouts.length;i++){
			if(layouts[i].id === id){
				return layouts[i];
			}
		}
	},
	
	getDefaultLayout: function() {
		var layouts = this.getLayouts();
		for(var i=0;i<layouts.length;i++){
			if(layouts[i].isDefault){
				return layouts[i];
			}
		}
	},
	
	getDefaultLayoutId: function() {
		var layouts = this.getLayouts();
		for(var i=0;i<layouts.length;i++){
			if(layouts[i].isDefault){
				return layouts[i].id;
			}
		}
	},
		
	getRenderRelationshipsInsteadOfLinks: function(){
		var returnValue = this._widgetDef.topology.renderRelationshipsInsteadOfLinks;
		if(returnValue===undefined) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getRenderRelationshipsInsteadOfLinks) {
				returnValue = parent.getRenderRelationshipsInsteadOfLinks();
			}
		}
		if(returnValue!==undefined){
			return returnValue;
		} else {
			return false;
		}		
	}
	
	/////////////////////////////
	// private methods
	/////////////////////////////	

});

});

},
'dojoe/wizard/PageContainer':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/i18n!dojoe/wizard/nls/Wizard","dojo/require!dijit/layout/BorderContainer,dijit/layout/StackContainer,dijit/layout/ContentPane,dijit/form/Button,dijit/form/_FormMixin,dojo/i18n"], function(dijit,dojo,dojox){
dojo.provide("dojoe.wizard.PageContainer");

dojo.require("dijit.layout.BorderContainer");
dojo.require("dijit.layout.StackContainer");
dojo.require("dijit.layout.ContentPane");
dojo.require("dijit.form.Button");
dojo.require("dijit.form._FormMixin");
dojo.require("dojo.i18n");
dojo.requireLocalization("dojoe.wizard", "Wizard");

/*
 * Page container use by notebook and wizard widgets.
 *
 * Events:
 *  OnPageSelected: when a page is made active
 *  OnOK: when the Finish/OK button is clicked
 *  OnCancel: when the Cancel button is clicked
 */

dojo.declare("dojoe.wizard.PageContainer", [dijit._Container],
	{
		mode: "",          // "n" for notebook or "w" for wizard (set by subclass)
		layout: "v",       // "vertical" or "horizontal" (actually, anything starting with "v" | "h")
		nested: false,     // contained <form> is used for containerNode (default), if true <div> used instead
		formId: "",        // id of form to submit when container is being nested
		action: ".",       // the action attribute on the contained <form>
		method: "POST",    // the method attribute on the contained <form>
		_pages: null,
		_pageChangeConnections: null,
		_containerSubscriptions: null,
		_origOK: null,
  
		constructor: function () {
			this._origOK = this.OnOK;
			this._pages = [];
			this._pageChangeConnections = {};
			this._containerSubscriptions = [];
		},
		
		postMixInProperties: function() {
			if (this.layout && this.layout.length > 0) {
				this.layout = this.layout.charAt(0).toLowerCase();
			}
			this.inherited(arguments);
		},
		
		buildRendering: function () {
			this.inherited(arguments);
			this._pageContainer.containerNode = this.containerNode;
			this._addFooterButtons();
    
			// if ToC layout is horizontal... 
			if ('h' === this.layout) {
				// move toc region and update class
				var child = this._tocArea;
				this._borderArea.removeChild(child);
				child.region = "top";
				//Latha : 070212 _setClassAttr is deprecated for ContentPane in Dojo 1.7. It is replaced with set method.
				child.set("class", "tiv-toc-horizontal");
				this._borderArea.addChild(child, 0);                    
			}
		},
		
		postCreate: function () {
			this.inherited(arguments);
		},
		
		startup: function () {
			if (this._started) { 
				return; 
			}
			this.inherited(arguments);
			this._containerSubscriptions.push(dojo.subscribe(this._pageContainer.id + "-selectChild", this, this._onPageSelected));
			this._containerSubscriptions.push(dojo.subscribe(this._pageContainer.id + "-addChild", this, this._onPageAdded));
			this._containerSubscriptions.push(dojo.subscribe(this._pageContainer.id + "-removeChild", this, this._onPageRemoved));
			// get the pages provided in markup
			this._pages = this._pageContainer.getChildren();
			// save reference to ourselves as an attribute so each page has access
			// must occur before _buildToC
			dojo.forEach(this._pages, this._setContainerType, this);
			// build the initial table of contents from the pages provided in markup
			this._buildToC();
			// Setup each page
			dojo.forEach(this._pages, this._pageContainer._setupChild, this._pageContainer);
			dojo.forEach(this._pages, function (p, i) {
				this._onPageAdded(p, i);
			}, this);
    
			var selectedChild = null;
			dojo.some(this._pages, function (child) {
				if (child.selected) {
					selectedChild = child;
				}
				return child.selected;
			}, this);
			if (!selectedChild && this._pages[0]) {
				selectedChild = this._pages[0];
			}
			if (selectedChild) {
				//The _onPageSelected function wasn't getting called for a selectPage
				//this.selectPage(selectedChild);
				this._onPageSelected(selectedChild);
			}
			this._outerContainer.resize();
		},  
		
		_addFooterButtons: function () {
			// subclass should override
		}, 
		
		_setContainerType: function (page) {
			// subclass should override
		},  
		
		_addToCItemConnections: function (toc_item) {
			// subclass should override
		},
		
		/**
		* submit the contained form
		* wire this function to the OnOK event
		*/
		submit: function (wiz) {
			var _form = (this.nested) ? this.formId : this.id + "_form";
			if ('' === _form) {
				return null;
			}
			// our form is the containerNode
			var f;
			switch (this.method.toLowerCase()) {
				case "get":
				f = dojo.xhrGet;
				break;
				case "post":
				f = dojo.xhrPost;
				break;
				case "put":
				f = dojo.xhrPut;
				break;
				default:
				f = dojo.xhrPost;
			}
			var dfd = f({
				url: this.action,
				preventCache: true,
				form: _form, 
				load: function (response) {
				},
				error: function(response) {
				}
			});
			return dfd;
		},
		
		_buildToC: function () {
			if (this._toc) {    
				while (this._toc.firstChild) {
					this._toc.removeChild(this._toc.firstChild);
				}
				var page_count = this._pages.length;
				dojo.forEach(this._pages, function (p, i) {
					var toc_item = p.buildTOCRendring();
					if (toc_item != null) {
						dojo.place(toc_item, this._toc, "last");
						this._addToCItemConnections(toc_item);
					}
				}, this);    
			}
		},
		
		destroy: function () {
			var alerter;
			while (alerter = this._containerSubscriptions.pop()) {
				dojo.unsubscribe(alerter);
			}
			var p;
			for(p in this._pageChangeConnections) {
				dojo.disconnect(this._pageChangeConnections[p]);
			}
			this.inherited(arguments);
		},
		
		destroyRecursive: function () {
			this.inherited(arguments);
		},
		
		selectPage: function (p) {	//p is a widget
			this._pageContainer.selectChild(p);
		},
		
		addPage: function (p) {	//p is a widget
			this._setContainerType(p);
			this._pageContainer.addChild(p);
			this._pages = this._pageContainer.getChildren();
			this._buildToC();
			this.updateUI();
		}, 
		
		removePage: function (p) {	//p is a widget
			// If removing selected page, StackContainer will select next
			// page by default.
			this._pageContainer.removeChild(p);
			this._pages = this._pageContainer.getChildren();
			this._buildToC();
			this.updateUI();
		}, 
		
		updateUI: function () {
			var curr_page = this._pageContainer.selectedChildWidget;
			if (curr_page!=null) {
				// update the ToC
				if (this._toc) {
					dojo.query("li[pageID]", this._toc).forEach(function (n, i) {
						var pid = dojo.attr(n, "pageID");
						var p = dijit.byId(pid);
						if (p.visited) {
							dojo.query("a",n).forEach(function (a) {
								a.setAttribute("title","visited");
							},this);
						}
						dojo[(p && p.visited) ? "addClass" : "removeClass"](n, "tiv-page-visited");
						dojo[(p && (!p.visited || p.validated)) ? "removeClass" : "addClass"](n, "tiv-page-invalid");
						dojo[(p == curr_page) ? "addClass" : "removeClass"](n, "tiv-page-selected");
					});
				}
				//To make the current page visible
				if (curr_page) {
					dojo.removeClass(curr_page.domNode,"dijitHidden");
					dojo.addClass(curr_page.domNode,"dijitVisible");
				}
			}
		},
		
		_onPageSelected: function (page) {
			this.OnPageSelected(this, page);
			if ("function" == typeof page.OnPageSelected) {
				page.OnPageSelected(this);
			}
			this.updateUI();
			// if the page can handle this function
			if ("function" == typeof page.AfterPageSelected) {
				// let the page get final control
				page.AfterPageSelected(this);
			}
		},
		
		_onPageContentChange: function (page) {
			this.updateUI();
		},
		
		_onPageAdded: function (page, insertIndex) {
			this._pageChangeConnections[page.id] = dojo.connect(page, "OnPageContentChange", this, this._onPageContentChange);
			this.updateUI();
		},
		
		_onPageRemoved: function (page) {
			dojo.disconnect(this._pageChangeConnections[page.id]);
			delete this._pageChangeConnections[page.id];
			this.updateUI();
		},
		
		_onClickFinish: function (evt) {
			this.OnOK(this);
		},
		
		_onClickCancel: function (evt) {
			this.OnCancel(this);
		},
		
		// fired events
		OnPageSelected: function (wiz, page) {
			// override or connect to me
		},
		
		OnOK: function (wiz) {
			// override or connect to me
			// if this is still the same function as it was originally,
			// then nobody has connected to the event. do the default
			if (this._origOK === this.OnOK) {
				// nothing connected, do the default
				this.submit(this);
			}
		},
		
		OnCancel: function (wiz) {
			// override or connect to me
		},
  
		/**
		* Enable (or disable) the Finish/OK button
		*/
		enableFinishButton: function (shouldEnable) {
			this._finishButton.attr("disabled", !shouldEnable);
		},

		/** 
		* Enable (or disable) the Cancel button
		*/
		enableCancelButton: function (shouldEnable) {
			this._cancelButton.attr("disabled", !shouldEnable);
		},

		/**
		* Hide (or show) the Finish/OK button
		*/
		hideFinishButton: function (shouldHide) {
			if (shouldHide) {
				dojo.style(this._finishButton.domNode, "display", "none");
			} else {
				dojo.style(this._finishButton.domNode, "display", "inline-block");
			}
		},

		/**
		* Hide (or show) the Cancel button
		*/
		hideCancelButton: function (shouldHide) {
			if (shouldHide) {
			    dojo.style(this._cancelButton.domNode, "display", "none");
			} else {
				dojo.style(this._cancelButton.domNode, "display", "inline-block");
			}
		},
  
		/**
		* Returns the currently selected page
		*/
		getSelectedPage: function () {
			return this._pageContainer.selectedChildWidget; 
		},
  
		/**
		* Resizes this container according to the new dimensions.
		* @param newSize  the new dimensions with properties w for width 
		*                 and h for height
		*/
		resize: function (newSize) {
			if (newSize != null) {
				if (newSize.w != null) {
					dojo.style(this.domNode, 'width', newSize.w + 'px');
				}
				if (newSize.h != null) {
				    dojo.style(this.domNode, 'height', newSize.h + 'px');
				}
			}
			this._outerContainer.resize(newSize);
		}  
	});

});

},
'gridx/modules/pagination/PaginationBarDD':function(){
define([
	"dojo/_base/declare",
	"../../core/_Module",
	"./_PaginationBarBase",
	"./DropDownPager",
	"dijit/form/FilteringSelect",
	"dijit/form/Select"
], function(declare, _Module, _PaginationBarBase, Pager, FilteringSelect, Select){

	return declare(/*===== "gridx.modules.pagination.PaginationBar", =====*/_PaginationBarBase, {
		// summary:
		//		This module implements a pagination bar UI that uses drop down lists for pages and page sizes.
		//		This implementation saves more horizontal space compared to the link button version of pagination bar.

		// stepperClass: [private]
		stepperClass: FilteringSelect,

		// sizeSwitchCalss [private]
		sizeSwitchClass: Select,

	/*=====
		// Configurable texts on the pagination bar:
		pageSizeAllText: '',
		descriptionTemplate: '',
		descriptionSelectionTemplate: '',
	=====*/

		// pagerClass: [private]
		pagerClass: Pager
	});	
});

},
'url:ibm/usmi/dashboard/templates/ResourceDashBoard.html':"<div class=\"res-dashboard\">\r\n<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n\t<div id=\"dashbaordFullContainer\" data-dojo-type=\"dijit.layout.BorderContainer\" data-dojo-props=\"design: 'headline', gutters: false\" class=\"spacious\" style=\"padding:0; margin:0;\">\r\n\t\t<div id=\"dashBoardGroupsContainer\" data-dojo-props=\"region: 'left'\" data-dojo-type=\"dijit.layout.ContentPane\">\r\n\t\t\t\t\t<div id=\"chassisPane\" class=\"dashboardGroup\">\r\n\t\t\t\t\t\t<div id=\"chassis\"></div>\r\n\t\t\t\t\t</div>\r\n\t\t\t\t\t<div id=\"virtualserversPane\" class=\"dashboardGroup\">\r\n\t\t\t\t\t\t<div id=\"virtualservers\"></div>\r\n\t\t\t\t\t</div>\r\n\t\t\t\t\t<div id=\"networkPane\" class=\"dashboardGroup\">\r\n\t\t\t\t\t\t<div id=\"network\"></div>\r\n\t\t\t\t\t</div>\r\n\t\t\t\t\t<div id=\"storagePane\" class=\"dashboardGroup\">\r\n\t\t\t\t\t\t<div id=\"storage\"></div>\r\n\t\t\t\t\t</div>\r\n\t\t\t\t\t<div id=\"favoritesPane\" class=\"dashboardGroup\">\r\n\t\t\t\t\t\t<div id=\"favorites\"></div>\r\n\t\t\t\t\t</div>\r\n\t\t\t\t\t<div id=\"systemsPane\" data-dojo-type=\"dijit.layout.ContentPane\" data-dojo-props=\"splitter:false, region:'leading', layoutPriority:1\" class=\"dashboardGroup\">\r\n\t\t\t\t\t\t<div id=\"systems\"></div>\r\n\t\t\t\t\t</div>\r\n\t\t</div>\r\n\t\t\r\n\t\t<div id=\"dashboardSelectorContainer\" data-dojo-props=\"region: 'center', splitter: false\" data-dojo-type=\"dijit.layout.ContentPane\" class=\"dashboardSelectorContainer\">\r\n\t\t\t<div id=\"dashboard_selectorTreeTopper\" style=\"width:100%;\"></div>\r\n\t\t\t<div id=\"dashboard_selectorTree\" style=\"width:100%; height:100%;\"></div>\r\n\t\t</div>\r\n\r\n\t</div>\r\n </div>",
'url:ibm/usmi/statuspod/templates/jobMenuItemLabel.html':"<table>\r\n<!--\r\n\r\nLicensed Materials - Property of IBM\r\n\r\n(C) Copyright IBM Corp. 2012 All Rights Reserved\r\n\r\nUS Government Users Restricted Rights - Use, duplicate or\r\ndisclosure restricted by GSA ADP Schedule Contract with\r\nIBM Corp.\r\n\r\n-->\r\n\t<tbody>\r\n\t\t<tr>\r\n\t\t\t<td class=\"statusPod-sourceName\"><div style=\"width:200px;text-overflow:ellipsis;overflow: hidden;white-space:nowrap;\">${Name}</div></td>\r\n\t\t\t<td class=\"statusPod-shortTime\">${StartedLabel}: ${lastRun}</td>\r\n\t\t</tr>\r\n\t<tbody>\r\n</table>",
'dojo/parser':function(){
define(
	["./_base/kernel", "./_base/lang", "./_base/array", "./_base/config", "./_base/html", "./_base/window", "./_base/url",
	 	"./_base/json", "./aspect", "./date/stamp", "./has", "./query", "./on", "./ready"],
	function(dojo, dlang, darray, config, dhtml, dwindow, _Url, djson, aspect, dates, has, query, don, ready){

// module:
//		dojo/parser
// summary:
//		The Dom/Widget parsing package

new Date("X"); // workaround for #11279, new Date("") == NaN

if (1) {
	var form = document.createElement("form");
	// Test if DOMNode.attributes only lists the attributes the user specified, not attributes w/default values.
	has.add("dom-attributes-explicit", form.attributes.length == 0);

	// IE8 will erroneously list a few attributes that weren't specified,
	// but we know to skip them because they have a specified flag which is false
	has.add("dom-attributes-specified-flag", form.attributes.length < 40);

	// Otherwise, it's IE6-7 form.attributes will list hundreds of values, need to do outerHTML instead.
}

dojo.parser = new function(){
	// summary:
	//		The Dom/Widget parsing package

	var _nameMap = {
		// Map from widget name (ex: "dijit.form.Button") to structure mapping
		// lowercase version of attribute names to the version in the widget ex:
		//	{
		//		label: "label",
		//		onclick: "onClick"
		//	}
	};
	function getNameMap(proto){
		// summary:
		//		Returns map from lowercase name to attribute name in class, ex: {onclick: "onClick"}
		var map = {};
		for(var name in proto){
			if(name.charAt(0)=="_"){ continue; }	// skip internal properties
			map[name.toLowerCase()] = name;
		}
		return map;
	}
	// Widgets like BorderContainer add properties to _Widget via dojo.extend().
	// If BorderContainer is loaded after _Widget's parameter list has been cached,
	// we need to refresh that parameter list (for _Widget and all widgets that extend _Widget).
	aspect.after(dlang, "extend", function(){
		_nameMap = {};
	}, true);

	// Map from widget name (ex: "dijit.form.Button") to a map of { "list-of-mixins": ctor }
	// if "list-of-mixins" is "__type" this is the raw type without mixins
	var _ctorMap = {};


	function getCtor(type){
		var map = _ctorMap[type] || (_ctorMap[type] = {});
		return map["__type"] || (map["__type"] = (dlang.getObject(type) || require(type)));
	}

	this._functionFromScript = function(script, attrData){
		// summary:
		//		Convert a <script type="dojo/method" args="a, b, c"> ... </script>
		//		into a function
		// script: DOMNode
		//		The <script> DOMNode
		// attrData: String
		//		For HTML5 compliance, searches for attrData + "args" (typically
		//		"data-dojo-args") instead of "args"
		var preamble = "";
		var suffix = "";
		var argsStr = (script.getAttribute(attrData + "args") || script.getAttribute("args"));
		if(argsStr){
			darray.forEach(argsStr.split(/\s*,\s*/), function(part, idx){
				preamble += "var "+part+" = arguments["+idx+"]; ";
			});
		}
		var withStr = script.getAttribute("with");
		if(withStr && withStr.length){
			darray.forEach(withStr.split(/\s*,\s*/), function(part){
				preamble += "with("+part+"){";
				suffix += "}";
			});
		}
		return new Function(preamble+script.innerHTML+suffix);
	};

	this.instantiate = /*====== dojo.parser.instantiate= ======*/ function(nodes, mixin, options) {
		// summary:
		//		Takes array of nodes, and turns them into class instances and
		//		potentially calls a startup method to allow them to connect with
		//		any children.
		// nodes: Array
		//		Array of DOM nodes
		// mixin: Object?
		//		An object that will be mixed in with each node in the array.
		//		Values in the mixin will override values in the node, if they
		//		exist.
		// options: Object?
		//		An object used to hold kwArgs for instantiation.
		//		See parse.options argument for details.

		mixin = mixin || {};
		options = options || {};

		var dojoType = (options.scope || dojo._scopeName) + "Type",		// typically "dojoType"
			attrData = "data-" + (options.scope || dojo._scopeName) + "-",// typically "data-dojo-"
			dataDojoType = attrData + "type";						// typically "data-dojo-type"

		var list = [];
		darray.forEach(nodes, function(node){
			var type = dojoType in mixin ? mixin[dojoType] : node.getAttribute(dataDojoType) || node.getAttribute(dojoType);
			if(type){
				list.push({
					node: node,
					"type": type
				});
			}
		});

		// Instantiate the nodes and return the objects
		return this._instantiate(list, mixin, options);
	};

	this._instantiate = /*====== dojo.parser.instantiate= ======*/ function(nodes, mixin, options){
		// summary:
		//		Takes array of objects representing nodes, and turns them into class instances and
		//		potentially calls a startup method to allow them to connect with
		//		any children.
		// nodes: Array
		//		Array of objects like
		//	|		{
		//	|			type: "dijit.form.Button",
		//	|			node: DOMNode,
		//	|			scripts: [ ... ],	// array of <script type="dojo/..."> children of node
		//	|			inherited: { ... }	// settings inherited from ancestors like dir, theme, etc.
		//	|		}
		// mixin: Object
		//		An object that will be mixed in with each node in the array.
		//		Values in the mixin will override values in the node, if they
		//		exist.
		// options: Object
		//		An options object used to hold kwArgs for instantiation.
		//		See parse.options argument for details.

		var thelist = [];

		// Precompute names of special attributes we are looking for
		// TODO: for 2.0 default to data-dojo- regardless of scopeName (or maybe scopeName won't exist in 2.0)
		var dojoType = (options.scope || dojo._scopeName) + "Type",		// typically "dojoType"
			attrData = "data-" + (options.scope || dojo._scopeName) + "-",// typically "data-dojo-"
			dataDojoType = attrData + "type",						// typically "data-dojo-type"
			dataDojoProps = attrData + "props",						// typically "data-dojo-props"
			dataDojoAttachPoint = attrData + "attach-point",
			dataDojoAttachEvent = attrData + "attach-event",
			dataDojoId = attrData + "id",
			dataDojoMixins = attrData + "mixins";

		// And make hash to quickly check if a given attribute is special, and to map the name to something friendly
		var specialAttrs = {};
		darray.forEach([dataDojoProps, dataDojoType, dojoType, dataDojoId, "jsId", dataDojoAttachPoint,
				dataDojoAttachEvent, "dojoAttachPoint", "dojoAttachEvent", "class", "style", dataDojoMixins], function(name){
			specialAttrs[name.toLowerCase()] = name.replace(options.scope, "dojo");
		});

		function extend(type, mixins){
			return type.createSubclass && type.createSubclass(mixins) || type.extend.apply(type, mixins);
		}

		darray.forEach(nodes, function(obj){
			if(!obj){ return; }

			var node = obj.node,
				type = obj.type,
				mixins = node.getAttribute(dataDojoMixins), ctor;

			if(mixins){
				var map = _ctorMap[type];
				// remove whitespaces
				mixins = mixins.replace(/ /g, "");
				ctor = map && map[mixins];
				if(!ctor){
					// first get ctor for raw type (& create _ctorMap[type] if needed (should not be))
					ctor = getCtor(type);
					// then do the mixin
					ctor = _ctorMap[type][mixins] = extend(ctor, darray.map(mixins.split(","), getCtor));
				}
			}else{
				ctor = getCtor(type);
			}

			var proto = ctor && ctor.prototype;

			// Setup hash to hold parameter settings for this widget.	Start with the parameter
			// settings inherited from ancestors ("dir" and "lang").
			// Inherited setting may later be overridden by explicit settings on node itself.
			var params = {};

			if(options.defaults){
				// settings for the document itself (or whatever subtree is being parsed)
				dlang.mixin(params, options.defaults);
			}
			if(obj.inherited){
				// settings from dir=rtl or lang=... on a node above this node
				dlang.mixin(params, obj.inherited);
			}

			// Get list of attributes explicitly listed in the markup
			var attributes;
			if(has("dom-attributes-explicit")){
				// Standard path to get list of user specified attributes
				attributes = node.attributes;
			}else if(has("dom-attributes-specified-flag")){
				// Special processing needed for IE8, to skip a few faux values in attributes[]
				attributes = darray.filter(node.attributes, function(a){ return a.specified;});
			}else{
				// Special path for IE6-7, avoid (sometimes >100) bogus entries in node.attributes
				var clone = /^input$|^img$/i.test(node.nodeName) ? node : node.cloneNode(false),
					attrs = clone.outerHTML.replace(/=[^\s"']+|="[^"]*"|='[^']*'/g, "").replace(/^\s*<[a-zA-Z0-9]*\s*/, "").replace(/\s*>.*$/, "");

				attributes = darray.map(attrs.split(/\s+/), function(name){
					var lcName = name.toLowerCase();
					return {
						name: name,
						// getAttribute() doesn't work for button.value, returns innerHTML of button.
						// but getAttributeNode().value doesn't work for the form.encType or li.value
						value: (node.nodeName == "LI" && name == "value") || lcName == "enctype" ?
								node.getAttribute(lcName) : node.getAttributeNode(lcName).value
					};
				});
			}

			// Read in attributes and process them, including data-dojo-props, data-dojo-type,
			// dojoAttachPoint, etc., as well as normal foo=bar attributes.
			var i=0, item;
			while(item = attributes[i++]){
				var name = item.name,
					lcName = name.toLowerCase(),
					value = item.value;

				if(lcName in specialAttrs){
					switch(specialAttrs[lcName]){

					// Data-dojo-props.   Save for later to make sure it overrides direct foo=bar settings
					case "data-dojo-props":
						var extra = value;
						break;

					// data-dojo-id or jsId. TODO: drop jsId in 2.0
					case "data-dojo-id":
					case "jsId":
						var jsname = value;
						break;

					// For the benefit of _Templated
					case "data-dojo-attach-point":
					case "dojoAttachPoint":
						params.dojoAttachPoint = value;
						break;
					case "data-dojo-attach-event":
					case "dojoAttachEvent":
						params.dojoAttachEvent = value;
						break;

					// Special parameter handling needed for IE
					case "class":
						params["class"] = node.className;
						break;
					case "style":
						params["style"] = node.style && node.style.cssText;
						break;
					}
				}else{
					// Normal attribute, ex: value="123"

					// Find attribute in widget corresponding to specified name.
					// May involve case conversion, ex: onclick --> onClick
					if(!(name in proto)){
						var map = (_nameMap[type] || (_nameMap[type] = getNameMap(proto)));
						name = map[lcName] || name;
					}

					// Set params[name] to value, doing type conversion
					if(name in proto){
						switch(typeof proto[name]){
						case "string":
							params[name] = value;
							break;
						case "number":
							params[name] = value.length ? Number(value) : NaN;
							break;
						case "boolean":
							// for checked/disabled value might be "" or "checked".	 interpret as true.
							params[name] = value.toLowerCase() != "false";
							break;
						case "function":
							if(value === "" || value.search(/[^\w\.]+/i) != -1){
								// The user has specified some text for a function like "return x+5"
								params[name] = new Function(value);
							}else{
								// The user has specified the name of a function like "myOnClick"
								// or a single word function "return"
								params[name] = dlang.getObject(value, false) || new Function(value);
							}
							break;
						default:
							var pVal = proto[name];
							params[name] =
								(pVal && "length" in pVal) ? (value ? value.split(/\s*,\s*/) : []) :	// array
									(pVal instanceof Date) ?
										(value == "" ? new Date("") :	// the NaN of dates
										value == "now" ? new Date() :	// current date
										dates.fromISOString(value)) :
								(pVal instanceof dojo._Url) ? (dojo.baseUrl + value) :
								djson.fromJson(value);
						}
					}else{
						params[name] = value;
					}
				}
			}

			// Mix things found in data-dojo-props into the params, overriding any direct settings
			if(extra){
				try{
					extra = djson.fromJson.call(options.propsThis, "{" + extra + "}");
					dlang.mixin(params, extra);
				}catch(e){
					// give the user a pointer to their invalid parameters. FIXME: can we kill this in production?
					throw new Error(e.toString() + " in data-dojo-props='" + extra + "'");
				}
			}

			// Any parameters specified in "mixin" override everything else.
			dlang.mixin(params, mixin);

			var scripts = obj.scripts || (ctor && (ctor._noScript || proto._noScript) ? [] :
						query("> script[type^='dojo/']", node));

			// Process <script type="dojo/*"> script tags
			// <script type="dojo/method" event="foo"> tags are added to params, and passed to
			// the widget on instantiation.
			// <script type="dojo/method"> tags (with no event) are executed after instantiation
			// <script type="dojo/connect" data-dojo-event="foo"> tags are dojo.connected after instantiation
			// <script type="dojo/watch" data-dojo-prop="foo"> tags are dojo.watch after instantiation
			// <script type="dojo/on" data-dojo-event="foo"> tags are dojo.on after instantiation
			// note: dojo/* script tags cannot exist in self closing widgets, like <input />
			var connects = [],	// functions to connect after instantiation
				calls = [],		// functions to call after instantiation
				watch = [],  //functions to watch after instantiation
				on = []; //functions to on after instantiation

			if(scripts){
				for(i=0; i<scripts.length; i++){
					var script = scripts[i];
					node.removeChild(script);
					// FIXME: drop event="" support in 2.0. use data-dojo-event="" instead
					var event = (script.getAttribute(attrData + "event") || script.getAttribute("event")),
						prop = script.getAttribute(attrData + "prop"),
						scriptType = script.getAttribute("type"),
						nf = this._functionFromScript(script, attrData);
					if(event){
						if(scriptType == "dojo/connect"){
							connects.push({event: event, func: nf});
						}else if(scriptType == "dojo/on"){
							on.push({event: event, func: nf});
						}else{
							params[event] = nf;
						}
					}else if(scriptType == "dojo/watch"){
						watch.push({prop: prop, func: nf});
					}else{
						calls.push(nf);
					}
				}
			}

			// create the instance
			var markupFactory = ctor.markupFactory || proto.markupFactory;
			var instance = markupFactory ? markupFactory(params, node, ctor) : new ctor(params, node);
			thelist.push(instance);

			// map it to the JS namespace if that makes sense
			if(jsname){
				dlang.setObject(jsname, instance);
			}

			// process connections and startup functions
			for(i=0; i<connects.length; i++){
				aspect.after(instance, connects[i].event, dojo.hitch(instance, connects[i].func), true);
			}
			for(i=0; i<calls.length; i++){
				calls[i].call(instance);
			}
			for(i=0; i<watch.length; i++){
				instance.watch(watch[i].prop, watch[i].func);
			}
			for(i=0; i<on.length; i++){
				don(instance, on[i].event, on[i].func);
			}
		}, this);

		// Call startup on each top level instance if it makes sense (as for
		// widgets).  Parent widgets will recursively call startup on their
		// (non-top level) children
		if(!mixin._started){
			darray.forEach(thelist, function(instance){
				if( !options.noStart && instance  &&
					dlang.isFunction(instance.startup) &&
					!instance._started
				){
					instance.startup();
				}
			});
		}
		return thelist;
	};

	this.scan = /*====== dojo.parser.scan= ======*/ function(root, options){
		// summary:
		//		Scan a DOM tree and return an array of objects representing the DOMNodes
		//		that need to be turned into widgets.
		// description:
		//		Search specified node (or document root node) recursively for class instances
		//		and return an array of objects that represent potential widgets to be
		//		instantiated. Searches for either data-dojo-type="MID" or dojoType="MID" where
		//		"MID" is a module ID like "dijit/form/Button" or a fully qualified Class name
		//		like "dijit.form.Button".
		//
		//		See parser.parse() for details of markup.
		// root: DomNode?
		//		A default starting root node from which to start the parsing. Can be
		//		omitted, defaulting to the entire document. If omitted, the `options`
		//		object can be passed in this place. If the `options` object has a
		//		`rootNode` member, that is used.
		// options: Object
		//		a kwArgs options object, see parse() for details

		// Output list
		var list = [];

		var dojoType = (options.scope || dojo._scopeName) + "Type",		// typically "dojoType"
			attrData = "data-" + (options.scope || dojo._scopeName) + "-",	// typically "data-dojo-"
			dataDojoType = attrData + "type",						// typically "data-dojo-type"
			dataDojoTextDir = attrData + "textdir";					// typically "data-dojo-textdir"

		// Info on DOMNode currently being processed
		var node = root.firstChild;

		// Info on parent of DOMNode currently being processed
		//	- inherited: dir, lang, and textDir setting of parent, or inherited by parent
		//	- parent: pointer to identical structure for my parent (or null if no parent)
		//	- scripts: if specified, collects <script type="dojo/..."> type nodes from children
		var inherited = options.inherited;
		if(!inherited){
			function findAncestorAttr(node, attr){
				return (node.getAttribute && node.getAttribute(attr)) ||
					(node !== dwindow.doc && node !== dwindow.doc.documentElement && node.parentNode ? findAncestorAttr(node.parentNode, attr) : null);
			}
			inherited = {
				dir: findAncestorAttr(root, "dir"),
				lang: findAncestorAttr(root, "lang"),
				textDir: findAncestorAttr(root, dataDojoTextDir)
			};
			for(var key in inherited){
				if(!inherited[key]){ delete inherited[key]; }
			}
		}
		var parent = {
			inherited: inherited
		};

		// For collecting <script type="dojo/..."> type nodes (when null, we don't need to collect)
		var scripts;

		// when true, only look for <script type="dojo/..."> tags, and don't recurse to children
		var scriptsOnly;

		function getEffective(parent){
			// summary:
			//		Get effective dir, lang, textDir settings for specified obj
			//		(matching "parent" object structure above), and do caching.
			//		Take care not to return null entries.
			if(!parent.inherited){
				parent.inherited = {};
				var node = parent.node,
					grandparent = getEffective(parent.parent);
				var inherited  = {
					dir: node.getAttribute("dir") || grandparent.dir,
					lang: node.getAttribute("lang") || grandparent.lang,
					textDir: node.getAttribute(dataDojoTextDir) || grandparent.textDir
				};
				for(var key in inherited){
					if(inherited[key]){
						parent.inherited[key] = inherited[key];
					}
				}
			}
			return parent.inherited;
		}

		// DFS on DOM tree, collecting nodes with data-dojo-type specified.
		while(true){
			if(!node){
				// Finished this level, continue to parent's next sibling
				if(!parent || !parent.node){
					break;
				}
				node = parent.node.nextSibling;
				scripts = parent.scripts;
				scriptsOnly = false;
				parent = parent.parent;
				continue;
			}

			if(node.nodeType != 1){
				// Text or comment node, skip to next sibling
				node = node.nextSibling;
				continue;
			}

			if(scripts && node.nodeName.toLowerCase() == "script"){
				// Save <script type="dojo/..."> for parent, then continue to next sibling
				type = node.getAttribute("type");
				if(type && /^dojo\/\w/i.test(type)){
					scripts.push(node);
				}
				node = node.nextSibling;
				continue;
			}
			if(scriptsOnly){
				node = node.nextSibling;
				continue;
			}

			// Check for data-dojo-type attribute, fallback to backward compatible dojoType
			var type = node.getAttribute(dataDojoType) || node.getAttribute(dojoType);

			// Short circuit for leaf nodes containing nothing [but text]
			var firstChild = node.firstChild;
			if(!type && (!firstChild || (firstChild.nodeType == 3 && !firstChild.nextSibling))){
				node = node.nextSibling;
				continue;
			}

			// Setup data structure to save info on current node for when we return from processing descendant nodes
			var current = {
				node: node,
				scripts: scripts,
				parent: parent
			};

			// If dojoType/data-dojo-type specified, add to output array of nodes to instantiate
			// Note: won't find classes declared via dojo.Declaration, so use try/catch to avoid throw from require()
			// We don't care yet about mixins ctors, we check script stop only on main class
			var ctor;
			try{
				ctor = type && getCtor(type);
			}catch(e){
			}
			var childScripts = ctor && !ctor.prototype._noScript ? [] : null; // <script> nodes that are parent's children
			if(type){
				list.push({
					"type": type,
					node: node,
					scripts: childScripts,
					inherited: getEffective(current) // dir & lang settings for current node, explicit or inherited
				});
			}

			// Recurse, collecting <script type="dojo/..."> children, and also looking for
			// descendant nodes with dojoType specified (unless the widget has the stopParser flag).
			// When finished with children, go to my next sibling.
			node = firstChild;
			scripts = childScripts;
			scriptsOnly = ctor && ctor.prototype.stopParser && !(options.template);
			parent = current;
		}

		return list;
	};

	this.parse = /*====== dojo.parser.parse= ======*/ function(rootNode, options){
		// summary:
		//		Scan the DOM for class instances, and instantiate them.
		//
		// description:
		//		Search specified node (or root node) recursively for class instances,
		//		and instantiate them. Searches for either data-dojo-type="Class" or
		//		dojoType="Class" where "Class" is a a fully qualified class name,
		//		like `dijit.form.Button`
		//
		//		Using `data-dojo-type`:
		//		Attributes using can be mixed into the parameters used to instantiate the
		//		Class by using a `data-dojo-props` attribute on the node being converted.
		//		`data-dojo-props` should be a string attribute to be converted from JSON.
		//
		//		Using `dojoType`:
		//		Attributes are read from the original domNode and converted to appropriate
		//		types by looking up the Class prototype values. This is the default behavior
		//		from Dojo 1.0 to Dojo 1.5. `dojoType` support is deprecated, and will
		//		go away in Dojo 2.0.
		//
		// rootNode: DomNode?
		//		A default starting root node from which to start the parsing. Can be
		//		omitted, defaulting to the entire document. If omitted, the `options`
		//		object can be passed in this place. If the `options` object has a
		//		`rootNode` member, that is used.
		//
		// options: Object?
		//		A hash of options.
		//
		//			* noStart: Boolean?
		//				when set will prevent the parser from calling .startup()
		//				when locating the nodes.
		//			* rootNode: DomNode?
		//				identical to the function's `rootNode` argument, though
		//				allowed to be passed in via this `options object.
		//			* template: Boolean
		//				If true, ignores ContentPane's stopParser flag and parses contents inside of
		//				a ContentPane inside of a template.   This allows dojoAttachPoint on widgets/nodes
		//				nested inside the ContentPane to work.
		//			* inherited: Object
		//				Hash possibly containing dir and lang settings to be applied to
		//				parsed widgets, unless there's another setting on a sub-node that overrides
		//			* scope: String
		//				Root for attribute names to search for.   If scopeName is dojo,
		//				will search for data-dojo-type (or dojoType).   For backwards compatibility
		//				reasons defaults to dojo._scopeName (which is "dojo" except when
		//				multi-version support is used, when it will be something like dojo16, dojo20, etc.)
		//			* propsThis: Object
		//				If specified, "this" referenced from data-dojo-props will refer to propsThis.
		//				Intended for use from the widgets-in-template feature of `dijit._WidgetsInTemplateMixin`
		//
		// example:
		//		Parse all widgets on a page:
		//	|		dojo.parser.parse();
		//
		// example:
		//		Parse all classes within the node with id="foo"
		//	|		dojo.parser.parse(dojo.byId('foo'));
		//
		// example:
		//		Parse all classes in a page, but do not call .startup() on any
		//		child
		//	|		dojo.parser.parse({ noStart: true })
		//
		// example:
		//		Parse all classes in a node, but do not call .startup()
		//	|		dojo.parser.parse(someNode, { noStart:true });
		//	|		// or
		//	|		dojo.parser.parse({ noStart:true, rootNode: someNode });

		// determine the root node and options based on the passed arguments.
		var root;
		if(!options && rootNode && rootNode.rootNode){
			options = rootNode;
			root = options.rootNode;
		}else if(rootNode && dlang.isObject(rootNode) && !("nodeType" in rootNode)){
			options = rootNode;
		}else{
			root = rootNode;
		}
		root = root ? dhtml.byId(root) : dwindow.body();

		options = options || {};

		// List of all nodes on page w/dojoType specified
		var list = this.scan(root, options);

		// go build the object instances
		var mixin = options.template ? {template: true} : {};
		return this._instantiate(list, mixin, options); // Array
	};
}();


//Register the parser callback. It should be the first callback
//after the a11y test.
if(config.parseOnLoad){
	ready(100, dojo.parser, "parse");
}

return dojo.parser;
});

},
'dijit/layout/BorderContainer':function(){
define([
	"dojo/_base/array", // array.filter array.forEach array.map
	"dojo/cookie", // cookie
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.add domClass.remove domClass.toggle
	"dojo/dom-construct", // domConstruct.destroy domConstruct.place
	"dojo/dom-geometry", // domGeometry.marginBox
	"dojo/dom-style", // domStyle.style
	"dojo/_base/event", // event.stop
	"dojo/keys",
	"dojo/_base/lang", // lang.getObject lang.hitch
	"dojo/on",
	"dojo/touch",
	"dojo/_base/window", // win.body win.doc win.doc.createElement
	"../_WidgetBase",
	"../_Widget",
	"../_TemplatedMixin",
	"./_LayoutWidget",
	"./utils"		// layoutUtils.layoutChildren
], function(array, cookie, declare, domClass, domConstruct, domGeometry, domStyle, event, keys, lang, on, touch, win,
			_WidgetBase, _Widget, _TemplatedMixin, _LayoutWidget, layoutUtils){

/*=====
	var _WidgetBase = dijit._WidgetBase;
	var _Widget = dijit._Widget;
	var _TemplatedMixin = dijit._TemplatedMixin;
	var _LayoutWidget = dijit.layout._LayoutWidget;
=====*/

// module:
//		dijit/layout/BorderContainer
// summary:
//		Provides layout in up to 5 regions, a mandatory center with optional borders along its 4 sides.

var _Splitter = declare("dijit.layout._Splitter", [_Widget, _TemplatedMixin ],
{
	// summary:
	//		A draggable spacer between two items in a `dijit.layout.BorderContainer`.
	// description:
	//		This is instantiated by `dijit.layout.BorderContainer`.  Users should not
	//		create it directly.
	// tags:
	//		private

/*=====
 	// container: [const] dijit.layout.BorderContainer
 	//		Pointer to the parent BorderContainer
	container: null,

	// child: [const] dijit.layout._LayoutWidget
	//		Pointer to the pane associated with this splitter
	child: null,

	// region: [const] String
	//		Region of pane associated with this splitter.
	//		"top", "bottom", "left", "right".
	region: null,
=====*/

	// live: [const] Boolean
	//		If true, the child's size changes and the child widget is redrawn as you drag the splitter;
	//		otherwise, the size doesn't change until you drop the splitter (by mouse-up)
	live: true,

	templateString: '<div class="dijitSplitter" data-dojo-attach-event="onkeypress:_onKeyPress,press:_startDrag,onmouseenter:_onMouse,onmouseleave:_onMouse" tabIndex="0" role="separator"><div class="dijitSplitterThumb"></div></div>',

	constructor: function(){
		this._handlers = [];
	},

	postMixInProperties: function(){
		this.inherited(arguments);

		this.horizontal = /top|bottom/.test(this.region);
		this._factor = /top|left/.test(this.region) ? 1 : -1;
		this._cookieName = this.container.id + "_" + this.region;
	},

	buildRendering: function(){
		this.inherited(arguments);

		domClass.add(this.domNode, "dijitSplitter" + (this.horizontal ? "H" : "V"));

		if(this.container.persist){
			// restore old size
			var persistSize = cookie(this._cookieName);
			if(persistSize){
				this.child.domNode.style[this.horizontal ? "height" : "width"] = persistSize;
			}
		}
	},

	_computeMaxSize: function(){
		// summary:
		//		Return the maximum size that my corresponding pane can be set to

		var dim = this.horizontal ? 'h' : 'w',
			childSize = domGeometry.getMarginBox(this.child.domNode)[dim],
			center = array.filter(this.container.getChildren(), function(child){ return child.region == "center";})[0],
			spaceAvailable = domGeometry.getMarginBox(center.domNode)[dim];	// can expand until center is crushed to 0

		return Math.min(this.child.maxSize, childSize + spaceAvailable);
	},

	_startDrag: function(e){
		if(!this.cover){
			this.cover = win.doc.createElement('div');
			domClass.add(this.cover, "dijitSplitterCover");
			domConstruct.place(this.cover, this.child.domNode, "after");
		}
		domClass.add(this.cover, "dijitSplitterCoverActive");

		// Safeguard in case the stop event was missed.  Shouldn't be necessary if we always get the mouse up.
		if(this.fake){ domConstruct.destroy(this.fake); }
		if(!(this._resize = this.live)){ //TODO: disable live for IE6?
			// create fake splitter to display at old position while we drag
			(this.fake = this.domNode.cloneNode(true)).removeAttribute("id");
			domClass.add(this.domNode, "dijitSplitterShadow");
			domConstruct.place(this.fake, this.domNode, "after");
		}
		domClass.add(this.domNode, "dijitSplitterActive dijitSplitter" + (this.horizontal ? "H" : "V") + "Active");
		if(this.fake){
			domClass.remove(this.fake, "dijitSplitterHover dijitSplitter" + (this.horizontal ? "H" : "V") + "Hover");
		}

		//Performance: load data info local vars for onmousevent function closure
		var factor = this._factor,
			isHorizontal = this.horizontal,
			axis = isHorizontal ? "pageY" : "pageX",
			pageStart = e[axis],
			splitterStyle = this.domNode.style,
			dim = isHorizontal ? 'h' : 'w',
			childStart = domGeometry.getMarginBox(this.child.domNode)[dim],
			max = this._computeMaxSize(),
			min = this.child.minSize || 20,
			region = this.region,
			splitterAttr = region == "top" || region == "bottom" ? "top" : "left",	// style attribute of splitter to adjust
			splitterStart = parseInt(splitterStyle[splitterAttr], 10),
			resize = this._resize,
			layoutFunc = lang.hitch(this.container, "_layoutChildren", this.child.id),
			de = win.doc;

		this._handlers = this._handlers.concat([
			on(de, touch.move, this._drag = function(e, forceResize){
				var delta = e[axis] - pageStart,
					childSize = factor * delta + childStart,
					boundChildSize = Math.max(Math.min(childSize, max), min);

				if(resize || forceResize){
					layoutFunc(boundChildSize);
				}
				// TODO: setting style directly (usually) sets content box size, need to set margin box size
				splitterStyle[splitterAttr] = delta + splitterStart + factor*(boundChildSize - childSize) + "px";
			}),
			on(de, "dragstart", event.stop),
			on(win.body(), "selectstart", event.stop),
			on(de, touch.release, lang.hitch(this, "_stopDrag"))
		]);
		event.stop(e);
	},

	_onMouse: function(e){
		// summary:
		//		Handler for onmouseenter / onmouseleave events
		var o = (e.type == "mouseover" || e.type == "mouseenter");
		domClass.toggle(this.domNode, "dijitSplitterHover", o);
		domClass.toggle(this.domNode, "dijitSplitter" + (this.horizontal ? "H" : "V") + "Hover", o);
	},

	_stopDrag: function(e){
		try{
			if(this.cover){
				domClass.remove(this.cover, "dijitSplitterCoverActive");
			}
			if(this.fake){ domConstruct.destroy(this.fake); }
			domClass.remove(this.domNode, "dijitSplitterActive dijitSplitter"
				+ (this.horizontal ? "H" : "V") + "Active dijitSplitterShadow");
			this._drag(e); //TODO: redundant with onmousemove?
			this._drag(e, true);
		}finally{
			this._cleanupHandlers();
			delete this._drag;
		}

		if(this.container.persist){
			cookie(this._cookieName, this.child.domNode.style[this.horizontal ? "height" : "width"], {expires:365});
		}
	},

	_cleanupHandlers: function(){
		var h;
		while(h = this._handlers.pop()){ h.remove(); }
	},

	_onKeyPress: function(/*Event*/ e){
		// should we apply typematic to this?
		this._resize = true;
		var horizontal = this.horizontal;
		var tick = 1;
		switch(e.charOrCode){
			case horizontal ? keys.UP_ARROW : keys.LEFT_ARROW:
				tick *= -1;
//				break;
			case horizontal ? keys.DOWN_ARROW : keys.RIGHT_ARROW:
				break;
			default:
//				this.inherited(arguments);
				return;
		}
		var childSize = domGeometry.getMarginSize(this.child.domNode)[ horizontal ? 'h' : 'w' ] + this._factor * tick;
		this.container._layoutChildren(this.child.id, Math.max(Math.min(childSize, this._computeMaxSize()), this.child.minSize));
		event.stop(e);
	},

	destroy: function(){
		this._cleanupHandlers();
		delete this.child;
		delete this.container;
		delete this.cover;
		delete this.fake;
		this.inherited(arguments);
	}
});

var _Gutter = declare("dijit.layout._Gutter", [_Widget, _TemplatedMixin],
{
	// summary:
	// 		Just a spacer div to separate side pane from center pane.
	//		Basically a trick to lookup the gutter/splitter width from the theme.
	// description:
	//		Instantiated by `dijit.layout.BorderContainer`.  Users should not
	//		create directly.
	// tags:
	//		private

	templateString: '<div class="dijitGutter" role="presentation"></div>',

	postMixInProperties: function(){
		this.inherited(arguments);
		this.horizontal = /top|bottom/.test(this.region);
	},

	buildRendering: function(){
		this.inherited(arguments);
		domClass.add(this.domNode, "dijitGutter" + (this.horizontal ? "H" : "V"));
	}
});

var BorderContainer = declare("dijit.layout.BorderContainer", _LayoutWidget, {
	// summary:
	//		Provides layout in up to 5 regions, a mandatory center with optional borders along its 4 sides.
	//
	// description:
	//		A BorderContainer is a box with a specified size, such as style="width: 500px; height: 500px;",
	//		that contains a child widget marked region="center" and optionally children widgets marked
	//		region equal to "top", "bottom", "leading", "trailing", "left" or "right".
	//		Children along the edges will be laid out according to width or height dimensions and may
	//		include optional splitters (splitter="true") to make them resizable by the user.  The remaining
	//		space is designated for the center region.
	//
	//		The outer size must be specified on the BorderContainer node.  Width must be specified for the sides
	//		and height for the top and bottom, respectively.  No dimensions should be specified on the center;
	//		it will fill the remaining space.  Regions named "leading" and "trailing" may be used just like
	//		"left" and "right" except that they will be reversed in right-to-left environments.
	//
	//		For complex layouts, multiple children can be specified for a single region.   In this case, the
	//		layoutPriority flag on the children determines which child is closer to the edge (low layoutPriority)
	//		and which child is closer to the center (high layoutPriority).   layoutPriority can also be used
	//		instead of the design attribute to control layout precedence of horizontal vs. vertical panes.
	// example:
	// |	<div data-dojo-type="dijit.layout.BorderContainer" data-dojo-props="design: 'sidebar', gutters: false"
	// |            style="width: 400px; height: 300px;">
	// |		<div data-dojo-type="dijit.layout.ContentPane" data-dojo-props="region: 'top'">header text</div>
	// |		<div data-dojo-type="dijit.layout.ContentPane" data-dojo-props="region: 'right', splitter: true" style="width: 200px;">table of contents</div>
	// |		<div data-dojo-type="dijit.layout.ContentPane" data-dojo-props="region: 'center'">client area</div>
	// |	</div>

	// design: String
	//		Which design is used for the layout:
	//			- "headline" (default) where the top and bottom extend
	//				the full width of the container
	//			- "sidebar" where the left and right sides extend from top to bottom.
	design: "headline",

	// gutters: [const] Boolean
	//		Give each pane a border and margin.
	//		Margin determined by domNode.paddingLeft.
	//		When false, only resizable panes have a gutter (i.e. draggable splitter) for resizing.
	gutters: true,

	// liveSplitters: [const] Boolean
	//		Specifies whether splitters resize as you drag (true) or only upon mouseup (false)
	liveSplitters: true,

	// persist: Boolean
	//		Save splitter positions in a cookie.
	persist: false,

	baseClass: "dijitBorderContainer",

	// _splitterClass: Function||String
	// 		Optional hook to override the default Splitter widget used by BorderContainer
	_splitterClass: _Splitter,

	postMixInProperties: function(){
		// change class name to indicate that BorderContainer is being used purely for
		// layout (like LayoutContainer) rather than for pretty formatting.
		if(!this.gutters){
			this.baseClass += "NoGutter";
		}
		this.inherited(arguments);
	},

	startup: function(){
		if(this._started){ return; }
		array.forEach(this.getChildren(), this._setupChild, this);
		this.inherited(arguments);
	},

	_setupChild: function(/*dijit._Widget*/ child){
		// Override _LayoutWidget._setupChild().

		var region = child.region;
		if(region){
			this.inherited(arguments);

			domClass.add(child.domNode, this.baseClass+"Pane");

			var ltr = this.isLeftToRight();
			if(region == "leading"){ region = ltr ? "left" : "right"; }
			if(region == "trailing"){ region = ltr ? "right" : "left"; }

			// Create draggable splitter for resizing pane,
			// or alternately if splitter=false but BorderContainer.gutters=true then
			// insert dummy div just for spacing
			if(region != "center" && (child.splitter || this.gutters) && !child._splitterWidget){
				var _Splitter = child.splitter ? this._splitterClass : _Gutter;
				if(lang.isString(_Splitter)){
					_Splitter = lang.getObject(_Splitter);	// for back-compat, remove in 2.0
				}
				var splitter = new _Splitter({
					id: child.id + "_splitter",
					container: this,
					child: child,
					region: region,
					live: this.liveSplitters
				});
				splitter.isSplitter = true;
				child._splitterWidget = splitter;

				domConstruct.place(splitter.domNode, child.domNode, "after");

				// Splitters aren't added as Contained children, so we need to call startup explicitly
				splitter.startup();
			}
			child.region = region;	// TODO: technically wrong since it overwrites "trailing" with "left" etc.
		}
	},

	layout: function(){
		// Implement _LayoutWidget.layout() virtual method.
		this._layoutChildren();
	},

	addChild: function(/*dijit._Widget*/ child, /*Integer?*/ insertIndex){
		// Override _LayoutWidget.addChild().
		this.inherited(arguments);
		if(this._started){
			this.layout(); //OPT
		}
	},

	removeChild: function(/*dijit._Widget*/ child){
		// Override _LayoutWidget.removeChild().

		var region = child.region;
		var splitter = child._splitterWidget;
		if(splitter){
			splitter.destroy();
			delete child._splitterWidget;
		}
		this.inherited(arguments);

		if(this._started){
			this._layoutChildren();
		}
		// Clean up whatever style changes we made to the child pane.
		// Unclear how height and width should be handled.
		domClass.remove(child.domNode, this.baseClass+"Pane");
		domStyle.set(child.domNode, {
			top: "auto",
			bottom: "auto",
			left: "auto",
			right: "auto",
			position: "static"
		});
		domStyle.set(child.domNode, region == "top" || region == "bottom" ? "width" : "height", "auto");
	},

	getChildren: function(){
		// Override _LayoutWidget.getChildren() to only return real children, not the splitters.
		return array.filter(this.inherited(arguments), function(widget){
			return !widget.isSplitter;
		});
	},

	// TODO: remove in 2.0
	getSplitter: function(/*String*/region){
		// summary:
		//		Returns the widget responsible for rendering the splitter associated with region
		// tags:
		//		deprecated
		return array.filter(this.getChildren(), function(child){
			return child.region == region;
		})[0]._splitterWidget;
	},

	resize: function(newSize, currentSize){
		// Overrides _LayoutWidget.resize().

		// resetting potential padding to 0px to provide support for 100% width/height + padding
		// TODO: this hack doesn't respect the box model and is a temporary fix
		if(!this.cs || !this.pe){
			var node = this.domNode;
			this.cs = domStyle.getComputedStyle(node);
			this.pe = domGeometry.getPadExtents(node, this.cs);
			this.pe.r = domStyle.toPixelValue(node, this.cs.paddingRight);
			this.pe.b = domStyle.toPixelValue(node, this.cs.paddingBottom);

			domStyle.set(node, "padding", "0px");
		}

		this.inherited(arguments);
	},

	_layoutChildren: function(/*String?*/ changedChildId, /*Number?*/ changedChildSize){
		// summary:
		//		This is the main routine for setting size/position of each child.
		// description:
		//		With no arguments, measures the height of top/bottom panes, the width
		//		of left/right panes, and then sizes all panes accordingly.
		//
		//		With changedRegion specified (as "left", "top", "bottom", or "right"),
		//		it changes that region's width/height to changedRegionSize and
		//		then resizes other regions that were affected.
		// changedChildId:
		//		Id of the child which should be resized because splitter was dragged.
		// changedChildSize:
		//		The new width/height (in pixels) to make specified child

		if(!this._borderBox || !this._borderBox.h){
			// We are currently hidden, or we haven't been sized by our parent yet.
			// Abort.   Someone will resize us later.
			return;
		}

		// Generate list of wrappers of my children in the order that I want layoutChildren()
		// to process them (i.e. from the outside to the inside)
		var wrappers = array.map(this.getChildren(), function(child, idx){
			return {
				pane: child,
				weight: [
					child.region == "center" ? Infinity : 0,
					child.layoutPriority,
					(this.design == "sidebar" ? 1 : -1) * (/top|bottom/.test(child.region) ? 1 : -1),
					idx
				]
			};
		}, this);
		wrappers.sort(function(a, b){
			var aw = a.weight, bw = b.weight;
			for(var i=0; i<aw.length; i++){
				if(aw[i] != bw[i]){
					return aw[i] - bw[i];
				}
			}
			return 0;
		});

		// Make new list, combining the externally specified children with splitters and gutters
		var childrenAndSplitters = [];
		array.forEach(wrappers, function(wrapper){
			var pane = wrapper.pane;
			childrenAndSplitters.push(pane);
			if(pane._splitterWidget){
				childrenAndSplitters.push(pane._splitterWidget);
			}
		});

		// Compute the box in which to lay out my children
		var dim = {
			l: this.pe.l,
			t: this.pe.t,
			w: this._borderBox.w - this.pe.w,
			h: this._borderBox.h - this.pe.h
		};

		// Layout the children, possibly changing size due to a splitter drag
		layoutUtils.layoutChildren(this.domNode, dim, childrenAndSplitters,
			changedChildId, changedChildSize);
	},

	destroyRecursive: function(){
		// Destroy splitters first, while getChildren() still works
		array.forEach(this.getChildren(), function(child){
			var splitter = child._splitterWidget;
			if(splitter){
				splitter.destroy();
			}
			delete child._splitterWidget;
		});

		// Then destroy the real children, and myself
		this.inherited(arguments);
	}
});

// This argument can be specified for the children of a BorderContainer.
// Since any widget can be specified as a LayoutContainer child, mix it
// into the base widget class.  (This is a hack, but it's effective.)
lang.extend(_WidgetBase, {
	// region: [const] String
	//		Parameter for children of `dijit.layout.BorderContainer`.
	//		Values: "top", "bottom", "leading", "trailing", "left", "right", "center".
	//		See the `dijit.layout.BorderContainer` description for details.
	region: '',

	// layoutPriority: [const] Number
	//		Parameter for children of `dijit.layout.BorderContainer`.
	//		Children with a higher layoutPriority will be placed closer to the BorderContainer center,
	//		between children with a lower layoutPriority.
	layoutPriority: 0,

	// splitter: [const] Boolean
	//		Parameter for child of `dijit.layout.BorderContainer` where region != "center".
	//		If true, enables user to resize the widget by putting a draggable splitter between
	//		this widget and the region=center widget.
	splitter: false,

	// minSize: [const] Number
	//		Parameter for children of `dijit.layout.BorderContainer`.
	//		Specifies a minimum size (in pixels) for this widget when resized by a splitter.
	minSize: 0,

	// maxSize: [const] Number
	//		Parameter for children of `dijit.layout.BorderContainer`.
	//		Specifies a maximum size (in pixels) for this widget when resized by a splitter.
	maxSize: Infinity
});

// For monkey patching
BorderContainer._Splitter = _Splitter;
BorderContainer._Gutter = _Gutter;

return BorderContainer;
});

},
'ibm/usmi/dashboard/DatasetItemStoreAdapter':function(){
/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/dashboard/DatasetItemStoreAdapter", ["dojo/_base/declare",
		"ibm/tivoli/navmodel/Dataset",
		"ibm/tivoli/navmodel/DatasetItemsStore",
		"ibm/usmi/util/DefaultMessageHandler",
		"dojo/_base/Deferred",
		"dojo/_base/lang"
	], function (declare, Dataset, DatasetItemsStore, DefaultMessageHandler, Deferred, lang) {
	return declare(null, {
		dataset: "EVENT_LOG",
		sourceToken: null,
		_sort: [{attribute:"DateTime", descending: true}],
		_start: 0,
		_count: 3,
		_defaultProperties: ['Source', 'Severity', 'EventText', 'DateTime'],
		new_dataset_store: null,
		
		_deferredDataStore: null,
		
		constructor: function(/*constructor_kwArgs*/ kwArgs) {	
			declare.safeMixin(this, kwArgs);	
		},
		
		
		/**
		*/
		getStore: function(){
			this._deferredDataStore = new Deferred();
			var options={
				onComplete : lang.hitch(this, function (new_dataset, kwArgs) {
					this.new_dataset_store = new DatasetItemsStore({
                        dataset: new_dataset,
						defaultProperties: this._defaultProperties
                    });
					this.fetchItems();
				}),
				onError : function (error, exception, ioArgs) {
					var msgHandler = new DefaultMessageHandler({});
					msgHandler.displayException(exception);
				},
				baseUrl : "/ibm/tivoli/rest",
				provider : "USMI",
				datasource : "",
				dataset : this.dataset,
				clearCacheOnRefresh : true,
				createEventRouter: false, //do not create event router
				defaultProperties : []
			};
			if(this.sourceToken){
				//console.debug("sourceToken:"+this.sourceToken);
				options.parameters = {sourceToken: this.sourceToken};
			}
			var tctDataset = new Dataset(options);
			return this._deferredDataStore;
		},
		
		fetchItems: function(){
			if(!this.new_dataset_store){
				//console.error("data store is not available yet!");
				return;
			}
			this.new_dataset_store.fetch({
				onComplete:  function (items, kwArgs) {
					//console.debug("finishing load items");
					this._deferredDataStore.resolve(items);
				},
				onError : function (error, exception, ioArgs) {
					var msgHandler = new DefaultMessageHandler({});
					msgHandler.displayException(exception);
				},
				start: this._start,
				count: this._count,
				sort: this._sort,
				scope: this
			});
		}
		
		
		
    });
});

},
'dijit/form/DataList':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.byId
	"dojo/_base/lang", // lang.trim
	"dojo/query", // query
	"dojo/store/Memory", // dojo.store.Memory
	"../registry"	// registry.add registry.remove
], function(declare, dom, lang, query, MemoryStore, registry){

	// module:
	//		dijit/form/DataList
	// summary:
	//		Inefficient but small data store specialized for inlined data via OPTION tags

	function toItem(/*DOMNode*/ option){
		// summary:
		//		Convert <option> node to hash
		return {
			id: option.value,
			value: option.value,
			name: lang.trim(option.innerText || option.textContent || '')
		};
	}

	return declare("dijit.form.DataList", MemoryStore, {
		// summary:
		//		Inefficient but small data store specialized for inlined data via OPTION tags
		//
		// description:
		//		Provides a store for inlined data like:
		//
		//	|	<datalist>
		//	|		<option value="AL">Alabama</option>
		//	|		...

		constructor: function(/*Object?*/ params, /*DomNode|String*/ srcNodeRef){
			// store pointer to original DOM tree
			this.domNode = dom.byId(srcNodeRef);

			lang.mixin(this, params);
			if(this.id){
				registry.add(this); // add to registry so it can be easily found by id
			}
			this.domNode.style.display = "none";

			this.inherited(arguments, [{
				data: query("option", this.domNode).map(toItem)
			}]);
		},

		destroy: function(){
			registry.remove(this.id);
		},

		fetchSelectedItem: function(){
			// summary:
			//		Get the option marked as selected, like `<option selected>`.
			//		Not part of dojo.data API.
			var option = query("> option[selected]", this.domNode)[0] || query("> option", this.domNode)[0];
			return option && toItem(option);
		}
	});
});

},
'url:gridx/templates/Grid.html':"<div class=\"gridx\" role=\"grid\" tabindex=\"0\" aria-readonly=\"true\"\n\t><div class=\"gridxHeader\" role=\"presentation\" data-dojo-attach-point=\"headerNode\"></div\n\t><div class=\"gridxMain\" role=\"presentation\" data-dojo-attach-point=\"mainNode\"\n\t\t><div class=\"gridxBodyEmpty\" role=\"alert\" tabindex=\"-1\" data-dojo-attach-point=\"emptyNode\"></div\n\t\t><div class=\"gridxBody\" role=\"presentation\" data-dojo-attach-point=\"bodyNode\"></div\n\t\t><div class=\"gridxVScroller\" data-dojo-attach-point=\"vScrollerNode\" tabindex=\"-1\"\n\t\t\t><div style='width: 1px;'></div\n\t\t></div\n\t></div\n\t><div class=\"gridxFooter\" data-dojo-attach-point=\"footerNode\"\n\t\t><div class=\"gridxHScroller\"\n\t\t\t><div class=\"gridxHScrollerInner\" data-dojo-attach-point=\"hScrollerNode\" tabindex=\"-1\"\n\t\t\t\t><div style=\"width:1px; height: 1px;\"></div\n\t\t\t></div\n\t\t></div\n\t></div\n\t><span data-dojo-attach-point=\"lastFocusNode\" tabindex=\"0\"></span\n></div>\n",
'url:dojox/grid/resources/_Grid.html':"<div hidefocus=\"hidefocus\" role=\"grid\" dojoAttachEvent=\"onmouseout:_mouseOut\">\r\n\t<div class=\"dojoxGridMasterHeader\" dojoAttachPoint=\"viewsHeaderNode\" role=\"presentation\"></div>\r\n\t<div class=\"dojoxGridMasterView\" dojoAttachPoint=\"viewsNode\" role=\"presentation\"></div>\r\n\t<div class=\"dojoxGridMasterMessages\" style=\"display: none;\" dojoAttachPoint=\"messagesNode\" role=\"alert\" tabindex='-1'></div>\r\n\t<span dojoAttachPoint=\"lastFocusNode\" tabindex=\"0\"></span>\r\n</div>\r\n",
'ibm/tivoli/tip/navwidget/table/dijit/LinksTableView':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!dojo/data/ItemFileReadStore,ibm/tivoli/tip/navwidget/topology/dijit/TopologyMenuLayer,ibm/tivoli/tip/navwidget/topology/dijit/MenuProviderHelper,ibm/tivoli/navmodel/DatasetLinksStore,ibm/tivoli/tip/navwidget/table/dijit/AbstractTableView,ibm/tivoli/tip/navwidget/table/dijit/NavModelTable"], function(dijit,dojo,dojox){
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide ("ibm.tivoli.tip.navwidget.table.dijit.LinksTableView");

dojo.require("dojo.data.ItemFileReadStore");   // 49426  Gridx
dojo.require("ibm.tivoli.tip.navwidget.topology.dijit.TopologyMenuLayer");
dojo.require("ibm.tivoli.tip.navwidget.topology.dijit.MenuProviderHelper");
dojo.require("ibm.tivoli.navmodel.DatasetLinksStore");
dojo.require ("ibm.tivoli.tip.navwidget.table.dijit.AbstractTableView");
dojo.require("ibm.tivoli.tip.navwidget.table.dijit.NavModelTable");

ibm.tivoli.tip.navwidget.table.dijit.LinksTableView._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.tip.navwidget.table.dijit.LinksTableView");

dojo.declare ("ibm.tivoli.tip.navwidget.table.dijit.LinksTableView",
	ibm.tivoli.tip.navwidget.table.dijit.AbstractTableView,
	{
	// summary:
	//		Coming soon
	// description:
	//		Coming soon
	
	//***************************
	// "Private"
	//***************************
	// _trace: [private] Trace
	//		Utility for trace and logging
	_trace: ibm.tivoli.tip.navwidget.table.dijit.LinksTableView._trace,
	
	constructor: function () {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		this._global = ibm.tivoli.tip.dojo.Global.setSingleton( "867", " ");
		this.title = this.commonResources.LINK_VIEW;
		this.labelIconClass = "linkViewIcon";
	    this._pDEFAULT_WIDGET_DEF_INFO = {
				className: "ibm.tivoli.navmodel.TableWidgetDef",
				baseUrl: "/ibm/tivoli/rest",
		    	provider: "any",
		    	datasource: "any",
		    	widget: "table",
		    	definition: "DefaultLinksTable"
			},
		this._pTableType = "link";
		this._pTableClassName = "ibm.tivoli.tip.navwidget.table.dijit.NavModelTable";
		this._pDefaultColumns = [
				{id: "item2", label: this.commonResources.ITEM2, type: "string"},
				{id: "label", label: this.commonResources.TYPE,  type: "string"},
				{id: "item1", label:this.commonResources.ITEM1,  type: "string", sortOrder: 0} ];
	},

	isApplicable: function(/*ibm.tivoli.navmodel.Dataset*/dataset){
		// Primarily interested in the following view types: 
		//	viewType == "graph" || viewType == "tree" || viewType == "tree:reverse" || viewType == "table_relationship";
		return this.inherited(arguments);
	},
	
	_createTableStore: function(columnIds){
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// columnIds:
		//		
		if(!columnIds) {
			columnIds = ["all"];
		}
		// create or update store

		if (!this._dataStore) {   // 49426  Gridx (split out to always return some store)
            if (this.dataset) {
            	this._dataStore = new ibm.tivoli.navmodel.DatasetLinksStore({dataset:this.dataset, returnItem:true, defaultProperties:columnIds});
            }
            else {
				//No dataset, so just create an empty store
				var emptyStore = 	new dojo.data.ItemFileReadStore({
					  					data: { items: []}
	                				});
				this._dataStore = emptyStore;
            }
		   this.inherited(arguments);
		}
	},	
	
	
	
	/////////////////////////////////////////////////
	// Selection change handling
	/////////////////////////////////////////////////
	_handlingSelectOrDeselectAll: false,
	_handlingSelectionChangedCallback: false,
	_gridSelectionChangedCallback: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		this._handlingSelectionChangedCallback = true;
		
		if (!this.selection.isChanging()&& ! this._handlingSelectOrDeselectAll) {
			//Get the grid's selection... will be an array of Client Dataset Relationships
			var selectedRows = this._grid.selection.getSelected(); 

			//Set the selected links on the view's selection object
			this.selection.setSelectedObjects(null, selectedRows, null, null);
		}
		
		this._handlingSelectionChangedCallback = false;
	}	
});

});

},
'ibm/tivoli/tip/dijit/ContextMenuAdapter':function(){
// wrapped by build app
define("ibm/tivoli/tip/dijit/ContextMenuAdapter", ["dijit","dojo","dojox","dojo/require!dijit/Menu"], function(dijit,dojo,dojox){
/*
	Copyright (c) 2004-2011, The Dojo Foundation All Rights Reserved.
	Available via Academic Free License >= 2.1 OR the modified BSD license.
	see: http://dojotoolkit.org/license for details
*/


if(!dojo._hasResource["ibm.tivoli.tip.dijit.ContextMenuAdapter"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.ContextMenuAdapter"] = true;
/*********************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* 5724-Q87 
*
* (C) Copyright IBM Corp. 2008
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
************************************************************ {COPYRIGHT-END} **/
dojo.require( "dijit.Menu" );

dojo.provide( "ibm.tivoli.tip.dijit.ContextMenuAdapter" );
/*******************************************************************************
* ContextMenuAdapter is responsible for constructing and displaying dynamic
* context menus.  That is it will ask for the items that should be displayed
* each time it is told to present the context menu. 
*
* It can handle calling a JSON Url to retreive the items for you.
*
* It also has a method that can be overloaded to provide context to the code
* building the menu. (For instance -- the row of a table that is selected)
*
* The menu model is as follows:
* 
* [ { 
*     id: "uniqueID", 
*     label: "label", 
*     iconClass: "iconClass", 
*     submenu_items: [ {},{} ] 
*     actionID: "actionToFire" 
    } 
* ... ]
*
* id, iconClass, and submenu_items are all optional
*
* NOTE:
* ! IF "_separator_" is passed as actionID then a menu separator is created !
*******************************************************************************/
dojo.declare( "ibm.tivoli.tip.dijit.ContextMenuAdapter", null,
{
   /* [Optional] The JSON Url to request context menu items from */
   url: null,
   
   /* [Optional] The unique name of the context menu -- for lookup of items */
   name: null,
   
   /****************************************************************************
   * The constructor of the adapter.  Mixes in parameters.
   ****************************************************************************/
   constructor: function( args )
   {
     dojo.mixin( this, args );
     
     this.menuItems = [];
     this.subMenus = [];
   },
      
   /****************************************************************************
   * Ensures that only one context menu is used during the life of this adapter.
   * It destroys any previously created menus and recreates a new menu.
   ****************************************************************************/
   createContextMenu: function()
   {
     this._menu = new dijit.Menu();
     
     this._onCloseListener = dojo.connect( this._menu, "onClose", this, "_destroyMenu" );
     
     //populate the menu
     this.populateMenu( this._menu ); 
     
     return this._menu;         
   },
   
   /****************************************************************************
   * Return the current dijit.Menu context menu popup or null if there is not
   * a current context menu popup.
   ****************************************************************************/   
   getMenu: function() {
    	return ("_menu" in this) ? this._menu : null;
   },  
   
   /****************************************************************************
   * When the menu closes -- cleanly destroy it.  There is a bug in Dojo that
   * requires us to find all orphaned pieces of the menu to cleanly destroy
   ****************************************************************************/   
   _destroyMenu: function()
   {
     dojo.disconnect( this._onCloseListener );
     
     delete this._onCloseListener;
     
     // This line causes a node is null error in dojo 1.3.1, so get rid of it... ?
     // Get dojo itself to destroy first. 
   	 this._menu.destroyRecursive();

	 for ( var i in this.menuItems )
	 {
	   if (i) {
	   try{
	 	this.menuItems[ i ].destroy();
		}catch(e){ console.log("destory menu items"); console.log(e);
		}
	   }
	 }
     
	 for ( i in this.subMenus )
	 {
	   if (i) {
	   try{
	 	this.subMenus[ i ].destroy();
		}catch(e){ console.log("destory sub menu items");console.log(e);
		}
	   }
	 }
	 
	 // Mark A.: Apr 8,2011: for defect 5339 on CSNext RTC
   	 // Commenting out this line as it destroys other popups on the page, e.g., drop down menus on toolbars.
   	 // It appears this was here for earlier versions of Dojo that did not clean up menus correctly.
     // dojo.query( ".dijitPopup" ).forEach( function( domNode ) { try{dojo._destroyElement( domNode );}catch(e){} } );       
     
     delete this._menu;            
     this.menuItems = [];
     this.subMenus = [];
   },
   
   /****************************************************************************
   * This method must be called with a mouse event to show the Context menu.
   * You could pass in an object with x:xPos and y:yPos for event to trick the 
   * context menu to open at any given point on screen.
   *
   * The context object is any object that you want to send back to the server
   * or pass to the getMenuItems method.
   ****************************************************************************/
   contextMenuOpen: function ( event, context , contentIsolation)
   {
		if(contentIsolation === null || contentIsolation === undefined)
			contentIsolation = false;
		this._context = context;
		// If in TIP content pane or in isolation iframe (i.e. with PopupUtil available)...
		if( contentIsolation == true  && typeof( PopupUtil ) != 'undefined' ){
			// Get the namespace identifying the content that launched the context menu
			var namespace="ns__N1468806017_";
			if(this.cmProvider.global != null)
			 namespace = this.cmProvider.global.namespace;
			//Get the location of the event
			var eCoords = {x:0,y:0};
			try {
				if( event.pageX || event.pageY ) 	{
					eCoords.x = event.pageX;
					eCoords.y = event.pageY;
				}
				else if (event.clientX || event.clientY) 	{
					eCoords.x = event.clientX + document.body.scrollLeft + document.documentElement.scrollLeft;
					eCoords.y = event.clientY + document.body.scrollTop + document.documentElement.scrollTop;
				}
			} catch( err ) {
				eCoords = dojo.coords( event.target, true );
			}
			dojo.stopEvent(event);  //To ensure the browser right-click event is stopped.
			//Close (just in case) and then open the popup covering the whole page
			var old_popup_iframe = PopupUtil.ClosePopup( namespace+'ibm.tivoli.tip.dijit.ContextMenuAdapter' );
			//Changes for fixing Content Isolation issue with Claro theme
			var theme = dojo.body().className;
			var popup_iframe = PopupUtil.OpenPopup( 
				namespace, 
				namespace+'ibm.tivoli.tip.dijit.ContextMenuAdapter', 
				true, 
				null, 
				null, 
				null, 
				null, 
				dojo.baseUrl+'../html/TIPCMPopup.html?PopupID='+namespace+'ibm.tivoli.tip.dijit.ContextMenuAdapter&eLeft='+eCoords.x+'&eTop='+eCoords.y+"&theme="+theme, 
				null
			);
			//Initialize the contextMenu when the iframe popup is finished loading
			var tThis = this;
			if( popup_iframe.addEventListener ){
				popup_iframe.addEventListener(
					"load", 
					function(){	

					   if(dojo.isIE){
							popup_iframe.contentWindow.setMenuItems(tThis.cmProvider.getMenuItems(context));
							popup_iframe.contentWindow.setMenuProvider(tThis.cmProvider);
							popup_iframe.contentWindow.makeTIPContextMenu( null,context );
						} else {
							popup_iframe.contentWindow.makeTIPContextMenu( tThis.cmProvider,context );
						}
					}, 
					false
				);
			}
			else if (popup_iframe.attachEvent){
				popup_iframe.attachEvent(
					"onload", 
					function(){ 

						if(dojo.isIE){
							popup_iframe.contentWindow.setMenuItems(tThis.cmProvider.getMenuItems(context));
							popup_iframe.contentWindow.setMenuProvider(tThis.cmProvider);
							popup_iframe.contentWindow.makeTIPContextMenu( null,context );
						} else {
							popup_iframe.contentWindow.makeTIPContextMenu( tThis.cmProvider,context );
						}
					}
				);
			}
		}
		else{

		try{

			var menu = this.createContextMenu();
			//Added as a fix for IBM Dojo 1.5
			dojo.stopEvent(event);  //To ensure the browser right-click event is stopped.
			//Vyshantha: Feb18,2011: Defect 903 on CSNext RTC or #158382 on the CS Bug Tracker: start of fix
			coords = {x: event.pageX, y: event.pageY};
			event.coords = coords;
			//end of fix.
			menu._contextMenuWithMouse = true;
			//Vyshantha:Mar3,2011: Hack to fix context menu opening issue on IE 7.0 : start
			if(dojo.isIE === 7){
				var self = menu;
				var savedFocus = dijit.getFocus(menu);
				function closeAndRestoreFocus(){
					// user has clicked on a menu or popup
					if(self.refocus){
						dijit.focus(savedFocus);
					}
					dijit.popup.close(self);
				}
				dijit.popup.open({
					popup: menu,
					x: event.coords.x + 10,
					y: event.coords.y + 10,
					onExecute: closeAndRestoreFocus,
					onCancel: closeAndRestoreFocus,
					orient: menu.isLeftToRight() ? 'L' : 'R'
				});
				if(dojo.isIE !== 7)
					menu.focus();

				menu._onBlur = function(){
					menu.inherited('_onBlur', arguments);
					// Usually the parent closes the child widget but if this is a context
					// menu then there is no parent
					dijit.popup.close(menu);
					// don't try to restore focus; user has clicked another part of the screen
					// and set focus there
				};
			} else 
				menu._openMyself( event );
			}catch(e){console.log("openmyself"); console.log(e);}
		}
   },
   
   /****************************************************************************
   * Asks the JSON URL or callback method for the menu items then recursively
   * builds the menu.
   ****************************************************************************/   
   populateMenu: function( menu )
   {
     var model = null;
     
     if ( this.url !== null )
     { 
       model = this._loadMenuItems(); 
     }
     else
     {
       model = this.getMenuItems( this._context );
     }
     
     if ( model !== undefined && model !== null )
     {
       this._addItemsToMenu( menu, model );

	 
       try{

       menu.startup();

	 
	   }catch(e){console.log("startup");console.log(e);
    	  
      }
     }               
   },
   
   /****************************************************************************
   * Recursive function to construct menu from array of items
   ****************************************************************************/
   _addItemsToMenu:function( parent, items )
   {
      var params;
      var subMenuIndex;
      var menuItemIndex;
	  
      try{
      for ( var i in items )
      {
         if ( items[i].submenu_items )
         {
           subMenuIndex = this.subMenus.length;
           this.subMenus[ subMenuIndex ] = new dijit.Menu();
 
           params = 
           {
               popup: this.subMenus[ subMenuIndex ]
           };

           dojo.mixin( params, items[i] );
        
         //  menuItemIndex = this.menuItems.length; 
          // this.menuItems[ menuItemIndex ] = new dijit.PopupMenuItem( params );
           //parent.addChild(  this.menuItems[ menuItemIndex ] );  
			parent.addChild( new dijit.PopupMenuItem( params ) ); 
           
           this._addItemsToMenu( this.subMenus[ subMenuIndex ], items[i].submenu_items );           
         }
         else
         {
           // [2/15/08] The following if clause added for backward compatibility
           // with the previous version of TIPContextMenu
           if (items[i].id && items[i].id.toLowerCase() === "separator")
           {
              // Previous TIPContextMenu users indicated separators via the id
              items[i].actionID = "_separator_";
           } 
           else if (items[i].id && !items[i].actionID) 
           {
              // Previous TIPContextMenu users didn't use actionID
              items[i].actionID = items[i].id;
           }

           if ( items[i].actionID === "_separator_" )
           {
        //     menuItemIndex = this.menuItems.length;
             
          //   this.menuItems[ menuItemIndex ]  = new dijit.MenuSeparator();
             
            // parent.addChild( this.menuItems[ menuItemIndex ] );
			 
			parent.addChild( new dijit.MenuSeparator() ); 
           }
           else
           {
             params = {
               adapter: this,
               onClick: function() { this.adapter.onMenuItemClick( this ); }
             };
             
             dojo.mixin( params, items[i] );

            // menuItemIndex = this.menuItems.length;
             
             //this.menuItems[ menuItemIndex ] = new dijit.MenuItem( params );
             
             //parent.addChild( this.menuItems[ menuItemIndex ] );
			
			parent.addChild( new dijit.MenuItem( params ) );
  
           }
         }         
      }}catch(e){console.log("in add items to menu");}
   },
      
   /****************************************************************************
   * Callback that should be overloaded to be told when a menu item is clicked
   * and an action should occur
   ****************************************************************************/
   onMenuItemClick: function( menuItem ){ /*required overload*/ },
   
   /****************************************************************************
   * Synchronously Call a JSON Url to get the menu items
   *
   * Calls the provided url with the following:
   *    name: this.name (unique name of context menu to retrieve)
   *    content: this._context (the context object passed to contextMenuOpen)
   *
   * Theoretically this method could be overloaded to do something other than
   * dojo.xhrPost()
   ****************************************************************************/
   _loadMenuItems: function()
   {
     var kw = {
            sync:         true,
            url:          this.url,
            name:         this.name,
            content:      this._context,
            handleAs:     "json",
            mimetype:     "application/json",            
            load:         dojo.hitch( this, function( response ) { this._menuitems = response; } )       
          };
     
     //call synchronously                                                                       
     dojo.xhrGet( kw );
     
     return this._menuitems;
   },  

   /****************************************************************************
   * This method will be called if 'url' parameter is not set.  Provides a means
   * for javascript to build the menu items.
   ****************************************************************************/
   getMenuItems: function( context ){ /* must overload to provide menu items if URL is not used for JSON */ }
   
});

}

});

},
'dijit/_OnDijitClickMixin':function(){
define([
	"dojo/on",
	"dojo/_base/array", // array.forEach
	"dojo/keys", // keys.ENTER keys.SPACE
	"dojo/_base/declare", // declare
	"dojo/_base/sniff", // has("ie")
	"dojo/_base/unload", // unload.addOnWindowUnload
	"dojo/_base/window" // win.doc.addEventListener win.doc.attachEvent win.doc.detachEvent
], function(on, array, keys, declare, has, unload, win){

	// module:
	//		dijit/_OnDijitClickMixin
	// summary:
	//		Mixin so you can pass "ondijitclick" to this.connect() method,
	//		as a way to handle clicks by mouse, or by keyboard (SPACE/ENTER key)


	// Keep track of where the last keydown event was, to help avoid generating
	// spurious ondijitclick events when:
	// 1. focus is on a <button> or <a>
	// 2. user presses then releases the ENTER key
	// 3. onclick handler fires and shifts focus to another node, with an ondijitclick handler
	// 4. onkeyup event fires, causing the ondijitclick handler to fire
	var lastKeyDownNode = null;
	if(has("ie")){
		(function(){
			var keydownCallback = function(evt){
				lastKeyDownNode = evt.srcElement;
			};
			win.doc.attachEvent('onkeydown', keydownCallback);
			unload.addOnWindowUnload(function(){
				win.doc.detachEvent('onkeydown', keydownCallback);
			});
		})();
	}else{
		win.doc.addEventListener('keydown', function(evt){
			lastKeyDownNode = evt.target;
		}, true);
	}

	// Custom a11yclick (a.k.a. ondijitclick) event
	var a11yclick = function(node, listener){
		if(/input|button/i.test(node.nodeName)){
			// pass through, the browser already generates click event on SPACE/ENTER key
			return on(node, "click", listener);
		}else{
			// Don't fire the click event unless both the keydown and keyup occur on this node.
			// Avoids problems where focus shifted to this node or away from the node on keydown,
			// either causing this node to process a stray keyup event, or causing another node
			// to get a stray keyup event.

			function clickKey(/*Event*/ e){
				return (e.keyCode == keys.ENTER || e.keyCode == keys.SPACE) &&
						!e.ctrlKey && !e.shiftKey && !e.altKey && !e.metaKey;
			}
			var handles = [
				on(node, "keypress", function(e){
					//console.log(this.id + ": onkeydown, e.target = ", e.target, ", lastKeyDownNode was ", lastKeyDownNode, ", equality is ", (e.target === lastKeyDownNode));
					if(clickKey(e)){
						// needed on IE for when focus changes between keydown and keyup - otherwise dropdown menus do not work
						lastKeyDownNode = e.target;

						// Prevent viewport scrolling on space key in IE<9.
						// (Reproducible on test_Button.html on any of the first dijit.form.Button examples)
						// Do this onkeypress rather than onkeydown because onkeydown.preventDefault() will
						// suppress the onkeypress event, breaking _HasDropDown
						e.preventDefault();
					}
				}),

				on(node, "keyup", function(e){
					//console.log(this.id + ": onkeyup, e.target = ", e.target, ", lastKeyDownNode was ", lastKeyDownNode, ", equality is ", (e.target === lastKeyDownNode));
					if(clickKey(e) && e.target == lastKeyDownNode){	// === breaks greasemonkey
						//need reset here or have problems in FF when focus returns to trigger element after closing popup/alert
						lastKeyDownNode = null;
						listener.call(this, e);
					}
				}),

				on(node, "click", function(e){
					// and connect for mouse clicks too (or touch-clicks on mobile)
					listener.call(this, e);
				})
			];

			return {
				remove: function(){
					array.forEach(handles, function(h){ h.remove(); });
				}
			};
		}
	};

	return declare("dijit._OnDijitClickMixin", null, {
		connect: function(
				/*Object|null*/ obj,
				/*String|Function*/ event,
				/*String|Function*/ method){
			// summary:
			//		Connects specified obj/event to specified method of this object
			//		and registers for disconnect() on widget destroy.
			// description:
			//		Provide widget-specific analog to connect.connect, except with the
			//		implicit use of this widget as the target object.
			//		This version of connect also provides a special "ondijitclick"
			//		event which triggers on a click or space or enter keyup.
			//		Events connected with `this.connect` are disconnected upon
			//		destruction.
			// returns:
			//		A handle that can be passed to `disconnect` in order to disconnect before
			//		the widget is destroyed.
			// example:
			//	|	var btn = new dijit.form.Button();
			//	|	// when foo.bar() is called, call the listener we're going to
			//	|	// provide in the scope of btn
			//	|	btn.connect(foo, "bar", function(){
			//	|		console.debug(this.toString());
			//	|	});
			// tags:
			//		protected

			return this.inherited(arguments, [obj, event == "ondijitclick" ? a11yclick : event, method]);
		}
	});
});

},
'url:dojox/grid/resources/View.html':"<div class=\"dojoxGridView\" role=\"presentation\">\r\n\t<div class=\"dojoxGridHeader\" dojoAttachPoint=\"headerNode\" role=\"presentation\">\r\n\t\t<div dojoAttachPoint=\"headerNodeContainer\" style=\"width:9000em\" role=\"presentation\">\r\n\t\t\t<div dojoAttachPoint=\"headerContentNode\" role=\"row\"></div>\r\n\t\t</div>\r\n\t</div>\r\n\t<input type=\"checkbox\" class=\"dojoxGridHiddenFocus\" dojoAttachPoint=\"hiddenFocusNode\" role=\"presentation\" />\r\n\t<input type=\"checkbox\" class=\"dojoxGridHiddenFocus\" role=\"presentation\" />\r\n\t<div class=\"dojoxGridScrollbox\" dojoAttachPoint=\"scrollboxNode\" role=\"presentation\">\r\n\t\t<div class=\"dojoxGridContent\" dojoAttachPoint=\"contentNode\" hidefocus=\"hidefocus\" role=\"presentation\"></div>\r\n\t</div>\r\n</div>\r\n",
'dojox/grid/util':function(){
define([
	"../main",
	"dojo/_base/lang",
	"dojo/dom"
], function(dojox, lang, dom){

// summary: grid utility library
	var dgu = lang.getObject("grid.util", true, dojox);

	dgu.na = '...';
	dgu.rowIndexTag = "gridRowIndex";
	dgu.gridViewTag = "gridView";


	dgu.fire = function(ob, ev, args){
		var fn = ob && ev && ob[ev];
		return fn && (args ? fn.apply(ob, args) : ob[ev]());
	};
	
	dgu.setStyleHeightPx = function(inElement, inHeight){
		if(inHeight >= 0){
			var s = inElement.style;
			var v = inHeight + 'px';
			if(inElement && s['height'] != v){
				s['height'] = v;
			}
		}
	};
	
	dgu.mouseEvents = [ 'mouseover', 'mouseout', /*'mousemove',*/ 'mousedown', 'mouseup', 'click', 'dblclick', 'contextmenu' ];

	dgu.keyEvents = [ 'keyup', 'keydown', 'keypress' ];

	dgu.funnelEvents = function(inNode, inObject, inMethod, inEvents){
		var evts = (inEvents ? inEvents : dgu.mouseEvents.concat(dgu.keyEvents));
		for (var i=0, l=evts.length; i<l; i++){
			inObject.connect(inNode, 'on' + evts[i], inMethod);
		}
	};

	dgu.removeNode = function(inNode){
		inNode = dom.byId(inNode);
		inNode && inNode.parentNode && inNode.parentNode.removeChild(inNode);
		return inNode;
	};
	
	dgu.arrayCompare = function(inA, inB){
		for(var i=0,l=inA.length; i<l; i++){
			if(inA[i] != inB[i]){return false;}
		}
		return (inA.length == inB.length);
	};
	
	dgu.arrayInsert = function(inArray, inIndex, inValue){
		if(inArray.length <= inIndex){
			inArray[inIndex] = inValue;
		}else{
			inArray.splice(inIndex, 0, inValue);
		}
	};
	
	dgu.arrayRemove = function(inArray, inIndex){
		inArray.splice(inIndex, 1);
	};
	
	dgu.arraySwap = function(inArray, inI, inJ){
		var cache = inArray[inI];
		inArray[inI] = inArray[inJ];
		inArray[inJ] = cache;
	};

	dgu.fixIEEmptyCell = function(cellContent){
		if((dojo.isIE < 8 || (dojo.isIE && dojo.isQuirks)) && 
				(cellContent === null || cellContent === '' || /^\s+$/.test(cellContent))){
			//fix IE 6/7 quirks - border style not effective for empty td
			return '&nbsp;';
		}else{
			return cellContent;
		}
	};

	return dojox.grid.util;

});
},
'dijit/form/_ListBase':function(){
define([
	"dojo/_base/declare",	// declare
	"dojo/window" // winUtils.scrollIntoView
], function(declare, winUtils){

// module:
//		dijit/form/_ListBase
// summary:
//		Focus-less menu to handle UI events consistently

return declare( "dijit.form._ListBase", null, {
	// summary:
	//		Focus-less menu to handle UI events consistently
	//		Abstract methods that must be defined externally:
	//			onSelect: item is active (mousedown but not yet mouseup, or keyboard arrow selected but no Enter)
	//			onDeselect:  cancels onSelect
	// tags:
	//		private

	// selected: DOMnode
	//		currently selected node
	selected: null,

	_getTarget: function(/*Event*/ evt){
		var tgt = evt.target;
		var container = this.containerNode;
		if(tgt == container || tgt == this.domNode){ return null; }
		while(tgt && tgt.parentNode != container){
			// recurse to the top
			tgt = tgt.parentNode;
		}
		return tgt;
	},

	selectFirstNode: function(){
		// summary:
		// 		Select the first displayed item in the list.
		var first = this.containerNode.firstChild;
		while(first && first.style.display == "none"){
			first = first.nextSibling;
		}
		this._setSelectedAttr(first);
	},

	selectLastNode: function(){
		// summary:
		// 		Select the last displayed item in the list
		var last = this.containerNode.lastChild;
		while(last && last.style.display == "none"){
			last = last.previousSibling;
		}
		this._setSelectedAttr(last);
	},

	selectNextNode: function(){
		// summary:
		// 		Select the item just below the current selection.
		// 		If nothing selected, select first node.
		var selectedNode = this._getSelectedAttr();
		if(!selectedNode){
			this.selectFirstNode();
		}else{
			var next = selectedNode.nextSibling;
			while(next && next.style.display == "none"){
				next = next.nextSibling;
			}
			if(!next){
				this.selectFirstNode();
			}else{
				this._setSelectedAttr(next);
			}
		}
	},

	selectPreviousNode: function(){
		// summary:
		// 		Select the item just above the current selection.
		// 		If nothing selected, select last node (if
		// 		you select Previous and try to keep scrolling up the list).
		var selectedNode = this._getSelectedAttr();
		if(!selectedNode){
			this.selectLastNode();
		}else{
			var prev = selectedNode.previousSibling;
			while(prev && prev.style.display == "none"){
				prev = prev.previousSibling;
			}
			if(!prev){
				this.selectLastNode();
			}else{
				this._setSelectedAttr(prev);
			}
		}
	},

	_setSelectedAttr: function(/*DomNode*/ node){
		// summary:
		//		Does the actual select.
		if(this.selected != node){
			var selectedNode = this._getSelectedAttr();
			if(selectedNode){
				this.onDeselect(selectedNode);
				this.selected = null;
			}
			if(node && node.parentNode == this.containerNode){
				this.selected = node;
				winUtils.scrollIntoView(node);
				this.onSelect(node);
			}
		}else if(node){
			this.onSelect(node);
		}
	},

	_getSelectedAttr: function(){
		// summary:
		//		Returns the selected node.
		var v = this.selected;
		return (v && v.parentNode == this.containerNode) ? v : (this.selected = null);
	}
});

});

},
'*now':function(r){r(['dojo/i18n!*preload*ibm/usmi/core/framework/nls/FrameworkAll*["ar","cs","da","de","en","en-us","es","fi","fr","he","hr","hu","it","ja","ko","nl","no","pl","pt-br","ru","sl","sv","tr","zh-cn","zh-tw","ROOT"]']);}
}});
// wrapped by build app
define("ibm/usmi/core/framework/FrameworkAll", ["dijit","dojo","dojox","dojo/require!ibm/usmi/core/framework/ContentTitle,ibm/usmi/menus/Menu,ibm/usmi/menus/NextGenUIMenuBar,idx/oneui/MenuBar,idx/oneui/MenuDialog,idx/oneui/Menu,idx/oneui/MenuHeading,idx/oneui/Header,ibm/usmi/widgets/FrameworkMessage,ibm/usmi/dashboard/ResourceDashBoardPreload,ibm/usmi/dashboard/ResourceDashBoard,ibm/usmi/dashboard/ResourceGroup,ibm/usmi/dashboard/DatasetItemStoreAdapter,ibm/usmi/dashboard/Constants,ibm/usmi/dashboard/ResourceGroupWidget,ibm/usmi/dashboard/HealthTip,ibm/usmi/dashboard/DashBoardResourceNavigator,ibm/usmi/util/DefaultUserInterfaceTask,ibm/usmi/widgets/ResourceNavigator,ibm/usmi/widgets/TreeSelector,ibm/usmi/widgets/DetailsSelector,ibm/usmi/widgets/FrameworkMessage,ibm/usmi/widgets/WorkArea,ibm/usmi/widgets/util/ResourceNavigatorDAHelper,ibm/usmi/widgets/util/DefaultContextMenuProvider,ibm/usmi/widgets/util/ResourceMenuProviderHelper,ibm/usmi/widgets/util/DefaultMenuProviderHelper,ibm/usmi/core/utils/Logger,ibm/usmi/widgets/finder/Finder,ibm/usmi/widgets/finder/TaskDataStore,ibm/usmi/widgets/finder/FinderBaseDataStore,ibm/usmi/widgets/finder/TaskCategory,ibm/usmi/widgets/finder/ResourceDataStore,ibm/usmi/widgets/finder/GroupDataStore,ibm/usmi/widgets/util/Constants,ibm/usmi/util/BaseWidget,ibm/usmi/util/DataStore,ibm/usmi/util/DefaultDialogTask,ibm/usmi/util/DefaultMessageHandler,ibm/usmi/util/DefaultURLLaunchTask,ibm/usmi/util/Utils,ibm/usmi/widgets/table/DefaultItemsTableView,ibm/usmi/widgets/table/DefaultTreeTableView,ibm/usmi/widgets/table/ConfigureColumnsDialog,ibm/usmi/widgets/wizard/DirectorBorderContainer,ibm/usmi/widgets/wizard/DirectorWizard,ibm/usmi/widgets/wizard/DirectorPage,ibm/usmi/menus/Menu,ibm/usmi/menus/NextGenUIMenuBar,ibm/usmi/menus/NextGenUIHelpMenu,ibm/usmi/scheduler/JobLaunchWizard,ibm/usmi/scheduler/wizard_pages/SchedulePage,ibm/usmi/scheduler/wizard_pages/NotificationsPage,ibm/usmi/scheduler/wizard_pages/OptionsPage"], function(dijit,dojo,dojox){
dojo.provide("ibm.usmi.core.framework.FrameworkAll");
dojo.require("ibm.usmi.core.framework.ContentTitle");
dojo.require("ibm.usmi.menus.Menu");
dojo.require("ibm.usmi.menus.NextGenUIMenuBar");
dojo.require("idx.oneui.MenuBar");
dojo.require("idx.oneui.MenuDialog");
dojo.require("idx.oneui.Menu");
dojo.require("idx.oneui.MenuHeading");
dojo.require("idx.oneui.Header");
dojo.require("ibm.usmi.widgets.FrameworkMessage");
dojo.require("ibm.usmi.dashboard.ResourceDashBoardPreload");
dojo.require("ibm.usmi.dashboard.ResourceDashBoard");
dojo.require("ibm.usmi.dashboard.ResourceGroup");
dojo.require("ibm.usmi.dashboard.DatasetItemStoreAdapter");
dojo.require("ibm.usmi.dashboard.Constants");
dojo.require("ibm.usmi.dashboard.ResourceGroupWidget");
dojo.require("ibm.usmi.dashboard.HealthTip");
dojo.require("ibm.usmi.dashboard.DashBoardResourceNavigator");

dojo.require("ibm.usmi.util.DefaultUserInterfaceTask");
dojo.require("ibm.usmi.widgets.ResourceNavigator");
dojo.require("ibm.usmi.widgets.TreeSelector");
dojo.require("ibm.usmi.widgets.DetailsSelector");
dojo.require("ibm.usmi.widgets.FrameworkMessage");
dojo.require("ibm.usmi.widgets.WorkArea");
dojo.require("ibm.usmi.widgets.util.ResourceNavigatorDAHelper");
dojo.require("ibm.usmi.widgets.util.DefaultContextMenuProvider");
dojo.require("ibm.usmi.widgets.util.ResourceMenuProviderHelper");
dojo.require("ibm.usmi.widgets.util.DefaultMenuProviderHelper");
dojo.require("ibm.usmi.core.utils.Logger");
dojo.require("ibm.usmi.widgets.finder.Finder");
dojo.require("ibm.usmi.widgets.finder.TaskDataStore");
dojo.require("ibm.usmi.widgets.finder.FinderBaseDataStore");
dojo.require("ibm.usmi.widgets.finder.TaskCategory");
dojo.require("ibm.usmi.widgets.finder.ResourceDataStore");
dojo.require("ibm.usmi.widgets.finder.GroupDataStore");
dojo.require("ibm.usmi.widgets.util.Constants");
dojo.require("ibm.usmi.util.BaseWidget");
dojo.require("ibm.usmi.util.DataStore");
dojo.require("ibm.usmi.util.DefaultDialogTask");
dojo.require("ibm.usmi.util.DefaultMessageHandler");
dojo.require("ibm.usmi.util.DefaultURLLaunchTask");
dojo.require("ibm.usmi.util.Utils");
dojo.require("ibm.usmi.widgets.table.DefaultItemsTableView");
dojo.require("ibm.usmi.widgets.table.DefaultTreeTableView");
dojo.require("ibm.usmi.widgets.table.ConfigureColumnsDialog");
dojo.require("ibm.usmi.widgets.wizard.DirectorBorderContainer");
dojo.require("ibm.usmi.widgets.wizard.DirectorWizard");
dojo.require("ibm.usmi.widgets.wizard.DirectorPage");
dojo.require("ibm.usmi.menus.Menu");
dojo.require("ibm.usmi.menus.NextGenUIMenuBar");
dojo.require("ibm.usmi.menus.NextGenUIHelpMenu");
dojo.require("ibm.usmi.scheduler.JobLaunchWizard");
dojo.require("ibm.usmi.scheduler.wizard_pages.SchedulePage");
dojo.require("ibm.usmi.scheduler.wizard_pages.NotificationsPage");
dojo.require("ibm.usmi.scheduler.wizard_pages.OptionsPage");






});
